var count = (function minus(Decimal) {
    var start = +new Date(),
        log,
        error,
        undefined,
        passed = 0,
        total = 0,
        n = 'null',
        N = 'NaN',
        I = 'Infinity';

    if (typeof window === 'undefined') {
        log = console.log;
        error = console.error;
    } else {
        log = function (str) { document.body.innerHTML += str.replace('\n', '<br>') };
        error = function (str) { document.body.innerHTML += '<div style="color: red">' +
          str.replace('\n', '<br>') + '</div>' };
    }

    if (!Decimal && typeof require === 'function') Decimal = require('../decimal');

    function assert(expected, actual) {
        total++;
        if (expected !== actual) {
           error('\n Test number: ' + total + ' failed');
           error(' Expected: ' + expected);
           error(' Actual:   ' + actual);
           //process.exit();
        } else {
            passed++;
            //log('\n Expected and actual: ' + actual);
        }
    }

    function T(minuend, subtrahend, expected, sd, rm) {

        if ( sd != null ) {
            Decimal.precision = sd;
            Decimal.rounding = rm;
        }
        assert(String(expected), String(new Decimal(minuend).minus(subtrahend)));
    }

    function isMinusZero(n) {
        return n.toString() === '0' && n.s == -1;
    }

    log('\n Testing minus...');

    Decimal.config({
        precision: 200,
        rounding: 4,
        toExpNeg: -9e9,
        toExpPos: 9e9,
        minE: -9e15,
        maxE: 9e15,
        errors: false
    });

    T(1, 0, 1);
    T(1, -0, 1);
    T(-1, 0, -1);
    T(-1, -0, -1);
    T(1, N, N);
    T(-1, N, N);
    T(1, I, -I);
    T(1, -I, I);
    T(-1, I, -I);
    T(-1, -I, I);
    T(0, 1, -1);
    T(0, -1, 1);
    T(-0, 1, -1);
    T(-0, -1, 1);

    /*
      IEEE 754 - 2008 section 6.3
      When the difference of two operands with like signs is exactly zero, the
      sign of that difference shall be +0 in all rounding-direction attributes
      except roundTowardNegative; under that attribute, the sign of an exact
      difference shall be −0.
      However, x + x = x −(−x) retains the same sign as x even when x is zero.
    */

    Decimal.rounding = 3;
    assert(false, isMinusZero(new Decimal(0).minus(-0)));     //   0 - -0 =  0
    assert(true, isMinusZero(new Decimal(-0).minus(0)));      //  -0 -  0 = -0
    assert(true, isMinusZero(new Decimal(0).minus(0)));       //   0 -  0 = -0
    assert(true, isMinusZero(new Decimal(-0).minus(-0)));     //  -0 - -0 = -0
    assert(true, isMinusZero(new Decimal(1).minus(1)));       //   1 -  1 = -0
    assert(true, isMinusZero(new Decimal(-1).minus(-1)));     //  -1 - -1 = -0

    Decimal.rounding = 4;
    assert(false, isMinusZero(new Decimal(0).minus(-0)));     //   0 - -0 =  0
    assert(true, isMinusZero(new Decimal(-0).minus(0)));      //  -0 -  0 = -0
    assert(false, isMinusZero(new Decimal(0).minus(0)));      //   0 -  0 =  0
    assert(false, isMinusZero(new Decimal(-0).minus(-0)));    //  -0 - -0 =  0
    assert(false, isMinusZero(new Decimal(1).minus(1)));      //   1 -  1 =  0
    assert(false, isMinusZero(new Decimal(-1).minus(-1)));    //  -1 - -1 =  0

    T(0, N, N);
    T(-0, N, N);
    T(0, I, -I);
    T(0, -I, I);
    T(-0, I, -I);
    T(-0, -I, I);
    T(N, 1, N);
    T(N, -1, N);
    T(N, 0, N);
    T(N, -0, N);
    T(N, N, N);
    T(N, I, N);
    T(N, -I, N);
    T(I, 1, I);
    T(I, -1, I);
    T(-I, 1, -I);
    T(-I, -1, -I);
    T(I, 0, I);
    T(I, -0, I);
    T(-I, 0, -I);
    T(-I, -0, -I);
    T(I, N, N);
    T(-I, N, N);
    T(I, I, N);
    T(I, -I, I);
    T(-I, I, -I);
    T(-I, -I, N);

    T(1, '0', '1');
    T(1, '1', '0');
    T(1, '-45', '46');
    T(1, '22', '-21');
    T(1, 0144, '-99');
    T(1, '0144', '-143');
    T(1, '6.1915', '-5.1915');
    T(1, '-1.02', '2.02');
    T(1, '0.09', '0.91');
    T(1, '-0.0001', '1.0001');
    T(1, '8e5', '-799999');
    T(1, '9E12', '-8999999999999');
    T(1, '1e-14', '0.99999999999999');
    T(1, '3.345E-9', '0.999999996655');
    T(1, '-345.43e+4', '3454301');
    T(1, '-94.12E+0', '95.12');
    T(1, '', N);
    T(1, '    ', N);
    T(1, '\t\t', N);
    T(1, 'ertgrt546', N);
    T(1, 'qweqwdewee', N);
    T(1, true, N);
    T(1, false, N);
    T(1, 'e 4.3', N);
    T(1, '4 .3', N);
    T(1, '4.0 01e', N);
    T(1, ' 4.001', '-3.001');
    T(1, '4.001 ', '-3.001');
    T(1, ' 4.001 ', '-3.001');
    T(1, '    4.001', '-3.001');
    T(1, ' 4.0 01', N);
    T(1, '4. 001', N);
    T(1, '4. 001 ', N);
    T(1, '  4.001e ', N);
    T(1, ' 4 .001 e ', N);
    T(1, undefined, N);
    T(1, null, N);
    T(1, Number.POSITIVE_INFINITY, -I);
    T(1, Number.NEGATIVE_INFINITY, I);
    T(1, new Date(2012, 11, 4), N);
    T(1, new Object(), N);
    T(1, function () {}, N);
    T('0', 0, '0');
    T(0, '+0', '0');
    T('0', '0', '0');
    T(3, -0, '3');
    T(9.654, 0, '9.654');
    T(0, '0.001', '-0.001');
    T(0, '111.1111111110000', '-111.111111111');
    T(-1, 1, '-2');
    T(-0.01, 0.01, '-0.02');
    T(54, -54, '108');
    T(9.99, '-9.99', '19.98');
    T('0.0000023432495704937', '-0.0000023432495704937', '0.0000046864991409874');
    T(NaN, NaN, N);
    T(NaN, N, N);
    T(N, NaN, N);
    T(N, 4, N);
    T(N, '4534534.45435435', N);
    T(N, 99999.999, N);
    T(Infinity, '354.345341', I);
    T(3, -I, I);
    T(-Infinity, -I, N);
    T(-I, -Infinity, N);
    T(I, '-999e999', I);
    T('1.21123e43', -I, I);
    T('-999.0', I, -I);
    T('657.342e-45', -I, I);
    T(I, 123, I);
    T(-0, I, -I);
    T(100, 100, '0');
    T(-999.99, '0.01', '-1000');
    T('10 ', 4, '6');
    T('03.333', -4, '7.333');
    T(-1, -0.1, '-0.9');
    T(43534.5435, '0.054645', '43534.488855');
    T('99999', '1', '99998');
    T('3e', 8, N);
    T('-3..0', 13, N);
    T('0 0', -0.4, N);
    T(' +3e0', 4, '-1');
    T(9.9806, '+ 1', N);
    T(' +2 0', '1e1', N);
    T('e3', 4, N);
    T(' ', 0, N);
    T(323, null, N);
    T(undefined, undefined, N);
    T('undefined', undefined, N);
    T(null, null, N);

    T('0.000001', '1E-13', '0.0000009999999', 12, 5);
    T('1E-29', '0.1', '-0.09999999999999999999999999999', 30, 4);
    T('1E-9', '1E-40', '0.0000000009999999999999999999999999999999', 72, 6);
    T('1E-12', '1E-7', '-0.000000099999', 42, 3);
    T('1E-14', '1E-13', '-0.00000000000009', 76, 0);
    T('1E-21', '1E-32', '0.00000000000000000000099999999999', 77, 4);
    T('1E-19', '1E-16', '-0.0000000000000000999', 43, 3);
    T('1E-17', '1E-39', '0.000000000000000009999999999999999999999', 68, 2);
    T('1E-17', '1E-16', '-0.00000000000000009', 67, 1);
    T('1E-37', '1E-13', '-0.0000000000000999999999999999999999999', 52, 0);
    T('1E-27', '0.1', '-0.1', 25, 5);
    T('1E-28', '0.0001', '-0.0001', 6, 4);
    T('1E-20', '1E-34', '0.0000000000000000000099999999999999', 26, 0);
    T('1E-20', '0.1', '-0.1', 1, 6);
    T('1E-21', '1E-35', '0.00000000000000000000099999999999999', 57, 3);
    T('1E-30', '0.000001', '-0.000000999999999999999999999999', 81, 0);
    T('0.001', '1E-37', '0.00099999999999', 11, 1);
    T('1E-37', '1E-22', '-0.0000000000000000000000999999999999999', 73, 6);
    T('1E-25', '1E-34', '0.0000000000000000000000000999999999', 77, 5);
    T('1E-35', '1E-15', '-0.00000000000000099999999999999999999', 72, 1);
    T('1E-21', '1E-18', '-0.000000000000000000999', 16, 3);
    T('1E-31', '0.01', '-0.0099999999999999999999999999999', 36, 1);
    T('1E-13', '1E-10', '-0.0000000000999', 47, 6);
    T('1E-18', '0.001', '-0.000999999999999999', 91, 5);
    T('1E-38', '1E-11', '-0.00000000000999999999999999999999999999', 35, 5);
    T('0.001', '1E-23', '0.001', 17, 2);
    T('1E-38', '1E-7', '-0.00000009999999999999999999999999999999', 80, 5);
    T('0.000001', '1E-32', '0.000001', 21, 2);
    T('1E-20', '1E-30', '0.000000000000000000009999999999', 99, 1);
    T('1E-23', '1E-17', '-0.00000000000000000999999', 7, 0);
    T('1E-27', '1E-13', '-0.000000000000099999999999999', 94, 4);
    T('0.000001', '1E-39', '0.000000999999999999999999999999999999999', 77, 0);
    T('1E-32', '1E-23', '-0.00000000000000000000000999999999', 26, 0);
    T('1E-37', '1E-19', '-0.0000000000000000001', 17, 4);
    T('1E-20', '1E-34', '0.0000000000000000000099999999999999', 41, 1);
    T('1E-16', '1E-30', '0.000000000000000099999999999999', 37, 4);
    T('1E-9', '1E-14', '0.00000000099999', 54, 2);
    T('1E-33', '1E-12', '-0.000000000000999999999999999999999', 39, 6);
    T('1E-26', '1E-40', '0.0000000000000000000000000099999999999999', 36, 3);
    T('0.0001', '0.1', '-0.0999', 27, 5);
    T('1E-39', '1E-26', '-0.000000000000000000000000009999999999999', 28, 5);
    T('1E-37', '1E-14', '-0.0000000000000099999999999999999999999', 99, 1);
    T('1E-21', '1E-14', '-0.000000000000009999999', 79, 6);
    T('1E-30', '1E-33', '0.000000000000000000000000000000999', 5, 0);
    T('1E-24', '1E-16', '-0.000000000000000099999999', 8, 4);
    T('0.1', '1E-29', '0.09999999999999999999999999999', 53, 1);
    T('1E-21', '1E-16', '-0.000000000000000099999', 69, 5);
    T('1E-13', '1E-32', '0.00000000000009999999999999999999', 80, 3);
    T('1E-27', '1E-27', '0', 14, 1);
    T('1E-36', '1E-17', '-0.00000000000000001', 10, 5);
    T('1E-20', '1E-39', '0.000000000000000000009999999999999999999', 39, 6);
    T('1E-32', '1E-13', '-0.00000000000009999999999999999999', 69, 0);
    T('1E-35', '1E-22', '-0.00000000000000000000009999999999999', 40, 0);
    T('1E-17', '1E-37', '0.00000000000000001', 8, 2);
    T('1E-19', '1E-9', '-0.0000000009999999999', 60, 5);
    T('1E-33', '1E-28', '-0.000000000000000000000000000099999', 6, 1);
    T('0.000001', '1E-27', '0.000001', 13, 2);
    T('1E-34', '1E-12', '-0.0000000000009999999999999999999999', 66, 2);
    T('1E-36', '1E-27', '-0.000000000000000000000000000999999999', 22, 3);
    T('1E-40', '1E-20', '-0.0000000000000000000099999999999999999999', 91, 5);
    T('1E-26', '1E-17', '-0.00000000000000001', 7, 0);
    T('1E-25', '1E-34', '0.0000000000000000000000000999999999', 76, 5);
    T('1E-37', '1E-29', '-0.0000000000000000000000000000099999999', 29, 0);
    T('1E-16', '1E-9', '-0.0000000009999999', 74, 1);
    T('1E-20', '1E-26', '0.00000000000000000000999999', 88, 1);
    T('1E-35', '1E-22', '-0.00000000000000000000009999999999999', 84, 5);
    T('1E-14', '1E-32', '0.00000000000000999999999999999999', 68, 1);
    T('1E-9', '1E-38', '0.0000000009999999999999999999999999999', 28, 3);
    T('1E-15', '1E-18', '0.000000000000000999', 46, 1);
    T('1E-8', '1E-20', '0.00000001', 1, 5);
    T('1E-33', '1E-33', '0', 98, 2);
    T('1E-30', '1E-38', '0.00000000000000000000000000000099999999', 22, 3);
    T('1E-19', '0.1', '-0.0999999999999999999', 53, 6);
    T('0.000001', '1E-14', '0.00000099999999', 53, 2);
    T('0.000001', '1E-25', '0.0000009999999999999999999', 96, 2);
    T('1E-13', '1E-16', '0.0000000000000999', 89, 2);
    T('1E-25', '1E-40', '0.0000000000000000000000000999999999999999', 74, 5);
    T('1E-33', '1E-18', '-0.000000000000000000999999999999999', 86, 3);
    T('1E-13', '1E-21', '0.000000000000099999999', 44, 5);
    T('1E-29', '0.01', '-0.01', 17, 6);
    T('0.1', '1E-29', '0.09999999999999999999999999999', 30, 5);
    T('1E-9', '1E-18', '0.000000000999999999', 10, 0);
    T('1E-34', '1E-31', '-0.0000000000000000000000000000000999', 90, 1);
    T('1E-35', '0.1', '-0.099999', 5, 2);
    T('1E-32', '1E-7', '-0.00000009999999999999999999999999', 27, 3);
    T('1E-25', '0.00001', '-0.0000099999999999999999999', 45, 5);
    T('1E-30', '1E-22', '-0.000000000000000000000099999999', 29, 6);
    T('1E-21', '1E-23', '0.00000000000000000000099', 4, 4);
    T('1E-23', '1E-36', '0.000000000000000000000009', 1, 3);
    T('1E-31', '1E-37', '0.0000000000000000000000000000000999999', 73, 0);
    T('1E-9', '1E-36', '0.000000000999999999999999999999999999', 90, 4);
    T('1E-39', '1E-31', '-0.000000000000000000000000000000099999999', 45, 6);
    T('0.0001', '0.000001', '0.000099', 7, 0);
    T('1E-7', '1E-36', '0.000000099999999999999999999999999999', 60, 0);
    T('1E-27', '1E-28', '0.0000000000000000000000000009', 95, 5);
    T('0.01', '0.0001', '0.0099', 27, 1);
    T('1E-9', '1E-30', '0.000000001', 6, 2);
    T('0.1', '1E-23', '0.1', 17, 6);
    T('1E-21', '1E-18', '-0.000000000000000000999', 46, 0);
    T('1E-19', '1E-23', '0.00000000000000000009999', 27, 4);
    T('1E-24', '1E-19', '-0.000000000000000000099999', 41, 0);
    T('1E-26', '1E-20', '-0.00000000000000000000999999', 53, 0);
    T('1E-19', '0.1', '-0.0999999999999999999', 80, 5);
    T('1E-14', '1E-24', '0.000000000000009999999999', 12, 3);
    T('1E-19', '1E-17', '-0.0000000000000000099', 59, 5);
    T('1E-27', '1E-22', '-0.000000000000000000000099999', 72, 4);
    T('1E-28', '1E-8', '-0.0000000099999999999999999999', 40, 2);
    T('1E-32', '1E-34', '0.0000000000000000000000000000000099', 60, 3);
    T('1E-23', '0.1', '-0.09999999999999999999999', 32, 5);
    T('0.01', '1E-26', '0.00999999999999999999999999', 100, 4);
    T('1E-22', '0.01', '-0.0099999999999999999999', 62, 1);
    T('1E-15', '1E-7', '-0.000000099999999', 57, 3);
    T('1E-11', '1E-7', '-0.00000009999', 51, 4);
    T('0.001', '1E-17', '0.00099999999999999', 19, 0);
    T('1E-12', '0.001', '-0.00099999', 5, 2);
    T('1E-24', '1E-14', '-0.000000000000009999999999', 22, 2);
    T('0.1', '1E-27', '0.099999999999999999999999999', 27, 3);
    T('1E-18', '1E-33', '0.000000000000000000999999999999999', 98, 2);
    T('0.1', '1E-17', '0.09999999999999999', 48, 6);
    T('1E-30', '1E-33', '0.000000000000000000000000000000999', 34, 2);
    T('1E-31', '1E-10', '-0.0000000000999999999999999999999', 37, 2);
    T('1E-28', '1E-12', '-0.000000000001', 14, 3);
    T('1E-23', '1E-19', '-0.00000000000000000009999', 41, 4);
    T('1E-8', '1E-9', '0.000000009', 31, 2);
    T('1E-17', '1E-8', '-0.00000000999999999', 67, 4);
    T('1E-31', '1E-17', '-0.0000000000000000099999999999999', 67, 6);
    T('0.000001', '1E-24', '0.000000999999999999999999', 93, 0);
    T('1E-17', '1E-33', '0.000000000000000009999999999999999', 57, 2);
    T('1E-28', '1E-10', '-0.0000000000999999999999999999', 71, 4);
    T('1E-17', '1E-26', '0.00000000000000000999999999', 82, 0);
    T('0.1', '0.00001', '0.09999', 67, 2);
    T('1E-21', '1E-25', '0.0000000000000000000009999', 93, 3);
    T('1E-19', '0.1', '-0.0999999999999999999', 64, 1);
    T('1E-10', '1E-21', '0.000000000099999999999', 49, 0);
    T('1E-37', '1E-16', '-0.0000000000000000999999999999999999999', 74, 0);
    T('1E-31', '1E-33', '0.000000000000000000000000000000099', 2, 4);
    T('1E-21', '1E-28', '0.0000000000000000000009999999', 54, 5);
    T('1E-26', '1E-28', '0.0000000000000000000000000099', 5, 3);
    T('1E-28', '1E-32', '0.00000000000000000000000000009999', 72, 5);
    T('1E-9', '1E-26', '0.00000000099999999999999999', 34, 4);
    T('1E-40', '1E-12', '-0.0000000000009999999999999999999999999999', 98, 1);
    T('1E-30', '0.0001', '-0.000099999999999999999999999999', 59, 5);
    T('1E-40', '1E-30', '-0.0000000000000000000000000000009999999999', 39, 0);
    T('1E-33', '1E-17', '-0.000000000000000009999999999999999', 30, 0);
    T('1E-37', '1E-16', '-0.0000000000000000999999999999999999999', 82, 1);
    T('0.1', '1E-11', '0.09999999999', 88, 1);
    T('1E-14', '0.1', '-0.09999999999999', 62, 6);
    T('0.001', '1E-14', '0.00099999999999', 56, 1);
    T('1E-20', '1E-9', '-0.00000000099999999999', 34, 0);
    T('1E-39', '1E-34', '-0.000000000000000000000000000000000099999', 73, 4);
    T('1E-14', '1E-31', '0.0000000000000099999999999999999', 42, 1);
    T('1E-31', '1E-12', '-0.0000000000009999999999999999999', 34, 6);
    T('0.01', '1E-30', '0.009999999999999999999999999999', 60, 6);
    T('1E-30', '1E-14', '-0.000000000000009999999999999999', 70, 2);
    T('0.01', '1E-17', '0.00999999999999999', 31, 5);
    T('1E-22', '0.001', '-0.0009999999999999999999', 46, 4);
    T('1E-35', '1E-31', '-0.00000000000000000000000000000009999', 10, 6);
    T('0.1', '1E-28', '0.1', 23, 5);
    T('0.00001', '1E-40', '0.00001', 2, 4);
    T('0.00001', '1E-35', '0.0000099999999999999999999999', 23, 1);
    T('1E-25', '1E-12', '-0.0000000000009999999999999', 85, 0);
    T('0.00001', '1E-29', '0.00000999999999999999999999999', 90, 0);
    T('0.0001', '1E-22', '0.0000999999999999999999', 20, 3);
    T('1E-19', '1E-25', '0.0000000000000000000999999', 87, 4);
    T('1E-22', '1E-23', '0.00000000000000000000009', 22, 6);
    T('1E-21', '1E-23', '0.00000000000000000000099', 23, 6);
    T('1E-12', '1E-7', '-0.000000099999', 55, 0);
    T('0.000001', '1E-17', '0.00000099999999999', 19, 1);
    T('1E-17', '1E-19', '0.0000000000000000099', 90, 4);
    T('1E-16', '1E-37', '0.0000000000000000999999999999999999999', 50, 1);
    T('1E-9', '1E-31', '0.0000000009999999999999999999999', 37, 2);
    T('1E-35', '0.1', '-0.09999999999999999999999999999999999', 87, 2);
    T('1E-32', '1E-13', '-0.0000000000001', 15, 5);
    T('1E-10', '1E-18', '0.000000000099999999', 91, 3);
    T('0.1', '1E-25', '0.099999999999999999', 17, 3);
    T('1E-32', '1E-21', '-0.000000000000000000001', 10, 0);
    T('1E-23', '1E-10', '-0.00000000009999999999999', 77, 0);
    T('1E-20', '1E-18', '-0.00000000000000000099', 71, 0);
    T('1E-17', '0.01', '-0.00999999999999999', 76, 6);
    T('1E-18', '1E-15', '-0.000000000000000999', 70, 1);
    T('1E-34', '1E-31', '-0.0000000000000000000000000000000999', 21, 0);
    T('0.1', '1E-28', '0.0999999999999999999999999999', 72, 2);
    T('1E-27', '1E-30', '0.000000000000000000000000000999', 76, 1);
    T('1E-14', '1E-11', '-0.00000000000999', 95, 2);
    T('1E-19', '1E-18', '-0.0000000000000000009', 43, 2);
    T('1E-26', '1E-28', '0.0000000000000000000000000099', 87, 5);
    T('1E-39', '1E-36', '-0.000000000000000000000000000000000000999', 97, 5);
    T('0.000001', '1E-35', '0.00000099999999999999999999999999999', 40, 1);
    T('1E-20', '1E-27', '0.000000000000000000009999999', 32, 1);
    T('1E-17', '0.1', '-0.09999999999999999', 30, 0);
    T('1E-37', '0.000001', '-0.0000009999999999999999999999999999999', 33, 6);
    T('1E-8', '1E-36', '0.000000009999999999999999999999999999', 46, 4);
    T('0.1', '1E-25', '0.0999999999999999999999999', 75, 1);
    T('1E-21', '1E-13', '-0.000000000000099999999', 45, 6);
    T('1E-28', '1E-13', '-0.0000000000000999999999999999', 63, 6);
    T('0.0001', '1E-23', '0.00009999999999999999999', 39, 2);
    T('1E-33', '1E-19', '-0.000000000000000000099999999999999', 15, 1);
    T('1E-38', '1E-24', '-0.00000000000000000000000099999999999999', 20, 2);
    T('1E-37', '1E-27', '-0.0000000000000000000000000009999999999', 24, 4);
    T('1E-29', '1E-18', '-0.00000000000000000099999999999', 92, 3);
    T('1E-7', '1E-34', '0.0000000999999999999999999999999999', 96, 1);
    T('0.00001', '1E-25', '0.0000099999999999999999999', 93, 0);
    T('1E-30', '1E-36', '0.000000000000000000000000000000999999', 37, 6);
    T('1E-24', '1E-37', '0.0000000000000000000000009999999999999', 85, 5);
    T('1E-21', '0.001', '-0.001', 17, 6);
    T('1E-31', '1E-23', '-0.0000000000000000000000099999999', 75, 2);
    T('0.000001', '1E-40', '0.0000009', 1, 3);
    T('1E-25', '1E-17', '-0.0000000000000000099999999', 37, 2);
    T('0.1', '1E-31', '0.0999999999999999999999999999999', 50, 3);
    T('1E-33', '1E-23', '-0.000000000000000000000009999999999', 95, 5);
    T('1E-32', '1E-35', '0.00000000000000000000000000000000999', 4, 1);
    T('1E-8', '1E-9', '0.000000009', 64, 5);
    T('1E-11', '1E-17', '0.00000000000999999', 67, 5);
    T('0.00001', '1E-12', '0.000009999999', 80, 4);
    T('0.001', '1E-12', '0.000999999999', 27, 5);
    T('1E-40', '1E-39', '-0.0000000000000000000000000000000000000009', 4, 2);
    T('1E-13', '1E-15', '0.000000000000099', 51, 2);
    T('0.1', '0.01', '0.09', 21, 0);
    T('1E-35', '1E-39', '0.000000000000000000000000000000000009999', 50, 5);
    T('0.1', '1E-37', '0.0999999999999999999999999999999999999', 93, 0);
    T('0.1', '1E-31', '0.0999999999999999999999999999999', 31, 1);
    T('1E-12', '1E-35', '0.00000000000099999999999999999999999', 43, 3);
    T('1E-10', '1E-22', '0.00000000009', 1, 3);
    T('1E-7', '1E-19', '0.0000000999999999999', 33, 2);
    T('1E-23', '1E-20', '-0.00000000000000000000999', 4, 6);
    T('1E-33', '1E-31', '-0.000000000000000000000000000000099', 65, 5);
    T('0.01', '1E-17', '0.00999999999999999', 50, 0);
    T('1E-21', '1E-12', '-0.00000000000099999', 5, 1);
    T('1E-15', '1E-40', '0.000000000000001', 5, 5);
    T('1E-19', '0.00001', '-0.0000099999999999999', 21, 3);
    T('0.1', '0.1', '0', 71, 5);
    T('1E-9', '1E-33', '0.000000000999999999999999999999999', 95, 3);
    T('1E-15', '1E-19', '0.0000000000000009999', 77, 5);
    T('1E-34', '0.01', '-0.0099999999999999999999999999999999', 61, 2);
    T('1E-17', '0.1', '-0.1', 6, 0);
    T('0.000001', '0.000001', '0', 72, 0);
    T('1E-15', '1E-40', '0.0000000000000009999999999999999999999999', 37, 0);
    T('1E-34', '1E-24', '-0.0000000000000000000000009999999999', 14, 3);
    T('1E-38', '0.1', '-0.09999999999999999999', 19, 1);
    T('1E-36', '0.0001', '-0.000099999999999999999999999999999999', 48, 0);
    T('0.01', '1E-39', '0.009999999999999999999999999999999999999', 94, 4);
    T('1E-24', '1E-13', '-0.000000000000099999999999', 48, 5);
    T('1E-31', '0.1', '-0.0999999999999999999999999999999', 50, 3);
    T('0.1', '1E-7', '0.0999999', 79, 5);
    T('0.01', '1E-36', '0.01', 26, 4);
    T('1E-22', '1E-8', '-0.00000001', 3, 6);
    T('1E-25', '0.1', '-0.0999999999999999999999999', 83, 1);
    T('1E-38', '0.000001', '-0.000001', 20, 4);
    T('0.001', '1E-21', '0.000999999999999999999', 100, 1);
    T('1E-26', '1E-35', '0.00000000000000000000000000999999999', 40, 0);
    T('1E-12', '1E-25', '0.0000000000009999999999999', 71, 2);
    T('1E-11', '1E-33', '0.000000000009999999999999999999999', 78, 0);
    T('1E-13', '1E-25', '0.0000000000000999999999999', 17, 0);
    T('1E-25', '0.1', '-0.0999999999999999999999999', 33, 2);
    T('1E-20', '1E-20', '0', 15, 4);
    T('1E-9', '1E-23', '0.000000001', 12, 6);
    T('1E-29', '1E-40', '0.0000000000000000000000000000099999999999', 29, 2);
    T('1E-32', '1E-36', '0.000000000000000000000000000000009999', 73, 2);
    T('1E-29', '0.00001', '-0.00000999999999999999999999999', 69, 3);
    T('0.00001', '1E-33', '0.00001', 16, 4);
    T('1E-10', '0.1', '-0.0999999999', 58, 1);
    T('1E-18', '1E-39', '0.000000000000000000999999999999999999999', 84, 1);
    T('1E-24', '0.00001', '-0.000009999999999999999999', 21, 0);
    T('1E-23', '1E-39', '0.000000000000000000000009999999999999999', 66, 2);
    T('1E-21', '1E-19', '-0.000000000000000000099', 71, 6);
    T('1E-24', '1E-27', '0.000000000000000000000000999', 81, 5);
    T('1E-40', '1E-9', '-0.000000001', 8, 6);
    T('1E-32', '1E-14', '-0.00000000000000999999999999999999', 24, 5);
    T('1E-11', '1E-11', '0', 70, 4);
    T('0.00001', '1E-29', '0.00000999999999999999999999999', 96, 0);
    T('1E-20', '1E-13', '-0.00000000000009999999', 43, 6);
    T('1E-34', '1E-16', '-0.0000000000000000999999999999999999', 69, 3);
    T('1E-29', '1E-33', '0.000000000000000000000000000009999', 83, 2);
    T('1E-26', '1E-27', '0.000000000000000000000000009', 39, 4);
    T('1E-36', '1E-23', '-0.000000000000000000000009999999999999', 64, 0);
    T('1E-35', '1E-28', '-0.00000000000000000000000000009999999', 49, 0);
    T('1E-11', '1E-29', '0.00000000000999999999999999999', 24, 5);
    T('1E-16', '1E-39', '0.00000000000000009999999999999999999', 19, 1);
    T('1E-36', '1E-21', '-0.000000000000000000000999999999999999', 27, 4);
    T('1E-8', '1E-34', '0.0000000099999999999999999999999999', 52, 0);
    T('1E-39', '1E-18', '-0.000000000000000000999999999999999999999', 79, 5);
    T('1E-23', '1E-36', '0.00000000000000000000001', 3, 6);
    T('1E-29', '1E-15', '-0.00000000000000099999999999999', 22, 3);
    T('0.01', '1E-17', '0.00999999999999999', 37, 2);
    T('1E-33', '0.0001', '-0.000099999999999999999999999999999', 43, 5);
    T('1E-33', '1E-32', '-0.000000000000000000000000000000009', 8, 2);
    T('1E-31', '1E-20', '-0.0000000000000000000099999999999', 35, 4);
    T('1E-12', '1E-19', '0.0000000000009999999', 79, 0);
    T('0.000001', '1E-37', '0.00000099999999999999999', 17, 3);
    T('1E-13', '1E-40', '0.0000000000000999999999999999999999999999', 82, 6);
    T('1E-24', '1E-36', '0.000000000000000000000000999999999999', 25, 1);
    T('1E-10', '1E-39', '0.000000000099999999999999999999999999999', 67, 1);
    T('1E-39', '1E-37', '-0.000000000000000000000000000000000000099', 28, 3);
    T('1E-24', '1E-30', '0.00000000000000000000000099', 2, 1);
    T('0.01', '1E-39', '0.009999999999999999999999999999999999999', 37, 3);
    T('0.001', '1E-23', '0.001', 15, 4);
    T('1E-23', '1E-37', '0.0000000000000000000000099999999999999', 55, 1);
    T('1E-10', '1E-21', '0.000000000099999999999', 60, 2);
    T('1E-15', '1E-25', '0.0000000000000009999999999', 82, 0);
    T('1E-35', '1E-21', '-0.00000000000000000000099999999999999', 90, 3);
    T('1E-8', '0.00001', '-0.00000999', 82, 6);
    T('1E-8', '1E-17', '0.00000000999999999', 39, 2);
    T('1E-38', '0.1', '-0.09999999999999999999999999999999999999', 58, 2);
    T('0.0001', '1E-28', '0.0001', 10, 6);
    T('0.1', '1E-39', '0.099999999999999999999999999999999999999', 90, 2);
    T('1E-18', '1E-34', '0.0000000000000000009999999999999999', 16, 3);
    T('1E-8', '1E-33', '0.000000009999999999999999999999999', 31, 0);
    T('1E-27', '1E-39', '0.000000000000000000000000000999999999999', 27, 1);
    T('1E-24', '1E-8', '-0.000000009999999999999999', 45, 5);
    T('0.00001', '1E-27', '0.000009999999999999999999999', 28, 3);
    T('1E-21', '0.01', '-0.009999999999999999999', 52, 2);
    T('1E-33', '1E-9', '-0.000000001', 13, 3);
    T('1E-14', '1E-9', '-0.00000000099999', 60, 3);
    T('1E-28', '1E-27', '-0.0000000000000000000000000009', 86, 1);
    T('1E-33', '1E-33', '0', 21, 3);
    T('1E-13', '1E-26', '0.00000000000009999999999999', 66, 4);
    T('1E-16', '1E-20', '0.00000000000000009999', 10, 3);
    T('1E-8', '1E-40', '0.0000000099999999999999999999999999999999', 80, 2);
    T('1E-25', '1E-8', '-0.0000000099999999999999999', 53, 2);
    T('1E-22', '1E-21', '-0.0000000000000000000009', 79, 5);
    T('0.000001', '1E-40', '0.0000009999999999999999999999999999999999', 94, 3);
    T('1E-10', '1E-33', '0.000000000099999999999999999999999', 55, 4);
    T('1E-37', '1E-12', '-0.0000000000009999999999999999999999999', 52, 4);
    T('1E-39', '0.01', '-0.009999999999999999999999999999999999999', 55, 2);
    T('1E-16', '0.0001', '-0.0000999999999999', 91, 5);
    T('1E-13', '1E-34', '0.0000000000000999999999999999999999', 22, 2);
    T('0.0001', '0.1', '-0.0999', 16, 4);
    T('0.000001', '1E-16', '0.0000009999999999', 35, 1);
    T('1E-8', '1E-19', '0.0000000099999999999', 24, 6);
    T('1E-17', '1E-7', '-0.00000009999999999', 58, 4);
    T('0.1', '0.000001', '0.099999', 53, 0);
    T('1E-39', '0.1', '-0.099999999999999999999999999999999999999', 38, 6);
    T('1E-7', '1E-22', '0.0000001', 12, 2);
    T('1E-28', '1E-12', '-0.0000000000009999999999999999', 40, 0);
    T('1E-20', '1E-28', '0.00000000000000000001', 5, 4);
    T('0.1', '1E-31', '0.0999999999999999999999999999999', 75, 2);
    T('0.0001', '0.01', '-0.0099', 36, 2);
    T('1E-15', '1E-8', '-0.000000009999999', 77, 0);
    T('1E-30', '1E-13', '-0.000000000000099999999999999999', 45, 4);
    T('1E-26', '0.00001', '-0.00000999999999999999999999', 53, 5);
    T('1E-20', '1E-9', '-0.00000000099999999999', 37, 6);
    T('1E-34', '0.1', '-0.1', 18, 4);
    T('1E-8', '1E-17', '0.00000000999999999', 46, 3);
    T('1E-11', '1E-12', '0.000000000009', 15, 4);
    T('1E-25', '0.0001', '-0.0000999999999999999999999', 54, 1);
    T('1E-12', '0.00001', '-0.000009999999', 50, 1);
    T('0.001', '0.0001', '0.0009', 9, 5);
    T('0.1', '1E-23', '0.0999999999', 9, 1);
    T('1E-30', '1E-29', '-0.000000000000000000000000000009', 5, 0);
    T('1E-25', '1E-11', '-0.0000000000099999999999999', 73, 2);
    T('1E-29', '1E-7', '-0.00000009999999999999999999999', 25, 4);
    T('1E-35', '0.00001', '-0.00000999999999999999999999999999999', 73, 0);
    T('1E-25', '1E-29', '0.00000000000000000000000009999', 35, 2);
    T('0.01', '1E-10', '0.0099999999', 52, 1);
    T('1E-35', '1E-33', '-0.00000000000000000000000000000000099', 53, 5);
    T('1E-24', '0.1', '-0.099999999999999999999999', 29, 5);
    T('1E-34', '0.01', '-0.0099', 2, 2);
    T('0.1', '1E-12', '0.09999', 4, 1);
    T('1E-26', '1E-37', '0.0000000000000000000000000099999999999', 70, 1);
    T('1E-23', '1E-14', '-0.00000000000000999999999', 56, 1);
    T('0.00001', '1E-17', '0.00000999999999999', 42, 4);
    T('1E-40', '1E-38', '-0.0000000000000000000000000000000000000099', 30, 4);
    T('0.1', '1E-38', '0.09999999999999999999999999999999999999', 82, 3);
    T('1E-18', '0.1', '-0.099999999999999999', 29, 0);
    T('1E-13', '0.1', '-0.0999999999999', 20, 6);
    T('1E-21', '1E-27', '0.000000000000000000000999999', 53, 6);
    T('1E-29', '1E-23', '-0.00000000000000000000000999999', 41, 5);
    T('1E-26', '1E-20', '-0.00000000000000000000999999', 33, 2);
    T('1E-7', '1E-15', '0.0000001', 4, 6);
    T('0.01', '1E-20', '0.00999999999999999999', 88, 5);
    T('1E-24', '1E-26', '0.00000000000000000000000099', 67, 3);
    T('1E-20', '0.1', '-0.09999999999999999999', 63, 5);
    T('1E-24', '0.01', '-0.009999999999999999999999', 29, 2);
    T('0.001', '1E-37', '0.0009999999999999999999999999999999999', 92, 3);
    T('1E-21', '0.0001', '-0.000099999999999999999', 39, 1);
    T('0.000001', '0.00001', '-0.000009', 35, 6);
    T('1E-15', '1E-34', '0.0000000000000009999999999999999999', 26, 4);
    T('1E-14', '1E-13', '-0.00000000000009', 63, 5);
    T('1E-27', '1E-25', '-0.000000000000000000000000099', 98, 4);
    T('1E-19', '1E-32', '0.00000000000000000009999999999999', 61, 6);
    T('1E-22', '1E-15', '-0.0000000000000009999999', 53, 3);
    T('1E-40', '1E-30', '-0.0000000000000000000000000000009999999999', 80, 4);
    T('1E-10', '0.1', '-0.0999999999', 26, 1);
    T('1E-31', '1E-33', '0.000000000000000000000000000000099', 38, 5);
    T('1E-8', '1E-27', '0.000000009999999999999999999', 95, 4);
    T('1E-28', '1E-31', '0.0000000000000000000000000000999', 30, 6);
    T('1E-35', '1E-9', '-0.000000001', 21, 4);
    T('1E-17', '0.01', '-0.00999999999999999', 20, 1);
    T('1E-22', '1E-24', '0.000000000000000000000099', 51, 2);
    T('0.000001', '1E-10', '0.0000009999', 96, 2);
    T('1E-26', '1E-21', '-0.00000000000000000000099999', 85, 1);
    T('1E-27', '1E-31', '0.0000000000000000000000000009999', 52, 5);
    T('1E-12', '1E-19', '0.0000000000009999999', 78, 3);
    T('0.000001', '1E-17', '0.00000099999999999', 47, 0);
    T('0.01', '1E-11', '0.00999999999', 57, 5);
    T('1E-30', '1E-30', '0', 19, 2);
    T('0.000001', '1E-33', '0.000000999999999999999999999999999', 90, 1);
    T('1E-34', '1E-27', '-0.0000000000000000000000000009999999', 56, 2);
    T('0.000001', '1E-14', '0.00000099999999', 80, 0);
    T('1E-26', '0.0001', '-0.00009999999999999999999999', 41, 2);
    T('1E-16', '0.001', '-0.0009999999999999', 33, 3);
    T('1E-35', '1E-12', '-0.00000000000099999999999999999999999', 89, 4);
    T('1E-26', '1E-25', '-0.00000000000000000000000009', 85, 2);
    T('1E-29', '1E-17', '-0.00000000000000000999999999999', 15, 5);
    T('1E-27', '0.1', '-0.099999999999999999999999999', 84, 1);
    T('1E-27', '0.0001', '-0.000099999999999999999999999', 54, 5);
    T('1E-31', '1E-7', '-0.0000000999999999999999999999999', 75, 4);
    T('0.000001', '1E-22', '0.000001', 11, 5);
    T('1E-19', '1E-20', '0.00000000000000000009', 24, 3);
    T('0.00001', '1E-35', '0.00000999999999999999999999999999999', 95, 0);
    T('1E-37', '1E-28', '-0.0000000000000000000000000000999999999', 31, 2);
    T('1E-28', '1E-10', '-0.0000000000999999999999999999', 74, 3);
    T('1E-21', '1E-18', '-0.000000000000000000999', 88, 1);
    T('1E-40', '1E-30', '-0.0000000000000000000000000000009999999999', 23, 5);
    T('1E-10', '1E-14', '0.00000000009999', 99, 2);
    T('1E-9', '1E-22', '0.0000000009999999999999', 51, 5);
    T('1E-16', '1E-16', '0', 56, 1);
    T('1E-20', '0.1', '-0.09999999999999999999', 19, 3);
    T('1E-37', '0.000001', '-0.0000009999999999999999999999999999999', 33, 1);
    T('1E-16', '1E-16', '0', 96, 2);
    T('1E-27', '0.00001', '-0.000009999999999999999999999', 93, 2);
    T('1E-17', '1E-22', '0.0000000000000000099999', 38, 4);
    T('1E-26', '1E-10', '-0.00000000009999999999999999', 56, 0);
    T('1E-40', '1E-11', '-0.0000000000099999999999999999999999999999', 75, 2);
    T('1E-8', '1E-10', '0.0000000099', 42, 5);
    T('1E-36', '1E-16', '-0.000000000000000099999999999999999999', 90, 0);
    T('1E-20', '1E-22', '0.0000000000000000000099', 51, 1);
    T('1E-25', '1E-26', '0.00000000000000000000000009', 16, 2);
    T('1E-36', '1E-40', '0.0000000000000000000000000000000000009999', 17, 3);
    T('1E-17', '1E-31', '0.00000000000000001', 8, 0);
    T('1E-32', '1E-19', '-0.00000000000000000009999999999999', 51, 4);
    T('1E-30', '0.000001', '-0.000000999999999999999999999999', 92, 5);
    T('1E-17', '0.01', '-0.00999999999999999', 37, 3);
    T('1E-16', '1E-30', '0.000000000000000099999999999999', 20, 1);
    T('1E-10', '1E-32', '0.00000000009999999999999999999999', 84, 0);
    T('1E-12', '1E-17', '0.00000000000099999', 37, 6);
    T('1E-22', '1E-10', '-0.0000000000999999999999', 87, 1);
    T('1E-9', '1E-11', '0.00000000099', 4, 0);
    T('1E-16', '1E-26', '0.00000000000000009999999999', 13, 0);
    T('1E-37', '0.001', '-0.001', 32, 5);
    T('1E-16', '1E-24', '0.000000000000000099999999', 17, 2);
    T('1E-7', '1E-27', '0.000000099999999999999999999', 29, 3);
    T('1E-25', '1E-26', '0.00000000000000000000000009', 24, 4);
    T('1E-11', '1E-20', '0.00000000000999999999', 47, 1);
    T('0.1', '1E-24', '0.099999999999999999999999', 46, 1);
    T('1E-17', '1E-16', '-0.00000000000000009', 27, 5);
    T('1E-27', '1E-13', '-0.000000000000099999999999999', 42, 0);
    T('1E-25', '0.001', '-0.0009999999999999999999999', 42, 1);
    T('0.01', '1E-35', '0.00999999999999999999999999999999999', 86, 3);
    T('1E-35', '1E-24', '-0.000000000000000000000001', 5, 0);
    T('1E-15', '1E-14', '-0.000000000000009', 62, 6);
    T('1E-28', '1E-10', '-0.0000000000999999999999999999', 91, 3);
    T('1E-37', '1E-35', '-0.0000000000000000000000000000000000099', 67, 0);
    T('1E-7', '1E-35', '0.00000009999999999999999999999999999', 80, 2);
    T('1E-35', '1E-20', '-0.00000000000000000000999999999999999', 84, 4);
    T('1E-24', '1E-27', '0.000000000000000000000000999', 88, 2);
    T('1E-21', '1E-11', '-0.000000000009999999999', 33, 0);
    T('1E-15', '1E-37', '0.00000000000000099999999999', 11, 1);
    T('1E-14', '1E-34', '0.0000000000000099999999999999999999', 35, 0);
    T('1E-8', '1E-34', '0.00000001', 12, 5);
    T('1E-17', '1E-19', '0.0000000000000000099', 52, 2);
    T('1E-33', '1E-33', '0', 100, 4);
    T('0.0001', '0.00001', '0.00009', 55, 5);
    T('1E-8', '1E-32', '0.00000000999999999999999999999999', 83, 0);
    T('1E-7', '1E-33', '0.000000099999999999999999999999999', 26, 0);
    T('1E-35', '1E-10', '-0.00000000009999999999999999999999999', 87, 4);
    T('1E-35', '1E-28', '-0.00000000000000000000000000009999999', 58, 4);
    T('0.01', '1E-26', '0.00999999999999999999999999', 83, 3);
    T('1E-28', '1E-26', '-0.0000000000000000000000000099', 35, 2);
    T('1E-14', '1E-38', '0.00000000000000999999999999999999999999', 54, 3);
    T('1E-16', '1E-24', '0.000000000000000099999999', 11, 4);
    T('1E-25', '1E-21', '-0.0000000000000000000009999', 11, 5);
    T('1E-9', '1E-9', '0', 48, 4);
    T('1E-10', '1E-31', '0.0000000000999999999999999999999', 46, 5);
    T('1E-22', '0.001', '-0.0009999999999999999999', 81, 1);
    T('1E-17', '1E-22', '0.0000000000000000099999', 75, 3);
    T('0.0001', '1E-22', '0.0000999999999999999999', 43, 5);
    T('1E-10', '1E-40', '0.0000000000999999999999999999999999999999', 35, 3);
    T('1E-24', '1E-12', '-0.000000000000999999999999', 20, 1);
    T('1E-40', '1E-33', '-0.0000000000000000000000000000000009999999', 78, 5);
    T('1E-39', '1E-23', '-0.00000000000000000000001', 1, 6);
    T('0.1', '0.01', '0.09', 38, 5);
    T('1E-29', '0.0001', '-0.00009999999999999999999999999', 49, 6);
    T('1E-17', '1E-28', '0.0000000000000000099999999999', 100, 2);
    T('1E-27', '1E-23', '-0.000000000000000000000009999', 94, 5);
    T('1E-31', '1E-13', '-0.0000000000000999999999999999999', 40, 4);
    T('1E-34', '1E-9', '-0.0000000009999999999999999999999999', 87, 4);
    T('1E-27', '1E-40', '0.0000000000000000000000000009999999999999', 61, 6);
    T('1E-12', '1E-17', '0.00000000000099999', 42, 6);
    T('1E-12', '1E-33', '0.000000000000999999999999999999999', 33, 2);
    T('1E-13', '1E-33', '0.000000000000099999999', 8, 3);
    T('1E-7', '1E-13', '0.0000001', 2, 4);
    T('0.000001', '1E-23', '0.00000099999999999999999', 56, 0);
    T('1E-8', '1E-18', '0.000000009999999999', 88, 2);
    T('1E-39', '1E-22', '-0.000000000000000000000099999999999999999', 48, 0);
    T('1E-19', '1E-9', '-0.000000001', 1, 3);
    T('1E-30', '1E-16', '-0.000000000000000099999999999999', 81, 0);
    T('1E-24', '1E-33', '0.000000000000000000000001', 6, 6);
    T('0.00001', '1E-9', '0.000009999', 56, 3);
    T('0.1', '1E-15', '0.099999999999999', 29, 2);
    T('0.0001', '1E-15', '0.000099999999999', 60, 2);
    T('1E-17', '1E-39', '0.00000000000000001', 4, 2);
    T('1E-40', '1E-8', '-0.00000001', 23, 5);
    T('1E-8', '0.1', '-0.09999999', 79, 0);
    T('1E-36', '1E-37', '0.0000000000000000000000000000000000009', 90, 0);
    T('1E-21', '1E-28', '0.0000000000000000000009999999', 76, 2);
    T('1E-31', '1E-21', '-0.0000000000000000000009999999999', 37, 4);
    T('1E-24', '1E-9', '-0.000000000999999999999999', 32, 1);
    T('1E-15', '1E-25', '0.000000000000001', 8, 4);
    T('1E-37', '1E-9', '-0.0000000009999999999999999999999999999', 44, 0);
    T('0.01', '1E-40', '0.01', 12, 4);
    T('1E-21', '1E-16', '-0.000000000000000099999', 43, 3);
    T('1E-8', '1E-30', '0.000000009999999999999999999999', 74, 3);
    T('1E-10', '1E-36', '0.000000000099999999999999999999999999', 97, 1);
    T('1E-17', '1E-32', '0.00000000000000000999999999999999', 55, 3);
    T('1E-37', '1E-25', '-0.0000000000000000000000000999999999999', 47, 4);
    T('1E-36', '1E-39', '0.000000000000000000000000000000000000999', 16, 6);
    T('1E-39', '0.00001', '-0.000009999999999999999999999999999999999', 46, 5);
    T('1E-24', '1E-23', '-0.000000000000000000000009', 74, 3);
    T('0.01', '1E-24', '0.009999999999999999999999', 87, 3);
    T('1E-14', '0.00001', '-0.00000999999999', 17, 3);
    T('0.000001', '1E-38', '0.00000099999999999999999999999999999999', 88, 0);
    T('0.1', '1E-21', '0.099999999999999999999', 35, 3);
    T('1E-14', '1E-38', '0.00000000000000999999999999999999999999', 43, 0);
    T('1E-11', '1E-37', '0.00000000001', 4, 6);
    T('1E-31', '1E-31', '0', 97, 3);
    T('1E-32', '0.1', '-0.09999999999999999999999999999999', 55, 5);
    T('1E-18', '1E-14', '-0.000000000000009999', 6, 0);
    T('1E-32', '1E-12', '-0.00000000000099999999999999999999', 67, 3);
    T('1E-14', '1E-38', '0.00000000000000999999999999999999999999', 65, 1);
    T('1E-38', '1E-20', '-0.00000000000000000000999999999999999999', 20, 0);
    T('1E-27', '1E-15', '-0.0000000000000009999999999', 10, 2);
    T('1E-25', '0.001', '-0.0009999', 4, 2);
    T('1E-9', '0.01', '-0.009999999', 10, 0);
    T('0.1', '0.000001', '0.099999', 9, 2);
    T('1E-23', '1E-12', '-0.00000000000099999999999', 83, 2);
    T('0.001', '1E-19', '0.0009999999999999999', 71, 0);
    T('1E-26', '1E-13', '-0.00000000000009999999999999', 51, 1);
    T('1E-21', '1E-12', '-0.000000000000999999999', 73, 2);
    T('1E-35', '1E-38', '0.00000000000000000000000000000000000999', 42, 1);
    T('1E-21', '1E-27', '0.000000000000000000000999999', 96, 2);
    T('1E-30', '1E-11', '-0.000000000009999999999999999999', 95, 4);
    T('1E-23', '1E-10', '-0.00000000009999999999999', 79, 1);
    T('1E-29', '0.1', '-0.09999999999999999999999999999', 49, 2);
    T('0.000001', '1E-36', '0.000000999999999999999999999999999999', 33, 6);
    T('1E-22', '1E-29', '0.00000000000000000000009999999', 61, 3);
    T('1E-26', '1E-9', '-0.00000000099999999999999999', 97, 2);
    T('1E-30', '1E-7', '-0.000000099999999', 8, 1);
    T('1E-22', '0.1', '-0.1', 3, 4);
    T('1E-34', '1E-30', '-0.0000000000000000000000000000009999', 52, 0);
    T('1E-34', '1E-40', '0.0000000000000000000000000000000000999999', 9, 6);
    T('0.00001', '1E-22', '0.00001', 4, 5);
    T('1E-20', '0.1', '-0.09999999999999999999', 27, 4);
    T('1E-25', '0.1', '-0.09', 1, 2);
    T('1E-16', '1E-38', '0.0000000000000001', 12, 0);
    T('1E-17', '1E-12', '-0.00000000000099999', 33, 4);
    T('1E-30', '1E-31', '0.0000000000000000000000000000009', 6, 5);
    T('1E-30', '1E-9', '-0.000000000999999999999999999999', 28, 6);
    T('1E-10', '1E-26', '0.00000000009999999999999999', 27, 4);
    T('1E-7', '1E-35', '0.00000009999999999999999999999999999', 43, 5);
    T('0.01', '0.001', '0.009', 4, 6);
    T('0.00001', '1E-32', '0.00000999999999999', 12, 3);
    T('1E-16', '1E-35', '0.00000000000000009999999999999999999', 34, 2);
    T('1E-8', '0.001', '-0.00099999', 70, 4);
    T('1E-21', '0.0001', '-0.0001', 11, 6);
    T('0.00001', '1E-14', '0.00000999999999', 10, 4);
    T('1E-18', '1E-30', '0.000000000000000001', 5, 0);
    T('1E-39', '1E-22', '-0.000000000000000000000099999999999999999', 92, 2);
    T('1E-29', '1E-19', '-0.00000000000000000009999999999', 100, 5);
    T('1E-11', '0.1', '-0.09999999999', 60, 0);
    T('1E-36', '1E-24', '-0.000000000000000000000000999999999999', 55, 1);
    T('1E-18', '1E-12', '-0.000000000000999999', 82, 5);
    T('1E-33', '1E-36', '0.000000000000000000000000000000000999', 49, 2);
    T('1E-35', '1E-33', '-0.00000000000000000000000000000000099', 57, 1);
    T('1E-7', '0.01', '-0.0099999', 34, 6);
    T('1E-7', '1E-30', '0.000000099999999999999999999999', 74, 4);
    T('1E-25', '1E-31', '0.0000000000000000000000000999999', 18, 6);
    T('1E-16', '1E-39', '0.0000000000000001', 5, 4);
    T('1E-26', '1E-7', '-0.0000001', 8, 6);
    T('1E-28', '0.1', '-0.1', 4, 5);
    T('1E-26', '1E-16', '-0.00000000000000009999999999', 75, 4);
    T('1E-10', '1E-37', '0.0000000000999999999999999999999999999', 88, 4);
    T('1E-18', '1E-32', '0.00000000000000000099999999999999', 62, 3);
    T('1E-40', '1E-27', '-0.0000000000000000000000000009999999999999', 22, 0);
    T('1E-20', '1E-11', '-0.00000000000999999999', 37, 3);
    T('1E-11', '1E-32', '0.00000000001', 3, 0);
    T('1E-37', '1E-11', '-0.0000000000099999999999999999999999999', 46, 4);
    T('1E-12', '1E-34', '0.0000000000009999999999999999999999', 27, 5);
    T('1E-11', '1E-12', '0.000000000009', 71, 0);
    T('1E-25', '1E-36', '0.000000000000000000000000099999999999', 61, 2);
    T('1E-16', '1E-8', '-0.0000000099999999', 30, 3);
    T('1E-15', '1E-20', '0.00000000000000099999', 39, 2);
    T('0.001', '1E-36', '0.000999999999999999999999999999999999', 83, 0);
    T('0.00001', '0.1', '-0.09999', 5, 6);
    T('1E-7', '1E-34', '0.0000000999999999999999999999999999', 99, 1);
    T('1E-10', '1E-25', '0.0000000000999999999999999', 47, 5);
    T('1E-23', '1E-26', '0.00000000000000000000000999', 76, 4);
    T('1E-23', '1E-19', '-0.00000000000000000009999', 100, 1);
    T('1E-13', '1E-36', '0.000000000000099999999999999999999999', 94, 5);
    T('1E-12', '1E-36', '0.000000000000999999999999999999999999', 51, 4);
    T('1E-33', '1E-35', '0.00000000000000000000000000000000099', 72, 6);
    T('1E-18', '1E-24', '0.000000000000000000999999', 18, 5);
    T('1E-25', '1E-18', '-0.0000000000000000009999999', 53, 3);
    T('1E-22', '1E-12', '-0.0000000000009999999999', 30, 1);
    T('1E-14', '1E-33', '0.000000000000009999999999999999999', 65, 3);
    T('1E-40', '1E-11', '-0.0000000000099999999999999999999999999999', 47, 4);
    T('1E-14', '1E-39', '0.000000000000009999999999999999999999999', 45, 6);
    T('1E-12', '0.000001', '-0.000000999999', 94, 3);
    T('1E-16', '1E-29', '0.0000000000000001', 2, 6);
    T('1E-8', '1E-27', '0.0000000099999', 5, 3);
    T('1E-10', '1E-18', '0.000000000099999999', 95, 4);
    T('1E-11', '1E-30', '0.000000000009999999999999999999', 96, 2);
    T('0.00001', '1E-38', '0.00001', 14, 2);
    T('0.0001', '1E-30', '0.000099999999999999999999999999', 42, 0);
    T('1E-9', '1E-23', '0.00000000099999999999999', 14, 2);
    T('1E-8', '0.1', '-0.09999999', 43, 1);
    T('1E-31', '1E-40', '0.0000000000000000000000000000000999999999', 47, 1);
    T('1E-23', '1E-19', '-0.00000000000000000009999', 44, 3);
    T('1E-32', '1E-22', '-0.00000000000000000000009999999999', 26, 5);
    T('1E-16', '1E-40', '0.0000000000000000999999999999999999999999', 34, 6);
    T('1E-12', '1E-10', '-0.000000000099', 9, 1);
    T('1E-8', '1E-10', '0.000000009', 1, 1);
    T('1E-33', '1E-11', '-0.000000000009999999999999999999999', 58, 6);
    T('1E-29', '1E-40', '0.0000000000000000000000000000099999999999', 62, 1);
    T('0.0001', '0.1', '-0.0999', 41, 2);
    T('1E-21', '1E-38', '0.00000000000000000000099999999999999999', 73, 6);
    T('1E-9', '0.00001', '-0.000009999', 42, 5);
    T('1E-14', '1E-9', '-0.00000000099999', 23, 6);
    T('1E-40', '1E-11', '-0.0000000000099999999999999999999999999999', 60, 6);
    T('1E-33', '1E-37', '0.0000000000000000000000000000000009999', 19, 1);
    T('1E-21', '1E-8', '-0.000000009999999999999', 72, 0);
    T('1E-10', '1E-38', '0.0000000001', 6, 5);
    T('1E-27', '1E-16', '-0.000000000000000099999999999', 95, 6);
    T('1E-17', '1E-7', '-0.00000009999999999', 22, 5);
    T('1E-12', '0.1', '-0.099999999999', 81, 3);
    T('1E-18', '1E-39', '0.000000000000000000999999999999999999999', 49, 6);
    T('1E-13', '1E-9', '-0.0000000009999', 97, 5);
    T('1E-35', '1E-24', '-0.00000000000000000000000099999999999', 37, 4);
    T('1E-25', '1E-24', '-0.0000000000000000000000009', 27, 1);
    T('0.0001', '1E-21', '0.000099999999999999999', 44, 5);
    T('1E-27', '0.000001', '-0.000000999999999999999999999', 92, 0);
    T('1E-8', '0.001', '-0.00099999', 46, 1);
    T('1E-33', '1E-36', '0.000000000000000000000000000000000999', 39, 1);
    T('0.00001', '0.000001', '0.000009', 30, 5);
    T('0.01', '1E-11', '0.00999999999', 62, 6);
    T('0.00001', '1E-12', '0.000009999999', 14, 3);
    T('0.1', '1E-34', '0.0999999999999999999999999999999999', 44, 6);
    T('1E-15', '1E-19', '0.0000000000000009999', 69, 1);
    T('1E-28', '1E-30', '0.000000000000000000000000000099', 32, 2);
    T('1E-32', '1E-19', '-0.00000000000000000009999999999999', 93, 5);
    T('1E-14', '1E-16', '0.0000000000000099', 32, 4);
    T('1E-27', '1E-35', '0.00000000000000000000000000099999999', 23, 1);
    T('1E-21', '1E-14', '-0.000000000000009999999', 55, 4);
    T('1E-20', '1E-35', '0.00000000000000000000999999999999999', 38, 4);
    T('1E-11', '1E-33', '0.00000000001', 21, 2);
    T('1E-37', '1E-31', '-0.0000000000000000000000000000000999999', 86, 3);
    T('1E-20', '1E-14', '-0.00000000000000999999', 35, 3);
    T('0.0001', '1E-18', '0.000099999999999999', 22, 5);
    T('1E-13', '1E-30', '0.000000000000099999999999999999', 30, 1);
    T('0.0001', '0.0001', '0', 19, 2);
    T('1E-38', '1E-25', '-0.00000000000000000000000009999999999999', 18, 0);
    T('1E-25', '1E-29', '0.00000000000000000000000009999', 15, 2);
    T('1E-15', '1E-15', '0', 8, 4);
    T('1E-36', '1E-25', '-0.000000000000000000000000099999999999', 88, 6);
    T('1E-33', '1E-15', '-0.000000000000000999999999999999999', 86, 6);
    T('1E-39', '1E-22', '-0.000000000000000000000099999999999999999', 19, 0);
    T('0.000001', '1E-30', '0.000000999999999999999999999999', 100, 4);
    T('1E-19', '1E-31', '0.0000000000000000000999999999999', 22, 0);
    T('0.00001', '1E-16', '0.000009999', 4, 1);
    T('1E-16', '0.1', '-0.0999999999999999', 91, 5);
    T('1E-23', '1E-11', '-0.00000000001', 9, 3);
    T('0.000001', '1E-16', '0.0000009999999999', 94, 5);
    T('1E-9', '1E-31', '0.0000000009999999999999999999999', 29, 2);
    T('1E-8', '1E-8', '0', 56, 2);
    T('1E-17', '0.00001', '-0.00000999999999999', 32, 6);
    T('1E-30', '1E-34', '0.0000000000000000000000000000009999', 27, 6);
    T('1E-13', '1E-21', '0.000000000000099999999', 16, 1);
    T('1E-14', '0.1', '-0.09999999999999', 56, 2);
    T('1E-34', '1E-7', '-0.0000000999999999999999999999999999', 35, 2);
    T('1E-38', '1E-15', '-0.00000000000000099999999999999999999999', 100, 0);
    T('1E-29', '1E-39', '0.000000000000000000000000000009999999999', 99, 2);
    T('1E-15', '0.00001', '-0.000009999999999', 38, 2);
    T('1E-10', '1E-39', '0.000000000099999999999999999999999999999', 31, 3);
    T('1E-15', '1E-15', '0', 53, 0);
    T('1E-39', '1E-39', '0', 44, 4);
    T('1E-13', '0.01', '-0.0099999999999', 60, 4);
    T('1E-18', '1E-12', '-0.000000000000999999', 16, 3);
    T('1E-12', '1E-40', '0.0000000000009999999999999999999999999999', 48, 1);
    T('1E-15', '1E-11', '-0.000000000009999', 8, 1);
    T('1E-7', '1E-24', '0.000000099999999999999999', 73, 2);
    T('1E-36', '1E-23', '-0.000000000000000000000009', 1, 2);
    T('1E-40', '1E-32', '-0.0000000000000000000000000000000099999999', 32, 5);
    T('1E-28', '1E-37', '0.0000000000000000000000000000999999999', 29, 6);
    T('1E-12', '0.000001', '-0.000000999999', 14, 5);
    T('1E-21', '0.001', '-0.000999999999999999999', 23, 6);
    T('1E-39', '0.000001', '-0.000000999999999999999999999999999999999', 77, 1);
    T('1E-23', '0.00001', '-0.00000999999999999999999', 24, 5);
    T('1E-25', '1E-28', '0.0000000000000000000000000999', 27, 2);
    T('1E-30', '1E-29', '-0.000000000000000000000000000009', 86, 1);
    T('1E-18', '1E-26', '0.00000000000000000099999999', 95, 6);
    T('1E-23', '1E-18', '-0.00000000000000000099999', 19, 4);
    T('1E-21', '0.001', '-0.000999999999999999999', 81, 4);
    T('1E-40', '1E-7', '-0.0000000999999999999999999999999999999999', 69, 1);
    T('1E-8', '0.001', '-0.00099999', 97, 5);
    T('0.00001', '1E-32', '0.00000999999999999999999999999999', 39, 0);
    T('1E-36', '1E-7', '-0.000000099999999999999999999999999999', 63, 4);
    T('1E-38', '1E-12', '-0.00000000000099999999999999999999999999', 47, 6);
    T('1E-35', '1E-10', '-0.00000000009999999999999999999999999', 53, 5);
    T('1E-15', '1E-16', '0.0000000000000009', 69, 1);
    T('1E-32', '1E-10', '-0.00000000009999999999999999999999', 39, 1);
    T('1E-15', '1E-24', '0.000000000000001', 1, 4);
    T('1E-33', '0.1', '-0.099999999999999999999999999999999', 47, 0);
    T('1E-26', '1E-10', '-0.00000000009999999999999999', 57, 4);
    T('1E-21', '1E-23', '0.00000000000000000000099', 16, 5);
    T('0.000001', '1E-39', '0.000000999999999999999999999999999999999', 37, 4);
    T('1E-21', '1E-13', '-0.000000000000099999999', 36, 4);
    T('1E-15', '1E-16', '0.0000000000000009', 68, 4);
    T('1E-21', '1E-29', '0.00000000000000000000099999999', 31, 0);
    T('0.00001', '1E-35', '0.00000999999999999999999999999999999', 37, 1);
    T('1E-29', '1E-7', '-0.00000009999999999999999999999', 29, 0);
    T('1E-36', '1E-19', '-0.000000000000000000099999999999999999', 69, 3);
    T('1E-37', '1E-34', '-0.0000000000000000000000000000000000999', 71, 1);
    T('1E-7', '0.00001', '-0.0000099', 98, 3);
    T('1E-7', '0.000001', '-0.0000009', 76, 4);
    T('1E-11', '1E-40', '0.0000000000099999999999999999999999999999', 68, 6);
    T('0.1', '1E-7', '0.0999999', 61, 1);
    T('1E-25', '1E-35', '0.00000000000000000000000009999999999', 46, 4);
    T('1E-20', '0.000001', '-0.00000099999999999999', 42, 0);
    T('1E-9', '1E-13', '0.0000000009999', 48, 2);
    T('1E-38', '1E-20', '-0.00000000000000000000999999999999999999', 93, 5);
    T('0.0001', '1E-17', '0.00009999999999999', 30, 4);
    T('0.00001', '1E-9', '0.000009999', 80, 5);
    T('1E-40', '0.1', '-0.1', 8, 0);
    T('1E-39', '0.000001', '-0.000001', 5, 6);
    T('1E-7', '0.000001', '-0.0000009', 86, 2);
    T('1E-27', '1E-31', '0.0000000000000000000000000009999', 80, 6);
    T('1E-24', '1E-22', '-0.000000000000000000000099', 4, 2);
    T('1E-11', '1E-14', '0.00000000000999', 67, 2);
    T('1E-19', '1E-25', '0.0000000000000000000999999', 84, 5);
    T('1E-17', '1E-10', '-0.00000000009999999', 75, 2);
    T('1E-12', '1E-27', '0.000000000000999999999999999', 66, 6);
    T('1E-19', '0.0001', '-0.0000999999999999999', 39, 4);
    T('1E-19', '1E-7', '-0.0000000999999999999', 40, 6);
    T('1E-25', '1E-25', '0', 52, 0);
    T('1E-20', '1E-40', '0.0000000000000000000099999999999999999999', 71, 6);
    T('1E-37', '1E-31', '-0.0000000000000000000000000000000999999', 66, 5);
    T('0.01', '1E-36', '0.009999999999999999999999999999999999', 88, 6);
    T('1E-26', '0.1', '-0.09999999999999999999999999', 70, 0);
    T('1E-15', '1E-35', '0.00000000000000099999999999999999999', 87, 1);
    T('1E-15', '1E-38', '0.00000000000000099999999999999999999999', 67, 5);
    T('1E-10', '1E-31', '0.0000000000999999999999999999999', 47, 0);
    T('1E-23', '1E-40', '0.0000000000000000000000099999999999999999', 78, 3);
    T('1E-16', '1E-37', '0.0000000000000000999999999999999999999', 35, 3);
    T('1E-20', '1E-8', '-0.00000000999999999999', 55, 5);
    T('1E-38', '1E-18', '-0.00000000000000000099999999999999999999', 44, 3);
    T('1E-37', '1E-39', '0.000000000000000000000000000000000000099', 53, 2);
    T('1E-35', '1E-14', '-0.00000000000000999999999999999999999', 74, 2);
    T('1E-33', '0.0001', '-0.000099999999999999999999999999999', 94, 6);
    T('1E-13', '1E-32', '0.00000000000009999999999999999999', 30, 0);
    T('1E-7', '0.01', '-0.01', 1, 0);
    T('1E-40', '1E-33', '-0.0000000000000000000000000000000009999999', 75, 0);
    T('0.0001', '1E-17', '0.00009999999999999', 61, 4);
    T('1E-7', '1E-14', '0.00000009999999', 8, 5);
    T('1E-36', '1E-34', '-0.000000000000000000000000000000000099', 21, 3);
    T('1E-32', '1E-15', '-0.00000000000000099999999999999999', 47, 0);
    T('1E-9', '1E-40', '0.0000000009999999999999999999999999999999', 45, 5);
    T('1E-19', '1E-9', '-0.0000000009999999', 7, 1);
    T('0.0001', '1E-18', '0.000099999999999999', 59, 0);
    T('1E-17', '1E-25', '0.0000000000000000099999999', 99, 4);
    T('1E-29', '0.1', '-0.09999999999999', 13, 1);
    T('1E-18', '1E-37', '0.000000000000000001', 6, 4);
    T('1E-17', '1E-36', '0.000000000000000009999999999999', 13, 3);
    T('0.1', '1E-22', '0.0999999999999999999999', 78, 4);
    T('1E-9', '1E-14', '0.00000000099999', 53, 0);
    T('1E-34', '1E-26', '-0.0000000000000000000000000099999999', 81, 1);
    T('1E-21', '0.1', '-0.099999999999999999999', 71, 2);
    T('1E-7', '1E-39', '0.000000099999999999999999999999999999999', 55, 4);
    T('1E-33', '1E-18', '-0.000000000000000000999999999999999', 38, 5);
    T('0.001', '1E-23', '0.00099999999999999999999', 90, 1);
    T('1E-38', '1E-20', '-0.00000000000000000000999999999999999999', 75, 5);
    T('0.00001', '1E-7', '0.0000099', 47, 4);
    T('0.0001', '0.000001', '0.000099', 39, 4);
    T('0.00001', '1E-29', '0.00000999999999999999999999999', 100, 0);
    T('1E-9', '0.1', '-0.099999999', 57, 1);
    T('1E-32', '1E-24', '-0.00000000000000000000000099999999', 70, 6);
    T('1E-12', '1E-19', '0.0000000000009999999', 61, 5);
    T('1E-34', '0.0001', '-0.0001', 22, 5);
    T('1E-7', '0.1', '-0.0999999', 88, 6);
    T('1E-37', '1E-26', '-0.0000000000000000000000000099999999999', 73, 4);
    T('1E-11', '1E-40', '0.0000000000099999999999999999999999999999', 30, 1);
    T('1E-37', '1E-16', '-0.0000000000000000999', 3, 2);
    T('1E-30', '1E-20', '-0.000000000000000000009999999999', 87, 2);
    T('1E-28', '1E-10', '-0.0000000000999999999999999999', 53, 1);
    T('1E-39', '1E-17', '-0.000000000000000009999999999999999999999', 70, 6);
    T('1E-34', '1E-16', '-0.0000000000000000999999999999999999', 41, 4);
    T('1E-24', '1E-11', '-0.000000000009999999999999', 38, 3);
    T('1E-29', '1E-35', '0.00000000000000000000000000000999999', 84, 5);
    T('1E-26', '1E-25', '-0.00000000000000000000000009', 57, 4);
    T('0.000001', '1E-40', '0.0000009999999999999999999999999999999999', 82, 6);
    T('1E-31', '0.000001', '-0.0000009999999999999999999999999', 33, 6);
    T('1E-10', '1E-40', '0.0000000000999999999999999999999999999999', 62, 5);
    T('1E-14', '1E-38', '0.00000000000000999999999999999999999999', 32, 2);
    T('1E-39', '1E-10', '-0.000000000099999999999999999999999999999', 55, 4);
    T('1E-10', '1E-19', '0.0000000000999999999', 50, 1);
    T('1E-15', '0.1', '-0.099999999999999', 20, 5);
    T('1E-32', '1E-37', '0.0000000000000000000000000000000099999', 98, 2);
    T('1E-15', '1E-8', '-0.0000000099999', 5, 1);
    T('1E-17', '1E-30', '0.000000000000000009999999999999', 76, 6);
    T('1E-15', '1E-25', '0.0000000000000009999999999', 22, 1);
    T('1E-7', '1E-23', '0.00000009999999999999999', 87, 1);
    T('1E-35', '1E-34', '-0.00000000000000000000000000000000009', 65, 2);
    T('1E-20', '1E-35', '0.00000000000000000000999999999999999', 50, 6);
    T('1E-24', '1E-25', '0.0000000000000000000000009', 92, 5);
    T('1E-9', '1E-10', '0.0000000009', 100, 3);
    T('1E-30', '1E-19', '-0.000000000000000000099999999999', 95, 1);
    T('1E-19', '1E-11', '-0.0000000000099999999', 99, 0);
    T('1E-7', '1E-15', '0.000000099999999', 52, 4);
    T('0.0001', '1E-34', '0.0000999999999999999999999999999999', 52, 6);
    T('1E-15', '1E-29', '0.00000000000000099999999999999', 29, 2);
    T('1E-40', '1E-17', '-0.0000000000000000099999999999999999999999', 80, 0);
    T('0.0001', '1E-29', '0.0001', 8, 0);
    T('1E-23', '1E-22', '-0.00000000000000000000009', 52, 1);
    T('1E-33', '0.000001', '-0.000000999999999999999999999999999', 73, 1);
    T('1E-24', '1E-7', '-0.000000099999999999999999', 39, 5);
    T('0.001', '1E-34', '0.0009999999999999999999999999999999', 95, 0);
    T('1E-21', '1E-38', '0.00000000000000000000099999999999999999', 98, 3);
    T('1E-25', '1E-14', '-0.0000000000000099999999999', 63, 6);
    T('1E-15', '0.00001', '-0.000009999999999', 28, 0);
    T('1E-24', '1E-23', '-0.000000000000000000000009', 34, 5);
    T('1E-32', '1E-30', '-0.00000000000000000000000000000099', 56, 0);
    T('1E-26', '1E-12', '-0.00000000000099999999999999', 65, 5);
    T('1E-39', '1E-30', '-0.000000000000000000000000000000999999999', 63, 0);
    T('1E-36', '1E-9', '-0.000000000999999999999999999999999999', 71, 0);
    T('1E-17', '1E-37', '0.0000000000000000099999999999999999999', 56, 1);
    T('1E-9', '1E-30', '0.000000000999999999999999999999', 21, 1);
    T('1E-33', '1E-21', '-0.000000000000000000000999999999999', 34, 1);
    T('1E-30', '0.1', '-0.1', 25, 4);
    T('1E-10', '1E-35', '0.0000000001', 19, 2);
    T('1E-29', '0.000001', '-0.00000099999999999999999999999', 71, 0);
    T('0.001', '1E-30', '0.000999999999999999999999999999', 41, 6);
    T('1E-26', '1E-14', '-0.00000000000000999999999999', 38, 4);
    T('0.01', '1E-37', '0.0099999999999999999999999999999999999', 41, 0);
    T('1E-17', '1E-32', '0.00000000000000000999999999999999', 33, 1);
    T('1E-16', '1E-23', '0.00000000000000009999999', 62, 5);
    T('0.00001', '1E-31', '0.00001', 13, 5);
    T('1E-13', '1E-22', '0.0000000000000999999999', 65, 6);
    T('1E-9', '1E-10', '0.0000000009', 72, 4);
    T('1E-32', '1E-8', '-0.00000000999999999999999999999999', 63, 1);
    T('1E-15', '1E-17', '0.00000000000000099', 77, 6);
    T('1E-10', '0.1', '-0.0999999999', 38, 6);
    T('1E-35', '1E-10', '-0.00000000009999999999999999999999999', 38, 5);
    T('0.00001', '0.00001', '0', 51, 4);
    T('0.1', '1E-31', '0.09999999999999999999999999', 25, 1);
    T('1E-15', '1E-7', '-0.000000099999999', 78, 5);
    T('1E-14', '1E-27', '0.000000000000009999999999999', 79, 5);
    T('1E-16', '0.000001', '-0.0000009999999999', 96, 2);
    T('0.000001', '1E-31', '0.0000009999999999999999999999999', 89, 6);
    T('0.001', '1E-29', '0.001', 24, 2);
    T('1E-36', '0.001', '-0.000999999999999999999999999999999999', 45, 3);
    T('0.000001', '1E-38', '0.000001', 2, 4);
    T('1E-33', '1E-40', '0.0000000000000000000000000000000009999999', 88, 1);
    T('1E-14', '0.0001', '-0.00009999999999', 63, 3);
    T('1E-30', '1E-24', '-0.000000000000000000000000999999', 21, 4);
    T('1E-34', '0.1', '-0.1', 11, 0);
    T('0.01', '1E-16', '0.0099999999999999', 37, 1);
    T('1E-25', '1E-36', '0.000000000000000000000000099999999999', 44, 5);
    T('1E-33', '1E-37', '0.0000000000000000000000000000000009999', 86, 2);
    T('1E-38', '1E-21', '-0.00000000000000000000099999999999999999', 21, 2);
    T('1E-34', '0.1', '-0.0999999999999999999999999999999999', 73, 6);
    T('1E-9', '1E-31', '0.0000000009999999999999999999999', 65, 5);
    T('1E-13', '1E-17', '0.00000000000009999', 10, 5);
    T('1E-17', '1E-26', '0.00000000000000000999999999', 96, 1);
    T('1E-17', '1E-35', '0.00000000000000000999999999999999999', 18, 1);
    T('1E-40', '1E-10', '-0.0000000000999999999999999999999999999999', 88, 5);
    T('1E-7', '1E-24', '0.000000099999999999999999', 96, 6);
    T('1E-38', '0.1', '-0.09999999999999999999999999999999999999', 56, 4);
    T('0.01', '1E-40', '0.0099999999999999999999999999999999999999', 52, 1);
    T('1E-34', '1E-18', '-0.0000000000000000009999999999999999', 49, 2);
    T('0.0001', '1E-33', '0.000099999999999999999999999999999', 95, 6);
    T('1E-27', '1E-20', '-0.000000000000000000009999999', 31, 5);
    T('0.000001', '0.00001', '-0.000009', 73, 6);
    T('1E-10', '1E-25', '0.0000000000999999999999999', 22, 6);
    T('1E-29', '1E-28', '-0.00000000000000000000000000009', 22, 3);
    T('0.001', '1E-38', '0.001', 25, 2);
    T('1E-33', '0.00001', '-0.000009999999999999999999999999999', 35, 0);
    T('1E-31', '1E-37', '0.0000000000000000000000000000000999999', 41, 1);
    T('1E-40', '1E-24', '-0.0000000000000000000000009999999999999999', 96, 1);
    T('1E-34', '1E-18', '-0.0000000000000000009999999999999999', 96, 5);
    T('1E-40', '0.1', '-0.1', 20, 5);
    T('1E-11', '1E-37', '0.00000000001', 7, 5);
    T('1E-20', '1E-37', '0.00000000000000000001', 16, 0);
    T('1E-25', '0.1', '-0.0999999999999999999999999', 69, 4);
    T('0.1', '1E-15', '0.099999999999999', 27, 3);
    T('1E-24', '1E-12', '-0.000000000001', 8, 5);
    T('1E-20', '1E-36', '0.000000000000000000009999999999999999', 34, 2);
    T('1E-38', '1E-35', '-0.00000000000000000000000000000000000999', 28, 0);
    T('1E-36', '1E-28', '-0.000000000000000000000000000099999999', 28, 6);
    T('1E-12', '0.1', '-0.099999999999', 67, 5);
    T('1E-38', '1E-26', '-0.00000000000000000000000000999999999999', 66, 6);
    T('1E-24', '1E-25', '0.0000000000000000000000009', 24, 2);
    T('1E-40', '1E-7', '-0.0000000999999999999999999999999999999999', 86, 6);
    T('0.1', '0.00001', '0.09999', 50, 0);
    T('1E-9', '1E-28', '0.0000000009999999999999999999', 40, 4);
    T('1E-27', '1E-38', '0.00000000000000000000000000099999999999', 60, 3);
    T('1E-26', '1E-14', '-0.00000000000000999999999999', 18, 5);
    T('1E-34', '1E-21', '-0.0000000000000000000009999999999999', 49, 2);
    T('1E-17', '1E-29', '0.00000000000000000999999999999', 90, 3);
    T('0.0001', '1E-33', '0.000099999999999999999999999999999', 54, 1);
    T('1E-17', '1E-18', '0.000000000000000009', 15, 5);
    T('1E-30', '1E-28', '-0.000000000000000000000000000099', 5, 3);
    T('0.01', '1E-9', '0.009999999', 92, 1);
    T('1E-31', '1E-7', '-0.0000000999999999999999999999999', 43, 4);
    T('1E-22', '0.001', '-0.001', 18, 4);
    T('0.1', '1E-37', '0.1', 17, 5);
    T('1E-16', '1E-16', '0', 68, 0);
    T('1E-11', '1E-18', '0.00000000001', 6, 5);
    T('1E-37', '1E-37', '0', 9, 4);
    T('0.001', '1E-28', '0.0009999999999999999999999999', 84, 2);
    T('1E-26', '1E-28', '0.0000000000000000000000000099', 100, 0);
    T('0.1', '1E-26', '0.09999999999999999999999999', 33, 0);
    T('1E-26', '1E-24', '-0.00000000000000000000000099', 62, 1);
    T('1E-23', '1E-24', '0.000000000000000000000009', 46, 3);
    T('1E-25', '0.00001', '-0.0000099999999999999999999', 67, 1);
    T('1E-18', '0.00001', '-0.000009999999999999', 64, 2);
    T('1E-18', '1E-34', '0.0000000000000000009999999999999999', 25, 1);
    T('1E-30', '1E-19', '-0.000000000000000000099999999999', 13, 3);
    T('1E-13', '0.01', '-0.0099999999999', 28, 3);
    T('0.000001', '1E-29', '0.00000099999999999999999999999', 75, 3);
    T('1E-38', '1E-30', '-0.00000000000000000000000000000099999999', 16, 5);
    T('0.1', '0.00001', '0.09999', 83, 3);
    T('1E-14', '1E-10', '-0.00000000009999', 18, 4);
    T('0.0001', '0.0001', '0', 58, 3);
    T('0.000001', '1E-35', '0.00000099999999999999999999999999999', 64, 0);
    T('0.001', '1E-31', '0.0009999999999999999999999999999', 75, 6);
    T('1E-32', '1E-23', '-0.00000000000000000000000999999999', 66, 1);
    T('1E-35', '1E-30', '-0.00000000000000000000000000000099999', 63, 3);
    T('0.1', '1E-22', '0.0999999999999999999999', 100, 3);
    T('1E-8', '1E-13', '0.0000000099999', 58, 4);
    T('0.000001', '1E-17', '0.000001', 6, 2);
    T('1E-20', '1E-35', '0.00000000000000000000999999999999999', 25, 3);
    T('1E-28', '1E-20', '-0.0000000000000000000099999999', 23, 0);
    T('0.0001', '0.01', '-0.0099', 4, 4);
    T('1E-19', '0.1', '-0.0999999999999999999', 44, 4);
    T('1E-23', '0.00001', '-0.00000999999999999999999', 69, 0);
    T('1E-35', '1E-32', '-0.00000000000000000000000000000000999', 65, 1);
    T('1E-38', '1E-29', '-0.00000000000000000000000000000999999999', 52, 3);
    T('0.0001', '1E-15', '0.00009999', 4, 3);
    T('1E-13', '1E-34', '0.0000000000000999999999999999999999', 45, 4);
    T('1E-7', '1E-26', '0.00000009999999999999999999', 83, 4);
    T('1E-18', '1E-9', '-0.000000000999999999', 19, 5);
    T('0.1', '1E-7', '0.0999999', 64, 5);
    T('1E-36', '1E-13', '-0.000000000000099999999999999999999999', 60, 3);
    T('1E-17', '0.01', '-0.00999999999999999', 85, 0);
    T('1E-18', '0.001', '-0.000999999999999999', 89, 3);
    T('1E-40', '1E-15', '-0.0000000000000009999999999999999999999999', 57, 3);
    T('0.000001', '1E-15', '0.000000999999999', 41, 4);
    T('1E-35', '1E-20', '-0.00000000000000000000999999999999999', 91, 6);
    T('1E-31', '1E-22', '-0.0000000000000000000000999999999', 62, 1);
    T('0.1', '1E-35', '0.09999999999999999999999999999999999', 55, 3);
    T('1E-8', '1E-28', '0.0000000099999999999999999999', 90, 3);
    T('1E-28', '1E-26', '-0.0000000000000000000000000099', 53, 2);
    T('1E-36', '1E-24', '-0.000000000000000000000000999999999999', 89, 1);
    T('1E-33', '1E-25', '-0.000000000000000000000000099999999', 92, 2);
    T('0.01', '1E-25', '0.0099999999999999999999999', 77, 2);
    T('1E-16', '1E-38', '0.00000000000000009999999999999999999999', 41, 6);
    T('1E-7', '1E-25', '0.0000000999999999999999999', 92, 6);
    T('1E-20', '0.00001', '-0.00000999999999999999', 49, 0);
    T('1E-40', '1E-16', '-0.0000000000000000999999999999999999999999', 30, 5);
    T('1E-29', '0.1', '-0.09999999999999999999999999999', 50, 0);
    T('1E-38', '1E-26', '-0.00000000000000000000000000999999999999', 93, 2);
    T('0.0001', '1E-17', '0.00009999999999999', 100, 1);
    T('1E-25', '0.01', '-0.0099999999999999999999999', 55, 5);
    T('1E-40', '1E-23', '-0.0000000000000000000000099999999999999999', 67, 6);
    T('1E-25', '1E-29', '0.00000000000000000000000009999', 50, 5);
    T('1E-32', '1E-31', '-0.00000000000000000000000000000009', 46, 2);
    T('0.1', '1E-37', '0.0999999999999999999999999999999999999', 87, 3);
    T('1E-15', '1E-21', '0.000000000000000999999', 18, 3);
    T('1E-30', '0.001', '-0.000999999999999999999999999999', 29, 3);
    T('1E-29', '1E-21', '-0.00000000000000000000099999999', 53, 5);
    T('1E-36', '1E-29', '-0.000000000000000000000000000009999999', 26, 5);
    T('0.001', '1E-11', '0.00099999999', 61, 1);
    T('1E-31', '0.001', '-0.001', 12, 0);
    T('1E-24', '1E-20', '-0.000000000000000000009999', 25, 2);
    T('1E-37', '1E-35', '-0.0000000000000000000000000000000000099', 98, 2);
    T('1E-17', '0.00001', '-0.00000999999999999', 95, 1);
    T('1E-27', '0.00001', '-0.00001', 14, 3);
    T('1E-37', '0.001', '-0.0009999999999999999999999999999999999', 70, 0);
    T('0.1', '0.00001', '0.09999', 33, 3);
    T('0.1', '1E-28', '0.0999999999999999999999999999', 80, 0);
    T('1E-28', '1E-36', '0.00000000000000000000000000009999999', 7, 3);
    T('1E-38', '1E-32', '-0.00000000000000000000000000000000999999', 41, 6);
    T('1E-39', '0.01', '-0.009999999999999999999999999999999999999', 99, 0);
    T('1E-17', '1E-21', '0.000000000000000009999', 62, 0);
    T('0.01', '1E-35', '0.00999999999999999999999999999999999', 51, 4);
    T('1E-8', '0.1', '-0.09999999', 99, 3);
    T('1E-21', '0.1', '-0.099999999999999999999', 65, 0);
    T('1E-25', '1E-29', '0.00000000000000000000000009999', 44, 4);
    T('1E-24', '1E-21', '-0.000000000000000000000999', 86, 6);
    T('1E-16', '1E-34', '0.0000000000000001', 3, 5);
    T('1E-35', '0.001', '-0.00099999999999999999999999999999999', 50, 1);
    T('1E-30', '1E-13', '-0.000000000000099999999999999999', 88, 6);
    T('1E-28', '0.1', '-0.1', 5, 4);

    T('0.9999999999999999999876881224', '640033848998621900.19735', '-640033848998621899.1973500000000000000123118776', 49, 2);
    T('1.000000000000000000000000119823914072125503157964', '3062705906358982193618.98485068689066024569704943195219', '-3062705906358982193617.984850686890660245697049312128275927874496842036', 94, 6);
    T('0.9999999999999998265534546200191793251275036241640334423212917258444307775885354012967962729546', '258319.60193860385931337782', '-258318.60193860385931355126654537998082067487249637583596655767870827415556922241146459870320372705', 98, 4);
    T('0.99999999999999999989761950386191857639251221379227189329555371776218724908031098142396778431218707', '726355355995140387099932991211229400.158448248205932318709', '-726355355995140387099932991211229399.1584482482059323188113804961380814236074877862077281067044463', 97, 5);
    T('1.000000000000000000000000000000000000027702350484831926292132972134993320024115135851747430016246', '79385320209954140084685835.28094025821251260106069305881382567515', '-79385320209954000000000000', 14, 4);
    T('0.99998712469413', '794547422607164224869232243086324.8360594446035869419718131828058', '-794547422607164224869232243086323.836072319909456942', 52, 3);
    T('0.9999999999998534962024501557924401736482182589661527', '5698985430281057199751490373953345194.59621391632', '-5698985430281057199751490373953345193.596213916320146503797549844207559826351781741', 82, 2);
    T('0.9999999999999999999999985533840413028010928652936714305576280330060940667532288645', '78657501050716794990357190861.70489425564556943595076975008', '-78657501050716794990357190000', 25, 5);
    T('0.9999999961220652049373184', '914035313573641721959451176671.848582838348529711402806', '-914035314000000000000000000000', 9, 5);
    T('1.11874482', '48840191373078238086640881533698465.717620778949', '-48840191373100000000000000000000000', 12, 5);
    T('0.281969520841', '525591380542776020976261376042231280.99050263561330377859863953', '-525591380542776020976261376042231280.70853311477230377859863953', 97, 6);
    T('0.9999999999999959369334762196188619130301649399733281630919124260101076514635673054327688686405515', '6892460359743232550483058284.326802388', '-6892460359743232550483058283.326802388000004063067', 49, 3);
    T('1.000000000000000000000000138365204598651999681848', '4.585694782001886784351261812739727', '-3.585694782001886784351261674374522401348000318152', 93, 4);
    T('1.00000000000000000001348089658161552137212593704802977308725958902187035801449881713928406671', '2599076526329799757363350602243099450.6957978861778433554878885203926487289', '-2599076526329800000000000000000000000', 14, 5);
    T('1.0000000000000', '66366654979619144184658698072192.357935388883183470438181288188381307278', '-66366650000000000000000000000000', 7, 5);
    T('1.000000', '18105728986041761640887595533283120477.2742082541073753474', '-18100000000000000000000000000000000000', 4, 1);
    T('0.9999', '560937701245564854434830710385849233.169675002587006894673914472266909', '-560937701245564854434830710385849232.169775002587006894673914472266909', 79, 2);
    T('0.9999998719213481023531897427716397778263888401893910733910220314504983682315253', '38792839790712300540097766016170179.72942773540650288197576877282', '-38792839790712300540097766016170178.72942786348515477', 52, 2);
    T('1.000000000000000000000000000000011337132819044149107', '97753977764753722505638575181479019.579624307127638397405681', '-97753977764753722505638575181479018.579624307127638397405681', 60, 4);
    T('1.01261395071914172725658272307075493652694434213346101402984', '1049710327243884264.3689519321', '-1049710327243884263.35633798138085827274341727692924506347305565786653898597016', 95, 6);
    T('0.99999999998635625720009473958665342273155724466867056964747162778310004832306258429717336387169', '12.1520940727652904005353452973720184', '-11.1520940727789341433352505577853649772684427553313294303525283722168999516769374157', 84, 2);
    T('0.999999999999999999999999999999999524622195377650474620654559769514364455807304047052', '7002515.6669052486740556972355374422866181829657', '-7002514.6669052486740556972355374', 32, 5);
    T('0.99999998878183120416824285037203', '38327.8407', '-38326.8407000112181687958317', 27, 1);
    T('0.999999999999999999999999999', '2.9094462300972', '-1.909446230097200000000000001', 55, 4);
    T('0.9999999999999999999999999996779494408046302584202610482', '530770914537012205776892173.865394333047000705228144653463710904', '-530770914537012205776892172.8653943330470007052281446537857614631953697415797389518', 92, 1);
    T('1.0000000000000000000000000000000000000211924266373601875662343025201383719314762448306414697', '33086682481347.6314289196308731545826716', '-33086682481346.63142891963087315458267159999999999997880757336264', 64, 6);
    T('0.9999999999999999999999999999999999999995086077216389080791500865', '43010632125906095550264.572650474808124', '-43010632125906095550263.572650474808124', 51, 4);
    T('0.999999999999999999999999999999392910865226086344321', '4584324554085375538027273692534240749302.742306700543168212764946899734664', '-4584324554085375538027273692534240749301.742306700543168212764946899735271089134773913655679', 91, 3);
    T('0.9999999999999999999999999999999999999921018515820697887426805224100947352172147724109795779', '830076114723591620059141676694501404709.4930298672', '-830076114723591620059141676694501404708.4930298672', 70, 4);
    T('1.000154688825648481267644205561160017273389804317525860318979525925354', '5590.1166644928260', '-5589.1165098040003515187323557944388', 35, 1);
    T('0.99999999999006027143696724994249662', '88359515632902648.759516639917666812111537547', '-88359515632902600', 15, 1);
    T('1.00000000000000', '95.33659318661235', '-94.33659318661235', 70, 4);
    T('0.9999999999999999999775634801237813332795738416510297362767757177787528', '10739394560873313157162707656670.252363942865247660125793434721595385', '-10739394560873313157163000000000', 23, 6);
    T('0.99999998955431286156626', '4902.7915368894281469434', '-4901.79153689987383408183374', 80, 2);
    T('0.99999999999999898259041082937473685908796752941185241324258874667581114315116781', '5517546011621870332399416576575.811785782758678907878062576', '-5517546011621870332399416576574.81178578275867992528765174662526314091203247058814758675741126', 93, 0);
    T('1.13135445288218095176064019413277569648722343676398316921046471781497209018036761926374980678', '3619072849985061077293.9012666679115988775', '-3619072849985061077292.76991222', 30, 4);
    T('0.999964371653162345075430201881', '168105725527169314379065790662916050221.475269915782886990770332731552414180376', '-168105725527169314379066000000000000000', 24, 5);
    T('0.999999999998343581666045', '21225778421730442449952304083.507476189532490354504704824693390193', '-21225778422000000000000000000', 11, 4);
    T('1.0000000000', '53.0721765572571058497526522442', '-52.0721765572571058497526522442', 68, 1);
    T('1.0000000000000000010868737982065976552678875918917937663867883830744', '72274249298653961307161266.2429171261996071156986500328106121752', '-72274249298653961307161265.2429171261996071146117762346040145199321124081082062336132116', 87, 1);
    T('1.00000000000000000000000000000000001026071005118758019147403591153361193415866494922695', '145.57075488191632841598918', '-144.57075', 8, 4);
    T('1.0000000', '526650518510726171000913231536479079.037380846608', '-526650518510726171000913231536479078.037380846608', 75, 3);
    T('0.999999999999999999999999999999999999997106197957371323499604358266134753009', '3584419408840321948825.676382768', '-3584419408840321948824.6763827680000000000000000000000000000028938020426286765003956', 83, 2);
    T('0.99999999880681185203794773690375608443466417194000936368334849553283470358861975018', '5.0975685392156383162823410521', '-4.0975685404088264642443933152', 30, 3);
    T('0.999999998177929451382948935869', '10103621840183015475172718.257', '-10103621840183015475172717.257000001822070548617051064131', 70, 4);
    T('1.000000000000000000000000000000000000000144374259221118949564393862079', '99403923311484468697407136499693668.59308470138007980955', '-99403923311484468697407136499693667.5930847013800798095499999999999999999998556257407788811', 90, 5);
    T('0.999999975691341207638223751903870453583011108741353767405960466759765078360050260758989020', '4.6084648305890876715', '-3.60846485489774646386177624809612954641698889125864623259403', 60, 2);
    T('0.999999999999999999895045963233918813836521284190202155769449', '429720283413999973159.756715979', '-429720283413999973158.756715979000000000104954036766081186163478715809797', 72, 2);
    T('0.999999999999999999998773564582848665317660249642891385732648080962811751', '59990316469528.036171429611687', '-59990316469527.036171429611687000001226435417151334682339750357108614267351919037188249', 100, 2);
    T('1.000011075533608457976204143394074562732956861700803109887313366700147999338062991286462275342326012', '2792362408227880809678840930573.860305000', '-2792362408227880809678840930572.8602939244663915420237958566059254372670431382991968901126866333', 97, 6);
    T('1.000000000000000000000000000000000000015973553865744063022762207811071071600821959910304633272757', '1809301547973844603902311.5654087887338690', '-1809301547973844603902310.56540878873386899999999999999999999998402644613425593697723779218893', 93, 3);
    T('0.999999999999999999999999999999999999893329386351827298183253105991893674068332982263468835259', '2757167858879.682885', '-2757167858878.6828850000000000000000000000000000001066706136481727018167468940081063259317', 89, 4);
    T('0.99999999982184371683057085035594911410395184560767218394685221997837836267147789698615203', '100233.5013071235260730308028825861295323988130', '-100232.501307123704229313972311735773583284', 42, 1);
    T('1.00000000000000000000000000181340233818266', '6799652442.01460017', '-6799652441.014600169999999999999999998186597661817', 49, 2);
    T('1.000000000000000000000000000000000000147347387734', '63418660698725920.16721950206749192378012876192165744163', '-63418660698725919.16721950206749', 31, 1);
    T('0.9999999999999997198153866667400', '1355024592090965716.837119014286476852941', '-1355024592090965715.84', 21, 4);
    T('0.999999999999998506465', '2461571008473937929002.5', '-2461571008473937929001.5', 33, 6);
    T('0.999999999999999999999859026678320', '4313116.93223193257221953555238694917', '-4313115.93223193257221953555252792249168', 58, 0);
    T('0.9999999999999999999999999942778714703892065564058697556455', '355707575.5995458853407212870254625491043', '-355707574.5995458853407212870254625548264285296107934435941302443545', 96, 3);
    T('1.00000000000000000000000000000000000045411070649714495271991171470226781380419536261276576538560', '935234477079367359643462700583100951.671609085', '-935234477079367359643462700583100950.671609085', 58, 4);
    T('0.9999999999999999999999999998239139866375089786394034990189', '59794771768041958355410999670855.5147302227307120624560145962', '-59794771768041958355410999670854.5147302227307120624560145963760860133624910213605965009811', 99, 3);
    T('0.999999999999999999999999999999999999866773382233271762733129920026433913893459188712144625360687039', '281983374484.2923625', '-281983374483.2923625000000000000000001', 37, 0);
    T('0.9999999999999999999999999915562405177', '4.8879228750242011131569392', '-3.8879228750242011131569392084437594823', 83, 2);
    T('0.99999999999999999999999999999999999999898331649026818803643517897819544282741973223', '2205779459907407.772799722607117864973834884620146665516', '-2205779459907406.77279972260711786497', 36, 1);
    T('0.999999999999999812652797205363536057', '59305867047143667.4833618328045501', '-59305867047143666.48336183280455', 31, 6);
    T('1.000000000000000000000000000000', '131932595329118837926271.439179993115766046712328621854477', '-131932595329118837926270.439179993115766046712328621854477', 72, 1);
    T('0.999999999999999999999999999999999999994618195010129632217656992150293967', '5100955857.6666766984793247193108952', '-5100955856.666676698479324719310895200000001', 43, 3);
    T('1.000000000000000000000000000000030962131781', '299087411174134505326775400.2316271737473933705111', '-299087411174134505326775399.231627173747393370511099999999969037868219', 83, 6);
    T('0.99999965725659359656663892271019023612146610336993570765836742099905414908064343893', '258451.85378858902035138944534161417', '-258450.85378893176375779287870269145980976387853389663006429234163257900094585091935656107', 94, 2);
    T('1.000000000000000000000000000000000000179032448739043789403008517741926967437507970234495653801', '7041439829397208847523864162.0438732047', '-7041439829397208847523864161.043873204699999999999999999999999999820967551260956210596991483', 91, 3);
    T('0.9999999999999999999999999999966794315860237355747994703098798329484734037618957547641207960', '369278339.296378561025664', '-369278338', 9, 6);
    T('1.00000000', '50763.1553797642192415220375833101550549', '-50762.1553797642192415', 21, 4);
    T('1.00000000000011102102', '3608157394274649752797086018479612.3146147987011503002562683210', '-3608157394274649752797090000000000', 24, 0);
    T('1.00000000000000000000000', '83757.9', '-83760', 4, 5);
    T('0.9829', '609173030605741531450617609001.50247941012981829314544882', '-609173030605741531450620000000', 23, 5);
    T('0.99999999999999999999986', '772965177259950335115917494555177.14753490003584009', '-772965177259950335115917494555176.147535', 40, 0);
    T('1.000000000000000000000000000000000005912', '3365062705481609203587.5654314246203', '-3365062705481609203586.565431424620299999999999999999999994088', 75, 5);
    T('0.999999999999999999999999999999999999999873953298224792079305680580299426378', '90995686685396053996614001882.613682700597595804743739181115423340', '-90995686685396053996614001881.613682700597595804743739181115424', 62, 0);
    T('1.0000000004768218766863405499455555198492041378526511670528250', '327693783809041277605336.477169486864602960606149988', '-327693783809041277605335.47', 26, 2);
    T('1.0000000000001115448552593', '119611168248918293673391484287938020.241473461218556201', '-119611168248918293673391484287938019.24147346121', 47, 2);
    T('0.99999999999999999997213662540471018211929305708', '695118522544.37386448582800410989173629581370022', '-695118500000', 7, 5);
    T('0.9998547', '531239286937787488557411839.53843979311890847976494', '-531239286937787488557411838.5385850931', 37, 1);
    T('0.9999999999999999999999999958976735361', '5.021978884365540907945727890089229', '-4.0219788843655409079457278941915554639', 55, 1);
    T('0.9999829654582078810307601238411188287049063738996750661', '64838527660493105192136697.092560590360809508', '-64838527660493105192136696.092577624902601626969239876158881171', 62, 6);
    T('0.99999999999997970956151152048865510622912337678264001951962626', '18555980270360070509245369932508311.23315449380880811363692909675486264336', '-18555980270360070509245369932508310.2331544938088284040754175762662075371', 72, 1);
    T('0.999999999999999872215641172070903904858187811256571170536544114228953980', '829317789003849.3288953451686053545120630140449292', '-829317789003848.32889534516860548229642184197402529514181218874', 62, 4);
    T('0.99999999999999999999999999873507810922119355412195090450426839', '5724246388547083376998886329.000090130582393512', '-5724246388547083376998886328.00009013', 36, 2);
    T('0.999999999999999999999999999999999989', '6226063980835640631387.378254895998350608', '-6226063980835640631386.3783', 26, 5);
    T('1.0000000000000000000000000', '52384141754234.87897499736922006785549028', '-52384141754233.87897499736922006785549028', 41, 5);
    T('1.0000147190429089538881823748292591022561', '3.2265', '-2.2264852809570910461118176251707408977439', 79, 1);
    T('0.9999999999999809680564995721128049016343911661835791532770594985455125897576632', '58702586608021529152674.0708532760511602407265992', '-58702586608021529152673.0708532760511792726700996278871950983656088338164208467229405014', 87, 1);
    T('0.999999999995788445035372767161319328114769493380751862570547427295048179500671475650713', '7622776168030506645656.2153596001290878092258', '-7622776168030506645655.2153596', 29, 6);
    T('1.0000000000000000000000000000', '5757397.8000779651287294244168097', '-5757396.8000779651287294244168097', 90, 5);
    T('0.999999999999999999999999999982952303462524899846961997349063227383009113161081213840415607', '58333311391507063143.7098472927783675445320948054809448', '-58333311391507063200', 18, 0);
    T('1.00000000000000001', '745406041003423360983237653254839013021.4765158157799183157766818', '-745406041003400000000000000000000000000', 13, 2);
    T('1.000000000000', '46464603886370123993150098722.5292342596815050', '-47000000000000000000000000000', 2, 3);
    T('1.0', '2628223577379628319586.2', '-2628223577379628319585.2', 33, 1);
    T('0.999999999999999999999997098704411', '3333128399054159879225613330068577.4203473607', '-3333128399054159879225613330068576.420347360700000000000002901295589', 95, 2);
    T('1.0000000000000000000000001271579254537646919737399571922194818994525802288497710410715097031221', '5966776506713970539.80', '-5966776506713970538.79', 21, 1);
    T('1.01125126840490938144', '4554548868057820662.779627389940349610075', '-4554548868057820661.768376121535440228635', 53, 1);
    T('0.99999999999999999999999999999928408436317123855389418', '3966259541217242026.324797442870', '-3966259541217242025.324797442870000000001', 40, 0);
    T('0.9999999999999997017447921881669463444374', '261834674.21528924800798507511499649179350461792', '-261834673.2152892480079853733702043036265582734826', 88, 2);
    T('1.000000000000000016866642902279642585887406438829000860817918253948710003452606793667316', '2741330.9050040', '-2741329.9050039999999999831333570977203574141125935611709991391820817460512', 74, 2);
    T('0.99999999999999999999999999888521642213176553082417332389364843045899110317', '464542202963266251170405880056976.804449917', '-464542202963266251170405880056975.804449917', 56, 6);
    T('1.0000000000000000000000000000000000191403610436659377704', '3826846816203275742160061333640838894.2352057422515391350319129693619364364', '-3826846816203275742160061333640838893.235205742252', 49, 5);
    T('0.9999999999999999999999999999999982883811961777287', '9916527854601952670497888.1014152529874', '-9916527854601952670500000', 21, 5);
    T('1.00000000000000000000', '406069081921716207.626532', '-406069081921716206.626532', 52, 2);
    T('0.9999999999999999999999999999999999999997191651482958', '4541.4570', '-4540.4570000000000000000000000000000000000002808348517042', 80, 6);
    T('1.0000000000000000000000000000000000003382423266828210848301316627797538831502074', '250758895877039146.35263876665069821122908216', '-250758895877039145.352638766650698211229082159999999999661757673317178', 69, 2);
    T('0.999999999999999999999999999999998421889138248090469596045969100750215', '3672269886027837725194718.474455729975708930786296865789521192', '-3672269887000000000000000', 10, 0);
    T('0.9999985561', '6104869911031.888009167403917952127', '-6104869911030.888010611303917952127', 35, 4);
    T('1.0000000000000000000000000000000000', '4148556816657568.5403771308716490490', '-4148556816657567.540377130871649049', 49, 4);
    T('0.9999999999999999999999999974433831065177955832963', '1074579424720254796792989094879990841016.607862054187151121298887867', '-1074579424730000000000000000000000000000', 12, 0);
    T('0.999885910429', '8422671301151193.2933254', '-8422671301151192.293439489571', 38, 0);
    T('0.99998774856182867000827026419948966953', '966682506415.056', '-966682506414.05601225143817132999172973580051033047', 65, 0);
    T('1.0000000000000914301621563952953021396458934531642130294648348821', '73817333290363249161619281471.098503082624874', '-73820000000000000000000000000', 4, 3);
    T('0.9999', '1466985852009898.607972', '-1466985852000000', 10, 4);
    T('0.9999992295', '245681958361650215863283.85564479178973664945060844250345659774', '-245681958361650215863282.8556455622897367', 40, 3);
    T('0.9999999999999999999999999999999999989497', '790.67765966564424', '-789.6776596656442400000000000000000000010503', 47, 4);
    T('1.0000000001476453538223248840416544136188542298', '442730047276504870501600.30112133723', '-442730047276504870501599.3011213370823546461776751159583', 55, 5);
    T('1.000000000001878397089370861337349983294307967805603200', '13316960598062410509461564271.5177015134989388', '-13316960598062410509461565000', 26, 0);
    T('1.0000000000000027854187334', '840623745284573159706.886270686676527007735193', '-840623745284573159705.9', 22, 4);
    T('1.000000000000000000001', '26193409118189273079255684.3393929', '-26193409118189273079255683.3393929', 33, 5);
    T('0.999999999999999999563427461738899', '5402977392544742807104850102972000011040.02172148', '-5402977392544742807104850102972000011039.0217214800000000004366', 62, 3);
    T('0.99999999999999999999999999999999922664541864674700220492294990', '526386517069599544298.815319906505988794727369037132', '-526386517069599544297.8153199065059887947273690371320007733545813532529978', 73, 3);
    T('1.000000529236108704606686705308613180996748112350950890902073515285926196', '8169694522387.973882892536207', '-8169694522387', 14, 6);
    T('0.99999999994623', '4485822756967.7071757186471776699466091', '-4485822756966.71', 15, 4);
    T('1.00000000000000', '362957936237682713293572778560787.0227779909697121639647936930884', '-362957936237682713293580000000000', 23, 0);
    T('1.0000000000000000002757', '943.215052124585716721532585020', '-942.215052124585', 15, 2);
    T('1.00000000000001', '415689560328897232825721262983.96558950', '-415689560328897232825721262982.96558949999999', 99, 6);
    T('0.999999999830716169035451167119170287280473289119003111538059553431730404094828141735614561099393278', '33798947034201993630384867944.00889532703167417358586', '-33798947034201993630384867943.00889532720095800455041', 52, 5);
    T('0.9999999999999999897740826312882891145911517275839507', '784642258141072656663.42781', '-784642258141072656662.4278100000000000102259173687117108854088482724160493', 92, 1);
    T('1.0000000000000000000000001744186778303574246334038623580705246539732852259295845619678235252', '90828607117149.3998962199543094', '-90829000000000', 5, 3);
    T('0.99999', '42910.0', '-42909.00001', 92, 4);
    T('0.99999999942688824328075455890194153302165885723583396085101906946359332145130066417482877018', '8926078822224253080553137211428314.909364484068974654978762566318505684', '-8926078822224253080553137211428313.90936448464208641169800800741656415097834114276416603914898', 94, 1);
    T('1.000000000000000000000000000000001652054748057935508257498627900900464748257036605016148181032', '1861456057293744323400805128618667.84502882888', '-1861000000000000000000000000000000', 4, 1);
    T('0.9999', '826243539491006388632863488.34264199394918691483928557393765339961', '-826243539491006388632863487.3427419939492', 40, 4);
    T('0.9999999999999999999845819838554488656147', '836703956126.167469475558161534787376', '-836703956125.1674694755581615348027940161445511343853', 95, 2);
    T('0.99999999999999986473758609349243119235331908592398139085287957753644217444430426674768604970381', '7769187595826.733021525', '-7769187595825.733021525000000135262413906507', 43, 1);
    T('1.000000000000000000000030129591409141159116', '57459269652887618221095728.973039911430500240474058108', '-57459269652887618221095727.97303991143', 37, 5);
    T('1.000000000000000000000000000000194438', '790974522709091770730428752096.1555992589330527111751833287257469', '-790974522709091770730428752095.15559925893305271117518333', 56, 5);
    T('0.99999999999871997796576030206218210256551172443028283314', '98660.57347767', '-98659.57347767000128002203', 25, 5);
    T('0.9999999976002338520081043', '4519479702850673537.31298866', '-4519479702850673536.3129886623997661479918957', 73, 3);
    T('0.9999999999999999999999999999999999999985906216652973520931280755369418845', '9793983111083047692769851309311168661866.17', '-9793983111083047692769851309000000000000', 28, 4);
    T('0.9999999999999999999999983685', '931207682026.371988322768', '-931207682025.37198832276800000000000163', 38, 4);
    T('1.0000000000000000000000000000000000002', '853248157574768217494071789.721014783916110', '-853248157574768217494071788.722', 30, 0);
    T('1.00000000000000116429247597995', '7.4513735865813458131721959011669041701382', '-6.4513735865813446488797199212169041701382', 51, 4);
    T('0.999999999999999999999999999999999996293695499151697', '95341451033174011581135.27691156307802824', '-95341451033174011581134.276911563078028240000000000000000003706304500848303', 76, 2);
    T('0.9999999999999998887345130769773651525717640250460768764454755058851613400153', '5034.378551', '-5033.3785510000000001112654869230226348474282359749539231235545', 62, 4);
    T('1.0001411261235921516180523490041282544285410612347496482085062', '519493128201717.5049113756110133680402195783151', '-519493128201716.504770249487421216422167229310971745571458939', 60, 6);
    T('1.00000000001568468962725886867287061547069627641650312668261', '6727335056139513626.14669176879652840726', '-6727335056139513625.14669176878084371763', 39, 1);
    T('0.999999999999999999999999999999999865630', '9008106278914219164492956446381418935.6022', '-9008106278914219164492956446380000000', 30, 1);
    T('0.999960990237345825157533358392', '51705074111506014110182664950.0078003551223713068085466985950', '-51705074111506014110182664949.007839364', 38, 2);
    T('1.0000000000000000000000000000000000077', '25819.283845079900314903', '-25818.283845079900314903', 30, 6);
    T('1.02430967927735460444272477125', '8299294718879217.1089758784796002594', '-8299294718879216.08466619920224565495727522875', 60, 3);
    T('1.00020647459901045391165672647412963762168271767775200867779', '31317096481089921972525140382.5973545944011', '-31317096481090000000000000000', 13, 6);
    T('1.000000000000', '6305494369782799235522188078.2297127496649288166676635169', '-6305494369782799235522188077.2297127496649288166676635169', 93, 3);
    T('1.00000000000000000208408678737225818727927173462081278109922179094999484829778634202348290369', '3.05182881784576881758772659675644812', '-2.05182881784576881550363980938418993272072826537918721890077820905000515170221365797651709631', 96, 0);
    T('0.999999999999999957924513959747894546391137666949675272602714680124062538238865738475137024868450', '6131013616.7382089043654404285882116794541', '-6131013615.73820890436544047066369771970620545', 45, 6);
    T('1.00000000000000000000000000000001364137599525', '56170932196.0', '-56170932194.9999999999999999999999999999999864', 45, 3);
    T('0.999999999999999999999820198332498683926248475', '9561837696556878429572537926.7428170871699654128096428966414', '-9561837696556878429572537925.742817087169965412809822698308901316073751525', 77, 5);
    T('0.99999999999999999999999999999999999852197565747200835843211308778296306760405', '17711943134026161473202099.0', '-17711943134026161473202098.000000000000000000000000000000000001478024342527991641567886912217', 92, 2);
    T('0.99999999999999999999999999999999999994899204826633718805589662904966947312356251655499686374', '92604696573081710477703.959699422511476', '-92604696573081710477702.959699422511476', 60, 1);
    T('0.999999999999999999999999999999999999999884353504508754025336', '2245902558232490985541861738049049798822.3461968055421', '-2245902558232490985541861738050000000000', 30, 3);
    T('0.999999999999999999875364040491', '57103135628496637300.6272243511976919174366556', '-57103135628496637299.627224351197691917561291559509', 93, 0);
    T('0.999999999999999999999999999999999999984424139344068045245403212643098495795646000938785355727', '89413521586.1381465707734564956076666', '-89413000000', 5, 2);
    T('1.0000', '73534161253070041976125954977.8268388849', '-73534161253070041976125954976.8268388849', 64, 2);
    T('0.9999999999999999999999999793708887732517235094194432188345140147', '34938958489322.82202', '-34938958489321.8220200000000000000000000206291112267482764905805567811654859853', 100, 4);
    T('0.9999999999999986559219011137039499068155015172952119180753345610530921107949406288870961388683499909', '4896264589050170.19', '-4896264589050169.190000000000001344078098886296050093184498482704788081924665438946907889205059', 94, 6);
    T('0.9999999999988935155098918402212980214550695468929473513481476648115125969144236104878932639525', '7385673329375.72716167943', '-7385673329374.7271616794311064844901081597787019785449304531070526486518523351884875', 83, 3);
    T('0.99999999999999999999999999999999999994326913720251686097309068234577685863001332759', '7307534658233765297098608418365681434.301729537114955046871994758872', '-7307534658233765297098608418365681433.301729537114955046871994758872000000056730862797', 85, 1);
    T('0.9999999999999999999869501955540243314684164072475035911574440087239908410882105755215', '4361731.2588204059372385970851605618110468184', '-4361730.25882040593723859709821036625702248693158359275249640884255599128', 72, 4);
    T('1.00028084122979969789625475742022213752316431845012480025489649404271559', '8589228239493087487293166812521562.15039245581249074', '-8589228239493087487293166812521561.1501116145826910421037452425797778624768356815498751997451036', 95, 3);
    T('0.99999999999999999999999999997916871584322024542381517405943759357571983878057795051103294301948', '23217503757685007289304057033762286325.7644316323511125168529742357', '-23217503757685007289304057034000000000', 29, 3);
    T('1.0000000000000000000000000000', '162690137.03256574949968478712994', '-162700000', 4, 0);
    T('1.000', '2412189089661878474238918.325182753103217223', '-2400000000000000000000000', 2, 1);
    T('0.99', '10519740263335828568179399.0922527617604450455398471522002462523', '-10519740263335828568179398.1022527617604450455398471522002462523', 86, 6);
    T('0.99999999999999999999999999645197298706273914804338537494259386026976491666801734106434', '1130359.589419265', '-1130358.5894192650000000000000000035480270129', 44, 1);
    T('0.99999999999999999999999982749436257825153227', '90864618186660.04815320867153', '-90864618186659.04815320867153000000000017250563742174846', 55, 1);
    T('0.9999999999999923448165594287638519607307683173124741611', '2192.8354956558196189044122452149', '-2191.83549565581962655959568579', 30, 6);
    T('1.00000000000000000000000026116', '979.26982527763216332023020667398928', '-978.26982527763216332023020641282928', 67, 1);
    T('1.000000000000000000000000255519100872940959211166928134541590294321', '726976763690761219989505938631351.0', '-726976763690761219989505938631350', 38, 4);
    T('0.99999999999999999999999999999999999103', '7234136662724593683379985.9445701653156883080075292241746605022712', '-7234136662724593683379984.9445701653156883080075292241746605112412', 92, 3);
    T('1.0000000000000000000000000015799341380554967888095438538982251347637897805799751393751166433', '2866977303682581969827.84', '-2866977303682581969826.83999999999999999999999999842006586194450321119045614610177486523621021942', 98, 2);
    T('0.99999999999564564331991136401164770733579715091755469876743939012795893534192323399808178967819714', '25848812343620161691245692168381563919.402100098852264573747484894480281', '-25848812343620161691245692168381563918.4021000988566189304275735304686332', 72, 2);
    T('1.00000000000000000000000000000003013146939444370975721218867286890503896', '310936895627496766659224310131094846947.39155496', '-310936895700000000000000000000000000000', 10, 0);
    T('1.000000001', '6651126169383070778069767468301900533.84076900', '-6651126169383070778069767470000000000', 27, 4);
    T('1.00000000000000000000000000', '9666332651902909.174316204378705926838888897321', '-9666332651902908.174316204378705926838888897321', 97, 2);
    T('0.9999999999999999851674140132264686067678922959050175722468590920355104870384914131416443682583361', '464192749761457039.857392233932024710598344139338882278', '-464192749761457038.857392233932024725430930126112413671232107704095', 67, 5);
    T('1.116359297836286254955688165247166646796378677337200140966070743632', '12006794263623226133911260919508230916.5865189', '-12006794263623226133911260919508230915.5', 39, 6);
    T('0.999999999999999999987692074123313223470', '57130484782560760233184921634269336042.16097', '-57130484782560760233184922000000000000', 26, 4);
    T('1.0000000000000000000000000', '374191585457888206.153452501954910809273040', '-374191585457888205.15345250195491080927304', 78, 0);
    T('0.999999999999999999999999999999999999206239019531785025072957542560441955843761', '82785822813674707890490563.90486', '-82785822813674707890490562.9048', 30, 2);
    T('0.999999999999999999999999999970', '2459885176272576405768.9425', '-2459885176272576405767.94250000000000000000000000003', 55, 6);
    T('0.9998824828633175614175661374798893597360844936968897328', '52767796.7935661676706697210655506679251549428', '-52767795.79368368480735216', 25, 4);
    T('0.9999999999999998293869543', '78033608145193864042209695292861552515.292698448590946536', '-78033608145193864042209695292861552514.2926984485909467066130457', 92, 0);
    T('0.99999999999999999999980029765785765187921930978745985632085036661887883762715620612361336237', '78635603004887024583182531597237405507.067826199407762925378', '-78635603004887024583182531597237405506.06782', 43, 2);
    T('1.000000002246162595588022639932596887186800', '5184991250279246493732106.7962114119297060065', '-5184991250279200000000000', 14, 4);
    T('1.0000000000000', '122199722843287735602.27954449810228998371', '-122199722843287735601.279544498102', 33, 5);
    T('1.00000000000000000000000000000000', '435689597001062127661136030520497389786.5', '-435689597001062127661136030520497389785.5', 47, 4);
    T('0.99999999999999759723939816013059114882226072615', '7668489034776529481151118861006895265.462127231', '-7668489034776529481151118861006895264.46212723100000240276060183986940885117773927385', 89, 2);
    T('1.0000000343315352322169384869898135026940033387695475826138797158475772937550065961670415', '7360512278802637330430179790362197677045.288280949279175022462749312958', '-7360512278802637330430179790362197677044.2882809149', 50, 1);
    T('0.99999999999999999999999999887905934206681716552592', '5146586889926747.0', '-5146586889926746.00000000000000000000000000112094065793318283447408', 79, 1);
    T('0.9999999999999999959815098030696209726025952484273510250316247250632075851911166', '97514680018124821886552210.750965372923671649660923574668401862', '-97514680018124821886552209.750965372923671653679413771598780889397404751572648974968375274936792', 95, 5);
    T('0.87545226047152693790545560810557955903361675815228068743509299811348632481693566236226695068284', '5731668970228227801087563232671.626907445993325678995432356314115105', '-5731668970228227801087563232670.75145518553', 42, 3);
    T('1.00000000000001263362187491234146054520483', '65118713334390.8775841302105', '-65118713334389.87758413021048736637812508765853945479517', 73, 2);
    T('0.9999999999999999999998960173587073486205777909541238990652563224096323499', '10203841.227023918810597287690342795', '-10203840.227024', 14, 4);
    T('1.0000000000000', '55182548652656427721312.3', '-55182548652656427721311.3', 55, 1);
    T('1.000000000000000001932377208160004609028715927487203529', '8904280.36802654757440517720122302', '-8904279.36802654757440517526884581183999539097128407251', 54, 2);
    T('0.999999999999999999985216631877682229005858202234381470', '39844572202558325164917312688.69537085920163925116439132450512', '-39844500000000000000000000000', 6, 2);
    T('1.00000', '9951679087175860441300095392998186841014.0', '-9951679087000000000000000000000000000000', 10, 4);
    T('0.9999999998', '371943.1697274198', '-371942.16972742', 21, 4);
    T('0.999999999848512155782', '296653353868.908229563', '-296653353867.9082295631514878443', 31, 3);
    T('0.9999999999999999999999986346027', '54233087957373743220084.79028150', '-54233087957373743220083.7902815', 42, 1);
    T('1.00000000000000000000000000000001307140051', '889122843617186385714764.80469690386158711', '-889122843617186385714763.80469690386158711', 55, 0);
    T('1.000000000000010035547321118180166044299967593518001652', '5607623858955611895428301092666.0558035893528235', '-5607623858955611895428301092665.0558035893528134645', 50, 0);
    T('1.0000000000000000000000000165265285595012720846971', '331854342877490169202493169779330582.224783531711470227400851889', '-331854342877490169202493169779330581.2247835317114702274008518724734714404987279153029', 95, 5);
    T('1.0000000000000000000268078230511658495385273046174843751464646421142887055114550087857809109893', '7574073290799959944544790824535322414773.4867139417542120984405903', '-7574073290799959944544790824535322414772.48671394175421209841378248', 66, 4);
    T('0.999768843900197838070962771809157590385753307052046603926268522980', '552472047623863.36932777443680923086726490811523', '-552472047623862.36955893053661139', 32, 1);
    T('0.99999999999999999999999999999738', '367649823836062718928.6957379106703869412648914481011129673228', '-367649823836062718927.6957379106703869412648914481037329673228', 87, 4);
    T('0.999999998821645863265422497797570792379361351740100308775202381462172877899400018360493', '8477402424285105.870118619603235827291', '-8477402424285104.8701186207815899640255775022024292076206386482598996912', 71, 2);
    T('0.99335818097018963130707514019777356910730461647297893599368202461027373', '90.9154988271990912821595158711833523248758', '-89.92214', 8, 2);
    T('1.0000000', '0.80341905964449668522', '0.196580940355504', 15, 2);
    T('0.997965941728299', '5071587917290539270776749427.319', '-5071587917290539270776749426.32103405827', 39, 5);
    T('0.999998797627335325689068336886753604634', '9913596749.0969483253197898065717163461497535124027', '-9913596748.0969495276924544808826480092629999077687', 75, 6);
    T('0.999999999999999999999999999999958550506238100118217519560912775803432399472', '43347192642980658648517028042682653466.954536995907384512931183267778', '-43347192642980658648517028042682653465.9545369959073845129311832677780414494937618999', 84, 4);
    T('1.00000000000000000000000013224384518194999877705352901889338694572849294791739506751', '85991989880840758578746.2108', '-85991989880840758578745.2108', 43, 5);
    T('0.9999999999999999999999891967040998734160675', '91036792760966319548.62640837482078', '-91036792760966319547.6264083748207800000000108032959001265839325', 94, 0);
    T('0.9999999999999999999999999999999998448943439002189467008661474943047777814847524836143028728796', '5257855381679148205172778311.730552560322871559075862723073212149763', '-5257855381679148205172778310.73055256032287155907586272307321231', 63, 3);
    T('0.999999999999999999998659074178068759610880890091066217653809545', '5.2036947503560890870334049005894327765', '-4.2036947503560891', 17, 5);
    T('0.99999539740714817', '550158874425516.814666027837', '-550158874425515.81467063042985183', 79, 6);
    T('0.9999999999999999999999999999999999999143518750146916706521367370216415038531028', '977297648272749772609466170878895116.78181288685329616420099004943342', '-977297648272749772609466170878895115.78181288685329616420099004943342', 71, 5);
    T('0.99999989319090217411304281843022586441638548065613557391859540196005902', '794368769756.805108351842129890380450002286429486', '-794368769755.805108458651227716267407183856203621583614519343864426082', 69, 3);
    T('0.999999999999999999999999999999998832733596949868717179447284797421230', '18467243538797.169745408568265051581269127026', '-18467243538796.16974540856826505158126912702600116726640305013128282055271520257877', 84, 2);
    T('1.00000000000000000000000000000279828083311079830504488410605550245855586549024615687', '4102199570546445806701249683880.603567508366315615786661828420007762135', '-4102199570546445806701249683879.603567508366315615786661828417209481301889201694955116', 85, 3);
    T('0.9999999999999998693196377357851605604876220102759265266411693131448050918778786387978', '8641208740365431464553868.27411246026893657', '-8641208740365431464553867.2741124602689367006803622642148394395123779897240734', 77, 1);
    T('1.0000000000000000000000000000010699536270309277550390323938587015185448116220', '256417923654863043472.5930611313241', '-256417923654863043471.5930611313241', 40, 6);
    T('0.9999999999999999999998481200831161013311', '709215121030763835733636014.9419', '-709215121030763835733636013.9', 28, 6);
    T('1.000000000000000000000001032', '819.659503462642646299422869', '-819', 3, 6);
    T('1.00', '333247495.511429251', '-333247494.511429251', 23, 0);
    T('0.99979363251403741687073', '691020736578316888904655.0060882666003454006912450311874', '-691020736578316888904654.0062946340863079838', 43, 4);
    T('0.9999999999999999999999999999999999999367978', '91413272261524269.8', '-91413272261524268.80000000000000000000000001', 43, 0);
    T('0.9999999999999999999999999999999999999740411', '434850673773.209159480431637232004230949524', '-434850673772.2091594804316372320042309495240000000259589', 69, 1);
    T('1.00000000000000000000000000000000032789702126455547537089340858637253', '8625988741.3630857149811826679880', '-8625988740.363085714981182667987', 31, 2);
    T('0.9999999999999999999999999999997', '713350005209167.58194219144505095022137653812360923405', '-713350005209166', 15, 1);
    T('1.00000000000000', '28943583887478760059040471035755178.97810765994077607109139671761', '-28943583887478760059040471035755177.97810765994077607109139671761', 65, 1);
    T('1.0000000000000000000114575566695143979046885734482444615934645613', '5.25', '-4.25', 12, 6);
    T('0.999999998207', '2591059996032113734865920408723707.16146526944326945873', '-2591059996032113734865920400000000', 26, 6);
    T('0.99999999999999999999997712', '903988301876493471349822755280.553270267971930665', '-903988301876493471349822755279.55327026797193066500002288', 62, 5);
    T('1.0000000000000000000000000000000000029880603896640637097980514', '9444.3182523902548660214061', '-9443.318252390254866', 20, 5);
    T('1.0000000000000000000', '762847796163802091.15467728342883917550', '-762847796163802090.1546772834288391755', 89, 6);
    T('0.99999999999999999720242098024516555242791097962510156574473158622828537925325588376801301', '3443114943303732263089346224494114185303.26170052865302833735375410144502840549', '-3443114943303732263089346224494114185302.261700529', 49, 5);
    T('0.9999999999999999799746442350504316060077200545490', '5553964924522319251508333232696623170560.728276616710038896767', '-5553964924522319251508333232696623170559.7282766167100389167924', 62, 5);
    T('0.99999999999999999880133178729619427104517624998756093203854710', '81380040079144580.6826916989390212835696184129913272', '-81381000000000000', 5, 0);
    T('0.98951', '6651001217146776682333820745218.3789087512068', '-6651001217146776682333820745220', 30, 6);
    T('0.999999998859835333672686675562', '83005131163825085131479808086925885.324031584701866566547866435199620', '-83005131163825085131479808086925884.32403158584203123287517975963762', 68, 3);
    T('0.99999999999999999999999845016573097409291367175297907905455215527642', '92469.851', '-92468.85100000000000000000000154983', 34, 1);
    T('0.9970075427728800003114252630229565', '98044302.664844891', '-98044301.6678373482271199996885747369770435', 100, 1);
    T('0.9999999999999999999999999999999633902172192279406539610670040757742667814307301141105874098083623021', '8607358652992068918723444452462175.212249986428226587580', '-8607358652992068918723444452462174.21224998642822658758000000000003660978278077205934603893', 90, 5);
    T('0.999999999999999999999999999999999986401149157886319016855176911509467641207239255494166131', '849047873104905525395221.743154254617697414271526102189987682', '-849047873104905525395220.743154254617697414272', 45, 0);
    T('0.999999998578584913727138386711374082570994405', '9966418671958737489247396398935859379.1106869161337337206414080723095584272', '-9966418671958737489247396398935859378.11068692', 45, 3);
    T('0.999999999999999994757231449267801503862594420', '39020738973068700.9749', '-39020738973068699.97490000000000000524276855073219849613740558', 65, 3);
    T('0.99999999999999999999999999999999999999716193009302594831891670812', '26686.96159554474221405420312068523735956359', '-26685.96159554474221405420312068523735956359283807', 50, 0);
    T('0.99999999999999999999999999999999868040125604735805', '4045051037393306206241288958863888771.5924132722775211500640', '-4045051000000000000000000000000000000', 7, 6);
    T('1.000000', '385936.9', '-385935.9', 41, 1);
    T('1.0000000000000000000020217799809807701726', '6772347.98350555495067629', '-6772346.9835055549506762', 23, 1);
    T('0.99999999999999999999999', '841683206719559381064975720523811810494.8148842484572991132547155311', '-841683206719559381064975720523811810493.8148842484572991132547255311', 87, 6);
    T('0.999999999999999999999999746050927661174056519987424675225908211811067018012730087453096', '966832658903489194713509391851599080989.289399903993813593128', '-966832658903489194713509391851599080988.2893999039938135931280002539491', 70, 4);
    T('1.000000000000000000000015281991869920266006275273948718897010775324160', '6481841918646315665647558290597361100.5972882220630352815329184709', '-6481841918646315665647558290597361099.597288222063035281532903188', 64, 2);
    T('0.9999999999840129827730192554955880958436905255356993761277998779', '2083328402499.6881464818727436', '-2083328402498.6881464818887306172269807445044119041563094744643006238722001221', 99, 1);
    T('0.999999999999999999999999999797959260679340127539208621170654444315182785666109748130825727627319530', '895953.4108425', '-895952.4108425000000000000000000002020407393206598724607914', 58, 0);
    T('0.9999999999999999999999999989601882981230005460749639467039', '223127325586087907.0', '-223127325586087906.000000000000000000000000001039811701877', 60, 0);
    T('0.999999999950542399611504865748053330662404243060976669029362', '736689421107140265355972859032.8721284443056725148019278419', '-736689421107140265355972859031.87212844435513011519042297615194666933759575693902333097064', 89, 3);
    T('0.99999999962823728892529654657694818947252623125782403889479306133979560661660230452488504', '31171739955.059304131215351909513684022', '-31171739954.059304131587114620588387475', 38, 2);
    T('0.99999999999999999982353262409065864162', '2442381454981769632202781.732626743821931546100166601364547158214', '-2442381454981769632202780.7', 26, 2);
    T('1.0000000000000000000000001', '4619480.9784892', '-4619479.9784891999999999999999999', 53, 2);
    T('0.99999999999999989464959344926182385448994885252804534', '279079523561007320234.0', '-279079523561007300000', 16, 2);
    T('1.000000000000001', '22262510956227802126761619765024325226.1962403416', '-22262510956227802126761620000000000000', 26, 6);
    T('0.99999999999874733796049147', '557249683392335073964825275087748.203947654036386', '-557249683392335073964825275087747.20394765403763866203950853', 59, 0);
    T('1.0000000000000000000000000000014320237842109746033851303479', '380484151359864388081928.5', '-380484151359900000000000', 13, 3);
    T('0.9999999999999', '997981426203753588183674552.348648038757', '-997981426203753588183670000', 23, 5);
    T('1.0000000000000000000000', '185111146812063648021036142420159050.192', '-185111146812063648021036142420159100', 34, 0);
    T('3.702702702702702702702702702702703', '1589292555179193561935589.89379', '-1589292600000000000000000', 8, 5);
    T('1.0000000', '58886045591920838054955271936442472.007871493315273645805327918605047497', '-58886045591920838054955271936442471', 37, 1);
    T('0.9999999999989712469485045558795214707', '18321921.1436829949823907630320305656473372001874', '-18321920.143682994983419516083526009767', 38, 1);
    T('0.99999845516841020173178987054538507991', '87.344949881119543429551521792647578093382', '-86.344951425951133227819731922102193013472', 99, 1);
    T('1.00000000000000001471280668509036485220998740047286582178789358666130972042377896175334976674262231', '477726437655624002195473.684586558186212655796675789', '-477726437655624002000000', 18, 1);
    T('1.00000000000000000', '178254592637828017868529651369.613737085434', '-178254592637828017868529651368.613737085434', 99, 4);
    T('1.00000000000000000000055315694072936986231262513240077740004227870841219251988112146362074957191137', '65422708986415990970071657435074314573.79661277546315291191631949231', '-65422708986415990970071657435074314572.796612775463152911915766335369270630137687374867599222', 92, 2);
    T('0.99999999999999999999683538936872560849718952001244398525447485336320629710963486029276606342', '90948627550375166105240689043018518033.0', '-90948627550375166105240689043018518032.00000000000000000000316461063127439151', 76, 3);
    T('1.000000000000000000000012419723999651140622600072430169797880484035612469635', '184765.519590717604357', '-184764.519590717604356999999987580276000348859377399927569830202', 63, 1);
    T('0.999999999999999999999999999999999999997873654395323550864504485', '2515616882634422711.22', '-2515616882634422710.220000000000000000000000000000000000002126345604676449135495515', 84, 5);
    T('1.00000000000000000000000000000000084193835763941543343790252225562672612303147761261', '83082202458351596359476613310432.1542309551139837701823903340115516590472', '-83082202458351596359476613310431.15423095', 40, 1);
    T('0.9999999663437969504044365633439635145740', '3314154906446113.815033128579', '-3314154906446112.815033162235203049595563436656036485426', 76, 6);
    T('0.9998836467158086652308146243854670176782303812929699735482964893822897278194', '2752412206.293594', '-2752412205.293710353284191334769185375614532982321769618707030026', 64, 2);
    T('0.9999999999999999996675945380458424577104198374369718908616365095', '917017115876984377413432226938894496220.028484493067999143054977', '-917017115876984377413432226938894400000', 34, 2);
    T('1.000000000000000000001424433958494', '9245672710008.703902499240123355142067127595933780751', '-9245672710007.703902499240123355140642693637439780751', 71, 5);
    T('0.99999999999999999977034904244131888455371480914970920042691625744845674', '622357120301584549593662993.12761047598843659360682504', '-622357100000000000000000000', 7, 6);
    T('1.000000', '2508.75861024057892416296857348308', '-2507.75861024057892416296857348308', 52, 3);
    T('1.000000001880439173598853085601963921882648503841340', '5152846629671.5115675600', '-5152846629670.51156755811956082640114691439803607811735149615866', 90, 1);
    T('0.999999999999999999999999999881493121446012000596690604347265536390113996729410595136008358', '659823017756004.571914305852257816981881132967004880', '-659823017756003.5719143058522578169818811330855117585539879994033093956527344636098860032705894', 94, 4);
    T('1.000000000000000000018339392770355382292770424124646677038738961762408110311603556679862448859', '6756656976690626759463.27086363281586919963360292', '-6756656976690626759462.2708636328158691996152635272296', 53, 2);
    T('0.99999999999999999999999999999998770389674283443149531', '56687043396700043393732109.0159102841916811718329773061844393878172', '-56700000000000000000000000', 3, 4);
    T('1.00', '92430600702548410604590.32102296339796387552', '-92430600702548410604589.32102296339796387552', 44, 4);
    T('0.99999999999999999999844532007613563388475886', '42.5056590', '-41.50565900000000000000155467992386436611524114', 80, 0);
    T('0.99999978328940046227014606675235442398912379366570760778944067866220197716487179', '488606436390830977901437068251885953.5098968875797728711408561703', '-488606436390830977901437070000000000', 26, 4);
    T('0.99999999999999999999254507', '726182710880535123298166266.535681365988838696338238392944160639403', '-726182710880535123298166265.535681365988838696345693322944160639403', 93, 2);
    T('1.00000000000000000000000000001704384287682323784417487570475353230629036090919281144168010376', '18741103333535894322133.90505722777553335290656847106', '-18741103333535894000000', 17, 4);
    T('1.0000000000000000', '934398689860054170453708194484831913785.6924590637', '-934398689860054170453708194484800000000', 31, 4);
    T('0.9847792998477929984779299847793', '1035092530560635535991.73479549252790741902739', '-1035092530560635535990.7500161926801144205494600152207', 60, 1);
    T('0.999999998049059245698663995215516898366901466700414898365248', '29129467287175.6409673695', '-29129467287174.64096737145094075430133600478', 43, 4);
    T('2.993167155905158226309010386986020067083349', '694079.57751816636816', '-694076.584351010463001773690989613013979932916651', 72, 5);
    T('1.00000000000000000000000175953113257005708272119187671009207323883937103', '722027353.047576240920998238554102379943140', '-722027352.0475762409209982385541006205', 37, 3);
    T('0.8177004764303834092854061206047238019', '55926474228334768669593479636036.51', '-55926474228334768669593479636040', 31, 5);
    T('0.9999999999999999999999333787853', '67192398.91217960669733049100816330354427169', '-67192397.91217960669733049100822992475897169', 98, 3);
    T('0.99999999999999999999999999999999999602634627197139239707918934341773129521123922966391352287861186', '285961561257514424.86755529184719998160279128897785558', '-285961561257514423.867555291847199981602791289', 46, 3);
    T('0.9896075609007591259921263187911295268309726592', '888097251736946572536948755496223461.6265953153897994738', '-888097251736946572536948755496223460.6369877544890403478078736812', 65, 2);
    T('0.99999999999999999999999999996114645505825293000662655050048747679', '506361625484557941760.759204006918230997', '-506361625484557941759.7592040069182309970001', 43, 3);
    T('0.9999828858131632691982398210855969071087361', '785493709704062783807292303.7487', '-785493709704062783807292302.74871711418684', 41, 6);
    T('0.99999999999999999968973028685154938943721', '8335719823466.5743589467436019252318907', '-8335719823465.57435894674360192554216041314845061', 48, 1);
    T('0.999999999999999999999999999625043960414611269907', '443355723852144337703167320479727109.0704', '-443355723860000000000000000000000000', 11, 0);
    T('1.0000000000', '8205815971660941895627.82050334572310114640057696496096001', '-8205815971660941895626.8205033457231011464', 43, 2);
    T('0.9999999999999999988846912053808685916921763', '48008111650437869182964086504193977.143953405954708070143547498428', '-40000000000000000000000000000000000', 1, 2);
    T('2', '994402752905082058.93243542996490778022398896857', '-994402752905082056.93243542996490778022398896857', 83, 6);
    T('0.99999999999999999999999999897252502927383603224354108240297611925167966243164245937', '53270810817826247666494241286920.93190518162456582225989456783099677', '-53270810817826247666494241286919.93190518162456582225989456885847174072616396775645891759702389', 94, 0);
    T('0.8482549317147192716236722306525037936267071320182094081942336874051593323216995447647951441578', '41445460.10445549942', '-41445459.2562005677', 18, 6);
    T('0.999999999986304003', '2680037999322184.36465254804061143807409711', '-2680037999322183.364652548', 25, 4);
    T('0.9999999999999999997742404', '13715013766752.1', '-13715013766751.1000000000000000002257596', 48, 4);
    T('0.999999759121254163437427605304602426340240090302', '4.6112395138039420713584442793276755148286', '-3.61123975468268790792101667402307', 33, 4);
    T('1.00000002461707348941416541181871', '9522768809093.9536665289781', '-9522768809092.9536665044', 23, 3);
    T('0.99999999999999999999630', '1254646159553558178073.6460', '-1254650000000000000000', 6, 6);
    T('1.00000', '79.071693793011203223224097', '-78.071693793011203223224097', 71, 1);
    T('1.0000000000000000000000000000000000000009353908707652106374214940819409', '9230837326906151604622.56609308221707283165823491033', '-9230837326906151604621.5660930822170728316582349103299999999990646', 65, 2);
    T('0.99999999999999999999999999999999998600482106522737516628728934142', '4227275458550719773672054924005046.22562039438518', '-4227275458550719773672054924005045.22562039438518', 67, 4);
    T('0.9999999999999999999999853503621906009069811367065931234435251360154', '2695469732971304.797988493098', '-2695469732971303.79798849309800000000001464963780939909301886329340688', 69, 6);
    T('1.00000000000000001778161472076084371618871024203056924498909643669261', '900927102831126810469581816.88877782488196130', '-900927102900000000000000000', 10, 0);
    T('1.000000000', '44228977100181757679478592739.28437004557155944423', '-44228977100181757679478592738.284370045571559444', 47, 4);
    T('0.58586628276205466204981003409667278981313303469', '6286353049468595.687801928532', '-6286353049468595.10193564576994533795018996590332721018686696531', 93, 5);
    T('1.00000000000000000000001149534819151604747184923395614694496273484221809358619596993308014030', '982507494079101416830.19', '-982507494079101416829.189999999999999999999988504651808483952528150766043853055037', 81, 5);
    T('1.00000000000000000000000000000000000019386397720356844217552036752402007075447772', '7997229705019364296084384629417.0222040597332263049186489424956', '-7997229705019364296084384629416.0222040597332263049186489424956', 66, 4);
    T('1.00114761260160898442302339041437164669107199258008655682520914512', '7299654127705135090.1871366934591', '-7299654127705135089.185989', 26, 2);
    T('1.0000000000000000000000000', '4525006010872186618491027.95062861662483651670908070658778', '-4525006010872186618491026.95062861662483651670908070658778', 91, 4);
    T('0.9999999999999999999999999999999992496586113661276530056142644017316044904249244993453467285', '8777315178370192490.047111485467942270', '-8777315178370192489.05', 21, 0);
    T('0.999999999984167224747', '27905337068.2701206849914438757', '-27905337070', 10, 6);
    T('1.000000000', '91167541492495437638.6166960793429570381728184431832024990', '-91167541492500000000', 12, 3);
    T('0.999999999999', '222034.467909604291650488', '-222033.4679096', 14, 6);
    T('0.98591968552239', '23519.614439735860131587107517768', '-23518.628520050337741587107517768', 40, 4);
    T('0.9965773699710055416854431413789495149868', '8612866741570788377063803897701725847359.28809', '-8612866741570788377063803897701725847358.2915126300289944583145568586210504850132', 93, 3);
    T('1.00000000000000000019404623740852368546970184914260893086008861706018407278631860113119892837', '118040740982897.88142026', '-118040740982896.881421', 21, 0);
    T('1.00000000000000000000', '2879378367472432498458.3885698696516989502455265766154893953758', '-2879378400000000000000', 8, 5);
    T('1.0000000', '496890.2601809855607740262984580', '-496889.260180985560774026298458', 48, 5);
    T('1.00000000000000000000000000000071058325057412354', '12943299817166213912.143093163', '-12943299817166213911.14309316299999999999999999999928941674942587646', 68, 4);
    T('1.0000000000000000000124318449033880583337350971365828169602352873211', '443348080877542210264061034704225.56050665647717309541201', '-443348080877542210264061034704224.560507', 39, 6);
    T('0.999999999999994993094941498124183242495', '7734773961400.536315361751567999160566445366', '-7734773961399.536315361751573006065624947241816757505', 54, 1);
    T('0.999999999999999981060169668178820167240997574651265346', '96831383160210872409075.547920010', '-96831383160210872409074.54792001000000001894', 44, 5);
    T('0.99999999999887215301941704285030571603732854', '604969224564816745220.29', '-604969224564816745219.290000000001127846980582957149694283963', 60, 0);
    T('0.999987688010574186522619584805267563115770069803301807275227', '79468025.734276444997378425665947043', '-79468024.7342887569868042391433274581947324', 42, 2);
    T('1.00000000000000000000000000440365079263578', '19233543.31', '-19233542.30999999999999999999999999559634920736422', 90, 3);
    T('0.999998675369666088836892704700613659312172421194911859484927007959706454420277960516559753', '6100022344289439272171298525095.9079476603150687952341', '-6100022344289439272171298525094.9079489849454027064', 51, 0);
    T('0.999999999999999999980589718829880861988483622527248', '7539082380700098061.092352391154569998194', '-7539082380700000000', 13, 1);
    T('1.000000000000', '91.5603', '-90.5603', 72, 0);
    T('1.00000001101397498207836154719403611401119606461486119119857046862665648', '132401984460688723489156707.5091567', '-132401984460688723489156706.50916', 32, 0);
    T('0.99999999999999999999897042369100', '1177427936262.522663285439709114011629761028004', '-1177427936261.5226632854397091140126593', 38, 4);
    T('0.99999999999999999999975861048782686427', '543799842628875.66673190121028382239405210189567', '-543799842628874.66673190121028382239429349140784313573', 71, 0);
    T('0.99999999999999986127058996245729761698805486432277279437058035332670898', '748243785456850909144367534.5562252', '-748243785456850909144367533.5562252000000001387294100375427023830119451356772272056294196467', 91, 3);
    T('1.000000000000000000000000000000010798034938457032190163625052362974707718525931924124576662', '91823314.96', '-91823313.96', 31, 5);
    T('1.000000000000000000000000000000', '447.92379407629084448', '-446.92379407629084448', 47, 0);
    T('0.99999999999999999982567094788093102908704336203', '813817611527304102211392008307505075475.1205682207933989925211486352654553', '-813817611527304102211392008307505075474.120568220793398992695477687384524270912', 78, 2);
    T('0.999999999999999999999999999999999998851336457298180621598573836148960690118053', '48202477540138922.2', '-48202477540138921.2000000000000000000000000000000000011486635427018194', 69, 0);
    T('0.999999999998749955406643290124816933558097776611535453407330503827803', '4820609164548884692635706720003767.58890306809176354393', '-4820609164548884692635706720003766.5889030680930135885233567098752', 65, 3);
    T('0.99999999603429977151473281449151703134142622029365952630925385440', '457080632495501.94110774350229957817596', '-457080632495500.941107747467999806661227185508482968658573779706340473690746146', 78, 6);
    T('0.999999999999999999999999999999999999999676249828879926919607657191294010368511', '35337812480320972206773378204313338322.1596147342550', '-35337812480320972206773378204313338321.15961473425500000000000000000000000000032375017112', 90, 1);
    T('0.999999999999999999983122969158340254983644071985437764713993068669930795619805857865253432', '2963356877484828288727.465876117575', '-3000000000000000000000', 2, 6);
    T('1.0', '6713346411702653404361999447597215711.0406498020159', '-6713346411702653404361999447597215710.0406498020159', 100, 2);
    T('-0.408449158110263712918771982277387458778187876796313779873245687775471649857626640020105127814670985', '1185175323478721243691827143807.1701137967896571904443', '-1185175323478721243691827143807.5785629548999209033630719822773874587781878767963', 80, 5);
    T('0.99986333139972777834979524076739554483510953897851108115254566096546332295282446', '345396627698075805094410889829.71084231439337364633859735580899944253', '-345396627698075805094410889828.7109789829936458679888021150416038976948904610214889188475', 88, 3);
    T('1.000001870693094653729986455362532696732342460198069256792504583586352160825651046245137', '6895329.951116', '-6895328.95111412930690535', 24, 0);
    T('1.00000000000000000000001', '2931.5882187541386733669', '-2930.58821875413867336689999', 67, 3);
    T('0.9999999999999999757128854250954388755212642712520453363904084191108', '113.882093962053', '-112.8820939620530000242871145749045611244787357287479547', 55, 5);
    T('1.000000000000000142100725333220143773935460490715729292', '6.1133775808949', '-5.11337758089489985789927466677986', 33, 0);
    T('0.999999999999999999999999250069410', '63323938.336021157683613783786811340', '-63323937.33602115768361378378681208993059', 83, 4);
    T('0.99999999999999999999999997852468713629926030600156416028550057017625067711237690034', '8649307819192363328362.644759128539089609793', '-8649307819192363328361.64475912853908960979300002147531286370073969399843584', 75, 0);
    T('0.994944930392460489131485070752031007017613461873270253043636784884498572631', '302954902035100728.69419802896112560', '-302954902035100727.6992530985686651108685149292479689929824', 58, 6);
    T('0.9999999999999999999999998176234055274222473843010278290097', '5195.791519777644231277806542261003712', '-5194.79151977764423127780654244338030647257775261569897217099', 60, 1);
    T('0.999999999999992234557937865398598876493534856140536890455746', '84980412601.34964710646781361', '-84980412600.349647106467821375442062134601401123506465143859464', 62, 0);
    T('0.99995501995477359826046905861953565059940250037626012434705759', '8702892440949258178687649444501.3435803857008', '-8702892440949258178687649444500.34362536574602640173953094138046434940059749962374', 81, 0);
    T('0.999999999999999999999999999999999988852429008830639383802463012779030141585916798', '483101768586793035841.76', '-483101768586793035840.76', 35, 1);
    T('1.0000000000000000000000000066322477', '7177879868879230987932.752575326277715', '-7177879868879230987931.7525753262777149999999999933677523', 77, 2);
    T('0.99999999999999999999863050900848656719680911744110382417520539384151037330010', '6643729835811095901.867514043606675', '-6643720000000000000', 6, 1);
    T('0.999999', '151.1052', '-150.105201', 85, 2);
    T('1.000000000000000187783725926998231240474894375407056821287537483901818724753698620317', '785076301170341332100987547.969823359591419132394', '-785076301170341332100987546.969823359591418944610274073001768759525105624592943178712', 84, 2);
    T('1.00000000000000011565947380810625219768876722861889871034846436208086032638525113018', '439743632833.85513861832157543', '-439743632832.85513861832157531434052619189374780231123277138110128965153563791913967361474886982', 95, 2);
    T('0.88820507051233074573981137471912743866783451488053540883197945741406875632726757404199789904925', '9175616310.0852302175', '-9175616309.1970251469876692542601886252808725613321654851194645911680205425859313', 80, 0);
    T('0.99999999999999999999999999999756270258349218952278966926561547445077676042844324', '43723815215406705352023721058051569.48292327390', '-43723820000000000000000000000000000', 7, 4);
    T('1.000000', '49313783003443364349.475396547386721798459710', '-49313783003443364348.47539654738672179845971', 44, 5);
    T('0.99999999999999999999999999999999999987381290', '95138709341585757.53385749091634933802', '-95138709341585756.533857490916349', 32, 1);
    T('0.99999999999999999999999999999999', '6502038664382637853.3773175610495464620', '-6502038664382637852.377317561049546462', 42, 6);
    T('1.00000000000000000000000000000004421965828', '4255570887531700706035490847612211368698.0', '-4255570887531700706035490847612200000000', 32, 4);
    T('1.0000000000000000000000000000000', '53365.354816987', '-53364.354816987', 44, 3);
    T('1.0000000000000000000000000000000000000021534', '7998379781872770187335176.1739103', '-7998379781872770187335175.1739102999999999999999999999999999999978466', 78, 0);
    T('0.999999999999950893141779504405862059148463210290987554556208379720761429731230', '43792026.279728044057628930167782855482238', '-43792025.27972804405767803702600335107637594085153678970901244544379162027923857026877', 99, 1);
    T('1.00000001570856770223754966346728694508658721092823333833606568335899376611952358504927440676029026', '5836390.6297667530472007489938', '-5836389.629766737338633046756250336532713054913412789071766661663934317', 70, 5);
    T('1.000000000000000000000000000000013551988375997128269833629793257971741', '0.92469972967603919810', '0.075300270323960801900000000000013551988375997128269833629793257971741', 78, 1);
    T('0.999999999999999999999988972366482742599958616938696005638540472404602858308312', '666399026.86278085897', '-666399025.86278', 14, 1);
    T('0.99999999999999', '786779725903652214156905082885.171938889192204123923195310', '-786779725903652214156905082900', 28, 6);
    T('1.000000000000000000011500613', '1434083.253895591871173798', '-1434082.253895591871173797988499387', 62, 2);
    T('0.99999999999999999999862398810523900237700116991353588079775945', '76461396332.811283722', '-76461396331.81128372200000000000137601189476099762299883008646411920224055', 88, 0);
    T('0.9999999999999999999999999999999999997171990205502151057931113033334185325557437050600907', '49974125848998616437873664977784908484.9124', '-49974125848998616437873664977784908483.9124000000000000000000000000000000002828', 80, 1);
    T('0.99999999999999977791765901759291507086598158643668565313770323975222919496932199', '8.187', '-7.187', 7, 5);
    T('1.000000', '63947801506546848974368.322151263734344148943', '-63947801506546800000000', 15, 2);
    T('1.00000000000000', '941149476556.4659270864', '-941149476555.4659270864', 38, 2);
    T('0.99999999985557977560162543954157359653', '8417892309066.785923', '-8417892309065.785923', 20, 5);
    T('0.98666169642349715680', '8328488689155311624026529657.1245763477423052945', '-8328488689155311624026529656.1379146513188081377', 87, 3);
    T('0.999999999999996478186376701540556390451966787570126271919454925881415317269375223037795', '910.94822614699062303484352', '-909.94822614699062655665714329845944360954803321242987372808054507411858469', 74, 0);
    T('0.99999999692433521484815326963328451823767134916768589099637384480582882095', '5567065934446918.1463366996108285152224146458379', '-5567065934446917.1463367026864933003742613762046154817623286508323141090036261551942', 83, 5);
    T('0.99999999999999999999999999999995428072840045305107866683225500617008148076748781163', '935245315037410797358895.81561029710600001060007985722670', '-935245315037410797358894.815610297106', 36, 2);
    T('0.999999999999999999998756864049496111', '769738444542839279.447328950917588049', '-769738444542839278.447328950917588049001243135950503889', 66, 5);
    T('0.999999999999999999999999998999573673370329191401592444902781933839', '46790273305080696902705757303351815.6683551462177520007170', '-46790273305080696902705757000000000', 26, 5);
    T('1.0000000000000000000000000', '4401646672496.9896438936', '-4401646672495.9896438936', 51, 5);
    T('0.999999999999999999999999999999586780802134961843198', '4365417634470052289.8042100', '-4365417634470052288.804210000000000000000000000000413219197865038156802', 89, 5);
    T('1.0000000000000000088001129146150462966612282427565851869646314258128680028227166842524335', '6204.7624499344', '-6204', 4, 3);
    T('0.9999', '8254968976434579465184576489825.075407156699135398698462806552732', '-8254968976434579465184576489824.075507156699135398698462806552732', 83, 6);
    T('0.99999999999999999851647004636582881828217317224184547790928824998145537220605631623', '81463757984.3967', '-81463757983.39670000000000000148352995363417118171782682775815452209071175', 74, 4);
    T('1.0000765104448611358875374061647638800032867378123433243121247212611601280', '6713888521505912296732292158.436739159436186115654093490685171', '-6713888521505912296733000000', 22, 3);
    T('1.000000000000000000000', '8898625261879536026632279900439761138386.87290802240296', '-8898625261879536026632279900439761138385.87290802240296', 97, 1);
    T('0.9999999999999999999999999967183430576394122278077387279258147865491679382197598450482597144294676', '8350924568096.5772720562444', '-8350924568095.577', 16, 4);
    T('1.000000000000000', '83469118679717920.325394021677728014164369058663459725', '-83469118679717919.325394021677728014164369058663459725', 62, 6);
    T('1.0000000000000000000000101473383451491347155757499217582427846', '83441.03510', '-83440.0350999999999999999999898526616548508652844242500782417572154', 84, 6);
    T('0.999998972167595972360199023871761930903861601395772947729446394388469830722638142502', '114254511833.216287907486708206587000', '-114254511832.216288935319112234226800976128238069096138398604227052271', 69, 6);
    T('0.999999999999804031751624364194481197934879536882498381070259376528912', '240024662536219206315744503776.427794718613029208', '-240024662536219206315740000000', 23, 1);
    T('0.96819299634261700488604', '9066850455281208610640021393.8674529621061531329992081476182', '-9066850455281208610640021392.8992599657635361281131681476182', 90, 2);
    T('1.00000000000000000000000000000000002172161284574233555796808926105492741901868922578407868', '88184614510067355.80561516771756214406427966719', '-88184614510067354.80561516771756214406427966718999997827838', 58, 2);
    T('0.9999999999999958347982045339690727638081283170211824676948', '45875508857136506105.9457428434555262315754944224', '-45875508857136506104.94574284345553039677728988', 46, 2);
    T('0.9999999999999999999999988445458847365544666935', '56211094307804782041.509', '-56211094307804782040.509000000000000000000001155454', 50, 2);
    T('0.99999999999999997278215489654731927885346650293494', '894703306762256028311875497262.538257389154218399122383689005984', '-894703306762256028311875497261.538257389154218426340228792', 57, 6);
    T('0.999999999999953011251229014166682401824615708406966158082', '46337140.1144', '-46337139.114400000000046988748770985833317598175384291593033841918', 100, 1);
    T('0.9999999999999999999999999999999999896056143992086510464518221750813062587318492000665042591710357491', '99391255405800.0918491624200906501407879', '-99391255405799.091849162420090650140787900000000010394385600791348953548177824918693741', 86, 1);
    T('0.99999984819248484991309382711138490839073984925287441787073196369470066241386606264998', '4587095184907866714.64962537817941703545843724957', '-4587095184907866713.649625529986932185545343422459', 49, 3);
    T('1.000000000000000000000114047326084686813408612054', '55526394985076003625537.2855404328721244405875952321', '-55526394985076003625536.2855404', 30, 4);
    T('1.00000000000000000000066757865', '4698751713110527100044493661851832.1280515921976921536930307126842047775603', '-4698751713110527100044493661851831.128051592197692153', 52, 2);
    T('0.9999999999999999999999999999898520639023234373113970', '597887430821816.37868762728048', '-597887430821815.3786876272804800000000000000101479', 49, 2);
    T('0.9999999999999999999999943659002311912637536936265', '372363126984981504763281221.016008120717944883030216852070969814', '-372363126984981504763281220.0160081207179448830302224861707386227362463063735', 79, 4);
    T('2.560358930438398373253129166204747663743589115470183976162133886573122965263759', '14276379.754079790292243643227792224707512761133', '-14276377.1937208598538453', 24, 3);
    T('0.997279924099002058553441', '292444431675922883574601415127277.65412483437377258266206834', '-292444431675922883574601415127276.65684491027477052410862734', 81, 5);
    T('0.999999999999984260323122546400509968794159996915642849427341582908940713264898497', '896892106971913.00249004517396547992799690512363', '-896892106971912.002490045173981219604874358723120031205840003084357150572658417091059286736', 90, 0);
    T('1.00000000000000000000000000000000005992793499496659393339544351894322459867667720100224324', '3318982239965256210087752679870583633.0', '-3318982239965256210087752679870583632', 58, 3);
    T('0.9999999999999999999999999770600438420427049147179454692729', '3628720682254007674215981409306.31031453402970815414162933217', '-3628720682254007674215981409305.31031453402970815414163', 55, 0);
    T('0.9999999999999999999999999996657399554762389241879623907', '1491933311244294962.2716549', '-1491933311244294961.271654900000000000000000000334260044523761075813', 67, 3);
    T('0.9999999982907185009961319194614700241048636901018065442376513896078', '8905465022159428124649388217335894739871.48764344311998', '-8905465022159428124649388217335894739870.4876434448292614990038680805385299758951363099', 86, 5);
    T('1.0000000000000000000000000000010541205930250083169945239531834166203868323693982578921', '82957456835248475330437467.711575474660996111464515550931003660', '-82957456835248475330437466.711575474660996111464515550929949539406974991683005476046', 83, 1);
    T('0.99999845314951795088008978452157914898351588874160649071066581157839748076473923844', '2390301.6522300710322967688719072305939', '-2390300.652231', 13, 1);
    T('0.9999999999999', '7410520978919018.620589584077521815643048008795826446', '-7410520978919017.62058', 21, 1);
    T('0.999999999998866080992', '9754979740021368214066.307', '-9754979740021368214066', 22, 3);
    T('0.9999999999999987985567', '61313754256884800898494009.06650419', '-61313754256884800898494008.0665041900000012014433', 66, 6);
    T('1.00000000000000000000000000000000001828033388964128406703943036348', '712047527675.3127883657482944397881448440063341881', '-712047527674.31278836574829443978814484400633416981966611035871593296056963652', 81, 2);
    T('0.982318835784415567271856091844159086526730055982235607255630717809443899150761701', '170941145154208987908.549070736688958', '-170941145154208987907.5667519009045424327281439082', 49, 5);
    T('0.9999999999999999987696401057589439340708226866296', '652280046365792709548027318804722090563.37300', '-652280046365792709548027000000000000000', 24, 5);
    T('0.979491578768789026964731', '180300492694817776805.281302217', '-180300492694817776804.301810638231210973035269', 94, 1);
    T('0.9999999999999999999999998167386236163069370389059428230777358991907965470634937011634523331012', '888478399577714215915090483649341901659.5691552537367346416242164942', '-888478399577700000000000000000000000000', 13, 4);
    T('1.000000000000', '7611939122971781377571714.94211704213151086081977197474439', '-7611939122971781377571713.942117042132', 37, 3);
    T('0.9999999999999999999999999999999989340', '4154962352714.50652458600', '-4154962352713.506524586000000000000000000000002', 46, 3);
    T('0.9999999999999999999999999852042043715375191753', '830396780505539823368538401.1', '-830396780505539823368538400.1000000000000000000000001', 52, 3);
    T('0.9999999999999999999999999999710768031153693331025300', '2123231702008454660589023938.98', '-2100000000000000000000000000', 2, 2);
    T('1.0', '53948319822781571615601777869131321791.707834735727647404742835478', '-53948319822781571615601777869131320000', 34, 1);
    T('1.000000000000016924826829710808766', '794752676978903486715565479.892048372416941374012948705038771038185', '-794752676978903486715565478.892048372', 36, 5);
    T('1.00000000000000000000000018255994645', '147.9468548321791', '-146.94685483217909999999999981744005355', 92, 5);
    T('0.99999860341827343394308091515207378420577793791914909068563783284033998385551524089638201538', '5455713509.106936838213888274069703085952', '-5455713600', 8, 3);
    T('0.99999999', '60943424947625.10645509951005756', '-60943424947624.10645510951005756', 68, 1);
    T('0.99996674233763733613552339134849373114699021536030911127626975400066', '555365074382953371691645860071860.461', '-555365000000000000000000000000000', 6, 4);
    T('1.00000', '9437021163908553824168349.250732215142218961931', '-9437021163908553824168348.250732215142218961931', 62, 6);
    T('1.0000015798040303635958376810456660344920742352902218838303783', '3195564041391750195273729271.8426664379922', '-3195564041391750195273729270.8426648581881696364041623189543339655', 65, 1);
    T('0.99995999682767255727671193308353698399183159014632010465950622244', '80.32169864786651492', '-79.32173865103884236272328807', 28, 4);
    T('1.000000000000000000128294595', '100554068831181945.4802829392287267273', '-100554068831181944.48028293922872', 32, 1);
    T('0.99999999999999999999999999250388', '76776357988492325.7470262184560571502061249', '-76776357988492324.74702621845605715020612490749612', 74, 5);
    T('0.99999999999999999999999999999999999999869647825946421313130131059342656879', '94892455922.46853660428649014', '-94892455921.4685366042864901400000000000000000000013035217405357868686986', 72, 2);
    T('0.999999999999999017402016384226700740777645283168548804633187229892467423', '854509462636249455.196659', '-854509462636249454.196659000000000982597983615773299259222354716831451195367', 75, 0);
    T('0.999772424792206347937818644789958103130268920746466912314514364496750147550', '982763.5825922126349', '-982762.58281978784269365206218135521004189686973107925353308768548563550324985245', 100, 3);
    T('0.9999999999965709881316749500091512618463173685757423462949491878188559903276718111283023044404610030', '40318911053230521.601558016', '-40318911053230520.60155801600342901186832504999084873815368263142425765', 70, 4);
    T('0.9999999999999999999999999987501202308066413018599824531878412896566881', '52062239790271008826999062691896.29748520507560492526212759522788080394', '-52062239790271008826999062691895.2974852050756049252621275964777605731334', 72, 0);
    T('0.999999999999999999999999999998600637682934991787071469834665121277825288', '8190348303084132484797818835893963104.01584', '-8190348303084132484797818835893963000', 34, 1);
    T('1.000000000000000000000000006303139', '19330560435382276350302.894340629043581603439479528785', '-19330560435382276350301.8943406', 30, 6);
    T('0.999999999999999999911079371444', '8926750342689165130170306.22408818229187241256457225077284940', '-8926750342689165130170305.224088182291872', 40, 2);
    T('1.002443830707835798989608233953459574429', '918608007889297288090207443744.178191406', '-918608007889297288090207443743.175747575', 39, 5);
    T('1.00030010732562628400064433800711382866', '9515504533610954329473140.573078', '-9515504533610954329473139.57277789267437371599935566199288617134', 70, 1);
    T('1.000000000000000000000000000000322761653001562947451027045758693736981', '9269988.77566704669', '-9300000', 2, 6);
    T('1.0', '968093760535.907576', '-968093760534.907576', 96, 4);
    T('0.999999999999999999986523283835752542241305454036367413718456034056606296924540412592972104282736', '3967665509304540045763096.3585281045367185090842294207245289526', '-3967665509304540045763095.3585281045367185090977061368887764103586945459636325862815', 83, 5);
    T('0.99999999999999980555301285088011319181888558502184033200791571826610735203665749890', '8118681277483025956593950476955302800.090099327416775', '-8118681277483025956593950476955302799.0900993274167751944469871491198868081', 74, 1);
    T('0.99999999999999999999999999999989644648068087973543926511534672924318338715', '331142069468490085822765778378686856.5380748194655664725546831712336732571', '-331142069468490080000000000000000000', 17, 2);
    T('0.99985699241130618', '53.0', '-52.00014300758869382', 74, 4);
    T('0.98764926629197147404135522750515545279848109784868691548705000592737114058', '266480605553559688.63660', '-266480605553559687.6489507337080285259586447724948445', 52, 2);
    T('0.9999999999999999999999999999999994125356465600840624', '22316724829186694.70946019369', '-22316724829186693.7094601936900000000000000000000005874643534399159376', 80, 4);
    T('0.99999999999999999999999998502177100066119236185838930012379325650789592881034983', '7277195065.3960868490207717', '-7277195070', 9, 3);
    T('0.999999999', '937504066205078177688375292.226144475357149370401084733058491', '-937504066205078177688375291.226144476357149370401084733058491', 72, 6);
    T('0.999999999984567202385156573676911055816584143846117118337973326196356863', '1.337077750605524275965518', '-0.33707775', 9, 2);
    T('1.00000000', '5064544828063314847094366578077533356165.3033505066', '-5064544828063314847094366578077533356164.3033505066', 69, 1);
    T('0.999999999999999989289665156594527371862907436234430149822995950865458', '13388603038561301055.6751', '-13388603038561301054.675100000000000010710334843405472628137092563765569850177004', 80, 5);
    T('0.97669704905410991815537568784485454531049084869813403950595879758637355282848888', '512073997931528222235205182589552758.0', '-512073997931528222235205182589552757.02330295094589008184462431', 62, 1);
    T('0.99999999999999999999965643573153862626206372312354754566709302', '4900683600570976453703602247732648431.8286601405461978166067292', '-4900683600570976453703602247732648430.828660140546197816607', 58, 5);
    T('0.9999999998087579587687973666145764948157998830116744769174', '821871777514396133499014.900389462741576296348905082020420', '-821871777514396133499013.9', 25, 2);
    T('0.9999999999999999798379905', '215673782738267239815417089454830394.7186341', '-220000000000000000000000000000000000', 2, 4);
    T('1.0', '9784.012257658037159114866', '-9783.01225765', 12, 1);
    T('1.00000000000', '4962651540727002.0', '-4962651540727001', 88, 4);
    T('1.000000000000000000000124742901762255467246359305481289149335101524958079474654327857465', '7855.45677985592663', '-7854.45677985592662999999987525709823774453275364069452', 54, 6);
    T('0.999999989984651523566473193125218408990555172956663500', '3958857340310713.8438437144974885440148761778207311556', '-3958857340310712.8438437245128370204484029846955127466094448270433365', 91, 4);
    T('1.000000000000000000000001093634163454292564440084533994042802891996297302996164682311725618', '74825911658090344.2813759185525296659260281863792828', '-74825911658090343.3', 18, 0);
    T('1.00000000000000000', '523745363135186212.17', '-523745363135186211.17', 69, 5);
    T('0.999999999999561384074584039903849397159905814910356142548725694963570', '102720359840081823.3649784042431385007352441580783158572', '-102720359840081822.36497840424357711666066011817446646004009418508964385745127430503643', 99, 1);
    T('1.00000000002376638030340104936646520826867730324589378523641914512071139602856821853410598601001921', '67726005853812604277006.9302123416039338', '-67726005853812604277005.93021234158016741969659895063353479173132269675410621476358085487928860397', 97, 6);
    T('0.9987771036104615558864720850070552267296088285223290956773497532549093029471118155992343839842126', '78987.0475937574835715831', '-78986.0488166538731100272135279149929447732703911714776709043226502467450906970528881844', 87, 1);
    T('1.00000010180398994679553772847507503336837954079030179570914644952543914944959531795159', '5430561729599019851446716048542937.773584957370975904344734318393344', '-5430561729599019851446716048542936.7735848555669859575491965899182689666316204592096982042908536', 95, 5);
    T('1.8627235560519940531594039254786420594086758173813206633313662957033001221740484802598736318715', '56524605266077026330371484798.91221972821798502827', '-56524605266077026330371484797.04949617216599097511059607452135794059132418262', 76, 4);
    T('0.9999999999999999999999999999998809475492060830843219136818040376276702137460', '6728820626716913903.6', '-6728820626716913902.60000000000000000000000000000011905245079391691567809', 72, 3);
    T('0.999999999999999999792620941053406475600134926057253894844521606321567134', '99.7650821411993', '-98.765082141199300000207379058946593524399865073942746105155478393678432866', 94, 3);
    T('0.9999999999999999999999999999999999971484360423894231411477492269960038515927714894791397587827', '916604932359056605315496144.0', '-916604932359056605315496143', 42, 6);
    T('0.999999981412099151769605290613336362140470', '7427175537493515137329055.1159549', '-7427175537493515137329054.11595491858790084823039470938666363785953', 87, 5);
    T('0.999999999999999999995449539908791161302108306318401437876181443687111753336208730693054', '4.84795192292514700629749074836154288', '-3.8479519', 8, 2);
    T('1.0000001', '6488388054879836420.059441885454251234122418289996030655', '-6488388054879836419.06', 22, 3);
    T('1.000000000000000000000', '8930553499035657.893467358123016398910', '-8930553499035656.89346735812301639', 33, 1);
    T('1.00000000000000318481389151586201', '6090969.320481817095924948382427662616', '-6090968.32048181709592176356853614675399', 58, 4);
    T('0.9986566072867656335841837885514626934896023798489735885261', '32718634016642744888.97134529256047901765580277506732820', '-32718634016642744887.972688685273713384071618986515865506510397620152', 68, 3);
    T('1.0000000000000000000000001555716721393333781735034830922584608814457', '767649422241274903254885798587.9', '-767649422241274903254885798586.8', 31, 2);
    T('1.000000000000000000000000186056', '687297780532553275.488908459', '-687297780532553274.488908459', 39, 6);
    T('0.999999999999999997709042382345098121945', '5.140737898219845', '-4.1407378982198', 14, 2);
    T('1.0000000000000', '20313313091.912568728511493864413', '-20313313090.912568728511493864413', 32, 2);
    T('1.0000000000000000000000000000001', '91814.593588', '-91813.5935879999999999999999999999999', 48, 5);
    T('1.00000000000000001460974021305961258400065681294', '17686.75162394296958428', '-17685.75162394296958426539025978694038741599934318706', 76, 4);
    T('1.000000000000000000000000000000535911411323148488373592894048045395893093472', '480674310432978817916372313.6204648747', '-480674310432978817916372312.620464874699999999999999999999464088588676851511626', 78, 2);
    T('0.999996889783349618795098703759641745397129892618258664272665077457803993444214', '900675321363475008277609188.63151022135205052735', '-900675321363475008277609187.631513331568701', 43, 0);
    T('1.000000000000000000000000000004181856135263', '94812620199583543530320.0', '-94812620199583543530319', 42, 4);
    T('1.00000000015296185628351374111355040102079', '66786247006198270269679017.8453838159847537994502656203549', '-66786247006198270269679016.84538381583179194316675188', 53, 0);
    T('1.0000000000000000000000000108594116618450221081156319', '87940424753.013368930935198329334511647444', '-87940424752.0133689309351983293345116365845883381549778918843681', 70, 0);
    T('0.9999999999999999999999999999998801363614847399894493992822338382292863', '483393765583013.64086790635946309', '-483393765583012.6408679063594630900000000000001198636385152600105506007177661617707', 82, 6);
    T('1.000000000000000000000000000197575106156880291029367934288526701171331582537930062', '219262.565661518779325960425', '-219261.565662', 12, 4);
    T('1.00000000000', '3166911694606689449872793.857667155425333789167330', '-3166911694606689449872792.85766715543', 36, 3);
    T('0.988921387278028188069786621199521370', '6407758068040.04709144560445418853870465636141', '-6407758069000', 10, 3);
    T('1.000000000', '27.74017745836106954388833250354944329235', '-26.74017745836106954388833250354944329235', 92, 4);
    T('0.99999995073498444209755984579172940737936972046591438502390161815440926766206850282109992029', '14454426653626963698457.58037394', '-14454426653626963698456.58037398926501555790244015', 49, 6);
    T('1.000000000000000000000000002051685981764346168512', '543.473659944245641452069722728', '-542.473659944245641452069722725948314018235653831488', 99, 5);
    T('0.999999999999999805616768748689063390723961859032417667516119772810367006482242085793696150353702604', '2754259977772877527566078498.56037720533843616397683650', '-2754300000000000000000000000', 5, 4);
    T('1.0000', '2708595954684743.07329558357', '-2708595954684742.07329558357', 73, 5);
    T('0.9999999999999999999999999999999881769298329948751124567503894060246592475', '5076257878740599686556580562569751.022', '-5076257878740599686556580562569750.022000000000000000000000000000011823070168', 76, 3);
    T('0.9833333333333333333333333333333333333333333333333333333333333333333333333333', '35759670270716022271714370991.461146062951048', '-35759670270716022271714370990.47781272961771466666666666666666666666666666666666666666667', 88, 0);
    T('0.9999999999999999999999999999999999999964574253340454279449892943045925983538897413872038', '9243296429699058122149427426196354038774.885883050705152098723561163481304064687', '-9243296429699058122149427426196400000000', 32, 3);
    T('1.0000000000000000000000000000000', '32415117924076275399479823862000176366.4049217919162698', '-32415117924076275399479823862000176365.4049217919162698', 62, 5);
    T('1.0000000000000000000001038686307287398316377776636468498808658', '98.490506351009906419886999147063', '-97.4905063510099064198868952784322712601683622223363531501191342', 86, 4);
    T('1.0000000000000136066266215603718491900380688601599114963480181749634817714673460004448', '9448738667470579759125311918568809464.3063613358456987', '-9448738667470579759125311918568809463.306361335845685093', 55, 2);
    T('0.9999999999999999999999999999999998467196394230434514489', '5123022321695842514947975358286784.54117558559570', '-5123022321695842514947975358286780', 33, 5);
    T('0.999999999999995603472723020821424', '58675393873.4594578052150', '-58675393872.459457805215004396527276979178576', 86, 1);
    T('0.99999999999999999999999999987962466649777961405890166511458247512536467310173637954218', '508807233744429796600893060243.0969469546900184611', '-510000000000000000000000000000', 2, 3);
    T('0.99', '120571690278584380368378872165095154438.6284078149601212936793326584791', '-120571690278584380368378872165095154000', 36, 2);
    T('0.999999502793234024648162272282334145', '7235.064522', '-7234.064522497206765975351838', 28, 6);
    T('0.9999999999999999975371290795', '7136028508656397.29629103398', '-7136028508656396.29629103398000000246288', 39, 0);
    T('0.999999999999999999999999999999999867569', '73660230.61382030039727962', '-73660229.614', 11, 0);
    T('1.0000000001', '92326766290376243337.7', '-92326766290376243336.6999999999', 69, 2);
    T('0.999999999999999999999892437783280012049434497924079908264375142178911', '1.092056965609520029843', '-0.09205696560952002985', 19, 0);
    T('1.000000000000000023', '900239320488174.8211857467561686161759614052002447445992', '-900239320488173.821185746756168593175961405200244', 48, 2);
    T('1.00000000000000000130029886163686327962196642789', '1209971245145861800955411383758.5786422', '-1209971245145861800955411380000', 27, 6);
    T('1.00000000000000000000000000', '533329708660893791982232969966899.4141536866774919353100308516511089971744', '-533329708660893791982232969966898.41415369', 41, 5);
    T('0.99999999986942799491015121840113489262631', '12217169.6927240667898482112546', '-12217168.69272406692042021634444878159886510737369', 76, 2);
    T('1.000000000114274749074232123965073100366606082046727974511221316820448358860', '549114746426015254474052879.6131757669647805072103515167975', '-549114746000000000000000000', 9, 4);
    T('0.671543055', '86603524686542894607605070142832817.30683078853998766236958348557917366', '-86603524686542894607605070142832816.6', 36, 6);
    T('0.999999999999999999999807397806936980', '3971944261963954.7397321596576847329124507401924713399043', '-3971944261963953.739732159657684732912644', 40, 3);
    T('0.9999999999999999999999896462813867993931', '96874526335392319227.772839046341297107152721070', '-96874526335392319226.7728390463412971071528', 42, 0);
    T('0.999999999999999999999999836417004026509664', '232894684125835563293296112336766523.58101769563282236862413742352019484', '-232894684125835563293296112336766522.58101769563282236863', 56, 0);
    T('0.99999999999999999999999999999999999999841243338324826819', '790684047406222159162262.0', '-790000000000000000000000', 2, 5);
    T('1.0', '1339904898013391294228202627749955304.68852120726686275', '-1339904898013391294228202627749956000', 34, 3);
    T('1.000000000000000000000000000000013', '89671721009250088688506468.64987631424895128552752608783713', '-89671721009250088688506467.649876314248951285527526087837117', 59, 1);
    T('1.0001066932921455857105564575485900771968795360779853980187', '31.70758287828855229373178139', '-30.7074761849964067080212249324514099228031204639220146019813', 79, 2);
    T('0.9999999999999999999999999984233859690147849372180504705402732504176978814006803', '285130839181807870.67050439671322009525528247778614161', '-285130839181807869.6705043967132200952552824793627556409852150627819495', 70, 2);
    T('1.000000000024730876553954996348953033399600575601567273685012234997522', '777938498281784886241666307312.260547761700651158918714', '-780000000000000000000000000000', 2, 0);
    T('1.0', '13346803278607028.09043350999359889081', '-13346803278607027.0904335099935', 30, 2);
    T('0.999999421145381305411245098424', '870256253121951735722208461789410454.2802606609794486', '-870256253121951735722208461789410450', 35, 4);
    T('0.99999999999999999999999999999879563', '958460644067411254115010127120540474.71869689011682719962342933722', '-958460644067411254115010127120540473.7187', 41, 6);
    T('0.99999999999822434089381924153542416194482', '796981944508079419756163811771278626.202259100839083616772362734253133', '-796981944508079419756163811771278600', 34, 2);
    T('0.9999999999999999999999999999999999', '192578361656909473060171072569158797628.864132774063', '-193000000000000000000000000000000000000', 3, 5);
    T('1.58', '12389351873789686114907585983345754742.52067697108150758940', '-12389351873789686114907585983345754740.94067697108150759', 55, 5);
    T('0.9999999999999999999999998772566052894683281237622210110', '481544.2107916620848341978167816575732682746', '-481540', 5, 4);
    T('1.0000', '224312928491088535491078867518.209582621966231672', '-224312928491088535491078867517.209582621966231672', 48, 1);
    T('1.00000000000000000003095344092698089884798438559', '296191604487069561489309352105.68', '-296191604487069561489309352104.67999999999999999996904655907302', 62, 6);
    T('0.76905311778290993071593533487297921478060046189376443418013857', '8407773.632', '-8407772.86294688221709006928406466512702078521939953810623556581986143', 86, 0);
    T('1.0000000000185965456458031655253334638757311179701311453497853225461606259362087556127', '9309910999303.9839174076506069682058368698829829', '-9309910999302.983917407632010422560033704357649436124268882029868854650214677453839374063791244', 94, 6);
    T('1.000000000000000000000000000000008392480073930219033520326483819552645833758278284616285586293', '563237.4135181051', '-563236.41351810509999999999999999999999160751992607', 51, 3);
    T('1.00000000000000000000000000000000173791249784601545', '36831012283.17604867882082635269432645754', '-36831012282.17604867882082635269432645753999826208750215398455', 99, 2);
    T('1.00000000000000000000012750317283637335579510790689761231095636226607116227629047443610652534507966', '92564119958417.8748994922158424232005961831', '-92564119958416.8748994922158424232004686799271636266442048920931023876890436', 75, 6);
    T('0.998516796478580341790335696309894805127098159112503597494124102198029899742', '8184903268.1445427297440022417177939179595055380', '-8184903267.14602593326542', 24, 1);
    T('1.00000002024096609806046', '47783826.1', '-47783825.09999997975903390193954', 37, 0);
    T('0.9982936719052187825031687750854676398', '13277642397665723374.047324451029771276928024', '-13277642397665723373.0490307791245524944248552249145323602', 84, 5);
    T('1.4', '353359280649551292084180478.597907759915257816362081', '-353359280649551292084180477.197907759915257816362081', 94, 6);
    T('1.000000000000000353230210919399860868204213401471460222851000051555853475199990855519614012530', '24603149595727.05965836452495115745544478238909931485', '-24603149595726', 15, 1);
    T('1.00000000000607', '4598452832610736.89586574692195836900903', '-4598452832610735.89586574691588836900903', 69, 4);
    T('1.00000000000000000000000000000015250802705521539647538645284454646619', '78008648680958909732.0', '-70000000000000000000', 1, 1);
    T('0.9', '75.796258051898302', '-74.896258051898302', 55, 6);
    T('1.000000000000000000000000000000000000001360184978748427', '561995082950640232690800228541974371605.175598996298350692318714', '-560000000000000000000000000000000000000', 2, 1);
    T('1.0', '4967293396627539703394121434693.556991311414208903543128556272623217066', '-4967293396627539703394121434692.5569913114142089035431', 53, 4);
    T('0.99999999999999999999999999999461225532013076840957982', '394.108693417', '-393.10869341700000000000001', 26, 3);
    T('1.0000000000024269288078486', '7096432752575758379943874819778337347.781407215981356003309717700773535', '-7096432752575758379943874820000000000', 27, 5);
    T('1.00000000000000000000000000', '21691422.35039151001802442750147545968844451', '-21691421.35039151001802442750147545968844451', 75, 6);
    T('0.982314032046973930883237239574121891691132255668352728944855151922464716494', '268058632054477091.124174355082975338862386', '-268058632054477090.14186032303600140797914876042587810831', 56, 6);
    T('0.99999999999999999999999935155979414474473147050752268991', '85790985870659589934899584834.225997707624150', '-86000000000000000000000000000', 2, 4);
    T('1.0', '0.02344752131344004', '0.97655247868655996', 82, 4);
    T('0.9999999999999999999999965236293925667660593904153237137291131225357895670371798036', '5354049.56963556256710502711', '-5354048.5696355625671050271100034763706074332339406096', 53, 4);
    T('1.0000000000000000000000132633066537326453355501221876', '606134418080833390128329243.4255126431809', '-610000000000000000000000000', 2, 6);
    T('1.0', '633497481337998433146414038.6628194607045363419671496672081130', '-633000000000000000000000000', 3, 2);
    T('1.01', '6288803387590734396762.874030467883', '-6288803387590734396761.864030467883', 36, 3);
    T('1.00000000000000001168102938198829585', '833770006428163044228278012.7453341461387', '-840000000000000000000000000', 2, 0);
    T('1.0', '696580765459230665.876179950669059', '-696580765460000000', 11, 6);
    T('0.99999973851', '22170385143393613373114351872662049431.69627622583039506194793140', '-22170386000000000000000000000000000000', 8, 0);
    T('1.0014920', '711.784243840187937283030304365340527314517', '-710.782751840187937283030304365340527314517', 83, 6);
    T('0.99999999999999999999999999999999999981938442156349351222313367542297451961604400502', '90.779986', '-89.7799860000000000000000000000000000001806155784', 48, 1);
    T('0.999999999999999999999998243468525701389156169593', '5683777050532549401878175915915892759797.984383751094945226738', '-5683777100000000000000000000000000000000', 8, 5);
    T('1.0000000', '2797178468806.26093358409144454631489893497', '-2797178468805.2609335840914445463', 32, 4);
    T('1.0000000000000000000000000000000', '3927958941796.505464056593559936498', '-3927958941795.505464056593559936498', 48, 1);
    T('0.999999999999999999999999999999999999998047616073', '9028.439985695082026903090149', '-9027.43998569508202690309015', 27, 5);
    T('0.982162335252275743709709769', '7173.692702672161608330869958042221772', '-7172.710540336909332587160248273221772', 95, 1);
    T('0.99999999999999999999998893308403782611294263345766344733801633920368387672259457033820373629675', '341429211994033358333494181402149695729.904624760035', '-341429211994033358333494181402149695728.9046247600350000000000110669159621738870573665423', 88, 6);
    T('0.9999999999999999999999999999999999999885904300074106360137402782884504871123394299985367', '3154266650863347662766400595651.9645772819783423566651748', '-3154266650863347662766400595650.96457728197834235666517480000000000001140956999258936398626', 91, 4);
    T('0.9999999999999999999999999997928806063488968023095949111138819251956550121153654965899021513', '83263670886170673970551423161683432832.8061356966110062730147497644775310092', '-83263670886170673970551423161683432831.8061356966110062730148', 60, 3);
    T('0.999999999999999999999999999938661647999894053487270229390072', '947367929382070191678015166112941670332.364', '-947367929382070191678015167000000000000', 27, 3);
    T('0.999999999999999999999999997', '1597633952458070708476606701961049593428.708215699129760784147714', '-1597633952458070708476606701961049593427.708215699129760784147714003', 77, 0);
    T('1.0000000000000000000000000000000000001008563640018091560127112918844781953856', '586854210762409680465443544.8749314975742', '-586854210762409680465443543.8749', 31, 1);
    T('0.9999999999999999999998336226807', '2862986575461280.65656654445561491703', '-2862986575461279.6565665444556149170301663773193', 81, 4);
    T('0.999999999999999879052118627176734402800019042169104757371542602179165745256807243', '74014333.43267960327310926035278033920322885193', '-74014332.43267960327310938130066171202649444912999', 49, 3);
    T('0.9999999999977832175355313924283678251553091466355', '483814.109108634369', '-483813.1091086343712167824644686075716321748446908533645', 78, 3);
    T('0.999999999999999999999999999999999998690080712744239940920645145812909499572896', '2059473832107202960359176607976521.92979712555927', '-2059473832107202960359176607976520.92979712555927000000000000000000000130991928725576005907935', 93, 2);
    T('0.999999999999999999944933080259575358631464720283071577663330686505172989509081621278591909223', '4895555919074942465784442817550175385154.354618368526558824239549', '-4895555919074942465784442800000000000000', 26, 5);
    T('1.0000000000000000000000000', '2652208.741', '-2652207.741', 42, 6);
    T('0.999999999999999953903219067902345333147690', '984556546226135343746576431.19', '-984556546226135343746576430.19000000000000004609678093209765466685231', 96, 3);
    T('1.00000000000001179029641797259255488262396125535603777129769645409917984735659084564335773021414', '88000415290513.79068803', '-88000415290512.7906880299999882097035820274074451173760387446439622287023035459', 79, 4);
    T('0.9999999999999999999999256208756152437423521396222414058143972369390360771892004', '7232470.0', '-7232469.00000000000000000000007437912', 36, 2);
    T('0.999999999999999999999999997750431982', '29.513755704523057912861751', '-28.51375570452', 13, 4);
    T('0.9999999985553', '6340.16220560066849606118159172930044', '-6339.162205602', 13, 5);
    T('1.000000000000', '369844547836658.9136732633397129696002842440340622780', '-369844547836657.913673263339712969600284244034062278', 99, 5);
    T('1.00000176032568443080505682310752259009887559817496849456608283818671745892532426570972624463256058', '2.611199736022863347572344761157', '-1.61119797569717891676728793804947740990112440182503150543391716181328254107467573429027375536743942', 100, 0);
    T('0.9817265931172036200675583712401383880906981933695239842188546491376516949135704319778849664044421191', '43542977051335016217833436.670621186804443636650703067710293006889', '-43543000000000000000000000', 6, 3);
    T('1.00000', '4394355.98', '-4394354.98', 20, 0);
    T('1.0000000000000000000', '8488775420653.675682394', '-8488775420652.675682394', 24, 0);
    T('1.00000000000000000000001', '98853992241270916014172440649276444.7515715', '-98853992241270916014172440649276443.75157149999999999999999', 96, 0);
    T('1.00000000000000000000000000000000003008617167791640288883527008077531343840035432259403990627216', '554658612163206683485071545654054.531437', '-600000000000000000000000000000000', 1, 0);
    T('2', '15197120427416026090138967079151.21', '-15197120427416026090138967079149.21', 45, 2);
    T('0.999999999999999999999999971905113034749799892', '4008207675756969827.92001', '-4008207675756969820', 18, 2);
    T('1.00000000000000000', '8032264542494105.599601076543158', '-8032264542494100', 15, 2);
    T('1.00000000000001', '8849919349501711992777003484842.718305217124769174278598472631278095419', '-8849919349501711992777003484841.718305217124759174278598472631278095419', 91, 0);
    T('0.9999999999999997292040301978673027890638353207809734529560298087721670274695147455799712466', '9119010052283765.44008231779999522', '-9119010052280000', 12, 1);
    T('0.999999999999', '4108221685083231476.7612142409066023589560128', '-4108221685083231475.8', 20, 4);
    T('0.98986383839199220140', '284170769640076812980425103215329.8430727134910182857860228708', '-284170769640076812980425103215328.8532088750990260843860228708', 61, 5);
    T('1.025334109317563559633227858832616086500618767757136800955918', '14991490405185767758089213090114.3572929135003167286447080453040389099280', '-14991490405185767758089213090113.331958804182753169011480186471422823427381232242863199044082', 95, 0);
    T('1.0000000000001393416841031711306739999509532881430315909567091666140931376726446660297476949364', '433073066217388676733605056.0', '-433073066217388677000000000', 18, 3);
    T('0.999999999999998632', '45254583559664040216562.0173877553', '-45254583559664040216561.017387755300001368', 50, 3);
    T('0.99999999999999875701525702928427905701295983057278', '731360668024103139162937233015.2528017761078', '-731360668024103139162937233014.25280177610780124298474297', 56, 1);
    T('-0.1886168867005764047082125297953992366301391749240237451', '408978287702425.18707', '-408978287702425.3756868867005764047082125297953992366301391749240237451', 82, 0);
    T('0.9999999999999999999999999998544830660789857279055922910482380891419763982990139854', '254878106603663593850295506682529196409.3414260', '-254878106603663593850295506682529196408.341426000000000000001', 60, 0);
    T('0.999999999925949267460215505273249651937093203167167799440079', '674341886827614.2358360046173568879350027993696952308', '-674341886827613.235836004691', 27, 6);
    T('0.999999999999999986576837861', '7302123011891917441254970888331408982.5689629287524197263873408022031556126', '-7302123011891917441254970888331408981.5689629287524197398105029412031556126', 98, 0);
    T('0.99983753007227321013454100425336510222297665791853317818108408525344413323946078721430653325339496', '55343927521021024498.6719939847673231221', '-55343927521000000000', 12, 2);
    T('1.00000000001', '4103505309.891322064831', '-4103505308.891322064821', 48, 0);
    T('1.00000000000000000000000000000000000169952535613', '91166208.75266586309297608601563071660', '-91166207.752665863092976', 23, 6);
    T('1.0000000000000000000000', '771.01962657755496', '-770.01962657755496', 99, 1);
    T('0.999999999999898906976715118265619960471237077473726625121927456714128157670313775382972949264474993', '5912720938599.6638312056446520560693223889233131525001', '-5912720938598.66383120564475314909260727065769319202886292252627337487807254328587184233', 87, 3);
    T('0.999998794508491044834566741831552951388967960299976750882286096354517280298391812641778', '7384.222972', '-7383.222973205491508955165433258168447048611032039700023', 55, 2);
    T('1.000000000000011239825627716800462768077695585615526580', '5756355649216889746026091.2689239217133183518427264', '-5756355649216889746026090.26892392171330711201709868319953723192230441438447342', 87, 0);
    T('0.999999999999999999999999999999999987380094131916618416710647416521642329836301723357646', '8428276895095820328436479.8657', '-8428276895095820328436478.8657', 31, 4);
    T('0.9999999999997001416194793161205', '5152776871223841945455960.5283475769054924408', '-5152776871223841945455959.5283475769057922991805206838795', 74, 6);
    T('1.0000000000000000000000000000000000000118913578725192810209887891961351775', '516573.6461696413172270589038772723334', '-516572.6461696413172270589038772723333999999881086421274807189790112108038648225', 84, 5);
    T('0.988047722768330819695216271215385500718718161554270898961598387068316368326166887034', '9742996697374540.97005704636905535944', '-9742996697374539.982009323600724539744783728784614499281281838445729101038401613', 79, 0);
    T('0.9999999999999999999999999999998201136655523088935277434007490305517823146574555', '62219933610011.16426043113116781010214047925634829', '-62219933610010.1642604311311678101021404792', 42, 2);
    T('0.999999999999999999999989969965324174187833', '905897758670836950647064729055.63018248819197591443974770029456849', '-905897758670836950647064729054.630183', 36, 0);
    T('1.00000000000000000000000000017873942', '3883908.68669556429115171', '-3883907.68669556429115170999999999982126058', 53, 6);
    T('1.0000000000000000000000000540274580375731096498634210', '67357021321853243314358.2428610469930264571', '-67357021321853243314360', 22, 3);
    T('0.9999999999999999999999', '709393256.47249517836182077569773830693', '-709393255.47249517836182077569783830693', 99, 5);
    T('0.999999982315870315315802230720438186424215874426113004415522553873303035927987934236552331103854384', '61107696451361.9873895400629248554583115', '-61107696451360.987389557747054540142509', 38, 6);
    T('0.99999999131472471735565311088200644442', '5263969539829125924784475413109001033748.726700932617168109794995844', '-5263969539829125924784475413109001033750', 39, 6);
    T('0.962350520677946997496942798446481844609', '5282601797278.0', '-5300000000000', 2, 6);
    T('1.0', '25530208213361005476556746375094356159.226937585162753200489513665658', '-25530208213000000000000000000000000000', 11, 6);
    T('1.0000000000', '392949650190513662518485631974741185.757121478149117525620', '-392949700000000000000000000000000000', 7, 0);
    T('1.000000', '2516260934387347819688535397610629672565.81360345195818357', '-2516260934387347819000000000000000000000', 19, 2);
    T('1.000000000000000000', '5.2', '-4.2', 36, 3);
    T('0.999999999999999999999999999971923862', '5447514.611080432938857186513026790788740050', '-5447513.611080432938857186513027', 31, 0);
    T('1.000000014759420981876853076001', '670315571930474314.177935757206', '-670315571930474313.177935742446579018123146923999', 81, 2);
    T('0.999999999999999999999999999999868391331374649959856513057168040195723836626710421', '94456355440187760357.6690576421804253808784853454372073850694', '-94456355440200000000', 12, 4);
    T('1.00000000000', '6.986069901428310444100998', '-5.986069901428310444100998', 44, 6);
    T('0.99999989320494131648847731806446344463924605', '3371.28442101179', '-3370.28442111', 12, 2);
    T('1.00000000000', '72619290398513260609431348105093223261.3085', '-72619290398513260609431348105093223260.3085', 81, 6);
    T('0.999999999999999999999999999999998866682335208718149737443491704100446907518498179', '1496133518704751972128441988528628.96243260323711063181802428740837925', '-1496133518704751972128441988528627.96243260323711063181802428740838038331766479128185026', 87, 5);
    T('0.999999999999999999999999999999807621936668641930351162511589470409172907288869795587759', '2877508320547283413337227989821.153968496983044175832336', '-2877508320547283413337227989820.154', 35, 4);
    T('1.0000000000000000000000000000000000', '907835852835183508297232254405034526311.2778650181516030504011605185217140', '-907835852835183508290000000000000000000', 20, 2);
    T('1.0000000000000274277', '9608983121397048012.51459949347802679001326068208800392990', '-9608983121397048011.51459949347799936231326068', 45, 4);
    T('0.999999999999999999999999999999999999999890294', '911521025.7860821894887199034376382537234590', '-911521024.786082189488719903437638253723459000000109706', 66, 1);
    T('0.999999999999999999999033959501596152456959716066853511140752873356', '91284529365231552621913.784092768548660819501942649664815045', '-91284529365231552621912.7840927685486608', 39, 5);
    T('1.00000000000000000000001266823553432077', '1504901357788478194357512954662743691047.3353511885831276441723', '-1504901357790000000000000000000000000000', 12, 0);
    T('1.00000000000', '394536776753332.0736697027469645098865144465694388', '-394536776753330', 14, 6);
    T('1.0000000000000', '2746843188255255957.61220702', '-2746843188255255956.61220702', 64, 4);
    T('0.9999999999999999999999999999994485779395910087416038656763838605', '3066766097370300329877.64589710249', '-3066766097370300329876.64589710249000000000000000001', 51, 0);
    T('0.999999999999999999999999999805774559915017253729582', '577105452819779164713290850049.36235022', '-577105452819779164713290850048.362350220000000000000000000194225440084982746270418', 91, 0);
    T('0.9999999982409871234343692471200767384909998970003826852029660357299454646189108816273046081', '96472.694035343064525262851', '-96000', 2, 6);
    T('1.0', '20335351690428469618918296983.18633117672489507511594977142425771', '-20335351690428469618918296982.18633117672489507511594977142425771', 86, 3);
    T('1.0000000000000000000000000000000000104810123069938621726156767586009733957316493618874', '282204071324320839200040099433.1270803', '-282204071324320839200040099432.127080299999999999999999999999999989518987693006137827384', 87, 2);
    T('0.999999999999999999999999999999999999988735356259170716526385270631871227470562118098767', '45279629210870954458200066757853.7698704804', '-45279629210870954458200066757852.76987048', 40, 2);
    T('0.9998830899653607540143943556759710939059', '804535529183261748371966600645815.50994035338821027000578372499182', '-804535529183261748371966600645814.51006', 38, 3);
    T('1.0000000000000000010448413415474138815', '428.0866154598', '-427.0866154597999999989551586584525861185', 96, 3);
    T('1.00000000000000000003188181902396629580149496927879062381823361180916048697700392524622389585484', '314856003.355894229947336165129745432390535881308', '-314856002.3558942299473361650978636133665695855065050307212093761817663881908395', 79, 5);
    T('0.9999999999999999999463262399630272126748053695829244107006379039057976923389075', '329504038989052.4921120548589239623071', '-329504038989051.4921120549', 25, 5);
    T('0.9999999999999999756933117', '72601297477338052032573900233321128.584794750', '-72601297477338052032573900233321127.5847', 39, 1);
    T('0.999999999999999999910339061414778033140', '1717271756262808927594161141164929549507.28288250933639195353755674', '-1717271756262808927594170000000000000000', 24, 0);
    T('1.00000000000000000000001', '6888203535179038937309447377165.321468771742', '-6888203535179038937309447377164.32146877174199999999999', 89, 5);
    T('0.99999999999999999999999999999940378558395505377105849594182076849092381832555284400547977', '3062.0', '-3061.0000000000000000000000000000005962144160449462', 50, 2);
    T('0.99999999999998674799882855086646200520213771452614', '13.5711632878985796664121783', '-12.57116328789859291841334974913353799479786228547386', 70, 0);
    T('0.9999999999989015270373399936207998840391573305048238259544413572972974', '19122143834856172389857.36661371506099711991543353', '-19122143834856172389856.3666137150620955928780935363792001159608426694951761740455586427027', 90, 1);
    T('1.00000000000000124934319049161561942983634488875102774908171460594896723028742915019413382', '944777085276680317227892910258657.2734660388162914', '-944777085276680317227892910258656.28', 35, 0);
    T('1.0000000000000000000000000000000000', '8454567315948932593261.084948', '-8454567315948932593260.084948', 51, 5);
    T('0.999999999999999999999999998987110992200676423687761', '84344309898256039711176326465.808323', '-84344309898256039711176326464.8083230000000000000001', 51, 3);
    T('1.00378090352572055210845355249483160941201952933342', '9508070748454933948268631427535.88347380001622', '-9500000000000000000000000000000', 2, 1);
    T('0.99', '61.0938162878', '-60.1038162878', 81, 4);
    T('0.999999999999999999999995584599192068192720372856980557501659936575191800354014486', '4438378473540106.7284026555961231182', '-4000000000000000', 1, 6);
    T('1', '6998422369775404909709582603366695.304399091552964849714257', '-6998422369775404909709582603366694.304399091552964849714257', 81, 5);
    T('1.00000000000000000000000000000012291793370327867460835145673885345638687263237624', '5809720001731585624354277.45217950668251782097144836', '-5809720001731585624354276.45217950668251782097144835999987708206629672132539165', 78, 6);
    T('0.999999999981214784820881540005873127487736611832440478519680540795189169729039', '6807.315394528', '-6806.315394528018785215179118459994126872512263388167559521480319459204810830270961', 91, 1);
    T('0.9896002802876803457293275811279980643113696731166571618837794007067573039336808780224977395', '941474916676736071133389886065.8641632267543', '-941474916676736071133389886064.8745629464666196542706724188720019356886303268833428381', 85, 2);
    T('0.9999999999984067007845751003490110478860712703226248743149216393074852372685154066706', '8098385.89280749574032053920996', '-8098380', 6, 2);
    T('1.00001', '221163061736.173189736051189934016855327865651038333', '-221163061735.17317973605118993', 29, 4);
    T('1.0000000000000000000000000000', '236050424264564671037003121532.814766220473425251903036', '-236050424264564671037003121531.81476622047', 41, 5);
    T('1.0000000000000000000000000000000000000002', '13363451.6484', '-13363450.6484', 39, 5);
    T('0.999999999999999999999999999999999999999', '509902882441482296770217054732008254.90958930942735', '-509902882441482296770217054732008253.909589309427350000000000000000000000001', 78, 3);
    T('0.999999999999999999999999999999999989812960347813027839606947231347239294948990', '707562820.078539259', '-707562819.07853925900000000000000000000000001018703965218697216039305276865276', 78, 1);
    T('0.999999999999999999999846395573469943783018920354196359090452724118917787827674', '75471416390.44200107584137980868925513516428', '-75471416389.442001075841379808689408739590810056216981', 53, 1);
    T('0.99999999999998891101647850501722997031217660444288117', '56882407983.68194648894549590150596767', '-56882407982.681946488945506990489489164983', 41, 6);
    T('0.99999857982442231596048674698358688165920', '1158158339.04', '-1158158338.0400014', 17, 2);
    T('1.0000000000000000', '1196737942534330472313839051498896457.6912811885596077087404', '-1196737942534330472313839051498896456.6913', 41, 4);
    T('0.99999999998750786309172015781176013476822', '970872086866.1362697', '-970872086865.136269700012492136908279842', 39, 6);
    T('0.999999999999999999999999999986122771577', '45837794797.252564052346078051753837681214886740160', '-45837794796.252564052346078051753837681228763968583', 82, 5);
    T('1.317592051316048533938353305152355269973056598762605535390053550443909160615378084', '247679.5', '-247678.182407948683951466061646694847644730026943401237394464609946449556090839384622', 84, 4);
    T('0.999999999999999999983517773172686381200149591544033025403701494895151105394878640566', '759583825457791.2309126635565535873902865819442925', '-759583825457790.230912663556553587406768808771606118799850408', 60, 4);
    T('1.00000000000000181930762633804197956500519486832989253837021', '59304942076072786686877461631947.0', '-59304942076072786686877461631945.999999999999998180692373661958020434994805131671', 80, 3);
    T('1.0004716115028958973735700845507738178173927127509541510939656752461726423279505', '1752763885890336220194.06873112354', '-1752763885890336220193.06825951203710410262642991544922618218260728724904584890603432475383', 90, 3);
    T('1.00000000000000000007392018206163251072127526301729664608280774766164116178813989404011281', '5568553964312073702064527630784358.508440503', '-5568553964312073702064500000000000', 23, 2);
    T('0.99999999999999996771820', '7378515885944032959447180.55859886015844931412', '-7378515885944032959447179.5585988601584493464018', 81, 2);
    T('0.999999897900296658988763189772765667900826065349529967827761690054012522665224179', '4778686037565644741792099048195453574.0', '-4778686037565644741792099048195453573.000000102', 47, 1);
    T('1.0000000000000000000000012081199453722227238214', '892836377648154201891949687364378.42495673166867748', '-892836377648154201891949687000000', 27, 2);
    T('1.00000000000001149287596525', '9530.7657676529808825162663210173594215088173', '-9529.765767652980871023390355768', 31, 3);
    T('0.9999999999999999999999999999883', '812185.3971938241228415', '-812184.397193824122841500000000001', 33, 0);
    T('1.00000000000000000000000000077751', '34857292050544292494789648042183697505.95982746462965147', '-34857292050544292494789648042183697504.95982746462965146999999999922249', 79, 0);
    T('0.9999976123736348746257395672665975846771690391713981462468901166594242000257864', '34006429901.27341569144110053856496618638338', '-34006429910', 10, 0);
    T('1.000000000', '82235975155349.83', '-82235975155348.83', 47, 6);
    T('0.99999999999999999730357028583506085115218599114', '7611753576.784962906950', '-7611753575.7849629069500000026964297141649391488', 47, 1);
    T('1.0000000000012861047894438710266484126995762968', '5326623093545972422634427.1174970730382336598263521298308', '-5326623093545972422634426.1174970730369475550369082588041515873004237032', 89, 2);
    T('0.99999999999999999705439745108949723375235602197473665704294083160388041379166011761320973', '341337621.471216353190616752440', '-341337620.471216353190616755385602549', 36, 4);
    T('0.999999999999999999999701336302936580', '65211693530552951481399717958435383169.898232825084306293795381750259922507', '-65211693530552951481399717958435383168.898232825084306293795680414', 66, 0);
    T('0.999999999999999999999999999871841358804699627364576531488751013593', '251988688221659619254725868400380366.2873490', '-251988688221659619254800000000000000', 22, 3);
    T('0.9999999999999999999999', '7824658833.42290778358588258426938555', '-7824658832.422907783585882584', 28, 1);
    T('1.000000000000000000000103253', '33424495968067509696.2828672034', '-33424495968067509695.282867203399999999999896747', 70, 2);
    T('1.000000000000000000027246932116696798973053293301266811177098505869624', '51621604323272250745612.570084836233712131518931820222440592521', '-60000000000000000000000', 1, 0);
    T('2', '1121193153.3624218157158069278303665820459', '-1121193151.3624218157158069278303665820459', 80, 0);
    T('1.0000000000000000000000000001472342640322339761564491430808760529341436641514275', '61104531589.548', '-61104531588.5479999999999999999999999998527657359678', 51, 5);
    T('0.996697037575045661064478364551584697771206979120673', '105635867848287515.18975846722612', '-105635867848287514.1930614296510743389355216', 43, 5);
    T('0.9999999999999999999999999999996683322151074', '1305645196527803298330571314914.554245051937350784211357108477', '-1305645196527803298330571314913.5542450519373507842113571084773316677848926', 91, 4);
    T('0.9999996679035012325164465273241745317908553178924758169005856149723089217436077040973235285', '3306291889487788817998476494892.898439141065231497484785797', '-3306291889487788817998476494891.8984394731617302649683392696758254682091446821075241830994143850277', 99, 3);
    T('1.00000857446899022621644711361411328593658853755072647486998560072045634219056043262656478537108720', '26464024134550007230692973521697829002.8196568707138618215529238', '-26464024134550007230000000000000000000', 19, 2);
    T('1.000000000000000001', '281927393286638248086477.159280250753', '-281927393286638248086476.1592802', 31, 1);
    T('0.9999999999999999999987361485055', '85349066420282282026680.25356333', '-85349067000000000000000', 8, 0);
    T('1.0000001', '9833434833081068018998.0', '-9833434833081068018996.9999999', 44, 1);
    T('0.99999999999999997057643850622700970154677220', '792428015258921327781231055127263937541.985877072511017', '-792428015258930000000000000000000000000', 14, 0);
    T('0.99999989403799', '658055196467617788582.345008903195799268099626', '-658055196467617788581.345009009157809268099626', 81, 2);
    T('0.999999999999999999999999998931543093167111562586396041702375661950456279785213396', '4921252739115739944847505282261098.4966034022203375647653286882768679', '-4921252739115739944847505282261097.49660340222033756476532868934532480683289', 75, 4);
    T('1.00000000000002415536355374741560311097179999094030312426108763763652737412', '65520412917210.5426820949146351598833131885049077', '-65520412917209.5426820949146110045197594410893045890282000090597', 63, 4);
    T('0.999999999999999999999999999999812664525001935597957444753150696', '675802148051501728043114703405700.7191743332764053240084', '-675802148051501728043110000000000', 23, 2);
    T('1.0000000000000000000000', '64580595383469059818115.19932469918', '-64580595383469059818114.19932469918', 96, 0);
    T('0.999999999997865406827665661016259769583047758262161903850994450745437176632216051223368659630940', '4734850880618601417789.80321298131980064768864285586667713', '-4734850880000000000000', 9, 1);
    T('0.999881117', '868507.1098440045088589773657886277701375507', '-868506.1099628875088589773657886277701375507', 54, 6);
    T('1.00000000000000000000000000000000000257677171584721956', '28917944187.5214923555573687603415847740', '-20000000000', 1, 2);
    T('1', '1748031370554696198639.105798789998', '-1748031370554696200000', 18, 0);
    T('1.00000000000000001', '8328466.53378861777', '-8328465.53378861776999999', 61, 0);
    T('0.9997955451466602735689376945231532187428453918269568746632357', '11328964278241548976710887418549319.405', '-11328964278241548976710887418549318.405204454853339726431062305476846781257155', 77, 6);
    T('0.99999999999999999888591714268380879432198835319715295646133704134267246802106', '116782261224.940485942177273184725753980', '-116782261223.9404859421773', 25, 0);
    T('1.000000000000000000000001', '6844548166488304526771009241072.683', '-6900000000000000000000000000000', 2, 0);
    T('1.1', '4192528042591.890855255276227', '-4192528042590.790855255276227', 100, 3);
    T('1.000000000000000001429022458179353313554737428281814379718787848686262758477477924197850849184984377', '50711812084510124.003697763065290071874735596933159', '-50711812000000000', 8, 4);
    T('1.0000000', '87023.058453662260791897668448143822512', '-87022.058453662260791897668448143822512', 81, 1);
    T('1.00000000000000000000000000020018005587148143687937962703106204979426893687486874', '53322753.0', '-53322752', 34, 5);
    T('0.9999999999999999999999999622849638', '2628923590718434119665715835933431703766.3766952738432641743471499076207773337', '-2628923590718434119665800000000000000000', 23, 3);
    T('1.0000000000000000000000', '205.618702193650895837685810845694639', '-204.618702193650895837685810845694639', 59, 6);
    T('0.99999999999999999999999999999999999997692816388794646958982', '2.710832277598525', '-1.71083227759852500000000000000000000002307183611205353041018', 70, 1);
    T('0.9999999999998696365536454024005971701191929235718473708256597382083274', '782408.779698552941154126', '-782407.7796985529412844894463545975994028298808070764281526291743402617916726', 79, 6);
    T('1.000000000000000000000148781899526765350397513425399530279594960460096255427356', '83211437509868211149188906.485220974', '-83211437509868211149188905.48522097399999999999985121810047323464960248657460046972040503954', 91, 6);
    T('0.9999998533661447587605352209878288604648334253948075490210248441842678435856516084229858237', '3532980130496946123943005902.762', '-3532980130496946123943005901.76200014663385524123946477901217113953516657460519245097898', 87, 6);
    T('1.00000000000000000000016789096749974599811753291180506434251069835111795344885321421258', '5531835504653590253.0', '-5531835504653590252', 33, 4);
    T('0.999999999999999999999999822849923', '6532274700750975441917352502857.6860620', '-6532274700750975441917000000000', 22, 4);
    T('0.9999999999999999999996', '671763148618.76790109785', '-671763148617.7679010978500000000004', 35, 0);
    T('1.0000000000000000000000000000007115', '27342020.413453263895851027', '-27342019.41345326389585', 22, 6);
    T('1.000000000000000000000', '9808.49265028688774510891158817', '-9807.49265028688774510891158817', 46, 4);
    T('0.9999999999999999999999999999636521332118981311', '831403246984181428772594.0070305436390283810891661704113032785', '-831403250000000000000000', 8, 3);
    T('0.99999999', '143220130.4326', '-143220129.43260001', 52, 0);
    T('0.9999999999999999750243469407474340834000349894118430', '79660.3990243540', '-79659.399024354000000025', 23, 4);
    T('0.99865609414863900556660', '62245.40', '-62244.4013439058513609944334', 83, 5);
    T('0.99999999999999999999999989242323917007004240518683079666052052096256962260699484660', '348745758570380396867827190339233986.2', '-348745758570380396867827190339234000', 33, 5);
    T('1.00000000000000000000000000002075', '60912498.31', '-60912497.30999999999999999999999999997925', 57, 1);
    T('0.999973954883037011015926836993286624997380293073217744400', '13754718594941187467543076845.3842705', '-13754718594941187467543076844.3842965451169629889840731630067133750026197069267822556', 85, 3);
    T('0.9999999999999992669583212233303959147711165500766529262685563694563145448442591869860', '39630395528902727044665752588620.1949967441747422010490', '-39630395528902727044665752588619.194996744174742934090678776669604085228883449923347073731443630543', 98, 2);
    T('1.0000000000000000000000000108720976447153416368028206010274789284486088578878161145094595621578938', '331664042089951214172626383199357238517.303391552445770905', '-331664042089951214172626383199357238516.3033915524457709049999999891279023552846583631971793', 91, 2);
    T('0.9650914984011462088837093941572290416199526404011004893677764680714911766748519923401996422', '8009761972699671510902050260.0637218433543034509052510188637', '-8009770000000000000000000000', 6, 0);
    T('1.00000', '267138822959614444699794.810796672166392461584320', '-267138822950000000000000', 11, 2);
    T('1.0000000000', '808616.649046', '-808615.649046', 78, 2);
    T('0.999999999999999999999999999999999980333794578015241248836867472930971404278615', '482966498.5918242968565416105363252339400', '-482966497.591824296856541610536325233940000019', 45, 2);
    T('0.999999999999999999999999999999999999999191217', '78253166219948257943625803075041.0448', '-78254000000000000000000000000000', 5, 3);
    T('0.99999', '59.07173334966', '-58.07174334966', 100, 2);
    T('1.000000000000000000000000000000000000010766628227316852530728209682588836038128240446397842949033035', '9209440.16260590409807027257916884346166306556', '-9209439.16260590409807027257916884346166306555', 45, 3);
    T('0.999999999999999999999999999999998190637206999', '2684504779301737.43778572256', '-2684510000000000', 6, 0);
    T('1.00000', '76846539407755357016.14076748817944134', '-76846539407755357015.14076748817944134', 42, 3);
    T('0.999999999999999999999999889277786168066994', '395.146405323113673705997289703', '-394.14641', 8, 3);
    T('0.99999999', '3746198.956901949512', '-3746197.956902', 14, 4);
    T('1.0000000000000', '7180107.74289182442608', '-7180106.74289', 12, 6);
    T('0.999999997752', '6739663254195204347105393010210807.40490400050625822543', '-6739663254195204347105393010210806.40490400275425822543', 75, 4);
    T('0.999999999999989787876558211700745775746554587922803180429361044966579241046', '308912029588614955063.134343404989273380355653751190091735', '-308912029588614955062.13434', 26, 5);
    T('0.99999999999986318129732948', '80.878284845033216829075', '-79.87828484503335364777768', 25, 0);
    T('0.9999999989293309914858073', '482.71277296626', '-481.7127729673306690085141927', 55, 1);
    T('1.000000000000000000000000000005604676308676816943403886', '72945002.91', '-72945001.90999999999999', 22, 1);
    T('0.9999999999999999999999', '203713219190582143573999414681011813.4792254', '-203713219190582143573999414681011812.4792254000000000000001', 75, 4);
    T('1.00000000000000003771615772815970044715119007053695950683255807686392680257', '3915373248497750171193552551.333800936724691823126459006750834906', '-3915373248497750171193552550.333800936724691785410301278591134458848809929463', 76, 2);
    T('0.9999999999999999999998333555466400235127630174214693764342906410612894202293', '9008478891133573838.05202487771233038433263727166192386859', '-9008478891133573837.05202487771233038433280391611528385', 54, 5);
    T('0.999999999999999999999999999999999999998746509516292553', '895662832951351252782552.802187', '-895662832951351252782551.80218700000000000000000000000000000000125349', 69, 1);
    T('1.00000000000000000000000000000000004009773002968465921746639179922322', '32738882683244078.225594', '-32738882683244077.225594', 34, 6);
    T('0.9999999999999999999999999974600105', '22521251367609104720453852092.46452822792485123344881007', '-22521251367609104720453852091.464528227924851', 44, 1);
    T('0.99999999999865517755927219537877694310570400', '31504337957743870886684401243.872870565065268983503535486393328', '-31504337957743870886684401242.872870565067', 41, 6);
    T('1.0000000000000000194904668458425315207775', '5546030307018.87261', '-5546030307017.87261', 26, 5);
    T('1.0000000000000000000037639', '85322028147782012870685111026005.5692908978920839048', '-85322028147782012800000000000000', 18, 1);
    T('1.00000000000000000', '603414128675893874.2972020306292421870219685171644927', '-603414128675893873.2973', 22, 0);
    T('0.9999999999999999999688', '4346185639948585276714556978916758205239.9296587871879220708', '-4346185639948585276714556978916758205238.92965878718792', 54, 6);
    T('1.00000000000000000000022207702950299007803992374952804', '48031687162673.5001', '-48031687162672.50009999999999999999977792297049700992196007626', 61, 3);
    T('0.9999998226058432260676531265917777917251636163505240430309838', '66992.61342215714255275468890796921', '-66991.6134223345367095286212548426182222082748363836494', 54, 1);
    T('0.999999999999999999998663415840600641439463208257099450', '9759743028255434942195517.488606531535823781244417', '-9800000000000000000000000', 2, 4);
    T('1.0', '57996626190213220072214829922521.574229827609177731', '-57996600000000000000000000000000', 6, 4);
    T('1.00000', '42766104396525163538386390.854037290724211371783', '-42766104396525163540000000', 19, 6);
    T('1.000000012535693455', '23901211509497672160233654846.552877827380939', '-23901211509497672160233654845.552877814845245545', 73, 2);
    T('0.9999975286865014007336333149606822452534237936192080359749496986117329972', '30300512583446204681983962960534822039.514495', '-30300512583446204681984000000000000000', 24, 4);
    T('1.00000000000000000000000', '808861142371014407206442426416346765736.973360271520', '-808861143000000000000000000000000000000', 9, 3);
    T('0.997628526', '20455649982770.6604107744339774100757031', '-20455649982769.6627822484339774100757', 36, 1);
    T('0.999999999999999999999999999778826081', '2521310021.32177625539', '-2521310020.32177625539', 34, 1);
    T('0.9999999999999999999999999999793434', '21936100226096.573248567731634795120776', '-30000000000000', 1, 3);
    T('1', '581893.2647377162535792275883699241495', '-581892.2647377162535792', 22, 4);
    T('1.000000000000000000000', '8086056346236293399064.030853990', '-8086056350000000000000', 9, 6);
    T('1.00000000', '2745507124474150374939390971839406.07698517611137478516', '-2745507124474150374939390971839405.07698517611137478516', 64, 2);
    T('0.9999999999999999999961969390043253584054886252551742158282439819', '80567080727939896393214777505.2877316', '-80567080727939896393214777504.28773160000001', 43, 3);
    T('0.9999999999942281317560476766162406930155834', '7496286727.752228654687288224537', '-7496286726.752228655', 19, 3);
    T('0.9598277093624871779', '8808487960785634718643080730617970.1', '-8808487960785634718643080730618000', 32, 3);
    T('0.99999999999999999999999999999999', '933.90530257652054589325530333562723271777', '-932.90530257652054589325530333562724271777', 48, 5);
    T('1.00000000000018698217063635445216286608834280695', '313810040196.1504116503331780', '-313810040195.1504116503329910178', 31, 2);
    T('0.9999997326878959161363227822658', '41827790695028.304003775597103602', '-41827790695027.3040040429092076858636772177342', 77, 6);
    T('1.0000000000000000000000000000022030347361974678296281105925102574404669066794', '67202472999988393177134288287.845044748850063994793913100766975717896', '-67202000000000000000000000000', 5, 4);
    T('1.0000', '87288400107199728165933.154091916090720107691321276401656', '-87288400107199728165932.154091916090720107691321276401656', 84, 0);
    T('0.999999999778437175547930039078492975167389949538933966957189746657935249814012410410', '4410966815027378018187292523134157424.88900106367261296833907945850', '-4410966815027378018187292523134157423.889001063894175792791149419421507024832610050461066033', 92, 2);
    T('0.99999999999999999999999999999999999996333767906104893118268797167018129307428635846072413344', '5105727461735242811433329608130373.441509000683290601869180471936569', '-5105727461735242811433329608130372', 34, 6);
    T('0.9999999999999999999999999999999982', '44.00745629840957', '-43.0074562984095700000000000000000018', 82, 3);
    T('0.9999999999999969798809691173753787463621802432762054740955305567990162333051438703', '71130515931992016536748352240977334.109263497184981083078198906221', '-71130515931992020000000000000000000', 16, 6);
    T('1.000000000000000', '162192452294227482053874668187613287.60323209854155', '-162192452294227482053874668187613286.60323209854155', 70, 3);
    T('1.000000000000000000000000000002768047357904191496970713046813434624824', '489507399716.4206557133797867949836415', '-489507399715.420655713379786794983641499997231952642095808503029286953', 69, 4);
    T('0.999999999999999999999999999999982025787720292242064621843298120050086', '5848621454487.4018645897794559085855651717856', '-5849000000000', 4, 0);
    T('1.000', '9.38368086262704949971072426572541189109', '-8.38368086262704949971072426572541189109', 80, 4);
    T('0.99999999999999999999999999999861173416339319266302412992271072195798867653633870', '20064119943830522205.67', '-20064119943830522204.6700000000000000000000000000013882658366068073369758700772892780420113234636613', 99, 1);
    T('0.999999999999999999999999999998702445274324957953483861311085685127385680238661929576903894136812812', '808299736003634863481.30078742930002', '-808299736003634863490', 20, 0);
    T('1.0000000000000000000', '475957.7174820811835', '-475956.7174820811835', 19, 2);
    T('1.008283019515387335', '9126082400343495.19410664119957538192448456', '-9126082400343494.18582362168418804692448456', 76, 6);
    T('0.9986118948245160949753393829845712872786378920011564474676271962183984628679', '790945916047.89772159203860998', '-790945916046.8991096972140938850246606170155', 43, 3);
    T('1.000000000000000000000000000175332618389661', '4101088458327945.2495925433852936833299097', '-4101088458327944.24959254338529368333', 36, 3);
    T('1.00000000000000000000000000000000001', '121999305545280518859103346080125112204.018497439648825809', '-121999305545280518859103346100000000000', 28, 5);
    T('0.9999999999999999999840845914', '36093589209884818303191776.64340898080560853', '-36093589209884818303191775.6434089808056085300159154086', 66, 6);
    T('1.00000011314107259265817449113623010501108344679469879519760879493', '719.0', '-717.99999988685892740734182551', 29, 0);
    T('1.0000000000000000000000000000', '6132490617066440456622327.41382390663680581339', '-6132490617066440456622326.41382390663680581339', 94, 1);
    T('0.9999999999742208649739147546992531069426992188730241751879003434092140257522116216101803450587', '6168388943.867346010385646335385', '-6168388942.867346010411425470411085245300746893057300781126975824812099656590785974', 82, 1);
    T('0.9999999999999999999994628793563725707659721617494319320350612648592016330824000812', '9536419.518621091460', '-9536418.51862109146000000000053712064362742923402783825056807', 60, 4);
    T('0.999999999999999999999999999965981891122867194657993018042801', '129192539572564059.85130240404100760', '-129192539572564058.851302404041', 32, 1);
    T('0.99999999425552234636004874101756', '611103950473185416.0', '-611103950473185415.00000000574447765363995', 41, 2);
    T('1.0000000000000000000000000000000000000362', '8330392910365975839322457446670553380064.37567397728249951851526084708003145', '-8330392910365975839322457446700000000000', 29, 4);
    T('0.99999999999999999998030944512', '923357623718732146219.9', '-923357623718732146218.90000000000000000001969055488', 91, 1);
    T('0.9999999999999999999725229187887324620787426710525614006657551747992119372796386907749097411', '86716862576355298372804882.826668596807101721', '-86716862576355298372804881.8266685968071017210274770812112675', 60, 2);
    T('0.891360252258046651666262450066242819533180611531955037995762', '6563959565914322881383757886.917100159431426330941391045706666', '-6563959565914322881383757886.025739907173379679275128595640423180466819388468044962004238', 95, 3);
    T('1.0000000016696140717743369622076040264952378165826444966535335036755462546339754960189616866839', '6.5946', '-5.594599998330385928225663', 25, 2);
    T('0.9999999999999999998414225', '704678022413170184851204169623860968.32693', '-704678022413170185000000000000000000', 18, 3);
    T('1.00013759767307011', '812460791005158087007559833236903974.18', '-812460791005158087007560000000000000', 24, 3);
    T('0.999999999999999999999999', '56.91013851065', '-55.910138510650000000000001', 93, 3);
    T('0.999999999999999999999999999999999078651717684986187830763353249101083083672131793378747463345', '53729679311833879916.72212643479333787699624855', '-53729679311833879915.7221264347933378769962486', 45, 4);
    T('1.00000000000000000000000013624180937562885852', '61162912507298393268.408918697', '-61162912507298393267.40891869699999999999999986375819062437114148', 71, 1);
    T('1.0000000000000000000000000000000000000012196796343038420292497993127537', '0.1230788644469531572484648836427', '0.876921135553046842', 18, 3);
    T('0.999999999999999999', '4138159360.83318663253828394972153250897', '-4138159359.8331866325382839', 26, 1);
    T('1.0000000000000001229740830', '67250706485447175584.0826889887298', '-67250706485447175583.0826889887297998', 36, 2);
    T('0.999999999999999999999988508748275665', '49681804679674188281.3990098912449352019050454849', '-49681804679674188280.399009891244935201905056976151724335', 75, 5);
    T('0.999999999999999999999999984381079568521698043614893833161755961184793502163', '757.9757621719110', '-756.97576217191100000000000001', 29, 1);
    T('1.0000000000000000000000000000', '44684169546606924.0', '-44684169546606923', 88, 3);
    T('0.9999999999999999999999999999985349768187933463185809276641225783693470785950622286831223', '1397938112119281648080358259.0784', '-1397938112119281648080358258.0784000000000000000000000000014650232', 65, 6);
    T('0.99999999999999999999999999999999999998929567431869725814875071229', '45740.34167087459', '-45739.34167087459000000000000000000000000001070432568130274185124928771', 90, 6);
    T('1.00000000000000001390092664698930467946105489195772526646293817052543181112049828448801724', '9059865284.267920255653703178432682238', '-9059865283.267920255653703164531755591010695', 43, 5);
    T('1.000000000000000000000000000000052920506071', '740847801.1', '-740847800.099999999999999999999999999999947079493929', 71, 0);
    T('0.99999999999999999999999599434558980634460957596647786282742325586285727', '564512184858707390903856274829159893563.887937740018581', '-564512184860000000000000000000000000000', 11, 5);
    T('1.0000000000', '153849346762475240207058745378627816873.6340650122903081316145', '-153849346762475240207059000000000000000', 24, 0);
    T('0.999999999999999999999999', '339481376483346777.716459913', '-339481376483346776', 18, 1);
    T('0.999999999999999999', '7682998264914846754675226235761.15', '-7682998264914846754675226235760.150000000000000001', 68, 0);
    T('0.99999999999999999987759155078574327009587492397829699422556803680419', '543865157447.9133934243168828793512357382709', '-543865157446.91339342431688287947364418748515672990412507602170300577443196319581', 98, 6);
    T('1.0000000000000000001971379884390817777013429914562809205000964831814708357272016513519464017669653', '9391889778195287550572021854607798.9441134047513229138124563321084361410472', '-9391889778195287550572021854607797.944113404751323', 49, 5);
    T('1.000000000000000000000000000001603833602106530508', '17653801778668959993.019', '-17653801778668959992.018999999999999999999999999998396166397893469492', 87, 5);
    T('1.16088070819264051244592993527669393862793390312432349021070783294193129399701531188952', '95814776.050458569220832451352042715594', '-95814774.889577861', 17, 4);
    T('1.0000000000000000', '26593413446.28392836746470398529993614270', '-26593413445.2839283674647039852999361427', 62, 4);
    T('0.99999999999896040874808483469225595965454305107211951110783989', '184.5', '-183.5000000000010396', 20, 0);
    T('0.99999999999999986625', '796448784122916149284603822103944022819.0828427169457701071435206004759444357978', '-796448784122916149284603822103944022818.08284272', 47, 3);
    T('0.99999999999999999999999685135770251005790217019', '8423330931383344868848187218421434.91224', '-8423330931383344868848187218421433.91224', 54, 4);
    T('0.999999999999999999999999999999999999999563870498926744', '4940144455034139671361508425076226049.42095978674228916567', '-4940144455034139671361508425000000000', 28, 6);
    T('0.9999999999999998753103961715', '827591481211718535767229211726324048834.08969605261037956095', '-827591481211718535767229211726324048833.0896960526103796856396038285', 76, 6);
    T('0.9999999998925776049790811040703676137621686324578235230823963833776324698125', '17105322063144.34954926372284346366485', '-17100000000000', 4, 1);
    T('0.9999', '9697755282713658403298481596538459.432', '-9697755282713658403298481596538458.4321', 53, 6);
    T('0.99999999999999999999999999999999999999919577293085119', '58074461498.776610967839919095666519871419828622', '-58074460000', 7, 4);
    T('1.000000', '9681611810639967853298.75547518805535', '-9700000000000000000000', 2, 3);
    T('0.99', '907.6672102201601873578168026900', '-906.67721022016018735781680269', 86, 4);
    T('1.0000000000001906359902810341707844322842312708661624834604802296343267483734568070737', '9264.17290050566779073', '-9263.17290050566760009400971896582921556771576872913383751653951977036567', 72, 2);
    T('1.00000000219410500682836984887295739923297226436634289249667195876926419', '7168350230200.64203769822366', '-7168350230199.642037696029554993171630151127042600767027735633657107503328', 73, 4);
    T('1.000000000026611428946684292756963884343365697914358310768602980925781433', '30711197495668165509178419293328957889.36966064424463', '-30711197495668165509178419293328957888.3', 39, 2);
    T('1.00000000000000000000000423834835966728', '26507901382376737215232128.24995126', '-26507901382376737215232127.24995125999999999999999576165164033272', 75, 3);
    T('1.00000000000000000000000000000002425325792513188754232947683287476102568324', '4862869567691324266620104698670417082.472621', '-4862869567691324266620104698670417081.4726', 41, 6);
    T('1.0000000029521800634535867022223733122695', '4506589330255505916380958839516.704240551', '-4506589330255505916380958839515.70424054804781993', 48, 1);
    T('0.999999999999999999999999999999999976851540104416', '42293529505.561569173372051112612975052', '-42293529504.5615691733721', 24, 0);
    T('0.999999999999999998859010', '1661671450569088.0', '-1661671450569087.00000000000000000114099', 80, 0);
    T('1.0000000000000000000000000000111360158322886005852576303546314011852429419271247', '93939314.46578658799426616', '-93939313.46578658799426615999999999998886398416771139941474236964536859881475705807', 82, 2);
    T('0.9999999999999999999998770154790480869633545879631257941833370190599047521999789229', '199829413575720837193657766.733709709179385212202071225', '-199829413575720837193657765.734', 30, 5);
    T('0.999999999999999999999999998135', '554606063.013454693418100772398897875014035248', '-554606062.01346', 14, 0);
    T('0.98773519715148', '2549541405629529059884688169455754452231.105907313888980241014461140', '-2549541405629529059884688169455754452230.11817211673750024101446114', 74, 6);
    T('0.99999999999999999999999999999999999999988230202698086139860756333417913240', '690065016931922.4085310572334', '-690065016931921', 15, 4);
    T('1.00000000000000', '1289887282480591855765.347633806181', '-1289887282480591855764.347633806181', 51, 0);
    T('0.999999999999999999999999999995658364099102648491089', '414085174968159470504048.51', '-414085174968159470504047.510000000000000000000000001', 51, 3);
    T('0.999999999999999295252487060650057351383150516174879', '6306669915887513524394303183172309.934564178035290474234779480', '-6306669915887513524394303183172308.934564178035291178982292419349942648616849483825121', 99, 4);
    T('1.00000000000000000000081253348145920781026085647520801897170982656182514222627649830571537940570097', '34180955261225283278.9546214472426960215969277072', '-34180955261225283277.954621447242696021596115173718540792189739143524791981028290173438174857773723', 98, 1);
    T('0.99999999999999999999999856479373626889523595783214491795216121174762099354921379755267504653649134', '8233795130549636376322.3369072392151862574878', '-8233800000000000000000', 5, 0);
    T('1.0001', '1607764700164263.45772807689415459406211472014192328', '-1607764700164262.45762807689415459406211472014192328', 65, 3);
    T('0.99999999999999999999999999999999999998756496998614330743704048912', '36423078435.49585469762969072900556033', '-36423078434.49585469762969072900556033000000000001243503', 56, 1);
    T('0.99999891041356952032727215514401427884704660602746473835', '195363860454397925.197522412317', '-195364000000000000', 6, 4);
    T('1.00000', '781339417511868985543776052995.98028599275239777987', '-781339417511868986000000000000', 18, 0);
    T('1.00000000000000001', '72.130607640926138152719501354908', '-71.130607640926138142719501354908', 74, 0);
    T('0.99999999999999999999999999999815156221279091250236161202515555239214607133', '184535429818126810905.866146730925744980274670738002788577', '-184535429818126810904.866146730925744980274670738004637014787209087497', 69, 1);
    T('0.999999999999999999999331856438039650339670670738698379280474551045601', '175218340352452.70507614829', '-175218340000000', 8, 5);
    T('1.0000000', '6626035360301835643744155055681183579.72680430492878230246147491362', '-6626035360301835643744155055681183578.7268', 41, 4);
    T('1.0000000000000000014816657759635935574232', '32818928503629136.252295990697644196564215870800902906892', '-32818928503629135.2522959906976', 30, 5);
    T('1.00000000000000000000000000000', '13043617905561942222230054077.21749663178357', '-13043617910000000000000000000', 10, 6);
    T('1.000000000', '468078670327435157159036637213089.68427', '-468078670327435157159036637213088.68427', 91, 2);
    T('1.000000000000000000000364962235111663114736570947300481441647272089259984421671059783280000', '91128496181.82733', '-91128496180.82733', 27, 0);
    T('1.00000000000000000000000000', '248084803156.38225825687792564146883', '-248084803155.38225825687792564146883', 45, 6);
    T('0.999996430603395547879085578374943218533075889', '752798095516784566364107011187.1828743644733064738998075060481075333023', '-752798095516784566364107011186.182877933869910926020721927673164314769224111', 88, 3);
    T('0.9999999864688912316764271500401957202149887487506193902184177755581162789923303216472682', '434.57654697091569924806', '-433.576546984446808016383572849959804279785011251249380609781582224441883721007669678353', 87, 5);
    T('0.999999999999999999999999999982814515318881641819728269066787504001787393727056279440923', '83875621901513231603980702356510524090.18029474594', '-83875621901513231603980702356510524089.18029474594', 66, 2);
    T('1.10455706926852759636854514325898495936688404757420394401166868002', '35601365816445214252.5525881378161872928170', '-35601365816445214251.44803106854765969644845485674101504063311595242579605598833131998', 98, 1);
    T('1.0000000000000000000000000100777014575445669415762036017243568735466409286660372771012100670802724', '899760814583915.493920081239', '-899760814583914.49392008123899999999999999', 42, 6);
    T('0.984114021239868072775897442427054089380451', '5368174736.76894671250154921560010347054', '-5368174735.784832691262', 22, 0);
    T('0.9999999999380984950253', '83972321187501515671178201780968.114223326', '-83972321187501515670000000000000', 19, 1);
    T('0.9999999999999999999', '977684073047763681239701380533.7210843444785440991219674', '-977684073047763681239701380532.7210843445', 40, 6);
    T('1.000182570092457168104845853335311043776', '571449324605826766.57809543188536449255427224746102', '-571449324605826765.5779129', 25, 5);
    T('1.000002907816994100773837', '728233850755.2332313401141380833836258554', '-728233850754.2332284323', 23, 4);
    T('1.0000000000000000000000', '68592074796033717984633919704192.29865862981318592502795535346119', '-68592074796033720000000000000000', 16, 4);
    T('1.000000000000000', '903783422370949770321379953874545365.79210759958265665013787195048', '-903783422370949770321379953874545364.79210759958265665013787195048', 90, 6);
    T('0.999999999999999999999999999999999999717820449394897427433674919219243501563657735037982299', '302299275353390.8981647238924456616512453522112', '-302299275353389.8981647238924456616512453522112000002821795506', 61, 4);
    T('0.9999999999999999999999999999999999984487898167125641473658922', '3585319.7759370122633314703102', '-3585318.77593701', 15, 5);
    T('1.00000000000000', '203506155814552758148.37264782776486', '-203506155814552758147.37264782776486', 87, 2);
    T('1.00000000000000120579185273161226763034133156520200356939876275446105426421498158283412', '4739163351387537956850958064407251558.61807582', '-4739163351387537956850958064407251557.618075819999998794', 55, 6);
    T('1.000000000000000000143533559194819040300297231250823252', '15868546371105409745483539490.0', '-15868546371100000000000000000', 12, 5);
    T('1.00000000000', '54832392.01861789482959289047969611', '-54832391.0186178948295928904797', 31, 4);
    T('0.9999999999998917142347210910448', '7645882914233858938098515181269111721255.2', '-7645882914233858938098515181269111721254.2', 43, 5);
    T('1.000000000019026254756354259135217413250706', '2347731114172144002848808544526073254840.183', '-2347731114172144002850000000000000000000', 21, 4);
    T('1.00000000000000000000', '5823756950779433872739279788630725.76794045546', '-5823756950779433872739279788630724.76794045546', 89, 3);
    T('0.99999969198488396329717135352024457590848674566799393432083784906001565273156394858272936', '454231689016.0', '-454231689015.000000308015116036702828646479755424091513254332006065679162150939984347268436051417271', 99, 0);
    T('0.999983024504112322076111878226563575778225060676062233866162469025247716629315046304590555306536650', '92208.949978', '-92207.9499949754958876779238881217734364242217749393239377661338375309747522833707', 81, 3);
    T('0.999999999999999999988633888832566250132656497801403179018156138677466111241821611', '9129486452431567225984.698928988441611973802466974072821', '-9129486452431567225983.6989289884416119738139', 44, 0);
    T('0.99822719738396736525667460628136940989020697', '531550942791472583639.938880089389348', '-531550942791472583638.94065289200538063474332539371864', 53, 0);
    T('0.99629097248777667878668059904642041828035606099947021', '55813978579621.14619474187723', '-55813978579620.14990377', 22, 5);
    T('1.000000000000000000000', '99056936495226365315892.123168349533765718909', '-99056936495226365315891.123168349533766', 38, 6);
    T('0.99999999999999999999999999999998537924', '8448121780.740448713003666762457063113', '-8448121779.74044871300366676245706311300001462076', 92, 5);
    T('0.99999999988101657646132948979847662022351119638102579488054328360426024854964444951581938034', '26431528769156126902185.5455342185734', '-26431528769156126902184.5455342186923834235386705102015233797764888', 66, 6);
    T('0.999999999999999999999999999999999773647865082540886511424610602720', '94.88', '-90', 1, 4);
    T('1', '28714265844773879322429567222.0', '-28714265844773879322429567221', 51, 3);
    T('1.00000000000000000000000000000002230745989026902237', '5182909643868660181.616326849296578520834', '-5182909643868660180.61632684929657852083399999999997769254010973097763', 74, 1);
    T('0.99999999998849524198691154110204351283244388511160071147040874333277620620', '487171785.353436844076', '-487171784.35343684408750476', 26, 6);
    T('0.99999999999999999995826254', '623264646061209021513606736676.4070464780349804249191774960065', '-623264646061209021513606736675.4070464780349804249609149560065', 76, 4);
    T('0.9999999999999999999999840316000599556090993211977492120887044730644124505912', '3400141293380019543581.846146', '-3400141293380019543580.84614600000000000000001596839994004439090067880225079', 75, 0);
    T('1.00000000000000000000000000000000017890501993570751536367804830406693776445', '7596063323395135740831786963557300.828122807129946719956', '-7596063323395135740831786963557299.82812280712994671995599999999999982109498006429248463', 87, 1);
    T('0.999999999999999999999999981830537099429946248094516383336914915456449167384292448663913', '6160.9833347271963', '-6159.98333472719630000000001', 27, 0);
    T('1.00000000000000000000000000', '140694913008020347351867336278.2604696945968', '-140694913008020347351867336277.2604696945968', 93, 2);
    T('1.00000000000000183062574240181614811643732437563124607364762306403722737412422201569764119662', '315468725452602772046101176967.0405677893315761072465140617', '-315468725452602772046101176966.040567789331574276620771659883851883562675624368753926352376935962', 96, 2);
    T('1.00000000000000020487871739872142801523484304068051994930096447885572730358527340688870270096190', '3223913266.58144343252766526385', '-3223913265.5814434325276650589712826012785719847651569593194800506990355211442727', 80, 4);
    T('0.83864421727143491142189501983147051332126197163408403622694220682570328285455003', '6177991817255640.0', '-6177991800000000', 8, 4);
    T('1.0000000', '52385.2928549712746987395557277', '-52384.2928549712746987395557277', 31, 6);
    T('0.9999999999999999999999969656277', '189067655662557843433650599.27270881530157259', '-189067655670000000000000000', 11, 0);
    T('1.0000000000', '80884.39846', '-80883.39846', 26, 5);
    T('0.99999999999999999999999620', '8104102.905199872665141004926', '-8104101.905199872666', 19, 3);
    T('0.9999999999999999999', '83532645279145516530203.4814629819050362463403556437845', '-83530000000000000000000', 4, 5);
    T('1.000', '895694428428700219865817784521974121281.8617', '-895694428428700219865817784521974121280.8617', 74, 2);
    T('1.0000000000000000000000000000000000000229021152905485877265916049147377596', '46587749069373859988241253.489', '-46587749069373859988241252.489', 63, 0);
    T('1.00000000000000000000000000003315723137560736009063125977104295', '125786914090542197734.1243843248365770', '-125786914090542197733.124384324836577', 38, 4);
    T('0.99999999999999999999999880956718792058', '7160823998851.8329358', '-7160823998850.8329358000001', 26, 0);
    T('1.0000000000000000000000000', '168185946163980552320.267147742897536930695190349307194331', '-168185946163980552319.267147742897537', 37, 3);
    T('1.013368631237074269967629841470163611', '705721440565168596203735309275.2226143048086127376737122109829328049', '-705721440565168596203735309274.2092456735715384677060823695127691939', 100, 5);
    T('0.9999999999999999999999895953786771511342299983348190561873466109129549287631216974725051767872153303', '564417756540611857691.339744659093279108641571889', '-564417756540611857690.33974465909327910864158229362132284886577000166518094382', 77, 3);
    T('0.99999999999999999998540837496861607141739434483624886985909476644326279641174', '702032398191198338390179.03234814268263959712', '-702032398191198338390178.03234814268263959713459162503138392858260566', 68, 3);
    T('0.99999999997144379431778592551299579284817269428143146335310624910486', '5744510660347753705135441208927833216328.435064197579250286979352170', '-5744510660347753705135441208927833216327.435064197607806492661566244487004', 73, 1);
    T('0.9999999999999999999999999999999999999998703780041291982139945219878674624', '5520834448.5302613321828479018112575569', '-5520834447.5302613321828479018112575569000000000001296219958708017860054780121325376', 90, 0);
    T('1.00000000000000000021756877520948475176291147979521588259993838886521586727381224303946367', '8458303053559696574120.570523', '-8458303053559696574100', 20, 2);
    T('1.0000000000000000000', '5966.1758018', '-5965.1758018', 58, 1);

    T('606.254575304', '10913610393350.2606', '-10913610392744.006024696', 37, 1);
    T('3637644346217992.6517548755488192', '61.23387301712754337', '3637644346217931.41788185842', 27, 6);
    T('8562.3', '3857989719151989094.28436267039053249', '-3858000000000000000', 4, 5);
    T('42885995.8922914272', '706475.796938411', '42170000', 4, 1);
    T('7744926264.9828979895736', '23.42189', '7744926241.561008', 16, 2);
    T('2652458869.8552735', '275855874.003329237660489', '2376602995.851944262339511', 30, 4);
    T('982009515496588214.2961068036', '6139.9730427718831', '982009515496600000', 13, 6);
    T('674668148428.007557069664', '9730741.87736', '674658417686.1302', 16, 6);
    T('724155.70665100', '7644638.16160612893825643332', '-6920482.45495512893825643332', 29, 2);
    T('7489512119755281.0', '5696661.38689', '7489512114058619.61311', 27, 2);
    T('34021417679098405263.210121640063', '469015725494952.249997154054', '34020948663372910310.9601245', 27, 6);
    T('8104166310518.2057288', '331717.89', '8104165978800.3157288', 38, 4);
    T('685.46727959004970110362', '69830719.4666357936145496499', '-69830033.9993562036', 18, 6);
    T('91409621890423.735243', '583761545405875499.298205826415', '-583670135783985075.562962826415', 40, 3);
    T('63.62', '816473804427740.32776832822474', '-816473804427676.70776832822474', 29, 3);
    T('93913790211035413955.0', '37517895256.691410825590', '93913790173517518700', 19, 4);
    T('8171950.0', '7916992.81283069266914681297', '254957.18716930733085318703', 33, 2);
    T('70570310990471372.535740', '306380460.13915815763162', '70570310684090912.396581842368', 29, 6);
    T('904002719243514578.199046601', '392302713454.30007186959883636', '904002326940802000', 15, 2);
    T('61222626759880119720.6', '38.1763782264368', '61222626759880119682.4236217735632', 40, 5);
    T('106992249842.454676', '9783343723646590.895277', '-9783236731396748.44', 18, 5);
    T('315594693.3230', '601427787.397048404134', '-285833094.074048404134', 37, 5);
    T('2624664081780.107816961929532', '638739152166230.6570284141825715', '-636114488084450.5492114522530395', 34, 3);
    T('850761485.0159962973', '16557587439532.009816893', '-16556736678000', 11, 4);
    T('29143073887560442.2732826582829', '817114655795.2399384597929170', '29142256772904650', 16, 0);
    T('13683256304217.3', '79044625035670693.5671293', '-79030941779366476', 17, 4);
    T('6052717165.1446', '65455223170949076520.8060582718286379316', '-65455223164896300000', 15, 1);
    T('54131.622129937389021', '998107.4', '-943975.7778700627', 16, 0);
    T('4134679559878377.722073883', '77947153111703354860.05228739542293', '-80000000000000000000', 1, 3);
    T('5955452.7441572243480999', '916606845979019189.694793932', '-916606845973063000', 15, 1);
    T('1719979601587.803845', '582.660259024590', '1719979601005.14358597541', 33, 6);
    T('179.7986834569433903', '5.3035430636', '174.4951403933433903', 34, 4);
    T('2571182.9104', '5112317485037139357.0326821757855538364', '-5112317485034568174.12228217578555383', 36, 1);
    T('33777941893928234.1403399701827010', '4007790318278125722.91052580512126369', '-3000000000000000000', 1, 1);
    T('1608.6048924999255577935', '272293.285196', '-270680', 5, 4);
    T('53611325447739.249', '633369797029.0', '52977955650710.249', 20, 5);
    T('710416580769.486', '33.64813473834309', '710416580735.8378652', 19, 3);
    T('9611898517968409.49495', '3.912', '9611898517968405.583', 19, 0);
    T('28302362177443.355900', '6595.574858694980', '28302362170847.78104130502', 30, 2);
    T('61066.29222627342560448', '12924.77619098', '48150', 4, 2);
    T('132.65466', '712167.8700932215292641', '-712035.2', 7, 4);
    T('688767986059678709.380854', '32846480.81710', '688767986026832000', 15, 6);
    T('1768.11258632743', '33118961361.6262', '-33118960000', 7, 4);
    T('3648851903469184.5520025290', '616248879.263262', '3648800000000000', 5, 3);
    T('890038068185855.960980', '2709407759.1604115152745810269', '890035358778096.8005684847254189731', 38, 4);
    T('102.0', '1.670386774', '100.329613226', 38, 4);
    T('188638666620373.0', '264.8829877746255031', '188640000000000', 5, 2);
    T('63940.75794950117047', '31279.29426406007672410', '32661', 5, 4);
    T('79999.339185864', '77281398922.8', '-77281318923.460814136', 29, 2);
    T('2043.1155258', '8703901.5', '-8701858.3844742', 20, 2);
    T('6409343803262.388', '823497316.0', '6408520305946.388', 38, 1);
    T('78753063306376138.506170034261398', '394.626015216966077', '78753063306375743.88015481729533', 31, 2);
    T('3396217318606541981.25623355', '8090252540252652303.5496813397654', '-4694035221646110320', 18, 4);
    T('572125232723470402.673', '450102582792560243.4928609852', '122000000000000000', 4, 6);
    T('73.1469871709778', '91.10583849', '-17.958851319022', 14, 5);
    T('35811.99', '6730744506562.519', '-6000000000000', 1, 1);
    T('12948569823926516.1', '6032050833605576786.28750720978465442', '-6019102263781650270.18751', 24, 4);
    T('73.460260127414', '5580002381112368469.58572385874194', '-5580002381112368396.125464', 25, 0);
    T('36533488928501951.648348833', '34832817263.64', '36533454095684688.008348833', 34, 4);
    T('941537063116169.19', '294.64269', '941537063115874.5', 16, 5);
    T('5436517709034324.73139', '2494962.7', '5436517706539362.03139', 36, 2);
    T('3055391195233077250.6394833837101256009', '6427707361341089.0024974560131565626', '3000000000000000000', 1, 6);
    T('29074997967.4033077914', '566332182964.6602808', '-537250000000', 5, 1);
    T('685.6230300327385', '8297.123231678604', '-7611.5003', 8, 0);
    T('9529.144771387', '8895131556.883', '-8895122027.738228613', 34, 1);
    T('265633925199908208.42314005298861', '565128025.144', '265633924634780183.27915', 23, 0);
    T('1376622519151175.50445', '75251508328.9053133011113', '1376547267642846.5991366988887', 40, 1);
    T('3338928425.7256095268', '44744429118812497.041340991400', '-44744425780000000', 10, 5);
    T('6505421540677.424332952118826', '5946601.3849316529', '6505415594076.0394', 17, 3);
    T('2967456.687326291008', '53066388425896878.90368577441', '-53066390000000000', 7, 6);
    T('5688144351213475003.3732447400317', '189.14292', '6000000000000000000', 1, 6);
    T('3.5906', '8268616900672.008806449513', '-8268616900668.418206449513', 31, 1);
    T('4073.254750128669355', '6703655796774.76684', '-6703655792701.512089871330645', 33, 3);
    T('35878985311.78005477', '1662287231.82062416579', '34216698079.95943060421', 29, 5);
    T('5518886767675023.49851818350477084', '7785.94593827371287', '5518886767667000', 13, 6);
    T('94524.665979427', '255044965477.7944', '-255044870953.128420573', 25, 3);
    T('309493.41431718', '4268094596992250.4638525', '-4268094596680000', 12, 2);
    T('85445452.508307930204998', '2139434062.15737168420518', '-2053988609.649063754000182', 35, 5);
    T('65890696226861301.0', '247319133365.0', '65890448907727900', 15, 6);
    T('254299389.31661528', '686232161186112224.325327639', '-686232160931812835.008712359', 33, 0);
    T('8746064.3941922553186', '30475233664960.27116225', '-30475300000000', 6, 0);
    T('74633171370781.84', '372860604.06652842196', '74632798510177.773', 17, 5);
    T('39171681566636341.16322920929', '53478965285906.1940', '39118202601350434.9693', 21, 2);
    T('3283934516151.85893', '3261449.6074498525584', '3283931254702.2514801475', 23, 2);
    T('48.9032428880969972694', '84157069090727466355.36155969082160856867', '-84157069090727400000', 15, 2);
    T('482122041793574.2584', '8181.7124361033119916779', '482122041785392.5459638966880083221', 40, 6);
    T('371988477.0', '4.8', '371988472.2', 26, 4);
    T('4204413431.0', '6561257418014417976.664233904856', '-6561257413810004545.6642', 23, 2);
    T('1.869208540124719', '3336691047.396182620547109148', '-3336691045.526974080422390148', 39, 2);
    T('311487949263.0', '27436.0', '311487921827', 26, 5);
    T('39276275576766512058.65296', '23903.0718', '40000000000000000000', 1, 4);
    T('5799735376769987608.89', '8863656.680309484238287037', '5800000000000000000', 2, 2);
    T('2916405979959.0', '8.51446', '2916405979950.48554', 21, 2);
    T('2000967495536332646.9641612344756769010', '5107990.7966488733008', '2000967495531224656.1675123611748769', 36, 1);
    T('586206.365623013', '586340268113160595.8', '-586340268112574389.434376987', 33, 3);
    T('3158566821078150.0', '85805956.1426886022039230', '3158566735272193.857311397796077', 40, 5);
    T('276540212645.543615500791', '79812772471.272395863479', '196727440174.271219637312', 28, 4);
    T('72604862414410.6855', '2767572833795397.587962782761360', '-2694967971380986.90246278276136', 30, 3);
    T('7284755405592556.585332786034524', '658664349.53356', '7284754746928207.051773', 22, 0);
    T('20581860035495.3667819018', '3167954.71487188838', '20581856867540.65191001', 22, 6);
    T('71580.0', '34066322279572646.5682', '-34066322279501070', 16, 3);
    T('34127000653558353.220', '4145916073059525969.34', '-4111789072405967616.12', 24, 0);
    T('1.294110228213', '9572772006158671.3194638867183841', '-9572800000000000', 5, 0);
    T('18118401318728.4944570040036468899', '9873350772170042692.9790644859764', '-9873332700000000000', 8, 4);
    T('6.06777533', '761826794.5384800063', '-761826788.47070468', 17, 5);
    T('681752924374665743.0', '86.5276', '681752924374665656.4724', 27, 2);
    T('2715634607636096.7609', '41192413317.0130568175', '2715593415222779.7478431825', 32, 3);
    T('363389273527469204.0', '819362054125.1', '363388454165415078.9', 33, 1);
    T('8195451.716732', '2408.6009578', '8190000', 3, 1);
    T('8331877169084.914244790304132844', '94351869774433174.7', '-94343537897264089.785755209695867156', 38, 2);
    T('2718.764830436903', '43343292310.1323', '-43343289591.367469563097', 35, 0);
    T('97790332917038.0', '85354649952302454335.4912443823058', '-85354552161969537297.492', 23, 0);
    T('12098621579.2242273303', '9646.4286506991194', '12098611932.7955766311806', 38, 2);
    T('9854538582584.5049980836200596770', '9580773.26531373', '9854529001811.23968435362005968', 30, 6);
    T('648515804431666.600171197170565', '8106953537682867.96965671411184488', '-7458437733000000', 10, 4);
    T('155356608096526.523724308', '73043.232913067221', '155356608023500', 13, 0);
    T('7147.0', '7485.873033714645507', '-338.873033714645507', 39, 3);
    T('8278932.37164819', '7848984867306403.82464895848923', '-7800000000000000', 2, 2);
    T('4409322.136711824513', '4232.09453731141905747910', '4405090.0421745130939425209', 31, 4);
    T('9978.0951413924366012', '20422291737901004671.0', '-20422290000000000000', 7, 5);
    T('253513040.7759', '1612096.6523123703887499', '251900944.12359', 14, 4);
    T('3498372367282093480.7393576756578', '5593.13354718', '3498372367282087887.60581049566', 30, 0);
    T('14.931992', '66638165.0402436358', '-66638150.1082516358', 19, 4);
    T('9201.03850088905238568807', '833683647771784.54797343887', '-833683647762583.509473', 21, 0);
    T('34326320413440959.98142228583054270', '4944183.728789425', '34326320400000000', 9, 6);
    T('4707871169.333', '796993155222912.97081', '-700000000000000', 1, 2);
    T('1756.4341533', '9.9219972723084312491', '1746.5122', 8, 2);
    T('30352560855041876.0', '26814935499185695364.61', '-26784582938300000000', 12, 5);
    T('4116192598.3856110051028', '7130927.60', '4109061670.785611', 16, 4);
    T('33059122.5317491499591', '83116760990555162.0', '-83116760957496039.4682508500409', 31, 6);
    T('1113.784156826813617402', '471337101398976.0248313066305725', '-471337101397862.24067447981696', 29, 5);
    T('10526617358273065110.29', '887505505721.904710613030359', '10526616400000000000', 9, 1);
    T('846472571868323.12928049', '4233.177703823820196240', '846472571864089.9515766661798', 28, 4);
    T('45358028055983973939.91670060099257792704', '605711.37', '45358029000000000000', 8, 0);
    T('27009004.514606778', '9931279311584.625541', '-9931252302580.110934222', 29, 3);
    T('7.071838891822', '8378.3405827502968', '-8380', 3, 0);
    T('78270818.4229056949312838869', '74020604043839894.596806', '-74020603965569076.1739003', 24, 1);
    T('391876330195413.78016', '43754182.345314', '391876286441231.434846', 39, 6);
    T('4356704.8897554536679839', '3495558.17903653026064', '861146.7107189234073439', 31, 0);
    T('19.349180762', '516953012716.24950144446', '-516953012696.90032068246', 32, 4);
    T('7073583.97077097503491311', '9527.6170', '7064056.3538', 11, 4);
    T('842244175006.81', '16227827487006.9147633036', '-15385583312000', 14, 1);
    T('24.0', '7336608322001976419.32302088', '-7336608322001976395.3230209', 26, 5);
    T('9.8564449290940041', '8978837574149037522.31155410', '-8979000000000000000', 4, 3);
    T('4.576721818', '459742227106132188.06055111589065107', '-459742227106132183.483829298', 27, 3);
    T('6111164725018.16368829130836', '80.0103682199', '6000000000000', 1, 6);
    T('702711749568774.65154', '217514.60139537', '702711749351260.05014463', 37, 1);
    T('4.332653', '6.435039966127344', '-2.1', 3, 4);
    T('680.6789659329', '34029900096.5', '-34029899415.821', 15, 5);
    T('891848124150105.0894717570460054', '376408493090.77', '891471715657014.32', 17, 0);
    T('46.5830305261', '83.135682960368312', '-36.552652434268312', 35, 2);
    T('130814327007019.3658785746649', '86.3', '130814327006933.0658785746649', 34, 3);
    T('1570560658033732534.614590616641', '389065232275770.24', '1570171592801456764.374590616641', 33, 2);
    T('14971.669356895657', '8906472037023014822.6131297217', '-8906472037020000000', 12, 5);
    T('4553702.5616771', '8157910968.9479149280746', '-8153357266.386237828', 19, 1);
    T('3823437.9', '6708438697970111424.567306735649', '-6710000000000000000', 3, 6);
    T('104797806.75', '98.7347221852', '104797708.02', 11, 4);
    T('563901.05596', '469066002805.122609', '-469065438904.066649', 20, 0);
    T('90981023855.9032458', '939585287335292652.582249606591838121', '-939585196354268796.679003806591838121', 36, 1);
    T('5889879050.10387', '719.00392782801', '5889878300', 8, 4);
    T('298779689561.6987363', '835678160393.440791', '-536898470831.7420547', 20, 0);
    T('3.42953872865273475804', '7467471997.57993', '-7467472000', 9, 3);
    T('589520323415531.6618', '756016566043390546.2307772144683587067', '-755427045719975014.56897721446836', 32, 0);
    T('857791142882.0', '6680340499.432859594695', '851110802300', 10, 1);
    T('12291.4960572589495', '182064202.5', '-182051911.0039427410505', 27, 4);
    T('60397.29', '292886242126.771548181642180', '-292886181729.48154818164218', 31, 2);
    T('594260193958.68887562089623059361', '85252057669518.304176988474064', '-84657797475559.6153013675778334063', 33, 2);
    T('9131472.73600', '914725432884331.5710081220478', '-914725423752858.8350081220478', 40, 5);
    T('179697960218.5446584', '2825955673306.653488350764117', '-2646257700000', 8, 5);
    T('4215.662831', '8.71539758595302897', '4206.94743341404697103', 25, 0);
    T('5145.3978621', '62796525532.122049355412', '-62796520386.724187255412', 32, 3);
    T('1404983.793', '72636278375833801.9', '-72636279000000000', 8, 0);
    T('7007745886193544.650', '87141632.058914308832938760', '7007745799051912.59108569', 24, 6);
    T('69787711549650390.519695469114023', '9383063839.8729475100233', '70000000000000000', 1, 5);
    T('1913028612183966.09341866687695891619', '415199081926401.368643718788808', '1497830000000000', 6, 6);
    T('88149773402803893.54468656305713', '315450422994692371.451866', '-227300649591888477.90717943694287', 40, 6);
    T('3086569693.5', '1492568410232474.9488303107465378934', '-1492565323662781.4488303107465378934', 37, 0);
    T('170.0343924287', '85514537881740899.00363441747', '-85514537881740728.96924198877', 31, 4);
    T('54058855231846.51538336', '6128118187513.238059', '47930737044333.27732436', 37, 6);
    T('84.4387378817', '9165905.5445285803132065', '-9165821.1057906986132065', 26, 0);
    T('77542740871.53285689242325', '71829637875331906.017364984716677325', '-71829560332591034.484508092293', 29, 1);
    T('50359725.724249299117479788', '292428834.94080856001086930', '-242069109.216559260893389512', 40, 4);
    T('6843951.184', '4035.22', '6839915.964', 34, 5);
    T('3003858.3085183936398', '6725851574.118463198995466', '-6722847715.8099449', 17, 0);
    T('10783713.05', '6384013985064.7391546', '-6384003201351.6891546', 20, 4);
    T('3162.0288687941', '1510924179413.2789052874727894', '-1511000000000', 4, 6);
    T('12354899357783296.96', '766114072714.278', '12354100000000000', 6, 4);
    T('3601141474.03064612522746643', '53575025812903872540.778242133359', '-53575025809302731067', 20, 5);
    T('82790.29356650348585', '206365358512.459022867347232069', '-206370000000', 5, 5);
    T('43732778.30', '6.0972', '43733000', 5, 6);
    T('0.0', '8019.9368621260784', '-8019.9368621260784', 25, 2);
    T('3.57652012338362388', '11586263489.32544024469840744', '-11586263485.74892012131478356', 32, 3);
    T('888740252471.9454997613427466', '5823.15717335599978843', '880000000000', 2, 3);
    T('833816959543875937.3395022716962426', '39.92886973853913', '833816959543875897.4107', 22, 0);
    T('67.48818', '4635859874166.37613031190467439', '-4635860000000', 7, 3);
    T('82485949622.29234327', '5.481324864', '82485949616.811018406', 40, 2);
    T('745644707.1', '85877691701777418.0909848571696012', '-85877690956132710.9909848571696012', 40, 3);
    T('35.5522439', '8930051195900421.66322215013', '-8930051195900386.11098', 21, 6);
    T('942624675.822828569', '41.429520', '942624634.393308569', 23, 1);
    T('14819688999426512.06416567687152973218', '54.0', '14900000000000000', 3, 2);
    T('81931491501496086171.0', '2.6540009', '81931491501496086168.3459991', 30, 6);
    T('85003.289368659941366857', '680.3078649372', '84322.981503722741366857', 35, 2);
    T('671318.25707841', '82887949056785243.80148941', '-82888000000000000', 5, 3);
    T('69591422716401580893.58', '480724342550.53796522908104613733', '69591422235700000000', 12, 0);
    T('1315515.0', '92811909.19727663489031', '-91496394.1972767', 15, 0);
    T('13193221107217343.392909', '4731813.340', '13193221100000000', 10, 1);
    T('5.6658022', '109150689431698.169502', '-109150689431692.5036998', 30, 3);
    T('551.5960988456654', '9766671100600.0686748669472291', '-9766671100000', 8, 1);
    T('44292293094.228544750', '9260496400687152.799423177', '-9300000000000000', 2, 6);
    T('89366851966595321.1748075557976280472', '6025464855374.9947624257', '89360826501739946.1800451301', 27, 6);
    T('6743185101.4680', '972.3081244647701', '6743184129.1598755352299', 40, 6);
    T('3122360.689313', '0.0664', '3122360.622913', 29, 3);
    T('57631621187.999', '11281.3168337674839688', '57631609906.6821662325160312', 27, 0);
    T('558778421948394214.0', '77346004621120.1526160', '558701075943773100', 16, 5);
    T('19.56142227789823686', '41932570057183691267.440106340', '-41932570057183691247.87868406210176314', 40, 6);
    T('53909647700384.8149686913693', '25450362.37866838272112662040', '53909622250022.437', 17, 0);
    T('6234240734297816.32528', '641.64031159203', '6234240734297174.685', 20, 2);
    T('29482.46676979501808278052', '2993072493316.73190', '-2993072463834.265130204981', 25, 2);
    T('991454337548146.01180601392', '45242.962401812393949500', '991454337502903.049404202', 24, 4);
    T('82114097517126406834.202756725659892', '4248023535546.66911122602', '82114093269102871288', 20, 5);
    T('1.58737261819809725359', '405893625.0', '-405893623.41', 11, 6);
    T('7213165519331.84868599248789030', '622086.979219398', '7213164897244.8694665944878903', 34, 6);
    T('75.1583', '7449587423.212355773087072', '-7449587348.054055773087072', 35, 2);
    T('6156901786095.167', '939889013614347522.91665714557', '-939882856712561427.74965714557', 39, 0);
    T('51709300141707136909.215506430741742850', '2428.5127', '51709300141707134480', 19, 6);
    T('937480901.6973', '31006.51105554', '937449895.186244', 15, 5);
    T('93905583.5924311435776341120', '8440174872408463651.54074575019439654', '-8440174872314558067.9483146', 27, 2);
    T('17661231225990774959.71255923561035090', '76.8921365927677941', '17661231225990774882.8204226428', 30, 4);
    T('45675964984950434.8974988', '3022353312.4858109', '45675961962597122.4', 18, 5);
    T('51.887758429', '3563586683.4420582', '-3563586631.5542997', 17, 1);
    T('70643247.124365815639833', '8904416815611079834.2673396174', '-8904416815540436587.142974', 25, 3);
    T('309113404030.366138', '8922800143.5113083469048302', '300190600000', 8, 1);
    T('572609389.0573094224', '8738144846503525351.941603864156784', '-8738144845930915962.88429445', 27, 3);
    T('3343561.9855693716720183464', '21722216066.776118979645129', '-21718900000', 6, 4);
    T('6273.549', '99.8481026641427834', '6173.7008973358572166', 39, 6);
    T('35653905635.0', '529092.0', '35653376543', 33, 0);
    T('49752.91', '3862052.0565121294', '-3812299.14', 9, 1);
    T('3791.9402046601563311', '89.53', '3702.4102046601563311', 26, 0);
    T('36699126285886.6649934579220', '36.3469236932641680800', '36699126285850.31806976465783192', 39, 5);
    T('4.59676826485173', '213000305.43', '-213000300.83323173514827', 40, 1);
    T('625227.668', '1272062393276739.303162166', '-1272062392651511.635162166', 35, 1);
    T('55782429435215.481586499268855721', '72241618.454624', '55782357200000', 9, 2);
    T('19624.89', '385093937680676.428151135548', '-385093937661051.538151135548', 28, 6);
    T('9.03510923532', '8608.1854166', '-8599.15030736468', 23, 1);
    T('205015530.79871211575860112246', '791697745034358897.82119551900', '-791697744829343367', 18, 4);
    T('2816480449711631.4161', '943007809.3927928970145899657', '2816479506703822.02', 18, 5);
    T('364230051257482014.285438062153915', '9878644773483679911.3', '-9514414722200000000', 11, 6);
    T('547.118413622299409', '206848039918577.5', '-206848039918030.38159', 20, 6);
    T('55732912136577841.0497731324660074923', '3383.8434732', '55732912136574457.2062999324660074', 33, 1);
    T('80044829275440496.97552977522', '3161578834.946', '80044826114000000', 11, 2);
    T('164911416498812.07', '1069588315825771.440', '-904676899326959.37', 26, 4);
    T('362241587347.462671813', '96447054661185.0598153993144', '-96084813073837.59714358632', 25, 3);
    T('2028240556.75434606639741074', '3346714245918.0860095002422341', '-3344686005361.331663', 19, 1);
    T('2091.06395421650898909566', '22253765195121336632.0', '-22253765195121334000', 17, 1);
    T('10734042756258224790.9392463561916', '5233439604995693.38726376916', '10728809316653229097.5519825870316', 37, 3);
    T('32.6764124048368460', '11313.068644281528', '-11280.392231876691154', 33, 2);
    T('60059048676283.4912931549689', '6673675.040892', '60059042002608.4504011', 21, 3);
    T('81222.50750', '6511390415653840.0', '-6511390415572617.492', 19, 5);
    T('5869909.73', '45353301.46600360963515', '-39483391.73600360963515', 35, 4);
    T('604744388065613.57719551133', '9019617081.676', '604735368448531.90119551133', 39, 4);
    T('9372.946', '78427814.8789863', '-78418441.9329863', 28, 1);
    T('769142413755335.516558144167', '48658273157761.4933568', '721000000000000', 3, 2);
    T('51630417513168.6954158', '6336390096272004508.47069528893800544713', '-6336338465854491339.77527948893800544713', 40, 0);
    T('3429950675.131503198803308', '46989223091877539517.583188065866554', '-46989000000000000000', 5, 2);
    T('7507523.85932692765', '979717350747796.116', '-979717343240272.25667307235', 28, 6);
    T('402876250198700.7578970688163933', '36718441409427094395.82182844996734798', '-36718038533180000000', 13, 0);
    T('55592198626425.2097084432490', '250018567774083150.1', '-249962975575456725', 18, 3);
    T('43743972719626444.33061529', '9457929815953975964.930282321', '-9414185843234350000', 16, 3);
    T('9396089126280.94537914435158190', '95656994737964792009.8390', '-95656985341875665728.8936208556484181', 40, 5);
    T('106284670.07', '3915.9164287', '106000000', 3, 6);
    T('626106631429889599.591315', '68624953835335.0', '626038006476054270', 17, 0);
    T('72267965.29425904814375', '2641598013393.3671844071103', '-2641525745428.072925359', 23, 5);
    T('652927968056040.4527674544422380', '50893.0932661449936673', '652927968005147.35950131', 23, 5);
    T('23176283336042.0516232222', '3.8766221', '23176283336000', 11, 3);
    T('141.3512523883786374', '4589.0', '-4447.6487476116213626', 36, 4);
    T('5712615407874.74474129967', '4476034154913.551880724929061', '1236581252961.1928605747', 23, 3);
    T('312272274185.15163663017', '9411885523.4657465466151', '300000000000', 1, 1);
    T('616088577563.4169709383921878481', '13745364436.6966638322130630', '602344000000', 6, 0);
    T('398221654369141038.64', '7495116973382235679.0', '-7096895319020000000', 12, 0);
    T('5345816.24747728532543100432', '9441155517597.46715505366710', '-9441150172000', 10, 5);
    T('350.929470269', '27941129805.0', '-27941129454.070529731', 20, 3);
    T('874735782092.38232', '5133399417753302443.80292', '-5133398543017520351.4206', 30, 2);
    T('171554375.151', '266.91555385946662', '171554100', 8, 1);
    T('5538.016819342', '70312536.429596729466', '-80000000', 1, 0);
    T('208306534585815.6409', '232917865972154065.22251018776270613', '-200000000000000000', 1, 1);
    T('838217.56', '420.7673349338963348', '837796.7926650661036652', 37, 3);
    T('5015809168272398.86724337', '8953412089614159191.0', '-8948396280445886792.13275663', 31, 2);
    T('3229335803432629.100552040557986657', '3903057.071095726', '3229335790000000', 9, 3);
    T('472878200239.17975', '4.67866540162537633792', '472878200234.50108459837462366208', 32, 6);
    T('9463981866326.4', '57369.794754428153998', '9463981808956.60524557185', 24, 2);
    T('393254775.85367089', '34245.8867544536', '393220530', 8, 2);
    T('7329611265.811963351985989', '389873.823', '7329221400', 8, 4);
    T('8696.892394', '92243784.62322080147275403', '-92235087.73082680147275403', 40, 0);
    T('6.5959517518353452057', '693834.27200737', '-693827.6760556181646547943', 32, 1);
    T('4974463.763379', '6.0', '4974457.763379', 16, 4);
    T('4847555508.153085155971', '4596.325615166554883693', '4847550900', 8, 6);
    T('5321.16762420631628987', '22791383668374395.87419011', '-22791383668369074.7065659036837102', 33, 0);
    T('1658703608765768.83062415606', '679364355725915000.86634165085', '-677705652117149232.03571749479', 33, 3);
    T('539935289627.0917721998012989699', '204505494.7', '539730784132.3917722', 21, 5);
    T('82039201427284.0', '19236136581651517.6717853633393', '-19154097380224000', 14, 2);
    T('394920300.0', '3407850704.91714', '-3012930404.91714', 15, 4);
    T('3.660639066283234', '42324005804867.0044536901627', '-42324005804863.34381462388', 25, 5);
    T('5.0058359578191', '818246013729.56399770497276807', '-820000000000', 2, 3);
    T('886648996.2201547', '6362.3350243747', '886643000', 6, 4);
    T('8.89619211585661', '993458.99', '-993450.09380788414339', 26, 0);
    T('5.215448', '7360185.2', '-7360179.9', 8, 1);
    T('379719732445718402.745191890', '257624149112401.458137010092', '379462108296606001.287054879908', 40, 1);
    T('461655.2992884', '6102177243747.06', '-6102176782091.7607116', 39, 0);
    T('417780.2626853580665', '9031992514188.961525833168380432', '-9031992096408.6988404', 20, 1);
    T('447941468285475980.4757895751394268', '398.04069245', '447941468285475582.4350971252', 28, 2);
    T('35085006802138830709.318869', '7589344523755175213.62654152', '27495662278383655495.69232748', 34, 3);
    T('39703729146569556.79450848401356', '3985370.129366101', '39703729142584186.66514238301356', 38, 1);
    T('233326231648.477346', '95767759.5106905720122', '233300000000', 4, 0);
    T('7.151', '6.0', '1', 1, 3);
    T('4752243423.94820886975209', '457.0', '4752242966.94821', 15, 5);
    T('11016540.9', '823772453.6119232464999', '-812755912.7119232464999', 29, 2);
    T('54277787870653.619919349436', '88299514418870.761290', '-34021726548217.141370650564', 33, 2);
    T('87.5489223548291', '27325208504.46780', '-27325208417', 12, 3);
    T('3682611439968930264.0', '76.0380', '3682611439968930187.962', 40, 5);
    T('2438334962416385698.0', '6542.5', '2438334962416379155.5', 21, 6);
    T('69120425457719.1907495799409029359', '51071503323694666036.22463199974232', '-51071434203269208317.033882419801417064', 38, 2);
    T('1366932.5567918', '805922103363.0', '-805920736430.4432082', 25, 6);
    T('38094090199207026280.92085313880448405', '6805.30089659648369', '38090000000000000000', 4, 3);
    T('3912114067711.799666', '321.8', '3912114067389.999666', 37, 0);
    T('2445391267.348825', '96876269018.1435', '-94430877750.794675', 25, 6);
    T('83854709844471471245.576571', '42211627172377558188.7922624425', '41643082700000000000', 9, 5);
    T('929067210299412906.0', '74170267926.29959377819', '929067000000000000', 6, 1);
    T('14528185036152.554386919809', '32885692390678427.78319', '-32871164205642275.228803080191', 31, 1);
    T('535.92769914560227346', '9610167318627070976.769964096960645', '-9610167318627070440.84', 21, 1);
    T('441025193.74417795', '3028527402299.64101', '-3028086377105.89683205', 26, 5);
    T('893718.05013232134', '614335.410573390705957', '279382.63955893', 14, 5);
    T('61519218826154010.29468', '42.5618602901094', '61519218826153968', 17, 5);
    T('6299586111562.7', '9742355304199.6555', '-3442769192636.9555', 22, 1);
    T('72297125.4384377299306', '997.50067870490', '72296127.9377590250306', 36, 3);
    T('840200046654.5909', '71572.34921980', '840199975082.2416802', 22, 3);
    T('33.78613885704761703358', '69303732594379610229.38162519795', '-69303732594379610195.59548634', 29, 1);
    T('536454353069.87140089', '26777486555107833.6936318190679', '-26776950100754763', 17, 2);
    T('33941979304963323054.48405081614', '9081.595', '33941979304963313900', 18, 3);
    T('357539266486704927.36616748580115916779', '131887345441547025.40335276323', '225651921000000000', 10, 6);
    T('9473028021357006.301625', '9228630.7276', '9473028012128375.574025', 29, 5);
    T('32585021.3571438200385052', '26750182.6876948964824', '5834838.669449', 13, 6);
    T('80228649.6276518150362', '78102969946399.0602', '-78102889717749.43255', 19, 3);
    T('19320427014729636186.037788', '2588704327790.12586988853', '19000000000000000000', 2, 6);
    T('97.56886', '5567195180915585462.52887601887912306270', '-5568000000000000000', 4, 0);
    T('5647420681776460236.0', '971.1394744476164', '5647420681776459264.86052555238', 30, 1);
    T('54089176756135230.5733', '7322.407893018606631', '54089176756128000', 15, 0);
    T('922697421287971.16', '70067052844.48408101', '922627300000000', 7, 3);
    T('3.4965332667109404', '3759051.69621947928', '-3759048.2', 8, 5);
    T('2164954911.92733371229790276056', '4911639.877526607', '2160043272.04980710529790276056', 37, 1);
    T('1711726104915570648.3539929565343708', '57560450064069.603912474337281', '1712000000000000000', 4, 0);
    T('854352955005.512016220546', '3.64902', '854352955001.862996220546', 35, 0);
    T('33819026059930.57', '95727122347689.113488178483541395', '-61908096300000', 9, 6);
    T('6097497101245798754.7425814771026', '8875541.8380373420', '6097497101236923212.9045441351', 30, 3);
    T('716717152267010155.83525122', '1.8706469', '716717152267010150', 17, 3);
    T('1439332636962612.685627', '74288024186472.0621332706', '1365044612776140.62349372', 24, 3);
    T('450805631017.84967892605484011899', '5836938256658306360.0376912', '-5836938000000000000', 7, 3);
    T('36355.85284449645657', '6265950335445114489.269', '-6265950335445078000', 16, 4);
    T('7192498485496980.263998164966093951', '1.74301', '7190000000000000', 3, 3);
    T('75.0247581316761909764', '14004.364690968', '-13929.3399328363238090236', 36, 4);
    T('7437176862931.0', '326013075189134834.297873884345513', '-326005638012000000', 12, 4);
    T('76625497250096.65550147749583607', '7539438525453427.863353125082809', '-7462813028203331.20785', 21, 1);
    T('5664266892385804687.4350674', '1034593664750.00135145013', '5664265858000000000', 10, 4);
    T('8353749.9051154392829499156', '3109772931480004.7906', '-3109772923126254.88548456072', 27, 3);
    T('178.1420610290526661316', '9844632826.4695875793952', '-9844632648.3275265503425338684', 31, 2);
    T('79.0131117259', '596.29299909417822802', '-517.2798874', 10, 6);
    T('14537710831681894133.5941176972646899163', '258566117076.80368332161', '10000000000000000000', 1, 5);
    T('2692.3077373', '79342303898.05142119514019', '-79342301205.74368389514019', 30, 4);
    T('86.5549', '1797967.187150123637637', '-1797880.632250123637637', 22, 5);
    T('8320.97197879838', '83835016722506500.01927235467818', '-83835016722498000', 14, 2);
    T('983304550050.00533', '67543163.028775322014614', '983237006886.976554678', 22, 6);
    T('69174771669454.7369', '1834129232.710106583182754711', '69172937540222.03', 16, 5);
    T('817054585216.41455644226', '774646165.0', '816279939051.41', 14, 5);
    T('763244594697.2340083455885784030', '1043370088.347', '762201000000', 6, 6);
    T('2317.26334976', '21830.773042765', '-19513.509693005', 20, 5);
    T('62.13732923639173114', '58530.94192933191519320', '-58468.80460009552346206', 22, 6);
    T('3813165375920055.38747', '7047208255238.0', '3806118167664817.38747', 38, 0);
    T('22617135538806924330.97860712019914274', '3992542466943660362.3814', '18624593071863263968.5972071202', 30, 6);
    T('59607488516856173.867679', '46380.143970622', '59607488516809793.723708378', 38, 1);
    T('6270703.0983', '9438382906.8652341392369798162', '-9432112203.7669341392369799', 26, 0);
    T('18017993023702.580893173', '518399531738180266.41443930014', '-518381513745156563.83354612714', 36, 4);
    T('913513868.94264723657900902110', '42775311.37738528754', '870738557.5652619490390090211', 36, 5);
    T('6137.00965622', '5124362996965.881705677289', '-5124362990000', 9, 4);
    T('7606578014.2', '51771218161590.0', '-51763611583575.8', 40, 5);
    T('2582491851.5', '5978938902286.797551813641507595', '-5976356410435.29755181365', 24, 0);
    T('460.3206760', '7365199530682785432.688041560447619', '-7365199530682784972.367365561', 28, 0);
    T('2222815906358.32', '55117113729240139809.0', '-55117111506424233450.68', 25, 6);
    T('99543289.10055760069146311222', '21569807069869168.37072436069258', '-20000000000000000', 1, 5);
    T('46.3817606', '9915.97790', '-9869.5961394', 15, 6);
    T('224.40550672365460562', '4326268765121392414.4893', '-4326268765121392190.0837932763454', 33, 0);
    T('549538836831086032.2348350543168344', '1050136632931924677.0', '-500597796100838645', 18, 3);
    T('71381287.82', '91545.4584', '71290000', 4, 2);
    T('95628368701675988.03362837652787451', '3399894454307113505.2918983', '-3304266085605437520', 18, 3);
    T('86386989345.011', '2432969215278.446647019857', '-2346582225933.435647019857', 35, 5);
    T('6303019264540888.486463945491813', '160.727855772098', '6303019264540727.758608173', 25, 5);
    T('8725434.0', '84399.86633156', '8641034.13366844', 34, 5);
    T('5.4602526357198185446', '2425549.6974336406649738', '-2425544.2372', 11, 4);
    T('6789622245.32018028157848096738', '5138452563881.31163124193991333791', '-5131662941635.99145096036143237053', 33, 6);
    T('47842638718425014684.112265', '610675.108', '47842638718424404009.004265', 36, 5);
    T('990965788555634345.09352677849255783985', '637.31450036289', '990965788555633707.8', 19, 6);
    T('8490940518235.2476654203', '738787371456.4', '7752100000000', 5, 1);
    T('4883943113.8598158', '92488806.37555905', '4791400000', 5, 3);
    T('812468877.798386', '1.8639481431751078457', '812468876', 9, 5);
    T('6182358111067.93450726980624263069', '8.134876721798414', '6182358111059.79963054800783', 27, 4);
    T('67718589674095905.638', '1469304.21894686863202234', '67719000000000000', 5, 6);
    T('1418708176264728.105170', '4246239600.4885151000232', '1418703930025127.62', 18, 5);
    T('233072104.7108657991399836', '9120447095826432.8590646962722249', '-9120446862754328.148199', 22, 3);
    T('317692502172385111.6877258315', '40122042611185742.64997116466603431', '277570459561199369.03775466683396569', 40, 5);
    T('913.7925243042450431276', '297545312872182.1793388772140426253', '-297545312871268.3868145729689994977', 35, 3);
    T('69730610832936.91984038068527615', '60148.9295262', '69730611000000', 8, 5);
    T('8170093.0', '52027722532965259.30660', '-52027722524795166.3066', 37, 6);
    T('2240343867.74', '636385.69443517', '2200000000', 2, 4);
    T('89281062.3293632615125', '1.72314497', '89281060.6062182915125', 32, 2);
    T('326213959455317742.233164311055964', '390258199416590469.559898253463568', '-64044239961272727.326733942407604', 38, 0);
    T('373429463259.459619', '92825369374801946740.5', '-92825369001372483481.040381', 35, 2);
    T('289766140880268.713944667658681344', '4962532998196899.0', '-4672766857316630.286', 19, 5);
    T('2443199523.93883259081', '7932115.591801539816910', '2435267408', 10, 5);
    T('433259218894637259.0523953956325231102', '206396417364.9', '433259012498219894.15', 20, 5);
    T('2747082860.84', '8254172.39114568562630240', '2738828688.4488543143736976', 40, 5);
    T('4054906.46847723799888576899', '516588770090175213.0', '-516588770086100000', 13, 1);
    T('5393115701622154739.4621423497983311826', '7647706708375.43253334122363', '5393108053915446364.0296090085747011826', 39, 5);
    T('647554.084479', '867353324722.25732358235752', '-867352677168.17284458235752', 28, 2);
    T('46113237.0426', '194517422937241.81864', '-194517376824004.77604', 30, 4);
    T('13476346362819.96796989579454', '2537.84472306', '13476346300000', 9, 3);
    T('7870181081020061051.56396350', '265.0', '7870181081020060786.5639635', 34, 5);
    T('0.603970447', '7003473464.993656767345', '-7003473464.3897', 14, 0);
    T('37842542203555768.09', '5251121463724.3', '37837291082092043.79', 34, 5);
    T('6.6255757779313914', '30040740941376020161.34461741568058539', '-30040740941400000000', 12, 0);
    T('90.2761536', '696.9706', '-606.6944464', 35, 4);
    T('75.5', '34648855333000435488.3863221804163546', '-34648855333000435412', 20, 2);
    T('386022980689.200594', '5410593182286753.0085406880', '-5410207159306063.807946688', 37, 5);
    T('688216410112.258456072060335437', '71659030196656784.396504123', '-71658341980246670', 16, 5);
    T('328448370.15', '7.3501811399100814', '328448362.7998188601', 19, 5);
    T('10851750.117', '768169.066548707441', '10083581.0505', 12, 6);
    T('602137290216460550.49009458519835730', '7753595425728559478.80355331679319074', '-7000000000000000000', 1, 5);
    T('4116439.5893365139', '6731.87893554086266771713', '4109707.71040098', 15, 2);
    T('8118047509.253406769113', '2238754843.135051420', '5879292666.118355349113', 39, 5);
    T('9664288551858545.20601223036463179', '255800510040718.0', '9408488041817827.20601223036464', 30, 0);
    T('88422227357424.0231045', '63850333604923378.27349479597144098127', '-63761911377565954.3', 18, 5);
    T('35418263.3846513346960031', '565436314619917858.568', '-565436314584499595.183348665303', 30, 1);
    T('62785388808106.6629756755110741', '4.52159683710748', '62785388808102.1413788384035941', 39, 1);
    T('607680648353991839.023751862863882', '756139077068859.3965', '606924509276922979.627251862863882', 37, 2);
    T('635842214754972.7421', '4915185574217360.303690363211431', '-4279343359462387.561590363211431', 39, 1);
    T('20.440884552', '5412769089577405082.5296146576801355334', '-5412769089577406000', 16, 0);
    T('860657.69217756825', '4.2513414229264993', '860000', 2, 4);
    T('78429032940745313928.007061837463811804', '7414096040577.116675643634650443', '78429025526649273350.89039', 25, 5);
    T('614.4666503898753', '7.48', '606.9866503898753', 38, 4);
    T('484.1346', '2540840457728512.88874435241535710', '-2540840457728028.7541443524153571', 34, 2);
    T('3521898.74619318', '8338767.9', '-4816869.15380682', 28, 2);
    T('45.587439', '3930139629481.4590287627405788611', '-3930139629435.871589', 19, 1);
    T('92905328.9661118', '887573744.35394118873', '-794668415.3878293887', 19, 5);
    T('30.08433365834414', '6751.1753562', '-6721.09102254165586', 31, 6);
    T('4.81176', '4687758426599.7', '-4687758426594.88824', 40, 0);
    T('82952099062688248.18576122', '284040.7072939694', '82952099062404207.4784672506', 30, 3);
    T('3859813.713032872233', '2.986423079574', '3800000', 2, 1);
    T('6512.47263256835270', '4720643096101485.27593461', '-4720643096094972.8033020416473', 37, 3);
    T('7037.1776363057936469495', '7.765873007255', '7029.41177', 9, 0);
    T('57746374.323572714853605', '72.31124412470350325430', '57746302.0123285901501017457', 32, 4);
    T('597381525544319.868', '8041279177366474262.29944549220', '-8040000000000000000', 4, 2);
    T('543.7805222685099585347', '9468.88', '-8925.0994777314900414653', 35, 5);
    T('1671144.533', '832186802225.694799380', '-832185131081.16179938', 38, 6);
    T('51010.02994250315458', '5645622.381519270846', '-5594612.35157', 12, 2);
    T('1688.93081031320736935', '434.16897350318669457697', '1254.76183681002067477303', 28, 2);
    T('573221814450215599.36215963560', '5.9', '573221814450220000', 14, 4);
    T('78495011.0507779', '551.64922503505491', '78494459.40155286494509', 32, 1);
    T('829929044.686843717690572362', '629240817767.0', '-628410888722.31315628230942', 26, 2);
    T('451511655663.594543579147078923', '56618022430497.2', '-56166510774833.605456420852921077', 40, 5);
    T('7290435560.35210717', '614755.3566853404507', '7289820000', 6, 3);
    T('7.0125701418077383529', '20315803438051488239.05680189206', '-20315803438051488232.0442317502522616471', 39, 3);
    T('37611347042691610.20', '747068.307231448184', '37611347041944541.8927685519', 27, 0);
    T('8732.26250848411', '82.19758', '8650.06492848411', 29, 0);
    T('4161.05440', '65737198779833943716.900797912', '-70000000000000000000', 1, 6);
    T('5603.72777749', '80616809086.8146711434562', '-80616803483.0868', 15, 1);
    T('595470987550.2768137272212382', '49.536628754492', '595470987500.7401849727292382', 36, 5);
    T('707459663499819905.64108429253606', '7828328875036.665487857121609795', '707451835170944870', 17, 0);
    T('388.4322460293520', '808347077510408.094899540913319018', '-808347080000000', 8, 6);
    T('1.807840106459', '163312228.6116071177', '-163312226.803767011', 18, 5);
    T('776032541946778295.2385512', '954877534.7144742654', '776032540990000000', 11, 6);
    T('91449252664905.109608011046412', '86.1985435885141', '91449252664818.911064422532312', 32, 1);
    T('7209980874154194.437351368543210578', '32203695052776.5571999884953914', '7177777179100000', 12, 6);
    T('53125830168757174.4445', '9021831200068773469.50401', '-8968705369900016295.05951', 28, 6);
    T('853892234451798445.1553584395935722', '6471084435392143.472927013053511677', '847421150016406301.6824314265401', 31, 6);
    T('8303534085390596.18764858299931909', '52132671754.6652079849', '8303481952718841.52244059809932', 31, 0);
    T('3923199565427489.0', '3.731', '3920000000000000', 3, 4);
    T('18582.975790426', '53063.18884', '-34480.213049574', 25, 1);
    T('199710450469728.93084933662', '1248861812.38796509', '199709201607920', 14, 4);
    T('5783956847973.738255127', '8866715319226177.4270144304', '-8860931360000000', 9, 4);
    T('10654721132329677050.6400', '31870681034.6767506025741575', '10000000000000000000', 1, 3);
    T('25397765.43988461743000', '45.276203', '30000000', 1, 4);
    T('4336.861422422250046', '58582038143.87054856726413', '-58582033807', 11, 5);
    T('8.23695447796025666', '8996656.11979593', '-8996647.88', 9, 5);
    T('8568104772.95311945', '4814085774.3208839950994668373', '3754018998.633', 13, 2);
    T('63882610.0687', '1.3', '63882608.7687', 32, 3);
    T('887.2632903647639113', '8.49133314', '878.771957224764', 15, 0);
    T('374067.8576501773', '99718787778777928545.48137933535', '-99718787778000000000', 11, 1);
    T('6118112.43', '540.3418786030', '6117572.088121397', 36, 3);
    T('103.518414500502519312', '5890314.8046003', '-5890211.2862', 11, 5);
    T('300390934251375.449606', '6426428803.3702120865539418836', '300384507830000', 11, 2);
    T('667.83718493109453600539', '951451321134.60690378371609764372', '-951451320466.76971885262156163', 29, 1);
    T('23.208308612839000477', '6.701123750459', '16.507184862380000477', 36, 1);
    T('883180048555133127.107883542830', '4019392.628', '883180048551113734.47988354283', 38, 0);
    T('126.35270433286', '87122962630543821.92369341354883452', '-87122962630543695.58', 19, 0);
    T('809.0', '66955.95119303185', '-66147', 6, 3);
    T('50968920741711.5945171181971', '7262539.6906854177577583', '50968913479171.90383170044', 25, 4);
    T('5.29853733', '498069352288.8135', '-498069352283.51496267', 21, 2);
    T('9898346242359387.013036267452', '741.0535', '9898346242358645.959536267452', 38, 0);
    T('67.159089123', '758286.988', '-758219.828910877', 16, 3);
    T('4152235098553.1139164407151734', '2047976154184520183.29037', '-2047972001949421000', 16, 2);
    T('8149494177900327.829664', '743695955082848046.07646612728469359140', '-735600000000000000', 4, 0);
    T('4562.902933785563', '15598706865395.143178682200813277', '-15598706860832.240244896637813277', 38, 2);
    T('6933141952209.0515532921925933050', '595657797756.4492826', '6337484154452.602270692192593', 28, 6);
    T('6568016989029.389668', '86612535873988279.73640861975', '-86606000000000000', 6, 0);
    T('23209445479.7516621795622', '429807702526.455', '-406590000000', 5, 2);
    T('155969857490316.9001612826941988481', '139121717990685300.37716863791131135099', '-139000000000000000', 3, 3);
    T('500016.423407410118', '401884643.3', '-401385000', 6, 4);
    T('416388854970646192.6', '91758551858745238053.12524', '-91000000000000000000', 2, 6);
    T('704210.56378074667', '77325.88294913126866795896', '626884.68083161540133204104', 36, 5);
    T('5172920324252.95691637745618611664', '12342974.453174', '5172907981278.5037423774561861167', 32, 2);
    T('867689.54041065466', '298217792316.9202048363469852387', '-298216924627.3797941816869852387', 39, 2);
    T('9732755926063905758.274037748545', '2.830928455823963605', '9732755926063905755.44310929272103', 33, 3);
    T('9569.6534673779644186008', '147803.0', '-138233.3465326220355814', 22, 4);
    T('5116176367904892195.8719995081', '15401.2372863', '5116176367904800000', 14, 1);
    T('94946536.042061095401687006', '16928.665', '94929607.377061095401687006', 37, 6);
    T('23.5954', '2117827977.0591264', '-2117827953.4637264', 27, 4);
    T('273599785822922.12169682', '97256642300274090.49736', '-96983042514451168.37566318', 35, 5);
    T('64.55113370227', '6.136743848313', '58.41', 4, 1);
    T('592.046306370277639936', '7815594736603.8687799189041', '-7810000000000', 3, 2);
    T('89912.5560827120', '359932.1053750', '-270000', 2, 2);
    T('51279421536828.6619772849130803', '507103.0', '51279421029725.661977', 20, 1);
    T('59607027169796227.20611695', '6519798703319.91549831239901', '59600507371092907.2906186376', 27, 6);
    T('5279312342.622236', '6974.64', '5279305367.982236', 37, 4);
    T('9210591443362.55599662185', '940560884.27692948', '9209650882478.27906714185', 31, 0);
    T('363866.0', '831889321135406247.7394970980924197039', '-831880000000000000', 5, 2);
    T('46946706948622.92543400265040875504', '4565686763436795626.0', '-4565639816729847003.07456599734959124', 36, 5);
    T('495526646624849504.142134', '7.0', '495526646624849497.14213', 23, 1);
    T('9714035906800816.723173', '13151993592042181.8886639', '-3437957685241365.1654909', 39, 4);
    T('1795229.8610889049897', '6284098.698210149795623', '-4488868.837121244805923', 33, 5);
    T('195.1760717508089982', '1983476135931.3986701340911020276', '-1983476135736.22259838328', 24, 5);
    T('15.5405326639488463019', '1779515984220150.16289039795138729965', '-2000000000000000', 1, 4);
    T('396.31090066834961879699', '217766048833.410269413801694222', '-217766048437', 12, 2);
    T('450688454578.2443582330479', '2041146201789.804979', '-1590457747211.5606207669521', 38, 6);
    T('1034586971907961945.911515606030', '9669576533.6165020605509710', '1034586962238385412.295013545479029', 34, 1);
    T('873247867.606290755698867012', '29812665.407391831567757991', '843435202.198898924131109021', 38, 3);
    T('9918.93630897884154809', '3267.288447121668579', '6651.64786185717296909', 31, 5);
    T('83156.7738167902', '844.31445', '82312.45936679', 13, 5);
    T('2795738768027.72372', '82959619453.4875213', '2712779148574.2361987', 32, 0);
    T('4962705.789358198359289', '85375.4710620990656142834', '4877000', 4, 6);
    T('20444267898394512849.497109', '911552980.05440731134984', '20444267897482959869.44270168865016', 34, 3);
    T('193246081599904.9501224249', '6356008706934.22905422379642247', '186890072892970.721068', 21, 4);
    T('945005269826.7498771063434744', '80.428056534279', '945005269746.321820572', 22, 3);
    T('494519082073455.641720919826910006', '5794652927055664049.0232893698709966596', '-5795000000000000000', 4, 3);
    T('8934989011241102.59734807880511', '99756965930.46', '8934889254275172.13734', 21, 3);
    T('23091685.18978615100317', '200932098206864983.0540360195851255098', '-200932098183773297.86424986858195', 32, 2);
    T('524715.4293044', '12878408.71651235061381', '-12353693.28720795061381', 39, 0);
    T('633942235211127165.59376068804671859256', '863966775022659104.3', '-230024539812000000', 12, 0);
    T('2635040582.6664031810044399', '915604596169.3398869693546468166', '-912969555586.6734837883502069166', 38, 1);
    T('62775686320.3307096', '564247295092393.044851100883490', '-564184519406072.71414150088349', 30, 2);
    T('935921852961.1770', '61572541756.602', '874349310000', 8, 6);
    T('7325263608888.9', '897037.6001611651484', '7325262711851.2998388348516', 40, 1);
    T('8110.4802812661516480', '880858.881259211015373', '-872748.400977944863725', 26, 2);
    T('8207119937310917274.4071898', '40145.088263055090', '8207119937000000000', 10, 1);
    T('5.19689643997933475830', '6400585521.4133973', '-6000000000', 1, 2);
    T('33756620.36589480432', '739947.762161914', '33016672.60373289032', 30, 1);
    T('4558069.0', '4490.93817363', '4553578.06182637', 27, 6);
    T('5100254983059506826.77190396604025565', '4.3081565', '5100254983059506822.46374746604025565', 37, 2);
    T('13.57289', '543991319101181.68742209295365', '-543991319101168.11453209295365', 29, 3);
    T('65419661540575769.066764855955235042', '1411152.20737988149537197576', '65419600000000000', 6, 3);
    T('25527180769924.191562', '7797.0', '25000000000000', 2, 3);
    T('6956481911.78610881747', '6695985716365.407032585', '-6689029234453.62092376753', 37, 1);
    T('6622198039027223566.56', '98741772163608533.575969465082731618', '6523000000000000000', 4, 1);
    T('790322.37009458386406080', '72976346160462670.06', '-72976346159672347.6899054161359392', 35, 4);
    T('55248.971760240699830', '3.120539996', '55245.85122024469983', 29, 5);
    T('8404.0398275816042202203', '87121580.686528842216018997', '-87113176.6467012606117987767', 35, 0);
    T('63185045525.0', '88755201136.0', '-25570155611', 36, 1);
    T('64.4486', '5363054.97312783', '-5362990.52452783', 31, 0);
    T('25.2821', '8927109609019187.62286523596', '-8927110000000000', 7, 4);
    T('36642.4674478461824013829', '659945302115.90422172601232396818', '-659946000000', 6, 3);
    T('7142306516196019.5715488983863', '1066391009095834324.430538969242145923', '-1059248702579638304.8589900709', 29, 5);
    T('1207328226948117.34847328818', '3149954947350731.66785443', '-1942626720402614.31938114182', 29, 5);
    T('43573808454115935463.052698', '98835911501687804855.0334', '-55262103047571900000', 15, 0);
    T('4583347.18506', '625594.36436628468990001400', '3957752.8206937153', 17, 6);
    T('33116.0400464221', '8657399.4021674923836427400', '-8624300', 5, 6);
    T('39766301189381253.08414128078', '8.0192539042', '39766301189381245.06488737658', 29, 4);
    T('6196.2035424156165324', '96937220434.744412750225371178', '-96937214238.5408703346', 21, 6);
    T('3739.715083077', '1160.34137', '2579.373713077', 25, 0);
    T('7501088948229.3106', '322230854409.6527527551954397681', '7178858093819.6578472448045602319', 35, 6);
    T('52.623684825235', '91993.3757526815631882', '-91940.752', 8, 5);
    T('228208932.7735', '1.054396152', '228208931.719103848', 37, 3);
    T('85.3648778054168702785', '915.786611049290341', '-830.4217332438734707215', 36, 5);
    T('3982113423815727464.11', '984.96320436271', '3980000000000000000', 3, 3);
    T('7193363640181.188089390228', '1304824326.94073223995791471400', '7192060000000', 6, 4);
    T('6895249772.516645590186', '6771.84811106', '6895243000.668535', 16, 2);
    T('76.916', '38.0', '38.916', 22, 6);
    T('990387890044.48316', '669055336165.73858684107', '321332553000', 9, 3);
    T('64573.3', '8570830.267631837', '-8506256.967632', 13, 3);
    T('4449894077817600253.86339009320', '117285.4', '4449894077817482968.4633900932', 30, 2);
    T('24.93', '85.5649264663723444', '-60.6349264663723', 15, 6);
    T('93451808251307910.7997833772888', '1050073.0', '93451808250260000', 13, 2);
    T('224.92396232731', '242308863531.31889', '-242308860000', 8, 6);
    T('87005.6248390', '510478.53775937378709', '-423472.9', 7, 1);
    T('56201984785787.65104058535904', '1.80636', '56201984785785.84', 16, 5);
    T('4837542660878106951.4155', '47256680422482064.686246', '4800000000000000000', 2, 4);
    T('982879822477017118.0164014237338183', '983994609844.5529922448868330385', '982878838480000000', 11, 5);
    T('873526.72003528025237', '56454564.24913023336', '-55581037.52909', 13, 1);
    T('7974361346475.36219377040958', '20463154169745620654.01544', '-20463146195384274179', 20, 3);
    T('9289954.8494', '5920333619514.59833551054890105', '-5900000000000', 2, 1);
    T('714681830285517037.4', '427494.10891544143', '714000000000000000', 3, 1);
    T('483.430670', '580099930264.75872005869', '-580099929781.3280500586', 22, 1);
    T('272.51849204852998', '190.0436792165533', '82.47481283197668', 39, 2);
    T('86073937627183.953371968599393', '65582836583385577.838608611406', '-65496762645758393.885236642806607', 35, 6);
    T('83773.71852675294244797450', '6818642662473.164870222810', '-6818642578699.4', 14, 1);
    T('1776311.85198780910226698', '1263865207703906.62768380093764', '-1270000000000000', 3, 0);
    T('45.1692536179417454', '4743239.5840', '-5000000', 1, 4);
    T('2196096538118699.4496282839', '418471010451762881.580807494500044912', '-416274913913644182.1311792106', 28, 4);
    T('2340111497.0', '4658.46688', '2340106838.53312', 16, 5);
    T('5368851478422145.953006', '46003028.62368955354', '5368851432419117.3293164', 23, 4);
    T('131255868434945.5902001272668508241', '750728725111699186.98984337610914809084', '-750597469243264241.399643248842297267', 36, 4);
    T('4941306.3183377800298761', '73158.04304398561373549544', '4868148.3', 8, 4);
    T('3328698960865.431501030', '2026781314233.6157454', '1300000000000', 2, 4);
    T('9275617358816.81', '8255973.51480267631270', '9275609102850', 12, 2);
    T('7236998469709858946.50722511962111938', '4.0570296957254116438', '7236998469709858942.45019542389', 30, 3);
    T('5138017479617050.5271240830', '32494362135.950871679583827565', '5137984985250000', 12, 6);
    T('1514819700.97391969994051819', '7445734523752638.6456', '-7445733008932937.671680300059', 28, 4);
    T('598260515604558.947340985', '7.82831160425467818', '598260515604551.11902938074532182', 40, 0);
    T('29.775204513060752', '86840326161405.3711998762', '-86840326161375.59599536313924', 28, 1);
    T('861924914602668047.056747375731066', '2193319381336.55073740644547', '861922721283286710.50600996928559', 32, 1);
    T('784016.407150794226888474', '3072.241431605537805', '780944.16571918868909', 20, 0);
    T('8916723259187.626', '3964.1904131503909413', '8910000000000', 3, 3);
    T('772006719329668135.2062412451', '911047653292306.0395259019357716', '771095671676375829.1667153431642284', 40, 5);
    T('57846785.7299194', '954438633.67369322997501540871', '-896591847.94377382997501540871', 30, 3);
    T('20612103794992102707.9351', '4131.6', '20612103700000000000', 9, 3);
    T('5244.6308875884008225842', '28866028882240851.73404804827329160', '-28866028882235607.10316045987', 28, 2);
    T('354290841140163745.7', '35427606322496121579.97349060', '-35073315481355958000', 17, 4);
    T('513687287.849873028484', '30211872968.47274111499143', '-29698185680.62286808650743', 39, 5);
    T('1428510051.03926065984177096993', '35721687785861.829', '-35720259275810.7897393401582291', 30, 3);
    T('28601116848.010', '48194459817.16615163830690654610', '-19593342969.1561516383069', 24, 1);
    T('236269606204065146.634238299222455', '471284018228.32413901498141928613', '236269134920046918.3100992842410357139', 37, 6);
    T('99185.9575497818760571', '95480.4814780444403818587', '3705.4760717374356752413', 35, 4);
    T('195316669459.836788536774', '203107556880576908.40139758', '-203000000000000000', 3, 5);
    T('147569714822744.646587900572594', '2068874319459.19825341555278', '145500840503300', 13, 2);
    T('374447000356107927.05382877298805', '2920578560931.97724946195504', '374444079777547000', 16, 5);
    T('354446006488131019.340433', '7804653431222.4829862', '354438201834600000', 13, 1);
    T('719137813.1909709953239', '713188.919', '718424624.2719709953239', 25, 2);
    T('995103518362403343.9918522363973', '5708300008671726998.0', '-4713196490309323654.0081477636027', 36, 6);
    T('6.5', '637.712404103506267', '-631.2124041035', 13, 6);
    T('60.877526561879964446', '985112839360048811.59909741685414', '-985112839360048750.72157', 23, 1);
    T('47.6107800', '38810445359365625924.5760842755', '-38810445360000000000', 10, 0);
    T('713524239107801.39880400927129986475', '888677820.02715882670793528370', '713523350429981.37164518256336458105', 36, 0);
    T('9830706543776300.5', '43959483159608.098744', '9786747060616692.401256', 31, 6);
    T('6648629787274.931571895759938632', '7458985.533735857300', '6648622328289.397836038459938', 28, 1);
    T('332.16993', '2876117764383.729966751776314', '-2876118000000', 7, 3);
    T('7733332972420.03960', '6941952.5', '7700000000000', 2, 6);
    T('704496393316181.7789364006721128', '2960493398591759035.409902', '-2959788902198442853.6309655993278872', 36, 3);
    T('547578.33385592429312397236', '765231855613.934811056581', '-765231308035.60095513228787602764', 33, 5);
    T('474257161.429963', '33548.779468707706', '474223612.65049', 14, 4);
    T('9892.125229568059411', '16622342946313626209.4329', '-16622342946313616317.307670431940589', 36, 5);
    T('1565.77874485971893', '636041865.765', '-636040200', 7, 1);
    T('5258699104445707921.4919646566', '9113851.46', '5258699104400000000', 11, 1);
    T('7.2066952834064185191', '495009056024457.7928', '-495009056024451', 15, 5);
    T('5967818.756912', '31189.7', '5936629.056912', 38, 1);
    T('579340057900072601.217', '1452463564068409.81562048084089', '600000000000000000', 1, 5);
    T('43116417658895.2163132965715917658', '638123430147131.3154057195', '-595007012488200', 13, 2);
    T('339.914448666458631', '835200479105455120.897924712453', '-835200479100000000', 10, 1);
    T('1112974305704762.0050361723227744', '871026206666041228.1375', '-869913232360336466.14', 20, 3);
    T('46403640084731219138.4090542', '15.8672441044', '46403640084731219122.5418100956', 34, 6);
    T('95635877862965160.8754595647636488248', '843682132086.0538', '95635034180833080', 16, 0);
    T('560772167501.5988983035768127', '2972.27856256', '560772164529.3203357435768127', 40, 5);
    T('9871514953652844165.241999674157', '9.5', '9871510000000000000', 6, 4);
    T('91522422101280.70817938215019979', '783.285295633711719', '91522422100000', 9, 3);
    T('6810441023765625358.0584504', '140271.00', '6810441023765485087.0584504', 39, 6);
    T('5337012508778.31', '2036301269.5459705853', '5334976207508.7640294147', 38, 6);
    T('29014.8484363082', '73681547050329.6452', '-73681547021314.7967636918', 32, 3);
    T('70.31505605328493396032', '431007116176841.924', '-431007000000000', 6, 1);
    T('2320833.7694585499028723473', '288.08206511177186', '2320545.6873934381310123473', 37, 6);
    T('951638428456857691.2558375309512', '167284462.9538058124031648977', '951638428289573228.30203171855', 29, 6);
    T('23233620.8386056773624', '2878.38537366', '23300000', 3, 0);
    T('86379.4033170', '64120429.5107333110047748114', '-64034050.1074163110047748114', 32, 1);
    T('68674.03843023356', '2412549.796636592113620286', '-2300000', 2, 1);
    T('18124489.0', '8752715713.0', '-8734591224', 39, 5);
    T('4817.7661954', '478985.542', '-474167.7758046', 30, 3);
    T('5575800630.06805724007', '2041.0', '5575798589.06805725', 18, 2);
    T('8763545.96', '134745852137692119.02437325', '-100000000000000000', 1, 5);
    T('3502480932389584224.908', '6498267716386737873.1996993675834', '-2000000000000000000', 1, 1);
    T('870.6297175948904488016', '0.6399484942706', '869.9898', 7, 6);
    T('663671713110235.488978320338', '50182519734336605364.00003', '-50181856100000000000', 9, 0);
    T('94185699585881544.0759007328125561', '92429.6', '94185699585789114.4759', 22, 4);
    T('7526810.35803968906709', '89075.70', '7437734.6580396890671', 20, 0);
    T('8087.859142378603252', '5296119824467940328.80726', '-5296119824467932240.94811', 24, 1);
    T('93248792249650879627.642890953236431323', '52059383845.4999506070', '93248792197591495782.1429403', 27, 6);
    T('96499952955149657.73438587727586', '388894547090224.8336', '96111058408059432.900785877', 26, 6);
    T('66024062517.065598278', '5462299214290.869909966117', '-5396275000000', 7, 2);
    T('20565931493295235747.68742806', '9548.3832670770569', '21000000000000000000', 2, 2);
    T('16286740818021121833.75540156722618', '7275977887.51906386264861005', '16286740810745143946.23633770457756995', 40, 2);
    T('3081485386816315896.9216573058664107676', '8297852582942751785.530290157', '-5216367200000000000', 8, 5);
    T('224938304841060409.4437341030049', '96317142.2700193873', '224938304744700000', 13, 6);
    T('2.0155013450', '87971699610932.9239196230204', '-87971699610930.9084', 18, 1);
    T('431527070.0668472060815022', '456106060737511770.585768258650894', '-456106060305984700.5189210525693918', 39, 1);
    T('68202083532420.71386', '4006269831561.870766942434602', '60000000000000', 1, 3);
    T('515.08300948609455953', '38784654.7968413599085264', '-38784139.7139', 12, 0);
    T('71070295676601227526.0822331', '97512438271504245.02272', '70972783238329723281.0595131', 39, 3);
    T('67132.39437399805739169', '95221373.19729467852602675', '-95154240.80292068046863506', 33, 0);
    T('547755646707.98', '9142614658479792087.76198778', '-9142614110724145379.7819877', 26, 1);
    T('3664659291.68340560', '8410589209819344.85805652763621183952', '-8410585545160053.17465', 21, 5);
    T('60130.293885996866822077', '934823675763701.79314656', '-934823675703571.49926056313', 26, 2);
    T('83874011149.98193664', '429160538132590.42223400210955939', '-429076664121440.44029736211', 26, 0);
    T('3043.0', '665286245.052524088194', '-665283202.052524088194', 28, 3);
    T('678.16612', '84917902548358373403.963236', '-90000000000000000000', 1, 3);
    T('36771.95127859093357436', '1536944283.84001606643965744', '-1536000000', 4, 2);
    T('6560131036805818.4033192108925', '951140004246839081.262', '-944579873210033262.8586807891075', 32, 3);
    T('1667563.41181645687', '1129620805752475.29991903870512', '-1129620804084911.88810258183512', 38, 4);
    T('6.45649598091988', '55063455439708183538.647', '-55063455439708183530', 19, 1);
    T('5.1289832244383497623', '30889249652568.33699639884800766756', '-30889249650000', 10, 4);
    T('2779.205', '85176409261683069914.75841547154042', '-85100000000000000000', 3, 2);
    T('4.4487754969481956', '9098912539398373.262', '-9098912539398368.8132245030518044', 40, 6);
    T('1834382239010.668968417507335', '13308508213533099165.2', '-13309000000000000000', 5, 0);
    T('589724.6996938189794919', '9592081.360877905', '-9002356.66118', 12, 1);
    T('40924207.293978049', '378.29683866', '40923828.997139389', 22, 2);
    T('37734939188461283996.0', '599012278.551335825488', '37734939187862271717.45', 22, 0);
    T('266993364534019.61518280997', '157455348766419379.6', '-157188355401885359.98481719', 26, 6);
    T('4861875.136348664085080', '977171.22397046962463499388', '3884703.912378194461', 19, 2);
    T('996768008.81222563', '64081111697935.35272955497', '-64080000000000', 5, 1);
    T('209503332540438183.0', '5.681277571200', '209503332540438177.3187224288', 36, 0);
    T('453898871662637.0', '7.1937700224302', '453898871662629.8062299775698', 32, 2);
    T('5667738775893.0047', '84.220712227507277', '5667738775810', 12, 0);
    T('71478879030665.0', '41109980.676701', '71478837920680', 13, 5);
    T('4634410231334247.133790433627859395', '633524.3310', '4634410230701000', 13, 0);
    T('59355154676.413428948259969', '2094697509346808733.0646903492314760220', '-2094697449991655000', 16, 0);
    T('9790792393784833.938340615', '4089850356914257335.29559633867', '-4090000000000000000', 3, 0);
    T('741554484961300764.98147851356', '6821955385507193459.436388172', '-6080400900550000000', 12, 0);
    T('56453.8169', '9.092', '56000', 2, 6);
    T('5882181349618206.856247', '35.340537265750265', '5882181349618171.515709734249735', 38, 4);
    T('4496671114544409457.8', '41.29', '4496671115000000000', 10, 4);
    T('3302059.121989', '1643072.4366358620', '1658000', 4, 3);
    T('5555.072', '418377229023.9938', '-500000000000', 1, 0);
    T('1252804180415904672.0', '84793645018101961.07731661323279', '1168010535397802800', 17, 2);
    T('500462.19578646500', '71235.58', '429226.6158', 10, 5);
    T('25496202.71711', '7694312672.261493093', '-7668816469.544383093', 22, 5);
    T('61940.05336258', '5351515.9070324090759', '-5289575.85366982908', 18, 0);
    T('49663432543.534300', '71.80071396080640103803', '49663432471.733586', 18, 4);
    T('0.652233880488140', '325022.634', '-325030', 5, 0);
    T('5920787148762235955.1065101617', '2.137262039106', '5920787148762236000', 16, 0);
    T('7700.424529486150', '818930593.5912949633725787156', '-818922893.16676548', 17, 4);
    T('5596479289238670.686401094738586', '372065190224800.34565306597', '5224414099013900', 14, 0);
    T('20564600.39296187', '30372646.577854822', '-9808046.184893', 14, 0);
    T('689833.63145198499', '345857.609432701', '343976.02201928399', 35, 2);
    T('4108649009554.45502', '987691628286663820.0', '-987687520000000000', 8, 4);
    T('2563460834725.748', '2460135810.373101602', '2561000698915.374898398', 29, 6);
    T('7418857483171.368486', '42482195143.0', '7376375288028.368486', 34, 0);
    T('96899599.8196', '6210.27921', '96893389.54039', 20, 6);
    T('26728366570366.160', '54518132557583056.5902639028433501984', '-54491404191012690.43027', 22, 0);
    T('620.532461900786', '224326280.480', '-224325659.947538099214', 21, 5);
    T('14908224346.3492507734', '48140260.0', '14900000000', 3, 6);
    T('863839.34000851951', '41.642', '863797.69800851951', 28, 1);
    T('1746156743731.97553115347435114', '712924.887041953', '1746100000000', 5, 3);
    T('8073.7', '63530.12432106', '-55456.42432106', 16, 5);
    T('1530089.46338579469480495', '508016528.063', '-506486438.59961420530519505', 29, 3);
    T('62049430189414.47633621', '192385210.298888645970346', '62000000000000', 3, 1);
    T('569422281426690.542973755436359099', '12465248116778852586.333', '-12464678690000000000', 10, 1);
    T('73876806437.041564216460', '573.5353', '73876805863.50626421646', 22, 4);
    T('660469518519.329423104417882', '3870019.5108138413669122', '660465648499.82', 14, 0);
    T('4789976187303466.270474250', '7241766057156027.39685271302353284767', '-2451789869852561.12637846303', 27, 3);
    T('25376701801153.9964588322033', '27619.21', '25376701773534.7864588322', 24, 6);
    T('72.81763', '147955.342895079', '-147882.5252', 10, 1);
    T('281.6956', '577975747200.937691656119662', '-577975746919.24209165612', 24, 6);
    T('37787456030.14513468', '16644263486988479.50836', '-16644225699532449.36322532', 27, 2);
    T('779667651.04524641385270', '895763565.70417783422173751867', '-116095914.65893142', 18, 4);
    T('471429310.0', '3.5484248288798882489', '471400000', 4, 3);
    T('49716664.47196', '59950528613396645.625751773', '-59950528563679981.153791773', 32, 5);
    T('34413.7382676481', '33586717.31117534327916012961', '-33552303.57290769517916012961', 40, 1);
    T('956545957.7970116400942354', '118744248958.81841990394360276834', '-117787703001.021', 15, 5);
    T('8.99662417598233', '1544057.65946631', '-1544048.6628421340176', 20, 2);
    T('38763315298.99480965532', '485.41524676927955584', '38764000000', 5, 2);
    T('11202193517.55824146098', '19982.1875257896373911', '11202173535.37072', 16, 2);
    T('933919790734887.4486642071499508', '13.44285015925283346280', '933919790734874.005814047897118', 30, 0);
    T('7954749634782003523.154078242151888', '133394620652053313.531', '7821355014129950209.623078242151888', 39, 6);
    T('65698429385685001699.562364675983', '25597066906731.213788046403850114', '65698403788618094968.348576629579', 32, 6);
    T('197694176343.1367602649175036', '386193275365014.0', '-385995581188670.8632397350824964', 40, 1);
    T('5422.53219275029963774980', '57797438.652', '-57792016.1198072497003623', 24, 3);
    T('63694522.1886438959791', '57488311630651174331.871004', '-57488311630587479809.6823601040209', 33, 3);
    T('97200059.8564549731288', '17272097758129827899.7813207821269', '-17272097758032627839.9248658089981', 34, 6);
    T('52003494.9', '15096304.5151858', '36907190.38481', 13, 5);
    T('601030365177800025.7331013719', '5773733432838542560.1738011602131', '-5000000000000000000', 1, 2);
    T('429918788100862.0023209416382', '9297443922103336.81660766389080550485', '-8867525134002500', 14, 5);
    T('648482.6263067234135245505', '4464684930091439.09947128390', '-4464684929442956.4731645604864754495', 36, 5);
    T('791338048339053327.246324656801774338', '32737475473609350709.292015684481', '-31946137425270297382.04569103', 28, 4);
    T('3300223158147.5', '924269518.0', '3299298888629.5', 17, 4);
    T('2555558.95', '99030288836087916.57558', '-99030288833532357.62558', 38, 2);
    T('61765994739638.816867884002791101', '49595688.09305506', '61766000000000', 5, 5);
    T('9627880105957058.34174635809621268', '69095483998324688.13160296551322', '-59467603892400000', 12, 0);
    T('2205175454568737032.06255056605', '90.7548527059239960', '2205175454568736941.308', 22, 2);
    T('71192009998591413236.999274604259', '32814407791018.631880121', '72000000000000000000', 2, 2);
    T('50787400380390751205.4721706248669938702', '565298.714878081700939231', '50787400380390000000', 13, 3);
    T('47984235859366964.3507034', '0.20705797751840983802', '47984235859366964.1', 18, 4);
    T('42074779410232894.236094761525', '26636205867.0', '42074752774027027.236094761525', 29, 6);
    T('8.617864801619967421', '7980895220.41903823', '-7980895211.801173428380032579', 29, 0);
    T('55023714.5391', '3515406206.13579', '-3460382491.59669', 25, 1);
    T('252.1852', '5774929434.8484736545289', '-5800000000', 2, 5);
    T('2.56003', '899771308153669.718789860755312', '-899771400000000', 7, 3);
    T('440151040.9079927993937', '481255785936418.77675655', '-481255345785380', 14, 5);
    T('4833184824.0', '1903223322924049.49123128419', '-1904000000000000', 4, 3);
    T('55588.995', '7088031043198.199683948994299', '-7088030987609.204683948994299', 34, 4);
    T('897.359117227700959', '64686071106350.8549915765402682', '-64686071105453.495874348839', 26, 4);
    T('11685959559275.10', '568143.266573', '11685958991131.833427', 39, 5);
    T('6965400767389107584.21531', '32649664935.093166503511246254', '6965400734739442649.1221434964888', 32, 4);
    T('8322430021870074161.1390792801321787', '2766848.11284', '8322430020000000000', 9, 5);
    T('7345755.748551', '7358856.25', '-13110', 4, 3);
    T('7.87002778210', '8.0581', '-0.1880722179', 36, 4);
    T('780.804883529003528', '90496867027237090.0', '-90496867027236309.195116470996472', 38, 1);
    T('239189711560894310.22586', '884.5447007', '239189711500000000', 10, 3);
    T('80245020888689325739.2338', '811821444364034699.38350103935', '80000000000000000000', 1, 5);
    T('726.27', '532676748875370.29', '-532676748874644.02', 20, 3);
    T('29715.63734112681961', '6347.03111601394798', '23368.60622511287163', 34, 2);
    T('5824156547.897817016654001764', '2654.15061', '5824153893.747207016654001764', 34, 1);
    T('607462421.064753', '31387915384981.5', '-31387307922560', 13, 6);
    T('0.291954108400112', '675616992727804888.9161534445292116', '-675616992727800000', 14, 4);
    T('1474521438327.0', '406161.574875063940567', '1474521032160', 12, 3);
    T('1678.0344231', '6996988477661500748.664', '-6996988500000000000', 8, 0);
    T('9904080216.85589', '2942121.943753734864644', '9901138094.912136265135356', 25, 3);
    T('19903.7', '32510580673279315.35', '-32510580673259412', 17, 6);
    T('5563709581687981.3788428575', '6677713035013999172.63790907366573102871', '-6672149325432311191.2', 20, 1);
    T('998754.517689260', '79107.89', '919646.62769', 11, 6);
    T('63285254951.3', '52091192.80491158', '63233163758.4950884', 18, 6);
    T('2783993674240.77', '7631094192.90876627799', '2800000000000', 2, 2);
    T('7386.0', '640889847.71538965560999492', '-640882461.71539', 14, 0);
    T('788050.48145483358974713', '5396700161026439.5976', '-5396700160238389.11614516641025287', 38, 2);
    T('9952527129224779192.4602212', '81061614386217320956.43054185', '-71109087256992541763.97032065', 35, 1);
    T('439322758098.3522', '354446.523815583', '439322403650', 11, 1);
    T('6.197655751393', '988540415556.5486', '-988540415550.350944248607', 31, 3);
    T('5287580619043403861.441226503431134483', '51607156965546076947.821552616817668', '-46319580000000000000', 7, 5);
    T('372669991199295360.245779529980622', '87979408.617246', '372669991111315951.628533529980622', 38, 6);
    T('20550388.7825745177840439686', '426508574244552076.5983860', '-426508574224001687.82', 20, 3);
    T('972247870084632.8995745697', '635963439832687.9223277322962769178', '336284430251944.9772468374037230822', 35, 4);
    T('75639.0792159338361304', '6764321752751505.38385632777556148', '-6764321752675866.30464', 22, 6);
    T('3419162859.84578752', '21644.339626763110791548', '3419141215.506160756889208452', 31, 1);
    T('836845302949.6508663', '57.868587', '836845302891.7822793', 37, 6);
    T('73377822.8581581', '9523895264353395674.24455647209853944060', '-9523895270000000000', 9, 0);
    T('8631081.931719582234310301', '23489433.63', '-14858351.698280417765689699', 38, 5);
    T('25081802357091.66034299471084', '215377.8339014623827', '25081802141713.82644153', 22, 4);
    T('50994.858565131045842931', '7884.425527', '43110.433038131045842931', 40, 3);
    T('80315166408.0', '19.16265018466710718', '80315166388.83734981533289282', 32, 4);
    T('7.762602376446', '976.3860655797882989155', '-968.6234632033422989155', 24, 5);
    T('8123653395033406615.797862577929502', '47862.834320944699', '8123653400000000000', 9, 0);
    T('851278150.850', '26022.87075353514916632270', '851252127.9793', 13, 2);
    T('49821638435.0', '51.07760414365759', '49821638383.92239585634241', 37, 1);
    T('13690956413504329169.7550539047213972', '2482169.5701', '13690956413501847000.185', 23, 2);
    T('0.139915949175691150', '33463241189111.3915447290328657', '-33463241189111.3', 15, 3);
    T('363512203854802787.22276594642885549908', '67955158972.825491923202', '363512135899643814.397274', 25, 5);
    T('605976623700.3802', '764091865526964496.95', '-764091259550340800', 16, 6);
    T('6049.75613969', '895615212.537545421976886624', '-895609162.781405731976886624', 34, 4);
    T('4800.194', '834.97962907664894592', '3965.2143709233511', 17, 5);
    T('72974972440854627917.07670800032697743595', '83327065626289227.40045929', '72891645300000000000', 9, 1);
    T('8303511.27403293', '5821955227082.99325263546', '-5821950000000', 6, 6);
    T('4653517512250375983.49984642750', '2954165663714701.84516225973', '4650563346586661281.65468416777', 37, 1);
    T('450345.99458347990', '165883.15620917503551201479', '284462.8383743', 14, 5);
    T('567505627083450249.4176569574', '1.0', '567505627083450248.4176569574', 31, 0);
    T('2557876237939.37249502883132245', '43228423149.746009944332673676', '2514647814789.626485084498649', 28, 2);
    T('5825294893149.51969904062173267653', '883984983.532715543799424794', '5824410908165.986984', 19, 2);
    T('2640.06', '16684277879126584760.9', '-16684300000000000000', 6, 3);
    T('963.12897420117501', '65458540135.929922546083692', '-65458539172.801', 14, 3);
    T('59612998658917389.61400', '57.51', '59613000000000000', 5, 4);
    T('36337.1991935232', '982244802.9247576', '-982200000', 4, 4);
    T('89957282508740.330614826563188', '9.90', '89957282508730.430614826563188', 29, 5);
    T('94.92681204349808909', '3477341.0771759', '-3477246.15036385650191091', 38, 0);
    T('96225445710.268319117', '84.3', '96225445625.968319117', 22, 5);
    T('627184381.14885909448049', '87979589408513.54490', '-87978962224132.39604090551951', 30, 3);
    T('311347.145926385497056', '1968.093079648293', '310000', 2, 2);
    T('84671658119583301.912', '333476638540858298.0', '-248804980420000000', 11, 4);
    T('122.32110', '497122148.54656510130933437888', '-497122026.2254651013', 19, 6);
    T('16671307453174.7986102129085', '383491512567.73', '16287815940000', 10, 5);
    T('52191273376908.29308', '9730102.475075', '52191263646805.818005', 20, 2);
    T('9690.5384748720652', '9942082645087197170.972582154', '-9942082645087187480.4341072819348', 36, 2);
    T('6412583.1171666543410006', '415118.93584011063', '5997464.1813265437110006', 28, 3);
    T('5429.448135780', '81007010586955537092.140', '-81007010586955531700', 18, 6);
    T('7973.30939183503757', '1758763395447158.1324924587874902', '-1758800000000000', 5, 6);
    T('81287940640621255.26050182820256848705', '2.95238659022593679', '81287940640621252.308115237976631697', 36, 5);
    T('253854573.12753810', '18339565.73', '235515007.3975381', 38, 2);
    T('26901.8203694446607180176', '81.511006078077447575', '26820.3093633665832704426', 28, 5);
    T('85008.4990648042987313986', '93.281361860744800611', '84915.2177029435539307876', 29, 2);
    T('5.9264210399437061784', '3083487697866.18039829924359878', '-3083487697860.2539772593', 23, 4);
    T('27101843087911503.98312813958310794', '9.8724865971393798135', '27101843087911494.1106415424437281265', 36, 0);
    T('69261234.7017149302439', '66213657774247351992.8157681262241', '-66213657774200000000', 12, 6);
    T('258309652993793.884907275264', '623158592231235744.566', '-622900282578241950.681092724736', 37, 0);
    T('22081244.093', '112.41908', '22081131.67392', 19, 4);
    T('93901221869854748550.82965102085', '89617.1630842479063', '93901221869854658930', 19, 6);
    T('715734742253146532.02', '100465.373626956991219', '715734800000000000', 7, 0);
    T('12314742157972.80902559252118127677', '864278974931.86', '11450463183041', 15, 0);
    T('9642.290', '2294112823873.77320', '-2294112814000', 10, 4);
    T('643498194811.706768562515105848', '662.273312925971334370', '643498194149.43345563654', 23, 4);
    T('21625146924983987.10917564412', '362.46314880', '21625146924983624.64', 19, 1);
    T('66278.296443251781321', '6889983.695070529700', '-6823705.39862728', 15, 3);
    T('45849781919186.763', '543.645968385485', '45849781918643.117031614515', 38, 4);
    T('294.2157244745048', '3.91438594', '290.3013385345048', 29, 5);
    T('503.21419022054360', '557.90513851696396', '-54.69094829642036', 28, 5);
    T('350038559.5636143686781', '4334462775.4472348', '-3984424215.8836204313219', 35, 0);
    T('2538394151041523411.53', '47616.536', '2538394151041475700', 17, 3);
    T('7.4514683307397319738', '615.788590749172937', '-608.3371224184332050262', 37, 6);
    T('172369186.0', '658610868326787818.61', '-658610868150000000', 11, 5);
    T('4737705897201214692.57515171', '29034964592954919358.912162513', '-24297259000000000000', 8, 0);
    T('6713589.41083311', '9184.3389612065040537218', '6704500', 5, 0);
    T('871678387.42968477235703497976', '960653223019302.102998592443015152', '-960000000000000', 2, 6);
    T('448959503193.8061241849', '455076099967727952.4242916932', '-455075651008220000', 14, 6);
    T('966746446011789387.7520045872605', '78592784799532341177.0', '-77626038353520551789.2479954127395', 37, 4);
    T('46.1262421156318', '65578330393302750596.10205543454630412', '-65578330393302750500', 18, 4);
    T('8823112878.0204050901836094', '8581086541423524828.77451702382575006', '-8581086530000000000', 9, 5);
    T('9.921273', '8152930973.09', '-8152930963.168727', 26, 5);
    T('95273.053417362', '60656.95648618709455782', '34616.09693117490544218', 27, 1);
    T('2512682773888597.7980363', '7.877185386', '2512682773888589.920850914', 25, 1);
    T('35135396913628.4641087', '967954216160094321.536755148', '-967919080800000000', 10, 0);
    T('912772045199.8940108', '76300154.431218156', '912695745045.462792644', 36, 5);
    T('120.2', '6576762.193778134240505530', '-6576641.99377813424050553', 33, 1);
    T('1589329.0', '648798.469076', '940530.530924', 38, 1);
    T('186250616737603580.2', '3441736370.0979554481768', '186250613295867210.102', 21, 4);
    T('50157714.28920075753272814220', '481672009530802955.0285384', '-481672009480645240.739337643', 27, 0);
    T('276437.710058', '69234351748471559575.709177638944174602', '-69234351748471283000', 17, 6);
    T('3755.39076', '275273.46', '-271518.06924', 23, 4);
    T('0.3533644', '9522732477133422981.412', '-9522732477133422981.06', 21, 4);
    T('57200949039686.91732', '274770424687026.55919522505895364473', '-217569475647339.64187522505895364473', 35, 2);
    T('917303880715.23921121', '85431.1218968585578', '917303800000', 7, 2);
    T('62.50542226014523883', '41918857.1679188014586', '-41918794.66', 10, 2);
    T('225348912146275180.53131911451116139', '849265539594.69251463898366270202', '225348062880735600', 16, 0);
    T('193671981.898', '90555.9686930621583', '193581425.929306937842', 21, 2);
    T('890903148222044.03259702142115473541', '63942578909801.3319925343936087', '826960569312242.7', 18, 1);
    T('31.3091446', '1.30388725', '30.00525735', 13, 4);
    T('50309357035803140581.205735473610840147', '358024100.1409048', '50309357035445120000', 16, 2);
    T('510584.45671', '960987307271744.7', '-960987306761160', 15, 2);
    T('6878574061560579.1', '1030153402489.80144896830435855', '6877543908158089.299', 19, 6);
    T('103767947968.2035909032993155', '5589.104549170', '103767942379.099041733299', 24, 4);
    T('58434.17', '11387308863918319910.95872617941508027592', '-11387308863918261476', 20, 2);
    T('76801.7005', '139315.467', '-62513.7665', 40, 2);
    T('5123856785982.96440850769546871578', '397.55458072', '5123856786000', 10, 5);
    T('6259380689263143067.964936639977033', '88.154191731550767107', '6259380689263142980', 19, 2);
    T('335628165548231.1579142479262', '7299802.9233993536902613', '335628158248428.234515', 21, 5);
    T('87414.110030879042', '50662238962686129.1449640753937571426', '-50662238963000000', 11, 5);
    T('752952181394949.88358492715', '1391106225719.39036502993438', '751561075169230.49321989721562', 31, 2);
    T('341776471532728915.94071167405801284', '2799571127953183947.50599354340', '-2460000000000000000', 3, 0);
    T('39826115377.96758576629569490129', '5913570865.423105', '33912544510', 10, 1);
    T('83335490.30', '972871854339514062.69', '-972871854256178572', 18, 5);
    T('5709520241.265', '11268129938.34243785', '-5558609697.07743785', 38, 6);
    T('373906124843815.140439309', '917.36', '373906124842897.780439309', 40, 3);
    T('3830252.743061849611', '5306872.223619', '-1476619.480557150389', 31, 3);
    T('3.9055942', '10438854888904205.86', '-10400000000000000', 3, 4);
    T('1792549504004.37772818', '7920208192677.7693317506017', '-6127658688673.39', 15, 5);
    T('4847112358085364.5677825', '955.1610629462723369', '4847112358080000', 12, 5);
    T('192203187552611.887174091802440075', '40793935185075.919423994', '151409252367535.9677500978', 26, 3);
    T('94465619.8319502836810', '376389627636.0395367991135350', '-376295162010', 11, 2);
    T('67.381320992710306', '541425118.08190894', '-541430000', 5, 3);
    T('319591769439537571.381177340', '8.0750328', '319590000000000000', 5, 3);
    T('21735767289579938368.760493898153802344', '8968832.72055689350939574', '21735767289570969536.03', 22, 1);
    T('2686.25575304', '95747474940429.3877', '-95747474937750', 13, 0);
    T('193502.6735338', '921.556512548568143143', '192581.117021251431856857', 27, 5);
    T('7423072248998403935.539292260', '486426010371512.0380', '7422585822988032423.50129226', 32, 6);
    T('517.00074', '19897007066.199511662502043923', '-19897006549.198771662502043923', 38, 1);
    T('70894805676539155336.74600804', '8425637.10', '70894805676530729699.64600804', 35, 2);
    T('5005670.216610002', '72293537113185680.910', '-72293537108180010.69339', 23, 4);
    T('11238484306324005571.651833574742121254', '96626720198.056658603518726518', '11238484209697000000', 14, 6);
    T('6061928250859.1061417431', '44299047146420532554.7193430003644361476', '-44299041084492281695.61320126', 28, 5);
    T('632469.1981976435986', '9448796095391429631.01', '-9448796095390797161.8119', 23, 3);
    T('7673049.356632000', '99560198.3611471431568', '-91887149.0045151431568', 32, 6);
    T('441438073660804478.060', '52588768.94945', '441438073608215709.11055', 31, 6);
    T('519679.6028', '7362892487528.1964389721141236825', '-7362891967848.5936389721141236825', 35, 0);
    T('96922728077411.293409083247', '2258.0474946568', '96922728075153.245914426447', 32, 2);
    T('569.891610', '5960785796.94842930288', '-5960785227.05682', 15, 3);
    T('89833135524947.13927', '46776.5', '89833135478170.63927', 27, 5);
    T('91476.763114301693', '482740314423703519.61889824093625310', '-482740314423610000', 14, 2);
    T('22190525630934391.0', '57721.87384665945356232010', '22190525630876669.1261533405464376799', 37, 4);
    T('6748767812941856436.7892973', '131110364.045828', '6748767812810746000', 17, 1);
    T('9.4699287778672', '5992.5818056', '-5983.1118768221328', 19, 1);
    T('438365266105194.015187941', '349.2477767', '438365266104844.7', 16, 1);
    T('86093326.445755979873', '62102039793870644637.0', '-62102039793784551311', 20, 6);
    T('15735792964969.84', '13803300351358789.129696246125211', '-13787564558393819.289697', 23, 3);
    T('0.50557752624237', '427906255566933294.8333779590787', '-427906255566933294.32780043283633', 34, 2);
    T('6171105945082.41301024', '14909511528404237362.9204312595727124808', '-14909505357290000000', 13, 2);
    T('93924.811098', '954456096335725.946500620242292', '-954456096241801.13540262', 24, 2);
    T('5.826537', '29984.9866', '-29979.160063', 25, 5);
    T('13522656700524007452.42580857677749330', '9912090425905.902870684471', '13522646788433581546.5229378923064933', 36, 4);
    T('7.00972104727326370298', '91230077624362734312.821811695899', '-91230077624362734310', 19, 0);
    T('0.432660117', '936879.089115755', '-936880', 5, 3);
    T('211930.26332785783516768617', '1313546.9168009074093', '-1101616.65347304957413231383', 34, 6);
    T('44.3300536118361621', '64.9368589368846024', '-20.6068053250484403', 21, 2);
    T('103144.648462214175252800', '32.50757551539448', '103112.140887', 12, 4);
    T('15193986438475545142.6818349438061', '22859320616184380.514254166', '15171127117859360762.2', 21, 5);
    T('384.60096', '8329787462654978364.39619763128968162554', '-8329790000000000000', 6, 5);
    T('9831470317065347784.949343486', '6292635942011735.2311971830247984927', '9825177681123336049.8', 20, 2);
    T('424529.9526476730267118228', '40022917866768511166.44337185', '-40022917866768086640', 19, 3);
    T('305430489891.10341041', '66825249.055762850270', '310000000000', 2, 0);
    T('7632693699.943735992', '5.5058061884', '7632693694.4379298036', 31, 4);
    T('15844045928464825525.191057336292', '86962328.917243340371', '15844045928377860000', 16, 1);
    T('8308794283937.658039525712551307', '491652189.445', '8308302630000', 9, 6);
    T('7345500993424446.91364145518', '7907832859.0682905826825', '7345493085591587.8453508724975', 33, 2);
    T('109.3068116208', '1569864017.106947552716746879', '-1569863907.8001359319', 20, 6);
    T('636158.04363277', '81628110835919088.251482097713402', '-81628110835282930.2078493277', 27, 6);
    T('95.060097617424', '3803950746561.9', '-3803950746466.8399023826', 23, 4);
    T('569150.829016842540017', '1885.9260250350286526', '567265', 6, 4);
    T('76903813593814457744.729569', '695.96207962552', '76903813593814457048.76748937448', 34, 6);
    T('98416056065787097390.2961714724700890', '9638831167557479.262426', '98406417234619539911.0337454725', 30, 2);
    T('473715505.66', '3468502341097956687.52465853176186552', '-3468502340620000000', 12, 2);
    T('5019697534597.99409147899358729', '10922653.14909105', '5019686611944.84500042899', 24, 1);
    T('7948747388401978.9650281697', '896142580.0', '7948746492259398.9650281697', 31, 3);
    T('888.903', '211615051377580787.340209', '-211615051377580000', 15, 3);
    T('20694077476304889.0550731254356475713', '661205.7', '20694077475643683.36', 19, 4);
    T('807839370107413315.523', '55027820.1133464735', '807839370052385000', 15, 3);
    T('91191662.679625', '35624437702188244.862445450862669043', '-35624437610996582.182820451', 26, 6);
    T('40751.1372707966391356486', '33071232130.0', '-33071191378.86272920336', 23, 2);
    T('384.28595903', '783415387217751508.0', '-783415387217751100', 16, 1);
    T('26.6403678062173', '6242.530433506489181185', '-6215.890066', 10, 6);
    T('9658666248170767866.112261700038', '14626.7100713042688', '9658666248170753239.4021904', 26, 6);
    T('323.35577229', '78528045553305.6169999459286811129', '-78528045552982.2612276559286', 27, 2);
    T('567.907', '8942883723130.12603811994468711025', '-8942883722562.21903811994468711025', 36, 2);
    T('585092951068327.7', '11696106454.3219468046', '585081254970000', 11, 0);
    T('7143552806.2489064', '490831234500.65214416369148', '-483687681694.40323776369148', 37, 3);
    T('1898942.2078309874376903860', '59.7940745051628984', '1898882.41375648', 15, 4);
    T('4846349211390292240.52651', '1443759.867229265605861', '4846349211389000000', 13, 6);
    T('540.09764034687737', '95029.3953620', '-94489.297', 8, 1);
    T('14.31075', '29134042992246717011.6416223313145945', '-29134042992247000000', 14, 0);
    T('6823.85', '81870201450.14896924709945913', '-81870194000', 8, 1);
    T('990496509554.8008249501', '577320077.93845345763685481007', '989919189476.862', 15, 3);
    T('18.8129965065769536', '974.77588184361520', '-956', 3, 0);
    T('8097707287911218333.0', '74436565330921439167.5770882918467', '-66338858043010220834.57709', 25, 5);
    T('39.9254440', '91610880662.314', '-91610880622.4', 12, 4);
    T('28397539.24768023', '438688231094.831', '-438659833600', 10, 3);
    T('33457073562488610.4387458', '557459612537766.933932', '32899613949950844', 17, 0);
    T('1274522039001.750075770093931119', '47852236156.8888741', '1226669802844.861201670093', 25, 3);
    T('4522871126.785366139055252', '611044844928.33153543687', '-606521973801.54616929781', 23, 2);
    T('82296.94646', '8831910875.9438971196537994', '-8831828578.997437119654', 22, 0);
    T('8293328950.21917777329352', '2195497145.0', '6097831805.21917777329352', 31, 5);
    T('93097.4736844', '452999.639758724312929', '-359902.166074324313', 19, 3);
    T('6238267972844204097.86776809835657', '10312536926571.288319565126170355', '6238257700000000000', 8, 4);
    T('405906.855', '179735904316.988446893771057', '-179735498410.13344689377106', 26, 0);
    T('9435163538.0', '583596525684653.33844735376', '-583587000000000', 6, 1);
    T('8872500263369598095.886886102626987924', '86238584.10390058', '8872500263283359511.782985522627', 32, 5);
    T('8.95506731876419', '27.46213170561', '-18.50706438684581', 31, 1);
    T('7736822595186723877.97696538628781', '5087.45781', '7736822595186718790.51915538', 27, 1);
    T('1285.1163', '481410582943.2764742527621467477', '-481410581658.160174252762', 24, 5);
    T('50453778906253127.0458948340390338621', '368767207385306.9010', '50085000000000000', 5, 1);
    T('716.4941671761544385', '95998711710499632991.5712080617', '-95998711710499632300', 18, 6);
    T('6489.382879964327259057', '2.930946267148101967', '7000', 1, 2);
    T('59082.227713514', '199.857', '58882.370713514', 22, 4);
    T('147823253408170.53959327317282308338', '64006781197163.848627033', '83816472211006.7', 16, 0);
    T('15540541319768.511', '525.780198693675', '15540541319242.730801306325', 39, 6);
    T('53471101267370473.887754', '171693.4064708471661490', '53471101267198780.48128', 22, 4);
    T('914353713140893.542765', '20723899.323467308973', '914353692416994.219', 18, 3);
    T('601011635114034815.774886921', '2074376.491057105787', '601011635111900000', 13, 3);
    T('8054.087', '6461627664676317075.604', '-6461627664000000000', 10, 1);
    T('684176334.669665', '3003447785.6', '-2319270000', 6, 1);
    T('205762351572070.10516422647278425', '331.55692', '205762351571738.5482', 19, 1);
    T('88742000865849423.60537777873936', '3793747.394722823', '88742001000000000', 8, 4);
    T('465.9370', '53412740093838030.852648798796746805', '-53412740093837564.9156488', 25, 0);
    T('584211168029481.14759', '321141639214791070.1255126856786', '-320557428046761588.98', 20, 4);
    T('779736249.1', '71361782330791340748.833', '-71361782330011604499.733', 29, 2);
    T('3453.386039582692717384', '368341283268.7', '-368341279815.313960417307282616', 33, 0);
    T('7.330994727592719561', '50651.164', '-50000', 2, 1);
    T('9359514215479867974.0', '4385836865.3', '9359514211094031108.7', 20, 2);
    T('59571054597288175.2940890', '35805208.55304668', '59570000000000000', 4, 3);
    T('7656645.88895', '7790923914.81664906739', '-7783267268.92769906739', 30, 1);
    T('80936987146.8', '9307006949.52552913', '71629980200', 9, 5);
    T('5253843.807', '8106859659885926392.0587024', '-8106859659880672548.2517024', 29, 4);
    T('76902488100.16944782', '870174.2', '76901617925.96944782', 23, 3);
    T('96.86083946690367', '9833038.67046415351311861032', '-9832941.80962468660944861', 24, 2);
    T('49188502419294.8416331111391', '833565570851.9514745', '48350000000000', 4, 1);
    T('82.25', '10120525867020013511.2393328', '-10120526000000000000', 8, 4);
    T('6441845.46816633407687', '592220844872.908870185830', '-592214403027.440704', 18, 0);
    T('91241220034544025.5281', '62188230925436013200.73985045137', '-62096989705401469175.21175045137', 36, 1);
    T('39613563373494.0', '3872.140296', '39613563369621.86', 17, 0);
    T('5.375101405012956248', '5796046216.8922283745324', '-5796046211.517126969519444', 25, 3);
    T('9162747029116.119', '346683428.305530734755296535', '9162400346000', 10, 6);
    T('9950.7369405164255', '1729.4806', '8221.2563405164255', 34, 6);
    T('835942635.717095640353485987', '79255002853244.4979207191353987', '-79254166910608.780825078781912713', 32, 3);
    T('1143863476492109.64531041133', '9824.723845384246660', '1143863476482284.92146502708334', 33, 2);
    T('20785.57811420374321', '83359030160816763501.38292096175849641458', '-83359030160816742715.8048067580152864146', 39, 0);
    T('147078.17586609297563333', '54.34100051', '147023.834865583', 16, 6);
    T('8606572895680.50454', '547200777.78141235', '8606025694902.72312765', 21, 2);
    T('61857110133882747601.52717232475006', '104.829658', '61857110133882747496.698', 23, 0);
    T('5652397175297693.872844281824', '35875.796', '5652397175261818.076844281824', 29, 4);
    T('7128226716.7061246936', '58684982812.58381058853591477317', '-50000000000', 1, 4);
    T('927629760778.6056875861', '47340.9873', '927629800000', 7, 0);
    T('26542071249797213.9843584092279', '562147117838011860.72837416458572', '-535605046588214646.74401575535782', 37, 2);
    T('3770828730.06854448548911629', '336773268.8165979248448491362', '3434055461.25194656064426', 24, 1);
    T('9.0974768986294011', '2558267879711983.47059777', '-2558267879711974.3731208713705989', 40, 2);
    T('161996857097.2288786131498', '3012755449462579092.06094569217035263', '-3012755287465721994.83206707902', 31, 2);
    T('426529.071', '108681.38634143293372', '317847.68465856706628', 29, 3);
    T('4.700646194022', '9824117045304.467888364206998476', '-9824117045299.76724217018499', 27, 2);
    T('414900120236.70916440', '62029582.887070886552', '414838090653.8221', 17, 0);
    T('4833777839922584943.956311023341063571', '9799367.85', '4833777839913000000', 13, 4);
    T('444513005095268510.5637781456', '9846197.314759615', '444513005085422300', 16, 6);
    T('4061.649450', '718.94478094624882', '3342.70466906', 12, 2);
    T('72990152206194920565.95106398917', '986.0', '72990152210000000000', 10, 4);
    T('934786.45636528653505409', '990988023050365586.7346871', '-990988023100000000', 10, 0);
    T('764.4384024779105', '203.9650', '560.4734024779105', 34, 2);
    T('85622594513264190503.828', '747242593637.79469100874533', '85622593766021596866', 21, 5);
    T('350771636889.8119447', '509168.188363329', '350771127721.623581371', 40, 5);
    T('53681401647.3011943180', '34752.486705', '53681366894.814489', 17, 3);
    T('6061132994346.6499560674743200358', '22.8506569179357', '6060000000000', 3, 4);
    T('9.248', '0.73776', '8.51024', 33, 0);
    T('521265372483.883332943115709', '24282.42553551537936', '521265400000', 7, 0);
    T('63869152134582.7782569880936', '4.3049', '63869152134578.4734', 18, 0);
    T('5349645.9340780829727', '5379579364350814.90240', '-5379579359001168.9683219170273', 31, 3);
    T('53558478379083738525.010959443936352736', '9.96', '53558478379083738515.05', 22, 3);
    T('13910271.2766000', '246658889469004656.0545517532973565156', '-246658889455000000', 13, 1);
    T('2.754234368448', '550299549453133.85050999', '-550299549453131.096276', 21, 0);
    T('396222914.805036', '991060781276786598.633', '-991060780880563683.827', 21, 1);
    T('7597262.107092735390139', '408184596259642478.65196507424173298984', '-408184596252045216.545', 21, 6);
    T('352738.68', '120.71832125322998379800', '352617.961678746770016202', 28, 1);
    T('9729943948.930303714', '2651543275656819267.81224060', '-2651543265926875300', 17, 1);
    T('76490561724.8720071007952429618', '189155714065511.40611502870715', '-189080000000000', 6, 0);
    T('982333.0978638140', '118865069169692167.0', '-118865069168709833.902136186', 33, 3);
    T('50124.7', '3845578058.2192917617329718', '-3845527933.51929176173297', 24, 4);
    T('18873973503085643498.5100878705585841', '7436983494.319154', '18873973495648660004.19093387055859', 34, 2);
    T('14971864161265.61366759919879973712', '9435857739293.225474185', '5000000000000', 1, 3);
    T('42382.0', '703174561529464906.0505', '-703174561529423000', 15, 6);
    T('698471447425.409', '6138324457.287792981562', '692333000000', 6, 1);
    T('8021913.23366674869372', '989750952353723.9950040902816287811', '-989750944331810.76133734158790878', 32, 6);
    T('267981888886466376.2838881888379086', '189835.6435591', '267981888886276600', 16, 0);
    T('437.89222009473', '32.82', '405.07222009473', 33, 5);
    T('98626908.0', '960643902062.26467739', '-960545275154.2646774', 19, 3);
    T('30287.68994', '21231522826.892448254982', '-21200000000', 3, 1);
    T('9.0025', '121746005.64797715694191273', '-121745996.64547715694191273', 30, 1);
    T('829375190692818.000630771128', '76063695831661.181701221635', '753311494861156.81893', 20, 6);
    T('23305796.693558755230880073', '683.67', '23305113.023558755230881', 23, 2);
    T('9050825845146.4122136040385190215', '74514088364237188.17285', '-74505037538392041.760636395962', 29, 3);
    T('8195885.68097547', '889.3266767722871610', '8194996.354298697712839', 29, 5);
    T('917.0779585870', '103032.54928204402184582', '-102115.4713235', 13, 0);
    T('16409.888358724', '77074.32589946', '-60664.437540736', 35, 5);
    T('97069238172276.29170539414333415650', '81931651129945400661.0', '-81931554060707220000', 16, 1);
    T('5026.7673584088134148', '3894903522.163', '-3894898495.3956415911865852', 33, 5);
    T('80294.3141463800242583', '419.066337970100925', '79875.2478084099233333', 36, 0);
    T('93261.586220297507', '853529127.993869', '-853435866.407648702493', 31, 0);
    T('67.0', '710296263468.4', '-710296300000', 7, 3);
    T('93009830261693436.4324615810437494957', '986578.3013', '93009830260706858.131161581', 27, 3);
    T('122986780687.47678528240630011265', '893727.219772434153298728', '122985886960.25701284825300138465', 33, 5);
    T('642.96617908606531', '536048577952313047.6395912429969961368', '-536048577952312404.6734121569316861368', 39, 0);
    T('127916799454544341.2018', '66191662569977159342.0', '-66063745770522615000.7982', 34, 5);
    T('2438367070375.6109791514318512646', '736119395842168.51880323253', '-733681028771793', 15, 4);
    T('70065862308953.10072547', '9829288330.74238', '70056033020620', 13, 1);
    T('9205.6835251597076641708', '81964390590930418088.178508', '-81964390000000000000', 8, 1);
    T('9815101857945852606.1751724', '5282004517562173.2798276838502910', '9809819853428300000', 14, 5);
    T('92774260.23993643', '138553757291.93694368048', '-138461000000', 6, 4);
    T('98.00452276500961710439', '936325.62559829280807602', '-936200', 4, 1);
    T('4080233518723.4449089305467', '533190254132.30345157', '3547043264591.1414573605467', 38, 2);
    T('84307268101554.0', '7.95659798', '84307268101546.04341', 19, 0);
    T('424826135096.0342499174', '304566651912.174', '120259483184', 12, 5);
    T('8377.7946331432335873', '79170154770725294.87686', '-79170154770716917.082226856', 26, 1);
    T('4.8690175113641', '6896.764', '-6891.8949824886359', 26, 0);
    T('434136.897949981989873', '92.8630254068227', '434000', 4, 4);
    T('8.0', '481419672994466945.3099019715119994527', '-481420000000000000', 6, 4);
    T('782811827.98499033450', '2856.26476365628104052', '782808971.7202266', 16, 3);
    T('721350039627.137837451045352627', '713242270060300201.772981367', '-713241548710260580', 17, 0);
    T('5834633893149.2153314145', '16712.5', '5834633876436.715332', 19, 0);
    T('776853850973237.82', '778.8', '800000000000000', 1, 2);
    T('711007188983427434.596951224896103', '8770344320.667', '711000000000000000', 5, 3);
    T('9545650983.0716242248211206', '8332701028469787163.7818813', '-8332701018924200000', 14, 0);
    T('559829.221014', '18560854.76348504', '-18001025.54247104', 17, 0);
    T('395861477.506', '208.632828042', '395861268.8732', 13, 0);
    T('7182734178236471490.9408492320616699381', '13198577293060013.887', '7169535600943411470', 18, 3);
    T('52677.2771', '71362999774959803.4591964208631818', '-71362999774907126.182', 20, 4);
    T('6057505378351798.5872985', '73593332832905.22153846299254', '5983912045518893.36576003700746', 38, 4);
    T('477197.72', '6014141422.18', '-6013664224.46', 40, 2);
    T('4730069508488940617.655604201', '198764355104800418.5823363731', '4531305153384140199.1', 20, 6);
    T('47517900.322292924494393', '5233749.61866212868', '42280000', 4, 5);
    T('2835291871265414.05144', '28061606093490326.966', '-25226314222224912.91456', 40, 5);
    T('582605569.5216', '322207882.7029349439279415714', '260397686.8', 10, 1);
    T('6272466511154176.79026797', '699.21369746407398', '6272466511153477', 16, 3);
    T('90.8197808205177', '997222354788830784.18594619397', '-997222400000000000', 7, 5);
    T('1301336798829694933.19374124', '9240081793966929681.0', '-7938744995137234747.806', 22, 6);
    T('76438076169134.09290085', '873407047621063.2729467806723', '-796968971451929.1800459306723', 34, 5);
    T('87305011783.06285693625988', '410349117030667.488757', '-420000000000000', 2, 3);
    T('354459492810.98', '6869746.496', '354452623064.484', 26, 3);
    T('64430.60283972', '52.632323391', '64377.970516329', 19, 1);
    T('75.688672610785367', '89984.551', '-89908.862327389214633', 25, 6);
    T('6644855867587465.183', '8623402518926067888.925191316639567', '-8616760000000000000', 6, 5);
    T('1709109582243.8832760934663', '3137314150.243445', '1705972268093.63', 15, 1);
    T('892858989831508.892944991513', '48812116395156399958.42828408', '-48811200000000000000', 6, 6);
    T('8.8589238675626726534', '1890132468158221398.20873452559', '-1890132468158221389.349810658', 28, 6);
    T('47405436.5', '6819.51477394', '47398616.98522606', 30, 6);
    T('8065.0', '46045867743458103.3377029', '-46045867743450038.337703', 23, 6);
    T('40938178.18881232722413', '8.3092240615528', '40938169.8795882', 15, 3);
    T('706305.2', '663334.1658', '42971.0342', 34, 4);
    T('57931021702286616094.7', '275531937944.748446614251315751', '57931021426754678149.9516', 24, 2);
    T('84825396261.57092448', '830685212.0', '83994710000', 7, 3);
    T('171310561.378818171557364', '32307.0479968', '171278254.330821371557364', 36, 6);
    T('540767882.13196967597', '369828150121.5249961', '-369287380000', 8, 5);
    T('7160203470767409.585838569003', '29283595410.31227783269746045', '7160174187171999.27', 18, 3);
    T('785365151.184018', '180520350.1024765', '604840000', 5, 3);
    T('76551.1307586362365104461', '37132402498869279.75340307670284', '-37132402498793000', 14, 5);
    T('1083198153904737.4', '766406122505693.686939314404', '316792031399040', 14, 3);
    T('887087848988.34363', '532315.30206965', '887087316673', 13, 5);
    T('2628.72640322559587709', '6301.86655347443', '-3673.14015024883412291', 21, 4);
    T('2208203618.1', '26201336719064.02222150118', '-26199128515446', 14, 6);
    T('69426686.8971433196771637512', '99871003.99686076', '-30444317.099717', 14, 4);
    T('29735.0', '649070714003108.3879', '-649070713970000', 11, 2);
    T('4916091674733.55671860874', '9903475872551.098', '-4987384197817.54128139', 21, 4);
    T('5285035478361925960.211504479620', '919.159421128', '5285035478362000000', 13, 5);
    T('4004.4317654824', '4409.313861496659', '-404.882096014259', 21, 4);
    T('675.06086521616881050409', '60242478207.223', '-60242477532.16213478383118949591', 38, 5);
    T('3.893527982283', '7084036380.727229645618570011', '-7084036376.833701663335570011', 38, 0);
    T('64136980248.57666105610', '444615362.82710533537635', '63692300000', 6, 1);
    T('75898786090939366577.0489214813285238', '22785.78', '75898786090939343791.2689214813285238', 37, 4);
    T('7.3', '8388.33399', '-8381.03399', 36, 5);
    T('4729.167930', '99.841400913498569', '4629.326529086501431', 32, 5);
    T('9.880236212058017770', '582353486670.1', '-600000000000', 1, 4);
    T('61902586012399186144.5921433072214', '434147.8978170780235', '61902000000000000000', 5, 1);
    T('4.577135', '967808616041073224.103718057278254502', '-967800000000000000', 4, 6);
    T('25286756863221.893945532202925', '1494886722673815835.4089051', '-1494861435916952613.5', 20, 4);
    T('2380406107181102023.68428126287817003672', '43006065480505934845.939909009003', '-40625659373324832822.255627746124', 32, 1);
    T('34.50522290212914', '12.14', '22.3652229021', 12, 3);
    T('356.0', '177.9', '178.1', 7, 3);
    T('6439920111.653430475812875', '5.10886419', '6430000000', 3, 1);
    T('7.04364935955149', '23.749774980216767479', '-16.706125620665277479', 24, 4);
    T('9.47192458346026916', '43134136663026.566408275608183', '-43134136663017.094', 17, 5);
    T('145371648883.8399224356', '9343958552.3524633395153949102', '136030000000', 5, 0);
    T('9933057.68449764081002613983', '260.71872520083043', '9932796.96577243997959613983', 29, 3);
    T('569966139482048.964641159', '24374013239947779.0', '-23804047100500000', 12, 3);
    T('23.241463185296606405', '50244.0', '-50220.7585368', 12, 4);
    T('2555368963.4103769287177184', '69231.3558017856', '2555299732.0545751431177184', 31, 6);
    T('722940692751633703.8801805', '425066.01032323382881207', '722940692751208637.8698573', 25, 4);
    T('7507.35', '54934433.116', '-54926925.766', 19, 5);
    T('8782.3002309178681416175', '92566702865641942802.023538117768825957', '-92566702865641934019.7233071999006843', 36, 5);
    T('10216509230673827364.2694715949748678', '1.637324907', '10216509230673827362.633', 23, 0);
    T('1768477427863644.9401', '179154.43765738', '1768477427684490.51', 18, 2);
    T('932470246838.02582', '70425110671008035150.820171247856575458', '-70425109738537700000', 15, 1);
    T('42539.27426739', '9925666778.4936787842579', '-9925624239.2194113942579', 27, 5);
    T('915427.14478', '8766856757.6', '-8765941330.45522', 36, 0);
    T('10487609840840119.9', '5564010.81', '10500000000000000', 3, 2);
    T('2623291618.49461931841651809', '2076.786539', '2623289541.70808031841651809', 38, 2);
    T('496666.9654697', '3943601663931.145870928', '-3943601167264.180401228', 39, 1);
    T('60401374258794.429378896', '23570686595.764', '60377800000000', 6, 5);
    T('50183558.026', '230770434806204.0', '-230770384622645.974', 19, 2);
    T('113.946129682129210107', '6286324036.6144490889', '-6286323922.6683194067', 20, 2);
    T('756.059085458806133', '9263572.361928873045', '-9262816.302843414238867', 28, 6);
    T('88881894.0', '111089864631517.018157133345', '-111089775749623.018157133345', 33, 0);
    T('787671662811972230.58976350', '333744623246217.0', '787337918188726013.5897', 22, 3);
    T('7270715.34191953395810', '85409528497990105222.960044728878508985', '-85409528497982834507.6181251949204', 33, 2);
    T('9489232505509438.2231480989812', '48812608.112640', '9489232460000000', 9, 6);
    T('95828640171240047407.922', '947556343731336005.897610080280', '94881084000000000000', 8, 2);
    T('72007868.35', '6842128290576022.2160132326', '-6842128218568153.866013', 22, 1);
    T('654.12972177900309564815', '793954.83', '-793300.7003', 10, 6);
    T('8045762007.5056138551', '220.23098785', '8045761787.2746260051', 32, 5);
    T('432390351988177.6280235246065143', '103838655279.08425417092551891', '432286513332898.54377', 20, 0);
    T('2256323877.383471817842117098', '7969894.91628110', '2248353982.467190717842117098', 35, 0);
    T('93182310933730090802.370', '44.0', '93182310933730090758.37', 27, 6);
    T('421.7', '901850101504094910.12564', '-901850101504100000', 13, 4);
    T('27.332197', '3550399952.1350041345', '-3550399924.8028071345', 36, 6);
    T('7981588934.30817365642332', '6661731818.4224314877002', '1319857115.88574216872312', 27, 5);
    T('8931079930874878101.78104028244444724216', '9.8853098531970449519', '8931079930874878091.89573043', 27, 0);
    T('81238445.8316983', '302784105787845906.00552', '-302784105706607460.1738217', 31, 2);
    T('85367526509645.973046', '1478407946048548021.392593248046511466', '-1478322578522038375.4195472480465114', 35, 1);
    T('69691.2', '7034420795007.95120507944509958472', '-7034420000000', 6, 2);
    T('5512719.60104792484507968', '427789148982330601.893178835', '-427789148976817882.2921309102', 28, 4);
    T('982.920641010', '8185.55685846', '-7202.63621745', 24, 4);
    T('34.950142186', '308782.090328', '-308747.14019', 11, 3);
    T('5728.337425427', '323436088025.273401544964', '-323436082296.935976117964', 40, 5);
    T('42052042287.7866', '427870.61146568379', '42051614417.17513431621', 35, 5);
    T('3100546058.9594', '69139106938.83712323733203054752', '-66038560879.877723237332031', 26, 0);
    T('8.232', '85424.0646794761972411102', '-85415.8326794761972411102', 32, 6);
    T('4509331732.4657526416505096', '4.341247', '4509331728.1245056416505096', 34, 5);
    T('97870.747497846', '63.15581319657', '97807.59168464943', 20, 3);
    T('5913288955.15357597037331269', '74935925.4', '5838353029.75357597037331269', 29, 3);
    T('5437774705252.45825', '110043.6498', '5437774595208.80845', 18, 0);
    T('643291.736', '42959559649371909.1800153', '-40000000000000000', 1, 1);
    T('1.5962492167873', '134901037253.0421077701', '-130000000000', 2, 6);
    T('545407293331408.50234526', '6619164.7657631557609', '545407286720000', 11, 2);
    T('5590995.4025', '8441650814086770.007953', '-8441650808495774.60545', 21, 1);
    T('7.3805690375128', '4.556', '2.8245690375128', 29, 1);
    T('2481464.05', '45261981.400901051283394', '-42780517.350901051283394', 27, 6);
    T('28.5498116999', '28154852.95611466407867893014', '-28154824.406303', 14, 3);
    T('66295631024417565.93', '88882977811.1858285585184365', '66295542141439754.74417144', 25, 5);
    T('9761708526033.0', '610444054282485631.55837558806694564', '-610430000000000000', 5, 6);
    T('1163568.0', '9.6481358835300', '1163558.35186411647', 40, 1);
    T('741585.63936', '36735637439284.25', '-36735636697698.61064', 27, 2);
    T('6714.0', '5350001094288883201.0', '-5350001094200000000', 11, 1);
    T('5734214700.873916716522881558', '75997253550058.4354873777', '-75991519335357.561570661177119', 29, 3);
    T('9572083694844013677.4', '919005.04449271788', '9572083694843094672.3555072821', 29, 3);
    T('8.019969515', '468296154206145040.6057', '-468296154000000000', 9, 6);
    T('3734194233342565948.5954355', '1849473.1032', '3734194233340716475.4922355', 29, 1);
    T('499120355072002.0240868452', '2966376142598560.59523005613110235853', '-2470000000000000', 3, 6);
    T('15332921735922.48', '4555044.94583625', '15332917180000', 11, 1);
    T('893863.24419917973', '3188763174793.3876', '-3188762280930.14340082027', 28, 4);
    T('51.94673185648', '32586.2710322758', '-32534.32430041932', 33, 2);
    T('828816.14956821030093995484', '13392745154989982.81696745819', '-13392745154161166.66739925', 25, 4);
    T('18284.9', '33000087522710.53509337', '-33000087504400', 12, 5);
    T('4702785.335896', '703487825461.2577205', '-703483122675.9218245', 25, 2);
    T('51596791385.194715494', '5176413682.53100926798', '46420377702.66370622602', 23, 1);
    T('5782587484003410424.984503', '6806258.609', '5782587483990000000', 12, 1);
    T('94975.089086370', '4829912808936199237.7692745032808518181', '-4829912808936000000', 13, 6);
    T('4767.48788295781898', '8714663.15706345332571972837', '-8709800', 5, 2);
    T('46328.728680844', '542793917824.330948649520074257', '-542793871495.6022', 16, 1);
    T('7406.6', '98901680452.10', '-98902000000', 5, 5);
    T('692184085795405.541917899437', '37351123.480939870', '693000000000000', 3, 2);
    T('18881478.0', '72587993847.05524370140367014', '-72569112369.056', 14, 3);
    T('7.0', '6952269659511857914.0', '-6952269659511857907', 24, 3);
    T('291.319933009', '909720537.502', '-909720246.182066991', 26, 0);
    T('548309492213818.12453505', '5366880.6076539', '548309486846937.51688115', 26, 0);
    T('579356940214947247.36', '1356294.6170458159577737', '579356940213590952', 18, 3);
    T('17147163642054.391', '1788110040414892003.5505', '-1788092893251249949.2', 20, 0);
    T('6549.75', '4.9353890', '6544.814611', 33, 3);
    T('937033294579560283.3', '2281625145.9689893684797216218', '937033292300000000', 11, 4);
    T('729810738394.418', '47585845.183301816', '729763200000', 7, 4);
    T('788.1678580820', '293633.5630869018554505', '-292845.3952288198554505', 39, 2);
    T('285120562158818.498104699420', '8618778014454.4747789144544476', '276501784144364', 15, 3);
    T('89654420097159486.854690541050976473', '47098782.08', '90000000000000000', 2, 2);
    T('637166054501.28476896687322640645', '445844205475.5552382122', '191321849026', 12, 4);
    T('422034.32899803268', '35006.9', '387027.42899803268', 36, 3);
    T('4821513851374.54', '9050951.823945829961', '4830000000000', 3, 2);
    T('3162996.581030', '1930268.7389817744120528848', '1000000', 1, 4);
    T('8313250315590185.683116', '52039135515.21842881512552517488', '8313198276454670.46468718487447482512', 37, 0);
    T('34368594810761.90499239', '391687791392527265.598771', '-391653422797716503.69377861', 33, 3);
    T('462577995780617.4909310146594187854', '24847037291620811.5918679784948622', '-24384459300000000', 9, 5);
    T('1335860262.6', '316141987749087.36', '-316140651888824.76', 38, 1);
    T('2.675814982644045149', '9732.65781817568539693636', '-9729.98200319304135178736', 35, 3);
    T('8766351473.05', '11809.4519859302066625479', '8766339663.598014069793338', 25, 0);
    T('919756694291856199.78164012773295918934', '1470570.78442277565267207', '919756694290385628.99721735208', 30, 4);
    T('47631790468008231208.650604043', '344289.62178', '47000000000000000000', 2, 1);
    T('515442.985385429', '5683.8490', '509759.1363854', 13, 6);
    T('62024.35463690451799', '13166618.4252242134515241', '-13110000', 4, 3);
    T('27.88', '6138674485797654.4601999476176059083', '-6138674485700000', 11, 1);
    T('0.05134148217435', '55579.365171475775834', '-55579.313829993601', 17, 6);
    T('17842331.7650588997', '68847523692544649074.01', '-68847523692526806742.2449411003', 33, 4);
    T('3825442.073057752639', '75202406993444.83724432', '-75202403168002.764186567361', 34, 2);
    T('745691306.3937', '163865035972835557.832339297813', '-163865035227144251.44', 20, 0);
    T('9046303589803522.703168919394', '425884111977080401.52828412316655046516', '-420000000000000000', 2, 6);
    T('4391.06', '28197.5715818', '-23800', 3, 4);
    T('826953985023.38523494680564019989', '719010741136982041.936', '-719009914183000000', 14, 5);
    T('37947219769.93518421', '366431406809.864039012860318516', '-328484187039.928854802860318516', 37, 2);
    T('178965637746375.554754019036309', '722.063700036874719096', '178965637745653.491053982161589904', 40, 3);
    T('15578232863.06028791', '2482133.6089977490749', '15575750729.45129', 16, 6);
    T('56708117826486.6', '695682092411844.89600908970', '-638973974585358.29600909', 23, 6);
    T('4058883473588.3', '73422734798.3', '3985460738800', 11, 0);
    T('73060507632.084', '216372689824350.0', '-216299629316717.916', 35, 1);
    T('554995.11303541', '910801832367600.8745367778560', '-910801831812605.761501367856', 34, 1);
    T('762.67996761072', '653381089228.95226905672', '-653381088466.272301446', 32, 6);
    T('78575001937910.18098511292669891', '20932.527363419895023', '78575010000000', 7, 2);
    T('5.42200547', '9764050952.1726485769848', '-9700000000', 2, 2);
    T('572668168206.10135520240007240199', '4920248936874103.20291894719912719', '-4919676269000000', 10, 4);
    T('51464869519168922.58101870691934', '651239.914', '51464869518517682.66701870691934', 35, 4);
    T('225930312391536.343405547625019064', '362441.0732', '225930312029095.2702055476', 25, 4);
    T('5946.23584834614995006', '1.521322618225', '5945', 4, 5);
    T('121.94', '1462891.8821270', '-1462769.942127', 29, 3);
    T('483851805429.2978450195787', '2750586085609.7756764228657923', '-2266734280180.4778314032870923', 36, 2);
    T('69.2692', '7409439825897493137.0238763861623591', '-7409439825890000000', 12, 1);
    T('14684109997988824.354069', '98.1934163524', '14684110000000000', 8, 5);
    T('592436237415547433.814385330810875', '674356303140088753.68239', '-81920065724541319.868004669189125', 36, 6);
    T('7142765.52', '42702551405506287.4604018', '-42702551398363521.9404018', 39, 0);
    T('50686273476505.0', '46331913335.576494683112948', '50639941563169.423505316888', 26, 2);
    T('10005.152072581972017', '63.31545', '9941.83662258197', 15, 6);
    T('91658.32039520721', '47037716649675077.93970999101', '-47040000000000000', 4, 3);
    T('10910069268242622.86247077025306', '2649619958068.8096528560300628', '10907419648284554.052817', 23, 3);
    T('740159133914127945.664885840510754241', '920389828084768049.19743', '-200000000000000000', 1, 3);
    T('83941475044820.34705372975380102', '76944911111564.3625583623149404356', '6996563933256', 13, 6);
    T('7284404431032191.76059659154372577', '1953421.590983072295', '7284404429078770.16961351924873', 30, 6);
    T('666823.8250536846737478148', '17751667570754934134.533299223689', '-17751700000000000000', 6, 6);
    T('4.45729127899131', '903069751733048.49675800', '-903069751733044.03946672100869', 35, 5);
    T('36157466457811.36911382240495659', '862939707571019629.85939', '-862903550100000000', 11, 1);
    T('36611824227963.80810749451240709634', '587424758152774728.65157336', '-587000000000000000', 3, 5);
    T('9.9503194707717267', '6744856779768003.9805', '-6744856779767994.03018', 21, 2);
    T('7990358854689703.31091724816922356635', '9953951120.4169033015940743', '7990000000000000', 3, 5);
    T('45130025420274877.27110111865279729', '75725214.443979859390184596', '45130025344549662.8271212593', 27, 4);
    T('8896862439349380.391623364992340134', '6768372839211.8', '8890094066510168.592', 19, 5);
    T('64.0751597512030729', '884238261952.63', '-884238261888.5548402487969271', 35, 4);
    T('13561212762.2533304', '676524395.775454232', '12884688366.5', 12, 2);
    T('618699756658.398', '326991.3509145173', '618699429667', 13, 5);
    T('78196680.1245907909235353', '8143653930093423871.736', '-8143653930015227000', 16, 5);
    T('60458168.485291915600199919', '38591034.25341', '22000000', 2, 6);
    T('73608.754668415', '5106.533912386152008609', '68502.22075602884799139', 22, 6);
    T('6371.81', '53896905732125.180774', '-53896905725753.4', 15, 0);
    T('520299.1', '9609928.801836', '-9089629.701836', 24, 6);
    T('32385.4759708', '55534976.0807', '-55502590.6047292', 40, 3);
    T('4471503077.5255796932', '4546651058053614908.4335321985148', '-4546651053582111830.9079525053148', 35, 6);
    T('15085.5779476587467', '84147265.0296472555', '-84132179.4516995967533', 24, 4);
    T('184675782513.589902532684461', '94946.77199589039340152', '184675687566.8179066', 19, 4);
    T('393966453738.166320983882662759', '2426.868652898591', '393966451311.297668085291662759', 31, 2);
    T('3446.32440901', '8289209.0', '-8285762.67559099', 18, 6);
    T('39176603386320182471.2297248', '91.3035', '39176603386320182379.9262248', 35, 2);
    T('159069783603.86459256474067775', '4625157694465.1642188092425041', '-4466087910861.3', 15, 4);
    T('424801.5279753380611', '8.01991502', '424793.5080603180611', 32, 4);
    T('38393737970147911.63097667581996796', '54855835960699.201355', '38338882134187212.429621675819968', 33, 0);
    T('4673419038915439.97617243', '45017927766.03418275341', '4673374000000000', 7, 3);
    T('7905.753345713', '1714798.2464671002346125912', '-2000000', 1, 0);
    T('74941012422.02221914710473906705', '2067612403776.6', '-1992671391354.57778085289526094', 30, 3);
    T('31669.9187616', '37725.47601890047264339254', '-6055.55725730047264339254', 39, 1);
    T('9853323463952202.04', '274561854607.2086283072735', '9853048902097594.8313716927265', 38, 4);
    T('51253687767.5467334', '3988203.6926020828', '51249700000', 8, 2);
    T('9.7', '35624298.18777708854252321863', '-35624288.48777708854252321863', 29, 5);
    T('96856053.873865', '4591.4663', '96851462.407565', 26, 4);
    T('72.821752893586', '480603024452077103.0', '-480603024452077030.178247106414', 33, 5);
    T('670536.862935142995084', '8060613837936321.55254455850767928', '-8060613837270000', 12, 5);
    T('928514.5', '9785652562605.95884597896672546678', '-9785651634100', 11, 6);
    T('56887546456000.5419', '382724636342150.66331873796512016975', '-325837089886151', 15, 0);
    T('571812.3066425293780', '6.0', '571806.306642529378', 28, 3);
    T('7.590717685861512804', '613186816150133107.725646584', '-610000000000000000', 2, 1);
    T('25050.745531188946357373', '38781624.4184037812501', '-38756573.6729', 12, 5);
    T('8792513396.454847', '635183186.681', '8157330209.773847', 23, 0);
    T('77.6393179849255517', '49738343764429787367.0944042179149593489', '-49738343764429787300', 18, 5);
    T('273933.264309045459716', '17951075.6269899154431559', '-17677142.36268086998344', 22, 0);
    T('179710.90327693751744678', '960744946468241.418868004568942', '-960744946288530.51559106705149522', 39, 2);
    T('15810235481206.73130398426160', '911085168063238320.03273415', '-911069357827760000', 14, 3);
    T('524982897926518.170706707919781', '49742525053.056055595692851752', '524933155401465.114651112227', 27, 5);
    T('301970695206078.1748270048528', '22612743656192534286.683095330334888476', '-22612441685497328208.5082683', 27, 1);
    T('613456344173.0', '280308562540074.219235012140061', '-279695106195901.219235012140061', 34, 1);
    T('5321.01686862953527', '1708.4212722541', '3612.59559637543527', 38, 4);
    T('14041474105619091.5498685849', '626454722491.2567058524332', '14040847650896600.29', 19, 3);
    T('4624.8292420004215', '727.42783689', '3897.4014051104215', 33, 5);
    T('3516342561730857535.8982152744886351', '800255579464.73488245333925935', '3516341761475278000', 16, 5);
    T('32661323645.91084223', '2626212917.33984966469152394', '30035110728.57099256530847606', 35, 6);
    T('46000028484752.13943887', '91415.6142927649723632294', '46000000000000', 3, 5);
    T('78.5676825', '218399.0', '-218320.4323175', 37, 3);
    T('661677685940176050.0', '4549.698359181259695851', '661000000000000000', 3, 3);
    T('66754196828740768.999564', '41286111335918644.41151024', '25468085500000000', 9, 4);
    T('456046200987758277.074', '241684960081861.3116509', '460000000000000000', 2, 0);
    T('837410296052.291045616', '61890.25207361453302', '837410234162.03897200146698', 38, 2);
    T('577152.05', '23851.07111872414833', '553300.97888127585167', 25, 6);
    T('1.7504736173712968', '561138374201.4021558327594', '-561138374199.6516822153881032', 30, 5);
    T('89792518089915728650.8835250070357616', '11.1444403415', '89792518089915728639.7', 21, 1);
    T('409656981.374111829630873817', '8717778347341.1691380947660', '-8717368690359.7950262651', 23, 2);
    T('97394858563953307.0273', '311197.29395483821884', '97394858563642109.73334516178116', 37, 2);
    T('1402821234.855558810', '312636152.555656544', '1090185080', 9, 6);
    T('5228596633.0', '37295395176017033849.285350782486615839', '-37295395170788437216.28535078248661584', 37, 4);
    T('194753.39943512876413', '5145404004293.59825109320005365373', '-5145403809540.19881596443592', 27, 1);
    T('636803591143781.573513457', '21806866074.48403823369283', '636781784277707.08947522330717', 35, 2);
    T('90500167941678.818562539364778666', '49421.903399729532625', '90500167890000', 10, 4);
    T('8557470002967609.9687989298', '244170721.0', '8557470000000000', 6, 0);
    T('1798689691620.023006229505', '90288234601882033.4192054335133150842', '-90286435912190413', 17, 1);
    T('20888681880037684.020989264425794205', '92582024.0', '20888681787455660.020989', 23, 5);
    T('8673724.1451752797143', '147753030473713597.2314618364490061528', '-147753030465000000', 12, 4);
    T('7.319664340564', '6133.909688922404106', '-6126.590024581840106', 32, 1);
    T('5142298357.0', '5582463037575.5219623739882453975', '-5577320739219', 13, 4);
    T('1129040058279403880.2113389439', '45807.97176668623', '1129040058279358072.2395723', 26, 4);
    T('91879.5792904520594620764', '65002411.9266329053147890', '-64910532.34734245325532693', 25, 3);
    T('9987.3234875400039127', '89281091345.37657162620', '-90000000000', 1, 0);
    T('1661.553', '177863125659250157.79744', '-177870000000000000', 5, 3);
    T('82986776.27406517249100', '4478.522280', '83000000', 3, 5);
    T('978920.87837523634', '9464241960599621253.53', '-9464240000000000000', 6, 6);
    T('7807332321.5916614', '22112158422323497.778758527936', '-22112150615000000', 12, 5);
    T('146885191452.409854569871803', '0.0', '140000000000', 2, 3);
    T('3415286.0729', '87.200048967591', '3415198.9', 8, 2);
    T('40226.025', '1738963026830713937.3844121286', '-1738963026830673711.4', 20, 6);
    T('4287888.0', '45539919.073148593336', '-41252031.073149', 14, 4);
    T('562095316001763.04108117266619566398', '7.94918388837603024590', '562095320000000', 8, 6);
    T('43120891.6', '2734658540944561.2300636564813', '-2734658497823669.6300636564813', 34, 4);
    T('65751922747814.97', '66375456750797.80693875181945243', '-623534002982.83693875181945243', 35, 4);
    T('71462754862.007953', '1837047.69', '71460917814.3', 12, 1);
    T('473838639611793601.495883261734381708', '855425057514693135.851', '-381586417902899534.355116', 24, 1);
    T('661243.65893147380518774686', '5668798739420021.292', '-5668798738758777.6330685262', 26, 0);
    T('23649529995599067602.144911', '4.7574305159039096768', '23649529995600000000', 12, 5);
    T('5225606982288397.9046107647513354', '760823767417329.84046616914770', '4464783214871068.0641446', 24, 0);
    T('3058742.088', '1061.55878494616551', '3057680.52921505383449', 22, 3);
    T('43057.747383548', '305938283056923.14480149307277', '-305938283013865.39741794507277', 32, 5);
    T('4082718230313983.70269797145505166542', '7.91', '4082718230000000', 10, 1);
    T('908740.643494574012966199', '8608157614975097786.3050227445106020035', '-8608157614974189045.661528170497635804', 37, 5);
    T('2197885062.9628450835737974762', '75229017267.228901133', '-73031132204.2660560494262025238', 30, 4);
    T('976753215790215822.634358087663248738', '16591348758304115062.6336451061144', '-15614595542513899239.999', 23, 5);
    T('417581.036711', '91210908716273950803.0', '-91210900000000000000', 6, 1);
    T('6572516246209.10562345', '2197345171791728514.8', '-2197300000000000000', 5, 6);
    T('8.2470387', '91.87533', '-83.6282913', 11, 2);
    T('5438.22938634553846', '89814872085.3726673', '-89814866647.14328095446154', 31, 0);
    T('9184956860573.613805838522389521', '987677580.0', '9183969182993.613805838522389521', 38, 2);
    T('2762444587.9933751057', '54.5534116510658', '2762444533.4399634546342', 33, 2);
    T('3.7211589438', '72529.69847410107', '-72525.97731515727', 31, 4);
    T('4531731087.9717', '42642477.4', '4489088610.5717', 26, 1);
    T('61367038668803016211.5579991', '3777724.35103171393', '61367038668799238487.20696738607', 31, 1);
    T('1831116.0291149615430072', '49830404694198823855.8893573223', '-49800000000000000000', 3, 2);
    T('185689.2671', '3844.1144449527', '181845.1526550473', 21, 2);
    T('908746950.14049930458646328563', '0.7116029036645', '908746949.42889640092196328563', 29, 4);
    T('84142576596252326067.787433860207955277', '50997110971499808.435484878', '84091579485280900000', 15, 2);
    T('16040.19612517125158402536', '567015269362371245.4201520066107840', '-567015269362355205.224026835359199975', 36, 3);
    T('4706946559246.642583314852810', '2339218324.0265108690', '4704607340922.61607244585281', 32, 3);
    T('982.37173813707155179418', '309278154.71468208642043', '-309277172.3429439493489', 22, 6);
    T('743642005.0', '2.492848114548852692', '743642002.507151885451147308', 40, 0);
    T('80283459.62811', '76028347841263911731.08926', '-76028347841183628271.4612', 24, 3);
    T('9501672132226.90358786920707638567', '5451951334813753085.9711576027', '-5451941833141620000', 15, 2);
    T('36.8653210610', '73575007842327330832.3440530', '-73575007842327330796', 20, 0);
    T('6865896132671.99890261983042', '285165105904888.4855784806', '-278299209772216.49', 17, 0);
    T('621396692654869293.15081401866', '6538957979.5', '621396686115911313.650814', 24, 4);
    T('6.04719736163', '520391021251944965.86527517679101980', '-520391021251944959.8180778', 25, 1);
    T('2326938368829414061.27619585', '40742.4866908976', '2326938368829373000', 16, 4);
    T('26721.594', '746236687179.0', '-746237000000', 6, 5);
    T('9035421116964.08263066516757522811', '675273224.947', '9034745843739.13563066516757522811', 38, 0);
    T('49555.0638446877', '5631697849612011.7486', '-5631697800000000', 8, 1);
    T('957.688', '50228895812.87815882748916', '-50230000000', 4, 6);
    T('9480656457.343022', '1279625.984810851186', '9000000000', 1, 4);
    T('1096483689385.0', '4737368793792356256.0', '-4737367697308666871', 39, 2);
    T('981149474720.3171427875733297766', '389784.1586', '981149085000', 10, 2);
    T('9756108405849967.77300574865204', '3732353829455.9256399230599', '9752376052020510', 15, 3);
    T('60159318447307380.95059083979', '5717984010926.234702148342632664', '60153600463296454.715888691447367336', 37, 0);
    T('8325944703460.73219764', '50414570280399719570.706162977', '-50414570000000000000', 7, 0);
    T('432.3624', '331179486701.8976944475176218', '-331179486269.53529444', 20, 1);
    T('235769842.2183526', '5956.8790519', '235763885.3393007', 30, 6);
    T('154832079099689.72481443445', '69303183111292.4', '90000000000000', 1, 0);
    T('10275676824962.1411143520545', '67012755181890353196.5', '-67012744906000000000', 11, 5);
    T('88017069301.309077737495', '44062289722163.1464655775377', '-43974272652861.8373878400427', 39, 1);
    T('977830.95', '4382933480008571080.32428026356', '-4380000000000000000', 3, 4);
    T('664827710415886886.2758', '2484166712206910113.909', '-1819339001791000000', 14, 1);
    T('5532075.6', '81037984.41772', '-75505908.81772', 37, 5);
    T('9.4', '97.8191742943', '-80', 1, 2);
    T('26328574094787.167999746857242', '74949752674.107079258', '30000000000000', 1, 2);
    T('3.088707', '22413437825337203.41015925069', '-22413437825337200.32145225069', 28, 5);
    T('1254.6', '4193854098.3079', '-4193852843.7079', 34, 2);
    T('48848201102786443.10', '48614913749993146.87', '233287352793296.23', 38, 0);
    T('58.9', '456045584447834.5679687', '-456045584447775.667968', 21, 1);
    T('762.345871', '651.14697505398271083524', '111.19889594601729', 18, 2);
    T('22369034217438650423.3798630892515', '4663.6358401337246', '22369034217438645759.744022955526', 32, 3);
    T('93285.245860', '21.5393849347', '93263.7064750653', 33, 3);
    T('6.7562547524664123', '92215959816864.047064545839559043', '-92215959820000', 10, 0);
    T('9421.39', '36793520148.027404781091265111', '-36793510726.637404781091265111', 40, 2);
    T('33414475.635281070', '83601246.254', '-50000000', 1, 6);
    T('8894357776250.98574', '925915723709859.97', '-917021365933608.98426', 39, 6);
    T('906104.287522397', '193192935506972532.939980363778622659', '-193192935506066428.65245796677863', 32, 3);
    T('65661084102559379.25259693121646175691', '8308107526.3344878722800', '65661000000000000', 5, 3);
    T('814.742324889051406039', '2886584.55254062', '-2885769.810215730948593961', 32, 4);
    T('23970750746.7244106253438593', '167210736.80620175640503314185', '23803540009.91820886894', 22, 6);
    T('124841.321384151', '2855.908', '121985.413384151', 26, 3);
    T('903578694.2', '5105043822.3537203409200', '-4201465128.15372034092', 22, 4);
    T('971253199.1695', '53395137.3209096136695364726', '917900000', 4, 5);
    T('465189106.189319471121147', '69761420.49879751504133396', '395000000', 3, 1);
    T('510.203519128141803', '8647104.99102584114643496', '-8646594.78750671', 15, 6);
    T('1622794965.1694723', '15.36787997351598', '1622794949.80159232648402', 32, 2);
    T('3944744.7', '2844832200918706495.85595959710', '-2844832201000000000', 11, 3);
    T('2305663.5308886076584884383', '213.8512', '2305449.6797', 11, 4);
    T('0.218', '82676721101.67150', '-82676721101.4535', 16, 6);
    T('25731546624.0', '67339478450344555722.6197', '-67339478424613009100', 18, 6);
    T('3333.418383697493859186', '310099923844.459017', '-310100000000', 4, 3);
    T('52.16214', '668.32696027849632069958', '-616.16482027849632069958', 31, 2);
    T('66478417314478336301.04443426754337', '977123237.9417458392691', '66478417313501213063.10268842827427', 40, 1);
    T('41603846031118.76166465834545', '190.8580393383746181232', '41603846000000', 9, 1);
    T('1597628423504576.8917622', '3.38', '1600000000000000', 3, 6);
    T('2894907917438050.78477165', '26314013247235070.6404', '-23419105329797000', 14, 6);
    T('7278494960759.4781', '386.87420110955035931730', '7278494960373', 13, 4);
    T('88464748227.8497343', '729076.5', '88460000000', 4, 6);
    T('94282931190310032.61618906089890538548', '993028596.39107844', '94282930197281440', 16, 6);
    T('966818172.37', '760862783264.93751517820640', '-759895965092.5675151782064', 36, 0);
    T('53501825961948991101.2748514309', '93341936733975256.658899132976977028', '53400000000000000000', 3, 5);
    T('4749906009.958641303782296411', '551071.736384522', '4749300000', 5, 1);
    T('5813751851264.94637', '52278117.3632897', '5813699573147.5830803', 38, 5);
    T('207891430970544970.0', '100891673.3452924', '210000000000000000', 2, 5);
    T('618112249.380144528919', '249084118.4495870989', '369028130.930557430019', 26, 6);
    T('70614144117854.999796', '658750682.8459814051', '70613485367172.1538145949', 24, 3);
    T('5459625198.517', '87611366.0', '5372013832.517', 23, 1);
    T('8096783883957918.4607398221906', '831833215.0', '8096783052124703.460739', 22, 3);
    T('93214236769133232.2871785555341', '5139806.572', '93214236763993425.7', 18, 3);
    T('816.655345545392320', '649475655150.12118476630136', '-649475654330', 11, 1);
    T('366686208814.27484425', '1.359953362708', '367000000000', 3, 2);
    T('108.2864037978866190', '99220821033393696099.7270690375', '-99220821033393695991.440665239613381', 39, 4);
    T('692911064.424', '6403040968776640893.0', '-6403040970000000000', 9, 3);
    T('872289685612.17745120454693482439', '2329512987.705083567559123549', '869960000000', 6, 3);
    T('8853.603887', '4608.534979810', '4245.06890719', 33, 2);
    T('5.93966919895', '7929518640933306996.0', '-7929518640933306990', 18, 2);
    T('366.28289207206319', '333062044076159.39', '-333062044075800', 14, 0);
    T('98944357968401.15501', '1920667049692675.5927', '-1821722691724274.4', 17, 6);
    T('37449034.0344206824423021', '257453100337391592.5907201772', '-257453100299942558.56', 20, 5);
    T('5366188751368404.375223613008', '18277272065523.640044597266753587', '5347911479302890', 15, 0);
    T('278.9537646459525197', '88473722323235.4', '-88473722322956.4', 15, 6);
    T('746130.72418338431531753', '7370.762397965', '738759.9617854193153175', 22, 4);
    T('38987973034362977.21993', '7731504033753079347.35153264320631', '-7692516060718716370.132', 22, 3);
    T('727708248115510.4103357', '37621057879.7987975949453576364', '727670627100000', 10, 2);
    T('6438221376448.0', '52894143260.7723049', '6385327233187.2276951', 21, 3);
    T('655279609577.778404511', '6.795356066934118', '655279609570.98304844406', 23, 3);
    T('1358.29003412047', '8.398', '1349.9', 6, 2);
    T('6999.36928', '30330871666091.841337', '-30330871659100', 12, 3);
    T('149003372895.0', '3204165261833.576390', '-3055161888938.57639', 30, 5);
    T('0.392681433', '45.55', '-45.157', 5, 6);
    T('5228.3845511799483097', '31053318200016.0', '-31053000000000', 5, 1);
    T('9.9323050', '23471713428390.1727', '-23471713428380.24039', 19, 2);
    T('2.877068744', '526393898953507.7', '-526393900000000', 7, 6);
    T('439965967344084.91', '846178494056408.83', '-406212530000000', 8, 5);
    T('623943323037859.58', '3940.6519681114928117374', '623943323033918.928031888507', 27, 6);
    T('8866734140216243.176887518792', '47159797001307299.4985528022826118', '-38293062861091056.3', 18, 4);
    T('173.97894203903', '132855345754747284.703', '-132855345754747110.72405796097', 37, 5);
    T('8727942.13463946', '28572993591090.34615', '-28572984863148.21151054', 39, 1);
    T('87419127.852119', '1.7202286105760220', '87420000', 4, 5);
    T('32744713055.962420156654021', '399313309666.429296003673', '-366568596610.466875847018979', 29, 3);
    T('196551573121015864.3', '7372343705.554027135', '196551565748672159', 18, 4);
    T('2681044179507177.2119', '696507697509.143227685', '2680347671809668.1', 17, 6);
    T('637339476032.055349252885003', '1902302325851058340.3052171360601198', '-1902301688511582308.2498678831751168', 37, 5);
    T('8262803921542257.212322', '477061931024.67213368135045', '8262326859611232.54018831864955', 33, 0);
    T('695482.302902827', '235018620.199593048', '-234320000', 5, 5);
    T('50.784135258060', '895284382560028.756666782', '-895284382559977.9725316', 22, 0);
    T('22556744444385140.34362586004491', '648753467754289.26387399089405174388', '21908000000000000', 6, 2);
    T('13899.15102218121', '207910684.0939010556', '-207896784.94287887439', 25, 5);
    T('720249177.24431420971651', '139489.383355681770983', '720100000', 5, 1);
    T('208.9', '952099497256063295.657987110895568175', '-952000000000000000', 3, 4);
    T('912954.40', '6499372635019.0', '-6499371722064.6', 26, 6);
    T('323910.15396187846392248171', '927875233371292560.2612738473339', '-927875233370968650.1073119', 25, 1);
    T('942391.5956385804', '918.849217458', '941472.746421', 12, 1);
    T('77795783546018749.844283718408867475', '926293444052.514', '77794857252574697.3302837184', 28, 1);
    T('7366872289.19853237924981862', '81376911676779030.45458702299129219', '-81377000000000000', 5, 5);
    T('8082587975.2882217393404031673', '915364614.7774572412263108', '7167230000', 6, 0);
    T('43624881197628955319.4441878233355956', '588980233388264.4986200465', '43624292217395567000', 18, 1);
    T('139.1449270968620', '899374021222429.0', '-899374021222289.86', 17, 0);
    T('26.952917680471', '8576845164231923.0626682693', '-8576845164231896.1098', 20, 3);
    T('73.174', '693.09086770', '-619.9168677', 20, 4);
    T('1752742.5789742709730929041', '60900989740.4', '-60899236997.82103', 16, 3);
    T('1308520219.1239042', '70684900905131.180189', '-70683592384912.0562848', 29, 6);
    T('2400683.3110485372680', '2.816812333212488', '2400680.494236204', 16, 6);
    T('668546803454655.34', '475821400103884624.3', '-470000000000000000', 2, 1);
    T('20040223807641.7864963948721', '3.82813514177', '20040223807638', 15, 0);
    T('99529760774.91896262312', '3113277401500.85892075', '-3013750000000', 6, 5);
    T('70134619900968382805.294725463590', '151.637978887608473', '70134619900968382653.65674657598152', 34, 3);
    T('2556.973', '15827979340540.88931668401506443679', '-15800000000000', 3, 5);
    T('687138033920.0', '29346547597791.957535610331535772', '-28659409563871.957535610331535772', 36, 3);
    T('86704931237068761034.9686516274813883', '706.14', '86704931237068760328.8286516274', 30, 3);
    T('3.08816', '3070343090599547094.2377664802', '-3070343090599547100', 17, 3);
    T('1.67712776548', '340999374268284.5222859056181', '-340999374268282.84515814', 24, 1);
    T('460893167002.71662704817285', '9873034225635507.60731009945643550130', '-9872573332468000', 13, 1);
    T('86924079344831172321.0', '4.89438647', '86924079344831172316.10561353', 37, 6);
    T('232616850118.70419561958136023', '18.1461923758648', '232616850100.55800324371656023', 37, 1);
    T('339164.29624761126106926', '8936758968.18859', '-8936419803.89', 12, 6);
    T('15560.32752547059908210', '383428809621.6678246983', '-383428794061.340299227701', 24, 0);
    T('3605728012072.4760329848577', '21745.0', '4000000000000', 1, 6);
    T('46.144614394', '61832.762489311', '-61786.617874917', 24, 1);
    T('37.00879311496699', '8714798847303.8484468761', '-8714800000000', 6, 5);
    T('1222.28329426011716500714', '9283040559370820.026571', '-9283040559369597.7432767398828', 29, 2);
    T('573277817387729870.37004528', '8602314661788.204150458', '573269215073060000', 14, 1);
    T('7731221.09676', '938501323.2134', '-930770102.11664', 20, 0);
    T('1307209050113.14365048227905119669', '4972479385304.015542620', '-3665270335190.87189213772094880331', 39, 5);
    T('463627940356.216367115', '65531271161464.2369687333604975', '-65100000000000', 3, 5);
    T('377.653597218132339', '5.4729106847577797', '372.1806865333745593', 32, 0);
    T('8.6665149685076319', '2166295226785.5948', '-2166295226776.9282850314923681', 34, 4);
    T('1002562011.3446627299330', '12101.2498641', '1002549910.094798629933', 27, 4);
    T('28088857903118701.7', '2363249166280372.665656314955926', '25725608736838329.034343685044074', 33, 5);
    T('196171569529.31700356391853814070', '54.7561174', '196171569474.560886164', 21, 2);
    T('980219917506467255.8', '957.749715589711730', '980219917506466000', 15, 3);
    T('2310900.7712', '2015854699.1364816', '-2013543798', 10, 1);
    T('234.357438919', '75.0', '159.357438919', 35, 4);
    T('481165739.78680344615605006', '534203.357541437943943', '480631536.42926200821210706', 40, 5);
    T('534728194962653.5708688809059842', '4.108', '534728194962649.46286889', 23, 0);
    T('4.069757821284', '52988757352944.621048799', '-52988757352940.551290977716', 36, 5);
    T('9403989905.21390828010', '8991260.78', '9394998644.4339082801', 24, 5);
    T('178729046.876', '986351953190943.0451578981905910', '-986351774461896.169157898190591', 31, 5);
    T('195289593.9922820588', '12420618346816913.37204492457', '-12420618152000000', 11, 5);
    T('9811594325.622709899225', '67794195742.277417', '-57982601416.654707100775', 39, 2);
    T('175206980404.0', '543.87607219707', '175206979860.12392780293', 37, 1);
    T('407.50', '1714829783.1084537624', '-1714829375.6084537', 17, 1);
    T('984001573.7945035641733393868', '18300522674507446395.383636750370', '-18000000000000000000', 2, 1);
    T('83593.87192', '3781722151906752.3', '-3781722151823158.42808', 35, 0);
    T('18852063222246430510.1781326278515684', '5722098086197.33', '18852057500148344312.849', 23, 2);
    T('116866.5784708673152409455', '605310689307.80619581231592', '-605310580000', 8, 3);
    T('57757193588327.833', '56462.88', '57757193531864.953', 28, 5);
    T('999298397751.334', '13510778.604142', '999284886972.73', 15, 2);
    T('4613.454881', '92928.2333805239215138', '-88314.7784995239215138', 27, 3);
    T('3380302307308811581.850', '848.37794359113599256302', '3380302307000000000', 10, 3);
    T('252962953312938.0922293246819258', '3.11024315985534', '252962953312940', 14, 2);
    T('5738008.4816135672', '4107.6', '5733900.8816135672', 37, 6);
    T('723974698.8195418593475442726', '1197938104919.52245219805', '-1197214130220.8', 14, 0);
    T('532950689204898787.904512447691', '57135856.47862325', '532950689147762931.42', 20, 3);
    T('49446157825199555876.1319', '5529291090865978923.8998948', '43916866734333576952.2320052', 35, 5);
    T('520.09977096751', '93856773224781676917.4019', '-93856773220000000000', 10, 1);
    T('5116494.80', '774962023329961440.3355', '-774962023324900000', 13, 0);
    T('1901673053660250515.7005', '96295345807277167.1909683689', '1805377707852973348.5095316311', 33, 3);
    T('8.098806856322512023', '5.546437529307', '2.552369327015512023', 26, 1);
    T('74559660397877.14167778082534595', '175455.690', '74559660222421.4516777808253459', 30, 5);
    T('18715.02564408857377', '215867.2893981488150068301', '-197152.26375406024123683', 23, 2);
    T('7857318015300036.7808815815', '170472757553360.7', '7686845257746676.0808815815', 33, 6);
    T('940923.99913459005660950', '15367167760169003658.388607', '-15367167760168000000', 15, 2);
    T('617606510020.50325677547', '23684687197.6813098', '593921822822.82194697547', 37, 2);
    T('9178621717.681369298830604851', '3121976080793877.6362407838', '-3121966902172159.9548714849693951', 32, 5);
    T('66203175752195.90058', '32699240955980.0', '33504000000000', 5, 5);
    T('65226160687611365.019263681', '24679.7904979', '65226160687586685.228765781', 33, 1);
    T('1211618715115564.68135248714139', '637287507055036441.168320417989', '-630000000000000000', 2, 2);
    T('2915.8653883', '9703858168565024.0683322035285914338', '-9703858168560000', 12, 2);
    T('56287741223.015307290', '84.412628769721', '56000000000', 2, 1);
    T('17676397290.17131701569601834926', '164471632999331205.901780989399', '-164471615322933915.73046397370298', 32, 5);
    T('72143348586608321700.7232223', '70128848781342898.34532', '72073219737826978802.3779023', 40, 4);
    T('9499596235248808809.86648740377', '7557869907.159911807894936497', '9499596227690938902.70657559587506', 33, 1);
    T('41553518730078127312.7', '597514358739622168.0554264313', '40956004371338505144.6445735687', 39, 3);
    T('10553948505572.69694596279', '795518539.062979403695700', '10553152987033.633966559', 24, 3);
    T('1.5478198418732', '0.1780407107297', '1.3697791311435', 24, 3);
    T('1726881574252.9842696', '676457.3782901', '1726880897795.6059795', 22, 3);
    T('4711.836', '94205519482031800933.2891536488223510441', '-94205519482031796221.45315', 25, 2);
    T('997514521279486.584828327', '110780187232532.6919', '886734334046953.892928327', 31, 2);
    T('175566109034541211.024458', '54975602463.01184021088841', '175566054058000000', 12, 3);
    T('6026128135884641.8037726020', '1156866324.384', '6026126979018317.419772602', 32, 2);
    T('3722459447946.5184611536', '79400418460177.2233', '-75677959000000', 8, 5);
    T('3634852171651594.32708986671637653048', '1.4264068776748385856', '3634852171651592.9006', 20, 3);
    T('562250801058405.7684', '42672436218.51608872015027', '563000000000000', 3, 2);
    T('1059622439125787.188082', '39631331427732019854.0', '-39600000000000000000', 3, 2);
    T('216215238892075865.77', '627830.8171846', '220000000000000000', 2, 0);
    T('842870323482904.076164132190', '67510251692642165107.1157737', '-67509408822318682203.0396096', 27, 4);
    T('92171354889334305989.145466273387292415', '21772434926.91196467699720708008', '92180000000000000000', 4, 2);
    T('348.50756455690528414255', '37543200627092313.337665388015556094', '-37543200627091964.830101', 23, 4);
    T('29272.5215968134', '457180959259.2026941586188123', '-457180929986.68', 14, 4);
    T('225061554.168113792', '805023747583.0', '-804800000000', 4, 4);
    T('5.362781904', '46.21249622034', '-40.8498', 6, 3);
    T('62563350.72243217896802601', '536.27877466047766765412', '62562814.4436576', 15, 0);
    T('892464675331156821.1650629', '4.385313927673', '892460000000000000', 5, 1);
    T('8286412.52263029834802042496', '75161078653301.6724', '-75161070366889.14976970165', 25, 4);
    T('145945.62', '36026748.8949944475136', '-35880803.274994447513', 20, 2);
    T('854162716840.6182509211522926943', '6.4621', '854162716834.1561509211522926943', 31, 2);
    T('44321.365118029327201942', '21074040216868.7004919993663979007', '-21074100000000', 6, 0);
    T('355630931889.3124055', '9973182.8572095994788', '355620958706.4551959005212', 40, 4);
    T('9770109.87288749222604559', '596377202487.525409987046388', '-596367432377.65252249482034', 26, 1);
    T('8371364224185466.85152', '78873488863346125.42192745', '-70502124639160658.57040745', 39, 3);
    T('238970944928665.28853219', '4.99724388', '238970944928660.29128831', 32, 6);
    T('88635.424', '7884996989494769.74580369258406874972', '-7884996989406134.3218036925840687', 32, 1);
    T('545791.42', '393938769962.774831555230493', '-393938224171.3548315552', 22, 4);
    T('99915142675.457457884628', '96470835893015135902.2128086804', '-96470835800000000000', 10, 0);
    T('8.531', '58505370633091.4', '-58505370633082.869', 38, 5);
    T('86582327654207.935785984497', '91.616326045', '86582327654116.319459939497', 34, 2);
    T('53340711219565780.67313', '4003434423017564790.4380606443954603', '-3950093711797999009.764931', 25, 0);
    T('153691980.8534323441834233601', '26419524494.524938', '-26265832513.6715056558165766399', 31, 5);
    T('630292.07628', '861769409008.90394944581', '-861768778716.82766944581', 28, 0);
    T('846.892', '48279321899023545.4758397651511', '-48279321899022698.5838397651511', 30, 4);
    T('83960.0285898500', '6380177318217084.4', '-6380177310000000', 9, 2);
    T('747802472602.6131204576419017335', '7453048430200.42351457', '-6705245957590', 12, 2);
    T('478017573062184307.265', '7762764211.126996486', '480000000000000000', 2, 5);
    T('5329042.0310401718957790', '84.03448683812738', '5328957.996554', 13, 0);
    T('544438.348118571725', '93386705258342106.69268950716531670', '-93386705257797668.345', 20, 4);
    T('717840593328619.750770908115084211', '31433668744134255.9077153', '-30715828100000000', 9, 2);
    T('3414669.288077', '954390628.16259', '-950980000', 5, 6);
    T('2468755030.1130050955431901', '85395582263639431.545654', '-85000000000000000', 2, 6);
    T('406951548487385983.158752049149', '2.983163701402', '410000000000000000', 2, 2);
    T('74257943176620757696.9056', '24186753.3712745', '74257943176596570943.5343255', 27, 4);
    T('3230258.0', '48553776.28685049978766', '-45323518.28685049978', 19, 1);
    T('145342939625.8868755461997', '8126102994576.29218546144114715', '-8000000000000', 1, 6);
    T('1306586098213.99', '17653.33713371078', '1306586080560.65286629', 21, 2);
    T('4.4358493', '92900.336584492131743', '-92900', 3, 0);
    T('7544861.37674', '917162929.96', '-910000000', 2, 0);
    T('778544270908022.308', '96490632075887975613.349132', '-96000000000000000000', 2, 4);
    T('793504407.889947', '7161480201373884.13776402716511218', '-7161479407869476.25', 18, 4);
    T('794048835.111983185625859641', '549185912976395360.32795503667784247', '-549185912182300000', 13, 4);
    T('2413787873.6', '367468181596626642.09', '-367468179182838768.49', 32, 1);
    T('8.797975', '8354534599121015131.16673219786615698378', '-8354534599121015122.3687571', 26, 2);
    T('185823.7673268480705', '2774.7008751144962558', '183049.0664517335742442', 37, 1);
    T('76060467248525982849.288063938611524', '969059626440040391.5344742522689932', '75091407622085942457.7535896863425308', 38, 5);
    T('95.471474069224636161', '21.630817968', '73.840656101224636161', 25, 1);
    T('6948901832901856744.0', '415.33864722189389480994', '6948901840000000000', 9, 2);
    T('330.243361220', '5.055865885966399', '325.2', 4, 4);
    T('498811767980.76', '8365086709326.724', '-7866274941345.964', 39, 0);
    T('44370699025414454.4572208741372473931', '8711401857864.18062918575725300859', '44361987623556590.276591688379995', 32, 0);
    T('377429913408526319.254604146250', '6200800.179', '377429913402325510', 17, 3);
    T('8863915664782868798.17372444166', '77.0989', '8860000000000000000', 3, 1);
    T('9259358994673529.0', '19908.58399850735970137', '9259358994653620.416001493', 25, 0);
    T('3344162.789749319', '79045507222165190228.909', '-79045507222161846066.119', 23, 1);
    T('4.23252076474012456310', '9.1641007', '-4.9315799352598754', 17, 5);
    T('731166.6301957604483823', '2546981802862405.584974760958', '-2546981802131238.9547790005096177', 38, 4);
    T('8723705646.8990407022264028476', '33486.0', '8723672160.8990407022264028476', 36, 5);
    T('18960475650856.24988069871951', '437100578721144.9485345817', '-418140103070000', 12, 6);
    T('3615.49214113225696224861', '302027571075178.52', '-302027571071563.02785886774303775139', 40, 0);
    T('649665279.3408620', '35852017005808591.7478258006012', '-35852016356143312.4', 18, 2);
    T('858017311756.0150618535339326', '98.0', '858017311658.0150618535339326', 33, 3);
    T('587232.9', '963157123455329.95', '-963157122868097.05', 21, 3);
    T('687819234229077713.0', '90872033.759167751753905061', '688000000000000000', 3, 4);
    T('97669950631962582.351383', '999152.832', '97670000000000000', 5, 5);
    T('695834312062330153.30059166929677048', '50650666563.9148178285184', '695830000000000000', 5, 5);
    T('1801839186827.346577', '320248564746886281.094', '-320246762907699453.747423', 34, 6);
    T('259.10222269', '638703635031.838129', '-638703634772.73590631', 29, 3);
    T('65351047098637753.067613342026607', '875706050804784.89125340190', '64475341048000000', 11, 4);
    T('19.3021', '7817427514860561507.9856428', '-7817427514860600000', 14, 0);
    T('92.4899527094', '88809.6119801014', '-88717.1', 6, 4);
    T('620551866093.32621640327648', '15867068421032.143995059797274', '-15246516554939', 14, 6);
    T('568.0', '481193.45831155001237679833', '-480625.45831155001237679833', 36, 6);
    T('22.265', '37437266.06122522523', '-37437243.79622522523', 24, 6);
    T('1.008316', '802865801172659.9969', '-802865801172658.988584', 23, 6);
    T('6973520750578105879.3625508246258438607', '91.903', '6973520750578105787.45955082462', 30, 3);
    T('67406.93307232609962725', '2981595564228.674034', '-2981595500000', 9, 5);
    T('5480716.71099541667', '4232090.4208624723345214', '1248626.2901329443354786', 39, 6);
    T('3.52345', '3305269.21699920', '-3305265.6935492', 17, 4);
    T('481069702.9273403971304339775', '190858274.2683117821202285', '290211428.6590286150102054775', 33, 3);
    T('4.0', '941181608424356.94', '-941181608424352.94', 30, 5);
    T('697477040812.49788840780401', '47.622268704163177', '697477040764.88', 14, 2);
    T('5.4739090', '40552471475229559648.18827426187', '-40552471475229559642.7', 21, 1);
    T('1581340338.5', '9732.0121441079655', '1581330606.4878558920345', 37, 3);
    T('5991836425.81959500182616062943', '10006063428337283.3318087787260', '-10006057436500857.51221377689983937057', 39, 3);
    T('72559678460612.520475', '4836855949126016.0592366', '-5000000000000000', 1, 6);
    T('25612454.954681385', '61171580.0247', '-35559125.070018615', 29, 2);
    T('417589695281453087.2211735', '500370.5', '417589695000000000', 9, 1);
    T('9356905008459765083.0136534', '562523.303381', '9356905008459202559.710272', 25, 5);
    T('298830867.0', '15638606045887641002.735544656200456676', '-15638606045590000000', 13, 6);
    T('4021011562482014175.66748630', '23173565220428460.1', '3997838000000000000', 9, 6);
    T('4933142774550316024.4001', '182.38843117280', '4933142774550315842.0116688272', 30, 2);
    T('7.633', '51.513', '-43.88', 39, 4);
    T('5842159861.817601', '370341887.5691023523', '5471817970', 9, 1);
    T('79.0002649344043886', '808.44', '-729.4397350655956114', 25, 1);
    T('737513095387272.963920424823671', '696174109.999249354213287', '737512399213160', 14, 4);
    T('8.4170832347050', '53.43799423792156150', '-45.020911003217', 14, 6);
    T('11036562057494766573.82', '918623.1453910438249919', '11036562050000000000', 10, 1);
    T('11360050.303266935328', '3879240758134.6', '-3879229398084.297', 16, 3);
    T('81854627425.53355992564098', '24793453501.619', '57061173923.91455992564098', 40, 6);
    T('509335.84962574278', '915192752586.1', '-915192243250.25037425722', 27, 3);
    T('61092502132284284220.8', '74017829733.806437954421', '61092502058266454486.993562045579', 38, 0);
    T('49519798628032169532.68292706', '7708652665995121227.7284647787374182377', '41811145962037048304.954462281262582', 35, 5);
    T('282658858610240833.1604670', '978063471223298318.76', '-695405000000000000', 6, 5);
    T('29571599414005542.488847500579710', '6902045888697317.298702', '22669553525308225.19014550057971', 39, 0);
    T('2.2324712334953335', '4.209621022971523262', '-1.977149789476189762', 22, 2);
    T('940508.1629814552884103', '264933.723325524', '675574.4396559312884', 19, 6);
    T('980.056', '2222046.142699', '-2221066.086699', 20, 3);
    T('544496520360.339550813', '60404473281381.68911532104', '-59859976800000', 9, 6);
    T('44424667.2113558', '87983036609675923020.59', '-87983036609631498353.3786', 24, 5);
    T('95246295889.0', '554596766466785.8479', '-554501000000000', 6, 2);
    T('3.17', '3386283.519840049261', '-3386280.349840049261', 32, 3);
    T('69215212004794190.319', '77.0929316382946629', '69215212000000000', 8, 3);
    T('893459.843', '4181373961055840.1992', '-4181373960162380.3562', 36, 1);
    T('188750790.1619182180', '463090366935778.5763520680433', '-463090178184988.41443385', 24, 5);
    T('353359020796592550.0', '6856316.63845992972732', '353359020789736230', 17, 6);
    T('6783726.031314716', '95051785955191.9448143861144', '-95051779171465.9134996701144', 36, 0);
    T('47126751115.3', '664711265589133.720414044109164983', '-664664138838020', 14, 4);
    T('2259448351012505888.1158', '6897856950874815.89669652958289', '2252600000000000000', 5, 5);
    T('45446425631519892.39681518608016545', '533.0', '45446425631519359.39681518608', 29, 1);
    T('80522463947932846565.28367901201230', '23943255930053680.95073774339', '80500000000000000000', 4, 4);
    T('9.185524', '28359408350136.993852464321295', '-28359408350000', 10, 5);
    T('2570301852736205355.019197946991175040', '70347459599165387181.32209646245', '-60000000000000000000', 1, 1);
    T('41089423730803.03736791296265', '117704878360462474.5466533971096592052', '-117663788936731671.5092854841470092052', 39, 1);
    T('469397519.01430298962874355867', '55.9477', '469397463.06660298962874355867', 32, 2);
    T('79473372734.9788896', '736942904699675901.0547395366530', '-736942825226303166.075849936', 27, 1);
    T('4706388091470508.70127482591505818', '406819755070.58093396101', '4000000000000000', 1, 1);
    T('82577.46', '967346533078.045', '-967300000000', 4, 1);
    T('285454375115945.82', '6722138.9790343383790691459', '285454368393806.8', 16, 6);
    T('201511188857541.63', '535.23', '201511000000000', 6, 5);
    T('93064322324153299257.81548379776257', '460072999286684658.11', '92604249324866614599.70548379776257', 40, 6);
    T('38843222630287805.9987190531821', '1106696225663.33191', '38842115935000000', 11, 0);
    T('35843664864489886.7743647', '39954125102632.404', '35803710739387254.3703647', 35, 2);
    T('281306320281.9359', '13323.97237641173039301', '281306306957.96352358', 20, 3);
    T('4759.6897018376', '8043.026622', '-3283.3369201624', 28, 4);
    T('950941681450407235.49191796838926231504', '872308654508012.32533722184410', '950069372795899223.166580746545162315', 36, 3);
    T('9698645123.47657083231', '901424398941600715.087609', '-1000000000000000000', 1, 0);
    T('5220643.27', '46491067643735.6275993582835', '-46491062423092.3575993582835', 29, 5);
    T('43468565.528920073416', '630.1922168686629229657', '43467935.336703', 14, 1);
    T('831.6574278170745091', '48056995302.5801297554129665', '-50000000000', 1, 5);
    T('5090581.988784', '39.24', '5090542.748784', 20, 3);
    T('95977712363.3', '2635648239440590.011', '-2635552261728226.711', 36, 0);
    T('310.910820', '47386444400759736.10830170705', '-47386444400759425.19748170705', 36, 6);
    T('251192412719.07755165845892', '21732.38439463148153974512', '251192390986.7', 13, 2);
    T('994554868343.41544524354', '104108543.03980796919', '994450759800.37563727435', 23, 2);
    T('226111529.0', '759920474.578952353', '-533808945.578952353', 25, 3);
    T('8527.314398', '15683.40395064', '-7156.08955264', 19, 1);
    T('250193022193.0', '5323295339.261645', '244869726853', 12, 1);
    T('5982847829936.33707938613', '38306447986374.10', '-32323600156437.76292061387', 39, 0);
    T('60664.483453512174', '30668879435618280538.71411', '-30668879435618219874.230656487826', 40, 6);
    T('199.0', '989.0301288', '-790.0301288', 15, 3);
    T('1734649472270180.256669', '346003587929.429013852821398', '1734303468682250.82766', 21, 2);
    T('522453360727840.17933', '870821565.46', '522452489906274.71933', 37, 3);
    T('1044396258927688.7254613', '88846230018045674.2249999', '-87801833759117985.4995386', 38, 4);
    T('751355078275.7683', '42155817058.7984901237', '709199261216.9698098763', 38, 1);
    T('688342178800.4846452074674648', '40469.5298226730', '688342138330.954822534467464', 27, 3);
    T('13256.4', '4969719935.0', '-4969706678.6', 38, 1);
    T('485123418819518.26', '7763540869565046264.36822175703', '-7763055746100000000', 11, 1);
    T('220.984187998', '97719175.546', '-97718954.5618', 12, 5);
    T('489255265.333132157312122351', '0.234035592200540676', '489255265.09909', 14, 1);
    T('8876754402308576513.354290315525370', '556746251.43365148642211', '8876754401750000000', 12, 3);
    T('74512182417920.7720500', '246455539569.1383318623', '74265700000000', 6, 6);
    T('358.8055186161072341', '49.6690', '309.1365186161072341', 29, 6);
    T('610.9', '646196557899038.921635803681078', '-646196558000000', 9, 6);
    T('9378088870046.8959925104', '331133747711813487.0789814', '-331200000000000000', 4, 3);
    T('7275346133.466275566640', '790920.40183146', '7274555213.064444107', 19, 2);
    T('43184823.79809', '1474241427737173.541991808', '-1474241384552340', 15, 2);
    T('7822398508961609.25760551826', '325464653426.3006', '7822073044400000', 11, 2);
    T('6728.148580548', '450.39', '6277.758580548', 24, 0);
    T('63273375643062512760.305973590254653599', '1536142.67732', '64000000000000000000', 2, 2);
    T('541437771525.392910670302', '3110110.2', '541434661415.192910670302', 31, 1);
    T('6433415971915225961.08386', '43852797.76042', '6433416000000000000', 8, 0);
    T('6823315359365.64509206', '414276954.672562226', '6822901082410.972529', 19, 1);
    T('370954196.21627468', '45984940951289176.972855153543438243', '-45984940580334980.8', 18, 4);
    T('6889604819.475745071', '3.2370615796018563', '6889604816.2386834913981437', 30, 2);
    T('63.7794824685861285', '76598959.80', '-76598896.0205175314138715', 36, 2);
    T('7362.0', '4581126625573.3', '-4581126618211.3', 14, 0);
    T('827079884342.9869', '9055105651504375.611635752', '-9054278571620032.624735752', 25, 3);
    T('96845388532303241.95375042274997027', '96406504071494.404174963703', '96748982028231700', 15, 1);
    T('934119466958.0473026230', '3036.895707137005554500', '934119463921.1515954859944455', 31, 0);
    T('2306136516693689.06176656601356578', '20243473423588.21456226873208072071', '2285893000000000', 7, 3);
    T('4270.14987624126410684', '58.02457796487619407', '4212.12529827638791277', 37, 1);
    T('897369.5153940028457123376', '2342360232335101.7', '-2342360231000000', 10, 6);
    T('278288813.59887554623493606', '93254425668595944.0', '-93254425390307130.4011244537', 27, 1);
    T('94869.7', '392970.3560106922', '-298101', 6, 5);
    T('4.7524', '2528793189.69', '-2528793190', 9, 3);
    T('1995994.02500', '2915093319291701.223117753431923', '-2915093317295707.198117753431923', 38, 4);
    T('496916835132434.7352251', '3154141896385831803.56748', '-3153644979550699368.8322549', 30, 3);
    T('476585374705.1393496', '476826.174238060941210', '476584897879', 12, 4);
    T('417070945183363.79588684659', '11208634285620489695.2880626249666185', '-11208217214675306331.5', 22, 3);
    T('15355969625428706.78751', '7637713457381316.7868690054630', '7718256168000000', 10, 1);
    T('5537115103.9681793156', '8241.22319609538585118', '5537106862.744983', 16, 6);
    T('1656656.85', '2990525352248660665.156430266144', '-2990525352247004008.306430266144', 36, 1);
    T('325743544.29', '721804734862534.16', '-721804409118989.87', 19, 2);
    T('168369538236656.35599887', '684792553488.577902840502', '167680000000000', 5, 3);
    T('39285902935.942934385765032', '5251.368', '39285897684.575', 15, 2);
    T('663634767708517339.415583963986672', '31294055626.96934794', '663634736414000000', 12, 4);
    T('38.27524382293', '63052484.88539041364', '-63000000', 2, 5);
    T('86710.8506', '6.3377888178205', '86704.5128111821795', 18, 6);
    T('72027.9', '70620287107500610.6816575982', '-70620287107428582.7816575982', 37, 1);
    T('33482.538769251000616187', '41.67155852228773', '33440.867210728712886187', 27, 0);
    T('877242913285051588.445', '979766.345327014', '877242913284071822.099672986', 36, 4);
    T('4301235436.40880728495785994166', '959654377127.328324013829', '-955353141690.919', 15, 2);
    T('3818.317768826', '268186919.81767778', '-268000000', 3, 2);
    T('27467821645466615.43118366289750', '747345.241118950417', '27467821644719270.190064712', 26, 6);
    T('6610528.650710051992', '73816208.42', '-67200000', 3, 4);
    T('8424322891410595.343', '60791164761045393.26219', '-52366841869634797.91919', 26, 3);
    T('8704392182990.3659733710978', '1000726015467788.4130573498', '-992021623284798.0470839787022', 37, 0);
    T('85372656655820183259.821187624656', '9246506965150993290.8585633550', '76126149690669189968.962624269656', 38, 6);
    T('168642732206.0', '95035188155457615589.026004', '-95035187986814883383.026004', 35, 2);
    T('144222852.316980956884226027', '53591789208197524.815400', '-53591789063974672.49841904311', 28, 1);
    T('525942.3', '92178.9008553223669816', '433763.3992', 10, 2);
    T('694886758.371422969', '6216598137.87918717733', '-5521711380', 10, 0);
    T('24171.585310273452032', '3253445.03847335', '-3229273.5', 8, 6);
    T('7191058783665906766.569785', '730116420052420630.3098987302958929', '6460942364000000000', 10, 5);
    T('983997322369.19725858369374189939', '8594590747863.43943591301464', '-7610000000000', 4, 1);
    T('32.046469887972', '61836027987863.6477922711322836', '-61836027990000', 10, 0);
    T('2529229.0', '301269490999.0980355', '-301266961770.0980355', 38, 0);
    T('7262108147.2', '96869.62893896', '7262011277.57106104', 38, 2);
    T('60301175620076229.66498331142963930', '3684668243807504.389', '56616507376268725.2759833114296393', 40, 3);
    T('1569312.84', '79331568351885.64457440', '-79331567000000', 8, 6);
    T('54744887.91072342294431165694', '8674006373307273368.6', '-8674006373252528480.689276577055688', 34, 4);
    T('6115461919994906.9145834595929741', '5958865486479.4570228985', '6109503054500000', 11, 3);
    T('586908490.95757444090', '157960851846726138.24888', '-157960851259817647.291305559', 27, 5);
    T('525.044266', '6096.10393150819160410746', '-5571.059665508191605', 19, 0);
    T('0.088884036384', '7301.115728', '-7301.026843963616', 18, 0);
    T('3460.161', '965297.3582', '-961837.1972', 28, 2);
    T('28161182897122672.5454656546551532', '5845433504535592346.424573249818', '-5817272321638469673.87', 21, 2);
    T('730676499230049.1353', '588561523.8456018354277990', '730675910668525.289698164', 24, 3);
    T('964.46653763', '4110450993.245974321005', '-4110450028.779436691005', 28, 4);
    T('78091789743.1570840831889797', '9.6037', '78091789733.5533840831889797', 38, 4);
    T('99499693147684794888.97', '970213097349326.8234289430950', '99498722934587445562.14657', 25, 5);
    T('348192.905018071', '3970732549.6175', '-4000000000', 1, 0);
    T('55744312984337497.1185', '5828237522.1937859', '55744307156100000', 15, 5);
    T('436027.5307', '638236894178177402.4', '-638236894177741374.8693', 36, 3);
    T('8326460960571660419.0', '81976403923006320.36859748677206', '8244484556648654098.63140251322794', 39, 4);
    T('197957801264319.483800617540', '6090143277051.0486676934398100188', '191867657987268.4351329241001899812', 39, 3);
    T('794813574.0862', '59473114133087.942529', '-59472319319513.856329', 35, 4);
    T('69615316394674236871.7638796994', '27705596.483307161724163481', '69616000000000000000', 5, 2);
    T('995686875.828805346', '97616204.3', '898070671.528805346', 39, 5);
    T('11356.56445110358302795', '9.57644690142969530744', '11346.98800420215', 16, 3);
    T('570730.5', '131.6616619', '570598.8383381', 35, 0);
    T('2273641497931052018.156', '233.50553557922249', '2273641497931051784.6504644208', 29, 4);
    T('97805235453101198988.3808239314747', '5830124275425853561.0', '91975111177675345427.3808', 24, 5);
    T('18702768269005.06384554', '513496034330718292.7', '-513477331562450000', 14, 6);
    T('626924741.9079', '18711096784321.40635723932753', '-18710469859579.4984572393', 24, 6);
    T('255554394628809852.4088', '35914622308.0261691660531978192', '255554358714187544.3826308339468', 32, 3);
    T('2.182498248484895', '2.24063157', '-0.058133321515105', 29, 3);
    T('26523641099982451005.666361232', '278305549497376357.489836880409068', '26245335550485074600', 18, 5);
    T('9986753911989309004.59225', '3.883198', '9986753911989309000', 17, 6);
    T('22447879612.241417221', '3987307539043393.32545365', '-3987285091163781.084036429', 29, 5);
    T('28193940552034.602993022136536', '4456.562', '28194000000000', 5, 2);
    T('4512288.3747882542421288255', '6.35824297127972137', '4512282.02', 9, 5);
    T('3.408816997732079', '2.10560', '1.303216997732079', 18, 5);
    T('7209885588672719.71425581487938647575', '252928.17', '7209885588419791.5442558148793865', 32, 5);
    T('7867642230998619774.40580393', '358.621832', '7867642230998619415.78397193', 30, 6);
    T('243229249304.595643931373', '8985657255646798.874816958220539', '-8985414026397494.279', 19, 1);
    T('71.79558095202', '11594947920908.15228714873516401', '-11594947920836.356706196715', 26, 1);
    T('92480584410904.3482643041608995190', '868953491607287.26732649467', '-776472907196382.919062190509100481', 35, 2);
    T('7455659144905742.10252', '67.46939', '7456000000000000', 4, 4);
    T('4889878.578984188488824871', '8280.852660749822605190', '4881597.726323438666219681', 40, 1);
    T('712031756629.8607870055750', '21201606509479355766.470', '-21201605797447599136.60921299', 28, 4);
    T('120741.7316531467', '81359194817994.35130279746356956470', '-81359194697250', 13, 5);
    T('7037510313281382040.9221420', '32.5838661699', '7037510313000000000', 10, 4);
    T('3420704954120268.309941353781', '9697879137960094926.53931211570908', '-9694458433005974658', 19, 2);
    T('6410723232.26345004096737178', '695681.1617645787455702323', '6410027551.1016854622218015477', 34, 3);
    T('6144461.4', '3293405.3549080542186417806', '2851056.045091', 13, 3);
    T('3305972275.987964084795070424', '234957634700359.20472367408', '-235000000000000', 3, 5);
    T('3664727.48835066', '6.460145', '3664721.02820566', 23, 6);
    T('53155125312126.2', '441700838.1318072468748890', '53154680000000', 7, 1);
    T('85878318780.5896696796278855', '1468.21357107843533217', '85878317312.3760986011926', 24, 4);
    T('122.0', '3730496975.54437900931187924020', '-3730496853.5443790093118792402', 36, 2);
    T('4505448886563417.0186657895020209', '31910415407226341911.6', '-31905909958339778494.5814', 24, 0);
    T('3008790889995915151.47497499453383', '48605339184294077607.190031264638992447', '-45596548300000000000', 10, 3);
    T('6207062925781114229.72245506057', '30761704715010.44112752854670', '6207032164076399219.2813275320233', 37, 3);
    T('2608802240801091665.70', '5948507255655815220.7433752', '-3339705014854723560', 18, 6);
    T('566556980.9729733068216729', '3.7971530085775642877', '566556977.175820298244', 21, 3);
    T('767963001109055.103915511850690', '3.210569724', '767963001109051.893345787851', 27, 6);
    T('627107798710505.03982277102596', '2594204233.87017542628', '627105204506271.17', 17, 0);
    T('6346977739089892302.88365493325365', '36967.9041', '6346977739090000000', 12, 5);
    T('698098594536588701.3084', '9618.353099725916581', '698098594536579082.955300274083419', 35, 3);
    T('699526.0', '91866350084400.3006', '-91866349385000', 11, 5);
    T('91.475556578991627', '9329298733818541448.12', '-9329298733818541356.644443421008373', 35, 0);
    T('856542.4845', '62811544616.5662692184522', '-62810688074.0817692184522', 24, 6);
    T('167802757251800.0161142153288903261', '29066541584611540.1653091633409', '-28898740000000000', 7, 4);
    T('76250962.34526400770018353768', '4476621304598241536.565511474169', '-4476621300000000000', 8, 2);
    T('4248054659499.8435581', '5466.0', '4248054654033.8436', 17, 4);
    T('980349876140718052.1', '13468648911656241.776558', '966881227229061810.323442', 34, 0);
    T('947245451320543894.46862589008043643857', '962650589807.3482882380571', '947244488669954087.1203376520233', 31, 1);
    T('8828619491530873616.53', '42959.966454787803608656', '8828619491530830657', 19, 5);
    T('50769.9765055436028160493', '38034.0', '12735.9765055436028160493', 36, 0);
    T('3238577420670.11898306580115067547', '217602015353942.127461364705345794', '-214363437933272.008', 18, 1);
    T('359940600646044715.05685996', '2.80025342', '359940600646044712.25660654', 39, 4);
    T('610236496.690517309714198701', '2394816901.00652723640704', '-1784580404.316009926692841299', 30, 1);
    T('7419798060.5682205544', '44766121159.937765648619571', '-37346323099.4', 12, 0);
    T('9775333049572.0', '9.9944633339', '9800000000000', 2, 4);
    T('3.2919012554591', '72786759567.52814600477673392', '-72786759564.236244749', 20, 1);
    T('3864552649312.081708', '50847174072.06073', '3813705475240.020978', 26, 0);
    T('833605369579.61742313238009', '1212809948.0', '832392559631.61', 14, 1);
    T('3345350350.0047', '51.07589264837413169', '3345350298.9288073516', 20, 6);
    T('2450780270.681902387', '3176755880690637177.898598330390668397', '-3176755878239856907.21669594339', 30, 6);
    T('3955.36780450356', '51.9295138', '3903.43829070356', 30, 1);
    T('3778922448759.571283037697', '91923082136808964.6077084523347929', '-91919303214360205.036425415', 26, 0);
    T('2889930315843.930', '6411117.56100629649526918638', '2889923904726.369', 18, 2);
    T('2646613146.0', '1.3', '2646600000', 5, 3);
    T('40988829708829886085.1490001239607', '787547392450112.36211', '40988042161437400000', 15, 3);
    T('76462290189161134331.5611820166122188511', '3393193.31', '76462290189157741138.25118202', 28, 2);
    T('29215.829751', '538091657.35550559', '-538062441.6', 10, 0);
    T('458271372899155.99511086643', '7.92317790547', '458271372899148.0719329', 22, 3);
    T('97612173400365238.05285804', '23125729906.0', '97612150274635332.05285804', 40, 0);
    T('5.69505', '58.0', '-52.3049', 6, 2);
    T('5.684764071012736850', '341.497431447011005410', '-335.8126674', 10, 5);
    T('669160.59413541235', '933634.08139146825924', '-264473.48725605590924', 36, 1);
    T('35191990087.12461463185650104', '6207719036.38987414097373', '28984271050.734740490882771', 27, 5);
    T('2525004836790.8', '2882643.82399', '2525001954146.97601', 26, 0);
    T('46.0960859', '3534769725628738.9849250', '-3534769725628692.88884', 21, 6);
    T('2362416.3652382330880010', '543774518.7198723', '-541412102.36', 11, 0);
    T('5.61', '665647187709.284332', '-665647187703.674332', 33, 0);
    T('700484534662273.615717691303209192', '58358151017740.9298181759813951952', '642126383644532.68589951', 23, 3);
    T('95312613654947916049.3326464576', '1194721772.876912263302', '95312613654000000000', 11, 2);
    T('7305.436425813', '4996790.4633016192208398199', '-4989485.0268758062208398199', 33, 1);
    T('9217301049357289911.1167303735464115', '3600697880588688.168787890588626791', '9213700351476701222.94794', 24, 4);
    T('564864079337.71851307853', '694900248622571345.5929524198', '-694899683758492007.87443934127', 32, 1);
    T('9569133276463482359.8088245813165469', '367413612.14577700864', '9569133276096068747.66304757267655', 33, 4);
    T('507346228.2795349637996', '315980126.57', '191366101.709534964', 18, 2);
    T('9296776706.0', '54137190461249597073.6932', '-54137190451960000000', 13, 3);
    T('49086149.990996344', '1.962', '49086148.028996344', 21, 0);
    T('131.864959356', '4457951466489039.09626531228', '-4457951466488907.23130595628', 32, 4);
    T('60.12329136488935', '1623778328047.20313095457', '-1623778327987.079839589681', 25, 3);
    T('725669927224.402', '67865573806931381324.2', '-67865573081261454099.798', 35, 6);
    T('5.8876379583399511852', '32129949.4197561206712781', '-32129000', 5, 1);
    T('76683314518143059312.80015084', '14304025.64218792', '76683314518128755287.158', 24, 0);
    T('83712346825392018863.96', '4203742231959.11212', '83712342621649786904.84788', 29, 0);
    T('19803723130651.013466', '4.4469820483', '19800000000000', 4, 1);
    T('85435678366.134457815025158177', '3138992.7714738400', '85432539373.362983975025158177', 39, 4);
    T('19878143586333.00786843370117097632', '5330519647543506714.94912619112661038', '-5330499769399920381.941257757425', 31, 6);
    T('812308240424.0', '98921999595285456.389034973118946763', '-98921187287045032.389034973118946763', 37, 5);
    T('445.319255620457128739', '835541042507110.75858147508940061472', '-835541042600000', 10, 0);
    T('142400873.3686674870', '2432349499.0', '-2289948626', 10, 6);
    T('5078.69', '672596956492915943.880', '-672596956492910000', 14, 1);
    T('229473823119311043.5', '189364707046291.1538519', '200000000000000000', 1, 5);
    T('570346.656162741964', '84344453977285.462823892', '-84344453406938.8066611', 21, 1);
    T('7963529423751548943.56237', '18203169.749', '7963529423700000000', 11, 3);
    T('80635212643946.33567871895214376301', '6.555374050679163', '80640000000000', 4, 5);
    T('48312034693832.936311084768', '5035821200111946738.8354468', '-5035772888077252905.899135715232', 37, 1);
    T('2810051.57427466331530', '1188713594911535857.2', '-1188713594908725805.6257253366847', 33, 5);
    T('30524708682054.387006260625', '7287913906.98', '30517420768147.408', 17, 2);
    T('381234557360541578.195655968356196', '238872.254393367246015', '381200000000000000', 4, 5);
    T('215.55418302465462', '94893.0', '-94677.44581697535', 16, 5);
    T('231732.85613022', '91191820806.811651033319182', '-91190000000', 4, 1);
    T('73044941277658.0', '20701948830262562406.1545725409187158', '-20701875785321284748.1', 21, 2);
    T('410544000.0000685083', '858612716299.0206933117', '-858200000000', 4, 5);
    T('5.295', '8868512.147200041', '-8868506.852200041', 25, 5);
    T('6947435852081.24725470665579', '2492949630.76551', '6000000000000', 1, 3);
    T('686043058707091465.758135968466476', '504.30100827333585', '686043058000000000', 9, 3);
    T('38928092386879218.069', '502507856079696610.2741635', '-463579763692817392.2051635', 29, 6);
    T('50864486747.5422638521798707743', '9.71597224629', '50864486737.8262916058898707743', 35, 4);
    T('529581806194226542.9874637140', '51083465.387', '529581806144000000', 12, 0);
    T('6375185907408250.8', '69.91562322933313324255', '6375185907408180.88437677066686675745', 39, 3);
    T('96.39446509278301150304', '33.119582723262', '63.27488236952101150304', 33, 3);
    T('75063209651593919532.808236504', '791164444790.979044', '75063208860429474741.829192504', 31, 4);
    T('7026218231979.1314644845305435', '8.03037504923682162', '7026218231971.10108943529372188', 38, 4);
    T('53.21801872019', '4077381827815434958.2407681733407', '-4077381827815434905', 19, 2);
    T('8254795816926833.61', '62403589.54233190101', '8254795754523244.06766809899', 34, 0);
    T('678196470913.50777080', '585175.0304324154', '678195885738.48', 14, 5);
    T('533487039.7475', '933867.2985474', '532553172.4489526', 40, 6);
    T('617124651308.62753901303', '653244104051984170.1916', '-653243486927332861.6', 19, 0);
    T('962400344.7424703', '591868519301975796.2504034860071115262', '-591868500000000000', 7, 2);
    T('668.552434130525', '44111077.0', '-44110408.447565869475', 24, 2);
    T('8.8876', '767524055.1333567951', '-767524046.2457567951', 37, 6);
    T('8047294156.744980801481', '29551253.54706975049635741', '8017742903.19791105098464259', 38, 6);
    T('4451.79103118697308', '12.016529998263231', '4439.774501188709849', 19, 5);
    T('13102787376638986.333590', '66.866382670277', '13102787376638919.467207329723', 29, 3);
    T('65507243647945274922.1', '16972.183598955', '65507243647945250000', 16, 3);
    T('576739665639761.6', '377.4717382', '576739665639384.1282618', 26, 6);
    T('1556475201.0', '821.641', '1556474380', 9, 0);
    T('73.96444588438422398890', '5005570304.52380334813', '-5005570230.5593574637458', 23, 4);
    T('38730654105638848.95536155', '7109.993948393500151777', '38730654105631738.9614', 21, 4);
    T('957608.1', '96043498333242.2511383292369243', '-96043497375634.1511383292369', 27, 4);
    T('8402926848418529.405', '11726856579.47167', '8500000000000000', 2, 0);
    T('820924015.87537821902', '974.490637', '820923040', 8, 3);
    T('522202.6', '466078119662166.4548538', '-466078119139963.8548538', 31, 4);
    T('770034578004138.88542371485', '428738847434751.9187074', '341295730600000', 10, 2);
    T('24333.396510', '10008053029436371.44', '-10008053029412038.04349', 37, 4);
    T('8193560352.43936579136146', '5672296430.81', '2521263921.6293657913615', 23, 5);
    T('94468325280229.024529352216', '9774732044429808.45486', '-9680263719149579.43033', 21, 2);
    T('400827792271972.0395255571929', '213.0536925689890458', '400830000000000', 5, 5);
    T('9472841289324.76353814507', '76361682746327.6271059', '-66888841457100', 12, 3);
    T('141552.2552973630466656279', '2.32431150500', '141540', 5, 3);
    T('716498468836861.361092304', '23135928.572413877663', '716498445700932.788678426337', 39, 0);
    T('470540599488.30', '7293689404416941.0', '-7293218863817452.7', 36, 5);
    T('9899250999905550108.59', '820698546.1983074363621902719', '9899251000000000000', 9, 0);
    T('1478081871055.50123389659537520', '9.0', '1480000000000', 3, 2);
    T('9903651808.711826335', '26442019960339947.5815932237205054072', '-26442010056688138.86', 19, 1);
    T('581639024168351.3859368355483494064', '79393877568980418.20107815582437', '-78812238544812066.8151413202760205936', 37, 4);
    T('99371701155729137.11770022', '5325734.9408205420', '99371701150403402.176879678', 27, 4);
    T('999444292395179974.16087', '2698208717245.081', '999441594186462729.07987', 29, 4);
    T('689626741145253737.8818316886553467670', '82332929.88876236', '689626741060000000', 11, 3);
    T('2720071648832571.7953', '4059554207117967.2829193842681620504', '-1339482558285395.487619384268162', 31, 6);
    T('926372724572228.6', '6151723136764.321306', '920221001435464.27869', 20, 5);
    T('4635441012172.7', '994812.01003', '4635440017360.68997', 22, 5);
    T('530550522.0', '508945958.50208715347025', '21604563.5', 9, 2);
    T('787186436128214.5793', '76889668143359123.730554', '-76102481707230909.151254', 33, 5);
    T('362851196857988.3', '9324628199484.96575749', '353526568658503.33424251', 34, 0);
    T('2436140484117769052.1145339928444', '2790474.965989872433838', '2436140484114980000', 15, 2);
    T('9832266342564.0625607899', '7057.0', '10000000000000', 1, 0);
    T('6868512589234570.3189383', '771776611.0', '6868511817457959.3189383', 38, 0);
    T('57997212033408.651738', '592471.82127315936536352', '50000000000000', 1, 3);
    T('76139691294.5', '147307031932219437.2017375442', '-147306955792528142.7017375442', 39, 0);
    T('9783.03', '912619.47225', '-902836.44225', 39, 0);
    T('2006462.453659501', '28586251.6781534972143781574', '-26579789.224494', 14, 3);
    T('414885993969915323.9574902322838', '5815931545546815.7842264890211559', '409070062424368508.17326', 23, 6);
    T('92901107750999290623.0', '95018188193461946.5934296', '92806089562805828676.4065704', 35, 4);
    T('8022.6262135980640475', '8174763162232.87660161563116036463', '-8174760000000', 6, 5);
    T('7220479052488699436.400', '270928671969.9115129648916940', '7220478781560027466.488487035108306', 40, 5);
    T('331980991.29153', '116.5', '331980000', 6, 3);
    T('2483343175.30971310277', '9819304363598.3078526356987219', '-9816821020422.9981395329287219', 32, 5);
    T('68714010523925410.1314205632854730453', '6895039778411775819.090834087557', '-6826325767887850408.959413524271526955', 37, 5);
    T('6810984.039880962665798208', '8522.143360059837587352', '6800000', 2, 5);
    T('311897607615.40487118898093', '78979539.8601', '311818628076', 12, 5);
    T('97145077.175', '2129500655193577367.6893986778428084681', '-2130000000000000000', 3, 3);
    T('14070.750731829222561116', '7292267520219.59552', '-7292267506148.8447881707774', 26, 4);
    T('5394209.62503927630', '6719497147823330284.32869059632265070', '-6719497147817936074.70365', 24, 1);
    T('21473190077.809', '98284.618776859', '21473091793.190223141', 23, 0);
    T('7394369422525.2059493220453230216', '852691.05812', '7394368569834.147829322', 22, 4);
    T('945980223087.2938716', '135956797875.62965', '810023425211.6642216', 29, 1);
    T('6843199443683.0', '80668.271330182137', '6843199363014.728669817863', 36, 0);
    T('310604.80559956941106673445', '10.39939386', '310600', 4, 0);
    T('229127591297064222.178041985843048408', '75782906081289481.02', '153344685215800000', 13, 5);
    T('62.6067', '5120076237.7646596155500852', '-5120076175.158', 13, 0);
    T('9654638079.742967137735120', '686002612887366.44500331799819794', '-685992958249286.70203618026307794', 33, 6);
    T('61457402.543794792158188', '450768374.461', '-389310971.917205207841812', 39, 5);
    T('110012477987570632.2', '466079794423542608.753', '-356067316435971976.553', 22, 1);
    T('2774.8', '68886013037.359738217094', '-68886010262', 11, 1);
    T('8.28492603', '2384439393392018475.9195066295', '-2384439400000000000', 8, 4);
    T('2.45925785671448', '638179.5261189625823500221', '-638177.06686111', 14, 0);
    T('2237637992.45652295348993818922', '85128.56348381459', '2237552863.89303913889993818922', 32, 6);
    T('85786839.23098296', '53616418466823740.4647094489028722', '-53616418400000000', 9, 4);
    T('522533996956100027.181454723821730498', '66.06919', '522533996956099961.112', 21, 4);
    T('1095048.9', '8021.532990', '1087027.36701', 38, 5);
    T('34.4', '5913185203460137.8615764721', '-5913185203460103.4615764721', 28, 4);
    T('965444860.67238607916066859158', '2408.62942996593271495', '965442452.04295611322795364158', 39, 0);
    T('86779570805869342.5', '39160870856.6956426953328321939', '86779531644998485.8044', 21, 4);
    T('72348235538114.7782160266218880240', '54010687074196128522.578644589076', '-54010614700000000000', 9, 6);
    T('87717806300528051.3102626067988', '39653109064335.37390271936', '87678153191463715.9363598874388', 35, 4);
    T('852785997922487368.4347551561820268', '6825449185641474.20567879479601919', '845960548737000000', 12, 6);
    T('14991696262232.2309731535', '734.830791448155678028', '14991696261497.400181705344321972', 33, 5);
    T('24976.9', '5375875982447894826.597257', '-5375876000000000000', 7, 4);
    T('18729065.95', '700636592.65530059005739559535', '-681907526.705300590057', 21, 1);
    T('573.8', '558412393701630931.051928013911134', '-558412393701630300', 16, 2);
    T('34837348086.644602719766943160', '348.347789332274', '34837347738.296813387493', 24, 0);
    T('55011.27467957007086', '1116083688971.0', '-1000000000000', 1, 4);
    T('128600134787522818.9409928', '2279988332.025058', '128600132507534486.92', 20, 5);
    T('6594270103603908292.98498278170678', '5871732634735851502.277155', '722537468000000000', 9, 3);
    T('151221780070.496', '2796300713779703.7519346275107256743', '-2796149491999633.255935', 22, 4);
    T('7193802993247334.990528091675', '42178124950391538084.250819957152124359', '-42170931147398291000', 17, 6);
    T('9470545746.339666432208', '238754473.99118823576182200', '9231000000', 4, 3);
    T('692961641531.726989', '955046.943427', '692960686484.783562', 21, 5);
    T('302163274064.6744924243423', '16500852906011.1747', '-16200000000000', 3, 0);
    T('633964552628549.44555895', '472.0', '633964550000000', 8, 6);
    T('14399807820355571.0674236816', '4644944149485800.02136028959764762', '9754863670900000', 11, 2);
    T('4025451083084.721061087', '8581836338973909.0596719166126581749', '-8577810887890824.33861082961265817', 33, 2);
    T('2018.40315745845', '5263583931628.27', '-5263583929609.86684', 18, 4);
    T('79484834.80065181109', '925195.3122785263468665540', '78559639.48838', 13, 2);
    T('564.79', '89250.6819311790379427', '-88685.8919311790379427', 32, 0);
    T('8974033175826.84580329900', '503987.3952076180', '8974032671839.450595681', 40, 6);
    T('4420922368777.3816062361394940', '47973236314567071.0929900583', '-47968815392198293.7', 18, 2);
    T('5545360222094674166.026302919457519', '35.6153', '5545360000000000000', 6, 6);
    T('2721879702145610.5311308487841698', '83704272683836308539.0173562473521183412', '-83701550804134000000', 14, 5);
    T('737363851.26729615711143113', '69143873.235832', '668219978.03146415711143113', 30, 0);
    T('637.432085736', '39286545285.152', '-39286544647.719914264', 29, 6);
    T('55608374.44239118036824512838', '886080887.946707272864685007', '-830472513.5044', 13, 0);
    T('86875747931.80217', '4.4', '86875700000', 6, 1);
    T('59948.25049631338', '46836377807563349334.0967598', '-46836000000000000000', 5, 6);
    T('233909328294.1731252', '85443.7107450337', '233909242850.4623801663', 37, 3);
    T('46530776557910.237035420', '984955787.640315100169528764', '46529791602122.596720319830471236', 34, 2);
    T('46760383904889.7742659001', '8442.65400', '46760383896447.1202659001', 29, 6);
    T('131.2543', '47052254603742545889.2873999', '-47052254603742500000', 15, 5);
    T('466376878611947445.0', '87.9934250869', '466376878611947357.00657491', 26, 5);
    T('2.0', '82540173227.846973931774659471', '-82540173225.846973932', 20, 3);
    T('42.501', '91408255.6', '-91408213.099', 20, 6);
    T('602966.08659547', '454.5734762', '602511.51', 8, 3);
    T('361864729704996.0', '9051824606949.910551', '352812905098046.089', 18, 6);
    T('44283778882800161.0', '647090309011.299867021305', '44283131792491149.70013298', 25, 6);
    T('84.859993458', '569702.21516771358100', '-569617.3551742556', 16, 5);
    T('58450257227328749581.3826545027618049', '9.25764285936475293', '58450257227328749572.12501164339705197', 38, 0);
    T('5078952074096.888912886032', '6.035244350', '5078952074090.853668536032', 27, 4);
    T('7144465899.044342425789510', '63785642196104801.4586183', '-63785600000000000', 6, 1);
    T('53538368681.2606223655791973193', '6459844436051.3', '-6410000000000', 3, 3);
    T('4.0048489', '894.1954', '-890.1905511', 20, 3);
    T('218.4936738631522172915', '16396199226123103.116737341', '-16396200000000000', 8, 0);
    T('46969.52655023060662', '1220338119.75799', '-1220291150', 9, 4);
    T('37866620622258.2187688', '967.323773712', '37900000000000', 3, 0);
    T('315086596022.749656165', '6068135580.5', '309018460442.249656165', 30, 3);
    T('3612008535708527.8096778827327723', '60647.4135', '3612008535647880.3962', 20, 0);
    T('2026003.05586391748627410064', '940571193700017216.6172276', '-940571193697991213.56136368251372589936', 39, 0);
    T('7872443568505893.59', '42.3099831678426030', '7872443568505851.280016832157397', 32, 1);
    T('26.14292985080999134', '67666873914211.4541', '-67666873914185.31117014919000866', 40, 4);
    T('4.8831977571062606', '375.810392254', '-370.9271', 7, 1);
    T('654786510480874.7796', '766403999334445479.0', '-765749212823964604.2204', 37, 1);
    T('940763339625632.34448251374', '9942232985598314459.242767', '-9941292222258688826.898', 22, 5);
    T('6527.8210289198510319', '67.6292859582', '6460', 3, 3);
    T('13055679922712649449.493110596732015', '109963.6066255', '13055679922712540000', 16, 5);
    T('66939664312738.47705253', '273201711450245.3272', '-206262047137506.8501', 19, 2);
    T('385.430022734283821', '58559781020.071544272', '-58559780634.641521537716179', 33, 5);
    T('8944720793482215.41167861810', '94267.57616682', '8944720793387947.9', 17, 2);
    T('0.91179245', '88.628930657135208207', '-87.71713820714', 13, 5);
    T('68542712012118.87411165', '14304336719842547.227987013024', '-14235700000000000', 6, 1);
    T('820.89309202876826', '319012592471.53374194113655', '-319012590000', 8, 4);
    T('193474721313739.5912242208444221761', '10719.70', '193474721303019.8912242208444221761', 35, 6);
    T('40763945562746.3', '74607829.00857171720', '40763870954917.2914282828', 36, 6);
    T('79623364561608913.5242', '58229893798187643255.632668', '-58150270433626034340', 19, 1);
    T('411966.2539124', '74261746138318488698.778', '-74261746138318076732.53', 22, 0);
    T('96014.950974404', '9194254102717701203.0696052505719780551', '-9194254102717605188.1186308', 26, 2);
    T('48153334.7238335791488032', '94119237455.47350', '-94071084120.749666', 17, 5);
    T('1492.77', '4169689080220073323.042050954197970', '-4169689090000000000', 9, 0);
    T('8973168866778.0111956', '8014843982331047.9453131320986569', '-8000000000000000', 3, 1);
    T('675.7235561415604', '1586754652936414.6223677985', '-1586754652935740', 15, 5);
    T('631032247468.113498772124098', '8396734789151272532.19072822', '-8396734158119025064.07723', 24, 4);
    T('89942.415703', '84267741493283.2', '-84267741403340.784297', 34, 6);
    T('96.6', '96349042232666.759496489773063536', '-96349042232570.15949648977306354', 31, 0);
    T('644686419803386268.6189672551289685', '324642473829151.04496771338', '644361777329557117.573999542', 27, 5);
    T('7308460584019144.250', '9836.422246471517744444', '7308460584009307.827753528482255556', 39, 1);
    T('193978111210812.0', '2390451760677052.1248234414509379461', '-2196473649466240.124', 19, 1);
    T('550271768975376.30530694', '8135600614773969.4', '-7585328845798593.1', 17, 4);
    T('32206.522746695187', '43.12098582', '32163.401760875187', 37, 0);
    T('414.02164', '860189968750355.52', '-860189968749941.49836', 27, 3);
    T('49132593106.91497189313601', '7216403.0', '49125376703.91497189313601', 32, 5);
    T('791905898.790884156881839527', '5460515619236650986.114952544449672', '-5460515618444745087.3240683', 26, 2);
    T('25483418211307252330.384712', '4657059650030512747.5065843', '20826358561276739500', 18, 3);
    T('3117743485211.61436', '15831643512643992336.78600163430577', '-15831640394900507125.171641634306', 32, 0);
    T('5786499438897091.857131622', '173587085748.0468554312784802207', '5786325851811340', 15, 4);
    T('23339783277315725222.19217787', '89894.272767768', '23339783277000000000', 11, 6);
    T('6.832295536', '974251094.803678104249867', '-974251087.97138256825', 21, 0);
    T('7372168087318069.431265825773785', '90902169672748.325450624730790', '7281265917645321.10581520105', 27, 0);
    T('265479933405.542778691453684889', '7.872736357013', '265479933300', 10, 1);
    T('5711021511105892.6280453', '57076819309141267564.0166265133937868389', '-57071108287630161671.3885812133937868389', 39, 0);
    T('8.5388671050087646', '3305195101535068.6641516302327946', '-3305195101535060.12528452522403', 37, 4);
    T('47.48998', '3870240296074071152.7966773512888869611', '-3870240296074071105.3066973512888', 32, 1);
    T('93.29789', '744.444904099', '-651.147014099', 37, 6);
    T('58535965646909719370.3671823656719', '79108205792.2072', '58535965567801510000', 16, 1);
    T('14455090245498471.84606836270', '76684668.70531008', '14455090168813803.14075828', 25, 1);
    T('646806165583.26983575', '576684693482465969.349940', '-576684046676300386.08010425', 35, 6);
    T('11679.394', '495151859990964.6812774231178', '-495151859979285.28727', 20, 2);
    T('1144252938069.4608871152163141298', '23049866869552.0', '-21905613931482.539112884784', 26, 6);
    T('1917777644242.706610704965432', '585282832325.680134513682', '1333000000000', 4, 0);
    T('73778295.7', '95.7409498885323729', '73778199.9590501114676271', 28, 5);
    T('7044831.232061426916385910', '5702573.98419644173939884212', '1342257.3', 8, 2);
    T('456.210', '8012022201904689456.4704311419', '-8012022201904689000.2604311419', 38, 0);
    T('574674990825.52655423014835', '518293405292.64732107998381771', '56381585532.87923315016453229', 33, 0);
    T('797038.2783290799370646', '18690804792517.2762', '-18690803995478.9', 15, 1);
    T('3985.50011786815326029136', '8897838445.0', '-8897834459.49988213', 18, 6);
    T('4.4433657976', '49174470566518.60094189545357681955', '-49174000000000', 5, 4);
    T('13885.36268920655', '863001.9862', '-849116.62351079345', 36, 3);
    T('379286240.505061', '648355350771.67375461720514784', '-650000000000', 2, 0);
    T('77625770901702868.014', '97748171210365809624.5046992104382076116', '-97670545439464106756.49', 22, 6);
    T('4702324969915554.2711268859398193', '80409389818927982089.4698540', '-80404687493958066535.198727115', 29, 3);
    T('206.0007704662970877232', '93.3489596163088', '112.6518108499882877232', 26, 3);
    T('7508345216900.8209557858113', '35995983.7266369926621496634', '7508309220000', 9, 6);
    T('56539290134.35', '49410285.1169877260777143', '56489879849.2330122739222857', 38, 0);
    T('726.264603786', '2588282.4957151913', '-2587556.23', 9, 6);
    T('6833.21291110120211', '52729202.5607', '-52722369.34', 10, 1);
    T('602724968671429.256762', '8910613648103.6944', '593814355023325.562362', 29, 1);
    T('2.11419546', '92333564820757723009.597789141778012456', '-92333564820757723007.5', 21, 3);
    T('697431856547386.6213328508531509', '805235197056.88434181217718489819', '697000000000000', 3, 6);
    T('8588504384532685119.54818469440788', '67347609338.0', '8588600000000000000', 5, 2);
    T('792265312706.46729423433062405385', '98260848752.1168277689', '694004463954.350467', 18, 0);
    T('790.6188665757144647', '1185841638802484.512681620233519201', '-1185841638801693.893815044519054501', 37, 4);
    T('691.7546136750', '79632858.619011021', '-79632166.864397346', 20, 6);
    T('605109.5011', '3423.66', '601685.8411', 18, 4);
    T('8728058376229310.249991230420659449', '16483220032617427.55327986071897156', '-7755161656000000', 10, 1);
    T('80601864878915.0787467920805396847', '75341859362.518173', '80526523019552.5605737920805396847', 38, 3);
    T('99991088701634.33411', '5332703.12619168', '99991083369000', 11, 5);
    T('269160250884119.0', '32297418711.488697894639143305', '269127900000000', 7, 3);
    T('1967476388053.5446', '5957937.19210', '1967470430116.3525', 23, 3);
    T('719.27076589187554316', '71162465410639.5666812', '-71000000000000', 2, 2);
    T('84.6663504749879431', '30851077734860145.540676503928057', '-30851077734860060.87432602894011', 31, 6);
    T('707295804513998072.96422183378872', '9659132090965483.6426186053480719', '697637000000000000', 6, 0);
    T('368915183864560049.126', '37239298849336.465466985536274140', '368877944565710712.66053301', 26, 3);
    T('28420930947500.85461743641518536708', '30.6', '28420930000000', 7, 4);
    T('23.931', '85980708.89444405', '-85980700', 6, 3);
    T('3766608795065913.3819668', '83804626.6209852517', '3766608711300000', 11, 2);
    T('670278762.16369240376', '248083882195643029.5130388444', '-248083881525364268', 18, 0);
    T('934085862538823315.822583865327962', '925846936546290.91067119427339977511', '933160015602277024.91191267105', 29, 1);
    T('3162335699.93783324', '9146853730783475080.76251390895', '-9147000000000000000', 4, 6);
    T('75.87877209450388487', '610841236819.7665854872', '-610841236743.88781339269611513', 38, 0);
    T('50.26280839747967660136', '713.007181089', '-662.7', 4, 6);
    T('40371240893.5985277325429126216', '4.0', '40371240900', 9, 4);
    T('4885518812101110779.82431932084', '29.90', '4885518812101110800', 17, 0);
    T('251227862576939.25518506084074678952', '680.29936284176697956', '251227862576258.955822219074', 27, 5);
    T('93605529.8477712', '436545108133163.882', '-436545014527634.0342288', 24, 5);
    T('431195355986273837.24410766454', '7807.091', '431195355986000000', 12, 3);
    T('34839854592281412793.4253281967000006', '6930531.541383747', '34839854592274482261.883944449', 29, 3);
    T('85.0880784587', '54923480225.4292345474446', '-54920000000', 4, 4);
    T('2.0', '826617749268383.497771425472', '-826617749268381.497771425472', 37, 0);
    T('61320606477427.5187005582118853', '905.48210787', '60000000000000', 1, 3);
    T('3182940010153.277389', '5189297.23662', '3182934820856.040769', 29, 4);
    T('747547914009.51', '7617936868922009.9554519517813343', '-7617189321008010', 15, 0);
    T('39244887643847353.50249360447059794303', '3192456.3660621899687497924', '39244887640654897.13643', 22, 4);
    T('2463415906025.59514483211797338', '76491353874015.30388961867498', '-74027938000000', 9, 5);
    T('8593.923898', '9535272321.0', '-9535263727.076102', 21, 3);
    T('237.3', '5236.499607446', '-4999.199607446', 35, 2);
    T('64735.4496', '1076065323409981947.9953463130', '-1076065323400000000', 12, 1);
    T('920325210.9386744670204749', '646057936537089972.8678', '-646057935617000000', 12, 3);
    T('66046996.9360389995', '63308523811898425.1069986376', '-63308523745851000', 14, 4);
    T('23701010671951.2054267682132', '7243.105706133980', '23701010664708.0997206342332', 31, 5);
    T('4872.73905422787877599', '50.2303', '4822.50875422787877599', 39, 5);
    T('473367588880.38545639203915697988', '505086411262387976.4613501', '-505085937894799096.075', 21, 1);
    T('461945746.72082310034072932', '50475465932.37393302395553883', '-50013520185.65310992361480951', 28, 2);
    T('583747568625040.15874718542536', '580761160.59543083162879', '583746987863879.56331635379657', 29, 5);
    T('45139530854794357864.884824995740626681', '3773800373132573566.215705261038586954', '41365730481661784298.66911974', 28, 2);
    T('37642585576.0', '160242758216059.642709', '-160205115630480', 14, 1);
    T('984873.75065284847038734798', '157215.974778887228610', '827657.77587396', 14, 1);
    T('750373.972491205653', '290906.5008548915674317', '459467.47164', 11, 4);
    T('7477048447263.650573050122', '6133960515.9736387316520533', '7470914486747.6769343184699', 26, 3);
    T('1163488915357110.0', '8685184019091323.298816982206', '-7521695100000000', 8, 5);
    T('64968.624219807981954818', '1.4682840857644', '60000', 1, 6);
    T('91667703811716763861.5343268512369325082', '823.35351992168', '91667703811716763038.181', 23, 5);
    T('822732357.417002865759820805', '80136041.9851743076334', '742596315.431828558126420805', 40, 0);
    T('12453069575760208.023', '5026842893.921046434264515093', '12453064548917314.101953566', 26, 6);
    T('7914284603588962317.687', '51.0', '7914284603588962266.7', 20, 4);
    T('0.95422817', '61377771.52408282311141872', '-61377700', 6, 2);
    T('81211.616231489', '28800.851517823', '52410.764713666', 38, 6);
    T('954632071092.8', '93690739814.220833427858138', '860941331278.579166572141862', 36, 3);
    T('35422337455701599.25683822314400212', '37500746752794144996.0083142329340', '-37465324415338443396.751476', 27, 2);
    T('72.487255521199651', '70152158221295998.9191942', '-70152158221295926.431', 20, 1);
    T('11.133070', '6550.772877519', '-6539.639807519', 20, 1);
    T('707702621977193124.575542072282', '63748145494.12923', '707702600000000000', 7, 4);
    T('305457893342119937.095', '8672372822669271.693036', '296785520519450665.401', 21, 3);
    T('8188.98951977202853', '825873.80758666121840701602', '-817684.81806688918987701602', 36, 3);
    T('708398146992312.89942876230847', '26924163512.261863272196583516', '708371222828800', 13, 6);
    T('55005858769.0', '411358424528333.859433248432', '-411303418669564.859433248432', 40, 0);
    T('627541339565.1927709', '56414898720.029225', '571126440845.1635459', 27, 4);
    T('475982910660358.827150922541122', '8.0', '475982910660350.827150922541122', 30, 5);
    T('767621725.297612500312055779', '80123351945534886.0099447353286', '-80123351178000000', 11, 6);
    T('76.6374898218171', '3.067', '73.5704898218171', 17, 6);
    T('1887211652790891624.6774', '37.916459', '1887211652790800000', 14, 1);
    T('5164373556893405250.9270371009859691', '6679806.31999841571173290629', '5164373556886725444.60703868527423619371', 40, 6);
    T('542200312.17925317', '7572179842517996.1921755646710', '-7572179300317684.012922394671', 30, 1);
    T('4335950.4215501', '58668412742007010703.737584391178026704', '-58668412742002674753.3160342912', 30, 0);
    T('1.197358883455338108', '3202.55212', '-3201.354761116', 13, 1);
    T('379237955194192090.697048486', '98448256491893.968412930494666370', '379139506937700196.72863555550533363', 38, 3);
    T('77634179140800.0', '63017213524112.5191880094', '14616965616687.4808119906', 36, 3);
    T('3146210965.4865527', '96.56973', '3146210900', 8, 2);
    T('41511161700714228.0', '643970195905037066.332', '-602400000000000000', 4, 1);
    T('584.7444951935493346424', '53.2063', '531.53819519', 11, 3);
    T('946034.8', '70257151467594054.46', '-70257151466648019.66', 31, 4);
    T('118259425604425.1962363233352081', '4863.59977542437', '118259430000000', 8, 2);
    T('25503036.00686261620363', '7732205572249030408.61091530607414931082', '-7732205572223527300', 17, 2);
    T('2646940.8458014403644024463', '4496139834102.7012300054911168653', '-4496137187161.8554285652', 23, 0);
    T('7327373.97341958', '191624652374114999.94346574535673530153', '-191624652366787625.9700461653568', 31, 0);
    T('341106722502940479.9', '57624593624.4204472807113628381', '341106664878346000', 15, 3);
    T('662098081603.431753997441414879', '2636042753804.056385714904295', '-1973944672200', 12, 6);
    T('35628.419374710715', '1042473912966.0', '-1042473877337.580625289285', 31, 6);
    T('97984533788383360968.37326648517', '7.636961328585362', '97984540000000000000', 7, 2);
    T('453069840635329.73195494241', '243995.20631', '453000000000000', 3, 4);
    T('84380577.188209587539774', '42621577742.0807266', '-42537197164.892517012460226', 31, 3);
    T('8642897334956522595.675040452175255', '9117149555735.33', '8642888217806966860', 18, 4);
    T('89667879401.23489210128797221508', '302293.2489183', '89667577107.985973801288', 24, 0);
    T('1457833555.0394211831482', '374.09102142828567966459', '1457833180.948399754863', 22, 6);
    T('74.803145', '82171018540590145185.9954085', '-82171018540590145111.1922635', 32, 3);
    T('5278658468.52273777783474', '11879846514123402849.069485678346', '-11879846508844744380.5467479005112', 33, 1);
    T('514002342712196037.157269797', '146541511487161783.477094964', '367460831225034253.680174833', 28, 2);
    T('98.483', '83855.6046292953401386', '-80000', 1, 2);
    T('22.86135830815402', '9858870.72222', '-9858847.86086169184598', 31, 6);
    T('55.401', '1122043928154868.0', '-1122043928200000', 11, 5);
    T('837837026264512.4119', '5507661679914385399.9', '-5506823842888120887.4881', 35, 5);
    T('6246111642794526568.513040959999375445', '4262270845097744677.601745', '1983840797696781890.91129595999937', 33, 3);
    T('25646518260276644.6', '587853025.578086379037482186', '25650000000000000', 4, 5);
    T('5.4731', '5433759068199.697', '-5433759068194.2239', 32, 4);
    T('8036960124893.0', '80266645366999511.571', '-80258608406874618.571', 24, 2);
    T('39410459108715.0790062663965', '90.13597410376569377', '39410459108624.94303216263080623', 31, 0);
    T('973399.82353', '5480.40', '967919.42353', 18, 3);
    T('610953.1200219149', '48.67745993750660797', '610904.44256197739339203', 26, 6);
    T('1285173815.446343332', '417014600.0594', '868159215.386943332', 30, 6);
    T('2.4', '69.87', '-67.47', 32, 4);
    T('7474680.320', '89490229168597719867.112790', '-89490229168591000000', 14, 3);
    T('8489395599284.2', '7265629500.6661', '8482129969790', 12, 2);
    T('33914.0', '645.12', '33268.88', 28, 3);
    T('601322320776579638.27195294782134649137', '907.187', '601322320776578731.08495294782', 29, 1);
    T('78982.9105904657855713', '9283993.63267', '-9205010.7220795342144287', 28, 5);
    T('193.4', '5516424007106723.0', '-5516424010000000', 9, 6);
    T('248337.2', '6299444601019513812.0', '-6299444601019200000', 14, 1);
    T('9732539357173152.39', '64145261.733063687856', '9732540000000000', 7, 0);
    T('1366615544549.0', '15.28248', '1366615544533', 13, 1);
    T('2167.58785270501214475', '70129895720060.178691', '-70129895717892.6', 15, 5);
    T('73265008617.418', '2805773668.6', '70459234948.818', 14, 3);
    T('38334672693407957.1085822287', '4811414121424097760.446838', '-4773079448730689803.3382557713', 37, 4);
    T('6773949.0', '48340536577547.017', '-48340529800000', 10, 2);
    T('0.9609601287026973', '826030742.87304593296317527707', '-826030741.91208580426', 20, 5);
    T('7198.40038436807', '14105310188422089489.046524537', '-14100000000000000000', 3, 6);
    T('5311996381.0', '697024.74845814415', '5000000000', 1, 1);
    T('7178614274.14658967', '737388005563.5962', '-730209391289.45', 14, 4);
    T('6.2887', '82232748911394633.0815', '-82232748911394627', 17, 4);
    T('6962969.8805', '6.75236355008275540130', '6963000', 5, 4);
    T('129231661.0658', '4605209.8573176681131614', '124626451.20848233189', 20, 5);
    T('8138576411307648798.6207860348', '838615624131.99379048529702280', '8138575572692025000', 16, 0);
    T('880194897007.77133830860128890755', '41360446393270285345.95', '-41360445513075388340', 19, 5);
    T('11851675596.0597957078957504172', '818533994.6166884124848843851', '11033141601.4431072954108660321', 40, 2);
    T('7.202378446046', '7008.5788798547443398602', '-7001.3765014086983398', 20, 1);
    T('607.0395361071157804', '9449558148614162981.5445154116697272', '-9449558148614162374.504979304554', 31, 6);
    T('715139729084451.67862879502609180188', '739476178905428167.40127695', '-738761039176343715.72264815497390819812', 39, 4);
    T('87655967.50966984960', '295659773511.666783', '-295572117544.157113151', 21, 3);
    T('67.882745', '2123541.43321862309222', '-2123473.5', 8, 2);
    T('0.92271', '7188023.17514803305751297910', '-7188022.25243803305751298', 24, 5);
    T('430.20193', '1649.16', '-1218.95807', 9, 3);
    T('388644307330.0', '32197401707488398404.50634019687095491', '-32197402000000000000', 8, 3);
    T('10142907703246.2086812786', '166801867007.96795065408089', '9976105836238.2407306245191', 26, 5);
    T('77631072.89825562887', '7642111253392027295.11086682727077', '-7642200000000000000', 5, 3);
    T('4839239437.1466', '5327479841.3052240507', '-488240404.1586240507', 35, 4);
    T('562363257215.457962606383684', '77677917932920.88486593246711877', '-77115555000000', 8, 6);
    T('6100718783.83499', '8.9983532012732540055', '6100718774.8', 11, 5);
    T('419.16867323313903', '9178100663693.3077733258314742534', '-9178000000000', 4, 2);
    T('6262459.43507943645839600214', '68674809.0925345939386095934', '-62412349.6574551574802135913', 27, 6);
    T('47648.5946', '851.22', '46797.3746', 35, 3);
    T('905068439983827857.41572305613546571', '90.14053341495082614', '905068439983827800', 16, 0);
    T('6020080.3228715466541965', '7060.8815209897830', '6013019.441', 10, 3);
    T('2307993283865509531.661392', '5482.15262107352686', '2307993283865504049.5087709264731', 32, 5);
    T('8415591024.9323342', '707231270731744.64', '-707222855100000', 10, 1);
    T('3780456093502.201121659558', '3945.83520300060', '3780456090000', 9, 6);
    T('88729456680678258095.188671585', '41659234113681245.50447', '88687797446564576850', 19, 6);
    T('333848717805369.0', '180591.93843039', '333848717624777.06156961', 28, 2);
    T('4.036', '68674044.616', '-68674040.58', 11, 2);
    T('73800417.33980967034371', '43306550060642.8708709236163', '-43306476260225.53106125327259', 36, 6);
    T('3.059', '7199.050091761472', '-7196', 5, 0);
    T('44573368188717078924.9', '216977.90747829825886642', '44573368188716861946', 20, 1);
    T('4632999.3199497352925599226', '187227868790121.3066759513196970967', '-188000000000000', 3, 3);
    T('3791381626.0719423761499534353', '1068104963711.5346161732648942275', '-1064313582085.46', 15, 6);
    T('3743236755260172.0', '878987180310.277716', '3742357768079861.722284', 33, 4);
    T('67851025197133.069616533181504', '76462611780750382755.584', '-76462543929725185623', 20, 4);
    T('21706991.641713102743937', '842.51404899291649135690', '21706149.127664109827445643', 26, 1);
    T('361474705.671', '4605308507332518758.8087573', '-4605308506971044053.1377573', 26, 1);
    T('7.74', '92287453832355175.6', '-92287453832355167.86', 31, 5);
    T('9879314741077206.1905729783738217310', '520223317977.6354430', '9900000000000000', 2, 4);
    T('616365619935.04699', '4501761845271277.082', '-4501150000000000', 6, 4);
    T('81890840288089284.453656424', '424307714.1177', '81890839863781570.336', 21, 0);
    T('7372487929820072.305', '41919780684.6961869790056', '7372446010039387.609', 19, 0);
    T('785285309313441184.134774291606', '1585630720.50347539114512229', '785285307727811000', 15, 0);
    T('81639751726238826.4419234454893803855', '5345.66607553702', '81639751726233480.7758479084693804', 33, 2);
    T('34244630523168.97', '9877.267486141486023068', '34244630513291.702513858514', 27, 4);
    T('3671423212694394.0129', '28934576124680398.59595688892', '-25263152900000000', 9, 6);
    T('1.072232', '633136700571756470.815149868884012', '-633136700571756469.742917868884012', 38, 4);
    T('30285657587997216.24871974048459', '8692711585181079445.14598364389685555', '-8662425927593082228.9', 20, 6);
    T('454677.01', '245525219925540133.54722006496296671328', '-245525219925085456.537', 21, 6);
    T('7972374064483290521.90065436', '755906368748796.22680590', '7971618158114541725.67384846', 35, 0);
    T('7482547925.91792324881151088', '1510969692417518.0706922422149320541', '-1510000000000000', 3, 1);
    T('70714924120176499.9', '565682.9342', '70714924119610816.9658', 27, 4);
    T('7172279.576559', '2609800147.030125505', '-2602627867.45357', 15, 5);
    T('862240520080268731.225559716188070927', '388.70', '862240520080268342.525559', 24, 1);
    T('885751.8678616496484', '22783388229063318233.25', '-22783388229062432481.3821', 24, 5);
    T('791859.11092280666146', '574402708974520051.48837137588826', '-574402708973728192.377448569227', 30, 5);
    T('972580632117.13264633458061258', '499105396406707955.43', '-500000000000000000', 2, 5);
    T('491.7943887819', '190348.70808309310', '-189856.91', 8, 4);
    T('7385.91083135603', '402124.5233198191234686', '-394738.61', 8, 4);
    T('86079295.4868233521', '9634219290.4052933504989532617', '-9548139994.9184699983989533', 26, 0);
    T('218322363.41857826736503828', '4.90034371873509', '218322358.5182345', 16, 3);
    T('678264668970851772.424', '818990166229.12332953', '678263849980685543.30067047', 28, 5);
    T('8629.547863436207951866', '604683472270653.954314288096', '-604683473000000', 9, 3);
    T('97143314006221.5227771969772590141', '8201.0', '97143313900000', 9, 1);
    T('3337801022783228331.101096070291440684', '271209502364.44916325432586', '3300000000000000000', 2, 5);
    T('762.3483168477', '53094714502.5874726484615', '-53094713740.2391558007615', 37, 6);
    T('2990856850970079180.4452681928123', '8.0916793562', '2990856850970079172', 19, 1);
    T('8988381052.6716295026005481', '269852.9980761852168025120', '8988111199.673553317384', 22, 4);
    T('4418.466472465017582', '308912687.982', '-308000000', 3, 1);
    T('1015726.20271', '59998926561.702021404149814', '-59997910835.499311404149814', 35, 3);
    T('345070.27597667', '742.4090911372', '344327.8668855328', 37, 5);
    T('39702192584.77681747622', '5203339.6049451993853176181', '39697000000', 6, 0);
    T('868039794047171566.044', '2914132662.2033', '868039791134000000', 12, 2);
    T('58704076691083342266.92', '45672915670.462181', '58704076645410426596', 20, 3);
    T('542531019797165852.656595', '6358342220834436.937944', '536172677576000000', 12, 6);
    T('72570588.6050580458762039003', '99561828659582574568.31835959575', '-99561828659510003980', 20, 0);
    T('63746295.72802551320938066', '478083565.314823', '-414337269.58679748679061934', 34, 5);
    T('93531807857596429160.3168278372760274', '29361236347.0', '93531807828235192813.3168278372760274', 37, 0);
    T('7316475359.1678178', '128931.69', '7316350000', 6, 6);
    T('2176493.23', '55155676389.55589613206340656479', '-55153499896.325896132063406565', 29, 0);
    T('3415.923814588422', '95427939708638175362.59482', '-95427939708638171946.671005411578', 33, 4);
    T('973.84666954811200316', '736940086566761.98792420277', '-736940087000000', 9, 5);
    T('3937751168910.5140118144', '4.91450', '4000000000000', 2, 0);
    T('340498505.570', '1017426930349916.017291588085044', '-1017426589851410.4472915', 23, 1);
    T('392448.52', '316055.72842912807682327', '76000', 2, 1);
    T('41881322.405528522244103', '15517539498.88069', '-15475658176.475161477755897', 30, 1);
    T('816505418884075208.23829172436442378', '120.344311120164427142', '816505418884075087.893980604199996638', 38, 4);
    T('17614836329.998490636458531824', '6352.897748792066', '17614829977.100741845', 20, 0);
    T('645487.4532205913813', '7024160.32003993494128879315', '-6378672.86681934355998879315', 39, 6);
    T('293890517662195.565173237', '90282158797223720.464002957850661362', '-90000000000000000', 1, 6);
    T('398928214941901.3193837341', '678206980559419083.0', '-677808052344477181.68', 20, 2);
    T('75905518423.4563062', '89957374163865526581.23251500797874422426', '-89957374087960008158', 20, 4);
    T('207280497178450869.48050476214', '6694880489.289239734010302', '207280000000000000', 5, 3);
    T('446625246.2907', '992733795054.84766307', '-992287169808.55696307', 25, 5);
    T('99319275345615788.572822144100002', '125780.69586', '99319275345490007.8769621441', 30, 3);
    T('79333561876725255.3', '4136565.998', '79333561880000000', 10, 0);
    T('7.175060604682702440', '851453.4251', '-851446.25003939531729756', 38, 5);
    T('68852569.176', '3407861414274119529.0', '-3407861414205266959.824', 40, 3);
    T('42367187186.507351079605', '2895673814.9242291898027', '39471513372', 11, 0);
    T('199.0', '8929144.05575919556', '-8929000', 5, 0);
    T('71.815329554000361633', '7395.50', '-7323.7', 5, 0);
    T('9046983154809350260.008', '7992757.03', '9046983154801357502.978', 38, 4);
    T('32243.0213406365590', '1961.23603961641625768', '30281.78531', 10, 2);
    T('16483124294842560914.1', '8409579302571570862.70378', '8073544992270990051.39622', 25, 0);
    T('547674766.4882062439679411408', '5333751570.4605146', '-4786076803', 10, 2);
    T('24152009293869856.1277321118307748220', '898194158.69', '24152008395675697.43773211183078', 31, 2);
    T('1672558.243329804805770', '8972825225.21275101561673235565', '-8980000000', 3, 0);
    T('4.265692359960829', '504207954733695.88472349326533682', '-504207954733691', 15, 1);
    T('51333274482310846.6', '5969976579623856.5765844', '45363297902686990.0234156', 30, 5);
    T('61117.18067077677637288628', '898779068429732709.07560922959942', '-898779068429671591.89493845282304711372', 40, 5);
    T('4959447317640871.417119520059', '8704070297.81395430', '4959438610000000', 9, 1);
    T('709228577697823.993688', '698193221881.21383270529640', '708530384475942.7798552947036', 39, 4);
    T('97070252067019.8400034', '9117854549866.5487203971312', '87953000000000', 5, 0);
    T('8734.889618743815352396', '314408469.51987976849151', '-314399734.630261024676157604', 35, 4);
    T('4762149288231169563.53741800273', '16472.002524473790', '4800000000000000000', 2, 0);
    T('58758523015893110.8019733329749359', '475542426831051.8538620835015901926', '58282980589100000', 12, 6);
    T('238.3228346745', '9.730', '228.5928346745', 13, 3);
    T('171.38036543968', '766163822471681.28982419', '-766163822471509.90945876', 23, 0);
    T('3039175004837.24971034872', '1756321.1349061', '3039173248516.1', 14, 4);
    T('788779.305', '65017598401133.241828428222145', '-65017597612353.9368284282222', 27, 0);
    T('55088786005229343234.20509728903990931680', '5.1102785', '55088786005229343000', 17, 5);
    T('944635309.84197156359117310', '9.189942560', '1000000000', 1, 0);
    T('8282123178408521411.9977501', '202793868206345.9084642249400947673', '8281920384540315066.089', 22, 4);
    T('1394946815467.460076444922058148', '850493131959104813.067809923305088', '-850491737012289345.607733478383029852', 39, 5);
    T('53760898542.0472886', '90051877530669681.377932730', '-90051823800000000', 9, 3);
    T('45620123112566976731.9345833624389002169', '5353195422.469248793434010580', '45620123107213781309.4654', 24, 0);
    T('934836201163998061.05227018', '8635923458.1816903442347856', '934836192528070000', 14, 1);
    T('60503220880148.8365185728426572', '36629874152.40532422205', '60466591005996.4311943507926572', 34, 2);
    T('1448107.37898458360625913', '943340533763.003569976', '-943339085655.62458539239374', 26, 5);
    T('579254.3987', '11405.495819759201390962', '567848.902880240798609038', 39, 0);
    T('416300.13', '6195784231545598.100495', '-6195784231130000', 13, 0);
    T('739913244.45121070', '860368559009.796038944105100274', '-859630000000', 5, 5);
    T('861764329971601683.22859', '2143.736206745', '861760000000000000', 5, 3);
    T('745657998914168.484322389', '148060343272051012.9226326609', '-147314685273136844.4', 19, 1);
    T('268901.98203476500', '855.01', '268046.972034765', 37, 5);
    T('639909342.00304640', '44449445535.55077454347', '-43809536193.54772814347', 40, 6);
    T('7902.428166893698163', '85857754789521.06', '-85857754781618.631833106', 23, 6);
    T('83445.3254391483175', '9834649.48792', '-9751204.1624808516825', 22, 3);
    T('29060984660.1270107148349', '64.949714105991919908', '30000000000', 1, 6);
    T('440636324000477.0', '674662787919.575016364431', '439962000000000', 6, 4);
    T('54180.709635644', '1297613319706294.0', '-1297613319652113.29', 18, 2);
    T('2972613693267590.0760894882219202', '389260346762856.0108568851', '2583353346504734.0652326', 24, 1);
    T('977349543133490086.1493299477663843', '97855455369010744642.68159', '-96870000000000000000', 4, 1);
    T('99217614.3242', '4099093283987205.73206614131103609', '-4099093184769591.4', 18, 2);
    T('9559000880088290581.9094', '15300691.145', '9559000880072989890.7644', 40, 3);
    T('9208759370.83724848', '792416462143.2546', '-783207702772.41735152', 20, 2);
    T('70562.54774383786352', '6979874313341515669.47055974434451429', '-6979874313341445107', 19, 3);
    T('40400045928491568708.88', '481871559107127959.083', '39918174369384440749.797', 32, 5);
    T('42645892213767612341.6797040', '4578809.96410890', '42645892213763033531.7155951', 30, 3);
    T('1720611.053431405526', '954069807281478459.4249376331', '-954069807279800000', 13, 0);
    T('32042368404659.65521675383316414', '3472738983828.3600498906744228344', '28569629420831.3', 15, 5);
    T('26.991480625', '43455568218904453.66395535976', '-43455568218904426.67247', 22, 4);
    T('51241494121.1929688893647232', '311917951.0', '50929576170.19296889', 20, 0);
    T('317125.16119958661600097', '4627007586225414525.508160402613', '-4627007586225097400.34696082', 27, 6);
    T('7480633.057', '65663236.880560', '-58182603.82356', 34, 0);
    T('5708671374.713741', '23689759.2', '5684980000', 6, 5);
    T('706035.791203', '96341627306155.77672909', '-96341626600119.98552609', 33, 0);
    T('36483981612.3585496994694285923', '99929.759270067263207', '36483881682.59928', 16, 5);
    T('961170447.53', '82783.579551754306143209', '961087663.9504482456938567', 25, 3);
    T('24241086513070.2', '68205019443155.993740698768', '-43963932930085.793740698768', 34, 1);
    T('20.572977014917401', '1269124.57471918894301673992', '-1200000', 2, 2);
    T('67991.2', '8241663477805187322.9568658', '-8241663477805119000', 16, 2);
    T('4656507.22966714', '21599679696291921964.257731705486814140', '-21599679696287265457.02807', 25, 0);
    T('55551.1074889631731168155', '3105858878644775.451021528035894', '-3105858879000000', 10, 3);
    T('359984059275.963978525', '33734655635.1091983', '326249403640.854780225', 32, 6);
    T('420233447979.100079', '8.994222', '420233447970.105857', 20, 5);
    T('66113884463523.775000075497554469', '553493042059850.95505544135', '-487379150000000', 8, 1);
    T('100686261811.5', '8.9107583324', '100686261802.5892416676', 35, 2);
    T('266501947352.64593184309927456', '84723562165350452375.70', '-84723600000000000000', 6, 0);
    T('480246.7', '51337489174.062321676884401', '-51337008927.362321676884401', 31, 4);
    T('45963746.989100721572', '894298640002.119', '-894252700000', 7, 5);
    T('2969579317.181931633463140', '3103.010202093350499654', '2969576214.171729540112640346', 40, 3);
    T('36197397.19173930351575613666', '17.777687073308413', '36197379.41405223020734313666', 34, 1);
    T('31431.7163', '18403602.9737644', '-18400000', 3, 4);
    T('70578226509734425210.4383282780437496', '7936.626697630342', '70578226509734400000', 15, 4);
    T('317830955046200398.464428193135854', '184977709221376467.51', '132853245824823930.9544282', 25, 4);
    T('32.19211483669797', '9419920776.64351217240229', '-9419920744.45139733570432', 31, 5);
    T('4477543282.3498976446801901', '63933.4', '4477479348.9498976446801901', 35, 2);
    T('738432601089.18485445', '303996559388871.552280911885', '-303258126788000', 12, 5);
    T('6537.9', '8318080416820182.8294992860', '-8320000000000000', 3, 0);
    T('27784382922080.56204526467', '5.39830220', '27784382922075.2', 15, 2);
    T('6494306189597938520.85309204298444735', '547545731.0', '6494306189050392789.8530921', 26, 0);
    T('8907864026372468890.584670699', '3799484.8028294461299775', '8907864030000000000', 9, 0);
    T('2.428561399435', '61975950.442997922499', '-61980000', 4, 6);
    T('22.71281437822', '412746910762.6369117822166296', '-412746910739.92', 14, 5);
    T('12602983836577155595.5', '49729207295425766.2742813552353', '12553254629281729829.2', 21, 4);
    T('532.440622496375077754', '52089329858197633.782', '-52089329858197101.341377503625', 29, 6);
    T('4998740719.1540009447758714', '87144231699939.33577900', '-87139232959220.1817780552241286', 36, 6);
    T('301574806471.41120711612973146', '90281.3', '301574716190.111207', 18, 1);
    T('227874.397965722041', '7342044307234027.533', '-7342000000000000', 5, 6);
    T('596140542547273419.7572998951251684647', '161641626219858.76870244900179', '595979000000000000', 6, 2);
    T('169587109.65969570135206001744', '9325474248800840.974809401897148961', '-9325474079200000', 11, 4);
    T('442250.493914362360331836', '398092550510793.975252669521', '-398092550068543.481338307161', 27, 0);
    T('57.5638936117527481', '43677475.06', '-43680000', 4, 4);
    T('74088493880276790.467722816', '3293488635678240004.7858062', '-3219400000000000000', 5, 4);
    T('4737232004.20489174916', '359919.1', '4736872085.10489174916', 35, 1);
    T('317318908711939.6724396', '45729690847520.735540280242647', '271589217900000', 10, 4);
    T('379350603629.842', '63427011.00262011259', '379287176618.83937988741', 31, 6);
    T('33066357397.520057422340405196', '4204790623436198.6851537614247018', '-4204757557100000', 11, 6);
    T('64847850512337717931.019640453463440', '4692779509.926554504985827', '64847850510000000000', 10, 4);
    T('3612789.8115521', '9.0', '4000000', 1, 4);
    T('6542688211.778814396653924541', '78479.8', '6550000000', 3, 0);
    T('1338506747271852.4438', '18619093192974956211.395056920042113745', '-18617754686227684358.95126', 25, 4);
    T('159095.260360503', '234928734268577733.3696574228399454', '-234928734268418638.1092969198399454', 36, 0);
    T('9122718.2614332200882', '8011.513698048542311', '9114706.74774', 12, 5);
    T('6539.0522533', '144973253362887.9', '-144973253356348.8477467', 27, 0);
    T('721.96227841048', '4267.472732330054156', '-3550', 3, 5);
    T('95501190788087079674.77', '63735017991791.1987423208510', '95501127053069087883.571257', 26, 3);
    T('277844425075085238.89511465490', '50278.0', '277844425075034960.8951146549', 28, 4);
    T('80889324766.864983088029336', '18623913266129081273.7929', '-18600000000000000000', 3, 4);
    T('46.65933628183457', '99843161.03562471982241688', '-100000000', 2, 4);
    T('5648510523716.392395', '877.3611403250329', '5648510522839.0312546749671', 39, 6);
    T('4396.879473684', '4361915.94626296666580602', '-4357519.06678928266580602', 29, 5);
    T('999385517776042299.278417028648', '229648101526445568.92799703739029', '769737416249596730.35041999125771', 39, 2);
    T('860139660673640052.19188841867662722309', '705645612177270.8928358992197674781', '859434015061462781.29905', 23, 5);
    T('86955203827598.0', '51134011998690.24849847081874', '35821200000000', 6, 2);
    T('3934589574084100.1000418453599061078', '3788925188338.360799063', '3930800648895761.73924278236', 27, 4);
    T('66005.868', '88097814632507579.4249904994', '-88097814632441573.5569904994', 36, 2);
    T('5268845448759403.17519346077745986085', '8956236.9397069784133116', '5268845439803166.235486', 22, 1);
    T('61763184.508080621162791', '189357294.750220868', '-127594110.24215', 14, 0);
    T('6424785260585508.88885291074', '83742394988752.8504632214', '6341042865596756.038389', 22, 3);
    T('60850515680404526.95019294200722672772', '306029404876636627.1121962456213148', '-245178889196232100.1620033037', 28, 3);
    T('5.6012442131069873', '884447623549.892720429267394142', '-884447623544.2', 13, 2);
    T('99359881211562.093812451586950595', '1.396', '99360000000000', 5, 4);
    T('8394839096213826662.8630312105735757', '68782145951656631.19754', '8326056950262170032', 19, 4);
    T('835563.86223923193138', '7593340944999.69746019', '-7593340109435.835220958069', 25, 6);
    T('20429221995758225.67558545885105', '8639191668.8750787284056', '20429213356566556.80050673044545', 32, 4);
    T('6.0', '6800012.69877953176176', '-6800006.69877953176176', 21, 2);
    T('8687807061300959319.0', '6963865880706.711461', '9000000000000000000', 1, 6);
    T('7565304075.360883732768461854', '79537065877956.00487977946494980721', '-79529500573880.643996046696', 26, 2);
    T('696971624062909393.1837233325127890', '52638854991703.6088785988856763777', '696918985207917689.57484473362', 29, 3);
    T('72381283027122215644.88276', '823.16068418202', '72381283027122214821.72207581798', 35, 6);
    T('90400353.499963723205488', '9366098801.136', '-9275698447.636036276794512', 32, 5);
    T('10073446.946065', '426342.475393836585754', '9647104.470671163414246', 23, 5);
    T('49861.4707424361166163477', '75036335314980225.981390690858606355', '-75036400000000000', 6, 3);
    T('29.0', '36848075423135.99560149510019834668', '-36848075423106.99560149510019834668', 37, 3);
    T('9988578382107905704.728210899550354', '92701506.2986885881', '9988578382015204198.429522311450354', 40, 6);
    T('44748715619884149834.0', '67615367968579570596.79528654368781930', '-22866652348695420800', 18, 4);
    T('83583438527043770327.57694312772370', '1059971.40740', '83583438527042710356.1695', 24, 1);
    T('89.6246', '3280130569000.176', '-3280130568910.5514', 25, 0);
    T('64.8', '882.673033592653715', '-817.873033592653715', 35, 6);
    T('45470328805.1465083050354123766', '29758469765.3640172432547606', '15711859039.782491061781', 23, 5);
    T('326.21914735816', '10942.367784794', '-10610', 4, 2);
    T('6.8685837948010', '831.508721561592440', '-824.64013776679144', 31, 4);
    T('555376402.60993404675139212', '114768880958628.4', '-114768325582225.790065953', 24, 4);
    T('52006882098858652.0', '72337907.176020071452636', '52006882026520744.823979928547364', 34, 5);
    T('9.2438362', '80574685381.70566149955380617378', '-80574685372.462', 14, 3);
    T('265419416714242.322463557', '378793523141328589.217160272048645662', '-378528103725000000', 12, 0);
    T('1840516.0', '710616630955666.61502057178', '-711000000000000', 3, 0);
    T('7519061.2714055348063804790', '7008.9', '7512052.371405534806380479', 39, 3);
    T('140870707284223.207453412348167', '60.4770862030594', '140870707284162.74', 17, 2);
    T('3819.0', '9558.6434115', '-5739.6434115', 11, 0);
    T('8703210.65', '12.1841000086', '8703198.47', 9, 4);
    T('98470417997400600.7416115139', '808781491087260335.849713413054732947', '-710311073089859735.10810189915473', 32, 4);
    T('629.1551137358867930', '643831248649309502.77', '-643831248649308873.61489', 23, 6);
    T('71907283315721865.1', '1821973540022329.39107512547432818094', '70085309775699535.7089248745', 27, 4);
    T('4634008199.54897851218', '99888887343218365513.68109779', '-99888887338584357314.13211927782', 40, 1);
    T('872933049173.436', '3029297.2516116835182471270', '872930019876.184388316481752873', 35, 5);
    T('1634436841.66589556390561722987', '9918222429811870.745846126775039553', '-9918220795375029.0799505628694223', 32, 6);
    T('22.71667634973003171', '36005671061.57775774568371106', '-36010000000', 4, 3);
    T('961321.82183', '101532422180719.2045173909', '-101532421219397.3826873909', 36, 4);
    T('118077.4593203947755231', '87173138249.4', '-87173020171.94067960523', 22, 3);
    T('477976.15295185999555', '89.191441590226054260', '477886.9615102698', 16, 5);
    T('4753071069.37', '4591.730678', '4753066477.639322', 27, 1);
    T('539025938827.751333', '5183018001.0582', '533842920826.693133', 26, 4);
    T('71968254056.53166379981333', '4940304368899.842519185', '-4868336114843.31086', 18, 5);
    T('4.81819', '95380318.50302713431823636020', '-95400000', 3, 3);
    T('636349341593029.943328856076119152', '28094815164362.333484894586861', '608254526428667.609843961489258152', 37, 1);
    T('31.04454', '9601215638739320.68630384045646056280', '-9601220000000000', 6, 3);
    T('98853371809280.6', '99695198.3865', '98853272114082.213', 17, 5);
    T('31671911710176187.031348', '7696281.194800397910', '31671911702479905.837', 20, 0);
    T('37421396.50475130840047928071', '17674592158861365501.01098301072', '-17674592158823944104.50623170231952071929', 40, 0);
    T('7.6', '638100171686.6', '-638100171680', 11, 5);
    T('6765784888347847069.396474787', '934194430907510437.114731589883686653', '5831590000000000000', 7, 1);
    T('1.896164673862015427', '811401509.49805', '-811401507.601885326137984573', 39, 2);
    T('6806099564740.979640', '352.99240877961', '6806099564400', 11, 5);
    T('54231083.368240149184', '246454942062936.0533007', '-246454887831852.685060550816', 30, 1);
    T('534768099068.4900920498', '3445861.0', '534764653207.4900920498', 38, 1);
    T('26062.6695543690113194', '85478208954774.643944565', '-85480000000000', 4, 5);
    T('37.658212443336004212', '13780034853.0', '-13780034815.3', 12, 4);
    T('37502467996613117.5317589463552428', '680250672.390346', '40000000000000000', 1, 4);
    T('1605.195494792', '9230561896175562131.5162679789459371', '-9230561896175560526.3207731869', 29, 4);
    T('830748515.2982998415326054', '47544113795209801720.0', '-47544113794379053204.7017001584674', 34, 0);
    T('3102743105765240.84976505130', '3904.83151876', '3102743110000000', 9, 4);
    T('615702120689885833.56685994547327', '792.135300', '615702120689885041.432', 21, 5);
    T('41245.0', '472561072440.975', '-472600000000', 4, 5);
    T('84350717638.7559039029', '766.3073', '80000000000', 1, 6);
    T('6.01268', '384014.17', '-384008.15732', 25, 5);
    T('6017550703.93760538', '830087.81650833621656578', '6016720616.12109704378343422', 30, 3);
    T('416.727', '4353458876651.65', '-4353500000000', 5, 5);
    T('94610.5611298553', '14584700895736153.854879', '-14584700900000000', 9, 4);
    T('68895.706995401635648', '6928343661658.100358506470082582', '-6928343592762.39336310483', 24, 2);
    T('40406905579956214220.513205197971', '75418791457910912.0', '40000000000000000000', 1, 5);
    T('5949744277013.1510627', '15356.853646002052966', '5949744261000', 10, 1);
    T('1053.7549967678862', '143834.45427264842905952', '-142780.6992758805428595', 22, 1);
    T('48192790.72', '2416976.05438', '45775814.6657', 12, 2);
    T('9363.931075622', '95056.717291408104', '-85692.786215786104', 38, 6);
    T('81852501092313887953.4180383758885', '2750063.883413960450304', '81852501092311137889.534624415438196', 35, 4);
    T('7151.9049042588918', '72342821771269.2', '-72342821764117.2950957411082', 38, 6);
    T('62242217348416316042.15761519736', '1309.5407178722383180', '62242217348416314732.6168973252', 30, 0);
    T('56967803791956.914355932231', '5076600.35249', '56967798715356.561866', 20, 4);
    T('518255415351.9402994880455', '73549593309773712.11049274740', '-73549075054358360.2', 18, 5);
    T('37927062751378957599.7', '94160116923.4568702726194828579', '37927062657218840680', 19, 0);
    T('584484.3718309', '90035.2234149906054690', '494449.148415909394531', 28, 0);
    T('539.17662888548148', '2372.715', '-1833.5383', 8, 1);
    T('139007.961370392759753798', '733.3522664258091', '138270', 5, 4);
    T('5.131', '365684008233607.44255803', '-365684008233602.31155803', 40, 6);
    T('9.79652873', '136747528326184.82199179412453742815', '-136747528326175.02546306', 23, 5);
    T('72935446074520.3975790', '3132063545549.0', '69803382528971.397579', 23, 4);
    T('338512500867031506.63', '5849460.6219', '340000000000000000', 2, 0);
    T('582672773.19793803045648941035', '84735741228.996332920613', '-84153068455.798394890156511', 26, 5);
    T('485655417818961211.6209', '39538168746104.0153869596265346095', '485615879650215200', 16, 2);
    T('30223.644647262240530', '96240002790938614.81316096690804541', '-96240002790900000', 12, 2);
    T('71585210521.4889501', '83308687705466908660.77', '-83308687633881698100', 18, 4);
    T('61021292579.4045', '851197.056577674', '61020441382.34792233', 19, 2);
    T('6091144699.06166533662853145165', '39926012489022.4186292751023968893', '-39919921344323.3569639384738655', 30, 3);
    T('180095784.25153129109905', '699.376827781095', '180095084.9', 10, 5);
    T('41014752629761981625.1336179816085', '31.56', '41014752629761981593.573617982', 29, 6);
    T('6.6530690993838914401', '39043.0', '-39036.34', 7, 2);
    T('3238.9726209059936496', '5605420.4647048', '-5602182', 7, 3);
    T('24491236058284.134244333', '82663493888.9320470048651349484', '24408572564395.20219732813486505', 31, 3);
    T('700339385750.839975327987474', '19576045593541.00434008', '-18875706207790.164364752012526', 35, 1);
    T('43111701901319913.66506635', '534073076047.860579973926', '43111167828243866', 17, 2);
    T('52040719651916886.8712886210', '2393756.95190009', '52040719649523129.91938853', 25, 3);
    T('121112.390', '877449554.101105972', '-877328441.711105972', 31, 2);
    T('7118217651931.2', '89469789558119.9752879', '-82351571906188.7752879', 22, 0);
    T('69447811711.381', '13.348669', '69448000000', 5, 5);
    T('36941668327197.978', '641691358986368.22', '-604749690659170', 15, 1);
    T('55928421228854056961.77709880023844771524', '25849764632213.9141836939644765490', '55928395379090000000', 14, 2);
    T('459.3120096743', '390334685.80', '-390334226.4879903257', 37, 6);
    T('82812395299972437.1106666342168067', '297479542183.70375', '82812097820430254', 17, 0);
    T('64307576312601953306.339', '4802472.7734627', '64307576312597150833.56553', 25, 1);
    T('82318.9889515683560', '7478567745550354833.1', '-7480000000000000000', 3, 3);
    T('25469521986.1995146', '1.12558986805667890', '25469521985.0739247319433211', 33, 0);
    T('3.0', '179788.26', '-179785.26', 25, 3);
    T('1877.30595', '2066805184063365521.916', '-2000000000000000000', 2, 1);
    T('8.2278412469938645', '28430.958922971', '-28422.73108172', 13, 4);
    T('3418975311822.96475080389880', '99324698804.2409279230523593', '3319650613018.7238228808464407', 33, 6);
    T('976057392736.63103', '59411159.809405', '975997981576.821625', 21, 4);
    T('1431997270025.0', '8268339659808004725.41720436', '-8268338227810734710', 18, 0);
    T('4780883055.94574400006045', '69325191571305616322.786004', '-69325191570000000000', 10, 0);
    T('61007974110046.739763470497126', '6752921024019777899.4161', '-6752860016045667852.6763366', 26, 0);
    T('8773.95', '113.71495174270843277517', '8660.23504825729156722483', 34, 2);
    T('7325462189062.0584241', '3095639407253.10039', '4229822781808.9580341', 40, 0);
    T('965018330.5628', '53.5', '965018277.1', 10, 6);
    T('48695004092424749.766523601', '3812773386204588.8055471519344046', '44882230706220160.96097645', 25, 2);
    T('2590574757388450.123613881', '9492775396.1814568369637449', '2590565264613053.9421571', 23, 0);
    T('5071568798.5', '74822853547.7102084', '-69752000000', 5, 3);
    T('21984.68508', '77526703315.657830282', '-77526681330.972750282', 25, 4);
    T('524253903308.57239484477062653819', '407154.91', '524253496153.66239484477062653819', 34, 3);
    T('9044486.953', '220816.1116547', '8000000', 1, 1);
    T('933773018062777901.937379504979', '604460254.5327714', '933773010000000000', 8, 3);
    T('23000.6', '22660724883748222.15586514511282', '-22660724883725221.55586514511282', 36, 1);
    T('13778.638088126420001163', '70.0', '13708.638088126420001163', 40, 1);
    T('20590628.041875925026966598', '9367926729147.4619948943', '-9367906138519.420118969273033402', 32, 6);
    T('866838208762970409.2856249371640051826', '616934.2461333', '866838208762353475.1', 19, 2);
    T('2142791584697291426.13170621260180610', '5239103296066466919.17710842225', '-3096311711369175490', 18, 4);
    T('99.3360074981697752', '972675927852.2209804254', '-972675930000', 8, 6);
    T('2224.231599408060740', '817117848304477979.9270036643101388841', '-817117848304475755.7', 19, 3);
    T('479275940198.300648068403', '322310249463624350.297431395', '-322309770000000000', 9, 2);
    T('5752918826550574083.0740034904287', '818995359795178714.7084746984156663', '4934000000000000000', 4, 6);
    T('36505782550588399006.0', '12.30948296739074', '36505782550588399000', 19, 0);
    T('255351405679539031.0809908920', '48444709892799298219.3233495', '-48189350000000000000', 7, 2);
    T('48854.0', '73.95', '48780.05', 15, 3);
    T('173.228371', '1861308883992.11106180', '-1861308883818.8826908', 35, 5);
    T('5921897.700844566', '46899191205.3244382094', '-46893269310', 10, 6);
    T('89993567247.20385', '7310945859.7577830', '82682621387.446067', 40, 4);
    T('4.00930994992', '811331.70741', '-811327.69810005008', 38, 2);
    T('90.348', '389659400413921.769471', '-389659400413831.421471', 37, 5);
    T('55301122111.870329047', '22813445390.9598924', '32487676720.910436647', 32, 1);
    T('149799135095395704.92899166', '2.09001271165872417', '149799135095400000', 13, 0);
    T('593.84603', '36341019902817932473.038473650', '-36341019902810000000', 13, 1);
    T('9801248845645.78062688', '2585.49', '9801248843060.29062688', 27, 6);
    T('5327521410154213.728350691918789380', '11200272498691555.4076718444867943', '-5872751088537341.67932115256800492', 34, 1);
    T('1393389764.287999', '8631398926188.18', '-8630005536423.8921', 17, 0);
    T('1023981224393725.1919099560033347500', '11723147495.5161453', '1023969501246229.67576465600333475', 35, 1);
    T('846465196268509077.7720115520', '808863.4538210167713325996', '846465196267700214.3181905352286674004', 38, 6);
    T('69047476.60', '6416103768716294.5718166', '-6416103699668817.9718166', 29, 3);
    T('662188286415824.37122', '6463002752484.0', '655725283663340.37', 17, 3);
    T('725920.2766810737522414', '17524513811368.0', '-17524513085447.7233189262477586', 35, 3);
    T('81149.380934560289392213', '112653304552.2756706', '-112653223402.894736039710607787', 36, 2);
    T('642.28748483', '714.2020536021175422037', '-71.9145687721175422037', 39, 1);
    T('6147633295512336836.99137076758', '80899911785.0', '6140000000000000000', 3, 1);
    T('9686565.559106', '7048794885003326329.473493307338', '-7048794885000000000', 11, 5);
    T('610.6368882658', '1090100514.073572501', '-2000000000', 1, 0);
    T('995861133.2', '9323139546537732605.685', '-9323139545541900000', 14, 6);
    T('846871325975703.00239310', '94018895547728.4722456', '800000000000000', 1, 5);
    T('6178337233945.32429908796008960', '402063386.0877501', '6200000000000', 2, 2);
    T('3178.1262955343931', '93161800971499.2551021192222', '-90000000000000', 1, 2);
    T('6786.0595', '746164429206.185198689769', '-746164422400', 10, 2);
    T('0.11821088088926348023', '15096535460851615.313568130583949', '-15096535460851615.2', 18, 3);
    T('454364.24346275800112', '35832762948.1886444962308255', '-35832308583.945', 14, 4);
    T('2532074092.0', '88009842.58', '2000000000', 1, 1);
    T('0.090211578001781023', '913.4', '-913.309', 6, 1);
    T('70561843487357.2725294362160487', '38217338666852757452.557367', '-38217268100000000000', 9, 1);
    T('425728459427.41483833679635646611', '9240969.9377655503665567842', '425719218457.477073', 18, 6);
    T('355807102659.3374787303', '90042695901640866.96517114312', '-90042340094538207.63', 19, 6);
    T('6.19521409639743620', '228313676310580.2151818', '-228313676310570', 14, 6);
    T('57887968208927903375.001511', '54029382616159150.0', '57833938826311744225.001511', 31, 3);
    T('490792.364893065061875', '203506552679889875.235951633', '-203506553000000000', 9, 3);
    T('46781825327.935542', '460172504032799.6', '-460100000000000', 4, 6);
    T('9398.98023945160358591', '3.39537177483', '9395.58486767678', 15, 0);
    T('72880648.76790865671148674815', '79824400994.53129172', '-79751521000', 8, 3);
    T('8.321', '928.90373405036188374', '-920.58273405036188374', 20, 3);
    T('3984318.7653057235454552213', '2499385577373145031.334787401', '-2499400000000000000', 5, 5);
    T('187675764.2734958789', '94285783119567497.7866341', '-94285800000000000', 6, 4);
    T('5697658857321760202.2', '412126368210.8460840965', '5697658445196000000', 13, 2);
    T('2302924597.5363', '286520597.258035616524936', '2016404000.278264383475064', 35, 3);
    T('499105963761743.53571', '41624344446761554.569298747295308961', '-41125238482999811.03358874729531', 31, 0);
    T('7794139.88069', '848385.3044722727915655', '7000000', 1, 0);
    T('46732.36703439376085', '163118173025799711.4385817661', '-170000000000000000', 2, 3);
    T('72340807208.5688748305325360', '90331278977.36382047927625843', '-17990471768.8', 13, 0);
    T('509150074110278948.1058522959114063113', '8.7997475', '509150074110278939.3', 19, 4);
    T('954.917', '7380284764.80412907902786204615', '-7380283809.88712907902786204615', 39, 5);
    T('42663693.64', '991.37600220245380', '42662702.2639977975462', 23, 5);
    T('162174371.4', '80.08037134709595', '162174300', 8, 0);
    T('58905425027160020.807326281', '2.03312853854440880', '58905425027160019', 17, 2);
    T('8884181501497978.20695203606988650807', '8414598.0', '8884181493083380.20695203607', 28, 4);
    T('80269217427099.2485191737855802549', '177506.40', '80269217249592.84851917379', 25, 5);
    T('6254589.964318712682', '34979522686512944708.1602457409458831592', '-34979522686506690118.195927028263883159', 38, 6);
    T('5890839280126243.75309925061202', '48.29', '5890839280126195.46309925061202', 37, 4);
    T('783.042353', '7671156072027135.1993057511', '-7671156072026352.1569527511', 32, 3);
    T('1231.04005621616316', '8035578985951529589.3055519103699567155', '-8035578990000000000', 9, 3);
    T('1859972.0', '287236446831.8156836584', '-287234586859.815', 15, 1);
    T('58.1200476963275859', '8765640923867.862', '-8765640923900', 11, 3);
    T('428095.821869313667576258', '315931366.41605475957429717', '-315500000', 5, 5);
    T('473344638846196757.72600', '4827322194916.9234592678202', '473339811500000000', 10, 4);
    T('1182567338046723477.0045', '55523622350729832.4955909866470631067', '1127043715695993644.50891', 25, 0);
    T('27757335127.21814', '7.9089', '27000000000', 2, 1);
    T('576.0', '8217486035910565.9220609620905', '-8217486040000000', 9, 5);
    T('306.245', '376129885124829508.113', '-376129885124829201.868', 30, 4);
    T('149.193148677954298', '6924025191.96669404354851082216', '-7000000000', 1, 3);
    T('59534695.0', '260375266.0', '-200840571', 36, 0);
    T('8355812.6543880815', '724.85982096033', '8355087.79456712117', 26, 5);
    T('27252341996.0', '60452958642532.51119475440', '-60430000000000', 4, 0);
    T('5.0', '50051117926339.21372', '-50051117926334.21372', 35, 2);
    T('25508756.1493518', '43301052453000820.67008779', '-43301052427500000', 12, 3);
    T('2782565590562772.459162445', '26134.195780544120', '2782565590536630', 15, 1);
    T('5378901.8611008743', '3629471993.5660664387', '-3624093091.7049655644', 38, 5);
    T('5.24631431717', '1348247435812.647195448250964', '-1348247435807.4', 15, 4);
    T('745151193636112.588222', '53.9602816152', '745151193636058.627', 18, 1);
    T('66102698295735185366.4730507306081856357', '4735839344859283.350982907900604882', '66097962456390326083.123', 23, 2);
    T('978708.4711752892969570738', '4913595947674429907.4821092478061', '-4913595947673451199.010933958509143', 35, 0);
    T('528473.0604758887989519733', '853448020752423.739154', '-853448020223950.6786781', 22, 4);
    T('72406305297461889.5675734916124', '8884211.485102376197106', '72406305289000000', 11, 2);
    T('985466.3566407', '1509.8295140494468967', '983956.527126650553', 18, 3);
    T('8172237755931588.0', '14.49988', '8172237755931573.50012', 26, 1);
    T('9368491.9162213234', '40160701.6988408495', '-30792209.7826195261', 23, 4);
    T('1767632735691.3', '6527.0', '1767632729164.3', 14, 0);
    T('46619599440.4428762374', '9341.807959896', '46619590098.6', 12, 6);
    T('31706110.917387617727836054', '9633682805078144.0711782', '-9633682773372033.153790582272163946', 38, 2);
    T('6830006291320.81595911', '8997594000412689.9815229035', '-8990764000000000', 8, 3);
    T('7253880009975649989.317737103', '1304464.0', '7253880009000000000', 10, 1);
    T('6979275582.818540', '294643848758.247527365089853', '-287664573175.4', 13, 2);
    T('959106275661392856.497924', '4695076.90518178069601', '959106275656697779.59274221930399', 35, 2);
    T('21964537652081381.0', '298576905249.5911', '21964239075176131.4089', 39, 0);
    T('193635248.247406469998', '886354.7035527', '192748893.543853769998', 37, 5);
    T('4656.42320', '95755384748825984351.18684164061051', '-95755384748825980000', 16, 0);
    T('65.7374890967210', '534255473745.0', '-534255000000', 6, 2);
    T('361115.97005626', '401.757422024294450611', '360714.21', 8, 1);
    T('26762.969', '4.437004045320769', '26758.531995954679231', 23, 3);
    T('2097792104.5', '99875202749.157686578104056116', '-97777410644.657686578104056116', 38, 3);
    T('843325690829.52006', '87937718805918.0', '-87094393115090', 13, 3);
    T('6012.46819830334881974960', '8021038136.5', '-8021032124.03180169665118', 24, 6);
    T('2912563492974960587.742500468797786', '3130120847317709121.7546214882737356766', '-217557354342748534.012121019', 27, 4);
    T('82410353686.49261080', '1150863114117372324.02515274910122', '-1150863031707018637.53254194910122', 39, 3);
    T('411594.219188023038757129', '351299227103981024.0', '-351299227103569429.780811976961242871', 39, 2);
    T('96102828.25978782093', '2296.0233203189160624258', '90000000', 1, 3);
    T('9173.8', '4946.87332', '4226.92668', 23, 4);
    T('96540472.1277153321636', '710.0', '96539762.1277153321636', 25, 5);
    T('674450014197.0', '935.459193', '674450013261.540807', 25, 4);
    T('66.5178059', '7000594458054.0098774698100', '-7000594457987.49', 15, 4);
    T('71522471802711404030.30', '8330.92019263756655013006', '71522471802711395699.37980736243345', 34, 0);
    T('84443193896865439682.657203970', '9330730110.115441', '84443193887534709573', 20, 5);
    T('149357920355813642.68653', '380678813.29483', '149357919975134829.3917', 38, 5);
    T('9366.79043', '59987.6605538152021349', '-50700', 3, 3);
    T('324375574976.3052529', '1814163549711552384.2041775843986', '-1815000000000000000', 4, 3);
    T('95508580586.91831124077089473', '92.669718', '95508580494.24859324077089473', 32, 0);
    T('7279084594.23907407069341', '331303.1435081726', '7278753291.0955659', 18, 2);
    T('446.38213', '144537914365827239.72830046630', '-144537914365826793.3461704663', 36, 0);
    T('247303582822162793.1066393', '43813821655319166.720189', '203489761166843626.3864503', 31, 6);
    T('166058713786301.6614874', '383253578902321352.7164036094', '-383087520188535051.0549162094', 38, 0);
    T('80333756789292.572', '2924141926125.462702', '77409614863167.109298', 31, 5);
    T('70.0', '18154936899625304421.619', '-18154936899625304352', 20, 6);
    T('6.4291544', '28701069790855543.20136', '-28710000000000000', 4, 3);
    T('7.679923205281955', '66983573638427635653.931873262970', '-66983573638427635646.25195', 25, 1);
    T('538.4', '9321082708.0', '-9321082169.6', 14, 1);
    T('9340.39358', '5070227.2707', '-5060886.87712', 21, 1);
    T('514294370327759.2', '22.75998850', '514294370327736.4400115', 27, 4);
    T('9729858363306.899', '736948281452291791.1278', '-736938551593928480', 17, 6);
    T('41209343352.915321', '9980032417025737792.1285827921', '-9980032375816394439.2132617921', 33, 1);
    T('500101562393054.2573667209732', '85605.50', '500101562307000', 12, 4);
    T('797376521534913.923903', '6778347525.943676285197931122', '797369743187387.98', 17, 5);
    T('1689.5162462718380753234', '54890.405637774197', '-53200.89', 7, 3);
    T('666162720880444569.5306923245424776462', '35286752234.8102591', '666162685593692334.7204', 22, 6);
    T('304.188271149521063', '6196.0692451', '-5891.880973950478937', 33, 0);
    T('8.697410422672947355', '99785377811182891278.410326945', '-99785377811182891269.712916522327052645', 39, 2);
    T('552170005797809381.63621234011112', '3313405348643.34915126957898', '552166692392460738.2870611', 25, 6);
    T('53751745198872515.2', '269816992454.5141371239418677237', '53751475381880100', 15, 6);
    T('54443814272199283926.0', '63990415207.70', '54443814208208868718.3', 30, 5);
    T('10570762.7751326180825112', '4.26729', '10570758.5', 9, 1);
    T('6486.5512664787868', '883381534057.7525215', '-883381520000', 8, 1);
    T('96.614601445149', '29.8370985207', '66.777502924', 11, 4);
    T('7329412421438366.5295544', '25948976650488.29352008430162502', '7303463444787878.236', 20, 6);
    T('887723579.0', '1825052566333267144.52434056906626038', '-1825052565445543565.524340569', 28, 2);
    T('86164.5598983929323514121', '36138904132065596722.8602', '-36138904132065510558.3003016070676485879', 39, 1);
    T('70704481640998489.12614450965526', '53839354.66', '70704481587159134.466144', 23, 3);
    T('71632877601.84', '4.16365', '71632877597.67635', 20, 0);
    T('71011793396477.21202926566646', '824185.171', '71011792572292.041029265667', 26, 2);
    T('3888123569567956.7329', '869.1', '3888123569567090', 15, 6);
    T('4.7491085', '8523731832285849.3772159', '-8523731832300000', 11, 3);
    T('559038264279078122.754', '955747491263.7733168382407832382', '559037308531586858.98068316175921676', 35, 3);
    T('10214327197862.432069', '8161.38340180580916385853', '10214327189701.048667', 20, 4);
    T('32632.0', '416067255273.69893', '-416067222641.69893', 36, 2);
    T('7823218925974.184', '6257123131343862962.98742739448563154211', '-6257000000000000000', 4, 6);
    T('211355461347.7437303258012349', '8514196418.2550', '202841264929.488730325801235', 27, 6);
    T('64729.9110', '296502128215.669741687', '-296502063485.75874', 17, 6);
    T('28430355111160.52004960586313043009', '5905577991549234898.2351248', '-5905549561194123800', 17, 3);
    T('95640748472.43315315254185785', '239.94237995658446642271', '95640748232.49077319595739142729', 31, 5);
    T('8773930425051.554532221952718', '6556770.15063445061766680454', '8773923868281.40389777133505119546', 35, 4);
    T('7735320845171821.7', '7341.6201876070006833', '7730000000000000', 3, 1);
    T('2387196.7125606309668163405', '173099.905', '2214097', 7, 0);
    T('359884555.40', '55.96084', '359884499.43916', 16, 4);
    T('220.259978170', '9229182284656878498.9156627', '-9229182284656878278.65568', 24, 1);
    T('47404.0', '689842.6547799145', '-642438.6547799145', 20, 2);
    T('1002647560105586.1883394', '6749764084043575417.2076866242045401', '-6748761436483469831.0193472242045401', 37, 3);
    T('71.19001913886540464483', '2142744439100289259.05111142765731647', '-2142744439100289187.86109228879191182517', 39, 6);
    T('567562021594857.3192', '90.536434444148', '567560000000000', 5, 5);
    T('9380.18666061181118820395', '9064151861928334.5976059484', '-9064151861920000', 12, 3);
    T('95683594578659.8183', '378709581417.8916891420018330867', '95304884997241.92', 16, 3);
    T('19011114830303.893', '30922373750.9304889439888463911', '20000000000000', 1, 6);
    T('14278125038.05430337397363287913', '83057344119025.57', '-83043065993987.51569663', 22, 5);
    T('135957149.0', '72236392786184.0', '-72236256829000', 12, 4);
    T('34.980', '475255391091144.8272607111', '-480000000000000', 2, 4);
    T('268728.40019311803782', '68817.92405131', '199910.47614180803782', 20, 6);
    T('700959805.306334430788', '348445569.6011390710', '352514235.705195359788', 32, 5);
    T('6705319219223594.14109235817896', '7870121844519504869.0', '-7863416525300281274.85890764182104', 38, 4);
    T('13048017178.0', '49784516.97252399791180', '12998232661.027476', 19, 3);
    T('389.8970348', '7145509126755903261.24168', '-7145509126755902871.3446452', 30, 0);
    T('6678638703556.389056723157537', '1945218132.65555729152674', '6676693485400', 11, 6);
    T('4509.210936049454322832', '62938822894975364068.50875890', '-62938822894975359559.29782', 25, 2);
    T('42.3035427504040', '99.17', '-56.866457249596', 40, 6);
    T('84.60436692', '248485912917127644.59845512', '-248485912917127559.99408', 23, 1);
    T('7158571.72997952586565053', '830022.944122257640372', '6328548.78585726822527853', 36, 4);
    T('232439462614520032.5713650', '82.1106', '232439462600000000', 10, 4);
    T('167196514360.2248000911972', '73272566982.321767661262', '93923947370', 10, 1);
    T('9647169207596244696.802529280115437871', '1295152982362704214.9916571431890', '8352016225233540481.810872', 25, 5);
    T('13489087613498695674.25357770671406203', '57085803104163.5585949388751412', '13489030527695591510.695', 24, 6);
    T('3142830.93611019709', '2212485135530074781.6', '-2212485135526931950.66388980291', 38, 4);
    T('57946189842645937.7972975248280', '20781282159755222.624', '37164907682890700', 15, 4);
    T('96634765300327.031791289335', '784334.170005627534', '96634764515992.861785661801', 29, 0);
    T('57240555.9146', '5617328839433877289.9587', '-5000000000000000000', 1, 1);
    T('54893977713049.39598152178106926', '443259077950054.0977261', '-388365100237004.7017445782189307', 31, 4);
    T('2663.631143938660129918', '309567978280.1', '-309567975616.468856061339870082', 40, 1);
    T('20566470892408.86989906562015581144', '30418823205.74325', '20536052069203.126649065620156', 29, 4);
    T('79353173243537228.4339581983892727', '18373141969215422323.060306990964257', '-18293788795980000000', 13, 0);
    T('6142320896024941458.804811945126', '14897113640006396.906735634887394100', '6127424000000000000', 7, 2);
    T('56026232021321.9', '734.3344777300553963505', '56026232020587.5655222699446', 27, 6);
    T('569872.37276201775931342', '446108584211271.98746978714222902', '-446108583641399.61470776938292', 29, 5);
    T('17.8429047', '11457160280770.609445', '-11457160280000', 11, 1);
    T('648.45068834231782591', '4416009184315.38347651569514895', '-4416000000000', 5, 4);
    T('29.848627', '709839192882.36621218369027128', '-709839192852.51758518369027128', 30, 1);
    T('802192709600620.0392', '971786242546749.491291913344748942', '-169593532946129.452091913344748942', 39, 3);
    T('3883574617221.48237322880195917308', '572653009664716084.95708484379635714', '-572649126090098863.474', 21, 1);
    T('6979458387279385429.87332139683', '2387284580423.4457946334484', '7000000000000000000', 2, 5);
    T('7927.65873347', '592.263382216838', '7335.395351253162', 19, 5);
    T('9009920689035903.72784', '93314.869109582', '9009920688942588.858730418', 32, 1);
    T('94097369566.0966881771', '37610596.8213540409', '94059758969.2753341362', 26, 2);
    T('477399779269747404.588762626206902', '113588146.27051641271952', '477399780000000000', 8, 0);
    T('6801053738899205785.2796639083020', '602.290', '6801053739000000000', 10, 2);
    T('841539.59890584005253495', '547032.536595156', '294507.0623106840526', 19, 0);
    T('213035252863.99151471723211491', '205731693192101175.0089707836011380418', '-205731480156848311.0174560663690231', 34, 6);
    T('602811415017154.25769417689', '6194997178300322937.8949263366', '-6194394366885305783.637232', 25, 1);
    T('219.009333192', '706833624989012721.00589', '-706833600000000000', 7, 4);
    T('46292867419275983718.4850109087607677189', '136980.34788497477', '46292867419275850000', 16, 5);
    T('32465081982482336.885', '98.1', '32465081000000000', 8, 3);
    T('27421522619.376233461697887703', '455637493740094.425393724531', '-455700000000000', 4, 3);
    T('5341978937487.92891519768313424496', '380279839037373316.747688369', '-380274497058435828.81877317131', 29, 1);
    T('7025371015.58326610753215', '3683801035904367.421', '-3683794010533351.83773389246785', 33, 1);
    T('14.816862648', '1558853051250.0751704014135014', '-1558853051235.2583077534135014', 29, 4);
    T('80314531.355329038099', '455900904321303.73611266566793', '-455900824006772.38078362756893', 30, 3);
    T('6768.7841805', '172559413112569984.4', '-172559413112563220', 17, 5);
    T('5374670166272977966.5', '8958639225790.962', '5374661207633752100', 17, 1);
    T('339860751491531.45675786485808455452', '234008115825461929.6748081', '-233668200000000000', 7, 1);
    T('8142664562021.0', '59106615106357.3019', '-50963950544300', 12, 2);
    T('85.85', '83977755797120.0', '-83977755797034.15', 26, 0);
    T('6671770400.14164191728662485', '249227172097.403363607936439', '-242556000000', 6, 3);
    T('8830913.85067107237026301', '5797.638232577914385021', '8825116.21243849445587', 21, 1);
    T('3010.0889', '1098369331977979444.1569231705286', '-1098369330000000000', 9, 4);
    T('425.5', '746562841557.73388418569755580', '-746562841132.2338841856975558', 35, 3);
    T('2025160231659278.876152027365', '726.81409639240645', '2030000000000000', 3, 4);
    T('1098426617.296741855620381', '993.941790573640486', '1098425623.35', 12, 3);
    T('39522151.251', '28.756473', '39522122.494527', 33, 4);
    T('44.88935', '27300451115.0', '-27300452000', 8, 0);
    T('50078117723915297953.22399618858', '3424619.18391389609897955', '50078117723900000000', 12, 4);
    T('8.040202661499175015', '448995896.1221282881', '-448995888.081925626600825', 25, 0);
    T('3189.161179681914872', '56523.652427704', '-53334.49124802', 13, 6);
    T('917921339762.3796421', '61662373723093.67558262695603028541', '-60744452383331.29594052695603028541', 37, 1);
    T('62716037967425059800.783807586997668946', '213926081202.3499', '62716037753498978598.433907587', 29, 0);
    T('32563910005815138875.557075203631442', '0.501687158907408', '32563910005815138875.05538804', 28, 4);
    T('7.2994029530434', '823317453097.578116209168', '-823317453090.278713256124', 24, 1);
    T('60990.5436', '9543228492000711985.3718', '-9600000000000000000', 2, 0);
    T('2831827558.38506962575510700893', '11170243180.2228147969774', '-8338415621.837745171222', 22, 2);
    T('749354063663556557.125', '3582917379389.15', '749350480746177167.975', 23, 5);
    T('434544390799.383', '186502997837530272.19555977495946', '-186502563293139472.81255977495946', 36, 6);
    T('1359778780165982.404558', '3861595868169.901030997701397', '1355917184297812.5035270023', 26, 4);
    T('692098806991981.00689669806962527', '9.01226824839607', '692098806991971.99462844967355527', 39, 0);
    T('54689996958.6450792019551336845', '9404506852.6978098717220416', '45285490106', 11, 5);
    T('19320077583661160606.03947099271483', '18224789183028865002.3640877053177811201', '1100000000000000000', 2, 6);
    T('4918.49509179648940452087', '7467203482.5', '-7467199000', 7, 0);
    T('78318238637938289.495775894453053', '594638980753.431960369082283851', '78317643999000000', 11, 5);
    T('96234.4734563', '34.4058299660742005', '96200.0676263339257995', 34, 0);
    T('20.19500', '99.7703909880', '-79.575390988', 25, 5);
    T('5444700940.9274818', '4439744300777890.861564075', '-4439738856076949.94', 18, 3);
    T('8257149700.0081726149493170847', '5892320277698.95452', '-5884063127999', 13, 4);
    T('8874689484.19290299160', '632419139327.673120357263445831', '-623544449843.4802173656634458', 28, 4);
    T('677511509.04348848029197878', '667175584.2366775', '10335924.8068109803', 18, 0);
    T('10023.30164211338054750958', '347.72206680510045431023', '9675.5795753083', 14, 5);
    T('6911027518577.3', '7028424932443415988.52535749627976005', '-7028418021415897000', 16, 4);
    T('46724286074098.0', '22689596161684099.9', '-22642870000000000', 7, 2);
    T('562981.8', '607461738914.8335808820295823121', '-607461175933.0335808820296', 25, 4);
    T('550704.679585566555', '55412736.82562648218', '-54862032.146040915625', 32, 4);
    T('95552028061417599430.116284926887', '52.0', '95552028060000000000', 10, 6);
    T('0.5309232703', '828890962750.368959', '-828890963000', 9, 6);
    T('78339519.574500615412', '118085754145251473.24771378686288', '-118085754066911953.6732131714509', 31, 3);
    T('6971702653.32884', '820629.9010327649269185875', '6970882023.427807', 16, 1);
    T('980501093.68815074936', '44110.02342624743026250072', '980456983.6647246', 16, 2);
    T('20561.234292979455612034', '78171606814856.12793238086790296341', '-78171606794294.89363940141', 25, 2);
    T('88655202239.92610086997622368', '155643971685758.8772390636', '-155555316483518.95113819362377632', 40, 4);
    T('51.9118631827881568', '2970780773119328986.6496', '-2970780773119328934.738', 22, 3);
    T('23116067912.203229', '905663410.39348327', '22210404501.8098', 15, 2);
    T('473014441510511.645559695015', '28783323629.031', '472985658000000', 9, 1);
    T('1038045082522.681963246102253727', '40884319733867.97', '-39846274651345.289', 17, 0);
    T('899932729698216.9471564911', '94064173.7', '899932635634000', 12, 6);
    T('269877796051987229.78702169053907555927', '25512613.12', '269877796020000000', 11, 1);
    T('275216464281882788.027724144974866', '84424611972000.1135310597494691', '275132039670000000', 12, 5);
    T('349.044815253', '2630648.35751422221', '-2630299.31269896921', 37, 3);
    T('880206033.832339', '414828047117.9632', '-413947800000', 7, 1);
    T('876298475521270.742831685550245256', '5372095241.042406500631', '876200000000000', 4, 1);
    T('3202875050974.9791', '162483725715109.341189631', '-159300000000000', 4, 3);
    T('5088579330.3488', '987313.70737976526', '5087592016.64142023474', 37, 0);
    T('2516966222082488291.90089798', '4.0', '2516966222082488287.90089798', 33, 1);
    T('493449017186123385.2846052057', '1486459417084.4', '493447530726706300.8846052057', 28, 2);
    T('3367.493', '38428560.616084338087081755', '-38430000', 4, 5);
    T('4148276875462.5170296', '656266.125234353', '4148276219190', 12, 3);
    T('17781067569942.5719941999229793225', '3.29538908253365151167', '17781067569939.27660512', 22, 0);
    T('2307770353224.098', '7515131147102658.07417066', '-7512823376749434', 17, 6);
    T('9.46674051140359', '3015.8465203251183188', '-3006.3797799', 11, 3);
    T('6074020676942974461.4549146763508', '7649096920.30558129347', '6074020669293877541.149333', 25, 6);
    T('1797914.15119182791', '281379560.6458', '-279581646.49460817209', 34, 4);
    T('65543.3481889317868', '3454.65576', '62088.6924289317868', 32, 1);
    T('96.787', '87461501.83592006472837', '-87461405.04892006472837', 25, 4);
    T('4489690205.7106686002800500', '843.001583499070500705', '4489689362.709085101209549', 25, 1);
    T('39938415851502.67730076396096', '761457424.6131422674844492316', '39937654394080', 13, 4);
    T('81389504437631737939.03943377774', '8660315522475.84093165832', '81389495800000000000', 9, 4);
    T('4400095708251.259081091218', '51221084548.826798012939717950', '4348874623702.43', 15, 6);
    T('453372108471.577787472333', '87228.34793546485409111', '453372020000', 8, 4);
    T('97723986209.719141664691833906', '401938914470.70126544941', '-304215000000', 6, 3);
    T('5894035316273661.1', '45.56', '5894035400000000', 8, 2);
    T('8486192219093.73183062341154', '79.8155160240267609', '8486192219014', 13, 4);
    T('25538383521.14730199', '611660363.5951108580', '24926723157.5', 12, 3);
    T('7593058571.527287859871299558', '581200656869913379.852377', '-581200649276854808', 18, 6);
    T('583354977421359776.05259317222451789', '6957613115990166403.0', '-6374258138568806626.94740682777548211', 40, 4);
    T('722537950431841.74634', '384581282349495.14260404126282', '337956600000000', 7, 1);
    T('874207.80400246169213808082', '9309747873009880011.7450', '-9309747873009005803.94099', 24, 1);
    T('167123725366.56857441927238822', '804474190463.43750607691072', '-637350465096.86893165763833178', 30, 3);
    T('3566956426391533171.32226583874', '78810284653349209.875', '3488146141738183961.44726583874', 31, 3);
    T('705744133037002730.08292', '20786.1022899326', '705744133036981943.9806300674', 39, 3);
    T('6228601.0335906834966', '49075624781870787.46763', '-49075624780000000', 10, 4);
    T('16883358981753879.85594856788', '98.9574', '16883358981753780.89854856788', 30, 6);
    T('6286865.90695030367', '8763.50588332454715050461', '6278102.40106697912284949539', 32, 0);
    T('6776149.5591', '6226.985124807960419', '6769922.573975192039581', 30, 5);
    T('4377534.0', '454.26103', '4377079.73897', 27, 5);
    T('2346134666957.29500872791874958382', '3638534229.3863367', '2342496132727.908672027919', 25, 6);
    T('501177.860517701889784', '8.12259741924524516', '500000', 1, 3);
    T('110181120257629244.3369826277802625075', '1289331942740348068.9', '-1179150822482718824.563', 22, 4);
    T('60063401.1271936258', '931832255545876580.76944', '-931832255485813170', 17, 1);
    T('47.3696154602', '2157390640378335029.34726393527117', '-2157391000000000000', 7, 6);
    T('8184954701.2966253444929', '1253.8930733363759786140', '8185000000', 5, 4);
    T('277255052470.8869052139763321775', '91177829708057.62286279630', '-90900574655586.7359575823236679', 30, 0);
    T('78099360.451088555047', '67.7146670024817080554', '78099292.73642155256529', 22, 1);
    T('218495414.318375121', '2177493247513.0', '-2177274752098.681625', 19, 6);
    T('7337391.0793270225782658', '9.6257', '7337381.4536270225782658', 29, 2);
    T('69.19638025072', '1945853522336000.550106341762641', '-1946000000000000', 4, 5);
    T('83574197.0', '267058073881411531.00649428403672', '-267058073797837334', 20, 1);
    T('514570.398208928', '0.0807', '514570.317508928', 39, 3);
    T('1411555.5621', '93559520711075.1671793533795933729', '-93559519299520', 14, 4);
    T('341165411589.230466', '39971412719595208.753923560276409', '-39971071554183619.523457560276409', 33, 3);
    T('94777018865349.36166531285068', '57708690800235718073.57', '-57708596023220000000', 13, 4);
    T('501.50620631476638324758', '19844965768.068253083369', '-19844965266.56204676860261675242', 35, 5);
    T('508453830061965963.5', '569574462303894.2894413300763072', '507884255599662069.2105586699236928', 39, 6);
    T('72992.81336163939', '2686.786855636', '70306.02650600339', 17, 5);
    T('57136417169.45910946235092957', '30658352.37429', '57105758817.08481946235092957', 38, 4);
    T('5721239030615590.530454237', '765504211181701.74980', '4955740000000000', 6, 0);
    T('24458953693061349.6873729559292056160', '5691.261218250', '24458953693055658', 17, 6);
    T('7600383256468.89', '830.430563200', '7600383255638.4594368', 37, 1);
    T('858639697.68223118', '5.2147183047735625', '858639692.4675128752264375', 34, 1);
    T('26206631523296.457462390', '99941.0', '26206631423355.45746239', 32, 5);
    T('544082608011384985.064737655', '48351866325466.173', '544034256145059518.89', 20, 5);
    T('2043.0', '7.22928139', '2035.77071861', 17, 5);
    T('920489578988940.81614605337', '359990748.048945477376244750', '920489218998192.7672005759937552', 31, 5);
    T('473520029.6338091194909980', '4190120225642411356.4145694337', '-4190120225168891326.780760314209002', 39, 1);
    T('3166917650261036828.719817987064876', '561341.9372314', '3166917650260475486.78', 21, 1);
    T('6610008.840531594557', '231787969050861.31788872195700638', '-231788000000000', 7, 3);
    T('622060.939849686239890', '19826279373549094270.31239', '-19826279373548472209.38', 22, 3);
    T('7671411.669804345440', '66177706689020.26887', '-70000000000000', 1, 0);
    T('54744427384910.565629229243236127', '9371.2232117730519', '50000000000000', 1, 1);
    T('99341152780200670041.3421280746991017099', '815065388.4623', '99341152779400000000', 12, 5);
    T('3016.72732378', '85405017.6996771', '-85402000.97235332', 17, 2);
    T('914884366.12451729351377977224', '5758470152579563579.2584141145', '-5758470151664679213.1338968209862202', 35, 6);
    T('84135409093955382738.00559469160808620', '637219844565112854.099088740995063', '80000000000000000000', 1, 3);
    T('37746.9', '1700.099', '36046.8', 6, 1);
    T('67012.73028', '767.87203283248078129', '66244.858247', 11, 5);
    T('5800408740864755.81679331', '52.07067839193388920959', '5800408740864703.7461149180662', 29, 0);
    T('200525476350810.0', '8.9522', '200525476350801.0478', 40, 6);
    T('54060563706539958.842735156723242436', '306564554290247.861', '53753999152249710.981735156723242436', 38, 6);
    T('339.44', '93642759.513218538521601', '-93642420.0732185', 15, 4);
    T('1721710.1', '459.74418', '1721250.35582', 34, 4);
    T('4192260513.69872', '4163966107671121.634738377899399', '-4163961900000000', 8, 4);
    T('675606371683857.006297490165631', '71938395431336.1312477181749896404', '603667976252520.875049', 21, 1);
    T('5214290993428.124881409246276920', '47040055349061103.66597385', '-47034841058067675.5411', 21, 6);
    T('889905637840386.0', '102442852149080555.97', '-101560000000000000', 5, 3);
    T('8.9504789544336', '99174.02380', '-99165.0733210455664', 34, 2);
    T('5998.397', '2914747195956694.009067', '-2900000000000000', 2, 2);
    T('2371005760625376.73576437', '95572478762640740844.0', '-95570107756880115467.26', 22, 4);
    T('798691965.408252468', '21136382841665616.5953', '-21136382042974000', 14, 3);
    T('3043561705.005690212295', '27.093', '3043561678', 10, 6);
    T('431015321.914490450684507313', '4045141586871747485.88929500466390565893', '-4045141586400000000', 11, 1);
    T('694927399.49829265065179786', '6284691050.2989929932', '-5589763000', 7, 1);
    T('1093579032.6523555369943933', '8903045961.51395099996482754459', '-7900000000', 2, 3);
    T('84.85312200126000026777', '139282285.4798282944831', '-139282200.62670629322309973223', 36, 1);
    T('2128.532742753', '56182423.1301771045706596432', '-56180294.59743435157', 19, 4);
    T('310.6655862218594919', '3612438.577', '-3612127.9114137781405081', 23, 4);
    T('3984247509.315601547111431506', '978970092564121897.21811', '-978970088579874387.902509', 24, 3);
    T('338928102720.0485047', '990180050720253121.06165259261385', '-990179711792150401.013147893', 27, 4);
    T('400.380', '85171017956304061.642903144867218', '-85171017956303661.26', 19, 2);
    T('275235.60892029082', '3013275283.52649631844426', '-3013000047.91757602762', 21, 2);
    T('8066401.5740880143', '32888927762349463240.14069647872', '-32888930000000000000', 7, 3);
    T('437376172924731430.8742035855', '2584502074.55035190823745084', '437376170340229356.32385167726254916', 38, 5);
    T('1.0', '8328135.0', '-8328134', 23, 5);
    T('496.008680547205140', '1740470590268.103', '-1740470589772.09431945279486', 32, 2);
    T('1404167871429.6700282456571890224', '5502006357.4936047359975401258', '1398665865072.17', 15, 3);
    T('27921507652.0', '9319986943797740.7329463', '-9319959020000000', 9, 2);
    T('3892287934795244559.45234444669', '1.18539792073', '3892287934795244558.26694652596', 37, 5);
    T('8344896144197.47822193761247819978', '4703961615.298', '8340192182582.18022193761247819978', 37, 3);
    T('9961673328186338.2062369500604541', '85309494980267515.06881406003283363614', '-75347821652081176.86257711', 26, 5);
    T('169182538800183579.01895697779054613535', '720014604415606602.6366602', '-550832065620000000', 11, 6);
    T('948604950140.39487', '4479.899', '948604945660.49587', 37, 3);
    T('11489743581803.73611043550408135', '2814489.1806', '11489740767000', 11, 3);
    T('26636.363', '738815088.0', '-738788451.637', 26, 6);
    T('9157094309.10182554707895240', '23948.777109629652711', '9157070360.32471591742624', 24, 5);
    T('426488854545984791.990852027674250532', '689898.4864403480521', '426000000000000000', 3, 6);
    T('652879.3125509135954440888', '34321614.63049029884423381072', '-33668735.3179393852487897219', 27, 2);
    T('700.68266848104', '1159588.3914863224425606107', '-1158887.70881785', 15, 3);
    T('778351.413', '30633854.8752345490687233142', '-29855503.4622345491', 18, 5);
    T('79923297.8122', '8417873362.95235392459769534929', '-8337950065.14015392459769534929', 40, 1);
    T('39901.80963', '9492296999227629118.73814273', '-9492300000000000000', 6, 4);
    T('44266.67218484777860881768', '49194469328711353.1054128733404044', '-49194469328667086.433228026', 26, 0);
    T('7585762098.06968329334', '33.105', '7585762064.96468329334', 30, 1);
    T('750.86346303200', '64434509886317666211.2723190075477', '-64434509886317665460.4088559755477', 35, 2);
    T('23.1967393603936', '8935010207507.96336', '-8935010207484.7666206396064', 32, 5);
    T('9513813478033.0', '47419318899.36', '9466394159133.64', 26, 6);
    T('7042748196.54597598400250483', '198114476693.0698315491', '-191070000000', 5, 2);
    T('9263.670701217524', '0.1418128369', '9263.528888380624', 25, 1);
    T('708667316908188518.17', '23855208.78', '708667316890000000', 11, 0);
    T('3767630.273140', '514201156.504801', '-510433526.231661', 27, 3);
    T('609699382787192.77', '121565551291767.1350410624707', '488133831495425.6349589375293', 32, 6);
    T('12017.332', '8.728451', '12008.604', 8, 6);
    T('362845.3733070270', '664028583664241.14', '-660000000000000', 2, 2);
    T('8693307732716887.0780', '756820005894.78919711224520', '8692550912710992.2888029', 23, 2);
    T('79023549913006451293.272050948', '779.258', '79023549913006450514.014050948', 35, 5);
    T('1234545.9570327227030120097', '6614991820001400.8572550', '-6614991818766854.9002222772969879903', 37, 2);
    T('7436.700132981979', '39242055864819823.885817522254191', '-39242055864812400', 15, 6);
    T('133233112792.6413672', '8022419.0', '140000000000', 2, 2);
    T('717102156816641991.62', '284743016607466389.62316', '432359140200000000', 10, 6);
    T('66037411079644738477.686144876540957518', '2.133582', '66037411079645000000', 14, 4);
    T('5004831059244050.476871994712', '9414359700.76201048716748038', '5004821644884349.714862', 22, 6);
    T('55448938988.3693899640949065748', '240.28606974162695052338', '55448938748.0834', 15, 0);
    T('9999142520834909.1724', '58566251307402.3556073567546240739', '9940576269527506.816793', 22, 2);
    T('25003891514352867.7051325678807738', '50573465200.10', '25003840940887667.6051325678807738', 33, 3);
    T('21095761921958500491.5473200057240770489', '5.44', '21095761921958500486.1073200057', 30, 6);
    T('60409785.8796835510165263', '7149458968.0', '-7089049182.12031645', 18, 5);
    T('588953849362.0', '2109.58750575632099', '588953847252.41', 14, 6);
    T('5293373833.06408409452', '4.91394152979', '5293373828.15014256473', 29, 0);
    T('95904021564.72528116471995860138', '84724233417.2393762243602', '10000000000', 1, 6);
    T('180009064316.982446525209238', '310440539744179770.6479', '-310440000000000000', 6, 1);
    T('14017604722149125.992568', '80112.8838339350', '14017604722069013.109', 20, 2);
    T('61196.75', '528578893.46', '-528517696.71', 17, 6);
    T('7866517255301.8', '2821562221.9', '7863695600000', 8, 1);
    T('369827389648334244.79650608219695785304', '69786316761.749208866306', '369827319862017483.0473', 22, 4);
    T('79397643629345054143.596776576703352749', '6528367068245707866.08727655083021', '72869276561099346300', 18, 6);
    T('60270313.11224142277', '983379.578159', '59286933.53408242277', 30, 0);
    T('30142.0', '91866953906.3650944792380', '-91866923764.365095', 17, 3);
    T('350704439960575.9388805771543694557', '9799911027602713.4107', '-9449206587642137.4718194228456305443', 40, 2);
    T('7326.50453', '302.92556024538', '7023.57896975462', 20, 0);
    T('359223.59', '4706771049456544535.9824506052227', '-4706771049456185312.39245061', 27, 5);
    T('128752081083.23723423431289', '17220674228623400519.175586947475136924', '-17220674099871319435.9383527131', 30, 1);
    T('47079724.03018679425', '528585805301663.0212', '-528586000000000', 6, 5);
    T('93389873251983732017.2377764775789', '9905665155.34279', '93389873000000000000', 8, 4);
    T('1040755866.19132', '607.279522142720061', '1040755258.91179785727', 21, 1);
    T('27164547109.35639151032', '596039372152452409.710029524742158410', '-596039344988000000', 13, 3);
    T('175.7577', '1.2235209397', '174.5341790603', 22, 5);
    T('58149837053.6221021943464499454', '29.0', '58149837024.6221021943464499', 27, 4);
    T('850685108514098996.6959217842', '5839333719574.004057', '850679269180379422.6918647842', 37, 6);
    T('27595.64275', '775597357698775974.958971413519', '-775600000000000000', 4, 0);
    T('69534982148593.4613031491618', '85258.26912622', '69534982000000', 8, 4);
    T('9793668936863883912.0', '157.8624166744033921575', '9793668936863883754.137583326', 28, 4);
    T('4.51886312148834065', '9573745540.733595', '-9573745536.21473187851165935', 36, 4);
    T('277044631751219.80305148580193491', '645.998512356047', '277044631750573.804539129754935', 30, 0);
    T('170.78673560620772256', '2087231998359490505.6824313', '-2087231998359490334.89569569379227744', 40, 2);
    T('1407044748948353.7658180825652', '364387695885738540.376549366759', '-362980651136790186.6107', 22, 1);
    T('189.3323091', '82590092249501211986.39986946169935', '-82590000000000000000', 5, 6);
    T('1733430221813204934.674134694746', '1442291526867247314.43611', '291138694945957620.2380247', 26, 0);
    T('20815.500959354', '429149171631512704.39257938149758675', '-429149171631491888.89162002749759', 32, 4);
    T('1519762970.23770449383835389025', '736755394100.4561553597320715', '-735235631130', 11, 5);
    T('9.0828100', '218.7061', '-209.62329', 26, 0);
    T('69118203801679383.6457037530189', '335.08563', '69118203801679048.56007375302', 28, 4);
    T('819.94054020638271', '37241886618910458361.0', '-30000000000000000000', 1, 2);
    T('56405459243681586992.6180051', '5847886319505023309.59', '50557572924176563683.0280051', 40, 3);
    T('942.67', '5726424521768.544399', '-5726424520825.874399', 28, 0);
    T('406699982837.12', '457286677678.83426', '-50586690000', 7, 6);
    T('4955373376.263194404432', '51711520571468422610.1599', '-51800000000000000000', 3, 3);
    T('257003600.423439713634', '13248167222.818161', '-12991163600', 9, 4);
    T('34656857284908356.823', '9521448116771614.75040574602932641', '25135409168136742.07', 19, 3);
    T('370196042300034.015044591340044296', '8806985.623465295', '370196033493048.391579296340044', 30, 3);
    T('2300758415.4851195737190454537', '48937168850028.3', '-48934868091612.815', 17, 0);
    T('4375.529187285', '756841058857881.3968', '-756841058853500', 13, 6);
    T('75187.4042028337', '51.29081401', '75136.1133888237', 30, 5);
    T('53765335837.12427471', '291556789652718.3669360172441738200', '-291503024316881.242661307244174', 30, 6);
    T('5971489495776743.914', '1923228.35474406483075300699', '5971489493853515.55925593517', 28, 0);
    T('70782089.48778088616198', '14058295098077576029.46355250647004', '-14058295098006793939.9757716', 27, 4);
    T('46295988758935935632.19668243784232', '204677335732346350.1126714891065789041', '46100000000000000000', 3, 0);
    T('9540475.356697', '1776608373657.56172678997990', '-1776598833182.2050297899799', 36, 0);
    T('7.5338079', '44617449655841.2048', '-44617449655833.6709921', 32, 3);
    T('824232606960840739.696', '9817831102244320680.263294620', '-8993598495283479950', 18, 3);
    T('70174292141892246.0278945143536377525', '838557221255.0', '70200000000000000', 3, 2);
    T('5960689960.0', '8617708695.02290033790285536462', '-2660000000', 3, 3);
    T('54209282.7', '36.0', '54209246.7', 22, 0);
    T('6563642583183636.0', '94.17499111522648502344', '6563642583183541.825008885', 25, 6);
    T('16956817595232129.282447690639', '45422830967256451434.350', '-45405874149661219305.067552309361', 33, 5);
    T('5741.796542935025501', '2324.45230022', '3417.344242715025501', 28, 5);
    T('469322.341735087893985', '963308.40637707', '-493986.0647', 10, 3);
    T('4178747.9617', '959.398939', '4177788.562761', 27, 0);
    T('474.2843063', '5725113623009505.698527', '-5725114000000000', 7, 5);
    T('6288677202.558', '190668290509560548.57175014964684', '-190668280000000000', 8, 5);
    T('6693.480356384897859', '88.98', '6604.500356384897859', 27, 6);
    T('1185051093938137634.37683', '39954643642292.69784646790834314', '1185011139294495341.6789835', 26, 5);
    T('330672322162647802.1490586', '53.96318924', '330672322162647748.19', 20, 6);
    T('132329680667168.133499108152509681', '8615.35900', '132329680658552.77449910815251', 29, 5);
    T('7979921596318831971.39705195707494580', '64535340689182.463219336501613', '7979857060978142788.9339', 23, 2);
    T('4.38651466', '940635456601.381', '-940635456596.99448534', 38, 0);
    T('27488310186078.351302447', '4369920510.0', '30000000000000', 1, 2);
    T('48.2711481404598', '10235579014551.00182542', '-10235579014502.74', 16, 3);
    T('1735426293.88680', '517197345.721773786889', '1218228948.165026213111', 24, 2);
    T('7959683.39', '14272850984572472.0558', '-14272850976612788.6658', 39, 6);
    T('6289105549924.66199586', '8657552063725138.499317671', '-8651262958175213.837321811', 38, 4);
    T('13574.25232934513376', '10632476222768.68483523', '-10632476210000', 11, 3);
    T('535651077836531.322', '8102.31148', '535651077828429.01052', 30, 5);
    T('8869978018734.00392274480788913815', '124334716500184456.3685', '-124325847000000000', 9, 4);
    T('24708690.21283544944', '42896.821', '24665793.3918354', 15, 5);
    T('718081247294795256.87172433331647033', '58150946855246528941.1192168648', '-57432865607951733684.2474925314835297', 36, 4);
    T('25548767.3', '151996681419.5078708665572', '-151971132652.2078708666', 22, 5);
    T('80.26002684557254362615', '899911522537.048897', '-899911522456.78887015442745637385', 32, 0);
    T('84807157802589.0', '8108.24930829822600491512', '84807157794480.750691701774', 28, 4);
    T('3.064001583880792302', '995791068585.67410952709', '-995791068582.6101079433', 22, 0);
    T('4025849414871008.0068', '58947441179183.2475129868601959775', '3966901973691824.759287013139804', 31, 1);
    T('9190263385079650195.41604553585808968', '2303968687.95317778', '9190300000000000000', 5, 5);
    T('25474396745095890841.546995', '1354047450417784888.4749729424603843', '24120349294678105953.0720220575396157', 38, 0);
    T('8505831780937721.14211058587', '43130131382880292143.49', '-43121625560000000000', 10, 3);
    T('3176024.81525798179611552', '942006.0', '2234018.81525798179611552', 25, 6);
    T('214987747913578.9164395051494825', '8318434891947218503.4007057610865642192', '-8318219910000000000', 9, 0);
    T('52788.68901030', '328165697971.6574462529048268350', '-328165645182.968435952904826835', 36, 5);
    T('3.22785941', '22161.0', '-22157.77214059', 20, 5);
    T('88329543.4481397650380935', '8062.41620957', '88321481.0319301950380935', 29, 5);
    T('71911146232808.0257459', '3929592109598.643910090852195', '67981554124000', 11, 2);
    T('3773492.1363', '255243564207.39050734', '-255239790715.25420734', 36, 0);
    T('58506820.99280282009664', '64331566869.55049635780', '-64000000000', 2, 2);
    T('7.11291965507165', '48716523278563566286.487160655414', '-48716523278563566279.37424100034235', 35, 0);
    T('2411.2517014729', '93170465092466.612207193827658482', '-93170465090055.4', 15, 3);
    T('3857101755.51196193184097780', '110693779161922922.20833589724509051151', '-110693775304821166.696373965404113', 33, 6);
    T('30577837664548077783.61143531', '4499194264.8549301546', '30580000000000000000', 4, 4);
    T('280336448041157.76212813127452162445', '365941.652353640583258', '280336447675216.10977449069126362445', 35, 2);
    T('96828553.477044457', '6763157108930938.9960614', '-6763157012102385.519016943', 33, 0);
    T('508967922600557425.0749974117751658541', '433578.16844188709240', '508967922600123846.90655552469', 29, 2);
    T('4065.6185917071785377', '12625350.6634579043', '-12621285.0448661971214623', 33, 3);
    T('6614.0', '1430.8590448778904', '5183.1409551221096', 34, 3);
    T('99.308282866456993', '27251979.5993767658502', '-27252000', 5, 6);
    T('16909742934245592.0181971', '500326237905422921.84', '-483416000000000000', 6, 2);
    T('33182511860161468233.8591', '56829936.13317218', '33182511860104638297.725928', 26, 4);
    T('435444001377.2159254105624106438', '3512479317363377034.97762', '-3512478881919375658', 19, 6);
    T('37.74422895941', '78593841282758883.330920771', '-78593841282758845.5866919', 24, 3);
    T('7518619884535302.2667550304518', '80370508903309.241', '7438249375631993.0257551', 23, 2);
    T('72.75103431586', '136151296528104.8659473464', '-136000000000000', 3, 1);
    T('5765145596194.56770', '4531648803806848960.44979', '-4540000000000000000', 3, 3);
    T('63061591741489.0', '6.223393680089571829', '63061591741482.776606319910428171', 34, 5);
    T('71.279981', '713079003422150328.826902673847', '-713079003422150257.546921673847', 36, 6);
    T('125.7375', '64473239005896501810.03698921727819703314', '-64473239005896502000', 17, 3);
    T('5591886.878238', '69622780588.3902268522525251', '-69617188701.5119888522525251', 36, 3);
    T('163.39457988342951', '27.8', '130', 2, 1);
    T('2866145.0559582', '82428.36176259949775349458', '2783716.69', 9, 5);
    T('38804651896286683.029726', '88872694422018.833584229714', '38715779201864664.196141770286', 39, 1);
    T('73592200267080627.216561', '6975.911004862110369294', '73592200267073652', 17, 2);
    T('688236.1643460', '7656215.397002008863', '-6967979.3', 8, 0);
    T('821193921880969082.666405112652151', '6616126674739264.91842316602', '814577795206229817.747981946', 27, 1);
    T('97788769.466643198156962', '9420602.5917391336857751', '88368166.875', 11, 6);
    T('34421616686767890952.95460919478', '5167579427528.0', '34421611519188463424.95', 22, 6);
    T('4032015830002558.77551827796838836', '7.5014', '4032015830002551', 16, 4);
    T('49742.082424037826835', '9179.0868571458', '40562.9955669', 13, 2);
    T('13.83', '581622011.374036078254', '-581621997.544', 12, 2);
    T('9787415269746.8914348462', '8170018.300761', '9787407099728.5906738462', 35, 0);
    T('313273.3503900096488375440', '2621010.060397525927', '-2000000', 1, 2);
    T('66336.0706320', '142657940946620.1515820283982687540', '-142657941000000', 9, 6);
    T('9003670926937.7248558757763', '93475336164402841.8532660414982264914', '-93466332493480000', 13, 0);
    T('9688666980092283198.1570462560839982820', '775401550305.731361033', '9688666204690732892.4', 20, 6);
    T('1351461498851.6354', '6969005085536688637.0', '-6969003734075189785.36', 21, 1);
    T('61612797805037624.83043237', '47672089415.020872788941974', '61612750000000000', 7, 5);
    T('9518933.0512364031315872469', '97374186816388761441.4803', '-97374186816370000000', 13, 1);
    T('710600719418.0', '987431400217940959.719616251373', '-987430689617221540', 17, 6);
    T('167616.536410880157593', '209.6568840277138', '170000', 2, 4);
    T('53108627166289220403.3507947094037', '183843995335.14', '50000000000000000000', 1, 6);
    T('1.82', '1068134631631.9872', '-1068140000000', 6, 3);
    T('27356267208954.56889', '986253559931114125.349', '-986226203663905170.78011', 40, 4);
    T('3846.8', '77940043708478903509.277711611685522761', '-70000000000000000000', 1, 2);
    T('399964871938206247.26279263632908351', '19.780191747', '399964871938206227.48260088932908', 32, 1);
    T('75233346957819200.595099671231', '39911859356999410.23871883', '35321500000000000', 6, 5);
    T('5.9553', '26702530902176.0', '-26702530902170.0447', 37, 3);
    T('5714264.534703393', '1623567584.28', '-1617853319.745296607', 20, 6);
    T('7192224694126800.38', '14164910712430.43844549533239776239', '7180000000000000', 3, 0);
    T('34807651127623742.365688619586', '679711365974058390.18', '-644900000000000000', 4, 1);
    T('9470184630991.7560398280579092017', '8760544279111.4672', '709640351880.28883982806', 23, 6);
    T('6368228726932234099.0', '52240370965372.283099621290278812', '6368176486561270000', 15, 0);
    T('52.2641298748321641901', '59882.8358937', '-59830.5717638251678358099', 29, 3);
    T('362061913579.6748416041715719', '987.0', '362061912592.67484161', 20, 0);
    T('26984585748715322362.205', '1171423299571863400.107497137768', '25813000000000000000', 5, 5);
    T('15059.08', '13027948565.078036177189', '-13027933505.998036177189', 29, 3);
    T('43466195740.364475434773450', '66.8', '43466200000', 6, 6);
    T('4648785717358997.471379600', '82665047520153326594.4', '-82660398734435967596.928', 23, 2);
    T('20171.82946599822', '55737724981325100.12', '-60000000000000000', 1, 0);
    T('358745279679513.8043987335952', '99695748554529.812596943797016436', '259050000000000', 6, 6);
    T('947344203.2743403513037524', '6.0373538050526700', '947344197.23698', 14, 1);
    T('8246253299.29021109085215', '99.447237', '8246253199.84297409085215', 37, 2);
    T('7.44495950366428410', '30664988839253312.0', '-30664988839253304.5551', 21, 3);
    T('20409.253', '617277084.91660251', '-617256675.66360251', 19, 1);
    T('9757536.58665856593', '874669282576335469.677350946252', '-874669282566577933.090692380322', 35, 4);
    T('81498849658802797996.6208157812436354', '12934827514279899432.505573867517187', '68564022145000000000', 11, 5);
    T('319.361048499679', '2100998.9168', '-2100679.555751500321', 35, 2);
    T('44.6', '7694320284.15318', '-7694320239.5532', 14, 6);
    T('390860494402.02235327', '7.28851', '390860494394.73384327', 36, 6);
    T('938907.243584822', '814413900.51102345860', '-813500000', 4, 6);
    T('798811025297475.4688186', '1.44732870069731', '798811025297474.02148989930269', 38, 1);
    T('748242.0525151117', '86171487.81602654098946770178', '-85423245.763511429289467701', 26, 2);
    T('63720.01667910759721', '2022623241.7452933835771', '-2022559521.72861427597989', 27, 1);
    T('33700962661512376081.4727105634187050794', '7821943626921084634.21', '25879019034591291447.26271056341870507', 37, 3);
    T('807588551473322274.389664', '515.837', '807588551473321758.552664', 32, 5);
    T('2.857', '127279639689457.46973054411818060', '-127279639689454.6127305441181806', 39, 5);
    T('80403687006823365.8617672', '3.87463', '80403687006823361.9871372', 32, 5);
    T('63499966541620610159.956897232396691033', '233444796261111924.24748', '63266521000000000000', 8, 1);
    T('51.26', '9755.89', '-9704.63', 39, 4);
    T('831.32235', '825326439.03', '-825325610', 8, 4);
    T('9160141204.53051426931602', '669381.607552815667', '9159471822.92296145364902', 24, 1);
    T('890199681655323.9', '9588005813808117572.45366958563902825756', '-9587200000000000000', 5, 3);
    T('48177774291441982153.23', '536299643229265.3951216486883', '48177237990000000000', 10, 6);
    T('919961.38', '0.9101336950758809', '919960.4698663049241191', 28, 2);
    T('572305986648.5386', '228069.19902907767', '572305758579.33957092233', 24, 4);
    T('819188883136823693.84758438044', '49934454361438505.20566951225206065', '769300000000000000', 4, 2);
    T('35.7399129558657', '4882255190327145950.4151824903924168473', '-4882255190327145910', 18, 1);
    T('5440.1735', '611256460927438.341770820995120', '-611256460921998.16827082099512', 39, 2);
    T('60442562459660.4378647', '79474948.72807652051595', '60442000000000', 5, 1);
    T('331050833369.3148885303', '98169313.365812', '330952664060', 11, 2);
    T('46.059683436004506', '519317903.070071', '-519317857.010387563995494', 39, 1);
    T('8447111425391860884.1064682142', '863132777609180019.58669736971856', '7583979000000000000', 7, 6);
    T('887774188325.41697007570', '83775609342.4357564368', '800000000000', 2, 1);
    T('90341678.31419886', '47867828141036408422.287884711250091105', '-47900000000000000000', 3, 6);
    T('56715082608754180500.82899364066', '14638787165684.5345343', '56715067969967014816.29445934066', 37, 2);
    T('938.9898699375727', '242457688.65465036460', '-242456749.6647804270273', 36, 4);
    T('429392780060.06774708', '7035348.8', '429385744711.26774708', 35, 0);
    T('718100848000034391.192290240682711', '79.228524252', '718100848000034311.963765988682711', 34, 2);
    T('4289.41689055586116', '6425355333091.393343085041262', '-6425355328801.976452529180102', 36, 4);
    T('56108683474.841199580480808104', '82737675.83', '56025945799.01119958048', 22, 3);
    T('1884.1885119044687', '723.14327829603', '1161.1', 5, 0);
    T('7420425403.8143541', '2050663.0', '7418000000', 4, 6);
    T('904681858.69854960578439525', '183244050.0', '721437808.6985496057844', 23, 4);
    T('2267437984926.10766021', '720614321135.05176132012955', '1546823663791.056', 16, 4);
    T('6353304404324354728.40041298409487', '38265293643.58207509', '6353304366059061084.818', 22, 3);
    T('3929540019.03659416764', '89.360121534557319283', '3929539929.67648', 15, 2);
    T('446966870.3690922868', '60337505068028.9226164167', '-60337058101158.5535241299', 34, 6);
    T('31.9338970', '6663.286337072', '-6631.352440072', 36, 0);
    T('272058.0370091337560537', '4648068.285162', '-4376010.2481528662439463', 28, 2);
    T('7917.527', '2523139570440467.9389767023945697', '-2523139570432500', 14, 2);
    T('2207488606376.17978020407', '31531732069862.426768463105', '-29324243463486.24698826', 23, 0);
    T('8154711.96938704295466266', '469994.3145418724995661670', '7684717.654845170455096493', 40, 1);
    T('109766684873371682.78052938956', '404.190', '109766684873371278.59', 21, 1);
    T('7084730.092913368020', '290283674635.8', '-290276589905.70708663198', 38, 0);
    T('147079.1314386204', '6006785.339824373630', '-5859700', 5, 5);
    T('565768775012315573.2161210699', '81335.2238983314252', '570000000000000000', 2, 4);
    T('5669983755044270231.0', '7768235.96097494592586860', '5669983755036502000', 16, 2);
    T('212925640367814.0', '994306851359.706627008116', '211931333516450', 14, 5);
    T('718595044120976893.7877886916388', '9290229.547', '718595044111700000', 13, 2);
    T('46392136939.6412800187914', '613765838132804.47707', '-613719445995864.8357899812086', 30, 6);
    T('16289954618893788.475153510884623069', '35774.0589690', '16289954618858014.4161846', 24, 2);
    T('7976219587479981.694774859256269', '1.60513556611728994012', '7976219587479980.08963929313897905988', 39, 0);
    T('30186736778.872874727097875856', '7370819420629834.61981642878206', '-7370789233893100', 14, 5);
    T('4106339851.369', '88063377406.9164697', '-83957037555', 11, 1);
    T('20393217556493144418.7', '391503223768712.688', '20392826053269375710', 19, 6);
    T('6053663.37013', '1178991747436889216.960', '-1178991747430835553.6', 20, 0);
    T('39029690280420309.7814925', '3611393.2', '39029690276808916.5814', 21, 3);
    T('62309314328.3648108197', '66.65079', '62309314261.7140208197', 33, 4);
    T('341799494313555791.6656', '97951.03756454', '341799494313457840.62803546', 33, 5);
    T('3921792340162092.3574373', '677604.9650', '3921792339484487.3924373', 34, 0);
    T('84.152682681', '940696386325686.45500', '-940696386325602.302317319', 25, 2);
    T('755577600765759412.6282341892536415035', '95878089398.58684970337571134778', '755577504887670014.04138448587793015572', 38, 5);
    T('873479307720.6680392260597239', '734905889102221361.581205643', '-734905015622913640.91316641', 26, 2);
    T('225641112112.661', '719497439505.79666564092199', '-490000000000', 2, 2);
    T('550119.2450844', '2844.8', '547280', 5, 2);
    T('69588454174054132.25248017214262597', '64.9124', '69588454174054067.340080172', 26, 6);
    T('45574606883.5583', '88801888155303.0', '-88756313548419.4417', 38, 0);
    T('3342701555960293799.9234598466001041904', '9832636327.6165004779869', '3342701546127657472.3069594', 26, 6);
    T('8046442.2300229228499', '4940.6335599340051193172', '8041501.59646298884478', 21, 5);
    T('73968996015.279509278625883064', '65395771585.322733639260024', '8600000000', 2, 2);
    T('8.110074519640668', '38.80061987142', '-30.690545351', 11, 1);
    T('41964527881657.510305173285792996', '8643.142610', '41964527873014.3676952', 21, 0);
    T('9834.6766017643659698586', '7729700086453.77441609', '-7729700076619.0978143256340301414', 35, 1);
    T('49888830837.51073324060251', '231.4360929580438', '49888830606.0746402826', 21, 4);
    T('47974230039689.65292486746063670', '460005214507062.37776672156955323', '-412030984467372.72484185', 23, 6);
    T('8544909625515.78805', '64582401702.4483266688651', '8480327223813.3397233311349', 35, 3);
    T('836225247131.755', '62021028.80144509055359', '836163226102.95355490944641', 28, 6);
    T('21402218785626173766.0', '5974407874584131553.8930817795541', '15400000000000000000', 3, 5);
    T('671411169724683907.6380684917711731425', '79615809502119937.945530982', '591795360222563969.69', 20, 6);
    T('5877977908.4320573', '214933944982080653.6076723505821', '-214933939104102745.1756150505821', 39, 4);
    T('8764.5604587165699170694', '59405420970791.0', '-59405420962026.43', 16, 1);
    T('390.013310998765', '2683670006183.5', '-2683670005793.486689001235', 34, 1);
    T('8221956.9763684', '89801.0', '8132000', 4, 6);
    T('974705194.276607029945579', '14999851059715.4625693044090855238', '-14998876354521.1859622744635065238', 36, 4);
    T('19384935.60975312621076411', '617727227.018313863425914514', '-598342291.40856073721515041', 26, 3);
    T('2918025308413194487.0', '787037828275042.74598298714452563672', '2917238270584919444.254017012856', 31, 2);
    T('32757673078.9338667', '2.3888573905357353', '32757673076.5450093094643', 24, 2);
    T('959330921609991.220939853295022', '3376.976575743', '959330921606614.2443641', 22, 6);
    T('950871057181029868.0', '271.44931', '950871057181029596.55069', 34, 1);
    T('84321.21233326421', '6714.05657418', '77607.1558', 9, 4);
    T('21347641094334.050743667729343', '4742375.47091426258042624', '21347636351959', 14, 2);
    T('91.5149155987157', '32.86941505001731729', '59', 2, 2);
    T('6032618814.768823948083280', '120922257678828797.25376483500338193', '-120922251600000000', 10, 1);
    T('52.444453682', '8882800808686642389.85798578923694', '-8882800808686642338', 19, 3);
    T('79.1775542594767560673', '6266906822541.07299020452444721', '-6300000000000', 2, 3);
    T('9213.9353121', '41118274379757178435.5375911489', '-41118274379757000000', 14, 4);
    T('87768812246.42606954838244', '9700979639.114402', '78067832607.31166754838244', 39, 1);
    T('506805337748542453.55745317581864619', '64264.8749377245986138084', '506805337748478180', 17, 1);
    T('191.9', '8584.761', '-8392.86', 6, 6);
    T('506032189986640.635473486246', '80929441126968.611860957', '425102748859000', 12, 3);
    T('40262525003550.0', '1459364432.846241', '40261065639117.153759', 29, 3);
    T('111895124302.14142764392633063', '568.2784949712304', '111895123733.8629326727', 22, 6);
    T('8614750961732816.6526021502', '558216327.145378758235336', '8614750403520000', 12, 2);
    T('48622.0', '343217086.02196', '-343168464.02196', 26, 6);
    T('4705923253984.3113219991025897648', '51407889138849.5795874137', '-46700000000000', 4, 1);
    T('4757812133.03', '2472560646307524089.00264', '-2472560641549711955.97264', 39, 3);
    T('89.029003050', '2.9385784', '90', 1, 5);
    T('852171824971223.750796717175', '46937.98047853866948', '852171824924285.8', 16, 2);
    T('16284143767521.84888006', '4294123158960607901.861402', '-4294106874816840400', 17, 5);
    T('1771919142.0075759461619676362', '7112217561604506.1683140', '-7112215789685364.1607380538', 26, 1);
    T('734362830359.11677903', '2368520775270917.24541484283556249120', '-2367786412440558.1286358128355625', 32, 3);
    T('64914552833854.720833931990', '7722438669924.02781563768', '57192114163930.693', 17, 3);
    T('4344701736.1792229180360665', '806051581284055320.3658137427656153', '-806051576939353584.1865908247295488', 39, 2);
    T('26977094224469737.87831438', '96570531539929.590', '26880523692929808.288314', 23, 1);
    T('1631.400208001887602327', '7136257.19069521343402', '-7134625.79048721154', 18, 1);
    T('65932.1', '86133420347.5441905573667683031', '-86133354415.4441905573667683', 27, 1);
    T('258003605.8900920316409', '70871673267889117412.91040916490', '-70871673267000000000', 11, 2);
    T('80195474569176774435.043622', '2915.6391', '80195474569176771519.4045', 24, 6);
    T('34.7911', '7332951320452818590.12932888198', '-7332951320452818555.33822888', 27, 4);
    T('580287.378530800', '97321809654920.430331', '-97321809074633.0518002', 38, 2);
    T('117723424138722977.30271939', '4098451891.6110270', '117723420040271085.69169239', 26, 5);
    T('30349227450983.9429373680', '4.5', '30349227450979.442937368', 39, 6);
    T('7821891597381930704.5509953503', '2504987635287.832', '7821889092394295416.7189953503', 34, 2);
    T('872418695477419826.5', '6.0', '872418695477420000', 14, 6);
    T('6513095910363.61550402', '61867466266.4120887760287', '6451228444000', 11, 1);
    T('45988.0267768', '9884722485107754999.79017382859', '-9884722485107709011.76339702859', 40, 3);
    T('1359889340908783547.01782119416756', '684.479', '1359800000000000000', 5, 3);
    T('63069193433038251565.9789', '154082725998499.209314284', '63069039350312253070', 19, 4);

    T('7069481538885792.8453024987800665847448364515', '8799816975500725526.14173904507873437', '-8792747493961839733.2964365462986677852551635485', 81, 0);
    T('7471778599451304456854816606172387414.972896785761455054976515', '41510925697.01725991595659820189734238057', '7471778599451304456854816564661461717.95563686980485685307917261943', 81, 5);
    T('54287422500505266597760281635897425.31893091514580274449559', '59881836.58772223664596161696342807', '54287422500505266597700000000000000', 21, 3);
    T('857252363405.28041705759056035287704626507136004', '9759494861927569651313972507873246424463.3118978250710876461304611335126219687753', '-9759494861927569651313972507015994061058.0314807674805272932534148684412619287753', 81, 0);
    T('5707309958597.17886368215534600', '41690453358185379509319720741969900.21810985007004676596339392858058', '-41690453358185379509314013000000000', 26, 1);
    T('847340135061411.2802565891173617018192606744754', '6288786190771157848196740916169863146.102508745483002595264893213582092318933', '-6288786190771157848195893576034801734.822252156365640893445632539106692318933', 80, 4);
    T('796181873981752156039805092.4', '72914582965162333.189623227479611', '796181873908837600000000000', 16, 2);
    T('3655184921819045.4409085969760204374602710095048304', '561720354565305547.96293100639323123773875332', '-558065169643486502.5220224094172108002784823104951696', 82, 2);
    T('76913374422317528574037956119155886125.54389972893', '6310998704783897100947503702304317952.4245547195953947821401', '70602375717533631473090452416852000000', 32, 0);
    T('17261.922628198324156664578954046', '312982205250455260815488098538202596870.43389487634705856712', '-312982205250455260815488098538202579608.5', 40, 4);
    T('3893782288862497.9547', '13467645390982208450581135330602744368.200603207219743', '-13467645390982208450577200000000000000', 24, 6);
    T('785667.037675', '295658448419843229132671.02', '-295658448419843228347000', 22, 6);
    T('4069477717571477882292878628886338902215.4130', '78300051390.175627986933200537965010886', '4069477717571477882292878628808038850825.23737201306679', 54, 3);
    T('5805829007065466356315.4228691424055568322143818', '234918418761300292615807213588704.83909129752699363699810178528824340705', '-234918418755494463608741747232389.41622215512143680478372', 57, 6);
    T('518645159044774324484177076005.724150279431741639046593309057', '25737792186.3928117', '518645159044774324458439283820', 29, 2);
    T('127960727248.936079606260785987795694092625929433', '492.96031787', '127960726755.975761736260785987795694092625929433', 98, 2);
    T('89815869889.4', '391859959437648447482899686027415.30534528672415638488786082717603815573', '-391859960000000000000000000000000', 8, 0);
    T('87828538837239976664944271800928569851.693751719987861907', '4377184966109.0', '87828538837230000000000000000000000000', 13, 1);
    T('84708.1823311473188833143004697387', '56875814946763.84511770689345194940064241649', '-56875814862055.66278655957456863510017267779', 54, 1);
    T('284563413246424.610808843', '6123518115152093.49556179420291177', '-5838954701905668.88475295120291177', 50, 4);
    T('6616256626565605734559328076475148.68208545193884517467009', '23.29265', '6616256630000000000000000000000000', 9, 4);
    T('2822017107727619537484849285006.7984', '56511512303539.558337492556392422985099', '2822017107727619480973336981467.2400625', 39, 1);
    T('5648374288169203098951938884.2268924363848706284886607016', '63.895411679919164607654200404116668153', '5648374288169203000000000000', 17, 3);
    T('48892727607729871637419217885.8409068570730', '9043895004.19009', '48892727607729871628375322881.650816857073', 89, 0);
    T('8068733507496.1469020975906444737340276803183935587', '41834657331260307768123112973221.6', '-41834657331260307760054379465725.4530979024093555262659723196816064', 66, 6);
    T('7954132788764836.8127062842405802933', '3209760513341348009242231343536.1', '-3209760513340000000000000000000', 12, 6);
    T('989.06219134953678321641387362', '5864.652772904791252100296', '-4875.59058155525446888388212638', 98, 5);
    T('322237585394911775337773839481541599083.6037343339', '2135310090975756500839977.9819995', '322237585394909640027682863725040759105.6217348339', 63, 0);
    T('517246274770059026183.134667470397336430369314960352634946', '5739599.18048884899864', '517246274770053286583.954178621398696430369314960352634946', 71, 6);
    T('318828452216302939011833932591119.16055120291646982988', '19100353732874292660145133125.903457211851691597237', '318809351862570064719173787457993.257093991064778232643', 96, 6);
    T('707496206141321907526732443.7673', '29211577975737716520526037.9349660583955780270', '678284628165584191006206405.8324', 31, 2);
    T('6112412634933658779072944753488612.07', '21810839.011853754548220082443271587614535', '6112412634933658779072944731677773.058146245451779917556728412385465', 95, 6);
    T('5549420422927623871488668959408.07554094299666964265030178694169984', '5014141224.266726059429520362', '5549420422927623871483654818183.80881', 36, 1);
    T('4831374132610019209216252.666919047', '3055.437526359805778104', '4831374132610019209213000', 22, 5);
    T('87283258218.866513101822735', '8206861.5590', '87275051357.307513101822735', 75, 5);
    T('183719464270577278602078845064363184.003139605735', '3113563329057169063.80030643220582947558', '183719464270577275488515516007194120.20283317352917052442', 92, 5);
    T('93395.6342270125800462330892', '1701007552323798339154517067663176969372.81338035155044707', '-1701007552323798339154517067663176875977.179', 43, 6);
    T('20685620806476720644534475538837377.0647916674823433367387023293480355', '3543196775726022864816134.05589736388423801840303', '20685620802933523868808452674021243.0088943035981053183356723293480355', 97, 5);
    T('43953332514457476101856597.1041063270219469666818091211131388', '9144960.29266167421', '43953332514457476092711636.8114446528119469666818091211131388', 99, 3);
    T('1677345568129152447.0', '775620171702678631.0261896843774196837652', '901725396426473815.97381031563', 29, 0);
    T('608.329530951846128416732', '5907676117735942796452.262808324428254527552644452170830', '-5907676117735942795843.93327737258212611082', 43, 1);
    T('163763542.45173606622936', '85501152394.563184250822511', '-85337388852.111448184593151', 74, 0);
    T('980.07387938358825707', '9154043179862724861074.53543805466363011113675078', '-9154043179862724860094.46155867107537304113675078', 79, 0);
    T('6714379986391852202410973.346406550711263', '769019081.899615576280', '6714379986391851433391891.446790974431263', 73, 0);
    T('5660248559651.73757594457649872193580716553505981', '35970947807361.489452689096131', '-30310699247709.75187674451963227806419283446', 43, 5);
    T('934005636127436.601698024551614978329287132', '98469700039091250419511155886136.1056759852', '-98469700039091249485505519758699.503978', 39, 4);
    T('68859694748338724605788433121.99369920874734065949212', '1346191693530.3841114994615077028194684774052760908907', '68859694748338723259596739591.6095877092858329566726515225947239091093', 81, 2);
    T('962648770348538924813184438554492813.016870705575138076288401908496111554624', '45325319781331608108.26346521299', '962648770348538879487864657222884704.753405492585138', 51, 6);
    T('3646747120057094.76155', '25880323298404645184532715740.270306369523497368949395273835553019', '-25880323298400998437412658645.508756369523497368949395273835553019', 87, 2);
    T('84031809137795057941588536.920683497841670600369025458522161766', '20779.405528400664729887522605946604002247', '84031809137795057900000000', 18, 6);
    T('923238139650849512410164955923044760.68380499673656785250510746213955022869', '826036150894645215878.48286693183729', '923238139650848686374014061277828882.20093806489927785250510746', 62, 6);
    T('55571.90220651689', '4173925292298788928169.818773202228108061859989', '-4173925292298788872597.916566685338108061859989', 77, 3);
    T('6610901646439267864.67679538601589458', '82931810315.57529148708467925', '6610901563507457549.10150389893121533', 91, 4);
    T('967.9279315999359879803356408402007785775', '767709984187089428713978961776105.906714829685708405', '-767709984187089428713978961775137.9787832297497204246643591598', 61, 4);
    T('2843291217333719172383160249.67886573', '47532256966982491874401895.96905443930325290277', '2795758960366736680508758353.70981129069674709723', 76, 3);
    T('76469069899341775650032981768677416866.5069645477843308197379040533927', '908102728504641383447566742.06979840838195579', '76469069898433672921528340385229850124.4371661394023750297379040533927', 99, 0);
    T('4252417949972512073842197.820735873386114594411', '1395722814808935499155494087890142754879.44', '-1395722814808931246737544115378068912681.619264', 46, 2);
    T('698890066981067.8507578067662568', '7493355015.657609871', '698882573626052.1931479357662568', 88, 5);
    T('6237133310952527266262167535147219.0097282816338662559975724448522', '37222409666027180186342377726.757757432955738', '6237100000000000000000000000000000', 5, 6);
    T('444472135314587755.319479540707', '3962620798.04749503839000', '444472131351966957.271984502317', 95, 2);
    T('8291859467.036771800071', '178368223105505790128385690893042490.183975576276104696292601', '-178368223105505790128385682601183023.147203776205104696292601', 72, 0);
    T('94965339455591219499185014559421541163.6633350', '385403214854887982612496285284.6216684061257091131288152278', '94965339070188004644297031946925255879.0416665938742908', 54, 1);
    T('5887748429183540.749777121369267029897316042', '9998770679567425019.99197386706360765976943277', '-9992882931138241479.24219674569434062987212', 42, 6);
    T('13325340544999662361893998134.5602785849796499690345021', '39465827301376598.0288844331101947477444', '13325340544960196534592621536.5313941518694552212901021', 67, 6);
    T('3946747221918747.902506726322005', '758927103159340848716342382526278718.7299', '-758927103159000000000000000000000000', 12, 5);
    T('33562.0160526756', '73209155556070785504107545.710', '-73209155556070786000000000', 17, 6);
    T('670.184112690494563955235896396921', '9271292.3420989963207125385201419797522756', '-9270622.158', 10, 5);
    T('593241103079025514337409915.70114178018058563738728', '450294.69476439077899021362564926408526481', '593241103079025514336959621.00637738940159542376163073591473519', 97, 4);
    T('4329305125085177.1', '3106209187006001173016550489085.739532527987166366', '-3106209187005996843711425403908.639532527987166366', 81, 4);
    T('577536623220532793390850010105078974.0065263525525362155565737564714221027', '6064686726796743369541.57762737151', '577536623220526728704123213361710000', 32, 4);
    T('8625287686.91851865807439150053374914377478', '9930909287165675728516980800866.6571678253415208945641375878466', '-9930909287165675728508355513179.73864916726712939403038844407182', 68, 4);
    T('75188207444044491.17', '774412099.396423740225305545344856735016443191', '75188206669632391.773576259774694454655143264983556809', 74, 5);
    T('508495061.9', '2968235913282085434590095.733463498737483735008370005598417381322', '-2968235913282084926095033.833463498737483735008370005598417381322', 87, 3);
    T('6.14520286450274', '7198916039347246757920.533', '-7198916039347246758000', 19, 3);
    T('6020709.218952577212729', '2451525.38148183298581947717459', '3569183.84', 9, 2);
    T('713665740932314968765.935256209040524865850', '23806211323785452136375149007435848737.705643', '-20000000000000000000000000000000000000', 1, 4);
    T('870678983259237111562315.08956864575174337185693817742138863', '89050590228069500762.53171058371273084923720149137315', '870589932669009042061552.55785806203901252261973668604823863', 82, 2);
    T('9171223913.73945116154077426493', '46998718471206597835461320180795791.640044752998500064936219751387359194529', '-46998718471206597835461311009571877.90059359145772580000621975138736', 67, 5);
    T('8233268852335773.3549123328229582429153023603760903987491', '6942575000540916469754.850771163994620521473', '-6942566767272064133981.4958588311716622785576976396239096012509', 100, 0);
    T('4488139801683055542240107146695105.848647978969869361853991165484950129', '3259771224356124379590333742823217142.644680246063912687', '-3255283084554441324048093635676522036.796032267094043325146008834515049871', 96, 3);
    T('1349406097534739959597856.4574', '85069032781311369711660512094026156.564718780397', '-85069032779961963614125772134428300.1073', 39, 1);
    T('18652559385267089599742.6762401874851019862749796555096056125935', '9465350012317368387476987380134222333.376197308', '-9465350012317349734917602113044622590.7', 41, 5);
    T('55.33760223957494', '6748462273223637.823953048', '-6748462273223582.48635080842506', 69, 5);
    T('2475532871893240844775588055535647.60341974858269079164931079527', '843956647170772298946248.03480690552497592317276707125', '2475532871049284197604815756580000', 30, 1);
    T('7379160998931.2200886129676503756086', '432462328487918453306025797587768349.559552118696526639713628305686517', '-432462328487918453306018418426769418.339463505729', 48, 4);
    T('698846422.33634747495289635249655320296912753437', '23885426.595534493387247539074336971386723', '674961000', 7, 2);
    T('48158886.097118637622227526276147', '72187186420492196721456.97482826610485901231407', '-72187186420492148562570.87770962848', 34, 4);
    T('9687014230679175112179922.8571917630496037252101677', '914352905675821533533403931123282749093.2626774791543872174719945141767', '-914352905675811846519173251948170569170.40549', 44, 6);
    T('928553.0', '207.274319331351302827533868772715443419', '928345.725680668648697172466131227284556581', 85, 4);
    T('7753058170437984402154390860451976247.4982486088982623444526417708', '38717.734422629168331715', '7753000000000000000000000000000000000', 4, 1);
    T('50495.0965393931424896', '129700782881307789022478898014.549743579871962035144075689966370', '-129700782881307789022478847519.45320418672947243514407568996637', 81, 1);
    T('6.08521892630930037082903', '370.317019594390972597745851184823860', '-364.23180066808167222691682118482386', 43, 0);
    T('799834170024755273487893413481897798899.40126365797128365654', '822524174520894747.273422702727730807352885420880', '799834170024755273487070889307376904152.12784095524355284918711457912', 86, 4);
    T('96142536108507607967701223.1121720245840978999806523243927', '3716340227448182415851685351.021025', '-3620197691339674807883984127.9088529754159021000193476756073', 84, 2);
    T('528776432439798728337318077953.84514218080450606', '6288595019710618861189.631452000751854049782164045702477', '528776426151203708626699216764.213690180052652010217835955', 57, 0);
    T('9247437531337254547862280471915196390.50258078752261063424692870276975101', '98773.30849509492003656898728391091364', '9247437531337254547862280472000000000', 29, 0);
    T('4365378193252707719689073.0342731284831072307', '77750951741329236018545.310574533842', '4287627241511380000000000', 15, 2);
    T('9292.98036397716307559', '5.508541', '9287.47182297716307559', 44, 3);
    T('9156753.71145521380338537169993594038551659', '52647609651.3984221700163', '-52638452897.68696695621291462830006405961448341', 93, 0);
    T('39109.4716642233043053506670349947116000032851', '18199894468049801251503271237.784102199534271743083738503670573', '-18199900000000000000000000000', 6, 4);
    T('59202643.7948126', '57380580520316155422538.628416958163', '-57380580520316096219894.833604358163', 79, 3);
    T('5.0983580068050769876', '879078562976849474512925088.485329320146339487287868459', '-879078562976849474512925083.386971313341262499687868459', 56, 0);
    T('441493284627533887436572381.9096205816449085498553320163446828608', '970206153731841.73485214799115', '441493284626563681282840540.17476843365375854985533202', 53, 0);
    T('67256366358228202232548158735873.74250338963850841', '83009007572461518979296782158158.67677985337800014218821864363', '-15752641214233316746748623422284.93427646374', 44, 0);
    T('380323811446514704183.348020466548047999881347573693426', '597209608863039768193.3057946161886775489675769754019561728', '-216885797416525064009.9577741496406295490862294017085301728', 69, 2);
    T('528539198909100533.28804311417075', '76418655430938288055069522066293.6689322879127636764309380081484101', '-76418655430937759515870612900000', 27, 1);
    T('744589.43229123271276218336066136314530800170', '6912143.0', '-6167553.567708767287237816639338636854692', 42, 0);
    T('5655885615715.353421874901147962476211281036321', '11756021984037215704447637234.659', '-11756021984037210048562021519.30558', 34, 5);
    T('3147796241646067922137919071.889', '8334762282988247764448155517.956251', '-5186966041342179842310236446.067251', 75, 2);
    T('2462762133590426031.2480365146714634', '21697940770.5898287025268179', '2462762110000000000', 9, 3);
    T('948829083.25969843944161211381493608', '212604531578421624058047718.5125357776102393298563', '-212604531578421623109218635.252837338', 36, 5);
    T('189561245623591553645915815.95815521314115884058693581567634', '487632539618914235264852331420000994858.7522413086214272888047432037063809983', '-487632539618724674019228739866355079042.794086095480268448217807388', 67, 5);
    T('81168013798591067766410683.1758901348107825773395981016092040629443', '765493013045.079175833284631396', '81168013798590302273397638.096714301526151181339598102', 53, 5);
    T('3772976243185961900049726993979584.606869', '64379699651979.0107779330306118810', '3772976243185961899985347294327605.596091066969388119', 82, 5);
    T('5735208.432162329176646913277302851593970', '614025286627535477213.322903254379769840689871352789206268', '-614025286627529742004.890740925203122', 36, 2);
    T('557239721.5082561292225093137594339', '847652641183846821200733186344133081528.04876791896353906046555', '-847652641183846821200733186343575841806.5405117897410297467061161', 70, 5);
    T('6112350884868913772958228785.8257686345334493989345118587', '9854881435675665891.04475267712532523', '6112350875014032337282562894.7810159574081241689345118587', 57, 5);
    T('1.96', '79840119520386.795521787407525043', '-79840119520384.9', 15, 0);
    T('6743159945600993462178689471258.44943703713759656834235916666', '232033936184943418.99287893959476268372544', '6743159945600761428242504527839.4565581', 38, 5);
    T('2882278741660769256133414900.71679187685737765745952731009884008', '3363.222365', '2882278741660769256133411537.494426876857377657', 46, 5);
    T('7278877492741322.3', '6336697594454085745196727442492154838590.9256', '-6336697594454085745196720163614662097268.6256', 76, 1);
    T('6475911796117175336344383608237076964.7216429049874340401023934982051', '30555176958755542528698008012584747543.152344459849494080200135013', '-24079000000000000000000000000000000000', 5, 1);
    T('7909194963307317419258705407119.9122325003980666971322340641480574', '7306700236763196.4498995', '7909194963307310112558468643923.4623330003980666971322340641480574', 70, 3);
    T('38175782708906548341002021513157780005.045812763991146840', '948.8108370005614442', '38175782709000000000000000000000000000', 11, 4);
    T('940.0040292670153265735098616599', '19055519926734663045228.328876416462874399010649317627783217', '-20000000000000000000000', 1, 3);
    T('12449737365422672609380279721739274.97120479725039865753471133639', '93607149.545711594264079009047469021355857034409', '12449737365422672609380279628132125.425493202986319', 50, 1);
    T('7030331218896995067709337292284.88544667999717', '4507594095857266466906.97003983048540532232', '7030331214389400971852070825377.91540684951176467768', 87, 4);
    T('2435367926899578697391789997755291469152.1112780803485460171', '2985999556512972253269913808270802.85809439196500', '2435364940900022184419536727841483198349.2531836883835460171', 72, 5);
    T('987321233893202734879885968243769275.3607792215178', '172731349939437889811.805715', '987321233893202562148536028805879463.5550642215178', 53, 6);
    T('559870236947280542632991611232330694559.3977447556169618675', '6611115193983141599140521108019817140713.43502071108', '-6051244957035861056507529496787486446154.0372759554630381325', 66, 2);
    T('69262065710216136281481687925516860777.0092888266531525618463510148372882', '918770373347082985411993875802.61628524353581691218293075411057018', '69262064791445762934398702513522984974.393004', 44, 0);
    T('27218374564004961452232041.691200816988251048184868661165852', '643552961290036450535558169662406.0', '-643552934071670000000000000000000', 14, 3);
    T('2916853350758787902600347470008167.46415302900472569410553015', '827730189582648940.680432141', '2916853350758787074870157887359226.78372088800472569410553015', 88, 4);
    T('95967009069274.132488988277929586', '59262786010127.7367260418905', '36704223059146.395762946387429586', 44, 3);
    T('65636518167088596738928958.454823955797', '9214129443293787035471181593038975130613.9883574363123', '-9214129443293721398953014504442236201655.533533480515', 52, 6);
    T('63731652803461998779183259875537174.51713', '2927624545109076.459', '63731652803461998776255635330428098.05813', 63, 1);
    T('4129846543749031.7259974904780056379141774293722418287790', '3189082985728316727610973.357123', '-3189100000000000000000000', 5, 3);
    T('1483885606326658683601087319308.67485558386776', '236150283320393615712078407319.08306528304776340378021408151540', '1247735323006265067889008911989.59179031', 39, 0);
    T('591865242823368166.4757984', '65796690.0339931', '591865242757571400', 16, 1);
    T('120958348651.806584943259371268881466718470323', '210402823666341829871118195945624219087.60208077163109415626', '-210402823666341829871120000000000000000', 23, 0);
    T('3968688374645966469.62571378618383828', '7215880671191811.899089287583376198550599949530820023884', '3961472493974774657.726624498600462081449400050469179976116', 71, 3);
    T('9700423231.07506854942649782627306455', '47050930120246496.1709581758372432409108500480571263758351', '-47050920419823265.0958896264107454146377854980571263758351', 58, 5);
    T('3150822822884386664075973844258644856935.067', '602009232067272091276993626.30805978483986859833471506040', '3150822822880000000000000000000000000000', 12, 1);
    T('87252143142.305698', '41745958175457420145410827589003204155.4847070', '-41745958175457420145410827000000000000', 26, 1);
    T('62023354131271356783471884915741372750.21768', '5663404460157941608036273.08671537759718926529', '62023354131265693379011726974133336477.13096462240281073471', 74, 2);
    T('73354265713856.62060777', '4498948713774792.094690336436913019983653', '-4425594448060935.474082566436913019983653', 49, 1);
    T('6429266921531860439967582800469740286.0113530738698', '63268.545', '6429266921531860439967582800469677017.4663530738698', 69, 4);
    T('19236225408852765448594331.91536', '3206082920006706322.10394349662771901884', '19236222202800000000000000', 12, 6);
    T('764243978862621658568488673.412677598394', '17555948048.644766442857476081792738', '764243978862621641012540624.767911', 33, 4);
    T('2246074445885905950545695147966482.56305675526', '63184879368227924819167955994208875.258684564221450884101704921461937', '-60938804922342018868622260846242392.695627808961450884101704921461937', 68, 6);
    T('903434793.002412831279', '16406359250264206138570450754.0425499565282025342192799285885607', '-16406359250264206137667015961.040137125249202534219279929', 56, 4);
    T('1149069172784359347.5259640571911371858989555108363859014', '6172371030360336064621912621956348507471.073593322361565750', '-6172371030360336064620763552783564148123.5476292651704', 53, 4);
    T('882747716641969.7891429843717101331496524684510161322', '67854562871567668095013762515.2869669786437069994908035379', '-67854562871566785347297120545.4978239942719968663411510694489838678', 69, 3);
    T('467329280317898053.91593833808201116391176850', '23014904430700666017164347188460.460778616821881072317448275729665', '-23014904430700198687884029290406.544840278739869908405679775729665', 83, 1);
    T('53522676249056.26686772493645924971292067', '7045286418277772668712569576724963.144337', '-7045286418277772668659000000000000', 23, 4);
    T('5870318966594915907426135680072279374.561963467', '35.1109585395095368897408150132', '5870318966594915907426135680072279339.4510049274904631102591849868', 75, 0);
    T('62658219957774686268169568052338506.2416096307138110419899835970800', '2.44372321840498314012004896685129401', '62658219957774686268169600000000000', 24, 2);
    T('8558357232370379648399859749.89791', '9169803177240528923735853749.20425883822447291954607462', '-611445944870149275335993999.30634883822447291954607462', 53, 0);
    T('537335283063408776825566033.99141072226742511173884221177', '19257570.79608401645713836439956', '537335283063408776806308500', 25, 5);
    T('9290185.89', '8277705147712241681216525.924', '-8277705147712241671926340.034', 60, 2);
    T('5272974.8182045369814286060541464517561', '25020.67818304623211643076', '5247954.1400214907493121752941464517561', 50, 6);
    T('224292182501399729946382.663961088565278944075664165177178910', '24421023.051877772106165619465715175783618112', '224292182501399705525359.612083316459113324609949', 48, 2);
    T('83239607644312581.163242631', '56225775429436528315383367618772.97358387903619', '-56225775429436445075775723306191.810341248', 42, 4);
    T('61526954597215607436.55684014154503046411', '34629271771011181617074264602838926.26446', '-34629271771011120090119667387231489.707619858454969536', 53, 6);
    T('79251201348161.2143091569657701767924', '66480956387597308230935.496002279418400861029665578119', '-66480956308346106882774.281693122452630684237265578119', 61, 0);
    T('27821527646530921571.51857', '826.98745322744151415698237570248217048600', '27821527646530920744.5311167725584858430176243', 46, 4);
    T('15116856952611724474320267345092520497.2929065129491705544344695105878846672248', '788.23618161063463199045823038', '15116856952611724474320267345090000000', 31, 1);
    T('226796670963654040369569389.241434848474491779511948032', '9275866078.31514642837054420309475', '226796670963654031093703310.926288420103947576417198032', 85, 5);
    T('5343153732203496350888385795485619.0461485', '8353905591093296818382890.5151015115733000304516774531860', '5343153723849590759790000000000000', 21, 1);
    T('3933990949667233301534785190267654669.04031387283948011208', '378433295349981656.362118566600926473621047165032', '3933990949667233301156351894917673012.6781953062385536384589528', 62, 6);
    T('781993101417796337226480.3121169329738', '9131762338834492630031022091.50844026863226743475576939604427870984', '-9130980345733074833700000000', 20, 6);
    T('95642347210016140677540876260.1390035735119800363181321881', '2.0', '95642300000000000000000000000', 6, 1);
    T('191967041971199.13366833589009261', '82606404.777409721313535', '191966959364794.35625861457655761', 82, 1);
    T('65192266833.03009504268413', '64592313975536705307835504886537828532.893694632757066235315891156292558503033', '-64592313975536705307835504821345561699.863599590072936235315891156292558503033', 78, 4);
    T('828732.0582437874751532558', '45078905206.393671736452', '-45078076474.3354279489768467442', 98, 4);
    T('8942062116692524518813903745.4355947860838304153642621036163541', '7058848612594843714904536051352.2353391369063009577178832303164680399820', '-7049906550478151190385722147606.8', 32, 5);
    T('393653844646189201948298959572042605317.37492007703', '53592214571506199249125227027.49975466058251468920704475984643246', '393653844592596000000000000000000000000', 15, 1);
    T('309638039158190896128455.2695125', '97127.1', '309638039158190896031328.1695125', 54, 1);
    T('1678962317.41463138922375160646798707', '41739843800513184615225412980846132.884793900611897184526323916302339', '-40000000000000000000000000000000000', 1, 2);
    T('89919693142828475845550336.40984', '66234353.37640012897941395382311572710', '89919693142828475779315983.0334398710205860461768842729', 95, 6);
    T('876.315537184922459', '224981715031368440288667871567382989310.50203752361027555', '-224981715031368500000000000000000000000', 16, 3);
    T('2697029301188522531915121273888.0', '32.29126157861271429757', '2700000000000000000000000000000', 2, 5);
    T('4991127324428588843765.28478266664204932038828963180', '4578559878759889389617175460463168242299.3014763229891552386910', '-4578559878759889384626048136034579400000', 36, 4);
    T('86467570231.494789006', '6502.593559028488053903332229853313925', '86467563728.901229977511946096667770146686075', 49, 6);
    T('6441724890494119818178613119634559.627787083201413112570586', '6189326269210.0340', '6441724890000000000000000000000000', 10, 1);
    T('623380253187722028.48900764286234547681511476703197928', '808715497807031098153717247562053304.09459275', '-808715497807031097530336994374331275.6055851071376545231848852', 61, 5);
    T('493733412116.98624503109127255596239097264', '63551976697335281945.53800201531488995362952312625', '-64000000000000000000', 2, 4);
    T('880211996777371197187599937408.72919377467', '43.35073788960246324182504437071', '880211996777371197187599937365.37845588506753675817495562929', 72, 1);
    T('84.0', '45406693.7306798347436637', '-45406609.7306798347436637', 50, 2);
    T('71289038.38790248', '95196709300941507.500481594058421029747609355081075831282', '-95196709229652469.112579114058421029747609355081075', 50, 2);
    T('9879510817128.0600', '1416509866565065496.619946318556414662110161965', '-1416499987054248368.559946318556414662110161965', 97, 6);
    T('37190987217873343325846895358809539.359145998', '94607.485640051542', '37190987217873343325846895358714931.8735', 40, 1);
    T('6934222394621.133615264567432945550873175510', '7533709333372691470691917103459051.16494500986983032020012', '-7533709333372691470684982881064430.031329745302397374649', 55, 6);
    T('9420615064698986537884236111165023943.69958586969269119674', '4595059286840758972.093742877799', '9420615064698986533289176824324264971.605843', 43, 0);
    T('5698153.226086621809503036930939', '972553626901.109258947830799590', '-972547928747.883172326021296', 27, 1);
    T('4353.15872196518614713932629200781', '98119530918464835641402092.95398306336943121069466060', '-98119530918464835641397739.79526109818328407136836859', 52, 1);
    T('57271721119746028768968039104999268.720392634010000671182531677', '5.74624260232986109630029405452776', '57271721119746028768968039104999262.9741500316801', 48, 5);
    T('98517737043.149059862352629322879201019180', '988140577970.34106484119487227378604447354672', '-889622840927.19200497884224295090684', 35, 4);
    T('7.041882040943482167', '17908306418973789454270.02273583487830011537014746076', '-17908306418973789454262.98085379393481794837014746076', 99, 2);
    T('3664703927684581313957329264689.503182020726644045090749122755902572097', '569748.18730185317924812', '3664703927684581313957328694900', 29, 1);
    T('7709508821509820373192975921129690086999.4599467983303206516688454570178736428', '43351431915873.191149852749699626462', '7709508821509820373192975877778258171126.2687969455806210252068', 62, 4);
    T('4482672431421.7267254', '154210538265641345323062.45648967085538629012945918453239962', '-154210538261158672891640.72976427085538629012945918453239962', 67, 5);
    T('45138341921836.6892192823276749558485782478183474792', '3930515844607782196295856107754795590.549439', '-3930515844607782196295810969412873753.8602197176723250441514217521816525208', 83, 2);
    T('322505679310127264450213401109617719063.2790695630', '331541510700115753285.6462148', '322505679310127264118671890409501965777.632854763', 99, 1);
    T('640277117256433725922035155.9766924239419044179331820056067', '745042299172435.72704', '640277118000000000000000000', 9, 2);
    T('78874412818493559651815297062013043816.4289', '99752322542507928185150785.5554391353698', '78874412818393807329272789133827893030.8734608646302', 63, 1);
    T('6540234.8983835179901748520', '64739046801176.893835864702', '-64739040260941.995452346711825148', 56, 0);
    T('4537639510539735037212174622.48292008571857066992529560', '7643128986.363699069730183202179190709', '4537639510539735100000000000', 17, 0);
    T('8545077161947881760561597.412938102', '3475859362701616596.85665285230660479447', '8545073686088519058945000.55628524969339520553', 77, 1);
    T('91102482671144071007057839108.19095589580084537377435352425376178228', '127.9843089627612563861423061052009298', '91102482671144071007057838980', 29, 5);
    T('68583093720021.27809546822139444274', '1608644943478870604188330990187766490355.3080206803366655287274', '-1608644943478870604189000000000000000000', 22, 3);
    T('675363974814134209041435425967299.919739212959071', '706478.121556605052138665453359528641081308', '675363974814134209041435425260821.798182607906932334546640471358918692', 92, 5);
    T('7244152015082246916277460060877492590.94', '141090486999625892928653.0108320217709852731', '7244152100000000000000000000000000000', 8, 2);
    T('80873010708103754145658759.6482', '84632025371445.6843052581424881786612287497941165', '80873010708019122120287313.9638947418575118213387712502058835', 72, 3);
    T('9966909760.246', '9515.09', '9966900245.156', 98, 0);
    T('6597916803827.49687237605739', '539581699066083.9009497486082853941570838471179188991', '-532983782262256.4040773725508953941570838471179188991', 99, 1);
    T('4043462449726597.800616737695096331901', '37525766377848169.17026391661574197416849284951891824', '-33000000000000000', 2, 4);
    T('15.3360393402541868242282792325', '6872380801009472291033047.8246161', '-6872380801009472291033032.488576759745813176', 43, 3);
    T('5277909210543372929112504.40', '69409215376840.3364848898317638648742', '5270000000000000000000000', 3, 3);
    T('4642037362656675989238.0', '479267717463884516668928098042.577', '-479267712821847154012252108804.577', 92, 5);
    T('43914436114421636659260305238.93653483354408821588781037304343362', '92597648808653016462.37027319233683112826090148289132', '43914436021823987850607288776.566261641207257087626908890152113', 62, 1);
    T('38067252396619.44086394207342392371', '326917431423536.3565450914738905466315', '-288850179026916.9156811494004666229215', 53, 6);
    T('46912860949883790470835449262.2593118849292097785424663569234627066358', '9053126867.72713372313853115625414719', '46900000000000000000000000000', 3, 1);
    T('55830762149125958871054001535.7120652978', '756133751697524181432472266576.6998818', '-700302989548398222560000000000', 20, 4);
    T('4273116020416772635597436206.6188559329322176528374270752', '2698494712240176491576176779152244.2080550847023233318214467910559281850164', '-2698490439124156074803541181716037.5891991517701056789840197158559281850164', 80, 6);
    T('76944508893466559935893.294428466', '185413955547005033361388702717871682.14824957584304', '-185413955546928088852495236157935788.85382110984304', 56, 1);
    T('69967490625972413515682567193247626.1992155267786', '869342276198939419615640356107679350312.0', '-869272308708313447202100000000000000000', 22, 6);
    T('1.32835790663138694673485', '45443.03', '-45441.70164209336861305326515', 45, 3);
    T('111136857840674402327120.76396347', '8347900862891063256.848661668625892345', '111128509940000000000000', 11, 0);
    T('6941904287763968471683274506582.19815269469742486004784896247651', '7565.530762736492187310677202023575384', '6941904290000000000000000000000', 9, 6);
    T('608.6088350054003335708255446590644736797', '45228449647551660061993964.80', '-45228449647551660061993356.1911649945996664291744553409355263203', 64, 4);
    T('518034210094488640505.8125500649589274472512541173231789988', '9295968238198.5342502820448', '518034200798520402307.2782997829141274472512541173231789988', 65, 0);
    T('8.82700212733657341685204', '86162844131048461841.59', '-86162844131048461832.762997872663', 32, 1);
    T('39175863046649887301649047907.26159554590544848747394419614532441', '52747937626962664957636952.75341398858594824336871', '39123115109022924636691410954.50818155731950024410523419614532441', 75, 3);
    T('585823360827408570731285340.3571222469714245001493669920459635', '898038268865688200449466493.7990330732490673511239049465344528288125', '-312214908038279629718181153.4419108262776428509745379544884893288125', 83, 4);
    T('89415269389311115973.44229924075681442', '4525851955714943303491846328761826.1741537120822130045342303700', '-4525851955714853888222457017645852.7318544713253', 47, 1);
    T('7301927836409551787.705877', '60226.8747121376646647771917172190290224', '7301927836409491560.8311648623353352228082827809709776', 70, 1);
    T('4416160188321458.5210533382405402872142190354885669', '4007697471280288536849128609850159335332.9667987', '-4007697471280288536850000000000000000000', 22, 0);
    T('68420311447787407.00271652376083742805', '7692888315485548016490740753659.31', '-7692888315485479596180000000000', 22, 3);
    T('9464408970028437596533198790188.735181432987632649', '24206692600607731484401290939669466685.85306997748786144442125417536359881722', '-24200000000000000000000000000000000000', 3, 5);
    T('87.4385294', '970518876842408439848707.6922550303127836584968326197698508', '-970518876842408439848620', 23, 2);
    T('129263426653993556321645179612267.0756845', '150100615.6138579569132007158351043428927', '129263426653993556321645029511651.5', 34, 0);
    T('1653155582056136218.2135157183065603355394102292', '2226892153553444721364063555967856043.53', '-2226892153553444719710907973911719825.3164842816934396644605897708', 98, 5);
    T('2.4926718997715036751218120412162938', '2999007153065346784.05273', '-2999007200000000000', 8, 4);
    T('37048434753690066561923974959461311.4981369', '24707674152607036354345528.403132807030373760', '37048434728982392409316938605115783.09500409296962624', 95, 5);
    T('1212134197.4437613204', '2394.1754315819208966740331359356386', '1212131803.2683297384791033259668640643614', 95, 5);
    T('2880666157070119285933078.9184798', '46145239869947246728510065642.1884', '-46142359203790176609224132563.2699202', 45, 3);
    T('663761189229054420690540004511693937.65973', '565707883500319.71585', '663761189229054420689974296628193617.94388', 64, 0);
    T('85969738.762227769425011132160672223999492', '849399382348.1295152704332092', '-849313412609.367287501008198067839327776000508', 97, 6);
    T('438315866909122650146.372438042915595480648451802', '1016329836246527280310895.348', '-1000000000000000000000000', 1, 1);
    T('9821014042303938416.747455513424228470935', '646841121087939077120011849363998136351.182', '-646841121087939077110190835321694197934.434544486575771529065', 92, 4);
    T('592.860826813794899570308435813493', '789784.7733', '-789191.912473186205100429691564186507', 75, 0);
    T('5686399848626795288917859972710.42339496357422430084886193395342868463', '4876003016841695409035465535.66246357268518', '5681523845609900000000000000000', 14, 1);
    T('33456329894850.1465768079939667008476890564199647727394', '10.47', '33456300000000', 6, 6);
    T('11417089507637279.6340239', '1796297493947763983436542782900.586636559778701762519248', '-1796297493947752566347035145620.952612659778701762519248', 70, 4);
    T('65314876261744.8113628420931490087799286552431909833', '4521403.4114426493', '65314871740341.3999201927931490087799286552431909833', 60, 3);
    T('8.88223052274489629227', '42046.177436581278418112477560124464887661', '-42037.295206058533521820207560124464887661', 43, 1);
    T('849619144121121.30', '13.3647490943037524588889594204', '849619144121000', 12, 5);
    T('766.279319053021720112', '930634444854678385428669641954971690.9313743029649490216049', '-930634444854678385428669641950000000', 29, 6);
    T('3222954848803885945471666711355832791204.511400', '4744077169252772429106690699504263.31668123', '3222950104726716692699237604665133286941.19471877', 88, 0);
    T('884042287430606526821.37283665259517709919702729146145802', '49733.8689986463105', '884042287430606477087.50383800628467709919702729146145802', 75, 5);
    T('91.73', '1790017034802966501052395872180808954.86861039944265089193200394114565', '-1790017034802966501052395872180808863.13861', 42, 6);
    T('25346887673836478747.68912146967528106382967002423735', '5447435822127328.64570946808322904114432218548490641361', '25341440238014351419.04341200159205202268534783875244358639', 83, 0);
    T('105335655284028149104560972.149873200212658', '1303170708219728153159542227465718.0', '-1303170602884072869131393122904750', 33, 0);
    T('70659.309287758439600125763', '757011.33830351048267924511655746226260199', '-686352.0290157520430791194', 25, 3);
    T('7343362602910820979195805.91468125760783957174480217541', '986727773042594846726427942255571.9', '-986727765699232243815606963059765.98531874239216042825519782459', 73, 5);
    T('788946878729563782619587.184916278845997', '429722667058613379.22061136', '788946449006896724006207.965', 27, 0);
    T('163665870168184108808.33472173492844135474838648', '74149569111271706135455142514550.75672086323404654296', '-74149569111108040265286958405742.421999', 38, 2);
    T('73606555229063.033023957630', '6896009039140.049738750796881626610', '66710546189922.98328520683311837339', 36, 3);
    T('7511946320918692687271362275014303357631.84867551781525504720261', '273278838885231509756204815537.15826', '7511946320645413848386130765258098542094.69041551781', 51, 3);
    T('59677627038088517989739.0594839635654647807213972673241', '3156298996439287998362990963284193.1435862', '-3156298996379610371324902000000000', 25, 5);
    T('3682157425186159187379830.978737842714670656506810854', '9022713.53302838070565298', '3680000000000000000000000', 3, 6);
    T('398719041511642781.99108865', '3059011873386711720861521762619038507.41280', '-3050000000000000000000000000000000000', 3, 1);
    T('1200419.23', '3093303.11976', '-1892883.88976', 15, 4);
    T('793106665049785191709.58871', '12918293489.6984072719805562', '793106665036866898219.8903027280194438', 98, 1);
    T('899210683810120681460070888.5884', '53641451730651891048187281282654465586.35869539403585584001', '-53641451729752680364377160601194394697.77029539403585584001', 73, 0);
    T('40731450.252126746919874607855342399734723508', '951623687627061898080921.520298950965545671235799501', '-951623687627061857349471.2681722', 31, 2);
    T('3546474544975153444548.93666960988303723489112609254', '660161051.2068', '3546474545000000000000', 10, 4);
    T('2152157587161589456346072998391391128060.3976', '7681755871004871827017231.7857804054933614626360167904289790', '2152157587161581774500000000000000000000', 20, 3);
    T('761.3249588878617122363627384', '54.41328', '706.9116788878617122363627384', 64, 1);
    T('8765930584220890053496261026552082.8593932655020288514323075018094673', '486685397198363.82384614542', '8765930584220890053009575629353719.0355471200820288514323075018094673', 82, 4);
    T('18097960072651951367103527558.80597913726335821754199196279', '2336399394412854199471111075894893.35685143394494693267414003879370882', '-2336381296452781547519743972367334.550872296681588715132148076004', 64, 6);
    T('537056.9056597026671648652440558600569802', '8420790272.297908313828538219', '-8420253215.3922486111613733537559441399430198', 82, 2);
    T('318849051640275482862285941534627217433.286551128', '369846038904134503420513.1147895106720844662335584997329764695891', '318849051640275120000000000000000000000', 17, 2);
    T('874995793457992834255395822.869262540796', '1778115.241471815133324682151931125029', '874995793457992834254000000', 21, 2);
    T('77523321.0766017826614863584751185359195049183', '3552322427094537622089601225828481365.5559294499545477465359814743500431', '-3552322427094537622089601225750958044', 37, 6);
    T('42513833223665667936586935362685.4719856360437685874381757424966542446', '958.094234150760581591155618', '42513833223665667936586935361727.3777514852831869962825577424966542446', 97, 3);
    T('166.831351373', '31541962274.70436972012781270841423863580106347170', '-31541962107.8730183471279', 24, 3);
    T('16186834778495867441413375.9912325209622522394920', '89.8112803118', '16186000000000000000000000', 5, 3);
    T('45320080941002633915325056356068544.55005648474598306601244216578874', '52695752757798.0', '45300000000000000000000000000000000', 3, 4);
    T('7843045311839250582691391259419.9025', '4927783316711865542690216815463.0459265315546118038761757370316118431', '2915262000000000000000000000000', 8, 4);
    T('17877839499266592513.3173779606048436077451870320293', '778151869790.730101589217995481856348826016782094', '17877838721114722722.587276371386848125888838206012517906', 95, 4);
    T('79714135813157415.574341078', '1.820756765451378166779936771036', '79714135813157413.753584312548621833220063228964', 76, 3);
    T('9719017546681.571061240567187018736376', '64720823.5', '9718952825858.071061240567187018736376', 49, 0);
    T('99687206.2939019877381109772239330', '4702932607.5574202267', '-4603245401.263518238961889022776067', 96, 3);
    T('88159765.8899462859388875145', '6261345123737026023622274.4810', '-6261345123737025935462508.5910537140611124855', 71, 2);
    T('38677673491154.7100549363424868428913', '14603387914985538891362532274470784.6225832767079676311963857', '-14603387914985538891323854600979629.9125283403654807883050857', 98, 2);
    T('50532.6087872008829', '5213916097828760300457.026473147359489676086', '-5213916097828760249924.417685946476589676086', 46, 4);
    T('232728465723847746.25024', '9171412651221.1157440754606715676002289878', '232719294311196525.1344959245393284323997710122', 77, 2);
    T('69646377312120273097543962895846470583.573546621125939013153808341671492145926', '4169488967426439.575129224671520349266038448208357291486', '69646377312120273097539793406879044143.9984174', 46, 0);
    T('91368062832.52498780175706', '7206669017117879796450444751.87905940', '-7206669017117879705082381919.354', 31, 2);
    T('76585922290084901276156.8805611', '46499965204534005774803884811101.5480781177990', '-46499965127948083484718983534944.667517017799', 88, 0);
    T('1513897250834502416576743.53090378666', '6014372858768512.64436205962', '1513897244820129557808230.88654172704', 50, 6);
    T('760680994217020612124721911573360852.912578246737024753075071644570', '978167460.709716361180131', '760680994217020612124721910595193392.202861886', 45, 2);
    T('5544465762225812586.4100874086485238', '724864338504555727.35012', '4819601423721256859.0599674087', 29, 0);
    T('688794.0344507174226626647188514821', '4.48160895921298053', '688789.5528417582096821347188514821', 96, 3);
    T('1680360946930986.9174764283111369732618353744677219', '72544529459412662201.2634111693366804808047582', '-72542849098470000000', 13, 6);
    T('551095780641170216643353626586146547730.9645304701623', '281276307.7615725669254266398278', '551095780641170216643353626585865271423.2029579032368733601722', 84, 2);
    T('595566.554855922158047828471558552501', '64942048978223271336181062.4573402417053794992173090540659270', '-64950000000000000000000000', 4, 3);
    T('901857391220443719636859683460844054.4646159199311845', '76255078296931859155203041874331971707.179552616854', '-75353220905711415435566182190871127652.7149', 42, 5);
    T('330901733262.045838685443384047402682832164', '4075428314053863.066560963067885697485266524781', '-4075097412320601.02072', 21, 2);
    T('77.6265793', '9.64950921788572560826270182526392012', '67.97707008211427439173729817473607988', 75, 4);
    T('8626838055196990763800316979997142045383.24374186450587980860017182807011632', '73478085112852.4332132737766118810750169', '8626838055196990763800316906519056932530.81052859072', 51, 3);
    T('3909809315.22', '3953076759.11772767001201318350767498708412903', '-43267443.897728', 14, 5);
    T('5220039940413277558929156725330.956639040794369796154708348642095527', '76675346295795173393510.0407875836162885972096153901', '5220039863737931263133983331820.915851457178081198945092958542095527', 93, 2);
    T('20435446457316848978772629.5191623139', '973656329483982547.4', '20435445483660519494790082.1191623139', 93, 5);
    T('681411541504599185798.0046', '356225677952577392898391239514278726857.73', '-356225677952577392217000000000000000000', 21, 4);
    T('807855871724099017725795999714.7034844938689717613951495780314', '9786472395216982133788071343507977.0', '-9785664539345258034770345547508262.2965155061310282386048504219686', 85, 1);
    T('8103299951178509510481118480120164955314.7521389', '46547628592414633930900.92272090638443299595780942875', '8103299951178509463933489887705531024413.829417993615567005', 58, 0);
    T('83657002993207730977596624082.5365345212609781484619640696316507', '5377565829170858852085637510922578.21636418055179', '-5377482172167865644354659914300000', 30, 4);
    T('361627457311251756395.69274973405143219296014615', '23651994532624643137760162321387115368.9539694028481996384956772825816618023238', '-23651994532624642777000000000000000000', 20, 3);
    T('2824466854465.35153074896851147704743773', '83.35203013414776604584991304299988445615', '2824466854381.99950061482074543119752468700011554385', 53, 5);
    T('751314625333338879791601800200084.5649906132244488582376805', '8350178050694395780520.23419', '751314625325000000000000000000000', 13, 5);
    T('4654727830583682635504.84124', '685444.7004', '4654727830583680000000', 15, 5);
    T('382404489889254291249058118253821910453.63436273568797321016334', '5347721517681603898048497022217173825.08760198160573', '377056768371572687351009621231604736628.546760754082243', 54, 5);
    T('881023848336.15', '296627872686.363623625514908647994950998767474501', '584395975649.786377', 18, 2);
    T('907610761357461701727488638225081050.3630100542347248', '91705296435698846322705.5165814427054489710945973927', '907610761357369996431052939378758344.8464286115292758289054026073', 89, 6);
    T('8448619729.73718512706745191322779003', '66.57081', '8448619663.16637512706745191322779003', 58, 5);
    T('546766256516876.76876049107861220116', '44.28542611533030460792', '546766256516832.4833343757483075932', 34, 1);
    T('854.6644463111677623376403939185288346790', '6839333245790786312410115196505899503.24831', '-6839333245790786312410115196505898648.583863688832237662359606081471165321', 100, 6);
    T('632218902016238422948994128.274641', '64143864022205.3', '632218902016174279084971922.974641', 41, 3);
    T('2437042482.026680436025574794449912', '1080259787517153130497220211097042772.7736495676682095388712770042', '-1080259787517153130497220208660000290.7469691316426347445', 56, 0);
    T('80237698837.8642221277019813108868330', '99820755585882.33573985462500340559169728780912424', '-99740517887044.47151772692302209470486428780912424', 60, 6);
    T('2659535608780735819.0', '33413764038640443257699878.2939920', '-33413761379104834476964059.293992', 71, 5);
    T('5339.27305399029851838493923073368', '8649983837166.6670655', '-8649983831827.39401150970148161506076926632', 97, 6);
    T('1258.819377855997396281', '87709409856994733439738199335.469', '-87709409856994733439738198076.649622144002603719', 69, 1);
    T('3155023389.792484287098527505387119', '43937962685162855337.604464297884190596566999576770423535', '-43937962682007831947.81198', 27, 5);
    T('322270.23478668', '315395835662155254040545359427.1863403724029196', '-315395835662155254040550000000', 23, 0);
    T('547656626448433538964694147500689672.0364', '68654702060218880170583867874620796.03263258635', '479001924388214658790000000000000000', 20, 4);
    T('422269488056638.359433575453469884655385458850652201453', '9519851987083648.0549', '-9097582499027009.695466424546530115344614541149347798547', 73, 1);
    T('694561202091521529176909789121118.23024669', '79897750363461380791.11267807933041129', '694561202091441631426546327740327.11756861066958871', 65, 2);
    T('4432533474030876561963495732881.992569980565152167629527972', '1130521706549823798656068101095084.544398929775684671610199', '-1126089173075792922094104605400000', 29, 0);
    T('202220604.58460578409', '20488112841741847061595.2449259008958626', '-20488112841741644840990.6603201168058626', 63, 6);
    T('941074080.6526096429478466766569946655125092966301', '684023022588294149795.43580613449807977833871', '-684023022587353075714.78319649155023310168171533448', 50, 2);
    T('99065842204645612168224622085957690.26775367580', '3113936537642024481602603848.7299398109936085', '99065839090709074526200140483353900', 33, 0);
    T('728100365972219334471685108032996.31', '380931206284.088484639202227681574164087535', '728100365972219334471304176826712.221515360797772318425835912465', 90, 6);
    T('5171973722995532532647156.857734516573227885372', '4175.9977019', '5171973722995532532642980.860032616573227885372', 76, 2);
    T('182548331060798822282.034596693703790567933664143001728276', '473207225798328.835411287343768894365296', '182547857853573023953.19918540637', 32, 2);
    T('22132142257598737761328613695955.302803', '44938559125.368814059596393286075084326591809934', '22132142257598737761283675136829.933988940403606713924915673408190066', 89, 3);
    T('4397427540508203754545067149.612936613197242842666251525436220020431', '4869.7284531316870', '4397427540508203754545062279.884483481510242842666251525436220020431', 67, 3);
    T('7137270887183.3859057727', '293377919005718926548855.62669961363321305875274', '-293377918998581655661672.24079384093321305875274', 83, 4);
    T('8122197172115840300727794142083443.641570504185', '341.102', '8122197172115840300727790000000000', 24, 3);
    T('573593107210668162.40108006457', '179.8496188162878563297829422156411', '573593107210667982.5514612482821436702170577843589', 82, 4);
    T('90644775237951.2429291710254560403622761468422661406', '896415534295520847325849805412987449393.65739', '-896415534295520847325849714768212211442.414460828974543959637723853158', 69, 3);
    T('61.0', '7658264354996915881627053902246311395930.301728284808682735582899439915747', '-7658300000000000000000000000000000000000', 5, 4);
    T('65524056.71', '9501796304384249768418.67915355293863807709317047', '-9501796304384184244361.96915355293863807709317047', 96, 4);
    T('39926713641730393153444.3', '1623850046815629599.0', '39925089791683577523845.3', 37, 3);
    T('90054129031445786683966724938478.95228935', '386211681327.082022143334342301149667472874', '90054129031445786683580513257151.870267206665657698850332528', 59, 2);
    T('2892454059374375231913219906.624785829', '49962142416708059.20841492578102130083', '2892454059324413089496500000', 23, 5);
    T('793797579418.8623085177694539420977158096546012097116', '7606163647249.7775913337321700714161', '-6812300000000', 5, 2);
    T('6565313134203714080832241148881.34380657911504590290280', '5226100677166730197430202065419552704298.24040399868281547210357746983047098', '-5226100670601417100000000000000000000000', 17, 0);
    T('53229008673408525684063041003531.5105920303', '111653275403.188470338454905650190087437710139', '53229008673408525683951387000000', 26, 3);
    T('9959520666659332634796194267027.270646757559889585434', '7.25859257644', '9959520666659332634800000000000', 21, 2);
    T('74729467549370985202328946574682217220.76513', '47233659780165107168471849.099899165713099218464930250', '75000000000000000000000000000000000000', 2, 6);
    T('115029286587352004658886467741042265425.4950420918711', '575530390218157833363.9195986567128', '115029286587352004083356077522884432061.5754434351583', 62, 1);
    T('348563172801355190.78845277', '31.10304237737202464328687805316883', '348563000000000000', 6, 5);
    T('6260678181.670730045256897266431000876', '105964907948.15535663754', '-99704229766.484626592283102733568999124', 63, 0);
    T('317632838617086440160554714589.96308368368276', '98.05749129994726901116895383759419', '317632838617086440160554714491.906', 33, 0);
    T('18124713090465471590559055205.4425682057880498371752', '510.304795199', '18124713090465471590559054695.137773006788049837176', 50, 2);
    T('6995.18097445237004746612', '50960972938919423953541913935672.4831470363787188', '-50960972938919423953541900000000', 24, 4);
    T('4722980805676434090849767136.504809587579876300692695611122147867', '308368666049149286020777.8733020883330828530246743227525248789', '4722672437010384941570000000', 21, 2);
    T('37619107366935794131.2078941762301745719937087211229202', '7510221150379443090533157.19193901825367609636947789591', '-7510183531272076154739025.98404484202350152437576918', 51, 3);
    T('38750059770365748297039.05012232282949007101144678', '984623026291518460406867018017.7175313246290930244572909158', '-984622987541458690041118720978.6674090017996029534458441358', 61, 6);
    T('767281329496064130271716153238.8892751800254058432680301865890241', '43254691237852764004.64097076722252328402', '767281329452809439033863389234.24830441280288255', 47, 3);
    T('508824285281875.10', '153197043763409523285368833383488649393.66831', '-153197043763409523285368324559203367518.568', 42, 1);
    T('676566369168299340837312571341.8488940708503548664991038590594376', '3202147070213626425583.12250638332568426259897410983712', '676566365966152270623686145758.7263876875246706039001297492223176', 68, 6);
    T('4283080836237602922622313252.11655016334634177220140', '5267989141484686146211535875359860.050624', '-5267984858403849908608613253050000', 30, 6);
    T('47418361060537515217421827380593.42321343666864972027808055287585180', '4475392865989546406910426567406969.1220882', '-4427974504929008900000000000000000', 17, 4);
    T('7661185775064086936555957.32', '28311404967606152738829034091.03172167439905247035452195', '-28303743781831088651892478133.71172167439905247035452195', 69, 0);
    T('663460741479002279588215051231416002.06223559288892', '43528678000487964454907650488203.43365918464988163844', '663417212801001791623760143580927798.62857640823903836156', 67, 4);
    T('7725140757469069653094109.0', '56248229491166328525773.473810706', '7668892527900000000000000', 11, 3);
    T('6896615777619982376715.0525713909818833032353840', '91788725896973826849776621351740156940.5638395323193500316391547339403424485162', '-91788725896973819953160843731757780225.5112681413374667284037707339403424485162', 91, 3);
    T('42425844858040007.7342164722382385753498917308832961044', '88073052668288790.700578397230327267689', '-45647207810000000', 11, 2);
    T('4969603559136552246.430440199084884288982707164807256779', '250772.1254698', '4969603559136301474.30497039908488428898270716480725678', 54, 5);
    T('35429524004587056.647991393089500940845', '48217891201929.843509968087404024789', '35381306113385126.804481425002096916056', 76, 1);
    T('573797163651801869118869.648630975531506912156704994547125', '611.109457883294709997738830042707906193557', '573797163651801869118258.539173', 30, 3);
    T('78.2580328647164341385385087566232402801134', '73376979305934407654901272957.7717552666592858916', '-73376979305934407654901272879.513722', 35, 2);
    T('36888334982728122707885.49274350', '652210090195778498288690607018247125.9', '-652210090195741609953707878895539240.407256', 42, 1);
    T('21029942490160452094716704822021583.62228860442979853821528796302597829', '49837685744443040102377165256967933.2314547734358904866515186308352354', '-28807743254282588007660460434946349.609166169006091948436231', 59, 4);
    T('672868698388893809.5363574465635091740272105441501851', '4380740817813.0545049274733', '672864317648075996.4818525190903', 31, 0);
    T('51.82032132534729756656314', '4956045487938870204.69', '-4956045487938870152.86967867465270243343686', 51, 6);
    T('3.0805391773301751786365722', '1156575480000776366542553752958584.83189056719471779', '-1156575480000700000000000000000000', 14, 2);
    T('9529296292922339922077975699198703913.0835610179555374428868', '470939965858706373291742919526716543365.2249891042328641503061900833464745301', '-461410669565784033369664943827517839452.141428086277326707419391', 63, 0);
    T('789241.5389867090325479225061112624130448', '4432112093283941724900521460711793530.316784577761239254', '-4432112093283941724900521460711004288.777798', 43, 0);
    T('1874949.860918067919551184928070767998806832', '7427343872152643194.90158799603243189', '-7427343872150768245.0406699281128807050719292320011932', 53, 5);
    T('2386568825037119545.25072664833553826677694110698241', '36436814250864474424662944.8343741569956026100356125709', '-36436811864295649387543399.58364750866006434325867146391759', 90, 3);
    T('73591216.45676087055666475577676459013969677500', '389526023944855.962535083012297974185726560629947', '-389525950353639.505774212455633218408961970490250225', 89, 0);
    T('23.088311758298893795080659846294082', '39878932.997947173932272', '-39878909.91', 10, 4);
    T('411323591831575029472312103038515.980707154263150656878103', '26350619321311068755058781890401817.6118043803548203594790204818695', '-25939295700000000000000000000000000', 9, 1);
    T('99244762361847.925913627231402556675', '46206733012943463126771862170683.440', '-47000000000000000000000000000000', 2, 3);
    T('64870446.2148407', '934581.56175135', '63935864.65308935', 83, 5);
    T('3112861264852326935194.503616048077956174724151430765', '81159099043609225.941550537666370', '3112780105753283325968.56206551', 31, 3);
    T('8842866400.90', '57.833457946360526293921010', '8842866343.066542054', 19, 2);
    T('13699011162420.5445016257421262179834532825776667', '95679204229422162399882930493789.47303861842', '-95679204229422162386183919331368.9285369926778737820165467174223333', 89, 3);
    T('1402206344.333710065953172700963781164630360906', '253625.39556547913729008335074', '1401952720', 9, 2);
    T('2945419941806575892154000162081504749360.53174440828263131', '551493662440.313511265758932913524085', '2945419941806576000000000000000000000000', 16, 5);
    T('4421404545336331448461.682832928085327178', '234969537675824968930397.8280544777', '-230548133130488637482000', 21, 4);
    T('5034341252869739270688503383037700.19548756315117360170502469147219', '7931849560344715208424.6927043074823', '5034341253000000000000000000000000', 10, 6);
    T('7430301841640215298592775473.2091519921236955016', '9084633.76745874068', '7430301841640215298583690839.4416932514436955016', 73, 1);
    T('552.0908843', '8.38974571416191964013', '543.70113858583808035987', 45, 0);
    T('4861845895911385982632220193612120.9262959196446406707680520799249', '531.29420868362196607147', '4861845895900000000000000000000000', 11, 4);
    T('45554302505244701441855299.7634041626812753156665382292', '253710882659204735569310184006549982716.23571992612432572575466883', '-253710882659159181266804939305000000000', 30, 2);
    T('42826218983470281359222.6823095130086454', '290925911448998116015856695534.0607154054186', '-290925868622779132545575336311.3784058925', 40, 0);
    T('9385306955475.901459296389', '53.791187', '9385306955422.110272296389', 95, 5);
    T('2571128311912283496771960971366741579804.861855509', '3234902727323877276984238149462.4471508723426931401429678208394978661', '2571128308677380769448083694382503430342.4147046366573068598570322', 65, 0);
    T('61445340289786940396124382.08986529237958405304776864343927666', '126197140997452.299', '61450000000000000000000000', 4, 5);
    T('2959158.49831', '697801215901832242914675095250.9301822', '-697801215901832242914672136092.4318722', 54, 6);
    T('66671383867124698870282316855.04874137444', '5280796738669315470527666167318812894.61547785083', '-5280796671997931603402967297036496039.56673647639', 91, 2);
    T('585.9872712833873918340159295757221851', '41.08412246771032570351354033929', '544.9031488156770661305023892364321851', 84, 6);
    T('2726420338348650015280572193849763414.8732773227498066772655695', '14307658661.757234959636360', '2000000000000000000000000000000000000', 1, 3);
    T('154457.70234435539829487909173339524890707', '8810139879917502721701564780.0868508121435555762239', '-8810139879917600000000000000', 14, 3);
    T('838318479630767031.6307554', '797965670744801297576152761895331.6540867720465961507194584402279837', '-797965670800000000000000000000000', 10, 3);
    T('2691.11705205', '5044352.118666804274977823576553005836337', '-5041661.001614754274977823576553', 33, 2);
    T('819182758.5792802', '5888.5', '819176870.0792802', 51, 0);
    T('88349139411895422901.4675959220699228', '254789746844334844.150993362837732259133457602078', '88094349665051088057.316602559232190540866542397922', 63, 0);
    T('4438001953.301789000204748437448030448006', '208056803444590655085658217305.7003007633405591595841', '-208056803444590655081220215352.398511763135810722136069551994', 94, 2);
    T('82164602231375752433474712575.702038593165781899288662', '7715679.3634532654037', '82164602231375752433500000000', 21, 4);
    T('99318724872494239776883195312.3', '102192524349996975997817.9726756191949582', '99318622679969889779000000000', 20, 1);
    T('306562539691500208877157792313217462.284414', '339500234273098346171.9910371948163195722012981807675', '306562539691490000000000000000000000', 14, 3);
    T('123410521.355762689244017055081606963', '9538210745.4896123980216', '-9414800220', 9, 5);
    T('5.713861926696', '251764640800807.20634890264510278', '-251764640800000', 10, 1);
    T('323095122462192776018903828320347937.73856158627391373439709067524113', '0.492606124525167367699370033037670', '323095122462000000000000000000000000', 12, 3);
    T('524501.0206955833297215', '75.1022231856203070883493063591', '524425.918472397709414', 21, 4);
    T('155248151108346776225984271990114434111.238646854995', '4.70861968476247296', '155300000000000000000000000000000000000', 4, 2);
    T('2642462672373518459993991846484.81054282', '525687852336517621100148874136699.8761336172330680814540', '-523045389664144102640154882290215.065590797233068081454', 74, 6);
    T('57331685649404581131.8515009853426341392717133', '28694025.01000429232099414', '57331685649375887106.8414966930216399992717133', 48, 3);
    T('6864627090236162752810872915859970493.9829241988372752335410706150715919379020', '5249.6121433709713864', '6864627090236162752810872915859965244.370780827865888833541070615071591937902', 94, 5);
    T('729503574610589530614690975.483304382836143278160510761', '84185238672150957711983744.16', '645318336000000000000000000', 10, 2);
    T('179350186935399253206719338845650084685.54500958054469824', '721896283879779822035012.031119760231773009', '179350186935398500000000000000000000000', 16, 1);
    T('2915807253002510741773106236461229591477.80264998156176647', '324299.03271102517174110', '2915807253002510741773106236461229267178.76993895639002537', 83, 5);
    T('901949181566248568.0', '4767728642492439942472369732349.3798', '-4767728642491537993290803483781.3798', 55, 5);
    T('8706252530415216495458570274075.543221646314083312', '42454191008731706406146165894965172054.0', '-42454182302479175990929670436400000000', 31, 3);
    T('529051403648673242568456219.0756654855884973440032', '1367163638864.9000238136691411015932014299003722098', '529051403649000000000000000', 12, 0);
    T('162807291829550440289.38711480124822', '320911185633008130739621341704989655.390662858984203936179109883420', '-320911185633007967932329512154549366.0035480577359839361791098835', 64, 3);
    T('5665.325923352166844686', '6154428943435834334268419921516.659395070255703788329148031353606', '-6154428943435834334268419915851.333471718', 40, 4);
    T('3650453639842033477658892.395036385313179606531', '394075165594.15003667', '3650453639841639402493298.244999715313179606531', 58, 2);
    T('255324323348987140620527093436712.8955912195355148797317661671748', '38168128727771954799631461319139381600.573296674255624704', '-38167873403448605812490840792045944887.67770545472010982427', 58, 5);
    T('8167131952790749840293.261672478189964', '6609487388225995203291549.3613365682671210668215', '-6601320256273204453451256.1', 28, 0);
    T('12242198138863747618936577771912.845089568603396298465100801297715214737', '28.55405331320', '12242198138863747618936577771884.291036255403396298465100801297715214737', 100, 2);
    T('697069185854.0', '943799171.64128', '696125386682.35872', 28, 1);
    T('45530857.848986022591812190166', '960694141135188.232', '-960694095604330.383013977408187809834', 52, 2);
    T('245660.2434979739227842218680544203350', '308550641014816009956734735512.2406202729389962069', '-308550641014816009956734489851.997122299016211985031945579665', 87, 2);
    T('69407042023572214.67653078455111643892', '23072309385925591.0', '47000000000000000', 2, 0);
    T('58.6', '1896473820544336281826060.8023520248139891564317793955828871792', '-1896473820544300000000000', 14, 6);
    T('3013196935391102911400665916879124.68567781', '4814040031629088173761172182526213600723.502280291', '-4814037018432152782658260781860296721598.816602481', 100, 1);
    T('2348.57514498', '913434046834011.2367483345472690208767666047642', '-913434046800000', 10, 2);
    T('690385665783744626610664961.0165156', '267575579554144.01631829183264091211349583', '690385665783477051031110817.00019730816735908788650417', 88, 6);
    T('78.98110483282419720164318', '9974419040147122698866480613142.65916806775423604842863491188647685', '-9974419040147122698866480613063.67806323493', 43, 6);
    T('2920413.77767179', '35178632188132388201.7832865008496578090484904657206', '-35178632188129467788.0056147108496578090484904657206', 79, 6);
    T('8073659073002998155543685.240', '7797447916439698004678845130492434065.294019805351601447', '-7797447916431624345605842132336890380.05401980536', 48, 0);
    T('9404434350190761.3', '73168011.2132549992367574432973863030410500', '9404400000000000', 5, 5);
    T('157047.773668', '389270666699148923952471706914.6159646630', '-389270666699148923952471549866.842296663', 82, 0);
    T('463007771514102966780588317460323777084.2909', '3026584154421790140690803521.16079633084871737657094799056725461', '463007771511076382626166527319600000000', 31, 4);
    T('584125709683109585684510327996299361330.7772946303770724499082584556', '9576.534674', '584125709683109585684510327996299351754.2426206303770724499082584556', 91, 0);
    T('42969888373758210005063.639', '45986473458696904010088.1562274155317094552427709339', '-3016585084938694005100', 20, 3);
    T('3115812.687046', '3373363486592.739679', '-3373360370780.052633', 39, 0);
    T('8464.43896444', '64494458197295199.604285338553350095', '-64494458197286735.165320898553350095', 66, 3);
    T('1614.2562919109154458646402491929008', '376897399114290109614954185717819.451876042000600187551845', '-376897399114290109614954185716205.1955841310851543229115958070992', 99, 5);
    T('805290033673498249.7243644891956354743164953284491198331', '7066800.9346480783714171320539289124301', '805290033666431448.7897164108242183422625664160190198331', 72, 5);
    T('2980977930472288823040081946713.97131505642050722268786804386193', '94985058721430.454301533237573740', '2980977930472288728055023225283.517013523182934', 46, 0);
    T('52114937677789678.78', '87679510594835952381002172936123.949816027357215537234', '-87679510594835900266064495146445.169816027357215537234', 60, 4);
    T('111030222746423855723091.81023761570113', '6018748040550722331667.74735058608794', '105011474705873133391424.06288702961319', 89, 5);
    T('1210.467000', '3098327.8', '-3097117.333', 77, 5);
    T('11249701551138.48736', '2.533115507520882346510', '11249701551135.95424449247911765349', 49, 6);
    T('7429172.81', '54116286966446471123421978202.4652573831', '-54116286966446471123414549029.6552573831', 68, 2);
    T('237989.16', '4343828521340962815858540878.179112504961855523361249088263732', '-4343828521340962815858302889.019112504961855523361249088263732', 64, 3);
    T('8470992034990097344234026.713478', '294386615371691.915553967792129704', '8470992034695710728862334.797924032207870296', 83, 2);
    T('599278385495587513051863750.835953797', '406782990440101906296654271213300994.190820564710085303097361234705918119167', '-406800000000000000000000000000000000', 4, 6);
    T('38280482.573996597739736', '41.44979422579311393680292869', '38280441.12420237194662206319707131', 93, 6);
    T('338122155154462946159822074414955.47673667055088358727840117110726848', '7505626078564931668224405043055201552.16195895362998909778', '-7505287956409777205278245220980786596.68522228307910551050159882889273152', 78, 4);
    T('53492337707079.68', '2708573879074859930788363515.72301827861002477859445265', '-2708573879074806438450656436.04301827861002477859445265', 64, 3);
    T('9928687993052437747171645858956996126.59', '8481749294255385518426215660558246.20922663384380630911399469675', '9920206243758182361653219643296437880.38077336615619369088600530325', 72, 4);
    T('57848623901142518406785339284168.6368090593380782664753432284363041885', '349793935657772640677358532437814488409.173871647319106391880876500536', '-349793877809148739534840125652475204240.5370625879810281254055332720996958115', 88, 2);
    T('232759749299922487705662225331325.404103656675389333281031401609', '988119258180269608624236057907740234332.5701008039', '-988119025420521000000000000000000000000', 15, 0);
    T('8.7054093176424114521281732235', '1364012731233365549176899734871.85777842472098489630257531504825852', '-1364012731233365549176899734863.15236910707857344417440209154825852', 70, 6);
    T('964652880377.884724419142600217305431086670508124023', '93472233700285006656824770567.076013989758437', '-93472240000000000000000000000', 7, 0);
    T('158309174995371901405555327634497430702.747568189252106603310810946073', '8464077429087935294745250926260877.725032', '158300710917942813470260582383571169825.022536189252106603310810946073', 99, 3);
    T('9220315284885933187941318067269455.039982781427854458274197969', '4186485493083409014556895762.6706396056899289110414720027206', '9220311098400440104532303510373692.369343175738', 47, 2);
    T('859192996929919377894.3201867069817890364548798387421390437', '399073724913084949207899443123157.030661600048706952976297151751186185', '-399073724912225756210969523745262.71047489306691791652141731300904', 65, 1);
    T('9832955942326210.24', '329.6783772654269911996', '9832900000000000', 5, 3);
    T('8006557338249875391874258.416', '67187080284668859.58573225684585345571', '8006557271062795107205398.83026774315414654429', 67, 2);
    T('71844959454362269.896518622289244', '6968404745546372099532980118944560799.2423154145', '-7000000000000000000000000000000000000', 2, 0);
    T('749631717857637931804505.3644571282205', '310728.0', '749631717857637931493777.3644571282205', 70, 0);
    T('8995240888842705583787609.178272821725', '74.6206564989329406710994919624708802', '8995240888842705583787530', 24, 1);
    T('9281551395919690792320240261390360.2677442974735620930691959568958874', '195537.2527532052314355065971808', '9280000000000000000000000000000000', 3, 1);
    T('738869642454445514461026460118231.1082384508824598938102138', '33658216416764907565452659029330.558887596527992', '705211426037680606895573801088900.5493508543544678938102138', 73, 1);
    T('84881958.492275228969973757486893270033558', '887384027500582475411228.315282092951758068995891', '-887384027500582390529269.823006863981784311508997729966442', 57, 4);
    T('73759004853564500308551003629809.251260963181088016958070394634372732', '72496416601348361741917456405238516.49184691190286694354264244629620188', '-72422657596494797241608905401608707.240585948721778926584572051661829148', 74, 6);
    T('861877.13208234198679298366365848381', '22458.1', '839419.03208234198679298366365848381', 65, 2);
    T('762147786.1052647550', '94934.07773654166472443028133423603367', '762052852.02752821333527556971866576396633', 96, 1);
    T('22571929421156.52161450133143141998188553801400', '266787732903137281192871275582999255.2453570704690276420711450789528988427', '-266787732903137281192848703653578098.7237425691375962220892595409388988427', 86, 1);
    T('17383758359213848650.59689229484283775498', '101960302268778319257348787644.8', '-101960302251394561000000000000', 18, 3);
    T('5349564027175944678515548.694849334', '135476.1458520', '5349564027175944678380072.548997334', 96, 6);
    T('478498038967014170488119814015.16152934978983051140988158', '855493116051.81431672304497946757', '478498038967014169632626697963.34721262674485104383988158', 57, 6);
    T('470108291472930633691815.90979287986383558903303669947162365218', '82699294754520533.718576231868401258429', '470108208773635879171282.191216647995434330604', 46, 4);
    T('3560941825486375577232371.29987256580515611', '2.0964361194731735168018314', '3560900000000000000000000', 5, 4);
    T('500841276862.467681701', '92082.0', '500841184780.46768', 17, 1);
    T('2190460979231.9', '221.067845884238834960', '2190460979010.83215411576116504', 62, 2);
    T('48812934329098409.9209916097334817874393669890355', '73461.0', '48812940000000000', 7, 0);
    T('60109740365723243.843521548649', '74.483', '60109740365723169.360521548649', 64, 6);
    T('53599477183916640877.10426471095596682769081754', '2727295060904536293002794.8611', '-2727241461427352370000000', 18, 1);
    T('49520110189.9828915884205520431339817277970772940920', '11493.600707007997632348', '49520098696.382184580422919695133981727797077294092', 50, 1);
    T('813.428508678595300200', '88194191120712800444065080727030349.0541338821084165749660871664897848', '-88194191120712800444065080727029535.6256252035131163749660871664897848', 99, 2);
    T('16894999663959.067683659021061293', '977214303972.2403432', '15917785359986.827340459021061293', 38, 5);
    T('2176125024660813470294909469932295.09308785075650568104100108468157', '59433959555935492327256019013552.92243936324', '2116691065104877977967653450918742.170648487516505681041', 55, 1);
    T('966343519832796915036923.481989409329334602289078303003', '489697016396016913.3461486986664472614586696', '966343030135780519020010.135840710662887340830408703003', 86, 4);
    T('66609352595790875567351.75160442649412659652', '85002291316618460390791245374.7326835497241149400652416297956112', '-85002224707265864599915678022.9810791232299883435452416297956112', 79, 0);
    T('468546629051970506519566151676932155.872154973264602791039063', '98252046590.476502153329921964517435', '468546630000000000000000000000000000', 9, 2);
    T('37878211078231261202615598.0406950337704915641', '539150781670548.252327996543677094437697', '37878210000000000000000000', 7, 4);
    T('4122217327384055327972183.72093781272280610239265954073571740', '65067139432271749311680.340284287256052729', '4057150187951783578660503.3806535254667533733926595407357174', 78, 0);
    T('349369078389815.00861547916949007610557463714387547777', '30216767980882265245370.1233', '-30217000000000000000000', 5, 6);
    T('4133935195.77603412480', '8389594.21', '4125545601.5660341248', 69, 3);
    T('9915.1953052', '71763662297133575684.7128630271107347942996761380371825', '-71763662297133565769.5175578271107347942996761380371825', 79, 3);
    T('4615748652579148299823427620454452.949436956654140209595', '85700590843442119.7210363135172699', '4615748652579148214122836777012333.228400643136871', 49, 2);
    T('4490690802122198870573830.958255240716977115453365', '27378964006773.9753619049525765579894600058', '4490690802000000000000000', 11, 1);
    T('1138422109930805088797402671868059.5473526798661991166915029', '72246055930108418.7150888635960098004', '1138422109930805016551346741759640.8322638162701893162915029', 84, 3);
    T('50908175078580952275906813.96837', '7394889895431782485942017218250961407.8', '-7394889895380874310863436265975054593.83163', 44, 6);
    T('570261713493605028683814088944.0686132863178226', '946793973908859915185510060.3424457036797718860010307817205598043033', '569314919519696168768628578883', 30, 3);
    T('8348083742337450497857.6582663789985549442442', '84050271279147815894589.3154395658503006370067187', '-75702187536810365396731.6571731869', 33, 6);
    T('8767052748636853.251', '976.032215788490398416933', '8767052748635877.218784211509601583067', 70, 2);
    T('228612163412368.561775179774522981994348066080774255761', '942465702120.385', '227669697710248.2', 16, 6);
    T('7638294682.300439496466513413214781574', '36319484240790546948466392680458874.265562070838276258015254576623254615', '-36319484240790546948466385042164191.965122574371762844800473002623254615', 73, 4);
    T('70445132820478108775667110314175.2027709657981420082866212819946712', '95677754455093320933355067251402381.918797173193337', '-95607309322272842824579400141088206.7160262073951949917133787180053288', 89, 2);
    T('337660635804776126243599.73342419972976300101948948', '936886948729393400486818366390005878761.979862987631908647499994552691838658', '-936886948729393062826182561613879635162.2464387879021456464805050726918', 70, 5);
    T('3141384871520368025761196184640968973810.671712312046576818714971', '6545791522.41249668', '3141384871520368025761196184634423182288.259215632046576818714971', 71, 3);
    T('32070289012238141659916735.1484337113732464261500', '34919826526454652568954189115574.143521195341', '-34919794456165640300000000000000', 18, 2);
    T('859885.46541558934649565976806218773266405', '3405957228130698587846888960562562.0526363978227326562', '-3405957228130698587846888959702676.587220808476236996432', 56, 3);
    T('309406.0598549335210598417226', '127.0991135090974683078879338399843257224', '309278.96', 8, 6);
    T('941954231923283.053886469480845202', '89258324852.1999678', '941864973598430.8539186694808', 28, 4);
    T('59516790108530211135349748629611.22239289044411766523725', '74742585879326726903257984553.2436552331837025526367551031', '59442047522650884408446490645057.978737657260415', 47, 4);
    T('938997681559498529499618050395091048708.91194292754312403783056844558625572', '94986899602161058836098982406598545.05338707089016250807648852431602', '938902694659896368440781951412684450163.85855585665296152975408', 62, 6);
    T('10418346608842.82356868653780642117184002858442906676', '5995765896949631963158591744033.5307627612670', '-5995765896949631952740245135190.70719407472919357882815997141557093324', 83, 1);
    T('530973559501535348399574.34246487', '0.66913537', '530973559501535348399573.6733295', 42, 6);
    T('506136774075357291730784469862522519471.0072161081975771365', '4057333029493371861535958945301.10117624479772852', '506136770018024262237412608326563574169.9060398633998486165', 93, 0);
    T('74166036407850163820392617656978683666.50293318240860093', '77095607535586830108915527946.5691905205590378090226229458470509844', '74166036330754556284805787600000000000', 27, 0);
    T('9568453480475726192179972274190625.054907756', '8938952.0840687289452059541828', '9568453480475726190000000000000000', 18, 1);
    T('40948455.966996949', '2150448107984784458563693150895.06805885381730203048812542654949', '-2150448107984784458563652202439.10106190481730203048812542655', 61, 0);
    T('91159655685766848386.1783590528743994311', '365.8459177073263539897143976183724968305489', '91159655685766848020.3324413455480454413856023816275031694511', 99, 6);
    T('720962098691615192528.7388', '795912280846172083.51163943108936176', '720166186410769020445.22716056891063824', 99, 6);
    T('470825734752.9546162525043832', '874358178.588640810907307194704180971', '469951376574.365975441597076005295819029', 94, 3);
    T('73914693249.399385431036537180134658222602720117', '11738496073475524360452225403.0761350823250069814730673359', '-11738496073475524286537532153.6767496513', 39, 5);
    T('415331639475753542796073264692906.272091699518', '323827555602.4016903250923970', '415331639475753542795749437100000', 28, 6);
    T('1.57700354570', '33932994606133586103181457824.8245933401817516162913084562476448962', '-33932994606133586000000000000', 17, 5);
    T('35711921795893210.1969770082048551931', '2878061.3375179821965321663396409426315183714615', '35711921793015148.8594590260084', 30, 0);
    T('6667372.7224437497369544593253892689106122', '580.28206829995842730360521931182', '6666792.4403754497785271557201699570906122', 79, 5);
    T('930598239251271753109621959807.1069', '45622651791429872596061.3854642080090120971793760323396467', '930598193628619961679749363745.7214357919909879028206239676604', 61, 0);
    T('121732522.77395417782011840024465873', '4390803557233877171.1', '-4390803557112144648.32604582217988159975534127', 87, 4);
    T('67942912881285622118351738.4629764473330', '4648416159464689295576226626497.5011', '-4648348216551808009954108274759.038123552667', 81, 0);
    T('4247680593109665362401169336874077677786.776920226372886', '3.69488514546780938036644056904', '4247680593109665362401169336874077677783.0820350809', 50, 1);
    T('296252223047477325015.2717569790755588412', '7399292769253966179567464469037004616890.22571359976607902737674220087163', '-7399292769254000000000000000000000000000', 14, 0);
    T('204265584401778679930114418986678217357.18135672270261765016596173597861', '812107241823455023285224379506712.0381499418371520824757991159945489814', '204264772294536856475091133762000000000', 30, 5);
    T('26467226099099075.227330037', '744.67', '26467226099100000', 12, 2);
    T('943058798364.395479068079592238958593748', '4262.860991837775288450967406945', '943058794101.534487230304303787991186803', 40, 5);
    T('3961880712.1739466698112561773101546317748', '768274235.957926640149693329', '3193606476.216020029662', 22, 4);
    T('705240179905.01961035839697848937836', '5.730085107325974', '705240179899.28952525107100448937836', 35, 4);
    T('4.677135032771256809341543004', '368037386167.5782655174332095276', '-368037386162.901130484661952718258456996', 94, 3);
    T('1634482637211224074.5176185357539524070468846', '51959012150.180327166277012544968432200', '1634482585252211924.3372913694769398620784524', 71, 4);
    T('7062679212073143326888924001163.02786', '670367382193775796879309.97397127870279732280890295', '7062678541705761133000000000000', 19, 1);
    T('4159462381849975492409125.0', '659.758349477474619798183023482274815728', '4160000000000000000000000', 3, 2);
    T('386.361433432357526852389133', '161581451832534896068758.824566698062465908442798606434258', '-161581451832534896068372.4631333', 31, 0);
    T('2989170915162203782.110570', '78042238635700889.19875125602523287111', '2911128676526502892.91181874397476712889', 96, 0);
    T('24459694394745404014469850905.84318799775716933', '206361207751899.635824898631920532', '24459694394745197653262099006.207363099125248798', 97, 0);
    T('961453869.879856', '6205777354.0303077454349819728607', '-5244323484.1504517454349819728607', 50, 3);
    T('224978326.9', '46099.92658151985887055028304994', '224932226.97342', 14, 2);
    T('4795500.09497120551913045', '43866652015.225143790341972440190589912', '-43861856515.13017258482284199', 29, 6);
    T('785759919848321456608.53471973157141969206251316', '50515325074711677303149183246092378.4', '-50515325074710892000000000000000000', 17, 6);
    T('1052.813283166117082', '573714957335293801886430730.75448811348282631726775089262354663', '-573714957335293801886429677.94120494736574431726775089262354663', 70, 4);
    T('16271428.62516560', '32898412714198161463717.4889152072', '-32898412714198200000000', 15, 3);
    T('900560899.913104266482415717003', '5952428176343381136666681609845550966809.5', '-5952428176343381136666681609844650405909.586895733517584282997', 79, 0);
    T('96646163996160433101650646872419298.8222', '589774359042273020.4302680978027718659076738429520984', '96646163996160432511876287840000000', 28, 0);
    T('7560498588.138', '7703300420181340100559937071730.98176915487027', '-7703300420181340100552376573142.8437691548703', 44, 0);
    T('8522365103427764558030271.51366810225', '88935863758311187415.13400', '8522276167564006246842856.37966810225', 45, 2);
    T('269622299212099.5658', '67080856574410241738.2', '-67080586952111029638.6342', 43, 1);
    T('178101542842138132551032196400994301.93920152593345597972723530172437819734', '9928207710724533478360637492329686410.096810135482', '-9750000000000000000000000000000000000', 4, 4);
    T('347539.651654349', '999164349246243097554653376.101753100580453', '-999164349246243000000000000', 16, 1);
    T('67.82933602918033391813', '80456255662446245353590549764939356874.08639717592445030759922844', '-80456255662446245353590549764939356806.257061146745', 50, 3);
    T('1286189309439198325290138412924.71091621710744938298299045634143848', '55925281201729145416213246160396310313.033984382012126643460839793', '-55925279915539835977014920870257897388.32306816490467726047784933665856152', 92, 0);
    T('409218380262925.03329205577858741776', '98299234078747591.9319529133359131207968770243284522', '-97890015698484666.8986608575573257030368770243284522', 65, 3);
    T('589630075341582712237298018007.626791554398075374457001806', '37072203775488.5460924874', '589630075341582675165094242519.080699066998075374457001806', 80, 1);
    T('952406921388034785124870359466.4208', '92386404607924941831558.493321724284807184287113592', '952406829001630177199928527907.927478275715192815712886408', 65, 6);
    T('239797538.62924219574590', '62.7179', '239797475.9113421957459', 29, 1);
    T('9117573077.38203829394236794264681559', '9485399161355567.024972352', '-9485390040000000', 9, 6);
    T('195854152327661808049809012.152492452144772559', '52666453496377990502.8656918042014982268', '195854099661208311671818509.2868006479432743322', 61, 6);
    T('68389708692892364357038.550642618', '346589125.87181', '68389708692892017767912.678832618', 74, 1);
    T('22366800802617651725451014763352512558.88384367', '56706199621325498.77362822019', '22366800802617651725394308563731187060.11021544981', 77, 1);
    T('222099131902528.823008', '23424944387813665353358.766033498779988076936', '-23424944165700000000000', 12, 4);
    T('94369.805612979192617825311244420237761233', '1404995131175289820.12435079015027778619', '-1404995131175195450.318737810957659960878755579762238767', 77, 2);
    T('51595640870900081.31194767388', '7760147060689.98166496533407685834688993271909084', '51590000000000000', 4, 6);
    T('9773886938068800575328768856352095468.6307897839608910717952963742165591', '444564767846600894514602578978208.76080300483076311650', '9773442373300953974434254253773117259.8699867791301279552952963742165591', 73, 5);
    T('999567934641977983949863368480.115289355375579041', '1165149936.615876680281694880228', '999567934641977983948698218543.499412675093884160772', 58, 3);
    T('5501088.6084944677846990708389945743093', '664927.656419331398418627625297930574', '4836160.9520751363862804432136966437353', 53, 1);
    T('39.48037672881845226078909407', '5341174765458518115561880076247.366376615978250578161608182949283345', '-5341174765458518115561880076207.885999887159798317372514112949283345', 85, 6);
    T('8117017575502275847692679.264176875515281734668365581147676', '8659713925984172141543085.308002265245744274949016821247', '-542696350481896293850406.043825389730462540280651240099324', 58, 3);
    T('553909918921981594.751577774245505483484884974672361660', '8054222209080795584363855.323441', '-8054221655170876662382260', 24, 2);
    T('962163351664.0', '332173.51965410', '962163019490.4803459', 51, 4);
    T('78570476306710514328422903.566399424510435349164985564', '20645082983476005744595606474867.7059235785374', '-20645004412999699034081278051964.139524154026964650835014436', 71, 3);
    T('666631867650769350383766311.24823253716', '232735963045186976112963962.24048736757366948', '433895904605582374270802349.00774516958633052', 81, 3);
    T('759785915873173118941850738960455977789.03259903086677271090681583525083391', '6239262109019238233981735614.87249900850446524387856346136836303', '759790000000000000000000000000000000000', 5, 5);
    T('47417002283487491793133.162876598099522479641', '5798942052554.5198526293050', '47417002277688549740578.64', 25, 4);
    T('6626460939175437003981392048059079.7245933323187090582585544', '909561620735130820650.3713179', '6626460939174527442360656917238429.3532754323187090582585544', 60, 3);
    T('65718846993876485099499095393267604350.5432361999471902331', '9133254.0', '65718847000000000000000000000000000000', 10, 2);
    T('9831060534.4113885', '40.264960990851888058', '9831060494.146427509148111942', 97, 5);
    T('618365246.16154228299443804064726416551494052021', '4175245358857853690274.6574698427569', '-4200000000000000000000', 2, 3);
    T('5794058568457386772009082293111862955.1723471', '7874.10120139', '5794058568457386772009082293111855081.07114571', 58, 3);
    T('115514655034192092805976.359058975764477', '7321838425189291796508643.82134639311753', '-7206323770155099703703000', 22, 0);
    T('4979836.458', '4343643405.293', '-4338663568.835', 93, 4);
    T('6962719933111298474232811011939872095.31542793502134199201685731559857711019', '45898320993079771121163323807243350900.791045197014396089', '-38935601059968472646930512795303478805.475617261993', 50, 2);
    T('96.45114046', '43788964069986958954302793689691871.503332629337662292231604855967058576', '-43788964069986958954302793689691775.052192169337662292231604855967058576', 83, 0);
    T('19.706629067026369851443323141433225326543', '9130912170755702578573915.749688', '-9130912170755702578573896.04305893297', 36, 2);
    T('999909783662285281933157327721667.775471879628', '2795909691798312648666149730968954.548860473478014471571999555369104559', '-1795999908136027366732992403247280', 33, 1);
    T('9055989.3253617695654064149036971384419806539658', '16700012813767870416936045681260.961743541', '-16700012813767870416936036625271.6363817714345935850963028615580193460342', 92, 2);
    T('3761160947240340619655699804028514166.1545031706631', '90008594170211932230610878657643955.0766771120853201398452422803203', '3671000000000000000000000000000000000', 4, 1);
    T('7052186163764582.7718768047025626908949225631756923472405', '279825326258.044979268268396', '7051906338438324.7268975364341666908949225631756923472405', 85, 6);
    T('55148738462073723350464458331198071161.324235957257244236', '890533822519200952605943.67027149110407422724883', '55148738462072832816641939130245465210', 37, 1);
    T('22048566656678706770922535389009.8800482', '691835258073242811318135316422877.78205857134', '-669786691416564104547212781033867.90201037134', 85, 5);
    T('360228259046240.247611499704462968697207149395', '2086914304371111332610306394896990.291351096738500858888680941788348731605', '-2086914304371110000000000000000000', 15, 4);
    T('237765665865861098214622.6280166600682705261137847840237788088', '49282802218452.8192989767794767678363', '237765665816578295996169.8087176832887937582774847840237788088', 73, 2);
    T('94963576434071265035613300794191.69660087123956630049887827698659406', '991367147561458404473577.06', '94963575442704117474160000000000', 22, 0);
    T('21039.0941271849563698186545060837025468', '61.986522669935', '20000', 1, 3);
    T('5527758.208579037564171467506', '57746543998576.5', '-57746538470818.291420962435828532494', 57, 1);
    T('53.526728183052625', '4566777936616620019069108049674.2001694968892442806307029510968199973907', '-4566777936616620019069108049620.6734413138366192806307029510968199973907', 81, 4);
    T('8398.06881519554924106264640920848', '78419713421175411471008565354902971834.199590637228446719811837169766362', '-78419713421175411471008500000000000000', 24, 2);
    T('46651569150015595210809704661940.8', '96834650155790.328525', '46651569150010000000000000000000', 13, 3);
    T('203662156936266018.56857105801974209381240694301095451', '77270.69753652195683217690846412859892084', '203662156900000000', 10, 3);
    T('69434.88031535436784733509731541971007', '8.5519048231739842309', '69426.3284105311938631', 21, 4);
    T('695265208240345480799.465208690311427717020174', '963908834.1637317007838017919941336969673787756325', '695265208239381571965.3014769895276259250260403030326212243675', 70, 5);
    T('875031885506622202313972150.7266913315589066826744', '581696758787378729955423404234905245415.0607657848516406595', '-581696758786503698069916782100000000000', 28, 0);
    T('3850085716409132708226314757323424740259.29106821935', '318215482803.0830194832761343614525696063', '3850085716409132708226314757005209257456.208048736074', 52, 4);
    T('2077002389327078369687189002.91', '239837588404470046844798445464872593503.053633023008384327820145755683896617', '-239837588402393044455471367095185404500.143633023008384327820145755683896617', 95, 2);
    T('525304094036133422209603266609297.83269311554678194352008621', '3352818947475215852524.43621', '525304094032780603262128100000000', 25, 5);
    T('2481703604661998598312313771116.683575522902', '121527168476013300130172028650996952931.129337247013280289581101702083117322', '-121527165994309695468173430338683181814.4457617241', 49, 5);
    T('729755648.217572562103468478434322387136565', '843447200457297.639006250839608805585207851529624505', '-843446470710000', 11, 3);
    T('387838224.031176138', '502511403087739175653052910412155182.44006582048399018208099', '-502511403087739175653052910024316958.408889682483990182081', 58, 6);
    T('69132135722.58295408468070686447778925874', '1453815723479122700218782754.639093106402300013304188564319839138', '-1453815723479122631086647032.05613902172159314', 45, 2);
    T('456578903803098400.0', '1905158313568197958999531007564638.80281445760356043208682451', '-1905158313568197502420627000000000', 25, 5);
    T('838363997375241408512205.91331625707234', '1817185772520982149742322396855847.0774430711345644250', '-1817185771682618152367080988343641.164126814062224425', 60, 0);
    T('7238789566926067985363334642130.599058758518787968', '6343845415.777256414176861661', '7238789566926067985356990796714.821802344341926307', 51, 3);
    T('9035266192537122485859011581230277071.66843338494296468220058', '360821803539883788027990763992.0', '9035265831000000000000000000000000000', 10, 3);
    T('3516070859853058886674366.675', '777750746025759323785264.6302948541581587174', '2738320113827299562889102.04', 27, 6);
    T('234667843801502022403492664022.51274243509522937067257601080027353748', '7434662831014255662256399.51019323374327636911', '234660409138671008147830407623.00254920135195300156257601080027353748', 96, 3);
    T('801.228464206716', '79633403862832778.892817935631812561652', '-79633403862831977.664353728915812561652', 57, 4);
    T('148761420566882053922085.5584439797471528210930019899', '1048230527982649187568473020741814535186.994351', '-1048230527982000000000000000000000000000', 13, 2);
    T('320094453723.90491004868317673103779600426809241', '6105321.02334476276', '320088348402.89', 14, 2);
    T('4470885354629185573049940913387317010.5199552238351', '584051218937333672.77333925100575115', '4470885354629185572465889694449983337.74661597282934885', 88, 2);
    T('213143.8021123372696220632021010884971460171', '9503836216.62157633547688479807890', '-9503623072.8194639982072627348767989115028539829', 87, 5);

    Decimal.toExpNeg = Decimal.toExpPos = 0;

    T('1.792316612884487564311771281e-23932057', '5.69976456866047320412517423508310354728679470757328490000343592950016294088615989268093866e-32517', '-5.6997645686604732041251742351e-32517', 29, 5);
    T('1.99169479851016450011043646901687400115512860504018833941164718920781076854216762430238626408343644e-9931933', '6.38526426739e+8626025', '-6.38526426739e+8626025', 22, 4);
    T('-6.2542519003586694651026234e+87', '-6.5757404951359531828365479313153601424485747649226579274168996862092441994766035e-49849', '-6.2542519003586694651026234e+87', 53, 3);
    T('-9.604772015363615559582304998733675e+5850', '8.1340843005908425924790547473450988769558068511930082393e+138332923', '-8.1340843005908425924790547473450988769558068511930082393e+138332923', 96, 2);
    T('-4.228107205843224077015863084700000947581407920789023984663576639e+72', '-2.3646076187718393062198539736528060703203e-18947270', '-4.22810720584322407701586308470000094758140792078902398466357663899999e+72', 69, 2);
    T('3.42164002916939306745661852229207679178670363e-739642191', '48413603180304870097616580848566368', '-4.8413603180304870097616580848566368e+34', 43, 5);
    T('-2.7082067257439001248731034e+255', '1.463357067849197668312169937835142233343040728649218403546975321163316191984829045020133415427186882e-813', '-2.708207e+255', 7, 5);
    T('1.80544723819507780400985933621596829e-23912898', '-6.8902351056372730055504911816243114779696717903217778350020023536206268396056597e+8117', '6.8902351056372730055504911816243114779696717903218e+8117', 50, 0);
    T('-1.688322353309930289764085302393645579e-2061', '8.304874790464942866574379155442852235973389929444676317853978559698488e+320', '-8.304874790464942866574379155442852235973389929444676317853978559698488e+320', 80, 6);
    T('5.323551831268452061208291787260167129e+3404403', '1.731761449136485742878138678929997425806089831701987933870812710697622052696577114307158491381462e+545', '5.32355183126845206e+3404403', 18, 5);
    T('-478232737.497', '4.0462091688110303902787237905655428288294712510868875182578760417628600744075185622547055498e+3079062', '-4.04620916881103039027872379056554282882947125e+3079062', 45, 6);
    T('3.0612417422416523917303965310500924897578e+6491', '7.827139363104333343854465329937791735830391528894758013298610462720743920', '3.0612417422416523917303965310500924897578e+6491', 54, 2);
    T('-5.3813883054823380e+324', '-1.2369677988e-57833611', '-5.381388305482338e+324', 75, 5);
    T('9.21805739519561831297761022643364049172592e-918', '-48895883124010996460744080319445610017323327196869255668.4572', '4.88958831240109964607440803194456100173233271968692556684572e+55', 66, 6);
    T('-9.252715780068260418043e+72', '-12590139569136468029213593074424819654932792387400646295775156790571242103916622815884889.5892', '1.25901395691364587764978130061644016119e+88', 39, 6);
    T('-5.71550608647635849585421199679478092798616968053825229082869453927359310584963054663809359831818000e+75134426', '-1e-353086', '-5.7155060864763584958542119968e+75134426', 30, 0);
    T('-1.82078636582034050e-58370', '6.949920119724528258522072178e-45427578', '-1.8207863658203405e-58370', 71, 6);
    T('-2.795864396006197633901037932062163941946590578075449507267908969758559434326789e+742563919', '-3.486025739887413584063563381470972389366187865215964884178573941676391907e-39238980', '-2.79586439600619763390103793206216394195e+742563919', 39, 5);
    T('9.481503849304826928973950761062868210560506712428214283679344428199e+986553613', '-6.44018958804822470871191968385562129218994153583004246052413787928461266956e-183768', '9.4815038493048269289739507610628682105605067124282142836793e+986553613', 59, 5);
    T('8.42058906940015718798e+23', '-5.12818647090105520279002851228627516477188332698656845964525170861373277e+77', '5.12818647091e+77', 12, 0);
    T('-6.66867298424442973991017135004123e-6016', '1.122942426365938279781316944e-50646915', '-6.66867298424442973991017135004123e-6016', 89, 2);
    T('-14434824877681671252494.50', '-4744180884604619155005023739354631157756385352850463825687419180.317893158', '4.744180884604619155005023739354631157756370918025586144016e+63', 58, 1);
    T('3.021351757092074226662594776398163721367430180658722006837511458091887e+754145', '-7.7281874393523698296251147e-342', '3.021351757092074226662594776398163721367430180658722006837511e+754145', 61, 3);
    T('-9.222486718751634618635052066622543e+36', '3.38442051790053e+18', '-9.22248672e+36', 9, 3);
    T('7.09899984575865359734e+84', '7.78137675765375240317008124083145599412855878668147615343173099333e-778', '7.09899984575865359734e+84', 36, 0);
    T('-7.731336555060253e+61', '-3.797617486236881717819332e+113', '3.79761748624e+113', 12, 6);
    T('-3203091812097591345956128', '1.54e+804483', '-1.540000000000001e+804483', 16, 3);
    T('-6.8790104211130242454991601642749891042049402313899949070009698688834394612168393565e-552', '-6.47132322792558510008703e+977', '6.4713232279255851000870299999999999999999999999999999999999999999999e+977', 68, 3);
    T('5.11288463e+804121757', '-3.233654093376025263229529432009804011579062041719e-323', '5.1128846300000000000000000000000000000000000000000000000000000000000000000000000000000000001e+804121757', 92, 0);
    T('2215721.822336', '1450572422564612919038700170868560596636485142539113457.847312756478875123323372035942316', '-1.450572422564612919038700170868561e+54', 34, 0);
    T('3.856021532534216364576565205962498e-680406380', '4.0748021e+910929', '-4.0748020999999999999999999999999999999999999999999999999999999999999999999999999999e+910929', 83, 2);
    T('587477294349.9128', '5.9009418178397792919787923374434637609777539615615913481542114343091e-601914', '5.874772943499128e+11', 53, 0);
    T('63490679413147320151422267902664.025528745', '-1.1511e+1066189', '1.1511e+1066189', 57, 4);
    T('4.339186830776212855076142e+864782108', '6.8227815138786859587759979265847699809e-704', '4.339186830776212855076141999999999999e+864782108', 37, 3);
    T('-42.683', '-2.1402464208082777156219367044346794646671634131699440029334548496490648e+95393976', '2.140246420808278e+95393976', 16, 4);
    T('-5.86269349525893423867699327633641571932e+53447445', '4.4471801597664471649566620984480e+70072792', '-4.447180159766447164956662098448e+70072792', 95, 6);
    T('2.01368392850485088e-7530', '7.80493785717377845822211894268763294620830205670933641096287239110624600304801026405564172615080e+76373', '-7.8049378571737784582221189426876329462083020567093364109628723911062460030480102640556e+76373', 86, 2);
    T('-9.443635658940048035845e-138325', '-1.35559639536973582525930597369597363666745264482626811e-35473', '1.355596395369735825259305973695973636667452644826268e-35473', 52, 5);
    T('1.8616211493256324295274776609561325391953686920009343298139595509785385e-3501', '1.03012623565635246794472661292428160395721926954319341162059243941806e+69', '-1.0301262356563524679447266129242816039572192695431934116205924394e+69', 65, 4);
    T('4.511087857625328082657257239895665113144003584405761127577260517484874613e+778771121', '1.8325842443473932072e-59442002', '4.511087857625328082657257239895665113144003584405761127577260517484874612999999999999999e+778771121', 88, 1);
    T('-1.287219294194087073000916324326305e+96', '7.1699261108103554603864614680e-7547516', '-1.28721929419408707300091632432630500001e+96', 39, 0);
    T('5.0461889643211e-8150', '-5.3418458279074099377871e+49476', '5.3418458279074099377871e+49476', 56, 4);
    T('-6.71547813417393495728240002804006106537518734009661124726571518977786981532e+3318', '-7.2398883759082488170424e+75376', '7.2398883759082488170424e+75376', 56, 5);
    T('1.44735793243277391262764608662589226432402931e+41712893', '-340885521769586478087899909642349839174961017948388220545761927469.281', '1e+41712893', 1, 6);
    T('6.561512285692418871312354839741304442546111797417416511e+94599649', '-440795478877.624', '6.561512285692418871312354839741304442546111797417416511e+94599649', 66, 6);
    T('-688837253042614877399.5', '-3.632441039346975825180266180783847897335359529888336884091855e+188507', '3.6324410393469758251802661807838478973353595298883368840918549999999999999999999999e+188507', 83, 1);
    T('1.0360675995358536872163314493066410401378536425375038250374718871686217599980919432e+553425', '8.704062929984294713140931351834846849951498e-593673477', '1.0360675995358536872163314493066410401378536425375038250374718871686217599980919431e+553425', 83, 3);
    T('7.43404688761493575549516036492094029880287e-26712037', '-7.9066293190521765772848055594328963843287417065639566213443958915377769052389889e+928963854', '7.90662931905217657728480555943289638432874170656395662134439e+928963854', 60, 1);
    T('1686783212633289750026735931738732785983663746521931468526088783025141583828124.465', '8768590549.7241944309880627200749822214571362095236108237501203844509903955330291000151422', '1.686783212633289750026735931739e+78', 31, 4);
    T('-1.30273886137268e+5289457', '-1.2899382417e+647330', '-1.30274e+5289457', 6, 4);
    T('42050878099274497028907191938263035774766181789514211433098485981439.75', '1.43994506473742424499458038815027395361046188494352856430241622193144915292115196755951196e-5923', '4.2050878099274497028907191938e+67', 29, 5);
    T('-4.320626740890915234398134986114517e+94658', '2.78894990560963181013521759e+8249', '-4.32062674089091523439813498e+94658', 27, 1);
    T('4.01525027773499792223184009769672475650267777439e+2993', '-9.7541734788872345649674286932141725163703554472540789341298781328709290320275810e-6547602', '4.01525027773499792223184009769672475650267777439e+2993', 67, 4);
    T('-3.95785198444775477048e-46', '-6.870396030942305e-7108', '-3.957851984447754770479999999999999999999999999999999999999999999999999999e-46', 73, 1);
    T('-9711.6203836557758490544267', '9.188784153327143382542570876093e-23501', '-9.7116203836557758490544267e+3', 29, 4);
    T('7.355460006727234208680888285222108269270481742254887642198770398692713606694685939591959411955545e-487104390', '-32815863771944.87', '3.281586377194487e+13', 24, 5);
    T('-2.6583825751466329584377255122872320456516043330212891532766393709048631359035760266577469045556542e-99686492', '1.3196779556e-3010', '-1.319677955600000000000000000000000000000000000001e-3010', 49, 3);
    T('-7.17914921169565067656826325978347627539814084292339438679298679106912228033991102060926370758854e-7922150', '7.149725093137932026784009177225420e+497084531', '-7.14972509313793202678400917722542e+497084531', 65, 1);
    T('6.8401837269807432719138151869469924006494010653417334710e+59', '-3.3013338878941172505094851600658908071364455751e+267', '3.30133388789411725050948516006589080713644557510000000000000000000000001e+267', 72, 0);
    T('-4.54866584280149572085681473646934865513679804760160868303937609538161216745e+50712', '6008765.20498946344036890775138116504265390042946401212817895579352563718709273657', '-4.5486658e+50712', 8, 5);
    T('-2.17306915251811963448649934897203802911487529061830244254311067431220811484827762498e-8070467', '9.76024516020697735879087332971502859486313033396978241761064994198378347207016488940e-15', '-9.76024516020697735879087332971502859486313033396978241761064994198378347207016489e-15', 81, 6);
    T('1.07099326000068746588224496478208445416181740811806641950018390004007596301502960407795142041621e-48973759', '-6.86643594114206914494853806604921827520006935681793235523223528319863051850507291287e-619', '6.8664359411420691449485380660492182752000693e-619', 44, 3);
    T('8.09229557883260745215088204147149523134000618264060088333656193879542292e-9131', '6.8621966724589426049058030194215748202384645065147845415643636106228614429216889627e+5710', '-6.862196672458942604905803019421574820238464506514784541564363610623e+5710', 67, 6);
    T('9.581451793682701427886005083345394028175040878149375394273513e-378782', '-3.042744613335733864056288278187491527807214836284653551560072721302128870257120169275325e+92742406', '3.04274461333573386405628827818749152780721483628e+92742406', 48, 4);
    T('5.7313e+4575578', '-6.364840640e+39426', '5.7313e+4575578', 73, 6);
    T('8.356458499777e-30', '-4.8e-10939393', '8.356458499777e-30', 40, 1);
    T('6.5195472944708148433514527800921108648346637999108500193846733263849685289748056063562486569260648e-3279', '1.937421712038539810513850428488208588363123168835091110018815864672320921451430e+67547265', '-1.9374217120385398105138504284882085883631231688350911100188158647e+67547265', 65, 5);
    T('-3.57599586681284801380522107189891356706964779329156399436948121466556878573027334286577e+33356', '-6.6883693946947144556866042137800276517141450815792257780747268780004604e+17956', '-3.57599586681284801380522107189891356707e+33356', 39, 4);
    T('-9.7224e-560399300', '2.6205204246262938260068273679375013815833069770430762996326779771138775544825087639482895011e-7856317', '-2.62052042463e-7856317', 12, 0);
    T('-3.715681211128611433143970130865055335882020223143276e+52288355', '287789445446044936290241879917633998688', '-3.715681211128611433143970130865056e+52288355', 34, 0);
    T('-8.2493944690319859817818378061643226370038799191809830064274762378667243798969812972072673e+113', '-1.845641201032288327245060081214715637229922954057240e+444094', '1.8456412010322883272450601e+444094', 26, 5);
    T('-5.89309192204153001500843135046e+69621244', '5.23486399668082298543743497700744461634328561270113191e+51146867', '-5.89309192204153001500843135046e+69621244', 85, 6);
    T('-9.332533364617829013735842749683e-1172', '5.1886033191439624397804333255765500761063229061403068373806210152358593405375e+93591', '-5.1886033191439624397804333255765500761063229061403068373806210152358593405375000000000000000001e+93591', 95, 0);
    T('7.5445762070e+288854366', '3.8063974317472361901e+3723940', '7.544576207e+288854366', 26, 0);
    T('-1.10202097949692682919643989760625683657125288006838604142487011903806380534e-57', '-7.686349839017994404992480435117213704809681451389739987668242695205627997632727166e-756892681', '-1e-57', 1, 2);
    T('-2.6397824546988e+8167', '2.24335709768620433408676393186381185893686419138146661581115354856133911663897900777674634334e-667005', '-2.6397824546988e+8167', 96, 2);
    T('8.95701118636167267321626e+31', '3.866403036598893507073274582036553740764934411841223762724e+692437', '-3.866403036598893507073274582036553741e+692437', 37, 3);
    T('-5.7548994542146425852173016343954785000442443441023113369093428041115489e+5485526', '2.377230710793e+240213000', '-2.3772307107930000000000000000000000000000000000000000000000000000000000000000000000000001e+240213000', 89, 3);
    T('6.82187852561709017305422060576561581323016263253171768648527e+5000', '7.04611109462407074965728925055745664e-47', '6.82187852561709017305422060576561581323016263253171768648527e+5000', 86, 4);
    T('-7e+346', '-5.9739701543133697799688640180015825274829505377320867981068366951396904e+48689101', '5.97397015431337e+48689101', 16, 4);
    T('98575514219130865770354786583585231780940810430604135675363131502461134646460224095552185218.067', '8.6917047455394e-6901', '9.8575514219130865770354786583585231780940810430604135675363132e+91', 62, 2);
    T('-307326217766153207641296337408407178240069815', '-2.08928068512362083349037857638914402632806700732817216924871595627504976e+79', '2.0892806851236208334903785763891439955954452307128514051190822154343319359930185e+79', 92, 4);
    T('-4.555772492857631171176698810e+267939772', '-6.5528127257655266409752878244003501007476069420122758335606274879164568165395899170518329221501437e+1605318', '-4.55577249285763117117669881e+267939772', 32, 4);
    T('5.991948215377e-6123814', '6.38169244547e-151999802', '5.991948e-6123814', 7, 1);
    T('-5.21392008878352879914684467839557603e-6383', '7.9128406143023651297161503149801950528574678665327896050715374145940581e+68652', '-7.91284e+68652', 7, 1);
    T('9.161508948580791329450886540869770096703513327961216315071048543864660403509816151467893300e+47184', '-875581155292196024043167430367.03', '9.161508948580791e+47184', 16, 4);
    T('-4.7064408883914958217230526613884166e-5322567', '4.351015580014501018829827548653770705101061619352333235878130703968596958820676e-219807', '-4.3510155800145010188298275486537707051010616194e-219807', 47, 0);
    T('-5.86864922318646549356624038e-6388', '5.15864e-74', '-5.15864e-74', 67, 6);
    T('6.85870e-682', '-5.937075111928240207419060418231137e+5622', '5.937075111928240207419060418231137e+5622', 60, 5);
    T('-6.2846594717691810204240002443373127012520917569388241491889161091457255e+405', '8.0153128400211520487097219622809800495350464749634911270946865114372247349824051499281e-403780935', '-6.284659471769181020424000244337312701252e+405', 41, 1);
    T('-2.51033165877791728750241775443190891984391066633109045373979918847785114646268e+11923016', '4.55723346737173346604225e+453358630', '-4.5572335e+453358630', 8, 5);
    T('-6.4628942182810825075881700012135312694160551114463952887191657863131398435523742532514461e-6893', '-2.02225243286523769309415121074539512381866987210186008388367868075931147507424746610012554354e+933', '2.0222524328652376930941512107453951238186698721018600838836786807593e+933', 68, 3);
    T('-3.962623644340542242418789633415625567324136758413e-1340635', '-8.525681521139144430830605987294226705653004742663e+143', '8.5256815211391e+143', 14, 6);
    T('5.04188783533720818821892800831885659358875702648575435572892191640197223326922385205881e+94', '-4.044799176079958739979061829854269217248659863992017958880957143157173214518526499880140332437e-114', '5.04188783533720818821892800831885659358875702648575435572892191640197223326922385205881e+94', 93, 1);
    T('1.907022118659156494293209737808617967820791932710407270985579455491981584539790796605649238854197520e-848365946', '1011010673605111764248575166293951178735325592521363033821328588157771619.377853', '-1.0110106736051118e+72', 17, 5);
    T('-2.5823111978958804549209956134391354686605e-39', '-2.3756e+28994781', '2.3756e+28994781', 70, 4);
    T('7.3611e-4667', '-8.2885529813162426964849138072454671342893024340594245978772222282845e-4350', '8.288552981316242696484914e-4350', 25, 6);
    T('2.81803362355775841871831055053034976376902815619094990590e-5055230', '-4.63574008674183265911935834304869991074733393973186872200729701292356572e+896792', '4.6357400867418326591193583430486999107473339397318687220072971e+896792', 62, 0);
    T('7.9e-554925', '-9.560683183921193703634759160036460577238395402766596695837039927663889e+632788428', '9.56068318392119370363475917e+632788428', 27, 0);
    T('-3.32941405604212020e-471060', '5.8742682933034054748993473533381157135243211041287576477331104705911641772138481682906049360e-6416', '-5.87426829330340547489934735333811571352432110412875764773311047059116417721384e-6416', 78, 1);
    T('-5.7215753817629721723180740209343545067304363734183776559544e+941', '7016523421977501249723778592612429967458684107745534160459415336808848152279743423.0668', '-5.721575381762972172318074020934e+941', 31, 2);
    T('-7.02702276643566900354455889172971653055573e+208095', '3.1385249373648119405249018029681510060240311651663321860133526277691069177182640849572656671e+1153567', '-3.13852493736481194052490180296815100602403116516633218601335262776910691771826408495726566710001e+1153567', 96, 3);
    T('-1.7832354748159574129369361988269904804308254487309290902349465951236972e+55700094', '9.90973236504901221625863735557853554449248096229915839712817629426762063100501286822728059080142235e+78756406', '-9.909733e+78756406', 7, 0);
    T('5.806100701116837194078609598286654001909211492267547200e-6351', '-3.1322555058724231886103050203444286e+8361722', '3.1322555058724231886103050203444286e+8361722', 89, 1);
    T('6.858541567594014583569e-92403457', '932344125683735556729917290084013738442958.911626146652618624368048421976479', '-9.323441256837355567e+41', 19, 1);
    T('-515603412562141323976260579.7608', '-4.3459986742e+61871', '4.3459986742e+61871', 24, 0);
    T('-2482.166560', '-8.7529651707683159200571472364119589276981748230359366217157560936540211030e+129', '8.75296518e+129', 9, 2);
    T('7137625499977739961.174647264422176975940443694511971170437331664836965532929116335954039', '6.344746501443024903148745334379114127368257359979286387628469760443123519696786e-564190646', '7.1376254999777399611746472644221769759404436945119711704373317e+18', 62, 5);
    T('-9.718417655944792606998316560239480283922e-4697', '-4.215428946661920636536185801565059268e-4262', '4.21542894666192063653618580156505926799999999999999999999999999999999999999999999999999999999999999e-4262', 99, 3);
    T('-8.73155178653866580e-972', '5.65980788569468390921784398580677872138446786064513360271081e-9214177', '-8.731551786538665800000000000000000000000000000000000000000000000000000000000000000000000000000000001e-972', 100, 3);
    T('-7.4825478541105716433040812574e-2957384', '-8933987726393636136642392373893254836880610561986804046816378730701943482730.285263', '8.93398772639363613664239237389325483688061056198680404681637873070194348e+75', 72, 1);
    T('-6.1868661228611360547121138333173109576855395466826784e-9251', '-3.3407355564230790311536623458601152993159e-206708339', '-6.18686612286113605471211383331731095768553954668267839999999999999999999e-9251', 72, 2);
    T('-8.2894772800817807e-795566101', '-882094911847133130492689954681328382693767501748865135497984118817292143479942556093392300518133.40', '8.820949118471331304926899546813283826937675017488651354979841188172921434799425560933923005181334e+95', 97, 2);
    T('-3.71e-54', '8.1409094135303963371158960125104271236131e-52265', '-3.710000000000000000000000000000000000000000000000000000000000001e-54', 64, 3);
    T('-4.58273179956741529868729975821855480507520405983306366e+367406391', '-515243.773686779', '-4.5827317995674152986872997582185548050752040598330636599999999999999999999999999999e+367406391', 83, 1);
    T('-2.50886394616743544074971738242958733074732722171141314428860194324273169937311503648610384546609371e+446832580', '-5.82521872374951568851451016671051524344404779233337611028586067667910e+9913', '-2.508863946167435440749717382429587330747327221711413144288601943e+446832580', 64, 4);
    T('-3.08638420514075946627416469567923647833e+31201911', '2.582887131300488984123421452193110e-85847', '-3.0863842051407594662741646956792e+31201911', 32, 2);
    T('-2e+336206', '4.4383829311617309852334502390172388258818749837003695255564821728180233810987e-4959819', '-2e+336206', 3, 1);
    T('2.6794856911009205433777991572659573343613490541699313483127116085508474484735665637332918013508556e+680071', '5.47332627908015431943692541381747512140434659706307200305422698930475430383135e+991583', '-5.47332627908015431943692541381747512140434659706307200305422698930475431e+991583', 72, 3);
    T('6.4599399522e+1165', '-8.7834178694328129118112945369e-81913892', '6.4599399522e+1165', 49, 5);
    T('-1.72532708039484122658385228191583603951884425901167141095585952839e+71', '7.5296784735417463818823886e+50', '-1.7253270803948e+71', 14, 4);
    T('-9.9070108377642261010410510388848272095080170377624969033564701813900228785652675284413e+90', '3.78110238e-67457', '-9.90701083776422610104105103888482720950801703776249690335647e+90', 61, 1);
    T('-5.945610113024962204245211788406592298162776122881254608290852e+12409', '5.568122368839261603872872210425574182695766476e+8763', '-5.945610113024962204245211788406592298162776122881254608290852e+12409', 64, 4);
    T('-9.07925466154073543548457838532802919476309542726034628833877648494e-72944569', '6.00933335515645664229607742147824857498170338484814606000518666270494093472351958e-2629373', '-6.00933335515646e-2629373', 15, 3);
    T('9.53156852514536542095133048116670868572512024671451970029354135806988270590419220e-123', '2.89873367982439e+5303', '-2.89873367982439e+5303', 100, 0);
    T('7.7481082493747523436144380848518313167053450e+93699', '-4.84266765757548545860681e+534417', '4.84266765757548545860681e+534417', 45, 5);
    T('-3.046764338313819642e-7807', '-1.820989056774962561773870148812777071515717803060507292464241208253429929749833767e-421612', '-3.04676433831382e-7807', 15, 3);
    T('3.434000397680570647046626926049816549135425060512894715428840040169072905469889685416040624109708745e+101', '6.83178641183044249610662118286392819259086919669224889580065354663e+80', '3.4340003976805706470398e+101', 23, 4);
    T('3.6710759873707038170701248921953e+89740', '-9.4777107378703013089215885553425442333269640628015444659126296266995e+623111', '9.47771073787030130892158855534e+623111', 30, 6);
    T('-7.99969311902347229966323331947500084903521378103215807918137349336682342519547120597971e+783', '-4.118486471389713473799534109941132804151258705157e-504282013', '-7.999693119023472299663233319475000849035213781032158079181373493366823425195471205979e+783', 85, 2);
    T('5.06974738814303080345474605734351944606612723252576197315108693052948914613168462215334e+857786600', '-9.1821920130691124396778583029e+719111', '6e+857786600', 1, 0);
    T('-5.51196551477469076446127864952627136224889320274229350467212536435980789759284365149585278575e+99', '-4.600789570322075556249889196288474572932098897808305307862991611409255822049809896865195001570015852e-336751', '-5.511966e+99', 7, 5);
    T('2.447946992988765452253214155512149874230776841656168496e+12877', '-9.7683521315833304895128454659e+37472258', '9.7683521315833304895128454659e+37472258', 38, 1);
    T('-7.07480972085588217570926052336434361065442e-265752', '8.3607482546e+5796', '-8.3607482546e+5796', 65, 4);
    T('5.247748908058370115262365727033118815535130315e-7861503', '8.81677e-455745528991', '5.247e-7861503', 4, 1);
    T('6.97638128797040535031424508360768932857e+109', '1.27323617514894482046317027933e-7457958333', '6.9763812879704054e+109', 17, 6);
    T('-2.97910168464030203329440283750597861625601722239384154154e+9115468942', '-9.73236485020594108557589852232e+32', '-3e+9115468942', 1, 0);
    T('8.7577985176674410641101622754779299960257972455e-3238630', '-5.76577171460235123496963574517615124600301754e-617', '5.765771715e-617', 10, 5);
    T('-91415451.098913685', '9.1300491348126212315480650501310448709101743382e-863673999433', '-9.1415451098913685e+7', 98, 6);
    T('7.164719779347900269544078676422023139072123960718e+28631134947018', '-1.9174969267374832994774816670210305314746e+93742510919799', '1.917496926737483299477481667e+93742510919799', 29, 6);
    T('3.37134930462256648941043492911871591302261004e+9397187974534', '5.1461276839896584904359927299286910561429650e+98420627228937', '-5.146127683989658490435992729928691056142964999e+98420627228937', 46, 2);
    T('-1.8192790646932e-57', '-8.39869523721123769313286928134570e-5826065124', '-1.8192790646932e-57', 42, 3);
    T('-8.6335731137923359e+70', '-1.174389771385881695755351822152596960339939e-748434172494', '-8.6335731137923359e+70', 31, 6);
    T('-9e-1759588392', '8.5066261421887970349e+972060061814', '-8.5066261421887970349000000000001e+972060061814', 32, 0);
    T('-2.50e-962653', '7.67479404788022713286215860106710817766054752892828220710202e+9936', '-7.6748e+9936', 6, 3);
    T('-3.6660120505495349180739815259664204885693804867224936420e-82043', '4.393324186512030161923977662e+758739731', '-4.3933241865120301619239776620000001e+758739731', 35, 3);
    T('-2.76354347472604535658989666e-224813913', '1.151768969866837131297756e+31', '-1.1517689698668e+31', 14, 2);
    T('-8.5146461337257e-620784412505253', '9.6771367497108069686e-14', '-9.6771367497108069686e-14', 54, 5);
    T('-6.4631584822609891449701692730072342871135279e-637680638020886', '-4.35840601913850332225444372139265376444376076080283529796802e-93606570576783', '4.358406019138503322254443721392653764443760760802835298e-93606570576783', 55, 6);
    T('671396556384448.7', '-8.5994913400072035669544546614e+68365', '8.599491340007e+68365', 13, 3);
    T('-4.94756514821316446748246663325837614445576022339541723e-10055192403', '-4.011147682263251e+23', '4.011147682263251e+23', 25, 0);
    T('-5.79601794155057051726702122383e+92', '-4.2460821225463503240058754e+35048979251156', '4.24608212254635032400587539999999999999999999999999999999999999999e+35048979251156', 66, 3);
    T('-5.582208920416913757148923376051543919406e+4454000770', '-9.509581784398061103395808014443613811157672738629880872e-25919035', '-5.58220892041691375714892337605154392e+4454000770', 36, 3);
    T('-5.434336805875353791090825075e+217802830', '-5.444637245541474112160089376389889835070616392024123524572e-4115865', '-5.434336805875353791090825074999999999999999999999999999999999999999999999e+217802830', 73, 1);
    T('4.05198105509081694001e-3058629488', '7.297621153e+538192295580590', '-7.2977e+538192295580590', 5, 3);
    T('-1.8427747e+26402621494', '4.8703714249012066e+36395864703', '-4.87037142490120660000000000000000001e+36395864703', 36, 0);
    T('-9.7878835e-13250975573', '8.82716618688634207816461738090421506082254414152759e-2683676105', '-8.827166186886342078164617380904215060822544141527590000000001e-2683676105', 61, 0);
    T('-5.5580336815403580282199059804e+9196497957', '-5.267723928166771951293527672967890e+68559', '-5.55803368154035802822e+9196497957', 21, 6);
    T('-8.59202285900982666769027215110899978132398923842126531857649e+96778043508556', '2.029358778257183e-6380768921', '-8.59202285900982667e+96778043508556', 18, 0);
    T('-5.153146420297204596577154983413e-38765', '2.5330806935472675981658321001818054986290e-92493016400227', '-5.153146420297204596577154983413e-38765', 76, 2);
    T('9.18830826643626e-3103', '8.4782501245154005852967766498e-184149', '9.1883082664362599999999999999999999999999999999e-3103', 47, 3);
    T('8.339473853e+2619', '7.886616060202768228712645073e+335781080658003', '-7.89e+335781080658003', 3, 0);
    T('-9.0093997804112858203394409623467728372123880682476427275e+512578', '-6.6759963150565788918336e+63669088', '6.6759963150565788918e+63669088', 20, 5);
    T('3.201044888648203386137531224285016728969451792577638e+17965523', '-9.15979925587717638062478490833299987527582e-5107551', '3.2010448886482033861375312242850167289694e+17965523', 41, 3);
    T('9.46123118224e+2840803', '4.71896738627783071464492849502651442956692305753982413232446e+993797829273', '-4.718967386277830714644928495026514429567e+993797829273', 40, 3);
    T('3.9942233140024327123910556567768e+523038000', '-2.59e+3', '3.99423e+523038000', 6, 0);
    T('3.0171005301057291451367e+32', '-9.0153577054027148480398511562548e+1503362', '9.0153577054027148480398511562548e+1503362', 47, 6);
    T('2.4738936394878537790256475329307238084159065e+60291119125', '-7.15509629471139e+1255', '2.473893639487853779025647532930723808415906500000000000000000000000000000000000000000000000000000001e+60291119125', 100, 2);
    T('-5.2698092e+2254584', '2.704588019339268152451536353173364012584257080810656044e-51441177467', '-5.26980920000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+2254584', 96, 3);
    T('8.47619399e-36888601506', '5.96055194249e+11462808476124', '-5.96055194249e+11462808476124', 97, 3);
    T('1.549257214651655803286319151590370051762763249222e-95693567', '-6.425845579310581288e-60646', '6.425845579310581288000000000000000001e-60646', 37, 0);
    T('-4.5785468274e-795', '1.582909564498901803523142223024093092737e+113', '-1.582909564498901803523142223024093092737e+113', 94, 5);
    T('-3.68112321200601956605298171117125331200314e+1982', '-7605461834651758528286', '-3.68112321200601956605298171117125331200314e+1982', 86, 6);
    T('0.000006526', '-5.95608798764286586977208279181987863512e-5840343', '6.526e-6', 41, 3);
    T('5.9652495908e-656958624981', '1.81236871e+948', '-1.81236871e+948', 44, 5);
    T('-1.546059484504328288219039430875363762014141e+36257087056193', '8e+3692747853457', '-1.54605948450432828821904e+36257087056193', 25, 3);
    T('-4.435144893708968886945668313208209194e-99676', '-58967003042430747082197641373846979467.258171292', '5.8967003042430747082197641373846979467258171292e+37', 60, 4);
    T('-2.42607062711230896502630980798636e+73641595224190', '-3.9e-614', '-2.42607062711230896502630980798636e+73641595224190', 92, 4);
    T('9.496501466689530174e-854877', '2.84969445849270208487626017295e-87288254', '9.496501466689530174e-854877', 93, 0);
    T('-8.1350034137814188915901790752952182086412734744145813e+8206', '1.67643739537945372105277e+764', '-8.1350034137814188915901790752952182086412734744145813e+8206', 59, 6);
    T('5.9621970566085475692566390525111169257916979268012e+510801088857', '-6.94860995113716657795696501194495591e+73364977403', '5.962197056608547569e+510801088857', 19, 5);
    T('-7.4786565949838735924063437046921828313588793414819e+394', '2.28379e-63758841123056', '-7.4786565949838735924063437046921828313588793414819000000000000000000000000000000001e+394', 83, 0);
    T('-87174.49', '8.5758694384784046175964817980e-89869', '-8.717449e+4', 32, 5);
    T('-7.66548173083054716648157952428158704678659773e-41800439', '4.3217448487735935410e-213721848', '-7.66548173083054716648157952428158704679e-41800439', 39, 0);
    T('2.51516846379257e+19393672', '1.7816979463775184497737076337270220772248194703e-9756', '2.51516846379257e+19393672', 72, 5);
    T('8.55062147028942717125440100630662644208799539950157956888930e-2571', '4.802029260413e+796228403', '-4.802029260413e+796228403', 14, 4);
    T('-2.271649384362705275e-44675608902286', '-5.591196654788702260619445399662e-65705872', '5.591196654788702260619445399662e-65705872', 99, 4);
    T('-3.3473239871181288704974279872525827e+99191431333', '3.50637593255651376e+274517549304', '-3.6e+274517549304', 2, 0);
    T('-8.133082539570244973468220276117357122459662e-6694069555', '-1.9502829210287908229629588937626e-60359721259', '-8.133082539570244973468220276117357122459662e-6694069555', 90, 3);
    T('-9794407559527147565157901779303824243853990305261', '-3.50101549328801902165718000882827269e+68', '3.5010154932880190215592359332330012143484209822e+68', 47, 6);
    T('-1.11946903366406263e+19', '-6.282434731899078851249904166770898108126123e+826131890173', '6.2824347318990788512499041e+826131890173', 26, 1);
    T('4.115e+7054131701', '-2.332633040892211927748965941152720067e-6497610', '4.11500000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+7054131701', 90, 0);
    T('5.54740026891344925153524874660696222827428322952e-48850037896', '-9.688190482501248392190325038238744049036374515227e-1435', '9.6881904825012483921903250382387440490363745152270000000000000000000000000001e-1435', 77, 2);
    T('4.7487625810912081570490355933225502e-8414000416', '2.072438009760896614793925484749950544440662848581528511822e+2578884005', '-2.07243800976089661479392548474995055e+2578884005', 36, 3);
    T('7.372213967475921976218744037327141659982389373e+87015873', '-6.0985243401925170213026132265379487511980478894e+51395', '7.372213967475921976218744037327141659982389373e+87015873', 86, 5);
    T('6.91731663333991471357546421485144135553622167690136244837e-353800135', '6.004863311891689555955056394462773079180139799e+4528934103917', '-6.00486331189168955595505639446277307918013979899999999e+4528934103917', 54, 2);
    T('5.5758e-3353111543815', '-8.4436988831191000994e+896301383971', '8.4436988831191000994e+896301383971', 76, 5);
    T('-4.5e+83916', '2.1621e+7611162679', '-2.1621e+7611162679', 54, 1);
    T('-8.47646457788003205859668240163799320687526e+44', '-7.9245791710077231958320026463007956e+42', '-8.39721878616995482663836237517498525087526e+44', 53, 2);
    T('-5.26136045614480896420720939777233122823572e-35', '3.6939712794766358636783935449349222924748635076587010517e-189477', '-5.261360456144808964207209398e-35', 28, 5);
    T('2.025142229623470312295178400031008763890700290605875060645e+58', '6.3402195543904218e+88968193342775', '-6.3402195543904217999999999999e+88968193342775', 29, 1);
    T('-90907794680581535444669', '-7.247219338304746568102e-51394521799631', '-9.09077946805815354446689999999e+22', 30, 1);
    T('-1.1471791137469236611749639689581559677894433640865e-1342272363610', '52419370783440632369660458.353317', '-5.24193707834406323696604583533170000000000000000000001e+25', 54, 0);
    T('-1.3754222271507295824945620236288558997114e-34278804994', '-7.1820152028675259412007112696976832953091287622e+422853', '7.182015203e+422853', 10, 4);
    T('-3.662604278254181230e+55822020', '-7.899458e+185599', '-3.6626e+55822020', 5, 4);
    T('-7.6360514096047321922768295901851267107367e+42', '5.49093846179782444494725540044e+591846445', '-5.49093846179783e+591846445', 15, 0);
    T('434054086490221652448176982149885615198.6111559', '-4.9683860015756932705879738829951302462237803816898e+305705268', '4.968386001575693270587973882995130246223780381689800000000000000000000000000000001e+305705268', 82, 0);
    T('-8.3389794087115e+92785', '6.4629402665480885961381278312e-13269749107', '-8.3389794087115e+92785', 53, 4);
    T('-3.96001339555013506447437e-4855262985704', '-9.814184492699220392737921280586581732520651e+58416885088170', '9.814184492699220392737921280586581732520650999999999999999999999999999999999999999999999999e+58416885088170', 91, 3);
    T('3.4721130998484830137e-882', '2.1629e-291203301', '3.472113099848483013699999999999999999999999999999999999999999999999999999999999999999999999999999e-882', 97, 3);
    T('5.059517115082448956e-41022955544', '-9.804306235682302225680667e+923176', '9.8043062356823022256806670000000000000000000000001e+923176', 50, 2);
    T('7.743575783165164237476111035293280127e+957376234705642', '-5.3670904807762815490849905018843e+1620944', '7.74357578316516423747611103529328012700000000000000000000001e+957376234705642', 60, 2);
    T('6.42202418446474431e+34159099517696', '-1.87533533481985341722146e+60767913', '6.42202418446474431e+34159099517696', 61, 6);
    T('4.134300900974190752773425823573262726811006222859e-84556975184884', '8.93126282457154122698418855909819564370703575551154e-3629661242241', '-8.93126282457154122698418855909819564370703575551154e-3629661242241', 86, 5);
    T('8.405690198802698624236055326374907586e+11907597230886', '-5.237819038804862516440247e-234355791055', '8.406e+11907597230886', 4, 2);
    T('6.914497958110390245797666785968396058627503185e+81595512423', '9.7153678206885995329408781642499900158981481e+74245053265', '6.914497958110390245797666785968396058627503185e+81595512423', 72, 0);
    T('5.765106544e+6942', '-6.21e-53093938972059', '5.765106544000000000000000000000000000000001e+6942', 43, 2);
    T('3.71e+9421399', '9.7576203757514824936e+7420', '3.71e+9421399', 58, 4);
    T('-274571273897523541382.538674123', '-3.59542288480471272371076883254147733938873737e-30558658231233', '-2.745712739e+20', 10, 5);
    T('3.58750329e-71476', '-3.88189528586860280823e-60022254035', '3.58750329e-71476', 45, 5);
    T('4.27633076486458869912655005736204425088437691159242e+1639329581', '-3.1826270167555479266435590047920e-76332581379874', '4.2763307648645886991265500573620442508843e+1639329581', 41, 1);
    T('8.580847134245241773102432361e-77027617882', '-2.06342463016937632827971858406104815002232482052984448e+270837769', '2.063424630169376328279718584061048e+270837769', 34, 1);
    T('2.2e+806774', '3.55426700825494052e-5219960971', '2.2e+806774', 72, 0);
    T('-6.08e+186501155573116', '-8.87632314516200621450669e-711562152237', '-6.079999999999999999999999e+186501155573116', 25, 2);
    T('-3.57e-75172', '1.4342163863410376754076760563092212e-465673498', '-3.57e-75172', 16, 1);
    T('7.885419113424553594887148e-1886767338', '-4.3824917705327e+70972400', '4.382491770532700000000000000000000000000000001e+70972400', 46, 0);
    T('9.7639022460027431762427133179e+32', '-5.33175107236e+9805446952562', '5.33175107236000000000000000000000000000000000000000000000000000000001e+9805446952562', 69, 2);
    T('6.48736728191063235821960113805593945214223035271e+247313250', '-5.3113628664167730735305364733909142738988e+76194545734', '5.3113628664167730735305364733909142738988e+76194545734', 50, 5);
    T('5.457170e-8418', '1.2317635713229557211e-83460249440', '5.45717e-8418', 64, 0);
    T('-1.13e-74252', '9.23248235830478733461368431708698604215261634622535264686071e+492', '-9.23248235830478733461368431708698604215261634622535264686071e+492', 65, 1);
    T('9.975476738361265502273332056e+791124670667272', '3.9765355883603772240489286564012531910100705803301901527e-203840678', '9.975476738361e+791124670667272', 13, 5);
    T('-2.44215749162076186926886958632910e+8618966', '8.1947555598086597391e+2863', '-2.4421574916207618692688695863291e+8618966', 73, 6);
    T('-9.2701903183755901004647286884e+62037', '-8.334628841088324917e-99163684480', '-9.2701903183755901004647286884e+62037', 78, 6);
    T('-9.08415664092586830694329060232438239586279373e+7618418', '4.78288544345833906476047777502014132879e-553145901813836', '-9.0841566409258683069432906023243823958627937300001e+7618418', 50, 3);
    T('2.552188723018068199931188023149362612926655996716599721442e+503099325341', '-390107286.552512818', '2.552188723018e+503099325341', 13, 6);
    T('2.278018890518437189850819160324292670090797e-287', '9.66615106232887e-29269207', '2.278018890518437189850819160324292670090797e-287', 60, 5);
    T('-4.90645813538321079959584787057369548e+8626849783', '-3.1929453424828144068859e-556423', '-4.906458135383210799595847870573695479999999999999999999999999999999999999999999999999999999e+8626849783', 91, 2);
    T('-3.501491235536522216544583378467e+77838362', '-6.6904179139e+40712052580859', '6.690418e+40712052580859', 7, 6);
    T('-9.14221627924496980e-6617586870', '1.588493623004966553354092e+2700', '-1.588493623004967e+2700', 16, 5);
    T('5.57397105298086025809828558474734793697485e-832230173742656', '2.9792972845413708e+8644349', '-2.9792972845413708e+8644349', 46, 4);
    T('7.405369116845986142e-53484536279', '6.281606320480843443042324450788842456365864589e-156332', '-6.281606320480843e-156332', 16, 2);
    T('-1.3469393855881820e-77952031559696', '-2.65420963716613e-918536648587290', '-1.3469393855881819999999999999999999999999999999999999999999999999999999999999999999999999999e-77952031559696', 92, 2);
    T('-4.218252012885871613004917931920186950902113e+89744863461594', '-9.93337047817539049819025227988902026657469428069943213e+916500', '-4.2182520128858716131e+89744863461594', 20, 0);
    T('-8.054462243243253221610656270612083927755533160010026750930e+2461332713663', '8.0090674457728010573135e-13', '-8.05446224324325322161065627061208392775553316001002675093e+2461332713663', 77, 5);
    T('8.574123677582687103032670832e-17157526564', '-5.5716846e+7388', '5.57168460000000000000000000000000000000000000000000000000000001e+7388', 63, 0);
    T('978690073537649888513', '-9.98825615048910074530543062378e-310452496706', '9.78690073537649888513e+20', 100, 6);
    T('-8.2642699498126986607615902416083652828404043335589e-927', '5.5e+57', '-5.5e+57', 90, 5);
    T('-4.9663994295376e-135094', '9.249e-49515', '-9.249e-49515', 62, 6);
    T('1.51873481e-25375373342696', '2.75875072328647130e-1785055753', '-2.7587507233e-1785055753', 11, 0);
    T('-5.1087064214178751326912973e-23', '-9.0829693290329693414561411106939748e+1696726258', '9.083e+1696726258', 4, 0);
    T('-7.912e-252812', '1.268154063726430685475102088108203102041962436626e-9260479', '-7.912e-252812', 57, 1);
    T('-8.95906391070806772511478556331188010965e+466', '-6.557774718e-69073887422751', '-8.95906391070806772511478556331188010965e+466', 94, 0);
    T('3.480413466171352052129211e+97', '4.86705655992683295450e-34', '3.480413466171352052129211e+97', 79, 6);
    T('5.24342835461049422295130946197546180638621e+969', '-1.193046595471292954792994384e+66677367', '1.1930465954712929547929943840000000000000001e+66677367', 44, 2);
    T('-9.19509981e+8964096114820', '4.43157455e-426646098', '-9.19509981e+8964096114820', 87, 4);
    T('2.04291568282561e+59', '6.47442567045529610143086486516063245376670e-67581540', '2.042915682825e+59', 13, 3);
    T('-6.059e-2398', '4.732e+565', '-4.7320000000000000000000000000000000000000000001e+565', 47, 3);
    T('-4.28288862386659735238372140914728831e-934547456920043', '-9.751983553997341794504054107986912354162496508388756359e+795095', '9.751983553997342e+795095', 16, 6);
    T('-7.1642188181728502799030845947e+85466810723902', '6.62303410e-7596', '-7.1642188181728502799030845947e+85466810723902', 93, 6);
    T('1.337374651765549634521456344893181592217910620e-96538425262469', '-9.85968428210947585897e+25979245', '9.85968428210947585897e+25979245', 100, 5);
    T('-3.73658827113731611010436534711502557344e+775', '7.3220490353148981280559429783199437171890208206e-544427533763', '-3.736588271137316110104365347115025573440000000000000000001e+775', 58, 0);
    T('4.83328e-101539471', '-1.6744866554912271932407403388546e+9813054664', '1.674486655491227193240740338854600000000000001e+9813054664', 46, 2);
    T('6.344861684327234064e+6757028679', '-8.2327827231313230299747822e-1783', '6.344861684327234064e+6757028679', 87, 5);
    T('9.70351284617570731695045351183806913408452501912200659274e+3415221637104', '9.703862710969174051545782816032641937892473306766264e-1811904', '9.70351284617570731695045351183806913408452501912200659274e+3415221637104', 85, 2);
    T('-0.031358880437211983722297590995479661706017210121178001', '-6.941206337488208416686619173519814297416718210433695e-818799080137', '-3.1358880437211983722297590995479661706017210121178e-2', 53, 1);
    T('-6.9455982399936725122687717917648502567453265569897874742e-11907', '4.714091731956854763167235996093495226469147669410405e+76', '-4.714091731956854763167235996093495226469147669410405000000000000001e+76', 67, 3);
    T('-4.7948828157733417544e-589', '-9.08733069929587759816935304431206934951372636e-83', '9.0873306992958775981693530443120693495137e-83', 41, 4);
    T('-1.55729401832e-73650713770', '-1.228347241532152303040201217e-8076', '1.2283472415321523030402012169999999999999999999999999999999999999999999999999999999e-8076', 83, 1);
    T('-1.1741617067187547126042186125329678845218e+23250187644769', '6.31884356436020755e-60486040161', '-1.17416170671875471260421861253296788452180000000000000000000000000000000000000000000000000000001e+23250187644769', 96, 0);
    T('3.42618537472834634761041229661058098720745685e+110', '9.026721172151607e+5086', '-9.026721172151607e+5086', 43, 0);
    T('-4.6193167404532731635585622674782458126403217495571723044e-9235', '-2.3483331735353715462820171644454359329346e+436001192041', '2.3483331735353715462820171644454359329346e+436001192041', 43, 4);
    T('-4.860044543255687506208399289931718081548991788848e+2771096', '5.57439785701666e-269484174874', '-4.860044543255687506208399289931718081548991788848e+2771096', 63, 1);
    T('-5.233119564e-340934784166267', '5.52601684811e-48747181836184', '-5.5260168481100000000000000000000001e-48747181836184', 35, 3);
    T('4.68e+73200514', '9.6024e+4025982920', '-9.60239999999999999999999999999999999999999999999999999999999999999e+4025982920', 66, 1);
    T('4.3093627645161630672931417702596786959804e-37', '-917954.89688466', '9.1795489688466e+5', 31, 1);
    T('-4.221748585e-9722154', '-6.59874476830722878436475923510e-7946196601283', '-4.221748585e-9722154', 40, 5);
    T('8.118642578845491253776787588885353633486585947587084021700e-6616156450', '5.051440665255113540120421120228843180e-866269910713', '8.118642578e-6616156450', 10, 1);
    T('-7.382549879061955857185322132705357909826634636843837454e+550720821', '2e-3779410', '-7.382549879061955857185322132705357909826634636843837454e+550720821', 61, 1);
    T('-1.52198415671e-683267206879097', '-3.832655362641910281e+914232', '3.83265536264191028099999999999999999999999999999e+914232', 48, 1);
    T('2.180274796455202242872032228798990321271113585921e-33833', '-4.4825445335553170316780e+26', '4.482544533555317031678e+26', 22, 6);
    T('-6.7e-267504198', '9.8433895407472743915070911746767340616e-14756582901746', '-6.7e-267504198', 32, 2);
    T('-6.0521331596119373895284e-26380533', '-4.615839242540296719882660548071e-549349138465701', '-6.052133159611937389528e-26380533', 22, 4);
    T('-7.20381172460620287544053344390794420e-76776871687200', '-6.77743432191953058e+7962623', '6.77743432191953058e+7962623', 32, 4);
    T('-3.386490936595343595707970739804645359060435417034768e+9047387', '-7.54689327429287096243855251269872766e-76486536481', '-3.386490936595343595707970739804645359060435417e+9047387', 47, 2);
    T('4.9331560430258e+3380461', '-1.07664424901041106498540429953e-476', '4.9331560430258001e+3380461', 17, 0);
    T('-8.353429e+48841095', '-64960379913907408734624064846134628779751', '-8.353429e+48841095', 89, 0);
    T('-5.920694404061083913407903257785437117e-752776', '4.8904894406734526883619581424572170379073041184270638809e-1114610', '-5.92069440406108391340790325778543711700000000000000001e-752776', 54, 3);
    T('2.6880177569180798312898708968315823868336472264091797744673e-61733', '8.45359452634040745e-3292263115407', '2.6880177569180798312898708968315823868336472264091797744673e-61733', 95, 6);
    T('4.128989778315672202413012913819622e-423', '1.86385181348623831592255883279436538806825e+2872356337381', '-1.86385181348623831592255883279436538806825e+2872356337381', 75, 5);
    T('-3.10286520631e+86', '6.54630595618951953265695459515697943278570939760296e-639668', '-3.102865206e+86', 10, 1);
    T('1.581599992365108640771734184462069e+88543339792246', '8.11777209e+26497308541672', '1.581599992365108640771734184462069e+88543339792246', 42, 6);
    T('1.5202201485221737998096991676208e-63196496249', '-7.8111276895992267248644208790447420626688815e+81907829', '7.8111276895992267248644208790447420626688815e+81907829', 56, 3);
    T('3.229873585821815746160e-40253939650092', '9.124163161897756328103088077420295643946909929673998e+781448', '-9.12416316189775632810308807742029564e+781448', 36, 1);
    T('7.9407372049484817105163627074329247423552749332e+2655797332', '-2.8647895606760403190495e+5261', '7.94073720494848171051636271e+2655797332', 27, 4);
    T('7.50157991658003696730680851667075e-564', '2.68540616198848166834949613e-553518122403', '7.5015799165800369673068085166e-564', 29, 3);
    T('9.143293397e-669936623', '-9.4700219227272350183063275114591088815333156054984e+1698408', '9.4700219227272350183063275114591088815333156e+1698408', 45, 3);
    T('-4.281149448373575592853575179797e+638518576', '-7.389997610708482931357082831908838054942768828083e+1401129213', '7.3899976107084829313570828319088380549427688e+1401129213', 44, 3);
    T('3342603899393420459663130700716032600.877210', '-1.6767941839575528e+34330809510', '1.67679418395755280000000000001e+34330809510', 30, 2);
    T('-4.5937857694259904571889058183e-1403815578784', '-8.50013911137530502469341007e-435', '8.50013911137530502469341007e-435', 85, 2);
    T('-3.1304712572762744323321747078593304e+98371', '4.8925687e-3919', '-3.1304712572762744323321747078593304e+98371', 83, 4);
    T('-2.7351528696400e-3025635', '-6.16610389221931473e+90339933505679', '6.16610389221931473e+90339933505679', 26, 6);
    T('-7.6794917253738384843994536567904e+33', '-9.20734516824830949768905232910226851950310863091462403969046e-366864222362174', '-7.6794917253738385e+33', 17, 0);
    T('618689895836818243740178599692868572634.646409771', '-4.5707e+15694203968126', '4.5707e+15694203968126', 100, 3);
    T('6.854267584560831969262534459007967122444138090e+276750995955', '-6.443980643009043948e+19949762788515', '6.443980643009043948e+19949762788515', 52, 6);
    T('3.955446839325477271024262624242159742827437314438660399e+61874720210', '8.270680696534106379977063066109004796e-5712412300', '3.95544683932547727102426262424215974282743731443866039899999999999999999999e+61874720210', 75, 3);
    T('-9.2545042538e+15', '7.517204851e-353726814615', '-9.2545042538000000000000000000000000000000000000000000000000000000000000001e+15', 74, 3);
    T('-16772363931121965593752376458491261809288833388574', '-8.56381357663002e+8088', '8.56381357663002e+8088', 96, 0);
    T('7.7e+836906491996', '1.67562615045533289672415236111e+59689305', '7.7e+836906491996', 26, 4);
    T('2.0496e-69', '6.83482262934270625148512296723200008994915414e-619850481659570', '2.0496e-69', 71, 2);
    T('6.60258e+829', '9.7365541e+86', '6.60258e+829', 73, 0);
    T('5.709500026481435444527236139284084636151407003977929965e-249317964111802', '-4.937617902397499092166753e-54966', '4.9376179023974990921667530000000001e-54966', 35, 0);
    T('6.6479310846483801808487379456198812690e+4020988', '2e-419295910128', '6.647931084648380180848737945619881269e+4020988', 66, 6);
    T('-8.5296852571492054862232567663976468919212e-708379938918', '-5e+7923690810', '5e+7923690810', 9, 2);
    T('-1.5273902346545667229827766325518166659492054337e-8004', '4.56600735716214407225479050470226812397506168263460e+68314', '-4.566007357162144072255e+68314', 22, 4);
    T('-9.1051161661367892078789516468146987332e+242', '1.27194249761006318309430320044635e+4918', '-1.27194249761006318309430320044635e+4918', 40, 2);
    T('-1.25129896135499513487822950643286908212e+4518370786600', '8.17002575165277351578549870577669039754947175002788915122e+61', '-1.25129896135499513487822950643286908212e+4518370786600', 84, 6);
    T('4.135208036034384883e-7421', '3.4281036585773439579666914610192992124339903457750e-2357', '-3.428103658577343957966691461019299212433990345774999999999999999999e-2357', 67, 2);
    T('-9.598940509383809363321459989430500241066970445250841e+613', '7.813523504770492665113139649557527922954386887222140e-6013119299472', '-9.598940509383809363321459989430500241066970445250841e+613', 65, 1);
    T('-7.7200816713896600602098959709838236249540009305488054698e+543', '-7.32881001429e-808221', '-7.7200816713896600602098959709838236249540009305488054698e+543', 60, 5);
    T('-2.0906108e+8089795925250', '-2.83205e-3304584237547', '-2.0906108e+8089795925250', 78, 0);
    T('5.9741259646123724881769010367e+6457428487', '1.067997814678679187393e+780456', '5.9741259646123724881769010367e+6457428487', 70, 4);
    T('-8.9078429285014476435540324096417361651392809728e+313462664490', '7.97903286961e+88050839798', '-8.907842928501447644e+313462664490', 19, 0);
    T('-7.2e-5334174', '5.63759133164933320101532100284417851286887844476e+3642', '-5.6375913316e+3642', 11, 6);
    T('-4.46713e-9442166608213', '-2.43256326e-20', '2.43256326e-20', 20, 0);
    T('7.538530282235431325288170487276e+51298867629', '58790231246644438700334451397702284420483.375', '7.53853028223543132529e+51298867629', 21, 4);
    T('-1.0084446774907567678181082790221573517691555936269120791e-9968126942', '8.97774559828759628e-496', '-8.97774559828759628e-496', 26, 1);
    T('8.4834424284973988778123917094274167226998664101005019e+559', '2880099748282768151149665939424578222456634175646619.0465715', '8.483442428497398877812391709427416722699866410101e+559', 49, 5);
    T('-1.287254506933504000741970e+90338', '-8.58619223128306e+6090397935', '8.58619e+6090397935', 6, 3);
    T('7.1135330119151358900623038444306e-5919', '-9.02507e+755036284970139', '9.1e+755036284970139', 2, 2);
    T('1.39913074284939478076095721229880522524873614e+548008414557', '3.462555046828612041176304325040162e-763', '1.39913074284939478076095721229880522524873614e+548008414557', 57, 4);
    T('7.76785679085504705e+56690430', '9.74665232563693e+528514184365', '-9.7466523256369299999999999999999999999999999999999999999999999999999999999999999999999999999999e+528514184365', 95, 1);
    T('4.72099229929846806294440103041433678535910256553487e+2167783637', '-5.66324102718997981e-1753', '4.720992299298468062945e+2167783637', 22, 0);
    T('-3.5420195571507891934665462328e-18343', '-9.34360466985385454432994205368332608e+112', '9.34360466985385454432994205368332608e+112', 48, 0);
    T('4.3876719123345885136614007475100e-5852', '-7.1e-553303936', '4.387671912334588513661401e-5852', 25, 2);
    T('-1.6749661733917187e+7086085850812', '7.39859661411530691144168506843958129889812842e-767969573', '-1.6749661734e+7086085850812', 12, 3);
    T('-2.11185803355934362601866930019127756445361347908939436446e-383778042204195', '6.62346152761631362509844883597306244716124240147652e-3167974', '-6.62346152761631362509844883597306244716124240147652000000000000000000000001e-3167974', 75, 0);
    T('-9.03122597607287078805823983997325503403809e-1443935', '-3.734496668585059301660275950750623364779997977681861873678e+608654791818', '3.7344966686e+608654791818', 11, 2);
    T('4.436666210563150438662521327979852442645181595674078533e-245474411022', '7.3456782815664510875078721890746508037e-29679077331', '-7.3456782815664510875078721890746508037e-29679077331', 75, 0);
    T('1.4807070920255121447000906955378250e-407', '-5.392624281671689696832589774660864998811820e+129', '5.392624281671689e+129', 16, 3);
    T('9.19676990617795362520501e-871', '3.40854223214818283175222199602151699e+100', '-3.40854223215e+100', 12, 4);
    T('-3.85e+8043751456779', '285285423.94', '-3.85e+8043751456779', 88, 4);
    T('-7.155951e-9490159593501', '4.294620694604831023762296544905431986812693961534018414e-324', '-4.2946206946048310237622965449054319868e-324', 38, 6);
    T('-8.034773618776391690856869938787830026255891361262284798e+595603029210', '-3.555e-242', '-8.034773618776391690856869938787830026255891361262284798e+595603029210', 86, 4);
    T('5.154946229304204579177906055509441466106370160e+46', '3.02073008455708760100736551325033197769885338e-52271067', '5.154946229304204579177906055509e+46', 31, 1);
    T('1.2773130068461922596108548076079055801287931090933e+90', '512231774', '1.2773130068461922596108548076079055801288e+90', 41, 0);
    T('7e-878429351216935', '2.1006223961358478286274732672536694218e-3107396574', '-2.1006223961358478286274732672536694218e-3107396574', 90, 3);
    T('-5.90742077280486383431451439e+94716', '-6.876676532100304761790559132882702415333347339688176e-520297533729790', '-5.90742077280486383431451439e+94716', 72, 3);
    T('-1.30e-5260', '-8.88673280173966749484e+7106399', '8.89e+7106399', 3, 0);
    T('-3.668e+9673', '-9.749459525e-21677871728725', '-3.667999999999999999999999999999999999999e+9673', 40, 1);
    T('-5.05824722643409723440887384324037097353652401e+109', '9.719265422335934306541007369706805962751416567173e+1552276078463', '-9.7192654223359343065410073697068059627514165671730000000000000001e+1552276078463', 65, 0);
    T('-8.334767868548256916165377245255027280371237e-64274', '9.63402557025886715703991107e-6042', '-9.63402557025886715703991107e-6042', 84, 1);
    T('-1.776789971526795771e-7473420604', '-8.531097228843262792022957882382e-90152', '8.5310972288432627e-90152', 17, 1);
    T('-5.3845327292601026833882e-5145715595339', '2.88e-75380', '-2.88e-75380', 69, 6);
    T('-707235900375.5501214596095377287998342912748433801784183', '-5.7876655780592164848088475860261e+71343055533274', '5.8e+71343055533274', 2, 6);
    T('-9.2413419560149420433080341187680911933956721414301206065e+48695', '1.7279037253681e+2545022', '-1.7279037253681e+2545022', 17, 1);
    T('1.86e+55654', '-9.26567737e-84933', '1.86e+55654', 41, 4);
    T('5.2675671583076014834898158535712394558106290269249488318489e+723155', '-5795724.489568506', '5.2675671583076014834898158535712394558106290269249488318489e+723155', 74, 1);
    T('9.1354085571087595061293577985890960923714658e-1644', '43274005999493366584742296277550037.23451954', '-4.3274006e+34', 10, 6);
    T('-4.118057777099760893739348944529691e+5586164', '3.1412440775527535851095929518629839261626266918201669e-9923245646476', '-4.118057777099760893739349e+5586164', 25, 5);
    T('9.8986679385989528007434586292287136648380866522294897783e-16584260743549', '7.77352730e+23314', '-7.7735273e+23314', 81, 5);
    T('5.34978546119173903165818293822683493142480838724363060369141e-30465147941717', '-1.933718344e+6234589325', '1.933718344e+6234589325', 55, 5);
    T('-8.15873934961e+59901310410', '-71', '-8.15873935e+59901310410', 10, 0);
    T('4.413885118870396524263e-31763708', '-4.15523607651049249127018951833048634482475e+74247785137582', '4.15523607651049249127018951833048634482475e+74247785137582', 99, 6);
    T('1.9622523376718e+4186258034', '2.45757930124333151482501164094170597279264451e-760977421', '1.9622523376718e+4186258034', 21, 5);
    T('5.217521786989065017596428652697643e-42', '9.1173673920909633933789009e+63834', '-9.1173673920909633933789009e+63834', 69, 4);
    T('-3e+862', '9.4422516783560499632788604714241476e+753728257465690', '-9.4422516783560499632788604714241476e+753728257465690', 72, 4);
    T('9.8481e-89387', '-1.78120087099345744345692292539868050807626e+788266450', '1.781200870993457443456923e+788266450', 26, 0);
    T('6.213748437108474709054003823e+159', '-8.043810132400430937851042743840512082607182e-833789527418', '6.213748437108474709054003823e+159', 91, 5);
    T('-9.4918590731915946e+2327355653', '3.4554910893979469375264508606766595988025498150166e-4955095', '-9.49186e+2327355653', 6, 6);
    T('9.32056936844455116127117179717e-89871711482', '4.4763879353673346836e+9119216799082', '-4.4763879353673346836e+9119216799082', 66, 6);
    T('2.702121126735692661e-12', '5.6718738740931115833146920907e-29', '2.70212112673569260428126125906888416685308e-12', 42, 4);
    T('2.0914737305869904348445254279250027928995276214740637316131e+8232346135', '-1.3634739133639035763332717e+911045365427', '1.3634739133639035763332717e+911045365427', 48, 4);
    T('-9.74170242392547988953751297051e+68676973281', '1.636e+9', '-9.742e+68676973281', 4, 5);
    T('-6.39756446689710470424785627008575819492619929625965e+56295051', '7.790279442133191543824646941491552722705401337168337804e-7147925', '-6.39756446689710470424785627008575e+56295051', 33, 1);
    T('-3.28111202746519480627936719533542121005130068455e-286569', '-1.522199830729233e+1578924924437', '1.522199830729233e+1578924924437', 91, 6);
    T('-3.438353757098981e+711917', '-2.90168172080488360252694266938761630557577761072993e-3140', '-3.4383537570989809999e+711917', 20, 2);
    T('-4.083187182462379542955e-2885689', '-4.298655762497374914545466115e+98908548226917', '4.29e+98908548226917', 3, 3);
    T('1.68847175683674e+88138599', '-2.94286627224877316573315900100370718004e+1011404', '1.688471756836740000000000000000000000000000000000000000000000000000000000000000000000001e+88138599', 88, 0);
    T('-1.3796436364940511143e-88012600', '-4.5026533e+8190239527', '4.5026533e+8190239527', 43, 4);
    T('-9.72920145945922609507224740747e+71716', '-3.39907518576656668e+66', '-9.72920145945922609507224740746999999999999999999999999999999999999999999999999e+71716', 78, 2);
    T('3.40205273239399272384799e+986813', '-3.550922335e-4498409203', '3.4020527323939927238479900000000000000000000000000000001e+986813', 56, 0);
    T('-7.7336887e+6853493745278', '6.000554834536363369148227245863999622151769615831e-781326584', '-7.7336887000000000000000000000000000000000000001e+6853493745278', 47, 0);
    T('4.75567233053735801610679874552736662578980e+71565886836', '-2.0281307575612211100e+394799192', '4.7556723305373580161067987455273666257898e+71565886836', 96, 1);
    T('-6.13004916538091446967547629011177743282919339136804989e+8734', '-6.06836341568233982263708801139666666053473543e+1426021749', '6.06836341568233982263708801139666666053473543e+1426021749', 56, 6);
    T('7.00647381114745573629166022502455556217861873038e-525038575248027', '-6.14072332651453508387356609e-712351353185476', '7.00647381114745573629166022502455556e-525038575248027', 36, 4);
    T('-4.103446473647331953215223064348304471655850934128606252960e+66', '-4.411965685282520070534508182861465489421596451523013e+52', '-4.10344647364728783355837023914759912657402231947371203699548476987e+66', 73, 5);
    T('-1.18412965537180390297677855102877781998271905755898970777e-99569231', '-3.791312322193391998835168736055994839e+8197353086', '3.791312322193391998835168736055994839e+8197353086', 71, 0);
    T('-9.9251325895964959497471603359176818061568242026826859e+426092115', '1.72128912231109983189570456350e+164279937', '-9.9251325895964959497471603359176818061568242026826859e+426092115', 56, 5);
    T('-2.136541576333740072886228317459629438553384929853e-699816124', '-5.46823504332922707e+1669304', '5.468235043329227069999999999999999999999999999999999999999999999999999999999999999999999999999999e+1669304', 97, 3);
    T('-1.2322451395151864994757225523634729e-573877', '-9.16794179e+621408178', '9.16794179e+621408178', 21, 6);
    T('1.8e-504249', '3.871106955753691756331495252e-2165666', '1.8e-504249', 62, 0);
    T('-5.542137173516766135714285740177e+18338845649556', '9.0180702431298068226476133730821104728279238261861195639302e+64', '-5.542137173516766135714285740177e+18338845649556', 64, 1);
    T('-6.201109125586968851326733946687013230446641337860132466273e+976', '-2.533196771656266392012687151894764361673840e-2556857876217', '-6.20110912558696885132673394668701323044664133786013246627e+976', 57, 4);
    T('3.6692946240771971614446317444e+75322253', '4.962480e-241111', '3.6692946240771971614446317444e+75322253', 31, 6);
    T('-5.37415332064e-351536865168', '-3.894891558907812186574316169725119e+64', '3.894891558907812186574316169725119e+64', 39, 2);
    T('-1.4336541931353281430245002490452642436769490404e+7006764942443', '2.3264528417974952e-15649049704543', '-1.4336541931353281430245002490452642436769490404e+7006764942443', 74, 5);
    T('5.19516071194179314868110783210762944923508e+768476107472', '1.29020651828377699977238849257684483934763351194937e+596447', '5.19516071194179e+768476107472', 15, 4);
    T('7.9310207911694734558e-8309238', '-1.54911804087724584717376769909545291661212079618991e+2393488656', '1.549118040877245847e+2393488656', 19, 3);
    T('7.4714184623170096197855655702091152507905e+90829094614729', '4.09720402e-83121972466080', '7.4714184623170096197855655702091152507905e+90829094614729', 76, 5);
    T('8.4540905968430091829479575465281840391491846387610547626e+26105956635', '9.82753830550732492e-19772385', '8.45409059684300918e+26105956635', 18, 6);
    T('585318619198485099707896916291760.371309006', '-1.6032160e+9036', '1.603216e+9036', 22, 4);
    T('-2.35369064818725126639522368525538762328624e-649762768', '5.680508237565494284464043346360712352565866188288e+2739098', '-5.680508237565494284464043346360712352565866188288e+2739098', 98, 1);
    T('6.2421731359930769275583307744160503693449318052816711e-801773866504', '8.33555383695860091911692633e-445', '-8.33555383695860091911692633e-445', 40, 5);
    T('4.2600925749604921227857923e+48963', '-3.84777956994716279155066e-2698496896', '4.26009257496049212278579230000000000000000000000000000000000000000000000000001e+48963', 78, 2);
    T('22550167.15354294', '2.917528009832928123858173042935456621375145441859193e+93940017', '-2.91752800983292812385817304293e+93940017', 30, 2);
    T('-8601786.0128211119886960382482220895', '-7.33430620423704744778315e-60842224298', '-8.6017860128211119886960382482220895e+6', 56, 6);
    T('-3.90771641395528800949278805638668e-585355768104', '-6.67550498884035068868316946e-758734', '6.67550498884e-758734', 13, 1);
    T('-9.012056718777592918e-84689082198', '1.48619061944525271916970794e+157681', '-1.486190619445252719169708e+157681', 25, 0);
    T('1.678478051189e-96586846', '-8.098576304375220921738783675e-36', '8.098576304375220921738783675e-36', 38, 1);
    T('-6.162528108481202418798038504569e+31', '-2.833e-802233902', '-6.162528108481202418799e+31', 22, 0);
    T('4.931374e-66', '5.58800767007266094706969857225916513222197898997140471821776e-348693000121199', '4.931374e-66', 72, 6);
    T('-3.708378431842309e-1506', '-1.3852896220785686562375590e-270', '1.385289622078568656237559e-270', 45, 0);
    T('8.6584274600521227418e+298811681257', '-7.62792558786679183265969248617701976090302608499195e-6726', '8.6584274600521227418e+298811681257', 64, 4);
    T('2.4788950352386228659236745630e-44805456031', '6.246572934822131143620190e+35404543', '-6.2465729348221311436201899999999999999999999999999999999999999999e+35404543', 65, 1);
    T('-1.199991224502451875032985951387303322544658444e+564001801', '-152085178948863252660001898610082103443361226921968.222072', '-1.199e+564001801', 4, 2);
    T('-6.43161674633344113284524123e+41281001', '-0.98226474', '-6.43161674633344113284524123e+41281001', 85, 0);
    T('-1.230711757670788955142872557491436731981888795053629993e+280', '5.28457008556650286190589380904715674526255052949e+76687769550026', '-5.28457008556650286190589380904715674526255052949e+76687769550026', 78, 1);
    T('2.710852405855932025179457798799534697905377204214112164e+266953', '5.87516421480173094208599533271861823877967271847913683704e-1760874', '2.710852405855932025179457798799534697905377204214112164e+266953', 62, 0);
    T('-5.0620653737955274236493031140918698210901741062308e-44475', '-1e-663', '1e-663', 15, 2);
    T('-1.2700839130972024096972372278968986445671159768547146163e-8315', '3.812931982367291e-12432161', '-1.2700839130972024096972372e-8315', 26, 1);
    T('8.540053656186675e+6217298', '-9.7437e-22268331936', '8.540053656186675000000000000000000000001e+6217298', 40, 0);
    T('2.5034863851562034691957e+788195756594', '1e+5101745101719', '-1e+5101745101719', 97, 3);
    T('6.56152892448380857952485333351e+45031630640704', '-1.65016390e-4970867864', '6.56152892448380857952485333351e+45031630640704', 84, 3);
    T('8.913767112356906854898275928446148140301e+71', '4.997272439063117232111317645026376535118349e+99', '-4.997272439063117232111317644134999823882658314510172407155385186e+99', 64, 3);
    T('7.02406682033709042320644655463639633495101627122351029e-37', '9.8527347322991719089053905311306e+6690269581287', '-9.8527347322991719089053905311306e+6690269581287', 60, 0);
    T('7.12498480341358793651269515854720079173444061e+46425571954656', '8.172737781583183865740133629291028861836212e+98942', '7.12498480341358793651269515854720079173444061e+46425571954656', 66, 4);
    T('2.9468015724002228409286754e+195724', '-4.94291550483987820991231290191459194279254e+4361754426925', '4.94291550483987820991231290191459194279254e+4361754426925', 88, 1);
    T('-6.4408733361557160235590e-63520', '-1.74060742982356779579190918259610103363e+77529867822545', '1.74060742982356779579190918259610103363e+77529867822545', 47, 4);
    T('-5.2160985130661908440557031429855060893e+720525', '9.168692898679800448197925435800974274941769990002300576212e-276492531', '-5.216098513066190844e+720525', 19, 4);
    T('4.45685079730774849534336443815441358728639e+9336306', '-1.84354578830299538361354e+832888617', '1.84354578830299538361354e+832888617', 100, 4);
    T('-6.84537576665558669974e+494532690419', '6.540060422193379969190123943079291036180532651236e+95880488', '-6.84537576665558669974e+494532690419', 89, 5);
    T('9e+461', '6.308902539711492141347803994e-979656227', '9e+461', 45, 4);
    T('-6.3142221344129345040235804e+8728103326', '-5.773979883697482652500e-729297954', '-6.3142221344129345040235803999999999999999999999999999e+8728103326', 53, 1);
    T('5.0293915994772011065461832882741593959549e-8993147854', '4.3691526173e+7388087185347', '-4.369152617299999999999999999999999e+7388087185347', 34, 1);
    T('-7.87218352779098763004068276e+60022', '2.61073516854798898460e+4564381506', '-2.61073516854798898460000000000000000000000000000000000000000000000000000000000000000000000000001e+4564381506', 96, 0);
    T('5.21883125121161191051742709897e+931157628854918', '-7.781492130428848375904e+113', '5.21883125121161191051742709897e+931157628854918', 78, 5);
    T('9.2512374509815866525625718e+56327406', '82963093804090966979940051.1074', '9.2512374509815866525625718e+56327406', 28, 5);
    T('1.402143174110464412243637138e+713731642512838', '-7.371e+41', '1.402143174110464412243637138e+713731642512838', 48, 6);
    T('1.816e+3464331', '3.814492060e+2504205202863', '-3.814492059999999e+2504205202863', 16, 1);
    T('4.53971309161395152025729057699194958e+131', '2.0491820311330723613201677166712183747407686310183466e-29', '4.54e+131', 3, 0);
    T('-9.1233177353622001049405911395689222e-55342', '-3.33952300215321746822022061363154258849401421205608044813519e+16739266293009', '3.33952300215321746823e+16739266293009', 21, 0);
    T('-4.148746111102246987268731126262166e-42223913379', '4.4e-81922479434', '-4.148746111102246987268731126262166000000000000000000000000000000000000000000000000000001e-42223913379', 88, 3);
    T('-7.7181148111853219110873092672378349791e-86939', '2.3618335583201068278084567973810207165e-2443969696', '-7.7181148111853219110873092672378349791e-86939', 99, 5);
    T('-1.332769174592036289777826070392290127524986772718452009328e+933562013', '8.4503e+720830541', '-1.33276917459203628977782607039229012752498677271845200932800000000000000000000000001e+933562013', 84, 3);
    T('2.9776547908887935910674226600402192495245642467814405469847e+27532993', '-2.2513823329555468542100269370754495631939818621460684e-92993024', '2.977654790888793591067422660040219249524564246781440547e+27532993', 55, 4);
    T('2.70617724096805846145267309940337654139035613671530972e-82', '-3.03290973266675311172860755391210738e+1611811269542', '3.0329097326667531117286075539121073800000000000000000000000000000000000000001e+1611811269542', 77, 0);
    T('5.102905605201391283899073442170170614640284933972046e+97957779', '-5.38160183385662694151405779942275355092636622119114744569958e-90581271', '5.10290560520139128389907345e+97957779', 27, 0);
    T('2.458619355971572898257349e+191', '66317292450917843.04', '2.4586193559715728982573489999999999999999999999999999999999999999e+191', 65, 1);
    T('-4.236565973635664067089792592767407238629e-2638319', '9.3408830940888386151535211140603267453288688248708e+58', '-9.3408830940888386151535211140603267453288688248708e+58', 99, 5);
    T('-4.7185523746771087491374886989494237754098554e+57', '5.2158e+6366564861171', '-5.2158000000000000000000000000000000000000000000000000000000000000000000000000000000001e+6366564861171', 86, 3);
    T('-7.785611364099390329335141217991877376499620251678e-472607729619', '1.56087279079793556384632215664237618479e+18129', '-1.5608727907979355638e+18129', 20, 2);
    T('4.99355984190e+72453177467', '-4.7514685634479937263371565e+977852', '4.9935598419e+72453177467', 32, 6);
    T('47751533726558981545995437', '-5.236932624001577452425006840478e-432954', '4.7751533726558981545995437e+25', 60, 3);
    T('-6.900449180756333050480373012186774990208527322e+3678603439', '8.908199644138635558008370513549179168129431342467941e+11080', '-6.9004491807563330504803730121867749902085273220000000000000000000000000000000000001e+3678603439', 83, 0);
    T('-4.0731801488e+927', '4.528e+58879', '-4.528e+58879', 9, 1);
    T('-9.34738287408315374509738494252821988139982e-11139', '9.3478612e-61269089', '-9.34738287408316e-11139', 15, 3);
    T('6.341011748035115908873862658422980e+101', '6.5330863143788054245113281225017190490378259763505867127e+35088683460', '-6.53308631437880542451132812250171904903782598e+35088683460', 45, 0);
    T('-6.01171939940847e+102', '-7.3216259133539346070865835658345e+32340347408707', '7.3216259133539346070865835658345e+32340347408707', 43, 2);
    T('-7.32747367463206035220594365752780174756469690914426e+92131104', '-9.1639566908445963209768833612769529e-42575', '-7.327473674632060352205943657527801747564696909144259999999999999999999999999999999999999999999e+92131104', 94, 2);
    T('-7.8528e+398530', '3.629265072486107323648917136882090156807411e+461122512900358', '-3.629265072e+461122512900358', 10, 1);
    T('-2.46882393118890633511271987036192801464889266680463e-592851363', '7.5e+3982676', '-7.5e+3982676', 13, 6);
    T('-3.04333420479085657263050221575206e+434', '-2.040172715303392525069e-830', '-3.04333420479085657263050221575206e+434', 57, 6);
    T('-4.129703726456893703254e-5244775696', '2.605361860860913822947588482505712451104113193923794524e+5462', '-2.60536186086091382294758848250571245110411319392379e+5462', 51, 1);
    T('-9.5879739973438079057931e-727', '7.91344831321449369235e-25446', '-9.587973997343807905794e-727', 22, 3);
    T('-9.220981366415211743429855756374446364367271372251e+18811', '6.0185221e-848490069', '-9.220981366415211743429855756374446364367271372251e+18811', 96, 6);
    T('8.13437251533637594764035285423394382240360865926115764994199e+858445955414315', '3.021e+11840931820', '8.13437251533637594764035285423394382240360865926115764994199e+858445955414315', 64, 0);
    T('-4.34064795137128524491064e-260483967045', '1e-939622094', '-1e-939622094', 63, 1);
    T('-9.4924941989453309997289321e+9151940010', '-6.5120552471527423874926257900321050e-89571215', '-9.49249419894533099972893209999999999999999e+9151940010', 42, 1);
    T('7.47597271704821188661e-54', '8.856488463108803e-108722069239900', '7.47597271704821188661e-54', 77, 6);
    T('5.238637735620290542839e+18325865', '-4.2956063642327545687841179029689277089e+86309055494962', '4.2956063642327545687841179029689277089e+86309055494962', 45, 4);
    T('5.863525922510657699864645807028369002227194332843659354188e+6706', '5.8719e+13', '5.86352592251065769986464580702836900222719433284365935419e+6706', 57, 2);
    T('6.202738464923563525792e+3664', '757454933924.1', '6.2e+3664', 2, 5);
    T('-9.7876195e-832381', '-1.4482090849925918493138474e+7608322', '1.4482090849925918493138474e+7608322', 54, 0);
    T('-4.883128260782764363686200258351881484e+2982353324970', '-9.360902685880161e-235948664317548', '-4.8831282607827643636862002583518814839999999999999999999999999999e+2982353324970', 65, 1);
    T('4.365081329654040292466243412557546121065905127425e-6482339545046', '7.93123279082067e+2890935913', '-7.931232790820669999999999999999999999999999999e+2890935913', 46, 1);
    T('-2.4151101832375663775795105079316714527234245329073804464e+679743748', '5.586649826929901555794343267851385317555655384204814986e-3970633334957', '-2.4151101832375663775795105079316714527234245329073804464e+679743748', 95, 2);
    T('6.54e+4065755941', '-4.057410790e+30595327301', '4.05741079e+30595327301', 98, 1);
    T('-8.7449799559498917e+1897708', '-9.89092945225495432781167066345862071104804340905293183752304e+1516', '-8.74497995594989169999999999999999999e+1897708', 36, 1);
    T('-8.39132913051150580649507429843329889703956220e-42', '-7.238397267738962346904837964740987e+420908', '7.2383972677e+420908', 11, 3);
    T('-7.538358257e-5431759', '-4.268237828901606009340592028970811953462865071021706076e-5701', '4.2682378289016060093405920289709e-5701', 32, 0);
    T('3.730445949055418425779534840644808591345160131774882957605e+1753759605224', '5.729e-664170213', '3.730445949055418425779534840644808591345160131774882957605e+1753759605224', 60, 6);
    T('-2.310700118157784103681126045407019683258391475e+53', '6.8861184257222602301e-437167998', '-2.310700118157784103681126045407019683258391475001e+53', 49, 3);
    T('-4.1379775118991746999861247638454e+348', '2.0229236e-529', '-4.1379775118991746999861247638454e+348', 86, 2);
    T('5.285576265111572919496082349212742698302979938686e-777668935', '-3.860130104718177458e+9572898', '3.860130104718177458e+9572898', 74, 4);
    T('-9.26773770932906488646662334977803662207482485315e+7059845809', '-7.14727087e-5081769407', '-9.2677377093290648864666233498e+7059845809', 29, 3);
    T('-3.56774712605752536423830e-9387552', '7.289703034354789e-2579522267', '-3.567747126057525364238300000000000000000000000000000001e-9387552', 55, 0);
    T('-6.46138663188617774190446e+914418437', '-7.2433911980e-6888', '-6.4613866318861777419e+914418437', 20, 5);
    T('9.4079106999687203e+3056423676', '-6.64045268359758664268721943275029724373586125615365569842e+40552570349397', '6.640452683597586642687219432750297243735861e+40552570349397', 43, 4);
    T('9.750277675e+17', '-6.475653467275843453723533415473121e+122', '6.475653467275843453723533415473121000000000000000000001e+122', 55, 0);
    T('2.91198471916642039546e-9020', '-6.76880601877853383102287130684982787785e-38980463', '2.9119847192e-9020', 11, 4);
    T('-2.3865607848148910106129329464420703434934211900606e+69224', '4.40168964012834711708128293083772340182866674e+413240669', '-4.4016896401283471170812829308377234018286668e+413240669', 44, 0);
    T('-1.47783170714634196565651327359660227434845170179e-450036155401', '1.2741444818998173984784018105732052282892049068377247764491e-222446', '-1.2741444818998173984784018105732052282892049068377247764491000000000000000000000000000001e-222446', 89, 3);
    T('5.237546679029340654e-83633', '-4.05016832351e-715', '4.05016832351e-715', 82, 1);
    T('5.91392726946721178352797554e+5490891', '-6.427995979931298752338e+505300626266', '6.4279959799312987523380000000001e+505300626266', 32, 2);
    T('-1.215126040489444095e-2243599641', '-6.02587293783424e-298011674377513', '-1.2151260404894e-2243599641', 14, 6);
    T('-8.249600063569831037335072459359133019046120315636e-7099472', '9.51865886424562284465746e-359398181', '-8.24960006356983103733507245935913301904612031563600000000000000001e-7099472', 66, 0);
    T('6.19760e+11', '8.4101986232763699167984609842782109462118340e+6663945288', '-8.410198623276369916798460984278210946211834e+6663945288', 54, 3);
    T('6.8893458101303761338797404e+30', '-4.545124832770500611742633954783853518667946638002e+2396', '4.545124832770500611742633954783853518667946638002e+2396', 99, 4);
    T('-59692719.906354', '9.31733748700887457518848239767163966855376e-713351', '-6e+7', 1, 3);
    T('-6.1860224203093247234771369322974088195219451675e-74747996694377', '5.87112376576716e+610585177113305', '-5.87112376576716e+610585177113305', 30, 2);
    T('4.073056544089467149451403577625995531835965575090210e-72552763935230', '-5.70567072672158e+68461510', '5.70567072672158e+68461510', 45, 5);
    T('8.396339105523533645204498792498795569617412e+785245817113', '5.5909987028963813687414328511001736199676824193e+40913458880563', '-5.5909987028963e+40913458880563', 14, 1);
    T('6.95678788011025321754040691344668626335721763821467405e-349240111471', '-6.13727468579730708501182117885960099375225661639385087e+24688222368922', '6.1372746857973070850118211788596009937523e+24688222368922', 41, 5);
    T('2.582331412022007048123963392267087581814680477553220990782e-56099', '2.8340483395267e+147927940495', '-2.8340483395267e+147927940495', 54, 4);
    T('9.4856683670099876847442460836221281962590919744080e+96899195896927', '-3.030138564e-705', '9.48566836701e+96899195896927', 13, 4);
    T('-2.94919610127213009486449040176271421311e-8734057175967', '3.586903868723996437e+88', '-3.586903868723996437e+88', 61, 2);
    T('-5.07440807973539e-202178144819', '2.68532792388848872535002250645759991806855726738247755740889e+5648077586773', '-2.685327923888488725350022506457599918068557267382477558e+5648077586773', 55, 0);
    T('7.60370966420034059191143277883190861e+44', '3.696100857424114532594674250106757655694168821511e-59216772', '7.6037096642003405919114327788319086099999999999999999999999999e+44', 62, 3);
    T('-7.63688756616220908357416933895814828716636173619444506605e+203274852902', '7.3026375584265576350477364169637835357294e+421310871', '-7.63688756616220908357416933895814828716636173619444506605e+203274852902', 63, 2);
    T('-2.1611259023653037828867304864943148630809149157e-81644785', '-9.3408016269840454284234183e+437365735', '9.34080162698404542842341829999999999999999999e+437365735', 45, 3);
    T('1.46568239545221843346051578306176579272204531651251771682e+84359405', '-6.035710063724680451430917460764708911610296891054736532e-55511979232', '1.46568239545221843346051578306176579272204531651251771682e+84359405', 95, 1);
    T('-6.308171392723362575333753036983515971824e+46', '-2.4332340453779538e-9004', '-6.308171392723362575333753036983515971824e+46', 54, 0);
    T('-6.04e+45368676929813', '1.62353626097142294088851090e+542495485', '-6.04e+45368676929813', 28, 1);
    T('4.23716394714002119220713501657338775590553645113209128e-293196233138', '-8.674696578754317301435271168541352707e+50290826780925', '8.674696578754317301435271168541352707e+50290826780925', 95, 1);
    T('3.9469739889647388278899985839747914897802227222258e+21626242096739', '-64765328050557788419670304626106443542.53391', '3.9469739889647388278899985839748e+21626242096739', 32, 4);
    T('-820236207964078777165486399849638156772.469916686', '-6.483068e-83172674204069', '-8.202362079641e+38', 13, 3);
    T('9.9101499067166205064656832092e-6876', '6.238621766547835451219740601707399285870829021580626261917e+1242', '-6.23863e+1242', 6, 0);
    T('1.04255587646038642938858711492666071173095e-98329539003', '8.5847381089900244512787249e+450', '-8.5847381089900244512787248999e+450', 29, 2);
    T('-6.40616055581427092297270309631966297739654559414573425394e-178902235354', '-8.6944073448251018524277853576317e+40', '8.6944073448251018524277853576317e+40', 99, 2);
    T('6.4993864322458112425076884829678445618922456641573378408e+89372334', '-5.20434694056558997425227413131788316187787e-118084851', '6.499386432245811242507688482967844561892245664157338e+89372334', 52, 5);
    T('-8.444e-702', '9.42945370597966245097008899512169837168142946e+12058', '-9.42945370597966245097008899512169837168142946e+12058', 58, 5);
    T('-4.676850433335625e+8340991858', '6.18721046176357220341728790447200477195163113449646105208e-478321887006847', '-4.676850433335625000000000000000000000000000000000000000000000000000000000000000000000000001e+8340991858', 91, 3);
    T('-2.0874318164298780205314e-965864', '9.19050324929694084081244723347392051e-2602277', '-2.087431816429878020531400000000000000000000000000000000000000000001e-965864', 67, 3);
    T('3.2495745960817987630721566935877339251276100922642000777e-11409232', '-7.97741987686070688387307e-14848903', '3.2495745960817987630721566935877339251276100922642001e-11409232', 53, 6);
    T('7.8453176025969995674861358546399912013713035052904362997e+2493', '7.7036621177172936193511916719376703089009935812755458445e+111', '7.8453176025969995674861358546399912013713035052904362997e+2493', 67, 5);
    T('-5.169838650064752135633126184048380408e+810075036', '-4e-85823', '-5.16984e+810075036', 6, 6);
    T('-5.11729138e+16365149912', '-5.468963779305111465745198579962367494173208255983298808e+980968299691', '5.46896377930511146574519857996237e+980968299691', 33, 2);
    T('3.0774243072975812614585708256239199575429792e+225240640', '3.5370115130715751650e-61626019417', '3.0774243072975812614585708256239199575429792e+225240640', 86, 5);
    T('9.6254332446288002823432789883571340047e-25887715916', '-1.5721461e+411', '1.5721461e+411', 84, 3);
    T('-7.70719296002760962759443835999746417517581e+2922146912', '4.902916463700146687419025e+736428284', '-7.707192960027609627594438359997e+2922146912', 31, 4);
    T('6.7510244182001849378311e-20333989030', '-1.937424584e+10', '1.93742458400000000000000000000000000000000000000000000000000000000000000000000000001e+10', 84, 2);
    T('4.175401440092929796258922e+229404314', '-3.21096603830484371251910512e-5783877298712', '4.175401440092929796258922000000000000000001e+229404314', 43, 0);
    T('7.7196', '6e+5799452', '-5.999999999999999999e+5799452', 19, 2);
    T('7.5091007036551820484267535534591442094980406066084e-84345149668216', '8.66452e+2899701195476', '-8.664519999999999999999999999999999999999999999999999e+2899701195476', 52, 2);
    T('8.11770e-9899891274', '-6.9579278152023489558277478074502669565e+660283191', '6.95792781520234895582774780745026695650000000000000000000000000000000000000000000000000000001e+660283191', 93, 2);
    T('-8.88e+3', '2.3428716e-54750841662', '-8.88e+3', 42, 5);
    T('-4e+97', '3.67667839502267424541610493306824196e+247626964051', '-3.67667839502267425e+247626964051', 18, 0);
    T('-8.22470708584208074205486083440343843469786951325566440079e+28770827966', '-5e-8735405437545', '-8.2247070858420807420548608344034384346978695132556644007899999999999999999999999999999999999e+28770827966', 92, 2);
    T('-2.55737905433603318390228311e+5310559189', '-3.74370e+1560036868167', '3.7437e+1560036868167', 36, 0);
    T('6.943511133274806663751353547101806361559051390e-301771811958', '7.40918545487506038313361463666082978823566254574882620575e+7919515645', '-7.4091854548750603831336146366608297882356625457e+7919515645', 47, 6);
    T('5.2835098566439106e+386402490', '-7.94801813957330675909079329684793671174533672916525924949635e-8031225', '5.2835098566439106e+386402490', 43, 1);
    T('4.85545521e+4774390255', '6.916203831100972535350905864336503740659504694445e-7669', '4.85545521e+4774390255', 87, 6);
    T('8.3693300874462602361261131296650359669e-79202', '4.637171122629e-134702109', '8.37e-79202', 4, 2);
    T('7.62757653809688306601e+49922416', '-8.689495592248679428348145274061926026343647760729802501582e+6211384', '7.6275765380968830660100000000000000000001e+49922416', 41, 0);
    T('5.0096014189709891527897837301041805552379296468676421698810e-604320905438', '-2.141100177885744917484151842607275201411210752e+922370', '2.1411001778857449174841518426072752014112107520000000000000000000000000000000000000000000001e+922370', 92, 2);
    T('-7.74797523132649e+3619485', '8.055335597333669212236565171084702774795343232e-91870', '-7.74797523132649e+3619485', 20, 2);
    T('7.03076765558981771099430616786764777374824816837515403e-361506616769194', '4.116657223130e-21116', '-4.116657223129999999999999999999999999999999999999999999999999999999999999e-21116', 73, 2);
    T('-2.331187777204834958898e+61', '-1.1044255898973511160042141e-6760', '-2.331187777204834958898e+61', 90, 4);
    T('1.710871297344026284552898956e-681036', '-7.949010579019004559672375884354925e-2672017', '1e-681036', 1, 1);
    T('9.88393603529724341882249832680278031402e-718053989', '1.59420428682542125630865971689214051299746711162505082e-12097564', '-1.59e-12097564', 3, 5);
    T('1.00359467608774734616149970314485249837e-55241181', '6.803568820954080873519013474166903780833e-630519177', '1.00359467608774734616149970314485249837e-55241181', 46, 0);
    T('-9.588897910265399368e+93855', '-7.4101789980212250524655451736837282921937680620117367e-7265891811244', '-9.588898e+93855', 8, 3);
    T('-7.9951571637796645110378417158811506827562793563506e+2686838004', '-5.8103967378278477901702e+2523831986', '-7.9951571637796645110378417158811506827562793563505e+2686838004', 50, 1);
    T('3.2559e-139021610908', '8.32894968354631551394351184043e+9598396755', '-8.32894968354631551394351184042999999999999999999999999999999999999999999999999999999999e+9598396755', 87, 1);
    T('-1.619688328336477773970952725306577638748337724396e-2836249', '-5.044467085485493418897863933161491336977603759730318e+6917977800', '5.044467085485493418897863933161491336977603759730318e+6917977800', 92, 2);
    T('1.01591707529710476475397847671753978416567512746e+67615885000324', '7.72019219050957587183073897729378528741402317712632e+3962551222', '1.01591707529710476475397847671753978416567512746e+67615885000324', 60, 0);
    T('2.30777949179187961371037144690e+121', '-4.92205072262919e-84137855', '2.3077794917918796137103714469e+121', 90, 5);
    T('-9.632257710246006222016308170908295e-607644094808416', '-6.46776584144365319541411316137862720971204e-5596', '6.4677658414436531954141131e-5596', 26, 1);
    T('-9.802978484644751e-5390617', '-2.24828873725998630396756455e-28644078125466', '-9.80297848464475099999999999999999999999999999999999999999999999999999999999999999999e-5390617', 84, 2);
    T('-7.3904350205847609853665874842407224822622447079160968625e+60', '-1.3024188142127318622091655320931585802436560120980397505031e+3318', '1.3024188142127318622091655320931585802436560121e+3318', 48, 6);
    T('-1.71927693667760995524424464078e+3485', '-5.94198555593649351959966968204e+404787549505', '5.94198555593649351959966968204e+404787549505', 47, 6);
    T('-5.615148134515726334789056481335213178499891779470804941e+230', '7.2104', '-5.615148134515726334789056481335213178499891779470804941e+230', 63, 5);
    T('8.950095242452911925e+493775', '-9.984071112972138882216853640023462780774629829032e-498096178704', '8.950095242452911925e+493775', 79, 1);
    T('-7.610310998436851447316905121276945958e-7004', '1.0768869628685885e+7593253189', '-1.0768869628685885e+7593253189', 79, 1);
    T('8.6468e-51694', '3.37699e-97523', '8.6468e-51694', 10, 4);
    T('7.2183101182122811906238830821504942697561910687282557e+112', '-5.065867997825492523228884092748887816199914612e+46', '7.21831011821228119062388308215049426975619107e+112', 45, 0);
    T('4.96373e-4447950873', '-8.816485e-274075746357026', '4.9637300000000000000000000000000000000000000000000000000000000000000000000000000000000001e-4447950873', 89, 2);
    T('-8.927455488167164265674185663136442353277711930200352953e-183476914098484', '-3.83224723294221e-990519', '3.83224723294221e-990519', 87, 6);
    T('2.367434060455382023143507236562447e-61216009136', '3.69827381473755119467491860384334200127058681626454e-26593525856071', '2.367434060455382023143507236562446999999999999999999999999999999999999999999999999999999e-61216009136', 88, 3);
    T('-8.376984450440083088788426514113770315e-6135620', '3.557996686179498888684776074120683887830778749688105e-76060850433', '-8.376984450440083088788426514113770315e-6135620', 100, 2);
    T('4.9917220871790277686062825098098716867669268527e+53', '-6.609444428853033707070703906625733616626278e+74511711370', '6.609444428853033707070704e+74511711370', 26, 0);
    T('3.0e+19', '4', '2.9999999999999999996e+19', 50, 6);
    T('-9.877272709217806657983e-61457', '-2.357908778291622917700270708950912616339862517350e-9955952680', '-9.877272709e-61457', 10, 6);
    T('1.04738981427797669807284149009735035e-54794757', '-2.3395479174715005088911e+755674', '2.3395479174715005088911e+755674', 81, 5);
    T('-8.101539e+145747', '3.44121505235410985772282993914529248134371e-7993856', '-8.101539e+145747', 13, 5);
    T('-3.264309815610214e-592943670652', '7.516060286487502e+4434449472787', '-7.516060286487502000000000000000000001e+4434449472787', 37, 3);
    T('-5.14008175827567790678776262681206658620960497008e+4059631262', '-1.3596653569e+80442897', '-5.140081758275677906787762627e+4059631262', 28, 3);
    T('9.4302975473621941846533361642208114499388534e+45068', '4.261887243590228415e-3745', '9.4302975473621941846533361642208114499388533999999999999999999999999999999e+45068', 74, 3);
    T('2.2178593693403706572e-550', '3.173269687914726730666021871e-356', '-3.173269687914726730666021870999999999999999999999999999999999999999999e-356', 70, 1);
    T('-5.47572588079418225e+13482174776801', '-6.44165080219544791577926926661084e+5886582', '-5.47572588079418225e+13482174776801', 93, 4);
    T('4.720828284905243290939687e-263288230904830', '-6.96042688109929458531854114776280910016703296356680800222e+26845912', '6.9e+26845912', 2, 3);
    T('8.2642207603195647e-242', '7.6266489438950350464e+1985903', '-7.62664894389503504639999999999999999999999999999999999999e+1985903', 57, 2);
    T('5.018372e-586134995099', '6.23886914e+364345407539', '-6.23886914e+364345407539', 70, 0);
    T('5.8743244284097564495320945965784215078617297187602272071875e-649', '-5.003457979610e+64031437', '5.00345797961e+64031437', 41, 4);
    T('-2.34121224738331783791042428215e+810422544126368', '-82072.9', '-2.3412122473833178379104242e+810422544126368', 26, 1);
    T('-4.7205893548979014867075928204298e+718', '2.560846858830598351686147e+6281132244', '-2.560846858830598351686147e+6281132244', 82, 4);
    T('-2.283598296751982149198824964943e-157130368066535', '-3.58432737142749705329541491372736916732039e+6178', '3.58432737142749705329541491372736916732038999999999999999999999999999999999999999999999e+6178', 87, 3);
    T('4.871390850890031517925040082272174299032468721e-86360794716362', '-2.3739997141741516258509083271e-187392338', '2.37399971417415162585090832710000000000000000000000000000001e-187392338', 60, 0);
    T('-3.0417003789368813085328081244073358316097955e+409407590092', '6.943850468212507975366640532693156834576064e-9790314707', '-3.041700378936881308532808124407335831609795500001e+409407590092', 49, 0);
    T('9.30575578020060818066e-77370129496935', '-7.22372827690503907e+1342', '7.22e+1342', 3, 4);
    T('7.2e-5053', '9.9884368961670673055914546753020113058583213040e+1721794997069', '-9.98843689617e+1721794997069', 12, 0);
    T('-6.218741912692653293304508391555452506142e-343987', '-9.942899908e-37292232234', '-6.21874191269265329331e-343987', 21, 0);
    T('-5.68102405868288433708e-821067256198', '5.46751099141048365313810515237857038131422093422047943e-21898214697026', '-5.68102405868288433708e-821067256198', 98, 1);
    T('3.4267926337240129974671230312196102993963107919318568e+987', '7.565664147067114152931943326021709996967e-10', '3.4267926337240129974671230312196102993963108e+987', 44, 0);
    T('-9.6243644256796407342823227218633381161e+23154607', '-4.70045129872943649705773048598101527e-49918', '-9.624364425679641e+23154607', 16, 0);
    T('-9.31024290108e-274845', '-2.5043e-2685787237280', '-9.31024290108e-274845', 13, 3);
    T('9.7804500494455353778668650835323e-101575', '4.5073342072297470341430182606378549425889721977103617830e+3014015', '-4.507334207229747035e+3014015', 19, 3);
    T('-5.494386492561165826943042194582536e-56240002742682', '-1.058895918850210620854484595e+1243', '1.058895918850210620854484595e+1243', 43, 0);
    T('4.005934361980898584204950181685e-228498', '-1.038522671435587584418885027198e+114', '1.1e+114', 2, 2);
    T('-9.792725611459657969137e-3888746959', '-7e-9640557484', '-9.792725611459657969136999999999999999999999999999999999999999999999999e-3888746959', 70, 2);
    T('-4.7465609135570e-14795', '9.920945079528e-94674666', '-4.7465609135570000000000000000000000000000000000000000000000000000001e-14795', 68, 3);
    T('6.83005393133551705727889838363687948931e-4183616865', '-4.29473720530511972974940213288591262727e-5746642655515', '6.83005393133551705727889838363687948931e-4183616865', 87, 5);
    T('2.23206717261391545e+6136833789454', '-7.2561e-40447234', '2.23206717261391545e+6136833789454', 45, 4);
    T('1.1585540028611e+226290850154858', '-4.71630099946696306377074485965513143833097076827614818957597e+68038025309', '1.1585540028611e+226290850154858', 26, 5);
    T('8.74960543166781503435959729235322073907702e+216995546440', '1.958814282138379553900e+942080811572315', '-1.9588142821383795539e+942080811572315', 96, 5);
    T('1.70656482421750093370601321200194587743372417884444137157e-490898222400', '1.6418129022688820988800731136e+159', '-1.6e+159', 2, 6);
    T('2.01595904137536311325037017003413876164e+785711', '-6.249879055738014008550103846482949799678386226e-513260886', '2.0159590413753631132504e+785711', 23, 2);
    T('-6.08534658027546827576316718725150949431098167396282615e+13891', '-7.165811288523e-10616', '-6.0853465802754682757631671872515094943109816739628261499999999999999999999999999e+13891', 80, 2);
    T('-9e-4200', '7.4029179872643445254e-62', '-7.4029179872643445254e-62', 70, 5);
    T('-4.6487361998005856913255473084941443349485501586065780e+55', '-5.37e+2794652009389', '5.37e+2794652009389', 92, 4);
    T('-5.0650584462273643546605468868099850371305533821808711e+3978446939162', '1.569457e+99909', '-5.065058446227365e+3978446939162', 16, 3);
    T('-13912715645554540390862229.56', '6.4e+681715143', '-6.400000000000000000000000000000000000000000000001e+681715143', 49, 0);
    T('-8.7838e-256931037', '-6.72499979152237999977361933342269978e-9911667', '6.72499979152238e-9911667', 17, 5);
    T('-5.61162183882584298114280316002104381844258e-5711', '-7.91258270657476610146343e+31442611424', '7.91258270657476610146343e+31442611424', 27, 5);
    T('1.8421528983189681776053142334e+34', '-7.71124e-9765263', '1.842152898318969e+34', 16, 0);
    T('8.62575719512528629284364775802675355750007332255900980e+106133349', '-4.20759084237641954440441336760243595304e+131', '8.6257571951252862928436477580267535575000733225590098e+106133349', 100, 3);
    T('9.54936818805256853289838689545658868285944e-94487', '-4.80333097399735716304209319542978958549724654761579034e+12070', '4.8033309739973572e+12070', 17, 6);
    T('-4.837498110910523181452596e-934229235047344', '-4.24754943829499219049591e+28', '4.24754943829499219049591e+28', 28, 2);
    T('-3.926700088602849689488128062481980276625882937351110e+366383491487', '9.9426700975655124052505315e+65383', '-3.9267000886028496894881280624819802766259e+366383491487', 41, 5);
    T('7.3656521683457918127635305637673644648704364966960e+3854382540660', '-2.98588957686744876641610e+25', '7.365652168345791812763530563767364464870436496696e+3854382540660', 49, 4);
    T('-7.6878950639388819872403698358408283670560e-47', '6.71789639304139456104690263e-7587231334', '-7.687896e-47', 7, 0);
    T('9.4872020362420455680254113039937634700393677535045208044e-10588290453861', '2.5595791697e+14', '-2.5595791697e+14', 52, 4);
    T('-9.7731769325555759074495603492529993806090480332091e+7679657', '6.1433663879991735043465705000e-33593473', '-9.77317693255557590744956e+7679657', 24, 2);
    T('8.76487261e+4473360847403', '5.65251048872303014581229644350465274063310579510946531417e-451803057127691', '8.7648726099999999999999999999999999999999999999999999999999999999999999999999999999999999e+4473360847403', 89, 3);
    T('-4.9124758509938393680609802143381058339558769902e+348501', '3.152e+674685853475294', '-3.152e+674685853475294', 50, 1);
    T('-5.75999463694570726921467963e+41', '1.7800137099288278868436135008828524799803026420196e-2998375996303', '-5.75999463694570726921467963000000000000000000000000001e+41', 54, 0);
    T('-9.9151420997240626536454491e-14338', '-8.220491553206159377014307e+1417', '8.220491553206159377014307e+1417', 37, 6);
    T('-2.511482535097751075386944596663948147e+3890397784', '-5.926207335417826414884299370930e-465546484', '-2.511482535097751075386944596663948147e+3890397784', 68, 0);
    T('-4.29e-392843865', '-4e-7313', '3.999999999999999999999999e-7313', 25, 3);
    T('-1.2461316251448232579841910948383699132465726890303399860447e-970749990809687', '2.96063254420081462820931700361118256617994940814746e-96725965949', '-2.961e-96725965949', 4, 5);
    T('7.974e-97009', '-3.810715450891576014562011296667364952e+97620', '3.81071545e+97620', 10, 3);
    T('-2.03958790e+36564948090', '-1.72433561616310832919e-830317588806', '-2.0395879e+36564948090', 16, 0);
    T('-4.21799e+8328420085452', '1.4e+201672968361716', '-1.4e+201672968361716', 87, 2);
    T('3.71845714146347119717263064583e-348691045003', '-8.213827561336452864596071406151047925062407979e-1578297349', '8.22e-1578297349', 3, 0);
    T('-310655085.17622', '7.68579433573235215951645534694605507e-782890967017', '-3.1065508517622e+8', 68, 2);
    T('-5e-19886', '-9.832026293614e+482936', '9.832026293614e+482936', 61, 6);
    T('5.01035596995367635870159e+85794', '7.5217619003818e-17164602', '5.01035596995367635870158999999999999999999999999999999999999999999999999999999999999e+85794', 84, 1);
    T('2.006442872703003900588223097564026363906458725e-4192412859', '-3.0677011187547963e+407854325342508', '3.06770111875479630000000001e+407854325342508', 27, 0);
    T('-5.7197055095536529492281610166141561876859947928962e-8813435613', '6.503840e-12', '-6.50384e-12', 100, 2);
    T('-6.4306832838137644335170893747195288149683e+59282', '-1.9266e-99011916251456', '-6.4306832838137644335170893747195288149683e+59282', 48, 4);
    T('-6.65952548149019e+60528', '9.228446376782969141888835559259566826664e-264205059862', '-6.6595254814901900000000000000000000000000000000000000000000000001e+60528', 65, 0);
    T('-1.3272755991e+274625394370', '4.61025400423543117657836754183e+19089409736465', '-4.61025400423543117657836754183e+19089409736465', 50, 2);
    T('-1.2002406796027444185e-859345721556470', '-8.1810292125652948021289728218474882866321455e+101', '8.1810292125652948021289728218474882866321455e+101', 87, 2);
    T('-7.47170470224326561302676503160179757e+814267', '65405029432666752438424055024', '-7.471704702243265613026765031601797570000000000000000000000000000000000000000000000000000000000001e+814267', 97, 3);
    T('1.6e+91108950840621', '9.79536794982694710254905479471981695911308010876852e-302972851', '1.6e+91108950840621', 27, 6);
    T('7.08657766084501404023470057517871907543146597e-1100244335', '8.801687840447287202727680463013037e+6118776', '-8.80168784044728720272768046301304e+6118776', 33, 3);
    T('-3.2510172e-2882648', '2.178314760931981795262912e+59706283248449', '-2.178314760931981795262912e+59706283248449', 28, 1);
    T('5.2731547934308510736065697469729620524724e-349336079664', '-7.8316457231461796559290666252888431797023374177495724306213e-968063', '7.8316457231461796559290666252888431797023374177495724306213e-968063', 68, 4);
    T('5.82672456526947625089e+6505246929', '-6.26453851743460988608839263606904534098e+3264', '5.8267245652694762508900000000000000001e+6505246929', 38, 2);
    T('3.19077095038546540149071968300447897856631032e-808', '-7.8450306451441560165799840590247884096900e-68000171774496', '3.1907709503854654014907196830044789785663103200000000000000000000000000000000000000000001e-808', 89, 2);
    T('-3.5000068762346513059133359900589057594e-39674', '-3.114923588045564828402951260905943291711156929139328235e+81203', '3.114923588045564828402951260905943291711156929139328235e+81203', 79, 2);
    T('8.83352041960494937e+71790103', '4.8191785349535309519299839614690535561324944200316651497465e+21291', '8.83352e+71790103', 6, 3);
    T('-5.5728e-25', '-7.29329070348745882399495069932906132892007767291871913048e+78', '7.29329070348745882399495069932906132892007767291871913048e+78', 61, 4);
    T('-6.40663709614730124137182173744313120e-4384220955957', '7.063672890427462174147598381033945615018e+606455', '-7.063672890427462174147598381033945615018e+606455', 89, 2);
    T('-7.39462745595809865471010301188329686836981652404e+5989139', '-8.177795762269445e+71043882644', '8.177795762269445e+71043882644', 88, 2);
    T('5.575418592940731461445666025473e-949', '5.31376e-35245', '5.575418592940731461445666025473e-949', 43, 2);
    T('-9.456285901037981625937520968429267080473172e+56631', '9.2774773691456085220817170582253590298054e-4918790461', '-9.45628590103798162593752096842926708047317e+56631', 42, 5);
    T('-8.5945094163944832648492190749526304596430e-197871795', '7.060640691440e-614', '-7.06064069144e-614', 53, 6);
    T('-9.0498994513228860484707550779756e+4236386225', '-8.064890002331378595413644104e-6723330158219', '-9.0498994513228860484707550779756e+4236386225', 86, 6);
    T('-6.3511605569866399436768400821323234062845130e-4611301', '-9.285600620461702442328885513390395695450967098357352e+57', '9.285600620461702442328885513390395695450967098357352e+57', 59, 0);
    T('9.7540981978443e-610054', '8.00176511601498207027537076233277203e-7259265064', '9.7540981978443e-610054', 68, 6);
    T('-2.469363518230099798999955045596737210294e+45', '8.9543560674615093999204474501e-966709149', '-2.4693635182300997989999550455967372102940000000000000000000000000000000000000001e+45', 80, 0);
    T('-9.057287349409338477198566199308090409975416772993801281e+425878809', '-7.50510595147749949173444070401627447716425488146085e-8452720395820', '-9.057287349409338477198566199308090409975416772993801281e+425878809', 81, 6);
    T('-3.5041357792677356939359365231018876115698716783942e+3136659977', '-7.5450989384724520111e+64853051252164', '7.5450989384e+64853051252164', 11, 1);
    T('-6.56720134632021834375401135e-169', '5.19859906284255153606663247e+182794721635431', '-5.198599063e+182794721635431', 10, 0);
    T('-1.4688246534724475579426762147986719273e+2457', '-2.30355917986105660227968012682175463093448576161249357490e+6130357', '2.3035591798610566022796801268217546309344857616124935749e+6130357', 100, 5);
    T('4.5736141976517677316144412555278625447679759311561973832e+5494714', '-6025920453149533087646697540200410037019638.06225', '4.5736141976517677316144412555278625e+5494714', 35, 4);
    T('5.2671685059061e+15604681746330', '2.2714e+87978', '5.2671685059061e+15604681746330', 38, 4);
    T('-4.92063230605007692695380615575845129434e-28735167233', '-7.446619150474863417273654531e-974412451761941', '-4.920632306050076926953806155758451294339999999e-28735167233', 46, 1);
    T('9.9873724082604727872074797921709228797612e+51029031', '9.023983475371753073914329e-220427543251', '9.9873724082604727872074797921709228797612e+51029031', 73, 2);
    T('7.3292649917557622929162514539463267551230350678766e+2373704', '6.96967526684875487509010580140e+73425', '7.3292649917e+2373704', 11, 3);
    T('-7.21844722508619025752412e+5750872858130', '-1.4555465445e-4000', '-7.2184472250861902575241199999999999e+5750872858130', 35, 1);
    T('4.057445679906067912812967325355495720756361507353718571e-59192620773', '0.000445702785606755191859824782326007976207563255259', '-4.46e-4', 3, 6);
    T('6.800730863507902433238726e+1559', '-8.666536842898272917503460128401681561751134680234189e+495975', '8.666536842898272917503460128401681561751134680234189000000000000000000000000000000000000000000000001e+495975', 100, 2);
    T('-1.296515302014388383436083e-360844', '8.5308777011362334976469265524e+31', '-8.5308777011362334976469265524e+31', 48, 6);
    T('3.26668195980715709593991599464605486179487379249223687548e+36688', '-5.65383463813801437087757664335227452e-453', '3.26668195980715709593991599464605486179487379249223687548000000000001e+36688', 69, 0);
    T('5.7523145830466151675e+90761147702', '-2.442961974610113789543959124e-205039', '5.7523145830466151675e+90761147702', 80, 3);
    T('-3.71519e-3916', '5.4717030767e-4661881', '-3.715190001e-3916', 10, 3);
    T('8.1573375065562133617886162173344325044315655137e-94', '-1.86229238775526366054134536525425940e+8637', '1.862292387755263660541345365254259400000000000000000000000000000000001e+8637', 70, 2);
    T('-1.6134276310882664317780819765244515e-395819464', '5.115401795552556021686040233534447979904193254301560e-4822640', '-5.11540179555255602168604023353444797990419325430156e-4822640', 81, 2);
    T('4.9157211882942125995765626e-5652546', '8.808686219e+127867447', '-8.809e+127867447', 4, 3);
    T('9.53990543950254049791465211e-8683434189286', '2.01732723687413672988059171471762583721340e+84193699114040', '-2.0173272368741367298805917147176258372134e+84193699114040', 99, 5);
    T('8.69099493912594353920239e-95784050183645', '9.00565798025347568398701465271e+98865823131163', '-9.00565798025347568398701465271e+98865823131163', 32, 3);
    T('-8.14486652512748366476337545933335990550787391681444946787e-336', '7.339102844192835858621033145148265e-3478790', '-8.14486652512748366476337545933335990550787391681444946787e-336', 90, 1);
    T('4.49476936844406959482063840463315592927490531355573948294144e-322195095497595', '4.9964949', '-4.9964949e+0', 79, 6);
    T('-3.124761135100527182971050136804204466784570955876923451e+4125735446208', '5.05925476641e+20', '-3.124761135100527182971050136804204466784570955876923451e+4125735446208', 80, 5);
    T('8.1e+446044502', '9.692068727411927134156697490e+84920730348307', '-9.69206872741192713415669748999999999999999999999999999999999999999999999999999999999e+84920730348307', 84, 2);
    T('-6.569948463496191215397932957270993323492242055177e+7909', '-4.264224115028600510269722078515280413066e+19353387074890', '4.264224115028600510269722078e+19353387074890', 28, 1);
    T('-3.52687651702729008858730566213e-6779', '1.376174250527624643373945558696405158811641192887375903765e+991816030511303', '-1.376174250527624643373945558696405158811641192887375903765e+991816030511303', 75, 1);
    T('3.171666891753888280629461e+851', '1.919551215718028230320802710899729085495548037016602e-250678836', '3.171666891753888280629461e+851', 59, 2);
    T('6.345956097694190555927046894310530582062e-7008', '1.920050984e+21466829717468', '-1.92005098399999999999999999999999999999999e+21466829717468', 42, 1);
    T('3.6391039323502266568075130754260569667436e-387632282', '-8.8480726366858906244377245430462917960550697588000e+557038', '8.8480726366858906244377245430462917960550697588e+557038', 51, 1);
    T('3.250532039871657853067700925372928220416e+1265764', '-8.195246632539832590369862566276072976e-81057258189950', '3.250532039871657853067700925372928220416e+1265764', 42, 1);
    T('-4.79921364416003604327453325259803079161813024231052e-444013155138', '2.077508742650e+12146320482', '-2.07750874265e+12146320482', 86, 6);
    T('-6.6978688811736430487296538865888487115289719e+97853198', '2.94588048735119371011875229371318363615737121677089148213e-74509', '-6.697868881173644e+97853198', 16, 0);
    T('3.32514302121491194423750321692270996417832487202344e-3823', '3.32323962746396181207912884e-693440005298179', '3.325143021214911944237503216922709964178324872023439999999999999999e-3823', 67, 3);
    T('9.81e+5448760572201', '-4.5745069582010711528773e+321', '9.810000000000000000000000000000000000001e+5448760572201', 40, 2);
    T('2.16239764604623840825019417875162e+844695022', '-2.2390333561e-5778633258', '2.16239764604623840825019417875162e+844695022', 36, 5);
    T('2.9319003155447467069050595644795776718029283800598184e+124', '-7.24836724094999e+646616', '7.24836724094999e+646616', 18, 4);
    T('-4.58290134876556851224904782109671337065152147e-32230495190', '-1.03087401741399278779824384903831951521e-14857596', '1.03087401741399278779824384903831951521e-14857596', 89, 2);
    T('-5.2080016174704055284954e+87918', '1.4986062492620930606859277163608e+2022334', '-1.498606249262093060685927716361e+2022334', 31, 4);
    T('-9.696541507e+75714787241', '7.48114961e+170147', '-9.696541507e+75714787241', 61, 5);
    T('-9.26643657953033170979986029949367987938635052286e-9487206', '4.99376333924004487624046686017151558445487867442e-83405', '-4.99376333924e-83405', 12, 2);
    T('6.843640208954603496210217e+428', '-7e-759207', '6.843640208954603496210217e+428', 78, 1);
    T('2.0061748e+706805865321544', '-9.1104394440630179934413906970622319920175344239314e-11300103', '2.0061748e+706805865321544', 75, 6);
    T('-8.4555090403154043515736506369e+37', '4.328561094852291605606065531940369e-319214556400025', '-8.45550904031540435e+37', 18, 2);
    T('-30817.4', '-5.796092723403681927268029603204786383390093090951197e+25352', '5.796092723403681927268029603204786383390093090951197e+25352', 69, 5);
    T('-1.8023690719e+34275626', '2.01481459227968576556917796426489806e-428844197315410', '-1.8023691e+34275626', 8, 0);
    T('-8.9500915915549033e-84207679', '2.156840204085785255502688598616e-50130', '-2.156840204085785255502688598616e-50130', 42, 2);
    T('8.45112529454394266e-782469313458037', '1.550366728296152760585755743696756728916215773358939840748e+4145', '-1.5503667282961527605857557436967567289162157733589398407479999999999999999999999e+4145', 80, 2);
    T('-9648852.92138', '-9.61320554086e-900', '-9.64885292e+6', 9, 6);
    T('1.762750103154936e+4445', '5.70836841746215897489e+2038632', '-5.70836841746215897489e+2038632', 94, 3);
    T('-6.317e-24742', '8.8467715652303836953718e+176407642919', '-8.8468e+176407642919', 5, 3);
    T('-9.4897e+71893470989', '-5.572297811e+88242633157', '5.572297811e+88242633157', 95, 4);
    T('-4.41015564876879634427522725949181553231725e+8572985897518', '-7.330145000653379481489211163e+63551629871', '-4.410155648768796344275227259491e+8572985897518', 31, 1);
    T('8.678e+4357', '7.809051928899405345185116681273428009491966702394126759148e-687804332643239', '8.678e+4357', 62, 0);
    T('-4.6051612300077737480441618e-521', '2.7093377462528687346528610634132096092112367173e-4555803', '-4.6051612300077737480441618e-521', 75, 4);
    T('7.69499682064731813453394e-5711358', '-6.540449148492285388162e+3780780193', '6.540449148492285388162000000000000000000000000000000000000000000001e+3780780193', 67, 2);
    T('-3.53094901e+756513364747', '-6.16709754739e-45115032305', '-3.5309490099999999999999999999999999999999999999999999999999999999999999999999999e+756513364747', 80, 2);
    T('2.79298412326985624206308814484443486017745599588424597703e+48694', '-2.40675799554640829102872494148347734227522210464824739e+413', '2.792984123269856242063e+48694', 22, 6);
    T('2.92741030786128702055074606117039220418215620215896969752e-8947', '1.43772536245255048104e+41527', '-1.43772536245255048104e+41527', 53, 3);
    T('-6.12531353879018123097e-21034', '7.32089565086316608808893223864500831972803857e+1192043827573', '-7.32089565086316608808893e+1192043827573', 24, 1);
    T('7.5551183506835137548078514208067709872279071347e-721', '-2.8222975579948183718005413352589117673875914072e-856', '7.556e-721', 4, 0);
    T('9.378862e-19', '-5.43312638454635909089763117244377984624176378e+44818', '5.433126e+44818', 7, 5);
    T('-14437197140674995467827487.20320928397982881106973', '-1.0621032277192842571352529092041556000229841252e-36395504026782', '-1.443719714067499546782748720320928397982881106973e+25', 95, 5);
    T('1.121586291161605638153693715343568504415815360e-39966', '-9.799592003373589340968107769296749018425627598205e-87281809', '1.1215862911616056381536937153435685044158153600000000000000000000000000000000000000000001e-39966', 89, 0);
    T('-9.9834922232798e-70023243037', '-2.02195693937717933252256246010e-6573244802512', '-9.9834922232798e-70023243037', 82, 4);
    T('-1.3539301275179e+2865', '3.23271118e+710180997', '-3.2327111800000000000000000000000000000000000000000000000001e+710180997', 59, 0);
    T('-4.6784161273482528e+49736357895', '1.42332138843785028035262886461e+98317411', '-4.6784161273482528000000000000000000000000000000000000000000000001e+49736357895', 65, 3);
    T('3.7725577e-27498782535', '70534572730494872318048375169506167.942233', '-7.05345727304948723180483751695061679e+34', 36, 6);
    T('-5.52377e+3047', '9.8553537601811645561518892952e-13236869743', '-5.52377e+3047', 43, 4);
    T('8.27e-2345800', '-9.97450674788625140212711493193865692623e+30300930175923', '9.9745067478862514e+30300930175923', 18, 6);
    T('-6.6827266059263012613e+24', '-6.3e+2949248839', '6.3e+2949248839', 95, 6);
    T('-8.62766715362350849438692e+2603719575', '-2.045643822468122834991628538227002e+80010652791', '2.0456438225e+80010652791', 11, 6);
    T('-4.084061687630852e+22', '-2.312939867e+658284', '2.312939867e+658284', 79, 5);
    T('9.197121264670e-53852009962925', '2.7e+847138627238', '-2.7e+847138627238', 87, 4);
    T('-4.3737161760800e-5224602440', '-5.92129e+337', '5.92129e+337', 81, 5);
    T('3.1493132e+5743107376803', '-3.246478472228e-2828131660698', '3.1493132e+5743107376803', 93, 5);
    T('0.000035614', '9.40897001609380203488197534772663989443888926473549406507382e+115', '-9.40897001609380203488197534773e+115', 30, 0);
    T('5.2998290227899600454385e-1601359539', '4.086066806205863792567683716806555e-6149499728', '5.2998290227899601e-1601359539', 17, 0);
    T('1.40868579814140447296342680211922855769660487523525175779e-21041087464', '-4.782967509493247857027284872631221071845425e-318826', '4.78296750949324785e-318826', 18, 3);
    T('6.2916417068546555e+17651531', '2.27086777333609968290184e+31', '6.2916417068546555e+17651531', 56, 2);
    T('9.21519974108494269199e-585150531', '-8.50646281062860123e-7005443016426', '9.215199741084942691991e-585150531', 22, 0);
    T('8.835e+63887460', '1.49682228e+84700603605820', '-1.49682228e+84700603605820', 22, 5);
    T('165414410888904858046029628290397181060.5131', '-4.136269276561887956809877e+33', '1.654185471581814199339864381673971810605131e+38', 60, 1);
    T('2.8445264195350421011518e-73332733548236', '-2.6e+987602983998', '2.6e+987602983998', 59, 6);
    T('-5.6560208e+2935325539039', '2.9446426482e-793423746921613', '-5.6560208e+2935325539039', 86, 5);
    T('-4.1696106180025237023017844926864908430519562601857e-611209', '-4.774818322226975525601105071395376463412147632305492971613e+59', '4.774818322226975525601105071395376463412147632305492971613e+59', 96, 2);
    T('-7.401413536278320856723514060872754044259982755e-9781474', '3.93460852059780816593e+764', '-3.93460852059780816593e+764', 80, 2);
    T('-3.838351308241e+3599237', '-6.343135433728974889593278362912835656139915524e+899600234917', '6.34313543372897489e+899600234917', 18, 6);
    T('-2.792609296e+434', '-5.5399609106675725444581380753254696334269351e+1334715591', '5.539960910667572544458138075325469633426935e+1334715591', 43, 3);
    T('2.38782030114352615724003451383035329089e+64230719690', '5.792657041096460154778020788925e+31', '2.38782030114352615724003451383035329089e+64230719690', 86, 0);
    T('-5.969985100596703475378092351e+87199362', '-2.6062260399830383651e+945009', '-5.969985100596703475378092351e+87199362', 95, 0);
    T('-1.1e+4393451586766', '-3.436517e-83236', '-1.099999999999999999e+4393451586766', 19, 2);
    T('-3.0615576680042764056413645726687835673706122614e+737177400419488', '-5.847026188786107317683802780631457e+57749070140', '-3.0615576680042764056413645726687835673706122614e+737177400419488', 78, 5);
    T('-2.959867e-8081022', '-6.0865726626839680218127202056812856710857434691895465006491e-753211256294', '-2.959867e-8081022', 93, 6);
    T('8.33892628339157e+38', '-192908853420928229736222175.39880', '8.338926283393499088534209282297362221754e+38', 41, 5);
    T('3.46875258300701419506975409e+499515571', '-5.7974662302593238057528908411622439371579810e+110', '3.46875258300701419506975409e+499515571', 44, 4);
    T('-4.27399841701994205628553537837973970296205380146571e-7574', '9.205031767148606411747470731821290967122325e+30747140825739', '-9.2050317671486064117474707318212909671223250000000000000000000000000000001e+30747140825739', 74, 0);
    T('6.531179968811e+86918', '-2.06169411803152956780940658307063938e+6525190567', '2.061694118031529567809406583070639380000000000000000000000001e+6525190567', 61, 0);
    T('8.39268733891513573782390294679692785635511362965898693581425e+21557633004', '5.476829399017573432129094420814592026486987764589804813e-5845', '8.3926873389151357378239029467969278563551136296589869358e+21557633004', 56, 1);
    T('9.120900593741450411895e-220122', '6.60871277012369283105579213607850725465827297322763430558e-53657', '-6.60871277012369283105579e-53657', 24, 6);
    T('-4.34097033e+16', '-9.18419105968477059904e+26', '9.18419105925067356604e+26', 34, 6);
    T('-9.89e+212', '-2.218293252585569929421922156840279934190838858e+2068293657139', '2.2182932525855699294219e+2068293657139', 23, 5);
    T('-2.79962e+9886837895', '2.1313116469577476186062077958526099026734583e-23', '-2.799620000000001e+9886837895', 16, 0);
    T('-4.74351521927140290767113476e-9398942', '-3.237269254744538378420014311765888807e-129698312918', '-4.74351521927140290767113476e-9398942', 39, 3);
    T('-6.4319819101607152812373e+51138701948520', '6.155660403089650404608175257740e+1106187025833', '-6.4319819101607152812373e+51138701948520', 67, 4);
    T('-3.8482621353410973e+55021018474638', '1.991887031053053398586911808583503650785270008e+26287373', '-3.8482621353410973e+55021018474638', 23, 5);
    T('9.701117894e-982503099959', '9.0686792441778808888342861122e+59405', '-9.06867924417788088883428611e+59405', 27, 5);
    T('-5.8148291216931112435652751426289833222287568e+6922768683', '-9.47209453536239690069e+324630010888', '9.47209453536239690069e+324630010888', 76, 2);
    T('8.2881023678416129238094955294082963116486992715e+10293744215473', '9.38753241516684e+16', '8.288102367841612923809495529408296311648699e+10293744215473', 43, 6);
    T('7.032362022801793058130e+368440014151', '5.12982472105582364640185696409382624871754503146e-82370902961', '7.03236202280179305813e+368440014151', 90, 4);
    T('-5.23e+75802274795655', '5.444570078497848791700680431938054606033485204846530e+4590332431', '-5.23e+75802274795655', 82, 2);
    T('-5.8735e+73', '5.586298588948431114626054469449038238462532135353e+566068771590', '-5.5862985889484311146260544694490382384625321353530000000000000000000000000000000000000000001e+566068771590', 92, 3);
    T('-6.598262963197770416e+8419163441', '3.5761441418496137623766854999906929084471528835582e+44276955344342', '-3.5761441418496137623766855e+44276955344342', 28, 3);
    T('-8.377795136052201535105579274669e+510589736187430', '2.64905905493411438e-6981', '-8.377795136052201535105579274669e+510589736187430', 33, 5);
    T('-9.07366940630481903187326259320372634842531735e+1949131488182', '1.11703751670769e+434944546814', '-9.07366940630481903187326259320372634842531735e+1949131488182', 45, 2);
    T('5.8643125488229616736245014529281261941101074141284285228e-70803285312622', '7.2472e+5847', '-7.2471999999999999999999999999999999999999999999999999999999e+5847', 59, 2);
    T('3.20937198424802765142247155519153096009515470191436170654e-7912138609136', '3.25662607524059803528957371641303095266271048092405889337e-333745', '-3.25662607524059803528957371641303095266271048092405889337e-333745', 100, 3);
    T('6.76585765539291143539e+18808776452', '3.6410672450977629481414221109648503977e+252133491050', '-3.641067245097e+252133491050', 13, 1);
    T('-8.30021e+6325929240', '-7053834274549921258502585346665640819.725', '-8.30021e+6325929240', 71, 5);
    T('8.534862295708721264324291e-9053973473124', '2.3450455399e+24', '-2.3450455399e+24', 50, 5);
    T('-3.244829987726541747897056343745838805639e-71835993170689', '3.1841613744249790729224865823e+580539', '-3.18416137442497907293e+580539', 21, 3);
    T('1.29735605016022402458041604691452059231510156874225883707938e+812943359607', '5.77500054294580350e+31186398352469', '-5.7750005429458034999999999999999999999999999999999999999999999e+31186398352469', 62, 2);
    T('-7.59852804457412707183967318133782343994162248872408841755060e+1634304693', '-7.4118087234842499822356371859e-367084995197099', '-7.598528044574127071839673181337823439941622488724088417550599999e+1634304693', 64, 1);
    T('5.50807e+1586', '-5.846573420455900151881714665348648659492414514877959052e+129', '6e+1586', 1, 6);
    T('-1.34946610422538646161509050561150075261180019080443962560e+8397763058', '7.2243147652580795086830708264878181e-9457437986', '-1.3e+8397763058', 2, 4);
    T('15590.3', '4.147044467688996526789137409e+7539141623', '-4.147044467688996526789137408999999999999e+7539141623', 40, 2);
    T('4.995044838270221507553437217712843325980921757757405e+5479699', '166657550.7678136', '4.995044838270221507553437217e+5479699', 28, 3);
    T('-6.96849088e+954178733217460', '4.62357574e+692', '-6.96849e+954178733217460', 6, 2);
    T('8.6962958e+748821306', '-8.52301420105211199917776463494846133213241938649783e+96737720278935', '8.52301420105211e+96737720278935', 15, 1);
    T('1.6604605607341948418121596098571897404568e+312227625806', '-1.0486542869781206278911593973595e-96215069', '1.6604605607341948418121596098571e+312227625806', 32, 3);
    T('-5.195111568497893827872324598703391563095809222562163904658e+9407102', '3.634805189843634700082687637895297225858203e-511973244442185', '-5.195111568497893827872324598703391563095809222e+9407102', 46, 1);
    T('9.28230480791920e-336701591025606', '169178918028628227984.452031', '-1.6917891802862822798445203099999999999999999999999999999999e+20', 59, 1);
    T('-4.2557020338589975011946376002220791571853957987194300933e-51', '11482203630063747.708262758', '-1.148220363006374771e+16', 19, 5);
    T('-9.4511673002570127241988430761196322316435438166e-7025397', '-4.7609652171111828563981533266336278401813876748317157e+35900681127', '4.7609652171111828563981533266336278401813876748317157e+35900681127', 83, 0);
    T('4.65086477413967207868829071657757898248e+40629646122', '8.997246503581785e+3237', '4.65086477413967207868829071657757898248e+40629646122', 52, 4);
    T('7.097106192769555187080306274122056153597001e-9783987124', '3.527987569039241328639500303849689002115630393901902294e+73806487', '-3.5279875690393e+73806487', 14, 0);
    T('-6.724769103257448468638066558579574176121676051388490e+4259644306', '-8.3767164027398648434216238269742968375740804134971949492e-8', '-6.72476910325744846863806655857957417612167605138849e+4259644306', 89, 5);
    T('-3.71774284363312849333507544410039611153562e-7666184228297', '-8.74273e+43687170801', '8.74273e+43687170801', 8, 4);
    T('-8.8731724504e-91192', '-7.812940645261271094380418344411e+77355499766', '7.812940645261271094380418344411e+77355499766', 51, 0);
    T('4.1983846095973844354002e-939175', '-1.52745506304112728418143503587714506e+88826032', '1.52745506304112728418143503587714506e+88826032', 91, 3);
    T('-5.1262952061578216295497211101959166083948818619698739322476e-42393', '-3.47335572153420528394766181654177948221643733846786578e-976097', '-5.126295206157821629549721110196e-42393', 32, 3);
    T('7.505993166567115216141596669078323155e-849405253', '1.401027995545670914901716035830526932e-11773232', '-1.401027995545670914901716035830526932e-11773232', 58, 5);
    T('-4.912291156e+58768542649', '-4.87046686924076105052368978793417879111522e+49', '-4.912291156e+58768542649', 42, 3);
    T('8.0980298704850e+578458741', '2.40796955458801081973894715725264e-94834', '8.098029870485e+578458741', 19, 4);
    T('1.58308572979461570349163759586e-143703833', '5.9768701719863092083217038561084826618533212e+819995824494', '-5.97687017198630920832170385610848266185332119999999999e+819995824494', 54, 1);
    T('-6.665222849221e-5480424', '-2.7289929508187159800212090978103914e-283', '2.7289929508187159800212090978103914e-283', 86, 0);
    T('-3.450280728463119874462947088e+62306', '8.5832615462629554662186215768596516102985012820e+53', '-3.450280728463119874462947088e+62306', 31, 2);
    T('-7.0669245168e+66', '1.0793987542e+38490707028128', '-1.0793987542e+38490707028128', 80, 1);
    T('9.20924155665126559006917975725e+116', '-6.295246069774354201454386965606194133601531721699036e+3032736', '6.29524606977435420145438697e+3032736', 27, 5);
    T('-1.82185624813727786189e+66940', '-5.88065294809329128815069381117141564811200521823380514e+258343429258', '5.8806529e+258343429258', 8, 3);
    T('-7.9661833628252395459e+991', '9.047883330257254582450775479624194854924732544906950e-3066', '-7.966183362825239545900000000000000000000000000000000000000000000000000000000000000000000000001e+991', 94, 0);
    T('-4.4337506471528242931185648184124e-839226548688', '1.37097711479202809548535888374660034e-60559433295350', '-4.433750647152824293e-839226548688', 19, 6);
    T('-8.858021397301083081222560347890638701875177530268350e+200', '6.79633282896704886830372318324197928133754923095736097502661e+7028', '-6.79633282896e+7028', 12, 1);
    T('-3.875076584e+16581', '5.295255182189248538370629e+1287287511', '-5.295255182189248538370629e+1287287511', 25, 5);
    T('6.4041658614950268985e-806135708364603', '7.060775661289320485315235167220398917878974383843e-99495931364', '-7.060775661289320485315235167220398917878974383843e-99495931364', 89, 3);
    T('-7.611e-689444366', '3.278718863108146378174223342698270090e+9961284967', '-3.27871886310814637817422334269827009000000000000000000000000000000000000000000000000000000000001e+9961284967', 96, 3);
    T('2.762822486e-720982', '-1e-132', '1e-132', 22, 1);
    T('3.97908206436001516755265446145868326935819304680855719e-89380594', '6.87378020497091072277614883567592062924818297040e-30998491204930', '3.97908206436001516755265446145868326935819304680855719e-89380594', 95, 4);
    T('4.42340025408e-39804703467', '2.47962636898006003666086322231045277187505459666703e-65509', '-2.47962636898006003666086322231045277187505459666703e-65509', 89, 3);
    T('-1.72707245753022699752509700676029892643425344318956350705375e-275', '7.25e-45613920045', '-1.72707245753022699752509700676029892643425344318956350705375e-275', 61, 6);
    T('2.90339061657377e+9156566815', '5.5e+903368139311324', '-5.49999999e+903368139311324', 9, 1);
    T('4.729416424526254611551541e-91248', '855084227409419348927670653379543801034.613526', '-8.55084227409419348927670653379543801034613526e+38', 57, 0);
    T('6.6650887779380682678092026778532681876647e+92947517', '2e-59136213447', '6.665088778e+92947517', 11, 0);
    T('-4.6167781777742753e-949718734053542', '4.2459547e+618359', '-4.245954700000000000000000000000000000000000000000000000001e+618359', 58, 0);
    T('1.084176446333587552144262067531710328950030e+56920494725', '3.94731738343804995018464104371340484297204089e-63898727150409', '1.08417644633358755214426206753171032895003e+56920494725', 99, 2);
    T('6.2791026791600986449e+9945888201', '5.494457136029166142324576717851845093332242373212875843677e+3679', '6.27910267916e+9945888201', 14, 3);
    T('4.8724711902062270523212055146681935121058e+8897632802961', '-3.3034391886075783331710345225806857e+43', '4.87247119020622705232120551466819351e+8897632802961', 36, 4);
    T('-3.50501795862851087681772639355887890028406022463e-24999', '-4.516801265525557970897059623309142038661892791468770342802e+682', '4.516801265525557970897059623309142038661892791468770342802e+682', 100, 0);
    T('6.3208e+59516294', '-7.79817738775855e+156360564466936', '7.8e+156360564466936', 3, 2);
    T('-5.79e-87', '-5.693517060816647030774672647367069989517078433030079e-716', '-5.79e-87', 88, 6);
    T('4.131666561e+11808499', '-4.378008587360636607294103613962130750370986265093e-4308320742', '4.131666561e+11808499', 93, 3);
    T('-1.383300e-21', '7.912337536680715080825850118051377e+5308173918487', '-7.912337536680715080825850118051377e+5308173918487', 61, 1);
    T('-1.5975303841587973836272566913035775773739270580615502623495e+55926', '7.44552020625071819752406542e-45297487', '-1.597530384158797383627256691303577577373927058061550262349500000000000000000001e+55926', 79, 0);
    T('-3.59279761e+2811', '-9.08055922807807845779522448565218e-90953425586176', '-3.592797609999999999999999999999999999999999999999e+2811', 49, 2);
    T('6.0912773871843399418172527997886597475e+1034', '5.2131386145153473015758820940848047995e+80010673', '-5.2131386e+80010673', 8, 2);
    T('-9.66788789909741089094619606226583054670579560e+1232', '-1.757663250201721481283709935076197e+76941', '1.757663250201721481283709935076196999999999999999999999999999999999999999999999999999999999e+76941', 91, 1);
    T('1.93496371856505099462902814563451078898853824677e+613324314', '7.136624611845534763963415206e+389647171341', '-7.136624611845534763963415206e+389647171341', 31, 4);
    T('-4.000877673574404110e-39699029', '3.3689993025562394728130985963339556985e+8137879430082', '-3.3689993025562394728130985963339556985e+8137879430082', 69, 1);
    T('-6.9442873013367272057508e+7301536922', '5.63143065e+22291', '-6.9442873013367272057508e+7301536922', 55, 4);
    T('-3.6564901724703867336252487036244133239613870991e+228', '2.52530016720790706644024968832544633e+90990', '-3e+90990', 1, 3);
    T('-5.48451e-9614741655189', '-3.783784297262322501654089142871592020e+3470', '3.783784297262e+3470', 13, 5);
    T('2.03568796242701659339889833873208e-5407960', '-8.514923304396959872313277500260520799706865172239993e+930371645', '8.5149233043969598723133e+930371645', 23, 6);
    T('4.35486203e+13', '-4.3403215310e+4716804603440', '4.340321531e+4716804603440', 90, 3);
    T('8.1e+15', '2.0107334537151830817267956482652705699511834765380922487695e+664744447381', '-2.0107334537151830817267956482652705699511834765380922487695e+664744447381', 74, 6);
    T('9.955386867576535877085976784555680548648727e-5729', '7.14558034980548113e-3090', '-7.14558034980548113e-3090', 39, 0);
    T('-1.12468962550363049681353051586038805714631224085950130354709e+61', '-3.4466172391649419329673894962029909368614e+4707408961', '3.44661723916494193296738949620299093686139999999999999999e+4707408961', 57, 3);
    T('-9.7619514936890470595614010495007871189687722012e-88154827', '5.1680729992222149850931911014808181968e+226286769316879', '-5.1680729992222149850931911014808181968000000000000000000001e+226286769316879', 59, 3);
    T('-3.2831233960260460737755535080e-20536938861085', '-7.097868429693091450877459871054056519398853209452921057e+761', '7.097868429693091e+761', 16, 5);
    T('3.79192593498058542452219e+2460583', '-7.792651209417064757961566623735e+33', '3.79192593498058542e+2460583', 18, 4);
    T('-92857203722376568218707701378110045349394296240031', '-6.432295056069654136773830402265988e+646134744', '6.432295056069654136773830402265988e+646134744', 53, 5);
    T('-76553359174969197379766020554979143522289158846', '6.46223126253280e+1764866', '-6.46223126e+1764866', 9, 5);
    T('-8.98662335965271265508359608919027298515975027709e+301789761047915', '1.445443525449004540606163044242775712704013792577315285762e-15746039634', '-8.98662335965271265508e+301789761047915', 21, 1);
    T('-4.3510109732787877e+945608705848', '6.89538033767088e+84106', '-4.3510109733e+945608705848', 11, 6);
    T('-1.49736014952784444824205075737600430665722633360e+73035495426', '-7.50352726244508549195241448963e+4057420710', '-1.497360149527844448242050758e+73035495426', 28, 0);
    T('6.3073782379180212644848368667883290e-724005796474', '-2.034931075663155882410371306186082072907646487855824e+93296', '2.0349310756631558824103713061860820729076464878558240000000000000000001e+93296', 71, 0);
    T('-2.707933367083100635537215593156012280284465075785521e+748247565', '-2.66092958831397391030605850e+448049636861', '2.6609295883139739103060585e+448049636861', 62, 0);
    T('-4.7669886e-60344477544071', '5.0058145822069992526305864e+961218337984', '-5.0058145822069992526305864e+961218337984', 67, 6);
    T('1.868399546926630754720e-719776', '3.177026175765959997601492650e+21473', '-3.17702617576595999760149265e+21473', 88, 6);
    T('-7.880145077e-515077935381481', '-9.9e+401635267760981', '9.9e+401635267760981', 46, 2);
    T('-8.7374264304947413904275811038491317e-91522137', '7.20660e+24306393858', '-7.2066e+24306393858', 84, 4);
    T('-3.30156917764985785557286588387274335992682842494e+87826', '1.7449623932700400570106938143e+977026', '-1.744962394e+977026', 10, 0);
    T('8.020880091542086558871194912252188441544676534268937255626e+7308670415', '9.74635641953658951484043339e-421224', '8.02088e+7308670415', 7, 5);
    T('-1.04442905864529695207138643464165403538466549e-486', '2.369901940e-489101086', '-1.04442905864529695207138643464165403538466549e-486', 96, 4);
    T('7.216044844307440371390389353542333610227940032160e+7635226', '6.62e-3308', '7.21604484430744037139e+7635226', 21, 1);
    T('8.20439056949615052015690775163636372988559e-390988176', '5.4515631598458441746241389533679e+27588443620675', '-5.4515631598458441746241389533679e+27588443620675', 79, 0);
    T('-1.428443229236746656890179803243593816322934165809458266466e+6266955', '-2.60096548120776363009436551208097769833478994951755547464e+9420784819075', '2.600965481207763630094366e+9420784819075', 25, 6);
    T('5.02488389e-429558921867', '-1.7292065554840991842520557769862099851991864880e+946341309519945', '1.729206555484099184252055776986209985199186488e+946341309519945', 56, 3);
    T('-4.0895443e-24', '2.6384120675315787416638994668298190429865663499616422681e+7511631603', '-2.638412067531578741663899466829819042986e+7511631603', 40, 1);
    T('-5.7906020861322385099273498519600948758697300939336867773394e-1238353626', '-4.96809399110881099837163e+6056766517', '4.9680939911088109983716299999999999999999999999999999999999999999999999999e+6056766517', 74, 1);
    T('6.7854979742316244533e+2591391', '-7.90214276594565952859748776257244658e+111', '6.7854979742316244533e+2591391', 27, 6);
    T('3.432540968588132786924845e+33492427', '-8.360503729919876025e+100029097124562', '8.360503729919876025e+100029097124562', 92, 1);
    T('9.716449e+581', '-2.406422341139199529104439734933e+658359278493583', '2.40642234e+658359278493583', 9, 5);
    T('-5.6702770815428319115631418607113779e+314940013508', '-8.67661035274480687232015861153700132e+304652', '-5.6702770815428319115631418607113779e+314940013508', 40, 0);
    T('-2.750772522393340255301235470550441429686244823631143764e+714192195', '2.140024631795718689152788008369450604362778211863e+456', '-2.750772522393340255301235470550441429686244823631143764e+714192195', 80, 1);
    T('-6.408303909613660e+24669', '-1.4402433974784722774509691622989e-919799', '-6.40830390961366e+24669', 47, 4);
    T('6.5367072533442649e-520', '3.7101961195598352896805608951088633723519022910195840639e-556', '6.53670725e-520', 9, 6);
    T('3.32366053003155585695208e+26', '6.91536061916797861671160604924953e-533', '3.32366053003e+26', 12, 5);
    T('5.92e-3456', '-9.79139447162e-531966492360298', '5.920000000000000000000000000000000000000000000000000000000000000000000000000000001e-3456', 82, 0);
    T('5.8045502274567539505141593041116e-7459715688', '9.62358577248235588577565616803577332874101955411201297214e-410291059', '-9.62358577248235588577565616803577332874101955411201297214e-410291059', 94, 4);
    T('4.867339802294684067143557801e+1456305044', '1.515792068420556264108998e+688645', '4.867339802294684067143557801e+1456305044', 99, 0);
    T('-6.927208e-6870704977281', '1.934743413335347494870983897211023977916074360132146933e-678462667611', '-1.934743413335347494870983897211023977916074360132146933e-678462667611', 99, 2);
    T('-2.666031093047908580732799264802491932050274232617e+85084529251837', '1.340591075077285234973718856114288766150e+63135335', '-2.6660310930479085807327992648024919320502742326e+85084529251837', 47, 5);
    T('-9.917887656030303836489e+25418978344186', '1.15542300256086541560753683421240448147e-5489735988', '-9.917887656030303836489e+25418978344186', 48, 5);
    T('3.8346722058465480559751993751072289526e+5439768447773', '1.4582709913009705052755188860994823516751e+7973429593936', '-1.5e+7973429593936', 2, 4);
    T('-4.34060729762004329478663160903107669321405503327e+545792493135455', '2.2e-316226', '-4.34060729762e+545792493135455', 13, 5);
    T('-5.65e-1521972123', '-8.30953763492477243300648372621250892600388173946207914165e-9323047873', '-5.65e-1521972123', 79, 3);
    T('-8.05862994631678007e+8416791570342', '9.394129819371316948e-381275068', '-8.05862994631678007e+8416791570342', 56, 6);
    T('-6.3867453240635208069101596777656622587333231591875e-360', '-3654051969789.8360303', '3.6541e+12', 5, 2);
    T('8.62630485575e-596', '1.9e+1519484', '-1.9e+1519484', 76, 4);
    T('-4.162693251133747471135908702e-972089592', '-6.1386599022813335450885504113254054696786898e-319960', '6.1386599022813335450885504113254054696786898e-319960', 93, 5);
    T('-2.1265e-65339335173409', '2.927486532e+58', '-2.927486532e+58', 84, 6);
    T('-4.15906153305e+516919694399336', '1.29522694216440231529820129e+6475907245', '-4.159061533050000000000000000000000000000000000000000000000000000000000001e+516919694399336', 73, 0);
    T('-1.663007748118432406730196419039498586690468139979e+9748018096776', '68.631964', '-1.6630077481184324067301964190394985866904681399791e+9748018096776', 50, 3);
    T('-1.27868495493238609827093e-472', '8.41765151635151e-614149', '-1.27868495493238609827093e-472', 94, 2);
    T('2.47874123191038e-2692582', '8.05084862573291383306334438394811e-374', '-8.0508486257329138330633443839481099999999999999999999999999999999999999999999999999999e-374', 86, 1);
    T('5.00912431576062422992302261e-4052091', '-9.91381030448574834851045541170e-889443', '9.9138103044857483485104554117e-889443', 56, 1);
    T('3.27513161559989711059645473532e+81518414801382', '4.7132343297976605044199e+6779243013646', '3.27513161559989711059645473532e+81518414801382', 92, 5);
    T('-9.8784494926899978903321551e-9540566', '6.3208330675305717653632186745099e-1143286', '-6.3208330675305717653632186745099e-1143286', 81, 5);
    T('-7.03515071577893976408762784490503166316086698127e-13772110', '3.7891e+95145633009', '-3.7891e+95145633009', 40, 4);
    T('-9.151048913600070049532167804e+787636', '-4.57e+1272', '-9.151048913600070049532167803999999999999999999999999999999999999999999999e+787636', 73, 2);
    T('36329065464295405263892853765725766772', '-9.5841209157400219046746992e-9137133844066', '3.632906546429540526389285376572576677200000000000000000000000000000000000000000000000000000000000001e+37', 100, 2);
    T('7.2424695440712430920712553885834177747712509512e-22464539', '4.53743349096069058244499691777056505421697021604732e-466414018006', '7.2e-22464539', 2, 5);
    T('24713195475590418858049622602914541202.523', '-1.810856030521140050e+26', '2.471319547577150446110173660791455e+37', 34, 2);
    T('-5.44928964156e+15', '-5.644375e-732946775082', '-5.449289641559999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+15', 100, 1);
    T('-1.45906978551555402478467713912075840763080832459e-50816369982705', '-3.27639698930407305300306271516555609410e+7383761349404', '3.276396989304073053e+7383761349404', 20, 5);
    T('-3.263842560457371e+3886177', '-5.17599083377218e-72339804078', '-3.263842560457371e+3886177', 85, 6);
    T('3.2821551393009e-1317270', '-3.3311250537437740166299913314754126475738624282771399610908e+415835725163102', '3.3311250537437740166299913314754126475738624282771399610908000000000000001e+415835725163102', 74, 2);
    T('2.64022460751765855426479525980236252623900123913051e+497300658377', '6.0e-985936233062', '2.6402246075176585542647952598023625262390012391305099999999999999999999999999999999999999999e+497300658377', 92, 1);
    T('7.81233981425773487405e+9529', '-3.10415049416999454915429461730982083152992891844735e-767300', '7.812339814257734874050000001e+9529', 28, 0);
    T('6.6508919548918979854363568675029782197121274652731772e-3737473509', '-6.0728479350847383967022065e-795475900', '6.0728479350847383967022065e-795475900', 49, 5);
    T('-4.7089434265022266301104001575445503483553714008913633e-88738', '8.34688467157411027007535625210242993e-919', '-8.34688467157411027007535625210242993e-919', 62, 1);
    T('-9.10e+10', '4.3437340215746496704071253e-218380', '-9.1e+10', 28, 2);
    T('5.3697819281307078630338105915949330318292516820020497372e-14034909667969', '-2.335808578799699836967392676887815e+458', '2.33580857879969983696739267688781500000000000000000000000001e+458', 60, 2);
    T('5.92292679e+631718438', '-3.0e+362902713018', '3e+362902713018', 1, 6);
    T('-4.91999977389477123784244946587125047407873384825832323316328e-354245652512', '1.083202517075648e+231396', '-1.08320251707564801e+231396', 18, 0);
    T('-6.40733541910543434509393839818692047538785e+6322', '5.27422697716772e-216646978390525', '-6.4073354191054343450939384e+6322', 26, 3);
    T('8.6104556854805797434995510499280498849864853451e+579710086638772', '-1.6725115858335020546681606e-550', '8.61045568548057974349955104992804988498648534510000000000000000000000000000000000000000001e+579710086638772', 90, 0);
    T('4.1745840e-3339117110', '3.375128311248379325601415138584663540e+446477536989', '-3.37512831124837932560141513858466354e+446477536989', 55, 5);
    T('-3.786546345899e-189459414441', '6.02249706866143370360487850306435352189374841504e+61549', '-6.022497068661433703604878503064353521893748415040000000000000000000000000001e+61549', 76, 3);
    T('5.04971104626589141747626692103039911453e+45', '-5.0530263703564296605632251564592249404307e+53902', '5.05302637e+53902', 9, 1);
    T('-3.183723857205502840e+7291518290', '3.49268533656697942e+143530318', '-3.18372385720550284e+7291518290', 34, 6);
    T('4.74406393194724543735e-63', '4.43364885899910589e-7347986', '4.74406393194724543735e-63', 45, 2);
    T('-7.49610669169557e+867671', '2e+554300044', '-2e+554300044', 50, 1);
    T('-9.463776035260500781568692673484506079528289e+6492536052442', '6.83941870944890232363366773202197e+427', '-9.463776035260500781568692674e+6492536052442', 28, 0);
    T('-7.331006076559156425893605681959105278380884318343534e+108', '-6.93423625426122193602561723212322905519741428394571e+112', '6.9335031536535660203830278715550331446695761955138756466e+112', 69, 3);
    T('6.9e+7516720176878', '-5.8483744750055793931637498480817483805927761682599332244420e-757618', '6.9e+7516720176878', 28, 4);
    T('-9.7868124601194698648038444e-2242466160', '-8.9110117892977372751419467763e-7889911845', '-9.8e-2242466160', 2, 4);
    T('-4e+9', '-8.2098954e-94449370811498', '-4e+9', 92, 3);
    T('4.11194e-43', '5.1507859e-82526582202', '4.11194e-43', 85, 6);
    T('1.09723427e+3280289', '6.9236e-7346339368', '1.09723427e+3280289', 45, 5);
    T('-8.14108073436230215578e+4757774248382', '1.2901849870028548304e+7600449227', '-8.141080734362302155780000000000000000000000000000000000000000001e+4757774248382', 64, 3);
    T('-8.728022248551855473945013142298832052329695810389e+201', '1.0032129547724564480330970363098151408229562e+15584005947', '-1.0032129547724564480330970363098151408229562e+15584005947', 58, 6);
    T('-2.8843287106957172878553103194623777265736249588067599933056e+117', '4.5551975962829e-158', '-2.88432871069571728785531031946237772657362495880676e+117', 51, 5);
    T('-3.311783313080e-4560041745191', '6.346097476727798484737459828605281048e-8041142549', '-6.3460974767277984847374598286053e-8041142549', 32, 0);
    T('1.121164160931749948903696887668775848458e+6387317', '7.43051254e-448', '1.12116416093174995e+6387317', 18, 4);
    T('-4.45134254566247402195171e-27966', '5.1588e-20549329452467', '-4.4513425e-27966', 8, 6);
    T('-2.831351369192843997e-1272203', '-9.5113259886671760e+194950482174', '9.511325988667176e+194950482174', 19, 2);
    T('9.37590051e-5094126397194', '5.1252350671236564991962972504980e+890492413', '-5.12e+890492413', 3, 2);
    T('-8.821446795071e+4268048486535', '1.736e-492246509993128', '-8.8214468e+4268048486535', 9, 0);
    T('-4.686692330858e-8154198', '8.7959628649e+16', '-8.7e+16', 2, 2);
    T('5.0293193e+2666735660', '6.659873715508862008629e-730805298793', '5.0293192999999999e+2666735660', 17, 1);
    T('-3.10831620188714273743758468309807341054218778247775e+93888963236', '9.4213350241025035e-93012386158', '-3.1e+93888963236', 2, 2);
    T('8.90079685e-92435', '-2.0536813988442702947210287538706021183991030653395374363e+9012', '2.053681398844270294721028753870602118399103065339537436300000000000000000000001e+9012', 79, 2);
    T('9.1778921125450270973120392182042884422e+993997306', '6.4118980713e+3312149038651', '-6.4118980712999999999999999999999999999999999e+3312149038651', 44, 2);
    T('6.621144156284120440725507239780e-1643', '-1.21e-68701291', '6.62114415628412044072550723978e-1643', 52, 4);
    T('-8.982206494205515354021642562183745772e+94749', '-5.855469754921874895969110166e+456119057', '5.855469754921874895969110166e+456119057', 66, 6);
    T('7.69050392839767078274511076462173e+193188414661', '-279079429.7108', '7.690503928397670782745110764621730001e+193188414661', 37, 2);
    T('-3.2210823492404461134810837775538032842442487906927173e-55774', '1.28568e+6087500', '-1.28568e+6087500', 20, 6);
    T('-9.62315793898520473190490013896400327777360538895539393992078e-558440', '4.7251222306e-23561256767', '-9.623157938985e-558440', 13, 5);
    T('6.56e-39070253', '-6.59596225752322373679613837723634358098575799730695621473843e-312921139', '6.56e-39070253', 63, 5);
    T('9.445786223081715308898764312000738237041844529615422e+184977621806171', '-9.0113400107505e+3486680128686', '9.445786223081715308898764312000738237041844529615422e+184977621806171', 78, 5);
    T('3.5e+87801584', '3.85544379071716735408130023e-840827253868', '3.5e+87801584', 68, 5);
    T('2.391277098785267848e+54558897186', '-2.783784264482809206971928196657727430501343e-871', '2.391277098785267848e+54558897186', 95, 5);
    T('-7.112510630881972085872708963874754944685687e-156027594730534', '-3.0361963296423286e+9302710740666', '3.0361963296423286e+9302710740666', 21, 0);
    T('-8.513203681259558516096921219536891885e-52651168', '5.548581792e+683885874145078', '-5.548581792e+683885874145078', 80, 5);
    T('-7.956626080632819654979e-9990346', '-6.51106188e+786', '6.51106188e+786', 76, 0);
    T('1.13418064817932428131973078717437193722497e-44', '-6.639936806687143218321991509163e-396005233', '1.134180649e-44', 10, 2);
    T('7.6295178393355493800367722870509792232844e-865', '3.5216451e+10', '-3.5216451e+10', 48, 4);
    T('-36390023514497489171394505445842926102704469968520.118789', '-8.4319941473096627950842322889593e+40905028', '8.4319941473096627950842322889593e+40905028', 89, 6);
    T('2.09009544305092059430e+90990989', '8.4946602134836609907779313757598261186394235553826680249267e+62', '2.0900954430509205943e+90990989', 61, 4);
    T('9.6149819674423281346299436221987287092605324906518773386900e-3001133', '-7.97558147573098409293959595213818545006348902642011867e+1819677648', '7.9755814757309840929396e+1819677648', 24, 5);
    T('-4.82784445371807597778568299769732390074e-21', '5.299731847016075625274e-9644331288635', '-4.827844453718076e-21', 16, 0);
    T('1.091211107058614299299043663811089510209922432264108e-851935156447', '-4.0000097560263687640279839297772743e+546916583445', '4.0000097560263687640279839297772743e+546916583445', 86, 1);
    T('-6.93177276594796793376296e+883561', '8.623299885e+2571927', '-8.623299885e+2571927', 38, 2);
    T('-5.5068829883825979601952042469e+59926102', '-5.486646328766487115055e+306062453177', '5.486646328766487115055e+306062453177', 33, 2);
    T('-3.831579778986622897435009139439510794519506e-39567293853', '8.146523012162880e+70771658740', '-8.146523012e+70771658740', 10, 4);
    T('6.506851006423e+7419641420759', '4.27481136432078830960004602526e+93098574828022', '-4.274811364320788e+93098574828022', 16, 6);
    T('-2.8361123757252151185919405e+3504', '-6.6202432871594604191065469319963001842140e+41981', '6.6202432871594604191065e+41981', 23, 6);
    T('-1.17252912544937968657912734138973587058557610362488288e+88980429433', '-7.027378078565315935618667344821539357301201730454498e+908607', '-1.1725291254e+88980429433', 11, 1);
    T('2.1959525739376e-7103', '5.983311626544483483966617172374699390608522916625e+50', '-5.983311626544483483966617172374699390608522916625e+50', 65, 4);
    T('7.4837493386e+958311', '3.6583474550622811565338258695073554347971790611873e-298418770439', '7.4837493386e+958311', 30, 2);
    T('9.1062543073171318598384944691102406233920269847362465e-813', '-9.1641882200576715256309064779697e-8920180261', '9.10625430731713185983849446911024062339202698e-813', 45, 4);
    T('6.406278305565978958916287010911832429224852355006e-23564321', '3.676674632487343115849000199237402043565832811407778636e+5586', '-3.6766746324873432e+5586', 17, 0);
    T('-7.953494184350291259e+3599', '756.47428', '-7.953494184350291259e+3599', 52, 2);
    T('-1.49152124114433373304208537918976e+978664308', '9.8056303762370545951628510935981e+826613', '-1.49152124114433373304208537e+978664308', 27, 1);
    T('4.68220138656794386550605968677e+78113472847448', '-2.6686539861283299224968065450246727e-7582499', '4.682201386567943865506059686770000000000000000000000000000000000000000000000000000000000001e+78113472847448', 91, 0);
    T('-4.491735e-802742756951567', '-8.524963475598393393077709067338324926183854960808106694330e-65521184', '8.52496347559839339307770906733832492618385496080810669433e-65521184', 83, 0);
    T('-4.4430820122255910998045283212173749746495616501653405973e-827865011846119', '7.114513328685609430623640757501314859717324e-7120728530', '-7.11451332868560943062364075750131485971732400000000000000000000000000000000000000000000000001e-7120728530', 93, 0);
    T('1.9190806446636e+927864', '6.0920e+56', '1.9190806446636e+927864', 73, 0);
    T('-7.88263027397344213036923133242345252308367225574650e-88796848153437', '-5.5756368210189017431334124112392e-429621', '5.5756368210189017431334124112392e-429621', 40, 4);
    T('1.971243340235644074971025644303554363728299577789835e-571508', '-6.5519688444356632990328547582378140716480876405157919696995e+5064244', '6.5519688444356632990328547582378141e+5064244', 35, 0);
    T('3.948816151683182e+757', '7.1348224372662431594186785740e-72769341257', '3.948816151683182e+757', 77, 0);
    T('9.0e-59255', '-2.33919280024408067698194523101322791171940052045678e+3654381179682', '2.3391928002440806769819452310132279117194005204567800000000000000000000000000001e+3654381179682', 80, 2);
    T('7.5477184206946666075077020474e+416855', '8.226706287195296023863919137530032e-1018012', '7.547718420694666607507702e+416855', 25, 4);
    T('7.78319208605685596972467754789100439984872521201716674948e+57', '2.65256641975813504986025609068030588416038913e+74546300977736', '-2.65256641975813504986025609068030588416038913e+74546300977736', 78, 4);
    T('-5.147174146507801268796934e+1498367864', '9.67698824533175621362473127850553376637993879602541e+269648815', '-5.147174146507801268796934e+1498367864', 70, 6);
    T('-6.9094303088394213923476047903720631803604785525e-344', '8.2893988818128786207993428216100e-751833813909234', '-6.91e-344', 3, 6);
    T('4.6595e+4726817820426', '1.64015622242168428394544251558694927934367958108404e-99448425727127', '4.6595e+4726817820426', 42, 2);
    T('9866506145838.809373645', '6.61785353573102205935617656308519460507956061244772e-36204589078', '9.866506145838809373645e+12', 97, 2);
    T('-2.318067408042413919878840610692925121496560041557524460050e+23219570', '-2.3465212728541e-79723121755', '-2.31806740804241391987884061069292512149656004155752446005e+23219570', 99, 3);
    T('2.3064339708e+329953809890550', '4.4212458824772e+15', '2.306433970799999999e+329953809890550', 19, 1);
    T('-7.0964e-8816622', '9.321210755840210732005518262848e-8755816343292', '-7.0964e-8816622', 38, 1);
    T('-7.659098346360172190857190853986532401477719e-4892425914875', '-3.5515148943949610349561e-201752357978', '3.5515148943949610349561e-201752357978', 100, 5);
    T('2.0e+723446368', '-1.838717527951613e-1948', '2e+723446368', 98, 5);
    T('-7.65331435e-27620', '2.87378433963827234821147977087352240e+1779001396453', '-2.87378434e+1779001396453', 9, 3);
    T('-9.8781236464590194849831565800202343e+260671777416', '-2.82495451396507845530e+93802272354566', '2.82495451396508e+93802272354566', 15, 2);
    T('-3.26596e-77293', '-9.531145917178976043602806530795373494602e+130', '9.53114591718e+130', 12, 5);
    T('9.65037334e-681386', '-5.13294315443e+9412586319010', '5.132943154430000000000000000000000000000000000001e+9412586319010', 49, 2);
    T('-7e-484325373628991', '5.383856565319526009569676984021486352701245297782952e+38675', '-5.3839e+38675', 5, 4);
    T('7.8524445e+44260103629', '-2.751835961262779437540658e-31008977', '7.8524445e+44260103629', 18, 6);
    T('9.10776753217779759697962276359485481e+962080569', '634635371', '9.1e+962080569', 2, 3);
    T('4.69379921e+947353', '-3.4413040794326411771509788714883394720299338023126117e-7278', '4.69379921e+947353', 14, 1);
    T('6.01387602220584724412551e+94', '6.750437338095187617735740130198539985174356554684805e-99040207', '6.01387602220584724412551e+94', 79, 0);
    T('2.9245623522724287979315041752152347507070191297633e+9786829656', '-8.3606346472619330019461278643028251471e+1811973432042', '8.3606346472619330019461278643028251471e+1811973432042', 71, 5);
    T('8.010388258592757431315413056584058725148034846694426427e-9772', '6.7289271630023172e+310920408520', '-6.72892716300231719999999999999999999999999999999999999999999999999999e+310920408520', 69, 1);
    T('-3.925971e-76', '-3.1855531585225143724e-1115078351', '-3.925971e-76', 37, 3);
    T('9.909791653329558492727358600715159908562e-545358344030', '5.6089554537985322698688441201798897236253e+74', '-5.6089554537985323e+74', 17, 3);
    T('-9.198530034213843621172272541e-72957671257435', '-2.0861654073783480215725596e-76120', '2.0861654073783480215725596e-76120', 37, 4);
    T('-5.57341092114665920e-682002791582', '3.846272264000288e-5765870360098', '-5.5734109211466592e-682002791582', 38, 4);
    T('2.8550919897732378011228939448236386303389e+126765957003463', '6.37994109816590773211e-8190483416721', '2.8550919897732378011228939448236386303389e+126765957003463', 93, 4);
    T('-7.9478105710187940e-1207005937', '-9.51946e-7595609', '9.51946e-7595609', 60, 6);
    T('3.56767619458140021879115196538533290007478e+95620548', '-3.443531810131532890290700840175394113263743637e+314047', '3.56767619458140021879115196538533290007478000000000000000000000001e+95620548', 66, 2);
    T('-2.543550597e+24632336', '8510717759423.9954654', '-2.543550597e+24632336', 25, 5);
    T('-5.8801592892718561860914193125892904796823974728661622e-297752242249', '6.90376225546154171258114028400018256768465894237726627678249e+610747475', '-6.90376225e+610747475', 9, 2);
    T('2.3052089297617980221919457355562e+33', '-9.6136065373e-294', '2.305208929761798e+33', 16, 6);
    T('-4.6170106194968190425614908569894169e+840586134', '-1.1776246441078073984160175080357767e+9076465443786', '1.1776246441078073984160175080357767e+9076465443786', 35, 2);
    T('9.33975254481696341431469761847553518305988e-30805', '5.44325866573525195804806107151e-625105', '9.3397525448169634143146976184755351830598e-30805', 41, 3);
    T('-9.074405464221987527326371323056494513e-871', '8.414330470846497176644083677073662748577517e-4938412176097', '-9.074405464221987527326371323056494513e-871', 76, 5);
    T('-9.676e+8187369', '9.193459360690968478356669322342489798123885378850331515497e+23823002', '-9.193e+23823002', 4, 6);
    T('7.605212412942278095084576522987857028127856223390476738e+594', '-8.1527102916467986833e+607', '8.1527102916475592045412942278095084576522987857028127856223390476738e+607', 85, 0);
    T('7.680330292082e+91585127004320', '-3.314814216521271931025692295950805961688526020235646481e-85069853', '7.680330292082e+91585127004320', 73, 1);
    T('9.0556442670181e-78170324218', '9.630687102e-494536', '-9.630687102e-494536', 76, 0);
    T('-4.7763378242013107841319961434580643614807200e+46', '-9.15054677609926649377527581071015480671155692626654034e-14375643909', '-4.77633782420131078413e+46', 21, 6);
    T('8.413277608772948361804211758e+32', '-8.429069057354931915730623285731675243995470386730903755677e+92', '8.42906905735493191573062328573167524399547038673090375567700841327760877294836180421e+92', 84, 6);
    T('2.27161556605824024883878162257906568794566653e+259905', '7.735242062357027143301031514713284999937352484440755e-637', '2.27161556605824024883878162257906568794566653e+259905', 65, 4);
    T('-9.796037005539050898273934070503343436243801339915e-56679693433', '-1.444516877581988534308555401569697435525783768275524e+64752744', '1.44451687759e+64752744', 12, 2);
    T('4.13141777217890680810174014364432322e+336366558505523', '5769505734731191.985200803929349596405121247233768517156', '4.13141777217890680810174014364432322e+336366558505523', 66, 2);
    T('-3.08964715136510e+9111515615046', '2.682331683129603898849417497290240190e+96296', '-3.0896471513651000000000000000000000000000000000000000000000000000000000000000000000000000000001e+9111515615046', 95, 3);
    T('5.5301070599492540403037913404400403016791922432881167799e+362877115', '-1.944987143436008988810e+92887885645', '1.94498714343600898881e+92887885645', 66, 5);
    T('-9.727310253275325024752728364040141846874616657e+84940288', '-8.5623e+13', '-9.7273102532753250248e+84940288', 20, 0);
    T('-1.88468695489577228e+38307062798', '1.9693054471013343416313450324944678008513335929734897e+433', '-1.88468695489577228e+38307062798', 81, 5);
    T('6.762e-1534', '-6.545323356238258326501482098081140758976279045e-411161945', '6.76200000000000001e-1534', 18, 2);
    T('2.805072816263200161812518904303751691017e-58160554551831', '5.48758440980089930252121863237078651647442956940438410e+943398458124133', '-5.48758440980089930252121863237078651647442957e+943398458124133', 45, 5);
    T('-1.7716630927134120753268672243750220647036614542532e-17895', '5e-476186665268130', '-1.771663092713412e-17895', 16, 1);
    T('5.394350873516358717831682160430407045627115368636374e-13055143197699', '-5.20378776479684167e+41635573529188', '5.20378776479684167e+41635573529188', 94, 4);
    T('3.639e-47321492', '-7.6945070893449121632734243e+200803072492915', '7.694507089344912163273424300000000000000000000000001e+200803072492915', 52, 2);
    T('4.5377460717715e-58814268683196', '8.8356924773141833171385107443e-3264', '-8.83569247731418331713851074429999999999999999999999999999999999999999999999999999999999e-3264', 87, 1);
    T('8.836911849281817169385413833156837289587284e+110', '-7.817889321541837340914763906223230678550053848399e-14610533571610', '8.836911849281817169385413833156837289587284e+110', 76, 1);
    T('9.26652066014267774571789241490102071843935e+459308', '5.808015667201611482620e+748666419', '-5.80801566720161148262e+748666419', 74, 0);
    T('6.26725058189477116614169893520926725e-995934739594', '7.6460370892559827941153e-525', '-7.6460370892559827941153e-525', 42, 5);
    T('-6.18691249885761802903926799787e+57915897', '-8.326025261905499e-88083014', '-6.186912e+57915897', 7, 2);
    T('2.813941111038116954193e-48167278080498', '5.495345455718582775876407544075550626558108429425843e-1678959042203', '-5.495345455718582775876407544075550626558108429425843e-1678959042203', 84, 3);
    T('-365481054421022979.868', '-6.332503746670346253059003451380091855037558258780436e+7917', '6.332503746670346253059003451380091855037558258780436e+7917', 72, 4);
    T('7.00673701136459374158574778075115e+5429864', '-3.4413092458100429661468386317868516e-513178436412614', '7.006737011364593741585747780751e+5429864', 31, 4);
    T('-4.46710450597204914266409974325171280263604648638875296e+4818643514901', '6.194873384546635e+115847793851254', '-6.194873384546635e+115847793851254', 47, 1);
    T('-0.00310940', '-1.62823163830183471521868506275966859e+133', '1.62823163830183471521868506275966859e+133', 40, 6);
    T('-6.9559755633822523e-868890127690989', '-9.39572423440279453275680673221944517678072721319e+15228', '9.39572423440279453275680673221944517678072721319e+15228', 68, 0);
    T('5.94258234875395463e+19', '-4.175472146170419330032094866361215267109646732474e-939', '5.94258234875395463e+19', 42, 4);
    T('-6.275e-640324529', '-8.8986467302724804751317962111620482519174615240785100e-6009413216', '-6.275e-640324529', 13, 0);
    T('9.102881931907326810077892203704e+707107405173', '-2e-584', '9.102881931907326810077892203704e+707107405173', 84, 4);
    T('5.2431249201025001747942778496620595889317231309479e+37268594674', '3.1413733298554182827e+23', '5.2431249201025002e+37268594674', 17, 0);
    T('-4e-191037898434021', '3.6e+78353797945', '-3.6e+78353797945', 72, 2);
    T('-2.94794601113784e+16', '3.11082873837535774885189653491189915460459339388951e+838808303457', '-3.110828738375e+838808303457', 13, 4);
    T('4.61177155e+9690274489767', '-4.440485583548078216110449885285e-5646013', '4.61177155e+9690274489767', 30, 4);
    T('-7.5016559682327790443155504e+2993', '-2.511685649252390706804396492433866314946220400167546289e-8544', '-7.5016559682327790443155504e+2993', 39, 3);
    T('-9.6426576058935e+963470582534', '2.736258913226553040615049181633413581365437e-546362353904582', '-9.64265760589350000000000000000000000000000000000000000000000000000000000000000000000000000000001e+963470582534', 96, 0);
    T('-8.513794580763726479821565e-996846532', '4.286199221428593149747128607449302736e+3377', '-4.286199221428593149747128607449302736000001e+3377', 43, 3);
    T('-6.0100329318032e-3936486840', '3.84226242725386823676e-28173406', '-3.84226242725386823676e-28173406', 34, 2);
    T('-5.49612250073412246795491e-2321195', '-1.9966006545859022216306630059761958e+4970542', '1.9966006545859022216306630059761958e+4970542', 55, 5);
    T('-6.12250997551708977733538885447920165861728e+7055622', '1.55422730e-68292377', '-6.12250997551708977733538885447920165861728000001e+7055622', 48, 3);
    T('-9.712951665938717625947623633483870391548504023788e-8122349', '7.671526e-7123710975', '-9.71295167e-8122349', 9, 5);
    T('-3.48822153931596831874927049605738402134899549494653554e+57764170377', '3.3236851988651433129931160171386e+2979', '-3.48822153931596831874927049605738402134899549494653554e+57764170377', 68, 5);
    T('-4.9483217212145142600e+9787229195876', '4.494172822149202199e+425455', '-4.948321721214514260000000000000000000000000000000000000000000000000000000000000000000000000001e+9787229195876', 94, 3);
    T('5.152925360137e-17099', '-9.485360656389593044622258947631781e-278992985927694', '5.152925360137e-17099', 13, 4);
    T('4.47951223e+3380297', '9.444508732481675139580e+8563801172947', '-9.4445087324816751395799999999999999999999999e+8563801172947', 44, 2);
    T('-6.16559989093103824260844860598406481404277963670e+879932', '-6.3789578202326155626719235047585641e+6494795083', '6.3789578202326155626719235047585641e+6494795083', 94, 4);
    T('-3.54e+33', '-3.84033642479605e+65', '3.84033642479605e+65', 22, 4);
    T('-8.43626639815762950686e-30490', '3.63779392400628979713e-89253', '-8.43626639815762950686e-30490', 76, 5);
    T('-5.24373406422008046129669166187888672115344901291631361144254e-670', '5.59857775040606701764763341392135953646133082680e-8736', '-5.24373406422008046129669166187888672115344901291631361144254000000000000000000000000000000001e-670', 93, 0);
    T('-8.4074875e-660679890204187', '2.16432062461e+332950', '-2.16432062461e+332950', 41, 1);
    T('-7.02367716e+6220690011021', '7.831443055438e+9855', '-7.02367716000000000000000000000000000000001e+6220690011021', 42, 3);
    T('9.215169743432122002654469e-209105034', '4.706060016814027799262327491233248036e-77933616621379', '9.215169743432122002654469e-209105034', 81, 4);
    T('-9.5502661954111896733128524940100263309901268556748716e+769', '-810658359.1674797340687799569692902593093518040075530', '-9.5502661954111896733128524940100263309901268556748716e+769', 65, 5);
    T('-4.752364283389100798256977470114891889673179085341e+53706460935641', '-4.1512296174123185512348136128967478631748798492333928787904e+941370683', '-4.752364283389100798256977470114891889673179085340999999999999999999999999999e+53706460935641', 76, 2);
    T('-5.49491225637292172609224204049857336260107843148711532e-4555395', '4.8082713602132978199e-315', '-4.80827136e-315', 10, 4);
    T('-9.26888502789232976567374328963080593e+9042094380', '-2.39123331439958e-998817283772', '-9.268885e+9042094380', 8, 2);
    T('-2.33743967242310581677826510505986975903342101e+811349275258722', '-5.98427671992575685888611539100e-875724', '-2.337439672423105816778265105059e+811349275258722', 31, 2);
    T('6', '2.19668451444806131273451197906242474677371671532476e+80', '-2.19668451444806131273451197906242474677371671532476e+80', 75, 3);
    T('-2.6733802693e-2158203', '5.8417245059562068611556688875183116272174679771495e+317652', '-5.8417245059562068611556689e+317652', 26, 3);
    T('7.669846618695e-4895843545633', '-7.2883082419046072906088105013e-65448', '7.288308241904607290608810501300000000001e-65448', 40, 2);
    T('1.9405128909388617e+776937995780', '6.56035346878301707400008e+899289391', '1.9405128909388617e+776937995780', 84, 2);
    T('-5.4623378176039520885245149007734974642732e-9318512778143', '-1.32960686080163e+80727783281', '1.32960686080162999999999999999999999999999999999999999999999999999999999e+80727783281', 72, 3);
    T('8.3062616698902539745763811434559930846813223975e-153', '-7.034192651984e+953458627', '7.034192651984e+953458627', 53, 3);
    T('7.2249976784761925037712036916040e-544504', '-9.858471643611212976199942e-734', '9.85847164361121297619995e-734', 24, 0);
    T('4.51219846067670e+92820353486', '4.09782630120780777938314658175826327686276160128133455073e-8024161265370', '4.5121984606767e+92820353486', 99, 5);
    T('-7.6e-4378912189933', '-3.4102165201814178212574474667070393714468e+109430625310', '3.4102165201814178212574474667070393714468e+109430625310', 61, 2);
    T('6.20117e-298', '-9.360512e-909817', '6.20117e-298', 16, 3);
    T('6.5706485889979626076286703e-676712', '-2.33852096173309188027017328993e-5687', '2.33852096173e-5687', 12, 5);
    T('-5.7949234867195182159792034880068265424494745057875115269912e+63', '7e+340268', '-7e+340268', 64, 5);
    T('8.8951778856883702962629965272861429386340094266e-41538833', '-6.4739566343556178000184793108833709900e-2387', '6.4739566343557e-2387', 14, 2);
    T('-9.81750713959759470138107055010e-672612146375405', '-1.05312326436e+71354802', '1.05e+71354802', 3, 6);
    T('-1.571949319885709442302502137315851988192759e-34129', '-9.82708223e+668750979279', '9.82708223e+668750979279', 68, 6);
    T('-4.2034749474131443995999609664e+492', '-4.012503696e+76', '-4.2034749474131443995999609664e+492', 45, 6);
    T('-4.502758860318012828487157011750634646543506e-39414557', '8.920051614269765032685766271814980891e-28307995', '-8.920051614269765032685766e-28307995', 25, 5);
    T('9.90576633e-60', '-90778242545316461196396580695695754967075.7206', '9.0778242545316461196396580695695754967075e+40', 41, 3);
    T('-1.058236057566287514515962e+8483', '-3.59502423333506792e-944401499562', '-1.058236057566287514515961999999999e+8483', 34, 1);
    T('4.47174343472145696438908841719313214982148326e-19504', '3.5169793162358412802108301060467464795e-3525578714', '4.47174343472145696438908841719313214982148326e-19504', 89, 4);
    T('2.114460489987004e-64368742463', '7.29526935637308243967124e+24', '-7.3e+24', 2, 0);
    T('3.2121598745611704346801780926375430e-1761951258', '3.46798430914330581815255242483941916275469678750264355749453e-24739706735539', '3e-1761951258', 1, 6);
    T('4.776055982248379e-45', '8.09098592654894122958232477340954209280234574222109331962767e+449', '-8.09098592654894122958232477340954209280234574222e+449', 48, 1);
    T('3.6225318533888453247555559584591314039891244189884401586122e+184', '2.857588980129931707892e+23', '3.622531853388845324755555958459131403989124418988440158e+184', 55, 1);
    T('-3.82594823016710777803926041422747301473169690637e-37358821', '6.6461437827818796313357588571100751268409510892251080605e+837', '-6.64614378278187963133575885711007512684095108922510806050000000000000000001e+837', 75, 3);
    T('8.7185693319882949889490766281271472994511006385825059216086e-86305', '2.2031850154239123523377e-22', '-2.2031851e-22', 8, 3);
    T('7.6707016820574e+86063116', '-9.88296115915e-20', '7.6707016820574e+86063116', 65, 1);
    T('6.9688878306783307439195096690870119331221624834055122e-228511055228', '-2.9013245597131857801948596e+56389', '2.9013245597131857801948596e+56389', 39, 3);
    T('1.95211195671371819674599e+6606799827714', '-6.778300436207980041922235575e-70722', '1.95211195671371819674599e+6606799827714', 67, 6);
    T('3.61187730740366059893319956906034901934369971299714308e+116184476429471', '6.688454473452e+14', '3.61187730740366059893319956906034901934369971299714308e+116184476429471', 66, 4);
    T('5.437487428669730795300234833449976779166175807280226e+3387', '-8.2475e+77893', '8.2475e+77893', 60, 5);
    T('6.6821808257708805316529458604170e+420853933844071', '-8.9162004949156537642737692e-9983340', '6.68218082577088053165294586041700000000000000000000000000000001e+420853933844071', 63, 0);
    T('-3.04614346964912935128570999594591068932073937e+67718350985', '3.203727740e-326', '-3.04614346964912935128570999594591068932073937e+67718350985', 49, 1);
    T('4e+763895', '3.22151612372247529e+507', '4e+763895', 6, 0);
    T('-5.931963708559029527263250781844456838e+8850429047', '1.677491389092359340342893166256415931741213343307745686937e-95294623191', '-5.93197e+8850429047', 6, 3);
    T('-8.3401802795893752678190423901075236417e-6159689886289', '-1.7240286765728797792941644546100195808086467418617e+72603232771816', '1.7240286765728797792941644546100195808086467418616999999999999999999999999999999999999999999999999e+72603232771816', 98, 1);
    T('-7.75889e-1099', '-9.16510469777130835837207305980279381231405900e+95', '9.165104697771308358372073059802793812314058999999999e+95', 52, 3);
    T('-3.1390714991997e+2493388', '-1.1809776797160814371012370109478920e+82182743903', '1.18097767971608143710123701094789199999999999e+82182743903', 45, 1);
    T('-3.0036210684015645163181862371336231450352877182737452e-719839454864175', '-4.618305371745011273054881753132446082902254948839437581e+75980675879606', '4.6183053717450112730548817531e+75980675879606', 29, 5);
    T('5.8537e+434206271', '-1.5526313215727028427e+903959358537', '1.55263132157270284270000000000000000001e+903959358537', 39, 2);
    T('3.60e+163', '5.7400930826119010754961016745091281448490933199110e-13651442529977', '3.6e+163', 65, 4);
    T('-8.570968871334557813293021409112066967336062252335535e+81', '2.15247463306358445385820172528402491503073885e+53', '-8.570968871334557813293021409327314430642420697721355172528402e+81', 61, 2);
    T('7.25070126559239754451545350858057e-328753866984001', '2.5897984e-343626208957', '-2.5897984e-343626208957', 38, 0);
    T('5.55005643867939149828391359824075376283986959949196e-1033449', '2.6032065422383937790442421380704222162566e-715', '-2.6032065422383937790442421380704222162566e-715', 84, 6);
    T('5.7302690e-61218150817', '1.781494010e+7270', '-1.78149401e+7270', 12, 4);
    T('-9.81444274449816165755542e+577571', '-4.91907193622137536280098585968616204851048e+42', '-9e+577571', 1, 1);
    T('3.5051e-31', '1.571569783712888431782918820403930740951032788247356908e-203182837917847', '3.5051e-31', 11, 5);
    T('-5.6915646398431e-205944817834', '-1.04357907545831587340335059081614055e+553167125418', '1.0435790754583159e+553167125418', 17, 5);
    T('-8.66361370764323228751242197323079501567806750247e-75191022', '-3.6351433115401057e-8564917806', '-8.6636137076432322875124219732307950156780675024699999999999999999999999999999999e-75191022', 80, 1);
    T('9.569e-8808589175104', '9.092581060683456e+6046461234', '-9.09258106068345599999999999999999e+6046461234', 33, 1);
    T('3.26829906669910786266280310636871169692318216761047997e+6620205724469', '-5.5555579780810025897945076759553056e-2917133', '3.26829906669910786266280310636871169692318216761e+6620205724469', 48, 3);
    T('-5.8576146142564e-23663885184', '3.2583874960109352149518424483357e-411989972297', '-5.8576146142564e-23663885184', 74, 5);
    T('4.2999188774021e+755298583844', '5.173325528241898e+4307554512', '4.2999188774020999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+755298583844', 98, 3);
    T('-6.844855142277930316509179939976522719e+76895934606469', '-4.91148799946854980717e-667163', '-6.844855142277930316509179939976522719e+76895934606469', 89, 4);
    T('-5.151362720388665212077749916047328920022e-561223', '9.64880225451799e-6037704', '-5.15136272038866521207774991604732892e-561223', 36, 6);
    T('4.6925324166302763704542734e+748017972', '4.6324127884869945917985684228814772604542e+86493908660575', '-4.6324127884869945917985684228814772604541999999999999999e+86493908660575', 56, 1);
    T('3.296558894564620752018494491943553531408118858363118e+204065523547503', '-3.567239355826075766299466507245355664649e+731', '3.2965588945646207520184944919435535314081188583631e+204065523547503', 50, 5);
    T('9.6739218900591263736870116853841358043852777340e-4382239356', '-9.42519274370986549133545541464e-83603967027863', '9.6739218900591263736870116853841e-4382239356', 32, 4);
    T('-4.0e-3992', '-4.408038713737823077147683616710033033726070e-63093771647', '-3.9999999999999999999e-3992', 20, 1);
    T('-5.58082242592860005128360045063256659502772064611e+45343', '-1.144121227399292452699821755576597402088160363771411911238e+623194447878', '1.14412122739929245269982175557659740208816036377141191123799999999999999e+623194447878', 72, 1);
    T('-3.22818657910084166144699361542550200448412e+230347', '1.41947364939475097722e+23', '-3.22818657910084166144699361542550200448412e+230347', 76, 4);
    T('-1.3258129920964158885728e+25', '-4.7226e-45668521722809', '-1.32581299209641588857279999999999e+25', 33, 1);
    T('3.995613e+6040', '-1.027344347675540352357357467144667556029001972776271236e-245984528729', '3.995613e+6040', 89, 1);
    T('-5.11811248556827899090262768623197e-1241131', '1.30998473440470949898483565226346163265e-946340845731', '-5.11811248556827899090262768623197e-1241131', 88, 1);
    T('-4.822030801072782934569841275772250632653519162781286575e-29045518', '4.8568385e+52888561', '-4.85683850000000000000000000001e+52888561', 30, 0);
    T('4.821922911474701337333075927720907377632e-48094', '-2.53e-695990301', '4.821922911474701337333075927720907377632e-48094', 48, 6);
    T('8.823541524081785087484087466945e+99', '6.589015263572818549102205678e+1433729978', '-6.589015263572818549102205678e+1433729978', 43, 0);
    T('-4.2423901738848364448880045885764602007234913657120204334959e+4835427', '5.3136905103805000992402715148150689917661640188e-4767427', '-4.242390173884836444888004588576460200723491365712020433496e+4835427', 59, 3);
    T('-7.95929795976586e-732293902092', '-4.824072996905415589454721095081212717533171028099435855e-6554', '4.8240729969054155894547211e-6554', 26, 6);
    T('-4004190955999968569755075866803472383064919563008.024', '-6.6743104063427234238506e+4133627', '6.6743104063427234238506e+4133627', 42, 0);
    T('-4.751089631105958e-17808241', '2.9891916231267519328039254142893737518564226658e-652636166491', '-4.751089631105958000000000000000000000000000001e-17808241', 46, 3);
    T('9.27119400904e+7206165746269', '5.4357948349129069902284920095241218950e-88523875018', '9.27119400904e+7206165746269', 61, 6);
    T('-3.01529829611672473265773382e+1011', '3.84443055767921760e+23215890', '-3.8444305576792176e+23215890', 46, 2);
    T('1.692933177184507496162166053113341356947430976826424220143e+33047457962264', '-9.0668756811e+19163716', '1.692933177184507496162166053113341356947430976826424220143e+33047457962264', 79, 1);
    T('-9.8163181198181897e-77', '-6.2316e-16', '6.23159999999999999999999999999999999999999999999999999999999901836818801818103e-16', 89, 6);
    T('-4.4493091322983520895726998913e-731096264640363', '-6.1505019554657127819750562e-19', '6.1505019554657127819750562e-19', 76, 6);
    T('8.238042747953199254898168180342929312322593285517834117e+3791183158', '-2.84880798904050e+773679780981037', '2.8488079890405e+773679780981037', 68, 4);
    T('7.3258152864234124955549805804388397822935e-548968065242', '3.47975588661493e-8275839839066', '7.3258152864234124955549805804388397822935e-548968065242', 88, 6);
    T('1.394266649683544171302552338e-7595', '8.64892498149166979075868286446890e+111', '-8.6489249814916697907586828644689e+111', 75, 0);
    T('-3.781588157830589615104337107222569401600762e-89044079621203', '4.01210961886316880239716e+1388561152830', '-4.01210961886316880239716e+1388561152830', 89, 1);
    T('-118874494002212424002430937233160905663001730656', '-9.852089659444831762169416166483893441986975e+46051', '9.852089659444831762169416166483893441986975e+46051', 45, 0);
    T('-3.29344232808167841176779396761e-54232396286446', '-4.35153497161483294659e+28', '4.35153497161483294659e+28', 28, 0);
    T('3.17323683793108531048536634442759418655e-596', '-2.63587641959622133333797512e+8198976005457', '2.63587641959622133333797512e+8198976005457', 90, 4);
    T('1.405219549923214533674624141963790882123270596521145132707e+6241701359138', '1.76482055034157088438069357541607208528934442175579e+367502237970141', '-1.76482055034157088438069357541607208528934442175579e+367502237970141', 90, 3);
    T('1.0823424661653508198705515306e-21259276841589', '5.756224708019286878529931831404208017e-1608957047', '-5.756224708019286878529931831404208017e-1608957047', 99, 4);
    T('-5.1e-63356719', '6.3e+26787114', '-6.300000000000000000000000000000000000000000001e+26787114', 46, 0);
    T('5.7234e+566', '-8.08e+834', '8.08e+834', 75, 3);
    T('-2.368274647064623775640509114059e+70831578886', '2.2560903821241541732412169173440246172e-1072039397706', '-2.368274647064623775640509114059000000001e+70831578886', 40, 0);
    T('5.7687e+69783135711', '7.2810116103972488014210881e-139626', '5.768699999999999999999999999999999999999999999999999999999999e+69783135711', 61, 3);
    T('8.890722474772321647903638184482896465095875332965790316e-1064277', '104458690764647007791598048343086903428730008011301666.291703', '-1.04458690764647007791598048343086903428730008011301666291703e+53', 60, 3);
    T('1.85102654797769777523621644100347249386931412114367421803633e-929538968', '-9.3693056538504908625284795857468504e+1466', '9.36930565385049086e+1466', 18, 4);
    T('-6.1598581498549e-737499792152142', '-3.501576839920714230840e-23588652499336', '3.50157683992071423083999999999999999999999999999999999999999999999999999999999999999999999999999999e-23588652499336', 99, 3);
    T('-3.0333767727536314348849067343621756116614244990578e-8294421', '-5.59868268748557195371214e+4184391', '5.59868268748557195371214e+4184391', 69, 5);
    T('9.779686091841019031369118449615661496823894126147e+6970321411', '1.92160429988961698156913229664024578085183486986311e+489', '9.7796860918410190313691184496e+6970321411', 29, 3);
    T('-8.27065293798533369451990154e-27747', '-6.999175206462426212814953115600032e-880971760896', '-8.27065294e-27747', 9, 5);
    T('4.10570055527506520e+578569375123438', '4.95473720163145156235e-861', '4.1057005552750651999999999999999999999999999999999999999999999999999999999999999999999999e+578569375123438', 89, 3);
    T('3.31902535077761172450427398e+248', '-5.31918016122772261909226445605983940870398322135e-41', '3.31902535077761172450427398e+248', 70, 4);
    T('5.201181100276580081453308339e+6166122', '-1.2013298266809081834752267909924163e+404471151', '1.2013298266809081834752267909924163e+404471151', 62, 3);
    T('-3.477e+679386308111588', '-1.04195540701285342439526138641489593421837597097309255326e+763745384', '-3.477e+679386308111588', 46, 5);
    T('-4.44034648433157119901207e+88', '-2.5431031024174996198654181564198665683135537757837944e-329369770050', '-4.44034648433157119901207e+88', 60, 3);
    T('-1.4471e+768259786776945', '5.7662547443638728741116065378572704770963092e-484238420999', '-1.5e+768259786776945', 2, 0);
    T('-4.959481510920972256950015871023338958006611e+4230', '2.8009396459762762120e-3259799', '-4.959481510920972256950015871023338958006611e+4230', 51, 5);
    T('9.209674e-4275355597479', '-1.420660975056334407127189922778848092522180218834379817747e+490202451156', '1.420660975056334407127189922778848092522180218834379817747e+490202451156', 84, 3);
    T('7.393141997585651023038927905440063e-15063060563353', '3.0731190570214978353106154894529303e+532554370526', '-3.073119057e+532554370526', 11, 5);
    T('870626096789628009047821447051809446234.143461', '9.92161038631636834227454710455790389666377186265651e-415455782', '8.7062609678962800904782144705180944623414346e+38', 44, 1);
    T('8.355204548642438741611791753157317957694906249111e+785', '-5.0880596e+87163', '5.0880596e+87163', 26, 1);
    T('-6.6528982e-2823674632', '-15400892796099181272047038677150238923021.54170286', '1.54008927961e+40', 12, 6);
    T('3.08584875573105044922982929441388491557349329e-7522208', '4.902010477993569295455353789764430888846766304193185584e-4828856', '-4.90201047799356929545535378976443088884676630419318e-4828856', 51, 1);
    T('4.05290367375859678576442602651809e-1167968814175', '4.73107075489831240542938966415973602908164946525e+344437657039441', '-4.73107075489831240542938966415973602908164946525e+344437657039441', 56, 6);
    T('2.9629930820e+87925704', '-6.0e+94288028', '6e+94288028', 70, 4);
    T('-1.17053845284564445671747508828e+4746867', '-2.92651448349886e+503', '-1.17053845284564445671747508828e+4746867', 87, 5);
    T('5.45226803757815620341513762284811931280200e+7111402737', '8.31866540214753335786340029530484764886074810e+71815600409176', '-8.3186654021475333578634002953048476488607481e+71815600409176', 95, 6);
    T('-1.87514262495802883602977769e+100891474', '4.174325304912361700e-951214181701792', '-1.87514262495802883602977769e+100891474', 92, 2);
    T('2.59680381510679e+4032', '-4.7e+44282759', '4.7e+44282759', 66, 1);
    T('-2.964621821027291033545419911965141553348372057105e-5566020166000', '-4.448075414808998516e-98430', '4.448075414808998516e-98430', 59, 0);
    T('9.7319768906e-14', '-7.2594e-852', '9.73197689060000000000001e-14', 24, 0);
    T('5.872e+152936394185236', '5.350154871236928521891185761381314780262413930201821e-8794', '5.872e+152936394185236', 100, 0);
    T('-7.123203857e+5575504', '5.20898275192383e-254287228591996', '-7.123203857e+5575504', 75, 1);
    T('9.4402e-975964415465', '-1.06825664e-36820491', '1.0682566e-36820491', 8, 4);
    T('-1.30521871e+59', '9.902332101243237565943948125e-8873103177', '-1.3052e+59', 5, 1);
    T('1.29810309305535e-20', '-8.31287123445357132996486868633456093717463363291295458212e+808609752028', '8.31287123445357132996486868633456093717463e+808609752028', 42, 6);
    T('-9.8882983013365535470452422514268e-6446424431779', '3.9165358e+61365865', '-3.9165358000000000000000000000000000000000000000000000000000000000000001e+61365865', 71, 0);
    T('3.550e-2629246', '-7553069444059880.3935735', '7.5530694440598803935735e+15', 77, 5);
    T('-4.84207786090241071933548793953035363428107845e+4313436361325', '-3.67666562501661890049e-898862176', '-4.84207786090241071933548793953035363428107844999999999999999e+4313436361325', 60, 2);
    T('-9.884185055940801e+69380', '5.2307914005154677612374092614440101631725269259655e-38', '-9.884185055940801e+69380', 95, 5);
    T('9.045796802279416612644327301192431660485205e+9771386659', '2.20091940314386632159e-42520889745980', '9e+9771386659', 2, 4);
    T('3.18008878235397902352587486945128744195673761699e-66755337601444', '9e+351', '-8.9999999e+351', 8, 1);
    T('-3.243330871464169948428597e+112', '-8.011739886775874144088756835674585879724553345063e-966759210713586', '-3.2433308714641699484285969999999999999999999999999999999999999999999999999999999999999999999999e+112', 95, 2);
    T('7.7929687842e+89672526290927', '-8.092345284e-307', '7.7929e+89672526290927', 5, 3);
    T('-9.5250004943569530e+110', '1.6e+36225399831374', '-1.6e+36225399831374', 13, 6);
    T('-2.5786657035628179509293437174399728422473187e+8612437557070', '4e+5116167546', '-2.58e+8612437557070', 3, 5);
    T('-4.7214577281514793519189444076112657778131822815714243e-7875033', '-3.93757e-5118', '3.93757e-5118', 96, 5);
    T('9.80416841083823867840333992732e+3405096', '-9.8806037541438304356e-380222', '9.8041684108382386784033399273e+3405096', 29, 1);
    T('-7.84854026687635634785837814453485004915309955712999610969e+640071812338664', '-8.685355490464450964112139425249820967232584281e-93687481', '-7.84854026687635634785837814453485004915309955712999610969e+640071812338664', 72, 0);
    T('-8.97087951299668373485237e-206859093', '-3.3431789292651313513421685897884910077292964516e-7503', '3.34317892926513135134216858978849100772929645159999999e-7503', 54, 3);
    T('-4.034508771846740503508337022965905332738232907572591e-60214', '3.728440368304179816393e-270', '-3.728440368304179816393000000000000000000000001e-270', 46, 0);
    T('-9.4580412788771948446654559589431092136e-705807838986', '1.2286538764020376258386993129291064400684928e-84', '-1.228653876402037625838699313e-84', 28, 3);
    T('-5.216020316328025791935e+18721', '-6.03914064805110e+17695316437417', '6.039140648051099999999999999999999999999999999999999999999999999999999999999999999999999999999e+17695316437417', 94, 3);
    T('-2.4588988005975523314266906976110842815754438397793896824e-9446', '4.38251779055964743e-32221094', '-2.45889880059755e-9446', 15, 1);
    T('3.864359361442e-772199330254', '1.2086917134309821097047440095971099370781979374831811489e-719992591694', '-1.2086917134309821097047440095971099370782e-719992591694', 41, 5);
    T('9.0179376754641913062512e+645', '9e-33950', '9.0179376754641913062512e+645', 48, 2);
    T('1.3956472059932617171325171087677232508e-11886460', '5.2610238638497840391439927633401979238961946418323582523e+64', '-5.2610238638497e+64', 14, 1);
    T('-8.2682e-70768846', '-2.401069640169231672e-77083325213', '-8.2681999999999999999999999999999999999999999999999999999999999999999999999999999e-70768846', 80, 2);
    T('-7.0400091024089218307279565647e+913', '-4.9457989202284399553656561091394807396e-58821961136', '-7.0400091024089218307279565647e+913', 66, 6);
    T('-5.9089e-25877680892', '9.0599e+795', '-9e+795', 1, 2);
    T('7.35321041718849400604322196940101216719117479809493143283e-17033375', '-5.172559e+6755761134', '5.172559e+6755761134', 67, 3);
    T('3.347866671170107342509268452766446e-56977629901', '4.276043127380272695640108547764886489847368766770437e+258', '-4.276043127380272695640108547764886489847368766770437e+258', 66, 0);
    T('-0.000062274281170765180', '-1e+516717844348', '9.99999999999999999999999999999e+516717844347', 30, 3);
    T('-4.1042045264015434268027893520664695051680555378087e-55525379', '-43801030709014805247442083871731482692642046313139', '4.3801030709014805247442083871731482692642046313139e+49', 77, 2);
    T('-9.84799391161587748776555988020775607222e-49122214226258', '-5.873192778833062438564027258e-46353213396', '5.873192778833062438564027258e-46353213396', 99, 0);
    T('3.971083283596021449702956470764411e-4348', '6.04895215824482248e+844', '-6.0489522e+844', 8, 6);
    T('-5.57468264127849049e-4987', '9.393129264840907568827742116471041e+2612747817', '-9.39312926484090756882774211e+2612747817', 27, 1);
    T('1.70479319603564907846606653828e-5538', '-3.555e+61455045', '3.555e+61455045', 76, 6);
    T('-8.63809139656325553595726573299115912659061335571916e+924065', '-5.44685921022233843839364069080620447901480266189e+85411300233097', '5.44685921022233843839364069080620447901480266189e+85411300233097', 97, 4);
    T('-9.1187223818146209855e+59305532784', '-5.335594592088549047e+8262', '-9.1187223818146209855e+59305532784', 36, 5);
    T('-4.442064228085407037854628897215033265214269136257e-32550491696', '-5.3460587161e+823', '5.3460587161e+823', 68, 6);
    T('-8.3606305246297265384065220475775214480812139689058450382629e-83801', '-9.8771828804453757020417853073828127439e+410447240497', '9.8771828804453757020417853073828127439e+410447240497', 54, 0);
    T('-8.1925904845796406449360163935222853937431059419676233e-36542484408579', '-7.69025425950e+1287151744', '7.6902542595e+1287151744', 71, 2);
    T('6.660e-139061565', '-9.5842085519337757224099020351360905024318421252820693e+5265', '9.5842086e+5265', 8, 6);
    T('-3.020414107e-5913167614231', '2.7525081e-21654936', '-2.75250810000000000000000001e-21654936', 27, 0);
    T('4.50124998e-394237061', '-7.561582459017716506048081780037872246386385532730410018e+63514748826839', '7.56158245901771e+63514748826839', 15, 3);
    T('5.422085412264225006129782016739332e-1215295688', '3.746810161952551095539438857376e+2300', '-3.74681016195255109553943885737599999999999999999999999999999999999999999999999999999999999999999e+2300', 96, 1);
    T('-4.2560963565148091001234791544297647874964476594560303704510e-33390334701', '1.33459855690431600277e-8730', '-1.33459855690431600277e-8730', 55, 1);
    T('-6.780077749937e-282718280725', '-9.8912625421012494604139615011260580128827899749762119341307e+142014', '9.8912625421012494604139615011260580128827899749762119341307e+142014', 76, 0);
    T('8.114258e-929304608323020', '4.28407420460096566257929142993e+50187175496634', '-4.284075e+50187175496634', 7, 0);
    T('-9.75376623416603204384090061655521410975463367535513094297e+33695', '4.25991834498067791289121e+292708623039', '-4.25991834498067791289121e+292708623039', 71, 1);
    T('-6.050429831780693606473061721305185725735387e-9426680453', '7.4974883279495927650e-17569723356', '-6.0504298317806936064730617213051857257353870000000000000000000000000000000000000000000000000000001e-9426680453', 98, 0);
    T('-5.239e+59', '7.50018702452111e-897150', '-5.239e+59', 60, 5);
    T('9.4e-77642222021', '7.46014e+8167796', '-7.46014e+8167796', 59, 5);
    T('5.16146815e-7227300143', '-4.05656938626251417e+98206450413323', '4.05656938626251417e+98206450413323', 85, 5);
    T('-7.64702951115338634923570876654702644887739186020399312541e-6685264827282', '5.419192169000624366884395427581906007916334584989406233654e-341604', '-5.41919216900062436688439542758190600791633458499e-341604', 48, 6);
    T('525142917457870623641808368863133633.236552', '-4.2980036846175737723295358348874990300705673674e+786143413', '4.2980036846175737723295358348874990300705673674e+786143413', 77, 6);
    T('4.51430354e-721042272580239', '-8.32933065146980620032068735948386122676728764659419973e+601648006250513', '8.32933065146980620032068735948386122676728764659419973e+601648006250513', 80, 5);
    T('-5.813759860069104815160178e+898603160', '8.92268825385282743528150131308e-707508993193', '-5.813759860069104815160178000000000000000000000000000000000000000000000000000000000000000000000001e+898603160', 97, 3);
    T('-4.9079618856780568048751409392196353895486181914644404454105e+712671655', '8.6004285686676033067540579689813241829578008410444175179064e-119', '-4.9079618856780568048751409e+712671655', 26, 2);
    T('-6.5533781295e+773249', '-4.22163777348120102436586e-803384432680', '-6.55337812949999999999999999999999999999999999999999e+773249', 51, 2);
    T('8.3428990210946214905860229879804479157643578587908361820325e-21643667441191', '-9.4037473554812909565846941578634375688532e-19885831000', '9.4037473554812909565846941578634375688532e-19885831000', 99, 1);
    T('-5.0073e-56436966998', '-6e+9', '6e+9', 100, 5);
    T('239764147322.769', '8.1066713224686783738031895878911883183319504746954e+3260062096184', '-8.106671322468678373803189587891188319e+3260062096184', 37, 0);
    T('-9.4138615849368291e+74', '-494971778899917196943186115850461390775.62648', '-9.4138615849368290999999999999999999950502822110008280305681388414953860922437352e+74', 98, 4);
    T('-2.24420267770001313597768287889804186038e+2109525421289', '3.232892504617990096781559251e+79', '-2.24420267770001313597768287889804186038e+2109525421289', 62, 6);
    T('-2.27902047319e+36', '-1.08e+743', '1.08e+743', 91, 4);
    T('5.91e-82273579065905', '-2.7132646255413592255079944773102895402203833683764407930626e+4613868503461', '2.713264625541359225508e+4613868503461', 22, 5);
    T('-9.74e+171050250565180', '4.102470435563033675442693148371792891489183337890561678e-758340204307010', '-9.7400000000000000000000000000000001e+171050250565180', 35, 0);
    T('-8.18445698455525147e-91815218525', '-6.441533624846232245138100e-14123', '6.441533e-14123', 7, 3);
    T('-4.90978306521241473125106067441582402893131187879e-9335715355', '8.87938369375444836e+6418500925', '-8.8793836937544483601e+6418500925', 20, 0);
    T('2.2203678342629095978317482057595096659979558547e+11547799', '-6.75481339e-4542785', '2.2203678342629095978317482057595096659979558547e+11547799', 52, 5);
    T('9.83025418894386314710955218862623579802571204425220828551769e-25623710', '9.5680291666572563417934050e-9969295671', '9.8302541889438631472e-25623710', 20, 2);
    T('-3.41986e-751042275', '6.274086189239135439941978227894200489708815860036343724671e+97066727847', '-6.274086189239135439941978227894200489708815860036343724671e+97066727847', 95, 5);
    T('-1.9053534558261443666914370102996809379900053051427e-609071690703289', '-7.6429517899806807191725117603459060758729126094e+25986796', '7.6429517899806807191725117603459060758729126094e+25986796', 81, 5);
    T('8.4284728421475779061828e-638335998', '-6.8608175023341e-39826570056799', '8.4285e-638335998', 5, 6);
    T('4e-42947685302', '5.3522488298417335e-6740', '-5.3522488298417335e-6740', 35, 4);
    T('-8.1643334796777581425416426084717017297359188e+1107165856', '-7.76840879787172744278904194029200041536118334767669141880e+432918109171', '7.768408797871727442789041940292000415361183347676691418799999999999999999999999999999999999999999999e+432918109171', 100, 3);
    T('-3.742e+44224', '-6.5449374339043960167767491e-16', '-3.741999999999999999999999999999999999999e+44224', 40, 1);
    T('2.1e-60512', '-2.459299770329686e+2889071', '2.459299770329686e+2889071', 97, 1);
    T('1.36554588516676113261218216443575956947e+8148768711', '-3.333031237964069070879105e+47', '1.36554588516676113261218216443575956947e+8148768711', 47, 1);
    T('-8.83020161028755966605806759506e-22895631', '8.51784103431971705651068440691201007e+225725402918386', '-8.517841034319717056510684406912010070000000001e+225725402918386', 46, 3);
    T('-1.5983487933e+27', '-6e+8682027943', '6e+8682027943', 39, 2);
    T('-2.92086932390107957867195190348207637510282012763607e-6746123869', '7.2361064019059e+159094978981', '-7.2361064019e+159094978981', 11, 5);
    T('6.771985832690232217955e+936538', '-1.684957171e-81499593872', '6.771985832690232217955e+936538', 53, 1);
    T('-5.073712838262407e+54808', '-6.1577427231908841377794008143798286e+7223480', '6.157742723190884137779400814379828599999999999999999999999999999999999999999999e+7223480', 79, 3);
    T('4.547564964283821619082866936184607701755250608972645087721e+168269', '-9.0941637826178252158666168277696526147075357e-231', '4.547564964283821619082866936184607701755e+168269', 40, 3);
    T('-5.8765925212050708679917815442e+183973', '-8.9161e-913267124498402', '-5.8765925212050708679917815442e+183973', 76, 4);
    T('2.4420e+9565', '3.66949e+913999733786538', '-3.66949e+913999733786538', 66, 0);
    T('-2.3545002051685382626332004426081349063679e+991810858540', '-2.105081607422193e+95', '-2.3545002051685382626332e+991810858540', 25, 1);
    T('5.69e+556', '7.82296869844101441394e-315', '5.69e+556', 74, 4);
    T('9.9481243826898305326101958916631430495439929760016256210e+874288507688', '583702758101626845342.881', '9.94812438e+874288507688', 9, 3);
    T('-9.127365215527440884222665285082207110634523027680957164678e+39349266', '-2.9585676021935026373761268905950392927495303063160060e-1812', '-9.127365215527440884222665e+39349266', 25, 5);
    T('5.24189255991587248623320043720378740320775e+887490572', '5.2324408104698305107163498198390358e+8954060', '5.24189255991587248623320043720378740320775e+887490572', 48, 5);
    T('-4.187887267335149393921714644931664210e+22617286', '8e+1121', '-4.18788726733514939392171464493166421e+22617286', 78, 1);
    T('-2e+9', '-2.0039170121084714906056e-35172623762981', '-2e+9', 76, 3);
    T('-95385170847396.73', '5.914581567004483e-54', '-9.5385170847396730000000000000000000000000000000000000000000000000006e+13', 68, 6);
    T('-8.086476331161002808173393333130930577316312140808308e-2239743523597', '-3.07e+66595922551', '3.0699999999999999999999999999999999e+66595922551', 35, 1);
    T('-802884.174889662223361681433303247901675969921090146', '-9.117076348e-822', '-8.02884174889662223361681433303247901675969921090146e+5', 88, 4);
    T('5.6881541111294094911e+122', '1.612478329659403772e+49', '5.6881541111294094911e+122', 22, 5);
    T('-8.0874327640e+9855', '-9.910538117244181009816408526876362436822576209480712e+114', '-8.087432764e+9855', 27, 5);
    T('-2.9238365e-978721097', '-5.3908659e+2406682', '5.3908659e+2406682', 94, 2);
    T('-8.085429354945797479863146231e+9172004', '-8.248698857154e+569306097699', '8.248698857154e+569306097699', 56, 2);
    T('6.773876366048456679741e-254499', '-2.36869742185530683029705356039380826233705633748766e+51', '2.3686974219e+51', 11, 5);
    T('6.927411861140364e+1705', '6.12826294080931071852931943220917454787e+130', '6.927411861140364e+1705', 20, 5);
    T('-8.3088954146335315412512802864765728840193096844e-6150415806521', '-8.954735623464121e-81850629', '8.9547356234641209999e-81850629', 20, 3);
    T('8.6157440169425028267026073329034438874770239349e+797791207625', '7.72448185e+3202', '8.6157440169425028267026073329034438874770239348999999999999999999999999e+797791207625', 71, 1);
    T('6.5936205557962006426223571880602e+57462110960', '-0.48676', '6.5936205557962006426223571880602e+57462110960', 62, 5);
    T('-6.488221e-110480796202', '8.77164658497790140362856502591157415763e-223915952173', '-6.488221e-110480796202', 30, 2);
    T('4.7637642380759462213033172220693334121676581370385342e+88483069856', '7.19325989e-21', '4.7637642380759462213033172220693334121676581370385342e+88483069856', 87, 4);
    T('-2.869449e-323146735975548', '1.3178600664930843377160567875499928563573e-2726852751', '-1.3178600664930843377160567875499928563573e-2726852751', 62, 6);
    T('-1.078568859e-59', '-3.13773097e+253132939820', '3.13773097e+253132939820', 30, 6);
    T('-5.050983e+10226844652129', '9.66780034320395412774794594e-18698949660', '-5.0509830000000000000000000000000000000001e+10226844652129', 41, 0);
    T('8.89e-4008029097033', '4.86215238032e-8687832572055', '8.89e-4008029097033', 11, 2);
    T('-8.40834431901519885688419e+871901431162712', '1.2571617269e-32352998', '-8.408344319015198856884e+871901431162712', 22, 1);
    T('-6e-17964221416', '-8.200772341979574451871956353668125136345709484860765010160e+815325332', '8.200772341979574451871956353668125136345709484860765010159999999999999999999999999999999999999999999e+815325332', 100, 3);
    T('-8.55901692167888644732261966129884837e+210489941', '7.4859772174343782070201268229595370e+9353822159237', '-7.485977217434378207020126822959537e+9353822159237', 99, 6);
    T('-8.127373e+37559056914', '2.4749510237153428919417743598528463858205e+44', '-8.127373e+37559056914', 70, 5);
    T('5.3e+758', '-8.45028575389564e-50688200663', '5.3e+758', 75, 6);
    T('-2.0166782450031439203997e-5991410467', '-4.813484273153494e+59088', '4.813484273153494e+59088', 81, 0);
    T('302193.11493', '-1.67643566889366019646443246773884279915869096957725e-86416777923', '3.021931149300000000000000000000000000000000000000000000000000000000000000000000000000000000001e+5', 94, 2);
    T('-7.824903861734536614125671353180355991969179954098e-9503826905325', '2.3227755270033345633086640987381619925431e+109767519987468', '-2.32277553e+109767519987468', 9, 6);
    T('-9.44596140976187667594021817299608478577174016210026e+4845239', '3766969813550986322373024479572226686745078326932135.70', '-9.44596140976187667594021817299608478577174016210026e+4845239', 79, 4);
    T('2.42180078183467994731762476181473970907578129876525e+755896777610591', '9.6063911174e-30138208', '2.42180078183467994731762476181473970907578129876525e+755896777610591', 79, 5);
    T('-6.7737678187e+73844', '3.86872987975274057908584939516676e+47743920146654', '-3.86872987975274057908584939516676e+47743920146654', 65, 4);
    T('2.30831007592178781434376743956243122374e-53097307257972', '-6.3230622146262287142e+482261', '6.3230622146262287142e+482261', 76, 6);
    T('103511114746479227681392.34741392', '-9.426095740341e+83008627262', '9.426095740341e+83008627262', 91, 3);
    T('-4.503155487186052272529060928e-72452', '-6.4860784564573145760502977630731182112143678731e-7742031630', '-4.503155487186052272529060928e-72452', 30, 5);
    T('-6.3450585589084730970487327614992283258191604347178418166e+46668498905', '-2.9264769358789913309226940413357101948245450222355674811713e-589', '-6.34505855890847309704873276149922832581916043472e+46668498905', 48, 4);
    T('-5.324773457621479e+7292551', '1.60446394526578325902922e+29', '-5.324773e+7292551', 7, 6);
    T('-3.295299927727299971962428544784181393301731115727660e+90739', '8.296673752914e+149', '-3.295299927727299971962428544784181393301731115727660000001e+90739', 58, 0);
    T('1.2493170842844930159954638224175322e-1882106', '-2.74792709009144980421977893645133430137223620530766822369e+3641220902573', '2.74792709009144980421977893645133430137223620530766822369e+3641220902573', 61, 6);
    T('1.080118986733308793386713960e-1751356418398', '-6.544549870478515790879005547938313207663631e+354666336392', '6.54454987047851579088e+354666336392', 21, 5);
    T('-9.03782069542442043714550e-47159542295', '-5.709523931659472232198042480146822288977433036222e+1028', '5.709523931659472232198042480146822288977433036222e+1028', 93, 5);
    T('3.537921124760330405337827642586992442457070594605381397e-3143357979', '-4.450853590e+83019', '4.45085359e+83019', 59, 6);
    T('-8.93391856968879304880192013806523547114879838589795269e-1088', '8.64426138771683693866180440121182173302075460045e+637376966', '-8.64426138771683693866180440121182173302075460045e+637376966', 62, 2);
    T('3.414e+25385403647', '-5.267944916095500363598022038e-938786879', '3.4140000000001e+25385403647', 14, 2);
    T('-3.3892e+47252526', '-8e-204487692', '-3.3892e+47252526', 94, 4);
    T('8.22946083538e+12', '-9.1548512e+24773320424', '9.15485120000000000000000000000000000000000000000000000000000000000000000000001e+24773320424', 78, 2);
    T('9.3287559857125611088251218573011621473649202e-13583647164', '-1.85681827278129569474484764548808490316785708895926057748876e+5772', '1.85681828e+5772', 9, 0);
    T('-5.8564582391443233561301486823190810013402132e+152293506957', '-6.964082036444e-6419385120', '-5.8564582391443233561301e+152293506957', 23, 6);
    T('-8.803162144727874518186361e+62393626', '-8.347900346405020011744783746063934600338456e+45', '-8.803162144727874518186361e+62393626', 87, 0);
    T('-8.585588936667e-6709258132', '-3.9773208466513723578291138233756938779440079888570544323e-1367027', '3.9773208466513723578291138233756939e-1367027', 35, 5);
    T('-1.026219630188385013868405400430258234160075084e+54', '8.60688618150519450499563952560855365463525397014438e+9565493369923', '-8.606886181505e+9565493369923', 13, 1);
    T('6.3192676182315166199162824194536033095875763014887800641e+215124252702197', '-6.51e+178599905', '6.319267618231516619916282419453603309587576301488780064100000000000000000000000000000000000000000001e+215124252702197', 100, 2);
    T('3.46803884930060886590383884462345895973428786722540779e+22138361', '5.4038833689686474299585e+6139841728881', '-5.4038e+6139841728881', 5, 1);
    T('-3.79835646617452008898634362354419e+8046', '3.88593604345674160433395711001113633892014796094171447390636e-4688463048359', '-3.79835646617452008898634362354419e+8046', 36, 2);
    T('3.2568733568821370e-133198474247701', '3.660387243310104e-628949893029', '-3.66e-628949893029', 3, 5);
    T('-3.0681557038322674501501024791374e+26030518178892', '1.1625092819246095064186164471208773810681e-23984001194695', '-3.068155703832e+26030518178892', 13, 1);
    T('-5.369714865941334430141469245320125124288244808506e-66843', '-7.2355486441704659166969580199867817247e+132344165', '7.23554864417046591669695801998678172469999999999999999999e+132344165', 57, 3);
    T('1.74478650497137008620845475555918311794090472212473691961e+905227108224745', '-8.9660900250042272178822452960858027034828228063724824330e-730500422', '1.74478650497137008620845475555918311794090472212473691961e+905227108224745', 80, 4);
    T('1.236864567906059173163681133771110786991848872907921985145e+61', '-6.8433441038468077311585e-59376', '1.236864567906059173163681133771110786991848872907921985145000000000000000000000000000000000000001e+61', 97, 2);
    T('6.442627047133527013857199931056359775828462469152426e+2924090697', '-9.4181450585812497023819999531319680453487400253899167496e-812210130', '6.442627047133527013857199931056359775828462469152426e+2924090697', 91, 3);
    T('-1.369778045270692223628461395104e-58800355918', '-7.1e+7873760010', '7.0999999999999999999999999999999999999999999999e+7873760010', 47, 3);
    T('-2.735014750302615150348818855574212270e-7744', '-3.10554794e+4623886442781', '3.10554794e+4623886442781', 97, 6);
    T('2.88932662179305661928982339577891201484454382330432678214149e+1084696', '-3.3887752e+609728763', '3.3887752e+609728763', 42, 5);
    T('19739786425370360733081458131612556816373.5354746', '-8.91110653577e-5829671', '1.9739786425370360733081458131612556816373535e+40', 44, 1);
    T('8.49336726839e-13423', '-6.11975453464166519080676331318743040172375862116e+8835307436', '6.11975453464166519080676331318743040172375862116e+8835307436', 77, 4);
    T('7.765797209367817876429868518678124392390886067e+54', '-1.52739907550269326051433990654020155978e-1291', '7.765797209367817876429868518678124392390886067e+54', 53, 6);
    T('3.14144927922145488797374770060715415454911781857276649401e+174086201230', '1.8040828179131914729126366742306697e+1824603', '3.1414492792214548879737477006e+174086201230', 29, 3);
    T('7.5942021929937969e-6944', '-6.744822511886152339e-3652940', '7.59420219299379690000000000000000000000000000000000000000000000000000000001e-6944', 75, 0);
    T('-5.727449857630974849335308296722250e-61', '1.8207112287440948974510156283e-7568782387477', '-5.72744985763097484933530829672225e-61', 47, 5);
    T('9.2797056743782539558484433479438185231e+399611603', '3.747420608773667666e-65414724409', '9.2797056743782539558484433479438185231e+399611603', 94, 5);
    T('-8.7e-6869991303263', '2.53891562856461042138058312097085752565919381760e+820', '-2.5389156285646104213805831209708575256591938176e+820', 60, 5);
    T('-3.0193726609284055e+62992', '-4.3261580254183353031678750154400220843046e-270118920', '-3.0193726609284055e+62992', 35, 4);
    T('5.025056557220625978393965182e-29078781656046', '1.7923100389768938333207041360772162266175225e-800', '-1.792310038976893833e-800', 19, 5);
    T('9.97377250975550837981312064989771964e+6451', '-9.155888246020909336565936413281843877390e+47', '9.9737725097555083798131206498977196400000000000000000000000000000000000000000000000000001e+6451', 89, 2);
    T('-467977621946470.55', '5.87496544747097688882806127011541070051717e-218', '-4.6797762194647055e+14', 87, 6);
    T('-7.159949496769e+2772493795', '7.899992078841292e+21', '-7.159949496769e+2772493795', 82, 5);
    T('-3.53e-2079114', '8729348936.545209', '-8.729348e+9', 7, 2);
    T('-9.76e+11', '-1.71766731068554126825042407657463725815495685710e-24364645585', '-9.76e+11', 58, 0);
    T('8.60825273337671e+3738561488423', '8.216732871220847597483331783579167996742710131843066872e-14158', '8.608252733376709999999999999999999999999999999999e+3738561488423', 49, 1);
    T('5.88063037352255e+911175645569267', '9.3106731e+1108', '5.880630373522549999999999999999999999999999999999999999999999999999999999999999999999999999999e+911175645569267', 94, 3);
    T('8.29911e+94388375860775', '-3.1e+3', '8.299110000000000000000000000000000000001e+94388375860775', 40, 0);
    T('-6.1837654753006814579848710536331455709651136860309946e+1946', '3.19329783305147286076636e-92289330', '-6.1837654753006814579848710536331455709651136860309946e+1946', 98, 2);
    T('-6.040737632340237491718965440397263089281321329057176499002e+86126515920', '-9.63527662856516910835403701732e+900974633056757', '9.63527662856517e+900974633056757', 15, 2);
    T('6.71415049738177009270376178036272742908e-27', '7.962913632e+7307367', '-8e+7307367', 2, 5);
    T('-4.4800930053312096717354757804228e-910733', '-1.70911854642869173939275552201032404508495716995e+823', '1.70911854642869173939e+823', 21, 6);
    T('4.9392116393241294030540920408681330519738180694304251652e+25401358829', '6.1019632014497451293224686931379099325465441186e+56856482425', '-6.1019632014497451293224686931379099325465441186e+56856482425', 61, 4);
    T('-3.42938844042941090056226833092752854046605617313672861e-73250015', '-5.057976170536e+49450346', '5.057976170536e+49450346', 59, 4);
    T('-9.53888070212305418492930858002782120410971e+374365', '-9.370e-37824864141', '-9.53889e+374365', 6, 0);
    T('-3.638039775051940943907867195958677064846969584167817095822e-3608', '-2e+304567245', '2e+304567245', 22, 4);
    T('-328174.14863661965894067525', '3.63677664130574007050485767789877887772670331854e+46584054', '-3.63677664130574007050485767789877887772670331854e+46584054', 65, 2);
    T('-1.9813135818267024445633466752883902332060002546668049e-5813837138', '2.27201064786167091e-22907966007', '-1.9813135818267024445633466752883902332060002546668e-5813837138', 51, 2);
    T('-2.853250248377206005908329697263799537030895719813735e+350376781739', '6.1612392584e-357256110', '-2.853250248377206005908329697263e+350376781739', 31, 2);
    T('-3.90213444815948167200973376928037970854794070718470208e-782056532', '6.28553177917e+8572', '-6.28553177917e+8572', 37, 2);
    T('-9.16902501284989e+6768845', '4.2093609492327593429483297e+161', '-9.16902501284989e+6768845', 100, 5);
    T('-5.732491191624677421229101517423303e-58186', '-7.639919673862872253874632891080283087148731612151004e+19795665254', '7.639919673862872253874632891080283087148731612151004e+19795665254', 97, 0);
    T('5510779112858558469281.645879767038421', '-4.313935139426523186084135435023626439024401e+664716557', '4.313935139426523186084135435023626439024401e+664716557', 95, 3);
    T('-3.14282140546721726960395382930890560742759e+83', '7.26079581486845879504588305694502753365523818e-12565583395', '-3.14282140546721726960395382930890560742759e+83', 63, 4);
    T('4.759303436095650690672299597236719026351719521799709065e+980072941432463', '5.154731109644712e+232', '4.759303436095650690672299597236719026351719521799709065e+980072941432463', 80, 6);
    T('-2.779604367138605691562413954598369999434250693297e+418', '8.126740714824e+697806', '-8.126740714824e+697806', 65, 2);
    T('-1.68123971948802521e-7754343673268', '6.4874250155547574344331694493752747085e+26052535794', '-6.4874250155547574344331694493752747085e+26052535794', 71, 1);
    T('-1.206416098220704816767819e-825641', '3.574412637055460006748e+68049267075', '-3.574412637055460006748e+68049267075', 81, 4);
    T('-7.70232439081e+87329461187', '7.51040154795294199924140712e+1587165318839', '-7.51040154795294199924140712e+1587165318839', 57, 4);
    T('2.438934170507596055907407169e+82773221', '7.321856820103175294222436006916445453048684909608984031451e-8268155194645', '2.438934170507596055907e+82773221', 22, 5);
    T('-5.3595506283196869803e+569294305', '-3.8411e-287634121944', '-5.3595506283196869802999999999999999999999999e+569294305', 44, 2);
    T('-3.8363081853714195916308580705580880889441720e+102', '-5.64698448752798262649717855368e+268661023', '5.64698448752798262649717855368e+268661023', 71, 0);
    T('2.3001425770e-6587149912161', '7.59776018621834018893006059e+72545', '-7.6e+72545', 3, 6);
    T('6.248430087565182244520565332533298480876492000032147e-8043389', '9.14144368921908662981627950804e+86708', '-9.14144368921908662981627950804e+86708', 95, 0);
    T('7.417376648e-905803857268613', '-2.02059691643897565361230083228378e+87482971448682', '2.0205969164389756536123008322837800000001e+87482971448682', 41, 0);
    T('-2.489145e+9', '-6.47078596808575558367032031143676635e+331', '6.47078596808575558367032031143676635e+331', 90, 0);
    T('-8.63154471570847458e-78507195', '-4.030490685383916551e+6346', '4.03049068538391655099999999999999999e+6346', 36, 1);
    T('-9.0313986679146879920148109894739e-8855951166', '-7e-762031800', '7e-762031800', 59, 4);
    T('2.4203743609122605985209423444056997539762406600368716415519e+5832403503242', '3.897311434093844272922041e+5920523864924', '-3.897311434093844272922041e+5920523864924', 82, 5);
    T('5.657121763300724412400443363114307e+632', '2.3137196323653080621412006319138728366391608277518e-17062', '5.657121763300724412400443363114306999999999999e+632', 46, 3);
    T('3.5232035e+262', '8.79727557687304455308317e+26', '3.5232035e+262', 16, 6);
    T('2.92838243961582867782e-898484203476589', '3.782532786366483120522720005220714785914284e+45', '-3.7825327863664831205227200052207147859142839999999999999999999999999999999999999999999e+45', 86, 2);
    T('6.272992489890035170480633749174274951325175922222343940e+938030813030', '-8.97485165390980989094039115481250400201350019782405440138e+108', '6.27299248989003517048063374917427495132517592222234394e+938030813030', 61, 4);
    T('-4.62951442363870435241816151835753162859320756563253906e+297437431766756', '7.8636213244400903351399756575945461e-75634444', '-4.62951442363870435241817e+297437431766756', 24, 3);
    T('-3.649148435216712014148574132742835e+676', '16584147108068836.072319', '-3.649148435216712014148574132742835e+676', 99, 1);
    T('2.86744745e-47045556104', '-2.86271029873557101431237973596e-3136', '2.86271029873557101431237973596e-3136', 52, 5);
    T('2.230457485196708058335485930594547881404191291177609869e+4550172653', '-9.937267338226243530775750997586198102719808427264686e+101', '2.23045748519e+4550172653', 12, 1);
    T('-1.6088717651286855079324663e-264059723951565', '4.1e+938050', '-4.1e+938050', 73, 4);
    T('9.523453631140033290085916547152312e-236068', '-3.5e+992039', '3.500000000001e+992039', 13, 2);
    T('-8.2292930611320692587144785797228140726325279696073606e+195', '-9.51427305245073e+399867659904', '9.514273052451e+399867659904', 13, 5);
    T('3.4855791067046915176033661840018055962534860061404e+7745', '6.42e+49', '3.4855791067046915176033661840018e+7745', 33, 1);
    T('-9.70625364494823480320614404316e+224752', '1.1821982819707e-334176', '-9.706253644948234803206144043160000000000000000000000000000000000000000000000001e+224752', 79, 3);
    T('3.354532214064e+103872', '5.2e-12', '3.35453e+103872', 6, 6);
    T('7.436777908571435941430934949373489e-692', '-2.5694086820517795152896901323131e-53921599469', '7.436777908571435941430934949373489e-692', 53, 5);
    T('4.4806764015266269761850439e-74691', '7.56577382338391613e-151759', '4.4806764015266269761850439e-74691', 74, 2);
    T('-9.7632959462e-32000147964', '-2.77e+986254007', '2.77e+986254007', 70, 6);
    T('4.8e+28375764', '5.885147287672172225052958011117438127658630e+95', '4.8e+28375764', 92, 5);
    T('-6.756556550419967762992853095216934895982e+59659126352776', '3.817662845180140947906187301195777560477821722928256069278e+93185342', '-6.756556550419967763e+59659126352776', 19, 3);
    T('-8.3683124136043907053783999743925540169323547443534812019051e+81278', '458387101454616625672981597353302499', '-8.3683124136043907053783999743925540169323547443534812019051e+81278', 67, 2);
    T('-5727969477360862.62', '3.5501943226954851192139609389361143688090e+812093583739', '-3.550194322695485119213960938936114368809e+812093583739', 89, 6);
    T('1.3123e+43828813475', '-2.1621904211470639e+48877144716289', '2.162190421e+48877144716289', 10, 1);
    T('-2.8584710335566169618e-967561046758233', '5.98e-25429723120480', '-5.98e-25429723120480', 38, 4);
    T('7.054446825658337165084474e-5349820', '-3.6077952900700715434866140571463526174e-34885', '3.607795290070071543486614e-34885', 25, 1);
    T('6.311473177576288032828573952770947e-81616465367', '5.7476929110441941834e-2295', '-5.7476929110441941834e-2295', 22, 5);
    T('-6.7676000e+720747063', '3.18340980281641113797528363318447303044733486706866123e+61471490', '-6.767600000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+720747063', 100, 3);
    T('0.23484', '2.308277e-75310390841597', '2.3484e-1', 77, 2);
    T('-1.827013297467066253652104781474028894320743e-632750550486', '6.060648851108726729667801745858863358e-91', '-6.0606488511087267296678017458588633580000001e-91', 44, 3);
    T('5.7123e-60169104927', '6.3679823668393305779188424131606976e+6692327414579', '-6.4e+6692327414579', 2, 0);
    T('3.11e-32513441426', '9.048199782223667975198065214005836594242672402531928e+78949020', '-9.048199782223667975198065214005836594242672402531928e+78949020', 93, 5);
    T('3.762096996208011071e+557349640', '5.23859566037514352785883284e-70540118', '3.762096996208011071e+557349640', 48, 5);
    T('6.362166358626479651995e+439288807485', '-3.075969421583665055351605002973786941e-207696818421', '6.362166358626479651995e+439288807485', 32, 6);
    T('-7.78290e+848109724761793', '-1.99588602316913340170992120310569700616272684744003062434e+134', '-7.78289999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+848109724761793', 96, 1);
    T('1.000274738509515273404017227770194290889601729602944e-14086170801', '4.923385162209547452722695960596204306736579675215102936096e+30049894', '-4.9233851622095474527226959605962043067e+30049894', 38, 5);
    T('9.3164e-2833568262', '9.06455806580947213953e+913703927159622', '-9.06455806580947213953e+913703927159622', 28, 6);
    T('9.57421e+1914179131874', '8.46986957e-93674289620', '9.5742099999999999999999999999999999999999999999999999999999999999999999999999999999999e+1914179131874', 86, 3);
    T('-3.475052663576539529966294114182155e-2043632676273', '6.59131187053004084755e+23', '-6.59131187053004e+23', 15, 1);
    T('-4.19852688280753626982482979042693461831509836253340e-773050059', '-6.302307360060051238258949e+428814', '6.30230736e+428814', 11, 3);
    T('48127135580.8553027942608852796823455323246672253616372', '9.6119695215e-849067692', '5e+10', 1, 0);
    T('5.16e-6582014446485', '-3.11239458658263038463822456303294155167666376222780704448e+4988115', '3.11239458e+4988115', 9, 1);
    T('1.4594430764565198005027022636269140e-366008681', '7.732275549894340238134657e+30', '-7.732275549894340238134657e+30', 86, 4);
    T('-5.7474977169945394137072439402842710658674361151485860881957e-3433', '-9.86390724832474107874765551081420224559e-277', '9.8639072483247410787476555108142022455899999999999999999999e-277', 59, 1);
    T('-1.1e+943', '9.3999132007878872e+640', '-1.1e+943', 64, 6);
    T('8.9959439e-3296577456321', '2.81e-3582143', '-2.81e-3582143', 39, 3);
    T('2.767434266209348823422566258486616637405828253e-2379977', '-6.766401272663126861e+1281947125', '6.7664012726632e+1281947125', 14, 2);
    T('-8.7380881e+63106530538', '3.0695074722208837904055607883006836430163433599e-462', '-8.7380881e+63106530538', 57, 4);
    T('8.6942161440085592096328719162157980790386261392192864028768e+365368', '5.6974088841916056461779685571e-870736016157607', '8.694216144e+365368', 12, 3);
    T('-4.11371e+576047376', '-2.4914110798635841195e+27', '-4.1137099999999999999999999999999999999999999e+576047376', 44, 1);
    T('-5.43227206169523994498007770126844184465221419311e+121', '-6.58006031e+4601283', '6.58006030999999999999999999999999e+4601283', 33, 3);
    T('-7.37417940099406423781862101704e-48869632', '-4.429263715664414537853219807929389966e+824493770', '4.42926371566441454e+824493770', 18, 5);
    T('-7.647355741537146036807560701818297381941517220e+3606941', '-7.81203867212049136036416788635748527599197738292363616e-8597224582012', '-7.64735574153714603680756070181829738194151722e+3606941', 86, 6);
    T('-3.49370528078724007792504735134e-48748749204176', '1.7739832821439e-162370096167572', '-3.49370528078724007792504735134e-48748749204176', 35, 1);
    T('5.1790830830242e+20', '-3.7425507605608401261623585222206545e+760500', '3.7425507605608401261623585222206545e+760500', 80, 4);
    T('-9.548458433671420533699353163679236e+5368645019', '-3.19e-734197541', '-9.54845843e+5368645019', 9, 6);
    T('-8.904772020650826391057864658079e+11514669844', '4.61664033744365772e+23', '-8.904772020650826391057864658079e+11514669844', 93, 4);
    T('9.8532324439240981490509e-727', '-5.2957649095936074122303719842128e-45891392188', '9.8532324439240981490509e-727', 52, 6);
    T('-9.6742821315043131e-9572398732426', '-5.4399052445648081395e+221362519259', '5.4399052445648081395e+221362519259', 56, 4);
    T('2.114029219960083106479145e+93343431', '793203349539414001.4', '2.11402921996008310648e+93343431', 22, 2);
    T('-6.646790204332596882070057670661487156258472400985e-63625341', '-5.837e-3427192298149', '-6.6467902043325968820700576706614871562584724009849999999999999999999999999999999999999e-63625341', 86, 2);
    T('-5.3606343129933654194763501116240432794841e-36210555474455', '-4.6232027138655163777900044126772375208801503494272075220e-500', '4.6232027138655163777900044126772375208801503494e-500', 47, 1);
    T('-5.163214286118385404055487715e-92955853896', '-3790652635782.4', '3.7906526357824e+12', 89, 2);
    T('-4.45143e+951495', '7.9663269150187679862429446e+960873444', '-7.9663269150187679862429446e+960873444', 78, 4);
    T('-1.59067e+848', '4.11872210e+100344608876', '-4.1187221000000000000000000000000000000000000000000000000000000000000000000000000000001e+100344608876', 86, 0);
    T('-5.43173250337939509429696480773164833e+83396', '-3.716568e-679', '-5.4317325e+83396', 8, 4);
    T('2.3942428460863991272627936455202357666883e+3232985918553', '1.3745042363e+86413561', '2.3942428460863991272627936455202358e+3232985918553', 35, 4);
    T('6839472404043904745826466187358.34237914', '2.5388735e-8664790355412', '6.83947240404390474582646618735834237914e+30', 98, 4);
    T('6.06621836379380679504328338912511e+72438725', '9.169736574129817e+19', '6.066218363793806795043283389125109999e+72438725', 37, 1);
    T('1.8343207538644441121151922997154056729318e+1834326092460', '-6.678019829019767167611824952470055818928876303804841762294e-65732132928899', '1.8343207538644441121151922997154056729318e+1834326092460', 70, 4);
    T('-3.7961831393155747832649980e+137595328', '-21543435642194454673640883.78395948967269844530', '-3.796183139315574783264998e+137595328', 73, 5);
    T('-71334654433341016881297995.7147243', '7.917803633073255432025286545e-44924302349003', '-7.13346544333410168813e+25', 22, 6);
    T('5.18625552892698013162533966154870583341625162276543810045e+310710640930635', '-1.9566398750911452776e-349', '5.18625552892698013162533966154870583341625162276543810045e+310710640930635', 86, 5);
    T('1.32729975812720253101212494778280749e+4776', '2.723410614149697224461655580971004225225e+47', '1.32729975812720253101212494778280749e+4776', 70, 0);
    T('4.834661104491473241439686855158392357868594741571465732e+31302660926', '-7.860596949572e-72300', '4.834661104491473241439686855158392357868594741571465732e+31302660926', 58, 5);
    T('-4.829522444852211895394350668740966322173278570961345459e+1368368434', '8.10022e+4142506651', '-8.1002200001e+4142506651', 11, 3);
    T('-4.7714917304184e-466145', '-1.249823989611424317976615643496287e+90077304', '1.249823989611424317976615643496287e+90077304', 52, 0);
    T('-5.9250339e-335026', '-1.81028510210544580828924564393492490578e+262', '1.81028510210544580828924564393492490578e+262', 83, 0);
    T('-6.75088466941060795501328825e+9679661695908', '5.831334230e-43856735161688', '-6.75088466941060795501328825e+9679661695908', 89, 6);
    T('-1.6813066507e-17647659016198', '7.837502915025e-910922874', '-7.837502915025e-910922874', 77, 1);
    T('-4.50430765044313e-377', '1.01514645219441605786518604735162435468859e-1818757645317', '-4.50430765044313e-377', 94, 2);
    T('7.3486288636369330981964380408414073124e+206282137', '-4.59067186883940124606601454614618932809823763147e+699592848609035', '4.590671868839401246066014546146189328098237631471e+699592848609035', 49, 0);
    T('3.88667847e-7013563', '9.94655233374e-40', '-9.94655233374e-40', 70, 3);
    T('-1.72953139427670795048432718418843697843530855755625e+43721196', '7.8e+9721625028', '-7.8e+9721625028', 90, 2);
    T('2.0521864391256682472660499671113149e-609001601031711', '-4.746333466412630183697070449695827380453711518045182e+9239882', '4.7463334664126301836970704496958273804537115180451820000000000000000001e+9239882', 71, 2);
    T('6.95353e-3588842590', '8.5539596447098743849e-286069269822', '6.95353e-3588842590', 12, 0);
    T('3.03586479096631210504065e+27717551133986', '8.83091e-952588339319164', '3.035865e+27717551133986', 7, 4);
    T('8.18715963749304093116e-42086862838', '59389916968629954397920095106927231418614892.27', '-5.93899169686299543979200951069272314186148922699999999999999999999999999e+43', 72, 1);
    T('6.76316683587350923077360697965e-850989361', '-3.54672313e+676652', '3.54672313e+676652', 88, 6);
    T('-8.51e+1843301', '6.3e+354458423412481', '-6.3e+354458423412481', 53, 1);
    T('6.08129070378777172270927076830587953761e+446634371555458', '1.49937206263484741110160086771875531791551039e+10944983', '6.0812907037877717227e+446634371555458', 21, 3);
    T('-3.498542855611317451276572943618758e+344773623594472', '-2.65333662790310545e+75634862661230', '-3.498542855611317451276572943618758e+344773623594472', 36, 6);
    T('-7.06236449567750695749206491162512687033959043906e-21', '1.42e+7027', '-1.420000000000000000000000000000001e+7027', 34, 0);
    T('-8.1603825320098511944727281944143120385337488980e+28138', '4.71848e+809234469511935', '-4.71848e+809234469511935', 50, 6);
    T('-9.357709270671e-74172', '-3.99429907630040721098339038886362274230350609707193583e-87685', '-9.357709270671e-74172', 45, 3);
    T('-5.80264e-428046446625', '7.338641626466420806853816869529203689523688382509e-15924543711185', '-5.80264e-428046446625', 21, 4);
    T('19407344', '7884870969214657512284489749206.57783967182840', '-7.8848709692146575122844703418625778396718284e+30', 59, 5);
    T('2.811634405615552753506546083860357336046325519149e+556', '-7.9159851892657991e+2788433236815', '7.9159851892657991e+2788433236815', 80, 1);
    T('-1.728186520374151078588595992007518136086027542354e+500090', '-1.85784350107156179604294588277479590219e+8230365593367', '1.8578435010715617960429458827747959021899999999999999999999999999999999999e+8230365593367', 74, 1);
    T('-5.9381537573568e+87199322327', '3.60068055571077918344191220790494158159447258e-83616', '-5.939e+87199322327', 4, 0);
    T('5.69106e-3225', '6.3713025435620589574407789163877919246580e+53290003888957', '-6.371302543562058957440778916387791924658e+53290003888957', 71, 5);
    T('-6.65947621080382970587161029529812438506884562312271e+231684447024', '8.67371518927828122198766845666329e-26', '-6.659476210803829706e+231684447024', 19, 0);
    T('3.303057437810867e+902865659', '-1.0211e-475', '3.303057437810867e+902865659', 20, 5);
    T('-8560044026911.5979819', '-7.266458e+468', '7.266458e+468', 96, 5);
    T('-4.13830812782e+84700612', '-7.97069861061164645299034866895034401922719590865e+87', '-4.1383081278199999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+84700612', 98, 1);
    T('-1.2334380e+8946', '-7.783642943076403479979879101119641665166029e+8353', '-1.233438e+8946', 91, 4);
    T('1.7816713600701032756668528319155e+978261126344056', '-3e+8744504', '1.7816713600701032756668528319155e+978261126344056', 68, 4);
    T('-9e-550249', '1.33539861096378651e-16159550836585', '-9.00000000000001e-550249', 15, 0);
    T('4.83628784339818095626635643577068862e+21411', '1.53402458406815607596397066392732372884130627886995156673e+57', '4.8363e+21411', 5, 6);
    T('9.19051698e+442', '-9.929789e+15', '9.19051698000000000000000000000000000000000000000000000000000000000000000001e+442', 75, 2);
    T('-1.6396888650598984e-26913', '-6.85811277677652709774157370e-78604', '-1.6396888650598984e-26913', 68, 5);
    T('-9.72724736061735402326186480241398939309740458607973142e+375946445', '2.7447312587499672627e+432494611768', '-2.7447312587499672627e+432494611768', 50, 2);
    T('-4.532846728266511089538243290777052e+2429712508648', '-4.44217566984070e+32866470031865', '4.4421756698406999999999999999999999999999999999999999999999999999999999999999999999999e+32866470031865', 86, 1);
    T('-8.78617240880091689560258093284354653379e+953855171183', '-8.893176184935645637910349355421314961937787895e-19978559602473', '-8.78617240880091689560258093284354653379e+953855171183', 68, 4);
    T('-9.940862808845773713105541023265895933308956324984715504e+823224614950330', '-4.7954854100425150423330827151046076474e+8151530217', '-9.940862808845773713105541023265895933309e+823224614950330', 40, 5);
    T('9.604169335e-2366167', '1.705772726695543976489e+96147172428', '-1.7057727266955439764889999999999999999999999999999999999999999999999999999999999e+96147172428', 80, 1);
    T('-9.0439563034351961704940291112049864962123021384e-18032', '3.048235471166845686703133452557615271969e-5724935704', '-9.0439563034351961704940291112049864962123021384e-18032', 53, 5);
    T('-4.73521627673746583548e+713', '84489675646363328890860.43', '-4.73521627673746583548e+713', 86, 5);
    T('-1.351530945911521023e-29404329', '-7.689769898247965398748297629054548877192265939953e+60', '7.689769898247965398748297629054548877192265939953e+60', 98, 5);
    T('8.278757511857e-47402421923212', '-6.38087813524843783963244471582694874681441531135e+62716676', '6.38087813524843783963244471582694874681441531135e+62716676', 56, 3);
    T('1.4804259906293e+5399967257', '-6.31444000706e+17', '1.4804259906293e+5399967257', 83, 6);
    T('-6.690367352541966257656563237697018942125109083e+7058117887985', '2.9807323408620300083922378795647598623599552886132211958e+1619231320314', '-6.690367352541966257656563237697e+7058117887985', 32, 6);
    T('2.01811500822142742647193187119e-4367599581', '-2.0876344583463717948615056559373439141720e+43', '2.08763445834637179486150565593734391e+43', 36, 3);
    T('-5.77421209957728593669427125174339851758344377047855553e+761324791159624', '4.26621599766525336655982545876638e+2792992781362', '-5.77421209957728593669427125174339851758344377047855553e+761324791159624', 81, 4);
    T('8.566100031022e+63', '7.6752677367229727082728586e+74', '-7.6752677366373117079626386e+74', 42, 6);
    T('-4.07993031986925581573939041446504019724855248654324993992e-2490', '-9.91728941e-180504851', '-4.0799303198693e-2490', 14, 0);
    T('-8.179544331762842126646323419389e+555365416031453', '-2.43866218246409056620157942e+414538055', '-8.179544331762842126646323419389e+555365416031453', 39, 5);
    T('8.240006591504157699797439155543250e+828', '7.501458016248657583106358105439855506896198021e-77487686677279', '8.24000659e+828', 9, 4);
    T('6.787525791360194130874300438732889e-72', '-1.3184209821e+205097614627', '1.3184209821e+205097614627', 88, 3);
    T('-7.6102943955794447e-723792', '-5e-9783', '4.99999999999999999999999999999e-9783', 30, 3);
    T('5.108058752955239271865823812416539824963287656465065416456e+117', '1.4711135383876119302283760180740097425654758300550585e+140', '-1.4711136e+140', 8, 3);
    T('-1.993720534820023259072487648028576584481790718443235824e+22456600528', '-6.67865719111284749717623713387179380501846442245235e-69958556834199', '-1.993720534820023259072487648028576584481790718443235824e+22456600528', 83, 4);
    T('2.808318780743727730583865503583955818351770e+121', '5.6455300415476703776825186767942952e-83008058', '2.80831878074372773058386550358395581835e+121', 39, 1);
    T('-5.4403172505262072634474613909810059975049865e+204708067196', '1.694952998516310366251474281908815128530558180258209388630e-45875364625', '-5.4403172505262072634474613909e+204708067196', 29, 2);
    T('8.344640323192921681181666276561987412511126e+46', '4.164515729298088381504681286050e+2945', '-4.16451572929808838150468128605e+2945', 40, 6);
    T('3.68049e+968', '-8.789783267579818e-770542988934730', '3.68049e+968', 30, 6);
    T('1.72048e+573', '8.1962569316397612758e-293356', '1.72048e+573', 23, 6);
    T('-4.24252646023766319285286363464912585986945707238533979454e+43769175091636', '5.0302553411331072209135902e-956164149144', '-4.24252646e+43769175091636', 9, 6);
    T('-4.6510669949775634219286650851694405897904364364963374418e-8608418', '-2848809441668393653923364905359361042200942025.25582', '2.8488094416683936539233649e+45', 26, 4);
    T('-5.557361304117894263292160739130377684795389388940942e-68786856', '6.71484880599344056750907527646206894868766e-98166', '-7e-98166', 1, 6);
    T('-7.23101914880939112948458332e-40987566980', '-8.795683876598449527788639e+79917056616134', '8.7956838765984495277886389999999999999999999999999999999999999e+79917056616134', 62, 1);
    T('-2.75193907084518211364119674775681499598689356492e-8241450002385', '3.96178e-630158260', '-3.96178e-630158260', 85, 1);
    T('9.1288151775073724758702445298274811e+1017', '-4.8849070617e-92', '9.128815177507372475870244529827481100000000000000000000000000000000000000001e+1017', 76, 0);
    T('-3.624241640866541594058564009735e-3517', '4.111120333881296987226108736835149e+679636658325532', '-4.111120333881296987226108736835149e+679636658325532', 81, 6);
    T('7.4208562473230391801018245841833968935002e+616213452411', '1.3261576331525007712583894e+614', '7.4208562473230391801018245841834e+616213452411', 32, 6);
    T('-4.18655980433114e+5720221973157', '-8.94380951607533513775377872485738748e+63', '-4.18655980433114e+5720221973157', 34, 5);
    T('3.02671808658380465658177384234845029877920e-86973781749', '5.5752424864165594674518689237e-372025830780100', '3.0267180865838046565817738423484502987792e-86973781749', 100, 0);
    T('-8.625618357751666622264312370968092040933903461980964779346e-911252446946', '4.9878036310042940757819827778536237e+983995', '-4.98780363100429407578198277785362370000000000000000000001e+983995', 57, 3);
    T('8.436306718301e+323080', '6.70168674894777890370e-728079022861986', '8.43630671830099999999999999999999999999999999999999e+323080', 51, 1);
    T('-9.5918817464761e-567766598324219', '-1.086032453702773e+83148289985', '1.086032453702773e+83148289985', 44, 0);
    T('-1.181530494260966766189225697928657731840645e+5233930', '-6.33644370608930547396106319764973816167510178e+44654154829159', '6.33644370608930547396106319764973816167510178e+44654154829159', 88, 5);
    T('2.40131529854968258581390090247121763e-69745837186', '-6.559029e-62335269934', '6.559029e-62335269934', 28, 5);
    T('-5.59423149553986133159988221653e-7829442912', '9.0942898635047525433896304528234553799611304296e-968177', '-9.0942898635047525433896304528234553799611304296e-968177', 90, 6);
    T('-9.366524268745916273156274255889792130669691e-439085', '7.83019671897017238131e-26532550841', '-9.366524e-439085', 7, 6);
    T('6.36077179349976710953954940086497838876755829e+9263', '1.71221646292005617299911e+35367893', '-2e+35367893', 1, 4);
    T('7.816375693654064821836403076074463103884269231038686536733e-9078', '-2.104268e-777485339894669', '7.8163756936540648218364030760744631038842692311e-9078', 47, 2);
    T('-4.66028859304662127244579583232009214296244400261e+8299574562108', '-2.81723756132556761035242061543797022e-24821', '-4.66028859304662127244579583232009214296244401e+8299574562108', 45, 0);
    T('8169236.85039', '49772001240831037201471203715045441.193', '-4.97720012408310372014712037e+34', 27, 1);
    T('4.9309225840514944869e+999652', '9.4025749598154862310532979674056569481534e-394', '4.9309225840514944869e+999652', 58, 5);
    T('-6.4766525524118822394048695918506183e-660885', '-7.4710541531e-6890902', '-6.4766525524118822394048695918506183e-660885', 85, 3);
    T('-5.691528301e-289636575404', '-1.9e+52673354288', '1.9e+52673354288', 3, 4);
    T('9.873e+148', '-7.95129320072837264553134125308881726096066683e+37234934132', '7.951293200728372646e+37234934132', 19, 6);
    T('3.14339538806790098096743747514228022796885054365e-483498', '-8.6479033336167709578156589425517405063877647023e+94736216', '8.6479033336167709578156589425517405063877647e+94736216', 45, 4);
    T('2.381697850457572e+760325181284716', '6.9352372392368103e+72360261073', '2.381697850457572e+760325181284716', 57, 5);
    T('-7.395336644e+95740', '-3.76022059696270963501e-26498046802865', '-7.395336644e+95740', 57, 5);
    T('9.291501277711725532574683890446e+143391', '-9.438469763607377477797227148280e+87235087717000', '9.43846976360737747779722714828000000000000001e+87235087717000', 45, 2);
    T('-2.49743589924942974969277890790768477639575873975036150e+98361', '-6.901475597756600836e+88653852223', '6.901475597756600835999999999e+88653852223', 28, 1);
    T('-2.574851861733490921920528824417953559807135437338954695e+226793', '-4.880393777166306922981496862329e+44385321', '4.880393777166306922981496862329e+44385321', 89, 5);
    T('-3.042813740331601131567e+7402088958', '-8.042827e-93825', '-3.042813740331601131567e+7402088958', 82, 4);
    T('2.3849971540398039e+17', '-7.248539606e-925041018', '2.3849971540398039e+17', 89, 1);
    T('-7.7598491615718e-27', '-8.13051599e-76031117791338', '-7.7598491615718e-27', 67, 5);
    T('1.7066864673360143651294537710474112121e+10780028', '-7.606221614886271028651510240283948280', '1.70668e+10780028', 6, 3);
    T('-2.35168141254571562324688047893618668945882619727552874e+114', '6.2652553286129638405904998555905771100e-209575719525', '-2.3516814125457156232468804789361866894588261972755287e+114', 53, 6);
    T('-1.0e+44623546724399', '5.7744370407226857192146321242e+106', '-1e+44623546724399', 19, 4);
    T('2.378083481872100373979174147e+643824485262', '7.678853084619258e+795482265052', '-7.678853084619258e+795482265052', 19, 3);
    T('1.792163586672903881209e+71179725767901', '-4.9e-576957', '1.79216358667290388e+71179725767901', 18, 4);
    T('6.25713279679967395e-4355754112745', '6.9725540690855781e-4157', '-6.9725540690855781e-4157', 88, 4);
    T('-3.6896515181173744960478277088590568570425807887508e+72590950766', '-9.52342578720213e-66314295045104', '-3.689651518117374e+72590950766', 16, 5);
    T('4.41273144338291882340951762816654214213369153867244980710e+88013', '-6.031323999467818868921129116756074787854529210784343159e+30853771838068', '6.03132399946781886892112911675607478785452921e+30853771838068', 46, 1);
    T('5062796982513169917085.02', '1.02342019495391101079862890279498135646555459e-32', '5.0627969826e+21', 11, 0);
    T('3.344142190214229515377050899628e-1861588', '3.18000e+31518484', '-3.1799999999999999999999999999999999999999999999999999999e+31518484', 56, 1);
    T('-4.4169475128604762716670550084413293978578636612e-5882986519', '8.8811506650708009929559604757261308192325713218e-2470668', '-8.8811506650708009929559604757261308192326e-2470668', 41, 4);
    T('-1.6302719484822486416756920784e-894', '-8e-2369941002', '-1.6302719484822486416756920784e-894', 29, 3);
    T('-4.082663599488459142518809655028047060450589498216727527e-8630544532713', '7.8341677453383203527e-829', '-7.8341677453383203527e-829', 59, 6);
    T('4.2221527698767351866467394152513759542e-804', '1.1553037368065463168473144488421530459600215745225193086651e-1762380195', '4.222152769876735186646739415251e-804', 31, 4);
    T('27656832373902.69292179413446274662707405177088588109', '9.29164106299745785075140186628402228373466175078451e+4231946524077', '-9.291642e+4231946524077', 7, 0);
    T('6.2643697578754216265090996290569248916e+81939945688', '-5.7160700878411700254980417562286631915885e+416894376419', '5.7160700878411700254980417562286631915885e+416894376419', 51, 6);
    T('-9.26395e+27082325385410', '1.4e-260877541914', '-9.26395e+27082325385410', 84, 1);
    T('-4.04966740634220435764781679072437354e-4208484598158', '-8.1623935933209331188014681117313755041864374714791e+124', '8.162394e+124', 7, 5);
    T('473793093124675927666486850', '-8.118320766212597226167689392599783506797e-63167776', '4.7379309312467592766648685e+26', 54, 3);
    T('-5.3858051462e+373', '7.091161248e+20407416', '-7.091161248e+20407416', 80, 2);
    T('-318673593.026550816', '1.78907168e+8757335', '-1.7890716800000001e+8757335', 17, 0);
    T('5.762747961712470e-68622394666', '4.4589161690368265194e+106', '-4.4589161690368265194e+106', 46, 5);
    T('-7.35078e+9333670077198', '3.4227230010775639747197705906267909726583882662010886331e+63', '-7.35078e+9333670077198', 82, 2);
    T('5.8971066853835e-22876', '-1.99966244554893495e+125132953877', '1.99966244554893495e+125132953877', 24, 4);
    T('-8.22456830879177966639629880568569219263e+987261059673', '8.99608224289157e-5826428175', '-8.22456830879177966639629880568569219263e+987261059673', 40, 2);
    T('-6.6294373317596165430902773276118264e+8523988', '8.03914231130356090076652931061e-1840048119703', '-6.62943733175961654309027732761182640000000000000000000000000000000000000000000000000001e+8523988', 87, 3);
    T('7.30237091629992979914761520959143632077909403e-7517923509', '8.07187226261420756117606257009638797549659129569396436662117e-58072318', '-8.071872262614207561176063e-58072318', 25, 6);
    T('-9.160e-40428922809', '-6094022222887375760957.40579413', '6.09402222288737576095740579412e+21', 30, 1);
    T('-8.94192861988373932e-62203002251', '-1.2482941183900075627629757239314037e-2845', '1.2482941183900075627629757239314036999999999999999999999e-2845', 56, 1);
    T('8.493439693812217864892044629095e+1215', '4.788117912793114e+27701597', '-4.788117912793114e+27701597', 83, 5);
    T('4.7201522754284936187241159224723065074721132042e-796', '-2.861741907556448209354950865242901272264393507e-5789', '4.7201522754284936187241159224723065074721132042e-796', 54, 5);
    T('-3.85024e+31021210', '2.31721e-44441043721745', '-3.85024000000000000000000000000000000000000000000000000000000000000001e+31021210', 69, 3);
    T('1.122227312709949968473822134880765574914143e+732', '7.07819238e+26', '1.122227312709949968473822134880765574914143e+732', 61, 6);
    T('6.7058244712691592296555508156065e+64230', '-7.5236176836695382711155695576815403369635818949067025e+127', '6.705824471269159229655550815606500000000000001e+64230', 46, 0);
    T('-7.5652129750e+66120903', '3.21939701364493584369654e+652', '-7.565e+66120903', 4, 2);
    T('-3.56796551189076765574227951304137534054259800617068666e+3089151352014', '-4.0331879327592074534151e-41652681', '-3.56796551189076765574227951304137534054e+3089151352014', 39, 5);
    T('9.9801154920070153773926582727e+7152605087', '-9.82048675614214546822239802254799011e-77197883', '9.9801154920070153773926582727e+7152605087', 39, 3);
    T('-1.918187997891641973800543793772806839929e-7362734', '9.1757e+88191385', '-9.175700000000000000000000000000000000000000001e+88191385', 46, 3);
    T('4.5257280558568110692856347596085480061385392522910310e-3271', '-4.96967e+4048', '4.96967e+4048', 22, 5);
    T('-1.05536778013204011827540115218628866892232e+547607', '-6.67818386950689945579e+9810882581', '6.67818386950689945579e+9810882581', 55, 5);
    T('2.8194111e-1580740138', '2.950987e+359200', '-2.950987e+359200', 32, 4);
    T('-9.40270937137684326561719918096617871051157449219559e+1951655', '-6.894681620333986767415522646799959939184068584375184713e+4733595', '6.894681620333986767415522646799959939184068584375184713e+4733595', 75, 4);
    T('-3.341e+61867941488', '7.8670160515521721324302163719970767159945e+47', '-3e+61867941488', 1, 2);
    T('-3.6125106335161194710244781386836295905128151509787077694367e-86164359037', '4.1382591208167e-668457', '-4.1382591208167e-668457', 40, 4);
    T('-5.4e-161670456', '2.6203886351881518718060100531205231e+52067298', '-2.62e+52067298', 4, 4);
    T('7.77553582921367595659720561015860186359878987655123568e-8312582816', '8.624e+71463245689', '-8.624e+71463245689', 58, 3);
    T('-4.708713755209414577767499345e+42402539074', '4.845144900905129735616778210413283280457404348552121e-5350', '-4.7087137552094145777675e+42402539074', 24, 5);
    T('-7.955497426744252250902217e-956639368765310', '5.501679e+15', '-5.501679e+15', 41, 4);
    T('561518614552079521801336307821.723037', '5.57770580272708e+74308196515597', '-5.57770580272708e+74308196515597', 83, 3);
    T('8.603612694494864770e-4397795454521', '-6.68414713713126383200642560354681416158792852576244645196622e-82165948589314', '8.60361269449486477e-4397795454521', 45, 4);
    T('3.945707116428470630747585205281800282641936919563510154e-9714149', '5.1503112120280170805738165000683424395409e+66072', '-5.1503112120280170805738165000683424395409e+66072', 90, 0);
    T('-8.45625092919738100304077565020331131389580e+5745462042', '9.27e-60383828727206', '-8.456250929197e+5745462042', 13, 1);
    T('7.03002587579872347866387572811608794573280e-19', '-9.1191833416935925131536717313e-30781', '7.03e-19', 4, 5);
    T('7.28648333296662553231016465174352812111622089717943e-687407582', '-9.660174e-717482696', '7.28648333296662553231016465174352812111622089717943e-687407582', 74, 3);
    T('3.008194397625815017646301021418373451140137104393911493801e+850334', '9.3445261390488261301894266565576403647036810e-764310', '3.00819439762581501764630102141837345114013710439391149380099999999999999999999999999999e+850334', 87, 3);
    T('9.3223779336e+42476106334', '3.0364e-314', '9.3223779336e+42476106334', 64, 0);
    T('-5.79e+8880605', '5.795e+75971709871719', '-5.795e+75971709871719', 29, 5);
    T('9.32574303702888338623504514e+39969913430161', '8.4301958853400812593682484199571409665560428436e-54266317', '9.32574303702888339e+39969913430161', 18, 6);
    T('4.2117641764e-9189839562', '7.6135960836e-97765', '-7.6135960836e-97765', 22, 0);
    T('9.3432591437625956729233450855e+7799450671', '1.0839090972988929206404746303250413767283984712804297983807e-615401', '9.343259143762595672923345085499999999999999999999999999999999999999999999999999e+7799450671', 79, 3);
    T('-2.705647363068318922443e-5137', '5.56653682412273591585123001862e-71931436856', '-2.705647363068318922443e-5137', 24, 4);
    T('-8.17902066239653482894386275540073762422472439163509044242138e-355', '-1.484859633200777516898963051451107919929465352402e-867627509', '-8.179020662396534828943862755400737624225e-355', 40, 4);
    T('-1.39430291314805183e-62345512303590', '-1.20484071432e+84490274', '1.20484071432e+84490274', 70, 4);
    T('9.05918910194209451e-7765376', '7.6486639878613617191558070018e+33', '-7.64866398786136171915580700179999e+33', 33, 2);
    T('1.9360273198736258723075e+104356', '-4.70316280735914960164009569103022e+450018305297508', '4.70316280735914960164009569103022e+450018305297508', 90, 3);
    T('-7.6323486091102117169661832163538038274338505077e-216', '2.53033176334954570528e-6481305021670', '-7.6323486091102117169661832163538038274338505077000000000000000000000000000000000000000001e-216', 89, 3);
    T('-2.2471152562467268928146363182e-985971100494077', '5.93178238037916713081620267718e-93642160312729', '-5.93178238037916713081620267718e-93642160312729', 74, 5);
    T('1.6199346239120564405747597090374769454301656069628e-545', '8.1408201478238562626093768885736e+49374193546', '-8.1408201478238562626093768885736e+49374193546', 78, 6);
    T('-7.77384039097767e-894145621839566', '1.0896009975424126835395415028524e+57329250065', '-1.089601e+57329250065', 8, 6);
    T('8.4154072465812928668349695623579668e+7496', '2.8435126621341749673012811981104206714e+60025338256305', '-2.9e+60025338256305', 2, 3);
    T('6e-6384230', '7.454498902658041923757122708526675728830548938570e+8612', '-7.454499e+8612', 7, 5);
    T('0.7', '-7.173466749314991565101244961185862909357855915980e-32284386985612', '7.00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e-1', 93, 2);
    T('5.32343009272596309905836283702583467856820203652056210e-473779225881', '-6.532203956021415465864736266235901607098688e-86579339', '6.532e-86579339', 4, 4);
    T('2.8983898603412050e-2294143934', '-3.8493864602295614399404587457138012416634034003007e-778240751', '3.84938646022956144e-778240751', 19, 6);
    T('7.2558474067782635e+31344436730047', '-3.343912579990369675458057423693653471e+315711288', '7.2558474067782635e+31344436730047', 83, 1);
    T('-7.1e+7689045021', '2.1457736919952415134998680602149712848625577958900572e+172053868765', '-2.1457736919952415135e+172053868765', 20, 5);
    T('8.735982537492159901291684193142535e-115114709103', '1.510141597443676411798338489792863e+34', '-1.5101415974436764118e+34', 21, 5);
    T('-3.43184e+1901', '-6.38675852268594601398079534871413129603641234e+79', '-3.431839999999999999999999999999999999999999999999999999999999999999999e+1901', 70, 1);
    T('7.014279623e+87032', '-1.185422332555403e+752797173527', '1.1854223325554030000000000000000000000000000000000000000000000000000000000000000000000001e+752797173527', 89, 0);
    T('-1.2937906992e+65626', '-6.8639151867637634237012337259638449597695766870379875860867e+188998896', '6.8639151867637634237012337259638449597695766870379875860867e+188998896', 93, 5);
    T('8.5656789396544415874726260057415278948816117639910434e+741', '-6.78711815078474524221456550931251554222e-32150322', '8.565678939654441587472626005741527894881611763991043400000000000000000000000000000000000001e+741', 91, 2);
    T('1.8950184760528088300956705512229334442903709888468462893676e+354587042', '7.64913983511513e+3916302672', '-7.65e+3916302672', 4, 0);
    T('-7.8983726470942209e+724504390592', '679740181437452679.739534741056507278951941', '-7.8983726470942209e+724504390592', 89, 4);
    T('-5.79176947172149961095773774440314277640414695e+264920870', '2.55429805960430424154823467344e-7002014', '-5.791769471721499611e+264920870', 20, 4);
    T('-4.45400324829346441381151739415086667370052853e-7722', '-2.963206178646811894754069217727471153236221e+8458636', '2.9632061786468118947540692177274711532362209999999999999999999999999999999999999999999999999999e+8458636', 95, 1);
    T('2.43590780882508110609744149128635004443673e+9040914939907', '-7.03334926584410767648628815650176838e-831436226004', '2.43590780882508110609744149128635004443673e+9040914939907', 59, 6);
    T('-8.35665480596906529e-99142939830', '4.5577564e-813976413077817', '-8.35665480596906529e-99142939830', 63, 4);
    T('1.858255926099704216303880722e-34224807026928', '2e+496304310484140', '-2e+496304310484140', 56, 3);
    T('3.9413634780106261670694164376786889181361621e-9396511392059', '8.220249510669553000720122409327834652749657858942079e-568349783749', '-8.221e-568349783749', 4, 0);
    T('-5.07254156426421412453245330510054497024039564308182e+75333399190807', '6.497142516710552475126074112329957855e+7168112740', '-5.07254156426421412453245330510054e+75333399190807', 33, 1);
    T('-3.38755739651e+18', '-2.07903244910856291743223622122485203873903568375367785e+111', '2.07903244910856291743223622122485203873903568375367785e+111', 68, 0);
    T('-1.55024339164074244407e+7389', '-8.767115268267896208542206e+12699414588', '8.767116e+12699414588', 7, 2);
    T('1.69114024088078792284372223381273981187102390085047623621632e-6231156218', '1.752912893e+82853517738075', '-1.752912893e+82853517738075', 12, 5);
    T('-7.54356503553379174683703564528241324387100e+1768', '1.1826423618497281223486368147641097697915014548e-7775563255', '-7.5435650355337917468370356452824e+1768', 32, 5);
    T('17', '9.4115e+1249', '-9.4115e+1249', 52, 5);
    T('4.4948588884727051288691926774110072042694e-450062392435879', '-7.4858921065053e-853423205', '7.5e-853423205', 2, 5);
    T('-3.806154705856846980411066239624389070637839815e+71', '2.52954421838563800737821546133044564417e+6705', '-2.5295442183856380073782154613304456441700000000000000000000000000000000000000000000000000000001e+6705', 95, 0);
    T('5.5799363975402825880485294841118635991507815e-92663127999', '-3.2e+675522526885829', '3.2e+675522526885829', 43, 1);
    T('3.0180586061198730817995103370784246944751811448e-868775665664', '5.14511768417162499783442060965193285637382700062541534148775e+402', '-5.14511768417e+402', 12, 6);
    T('1.888612911302557326597e-513944134954331', '7.293e+5950482290', '-7.293e+5950482290', 63, 6);
    T('1.515e+3955130', '4.0997649094735432137e+7962955668', '-4.0997649094735432137e+7962955668', 78, 6);
    T('5.722166421529449665742031339e-833489', '9.638468018044455641337087e+19441', '-9.63846801804445564133708699999999999999999999999999999999999999999999999999999999999999999999999999e+19441', 99, 2);
    T('-8.2355197951581067872533904800952490772916465632298e+5457726', '-5.3970716771688611782362918084669585471337298732753e-6046', '-8.235519795158106787253390480095e+5457726', 31, 4);
    T('1.2e+160361185944', '-6.65499457939558325486645e-8043', '1.200000000000000000000000000000000000000000000000000000000000001e+160361185944', 64, 2);
    T('8e-28', '2.91733279330505558350668056219532419237844979572e-96111588894390', '8e-28', 55, 0);
    T('6.6176e+44', '8.11613342493587446826e-62269702', '6.6176e+44', 92, 4);
    T('-7.33e+5305', '1.27339121654329120136366996307134294990387e+106', '-7.33e+5305', 46, 5);
    T('4.883220205277593178696133091e+507264732948856', '-1.2167946709661136692566631279071e-355072651', '4.883220205277593178696133091e+507264732948856', 61, 1);
    T('-6.4494814623826245122992133903281519562582476352e-4569978', '4.2749603121745915e+569709', '-4.2749603121745915000000000000000000000000000000000000000000001e+569709', 62, 3);
    T('-9.576064115565854647318550377398931671115244253656805823e-4113', '1.98567123463946e+96492418019', '-1.9856712346e+96492418019', 11, 5);
    T('7.779124596870444459358571436e-64685457', '5.1730926602135e+501770112', '-5.17e+501770112', 3, 2);
    T('-9.438538878997859392114676202909e+89747811953', '-6.5515150686345824330334022821951614e-475728588', '-9.438538878997859392114676202909e+89747811953', 43, 4);
    T('5.96303e+3311348356', '-8.1549561e-1810396634', '5.96303e+3311348356', 45, 5);
    T('-2.623380729026375201e+82861334508', '-8.7882025640498027e+3637', '-2.623380729026375201e+82861334508', 22, 0);
    T('8.1777824938340982848190599066073052599287334293630462254067e-836797771', '-5.50641175023474349e+188070103202772', '5.50641175023474349e+188070103202772', 23, 3);
    T('-9.26360465603704412300604919427805964653612441757846333e-109483', '-5.510687e-58151', '5.510687e-58151', 24, 6);
    T('7.6839863503678736756118169100357097364411e+2284864106', '7.358219367045696314245276721054995e+739843146438', '-7.358219367045696314245276721054995e+739843146438', 68, 5);
    T('8.481620520438362e+590', '6.9e-20', '8.481620520438362e+590', 80, 5);
    T('-2.478231115230817034317960858914132242866981767435598e-3944838324', '-5.760913243810962900244346316e+61101', '5.760913243810962900244346316e+61101', 67, 6);
    T('3.8192878894028732940977617430914578615e-71889982382', '6.011270504528e-73290767003', '3.819287889402873294097761743091457861499999999999999999e-71889982382', 55, 3);
    T('-8.48259398258389595525579884591346602497000709759433258366e-2535', '-3.705573386266177934e+5679768335462', '3.705573386266177934e+5679768335462', 90, 0);
    T('5.6220512275067261251570940983971510e+838723657121271', '-2.475818044959e+1855422', '5.622051227506726125157094098397152e+838723657121271', 34, 2);
    T('-9.7948623532158241251752271854625573822e-7243962', '5.633423913201027198024e-620171703151', '-9.7948623532158241251752271854e-7243962', 29, 2);
    T('1.235213806820842090943744928174e+56075', '-9.66567474124278241325284009831752420728899e+5870589184', '9.665675e+5870589184', 7, 0);
    T('21345803229085057196.1758575', '1.5610247800629934790095458818855055704892217669850423645e-750745370', '2.13458032290850571961758575e+19', 88, 4);
    T('-7.30773513753549020969042208805619089404448373926257e-7541416863150', '8.32427e-54708', '-8.32427e-54708', 65, 6);
    T('2e-799459299590286', '5.889778178794239632e-54355', '-5.8898e-54355', 5, 4);
    T('-9.3556377183327394396683789713e-1334168', '5.58396402329231483858e-7724290647464', '-9.3556377183327394396683789713e-1334168', 75, 2);
    T('-6.88040467087507e-3843619', '3.84573170161266434e+2676608', '-3.84573170161266434e+2676608', 32, 5);
    T('1.0785343520992779070873e+3979', '7.44186442481045073514472795083781168893e-46', '1.0785343520992779070873e+3979', 52, 6);
    T('1.8e-832981590', '-25100320742.7178', '2.510032074271780000000000000000000000000000000000000000000000000000001e+10', 70, 0);
    T('-2826435144396116190659065520082331345556033567.06', '-9.0748057695226490986492375982992274e+357845379970', '9.0748057695226490986492375982992274e+357845379970', 59, 2);
    T('-7.61328233271751766600739347682143211334997328040726235477e-33144', '-8.80240734112890877906248173884e-12125009666', '-7.6132823327175176660073934768214321133499732804072623547699999999999e-33144', 68, 1);
    T('-6.0587277537740085333231806617966323970443601e-5428591571', '5.6603431e+59397', '-5.6603431e+59397', 99, 4);
    T('8.08730575e-49', '-9.540349727367256661863351759e-292781934748', '8.09e-49', 3, 2);
    T('9.41848969863e-422444090681453', '9.304382202363843007867042e-65923', '-9.304382202363843e-65923', 17, 5);
    T('-7.394607446525877482929884150552236138314846576257942766445e-6332634', '-3.3365449542261843116750766632203761604766852504532316861e-227407331998813', '-7.395e-6332634', 4, 0);
    T('-9.25e+7140725499', '8.93024753781640153289655426349746822397023249170778486346019e+2059001', '-9.25000000000000000000000000000000000001e+7140725499', 39, 3);
    T('8.7072916332704607982909545e-20414408363', '-9.27063862316e-305328', '9.27063862316e-305328', 26, 6);
    T('-3.245483047125065656393814116958851843280985413e-40', '3.3671558156260651729543326362219e-253072237886', '-3.2454830471250656563938141169588518432809854e-40', 44, 6);
    T('-1.21032677127723124530616568787856029709830004183153e+911', '-3.21156480374158120806092866e+959900339', '3.21156480374158120806092866e+959900339', 91, 2);
    T('-4.66349072461744e+80165282', '6.0934026526630019535571475668430328e+5685776118', '-6.09340265267e+5685776118', 12, 0);
    T('2.9e+16765644', '55800446223181999237367.37', '2.899999999999999999999999999999999999999999999999999999999999999999999e+16765644', 70, 3);
    T('2550459696088739140414233627660954962202880209332482', '7.525662e-275125', '2.550459696088739140414233627660954962202880209332482e+51', 82, 0);
    T('7.7956556559544011008727119e+811496664195842', '3.58993459371424487e-70', '7.7956557e+811496664195842', 8, 6);
    T('3.09245885876046422970376537102573163563332825510412e+25758177640', '1.7329208709777144607335766304198651145433404733e+921509018368', '-1.7329208709777144607335766304198651145433404733e+921509018368', 70, 5);
    T('9.3864853995101134449426354766976989752411318955944160385552e-56931752764984', '-52056984443918827935923427024578823590098468842', '5.2056984443918827935923427024578823590098468842000000000000000000001e+46', 68, 2);
    T('-6.3509000808926348830351561e-273156', '7.617464265e+4102273969', '-7.617464265e+4102273969', 28, 4);
    T('-2.2636016619917943751319765790228552588802854070620012e-8043553878', '-1.16196342893285866014e-5986', '1.16196342893285866014e-5986', 23, 4);
    T('-5.24454e-820786', '-2.9855818651534623972665731375e-719', '2.9855818651534623972665731374999999999999999999999999999999999999999999999999999999999999e-719', 89, 1);
    T('3.50492143590392037287847996562446828639764894e+45894137514', '-1.7532067558180669640226783459002783988005974008207424e+5810685881', '3e+45894137514', 1, 3);
    T('2563309558124.5554466', '-5.846347920451e-28266698', '2.5633095581245554466e+12', 54, 1);
    T('-2.421762188559232136464406232480730e-59', '-1.823e+593927103316', '1.8229999999999999999999999999e+593927103316', 29, 3);
    T('4.6538481918062256407281394922768171168889557608791678e-59628547', '5.142861542329964034709036609003652753e+719655280271', '-5.142861542329964034709036609003652753e+719655280271', 66, 5);
    T('2.4163288821778418356548531e-11', '4.9722843812928018254866651078499308e+544327313923552', '-4.9722843812928018254866651078499308e+544327313923552', 60, 3);
    T('-4.55313540754732050267700347488202038789381800445867574346e+854294862380674', '3.394628703338633077471193562206510617e+50499916', '-4.55313540754732050267700347488202038789381800445867574346000000000000000001e+854294862380674', 75, 3);
    T('-2.3038231231162262139220675821952584413099439236665e-526', '-2.14667507811343829184401567659381294444e-1584330689', '-2.303823123116226213922e-526', 22, 6);
    T('-43184513461118598871405180281414891363959586928695.5617997', '6.7753359391198150734696292015050862e+855536758', '-6.7753359391198150734696292015050862e+855536758', 47, 5);
    T('9.54501002517807490865344212656248151594880187373316504517e-9213', '8.550460055849533398378561526647307570431e-121966670355245', '9.54501002517807491e-9213', 18, 4);
    T('-5.16871440809734274740831e+14860406', '-3.0722091112490272067815709996741890e+59738952663857', '3.07220911124902720678157099967418899999999e+59738952663857', 42, 3);
    T('-9.46821828304623574230045784705809167480048529694102451643013e+476764908229', '-4.71445805848178212e-72161150291950', '-9.468218284e+476764908229', 10, 0);
    T('7.3641392955502224099765435605809048618995311387672950e+754261860047', '-9.5964e+83286473556933', '9.5964e+83286473556933', 29, 4);
    T('-6.6624e-4437097', '-8.07708e-2988', '8.07708e-2988', 84, 5);
    T('2.9967269867181743449214200039181257823224e-24974975', '5.4588022953779963499159599133126968706153e+88893512', '-5.4588022953779963499159599133126968706153e+88893512', 47, 0);
    T('9.7e+41670412266753', '6.80751e+9360616', '9.699999999999999999999999e+41670412266753', 25, 1);
    T('-7.6444167462973460611232932723105761310765159539424448904e+737', '7.8504329064339421419076367579191928309300582364e-1629307431', '-7.6444167462973460611232932723105761310765159539424448904e+737', 73, 5);
    T('-8.888417724314760583539397921565969459003382887197441071e-193307719396310', '-9.877393433405e-763520094326589', '-8.888417724314760583539397921565969459003382887197441071e-193307719396310', 79, 0);
    T('-8.626364763774006e-4048', '1.7427e+308969', '-1.7427e+308969', 5, 6);
    T('-3.108919809460e+7946700547', '8.73188e-875604806', '-3.10891980946e+7946700547', 83, 4);
    T('-4.73146132541007094694306145101686893495475053014e-6300', '5.02464499532619e-8217155', '-4.73146132541007094694306145101686893495475053014e-6300', 79, 1);
    T('5.53425539146287742e-73', '9.4933386655653252426396759901534203782413835583048233608484e-71357230708', '5.53425539146287742e-73', 71, 5);
    T('-5.6e-2330862', '-3.57014858082924937445e+29729', '3.5701485808292493744499999999e+29729', 29, 1);
    T('9.5864668914963513215e-5000095111', '8.993165278e+64', '-8.9931653e+64', 8, 5);
    T('-5.1435801508514289329134440130754714684210484896211371740e-4839', '-3.95293322976198479185736909337118391905631298107718296157874e+8942315999215', '3.9529332297619847918573690933711839190563129810771829616e+8942315999215', 56, 5);
    T('-9.34359893806794184004596057181461102545534426580648163e+58784660', '3.233625676145687940954299813602858809672e-425745', '-9.3435989380679418400459605718146110254553e+58784660', 41, 2);
    T('1.8749302011e-110550272', '-8.943812639874856577849628477098460532611189550776e+318', '8.9438126398749e+318', 14, 2);
    T('-9.69412869e-774767', '2.1e-71', '-2.1e-71', 14, 6);
    T('4.2038e+292515528763', '-6.899612772469722183628585603350495011e-785117015195964', '4.203800000000000000000000000000000000000000000000000000000000000000000000000001e+292515528763', 79, 2);
    T('1.285894827142626712190440890168602207706e+7880735', '3.681671599259792931867210146120e-6333251', '1.29e+7880735', 3, 5);
    T('5.600747093242521530953689600461398808485777684453488341686e+25929169396149', '3.80127189686382582995e-89481175029', '5.600747093242521530953689600461398808485777684453488341686e+25929169396149', 64, 6);
    T('6.28851552603059911851794581614081686837784743437662e+123', '-4.7619145996100080408555515396937e+40', '6.28851552603059911851794581614081686837784743437662e+123', 56, 1);
    T('-4.16599862649293335e-32112528', '5.46225838779916674859809870195130218e+40923', '-5.462258388e+40923', 10, 0);
    T('-1.2398732e-894003563', '-8.231889676104496662e+24', '8.2e+24', 2, 4);
    T('-6.899281856221198662256683294555510298770252654e+55062236', '-8.97774658729049568166576881050768393579174555600849524039e+11613871718', '8.97774658729049568166576881050768393579174555600849524039e+11613871718', 81, 4);
    T('9.181e+820275833141095', '2.9772090148161159455680896e+3763025', '9.1809999999999999999999e+820275833141095', 23, 1);
    T('9.934587062886203879439863009297467704e+384', '8.104e-3724407804717', '9.934587062886203879439863009297467704e+384', 57, 5);
    T('-9.23223063874034310e+5594438', '6.131e+921504822', '-6.13100000000000000000000000000000000000000000000000000001e+921504822', 57, 3);
    T('2.327970e+3523', '-7.0e-8425', '2.32797e+3523', 75, 3);
    T('-69470696085534334833604791145883631027080.06786470', '-5.082831221595e+538', '5.082831221595e+538', 56, 6);
    T('-9.16e+858434842464', '5.21409165162631108404314924849133263e-9442901768', '-9.16e+858434842464', 98, 4);
    T('3.2717654260675052183988099775856208e-70331', '-3.9999113794076801375954891596251608552975750316825e+136', '3.9999113794076801375954891596251608552975750316825e+136', 61, 6);
    T('7.74388436e+90289657', '6.7262664843256930945916274749115717924937176107608030671e-440', '7.74388436e+90289657', 28, 4);
    T('1.1654384235353059e-125119419518047', '-7.28893517798470167486717698104358162815e-581381352', '7.288935177984701674867176981043581628150000001e-581381352', 46, 2);
    T('-5.096005693195796872597452676991154386338730278042e+257943001', '5.285025358686318789477449695697168822205277754836877e+336523221970463', '-5.285025358686318789477449695697168822205277754836877e+336523221970463', 69, 5);
    T('-2.98e+19', '7.37744089145376792929000879154911585353367456939398376426883e+63', '-7.38e+63', 3, 3);
    T('8.236853927348747e-120257800901', '-5.70186004221825088171294966954354321976e-37', '5.7018600422182508817129496695e-37', 29, 5);
    T('-5.6891148579768601592861631775113091411941964303435693439497e-43203046443', '8.57734915156473425063e+2429000890918', '-8.57734915156473425063e+2429000890918', 25, 4);
    T('6.8703399431630e-523698825657', '-1.9766625272163193087734147366605018e+97352182', '1.9766625272163193087734147366605018000000000000000000000000000000000000000000000000000001e+97352182', 89, 0);
    T('-9.6327611077188515041881279029e+65590', '-4.61802416166938583027387631083867e+30956085211358', '4.61802416166938583027387631083867e+30956085211358', 46, 6);
    T('8.3470789021226286560926331466287178629e+627966', '5.8806116e+398921413763807', '-5.881e+398921413763807', 4, 4);
    T('-5.1527467137722696265e-41', '6.7e+4438558357', '-6.7e+4438558357', 35, 6);
    T('-8.34497034851016e-51690883174639', '9.17623e-48784242', '-9.176230000000000000000001e-48784242', 25, 3);
    T('5e-91915', '5.4089129382112698662952489333256826e+406978972028293', '-5.40891293821126986629524893332568259999999999999999999999999999e+406978972028293', 63, 2);
    T('9.20234617584044351979652628540498432387988628581476973e-19', '1.61871037236749940205009146416261863580e-53823135070', '9.2023461758404435197965263e-19', 26, 5);
    T('2.6148323023e+4228560832234', '6.82660158076517937471652068e+735850560930575', '-6.82660158076517937471652068e+735850560930575', 77, 4);
    T('-3.0349158869406842602128237728436582757029891647049535335539e+8003334523442', '-3.72566798881150337917e-420403261', '-3.034916e+8003334523442', 7, 3);
    T('6.111407e+8503', '-3.572220084344621658655e+321184', '3.572220084344621658655e+321184', 45, 1);
    T('9.8310109399469e-6732460552249', '4.3395089326307006629118201477315302390129e+3175130466982', '-4.3395089326307006629118201477315302390129e+3175130466982', 78, 0);
    T('9.9848490160898840607520829e-311753', '4.214348110899461471577712e+205484730', '-4.214348110899461471577712e+205484730', 86, 0);
    T('7.39168500894446064061543e+7705007544591', '-5.009064861918282877216424e+735708834', '7.39168500894446064061543e+7705007544591', 41, 3);
    T('7.892900138070537820757244076e+37686791403473', '-3.781447558119058565388517604003603701e+39', '7.892900138070537820757244076e+37686791403473', 78, 6);
    T('-9.259416976464079272818314e+107', '19.627992792', '-9.259416976464079272818314e+107', 30, 4);
    T('-7.412425009096217537052775205240074508786e-19232783', '-2.9238031792010933170235761404e+1569747587', '2.9238031792010933170235761403999999999999999999999999999999999999999999999e+1569747587', 74, 3);
    T('5.9795528821725e-32', '2.533913940166677602116476533969378804430504143522920e-4413495969022', '5.9795528821725e-32', 65, 2);
    T('6.8172795160641909186486136441979061e-4545059913042', '2.10756299189367199582328807096125498797597354252104e+24726', '-2.10756299189367199582328807096125498e+24726', 36, 2);
    T('1.547749563786438188567065842412102938754638495127e+9696283846', '3.22996787943849983445748312778983430154127442265043435251697e-867069', '1.547749563786438188567065842412103e+9696283846', 35, 2);
    T('5.9579762560395326241309662465545991081908543757395430732e-98351', '-3.810543463466858834912105767319363915682353e+85539', '3.810543463467e+85539', 13, 5);
    T('-2.5095670808817736126858642783033911156e-615138', '3.4813870418457808904203714299226057169581758477527352760e+894', '-3.481387041845780890420371429922605716958175847752735276e+894', 93, 5);
    T('-8.20231705939681288792633079302353456e-366069', '-9.80765905e-4935881', '-8.202317e-366069', 7, 2);
    T('-7.862955205334148372432955225619467355911662e+78798488450', '-5.527369933611650e-430619338', '-7.8629552053341483724329552256194673559116619999999999999999999e+78798488450', 62, 2);
    T('-9.699154604113520596598562530516e+544810410', '-3.2178041752968668147779872511e+12120604670', '3.2178041752968668147779872511e+12120604670', 84, 2);
    T('-4.35800272789338488553464805622981e+39', '2.115152132972358623672452648260509248020e-7631797', '-4.358002727893384885534648057e+39', 28, 3);
    T('8.060570830119984073736e+761113769107', '2.5644e+815141989015', '-2.5644e+815141989015', 67, 0);
    T('8.078290620008e+15', '9.507820029725063977910623287196e+141890252924656', '-9.50782002972506397791063e+141890252924656', 24, 0);
    T('-1.146363983668206e+4756', '-2.304481273589521508438016939641420270592566560e-30565070160444', '-1.146363983668206e+4756', 26, 3);
    T('9.18147691145582550368401605387812288927803825332e+284306406556', '-9.355368555315906257861189734107483047e-75427', '9.18147691145582550368401605387812288927803825332e+284306406556', 80, 1);
    T('9.548264837239140290814368557328140798919e+69', '3.952694035539589982727511833926627595e+132', '-3.95269403553958998272751183392662759e+132', 36, 5);
    T('-1.1993453162228e+7055392947', '-6.4565276575865518194174905900e+7711550210586', '6.4565276575865518194174905899999999999999999999999999999999e+7711550210586', 59, 3);
    T('-9.1967225166238735741573202331723690e+504', '8.020055397031371440526756140124831250530441364235438821e+752592984', '-8.02005539703137144052675614e+752592984', 28, 5);
    T('-5.5236002640568734326966284e+2360052', '-4.0727858877e-83336725', '-5.5236002e+2360052', 8, 1);
    T('-905045794597311894095263765.46', '2.8741884151136637651140973589797721136468896441924467583572e+320041', '-2.87418841511366376511409735897977211364688964419244675835720000000000000000000000001e+320041', 84, 0);
    T('4.7925107610936691579411139323078861e-21445215648', '5.870761017590856498945803316098609630440805493064e-903069806556', '4.7925107610936691579411139323078861e-21445215648', 54, 4);
    T('6.9938211301e+78539661746', '6268186277168261070', '6.9938211301e+78539661746', 27, 6);
    T('-1.7e+59645785925416', '6.516736203790037717888897308284552888848896057e+256310504873211', '-6.5167362037900377178888973082845528888488960570000000000000000000000000000001e+256310504873211', 77, 3);
    T('-4.1060721552134255198426e-832517005815187', '8.9e-130', '-8.9e-130', 52, 2);
    T('5.8506516565023885e+701', '4e+19979292', '-4e+19979292', 93, 3);
    T('9.7598440601051809e-494411151491549', '4.11e-192302784', '-4.11e-192302784', 36, 0);
    T('7.0639098764999009e-76936', '-9.574677889419206555806e-694164524304', '7.0639098764999009e-76936', 48, 5);
    T('-70967727.1889079395896957683252', '-9.990178977731023803488725605313604189644085924e-295', '-7.09677271889079395896957683252e+7', 41, 6);
    T('5.5490998355429437272390175898940399853036846731295e+965150225902', '-7.6282108870729593863318588883741850675849972e-27872052', '5.5491e+965150225902', 7, 4);
    T('8.51336444830070458901063917974e-3115', '-8.252746572374293565413622652188787421590664732113382e+115', '8.252746572374293566e+115', 19, 0);
    T('6.187139572321490021172186512095e-5006312313409', '8.6119350313019e+3201950', '-8.6119350313019e+3201950', 35, 6);
    T('18243765159323674723227658549.8768281', '4.11681731632701221e+666902353717840', '-4.11681731632701221e+666902353717840', 88, 6);
    T('6.1168429e+12', '-9.551341866e+6493', '9.551341866e+6493', 100, 3);
    T('6.6571633627964672372419854645211539104211850337475174107e+3054494', '-7.1349686893432296066446663250687e-390639382051', '6.657163362796468e+3054494', 16, 2);
    T('6.129984599148238556248560299642596719167525672920125467e+13159', '4.29471670783165323e+1709', '6.1299845991482385562485602996425967191675256729201254669999999999999999e+13159', 71, 1);
    T('4.798812601527254992127200557319548477e+538479', '3.0681906551429796117458e-15108550', '4.798812601527254992127200557319548477e+538479', 41, 4);
    T('-3.9102148687855852457588924487e-233126534', '73.98943345036071514639845582390122308555238057110', '-7.3989433450360715146398456e+1', 26, 0);
    T('-5.328670903549321977862634421739971e+92803160569399', '9.930938606e+151810025199', '-5.328670903549321977862634421739971000000000000000000000000000000000000001e+92803160569399', 73, 3);
    T('3.84907198244410944754912732655305837366493160454591e-6996742620569', '4.0313280593668980e+114454117', '-4.031328059366898e+114454117', 98, 0);
    T('-1.191309704647196083216e+270', '8.60518950805573072437e-548892', '-1.19130970464719608e+270', 18, 1);
    T('-9.92150926129651844155e-189365', '8.6298043928193099460831610646456e-2789177773', '-9.9215092612e-189365', 11, 2);
    T('-6.5938407280620139804469277746402284093937822e+4856', '-2.40642291922993607351515e-73', '-6.59384072806201398e+4856', 18, 5);
    T('6.719259425612171096169867551570887414270893e+3831', '8.1790301055101702486021303459125777522661126325e+77927953907700', '-8.17903010551017024860213034591257775226611263249999999999999999999999999999999999999999e+77927953907700', 87, 2);
    T('-4.664049017539417445855102058816e+331538884', '1.679947712533859e-76293189', '-4.664049017539417445855102058816e+331538884', 32, 1);
    T('4.012972007879e+18892748625', '8.2160897934686769491590549771446393727321762402120768e-225980608657263', '4.0129721e+18892748625', 8, 2);
    T('2.50113084280917e+19', '8.15891465313491e-250985', '2.50113084280917e+19', 96, 0);
    T('-9.2257238498943652891403028061104084262243522718538032e-3083', '1.83331144560027282065656217929025596588973e-183207337346760', '-9.2257238e-3083', 8, 5);
    T('-3.912170674381241370e-129', '7.416069839417106144310466233744104559658046251151e+96996270496446', '-7.4160698394171061443104662337441045596580462511510000001e+96996270496446', 56, 3);
    T('-4.505043676661012064598051136888756754261984512e-156160', '81811661939990.6239613', '-8.181166193999062397e+13', 19, 3);
    T('-9.060099223900357125e+159', '-8.62802482035034711807960568060760738327792555e-925212720', '-9.060099223900357125e+159', 76, 6);
    T('-9.513437005100304e-8525321', '-9.9434468624301635194404427110536214637327077e+80574', '9.9434468624301635194404427110536214637327077e+80574', 55, 0);
    T('7.6524107246689052e-42964244', '-7.3618343721878581e-13', '7.3618343721878581e-13', 57, 1);
    T('-8.3777923867482325359259815162e+9584', '-1.11524958497242569723481272101668706566552096844467812713471e+98', '-8.3777923867482325e+9584', 17, 1);
    T('-2.002457152322708498229440089679238661616189e+69822965230493', '-2.4836505942613084916122525e-295111', '-2.002457152322708498229440089679238661616189e+69822965230493', 47, 5);
    T('2.5389212222320819141247229229e+69745409864', '-5.01833799262021629099152391389080537e-26485815819', '2.5389212222321e+69745409864', 14, 6);
    T('7.05394938354464700802192e-281337242522513', '3.169374178265546621481559531463921970e-3308866457', '-3.16937417826554662148155953146392197e-3308866457', 81, 0);
    T('8.602489082584254133573608187982786337387052e+47', '-9.3780377028310774743465326230008130387701648158e-555432177471045', '8.602489e+47', 8, 1);
    T('-1.28743466509911e-438906705', '2.9232454028112394e+55', '-2.9232454028112394e+55', 31, 1);
    T('3.1951077933642788624953687974390926416477136642441905010391e-2961799', '-8.35912424218266138995189268763667403484279790987973206766e+465603127445', '8.35912424218266138995189268763667403484279790987973206766000000000000000000000000000000000001e+465603127445', 93, 2);
    T('7.043698822150986750498807034146560791420757297751942069e-83768412623', '6.769383531234401214944266907461430e+7058353436', '-6.76938353123440121494426690746143e+7058353436', 60, 0);
    T('-5.841112741902923359354424e+39914657329', '-3.7854536791941005931605426803536157076893927337939976411898e-237254367320', '-5.841112741903e+39914657329', 13, 4);
    T('-8.0273853927863384108711342e-99488', '1.905861871108863536488515156477813583359731708204669676609e-9183', '-1.9058618711088635364885151564778135833597317082046696766090000000000000000000000000000000001e-9183', 92, 0);
    T('-1.3349125199943422153390279633754696368203170073582473e-723', '-6.2565840294104889855620219817641072831822e+974288795248202', '6.256584029410488985562021981764107283182199999999999e+974288795248202', 52, 3);
    T('2.3095825625092325925219393092888564255609798e+325', '2.96515184952377606525754245609657536e-773939356300790', '2.30958256250923259252193930928885642556098e+325', 43, 5);
    T('-5.565717625622197296758156772896885893680504170795170737e-80905339900', '9.06012607905250589694212096271591127366248630502708e-330495877664', '-5.565717625622197296758156772896885893680504170795170737e-80905339900', 74, 1);
    T('-7.242883349661916994497014601200613232005402115069722406475e+17590707465084', '4.7134859e+6642', '-7.242883349661916994497015e+17590707465084', 25, 5);
    T('1.6238003747768908518389389640572487159969193678e+933', '2.4135341600845045559776159675506392869992225042e+6893905940198', '-2.41353416008450455597761596755063928699922250419999999999999999999999999999999999999999999999e+6893905940198', 93, 2);
    T('-6.7739753601829814433207e-58352058047509', '-1.4039895611e-164194152756', '1.4039895611e-164194152756', 63, 0);
    T('6.3656606710570006308487325825293909929430033688991e+3353543870', '7.24868322506419306506745932522474e+127834557074317', '-7.24868322506419306506746e+127834557074317', 25, 0);
    T('-17499640693508117992431869490', '7.8732151106544399450696594913834648563040156998523e-1538', '-1.74997e+28', 6, 3);
    T('2.37780965455025695388012343812048585348276e-11224044678', '8.34035353992114503626e-77286144001', '2.37780965455025695388012343812048585348276e-11224044678', 59, 6);
    T('9.0909577411511448109341182185848912290778322355826016769648e+904852', '-6.817931150349322940011446319240251311750949652790e-844511', '9.09095774115114481093411821858489123e+904852', 36, 6);
    T('-4.5654855098410514e+36124', '2.81396198224674505438443662064342247303761677335667062067330e-64948', '-4.5654855098410514e+36124', 86, 5);
    T('-6.9579764674634e-91931', '4.248951583090594159100747882130617e-3738', '-4.249e-3738', 5, 4);
    T('-7.69771027446787868979830530725995508488550188e-46647713687372', '5.7711048886518040e+25', '-5.771104888651804e+25', 27, 2);
    T('8.758724677647342044030887642043925297551211996542352e+493946270380923', '-3.914307660132861293482798159e-27431798109154', '8.758724677647342044030887642043925297551212e+493946270380923', 44, 5);
    T('3.97664816753505544633402828660781752288683185100941888129e-576444929', '2.84026058101461892781675650179815264945134255640073047994e+42189', '-2.84026058101461892781675650179815264945134255640073047994e+42189', 72, 6);
    T('-6.055813212128e-2796234', '8.17324269196384315422246631050097326029024134716712851453e-846907945662', '-6.055813212128e-2796234', 64, 2);
    T('-2.93930353e+93', '-6.496141878471784160455041823762910948608e+24326012611', '6.496141878471784160455041823762910948608e+24326012611', 69, 5);
    T('-7.04e-45063733', '-5.862002958094260977544480619161e-15', '5.86200295809426097754448061916099999999999999999999999999999999999999999999999999999e-15', 84, 3);
    T('4.6115859e+114674153252623', '9.25144259e-5363863', '4.6115859e+114674153252623', 22, 5);
    T('-5.597860214717009642453670499972236e+7482031084', '-732719.79', '-5.597860214717009642453670499972235999999999999999999999999999e+7482031084', 61, 1);
    T('-7.2244451515155875283619080529482655776e-687694', '1.2e-2039', '-1.2e-2039', 30, 5);
    T('-5.9114433345380924747444525785460953213190e-799998251139828', '-3.60414240526171015894337359512097e-658205', '3.60414240526171015894337359512097e-658205', 63, 5);
    T('5.1564837922839126157417040122976497216765084e-8482783928255', '-5.2656258782408490085391e-150', '5.2656258782408490085391000000000000000001e-150', 41, 0);
    T('-7.40296835300329241095042112059477310739040215420982531e+1788393532098', '-5.080784218792475370539e-6917', '-7.402968353003292410950421120594773107390402154209825309e+1788393532098', 55, 1);
    T('-8.217106312506457369600839133116763e+7224566794836', '-4.95791979254888193413681226384887263305e-476349', '-8.21710631250645736960083913311676e+7224566794836', 33, 4);
    T('-8.064840884660597272607920e-645555390325478', '6.88096229679610979e+35355245609', '-6.88096229679610979e+35355245609', 98, 4);
    T('-4.896594699572430886455832216535122235596770199e-806218721819', '-2.812131759251916e-68771', '2.812131759251916e-68771', 62, 5);
    T('6.31329537238509109213e+618970759311387', '-5.180913308888616126451519424899926168586381718182744255463e+4594538117', '6.313295372385091092130000000000000000000000000000000000000000000000000000000000001e+618970759311387', 82, 0);
    T('-9.4583012729179455700e-21937687781', '-43729669325667191.08', '4.372966932566719108e+16', 25, 5);
    T('-5.25117177963357e-5900755689', '-2.9989882198135546775974850187520654794911805265e+8141015970311', '2.998988219813554677597485018752e+8141015970311', 31, 1);
    T('9.18042392e-4454351005', '-3.76538893e-23170191', '3.76538893e-23170191', 30, 5);
    T('4.184120441770296357e-45062', '-1.562162490532979701023549468550498705e+32967', '1.5621624905329797010235494685504987050000000000000000000000000000000000000001e+32967', 77, 0);
    T('-9.2e+8232523353', '-4.8540699781711337003e+979124', '-9.19999999999999999999999999999999e+8232523353', 33, 1);
    T('7.3102579018601072618523143e+32', '-7.24646779064972402245891429011774136479836e+496203', '7.24646779064972402245891e+496203', 24, 3);
    T('2.196502902167392171416897042057023955361549864520e-33113', '6.46541416273904407948105736174366643933280e+7961623043134', '-6.4654141627390440794810573617436664393328e+7961623043134', 88, 3);
    T('-9.45142883165051732401308373534678920378094212854301e-88897938619738', '4.49892166960957577878835193091e+721706', '-4.49892166960957577878835193091e+721706', 55, 2);
    T('9.41345542150638692e+64', '-7.750221044951664121374622889563704912674819884844e+64580247', '7.750221044951664121374622889563704912674819884844e+64580247', 95, 4);
    T('-7.5726331631876e-2650958297', '-2.4356e-5798', '2.4356e-5798', 71, 0);
    T('-4.10e+98025976000', '1.8044196519290463270774460731628004006489467e+881128', '-4.1e+98025976000', 20, 6);
    T('4.5975914332488630220645575614572773207234659771573623727e+712008866', '-2.7884478202506105469304638543428382897786423e+1616', '4.5975914332488630220645575614572773207234659771573623727e+712008866', 99, 3);
    T('-1.674918930393319327377572002339467232e+530033736', '5.535874839220679614743801006563101243654994386507e-37143315800803', '-1.674918930393319327377572002339467232e+530033736', 70, 2);
    T('-9.69983600054387538625690358137716018086086281269396202e+270', '-3.7860377266749533148597957820697551785159223e+85710348201', '3.78603772667495331485979578206975517851592229999e+85710348201', 48, 3);
    T('-9.575325302947852998971829130694291027698555895153768285800e-7220', '-7.01450409694995073856955303770830980209767219413377e-3268261251', '-9.575325302947852998971829130694291028e-7220', 37, 5);
    T('-6.225085775907479685005e-88334652', '2.11707301961787290195261466187491e+27681', '-2.11707301961787290195261466187491e+27681', 38, 6);
    T('-9.4195913835256093877264471903594594496188093678398155e+1574524', '9.88539769461282535645e-5617', '-9.4195913835256093877264471903594594496188093678398155e+1574524', 59, 2);
    T('2.3394425732277561e+23808265414', '4.59243482636290040377886503927028727246e+21154843709325', '-4.592434826363e+21154843709325', 14, 3);
    T('8.70224e-8868779', '-1.817629001490e+8385695', '1.81762900149000000000000000000000000000000000001e+8385695', 48, 2);
    T('-6.966270670810549899294922284114861849e+73265109288', '223311451222283720839021326462054783455.342', '-6.966270670810549899294922284114861849000000000000000000000000000000000000000000000000001e+73265109288', 88, 0);
    T('-4.141121e+10687422', '1.2275074806542257135449265336899896878052952280e+438938040006412', '-1.227507480654225713544926533689989687805295228e+438938040006412', 56, 4);
    T('7.630807892738951979609499228913171866342266398886432e-730760161', '1.4404838579938e+15', '-1.4404838579938e+15', 67, 0);
    T('6.921685289351881047189150e+64259427', '5.299679915414756945682464718709014044698283143108631e-946345997929', '6.92168528935188104718915e+64259427', 74, 0);
    T('-5.32284023352055365509895e-67', '4.38869677078049549884591038511107750339e-81', '-5.3228402335205975420666578049549884591038511107750339e-67', 100, 4);
    T('1.0446961237e-5965476660324', '-9.324349824190761058679061217e-8396', '9.324349824190761058679061217e-8396', 54, 5);
    T('5.6663649e+6180785741531', '3.20407037942318277401991928859395e-7100', '5.6663649e+6180785741531', 22, 0);
    T('-1e-6058463080', '4.4694642213240666297803823827124985458e-9825', '-4.4694642213240666297803824e-9825', 26, 0);
    T('7.932253283484e-5709259', '9.75882354489387991400604440137407608778210839278667277e+754017', '-9.75882354489387991400604440137407608778210839278667277e+754017', 69, 4);
    T('-2.482213613174369021443121640552344939339572379732099321929e-595142645', '-5.7984902e-655903373', '-2.4822136131743690215e-595142645', 20, 3);
    T('-33563543178124100438167536602339057998374090387784249.179', '-1.196485e-9154737', '-3.3563543178124100438167536602339058e+52', 35, 5);
    T('-1.4117504300065171878095851885507965236973731792041e+877315259896', '-6.5965287540091070628705063791132390176667538289e+241005833036856', '6.5965287540091070628705063791132390176667538289e+241005833036856', 94, 6);
    T('4.202836245333520118787606057370e-195', '-6.6350941779309323988961310778978336997444619170704817e-53474379719698', '4.20283624533352011878760605737e-195', 37, 4);
    T('7.450e-97622212', '-8.84362008530173167249724822026964132358842290599600412e-25', '8.843620085301731672497e-25', 22, 3);
    T('-4.86878396841247174131729308921988977001258878767e-94470075672887', '-6.2243007104368233781929357992318051821043822403255457e+11693303', '6.22430071043682e+11693303', 15, 4);
    T('4.35e-94779169736180', '-9.5881174862936263762389730023448816867902813911e-2798763288', '9.58811748629362637623897300234488168679028139110000000000000000000000001e-2798763288', 72, 0);
    T('2.776950977524188363904374809155607830640838018620260e-547699', '7.895091e+6100195', '-7.895091e+6100195', 68, 3);
    T('-3.784055373495e+9518934', '-9.3255e-71391420', '-3.784055373494999999999999e+9518934', 25, 2);
    T('968099569188491711744.272137630265319620296858', '-9.922409976350833398713e-8', '9.6809956918849171174427213772948941938380519198713e+20', 85, 2);
    T('-9.572591747337403168727843813444686562e-1683491111', '-3.5814400965385197e+178043912382', '3.5814400965385197e+178043912382', 25, 6);
    T('-1.5288870474033089940049048310129440e+46889', '7.16535e-444821', '-1.528887047403308994004904831e+46889', 29, 6);
    T('8067113641252097787277242434721842134929683554570', '-7.570905707746085146607486402949712991956934445675473556e+864', '7.57090570774608514660748640294971299195693444567547e+864', 51, 3);
    T('-4.22812850005899657980462463055255011351751236134e+5973', '-9.4184718384377794873e+730660879', '9.41847183843777948729999999999999999999999999999e+730660879', 48, 1);
    T('6.587062679658123303059265288325099832370363038315497819e+837', '5.89e+97108209155353', '-5.89e+97108209155353', 43, 4);
    T('-8.931891108e-61035644617', '-6.617655445e+490486430523512', '6.617655445e+490486430523512', 62, 5);
    T('7.90840943174e-5182', '4.4054237360075882058931944667318156188446356784e-16175723325779', '7.90840943174e-5182', 60, 4);
    T('6.44350274163275377720807e-39317', '-3.2719058255288786e+33051511', '3.2719058255288786000000000000000000000000000000000000000000000000000000000001e+33051511', 77, 2);
    T('5.2970854132680315834313858e+552424', '8.853222354094265424432101234722370796305016941253e+486746933358', '-8.853222354094265424432101234722370796305016941253e+486746933358', 77, 5);
    T('2.57483078399303388209735e+67660814', '3.85884206626533163970e+43', '2.5749e+67660814', 5, 2);
    T('8.784433854317997326451876312454463419786e+79870692', '-6.054609e+64717327092', '6.05e+64717327092', 3, 6);
    T('2.317111491346e+4143017', '-1.5155570172153474152201192181552202472814636e-7784233521273', '2.317111491346e+4143017', 86, 5);
    T('-6.05022698538454876686451e-32862386111780', '-5.6586e-788952595249126', '-6.05022698538454876686451e-32862386111780', 32, 0);
    T('-1.100140965620804991205213e-8010', '9.36002105729192039261763157e+707421525', '-9.36002105729192039261763157e+707421525', 81, 4);
    T('-9.0510944522219476087605211134e-384019920852', '-2.5497857570584893089e-1138946086', '2.5497857570584893089e-1138946086', 66, 5);
    T('7.686434483762638138561935065914e-191350555', '-1.1127505177270002990141909873343306e+94777163', '1.1127505177270002990141909873343306e+94777163', 81, 3);
    T('-6.3352e-58856031131117', '-5.6682574870861299705666748677126198957862805e+240297932700434', '5.668257487086e+240297932700434', 13, 5);
    T('1.975889896201642765096154390e+1901', '-6.0005810607132172098e+12076893756', '6.00058106071321e+12076893756', 15, 1);
    T('-9.610000329624248013003132396011e+722036', '-8.786810110005726202370623e+309171651153', '8.786810110005726202370623e+309171651153', 53, 6);
    T('0.000091457353786881728706629925237263545076902104650850246413', '-8.9914226169043487e+30', '8.991422616904348700000000000000000091457353786881729e+30', 52, 6);
    T('7.98239145191941120735912e-969770384996', '-73536355892897529166541393783.085', '7.3536355893e+28', 11, 6);
    T('-6.3737959e+533579666947396', '7.8206989036386642236904796423103665829172688877685013e+189605', '-6.3737959e+533579666947396', 74, 2);
    T('-4.16355025584660921657483449370697001753139954663e-276699', '-6.7061820757456758222167883215037125859008226702767627967470e-70137499', '-4.16355025584660921657483449370697001753139954663e-276699', 97, 4);
    T('-3.5936757786162571752605376212400274393917473006574705909e-23698371', '2.20517479291007554220994762032744543e-756995161427832', '-3.59367e-23698371', 6, 2);
    T('8.232795742286483815377284896347098923743571345288770923042e-7823263371', '-5.8e+258253887226490', '5.80000000000000000000000000001e+258253887226490', 30, 0);
    T('9.4966645520710454322036958977e-5964346020489', '-7.2099877501229143261913364839552e+3005', '7.2099877501229143261913364839552e+3005', 38, 1);
    T('-1.565263650814583296869987559610e-68', '-2.104225274213911692903443908709522644686476564e-61639421892', '-1.56526365081458329686998755961e-68', 73, 3);
    T('8.99451503710653923253e+52509', '-357105588583144906310219375111269631694202.0202', '8.99451503710653923253e+52509', 77, 1);
    T('7.81454e+3845927038', '8.3616147907491934368751889e+218', '7.814539999999999999999999999999999999e+3845927038', 37, 1);
    T('-5.5048551549478687809094780425601793309720853704475340981309e-67478', '6.177281332078827521464389008652612604322656052229e+17787578', '-6.1772813320788275214644e+17787578', 23, 5);
    T('5.412369631701149140965043796360456769e-6784706590794', '3.1892376814909603964280299897329952715981364581137e+273', '-3.18923768149096e+273', 16, 2);
    T('-1.9596865272107046464891529279422056e+58395202364934', '4.8095088656442804215057348083340176856074006501804030312906e+71806134', '-1.9596865272107046464891529279e+58395202364934', 29, 4);
    T('4.276795880141e-83563', '6.41293764705926504920521436426867e+852034070', '-6.412937647059265049e+852034070', 19, 2);
    T('-5.7e+32329181352', '1.28572362566e+3267', '-5.7000000000000000000000000000000000000000000000000000000000000000000000000000000001e+32329181352', 83, 3);
    T('-7.44507e-6257', '-1.4910297824543372875214229200117907690530222161463438518981e+52702904751', '1.4910297824543372875214229200117907690530222161463438e+52702904751', 53, 1);
    T('-5.6755001011363605418161627560514603668192215344812823927694e-570079', '8.866420334334813889215795746752829628012153406e-1796', '-8.8664e-1796', 5, 5);
    T('-8.266297343102966610080615171776017154905627599155227942e-441983658751760', '-7.7514665e-86081080038', '7.7514665e-86081080038', 46, 0);
    T('5.4393417565444e+55854', '-4.735095149648033e-406', '5.4393417565444e+55854', 88, 1);
    T('-3.1167427545849e-667097421424297', '-5.9383139302196134108846331752927639971676257158901473627e-182304142910134', '5.9383139302196134108846331752927639971676257158901473627e-182304142910134', 86, 4);
    T('7.55662016006962002975126086868881090e+92836539313768', '7.389010362382571490934624701943275609639188368067594e+217', '7.5566201600696200297512608686888109e+92836539313768', 75, 5);
    T('-3.0013934546560704050078920708189692399117270883140217e+11075', '-5.1206e-235', '-3.0013934546560704050078920708189692399117270883140217e+11075', 81, 0);
    T('7.66223342859066576592906505101217897398546325495846e-82754189689399', '5.95702361430732425683730251983e+2864020455766', '-5.95702361430732425683730251983e+2864020455766', 79, 3);
    T('-1.036414e-2174635', '2.91036351987667801029e-546512366029', '-1.036414000000000000000000000000000000000000000000000000000000000000000000000000001e-2174635', 82, 0);
    T('-7.73669558555582012461580395521095204434412941e+50', '-3.81765105541144931054751705959658314711971196902605985e+61', '3.8176510553340823546919588583504251075676024485826185559e+61', 94, 2);
    T('7.586282e+1388578536463', '3.7340962072168169109e+196099', '7.58628199999999999999999999999999999999999999999999999999999999999999999999999e+1388578536463', 78, 1);
    T('-1.93587268078935271941894136873985162017862507542818e+321163124370911', '6.943818590038777426128705240184664316847451787704461811e+408423', '-1.93587269e+321163124370911', 9, 3);
    T('9.158021383652e-154', '7.83728703110783259839081960e-126678274493', '9.158021383652e-154', 71, 6);
    T('1.120377703769e-9063292827149', '-1.157270649404e-81777595', '1.157270649404e-81777595', 40, 5);
    T('3.00956694635622221640599503737923785284858333e+23793', '-8.296841e-62036', '3.0095669463562222164059950373792e+23793', 32, 3);
    T('-6.937410732622974517e-132898871', '1.787915338219063663284766465308261910571839890736999688e-7092051360', '-6.937410732622974517e-132898871', 54, 6);
    T('-5.49594663201325125201554400537805734945030601871644938e-7511590982452', '8.913069705406116253758138765590173639444565020055470e-664545078', '-8.91306970540612e-664545078', 15, 6);
    T('-5.7228945304559887607072831695372431498529e+62465478235', '-2.313661821055559260523012352193557946985749105836e+555426852', '-5.7229e+62465478235', 6, 0);
    T('8.290579601780532561093450657307118645439e-5252', '3.7045223552391549833354415275186787432226e+25817549722', '-3.70452235523915498333544e+25817549722', 24, 5);
    T('6.1011483036588e+8525286615', '-7.9407635934023640031477e-64483629012', '6.1011483036588e+8525286615', 26, 5);
    T('1.79827442013964442705151300507360553757969774e+8581546069469', '-2.93692061953654e-921366480181', '1.798274420139644427051513005e+8581546069469', 28, 6);
    T('6.0238560e-3338', '-4.884692435849030087442302e+679017305497', '4.88469243584903008744231e+679017305497', 24, 2);
    T('-6.818842883657318590982754425151e+822736749188241', '9.660840460764636791536129324692825710570798631e-8023414384021', '-6.818842883657318590982754425151e+822736749188241', 68, 4);
    T('-8.50313303653207028184245769163492065224586037763879855379e+6523703', '-6.1905775109545542289079e-500857013166', '-8.50313303653207028184245769163492065224586037764e+6523703', 48, 4);
    T('-6.9345081514848526e+408422427843483', '5.845535e+825', '-6.9345081514848526e+408422427843483', 67, 4);
    T('8.3458844381453492e-421249029', '-6.67604793981330525e+663653533', '6.676047e+663653533', 7, 1);
    T('-9.8255505601011771562122999588702792722155986813452e-6453828404355', '-3.156966e-69319998294053', '-9.8255505601011771562122999588702792722155986813452e-6453828404355', 69, 5);
    T('5.7479189192065655670535710718063446e+3178639', '2.36584147826729754742435968924084514177878e+9701', '5.747918919206565567e+3178639', 19, 1);
    T('-9.6451682304877638471486445e-751', '7.474448211362317659648754809e+2092', '-7.47444821136231765964876e+2092', 24, 3);
    T('-3.27e+9640492', '-5.025658584718327200176000242331331583e-12382225537', '-3.269999999999999999999e+9640492', 22, 1);
    T('-3e-17009869', '4.58000e-34285139487', '-3e-17009869', 12, 6);
    T('1.6939481662e+3742759810', '-5.0722197601116875316737838662477469887668133165467097784e-733009335', '1.6939481662e+3742759810', 63, 5);
    T('-6.39963783869163390347546390254487607732868760405455253e+651383003790', '-1.9135201650090626893223584406041574079590e+51', '-6.39963783869163390347546390254487607732868760405455253e+651383003790', 98, 3);
    T('5.32531e-22728729232', '9.589422322010781085684383010495470639608180808868e+80', '-9.589422322010781085684383010495470639608180808868e+80', 77, 0);
    T('-6.8890733906576999238363230000976152778871890965e+45594183', '6.26366187293795199769687089620760865e+74432670586149', '-6.263661872938e+74432670586149', 14, 5);
    T('-8.220407156509335347e-24775430564923', '-8.77031616700265016529817084401875950557e-9722', '8.77031616700265016529817084402e-9722', 30, 5);
    T('-4.40318354347918923678016782508294771754751e+343285906969', '5.54317e-21943645895137', '-4.40318354347918923678016782508294771754751e+343285906969', 58, 5);
    T('-8.01267104446171107371931147420066896577783e+290724285965792', '3.922853626534224351037706799707066882529610220033686e-2533893', '-8.01267104446171107371931147420066896577783e+290724285965792', 62, 6);
    T('-7.46711816429017899052142290474804396940427468378317e+88166', '1.70350767500e-94214', '-7.46711816429018e+88166', 15, 5);
    T('8.2720358e-59', '-2.12300256073022043654177442922727e+232472', '2.12300256073022043654177442922727e+232472', 99, 6);
    T('9.6731727866346733935637755818e-49876', '6.828187558754210917313361e-3938333540', '9.673172786634673393564e-49876', 22, 5);
    T('4.970290384206372400110717845553304575451691347206788136e+7409600674', '-7.214070106511e-14848736914', '4.9702e+7409600674', 5, 1);
    T('-5.286227808362139860982903863334427890e+7540', '8.9436097871e+65541134042', '-8.9436097871000000000000000000000000000000000000000001e+65541134042', 53, 3);
    T('5.806187873544329364e-598311333', '-8.59117696e-81034920149893', '5.80618787e-598311333', 9, 4);
    T('-4.0513814801266717e-701', '8.8350642099712686045e-44802039348883', '-4.0513814801266717e-701', 24, 4);
    T('-3.9495250612559823287066415204896688223850154237010820e-345907678', '-40.760547', '4.0760547e+1', 72, 4);
    T('8.9101600053331523669e+6543334385', '-5.596528154634386943001e+32715', '8.910160005333152366900000000000000000000000000000000000000000000000000000000000000000000000000001e+6543334385', 97, 0);
    T('-5.317e+8801157', '-0.8124384261972401849820208783631770801553219189', '-5.3169999999999999999999999999999999999999999999e+8801157', 47, 1);
    T('1.458220549522405632897617412303549241350902e+910585', '7.12440e+80698861', '-7.1244e+80698861', 28, 3);
    T('3.607225737556892046190180381e+14383177004580', '7.1995282911743022122993582787775e+42', '3.6072257375568920461901803809999999999999999999999999999999999999999999999999999999999999e+14383177004580', 89, 3);
    T('5.384257489402764868639219663415702327739707095177059024183e+796866015902231', '-5.6629857407679740298073032312781087279134036073166463269e+8642441692528', '5.3842574894027648686392196635e+796866015902231', 29, 2);
    T('-2.67374286061942701120442227319582e-796929', '-3.1137170092937801921265e+19713890848509', '3.1137170092937801921265e+19713890848509', 36, 6);
    T('-2.2023441762566828989021729390663400121946e+9644', '8.3950986e-3357883498', '-2.2023441762566828989021729390663400121946000000000000000000000000000000000000000001e+9644', 83, 0);
    T('1.9120536109908e-359181513460214', '6.90e+162344916', '-6.9e+162344916', 25, 3);
    T('9.0084e+9036412557', '-1.6131384829610342221722228276029e+17030064206', '1.6e+17030064206', 2, 3);
    T('6.94006040679377396800484e-328702744036690', '5.7631866190064e-84526559664615', '-5.7631866190064e-84526559664615', 40, 4);
    T('-3.965603782726016772443671787732867699885140982e+635', '4.483383243584369787434747557978049901909058715479e+50', '-3.96560378272601677244367179e+635', 27, 3);
    T('-243590213793708459406772932437010216172635882306475831', '-6.7e-44728', '-2.43590213793708459406772932437010216172635882306475831e+53', 98, 4);
    T('9.9474599228652036516232719946e+598379030641', '6.892e+694929631077', '-6.892e+694929631077', 94, 0);
    T('4.08271183079811717670216539980157688694805125543e+99498574232162', '-5.686625469623280e+20', '4.0827118307981171767021653998015768869480512554300000001e+99498574232162', 56, 0);
    T('6.20798160698725703136842e+282', '-2.627314455483202315342281139415158228828821712570e-74408922042', '6.20798160698725703136842e+282', 89, 6);
    T('-6.930249009921875230376299574192579661073e+902', '-2.933413062992299322275973329511437767e-374426', '-6.930249009921875231e+902', 19, 3);
    T('9.6840608221583258393611779032323e-574164277585049', '-8.023141542715077397102901e-35682858', '8.0231415427150773971029010000000000000000000000000000000000000001e-35682858', 65, 2);
    T('7.53765e-6003', '-9.322937379904932682791856691900962731116251139e+80', '9.322937379904932682791856691900962731116251139e+80', 47, 4);
    T('-6.16797633780774884931256e+51750563848802', '9.91e-507415095824', '-6.16797633780774884931256e+51750563848802', 97, 5);
    T('9.8391782e+7270203', '-8.252219232142684513e-5165787617133', '9.8391782e+7270203', 22, 6);
    T('-8690824183721974257625', '1.77239995570673315504728712241568179350193770090e+2828613430', '-1.8e+2828613430', 2, 0);
    T('-4.15320797140256999983517832811957289905703350967471e-421218825616', '-2.363484504813444893389034747608010120149305982701e+49', '2.3634845048134448933891e+49', 23, 2);
    T('-3.513248468889269649988568563831917152121994e+4236948307232', '-669597.8475', '-3.51324846888e+4236948307232', 12, 2);
    T('40848574001044043117429880136.39330426', '-1.415317437075759007714202540601413e+829218144', '1.4153174370757591e+829218144', 17, 0);
    T('-683140928499589688768209268911600830386.59', '-7.623683585221188219564710325e-7360701', '-6.8314092849958968876820926891160083038659e+38', 64, 3);
    T('8.0783429626819202989724431494338824572325479329417317556298e+911977', '-7.7992926927388178390166e-50299117611887', '8.07834296268192029897e+911977', 21, 1);
    T('6.770552161045840939350671435e-418515708607877', '8.97833843335890802496278e+5470440757293', '-8.97833843335890802496278e+5470440757293', 59, 6);
    T('4.42508531685746557180021662427e+695448729', '-8.703921657942398357853588e+499380461', '4.42508531685746557180021662427e+695448729', 99, 6);
    T('-3.84930567932473474601693386138481713982679734312e-735619473', '-1.1647771e-45408987507', '-3.8493056793247347460169338613848171398268e-735619473', 42, 3);
    T('2.7979214141057542929122155167873828178928e-33006099286918', '8.99748962281864976594645e-6457', '-8.99748962281864977e-6457', 18, 5);
    T('-1.9463225510986585e-2339743', '-6.39680792858566161e-7458683', '-1.9463225510986585e-2339743', 67, 6);
    T('-9.41548737554334218412608905888742195213249016786e+20023920741986', '-6.60013409634830e+517810498721333', '6.6001340963483e+517810498721333', 32, 2);
    T('-6.07602011780755306557324989487965722342e+90281727315288', '5.8750457572327319570806229112287e-254016713846398', '-6.07602011780755306557324989487965722342e+90281727315288', 52, 2);
    T('-9.1e-64', '3.17050155164397544109492520994716e+33', '-3.17e+33', 4, 1);
    T('3.3006599191063140476080953060391784666473188419118e-5640', '1.53664064248484e+5232827', '-1.536640642e+5232827', 10, 5);
    T('1.5031728844877389e+169129722', '3.246404583073e+167491954254', '-3.246404583072999999999999e+167491954254', 25, 1);
    T('3.60287459279787034104281152069235353471341993e-709139649172', '1.82892596281048747959189090278619429542465527294977015513e+901645615906', '-1.82892596281049e+901645615906', 15, 4);
    T('4.84290347600031634661193e+436957222808', '5.0890082e+68987376620836', '-5.08e+68987376620836', 3, 1);
    T('6.800869533954425512233e-549049217628828', '-3.133608722019e-72159888682907', '3.133608722019e-72159888682907', 60, 3);
    T('-2.7016557386485772585880067995416461059220948660e-26490', '-6e+621218688', '6e+621218688', 77, 0);
    T('5.8289867290269863534227771857694e+2850647589', '2.615663842659270191634723896615120142183232e-27', '5.8289867290269863534227771857694e+2850647589', 67, 2);
    T('-3080330420143.3393', '-5.4534550e+9246', '5.453455e+9246', 88, 0);
    T('5e-94', '-3.46572683403e+26716', '3.4e+26716', 2, 1);
    T('4.9532377794194070098183594072077e+261364268520', '-3.187773792435254939254e+5766045544096', '3.187773792435254939254000000000000000000000000000000000000000000000001e+5766045544096', 70, 2);
    T('-1.31686267631009876837728513900e-84207', '-1.08649394616320750936e+5130721', '1.08649394616320750936e+5130721', 46, 6);
    T('-7.097016379429068198875025586341961142594520e-68476140838354', '5.6166425019324707206083844325451590851351e+642026', '-5.6166425019324707206083844325451590851351e+642026', 67, 2);
    T('50128595098988982241882365267492298494.3791', '6.154508315463430219985361e-8113886', '5.0128595e+37', 8, 6);
    T('5.4624661365230990831e-467475217764268', '1.03561225992e+4211558519', '-1.03561225992e+4211558519', 41, 0);
    T('5.7958690631810114535e-972584829868595', '8.93122280431707903084007612901522303842886289816465e-451954173', '-8.93122280431707903084007612901522303842886289816464999999999999999999999999999e-451954173', 78, 1);
    T('6.23117384095783877642555479268569599277571466919167804696e+5823769', '-8.88957951466051529259026881527303928865903e-383944987000967', '6.231173840957838776425554792685695992775714669191678047e+5823769', 56, 6);
    T('-3.8862481595945706991538345567305571893705e-25736028', '-1.3879405376587443276017231393053008580615841e-25', '1.3879405376587443276017231393053008580615841e-25', 71, 5);
    T('-9.970125939372976490559159080747978541068839759984529925139e+432018', '5.98056550553408672133315380461463570e+896065133809', '-5.980565505534086721333153804615e+896065133809', 31, 3);
    T('4.8832591258746957740493453210504758579038e-746576406955', '-7.936e-9966668464', '7.937e-9966668464', 4, 0);
    T('-7.0799445802027375398954869491e-5796310856', '-6.5220741407805848e-268186', '6.52207414078058479999999999999999999999999999999999999999999999999999e-268186', 69, 3);
    T('-2.7101376909259065827e+28', '3.4058760584213411163987127471065442046291374712858412488e+59', '-3.405876058421341116e+59', 19, 1);
    T('-6.882253879e+473531464', '8.860499558339e+17', '-6.8822e+473531464', 5, 1);
    T('-5.16667489099146763738656435423011625400310594444218241232e-9229121', '-9.0063301294338986439532882603778665e+81629', '9.0063301294338986e+81629', 17, 1);
    T('-5.157509243239829091726964298324971469843e-4440376100', '0.098318', '-9.83180000000000000000000000000000000000000000000000000000000000000000000000000001e-2', 81, 3);
    T('5.334236990954020580067359234689397946843850183e-45216782', '6.8467195362204663178127556226517194397012631341e+705023210274081', '-6.8467195362204663178127556226517194397012631341e+705023210274081', 67, 6);
    T('1.9595e-40108065', '-4.17372268456512739501058621090605609e-811864145', '1.95951e-40108065', 6, 0);
    T('7.6788880750410844999e-7044594410309', '8.047202685713951170353064361268935552e+82151171', '-8.047202685713951170353064361268935552e+82151171', 58, 6);
    T('2.40947662418074187073869349805963702300e-717', '-1.06355212330104469252398131e+9616', '1.06355212330104469252398131e+9616', 36, 1);
    T('9.622739026648481019581291538336064434627e-8570361', '-4.794954460599355243327495614579630512029e+8680617120', '4.794954460599355243327495614579630512029e+8680617120', 78, 3);
    T('-2.0563286167099015337242288117913955687e-9871394', '3.751095998295388492830539896428445922247e+135126041545564', '-3.7510959982953884928305398964284459e+135126041545564', 35, 2);
    T('-1.14483905637604109768217454543982293329707e+136', '-3.828559e+756110', '3.828559e+756110', 100, 4);
    T('9.229565015903e-375832866', '5e+718174416577', '-5e+718174416577', 24, 0);
    T('-92552250262900710901996902640117245729959569306639786.727965', '4.130864178e-7561792905', '-9.255225026290071090199690264011724572995956930663978672796500000000001e+52', 70, 0);
    T('6.90567764929230720760129e-51930', '-1.34385677e+46632498930069', '1.34385677e+46632498930069', 50, 1);
    T('8.1e-514782', '-5.83718902014328689e+7504427', '5.83718902014328689e+7504427', 60, 4);
    T('-4.21031703e-90663063917733', '-6.2295688390884e+449', '6.2295688390884e+449', 34, 5);
    T('-2.50654571144190195536714089036230628359646827623129674e-6985582556', '4.254316209345275373e-7599901144', '-2.50654571144190195536714089036230628359646827623129674e-6985582556', 100, 5);
    T('6.0209182019411e-406155', '3.3144431518e+49061556661238', '-3.3144431518e+49061556661238', 82, 4);
    T('-1.0746447992612064e+20048642076332', '2.589728549403e+261743400670494', '-2.589728549403e+261743400670494', 28, 1);
    T('6.83187573311783133388675574862700236066016e+768408058313', '-4e-9621770', '6.83187573311783133388675574862700236066e+768408058313', 39, 4);
    T('-9.8426861786376745395338316736372372385247273802481182654e+7002259137', '7.3848984652766640441190992557464e+5426861661024', '-7.3848984652766640441190992557464e+5426861661024', 33, 2);
    T('1.365645930e+956', '-7.482252668478512051347446347357526447533057303870682e+91200734', '7.482252668478512051347446347357526447533057303870682e+91200734', 95, 4);
    T('821265270789800674983', '7.596594403677e-2238528524090', '8.21265270789800674982999999999999999999999999999999999999999999999999999999999999999999999999999999e+20', 99, 1);
    T('-6.22869706424229085199969277468838678203842929715e-190', '9.74428107595571762813020331585660542e+6573', '-9.744281075956e+6573', 13, 5);
    T('7.25480400089763062676753162197138598798192e-437323096', '5239066878381171241065019749598720532700', '-5.239066878381171241065e+39', 23, 6);
    T('-9.563458523474507229e+2853406524', '2.2936167691376e-541335752690', '-9.563458523e+2853406524', 10, 5);
    T('-746741192391547425313481301792128091704269688.5504230', '-8.77483e-7533724', '-7.46741192391547425313481e+44', 24, 1);
    T('2.40574779493779391402e+69623216485', '-3.1677789244383754929857461394776755376682858e+6492', '2.40574779493779391402e+69623216485', 85, 4);
    T('3.34438453305869590827959712699118081149176698675159e-89', '5.0678566855404829179e-81435009634014', '3.34438453305869590827959712699118081149176698675159e-89', 63, 2);
    T('-3e-59422', '5.07870e-341498', '-3.000000000000000000000000000000000000000000000000000000000000000000001e-59422', 70, 3);
    T('8.551234774993618924952949923583302046e+72512', '-4.92940959247030e+492477879', '4.9294095924703e+492477879', 84, 3);
    T('-1.361719249828564869117141563389842513120731412842167e-5163107709', '-2.3404697401571e-8138656364361', '-1.361719249828564869117141563389842513120731412842167e-5163107709', 74, 4);
    T('2.6953552134e+6413', '5.1391908907404147e+28973', '-5.1391908907404146e+28973', 17, 1);
    T('-2.79345665386595116510749612390e+97013391761', '-9.825296231966561e-9055696631085', '-2.8e+97013391761', 3, 3);
    T('3.0353463829791557484034577e-5380665', '-5.45164080531782789728429260454194592620406e-77368069', '3.0353463829791557484034577000000000000000000001e-5380665', 47, 0);
    T('-5.55498899931818682712758653208745138467508909e+700', '-7.85892782051560288574610948650345970082816862632720168e-4170018432', '-5.55498899931818682712758653208745138467508909e+700', 54, 3);
    T('-4.3055420058280237492e-971', '-5.3609492787581e-904484921106720', '-4.3055420058280237492e-971', 41, 5);
    T('1.868246817188649306845117414106e+50428', '7.5366455235647572770783e-635289', '1.868246817188649306845117414106e+50428', 99, 6);
    T('7.62931391828258531256199597381285333e-192827362', '-3.089793081e+69087062', '3e+69087062', 1, 1);
    T('6.406005211759073136733699837721e+47176258', '-6.623969165313732656715168e+8309', '6.406006e+47176258', 7, 2);
    T('-6.264316730463e-81678333645', '3.914213121658545192593587878080852e+4263246490061', '-3.914213e+4263246490061', 7, 5);
    T('-3.88601005610e-12', '-6.0291757685819877626299256926488058714710369715e-59454', '-3.8860100561e-12', 24, 5);
    T('-1.0e+504563', '-7.8136370806e-886174', '-1e+504563', 49, 0);
    T('5.16885495769133557973782019188749092718660471067407480501427e-28638128292376', '-1.64281332853e+6220172367282', '1.6429e+6220172367282', 5, 2);
    T('-7.281266371685117706140096937263219723646073352e+51087476171394', '-1.515549786183130085214412443033170110466e-79410181', '-7.2812663716851177061400969e+51087476171394', 26, 4);
    T('9.27612671438e-1561970318', '-9.08704285501155950e-837971', '9.0870428550115595e-837971', 47, 1);
    T('-95064446182266177065.4012175931295970485028108483228878878', '3.355554973493393325306084561740544574539e-53', '-9.5e+19', 3, 2);
    T('8.57168083278085708967811738763762e-2365', '-6.2594309702280039379e-85462774', '8.5716808e-2365', 8, 6);
    T('-3.766269161937361260549564054060639862714048898e-51750122', '-58117886443474726539693454817134221828209739211441699883.0691', '5.81178864434747265396934548171342218282097e+55', 42, 5);
    T('1.2313405405407068943876703e+2275989097815', '-6.482524622658050e-77', '1.231340540540706894387670300000001e+2275989097815', 34, 0);
    T('6.0922642060804279527625638878795991602336201198e+8097802680593', '2.57763698e+9', '6.092264206080427952763e+8097802680593', 22, 2);
    T('6.0637321458e+85799270780823', '-2.7446949815766289e+792187320745800', '2.7446949815766289e+792187320745800', 92, 3);
    T('-4.102603833841007057879773628652e+478', '9.2972584044416e+2666', '-9.2972584044416000000000000000000000000000000000001e+2666', 50, 0);
    T('-9.81204058479981557954751814453993009915994976521382547456e+6617716058', '6.4811554271243889730963124954354430903681035694227173e-20053707178', '-9.81204058479981557954751814453993009915994976521382547456e+6617716058', 59, 4);
    T('6.6256303869962582349781626601e-668437012211143', '6.301760e+87012176122', '-6.30176e+87012176122', 81, 5);
    T('-5.14944916148006571994769e+40683649', '-1.9617116429969291229337655417657996003e+365885841572', '1.961711643e+365885841572', 11, 6);
    T('-1.0052610707940017159606837131440042272999369292584366200e+64', '-7.89333705810291493259288758228104550243371116e-10009886559', '-1.00526107079400171596068371314400422729993692925843661999e+64', 57, 2);
    T('1.9490704616317326956716050729856196202214439391044118309e+808548815581', '9.4392019485857153260696072430234273374456984231962712699613e+4290210', '1.9490704616317326956716050729856196202214439391044118309e+808548815581', 67, 4);
    T('2.515679900317671341910697450255002166592617811e-90170262', '-1.398544e+4344939', '1.398544e+4344939', 91, 5);
    T('1.230929163997093391744134e-66296', '4.54283698426300e+37', '-4.542836984262999999999999999999e+37', 31, 1);
    T('-8.721571045657026492e-886536049', '-3.8305781011776041172334e-12', '3.8305781011776041172334e-12', 68, 4);
    T('5.409e+9570174', '-1.82926019940805067409398497882731194703443880876e+908055913089', '1.8292601994080506740939849788273119470344388087600000000000000000000000000000000000000001e+908055913089', 89, 0);
    T('-8.12542242825812e+670', '-5.395624536681751881e-3175', '-8.1254224282581199999999999999999999999999999999999999999999999999e+670', 65, 1);
    T('8.42774658679e+5436715', '2.310704391390602487662824748478443014886147517467215e-62274642049', '8.42774658679e+5436715', 27, 4);
    T('5.45037942785172644822499354624298903389106e+4316763235473', '-6.299211e-6799874875', '5.45037942785172644822499354624298903389106e+4316763235473', 78, 1);
    T('-1.53145179523004929108940648e-4786', '-5.84078999870447852081405e+337000150712', '5.84078999870447852081405e+337000150712', 99, 0);
    T('1.38555414120025793996740423514078792155515521e-654', '2.9834364219393e-37045', '1.38555414120025793996740423514078792155515521e-654', 66, 6);
    T('4.8309618859408904037e-93', '-5.31821579750016146315634174353567082363200e-243451594', '4.8309618859409e-93', 14, 6);
    T('7.622991798102762351576932136891725079227e-9018579156026', '-2.341900778658863154328180072493187412164576799462107434945e+6540939965591', '2.341900778658863154328180072493187412164576799462107434945e+6540939965591', 92, 6);
    T('-7.12456596459062138582172119206e+38', '2.19e-857010933584842', '-7.124565964590621385821721192060000000000000000001e+38', 49, 0);
    T('-4.756406954056679771989505403e-22815', '-737398047473708601', '7.37398047473708601e+17', 85, 5);
    T('3.873374351016985472406490686062e-238243', '-4.6762860021346716612217660589e+155153721747', '4.6762860021346716612217660589e+155153721747', 94, 4);
    T('-9.4356858523747133731079631010e-5472634943534', '3.86193124031370106632726251e+166', '-3.86193124031370106632726251e+166', 73, 5);
    T('-2.52524738045955935978637146663029992912771949797e-67843938', '-4.603949786228012492668533454248e+36', '4.603949786228012492668533454247999e+36', 34, 1);
    T('3.52019349007728628079170615e+5425085316', '-8.82596034568218999227066870942655351212954959e-9133609', '3.52019349007728628079170615e+5425085316', 27, 1);
    T('-3.556447849117481556770218013759688015489294098340718e+900880', '6.26376647117515095296767197361398832140376191583843224916605e+91696200', '-6.263766471175150952967671974e+91696200', 28, 3);
    T('-3.83958283372406132999820728337800305879e-5688671766', '-7.9395049548503776142560557540068648434311852160887256078348e-91127227964', '-3.8395828337240613299982072833780030587899e-5688671766', 41, 2);
    T('-1.300813538713692e+201', '-3.00391105020400564588155817692723506608792978919010565788e+71096320860117', '3.0039110502040057e+71096320860117', 17, 2);
    T('6.3969e-141', '-3.7019599723567186875129220e-6409901820', '6.3969e-141', 37, 3);
    T('-9.71371642650108639295160695771548854528e-56778309', '6.980165172877928197e-59755368114375', '-9.7137164265010863929516069577154885452800000000000000000000000000000000000000000000001e-56778309', 86, 0);
    T('9.5371963680356617658845092739747e-7159', '-7.7095483513e-36', '7.7095483513e-36', 51, 6);
    T('7.49984473799279956e+9886976325566', '4.185551138590363109380621936108421e-828549', '7.49984473799279956e+9886976325566', 27, 6);
    T('3.01874630153032416641772521473044896e-1761219792', '1.185271015671e-34862878', '-1.185271015671e-34862878', 40, 5);
    T('-5.6404626903691592864033350800102640e-688', '1.446311753651123907602073897994884135913e+35086', '-1.446311753651123907602073897994884135913e+35086', 50, 4);
    T('2.58981535424883305395920840e-67837148', '-6.09e-61474891', '6e-61474891', 1, 6);
    T('-5.82515656171815036585589841381284857936361239e-960684004462', '-5.8977696600211753e+64', '5.8977696600211753e+64', 96, 6);
    T('-7.8340195501475659495039688653518478720786278435896182e+88', '6.3661721127354953145105290708976809506529677316e-8871534141', '-7.8340195501475659495039688653518478720786278435896182e+88', 96, 2);
    T('-8.51897975763330241969074497857465601745229061e+38770890', '2.5389339972282087e+8927', '-8.51898e+38770890', 7, 6);
    T('-9.3380650404544864856154963057234789441679279e+137', '-3.03413491e+18955145', '3.03413491e+18955145', 20, 4);
    T('-9.20983036891690627556727451299849e+8514', '4.3581459999471678026410101746e-5579', '-9.2098303689169e+8514', 15, 1);
    T('4.794947628124200797254492691993397579929543385530182e+75106847', '1.9232141e+31', '4.794947628124200797254492e+75106847', 25, 1);
    T('1.9486268675614338561993976858380365576073e+49', '-1.3246154148916503571177611111e+337296083', '1.324615414891650357118e+337296083', 22, 4);
    T('-5.627086198081730068080580042002346925329e+126', '8.37759387894e+3780535386734', '-8.377593878940000000000000000000000001e+3780535386734', 37, 0);
    T('-4.384338608070383419808533917307917434372068906283901049919e+63796', '2.797419220483761948124072190891464302876987e+25884906', '-2.7974192204837619482e+25884906', 20, 3);
    T('3.4078189314391566e-18503495', '-5.30130976886555886723104017903745804786381e-2657183676248', '3.407818931439156600000000000000000000001e-18503495', 40, 2);
    T('-6.67814725996012261896056261457692063384899e+443203827813718', '-7.094322270992269363979536e-76468483749', '-6.67814725996012261896056261457692063384899e+443203827813718', 81, 3);
    T('-1.5944322052140703403667216972479558354768749626392e-532', '-4.1628930e+59342254762', '4.16289299999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+59342254762', 93, 3);
    T('1.57971169425091353727394009736431363630172297e+163827690763027', '-7.46518579653794634650213781762237234138605438635754074e-44086868362762', '1.57971169425091353727394e+163827690763027', 24, 4);
    T('-6.46570187699914586667020784e-57225863', '-3.605020406037536930365779608374490e-6333187499173', '-6.4657e-57225863', 6, 2);
    T('-3.265e-638101078', '-3.982560777e-266602009309', '-3.26499999999999999999999999999999999999999999999999999999999e-638101078', 60, 1);
    T('-5.2929203260738847351754414699e-877816999', '9.48121e+1086', '-9.48121e+1086', 20, 2);
    T('3.9543e+22', '1.552e-5878699304018', '3.9543e+22', 60, 0);
    T('-6.3210718612136373360663477297212906150882111736e+798441437403', '8.207e-910', '-6.32107186121363733606634772972129061509e+798441437403', 39, 3);
    T('9.4028817145182e+62656173966', '-9.6662367909511404899166370822281472369e-6738697', '9.4028817145182e+62656173966', 20, 3);
    T('5.0289051369840051730736955513954074020024300985917877271304e+993946', '-4.09419542e+3440218', '4.09419542e+3440218', 26, 6);
    T('-9.1380565757315539e+21230389', '-8.540959023510666542521289711858059655882912e-1353', '-9.1380565757315539e+21230389', 89, 4);
    T('-1.790066168849984584579974786814644869751254131948784306e-8105', '740396638717789074659.910965', '-7.403966387177890747e+20', 19, 4);
    T('7.192289686816426061201913416408e+65385387176', '-2.048098677723710505030e+31', '7.192289686816426061201913416408e+65385387176', 80, 4);
    T('-7.244729596127766237142009188521973749980648412771734e-643847', '-1.539134e+114025029390', '1.539133999999999999999999999999999999999999999999999999999e+114025029390', 58, 1);
    T('-6.935805486476e-7913', '-1.74069693505191176731745138363227986544641678e-80', '1.741e-80', 4, 4);
    T('5.6788814514562324096586930570893144636654306429185e+91840282929', '5.781105754464537505122e+18008012051881', '-5.7811057544645375051219999999999e+18008012051881', 32, 2);
    T('-1.8654767337123218841416266956132771240634407e+100109', '-1.1e-638821348967073', '-1.8654767337123218841416266956132771240634407e+100109', 100, 3);
    T('-1.8814718212110199581038089089e-207452', '4.0131276323653741e+85081972256255', '-4.01312763e+85081972256255', 9, 4);
    T('9.8285917673400966401386017736741e-49', '5.23696175223598017266816e+9298851629433', '-5.2369617522359801726681599999999999999999999999999999999e+9298851629433', 56, 2);
    T('4.9525543138230233733958171412861399192339633e-43763659', '-9.63088715693508555925127701612393468243974565890040e-8029790591', '4.952554313823023373395817141286139919234e-43763659', 41, 6);
    T('43807193118416220.30514532578', '6.82400537483533278238630867487901066254e-1258499622493', '4.380719311841622030514532578e+16', 79, 2);
    T('1.507551462138e-940780528817', '3767580439153.4834492438611658228494150678908293926214', '-3.7675804391534834492438611658228494150678908293926214e+12', 69, 5);
    T('6.40125268424961274702295045943396197917431995e-40', '-9.851694586465288857e+25', '9.851694586465288857e+25', 43, 5);
    T('7.8593200925617641963481899e-38', '-5246347002423763295469992802728403261102911772862336194.09800', '5.2463470024237633e+54', 17, 4);
    T('8.28530008502e+982226', '6.15384755580018259314526630e+68660', '8.28530008502e+982226', 51, 2);
    T('-1.8801497384408189640320015038999568842866e-24878900', '-9.5984187974385690224040783811998871469524823079514023707606e-84720704', '-1.88014e-24878900', 6, 1);
    T('5.512590233e+67', '458562359600543477716774789761183.9721', '5.51259023299999999999999999999999995414376403994565222832252102388160279e+67', 92, 4);
    T('-7.961981820e-9292', '3.4950250395425106341180691e+17229766', '-3.4950250395425106341180691e+17229766', 90, 2);
    T('2.84520934466976684398983886295756206370388080421241701e-9274365509', '-1.82468162077e-5933528505783', '2.84520934466976684398983886295756206370388080421241701000000001e-9274365509', 63, 2);
    T('3.0638426500180957038704291851460465744379e+97308187', '7.17e-20', '3.0638426500180957038704291851460465744379e+97308187', 62, 5);
    T('-6.7857051829097741322057124465916305503348059723667815267936e+39116833351', '-7.97213624518974727787110816512295e-901642235782', '-6.78570518290977413e+39116833351', 18, 6);
    T('3.98276442429730687734260200965410e+8863317597506', '1.92277105245650259537083208410791181900942924278e-5024771972', '3.9827644242973068773426020096541e+8863317597506', 67, 4);
    T('-2.842e-3940686', '9.304715575e+62694878085040', '-9.304715575e+62694878085040', 82, 2);
    T('-3.92161165070912505384406012966221178585348e+99519805', '9.3301e-561230434103', '-3.921611651e+99519805', 10, 3);
    T('5.30181e+814216', '4.9696154e+846105621352', '-4.9696154e+846105621352', 91, 3);
    T('-2.5611019664086413100496988294216275285518948213545e+34138816328', '9.39577481747702484298748797650912856487068e-3232', '-2.5611019664086413100496988294216275285518948213545e+34138816328', 55, 6);
    T('2.01405660274861423660870e-2947770', '2.3990771314310e+30066', '-2.399077131431e+30066', 94, 5);
    T('-3.5880750723188e+203135690', '-8.62405497639726425287e+324', '-3.588075072318799999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+203135690', 100, 2);
    T('-6.4277516e+16', '8.497413457204722329837654431514224414186e-6115121363743', '-6.4277516e+16', 90, 1);
    T('-5.7867359128524182860686536898184308043439775443158538276e-446025', '-2.72297877592674510275279155788952347642040690766e+9614786423', '2e+9614786423', 1, 3);
    T('-7.7380528494169615025024624992532846424348941391836740419e+121', '4.3090247913227408189977989098807606670689796865754779740e-824976197040', '-7.738052849416961502502462499253284642434e+121', 40, 1);
    T('-1.9838994044795552022209656344e+695133', '5.49818518525258315185697821167131491889755444845630635830e+753747200599', '-5.4981851852525831518569782116713149188975544484563063583e+753747200599', 76, 4);
    T('-1.150016011347174752768741177479456351829159228387857581405e-295047710879', '6.4446428e+263545176', '-6.4446e+263545176', 5, 5);
    T('4.647303861000e+65788930132', '-3.24612504040010e+20810923', '4.647303861000000000000000000000000000000000000000000000000000000000000000000000000000000001e+65788930132', 91, 2);
    T('-1.801294109424e+98057429', '4.040966752609839319727392644e-6284865', '-1.801294109424000000000000000000000000000000000000000000000000001e+98057429', 64, 3);
    T('-58094623085819236781.0873323671520373564445536295898455802790', '-3.959433266340496455405621983735347085278992775015577252378e+325', '3.959433266340496455405621983735e+325', 31, 6);
    T('-7.220271349955e+333174959697299', '8.0256e-41602464886483', '-7.220271349955000000000000000000000000000000000000000000000000000000001e+333174959697299', 70, 3);
    T('-7.1614887997801154861609740817345e-332', '-9.1570523163280492314234679199550214012573597453e-6070993324', '-7.161488799780115486160974081734499999999999999999999999999999999999999e-332', 70, 1);
    T('8.4661263903087015200066464e+636215921', '6.356311813708290991135162289758223335010987348194488e-45076676261642', '8.4661263903087015200066464e+636215921', 50, 2);
    T('-3.179220912627314213490201928731978343917149043084002307540e-188', '2.351981046e-359074096', '-3e-188', 1, 5);
    T('-7.53648663725991201657589848577218563721068525335115095e+31596435238', '6.1784221063838165429381221662285e+6193', '-7.53648663725991201657589848577218563721068525335115095e+31596435238', 65, 4);
    T('4.275744684310e+697261963', '-7.2972151556269e-346394077502', '4.27574468431e+697261963', 55, 3);
    T('-8.316522985907941149315819691027481359782131768998030479e+4529779', '-8.257319454209651091397501338084425468614497314434512123761e+3815826785', '8.257319454209651091397501338084425468614497314434512123761e+3815826785', 69, 5);
    T('4.28031321413696e-2909374376', '-4.7464794014730939797037045e+26', '4.7464794014730939797037045000000000000000000000000000000000000000000000000000001e+26', 80, 0);
    T('6.9e+15196', '5.0335129102281603697326119671313471e+26678916', '-5.0335129102281603697326119671313470999999999999999999999999999999999999999999999999e+26678916', 83, 2);
    T('6.05710130032e+845818', '-6.84799830993007983672455478441236697792984506399152863e+263489177077', '6.8479983099300798367245548e+263489177077', 26, 0);
    T('-7.259110232683602517137661640406e-233943708422', '-3.570702955230575509387e-6090175638854', '-7.259110232683603e-233943708422', 16, 3);
    T('7.9552646733297450705102587373601173926397989199630574390594e+5833873262374', '7.23783224793024937920994726457488630673692118062804852094260e-4170', '7.955264673329745070510258737360117392639798919963057439059399999999999999999999999999e+5833873262374', 85, 3);
    T('-7.2687487279738819376808747153e+6313791100', '-4.83830672934356168975871976324383414675499331185665164e+9298', '-7.2687487279738819376808747152999999999999999999999999999999999999999e+6313791100', 68, 2);
    T('-7.07467146418616292083341812757167652413246785917377096677e-47699461626684', '6.6991387249414103427562694697846378289791912524150778e+99510030', '-6.6991387249414103427562694697846378289791912524150778e+99510030', 77, 2);
    T('8.132226900705584931034570223e+151418707', '4.47896737121174465935e+65', '8.13222690070558493103457022299999999999999999999999999999999999e+151418707', 63, 1);
    T('-7.680329e-64369373896', '-8.86936847751159161492070692217201307431465e-12', '8.8693684775115916e-12', 17, 6);
    T('-9.1272774e-22', '4.9308788e-264041148275262', '-9.1272774e-22', 8, 6);
    T('3.09469643668714201931881268612265166223679170026e+5601', '-2.504934889780577924430e+553632762145', '2.50493488978057792443e+553632762145', 21, 3);
    T('600411042946215242444443514354730802028936610658249.3379671', '-3.926391325619777278099096053769e-499', '6.004110429462152424444435143547308020289366106582493379671e+50', 84, 6);
    T('-8.724550696169635824e+274', '6.8415225733013159453649324e+11283269904', '-6.8415225733013159453649324e+11283269904', 38, 2);
    T('8.427624662923634567403911175882345629417509156289165853875e+48218040396', '2.74135025367815262669135356630651665559942459334708889436e-70710056', '8.42762466292363456740391117588234562941751e+48218040396', 42, 5);
    T('4.95215145665004685485370879167478001995251121537908606e-3602503', '-1445835271209857096871204157111.851268914', '1.445835271209857096871204157111851268914e+30', 97, 4);
    T('-9.6252228629967217412478580285181491394780630788515190770e+771199296', '-9.408237852224864988334053317100981766076456227364823e-6490290', '-9.62522286299672174124785802851814913947806307885151907699999e+771199296', 60, 2);
    T('9e+74368303', '9.94439711082e+48365757', '8.9999999999999999999999999e+74368303', 26, 3);
    T('-9.07847642176817733746245574503310248719891102543502e-94647', '1.4431964582945037401302273217599e+84558414613397', '-1.44319645829450374013022732175990000000000000000001e+84558414613397', 51, 3);
    T('1.13536584934398441973718743368191365501907e+528158825134', '-7.9852912749e-582807067892', '1.13536584934398441973718743368191365501907000000001e+528158825134', 51, 0);
    T('7.06921284533836379067100373859e+80989579', '5.4170222478221175537355e+41251320', '7.06921284533836379067100373859e+80989579', 41, 4);
    T('-6.115992139533e+9179471', '-5.925667434415081206265594172862733420571290147754e-3292', '-6.1159921395329999999999999999999999999999999999999999999999999999999999999999999999999999999e+9179471', 92, 2);
    T('1.6892338e-800361', '-7.3747503270351045309957575533644775e-9049', '7.3747503270351045309957575533644775e-9049', 73, 1);
    T('6.53109551412214866194393223161215605507349639e-51093', '3.078942469077473597898399895490302402783173899997628433e-48663005165', '6.53109551412214866194393223161215605507349639e-51093', 82, 2);
    T('5.59742057611520047e+3552322', '5.64e+4459', '5.6e+3552322', 2, 6);
    T('7.54510721608822460477241973796627534026817669152105487080e+1761914014284', '3.81655', '7.54510721608822460477241973796627534026818e+1761914014284', 42, 6);
    T('8e+84009634135828', '-8.9199103652545586308706415241883736350286480439667e+594878567', '8.00000000001e+84009634135828', 12, 2);
    T('-3.89180144358806788778373188031808692629802047854064734128133e-785756692528', '3.9983785346140142433848621884008989731422175736040437e+859679', '-3.9983785346140142433848621884008989731422175736040437e+859679', 89, 4);
    T('-3.222627959711127957e+46545852045', '8.887845e-777', '-3.222627959711127957e+46545852045', 25, 5);
    T('-1.4316161254158178213993181607920956330661956015903741752763e+9930195922357', '8.56187326770736368592926162202463e+965801', '-1.4316161254158178213993181607920956330661956015903741752763e+9930195922357', 74, 5);
    T('2.2298592980539212318487967e+3032564', '-4.2359693e+1001223', '2.229859298053921231848796700000000000000000000000000000000000000000000000000000001e+3032564', 82, 0);
    T('-9.503673e-180762', '-4.198424087339172328e+3421329544', '4.198424087339172328e+3421329544', 21, 0);
    T('-5.26357264972008e+15', '1e+3', '-5.26357264972108e+15', 16, 0);
    T('-8.231e+370614657158', '5.215764918953247929573216e-5029', '-8.231e+370614657158', 43, 2);
    T('5.9516788105856762934e+876942', '9.09259032582453443609510068069e+6383044', '-9.09259032582453443609510068069e+6383044', 48, 0);
    T('4.576277e+11', '-9.26862017e-852556476', '4.576277e+11', 40, 3);
    T('8.88862799608826064053485180294360350781688462e-34269637827', '-8.0884120910744461760854780805570614368e-22051', '8.0884120910744461760854780805570614368e-22051', 38, 3);
    T('3.78710914830473787748145097405780e+57238', '-6.3449000e+66678043', '6.3449e+66678043', 52, 5);
    T('-5.501e+287083035540734', '30572013515960.24951911339608280119590005', '-5.501e+287083035540734', 97, 2);
    T('-7.4538044572887175362785604219880905346494994193587040017e-9319204710057', '-1.801315e+23990', '1.801315e+23990', 53, 6);
    T('3.884190419355692216686734634783764e+62554', '-3.0066759716255e-20782293684', '3.88419041935569221668673463478e+62554', 30, 3);
    T('-2.1884244000546e+52542884', '-8.08399270090640250839602144004782289086467953893470453e+30423148900', '8.08399270090640250839602144004782289086e+30423148900', 39, 3);
    T('-5.601236428557798589229981706328869806994e-9787', '580.287174993001836191529492652138093349623372788454260', '-5.8028717499300183619152949265213809334962337278845426e+2', 98, 2);
    T('-7.72512178321325768141637256389878099848605969433e+414126087', '-4.06e+39141460486104', '4.06e+39141460486104', 93, 4);
    T('-6.3063659353e+312707261', '-2.421570342667657460293993926360239938183237055e-2526243988', '-6.3063659353e+312707261', 17, 6);
    T('2.8595448491969e-40096', '-7858434608529091956363353829643.9249', '7.86e+30', 3, 5);
    T('-3.964884399903e+67748', '4.1816564411775783007e-11', '-3.964884399903e+67748', 75, 6);
    T('8.277207563465813253022163e+536415320', '5.50986820383916856859394065053352e-49', '8.277207563465813253022163e+536415320', 75, 4);
    T('-1.9170710273232807545475989352e-5794', '6.69273674967778749532474384075329837e-23935604', '-1.9170710273232807545475989352e-5794', 51, 2);
    T('5.119225582795e-9594', '4.4751916441243570482212788084887210128e-7752398549201', '5.119225582795e-9594', 37, 4);
    T('5.383360676537625529e+25', '-4.673059354417459575221059037291710536683e+45', '4.673059354417459575274892644057086791973e+45', 79, 1);
    T('2.64184857975341110856e-4098197', '5.5068593146e+12', '-5.5068593146e+12', 82, 6);
    T('-9.247081858869003812585785e-25410488', '-7.008200112504616469824514679422618548195812654260114234e+291542300314393', '7.01e+291542300314393', 3, 4);
    T('6.8765148483e-523301049', '1.8430110e-90231841', '-1.843e-90231841', 4, 5);
    T('-6.560129277204461344461293361221542984860217067181e+9320929597542', '-2.525145009439e+795', '-6.560129277204461344461293361221542984860217067181e+9320929597542', 96, 0);
    T('-3.278825836767847923578e-915486510', '-1.09997169807036348882865231104e-9859674331105', '-3.278825836767847923578e-915486510', 38, 3);
    T('9448928605.202801181416075202161498311', '-3.9113768328292181359e-9590613044', '9.449e+9', 5, 2);
    T('3.479078950090341397643894943826049788725489059367e+642', '-8.3384332663390742127548e+655708892859', '8.3384332663390742127548e+655708892859', 50, 6);
    T('5.948273817758e+9846673970', '7.2457546992293132952627333513061338722683314067863415935e-75524', '5.948273817758e+9846673970', 76, 2);
    T('-3.121398868392008643544345899217722683076145e+422569', '5.322435351503672162565421e-8658418017', '-3.121398868392008643544345899217722683076145e+422569', 45, 2);
    T('-4e-437932130', '5609005045962.165', '-5.60900504596216500000000000000000000000000000000000000000000000000000001e+12', 72, 3);
    T('5.25951041059449965597642429467746515330538899887815e-4363066846307', '9.4217907968040680191784339e+391779969', '-9.4217907968040680191784339e+391779969', 83, 0);
    T('1.3671234471805230570615830123e+3874140501', '-3.174531982616e+49278', '1.3671234471805230570615830123000000000000000000000000000001e+3874140501', 59, 0);
    T('8.2723904827481e-89632', '-4.313174673305717e-2874534020227', '8.2723904827481e-89632', 49, 6);
    T('9.506089433368458720426733e+9446', '-1.981136836340706387598110091293691e+40085876323273', '1.981136836340706387598110091293691000000000000000001e+40085876323273', 52, 0);
    T('-7.3729650470087842796110550948859579641594783e-8616', '1.4661278722993e-464617', '-7.3729650470087842796110550948859579641594783e-8616', 45, 4);
    T('8.177161434473679355021249157808e+77500', '8.74373621537387872983728e-94198997', '8.177161434473679355021249157808e+77500', 89, 0);
    T('2.2109510447041998591646994e+7443884', '-5.33127194564170404e-5652505501', '2.2109510447041998591646994e+7443884', 62, 4);
    T('-2.0717026031107535408090332517124269344976603300824478779515e-6563318727', '8.497123340500312494845936775455957178964709202064436553015e-407', '-8.49712334e-407', 10, 1);
    T('3.40142461784209999410588401636114298493243248023050335015e-882375404', '-8.21194e+84', '8.21194e+84', 12, 3);
    T('7.4818614356579163060994396699711830e-2230141977340', '-6.7225652465430311972330655175027576580927055948e+34024540', '6.73e+34024540', 3, 0);
    T('-9.0315474436062073722598361145294896477856576472135709423618e-446877168320', '2.237920471636e-52928145007994', '-9.03154744360620737225983611452948964778565764721357094236180000000000000000000000000001e-446877168320', 87, 0);
    T('-4.430189628422615851153e-623', '9589.733022017', '-9.5897330220170000000000000000000000000000000001e+3', 47, 3);
    T('2.049338347696569911122e+456164037415885', '-8.29609115516590820826911915504256406848742381570e-368', '2.049338347696569911122e+456164037415885', 35, 5);
    T('-1.9600327833e-4236209312376', '-9.215e+519406837854', '9.215e+519406837854', 61, 5);
    T('-1.11943e+6', '4.28533456221e-9165269', '-1.11943e+6', 84, 6);
    T('-3.5144258199031123298446925713673893776562061278884e-44', '6.045436107732280688168131679572978542613012e-571825187053129', '-3.5144258199e-44', 11, 4);
    T('8.5786691738027407891319882954668786712617843068080e-77101853978208', '-7e+80892', '7e+80892', 67, 4);
    T('9.71743151662898987076486931532270818354e+81290260059', '8.65707995575216807379477230201763322208784148233e-6879', '9.717431516629e+81290260059', 14, 5);
    T('1.4381176334617852424376079564302932e+44989231', '5.77489620622848654762645937191533298339700704586161932269e+90873651', '-5.77489620622848654762645937191533298339700704586161932269e+90873651', 86, 0);
    T('2.8868978e-417150300041563', '-5.11053934861e-148412', '5.11053934861e-148412', 25, 6);
    T('5.43640897670649624422702268854034256587965205143e+447544429371', '-1.89434473272475e-7708', '5.43640897670649624422702268854034256587965205143e+447544429371', 54, 3);
    T('-7.860668840258218708104631015e+170021718', '-3.398149449475544153949020926550e-390796984342', '-7.860668840258219e+170021718', 16, 5);
    T('-2.8065041361759370971763218634093630250e+50705634646', '-8.1941171562399644446257167809603776806171011218e+52', '-2.806504136175937097176321863409363025e+50705634646', 89, 0);
    T('6.7743303307e+3031', '2.010076436543451754659370417600233974930372327856110466e-43108765', '6.7743303307e+3031', 51, 0);
    T('8.180704591959525807e-727424638', '5069926143338502519090701079.707927509', '-5.069926143338502519090701079707927509e+27', 76, 5);
    T('-599097290513', '-9.54714141339233967256e+256', '9.54714141339233967256e+256', 78, 2);
    T('9220366991656490487536459855181168805158527967.2168467', '2.16356e-519154646245', '9.2203669916564904875364598551811688051585279672168467e+45', 92, 6);
    T('-7.201168692467551674710885374501054e+995482936', '4.140748131522389755424076508e+88575', '-7.201168692467551674710885374501054e+995482936', 53, 1);
    T('6.79698e-247802', '6.915237739992e+312140520126630', '-6.915237739992e+312140520126630', 78, 4);
    T('-3.7803689496010352574488780914135375681188608e+74513', '6.8879532155175678560635242690e-893682093', '-3.780368949601035e+74513', 16, 1);
    T('-2.332952796173595538121935524860330453e-61', '7.4283586966337795846180815490866238629560340466e+351153023619208', '-7.4283586966337795846180815490866238629560340466000000000000000000000000000000000000000000000001e+351153023619208', 95, 0);
    T('-5.0556e-684875294', '-4.28653667318633517680835278539512372518419448e+4862220', '4.28653667318633517680835278539512372518419448e+4862220', 48, 6);
    T('4.49721977389877014053584232917561254695219639095177706e+30183495541', '-5.11810853374631963560024659301630144e-6752', '4.497219773898770140535842329175612546952196391e+30183495541', 47, 2);
    T('-1.3916710352273664074897324569310729283733281087105596700214e-93056505194562', '6.8504167611e+495494630', '-6.8504167611e+495494630', 14, 5);
    T('5.302121166362064056272572617001e+979193907122746', '-2.140194491577699929228449141443067981078389565976273e+846415563847357', '5.3021212e+979193907122746', 8, 6);
    T('6.9924668387338566448775269070822277746269225461374974e-10', '-434658184946361587962259042903803.4631762512154273891066860', '4.3465818494636158796225904290380346317625191467407e+32', 50, 6);
    T('-3.5482039130675172187871298814463870025659e+49595213', '3.256821106422611035948333955484336546492114511741e+582757', '-3.5482039130675172187871298814463870025659e+49595213', 56, 6);
    T('5.348649288786103753302914318715126671316184036722568627e-877', '7.58670356391287755358724119262350462871668338202361778852e+90', '-7.5867e+90', 5, 1);
    T('5.14137771808085639619464120947469387940895121480261e-626697970958', '-5.55480e-814070108340', '5.14137771808085639619464120947469387940895121480261e-626697970958', 62, 6);
    T('9.4803741936103440789444016070961207068e-5815429880', '8.30115220242191631546735853554381238e-156456732496', '9.4803741936103440789444016070961207067999999999999999999999999999999999999999999999e-5815429880', 83, 3);
    T('-4.7396760819021341068053740420995565882606500256163946532262e+708876546', '2.98604834626e+413380586134', '-2.986048346260000000000000000001e+413380586134', 31, 0);
    T('5.303072009194817690955354e-16062', '-6.717860798882518566473924028509e-611869137090959', '5.30307201e-16062', 10, 2);
    T('-6.92756076700527096283033120404886970271037e+448217920878722', '3.235783429175125859e-5346293909662', '-6.92756076700527096283033120404886970271037e+448217920878722', 82, 2);
    T('-3.3496177521709775632973659539634545400740694923008054651446e+458', '-1.301592208509262765807355e-80244477390078', '-3.34961775217097756329736595396345454007406949230080546e+458', 54, 2);
    T('-7.515790353911815e+9888212857', '-6.34131634016011494433603826054301932826478314243077747246e-55778904920095', '-7.515790353911815e+9888212857', 80, 0);
    T('-4.44884517265646956115992644662684828860623053139624972299e-239607367', '5.46782162670701555786935015392871636e+789', '-5.4678216267070155578693502e+789', 26, 3);
    T('-9.61433331241760091476743108664e+672860346', '-8.08785788503e+83173', '-9.6e+672860346', 2, 5);
    T('5.6447769654572956550570932964950e-15685887', '3.37448525001713858791737379208041356073831723747798e+52', '-3.3744852500171385879173737920804135607e+52', 38, 6);
    T('-6.21747226975546007338639889979e-313300573638880', '-3e+8061672363', '3e+8061672363', 34, 5);
    T('2.568138641796829767150612360e-2562', '9.947588e-917', '-9.947588e-917', 88, 0);
    T('-2.0903379359243451114284055724581421964721805613733471733e-376393696271', '-5.5380016232056043802275089614065532566733e-241', '5.5380016232056043802275089614065532566e-241', 38, 1);
    T('8.4835237767e-52533872', '1.56475e+50960808', '-1.56475e+50960808', 72, 4);
    T('3.7e-4566', '9.74583199568643508464333669867765e-4372', '-9.74583199568643508464333669867765e-4372', 100, 3);
    T('9.8201390644550027912445270231798550e-193477', '5.3096367084318741934351187086630939322e-63298290505', '9.82013906445500279124452702318e-193477', 31, 4);
    T('6.9583406494395463e+68988339452747', '-9.391646484841232080309864562852961856952284152025514285536e-1001', '6.958340649439546300000000000000000000000000000000000000000000001e+68988339452747', 64, 0);
    T('-16930096731853.254', '3.784419e+847', '-3.784419e+847', 38, 6);
    T('5.3e+23', '-1.92e-5346456', '5.3e+23', 5, 1);
    T('-6.9750083450261374030772512925005e+3863002760', '-5.4903520262776673463e+9685', '-6.9750083450261374030772512925004999999999999999999e+3863002760', 50, 1);
    T('-6.837985227238674546038e-3948405019', '-3.233506148534431e-131909429', '3.233506148534431e-131909429', 50, 4);
    T('-4.52400180695615365789463840392520737e-8544364936190', '6.889251381013027874836461428494778480e-632711407183', '-6.88925138101302787483646142849477848e-632711407183', 56, 4);
    T('3.372628849873689240600543765792e+736219', '6.202999203022404868227149769206057343605698e+29639678000', '-6.20299920302240486822714976920605734360569799999999999999999999999999999999999e+29639678000', 78, 2);
    T('-4.8577957e-483', '-5.639e+304898349670', '5.639e+304898349670', 27, 2);
    T('-7.8846637919681616502526512873132252645241066138006029025802e-4443079328152', '-9.27607914718690189305218335522111392e+78', '9.276079e+78', 7, 1);
    T('-7900868292337060579019050551447072104303693.9', '5.0050e-1863130984', '-7.9008682923370605790190505514470721043036939e+42', 91, 5);
    T('4.36388947163333300904062032016145741545156100977316e+92546452999377', '-1.51e-90626519363125', '4.3638895e+92546452999377', 8, 5);
    T('2.081326009662131946e-739192394349859', '2.507987994555839906504622002627937e+617436268', '-2.507987994555839906504622002627937e+617436268', 85, 6);
    T('-9.475608684259540751759630306961793366794581742011251002432e+117', '-8.415156644875583891780e+324068492635', '8.4151566448756e+324068492635', 14, 5);
    T('-9.0536849548e-59494407', '-4.409841503084769776385634481836930897144310562971537897e+5659560644', '4.40984150308476977638563448183694e+5659560644', 33, 2);
    T('-9.41638078420174229075032212680031471265685959923583e+7056', '1.32262173418464973088361197643324476587590561426e-21688829', '-9.4163807842017422907503221268003147126568596e+7056', 45, 6);
    T('2.4948e+1528', '-2.753065701423660806078376904038350091673034045e+838442', '2.753065701423660806078376904038350091673034045e+838442', 82, 4);
    T('-4.37e-7373095224', '-4.69994455812096114643e+94124854', '4.69994455812096114643e+94124854', 71, 2);
    T('-6e-2103580321605', '-7.75467877445023408233934235498055608692813596e+119442214595', '7.75467877445023408233934235498055608692813596e+119442214595', 80, 2);
    T('-1.034438148926877696888889706550790676719532e-46373131643214', '-5.36432162258253676107126989841e-40703', '5.36432162258253676107126989841e-40703', 38, 0);
    T('-3.25534633472741694635298350053768232490659658082150e-774', '1.171023641917309564714798e+59093172', '-1.17102364191730956471479800000001e+59093172', 33, 3);
    T('1.54323412438242888471190104712797656169192758166569761470785e-62364549', '4.5774555131546730409069808109797476059708055878982509149117e-5766876116', '1.543234124382428885e-62364549', 19, 6);
    T('6.375951848835770849527923328551878586419979957726125055056e-14781002736046', '-4.0277987189865347803265050778173037077035e+366384336361', '4.0277987189865347803e+366384336361', 20, 6);
    T('-5.603373811760705810e+3961036', '4.3362127179364050e+696825139275', '-4.3362127179364050000000000000000001e+696825139275', 35, 0);
    T('-7.2e-7022682144', '-7.08202361930396227186699677688938007112177514619790e-654', '7.08202361930396227186699677688938007112177514619e-654', 48, 1);
    T('8.24835254151e+530', '1.311193800499343407986821640795379108448e+5455248383722', '-1.311193800499343407986821640795379108448e+5455248383722', 85, 3);
    T('2.5e-8', '3.7604873833816398e+106', '-3.7604873833816398e+106', 74, 5);
    T('5.826699552344331366992295652216e-2231066370', '5.3687559488428746961818726287676676799471617079622597e-550266300859244', '5.826699552344331366992295652216e-2231066370', 80, 2);
    T('-4.57271656925489998919e+110', '9824809438559771247479363395002713228886248917022.8', '-4.57271657e+110', 9, 0);
    T('-7.0218059637649399407256e+770', '6.250696784990654481329569057996144898313489131e+91905160634', '-6.250696784990654481329569057996144898313489131000000000000000000001e+91905160634', 67, 3);
    T('-9.7384037806100767357147227974836924254662186e+77757490271', '-4.920060142759397130e-809', '-9.7384037806100767357147227974836924254662185999999e+77757490271', 50, 1);
    T('-3.24255510386e+15', '-472133953712385044190524244711082046.27855342445', '4.7213395371238504418728168960722204627855342445e+35', 86, 2);
    T('-8.0973085e-93363', '-2.38493469736390e-54836626093203', '-8.0973084999999999999999999999999999999999999999999999999999999999999999999999999999999e-93363', 86, 1);
    T('-1.17187042632020e+9598', '-9.2383461357912323907552037e+29', '-1.1718704263202e+9598', 80, 4);
    T('-6.96287735093908248494819734685559818339679e+12517731', '-2.2786227147752519201407058e+40775606391', '2.2786227147752519201407058e+40775606391', 81, 2);
    T('-9.5998069318717939372118499276621834426e-22011222951224', '2.2408106010954e+6121400408571', '-2.2408106010954e+6121400408571', 43, 4);
    T('7.1146734676641557664450210797e+78957', '-6.4005849486422540222606180688851403556939546792e-64185', '7.1146734676641557664450210797e+78957', 50, 3);
    T('-9.959193184247750e+103898021211369', '1.839896278271790285792478759358063842338484327406684e+7546', '-9.95919318424775e+103898021211369', 74, 1);
    T('-4.5730152445643655648959e-23', '-6.5932885802321504283146984469816538070e+127', '6.593288580232150428314698446981653806999999999999999999999999999999999999999e+127', 76, 3);
    T('-5.03832514273869949e-35648799', '-9.48261341646350697479044236436542616181114344631313492720e+508301332769855', '9.482613416463506974790442364365426161811143446313134927199999999999999999999999999999999999e+508301332769855', 91, 3);
    T('-7.211143185253984520223272662716559318e-55259059', '8.265027599038356e-7258', '-8.265027599038356e-7258', 26, 6);
    T('-4.4733489707611901087e-890994329762', '7.00177493873730366496111643955190013227730278e+76751044101', '-7.00177493873730366496111643955190013227730278e+76751044101', 54, 5);
    T('-6.95190484507275670561521296510936e-437151788655', '-3.172684366811e+989', '3.1726843668109999999999999999999999999999999999999999999999999999999999e+989', 71, 3);
    T('5.31e+11', '-3.314915244857270e-66844027718601', '5.31e+11', 42, 5);
    T('7.288382635865773672905246099187431887296448e+1142531', '-5e+2', '7.288382635865773672905246099187431887296448e+1142531', 44, 4);
    T('-7.80840729997806645768241733018628599263122e+794833308', '9.141818e+1290911612324', '-9.141818e+1290911612324', 10, 5);
    T('-8.46755348182762616240928589842245380660613194623882e+952316614763744', '6.255769846856441700968642660258891631974301879523802e+74370259040', '-8.46755348182762616240928589842245380660613194623882e+952316614763744', 78, 2);
    T('-9.6425616456980183778393855834073322175298509599861e+34451845781144', '-4.9511743160241130641147173769281388345983245417458195e+61', '-9.6425616456980183778393855834073322175298509599861e+34451845781144', 95, 5);
    T('2.6782383025501e+672328306', '-9.469506780031115032204979251236534656458209e-44', '2.6782383025501000001e+672328306', 20, 2);
    T('2.294658511259352807168823235705817755441498634e+9495553', '6.45842344681e+82', '2.294658511259352807168823235705817755441498634e+9495553', 50, 2);
    T('-980985004672011557554287730669.025417', '9.4578305849696524e-250333', '-9.80985004672011557554287730669025417e+29', 96, 1);
    T('2.358791624920187905958530408810322120338822e-7556770', '-2.0151759635097351330019180125e-5528', '2.015175963509735133001918e-5528', 25, 4);
    T('-5.8562253e+71041', '8.3197303521581210e+1120787', '-8.319730352158121e+1120787', 28, 1);
    T('-7.25531823494452057084581181656121249961183114182359e-63770556586', '-7.7703100419018467090598459e+7723490141', '7.7703100419019e+7723490141', 14, 2);
    T('-5.910551859342587512e-998383307952172', '-2.8118506832459225988839322050930658021408323017966e+948085', '2.8118506832459225988839322050930658021408323017966e+948085', 82, 2);
    T('-1.36881545175602088396031007580073676379472211594012e+72', '0.0009499', '-1.36e+72', 3, 2);
    T('-3.204537777741158259539085169e-5580151998603', '4.392396984985575031901558425223222889878924699e+319058162', '-4.3923969849855750319015584252232228898789246e+319058162', 44, 1);
    T('-9.50360983153414532e-860851645', '4.9e-596144914692488', '-9.50360983153414532e-860851645', 26, 1);
    T('-9.57352e+932428', '9.87264177306821823601289996144456078499160407922300875255e-449938', '-9.57352e+932428', 40, 6);
    T('4.8484222950341234e+28510294227359', '-2.6954585020787789835587e+2248', '4.8484222950341234e+28510294227359', 95, 6);
    T('9.5671425376722911271085792254687431897559633137555494545081e+709', '4.465777175583245006566602074411e+78256', '-4.465777175583245006566602074411e+78256', 64, 4);
    T('-2.09159025875681444870267547e-77716152', '-1.09964895997622e-52726', '1.09964895997622e-52726', 68, 4);
    T('-1.98375269080179694362603e-6945081621', '8.197978266501717969675606791313520553610779254e+9513', '-8.19797826650171796967560679131352055e+9513', 36, 4);
    T('7.9531515922545408886879397485076003790098e+6162599520', '-3.0324487514448001412790002001091096812744338604e-39730', '7.95315159225454088868793974850760037900980000000000000000000000000000000000000000000000000000001e+6162599520', 96, 0);
    T('-4.28767155123367993153058e+805624', '7.97972189395783493454178471675621269249434619347916344e-565752163223993', '-4.28767155123367993153058e+805624', 47, 4);
    T('-9.5150485507276334153817888164807919617762026e+4816074662', '-2.90165851463751167039432934091e-2814560692', '-9.5150485507276334153817888e+4816074662', 26, 5);
    T('9.6058012750771754697971403178873097820730257905841e+76719844936174', '-6.3090461332265804944e-619560', '9.6058012750771754697971403178873097820730257905841000000000000000000000001e+76719844936174', 74, 0);
    T('5.7505722076565879064215478968785e+32061', '1.23795770114843319535384321790e+577518455163', '-1.2379577011484331953538432179e+577518455163', 100, 6);
    T('9.359132572927974408999250571e-122130616473580', '-8.37283359e+6709400480', '8.372833590000000000000000000000000000000001e+6709400480', 43, 2);
    T('-2.249656812542890243085913991196165759220252101829393044e-293955', '9.7326128156879359118920e+822922619', '-9.7326129e+822922619', 8, 0);
    T('7.675724906102487875031643521e-88611494', '8.418664465664547626064995e-6596', '-8.4186644656645476260649949999e-6596', 29, 2);
    T('5.88197973427043581527076405380e-77193704', '1.99385912620736365750419174011456e+69029463187916', '-1.9938591262073636575041917401145599999999999999999999999999999999999999999999999999e+69029463187916', 83, 2);
    T('3.993976060267667557432942156123928483852891574e+6981581', '-3.659377e+991817035300649', '3.659377e+991817035300649', 66, 6);
    T('9.38605558785495831548027464636937127311590354e+5544', '-1.12597710897220171e+257694', '1.125977108972e+257694', 13, 3);
    T('843781893621245559988123844809884579035225.067862061125', '2.4941438526399036007866032168848e-48', '8.437818936212455599881238448098845790352250678620611e+41', 52, 6);
    T('-4.65751318269940488358246777474810291213e+628959224', '-3.1106885489510760031226178003983050221e-2435', '-4.65751318269940488358e+628959224', 21, 1);
    T('6.265790823850035786911898705582772994306683495786614619700e-2053131918', '5.8340983322227181e+73706825183', '-5.8340983322227181e+73706825183', 80, 3);
    T('-607891394248664208280033785273031712.506134126456429502995247', '-8.590822563655145355630082119590315436285662743248355e-78526448', '-6.078914e+35', 7, 6);
    T('1.601146287413141842182107e+692', '8.153866963417053814259017e-52460721499', '1.601146287413141842182106999999999999999999999999999999999999999999e+692', 67, 1);
    T('-6.098339218609107903088201096570043159639123836026240e-7828', '-5.525989044620e-5721576', '-6.098339e-7828', 7, 1);
    T('3.3209750546e+1007', '9.18171557169038944316276397956247973515444590527e-6896890398705', '3.320975054599999999999999999999999999999999999999999999999999999999999999999999999999999999e+1007', 91, 3);
    T('3e+8', '7.5850636873136104673589733687409795711239217717e-376', '2.9999999999999999999999999999999999999999999999999999999999999999999999999999e+8', 77, 1);
    T('6.9435833e+2158', '3.662435104803365531462940153415028410544253563781128851028e+156', '6.9435833e+2158', 68, 4);
    T('8.6002162e+936892', '-8.4839008845441664025576334706e-77819019157', '8.6002e+936892', 5, 3);
    T('9.4024171e-39992160121680', '-7.985043e-5623', '7.98505e-5623', 6, 2);
    T('7.9471261e+627956846786814', '9.5476898083e+17213297485599', '7.9471261e+627956846786814', 79, 6);
    T('-1.37845449428853e-3409490', '-3.7222018000359680118094888367889555807887087601e+7196608', '3.72220180003596801180948883678895558078870876009999999999999e+7196608', 60, 1);
    T('-2.028080079118277261474234787961e+514220786765223', '-2643822571150488078956456972060869.521850121', '-2.028080079118277261474234787960999999999999999999999999e+514220786765223', 55, 2);
    T('-1351914590157171066336761183327355966.63', '-6.24264058693473960087e+96527764', '6.24264058693473960087e+96527764', 22, 0);
    T('9.9074129811754231066737888e-62', '-3.019688e-7139390498039', '9.9074129811754231066737888e-62', 84, 5);
    T('-3.2209421262870e-8824196842987', '-8.62914728408471846713883587146e-14855033', '8.62914728408471846713883587146e-14855033', 73, 2);
    T('4.198527962358613290825443573904087692259182418207696548e+4603346630624', '-2.9349995430608517743896206e-949372', '4.1985279623586132908254435739040876922591824182077e+4603346630624', 51, 5);
    T('-8.406278090050121603e+368332', '1.397564747844904403354990561934e-4512233', '-8.4062780900502e+368332', 14, 3);
    T('5.28878168824e-9886', '6.52383812910785650522074843102912e-4801159760', '5.288781688e-9886', 10, 1);
    T('-2.22048864675e-30634166', '3.042972229468681375802951937907e-3625494', '-3.042972229468681375802951937907e-3625494', 83, 1);
    T('-3.6143174457499413162e+26', '5.13519981164596286408945865341470312449201e-2343', '-3.6143174457499413162e+26', 80, 5);
    T('7.438511812463804782119822363840410168e+974', '9.7310893419845785816939465727218462615318284e-2930849', '7.438511812463804782119822363840410168e+974', 95, 2);
    T('-9.2211600932660362141870084939399e-766', '9.1e-17921691379', '-9.22116e-766', 7, 4);
    T('6.4729480400e-583833587633', '7.69842355104789130816003221892e-29922446', '-7.69842355104789130816003221892e-29922446', 99, 6);
    T('8.12949274236239050381759202749e-1164490286', '-3.464391610359107920082912376026409473400401504669127e+22491', '3.464391610359107920082912376026409473400401504669127e+22491', 72, 1);
    T('-6.9716895769e-81945045', '-1.56319e+6850641225', '1.56319e+6850641225', 42, 0);
    T('5.85328375294101282136369e+52130424500369', '-5.7832e+979765', '5.85328375294101282136369e+52130424500369', 71, 5);
    T('7.8012239125624817105978136701304636292697042308337713e-5156510778', '-7.971357188977177711354e-55172', '7.971357188977177711354e-55172', 85, 3);
    T('-7.240110590898109494695821642e-80611', '-9.453863592e+1005', '9.453863592e+1005', 71, 2);
    T('-1.431713339288946218405e-3289995309', '-1.5e-6733602', '1.499999999999999999999999999999999999999999999999999e-6733602', 52, 3);
    T('3.963189051762232792739348e+724806877', '5.1987056865e-94467336', '3.963189051762232792739348e+724806877', 90, 6);
    T('-2.81025211496e-33275951344547', '-2.23101172452926e-47187957010305', '-2.81025211496e-33275951344547', 37, 6);
    T('2.6768702547696673612880597491800105277623079385305735857454e+581197428513534', '3.3795546106099296503452631786e+99995', '2.6768702547696673612880597491800105277623079385305735857454e+581197428513534', 86, 5);
    T('-9.20179130281e+61523', '-2.7433692104694717770253734205350309865195493e+382341193914', '2.743369210469471777025e+382341193914', 22, 5);
    T('-1.53336441039262516992587804432522823570e-30886641147', '-6.992699995073198682147899552493251833423756e+85', '6.992699995073198682147899552493251833423755999999999999999999999999999999999999999999e+85', 85, 1);
    T('4.4593366560875062243240239875082496404767672e-5266261', '-3.434095356345511e-32468151', '4.4593366560875062243240239875082496404767672e-5266261', 56, 4);
    T('2.282142000733111115289564143486552246536380695043069e+757125021048', '-6.1792049811391767089455403634789432090e+38494729517755', '6.179204981139176708945540363478943209e+38494729517755', 51, 5);
    T('-9.54639678786638324186e-514', '3.84312917505500978554332e-698994', '-9.54639678786638324186e-514', 41, 2);
    T('-7.474058e-76154786746239', '3.25405266917406170075258e-16309628', '-3.25405266917406170075258e-16309628', 58, 4);
    T('3.1791188080785825134433870427663142014100e-962587856746', '-370242162557.09508', '3.70242162557095080000000000000000000000000000000000000000000000000001e+11', 69, 2);
    T('0.002429740755684907', '-9.006324433729238e+5238', '9.006324433729238e+5238', 100, 3);
    T('-1.984978754618809265194265760095530697990221e+1797511356', '6.736269178328010049272137118508270123677e+421541', '-1.9849787546188092651942658e+1797511356', 26, 0);
    T('-8.666335114217243342591150496568373502552206e-7044', '-3.16857642084395121e+455444704902', '3.1685764e+455444704902', 8, 1);
    T('-4.572296047099141131934398179078186973374e+508306832', '-5.437867746677750353275985406526809223198026625e+5163579', '-4.57229604709914113193439817907819e+508306832', 33, 6);
    T('5.4e+42183', '2.69405455511374375824387586249106e+29747', '5.4e+42183', 32, 5);
    T('-5.477053914595202722724159345167071e-95107014112112', '2.76962e+178', '-2.76962e+178', 32, 4);
    T('1.07265032e-9158894', '-7.521209719529207639604332098973195951066103285903601e+927', '7.52120971952920763960433209897319596e+927', 36, 2);
    T('-13574612084074106348344192569456880926131670892.72290', '-4.8998616070041581e+3801', '4.8998616070041581e+3801', 95, 2);
    T('-1.470355203590027803514e-186956235278', '5.89321452596258905062652329043981793e-926049', '-5.893214525962589050626524e-926049', 25, 3);
    T('5.16035836143777e+80726', '3.316382312945519414e-63566', '5.16035836143777e+80726', 22, 0);
    T('-6.97465340161859096653214697640050529207e+35097', '6.569470493795349376505270342e+72939651', '-6.569470493795349376505270342000000000001e+72939651', 40, 0);
    T('-7.7333247810445988909300569516938196e+143', '-7.33974396314668876493562758782234732041e+22961', '7.339743963146688764935628e+22961', 25, 5);
    T('-4.8994545591442111131384119548070041121387829755268080e-5554258', '5.48700014349878e-72', '-5.4870001434987800000000000000000000000000000000000000001e-72', 56, 0);
    T('7.52999689795842535837969e-7763990932', '-1.7378808419692663553989937646154335558209580555e-48204709314', '7.52999689795842535837969e-7763990932', 54, 6);
    T('-2.3465529544849576200135541124627916986858789778220467304e+715234555085985', '4.0846735622654999520366008306204786998567456135429189072830e+8650179450655', '-2.346552954484957620013554112462791698685878977822046730400000000000000000000000001e+715234555085985', 82, 0);
    T('-385866509296739236716.64', '-2.632385403791720102642493662095924e-940182343811628', '-3.858665092967392367166399999999999999e+20', 37, 1);
    T('-5.154439741223761815873e+54806', '-9.58507238551089e+9277738', '9.5850723856e+9277738', 11, 0);
    T('-59502155602911.2221', '9.7994733867031193681499305275069274642517e-710598950', '-5.95021556029112221e+13', 78, 1);
    T('4.423454630947456508e+6769', '835331025226671433612915326764175562712343975604.1', '4.423454630947456508e+6769', 56, 0);
    T('-7.3582106040794331537231652150411912e-744600196', '-7.705959833520065340698432e+690473', '7.705959833520065340698431999999999999999999999999999999999999999999999999e+690473', 73, 1);
    T('-7.99521695338638679468861718755343357383232326754e-6234154688', '2.716105739327369586323809e-1843975833', '-2.716105739327369586323809e-1843975833', 47, 2);
    T('-1.47566670344095162826591689228858536660384258196730788990e-54', '3.330205919e+420873', '-3.330205919e+420873', 29, 2);
    T('-9.477643331441604777777775180361297e+37', '7.45868331474835885465885911347919241722832120684001e+77101', '-7.45868331474835885465885911347919241722832120684001e+77101', 99, 4);
    T('3.487318355749921337600398479931378e+897085603482', '9.05872321880202042820864e+40645801312175', '-9.05872321880202042820864e+40645801312175', 80, 6);
    T('-5.1264368939729243082330184712766418022495159791602587e-5533280724140', '6.3045033070911551e-151207', '-6.3045033070911551e-151207', 96, 4);
    T('4.5287948754165298757411729405817280989880373765429984231e-205', '-3.424269004441616139213303094182447438327311e-766952726176', '4.52879487541652987574117294058172809898803737655e-205', 48, 0);
    T('-1.73060e+738171395818', '-7.285442959205201014e+87', '-1.7306e+738171395818', 57, 6);
    T('-8.2037e-795029', '-9.0977296894756056233406473e+474057', '9.0977296894756056233406473e+474057', 49, 6);
    T('5.035975649816445851e+71002', '2.32539581110574273328090364775733149401166e+987315286', '-2.32539581110574273328090364775733149e+987315286', 36, 1);
    T('-3.3197255585144400e-25611', '-8.17883784188647618770681545113498814587655713340e-462869296354', '-3.31972555851444e-25611', 60, 5);
    T('6.670756928635007467824e+88371438295434', '-4e+632345', '6.6707569286350074679e+88371438295434', 20, 2);
    T('6.979417578116255748350954e+2687', '8.335099e+7737475708844', '-8.335099e+7737475708844', 70, 3);
    T('-5.34737114916515e+5290566', '-4.969224824204038853062770168669587331745516704600027e-7220', '-5.34737114916515e+5290566', 47, 4);
    T('-2.230515009940380239597979524145953254740282e-3572850497945', '-5.118637403705036612965742905882749460919422e+391432', '5.118637403705036612965742905882749460919422e+391432', 84, 4);
    T('-6.02732577466010e-68042277931', '6.57958900590572445659832476365288897010710691096356952629649e-647', '-6.5795890059057244566e-647', 20, 5);
    T('-7.21219441903596981269e+767', '6.0693811221560027291725228664996424576165e+25642830397468', '-6.0693811221560027291725228664996424576165e+25642830397468', 82, 5);
    T('-3.316518688272327754629722255935424e+84610', '-6.57e-3024', '-3.3165186882723277546297222559354239999e+84610', 38, 1);
    T('-1.6901285182278215859210700460789375310e+44183252409296', '-8612744214.1', '-1.690128518227821585921070046078937531e+44183252409296', 90, 0);
    T('3.76014438002857033e-236458805', '5.298779378801020439758290957031117113250126173784728e+994283905843482', '-5.2987793788010204397582909570311171132501261737847279999999999999999999999e+994283905843482', 74, 1);
    T('-6.31595623e-285939163', '7.8979761457638207843340823937770651092221726811528677e-93765674797', '-6.31595623e-285939163', 65, 6);
    T('8.238460328832116754036364e-1783874', '4.7898180491853978262281868776507146e+12588489029926', '-4.8e+12588489029926', 2, 0);
    T('6.096446637717700782655299124569e-454', '7.375532308118017395e-44997108899680', '6.09644663772e-454', 12, 2);
    T('3.31385503237233460552030364132885360178099520420836826e+9013', '-1.083086336653158972431261947890109e-772', '3.313855032372334605520303641328853601780995204208368260000000000000000000000000000000000001e+9013', 91, 2);
    T('7.73718560561121630458583173378528304579658415470e+1049285', '-3.20189197226849955503288748536138071875978546145646866e-720086560', '7.7371856056112163045858e+1049285', 23, 6);
    T('-3.8860173739943564507150510205230461866640714517772e+5756721025', '-9.3539848059343754817e+311125339715657', '9.3539848059343754817e+311125339715657', 65, 4);
    T('-1.4749048405526842137e+29840526', '6.861152521210425375285105171492381816004599e+9822', '-1.47490484055268421370000000000000000000000001e+29840526', 45, 3);
    T('5.46385103264e-731909673242', '8.6325428803497469106310276026678217276e+41', '-8.63254288034974691063102760266782172759999999999999999999e+41', 57, 1);
    T('5.5648150857719594904614694426666409441641160196057e+3481', '2.7388173762617676574826397783686117755202455e-702553579757', '5.5648150857719594904614694426666409441641160196057e+3481', 53, 6);
    T('5.63443161e-78215932912126', '8.717670542567533869907530511863466227890591603995355e+427789845974', '-8.7176705425675338e+427789845974', 17, 1);
    T('-1.608551832332311957561614400344476437948830042075148605e+345931305210', '2.780818429467483647436272e+46451071', '-1.608551832332311957561614400344476437948830042075148605e+345931305210', 89, 5);
    T('-1.76139161249653e+174780', '6.87940928919397954970530256321531796220877360e-6810', '-1.76139161249653e+174780', 31, 2);
    T('1.1270293355974235e+1074516606', '-8.75e-12370719', '1.1270293355974235e+1074516606', 44, 4);
    T('-8.00256454913863077097334239e+96', '3.234088212121e-93795755257', '-8.0025645491386307709733424e+96', 26, 0);
    T('-4.3226e+7059455100', '6.23610514828346944260808511202349099856112120242889e+5149100847', '-4.3226e+7059455100', 65, 5);
    T('-7.57e+72702524725591', '4.76375868973649527543327548578501517244062079088e+2071789210', '-7.57e+72702524725591', 87, 1);
    T('2.2150945563073', '-7.978703050787884668570274954691161318400715184190718e+223500334382', '7.978703050787884668570274954691161318400715184191e+223500334382', 49, 5);
    T('1.357971867515843012682616313955654270950631198446e+87687', '-4.002488501566500904797929749002844802e-5877796', '1.35797186751584301268261631395565427095063119844600001e+87687', 54, 2);
    T('8.687765775977557203238e-41103387', '-7.995e+98', '8e+98', 3, 2);
    T('-701395525126618476213516495401952.6453888', '2.498513526682542168620680286734850664655e-4992170909825', '-7.013955251266184762135164954019526453888e+32', 90, 1);
    T('1.787118216756119647794470e+7156', '5.673113527e+18', '1.78711821675611964779447e+7156', 28, 6);
    T('2.998898560839e+412240527', '1.467283e-618', '2.998898560839e+412240527', 41, 2);
    T('-1.0631631886398191662738880200314741272789e+43', '7.6961407485340542310884072919749553040360186926829777843861e+383919248', '-7.6961407485340542310884072919749553040360187e+383919248', 44, 0);
    T('-8.9058157101692944e-86441893649', '0.04846958272942233944866781709617284000123071026796002', '-4.846958272942233944866782e-2', 25, 3);
    T('-1.46086807170020748743941970053961e-8336533', '-1.4474366332925553821394635275793145412945531362106325e-190615129403', '-1.460868071700207487439419700539609999999999999999999999999999999999999999999e-8336533', 76, 1);
    T('5.249e-9154609783542', '-1.52558089891912361631e+786331356466', '1.52558089891e+786331356466', 12, 3);
    T('6.183688149792336320329166798294620e+17875', '2.69630207205485925280147790525381e-109273952953853', '6.1836881497923363203291667982946e+17875', 32, 6);
    T('54501049963596175', '-3.354661411356672760631868664294491625886521783157523e-81727679585', '5.4501049963596175000000000000000000000000000000000000000000000000000000000000000000000001e+16', 89, 2);
    T('3.25018086582138829418025014445225957840200301718e-447020200311', '5.8e+489', '-5.8e+489', 53, 0);
    T('6.594e-108997634743', '9.0411380734570062979458e+639176132472', '-9.0411380734570062979458e+639176132472', 34, 4);
    T('-8.82686292564019437507276386934348511207613881590641105714255e+4014907643', '-6.5065273143e+85537382235', '6.506527314299999999999999999999999999999999e+85537382235', 43, 3);
    T('-3.63368256138412251593317045678e+672', '4.35889831060251663860e-800', '-3.63368256138412251593317045678e+672', 34, 6);
    T('-1.0415523146671821181451476147789315212550929303e-2837192', '8.281438905056294e-779428', '-8.281438905056294e-779428', 89, 2);
    T('-8.37745912536243305636069068664331e+3573699490', '1.6359050947040500e+4521591136', '-1.63590509470405000001e+4521591136', 21, 0);
    T('-6.7700263707565773702e+2051314765259', '-7.214384288805828624155303389657162260343452086488226836e+780192', '-6.77002637075657737019999999999999999999999999999999999e+2051314765259', 54, 1);
    T('8.435020584861e-9660324', '-3.0070267810425544887620153528844026e+71', '3.007026781042554e+71', 16, 5);
    T('-5.7831256653708596634547614829848319051880995542505494e-94728082189924', '8.6061391931296408901707e-87953', '-8.6061391931e-87953', 11, 1);
    T('-5.254920025798400189088288775836716504765900989718993569e-26268359535', '4.6e+15331355533645', '-4.6000000000000000000000001e+15331355533645', 26, 3);
    T('1.467214e+107', '4.212871e-996528509392387', '1.467214e+107', 8, 2);
    T('6.680861406884e-29250', '-1.78839050747955883e-143049527459', '6.68086140688400000000000000000000000000000000000000000000000000000000000000000000000000000001e-29250', 93, 0);
    T('8.5086035491912211945054007642032177725616e-5020178', '-7.8574322182844696188e+80906131', '7.8574322182844696188e+80906131', 23, 1);
    T('6.46447710840940659199867826e+8840775', '-7.5700e-3378360131890', '6.46447710840940659199867826e+8840775', 66, 4);
    T('8.943460078649e+181523423', '-6.736860036623e-97689006008448', '8.943460078649e+181523423', 100, 3);
    T('8.934024818903640321828420159865e-931019035399858', '4.21843e+6458', '-4.21843e+6458', 47, 4);
    T('-1.346502435899e+163', '1.750420363643096489e-9289639', '-1.346502435899e+163', 72, 1);
    T('8.5261e-4358831977', '-3.6970680813962023247206264e-71569', '3.697068081396202324720626400000000000000000000000000001e-71569', 55, 2);
    T('6.5928266734214050246876569119199516478590740200042460547798e+60', '-7.749947030e+384714512790', '7.749947e+384714512790', 7, 1);
    T('-9.3778866990743502948257415558199808280867460421524428301665e-62330', '9.9803323907287e+9988708830985', '-9.9803323907287e+9988708830985', 71, 4);
    T('-9.735566827046440688603833625385804384810393070405e-79747608042', '-1.97e+376692568', '1.97e+376692568', 27, 5);
    T('-2.653180399e-6084282832', '5.055246413556e+6861695975', '-5.055246413556e+6861695975', 18, 4);
    T('1.7302706306681561923397832412198e-5502077046961', '-4.56047115904380255293930562118026099e+660530368717857', '4.56047115904380255293930562118026099e+660530368717857', 46, 5);
    T('-4.591023017728542353299496941090037334302e-85287', '-6.4231402529826e-86323394319111', '-4.591023017728542353299496941090037334302e-85287', 40, 0);
    T('5.40166530778e-208994599733', '-1.4527970138825444519160233468938e+55122', '1.4527970138825444519160233468938e+55122', 49, 1);
    T('6.025e-8482712990937', '-2.89292562958587e-686945525827', '2.89292562958587e-686945525827', 29, 6);
    T('-3.08070972394441716369537802750840451240953071306708e+91034050', '-8.4229e-180', '-3.08070972394441716369537802750840451240953071306708e+91034050', 94, 3);
    T('-9.5554914097205e+85', '4.8227616653795620095443045555734977791670e-7457956586042', '-9.555491e+85', 7, 4);
    T('-2.74698697790e+617', '-6.5067614099467814e-38221075', '-2.74698697789999999999999999999999999999999e+617', 42, 1);
    T('-2.7853449920740778591e+414245990854808', '5.087605806510e-47896', '-2.7853449920740778591e+414245990854808', 84, 6);
    T('8.6194374526645075131379584054115e+852', '-9.43519442261464651e-383152104508218', '8.6194374526645075131379584054115e+852', 49, 6);
    T('6.36986529e-748995960663', '2.250710711e+10', '-2.25071071099999999999999999999999999e+10', 36, 2);
    T('-6.080691e+23440860543776', '6.269623278760728208400e+967', '-6.080691e+23440860543776', 39, 6);
    T('-7.388607882693247109282856882993391394020737028665692660515e+57493754100', '-79881014251480134048661299621239989', '-7.388607882693247109282856882993391394020737028665692660515e+57493754100', 82, 6);
    T('-3.96015744134061140286e+5865', '6.85253110113978704828e+2893938822218', '-6.85253110113978704828e+2893938822218', 27, 1);
    T('8.840453295798231236e-242619256744082', '2.790857066938185154616691538e-90461', '-2.7908570669381852e-90461', 17, 5);
    T('-3.187010306043e+6815177834777', '7.60479215396569519e-8008324749647', '-3.187010306043e+6815177834777', 62, 4);
    T('-844524.0005717', '-7.177705445765783211e-9582974830047', '-8.44524000571699999999999999999999999999999999999999999999999999999999999999999999999999999e+5', 90, 2);
    T('-7.266401512740563e+584976', '9.638010697500089e-210654786678', '-7.266401512740563e+584976', 73, 4);
    T('7.850838063802048045289034927146154850221211787181e+536461656650956', '-3.68839630269212705e+804901283', '7.850838063802048045289034927146154850221211787181e+536461656650956', 54, 6);
    T('-2.8e+362554', '-3.9186731811558159858268890687723710334e-7945372404', '-2.8e+362554', 54, 6);
    T('-3.8109e-312', '-5.915889331177153248459246387304975455063014057295560e+1046869', '5.91588933117715324845924638730497545506301405729556e+1046869', 73, 5);
    T('6.035864250928372547886410427457867681377816844345473e-4994226760', '-4.991102528330458811791621873040689847355376220e-20', '4.99110252833045881179162187304068984735537622e-20', 47, 6);
    T('-64803564013605286.0573500209249233', '8.76853989481672195633903369161553834405236731693907331e-765568855164288', '-6.480356e+16', 7, 5);
    T('6.429257574936943281e-456484044764', '-8.160788e-3857104020', '8.1607880000000000000000000000000000000000000000000000000000000000000000000000000000000000001e-3857104020', 92, 0);
    T('-9.39280632796493592760561964e+8842426', '1.713267541116e+970806', '-9.39280632796493592760561964e+8842426', 87, 5);
    T('-5.778450150789680950289462418e-499276', '-6.14904627781541636402115388075091768115543879055356802440539e-786689010026992', '-5.778450150789680950289462418e-499276', 87, 4);
    T('-5.23640e-9716111802859', '-9.591139438274659514739381481506385428306367654140589e-438764', '9.5911394382746596e-438764', 17, 0);
    T('-9.8273184465887487081440828765197770591576970376e-82453538724920', '6.16545810164239386287341047815786153676484610086e-791', '-6.16545810164239386287341047815786153676484610086e-791', 88, 5);
    T('-4.2061439259116712230596682403e+3027646', '4.12818361917885081498634185e-86652', '-4.2061439259116712230596682403e+3027646', 96, 5);
    T('9.9262010856148964377360370493797227879e-6401724', '7.4777807521593058014285459160900493672946943385e+717194356756035', '-7.47778075215930580142854591609004936729469433849999999999999999999999999999999999999e+717194356756035', 84, 2);
    T('5.46959943488e-629935516773565', '-8.83726e+430859529816188', '8.83726e+430859529816188', 98, 5);
    T('4.571138445809364041995201850668420489083e+281740911563', '-9.61728406455039299285563186392964952710148717e-7', '4.57113844580936404199520185066842e+281740911563', 34, 5);
    T('-7.92500658634742937056770363e-52723656', '-4.17241e+8', '4.17241e+8', 70, 0);
    T('8.1627678449231e+9384001637', '8.067e+26933', '8.1627678449231e+9384001637', 33, 5);
    T('-8.41849740767912860777882054248274977710191302932459e-14043157102', '5.63657956711912503058142202103922236778041932452736679838e+732489', '-5.636579567119126e+732489', 16, 3);
    T('2.5526692031360669038318547887743329875667138666938048338e-307136308842', '4.126938208974937256962683e+8754811284905', '-4.126938208974937256962683e+8754811284905', 44, 5);
    T('-5.936195803491524406929331e+78990431', '-2.856555614563529868036715628098e+24876', '-5.9361958034915244069293309999999999999999999999999999999999999999999e+78990431', 68, 2);
    T('-4.23120577478900780041746525419291403938312849572818844065e-25', '3.641638135090387930688728009268025309689361578398275e+57', '-3.641638135090387930688728009268025309689361578398275e+57', 53, 5);
    T('-9.92649356062e-704106273452999', '2.155171195793257271558350e-8649', '-2.15517119579325727155835e-8649', 32, 5);
    T('1.429282475528e-329', '-5.5350995168910543925e-28539579', '1.429282475528000000000000000000000000000000000000000000000001e-329', 61, 2);
    T('-5.758e+7170209', '7.665373406378987695441099098682034234648077228e+51', '-5.758e+7170209', 19, 1);
    T('9.7933528409e+55300', '-7.91573541593318550859712827329061742710420330e-352747258891', '9.7933528409e+55300', 39, 6);
    T('-7.82878961027835283761136357210620407613333e+38226069', '-5.868836236205936435393473901779601102040128858783e+52117723782', '5.868836236205936435393473901779601102040128858783e+52117723782', 65, 5);
    T('1.264578379701488854127475155649003739849328772890398098e-542479955171610', '5.969107443344687723952249522534745397086327693382785e-2954107', '-5.969107443344687723952249522534745397086327693382785e-2954107', 57, 5);
    T('-4.04593e+5594636227471', '-2.7729454912277140851990042095e-80672829', '-4.04593e+5594636227471', 59, 0);
    T('2.351617728831e+9278782', '-1723595523.6393', '2.35161772883100000000000000000000000000000000000000001e+9278782', 54, 0);
    T('-5.6210197e-49628295', '-3.4771838922324802307659851354278990e+12345073969394', '3.477183892232480230765985135427899e+12345073969394', 76, 2);
    T('6.64e+36279', '-3.74046795111900445618893872164456807389816052043096424e+1938386100', '3.740467951119004456188e+1938386100', 22, 1);
    T('4.4556727018110581546155976114e+5251', '1.330662e+9830', '-1.330662e+9830', 69, 5);
    T('6.81797571580346778964739730585722505893830785135618743641e+51397', '-4.42513906314805237551724493164161306531e-943434826', '6.81797571580346778964739730585722505893830785135618743641e+51397', 63, 6);
    T('8.5649361293629410996030155023480474149276473338996603e+57', '7.72784981908e-3192', '8.564936129362941099603015502348047414927647333899660299999999999999999999999999999999999e+57', 88, 1);
    T('-1.231147208331715311073714534266243063607922776e+7153327', '5.643710190818854296761e+101', '-1.231147208331715311073714534266243063607922776e+7153327', 98, 2);
    T('1.19944888633943446749352284e+983', '-2.16841335245583987054293699096063119632e-42', '1.19944888633943446749352284e+983', 29, 3);
    T('4.34729495181e-499312820', '-6.88e+389614618322', '6.88e+389614618322', 51, 3);
    T('7.071201695484156e+285365547', '6.96482388200532841e+101', '7.071201695484156e+285365547', 25, 6);
    T('166090507778540346585946.21', '9.72782113175789246997192e+30', '-9.72782096566738469143157341405379e+30', 79, 5);
    T('4.02e+508571700', '7.399635661705481443205336922e-420018360', '4.02e+508571700', 20, 6);
    T('9.0397752736836780e+960126', '-1.2742461752539847595143730962409349682379049e+526375095309', '1.2742461752539847595143730962409349682379049e+526375095309', 87, 1);
    T('5.575093237797609613007651727701141125120080e-417311337198', '-6.8452404032385502855411200523607105567443e-7699', '6.8452404e-7699', 8, 1);
    T('-4.5263236329308149108e+87742962308', '-4.46075206895e+230021277', '-4.5263236329308149108e+87742962308', 51, 6);
    T('2.59938119887459625e-349', '-2.13087687220749580861878140239121172647448237e+36583', '2.130876872207495808618781402391212e+36583', 34, 2);
    T('1.636402e+69', '2.93043341737305494947054336901e-49570975391', '1.636402e+69', 92, 4);
    T('-5.846579003549e-751154', '6.258178883825919644282916e+26', '-6.258178883825919644282916e+26', 89, 4);
    T('-3.90892560088704136088992226788e-7651665', '5.354591854002558e+5165416', '-5.354591854002558e+5165416', 94, 4);
    T('4.0477945419658346081726020547214481985460304124644e+51', '-3.8890218796010071146225935898116008e+46468136541', '3.889e+46468136541', 5, 6);
    T('1.518e+1809385761212', '8.05401410653061887963402134264796945997772517205874e+4420', '1.518e+1809385761212', 54, 2);
    T('5.241825973869948226824176572149693988658459068954e+2842708361', '6.720928721122643413241871489061e+888012891821777', '-6.72092872112264341324187148906099999999999999999999999999999999999999999999999999999999999999999999e+888012891821777', 99, 1);
    T('-7.41208401090711041314030841004490e+286952412078413', '1.19420636208260017826199595266049971312630e-817613', '-7.412084010907110413140308410044900000000000000000000000000000000000000000000000000000000000001e+286952412078413', 94, 3);
    T('8.109249406151155724681782103759846280968598611798768935716e-4272527841089', '5.323808834662558514896773417365726e-514227', '-5.3238088346625585148967734173657259999999999999999999999999999999999999999999999999999e-514227', 86, 2);
    T('-2.938660408383200902208256827974357565935817775928e-513938', '-3.43949e-8638157702', '-2.938660408383200902208256827974357565936e-513938', 40, 3);
    T('2.5897574065083988032402639003017505251016e-135766902484', '2.54909631925049695035022500575840417104896555e+93162102150', '-2.54909631925049695035022500575840417104896555e+93162102150', 60, 3);
    T('6.2033464912139281824481112390725732659113e-924', '7.553366846229128222702272917808e-50311', '6.2033465e-924', 9, 2);
    T('-7.510759329390389906020677323296900428274233947994920e+4860399473487', '1.765171587244973e+1689', '-7.5107593293903899060206773232969004282742e+4860399473487', 41, 6);
    T('4.476238456784872108833094777e-21112', '4.575e+86193', '-4.574999999999999999999999999999999999999999999999999999999999999999999e+86193', 70, 2);
    T('-1.0896286150639821594284535370807e+36', '6.667417156e+1881086997287', '-6.667417156000000000000000000000000000000000000000000000000000000000000000001e+1881086997287', 76, 0);
    T('5.2681465280724271396850522587756355521122842771e-11994162542', '-4.8766519280295827355340641833418057143391015539229208953685e-414333093174167', '5.26814652807243e-11994162542', 15, 0);
    T('-8.99e+8408228693', '3.987098337462114229756445816190032888840693446623903e+65963', '-8.9900000000000000000000000000000000000000000000000000001e+8408228693', 56, 3);
    T('-7.961168421e+270366', '1.1630e-72338017', '-7.961168421e+270366', 30, 5);
    T('-4.8653712802e+386496', '5.5647229139735685640621017737246560562545727145e-9252376148', '-4.8653712802000000000000000000000000000000000000000000000000000000000000000000001e+386496', 80, 0);
    T('-8.500191110893891916544724061217e-434803940153660', '1.56724445595278819379065410029e+105', '-1.567e+105', 4, 1);
    T('-3.0754063126415679366678648426060688097895567556027e+104', '2.1441738472780274669410144711e+8697310449287', '-2.1441738472780274669410144711e+8697310449287', 99, 1);
    T('-6.5705609920091810760459902e-84102694490998', '5.992627778118782606972671463232342134751191582138573751530e+449605', '-5.99262777811878260697267146323234213475119158213857375154e+449605', 57, 0);
    T('9.5285911922794627770349148737663784380922879e-8659529', '9.04e-14971038', '9.5e-8659529', 2, 3);
    T('2.6584423245565124529546720977796558564148737e-80928538097', '5.8505565228175512878568646754749055685496424e-67862969221', '-5.8505565228175512e-67862969221', 17, 2);
    T('-8.0114440539e-6760', '-7.2969503022677033668373632290845785e+287690596794972', '7.2969503022677033668373632290845785e+287690596794972', 75, 4);
    T('5.465413956687899907175271754828757838429029080541510e-93471334629', '-6.245568344216350691514736430565636911e+1994941', '6.245568344216350691514736430565636911e+1994941', 69, 1);
    T('-6.182498739999319669285893718e+86', '-4.57825800304541808573572496687488220535520427e+1538226715567', '4.578258003045418085735e+1538226715567', 22, 3);
    T('-1.4963349709022e-14541172576', '-6.2518072848651192870266054079524822354843e+797420', '6.251807284865119287026605408e+797420', 28, 5);
    T('4.3116133835923398259077479556011421287673204240e-3627', '-7.663576252808988853794726490593169301312e+611563052001', '7.663576252808988853794726490593169301312e+611563052001', 53, 1);
    T('-6.87722715828323189361235430487986589733996156313e-67271138908', '5.845258866647823447980e-443097', '-5.8452588666478234e-443097', 17, 1);
    T('6.2753991e+620276', '-5.04342929629658384062192154125717171877394432824946530714046e+4359829326', '5.043429296296583840621921541257171718773944328249465307140460000000000000000000000000000000000000001e+4359829326', 100, 0);
    T('7.8238686510515332284753524511931598393639375825e-1978251', '1.08693924676194941728266850743909120401386343e+82015223214771', '-1.08693924676194941728266850743909120401386342999999999999999999999999999999999999999999999999999e+82015223214771', 96, 2);
    T('5560930204860783594565584120949168079917.1', '-5.6128166721914334799097623683797360524730281452879246e+492919', '5.6128166721914334799097623683797360524730281452879246e+492919', 85, 4);
    T('2.913760696104445023957651789561592091071200815814e-89758229789', '-9.26096357744e+459', '9.2609635774400000000000000000000000000000000000000000000000001e+459', 62, 2);
    T('-1.1913538513479987826003172377681531772e-20980', '-3.6313193980225339e-875', '3.6313193980225339e-875', 52, 0);
    T('7.430243272276508068e-8491957178', '4.81318237434175752131831067709422451053631703044e-10195008258', '7.430243272276508068e-8491957178', 52, 2);
    T('-1.77948040335183402393402632549223904347e-4244551321994', '6.74825827605776746918669565e-853', '-6.74825827605776746918669565e-853', 54, 5);
    T('7.60452e+9735944', '4.149851796859122532759558322069797299327421579327e+938162220780', '-4.149851796859122532759558322069797299327e+938162220780', 40, 4);
    T('-7.65941437903410284884592593500454022e+89872', '-7.46270954360276477026657e-6100861702935', '-7.659414e+89872', 7, 1);
    T('2.75831612191109626962e+5149255', '-2.29587525635618373186728235921458254777803831e-223', '2.75831612191109626962e+5149255', 78, 6);
    T('2.3633142723514241323489107e-7790', '-1.419e+49', '1.419e+49', 73, 5);
    T('5.0665406763323756e+8209212', '3.045367665915784421696903682943370867633326819e-51395', '5.06654067633237559999999999999999e+8209212', 33, 3);
    T('9.1348403575660981552644463772777181e-891070', '9.66124109201216724767496163633916712617949679692029265502504e+3366', '-9.66124109201216724767496163633916712617949679692029265502503999999999999999999999999999999e+3366', 90, 1);
    T('3.076381060930342400123050902252720848826e+42910814', '-6.996827895174136935e+688922487190483', '6.996827895174136935e+688922487190483', 94, 4);
    T('-1e+3933', '-8.015771762640981897206050154322956353797e-33', '-9.99999999999999999999999999999999999999999999999999999999999999999999999e+3932', 72, 1);
    T('-3.827853332167502954912478307216432163846198e-35127857', '310259804592322996294800645593380765130.72488', '-3.102598045923229962948006e+38', 25, 5);
    T('-7.486662630292952791819489113409e+258733229432798', '9.63809219679342946867267016752978106687128252829486405847e+62', '-7.486662630292953e+258733229432798', 16, 6);
    T('-3.57485215694700162124931020171895403e-712353354157', '-2.6191777326787378003003514406510753949772174744e-24676403624', '2.6191777326787378003e-24676403624', 20, 6);
    T('-2.729285200e-57371', '6.3809e+33', '-6.3809e+33', 54, 1);
    T('5.49501434236858768659590447330019383064948e-468351930690', '4595180528596634', '-4.59518052859663399999999999e+15', 27, 1);
    T('-7.22051759709420065386165212e+85812827800092', '5.7717e+91374177', '-7.22051759709420065386165212000000000000000000000000000000000000000000000000000000000000000000000001e+85812827800092', 99, 3);
    T('3.84648073406e-4572248', '4.097415638263039521422105558454330154642286436828e+76', '-4.1e+76', 2, 0);
    T('2.998796253188021746324655690435156e-950621373498', '-7.7052657372061846512928531776117633779171291914857317855310e-35', '7.705265737206184651292853177611763377917129191485731785531e-35', 62, 4);
    T('8.610497262706347283251437266220101310732e-8870590667', '3.47406324462769044543e+35833136070202', '-3.47406324462769044542999999999999999999999999999999999999999999999999999999999999999999999e+35833136070202', 90, 2);
    T('-8.1321541565352851632157059254494687202530101461710937127e-9287', '2.02409946698227412038814154e-35', '-2.024099466982274120388141540000000000000000000000000000000000000000000000000001e-35', 79, 0);
    T('-6.909500470160632975749174e+42638', '-3.181320828236493145118360399282159507036889887953801e-7350978813807', '-6.909500470160632975749174e+42638', 32, 0);
    T('420512392114488518271568536.87356', '-2.61151719298796998155132297100928568221141818868095835495e-587064', '4.20512392114488518271568536873560001e+26', 36, 0);
    T('-9.0678481772459556251e-352652', '2.296074473806401687e+131906158', '-2.296074473806401687e+131906158', 39, 4);
    T('-9.892430931751070671105019531e+7564998', '1.54338536645121384773914287258570326e+865425599', '-1.54338536645121384773914287258570326e+865425599', 62, 5);
    T('7.5e+65803', '-1.6804560341920393822745616143246e-115702085791998', '7.50000000000000000000000000000000000000001e+65803', 42, 0);
    T('7.081121903333318393255618477205e-81553', '-6.575787514739107357810456546393688687e+782270410', '6.575787514739107357810456546393688687e+782270410', 97, 6);
    T('1.1594016308332493569295545000493264696525e+4397843502587', '1.4866792805179732843906266959077471804943123811042877805090e+560296528', '1.1594017e+4397843502587', 8, 0);
    T('3.882303628255925896058252738e+3438791511', '-3.35599480313297361355624908711574339808437054265499402057e-4643', '3.882303628255925896058252738e+3438791511', 57, 1);
    T('8.47541461842825220076956135392959e+4584205', '4.459182372927476883452495665930601282e+6976847161', '-4.459182372927476883452495665930601282e+6976847161', 98, 0);
    T('1.3686e-80071592697', '-620102164566445359.408572', '6.20102164566445359408572e+17', 93, 1);
    T('-5.115771370e-56478477', '1.5346834677788821813549230809892375811453206474114853663e+7826', '-1.5346834677788821813549230809892375811453206474114853663e+7826', 82, 6);
    T('-5.799265186697360764168320043008903515710712602129820e-887', '6.1693241417266610437412179905182926e+93510230', '-6.1693241417266610437412179905182926e+93510230', 64, 4);
    T('-5.5211714966654337375829093e+4804870', '87830668382108799593584622388609959.1735560187744034920208135', '-5.5211714966654337375829093000000000000000000000000001e+4804870', 53, 3);
    T('2.395453292e-294894753035', '-1.02477660041440377343e-355878336', '1.0247766004144037734300000000000000000000000000000000000000000000000000000000000000000000000001e-355878336', 95, 0);
    T('4.270381037706028433288630242191528092682291405e+995464', '-9.29391373e+4878', '4.270381037706028433288630242191528092682291405e+995464', 56, 3);
    T('-8.317e+9', '-3.16562021165818031137879234716285560008809462894364017e-48', '-8.31699999999999999999999999999999999999999999999999999999683437978834181968862120765e+9', 84, 4);
    T('-5.846096520557701953282161e+893788211', '-9.26863529537808345174176646529687367529736666826e-54202', '-5.846096520557701953282161e+893788211', 66, 3);
    T('-7.7386633533049845607566074843267998218261570868296214e+89595265', '3938650712582453025697062500840.05352', '-7.73866335330498456075660748432679982182615708682962140000000000000000000001e+89595265', 75, 3);
    T('1.52318466136859822330088976965984273087885e-842387540952', '-8.41194221294828123249503946151395323189790e+47', '8.4119422129482812324950394615139532318979e+47', 87, 1);
    T('8.83295605903974756586765032248755526437662326252430524376e+55710079', '8.8351053439608686939955095393681092001373626737e+977771290887', '-8.835105343960868693995509539368e+977771290887', 31, 6);
    T('3.07184835474868643272004170e-9181669381', '-8.89282636519927994463162192326266416726103711489363481e+19422827039', '8.892826365199279944632e+19422827039', 22, 4);
    T('5.44e-2223897', '8.872096897359484171008e-52617321076332', '5.44e-2223897', 24, 4);
    T('7.455925e+42792117447849', '3.877043550086540995192389444957055e-7649027151', '7.45592499999999999999999999999999999999999999999999999999999999999999999999e+42792117447849', 75, 3);
    T('-7.083774634834712840e-69027045', '-7.9236430603353898304828864501e-32491296046676', '-7.08377463483471284e-69027045', 59, 6);
    T('-7.000871121119168826582402e-950450808726369', '-6.29274038910259955439190693768355669694619920888570152e-159650247916', '6.2927403891026e-159650247916', 14, 5);
    T('-9.8829824486294133698782e+99149584', '-3.641480566355624473667911959491380515736742285e-58418781868', '-9.8829824486294133698782e+99149584', 57, 3);
    T('-8.09238989966663454238517262290424568897359275e+53', '-6.19713850102486010421378182512904447968975e-745', '-8.09238989966663454238517262290424568897359275e+53', 65, 5);
    T('3.082087925652051829155102956512613e+38', '-2.0650026195626030227313979755864420423830831455475844e-347124867815', '3.082087925652051829155102956512613000000000000000000000000000000000000000000000000000000000000000001e+38', 100, 2);
    T('9.70860953895978551476195348323216165729238728404029216594789e+3611508788', '7.12799447205628004174e-92731385651', '9.70860953895978551476195348e+3611508788', 27, 6);
    T('-9.83390537446384605487e+284956741631', '9.6191290961651265136147333748e+903768810232430', '-9.6191290961651265136147333748000000000000000000000000000000000000000000000000000001e+903768810232430', 83, 0);
    T('-1.50593383816402362672563209210805793820906555451e-149325276805', '-8.450785686761047279125690e-10', '8.4507856867610472791256899999999999999999999999999999999999999e-10', 62, 1);
    T('-5.689954e-42', '-6e+77373681184', '6e+77373681184', 87, 2);
    T('6.42532454136054899786388742045031614417e+5522884775224', '6.0687818467605881521777585109457963143800841858482531926538e+60', '6.42533e+5522884775224', 6, 2);
    T('-2.21381e-8167', '3.36', '-3.36e+0', 39, 6);
    T('-8.22585858624396847003924527480844e-4704726569313', '5.55333701640102594580479e+90981090622', '-5.55333701640102594580479e+90981090622', 27, 1);
    T('-9.48150801270901588355480741e+61922321', '1.2238591138181477692943336385993680987059822e+11044705203', '-1.2238591138181477692943336385993680987059822e+11044705203', 88, 5);
    T('-9.870191553125054245451072250315270e-375', '-3.1087345159089e-948342793849689', '-9.870191553125054245451072250315e-375', 31, 4);
    T('6.01081011158204658967587641822779676923121e-57491', '-6.745466714408929197681144424222290482259835870e+81160639899210', '6.74546671440892919768114442422229048225983587e+81160639899210', 78, 1);
    T('-76511278574705831994171393811118183058013188.110092948259', '8.489748640870483008487005386824097257858e-726', '-7.6511278574705831994171393811118183058013188110092948259e+43', 100, 6);
    T('-8e+3032228501334', '8.748361543919263031506948127e+88317841', '-8e+3032228501334', 23, 5);
    T('-1.5992e-906798', '-6e+6508497323', '6e+6508497323', 67, 0);
    T('-6.20208709281237052895417357720746140840574673858444263936e-349', '-2.636274e-2140952', '-6.20208709281237052895417357720746140840574673858444e-349', 51, 1);
    T('-4.1900829042612215172616973988116218e-485450', '4.0868678478472681835924557740518212725152353687236481385e+967391906402141', '-4.086867847847268183592455774051821e+967391906402141', 34, 1);
    T('-4.7095695812e+34772639657313', '1.467990153954498745e+87935935834426', '-1.5e+87935935834426', 2, 3);
    T('-9.0299671748851e-7927', '-9.437981e-86039131989', '-9.0299671748851e-7927', 94, 0);
    T('6.5661228995916034452435262297171638245188167e-331132588644205', '-7.381493244966636632859044183447617985e-48', '7.381493244966636632859044183447617985e-48', 93, 6);
    T('8.0011750289377572442945964987733291157848e-8995125161233', '6.336133908320935461285593269875008594075946047e-3824273072', '-6.3361339083209354612856e-3824273072', 24, 3);
    T('-1.817571e+19297118115424', '-83825545420.473677464', '-1.817571e+19297118115424', 72, 6);
    T('9.484948980497111194643040294697055724213066016669e+8395872951', '1.7629640106934822537858e+7628', '9.484948980497111194643040294697055724213066017e+8395872951', 46, 4);
    T('-5.8842248917886898215405465376723946056089e+2563607', '1.7502096449365400721172460579621292075e+69603', '-5.8842248917886898215405465376723946056089e+2563607', 48, 6);
    T('5.30701779864118606302687072846340233292282660740797532e+978519', '-9.3131690253435829757869955389779979966e+5622', '5.30701779864118606302687072846340233292e+978519', 39, 6);
    T('1.532810480710088436062793849351418724269e-500', '-7.16247482822304649622424171e-39118583', '1.532810480710088436062793849351418724269e-500', 90, 6);
    T('-8.888787948373475984602388639187e-152330', '-6e+939616427011', '6e+939616427011', 49, 5);
    T('5.9609836393496902989279242141e+37275637002624', '-4.269884554761365518393122313073829719747465e+1124072909837', '5.96098363934969029892792421e+37275637002624', 27, 3);
    T('-7.6791925439063571831343105416e-197', '-3.5243586144405056524161895131735792119614323763656406732e-93278447239', '-7.6791925439063571831343105416e-197', 71, 3);
    T('-93555.99937813450847164333297067763860098564', '-1.9513901052736797617222248530325358874062053e-643', '-9.355599937813450847164333297067763860098564e+4', 55, 0);
    T('-5.39323759e-7759721715', '-2e-3076', '2e-3076', 12, 0);
    T('8.35574200620959954104693279691942098769841916416e-83003406502', '-6.5888760868417957633984e+63509726281', '6.5888760868417957633984e+63509726281', 53, 1);
    T('4.36291190185421591528393269628378479009e+28853436374699', '6.91205905449443322342784e+969184060320', '4.36291190185421591528394e+28853436374699', 24, 0);
    T('-4.64942780272086112553447479498745601551279263426449395750809e-3670218', '-1.77060083847025142514357835e-82183480', '-4.6494278027208611255344748e-3670218', 27, 0);
    T('5.1508515308e+6037', '1.170812471789084726979947e+1327272', '-1.170812471789084e+1327272', 16, 1);
    T('2166823803231293564130391599240519', '-8.1015062088476542221776313894000848411498206855055001098e+51519424', '8.1015062088476542221776313894000848411498206855055001098000000000000000000000000000000000000001e+51519424', 95, 0);
    T('-1.3311e-489360', '-4.194844770119e+8628005', '4.194844770119e+8628005', 84, 0);
    T('-3.311201991518562813466504787568068712223210071489760e+958106', '-7.4356148246552164239716839277673588733917809866422057202541e+479479276664', '7.4356148246552164239717e+479479276664', 23, 0);
    T('1.3444544276229e-81993030', '5.455212735722150217e-7856697', '-5.455212735722150216999999999999999999999999999999999999999999999999999999999999999e-7856697', 82, 2);
    T('-1.671650985951963704445146013173783384963815e+7342677', '-9.642628747929451941889869313e-971248345338', '-1.67165098595196370444514601317378338496381e+7342677', 42, 1);
    T('-1.691706810078818e+19', '4.95062893381635051536848223809e-99422', '-1.7e+19', 2, 4);
    T('7.482436390430408477067138211502869605516447835176570e-51299801339061', '4.34980918139e+16', '-4.34980918138999999999e+16', 21, 2);
    T('7.768281982483215523072360237493236776e+38', '4.68996520044763486204428793e+59181', '-4.68996520044763486204428793e+59181', 83, 5);
    T('3.80803115392229957014830109020581367324785e-81834868', '2.18524029026577069e+23', '-2.18524029026577069e+23', 66, 3);
    T('9.7645598257e-87401559132409', '4.154e-6005615', '-4.154e-6005615', 22, 4);
    T('-5.28684720941568487484528e-580502896874', '-1.6212649426968560603229262379e+829180', '1.6212649426968560603229262379e+829180', 53, 0);
    T('7.7800e-52076584619335', '7.72588416649577698170497463405280909778290131730363086768e+21515', '-7.72588416649577698170497463405280909778290131730363086768e+21515', 57, 3);
    T('4.980294773155608358e+3385573362776', '-5.73118145041072908049264395418374987531069326290926187e-513706785', '4.980294773155608358e+3385573362776', 86, 3);
    T('-7.69311705151281069463945340558673814e-407764327290', '-9.773e-391343880', '9.773e-391343880', 73, 0);
    T('-5.58958513782886884864212599338174970599203423370e-1843536', '-9e-69943885279253', '-5.58958513782886884864212599338174971e-1843536', 36, 5);
    T('-7.74945923522959100e-571178', '5916874423117521841271900560.75', '-5.91687442311752184127190056075e+27', 100, 5);
    T('3.89264937070795184276913087688489560097358e+45', '-1.5220419780946187906e-259553', '3.89264937070795184276913087688489560097358e+45', 77, 3);
    T('-5.2893146225224606517531972253641444065874551968904714e+1435', '-4.2511924059503795384614238995042799121537827846e+1855283017', '4.251192405950379538461423899504e+1855283017', 31, 5);
    T('-5.26422446250660050241964118073034962000e-95051705', '2.15777281220507493586352435561679376728e+60658890502895', '-2.15777281220507493586352435561679376728e+60658890502895', 48, 2);
    T('8.9e-303720505177289', '8.877022686978584306865277642040333685274501e+108', '-8.8770226869785843e+108', 17, 5);
    T('-7.2478469410897950825383688785071495997262331147618334255000e-4552347539', '-7.1134094124015584505622670e+4508773648', '7.113409412401558450562267e+4508773648', 98, 2);
    T('0.000192', '-6.65801692881515881e+5644712', '6.65801692881515881e+5644712', 55, 3);
    T('1.6428792396078225834e-2739970929209', '2.8948082965187260261899211480468725336755333547685830e+13069544493', '-2.894808296518726026189921148046872533675533354768583e+13069544493', 79, 4);
    T('-5.568753e+2281', '-7.21074e-4141503999964', '-5.568752999999999999999999999999999999999999999999999999999999999999999999999999e+2281', 79, 2);
    T('6.4377552267024797101195e+6744', '6.35832980824509847982670937249019627741e-7558992098756', '6.43775522670247971011949999999999999999999999999999999999999999e+6744', 63, 1);
    T('-9.00956404972719742e-48762', '-8.7832389437449261107622e-411351732188', '-9.00956404972719742e-48762', 58, 0);
    T('-3.7172805634729386077682399579e+31', '1.90441370238e+38284579632', '-1.904413702381e+38284579632', 13, 0);
    T('6.8316451110741075882944e-46084267836', '-3.4260062312102827045149699662e+93219663', '3.4260062312102827045149699662e+93219663', 62, 5);
    T('8.8321392691867254506675699955381149819e-36', '-3.80666845908956935240442756193149821976394452484589537210920e+48409385525', '3.8066684590895693524044275619314982197639445248458953721092e+48409385525', 75, 5);
    T('1.53022011243443877808064804e+10963987051', '-5.09269265384473306506508135111176383210414904000172e+413028440375697', '5.09269265384473306506508135111176383210414904000172000000000000000001e+413028440375697', 69, 2);
    T('2.67711000827609463177375992929922439e-46901', '-7.15207922833508837577672607529109254325227030891831346996529e+348805609054', '7.15208e+348805609054', 6, 6);
    T('8.0207726678566289912657192532498788e-6397110928', '-7.3791203524126699215656743629620814150527781480777e-2840792', '7.379120352412669921565674363e-2840792', 28, 2);
    T('9.6106374907063443442333313796613020226091322798407849092942e+61750098482', '6.642333374966480290880457894201395296e-7504298268920', '9.61063749070634434423333137966130202260913227984078e+61750098482', 51, 6);
    T('5.06109625155751714664376408211893537740870332935887e+14878278508600', '1.603978439995654758810656e+6257563085368', '5.061096251557517146643764082118935377408704e+14878278508600', 43, 0);
    T('-5.66471338883681932003026267219556996403295116330590710034807e+1767164', '7.25929782603512e-8848721891963', '-5.664714e+1767164', 7, 0);
    T('-5.134046012501325259305001e+40198837682', '1.76280190534684107482523713777245405564228969520663905997422e-531', '-5.134046012501325259305001e+40198837682', 32, 1);
    T('6.711397918510179103575351693099811495683489351578485735230e+494454516', '-5.73935890575043077559325063593911095039803392635635e-43454422837', '6.711397918510179103e+494454516', 19, 1);
    T('3.414406203445981943e-3068656088807', '-4.40289007471959439904943821390745513931736189125375e+903818', '4.40289007471959439904943821390745513931736189125375e+903818', 67, 5);
    T('-5.45924113461096853059537229549682065e+838', '-2.6459659022495679255098707033610115684605181e+53', '-5.5e+838', 2, 5);
    T('-6.257494505273680279714346682491574517959247242738336397301e-74384', '7.36417353137890874260e+704736493071079', '-7.364173531378908742600000000000000000000000000000000000000000000000000001e+704736493071079', 73, 0);
    T('-3.389820761e+457', '7.786343980372706e-60906772', '-3.389820761e+457', 21, 6);
    T('4.3744742618438850477159938640749859908141705637207957e+152601028508', '2.626755201476939413284084e-33666725761', '4.37447426184388504771599386407498599081417056372e+152601028508', 48, 1);
    T('-6.74021136040064088940651555609042602841330171e+4650313714431', '4.473126852759480737249938058650300443e+39099', '-6.7402113604006408894065155560904260284133017100000000000000000000000000000000000000000000000001e+4650313714431', 95, 0);
    T('-5.98589180310979173637553351885553028542e-3460901783984', '-6.96743748053e-68232', '6.96743748053e-68232', 20, 2);
    T('-1.6353403275604779886e-27377636231992', '5.024575473581813124912583383435313e-677991641', '-5.024575473581813124912583383435313e-677991641', 63, 6);
    T('5.9570371366049017181811874871762557134504552239e+20242881792', '-1.267728545744910385209181844065291402375894130303e+56', '5.9570371366049017181811874871762557134504552239e+20242881792', 48, 5);
    T('-8.926563076341675431297219527153313635891620e-88167899368671', '0.37', '-3.7e-1', 7, 2);
    T('-8.3881964198890944916111269922641e-555424721350515', '-9.98656072317430696019e+628', '9.98656072317430696019e+628', 57, 6);
    T('6.896306672441069050379e-510', '-322729729615385173251860727505103599919419831044.3111802', '3.227297296153851732518607275051036e+47', 36, 4);
    T('-1.883936657191032889770988326104178730995e-86918359755857', '2.274831697733933470808174706e+2928026297', '-2.274831697733933470808174706e+2928026297', 42, 2);
    T('-6.0967748291260726882886072649052059117917944e+367', '-9.175094691037053289489703964660e+9754260629', '9.17509469103705328948970396466e+9754260629', 50, 2);
    T('2.14e+614260759336527', '-9.80621061554539789196392566e-160070915', '2.14e+614260759336527', 4, 1);
    T('4.702888564628441088131295e-61', '4.64653e+1003', '-4e+1003', 1, 2);
    T('5.32558588231660315526858081892913265199469461876913435789e-887', '-4.73065838101697342791367550528632067603281441815524644375946e-79448', '5.32558588231660315526858081892913265199469461876913435789e-887', 64, 3);
    T('8.152087768131129557271845153118034204081648254e+4899', '1.751659230795079799916912954495442598871149171716e+840288', '-1.7516592307950797999169129544954425988711e+840288', 41, 2);
    T('-9.69611325349476668084648746815835e+623', '4.764218528119183768857608410828381683487532372416315028e-52703257', '-9.69611325349476668084648746815835e+623', 79, 5);
    T('-5007166586366514959191079995846270980989995', '-9.4902952549857356212169860443970651143912995542230256736326e+55636765835', '9.490295255e+55636765835', 11, 2);
    T('-274948110594231914313359125129553607127949161730.7664', '6.760851567490149625957781e-78671', '-2.749481105942319143133591251295536e+47', 34, 4);
    T('-9.417275770067819119768691e+132417551', '9.561769513234094176993140770766739e-6884065241', '-9.417275770067819119768691e+132417551', 25, 4);
    T('8.73378202078558974332785120218867e+432', '-7.343196146048747770e+859375439', '7.3431961460487477700001e+859375439', 23, 0);
    T('2.97395933599663261437865464633424692658765e-947109', '9.8608482009479666211e-9222914', '2.97395933599663261437865464633424692658764999999999999999999e-947109', 60, 1);
    T('5.480578612785263865135599181720430806938e+228', '-5.58775348646105062015119172514869350919951725947569117617e-746', '5.480578612785263865135599181720430806938e+228', 99, 4);
    T('-3.63042224143268756478507111077726602728619375286106389e+864975983235', '6.0638557921525053764493648195435933915108975142963904e+884925396', '-3.63042224143268756478507111077726602728619375286106389e+864975983235', 97, 4);
    T('-2.67961889785345871824772812802064302e+365876', '-7.17431417018083894256940160554783372731055e+5306214762311', '7.17431417018083894256940160554783372731055e+5306214762311', 72, 4);
    T('5.5528061694772887e+27', '7.0439956e+10', '5.552806169477288629560044e+27', 25, 3);
    T('-9.834210001193229498589433777289989360286252561e-182186766', '2.8812252e+16', '-2.8812252e+16', 81, 6);
    T('9.110012e+829524688754', '8.347366791467485693857427606162e+192725205', '9.110012e+829524688754', 31, 6);
    T('-4.3935628117368180129842380611452701784492466703173744e-76634456442', '-6.16450541602995669683687068016606060524048070880932245817530e+63', '6.164505416029956696836870680166060605240480708809322458175e+63', 58, 4);
    T('7.5e+5714246', '-7.72708856282016935256827618893984297806120455e+992792509406000', '7.72708856282016935256827618894e+992792509406000', 30, 6);
    T('-2.55663951766786888415033041253819e-5006390135840', '4.30696808496950172204503295353659070371e-14347901013', '-4.306968084969501723e-14347901013', 19, 0);
    T('-6.200e-8787581864', '6.2e+8768720', '-6.200000000000000000000000001e+8768720', 28, 3);
    T('-3.83249834795543841740e-68', '-9.1794128694622845986143781165e+949823264402', '9.1794129e+949823264402', 8, 6);
    T('4.809658910856304446271883680390784153099575656e-15794795', '-9.95057893220213871487544315737554226187e-24', '9.9505789322021e-24', 14, 6);
    T('-3.39334837603447876913e-3505309000', '8.038438253218130063188968674647528642075328e+791025130', '-8.038438253218130063188968674647528642075328e+791025130', 44, 5);
    T('-3.8073195266344778892144701112362867554633944826966276413e-4029955692', '8.057122e-5528783875', '-3.80731952663448e-4029955692', 15, 6);
    T('-3.2642523129935564899690143380622213757765442e+204', '-5.3026869000133198925726167040780884763508e-870369229', '-3.26425231299355648996901433806e+204', 30, 2);
    T('7.267747143085502376002e-73454946272895', '7.650462390769220327e+48', '-7.6504624e+48', 8, 5);
    T('5.14030475441977672220343213773396948717e+59318716', '-8e-3557128970380', '5.1403047544197767222034321377e+59318716', 29, 5);
    T('-9.143924808182648985430772716672522992472997100691635e-839321148', '8.8006836239714194206033e+830095507608', '-8.800683623971419420603300000000000000001e+830095507608', 40, 3);
    T('-4.420928554534313992106627818898229555969842842986599259200e-7690904321', '4.6396333881e+93418', '-4.63963338810000000000000000000000000000000000000000000001e+93418', 57, 0);
    T('-5473.67848', '6.5e-22381320', '-5.47367848e+3', 24, 5);
    T('9.349145538826564559733274627842e-6418963747', '4.2362527613089182e-215', '-4.23625276130891819e-215', 18, 2);
    T('-2.9192332373457829818310348427300035546781997344204e+5408', '2.43756e+2092', '-2.919233237345783e+5408', 17, 4);
    T('-9.171942047830350612583746916343878788552579306057e+8336', '-7.9176784935446e+406847070159484', '7.9176784935446e+406847070159484', 53, 4);
    T('-6.301418378220720373509233140028099071821094582359553e+11349', '-3.5432250526987839178420425798392478300692779e-709', '-6.301418378220720373509233140028099071821094582359553e+11349', 59, 5);
    T('8.64087088e-64421', '-1.013103651740265025235429404936969e-8563', '1.013103651740265025235429404936969e-8563', 56, 4);
    T('5.9984544139019947804054134431624477206389148533010e-158360814', '3.8182759863583e-90672948181', '5.998454413901994780405413443162447720638914853301e-158360814', 72, 4);
    T('6.74836063068290556369e-5748088', '-5.2e+4408', '5.2000000000000000000000000000000000000001e+4408', 41, 0);
    T('8.8766401612021118507140967474659436245551693150402e+572618625288438', '-5.468770017353e+35730745715', '8.8766401612021118507140967474659e+572618625288438', 32, 4);
    T('7.6788266644891784524141582992610465240937057915112372599e+73430649033', '-1.16947043862447840e-54201368063804', '7.6788266644891784524141582992610465e+73430649033', 35, 5);
    T('-8.40701e+94830468971871', '7.29592452969343988075271117739228023167103197145585731e-708056194451', '-8.40701e+94830468971871', 9, 1);
    T('-1.0962430872808737387250454375484363155780036411773e-553121697791', '-9.566274700821686093647818188874e+99604409552938', '9.566274700821686093647818188874e+99604409552938', 47, 0);
    T('-8.4417391877434828895952519e+1969693339', '-3.474878975824824912520938490e+11582301', '-8.4e+1969693339', 2, 4);
    T('5.2321e-3066122', '-4.645749664077634e-314923465886', '5.2321000000000000000001e-3066122', 23, 0);
    T('-5.2694194946e-65550263', '6.5723570373957244051523466949093460845804483e+97616503757076', '-6.572357e+97616503757076', 8, 2);
    T('-7769336800775442886.4723110175377195705506', '3.461516022e+42170', '-3.461516022e+42170', 57, 1);
    T('8.53530151318350888311256296505568101741772972016196e+78248197036539', '-2.22e-73095439962', '8.535301513183508883112562965056e+78248197036539', 31, 4);
    T('4.2414267e+90175112853', '5.702841019448341095725536304908236862422e-555248', '4.2414267e+90175112853', 81, 4);
    T('-7.311905196515411987702220786473905004573805470062324e+14252651074498', '-8.15534569434086084204116264640157533386115219791945271e-377099483', '-7.31190519651541198770222e+14252651074498', 25, 1);
    T('-4.681246365810238558336947923496946702769752398499e+18677748256', '8.63489408531989908903959e+81', '-4.681246365810238558336947923496946702769752398499e+18677748256', 82, 6);
    T('8.316870603402779373745646477053242512621e-1134391360', '-27958535646.18128297524918000968776453', '2.795853564618e+10', 13, 6);
    T('1.6976409481227044416722732643613e+1648', '-4.854576271674591259199e+304', '1.6976409481227044416722732643613e+1648', 84, 6);
    T('6.9560138654211e+61562710030', '-1.6430886e+452505349490029', '1.6430886e+452505349490029', 98, 6);
    T('-5.69299376742750695115363189646819042869278221238378409794e-62273173153127', '-8.5197106602807597608572e+46213394', '8.51971066028075976085719999999999999999e+46213394', 39, 3);
    T('7.971635e+178', '2.59349035728298e-2175027992', '7.9716349999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+178', 95, 3);
    T('-3.9266256301907494659393628316035539547611e+70205194838570', '-6.7927417976588253551109386071236536703412922948077e+8406', '-3.9266256301907494659393628316035539547611e+70205194838570', 80, 6);
    T('5.69923012837472150336282940585e-9020', '4.520815482949796397771025692217326e-558', '-4.520815482949796397771025692217325999e-558', 37, 1);
    T('-4.0392656596e-40', '2.897844460646142381024851168286414845e+109229', '-2.8978444606461424e+109229', 17, 0);
    T('-5.2724604e+957843905302814', '-1.56228757251580498179273183469227583109910413828911809e+541256335970670', '-5.27246039999999999999999999999999999999999999999999999999999999999999e+957843905302814', 69, 2);
    T('9.86111e+19733912821538', '-5.3894167e-11247818653224', '9.86111e+19733912821538', 65, 1);
    T('-6.8511037418838349969883967e-850739180600470', '7.33899730384142720161152854281280054982026326918824113262e+82136', '-7.33899730384142720161152854281280054982026326918824113262e+82136', 91, 2);
    T('4.4303559621255164161768154427225982052e+46', '-5.162328179694182475330510697273e-947842667451', '4.43035596212551641617681544272259821e+46', 36, 0);
    T('3.58e-14957', '5.9009370487e-592963956', '3.5799999e-14957', 8, 3);
    T('-8.859610310134100427897813163147046859098207425491081e-394443329', '-9.534055139123971945922579498549100067665891752509806491e-4810450368710', '-8.85961031013410042789781316314705e-394443329', 33, 6);
    T('1.12724069945780102242277e-91861', '-8.03411344497794765765e+864879406588519', '8.03411344497794765765e+864879406588519', 68, 1);
    T('1.39889093490108269461898225090162291613e+3421378655', '-8.6250117631772324410241201837149119350836921038802495138183e-3873', '1.398890934901082694618982250901622916130000000000000000000000000001e+3421378655', 67, 2);
    T('-1.472e-6150', '2.765998683058460262990464920e-47114804', '-1.472e-6150', 69, 1);
    T('7.43061161802457986181964930315914366967900493421838140004e-77178428', '-4.890907985389208437256301086418604122381192712810714007235e-6961018702121', '7.43061161802457986181964930315914366967900493421838140004e-77178428', 78, 5);
    T('-3.70719374586870275162740166e+55284759013', '-4.3863914032707421340859557037173389833153693521e-24739958843', '-3.70719374586870275162740166e+55284759013', 55, 6);
    T('1.0756421629280120961712694819126660387638237178551872e+6613', '-9.97970024810e-756645653071', '1.07564216292801209617126948191266603876382371785518720001e+6613', 57, 0);
    T('-4.4910592417738514702668487649e-141878603841522', '-6.5413436187693929996136455990907410918317771573501790290113e-85440', '6.5413436187694e-85440', 14, 6);
    T('-5.3108869492806054787426072219408089724355860716628671686e+7891', '-6.24558294206307052477392998109123729009448611218934163159e-199885', '-5.31088694928060547874260722194080897243558607166286716859999999999999999999e+7891', 75, 1);
    T('7.564038654310381506e-372', '9.917305318834297612735053413155289265390e+171700', '-9.91730531883429761273505341315528926538999999999999999999999999999999999999999999e+171700', 81, 1);
    T('3.55158020370869918458622388982208130073071014856841074253e-548649074', '-4.43036700016365395748785458e+67566809', '4.43036700017e+67566809', 12, 2);
    T('-1.5280769355938610676077810448579570e-4265010625', '2.913250640661128128070281208e-8086992', '-2.913250640661128128070281208e-8086992', 67, 1);
    T('-1.259125933444304667880225137054414881512003841557893810292e+2214658532450', '-6.987566680316127296836065769109617455497656074729489e-46464326157', '-1.259125933444304667880225137054414881512003841557893810292e+2214658532450', 94, 6);
    T('-8.8712418892445016021671375712186662326303e-267679', '7.9030059241094442534531988273484323e+5063837', '-7.9030059241094442534531988e+5063837', 26, 6);
    T('3.468778756267709e-3223971361914', '6.14796559656117404367557580566857737232e-664736', '-6.14796559656117404367557580566857737232e-664736', 59, 6);
    T('4.02255236904e-5511654115', '9.955635896023403248000140716431504e+31867851', '-9.955635896023403248000140716431504e+31867851', 49, 0);
    T('-6.0597509481e-7663170606296', '6.783817435655912521969169734803757662091793110e+50', '-6.78381743565591252196916973480375766209179311e+50', 95, 1);
    T('6.28269488056118837299e+41459398', '-9.849414314384142092555638101715047173268906663995236111e-207453', '6.28269488056118837299000000000000000000000001e+41459398', 45, 0);
    T('4.801247355772710178338808694736320e-180972', '8.7873427717e-2519', '-8.7873427717e-2519', 68, 3);
    T('-8.823e+3707529', '6.8350306690805260e-8341482178322', '-8.823e+3707529', 42, 4);
    T('6.70430234535812227272581089284969191061667264e+274860', '6.4380487042468165355059893421652894146904508860750307453e-465785259333779', '6.70430234535812227272581089284e+274860', 30, 3);
    T('-4.4e+57606339', '5.07689830374899841428e-20717', '-4.4e+57606339', 85, 6);
    T('-4.668078933036976480568539276e-4055', '-3.7741843725336709885e-23', '3.7741843725336709885e-23', 64, 2);
    T('-2.876440e+105', '-3.3319104641208535641893073934294e-83031282', '-2.876439999999999999999999999999999999999999e+105', 43, 2);
    T('1.5567788219286696e+665', '-8.953771e+59782', '8.9537710000000000000000000000000001e+59782', 35, 0);
    T('-5.0283383819983200417807015922889234408e-88223', '8.13052352959456487510759535691882640446807118463214834634921e+1233531541', '-8.130523529594e+1233531541', 13, 2);
    T('3.2486788803535911579664970063765e-440', '7.71248561102879802166297338224929732711734255e-2077271', '3.2486788803535911579664970063765e-440', 63, 6);
    T('8.20217298265326454484732377867344486821518019758006311932290e+2409715033', '2.6058528e-67414861470', '8.2021729826532645448473237786734448682151801975800631193229e+2409715033', 72, 2);
    T('-4.14913626630635598958045276170523313164083e+23556300', '-6.459075216394315170259082520375014194035e-975557', '-4.14913626630635598958045276170523313164082999999999999999999999999999e+23556300', 69, 1);
    T('9.7949e-9401', '-7.120354054534770448349809035837e-452643852582', '9.79490000000000000000000000000000000001e-9401', 39, 2);
    T('-5.34110751480382896974624e-4123390115150', '4.544727414031271270607325568711877e+4177627', '-4.5447274140312712706073255687e+4177627', 29, 4);
    T('-6.799840356779583789022616892635227327e+638515', '-4.6974605008331754775352470830e-904646310334', '-6.79984035677958378902261689263522732699999999999999999e+638515', 54, 1);
    T('-3.5706505031963052e-34587389', '3.3404078400470777482180616643516715676550386e+74221', '-3.3404078400470778e+74221', 17, 3);
    T('6.44831735092111786007501788893308601461057549752416922e+360589', '2.551188380549917178213244310886793412653295e-73480764802', '6.4483173509211178600750178e+360589', 26, 1);
    T('9.73670889127747e-19908', '-1.189086781018e+8381490489', '1.189086782e+8381490489', 10, 2);
    T('-5.253028414637319e+761261034879', '3.2483719198738211995738e+521', '-5.253028414637319e+761261034879', 33, 6);
    T('-9.55921e+1293', '3.0994925084637520106886282481743434384e+64771660044', '-3.0994925085e+64771660044', 11, 5);
    T('-6.8953675743128483e-86677258395', '5.46892560623650838359749880600459925769439573112e+491107919938', '-5.468925606236508383597498806005e+491107919938', 31, 6);
    T('-7.50796537704619454899787674418682034762951985339161606e-77300466621499', '3.5651851706788564801281198477737473902322540132330430058481e+7787562420056', '-3.5651851706788564801281198477737473902322540132330430058481e+7787562420056', 72, 2);
    T('3.00231621154922220434687601411846e+331687091', '1.005883520176361360506296807685370992229942685090e+367021927389', '-1.0058835201763613606e+367021927389', 20, 3);
    T('-6.961403029042051644496e+746370488', '2.30681893220680806796634610e+6756035307', '-2.3068189322068080679663461e+6756035307', 28, 2);
    T('-3.112670204152008855e-79140481003388', '-5.872038809194711312618255692709534176833101757252700e+392', '5.8720388091948e+392', 14, 2);
    T('-5.28e-3455169', '1.86062641e-5619', '-1.8606264100001e-5619', 14, 3);
    T('-9.9313930379832267e-9886529152664', '6.82426149322756220e+4739084590666', '-6.8242614932275622e+4739084590666', 36, 4);
    T('7.18745153265792133204350110449e-37192', '-4.88290211577630142e-747999422809574', '7.187451532657921333e-37192', 19, 2);
    T('-9.6289197903687884980405342384320903e-92436', '-7.514354833e-281898433593193', '-9.6289197903687884980405342384320903e-92436', 93, 3);
    T('8.5683673202578484663196289026118958175170608290075e-11059087', '7.911343034352164311356455e+5519665272', '-7.911343034352164311356455e+5519665272', 62, 0);
    T('7.9803346284266882970231055785e+204411', '3.5303936433233839e+733859', '-3.5303936433233839e+733859', 39, 3);
    T('-9.346684e+22516202753650', '-9.478300787488133136686706267161413516707702835026305817600e-7744', '-9.3466839999999999999999999999999999999999999999e+22516202753650', 47, 2);
    T('-4.6894637384929868729078697852311231260880e-15', '-5.6312025e+1196894806340', '5.6312025e+1196894806340', 8, 4);
    T('-7.2589246550081e-55894943700', '1.536413289930145682878196405407411434555798550362796266e+7142387996', '-1.536413289930145682878196405407411434555798550362796266000000000000000000000000001e+7142387996', 82, 0);
    T('5.385340293779798312243271703424660218510870e+900248122246', '9.5686960986608819619945808450061292443002689299e+11933164112340', '-9.56869609866e+11933164112340', 13, 1);
    T('-2.48061734244110e-658738', '5.881027198069971661061317521459433824335e+61453094937706', '-5.881027198069971661061317521459434e+61453094937706', 34, 4);
    T('-3.5682192669501913505348461068806149874662e+6512662142', '-6.71111295571719818295018783308026293331924945e+542595980733226', '6.711112955717198182950188e+542595980733226', 25, 0);
    T('-8.6534331e-3915000', '-7.081e-12061', '7.081e-12061', 18, 4);
    T('-9.1827e+93497', '8.153017900818673106514295902519979939718653531737288e-661', '-9.182700000000000000001e+93497', 22, 3);
    T('-2.0889110330402355193353142593395295397298e+70414599', '7.0938824103161345861689038e+4132', '-2.0889110330402355193353142593395295397298000000000000000000000000000001e+70414599', 71, 3);
    T('5.0555e+24', '-2.97569049277867121458181519774e+352983', '2.97569049277868e+352983', 15, 2);
    T('-7.56585962344436290812023945887721826207975e+3847972', '-8.168761329654672282160139105841106640258503052702910e-568425', '-7.565859623444362908120239458877218262079749999999999999999999999999999999999999999e+3847972', 82, 2);
    T('-4.2810111175994854211151005804859437898e+506', '-3.13786845602058342671584373045420470771e-6875231927037', '-4.28101e+506', 6, 4);
    T('-1.451138939347070217565e-38991342', '-9.1595560160750858076816530663503e-66209', '9.16e-66209', 4, 2);
    T('-5.184254883255065e-7246', '9.63022864222979051138939649338309799052948841e-941857', '-5.184254883255065e-7246', 26, 4);
    T('-6.701936482622478910407333321033187e+966647', '2.77392939060236873636009957116356123237471514811751426608e-465357', '-6.702e+966647', 5, 3);
    T('-4.9510416315505092591103e+459768572792911', '-9.7028663680378467022421425886962014440529e+42', '-4.9510416315505092591103e+459768572792911', 81, 0);
    T('-9.365039360679113044341461942033387163062560578e-5346690924', '-1.578e-5586319291', '-9.36503936068e-5346690924', 12, 5);
    T('7.11045813124023869e-169', '-6.2155643434107625223924614534250663297397857e+8577', '6.2155643434107625223924614534250663297397857e+8577', 93, 4);
    T('-4.15926595411437639016223959626663634e+29486919474', '4.338e-490214808', '-4.1592659541143763901622395963e+29486919474', 29, 4);
    T('3.36661620530500771670e+4891404', '-3.580122752034320415826141e-700365088723', '3.3666162053050077167e+4891404', 59, 5);
    T('1.629958496937872172571351847335889e+2350876688967', '-1.243839006e+15868', '1.629958496937872172571351847335889e+2350876688967', 85, 4);
    T('-6.8237854682263447299446336093467613635951328733547756610e+628397', '4.5886963777325558341219410648719642967925184405861073937e-82903019653132', '-6.823785468226344729944633609346761363595132873354775661e+628397', 90, 4);
    T('9.802516819094660312863740e-524359067', '33524707580654648041523766557723944950855280157.9053717', '-3.3524707580654648041523766557723944950855280157905371699999999999999e+46', 68, 2);
    T('1.32229865732189231174697970e+31', '-6.693478839162113846959222548323021276444731667884e-31444', '1.3222986573218923117469797000000000000000001e+31', 44, 0);
    T('5.277554767753306747008e-487636', '-7.862474289065e-28875545486', '5.277554767753306747008e-487636', 25, 5);
    T('1.91328349385429839599e-76229924204', '3.575589764987127118372035200367866e+172523426440546', '-3.575589764987127118372035200367866e+172523426440546', 87, 3);
    T('-2.810434740207333021584566160674983446008516e+106', '-3.52147279511027384827e-82660898015949', '-2.810434740207333021584566160674983446008516e+106', 79, 4);
    T('7.23872661608091771142501116e+84672149341', '3.44870365237898130e+4089234478', '7.23872661608091771142501116e+84672149341', 70, 5);
    T('-4.844376758368070840816202930304566523654654e-193946', '-2.910474479325229381609771604675148884204035983e-11937097273', '-4.8443767583681e-193946', 14, 3);
    T('-6.6937399011e+87204461', '9.90628488552227599002942842e+31', '-6e+87204461', 1, 1);
    T('5.342971409533825005645320e-964681574', '-7.51395497431148381043002573777160903174764528599209353e+26242171304', '7.51395497431148381043002573777160903174764e+26242171304', 42, 3);
    T('3.653677645115888684e+203081670', '-1.449537324319614494113808579e-64', '3.653677645115888684e+203081670', 61, 4);
    T('8.301381067899981335280711e-429759696843', '3.37386232390097835551342568444011559647624e+2272966512482', '-3.37386232390097835551342568444011559647624e+2272966512482', 56, 0);
    T('-2.63396e-86509', '-8.248348210971872950396941875810792838738234363422509888407e-196989312', '-2.63396e-86509', 33, 3);
    T('-4.0084296954427339732221963877733032869e-17239237', '-1.95874759937443856259e-719726302', '-4.0084296954427339732221963877733032868999999999999999999999e-17239237', 59, 2);
    T('4.15751512092446769329296191301029370149658560661e-1400991302', '6.375651494025415439469413150487002899862928448123702887e+3309464', '-6.37565149402541543946941316e+3309464', 27, 3);
    T('-4.434651018e-863', '6.61572210553e+585401', '-6.615722105530000000000000000000000000000000000000000000000000000000000000000000000001e+585401', 85, 0);
    T('3.198071861e+426848285474685', '-4.854597192713549899309248267110579508190914364251223388e+57', '3.198071861e+426848285474685', 39, 1);
    T('9.38926731285775921082725214009398019999025984791750e+762', '7.5049908106006095033783713507588154671639780742487780672e-262304', '9.3892673128577592108272521400939801999902598479175e+762', 56, 0);
    T('-5.97447328583480799669593964909218151109232977005906e-93502378755', '-2.99351645914537552284006560343006951964746523739e+701315115270576', '2.99351645914537552284006560343006951964746523739e+701315115270576', 70, 5);
    T('-3.5894936408075592008928437231955415398300363855816592e+84724', '-8.71562698e-85566668', '-3.589493640807559200892843723195e+84724', 31, 2);
    T('43238928648634485768161795654474613219756048', '-2.4507257339153254619225144194306970105524376830604984e+15772547', '2.45072573391532546192251441943069701055243768e+15772547', 45, 3);
    T('6.34278759400007777e-52', '-4.311459506093702153444e+78', '4.3114595060937021534440000000000000001e+78', 38, 2);
    T('2.430609726248455612642655051484039503292100e+8855363386150', '9.70969830725582790811413636870835266287252435284436633e-35850972434819', '2.430609726248455612642655051e+8855363386150', 28, 4);
    T('9.5198886835101859568082e+45615', '4.36667762400572317710592063862180117841289995944e-335269412', '9.5198887e+45615', 8, 2);
    T('-8.11820673092116026e+43211791', '5.446164050597463605208946135834598276886792162e-7159', '-8.11820673092116026e+43211791', 80, 5);
    T('9e+259286', '1.33903945516427126337829567685095149556e+27801617864', '-1.33903945516427126337829567685095149556e+27801617864', 55, 5);
    T('7.2622589500214012975326445354729039e-793966403482', '9.37227321690537389645065840947222e+78424', '-9.372273216905373896450658409472219999999999999999999e+78424', 52, 1);
    T('4e-1073967', '1.38012070773935363414052e+1427280630', '-1.38012070773935363414052e+1427280630', 80, 5);
    T('3.63776677e+427561709447877', '-9.40736724884486962676382722964877099e+268847772586244', '3.6377667700000000000000000001e+427561709447877', 29, 2);
    T('-1.07742911092726447911942527851118397642810591229602696817689e+677517814', '9.163912885970646595205948314438036817362868226235934250920e-68813808116', '-1.0774291109272644791194252785111839764e+677517814', 38, 5);
    T('4.757e+218603', '-9.4359204351208115482329323901e-824436909625', '4.757e+218603', 67, 1);
    T('1.86942438733e+74114184783768', '-2.68820873140e+795029', '1.8694243873300000000000000000000000000000000000000000000000000000000000000000000000000000000001e+74114184783768', 95, 0);
    T('5.4405441938316109557570662171991294983559e-78912073753048', '4.6088128489169849856853e+5383', '-4.60881285e+5383', 9, 5);
    T('8e-770788015771', '7.6390506108998548014324167467e+9776', '-7.6390506108998548014324167467e+9776', 82, 3);
    T('1.269995071927991207818934941818223e+35', '-8.4743e+33659', '8.4743e+33659', 22, 4);
    T('9.26381738221504e-723979711860', '1.733551292568649418679208098333194788994e-29988487452', '-1.7335512925686494186792e-29988487452', 23, 6);
    T('-4.9746593509168450905324978015373486909e-70349885', '1.872814266425458406571191914068201202430422196393841714e-82107393510', '-4.9747e-70349885', 5, 5);
    T('3.134004067726e+262550', '-3.68336832267038908233897818008627e+7136033990323', '3.68336832267038908233897818008627e+7136033990323', 84, 3);
    T('-5.23741273150200645678638433e-8931735917049', '-2.108979580776e-566857127074858', '-5.23741273150200645678638433e-8931735917049', 77, 6);
    T('-59764050254229448759090850182458.16', '-7e+187411', '7e+187411', 8, 6);
    T('-5.67e+140453722651896', '-4.91748859267e-318836', '-5.67e+140453722651896', 67, 6);
    T('1e-1454388850000', '1.148063806746861e-58592205359440', '1e-1454388850000', 21, 4);
    T('-3.077173492680034254945157473170543391497885283105761014e+58500', '7.3434896e+9904', '-3.077173492680034254945157473170543391497885283105761014000000000000001e+58500', 70, 0);
    T('8.08e+7', '1.5641062925828194051395508703033850191346899919063e+1176384800', '-1.5641062925828194051395508703033850191346899919063e+1176384800', 88, 5);
    T('-9.712752805614217e+87172387387783', '5.33296057615925e+64609031988', '-9.712752805614e+87172387387783', 13, 4);
    T('2.71152065319785973394321824675319148e-50793381902', '-1.1409664822e-669809', '1.1409664822e-669809', 60, 3);
    T('-6.4636370924446679114764089677449235148256541917684686e+2676', '-9e+96644639091', '9e+96644639091', 95, 5);
    T('-5.8732168306271203855780917961264857158919876542347459663e+889721218097357', '1.108902661297679280638363889694763329212e-786702', '-5.8732168306271203855780917961264857158919876542347459663e+889721218097357', 60, 2);
    T('-5.82632250395609939573152340516579388492529991628e+352', '3.588160e+788642550', '-3.5881600000000000000000000000000000000000001e+788642550', 44, 0);
    T('9.2667661402400e+88', '2.84e-158444354076643', '9.266766140239999999999999999999999999999999e+88', 43, 3);
    T('-998222048742592874174', '-1.734991e+5032', '1.734991e+5032', 64, 6);
    T('9.135161703093106980431657e+471622162', '1.415917652680233722922609e+6709713587', '-1.415917652680233722922609e+6709713587', 62, 6);
    T('-4.60728313398136899578258177875782009634e+7109761', '6.192347590151261674152990316338830514e+7035150150', '-6.19234759015126167415299031633883051400000000000000000000000000000000001e+7035150150', 72, 3);
    T('3.0102736777318384202252984243640085e+42', '-8.46650415912343474030546044828044768906e+81601660', '8.46650415912343474030546044828044768906e+81601660', 41, 3);
    T('1.264397481720297417391936046226892250739179e-999478866', '-9.5264596830264164338973e+950', '9.5264596830264164338973e+950', 72, 4);
    T('9.59991364190234485290450793e-85104571507587', '-6.440525534230461878085669934422303689625011729093380e+856624936', '6.44052553423046187808566993442230368962501172909338000000000000000000000000000000000001e+856624936', 87, 0);
    T('9.919925204652671017912572083889329615659153414774466e+991216787167980', '-7.663701126157838244922064797608079114496925e+84851', '9.919925204652671017912572083889329615659153414774466e+991216787167980', 70, 1);
    T('8.7856803059410774628141721903698560911e-138791749805', '-739554722524.3376', '7.3955472252433760000000000000000000000000000000000000000000000000000000000000001e+11', 80, 2);
    T('7.14473309071822140375594684286361255e+89175324213178', '-1.33875432563951610647129000e-7304', '7.14473e+89175324213178', 6, 4);
    T('3.6566193e+91122670', '-9.865502133240965774e+450449351', '9.865502133240965774e+450449351', 72, 5);
    T('9.6e+84', '4.9988523476056524287611478156235352551842e-3149015', '9.599999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+84', 94, 3);
    T('6.5024882835733406008731523052399e+769', '9.07205122496947232273038809144897373295e+24896247983', '-9.07205122496947232273038809144897373295e+24896247983', 88, 5);
    T('3.016302635739200747104154981694022675e-7035288016027', '6.5260540062084526999e-648123', '-6.5260540062084526999e-648123', 88, 0);
    T('1.0527e+12', '-9.36012288387155566571397195e+308355657625', '9.3601e+308355657625', 5, 4);
    T('7.672880626817101e+185487309', '-6.5112385633536409021566311e+11191179118', '6.5112385633536409021566311e+11191179118', 96, 3);
    T('-2.0023297065689784883732886100921931684722426962392660489165e+148', '8.5792941935075578098666775567441191507837385653328234e+9551841644057', '-8.5792941935075578098666776e+9551841644057', 26, 5);
    T('7.7652449502716790275316e-6810727', '-6.71523354e-797330', '6.71523354e-797330', 68, 5);
    T('1.187343105213508881217993881557318293953290701680195083800e+4069747310296', '-1.911623115319951975e-124', '1.18735e+4069747310296', 6, 2);
    T('-2.51254127e-926701456', '4.304666920e+5277919125037', '-4.30466692e+5277919125037', 69, 5);
    T('125000427792604577417986', '-3.9256235600524529085026467956015783688114646806406999763e-55917', '1.25000427792604577418e+23', 21, 6);
    T('-9.81089307820507079869354175764e+747756459', '-6.948067120563e+765415012193', '6.948067120563e+765415012193', 94, 2);
    T('9.8565169642011430e-60884495', '-8.97931858367299156588179e+904999566764518', '8.97931858367299156588179000000000000000000000000000000000000000000000001e+904999566764518', 72, 2);
    T('-2.6483418197756535611342613358188794391941408718614e+63698204', '7.2516040550622823640051791293606475e+6151524', '-2.6483418197756535611342613358188794391941408718614001e+63698204', 53, 0);
    T('8.737860604336368839553e-67', '4.6710245145907276958846365e+2628100', '-4.6710245145907276958846364999999999999999999999999999999999999999999e+2628100', 68, 2);
    T('-4.954022910017968190285403325260767859e-94091988999', '5.732777854325310951047687805e+43124', '-5.732777854325310951047687805e+43124', 32, 6);
    T('6.324534070012049686282678437126e-8818', '-8517102738089446682105868872567675', '8.517102738089446682105868872567675e+33', 51, 4);
    T('-4.90e+99953', '-7.49206837495161959082766052345591293609533e+684148313', '7.49206837495161959082766052345591293609533e+684148313', 98, 4);
    T('2.76422483807910216930212e-691679', '8.77441705988954287042285786872580462373e+539440', '-8.7744170598895429e+539440', 17, 3);
    T('9.113450878955925129016973475076112016350067e+9988457533217', '-3e-7690085', '9.11345087895592512902e+9988457533217', 21, 5);
    T('2.3085e+784', '7.98718484412456499466813327337858120374753069779897688e+3887461812', '-7.98718484412456499466813328e+3887461812', 27, 0);
    T('5.5299513818123881426014968944e-25036236', '5.15420795384191865543760764386427884830647610076951623292e+370201396352', '-5.15420795384191865543760764386427884830647610076951623292e+370201396352', 61, 4);
    T('-4.45769541453829119167265921372e+3400309300289', '6.5286622978984178803866119238312123112261015623339e-9204026', '-4.457695414e+3400309300289', 10, 1);
    T('-3.794573467423925402903172635e-9303', '-8.0262022552724634596242556072532799037108976852468439250489e+49298226278', '8.026202255272463459624255607253279903710897686e+49298226278', 46, 2);
    T('9.980642666927612540533540916795201086665488550067698300460e-5785042770265', '7.87312e-666280', '-7.87312e-666280', 94, 6);
    T('-5.05208139280285808561789177494781530458786834135498e+52928', '-9.47849647170009202350096e-71664037', '-5.05208139280285808561789177494782e+52928', 33, 5);
    T('8809068223.848120', '-1.6298351828977416383360103742715804601071795390403551993046e-12400467', '8.809e+9', 4, 4);
    T('-3.7520e-74626282102551', '-1.9305536912780573459767654252424758e+472', '1.9305536912780573459767654252424757999999999999999999999999999999999e+472', 68, 1);
    T('8.26578373022514422464811506467345727077785730591e-27031441687', '6.712056989669192323127e-11293146112478', '8e-27031441687', 1, 6);
    T('1.8852295966287055620e-421', '3.57750494e+6872', '-3.57750494e+6872', 80, 6);
    T('-4.1010201903813075874954790375406412403105439989946362934662e-7474146845470', '-3.242547424933397770315593903025923231880477203534341166e-3233487431573', '3.24254742e-3233487431573', 9, 4);
    T('8.4162685056423121649195740135813813665632027546015e+37439', '4.40523245347788864e-812245570', '8.416268505642312164919574013581381366563202754601499999999999999999999999999999999999e+37439', 85, 3);
    T('6.148638547421554035528255020719951675786e-63987812', '162288751188863911595441173289529331525258495983595.825520', '-1.62288751188863911595441173289529331525258e+50', 42, 5);
    T('-2.6730912982841353603379705133705123109407430551712e-91830', '-6.678807801755576235893595594799550495027955551e-218524', '-2.6730912982841353603379e-91830', 23, 2);
    T('6.290218561950540070945154886364666074387965206847352903e+8007255971244', '5.8404421160930840872546474552211e+589853766', '6.29021856195e+8007255971244', 12, 1);
    T('2.3346746060029397229271031521777000177827022227282975e+29798805', '6.2688196037e+792680037819433', '-6.2688196036999999999999999999999999999999999999999999999999999999999999999999999999e+792680037819433', 83, 2);
    T('3.44358587165520144258942387979158674740233169e+689', '7.713371261132684930492803121664936422443072499336e-543252668570', '3.44358587165520144258942387979158674740233169e+689', 100, 6);
    T('-2.3131096885629154633945017e-676419589', '2.7139484303202100e-2623759451918', '-2.3131096885629154633945017e-676419589', 56, 6);
    T('8.274480811530e+359872673949', '-2.39345122140050541071425033590404555255036084342495942e+2965', '8.27448081153e+359872673949', 85, 3);
    T('-2.8245069982e-758303940', '7.7659586783487142119011103402514369422111760854890989644333e-974585478', '-2.824507e-758303940', 8, 5);
    T('-1.0125203749683589005e-2277220397439', '-6.11061355713111e-10', '6.11061355713111e-10', 57, 0);
    T('-8.08578730281667302264e+962710931400', '-2.482812715186810152320658e+276112', '-8.08578730281667302264e+962710931400', 89, 0);
    T('-8810660112013665428299184351178020383056224.69998', '-2.87818232864433589835455e+78165', '2.87818232864433589835455e+78165', 36, 0);
    T('5.490621280405004415359171971028555e+402889602', '4.216969187278697894280747e-661483', '5.49062128040500441535917197102855499999999999999999999999999999999999999999999999999e+402889602', 84, 1);
    T('-1.8848e+167', '-6.102033271909300e-2970', '-1.8847999999999999999999999999999999999999999999999999999999e+167', 59, 1);
    T('2.6e+38027677108', '-8.683306517382e-448570912440852', '2.6000000000000000000000000000000000001e+38027677108', 38, 0);
    T('8.67692071700150689993042905276947173e-3924785585276', '5.8294515e+94593582685076', '-5.8294515e+94593582685076', 33, 0);
    T('2.9506524801279083151281144827616427536001217643653669e-907835339217419', '1.2017589845245749732724957907468104382993812e+812368505934', '-1.2017589845245749732724957907e+812368505934', 29, 4);
    T('4.6983814229186724228801e+10639801', '2.702555878594205e+98', '4.6983814229186724228801e+10639801', 56, 5);
    T('-2.3097663410621728880155258592343e-5883803', '6.6587306051342307980138853370636605250849663959816e+146', '-6.6587306051342307980138853370636605250849663959e+146', 47, 1);
    T('-6.872762913088155847468465224047359604691636904623e+925544040561386', '4.2569846466449592184463101591817377282e+6717810305', '-6.872762913088155847468465224047359604691636904623e+925544040561386', 81, 2);
    T('7.589866264741729916935e+93149241304', '5.4294921650e+901321', '7.589866264741729916935e+93149241304', 86, 2);
    T('-8.819902081700764857666548988491452882560919361840125e-6616368074', '-7.296032782628337770787672165483446013115962683920584304957e+5163743391532', '7.296032782628337770787672165483446013115962683920584305e+5163743391532', 55, 4);
    T('-3.59372885091943620e-18911804538', '1.655073284763e+70341559586', '-1.655073284763e+70341559586', 46, 2);
    T('7.38400886767513295227520147237883993144573635122543350511e+28958', '3.5649144608221884e+22', '7.38400886767513295227520147237883993144573635122543350511e+28958', 71, 6);
    T('2.92980131817927e+22', '491954226907367771600.161', '2.88060589548853322283e+22', 21, 1);
    T('-8.44655699599e+4526683538930', '-9.802034595575e-165116222285', '-8.446556995989999e+4526683538930', 16, 1);
    T('7.607952221385214706619640945144154083048614e+49', '-1.95973354228497212589005495834045009647868394721458567576584e-76325211734717', '7.607952221385214706619640945144154083048614e+49', 76, 6);
    T('-1.549238e+7', '6.817988871887617946e+428166971939127', '-6.817988871887617946e+428166971939127', 85, 5);
    T('-7.1e-17212', '-2.04615154256218926043788e+498714', '2.04615154256218926043788e+498714', 62, 0);
    T('6.6889e+238022', '8.699e+4252436794553', '-8.699e+4252436794553', 40, 6);
    T('9.6237059053324388327523991072346697158667299045580340027e-657677024', '-8.4275799536711849e+526609', '8.4275799536711849e+526609', 76, 5);
    T('4.816739346658768294e-22', '-9.3758130294309384866597903808576656169026037e-4442', '4.816739346658768294000000000000000000000000000000000000000001e-22', 61, 2);
    T('-3.703655824650995677479850031151932091344062141e-798817', '-8.278983028302853637713216121424475641397048651614e+39507920996', '8.278983028302853637713216121424475641397048651614e+39507920996', 72, 4);
    T('-2.7548736777446196513980355084014843101241802670e-4806', '-8.465818e+8', '8.465818e+8', 58, 2);
    T('24390998', '1.17579704860633874623511152e-39309130321644', '2.4390998e+7', 92, 5);
    T('7.727249796306541211219902e+9059', '9.270698454003491115569980751123697831329633691e-2884338', '7.727249796306541211219902e+9059', 82, 6);
    T('4.467156623645903852265738e-7730911864', '3.775904077044224595742484760350171923794030952e-6483587412960', '4.467156623645903852265738e-7730911864', 36, 5);
    T('-2.39594036487634115147364927302489296541848738e+62716670', '9.38257e+66799239', '-9.38e+66799239', 3, 1);
    T('-9.0801833112285803624844163886331160e+16113327637122', '-8.221e+201921173348', '-9.080183311228580362484416388633116e+16113327637122', 86, 6);
    T('-7.35876460350640406340364879388948e+39', '6.51551567673749e-851861327', '-7.35876460350640406340364879388948000000000000000000000000001e+39', 60, 3);
    T('-9.4942221933612208742437909229201e-5525761937', '6.6579e+26792412', '-6.6579e+26792412', 39, 6);
    T('-2.68175611186796210068e-910626436400', '8.311199552488026259309597138689632806215575805588e+410500665', '-8.3111995524880262593095971386896e+410500665', 32, 5);
    T('-7.232688410580542e+691851420694', '-8.2994234841232627559398870822565191462771162769e-60870340974042', '-7.232688410580542e+691851420694', 42, 5);
    T('-9.251318339900790605e-668364542', '-2.909711254591818583694094880697443886060332704e-521102988', '2.909711254591818583694094880697443886e-521102988', 37, 5);
    T('6.4192867168187731103515294428229930692668386194358962071801e+24891', '3.438419221e-567614752365', '6.4192867168187731103515e+24891', 23, 4);
    T('-8.59890336562226261918064947793192693726713188855e+3313480015007', '5.32904e+522', '-8.598903365622262619180649477931926e+3313480015007', 34, 2);
    T('-1.51381245861983553464602298007325820527582926298451825562194e-6799434564', '-641054364203707299925.7364569154967254384849818692181213', '6.410543642037072999257364569154967254384849818692181212999999999999999999e+20', 73, 3);
    T('-9.0967991305831981411003690790950832143e+5267111344', '-5e+43625706', '-9.0967991305831981411003690790950832143e+5267111344', 97, 6);
    T('-7.04e+103296850320225', '-2.19322323163809746829316221034479656456878355811515585e-214900', '-7.04e+103296850320225', 40, 3);
    T('-3.328432812839061080e+22', '-441580027162921237819007240060901190428829950454.2418', '4.415800271629212378190072067765730620382191504542418e+47', 79, 0);
    T('1.142039437690482541383183925291749993e+114', '2.53604e+6440451', '-2.5e+6440451', 2, 2);
    T('-7.44323230013491747296529981126801870761364712837652e-6938081758571', '-7.0198e-8852331596090', '-7.4432323001349174729652998112680187076136471283765199999999999999999999999999999999999999999e-6938081758571', 92, 2);
    T('-3.79e+4174870', '6.4408689180042662890604799931595855076726523883e+896', '-3.79e+4174870', 34, 5);
    T('2.1972718411205892573946671808653883609824178443e+530881', '1.8675952586748307403135572245613796394424166e+178295', '2.1972718411205892573946671808653883609824178443e+530881', 47, 6);
    T('-5.857e+85233953120', '-5.9778973141548494198531950869429682490526266653e-369402102223', '-5.857e+85233953120', 99, 3);
    T('1.8060213646773393e+242', '8.40310702179e-4480940', '1.8060213646773393e+242', 32, 5);
    T('-474319276012347933.33', '3.309717764858145002478330543571780600358e+989106607', '-3.309717764858145002478330543571780600358e+989106607', 72, 6);
    T('-3.121822480617089169644566e+99147572162727', '6.0838883897544719607712246010760070e+13665965222282', '-3.121822480617089169644566e+99147572162727', 29, 1);
    T('3.0387696407592712236305505852185628435980240715996514039493e+70764113', '-3.40546482295950902953819885e+479171132366475', '3.41e+479171132366475', 3, 2);
    T('-2.605637516987472739801163683972e+116937017649', '-6.65093302416084684621328121024555e+1764967231', '-2.605637516987472739801163683971999999999e+116937017649', 40, 2);
    T('-9.54724496703162038803640734282729960799e+447', '7e-64194058101', '-9.547244967031620388036e+447', 22, 6);
    T('-5.961996818176678114116307395339041632427300793858243997e+800430851786945', '-729325375.587', '-5.9619e+800430851786945', 5, 1);
    T('-7.72614964112916926463557668648047255872e-94699175', '-1.7710990993148984822e+92', '1.771099099314898482e+92', 19, 4);
    T('-8.0412899400e+101', '-1.69933136047070872753392968918e+80042180474920', '1.69933136047070872753392968918e+80042180474920', 57, 6);
    T('-3.1630433271271793251261303869019638e+769688', '-7.98200017009262693303891515887031756267e+983286136706390', '7.982000170092626933038915158870317562669999999999999999999999e+983286136706390', 61, 1);
    T('-8.66340533952700595026508743e-87739172', '5.10740011740762834454355811524933307692842557070149337e+6082650301', '-5.10740011740762834454355811524933307692842557070149337e+6082650301', 62, 5);
    T('-6.04448285171075922e+4436191836', '-1.8281022524633420325789897299072977643675509675047e-56718289222266', '-6.0444828518e+4436191836', 11, 3);
    T('5.53541744890454786e+721', '-7.15268995864952760779814423667529e+871186561789', '7.152689958649527607798144237e+871186561789', 28, 4);
    T('-1.0150941666196404e+559423041', '9.1447991e+20659', '-1.0150941666196404e+559423041', 76, 4);
    T('7.45702901949692850e-371926496', '-8.8989633546595885119707128e-763917165', '7.4570290194969285e-371926496', 95, 3);
    T('-8.9623983147455423739966078933902340680536630e-637', '-7.8944242e-15166191', '-8.96239831474554237399660789339023e-637', 33, 5);
    T('7.57802275673918353866316e+27', '-6.4300888813860431210229588945122e+105', '6.43008888138604312102295889451220000000000000000000000000000000000000000000000757803e+105', 84, 0);
    T('1.6408263411759822000151192155422906151393240381e+3003', '-4.5905968653137662972712e+143365', '4.5905968653137662972712e+143365', 87, 4);
    T('5.9814595528297312475583499969429234778243290373173917e-62551998912', '173.446254806901388529169', '-1.734462548e+2', 10, 5);
    T('6.49385925900273400372163986711798512099022092e-77405620', '-4.980588561839306280928964386629564675723057600396432e+708', '4.980588561839306280928964386629564675723057600396432e+708', 96, 1);
    T('-9.3346869592772951600872216248640706950219654961250786771e+435994', '-2695256639389948.089', '-9.3346869592772951600872216248640706950219654961250786771e+435994', 92, 4);
    T('-4.17837698450276896267239767802e-7505577829138', '-8.20253720186573742694208933210356977979026260944057361157e-48', '8.20253720186573742694208933210356977979026260944057361157e-48', 82, 5);
    T('2.77853174920223099391466e+6327914799', '-9.85915763398078364275599036400036979054656e-235', '2.77854e+6327914799', 6, 2);
    T('-1.39278690e-7239276819', '7.417873066197664838683973e-3809', '-7.417873066197664838683973e-3809', 31, 2);
    T('2.9e-2083', '979668222357869620601011325326565891.182', '-9.79668222357869620601011325326565891182e+35', 66, 3);
    T('-7.740e-93731346769694', '-9.341618517323816832e-8732236922199', '9.341618517323816832e-8732236922199', 30, 2);
    T('5.45451469445161368939154e+98558639', '9.8e-565002052929', '5.45451469445161368939154e+98558639', 62, 0);
    T('-6.48627422263105192582382593e-151825826', '7.64896985601111641136861973049912850531e+245703831552', '-7.64896985601111641136861973049912850531e+245703831552', 90, 4);
    T('9953741894.7377621', '-8.90202470e-564522436880', '1e+10', 1, 4);
    T('6.68536280e+6538450', '-3.67295552990277434196262999609759521919437218302278908639491e+487431', '6.6853628e+6538450', 30, 3);
    T('8.14819005319031056257624478436e+4019315', '2.50e+81061', '8.14819005319e+4019315', 12, 3);
    T('-5.496581643374427e+2179', '-6.073708435129526776594291273e+938', '-5.496582e+2179', 7, 6);
    T('5.06048847439198312633518635589005734e-69207295097387', '2.06522497633077500064700071120498405392730276631967420536593e-518629265662723', '5.06048847439198312634e-69207295097387', 21, 2);
    T('-6.4617054152831629927023859481322501806070e-1547860357839', '-8.478224622227711360575841260564307e+211014968', '8.48e+211014968', 3, 4);
    T('5.62954368142884853362300086080042296491005e-639027', '-85544262618220969230231892983514413', '8.5544262618220969230231892983514413e+34', 47, 4);
    T('-8.939827890311835382230418173183956e+3004', '-4.3404054e+463094816', '4.3404054e+463094816', 35, 0);
    T('-5.0076350039106078949446671507976989774603136200e+55', '-3.632248653075e-514116560', '-5.00763500391060789494466715079769897746031362e+55', 96, 0);
    T('-9.7415427327302942825356152203759e-8569985190', '7.90856414080646079822975057302485530346197345e+50878001713540', '-7.90856414080646079822975057302485530346197345e+50878001713540', 58, 5);
    T('1.385106990564435935221379e+440745372978', '-2.35893459678842442827448787729594015613450598195930602e-42971206074', '1.3851069905644359352213790000000000000000000001e+440745372978', 47, 0);
    T('8.29676879427201220488983416951651e+2122410022', '-7.6091253203e-533', '8.29676879427201220488983416951651e+2122410022', 50, 6);
    T('-3.4349115386e+9440', '-3.004656332e-80023426302', '-3.43491153859999999999999999999999999999999999999999999999999999999999999999999999e+9440', 81, 2);
    T('-7.5174325038779901460752995052566e+30539640', '-8.434394e-321', '-7.5174325038779901460752995052566e+30539640', 63, 5);
    T('-5.9541439808652828665592030271904e+59981516236457', '3.710661399594878783245405004533868188415e+9830755', '-5.9541439808652828665592030271904e+59981516236457', 95, 6);
    T('-7.0901e-43', '-8.67e-6678230554', '-7.0901e-43', 48, 6);
    T('7.57108575902904584088003794319861423936988065023867781172e+41766', '9.6e-513148071578993', '7.571085759029045840880037943198614239369880650238677e+41766', 52, 3);
    T('-2.8221904694240614803308830182345580005707491600e+75443882811', '-7.849e+68300', '-2.8221904694240614803308830182345580005708e+75443882811', 41, 3);
    T('2.389990157776520744891434271126832387884e-322869804', '-8.1336953812698114794795311850055656268113059819862061170689e-2995276351', '2.38999015777652074489143427112683238788400000000000000000000000000000000000000000000000000001e-322869804', 93, 2);
    T('6.67893724954865653706433341096331129273289873e-87931313895616', '-9.6806116e+13228', '9.6806116e+13228', 56, 1);
    T('3.26625e+3680830348', '-5.419811243924007064924175756939447542381e+52', '3.26625e+3680830348', 62, 4);
    T('9.8502737302690e-28147', '-3.707512494e+306627440987166', '3.707512494000000000001e+306627440987166', 22, 2);
    T('3934736777662627858177963650941807391943192.1348', '-1.0716052052859265455352313656948130e-137231768602', '3.9347367776626278581779636509418073919431921348e+42', 100, 4);
    T('-1.763496739293531192760420377572261035934e+12864675691', '-6.287200425e-95756261797298', '-1.7634967393e+12864675691', 11, 0);
    T('1.3420464330e-1236', '-4.4867257310302593615870581610e-44934613489637', '1.342046433e-1236', 68, 3);
    T('7.89788656e+2224946841', '-9.22590552244083455010e-9750281', '7.89789e+2224946841', 6, 2);
    T('-5.65672338956340561299947105393611643951459633984977e+81313991794721', '6.106118716256033651228073224775e-721219148482927', '-5.65672338956340561299947105393611643951459633984977e+81313991794721', 57, 6);
    T('-9.64874591432455839e+7605807', '2.66058130862115690586288093e-251291', '-9.64874591432455839e+7605807', 98, 5);
    T('5.7158855998911170885124995983670e-26765198769', '-8.56726229496126734709270e+4158303', '8.5672622949612673470927e+4158303', 77, 3);
    T('-7.3170077406137396684445675891598445220564468395154126e+62', '-5.469216740204681386339924908901369e-7937', '-7.3170077406137396684445675891598445220564468395154125999999999999999999999999999999999e+62', 86, 1);
    T('3.61030867615850760569e+4859652577', '-3.59881106026700747091001417450696673298553706241577764e+790647', '3.61030867615851e+4859652577', 15, 2);
    T('2.5381229230714538834575832479265944767148637964036288e+31374', '17842.25001', '2.5381229230714538834575832479265944767148637964036288e+31374', 69, 0);
    T('4.239466192948972198016081983704888e+525916315152', '1.70020114517090574942260492838492728090e-5837867605', '4.239466192948972198016081983705e+525916315152', 31, 2);
    T('1.96e+75', '1.3260892988447537120436513415171288340834349335023823e+133', '-1.326089298844753712043651341517128834083434933502382299999804e+133', 95, 1);
    T('3.1855019171634624785757585278508534e-91605779648', '2.1205761890807561298109383741890334742e-2500375', '-2.1205761890807561298109383741890334742e-2500375', 57, 3);
    T('8.63711388881812457e-299', '-7.5862145023029673138023193492e+9339100111', '7.5862145023029673138023193492000000000000000000000000000000000000000000001e+9339100111', 74, 0);
    T('-2.297381536779345881697180108964955e+3047381', '-8.49374124406936563e-575263560545965', '-2.297381536779345881697180108964954999999999999999999999999e+3047381', 58, 2);
    T('6.82398882e+4059815', '8.3165887728520225909923e+25', '6.82398882e+4059815', 99, 2);
    T('9.89577371210611e+31443949978218', '-7.025017521572587406202942038935425710543489e+534534911', '9.89577371210611e+31443949978218', 92, 5);
    T('2.468931e-92', '9.07212563685348278253342655222146e+27535098', '-9.0721256368534827825334265522214599999999999999999999999999999999999999999999999999999e+27535098', 86, 2);
    T('9.42513511059156950e+356141201372', '7.02344702189819227883114261087234639030540185e-300087034512152', '9.4251351105915695e+356141201372', 26, 2);
    T('5.44755173667668618485e+999397452747', '-2.13236545526183512759528e+85174', '5.44755173667668618485e+999397452747', 63, 1);
    T('7.060333313474314849236212577788629702829e-4272211698566', '-5.540483303044593734794133073013808612271096e+3470', '5.540483303044593734794133073013808612271096e+3470', 87, 1);
    T('9.7095339131503084732019845911012951e+81', '8.8244767428608653017181235170485248416061977e-98193955', '9.70953391e+81', 9, 4);
    T('3.58631284751661073438444819731291238900802134298e+60122622599', '1.581959193851e-998173', '3.58631284751661073438444819731291238900802134298e+60122622599', 87, 2);
    T('115730167681159271016691719434.3960276', '-3.885339e-121', '1.15730167681159272e+29', 18, 0);
    T('-8.45274995762868968695984759901705138238e+568495680746060', '-1.0245628873520764479748526269879109368000818186717517502374e+2023', '-8.45e+568495680746060', 3, 1);
    T('-6.54320111494e-282305035935656', '8194848.3969', '-8.1948483969e+6', 84, 4);
    T('-4.240281907843013938253375421845749962033712888831064495e+55', '-5.95123382666446830198591535734439775557433277956203751532107e-79990', '-4.24028190784301393825337542184574996203371289e+55', 45, 3);
    T('-2.8587605457928474684106582928769112202149489240869635456214e-37', '6.2968065739979064850780221171101352e+6829535', '-6.2968065739979064850780221171101353e+6829535', 35, 3);
    T('8.1543936906646002004463981446223784439608999661886384373e-1424862055760', '-1.6374062053440e-184396', '1.637406205344e-184396', 45, 6);
    T('6.722688950977280256458840514902699015674605087355627e+5436075', '-2.94251518359970930390e-31186', '6.72268895097728025645884051490269901567e+5436075', 39, 5);
    T('-7.366752202e-4100749253612', '8.79047668956634462767077116464e-2474763573140', '-8.7904766895663446276707711646400000000001e-2474763573140', 41, 3);
    T('83818780457428553759991910241725922294619773977634930', '-8.748036211422125534120765014077782464066524e-11647', '8.3818780457428553759991910241725922294619774e+52', 44, 6);
    T('4.84443549480e+769751354906529', '-8.19470140173224970275561850187e-502677831156', '4.8444354948e+769751354906529', 34, 6);
    T('-7.29691811022254581666199402934284255937e-736885366053974', '36566325374735569359263634273198268010422832988762724.362221', '-3.65663253747355693592636e+52', 24, 2);
    T('-4.65454402294485257474897895176149292456418535263474218699e-24', '4.21364488329219386936301779480612356666892427351039905489e+59547551546637', '-4.213644883292193869363017794806123566668924273510399054890000001e+59547551546637', 64, 3);
    T('1.9186267194009101108004446808054958411014746388344110310e-316727303410285', '0.092281093294910494870041959888237210543', '-9.2281093294910494870041959888237210543e-2', 69, 0);
    T('-3.8968495496412949895651702843092800548894971071901e-7', '-20365.12594', '2.0365125939610315045036e+4', 23, 2);
    T('6.788024186212109349407753178080172149504e+45', '-5.1334696535887415445225015e-484', '6.788024186212e+45', 13, 3);
    T('-4.5564e+73', '7.18609269269593612080048657119991074609963171919641356e-4251662864944', '-4.5564e+73', 84, 5);
    T('-2.408858943262451902394365627743192407817580304331268980873e+543', '7e-243199117979', '-2.408858943262451902394365627743192407817580304331268980873e+543', 80, 2);
    T('-9.193067523e+45998181573330', '4.1e+289737', '-9.193067523e+45998181573330', 53, 6);
    T('-5.1308171576441930890511658030714174e+62852219', '2.8401297030349323364042921655818643637387535157e-5806128', '-5.130817157644193089051165803e+62852219', 29, 1);
    T('8.1534192388130628914417287303112926870453147631085132836e-30724', '-9.1818048e-48217628973', '8.153419e-30724', 7, 5);
    T('3.7131267222781873411976778899986178004901391057193e-4097283450852', '-4.195231518165945700925115103376053e+9870767534', '4.195231518165945700925115103376053000000000000000000000000000000000000000000000000001e+9870767534', 85, 2);
    T('-6.0183771166346e-920409546919929', '-3.4680114316933324221217499681065837599624629e-7963626779', '3.46801143e-7963626779', 9, 1);
    T('2.4e-96462140', '-9.68087317892508217016745359479964310136259313478124010337569e-20430102', '9.68087317892508217016745359479964310136259313478124010337569e-20430102', 88, 6);
    T('6.64157552941887505145879703219674e+51', '3.7853755434949332660271342616378387159560456662857100e+31038', '-3.785375543494933266027134261638e+31038', 31, 6);
    T('-5.10344819084365e-447810', '-2.692e+998213352457', '2.692e+998213352457', 38, 6);
    T('-1.93197326342231387617887342402388561523108837288610441581e-8832094609405', '-4.5e+360', '4.5e+360', 57, 0);
    T('-9e+472087839525896', '-7.855196525639982421786904120336697836e-3972170', '-9e+472087839525896', 36, 6);
    T('-3.3920245350148e-765306682', '7.6449473942378809496460867408333e-157718688230952', '-3.3920245350148000000000000001e-765306682', 29, 3);
    T('3.0850592422098971022489345593772077077506524e+61105', '29045655394539797869512963.749222821806873511383', '3.08505924220989710224893455e+61105', 27, 1);
    T('-7.4801506846972390e-962729333235237', '-2.40725764021629017986294114439371e-95106485940', '2.40725764021629017986294114439370999999999999999999999999999999999999999999999999e-95106485940', 81, 1);
    T('1.373485241301e+716230', '4.3275263129009207153596871751043701496049961e-11914496', '1.3734852413009999999999999999999999999999999999999999999e+716230', 56, 3);
    T('-626101660124819027107961830952264098606303122795908356', '3.091247977480151471494392282824278732875868370557e+816359843433', '-3.091247977480151471494392282824278732875868370557e+816359843433', 55, 1);
    T('-7.1846012831466038428113545769124162e+854105337888', '4.37416251072619024784107842860873e-7044', '-7.184601283146603842811354576912417e+854105337888', 34, 3);
    T('0.690654', '8.8080710e-85000', '6.90653e-1', 6, 3);
    T('1.53463204e+42733', '8.52300335104563993e+7508816883', '-8.523003351045639929999999999999999999999999999999999999999999e+7508816883', 61, 1);
    T('-1.43515782004699418397991459594594075861461418e+50', '-9.987284545334280726892566444660913682788167e+118079187036', '9.987284545334280726892566444660913682788167e+118079187036', 99, 6);
    T('-5.7831467148073744245200236609695405907e+62', '3.81279664433781610315664064439838016400e+691219380', '-3.81279664433781610315664064439838016400001e+691219380', 42, 0);
    T('3.0011151548656479647998798880664804863e-26277', '-4.94702982891e+359', '4.94702982891e+359', 19, 5);
    T('-7.288844430232215182875625001084197813344640130e+80388', '8.7911751488834759650189705165298816484211221189470e-698888298', '-7.28884443023221518287562500108419781334464013e+80388', 82, 2);
    T('5.904572860815367738225565083065e-5209691959783', '4.94143335698918234776334542148446261528128322427e-1098982841554', '-4.9414333569891e-1098982841554', 14, 2);
    T('-8.06409566193134561667863e+3725086', '-9.007354314239529894331513894678525491911959e+78347700', '9.0073543142395298943315138946785e+78347700', 32, 1);
    T('-1.4165310778748009096e-57522', '5.2513786425044553485131208287314106182e+5338850', '-5.2513786425044553485131208287314106182e+5338850', 73, 1);
    T('2.58248651990519065094514690856505942050148093e+78207655480', '3.2272104755e-486313208581', '2.582486519905190650945146908565059420501480929999999999999999999999e+78207655480', 67, 3);
    T('-7.999270225172141e+7342898', '9.8812827838834563318591566589464382030547996935724611979e-4920', '-7.999270225172141e+7342898', 26, 1);
    T('-690165089733775723867690260584290249064300726893578', '-3.533364344e+260572881', '3.533364344e+260572881', 53, 0);
    T('3.628312607511e+8499707', '-8.3945373156711975e+9748162489364', '8.394537315671197500000000000000000000000000000000000001e+9748162489364', 55, 0);
    T('-5.633849990073875767616199421e+33', '-5.5544879884e+2397084875', '5.554487988399999999999999999999999999999999999999999999999999999e+2397084875', 64, 3);
    T('-8.246203923414052292549547369626665326041265400972392093845e+484714313221', '6.17948216909099819398e-80833848844038', '-8.2462039234140522925495473696266653260412654009723920938450000000000000000000000000000001e+484714313221', 89, 3);
    T('1.235437450350064388004183577572139968214e+10758890363', '-1.7886562754289254227705e+369466778025', '1.7886562754289254227705e+369466778025', 43, 6);
    T('-1.995875558532037374657914906e-7510674669911', '-5.02268774822000362920431e+453451', '5.0226877482200036292043099999999999999999999999999999999999999999999999999999999e+453451', 80, 1);
    T('7.28429561638177256414382114251864088297782786105661621027e+129', '-7.761805207375439e-457060311187', '7.2842956163817725641438e+129', 23, 4);
    T('-5.667034126103596688416918e-87543', '8.98245267609042513543437325110840776456895252780950117e+57', '-8.9824526760904251354343732511084077645689525e+57', 44, 5);
    T('-3.098237946647039826401988181807732370804186936695443e+9034267', '9.4452191578426298625590585641211962656280943103e+79474725', '-9.4e+79474725', 2, 1);
    T('-1.5827615304426027423722e-955909', '5.9613200695892490e-454687102', '-1.5827615304426027423722e-955909', 67, 2);
    T('7.849353742687831335084790939281683424968907730189341619e+625', '-5.2247335174e+18636', '5.2247335174e+18636', 53, 3);
    T('2.6438624354447901879073099582563325641435e+804', '-7.3734923020642091817880240356e+1652', '7.3734923020642091817880240356e+1652', 85, 6);
    T('4.55629879773447354535426760402225185373e-228803181678153', '-5.315784992170432653941e+1451203', '5.315784992170432653941e+1451203', 90, 1);
    T('7.58548821581755632188051623234520636816038535e-507', '1.6711111216665296255e-458460', '7.5854882158175563218805162323452063681603853499999999999999999999999999999999999999e-507', 83, 3);
    T('-1.63681708e-2927321', '5.1480861821941944262015062664864252940777964204038924125e+64', '-5.1480861821941944262015062664864252940778e+64', 42, 0);
    T('1.422242835369977570335575486795046147363543e+1561095', '1.399148138067901548792590155324941088140e+42', '1.42224283536997757033557548679504614736354299999999999999999999999999999999999999999999e+1561095', 87, 3);
    T('-3.2405284424970e-30736683362', '-2.83e+7', '2.83e+7', 15, 5);
    T('-8.348900e-64693', '-1.57421378048935303e+462', '1.57421378048935303e+462', 58, 6);
    T('9.44190178e+16146894', '7.75848005264338287142828754566198265776740372173e+113180', '9.44191e+16146894', 6, 2);
    T('-61989654073593828959784968290348509924963183629475.602', '9.6098306215244175008251108835523364232844535376e-780', '-6.19896540735938289597849682903485099249631836294756021e+49', 54, 3);
    T('2.28e+7380', '7.60865082367363344743500918622998767726e+634171', '-7.60865082367363344743500918622998767726e+634171', 53, 4);
    T('6.262270646072501351899238e-4615', '7.24244460901179763360594938039842996243032011766e+55', '-7.2424446090117976336059493804e+55', 29, 3);
    T('2.49684528911840893471126535004288877468760887303088481327e+37023237', '-3.482937735949657270e+260929', '2.49684528911840893471126535004288877468760887303088481327e+37023237', 61, 5);
    T('-7.9111106376e-3042', '-8.385500793883556269647702701603483288710722e+49', '8.385500793883556269647702701603483288710722e+49', 96, 5);
    T('9.65e-9937143051756', '1.41445962435759613185978167606401849749451071996248e-823312', '-1.41445963e-823312', 9, 0);
    T('8.5155703509161276e+25096819', '8.42285362371238724654442229109282984704421884e-9051', '8.5155703509161275999999999999999999e+25096819', 35, 3);
    T('4e+513454951', '7.69611391446e+249440', '3.99999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+513454951', 96, 3);
    T('-96046935697433241389470275338.94022', '-8.065890e-37063737', '-9.60469356974332413894702753389402199999999999999999999999999e+28', 60, 1);
    T('-8.0114617602510417032136797968e+6127562344730', '4.343240124570657699517564184810e+14404100961478', '-4.34324012457065769951756418481e+14404100961478', 53, 5);
    T('3.74822531489858163e+712399688503668', '5.751121e+82825445271', '3.7482253149e+712399688503668', 12, 4);
    T('9.751714712868230594281508957008929315521e-2431772530862', '3.400987034010009996775285653813060494540397576913001e+9520993243', '-3.4009870340100099967753e+9520993243', 23, 4);
    T('3.4094242840157352663950634838945598922078427782237555e+66199', '-6.3247883829346720703632226421422206666853e+5872', '3.4094242840157352663950634838945598922078427782237555e+66199', 88, 4);
    T('5.62434990290150035996927481905859583290e+869265610774', '6.032160350474870794110937909255499034496153929069e+1271698826', '5.624349902901500359969274819e+869265610774', 28, 6);
    T('-7.73628561916387051119475577397788160087092480717293e+8845982049', '3035727921912712660630917851194083346353248322600201001202.2', '-7.73628561916387051119475577397788160087092480717293e+8845982049', 82, 4);
    T('-4.69134840424265332398633124984676677994e-109', '-1.594153942865595887127697888326670513746596665e+37332169743', '1.594153942865595887128e+37332169743', 22, 4);
    T('-2.83367813111894e+73763', '1.455721480283825198365e+442', '-2.83367813111894e+73763', 99, 2);
    T('-3.5045514024821213268119207037772467616e+793610475437', '6.842745844826557417747634582271135e-5030912047429', '-3.5045514024821213268119207037772467616e+793610475437', 64, 1);
    T('4.981640399353525002340089152796864191862874822e-83691', '2.1687148e+84528', '-2.1687148e+84528', 92, 5);
    T('-9572966182946181985296886670501021027955172139317731947.844', '-1.06684777378852378306209101323794117252e-1332737441', '-9.5729661829461819852968866e+54', 26, 1);
    T('-6.16573301671e+498882', '-5.887e+484952673406', '5.887e+484952673406', 18, 6);
    T('4.544352271842173737222846717347906502955011281515777e-723253', '6.92086496e+31870929272447', '-6.92086496e+31870929272447', 14, 5);
    T('716795983594383746703960211754273439899808899852097.42', '7.836526944347969402772102252609208893304292325613158439714e-95899', '7.1679598359438374670396021175427343989980889985209742e+50', 84, 4);
    T('1.487404065328307901416838560377208998213e-51486554792', '-2.080155e+102', '2.080155e+102', 17, 1);
    T('5.768810754240542093439431057002669706829824104753885825e-686874', '7.53908165e-9545632468372', '5.768810754240542093439431057002669706829824104753885825e-686874', 72, 2);
    T('-9.92921243121e+56829597151', '-9.977000390908140961849312766150241096e-28', '-9.929212431209999999999999999999999999999999999999999e+56829597151', 52, 2);
    T('-4.502e-824461079731458', '5.77e+305517', '-5.77e+305517', 72, 5);
    T('2.90229315184306894621e+81613353', '7.95e+5774419529', '-7.95e+5774419529', 81, 3);
    T('5.40392780911485257e+37606', '3e+336', '5.40392780911485256999e+37606', 21, 3);
    T('7.977102850840097234450400134989992382e-349425773779118', '2.691901236786759622281261756e-57671100975725', '-2.6919012367867596222812618e-57671100975725', 26, 0);
    T('3.9798024963105320449694832445e+22059', '-3.2177465122625675817555e+8313263854619', '3.2177465122625675817555e+8313263854619', 60, 1);
    T('7.558736870e-188', '-4.24747262403819651921122536e-985993', '7.55873687e-188', 77, 4);
    T('-1.07807872380623201399147006314424796527552722404e+40128743', '1.8512797025278382418266037e+4240758', '-1.07807872380623201399147e+40128743', 24, 6);
    T('-4.39084873109433315199816900683576537552677738e-45211023078060', '-8.4902973871196453728179794238610e+7742288714', '8.490297387119645372817979423861e+7742288714', 41, 2);
    T('2.7634748026138444240730774448299854e-7056', '-9.05704580764402691634e+440356735', '9.0570458076440269163400000000000000000000000001e+440356735', 47, 0);
    T('7.436592975660782e+863484358', '-6.856620298218448405990479963e-736550906', '7.436592975660782e+863484358', 30, 5);
    T('2.512652065311193013500e+19792140480728', '-1.3953231825385387162513866818200e+401', '2.5126520653111930135e+19792140480728', 42, 3);
    T('-8.3239293279226184107906132714e+2631768', '6.85e-524237455048337', '-8.323929327922618410790613271400000000000000000000000000000000001e+2631768', 64, 3);
    T('6.341134088753e+3381', '9.92310638567811734019822114553920133216169318600083145972714e+89561', '-9.92e+89561', 3, 2);
    T('-9.5051498181532501603320219577916505954170768988281057861e+91905292', '7.712e-449404195999691', '-9.505149818153250160333e+91905292', 22, 0);
    T('-7300242217081638159336118753.8', '1.2452854981296875789336757841753978113475925026297180146445e+8558', '-1.2452855e+8558', 8, 3);
    T('5.9230180447e+16783', '-2.7566789115321e+294823254414080', '2.7566789115321e+294823254414080', 91, 5);
    T('-9.50985842707340603685999858274081672686e-611', '-2.742174628687806787216814499287624e+1982984391', '2.742174628687806787216814499287624e+1982984391', 35, 4);
    T('-5.67427327982531467137063682469474e+1291830920', '6.6877891931611115776467958e-971250700766', '-5.67427327982531467137063682469474000000000000000000001e+1291830920', 54, 0);
    T('3.52057737242732917258026017095044150393325240442420105962032e+544905189196804', '3.28084798045777195542184913853425170906281860153659559865e+122', '3.52e+544905189196804', 3, 1);
    T('7.9999080529864186308541190639271185578185973e+48', '9.76375143652773509783603721444028913766082514e+6363062391', '-9.76375143652773509783603721444028913766082514e+6363062391', 49, 6);
    T('-9.511053749133e-8403', '5.91270226267151831e+3520', '-5.91270226267151831e+3520', 31, 6);
    T('-1.312e+912083615', '-6.4524544620197455414e+9781', '-1.31199999999999999999999999999999999999999999999999999999999999e+912083615', 63, 2);
    T('-2.508168819144490822259161551067764e+10001529666436', '5.695430901955012262354993565605294454671990e-29425336382', '-2.508168819144490822259161551067764e+10001529666436', 87, 4);
    T('-5.852445115e+630452', '-8.76971895705347664e-85032602', '-5.852445114999999999999999999999999999999999999999999999999999999999999999999e+630452', 76, 1);
    T('5.4616014595605396793305522165362232625430057459185e-899616028', '9.299315030432756855072687658519591572156538741544e+60256', '-9.299315e+60256', 8, 6);
    T('9.853962410680262712062598378490e-746198', '2.32303229743e-1226837', '9.8539624106802627120625983784899999999999999e-746198', 44, 1);
    T('-9.5528193249698911074011014468870818398824e-607', '-8.4830221750496e-4028952', '-9.55281932496989110740110145e-607', 27, 0);
    T('5.35023719579948394889803110262060110740358004849369425822e-37318546303469', '9.439821406217427596094562649168861268197053007330867696e-163562873', '-9.439821406217427596094562649168861268197053007330867696e-163562873', 88, 5);
    T('-1.1849679803222698747825252597617191200470921924e+9128445760', '7.58987740837285509298288472338228953198301e+115', '-1.19e+9128445760', 3, 3);
    T('-1.140226775267413789456854841790953864351e+1129982', '-2.529837546201415e+581', '-1.140226775267414e+1129982', 16, 6);
    T('5.2477537669491e+3540682', '-6.61273948074065351015113848189549477461759533360787e-6302018', '5.24775376694910000000000000000000000000000000000000000000000000000000000000000000000000000001e+3540682', 93, 2);
    T('3.7926e-3775301941', '-9.800487544083734071957144857900698281186408014288339e+5661125', '9.8004875440837340719571448579007e+5661125', 33, 5);
    T('-2.8979844710710544020439947e+1547', '8.580742e-758698439681', '-2.8979844710710544020439947e+1547', 63, 2);
    T('-1.8441008792550364302029367595322590726075260519090647e+39457029372060', '6.66810486829480862059883607822e+352271', '-1.84410087925503e+39457029372060', 15, 2);
    T('-9.240090484599404213310495e+7161619233933', '-4.95853197565493003885965974712876850920668374132921717e-67967405682115', '-9.240090484599404213310495e+7161619233933', 70, 0);
    T('-1.86881264793589200069920779625016600403e+89700284585', '4.124358290680343170609151481446989168493035566439124622000e+88', '-1.86881264793589200069920779625016600403e+89700284585', 83, 2);
    T('3.95197145982e-74557538', '-4.737825034567e+288893633556096', '4.737825034567e+288893633556096', 92, 1);
    T('4.4413782262772729691e-6334124', '-9.66271813577e-243805814993', '4.441378226277272969100000000000000001e-6334124', 37, 0);
    T('-7.056865861785390747862179423e+97615306', '-4.94966356096120e-2281715509379', '-7.05686586178539074786217942299999999e+97615306', 36, 1);
    T('8.707773089712106543033891741113383585e+165', '4e-696624', '8.707773089712106543033891741113383585e+165', 40, 6);
    T('-13439428799102792002.663824', '8.8235091024138117043284581349853613734013419e+9880', '-8.8235091024138117043284581349853613734e+9880', 38, 6);
    T('3.88708468483084291074e+490', '9.8994760718873003345665067183e-606', '3.88708468483084291074e+490', 47, 0);
    T('2.819658299735e+54', '-4.74791867097392552726660185097897e+97206020374420', '4.7479186709739255272666018509789700000000000000000000000000000000000000000000000000000000000000001e+97206020374420', 98, 0);
    T('-6.451454235814650077385e-596530', '-5.71155928937682728580668451795832260e+1219', '5.711559289376827285806684517958e+1219', 31, 1);
    T('-8.49963190006905360e-5141822', '-9.5859087745673771e+49356246453', '9.5859087745673771e+49356246453', 29, 2);
    T('3.0565241612e+2765465015990', '9.73602577444717116340419409364357707586018267671224e-1747121086', '3.0565241612e+2765465015990', 81, 4);
    T('-3.1542652323986078505221194683193042087780e+704324745312241', '-7.6969821635437477723434318e+5442', '-3.1542652323986078505221194683193042087779999999999999999999999999999999999999999e+704324745312241', 80, 2);
    T('-8.031690518186831487e+57605543', '2.9376967172847215822081319566e+416847717', '-2.9376967172847e+416847717', 14, 4);
    T('-8.9665564362566013539641662719966841067462926700641e-44011', '-6.17402202304618450085032839300339244724645184294883483978982e+111', '6.1740220230461845008503283930033924472464518429488348397898199999999999999999999999999e+111', 86, 1);
    T('3.0477485565830849424420268268127589314e+758874824168', '9.018724407758234902595522606213581854225738838976160e+7726964', '3.04774855658308494244202682681275893139999999999e+758874824168', 48, 3);
    T('7.07423981369121966750e+1734747', '-8.8739659991432072914776435042357017621960573085e+19085856735190', '8.8739659991432072914776435042357017621960573085e+19085856735190', 75, 1);
    T('-8.2654915699753999854581727688575e+4113665885630', '-9.5417277311483294470987821144822866248840675e-235298995', '-8.2654915699753999854581727688575e+4113665885630', 78, 6);
    T('3.372351895324160027583005055372313219826653944e+58762680', '4.5715970262161486950904625311598546082665607326433957172e+272964733', '-4.5715970262161486950904625312e+272964733', 29, 5);
    T('-4142397184585532200417371195937791245984686218', '5.194163012097309239944424757e-549883', '-4.15e+45', 3, 0);
    T('6.234617573954241521253291017408263250398453e-5490601629699', '9.78184843630983e-23317470', '-9.78184843630983e-23317470', 25, 6);
    T('-5.21433941917484e+56522659', '6.726e-21215980', '-5.21433941917484e+56522659', 41, 1);
    T('8.908332209286679201698050853250572260837496164163e+625831446799', '354779237078085001577106610', '8.908e+625831446799', 4, 5);
    T('-4.4346061312229105834307296084273710529433103230867e-3151421980501', '-8.1256190696003193129050886181313e-679886634', '8.1256190696003193129050886181313e-679886634', 82, 5);
    T('8.587e+2075467', '-3.5894513040251972553446378525642126616120917030568654e+8165', '8.587e+2075467', 85, 3);
    T('-2.1316588342151005130818673918560107836852448e-1338296173', '7.5704639e+8541', '-7.5704639e+8541', 76, 4);
    T('-7.0313296210766841938035024334272095027e-3152835113888', '8.040949891230385449793854916928104620428234564311213e+3444834', '-8.040949891230385449793854916928104620428235e+3444834', 43, 6);
    T('-9.823562746718570786245699623472434292364999254e+905', '1.1976139917055978875952409227359e+901', '-9.823682508117741346034459147564707882364999254e+905', 62, 5);
    T('-5.01691080752e+3822434233548', '-8.73396241126952042148216898583032277849666781714366862961e+64', '-5.01691080752e+3822434233548', 95, 3);
    T('-9.7505319409148683229826e+321', '6.729656447091110544401706658617722578338618340e+4174501545883', '-6.729656447091110544401706658617722579e+4174501545883', 37, 3);
    T('-2.5486955428991338778721775618810e+3425', '3e+14610014620', '-3e+14610014620', 79, 5);
    T('-5.8772e-8469455', '-9.90099839586714981034014719246e-39685896854', '-5.8771999e-8469455', 8, 1);
    T('344840816967218909082080235644244591180328028372573.62379', '-9.92906894091057993011905271232425659175368894792851184e+3268174', '9.92906894091057993011905271232425659175368894792851184e+3268174', 58, 1);
    T('8.40031692237400438321833593104623708826834e-2581', '3.49538060681797425629543200574522404496702495450792072364422e+58958391', '-3.49538060681797425629543200574522404496702495450792072364422e+58958391', 99, 4);
    T('-1.56438910233385661264641113081622949931112460996e+655752432245', '-6.9808200582944786427699694930e+375480899', '-1.56438910233385661264641113082e+655752432245', 30, 0);
    T('-9.32699094320460290929057e+51211482022', '-2.32024750791141503850635945660820246640859501646564284678e+51839', '-9.326990943204602909290569999999e+51211482022', 31, 2);
    T('2.0047833843938219381786197497529e+846367', '-8.549329e+25', '2e+846367', 3, 3);
    T('-124048091747674701331407246353883625530260914272910.8991693', '2.397228545333495607703014670e+35', '-1.240480917476749410542617797034443958317279142729108991693e+50', 87, 2);
    T('9e+21233894709319', '-7.83620694301514312945417895283979081150498753e+127', '9.0000000000000000000000000000000000000000000000000000000000000000001e+21233894709319', 68, 0);
    T('-6.41920303458576591745146788390206880466393448380e+8832151060', '6.287454553454351850e-73076', '-6.4192030345857659174514678839020688046639344838e+8832151060', 73, 5);
    T('-2.190128803e-124462', '-1.446382854387306585945335147439662065e+586286010961', '1.446382854387306585945335147439662065e+586286010961', 66, 4);
    T('-9.3697581372744697025013827098483328836850326e+892992475234', '7.228695080329225952254634585903771895294182466737e-379947451', '-9.3697581372744697025e+892992475234', 21, 2);
    T('7.085198268569620149575839184e+28846976013917', '-7.59627134845423e-25370915', '7.0851982685696201495758e+28846976013917', 23, 5);
    T('2.50864914134885177520e+821397683312', '-8.39357e-98696627863', '2.50865e+821397683312', 6, 6);
    T('-1.84564611693232e-66', '4.89210387787697424262984232099086517024261969433714324e-23', '-4.89210387787697424262984232099086517024261987890175493323e-23', 57, 2);
    T('8.48155551145380e+885', '9042272649623029590914362552467.01032999', '8.4815555114538e+885', 48, 5);
    T('5.422601429429606323856298171877588e+77546697464121', '9.00293838598080253665092477760610593114694e+76264', '5.4226014294296063238562981718775879999999999999999999999999e+77546697464121', 59, 3);
    T('4.47046016944833540209277922720813387340881623e-36469', '5.63501327355132678304550064023747625291919127406594429850e+17964482718571', '-5.63501327355132678304550064023747625291919127406594429849999999999999999999999999999999999999999e+17964482718571', 96, 1);
    T('1.31238e-8342', '-1.03514288667941876103710692331823373e+1785', '1.03514288667941876103710692331823373e+1785', 44, 3);
    T('8.5032714891343543866610086115495606584665085256495e-50710549', '3.8820162987126011937323746559305240030e-22301', '-3.882016298712601193732374655930524003e-22301', 76, 5);
    T('9510451979911525138620512476601282901444876970513166231.910', '-3.4279534e-954676689', '9.5104519799115251386205124766012829014448769705131662319100000001e+54', 65, 2);
    T('-9.876413685890181988836204199088721e+37630693', '3.2302224509291847252873312576649814940482768034790305817e-714409206', '-9.87641368589018198883620419908872100000000000000000000000000000000000000000000000000000000001e+37630693', 93, 0);
    T('-5e-37630737983057', '7.743649166924723708874027390672563110878e-70512392', '-7.743649166924723708874e-70512392', 23, 2);
    T('-7.28e+437', '-2.28494427431671312343360780556321265439131593e+634', '2.28494427431671312343360780556321265439131593e+634', 98, 2);
    T('2.1631915595392913013202535330605095064541e-47', '8.3430642754247711020588655931644491e-6801651622', '2.16319155953929130132025353306050950645409999999999999999999999999999e-47', 69, 1);
    T('-7.0154800875179962e-185479822927486', '-4.7587756806815e-77176623967', '4.7587756806815e-77176623967', 65, 0);
    T('6.651911808691413277791e+450', '2.59810322390503978477e+98639442184', '-2.59810322390503978477e+98639442184', 97, 4);
    T('-1.911857249019903662376330829150034887327273772988018e+193', '4.70542232362066523225995985607135888617e-9332885450', '-1.911857249019903662376330829150034887327273772988018e+193', 98, 2);
    T('-1.289331629906147809919162e+77395753509709', '-8.1869187938228123439115282835985e+72974610640', '-1.289331629906147809919162e+77395753509709', 83, 5);
    T('1.5053432667926767037669039634120242626215641757e-80843286440', '-6.954e+786', '6.954e+786', 62, 5);
    T('-1.472691564521468335430440060941285037448197e+53540532843', '9.24009503357833984540541187392882703345e-1690138859', '-1.472691564521e+53540532843', 13, 2);
    T('-6.665021321181957019165e-958080807', '-3.205101247979932848947885572701887e-3040632393268', '-6.665021321181957019165e-958080807', 82, 0);
    T('5.948080730328559439445629706869731258167850092e-999939490003961', '5.45452634106728151742081049127902972140564258e-121973618647912', '-5.454526341067281517420810491279029721405642579999999999999999999999999999999999999999e-121973618647912', 85, 2);
    T('7.956683324183788079872886934894600589e-328231', '-4.704082e+60', '4.704082e+60', 49, 4);
    T('7.82231364478996937719513059e+30556', '-9.49532961250961103757478664977022236e+6106516', '9.49532961250961103757478664977022236e+6106516', 87, 4);
    T('-6.7265429966e+60', '-5.506510732728175560459181841448013382864809e+5020316100', '5.50651073273e+5020316100', 12, 0);
    T('-8.882087709727897386081124362856498058e-278781724991654', '-69451983284220615283133560427979224749824609759356.6', '6.94519832842206152831335604279792247498246097593566e+49', 84, 0);
    T('1.7398586323449679640272580300890939614463142250521e-17526329937', '5.758284724e+648674193375765', '-5.758284723999999999999999999999999999999999999999999999999999999999999999999e+648674193375765', 76, 1);
    T('-2.70784e+5760', '1.8718584535237868891391931283726e-109', '-2.70784e+5760', 26, 1);
    T('-7.4e+391707936132', '6.62680355649626947188139118146695e-490074950953', '-7.4e+391707936132', 37, 6);
    T('5e-244131', '-7.521168713274e+87905245', '7.521168713274e+87905245', 65, 3);
    T('3.123985036051144306236387499806579897706833747100914793280e-88076414', '-5e-9338', '5e-9338', 57, 4);
    T('1.9507215955207721532480881706286615010076386200342690358202e+4629346', '-8.60893016717500019221612289492958312566e-48854727506', '1.950721595520772153248088170628661501007639e+4629346', 43, 4);
    T('-1.5028947326762e+78961216638264', '7.207510960573197709553769615165581e+17136650', '-1.5028947326762e+78961216638264', 69, 5);
    T('-3932161397475951313789855761700017434.605378', '2.716432489978e+7531409', '-2.716432489978e+7531409', 57, 2);
    T('2.185047007739418100436886106e-2944158408', '9.459988710320125491e-6810579', '-9.459988710320125491e-6810579', 90, 5);
    T('3e+135617356003076', '-8.488563529325698771648e-62904293244359', '3e+135617356003076', 82, 3);
    T('-9.5281105702402827876794995539650029542900293e-51972525392837', '-2.2710408e-922029326', '2.2710408e-922029326', 17, 0);
    T('9.108397384e+278131080718916', '2.4294644381312705e+169', '9.1083973839999999999999999999999999999999999999999999999999999999999999999999e+278131080718916', 77, 3);
    T('2.55e-460648', '-2.78847018196038718519e-81', '2.78847018196038718519e-81', 88, 5);
    T('-1.63130928435656106864e-2173310625479', '-2e+2627', '2e+2627', 14, 0);
    T('8.3178636599835978394034923982250855320919501713677e-987017', '2.08494440549382e+793075983646', '-2.08494440549382e+793075983646', 27, 4);
    T('-4.373036e-821', '-7.5907142512668923572510306616484700e-863780875782', '-4.3730359999999999999999999999999999999999999999999999999999999999999999999999e-821', 77, 2);
    T('3.594873334117397730456749835291765738238143331477042e+180155879183', '-5.736e-459229', '3.594873334117397730456749835291765738238143331477042e+180155879183', 72, 6);
    T('7.34e-908209669162043', '8.84977e+306604', '-8.849769999999999999999999999999999999999e+306604', 40, 2);
    T('1.4978477781682677734893518211898984635e+6132', '-4.148881821635251606965075325183784928064490517223316347751e+92968537472', '4.1488818216352516069e+92968537472', 20, 1);
    T('9.95624681679554448525475024563794548815828327463237547e-7247802550646', '25121434200860933.94666141574009211659032482659', '-2.51e+16', 3, 1);
    T('7.889041055960071688387418856813103789692415e+48958', '-2.2662793471337554182346631635907e+9271968', '2.26627934713375541e+9271968', 18, 1);
    T('7.1591716704439093539479127023e-9969551707', '5.907297809e+485003643635', '-5.907297809e+485003643635', 91, 5);
    T('-2.3282622367801899757134268569280457636570898e+588', '-8.8344996945e-655107935758867', '-2.3282622367801899757134268569280457e+588', 35, 2);
    T('-8.673323298e-956822565', '-3.270894886577420582286077880916836377418540173106334e-4469409355', '-8.6733232979999999999999999999999999999999999999999999999999999999999999999999999999e-956822565', 83, 1);
    T('-3.63656810207307891999988294237208057413617135275630954522718e+192257305084', '-7.17104540810783779062901289943552760325839e-6468637391', '-3.63656810207307891999988294237208057413617135275630955e+192257305084', 54, 5);
    T('-8.7285928195285743969803009808423e+62', '8.09123549153797358201067787566525129385663e-301826639', '-8.7285928195285743969803009808423e+62', 51, 2);
    T('3.7133275115693758413500420156e-56', '-3.63201014e-5635328464', '3.7133275115693758413500420156e-56', 50, 1);
    T('2.5241542241890251344096910067520547983e-13', '-7.06307483580387e+66820606827314', '7e+66820606827314', 1, 4);
    T('9.147993512097e+6393544', '-2.662571862689045374379167e-623', '9.1479935120970000000000000000000000000000000000000000000000000000000000000000000000000001e+6393544', 89, 0);
    T('-8.6229e+87179974537419', '-5.1e-805036239161', '-8.6229e+87179974537419', 23, 4);
    T('-3.7175615497289312085e+831', '-1.3425956881373562755303216178758051979052891671964464036e-5134584491', '-3.7175615497289312085e+831', 83, 3);
    T('3.05384897624215e+21', '8e+71', '-8e+71', 50, 0);
    T('7.8282462909890333428687885011427288017287e+7880460796672', '-3.358792e+3809628673', '7.8282462909890333428687885011427288017287e+7880460796672', 53, 3);
    T('5.06594462665e+708', '-7.554780868e+40', '5.06594462665e+708', 53, 1);
    T('-6.34654369622040009744620e+4882079434615', '-6.31942051948404e-82', '-6.346e+4882079434615', 4, 1);
    T('5.998786999e+15', '-5.2735871362713052e-83240', '5.998786999e+15', 12, 1);
    T('5.172632e-68', '-9.3200516970753213510256347079750599919176658900304228e+39045674660784', '9.32005169707532135102563470797506e+39045674660784', 34, 4);
    T('3.714386232131e+16609', '-2.41950601338636812012e-480401975666', '3.7e+16609', 2, 3);
    T('4.680153771797976081371433045435490690197277837059506328063e+273649849227', '-1.45305109695832920513438658770802e+744452', '4.68015377179797608137143304543549069019727783705950632806300000000000000001e+273649849227', 75, 2);
    T('2.7997681e-2391767406560', '-9.9880973735336554086799954012019658330780030651677739269858e-60643660', '9.989e-60643660', 4, 2);
    T('8.7731385749834933179761127345996e+836103185371', '-9.80889071e+937378334', '8.773138574983493317976112734599600000000000000000000000000000000000000000000000000000000000000000001e+836103185371', 100, 0);
    T('6.4498e-6447958', '-5.65396593257599036803205817413028766970580054e+45', '5.65396593257599036803205817413028766970580054e+45', 62, 6);
    T('1.467980618276425946715584394674236e+653277064803924', '4.4625711920749498387934218600083e-3833321210', '1.467980618276425946715584394674236e+653277064803924', 92, 4);
    T('7.080839903806e+66793121', '3.03517363470770900281064439835036129337482717e-923', '7.08083990380599999999999999e+66793121', 27, 3);
    T('-5.519589468922610171989837576053210692042945844161e+23700827', '-4.8775899260862012462431712e+393178719939', '4.8775899260862012462431712e+393178719939', 55, 2);
    T('-7.67823197275e+73734', '4.751769968115214098957062817913561225828190394861e-207', '-7.67823197275e+73734', 38, 1);
    T('-3.27096664196016294240e+683929905277', '-8.470442134802e+338969153148', '-3.2709666419601629424e+683929905277', 74, 6);
    T('6.77619649956505397066404435430093542479775550e-60869015', '7.59411722978848628722088785604509812001348074767070196346651e-94174641462', '6.776196499e-60869015', 10, 3);
    T('-8.2867606234479698430341e+5105709692344', '-2.6291214319566543673279535768425141893146227235937893934544e+9229141955', '-8.2867606234479698430341e+5105709692344', 36, 6);
    T('-5.967514550967946436347279144646105329788150735564e-88860420478107', '-8.37509724036266268912706e-9366471', '8.375097240362662689127e-9366471', 22, 6);
    T('-863731375837677326681860757359465410786605980267.36', '8.753137920329588996265692924980560367104422816389292778e-7114152', '-8.63731e+47', 6, 2);
    T('-5.194964625e-69955964875', '-5.030143167796786107431240660378234174392761515279346713374e+918343428867', '5.030143167796786107431241e+918343428867', 25, 6);
    T('6.7630887900445323868e-2296230657876', '-2.85471757345089156088680705362546179290669440e+436773351183', '2.8547175734508915608868070536254617929066944e+436773351183', 46, 3);
    T('3.759831145832032767084110983949111603649870230e-4395410', '-2.23199283699162481543e+7478789', '2.23199283699162481543000000000001e+7478789', 33, 0);
    T('4.6327273648e-39954', '5.5458211308763640253976731122227177372313340355198e-6178943', '4.63272e-39954', 6, 3);
    T('1.48986883554708663929710675522791e-926512', '7.57022964813687230980781909e-63691', '-7.5702296481368723098078190899999999999999e-63691', 41, 2);
    T('5.3479594457715948363744252969215600470201061474063e-40223306', '3.94741269310479455946013763376596593605533120e+8243286835', '-3.947412693104794559460137634e+8243286835', 28, 3);
    T('5.0828521918644223074942112e-907847525', '5.93e+4227210', '-5.92999999e+4227210', 9, 2);
    T('-4.304654534403250645656684171018e+648346701395', '9.541897413417472560874065370e-12899022741101', '-4.30465453440325064565668417102e+648346701395', 30, 6);
    T('-7.07251963113263168680525635e-78322', '-1.849819491632e-64581775', '-7.07251963113263168680525635e-78322', 41, 5);
    T('-2.45887297133910970775545709470801478300486517221e+107', '-8.569655933666637441575250164e+55985241441175', '8.569655933666637441575250164e+55985241441175', 34, 6);
    T('999852454042748127.90', '-9.5471586260659117414995673328233388720836641360599204962e-9477882528', '9.9985245404274812790000000000000000000000000001e+17', 47, 0);
    T('-2.29521757829220974112e+111', '-4.2133289668667243378302474255608397815e-99499', '-2.29521757829220974112e+111', 38, 5);
    T('-9.0090219123371362791107e-46678127737', '9.56e-76191', '-9.56e-76191', 90, 6);
    T('-2.236602976618144433881605923155730591341958e-27005540619', '2.12595564066739e+45', '-2.12595564066739e+45', 88, 5);
    T('-1.02172709146662e-998090', '-8.691536320e-528', '8.69153632e-528', 38, 4);
    T('-7.9799452138577466035252869042615165951e+41', '-2.7391639493055801772970813627563e+334', '2.7391639493055801e+334', 17, 3);
    T('-1.5558448e-86', '9.0777988542335984291135799013265121578838824577719202e+48930149', '-9.077798854233598e+48930149', 16, 6);
    T('-6.18253508141e+959', '5.67909902212811519875538207687297197599979675829920832437e+59038124', '-5.67909902212811519875538207687297197599979675829920832437000000000000000000000000000000000000001e+59038124', 96, 0);
    T('1.213566887093305891092018690775838993995423266797431094e-7803215', '-2.98382488775119936630825597878523e+599385425673', '2.98382488775119936630825597878523e+599385425673', 59, 6);
    T('2.76350076492e+16', '-4.333273900874256405749461802700667457670e+9543276014702', '4.33327390087425640574946180270066745767e+9543276014702', 90, 6);
    T('6.6762029258778378229341e-219830', '-8.13243163249403014533812706360081817e+843911411477', '8.13243163249403014533812706360081817e+843911411477', 80, 3);
    T('-6.892852998593868869196676790970032466467844763341415936770e-550', '1.013017004120703497000217763392506110252833e+9642952', '-1e+9642952', 2, 6);
    T('8.78596462081135258520424328220279355128643363402156e-56879435', '-6.6041247060651063795013254685018872714889398189965e-7017441', '6.6041247060651063795013254685018872714889398189965e-7017441', 98, 4);
    T('-7.3923327075987203717855651393144238067722353577413765259e-28604036701', '-7.5970546220838083600268661557523393263282828720e+845388656376', '7.597054622083808360026866155752339326328282872e+845388656376', 49, 4);
    T('9.49610725e+198908', '7.365986836133668660691753358700255544443909756033378597e-11426732', '9.496107249e+198908', 10, 3);
    T('-8.0590697646040815710914600565239860289305721638341720289e+434900455511443', '-9.9540417518442725851244822049429891606318463273072337368e-228', '-8.05906976460408157109146005652398e+434900455511443', 33, 1);
    T('3.54689212432067964e+7083532116687', '4.6373462802451566513e-20058', '3.54689212432067964e+7083532116687', 39, 6);
    T('9.240578455682036947729018052573852e-662448673', '-6.201235610807810e+61390804570899', '6.20123561080781e+61390804570899', 92, 5);
    T('3.581750341882083268957838656413115395904053581124954124e+56', '-4.697041753715715916790128326651832e-61909', '3.58175034188208326895783865641311539590405358112495412400000000000000000000000000000000000001e+56', 93, 0);
    T('3.934e-91089367281316', '-8.3145065183909086031309968995119692741591320960718755e+552312881752883', '8.314506518391e+552312881752883', 13, 4);
    T('6.2054711454812496576983745332288554118125141e+398938358360', '3.93738293e-801798', '6.205471145481249657698e+398938358360', 22, 5);
    T('-463901558344817107933584350841951174.463591833037732101', '1.48724803147273082841441802588181464336563332550452820061e-728904306990908', '-4.639015583448171079335843508419511744635918330377321010000000000000000000000000000001e+35', 85, 3);
    T('-7.921860260252635541456416137e-45166', '6e-3872137389444', '-7.921860260252635541456416137000000000000001e-45166', 43, 0);
    T('-1.1558358763665148112582867246202967853333610e-3309441', '3.399e-5089', '-3.39900000001e-5089', 12, 0);
    T('9.769906199797970257019645980370146379e+490560324', '6.35956856797160706997824344919520011e-69050', '9.769906199797970257019645980370146379e+490560324', 66, 5);
    T('7.1828046608960041953969758814035505731352952419156401317714e-945820552871', '-5.10872716594456328541991377350147681019619433927196957e-284135', '5.1087271659445632854199137735014768e-284135', 35, 5);
    T('-7.1042363e-18189266262', '-4.99478434308094147066e-665475', '4.99478434308094147066e-665475', 83, 5);
    T('7.87295760118941937156054334173232e-29087265554062', '1.039555410563099094072783144221496700220e-2038388839', '-1.039555410563099094073e-2038388839', 22, 5);
    T('-1.1470478240885021046388780635656298003053143e+66796', '-3.90678149007414408013992730780082798518661e+631746860285578', '3.90678149007414408013992730780082799e+631746860285578', 36, 6);
    T('-1.5043637714470e+46079439305', '-9.174606958727943162e+23', '-1.504363771447e+46079439305', 82, 6);
    T('7.64014825094466080407517920375661385423939549638e-30', '-2.205691727695820737826e+24', '2.205691728e+24', 10, 0);
    T('-6.18933848673567341898e+5087866510725', '7.62e+1457', '-6.18933848673567341898e+5087866510725', 39, 4);
    T('-8.873761778422556128755014756274088582e-8936496830', '4.450085446577725328076667455113431e-139263', '-4.450085446577725328076667455113431e-139263', 60, 4);
    T('8.103970062421652655204521893944499220408695370609e-22687751', '5.160562058e+79749591034', '-5.1605e+79749591034', 5, 1);
    T('65678328812313148938256707645649938988041908.7628', '-7.005219818630418086402007893153322528298163328533902667224e+47607900466', '7.00521981863e+47607900466', 12, 3);
    T('8.711717863756064982410464637693877202521926320152919068991e-17234435707', '4.64269159302449108840281896285720893402860468690e+346', '-4.6426915930244910884028189628572089340286046869e+346', 58, 4);
    T('2.8317635386862481516e-950', '-7.529911709443570573e+71500795542', '7.529911709443570573000000000000000000000000000000000000000000000000000000000000000000000000000001e+71500795542', 97, 0);
    T('-4.4540535893135943576e+52', '3.684380287696000326134529331748073368628194723734e-247', '-4.4540535893135943576000000001e+52', 29, 3);
    T('-1.53708220e-4055680616', '3.21130e+505', '-3.2113e+505', 6, 1);
    T('1.942548059414436975654132874324293042972376510191193e-824003213142', '7.747497e-7315278841', '-7.747497e-7315278841', 26, 0);
    T('7.37214315829763112722851449774641832558016015279331600e+70254173', '-7.3413e-459579', '7.37214316e+70254173', 9, 0);
    T('-7.36184429987472863136344887537e+46397967554', '9.4485758202124657809581389522666910783508974996901177884e+534058398553', '-9.448575820212465780958138952266691078350897499690118e+534058398553', 52, 5);
    T('-3.554073745443195038520983884784354865e+861', '1.444664588e-28573875', '-3.5540737454432e+861', 15, 3);
    T('-9.750524885094e-58', '-4.6039642520283465348993544389689878886553422274833559516e+63', '4.603964252028346534899354438968987888655343e+63', 43, 2);
    T('-2.864074940426775443152414709935844788333120219463221e-9442311958156', '5.743175459953600128129e-459', '-5.743175459953600128129e-459', 39, 4);
    T('-2.9708277069174330846769892166332604909842063772040034e-2074303', '-8.180161153019297855947146250824083891407100732e+497599423246', '8.18016115301929785594714625082408389e+497599423246', 36, 3);
    T('4.821386e+42181744', '-1.092631116353907313882128781880320103420605e+74807769', '1.09263111635390731388212878188032010342060500000000000000001e+74807769', 60, 0);
    T('-7.237692089757490737085761536229753711091152070242e+3012736', '-3.5294186364464373112223166455521548e-91646', '-7.237692089757490737085761536229753711091152070242e+3012736', 66, 0);
    T('3.99e-203', '5.25090911190904186522116799e+7977107298', '-5.250909112e+7977107298', 11, 3);
    T('-7.91964e+713857', '3.4e-4992', '-8e+713857', 1, 5);
    T('-2.151e-9', '-9.7085775490883410e+60354671', '9.7085775e+60354671', 8, 1);
    T('3.385095153272846106538585658719930381e-923044772', '7.372530153020347104758409281913903530203191686019e+700764039', '-7.37253015302034710475840928191390353e+700764039', 36, 5);
    T('-1.34055877117364647408552188494216298000e+2637802548538', '-1.241493333702244954713774078305234697976555240401540854044e-8504624516596', '-1.34055877117364647408552188494216298e+2637802548538', 42, 4);
    T('-8.8416e-973191653', '-2.1499987676469e+1196', '2.1499987676469e+1196', 73, 5);
    T('-1.85914260421416525517175281941663696647970e-96102185766', '8.521532960983e-43420220', '-8.521532960983e-43420220', 33, 5);
    T('-5.882226881378989343398290e+625333370', '-8.47121251664879079549830626072e-218816069141552', '-5.88222688137898934339829e+625333370', 95, 0);
    T('1.19249434145937635352366645010e-1037959824975', '-6.518113654174252619e-775632603348755', '1.19249434145937635352367e-1037959824975', 24, 5);
    T('-1.3417e+740818422021330', '1.39977309725489e+98582907629908', '-1.341700000000000000000000000000000000000000000000000000000000000000000000000000001e+740818422021330', 82, 3);
    T('-3.034217051367069682664378875686490871248906836660062013985e-88922689', '1.1705754163977311388211649e+169215', '-1.170575416397731138821e+169215', 22, 2);
    T('4.0477990062589255e-84225', '-4.3283484371437279630e+55768903', '4.328348437143e+55768903', 13, 3);
    T('-78941529551946538461981605340.958257', '-3.07113490033735e-315242165645', '-7.89415295519466e+28', 15, 0);
    T('3.675214554044556722877390865336265205339493e+137', '6.4012144156160792125747826626056554480465255577343e+44348279230185', '-6.4012144156160792125747827e+44348279230185', 26, 3);
    T('-3.1953022543564738450886669309932779910e+4054971815384', '-1.84526341525964057299978169384334578979913961499e-4796', '-3.195302254356473845088666930993277991e+4054971815384', 65, 4);
    T('-7.4394330126156420739353577802271448982623884308049e+683364132025441', '-2.4113302701370967684418030563623233845385640164462184996687e-641384898379063', '-7.43943301261564207393535778023e+683364132025441', 30, 5);
    T('-4.0044404191502804346604344797769901428212389937e-19', '-1.974437452236002865491393305536246924e-795013030791', '-4.0044404191502804346604344797769901428212389937e-19', 83, 3);
    T('1.90694e-57404362204', '-2.5e+59', '2.5e+59', 93, 1);
    T('7.621366617e-904574005', '-3.427939524351009578496245991961314070381447e+7891', '3.42793952435100957849624599196131407e+7891', 36, 1);
    T('4.736749231516559591e-83334248', '-5.9293821889948460e+9143128509215', '5.929382188994846e+9143128509215', 77, 6);
    T('9.4573457534778062265450461805055905343426518404473e+990', '1.76248188349e-877', '9.4573457534778062265450461805055905343426518404473e+990', 63, 4);
    T('2.302564840718255261864131197522173167544e-563731034662', '-7.1778493840276077235088318734261573912384058983008473e+9500229057415', '7.1779e+9500229057415', 5, 2);
    T('3.496192943880668343279181650980307323473e+392004432325', '5.30350569358e-3284334533', '3.496192943880668343279181650980307e+392004432325', 34, 3);
    T('-2.8700686553851632279376049865064199e+235', '-2.4817307084731937186288963773316794595192732124698e+50903580687', '2.4817307084731937186288963773316794595192732124697999999999e+50903580687', 59, 3);
    T('7.8759616397022832575429079130271e-228279', '-1.976182587657735960918301e+649', '1.976182587657735960918301e+649', 80, 6);
    T('4.431156210160164195105601208115300007085364171293552896170e-67020', '9.4477117194900228500e+23', '-9.44771171949002285e+23', 23, 5);
    T('2.09140772344104034851830338310844489491346669150130549338583e-64676', '6.847244272261491e-2659', '-6.847244272261491e-2659', 86, 4);
    T('5.178e-332', '-87208345104732076337.106581991', '8.7208345104732076337106581991e+19', 58, 6);
    T('9.47575139306677847323691306e-465996', '3.780814148915e-356940676477826', '9.47575139306677847323691306e-465996', 40, 4);
    T('-8.3e+5551871243368', '-8.3725527149193e-190', '-8.3e+5551871243368', 95, 3);
    T('-7.25264601846229283432018831932357133483575688678046299e-38778717802474', '6.772775595344311284614842496577828713192854706829e-290874788025306', '-7.25264601846229283432018831932357133483575688678046299e-38778717802474', 62, 1);
    T('2.7406845816458862173738502382323600100998e+199302', '6.64877002078877889961217365199228879599804045336e+106734032', '-6.64877002078877889961217366e+106734032', 27, 0);
    T('-5.232861811e+68841', '-3.40743295401613908402728032109755603636292061802970886e+65821908', '3.40743295401613908402728032109755603636292061802970886e+65821908', 90, 4);
    T('-3.741629304730969726287989030891964428684953e+950456075999881', '8.0412343589356e+15', '-3.741629304730969726287989031e+950456075999881', 28, 5);
    T('3.41604590552887771689643869242296703434476780427081629337385e+72', '-6.289131225e-95377895041016', '3.4160459055288777169e+72', 21, 4);
    T('-9.788478037167098770496e-160', '-9.3306662e-336968', '-9.788478037167098770496e-160', 74, 3);
    T('2.0227503765290251028486e+89100497', '-3.18539558858189222777122e-146391923', '2.02275037652902510284860000000000000000000000000000000000000000000000000000000000000000001e+89100497', 90, 2);
    T('-7.99563e+175988270', '6.629136e-5179227600624', '-7.99563e+175988270', 97, 5);
    T('-7.67186211359421930305367080951321e-549595422512468', '1.926034045384986572533073822e+266', '-1.926034045384986572533073822e+266', 37, 1);
    T('-5.48073883084760102870355235313296384933367625e-8073', '-4.57558063837245753287538758957603872613938773005200627114190e+644472', '4.575580638372457532875387589576038726139387730052006271141899999e+644472', 64, 1);
    T('6.6228594952657580837475e+319042816', '-3.3774470637698e+7495278', '6.6228594952657580837475000000000000000000000000000000000000000000000000001e+319042816', 74, 2);
    T('6.509746434895769869e-225210490966000', '4.10812626545809495639001986e+6519933', '-5e+6519933', 1, 3);
    T('3.3040556582213633824553824577500061834875628833553237414398e+92', '-1.11e+75428551604', '1.11e+75428551604', 22, 6);
    T('-7.9e-195898302', '8.9810887682724e+261457125', '-8.9810888e+261457125', 8, 6);
    T('9.99819802163626833741645308026916752482522973177247706535879e+2913216', '8.5754908222946895941985615372470797e+13318918169954', '-8.5754908222946895941985615373e+13318918169954', 29, 3);
    T('-4.599671856947047722457271633163299e-13019', '8.885412920923677741488e-52', '-8.885412920923677741488e-52', 46, 1);
    T('8.971360050e-77715603131289', '48.521663832358447100082546485450602338141972178', '-4.8521663832358447100082546485450602338141972178e+1', 82, 0);
    T('-9.156330341240e+5091366057', '-8.7539900116265221516588417546092751125395347838208871460312e-35461580069594', '-9.15633034123999999999999999999999999999999999e+5091366057', 45, 2);
    T('-3.927290895640937790354931137772100570e-15279560761', '-1.61306831e-613', '1.61306831e-613', 35, 5);
    T('-5.8370534217442923336818978059607916193866906917172e-66486778', '9.5', '-9.5e+0', 2, 2);
    T('-2e-1279', '8.0908274e+459641745', '-8.0908274000000000000000000000000000001e+459641745', 38, 3);
    T('7.00494933963e-868876418852', '-4.361444395151629213692547043908177376275839371539602759356e+94925', '4.361444395151629213692547043908177376275839371539602759356e+94925', 82, 5);
    T('-1.92491301148404203593186572466e-961318047404', '-7.2620164348036030323274223243742967328377089602650289878e-26847210', '7.2620164348036030323274223243742967328377e-26847210', 41, 5);
    T('8.601373646909470758974117594e-42384', '-5.25644271e+94', '5.2564427100000000001e+94', 20, 2);
    T('-5.76231162949818117368698428677623628077264035e-61698092847100', '-4.86863563867590378262721320017164833752e-50', '4.86863563867590378262721320017164833752e-50', 91, 4);
    T('-9.3204520618062226602209603152620300601120e+43091873753520', '-2.2371783827090045151490984865783892766491141e-292752210278', '-9.32045206180622266e+43091873753520', 19, 6);
    T('5.143279600403229531921048250594635852e+5600746266682', '9.193762e-22', '5.143279600403229531921048250594635852e+5600746266682', 80, 6);
    T('3.48377220694350478744958266118263e-21908538182981', '4.94735809745870013730868290829038474053e+245944700860', '-4.9473580974587e+245944700860', 16, 1);
    T('-8.1125911475671037115356893952642685507468087e+4293', '-2.627364245010932538391768158151127913541472868570362744596e+3049440614169', '2.62736424501093253839176815815112791354147286857036274459599999999999999999999999e+3049440614169', 81, 3);
    T('1.5022121124629757925315361774323e-11420401', '-2.6392015328987e+5452', '2.6392015328987e+5452', 34, 5);
    T('-9.9632513203080875e-1067413', '-3.075065005573744502286805831779708526371970488478764199e-18', '3.075065005573744502286805831779708526371970488478764199e-18', 65, 2);
    T('8.63044267711567249152834746416496e+9243472646909', '5.3219072487027788329091093796734845285e+46', '8.63044267711567249152e+9243472646909', 21, 3);
    T('9.618883370354542832043927886337000957283615672e-115283284', '-3.1151197453146171442801063e+59412706', '3.1151197453146171442801063e+59412706', 39, 5);
    T('6.0e-7982302340429', '4.8357656362638997e+5720', '-4.8357656362638997e+5720', 99, 4);
    T('8.17119162e+800058747802908', '-7.4253487077366719612554538195658173086690117987449822261945e-238009177553', '8.17119162e+800058747802908', 20, 5);
    T('-2.8332e+98009833334', '4.522777126917657624501519721968948301693515478e-48494963', '-2.8332e+98009833334', 77, 6);
    T('-7.7435421101662381560778815e-622900891331564', '-6.8667342531113752083534135606228979533e-6122892357', '6.8667342531113752083534135606228979533e-6122892357', 73, 2);
    T('4.29709255508873820821761080400357513687e+39', '9.0019727731853133278039193521927966708e-773', '4.297e+39', 5, 3);
    T('-3.2521678687127728711184260309e+55', '6.2884662560866971269243819823749498106843e+53442', '-6.2884662560866971269243e+53442', 23, 1);
    T('-4.48407138860276534103489e+835379222', '7.2241e-5743496359', '-4.48407138860276534103489e+835379222', 98, 2);
    T('9.432729e-362', '-8.275939239733865971636233893459880579e-339878104707', '9.432729000000000000000000000000000000000000000000000000000000000000000000000000000001e-362', 85, 2);
    T('2.740536303800847134e-641019', '7.05768601194354646605073009752028e+119660745', '-7.05768601194354646605073009752028e+119660745', 79, 3);
    T('6.9208839930e-635', '-5.433100726732657893248e-6728', '6.920883993e-635', 33, 6);
    T('-6.414735583115161359798433822364e-1563272025', '3.16303088472190666359795629555680180159699895861e+51152104807496', '-3.16303088472190666359795629555680180159699895861e+51152104807496', 52, 4);
    T('8.4e-348566395', '-6.6200508073228665e-7671496907568', '8.4001e-348566395', 5, 0);
    T('1.4761081802173311799843960653733207165305643679281e+10406030664', '-4.205581986657153975499691355500620066710211459342e-7471576', '1.4761081802173311799843960653733207165305643679281e+10406030664', 68, 5);
    T('8.40557e-4965819997100', '2.3292350629179225840466207450147650589607854070104e+8130', '-2.3292350629179225840466207450147650589607854070104e+8130', 56, 3);
    T('-6.9881102246124914573880204899464759276633455019309287455361e+41307606007832', '3.4120697551896384e+44346838401406', '-3.4120697551896384000000000000000000001e+44346838401406', 38, 0);
    T('8.787448432678431228075576852053949220411282509688237881963e-11004242', '354493410871711782608119381.665', '-3.54493410871711782608119381665e+26', 96, 0);
    T('-7.111276971403905257285544511e+8322024', '-6.4213016817217133072315013038320270e-3681462881107', '-7.111276971403905257285544510999999999999999999999999999999999999999999999999999999999999999999e+8322024', 94, 2);
    T('1.23305e+5989', '7.569665395748423798332920775402523376844464944059562949515e+1733513', '-7.569665395748423798332920775402523376844464944059562949515e+1733513', 82, 3);
    T('9.7584510237396572796e-24111', '-5.0902753339517563367329999686511226387218e-8573080275900', '9.8e-24111', 2, 6);
    T('-4.26714192205467236302573031811508780e-8169657708', '-3.0436876437066179573221781450156017613988360906782559e+1971258974949', '3.043687643706617957322178145015601761398836090678255899999999999999999999999999999999999999999999999e+1971258974949', 100, 1);
    T('1.897515625482307374066609615798536409336e-502331360395', '-3.842222527283617250385979366e+62655', '3.842222527283617250385979366e+62655', 97, 4);
    T('-8.55184987053917890890550897799709764996150309328518273376e+2468434', '6.6e+958346312', '-6.6e+958346312', 27, 5);
    T('2.245952207916828582795218748284914499773826229e-53153116', '-6.2463656189553045062692527668186820450270358326670780e+473302789753', '6.2463656189553e+473302789753', 14, 4);
    T('-2.120571563684012125606222559e+93927262697', '-4.3488576191731972429859529666071398677478e-3340800731673', '-2.1205715636840121256062e+93927262697', 23, 1);
    T('8.001746869662619784074950751016135497509073787925e-36', '6e+91', '-5.999999999999999999999999999999999999999999999999999999999999999999999e+91', 70, 2);
    T('-6.975856671139608160297807252533582548888259453824e-885897799', '-1.79550462841263897025231e+366983184416', '1.79550462841263897025231e+366983184416', 47, 5);
    T('2.034893786695674238285161966413e+382159981', '-5.8400491401180713589449816749462839e+979', '2.034893786695674238e+382159981', 19, 5);
    T('-7.99015188343535497229899187e+162414586302', '6.001048961318e-9279031878', '-7.99015188343535497229899187e+162414586302', 63, 5);
    T('-9.7097641927710729177519348460304609210491270122295157916481e-475453', '-4.57242283888415864657104686049192489102140e-740279', '-9.70976419277107291775193e-475453', 24, 4);
    T('1.805418340e+11', '7.43530395024982003539451322075094e+51019', '-7.43530395024982003539451322075094e+51019', 41, 0);
    T('-5.899643421693549751831243142291792461728720802997961807e-390120', '4.07390e+57', '-4.07390000000000000000000000000000000000000000000000000000000000000000000000000000000001e+57', 87, 3);
    T('5.2437167247703448020465834837340160094965e-768539', '-74295804263329136553212448247384639367541199448588.00122767', '7.429580426332913655321244824738463936754119944858800122767e+49', 83, 3);
    T('-6.90736158911688520892e-42810916', '7.54903103083885523e-68928070621', '-6.907361589116885208920000000000000000000000000000001e-42810916', 52, 0);
    T('-5.472e-86726114', '-6.589864e+96632066279', '6.589864e+96632066279', 75, 2);
    T('6.22144753472064778805542e-4030280598072', '2.7204723429428642299e+1687043098075', '-2.7204723429428642299e+1687043098075', 52, 3);
    T('6.3305758095591810696135793412713892158994222443251134870e-1613646', '1.9844136790348213799e+95590', '-1.9844136790348213799e+95590', 82, 3);
    T('-7.836835683499881933416266e+530817', '3.884146143565740915400136255380413466612414983909180250532e-2351', '-7.836835683499881933416266e+530817', 86, 4);
    T('-9.68095469470082e+9728973026', '7.60632512735060814137040257191163043e-6444739980', '-9.68095469470082e+9728973026', 99, 2);
    T('7.188173062054426621881896e+61', '-1.7352608245201117868022632201483468224549274954406860270e-1312357601564', '7.188173062054426621881896e+61', 42, 4);
    T('2.121e+73882089149956', '5.48077687927458409763878644427249792628282218420e+50', '2.121e+73882089149956', 41, 6);
    T('8.8e-8676169955', '8.3436980555804511499101594851231390524219989693264e-298512489652496', '8.79999999999999999999999999999999999999999999999999999999999999999999e-8676169955', 69, 1);
    T('-3.8591255266710579918049548157026e-641936940294273', '-9.58475497669070741008594513752334687e-80779254310186', '9.58475497669070741008594513752334687e-80779254310186', 52, 4);
    T('-8.76692101161796983153110264982284631998962e-64516100284632', '3.6559100817239008342197697663049316895311927348931035889635e+870558664652', '-3.65e+870558664652', 3, 2);
    T('-3.41593068975515583170241783844086110408591644381e+40442900966729', '-1.4278805386861950990569677808588929942298422e-1381806399045', '-3.41593068975515583170241783844086110408591644380999999999999999999999999999999e+40442900966729', 78, 2);
    T('-6.0e-46819', '-3.7574250454206603289861526122833351653919e-253017', '-5.9999999999999999999999999999999e-46819', 32, 1);
    T('8.1252922821995001925772459e-701183439184', '-5.7048481763168843105622383637036275538362234682699652616453e+4598922', '5.70484817631688431056223836370362755383622346826996526164531e+4598922', 60, 0);
    T('-9.08152206960113604749567871369e+5883603', '-2.22903205528573342880870258274e-86551', '-9.08152206960113604749567871368999999999999999999999999e+5883603', 54, 1);
    T('-1.8763863171834093660141382881415550234762013e-6384', '-9421232602506031.09404357551531826560980450', '9.42123e+15', 6, 6);
    T('-8.72117812142396006139e-6475298806', '7.2589447093802178321473962873914950978272838211800742129e+55646327650', '-7.258944709380218e+55646327650', 16, 3);
    T('-9e-335804943829', '1.49928535510378729783390686e+489517', '-1.49928535510378729783390686e+489517', 89, 5);
    T('7.524314197553982786327593321608202707573170958474e-5124068', '8.51683518050e+53097909312', '-8.51683518e+53097909312', 9, 1);
    T('6.90335e-95886507661', '-6.72865139877501e+9981189', '6.728651398775010000000000000000000000000000000000000000000000000000000000000001e+9981189', 79, 2);
    T('-8.078e+33', '-6.7573381811105251116e+6456717640640', '6.7573381811105251115999999999e+6456717640640', 29, 1);
    T('4.4831501377565658137685211841003182723140619174535e-1658705308487', '-6.5166688602173066915723524898490028124248922557079e+744008583075880', '6.51666886021730669157236e+744008583075880', 24, 0);
    T('9.577265913292890661087928538335742e+128', '5.119973031739461e+107', '9.577265913292890661082808565304002539e+128', 42, 6);
    T('-9.587459827934820997488046415393878e+8541196', '7.924533624467756198634329302718639507532922e+47', '-9.587459827934820997488046415393878e+8541196', 76, 4);
    T('-9.356569025990714209831450796e+334445504010629', '2.7573315955e+988684599', '-9.356569025990714209831450796e+334445504010629', 79, 2);
    T('3.54849134306416237086309355280661721497051628219857699019e-90615674', '8.153369079181998369697235497372607159009197939e-7464194', '-8.15336907918199836969723549737260715900919793899999999999e-7464194', 57, 2);
    T('-9.50930608039679684467e+4986602790460', '-2.97669610661e-59866561645968', '-9.50930608039679684467e+4986602790460', 46, 3);
    T('-6.02035593735243105522131451679690e-895043940210830', '8.93e-950202096083', '-8.93e-950202096083', 75, 4);
    T('-9.11362838528657094981363456660998373567767862610302521e+54', '-3.5714592711004647952177784697207407156542117561e+3987017755', '3.5714592711004647952177784697207407156542117561e+3987017755', 60, 4);
    T('-16382509940397101667984222243.9361482', '3.6475380164592199676530685067582347917153920834648455e-803497973955430', '-1.63825099403971016679842222439361482e+28', 57, 5);
    T('4.66571794937298015230e-721652174', '-9.7101666851711911110250e+487383925731257', '9.710166685171191111e+487383925731257', 20, 6);
    T('3.91866407585597216040670707116892852861304043508e-693708778003', '1.24458761164e-906', '-1.244587611639999999999999999999999999999999999999999999999999999999999999999e-906', 76, 2);
    T('6.54378105968243661762171358482957100940711611333407856718e+588376', '-5.51135328667149689741443019056686955907752764043415e+557697', '6.54378105968243661762171358482957100940711611333407856718e+588376', 70, 5);
    T('-1.3791698692673298749458e-761657294', '-7.401308136003316346e+178433147', '7.401308136003316346e+178433147', 38, 6);
    T('-6.91450165919045870375050330467898131976149262068910192473e+20995356', '7.9163043478996414919934747838285853606744126e+124', '-7e+20995356', 1, 4);
    T('-6.771467048882411951118447474384686715709917739e-47839265126', '-4.650871409254441249237838304999426261648001895051e-72343732950', '-6.771467048882411951118447474384686715709917739e-47839265126', 94, 3);
    T('-7.8708010627330610e-8158', '8.1084096888324222338439720241579762768225347166537236148e-65394802', '-7.870801062733061e-8158', 52, 6);
    T('1.8933088464804226593787950975741444e-4481569', '-6.53452202598530295144557803608230e+563073089540', '6.5345220259853e+563073089540', 14, 6);
    T('6.7712923e+87974290500913', '29561577673103174480778797651346889327250101365652352955', '6.7712923e+87974290500913', 91, 2);
    T('3.438368576002514596206e+24', '-1.172859144e+681532434958599', '1.172859144e+681532434958599', 59, 1);
    T('-1.4301031e+209854664292', '-3.98932414975110413045392554760703632534407414e+744', '-1.4301030999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+209854664292', 98, 1);
    T('-3.576618946626837781078820e-83388', '-8.5599705822590004258243951719171157265516169772250e-640882028', '-3.57661894662683778107881999999999999999999999999999999999999e-83388', 60, 1);
    T('3.5231783762639e+42844203121864', '-8.58419610385713938673338913655605182189222e+437537546', '3.523178376263900000000000000001e+42844203121864', 31, 2);
    T('9.7541421196205157323534411095963409725429466336248335e-844469873', '6.263591878479889188080874202379e-31931019', '-6.263591878479889188080874202379e-31931019', 77, 5);
    T('1.9981042078150406808144651454124861636865e+1294289', '-7.175651810e+16', '1.99810420781504068081446514541248616368650000000000000000000000000000000000001e+1294289', 78, 0);
    T('5.03193588873722425014727698038695441109654601257303383e+57510900859', '-9.4293312930419237e-767356434', '5.03193588873722425014727698038695441109654601257303383e+57510900859', 94, 6);
    T('2.70451793388927055204864856909027927921913566824683694811e+9642541467', '-4.19746898932e-1453785773', '2.70451793388927055204864856909027927921913566824683694811e+9642541467', 78, 1);
    T('2.9203872789e+18', '-1.6165398449287252107714048118130775853231557685816e-5068', '2.9203872789000000000000000000000000000000000000000000001e+18', 56, 2);
    T('1e-176585180', '-3.2621587733248876736485679004e+32', '3.2621587733e+32', 11, 6);
    T('3.403355824608640236031927034912942261e+3145', '-78814302114495660914264978077828792553018652695.633', '3.403355824608640236031927034912942e+3145', 34, 5);
    T('8.7330076024862352466229988138732913629802643565e-77220270', '-7.392426018e-207139447477', '8.7330076024862352466229988138732913629802643565e-77220270', 83, 4);
    T('-7.48189076e+58429747635', '7.200482137588801752923633e+252', '-7.48189076e+58429747635', 71, 1);
    T('-1438251.4', '-7.1866902688025048965924029239676343120395266108073668993e+38274982741220', '7.18669026880250489659240292396763431203952661080736689929999999e+38274982741220', 63, 3);
    T('-8.0992978305319000783224163356678878948456501551308473259e+27451414', '-8.9367630155187480188993742622936531131128956328355507644594e+640576991256', '8.9367630155187480188993742622936531131128956328355507644594e+640576991256', 80, 2);
    T('1.0975707860224781245585756547288306540678306082199888e+985024860753', '-8.5727353584359582995475671961067215053356408e+272090387362', '1.0975707860224781245585756547288306540678306082199888e+985024860753', 58, 4);
    T('9.981632697327812688705467926259e-3837280', '3.103954485973289503303719475474246057112e-621399663', '9.981632697327812688705467926259e-3837280', 50, 5);
    T('2.95548330052172857221730022018184381934708691246344e+115', '8.12779503015105752476853705088616240618738e-882907908', '2.955483300521728572217300220181843819e+115', 37, 6);
    T('-9.6534e-45820746584', '2.5184063905654362491344669653020311049080574148932e+54', '-2.518406390566e+54', 13, 0);
    T('7.2510126881107296185581291e-9368004796', '-7.4762826577445866689744480981641266200915465e+1495728054', '7.476282657744586668974448e+1495728054', 25, 1);
    T('9.147233439945353503139092224850421417e+4177', '-3.4552274370709122590785e-617', '9.147233439945353503139092224850421417e+4177', 69, 6);
    T('-6.993586594867910316077840380665692027041000637647770e-713079945105', '9.46940150133933e-9890321340807', '-6.9935865949e-713079945105', 11, 0);
    T('-7.661409182088553486794245690126522684339e-1434895', '1.088201793590047070405691145295604411783e-83077480455', '-7.66140918208855348679424569012652268433900000000000000000000000000001e-1434895', 69, 0);
    T('-6.4102180140295532671343441185147974306e+92', '9.26152664532041086099793e-45048082954', '-6.41021801402955326713434e+92', 24, 4);
    T('1.49905401800619857505299423592264477865085462868390e+46775840575787', '4.37347257073270413227875463383119e+51', '1.4990540180061985750529942359226447786508546286839e+46775840575787', 99, 0);
    T('-8.85037543421412288e-84058527848', '-5.42979218277895217411904301771775355e-56262', '5.42979218277895217411904301771775355e-56262', 42, 6);
    T('-6.27885009e-97802915578257', '-3.60274882e+9413967480', '3.6027488199999999999999999999999999999999999999999999999999999999999999e+9413967480', 71, 1);
    T('-4.748828e+104', '2.70279089355105677260472e+740212335612', '-2.7027908935510567726047200000000000000000000000000000000000000000000000001e+740212335612', 74, 3);
    T('9.4394616965942441773266889052466e-79360115733426', '-3.7696137559354314591469373e+3182214132802', '3.7696137559354314591469373e+3182214132802', 47, 4);
    T('-6.094212e+363736', '-4.8306090911081179703505665058425258481854906373e-954199431', '-6.094212e+363736', 87, 3);
    T('-6.00274663083790141331435423501967837344431271964872753e+215929847', '-16077366680313.02681874061047793878115668068088008791314', '-6.00274663083790141331435423501967837344431271964872753e+215929847', 94, 5);
    T('-8e+23862351284', '-9.125533343547113529937233650741e-5407962', '-8e+23862351284', 49, 6);
    T('9.248330025021e-682', '5.851433313447317920733170909781816592008037835e-811977', '9.2483300250209999999999999999999999999999999999999999999999999999999999999999999999999999e-682', 89, 3);
    T('7.4523544124e+132660338366849', '-2765673673380423264446930443878181591487230752296643000091', '7.4523544124e+132660338366849', 53, 1);
    T('5.89480005702116329260836188133561453417744037126207717667456e+45472758725', '8.00712958209502824e-259203352', '5.894800057021163292608361881335614534177440371262077176674559999999999999e+45472758725', 73, 3);
    T('6.2262046472171857288004687e+707799019120', '-8.2e+70986116151102', '8.2e+70986116151102', 41, 3);
    T('8.214e+910', '-7.3081216695875e-1473673', '8.214e+910', 53, 4);
    T('-6.851587847671455296807343436141464112e-11828813328', '-6.10409791399005937112e-727586689986122', '-6.9e-11828813328', 2, 0);
    T('3.31553130057003515177155823210972274387e-2798806434986', '-8.296651722709e+6509046553602', '8.296651722709e+6509046553602', 94, 5);
    T('3.9967909e-31670695526371', '-2.423e-44068435522', '2.423e-44068435522', 68, 4);
    T('5.703651854522262348781051544223607873833622565e+29521717167', '-5.692172279340660e+167169873721939', '5.692172279340660000000000000000001e+167169873721939', 34, 2);
    T('-4.7389753392766e-39139134372297', '-6e+1', '6e+1', 63, 4);
    T('7.0890234263447087623150624e-34618679499310', '4.8680837041689108e+9497339', '-4.8680837041689e+9497339', 14, 2);
    T('-6.825492809013095876823515e-6887021602', '1.08759704097573841474306039034348872203412e-8981360030365', '-6.825492809013095876823515e-6887021602', 50, 2);
    T('8.195879605121901535509913971718119975958610e-1508836453', '3.7931998882534608533966470123977434466e+26544336122651', '-3.7931998882534608533966470123977434466e+26544336122651', 51, 5);
    T('5.91848926424839751972276037187767674063500247e-27637028', '-2.407298997556625465e+331767188', '2.407298997556625465e+331767188', 100, 6);
    T('-2.389015884873968307893315608265576127299845606476026291948e+28717882', '7.62169226246199125e+39994695824', '-7.62169226246199125e+39994695824', 92, 5);
    T('-8.5009294876226386811565749822097204349e+95379618229369', '-1.133266160750362798232912491553110429e-6680837', '-8.5009294876226386811565749822097204349e+95379618229369', 46, 4);
    T('-7.73793129654856265e-3817996', '2.85485325688681646620169510218198000186324964e-10985', '-2.85485325688681646620169510218198000186324964e-10985', 64, 6);
    T('-4.837961471903113468185e+735533955484', '1.01898105050080000290418259106567229221201e-3579788306357', '-4.837961471903113468185e+735533955484', 65, 4);
    T('-3.8682278674399741171166852366834e-652163899', '-7.60768498113047975139920953585690721876523266488002425745e-922732903020', '-3.8682278674399741171166852366834e-652163899', 97, 0);
    T('8.123352140368401896447539913455293162436937968717780228116e-125', '9.614413943972451161273337245253e-17486362217206', '8.123352140368401896447539913455293162436937968717780228116e-125', 99, 5);
    T('-5.831571340622142246623410802375011010202e+736470100502172', '-8.93529029e+9383219', '-5.831571340622142246623410802375011010202e+736470100502172', 46, 5);
    T('5.146575054456965217055534986586669979866888969496431125828e-5642057315', '-6.4533872324049625782456012267471446516457180825440580530935e+67820', '6.4533872324049625782456012267471446516457180825440580530935e+67820', 99, 3);
    T('7.0482711465e-6882773063184', '-7.943714814484471889651104e-2735', '7.943714814484471889651104e-2735', 82, 3);
    T('-7.1889034807794564e-247131352', '-5.539268607317607122520851849516219652543465185e-95', '5.53926860731760712252085184951621966e-95', 36, 2);
    T('-9.044612977741938952955852163035939612435301295632418104857e+670', '3.09495671431713597611933e+12496888622', '-3.09495671431713597611933e+12496888622', 83, 6);
    T('-4.97877232023082899110136765445672e-87780350364', '7.418e-863858573', '-7.4180000000000000000000000000000000000000000000000000000001e-863858573', 59, 0);
    T('3.00448714404823691732354316067750638973544640463430239839e-320344759397', '9.62863817323435884266199348056689083875366093415805208e-131187679', '-9.628638173234358842661993480566890838753660934158e-131187679', 49, 6);
    T('2.76918239072142828e+6178', '2.418851178683016307954360e+811', '2.76918239072142828e+6178', 79, 5);
    T('3.4438848129830879e-4347', '4.195364367764151017e+58360', '-4.19536436776415102e+58360', 18, 6);
    T('-5.58621168065707433662731057664246539627385867484774747529004e-121721', '7.058437056995170433249997680158559336575499851156712e-419533314', '-5.586211680657074336627310576642465e-121721', 34, 1);
    T('-1.122054784138503375173642727303184463033855767474e+7253231828', '-2.45995e+65098', '-1.122054784139e+7253231828', 13, 0);
    T('-8.8166667723351612267497719787789552e+6031772932', '3.38724609230161010690619e+270371264438628', '-3.387246092301e+270371264438628', 13, 2);
    T('-5.0789223561672455726428e+9527920757556', '-5.76496138616955916079378272316e+92048', '-5.0789223561672455726428e+9527920757556', 97, 4);
    T('-4.471571e-18212902871', '-5.6199357534054894910354741561419687316842039e+227', '5.619935753405489491035474156141968731684204e+227', 43, 2);
    T('7.451334876811602e+16746152', '-6.249274421679380438517925105078789491e+610918993361251', '6.249274421679380438517925105078789491e+610918993361251', 91, 6);
    T('8.1857360317737385123677021533058201368890726541e-88424242033961', '-9.4116173641031e-5019617926162', '9.4116173641031e-5019617926162', 78, 3);
    T('2.627605806626169251087475621048284040096380355896200036e+252467414530', '-4.104405050061036878546162370816195816610e-8878535480726', '2.62760580662616925108e+252467414530', 21, 3);
    T('5.1506e-308293284871', '1.8885965547299874360280312740322487583289e+2119215699912', '-1.888596554729987436028031274e+2119215699912', 29, 1);
    T('-3.1785048176101156318666255e+58664873124504', '-6.746137234328831004522727571890011793970368485676218e-85995', '-3.2e+58664873124504', 2, 4);
    T('8.0067575e+906', '-6.394171513477795272766913987e+29581', '6.394171e+29581', 7, 3);
    T('-8.1353618980728776772265179261e+8064528437740', '6.425776868824e+28891005', '-8.1353618980728776772265179261000000000000000000000000001e+8064528437740', 56, 0);
    T('-1.732596230936755023566e-105', '9.1879935814462014363378e-1681', '-1.732596230936755023566e-105', 48, 4);
    T('7.0390404900195233485345e+96487182', '-2.32116380769425814159093471659853068445951774704575470e-16899068596808', '7.03904e+96487182', 7, 5);
    T('8.78264615180e+4757672', '-3.400934172106532229207675e+641098087911112', '3.400934e+641098087911112', 7, 1);
    T('8.01295515955736712508e+53', '6.3195690349676415675380133256924e-393845', '8.012955159e+53', 10, 3);
    T('3.125e-6176886894', '8.79863583670676223983770212879098547e+117', '-8.79863583670676223983770212879098547e+117', 54, 5);
    T('-8.09013052357e+14', '4.259537012481954172119316184e-4525349193', '-8.09013052357e+14', 80, 5);
    T('9.2936685540316182990787682371859966370703425014183e-6372975941668', '-7.74070e-9176842818', '7.7e-9176842818', 2, 6);
    T('-7.03982357328153950104836442172439363573476283424735e-2214160048', '9.469964822077960471607208725147229e+1874432', '-9.46996482207796047160720872514722901e+1874432', 36, 3);
    T('2.646366986736900791862195371807978670212110337e-7570742343', '8e-27502757', '-8e-27502757', 28, 4);
    T('6.478388927197981392701730947108875868086124855690e+55', '-9.081084205816512036288582183817156665e-6919', '6.47838892719798139270173094710887586808612485569e+55', 97, 6);
    T('8.62651298101900435504871521448e+149687237340439', '4.968217493851530058608197272665925598915136353e+196278819', '8.62651298101900436e+149687237340439', 18, 0);
    T('-2.713128151627205e+20', '9.6937617997844464531624025643476564482904634e+119', '-9.6937617997844464531624025643476564482904634e+119', 51, 5);
    T('-9.98e+9439295', '-6.43978644389249415268887233463e-52990687', '-9.98e+9439295', 49, 3);
    T('-998201196.51', '-9.769911044378864911636481509716224384711587731e+53', '9.769911044378864911636481509716224384711587721018e+53', 50, 6);
    T('1.7287287e-853258', '5.16341674166387158032562278368928974733483699693e+5962681121', '-5.16341674166387158032562278368928974733483699693e+5962681121', 87, 3);
    T('-8.304533845562663669263202099895979113779603145282563399e+411717814', '-2.240854654555266019324612019770447611665288069605326911e+76683518165599', '2.24085465455526601932461201977044761167e+76683518165599', 39, 4);
    T('-3.1643191403155653965038896354158835457136618190782008e-48835380376', '-3.51047690621833173644292770855761310988071853472566229103e-379685443', '3.51047690621833173644292770855761310988071853472566229103e-379685443', 98, 5);
    T('6.06851071100324204760728732015478464086258028398229368784e-15', '-6.98170229543191953469e-329487468334578', '6.06851071100324204760728732015479e-15', 33, 0);
    T('8.812168230e+195928037', '5.41223123614074591994399184854581863e-37300', '8.81216823e+195928037', 82, 5);
    T('-8.81944752288509647311498115958752e-50225', '-4.447114642062554757514703583666735190e+717914339339', '4.4471146420625547575147035836667e+717914339339', 32, 5);
    T('9.3133359012714793270566901753470294360164362767184765309e-2780', '-8.34224836665615699186795e+71478860235736', '8.34224836665615699187e+71478860235736', 21, 2);
    T('-8.91e+72218501', '6.789405291001049501e+4794136669336', '-6.789405291001049501e+4794136669336', 84, 1);
    T('-9.7431523383112001399494632586e+6264686984', '-7.518698634549239e-22', '-9.7431523383112001399494632586e+6264686984', 100, 6);
    T('-3.27687367e+33445517', '5.9003396231780704263755238443510121989171997e+594815689993385', '-5.9003396231780704263755238443510121989171997e+594815689993385', 79, 1);
    T('6.803119537757e-60833', '-8.8814628749416505329754687105172624350782840618e+331093', '8.881462e+331093', 7, 3);
    T('9.082e+3362597', '-8.552197496e-297312198', '9.08e+3362597', 3, 5);
    T('8.26468415137250812124261763263275746682489599158e-334368492', '3.907887773419235913464e-5386010773', '8.264684151372508121242617632632757466824896e-334368492', 45, 0);
    T('-6.2863894703178850349454431797e-4563075745600', '-6.96213429572479254101178590214809258078619619658266556176294e+1347670', '6.962134295724792541011786e+1347670', 26, 2);
    T('9.9468477121391759201e-6264865', '-5.760605767123e-92671648', '9.946848e-6264865', 7, 6);
    T('-5.3338913112259132894883046739516290019845e+12908', '-2.434e-12261930', '-5.3338913112259e+12908', 14, 6);
    T('9.9693794833393115797997341296879113435277706247592077575e-13928402', '3.645726774091626572e+258', '-3.645726774091626572e+258', 54, 4);
    T('-5.9787514e+9276374933', '4.4054630165320787566284e-864834644791', '-5.97875140000000000000000000000000000000000000000000000001e+9276374933', 57, 3);
    T('-3.6932814404e+63082307023', '9.27296898196619742713853213152195704e+9819568', '-3.6932814404e+63082307023', 50, 5);
    T('-4.80795731717932048207228425080434901337218012083648e+413028036171', '-5e+14948797', '-4.8079573171793204820722842508043490133721801208364799999999999999999999999999999999999999999e+413028036171', 92, 2);
    T('9.504063473636884361e-75793', '-4.11362644e+1889722187609', '4.11362644e+1889722187609', 56, 1);
    T('-1.436042768535278420972264497952365389704e-42', '8.3182472e+3838', '-8.3182472e+3838', 65, 6);
    T('3.0434860406305794324429651623986e-2434878756', '6.01337461858572593495053170e+695204361045', '-6.0133746185857259349505317e+695204361045', 45, 6);
    T('-6.771053e+30419247189', '4.201149097504940e+232867351628486', '-4.20114909750494e+232867351628486', 80, 2);
    T('2.978472046317503595055518118928e-88124094050', '-5.8231656072411100139069005402803022e+10910', '5.8231656072411100139069005402803022e+10910', 74, 1);
    T('4.03894204215206685735521442197691298209640346298e+55', '-1.89441911e+838264', '1.89441911e+838264', 19, 5);
    T('-9.699264965349725814693122952397115213e+683666586', '1.01372316703178821196744292132e-234478', '-9.699264965349725814693122952397115213e+683666586', 85, 1);
    T('8.1366480819826361657344130569e-370831053423416', '-4.324962154279769321083040599035535972630505940268489e+9251', '4.324962154279769321083040599035535972630505940268489e+9251', 74, 6);
    T('6.1578e+7658276746986', '1.8462276677702691899147200451505518462579456498563908331e+398016', '6.1577999999999999999999999e+7658276746986', 26, 3);
    T('-8.724e-2750', '7.942874564230459783524701033661864722244580004e-227406', '-8.724e-2750', 68, 4);
    T('9.76929715765149587675255894737449528e-16987683923982', '4.382783252254080530108951839e+63', '-4.382783252254080530108951839e+63', 34, 6);
    T('3.7489841e-36006734', '4.2e-4635939380', '3.7489841e-36006734', 41, 0);
    T('5.5281856775565790634362692803522492708e-42382538', '-3.7997528595699464382673041149938739021553634047595039035e+142606065656590', '3.799752859569946438267304114993873902155363404759503903e+142606065656590', 55, 3);
    T('-3.5e-4885574370', '6.21935915223666937774912e+673093990175294', '-6.21935915223666937774912e+673093990175294', 63, 6);
    T('-2.77489823078656697396980960e-392975817683065', '8.0543790026802988732151764275e+896', '-8.05437900268029887321517642750000000000000000000000000000000000000000000000000000001e+896', 84, 3);
    T('5.2961929434534e+800595715863', '6.110843951692678720639893488e+209975', '5.2961929434534e+800595715863', 86, 4);
    T('5.60701700588433980756797231111712225917185650156271435872377e-28448824261', '7.53975886705727302202500e+103', '-7.5397588670572730220249999999999999999999999999999999999999999999999999999999999999999999e+103', 89, 1);
    T('-1.081018648097401833e-57128', '-2.31445185188552517934046086995666287626e+449258129841', '2.31445185188552517934046086995666287626e+449258129841', 98, 5);
    T('-4.5916247936601225313783522423203228e+468748', '-1.304275e+7573438', '1.304275e+7573438', 87, 4);
    T('-8.1625026237494177232e-27642', '-7.152084116713e-3685981695', '-8.1625026237494177232e-27642', 29, 5);
    T('4.221823758465294875190618365032e-76637975054', '87052.6477555965865300697926588818341284', '-8.70526477555965865300697926588818341284e+4', 100, 6);
    T('7980571848616455.70826823921', '-4.688392200144505470535284460e+598356', '4.6883922e+598356', 8, 1);
    T('7.661316904024615e-559', '-4.33565472171963e-67461536737466', '7.661316904024615e-559', 97, 4);
    T('-0.0195762813242194291174970514007485796905', '-5.6886542032707413534205746461028e+91840', '5.6887e+91840', 5, 2);
    T('7.040162251188070159079961698081820521639494039e-50477649928', '6.9757871472694980849242368622532269043468662581e+273705326563', '-6e+273705326563', 1, 2);
    T('2.67343125285865591002580780950815423854264281445053e-10627', '3.47240110903395e+6746709', '-3.4724011090339499999999999999999999999999999999999999999999999999999999e+6746709', 71, 2);
    T('-9.32259931974701537952006088731099e+1018708764119', '-3.42266630968103e-32200', '-9.32259931974701537952006088731099e+1018708764119', 95, 5);
    T('-1.947406131344765083926376236601686803727231356918e-76482778004', '8.2649262328234254309026457085907471054495353296432284286233e-282124934664791', '-1.947406131344765083926376236601686803727231356918e-76482778004', 49, 5);
    T('-3.87307635915726455101781209943752752929e-8559645', '5.88227484174465530e+783912411', '-5.8822748417446553e+783912411', 66, 2);
    T('3.97690313387483413569530495186643905880002791903201652348e-246442737', '1.83950019252256983281591e+8476911647858', '-1.83950019252256983281591e+8476911647858', 65, 4);
    T('3.083097410169299632e-851204', '-6.970695168598e+83928106571138', '6.970695168598e+83928106571138', 70, 5);
    T('-4.5818007106892108916721156887020496676e+24194825544105', '1.122778265796041517073226250377138510e+45769549473011', '-1.1227782657960415170732262503771385100001e+45769549473011', 41, 3);
    T('-2.8314839949076666570213307795163704603200170515618537272269e+285668484983', '-3.8944312398445693423716174e-234495053', '-2.8314839949076666570213307795163704603200170515618537272269e+285668484983', 76, 3);
    T('4072705669017307756017302673833057', '-1.676834949506673973937879882115389748986898e+9285433', '1.67683494950667397393787988211538974899e+9285433', 39, 0);
    T('-5620376.832375493782862569453866833175', '1.42652519454462929111981432817492e-89726314648', '-5.6203768323754937828625694538668331751e+6', 38, 0);
    T('9.337575504375e+75', '-5.689921485870612274406203e-7738665851', '9.3375755e+75', 9, 6);
    T('-2.116788688334753945725009978541762613203309183e+6939', '2.85721175663481159355099830158481229e-41328', '-2.1167886884e+6939', 11, 0);
    T('2.2135062742702678793004987771561314312297302692428e+6029652971030', '-4.0414225455216126260254736023707288246074784991896531e+380', '2.213506274270268e+6029652971030', 16, 6);
    T('6.985033028811558082255e+305112009', '-5.60284348346903675930293712e+854976693', '5.6028434834690367593029371200000000000000000000000000000000000000000000000000000000000001e+854976693', 89, 0);
    T('4.78998573854113396969032799283626078e-789532648067', '5.515453063468843368527300294921205898849552e-849230010580', '4.78998573854113396969032799283626078e-789532648067', 49, 4);
    T('5.162e-27368598743', '9.216253e+86', '-9.217e+86', 4, 0);
    T('-7.53824054638678841942019511802122256924118099912154e-237428', '-7.606815558680148152470854649603388970426662456905157e-86878765840413', '-7.5382405463867884194201951180212225692411809991215399999999999999999999999999999999e-237428', 83, 2);
    T('4.9543961646e+3764517221', '9.1071077931832851810015130e+73158000112714', '-9.107107793183285181001513e+73158000112714', 38, 0);
    T('1.3390206151e-7022055', '4.471504207339408839963092984684988e+35', '-4.471504207339408839963093e+35', 26, 3);
    T('-3.75530644362195795937989977380041973581e-459713', '-8.7836248880886605990976453006005876586706009028e-7851342', '-3.755306443621957959379899773800419735809999999999999999999999999999999999999999999999999e-459713', 88, 1);
    T('9.9555442313628488942526395237e+522570344', '-5.53646539e-6227088447258', '9.9555442313628488942526395237e+522570344', 64, 6);
    T('-2.8466706343862945523389115588906167466518867201323e-35675943', '-21192951077258738149.372198035793', '2.1192951077258738149372198035793e+19', 46, 5);
    T('8.718391342346087652446366204153116364559312361390e+53283145214', '-6.650769901034848896370645247454927361484680422589615e+6199875009440', '6.650769901035e+6199875009440', 13, 5);
    T('4e+38804307', '-8.235433774e+70', '4e+38804307', 46, 5);
    T('-2.3e+10987', '-8.63049562579422581942291460100805637787076735031e-17746764', '-2.3e+10987', 83, 3);
    T('1.02906406086986233531211098359299e+2434', '-8.61631332068263395571480843873538193e-67692602638', '1.029064060869862335312e+2434', 22, 6);
    T('-9.056366986708973493926592484967889035790310e+176210082893', '7.8893417062117294166870755909295755864431585e+406753255075576', '-7.88934170621172941668707559092957558644315850001e+406753255075576', 48, 3);
    T('-7.6e+7125885819245', '2.510437871563e+8454', '-7.6e+7125885819245', 14, 2);
    T('-5.2712237256212885305388e+1759', '-1.68114795716863833145742191780721678266461e+11330', '1.68114795716863833145742191780721678266461e+11330', 92, 0);
    T('1.093166377204694828792613641559418747963507715536015283e+36480445358580', '-2.39069308519535481e+8672', '1.0931663772046948288e+36480445358580', 20, 4);
    T('9.50945920459645e-892246449062', '-1.052369336e-39127', '1.052369336e-39127', 64, 6);
    T('7.34684216393701765635826890047397012405292059585829393e-8086640216', '3.1e-808035', '-3.1e-808035', 32, 3);
    T('6.7619992991129793199673976797026477046200494792149316147e+4044757', '8.0526983980418998034498632020179545978819887424123509e+570', '6.7619992991129793199673976797026477046200494792149316147e+4044757', 61, 6);
    T('-8.3620269862255064700470951717423755868866388103e+516648', '8.9817757859006579925077401727599516609318529424660e-22194989399066', '-8.3620269862255064700470951717423755868866388103e+516648', 78, 4);
    T('1.9e-5421633199782', '-8.704358e+798', '8.7043580000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+798', 92, 0);
    T('-7.6872513e-74311992491', '-5.61256875372394580066452766535784301859613124278746914459e+400951333832', '5.61256875372394580066452766535784301859613124278746914459e+400951333832', 61, 5);
    T('1.326706863834723331e-223', '5.150567564021437924464737496208034253328118376598033990e-6086330', '1.326706863834723331e-223', 72, 2);
    T('2.5961377344474277247133813657436084433003991613182e-32163033406', '-3.085990295016751e+19', '3.085990295016751e+19', 43, 5);
    T('-8.13951893234390607444868069329243969584752016804409e-505952492651', '-2.18200513844425267479036796746891880540727424335850693672e+18522442627', '2.18200513844425267479036796746891880540727424335850693672e+18522442627', 91, 5);
    T('-5.86727429721746810154422060909717104011448675e+109', '4.607986896625680615982488643594e+818911109891', '-4.607986896625680615982488643594e+818911109891', 86, 2);
    T('-2.97931527516921090153358e+317922352', '-7.1133945211347090391122391105025e+86384', '-2.97931528e+317922352', 9, 5);
    T('-9.27039537399e-8645291509', '8.052229021811140187639742617680047429e-98787', '-8.052229021811140187639742617680047429e-98787', 41, 5);
    T('-3.2253533501217806873702845e+3002180237713', '8.81e-968009583530502', '-3.2253533501217806873702845e+3002180237713', 90, 6);
    T('3.7479518121565427643069126137739586038123418574e-42', '-4.23496785e-6615', '3.747951812156542764306912613773958603812341857400000000001e-42', 58, 2);
    T('-2.78593105963504043518504904292294485350598813677583402245e+93417194808198', '-9.28267091860328548545937012898025376035651253060080e-446', '-2.78593105963504043518504904292294485350598813677583e+93417194808198', 51, 4);
    T('-3.6607840036307402e+65055600635812', '-8.6043866e+84122817906', '-3.6607840036307402e+65055600635812', 45, 6);
    T('6.610307812300e+8263206492', '-4.66705521700187893834704333616e+9184335786789', '4.66705521700187893834704333616e+9184335786789', 37, 5);
    T('9.9489435228893554148677710990954795731248265444595957519845e-339957654', '5.6995272e-279104591941290', '9.9489435228893554148677710990954795731248265444595957519845e-339957654', 61, 2);
    T('5.80411399172421186351077795245439585852881073e-28776396986', '6.913715684219961224440628548345668028418247232189e-897956874225162', '5.804e-28776396986', 4, 1);
    T('5.030764578626050000859502886892541124903e+785108823', '1.701322e-369237', '5.0307645786260500008595028e+785108823', 26, 1);
    T('-9.0e-843537842270', '8.165210488666497722921864760093e+26322148', '-8.16521048866649772292186476e+26322148', 28, 6);
    T('-4.206969088527767445348593961e-1527', '2.926286784180254919309190427731767562590786763e-644803334904017', '-4.2069690885277674453485939610000000000000000000000000000000000000000000000000000000000000000000001e-1527', 98, 3);
    T('7.93871663e-417922186591332', '-8.3994267362626338690e-9645042710999', '8.39942673626263386900000000000000000000000000000000000000000000000000000000000000001e-9645042710999', 84, 0);
    T('-7.714585133804032125738717419992023499120089021278038e+9921990395', '2.635356314093645031686475933049478894893103e+77', '-7.71e+9921990395', 3, 5);
    T('9.351254653206824114e-58', '-2.168913423021036732923677304003954013891964245806e-7172095722783', '9.351254653206824114e-58', 87, 4);
    T('-8.91508179328541e-301852769425', '8.720192113572216149179820139692109314159e-21709', '-8.720192113572216149179820139692109314159e-21709', 40, 5);
    T('-9.6996037967e+5806270268', '5.9901539731768567722609099e+75290285747', '-5.990153973e+75290285747', 10, 6);
    T('-6.369184135023627537926808870e+973120444550', '4.32526075e+81049994', '-6.369184135023627537926809e+973120444550', 25, 6);
    T('-7.2313490802624290190e+4799126917', '-7.0606824929769362045330910641318e+35', '-7.23134908026e+4799126917', 12, 4);
    T('-6.584627626730460776041929351252358e+6891951714293', '-3.4261785049681325824206996154730656708803092449366085e-9474745625', '-6.584627626730460776041929351252358e+6891951714293', 55, 4);
    T('1.495030743242827876750600449775460119e+23720534', '1.4764840341075606302336470078730415231724398819441763231e-48685462266225', '1.49503074324282787675060044977e+23720534', 30, 1);
    T('-3.691833172748836738519809227661348955176319418105909186104e+73923', '-7.76993382617137615530821472721242548e-4416891270209', '-3.69183317e+73923', 9, 5);
    T('327683573754178287940406.55279', '-3.42709289706718966457308561264837300958866728116283916229087e-96345440796199', '3.2768357375417828794040655279e+23', 46, 3);
    T('1.1552280009747854869419389801812e+198407875', '4.76196670814790165079886856068158000210686914e+19353466507', '-4.76196670814790165079886856068158000210686914e+19353466507', 64, 4);
    T('-8.041e-43232651433316', '998423992651206337.9235850345459585121', '-9.984239926512063379235850345459585121e+17', 48, 6);
    T('-6.71830892741965481884340661237903331047298310499761066815e-8491406', '7.411431768530396765945653851156875772826643454929e-711085299', '-6.718308927419654818843406612379033310472983104998e-8491406', 49, 0);
    T('-8.238034860317127989192170831678258e-42230100362', '3.10046165650381389416931e-5400267269261', '-8.238034860317127989192170831678258e-42230100362', 54, 2);
    T('-7.7559723481528330302483939150191712590006301e+76840187802', '-7.356666962038491297421275260786e-5136586491', '-7.7559723481528330302483939150191712590006301e+76840187802', 92, 4);
    T('-9.9814750e+9525', '293187834254.189', '-9.981475e+9525', 32, 5);
    T('-8.16956098093041244689839199956833947083193407e+117', '6.271206451119744348e-29754', '-8.16956098093041244689839199956833947083193407e+117', 91, 2);
    T('1.757e-334870', '2.254471e-201025546995392', '1.756999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e-334870', 97, 3);
    T('-8.118178790001618059583629599852503948693287685411586e-8874748550991', '-3.2582403648564159334426494382455930761e+344', '3.2582403648564159334426494382455930761e+344', 93, 2);
    T('-5.768389868559837057258659590e+365298500', '8.04854535635576972465393328552769057823214611081030256e-32501', '-5.768389868559837057258659590000000000000000000000000000001e+365298500', 58, 0);
    T('7.031504272872572993e+31491539619', '-8.8115521636702773463216554994134486031706300428507011492e-69997306', '7.031504272872572993e+31491539619', 42, 6);
    T('-4.5375068964e-502', '9.211509534061195459566080779645697510e-73730249557139', '-4.537506896400000000000000000000000000000000000000000000000000000000000000000000000000000001e-502', 91, 0);
    T('-8503717971.6995799', '9e-764384', '-8.5037179716995799e+9', 73, 1);
    T('5.24144658579084579e-841576141117', '-2.6742368422e+277767357838257', '2.6742368422000000000001e+277767357838257', 23, 2);
    T('-9.2273954494400216149260032533465970776686693883171693e+651784286', '9.50657078432639458080747177152207595459382812827212e+7602264083309', '-9.5065707844e+7602264083309', 11, 0);
    T('6.53679847e-610447', '4.80737995488706971e-7289032609', '6.53679846999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e-610447', 96, 1);
    T('8.6800597347178208232077245041734174883e+236', '-3.3116974428860301969869168007236199113e-62', '8.6800597347178208232077245041734174883e+236', 69, 4);
    T('-8.740335430285042278651047887832453108263378951859e-4859', '8.32138947060999224641829725475834167681173947e-7935656444', '-8.740335430285042278651047887832453108263378951859e-4859', 81, 4);
    T('9.39376492088796640860003757215e-405665', '-5.743875951460721063533371880930460453629489278582343539172e-94036404307911', '9.3937649208879664086e-405665', 21, 1);
    T('-5.5923976572304391023907e+69030310338527', '-4.9200222203850965939304369094984123861e+46522549', '-5.5923976572304391023906999999999999999999999999999999999999999999999999999999999999999999999999999e+69030310338527', 98, 2);
    T('8.42210245489542327466648453605207035048e-22573081546', '8.9203651634547080730444392741188297340042735031834540015e+46448738146', '-8.9203651634547080730444392741188297340042735031834540015e+46448738146', 85, 5);
    T('-8.483965314299892385641777610318173309630704e+67590794856', '-1.76157023065163295115e+31492557257', '-8.483965314299892385641777610318173309630704e+67590794856', 89, 5);
    T('-3.697723425416264941309214460290474896e+96724600298', '9.1930130728e+185', '-3.6977234254162649413092144602904748960000000000000000001e+96724600298', 56, 3);
    T('9.1955063219339202591052369905434e+153758', '4.7720323956965093470e+944986783', '-4.772032395696509346e+944986783', 19, 2);
    T('6.74671218677440092501279904012240635824718e-75144216356072', '-5.78967582152400e+568', '5.78967582152400000000000001e+568', 27, 0);
    T('4.936989001e-2402', '7.175246425229612950449115033e-789736', '4.936989001e-2402', 22, 4);
    T('9.35892443915e+48', '7.5771051805145872308878830403e-578230406786', '9.35892443915e+48', 29, 0);
    T('8.718856129466237052010448780218241040438365313651755e+65281122', '3.9091564220910230373200999822e-92522137952', '8.7188561294662370520104487802182410404383653136517549999999999999e+65281122', 65, 3);
    T('8.860162731345051507486811530063537094664254095140721770e+87288043141', '5.05817360411235451383225489764340664e-74931135287', '8.8601627313450515075e+87288043141', 20, 6);
    T('6e+34920490323', '-6.68201087846e-291368483562', '6e+34920490323', 93, 5);
    T('1.6304654046e+886', '-9.14e-82042', '1.6304654046e+886', 33, 5);
    T('6.297532316569538596146893396833310e+334916954307', '3.12158642462566074665122764e-879', '6.29753231656953859614689339683331e+334916954307', 83, 0);
    T('5.7636241079779e+22', '6.7960541841471291776682280911464338664246967261e-91019637', '5.7636241079779e+22', 68, 0);
    T('-6.136757671975631631e-272302711987315', '6.3920185117213473361385430720010743328202945114891900877e+71588', '-6.39201851172134733613854307200107433282e+71588', 39, 6);
    T('2.83089457994164000929337454223e-597', '-5.7366927711e+850', '5.7367e+850', 6, 2);
    T('6.660189125473380664846441216723063516e-11329477', '-2.12e+30', '2.1200000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+30', 89, 2);
    T('-5.71571831224602052351912988363938290096756e-55672426', '3.9396706533727544199494158029782748384590925830981365815e-52754724333', '-5.7157183122e-55672426', 11, 1);
    T('6.58e-36', '-5.883071675444150e-7903988847833', '6.58e-36', 17, 4);
    T('-9.9e-371', '938240825879143140249.71958370', '-9.382408258791431402497195837e+20', 83, 1);
    T('9.4694503263056127089902e+203823082740', '-2.0502951566761667973498188776119066219119027593838e-14496835580994', '9.4694503263056127089902e+203823082740', 40, 5);
    T('-4.4618426518278302049349587677e-4572023743443', '-2.456043099155820518631291359362808490026388e+237126294698358', '2.456043099155820518631291359362808490026387999999999999999999999999999999e+237126294698358', 73, 1);
    T('6.4183240447942498484742863150541290676985915627e-43117421333', '9.6321698027742680761873194e+45', '-9.6321698027742680761873194e+45', 87, 4);
    T('9.2488431124588839961715801251243332470e+520362302314882', '7.031737249278282784588551e+31', '9.248843112458883996171580125124333247e+520362302314882', 77, 5);
    T('-1.185800580243972685071691645584725113429879365732587927507e+5776678555', '1.6932155411749759700853091749037016398630140811826e-6815087', '-1.185800580243972685071691645584725113429879365732587927507e+5776678555', 78, 4);
    T('-5.11330778646833e+2002595', '1.03442679773394151998273328260109994782333e-93941476916', '-5.11330778646833e+2002595', 45, 6);
    T('-4.9708767272745e+2190182', '-8.5958138129790698899906478111953323082910899e-533', '-4.9708767272745e+2190182', 30, 0);
    T('-7.6320535193058955721302136092839953131e+19262482', '8.75873671013682197e-212714', '-7.6320535193058955721302136092839953131e+19262482', 46, 4);
    T('-6.2207764e-317670540778', '6.664832026494935898238833202489215513630423842642087203437e+84558338662526', '-6.66483202649493589823883320249e+84558338662526', 30, 5);

    log('\n ' + passed + ' of ' + total + ' tests passed in ' + (+new Date() - start) + ' ms \n');
    return [passed, total];
})(this.Decimal);
if (typeof module !== 'undefined' && module.exports) module.exports = count;
