var count = (function mod(Decimal) {
    var start = +new Date(),
        log,
        error,
        undefined,
        passed = 0,
        total = 0,
        n = 'null',
        N = 'NaN',
        I = 'Infinity';

    if (typeof window === 'undefined') {
        log = console.log;
        error = console.error;
    } else {
        log = function (str) { document.body.innerHTML += str.replace('\n', '<br>') };
        error = function (str) { document.body.innerHTML += '<div style="color: red">' +
          str.replace('\n', '<br>') + '</div>' };
    }

    if (!Decimal && typeof require === 'function') Decimal = require('../decimal');

    function assert(expected, actual) {
        total++;
        if (expected !== actual) {
           error('\n Test number: ' + total + ' failed');
           error(' Expected: ' + expected);
           error(' Actual:   ' + actual);
           //process.exit();
        } else {
            passed++;
            //log('\n Expected and actual: ' + actual);
        }
    }

    function T(a, b, expected, sd, rm) {

        if ( sd != null ) {
            Decimal.precision = sd;
            Decimal.rounding = rm;
        }
        assert(String(expected), String(new Decimal(a).mod(b)));
    }

    function isMinusZero(n) {
        return n.toString() === '0' && n.s == -1;
    }

    log('\n Testing mod...');

    Decimal.config({
        precision: 400,
        rounding: 4,
        toExpNeg: -7,
        toExpPos: 21,
        minE: -9e15,
        maxE: 9e15,
        errors: false,
        modulo: 1
    });

    T(1, 0, N);
    T(1, -0, N);
    T(-1, 0, N);
    T(-1, -0, N);
    T(1, N, N);
    T(-1, N, N);
    T(1, I, 1);
    T(1, -I, 1);
    T(-1, I, -1);
    T(-1, -I, -1);
    assert(false, isMinusZero(new Decimal(0).mod(1)));
    assert(false, isMinusZero(new Decimal(0).mod(-1)));
    assert(true, isMinusZero(new Decimal(-0).mod(1)));
    assert(true, isMinusZero(new Decimal(-0).mod(-1)));
    T(0, 0, N);
    T(0, -0, N);
    T(-0, 0, N);
    T(-0, -0, N);
    T(0, N, N);
    T(-0, N, N);
    assert(false, isMinusZero(new Decimal(0).mod(I)));
    assert(false, isMinusZero(new Decimal(0).mod(-I)));
    assert(true, isMinusZero(new Decimal(-0).mod(I)));
    assert(true, isMinusZero(new Decimal(-0).mod(-I)));
    T(N, 1, N);
    T(N, -1, N);
    T(N, 0, N);
    T(N, -0, N);
    T(N, N, N);
    T(N, I, N);
    T(N, -I, N);
    T(I, 1, N);
    T(I, -1, N);
    T(-I, 1, N);
    T(-I, -1, N);
    T(I, 0, N);
    T(I, -0, N);
    T(-I, 0, N);
    T(-I, -0, N);
    T(I, N, N);
    T(-I, N, N);
    T(I, I, N);
    T(I, -I, N);
    T(-I, I, N);
    T(-I, -I, N);

    T(1, '0', N);
    assert(false, isMinusZero(new Decimal(1).mod(1)));
    T(1, '-45', '1');
    T(1, '22', '1');
    T(1, 0144, '1');
    T(1, '0144', '1');
    T(1, '6.1915', '1');
    T(1, '-1.02', '1');
    T(1, '-1.02', '1');
    T(1, 2, '1');
    T(1, -2, '1');
    T(-1, 2, '-1');
    T(-1, -2, '-1');

    /*
       Due to the double precision binary floating point representation of 0.1 the next 4 results
       match BigDecimal's remainder method, but not Javascript's % operator.

       0.1.toFixed(18)                              // '0.100000000000000006'
       new Decimal(1).mod('-0.100000000000000006')  // '0.099999999999999946'
     */
    T( 1, -0.1, '0');               // JS:   0.09999999999999995
    T(-1, -0.1, '0');               // JS:  -0.09999999999999995
    T( 2, -0.1, '0');               // JS:   0.0999999999999999
    T(-2, -0.1, '0');               // JS:  -0.0999999999999999

    T( 2, -0.5, '0');               // JS:   0
    T(-2, -0.5, '0');               // JS:   0
    T(999.99, -0.99, '0.09');
    T(-999.99, -0.99, '-0.09');
    T(999.99, '3.01', '0.67');
    T(-999.99, '3.01', '-0.67');
    T(1, '0.09', '0.01');
    assert(false, isMinusZero(new Decimal(1).mod('-0.0001')));
    assert(true, isMinusZero(new Decimal(-0).mod(1)));
    assert(true, isMinusZero(new Decimal(-0).mod(0.1)));
    assert(true, isMinusZero(new Decimal('-0').mod('-1')));
    assert(true, isMinusZero(new Decimal('-0').mod(Infinity)));
    T(1, '8e5', '1');
    T(1, '9E12', '1');
    assert(false, isMinusZero(new Decimal(1).mod('1e-14')));
    T(1, '3.345E-9', '6.1e-10');
    T(1, '-345.43e+4', '1');
    T(1, '-94.12E+0', '1');
    T(1, '', N);
    T(1, '    ', N);
    T(1, '\t\t', N);
    T(1, 'ertgrt546', N);
    T(1, 'qweqwdewee', N);
    T(1, true, N);
    T(1, false, N);
    T(1, 'e 4.3', N);
    T(1, '4 .3', N);
    T(1, '4.0 01e', N);
    T(1, ' 4.001', '1');
    T(1, '4.001 ', '1');
    T(1, ' 4.001 ', '1');
    T(1, '    4.001', '1');
    T(1, ' 4.0 01', N);
    T(1, '4. 001', N);
    T(1, '4. 001 ', N);
    T(1, '  4.001e ', N);
    T(1, ' 4 .001 e ', N);
    T(1, undefined, N);
    T(1, null, N);
    T(1, Number.POSITIVE_INFINITY, '1');
    T(1, Number.NEGATIVE_INFINITY, '1');
    T(1, new Date(2012, 11, 4), N);
    T(1, new Object(), N);
    T(1, function () {}, N);
    T('0', 0, N);
    T(0, '+0', N);
    T('0', '0', N);
    T(3, -0, N);
    T(9.654, 0, N);
    T(0, '0.001', '0');
    T(0, '111.1111111110000', '0');
    T(N, '0', N);
    T(-1, 1, '0');
    T(-0.01, 0.01, '0');
    T(54, -54, '0');
    T(9.99, '-9.99', '0');
    T('0.0000023432495704937', '-0.0000023432495704937', '0');
    T(NaN, NaN, N);
    T(NaN, N, N);
    T(N, NaN, N);
    T(N, 4, N);
    T(N, '4534534.45435435', N);
    T(N, 99999.999, N);
    T(Infinity, '354.345341', N);
    T(3, -I, '3');
    T(-Infinity, -I, N);
    T(-I, -Infinity, N);
    T(I, '-999e999', N);
    T('1.21123e43', -I, '1.21123e+43');
    T('-999.0', I, '-999');
    T('657.342e-45', -I, '6.57342e-43');
    T(I, 123, N);
    T(-0, I, '0');
    T(100, 100, '0');
    T('10 ', 4, '2');
    T('03.333', -4, '3.333');
    T(43534.5435, '0.054645', '0.019545');
    T('99999', '1', '0');
    T('3e', 8, N);
    T('-3..0', 13, N);
    T('0 0', -0.4, N);
    T(' +3e0', 4, '3');
    T(9.9806, '+ 1', N);
    T(' +2 0', '1e1', N);
    T('e3', 4, N);
    T(' ', 0, N);
    T(323, null, N);
    T(undefined, undefined, N);
    T('undefined', undefined, N);
    T(null, null, N);

    T('-0.111', '21', '-0.111');
    T('0.000621', '-1', '0.000621');
    T('6.41', '11', '6.41');
    T('21', '11', '10');
    T('1531', '-11', '2');
    T('21', '-0.0161', '0.0056');
    T('-41', '-21', '-20');
    T('31', '-1', '0');
    T('31', '2.21', '0.06');
    T('-842658.56', '-19.52', '-19.2');
    T('-34.871', '-7325043345', '-34.871');
    T('78691958', '49006.018397', '37298.472815');
    T('-11618324684', '621.3', '-394.1');
    T('0', '69.07362125', '0');
    T('0.27474921216', '0.00000000002460017868', '2.224673016e-11');
    T('0', '59137483772', '0');
    T('-0.000000000000000000014634493', '-4.2', '-1.4634493e-20');
    T('-5.470', '-1.8989', '-1.6722');
    T('-95519.5319132', '4.1', '-1.8319132');
    T('43.257', '-4704608304', '43.257');
    T('-32128.403', '68263.11', '-32128.403');
    T('85.4005884', '-0.000000000000000010953667', '4.132633e-18');
    T('3197.909', '-10738864.39', '3197.909');
    T('1.03436722695', '2.6', '1.03436722695');
    T('-4', '-56.36464628', '-4');
    T('92.0618', '-454.866', '92.0618');
    T('-6.147123243', '-53259.9', '-6.147123243');
    T('-3', '-0.4911', '-0.0534');
    T('0', '-12.3', '0');
    T('-0.001713264153', '0.00000000000000000001843605', '-1.36779e-20');
    T('2', '-9325.1665961', '2');
    T('42016273.6500', '-122.861496017', '99.24210634');
    T('35.9', '3867744417', '35.9');
    T('0.000000000000000479760268839', '-209975.082', '4.79760268839e-16');
    T('0.0000000000000000000428713848', '3', '4.28713848e-20');
    T('0.000000000000000104698757876', '-15902.7654', '1.04698757876e-16');
    T('-4', '3124473.52104', '-4');
    T('-9.58', '1.00', '-0.58');
    T('79', '1', '0');
    T('0', '-0.0000000000013618', '0');
    T('1.55', '-31', '1.55');
    T('-2.6', '-13.58133', '-2.6');
    T('0', '90950', '0');
    T('-46.1', '61.8', '-46.1');
    T('-0.00000000000023461', '-0.00000000134', '-2.3461e-13');
    T('-9761.89778', '4645363921.75', '-9761.89778');
    T('0.000000000000000090372203', '0.00000322253', '9.0372203e-17');
    T('-23560577.226', '-0.5304109598', '-0.3270269352');
    T('-11752006.868', '0.003773', '-0.002016');
    T('0.000000036524', '-1.2', '3.6524e-8');
    T('4713.9562', '0.000000000000000000561', '2.23e-19');
    T('-351870798', '28.8', '-1.2');
    T('-4.6', '-0.00000000058', '-4.4e-10');
    T('-21863.82', '4214.9937962', '-788.851019');
    T('-30.221041721', '20401.1', '-30.221041721');
    T('-2.35', '3', '-2.35');
    T('64462', '90.840', '56.44');
    T('46', '433563.55477', '46');
    T('47852501568', '-147426331397', '47852501568');
    T('0', '3812504.1', '0');
    T('0.000000000000142963770', '1210345642.9', '1.4296377e-13');
    T('-0.0024002323743', '-0.000000000000000013220659', '-3.535752e-18');
    T('225.773', '66128.74787', '225.773');
    T('-158.4165', '12.78', '-5.0565');
    T('-21042099137', '10.935', '-4.565');
    T('32791.03', '411590460', '32791.03');
    T('-1984910.86246', '-153845.7', '-138762.46246');
    T('75010460639.6', '-0.00000000005103249', '2.835305e-11');
    T('-0.00000000000000025595509', '25.023', '-2.5595509e-16');
    T('-103052291', '-350.4', '-1.4');
    T('0.00000000000000015307870924', '-0.0000035499', '1.5307870924e-16');
    T('4402230', '122.112', '92.4');
    T('-3073677', '532.95653726', '-116.64962158');
    T('-0.026619162', '-0.00000000000000010', '0');
    T('0.000065666975666', '-6.558', '0.000065666975666');
    T('-3.3', '0.000000000033194405282', '-1.2721464422e-11');
    T('94101.0', '-810285.8180', '94101');
    T('229868', '9654.7078', '7809.7206');
    T('0.000000001926', '-0.00000000000000000005918', '2.562e-20');
    T('67', '0.00000006016496', '2.00248e-8');
    T('0.0000000000000000000832655', '-327907966408', '8.32655e-20');
    T('-1534', '-293318.339', '-1534');
    T('0.00000000000000000511522652', '-1145.46888006', '5.11522652e-18');
    T('0.00000000465574', '707.0', '4.65574e-9');
    T('0.230910067', '-1712.01', '0.230910067');
    T('-31874.7', '-2617976302', '-31874.7');
    T('22.8', '3819', '22.8');
    T('-0.0000015419', '329', '-0.0000015419');
    T('-9176.3441', '741523.991', '-9176.3441');
    T('-5.1103', '-25460.136', '-5.1103');
    T('15232730973', '-2.28', '1.32');
    T('0.081', '-1111.578', '0.081');
    T('3.1', '-42.207', '3.1');
    T('-0.000000000000000433', '0.00000000000000000254528412564', '-3.016986412e-19');
    T('-83943.1', '-4303134750', '-83943.1');
    T('19.825539', '4432543.44', '19.825539');
    T('-2416', '8097.16337', '-2416');
    T('-3.836', '0.00000823841385579', '-0.00000502522549283');
    T('2290356160', '-965232.20', '825381.6');
    T('1.12', '-35806.5278', '1.12');
    T('0', '-0.0000144726783', '0');
    T('-1571629111.33', '6.4736', '-3.8676');
    T('-0.0053778', '1.1', '-0.0053778');
    T('-0.0000013840', '-12.1', '-0.000001384');
    T('-22.9310218204', '-1.6843970', '-1.0338608204');
    T('-71.300', '-25.868806', '-19.562388');
    T('0.02847', '-63397392.02', '0.02847');
    T('-0.0000000000000331096', '0.00000183271', '-3.31096e-14');
    T('-27821115', '-34.83906', '-0.40734');
    T('0.000000000000436294954175', '-2261.01154', '4.36294954175e-13');
    T('45.86441', '-530611.936214', '45.86441');
    T('-1999679428', '0.00000000000000000002297558409', '-5.828173e-23');
    T('-105924.2800', '-426.10612', '-249.96224');
    T('754.9', '-1', '0.9');
    T('0.000000238266109662', '29355.83', '2.38266109662e-7');
    T('25.3', '-0.00000000000000086417416', '5.8408656e-16');
    T('-1261.3', '39.0', '-13.3');
    T('1.017', '-153', '1.017');
    T('27.690', '-21507592', '27.69');
    T('-2623.072516', '3.06', '-0.652516');
    T('91', '13034138.923', '91');
    T('-5212749.21', '-0.00000000000000000001986395', '-2.1827e-21');
    T('117.4212', '-29.978', '27.4872');
    T('-47179.732', '-77504621680', '-47179.732');
    T('0', '-3.7234', '0');
    T('443705716.830', '3024.9575', '1925.7725');
    T('-0.000000000000000000017191196518', '-14.725715414', '-1.7191196518e-20');
    T('30.4', '399719414108', '30.4');
    T('16145.97', '-48642', '16145.97');
    T('2765166', '-18.6825', '6.54');
    T('37.63', '-1284.342', '37.63');
    T('-0.0000000000000000000177845858', '0.0000005860954024', '-1.77845858e-20');
    T('-2.3', '2582.24565000', '-2.3');
    T('16.7', '-0.0000000017369', '1.362e-9');
    T('170180366706', '4626.181', '2388.659');
    T('-1404312.6', '-17000256053', '-1404312.6');
    T('-7603530366', '316.496322753', '-236.943012537');
    T('8', '1.59866', '0.0067');
    T('-4.963', '8.73', '-4.963');
    T('25644160.8', '-5842.06855', '3321.93405');
    T('-4.31384', '-220.2', '-4.31384');
    T('186.9', '-48200.85', '186.9');
    T('5', '57089.99', '5');
    T('-244278052979', '-2', '-1');
    T('8', '-5763.4', '8');
    T('29549', '-1', '0');
    T('-2.0', '-0.00000000000000446955628', '-3.76062704e-15');
    T('737330.9', '-746370915533', '737330.9');
    T('-0.000882183174', '-16.5', '-0.000882183174');
    T('-45', '0.000000001106035', '-7.5858e-10');
    T('-594.06', '-29.85802821', '-26.75746401');
    T('-27.648237', '-1047.5249', '-27.648237');
    T('2', '-739163.624', '2');
    T('544827.1330', '54848981.1', '544827.133');
    T('1', '0.00000000000076553', '2.9959e-13');
    T('-10.89', '0.000000000000000000047', '-5e-21');
    T('-3052.938', '9327.5819201', '-3052.938');
    T('-24.1', '127758.429', '-24.1');
    T('3.9', '12173.38', '3.9');
    T('0.0000000000000213926861909', '-77.4', '2.13926861909e-14');
    T('-90173804.260', '-5062619.06909', '-4109280.08547');
    T('637', '43943.2610', '637');
    T('14867.1', '894449', '14867.1');
    T('7.3', '-1.1', '0.7');
    T('2894844.4', '-1', '0.4');
    T('-0.00000000000196', '-2.6741', '-1.96e-12');
    T('0.00000000000000071433493', '5.9227', '7.1433493e-16');
    T('69', '-7204969', '69');
    T('-12468163', '-10022891.867', '-2445271.133');
    T('45469.1932906', '-3095367650', '45469.1932906');
    T('73072906.39', '-112013211', '73072906.39');
    T('1.4', '972.159255713', '1.4');
    T('66924.3855375', '19.93', '19.3755375');
    T('26.4256214076', '11.4775634275', '3.4704945526');
    T('23362.0174956', '-0.0000000001748', '1.536e-10');
    T('18276818006', '-5.74', '1.1');
    T('-8319.8', '3', '-0.8');
    T('-7.7413714358', '43763934316.0', '-7.7413714358');
    T('-688566579', '-1.83', '-0.3');
    T('29854', '-268509731.22', '29854');
    T('-194.15246', '-3.2255572', '-0.619028');
    T('2.0186177', '-326915', '2.0186177');
    T('-4', '1259', '-4');
    T('3.56', '-0.000000000000000000515993', '1.2338e-19');
    T('0.000001783', '-16426085', '0.000001783');
    T('-1766987819', '-87.259', '-49.72');
    T('-17.1', '0.00000000000000021840564', '-5.750892e-17');
    T('-8589.659', '0.000000000000000505', '-3.75e-16');
    T('-65891.7', '-38663.83', '-27227.87');
    T('-1.0', '0.00000000000000003836', '-1.78e-17');
    T('-33378.26', '-1226528.13', '-33378.26');
    T('2', '492342441', '2');
    T('1274', '-3092459411.0', '1274');
    T('0.2631770', '-493249535.4', '0.263177');
    T('-0.00000000000000000141', '4201192116', '-1.41e-18');
    T('-574.29154', '133427.3', '-574.29154');
    T('104617.3', '293188753.50', '104617.3');
    T('0', '38.96345', '0');
    T('-0.0000017289693288', '-5.4', '-0.0000017289693288');
    T('0', '4', '0');
    T('9.65', '1.2', '0.05');
    T('0.000000000017', '-50377365', '1.7e-11');
    T('137.3064', '0.0000001012', '7.04e-8');
    T('-9.89405', '29.2', '-9.89405');
    T('3.23', '-3048751.09', '3.23');
    T('87020568364', '-1', '0');
    T('-3658', '-0.4067222', '-0.3472554');
    T('0.0000000000483229749', '-508858', '4.83229749e-11');
    T('3', '-0.000000000000000006525172012', '6.0850742e-19');
    T('-52.63', '-373.710', '-52.63');
    T('-6.49915814', '352.2', '-6.49915814');
    T('0', '-44190.198', '0');
    T('0.00000000000000003763584929', '6', '3.763584929e-17');
    T('1.299983', '194.7', '1.299983');
    T('1', '9.52', '1');
    T('-86603.9', '446867516', '-86603.9');
    T('-1482885.420', '-0.0000000018878840', '-9.643e-10');
    T('-170.531717', '-633436587.6', '-170.531717');
    T('15.36378', '6540.267', '15.36378');
    T('-20570.61', '18.955913858', '-3.44346407');
    T('77', '0.000000000000027', '2.3e-14');
    T('0.000000001687849', '-0.0131699248663', '1.687849e-9');
    T('-0.000000000000026', '3291407.1', '-2.6e-14');
    T('-290.613', '0.00000000000000000008353794', '-6.386436e-20');
    T('200', '66.97', '66.06');
    T('2.2', '-43.41589', '2.2');
    T('98201.177964', '0.007000', '0.004964');
    T('35.453991547', '128.68861', '35.453991547');
    T('5.27', '1', '0.27');
    T('-1', '0.000119683476689', '-0.000044552263405');
    T('-34.0', '3', '-1');
    T('3378', '-44180', '3378');
    T('15', '-4', '3');
    T('0.0000004190944243', '37285824.4', '4.190944243e-7');
    T('-298.4', '506466', '-298.4');
    T('-0.00000010893', '-0.000426882829396', '-1.0893e-7');
    T('-1.9756065817', '941.5', '-1.9756065817');
    T('-1026297.2', '-205657.802', '-203665.992');
    T('2', '0.0000000005621541447', '3.513430991e-10');
    T('-7951901.67', '-0.0000000000055383', '-2.3814e-12');
    T('9', '0.000000000015125851830', '7.24233339e-12');
    T('72.245762', '-39.70217', '32.543592');
    T('6.62321', '-912632', '6.62321');
    T('617021', '-42.5970821', '2.2657815');
    T('1469815.28770', '537.217915', '524.290175');
    T('345897.6', '-0.00000000000108024', '9.7032e-13');
    T('0', '1.8688906295', '0');
    T('164', '-82114.2', '164');
    T('57.84537', '-1.4', '0.44537');
    T('114.340500', '6333334050', '114.3405');
    T('4.80', '-6077773.6', '4.8');
    T('162', '-37.75886044', '10.96455824');
    T('0.000000016367092', '1', '1.6367092e-8');
    T('148.2', '-46', '10.2');
    T('1.5768318844', '-7512089036.8', '1.5768318844');
    T('-0.0000000000000014460', '0.0011257005', '-1.446e-15');
    T('64.079', '-141', '64.079');
    T('-0.0582', '-13.35353476', '-0.0582');
    T('-1.0007', '63155.153933', '-1.0007');
    T('-623313173.6', '0.0000000081031', '-2.4548e-9');
    T('-36.9', '-1', '-0.9');
    T('2.0857', '0.000000000000000000205859744928', '1.887979864e-19');
    T('0.0000000494417', '492420240', '4.94417e-8');
    T('0.000000000000012194', '0.000000000000000013778591965', '1.372470294e-17');
    T('-16179.166', '16.4610327', '-14.4318886');
    T('3.24', '55986', '3.24');
    T('-61938', '47825328585', '-61938');
    T('4.36', '-1', '0.36');
    T('-9857364.3', '-1', '-0.3');
    T('9943.75', '3.5', '0.25');
    T('151.4907', '-1', '0.4907');
    T('0.000000000113985670', '-2286508', '1.1398567e-10');
    T('-1468', '-12915320.7', '-1468');
    T('46.543901', '0.00000014040913', '3.32077e-8');
    T('3', '190.34681', '3');
    T('-468.5022', '-0.00000000309208106131', '-1.13251725725e-9');
    T('-0.00000000000153707', '-112649.31435', '-1.53707e-12');
    T('-1.5', '0.000000067829063910', '-6.729086517e-8');
    T('42031.2', '-1.165644', '0.408648');
    T('-137623012.4', '-71.9609', '-21.9379');
    T('-278354', '8', '-2');
    T('5370.3', '71543859', '5370.3');
    T('-0.2469826911', '-0.0000000000008289323096', '-3.584481336e-13');
    T('338.68861185', '0.000000000000371927032', '2.00995384e-13');
    T('-0.00000000144', '115767.5757', '-1.44e-9');
    T('-3183918.6386', '-980.559', '-43.5656');
    T('-0.000000000004661097', '-0.051176618', '-4.661097e-12');
    T('-1', '948.171', '-1');
    T('1057394189', '62.751', '54.866');
    T('167.9', '0.00000000000009753', '8.45e-15');
    T('-12.172', '800.4476', '-12.172');
    T('-1.4', '-139187231194', '-1.4');
    T('359356.2', '-0.006806', '0.005734');
    T('4744567.6711', '-1.74', '0.0511');
    T('-0.0000000000234390', '-75.75283631', '-2.3439e-11');
    T('-6557078727.02', '-0.000000000000000015', '-5e-18');
    T('-1.33', '-5.111037', '-1.33');
    T('-431.277', '-1', '-0.277');
    T('0.0000000000000000000697336', '-8212.9015367', '6.97336e-20');
    T('3370.2674507', '445.572089', '251.2628277');
    T('-2.33567', '-0.0000000104784', '-8.888e-9');
    T('0.00000983659', '83423921', '0.00000983659');
    T('-51.2', '-0.0027394686', '-0.0020713346');
    T('0.000000000819617035421', '0.000000011', '8.19617035421e-10');
    T('0.00079279091535', '7', '0.00079279091535');
    T('-0.000000000095324834', '28091.00', '-9.5324834e-11');
    T('-0.00000000000000015', '23.52', '-1.5e-16');
    T('0.1438', '1035688.4', '0.1438');
    T('16693', '-1', '0');
    T('-781.09402123', '6.9', '-1.39402123');
    T('1', '20.70', '1');
    T('-3', '0.000006782445', '-0.00000249249');
    T('0.00000000000000000022', '-54710642.596', '2.2e-19');
    T('-590.69625205', '26.40', '-9.89625205');
    T('10', '-5.67', '4.33');
    T('-7', '-0.00000000000643634802', '-3.80362768e-12');
    T('-9680099.05', '-62.737684457', '-50.764391642');
    T('-1.5', '9914.9781', '-1.5');
    T('-1', '0.0000000507688288763', '-3.15199093625e-8');
    T('-0.000000000032713211391', '-0.000000046529', '-3.2713211391e-11');
    T('5.40', '1612427.39748', '5.4');
    T('0', '-0.0000000000000000000146', '0');
    T('-0.000000000000000000817527', '-52.0', '-8.17527e-19');
    T('1053329.8', '-120573.937', '88738.304');
    T('0.0000000000713', '-100743.840', '7.13e-11');
    T('8219', '-7505759130.86', '8219');
    T('16655.7', '243', '131.7');
    T('259.650', '278.73', '259.65');
    T('-284.091', '-1', '-0.091');
    T('0.1250', '-23713.35', '0.125');
    T('0.00000000490018764513', '-4.41', '4.90018764513e-9');
    T('-584.70403', '11076992', '-584.70403');
    T('-221.643', '-30.5', '-8.143');
    T('-9.69', '1177241', '-9.69');
    T('-4.97845868064', '-1.3178692059', '-1.02485106294');
    T('-1.04328912629', '-12.828540', '-1.04328912629');
    T('26', '-4.91596', '1.4202');
    T('94.686', '7.5851', '3.6648');
    T('0', '-0.000000000000000363', '0');
    T('-1.431', '65.06874', '-1.431');
    T('-878.898828', '-5.21', '-3.618828');
    T('-0.00000000000000014146660', '1.05112178985', '-1.414666e-16');
    T('0', '360.6112165', '0');
    T('4.464476', '0.00016302503', '0.00003555345');
    T('1145096', '-217.15', '64.05');
    T('29.46', '104.40', '29.46');
    T('841.82', '-2171922.59229', '841.82');
    T('46', '1', '0');
    T('22829.8889', '-1578709411', '22829.8889');
    T('-0.00374341104', '1.2', '-0.00374341104');
    T('410', '7', '4');
    T('0.076125848849', '1.103', '0.076125848849');
    T('0.000001844875808', '0.000000000000020813', '2.0037e-14');
    T('6.685', '0.003905472818', '0.002736008402');
    T('41011740.6', '908.670191', '728.869597');
    T('-152.527', '-1', '-0.527');
    T('12815.035', '-1497.81264', '832.53388');
    T('-0.000000000000004048729', '0.00000000126', '-4.048729e-15');
    T('-19949.700625', '-314.64', '-127.380625');
    T('-2.4', '569419658.2', '-2.4');
    T('0.00000000000000597954708', '4', '5.97954708e-15');
    T('-17.0', '1.58', '-1.2');
    T('22811600498', '2', '0');
    T('3.1328', '-119317.4', '3.1328');
    T('2.26039694', '1567.2', '2.26039694');
    T('-0.0004559', '-1.59', '-0.0004559');
    T('-0.000000000000165', '3', '-1.65e-13');
    T('-215.9051', '14846316.4792', '-215.9051');
    T('-1', '6004363.2', '-1');
    T('173513829480', '-117.2937046', '37.6296904');
    T('2269413223', '44.831', '39.204');
    T('-1411953.506', '0.000003338150524', '-0.000001742597716');
    T('-26.596', '-7', '-5.596');
    T('-0.00000000000010245141710', '6652789979', '-1.024514171e-13');
    T('2.042', '24673', '2.042');
    T('-2.71', '68.05392544', '-2.71');
    T('-3', '220821.80', '-3');
    T('116.073', '-0.0000000000024392780', '2.029598e-12');
    T('-0.00000006283674', '0.0000000000000079', '-5.7e-15');
    T('-3', '-7', '-3');
    T('-0.0000000000000005058', '-1.5', '-5.058e-16');
    T('-683', '1013123.5', '-683');
    T('-6', '16985130.432', '-6');
    T('-76481.64', '-0.0002575', '-0.0001725');
    T('1610.8849133', '-2.02567', '0.4772633');
    T('-19.3482', '-1.5', '-1.3482');
    T('779.8394', '390457.98231', '779.8394');
    T('0.00000012', '-8855.3', '1.2e-7');
    T('27226552.2', '2168.811', '1467.717');
    T('6.353167529', '1283573.56', '6.353167529');
    T('2', '26.2339535', '2');
    T('-34', '5319.486', '-34');
    T('0', '3123027047', '0');
    T('1', '27465.6301', '1');
    T('-1.5', '-41136.47', '-1.5');
    T('0.00000000000000000001311857', '-23', '1.311857e-20');
    T('14831.7', '4.41', '0.87');
    T('2953.07357', '34.48', '22.27357');
    T('-1.5', '-2465544', '-1.5');
    T('-376', '-244.16', '-131.84');
    T('-1799906518', '219', '-49');
    T('16431536', '987.209', '429.404');
    T('-0.0000000000000000406462459', '36617.26003', '-4.06462459e-17');
    T('-0.00000004133646', '26594407.6', '-4.133646e-8');
    T('6.3642', '-0.00000000000000000009857799', '8.84487e-20');
    T('1.780751', '11652994', '1.780751');
    T('95.6', '19178496109.2', '95.6');
    T('2.57393833', '852658757', '2.57393833');
    T('147.77838', '27.129', '12.13338');
    T('-773.3025', '-6', '-5.3025');
    T('3', '-31458.09', '3');
    T('-2.93096796427', '2.3', '-0.63096796427');
    T('2', '6190.943', '2');
    T('-2', '-2.47', '-2');
    T('-60.45556497', '0.00000067724055444', '-5.489328444e-7');
    T('0', '52221', '0');
    T('0.003001960', '-277761107', '0.00300196');
    T('5.3848225844', '434804', '5.3848225844');
    T('650.7467123', '-4', '2.7467123');
    T('1.4', '-65970188', '1.4');
    T('-3500.45', '500254.11730', '-3500.45');
    T('-0.000000000049', '4919966334', '-4.9e-11');
    T('17.92802', '-4.826', '3.45002');
    T('248.79', '936.1', '248.79');
    T('0.000000000000000000062725', '-1.050053808', '6.2725e-20');
    T('-21.4103', '1.9075', '-0.4278');
    T('-7785809980', '1574', '-92');
    T('688.6', '86674028', '688.6');
    T('0.000000000002917443', '-245.8', '2.917443e-12');
    T('-0.07547', '-15.028', '-0.07547');
    T('0.0031519195605', '-2272651712.25', '0.0031519195605');
    T('150494.446', '-51709.2935', '47075.859');
    T('-12010', '0.000000000000000016290', '-3.28e-18');
    T('-32.408275623', '-1', '-0.408275623');
    T('0.000000000000383814', '-1', '3.83814e-13');
    T('0.000000000000000003469380524', '0.0000000000036', '3.469380524e-18');
    T('1', '15.8856255428', '1');
    T('119.378058192', '-2.4396301', '2.275813392');
    T('33773052.62', '0.00000000111954987477', '7.3387294787e-10');
    T('6', '-120890538.30', '6');
    T('-0.00000000000059065444364', '22714637', '-5.9065444364e-13');
    T('-1774625862.4', '8507651416', '-1774625862.4');
    T('30.1', '5.7', '1.6');
    T('-1.806', '38.81', '-1.806');
    T('3', '-1683506', '3');
    T('-88.75', '-1411.1', '-88.75');
    T('-31', '0.00007026872', '-0.00004067864');
    T('2437.18437', '2.70220899140', '2.4940687486');
    T('0', '-41630623', '0');
    T('-258.66793', '-182.14421', '-76.52372');
    T('-14228.6', '95437.0', '-14228.6');
    T('1733.518', '-128426.9', '1733.518');
    T('0.0000000000000000000532527998218', '-63.28534135', '5.32527998218e-20');
    T('-295.03967', '-3', '-1.03967');
    T('2.956649054', '899957.06', '2.956649054');
    T('-2313.023', '-63930.69', '-2313.023');
    T('13803.9', '5.8329456', '3.1507104');
    T('21.8238290', '-3', '0.823829');
    T('-22.3', '-77.5771049044', '-22.3');
    T('0', '10.70', '0');
    T('0.000000101', '-14430384604.6', '1.01e-7');
    T('-600271019.62', '-2.4', '-2.02');
    T('-53874920.5', '21.676509', '-16.648855');
    T('-0.000000000000000061', '-69.27505', '-6.1e-17');
    T('-0.00000000000117570154770', '874813.194864', '-1.1757015477e-12');
    T('3724726', '0.0000000000009126675', '4.24705e-13');
    T('6493', '-82605.38', '6493');
    T('-2.88', '10', '-2.88');
    T('53970', '-99064', '53970');
    T('-12.440', '-0.000000000019', '-5e-12');
    T('54.64', '-6.1', '5.84');
    T('0.004000', '14', '0.004');
    T('-1570593.12158', '-2.56612', '-1.9417');
    T('-0.0000000000000113891', '-0.000000000042', '-1.13891e-14');
    T('-29.9444', '-2245', '-29.9444');
    T('0.568740', '3.7564033753', '0.56874');
    T('-52606.48321', '-4.34014413083', '-3.9363443404');
    T('-0.0000000334', '0.000000000902008057677', '-2.5701865951e-11');
    T('-634849.178', '3953007.487', '-634849.178');
    T('127992.04562', '3009797.1', '127992.04562');
    T('1', '-43.9035960', '1');
    T('-11.0349', '-2', '-1.0349');
    T('158.7', '257566', '158.7');
    T('43047346', '-115148338070', '43047346');
    T('9691.0996', '-15.5219', '5.434');
    T('-77.9', '-2.0', '-1.9');
    T('-38.3896', '-0.000000882', '-5.14e-7');

    T('2318237922853539360.167157597022570', '-9702015238687.680494235371626139504030926823802574', '9295675788919.833075194557918488338251940141560718');
    T('-3346995255.54414928936', '-19484973963827626752773520333240590885639375', '-3346995255.54414928936');
    T('-270721769', '22539228.03492196260802407744489638356810047', '-251032.58093644870371107066124339718279436');
    T('-0.00000000000000000308066481692295683548642301228852180070143433425056969292333', '206232915.762239501154732324', '-3.08066481692295683548642301228852180070143433425056969292333e-18');
    T('-0.00000000156454306292867840840689', '2635', '-1.56454306292867840840689e-9');
    T('12886035692416820.3977457533594029154555584175743', '-113496127978790341.941210015456704515', '12886035692416820.3977457533594029154555584175743');
    T('-299395547.883083669188105176893563510454667', '-3152.9580', '-115.077083669188105176893563510454667');
    T('8076327350.789696755252236317', '9', '5.789696755252236317');
    T('2241730616485747642201052100686334', '15115372376051.1433309224128', '15028644606255.6407694748928');
    T('0.000000020545442143', '910205431326260.56522101419792183224139104902865869227', '2.0545442143e-8');
    T('-299.2', '-1095069605114867345071265098807585478368438601713540771676', '-299.2');
    T('0', '-0.00000000000000000006258032721', '0');
    T('-9561792051.46130', '-0.00000037912322661422', '-3.0664875638608e-7');
    T('-4589.274150042', '5463826693798.059180748770', '-4589.274150042');
    T('-177372488891124.8209441829612234253673371', '-2184484411972.871928873110', '-429251521322.1947054610512234253673371');
    T('2840794355990579101690516447419567416103576933551526', '-225384346136689418.311568171615', '4159399848092788.810925291935');
    T('433169940324134426366793327.235', '-1918606541369710.9167801826', '369802826989779.100695574');
    T('0.00000231962990639112417450960779176176525188784519762202020426278', '11491626.8146875740651454657723913302260', '0.00000231962990639112417450960779176176525188784519762202020426278');
    T('-118317530966985340050567.3794340230', '-1290337888625470391870404816877827630953112876.297', '-1.18317530966985340050567379434023e+23');
    T('507941745866485347809651792.3258074064530387681549475622', '-78.8876', '48.5294074064530387681549475622');
    T('3280516686867.41769205935464611912132313550145186748', '-0.00000000000000125477', '5.0338132313550145186748e-16');
    T('39.99', '-196.666', '39.99');
    T('198328732837738155510399443.82996', '0.049415792708', '0.034407057628');
    T('23.9909397975464820637937485614', '0.0000000001050229258855441', '8.54924991032156614e-11');
    T('-1157214583831665284482', '1.0988344', '-0.0759048');
    T('19626.043251177620', '0.0000000034291719721621', '1.7452276531105e-9');
    T('11144329315299884830592174718286.015391668774749336449044', '-276730.619204816478', '84261.983174911066749336449044');
    T('71.3348063927327012696442274193666742024', '1.003', '0.1218063927327012696442274193666742024');
    T('2353.964728726069', '-1147885339335726766004631.4827280116227481840172642411087841', '2353.964728726069');
    T('-3.4', '99841150890712628939215423679763543630485717', '-3.4');
    T('-17423609848720294448155093008156493860946579332348817', '-407401303.1', '-39793145.5');
    T('268219889', '66408.228187594864', '63463.578491939168');
    T('-7067928231578399800943353710459167713.4021506', '-44099.172874028', '-34277.265446476');
    T('0.0000000000003143', '0.763206816948175243403280699590867', '3.143e-13');
    T('4.036744136250', '0.00971177', '0.00635958625');
    T('1793819.269108', '493646013195.8520', '1793819.269108');
    T('-15826468630.86978', '-0.000000000000000000372629410691473337097845426967193919000496413569810554360', '-3.3602620906243362074116532307435310028959396493592065148e-19');
    T('-18.7707479011363', '-0.00000000000002581603178761489486464393960475608950385662413624987', '-9.1067040799565470223405647782611453765936052898943e-15');
    T('238432236541400642177165.39210', '-5', '0.3921');
    T('-6811268416946324947973610791528879575619937612025', '0.0000000000000001069504654335424482337205845600046597647661', '-8.16882733210848477277494710610839496492589e-17');
    T('-0.0000000000000001763427790981229873712832162', '-0.00000000000000000002454861590426229238192323806393241482688272809526858064170', '-1.00710578069411919285971867734642985013640916857852506689e-20');
    T('19755770505453660565232908.50109071', '11636774869145974997.1', '6173329669666681241.40109071');
    T('-0.0000000000002209009508639970169662640887364083811', '0.000000000000000089645450775854240465099382429782771424049138745143690592', '-1.4560152292168460259210429423632311142922131965946381312e-17');
    T('563088', '-0.00000008456562014728262541201891984898', '2.11408406705846430113462806523e-8');
    T('0.000000000000014349029064561994176595511', '-185.73981699167343715247600', '1.4349029064561994176595511e-14');
    T('-0.00000000001083082172818863769543', '1190273509470734307.14840', '-1.083082172818863769543e-11');
    T('-0.0000007000816503264538441903792097551608671', '426939', '-7.000816503264538441903792097551608671e-7');
    T('-74639664.5738469080670677219615636886365697', '11681605088890876252615.81485333', '-74639664.5738469080670677219615636886365697');
    T('-17473762843659544328007830640965916', '4735599287262364762422146713625.0556784154158620151196', '-4.1370729486807194325314144030856023255308850262237956e+30');
    T('-35798372944152782907433662314743475258228956226653087.007402', '2986648933436031786673397525017144049.527376', '-2.586979851276128040119426347481949619034666e+36');
    T('594962882338997095.95600951831319665393181826', '-0.000040093460748026675395936734376802508692547860', '0.00000106629014969426833642467613848819248141182');
    T('-730572579439411951784822207340668557', '0.0000000000000028', '-4e-16');
    T('0.000000000000000001168651449131923286954932276275033439404771143330881475', '-47210255.27658777658181021168421281357', '1.168651449131923286954932276275033439404771143330881475e-18');
    T('-30769567698.202004635902421797597470637', '-2117030646075139493515312478944809325583483881105', '-30769567698.202004635902421797597470637');
    T('0.22203824107047587018951755693974277131200048220', '-136.52076', '0.2220382410704758701895175569397427713120004822');
    T('4489045408103326401151265241469016732232592556114681884438', '0.000000000000001314763381159004315842197', '6.8074495798791174068935e-16');
    T('-21348129287570112842795163717617654521736400718052510', '-1995504519697835638540568077.0196418', '-1.1402642626986948193867205135517104e+27');
    T('-14.48287435139371979106', '-278112747003022610742126629257947172194929.64536', '-14.48287435139371979106');
    T('0.0000000000000110719145581948062251355493585472867', '-7978399748.1558009430388367158631487853516499292946094', '1.10719145581948062251355493585472867e-14');
    T('119501784974', '-0.0031818401885065375156179', '0.0016290398380208417060804');
    T('-0.133707475845003705950317295850288779512882217932', '329652021145697865591302212038174941275757995054.713783674845', '-0.133707475845003705950317295850288779512882217932');
    T('-12542871568091185279.0460234294798148233', '25754.120417', '-21877.7952474294798148233');
    T('-3818054566.20841286030331762281612', '-1936.24778639274331686587963417792515204476546099', '-472.43293099041187629213445041741051174848665583');
    T('118000308055.9373727402663957624604063', '9918.67574327356', '4846.8753496034663957624604063');
    T('-11.38', '-24193758424792525116909481098205.677446', '-11.38');
    T('2.46889727437003280961289299604392122014099570', '-4644753841435365046988433379518839103541882981334804890807.8', '2.4688972743700328096128929960439212201409957');
    T('1805667164641860305883.31324', '0.00000000016618955069904640141657751519707415214669847044', '1.046133052518583849356487736801523519251586328e-11');
    T('-0.000000000000000000012169689776926610597997556555980427323944885949030949402', '-434910402113045297454505.1302573189763520', '-1.2169689776926610597997556555980427323944885949030949402e-20');
    T('-1739.4', '0.155584511367509963502532516739270411', '-0.120747422606118005188995371696075431');
    T('-569589695561180399227800563933576000706960457.3656765081522', '3483.46493456384339944984809629410413910476502', '-3467.25917700145185407850776398960809939327178');
    T('27827944878804.043096443231553973596298120', '327784539432851688676.574297', '27827944878804.04309644323155397359629812');
    T('-0.0000000000000000000273570270893076789122004877379023084477997941182649713787', '55071786762378989.632689594', '-2.73570270893076789122004877379023084477997941182649713787e-20');
    T('-1', '30698458.1545228514266', '-1');
    T('-77.6796737181065072239544734403386720988476876081692968190', '-48043379.33911367817680827247020826', '-77.679673718106507223954473440338672098847687608169296819');
    T('-0.00000000002231552048713565363805177481170756001463248', '-7745938658044151', '-2.231552048713565363805177481170756001463248e-11');
    T('4080930738001215900471542815678.870', '-259094.384407758455239', '219215.641754292988748');
    T('-65148376658415.8821831041618603950', '176.7', '-41.982183104161860395');
    T('-20649466814.805802613083057980846632715554163812995', '-0.0000000001319860149268122557211741866249364289650956', '-9.623959984992160004489809813728963916899e-11');
    T('-8637039792424460231', '1171511157428179825785002.9414399798460302750', '-8637039792424460231');
    T('170120714324740496497515610676549361259437493939036', '0.0000001984234593401508456286993357944778724317751259644793106', '1.80272091831781979821089649811533712715612789460516662e-8');
    T('3040385342.35657760274052', '-372946621956569451.93539995558371617994622295531744', '3040385342.35657760274052');
    T('234417346052636032.106833697570476435658983423056', '405728269025466785030711845788149565209977.743', '234417346052636032.106833697570476435658983423056');
    T('3003698058950457901967364.923201608', '-0.00000000000033926708389437080558726155721138587500872389823', '2.3077148260391721408496333950768060426097166773e-13');
    T('-1', '13121297327432473503324600878404', '-1');
    T('-0.00000000000086987493647337590015438704985', '1126309963348141179998313355117347492625867522.06445459173', '-8.6987493647337590015438704985e-13');
    T('0.01175883004658442799157', '-1304796615695885174911897002429657121243.096878772927435101', '0.01175883004658442799157');
    T('-4.1', '-925940.0049', '-4.1');
    T('94413979984015715031.72143', '-44956570520496128861135438942192313346223.4679844', '94413979984015715031.72143');
    T('-2084.96276512750756221737052806049261165250425', '1.5202660037865707', '-0.67807393611913251737052806049261165250425');
    T('5530042131715182582', '0.00000076755550127601384796182326285660346554068838', '4.3620483782819522891660511451578934939479584e-7');
    T('-67042.802', '-28411888994.212820705785482909844033014', '-67042.802');
    T('-5360742.0', '327368856548045.08220112714929243516', '-5360742');
    T('47117230722.16973123915245237545229011004357913004', '-47488231.617918014245187991287717374685823103416490829', '8904957.195061107925965018036654421707060540881097632');
    T('-0.000000000000151343572349948255798487186356', '42.39640064100021994629331666', '-1.51343572349948255798487186356e-13');
    T('113840373593648670861500086', '16082411799435623101958677220820405667', '1.13840373593648670861500086e+26');
    T('-8.22', '-2867798109411674.996765068247085164513407273890902397064937', '-8.22');
    T('-3.50238614', '-6860188389136636488339219859054193101069684390', '-3.50238614');
    T('107107360259917227679425620190.388983860971684', '8800479523270977282088607107997626691064406556317263323.341', '1.07107360259917227679425620190388983860971684e+29');
    T('-175822.1223383477', '-88621983013742415713.67926832', '-175822.1223383477');
    T('-0.0000000001165498478337125015371156469966957212161218123244751', '-62550285299530518719359714148367546793907', '-1.165498478337125015371156469966957212161218123244751e-10');
    T('0.0000000000000000320110187403311537074296810233047925148', '-60407506473103.897451724471', '3.20110187403311537074296810233047925148e-17');
    T('-0.000000000000000338282064563114', '-96190376454071434357.783403611375151575341982065544942431', '-3.38282064563114e-16');
    T('-0.000000000000446753711341', '-9.4', '-4.46753711341e-13');
    T('369078971611659797359596697169091.7394557', '4390355307871218680185944', '3.2920328769890124360129157394557e+24');
    T('0.00167752355829931578551694576878658836500507142636702529088528', '3276066309050', '0.00167752355829931578551694576878658836500507142636702529088528');
    T('-40341913269616270298908054352.555424105652', '-76.6805753', '-22.343535105652');
    T('0.000000000000000234899796181467417005659941', '-0.00000000000000002455754630676905247611', '1.3881879420545944720669941e-17');
    T('-2288200091016313974155687089998490026813781889257797', '173612435224249143072698340434416969557762958', '-4.06305517399251906560023908396396030698941e+41');
    T('-119.570203264189', '0.00000000000000000270873280446606560343105459296495864', '-3.0654086667132932857371380394960016e-19');
    T('9636003428434421863325355207384758', '27189148267469.7985900247652', '763104629651.47867591268');
    T('336.422676596', '45198935114500.3922307046', '336.422676596');
    T('-14.0627943158485824648781649958871436175344604029182482603314', '514937038588634.2538384203447', '-14.0627943158485824648781649958871436175344604029182482603314');
    T('0.00000023418399644897328432613203542773200097298268', '150.05994', '2.3418399644897328432613203542773200097298268e-7');
    T('-7549019185595.78985813486261693520665842030485693423', '0.00000107047', '-3.7760261693520665842030485693423e-7');
    T('2084', '-4702062390.892', '2084');
    T('-0.000000000000004338', '23.449915', '-4.338e-15');
    T('355378988141704324089218417352212802.4373051939435896809', '-410089.6820', '397574.1193051939435896809');
    T('-0.00000001989462540690061730462809985539649615779', '-38.286442069994941913718432', '-1.989462540690061730462809985539649615779e-8');
    T('18131048627056749262739468342.355859006591', '-1199.6142723004281428', '211.04880824112304');
    T('-7485271015053088795456302791998022618551521.6832', '0.0000000000000000000469320709938399196323702081094328803321', '-2.69710660151015687863998038860418026399e-20');
    T('0.000000000951589985684289940380019704065655070039753', '-26373114201649323963859575.5038618827163', '9.51589985684289940380019704065655070039753e-10');
    T('29612852829840055823.4234313598', '-0.000000000000000000390000839472495342007727670085919905954881', '1.00427280201824924510562417012541840544546e-19');
    T('-5607158.61839129739841', '7', '-4.61839129739841');
    T('-5033.1047890916762223467249791215764', '-125308255061340894094178.363334', '-5033.1047890916762223467249791215764');
    T('721650415570508.30568044521', '40444523802228.33072733', '34093510932626.68331583521');
    T('2909689458580570441', '35832160731.298726', '11893825723.095282');
    T('-5860896283.24109', '-0.000000000000000001291619269537172108644', '-3.63300889125010974528e-19');
    T('-194225255714783295041136360134265455943072328542285', '69.6', '-40.2');
    T('-64931541870946970276.043833373186791755', '0.0023858644', '-0.001926882786791755');
    T('1768097901758505513385227.27886663656389579248832549927529', '49683790014635', '22391913150827.27886663656389579248832549927529');
    T('15607444724715380472693190116075827307544230542858', '-30863780897913937221259082805810494697060093553853613140', '1.5607444724715380472693190116075827307544230542858e+49');
    T('-5', '-4637217487418.54919740129555570192754032790658061945', '-5');
    T('-0.0000000000000000000828', '131673552653745805914768.46227', '-8.28e-20');
    T('-0.00000000764789734', '1709371095001247612801478608832281576263433223592134788490', '-7.64789734e-9');
    T('154437761882957023383482721504354803201034660811788829', '-0.000000000000006907027', '5.846749e-15');
    T('715263.3', '0.00000000000001885535871127301429501799', '1.588206408282257808507009e-14');
    T('2', '-139137', '2');
    T('-33.5', '252567485444977003649.689937162645', '-33.5');
    T('0.000041876254378856129586442165482266321783', '322959107295001633.58', '0.000041876254378856129586442165482266321783');
    T('-54186163890839675559037155.29991407802075734', '0.0000000000022878015271202914230826203651610666594361551538884626909', '-2.2681269475854053487147725795128202226083088877672780421e-12');
    T('0.0000000009449673438295910508', '-17.0733222', '9.449673438295910508e-10');
    T('21265617482392966322733539382375295590999877466103755238889', '-352694528885882768342.45632391826455624336', '221163195874277534438.6290731360549322576');
    T('-1307793651716492003679476694280841686029964.654735755057269084', '-89261449075265.032742854725660037428102593', '-30163701590263.071635290265954147983863299');
    T('5914702572.83224302033871291000363', '-0.000000000000000886450497717875025851850', '4.967313925042229243866e-16');
    T('263730784.16038091', '-58734788098538430712.3', '263730784.16038091');
    T('0.000000001854032486183296214519', '-0.000000000355070869879153', '7.8678136787531214519e-11');
    T('-31884740503448053282155167604', '-9334262264990899277546182645450.74129764939998707', '-3.1884740503448053282155167604e+28');
    T('11557392706097.3454852500293825329599950094143', '2.52303246491988770256', '0.4512976779915504676799950094143');
    T('-71596377460606253231921860426759937622737054', '4887438282022459980414.75188230501531', '-3.01046057949271437163960532658638822e+21');
    T('233.31782712896541142', '-0.000000000000061176056765670789881258454117135367568123364367998', '3.1635882250994428931750935802393986143468670072e-15');
    T('365.885010333042430035757670950257510558158528', '0.00012', '0.000090333042430035757670950257510558158528');
    T('535753130902782277401868.255371439243127', '-143.10142002690254095307', '118.497092196935045142');
    T('0.000000000470393111281920', '-51.9318603511297101', '4.7039311128192e-10');
    T('13458.12201247046122595270157739959756782786002', '6387839814.018150125742976314622109379608051443', '13458.12201247046122595270157739959756782786002');
    T('-5.831075554650765329716', '-11970348016752784794040399571598021585032773536121.380962', '-5.831075554650765329716');
    T('0.00000000000262395619618229355713043239', '-87185298877269401376712558913.7470', '2.62395619618229355713043239e-12');
    T('141.0', '188771.3963534769931324152', '141');
    T('-9236972803514324.7806408277606845', '-0.00000000000007809628857086731989071851354', '-1.650710030068608338878824096e-14');
    T('1.2974663175859402553689735548443', '-0.0000000000001062828009896091833554986467183627932744813138557760818085', '7.61109945460281796836350249437619994413734484557859166905e-14');
    T('0.0270550933935502424060575696830352678525170478229935415832', '-451291.14957148684', '0.0270550933935502424060575696830352678525170478229935415832');
    T('-92.3630313298', '-0.0000000025494749392548', '-1.4586947958892e-9');
    T('5', '2.230', '0.54');
    T('0.00000000000000047652193195365538282280574751077165811188614512152190254', '-0.2219408161751349997065647158660685689613558', '4.7652193195365538282280574751077165811188614512152190254e-16');
    T('-11584699154744825490947848080895226756397973538', '87574.623306', '-81181.758812');
    T('677064505276662938729662254128.29994114', '707.4', '272.89994114');
    T('-0.00006824223430307152214885541', '-259323624933420687586166700605273793769205140227.53768209', '-0.00006824223430307152214885541');
    T('42649.4427', '6691515410463', '42649.4427');
    T('-1168876626035009643806416736226233', '615863945.824703012224567830126', '-392657283.272481237688137233096');
    T('-99850601104584713060403542.32679025848626541', '4984333871554095648842233050848980249665273650', '-9.985060110458471306040354232679025848626541e+25');
    T('1', '0.000000000020714284873094147095452920238214225', '8.535434304189538408987784126387475e-12');
    T('-40382172868238902141831024410171370609.8497129774930', '1302788268944196991798424860.3558746', '-6.82458913661894485779787415457058777493e+26');
    T('-0.0000002901522230060366316110382713638332433750838395', '1797816163.6022664896252467454244679282', '-2.901522230060366316110382713638332433750838395e-7');
    T('15.07', '-202256.9', '15.07');
    T('0.00000000001513399901378093', '-0.0000011431', '1.513399901378093e-11');
    T('8067543766555814361258.9809653', '-1063782.5113', '771356.4731653');
    T('-0.000007452478259217127353', '-1269742061.7650', '-0.000007452478259217127353');
    T('3083816844578.0036462734892162357142', '-62.043', '12.0776462734892162357142');
    T('0', '19676669.3210', '0');
    T('12000.579726', '-0.00000000000000000011473979542082043429', '6.885862514870385948e-20');
    T('157760.0162346769710343762123266316522595', '0.00000000000000002072', '6.6923266316522595e-18');
    T('12.56649651', '0.0000000011486174569183055328395211586383654058895981238621', '6.243894925921526655230893363862847927286267313448e-10');
    T('1619418216785919248186484303127025755692.6', '-5.32529795', '2.4485417');
    T('-7894381522375439472424053746795845545020655311347324165373', '22494560920042179494908995572822416399654.3988', '-1.79805849224239031722296510506179645210727012e+40');
    T('0.0000000000144180437967392496819227076903747539933453407576', '21.003335', '1.44180437967392496819227076903747539933453407576e-11');
    T('508588836239.155636417726778522264', '-402450669.2', '293641039.555636417726778522264');
    T('15935406874866887008916010678224808183941097', '-20533021922984769932524329949941646599184', '1.781862630705541277130637070090422974313e+39');
    T('10577358966.62645197', '-3902498.5606677997620066743', '1587867.216714614961912647');
    T('-2.3', '-247.215776', '-2.3');
    T('-0.0000000000000000004534097652705', '21900.378494953943', '-4.534097652705e-19');
    T('-622302543239963404259053402472736239978107701', '-37144508134104022867053839784711386961366245133.798061', '-6.22302543239963404259053402472736239978107701e+44');
    T('963', '-1143859.8306131050309784880555908137246920', '963');
    T('-1105334398087211096372825533031660097846.368', '676850877469691.4610087010635866598320915855', '-528741020326514.019428617163961817534907474');
    T('-33.537', '0.00000000000000000240610602831844082653894660603468520326205996616490648230', '-5.055045646408436609503461106111907664625346674246231926e-19');
    T('989.69', '1818860793340242.34985869797714179945790', '989.69');
    T('10897111350996763207208776906688937.9374615427', '34065063399932552093174621472004861573778946476274074', '1.08971113509967632072087769066889379374615427e+34');
    T('-11632431937037532580778695673.04032', '74582043438.8721965449504177221562482147014523596481', '-53296552278.8841298987146598409845587169502001115997');
    T('-141.4055769529530650', '136398726400805051012116675708221546904997943.0983953382506760', '-141.405576952953065');
    T('2436945370.995', '1123321514602145559580', '2436945370.995');
    T('68966344530824.7651173919961179400965271017811497107820', '3016650.5058630595142431070', '430737.119987074593653282096527101781149710782');
    T('0.000000000149640169704895568509292298172960103', '2388965841.03588731292125318485549177319506347036', '1.49640169704895568509292298172960103e-10');
    T('0.000015', '-3304.73941676117143286536062311830598946', '0.000015');
    T('0.00000000047066315132394427031416', '-6192', '4.7066315132394427031416e-10');
    T('-0.0000000000000000000581928591150593239271964107701426222875128', '5909007611163710886717422164.5826709875973058037321', '-5.81928591150593239271964107701426222875128e-20');
    T('31384923094914821999292552425.2009815380728309425967', '77537672972.35166', '11679771345.2858615380728309425967');
    T('-37567843820039777389745510805385', '-71883985.6201303', '-21998426.2683717');
    T('-8457.2', '-0.0000001173257195508166367392288464056131809270998457', '-2.69885424250247290675786668651098778106586768e-8');
    T('25100609273081546346038727027615', '0.0000000000000886831920439450', '5.162428876817e-14');
    T('-18047576.1098270311193001138015699506418833418804938917', '192214125807611.95755841586183852694517418512423624080241', '-18047576.1098270311193001138015699506418833418804938917');
    T('-95.588906373514114132242', '-0.0000000003929994337493746273806772122562877169246192899290', '-3.19339159019421511190151559546325880388122429492e-10');
    T('-0.0000000000000000002355739833279382506655556792223696880503752520', '-0.00000000000000079651826138154154521027236636093398105375492817', '-2.35573983327938250665555679222369688050375252e-19');
    T('-2007', '-6160033729282.9851660562442', '-2007');
    T('0.00000000000094972368099037971103433', '4734161412127674820247110768919593576233188914.242822976357', '9.4972368099037971103433e-13');
    T('0.138175287929598290229868824644141412392715', '-31558584857557504718252045560.11304', '0.138175287929598290229868824644141412392715');
    T('1196620705.9415675785887940558444736246545644683738095', '-70465530484696222361776144167727350283884319.64044686', '1196620705.9415675785887940558444736246545644683738095');
    T('11074860894', '1561765758632.45', '11074860894');
    T('-4381924184574059696448709.53108872210260149095882186', '1.028', '-0.23908872210260149095882186');
    T('0.00000001155422786572893769342087552370206756710219340', '-13931734385.2', '1.15542278657289376934208755237020675671021934e-8');
    T('69191800707.52', '8416.42298638305', '181.0075723297');
    T('2081967242.78645435603091', '-0.0000000641307577303646051321340042511613883', '1.04729585981376168223728825373660659e-8');
    T('851318799560282460315775006926769346175917826.47889714', '3053712.3623410179455158701230239730564268230', '440718.735027453982530562337797644313071866');
    T('-10058222.5796344546386', '6627451174822895.34794097800071486253721590178533538', '-10058222.5796344546386');
    T('0.000000000037566442917643747268847042234718910033651246885819716699339', '1.3779', '3.7566442917643747268847042234718910033651246885819716699339e-11');
    T('875.90002405088', '0.000000970077655599692785652735177637', '3.5638028118889148093740648709e-7');
    T('-62.7931846870391496', '-636485693987795148000356.1626652406993787', '-62.7931846870391496');
    T('0.00000000000000100341179', '-544501716519662.633193865233920', '1.00341179e-15');
    T('0.00000000000000351499628540049570637673', '-223431373521.7926681956463703562745846396661', '3.51499628540049570637673e-15');
    T('-220603353886874667147934643649.87295033625852695', '14892.592908', '-14434.15489033625852695');

    /*
    T('-0.0000000000000001612847241392420250768267369', '17433478642639304.4236073577621811634134', '-1.612847241392420250768267369e-16');
    T('-1883174595828380403709352050091085026369077660279268.9', '-0.0012895', '-0.001097');
    T('-125549014214.7375819121786172926612698843039792917253281', '-101.5885766453', '-100.8153792133786172926612698843039792917253281');
    T('1065766474589037077293252213448', '104.84555', '81.7216');
    T('-0.0047533898584759553535897361825760775807323217889394835', '-18213101567414468503998329412491692494599546949.8', '-0.0047533898584759553535897361825760775807323217889394835');
    T('1270005291407.8881300436', '4138621074845956468.09769209', '1270005291407.8881300436');
    T('-260441856275307932803222450437995247080731478', '704992903069127.4325788', '-431855187882906.333326');
    T('-0.00000000000000000009754763692467059466885260432113614665268499743', '-1095452837.106034703346392429672156', '-9.754763692467059466885260432113614665268499743e-20');
    T('15.2', '-9898.141488362', '15.2');
    T('-4.642097816103323517795932640', '2320173035769155029864.2360071417144711754883993503027', '-4.64209781610332351779593264');
    T('27232043073441728507866158240.138995261078', '-215290045818292170941580969748443497', '2.7232043073441728507866158240138995261078e+28');
    T('-7820551181460133788154158506805766936661586187217825438.43', '9043807302.81268', '-8827979159.46068');
    T('40454701041551859098452066940764312196468.06490346453770', '1.43336157528769251880838', '0.053020665769834197966');
    T('0', '6893071800513.175', '0');
    T('-23294105221282442798534198559904763230604376248570', '1108006854160532581.643019501', '-634324850398077026.428796591');
    T('-70.48', '425148951767015.661297750583259821000170', '-70.48');
    T('-40097263.65021696329198038628255519578725029828361985869', '8008783649.929271', '-40097263.65021696329198038628255519578725029828361985869');
    T('-3744957147569480041094559658.8985734744', '-1.39589737574796529539305515486016767152', '-0.007414375129527604087594017204876948');
    T('1', '12243765348634238072286305372976115524601373993259986307', '1');
    T('1133226250873003731000285272922.678821486883559776563400259', '-48449121713.05851357863749277194726', '46399517902.028653911554106994622540259');
    T('-1110.00985672050392625354008504049692', '0.000000000000000144467905969585159040405525015114077352752345766119586', '-5.0667226393916955793430389451065661055080904169044368e-17');
    T('281516884461013501680.63', '-1.4', '1.23');
    T('1424834076734161906.49994168746374', '-0.0008975084074165482738612610040251114952525910764632040285', '0.0003696875090618801911161885049781983725485998646848861645');
    T('-128763476617069.02187580755667', '-6603.36282886993434', '-2812.31711094080867');
    T('572750920744637008914545508209975834156579.5795909538492834', '-11502534832438064096814', '5.5309188778180859429155795909538492834e+21');
    T('-0.00000000000371736421699926447258798163058670009', '257486.470069146779', '-3.71736421699926447258798163058670009e-12');
    T('-0.00000000000000130129857', '0.0000000000021237415884955381438534891507974579736322323393189949494486', '-1.30129857e-15');
    T('97510480169357047959.778521006822362538', '-0.0046089528809', '0.003841092011662538');
    T('0.000000049938779602084071943723403', '0.00000000000000000040598762102204974283610961143933322030167695316672919233780', '2.328791623754087509772477793320317342677727105556945004032e-19');
    T('-0.000000000000017579612833728413841360648325607406735458075688296134041848', '-0.00000000000000000046795016181219', '-1.29104929872111360648325607406735458075688296134041848e-19');
    T('417587148573267450086061214415849335522635545129971', '72162262599593511511391535.25753637130715', '1.588290674385543384224627617743657423135e+25');
    T('1899704999505146.65318875024320183643806671', '8011298158372506449467236619268.06739540', '1899704999505146.65318875024320183643806671');
    T('-0.00000000011026161484148139283420032525', '3', '-1.1026161484148139283420032525e-10');
    T('-641378081607809826819509989054058308747306', '-32614433220951683931.97290822167335112076028624', '-32321533365320656543.47914416134229484916012224');
    T('73426.2174993055579013322865502961038551', '-22991170194389000791863400240', '73426.2174993055579013322865502961038551');
    T('0.0753432448671128796055779867138537182985932603289568836', '0.000696623968026162316598353569181604985489965', '0.0001078563202873494129558012422403798656770403289568836');
    T('-77918852302.666', '1', '-0.666');
    T('40834438105282568531887549132894212920316057.4119', '-69096.2816004', '9282.443256');
    T('0.0000001081007656128892877603069470142579', '-146763.9202434867', '1.081007656128892877603069470142579e-7');
    T('-19065040207291206222014126191923749015295117270838828148117', '-59458447985079036923403398069307106616288796371525.0', '-6.933918951125046770982521962871037753580749088442e+48');
    T('-50.454890710', '1.57811064514885740036542902647362298773', '-1.53346071038542058867170017931768738037');
    T('0.0000000000000000029130545090762297054692', '-0.0000000000000031732390760633505614997707573729383', '2.9130545090762297054692e-18');
    T('-0.0000000149477116306405895605823556902142791919238574341861798', '206795087.6', '-1.49477116306405895605823556902142791919238574341861798e-8');
    T('0.0000000000000000071', '5470646583815966704709512378933741816164381248458912550', '7.1e-18');
    T('-2357250201772857325710', '-177521457545596938.745399459682135971137972095', '-120288482421173048.58597434059857523000652259');
    T('0.0000000000000029091', '-22982331952112767752908263644179706433315770.17359251594', '2.9091e-15');
    T('70410732817903246800961841933624181629.60085297037', '0.0000000000000000000103454336294636075289', '3.524329949160506449e-21');
    T('-27369983157.1458440282472352878', '23.763912432052919566354032773607568646834308263895', '-6.32537443505237861917552362621110083695094947093');
    T('65056.653', '32408712.331948093248921801757883672', '65056.653');
    T('-79299703594581340711344589476907732591.49009044336', '-105750.084896815429373214442762165', '-104265.478222642869860762619825095');
    T('0.81012834507591033602543', '42890384.300', '0.81012834507591033602543');
    T('231647806.280', '2306380.2577620003122660083794072550721890979450664365063025', '1009780.50379996877339916205927449278109020549335634936975');
    T('-0.04266283385', '-231833828.7857580897334', '-0.04266283385');
    T('2.9617', '-417567079608596894283345450125373793666347.08065279467956618', '2.9617');
    T('0.00000000000005671393697246617', '0.00000000002372488709978367633405147788335971135485863563547', '5.671393697246617e-14');
    T('20411507237949901.00007048085657674491003063099663777294', '0.0000000000000179718685180331186307', '6.87219923858062022287294e-15');
    T('78562464348554551960.51656952881040712219428', '-360452902524757862924847627712001.90071009646781972063', '78562464348554551960.51656952881040712219428');
    T('-1784.7', '1.107', '-0.216');
    T('-21207.62887', '-30083662469912.93830185385', '-21207.62887');
    T('181794573664525362505571198963.74048513161596812818', '-0.000000000000000006966183124261834480', '1.44054114626355864e-18');
    T('1497127573695.63303', '-7299620151322256127312252586', '1497127573695.63303');
    T('-143331341997458746373473632414689879129001.0510139759652713223', '-1039245311665254920360224447402081.203667', '-4.163981858938863521252654102664372514539759652713223e+32');
    T('-7.42318768495154312740078', '-5725.37969786010009373491349369', '-7.42318768495154312740078');
    T('1.7', '0.000000000000000000358545942655', '3.3438498447e-19');
    T('-0.0000000000000000230187885344529285768794826532', '-97947.8515169117863685621966428355043134881268104806819344', '-2.30187885344529285768794826532e-17');
    T('0.00000000000032661966', '28146830442002.99800238434978678633563002677', '3.2661966e-13');
    T('131444649737941070', '20345737092636.1111991821569377453431132947653', '11188119511791.653283266182165083488115816162');
    T('-70.95', '0.00000007627177390524757759931', '-3.355915079851121925592e-8');
    T('-0.0000000359844794247092743644998506637111', '-821842552305272024445101236331.431697024181064278117', '-3.59844794247092743644998506637111e-8');
    T('11559244775803152829995363390516552788279232383634', '-0.000000003810335052601512043915061311335400346835301683933', '5.89171798463495478689746049513730071363650371891e-10');
    T('2527.74183', '43431759.31595200153', '2527.74183');
    T('-6716182884374088980171.84153844', '74722140.737467381748608636560886010812295783760526770', '-35823304.85096169083049875979729671181231669302395569');
    T('12972207321585575187726304322503969808542965.246569633947495', '831901009396237267279615788531418438.849563723', '5.26615819172264979504828722953264235810155283947495e+35');
    T('7665883776671616025905367508643801886903976481377913', '-0.000685209961385', '0.000289003802725');
    T('0.00000000000000130', '4084984.0344696574643138186852656784876', '1.3e-15');
    T('-23744.44703412', '-346011607426.441393159348354522971440097840365544826', '-23744.44703412');
    T('8434642778165686647233872583840184835847700452.5722438', '46', '26.5722438');
    T('0.0000000000000044601453487291992988959798617835937678162460190934757181', '-34957408111696949000006895.170094', '4.4601453487291992988959798617835937678162460190934757181e-15');
    T('-717775516482666361197134085201665528104789114410', '-2', '0');
    T('8276714696969298506623725320011988.89729', '-0.00616112392295074383996719947907960025278866567521351783', '0.00016560800550555404330800366686568701046935505208586586');
    T('-742605.43840212025', '-33.033107325052', '-21.18573495129');
    T('1172677829393895.629925304143622500366643409500207216001030', '37.100', '5.22992530414362250036664340950020721600103');
    T('10537016513706770761008.26248373823335994957890378', '94969063464568723440795883667.8253648809642701', '1.053701651370677076100826248373823335994957890378e+22');
    T('11451.575753798226574650597066008283798', '740458328282622534051565222682743827901164966582017856', '11451.575753798226574650597066008283798');
    T('1.572', '49515379454772293729194042647.91538', '1.572');
    T('-1', '7.059040', '-1');
    T('-34927437284064606817941117126.257390774', '-8736962940.827', '-3438064168.261390774');
    T('25409.947283363583521450111285770286331002016996164554', '-50439325984133502226433618670969699480790', '25409.947283363583521450111285770286331002016996164554');
    T('-541726513791047904976207535366014345', '-406733502885622404519647553066345.1969741566', '-3.642214502844845605566422347088878273975654e+32');
    T('-890820344589514985.0665574100007448', '-159324785981391287370653452100749211478426067970583818.931403', '-890820344589514985.0665574100007448');
    T('515196923836873751650822811674732842614640352.18206884311573', '440807068101717819348891.987', '2.0134973484292983702212721006884311573e+23');
    T('-443343.14120147', '-81519350429738153710983021502973.9213162228', '-443343.14120147');
    T('-0.0000000000000902702192042232797334789149068563741867484672231', '-7320', '-9.02702192042232797334789149068563741867484672231e-14');
    T('-45277903240084126.84557360106009', '-20896812318476199906609422.521834898564460854802489', '-45277903240084126.84557360106009');
    T('28165869957795828694237799735446687517170194.348151719250977', '-38550284385557947404379815866117040.122550', '1.842769006556153994091762719885883071901719250977e+33');
    T('15021482085.44976192138587853899725782258348', '-9904.699', '5677.35076192138587853899725782258348');
    T('4917.303585861', '-18.964359', '5.534604861');
    T('175476267.7992032958565', '-1036388592.3909824125745400050986887954', '175476267.7992032958565');
    T('170405946320111', '-17818386128487328808126289961', '170405946320111');
    T('8550180809850301212655626511667743382400090.02', '-6540107956116752962567.2', '3.55071180489327368509322e+21');
    T('-818705136461275003.08078930092467399502130815326105604518', '-39778740814076648429007.124204603754658', '-818705136461275003.08078930092467399502130815326105604518');
    T('-1.20675758374006649995086560', '-3', '-1.2067575837400664999508656');
    T('109.3', '94.6', '14.7');
    T('0.000000065573671480225125603602776141267516038374303', '553.480300880442516', '6.5573671480225125603602776141267516038374303e-8');
    T('296362415.06624949768701', '244445717421187779.650675510094172662242508672751839189', '296362415.06624949768701');
    T('29374332532.3238432343394', '0.00000000000000000005936857517089413921548905299963269225088201364509247681385', '3.119291720989480852560154607439451452770778662623725737015e-20');
    T('328821752173776490.719819431468332405273703018690', '-2542341869177777917056573693425171357215.954882228930423158', '328821752173776490.71981943146833240527370301869');
    T('-0.0012143107904367445170056', '-1003.4406', '-0.0012143107904367445170056');
    T('-3168184610490460.8945194460195', '-3573568062298202180968377963159', '-3168184610490460.8945194460195');
    T('-16901853866719201501332692.8432369358905099363918280445405398', '-31691345775963588152383111237433553165931816.117915', '-1.69018538667192015013326928432369358905099363918280445405398e+25');
    T('-93621088901252.29805520418', '-2.0188684214862682192621695358466139055618', '-1.8934621792649249958167189632731171199736');
    T('242258091070118423201219529950729719863290357823444711.8', '1108727758065732675.6587799039404934075575195715350085051', '882685173218682761.8921148545569801605823909846489452171');
    T('-3399066202816.303625', '2618943', '-318805.303625');
    T('-28834481903864652309.71183', '0.0000000000000004565222353934092602845491745327666313637270997826', '-5.34810620719243797216459942508340612766671723426e-17');
    T('213672683755841644553544017650723723396202652336.302605922', '-0.00000033677276118692029744817391550088675892', '2.934185014134463864485655375166849324e-7');
    T('-6', '-999457121904797189706147649392540679689483876019', '-6');
    T('0.000000000622195463951304359', '-448.8352331', '6.22195463951304359e-10');
    T('0.00273159264338707604', '131503167', '0.00273159264338707604');
    T('-0.8826707721080745', '-70.82', '-0.8826707721080745');
    T('-6.169582503328100', '-1', '-0.1695825033281');
    T('161790666328890520068178.994438807', '22534.8', '14223.794438807');
    T('-617570557615637.65256125752701839451975945', '-0.0148696896889671858', '-0.01332652388531174471975945');
    T('131606.29662152417276937754010546930907406', '0.000000000000009288216788868504125788635720602972497436211', '4.548851830590558791346414703724411897415634e-15');
    T('51.3977490595725037876457161936459379346361103', '-143177764436300664530563651.36873324', '51.3977490595725037876457161936459379346361103');
    T('-0.000000000000000000055480154187811579505094934438713145361465492675145621278503', '5', '-5.5480154187811579505094934438713145361465492675145621278503e-20');
    T('-426820677348783115368590283660534000419588.633035253', '5649682945695760173704.00', '-3.898820592824218831748633035253e+21');
    T('-18877995762227202359719425947641943773826', '0.000000003168736894305713249647662065', '-2.410163138756323232829425645e-9');
    T('-230734625.125675830245801', '-24887751.105039540142964', '-6744865.180319968959125');
    T('-2.55', '-0.000000000000000003328273034182616010363199785', '-1.2785182565162876532434802e-19');
    T('0.83072894855068062175257214671400', '347832.9728307138', '0.830728948550680621752572146714');
    T('1554', '-0.000000000000000009695649707683834887551815366596414977600762431389183799355', '6.05878904773660946108899894496980695021759226671044100733e-18');
    T('295506566.953646020992273150648001797147299', '-21973527294862.8233628', '295506566.953646020992273150648001797147299');
    T('2181379173600069460.2272418575314841', '-33121557.64630663141', '31392819.5003625467214841');
    T('-1102425661172241572593712529.4318', '-1655029068876808.4683087946075879962249', '-666832641952964.5659918135994689948094');
    T('98281131957208481431777620.35596736255967554627181769', '556842234645404079383345567305273.303513803874857641931', '9.828113195720848143177762035596736255967554627181769e+25');
    T('11835752763446.82223054394121198', '0.0000000408069264362150926140815302294483025590', '1.2484009569088335298499242835706081261e-8');
    T('-50148655247666948.0767382', '-374032530098.019610', '-243774774968.8659882');
    T('-173042648372113278197497944864583449315536350193384127', '0.0000000000000000000366841720', '-3.265408e-21');
    T('2', '-3771771239046067919037045', '2');
    T('-0.0053190456442251124794436212618', '798725962261113.7', '-0.0053190456442251124794436212618');
    T('-370307', '152017395754118405461987924364280', '-370307');
    T('-277699892815627339975120', '-143.6810986', '-27.6564706');
    T('-5775916.6464283799155032293780149633514899100', '2134.505358', '-2079.65303837991550322937801496335148991');
    T('1429207694023.57717520337', '3', '1.57717520337');
    T('-5', '4', '-1');
    T('-52848862092.572286019', '813661418640716426286793.75702600471722081556694278784', '-52848862092.572286019');
    T('1844275644561406088.1', '66184292639.361611328054340279', '30103188873.61915648682705296');
    T('-195554.912236539033400998430626025643711716348904995', '0.00177428671859871088664745504177813109978841798', '-0.00128510649380978957377209918017012798784363842');
    T('78.250868739', '200592382754.965094921301649595609', '78.250868739');
    T('-1.605', '-0.00000050108803722427943381', '-4.8521755346940298951e-7');
    T('-175950845282749621749.3060626398844805224872563725036868663', '-0.00031', '-0.0001626398844805224872563725036868663');
    T('570951706006271.614500911029949202716327198256', '9579594454359595046.96141239240213595155989807741151781', '570951706006271.614500911029949202716327198256');
    T('-101348056.99', '1.2', '-0.19');
    T('0.000001115108462416442750881276832385920295416166957454340', '2.391104283537695024603255485729184', '0.00000111510846241644275088127683238592029541616695745434');
    T('-384.29267210', '-42.6115840552808285042372718273816', '-0.7884156024725434618645535535656');
    T('-0.00000000000000394320', '1490433052780511340442598422397421169.9727476', '-3.9432e-15');
    T('-2', '-124412814393298131.63102541685397', '-2');
    T('39255373365250694983.01012619735480364604931589', '0.00000000000028924174', '2.8356244931589e-13');
    T('83649682374745658097928332400566916840.904100062101', '12515556884', '3441295412.904100062101');
    T('-37978362152425454856329406585920723065797640812416', '-40807030880665280609832.633548429382227699775', '-8.44139811680207521268650854435671218648655e+21');
    T('0.00000000117630746081997641552101447', '126289176518.39164497878613023988707803064880568', '1.17630746081997641552101447e-9');
    T('-1880854096029528626232093236836285358.8229354826767064', '-13.55', '-12.4729354826767064');
    T('0.02073', '607.8621233700367', '0.02073');
    T('-804231633252810472161116.753061839', '-51.139385439676348957', '-3.050773812069762116');
    T('5877876231822600040431086011103959340188703599.31851144', '-5388452483964553888.345931647692324169003115975043825880851', '1930961800526326288.60986182402199524006990099301495612649');
    T('77946964651085243.218457918863112', '0.00000000001144602007465348949097950134087916681540997', '8.85127498118220306889399715954400610693283e-12');
    T('-288323254635330759933511.4835', '-0.000000799099661848570195554780769778567229952228099', '-6.07308413294153274384235191515598586543048706e-7');
    T('34156.9612032703394033850068373910911950393885281598409066705', '-790.014721903868173877309773404251366', '186.3281614040079266606865810082824570393885281598409066705');
    T('-62309462096741992398456920.82284105487052640595', '-0.0000000069200714440436038475285366066199260381235724356693', '-5.1591434837700767752333272066996752713093302820209e-9');
    T('-0.03081418420901964033772890', '-2258.6097737546792912978930040107448576', '-0.0308141842090196403377289');
    T('19440352005832939832434953349.524735305804224848165284560319', '13663546035.3870791', '9500641135.581258405804224848165284560319');
    T('0.000000000000000000174792814753795064312182753542642524992498259236217566516', '-77523522800322397605770.38924241467650641624', '1.74792814753795064312182753542642524992498259236217566516e-19');
    T('-1815265994282465978739.24367005494108', '0.00000000000000000687759', '-4.05185e-18');
    T('-1.34345147096631', '-6865002719881812387470285582221951284174906039012461831', '-1.34345147096631');
    T('-1331063.88165', '11341955270415278370914418086588317083074.0482', '-1331063.88165');
    T('2.046', '-0.0000000000000005877097220931', '4.012083803814e-16');
    T('3545.91036649009998376741645699670257681183407320627384466111', '250.0', '45.91036649009998376741645699670257681183407320627384466111');
    T('3304.02', '299035.58', '3304.02');
    T('-0.0000000000000000000942622', '-0.00000000000000000025221802', '-9.42622e-20');
    T('342502985242332681201199551069.576', '-25743002421256401.04397330499669057', '13427630853006016.61638681371921084');
    T('0.00000000000004233681060170499506821015920037', '255095113757561337754353944933355753406.38155', '4.233681060170499506821015920037e-14');
    T('-28.5269472861067005353912', '-270472762833365.3801132655767', '-28.5269472861067005353912');
    T('-277847759.515216', '135166968528972', '-277847759.515216');
    T('-29856675437162421995266099874726439278', '-19896996992.6267371979256450155346521949519355', '-17602172309.435172515150769689705114114019276');
    T('2', '-2017800319934087984840833728734', '2');
    T('0.00000000000000007184336760908973', '-579084.38782881989356541750992', '7.184336760908973e-17');
    T('-0.0000000000000199905729465678002272254515680192399295794760827', '0.832997729248146391449238508309446517475', '-1.99905729465678002272254515680192399295794760827e-14');
    T('-0.000000000000000127832283699455', '-17051853.75327', '-1.27832283699455e-16');
    T('-1698692.68276072', '0.000000000000342190555852264550226068807192', '-1.32201147548526749368535158272e-13');
    T('-89715846809778312.0866002761050235922348184666331', '5683352', '-4378864.0866002761050235922348184666331');
    T('1867077.167389665646672502266524422230512016', '-8121328626157873800223785135.111300799674416212197931655390', '1867077.167389665646672502266524422230512016');
    T('215901.879', '-29.8', '0.879');
    T('5119759698.60147051603509092616983122712457476', '97582629730470.286607', '5119759698.60147051603509092616983122712457476');
    T('5632091904144.80457549', '-260531502143715092510.9558622691', '5632091904144.80457549');
    T('-0.000691477562292642246090904144721704048411477714', '-202833029325548043615849420487515503451726003', '-0.000691477562292642246090904144721704048411477714');
    T('-5014290818995813355670219945.489013253728448913277', '-49', '-38.489013253728448913277');
    T('-1.19', '1.14', '-0.05');
    T('6.01200548016561079', '-25.3', '6.01200548016561079');
    T('0.0000000000000008934209997058692257', '-9845722515207.841', '8.934209997058692257e-16');
    T('-8271.180908766644265814', '-532.21', '-288.030908766644265814');
    T('18149268424457.205469', '21102403', '20109889.205469');
    T('0.000000000000000005630166790779595999136', '0.000579481131868747005452691950764827230956628754512075686', '5.630166790779595999136e-18');
    T('-0.0000969512096310107418573215633893505216', '146313161286844954805207556174167131617843885.68176097', '-0.0000969512096310107418573215633893505216');
    T('-4.4', '-6.1775206278626', '-4.4');
    T('-534834.70968332438874787934900992041831587', '-38363548234134309078598125637011026.84251944598377781303061', '-534834.70968332438874787934900992041831587');
    T('-0.00003406607808', '-0.0000652979835334948349904827499755959633629915945362770066', '-0.00003406607808');
    T('-1', '-2110983581171793713817325.1475619322694002968191006', '-1');
    T('-80227305.1656', '1944676344179122208154551672251566716245569224444.0', '-80227305.1656');
    T('-45889999955242942968871808966625304007272799508621', '8550404.862499169389885162309987843918802343917887', '-4729231.891447747380274990179385678025799484562412');
    T('-27878389918833569427320729603169170858.95474004228507890192', '23932009803269840591376795875', '-3.41193385706058529612983948395474004228507890192e+27');
    T('567202319309292978520604165782060160263365186515328568', '-11334572195439351811672.1473522129007618', '8.0497193228717959915213827171314311074e+21');
    T('-8697359171345893713842426564522034778871562.5154932', '15619404167072.618217767650', '-3493725216866.96939850825');
    T('-218.363776', '697131146810968473489.82299654842890962', '-218.363776');
    T('26202481114591937993.735307329625927271775', '-304156997.304184411106538785706126202394720899', '140113250.272976945765125367941961760435198404');
    T('-0.007731087980246464302345811575', '-24413038924588027145692146.387588856788321', '-0.007731087980246464302345811575');
    T('680497639988324180.39625512073408', '-4532942013750624897951996571799269425412122', '680497639988324180.39625512073408');
    T('-0.0000009293033204732942233089593840249955775555658283600103866', '-173511.41982571409206143199884579179605', '-9.293033204732942233089593840249955775555658283600103866e-7');
    T('0.0000000000098336953795657517132120257800838', '23300757376665714817666483157023722058388.5033214128449579573', '9.8336953795657517132120257800838e-12');
    T('41084687427290874759976776474299799533721185272283', '193585163.52662', '163743760.94914');
    T('-116.95132', '-39255072372928843286086966431.00', '-116.95132');
    T('2347142650267689935587322698.7284595028691', '-3', '1.7284595028691');
    T('0.000000000000014286361610', '0.3810969482393140582407695856576689976423039861', '1.428636161e-14');
    T('-7308074599589641011784024611055476301285', '-2979647.083231856789860049490151382783519022739462374', '-41940.765367177770459610205511353021036633288408806');
    T('357753139664202.5873944', '-41.95942286390296', '23.4743830801152');
    T('854571067694022997.4884', '6537373963.3180845607368152893095548460', '6053336053.291260561298904945337925438');
    T('-0.00000000000004386908462560891112864885989973', '129740635819911122760039.7464589619454', '-4.386908462560891112864885989973e-14');
    T('0.00000000000000021290574183432875072166863135108510281580702', '61908914489485349291471752909373768165637670.8208', '2.1290574183432875072166863135108510281580702e-16');
    T('2871883224301570575582468652.9376262287274595077514806', '-0.00000152916795423240707414339', '5.3155439780727520002144e-7');
    T('3', '80708788610217964553584510306368.549353793563', '3');
    T('-10835622995844.77266724348161957054233451560047028326613046', '-1175193791680339.76056817361012416666902891954476177079', '-10835622995844.77266724348161957054233451560047028326613046');
    T('1025038648031221253.57383030896217', '-5242.40711853563044167975', '2542.2793327788128154595');
    T('-0.00000000252', '-220187026862816215614922589820191309081445441861274232.84', '-2.52e-9');
    T('2438882128515521004992810616244816644660.875', '-14585625494005545128827621', '6.211192260353379557970309875e+24');
    T('-1253637.42184071686942470', '6043.294', '-2675.5638407168694247');
    T('30.3149681983737', '3.96', '2.5949681983737');
    T('458648663984496211521246069201336181', '0.00000000000000041796956694972368386032730115563266442065659', '1.189340470755219691379214537787056846685979e-16');
    T('-877569106991311.769260729863', '2246530785645583.53767418', '-877569106991311.769260729863');
    T('-33180244.9424', '68040037.571844483716882853268188712468340362', '-33180244.9424');
    T('26056925297726128046555.65872815844589322660275', '23799748744565261102060860.628349245228', '2.605692529772612804655565872815844589322660275e+22');
    T('-2.932266679268', '-2061492876.118067901623547245130932150882910', '-2.932266679268');
    T('-24303392384298907971178613116155773044858954198134', '0.000000144547733182595133021', '-6.2763446759435083203e-8');

    T('9977', '793339.568052455522323049704543732060826', '9977');
    T('3044797849984420605285366.474651143157', '25186260049851668345583291614301619318511974775753812002050.8326735955299366787084106211969141751506', '3.044797849984420605285366474651143157e+24');
    T('639737463904283952640443433078141655542321388667481890755297474427803997023609757112922145297681502347448794470168805201764741986941', '-14552442514714892538227035420.61249278214225741', '3.07716512012830819205367001585789874568359778e+27');
    T('0.0000009480666812553237528290117310639076036954221787358487462393246935364400792376', '1.0', '9.480666812553237528290117310639076036954221787358487462393246935364400792376e-7');
    T('16673429143256557896853574875292172327063389650998117777668136985538005.9156698099892691658894660539763873869793114863329930039146096813387028', '-107478408440617999633969729.9071529239187240331642550654002067338975836414285167506215860', '8.20260811545606661396908107304136068380872993912615354808000919495595106968750582594596813387028e+25');
    T('-2433298586307081.77465641473337660801763802817481653388429397435772288734383710955919905923279924807', '8945608818026652762135420517379.424606537007', '-2433298586307081.77465641473337660801763802817481653388429397435772288734383710955919905923279924807');
    T('-70843295832985633806095066724872543838444364150876355516508644344503020757888058408332911592444048015580157647', '159194370214483300881159411658565511338185028333524386712582766324718381977826309423371642975467856662959008464353958219661', '-7.0843295832985633806095066724872543838444364150876355516508644344503020757888058408332911592444048015580157647e+109');
    T('3.2263', '71756256022538928427069542438.00966178692388729446382444038774751007836179716329976575691385315023922435789196791754653262230986812059146', '3.2263');
    T('108061134.3910585273259047952072200021695505338334592792817809710438', '1260951025714501457005284501987116458944716243549637898428510181316826466436575714044748818353924236900363398289', '108061134.3910585273259047952072200021695505338334592792817809710438');
    T('43607983963968869595947769368216107706521387015680146006305613629433990860442947600228456010092', '121385730326389922895424600532542775257956396540395781939587936260842487516174244325430.384295616198856908112758', '1.1504925896106825966642741273863721105609952802207648727165046290817382897631508088378461034727326937297965944e+86');
    T('-2.6480956814', '2148671523057716670055099537649941488566244507589768590863804139800870729193528442.071130969310679730958621114', '-2.6480956814');
    T('-470479333263.84377245126007813562135983915023544360010239389244912214130585392185699431847364158901094167357758815083446794565320814454973581796', '27849448232164647723510637195798019946766053448137414448566670989993.67717797401551478004030639298176072944', '-470479333263.84377245126007813562135983915023544360010239389244912214130585392185699431847364158901094167357758815083446794565320814454973581796');
    T('0', '-6993020667326513594.01598727239030644128692238032346465300777782504759087255641592855556786881965341488969418534051814789', '0');
    T('-10372286284244779022048585', '-0.0000001930730138191035178741471788596965420386122185118633654121349245216394001018297980054707996074710125164147', '-8.47592995170001700266169749815180334434465081954780254569435821161404742961756995939834569063835498482552e-8');
    T('274897009856813.734380955979520483734957204601027214931003941720546146487845256149072527968056465339318846303131569226913408938616354923540700172412', '7', '1.734380955979520483734957204601027214931003941720546146487845256149072527968056465339318846303131569226913408938616354923540700172412');
    T('738408012842109785457164849764507323379297.79095792233769102310198027453532813226348191188862586945019214410536832', '15306177324248047715406533.5158700026', '1.506634005344982055089315836033881173769102310198027453532813226348191188862586945019214410536832e+25');
    T('0.000000001307002813605936443383593439823579112009200431906135937099617451361199016339230482184530115838223218189460633917243083200', '-156347131201.0667716470724866', '1.3070028136059364433835934398235791120092004319061359370996174513611990163392304821845301158382232181894606339172430832e-9');
    T('-4792567274869561212802263.9162146352653952716695372928501404982289832583166534206544945624150460220976057951121526613072397239125750526383369389', '-2571615359217819081745372380499521952091705406387642084116393860754896982150026573094709965566690085424189906211974018648786870967996877095142', '-4.7925672748695612128022639162146352653952716695372928501404982289832583166534206544945624150460220976057951121526613072397239125750526383369389e+24');
    T('-40.05748869168454725527441481953494139473673991070979113635160468475219376406826789262579745739976', '-41875468662578995790027634863433077814557063632577215827031.031982081918392876105825240291071725658144212578666', '-40.05748869168454725527441481953494139473673991070979113635160468475219376406826789262579745739976');
    T('-0.0000022907766811721016183225617138609000543876574860984718810338550238748128247445010491861049613', '-3279074504656188611648707.4983942082641992057679531912380705659521798322669577604617597486447068188912830230847051445600458515668443081070949175', '-0.0000022907766811721016183225617138609000543876574860984718810338550238748128247445010491861049613');
    T('-0.000000000340313277', '-0.000000001255112168870655023338247473949331250106676281883642901353991', '-3.40313277e-10');
    T('-50225365.1475566438998761801442412', '284028617245837000189928101232939099744527904229835535.835746075', '-50225365.1475566438998761801442412');
    T('63.344721423304857459331086841825237874925064097', '57155073593666.40553442288889921516416834083269742567707391607', '63.344721423304857459331086841825237874925064097');
    T('-405919933.1066287819696386581647229483641869199888974504501408794124002251348919063696581146156799005', '2647178729438306865645909642983752549443679075960964976334982663855617.77', '-405919933.1066287819696386581647229483641869199888974504501408794124002251348919063696581146156799005');
    T('-0.000000000000130754174234156809102364575558', '1512769765054847098913787446631276445464675564737758845532029029227218416561642292270741819169983956650914922547871552583245517440681359875', '-1.30754174234156809102364575558e-13');
    T('-42805325218726498366672740080295203192725414965527973209314316749.7120445518644682586008052614425260815242140724520046592053081', '0.000123298332335657505901061098051881119196170384724006568439947745355121025753230625359269654114501774394590171609541470588517', '-0.000032325304000891296915093212441483965264186154572168660357953720611674306961720593830907904561258705086770592138813164224068');
    T('-75210406056668422373802918016368260769067586508935441638493539318778550326.50528533489988415483952', '440622997.6355053630471029', '-340429684.42458949795004585483952');
    T('-1222922824327546500522220970819290434122898473371530650066004876062845284114.47721875136651190', '-1379027454245733767586', '-881180208974102493198.4772187513665119');
    T('-30656582150188875494703614722772772221409126232115063256882064964663663147033209144753246762464712.948730003091026599897432459556025696', '-2913934951186033025005819092772439514996587951119743767357433798109195725658300879072952407504422194931477368634077', '-3.0656582150188875494703614722772772221409126232115063256882064964663663147033209144753246762464712948730003091026599897432459556025696e+97');
    T('-6460522065030402275014019.2874194931729240210323452441436174892', '-84550416554285528170014238135907266547165974999578715609084038263.46129840157771271779704084743623515311185778729738812', '-6.4605220650304022750140192874194931729240210323452441436174892e+24');
    T('-0.016364386296444405533757351682555346465502820950062088651150128675773350177464733695707386307349689534592475248473689442', '-662351516465.9032743599806965674027961010966377214913163', '-0.016364386296444405533757351682555346465502820950062088651150128675773350177464733695707386307349689534592475248473689442');
    T('1103343525164099798605597537113983882179727144422100669678631936027042820706120285756898888893404677995338896062374110057717750895679', '91748792242920404655427917872546031638524.76204683127448751123961871264830271567888066463998429231705219642715054074868199195373673227635494751953719', '3.076328553647557636193208401331029365654020485144903415117912662361833098755573989373555529561896396760648323565651091850507799771898789127729144278e+40');
    T('-0.0894652716283348818337684886989151280762017763427032659791509101256464820297', '-12966.6884', '-0.0894652716283348818337684886989151280762017763427032659791509101256464820297');
    T('13114418006467273871814114136828793244459258873580991741468083458844936', '4759000553255960840361168620.3774494470837273198', '4.2761722071204063036259345611428685193209688534e+27');
    T('-28951495771059579052038784935583719855135835316878812739160571275658593082655402834569052032878823395641241968136.92574783376902191547024436698530', '-3031768663774740737176934765360849728180780437211886327623459816735182479695223019269147895285170079834032202516966937529989903070284157092473214668.7', '-2.89514957710595790520387849355837198551358353168788127391605712756585930826554028345690520328788233956412419681369257478337690219154702443669853e+112');
    T('2393723882767965821294763322394195744548891304060902', '-140981135960273230.0025778', '86668711566444988.9814302');
    T('-3224488416109581472101333552995382215707383396998952009319985210337309105631785072396803.91964338865842816515912815197781787', '102818159313377657005579111128189067224885718.140628085554012525428767372817189722551478405035418445112060974169840360487565259217', '-8.1595634025043932590241822413741540317627232887069206304733728507469999734166316619875329520357299837254473649437591096347440866e+43');
    T('0.000000000000000162614535957335504400161243268689702722565890737357214918357356590168738846464706769774484467251167640135858015634083627803179977581109102', '0.00000106803134832455205844954340150714254354766339175870844951355513521917239107042458407307482848507498559', '1.62614535957335504400161243268689702722565890737357214918357356590168738846464706769774484467251167640135858015634083627803179977581109102e-16');
    T('-3927.4031809', '-0.0000000035029583866611658486269733571456482523405154883369032001554318344330512146832289592781514675456658500756286236294841624823979375753461327906', '-2.6932691679676130460056959057199199083055272525449126446015947490329456323969370838199750438158748550553789684640297326857241719276712355958e-9');
    T('2592701578783037181466698143885146669128411307594485136303860886846006143235135122817836647040961317262173311873893155', '-2476735053659095937599211826013828027536167714634610749923083443278572078026871812469079060633677096037053974280521689349022693979242065', '2.592701578783037181466698143885146669128411307594485136303860886846006143235135122817836647040961317262173311873893155e+117');
    T('96.04335877730532081621787788158383111099350287030581038188446699532128180774027960706578', '19731419380883276630592147059117156462461951865864609673166.4479371059569659447212290002838890816175493435199181043231716596224989574671224595362826', '96.04335877730532081621787788158383111099350287030581038188446699532128180774027960706578');
    T('325023114939529014657978169628372760601480839073817238211782426113558723164836055970989007713696933704', '106223840476390782472173917090144170488887005908229635819404889799358852135763.040387492', '1.05150216674283695595324235267625323135707751092953285615150305395692443496735212656796e+77');
    T('-0.00000000000000033647992217979179179186957', '-0.00000000000000000003156150266634952697310752', '-2.74225383948473157029928e-21');
    T('67711097909563003399071203293654909244735459519561246144664768566612029202546308760389038038790661412078936611567061527752843866673871533', '0.0000000250914595238219383127319568059987481594970145735261435136883603429568', '1.1931039593784471503157310210681907892950978516052505236994695043296e-8');
    T('942307044555108835920493820461160734540503253601173239.92559776036964181595894387449189836528222244015903442453788542293705358104537', '-341315427404722210577937787468731.95', '4.792118403841194654638154768215812559776036964181595894387449189836528222244015903442453788542293705358104537e+31');
    T('-94130661760152187286400649382778774100247360104073487553172289492592909079685814047724040987.545360', '582.270609307311781203990506333968732999092917722', '-429.29359293653441023844377749905124203115916886');
    T('3313638785820621590180176562638.00992668522579291400041235393589181972606810182197653338554677836315505146', '90527335216395767852035210539.14', '5.465471803037394750690898322896992668522579291400041235393589181972606810182197653338554677836315505146e+28');
    T('-0.306955389372676043580907106603221153651127238512560719', '66.904579801977727781298', '-0.306955389372676043580907106603221153651127238512560719');
    T('141390900187450346.4446228025914244', '-255850256279441962530367023331027625163686895444551579538905939296353570502455036176395785785059148507467346341628419844281564795408561', '141390900187450346.4446228025914244');
    T('-31281694978574998092.32325767151', '-1542462855.73276869937293894166028800355184049470473', '-519767223.60416992161136296357867627633975428783569');
    T('0.00000000465065764192519767209223768411174160970559048882576195807880851927339930867660413363177803', '695983282190820825900286910552083743912340874031142607911981006600892487992712378.27501435873441215408642204715421306049', '4.65065764192519767209223768411174160970559048882576195807880851927339930867660413363177803e-9');
    T('0.000002299699', '105', '0.000002299699');
    T('-751157538.56375056723760379542086174039490718435868835658852', '-0.000000281235130627575144572363209737439923', '-2.4952865744315606513538295859410866335658852e-7');
    T('-1492151820229229.7080972636031810741838557504286290118227845200639080208767965213313891261997405771908981', '518534900915966328248424169467.8926376495890764066267652', '-1492151820229229.7080972636031810741838557504286290118227845200639080208767965213313891261997405771908981');
    T('477573736732380117689005270695410306243996300502014352674644518059825662478485969426161545468188704450446835856', '-44107433160804154887771872206618010747863690779283512394813143542934429409262133052168377457307.7801829320544', '4.26587795612760356325706643922768450442822164299827765862986444105362242670998534111196655345782505509351168e+94');
    T('-1708419546267714621908304198526.86627770247445328880', '-6561936.23828726436959004970934079513865615878124751998092886724055429258975571944079950471550813589210381712752008129366876', '-3255473.411304257056507423459569857877329230063250469248244750150261213529383645499126139366386431281819493395691345793149');
    T('0.0000000000000004527321152996104608068898547927166509996670208320178171898523653701667544358478578289218730097763063669131852', '0.000000000000011707656496202100248575019665652640050450842478555351860415163190039866', '4.527321152996104608068898547927166509996670208320178171898523653701667544358478578289218730097763063669131852e-16');

    T('-4.785068183e+1', '4.100602340813778290214569907944153985840287e+41', '-47.85068183');
    T('2.80464647349124269012895506839600253935327310965e+18', '-9.5524432014496391100798143628e+26', '2804646473491242690.12895506839600253935327310965');
    T('-2.8475390576301982648821864330615e+1', '2.699279786373606e-17', '-2.6651296488681e-17');
    T('-2.9148775309735883444412038961e+12', '-3.7791910283459953859509024618806015827414491747e-14', '-1.2484519780688810034904273898769796515468689534e-14');
    T('4.75952e+1', '-9.2480596298788483952588449013e+15', '47.5952');
    T('-1.059098439e-6', '-2.61519835738114698145e+4', '-0.000001059098439');
    T('2.0737545782540501468649951383811505929864648e-11', '8.001141125055718003273562341771790690597363615833e+48', '2.0737545782540501468649951383811505929864648e-11');
    T('-1.22658750490728083398e+13', '1.8753238083480045539938865768259755398e+8', '-132148191.84975394758545561202438438412');
    T('3.0610455075185277682819215983121128590345078192561e+45', '-7.2132877592078603887799551646099823518259e+16', '61373855855082630.483565489609741494295639');
    T('9.0188012787220251521914339e+21', '7.34972521506799782052e+20', '199131020640427767567.4339');
    T('2.732276e+6', '-2.155e+2', '167');
    T('1.81970803337505025123814446711e+19', '1.552273391587321102352777093263e+8', '121079294.0355602541613892510099');
    T('5.29521849750516603814171596807e-3', '-1.186023062792866863315593132074754530817309278318e-16', '9.65458477943067357182194529851732179008303939488e-17');
    T('4.17590665394570316906941267960925070278431e-5', '9.049524633791428612160450019368774628e+0', '0.0000417590665394570316906941267960925070278431');
    T('-3.349078310428002e+5', '-1.6540638667696404135343644190345e+18', '-334907.8310428002');
    T('3.1992755000743044238501551275241732747e+28', '6.7694639443884902632801069e+19', '58012283694485729642.404216747');
    T('7.824216663065598374539197703051941e+7', '2.8264285749437427237554422632944775049521496e+41', '78242166.63065598374539197703051941');
    T('-1.67058990412026696591650976e+2', '4.22502522584451063700771426002353559984275436807335e+7', '-167.058990412026696591650976');
    T('-2.45635416234991644154081e+10', '1.6511720053714564535061012818217797956053950352564e+3', '-169.9750792296892367202705840388216243464507038608');
    T('-1.234302917437106938892e+6', '-4.66222869142052358935242282170391162075612576838441480514e-11', '-1.16831454856809476137891894092632703280451615639647891166e-11');
    T('1.839710778457895734645309275939324e+23', '-1.16318585777868797619955e+17', '90437897170854441.0401789324');
    T('4.31278525467628823588407602838382746643207760247774704277223e+9', '2.23171038985847514593e+20', '4312785254.67628823588407602838382746643207760247774704277223');
    T('3.0774747e+7', '-1.292178569466e+6', '1054639.902282');
    T('7.1986256721501851444121681387782246892728127232221676165644e+45', '-1.13782811077947907584409056830168328555517006340535538318684e+28', '2.4664369028970742315721904655905118416446174179007597618692e+27');
    T('-5e+0', '1.8187074446258529890089548133661805568989582581e+46', '-5');
    T('2.90713213183891582843410750914399986513758162e-3', '-2.63153627e+1', '0.00290713213183891582843410750914399986513758162');
    T('-1.1994289233986543701822561078984450080807572663172692845e+26', '1.3308098373134e+12', '-1101301973800.58984450080807572663172692845');
    T('3.80222940163e+10', '-1.86439621006147e+8', '175050952.052159');
    T('-1.0589771547255325020757865564001533466983148718e+46', '-4.66653476326811630302573645173380627372e+30', '-1.34743461530630312563356791528734704544e+30');
    T('0e+0', '-5.8555382904168e+10', '0');
    T('-3.877410561846137535676828506580217538851e+39', '1.8546585782329946703671500695022434e+25', '-8.304732800863368666145000987852196e+24');
    T('-1.163627582049e+10', '-1.190310616961783801595291761751e+7', '-6941092.77337225841399948769273');
    T('5.50922178592853292e+8', '2.7726691028626755345614e+1', '9.067016077612284334026');
    T('4.585813032273175774206281699078636679246584e+2', '-1e+0', '0.5813032273175774206281699078636679246584');
    T('2.143209743784799995437859351e+27', '7.89686033593298254046863395059167e-18', '6.81868949928986360863748711628261e-18');
    T('-2.2928677299898057914670255997347455786e+11', '1.43685706448112139571439918598028632e+6', '-306934.405632425577309870670368346');
    T('4.354911010988471845818170482045217459828e-19', '-3.4255381103210061953947764102346444839325208e+5', '4.354911010988471845818170482045217459828e-19');
    T('7.6205756951722652979726396821744395258e-11', '-2.469701682300246385132843673488255945931294e+23', '7.6205756951722652979726396821744395258e-11');
    T('-3.0414336636936289483194714509757754918625515721268383983891e+22', '4.536514951e+9', '-2390201439.714509757754918625515721268383983891');
    T('6.1950756263395220720928988581672084e+12', '5.50299097228515217025542686233486e+6', '994424.9277391452982364908047321');
    T('1.5592703628193685590607028031257876838163995140476313e+52', '4.644158636847e+12', '1438695302628');
    T('-8.321953e-20', '3.425924048971005175376011737e-6', '-8.321953e-20');
    T('2.50545873839057298343115607524742030919655663396773e-12', '-3.81e-9', '2.50545873839057298343115607524742030919655663396773e-12');
    T('-6.626563801610720283429069e+24', '-7.318668911456127669428074219e+27', '-6.626563801610720283429069e+24');
    T('6.579221387782276980734304828884120337616419799251099653496e-1', '-3.8969000319182257446208467342247396396093882352e-15', '1.9767722676183043776377919004479568382127247136e-15');
    T('3.1401513e+2', '1.237032386460703e+15', '314.01513');
    T('4.232794809535764324617667621124324483982653495945584e+6', '7.3809184815598732203484798105535965548268885342e+0', '0.4405677713499586621163276690389136391131296364');
    T('-2.54046333929067768396256156730192176704492639570003e+1', '-1.43509613727037812846035760529266475517733244287446e+5', '-25.4046333929067768396256156730192176704492639570003');
    T('-1.7784812894976825e+5', '1.348176239051319766183648668812555533668836074076e+47', '-177848.12894976825');
    T('-1.34525103103779e+8', '-2.1e+0', '-1.903779');
    T('-7.5e+0', '1.1485396369968399088809663888589e-8', '-2.897466364518558819119797833239e-9');
    T('1.02502826667e+1', '9.3481096028282686300763053917132870455928e-15', '7.7634911550320902212280587730161073680336e-15');
    T('9.56858647594739448303715711e+11', '-4.2315e+2', '168.239448303715711');
    T('-1.121662781899762e+3', '-1.153e+3', '-1121.662781899762');
    T('-1.1987521321e+0', '3.3717546189279471902193720771211135e-6', '-0.0000023276964037193028773035373618806855');
    T('-3.96409663517864928212668914312890035732488673186557035e+16', '-3.503768915124379422490579250903005526551657e+9', '-1073469883.0404370184683678233259100444904567035');
    T('-9.780642443954005196506214589e+15', '-9.0202154557465647565999764323022181914347547218716e+44', '-9780642443954005.196506214589');
    T('6.22723611860453496632948443539098691396413e-10', '-2.77824094902753769648885834726391520861911e+5', '6.22723611860453496632948443539098691396413e-10');
    T('-4.7319718694e+5', '6.8346446241247783161447574947601984306026821486383635e+33', '-473197.18694');
    T('-6.1933379823501920314298624358203814e+30', '-8.089868231074264910103113854481018685411838271163628e+0', '-8.004699925089141256941499632457605509538837084388632');
    T('1.5e-3', '7.82546748299781600669e+20', '0.0015');
    T('-6.7242596508788e-7', '1.3806352e+7', '-6.7242596508788e-7');
    T('1.9762609860647814575863656621674468865989380864741e+45', '6.51277646556899014436272323991705259786529655e-5', '0.000014790203177222374955576884258500785345510175');
    T('7.004000748022736e-8', '4.736649441368895442161035451e+22', '7.004000748022736e-8');
    T('-4.29285701374473103585848390846585847338e-20', '-1.39970996245e+6', '-4.29285701374473103585848390846585847338e-20');
    T('1.44068396723397585225000633824556784092935879966094079042e+14', '-1.85739402953385383572444765551721122270675395703645487708e-6', '2.2082902769721875195705810453565730977529281295245227248e-7');
    T('-1.90763083929612502667113570129e-17', '-2.989745679222607307608195e+24', '-1.90763083929612502667113570129e-17');
    T('-1.8008915784029129140878223332849e+15', '1.2709184544773109938594286324169565032629e-8', '-8.198978038773931537165401098330611286589e-9');
    T('2.5283393571163e+11', '1.42605397244976960657e+16', '252833935711.63');
    T('-1.07482517991266460493961480563252855043997503069929863e+53', '4.0293147259356676513464190395e+2', '-327.97566262501963085789832435');
    T('-1.791459937048480305498333255301e-17', '-3.60140444532360031210882850839206992361061e+32', '-1.791459937048480305498333255301e-17');
    T('-3.7973618905e+4', '2.076660739009108092554128277835978289058948608e+27', '-37973.618905');
    T('4.296835337235213305241997742797979761241859185e+32', '1.016128323816741556885745789299051e+30', '8.773810728563935184150511955984541241859185e+29');
    T('-3.3476803669790089330287396786213536648e-19', '-1.0485228843496121655433179024642877076320529176e-16', '-3.3476803669790089330287396786213536648e-19');
    T('-9.265001828356e-19', '5.82146621596502038608123809e+5', '-9.265001828356e-19');
    T('1.6574237778185417162412521417931606051387220789593e+6', '4.3416420707553748006564234818e+29', '1657423.7778185417162412521417931606051387220789593');
    T('-9.774062277712948833378e-13', '3.080037033054419810654271948e-15', '-1.034488293043803360395792484e-15');
    T('1.0480042083686042910696085478682293620068215e+43', '-9.1636346772043682784439956904897547881091051932e+20', '711021412902451879505.0635467550321309923369716');
    T('-3.40923816647136940940085344632137226188233200061358e+20', '1.6073165168980479523827835495337e+31', '-340923816647136940940.085344632137226188233200061358');
    T('-8.7800373379e+9', '1.589967630656932636460925283862063976207125245919e-11', '-2.20858757786189096197325170019616955392212367465e-12');
    T('-1.508492483290420652523454481016576379245159230227181384539e+43', '9.65407339051e-1', '-0.95207917271339');
    T('1.70797882e-17', '-1.221287649130143636399302725332474299568133811751e+13', '1.70797882e-17');
    T('2.25e+0', '7.2018713275608502122703864334941124435400603003965e+49', '2.25');
    T('3.66e+3', '-2.00125759501079698887197092755e+20', '3660');
    T('1.074410572430091406447031457374002693e+8', '1.14072231468921335177378921904e+16', '107441057.2430091406447031457374002693');
    T('-3.8968391781145045605e+7', '3.823518283e-1', '-0.379345773205');
    T('3.8807293359084454309373e-14', '-6.54951383e-4', '3.8807293359084454309373e-14');
    T('8.3790348726489133341557987289125627922491641304972332e+10', '1.88761931097750519978e+9', '735099043.478904551237987289125627922491641304972332');
    T('1.5282357157991306e+9', '9.951e+0', '4.4051306');
    T('1.1776422045e-12', '1.94520506072334528824768019674726837220814205e+44', '1.1776422045e-12');
    T('-5.738265352165406462762631804597466473267412948e+18', '-1.5172659934372962e+16', '-2999896972426826.762631804597466473267412948');
    T('1.021717777232460732646336675651455853866e-8', '-1.31332207597e+9', '1.021717777232460732646336675651455853866e-8');
    T('6e+0', '9.3047796711618934817610173062034274e+10', '6');
    T('-1.182956037757676026896657463893805186423539855204970266169e-13', '1.7827584304157093780925160931953100051299721649998e-2', '-1.182956037757676026896657463893805186423539855204970266169e-13');
    T('2.2104533461720397849851124115599254191623707092353e-11', '-4.0310943429346438323447892668604172339295221650618858e+41', '2.2104533461720397849851124115599254191623707092353e-11');
    T('1.722426520663101950880778227697e+10', '-2.69592201025681600543464676120992735285696714886632140343386e-2', '0.0223237854099389241141349574091483444628934259089076259401818');
    T('-3.25603413629979799893671467886183127646824550309804207e+34', '1.47161894610216114600651e+17', '-34172500003558300.1070534550309804207');
    T('-1.08739354e+2', '2.9193728838924110750652340949692628787e+16', '-108.739354');
    T('3.206759288064547512180035e+2', '4.35800600237216221853047563599293340753408e+23', '320.6759288064547512180035');
    T('-2.21248e+1', '-2.6325594996226359777509157939720585725219065458496707707e+52', '-22.1248');
    T('7.073189319e+2', '1.08209742786e+6', '707.3189319');
    T('-2.1907210461355111297041e+18', '-9.2745316314605247152343780428919501619175952441700562494e+16', '-57578770899590445.200193050134851462758953093840887062638');
    T('-5.16796251732448951395790039385689407742691059264573e+50', '1.6780101630562e+12', '-1253088706638');
    T('3.51223602606475213080940254052176926846990239224958899e+38', '-1.63965e+2', '137.150239224958899');
    T('8.1882456091960208142365059003084619285549256998568554e+32', '4.64295172738148816415249852304269862921945245047076609476e-12', '1.24381368401600627920725181533134960575834680987278935368e-12');
    T('1.5813114524781473056173591272288745672e+30', '2.42e+0', '1.9745672');
    T('7.454553e+7', '1.538906639e+1', '15.23005577');
    T('1.51240970967848419751948592395604701e+29', '6.5611662e-15', '2.51558e-17');
    T('8.27911469397271464898e+18', '-1.2068203240037786014348e+22', '8279114693972714648.98');
    T('-3.20852252575e-6', '2.775014697501612276189583840612e+24', '-0.00000320852252575');
    T('-1.170515851548909718864331941709405324e+36', '7.57552e+1', '-34.552');
    T('-3.84525210626e-14', '1.010105187954696781e-14', '-8.14936542395909657e-15');
    T('-1.77609228049985890329999e+23', '-6.46964642626693630854488379333298772649377016e+44', '-1.77609228049985890329999e+23');
    T('-2.6073763079e+3', '-1.0248349080124110516237552578873628724235803267839057233638e+47', '-2607.3763079');
    T('-1.3843135851252423514202512138558e+2', '6.264100013e+5', '-138.43135851252423514202512138558');
    T('1.737e+1', '-8.726745986924851889490167717196421297465059646521e-5', '0.00002298524516705360208547066072715688662132777520597');
    T('-1.841251966e+6', '-4.01007093384313393333803996623363e+3', '-629.40736600152459783965549876383');
    T('-7.9035320783720205756630542761451846e+28', '-2.599e-14', '-7.27e-15');
    T('8.04680904556172181303021860819810222422429215449526e-9', '6.067916963498582356994976035938910323855806689699e+7', '8.04680904556172181303021860819810222422429215449526e-9');
    T('-3.2591991380971738720016943747147239739143e+31', '-1.73144320279140892221728899407693684286357446057721685e-18', '-1.27592813736665621853520468633947214694438394987425995e-18');
    T('-1.2588878e-17', '3.261793e+3', '-1.2588878e-17');
    T('5.43e+1', '1.052831202498586189323776187910882192514844872077623859821e+39', '54.3');
    T('4.06387835714e+8', '2.33202324535463287957487622607173811837788707819610667687974e+21', '406387835.714');
    T('5.67752073680296031467842e-19', '-8.601036147246856e-8', '5.67752073680296031467842e-19');
    T('-5.6e+0', '-1.8453950994974064478634206768933065169074111213e-6', '-9.38967160341442620882313109909923108359525446e-7');
    T('-2.401261282811e-19', '2.056364582076044663398637567011867699558230779795276645e-10', '-2.401261282811e-19');
    T('1.0071885115326611960091167358045295888291572049818e+17', '-1.155335154293012676567e+5', '25104.02532384871469245888291572049818');
    T('1.014158574325029197139040004809e+30', '1.232019764151304366892839731605883657742763048324806262665e+25', '9.1852662414944875300513403008082924671691409524768246786e+24');
    T('-1.2960509353603657966595552033959658985645676032863193e+52', '7.312148044790448624282108771e+15', '-3433711470718028.563736068134');
    T('3.01768950813493722706548378704076026752577936470035725e+18', '1.9656153202749223229990334e-3', '0.00073182798214321319862218230035725');
    T('1.138167248709526636888918903478056867722716985003746e+26', '3.6048159266517496763778284439e+18', '719311038898532672.7955886026722716985003746');
    T('1.8330798601350027076470730268735578842175174524571662455e+23', '5.956531185e+5', '15372.61868735578842175174524571662455');
    T('5.64642126873494799157e+20', '-8.4189838723261978274550066411375501139050432727008990287e+35', '564642126873494799157');
    T('-3.35038698584922285256354681261850266025e+11', '-4.971223451368832884824701430128927927628669283992608676e+11', '-335038698584.922285256354681261850266025');
    T('-2.18228628448939641477994931560863e+14', '-1.9848009280011383843320378679218954e+17', '-218228628448939.641477994931560863');
    T('-7.9673369189287490637419160450685121345328702274551959142e+1', '-4.36967584013770925112063732602257274835575300806170773838e+56', '-79.673369189287490637419160450685121345328702274551959142');
    T('-6e+0', '-1.355953074987858129012845374276873617e+9', '-6');
    T('2.8980599713949417129214483614111169203417835e+30', '2.09234393781487031432546596319751546520229578e-7', '3.5199820166059069684596128491376185333182764e-8');
    T('-1.05885215e+2', '6.96134e+3', '-105.885215');
    T('-6.63850464900027698e-11', '1.47441847556286416976794431734798800248343e+1', '-6.63850464900027698e-11');
    T('5.329e+0', '-4.2052296626044402413438748554367560246832050927e+46', '5.329');
    T('-3e+0', '6.310795365619607762092711e+14', '-3');
    T('-1e+0', '-6.151436486161577e+1', '-1');
    T('1.27372286268e+5', '1.4369236059213560786734339762024606028559549e-11', '1.1838867423641287048289062782741685583483082e-11');
    T('-2.47796406858881525393148342132e-19', '-1.114448296870693752476671e+24', '-2.47796406858881525393148342132e-19');
    T('3.31716916602e-3', '2.18973397623066249876504033115990763189329335e-11', '3.5801106985221822189595541951629188768773145e-12');
    T('3.36130377710864393490577917996361199494e+23', '-6.01033888502758381063377894374191325727e+38', '3.36130377710864393490577917996361199494e+23');
    T('-4.05078537570850079118549718e+24', '5.77875486336186078309057e+19', '-51579137737238062500.3271');
    T('-3.882925224099795993538502419426e+9', '1.77e+2', '-114.099795993538502419426');
    T('-3.1151827562342815382088011269e-4', '-9.5939986106878750496332e+22', '-0.00031151827562342815382088011269');
    T('1.263515691650061656757086438088937011975691e-9', '-7.548241773275363671025372541e+26', '1.263515691650061656757086438088937011975691e-9');
    T('-2.54581764865280637563363587278172942730649726409174798929e-16', '9.70709912979e+3', '-2.54581764865280637563363587278172942730649726409174798929e-16');
    T('4.544262038438149962548431721262064539926203e+22', '-3e+0', '0.31721262064539926203');
    T('6.7686482995324e-8', '3.57628817229923120251439846315e+11', '6.7686482995324e-8');
    T('-2.5837243515207675215858826036e+0', '4e+0', '-2.5837243515207675215858826036');
    T('3.5866287777543048262613523108599769647097333e-12', '6e+0', '3.5866287777543048262613523108599769647097333e-12');
    T('-1.12950543165722419512831984028282909551894468294664944607538e+48', '-8.8994740827559322848961144561345767240206011017e+24', '-4.3060230186943786250552209277543855025500782967e+24');
    T('1.32629615572864315471240257652101515250879349726101e+50', '1.24518406684607715300780841979783985e+7', '11819939.0138645251983859235118410325');
    T('-1.790115489391445059426395198670071179664972568459e+32', '-1.6684469764e+8', '-13018319.8379664972568459');
    T('-4.043663364e+4', '-2.201805700480933156346652411e-3', '-0.000764247759663998270288369046');
    T('1e+0', '-1.21869963007789517796039072048232e+19', '1');
    T('2.569643022111077e+3', '7.5897425087682454375918533810051138006394242e+43', '2569.643022111077');
    T('5.462e+2', '1.185649353252424966708806431224964085816087803853753518e+25', '546.2');
    T('1.77779806e+1', '1.884041077778132016302158377e-11', '3.69280262144004372920946142e-12');
    T('-4.30600314463832449294350446141214505790227492e+37', '3.62303831544018299428603870984582744317996855543826e+50', '-4.30600314463832449294350446141214505790227492e+37');
    T('1.46820318075561560819e-20', '-9.2923470300743146049252849193885672e-19', '1.46820318075561560819e-20');
    T('2.62e-8', '-2.272918110884608589603164353380779123924726e+42', '2.62e-8');
    T('-4.946872029105116445758491681281820008627914126e+12', '3.553464565566610158e+4', '-18242.079207968012581281820008627914126');
    T('-5.444799946684106145173105e+19', '3.2264562027032003440966e-1', '-0.29741218915109159338806');
    T('-6.6641862567771859738211631116152852541e-10', '4.53837850928448871996774473572127726597085345e+2', '-6.6641862567771859738211631116152852541e-10');
    T('2.04412473664652568585117378133e+21', '7.17977970071262225892475320830525724040709899422645111e+7', '6850552.6641959495615784496161192519568752280008915477');
    T('1.90389879643445938760898252e+18', '-2.820723572209116115220857118861930149808002989789986560298e+16', '14014003054351590.41100825036250679962863799684070900460034');
    T('3.54399572690074523717477041687706132576531580993088737845e+56', '3.86305546858881436338415083054757481145199924812285238142249e-8', '9.0010818560321987858218342657711149479512082543264309237906e-9');
    T('-2.81021903922014238242334e+23', '-3.81351478552287812572563406e+26', '-2.81021903922014238242334e+23');
    T('-6.6613160898e-13', '-1.35501662654877262254698379956968e+8', '-6.6613160898e-13');
    T('-1.37322505522894140472667261498601533454460065766486158656811e+56', '-6.101994414072054045611858917329e+20', '-111378121206362165144.3203824528');
    T('-7.174927106677644467861669419e+23', '7.3781216744e-6', '-0.0000029266955552');
    T('-2.6550841471735130755928352857461281e-13', '2.29146598195364764153947e+4', '-2.6550841471735130755928352857461281e-13');
    T('-1.98282995527163119617853095482026820489347564940576571365074e+44', '-6.7343972311468478371503508710588269021e+22', '-1.8425585476108166553720676776558276153e+22');
    T('3.45510770293428430743226307247e-10', '-3.453355686854751488683981e-4', '3.45510770293428430743226307247e-10');
    T('3.0940169147998267e+16', '-5.1679827115186675000881679620188097860809e-4', '0.00045494105050079175531229543905152624215823');
    T('-5.683707359160590767482385181054893368911539362771215016167e-10', '8.86164094021739567981e+19', '-5.683707359160590767482385181054893368911539362771215016167e-10');
    T('-5.1599750984803110105e+16', '-3.5042033e+8', '-313986840.105');
    T('-6.648136108552158227395470978468104077135e+30', '-7.670622e+6', '-1963500.104077135');
    T('5.7863814022935592013463113844081501644834124668e+33', '2.927394204321791269473573349413773145813098792e+20', '144279721918792318844.9709640995481667887767448');
    T('1.846283862301720586500752e+20', '-2.76680920710526658074e+14', '37882799751297.269222');
    T('-4.074907e+6', '1.2292e-10', '-1.156e-11');
    T('-2.4670327357021349034013736e+26', '-2.247694648202153e+3', '-1262.063905639966');
    T('4.5429277175861978297034645864160147622314348499084882862362e-18', '2.7386e-2', '4.5429277175861978297034645864160147622314348499084882862362e-18');
    T('2.157162452863075302863915e+18', '-3.60299993558464831757535049083884559e+35', '2157162452863075302.863915');
    T('7.7382289705506563990171491235511002471989281027800113e+22', '2.48970493035763788684e-5', '0.000011085865185482489695400113');
    T('-1.9064913834485914061878309425875120670837105e+5', '-3e+0', '-2.13834485914061878309425875120670837105');
    T('-5.48175104358179142491638230233e+25', '-1.52434523822871140847e-11', '-8.8309288545189098888e-12');
    T('1.07984359784457855e+6', '2.92533631912192887348822954567386e+32', '1079843.59784457855');
    T('-5.343344296409307514661858172266262866671109159598323e+9', '2.3273061846552018803169584402231121660442419923712e+8', '-223270690.167863377964549603775416101373776776381683');
    */

    /*
      modulo mode.

      Tests cases taken from:

        Division and Modulus for Computer Scientists, DAAN LEIJEN
        http://research.microsoft.com/en-us/um/people/daan/download/papers/divmodnote-letter.pdf

        On the Definition of Integer Division and Modulus in Programming Languages, BOYKO B. BANTCHEV
        http://www.math.bas.bg/bantchev/articles/divmod.pdf

        MSDN Math.IEEERemainder Method
        http://msdn.microsoft.com/en-us/library/system.math.ieeeremainder.aspx
    */

    // Truncated division (ROUND_DOWN)
    Decimal.modulo = 1;
    T( 8, 3, '2');
    T( 8, -3, '2');
    T( -8, 3, '-2');
    T( -8, -3, '-2');
    T( 1, 2, '1');
    T( 1, -2, '1');
    T( -1, 2, '-1');
    T( -1, -2, '-1');
    T( 34, 5, '4');
    T( 34, -5, '4');
    T( -34, 5, '-4');
    T( -34, -5, '-4');

    // Floored division (ROUND_FLOOR).
    Decimal.modulo = 3;
    T( 8, 3, '2');
    T( 8, -3, '-1');
    T( -8, 3, '1');
    T( -8, -3, '-2');
    T( 1, 2, '1');
    T( 1, -2, '-1');
    T( -1, 2, '1');
    T( -1, -2, '-1');
    T( 34, 5, '4');
    T( 34, -5, '-1');
    T( -34, 5, '1');
    T( -34, -5, '-4');

    // Euclidean division (ROUND_EUCLID).
    Decimal.modulo = 9;
    T( 8, 3, '2');
    T( 8, -3, '2');
    T( -8, 3, '1');
    T( -8, -3, '1');
    T( 1, 2, '1');
    T( 1, -2, '1');
    T( -1, 2, '1');
    T( -1, -2, '1');
    T( 34, 5, '4');
    T( 34, -5, '4');
    T( -34, 5, '1');
    T( -34, -5, '1');

    // IEEE 754 Remainder division (ROUND_HALF_EVEN).
    Decimal.modulo = 6;
    T( 3, 2, '-1');
    T( 4, 2, '0');
    T( 10, 3, '1');
    T( 11, 3, '-1');
    T( 27, 4, '-1');
    T( 28, 5, '-2');
    T( 17.8, 4, '1.8');
    T( 17.8, 4.1, '1.4');
    T( -16.3, 4.1, '0.1');
    T( 17.8, -4.1, '1.4');
    T( -17.8, -4.1, '-1.4');

    Decimal.modulo = 1;
    Decimal.toExpNeg = Decimal.toExpPos = 0;

    T('3.9847101340111444985551969110192e+78', '7.8192896077816402203429752892402304171e+72', '1.4988562064e+71', 11, 5);
    T('7.525177596134e+684', '9.8282164940216043e+83', '4.0495064397079841e+83', 77, 4);
    T('-4.0776540585024468960950672e+52', '31.326384321498963', '-1.5347871676038824e+1', 87, 1);
    T('2.71930227384417086149645043730044630193149600483263494737486e+5602', '-1.5679497132184942340987699514207303586000567e+8874', '2.71930227384417086149645043730044630193e+5602', 39, 5);
    T('8303854205.02459312987785248352547099465', '1.093198330e+577', '8.3038542050245931299e+9', 20, 5);
    T('8.0001410053879503556e+480', '45855.4087134820866475281492', '2.8534546640177323518511204e+3', 52, 2);
    T('-8.4143180366128953e+145', '5.484961037563025783988828591606299899658292180e+6729', '-8.4143180366128953e+145', 47, 0);
    T('2.0385885049008867807551855593307241270718650568e+9381', '7.88626400756054891e+734', '3.05023338950046255e+734', 89, 1);
    T('493.910164', '-6.91186703e+39', '4.93910164e+2', 85, 5);
    T('3.82446944891958e+35', '-4.2591421710168083244703618e+317', '3.82446944891958e+35', 55, 4);
    T('-3.32423079145230309263e+28', '796082.0391713561349', '-3.304057638177109877e+5', 35, 2);
    T('5.4371445345075505921247e+2228', '-8.34028281731454e+9678', '5.4371445345075505921247e+2228', 71, 1);
    T('8.29124936e+33', '6.7609518883535244356511089464911e+201', '8.29124936e+33', 76, 3);
    T('7.55955412775119275923171522928771771146018198097e+9904', '6.1230009142557473e+8525', '5.4499366041006607e+8525', 100, 4);
    T('4.3e+8', '-6.63964497065158444559565682740', '5.86746670364528e+0', 16, 1);
    T('-206.667872315417902655020705624311923225075195283', '-2.9428323e+703', '-2.07e+2', 3, 0);
    T('3.242906955360179886676058217376058792050640032439850e+79', '2.55702454128809442491569791115358e+3707', '3.24290695536017988667605821737605879205064003243985e+79', 64, 1);
    T('7.04533566041775496810200390121e+55', '9.7e+9288', '7e+55', 2, 4);
    T('1.2283833371710016443094754693819533752473615103864976971682e+87', '-7.9539796354992182320204496381064897933789159905880e+2755', '1.2283833371710016443094754693819533752473615103864976971682e+87', 85, 3);
    T('1.06681376023792587718921924508802027094581997963923e+7042', '924.4752248414322581640910784897', '9.138549365005626212976072195308e+2', 40, 2);
    T('8.3431045846e+96', '2.537748221924e+8554', '8e+96', 1, 3);
    T('1.8335886987947729654846e+648', '-9.737580128147257732044466499707664e+7385', '1.8335886987947729654846e+648', 54, 3);
    T('-1.083567926965528919464814502e+881', '-607.7738780243637322184519635873792582844810571261819985577', '-8.51152114253900963676970954676597351708929413602308949567e+1', 57, 3);
    T('-67312.21690037257045489100533209997575527372377871508681135', '-245080155.32013063590796829955679869654599410909654384834011', '-6.731221690037e+4', 13, 5);
    T('-4427.1631819611458760470399505729984433679946985875', '-9.90079108970898e+3277', '-4.427163181961146e+3', 16, 5);
    T('-9.572415449189682338551653754979758788e+1994', '-8.617389918923766749658464387197050801437e+51', '-5.155653306522656671427709409035320199782e+51', 62, 1);
    T('3.154840529083e+9880', '3.5591e+7', '1.0983e+7', 79, 4);
    T('6228237785422315.625', '8.39917805272734833417459e+369', '6.228237785422315625e+15', 61, 0);
    T('4.837600073778365259999679342e+9301', '-7.92600731209449e+57', '5.73548546545159e+57', 75, 0);
    T('41117.32951857892643897902149171702178340468646600818804', '-3.4224278757475940427355378668503e+310', '4.1117329518578926438979021491717022e+4', 35, 0);
    T('1.677271719646878456459513536505313983477493243826662210e+70', '-2.27887027593248863362e+5259', '1.67727171964687845645951353650531398347749324382666221e+70', 71, 1);
    T('-850608.502805556595006084', '-376849800.44', '-8.50608502805556595006084e+5', 50, 1);
    T('-3.558596223259585015570049730322940927429129337616785e+72', '-8055879543.94755940485715470561026941232', '-3.26538054178831996679606522921469336192e+9', 67, 3);
    T('-1.82904608596508352987201881596384400593088676642652', '-2.037051036112430409092459648707472', '-1.82904608596508352987201881596384400593088676642652e+0', 89, 5);
    T('5e+6772', '-1.0795e+77', '6e+76', 1, 4);
    T('8.843107515773050528628460543610812e+92', '-381.375612164440011792563083290', '1.5710347905313548211608203937e+2', 70, 3);
    T('20.5830712308417185461515196898618716352956536012486', '-2387445778644155.761135880926553589279332515014936744', '2.05830712308417185461515e+1', 24, 4);
    T('-4.27188441265746521329187924127183637001255195129134290e+2215', '8.8304786055388555295467e+270', '-7.3294432433723278928711e+270', 80, 4);
    T('81581997673.87', '9.4622946321538132008419591917679399891e+43', '8e+10', 1, 3);
    T('-4629.3011067053235186696142837855166278', '-6.1177301124899485601201808342971719e+2605', '-4.6294e+3', 5, 3);
    T('-1.1594895222385e+269', '-9.07121164e+5020', '-1.1594895222385e+269', 63, 2);
    T('-16.138765615754288839596362767624209', '-31995440621713029932767772.3821734766703201059', '-1.6138765615754288839596362767624209e+1', 77, 0);
    T('3.46263931222354027167806e+25', '5.0299483992527626e+5811', '3.462639313e+25', 10, 2);
    T('5993909.9219841745142973293469', '3.96740173708500010292977261e+949', '5.993909921984174514297329347e+6', 28, 5);
    T('5.4502702286837194284952766340046172646403261103189e+123', '-2.227657814268e+649', '5.4502702286837194284952766340046172646403261103189e+123', 60, 6);
    T('9.0396375793e+5137', '3.4097545730521345334829260714850344717568e+654', '2.76957098463153881258457e+654', 24, 1);
    T('-5.07105888881675273507373149079614466873664815401e+87', '-5.422e+6526', '-5.07105888881675273507374e+87', 24, 0);
    T('-122592682.3942695650173', '5.257552869286739639542185205148628579075275543773559441706e+3377', '-1.225926823942695650173e+8', 91, 4);
    T('-7.475580688464186996424599020e+78', '-9.43245262e+71', '-7.2417884099642459902e+71', 20, 3);
    T('-51.905', '8.59896540666572e+6507', '-5.1905e+1', 82, 4);
    T('1.833137344354726499448371957820423749710298688112685060e+693', '4.314650571634436640253604655886262710407345584373284e+93', '1.449120951121495959938552517587387362898328955355888e+93', 53, 5);
    T('2.081527574674059636672543372353767033341499e+377', '-5.459987420960437060568005649142739158665572034934e+57', '3.320231612254255042611100913997352399710582149926e+57', 89, 5);
    T('-64.29', '2825576403', '-6.429e+1', 55, 0);
    T('2.4242221075827e+4348', '4.383469842679135247011083626e+4375', '2.4242221075827e+4348', 14, 1);
    T('-7.149641377300407836281722897604786e+206', '-4.13580764788135658745e+909', '-7.1496413773005e+206', 14, 0);
    T('17260515.854028342668860269074771105598897034792', '1.064582006339004e+4110', '1.726051585402834266886026907477111e+7', 34, 0);
    T('4.2e+7818', '8.4767511827622219325396266905080240984180e+949', '5.5109108743801863911038782397136051e+948', 35, 0);
    T('417199865783.2299625136736348', '6.493824256987208864660630730e+205', '4.1719986578e+11', 11, 5);
    T('8.9485767778089694219405214e+912', '-4.08955715581486538663332188550623260621622e+65', '2.38940960278902552006800381827499237801e+63', 87, 6);
    T('6e+6079', '-5.8689968987253458916483750883309869e+276', '1.3207635301974613113289688114779815e+276', 82, 4);
    T('-8.3088e+114', '-2.290326038329693228582975013540168737293953958049912212319e+98', '-8.9e+96', 2, 5);
    T('8.67425688097083249392354379266625659e+646', '-2.200269887985941526305e+2370', '8.6742569e+646', 8, 2);
    T('-4.9906960320055925119e+850', '7.243820529113492161025615422915957784e+6266', '-4.9906960320055925119e+850', 96, 6);
    T('76520047.6240536818222442336998', '1.0805262e+943', '7.6520047624054e+7', 14, 0);
    T('8.70099260642759419518300843309587159975506542956711278361e+286', '9e+313', '8.70099260642759419518300843309e+286', 30, 1);
    T('-1.5591335500112068239773747738e+4002', '9847760445.9599124780112131934', '-6.9444710759003167424521302466e+9', 67, 4);
    T('-2.63295098716360685167519e+51', '-4.4123177873658441822920e+58', '-2.63295098716360685167519e+51', 50, 0);
    T('9.11295969059849131536079890990450811415666837310e+1907', '5012.35007064613885771952303924823', '1.6692533182869144750481072715067e+3', 67, 3);
    T('887452261982800800902198.4', '4.76279217896401501688730295687581e+1053', '8.874522619828008009021984e+23', 90, 2);
    T('-9.104953060325637284793759413994e+36', '6.645329252610016955946671029213327078185623109732787057e+9579', '-9.104953060325637284793759413994e+36', 53, 5);
    T('-3.4549396595948278e+8866', '7.07326052419113125891e+98', '-4.82534999204336326003e+98', 57, 6);
    T('-1.22770207085763493420987722090708110e+9221', '-1.35e+6600', '-5e+6598', 19, 0);
    T('2.858134366247697e+41', '8.7e+86', '2.858134366247697e+41', 77, 4);
    T('2.18611886867318189289452064510495989408379e+9686', '-7.2e+585', '2.4e+585', 45, 5);
    T('-1.80002364557882664469297583139520382510e+937', '2.37647776941624243754791115891641470681647070121286388e+350', '-3.07470977683802e+349', 15, 2);
    T('-3.2538083578459e+73', '-4.71913e+8', '-1.92526e+8', 69, 6);
    T('3.924089946304767195184227530345111408633264844181580785895e+5616', '-7.40058323983e+58', '5.81423621839e+58', 80, 4);
    T('-5.48219143236772548776159952014e+286', '-8.3644235367556486897517612310794391288176030e+278', '-2.87930020264035128081e+278', 21, 0);
    T('71.14347024238678981976', '784.6069', '7.114347024238678981976e+1', 80, 0);
    T('2.72321029251967181533934974135155198271992468839280144787e+2982', '-7.3846036032238481044580e+42', '3.69406263551196082571e+42', 23, 3);
    T('-40.86378602030270807679', '-9.84826914215619909882936933e+315', '-4.08637860203027080768e+1', 21, 5);
    T('-7.4719042432151333893608959960029528e+752', '-8.757270691774586981377188009624067046859e+967', '-7.4719042432151333893608959960029528e+752', 93, 5);
    T('9.8651928976e+611', '8.7770155819451855451575e+401', '6.3535618611874031388575e+401', 77, 4);
    T('713155.439948763187666370125425043185904735569022', '6.259514198069184058810345998082789e+177', '7.13155439948763187666370125425043185904735569022e+5', 58, 1);
    T('-8.55403716e+30', '1.364587219387831687323140375364e+477', '-8.55403716e+30', 81, 6);
    T('4.22089194381848790020304145602683935334786786e+6376', '4.370747864816498718396639337408630117223819400e+54', '3.9651091800442493827697474121604289443097342e+54', 81, 2);
    T('6639398376746993403183876364532055.8787495990228463', '9.149881e+8848', '6.6393983767469934031838763645320558787495990228463e+33', 90, 6);

    /*
    T('4.9254167792876314829598776810492640969038884898e+632', '7.5885681381282664049434053461660190e+890', '4.9254167792876314829598776810492640969038884898e+632', 48, 0);
    T('-5.1784467779290e+18', '2.06349797433e+2095', '-5.178446777929e+18', 19, 4);
    T('-6.78167483574797e+66', '-9.6821211638097e+4477', '-6.78167483574797e+66', 99, 0);
    T('2135736.866630842881874926736', '5.384e+340', '2.135736866630842881874926736e+6', 95, 6);
    T('-975260489.636160588176', '-4572.96555490450831683290204793453156369161960', '-2.4176038957174783143118451941915377430543864e+3', 47, 4);
    T('-1.31715030e+4710', '-9.2826561152399555085575717241294294786732955e+68', '-1.1004455130905348621304926e+68', 26, 4);
    T('-2.54715824007911e+97', '7.700214609478564313386800318179e+50', '-1.487204713182969508e+50', 20, 4);
    T('-9.11796217700378617635255542811706743584012547e+3743', '-6.63215805095519054102850749290155951237569406e+59', '-4.305822993e+59', 10, 0);
    T('-4.940740e+4241', '330.028345905748', '-1.1894842608684e+2', 19, 4);
    T('8.6209842732644893276627023243951762931889308662159448189750e+164', '-4.277720383990127366636096895209e+9705', '8.620984273264489327662702324396e+164', 31, 0);
    T('4.612004725e+2926', '9.517569373970372689522216493090594557389529933e+61', '2.625277465563984283242331e+60', 25, 5);
    T('7.808825567985633363296145e+67', '5.7266843861902471992246377527e+939', '7.808825567985633363296145e+67', 59, 5);
    T('-780284.973674803921675091', '9.158108e+5239', '-7.80284973674803921675091e+5', 89, 2);
    T('2.0903821960004e+7748', '-2307.3658099419978562698145562344', '1.9954392283671738510180554010144e+3', 36, 0);
    T('-69518.00973916124095182744871962022418660998', '-5.7549394928616348111669210595e+1134', '-7e+4', 1, 4);
    T('4.3378140889518888233483998858366176600684498839785907e+540', '8.245007810935209417729582026787247e+428', '4.495124448147519087145786221117372e+428', 67, 2);
    T('-1.72637070446340460759717600e+97', '-26846404926619.7132427616148940135466260923385719756', '-8.4728858518563955933597177e+12', 26, 2);
    T('672.2552000332825362230589215088907352', '5.9798307406825303773788221674694125045791e+1911', '6.722552000332825e+2', 16, 4);
    T('-8.5604243406374052872092355160471768042287317759015922e+417', '741463.27831689503305463097660167', '-3.749218122565160508770923808179e+4', 62, 3);
    T('-690516223.741849424571997946354169176', '-2.508979157450663729048459215e+6078', '-6.905162237418494245719979463541691e+8', 34, 2);
    T('-2.42917289890552192212935228554515976939662896343990e+590', '3.1373351e+28', '-1.4898281e+28', 28, 1);
    T('-2.278111953536253983346e+780', '189.555169605839261500762', '-1.23641316308833467735392e+2', 40, 2);
    T('5.427467276704350302450908396192566e+2523', '-9.661236e+13', '8e+13', 1, 2);
    T('9630617.78317038456989911509', '-3185357.27272167986244380769644820193063273142110596858025604', '7.454596500534498256769200065539420810180573668209425923188e+4', 65, 3);
    T('5.36258596263870840820688390751533869198728439792e+697', '1.602593266319689039816816409e+53', '6.48983249030372577070881106e+52', 87, 1);
    T('-3.74960547664158e+1154', '-6430868.347480558615067623183435936093684187472297370', '-3.908974e+6', 7, 1);
    T('-2.1231040976453e+6864', '8.56945003887503434601048084809103382719272146e+83', '-1.78504215148946579233873941516186124247059484e+83', 56, 5);
    T('-9.44353303730721506532273079833250e+1523', '-3184147.33848479138486385777417706250697363544318400411872', '-5.8924986020420924303380742218719828594649338241551746336e+5', 85, 4);
    T('-3.7615838463e+88', '4.83852094757e+45', '-3.2010303211e+45', 96, 5);
    T('9.9658625785039378427579578066984914530073420e+5915', '9.877013162168e+1821', '8.152856482096e+1821', 54, 5);
    T('-8.83161734500341610194315972922562661170843931452194621414e+682', '-2.84391434520373313615635937e+501', '-1.079537566e+501', 10, 1);
    T('1.78e+53', '9.459994345005154065726263446703644293423365241969711', '9.320628621716059722e+0', 19, 3);
    T('29929.37514786207249118016346191285729546504837507716', '-6.906456692371765666670070109969101e+795', '2.99293751478620724911801635e+4', 27, 4);
    T('18346216799503685372270372.593264', '5.44220553714422522656072186477189905330765172566505e+9530', '1.8346216799503685372270372593264e+25', 69, 1);
    T('-2090985.512838661507575616724825174', '-6.6471404625663888165978837406075354068e+1642', '-2.090985512838661507575616724825174e+6', 82, 1);
    T('-8.969563508604323411e+4344', '7.3770407320788935067316401685235204570763040237192229e+196', '-4.9e+196', 3, 1);
    T('-4.57298843826e+1530', '158683.71418', '-1.670532772e+4', 66, 4);
    T('9.5989480978e+91', '4.027662940075627121696845762678499848947543527045e+461', '9.5989480978e+91', 25, 6);
    T('-1.649133373732987511720174485113133143814157203137541e+693', '-1e+3712', '-1.649133374e+693', 10, 5);
    T('3.85306438824783590371059707069e+329', '1.801222e+84', '3.25642e+83', 82, 5);
    T('9.240295693915169879193e+38', '-4.913579469684156183274042852487949401543769299764014e+7593', '9.240295693915169879193e+38', 60, 1);
    T('-2.8023218008768736509991829412146459761e+1377', '7.7663208185819812826618816050230454866957960821563766455e+992', '-5.9929778979566e+992', 14, 2);
    T('-314.0906975743600809978743', '-254.6211843711580934082457538609943500272226471725716490', '-5.94695132e+1', 10, 5);
    T('57901.154781323841734192114731222107511385891072099470924', '-249204.568398044380635182325427266042440188008688329113839', '5.7901154781323841734192114731222107511385891072099470924e+4', 92, 0);
    T('7.4292838273827917924707983425978756049508090888200137215476e+472', '950900754.2732441766949300349227026806999', '7.4165647121657147464550589350888e+8', 32, 6);
    T('89814.1668297737485417', '-7.5e+194', '8.98141668297737485417e+4', 25, 6);
    T('7.7477562659511042561877945224800865e+732', '-1.2e+77', '0e+0', 34, 4);
    T('8.11506750808496011e+56', '-9.9331564147976238448889050402500565408482251898165611805215e+59', '8e+56', 1, 3);
    T('2.466e+2717', '1.84957415147694450e+3776', '2.466e+2717', 88, 2);
    T('-8.69780004128432e+209', '-2.4660625379603037658663261612676218014755485969890132595e+929', '-8.69780004128432e+209', 77, 3);
    T('-3.8277e+71', '-493434202.644167958255238121791482031075353', '-4.0245373513464265709305384241469749e+8', 35, 4);
    T('699511902.114149259430911151933243', '-30552.2856817777176926176569283941028293144870077596111', '1.73214298484128584298965576600157228448199573437038655e+4', 86, 0);
    T('-4e+30', '2.26777852100803693390794669560965779572174524e+7543', '-4e+30', 99, 1);
    T('-6.7240280150920372997e+427', '-5.419039322193226e+57', '-4.012670003370122e+57', 57, 0);
    T('-3.528591869222556312632032140289694236502558600692218', '7.59821234e+4422', '-3.528591869222556312632032140289694236502558600692218e+0', 95, 5);
    T('7.00119840449835764608982172892898005e+813', '4166.18885540446865287038843963071632733307461982152', '1.48228506188519149247510364010266000272058916558152e+3', 83, 6);
    T('694624.06893847566028683793058696956081174836264743702370', '2.023141145621431375030718734017146197449448373641313757e+605', '7e+5', 1, 6);
    T('-4.15788926715003e+25', '5.96123016232582264e+2928', '-4.1578893e+25', 8, 0);
    T('3.12885167504678983205465459391290590e+4591', '-5.5696373464370707908096451869715030461266550795319e+8845', '3.1288516750467898320546545939129059e+4591', 49, 1);
    T('-4.241821813324358348818574742049374466151748e+113', '-3630547', '-3.61269e+5', 79, 4);
    T('-8.8718540311210271215017483377843181645639792944370730759939e+2058', '-5.7997657676768599876644003261054e+2609', '-8.871854031121027121501748337784318164563979295e+2058', 46, 0);
    T('7.46e+6957', '3.00480305e+4285', '2.1249555e+4285', 61, 4);
    T('9.569210554815480620e+666', '-2.35267763379989', '9.4936008565059e-1', 67, 6);
    T('6507577.55571265483508936784455906084331674400215945433255291', '-5.06375423465352286581622517819606622035749930059e+851', '6.50757755571265483508936784455906084331674400215945433255291e+6', 71, 6);
    T('1.03989931314050876861123351622706e+4777', '-6.4486150676e+714', '3.7597142788e+714', 53, 2);
    T('-8.8760617672227922486e+99', '87.0341265257139388970449020849211180319876255872', '-1.568100101095567109944119125e+1', 28, 2);
    T('-1.37713103659439e+71', '-2.6e+4404', '-1.37713103659439e+71', 42, 1);
    T('3.050e+9', '8.336467e+7', '4.887188e+7', 9, 1);
    T('958864.431316211416017316', '7.10438395048908672589491119806875e+852', '9.58864431316211416017316e+5', 80, 1);
    T('4.855695311648320315426620447492557962352e+310', '7.52830227030392155240608578179256882870746610672559885600005e+74', '2.38508537985038045411854369858200982808022415152228704337965e+74', 85, 1);
    T('1115284404651699436156291276662587233193280656293', '-7.9902183229682188133520164615757249973639138579462174e+1184', '1.115284404651699436156291276662587233193280656293e+48', 90, 6);
    T('3930736125252684338899452.362842594851494272768081295382', '19998.000904074001247', '1.6504323034033847549272768081295382e+4', 98, 3);
    T('-5.8e+1925', '6.0821255577239423885180502e+8364', '-5.8e+1925', 48, 3);
    T('-5.02015380396963447930e+8219', '43642245.621709682861509775535679291436591551895827', '-2.4e+7', 2, 5);
    T('4821463175811674886883650785030939240679647154.323840593', '-6.130407265137568498965353108799473150709e+945', '4.821463175811674886883650785030939240679647154323840593e+45', 69, 3);
    T('1.6219863971099254566358002027827613983481383045228675650715', '-3.37e+49', '1.6219863971099254566358002e+0', 26, 3);
    T('7.8e+32', '-7368.29648', '4.01511632e+3', 74, 3);
    T('13.40272474', '-1.68662117059415474422999842732913683969543230e+321', '1.340272474e+1', 68, 2);
    T('-65.299', '1581861.1640141997633090', '-6.5299e+1', 76, 4);
    T('-7.604286086060400392558793216597868934938', '2.849654073042214381281182336124418981022120725465e+173', '-7.6042860860604003925587932165979e+0', 32, 0);
    T('495.17783361209012863695631111165945595288797262793297503', '6.96445374063e+222', '4.951778e+2', 7, 4);
    T('-3.87065001765185182599146e+33', '4.66435251822888489615128422182235851615476e+70', '-3.8706500176518518e+33', 17, 4);
    T('-2e+212', '-6.79426558089353774e+402', '-2e+212', 7, 4);
    T('-3.545753755627451048573767097381654e+53', '-9.878000046439459045588929938319431795e+186', '-3.545753755627451048573767097381654e+53', 89, 2);
    T('-1.177375433404596674700904e+840', '4911942486249188566274597666564141.458715312263', '-3.271345670615495072220539558e+32', 30, 4);
    T('80743844663469685738831.172905862977167911990', '-8.3558059911296253179570256101993929e+86', '8.074384466346968573883117290586297716791199e+22', 97, 1);
    T('728341749375212997823648415222245062055227673381037.413', '-1.5750e+8', '1.3381037413e+7', 33, 1);
    T('-672.89952', '3.6019682e+797', '-6.728996e+2', 7, 3);
    T('94226.7297807293543753391591160026057097673893410352757394', '-3.8476229e+458', '9.4226729780729e+4', 14, 1);
    T('-9569.417213350133206243107042593313227', '-3.24466498211045961647388827308925247991611370664645896e+5472', '-9.56941721336e+3', 12, 3);
    T('-7611629.529113480699308284587985331399620196', '4.123236e+809', '-7.6116295291134806993082845879853313996e+6', 38, 2);
    T('-2716214464.7158754977515123372004519', '7.9311226664467828530834364221e+311', '-2.7162144647158754977515123372004519e+9', 36, 6);
    T('-6.9815630179461929e+82', '6.89141e+6', '-3.52904e+6', 9, 0);
    T('-2.773175721720055e+451', '148699292.663607959003946414517050428956669', '-8.25945916829714e+6', 15, 2);
    T('-4.4956749043493659662864147380165986095251871581576e+427', '-7.846609062675402140867538605876377539832683681762e+88', '-6.59185408933659013143931678082854933332770856041e+88', 82, 4);
    T('-836423621.22159355623611431532629584', '-77575.12666322093402209049817420262508527589746847842', '-8.6055387454e+3', 11, 4);
    T('-7.92980740449193905e+9141', '2.0805854716554303242091831830344446125718642537315171401e+6985', '-2.91e+6984', 3, 2);
    T('-1.3048932574915980142686157425434131e+3515', '3.0723149182404754727080596686392335049514698772445e+7686', '-1.3048932574915980142686157425434131e+3515', 57, 2);
    T('-7.9557394217436968295246633505733887636250e+2648', '-5.8546561077e+16', '-5.4990130712e+16', 71, 6);
    T('-6.1912817491519862666175e+195', '6.684105119771569599988059670060e+611', '-6.19128e+195', 6, 1);
    T('-345268.1426803209464', '-4683302814753238028392427954.4', '-3.452681426803209464e+5', 81, 2);
    T('2971282.79592', '8.84230900005751251892082368075926975964398862698739095738e+139', '2.97128279592e+6', 47, 4);
    T('-4.854848732055778604765422186233717432448375e+9241', '-9.8534564645490875513651103074079e+54', '-7.7263622791243646709093602022676e+54', 43, 3);
    T('8.1860130539084888521756e+82', '6.02308955991420165807192788315818720961729714130841e+8332', '8.1860130539084888521756e+82', 23, 5);
    T('-685596.2381100485198536385909780906478720763887108781482631', '-7.2341733043127310529060989887346623246646429921155389441e+8994', '-6.855962381100485198536385909780906478720763887108781482631e+5', 77, 5);
    T('-2276690.89', '487049669.83576077166574738496414644744625', '-2.276691e+6', 7, 6);
    T('-9.10778893581094367423923649351844e+53', '-6.18e+19', '-3.84e+19', 40, 2);
    T('766.6948803699823737948', '-7.18135578379237602589200906355131e+246', '7.666948803699823737948e+2', 45, 1);
    T('8.06545508503457855728200656117806e+1459', '3.08901422749997692718687782594e+1092', '2.4e+1091', 2, 5);
    T('-3.47035614244312970051818531265692746496783618156742016e+9722', '4.6931101729433517513e+466', '-1.52306081396992347e+466', 18, 3);
    T('9.558255147720e+7488', '-777417.20385', '6.559736928e+5', 22, 5);
    T('-2e+3', '-8.946257802673404785596610610100297666839671695e+367', '-2e+3', 35, 5);
    T('605175769.8483266570974580798995139344022', '-24087.42797952993', '3.2292906166957774580798995139344022e+3', 75, 5);
    T('-7e+7389', '-8.235705e+9866', '-7e+7389', 91, 6);
    T('-8.08247298911560975713831068790754', '-811153159.1425684918573118853563598856031', '-8.0825e+0', 5, 5);
    T('-8.22301953595929356898196554568461342122176821653778285e+94', '469012.190043013738389564', '-1.91476044124637761e+5', 18, 3);
    T('2.02900386460496069434931318293261051936838720713142967299e+864', '7.20496395278944646463306734443423492930e+52', '1.193865332036150307533e+52', 22, 3);
    T('2.403099531055951783393759923979267550717402734343280764e+1376', '-3.5802597959007252104721e+2321', '2.403099531055951783393759923979267550717402734343280764e+1376', 94, 1);
    T('2.161623521086113801846282565768350860e+416', '-9.5362682589021213440998254e+2266', '2.16162352108611380184628256576835086e+416', 69, 0);
    T('407507513.290697', '-918274514103071350435326080359879194291.1142094', '4.07507513290697e+8', 60, 5);
    T('-52907665496055378614523738056238213829917413067.152267958450', '5.323347659933062007753631275008102e+738', '-5.290766e+46', 7, 2);
    T('6.762674721320242715073836634433315600610050366538076308188e+84', '-6.650864424774062509323195228036684565194426167369459e+110', '6.762674721321e+84', 13, 2);
    T('-1.3015e+982', '-235352.4862', '-1.211815156e+5', 89, 3);
    T('-22148340.0461945795', '-3653.431148354334471273060935307768460693', '-1.240424870603935142704610164307591279034e+3', 80, 3);
    T('147733.2707436529210942817266021', '-4.30762332639620645982441517e+78', '1.477332707436529210942817266021e+5', 72, 4);
    T('-1.8e+557', '17.857838223', '-7.310749266e+0', 18, 0);
    T('-8.295915715438786913837614131288900084512194312237760854927e+68', '-6.93889411e+323', '-8.2959157154387869138376141312889000845121943122377609e+68', 53, 6);
    T('-5.6542840179127639633839016823834260025774e+602', '1.859464358661841117564476209187453489e+8647', '-5.6542840179127639633839016823834260025774e+602', 53, 5);
    T('-86828103.111180155821289573919628863832952447894430071190672', '4.816001273073285587910923618e+7783', '-8.68281031111801558212e+7', 21, 2);
    T('-4.49122300736607331775274718052298409849184264844952105315824e+86', '7162028099115322011021694069015513648611049428', '-3.14457642707882818999647e+45', 24, 0);
    T('-9614308.2504324976423488058170623073372999260217999948208', '-9.6174419137540776265978347669654277171532817e+944', '-9.6143082504324976423488058170623073372999261e+6', 44, 0);
    T('-107932.03430697724816641', '-9.04428e+8477', '-1.0793203430697724816641e+5', 91, 2);
    T('7.6476313603975060138785314669508411718832626519784252028544e+5060', '-6.853715482399450636379342986057e+974', '2.372600058676057565507962429227e+974', 60, 4);
    T('2034.124238045736077548577288103', '2.208664110032118901029299167806624657987735662e+612', '2.034124238045736077548577288103e+3', 41, 2);
    T('-9.60358402179887892060439505501354239470800560262489e+4958', '1.896639122e+949', '-1.893659534e+949', 21, 0);
    T('6.1919715738606e+86', '2.7212249485003087075263961120238349408828416690340596468662e+491', '6.1919715739e+86', 11, 5);
    T('-1.7188050144396485181865782187827237636e+6398', '8.613538964812730184077241999524e+3990', '-7.51489593426045887092463877892e+3990', 55, 5);
    T('-7.017629475988223876277828609082e+2676', '912537.931714', '-1.79282004408e+5', 70, 4);
    T('-7.441993591474566807833465630712031096971529617e+2287', '-8.4596118417364808015082693831635415175654220010197e+545', '-5.3175591130534910758133638674326111119059932962459e+545', 76, 2);
    T('169335312.1650239673217510620433012', '3924842.994545910232', '5.670633995498273457510620433012e+5', 40, 5);
    T('1.546548284956275932111156218007313e+715', '2.1997369596623e+72', '1.266140536766e+71', 16, 0);
    T('-8.08876500090840914179810836518247029085457263896283684127184e+222', '9.8150311291600796104996750843034743221e+6571', '-8.08876500090840914179810836518247029085457263896283684127184e+222', 63, 6);
    T('-7.4e+40', '-555575324560347382011204.32706972112385', '-2.8957291682600115076128412705363939e+23', 35, 1);
    T('-3.0248949720e+8287', '-39.4532933499557348916047806463873068700447205338051621887', '-2.6496186002834282381149353419931308306706996676649348649e+1', 100, 4);
    T('6.246100e+72', '3.818440886289699568178242', '1.285929412228940129479754e+0', 40, 2);
    T('-6.3163477070002341310626459447833545529864651059600398754e+212', '5.4564289145125612517944741734351322608478480544e+2699', '-6.3163477070002341310626459447833545529864651059600398754e+212', 90, 1);
    T('2.8991701149009350607e+76', '-512669766.5211412951615766528448157916011527608979', '2.2534e+8', 5, 4);
    T('-1.892306028635e+13', '-1.091311226059e+613', '-1.892306028635e+13', 23, 6);
    T('5.634e+91', '-2.67285537331127009e+5360', '5.634e+91', 82, 1);
    T('-6104.280245565374147089651095346871122932036797148', '2.8094745143949244852290510147612986360280157237347130724262e+3187', '-6.104280245565374147089651095346871122932036797148e+3', 80, 5);
    T('-6.156965e+6436', '-5.258007528303507160680646252e+1420', '-5.10922203265289138304386456e+1419', 40, 3);
    T('-3.1e+4004', '2.4304855172649145297578727642472509621922920928166e+2440', '-4.2729719981399575140633587605334611e+2439', 35, 5);
    T('73045.48190646281937', '9.781502725754e+8053', '7.304548190646281937e+4', 19, 3);
    T('1.5037508675622994374489868765870e+833', '4.7883215585250184067882847016956545413616712539853458765142e+395', '3.30631039209756133439519834796647e+395', 33, 1);
    T('2e+428', '5.05350075805578476852029404603535e+584', '2e+428', 23, 4);
    T('15749537.05680', '-8.2292922890123890433700680255061978e+348', '1.57495370568e+7', 41, 6);
    T('2.577393070090096195775375351254049678141e+7762', '-5060.0061583', '1.7873283782e+3', 81, 0);
    T('-1.73955886901197333330379700456711304319030354340792e+52', '4.17524226948922088302514581983710201086232e+61', '-1.7395588690119733333e+52', 20, 1);
    T('5.165191608120423245633e+961', '-55327.57603492711161377122', '5.259749595519183325199794e+4', 87, 0);
    T('5.704579980340829219984543922800034526551576889', '9.579504728463e+9877', '5.704579980340829219984543922800034526551576889e+0', 81, 3);
    T('26818515158.130191370713038228786320807357403153016', '-3.57699205395138550223107922687661826326981286712728876915e+761', '2.681851515814e+10', 13, 0);
    T('-9.612782', '7.32926114067164466781603011786540012296e+653', '-9.612782e+0', 76, 6);
    T('5.721337441e+17', '7.318582351516791347506658305972e+8397', '5.721337441e+17', 14, 2);
    T('-9881818215557590325624654203660950602.18903292304240527539386', '9.992480592128574321855341648e+6249', '-9.88181821555759032562465420366095060218903292304240527539386e+36', 71, 3);
    T('-6.242783863263198895516738921499826984e+65', '133.2501222650008385040313196734886', '-2.3e+1', 2, 1);
    T('-4.926757115486539793925145529747664857775e+568', '-482344298.35238808', '-4.584723483799944e+7', 38, 0);
    T('-5.08175756194490746870404e+51', '-189.219904866378', '-8.19680135088e-1', 61, 1);
    T('-75859320.38897856882052419099364206', '8.4772754054906309244903365764974294e+6886', '-7.585932038897856882052419099364206e+7', 61, 5);
    T('-9.50464488558206413961414188325093579013353865301527138931e+510', '2.66016180092281486473878810072782744032652230864e+887', '-9.50464488558206413961415e+510', 24, 3);
    T('1.2432327919027963578722600078923242e+269', '-3e+4122', '1.2432327919027963e+269', 17, 1);
    T('6.2262291337163718845139662444940928673996414930288578e+787', '-7.4034802371859253768306530861159253075701e+965', '6.2262291337163718845139662444940928673996414930288578e+787', 80, 2);
    T('-3.722292940e+70', '5.4567272010008880117045671131362774879447599923223785e+594', '-3.72229294e+70', 71, 4);
    T('7.23286384923630794e+1210', '9.367609028452208e+947', '7.0295299198756e+947', 36, 6);
    T('113155413732561850.337709417515302086792016669925477612', '7928762436.080748767391197067740584947658', '1.338411083496398274651071651305666046345477612e+9', 80, 4);
    T('-1.1492955783746648570536506e+2817', '6e+4', '0e+0', 79, 3);
    T('4484030208377247171694176017611192924013688.3673156727912', '312.4806528830326624713586555970680789512', '2.859439443658921358e+2', 19, 0);
    T('4919312109.33201478223012', '-17639962423790464234153464.10247987673561682626830739575544', '4.91931210933201478223012e+9', 93, 6);
    T('2.2390687321203050194766155e+773', '-5.4483142e+52', '1.5824886e+52', 36, 3);
    T('8e+65', '698.4622627399997814462676574928570', '3.18671120956465230454740658434083e+2', 75, 3);
    T('8.66306e+3702', '-2e+384', '0e+0', 49, 6);
    T('-793276.28203811748641505218666466262613376952363711736395', '988.910', '-1.70462038117486415052187e+2', 24, 6);
    T('-9.5918025339e+2843', '-3.9837750701372440155890907651530333198956730448694597491e+9589', '-9.5918025339e+2843', 14, 3);
    T('58823951557095.8676475364755593894018233883293168', '-3.8563e+48', '5.88239515570958676475364755593894018233883293168e+13', 100, 5);
    T('-878397.97738', '3897847144.1207181084706384685112611176772698', '-8.7839797738e+5', 48, 4);
    T('-23097947.8877698237432809662645415737', '2.7717185937508e+8420', '-2.30979478877698237432809662645415737e+7', 96, 2);
    T('-744287.485599219477642684858364249396606308369082147069', '66666.5789355530061678915479915083387934225949543', '-1.0955117308136409795877830457657669878659824584847069e+4', 99, 0);
    T('-39958167613826562640563.68077220650641770904763754700538', '5.45e+6633', '-3.99581676138265626405636807722065064178e+22', 39, 3);
    T('-9.82574540246637178062431091145011806240078873e+97', '-3.00297153051693944510581266089778000248658962236887125667', '-1.79406974876227828582543339e+0', 27, 6);
    T('481335.40565106107', '563.307', '2.7122765106107e+2', 39, 0);
    T('-8.943550875826899174004147098160947209117401e+112', '8795986955015178649130170399228023.855728306', '-8.254602310669191899712579731892356266536504e+33', 57, 3);
    T('-2.736538371967866031325075682600169801042080643176906e+67', '-798.10121', '-5.1093943e+2', 15, 6);
    T('8.89244411964474967575e+995', '-1.22396282949406136970691396606e+143', '2.1076810292497291230174828448e+142', 37, 5);
    T('-4.133853260833105676637834626304395288e+331', '-4151623665.00637323381006132759572375', '-1.953e+9', 4, 0);
    T('-4.98716329433101e+63', '5.2324163e+989', '-4.98716329433101e+63', 42, 3);
    T('-1.21401788871257659e+26', '7606.68304751112951636636391962372647031362684321', '-3.8282851383973410080190582059122127821019964721e+3', 47, 4);
    T('5.098420853880029521477e+9004', '7.160182680398e+769', '6.109129834846e+769', 93, 4);
    T('38972.7', '8.884699481786844829e+413', '3.89727e+4', 31, 3);
    T('6114.6135258543884127880811426762677209603105075061581615', '3.0606472374614422625444168825327291936017622710500e+275', '6.1146135258543884127880811426762677209603105075061581615e+3', 98, 4);
    T('9.44899733642716929280554042413664887e+67', '3365420322.33466499588376037066560633919203780807168', '3.14018804773158888457522038228739693507536858699776e+9', 54, 4);
    T('4.649378943629298120865999754037420765850004956622433023174e+987', '-8.06433839e+21', '7.10669696e+21', 44, 4);
    T('-6.99689242488633036641737497911e+337', '-8.62087725552e+511', '-6.99689242488633036641737497911e+337', 39, 6);
    T('2.68888209327808354367202351104495e+405', '-8819404.2382061121909378683556', '1.1144088192608904221647532924e+6', 34, 0);
    T('95.999733394918432190664281885062439232831974613845764432', '260016173581199901254971273211250760375409527201652153.674', '9.5999733394918432190664281885062439232831974613845764432e+1', 79, 2);
    T('-65183.1712462898879902640839124139318475', '6.256224403276153e+18', '-6.518317124628988799026408391241393185e+4', 37, 3);
    T('6.73109e+5632', '6.2e+5', '5.4e+5', 62, 4);
    T('5.02028141167363618090367152620443275143849821e+65', '-5.1423741815608532540616250447301058274773391073e+397', '5.020281411673636180903672e+65', 25, 0);
    T('-5.92043551e+148', '-4.28952447049490303828404027555378e+6513', '-5.92043551e+148', 52, 3);
    T('-7.776298593803313194202564e+91', '-8.65584352736057897977696414850809690748461947332009869e+121', '-7.776298593803313194202564e+91', 58, 5);
    T('6.0468446816579233067128095e+47', '-9.25703241665044447133e+232', '6.0468446816579233067128095e+47', 61, 6);
    T('-2.8596931954822549177171791e+65', '-2842388394.3157048684464', '-2.23925487e+9', 9, 5);
    T('6.049098770167682103e+61', '6.62504441855149114634843111447292655983275898565177681129e+549', '6.049098770167682103e+61', 77, 5);
    T('-3.1766317506888756886345e+6581', '-1.02395581038358273020004954944197227e+8632', '-3e+6581', 1, 5);
    T('-6.494537598665515278622730e+407', '-3.693271952093896608738805402952058e+593', '-6.49453759866551527862273e+407', 55, 4);
    T('-54570662.9152087959960116', '3628345183.695935530586261340756849', '-5.4570662915209e+7', 14, 5);
    T('4.0937805605767803276628345776693834523568082109865317580e+7653', '4129.795194087170', '9.9197199795059e+2', 78, 0);
    T('19716777.12985866079587414603535351339087353731317', '4.15268790332934991803928616e+612', '1.971677713e+7', 10, 5);
    T('-653510.8843942609262130842269762944441504401', '-2.027688666442884256244726305622087828395505875189979280284e+936', '-6.535108843943e+5', 13, 3);
    T('-5.1464720662639698186031461962218221108584708551990576e+426', '-3.753738703787238056897308105489740e+5947', '-5.1464720662639698186031461962218221108584708551990576e+426', 77, 0);
    T('-185083.95424764097664278043736816641562291', '6.88483981698920539e+549', '-1.8508395424764097664278043736816641562291e+5', 52, 3);
    T('-8.4526212586667859362683394505144580845548023e+92', '8.56475540823390109934430835325058371670270846338329e+93', '-8.4526212586667859362683394505144580845548023e+92', 90, 5);
    T('-945636271.5759891', '-2.17432808397e+594', '-9.456362715759891e+8', 89, 0);
    T('3.548871234875504632967049284731417963316430094e+938', '8.58647841598924619155917147408449004293104046547183e+8404', '3.548871234875504632967049284731417963316430094e+938', 56, 6);
    T('-1.1430431170771599354075836985722188465803720293752e+2890', '8.77e+542', '-5.33e+542', 27, 3);
    T('-9.04312894420050284873550010294637483269334834464174461210e+871', '-4.224918240e+77', '-1.03574112e+77', 25, 6);
    T('1.3894699e+3042', '-2.31875507700977838117805540505980140857029476462714e+91', '1.9357052749968517239204685249164188278642182342502e+91', 61, 0);
    T('79.235', '4.4827898048205433701866299445e+255', '7.9235e+1', 75, 0);
    T('4.476999920584999735690940e+2947', '431.4007436129918618300842573436992667279830088', '2.177092162465684682859259261336151290563486416e+2', 73, 0);
    T('1.951106229235660612010096898495653211543461145647637e+61', '-3.3477064e+141', '1.951106229235660612010096898495653211543461145647637e+61', 89, 6);
    T('-9.916630e+84', '-66.064342', '-1.3177302e+1', 37, 4);
    T('-860.1444052695', '825080074792355962132959107719531393891221', '-8.601444052695e+2', 32, 3);
    T('6.1352951126363381185748747422972468541675591991781e+9278', '4.1560786995244e+1527', '2.651534168388e+1527', 34, 6);
    T('4.944978e+764', '5323614045.2252161212631613035133757836906706', '5.1535379640553098968737518932745870816240816e+9', 97, 1);
    T('9.09144e+6306', '3.50201226950969565080636878120146612e+55', '2.69469845381791678871664408741658344e+55', 96, 5);
    T('32172841276723723017320184344985646175887112.47790381', '5.58951e+9', '2.42307711247790381e+9', 20, 1);
    T('7.00620e+9706', '-8.08968082636983139609501341360433350633079179648541620232683e+9707', '7.0062e+9706', 42, 1);
    T('4.264042494837184947927734754557762536643709070327e+540', '9.95e+15', '3.95e+15', 45, 4);
    T('5.6105940856889306e+8254', '-7e+2', '6e+2', 93, 4);
    T('2526.4723101095362366485683598497883', '8.77812650378806065221188793462677511675764948870478079267e+94', '2.526472310109536236648569e+3', 25, 2);
    T('-4659.75203988732274192476946401', '6.060e+79', '-4.65975203988732274192476946401e+3', 72, 5);
    T('9.139217102709553745990883488575994294713541e+500', '-418253.135835407934425294007034135260164397764', '3.58109993522251265587238475937111423227918172e+5', 95, 0);
    T('-8.1086318916990298755700481699299539037472013444974422e+5755', '-1.5661587555799104561758191e+87', '-1.0074281153436182858841239e+87', 99, 5);
    T('-416737.42745', '-77173894789158506974126571178514075283378413478233.00755', '-4.1673742745e+5', 16, 6);
    T('8.333306239514058871243415468295297607303726047354266e+684', '3.502702752540721094141e+59', '1.367728644750058669789e+59', 61, 4);
    T('-6.25787639e+479', '9.9197660713292e+55', '-4.5153786834196e+55', 60, 0);
    T('3.72001563869117563677e+76', '-66669138.417849525425581002528394724049111419621217', '6.2279859967071260303160475364250607980294928950909e+7', 55, 2);
    T('-1.5482390020990842932e+1127', '-1.527299849113353346489254e+726', '-1.057674615874692739663148e+726', 63, 5);
    T('-1.81968692707021067164405981e+1010', '-8.0e+4383', '-1.819e+1010', 4, 1);
    T('-3053673848.9365372689017115785293', '-7.0224310e+2372', '-3.053673848936537268901711579e+9', 28, 3);
    T('54183.632867779042905454844274458562747', '-83662541.555738139789607530938580336298367634', '5.41836328678e+4', 12, 5);
    T('-4.22473209309078364216473134517737205060872591792014964962722e+9967', '-269720069.2925602198855004698047', '-1.596172726513e+7', 14, 5);
    T('-5.75017431522121151911584268476607397041271245184126e+6110', '3.1316103139139653e+59', '-1.8603166122551754e+59', 33, 3);
    T('-53.3032029466169433002012740687580043518816717487', '31641.6419229957', '-5.33032029466169433002012740687580043518816717487e+1', 70, 3);
    T('72217.81124137174718066449515357955447849981408895', '-8.7683588909862633417120718073873e+49', '7.221781124137174718066449515357955447849981408895e+4', 53, 1);
    T('4692222630612536029410534310094035996228840113566296.8', '1228263.06453688667212142398916324', '5.4696289351779412826381928353116e+5', 67, 0);
    T('-7.9540186730450607380161429790e+6430', '-4639.1111967549704803866386', '-1.07e+3', 3, 5);
    T('5.22e+7656', '-8.500204601289537368439368068473e+64', '1.31145091818334943531353566321e+64', 34, 5);
    T('9.87886266656141131566931816173489290346252900698068e+66', '-9.537278383404716745059580002e+195', '9.87886266656141131566931816173489290346252900698068e+66', 98, 4);
    T('-9031367.14748312394', '9.8676925037570147928180472738580628944445e+112', '-9.03136714748312394e+6', 93, 3);
    T('-3.237780151586765403233992546861473219e+103', '-4.213698590019308332103708310931502e+86', '-3.715913030258199339756353959945608e+86', 100, 5);
    T('-2e+624', '3.0630010e+754', '-2e+624', 24, 3);
    T('-26267210542733.60053963718272912688751521576671166112045188', '8.920557499910098814348531734605286864e+7549', '-2.62672105e+13', 9, 6);
    T('5.884563e+86', '5.544271814073647469677715398389e+68', '1.656897857425690453539776341638e+68', 49, 6);
    T('-2.9501e+1031', '-5.705094676024e+32', '-3.086833450968e+32', 38, 0);
    T('2190203683.01784039790649953183427044501728975229622892', '-129341.6581527620814546235644263675042036956912', '6.1385517120072635e+4', 17, 4);
    T('374.465410986072628695747577414265711313', '5.2879747143440196018217433252932138284305606201435437e+952', '3.7446541098607e+2', 14, 1);
    T('8.4784750153107531506270037576e+327', '6.4828775998173475131160558410831475175e+2971', '8.4784750153107531506270037576e+327', 34, 0);
    T('9.62707270e+501', '-6.342341847592514387089599362701767676614396026825953485e+539', '9.6270727e+501', 84, 1);
    T('7145500474663735829770809398299303175.875800464970535410224', '90828572.5113415651186621228687335129152209', '5.6290732076087447357393897802e+7', 29, 1);
    T('-6.05845881219948340925618875938e+91', '6.61836246027891675911003e+7034', '-6.0584588121994e+91', 14, 1);
    T('-7.43e+833', '-3.1891516544348227127456801051019337878811793572460629e+1748', '-7.43e+833', 39, 4);
    T('-586487137896561.9', '-7e+720', '-5.864871378965619e+14', 32, 3);
    T('-7004720099355232745.58685968254503', '464534361433180', '-6.46330431152558685968254503e+12', 83, 4);
    T('7376349279.6789964', '8.13158102169523546706061074913188868474536498e+1244', '7.3763492796789964e+9', 83, 1);
    T('-2.66694516312127080093289225754644e+5649', '-8.2788550315689695e+274', '-6.58461978336281e+273', 91, 3);
    T('-26377480.2334276377485822339988800328191768957719819966', '97339239938767981386925617.88982454562567822091088', '-2.6377480233427637748582233e+7', 26, 1);
    T('2.3644616729891784263411140e+3430', '-981855.982985359583764068205579338555912870110328229714', '5.192582575653964788593134550979615774147e+5', 40, 2);
    T('-2567.301939196367215654841', '-4.7110984724522e+20', '-2.567301939196367215654841e+3', 48, 3);
    T('-6.887177370604424697705428668658450e+36', '-7.67098e+3101', '-6.88717737060442469770542866865845e+36', 73, 1);
    T('2.3477568659602166267356708374e+758', '260693.9220345082421249094573691967212817853512', '2.359424256311595263045483114847279155538568672e+5', 55, 6);
    T('-5.033571988963017652865943270154512156255697111231944e+1306', '-1.04835155898865079e+703', '-9.344544833551976e+702', 72, 1);
    T('8.214139451354306802429236611686779e+5690', '-1.1263730322555479536183521284e+67', '3.74429518435572840395289768e+66', 50, 5);
    T('-8.1488000628720211739998282062206736158332e+75', '-2.001705e+6119', '-8.148800062872021173999828206220673615833e+75', 40, 1);
    T('-1080.0089303345457', '-2.70053959411289137357067400343848e+7462', '-1.0800089303345457e+3', 47, 3);
    T('-951.8120833154', '-4601803.9053077601346104732884149542122', '-9.518120833154e+2', 85, 1);
    T('623427944.6738997303017990', '8.80254156992034229e+39', '6.23427944673899730301799e+8', 67, 5);
    T('2.35391833099492200539685512e+541', '6.54579350680846293549701724417257e+6501', '2.35391833099492200539685512e+541', 78, 3);
    T('-5.802871745569351746236032657404971772902653184e+166', '-569601127769.9071963300149221373', '-3.942765500842812255133287266114e+11', 95, 2);
    T('58251.5299394592584913418694544936952', '9.922305336838175628353279413320037919974386628e+529', '5.82515299394592584913418694544936952e+4', 38, 0);
    T('178.810791', '5.9730850405607582e+127', '1.78810791e+2', 84, 5);
    T('-7.213282126127679116801393251769456078498115487567582164e+771', '9.6205166181505588836203473075832010219834163e+54', '-2.5117368798393502824500490159742596742822416e+54', 68, 2);
    T('4.4e+1415', '-1791.169782612342099849155398906941466798405915791267', '4.653087683667797453179502356881392981661075348818e+2', 49, 6);
    T('3.34143206113154599754e+6694', '-7.26959684903106538055444862542e+55', '3.3407965043731368445510652696e+54', 72, 6);
    T('7.01029485594801768569517050720798941155721968259835932500e+9422', '-7.0642669512907646431409065185107769129961433230267e+65', '2.2427484434740226146829404720536384416432051622105e+65', 95, 1);
    T('-4.33801305787072916172388791e+80', '5.22122485473591837565234669915e+391', '-4.33801305787072916172388791e+80', 27, 2);
    T('7.8492581941996654096258841e+325', '8.9130586564498055969723193446e+77', '6.957386304461095271086697014e+77', 33, 6);
    T('7.23792299537795282764181e+89', '84967.13729986244128821297358194722156577559385', '6.7621143544859443748328006466947263422e+4', 38, 0);
    T('5033998704.17776924377988587279152844', '8.08401900237472415884868065688485795856720872615446e+8410', '5.03399870417776924377988587279152e+9', 33, 3);
    T('-5.94302899773211952133552303e+187', '2.9972e+7', '-2.9248e+7', 64, 5);
    T('-9.708e+685', '9.19085856796459395e+9531', '-9.708e+685', 68, 0);
    T('-1.5465036868358516166872219595', '-6.6120784612856e+68', '-1.5465036868358516166872219595e+0', 35, 3);
    T('-3.3053551448370844845756700300850e+3913', '-8.3467925553134e+50', '-4.1896819159544e+50', 96, 1);
    T('9.09456645650818213193831463546071024679434471379e+666', '7.38724213405152e+6026', '9.09456645650818213193831463546071024679434471379e+666', 67, 5);
    T('4.58641736178041090644887341275999994080950314188e+1696', '42578392.4318675275571079327730844188901162369235804917550975', '1.23791564353744632853300363493236151396949910992682155531775e+7', 65, 5);
    T('7.9961e+998', '-1.1820804651008895509998463408241199118086e+9578', '7.9961e+998', 63, 2);
    T('-6.072e+5252', '-7.18009312791883624836138815068459810882308438794417e+8712', '-6.072e+5252', 47, 4);
    T('-4.5600994941281230126894362369720165703108388452e+2403', '-622038.4747064194717182782274608403027399105469860723330', '-5.4141438904500328e+5', 17, 6);
    T('-3.25278868174882601881e+2959', '7.8431549098046872891828825565737588458734074284671853699539e+633', '-2.3794416114981559662e+633', 20, 1);
    T('3.6343105960435681326882097841415419173146739156748857909293e+163', '-21', '1.5e+1', 34, 0);
    T('-8.518623612127699770931777976e+45', '2646604900335022323292722367397273217264.57291', '-1.71262916408899413908e+39', 21, 3);
    T('4174055292995644.9880231577567977226789530659873674', '8.360559715199137928831459829758326270411708783897324911671e+6398', '4.174055293e+15', 12, 6);
    T('61974599899119.9534242842171966733601323', '-5.453e+1783', '6.197e+13', 4, 3);
    T('6543503.0162949178948038773968906145153443230268592', '-7125491.922', '6.5435030162949178948038773968906145153443e+6', 41, 6);
    T('-29924.33147502565764364098940606', '-1592929.777384719554672688705249208524154144674784435883721', '-2.99243314750256576e+4', 18, 1);
    T('-4311819637633458353446934226481969087340.175', '3e+888', '-4.311819637633458353446934226481969087340175e+39', 69, 6);
    T('-701325358.8507151963018486', '5.02485985784225975e+2516', '-7.013253588507151963018486e+8', 69, 1);
    T('-6.3336349419868999600556396454550393580710353e+559', '30.032145617405641584200167823777472745177252592857', '-1.804780266424170907995664538747e+1', 31, 5);
    T('8208171.29438957568235282192379149634710643516674459', '5.526518296200919451845144699586071213e+56', '8.20817129438957568235282192379149634710643516674459e+6', 74, 6);
    T('-7026771.991488141635713537245992635089330556307', '-5.78495686345608787896765252e+87', '-7.026771991488141635713537245992635089330556307e+6', 78, 3);
    T('-682825', '-9.58106140947497715953494e+7650', '-6.82825e+5', 53, 1);
    T('1.0432667917954743913105334020914075684681873553141e+53', '-363639573.56541900063559754', '8.8041122284741437367e+6', 20, 2);
    T('9.294092190225380807622716e+554', '82510153.549550000881266062462945', '5.7307479561359672028887521773435e+7', 37, 0);
    T('-8.61826021489505921628320081423912946310238785369120278845e+4535', '4413386685.26795493710228068839622996984174618', '-3.38270650710585646963860751526981526607567182e+9', 85, 3);
    T('-2.672585214604e+621', '-9.578201948955624059202e+62', '-5.08879650726473396428e+61', 35, 4);
    T('6.9128357540377177371762216197787022755536508992068094e+523', '-3.2577610725568034259555913702314301e+502', '3.2025889458822105348276677851579185e+502', 94, 4);
    T('-5.441086784380795770829212', '671.1382228580292151191915154358159424235725030704', '-5.4410867843808e+0', 14, 6);
    T('8.151289172051252728e+49', '-288.51320484667811368085314331', '2e+2', 1, 5);
    T('8.075e+8', '-2.3439742423603951162113459', '1.5484257243982081068293252e+0', 98, 6);
    T('-5.8040872885301e+27', '-756632.178760425828115651424792498655635088327', '-6.7838751e+5', 8, 5);
    T('-6.30185023912358362453739e+43', '-5.8291686913947145494292529837761538164089993367780968996309e+182', '-6.301e+43', 4, 2);
    T('-2.3168408598819483038286410534187220e+54', '5.76821013360839181791090764e+895', '-2.316840859881948303828641053418722e+54', 64, 5);
    T('4.73433879920597402830e+517', '7.19935223140121316154319557870953e+83', '6.276715240828e+83', 13, 4);
    T('-2.81509e+6', '1.812e+246', '-2.81509e+6', 83, 3);
    T('3.074739e+61', '6.511691130037171765080186071677189986092513692281554225607e+872', '3.074739e+61', 45, 1);
    T('-3.7125440302535164287920458084935906', '9.4056833e+407', '-3.7125440302535164287920458084935906e+0', 100, 0);
    T('3.8089600875379985918666854598970946009387586848198798601e+64', '-9.842428987583196816730893684990759478255192322e+172', '3.808960087537998592e+64', 19, 2);
    T('-6.49308611559837353028439424226724e+681', '-6.9279627338450039699162972e+5038', '-6.49308611559837353028439424226724e+681', 52, 1);
    T('-3.283e+8', '1.59267743467980e+46', '-3.283e+8', 74, 5);
    T('8.69757929965438373165362773612974e+192', '-73.79436307426341549232457077000482745011', '5.145179803297395e+1', 16, 0);
    T('-8.2373007776535315311267037638881860091e+40', '-5.757730498009786912193794e+37', '-3.7461654995362466895783438881860091e+37', 89, 1);
    T('8.2753743577733496649229415e+370', '-88797.16162', '5.492178204e+4', 93, 4);
    T('1.48581817545381241776887741933992917284785358529', '5903622466634088045894004.22718934', '1.48581817545381241776887741933992917284785358529e+0', 71, 6);
    T('5.129086074208008222351499810040746978958761126e+293', '496793381949653476385416592315281192971751225.09020825', '4e+44', 1, 1);
    T('-2.80845381738689415933627794871721050391716565400744493348202e+159', '4042022.949118089592', '-5.1205e+5', 5, 5);
    T('-7.608461933481403715443327118274378005025e+6785', '73027440923359159570.9274647515324850715', '-4.0233559318242465845769751200091461814e+19', 100, 0);
    T('5.3656679337854155278424098140145779577342655431342620134014e+7681', '-9.34344633539969e+136', '3.2266e+135', 5, 0);
    T('-8.76e+15', '7.884013646400282186823118749015404657404866922926e+5546', '-9e+15', 1, 0);
    T('3.08613897206936240922007217437132e+9939', '5.60781020e+9', '4.424842e+9', 17, 2);
    T('17.347072298067804168620815', '9.6146989823337249222122068576315652901e+59', '1.7347072298067804168620815e+1', 81, 2);
    T('-3.3610512896674974771860856952304666e+76', '9.82609790457683994410', '-5.4089915183757222993e+0', 91, 3);
    T('-5.075266107871695934621156287679557190e+225', '-8950634354.102096867', '-3.320100822028796139e+9', 64, 4);
    T('1.80906977550635553008735110130154444563675928517e+222', '-2.3641653094828594182991e+850', '1.809069775506355530087351e+222', 25, 5);
    T('2.10724292529592973714942968646730881946629591268819155763e+96', '8.7469186241449712430e+308', '2.1072429252959297371494297e+96', 26, 5);
    T('2.511951653e+3360', '-62704201442675201961255868610.62', '2.566182839527647e+28', 16, 3);
    T('-243025.9013822376723836745951444', '-2.6675104229981642056765726341862e+9212', '-2.430259013822376723836745951444e+5', 91, 1);
    T('5.9150440852e+129', '1756386.26758530048962177340289455990042113', '1.51990747887335793925678558668139806689985e+6', 53, 5);
    T('-94252.40812498886857383293649', '-3.8032732751134162575094809941651e+621', '-9.425240812498886857383293649e+4', 96, 4);
    T('-6.00559063113006083268308811806162403396832594e+7303', '-526206391770735839309548148095066062264265607373.735063960260', '-2.5090125586506678664e+47', 20, 6);
    T('-2.2567835144492993286994200e+573', '3.86e+19', '-2.72e+19', 56, 0);
    T('45.40437334953326105840633843242', '8.165685418205048453423994094023522482e+91', '4.540437334953326105840633843242e+1', 55, 6);
    T('9.185046132683776162495324e+661', '3.58881073687353598e+59', '2.1843440628149424e+59', 32, 5);
    T('-4220.9', '-8.8329640035109528839928169445847205646e+219', '-4.2209e+3', 60, 0);
    T('8.30637166e+302', '-4.38491646911266096005916850074729e+3096', '8.30637166e+302', 88, 0);
    T('2.185711107406016977866393050197487144635034e+58', '3.06090665624414123912790787e+675', '2.185711107406016977866393050197487144635034e+58', 43, 2);
    T('1.5296066903357290860998727274804151388194688e+5974', '-1.466761715057714170319e+6473', '1.52961e+5974', 6, 4);
    T('8.7703002924606448024413649168923e+32', '2341754130.133539925544697897156103395877431', '2.1670356458e+9', 11, 1);
    T('487029898.752779911', '8.135101930942998542008193816044385765717919343e+566', '4.87029898752779911e+8', 64, 2);
    T('1.9150246697283444362652e+8564', '4.99978542700598575703984249728373787466381200086783589470188e+9131', '1.9150246697283444362652e+8564', 39, 3);
    T('4.1723663498935229337300847622e+64', '9.417364815e+750', '4.1723663498935229337300847622e+64', 93, 6);
    T('-5.80071262288385890879546110e+1304', '-44653033.5861971108811049733295433756519', '-1.32989241479727e+7', 15, 3);
    T('-6.410283966506585921734556645379985476e+7965', '-1.4452133411275314346797554348517494114080451e+8135', '-6.410283966506585921735e+7965', 22, 3);
    T('-4.463068952024849936585e+83', '759873561118.008235886', '-5.1577320288074538143e+11', 60, 5);
    T('4e+9283', '148.10763457479691472126638872989006390078', '9.36015177857864005742868937558949709e+1', 36, 4);
    T('-9678.67522487939495532884892712048941', '57158639153448763130801181566037936611476.352557841', '-9.67867522487939495532884892712048941e+3', 46, 4);
    T('-6.646536667518223646e+899', '-8.553077200743e+3307', '-6.646536667518223646e+899', 31, 1);
    T('-98249.481804612124455139938915843980298207095761218853659', '-6.45599175082348525e+18', '-9.82494818046121244551e+4', 21, 2);
    T('-3.4633479305842963815661474075922800237442578929571330e+86', '-1213454413.98166345449482322994387887637049771856379923', '-8.1469069894317996974242021853014743616418393431221607e+8', 68, 4);
    T('4.14042901482813011064076807878477624131714516397534517574780', '8.07529581456875095670507288e+836', '4.141e+0', 4, 2);
    T('-1.186381227797506773054e+884', '2.2342207335106787810238740521e+72', '-1.3779700408169503393279341886e+72', 31, 6);
    T('-3.803886995091072e+7906', '-23537245.7903626442959', '-5.61e+6', 3, 0);
    T('7.61e+42', '-2.473057862e+55', '7.61e+42', 39, 4);
    T('-9.20605548567388962794822308647192014617837e+4795', '9.26461690e+485', '-1.9732003e+485', 26, 3);
    T('7.362274234974147451334501139475e+47', '3.752291138218511016817421394058e+962', '7.362274234974147451334501139475e+47', 47, 2);
    T('1.86301872972652865213121062936142099233617439207583002e+5197', '1.2238639108231707297507630293818e+1780', '1.15275134331761150255199e+1780', 24, 5);
    T('3022365204.85965079209039546493297080428701476331624917', '-25.5223192750', '7.69017589209039546493297080428701476331624917e+0', 98, 2);
    T('42868595.45618635621119900149865739878256286850414433227', '3.3999e+361', '4.28685954561864e+7', 15, 6);
    T('-7.71688659375674832909796960916752526218874022375e+148', '-15753008697661046718358448750908544830607926613.40499400613', '-5.9e+45', 3, 5);
    T('1.9284246403722809858e+7734', '6525412101.639619888979581992666538221574033628762148', '5.40704761120424292013477392401319676412847917782328e+9', 87, 4);
    T('4.21029338621982102872635733165381649311875274461e+96', '-88.581446010534145096540314802137639730896703177300746', '5.8512338e+1', 8, 4);
    T('3.18341269489971059122183574142507e+970', '2.5602894530193765489e+33', '1.29935644e+33', 9, 6);
    T('-9.8531751283534981e+46', '7.169599753899455450209e+9552', '-9.8531751283534981e+46', 81, 6);
    T('-1.82712706169300139803523848589e+67', '-50457198519534924.0272604948051705361632655807883718751', '-4.79320031586887114037276099477117343700280055795171471e+16', 89, 3);
    T('25571654.941988195065442', '-2.7680952006258877305544969012e+195', '2.5571654941988195065442e+7', 24, 4);
    T('-9.21498178621962622636429553798082e+3010', '-4.85542560162929861805404e+144', '-4.23557136689192553352084e+144', 38, 4);
    T('-4.0684375974806074718064954614e+498', '-567320856766921843586492938451265820334718.0161173544510291', '-2.596664704445670358456516890797988708446242190372936733112e+41', 82, 5);
    T('2760708236.78969478155618197180525429455268', '4e+555', '2.7608e+9', 5, 2);
    T('263101.2598526827046503426', '5.723386e+237', '2.631012598526827046503426e+5', 30, 3);
    T('-646975554.81898753405432', '-8.865330721901060408e+412', '-6.4697555481898753405432e+8', 96, 3);
    T('-4481580.823', '1.22565368949672694293761354744772870602670565283e+2850', '-4.481580823e+6', 97, 2);
    T('7.488226094310517e+534', '-6.935066894481183019520572233e+981', '7.488226094310517e+534', 97, 6);
    T('-1.4743636175388240708103841633734229901e+9488', '9.257892129520633781587754607891239546901880479886309187e+335', '-5.941611095644815254925010764651709401482e+334', 40, 2);
    T('-5.3900320768788637979811560751343376e+8603', '-1.54777267853067419297661583888368047181267238891873986968e+951', '-1e+951', 1, 4);
    T('4.629931385594576336170105734976675843979563e+965', '624.5021799', '3.544985128e+2', 10, 5);
    T('-8.82623816584418e+120', '-9.36413e+6134', '-8.82623816584418e+120', 40, 0);
    T('-430.641209278912494930488723257', '8.127448472050740080986555418342711531781804693586e+97', '-4.30641209278912494930488723257e+2', 73, 5);
    T('-1.74594936802352551990623779708486158520804470e+658', '4.814015030341058265817384488402968502920033593289161e+63', '-2.358017137672851488178026143045682770277799038465656e+63', 91, 2);
    T('-6.2e+9', '-479713350100222762593480297529306001235068989.5496448325232', '-6.2e+9', 70, 1);
    T('6.721083110e+8186', '6.71654718774228267515197836771737799218698618561e+69', '2.47080526154174594259154529022793970796546273647e+69', 67, 0);
    T('-2.6229713e+983', '7.653422549118643031417949102129859784015577941326537181e+581', '-6.78681141563638781385468e+581', 24, 2);
    T('-31.997390328109', '-8.446335720690281787942014', '-6.6583831660381546362e+0', 20, 3);
    T('9.087444774996340465362053012734768182227190529631720970e+541', '5.010281594594381336103703273801020485062856082868e+785', '9.08744478e+541', 9, 0);
    T('-9.6737052470470547306480468149259080737202e+3479', '-26.24183578648', '-1.667322907936e+1', 78, 2);
    T('-7.799192e+76', '36955708.571561609235822792919693566957747801108915133735', '-1.3376573400381828494634600124331093928974607421761671965e+7', 65, 4);
    T('-1.744112455180e+987', '-5247.0941242379577623588873011483615127031780803', '-2.362202e+3', 7, 1);
    T('3.22361447e+427', '9.7114879230244941518772776e+89', '5.6218544953999493663011656e+89', 35, 1);
    T('-571471.5662', '-9.01880764963655566543592e+993', '-5.714715662e+5', 21, 3);
    T('-2.6867412e+9', '-329.3887762581982760468291629045426279799705042955118426', '-2.874222699864498543659658606328522595289991e+2', 44, 5);
    T('-5754161.90610295566499709295276526', '-21998.84234714889802970374513055940519469848482466', '-1.246405349709327924441547368925524418369546076374e+4', 72, 6);
    T('6.9689252986218726520669289578858143174e+3653', '1.1e+36', '3e+35', 67, 5);
    T('4336349193.91526301764', '-3.67887774049380331364635962987384349137476913', '1.31825125517099557938141473707093391973330314e+0', 48, 3);
    T('1.061e+646', '1650.361594108557632348905598993793018744', '1.378224059609582105132383332119267373104e+3', 80, 5);
    T('5.951100547546084517374184885463164727e+53', '5.304550310122073445169332e+26', '5.160746632622021866696032e+26', 69, 2);
    T('7.6247764966344654592387822', '-5922626968442349229379371499.2646210040970858392', '7.6247764966344654592387822e+0', 99, 6);
    T('-7.61066083595094149178668264780296163747782510620738e+7239', '54649.75705840752664806', '-1.3758448368126464187e+4', 42, 1);
    T('466962.07849619107391', '1.6354462741e+3136', '4.6696207849619107391e+5', 45, 1);
    T('1.86436223224321191631447e+7021', '-1.268271087259451717804309942284679238902347324860e+101', '5.3543284684601756688948712257686139190506362236e+100', 91, 6);
    T('4.154692e+57', '-4.482019298491157384455568475241948198e+415', '4.154692e+57', 86, 4);
    T('3.96922752592e+279', '8.95831219234629648686342391286832172230993712805825334086017e+815', '3.96922752592e+279', 90, 1);
    T('3.97282107740737915722363965030643028364955226962590357976359e+9126', '-3.4834267558151570382445e+142', '2.1794974244196568528475e+142', 86, 2);
    T('1.47753325012226485056054817e+30', '-926946906.4061911429042', '2.281088966897378582182e+8', 50, 2);
    T('2.599019e+737', '7.2472604142e+21', '3.001734104e+20', 46, 2);
    T('3.4465016460240534203012453197850521111198629696054119010679e+2357', '-4.842316344173226487404936763494163699649e+43', '4.018955584904623756979965257214140578128e+43', 60, 1);
    T('-6e+859', '-46', '-1.4e+1', 42, 4);
    T('5.4333230655538563349382833962157999e+6795', '1.5362128790912652866214291607427572701116e+608', '9.035577412107316652874584146846926968436e+607', 91, 0);
    T('-1.151629634426773950270204542285352551672690e+888', '6.373518443852738569858787777923013590483e+539', '-5.553634345917281745603641671873358085175e+539', 58, 5);
    T('-4.65139233497820e+716', '8.968295998626079429146537781860682373258204813736365494228e+969', '-4.6513923349782e+716', 61, 1);
    T('-7.686147022367753323067850106756497101960614e+51', '5.609951953717294747851894157404e+338', '-7.686147022367753323067850106756497101960614e+51', 88, 6);
    T('4.1387e+93', '1.6631119651166677828631637035355344360231456799668010743533e+960', '4.1387e+93', 88, 4);
    T('6.72016084343355720157702825681263386315661127e+782', '2.38042832683210047844232402039998552858384e+2348', '6.72016084343355720157702825681263386315661127e+782', 86, 1);
    T('2.529793091095528554e+592', '3.67761847814e+4813', '2.529793091095528554e+592', 29, 0);
    T('-2.79673984654301e+50', '-4.6422563812600727302177e+9156', '-2.79673984654301e+50', 87, 2);
    T('7.4913883420495868746699198113206057576225337883e+8900', '-9.8730811323789680067836e+99', '3.594e+99', 4, 6);
    T('-1192860099129534979713052811005278375523793330289.08787', '-250.3977129684690429583', '-2.254903592683258140982e+2', 62, 1);
    T('7.379424576912452086716433053315e+9936', '9.399e+290', '2.31e+289', 39, 5);
    T('8.291479626145109569096e+641', '2.2594066177317580203273840688124612310159295524597216e+6456', '8.291479626145109569096e+641', 45, 5);
    T('3.6790127035', '8.3e+192', '3.6790127035e+0', 33, 6);
    T('-5.02633872255680364456454180149388297773250543238304711048e+2543', '-7.733549211930421655397744949e+736', '-2.3085391901450429068968e+736', 23, 1);
    T('-6.974568115829018193554637982851375580259e+66', '328426915.4734338658334748727030182441550947', '-8.52706901901923044676986636768859508418238e+7', 48, 2);
    T('5.79267256899203e+6580', '6.13337226436486607709769573828455271482822861407900e+170', '6.0476111248591995407459176685877028876e+170', 38, 3);
    T('554290078585044.62117682677024141471359742431655155390214415', '-3.9893836e+5017', '5.5429007858504462117682677024141471359742431655155390214415e+14', 95, 6);
    T('2079780.51526478607343413293628894287627', '-98689.422040', '7.30265242478607343413293628894287627e+3', 77, 6);
    T('-5.04080093125549290497e+99', '-668429392.4847573500347624117023553186634865880221', '-2.518404698742946359768022409057306804425394149388e+8', 100, 4);
    T('-815554.924111152438943084021120781519792573', '-6.69804985030097052484387152741959122328', '-3.74338506267838094223942172092446e-1', 33, 5);
    T('9.25e+491', '2258.20823841415336717171466743157798', '2.8549500903175132917028755350494534e+2', 96, 3);
    T('-3.54250273117477052107230804834601090865035579e+2332', '6.58875531184828691390473562065248012e+2080', '-3.90962324939677493410094598981286432e+2080', 98, 1);
    T('-643842621533000743131754423166.99809619100459931', '3.23444245907453320352059210516024223873394216183601631e+124', '-6.43842621533000743131754423166998097e+29', 36, 3);
    T('-67.8375110893081122814343435849753339315588735099809', '9.94589629236212429184520597721110420413250660638527378e+887', '-6.78375110893081122814343435849753339315588e+1', 42, 2);
    T('-215312870014620540.89952277381648898', '266467.46161510648649135971799', '-1e+5', 2, 2);
    T('4.95258532482756132624176856710199255277140075e+6692', '4.20500836029267819251876096729027213617253836630222675398e+150', '4.02913662806823258760318533094996224449551352633366098644e+150', 68, 4);
    T('687815300792.2055391710619127364203437409482152', '-45.2250096712995316890883154986175507914748108036844102072', '4.03246294822884495553228233674e+1', 31, 4);
    T('-2.21755299543992822549146e+42', '3.321175e+2344', '-2.21755299543992822549146e+42', 37, 6);
    T('3541765.36917719531011772631346169973171129379813034906', '3.6226965934000358460155094526031245541773285315225467960579e+377', '3.541765369177195310117726313461699732e+6', 37, 4);
    T('-6.34345015903917491475745228240505619898471197474193306e+431', '-63528.631851339913868675', '-1.9999489398557025833e+4', 20, 3);
    T('-8.131998976994287561046923612615113e+2436', '-9907.3228549666708350', '-2.27584221817714488e+3', 52, 4);
    T('-3.28066653043601793599714751070134402000162e+680', '5.902106181369894e+95', '-3.499954661643666e+95', 21, 3);
    T('3.82725391504936840239534405e+29', '4.033172026724718355641534341914773042638756813923845e+967', '3.82725391504936840239534405e+29', 95, 0);
    T('7.4350987389168736714374248098790462005928699e+998', '36977050.41034287096', '8.87490871768689536e+6', 60, 6);
    T('-3.348647161868', '-3.254726386132960559332987104077793882775637109994e+79', '-3.348647161868e+0', 38, 0);
    T('-9.67923747106049546789444904718534e+813', '4.269179045530268294983343830323519606420394512561234653e+2136', '-9.6792e+813', 5, 5);
    T('-5386517788552.930316004437670735', '-7.020507899714899304634e+2568', '-5.38651778855293032e+12', 18, 3);
    T('-4.72111677393583081087328231616391098951e+506', '31349977539892190847.094069735672434417364893', '-1.7767400365105874364844021245772786174308825e+19', 55, 0);
    T('8453183.58496717141883346077656008445975664575106', '-2.712409225935e+606', '8.453183584967171418833460776560084459756645751e+6', 47, 3);
    T('5.087640487183540449143135009616e+461', '-5.96224051e+28', '1.39711607e+28', 15, 6);
    T('1.180205668e+70', '4.151479900606620e+919', '1.180205668e+70', 84, 5);
    T('3.184e+24', '1.16241364006575087506350856296510496755915115798e+9388', '3.184e+24', 49, 3);
    T('5.22333512190843647059219031326470329554930e+9908', '-6718186.32376290710067367131754700853658', '1.96822375094026115280310325658337183982e+6', 69, 6);
    T('633378984627988683936992676948002249358254519214.83', '1026771109734533361884230967714550095168285.6646194612903166', '8.507927054961916384252797340194523651509950106306101408576e+41', 63, 0);
    T('9656.606429462200683203999185352860951297313586', '-2.0780e+9352', '9.656606429462200683203999185352860951297313586e+3', 97, 6);
    T('-7.8194761398564201966057645332e+9210', '218042.83361919812961911222595949989857510865865402', '-2.880816385285626354695937975712023094414022441018e+4', 61, 0);
    T('8797824611.092780', '-9.861486615362188332028357602969726442001070e+563', '8.79782461109278e+9', 69, 3);
    T('-2.6299126586101947812264282706549550656605589385e+4624', '-1.52791e+336', '-1.42737e+336', 67, 4);
    T('2.627e+97', '8.629329471954961914684721664800612e+52', '6.7932453017371695332e+52', 20, 1);
    T('3.4666599055131821123668e+217', '5.739902868434337447732731224570766384970128443228959461e+951', '3.4666599055131821123668e+217', 60, 5);
    T('-367.4991457608582569732905836363264353881448', '1.502625400071e+48', '-3.674991457608582569732905836363264353881448e+2', 69, 6);
    T('5.4608146301756991483e+664', '-96641314.61868319884536544087194614551', '4.141346095543289833021081e+7', 25, 3);
    T('6.34284747940579441617710853246e+53', '2.95340140559615297380e+28', '1.2735277670733855474e+28', 20, 4);
    T('1.971e+35', '8270854797.1468', '2.4409857496456e+9', 29, 5);
    T('3.301429072509952083386812591717557e+64', '7.923404070066e+55', '2.822418734198812591717557e+55', 49, 4);
    T('9459.5368069036932374654996304057586018483518760008853', '5.894989462304238156753209850e+9218', '9.4595368069036932374654996304057586018483518760008853e+3', 55, 1);
    T('417126.772534636477', '653099265.50404961050772634993987906689568356645978519', '4.17126772534636477e+5', 52, 2);
    T('-8.229014378063724664701927059166244707324681332e+2955', '-48042593139165050967447.982123458629951554467', '-1.3483594649279771223474745692194101852e+22', 38, 4);
    T('7.58013927951538136e+823', '8.529027244983386603068574095481362831679e+4493', '7.58013927951538136e+823', 91, 6);
    T('-3.07436371e+1755', '-5532760.464680680187011119804409675', '-3.90906308177169496238485313290575e+6', 52, 0);
    T('-64.637341', '188271151.50', '-6.4637341e+1', 97, 2);
    T('8.111442780892164739792122e+83', '2.6577370144e+446', '8.111442780892164739792122e+83', 44, 2);
    T('-4.53193131053671927777509101240658963817e+462', '-5.877105925377111170254597440451933520353143020601259890354e+313', '-3.60754903492282737711552280024e+313', 30, 2);
    T('-56.17708223894564806846319939779528765159', '-90618.003696367493325', '-5.61770822389456480684631993978e+1', 30, 4);
    T('8.641266728803543394770548545539259580004484941073264254863e+93', '-3.090650539146630e+341', '8.641266728803543394770548545539259580004484941073264254863e+93', 92, 6);
    T('8.3472197183162077069e+974', '8.996986823958127051294560421258102e+5731', '8.3472197183162077069e+974', 47, 5);
    T('7.452667027651528204306450369e+5251', '2.3575e+3042', '4.6e+3041', 31, 6);
    T('1.3e+284', '-5.74279546149401884e+5537', '1.3e+284', 66, 2);
    T('-9.66415249360493e+861', '9.41464813629324e+83', '-8.218e+83', 4, 3);
    T('-3.5788238095e+15', '-4.7772446100344508198415000e+918', '-3.5788238095e+15', 80, 0);
    T('-1.18111260103608649601098854654e+39', '-6.794565975249168320420854605593e+215', '-1.181112601036086496e+39', 19, 4);
    T('-315581.6105161', '-6.27520168946210865978327597686925e+845', '-3.155816105161e+5', 97, 6);
    T('9.83', '9.997320437015236426558535100e+64', '9.83e+0', 78, 0);
    T('-484341.92324510018351210868626279443378578053883426246392', '-8.120757614171458363054212408916675325e+48', '-4.84341923245100183512108686262794433785780538834263e+5', 51, 3);
    T('-2.01156379295450881230496411380e+1225', '-8.89555688234223485702387053326e+1238', '-2.0115637929545088123049641138e+1225', 41, 4);
*/
    log('\n ' + passed + ' of ' + total + ' tests passed in ' + (+new Date() - start) + ' ms \n');
    return [passed, total];
})(this.Decimal);
if (typeof module !== 'undefined' && module.exports) module.exports = count;
