var count = (function plus(Decimal) {
    var start = +new Date(),
        log,
        error,
        undefined,
        passed = 0,
        total = 0,
        n = 'null',
        N = 'NaN',
        I = 'Infinity';

    if (typeof window === 'undefined') {
        log = console.log;
        error = console.error;
    } else {
        log = function (str) { document.body.innerHTML += str.replace('\n', '<br>') };
        error = function (str) { document.body.innerHTML += '<div style="color: red">' +
          str.replace('\n', '<br>') + '</div>' };
    }

    if (!Decimal && typeof require === 'function') Decimal = require('../decimal');

    function assert(expected, actual) {
        total++;
        if (expected !== actual) {
           error('\n Test number: ' + total + ' failed');
           error(' Expected: ' + expected);
           error(' Actual:   ' + actual);
           //process.exit();
        } else {
            passed++;
            //log('\n Expected and actual: ' + actual);
        }
    }

    function T(addendA, addendB, expected, sd, rm) {

        if ( sd != null ) {
            Decimal.precision = sd;
            Decimal.rounding = rm;
        }
        assert(String(expected), String(new Decimal(addendA).plus(addendB)));
    }

    function isMinusZero(n) {
        return n.toString() === '0' && n.s == -1;
    }

    log('\n Testing plus...');

    Decimal.config({
        precision: 200,
        rounding: 4,
        toExpNeg: -9e9,
        toExpPos: 9e9,
        minE: -9e15,
        maxE: 9e15,
        errors: false
    });

    T(1, 0, 1);
    T(1, -0, 1);
    T(-1, 0, -1);
    T(-1, -0, -1);
    T(1, N, N);
    T(-1, N, N);
    T(1, I, I);
    T(1, -I, -I);
    T(-1, I, I);
    T(-1, -I, -I);
    T(0, 1, 1);
    T(0, -1, -1);
    T(-0, 1, 1);
    T(-0, -1, -1);

    /*
      IEEE 754 - 2008 section 6.3
      When the sum of two operands with opposite signs is exactly zero, the sign of that sum shall
      be +0 in all rounding-direction attributes except roundTowardNegative; under that attribute,
      the sign of an exact zero sum shall be −0.
      However, x + x = x −(−x) retains the same sign as x even when x is zero.
    */
    Decimal.config( {rounding: 3} );
    assert(true, isMinusZero(new Decimal(0).plus(-0)));     //   0 + -0 = -0
    assert(true, isMinusZero(new Decimal(-0).plus(0)));     //  -0 +  0 = -0
    assert(false, isMinusZero(new Decimal(0).plus(0)));     //   0 +  0 =  0
    assert(true, isMinusZero(new Decimal(-0).plus(-0)));    //  -0 + -0 = -0
    assert(true, isMinusZero(new Decimal(1).plus(-1)));     //   1 + -1 = -0
    assert(true, isMinusZero(new Decimal(-1).plus(1)));     //  -1 +  1 = -0

    Decimal.config( {rounding: 4} );
    assert(false, isMinusZero(new Decimal(0).plus(-0)));    //   0 + -0 =  0
    assert(false, isMinusZero(new Decimal(-0).plus(0)));    //  -0 +  0 =  0
    assert(false, isMinusZero(new Decimal(0).plus(0)));     //   0 +  0 =  0
    assert(true, isMinusZero(new Decimal(-0).plus(-0)));    //  -0 + -0 = -0
    assert(false, isMinusZero(new Decimal(1).plus(-1)));    //   1 + -1 =  0
    assert(false, isMinusZero(new Decimal(-1).plus(1)));    //  -1 +  1 =  0

    T(0, N, N);
    T(-0, N, N);
    T(0, I, I);
    T(0, -I, -I);
    T(-0, I, I);
    T(-0, -I, -I);
    T(N, 1, N);
    T(N, -1, N);
    T(N, 0, N);
    T(N, -0, N);
    T(N, N, N);
    T(N, I, N);
    T(N, -I, N);
    T(I, 1, I);
    T(I, -1, I);
    T(-I, 1, -I);
    T(-I, -1, -I);
    T(I, 0, I);
    T(I, -0, I);
    T(-I, 0, -I);
    T(-I, -0, -I);
    T(I, N, N);
    T(-I, N, N);
    T(I, I, I);
    T(I, -I, N);
    T(-I, I, N);
    T(-I, -I, -I);

    T(1, '0', '1');
    T(1, '1', '2');
    T(1, '-45', '-44');
    T(1, '22', '23');
    T(1, 0144, '101');
    T(1, '0144', '145');
    T(1, '6.1915', '7.1915');
    T(1, '-1.02', '-0.02');
    T(1, '0.09', '1.09');
    T(1, '-0.0001', '0.9999');
    T(1, '8e5', '800001');
    T(1, '9E12', '9000000000001');
    T(1, '1e-14', '1.00000000000001');
    T(1, '3.345E-9', '1.000000003345');
    T(1, '-345.43e+4', '-3454299');
    T(1, '-94.12E+0', '-93.12');
    T(1, '', N);
    T(1, '    ', N);
    T(1, '\t\t', N);
    T(1, 'ertgrt546', N);
    T(1, 'qweqwdewee', N);
    T(1, true, N);
    T(1, false, N);
    T(1, 'e 4.3', N);
    T(1, '4 .3', N);
    T(1, '4.0 01e', N);
    T(1, ' 4.001', '5.001');
    T(1, '4.001 ', '5.001');
    T(1, ' 4.001 ', '5.001');
    T(1, '    4.001', '5.001');
    T(1, ' 4.0 01', N);
    T(1, '4. 001', N);
    T(1, '4. 001 ', N);
    T(1, '  4.001e ', N);
    T(1, ' 4 .001 e ', N);
    T(1, undefined, N);
    T(1, null, N);
    T(1, Number.POSITIVE_INFINITY, I);
    T(1, Number.NEGATIVE_INFINITY, -I);
    T(1, new Date(2012, 11, 4), N);
    T(1, new Object(), N);
    T(1, function () {}, N);
    T('0', 0, '0');
    T(0, '+0', '0');
    T('0', '0', '0');
    T(3, -0, '3');
    T(9.654, 0, '9.654');
    T(0, '0.001', '0.001');
    T(0, '111.1111111110000', '111.111111111');
    T(N, '0', N);
    T(-1, 1, '0');
    T(-0.01, 0.01, '0');
    T(54, -54, '0');
    T(9.99, '-9.99', '0');
    T('0.0000023432495704937', '-0.0000023432495704937', '0');
    T(NaN, NaN, N);
    T(NaN, N, N);
    T(N, NaN, N);
    T(N, 4, N);
    T(N, '4534534.45435435', N);
    T(N, 99999.999, N);
    T(Infinity, '354.345341', I);
    T(3, -I, -I);
    T(-Infinity, -I, -I);
    T(-I, -Infinity, -I);
    T(I, '-999e999', I);
    T('1.21123e43', -I, -I);
    T('-999.0', I, I);
    T('657.342e-45', -I, -I);
    T(I, 123, I);
    T(100, 100, '200');
    T(-999.99, '0.01', '-999.98');
    T('10 ', 4, '14');
    T('03.333', -4, '-0.667');
    T(-1, -0.1, '-1.1');
    T(43534.5435, '0.054645', '43534.598145');
    T('99999', '1', '100000');
    T('3e', 8, N);
    T('-3..0', 13, N);
    T('0 0', -0.4, N);
    T(' +3e0', 4, '7');
    T(9.9806, '+ 1', N);
    T(' +2 0', '1e1', N);
    T('e3', 4, N);
    T(' ', 0, N);
    T(323, null, N);
    T(undefined, undefined, N);
    T('undefined', undefined, N);
    T(null, null, N);

    T('593563412042.65071284077577238642546669796903116', '7399734315.5', '600963146358.15071284077577238642546669796903116', 78, 2);
    T('91834314419826105636913315.2250760351217688154427716325', '3453589484.0922087', '91834314419826109090502799.317284735121768815442772', 50, 2);
    T('1720025.190318830518434426408143', '66109246773146613017341021115.139667467395061272917480324885924044', '66109246773147000000000000000', 14, 5);
    T('84956243086401045.957714583212857676046952761360420789675', '236266946455107694023566236.1067', '236266946540063937109967282.064414583212857676046952761360420789675', 76, 4);
    T('27679002965950558720382323007580555707.1129725764', '3902452514422.1829293695210524511304537682971792', '27679002965950558720382326910033070000', 34, 6);
    T('9182914763445141913243843569172715469835.4885696227829197845538926', '7633444625534.6271670438', '9182914763445141913243843576806160095370.115', 43, 1);
    T('29495676.658886008145597040990327830', '6931766589765486564126938677173.788', '6931766589765486564126968172850.44688600814559704099032783', 97, 4);
    T('379110485.1879790330578106922187736430522', '44778134175412745601990207594724261558.24483', '50000000000000000000000000000000000000', 1, 0);
    T('294375205.06', '0.4210755138711679324480015392679141', '294375205.4810755138711679324480015392679141', 80, 3);
    T('97.8245', '678086929547037858952090.4586', '678086929547037858952188.2831', 47, 2);
    T('88488789676361292764625272698.8', '9635357185402290148967003271409315372896.608367362907029792956117646', '9635357185490778938643364564173941000000', 34, 4);
    T('6.6986623837182087397483631708665129676441', '4979636344219290763240.0069', '4979636344219290763246.705562383718208739748363170866513', 55, 6);
    T('70719500740711222218051791089542849911.838248343393159286401111269584528', '72611217359692246954.6315', '70719500740711222290663008449235096866.469748343393159286401111269584528', 100, 2);
    T('58732556421350640725226.234521198024002553600', '58121246202.94542710436381', '58732556421408700000000', 15, 3);
    T('3974581800977029548324180832.13384409524742094306099637', '3614381973858572.7418758814906985', '3974581800000000000000000000', 9, 6);
    T('697637766264066731981534075222456637269.8150', '84841485703909.9296471188463060', '697637766264066731981534160063942341179.744', 42, 3);
    T('47061.38', '1757281333528971368573.2805225778643923494834101231347769030', '1757281333528971415634.660522577864392349483410123134776903', 76, 2);
    T('1476403577722345877.206796162385539569704', '9942200550749890696264919346845234.2', '9942200550749892172700000000000000', 20, 4);
    T('12589380812542.2800574047701400763640862141749065', '6621312631.8566626344982182410585379', '12596002125174.136720039268', 26, 6);
    T('7882195982742109440258991594228896.97647548424454', '549984552171.2', '7882195982742109440259541578781068.17647548424454', 62, 3);
    T('79344567.9583146261700423456539738617142116267', '76770085769491324895202178331658.82467912070', '76770085769491324895202257676226.7829937468700423456539738617142116267', 75, 1);
    T('9803952036104667191144020833.60300205', '9495298573633466486861.2007267111985', '9803961531403240824610507694.8037287611985', 91, 6);
    T('42717742030595859.2062217089376928894751002693', '4392.7633096899809016584837135701339842639427', '42717742030600251.9695313989185945479588138394339842639427', 84, 0);
    T('175862554420496917507.72', '11748733600781956.625570241067964572154062958', '175874300000000000000', 7, 4);
    T('388528785785093377176597943386.8074346502385319459876897943393772766', '780579558740664211027356329768421.54760979278144', '780968087526449304404532927711808.355044443019971', 48, 3);
    T('64154311030289309886070986793918832.391977325870878', '85034840868876941709096163143445.05236', '64239345871158186827780082957000000', 29, 3);
    T('558109875620.787158', '4318677092534293169485.05154202819301085696282200030', '4318677090000000000000', 9, 6);
    T('8.1572164344501210656681193', '141383742681203268175705548539.5903424774279108405668653', '141383742681203268175705548547.7475589118780319062349846', 87, 1);
    T('475020553895752975125109317246970868141.19586602169742108309356725817022171', '50527762765152355076523195636814.169841064002562503683522141227541614', '475020604423515740277460000000000000000', 23, 5);
    T('623051342929926112359884960895415731939.5562490236742100882954960168', '503245645050146765988526273.758561914511343318476865211', '623051342930429358004935107661404258213.3148109381855534067723612278', 74, 6);
    T('531029963832660460416202620349662236.9', '1698435814148807364109817.622700275250303509772194494494138982090', '531030000000000000000000000000000000', 5, 4);
    T('6222.818987', '2745.84213504008632396977994287249342', '8968.66112204008632396977994287249342', 45, 5);
    T('83277468481757396400448039342892.13637251061', '949622766461448278187285221580222572.50707178970245998726148766810515863', '949706043929930035000000000000000000', 18, 3);
    T('50515084033799949576079332969.310908542889109621643886016029143', '9995275496554.2256723021298', '50515084033799959571354000000', 23, 3);
    T('58075371184786989216823527114006.8193336921861462723001823062864617438', '831577.37480626', '58075371184786989216823527945584.1941399521861463', 48, 4);
    T('354334.661931489528109', '49318851855669319.7668000614644666331306269353669354', '49000000000000000', 2, 5);
    T('171792414269.20500878', '7228500834975.6421662524366409', '7400293249244.8471750324366409', 86, 5);
    T('3686865567196977754914570753132702.86443144944286374816106555429', '252977292743732124354509767399171006.56792658736276850447471854354498', '256664158310929102109424338152303709.43235803680563225263578409783498', 76, 1);
    T('7772413527479.0292045567366574215506978466', '6951039673098790484081676762223797578.48972170861559580041987995', '6951039673098790484081684534637325057.5189262', 44, 1);
    T('897519.488463768126475213360003650464', '243345915368302813437999412228446907.03319139912318828693096', '243345915368302813437999412229344426.521655167249663500290963650464', 69, 1);
    T('9944701104687189448523634989441.1145387693210320276079647', '76461573432179845156280348475771563.06533912678377627209249', '76471518133284532345728872110761004.179877896', 44, 1);
    T('49868358282915444463601511745776323.6756', '426.127124853', '49868358282915444463601511745776749.802724853', 62, 1);
    T('462611390833337670596267664076995136589.8078310066750996553090298147792146374', '86220896429616059447274947806526227.378336884633', '462697611729767286655714939024801662817.18616789130809965530902981477921', 71, 5);
    T('3082844058.9630699111683783220405718767649310463', '9821678552282552.46', '9821681635126611.4230699111683783220405718767649310463', 85, 6);
    T('968956231687314239915639.8479448896945159', '2013053560776294882507697425832898126329.553542945365269900429', '2013053560776295851463929113147138041969.401487835059786', 55, 0);
    T('18829358460454562606796373222566693.8406444934478081754024757172', '245696827314924767344.62999383040940', '18829358460454808303623688147334038.4706383238572081754024757172', 65, 1);
    T('87272640829480092614399030.17403303652178770296488752252', '80645627676002739119663787.1389295', '167918268505482831700000000', 19, 4);
    T('110333142216083345526934754242807378414.8112810978', '8009375616663973267889.632407900829561884426774123507672', '110333142217000000000000000000000000000', 12, 2);
    T('7306882124433855860885172398759699317308.33181885682904', '7340.5772269222099866925020556183', '7306882124433855860885172398759699324648.9090457790390266925020556183', 73, 0);
    T('451443519.5', '40545287685504171383686533617478.0379433921591774655032923922180', '40545287685504171383686985060997.537943392159177465503292392218', 85, 1);
    T('9103707265251.90817165947074590284', '7132.1726911109784489', '9103707272384.08086277044919480284', 57, 1);
    T('1327536619868001635732671456652796081.82503453353162015625727828', '777720909904.3138244842545463', '1327536619868001635732672234373705986.13885901778616645625727828', 94, 6);
    T('94785498339319783.701937', '5.5284513', '94785498339319789.2303883', 45, 3);
    T('413864849197532888901858875400764791296.16643089', '1079561.227318262349', '413864849197532888901858875400000000000', 29, 3);
    T('45087586598522825025991129716000.35589454049730836', '85381176658152430597133806358376750.386015723730182079339361073643', '85426264244750953422159797488092750.741910264227490439339361073643', 95, 3);
    T('1.577', '5867361133548635578508631112769.512', '5867361133548635578508631112771.089', 37, 6);
    T('611867270001089057.5416980', '928462251839038030221601836154.34932586048493472075', '928462251839649897491602925211.89102386048493472075', 83, 0);
    T('2420842674752.5', '95330664601887717505762272.986252619687512204447', '95330664601890138348437025.486252619687512204447', 89, 4);
    T('81132109324361724111675518092.0430089148033907345892402992266', '56280915849510057765284537881333571.97531', '56280996981619382127008649556851664.018318914803390734', 53, 1);
    T('904671681146500577093476684800344905.4695', '94797652292709883543386718515254224988.65582754951556889876984', '95702323973856384120480195200054569894.12532754951556889876984', 82, 3);
    T('7088106609362197643328448383484980.038959814107063899', '670949858005717984030520198273540550.3057', '678037964615080181673848646657025530.3446598', 43, 5);
    T('99.21410642', '9083.34303307666325286270', '9182.5571394966632528627', 54, 4);
    T('3283.8936708900357134342', '496435946.937228974879461', '496439230.830899864915174435', 27, 2);
    T('676830762646205126.3112659907032497497725497603419138', '119932542098343475889901925878.775805518069408', '119932542099020400000000000000', 16, 2);
    T('340784617186492559520867952988798537.2875804', '619036430511269842219006698087.3401406612', '340000000000000000000000000000000000', 2, 5);
    T('173.3190299633996648505', '1824690614717784789521361964710.12123119395', '1824690614717784789521361964883.4402611573496648505', 96, 0);
    T('482415314400600371304757933326735596247.502', '64121617419177033485421455407.5858664539051557486457828966476314714', '482415314464721988723934966812157051655.0878664539051557486457828966476314714', 79, 5);
    T('375737470927645435406243965552430202253.631955167073623', '6338230241298196815836217859672.793', '375730000000000000000000000000000000000', 5, 3);
    T('29573885064956243070331189.3797092614486', '93819837825028613877.417105', '29573978884794068098945066.7968142614486', 96, 6);
    T('138120.700190473452', '968621311075606384.99548987853755995993', '968621311075744505.69568035198955995993', 45, 0);
    T('973.11990', '42154692435544969993439766.300521', '42154692435544969993440739.420421', 58, 5);
    T('554388861256.869722940300672388196938972302318081540', '1155262184508008.03323100857', '1155816573369264.90295394887067238819693897230231808154', 79, 2);
    T('3416418236.26400558205105933', '5.6479733', '3416418241.91197888205105933', 38, 5);
    T('2523992943408057896347889018.4465701438462144584', '506142712311227094740144.824697116', '2524499086120369130000000000', 18, 2);
    T('308403067969970653523590678072543444693.024992850150499393849129', '732241040738289252979020347.308655416021872045866556456848136589', '308403067970702894564328967325522465040.333648266172371439715685456848136589', 93, 1);
    T('3239064136731151.02891710688580557275640729053', '9083206824661083.5207932479503224412', '12322270961392234.54971035483612801395640729053', 80, 5);
    T('475331426870705.089157016478807', '9009215435143577267843833230423.14449365133718648192722331279072569', '9009215435143577743175260101128.233650667', 40, 3);
    T('105248227507845582838.20680935912346655579395894726346139163', '65223604191.5805291176792', '105248227573069190000', 17, 6);
    T('99.215190820118104610069782046', '16578.8209964235031908525606222947444696', '16678.0361872436212954626304043407444696', 63, 2);
    T('510539424488693733.41537434134721968131486', '327009.1496755290899995046724660116828516732779', '510539424489020742.56504987043721918598', 38, 1);
    T('315204602.81489858995', '3732180545.09', '4047385147.90489858995', 86, 4);
    T('5314592166616.1', '973281963604549959893809697900212092.07971879384048258785317865', '973281963604549959893815012492378708.17971879384', 48, 4);
    T('8.7166935890371592992002394888141553196926', '958659296273258091217856839735061.3647474524043063555717437013816', '958659296273258091217856839735070.08144104144146565477198319019575532', 69, 4);
    T('8540337154693663138.922681853498773232162632659387428466959', '714421549038288078923295717348706283749.09747291', '714421549038288078931836054503399946888.020154763498773232162632659387428466959', 96, 3);
    T('64218.4910654347361732', '1904584344376749176156995834410373768727.880243614504617078941694238', '1904584344376749176156995834410373832946.371309049240790279', 58, 2);
    T('79608939157757406640105379.1099825541008835456051880', '93.887979844359072766498', '79608900000000000000000000', 6, 4);
    T('997619509.8131634548180432', '408723341842195084378669.60436714294200657842422937736174', '408723341842196081998179.41753059776004978', 41, 0);
    T('581375185201895127979063.3148256', '14356334345.7574898', '581375185201909484313409.0723154', 33, 1);
    T('3527973624305181687800039808864.5573554755188', '12630611136837256867366017776380.0', '16158584761142438555166000000000', 23, 3);
    T('367630489717480484892012992734563672974.0', '45054048927965278885126985834716.66', '367630534771529412857291900000000000000', 25, 4);
    T('787513869174.71308620942912467', '23093894176823.452215801790110947022416582614', '23881408045998.165302011219235617022416582614', 69, 0);
    T('232517364185108600.0071492246367584299210401716808', '475775741395276794336919480763372.333394', '475775741395277026854283665871972.340543', 39, 3);
    T('4761258811.285130277020712133991031466866', '809651374274492903891681559142250310615.5', '809651374274492903891681559147011569426.785130277020712133991031466866', 83, 1);
    T('955589075153262998656048550747976309.25', '323944677033577864745438281583.6', '955589399097940032233913296186257892.85', 66, 0);
    T('9873059325000911597710502538051178.95608276083955310078852781053798025', '7270631337435350709738948.04009317786010295132783176444810', '9873059332271542935145853247700000', 29, 1);
    T('68390202874327257090574073967950908.9399497615001880819136518594963506449550', '50275784013286654621602.576089694107949935905295179', '68390202874377532874587360622572511.5160394556081380178189', 57, 3);
    T('3117495192110092176499956360384398018825.4566880671968015782324609875979940805', '547835092734852702707412331700.58677624372569659526993791784030', '3117495192657927269234809063091810350526.0434643', 47, 4);
    T('5512.13534226023315629689397133730', '294458821948239231568878814150650575760.02041102096', '294458821948239231568878814150650581272.1557532811931562968939713373', 76, 0);
    T('57753452879494271824903590.413829378140475084396185', '50545413776472197781.9967870405', '57753503424908048297101372.410616418640475084396185', 58, 2);
    T('17236192031420.2163137961486', '692838850166456533709637044970916752.36688025752892121241', '692838850166456533709654281162948172.58319405367752121241', 68, 0);
    T('4171050007467938807033605772358739230.9311059803557030664429028', '0.1923799694080', '4171050007467938807033605772358739231.123485949763703066', 55, 5);
    T('28478160393.9806', '277120391846395984.25', '277130000000000000', 5, 0);
    T('3061847804826051245630.65596944768453555988908615722', '40580833038231825168334767272165347.842548699388', '40580833038234887016139593323410978.4985181470725356', 51, 5);
    T('4307943411739045.1991060215780', '2498934952289.676303798694', '4310442346691334.875409820272', 100, 2);
    T('447264.0', '296280025996717104124065506095605789.16501275', '296280025996717104124065506096053053.16501275', 76, 3);
    T('7774084135404146930548457363682921350.47767504567574932067192218196557', '6356426099126393712.50823145833574225514684676449194435', '7774084135404146936904883462809315062.9859065040114915758', 56, 3);
    T('772351722055561484188074604225369559844.02', '4826191208867.1814570305', '772351722055561484190000000000000000000', 20, 4);
    T('252673622.185680951859328', '14604.688310802671654124022197881135913861968', '252688226.87399175453098212402219789', 35, 2);
    T('9844584158.071870640', '7376636821819775622965030201208199.7842455984620137382448558190', '7376636821819775622965040045792357.856116238462013738244855819', 67, 0);
    T('7402851913927.841295065397846204951008', '3095919939427099869935455000158274400904.445835412', '3095919939427099869935455007561126314832.287130477397846204951008', 76, 1);
    T('39454068592.5426357674', '539500064406478650256867488.44171988564589', '539500064406478689710936080.98435565304589', 78, 0);
    T('711378057536130546.05030965850391817985', '3519.9006655104107', '711378057536134065.95097516891461817985', 76, 0);
    T('4961172832120828815580484602760756.2012712908', '672041039664151754962281595761407065569.95071013621060637157419754', '672046000836983875791097176246009826326.15', 41, 1);
    T('7101.2850399893410837385161594686012015', '7447039100693496598.27485', '7447039100693503699.559889989341083738', 37, 3);
    T('4017400855744231199234363.40', '8680854252488567594955443.1697318101148721333039110519819430', '12698255108232798794189806.569731810114872133303911051981943', 91, 4);
    T('2059590390363552.40', '586325932054905969708260929.92327440514351633822636210224317619490', '586325932056965560098624482.3232744051435163382263621022', 55, 5);
    T('582983617.569929797014256654694471', '9981786309799768994686892934445.9295301169296065462066657484', '9981786309799768994687475918063.4994599139438632009011367484', 68, 6);
    T('2.6165698405910796049261368090493573329246', '1760827779548.1353871194941345580463266976978', '1760827779550.751956960085214162972463506747157333', 49, 4);
    T('42513713793037649412.20337267935799701850122725370100', '4884164426571880743457.9060113', '4926678140364918392870.109383979357997018501227253701', 88, 4);
    T('615013136392771057432168892493351121.844', '7501.90719259576291230', '615013136392771057432168892493358623.7511925957629123', 56, 1);
    T('439780502800870192246198170895028338.75503651737499', '54139159349728010041929589316148734373.238719628804657454831809670308', '54578939852528880234175787487043762700', 36, 1);
    T('2500282.56691918225466823870', '4468153172374579202195051.1192110124701356844021658', '4468153172374579204695333.6861301947248039231021658', 70, 3);
    T('3947906778978419575322273.5922', '527753211579218151805229658066345458285.5885089788593758900780037', '527753211579222099712008636485920780559.1807089789', 49, 4);
    T('224244602386820798357422068121543275.9722790644', '432961.0031220586705516514763153277784', '224244602386820798357422068121976236.97540112307055165148', 56, 4);
    T('41074785256868066207935.9658329758100047703778552011774401118298', '996313038437469337782350067491224840574.3439865837306077524137342817475940790588', '996313038437469378857135324359291048510.3098195595406125227915894829250341908886', 87, 3);
    T('78270192113308576879.230', '422705378961250342057111974970896025870.232502331134017349978026', '422705400000000000000000000000000000000', 7, 0);
    T('23154961663482879829897.443500', '49041587603060876.96', '23155010705070482890774.4035', 84, 3);
    T('612963.737191014', '2731304753434157847102688238900689.5878194', '2731304753434157800000000000000000', 17, 3);
    T('79.410900071205868462305254', '498855037827626504175503191449.252294529388770858879482', '498855037827626504175503191528.663194600594639321184736', 72, 2);
    T('6993101104301088.970619872035600302449779353214753901', '509465802045548078458021.19126386538627494681427019648867', '509465809038649182759110.161883737421875249264049549703423901', 90, 5);
    T('3173346.465280699493106096344514997257165695887', '25094425785458254607988571321.6118249859981254347581599952737191037300', '25094425785458000000000000000', 14, 3);
    T('4825986092816553.7635954902708301975634371809900', '6932262398609128466540251295633274.5129783804352371207490', '6932262398609128471366237388449828.27657387070606731831243718099', 87, 1);
    T('926199681466567517287973.10295', '228084522784513379301529587994.866786459974', '228085448984194845869046875967.969736459974', 49, 6);
    T('451757469368611461226744.675918562388474905198839780224990', '152843853624419159800008131.88722', '153295611093787771261234876.56313856238847490519883978022499', 60, 2);
    T('6779555902177154177229771584.099785903065', '7705328232185839682426202564296087172997.525840974954568455125376057221984043', '7705328232192619238328379718473316944581.625626878019568455125376057221984043', 86, 1);
    T('9536.995381842', '5743683556272978375327341031353872992805.228035142323187697966003805355354634434', '5743683556272978375327341031353873002342.223416984323187697', 58, 3);
    T('794180405697345273302591220141629717.0335032652959', '79253739255343474198020799617825451.0', '873434144952688747500612019759455000', 33, 4);
    T('7150644601128617400192415586930354936122.6334767464', '9109635205088641816430727590858041637.2704471583622041768929760514400617734', '7159754236333706042008846314521212977759.9039239047622041768929760514400618', 74, 5);
    T('73562280122283.3504585511130', '9831433225592360462.8352635', '9831506787872482746.185722051113', 98, 4);
    T('8956158610374625.509531016', '326445046650.0', '8956485055421275.509532', 22, 2);
    T('74356614405271046021497883758184078570.45526096329466906744072230232', '250086499545948148713695351897361487.2424378528824906540', '74606700904816994170211579110081440057.6976988161772', 51, 0);
    T('54438066500841374310337535135292.9180799710380482218974913', '93583236200855.457936825513836204', '54438066500841374403920800000000', 24, 6);
    T('992.10612', '1853924775.3578645099684579616948779829033812', '1853925767.4639845099684579616949', 32, 2);
    T('186864386191178904666377352418476.9026', '324887104993.09478140653570710242809082', '186864386191178904666700000000000', 22, 5);
    T('73975703304587932639.2181626587592846793346557', '385064.7711364633520856985649145914899', '73975703304588317703.9892991221113703778995702914899', 65, 1);
    T('444492371187821986647273.357556318861835197', '32419538466212897.7682790461375', '444492403607360452860171.125835364999335197', 90, 5);
    T('425.36', '4.8163620735655971354915724851187759884583', '430.1763620735655971354915724851187759884583', 58, 3);
    T('47140705104264.633543653650963362791573755527599', '5038770685966504.041420875593555678620', '5085911391070768.67496452924452', 30, 5);
    T('7680406.950844967098906022738005995808', '7170039143571111215189.2720128', '8000000000000000000000', 1, 2);
    T('56766742609.4324995621156186543537021757622234796', '5600154020118.606591501', '5656920762728.039091063115618654353702176', 40, 4);
    T('570657301559247483807209644.19', '347169349198778409985.34839429745774918204191942456937431', '570657648728596682585619629.53839429745774918204191942456937431', 74, 5);
    T('5361772037663933234405322789036276.33156433900413166799767144681154137323', '286704590022711173.40544742389525', '5361772037663933521109912811747449.73701176289938166799767', 57, 1);
    T('93732046861555957196550772609968.3472471313', '1859083952987593674898181.79957615855993107', '93732048720639910184144447508150.14682328985993107', 91, 3);
    T('92567721714303533.992771142280786530426395726718479', '4173646454390713911135550874061129.722450828911285695255462583174414', '4173646454390714003703272588364663.715221971192072225681858309892893', 98, 5);
    T('1375128263616714701.94878254650062', '46092727285436763451586.053119100', '46094102413700380166288.0019016465', 33, 4);
    T('73619022388103118.782952924468', '812143.274917215669264759', '73619022388915262.1', 18, 0);
    T('5863128081.032078098051593954', '3401644100547973363487331808740.559267563791452260218198', '3401644100547973363493194936821.591345661843046214218198', 76, 2);
    T('2035229845149050434432862.3913626', '324888035708707633659073773920117.1589359337264063', '324888037743937478808124208352979.5502985337264063', 82, 3);
    T('3809229847420761545133.0', '4820445969156504840260574583039569681560.574245660256', '4820445969156500000000000000000000000000', 15, 4);
    T('265038662972437534963025477565907689803.0955954524873', '8689727198065.214479051254116375289593', '265038662972437534960000000000000000000', 20, 5);
    T('467620089714294461431888766085.933012457041824326', '8454761303295998080511054.0533223', '467628500000000000000000000000', 7, 4);
    T('87894879970166522727794755.93237106514121963', '9.21735981924330694503993149149554469907', '87894879970166522727794765.1497308843845265750399314914955447', 61, 5);
    T('415465265615530648920289751139098887220.642143071162972307015579919362', '784797421468789423953053292044037319012.57789662938605515623570039453017199', '1200262687084320072873343043183136206233.22003970054', 51, 3);
    T('919416082026906893949479156604880336.255538841432616135466943633799', '786230185237.5400725225964740467962175280141', '919416082026906893949479942835065573.7956113', 43, 3);
    T('652545610682383466438682204.42146681125712', '90000817174099471.37775277', '652545610772384283612781675.79921958125712', 83, 1);
    T('834893074961901.316219159', '3494462.496814', '834893078456363.813033159', 31, 3);
    T('30955320958921614617178138289424354927.94679217117720836736847', '9343585893221.056', '30955320958921614617178147633010248149.00279217117720836736847', 93, 1);
    T('90487985719811418246283472474823.257227209130594638697376', '91288470132046108165996.51806996', '90487985811099888378329580640819.775297169130594638697376', 97, 3);
    T('8399816940955101.576691696184560166134', '1425122.40715', '8399816942380223.983841696184560166134', 73, 3);
    T('90835019536.019916982404274694597500578', '716647297150731356312375253472992368.5862189689245116804145741973840791708889', '716647297150731356312375344308011904.6061359513287863750120748', 61, 2);
    T('1.76835213775438870258717627564722245975', '7006467747024962494048481730509926588025.86559434478599359798030950728698427731', '7006467747024962494048481730509926588027.63394648254038230056748578293420673706', 81, 0);
    T('928715435770769947371.846861119120186848130047779221215181888', '54176632694146863536.023546974', '982892068464916810907.87', 23, 6);
    T('2592460567145.9081671605659594580786953609908858698', '7864631589857045160739640205084628.0', '7864631589857045160742232665651773.90816716056595945807869536099088587', 69, 5);
    T('550167718959711561547.0354625', '452302.08587611743779', '550167718959712013849.12133861743779', 53, 5);
    T('2012913.385532423507759516640479441', '103518982004003013118420061.943654190758892648', '103518982004003013120000000', 20, 5);
    T('49943050945228733544.6734986147', '368220952556842481.13025278166305776823174166647', '50311271897785576025.80375139636305776823174166647', 71, 6);
    T('6819435979706516062524.6356313748', '61269304801758183.91645059614218918999', '6819497249011317820708.55208197094218918999', 95, 5);
    T('7233786224735497889984695259.844810632657377449889809221767092', '6807990850759856236367040269664178400670.841947660632059789845', '6807990850767090022591775767554163095930.686758293289437239734809221767092', 92, 0);
    T('93353199365969854.9000648459972051740723', '9422.3888765011593114515314770219001723580808', '93353199365979277.2889413471565166256037770219001723580808', 78, 2);
    T('30980262.05713034537675', '778.835454408', '30981040.8925847533768', 21, 4);
    T('32615753444771455868335024685114814.9238176568928810345574', '7768512679320024.55715331942029263648769217', '32615753444771455876103537364434839.48097097631317367105', 55, 2);
    T('13632097595691451297658166154832.143013272134363070648360851205244642', '338823008282322586814270.072279678874150483663858', '13632097934514459579980752969102.2152929510085135543122189', 57, 4);
    T('6422084834544364179504578.997091732441389646011113', '749563746146212443758110.4060254814560251104', '7171648580690576623262689.4031172138974147564111', 47, 4);
    T('524628228909.16050485473888934', '157911548505.727158203964341656616832412311819792', '682539777414.887663058703230996616832412311819792', 95, 0);
    T('119792544.57876596369349895973', '27424886476597422405011090514661205.0', '27424886476597422405011090634453749.57876596369349895973', 95, 0);
    T('2069402156492526629947970885.71888875649933', '9983225233803399851557109.1989727', '2079385381726330029799527994.91786145649933', 81, 3);
    T('469964156809.608491', '595074692464717339784472687248240220.861973', '595074692464717339784473157212397030.470464', 82, 2);
    T('493360767167099894326435423534395819518.19695281187005', '7766465503.425829049644163982004', '493360767167099894326435423542162285021.622781861514213982004', 99, 2);
    T('99936880718101922830812712583.65032262815572509426570474', '33036974375228768881250337405734899.3544983736263577509194755440416', '33037074312109486983173168218447483.00482100178208284518518', 58, 1);
    T('7938089.9035748249886034322', '59938235780340155825294510015684098915.6306313894846960', '59938235780340155825294510015692037005.5342062', 45, 5);
    T('12020912075170693462106301094099.55717548283295340940081', '7925012204520701892409967356193913171038.423047370', '7925012216541613967580660818300214265137.98022285283295340940081', 69, 4);
    T('2403534785699117112878638516815488696057.917378367470941297385927', '6382502380424160003.479677660153263338013088885', '2403534785699117112885021019195912856061.397056027624204635399015885', 85, 1);
    T('698965328139372391512502.0393628796146', '349.3774499346277293528646834', '698965328139372391512851.4168128142', 34, 3);
    T('4557454883.68536507001647907979028423', '461212.6999283496597694751582137548', '4557916096.3852934196762485549484979848', 77, 5);
    T('4863433553099693015144122064473263537.90580375285849442237159037230012244', '729921864795343627150.6944153828593595441', '4863433553099693745065986859816890688.60021913571785396647159037230012244', 88, 1);
    T('8341821006482531885092.98411549285365', '3649787.9056678809068225979753383', '8341821006482535534880.8897833737604725979753383', 57, 4);
    T('50900271502855123535389576514508.6', '3529291535644.0', '50900271502855123538918868050152.6', 68, 3);
    T('65048735011187538001618360.6190283754497263475739381', '544369607320210256967259524797.5031', '544434656055221444505261143158.12212838', 38, 0);
    T('9980542928.586', '2.109934', '9980542930.695934', 55, 6);
    T('4691011429542960320268638161612830.142849169', '757715397475477414366.06830046', '4691011429543718035666113639027196.211149629', 83, 4);
    T('9874.54', '3856064270543742684218503.9123367482159408', '3856064270543742684228378.4523367482159408', 87, 6);
    T('68154879448120209877595453.386110402115092955259472912357', '27424826983.14981', '68154879448120237302422436.535920402115092955259472912357', 65, 6);
    T('37249053070475232499173177378.031244401', '907464305952.1105379598699518977559941948581426', '37249053070475233406637483330.1417823608699518977559941948581426', 65, 6);
    T('41.155993138147253723002006646182', '780464.5000578752798', '780505.656051013427053723002006646182', 89, 6);
    T('96920877597978100716777218.316958009791452354794880717409', '3188642196779775872022.94066144525722263195417766', '96924066240174880492649241.257619455048674986749058377409', 71, 6);
    T('816671190796023.957', '756549967023185081275882729414.639883423689748974601796203858786', '756550000000000000000000000000', 6, 6);
    T('3076171322682840.90303474815851099552968479985931936', '2787757192.94', '3076174110440033.8430347481585109955296847998593', 47, 6);
    T('2848944214201036899739.56442582032264708210474810029831860', '26670925074193.56548167869110309938557', '2848944240871961973933.1299074990137501814903181002983186', 90, 5);
    T('948894436900441147561922778096850998.025997593986', '443727475506008.5697805641108323511255902936', '948894436900441147000000000000000000', 18, 3);
    T('399834116109759859225128.86903386714268063275622585130318202065', '83243301579.66435004343194655785121635608927271', '399834116109843102526708.53338391057462719060744220739245473065', 63, 1);
    T('858875986652204266383404217992.291603983048765141', '6.0926465541742680273373988448676561', '858875986652204266383404217998.3842505372230331683373988448676561', 77, 1);
    T('776263916931670286731399.503267531195', '210175752246089961220452910504105327677.1086634983558182634326666828', '210175752246090737484369842174392059076.6119310295508182634326666828', 82, 5);
    T('8718333630210.37077854', '37817829567644957230084174.76767663', '37817829567653675600000000', 18, 5);
    T('175798484157966149637348754749.631917825059', '191069.39198', '175798484157966149637348945819.023897825059', 68, 0);
    T('71094655118453771005757530841.0396931519906710695577045349011371640774', '940850879022443685735297736247770183.1062859472086874437627996457041', '940850950117098804189068742005301024.1459790991994', 49, 4);
    T('55771.4302072966318037483204', '5074951905603164297.9', '5074951905603220069.3302072966318037483204', 43, 1);
    T('97573612523925803.471300446034721965973356', '884547563142872953757144.226968283178396', '884547660716485477682947.699', 27, 0);
    T('534430489256092354910779169321445983.67571072478274', '8134878658.8874328', '534430489256092354910779177456324642.563143524782', 48, 3);
    T('21752977621.53545695744', '9.27129494782115597264112710202', '21752977630.80675190526115597264112710202', 87, 2);
    T('9932777897276065862443227653669365455342.97666674233819577', '9658865807142952156279.44105819698310', '9932777897276065872102093460812317611000', 37, 1);
    T('123376130230056627372753581526479.897520335045821', '210087514417423605269213204478329905.5000222444600122718', '210210890547653661896585958059856385.3975425795058332718', 65, 2);
    T('69.90142090849675528097876785026473472', '79834.1060979095860549730563533311594891', '79904.00751881808281025403512118142422382', 100, 3);
    T('55295146117469331485301.39276631674751989526', '7152150280497482.0881738719503196277', '55295153269619611982783.48094018869783952296', 100, 0);
    T('2936.4729089310337543432927171', '592041503613529215374762022103.93452080904168592', '592041503613529215374762025040.407', 33, 5);
    T('3543847956755001.889', '35208784086977202.003447129500893889382686251144250', '38752632043732203.9', 18, 0);
    T('210.12201651568251832101964897378268382', '34.753746', '244.87576251568251832101964897378', 32, 4);
    T('88716032.541028216', '7257777720664327957948893098939678.91443795578792936', '7257777720664327950000000000000000', 18, 3);
    T('44129675251569160650555860751984.44296857695163101', '567695.63279516908886789079257067', '44129675251569160650555861319700', 30, 6);
    T('83283897197333851642528226761.317486565324', '1549.2573908490966060607', '83283897197333851642528228310.5748774144206060607', 73, 2);
    T('6996221992337457337.85327025', '667897527431759884729458692764632.058245604034043675214', '667897527431766880951451030221969.911515854034043675214', 100, 3);
    T('922619961162.2208', '47542640152456.2642306344645126128579941', '48465260113618.4850306344645126128579941', 98, 5);
    T('611397868296.756734797071956883451847', '196585892481617652694421837845193747022.4082193499020', '196585892481617652694421800000000000000', 25, 4);
    T('877868949660161908540.37474593654374', '9263.734978454268710907848665', '877868949660161917804.109724390812450907848665', 79, 4);
    T('668028.0', '80293619463333150.662604016585067202571976571183668062', '80293619464001178.662604016', 26, 3);
    T('742255892921.08552278588490', '92947236971652934320.381863823954476980190144705805612087685', '92947237713908827241.46738660983937698019014470580561', 52, 1);
    T('588640312892042.5229623908713504345901589967711144', '238642085600739223534691983689116795270.9392476910711070478806933690607710023', '238642085600739223534692572329429687313.4622100819424574824708523658318854023', 84, 5);
    T('693483.30191418214', '142589882557.20048662638259170691', '142590576040.50240080852259170691', 57, 1);
    T('9117137093.74844262790935196293617898088598527612', '73795973679442547725112.33063226581350299943451760384', '73795973679500000000000', 12, 6);
    T('4.526207787', '494677.2546106854088', '494681.7808184724088', 48, 1);
    T('30748018.2039098027620849591479954965780808709', '6852918.752696887318295520093985973', '37600936.9566066900803804792419814695780808709', 45, 1);
    T('83757651224762459585278205.84804608223565851299630130742301927', '7.64047420843', '83757651224762459585278213.488520290665658', 41, 3);
    T('975947153367486.28236437261704896554620492', '43249595796844818325786438957.99323092408', '43249595796845794272939806444.27559529669704896554620492', 68, 5);
    T('11.441690234434150722494554227811313', '875817973566587174821389748.1222354707915361953423889', '875817973566590000000000000', 14, 0);
    T('16958.2140542', '788926587.3973035876122169863973821', '788943545.6113577876122169863973821', 86, 0);
    T('417606670852399308849133225.281250676092118', '381.17737', '417606700000000000000000000', 7, 5);
    T('5801004654143445689815580111944080.0', '5042.588293662243775410837328482', '5801004654143445689815580111949122.588293662243775410837328482', 88, 4);
    T('9458108973104955501.19731713109763438574297', '25483717936.348437947448383180705503437809197004644', '9458108998588000000', 13, 1);
    T('599950356499300529789528173.8770713', '268880.548220513', '599950356499300529789797054.425291813', 43, 6);
    T('606420737207328260.923737', '2793960799108258019951495192.38660756661', '2793960800000000000000000000', 9, 6);
    T('537996460782034925311563841713892430891.857623392671766153564', '71846410869540749.3122050323622539301930763629', '537996460782034925311635688124761971641.1698284250340200837570763629', 95, 3);
    T('80855500804550283711094768191389.4531358771', '224613017765431377571817090000565541368.774524616422152973', '224613098620932182122100801095333732758.227660493522152973', 58, 4);
    T('27241652.017420652090726110', '46876731265932806262337193330223860.2863772593893330', '46876731265932806262337193357465512.30379791148006', 49, 5);
    T('703120.391502363183514079067', '79671.011', '782791.402502363183514079067', 53, 2);
    T('2946391497.026572366841464125208813086920', '189111065852431753063091488548.6924212587703', '189111065852431753066037880045.718993625612', 42, 0);
    T('70423417262.00132385773544766159557', '73526366752549062277219085279.1389002553495680928843377706747975638548', '73526366752549062347642502541.140224114', 38, 2);
    T('44343536119449484.887983227307430084581470194803169', '687286972712382711.817290490656982837', '731630500000000000', 7, 1);
    T('70666678103509980014.16613492442285597381', '7718081682856158711304.0858624585029', '7788748360959668691300', 20, 3);
    T('190892658814671027948020798562.66401', '3077914596519843221.11397426172', '190892658817748942544540641783.77798426172', 87, 1);
    T('857699.4648', '215172698578303.0', '215172699436002.4648', 30, 5);
    T('71367336317503469996089975313.78956860568717151126807150128475', '6004790033798702211031253178324995.49124913354575889902983817', '6004861401135019714501249268300309', 34, 6);
    T('5738.75872446155', '29858473503717618435783009538061.2367254587951510324891489061', '29858473503717618435783009500000', 27, 1);
    T('65523975946403470734536.20026566994471', '38814.767794272562047617', '65523975946403470773350.968059943', 32, 0);
    T('1009099102.7851258876510730342999502734', '81254320103901710.149545407089813758', '81254321200000000', 9, 0);
    T('0.80914', '45600517407671269885.9496812081291114319857829', '45600517407671269886.7588212081291114319857829', 46, 0);
    T('210699773499290371305723185122.758275542646367986145982574141331', '55837310555101717797702096605946323.1231534464473', '55837521254875217088073402329131445.881428989093667986145982574141331', 72, 5);
    T('46692439158758833041970355457275102904.33760089680805727178230', '5100.209160709238112205230327', '46692439158758833041970355457275108004.5467616060461694', 54, 1);
    T('6579188486857662760325.07263745375372', '736137.85702779', '6579188486857663496462.929665', 28, 4);
    T('6886266974462041003392255811.09', '157963384397039800487731671393.108839212437491153109722732638001', '164849651371501841491123920000', 26, 1);
    T('2637149.605497368130690', '337990492.8259301750025354342970385746411677689', '340627642.4314275431332254342970385746411677689', 59, 6);
    T('570544774319240136627673486124120453.421318090125', '627713154220186466312689860553.99818847011260119751', '570545402032394356814139798813981007.4195065602', 46, 6);
    T('1318.48981623281675', '53804888180681289555782776.08394916399261146860796354721', '53804888180681289555784094.57376539680936146860796354721', 65, 4);
    T('261560536522656708116178389636.96902957482448266', '43.894270959640597', '261560536522656708116178389680.86330053446507966', 58, 1);
    T('7389329930453.70597654613990586394144740106960', '7516027589853093719738857026838910413.94175665824167286', '7516027589853093000000000000000000000', 16, 1);
    T('567517025412398867248433926.8250784803633167567', '696941460339562352367494644161068447.0679405', '696941460907079377779893511409510000', 32, 0);
    T('82945697650.550467974', '1284942280086114734292766548209853298.612173192917', '1284942280086114734292766631155550949.162641166917', 65, 3);
    T('93979849901023407697509.7504500665642', '11239154317612957016099.3416669443153809735814058303182922492', '105219005000000000000000', 9, 2);
    T('644053055639524.76460493835227869', '24146983115497166.1003169554407', '24791036171136690.86492189379297869', 51, 2);
    T('654967.7798430818094341734', '977795808332853542.993', '977795808333508500', 16, 4);
    T('61824635978480492.6150785106270847', '667798149318896631450324413241.62326275146964221151041435124939752', '667798149318958456080000000000', 20, 3);
    T('14259971637546.243488016880492813', '47.75579735608706408', '14259971637593.999285372967556893', 38, 1);
    T('588735735873509776102380058677.1286952894401', '97.779232460469346660464003276', '588700000000000000000000000000', 4, 1);
    T('4957944374566137476540.9952771548077616024854644521329663489719', '266991814901251690329155686572698080.319489183525872908575867896', '266991814901256648273530252710174621.3', 37, 4);
    T('905145247499300612520097.253707113587775050584777734446', '2.67860616', '905145247499300612520099.93231327358777505', 41, 3);
    T('187589668011557947955872515425196212.068934581576274674451435281', '3714253520965421.0362', '187589668011557948000000000000000000', 19, 2);
    T('632409948559671833918283328879700466.0707', '0.0', '632410000000000000000000000000000000', 6, 2);
    T('984658744739418870.0476', '994.96727362001617', '984658744739419865.01487362001617', 49, 6);
    T('29683677807887236717503907093343.06771889170251887946265749329856798', '45075776650325326298409643816.981009335987971232267', '29728753584537562043802316737160.0487282276904901117296575', 57, 5);
    T('7558021712013738570217625988429759952877.233977937693484040846382622449065814187', '200795399044471614083706145185994.9737509075917941414185784409908246339265', '7558021912809137614689240072135905139000', 37, 6);
    T('92727445890741842949222790332510627.7224808091', '7609679843743118851669318656.448072904050281496386026279208447', '92727453500421686690000000000000000', 19, 1);
    T('92.908806076826839332', '92015624645614561808276089841.2399', '92015624645614561808280000000', 22, 4);
    T('377398015133086006.7276532802990993135', '28.905738023202267326815256101032838442', '377398015133086035.633391303501366640315256101032838442', 78, 4);
    T('621370624941874188939244.5200921561690076892575914021', '14510649911140399825678588475466309.79501016304482572013696', '14510649911761770450620462664405554.3151023192138334093945514021', 81, 5);
    T('20081300.1067371967724120952932454323', '9675429293924225905505754922506072343.970353197935010801782552732426097906028', '9675429293924225905505754922526153644.0770904', 44, 2);
    T('54659276818717923618921400660359661.2290315794669934292061764', '8321263373.178046641719253164256231', '54659276818717923618921408981623034.4070782211862465934624074', 65, 0);
    T('7495308563429194517.4950842956952536756132', '26.98098879', '7495308563400000000', 11, 1);
    T('88323438972.9', '3947737028583992592909690304161920258.9068619264090315473722446642669787780455', '3947737028583992592909690392485359231.806861926409031547372', 58, 1);
    T('91225.90076905019990035843212878517817', '2130936640071464704116655553.147183018975', '2130936640071464704116746779.04795206917490035843212878517817', 75, 5);
    T('20407744.328013599621804041238', '2232167919397693742359658567.87265821764005695380797468', '2232167919397693742380066312.20067181726186099504597468', 69, 6);
    T('957850276076148.677550288084933', '888359678658559270946031642.373', '888359678659600000000000000', 13, 2);
    T('74703809599234519251556274.9985711463724', '709853892003739943883074863956445427.0505267451606981', '709853892100000000000000000000000000', 10, 5);
    T('709178511125702540754989933967.422236522725392985745972858235163652', '428213017390007592115871937872.386068060216', '1137391528515710132870861871839.80830458294139299', 48, 5);
    T('530337022465.5659118274', '301051142868.206537562799420298004577157182501384107', '831388165333.77245', 17, 4);
    T('39403557595010122186833725730.5058035394497957071790', '18724770932943354346673517548.401096149086217', '58128328527953476533507240000', 25, 3);
    T('82428066.0483267045298279670012562565', '4331992005957183667051602911617712754498.899', '4300000000000000000000000000000000000000', 2, 6);
    T('80476913465044941102915.632723815887522711331529', '5887544380532992755.93511005801607595687540', '80000000000000000000000', 1, 3);
    T('76964133350127894939591167080295347.29740816733871', '772928008263955477554638996.3016794002524482346051388435140772', '76964134123000000000000000000000000', 12, 1);
    T('93613415080964701839073763939463.7495190652950339', '2046269205652007290620382.1448517814887759303162856189242851406961', '93613417127233907491081054559800', 30, 3);
    T('93178915455.978234064862278834', '70156785202107872073374655857728.50328804248218', '70156785202107872073467834773184.481522107344458834', 100, 1);
    T('5930.48411006091572345293786062592928', '89716089387944665472.79179673561851851', '89716089387944671403.27590679653424196293786062592928', 75, 2);
    T('9.09', '81260831717416.97800363', '81260831717426.06800363', 89, 1);
    T('5.86180704663201447', '502623012117081747364976254577298342892.5693576219320095015755173160', '502623012117081747400000000000000000000', 19, 5);
    T('472623181838332890363488612910764474071.586394798193158796943819', '8947794.638137404', '472623181838332890363488612910773421866.2245322021931', 52, 3);
    T('2918131158533557596.17923195829208', '736594397048603541.13479231839973012229918592', '3654725555582161138', 19, 2);
    T('82175442362893730960724401268.902099552468164410386448', '539613524689316.8064962190352269180519578275596107467', '82175442362894270574249090585.7085957715033913284384058275596107467', 67, 6);
    T('78844330615540128067328106904.417352885735996754512588977537835', '69839051335233721003254224037166.023137852571150860272482848011573', '69917895665849261131321552144070.440490738307147614785071825549408', 93, 5);
    T('83665060234963974331243072338458013.41317384589891075', '36334.350471648383518695', '83665060234963974331243072338494347.763645494282429445', 57, 5);
    T('16281170009138261525916508238.0992', '5448415536509973562150706380848655442293.0', '5448415536526254732159844642374571950531.0992', 67, 0);
    T('1855445938689865341002697566158764326.79725393455165540', '565.09791800109811921652723361425597442', '1855445938689865341002697566158764891.89517193564977461652723361425597442', 97, 1);
    T('59739822482533882530.588103741051908781', '1074461088804628208978814.625', '1074520828627110742861340', 24, 3);
    T('343073566004818040807197646892403723450.116757581689904546354687420', '81463061354543518130225835483953609.7480531965302510897362653301336597', '343155029066172584330000000000000000000', 20, 0);
    T('7029.86663560321963172780', '998093479032.085', '998093486061.9516356032196317278', 97, 3);
    T('6575500176279641875577.1641', '7042204323.867599405615408232', '6575500176286680000000', 15, 3);
    T('563573389370505100921783994.63820711261296', '6183290606357615817.2201', '563573395553795707279399811.85830711261296', 93, 1);
    T('0.11775276416997354994358051', '9390124793042023844.11441973650635879154582117867', '9390124793042023844.23217250067633234148940168867', 98, 5);
    T('34682163909803912305.58279868924', '3882592798.71', '34682163913686505104.29279868924', 32, 1);
    T('45556245389463100131676399335223730722.64187559429639385721653', '3139098333707395868305892593215872544698.3980421276727128', '3184654579096858968437568992551096275421.039917', 46, 1);
    T('42868.0', '22853181.7', '22896049.7', 15, 5);
    T('13324349843364165078434315416.4219558920586598551', '634910046173465606044.94350342', '13324350478274212000000000000', 17, 2);
    T('214681567801880599741149018665.4268904627994045437530044796095081166697', '30.24438352', '214681567801880599741149018695.671273982799404543753004', 54, 1);
    T('7523592953864446224655147634517806935677.8', '321960438573180275859530550631223297.2220693619034050159922710483945976', '7523914914303019404931007165068438158975.02206936190340501599227', 63, 4);
    T('8113872603185679969728486212332590783.602055287175599032912451', '6883368621751387.904574695331503', '8113872603185679969735369580954342000', 34, 1);
    T('4473325482.521859322286620356175998427324', '5517383386767642457749850088998162.9878404839737358471115523370', '5517383386767642457749000000000000', 22, 1);
    T('548897584678465172116.263299940', '72849524332345281737.582385686658613419466441920115889', '621747109010810453853.84568562665861341946644192012', 50, 4);
    T('117624754.6743723197932701087121335', '432230085746050.6326013435', '432230203370805.3069736633', 25, 2);
    T('211869120158.6755510559122396323632849492172', '3350176216485535906730909111199110098559.434382853205751669259307', '3350176216485535906730909111410979218718.1099339091179913016225919492172', 84, 1);
    T('9503417727903696377.142688585685289539282', '9877995505774365097359676968.96332983381', '9877995515277782825263373346.106018419495289539282', 73, 3);
    T('57724427547745158119515495.3817737615142', '4694471183681522510434.9730842784869109298930451804195715', '57729122018928839642025930.354858040001110929893045180419571', 59, 3);
    T('626926.688418991670094873504536127927874926961', '44847638554.28038', '44848265480.968798991670094874', 29, 5);
    T('714421546608288.75231442', '90204.1877149702887861501577366', '714421546698492.9400294', 22, 2);
    T('316760561115100328925527829.868748', '947550435614164286486134205461396980094.329237531119278646527505237738228861', '947550435614481047047249305790322507924.1979855311', 49, 4);
    T('4787781.830769633650', '28106931688076101944325980357112766666.4147856541674471029390', '28100000000000000000000000000000000000', 4, 1);
    T('57582322933702115191953750823.452324932759532448958683849760832537', '6208160069451660574.046992350532154118230361572466326413231', '57582322939910275261405411397.499317283291686567189045422227158950231', 88, 6);
    T('1355.2928651389134946509903601457219696', '19706467852518454.51194438', '19706467852500000', 12, 3);
    T('6976348867939077662220126242.325581450', '37632000780254976.39244499406494501679005669774384375', '6976348867976709663000381218.71802644406494501679005669774384375', 78, 5);
    T('6570085371399.73547644347999335127', '756784655571328405212254444.6082917039022814', '756784655571334975297625844.34376814738227475127', 95, 3);
    T('929638604406893166255480319.1', '2.417176', '929638604406893166255480321.517176', 38, 4);
    T('15333436830100284767892.8132821094757068516782384757893828', '6449148471872656116642573.4936072838460652424614060030402960899', '6464481910000000000000000', 9, 6);
    T('8533454661767.0', '9108117482.7818582174602901110', '8542562779249.781858217460290111', 98, 4);
    T('39994199552944997944674603137102.34481054478642979579556733', '48416362753816384.1186859947', '39994199552945046361037356953486.46349653948642979579556733', 58, 0);
    T('3434813658037057456771225.691575393610', '4750727681378762.621422066', '3434813662787785138149988.31299745961', 50, 2);
    T('5421870551709971643477646229788.9916151146275027436845830667747596930491', '1294187723.63552000599434', '5421870551709971643478940417512.6271351206218427436845830667747596930491', 84, 1);
    T('788800325137015309691.9869658106091643522880904508333', '21900937194317004352606581867113.74136675', '21900937195105804677743597000000', 26, 4);
    T('9617449543131441414744.740488254808748568321093692000048461', '235857647890636847089343613221.34934036368820', '235857657508086390220785027966.089828618496948568321093692000048461', 89, 5);
    T('66516175033671157522539424787063035.289018415345287705892', '457025403297589.48109451223206526106813370983486534540', '66516175030000000000000000000000000', 10, 1);
    T('1258923932952359915757977904.322911996152689501640222239052', '516664166.6315559709401451437542901201056045029', '1258923932952359916274642070.9544679670928346453945123591576045029', 95, 5);
    T('54150784958055693201041677324555.2911379033655054172110464032863412', '33037219866829.6261839599256770108242884962194958', '54150784958055693234078897191384.917321863291182428035334899505837', 81, 6);
    T('8913941949356907370902846091451.1977723298', '18118.359607978038552071087439', '8913941949356907380000000000000', 18, 0);
    T('420410527375765723.59880359173181300286368456', '51144376163959512362961732882333687.127621078003128120894', '51144376163959512783372260258099410.726', 38, 1);
    T('2349860389256052429839524907.98078795118546317910978141', '79456576116664247.9125601988131884602', '2349860389335509005956189155.89334814999865163930978141', 83, 6);
    T('17267416260184245345149414069.320310187240', '468091424192506118640825196.9226420172656985659417202247668848', '17735507684376751463790239266.2429522045056985659417202247668848', 88, 5);
    T('43036191260946830561733461288991373.899222463453441', '10395502788.8976', '43036191260946830561733471684494162.796822463453441', 57, 0);
    T('4909087864.1506006674710514486016876552', '99.292952416278708', '4909087963.4435530837497594486016876552', 41, 2);
    T('3.21003346071', '312572971640834499.5651635013104403556411852', '312572971640834502.7751969620204403556411852', 65, 1);
    T('35598083270443235311505086492314.01162518855759640920', '802559462829546799507.20438317038165345760210289811993', '35598083300000000000000000000000', 9, 4);
    T('4920535946382178542803785670.9541584521', '170.20', '4000000000000000000000000000', 1, 3);
    T('900521977018339396017913407962.091347787805459639979561619514892821', '86767128498.266492255048676', '900521977018339396104690000000', 23, 0);
    T('75902636101686648000.6', '359043382984760880361621433096498558.8522232641204', '359043300000000000000000000000000000', 7, 1);
    T('7241120.33794391441049422', '1335.1641429496763007692077381282', '7242455.5020868640867949892077382', 32, 0);
    T('31660394971452812.41546802595892146417789641819', '9553266936159140830011359638454773.3381424723086951716056', '9553266936159140861671754609907585.75361049826761663578349641819', 69, 0);
    T('234366.419036191593359028721642040021', '60990608004.6502084059964088971099341864756873167255', '60990842371.0692445975897679258315762264966873167255', 63, 3);
    T('595275472.57284296', '585213018909659275240537.83842', '585213018909659870516010.41126296', 77, 2);
    T('686195920001876075.42907969250854530243258040339799332818', '690209177.0029006268774080543176338133518164481718', '686195920692085252.431980319385953', 33, 6);
    T('7277582821973178066168.46693414550857472667581295031419181226', '977228.354318', '7277582821973179043396.8212521455', 33, 3);
    T('16292530.1051086167480590244141', '842.22481391817784261791644440600423', '16293372.32992253492590164233054440600423', 70, 0);
    T('446843063108997245517407765134496278.709613990646498002821658685419236096', '88.30183490297', '446843063108997245517407765134496367.01144889', 44, 3);
    T('971454488708956128182425493108.4703496410268788074591551814', '1500927701243584.018518512711227442104023548', '971454488708957629110126736692.4889', 34, 2);
    T('198396844046545593344953371792608987.3415868552607561', '918181535.23275182900', '198396844046545600000000000000000000', 17, 0);
    T('2972391991248.1400667620189362208168307', '5479621704745657230709696892535281281754.7235', '5479621704745657230709696895507673273002.863566762', 49, 5);
    T('167427692328032674.25414709703925071305926860225042563182', '89766918952455057053287822756397622107.3928497608193855537950881184312381', '89766918952455057053455250448725654781.64699685786', 49, 0);
    T('43769831251746560.795026217654974915196027543695', '75537438951336483269081.02941273170981073861080518572', '75537482720000000000000', 10, 5);
    T('3.790', '9378194.8767000998322295569523865', '9378198.66670009983222956', 24, 0);
    T('24851382261264597346888356037041.885', '1207.8573780576386389443173321131', '24851382261264597346888356038249.7423780576386389443173321131', 81, 0);
    T('857.7560861034887549930443754', '18504376084922251496220282291018.19697204676', '18504376000000000000000000000000', 8, 5);
    T('5603522276434.666278645458189168052', '9478916073657380583651187460768352706264.0922039031654960785', '9478916073657380583651187466371874982698.7584825487', 50, 0);
    T('272.20163189736699207276', '8009976948562717694664282.361198237345827', '8009976948562717694664554.56283013471281907276', 93, 1);
    T('75286597915548865172794905512308251.1787842766', '5484959.61054', '75286597915548865172794905600000000', 27, 2);
    T('93525713224070.09674242841815995541364018052052', '75768676.532480641692742503493', '93525788992746.6', 15, 1);
    T('78523021357694108868692752.91646650044759904173480959883557402997', '495.59', '78523021357700000000000000', 13, 0);
    T('3238721780678562900915193565802225587392.90', '748785061.66820334924209162463741', '3238721780678562910000000000000000000000', 18, 0);
    T('98846.81464384706741923274334684308930111', '9113.005045834', '107959.81968968106741923274335', 29, 0);
    T('9.8406692310667035312191105495866801000', '9089567678.5508097764778', '9089567688.39147900754450353121911054959', 39, 0);
    T('99065135312720.84941192157958140619229438117', '53476921863610672798515553218338324.959329170020735932026093766186541396', '53476921863610672798614618353600000', 30, 3);
    T('3176881371318006310361.3543736148140570707252111', '292524070460132222826177704057904019376.40698273353941530130447408152708766116', '292524070460132226003059075375910329737.761357', 45, 2);
    T('6.90498764441988249128502282530851578', '43.50645242771270044511', '50.41144007213258293639502282530851578', 45, 6);
    T('338450.823402603747752671009120897851', '5207.0099214953', '343657.833324099047752671009120897851', 59, 6);
    T('2919769949581629422304417.9039495411631604778178068863', '58958088180274381782196182656778.3384598201347480560152781708323013', '58958091100044331363825604961196.2424093612979085338330850571323013', 81, 2);
    T('858925411393538961692691.6390687813', '429652895153162504998871559279.1146093468874157436552', '429653754078573898537833251970.7536781281874157436552', 52, 5);
    T('1744056.272665115358269071', '6706.43717222508571881815348574554893', '1750762.70983734044398788915348574554893', 39, 1);
    T('56816849065678873267681718880725180435.50484439874591109198328584624568031872', '100.3391310721886', '56816849065678873267681718880725180535.84397547093451109198328584624568031872', 83, 1);
    T('325994361227633619628236023.45538252274702004372524442388152996', '6228863760768945706791.193052060', '326000590091394388573942814.64843458274702004372524442388152996', 79, 6);
    T('58883799430474776478.6780455638602331105161289191497741279128', '1957786726380541.6387643616607', '58885757217201157020.31680992552093311051612891914977412791', 58, 4);
    T('678747071040875754504020603117066784052.41403513158306431185445288005631783024', '5461331138.0', '678747071040880000000000000000000000000', 14, 2);
    T('6427630187605723.351', '104762563213642127099390816191096.12305484', '104762563213642133530000000000000', 20, 4);
    T('232891181347222.61', '81.56887307414412552900', '232891181347304.178873074144125529', 35, 1);
    T('12859912548601644207542423959196402948.57758993621777352101567741', '425.0', '12859912548601644207542423959196403373.57758993621777352101567741', 88, 4);
    T('35.61403552396659586793630020864', '5959067851450437531251104251996840.42165481660415', '5959067851450437531251104251996876.03569034057074586793630020864', 90, 0);
    T('7240.24200339051522742154493264792549999716', '917383518974558017521825751587076.34', '917383518974558017521825751594316.58200339051522742154493264792549999716', 100, 4);
    T('5153317142353.9312540486053511463005556205', '2128278787681923826573.181817739403', '2128278792835240968927.1130717880083511463', 41, 4);
    T('949998020311633330712125971869453.332548838793698618', '8204048128224235511876393687.52075303', '950006224359761554947637848263140.853302', 39, 2);
    T('2159454272813864993661426866.1766866480118', '9445267.427778447965453354', '2159454272813864993670872133.604465095977253354', 60, 0);
    T('982112809045795245186551043643573661.613089408357184555767803927521', '4.08044147186438330025040708498', '982112809045795000000000000000000000', 15, 1);
    T('220841406559147033687600.45593619231259', '13.3843619390330896811', '220841406559147033687613.8402981313456796811', 97, 1);
    T('1277117292387560216176284219283757627739.20122980104127185', '2290230103273.41804237119001126581', '1277117292387560216176284221573987731012.61927217223128311581', 78, 6);
    T('5439075.2560', '776153134927212577084.98204303378281834722509', '776153000000000000000', 6, 5);
    T('17734035356805089071.06160222005824540798956897325', '5779080727873460803535483384.0954216740874864352380340', '5779080745607496160340572455.15702389414573184322760297325', 65, 1);
    T('128959592072.8945412145924003309512547978', '917380075785615.716930150433510946069543422236145', '917509035377688.6114', 19, 3);
    T('4994151781801.1144546607287', '1.058021198177677923', '4994151781802.172475858906377923', 100, 0);
    T('3312403153719206305127341467810047370.625563779957760155682931605280', '7931999.9069121082505015328446242298147527745887', '3312403153719206300000000000000000000', 18, 1);
    T('233586621665721780275953582382633.520106', '89365467045921537784.22406', '233586621665811145742999503920417.744166', 68, 6);
    T('41740689098164600783354825.991062692033904048', '2861479122062597554008304304868218.0', '2861479200000000000000000000000000', 8, 4);
    T('8824698675.0900478767506801435007320868425267674', '5967548825.692416188831986215444561849457013748038', '14792247500.782464065582666', 26, 4);
    T('9092855192511227207633078029156402.376697310157137395903380', '350885818384473166930854582703599535.769939', '359978673576984394138487660732755938.1466363101571', 49, 3);
    T('2008477.864569000564070053915606', '52195739868070.90706991638238030', '52195741876548.77163891694645035392', 34, 4);
    T('711316598996.40', '429528059505566591354188.3006237', '429528059506277907953200', 22, 0);
    T('23777501833335386454990253103859785.8590708848855550785476', '5408464376741388658232405114749329594584.49829661', '5408488154243221993618860105002433454370.3573674948855550785476', 91, 6);
    T('930270149233089117187149290119825.97659302331041310093549408627949682', '797460684888997728399557566187052.19317879512148907878358', '1727730834122086845586706856306878.16977181843190217971907408627949682', 71, 3);
    T('632048795.75459738161125182585345168669614880880', '6992903891649002256827346760521990.270906372909598814041501037226', '6992903891649002256827347392570786.02550375452085063989495272392214881', 69, 0);
    T('38219302893241340992.6262632395451', '8356551887878893055732.312763757833440507801643971795228693960', '8394771190772134400000', 18, 0);
    T('4151250597090506169575372041.430040698692009697521031', '671085634857267850048171944269.713932155920255893683880812199634', '675236885400000000000000000000', 10, 1);
    T('58.347211933', '85188914577874.043954357864194579239099794091530373', '85188914577932.39116629086419457923909979409153', 46, 6);
    T('82656662139.073535804430927897857799247324506531667', '507810803625644562429751549760128746293.9000753663588625943389270577481347396', '507810803625644562429751549842785408432.973611170789790492196726305072641271267', 97, 4);
    T('23198872949855958.2229674', '511795481895701506012284998386971416152.886112201', '511790000000000000000000000000000000000', 5, 3);
    T('5931734335379696704997391.572566', '8982411.0748520317783855714095875880004823621696', '5931734336000000000000000', 10, 0);
    T('3.3721447023287770', '999918769814479605643836022317162569741.731383260527062658851352221808700155', '999918769814479605643836022317162569745.103527962855839658851352221808700155', 83, 2);
    T('21747325536276238277431439558.66970', '5.3736', '21747325536276240000000000000', 16, 5);
    T('809065928657129515917863708977139711698.400061943375627926413239012820787802680', '5006451.84177161157', '809065928657129515917863708977144718150.2418335549456279264133', 61, 0);
    T('33118290863418260681526.1674784172907030056', '7124.900559162441460591742096', '33118290863418260688651.068037579732163597342096', 57, 2);
    T('638689564522.093700881283850698770', '9900505969.5', '648590070491.59370088128385069877', 84, 6);
    T('0.60720', '7671144527937939.18758412585814', '7671144527937939.79478412585814', 94, 5);
    T('9122810652035444096.61384172', '9280.245322473736736365227', '9122810652035453376.8591642', 27, 2);
    T('81117.5', '80376644378.143336345880286740', '80376725495.64333634588028674', 89, 4);
    T('251144.6881313343', '7205300184678589.722', '7205300184929734.410131', 22, 6);
    T('7734478995486369746949605183722787902.112431540428996', '721614691508311346954318934353.71626929319557', '7734479000000000000000000000000000000', 7, 1);
    T('1375080909.11908', '52706130.93136172475864', '1427787040.05044172475864', 53, 4);
    T('1566.038232301958973338696083046427', '50621026719801272780247339793.8059761156180819', '50621026719801272780247341359.84420841757705523869608304643', 58, 4);
    T('63136600779612112190555477892457.607463933452471', '121946932359714724226.5788722287146344803320', '63136600779734059122915192616684.186336162168', 44, 0);
    T('20491950313680405883621.5465401546857', '41.1351', '20491950313680405883662.6816401546857', 65, 3);
    T('332621630633642675099359122800.6515951732506227', '370.51439579843212666496', '332621630633643000000000000000', 15, 6);
    T('3169550117554451552052990.295961830888307601717694740926', '6470973277240826134293838652.37', '6474142827358380585845891642.66596183', 36, 4);
    T('80204198614735089581150125477617930707.4953249087354657144042762004752271835', '952.0', '80204198614700000000000000000000000000', 12, 3);
    T('25979.857825410248162671244400737585722398', '7141649185887.0', '7141649211866.857825410248162671244400737585722398', 51, 5);
    T('23.41523510971420721913074', '8318785313093170493774.0192712949307028757136180380832', '8318785313093170493797.4345064046449100948443580380832', 60, 4);
    T('459961192131761837598771813286032954.235', '9.45645727442459720304039902', '459961192131761837598771813286032963.6914572744246', 49, 0);
    T('624.718638', '721147.2131071435522', '721771.9317451435522', 61, 3);
    T('19834948347785635027801448737317.80351233879606808383490073784071290477', '54134453.68133292811092456', '19834948347785635027801502871771.48484526690699264383490073784071290477', 93, 1);
    T('807717409962399082433440348975756236612.6757', '4271022412900.51063584935384', '807717409962399082433440353200000000000', 28, 5);
    T('838452118880192531720018132576.4025304768514767319018546553020139', '50037.09', '838452118880192531720018182613.4925304768514767319018546553020139', 73, 6);
    T('927.8885496901511', '71675754923085627004838026109222.783130063099390999152', '71675754923085627004838027000000', 26, 0);
    T('959137837946061556472005034.842290', '6870554901572582143668650969414959168.450949853517745830515081656150', '6870600000000000000000000000000000000', 5, 5);
    T('72673623260245070694.08935762380862436970216806649285349', '20339531012464014460.5525730742', '93013154272709085154.641930698008624369702168066493', 50, 5);
    T('33613063526435917584375.5459593484262603276282672379', '588234.6058117107157203', '33613063526435918172610.1517710591419806276282672379', 77, 0);
    T('89039252942347106692360918472032.0', '5970623548999709.9029498433536', '89039252942347112662984467471741.9029498433536', 79, 1);
    T('93244.42638914284081138462454027108633333892', '16172388524297380350185885964511900181.245999433896841442662951863032883', '16172388524297380350185885964511993425.67238857673765282728749213411921633892', 83, 0);
    T('582528630142259674025720.15958915910', '13089941414699984038297461057877.8144857926338294519466', '13089941990000000000000000000000', 10, 1);
    T('834338548.285888517782241664511978402056423438918', '80776411178458068624250428000.0', '80776411178458068625000000000', 20, 6);
    T('225614956767543856525.2199766676374884823351948698851335599', '3600432799346857658111185257161.55', '3600432799572472614878729113686.76997666763749', 45, 2);
    T('800441502535862581733594490.31511710831783313', '2756947859922.09146037667977320970916830400746158702', '800441502535870000000000000', 14, 5);
    T('5541505357974060.9226455678841946074571773593647861499167', '36946443482342503293446.563', '36946449023847861267507.4856455678841946074571774', 48, 5);
    T('9189037058814744580915.7745507990268', '435900241041442419936.40609930573052310156736', '9624937299856187000852.1806501', 30, 6);
    T('32142.685473762494', '4.0', '32147', 5, 0);
    T('4626.8888476317288338433512', '1511834882.091640575142391', '1511839508.9804882068712248433512', 38, 1);
    T('22675954569280642413.0652004159352812856665437027557', '1808600518706660046967340900124.955120736748131774281842817424', '1808600518729336001536621542538.0203211526834130599483865201797', 66, 6);
    T('74133095833453373024692471468427.23', '56846056385370.2259305396122915637762727779', '74133095833453373081538527853797.4559305396123', 45, 6);
    T('69419561829515398575196246.3692744486088225229960', '487097308728380.448266650118198839283694', '69419561830002495883924626.817541098727021362279694', 86, 5);
    T('25349888969747179603657871.275546814823808144541361066132984219982', '423457153765482588428478654695835.941041633504168026776051417096849001', '423457179115371558175658258350000', 29, 5);
    T('693415337532223547.37717296726293903513598624909403', '14988757592.5613039099791920846', '693415352520981139.93847687724213111973598624909403', 51, 5);
    T('587400237761982205929717093909.016353173', '76441955184371526919.83392741375534325408', '587400237838424161114088620828.85028058675534325408', 96, 6);
    T('4714907805454771467521507531579822295706.0110319441705466', '5820120172138711995282181126.528004', '4714907805460600000000000000000000000000', 14, 0);
    T('9322043496857284154723419242.255', '3519426408314730877869376589350756091809.558010778522532150979979838705920478160', '3519426408324052921366233873505479512000', 37, 2);
    T('97721814448023.19994676905451547004055263364202658', '3939339667385640243328960901060.821522248194384235232', '3939339667385640341050775349084.0214690172488997052725526336', 59, 1);
    T('24618982644568.189900', '6890196413.46483', '24625872840981.65473', 50, 1);
    T('962721420308133042941.88', '305556.5373060218', '962721420308133348498.4173060218', 31, 5);
    T('9019847.016339325271253513693880563115670', '3936095321.5', '3945115168.51633932527125351369388056311567', 45, 2);
    T('3062124615497900477123942.115', '6720207215463526418507777.8', '9782331830961426895700000', 20, 2);
    T('854697794447.4730055900329', '2556723056.396', '857254517503.8690055900329', 78, 0);
    T('841671642505163250615095530947155066.139452719814028864206174154095993608743', '5.5143406928408012586', '841671642505160000000000000000000000', 14, 3);
    T('244772829810112309632.779718416623226317996644430152', '885503969228844367463997.70362008', '885748742058654479773630.483338496623226317996644430152', 62, 5);
    T('767850235998288573057451927764.642721274674764063403763086923066', '31348716127713511958601621087452591.3012911948079848580505', '31349483977949510247174678540000000', 29, 0);
    T('708703117677058948270995583125349375894.765224531758285971703414531', '905443497.03824', '708703117677058948270995600000000000000', 25, 2);
    T('794619486.8100109008168988746', '615368480442507657160.74646030295836173', '615368480443302276647.5564712037752606046', 68, 4);
    T('9.574674979', '41588215.54', '41588230', 7, 2);
    T('2211.5717404950501', '368.185120500101198496557078086', '2579.756860995151298496557078086', 88, 6);
    T('31807.26046908225704', '3439439125134349031600877489.80958250986538968', '3439439125134349031610000000', 21, 0);
    T('6781987519408147305168.07448258339555441469678712006946', '537.96809616165593781051652679473', '6781987519408147305706.1', 23, 2);
    T('80204735327364874446670363.4812701914841437488474601268020432103584', '1923.409244965838251', '80204735327364874446672286.8905151573224', 39, 0);
    T('778039150342540255055.596707846325648924', '17705044934575878208255067422371450.967932513388673347072826891', '17705044934576656247405409962626506.564640359714322271072826891', 92, 6);
    T('60730832427061450598.09549229489686528745859297', '1379161311098128553458418499.252473184819758192470866278399456', '1379161371828960980519869097.3479654797166', 41, 6);
    T('258663998393439647817151252729374893.04770484297439060958', '8203468109488574383141.2122245', '258663998393447851285260741303758034.25992934297439060958', 69, 3);
    T('586789495544653566030813121232.757446508019298600424488416587526470348', '66962844109588409221.67814541653679728474130088', '586789495611616410140401530454.43559192', 38, 3);
    T('143810114299943978970310.360345373690937193473279904982', '196366394588588852023071479170355.2994155', '196366394732398966323015458140665.659760874', 42, 6);
    T('2558218408137407879706.16', '710910457411140020194900759747083851687.21276534908258656524461248710259009', '710910457411140022753119167884491731393.37276534908258656524461248710259009', 76, 5);
    T('77830707099798.555196530702867502801926406', '5921915540001689183.57062346761923412787060', '5921993000000000000', 7, 6);
    T('453642496162698547329.80899844', '78275269575327856.5566', '453720771432273875186.36559844', 47, 4);
    T('2879952268113117.2976597', '18477.1165738839930536348', '2879952268131594.4142335839930536348', 80, 5);
    T('5674482261116558031444181.0939607', '37815717323381389.822544327438308572354394', '5674482298932275354825570.916505027438308572354394', 81, 1);
    T('27643341321625756374973188264.51516654040989577510349366842', '29152070986590505969407134248299135.4028094865679411413451223618048171348', '29152098629931828000000000000000000', 17, 6);
    T('81389.80988082755212460213', '731140130482802345490498103613480549602.227038531155900726361873693', '731100000000000000000000000000000000000', 4, 3);
    T('4.2', '872822534587466037520.85765', '872822534587466037525.05765', 99, 1);
    T('3.788126841809160539267460990081668103552', '7404307709083963830655547651734186.88302441444262075057566784533068651587', '7404307709083963830655547651734190.671151256251781289843128835412354619422', 75, 4);
    T('777211.721510668871355334229301472379847611', '5.40367837048873787382059763', '777217.125189039360093208049899102379847611', 65, 3);
    T('88504800.945713430734921538237063716200642589602', '2271429405335040521043458.31133608914826421830724', '2271429405335040609548259.257049519883185756544303716200642589602', 97, 4);
    T('56570415329927426518443627.2257778006655507224408146180057159198', '385270127344209968615531304765613605796.9967599941420739571490327060', '385270127344266539030861232192132049424.22254', 44, 2);
    T('6321990591689430762285366572.7410030316267824439844539068', '7435667347482539784820162.19322366355325', '6329426259036913302070186734.9343', 32, 0);
    T('2825.882059718220', '2812752.753516265609330909268', '2815578.635575983829330909268', 66, 2);
    T('358993696888430899931695950048558866289.882853388832007091931924', '2868222.862775541730862991311398237330057926855', '358993696888430899932000000000000000000', 21, 6);
    T('1637473011565485226996.85', '7751859747604221522676418459799980912370.4790386530449455', '7751859747604221524313891471365500000000', 32, 4);
    T('61342576352829491784552940753066498.444701681154240116070393196624681652418', '40090153165718580.2', '61342576352829491824643093918785078.6447016811542401160703931966247', 66, 4);
    T('708698529476898875014557.13648', '939028189688946707799493736606444154732.092503', '939028000000000000000000000000000000000', 6, 5);
    T('884141514298740826377594102881712946004.804724498916975808867', '49839467019.4405375802', '884141514000000000000000000000000000000', 9, 3);
    T('907192751.83521887649208673652731016402229999', '91245794435410435970764307177080.304922246045548', '91245794435410400000000000000000', 15, 4);
    T('5532568137493.5440789478935923007793506920012188', '17.1567434650245244574831', '5532568137510.7', 14, 6);
    T('318487.7944702296478487421126235875763453', '8614125338084.126382045663579083412', '8614125656571.9208522753114278255246235875763453', 90, 2);
    T('1859782.53146835769263', '6364.410271233462630590469', '1866146.9417396', 14, 5);
    T('90.83983', '33146584973.1358624239302301', '33146585063.9756924239302301', 29, 5);
    T('453714593811169122379350578906.59681451359355938915749109', '91724637372613238061681.7565296876441619963607819285236782381814', '453714685535806494992588700000', 25, 2);
    T('44672691136956778298114033.781', '7619681627.59', '44672691136956785917795661.371', 58, 1);
    T('85160634174178611333316423.139108242', '4533194060141928947341392197984551.855550164636648046946', '4533194145302563121520003531300975', 35, 6);
    T('7139689090264562228209864149927219665.1', '173.1400327492408257293571643694360', '7139689090264562228209864149927219838.240032749240825729357164369436', 99, 5);
    T('4469086.317482', '79388565062366299244420916494.770701709830350341247743286384441', '70000000000000000000000000000', 1, 1);
    T('4003582763151.8191316074105', '7162089.491889196857951868376', '4003589925241.311020804268451868376', 83, 4);
    T('862607983783846.316', '70196547059878661429.9299637395184422812', '70197409667862445276.2459637395184422812', 98, 4);
    T('51180350378581179.1879475143988324190991227738904201', '629156297539418680818033354612444584.1776', '629156297539418680869213704991000000', 31, 1);
    T('5556792433015195169108590096099981805405.87381111489809335', '1051956815276301950754.71353501783020794378725151', '5556792433015195170100000000000000000000', 20, 3);
    T('99046673280499679999494167758315865265.034174298', '394030279222607941.1209521695828', '100000000000000000000000000000000000000', 1, 5);
    T('2301313737118799850.1689308673665640856373115875055004', '71.8480320', '2301313737118799922.016962867366564085637311587506', 49, 2);
    T('3856907008195667928161709614.976891776902', '579089647225376004115048111.0', '4435996655421043932276757725.976891776902', 96, 5);
    T('453783738567570431080104979632.21437332002', '91076702078335809624542574.9996', '453874816000000000000000000000', 9, 2);
    T('90405038511957416541502329016644.01451382355699005460385776809', '2693076.95388013153099212918798441770258620857', '90405038511957416541502331709720.96839395508798218379184218579258620857', 87, 2);
    T('4197426016344300300366210576.53678592780240911022859', '8341442300681884088270009674880872953.7392628693376021370921421770999097895500', '8341442304879310104614309975247083530.27604879714', 49, 1);
    T('8429344.698105321117190816501848544301142237', '76.9', '8429421.59', 9, 1);
    T('72796279253178556180616443.489322818045057990567999558558723072', '92211130236239490126680145182.790193958740333727', '92283926515492668682860761626.279', 32, 3);
    T('63183222594769122080540711685129028317.3976398720001969', '12650299146852.3897260418136738584678511396465558', '63183222594769122000000000000000000000', 17, 5);
    T('26454.958648062868675896343037440294044034', '2964329478504182869.3637230708057289016380076177340097', '2964329478504209324.32237113367440479798104505802805', 51, 6);
    T('118560392502035444043869433470329421906.309811671403360058874620274290131205', '34351.350492232', '118560392502035444043869433470329456257.660303903403360058874620274290131205', 98, 6);
    T('192900556486047.6877292744079673587298', '757528606549.573284295034', '193658085092597.2610135694419673587298', 59, 0);
    T('4759712470421294482164522572573.89703561117248210066513507557514420965', '2847951284.850747712093038', '4759712470421294482167370523858.7477833232655201006652', 53, 2);
    T('67064128614318050714412856547189096.5600453', '35631573078537084270313360799463.0511', '67100000000000000000000000000000000', 4, 4);
    T('69438643809765861320.0333023549075', '326341364344392.37617956138000857103055680917', '69438970151130205712.40948191628750857103055680917', 49, 6);
    T('83259109617510.792638475032301138', '92404.2486636857', '83259109709915.041302160732301138', 93, 4);
    T('88031757878.00823522049140834496074726944401354', '545613942907922087.18273787257813825', '545614030939679965.19097309306954659496074726944401354', 65, 2);
    T('70591.123171', '86650662386078421020.64354265739004749609719575153101694871', '86650662386078491611.766713657390047496097195', 44, 3);
    T('279096626092504621.0', '9787228.400481777765179988897589071947821317648', '279096626102291849.40048', 23, 6);
    T('3122793075227292049473411.294468540647914137015062032', '59488478899912646432324844.8764', '62611271975139938481798256.1708685406479141', 42, 3);
    T('330241503465324684886676128.2239382654750803816335655104421', '6090764522073.5549711329908785683871817056861780020', '330241503465330775651198201.778909398465958950020747216128278002', 66, 3);
    T('26085357734690519.4285275266394229784157568', '6665116626310767852389326.3802005715812', '6665116652396125587079845.8087280982206229784157568', 56, 2);
    T('5517723151515512761587249.4939', '87731859085497978.442', '5517723239247371847085227.9359', 97, 4);
    T('2962679579209086854450633431193.427', '574624266.1941852315598', '2962679579209086854451208055459.62', 33, 6);
    T('8883140035200051044.6802540343659779712523506715714783', '14820643474639832989015083691.94843846393156479723', '14820643483522973024215134736.6286925', 36, 5);
    T('58989.5604543583472905212965788733', '12569927191977296772672.8', '12569927191977296831662.360454358347290522', 41, 2);
    T('28574033241898993373655892356829975.13366617354276238928274870576', '58367621226.44929379', '28574033240000000000000000000000000', 10, 1);
    T('7484144544315436876963822427332446586.570163118', '264017233630140672404883948664643.182811529565', '7484408561549067017636227311281111229.752974647565', 75, 5);
    T('2.21668417380856023660014966232', '5557576713476755419.8072791275927847662144', '5557576713476755422.02396330140134500281454966232', 69, 0);
    T('15723570252256691480537872.583263984682873613424338214429370845', '459655326061815925097873767391752312462.58136431', '459655326061831648668126024000000000000', 27, 1);
    T('890773572181.1957325750481283', '1750798744403928173.1076015037', '1750799635177500354.303334078748', 31, 3);
    T('599859073760412.72051176648201', '758959989864657256123183248212958898481.87893183317254573', '758959989864660000000000000000000000000', 14, 5);
    T('5448089.9771456606173211151313945937188', '775366884662.753834389466', '775372300000', 7, 3);
    T('20609553992907917624105967407555.6369131328687', '738291.57966724', '20609553992907917624105968145847.22', 34, 6);
    T('787169582.126386605', '427278410335408852416301462.7066124750209015647902399760516389717', '427278410335408853203471044.8329990800209015647902399760516389717', 76, 2);
    T('951877913787811182307710109322478112.4101711668481780870', '16653092730369106386271.750577308986751650603927291600833951572', '951877913787827835400440478428864384.16074847583492974', 53, 2);
    T('542.1842840752355908268711', '602995321852117.412841360504497', '602995321852659.5971254357400878268711', 63, 6);
    T('643677.6132647339049345196544768145122660601953', '31748.0258223534820179740253', '675425.6390870873869524936797768146', 34, 0);
    T('5751138.43465792432682839597409168', '92172174551054.51895269016040385676294357124359492910', '92172180302192.953610614487232252737035251243595', 47, 2);
    T('81714859723113332596.451', '7491119.6699233178752502570785459', '81714859723120823716.1209233178752502570785459', 87, 6);
    T('6094672724745010333040939.416736859743080283355234021239', '699515023751899087905230119.204684439', '705609696477000000000000000', 12, 2);
    T('100608800490922789804453394969966.8133115747119740978346925727', '892604695229311256377.59', '100608800491815394499682706226344.40332', 38, 0);
    T('3718718396.0859232991204114126984162521558687509', '73298910782273240826.6934017361', '73298910785991959222.7793', 24, 4);
    T('42052659646349748442162206524796035.1217300646399700127', '78956.12926385', '42052659646349748442162206524000000', 29, 3);
    T('399218836580680664274508.342670540635925884', '57728252.110424172971994139648397768479750890214', '399218836580680722002760.453094713607920023648397768479750890214', 95, 4);
    T('986977342778199764488277876210.56273493793524', '796096120.126631387604250785575510404736', '986977342778199764489073972330.689366325539491', 45, 6);
    T('49408928188266035922.42627129', '11358165394244017272592.488395964575822894', '11407574322432283308514.914667254575822894', 95, 4);
    T('2709322626613333778245666956861928.93498', '5179700645475855.911485428124440671419061839753223829929', '2709322626613333783425300000000000', 23, 3);
    T('71314514571057597635891713022854.8924', '5.439522761050098346091082686791529', '71314514571057597635891713022860.3319227610500983461', 52, 2);
    T('62144375543413854578233339752473621513.34106744736314340468387207', '46689797101285381818399010136818.6732191296215994994743', '62144422233210955863615158151483758332.01428657698474290415817207', 88, 2);
    T('740840392186385371554408648614506.3354862205837528706552980406952701848613', '289521950381822987576580216477428350782.2123916', '289522691222215173961951770886076965288.54787782', 47, 4);
    T('211989071795615026131380637348302021.9830173223100706534', '468110274023638798635414.76953724986824101740851', '211989071796083136405404276146937436.75255457217831167080851', 96, 1);
    T('15538713014597075957649752849.769576708075', '12892259639898500139106289657.1735', '28430972654495576096756042506.943076708075', 100, 4);
    T('6043702758325201623846967003835177986506.46333261004578752168', '3931186347241150268168504785.6143569959', '6043702758329132810194208154103346491292.0776896059457', 53, 1);
    T('6784401755.8609', '74116627469534657562114.0', '74116627469541441963869.8609', 98, 2);
    T('141722860405048826499015879.5493319005338868430140399599', '47630087848867030252546480.3012155081773179000829849', '189352948253915856751562359.8505474087112047430970248599', 57, 1);
    T('5344213063493781963423629986904.8220072624373893928', '9426895479192921.89871362816563955570244766501631', '5344213063493791390319109180000', 27, 6);
    T('2159916593.964', '27307086066825962470700.4386095', '27307086066828122387000', 20, 4);
    T('2548339777148992332.400918910928126289860321236129', '80475288772474418985562186020.410947122506913677257662360458246', '80475288775022758762711178352.811866033435039967117983596587246', 75, 4);
    T('7519333796248216434709.048198533667956', '93366408.1672453403391224434', '7519333796248309801117.2154438740070784434', 68, 0);
    T('570454.5712578190470397099288558757898956895', '444121259844450337139696479132.08406858458005903814299152191907', '444121259844450337139697049586.655326403627098748071847397708965689', 66, 3);
    T('78786740220796149328077927158260262.057842014743431113191438740013668363', '261783983784151130811128948176198735.023808034599', '340570724004947280139206875334458997.081650049342431113191438740014', 66, 0);
    T('6436811604071099162448967.572', '68410062671340243895251365639457.288811156389133362', '68410069108151847966350528000000', 26, 1);
    T('76699651181895519477987863.30873090110105639367154280608965515634', '4163146640509100745767947051593557924.9322954722405920425', '4163146640585800396949000000000000000', 22, 1);
    T('896552003620615115.3495892941381133242', '63583534336295675297915.62364255828454305', '63584430888299295913030.9732318524226563742', 79, 4);
    T('696799160511.0198659439066704405007509583840517', '27896051194949440.35284992180244316607529749589515129', '27896747994109951.37271586570911360657604845427920299', 61, 6);
    T('5963356144408345127752089420905557.33349738072574698', '81279521600806787339378.43472609175618479272241564855662', '6000000000000000000000000000000000', 2, 6);
    T('795110296.0543769429602090002690', '595681231.330468', '1390791527.384844942960209000269', 84, 2);
    T('68990.4512457400186731395722182459153', '5609345.2437643593209321972687361590473832089322', '5678335.69501', 12, 6);
    T('62572575838911861153028.0371', '234312291820554628171726916289632.2908342958803', '234312291883127204010638777442660.3279342958803', 75, 0);
    T('469402025607157.96480800353974835974240253650733', '75320725675023513.868501', '75790127700630671.83330900353974835974240253650733', 90, 1);
    T('46608737627293205670151275220715178.5519954304245104405460905110262931004', '8052.690426304059079849', '46608737627293205670151275220723232', 35, 0);
    T('4.91', '65568002089283670580863.577137688304964146053', '65568002089283670580868.487137688304964146053', 52, 3);
    T('48076043164754.5838', '967298883791001941401496272.2178134263039245441178705390281621893978', '967298883791050017444661026.8016134263039245441178705390281621893978', 79, 5);
    T('55956813770199373769034106870301976.069531725044305984720979', '7092770776543.0589750144961065614', '55956813770199373769041199641078519.128506739540412546120979', 65, 5);
    T('35770593271608686717904820522708641.896530017647894823262579304832479758', '708329111.481094566116024', '35770593271608686717904821231037753.377624583763918823263', 56, 2);
    T('32803950044864.63218761581304371', '6357061542838738020216551.0', '6357061542871541970261415.63218761581304371', 42, 0);
    T('2510328624423929351879465269805.0', '2365967460010000020722124423144791638.8702611356655943339819062', '2365969970338624444651476302610061443.8702611356655943339819062', 83, 4);
    T('487873852399363801795598418.8191950934715092809402723568770477', '24985804590019171061444511.101', '512859656989382972857042929.9201950934715092809402723568770477', 78, 5);
    T('323646360250158518658079405736215405.66063958112006096', '952650297619132178422194475172.76303229270097210561', '330000000000000000000000000000000000', 2, 0);
    T('9809011860195163005688586041397138681.13714580426950637846343538272895', '23600221009387791658259978417.9679770935468143955013020644', '9809011883795384015076377699657117099.105', 40, 6);
    T('2972044755771516860664030718766621857669.2828558359792923051463684857125645760', '809823796236291.833813076', '2972044755771516860664030000000000000000', 24, 5);
    T('265.8551603407550916495', '6476370111512084010552642149929195019.7253466666011467068438460616', '6476370111512084010552642100000000000', 26, 1);
    T('55903500146.064228787043844278644654904181478', '4501425191277741638735620638.2342551000921829247520430137257485762', '4510000000000000000000000000', 3, 0);
    T('57637599189903139687771923999856311514.684409453', '5339448255970460648.18', '57637599189903139693111372255826772162.864409453', 99, 4);
    T('21509420621256335.2939634', '766135340811602888960.891592965', '766156850232224145296.185556365', 56, 6);
    T('3230299063231887771184840366.598014275', '93292280770144028.0712580712628284802389405516', '3230299063325180051954984394.6692723462628284802389405516', 57, 3);
    T('556.69114098728', '2681412795056448594972822905.6910488366383941332842400649334328', '2680000000000000000000000000', 3, 3);
    T('2740826043.547', '8957.945513190', '2740835001.49251319', 35, 2);
    T('5730103484481870361333484227618166518.205712533849030081', '5229.5647084348869439656138856259253278', '5730103484481870361333484227618171747.7704209687359740466138856259253278', 99, 5);
    T('67931439982631166513681126198472438353.4602582354349828877418', '66355.130540843049140', '67931439982631166513681126198470000000', 31, 1);
    T('1352489.84014285997786393433264224062028', '3275556658.1149', '3276909147.9550428599778639343326422407', 38, 2);
    T('90029728921994982823764821329601.68262108750709275053', '68258943709659278.2188952248509809317156486858', '90029728921995051082708530988879.901516312359', 44, 0);
    T('478042892321346301753.0886464485372552163603', '2577930.45082215873884708217208578847', '478042892321348879683.53946860727610229853238578847', 75, 3);
    T('47738349467.6312414617251', '40889979295247420400711524.78961933031754413759814945061067315588', '40889979295000000000000000', 11, 4);
    T('465718015732815327561886092.119227589635450522351', '256216220155007689189613932179188306057.52937390916906874134198982076', '256216220155473407205346747506750192149.64860149880451926369298982076', 87, 0);
    T('6640710600.0579999060582345766302376246', '55881912843028150081202113.7272586886700807586997465270914', '55881912843028156721912713.7852585947283153353299841516914', 88, 1);
    T('86820.2656993878727966957', '61220873107339732172875567813151930.295', '61220873107339732172875567813238750.5606993878727966957', 99, 3);
    T('53826517691946403158.581870769791771250981', '218605511463154978.7945035904323738998651304515', '54045123203409558137.3763743602241451508461304515', 88, 1);
    T('731418586370665023515410514828719681079.21059231843140199137319669540', '351944031.36720481001322350712248499051', '731418586370665023515410514829071600000', 34, 4);
    T('4.16483403519980', '493082317772877535.8839016080960577965127269', '493082317772877540.0487356432958577965127269', 69, 1);
    T('799594649136.0054853339', '875492760331576.392021282922129', '876292354980712.397506616822129', 98, 6);
    T('9933642.2390456613976', '418626370835.0895712233167351990364188', '418636304477.3286168847143351990364188', 73, 5);
    T('1070815277997618097699537964236349254.1936870516109580582', '675319380032300495426834767675104143.557656831986444326551935', '1746134658029918593126372731911453397.75134', 42, 4);
    T('829.17', '67556530.0844916621789963', '67557359.254491662', 17, 6);
    T('1918788754.032674281422271377722636813476490', '375879655.918128067210463347859799244975692632591', '2294668409.950802348632734725582436058452182632591', 82, 6);
    T('82450978365042214756445017.9804523005', '89524662733524727147672413077789255.773254557616802607043', '89524662815975705512714627834234273.753706858116802607043', 74, 1);
    T('95656867192740495372078005148360.47159882190028081857636922610731214', '141506179569399063798.41868970386570', '95700000000000000000000000000000', 3, 5);
    T('2.5', '5946125048994457645693208696342081494784.77138314204765450826604534979213', '5946125048994000000000000000000000000000', 13, 4);
    T('2761186579993266412708612426417.39', '2614420557797371310609245097179195509169.1630382042297464704854402', '2614420560558557890602511509887807935586.553038204229746', 55, 3);
    T('770092702674016011502941830468485023947.055054290415925071376', '4244524589712434445028456555634.89490119628118', '770092706918540601215376275496941579581.949955486697105071376', 75, 5);
    T('95494047311000394134.3225838', '72127137619516518872695557317454.690102888857596326931939', '72127137619612012920006557711589.012686688857596326931939', 93, 1);
    T('1063146868964.41', '68941222560374.78985836446491806', '70004369429339.19985836446491806', 92, 3);
    T('3.2264915731667407126008316', '499024340528439595143088410.50080590838784553423899', '499024340528439595143088413.7272974815545862468398216', 99, 1);
    T('23732.6336632204926011696290159941853', '85738963369285399.2924251404084047982990087915225', '85738963369309131.9260883609010059679280247857078', 54, 5);
    T('845749050.5087', '968827233087158246623650111542001.8059993116', '968827233087158246623650957291052.3146993', 40, 3);
    T('55703670797002005907118.99118073486949056632488269533', '647.9847031940598151', '55703670797002005907766.97588392892930566632488269533', 84, 1);
    T('58.170896940006', '5935854835841752323658688212.234397862904536499', '5936000000000000000000000000', 4, 5);
    T('8.8392279068440527177754306792676743087886', '864662321415666525030628178689998.3067712096843', '864660000000000000000000000000000', 5, 1);
    T('6.86955739708191389897267214', '452228666022799745749495300135092688.976154073', '452228666022799745749495300135092695.845711470081914', 51, 4);
    T('842061.145115749', '96387901158126.38366399429523139049934496669714', '96387902000187.52877974329523139049934496669714', 99, 5);
    T('5828120699872051813524005117681071576.3531235858', '530354381638244739005665484319.860141626791312', '5828121230226433451000000000000000000', 19, 1);
    T('31628100335505386543102.8814396781101867704580695011', '2868894417.460', '31628100335508255437520.341439678110186770458', 44, 3);
    T('527380720853701.9518282370765203779354205130851789667', '3807754317741993994329605627786784345260.38058446920105680306385791', '3807754317741993994329606155170000000000', 30, 2);
    T('1213877054955262217659333328599102.11623059154456', '3278195081381281690395388036554.2138989167790', '1217155250036643500000000000000000', 17, 2);
    T('9407909990937.40816737220251637', '3012713964090821230675585949635038552.8250476321930286', '3012713964090821230675595357545029490.23321500439554497', 70, 1);
    T('80605798842835723.69118340834400470018', '9826575601457892491008466.55213048606493892396085619215120986', '9826575682063691333844190.2433138944089436241408561922', 53, 4);
    T('2019.06', '4408968.0', '4410987.06', 90, 3);
    T('1873364986243720596991399356804667.8421619922466509417715462448230657090824', '661084720913298608912.41830455735', '1873364986244381681712312655413580.2605', 38, 4);
    T('415273108274788454749431.29431426132708248116879286807656385225', '470536415770731259479.5410492606175032672036720297898664', '415743644690559186008910.83536352194458574837246489786643025225', 97, 4);
    T('82206038099315631196560247941083092.1612534215203', '5646124907.759271786491151583849222800444', '82206038099315631196560253587207999.9205252', 43, 3);
    T('8829697190148183927849.963671776515154389220933', '494970035237083078707666488649370.84286207029725652933652904', '494970035245912775897814672577220.80653384681241091855746204', 73, 0);
    T('9524369019009213985475.27541955020584550106578509583', '615811701693888132867.0871099804255164', '10140180720703102118342.36252953063136190106578509583', 60, 0);
    T('2655619003342276.211481568', '153236830060720.54288125', '2808855833402996.754362818', 92, 0);
    T('81801039942902969443512.698009395512285289292987525140', '9808742434647157259.275727327711', '81810848685337616600771.97373672322328528929298752514', 62, 0);
    T('1326523051141191856541698391661.071350', '944428813.120405297195350390983112', '1326523051141191856542642820474.191755297195350390983112', 80, 5);
    T('794107573917336577733406126212.67849555953839951473', '8942071314849.4013878040920488789698200218', '794107573917336586675477441062.0798833636304483936998200218', 92, 4);
    T('2025816438114654901004541189.0', '6308195.189059578', '2025816438114654901010849384.189059578', 38, 1);
    T('518102129832133480857.99349403303350987312423871119', '36421784529000.766923553609344236731885', '518102166253920000000', 14, 4);
    T('99214263758303598888.0475071100303514740931051373896930813', '9987348.172165009273311597249674', '99214263758313586236.2196721193037', 33, 5);
    T('1622150764345251058.584209376094198', '636245057367685246415651762.8308', '636245058989836010760902821.415009376094198', 77, 5);
    T('52707566956.227463663324697834264679', '20494150409697069073587216103803720017.84', '20494150409697069073587216156511286974', 39, 3);
    T('329539208523703632121941358.33583398536489396188689', '62705060630661311393068974108934629.354966822364609614521', '62705060960200500000000000000000000', 15, 1);
    T('431373912982666514125581486025808.93317296113332', '73808.6', '431373912982666500000000000000000', 16, 1);
    T('79423053842412094017533765192.2950167391879859433490361564634', '565487674078207455405555339.414417591749201259749961630', '79988541516490301472939320531.7094343309371872030989977864634', 68, 1);
    T('61525449334232041954595386622157235.56340564352811816785418299270', '622883413011275218668.828954848388549626406767501418', '61525449334232664838008397897375904.392360491916667794260950494118', 78, 1);
    T('31636.75657093302347384964329', '34900937991948604493.0881401847670720002657781032504210571160', '34900000000000000000', 5, 1);
    T('5377203353065701430077778472410040386.0', '9357757337413993481873404.1111838', '5377203300000000000000000000000000000', 8, 3);
    T('842899996920.22957577315787537185655739120792', '4497532005.5912964447897951670899945597', '847400000000', 4, 4);
    T('9144064764273427094422.3574119364231309505001', '93675884664223.627989346865608306636582745', '9144060000000000000000', 6, 3);
    T('957037.7304635231477222861038598509273299509888', '879473537242221756632696297384043918011.52065110882913005274', '879473537242221757000000000000000000000', 18, 0);
    T('2970821.074782672336523485218725479034', '7531503771045894734943315.7548994521102377998148361', '7531503771045894737914136.82968212444676128503356', 48, 4);
    T('97143142695.852807990510091747140879067345232584', '9997215898278816065646392165877950.6224935321981888369969938881559340481', '9997215898278816065646489309020646.4753015227082805841378729555011666321', 82, 3);
    T('1482893942090234053695425124.39', '29463.8523198219390279672300654239777566123', '1482893942090234053695454588.24231982193903', 42, 0);
    T('44469.4974886770363513511519657411800', '87205500383732015.89', '87205500383776485.38748867703635135115196574118', 59, 0);
    T('73307192897.872280355362646378152434642553', '5954874385873.52029728642388', '6028181578771.392577641786526378152434642553', 85, 4);
    T('396.58435604886258245989783534334335150', '8526446649433177335219528735.756957793477760', '8526446649433177335219529132.3413138423403424598978353433433515', 94, 5);
    T('4786774541849183.58498774363143805325428438', '333870444880.9751695729199693253688330757842690', '4787108412294064.5602', 20, 6);
    T('956397918845650282741956.586647292026817037099504', '1044605959392226768219658543000562217.74961295027131030979758417355917941902', '1044605959393183166138504193283300000', 32, 3);
    T('54278348399.34254708759617723414501', '96563.44301215895693954342193071898', '54278444962.78555924655311677756694071898', 85, 6);
    T('8219285988560290195474603115888610379059.5191566650124145711', '11928584492859.40361370787722600', '8219285988560290195474603127817194871918.922770372889', 52, 1);
    T('209583263280628827050868207782609.0', '63641368938965607862241875316538645.96979161', '63850952202000000000000000000000000', 11, 4);
    T('899922029240418359.245927738', '6542134854859128688747397328538521142.2207758134038939743', '6542134854859128689647319357778939501.466', 40, 3);
    T('1982666284322445983192532232.069195147769', '858033957431535105323.2804326218272868826778349520', '1982667142356403414727637555.349627769596286882677834952', 77, 4);
    T('98936713.28722015399', '4622680674083934008.7008184359', '4622680674182870721.98803858989', 40, 2);
    T('253767329480.297300653398', '8317258677670024996183876301098.498002', '8317258677670024996437643630578.795302653', 40, 1);
    T('9344477426946458221652531175722293.222108512640755638705317727125879565', '151763021581.708987058009616', '9344477426946458221652682938743874.93109557065037164', 51, 6);
    T('460084499111.0268525803128181', '58887200601.1797854293926214561353', '518971700000', 8, 5);
    T('5597137119608.65712009725698848539550953735', '9666857.53394284148761', '5500000000000', 2, 1);
    T('486445996625154447944206385400900212550.0', '2840998065262205901971585594495360.74637499618494180460454253014049', '486448837623219710150108356986494707910.74637499618494180460454253014049', 98, 2);
    T('154507924096364628552511.33077302415729491580441604998', '51665255665774825813316094535959395123.9120226731489237538322455950', '51665255665774980321240190900587947635.3', 39, 0);
    T('56133611218962181101.15490726853844809694686779746150474918', '65231976462817780066344989599711379766.60551610199067139178', '65231976462817780122478600818673560867.76042337052911948872686779746150474918', 95, 1);
    T('18879606.62866052176820', '379476105009128784230.627583099525499', '379476105009148000000', 15, 6);
    T('109199.39', '2817339227592097593502.47909297040712479046877204410420', '2817339227592097702701.8690929704071247904687720441042', 55, 5);
    T('791340220.70094930540920145126', '853852467839074963669369181263456.48', '853852467839074963669369972603677.18094930540920145126', 79, 4);
    T('423525911266585636113102801186.0', '942365103505764461412023295454454715940.13447161138119591889', '942365103929290372678608931567557517126.13447161138119591889', 59, 0);
    T('29747562090.683670809519183509211036', '25607437080817693730066127347859707525.45908531293606509947472802700199171', '25607437080817693730066127377607269616.14275612245524860868576', 61, 6);
    T('7481344107.0213098135885037704580562845539640882', '60805813271998.737750314381831223544879289741195', '60813294616105.759060127970334', 29, 1);
    T('811.79217090249798600585', '678734756.01839801003495330570', '678735567.81056891253293931', 26, 6);
    T('362115806869064815.262354038733801083526505261783', '65633509417769037.90093114724064299', '427749316200000000', 10, 3);
    T('8993775107381018281887014984062011410461.403164961608853692841384', '735387257332820223739.6961042295785414554948287238127093962394', '8993800000000000000000000000000000000000', 5, 5);
    T('49293984750089238772838615734.508683097759938', '1195447.332099543898048959791962558614829446051', '49293984750089238772830000000', 22, 3);
    T('3737165669442214038957454.13143644973617839727695224143', '4689797855104733329421863187373.50871609771637', '4689801592270402771635902144827.640152547452548', 46, 5);
    T('6975279.29531969606192995473575814934990', '6215269209131688223060429812888592259969.1556209537060237460340581756071', '6215269209131688223060429812890000000000', 30, 5);
    T('28258832100919029737082.2711865713388440', '1.1505608687012913733984698026982652', '28258832100919029737083.4217474400401353733984698026982652', 60, 3);
    T('9234636070753312364392349078446925144.5559627634735', '6166997455417392960095529742025.408457477932168803974', '9234642237750767781785309173976667169.964420241405668803974', 96, 6);
    T('75499.9212715570843962367964978472524093542032', '263288.7238349060388474434', '338788.64511', 11, 4);
    T('451985385627110129460632.82578', '891998995162496053074195832329835.72047014752', '891998995614481438701305961790468.54625014752', 76, 0);
    T('1224820342528.460358927924356649494020328063', '5101043612237739799935684356.212324562292576516922', '5101043612237741024756026884.672683490216933166416020328063', 73, 4);
    T('17093307330327.0551673178542267', '6.9', '17093307330333.9551673178542267', 74, 0);
    T('310692057136409787389088868637987645.84925956054789824830979232545', '1574.85215786441206671023912', '310692057136409787389088868637989220.70141742', 44, 5);
    T('35695142132777461402581990189.0', '1190987.0', '35695142132777461402583181176', 62, 6);
    T('9284825352655767123968002395604261382148.347', '641026340679356453577435446.78849879584892853715555288044', '9284825352656408150308681752057838810000', 36, 1);
    T('79508746241011375381654255631080101.35892', '387843789806120168174260333793232840.443', '467352536047131543555914589424312941.80192', 51, 6);
    T('7494838119790348006324145447051.1977476072376983724234575620478194', '43339881272200810182785666037.308583495530578110087119', '7538178001062548816506931113088.50633110276827648251058', 54, 6);
    T('857776135573.155518632162126346688118570253590670775', '116613431434426.844667801850560508244481249', '117471207570000.000186434012686854932599819253590670775', 62, 1);
    T('4732047864.02', '96012808102367265927074850679091862666.58619415823310855802220', '96012808102367265927074850683823910530.6061941582331085580222', 81, 0);
    T('369351112810347547109456.4238523687427229405139226436567752671', '425663043154.0331167343513', '369351113000000000000000', 9, 4);
    T('3062987087676.4341633155235408824755719881177305418038', '230.092147659481985172881657756', '3062987087906.52631097500552605535723', 36, 2);
    T('164794711922471368811213969.232001414017397048427940040', '7960713164668620.070991804368737905676075', '164794711930432081975882589.30299322', 35, 0);
    T('69680680535663015277.97833837186', '42963.3029312430379811282110395633800931905', '69680680535663058241.281269614897981128211039563', 47, 3);
    T('806867796.40357652', '294312303578725426827.652722382020240', '294312304000000000000', 9, 4);
    T('2850351436758555966.798112', '9904352.7315630922310', '2850351436768460319.529675092231', 52, 4);
    T('5649327503118684.900521983143846072275552090132', '313.71639628431289332', '5649327503118998.616918267457', 28, 4);
    T('7.727080576316', '339.5984', '347.3254805763', 13, 3);
    T('51407800814400.473016239808774', '2.648021372147353466821583348', '51000000000000', 2, 5);
    T('2250098704417623204837375.428818374457412042645', '199149759305138596.824005162389650209321230787184', '2250098903567382509975972.252823536847062251966230787184', 88, 6);
    T('32518401.62964149399609276082789692', '207054853096134356200837661354.8134417076794', '207054853096134356200870179756.44308320167549276082789692', 80, 5);
    T('5305707346.56497865106017268753429864353', '798.85114327939064902680319097', '5305708145.41612193045082171433748961353', 66, 0);
    T('724916183251531912917265075499491072.161706', '74297348634324.884836790712524235929556075986', '724916183251531912917339372848125397.046542790712524235929556075', 63, 1);
    T('8647642133043924049993939242.6972804253749360522781046449182', '89302829832641555702.49642079437437009982132291', '8647642222346753882635494945.1937012197493061520994275549182', 69, 3);
    T('30105699.982634348', '6448842836342584574831571770460957399155.2039974', '6448842836342584574831571770460987504855', 40, 6);
    T('87416.37', '9051883186925.06308955495', '9051883274341.43308955495', 96, 6);
    T('9802448539371246989972850561474393234963.47', '820332717.799429502572', '9802448539371246989972850561475213567681.269429502572', 95, 2);
    T('9244735376049917391.63000098717331266', '3090640738959941076.044354551950949743', '12335376115009858467.674355539124262403', 45, 1);
    T('45220003920.7804365250060633286691252176455', '748674895610826170623.920149554422692678344425060476157', '748674895656046174544.7005860794287560070135503', 46, 4);
    T('88620811021834598145.3837912252464941179091144750517', '940970280209811877.539268667', '89561781302044410022.9230598922464941179091144750517', 54, 0);
    T('7959726072251539382036597660345.905848373358719995038101104566088', '859444085864890086568.35585879', '7900000000000000000000000000000', 2, 3);
    T('38297919331698786793471.2420312', '63394905007744388880509.0959349285624980297945725938661023759055', '101692824339443175673980.33796612856249803', 41, 4);
    T('44869602208293.0202921818905867622218025415360595660', '774471324055624698067251.9145150159100196028', '774471324100000000000000', 10, 3);
    T('172210452963609237577496214702821033492.018485406215580', '883572265814431824177066205053.85357983', '172210453847181503391928038879887238545.8720652362', 49, 6);
    T('8548825250070563002534432804920558.7909488282366311177366', '2821303349295090866353956933719755413360.644912409336278687601595862323880', '2821311898120340936916959468152560333919.43586123757290980533819586232388', 74, 0);
    T('6339571151704762042673047633.87949534892724108857393990', '8034.1333289854929027314', '6339571151704762042673055668.01282433', 36, 1);
    T('212926537801110194367927821.5814374345714398776961218413267910597', '4623705229209924291645250339.7134124126597114081696', '4836631767011034486013178161.29484984723115128586', 48, 3);
    T('606111630785625874498057312.44400118068102595645295', '88.6950754869530161828935', '606111630785625874498057401.13907666763404213934645', 57, 2);
    T('2812944720.3472128824274112296163336475', '77626694011731868250.7333356838640826760668274', '77626694010000000000', 10, 5);
    T('8275364730752.313454142521396521349457472516052564007', '45947808102618427210353357416128895846.77435272825321225542294', '45947808102618427210353365691493626599.0878068707746087767723', 60, 1);
    T('31875314652624711689141.684699108834074239196343', '98330075698344119639174753392610370.87710855831775984030429243', '98330075698375994953827378104299512.56180766715183407950063543', 73, 0);
    T('0.91441249430388927398', '27338388307660.90645341', '27338388307661.82086590430388927398', 93, 2);
    T('8521443534719.578731905808392754493429193899408', '98222020723499062006206340402307958962.1466073953654614943', '98222020723499062006206348923751493681.7253393011', 48, 1);
    T('2304210936608439736938709809178198.90385794942561140840885098245045265094', '836973575111216558.08566365713887635', '2304210940000000000000000000000000', 9, 6);
    T('65691465702.19', '980336975973837411933.8497691560192457695628056', '980336976039528877636.04', 23, 2);
    T('75961232627378620226915054549271120.8583023945946', '47198260276899652.4191639218952', '75961232627378620274113314900000000', 27, 2);
    T('2265576272.18505886691658615', '73734784668259040561205.7496673029553069586', '73734784668261306137477.934726169871893109', 41, 0);
    T('97747107883494.43589683236', '924340176873035427347314427522945694.230543704150393862927670825288', '920000000000000000000000000000000000', 2, 3);
    T('52687947891401269225924976637250320693.9939577070046217274314716483', '40923798664728.275233463', '52687947891401269225925017561048985422.2691911700046217274314716483', 91, 5);
    T('53.89504740629525546380', '843.277298294', '897.1723457002952554638', 48, 5);
    T('716609617447072862852.04508401', '973623127.3936', '716609617448000000000', 13, 6);
    T('92440629872.4015993738217975177341046466', '93258191469517235901.07512203582375064614553758', '93258191561957865773.4767214096455481638796422266', 79, 2);
    T('2299082894763967448514290855122.98496', '577338928400471.6220783720524582149142673327011766528', '2299082894763968025853219255594.6070383720524582149142673327011766528', 95, 5);
    T('79686344061822911360461296726.09326793', '34201598046616289009629.692767117629326670573', '79686378263420957976750306355.786035047629326670573', 76, 2);
    T('95254466533206520580284473557360958627.47383461537744585382540470950370478', '243096.637459', '95254466533206520580284473557361201724.111293615377445853825404709', 65, 1);
    T('667992685178741807856230410542.6452767195946074412033833622525', '43035504544398217386158.4652955241207749', '667992728214246352254447800000', 25, 4);
    T('4353241304313205.86534110469579657646992', '43906806115369599427445715.244165825544679029848335307', '43906806119722840731758921.109506930240475606318255307', 54, 1);
    T('8289677368169.346', '86500605084932807010509603330146772.4781626066086331', '86500605084932807010517893007514940', 34, 5);
    T('607075366.00119923674491761202333757439466', '11715952.942', '618791318.94319923674491761202333757439466', 80, 1);
    T('709624496369275183037797.9137726333508277', '967281891.140656841606339303342141511120242437', '709624496369276150319689.054429474957167003', 42, 4);
    T('501206834760.04420813103047', '43709232365859970224443039485.25718', '43709232365859970725649874245.30138813103047', 64, 3);
    T('75245141499683240706.6359439407325443610394676', '8381444628173580867884334494144303.5213422434178856603703332237', '8381444628173656113025834177385010', 33, 6);
    T('830768628.71944277364595271423067', '16282817363927918796798023.988', '16282817363927919627566650', 25, 4);
    T('98890942304882.731612304487834700567666958', '17774801053947098089959235605339220.1959776634075583163111', '17774801053947098090058126547644102.927589967895393016878766958', 82, 6);
    T('485418626513434904317526594.05242257373953191', '69215943317.39413494843253910825242731859758652', '485418626513434973533469911.44655752217207101825242731859758652', 95, 3);
    T('31093942446721196467934973101180460739.209409471385069917255778694', '80006410838288.26429153820163287860816906', '31093942446721196467935053107591299027.473701009586702795863947754', 84, 5);
    T('541113647277227420.20914595890195243947516075428002200', '882652236913123080601886015.5404336', '882652237454236727879113435.749579559', 37, 2);
    T('272273075938621551509868.2126109932324396', '9560838627978255907432609287278.995946030300396', '9560838900251331846054160797147.2085570235328356', 66, 4);
    T('99751394663590945797.1592208256647562', '9736350151722999902338514.40', '9736449903117663493284311.559', 28, 6);
    T('7891755675.2784490329838436001580426', '821818193992.42988293', '829709949667.7083319629838436001580426', 50, 5);
    T('1579788303.3310893', '1.580880591973', '1579788304.911969891973', 69, 6);
    T('11422292596589072024625714.096332399', '72827186979910295279431133794093.48', '73000000000000000000000000000000', 2, 2);
    T('435081823266800515865413766233130644.748065350378985401718858767351894630562', '9002399.6735823', '435000000000000000000000000000000000', 3, 1);
    T('985789631879150774531535687.1431413', '368580352526637759561458135957886964.3294495', '368580353512427391440608910489422652', 36, 0);
    T('8509076.8288097951661218044206322158760', '8065253986201653442502.3989790199018', '8065253986201661951579.227788815067921804420632215876', 60, 4);
    T('1354250016986753578991210458646262455441.44201793229155397289367608171591594938', '1616789011862854141298182.956231249130', '1354250016986755195780222321500403753624.39824918142155397289367608171591595', 76, 0);
    T('43318435537569317386674019940186.9283647968974', '9009965.3378622267', '43318435537569317386674028950000', 28, 3);
    T('937521082165518105718972976.396585825767548625209578019', '3253601040475672591.192726537653558352356786913389143658', '937521085419119146194645567.589312363421106977566364932389143658', 86, 3);
    T('2040134431417645946672363.74012001894684817357681323284393591', '863676792430393.45680344771', '2040134432281322739102757.1969235', 32, 2);
    T('98256764439078088032160543.86567', '9184091425912759744375.788735618718346', '98265948530504000800000000', 19, 0);
    T('192061.293819434036941645536518762372748924032', '15659439.5326603873074378521210157369', '15851500.8264798213444', 21, 6);
    T('8071198272152359934314107.5530016338837106625984768866388888137', '2025079333.27336490483964376651968', '8071198272152361959393440.826366538723354429118156886639', 55, 6);
    T('4463.229', '4959561376961864.15324110640', '4959561376966327.3822411064', 26, 2);
    T('851577603422136745739957633452072498546.869034167221320800533028', '39606605327948421167.68673992', '851577603422136745779564238780021000000', 33, 0);
    T('39140468323374979116997608487.8858002266552515486489751', '141009470604922863452836828.8', '39281477793979901980450445317', 29, 5);
    T('6.003742079', '2928062037707450918.16949848488837076', '2928062037707450924.17324056388837076', 100, 2);
    T('6675944201203942.67322101810623350796648737172486473', '72712670624764667530920787087.0', '73000000000000000000000000000', 2, 0);
    T('72564795.283666303775396123204', '574.110872251003625607', '72565369.394538554779021730204', 99, 6);
    T('577314303166490126382722350914121.233077178876346620082531191907', '2428775484343264912089711653670394172753.60055154573593562494995571319', '2428776061657568078579838036392745086874.833628724612282245032486905097', 89, 0);
    T('8379764220036866380687.2844382451334900759', '86095892117222303836261.073', '94475656337259170216948.3574382451334900759', 70, 3);
    T('646082943.6725280553872497309617796719599719', '694881679064221264726941.637060620114928819714594003630703490', '694881679070000000000000', 11, 0);
    T('55115835273111790906.9535092103534881415187686657702480388702', '49061751119716790741.159905242538975345175691', '104177586392828581648.1134144528924634866944596657702480388702', 92, 4);
    T('2217577.184', '1485563207436796101454786950.653644248864406662', '1485563207436796101457004527.837644248864406662', 85, 4);
    T('258792726693794814267.6968580781214771327777733344161893339153', '6863856465503.93', '258792733557651279771.6268580781214771327777733344161893339153', 84, 3);
    T('546060047455237905351636019693816.405817172453856850996100057180645566193', '520734382823871234161.1384854642138', '546060047455758639800000000000000', 19, 2);
    T('7334180521502309336066407404259915372.2823697422507483654145', '693623.206372873480196434801480814034', '7334180521502309336066407404260608995.488742615730944800215980814034', 87, 2);
    T('9857694318.42890552236498483277448536991344', '201758486675215258978020363.0', '201758486675215268835714681.42890552236498483277448536991344', 82, 1);
    T('19973653763482521246879.604716043', '491274616468136178125517521691.48579269642854984119425658598612', '491275000000000000000000000000', 6, 2);
    T('3277588068397420399043852995.277132355737871042106543', '4128992323003345152024347391385408550646.5950067624070849910', '4128992323006622740092744811784452403641.872139118144956033106543', 82, 3);
    T('3565.433805222', '9.166327109241381158686174779812216453', '3574.6001323312413811586861747798122164', 38, 1);
    T('9203318173605020702123146720927949.782332815896', '42087388620140193499262895333989401.7170540933402949279024760', '51290706793745214201386042000000000', 27, 3);
    T('806352404368478053.787863739460343997947206650727627786047', '34701546918759651503613733231427557514.0712', '34701546918759651504420085635796035567.859063739460343997947206650727627786047', 85, 4);
    T('90717116390.197546964029605827', '66673695642605272952910260364.704791861164292284952992913733232412105', '66673695642605273043627376754.902338825193898111952992913733232412105', 95, 4);
    T('50039583452332321055280112294.19800', '28217559914408974306212634759799027811.42036592520516352', '28217559964448557758544955815079140105.61836592520516352', 89, 4);
    T('96788807577579143898685435.91325965450', '97952516.4679', '96788807577579143996637952.3811596545', 96, 1);
    T('85951318271641948820027943602395.26068227713683804289717157661543181501', '6352905588237245780.0239293348514013309446605138214057944243', '85951318271648301725616180848175.2846116119882', 45, 6);
    T('357654572493317200329825059907907214464.1189', '30404493666843328.7939469479', '360000000000000000000000000000000000000', 2, 2);
    T('792531690174984488788567.558203801920179739196974179', '51723357405581589224664029953.8463273472341029054943990032483928', '51724149937271764209152818521.4045311491542826447', 48, 6);
    T('2940601056985003229045681171045474525138.7907755', '531296857701712354233601804.912468641685865', '2940601056985534525903382883399708126943.7032441', 47, 3);
    T('9582957481725611656918680181666943.3441342167104296600', '934859450112259340.52397175439719600403110283', '9582957481725612591778130293926283.86810597110762566403110283', 89, 2);
    T('4373492107443479521653470610.63011566', '249705979060921727092355385103642606.06644312718562058', '249705983434413834535834906757113216.69655878718562058', 55, 0);
    T('20674533577277233144.64767616223', '12155003371013030394275259313.06055806537933927', '12200000000000000000000000000', 3, 5);
    T('664280800085326052245213426025.467077794', '7718531338858288302419029133948.11032046276485946778614262', '8382812138943614354664242559973.57739825676485946778614262', 67, 0);
    T('8347700832908721921583491901277559215.25426462455209631954', '226395.19501', '8347700832908721921583491901277785610', 37, 1);
    T('658132427775431.4718211823199510', '91253.0203695535226809209836985651', '659000000000000', 3, 0);
    T('244401388636136748.34083900591687224902923', '9290356227367421197.40217613991574956113555266084', '9534757616003557945.74301514583262181016478266084', 56, 6);
    T('91.977642062454312269055891933100321775676', '697046321629425577725224571522365014158.665400327851231043687482', '697046321629425577725224571522365014250.6430423903055433127433739331003218', 73, 5);
    T('953441042003.700019354757', '56836732760876196320139038366593460.942', '56836732760876196320139991807635464.64201936', 43, 2);
    T('207248370676410588607271984938423.569', '2683432589.228945350526', '207248370676410588607274668371012.797945350526', 86, 4);
    T('42501063434522.48952019519048918272832745515036926', '2.1156889716794914186823743263483286786', '42501063434524.6052091668699806014107017814986979386', 99, 4);
    T('8658715022381508259867530194682839940.6944133201837227411661423795', '94311421615148421298960.35439648535342', '8658716000000000000000000000000000000', 7, 2);
    T('1470532082777155340059468060578397219.802851303907138529177724789179', '16466487840019931065.931154522504631', '1470533000000000000000000000000000000', 7, 0);
    T('67550022451232.95172541635336724078099741', '541260619779175063002616952546.0', '541260619779175130552639403778.95172541635336724078099741', 87, 1);
    T('471545839963021827023424981320056421438.617699140069774579643', '2701405.0', '471545839963021827023424981320059122843.617699140069774579643', 62, 5);
    T('804287535970748610086219410.5766931', '9872138548320067912671.57', '804297408109296930154132082.1466931', 58, 1);
    T('215529703516142718423052512939895.08496164230318354702947313322980', '52949893075047616334281440272895490345.122685797929', '52950108604751132477000000000000000000', 23, 6);
    T('45325823476868221.29733596020990982364993724079902463', '3816319074620354816313671873.47784483', '3816319074665680639790540094.77518079020990982364993724079902463', 73, 2);
    T('55808812320589.639212', '3143690129537695654442674.6245301', '3143690129593504466763264', 25, 5);
    T('6117.060478464737302862', '56756829390702915963259998.0057', '56756829390702915963266115.066178464737302862', 81, 5);
    T('12.648219677865950', '9845219426404029699410403093.777868715964732168', '9845300000000000000000000000', 5, 0);
    T('4522012999.29577621452850277148865684084790', '2314450.280', '4524000000', 4, 1);
    T('77806555798517596650371754052235.34', '64821962.6', '77806555800000000000000000000000', 10, 6);
    T('73751048957371812755822.113171487', '83107464.581549', '73751048957371895863286.694720487', 64, 2);
    T('50059057.94', '37252773511.3035653739971611314895884022767622494049', '37302832569.2435653739971611314895884022767622494049', 64, 5);
    T('9940334964654808694876769345805577880.76455535232920183554315878', '325062.865902', '9940334964654808694876769345805902943.6304573523293', 50, 0);
    T('928495249586511698939789888.2737006752449043', '614959282682216598370870969.7282274040466', '1543454532268728297310660858.0019280792915043', 94, 0);
    T('86726939282.07270135168', '949786817701648966290782617.771828203793372700213043824596', '949786817701649053017721899.844529555473372700213043824596', 81, 3);
    T('26545577189771186000509.988029374754674', '27168879999558677746186208647332.9', '27168880026104254935957394647842.888029374754674', 75, 0);
    T('8577152143676262.8778', '299131069.74', '8577152442807332.6178', 41, 0);
    T('39640963340647.943017421930699729085439', '2729772455803348721285738509.4239', '2729772455803388362249079157.366917421930699729085439', 73, 3);
    T('809142556150833809141396792501550.258445', '2641392772365242016590745347895367.34224019315190350383', '3450535328516075825732142140396917.60068519315190350383', 72, 3);
    T('240625095.5554939416', '25708683.073', '266333778.6284939416', 67, 3);
    T('253150657679233618949062835492679337318.23569014841118003262457494673591446', '7852.6467171472665992', '253150657679233618949062835492679345170.882407295677779232', 57, 1);
    T('2695873416975945463695.548303349157156018501000311809493916', '434335351007139307653171203344958675220.67945989180928436483046024599', '434335351007139310349044620320904139000', 36, 2);
    T('36.200981779', '38060939.4217703701427232065866636', '38060975.6227521491427232065866636', 98, 0);
    T('52578842865145276862899070095.595308174315952960225327061481071', '4317404806359830838456587049005453793.4060502545140049034983598669963701954', '4317404860000000000000000000000000000', 9, 2);
    T('2534128.869', '694326690496773.16265241563661473944431618347', '694326693030902.0316524156366', 28, 6);
    T('7242242782371376.68625216939403292775', '995892826.9249701320594459701242134231', '7242243778264203.6112223014534788978742134231', 47, 1);
    T('76962979590602547863.00032665539277', '758113815410015436930503607208901.366833457120176311419992802994', '758113815410092399910094209756764.36716011251294631', 50, 1);
    T('549032357442983947227616684577138305.2658861352310501103045875736999575145', '932439741503012446544524613328868.565786077571448011245991798', '549964797184486959674161209190467173.8316722128024981215505793716999575145', 80, 2);
    T('546544413623238742.3462358401827803675630367', '438331495211948581703894691475923636.7', '438331495211948582250439105099162379.0462358401828', 49, 4);
    T('949586762749.01278892', '21230051499899582.9357597835655676968945840690932859', '21231001086662331.9485487035655676968945840690932859', 73, 3);
    T('99726170.2118662624820888570245914', '209922864708717545335584585747.52796', '209922864708717545335684311917.73982626248', 41, 6);
    T('211653641596123147970830107628672.4179510703923582217498472310549615', '466317361490800328491.86585087139922211980630249737', '210000000000000000000000000000000', 2, 6);
    T('5.0278067769689798343041852430091374', '47804783665408014582484644820.49674677105121545019872', '47804783000000000000000000000', 8, 3);
    T('635215820752229358.5992508425598', '848428.807', '635215820753077787.4062508425598', 35, 6);
    T('1516820948896.705495741911825673663778682183478', '6562180665473982.3920236750330265165845587795017290773358', '6563697486422879.0975194169448521902483374616852070773358', 59, 0);
    T('26157976569763289123.5522536419486865982764392197', '43220628517379408583309140324724892947.6', '43220628517379408609467116894488182000', 35, 4);
    T('218283378474114032.159745937295676067648509530893750459632', '31372.73409086057824012347199442078', '218283378474145404.893836797873916191120504', 42, 4);
    T('9355400544755818877322962147897454978378.46', '57406351205553271818249.894858471402666442334605669351307253545', '9355400544755818934729313353450000000000', 31, 1);
    T('63112966765226452896498114.16866759432224000078099297931362', '2021781298125804945620007932889690565.77040728114695509420379167631650775', '2021781298188917912385234385786188679.93907487546919509498478465563012775', 88, 0);
    T('3838506.6', '46380897204358803786160527695783064.9', '46380897204358803786160527700000000', 28, 5);
    T('8686269934171446684079064822620.9987193', '47735633552.697875003990150503547692033349', '8686269934171446684126800456173.696594303990150503547692033349', 82, 3);
    T('84438872133347226156459789.196145730871671591313795378747877776', '852973123709325036018314506305581.8443367692410074588128061391768467975350', '800000000000000000000000000000000', 1, 3);
    T('3525836971981253889234341.98020640416041584209482753058267463', '44086951416600789989677920.849470744981471530553051390138598401943', '47612788388582043878912262.829677149141887', 41, 6);
    T('335562084795031231390021538.17769992', '9217415081734587816687361401402386734697.38487344523645759011871595557', '9217415081734923378772156432633776756235.562573365237', 52, 0);
    T('43984290680848540710894730755301367.0857187723100767015116835151', '3726.34882638575528433197807469763', '43984290680848540710894730755305093.434545158', 45, 3);
    T('578478056435923.25648055462572193828', '87734820400472749671725984366976225.566968957208416427402994978938029', '87734820400472749672304462423412148.823449511834138365682994978938029', 78, 2);
    T('2586948682528426091244.994256973470094003582', '7054951115674860246720220077744732682.864758166', '7054951115674862833668902606170823927.859015139470094003582', 95, 4);
    T('4604794595948095357996660937708438372887.99047293014568676168186229188166062255', '9312296872913190635940693044.9404281279267609917052399308614', '4604794595957407654869600000000000000000', 23, 2);
    T('6117602.8791805205880350203455424781', '6.58692315', '6117609.4661036705880350203455424781', 52, 0);
    T('78771797092648234369555419989.372251259829200931914536778748515', '1642426787315127695235693737194.4747723626811435846715193', '1721198584407775929605249157183.847023622510344516586056078748515', 82, 3);
    T('592.62353066132196292364963686837', '6067817.7', '6068410.32353066132196292364963686837', 69, 3);
    T('9664812120393147969790209105302.781101', '82.03511790296446953501349', '9664812120393147969790209105384.8162', 35, 3);
    T('6734383344.6779194303269665513345691560161', '7680755004007.98595384052', '7687489387352.6638732708469665513345691560161', 80, 2);
    T('6822875405015930655235222034900332447.836881596359073637901144812082646', '92223569880985322632031357785152542.924436790822136437018309', '6915098974896915977867253392685484990.761318387181210074919453812082646', 80, 3);
    T('375548513237756374664702485531668176.783', '653648372061065803.069689818279534074078881659038949756', '375548513237756375400000000000000000', 19, 2);
    T('659348926967505.59457478809893787708020101', '61601705999002657334380399827208843.27425003310957249684172822513797', '61601705999002657335040000000000000', 23, 0);
    T('640258846623191.039037107395356584122435', '56999441653641742736800071357298098287.612156625164320460542941951466', '56999441653641742736800711616144721478.651193733', 47, 2);
    T('9900949.3254136684895', '764600789968203235.0605551417487233900867027761', '764600789978104184.385968810238223390086702776', 45, 6);
    T('77392321024090614.63799362043816903569108826', '167373336057446585490294907871942860.49566558417232693307250', '167374000000000000000000000000000000', 6, 2);
    T('288568733.471484865033389929881188467', '67323672922918059481251.816797064815476871251740454', '67323672922918348049985.288281929848866801132928921', 54, 5);
    T('9513525914287502124686087983374714876.7958377097821328764111976668774', '723944462020263579.00212048330980065295884522684095752', '9513525914287502125410032445394978455.79795819309193352937004289371835752', 80, 4);
    T('678716308249113178240561672234335697330.2899901752470474624428391', '10180674862836214125389222744051.595378531966082444125237742989272', '678716318429788041076776000000000000000', 24, 0);
    T('9382925326015513.71447212', '15739345069224874053255610.8', '15739345078607799379271124.51447212', 55, 2);
    T('389168948191477298310317.0693895149363616692272932', '609146096167261633465599284932.2295093143867051511389695', '609146485336209824942897595249.2988988293230668203663', 52, 6);
    T('89431088791226587959867086028829.4155796575836199733203313888', '89096177378643763676293671356320454128.782942774756', '89096266809730000000000000000000000000', 13, 1);
    T('48769506.75341839890723426590595630504', '894395221610686162.997882978315280459846718', '894395221659455669.75130137722251472575267430504', 51, 1);
    T('10882763606668359969462817290788939.98047162748568254342001', '5569162409042854189989871.71631811505175276093', '10882763612237522378505671480778811.69678975', 43, 2);
    T('611724181210197269.2923678444281980316434', '52859930205215971626265711325029896286.597412645060403929176649142', '52859930205215971626877435506240093555.88978048948860196082004914', 64, 5);
    T('314.91', '2375188452755489682.6066256852096618866772558770449617811434', '2375188452755489997.5166256852096618866772558770449617811434', 96, 3);
    T('727296002377239314.80435951853', '733523119432625.4455815837228', '728029525496671940.2499411022528', 62, 6);
    T('361787149255020.7279528799', '2186334245.119806650539', '361789335589265.847759530439', 92, 4);
    T('736.7283631927502414344626806154412664977', '8.37860026335513137794933740', '745.106963456105372812412018015441', 33, 3);
    T('102282287196482472.580539347545581227', '83032.0', '102282287196565504.580539347545581227', 78, 6);
    T('754872.0119796632781903606', '827638369270404277759152446.400283247405289975824358', '827638369270404277759907318.412262910683480336424358', 99, 0);
    T('6674914870044.91907633308544003', '98437369889271790858264496888062161.48397951610060', '98437370000000000000000000000000000', 7, 4);
    T('34970271526.59785554605503953521076234', '231253.25471965205724860387036', '34970502779.8525752', 18, 2);
    T('68679.7905282693536592176', '579556505284614787408743.6954762627886', '579556505284614787477423.4860045321422592176', 67, 4);
    T('23027154651484887586153279581297265.491743147418224198825143', '392162313055958352951676.7794221426897725565065803757', '23027154651877049899209237934248942.2711652901079967553317233757', 85, 0);
    T('40343168942743.408566985', '74073987453211225381814.554197097061389278', '74073987493554000000000', 14, 6);
    T('93473743382040774628414533380.396967189333797', '40106.889', '93473743382040774628414573487.285967189333797', 98, 1);
    T('2648117476279954.65418038655960802048', '46436307040792526617.1196204582682314', '46438955158268806571.77380084482783942048', 76, 2);
    T('400877.2821228533507221137681803', '9920173078.33224176873844747385523775203123', '9920573955.61436462208916958762341805203123', 88, 2);
    T('8026791352599993018422288.009314581893835825971323874610813736335', '123632543334642323823216222788.19782575562842299896380781809', '123640570125995000000000000000', 15, 0);
    T('16006686795239266148058768938702.1362220135750390252', '487083247988.57932506804029000769254216', '16006686795239266148545852186690.71554708162', 43, 2);
    T('6750079339.0371632065917604430549', '1450190766600219471547477.760733', '1450190766600226221626816.7978962065917604430549', 73, 4);
    T('456801211351255805464.95759186200751834258559664765111', '731823570824648.77', '456000000000000000000', 3, 1);
    T('529352.9003231640958508461143763250581377152', '36855136973626860747976423946787.1814794446083304176350413', '36855136973626900000000000000000', 15, 6);
    T('422006056474333978132192.69665908763584498194781', '98997165098353658751.8031685049835570184825360920079368230', '500000000000000000000000', 1, 2);
    T('140957104006777977662222418929931637.1214344732131548598', '8341.153148313758083331108494370', '140957100000000000000000000000000000', 8, 1);
    T('3667162180557526937981152198260.964502268830280564058605758380715', '639834654.5954590055370945435523', '3667162180557526937981792032915.559961274367375107610905758380715', 88, 3);
    T('691565152701899022.49371149254115256067857101884087', '5047943.5607566888766342775095', '691565152706946966.054468181417786838189', 39, 2);
    T('72841967889878.431206011601', '3438438356961044241113425.9483718942715345408767488976810342', '3438438357030000000000000', 12, 5);
    T('8311422517930198859.2', '68766704975901226319211354803.598551434267921210026136', '68766704984212648837141553662.798551434267921210026136', 86, 5);
    T('3103503.295391053925081936698', '1785826913152598507049350.599', '1000000000000000000000000', 1, 1);
    T('936527866023978164160.39190680511131', '730272405351813042102924183659471.040990644244560283742587', '730272405352749569000000000000000', 18, 3);
    T('8551.079426011930381284644', '582087387.602', '582095938.681426011930381284644', 87, 3);
    T('6043444.32474790392627882202260', '6452832660786188841.3', '6452832660792232285.624747903926', 31, 3);
    T('7076216682307920315020.62412033356', '30605033615725147738268022236787610.87060637139696620102218262', '30605033615732223954950330157102631.4947267049569662', 52, 6);
    T('291097278268314194328857784.6479183119', '42853822203873950923.185', '300000000000000000000000000', 2, 2);
    T('3345590130241558194642968.8987852545188272', '869667377943721417961352019598.2860', '869670723533851659519546662567.18', 32, 6);
    T('1908563266458.808602782', '94266949434070412191015079.339', '94266949434072320754281538.147602782', 43, 2);
    T('2354220029350902321416885888.20329702540331', '6218513238026469707497773.5649157753264', '2360438000000000000000000000', 7, 1);
    T('77975143.1', '14817094734937835324.71321137079002496420740476780', '14817094735015810467.8132113707900249642074047678', 85, 6);
    T('404102378219509047142293706300.450774', '203612566407868733133743399.91751967643800422590975', '404305990785916915875427449700.3682936764380042259098', 52, 2);
    T('3775000201784474236446364.85443105309861781565277875340558625534', '919313149.7308904843', '3775000201784475155759514.58532153739861781565277875340558625534', 63, 0);
    T('652.2', '9447.7964835723168463966146', '10099.9964835723168463966146', 83, 5);
    T('3434.1672541673895603951465316', '881810920364.84405421750327343', '881810923799.0113083848928338251465316', 48, 3);
    T('897070163406062.1161', '50766582719227.5373840354841', '947836746125289.6534840354841', 42, 5);
    T('8546211910471383376147321779.3571263164973698', '79110128090765943.7330982221430376781', '8546211910550493504238087723.0902245386404074781', 81, 1);
    T('60453425254578966374137534557667603944.0046371958525231890311667632495928', '13492130530897347902898580602617164.09772752579476211958', '60466917385109863722040433138270221108.10236472164728531', 55, 0);
    T('5002301516.194571086912098778341947', '29.2688268928220713388', '5002301545.463397979734170117141947', 95, 0);
    T('9112914294624600207924745534417283609.40838', '16186.39608728977486146838804', '9112914294624600207924745534417000000', 31, 1);
    T('5506979783090360720319552.820903989162000984251066804576344', '76514640839433.8748150222429984969387', '5506979783166000000000000', 13, 3);
    T('636590853500424407040175.440282749775668270121658178', '97825835153621.58330522', '636590853598250242193797.023587969775668270121658178', 76, 6);
    T('49975914.4053512221605', '204005786549.1', '204055762463.5053512221605', 71, 5);
    T('996741089887412267105688679594705284565.4824', '3891821186657548923884038671774490.825715558842425321084726894509846645', '996744981708598924654612563633377059056.31', 41, 6);
    T('808033774342851937921.659161031', '53914781908780.4376013033', '808033828257633846702.0967623343', 59, 3);
    T('60570942839603116876218144.05475527691329611597810814163', '53131016886802.49', '60570942839656247893104946.54475527691329611597810814163', 93, 5);
    T('245633342503221472827051.909423583843286622985471555238945821', '94405739681932890724010192088952104.7264426920748413294772825529662972646990', '94405739700000000000000000000000000', 9, 6);
    T('61668276112877443177437802271062013.82257679126671205697', '7728746.3425690', '61668276112877443177437802278790760.16514579126671205697', 88, 2);
    T('7488444774730.4036', '9518312841997020033219949808234264.38785951038622394103', '9518312841997020033227438253008994.7914595104', 44, 0);
    T('314459071684754884682485649490093184.7960018849040994212941278019', '523985994245671216040230.0145560', '314500000000000000000000000000000000', 4, 2);
    T('7.26670251341677251878479610210096', '85017491581348.3359074680425009669830515946', '85017491581355.6026099814592734857678476967', 42, 6);
    T('378391769679.41567283648623949240170321793707718206', '341463442494679566248984295969.40293154582759335706', '341463442494679566627376065648.81860438231383285', 47, 4);
    T('6.28421827', '2524087605070811514254072985785268981414.7100472998768334683475445765', '2524087605070811514254072985785268981420.99426556987683347', 57, 6);
    T('7305.637293118562267842', '989796836083307374906738841548921968.166548365022444467195', '989796836083307374906738841548929273.803', 39, 1);
    T('47500595418070483375630.511252039545633934682891779440', '61.37786223613162410', '47500600000000000000000', 6, 2);
    T('897317376206.5479324', '59.127477748301', '897317376265.675410148301', 54, 0);
    T('127959501744805843345181161133078273.2351439009552420545632036902332', '48606457713824710049291213866565669.579214317839128402127165550844468462359', '176565959458630553394472374999643942.814358218794370456690369241077668462359', 79, 1);
    T('61272095550767842452313528372.77664515666593004', '691554299365842195.38393280786', '61272095551459396751679370568.16057796452593004', 57, 1);
    T('720838788530576234372808028.3464767329339046889650673735239512429', '1814610934140662.8262879329886903531190084119839718873716', '720838788532390845306948691.1727646659225950420840757855079231302716', 77, 5);
    T('316465.4640990492635147158', '6.960', '316472.4240990492635147158', 28, 3);
    T('7581778.514179796924945229731398302', '16068.8101372756435398', '7597847.324317072568485029731398302', 63, 4);
    T('1768166218545813920282505633200049409926.54317652790045981', '56264.092099553282528757', '1768166218545813920282505633200049466190.635276081182988567', 98, 4);
    T('18757.0860628', '69247830570108395765201631063628494.76698449991312754', '69247830570108000000000000000000000', 14, 4);
    T('90560602078729337209841.87537295191999418067181090338490', '41766981860832253558331532678837123570.92690774323576908205621036', '41766981860830000000000000000000000000', 13, 3);
    T('837868.659945051672864790', '384866583834407667335899022.78263611914079045496959722040', '384866583834407667336736891.4425811708136552449695972204', 74, 3);
    T('59067352957296728.837291706630', '481762539914178253804354962618.319781793', '481762539914237321157312259347.15707349963', 75, 3);
    T('69680210701617757468107075803369.3380640430', '155296515145533647701627464544751.242426359418107993695911', '224976720000000000000000000000000', 8, 3);
    T('441149177609793714149938728355.0', '881447889750486688150099.854765629520349141056279252837199', '441150059057683464636626878454.85476562952034914105627925', 56, 4);
    T('335769557215.1823474550754643546048130942133453', '9233953891312950540331782904642276.97732', '9233953891312950540332118674199492.1596674550754643546048130942133453', 91, 2);
    T('550026117462606038575737798669.76579658974985093760929231934', '4294962429498975853.83713799466308165007', '550026117466901001005236774523.602934584412', 42, 1);
    T('496460139287939.78047765246005544451697', '876949388522521.5696849914838467144142988325674', '1373409527810461.3501626439439021589312688325674', 61, 3);
    T('475918.101497533064', '501779552866558527825243.82426130760810943', '501779552866558528301161.92575884067210943', 85, 4);
    T('4051.489', '9172542076507.7144804225960656269', '9172542100000', 8, 2);
    T('73308.56395663610913546796556578739806457377', '164286119829017842549625554141992475.7865455025589194987942577948618510350', '164286119829017842549625554142065784.35050213866805496675982358225991560877', 78, 3);
    T('14493107917408.0', '37315030821436983.93935656428982816563912471378791330', '37329523930000000', 11, 0);
    T('29428379993.95070928922097997984661985064788955944', '470623070639.712286528107296298144848007614952588', '500051450633.66299581732827627799146786', 38, 5);
    T('3915468845469788794585393.679470484578478211197875932460384', '299584061394133246671499.110712954571444881671664665', '4215052906864000000000000', 13, 5);
    T('266505459879607594005.903', '181.829', '266505459879607594187.732', 42, 3);
    T('1824034024859.898500', '36928690014916.826', '38752724039776.7245', 64, 0);
    T('8922.934512687294492085073278', '5991023686.5432478110639878955095206822', '5991032609.4777604983584799805827986822', 83, 5);
    T('153.160191', '829478841268025961898272878376.0087263256319807293900831081780597', '829478841268025961898272878529.1689173256319807293900831081780597', 65, 6);
    T('1.65834281997466174008077368198784708', '9928868272389958387834861463.65106550684852336191849180101410681', '9928868272389958387834861465.30940832683', 39, 2);
    T('75896407210193861090.1879337077642937078912298696466', '4682830799999857852329835221457.400187026741513', '4682830800075754259540029082547.5881207345058067078912298696466', 87, 0);
    T('3735736972877386826242.416985931', '4330689973199772653489189985007781413.8343211262585685450449017', '4330689973199776389226162862400000000', 29, 5);
    T('8097666.505181138922891378109', '60613339948051841991474141888968143.8560136722193225131452', '60613339948051841991475000000000000', 23, 0);
    T('81098036003763650451809958113825775071.409', '9479617432544290050302273444843.1742', '81098045483381082996100008416099219914.6', 39, 0);
    T('8356.0258875023', '5670827079706001055673558076194428703345.72636064431156977665', '5670828000000000000000000000000000000000', 7, 0);
    T('9709602948597166697.75', '8408307.38976982809265345301', '9709602948605575005.13976982809265345301', 41, 6);
    T('2258385480054939697357.2208569848786586353', '2030739978452432577649270031819042841624.9823', '2030739978452432579907655511873982538982.2', 41, 6);
    T('742951447841228063520864.016584191911956261424906696703077324', '36.809902378304812654566707079243331293012', '742951447841228063520900.8264866', 31, 6);
    T('22124976.052253824835', '5.820', '22124981.872253824835', 88, 0);
    T('8183997298764247407554439265717229.5317825665838176947475281', '65146301707.36915944138862550', '8183000000000000000000000000000000', 4, 3);
    T('1631733880.1901397143389638655157571977637', '7699070089405674797933751165.840', '7699070089405674799565485046.0301397143', 38, 5);
    T('16342.8501533311663988902816114136643', '20490669265101044147813223281703157.901359719880612141784', '20490669300000000000000000000000000', 9, 4);
    T('4837783014.118932137109047246082332779', '39364761405674554925813785.956199', '39364761405674559763596800.075131137109047246082332779', 87, 0);
    T('32385771346717422710777715327075122072.419994669422865533468694160', '595100752.3518218460071780784847', '32385771346717422710777715327670222824.77181651543004361195339416', 81, 1);
    T('3456361147142715352645058349114449740.7502515735362254099344500', '95046821412266773293.890225281339730326720737303265308305', '3456361147000000000000000000000000000', 10, 1);
    T('84867.2962450294543388299', '65103323407587088145038719210026499678.450231227338716019456315', '65103323407587088145038719210000000000', 30, 3);
    T('460016522523815071345.1', '74396.825274386976687938035486', '460016522523815145741.925274386976687938035486', 56, 1);
    T('708390770.69', '848515472906499750994352473679619.589539505978232897657371', '848515472906499800000000000000000', 16, 0);
    T('237246357787402485043.5', '504492780978513795596193.0', '504730027336301198081236.5', 84, 6);
    T('8688977385205.7296441312022800', '8311933274.5648449266108873460259', '8697289318480.2944890578131673460259', 61, 3);
    T('7157432811.4226144537', '332090509033117481585656455680551727.21', '332090509033120000000000000000000000', 14, 4);
    T('8285414629715555630465826.73447989', '61890629410752756973.0378167121722449715353432535121624824803', '8285476520344966383222799.7722966021722449715353432535121624824803', 79, 4);
    T('9424546031461542799883010493385860710.47135616474715343409665553129988262987', '9869584113781397912413904389167856422.3470249567551', '19294130145242940712296914882553717132.81838112150225343409665553129988262987', 100, 2);
    T('999150525999768807175894295339698408530.0268578425435186905006424', '6500927348300663.250354547798761', '999150525999768807175900796267046709193.2772123903422796905006424', 71, 6);
    T('1992298710.72325731045', '6158365220545473.215605859059184321938', '6158367212844183.938863169509184321938', 65, 3);
    T('70970760831.45005195604969063033345839477', '8052664497975798707219573035795621626065.3268576308604785944661025679', '8052664497975798707219573035866592386890', 39, 3);
    T('475893079383861474699234.94402146606600572440344881', '4581667029285664629347134484603081.8364758157205647642084004890606939595456', '4581667029761557708730995959302316.7804972817865704886118492990606939595456', 89, 6);
    T('1959243189.1646850217056', '738.656578973169', '1959243927.8212639948746', 46, 5);
    T('245745476325366037505743175790264838690.366552032610239798', '413609516925121636.21063358688670577699', '245745476325366037506156785307189960326.5771', 43, 1);
    T('294548084634785442343852508.6254323089793606866514901295485625382210', '9313060.9655426070252551', '294548084634785442353165569.590974916004615786651490129548562538221', 94, 6);
    T('534076152048399020279209.73002057451370442351', '627200209659217054607826308.4211813450', '628000000000000000000000000', 3, 5);
    T('677958199285499233730488817651950.8772236', '1987116955340871519865209578027315125.44958566574052822558172265784', '1987794913540157019098940066844967076.3268', 42, 3);
    T('270288422.534243544929584210527882439232321743035', '692245377788.7154773', '692515666211.249720844929584210527882439232321743035', 91, 5);
    T('6165.9439561486860885886829164074784829', '2251701838987201716567.7994169477562394', '2251701838987201722733.7433730964423279886829164074784829', 74, 1);
    T('5534367435815601023786983764.406445046098564', '5510.3136', '5534367435815601023786989274.720045046098564', 51, 1);
    T('59080453238.9410431609486374854210963', '4770314222870334487985051512241016.0', '4770314222870334487985110000000000', 25, 3);
    T('85.901040246', '973560389481144894347132407671.736542399974810767397', '973560389481144894347132407757.637582645974810767397', 81, 3);
    T('794106569423551579288.193371008602099442355377378', '6247565032766498828117886494528921.259853911149779687737357893098977865429', '6247565032767292934687310046108209.453224919751879130092735271098977866', 70, 2);
    T('19621068621224538.10582771078559214874346505', '2126033838518554704411458.1246988326856785048000650217083862408', '2126033858139623325635996.2305265434712706535435300717083862408', 62, 3);
    T('2969349536972811462155744123.062257874804056487744428986192017763487', '74086932213365798307877240285671299.50064851592521', '74086935182710000000000000000000000', 13, 3);
    T('141176148326158176458744244363.99073', '6476520.493308074506', '141176148326158176458750720884.484038074506', 98, 0);
    T('12233301395815.665285910589627237109391902646', '136185359234943732349637427.4147', '136185359234955965651033243.079985910589627237109391902646', 61, 6);
    T('719993555655853.133456145308328406017851147070762541', '1489444856152332.697165930441299866490309820259081200694', '2209438411808185.830622075749628272508160967329843741694', 56, 5);
    T('18174058176.69112016609616163372334', '350262325123525570345.08386467829105762537669821118144', '350262325141699629000', 18, 4);
    T('330753073674470921.23075680710344851949147763145', '870100027162663586.8', '1200853100837134508.0308', 23, 2);
    T('316091340530983564710131463600663244288.097174372005753398289082238650499767676', '579020787093370912559153748917.8478941', '316091341110004351803502376159816993205.945068472005753398289082238650499767676', 97, 0);
    T('151090273889097825730065158983525.303491263363504869428885607427516858953', '31627948615008027234238134.82296', '151090305517046440738092393221660.12645126336350486942889', 56, 2);
    T('818408751782266683742262844677516203.541847170322860318860045382', '43159934712837478785871391088382.39077947716166122984023', '818451911716979521221048716068604600', 34, 6);
    T('7.275035669', '1584070900692489685929.960615711', '1584070900692489685937.23565138', 35, 0);
    T('897466902916701049588421.9728252302944570952222281570951', '787858946130.6', '897466902917488908534552.572826', 30, 0);
    T('8.0281308974', '58329632547737759594961093756.316329911646635305504', '58329632547737759594961093764.344460809046635305504', 60, 0);
    T('6768641456094978576394154074.2321535', '90561917507407378088326890224.765829486', '97330558963500000000000000000', 13, 6);
    T('9.56248643928', '268.1826', '277.74508643928', 16, 4);
    T('588191646545257.24047668', '475572361.31', '588192000000000', 6, 1);
    T('678420907353065062.476769409738011969065792', '6001586903318536718465102788450.7701138563640', '6001586903319215139372455853513.24688326610201', 45, 4);
    T('63097808675287312.905730943493434341264048416843', '3581058251581982285440126410858.4325384763778271767', '3581058251582045383248801698171.338269419871261517964048416843', 80, 4);
    T('87621280106167592837079.9', '4712036978807.7900374424529392158225', '87621280110879629815887.6900374424529392158225', 83, 3);
    T('124.9308508', '683.8925251949358', '808.8233759949358', 33, 5);
    T('26949621815.6207302109371713573260228588489474209', '19615509026616.820896599694207306390439', '19642458648432.4416268106313786637164618589', 42, 0);
    T('7305105.153698045175132', '23447634266103981.37683974026635530', '23447634273409086.53053779', 25, 6);
    T('349.93932188893074261', '3789797118274919934327651800183903.98666228806810935258855874', '3789797118274919934327651800184253.92598417699885196258855874', 67, 0);
    T('7556.5473602537331954962890067669942104', '75860308076046944.734040201884260612531', '75860308076054501.2814004556174561088200067669942104', 69, 4);
    T('8.3', '439101632675471456319930881053155568484.8574545104124375362989592251262', '439101632675471456319930881053155568493.1574545104124375362989592251262', 100, 1);
    T('67.43910520250676077328308', '76.0030525796165362018772723092456886335782', '143.4421577821232969751603523092456886335782', 79, 6);
    T('2609411.40707', '540037308641265630854332879.2925113', '540037308641265630856942290.6995813', 35, 1);
    T('2037530685294982417521956560263374732257.66403794493575393905171828939', '658139347508999829281284324540.697049982', '2000000000000000000000000000000000000000', 1, 6);
    T('9106352858783481526431491181.6448946071066843', '965905665343181602417810461175.132093', '975012018201965083944241952356.77698760711', 41, 5);
    T('317098323256806965.7982', '684207.604971', '317098323257491173.403171', 29, 5);
    T('75869.474778096200847368351', '92007212651535372752947.214452721219018314', '92007200000000000000000', 6, 6);
    T('118287456739669873937957947578980.60436680512531492142518593720792417889', '366902486749069355982.3429440909', '118287456740036776424707016934962.94731089602531492142518593720792417889', 77, 4);
    T('792214198.801347814682925756292520954', '3349268702435412538.193047005901859503613914122170084847622', '3349268703227626736.994394820584785259906435076170084847622', 62, 4);
    T('5265214174635335900834.947007642172777753558312344092175', '386097645937482575842901637128661112.72289390944812024970320', '386100000000000000000000000000000000', 4, 6);
    T('21102045164534562025425582697105979.2849038248380341', '46606894033912520868419567247805884.04631866673187686191878848138', '67708939198447082893845149944911863.3312', 39, 5);
    T('903265757031.403078419738764529', '5240848897436.13477810188775203', '6144114654467.537856521626516559', 42, 5);
    T('3456394712.307234415251005027218208395922', '571411910234089.020', '571415366628801.327234415251005027218208395922', 77, 4);
    T('77860759635030401681904962532.1647316038234922654026', '5965056088520330802.67647816727860545003426582', '77860759640995457770425293334.84120977110209771543686582', 70, 4);
    T('108986115550733359734389.68772', '75969565359843752870234.92470369394536154924764537', '184955680910577112604624.6125', 28, 2);
    T('834806717682088164542507437305.8259', '3892734395782242.388461083945469', '834806717682092057276903219548.214361083945469', 61, 0);
    T('688922953107.859162', '19069834.00347329152487924418', '688942022941.86263529152487924418', 61, 2);
    T('11824934370351060042074553.0725633334124627131514468', '27799861416593736304412344133.58168601838246', '27811686360000000000000000000', 10, 0);
    T('920606999754.18668513359795926', '48314956545.1767945562181053837725', '968921956299.3634796898160646437725', 35, 1);
    T('803499619401251942539237615850588077.273950488374521534', '39990634702511652279304293700495.04115870164', '803539610035954454191516920144288572.315109190014521534', 79, 6);
    T('7710751453357085255460795.7705469721206090800772', '2509.197689123419061102098', '7710751000000000000000000', 7, 3);
    T('4198684762091177466024682507029.9106874013894905024335', '318513266689672806885462.03426411638673043838909753521', '4198685080700000000000000000000', 11, 0);
    T('415269.7374598453860', '909460078029655383361617793.044607', '909460078029655383362033062.782066845386', 63, 4);
    T('23967813730.9782509566488619', '560645698996447818.904673698850517476', '560645722964261549.8829246554994', 31, 5);
    T('2062214142.482554752', '45332050908270340693814732812.374986563', '45332050908270340695876946954.857541315', 98, 6);
    T('6312.1316', '232436847928689.131220411638560650', '232436847935000', 12, 6);
    T('5413.961743369201', '238613770737780662305.0', '238613770737780667718.9617433', 28, 1);
    T('54932158674393804446681560.913502262638022276092121', '426315173172886449716789507.5370473131125', '481247331847280254170000000', 20, 0);
    T('568257889097245966524983842685732679.217695919158', '2420398024.24802875115563786172341726817', '568257889097245966524983845106130703.46572467031364', 50, 2);
    T('6594598928021308302859562154.078201596297531351063431774861219446', '346241950.2586396466653653406392719764902', '6594598928021308303205804104.336841242962896691702703751351419446', 97, 3);
    T('462639298415991954077894363574.69082677292931747', '3701592494.90022544561951571752257576266804530449', '462639298415991954081595956069.59105221854883318752257576266804530449', 81, 2);
    T('599623.780', '428752.5969075418', '1028376.4', 8, 5);
    T('82914920537471804.440567560877072381239594866', '8703690633634839815559720133915429617.70826612979858', '8703690633634839815642635054452901422.148833690675652381239594866', 75, 1);
    T('336456706163428112518363848925797622942.61646187681317629', '77582.96605874099424572941506361253', '336456706163428112518363840000000000000', 26, 3);
    T('398121313648238583440902338303.49249858477729014013537773360905', '96925254285261624.962969', '398121313648335508695187599928.4554675847772901401353777', 55, 5);
    T('91.2', '1310942375.03379016866279363041015741', '1310942466.23', 12, 5);
    T('73603623232186621057.2718302729535177', '982175196335059.16503387271936367628185089936346510930', '73604605407382956116.4368641456728813762818508993634651093', 90, 2);
    T('3.8887853848516013483102241279148', '32355134434.15838999', '32355134438.047175', 17, 1);
    T('22431990953412267427057.70334681525720940782', '1031113895682878791278559096537638449.4811781851761313248618', '1031113895682901223269512508805065507.1845250004333407326818', 81, 4);
    T('30494577879.1060330738079298489974199806065', '98348866219655.2458328387357009391423285', '100000000000000', 1, 6);
    T('1.96646174821968191285727705476', '70743643174862765003467602252.25989272503407798578174306208575238', '70743643174862765003467602254.22635447325375989863902011684575238', 97, 6);
    T('48885814765003.1409445828035826', '6.520696', '48885814765009.6616405828035826', 53, 6);
    T('85264285146515940200.2615883503287900550496828684', '12318.73760508431', '85264300000000000000', 6, 0);
    T('2186215511629329372820328838558972291892.67917825955539246290', '745117131851381110939723614735224.514456', '2186216256746461224201439778282587027117.1936342595553924629', 66, 5);
    T('75043315633280967226992.38267221913', '93893786370563938403701700.424377802364186757275618645', '93968829686197219370900000', 21, 6);
    T('894291088.584792269197243944922763096023458633494', '6340741294851066253479777509567200.15277995462388032862680320', '6340741295000000000000000000000000', 10, 4);
    T('89.578044550963443740949764', '935251457.33077103123176963627460057564446052894', '935251546.90881558219521337722436457564446052894', 95, 4);
    T('393405423537646.6923806648825332547494721883391254', '2276514676939548318400599357186.5', '2276514676939548711806022894833.1923806648825332547', 50, 5);
    T('861654054165340.36', '47780821615077563934746714363408806.27506073957039370108705251356233392', '47780821615077563935608368417574146.6350607395703937010870525135624', 66, 2);
    T('71064066555811273273047239134667771.083808162142944303010947153', '7963073672791726897346896995.8556579060992077565950044951937672794009', '71064074518884946064774136481564766.9394661', 42, 5);
    T('31664716897515847886197.727435261797', '51431952110169.158364288953225324946536459573', '31664716948947799996366.88579955075022532494653646', 49, 6);
    T('10132290773.034192536408388569517201104210', '527308454572.868990578867909133', '537440745345.90318311527629770251720110421', 73, 1);
    T('642975216746982161217.493513336720658950616172151983599127502', '1213450940241190365450473795415.28', '1213450940884165582197455956632.773513336720658950616172151983599127502', 89, 6);
    T('5576387204996788103857545633.579665', '6925005374078162533767798076280087.893002001381633547103004', '6925010950465367530555901933825721.472667001381633547103004', 71, 3);
    T('45003.7044609270184', '9429183.98578075467741144', '9474187.69024168169581144', 87, 4);
    T('537497382447175707220296.584796382182402421307283205347639084', '498695220031202860615250521877223214516.2543313507852363002', '498695220031203398112632969052930434812.839127732967638721507283205347639084', 95, 6);
    T('593910693502344585548063469365351.8249098796513854488522672856', '363297657548129110719214401294258640049.8', '363298251458822613064000000000000000000', 21, 2);
    T('95146157483280760002522007.697668801', '2406216774859814670909.0463760287730956982', '95148563700055619800000000', 18, 5);
    T('650059999979230593746533289110611497312.643869642419302195219', '48762432555971537239805724913327.377598715', '650060048700000000000000000000000000000', 10, 6);
    T('65593082863941544566575059898088112.2540099670173106653011', '91216701670710726774049620201156651434.3923725063444869701914762511223820489', '91000000000000000000000000000000000000', 2, 5);
    T('68212985621143409385909505099.7423232225657660347590069253098146', '973270163283197424313356689584438673.8385022659031394910642111245092939', '973270231496200000000000000000000000', 13, 0);
    T('64318293863982.4519', '816293305086089546.74011', '816357623379953529.19201', 78, 6);
    T('3774.20804967315979980473163655', '259479.0802429366731585121103581795215002', '263253.28', 8, 3);
    T('1473448.0223321695533482928734963980579296787', '79453659286819659241.9356264853166896709768172475', '79453659286821132689.9579586548700379638503136455579296787', 92, 3);
    T('2395.6492115248463764995994190399945821', '56766084.1940818347644129116660176858', '56768479.8432933596107894112654367257945821', 46, 4);
    T('1569441.69337152559960281', '17009.7437223454342544973174505780', '1586451.438', 10, 0);
    T('49123282703775756.53830', '52061829550735706940.3731514315178139217646966', '52110952833439482696.9114514315178139217646966', 75, 5);
    T('54403410100699788657061842.43565557236613516516385698116577352', '803310859632200581712927893.48122704', '857714269732900370369989735.9168826123661351651638569811657735', 61, 6);
    T('25749.493957459', '1446344168662423726301680787751283.950', '1446344168662423726301680787777033.443957459', 77, 3);
    T('1728527342260083409445805.525752420881669', '37948682.1063405237231945', '1728527342260083447394487.6320929446048635', 60, 4);
    T('6245068847966721393.03069816753932253', '4516.63031893269859118', '6245068848000000000', 10, 0);
    T('91075157780319154587245.3147607412497603837374536090890329901733', '12022150058.9842037641583231', '91075157780331176737304.2989645054080834837374536090890329901733', 69, 6);
    T('9914409926555025232759223208274142673.9042576', '9189.083', '9914409926555025232759223208274151862.9872576', 85, 2);
    T('5257241053041816180542246303776981.79035283790', '5234690903290767712783.290379490561231202601174027978041935226', '5257241053047050871445537071489765.0807323284612312026011740279780419352', 71, 4);
    T('11143314999945116116456437891559.279633670429', '90960642515109634091650.32885', '11143315090905758631566071983209.608483670429', 51, 6);
    T('412880874.4', '707806853882653887413466778405243154.08537', '707806853882653887413466778818124028.48537', 72, 3);
    T('5233120453458.78571501465246794259568459693724742827', '25957811859230198892075381959583327.449511', '25957811859230198892080615080036786.23522601465246794259568459693724742827', 100, 3);
    T('78899837205946258489720916.8891227265', '66162.556834854859862819697262088490276105093', '78899837205946258489787079.44595759', 34, 0);
    T('921656250028.71722505257952769058672839059273393', '90983133130078970571484410857193299527.020454086390234', '90983133130078970571484411778849549555.7376791', 45, 3);
    T('2685811610855349706210509462281801842565.386981956345969574939530224006131587758', '2648358331685.5449114225092550786353244923014', '2685811610855349706210509464930160174250.9318933788552246535748547163075315', 74, 3);
    T('89135262441168957119.5149928416550438149037476368365803233543', '3699656816686881232983183484620017068522.21560787660028441113454158', '3699656816686881233072318747061186025641.7306007182553282260382892168365803233543', 96, 1);
    T('42340330890060479.3117037426996221357', '3.2547992069', '42340330890060482.5665029495996221357', 42, 4);
    T('939575224278487487467518.0746308361290611011673209', '18363779912765756129.6567', '939593588058400253223647.7313308361290611011673209', 62, 4);
    T('4250182421883504743129174646162731582.935', '69564352297.32145840896321894149201197742651', '4250182421883504743129174715727083880.25645840896321894149201197742651', 89, 6);
    T('348.369014313809602534645929358600346', '7744175972937027132910.0875800222747882884088241642528609026126', '7744200000000000000000', 5, 4);
    T('8.3380916539115', '517284014707771896830027562.478770468528791150271', '517284014707771896830027570.816862122440291150271', 73, 2);
    T('2553.754893520', '8745421543073227817396.726070', '8745421543073227819950.48096352', 81, 6);
    T('52199856.530277550404369705928280841687614180', '97619589977285327374703435553835215.11470333537', '97619589977286000000000000000000000', 14, 2);
    T('2477497124395100713.42407927295081303327404692313781764894', '9.61233759', '2477497124395100700', 17, 1);
    T('950002069808334670422986.671013', '35437512553585922836822181763707.571453854829776694542586821', '35437513503587992645156852186694.24246685482977669455', 52, 0);
    T('56069004546214838264158.56930849884862079', '2826470633766407680115877941834686.472466610095579145168403', '2826470633822476684662000000000000', 22, 1);
    T('9706826139652719.24345625753787499119089382868047137', '32301703997540594817432314.897709062739176733711', '32301704007247420957085034.14116532027705172490189382868047137', 98, 0);
    T('72598398843938530860319032.722413253386', '3813060.65175779128204810184981', '72598398843938530864132093', 26, 5);
    T('6634482145272633.4042633375183', '8837704682613216.0045447921575710938', '15472186827885849.4088081296758710938', 47, 0);
    T('342504036347115.313399542473356019978938488214802', '8073602142814.8309093926892516696261033517398726294300', '350000000000000', 2, 6);
    T('2.418222007907863910534794620638633', '22320790932529039807821.5237144559837947044386', '22320790932529039807823.941936463891658614973394620638633', 68, 1);
    T('51246748410.391075189745612690413591381652868528', '453874470564763122274949092130.578617923253837242906105257087066', '453874470564763122326195840540.969693112999449933319696638739934528', 86, 3);
    T('283534269065793533614567215778.1424223958', '1.579300756104', '283534270000000000000000000000', 8, 6);
    T('93066326978660718737.1816826674372566655854716615', '378438347907076061671325250694.493', '378438348000142388649985969431.6746826674372566655854716615', 61, 3);
    T('50083880232016029362657390053.651695060342349858519344', '8720185480121978146247302724.32511324291918537027619075', '58804065712138007508904692777.97680830326153522879553475', 58, 2);
    T('2258753717968103446366.548741616435', '64170559601729.25622176558517129', '2258753782138670000000', 15, 0);
    T('732002436740.6586349376925736795270576374', '2518857300681560168225541642888326995163.14855486680369942057901', '2518857300681560168225541644000000000000', 28, 6);
    T('291523.6123658', '704144154661501079587633156.2', '704144154661501079587924679.8123658', 81, 2);
    T('20374826709514.8247204835175494749371054', '865.6992562366695171521547602609119234', '20374826710380.523976720187066627091865661', 41, 6);
    T('999480771211722098073049701045654166069.3856297672294800761209001', '987929168093835311719104260289476831.6722508', '1000468700379815933384770000000000000000', 24, 2);
    T('98.6483499064529227', '7119759883545235164359824879032213263966.71194937777215207082143380171492398', '7119759883545235164359824879032213264065.360299284225', 52, 4);
    T('898001000404463301480697254192636782374.9641920452785519554864898067344316977', '33370532992813384856192.345066667450403277872651427763791', '898001000404463334851230247006021638567.3092587127289552333591412344982226977', 79, 3);
    T('48601438199370958.114839993341540772370357', '84562308635942727400424017068588575570.4447220120', '84562308635942727400472620000000000000', 25, 0);
    T('707069536778444.34872386245917937700', '39136070311466266805292629.81257', '39136070312173336342071074.16129386', 34, 5);
    T('709765267066430594187414.862779798552411376311469593704930', '602650577061026218518349980141.480', '602651286826293284948944167556.34277979855241137631146959370493', 96, 6);
    T('23729472507413147148327664072093663.4129222842370516', '6.3455896605556440756479297241339843963545', '23729472507413147148327664072093669', 35, 3);
    T('851012387897269873850499665.28220162952802', '94746856640586694953296582745.035389900124626212', '95597869028483964827147082410.317591529652646212', 87, 3);
    T('420318228814091082264.51995640331603056543', '386077772248439574361995.101730008259', '386498090000000000000000', 8, 5);
    T('38736928648199203199.8667290195032891', '15826420823298082529291874801168.4392421627108233044508113982053831', '15826420823336819457940074004368.3059711822141124044508113982053831', 73, 5);
    T('8267643861352575653569000.64587124303492503082921428130', '54817769483976758755810430.236151105338135633693438053689607928647', '63085413345329334409379430.8821', 30, 2);
    T('3.01', '463527680999195327867877753957335.38503', '463527680999195327867877753957338.39503', 92, 3);
    T('36648217496266111523116719.3668667', '403953190502875605022957610953940965.002', '404000000000000000000000000000000000', 3, 5);
    T('249033511033035.206743430652664994037599', '979112376.44', '249034490145411.646743430652664994037599', 58, 1);
    T('491689161561185391591765931.7592026208193650', '2825047354824125594297077198.9', '3316736516385310985888843130.659202620819365', 82, 0);
    T('80032378242199988.292419919203613493520515628', '145953810146409328599.864258433113376453633944771', '146033842524651528588.156678352316989947154460399', 77, 5);
    T('8946932588247926247878500547823080499951.0447151', '7193254308482389332769077.5586812035537019254', '8946932588247933441132900000000000000000', 23, 0);
    T('6374834259078134657310132675.272', '5574827.9049', '6374834259078134657315707503.1769', 91, 5);
    T('65253.04515655601630417', '57292114181575923959.196870388571258717977126798', '57292114181575989212.242026944587562887977126798', 100, 6);
    T('465689048602443207.4759563498245451015654578135053023687475', '15489690133146670412310446457897.690', '15489690133147136101359048901105.1659563498245451015654578135053023687475', 92, 3);
    T('457252233824115951907972906567515105.73436975792063473992631', '627673643508.0', '457252233824116000000000000000000000', 15, 2);
    T('523229609732801600479678.8744492937157888', '90546688242250214904224394540.0545948562131680', '90547211471859947705824874218.9290441499289568', 96, 2);
    T('743304676634620107.36', '6891735696462945634284015814887.70442231865372513', '6891735696463688938960650434995.06442231865372513', 92, 2);
    T('66.1459741448184884510', '60574710392041949780476319477588244.0234193625', '60574710392041949780476319477588310.169393507318488451', 91, 3);
    T('903796364836170510762883.2140', '50217003102765908102774078.039415980248837854784556638193', '51120799467602078613536961.25341598025', 37, 0);
    T('91011948708920926241.41461352347612642', '588707475.518024333520', '91011948709509633716.93263785699612642', 62, 5);
    T('9822909604138325127939060079518.3171921259218512', '52919019158.6239497', '9822909604138325127991979098676.9411418259218512', 81, 6);
    T('432747126963022043456875402380135501397.1393684980697669714817747266', '6296991096.59474033817', '432747126963022043456875402386432492493.7341088362397669714817747266', 96, 5);
    T('65.8195', '57375600006854946295350.3945138646713825778', '57375600006854946295416.2140138646713825778', 94, 0);
    T('32358555021007952.516614696501', '5513968713883330915383139939439.6279', '5513968713883363273938160947392.144514696501', 71, 1);
    T('663609112527142236417.32', '35775511511298372665805473608.598039167130', '35775512174907485192947710025.91803916713', 91, 5);
    T('973561958.62778505845618215555298066142', '591143725109936937442551364914248.88349004814624373', '591143725109936937442552338476207.51127510660242588555298066142', 75, 3);
    T('736580.75798805', '796806336041139353076228249.902329704595313592638231815436745425', '796806336041139353076964830.66031775459531359263823181543674542', 62, 5);
    T('8139389042984.220953310616080748870236663516359', '882644711717401543074092139134.983', '882644711717401551213481182119.203953310616080748870236663516359', 72, 0);
    T('714823957.287', '560.08574863145015256167041926507960', '714824517.3727486314501525616704192650796', 96, 0);
    T('899632494622518658735.0182594234289145441', '387664841998661664286446341246787491930.33976979517403603020439379287', '387664841998661665186080000000000000000', 23, 2);
    T('25.02726', '2962539232024451883504579263.739568077186', '2962539232024451883504579288.766828077186', 54, 1);
    T('938867215113007871837397857.0842299847651317576383267283371774', '7947266730940057296967032059842987804.2392787153201733869170063508095498572', '7947266731878924512080039931680385661.32350870009', 48, 0);
    T('698767900436540266279012969423.7915301718786500871510542331680', '67078656085324564393842625917227414.0434457346920', '67079354853225000934108904930196837.8349759066', 45, 6);
    T('61012545503489891264315654868544657821.5', '7165986852569.0', '61012545503489891264315662034531510000', 34, 6);
    T('1555699443355785641410998635.867037316170091', '18322861940967711.902220017201701496495400593', '1555699443374108503351966347.769257333371792496495400593', 92, 1);
    T('11805843998731140945.845596135388199656893260144928198447795', '9536171097206.5226118933809122313002993617739', '11805853534902238152.368208028769111888193559506702098447795', 68, 0);
    T('35.2927996895419', '8.0256482149862076814505286971890753', '43.318447904528107681450528697189075', 35, 1);
    T('72453235617.4669718961515369670185982560255660', '98904566208770212038130681.8948761356441682691', '98904566208770284491366300', 25, 6);
    T('53443129.12781125894714804257089648656466738', '5813045930.3747971816284879607', '5866489059.50260844057563600327089648656466738', 55, 3);
    T('1241869742.116330683259411565148192786417830393', '827641003904563969348360200.496929448360', '827641003904563970590229942.613260131619411565148192786417830393', 86, 1);
    T('111418378465784610935144706635133713921.78181341233318646464', '95584883955615748.8057040541069', '111418378465784610935240291519089329670.58751746644008646464', 62, 0);
    T('1628421911972543750013940622268.649176796305', '59398964.20539659293885818331294', '1628421911972500000000000000000', 14, 3);
    T('9344.1604435958194987745', '190504257038115278708853107780120438632.6345802', '190504257038115278708853100000000000000', 25, 5);
    T('80142900004232575596899009928429.50003821', '524217.1599561513762818357057020332408468398', '80142900004232575596899010452646.65999436137628', 46, 6);
    T('9385016722478803320923121707749503777.539562', '54191972172303845861.7', '9385016722478803375115093880053349639.239562', 98, 4);
    T('4889363103971027582456604.05310564235348565861863705612613218', '69238981952566.593440646384', '4889363104040266564409170.64654628873748565861863705612613218', 65, 5);
    T('2069954644598253701850609.17140123860847966', '38482875811642.45115450907556', '2069954644636736577662300', 23, 2);
    T('431937846388698361247153.53587653552455662163709992087077690', '81863934454089269515932604963268853.808180835', '81863934454521207362321303324516007.3440573705245566216370999208707769', 70, 4);
    T('424033520487387.536203', '43868.544874597292615827070434', '424033520531256.081077597292615827070434', 91, 5);
    T('3834331915970509.657787791371190964763044260', '4945036445370224.7', '8779368361340734.35778779137119096476304426', 79, 6);
    T('7286431319182560548133.6526786199279289', '5917553231534806385195521005259.16332423215965016104', '5917553238821237704378081553392.81600285208757906104', 60, 2);
    T('3481368681263784413212051156176.8027328812752623990', '9964297021416054546505315985929644.8', '9967778390097318330918528037085820', 33, 1);
    T('9073984192842528318.5368693429', '959624174730613802012667145835.1772168536537', '959624174739687786205509674153.7140861965537', 55, 3);
    T('607.442117920837722157780580744447515680943', '65167095908706093789487165766156.51788388354213070135240352824', '65167095908706093789487165766763.9600018043', 42, 3);
    T('9635506279278152778675038571601.069033887141', '884732986.3480053169096384709293618813173247612696', '9635506279278152778675923304587.4170392040506384709293618813173247612696', 83, 2);
    T('87263727739639571697739626944290050490.6355481245', '294572242099470989983456230109538193247.00041674271484', '381835969839110561681195857053828243737.63596486721484', 68, 1);
    T('5132895251801433.499118', '45870482129651.445831', '5178765733931084.944949', 80, 4);
    T('93.166056767391691313604543452747386', '151560188440525274735933078672772803.511111494138327344239246727275184979478', '151560188440525274735933078672772896.67716826153001865784', 56, 3);
    T('804922781287532071664337805224920323.96609511', '816060133095520962128.33', '804922781287532887724470900745882452.29609511', 79, 2);
    T('8804612136452846669892752.8355685416157575820895146629148', '401164.25483117075', '8804612136452846670293917.0904', 30, 5);
    T('50118402578278742968877.35560526454137178268002177781182404031', '573238802801655389697.80150434254764', '50691641381080398358575.15710960708901178268002177781182404031', 83, 1);
    T('61316856414296855467954099.6478315296071327622317585', '8413196701559956145100888430773203306906.4627054521', '8413196701560017461957302727628671261006.1105369', 47, 3);
    T('105210699052355193203555582.0', '623.360546764490377602365318480648', '105210699052355193203556205.360546764490377602365318480648', 78, 0);
    T('5847993909397036005334505041752464.177585835290281342070585077848183106466', '84273254377879338759863852506671606.308836179248523849483221281', '90121248288000000000000000000000000', 11, 2);
    T('1049381825309084846152818797.643697199932834799438563706', '976089.85341967475073892851877881685', '1049381825309084846153794887.49711687468357372795734252285', 92, 2);
    T('843005344426.96432890898265053515375083765132522', '1837144971735628948573.762', '1837144972578634293000.726328908', 31, 3);
    T('880063540644858841008088.6690973790222174327', '93951334915458910128932008592296723343.196133478258906122486891526404346914', '93951334915459790192472653451137731431.865230857281123555186891526404346914', 93, 5);
    T('396400115096561.0715491482571746555522', '11.2252330535410752', '396400115096572.296782201799', 27, 2);
    T('792048432665842101.567', '4954810172513535318358872.00859128722748420667445', '4954810964561967984000000', 19, 1);
    T('19906.0', '736001659986863883531631.38419440463302784995792692957900', '736001659986863883551537.384194404633027849957926929579', 71, 6);
    T('426478226.2875641082084457449555014650214258884757', '83960452478465895145517989.16896354865557', '83960452478465895571996215.4565276568640157449555014650214258884757', 93, 1);
    T('8144.2389862566231043624683562366', '30299521694755640263956856117314305.369423712212430201243387364623379679', '30299521694755640263956856117322449.60841', 42, 0);
    T('449569950.604923151405898008714241424696025302', '642114.8799811949893898566', '450212065.4849043463952878653142414246960253', 43, 4);
    T('55077603881736907776958603738500.7257685498689271717452', '31980761213248213.286527053393650032641', '55077603881736939758000000000000', 20, 6);
    T('7475.749314457274', '55330485068420943150870739538426.971955774801437098311679', '55330485068420943150870739545902.721270232075437098311679', 98, 3);
    T('5465141116422463278815682503.048230905761071792338966819594801', '832462029475826726554719444.58359621311252564208', '6297603000000000000000000000', 7, 3);
    T('834405527275897606412455.361377344963837908595701769384421428052', '89719893730524760810275189701012154.827029533875', '89719893731359166337551087307424610.18841', 40, 5);
    T('2931095320906795473371263649.8107620990751621', '146568318538143.288258205275179513696259589780712949', '2931095320906942041689801793.0990203044', 38, 4);
    T('68272791227963691.5183986772775619363448622403912681', '4789713457416216696756542273569528406279.592443145901550672', '4789713457416216696756610546360756369971.110841823179112608344', 61, 1);
    T('538733519210992144679778899.4941662094784235353449846147978027144', '58282713807.49252', '538733520000000000000000000', 9, 2);
    T('5498473448709053590883001307.68996333511498876671', '521454200212251394666314642971.29329871761664482871269099399796030', '520000000000000000000000000000', 2, 3);
    T('64220571255511465783119516218.119575743185281442679877698149718453', '25.0', '64220571255511465783119516243.1195757431852814426799', 51, 6);
    T('9989008547825.50495172138687667038944855977', '8813133235226825227806536474134998294.7933889192516954762953768', '8813133236000000000000000000000000000', 10, 0);
    T('52754143.76797954762994893', '73710071302714589787955.0696126198060720878221363050278946643', '73710071302714642542098.8375921674360210178221363050278946643', 97, 6);
    T('4833063814249399860018982650165091650.2380095759049549280998048472038226', '4296267662851107125067503.14094785568569737761', '4833063814253696127681833757290159153.3789574315906523057098048472038226', 92, 0);
    T('35741024857416444249469830930179.2', '19059213591749024837107042.8364455075773391893887', '35741043916630035998494668037222', 32, 1);
    T('6568792903592971855814680482616367.8589101401658111794025722295445483957', '856011746389919008933972571453131824.01083010398066908313049383639805', '863000000000000000000000000000000000', 3, 5);
    T('7414684627719639779.24034098077929370445619529', '902400457595380962.460301479677767440398018168537711', '8317085085315020741.7006', 23, 3);
    T('5268884.124998518007281388630', '4997158832818663.1102921669462634947826153020148306390', '4997158838087547.2352906849535448834126153', 42, 5);
    T('1267261908.56049941196287845334660568', '1806390355991968594693058086324917553.743027280342', '1806390355991968594693058087592179462.30352669230487845334660568', 90, 3);
    T('699.04696691171626148067509562304', '380359789349493060801486422.78153062165279', '380359789349493000000000000', 15, 4);
    T('275595.937', '347213475267476598802123947098953397471.57438037941560734399150058403925496', '347213475267476598802123947098953673067.51138037941560734399150058404', 69, 2);
    T('3327192858.615', '64.67289337040', '3327192923.2878933704', 22, 0);
    T('87601235574465531459027326.3098992275092', '635423228015881.41211817966431300436563556886584', '87601235575100950000000000', 16, 1);
    T('537558896331588819729923733622492.11996192029854537514306', '4386231556712722947042646213541120.951982254210500751484', '4923790453044311766772569947000000', 28, 5);
    T('301036164842055436.6457536', '52574.48486682557566675362833247841', '301036164842108011.13062042557566675362833247841', 52, 5);
    T('950716620906719.55378866967526182343051174554', '77250214928395564977015086.57110379013709863902517982261805144150', '77250214929346281597921806.1248924598123604624', 45, 1);
    T('1099519.094420172327', '202664410015443.906962866831952222097489693279216971', '202664411114963.001383039158952222097489693279216971', 55, 0);
    T('36132080319776096951459038.892818', '44.89161070624678476649175602868', '36132080319776096951459083.78442870624678476649175602868', 82, 6);
    T('75089528318601235394916908318957.36209813491148038707097893800598015109', '369347718573537774663553651094519.1639761780', '444437246892139010058470559413476.526074312911480387070979', 57, 6);
    T('8.31775287473117513333550', '1110787955146644936553029507793998963932.6844514868737333042943326237310706971', '1110787000000000000000000000000000000000', 7, 3);
    T('6396168093517272311519965047.5819522664640466419626759747721', '866482103628646315247.0546672722', '6396168959999375940166280294.6366195386640466419626759747721', 66, 5);
    T('192350850473514490.3349890231270102903847452060632239767', '373102199023058341344495.893545811493440547657843874487', '373102391373908814858986.22853483462045083804259', 47, 0);
    T('29367.541490290405851683818026723232', '20600205882443307481988.1881470599299297263', '20600205882443307511355.729637350335781410118027', 47, 5);
    T('59939768748607999549981143.062022083941459813482908126834', '644628644192572636138.52887423641568080817476', '59940413377252192122617281.59089632035714062165766812683', 55, 6);
    T('6078008718329019956303994368.21173894489278411534500157437165604', '470892856995633750223906751311085444.0400212895211681585462016743367004156738', '500000000000000000000000000000000000', 1, 2);
    T('24719796032124258468575271.15377035891', '872289855685630651812.3106593', '24720668321979944099227083.46442965891', 88, 0);
    T('3227627716691979618460123167244110069664.922770120710684390054546079', '670815104254285322380906118046183270779.352582462709045392891', '3898442820946264940841029285290293340444.275352583419729782945546079', 92, 2);
    T('61549728351789248132830330.619107753245982033787182519', '74359731728366655.54732143166056919972255216693664729', '61549728426148979861196986.16642918491', 37, 6);
    T('82073341849349296012864.288853009', '87340407819.0', '82073341849436636420683.288853009', 73, 1);
    T('8838.75875578441669225802975', '53034325332785534025674067.80931267899745417147431828930641771', '50000000000000000000000000', 1, 5);
    T('55453.8223162275', '291747368045833.02076', '291747368101286.8430762275', 42, 6);
    T('49407910150867998395.5685853405380060', '818.2816', '49407910150867999213.850185340538006', 51, 2);
    T('667667737.5584228714924242368795704586', '936329936400245064.072141', '936329937067912801.6305638714924242368795704586', 49, 3);
    T('693860438864774987975959655.139', '64545202887970.0155782298825462789', '693860438864839533178847625.1545782298825462789', 71, 5);
    T('292094636792083080017506802.610341284847944853971381283423344101', '114009539949126866.187152523742923989', '292094636906092619966633668.797493808590868842971381283423344101', 78, 3);
    T('36335698134151728998670251815648.3', '3494496788867569716618.381474085061788267144328490307580828', '36335698137646225787537821532266.681474085061788267144328490307580828', 70, 1);
    T('53243404217842580022460258864.690293', '83014194494894760973608058672576046.94175980738616375143130820816', '83014247738298978816188081132834911.63205280738616375', 52, 6);
    T('49.2451948688422647194178994', '86670114568.51130782937164117813937555627144921025', '86670114617.75650269821390589755727495627144921025', 76, 5);
    T('8402065832377503689900959252686817530508.233244823241491717222475103060138', '8926959.530174169777783372928422380776631755378', '8402065832377503689900959252686826457467.76341899301927509015089748383', 69, 1);
    T('96407684588673285513246485019584116217.699892005639350297564536552791906', '8264118.91016975', '96407684588673285513246485019592380336.610061755639350298', 56, 0);
    T('19.232030038437038951215063465938998634483', '2877240270900684108110582836037.3673', '2877240270900684108110582836056.59', 33, 3);
    T('25133016.760355818432901074415550630764881', '13661.685575538116106901916439747532695815', '25146678.445931356549007976331990378297576815', 98, 0);
    T('5771253396851023905218519607515.973376471', '245068705207107753803.10096', '5771253397096092610425627361319.074336471', 70, 3);
    T('7113362622494347567788988442320795.14504306230620788707050166285', '58304816612908427446945720409649907582.331566998231933864820672970787339638391', '58311929975530921794513509398092228377.47661006053815', 52, 2);
    T('677441229480404.414845908880', '6848268.1055213160287548563', '677441236328672.5203672249087548563', 86, 4);
    T('4724204.0439098393701593523050062133914917627281', '9.02647797652506862841573026343216198553', '4724213.07039', 12, 4);
    T('206982371123211.58759291542115800103588', '36349580.820952350806080949', '206982407472792.40854526622723895003588', 92, 0);
    T('690485.23674833500467453', '87737000596907837314585541439805.188884087', '87737000596907837314585542130290.42563242200467453', 84, 5);
    T('8176648405317778657718820899.99', '94.5067060105171752660185061901716', '8176648405317778657718820994.497', 31, 2);
    T('722995503082120077353815617176350805915.5', '24560394770355340390120780.1086589135044582134', '722995503082144637748585972516740926695.6086589135', 49, 4);
    T('932111229466017068940.44193435024464998581338751', '41335772568516771536675.003', '42267883797982788605615.44493435024464998581338751', 58, 0);
    T('96654134552410.74893046532', '231709001451410793855523425037.1269026656450347451038432210419893', '231709001451410890509657977447.8758331309650347451038432211', 58, 2);
    T('34599138642184999421479.069214472707', '7300815317688196419189.982860313370295150266545', '41899953959873195840669.052074786077295150266545', 72, 3);
    T('1139131875026594118070315293785217.068914742143344780407642258206', '88576805825069.116813729394698338450', '1139131875026594118200000000000000', 20, 6);
    T('195700369937654371058759454.2259099987493', '3552036.19960415404771129574415', '195700369937654371062311490.4255141528', 38, 5);
    T('5689954252439564111872.449334', '2875434880641192026917568605.152010714758821912', '2875440570595444466481680477.601344714758822', 43, 6);
    T('54912456895715372951023279.176834552423731164395761', '801308.06149068268710867763552', '54912456895715400000000000', 15, 6);
    T('87577.034717161164882103938', '9925111385474.81772910067076024367518586666380552', '9925111473051.9', 14, 0);
    T('6416167.099742340079558844854667150518', '20.94081897642261053160800862222150925966', '6416188.04056131650216937646267577273950925966', 67, 0);
    T('34738917593710385339835734945484.52505026386343', '769475051156303377307671116650579076826.4948582421928614820000410707', '769475085895220971018056456486314022311.02', 41, 5);
    T('242273636638.45603937047545418', '55.375878779374352229041346096106337', '242273636693.831918149849806409041346096106337', 85, 4);
    T('76596095885139665234387.668786195200640525597', '32010739213392483616335528360.484085845275', '32010815809488368756000762748.152872040475640525597', 90, 5);
    T('1724274561.702454411195237049508340333', '2012081843.320612923857560862605562523068983639', '3736356405.023067335053', 22, 0);
    T('4083026772457321790829567797315.7403679789913859196658026323', '52977769957052081645428922015.2031173614192636703697664347155315553192', '4136004542414373872474996719330.94348534041064959003556906702', 60, 5);
    T('41162.399053212985493689', '34649302741.329564175256640679472387809556', '34649343903.728617388242134368472387809556', 75, 3);
    T('33870499560041654836858156940.47585045137479217556800397311791450279', '255446920692.478314639115021243934152636957874893', '33870499560041655092305077632.95416509048981341950215661007578939579', 79, 1);
    T('435935993484807.0101628785166190389828', '1614407.701040043359746429913532494682171', '435935995099214.711202921876365468896332494682171', 75, 6);
    T('5520.887037106957792075280011232', '5475407760.335523761236311720129068500618468622038', '5475413281.2225', 14, 3);
    T('1317809810401436.334086324158613261364175579', '2163.8108334724', '1310000000000000', 3, 1);
    T('3156826.413707256991289382819031632892320067130', '9994857376130745403051101358137701496383.84643', '9994857376130745403051101358137704653210.2', 41, 1);
    T('69254.0822027043384110241216390508', '3.7963673920', '69257.8785700963384110241216390508', 48, 0);
    T('55454579.975156116116664997329596146398', '1814564152791933236011.6176983685', '1814564152791988000000', 16, 1);
    T('576655800985349506277.774023081932503908181', '7146806489474822467842298449808538381491.34909859989744528081809161691134030', '7146806489474822468418954250793887887770', 39, 2);
    T('216162081653961467960635663990609171847.3255070942645631411546694945129009254758', '4960436110074267541035898566280.765196676925637767', '216000000000000000000000000000000000000', 3, 3);
    T('85806064070250675.0', '3370625260723157874671.47130321', '3370711066787228125346.47130321', 98, 6);
    T('312607054.17676495448202522981459742169875488552', '548765057189631010211691597663.6141246272085883083028192880', '548765057189631010212004204717.79088958169061353811741670969875488552', 95, 1);
    T('634986200956154199110182489395960071.5776389', '68922242263282180047550478361.895709752512184', '634986270000000000000000000000000000', 9, 5);
    T('21905605615808585974981.48950704105226978', '733674795087370935623769.660447626036661557430900203168520174842', '755580400703179521598751.14995466708893133743090020316852017484', 62, 4);
    T('336352142938227550580604.336124342834008918489697532704726220163', '922467946080560788074.5789841099821963108076021', '337274610884308111368678.915108452816205229297299632704726220163', 96, 6);
    T('2744143013304805974502774265671159182.598907788672614646098540191747804954107', '3168287998628104291860813666649131.096271249550850220848715738132', '2747311301303434078794635079337808313.695179038223464866947255929879804954107', 82, 1);
    T('69.4', '51353958068740006195163156553.9358333445236', '51353958068740006195163156623.3358333445236', 98, 2);
    T('1545372194979696161112.863581266111691227859285106121737945', '822652009171205996292006807056.666703537785328604088713', '822652010716578191271702968169.530284803897019831947998106121738', 63, 4);
    T('439.14622746560418064293355665273621234571', '224.0547993897', '663.20102', 8, 1);
    T('598757167727918017465861914.08741803668355241390340990', '6131769187776675508085.73427253765222142335942020459590577', '590000000000000000000000000', 2, 1);
    T('20382963633598546984695864414157161867.0481639503994807005702934691756004', '7603713200582642141159139729441833204.8382313', '27986676834181189125850000000000000000', 22, 1);
    T('924520265627692815136.31847207575075014874110053699505823', '912317535543459765843.0', '1836837801171152580979.31847207575075014874110053699505823', 86, 1);
    T('74900077954916747.92', '7857765450333627423.593250', '7932665528288544171.51325', 38, 1);
    T('5684224817.59963422730439382301', '74358128.276570262584498570962260567462173767281', '5758582945.876204489888892393972260567462173767281', 52, 5);
    T('87567386206951.889700224', '4165313713572615140928815.2560578328', '4165313713660182527135767.1457580568', 84, 3);
    T('6160110991942935025783315611511756.72519765332415289982014418411861', '8961.581202162713748458313525254247443890', '6160110991942935025783315611520718.3064', 39, 6);
    T('496.9823616545810', '784013933687.0985627137638628266819637463', '784013934184.0809', 16, 3);
    T('6369431007366474559609246413129392290445.59077105229056695244396220995108', '32908177529328378633299675328.74476459885', '6369431007399382737138574791762691965774.33553565114056695244396220995108', 92, 2);
    T('153378185866274.51234755074289083805', '562124035574.0669351186232588360509122550494479506', '153940309901848.5792826693661496741009122550494479506', 68, 3);
    T('366238268181160377496.7996', '9938045.15993245693335', '366238268181170315541.9', 22, 1);
    T('30416887144681.71945711930065970937740433815', '2474006.0', '30416889618687.71945711930065970937740433815', 83, 6);
    T('95747828195255063308.586760426557421034', '64771400994231582886971091.88795335', '64771496742059778142034400.4747', 30, 3);
    T('39772597881871127.4781453885873512057720257257972', '44.5', '39772597881871171.9781453885873512057720257257972', 80, 6);
    T('923379489.558236', '6417380648277602069790.0', '6417380648278525449279.558236', 71, 6);
    T('26114450773652231.136290291303578698850655786018041877632', '266206295272.202758', '26114716979947503.339048291303578698850655786018041877632', 73, 0);
    T('88228.4346909763696914177908', '78089798771017360.41518305645171', '78089798771105588.8498740328214014177908', 54, 2);
    T('281922102258525769697551546953071591.43', '976250370723811.93867273318625577950552730147031303080', '281922102258525769698527797323795403.3686727331862557795055273014703130308', 98, 4);
    T('5671350.1957444611503109415093045834', '75129850089.7893686726', '75135521439.9851131337503109415093045834', 66, 0);
    T('87812732011679667429899063023851.248179748707559263466846058739780007912', '1590354737919456758175942304415667.590573693337419737563', '1678167469931136425605841367439518.83875344204497900102984605873979', 66, 2);
    T('22092127035.833721896566518070985319410418', '9367948409899919.6136518433800190824699604947480498', '9367970502026955.44737373994654', 30, 2);
    T('1426562320530344144747241500408934.3441532455175', '3302815606.6478755209', '1426562320530344144747244803224540.9920287664175', 52, 0);
    T('831022157463106017010372052525715689.3508769', '97674592061758576.1752363926529215665612372044', '831022157463106017108046644587474265.5261132926529215665612372044', 98, 0);
    T('296636796809401.7096061', '9579.8203390199642280524399038721686', '296636796818981.5299451199642280524399038721686', 92, 3);
    T('9461608542.406441627234161374591147', '6273940439209292381334810264532627965.43650287086756929857583', '6273940439209292381334810273994236507.842944498101730673166977', 65, 6);
    T('214010936662.443934158509927733685376992', '4792518835428917334697282066795251501.4550198280470481667', '4792518835428917334697282280806188163.898953986556975900385376992', 76, 0);
    T('391515799793350171697975565018490.4', '307.34929983493211242', '391515799793350171697975565020000', 29, 2);
    T('6558733599379.0078998213742063663165247463965793', '54366096239726988027899380529844602897.1334666', '54366096239726988027899387088578202276.14136642137420636631653', 61, 0);
    T('690230192338173740472776923764436864138.93', '859377740641312192670011727956057792917.6053', '1549607932979485933142788651720494657056.5353', 90, 2);
    T('320452705039.5195205405039670545187471486013224', '46148442504460733.5270710497067498039990', '46140000000000000', 4, 3);
    T('79459836249976160084840181957.4', '187022347304836446464.9053467445514739', '79459836436998507389676628422.3053467445514739', 48, 3);
    T('51572200836871863622.75177827666959292932099740783420091552', '35285799.4024783110672217540258275358184', '51572200836907140000', 16, 3);
    T('9687850425371139921132139144.50213310916572584042', '1480482162532569546927977330.0726594164852297310982037061', '11168332587903709468060116474.5747925256509555715182037061', 79, 5);
    T('7903420428669.50373587197484634995', '68845.5859594791', '7903420497515.08969535107484634995', 46, 1);
    T('5411189745865119870798616229421435880.42474074158352248786878134204817', '135998315264236034963071795345148547345.59966285424497584834983285791258969958', '141409505010101154833870411574569983226.024403595828498336218614199961', 69, 5);
    T('255977493.96693622615175843780515182319', '1364414468410725384427667.8203272549819', '1364414468410725640405161.78726348113365843780515182319', 77, 4);
    T('21357464542984.74805382681484', '3462909724.611027', '21360927452709.35908082681484', 41, 1);
    T('77.20', '8699342406741037952506401891401.38653182011719941171195752441558085', '8699342410000000000000000000000', 9, 4);
    T('8643856773215095902.734407686203323597275562521210', '998225914646417037887461.870850053859033', '998234558503190252983364.60525774006235659727556252121', 63, 1);
    T('445671272835453821.21', '154217752120.3041991967210016', '445671427000000000', 10, 3);
    T('9175410692267497322109395387900024040418.592786128', '6.09868590467', '9175410692267497322109395387900024040424.69147203267', 71, 2);
    T('6393127137190175872269762072191377210.17401418332720467987223053972321', '60.3340168380835900634532', '6393127137190175872269762072191377270.6', 38, 2);
    T('872114870026716183985191341052935922.8517761011343269733509883499166044772128', '8527209415619928731675297721952.673467146269605099984946057', '872123397236131803913923016350657875.526', 39, 0);
    T('15729998.71665538', '6682308836256543713348.7947371693685123344144260', '6682308836256559443347.511392549368512334414426', 75, 5);
    T('74897466609161068705983572849591.6450132187908', '57806875589830.9743625838397144500091', '74897466609161068763790448439422.6193758026305144500091', 60, 3);
    T('95900647193836183812107.815361601282655959500301460211524166770', '52720898397024323733379617247725.422829', '52720898492924970927215801059833.24', 34, 6);
    T('850960478851278035290.55149642600', '7220523523050020709532838.628015596673273690621053', '7221374483528871987568129.179512022673273690621053', 62, 5);
    T('248969942507097921493.503025195', '46501636138518179880.348097074169', '295471578645616101373.851122269169', 91, 5);
    T('81580727901567274.0', '81218021009439680694584887962865011.8446085', '81218021009439680776165615864432285.84', 37, 4);
    T('69718469333086317189553249203.955', '49390956369477662187.674004321747665', '69718469382477273559030911391.629004321747665', 87, 6);
    T('18531756.1753722319001880259941457', '648759192601114911362966849926806275262.901828', '648759192601114911362966849926824807019.077', 42, 6);
    T('23687959.82362538310544004429274668', '451208267847.37703499303', '451232000000', 7, 6);
    T('505958450951667889312234259124638.958211357082', '63995976134018352962.55694462193265425310017492998', '505958450951731885288368277477601.51515597901', 44, 3);
    T('960486999.4341809632334', '619119086952.212597927843451993974503', '620079573951.646778891076851993974503', 95, 0);
    T('914910108386881998.481', '74846.966341719209743131700', '914911000000000000', 6, 0);
    T('6001889308.3417023', '5149433828143.0616051383254612913951752', '5155435000000', 7, 3);
    T('1207417898261157734184257189582038.75', '8304752081030710635961.14577316595638067843925', '1207417898269462486265287900217999.89577316595638067843925', 93, 3);
    T('480013792483843595628291.542375997336298536204696427712441262618', '99542098100089809736.800292756515211062771750', '480113334581943685438028.3426687538515095989764464', 49, 3);
    T('9951.9745793254934186401501465734504730056', '57685.33425549076164401268691635615198361906', '67640', 4, 0);
    T('90777.64', '801139732528767136568850379242655845.14833998659093222939946574510023062', '801130000000000000000000000000000000', 5, 3);
    T('3686949254735091324.5451891098317623567698968962994348168', '68864997049780738628820765272.09341235624503878833244686', '68900000000000000000000000000', 3, 6);
    T('4907079443373589556475922.72943179775404389678750545021106895280', '85815242677013789151605712806.4626166375590882504961975', '85820149756457162741162188729.192048435313132147283702951', 56, 0);
    T('6.37250582132921396', '25157123286321623251534741155143138.16209158501671273', '25157123286321623251534741155143144', 35, 1);
    T('31805411498460467457.780763805797956080183454656114', '755155240204686634651990802253.95622150997', '755155240236492046150451269711.73698532', 38, 5);
    T('61421577315017719821543.136', '14086131305110026380001433755849630610.831219343793022', '14086131305110087802000000000000000000', 20, 5);
    T('79.292082444673411601635415720957832418', '72374110522534835500064.1770515229', '72374110522534835500143.469133967573411601635415720957832418', 71, 0);
    T('59.44833413551605126481', '69366357.2186189966137755415411353329129983', '69366416.6669531321298268063511353329129983', 73, 4);
    T('8802731562892600810614503861873.8314818951', '39183509469.629181504244796355336087685228501146630', '8802731562892600810653687371343.46066339934479635533608768522850114663', 72, 2);
    T('927377152770313338.008930072229767', '74955301.0897128', '927377152845268639.098642872229767', 35, 3);
    T('25.21', '2579730998678837370.8673926783181', '2579730998678837396.0773926783181', 74, 6);
    T('892012925932340187560300.87405577938838868339900281487763702', '17.09', '892012925932340187560317.96405577938838868339900281487763', 56, 1);
    T('100.7974945598081573469336570157', '86304438146216054587275270315.3762028', '86304438146216054587275270416.1736973598081573469336570157', 73, 0);
    T('66190078523438.3180227358255184201931160174281014420525', '96025130355848808848670788257.0133990735091773259525862578889717', '96025130355848875038749311695.331421809334695746145702275317073142', 65, 5);
    T('236337628635762300468872762994827.9372237600742931378871056373902190707968', '250216.2', '236337628635762300468872763245044.2', 34, 2);
    T('51347345163804985219081274369321266658.142929768272966237351231289809223206575', '95523912202330444.9195010657854157213403', '51347345163804985219176798281523597103.062430834058381958691531289809223206575', 100, 0);
    T('146962641884945079494940749178.6562219305153334209106', '859800183586143447332.42544615700323576193164135390735930', '146962642744745263081084196511.0816680875185691828422413539073593', 94, 6);
    T('60544827629565442191770324230306171314.8662460834593696174054910018219933847', '9319104139333241425863162.863898220302014741', '60544827629574761296000000000000000000', 20, 6);
    T('92933061441050444383452.840216386543586845', '25.8254118486454132', '92933061441050444383478.665628235189000045', 67, 2);
    T('0.70321353038836101264168442134144', '2577597673846182636840033885.84873282218566261778095780766', '2577597673846182600000000000', 17, 3);
    T('849102202824.1523', '57254929494.87096583307959530292', '906357132319.02326583307959530292', 34, 0);
    T('304506508919002088.99771987313800470327286961295', '994177396.798117834359962550', '304500000000000000', 5, 1);
    T('29129034655052063285695206.67758777703060856859074442823546', '269181411327323541529875706.717053486651677892701583637200482', '298310445982375604815570913.3946413', 34, 6);
    T('4885917912968145707424218532541136.644394526852854724117783', '4589618.91', '4885917912968150000000000000000000', 15, 6);
    T('338111100289567212245259591692774.25388607749525279522980', '81.2', '338111100289567212245259591692855.4538860774952', 46, 3);
    T('12738207305829.11', '810623051433453164426465835.3677812993', '810623051433465902633771664.4777812993', 93, 1);
    T('2615105044826395707289844405156517025.2963796383705319160015502841499', '9914926088989465588037460453.0352509', '2615105054741321796279309993193977478.3316305383705319160015502841499', 94, 1);
    T('319083015361312268615.0', '420264345151.6217368', '319083015781576613766.6217368', 28, 3);
    T('199601787386280929442.7342283789260331415996518053526935647', '977172500772854237535221.59348302333791981288212989903', '977372102560240518464664.3277114022639529544817817043826935647', 84, 5);
    T('8370169011580743559.7826920862648965319485207384555375441654', '1419790.42240636339645840117604395768011384457', '8370169011582163350.2050984496613549331245646961356513887354', 61, 1);
    T('56884143819673125.040947899242991621755', '813275309945.027169016862403504766', '56884957094983070.068', 20, 4);
    T('805950.212113800', '155685582365144141492170702.43367913343767558700510208949444384', '155685582365144141492976652.64579293343767558700510208949444384', 76, 5);
    T('570021813690419005205208792132089005.92964501210341364178756636026', '74.14179567859314395194063232176680', '570000000000000000000000000000000000', 2, 4);
    T('7020598222521.37595305816826', '791.7675198952616139', '7020598223313.1434729534298739', 91, 3);
    T('952282324.11336932', '52658288376575542314.289015792', '52658288377527824638.402385112', 42, 5);
    T('723037426800121.857694982630914105798', '645761961618287.39224041204377', '1368799388418409.249935394674684105798', 87, 4);
    T('611844653839150844331.3980', '13096727689323914011239060148408279652.063473306281', '13096727689323914623083713987559123983.461473306281', 51, 6);
    T('5475909.3436215625423754756159818104', '8692001010969152960887071248200821143.163595', '8692001010969152960887071248206297052.5072', 41, 1);
    T('52766914865282.138678892106003707153', '547.184028282176668784234376', '52766914865829.322707174282672491387376', 89, 4);
    T('87771.0982876486738721438462505805377430156939', '41260500004889256390.3781861', '41260500004889344161.4764737486738721438462505805377430156939', 70, 5);
    T('35485675503578626075147812018062.7812603485908609346814', '739748244282255.1094785087920922', '35485675503578626814896056300317.8907388573829531346814', 74, 3);
    T('57922503636.0', '96190025234502.437350843509780080', '96247947738138.43735084350978008', 50, 1);
    T('453526632172545728375020938184405.572537', '753477894358399685.548052879987672778', '453526632172546481852915296584091.120589879987672778', 74, 2);
    T('8844589609002176.5381997449333258', '8613756334872618833747026381309.2043798972317257', '8613756334872627678336635383485.7425796421650515', 65, 0);
    T('48126124969492673049395984485579875204.04811665269047217712', '413309838797835036460766.798756471986816386549598619292', '48126124969493086359234782320616335970.846873124677288563669598619292', 85, 2);
    T('8863994023466.101112422246552752323265153188', '720229256.6105122644630507909', '8864714252722.711624686709603543223265153188', 49, 6);
    T('69699471.0', '4808746086260128841826157899085611.6417058083098810220022006287062869007', '4808746086260128842000000000000000', 19, 2);
    T('8805275184938344288685566574710718717.0', '89970724153751884192956781.725933701867434929536854', '8805275185028315012839318458903675498.725933701867434929536854', 71, 0);
    T('6395678370400976128902.19265640480512295235653', '3688336.3833083776', '6395678370400979817238.57596478240512295235653', 73, 1);
    T('479627175863101349725575521380900850213.28249793941459576622383090237307', '27777013459918121765947.33216188', '479627175863101377502588981299022616160.61465981941459576622383090237307', 86, 4);
    T('2021829758648639702946953930969138034.7844295860236904587686249216', '9.61129649332', '2021829758648639702946953930969138044.3957260793436904587686249216', 73, 0);
    T('904879844940085137.0', '453026260.7851522200332', '904879845393111397.7851522200332', 66, 1);
    T('4668.308449489328328443079113293964', '985421.24555918197826839', '990089.554008671306596833079113293964', 56, 2);
    T('7177746433903.705212409453', '68532485593.297442376801609', '7246278919497.002654786254609', 64, 4);
    T('6149680.6507890291739233526176258263958840464', '597040223477113775858689989.96674357184799043144993151004574', '597040223477113776000000000', 18, 6);
    T('13279.6787811973', '37558827528655127984487164647191.9150', '37558827528655128000000000000000', 17, 0);
    T('380934147463193811208795.163721758522726570986616015980229200404', '5477835483342556860631267.315897185665960', '5858769630805750671840062.4796189441886865709866160159802292005', 62, 2);
    T('788246422171173296635.079275430924628233120947', '50134357889496798103620142061421740.2890104918', '50134357889497586350042300000000000', 24, 3);
    T('46955.66921735443352', '34804960949212.5408010', '34804960996160', 13, 3);
    T('822482.91343918008197285750811436', '894336448352851515929550158996.039941', '894336448352851515929550981478.95338018008197285750811436', 73, 5);
    T('6965.411098911665226', '12.553704357240213381006192', '6977.964803268905439381006192', 29, 4);
    T('504058921147553356983676899942.64624810341', '3401984.691132066612584678', '504058921147553360000000000000', 17, 2);
    T('498224896670211943716731183671173096896.593', '77.8958081397125307505145110427', '498224896670211943716731183671173096974.48880813971253075051452', 62, 0);
    T('3062051051034628.83328969601895394108215559', '543379532880.5205838357971134974058878308759727429', '3062594430567509.353873531816067438488043', 40, 4);
    T('831518269.8', '8299785752237284.763450159506730645344295434353888040557', '8299786583755554.56345015950673064534429543435', 45, 1);
    T('9218840.36350398239915723806984', '2.629392474667181186676792093432519729', '9218842.992896457066338424746632093432519729', 92, 3);
    T('5.31485388385872412146553972', '44067642832756350924821570.306961940920519', '44067642832756350924821575.7', 27, 0);
    T('5449808445451485794125641.0434457700821536788894367848039044', '20520042562692.589305863472103397', '5449808445472005836688333.63275163355425707589', 45, 2);
    T('71493078045572943117258440.68977571077424', '943.0', '71493078045572943117259383.68977571077424', 57, 0);
    T('22071486563706407062691354397424.68829093560610350', '4520.102437', '22071486563706407062691354401944.790727935606', 44, 1);
    T('7564273077247301017332555902989656235.73', '707.66936319812382144043730225017259', '7560000000000000000000000000000000000', 3, 5);
    T('565601018089056299.9077710200983925394850894408361835853', '51999701973899602617956857722.4291895102', '51999701974465203636045914022.3369605302983925394850894408361835853', 96, 5);
    T('49216461698381977455345867.3352413214839647551', '1526085.602070848806448729142', '49216461698381977456871950', 25, 4);
    T('1177248567240858.120443649398491562461', '39232221362197215758335.971595447949801668529248616565850752971', '39232222539445782999194.092039097348293230990248616565850752971', 88, 4);
    T('2265360097047824080600741744247143874077.4145325271465', '7706.5592742442393879371327528222111124', '2265360097047824080600741744247143881783.97380677138588793', 57, 3);
    T('488463754110678659215.105155720031153994', '61302523438114176383026.724915353530835560714', '61790987192000000000000', 11, 4);
    T('725027736982000678339208675448584648.3075047607030394777985755971637', '93246718064417.3991139868299385583650033390', '725027736982000678339300000000000000', 22, 5);
    T('979601784893129.26297', '197407683.667885066354499389552238258', '979601982300810', 14, 6);
    T('519224608034836886697311646392096.8405', '100.230', '519224608034836886697311646392197.0705', 73, 6);
    T('115961812896848621910433685732929.272535576678459906860715298', '9352668967396094024314381.25333463252774623632178778920', '115961822249517589306527710047310.5258702092062061431825030872', 66, 5);
    T('6299958510302739276360220142233877137444.5126280785867358973696314306063929341522', '26327179.794542051609184647724213229546584525', '6299958510302739276360220142233903464625', 40, 2);
    T('135604147794977377524762.27105064242352422433085360377269', '191902416704692768664411.6', '327506564499670146189173.87105064242352422433085360377269', 66, 5);
    T('468860963.8730331170965603060522408812215549626362', '243712453717192527033468.06151839126062111895756333518937360', '243712453717192995894431.934551508357181425009804216411', 54, 4);
    T('5435369999404769.5905268686267391614380244248549418106786', '1404573600670802473765951049.5514230095599691891434827206659712543', '1404573600676237843765355819.1419498781867083505815071455209', 59, 5);
    T('536710110756556965155924705067.523583', '9.328035671517536238528', '536710110756556965155924705076.851618671517536238528', 87, 5);
    T('1500815.535008518185450459590628802572', '483057544983071475259734247733567253.28802386', '483057544983071475259734247735068068.823032378185450459590629', 60, 2);
    T('511871157090231459602616460283214684.8426805171148571100786165', '5573333808921304818376257.0011131601573010139752', '511871157095804793411537765101590941.8437936772721581240538165', 65, 2);
    T('3765089813862786892956.60974748527420177623397655', '441199414454410865841815480.2459362582400329287217', '441203179544224700000000000', 16, 6);
    T('4979572566593138355706992091878145332.89756', '656028942090.0411091515094530384421909', '4979572566593200000000000000000000000', 14, 0);
    T('69111081671823145849.770957154536701737988652390326101', '505813336235659128290585.65313249155716144815986768754', '505882447317330951436435.42408964609386', 38, 6);
    T('8769049937192826357848576996305980646.6498640953245134008039314148205950442', '402827637287585629299372.48541978988270678446304415067', '8769049937193229185485864581935280019.1352838852072201852669755654905950442', 88, 4);
    T('4315092.0811346', '5489231831176868333726470379681.6984394687976240254815962768284529', '5489231831176868333726474694773.7795740687976240254815962768284529', 75, 3);
    T('82214349153215.3920960476084527044443970603981356789751', '39667089654444913926733932044.96672762239249186929137275', '39667089654444996141083085260.3588236700009445737357698103981356789751', 79, 3);
    T('8797.719444799176895308675171', '89814475441037093213838677124233.2627340694899728221', '89814475441037093213838677133000', 30, 1);
    T('545516213958538798685322521233.20112', '640666328970005699539.49411285883815605038654721026632', '545516214599205127655328220772.69523285883815605038654721026632', 94, 6);
    T('5267427791503522987698.1426379309178728', '46366586737579546593161238.49512984222971301419950', '46371854165371050116148936.6377677731475859', 42, 2);
    T('3496377707329292013.378358157533363', '48154.31012', '3496377707329340167.688478157533363', 82, 0);
    T('194590287278304941454.386573146718257773329069601', '9260005666041641224657684362.596943', '9260005860631928502962625816.983516146718257773329069601', 55, 0);
    T('276643944106775487.9930863762041384', '245610088496887492.4248003422601539495271341125', '522254032604000000', 12, 0);
    T('613411.74839011168044613981643598166768850', '449398986575735509755523534722369885.94559', '449398986575735509755523534722900000', 31, 3);
    T('879617701497896931494638631468657199.08661438461047808129464', '1452340923990542887165591604.3', '870000000000000000000000000000000000', 2, 1);
    T('968632130165056122457532685057451125845.1203222419238070511773083978', '36096558794189392.9117642998231143', '968632130165056122457568781616245315238.0320865417469214', 55, 5);
    T('56076942312851512831292941.7387829985712714050', '2004110160890798396.032847', '56076944316961673722091337.771629998571271405', 57, 5);
    T('87427990217416654085949854944915752637.76210179056668449019149465035', '1775387.4108171', '87427990217416650000000000000000000000', 16, 5);
    T('35907371900941465855307821492134069333.724751257974428709751569150259791', '76242614425670907699916343300740142603.3147827961306113666363', '112149986326612373555224164792874211937.039534054105040076', 57, 4);
    T('521557067113662863134.303318', '510843.3303315', '521557100000000000000', 7, 2);
    T('7410.730655353106029753552312', '4495726136728547876786282385.56929511116653913915112340', '4495726136728547876786289796.29995046427', 39, 1);
    T('6853974957122986753802237007712653677033.06163598581429663672373443', '2795499644434810458787531.20', '6853974957122989549301881442523112464564.26163598581429663672373443', 78, 3);
    T('1031572323286158189713883978351079648.619784458', '3795870231755190253343598280.037528531201650316043017713802544343', '1031572327082028421469074231694677928.6573129892016503160430177', 62, 5);
    T('10097882153.00576849045654', '84200524474633797839135371623.634301101671423705493335497488790', '84000000000000000000000000000', 2, 1);
    T('41.5', '9694.731286264331', '9736.231286264331', 47, 0);
    T('822322928367257519891430699121172963337.75734', '666884.55919899704', '822322928367257519891430699121173630222.31653899704', 82, 3);
    T('21852794099133119840559400129431012.7327625644760887505758865396498', '7308.9363494237324658342512034652825', '21852794099133119840559400129438321.6691119882085545848270900049323', 75, 2);
    T('1864755132005081.2265410230222617', '29192871456524299192398581739978.6983056800174146764374547866', '29192871456524301057153713745059.924', 35, 1);
    T('90503102886864563944164647778027.237319891472798772036832316218', '231193190563687654020345363497352.221090142421925', '321696293450552217900000000000000', 19, 3);
    T('482163363118696669417623380539369342366.385961009887923536', '153432393601.27147944597785', '482163363118696669417623380692801735967.657440455865773536', 89, 3);
    T('8846694337.771', '8019897494649380211.96381679', '8019897503496074549.73481679', 66, 3);
    T('84796470553351476310868013648673.59763136', '3926217617983149943549408.270620259481373129021478677580008185535', '84796474479569094294017957198081.86825161948137312902147867759', 61, 0);
    T('2682.74983448736196865', '7616591641.57351932506296055539383909658408016216', '7616594324.32335381242492920539383909658408016216', 69, 1);
    T('57706804347901598156436118981877.340758161383291894729057', '901916386332396749.514149', '57706804347902500072822451378626.854907161383291894729057', 69, 6);
    T('47172.324489833155273772378176262', '399.231342773991087463010044370442544502', '47571.555832607146361235388220632442544502', 56, 6);
    T('2.55290', '53322.0', '53324.5529', 47, 5);
    T('54563932242.5003070125319099', '434793505591479254067846605594.066390814900701712474744321270204375617', '434793505591479254122410500000', 25, 5);
    T('1986330128.980734249338867476855598634801912166932', '2348673898377091.10287478457519833209', '2348675884707220.08360903', 24, 4);
    T('900202275622.1277', '327.556102942694593157163560699797450197599', '900202275949.6838029426945931571635', 34, 3);
    T('801707405695406919156909597566.732203250042650', '725172.69985', '801707405695406919156910300000', 25, 1);
    T('57894442247635747879211084510.151173646555331670302420941386', '616456349971672956834434460262829273322.1840611694230472515400959192815555', '616456350029567399082070208142040357832.3352348159783789218425168606675555', 74, 1);
    T('9009733332964008708799453537564646893.2146258686894148760400421944512205351156', '6843044259691.017368359', '9009733332964008708799460380608906584.23', 39, 1);
    T('428575080072538144055415899.7348536872550447', '4514149059012960.72202417427823614275621', '428575080077052293114428860.45687786153328084275621', 70, 3);
    T('9726.62898710014265739463374', '387806376020984427611943781.0158733191450098658854840', '387806376020984427611953507.64486041928767', 41, 0);
    T('106604215144449363464996576624188.69050857337174', '74047049687593052093468284396533198965.0', '74047156291808196542831749393109823153.691', 41, 2);
    T('7584456584.6534332400385632431638061', '553204152749810021866895760544858776926.06061225351992792', '553204152749810021866895760552443233510.71404549355849116316381', 62, 2);
    T('25189363172766209104001.619685545709893042151707418', '72796697714509165848167516162.48589470308869088790732053', '72796722903872338614376620164.105580248798583930059027948', 72, 6);
    T('169495385.1764379371653970', '376229415864718801826716.2120996506228200836405', '376229415864718971322101', 24, 3);
    T('638790044978658.4738584647582297760', '4560753791212702640679946046101285470536.486188395233', '4560753791212702640679946684892000000000', 31, 0);
    T('13642043990368.4155284189160388739153297', '39556931.901235', '13642083547400', 12, 0);
    T('483705468165029478.46778985406351409', '15713450630417.44139909462268', '483721181615659895.90918894868619409', 51, 5);
    T('875339288221602.9859103591797', '46045239224299240095949415202916805.694155866860437224658328959852367231', '46045239224299240096824754491138408.68006622604013722465', 55, 3);
    T('227917950630210509966794.4891567978', '6230826111683069881609569722422738591.7079699018', '6230826111683297799560199932932705386.1971266996', 48, 5);
    T('5860.7', '44204041573588.191403148052345293427997381', '44204041579448.891403148052345293427997381', 78, 4);
    T('758980052960832634704906646712.953963232169852708999221813', '64500.9915209818385476', '758980052960832634704906711213.945484214008400309', 48, 2);
    T('82960876733.898963158217496775608731', '54754906787.45880', '137715783521.357763159', 21, 2);
    T('6615233359652648318833.5630281446881037622821617224944339252452', '3559316734106783022894818210721992926.0', '3559316734106789638128177863370311759.563', 40, 6);
    T('810118940147432087176470266.362681509600', '7749245496144964.281639667288488830', '810118940155181332670000000', 20, 5);
    T('993.290308473', '1318974904999527151956318635697021.994559414', '1318974904999527151956318635698015.284867887', 94, 5);
    T('2779312991237361946546.34379284529034397349405854585851101638', '40473822780434918347524224699.26160277', '40473825559747909584890000000', 22, 5);
    T('4172051.4135667653643493843083212649143714', '574884665408551.40639493524949123381', '574884669580602.8199617006139', 28, 2);
    T('72216265485194986201698698044.5078477084095880794300236318158831', '311652672660461136152628064040383284.986367244', '311652744876726621347614265739081330', 35, 0);
    T('1224363455037152086096198.556089045952918716004627247', '9528334232344842619550.699973524', '1233891789269496928715749.25606', 30, 4);
    T('967097102.24163239416883291', '329305839906849003238.57064629', '329305839907816100340.81227868416883291', 99, 0);
    T('2135753732526523959352537605338370932174.76823', '58545792609730.503883313825938119751520568671', '2135753732526524000000000000000000000000', 16, 0);
    T('269585686547.423753591745220204995679530336446743896', '8969412998.65415090480386780876928054', '278555099546.07790449654908801376496007033644674', 47, 1);
    T('69381754816731679980536733685962489.66663666813453', '989571520598410484.0', '69381754816731680970108254284372973.66663666813453', 92, 4);
    T('631016279409419.80114', '79218347118893334427466.84442071469970856202', '79218347749909614000000', 17, 2);
    T('7260144842862683469502163203189614529794.7158448267219', '7033904151259976263297057363608685.702941540094055987156933865140727', '7260151880000000000000000000000000000000', 9, 0);
    T('9709010001763861991083008.8889525040460294693168962', '1783737474.16944874901158296670', '9709010001763863774820483.058401253057612436017', 46, 6);
    T('496.8828970176067027212', '4535.5016438765396', '5032.3845408941463027212', 36, 1);
    T('66430474066698480470589347840.9549829861483632212643446753002892539', '9622048257253073085133.1360879098706114617037103658202', '66430483688746737723662432974.0910709', 37, 0);
    T('316032013.104555420761648629350281297634633150386', '190753630947340485370.3462505350782', '190753630947656517383.450805955839848629350281297634633150386', 69, 4);
    T('196995175395872483593.3887060056530978583729993715005677676961', '0.95764883570520392483322', '196995175395872483594.34635485', 29, 2);
    T('252967683.42089991659', '516253612242858205420413924019418822385.0', '516253612242858205420413924019671790068.42089991', 47, 3);
    T('4648302309127290.2479752864567', '71405982725214956730191804663707222369.74602898', '71405982725214956730200000000000000000', 21, 5);
    T('6344840654832806716225488.09868237196731', '837704110161302727275837567617.942563', '837710455001957560082553793106.04124537196731', 92, 5);
    T('94321383813701737.2848740658', '69536701.56647411800500841044350798669709', '94321383883238438.85134818380500841044350798669709', 63, 2);
    T('38370.1932259685916', '7823214575740128.265803710719221946685601134748517', '7823214575778498.459029679310821946685601134749', 46, 6);
    T('28964218030051847275.98', '3855661181050272492858209.689177', '3850000000000000000000000', 3, 3);
    T('45.1260500035086', '3732798959.91994442813936668731979287654764159', '3732799005.0459944316479666873197928765', 38, 1);
    T('6669999175728326737459249062218.784536170215588933584509977480763922462', '9053488.6395', '6669999175728326737459258115707.424036170215588933584509977480763922462', 90, 2);
    T('82267071358.66861661695010234', '1.2585792242913562190', '82300000000', 3, 2);
    T('75866140292559059954.36723583802204', '638348386261050603892730117577109.3141640834341523274555641720', '638000000000000000000000000000000', 3, 3);
    T('4032364933932919138128400064.543522375962491845', '221561.38175614609540369081', '4032000000000000000000000000', 4, 5);
    T('9231433880723814600200463254075344013.06745898', '40091408565371.6815221', '9231433000000000000000000000000000000', 7, 3);
    T('3183114278566.32158820125293', '7.25364', '3183114278573.57522820125293', 52, 6);
    T('2475481205123178584168151240550068393298.427648408172612728266639473017225399', '73764983256531885560.9780144', '2475481205123178584241916223806600278859.4056628081726127282666394730172254', 75, 2);
    T('204035121320.59602840453472548381493638560', '9958991773580146944.800687274334630089959251', '9959000000000000000', 5, 5);
    T('948843404785773223992908751.898693644762880898', '688189364677893750430890.94440116875824563853558816158', '949531594150451117743339642.84309481352112653653558816158', 80, 4);
    T('81593002550922821101017.58063742336174057556980', '4134258026151926128987.4148889522552094892', '85727261000000000000000', 8, 5);
    T('17010.7700271306303181402217138594767595727517', '11496.126981022638214925103735201693', '28506.8970081532685330653254490611697595727517', 66, 6);
    T('7.58412846990698213843144177665', '4209505598319648624587806167904149.5743207327043016', '4209505598300000000000000000000000', 11, 3);
    T('2224316978072946404.875386', '1721947466419489443.1002330228227380805356', '3946264444492435847.9756190228227380805356', 50, 6);
    T('600443346586800563668302.61341908975', '58966623149534486595.28892658514964100326012633702', '600502313209950098154897.9023457', 31, 0);
    T('945587827041899294292884057.468575552441', '131329554178685520683.432080192477600971610803', '945000000000000000000000000', 3, 3);
    T('69939882991.281932602362835653040763855152625', '434507193200983686124878353496611.7602934595418998906513279173078', '434507193200983686124948293379603.042226061904735543692091772460425', 77, 4);
    T('464330008738685404742023681657894.280', '23921056467291764445394246877501872.29486033743062664547262189824815719165', '24385386476030449850136270559159766.57486033743062664547262189824815719165', 86, 2);
    T('3669927517035009667361104324501.391961198009731639', '464863626543410987139605789865920.01383139803675736875783856086054129', '468533554060445996806966894190421.40579259604648900775783856086054129', 78, 6);
    T('8825977937534954377243684451.6986047635672982110423939308635706232', '47098522175613441188.4', '8825977984633476552857125640.098605', 34, 6);
    T('540128103572160.89440498660375926408773326', '9092016371725.05502996415863', '549220120000000', 8, 2);
    T('254015.476505143448', '1995.58026677667371449375621441', '256011.05677192012171449375621441', 87, 3);
    T('6293219096485205187633690567.655807713943520367911055247932873980875', '9843218909.717333898238042316937585', '6293219096485205197476909477.373141612181562684848640248', 55, 0);
    T('1105687156205250631680.19927', '66192134571830999.5872128028586427505', '1105753348339822500000', 17, 6);
    T('671729170649863961395139.09977468387130347728559715319867051676', '19657227562823.242131940521537588437447202562548545', '671729170669521188957962.3419066243929', 37, 0);
    T('49887699475561.209563897262611555547736284399078847', '331921629.59665814', '49888031397190.80622203726261155554', 34, 3);
    T('504005528631106143882523534404283595.41470814309279875799821', '22845150501818150519431560.863623249801049842508730', '504005528653900000000000000000000000', 13, 1);
    T('501588285681230776091361285177.915209102', '3939536922.048', '501588285681200000000000000000', 13, 6);
    T('7198.0', '89930165266736959076592950666904491506.06922996536076661553', '89930165266736959076592950666904498704.06922996536076661553', 97, 0);
    T('65314059613860083068002125117203226.37648863623664825912', '3334789498205437825564947.90119274871451778256', '65314059617194872566207562942768174.277681384951', 47, 3);
    T('8927073.086473', '4553311739785106965307349.087959068984685023289331403', '4553311739785106974234422.174432068984685023289331403', 93, 4);
    T('424880115170314534.2833907823', '1565066.55707089857074999011115906995', '424880115171879700', 16, 0);
    T('24444831659525801200787546.3351989118870817198886505953115888797', '8838564197416057036504263449.6503734610574860', '8863009029075582837705050995.9855723729445677198886505953115888797', 72, 1);
    T('597063502385231220172485075712690615730.05017', '98495730621253407221847804885894893.383', '600000000000000000000000000000000000000', 2, 5);
    T('48246236077.448', '3374971202.79684063394816299960485089567613', '51621207280.24484063394816299960485089567613', 51, 3);
    T('99422839221428243464818998850.5062217742385407052163556752184261609', '1922745344396333.419', '99422839221430166210163395183.9252217742385407052163556752184261609', 93, 0);
    T('3082040296600936427420733130878.0152318866550456195', '656662516814949966314215648564308.87571742383', '659744557111550902741636381695186.8909493104850456195', 71, 2);
    T('2.6280254348878', '870159098958096451586153427166.37172624907625509299766919341080360523', '870159098958096451586153427168.99975168396405509299766919341080360523', 93, 4);
    T('9285544218279815803.68123166360816089109909116007', '49711288127817623406829763648171904.0171708241471940283169133757117561966', '49711288127817000000000000000000000', 14, 3);
    T('23274123568289.5446466489699593876195396', '47745899443.66342481271043357799119867', '23321869467733.20807146168039296561073827', 41, 0);
    T('8265945741113419029649489551692279.65800083379697232811867902535', '55938922999111671098.715051704420220765328462177907615', '8265945741113474968572488663363378.37305253821719', 48, 3);
    T('9577166246999579191546133220246.5536822133592171233394486638', '474659166091.6598090160904575900051814', '9577100000000000000000000000000', 5, 1);
    T('9675827715471386109025494423545952.71894159501147715368798358', '616863.72790234064003855258022', '9675827715471386109025494424162816.44684393565151570626820358', 74, 3);
    T('82534149765.14416266640098932586313801616349357091', '5825494609505533644.5073588895166352186710007483262747756', '5825494692039683409.65152155591762454453413876448976834651', 88, 0);
    T('77397183934397090.92736232101492557', '698262.78677361898377900201159271210448', '77397183935095353.71413593999870458', 34, 2);
    T('4112.7184677214', '1411.13262711445944784802024167589947', '5523.85109483585944784802024167589947', 84, 0);
    T('697911630975689156.818', '5654839102001526299.1075775098501', '6352750732977215455.92557751', 27, 5);
    T('63404393998.2070', '86953438.88027212', '63491347437.08727212', 50, 4);
    T('65990116936467249376375679619730.824198090203929284993506711553015041861', '20440553653967091.83038205699350336122628103710715488', '65990116936467269816929333586822.654580147197432646219787748660169921861', 77, 1);
    T('566336430.5614477075828', '8796171721195445943818513768.9493105324893', '8790000000000000000000000000', 3, 1);
    T('333459467073659101.16247209', '589165638.80208943440256684779434266303', '333459467662824739.96456152440256684779434266303', 54, 5);
    T('5911204903713610651091202.20963099436535311722533983', '65139004931727218221913390501420.423915093642718319490606977', '65139010842932121935524041593000', 29, 0);
    T('8531067165823807016161.469673', '30260255897293079093120708737.379733768074641341', '30260264428360244916927724898.84941', 34, 5);
    T('467584440897620610983522346256553032625.7570593334580134', '756212.71514464274688471061333171', '467584440897620610983522346256553788838.47220397620489811061333171', 66, 2);
    T('78936482371050125607.74122', '560408640082471800152941321.51311066869953476', '560408719018954171203066929.254330669', 36, 0);
    T('3845355269360107671390.10485745585470175873198827068897', '186874580174383911346990965857359.98237812416866611956810113723228829', '186874580178229266616351073528750.08723558002336787830008940792125829', 80, 6);
    T('7183509622957086969413133841094202.872254470147938656818773427874246754', '237747188013072519139911173540198566653.29257207770959563', '237754371522695476226880586670000000000', 29, 3);
    T('335071.05286672', '759.5390139251151738337111926147', '335830.5918806451151738337111926147', 80, 2);
    T('14533812586242560213383997.447678', '579889663066557476757889.3212382159465352144926025', '15113702249309117690141886.768916215946535214492602', 50, 5);
    T('56130763575663018786732731833843901438.47797589363062', '881047.871925633986', '56130763575663018786732731833844782486.34990152761662', 60, 6);
    T('5061451919936524903725497718.402577583036416309509528579443569', '54442.675643857386190', '5061451919936524903725552161.078221440422606309509528579443569', 96, 1);
    T('411510232353556948.387419089852414660075102480022664', '1252316726.86962881078280486919241363021074848', '411510233605873600', 16, 3);
    T('1770957456954728012.9203548101986', '727064.6851104211401728602915401', '1770957000000000000', 7, 3);
    T('9119586358263601920510908089075802131231.63930237769000152985', '33501367249578539774678.8190840560865216873689203900618168', '9119586358263601954012275338654341900000', 35, 4);
    T('9920815673832481037655276912502.053688927203495480655921', '2973.5283117609', '9920815673832481037655276915475.582000688103495480655921', 78, 2);
    T('128.24287447818566010796759819352461', '5842.5487970345187289826912547180485914314', '5970.7916715127043890906588529115732014314', 95, 1);
    T('1794779884442616455037761.4480803951516305652111085167981328774076', '28252.4582453517191238314213414402256655118878', '1794779884442616455066013.9063257468707543966324499570237983892954', 89, 1);
    T('78914603111282153659740688884604302.33078479096232904588055350178673944719', '319929452196405956760145700183.1242827451626086570', '78914923040734350065697449030304485.455067536', 44, 1);
    T('8965223048062310.601124760971148', '95099657705051475791498821835500845038.39459560125578685464970303934116230', '95099657705051475791507787058548907348.995720362226934854649703039341162', 71, 4);
    T('5804871932252195997575113369.752709213540165818959489388247897', '9537070047676108049357958580.0', '15341941979928304046933071949.752709213540165818959489388247', 59, 1);
    T('6520979966007144.976826346876280297865586886', '77402677159808600137629257138.5', '77402677159815121117595264283.47682634687628029786559', 52, 4);
    T('110824672201121395741323380334869.122255965485994230511085991', '11449205063043389572766499.911447021924924768123095704026822', '110824683650326458784712953100000', 28, 3);
    T('94744012923672256909378031835817913090.944292540042861178612642690536207330645', '515690216474.8603762450445813829992253151792664', '94744012923672256909378032351508129565.804668785087442561611868005715473', 71, 3);
    T('6441399406064032600431172718265.8', '643719023719159648.0775118279763992842843705222', '6441399406064676319454891877913.8775118279763992842843705222', 60, 2);
    T('7809596005.39', '287127263676240898087179975556169.15559480339271587', '287127263676240898087187785000000', 27, 1);
    T('482655619733764190851709880752177425770.0318593765932423798493825752630094', '2325688341950206250363815813533898472594.55447623527339202256912454507378159', '2808340000000000000000000000000000000000', 6, 5);
    T('826217104467755425259.4756527409008845781555639288349', '382041851959394454217547048713466799519.530475', '382041851959394455043764153181222224779.0061277409008845781555639288349', 99, 2);
    T('476772536409495429353.310832052776', '86.4254566099182409316', '476772536409495429439.7362886626942409316', 71, 6);
    T('451636.7761341617815', '3.63298113758663606138183180771167762170', '451640.40911529936813606138183180771', 35, 4);
    T('2198494769166248207319255291.50', '112323956181995536718.288219756661', '2198494881490204389314792000', 26, 1);
    T('514896394025982706.9676183158022568514', '4525234295.13861578474604160438810089714831', '514896398551217002.10623410054829845578810089714831', 74, 0);
    T('75885216093824.368481270755549729563306749739148105876', '5555376863583979220181.16', '5560000000000000000000', 3, 5);
    T('4314132149666849847479338499.795607826739891775245768508684', '60983.49293118356701987462327566187928865028', '4314132149666849847479399483.2885390103', 38, 6);
    T('191.0', '1872144307.06585445893960573528773', '1872144498.06585445893960573528773', 90, 3);
    T('77921542.5992461017775418285616907696424225', '45484679488780.658239466866244237134955896', '45484757410323.2574855686437860656966466656424225', 63, 2);
    T('45679336579261206043821390.679877895482496', '6629.791521749899304148761192513789008723', '45679336579261206043828020.471399645381800148761192513789008723', 94, 6);
    T('83422108131543753739088740.19193163940157750465', '9093051328165432042837803505506798511010.089183662326227173', '9093051328165515464945935000000000000000', 26, 6);
    T('931268818897863784410620.93936947725922702316273927196801572035', '879.7812749660954672761757972108', '931000000000000000000000', 3, 4);
    T('2916618169972420151882113075175.404987711309751422241164379998435', '54.469323292838369440452069998945796487361', '2916618169972420151882113075229.874311004148120862693234378944231487361', 99, 1);
    T('3614.212491441042533827186977', '572048478738636581391468986.0260507581495812925524678958087575593893', '572048478738636581391472600.2385421991921151197394448958087575593893', 82, 4);
    T('993402756971278696861.537623502646938693525999452', '9156896632001228030615362401854021970.6823164909', '9156896632001229024018119373100000000', 29, 5);
    T('4410237302.882731373099870712101104', '1540159557602149182597319.3752297945106', '1540159557602153592834622.257961167610470712101104', 85, 2);
    T('26659516065018962.218079186963821976', '2.0179804111201531291438', '26659516065018964.2360595980839751051438', 61, 2);
    T('10661826120697652502980407.7431769411875338985918050185', '635679480525458590918936281.6221225937087980941497', '646341306646156243421916689.3652995348963319927415050185', 89, 4);
    T('93063615357962587.1932426556915', '1.3414966219746978212415503074', '93063615357962588.5347392776661978212415503074', 98, 0);
    T('86681325705738866941343998829.78778220089798463855706736859717079', '351.1779664741388204', '87000000000000000000000000000', 2, 0);
    T('8404760325036052311110243142121685442339.489504575', '1028569248854547383.312', '8404760325036052311111271711370539989722.80150457', 48, 1);
    T('1894254877291315894260806663602106214539.9', '43014502729.5178979462145', '1894254877291315894260806663645100000000', 32, 1);
    T('2414934121708746995964899627665350.21751664814599209906317254785703', '8489415918903630570565981958068952.416486733463', '10904350040612377566530881585734302.63400338160899209906317254785703', 93, 6);
    T('25715.649096584', '51946855.765902', '51972571.414998584', 19, 3);
    T('53152691086023937013407390842.47037884452339093579378', '655489543543889358677785316563.18098641551253673317247', '708642234629913295691192707405.65136526003592766896625', 90, 1);
    T('46408025376025989601538839364808626173.4959675479148574590668241906624', '7322152291422357572.2397163564431127', '46408025376025989608870000000000000000', 22, 2);
    T('223.0683698816', '80078519915083447483459351.8210116884075543435631', '80078519915083447483459574.88938157000755435', 43, 0);
    T('13171859105866486068622052177.362137', '2823960067769562883685128854268.202446722022951215645849304704800307820', '2837131926875429369753750906450', 30, 5);
    T('50687933249168576803802269126769.0512211912171933541302596508311220', '9195018291195432859.1781', '50687933249177771822093464559628.229321', 38, 5);
    T('72392294399.531136983884292429946', '71484193772156.86391130939506969276270200426441527500', '71556586066556.395048293279362122708702004264415275', 52, 4);
    T('98687301099777896798.2122329567323827385206127730254620981', '24653601590439883261429153423751.4588796504299240937', '24653601590540000000000000000000', 13, 2);
    T('781169635952065.44758736174360534039564278519', '297415195102940350281.21951520630868717461965188624905893546', '297415976272576302346.66710256805229251501529467143905893546', 93, 6);
    T('31875824563477409512262754.1691801631698687176509473', '6253.070011360862949422456362', '31875824563477409512269007.2391915240328181401', 45, 5);
    T('4014235600026064192696863.95578042523531527733759024874754', '688555253671257528956111872258942314.45458654824', '688555253675271764556137936451639178.41036697347531527733759024874754', 93, 1);
    T('617892491.64854226354098680134889341323771857959', '82799616026.7142393985442402858066140', '83417508518.3627816620852270871555074132377185796', 48, 5);
    T('236.587066158424957309214876', '204788515314335316677.05852418126210889108549744', '204788515314335316913.6455', 25, 3);
    T('77178534472068807664478345034763.1131898086822292718361441365', '299898525062070257929.51640775428035663396247', '77178534472368706189540415292692.629597562962585905799', 53, 4);
    T('23822267143677925257513013748737.723361', '5397987551229625057282083638.96', '23827665131229154882570295832376.683361', 100, 4);
    T('20233363921377221532559866536971.3194534887748550731504413', '5548158043419436445638197952628734.835085986791755293192106606606', '5568391407340813667170757819165706.154539475566610366', 52, 6);
    T('723703682414.6832672912243315', '49885292429.76461', '773588974844.4478772912243315', 46, 2);
    T('56.6383', '845296291505840719.61023159535867498536', '845296291505840776.24853159535867498536', 48, 5);
    T('64689508355130192415.2095853196978282022995335177434975338843', '0.74420716191938064996762409902564797', '64689508355130192415.9537924816172088522671576167692', 51, 0);
    T('3833.9', '5775583235538851660.177073278646382161506251453362195', '5775583235538855494.1', 20, 0);
    T('71957881883376560013570133279670619.48365000454591421812009557598237', '39452547667321869999933008594634853.085293769085622451536', '111410429550698430013503141874305500', 34, 5);
    T('82586949176308.57999124691578589', '36056052069438245349044751303.552070334557156655229', '36056052069438327935993927612.132061581472942545229', 63, 5);
    T('57.7294749190383444664936794', '548903889306752225755512468633626794423.4883233899968105', '548903889306752225755512468633626794481.217798309035155', 55, 6);
    T('64532228432.4203443667716318257114940482233', '33027227019857333586948075.067081555633103508943481107543641', '33027227019857398119176507.487425', 32, 1);
    T('6729736467680729485712621586168.24053417452657151042062586004197100', '102709801466175.3271398944684959218911399', '6729736467680729588422423052343.5677', 35, 4);
    T('6483724058396516311919161837044393.4752876068596', '7535217152270396.58', '6483724058396516319000000000000000', 19, 1);
    T('42158294.72595729', '501214758630491578526232031264.11677302532', '501214758630491578526274189558.84273031532', 59, 3);
    T('98076667847155731681088966450188248.6172492335091279864915', '3948798171382334774.29970237436670625', '98076667847155735629887137832523022.9169516078758342364915', 71, 0);
    T('5617514129728137775566029234938361448.5749469795474779519263242963903957', '5020477317588758623513917645.8759271833174368719338378339966097', '5617514134748615093154787858452279094.4508741628649148238601621303870054', 75, 1);
    T('289341269382592544220.409781339980', '148.127303692776805456777027307269', '289341269382592544368.537085032756805456777027307269', 76, 0);
    T('375634147874346262382859181312497719.2543991762956418707318', '375474452735200966869593705223.5555744848981819739387547965', '375634523348798997580000000000000000', 20, 6);
    T('47427824116546533248757971855028370.7', '56827777032222816957385679458471.96688496305386363', '47484651893578756065715000000000000', 23, 6);
    T('484001158315407945541399080.382296227157121581536510720510668006', '9730558361908509871241776223.08108599362178125067894942127923292', '10214559520223917816783175303.46338222078', 40, 6);
    T('4753991375293457781976114.958701676190', '653618213815481495238676.40226326601760990183378', '5407609589108939277214791.36096494220760990183378', 70, 6);
    T('6382930947080210.7600109904146333114251682751540792249', '808372071388457679354.97391111877492621226463269', '808378454319404759565.7339221091895595236898009651540792249', 85, 1);
    T('7201774606572308863827763033318200.270971606949073548', '105411023890962788309156424379449226.368511885980812070908393', '112612798497535097172984187412767426.639483492929886', 51, 4);
    T('92333955506.95937343405444970', '632802712191512.114822740323084119639854948212172441', '632895046147019.0741961743775338', 31, 5);
    T('64179248836536665243.5473112101499024906706725350', '7438680.15383273251861458399', '64179249000000000000', 8, 2);
    T('25945723714891019275717419609215812581.9010', '4079397187209675970.60387694927927972620022091', '25945723714891019279796816796426000000', 32, 0);
    T('0.041597720116576355422853', '2347201095.652', '2347201095.6935977201165763555', 29, 0);
    T('158325426171076.53769006737174887071885985893', '9060.02650519206161249414350', '158325426180136.56419525943336136486235985893', 91, 0);
    T('1650264132015786790.699936', '256216869517873675177.1814267364725037001395', '257867133649889461967.8813627364725037001395', 68, 6);
    T('274407748553393633907638377.316086811384', '622111276790.02', '274407748553394256018915167.336086811384', 95, 5);
    T('26606.6', '4015357.3512485128657', '4041963.9512485128657', 82, 6);
    T('7692720315452813831320362302.69019321613264497152758733', '52437.335050413099064251092', '7692720315452813831320414740.02524362923', 39, 5);
    T('2653630888.6036299', '1.642631359505938', '2653630890.246261259505938', 63, 6);
    T('286882208458762640636527384515.078100817239104142517545516337', '80130049412908371667157476.6517807319260910424', '286962338508175549008194540000', 26, 3);
    T('448979234991597295628401885.66258447754010676471511236491', '320257616978198493.4743361968380779588120839113060179', '448979235311854912606600379.1369206743781847235271962762160179', 69, 1);
    T('52627.5964574602516208850732172140601844305270', '68426518306726004721778390594.133570674', '68426518306726004721778443221.730028134251620885073217214060184430527', 97, 5);
    T('91893885952288.363644311814408694487419146', '55.81037710661569844087827751656360573', '91893886000000', 9, 5);
    T('625326719030495183608343.83419320365', '75.5430188325234299', '625326719030495183608419.3772120361734299', 60, 4);
    T('52349323323099245279630435705731297.102459411368859128169', '807722515354230707396788182255736.12516923754611653260353071400', '53157045838453475987027223887987000', 32, 3);
    T('9030163177862443538041383.2903640425513666940398748111532893293', '8597366180798797378423799.94', '17627529358661240916465183.2303640425513666940398748111532893293', 74, 1);
    T('5302578214492709922.6530859216057752648010281987321750650975', '62437647.5142351793', '5302578214555147570.1673211009057752648010281987321750650975', 66, 0);
    T('25473447403151278437550948.67565738585583721817003532330594601891', '4000137561310728935462013557005.34', '4000163034758132086740451107954.0156573', 38, 3);
    T('2668854333.047876332348223415920658722810829794406', '9748.6123865346402126110624861004', '2668864081.6602628', 17, 1);
    T('43238019060963058287181.779834768521205431', '4358018351673441608955414553630168054.264232698086724819549117960', '4358018351673484846974475516688455236.04406746660793025055', 57, 5);
    T('86352566.7532064793676072384395492361687936', '9369561008421582006788814662589240.363080939818861', '9369561008421582006788814748941807.1162874191864682384395492361687936', 83, 4);
    T('477675957647000512808213692543.2040263307727827454766105576866098', '69295474528871783523365262868263302.0', '69295952204829430530000000000000000', 19, 0);
    T('28911430165642306726107440066736976701.1582486315284292', '22.38', '28911430165642306726107440066736976723.5382486315284292', 81, 1);
    T('8054383384617476242058375687648743.237', '308122.131019529180156872', '8054383384617476242058375687956865.368019529180156872', 89, 3);
    T('661430333876272033235240305764052035.97477197075352576056439842528566', '877031308009887.0224893703723988275582996582', '661430333876272033236117337072061922.99726134112592458812269808348566', 76, 3);
    T('72687.1939666798040730550844562284193781415', '837943821415099403.0907898803440282142040189671662148561340', '838000000000000000', 4, 2);
    T('727599.93', '78897353160105604011080.7470897587505217552201599', '78897353160100000000000', 12, 4);
    T('15315290082.903923943668320647780519', '3770306794029037999960430010889.920', '3770306794029037999975745300972.8239', 35, 6);
    T('416407835318061447609480848490587646.033919987037905738715417299702511895508', '1937504122486332916799284156500093564.0505896775609307216021', '2353911957804394364408765004990681210.084509664598836460317517299702511895508', 87, 3);
    T('1759129017097121060413982.50654', '3967662840152061661150081.778824559930889327372', '5726791857249182721564100', 23, 4);
    T('8128232815237159389877351756389863942687.8202246811774962302587360214193877512', '23901951.3230082605962713370095628607187231', '8128232815237159389877351756389887840000', 36, 6);
    T('198283136235071580.36168853689971243', '7835059942194267516420058.1989', '7835060140477403751491638.56058853689971243', 75, 6);
    T('6907962919636.354525718688913483371426830604042568', '33691452799747059868728942396.634617438340668901420015', '33700000000000000000000000000', 3, 6);
    T('57.906222288504385686137376309637067338377', '31961239874873379.2568626784500927186817536', '31962000000000000', 5, 2);
    T('22236798370284531940965386090466.7390871520195534606168', '532619484364546185.8968684534775835830450', '22236798370285064560449750636652.6359556054971370436618', 84, 3);
    T('119576726848432051874007522221277.80467962805379555355838564400906425', '317464480945298078127310356.1608253040172219963262894', '119577044312912997172086000000000', 24, 2);
    T('2303043875969528186703408829447612.536391267083597492333223671031302479267', '380414.75493439440196008185752123101', '2303043875000000000000000000000000', 10, 1);
    T('905241124872642660.1644971381476580218743617', '787580960727922842272448.1633166548116', '787581865969040000000000', 14, 1);
    T('65194949833789962.4145331', '51096140013743324523153555576.3683786585', '51096140013808519472987345538.7829117585', 64, 5);
    T('5619621989824905882958344880268777861.25809203162791192724795', '85676.860061', '5619621989824905882958340000000000000', 24, 6);
    T('37295585832100001832382224703518001343.2', '29358561850.905484012009935937341', '37295585832100001832382224732876000000', 32, 1);
    T('4766473263013003558914756.0', '968889329611255586.5256627515292090627647', '4766474231902334000000000', 16, 0);
    T('93258919273029362475507401025915725792.228759289503208287231820', '8830386822347632393133.884805198868995020947', '93258919273029371305894223373548118926.11356448837220330817882', 79, 5);
    T('1578553.756914399273591966563325937687', '328866628677628877334307.72610510600294767612789677671792691', '328866628600000000000000', 10, 3);
    T('52451.4249660935998', '9248634318.751062507631104754600117463', '9248686770.176028601230904754600117463', 41, 4);
    T('43919429479678020641.77455559339243216533219820174277', '3025360165324659383170.9153595', '3069279594804337403812.68991509339243216533219820174277', 84, 5);
    T('838226234673596014557590.09345290295707', '2936126809970132767671226699912620.9277358393161439', '2936126810808359002344822714470211.0211887422732139', 60, 6);
    T('407406246533.1551347730642930974751743791', '995471884405690431550937745312077721.7346771710623256260087914530238714', '995471884405690431550938152718324254.88981194412661872348396583212387', 68, 3);
    T('55080233067959697105819720093750967.466245838541820148545809849807096087921', '16653449183354506936.0175204033321288842290', '55080233067959713759268903448257903.48376624187394903277480984', 61, 3);
    T('733718038186992630335479755.03633084108579212803700', '52141357197443.507966060896510815435556', '733718038187044771692677198.544296901982302943', 45, 4);
    T('6621064642148.0931683548206859958292156130530308', '440.1089042637988663', '6621064642588.20207261862', 24, 5);
    T('33840386479908195416416865589819.515782877711749328571685674550619581357', '6417401445104067137871054282460.30977876', '40257787925012262554287919872280', 32, 5);
    T('770041962.510524', '8989894702062587433144.970999652674736639519637', '8989894702063357475107.48152365267', 33, 4);
    T('6103073776095591256.69250268', '502661035559648125088236.13065883132239548045759', '502667138633424220679492.82316151132239548045759', 89, 1);
    T('57670318693261477265708246896371.2044162309', '431711470.546069742580200799952753824424482', '57670318693261477266000000000000', 20, 5);
    T('5514230611103502728.4192006221', '6011010.7367', '5514230611109513740', 18, 0);
    T('67433752785274996110972424739740823060.93727621605753774241615794632', '31767756418947699428575714579.3980590563126', '67433752817042752529920124168316537640.33533527237013774241615794632', 95, 0);
    T('9323822139698476915230564842.47750298205576322848569390526', '798859259124.10569497', '9323822139698477714089823966.58319795205576322848569390526', 93, 4);
    T('43031586187955103408070899977356955.0275875213618980782100746649210823721820', '86412162428829478409510.06', '43031586188041515570499729455766465.0875875213618980782101', 57, 2);
    T('26477074621067853434259288774732.5578726327537', '223382622943274.421654337777915928184095193883586445', '26477074621067853657641911718006.979526970531615928184095193883586445', 71, 2);
    T('8030452497189470872161003575279618967178.45592695127317', '1283.0258415963131174858099047190739497', '8030452497189470872161003575279618968461.48176854758628748581', 61, 5);
    T('984203995402526788477700724.9', '56217235367.52392476138581889087936', '984203995402526844694936092.42392476138581889087936', 73, 4);
    T('402676979793880041715684976.604697252339129545709740439168725052074', '83760889697426012403269479392608375565.11497443523392752640580', '83760889697828689383063359434324060541.719671687573057072115540439168725052074', 81, 3);
    T('5435.0', '671.8683036357246432691773114', '6106.8683036357246432691773114', 35, 5);
    T('13050304503980765739275082.378946075264156989169663144159837052638', '66778508621555.0', '13050304504047544247896637.378946075264156989169663144159837052638', 85, 0);
    T('9983129.169027194572305164808553454038839142828', '12085136.13500017656407195144284112459915826', '22068265.304027371136377116251394578637997402828', 95, 4);
    T('4053681627306152072.4552653878222943155745394181001', '994237656264809493470.0', '998291337892115645542.45527', 26, 0);
    T('9219824256.75626422702665979822449647605996409', '6504845485226697530727154.91621809677128664313621660017300', '6504845485226706750551411.67248232379794644136071307623296409', 65, 5);
    T('12005755135523577250158658.2526', '32233461744079843967538.9137835956695391246477083331247568', '12037988597267657094126197.1663835956695391246477083331247568', 70, 2);
    T('48.676805573138887217', '619104403.0745994854388120886053', '619104451.7514050585776993056053', 34, 1);
    T('628713357242287493850599067506521.03409409966118', '97268744691742.60188995398415079583636232663', '628713357242287493947867812198263.63598405364533079583636232663', 92, 5);
    T('340408143059363976501178338660473909.444542482579615410', '516812947753856216151016370.34402913649633521448212320916142', '340408143576176924255034554811490279.7885716', 43, 5);
    T('94.9959455459002882495569977352', '56674786.228282862822618617', '56670000', 4, 3);
    T('36126434884034378.161777196605', '97.27588', '36126434884034475.437657196605', 100, 6);
    T('847714887976328574745034942184617.713788', '5265485471820367.1527460793706689037749', '847714887976328580010520414004984.8665340793706689037749', 68, 0);
    T('446882449734797160216385749.988071426', '7602347630072412072896730106182394633.464185705426851954973122', '7602347630519294522631527266398780383.452257131426851954973122', 99, 2);
    T('4775360098905271651182.67487108999893606741823213661183329215', '155340004553191398353558938739561550.8724785085137144382182861500697228049', '200000000000000000000000000000000000', 1, 6);
    T('60282807694719199738373903053543595.510839', '8089926391989532584449033471292156245635.5129741275148648866996737373745881985', '8089986674797227303648771845195209789231.023813128', 49, 5);
    T('82.5056245757', '50207780211939110481585734908469.538829667284021047153916', '50207780211939110481585734908552.044454242984', 44, 5);
    T('369185343563314302532.59876000102670901835522375320', '2699658830.8', '369185343566013961363.3987600010267090183552237532', 83, 6);
    T('859776.3007153822530820306866585344', '1551087115877934098729285169110987.0', '1551087115877934098729285169970763.3007153822530820306866585344', 95, 5);
    T('8988475237564163987582680045.6874563226401168', '930.39796435099572245042707251671395641', '8988475237564163987582680976.08542067363583925042707251671395641', 69, 5);
    T('1997341228569624017884387706097.7', '588806609685658163.18690525555', '1997341228570212824494073364260.88690525555', 97, 2);
    T('947707120514061552275852173.1785071050274', '4398668210145474.90759276218334436501087578321442', '947707120518460220485997648.086099867210744', 42, 6);
    T('28089654635345438.78253', '70514044140176857232629411866.733828245516731820', '70514044140200000000000000000', 13, 6);
    T('39101628515477576945544484109.380718052257065', '6153404665334143192116811.024135821609750693037034612945678', '39107781920142911088736600920.40485387386681569303703461295', 58, 4);
    T('94691277.177644091077930115', '1532999827467391980756719.6268442916282197457028345059788873158', '1532999827467392075447996.804488382706149860702834506', 52, 0);
    T('63571290026893655935176.058291574766271509001789066344497', '48316600258417303.236049294290020804724887690374625178510', '63571338343493914352479.2943408690563', 36, 2);
    T('98906727246908.550973763877135746173783076366854335684', '3810531260726785725804492760.416494431817657880737050658100', '3810531260726884632531739668.967468195694793626910833734466854335684', 88, 0);
    T('808266191752515391813769315.42170346245144570977134462853535', '86638541613954207924248581.6618960514718916933481', '894904733366469599738017897.083599513923337403119', 48, 3);
    T('1860810.04126442513072960298906', '373615391038.8535848319776695', '373617251848.89484925710839910298906', 61, 5);
    T('74884.392765119680', '14896.78174952195414340320148', '89781.174514641634', 17, 4);
    T('46729423560803742522916116726397.671064855', '0.6244910431321667', '46729423560803742522916116726398.29555589', 40, 3);
    T('78531578.7', '83631331439444727679863274.9802149134345', '83631331439444727758394853.6803', 30, 0);
    T('2804636414886057933692722084809954824.732211582769', '42518172.490007951315671591834', '2804636414886057933692722084852472997.222219534084671591834', 71, 6);
    T('306128071043354587652834.12854604228514929', '2456587.107515840275958965770651526251952853', '306128071043354590109421.23606188256110825577065152625195286', 59, 2);
    T('6947140458747.39', '75752331216333880664483392583.0', '75752331216333887611623851330.39', 51, 0);
    T('3917546285.7860373129389862219945539131947010798', '105042130641515273855165723731.891547226415', '105042130650000000000000000000', 11, 0);
    T('3816600901964.61930986605608159', '9202291345903196118.942860796', '9202295162504098083.56217066205608159', 93, 0);
    T('19819893570623559407760112656325.1815722671985698748', '3369397046252491044688936748322260151.49898720159108236584803155', '3369416866146061668248344508434916476.68055946878965224064803155', 70, 3);
    T('296877311356440.04823336576', '5629629731892109175662362998367541177.05338582102450246', '5629629731892109175662659875678897617.1016191867', 47, 3);
    T('26599.69892424183098117401581607345', '314942.31', '341542.00892424183098117401581607345', 41, 1);
    T('992965355550512.460381649239344677956457', '7056609336197166672193094285506434147.6140090572102', '7056609336197166672194087251000000000', 28, 0);
    T('32878.9293708667264192461549', '3.4787485605033898809099486021325568282', '32882.408119427229809127064849', 29, 5);
    T('484364287195.5452355893464920765371418081', '3.84066548833464565737188943152306401', '484364287199.3859010776811377339090312396230641', 46, 2);
    T('496496749023010017196130241881484.21', '626376.29784098', '496496749023010017196130242507860.50784098', 100, 2);
    T('139991382886074293551080665.628528149276083095362620129178801633', '684130047954175282799288160438506.2423997092898836458177', '684130187945558168873581711519171.870927858565966741180320129178801633', 94, 0);
    T('127007927852957926402539610149342.91181455214825517442132662345356571653', '3983869986987663809464803039.346137896176224753', '127011911722944914066349074952382.25795244832447992742132662345356571653', 100, 0);
    T('677895444656.407656684947194278541669454326201579125', '9519440669197218074625659827.11958796812666768427279', '9519440670000000000000000000', 9, 0);
    T('7687.583937029314124134414', '8787837788217219810696537342894.2269283417559711597507114', '8787837788217219810696537350581.8108653710700952941647114', 82, 5);
    T('7090397211192936925082854439713.205506605944507402167176636462', '28357791463923.981848320597077009076301518327737606282', '7090397211192936953440645903637.187354926541584411243478154789737606282', 98, 0);
    T('3848555465154372816231243075164.0', '463030655561087664084515170884801597.750066051748409523077516', '463034504116552818457331402200000000', 28, 0);
    T('8395648570.3981367139089721348117143', '5683288006927376467823684846449165.936301290217277375148', '5683288006927376467823693242097736.34', 36, 2);
    T('1720502604681479741565555623.9', '33577106284326616499244376.35654', '1754079710965806358064800000.25654', 46, 3);
    T('63.11579', '3562650866210222.315121627061689827632779122761877642897', '3562650866210285.4309116270616898', 32, 3);
    T('799340834715148267740981607516.07701071149921140668077095040492', '35297480825394678789597364631484156.958168951711708688293872143836021747604', '35298280166229393937865105613100000', 30, 0);
    T('2611345311725.619131024', '352331.299258079662716174897999896872135', '2611345664056.918389103662716174897999896872135', 89, 0);
    T('5328018420521590194244938369911401529343.15773390623155', '682840247219269599553.8233853321', '5328018420521590194927778617130671128896.981119238332', 52, 2);
    T('63997402437054258429030926.7451850202900', '254955424140933794782942.50738446300617615270810564219294326430', '64252357861195192223813869.2525694832961761527081056421929432643', 95, 1);
    T('8110511150580444837633313651703335746.7', '2950031220091199860602148.6666782417932565120191621120460', '8110511150583400000000000000000000000', 14, 0);
    T('8953068196419.78', '2687475008015288987363.011533265209589910602272494', '2687475016968357183782.791533265', 31, 5);
    T('24268058042843158672182074746119550.6760391593', '289997617152713753126158970729522634.575370885634318176612', '314265675195556911798341045475642185.251410044934318176612', 93, 5);
    T('2939885941046928.84435387065737410258584727', '99526219252609998621366680.44421000645054589537681516664024', '99526219255549884562413609.2885638771079', 39, 6);
    T('78030465131954856239127374670255.0697', '51717807192813989117059443130233844381.411615508507427993571109270625', '51717885223279121071915682257608514636', 38, 4);
    T('4316580900722440634454293404103828030.7934641761664668613982', '75.1614617124500959347370677826', '4316580900722440634454293000000000000', 25, 5);
    T('8759672522374077.26134117224645674857146872200848098874', '1987897726473089364.35323839168316711181778211365', '1996657398995463441.61457956392962386038925083565848098874', 81, 4);
    T('70174632668861270692949565725.256440468919', '5388158894984361331880899556693.4', '5458333527653222602573849122418.656440468919', 91, 2);
    T('7623.31092711092', '929709035325722936316048.503516166', '929709035325722936323671.81444327692', 56, 2);
    T('3581613195045123104559533551371326.4896259', '395.798987491120476723245187065580553463', '3581613195045123104559533551371722.288613391120476723245187065580553463', 92, 1);
    T('49897.99697565090091693903', '325661373454.0', '325661423351.99697565090091693903', 39, 5);
    T('44991467874.762118855365', '4057249999086.88378819861887', '4100000000000', 3, 1);
    T('50.5280527999528', '413257523454849789195457968150352.4627480101982665606759883', '413257523454849789195500000000000', 22, 5);
    T('88047825083209193854886.0', '8583822445729242696958241321525117.038495374912535051386903', '8583822445817291000000000000000000', 16, 0);
    T('32323414910687.84033', '2385484768.10228701315589011772171105033446162', '32325800395455.9426170131558901177217110503344616', 48, 3);
    T('2099005.32', '618316302842.2159700656010998197', '618318401847.5359700656010998197', 71, 5);
    T('4215550361.4227735636003311474594713', '49305727464089896170140041246484395399.173735021802390385867', '49305727464089896170140041250699945760.59650858540272153333', 58, 6);
    T('914366724406099551695557010222.63', '478105110829955620491729791715188954261.8047191223828785388', '478105112000000000000000000000000000000', 9, 2);
    T('85794906134474314.800643986726235372058294334297144371', '38110302226.72038166640729', '85794944244776541.521025653133525372058294334297144371', 64, 4);
    T('82386286907906453954.6696396', '878409359918357248617471865485352.564772441962579594', '878409400000000000000000000000000', 7, 4);
    T('854696610881988043435684954.9545246795170573414159879629839432685492', '1038205012011873726998631375.4582427429840082764667025', '1892901622893861770434316330.4127674225010656178826904629839432685492', 89, 4);
    T('275565517740434.371252062901247504808', '2923587434381475107593707366.8621605327396723524865189966720651656152', '2923587434381750673111447801.2334125956409198572945189966720651656152', 96, 6);
    T('92384112584947411663715187269274.35176422266184', '994603854788.22378270775653095256401331407581037073', '92384112584947411664709791124062.5755469304', 42, 3);
    T('39668853.424925966053811', '8696536.795950052450747724391328003565081244', '48365390.220876018504558724391328003565081244', 60, 5);
    T('4761958668536569.39854303012147943908274478760', '294170335212995202787440585268367745.8496', '294170335212995202792202543936904315.2481430301214794390827447876', 100, 0);
    T('33.1636079522702212', '98738488222154007526364097337446.097323547777836768649257', '98738488222154007526364097337479.260931500048057968649257', 65, 5);
    T('29918.0147253552245499', '12777196005079013626428009623334069.281580573008360897048600', '12777196005079013626428009623363987.2963059282329107970486', 84, 0);
    T('6238.6635168552231749748123865', '891566592720571933964.768243', '891566592720571940203.4317598552231749748123865', 74, 0);
    T('854.092549663330523647804848853', '918561565195249747973425207556283.828', '918561565195249747973425207557137.920549663330523647804848853', 76, 1);
    T('21.0', '534696688.5940189188701892808491204', '534696709.59401891887018928084912', 32, 3);
    T('243247005881.63360221370', '4081634795747726413414801667052113166.758140433129185712233359498862610502021', '4081634795747726413414900000000000000', 23, 0);
    T('317118805623922948290762978699753850.19259854240089284789867699153900', '78505.297428887', '317118805623922948290762978699832355.490027429400892847898676991539', 98, 6);
    T('800196016089631002731.9320228036288406702', '24364.33251293849314069366487382', '800196016089631027096.26453574212198136386487382', 51, 4);
    T('10018502448372313978516821310534113885.0384573458892987618716776484358900135976', '615474552026831806613084599907282093125.62691383960825', '625493054475204120591601421217816207010.6653711854975487618716776484358900135976', 94, 5);
    T('87893328356824012242567.600350918447678745486932009', '683106064807650226721.80557407790791630937476289002774584', '88576434421631662469289.405924996355595055', 41, 6);
    T('4708363772036277.22057335880253399', '3093964020228550868544681616529823.950368063558397481676', '3093964020228550900000000000000000', 17, 0);
    T('2964180622418398180215568693753026566882.07761331841840653777449530933017', '742.398482624644176806562495539', '2964180622418398180210000000000000000000', 21, 3);
    T('72612309385308.40691415235926408182764189217695242240', '500897811372441233701406069088337969247.655241309262460140758243258', '500897811372441233701400000000000000000', 23, 1);
    T('608426633747752437.33898089200787492688032292694672', '622113068770442668800616387340510.58192038012255481216125209', '622113068770443277227250135092947.92090127213042973904157501694672', 85, 1);
    T('4567810407157893.018212160852525385337332', '7797957157997422271258694.844001174992205180329', '7797957162565232678400000', 20, 4);
    T('4479834807417674693276084364698.09378', '809520946788861530394957.930906673036184766828484625380', '4479835616938621482137614700000', 26, 3);
    T('884407060865472183288625548.195173629', '4992522.04276213701681416347315162044', '884407060865472183293618070.23793576601681416347315162044', 59, 1);
    T('81052005707975632155322974990383.002865392804143', '720614740393877203443912295413002.0', '801666746101852835599235270403385.002865392804143', 86, 2);
    T('1456215052520585.0', '80726210992303533.9708660050138735', '82182426044824118.9708660050138735', 69, 4);
    T('67463060947185.9977243218564', '63271599653560263371481937670906.545034162', '63271500000000000000000000000000', 6, 1);
    T('282509601988198977742500998621161735.281996284671431806552', '1442197238118225.4018057718460707356344439379837514643', '282509700000000000000000000000000000', 7, 2);
    T('12154790573737972840974304359343111.293', '60363518225484.06655489483207826840', '12154800000000000000000000000000000', 6, 5);
    T('31439464505838237605076395488.792122052343092113451500', '8802517569715.908389881145286887080837109278015', '31439464505838246407593965204.700511933488379000532337109278015', 70, 1);
    T('3442804627106418943002953869.0492341104396009719083846733105', '160.38894235737', '3442804627106418943002954029.4381764678096009719083846733', 56, 4);
    T('1887370623009515.514248894446361673911929708579774', '618739771680050528864671768807788.20793937742789286804293979007647125351', '618739771680050530752042391817303.7221882718742545419548694986562452535', 70, 1);
    T('337201080.76779', '38428990757641225.48418051932513764309', '38428990000000000', 7, 3);
    T('2776978.7651418731089748068038164993251', '450600115119.052379923495436', '450602892097.817521796604410806804', 33, 6);
    T('45656.78526154468781447730982', '456284124704440.44452886539373502', '456284124750097.22979041008154949730982', 65, 0);
    T('5280446905.42516586883815130899', '852597251010131197194.143619378663681227', '852597251015411644099.56878524750183253599', 84, 1);
    T('7785684814763.82163702924460534', '2192180943412138462023911241202734198.650908681167947418773148844697060359510', '2192180943412138462023919026887548962.47254571042', 48, 2);
    T('8819375493051200963.597375019518468554794365', '90837320657390224086650049520115.57428', '90837320657399043462143100721079.171655019518468554795', 53, 0);
    T('251853086606.0858783', '6396140253294226403751.4656445519164967496842675674623634435120', '6396140253546079490357.551522851916496749684267567462363443512', 62, 2);
    T('611266164021364706242542809496668509.0', '2311666371484.8761597856572095031799777268815389476', '611266164021364706242545121163039993.8761597856572095031799777268815389476', 92, 0);
    T('4863841304401048509.7282101', '414953753595780441105727998511406560.8556645373474142036732369580160063109', '414953753595780445969569302912455070.584', 39, 6);
    T('81222961180033745571585676422150200858.0', '1501351312010201793019764319974.3300428500192270695803049385669517876509', '81222962681385000000000000000000000000', 14, 5);
    T('38469295218982526051780032240419123.0161680333616105246086218170421989', '92259.775519240043307', '38469295218982527000000000000000000', 17, 2);
    T('499726043532290.96513894708485425609849236231', '5725919995441222256039693644622299131831.7350905246342613035370', '5725919995441222256039694144348342664122.70022947171', 51, 1);
    T('5081291.1498968', '65424457261994177.008300200019179679471112591496', '65424457267075468.158197000019179679471112591496', 51, 3);
    T('750163978.07381470567831284782012533050', '82837264.02506284', '833001242.0988775456783128478201253305', 46, 1);
    T('70607062846458677102667615082.478331912635347607000600328500207', '83935184808214635475990.30498350374843634405993363515128101014', '70607146781643485317303091072.783315416383783951060533964', 56, 4);
    T('406.671101792', '8020.117498513870218560338516279958018', '8426.78860030587021856033851627995802', 36, 5);
    T('36507151530.20296585110010887164809505985', '1517129383274036925314575712444834948125.94', '1517129383274036925314575712481342099656.14296585110010887164809505985', 79, 5);
    T('55093676161512220668708732.05330', '503156610241445849303135154891979398697.86342092651130331777', '503156610241500942979296700000000000000', 25, 2);
    T('841261285088703437649432351419151360422.9612469895025040147251180', '25405500233086166934808342098851868782.4548155650584392940547', '866666785321789604584240693518003229205.416063', 45, 4);
    T('42800579973165927674767330367930327.125437032834', '80.952159654843185694', '42800579973165927674767330367930408.077596687677185694', 57, 6);
    T('735050777070311551073453.06542605509994948132498', '7846347109318.81108249651203698990035479341', '735050777078157898182771.876508551611986471226', 45, 2);
    T('3015939462264551339860050561643554.10040473528', '95708913657613419350349.444911094180646616370628897269632224313', '3015939462360260253517663980993903.5453158294606466163706288972697', 65, 2);
    T('6803808921631378578144622605704838878636.896004787678862320405976463256133', '1565908732567122348612543239480.9061787707411492524674', '6803808900000000000000000000000000000000', 8, 6);
    T('612300980255.019323161315948524979262562683', '8842660874642626344269439.72286516187055302', '8842660874643238645249694.742188323186501544979262562683', 62, 3);
    T('211807863224076856104.363046', '79939715093409095320342100829575547.8194009162', '79939715093409310000000000000000000', 16, 2);
    T('959296.3500720181', '710722750842762499299.512700390357811', '710722750842763458595.862772408457811', 93, 4);
    T('3886.865368726171560940103121495357834', '7454175089405.45019816739873123201', '7454175093292.315566893570292172113121495357834', 70, 1);
    T('3733882601505454472987179576997910.7870730016371', '83526800227071641.8150', '3733882601505454556513980000000000', 25, 2);
    T('307889899344996919313838971161626.255152945823092802022034288121223054', '40546776746650550257867.90', '307889899385543696060489521419494.155152945823092802022034288121223054', 91, 3);
    T('8027111358.2195219622422518310774271', '834473461654105407401991809.91611906281', '834473461654105415429103168.13564102505225', 41, 6);
    T('176.0', '393500504357172947115319082551521398493.9272144813269148013662122', '393500504357172947115319082551521398669.9272144813269148013662122', 94, 0);
    T('1701771732868040383466917603042651947042.413267880518966455602270710321299877', '65151007436264177171334.813908395235971755342', '1701771732868040448617925039300000000000', 29, 5);
    T('932372.7255365891611342', '3448.12483452', '935820.8503711091611342', 43, 1);
    T('277885918347233684697951729643844229358.70857157803757810169939712043395292', '3.51897846', '277885918347233684697951729643844229362.22755003803757', 53, 3);
    T('71720805364625712695173943.71325340', '90255.6108505189082043861647572967272', '72000000000000000000000000', 2, 2);
    T('34.51941478127759743171640678887550', '55835038.78705767078', '55830000', 4, 3);
    T('6376503493847757.4412972020553805820', '72036202365543888020700086184.6501058065949807047', '72036202365550000000000000000', 14, 6);
    T('49441657352425005367896328.9230119597287415556538860847', '856895590085562215467065.623776134', '50298552942510567583363394.5467880937287415556538860847', 76, 4);
    T('307072542333249287176634.810012473567887063639732905435', '8411139725227.0', '307072542341660426901861.81001247', 32, 1);
    T('64.178853049121314108437302932659822745', '1944707766614696589991655035.453016783666869891809611343225608152952', '1944707766614697000000000000', 16, 6);
    T('351643920800.7331816985580759609654941626381931747', '7.08732517430', '351643920807.8205068728580759609654941626381931747', 98, 1);
    T('812330678912743498758454250.1555681201176778', '83296585.0', '812330678912743498841750835.1555681201176778', 85, 2);
    T('184315268326057325526.39713922514458969430476606107', '702256219850031536249906965111.7763761038', '702256220034346804575964290639', 30, 0);
    T('5702773692323369179.63421684213977730401733674292448', '40050359141976463138.866298596589', '45753132834299832318.50051543872877730401733674292448', 74, 5);
    T('36.8', '313518019052224115840.6835939223686073344944475420583511848', '313518019052224115877.4835939223686073344944475420583511848', 58, 0);
    T('60002321371.88118010239', '4589284.480499530269966901197343213252618461258', '60006910656.361679632659966901197343213252618461258', 99, 1);
    T('611891833659013885179871564960.31555', '92512655420058726336581349.52861518083916473091189730591829561409', '611984346314433943906208146309.84416518083916473091189730591829561409', 86, 2);
    T('42573809031872261306771.276376659056902087167146590822742200', '25964362188824443252138768609251714.032880401270702722130762724500296916', '25964362188867017061170640000000000', 25, 5);
    T('2059841113864986665991673.555312335907890018472997519607056', '128.9981251', '2059841113864986665991802.55343743590789002', 42, 4);
    T('7861935379090528553241517749112442230257.92811424776', '346340911956065380155063479693403.32590458417', '7861935725431440509306897904175930000000', 33, 0);
    T('7422011581750746227158262213173470311.5815952527633012429117406859787917504', '1533372808926909415047743681528523.0', '7423544954559673136573309956854998834.581595252763301242912', 58, 6);
    T('668313935053428.2793', '98303699029.23775255936045576820651027674656', '668412238752457.51705255936045576820651027674656', 85, 1);
    T('200.8183918268963025322110713997', '96415261071105833271980038684.0254081925954489391243701357', '96415261071105833271980038884.8438000194', 39, 1);
    T('697341412914370454968716023395983474313.85104937', '58309788699424088435975046097.330931610307873100837', '697341412972680243668140111800000000000', 28, 1);
    T('49117473396974294699.862651193338897355276258588', '502626251201680568705890122621444352075.727558468615089231170', '502626251201680568755007596018418646775.5902096619539', 52, 1);
    T('97061366808828286829078026047339947.1795', '7490945491726779705523547.77498271854', '97000000000000000000000000000000000', 2, 5);
    T('8169850653296576384850966164536835451866.558941806509684', '828768952430304846.17539451386612678373421884036458264', '8169850653296576384851794933000000000000', 28, 3);
    T('443081381566376336683267557060818572939.6982862867141481', '3858149090735355.18979864656431', '443081381566376336683271416000000000000', 27, 2);
    T('57884438004917769571.095860579614316064201501', '194803381586948433163.98982625288132420618018', '252687819591866202735.085686832495640270381681', 91, 0);
    T('6150634169.312010184', '24974488450565777441952335918369733.821426906479397223160313518431570360782', '24974488450565777441952342069004000', 32, 5);
    T('1413955589908887599205043059589109508.84706112278699877892303750428386', '78573403817774861046751393875.39208869917842769313221189974', '1413955668482291416979904106340503384.239149821965426473', 55, 2);
    T('9375667233321276269228617291.104169528862734025351334634', '16105055158518963176078967362152924464.966148072949078648388090', '16105055167894630409400243631381541756.1', 39, 0);
    T('68416967080444521773.482728400468291592908862191097', '10090036621920464884135357430.8453310920700921876419339261', '10090036690337431964579879204.328059492538383780550796117197', 69, 3);
    T('9356789751202187.609722248321434964673000791503801', '53718227690889388362781834449874288.558236072', '53718227690890000000000000000000000', 13, 2);
    T('961844223.938931350989695029541372964889888', '4790297535075119818744877443401974768675.191713', '4790297535075119818744877443402936612899.13064435098969502954137296488989', 72, 5);
    T('3025425281.354909873091457837873199365437', '60541075619207714483100.3982', '60541075619210739908381.75311', 28, 6);
    T('17640875536303013156021986166722.842612378423067', '722088249081534730869373718827331209289.497758952508', '722088266722410268000000000000000000000', 18, 0);
    T('5.910509295339225198', '24526970270773999210313285553335659202.6957451087012514352864910', '24526970270773999210313285553335659208.60625', 43, 4);
    T('738226516027055958109059941.0897396802266', '5294726420237766783.979317676888049695941319070407570739', '738226521321782378346826725.069057357114649695941319070407570739', 97, 4);
    T('146593029314912556847874241855.291536', '237432494387.7071998118838270663336108340647676564', '146593029314912557085306736242.998735811883827066333610834064', 60, 1);
    T('576522076738.1945280981596896380935', '8.0184623365460200', '500000000000', 1, 3);

    T('3647215103143672.5602042551', '715293100019619237892753371358058313.670639521278580267484457463', '715293100019619237896400586461201986.230843776378580267484', 57, 4);
    T('5845545628325764.97148685704596049280180405659162', '91228404751683129305175.77468412005523036806635233263796774206', '91228410597229000000000', 14, 4);
    T('96192310173220985704391361402103763.459130', '11669812943137608310.5450893228351440798273874162961952', '96192310173220997374204304539712074.00421932283514408', 52, 5);
    T('96252394435159119.0276420397387270625430147302', '752473121833738055947127517258042468.28910395125736', '752473121833738056000000000000000000', 18, 3);
    T('263160041511839372095916310040094279.659516837482309', '7542604335506623080826133366694008234.474088975383327316197793593779664961257', '7805764377018462452922049676734102514.1336', 41, 6);
    T('832226099823651687376173963370141.183465310472246425904233279339900050207', '579635328444130120285354568545.694330888361021225505', '832805735152095817496500000000000', 22, 4);
    T('5734868187297656390264828201900413578.10517471554759732762829', '1351449679660806696871290500.7651057886565395676564630588928444122', '6000000000000000000000000000000000000', 1, 6);
    T('79675320820087943413076101383774.228660985087284452029714', '46980901079147758178301.45335625707877928110031', '79675320867068844492223859562100', 30, 4);
    T('11218063058838424962.9258', '47996126144.9551831660793094267361', '11218063106834551107.8809831660793094267361', 79, 0);
    T('3010940487217476996116478803478.359742728615764130984144638374', '2.08680399625665179732421794036818315', '3010940487217476996116478803480.45', 33, 5);
    T('22035368380854365947756835265720674065.9805516403', '4937.4209971386530553288168', '22035368380854365947756835265720679003.4015487789530553288168', 94, 6);
    T('46639417708063789369078.230963011424662701', '28361.52', '46639417708063789397439.750963011424662701', 51, 3);
    T('1738879033413234379.08864595974067618897453917119989', '59.06469084433569599', '1738879033413234438.15333680407637217897453917119989', 91, 6);
    T('656945028624289134061882978731.04726025566183850118833615425330790733', '78704524318184.29581043779204451410471600817685622914', '656945028624289212766407296915.34307069345388301529305216243016413647', 93, 0);
    T('1667034.725683271870623527678978221195634189', '13490135.08693', '15157169', 8, 1);
    T('5145789.737145762504553202092', '21.022166020604758313206609', '5145810.759311783109311515298609', 42, 0);
    T('8762259385340082259588951203028884.2741375583647529237347156384529235206', '7499234034908.726500828922726269810023757331731140', '8762259385340082259596450437063793.0006383872874791935447', 56, 6);
    T('13174239612962341132973101688.74705383891097096742039077', '776874297157551965970890249533589.4127', '777000000000000000000000000000000', 3, 0);
    T('79515672042063643304007655443314659766.433678227021165647349905110498338666837', '48765019192.5399837263830982422107012192225832486', '79515672043000000000000000000000000000', 11, 2);
    T('63.7797038817540574835416352768410929772898', '2049486497284291616860021646706322149749.0140106512955809655591001869', '2049486497284291616860021646706322149812.793714533049638449100735463741', 70, 1);
    T('44098397346249328459408222415079868.01222817', '7861629431226060671.5553261609502848851', '44098397346249336321037653641140539.5675543309502848851', 81, 2);
    T('0.372956706078089243825319077074486152787', '39208157235379832274952.4336590436160194097801120438512419', '39208157235379832274952.806615749694108653605431120925728052787', 64, 3);
    T('88144607438106319748260150951498484.305656766', '95691789767747860450331256746556174.1764414565430', '183836397205854180198591407699000000', 30, 2);
    T('83827221440079320771994649223847.2634608605185259051305', '3750808116820664048777.384441400615924552623668133659613', '83827221443830128888815000000000', 23, 1);
    T('38533245756.2378061107714348614919812012', '3349209699616839933777.5646155796578733195638782', '3349209699655373179533.8024216904293081810558594012', 55, 3);
    T('588121713456330558881.77287394427525', '562809282496582280880828033639949773944.106135042784136692908172829634743813109', '562809282496582281468949747096280332825.879008987059386692908172829634743813109', 79, 6);
    T('3889691475478261060200239995083.6318411', '9345922667.487846082100214139459676108415259208', '3889691475478261060209585917751.119687182100214139459676108415259208', 78, 0);
    T('613174934212284486707720726188240879195.34503498095299903603', '56767206990169988979110523928.70984603892219', '613174900000000000000000000000000000000', 7, 6);
    T('4245309314584159649958283990565135896.9754207510900152305746077950051404210', '69153181204917637688082572916539353883.13', '73398490519501797338040856907104489780.1054207510900152305746078', 64, 6);
    T('65564174873201630286.132239738861300325128910298261', '837803957844228081305103311608881684564.17327583514156739723', '837803957844228081370667486482083314850.305515574002867722358910299', 66, 2);
    T('15258150.7883712011686', '49547051046437948048376.903826167831835341424009381485237588658', '49547051046437963306527.692197369000435341424009381485237588658', 98, 5);
    T('4997095502810546381271833969.71438701879397975466', '13285.1643547788832048283310501753', '4997095502810546381271847254.8787417976771845829910501753', 71, 2);
    T('1623802467037219040309288.301036351555', '1315323458.329923299141067', '1623802467037220355632746.630959650696067', 87, 4);
    T('499525286340071267.12823937298077699562', '483493864833098.96672855035379152487328821636638408', '500008780204904366.094967923334568520494', 39, 0);
    T('4479030708594439387.26076503878256335888655545445186958', '42110580355.179490250077235030329040', '4479030750705019742.44025528885979838921559545445186958', 56, 3);
    T('414600995963267923979613796608762973.4', '6079553810706705324354441611207513651368.054105', '6079968411702668592278421225004122400000', 35, 4);
    T('886245273473183281.42', '561380768586499753331951624986229.411', '561380768586500639577225098169510.831', 66, 4);
    T('898032495546499.1', '438032117347784120402.589567314805952440330459174997', '438033015380279666901.6895673148059', 34, 1);
    T('19.5937', '292008601199701764739.40087924600890176370190', '292008601199701764758.9945792460089017637019', 63, 1);
    T('62410673089.2932', '668.861758249305003617913940172', '62410673758.1', 12, 3);
    T('52038199143155611737.564978716439194942272565843035104281', '587499708041012698970042830233056531.954891', '587499708041012751008241973388668269.519869716439194942273', 57, 6);
    T('60138173329162940.361553039074393', '74258.0149762175028673200265728479', '60138173329237198.3765292565772603200265728479', 77, 5);
    T('3260213395002231492343143916.26969656534389435005657200370', '1161376.29', '3260213395002231492344305292.56', 31, 4);
    T('86972594678519606725424512902408963565.26', '48226568827965036932249735600.64741395704974058636374260762', '86972594726746175560000000000000000000', 19, 0);
    T('21986456750124773795.83', '680699892654344.016155154479961956985862887888821580', '21987137450017428130', 19, 3);
    T('637254319052123037493340512030897598.984625845815058', '845108997974306264893628415834200928.592328599224600424235863280', '1482363317026429302386968900000000000', 26, 6);
    T('944514002.0', '2854448378161134.6481403137696957494777', '2854449322675136.64814', 22, 5);
    T('2248576241.4134182928545891301901414840369253', '8855925.7239697', '2257432167.1373879928545891301901414840369253', 77, 2);
    T('342213798.49', '75266520152942023639670014322247.51769718125142344448454658370134217', '75266520152942023639670356536046.00769718125142344448454658370134217', 87, 2);
    T('50048618456.3829772154959713538032', '7385008620173482269151268310272.6546672021176005573608592845503575751', '7385008620173482269201316928729.0376444176135719111640592845503575751', 72, 0);
    T('4464272321170381564197826534382583387.18646713870211243716117', '46652950359469046034714339696.4385217700329661994999736', '4464272367823331923666872569096923083.6249889', 44, 5);
    T('6255472203076457095582110125133958.2837619138987', '513933236088067.2019233545213765326', '6255472203076457096096043361222025.4856852684200765326', 58, 2);
    T('7191485756.25351396683973932891273', '1.833586462', '7191485758.08710042883973932891273', 88, 1);
    T('5516268387434380353586072.5001359731615042418895120392634', '240873969.4770160971728767473011860950079470834843', '5516268387434380594460041.97715207033438098919069813', 51, 6);
    T('20414176549763541201.86366936770008763789300516', '6756656744045967888548413047963915.29252', '6750000000000000000000000000000000', 3, 1);
    T('771003253615771669292022007883726317019.2470932022947505165180192508700', '6946583635812492678.478646203809909763', '771003250000000000000000000000000000000', 8, 5);
    T('48552122664959024468113.0', '42140515265727002.70542407', '48552164805474290195115.70542407', 69, 1);
    T('84995.623292309267528366498', '2721830272285506153964355.05180692383158417796125096', '2721830272285506154049350.67509923309911254445925096', 87, 6);
    T('457698743731.25326531915391', '39.51139317723678469703', '457698743770.76465849639069469703', 77, 3);
    T('4232904784855854731907.67697009176403', '7121281256535452188816371.48971908529959746', '7125514161320308043548279.16668917706362746', 79, 4);
    T('654169648530878865900160954905.2256102576462843805487272207', '77753.520634382988439390129', '654000000000000000000000000000', 3, 5);
    T('2473651927.50667', '203634535023391317426.49846156', '203634535025864969354.00513156', 90, 3);
    T('930985867119846273950851896208874.12', '786115088565228399370611397087.743', '931771982208411502350222507605961.863', 58, 2);
    T('32049202895434022017421381.98124910860548957028589037716', '82592317685974997549610068025346971.736033987739792225892995517011', '82592317718024200445044090042768353.8', 36, 0);
    T('40630995493.21348601720114191620439089389991392115', '192847655645238.33863157879480671601450038115', '192888286640731.5521175', 22, 3);
    T('870398255805539124497.96798205420947211080898336897213826', '5351638242192.3919109225684', '870398261157177366690.35989297677787211080898336897213826', 69, 6);
    T('22751582221706530943795083150369725721.805900123607152297', '54284821443279691956.6633493654391179873556', '22751582221706530998079904593649417678.469249489046270284356', 59, 2);
    T('151362546036389077910320.98309448787877960359566579789288374', '504.073873895866759236738303', '151362546036389077910825.05696838374553884033396879789288374', 82, 1);
    T('124594058748076998616.9255182556767725296', '384936.9', '120000000000000000000', 2, 3);
    T('43145024509424430926425111938142911856.208389693425519437402916690824046267610', '0.0349055', '43145024509424430926425111938142911856.24329519', 46, 3);
    T('64015698644191146566957836540719457.864634509773550', '218015646810116026983.525450027309493117085602574', '64015698644191400000000000000000000', 15, 6);
    T('39752890187060107.70479009918', '4592916748.125329222585862808652442631', '39752894779976855.830119321765862808652442631', 84, 2);
    T('94271339258536.8126861', '7445291454791803327331614606060.463', '7446000000000000000000000000000', 4, 2);
    T('188869331142767690533520341696044.79', '729170933661682596797986.369941297703515529', '188869331871938624195202938494031.159941297703515529', 81, 5);
    T('25397150350253595166919774265940720516.540102777927190', '159558420132187708310390.002424356038648334098903627911609811459', '25397150350253754725339906453649030906.5425271339658383340989036', 63, 4);
    T('88599927307.993493990113014108741413484', '319681001107676674.972504373694684085', '319681089707603982.965998363807698193741413484', 97, 0);
    T('1.212', '542712945793408920457983934153234106101.549691052153388528652884691062970', '542712945793408920457983930000000000000', 26, 4);
    T('602250720700.09236722262710685261430875871182775', '586623.648154987637', '610000000000', 2, 2);
    T('8984.3', '647.479417199773309339183121', '9631.779417199773309339183121', 62, 3);
    T('63585915388866.6836452811932760098530799', '5848483983351045.5636028240506', '5912069898739912.2472481052438760098530799', 73, 5);
    T('6330052732.4067868191063696036009982424086745', '937229988062624551428312988908359674.04873302363324438517232850292304024206', '937229988062624551428312995238412406.45552', 42, 6);
    T('3556637080088477382150153325141430.82367168531338078444464485375024477947', '27425120183542961507.8047541465', '3556637080088504807270336868102938.628425831813380784444645', 58, 6);
    T('69039653762181.6903117900748408661536046387813970876', '472376317860741760.624356529261236743429197462443754418888', '472445357514503942.314668319336077609582802101225151506488', 97, 4);
    T('6281000806.333204', '677700866962853726591992.8475194134404959496758748', '680000000000000000000000', 2, 0);
    T('3092780.1', '2517194928328871793156.4636145469708241377376', '2517194928328874885936.5636145469708241378', 41, 2);
    T('9053.03850525683679988567308366615699483', '8048410170156505243787.4398768', '8048410170156505252840.4784', 26, 5);
    T('939293947865859.73546728160982', '2905287680.7457509870381728447768274328770', '939296853153540.481218268647992844776827432877', 94, 0);
    T('49104058387.776036635716313217279705660882478', '420879107648149136.8149159183767', '420879156752207524.590952554093013217279705660882478', 76, 6);
    T('5488291109444920713.72611766973545445663495254320067250', '525838564772264114094741.183822', '500000000000000000000000', 1, 3);
    T('269551982997851607357913.51935', '5863.4', '269551982997851607363776.91935', 97, 2);
    T('87899.144591463962496603984143902111019862763', '10132159014222012676798096.536284755481998946881871250771', '10132159014222012676885995.680876219444495550866015152882019862763', 80, 1);
    T('320229002825.638314604241000616', '647397321717437.4902703817527449615607920082910874930', '647717550720263.128584986', 24, 2);
    T('8217732817059617446.2573365935', '81979609565048381972505986760262315993.0877490313221838612877120059', '81979609565048381980723719577321933439.3450856248221838612877120059', 98, 2);
    T('732953.933941316119827890604921127500200171226', '6566144617585350698595317241518196000.24158619135220530534', '6566144617585350698595317241519000000', 31, 4);
    T('726277.486804241014383974783362257', '17332566.5322026583965945817110454', '18058844.01900689941', 20, 1);
    T('63837827763774588.60136872688730007296179891', '618284715463070939888333262.022699779873', '618284715526908767652107850.624068506760300073', 46, 2);
    T('2299008040330596331042295033794057141614.053539829186119601632964796499', '14.34840169658420', '2299008040330596331042295033794057141628.401941525770319601632964796499', 93, 5);
    T('697209524000121112012063.15566329747979632057431210665950530389', '371461318410045426984404676192.2342906899650443435782155032539922500', '371462015620000000000000000000', 11, 5);
    T('3546665083234722.638859102754548219', '958772868276926510.9649182692310298211912078429', '962319533360161233.6037773719855780401912078429', 61, 0);
    T('23061220843399460677784458750594.098', '7765510591867922582415450.9219977430319542263110125809387163526330', '23061228608910052545707041166045.019997743031954226312', 53, 2);
    T('70101402239522876435919.9354334408675267', '389.63984002592992342224583164484771561', '70100000000000000000000', 4, 1);
    T('87046.630559413434137693237463230746620492', '59061054491789363931051428106700246187.690488073578362988319645045', '59061054491789363931051428106700333234.32104748701250068155710828', 64, 4);
    T('616455283055934.870285115286367695646599020355168', '6477.75331148576151', '616455283062412.6236', 19, 2);
    T('2986591244272839104407948514497492.2733990206238304055', '695164034117.361419958', '2986591244272839104408643678531609.6348189786238304055', 92, 5);
    T('83742683610317.614018318633669492990856053051123536', '50584727337901132882231.8699911372075844115685603', '50584727421643816492549.484009455841253904559416353051123536', 93, 0);
    T('6726551232987292460244591397776230330.9549504813', '632316888061134.38933430941217130', '6726551232987292460245223714664291465.3442847907121713', 72, 4);
    T('70965815983774323302804.962811830970109', '5777256655136972422053362845829.529270565148613', '5777256726102788405827686148634.492082396118722', 89, 5);
    T('887063218282215567178316175749543941.05246486802564258711', '106949.95453757691453551103972476', '887063218282215567178316175749650891.00700245', 44, 2);
    T('999452970295599974010122322225482812369.7310746636550', '54176268216143028255545587318613248851.8745688920401969592952606751045987354335', '1053629238511743002265667909544096061221.605643555695196959295260675104598735', 76, 6);
    T('98968051133440247607717224083588993.432', '6236507831609918240839270.344439518660248267663', '98968051139676755439327142324428263.776439518660248267663', 93, 3);
    T('163813229350884143455145761299662.58974179990998567912466447', '7792306952682262896614.6565910', '163813229358676450407828024197000', 30, 0);
    T('9464048098074673261.7', '488.7212852021397319804449188984842', '9464048098074673750.4212852021397319804449188984842', 93, 0);
    T('327128456672287085007746875218949975386.4660317154047200', '90093752171871142.4548260724013654630993121047', '327128456600000000000000000000000000000', 10, 1);
    T('563755313993104647354546.0656871429020915502981208604238418438', '86115065.8383480917216708', '563755313993104733469611.9040352346237623502981208604238418', 58, 3);
    T('35656820940323948325727150165366195275.932047733878127', '49324734791948947199491422041311939.4477592192481686239655132127072318372', '35706145675115897272926641587407507215.3798069531262956239655132127072318372', 81, 3);
    T('851995003684166257779.82575600431877155331912664416090', '313035499467400.1442606495879080420792672347470885871445', '851995316719665725179.97001665', 29, 6);
    T('7589777085407889043494184.0', '6067269221172461749624305496843103761.65596831490700651352258', '6067269221180051526709713385886597945.65596831490700651352258', 99, 5);
    T('7259541349775478.59166', '92756812518996299652651306247345325.9801507910', '92756812518996299659910847597120804.571810791', 47, 0);
    T('2937582855020239654234109619.73567', '513681587920583216465135740002326.45811639112634025108715', '513684525503438236704789974111946.19378639112634025108715', 64, 3);
    T('2707709592270231.334584888764137986445859596510004550', '68339738670.491809183348368148551850630623069', '2707777932008901.826394072112506135', 35, 0);
    T('898582504.0589857214436212279988056650088717681545', '184.45852063415133478157', '898582688.5175063555949560095688056650088717681545', 59, 4);
    T('765527600366064106915.14172105221101566971', '536606857.40946540658585375122218651085000', '765527600366600713772.55118645879686942093218651085', 98, 2);
    T('511249332748293001895980.93019695', '656787946564204736308221503330.3303694044985147061882642757', '656788457813537484601223399311.2605663544985147061882643', 55, 6);
    T('500.648517551440628035', '9191523843829.57160552823726', '9191523844330.220123079677888', 29, 1);
    T('74228843159.09688304400', '7723055532123482.0', '7723129760966641.096883044', 36, 1);
    T('268377888918435604221349427202264918.28165674106639807803838', '85799552925094777327523270063750407166.9634', '86067930814013212931744619490952672085.24505674107', 49, 0);
    T('61705592583.98742', '55877665872359610323888983258577072.778281931612082186105673629786', '55877665872359610323889044964169657', 35, 6);
    T('80824146894603.34923378828068147539346932971880598', '51881817486.89945774', '80876028712090.24869152828068147539346932971880598', 50, 0);
    T('43459120905980000027653546063.713345245', '78748435466013976930341545981.887686090911547843296633883', '122207556371993976957995092045.601031335911547843296633883', 84, 2);
    T('6744793851.594922787', '5506057867997387.03085057277634381121126777034', '5506064612791238.62577335977634381121126777034', 55, 0);
    T('5.5738799597537846', '7215047084.46693046738451771447845509956234471650', '7215047090.0408104271383023144784550995623447165', 56, 5);
    T('86043.9364211629673840010175238244099961517287', '12547173446527323.554984', '12547173446613367.4914051629673840010175238244099961517287', 89, 4);
    T('5702730050506.219942005728499151', '36540595532610873770.16009768319052', '36540601235340924276.380039688919019151', 61, 4);
    T('7440863.8690020263262300910', '16793763381155008785.16913704918794416985498162079740', '16793763381162449649.0381390755141742608549816207974', 71, 4);
    T('50.4290178700067175', '2.7666159328396564', '53.1956338028463739', 27, 6);
    T('27.73785559855833534', '16032353571475648798835552342548952.1247856156226363983177739364', '16032353571000000000000000000000000', 11, 1);
    T('9206899610496380171343926307816648773.309925483829', '6.0611249184022309321864220457691996569', '9206899610496380171343926307816648779.37105040223123093218642204576919966', 72, 2);
    T('92316372850343797298.05381632190201', '3166568052499.8701082632', '92316376016911849797.92392458510201', 69, 0);
    T('716457326.3323343409758463893905', '1985965745642654587111362955944651581.443490076156646', '1985965745642654587111362956661108907.775824417132492389391', 58, 2);
    T('99270841692.0547043697657803567147925565428011892739', '96719494714265659433017056020578.329930019050686018613542', '96719494714265659433116326862270.3846343888164664', 48, 2);
    T('58932561.13202078256294164972094870348', '6642748941761637448881.379632864676790132132354701464901671545', '6642748941761696381442.511654', 28, 2);
    T('84094158033497471695.66890927871070356271841666506', '27371956962859997686143673861552.724', '27371956962944091844177171333248.39290927871070356271841666506', 90, 0);
    T('2976532736168954886147194000464785018242.9506313472084265269253582618626265', '1054789438587261208915.07087788455900', '2976532736168954900000000000000000000000', 17, 2);
    T('8242735365813344139548235622122451602273.77360748892397792265325', '3980607597956281180699496195884840145.143443879585189', '8246715973411300420728935118318330000000', 33, 3);
    T('310541704878847704.01490', '336966740862818662283080320862229.1306794530', '336966740862818972824785199709933.145579453', 51, 6);
    T('1077866063848607155652.2764029368325220332935095237275770827', '25469331297773510591249054831036113648.34360785', '25469331297773511669115118679644000000', 32, 2);
    T('3760917687479569496503832277928164689290.52481924450935020776281216303', '9233578921540301.5984577574424371512', '3760917687479569496503841511507086229592.12327700195', 51, 4);
    T('1420826854.7425514092198565749718784541982677', '9258019.27661790802757310522287', '1430084874.0191693172474296801947484541982677', 84, 0);
    T('2176842853194.32614986914809577953709891731873995727', '27487501439048799150924462181.2163321724503524921714142663589670567367', '27487501439048801327767315375.5424820415984482717085131836777070140067', 89, 6);
    T('7719889368901.937', '122427557472529695293.761677134953140869418512141579601', '122427565192419064195.6986771349531408694185', 43, 4);
    T('846212907.8131430766910095747906311847634', '42920549001527076823747831135.18969026640883261117497311995898950', '42920549001527076824594044043', 29, 5);
    T('80822276203506933530642543.823886990472889857093303', '720832469154441740217.2226334173288705', '80823000000000000000000000', 7, 5);
    T('23201629746555239926578495030705273911.103494523190173189657997227500236', '269.89044071175491', '23202000000000000000000000000000000000', 5, 5);
    T('85.098761732322641344366', '41370815588228965229076498.72693513382092306790786384', '41370815588228965229076583.82569686614356441227386384', 81, 5);
    T('351417625480076602.35221792475981322806212084038133586', '797.7475604725335852614609557327125646086326', '351417625480077400.099778397293398489523076573094', 49, 0);
    T('57114691777415011528384274150.99901076776553360', '9434891112657564348901634315500330942.8484390', '9434891169772256126317000000000000000', 22, 5);
    T('3340661685959446463320323821633906225777.5609571427518617008', '8686896011216894408277348933661445221.925588566142298097309027319866693818', '3349348581970663357728601170567567670999.486545708894159798109', 62, 6);
    T('3128723412827753264586407936450569892.559695622540116571031246460722079215490', '642145550863447815569540.3334722437434204276625275669486962753', '3128723412828395410137271384266139432.8932', 41, 6);
    T('644362329836015428305473935858295.6357', '93316175708565128838727453095.993725916038340046', '644455646011723993434312663311391.629425916038340046', 79, 6);
    T('26.703862319555624060', '820307218214029294043597409639171019.6245927448733593301380961560332293592624', '820307218214029294043597409639171046.32845506442898339014', 56, 6);
    T('707353387112793834887518243.17075861588106453300253402772627', '4044691675795970775454552.93228912342', '711398078788589805662972796.10304773930106453300253402772627', 92, 2);
    T('7317644874642.51526261', '630272474244327078125892240.177710851412858', '630272474244400000000000000', 13, 0);
    T('7306601603678750014351875723789488353.20135064454990680102', '1795088438.5344163241100', '7306601603678750014351875725584576791.73576696865990680102', 89, 0);
    T('4172776310038442744543024.63428402188899', '93258477784.4', '4172776310038536003020809.03428402188899', 52, 0);
    T('184629401276367263.0838641397374411', '50779.41518994637878171306962797872499504672', '184629401276418042.49905408611622281306962797872499504672', 93, 1);
    T('911683719300867954.94', '113206938473806349601378681916065067530.919438055943525606907863699379', '113206938473806349602290365635365935485.859438055943525606907863699379', 90, 5);
    T('5422669702735088847.402470363155527243748875640', '8019702384822752.3700872535514452931128945', '5430689405119911599.77255761670697253686177014', 81, 2);
    T('44238161904963709961615005057014129.181', '350279514679.027957726697337191678621661577922074463', '44238000000000000000000000000000000', 5, 6);
    T('2503.453783', '667406349261.2524445614780792013522253856533868281', '667406351764.7062275614780792013522', 34, 4);
    T('68068868.1087228464682', '393483771138378254865165030346243057688.493259246059044084692840313', '393483771138378254865165030346311126556.60198209252724408469', 59, 6);
    T('57529118208914892152130.33755', '25524467122773842553740227418.451140', '25524524651892051468632379548.78869', 47, 3);
    T('59396267071604626338.0631412655754768126006608570045919090296', '419751468.35895648', '59396267072024377806.4220977456', 30, 0);
    T('2816602411.933856209485711715901636658816834', '28921556270057062324090205615122228474.3', '28921556270057062324090200000000000000', 24, 4);
    T('3497280.0090444685233', '62.47514078759186439914', '3497342.48418525611516439914', 98, 1);
    T('5690222923491800137510308838.4045384987019811796558029593161', '453044763786645562139.301', '5690223376536564000000000000', 16, 6);
    T('947234524517901032260029064.4029293396037290406102825851594', '3.37', '947234524517901032260029067.7729293396037290406102825851594', 96, 4);
    T('17.1911208960726888986594829', '53207956233.50313468718415996384', '53207956250.69426', 16, 4);
    T('3711872492173263201814207315969.331838170908760672759673848286', '94175734286337210718079537484953604.22062797598954402982499454864', '94179446158829383981281351692269573.552466146898304702584668396926', 100, 4);
    T('483613386220106500706930362.603412620401738344477871621841', '5176820920489438337.431378466106108783765431723304831', '483613391396927000000000000', 15, 1);
    T('99.5427164690668673', '80840009875220767848269198189083558539.698811594796923163', '80840009875220767848000000000000000000', 20, 6);
    T('595694280271615822378921496.0', '10576573305053656.504074837731125358', '595694280282192395683975152.5040748377311', 40, 4);
    T('2903320.91750504206604462379779496372710849960', '63488695643619.6507358255117241623136208949310', '63488698546940.5682408675777687861114158586581085', 49, 2);
    T('801816.874433117305374678029705810057949', '39179411772213.532298865539', '39179412574030.40673198284437467803', 34, 5);
    T('43943076765418942277.700236647', '8.27262432150', '43943076765418942285.9728609685', 88, 4);
    T('31339693998637200535065.0781', '7.925270508496444910186913896290573', '31339693998637200535073.003370508496444910186913896290573', 68, 4);
    T('87704577609034169095520481070.168795342621443', '99746552786595290320309709.954468392565', '87804324161820764385840790780.123263735186443', 79, 0);
    T('4244720215113472777.0929', '13458512347007571564848242.78384585663165', '13458516591727786678321019.87674585663165', 72, 1);
    T('16545941.98500', '45984517344516375807922100521594780.558569290344001022196905', '45984517344516400000000000000000000', 15, 4);
    T('8255242645107422769019230.0', '667140726803511152412180.77997488202509956989403414682197695', '8922383371910933921431410.77997488202509957', 43, 6);
    T('706473348238936.762122', '9914684275028634746934785303.813201878737889395961108414630937503383', '9914684275029341220283024240.575323878737889395961108414630937503383', 74, 0);
    T('18728803792291028.642849674202254746027414', '225597782527385847732705509689471282.90613086384788509570', '225597782527390000000000000000000000', 14, 6);
    T('9144364792861439071343228298427.93370180979163', '79710805967600180.5548111774114012820867941', '9144364792861518782149100000000', 23, 1);
    T('814792295707526422261992966777657441201.3545241744677', '7777440191217767323003593.4214696943466990978505855842651823953', '814792295707534199702184184544980444794.7759938688', 49, 6);
    T('67763907053118402822.2021253819582989270987729044961383969035', '233184978.588748625', '67763907053351587800.7908740069582989270987729044961383969035', 63, 6);
    T('592006478665688.5909484409394819848753110895586611038928', '75709072689677117010822469945612.7331650373062196227744774601', '75709072689677117602828948611301.32411347824570161', 49, 5);
    T('2009.620', '36009790397547552901845990.2389998825059194662366743288419878416', '36009790397547552901847999.858999882505919466236675', 50, 0);
    T('9470742289717496043006496.546089417997775417708197973045159511', '2382477974375609650071605572.3304151095755000406442354991934', '2391948716665327146114612068.876504527573275', 43, 6);
    T('341396.970428932', '159272878579558139684660894593.2758278738336598182386410417063699737801', '159272878579558139684661235990.2462568058336598182386410417063699737801', 76, 2);
    T('886732594520232247796564747.97862474163142993535839190', '12.74325730861103718315509955038', '886732594520232247796564760.72188205024246711851349145038', 72, 0);
    T('2244183029.0', '1466491775160377850819618148583463.0166542303217682781420600', '1466491775160377850819620392766492.01665423032176827814206', 73, 1);
    T('5459116188528850454765.769878762', '88899005377292793392026474.816316784278834315250024699', '88904464493481322242481240.586195546278834315250024699', 76, 6);
    T('6594530532321844424952.858008981873254602004082953718739', '728746292614029668165063075184.4128813', '728746299208560200486907500137.3', 31, 0);
    T('875620470678995309491335668424.2', '263842122273063.1', '875620470678995573333457941487.3', 34, 3);
    T('11838696.81340902169008012', '62974686874309571956491338417644.77507337', '62974686874309571956491350256341.58848239169008012', 74, 4);
    T('8936381.031873333521857619844361', '401884455201669617715747613343570798171.899019597162707933300887', '401884455201669617715747613343579734552.930892930684565553145248', 85, 4);
    T('238286546353271928171532716767886.5888339071725327378038251', '5.9763105', '238286546353271900000000000000000', 16, 3);
    T('91351357307058082379783347337.71859042119205', '652926732224816888098671089642287356.769925922949217637601', '652926823576174195156753469425634695', 36, 0);
    T('27429040634637857548532462723241276.027438994688692922115827', '938193189319043847035303258.163970973409983020417', '27429041572831046867576309758544534.191409968098675942532827', 88, 0);
    T('124.341465', '74331992261316973256645525264966367844.04938', '74331992261316973256645525264966367968.390845', 94, 3);
    T('23753508898229.835639415065297', '8662025832712142977.604385027011', '8662049600000000000', 8, 4);
    T('5831.0701055104171546918789229427361923', '762394660238695409694389246939599.3917069750347130319', '762394660238695409694389246945430.461812485451867723779', 55, 0);
    T('6686844783837489725638.71769430993631', '110221382373429086376249029613323.354524075933912133', '110221382380115931160086519338962.072218385870222133', 59, 5);
    T('930.96456506567315647', '2.655605467025184668574420868813403431', '933.620170532698341138574420868813403431', 68, 3);
    T('308695323708250340897756.43675760176488639553477161369155', '2.831537779212560592', '308695323708250340900000', 19, 2);
    T('710767344776754987500625667092407571.22973215395902142276', '3426981407291883753899109617026.1465849036841729756276814683119', '710770771758162000000000000000000000', 15, 5);
    T('177542038910467175229067948.78810403443413', '9683.92368832457526719323411014', '177542038910467175229077632.71179235900939719323411014', 98, 5);
    T('9993830840.0037664508810772102340443', '5541058717156795573228647031054891307.9349865835222538053567002044317', '5541058717156795573228647041048722147.9387530344033310155907445044', 65, 1);
    T('9772992455017164415229362.3946418629471921916632218597032', '412391656739975605676282835294136437665.129323214', '412391656739985378668737852458551667027.5239650769471921916632', 61, 3);
    T('40463004519372185701855020893343540988.39287308791', '815380.30526549742239960724965434', '40463004519372190000000000000000000000', 16, 4);
    T('16742595768630991247510312801583255.133604539644114081077359238335', '684099996655956437593650338172386.35061210792606811544', '17426695765286947685103963139755641.48421664757', 46, 3);
    T('6184734014180756037604456004266174672634.03512095256218938314725425049831868522', '747090591467481477496579824.70557066769827562014144414017618394', '6184734014181503130000000000000000000000', 18, 5);
    T('5440723470323322.6674', '37905514899284311101.7242465', '37910955622754634424.3916465', 57, 3);
    T('6031212936743999772649636.8068686972187488167583014739186886926', '950807534582509042.2462024215406439764884368634456330', '6031213887551534355158679.1', 26, 0);
    T('1275883789272591383.2984379236', '6930329265299362145914201413887.671273645348', '6930329265300639000000000000000', 16, 2);
    T('19680593509794594.66322718008243', '9100679514448143696942.7546117190657298299692557533', '9100699195041653491537.4178388991481598299692557533', 92, 2);
    T('30.8314543551285047652974026905056768080', '67951022439905968980821975125973690.5513969941375819689182', '67951022439905968980821975125973721.382852', 41, 0);
    T('77.131709180039651836738803236', '3082997582264630491350713702.489109243835611935058987254308601377', '3082997582300000000000000000', 11, 2);
    T('159927542563697209825868051167470.17544841864570935453064', '3142805682518607662676761040352696361092.778310846', '3142805842446150226373970866200000000000', 29, 3);
    T('63673439685760283397054.8639553681357', '872490629822971918223.336043339611091509900449728288050', '64545930315583255315278.19999870774679150990044972828805', 71, 4);
    T('18629478542949.14709371356015657935749', '820116722944494921613623.57223988814', '820116722963124400156572.71933360170015657935749', 73, 3);
    T('5632992799709897315074108866.29325577312713807', '8238961671167.7988891138582056734172113849661547354', '5632992799709905554035780034.0921448869853437434172113849661547354', 86, 6);
    T('9375254907040.9025034613672595340003902354051189', '43.18957189', '9375254907084.09207535136725', 27, 1);
    T('5442269728649630.4', '6632.9595623959204', '5442269728656263.3595623959204', 29, 3);
    T('10044042861.69435213247722751045974674605209', '18992.3816911412', '10044061854.07604327367722751045974674605209', 93, 6);
    T('910746820226407159410559856084.6385694811826891527303108212493647521784', '68.449261054847600687281480987', '910747000000000000000000000000', 6, 5);
    T('1874877046672113283.2060977892147', '9650715756791952360038899259650.191986246492474548494296701', '9650715756793830000000000000000', 15, 5);
    T('72814882826.0', '8.237', '72814882834.237', 21, 2);
    T('8762346990503284627336303150.22746246934026', '465907057969624.05799693915527235829528915849751210030', '8762346990503750534390000000', 21, 4);
    T('32049640435170140786025415.657122867813963168074439679646', '555040278308506711126172141361715454.0101', '555040278340556351561342282100000000', 28, 3);
    T('438558.32819488542', '842127157707465504474479705694844.996003330159642531617683390', '842127157707465504474479706133403.325', 36, 2);
    T('44379781.469694248995711354601779378', '238733036727124195517030960703.639', '238733036727124195517075340485.108694248995711354601779378', 91, 3);
    T('4430257885816948770682.7192439984535311506245548', '566059253921944425406397718564.3640925524537054515536', '566059258352202311223346489247.0833365509072366021781548', 63, 2);
    T('5731132411955770421.811817517741', '10247377068915628.26047844893048', '5741379789024686050.07229596667148', 100, 2);
    T('1635397579484827168863300.680664745570129714015842', '278022903895749753.55898625141034888785823552', '1635397857507731064613054.239651', 31, 0);
    T('862552464458061686.987744915080643987001230167184', '8152586130702301645.08571898684487941096005417305477', '9015138595160363332.07346390192552339796128434023877', 95, 3);
    T('2470605718330850457747643847857603412453.44', '5965737908529175.655382992510389759625127192947', '2470605718330850457747649813595511941629.095382992510389759625127192947', 81, 0);
    T('341335893635524921107434685182915674.244848813998711218899774965284309567', '8968075106528411.038296375', '341335893635524921116402760289440000', 32, 4);
    T('570085663465564193888661.4739150194486', '9178464942469514.890432157829', '570085672644029136358176.3643471772776', 51, 4);
    T('5815.46495214363618974138452828692943447461', '10204991218650578815784519.8987300893135184938998901099708', '10204991218650000000000000', 13, 3);
    T('3508645833336223475.154291548665417538696', '268438181941047766238191717046648619.44', '268438181941047769746837550382872094.594291548665417538696', 81, 0);
    T('49901837522701236577.4178804667300619070955989156402489', '88037.73531', '49901837522701324615.15319046674', 31, 0);
    T('179.8', '148969357820655615228595096048835747.0987254980442630976981245121713668924221', '148969357820655615228595096048835926.898725498044263097698124512171366892', 72, 3);
    T('187516313006409942615132458327682.196978119134647526', '366346724650462.099', '187516313006409942981479182978144.295978119134647526', 58, 6);
    T('7988812434708271553651196586616129.5047208391720610817004106651422662', '191192896669778.23023', '7988812434708271553842389400000000', 26, 1);
    T('6985875280.65485375051947123367728', '45043153741056.7357', '45050139616337.39055375051947123367728', 58, 4);
    T('351851132944293937439280842.9742739844518413754197730108', '5.4987747627405910615341245', '351851132944293937439280000', 23, 3);
    T('352886156428340754138332998214.41', '830520863.629202893895682266304133', '352886156428340754139163519078.039202893895682266304133', 65, 6);
    T('590336822381830063911938717788831165043.24761258902953413052613893', '820865574217007561261124240643.0663769921', '590336823202695638128946279049955405686.31398958112953413052613893', 71, 5);
    T('168261774416.00461423476203549579141252282735718614', '6482426225905.523036288785317460475677246062', '6650688000321.52765052354735295626708976888936', 45, 2);
    T('68265857.22910327386205296537208297419325637675', '56468268711.44301632359412394061', '56536534568.67211959745617690598208297419326', 43, 2);
    T('380450422136.0', '7694958892357728.3027287', '7695339342779864.3027287', 36, 6);
    T('638194947784138038762765078.99014463768954', '36919029280797.40489032', '638194947784174957792045876.39503495768954', 94, 0);
    T('6739103190238916602013329956580816.2929', '4046.04568', '6739103190238916602013329956500000', 29, 3);
    T('7.604', '124824247770.415865803620858152', '124824247778.019865803620858152', 96, 4);
    T('55090010154.39230063177440', '4026395323102333028.87995785572590078393599901894', '4026395378192343183.27225848750030078393599901894', 65, 1);
    T('277544274389134.35761310251709692', '55094617854573.231796024234187759479435854130593076060', '332638892243707.5894092', 22, 0);
    T('64055953785594076651543.508694209641954923940232530659', '15739918077527252901.98170', '64071693703671603904445.490394209641954923940232530659', 55, 1);
    T('936017678322008.1577196356206903', '78946721800574.461175198', '1014964400122582.6188948336206903', 100, 0);
    T('6093087576808.87175532', '237690.70890222441381924155467572841', '6093087814499.58065754441381924155467572841', 97, 2);
    T('4559.389', '2.968739953547185024121893240567767800', '4562.3577399535471850241218932405677678', 81, 1);
    T('56.222635', '356534459995.0665', '356534460051.289135', 80, 0);
    T('132.521800595431759894125032117', '8328234533651766877784144558067250.139952832656990066241783405905848', '8328234533651766877784144558067382.661753428088749960366815522905848', 100, 4);
    T('6668386010016997148662460933.956319968', '29363162.2584799242', '6668386010016997148691824096.2', 29, 1);
    T('5830603857064.3130639523644710207908', '32929625062758055963429139794045913.8526363068150282', '32929625062758055963434970397902978.1657002591794992207908', 64, 5);
    T('982.679327994372', '4174717059208.29157193439286223076876126', '4174717060190.97089992876486223076876126', 77, 0);
    T('9690386074192104258000016491414869035.7837171472779090862752', '86173558623168658.443412178781128758291450917', '9690386074192104258086190050038037694.227129326059037844566650917', 78, 2);
    T('181617530.38792110807111221211', '53962615562130870536660.16282910715318632931979974335', '53962615562131052154190.55075022', 31, 5);
    T('33571568146084533361420177.445498509', '54032686848.11802192245068281445195844627616937', '33571568146100000000000000', 12, 2);
    T('89068272875867.844459814917883822', '61424243179618780700086361576037.290578030314560013974027710624970199', '61424243179618780789154634451905.135037845232443835974027710624970199', 89, 3);
    T('358640571260045807013482199773.756608678742', '6657856136735511439589791868212647430.38022520598707664756402', '6657856495376082699635598881694847210', 36, 0);
    T('56055429546124058555226.011280854', '922437757535249046601160.681158982', '978493187081373105156386.692439836', 43, 4);
    T('8924437.408604702341307339567', '184301345897810.451661633911', '184301354822247.8602663', 22, 4);
    T('58535731427.116660758834469711978157285004330042651', '73470588238468.9919848943856432375795', '73529123969896.1086456532201', 27, 1);
    T('61434099219371.1001246268652', '206679842201074883502276286316281.050110788570722876', '206679842201074883563710385535652.150235415435922876', 83, 4);
    T('701984786903205359341023126662151697435.5523139057537980071129803', '985643497320564681713520265457.250830', '702000000000000000000000000000000000000', 4, 6);
    T('897523486296906.269764713', '13717882860963838.991305146668369098981143', '14615406347260745.261069859668369098981143', 84, 2);
    T('733852673.76676', '145738765255786430779512098245878.90000567199991361101999', '145738765255786430779512832098552.66676567199991361101999', 82, 5);
    T('724453979839639.83740877345242866006', '255214813020792427363841.566944555724318986', '200000000000000000000000', 1, 1);
    T('9184190602.020353', '7967846623169053792.9', '7967846632353244394.920353', 91, 3);
    T('7015612.7429470956532824', '712597520604008437365926852270161.31772793137426', '712597520604008437365926859285774.0606750270275424', 56, 1);
    T('815665548804727714.9844421151991639322952195', '9338807081613806094932455955814430047.328094713051127389380757565400579', '9338807081613806095748121504619157762.312536828250291321675977065401', 67, 4);
    T('21103273670198578.1992504702954382', '27806599584553696280277383.67233112493', '27806599605656969950475961.8715815952254382', 44, 3);
    T('535219538358851152044480758366311264126.9306271747635087259016540', '96253006387862036511822740180237036.00419563822771338358738870603', '535315791365239014081000000000000000000', 21, 4);
    T('4757.338244', '38270366.07553903992387078294644526954670108691', '38275123.41378303992387078294644526954', 37, 1);
    T('6240652295280070733492238978617196.6364', '4830201121595708611423376805556625.85588213502', '11070853416875779344915616000000000', 26, 5);
    T('7290.29622609478', '30602.35367797031', '37892.64990406509', 20, 4);
    T('76965893795168659219779182.11025009991478972134', '38321448804102996908488361741348684.83322183770734858225453742084697190', '38321448881068890703657020961127866.9434719376221383035945374208469719', 74, 1);
    T('818897043.6765310300854762733032312', '857871618693977.250139823354346344616271441', '857872437591020.9266708534398226179', 34, 6);
    T('6468968083585802935749353405411728851313.0', '602530384315690734375331.25276195', '6468968083585803538279700000000000000000', 23, 5);
    T('933245616.10047305702312161083050364577385171421', '77601953779591050510393912.7', '77601953779591051443639528.80047305702312161083050364577385171421', 95, 2);
    T('41636.542816789232495371', '332071854821416440606907078651819076.259495082425918399216433871952094', '332071854821416440606907078651860712.8023118716584137702164339', 61, 6);
    T('6539781959612503931545683604216139625020.937746228220906', '434245464289588965227710681213177542.651635622739460304398305840452849129786', '6540000000000000000000000000000000000000', 4, 3);
    T('983928148035285274873229404369164374.713957589934844282974506541034346', '265.00815380372', '983928148035285274873229404369164639.7221113936548443', 52, 2);
    T('287474804675091771279922.0469727750240361', '4343373.25254264727950390235127093', '287474804675091775623295.29951542230354000235127093', 69, 0);
    T('331652563152899053159298113112432.0113998961427', '5003.8218540261', '331652563152899053159298113117435.8332539222427', 67, 6);
    T('4302342009311968739659496353.262', '27340538821.29191894325', '4302342009311968767000035174.55391894325', 52, 6);
    T('370698426973984163716425328.0', '913715645410.685949465686927208929', '370698426973985077432070738.685949465686927208929', 80, 6);
    T('443243276063.262401442424614', '1124868311601427.149349923704630834419782549966841', '1125311554877490.411751366129244834419782549966841', 54, 2);
    T('957729251830610385146754657478.0056', '80564301.451604866174413828808044494861900491', '957729251830610385146835221779.457204866174', 42, 3);
    T('8034430533956126696417991632532188.84796608080', '788598104770709879209193926951.818829215925474122712256810361', '8035219132060897406297200826459140.666795296725474122712256810361', 76, 5);
    T('296823239.5056782047442637556179', '380479729.63668452823892', '677302969.1423627329831837556179', 77, 4);
    T('117728984892825962308497086597311.036401700291674029361424006767436', '210546752095926645794.565004679286101', '117728984893036509060593013243105.601406379577775029361424006767436', 77, 6);
    T('14509.7999382860515', '38036989887707370582013738484340.323187', '38036989887707370582013738498850.1231252860515', 70, 4);
    T('3026.6651151', '278092152851492090395435362595368342.49545261807425765', '278092152851492090395435362595371369.16056771807425765', 97, 6);
    T('911953088432083083895988406.4987707607', '8673565228.9710503562769170564196011460130775782', '911953088432083092569553635.4698211169769170564196011460131', 58, 6);
    T('177383191460963.923406347795257380541144640669', '966537172.976328162265922517457810505838227', '177384157998136.899734510061179897998955146507227', 65, 3);
    T('4246110247798537654745606.8026220674305', '5268045499476.74687093', '4246110000000000000000000', 7, 5);
    T('8443338172.9847377051265207765', '4337249565439300315203159779746498041.769', '4337249565439300315203159788189836214.7537377051265207765', 82, 1);
    T('7974366233430682138668880958738.642811549507879389778878', '79915615158436443701.6594124824834759364201373575718708349', '7974366233000000000000000000000', 10, 3);
    T('44924497190874235771000.4895388219078818887523', '84833454666108387312809275693379165.8760356545027678561877467192406206', '84833454666153311810000100000000000', 24, 6);
    T('925338237893772098672431705.44', '537475560121682874599087603947544.1990464295333227679', '537476485459920768371186276379249.63904642', 41, 3);
    T('20698030.3795520986995357090069946651840387', '251614037805312774353991173797.514795241597681963481900608061640122457', '251614037805312774354011871827.894347340297217672488895273245678822457', 93, 5);
    T('8709305513290688266.7841552487', '6042713709223810808714104184987.2305574627332459219441', '6042713709232520114227394873254.014712712', 40, 2);
    T('52942361075884533446018.45413565', '71.788701238212244508111', '52942361075884533446090.2428369', 30, 2);
    T('329104803873804451415449424869739.98379508497614', '448740548144863506627153992427248175398.051148456', '448740877249667380431605407876673045138.03494354097614', 96, 0);
    T('8185440912.299327876404885685593516482812149', '1472069103051654434.9462644339078211276777700', '1472069111237095347.245592310312706813271286482812149', 86, 0);
    T('12989521242360581159223336804290683013.51', '322378657959845233.7398283123064489321094115295', '12989521240000000000000000000000000000', 10, 1);
    T('8519866958361934726379886037876145.0', '36593232344586085417758.210', '8519866958398527958724472123293903.21', 62, 5);
    T('8923244492765524547671097693998.21836145482941959554210587431471850759', '3191342285145402487070.466629356578367655968', '8923244495956866832816500181068.684991', 37, 5);
    T('86373477115716732673076.84531949', '8576.1369814', '86373477115716732681652.98230089', 87, 5);
    T('673809996422545262142385871758385.14280611', '76911479084.0471963421575425049', '673800000000000000000000000000000', 4, 4);
    T('66056837393124300530179768909.726624678121807359694641603611', '14341673342323881010929564143819920987.027759026451989683385487188172', '14341673408380718404053864673999689896.7543837045738', 52, 4);
    T('38018624605502033801777122345849.87938844868329798', '2680438938411796.75321396649970363685531707', '38019000000000000000000000000000', 5, 2);
    T('6769.988772406', '3.261', '6773.249772406', 52, 3);
    T('814911534593258733042614067164756925720.078257128543245213969608922796759392', '4176371376451640572439564491.541026965233246692448950988971238663', '814911534597435104419100000000000000000', 22, 4);
    T('1866.151296720380134981890', '20540132165599723828817125695.69364824', '20540132160000000000000000000', 10, 1);
    T('661350939.812729542001074688800659638219', '511336763543797386793798.5731882043154955986081486', '511336763543000000000000', 12, 1);
    T('59258.0424852084276844', '3248213979714584120655937.34464368454774', '3248213979714584120715195.4', 26, 6);
    T('13044616367941798003949737.0', '23004574475647496622319.86', '13067620942417000000000000', 14, 1);
    T('7239088376821658438283540265961438639.320754', '7465092924652978935228102.4249318594020080258', '7239088000000000000000000000000000000', 7, 4);
    T('8931746914527588274625745.537809467294819728299739', '2556638459530870337.7586197303375535979912184698866526', '8931749471166047805496083.2964291976323733262909574698866526', 99, 6);
    T('790541542930758.765748611', '11857734931866077200379.110332902454503', '11857735722407620131137.876081513454503', 43, 1);
    T('27837520.7133114557988411141184548185140777972', '49315862832525313.45069992513445387490813938543300', '49310000000000000', 4, 1);
    T('277522044990455319124777593545091039275.386660896461071027', '0.3207', '277522044990455319120000000000000000000', 20, 1);
    T('143005230698564798512488125.344638651517415', '3345205411918085254543923538460310258470.30498793507354743537142521167', '3345205411918228000000000000000000000000', 16, 3);
    T('4325479.93', '590288.7777863360305419475499906048707576', '4915768.7077863360305419475499906048707576', 75, 6);
    T('724124.144341327993348455', '5357537651.7894693', '5358261775.933810627993348455', 84, 2);
    T('76095.593297177761372659745543912', '516827960582049407430701458018715.339765419735616990300695933191110', '516827960582049407430701458094810.93306259749698965004623984519', 62, 1);
    T('97381.673811581021461433485', '3584543401266460668638442775056461654065.35648', '3584543401266460668638442775056461751447.030291581021461433485', 64, 3);
    T('2620.5260719958', '2.6835806776837055282035700', '2623.20965267348370552820357', 61, 6);
    T('63879494474874023432781308574493105849.7042943985223543988706511720853288', '6987103400381588992801469.26090', '63879494474881010536181690163490000000', 31, 4);
    T('978212987592186080151.14293954', '8040532093640208.832662425584674898555034971', '978221028124279720359.9756019655847', 34, 0);
    T('38917085.1190872357101164320489467318415', '94656.375402081532660199147909526278539794', '39011741.494489317242776631196856258120039794', 79, 2);
    T('92531814659379749919692568353.70344980', '184672072118.40695125900362066', '92531814659379750104364640472.11040105900362066', 99, 0);
    T('33195792125328.49498820535697', '81229335930794198.0', '81262531722919526.49498820535697', 56, 5);
    T('611464113428054882867313854276409.921968', '111395103649.77386189', '611464113428054882867425249380059.6958', 37, 5);
    T('3463907774092818055333853966081035.2283708709913715507226306296502', '4108539202102566547864.1665', '3463907774096926594540000000000000', 21, 0);
    T('52316358807665499006595794027054.193146998499955888317', '42086969543.78230507391845134553', '52316358807665499006637880996597.975452072418407233847', 87, 3);
    T('3720914728741981056007488628535851322176.925261', '24115844199435340.188765789370373466695894809598907', '3720914728741981056007512744380050757517.114026789370373466695894809598907', 79, 0);
    T('713277750492883979345651620663611515951.64532275005089629', '5110816190553.1336387005060', '713277750492883979345651625774427706504.77896145055689629', 100, 1);
    T('414048612813004643017.9102561862008260679', '508386331793068.4901412615904890598447080876', '414049121199336436086.400397447792', 33, 2);
    T('19727604099933697371004260906188393.91403520130421185683611974', '79931876427843816544249148256215266.817364147633546868013', '99659480527777513915253409162403660.73139934893775872484911974', 97, 3);
    T('3692120469220962326829.72629625248599775', '884328347486726871663500077363095359260.34751642474385083652', '884328347486726875355620546584057686090.07381267722984858652', 82, 0);
    T('42247.125462827682898926319303700424411717', '93249260452748140843068211570600710.5541428418203057448', '93249260452748140843068211570642957.679605669503204671119303700424411717', 89, 6);
    T('307202279304534002391659543.99971147418729183235352', '918033443317360787232886900444471502230.5557728822504294460566362684736078931', '918033443317667989512191434446863161774.5554843564377212784101562684736078931', 84, 0);
    T('80401104863056831.839260824360055224468890', '795251057545798736006606078231635.58959850451528029887935', '795251057545798816407710941288467.42885932887', 44, 3);
    T('75504373386384167613443439694299.474910', '188984561953651948599418344710.00104744620453368474699504213474152732', '75693357948337819562042858039009.4759574462', 42, 3);
    T('39615968376631262893263813711789959.8495678641', '1.3384459042146906524532335561395651', '39615968376631260000000000000000000', 16, 1);
    T('47104060479842533623.2949524087418768667751628836608424568453', '736719412909871219066282549800519.422624904668', '736719412909918323126762392334142.7175773134098768667751628836608424568453', 99, 4);
    T('82084748747564.646197123429', '85773964.0335863', '82084834521528.679783423429', 57, 2);
    T('93790575893757839830577.081930124882894089067', '20182361499817403629628384992872101356.0016906493707337', '20182361499817497420204278750711931933.083620774253627789067', 99, 2);
    T('16180232463.881536934139804503901000', '4676.0', '16180237139.881536934139804503901', 43, 5);
    T('67382452298219027326795308609103454.08886891', '54.889976873039760', '67382452298219027326795308609103508.97884578303976', 98, 3);
    T('828176206532955528613721756951966908.5793396643753394845997', '301088382.45389034501405323927923174', '828176206532955528613721758000000000', 27, 0);
    T('723705389873106574828035.7440244453980766688604485894822517354088', '2949521098652275158359700353250381942746.6940126160666943421518726181194', '2949521098652275882065090226356956770782.4380370614647710110123212076017', 71, 6);
    T('902325380754847783596.4000253696003898399092253', '710917074716.8504702', '902325381465764858313.2504955696003898399092253', 99, 4);
    T('84133082.071279441350064331400491', '6636072033035298287190637.286782041682543461548737631', '6636072033035298371324000', 22, 6);
    T('4221020274864536713522243189502384.2', '4358383160202787224245788413.136242', '4221024633247696916309467435300000', 29, 2);
    T('4831273.49515377663892544686031629575557275209', '595905833035256577813612.9995028892996286431941', '595905833035256582644886.49465666593855409005441629575557275209', 73, 1);
    T('1789197621315602705149121358451.4290505499241387968124396607283137312', '71263675328680803777664600749969293.813277435275793', '71265464500000000000000000000000000', 9, 3);
    T('6329491373055851893467.49618146', '123112057.9948305557390534175', '6329491373055975005525.4910120157390534175', 48, 1);
    T('82622917550831197632148400991257.00', '86497649406648577368331559572212782.2257', '86580272324199408565963707973204039.2257', 54, 3);
    T('7752460342546429635576.50534519842811464', '14290306804600418670.6690968', '7766750649350000000000', 12, 4);
    T('290799.6149738', '722975153555903622381460436564820765.7562732946734060926', '722975153555903622381460436565111565.3712470946734060926', 65, 3);
    T('319801853786817446.62672452', '254696104709408175883473.506480557575017459', '254696424511261962700920.133205077575017459', 79, 1);
    T('220808098151760758018047628738.266451715887715', '3561982781568736172897.9047875198', '220808101710000000000000000000', 11, 6);
    T('5073.2701874050259122487762325350327677915587', '2120910611076836726208960476559.0291359', '2120910611076836726208960481632.2993233050259122487762325350327677915587', 91, 2);
    T('39163233199806422249444904.081703527100013', '48.51202194', '39163233199806422249444953', 26, 6);
    T('928504293071007165540783514217157131.5466905590', '7195075226910230340488602747.78777673', '928504300266082392451013854705759879.334467289', 53, 1);
    T('77.6402461063557105379', '47877379106204746676133710223002876.87644380931321993049739761937838819', '47877379106204740000000000000000000', 16, 3);
    T('12993622153.736234796257381', '45990535839200445875.5503987464067853583671444266644389845209', '45990535852194068000', 18, 5);
    T('6350555435074941229.7168661506897035580201273347917', '52658391591842602234299.077', '52664742147277677175528.793866150689704', 38, 6);
    T('72686406789356369134811935515.39428356254129222183', '209886542268039052582026464295415089.8454664260555026991766110742070581840', '209886614954445841938395599107350605.23974998859679492100661107420705819', 71, 2);
    T('96435.02407545937154979843786', '96291345434293.25178664938236525120646', '96291345530728.27586210875391504964432', 98, 0);
    T('80030136552318005.2676612004552143301515782072005411680211', '34220644568193073581.478750381728995737689282712295016', '34300674704000000000', 11, 3);
    T('1220443090200484068529.13583450220379543471050069577', '915239873772353652425387956553162111.77644374136784', '915239873772354872868478157037230640.912278243571635435', 54, 4);
    T('825891608115531141957390369131924456.3824425', '788444590833.56175398196438275931771572', '825891608115531141960000000000000000', 20, 5);
    T('2530.71641345218895240291758', '3987814291193843907250235335809850203870.0', '3987814200000000000000000000000000000000', 8, 1);
    T('646516538.69083885', '37428894655109374182689171640785155.6070053435786067424727', '37428894655109374182689172287301694.2978441935786067424727', 87, 5);
    T('147276105.541911414724203363627', '159769762649702110379379155.388528767009082675836781', '159769762649702110526655260.93', 29, 4);
    T('9589232658337239746171415051391773.82110071187247068865736', '1267816857184108831364375940452360.645457120491722786114851', '10857049515521348577535790991844134.466557832364193474772211', 85, 5);
    T('76671150832609068598867633769835065.0982491291939836315012488927813188', '35326650382833799527655.600987', '76671150832644395249250467569362720.6992361291939836315012488927813188', 91, 1);
    T('5650205183716911150748240857663802642793.14513798637814872952070219083071387289', '16712658815537521288457159.1089901390', '5650205183716927000000000000000000000000', 16, 1);
    T('71908164195421094.7961967063351397901865', '878165958.0', '71910000000000000', 4, 4);
    T('3198892394900643722200023250415.966454378', '57.922920484', '3198892394900643722200023250473.889374862', 59, 3);
    T('913284229.779424117967', '3375306605310235728386182418.220819040613859839', '3375306605310235729299466648.000243158580859839', 59, 0);
    T('68083276993775409649738.518917880053883059494405454181036', '4096439.035796534994521723', '68083276993775413746177.5547144150484047824944055', 48, 0);
    T('4924218671621351862051122040830660375.79461823626595102515820', '911973988619507774031422178505575313012.14174839407154889474497589190533', '917000000000000000000000000000000000000', 3, 4);
    T('101203968676718084675.72115097737588432477269', '565682251849885.0041', '101204534358969934000', 18, 1);
    T('651267.05706655155074', '9234358002.83602171049847716145', '9235009269.89308826204', 21, 3);
    T('805043941843651452921249597.6429693898928300494561507905467402', '258657185976.024698114336018753185941152941', '805043941843651711578435573.6676675', 34, 1);
    T('889691342646375943041155043711785923.451449337845218841527244876', '89340298597728214562708752460368245849.29193893983493463427524802406938175', '90229989940374590505749907504080031772.74338827768015347580249290006938175', 84, 3);
    T('140854668973955170387901.123746251353538781', '91014202240296353125954721959868264384.583898330482509531455484003817061522288', '91014202240296493980623695915038652285.7076445818360483124554840038170615', 72, 6);
    T('86.168302931', '60453467480.32', '60453467566.488302931', 63, 2);
    T('1810781746135.81049928', '507673828.83252316363662039554663', '1811289419964.64302245', 21, 2);
    T('51799.841511796', '7198072444895590.3880985148927', '7198072444000000', 10, 1);
    T('352346768522192623619171952638298571479.4601230279236', '4925951923663400818120471681546230402243.454122774161636100955420376170', '5278298692185593441739643634184528973722', 40, 3);
    T('3207907.6229370130660912422653226', '8787231385018572349532701490.5662600', '8787231385018572350000000000', 19, 2);
    T('5137765654578393021121776656714107968.46136', '4456476850543687815063.4884832815549965511830586', '5137765654578397477598627200401923032', 37, 2);
    T('98073024916014256112334.914378058072833791962', '497472654916420502407106305612468959.303462768979', '497472654916518575432022319868581294.217840827051833791962', 64, 2);
    T('34248533290408.4256627171', '689763191142727208662454.683280873667', '689763191176975741952863.108943590767', 76, 6);
    T('8932.78982514', '32495923884210808859395548565.850672780555088598', '32495923884210808859395558000', 26, 0);
    T('985843360422806647562681475.2930942550', '581.960566072520214550892629788528', '985843360422806647562682057.253660327520214550892629788528', 99, 5);
    T('2634.0737903817369', '1030316172553753772574743.910885985283949983692911399187', '1030316172553753772577377.984676367020849983692911399187', 97, 0);
    T('95006936.5389104643346200332816', '8051.2466197662392376253760', '95014987.7855302305738576586576', 67, 2);
    T('51.1784502704', '70490918003417439533773.9496805687230818061230940', '70490000000000000000000', 4, 4);
    T('9924411803633.67', '85519190399713982501030.988779828980163412338517122806429273', '85519190409638394304664.658779828980163412338517122806429273', 63, 3);
    T('9845.570', '1059084553883941553246313690339.7803556484', '1059084553883941553246313700185.3503556484', 100, 0);
    T('680562735446844.476884344455982195226200329272683736', '90105604.33620059613225117', '680562825552448.81308494058823336522620032927', 44, 1);
    T('493021089121800952334467696562.50', '5.3632464868917884', '493021089100000000000000000000', 10, 5);
    T('2043509854873089610455831316905359.554208871791504013271193071', '63142428249457803187978473410273821.5', '65185938104330892798434000000000000', 23, 3);
    T('1347.492800622166258643313204275951671995714', '5.160588492164814391724473774', '1352.65338911433107303503767804995167', 36, 3);
    T('6447922304883487698788985274.0', '5549998815597719.77032563707187184896419205509062222', '6447922304889037697604582993.77032563707187184896419', 51, 6);
    T('54.18', '598316199501171431224959263309984268413.5407532043502988619138719398806042909', '598316199501171431224959263309984268467.72', 42, 3);
    T('33.71627214', '764514.58311988627258306988064713615654470137', '764548.29939202627258306988064713615654470137', 90, 3);
    T('18992257087884047333015383.0', '19416017079.8277108444', '18992257087884066749032462.8277108444', 100, 1);
    T('29441981570716467517032669756608225494.6272226982454577', '849977018829887857303826.982473050587828091402627', '29440000000000000000000000000000000000', 4, 5);
    T('7508039040154066535105638490946331.477376335593781404363600582251', '98715991118574163933642154716.91047688472678945378047520981512', '7508137756145185109269572133101048.3878532203205708581440758', 59, 5);
    T('94714976486947129401168154801832.56921924874093164514374396', '46136096183.2633838', '94714976486947129401214290898015.83260304874093164514374396', 81, 5);
    T('6632538794415211997081168219244286992501.888976404044412499', '3727882265.6794', '6632538794415211997081168219248014874767.568376404', 50, 1);
    T('97266244307557795386722.56766172333525018185', '3927.397430124', '97266244307557795390649.96509185', 31, 4);
    T('57642.37999709822646828193716427884', '9387361012571847427186796976676.19266611817567365147', '9387361012571847427186797034318.57266321640214193340716427884', 93, 6);
    T('12293812916679218100283089995086866233.211', '87249356849258713381349504715.182778550461665205272222026027', '12293813003928574949541803376436370948.393778550461665206', 56, 0);
    T('2325558360973338.7942629240980696893254546411', '802599079051.941', '2326360960052390.7352629240980696893254546411', 72, 4);
    T('22226407508523675.14248023', '913011000972075720133824667.557464257227508361083289617178', '913011000994302127642348342.699944487227508362', 45, 0);
    T('1731197532352887622719.437854861833374373111624111781853835310', '122627659476921994115805359189174881651.81826617121094121692820275', '122627659476922000000000000000000000000', 15, 0);
    T('894402825970559426316965587589.254253961', '274.0195476888524656574187655', '894402825971000000000000000000', 12, 6);
    T('434.697605', '515084.5530867966114114115799170', '515519.250691796611411411579917', 38, 1);
    T('10354642.7388934349430', '4980832229382702677808258344753136.813383711803702004', '4980832229382702677808258355107000', 31, 3);
    T('9127584001294.547835908292729138320112391874', '31802717.723887225447733196075747446459826027', '9127615804012.27172313374046233', 30, 6);
    T('995164107959300006115.3680625240943461063335254', '6650738515827397839381949425407309222613.4753499388', '6650738520000000000000000000000000000000', 9, 5);
    T('2922994888887431102.25494309150515727552', '217329695490743993678567368873412.990', '217329695490746916673456256304515.244943', 39, 1);
    T('5918291583.1382988142', '813821820975761.760451509102783900684277659', '813827739267344.898750323302783900684277659', 83, 3);
    T('60513330597350190367999246.88044898667', '2117477518527149125020971806118550658.225948606098372', '2117500000000000000000000000000000000', 5, 0);
    T('641808450695.93591342378828847992', '8709637605.79760968806518989742217081316471711350', '650518088301.74', 14, 2);
    T('75205338093316503335054650968043623.0', '5999438967832974358.1', '75205338093316509334493618801017981.1', 100, 1);
    T('58713.504818094793840', '319040.333876576386951565828301023', '377753.838694671180791565828301023', 40, 5);
    T('309729.53521', '3833695697917066.8560', '3833695698226796.39121', 49, 6);
    T('73185950267784447772.5916', '63794360.0', '73185950267848242132.5916', 81, 6);
    T('87808677943169225455686346964711519.055108875785991978004292757938', '7742141299026932400399503446892451238557.49034981620945268105518708684739', '7742229107704875569624959133239415950076.54545869199544465905947984478539', 100, 5);
    T('159738404156473969.59236357286718881967308632222074031', '211220145233.274185523', '159738615376619202.86654909586718882', 35, 0);
    T('282972834699426795794894.6947460369270341498513859056945', '92697918143370462490148.4746451358181730547655488866390', '375670752842797258285043.1693911727', 34, 4);
    T('5224035867.00070679530409354922990', '458607811504738646046591329.110737264302006', '458607811504738651270627196.1114440596060995492299', 85, 4);
    T('4140636156826066063754733588925040611.13002887036949180034526536568', '294795131062815543306087395713436.101', '4140930951957128879298039676320754047.23102887036949180034526536568', 67, 1);
    T('605839397770730884889429736215.208974317921502590515978', '72.262335460503', '605839397770730884889429736287.471309778424', 42, 3);
    T('60607772129265384447.80249096375157232457312041907344785', '27190107835956276552808254641567738992.6321435918741735748586341350495', '27190107835956276613416026770833123440.43463456', 46, 2);
    T('669134378591735797957742341063344741.075740236258273', '6996.8043', '669134378591735797957742341063351737.880040236258273', 61, 6);
    T('989060167017275832706999.29738268286977', '8232648816058781427.42117897872183519730255525', '989068399666090000000000', 14, 3);
    T('88688600485952349253760270621845091933.09735681665942251702366254219215999106', '742708178156004.1830337779943592680630268129758', '88688600485952349253761013330023247937.280390594653781785087', 59, 2);
    T('4.232336853944953603215813733808530', '3629362256647.0116468931', '3629362256651.243983747045', 25, 0);
    T('56215747.734', '93090402327333797722370095331336.2412487905922717193085901087947182', '93090402327333797722370151547083.9752487905922717193085901087947182', 95, 6);
    T('15980542898339677723610490912758856636.34717406018531253', '32715.83085828765', '20000000000000000000000000000000000000', 1, 5);
    T('26524190794851346904234044190136684.3191724125507023088996', '3865934820327004619747.9189804417871760181952207094', '26524190794860000000000000000000000', 13, 0);
    T('4035971171807344117463225754231326729711.795460982665930548341269729808035702786', '1377361980645409653656284859292.0860545343720119', '4035971173184706098108635407887611589003.88151551704', 51, 2);
    T('631622321330974141015.6358937924544565645539648556241255916', '728033583251170.05928934357613336572367794231', '631623049364557392185.6951831360305899302776427979341255916', 62, 5);
    T('4.455955048', '407036519376562733083631267535263694.74029224157847311252617', '407036519376562733083700000000000000', 22, 0);
    T('854285361635465558222.0076720225256774307796', '73851005530327416429026697078139721758.84', '73851005530327417280000000000000000000', 19, 3);
    T('6113044111101343057690768234083935478955.141317943810379', '394397555.0969716010924810214580817733102387633670', '6113044111101343057690768234000000000000', 28, 4);
    T('62854368615516917900561064151500135.35316229750', '7509666724590.595534', '62854368615516917900600000000000000', 21, 2);
    T('786975756100129883.50067343125170327495522508974206', '38.541779538073459', '786975756100129922.0424529693251622749552250897', 46, 6);
    T('342731447914.503133973460593869085248435973864886', '448168.3230770819644', '342731896082.826211055424993869085248435973864886', 56, 2);
    T('62351630517523459114625607325951307.616670373990552894', '44998997179125446721885870258743.0', '62396629514702584561347493196000000', 29, 3);
    T('29698080964801036827220.93471492888311968658257029', '91063.02052138', '29698080964801036918283.95523630888311968658257029', 59, 3);
    T('59278939057665.27485655819043308089', '354702252440639.411808182546534715879352', '413981191498304.686664740736967796769352', 54, 0);
    T('5213167062885171288247238.98311161810', '6330129641538054186834108579119565458.61352497', '6330129641543267353896993750407812697.5966365881', 47, 6);
    T('865205240284.57887337', '14447951582062397831835313971834651113.63816', '14447951582062397831835314837039891398.21703337', 84, 4);
    T('59196.6988967', '6902977323670404243431125130.8174721636232421724014509627175334090854', '6902977323670404243431184327.5163688636232421724014509627175', 59, 5);
    T('66416207719815383460684630811606.6896240782058', '38000385932206632583061369.666415111081410973556830236600', '66416245720201315667317213000000', 26, 3);
    T('7897954760.3654091191576086701899', '941920499413.845', '949818454174.2104091191576086701899', 54, 2);
    T('712455.4380260844281563', '27385508637158203.42334533', '27390000000000000', 4, 6);
    T('2269966053.90287866597798', '1986033.29666887812004896126588383', '2271950000', 6, 4);
    T('882861700939.20528877', '45870856534751451821987190615262582.0196964', '45870856534751451821988073476963521.22498517', 45, 0);
    T('707706570148021045768330550.07491541782686055148', '9781578343034624166649312549.510136931899608206', '10489284913182645212417643099.58505234972646875748', 58, 4);
    T('26931.799925693062348114', '2908.592748669063913', '29840.392674362126261114', 67, 4);
    T('3525903305290.390573450179', '8923.95016627592105431256403655700076710638', '3525903314210', 12, 1);
    T('658552999113781822.537941185325806265842416141', '87393.2934429918', '658552999113869215.831384177125806265842416141', 54, 1);
    T('5406432102591706467638.58256559619479961296596034', '454442585661602723400525.0657178', '459849017765000000000000', 12, 0);
    T('87559989230721766028.993995383061', '84565620916.198675974785', '87559989315287386945.192671357846', 40, 0);
    T('779.653102862770362', '2776297.298593440184557716217416271412864041', '2777076.951696302954919716217416271412864041', 89, 3);
    T('4965185096418440241.909633', '234053854.9932718', '4965185096652494096.9029048', 94, 1);
    T('252656363333415.5909751126', '978774491975.344597777881120316215105293', '253635137825390.935572890481120316215105293', 50, 4);
    T('95945701274619610554154553276237660.83926370597178', '147702764554167979168068557153.0179115049928647488944425553', '95945848977400000000000000000000000', 12, 4);
    T('34930053264069713696166228108950807.3896095360152647649583643502', '3207930374569000258958644379.0143680670354398222604713211462', '34930056472000088265166487067595186.4039776030507045872188356713462', 71, 3);
    T('62066565348123283.1530127928323', '6900154673958.35591227161298804080340658', '62073465502797241.50892506444528804080340658', 73, 6);
    T('9352282.96083029799520289362858608000343158535', '99.31533809958676871933042', '9352382.276168397581971612959006080003431585', 43, 5);
    T('7241438844767945191172386323913104509778.0', '196934183.6187230410142457157190802874764808053466', '7241438844767945191172386323913301443961.61872304', 48, 5);
    T('82301127210944126445056048786265552818.76856', '547576.62143476192645873383839556031589998702', '82301127210944126445056048786266100395.3899947619265', 51, 2);
    T('9049130601351184653862305896063714.9888673572119', '361932099333822215368.9', '9049130601351546585961639718279083.8888673572119', 69, 5);
    T('449587917.6', '249791885546915068322173962811770.1597021065183497054069128', '249791885540000000000000000000000', 11, 1);
    T('851755649576.2222344149187236048398', '7664643.7042', '851763314219.9264344149187236048398', 77, 1);
    T('29703516305262250586133275139.8273215258718903760974192', '2795.8345644965427', '29703516305262250586133277935.6618860224145903760974192', 71, 5);
    T('93380761653051135180584729884306.994918830520', '31738491221318024782418.122', '93380761684789626401902754666725.11692', 37, 6);
    T('438612690776599796.007592131095735806038582732349077039140', '829513354963138815026279949674583247134.0626690039229535300752', '829600000000000000000000000000000000000', 4, 2);
    T('6.43454884948436744', '9.06008931404220034190997648176121', '15.49463816352656778190997648176121', 52, 3);
    T('59105033576896426.79291856530742729517913560554841249828', '192972008762087776638147450.0', '192970000000000000000000000', 5, 4);
    T('109754399996687262399028004113.34023', '425794178623906676373924459.08881959', '110180194175311169075401928572.42904959', 47, 5);
    T('2907.95528936789074924050901098812', '25715764313941307902007572953174.9508272124632653969378', '25715764313941307902007572956082.90611658035401463744681098812', 71, 4);
    T('94448208536475004661906251997.99558281031666795631162', '6780063257219391692013211251.5565676162152226', '101228271793694396353919463249.55215042653189055631162', 69, 3);
    T('9773779732182319939097.387547', '982176731939.64959416336646061309', '9773779733164496671037.03714116336646061309', 64, 1);
    T('526991899.9484385315945450120481816', '615766481947811855101879.676655912348166249502282535914', '615766500000000000000000', 7, 5);
    T('85356182507.0945347148289405543587274701333972', '376882583457439.5946139232', '376967939639946.6891486380289405543588', 37, 2);
    T('750.9178140520711916601876028', '3282903050419038302101588370752608.12472033564989241', '3282903050419038302101588370753359.0425343877210840701876028', 74, 1);
    T('418247880285305925.37385169222808979', '44981556955671081653592104.564768461450644492994803236887404654', '44981557373919000000000000', 14, 4);
    T('3989003984047839923161731102839236175.770836735476380193082504', '75535117055176120368470000036.03533086925830142', '4000000000000000000000000000000000000', 1, 4);
    T('49.0595', '16333247471074745075243586087556377129.74923011263139389159802', '16333247471074745075243586087556377178.80873011263139389159802', 68, 0);
    T('8.817', '13980685.9417360358633615572855159', '13980694.759', 11, 0);
    T('8247938308747691683.489040492793004394148099183830931534276', '905703881.07288759738455061009615', '8247938309653395564.5619280901775550042442', 41, 3);
    T('9569044747020332745840266784479527.9278164', '901182924635444624859.344931249658393802399035525399241068921', '9569044747021233928764902229104387.272747649658393802399035525399241068921', 80, 4);
    T('4361787576851081189530146484130574454.268392988573856506401126814771687398', '75247853970600945602991.823566438596603123936766181', '4361787576851156437384117085076177447', 37, 0);
    T('6120446610361662200449993525797806.69624705815119573', '54093.4451687534176412609841341', '6120446610361662200449993525850000', 30, 4);
    T('1040487719270466647672.6419227061941677184581', '370699322903634926159030647245396549.5243857042769481396997', '370699322903635966646749917712044222.1663084104711158582', 55, 6);
    T('81277525027834087851.69821458441448564214392961', '4.29847361107037393221659547503226', '81278000000000000000', 5, 0);
    T('662401469892035845412.6151952810048', '9.136391077604597', '662401469892035845420', 20, 6);
    T('33712991689281926041352646596.707', '13496758057609574737588389777601.041821793296244068797591253', '13530471049298856663629742424197.7488218', 39, 5);
    T('55078555.2112960', '5631590057017516625789406615.33151240496530966473424287', '5631590057017516625844485170.54280840496530966473424287', 76, 2);
    T('5162.38395', '213251485174412863556246961307182104.425410785', '213251485174412863556246961307187266.809360785', 80, 5);
    T('74430397530634950164883.177650636417466488674539657310289292761', '904256.1333918', '74430397530634951069139.3', 24, 6);
    T('8617215475815578487405133760010487147776.3312049867665083990536595646353209452822', '79414859584842014985292.928856098245729353804372688992886', '8617215475815578566819993344852502133069.2601', 44, 2);
    T('9156746987124260612486702670.4571204', '45289199085400235880698566.5466795', '9202036186209660848367401237.0037999', 89, 3);
    T('825069206858518335407974967605597.85604263836425896804815052475343', '99419629884920962521427116.8678379004055447274871176121696595117', '825069306278148220328937489032714.7238805387698036955352681369230895117', 98, 3);
    T('7063586.7', '9532710258555776911134450475135437897360.10', '9532710300000000000000000000000000000000', 8, 4);
    T('8627007686636823.7191046424652232256024529982918290301', '6779927440481508.637836267153933752052764622198267067', '15406935127118332.356940909619156977655217', 41, 3);
    T('79898.2564973507317161657035', '115018211633669245398192423051060775.99356302652429575368904685530089829088', '115018211633669245398192423051140674.25006037725601191939254685530089829088', 78, 6);
    T('2219506920904100805142.675976521847625224694587823326748376348', '100.99173817867054496252287450381', '2219506920904100805243.667714700518170187217462327136748376348', 66, 4);
    T('46909756632008429146411389137996.31281644057798610549103009526', '49126375639323792405977852397.09000826413870674228927911219032706', '46958883007647752938817366990000', 28, 6);
    T('146892911025.5511086414336019373699152420235143390499', '657.936820123822806488371083406020', '146892911600', 10, 3);
    T('3686365731844199445005123.9848733248695123717381997183141582', '286505571888010.98718406967391493833769756447183245', '3686365732130705016893100', 23, 1);
    T('404815640492591.18415434614460098457440756856971', '906.17347342764787874', '404815640493497.357627773792479724574408', 39, 0);
    T('298028663749854254447163.11811953345007651', '5008383072.551125630587529359', '298028663749859262830235.669245164037605869', 49, 1);
    T('13345601613859675845163039612.961091', '37857936546543639158290399541728358669.4250211965746108927486778115171441343', '37857936559889240772150075386891398282.3861121965746108927486778115171441343', 89, 4);
    T('59536589478066173980275.6', '698378184816303909393711914319689059.05160897106249666237542783768045951717', '698378184816363445983000000000000000', 21, 3);
    T('1226821647502815317053437048509.24', '758676032232709013539133.21794975218769', '1226822406178847549762450587642.45794975218769', 47, 1);
    T('1633999125849670186291.453', '83631596299.850509', '1630000000000000000000', 3, 1);
    T('80904735512.23584703389840644682048321691378615', '860399.41561804810672987008', '80905595911.65146508200513631690048321691378615', 67, 0);
    T('5106.77496995836533514086675493', '466691219784150896076502330648432650.17677742876428706190', '466691219784150896076502330648437756.9517474', 43, 6);
    T('4960796371260638146984489138305273117.058995853368607128613317117049395898682', '42330154653583409635.53072221631374', '4960796371260638189314643791888682752.58971807', 46, 2);
    T('559329392809538772449468.9502337193048327499762', '240018164155612868.4085133718', '559329632827702928062337.3587470911048327499762', 59, 2);
    T('31.8274415337700', '187972277417050579070321.9', '187972277417050579070353.72744153377', 42, 0);
    T('545726594312293268585938461902.36681213397211045', '817157970365994448243093600537493131707.0613398858426683', '817157970911721042555386869123431593609.42815201981477875', 60, 1);
    T('56142965043.9247838604213769076082', '93160401921956.96583248567224164981011185664792195', '93216544887000.89061634609361855741831185664792195', 72, 0);
    T('732275129307061209017571869818376530.2706571242503134218247899', '82898913799639642.24', '732275129307061209100470783618016172.5106571242503134218247899', 63, 5);
    T('289671120596184275088691732043.090817807664276170603824425383777010817', '93342745179522027803445000466.842797513559749691179', '383013865775706302890000000000', 20, 5);
    T('666308024878887434557098408623583823071.83181240230692', '53185247138.1221914887062483992661802399942060817', '666308024878887434557098408676769070209.9540038910131683992661802399942060817', 82, 3);
    T('17925222855321178000547628421.09914383819957415566', '74670909909393.39457902455015773052342', '17925222855321252671457537814.49372286274973188618342', 73, 0);
    T('916742243604213.39174793402409491940', '639337.25194036742774800959', '916742244243550.643688301452', 27, 4);
    T('229843.404852304152924090', '623306.466267', '853149.87111930415292409', 39, 2);
    T('4948658.0', '43185883138852792296490407.171651527968428499811228', '43185883138852792301439065.171651527968428499811228', 89, 5);
    T('898297286009712821813.86824013490636708010427431592', '9111035273576248615725481425322.25', '9111040000000000000000000000000', 6, 6);
    T('782190729835912402553616956048859018101.03255764', '6942.69637106588182878680456931513', '782190729835912402553616956048859025043.72892870588182878680456931513', 99, 5);
    T('3.4446674265644238266025429704', '105162140679046848825014.68562745681331543666', '105162140679046848825018', 24, 5);
    T('1764000509.059156221462740194810450', '346683261378.427266039327268658301280306925725025', '348447261887.486422260790008853112', 33, 4);
    T('20817812419213273234888786727550340160.4703448', '1732735746765.2133946002125585276', '20817812419213273235000000000000000000', 20, 6);
    T('7910140452155340850026071406184877.6', '7111520369305.29078798345578813', '7910140452155340850033182926554182.89078798345578813', 72, 0);

    Decimal.toExpNeg = Decimal.toExpPos = 0;

    T('8.840736845015822532056586640005286181903774282927185103e+5956407', '-4.76303102335411665356659606660554560326223146798605601032e-5086', '8.840736845015822532056586640005286181903774282927185102999999999999999999999999e+5956407', 79, 1);
    T('-9.2832578618177622671613035770423657339818778337530166851969623304548110e+3613737', '-6.144800616e+40307', '-9.2832578618177622671613035770423657339818778337530166851969623304548110000000001e+3613737', 80, 0);
    T('-4.332210548033052268134717549851417790171463618226607547037205479623081894915358777e+1992337', '-12131685860618008307048137853741683980986570972893206277840624629976094364862.45', '-4.332210548033052268134717549851417790171463618227e+1992337', 49, 3);
    T('-9.3265822825203983582e+946656919', '7.03206970684616632987278867692408933539740799e-703936588', '-9.326582282520398358199999999999999999e+946656919', 37, 1);
    T('3.0080355109427504607556911464149394757341721023227255046354597398719707995416743717915671e+8942343', '4.90343490287467844293243696690154e+93940997', '4.9034349028746784429324369669015400000000000000000000000000001e+93940997', 62, 2);
    T('5.594321816855149615563207777001721939745034316089334865136e+255684', '7.4792436265944725885387379798713228991360904115960786306665791e-564184556', '5.59432181685514961556320777700172193974503431608933486513600000000001e+255684', 69, 2);
    T('8.54731007585142130503596209711416237999068839174183546070889227749285428275114e+7374892', '2.55591447331067332339733818555183616136505947711245230539437691682088822398149364227e-5725', '8.5473100758514213050359620971141623799906883917418354607088922775e+7374892', 65, 6);
    T('-2.5463070572761626602611147029687706957429873987413446736816613e+551474292', '5.272300264368228551856579911481767933554209808e-785160548', '-2.546307057276162660261114702968770695742987398741344673681661e+551474292', 61, 4);
    T('8.439567936010479723640505865606432641021964900435195938281e+1531343', '-6.4467142986430551879904347442203e-744941', '8.43956793601047972364050586560644e+1531343', 33, 2);
    T('-4.02567548932176940671167206586427892135942969004052741849368790601647513e-41188', '-7.3069772260866442134030883772897244831272003054861289742071543516324597237293648103679482e+862759', '-7.3069772260866442134030883772897244831272003054e+862759', 47, 2);
    T('1.95155897e-3795296', '-6.090e+50992', '-6.08999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+50992', 99, 2);
    T('4.1494941202497255476128888042148490699521880589827571029417223716e-503', '-6.183656970147802113698500225260084476122859546130339238659390349354958585132e-43094184', '4.1494941202497255476128888042148490699521880589827571029417223716e-503', 79, 5);
    T('4.21515800763013663443714923e-788', '4.19798082911276687593881210146315332275056171126815023555380127693261200091638757894559e-637767', '4.21515800763013663443714923e-788', 68, 5);
    T('6.7635836580e+771', '9.908691912318322924513200928449202e+15959', '9.908691912318322924513200928449202e+15959', 93, 1);
    T('-8.792323234553580131219270925595365999077870960848241831219131097859e+157', '-4989518512416603807653481301760478737914905764793849318110349216974469179629027549744989.169531', '-8.7923232345535801312192709255953659990778709608482418312191310978590004989518e+157', 77, 1);
    T('-3.334777e-35307753', '-1.50493810486015795906903125697583982900726494886889411811795975879774034243615279313e+89', '-1.504938104860157959069031256975839829007264948868894118e+89', 55, 5);
    T('-4.05674564826e-1414573', '8.22792573e-376', '8.22792573e-376', 91, 5);
    T('4.45778061075244972840847153889535224346168943815211070961547583096567053789446116919210368712498580e+454', '-3.465810942555346913656600624237394147473113235578747631538277689597725550685496053664169544077240e-797777', '4.45778061075244972840847153889535224346169e+454', 43, 0);
    T('1.73748572200474488863984668299284325e-956', '-6.6977108422830484524202889931e+842326', '-6.6977108422830484524202889931e+842326', 99, 5);
    T('0.000009298188059736247935523565828241557027785962190397918', '3.6259374639306603039113842e-74775409', '9.29818805973624793552356582824155702778596219039791800000000000000000000000000000000000000000000001e-6', 99, 2);
    T('-6.115294843026494718582160e-63439', '6.00971901260069132403460789162e+407342824', '6.00971901260069132403460789162e+407342824', 92, 4);
    T('3553297354952711849443318560256681257099254538722', '-4.1499022364792698187932687788622079083965506389286e+52', '-4.1495469067437745476083244470061822402708407134747278e+52', 76, 6);
    T('1.110408682432013920e-5062757', '-6.824684268307979512101271130e-836971', '-6.82468426830797951210127113e-836971', 98, 3);
    T('6.007e+3726036', '6.9445227104409733461695619e+16999', '6.007000000000000000000000000000000000000000000000000000000000000001e+3726036', 67, 2);
    T('5.814574245724521569168447032309198277201140460187837732528473224381204425162793872611694224877e+154788159', '-903316863078.0000', '5.8145742457245215691684470323091982772011404602e+154788159', 47, 0);
    T('1.884715493572099e+93', '-7.2406998020074823406540777497635980636414704903534802255e-9263114', '1.884715493572099e+93', 85, 6);
    T('-8.96336e+49736', '-8.73651299024725374215616099402776895620283886756551985443e+713', '-8.96336e+49736', 73, 5);
    T('1.94613372460588438418595824615955778289867448813501774988722979e+987572035', '8.30185982172884536860546295862318451968333295581138319937215345379013251877459e+761', '1.94613372460588438418595824615955778289867448813502e+987572035', 51, 5);
    T('9.82e+6302', '-4.928268142032966503590942338264006102170600052847991623792003968187355921e-7515', '9.8199999999999999999999999999999e+6302', 32, 3);
    T('-6.11138269971929466823684945897168082509034914e-59', '-5.8158454225093833742533668793569186453248899073752105735024394680452891775e+5743', '-5.8158454225093833742533668793569186453248899073752105735024394680452891775000001e+5743', 80, 0);
    T('4.0449314876663263584e+7249752', '8.2647049151683285382582328506e+4742', '4.0449314876663263584000000000001e+7249752', 32, 2);
    T('9.0072691294746313644470937111520816465729505647742702608280751335547534455176903e-873092432', '5.77089842025345429090438e-979461', '5.77089842025345429090438e-979461', 56, 6);
    T('-5.76664283236561061244237642949496374521065115663269744485831577593763242910726e+70027215', '-2.282815961566736891431710113735575615916653171542149594494898e+106', '-5.766642832365610612442376429494963745210651156632697444859e+70027215', 58, 3);
    T('-5.3056927308787753e+6175', '2.78298173240481292233706221e+6172', '-5.30290974914637048707766293779e+6175', 95, 1);
    T('5.2190104738085197934178112610932397666172232094197340729938425275e+6908', '-6.1232749500229542725238818499874316781827246865101975970422729724653e+260177', '-6.1232749500229542725238818499874316781827246865101975970423e+260177', 59, 3);
    T('-6.725104939618550765431173928395192834650241283593014042083031593017666250753053463178926294e-272110829', '-1.96775801480956780977688222508943001427298681926731529227256e+5204', '-1.96775801480956780977688222509e+5204', 30, 0);
    T('-4e+34057932', '2.7204687736e-5372156', '-4e+34057932', 72, 5);
    T('-52778767844239138541000544903.419953077658801627228', '-2.568919655518682638698378406672016628733045433998472531847531586969997767088326907300846e-295621', '-5.27787678442391385e+28', 18, 1);
    T('-3.893893722141310195856477046403734430559210677136604818565413069e+69', '-906603890794444388395.570619495', '-3.893893722141310195856477046403734430559210677137511e+69', 52, 1);
    T('-8.81229338868001998299180327047661439503447161412750331996238e+8000833', '7.10e+72334107', '7.1e+72334107', 53, 6);
    T('1.9073523075210591628399217947229235482599979859542453968151639492761427374317677047016706e-68697921', '8.62926504714950268506252035982947537551143207926721435664667432668122376147823310485347e-497697206', '1.9073523075210591628399217947229235482599979859542453968151639492761427374317677047016706e-68697921', 91, 4);
    T('5.88838076280402448084115868115386533366211095705597080840349965661038755958058166613e-259505', '-4.966617177658939836527530662035272225160825307279295e+664730803', '-4.966617177658939836527530662035272225160825307279294999999999999999999999999999999999999999999999999e+664730803', 100, 2);
    T('-6.5797023279497078683311222459038721773549944713778509816700632922931144465371501372540504372e-110202', '5.5e+964526817', '5.499999e+964526817', 7, 1);
    T('-3.01513517603019507689774056350688755591385809793287015419593571927600374849438055408e+903', '1.04626335363980490094962419733628593730516074876985832836562010942417926207772062827166554e+705639', '1.04626335363980490094e+705639', 21, 3);
    T('9.1259303823101737016714070963127195343535068228e-65992153', '2.88888644372413516119168839e+118485', '2.88888644372413516119168839e+118485', 55, 4);
    T('8.897927399561052502546536263054e-1631', '-6.813207949104406910320571266749940824896888725025462382891003696718e-109023293', '8.8979273995e-1631', 11, 1);
    T('7.0176739402541611024147947302790852020734709909848021666972544604480446008721938019315940e-9076', '7.4328102914352196680215779310043115395841e+83', '7.432810291435219668021578e+83', 26, 2);
    T('4.18829791775321405962891058608624309e-18', '5.8279426959039716e+7878973', '5.8279426959039716e+7878973', 87, 5);
    T('-7.70187373630031726866991220055483837130190689493306317421757857558838319077132636796357374e-85977', '2.39428250484114320264745277190796285997565866e-8811', '2.39428250484114320264745277190796285997565866e-8811', 93, 4);
    T('-5.5687061084894160915272231739158646952999499868326954787341125122179105635116859e-98264', '-9.833875530427996297339069906022194479420479088997617615526241e-140387406', '-5.568706108489416091527223173915864e-98264', 34, 2);
    T('6.51546558e-351348', '-1.009311606942409666066958239337314246217936e-44463', '-1.009311606942409666066958239337314246217936e-44463', 92, 4);
    T('3.68986067462482385137062034599595717406221783923391776135402436025041005423806227e+8918', '-5.51578819646147976745822366267827292168525957472642791456563141903e-842509574', '3.68986067462482385137062034e+8918', 27, 3);
    T('9.5479029478413188629230e+24586', '-4.02341850961337792110933078343745099320928801167322669236e-504259100', '9.547902947841318862923e+24586', 57, 4);
    T('6.469174372402629749564158311861513e-248311', '-522549146859881200269505008464414413764466367424976805206643373294599234521261793189334.733658', '-5.225491468598812002695050084644144137644663674249768052066433732945992345212617931893347336579999e+86', 97, 2);
    T('7.4855867847004017007632083256370144835681594432130112051944e+62', '5.1362038189358186948242545757642568231387904823664566203446556325647494115801117354285e+78090718', '5.136203818935818694824254575764256823138790482366456620344655633e+78090718', 64, 6);
    T('-6.422669244979759375859202433493631341904448714924890484249010544080951911678267928471e+5013573', '-12215083913066', '-6.422669244979759375859202433493631341904448714924890484249e+5013573', 58, 4);
    T('4.754453446827023889514136086151828192086667555621043224264902014673600740830856030273446419375491666e-2991', '1.6656559992799115073333941417699145436431754495168423871774e-847', '1.665655999279911507333394141769914544e-847', 37, 6);
    T('9.1991973648141434848562566220101877518423784894280571348882973937222218708e+1027', '8.892562217544372997947423432530665e+443', '9.199197364814143484856256623e+1027', 28, 0);
    T('-5.7319064433730810731366683813373e+33', '-0.01142576011584022308164547250', '-5.731906443373081073136668381337300011e+33', 37, 5);
    T('7.45463756803576e+86632426', '7.8243613873852770113220158905643133315911529494349156183083189246427893011442956643e-7662646', '7.45463756803576e+86632426', 27, 4);
    T('2.665874058634827167280607062172186069256784528699366032357838900530594924775594840880460387463201e-28796', '-4.16552670334419537399164265321026e+333747', '-4.16552670334419537399e+333747', 21, 6);
    T('4.622845354966038122818839506132415767470916550958338871375369494423e-166498', '9.316519988165115583867482855987778106100697080512265225710661441e+5324', '9.316519988165115583867482e+5324', 25, 3);
    T('5.890431981519215467067779505923170756242980884806622660166564e-5068', '1.001666279422083460395773053424e+39', '1.001666279422083460395773053424e+39', 53, 5);
    T('-8.4860624623034619636406181534613548866111e-13468', '-38060970946162148908834150951716969267410243868921606869676127366658520961287733563979022524703.5890', '-3.806097094616214890883415095171696926741024386892160686967613e+94', 61, 4);
    T('9.7939485497449393280e+67956', '-2.2853974900004245405801588188029955726651055330641e-121429300', '9.79394854974493932799e+67956', 21, 3);
    T('767847404878147549148280535538198770635082', '8.47787595376752507e-22315', '7.67847404878147549148280535538198770635082e+41', 82, 4);
    T('-8.96972018632380994785200270743587043746286859713241839139e-91173256', '-69792129262854739441', '-6.9792129262854739441e+19', 51, 2);
    T('7.537637558103433471491337992476185919368482269628056463902944573690655524264100972948759e-3694', '-7.048127822736888787572887067612113550798700260590800798155e-874115013', '7.537637558103433471491337992476185919368482269628056463902944573690655524264100972948758e-3694', 88, 3);
    T('-7.726196119831501170874251553445166937467407894597363797727032765162710752774928941470517672657597e+104', '-1.99474931463072173010433839263190953974623123585558851787579500968352262863542162267935e-725', '-7.726196119831501170874251553446e+104', 31, 0);
    T('-3.69661372717258904970737897480508998818464e-19134', '-7.47086923777107641555576182599914404066632669648123408275505108164886163468e+33044', '-7.47086923777107641555576182599914404066632669648123408e+33044', 54, 6);
    T('5.795256200663810824305364285510940994288700107849078023296961463160082585688383695213289071461235901e+77556', '-7.5211496294461779491001231529502156898640639358e-332538272', '5.795256200663810824305364285510940994288700107849078023296961463160082585688e+77556', 76, 4);
    T('-3.8580216109622301742525501398725919783451883275417796120912944343393577535808347514911934e-486885245', '-1.9430466720784793002722900859293646643e-40065', '-1.94304667207847930027229008592936466430000001e-40065', 45, 0);
    T('5.04329505858931848228086347630932262310842034955981124452701943938132869361e-385566867', '6.027595735130391792660780e+309397', '6.02759573513039179266078e+309397', 94, 3);
    T('1.285959456274163855561657458189424860813362883334844403399989360119468991390e-7700', '-9.870365049680046337427469423856015206848399378e-78334', '1.2859594562741638555616574581894248608133628833348444033999893601194689913899999999999e-7700', 86, 3);
    T('-3.7760141603270767133445764679560536617406596295057734146232474229048601023434200350991092451e+12435885', '-8.30323470794798480e-70', '-3.77601416032707671334457646795605366174065962950577341462324742290486010234e+12435885', 75, 5);
    T('2.016593188705117334761368379694524157e+461145671', '-505397577622202792713626160491068445647.1', '2.016593188705117334761368379694524157e+461145671', 53, 5);
    T('-8.14604679283534971e-3890', '-8.031098412994961977616413907443e+109', '-8.03109841299496197761641390744300000000000000000000000000000000000000000001e+109', 75, 0);
    T('9792111391.3', '1.005749956072210632375434705273892303146691319296294596816827524669374380219237661331471174e-19357', '9.7921113913e+9', 90, 6);
    T('2.84534836547447881e-709511', '-1.16015225820012783459489995953716297354091e+44', '-1.160152258200127834594899959537162973540909999999999e+44', 52, 2);
    T('4.5e-72270', '7.0152402903762909142770707205984080961231293398285413630265845027918e+82', '7.0152402903762909142770707206e+82', 29, 5);
    T('9.783847011512377e-40039', '-4.89876037444063430742428597192e+15299', '-4.8987603744406e+15299', 14, 6);
    T('4.7483873695575212576441856515805578206779711837578433060572702998278992157455833618483866356e+11761903', '-4.547289989434959824921836172237872076170052898611928869923771e-51724', '4.7483873695575212576441856515805578206779711837578433060572702998278992157e+11761903', 74, 3);
    T('8.0643829283981266697895e+119', '3.01645493355842249153e+9145', '3.0164549335584224915300000000000000000000000000000000000000000000000000000000000000000000001e+9145', 92, 2);
    T('-3.433442882520230545610272894238237851264314e+105', '-7.153626896964457281411122026154422305136908343733576915743120246942e-5265494', '-3.433443e+105', 7, 4);
    T('7.45707780790771772229918004276914055315921684420357e+264326827', '8.6691153941599584771054658033260243348400586249382147334390159017850113427638013391928976666627305e-303667978', '7.45707780790771772229918004276914055315921684420357e+264326827', 100, 4);
    T('-473141360806058525843894201459971712370427575294852688175999669572807169185324.8', '6.525368077357389269950076672560198572376e+2085', '6.5253680773573892699500766725601986e+2085', 35, 2);
    T('7.46390312547437194212472057996859773589038449445135343449755616106597486002734375597664584363154e+104', '8.6819024083797412573496673169725037043686746997363288009458584860491504606039e-9735', '7.46390312547437194212472057996859773589038449e+104', 45, 1);
    T('0.251009', '-1.996693040468909474170912315758062984629485148724532287552031798880627e+9736', '-1.996694e+9736', 7, 3);
    T('6.8198786713673955264911365515491967520630617944389593907959521721e-845553264', '6.3307285723134059587080801911536063308503629849325247e-83766705', '6.330728572313405958708080191154e-83766705', 31, 4);
    T('-5.53236922684814624860837361807185633398430739846998114573465700215942015984788635385676514119e+96', '3.004203704315194255944047835376014679227488805189027604615304438542750190048355299651564249653640435e-235403057', '-5.5e+96', 2, 5);
    T('-6.0792749e+79738', '9.186399833934811620165094227516525715088240154608276365816579457255693456929665e+99', '-6.0792749e+79738', 81, 3);
    T('-8.9109700274504200737619882258231192482780451093327e+50', '-1967145946582052277958388222980452868444420189261874098136950132.646', '-1.967145946582943374961133264987829067267e+63', 41, 1);
    T('-9.741319622384215519301174968433e-1683', '3.56261412456745019464143655538024911232561728349513578714825508755261015524067092662112080902413e+961886026', '3.5626141245674501946414365553802491123256172834951357871482550875526101552e+961886026', 74, 5);
    T('-2.7242143568625293095416532779545356680058e-46264', '-6.2021987457908362191029975836536789e-2533', '-6.2021987457908362e-2533', 17, 2);
    T('6.094e-88', '2.39872277998826419090834804447064510344102426796658208135200927e+11563', '2.39872277998826419090834804448e+11563', 30, 2);
    T('2.610008350778557135390645318691097214652192600674324449793856895950157634794572176759482222660027394e-703243', '-3.46285789046914399751548677381295466171462306671013852558654583471036753e+48171014', '-3.462858e+48171014', 7, 4);
    T('7.739749077150006213103085e+818749', '-8.5895176349780205057e-996572', '7.739749077150006213103084999999999999999999999999999999999999999999999999999999999999999999999999999e+818749', 100, 1);
    T('-8.266047146053828296488426903288546586161755590889938809672980972449e-21', '7.4206477e+89728', '7.4206477e+89728', 37, 4);
    T('5.96890922779384826330659265775086e+17866558', '-4.96055025990087802585427083397192339034088825397722623342120051225473170442008473113241864e+8940892', '5.96890922779384826330659265775086e+17866558', 48, 4);
    T('2.06054244973159580725766979345178348650481337891480840329501736626457411151004955946189e+996545', '1.4269437380997882957035244e+322174667', '1.42694374e+322174667', 9, 6);
    T('8.1710018135327010786840696299515937976124929472763081285652297894699212803047747631231e+753', '-1.654339740234919484385818672289458938595063848212800249663447478721388659408475781612013708874540e-27487', '8.171e+753', 6, 4);
    T('8.1584990851131297848068313846058122501065e+48630', '-2.55343234997695739319123843708009425634296812135959526450756026494738557465466674592783513920120733e-75551', '8.1584990851131297848068313846058122501064999999999999999999999999999999999999e+48630', 77, 3);
    T('8.393251198259090326085066737441104596021811074e-46', '2.37806108174044779274360639508349408341691199717304627630971454718224230994129082208728393899e+77684', '2.378061081740447792743606395083495e+77684', 34, 2);
    T('3.22001557559812554706864282494213385304839969640856922781674893444565744490254223705106e+2581', '-8.047252501958113357848965791e-6091687', '3.22e+2581', 4, 6);
    T('-4.553852921387688850118295439465616603632572933358088831704414537e-882', '7.6573864105266915354757572065891477875731833336014565539283125283246092952328623136107e-35864674', '-4.55385292138768885011829543946561660363257293335808883170441453699999999999999999999999999999999e-882', 96, 1);
    T('-1.36179677419544199047100488285743782961001250775597618811593552222570603856429543794170254e+97', '-9.7149171117808641785241537597820769978194631443198888118e+4798100', '-9.714917e+4798100', 7, 1);
    T('2.021178e+943', '3.956700498930006726678782521e-65229186', '2.021178e+943', 88, 3);
    T('1.8306482475e-308', '2.34e+9392239', '2.3400000000000000000000000000000000000000000000000000000000000000000001e+9392239', 71, 0);
    T('-6.00275892715734344171416713988399732891564538727161567726078886706e+623', '7.3598060490377871971458096618063506806842657254946946371568075199453315031969880032507372317e-321', '-6.00275892715734344171416713988399732891564538727161567726078886706e+623', 88, 4);
    T('-3.4e-82', '4.88235275659397327527605746068176030155294e-87635', '-3.39999999999999999999999999999999999999999999999999999999999999999999999999999e-82', 78, 2);
    T('8.696035280262055430050166478e-6639', '-7.34192211939992515498067907981050555489337954279844354119838340815557864633732226724e+556522', '-7.341922119399925154980679079810505554893379542798443541198383e+556522', 61, 6);
    T('-9.033494810315202220070821e-573484', '-6.50304441561612865832331074882254253176104212416220824182517e+3473', '-6.50304441561612865832331074882254253176104212416220824182517000000000001e+3473', 72, 0);
    T('6.0904859634362106953699577118205e+54306774', '3.060330204429855655745414678243e+39', '6e+54306774', 1, 3);
    T('-4.6013600501640e-225609', '5.6722120586764486826199420972396957247614603206350891425854489571753572447947572429639521e+164', '5.67221205867644868261994209723e+164', 30, 3);
    T('3.262768634287997484891103068617020073714524982170862716722932660940054659e+7348', '-7.517273319615465382463311801034376018477618012638099793850676239e-4791', '3.262768634287997e+7348', 16, 5);
    T('7.8772842001316184672621517558373938684158780930057386537151962897e+999', '-6.788411854778315747923871360263915039336115995231524810610204534142787994951e+186375', '-6.78841185477831574e+186375', 18, 2);
    T('2.69239043995438551397001404834310974166721541816923008235709542952726424974586407874e-46187', '-2.24062784236266145335926965111027196248715e-100689534', '2.692390439954385513970014048343109741667215418169230082357095429e-46187', 64, 3);
    T('8.8599438367640064696469755096480277257e-1275', '-3.47657783339467679807560423941859681248949878127965e+563113', '-3.4765778333946767980756e+563113', 24, 5);
    T('-3.80764248419565605885959090743147724949000156e-556769299', '2.70423304541478179124503783e+261260', '2.70423304541478179124503783e+261260', 78, 5);
    T('7.2645560080400368305717312255561785630647751624932122991847359278407457722284114889457258e+95', '-3.6937643375430897481150284539784e-69288842', '7.264556008040036830571731226e+95', 28, 0);
    T('-5.72818567698753232527557546293442328147921134432741653017616623888142525489590639858657009577482957e+53587', '3.84060741342816912956460404184928675533607423266541775763181674305433349395442961918572900893e+94', '-5.728185676987532325275575462934423281479211344327416530176166238881425254895906398586570095774e+53587', 94, 2);
    T('4.79811419450615555295399227264439247404142344683769323625469747832387975876e-21504381', '9.74511543518148693184700361657110741802564e-76520464', '4.79811419450615555295399227264439247404142344683769323625469747832387975876001e-21504381', 78, 2);
    T('2.35063527438339157688797430806357843400161460099280945e-81507', '8.8722e+6327013', '8.8722000000000000000000000000000000000000000000000000000000000000000000000000000000001e+6327013', 86, 2);
    T('-6.066525551145769495e+636357', '9.394060436519028997723543278961689465156613e+604449', '-6.0665255511457694949999999999999999999999999999999999999999999999999999999999999999999999e+636357', 89, 2);
    T('-1.027101347011046734912290087298e-521', '-84381717899307722124621710498210992340843389969827312768048256494783060898902211524805636108781470', '-8.4381717899307722124621710498210992340843389969828e+97', 50, 3);
    T('1.63026301892462e-88656639', '-5.6734226672961866530946754814715819656670e-426', '-5.673422667296186653094675481471581965666999999999999999999999999999999999999999999999999e-426', 88, 2);
    T('-5593.45516', '5.618062072838632327200766221052528342546506169880170165938183511458057274129e-8821', '-5.59345516e+3', 98, 3);
    T('7.51011519148325170865082806118661533042908613286151630e+481', '-7.764069660e+108', '7.5101151914832517086508280611866153304291e+481', 41, 2);
    T('-2.52796472644530e+1035943', '4.5328556540354119765172623981136052280366801068412902187e-63218', '-2.5279647264453e+1035943', 14, 0);
    T('6.37432331536564905660167501807227526134268023661013400609464956136769621618743e+5092287', '-93182083277163459131057461172229013310.175855383989923607419659906854920562935884446785615045372', '6.374323315365649056601675018072275261342680236610134006e+5092287', 55, 5);
    T('-1.7207897892144056953125999947564615345826470564824817704779581214641e-7491538', '9.3307720061610831741876565200322e-679', '9.3307720061610831741876565200322e-679', 34, 6);
    T('8.4112081534975474988689937336707349206305063136391157111325823833216692483510784442e+504974174', '-2.8e-134', '8.4112081534975474988689937336707349206305063136391157111325823833216692483510784441999999999999999e+504974174', 98, 1);
    T('-1.40585784173841976337531568278761950480180970810087821173652180055e+574', '-4039419977893254.8', '-1.4058578417384197633753156827876195048018097081e+574', 48, 5);
    T('-9.562409414905394682639936263005556483913e+320208973', '-7.12217744331032143866178932206211343665161611598342672134632933853070564447621108440206570820764e-330283', '-9.562409414905394682639936263005556483913000000000000000000000000000000001e+320208973', 73, 3);
    T('3.31551293463483352396871130142657e+417361127', '-6.900e+21592633', '3.31551293463483352396871130142657e+417361127', 70, 5);
    T('-3.1933734587078071204233298641269972167148083346699093e+57', '2.7833143827909405342059399310486962835641543100512397e-34', '-3.1933734587078071204233298641269972167148083346699093e+57', 79, 6);
    T('-2.6209475215869937297519e-22656480', '5.809502604791910567240972783044164393561636631178848986519545449120578358199592160790453694e+97', '5.80950260479191056724097278304416439356163663e+97', 45, 3);
    T('-6766218283641217603898495.1940201543938812367531546977932251997229832083103366864921', '7.7618338185270963816155614087548786795058163860199511611788213334124739223451410326083329501393595e+684136', '7.761833818527096381615561408754878679505816386019951161178821333412e+684136', 67, 6);
    T('3.69610403601069408219226457159717682870490755565727216338616312386659890771877942304232521352e+175', '-6.82911054570e+2832953', '-6.8291105456999999999999999999999999999999999e+2832953', 44, 1);
    T('-3.3851537687294911509538171548979171633704470089590359547990e+16460305', '9.5846827260e+9901', '-3.385153768729491150953817154897917163370447008959035954799e+16460305', 87, 0);
    T('3.0675656830642235492592511913571634751007509162141331408274975567275509997e-2705', '-3.3522787489928723166e+26', '-3.3522787489928723166e+26', 87, 3);
    T('8.83035538507989715996485946452503885925468471385760381400410176992404937386223320802856313787e+930912946', '1.18013814801294331242126750827371102057485438774087036769284808973627108995e+143456', '8.83035538507989715996485946452503885925468471385760381400410176992404937386223320802856313787e+930912946', 98, 3);
    T('5.206642696306897833872170171705850018402e-212040', '-2.71664533905375354178480894938371008090673110853386107e-7626296', '5.206642696306897833872170171705850018402e-212040', 84, 0);
    T('1.85634060079822406840454482052531998594167290336e+4592255', '5.71877959975358015717831751102132737570034732410031355578213655129548850103440156145218557002e-5627', '1.856340600798224068404544820525319985941672903360000000000000000001e+4592255', 67, 0);
    T('1.92711768833483446249358821512914e+9427', '7.910071237e-106008', '1.92711768833483446249358821512915e+9427', 33, 0);
    T('-7.99728594691178442114264095644985260360194254176e+37037601', '-5.16018000058581384986243441752e+122', '-7.997285946911784421142641e+37037601', 25, 6);
    T('9.659790031211635668269849692007886464710160584899797346500715748728380194026058351609419e-268825', '-4.45035e-9', '-4.45035e-9', 49, 0);
    T('6.65956745298891819239386882475611287967359010501441932637433422383059249070260181764686e-937622', '-7.7161053787088734980842283512383707433549638932158960241824162291113503e-761987', '-7.72e-761987', 3, 4);
    T('1.551663837092639682970e+28', '-5.333456120888549465329221691e+2823666', '-5.333456120888549465329221691e+2823666', 94, 0);
    T('1.421675548832937000301269928922336304151169567998104e+948837539', '-4.82692431810245941e-424', '1.421675548832937000301269928922336304151169567998104e+948837539', 90, 5);
    T('3.470915482844452346885531917809471874994e+739638', '5.5764008075258909403341751676946901e-75965', '3.4709154828444523468855319178095e+739638', 32, 0);
    T('-2.9282840772812132152e-5478', '4.293226032557532606583691447618166801e+23789', '4.293226032557532606583691447618166801e+23789', 86, 6);
    T('6.90520662730295406597548896480707458427784152244033110628653072981640532237282416888356925e-230427', '-4.025499767526529401830711911838756837395866e-7709178', '6.90520662730295406597548896480707458427784e-230427', 42, 3);
    T('7.05808917121342118110575154695182659870449323534993160944951417635350765033310881081e+39832', '3.095e-51232', '7.0580891712134211811057515469518265987044932353499316094495e+39832', 59, 4);
    T('-2.6941449266358631036838678e+31', '82001768178628902149967407224650318226589979377901165763373475283925053672012.23216647', '8.2001768178628902149967407224650318226589979350959716497014844247086375672012233e+76', 80, 0);
    T('-4.1412056747995612645832143031467801813768e+304584', '-8.698636585974061540943122626123804806288060382725942372886870813753602355342199973213733391489881667e-81363192', '-4.1412056747995612645832143031467801813768e+304584', 71, 2);
    T('-1.56468698097475603125089379062022726568506935325210214399638008087489554275893807234593088311993538e+93147', '-1.21947338763147012774674323189554927438544981563465791807179820990e-675276169', '-1.564686980974756031250893790620227265685069353252102143996380080874895542758938072345930883119936e+93147', 97, 3);
    T('-9.324862266420274054407362069496251752314251671583385805285394377225609691497913e+50037', '3.176929074443859097533835202300390832746575898410761194783126842939238678391017703589219e-87539978', '-9.3248622664202740544073620694962517523142516715833858e+50037', 54, 2);
    T('-50047612866711045308931855561307562766243264197732420656473683', '-1238598581326108414786891247429464799748156822007147342730070240891494192283357109908137', '-1.238598581326108414786891297477077666459202130939002904037633007135e+87', 67, 5);
    T('7.8685866948919683241206647206325316115793431522570228741628126604240564954975633494518158523e+6045367', '1.284056833405742145560871861799104443178860517316e+66562220', '1.284056833405742145560871861799104443178860517316e+66562220', 58, 3);
    T('8.18284704714903006487591601362112616264801875100622493448971e-1222496', '5.3449996944074351747984586797e-62', '5.3449996944074351747984586797e-62', 68, 5);
    T('7.5715639673594e-9229', '-8.625680231618839397517158045885904649810168795462111746267190050245251505681e-880003', '7.5715639673594e-9229', 55, 2);
    T('-8.7292438801040e+112', '-3.293314996140164311017772946994e+112', '-1.2e+113', 3, 2);
    T('-8.67101641118349920587283854702964226821268727521731643231494983142487927240660e-4217600', '1.234e+84094', '1.234e+84094', 39, 6);
    T('-7.04746219516707940269745418013827026577602695264102285463096e-20498191', '5.0268720933792121049069269327440830090297903819477737676435174219792431688897937325995857e+9176', '5.02687209338e+9176', 12, 6);
    T('-1.2212197539377765620002220e+256993', '-3.0231728771020552056717e+6516847', '-3.0231728771020552056717e+6516847', 51, 6);
    T('4.8931250204867756083870762447460630590544588070120385626399207256394145992110e+965844', '8.967e-808824', '4.893125020486775608387076244746063059054458807012038562639920725639414599211e+965844', 81, 3);
    T('9.2481509679909185894e+28', '2.64738056049452807436510191158e-34429', '9.2481509679909185894000000000000000000000000000000000000000000000000000000000000000000000001e+28', 92, 2);
    T('3.9532813667971713e-6293', '-5.6399275733543826174756233516339853321492012000004634695989209524327109897114210194013578721846e+764764206', '-5.639928e+764764206', 7, 5);
    T('-40412201', '7.582466405970229408995337628034920999e-7868', '-4.041220099999999999999999999999999999999999999999999999e+7', 55, 2);
    T('-7.0035896573229870622954567768285949665496e+8964', '1.857725889319477705928865898385654669858277539997816387716232642431197091e-17', '-7.003589657322987062295456776828594966549599999999999e+8964', 52, 1);
    T('4.448099823807600239024890798480972754898016321914872985178507208508274257218910473291670588357e+94', '-2.6441385652e-771423225', '4.448099823807600239024890798480972754898016321914872985178507208508274257218910473291670588356999e+94', 97, 1);
    T('-2.56620576283492147033023e-96358643', '-22.524708538239688721848857515', '-2.2524708538239688721e+1', 20, 2);
    T('-3.6548595461292619338862610152601620789485603376240422182e-863216', '-7.308669935290571233688e+3212', '-7.308669935290571233688e+3212', 97, 2);
    T('-5.91470496718265350679321e+82482221', '-7.35134112801173598163317834286123882474543556161356950327475004329194264827838211192282501231373e+99', '-5.9147049671826535067932100000000000000000000000000000000001e+82482221', 59, 3);
    T('6.50581237148260957891899190619967814703493863711722684248734753772550167797771633012e+767437936', '-3.72968360175300077900039698701979623665903640e-33469', '6.50581237e+767437936', 9, 5);
    T('-2.582930219418110690190942902843511295419561917614943311e-537508', '-5.1670534599120810103411008821285249266136402512462e-55068', '-5.167053459912081010341100883e-55068', 28, 0);
    T('1.4316614496994503355e+123', '-9.0241777385500083200519316604151678327422985225410933306257205896360489040515827444891415443399e-23042041', '1.4316614496994503355e+123', 74, 6);
    T('2.569711735035466660327638909973556545299140873334244821509613e+977296733', '8.970298242147897e+2920', '2.5697117351e+977296733', 11, 2);
    T('-5.30778431467975806649864329467124337293049069233800987605399270687756644744e-545466683', '1.992568569163254125085e-481', '1.99256856916325412508499999999999999999999999999999e-481', 51, 3);
    T('-4.053612240508492317296513484866295539496142134107979181362033030e+132', '-1.9796865400421243852309857766321930821168098068347e-10626', '-4.05361224050849231729651348486629553e+132', 36, 1);
    T('-2.61477330980180919674502096899541532581192804029898280335960e+61', '-7.88224538608307974461e+978648', '-7.8822453860831e+978648', 14, 4);
    T('3.71639938399201110332451812976444323477266070120761212363351e+148', '-9.93643292115130872974936410409451688490298864537706103912559036e-774902483', '3.71639938399201110332451812976e+148', 30, 6);
    T('1.5251369225024491e+32078', '6.24418693075559633190243687793446701841415852270854286427948711300342664957030476919370288441499174e-9247825', '1.5251369225024491e+32078', 92, 3);
    T('9.2065470110692730616813e+118', '-9.47346225716582313412324348831105151876457777445331779641292631046615e-1335', '9.2066e+118', 5, 2);
    T('-4.98505415141433529876042612953826832678830238589297532653252691397904390805118230e-7', '2.1900512925156579029921605660519741257038757565e-829546853', '-4.98505415141433529876042612953826832679e-7', 39, 6);
    T('-2.23767782360357858298936238639849767015220029537199029212953685621911105526011489095741066097850e+277095', '4.0786424505995687416770150728050134024141961229705125544581204e+8920', '-2.2376778236035785829893623863984976701522002953719902921295368562191110552601148909575e+277095', 86, 0);
    T('-7.817942348061258550175085335549986120980823299892339802603592386501328835897994326109217e-975393', '4.10732e-3861328', '-7.8179423480612585501750853355499861209808232998923398026035923865013288358979943261092169999999999e-975393', 98, 1);
    T('4.948124802489922003783631522055733e+109', '-8.1958150588371907929305889340515212452466490467771906740527948816696464842e+128', '-8.1958150588371907924357764538025290448682858945716173740527948816696464842e+128', 91, 5);
    T('7.8811357470154e-3093', '-1.49129190228565098873720325581553629861434056105417704512693101330214304282255e+86', '-1.491291902285650988737203255815536298614340561054177045126931013302143042823e+86', 76, 5);
    T('8.0338972929513712016622526235492487e-5966736', '3.71409449957190177859092297591040055453732958077405972254920315438722e-84121', '3.71409449957190177859092297591040055453732958077405972254920315438722e-84121', 86, 1);
    T('495149947012791320814206776774528673875351373760302348413687735254718098698734344614347.50', '2.73431338462760552066575058136149711611496349145174768951417395130342e-463', '4.9514995e+86', 8, 2);
    T('-8.261543789020884060217963093552932695343256570161070800830617768372734231e+20765085', '-5.06974195714286874733797369698279047548922959721317691080093741e+2903', '-8.2615437890208840602179630935529326953432565701610708008306177683727342e+20765085', 71, 1);
    T('-3.075219971922e+3869175', '-3.208362766466276193795359160515663154589955708901089244775712679265221303796729711530950191886e+46984', '-3.076e+3869175', 4, 0);
    T('-7.69842412656578740066980907631553454553843215716864244176297068427106746201103891462933984175506e-9717', '6.1060437656005288165529770834879e-18220', '-7.69842412656578740066980907631553454553843215716864244176297068427107e-9717', 69, 5);
    T('8.36423496180117327818534815691085202807589942392448315978548670010941329010e-72516', '4.806096e+14', '4.806096e+14', 73, 3);
    T('-4067894494617326763804712335789112152867438309643614634684094515709990998066914511538784780', '3.9329995073534869157839337007806420795480703623889341409470902058728264672882359439e+6478256', '3.9329995e+6478256', 8, 3);
    T('-2.4272823796772716090658254075633008825876303596928966484540e+670', '7.9198271275911423475615648694694492946959024539414347e-7479331', '-2.42728237968e+670', 12, 5);
    T('-6.765392480574149512326037637e+87', '-6.025937872802712171290950405513267441656704839281993258158940997745262291873227285842584462661465386e-9612', '-6.765392480574149512326037637e+87', 66, 6);
    T('-9.24878912083640802207553949345e-37', '7.370954709017678226198926849165503641e-396834', '-9.24878912083640802207553949344999999999999999999999999999999e-37', 60, 2);
    T('5.5858728269e+8974', '-9.424807962345345290965112813e+667', '5.585872826899999999999999999999999999999e+8974', 40, 1);
    T('9.39974841247140821064313342423767167676061243147672621189124205911782172467291347251536883e+470595707', '-5.489693627100535911142495977962732919123e-716', '9.399748412471408210643133424237671676760612431476726211891e+470595707', 58, 5);
    T('-6.71540437266688574335684351191628689590086587742985766273788943548278641825790800262041633e-9620984', '5.309578699794740624319536820145722136081311765187903654173028389e+595', '5.3095786997947406243195368201457221360813117651879e+595', 51, 6);
    T('7.649640526833446079284157211706029e+3067', '-4.4365691702624690616190830870568775737733267003839807785235798578856e+99541580', '-4.4365691702625e+99541580', 14, 6);
    T('1.679615823445322006646e-62951290', '-2.76e+810975235', '-2.76e+810975235', 59, 6);
    T('-9.78523810741348875845955383478284798e+63313706', '-6.96817245381391067332184e+51', '-9.78523810741348875845955383478284798e+63313706', 69, 5);
    T('-0.0005228862724232090623110929226955438420138244997904', '4.1742136047701915125273819800301935779069914452e+2679874', '4.174213604770191512527381980030193577907e+2679874', 41, 4);
    T('-9.06949847573879496391733718949638081295021623044e-1528667', '-6.11531961e+520926100', '-6.11531961e+520926100', 98, 6);
    T('7.196188516548703946212e+1002', '7.67237556886149309232325235045487993305578581487248169117573036493883112919871262e-615', '7.1961885165487039462120000000000000000000000000000000000000001e+1002', 62, 2);
    T('5.67418359509831940639494598977631019782e+70', '9.7227670944e-6431120', '5.67418359509831940639494598977631019782e+70', 55, 3);
    T('-4.262198944701931031127653796983638773878299305441495357022e+88904', '-2.56856074786803526386018e-862363', '-4.2621989e+88904', 8, 4);
    T('-6.02845139832174831766185580297192702960639732330681442868705251559000999e-2628178', '3.56139432295927636448395189099556263281217589600791555280289135542e-286', '3.561394322959276364483951890995562632812175896007915552802891355419999999999999999999999999e-286', 91, 1);
    T('-5.810270111932230948692215034973187875479626496806489823444125678541838e-20660968', '-5.621010319241973751983737771079955435240725291112583431497877235482840758032096930940383468e+38463774', '-5.62101031924197375198373777107995543524072529e+38463774', 45, 2);
    T('5.57307017254141720212783459571115807292498422804485604756688871561428488697042e+102', '3.26550588353e+45258601', '3.26550588353e+45258601', 39, 3);
    T('-7.371467697030118638109894050656472962211184570785106738855458307960016e-93426', '-21599740151722331372247761101349731612285565888678688578369683661808432.5', '-2.1599740151722331372247761101e+70', 29, 4);
    T('-9.014e+376480', '-8.2942885706659338717479830105730932243810178439675385570793e+778049111', '-8.294288570665933871747983010573094e+778049111', 34, 0);
    T('9.23360259412875736400151265442285745939652073226938644608352108528352436e-21243010', '5.65837579410541127338782576966326939949951196361720143481125855084697129515017081631e-121737', '5.65837579410541127338782576966326939949951196361720143481125855084697129515017081631e-121737', 90, 4);
    T('1.7507998675198356452374326758253877852361242936710629023253782994e+7441289', '-6.142161325209531052374303348780562250953751595081655344081026099554632750884359062e+10071', '1.75079986751983e+7441289', 15, 1);
    T('8.39641593917020062503295804887268250312170026e-87126793', '-6.382187387926754771446684422e+528597', '-6.3821873879267547714466844219999999999999999999999e+528597', 50, 1);
    T('-5.751977193411097447480099765556921447183e+45', '-2.386185922060478323107329599307861389498960497355919896074878698498427678632687027012311e+5342522', '-2.3861859220604783231073295993078613894989604973559198960748787e+5342522', 62, 4);
    T('-2.7553117899484088901238955446115526359854e+8354', '1.6408434730396512268483214129329279634739293712e-92807348', '-2.75531178994841e+8354', 15, 6);
    T('9.157808293e+850', '7.30844490888146192e-43', '9e+850', 1, 6);
    T('-1.29761580078994424150723831613137943e+56127', '-2.17182117849332835462814688142843295982974408369e+508902', '-2.17182117849332835462814688142843295982974408369e+508902', 65, 6);
    T('-77132556664525126665030896033217740024590493025641333722.10', '-997555972860.565873553039988751202', '-7.713255666452512666503089603321774002459049402319730658266587355304e+55', 67, 4);
    T('-4693.78291482365775068140985278882007875465999549609362743465347', '9.895201047422713712e+5515', '9.895201047422713712e+5515', 82, 4);
    T('-7.611050081544864117394754564957059777764760654649166154256560327981493651e+5351708', '-9.9449482348434230337670049286609870e+507', '-7.61105008154486411739475456496e+5351708', 30, 0);
    T('-3.0963199090504505480152360268109594907561232404906e+723', '5.666519534856028927109631628735315091141235587421816633376082649890477797296833658631453e-11526', '-3.096319909050450549e+723', 19, 0);
    T('-5.521016312793e+424748', '-3.92835135185044599917537164274800735150680810842917217767277153722090797258721828558612965977288e+170703853', '-3.92835135185044599917537164274800735150680810842917217767277153722e+170703853', 66, 6);
    T('1.482884192851190010627982272175434111452812455e+717', '8.35186412128082975352473451470742296333582717686808881719561823139574376149930323e-8855337', '1.48288419285119e+717', 16, 3);
    T('1.1561107627243042062997e+154', '1.1315726276546919851605149507776933294562198290802253239805312361592076715263e+78', '1.16e+154', 3, 4);
    T('-1.6865380918641608288e+402', '-5.560788250934108716356432809100769967136948231232183753938e-79293063', '-1.6865380918641608288e+402', 78, 6);
    T('2.48696568376881594714903223295680892890906473940520205521599043076191631997748176264e-2245767', '2.1675158162435123926751716257877352917427765090803032604233e+61', '2.1675158162435123926751716257877352917427765090803032604233e+61', 83, 4);
    T('3.963388959853053000108931804196082663475942329796031975781566376838640846757291704908089997695174476e-505039795', '2.746773689393762379841465438591085532292986182800877521162e+98710480', '2.7467736893937623798414654385910855322929861828008775211620000000000000000000000000000000001e+98710480', 92, 2);
    T('-6.608156383701389094095787771872841032e-587', '-5.82134253095163834084221e+55177', '-5.82134253095163834084221e+55177', 32, 5);
    T('1.7122794327233279279736556628261282e+65', '-9.71794395298133431951498628615205971934470180065312721989782e-6347765', '1.7122794e+65', 8, 5);
    T('1.34674760721202397761516319754598720776875694643884378105588349886285160379224742763892172200720e+79216866', '-1.058358423012353514853e-62', '1.346747607212023977615163197545987207768756946438843781055e+79216866', 58, 1);
    T('-3.32346352589278970592537276207874844214244792999970050913628e+80343', '9.41731948054852199741807860050552950570599828346769082947491193e+194406', '9.41731948054852199741807860050552950570599828346769082947491193e+194406', 90, 2);
    T('7.8551251e+16', '2.768975636630641542522893504142537e+62', '2.76897563663064154252289350414253700000000000078551251e+62', 83, 6);
    T('3.65640477948484613941602496083648742563895576590896477482548758305e-182', '-2.139792878e+84', '-2.139792878e+84', 29, 4);
    T('-5.3965435946810660068528316542303059208958772967449953588177072874638e-60920473', '-5.966959243719309573204331702501286347582347e-5064064', '-5.966959243719309573204331702501286347582347e-5064064', 43, 6);
    T('-8.1927016650186822354545552078338874735470227195502469584758460478285061010887e-86095', '-737215396196099782948824.37493088758517404147667455154909841479889848', '-7.3721539619609978294882437493088758517404147667455154909841479889848000000000000000001e+23', 86, 0);
    T('-4.71069758394901913182087764856134638307470052330870233016367471673689e-926835066', '3.3173532454867270355942603830011307e-681306933', '3.3173532454867270355942603830011307e-681306933', 96, 0);
    T('7.022346791439643928510815439742587392166171618987281141027e-64848', '-2.6305372248173615379e+204809036', '-2.63053722481736e+204809036', 15, 5);
    T('8.630045316869819601366410293426674104412901085129807493855302358196853632318815004790854470e+121', '-8611068340692424057389956.743592013', '8.6300453e+121', 8, 4);
    T('-3.296846568034e+7036009', '93.45616', '-3.296846568034e+7036009', 67, 6);
    T('-5.0522746931612344535309e-9209', '-8.9962721721621793183951e+42490', '-8.996273e+42490', 7, 0);
    T('9.6117918218393646804754941037946289489e+42', '7.6022187493254696908721672e+8868', '7.6022187493254697e+8868', 17, 4);
    T('-3.065314e+39', '6.51626912203194769712922195770978e+6782978', '6.5162691220319476971292219577097799999999999999999999999999999999999999999999999999999e+6782978', 86, 1);
    T('9.519950272226654204277676753516772770644e-3076', '-5.468739166956044163017059805e+34', '-5.46873917e+34', 9, 5);
    T('1.13937341562671731492412092194519374764145615396006629e+991712', '-7.731434913389801857332801533720894009905918879225400062278574763067294597834774212433674405e-7924', '1.13937341562671731492412092194519374764145615396006629e+991712', 92, 4);
    T('8.13356475533675377347230978692768551301627765226405083763309097965599110248389537e-164', '1.65222157099804478455751450548539030816171e+214759782', '1.65222157099804478455751450548539030816171e+214759782', 66, 6);
    T('6.97315851620433976660078071633682417410166834451723998234356e+802043', '-9.090908419835496000038889672996419424331285454825435187066814718826667061391214337747756984822228e+97', '6.9731585162043397666007807163368241741016683445172399823e+802043', 56, 4);
    T('9.57530660488593029730563e-5252', '4004432014406393310793199077438839586259092882462233819385461772325330070485119', '4.004432014406393310793199077439e+78', 31, 4);
    T('-6.387044594871371067296688130741232078713103748206425779236104620675359442e-514319254', '-8.027050183788633312e+818492009', '-8.027050183788633312e+818492009', 35, 2);
    T('5.808607791354512315073266172944845020953458628561399585348448698008574346433837464737524218244561151e+159', '3.36696460456354926707920269093453310874240538779389393607205094212617368968924745284900019218e+57529357', '3.36696460456354926707920269093453310874240538779389393607205094212617368968924745284900019218e+57529357', 98, 1);
    T('-2.3787647610309334780540358e+115018810', '-2.69892630987314296218163646481402213631649768086468925800993871867758682766481171051240293e+80180691', '-2.3787647610309334780540358e+115018810', 44, 1);
    T('-0.040256066', '-2.70178101365558556985206154407763946072530e+79022', '-2.70178101365558556985206154407764e+79022', 33, 3);
    T('-2.223134112262050984580817727781565452320616468255423045576375989903425445934061e+45660061', '-3.425947383688493643197405406654270044080082e+645674550', '-3.42594739e+645674550', 9, 3);
    T('-4.40683348684888329969697151825956073139314989506e+169633', '2.46187120918e+292', '-4.40683348684888329969697151825956073139314989506e+169633', 81, 3);
    T('-5.8e-30576', '5.36444154469968691961394653233893953092567801528e-15728', '5.3644416e-15728', 8, 2);
    T('-796494495496828176330220044032362198037144777334176362971755641775556514441531070212.56103', '6.3862502e+625323', '6.3862502e+625323', 29, 0);
    T('-3.4723180107052080189464861287454800948160622e-643059634', '-200.42748179794102258971195754528687945887057924148301187731461636628535140947150', '-2.00427481797941022589711957545286879459e+2', 39, 4);
    T('-6.152855640536567422481045122925346245383063647676639758658445505573487469e+3109', '6.992726011381383e+759901', '6.993e+759901', 4, 6);
    T('4.30511467138826917225149106025351099954849095600218851e-66236889', '7.146531251374445460445416915796574616362480733199630897760127357085160146e+114', '7.146531251374445460445416915796574616362480733199630898e+114', 55, 4);
    T('-7.785186771164408667053424378521318762226931185570149706899120651138296366710736670978976586768e+678', '3.558659260422738669143278101895733079989849183302e-2690', '-7.785186771164408667053424378521318762226931185570149706899120651138296367e+678', 73, 4);
    T('-8.642155695154533e-66651582', '-7.41259753436e+793', '-7.41259753436e+793', 47, 2);
    T('3.0925348314473433751639531255e-3676898', '1.249281701970911921023511e+9836784', '1.24928170197e+9836784', 12, 3);
    T('1.83775142272693257921620203925703860456171986472158048098309174792039048821e-823459', '2.81543917766901387464912022543003334037770559642596259653580938011e-28977770', '1.83775142272693257921620203925703860456171986472158048098309174792039048821e-823459', 92, 4);
    T('7.8269e+9582727', '-4.1712668209059395927543129649094793e+253', '7.826899999999999999999999999999999999999999999999999999e+9582727', 55, 3);
    T('-4.0764620211323606951961811575021e+5942585', '0.025838', '-4.0764620211323606951961811575021e+5942585', 91, 6);
    T('-9.84278606027661663807132070469768580883635787821264e-176979283', '-5.98710187008e+17', '-5.98710187008e+17', 45, 5);
    T('-4.6691618976e-933089183', '-9.916854352233753364020701684682679748911324e-16895939', '-9.916854352233753364020701684683e-16895939', 31, 5);
    T('11500846090024.171983161086524596761449733055685043988511911939275776728200', '9.224783096721479334904240845318937152e+42', '9.224783096721479334904240845330437998090024171983161086524596761449733055685043988511911939275777e+42', 97, 5);
    T('-5.9054378818252268156698085197949633055033545938520741603646749262267468012288e+120', '-9.43956993934702724936010008390769308704039853309643165516056e+810914', '-9.43956993934702724936010008390769308704039853309643165516056e+810914', 84, 2);
    T('-4.7e+94723658', '-8.76962014811631139145338024515097093566266903573004556161610307644161e-20500667', '-4.7e+94723658', 16, 1);
    T('2.070950e+8225041', '-9.586719358395439812619751984204911102878590005075605951243980494274079e+183', '2.07095e+8225041', 58, 5);
    T('2.09298926604118762312145662683199405344767396634033e+11893', '-5.1308179969093242726481145317392149459e+101462631', '-5.1308179969093242726481145317392149e+101462631', 35, 6);
    T('7.965428044483582727568225933864224377920696913113819920527956640122402905255855584362214304801505e-8846012', '-9.1879889331294666401033727258814707727162391747e-6920', '-9.1879889331294666402e-6920', 20, 0);
    T('9.6969984627951e-446590633', '5.7e-6093623', '5.7e-6093623', 48, 6);
    T('1.09720561791979687633294007235805176247940587718357412952158999171188567675e-83275325', '-3.3602968575553749120903067754013978786741143932506845722402992800735289688759729558072e+91', '-3.3602968575553749120903067754013978786741143932506845722e+91', 56, 1);
    T('990569263837660232364951490989043442426160702049696547.8187453', '2.486849479311382336658797054802995224404692142035866646888782832348363649232977839e-646282', '9.90569263837660232364951490989043442426160702049696547818745e+53', 60, 1);
    T('458248017222535409393890390317082619359943606881414327533055389611666995509410649418', '4.74819428773611101632665177204687281248848766e+927280036', '4.74819428773611101632665177204687281248848766e+927280036', 57, 5);
    T('-8.3578990e-1495', '71156771858398217586009569139754240896.42739546782161846835500811906039147383064471796187668356159', '7.11567718584e+37', 12, 0);
    T('-2.19880342998505642563266174312395045086059831455225701896146103032261e+996937', '7.458953112240242202970387378120236368224073858450479804019e+898030', '-2.19880343e+996937', 11, 3);
    T('-6.3539879013295825438324737267973618128003035778908858949820169039457886854e+434', '-2.9105867356254922659112884000619819225208347905063077147275096331892544178459514519256025679e+5629605', '-2.9105867356254922659112884000619819225208347905063077147275096331e+5629605', 65, 2);
    T('-3.760034661339747360e+51289537', '-5.2848477181754637516918516181635e+660372060', '-5.28484771817546375169185161816350000000000000000000000000000000001e+660372060', 66, 0);
    T('8.81592813157498189087419192776850793080984313969432620e-763', '-2.1553768718363346753015179903981156877857698832392218187603833948680e+739847866', '-2.1553768718363346753015179903981156877857698832392218e+739847866', 53, 6);
    T('1.857849719837893945856044386822778328237351735112e-47', '-1.81112570575344129984693e+9493', '-1.81112570575344129984693e+9493', 32, 5);
    T('8.8193373e+95', '1.963593884912616975835e+303', '1.963593884912616975835e+303', 71, 1);
    T('-8.74391325287626533742931803940014926438238223682370802583785539595e-66334638', '12509787620945448181058963536605966383518851846914631195905429743130943999886169.74858752', '1.2509787620945448181058963536605e+79', 32, 1);
    T('9.80334794303086012e-455423', '7.2598799731366784553979731935712995223e+1009', '7.25987997313667845539797319357129952230000000000000000000001e+1009', 60, 0);
    T('6.5004798922428007024764069879e-933331', '-7.11167486e-52780885', '6.5004798922428007024764069879e-933331', 36, 5);
    T('3649062188192611081026186802204485197583244842027886572469351514.84', '-8.0761951270332043155106079532160141138222566256986908145e-660420289', '3.64906218819261108102619e+63', 24, 5);
    T('5.0819947517137e-75', '-4.14121812859342067206854502447321415963299903538025101329132686326e+66138301', '-4.1412181285934206720686e+66138301', 23, 0);
    T('-8.270689095668352105683409824543501913868202148377913699410653411937592471e+5611', '-1.5466644591351326863413689726954740202460910226912070993126932070e+15785', '-1.546664459135132686341368972695474020246091022691207099312693207e+15785', 83, 2);
    T('1.1612689203974392887841496794707784372085152048443522910615703573447401808755966821649242376357e+65981', '-5.519734917755644251633103766363716108236854501080512805447362218476043568636556061124e-4422498', '1.161268920397439288784149679470778437208515204844352291e+65981', 55, 6);
    T('5.49121362042383452044974313265581185602471635e+52', '-3.5219621764734445242e+51', '5.13901740277649006802974313265581186e+52', 36, 5);
    T('6.918572898945205011596492393429353748762206e+43', '6927.124185', '6.9185728989452050115964923934293537487628987124185e+43', 89, 3);
    T('-5.4782056073064513056457299497448268321947052961307016302014085506968077640247e+939830543', '6.43860196496e+56', '-5.4782056e+939830543', 8, 6);
    T('-3.55657704659733734508681270390544982619251190828724625403782e-96491', '14034886878181310548601627.6002', '1.40348868781813105486016276002e+25', 32, 2);
    T('-1.9140582935721911501037136892214e+581', '9.6403717595369419161347599143192801215847969509160022233828334919911971104e+75', '-1.914e+581', 4, 1);
    T('8.3139120077e+20', '-2.0671248146118856539244964061524513163159289359750978464687178e+7188018', '-2.0671248146118856539244964061524513163159289359750978464687178e+7188018', 63, 4);
    T('5.79836368116985836722574858378708279270493506418865627193044295316424659856718183263595284635035e-335223446', '-3.4455062713088044635944293382344253531297633048734040949e+230220544', '-3.445506271308804463594429e+230220544', 25, 1);
    T('-3.2558960624886070354071134019907956686340848700295195858114002250928294528e-876060', '-6.66495355e+2473', '-6.66495355e+2473', 94, 4);
    T('-5.021596186194524916358514040591713524287209339637516068804621986939422008851531442419e+77516', '8.086192628572309182696211796629e+337', '-5.0215961861945249163585140405917135242872093396375160688046e+77516', 59, 1);
    T('-6.725173839383772056399e-85890', '2.4313754421768198628389043507443338692401265896362683026596142618864708434856124953e-575', '2.4313754421768198628389043507443338692401265896362683026596142618864708434856124953e-575', 96, 4);
    T('9.53011475498485133900372592059229610539586204084304278e-914921040', '-7.734818014694938226614960905900733611132086076937477265509817867885641380356934704134087751889535e+61096114', '-7.7348180146949382266149609059007336111320860769374772655098178678856413804e+61096114', 74, 4);
    T('9.843937699558544104215635376218652920523015751264485982379958969273116412141329910e-642', '9.617353416596054541886658664753e+911236', '9.61735341659605454188665866475e+911236', 30, 4);
    T('4.3665916920305558976064795679569176010463016487405600323581298929840641826877938e+80', '-6.9089259285921094116274962202156370473634135042115774403020288424931325822266e+325524805', '-6.9089259285921094116274962202156370473634135042115774403020288424931325822266e+325524805', 83, 3);
    T('-2099508286233853483579385692483854819936822732005336.4327624623187', '3.5636911929053645302602449593897726094871023125984134560745667660449012963487e-8439', '-2.0995082862338534835793856924838548199368227320053364328e+51', 56, 0);
    T('9.605324338922864929447165678180e+84335152', '4.4474965304e-71821030', '9.605324339e+84335152', 10, 2);
    T('-48354139080211600609071405389989117617366.9894238', '8.5378990894170125436548354373771386860857294379413466818519244186336967421621456e+7195199', '8.537899089417012543654835437377138686085729437941346e+7195199', 52, 1);
    T('9.0671771107068956086211196804685620085261791060915e+270', '9.847933953184298119832041012162355084758653053617094602382122e-2730', '9.0671771107068956086211196804685620085261791060915e+270', 78, 1);
    T('-2.2114923296691544899983878983484287546992241414346349730e-893508146', '-5.9444941881962642501216275e+861932', '-5.9444941881962642501216275e+861932', 83, 1);
    T('9.8332804924099390596220192e-217724', '-4.897518161699550766862570034904029224007170160490254426884763730e+827', '-4e+827', 1, 1);
    T('7.130073935542019583424493603213861037766290697652842791028273049663667446144438102685280732e-7015', '8.3042273559170311944165023284733644573259541e+9299534', '8.30422735591703119441650232847336445e+9299534', 36, 3);
    T('3.8474477814205141211875642689754153261196315879354769770713596095120339660156471539458369e+254058898', '-9.5169799512558418476724974934220094999476520987734654211768877732985e+60947070', '3.84744779e+254058898', 9, 0);
    T('4.222964221191895624988937989e+4284', '-7.3536992231541346029550620671759342804119343e+179607', '-7.35369922315413460295506206717593428e+179607', 37, 6);
    T('-8.52827693710008357906849824890912793435727504032537168947004342269830950255871205835e+47080028', '79942042850724278796406879216799308603628037315796137252722354409495343312021515389493324.41970', '-8.52827693710008357906849824890912793435728e+47080028', 42, 0);
    T('3.7157545510094035893664162e-30', '-9.659598931635988365421121700464838460244362749673938333899128878657597322502765276586252e+4139280', '-9.659598931635988365421121700464838460244362749673938333899128878657597322502765276586252e+4139280', 89, 0);
    T('-155303467380418468671681389637181.91947494229901014036650545380513341704355507361227984075084423765', '5.58012176092095432891526476256882958e+4975', '5.58012176092095432891526476256882958e+4975', 70, 0);
    T('46166663339990120.1786', '1.898289696209044543782779680423856558271518688364773800e+57', '1.898289696209044543782e+57', 22, 1);
    T('-8.29796416712683631324177e+6194842', '3.282570422614800697518429151e+478', '-8.29796416712683631324177e+6194842', 68, 4);
    T('-32746148886909433342171179349220885119337671413869392449383815803164572312.03508', '9.44655054733163433799751346123e+738726', '9.4465505473316343379975134612299999999999999999999999e+738726', 53, 3);
    T('5.79557985735084968846474439364589924857844170822e+90889', '-9.2386009871993045741209175609421065580489834e+5454', '5.796e+90889', 4, 6);
    T('3.9285362292740183676707284281822940578963676549129458945362361e+90013', '6.44914918257916040288197711934410372966126602545054850413373e+548085568', '6.44914918257916040288197711934410372966126602545054850413373e+548085568', 79, 4);
    T('-2.37380667856165008623654390936822046656e-888525688', '4.2549549612303723e-339', '4.2549549612e-339', 11, 3);
    T('6.060516885305425637906669711071501805892334601922321003965258212449053690247140649e-15904', '3.37314581082239037302051584529206117144997154969948766552410860235698278585599953819e+24045', '3.37314581082239037302051584529206117144997154969948766552410860235698278586e+24045', 75, 5);
    T('-6.7916664466025399173055672317545911824392e-1083', '-5.42093713369487079648151528311610442145870032632983424822673037427865159e+16872', '-5.420937133694870796481515283116105e+16872', 34, 3);
    T('1.591739141098462167172345611672124311191724389130508207e-2162930', '9.8967408280822726339373494597247e-491474770', '1.591739141098462167172345611672124311191724389130508207e-2162930', 96, 5);
    T('8.2092853901222154022176164153219490051096458462987674e+143', '1.9439500773921165190775817863001945802015673535067332977567155634840943134885032885797666e+175', '1.943950077392116519077581786300277e+175', 34, 6);
    T('-3.9e+97', '-3.30166440301387189581439477446684965317615205444778668980496156405814709678344084852306457e+32309448', '-3.301664403013871895814394774467e+32309448', 31, 6);
    T('-13137556949781622296174376491.01', '3.7613455185567535510475217e-635493', '-1.313755694978162229617437649e+28', 28, 2);
    T('-7.858980669623863183899342094274997890136322830068246652328772114075102336549483324452255694393e-52588', '1.956751786531040271282374685145748536056424289830802370872393069e+55048905', '1.9567517865310402712823746851457485360564242898309e+55048905', 50, 2);
    T('6.557220636349196942643180476488648433931781798962365739585540e+381', '-5.4086098969208636122691449317485958719320e-192905', '6.557220636349196942643e+381', 22, 6);
    T('9.037963300296532593177236375957112628305082249898659457829762316109802533179450035358847e+430', '-2e-839705', '9.03796330029653259317723637595711262830508224989865945783e+430', 57, 4);
    T('2.3138570958049767597656e+70', '1.15098335351218420982215837906971377445821682037759524740803358319320376540575e-241', '2.313857095804976759765600000000000000000000000000000000001e+70', 58, 2);
    T('-9.597474560575205959855985311722175188531336214327922738087794735645608559e+900', '-7.6991991117888205090334265487098916746852977650937426822176370194739456863062e+95', '-9.597474560575205959855985311722175188531336214327922738087794735645608559e+900', 95, 2);
    T('-6.68339939182324606357811196291541111978719497234198378033477804719941052661356506504e+96293197', '-9.9006523400113313031893395416147102742121809796026647500e+30400375', '-6.68339939182324606357811196291541111978719497234198378033477804719941052661356506504e+96293197', 86, 2);
    T('-6.83901648833961885399910713686686363763521797501679571e+609493', '-8.9041521146840746665541887252517477663277119292587422342629819447565e-228', '-6.8390164883396188539991071368668636376352179751e+609493', 47, 3);
    T('8.30111031813330596493410e+493605884', '-5.752758786906397517389062817364376069208476124957e-320745119', '8.30111031813330596e+493605884', 18, 6);
    T('-8.997047109825797069960505889288838039160745522752603977974716240305739980405131416132e-467228', '-1.1991367715743425645918662540955898474927824373162514875412627148050e+5241389', '-1.19913677157434256459186625409558984749278243731625e+5241389', 51, 4);
    T('7.522636686189667e+3279', '7.2693312525643831732813490593917075632436794332228800932730058319096842965260447892753e-509', '7.522636686189667e+3279', 94, 6);
    T('2.3832799372532559623803776077568238164277887544995e+81665', '4.3532640966845059208469502877380350321149282008449873029444528213e+15552098', '4.353264096684505920846950287738035032114928200844e+15552098', 49, 3);
    T('2.099885751792247595618843471349555627531853105677904993414165214749947499642938e+853394', '9.858051250701232680293616849121472454646170553698206113649239900926656578732473485677853389409573e+3583', '2.099885751792247595618843471349555628e+853394', 37, 2);
    T('9.0165156697441686326282307866058230397131023533687167591750916559234709353e+135', '1.407924521828062105131169641790280353613526213273418489024918746259210591e-52114338', '9.016515669744168632628230786605823039713e+135', 40, 3);
    T('4149069647832208321066161695542913222640435890427583532326002712168235154624076.467629168630', '3.73503652532960720911867807412413370339e+19141', '3.73503652532960720911867807412413370339e+19141', 69, 5);
    T('1.77398356967530459345008811312818428508038935268602820218e-608853905', '7.16652952680765937095e-87748', '7.16652952680765937095e-87748', 44, 3);
    T('6.1146540097938317801975695490928141752902961943958744e+29202', '-7.338586454172221e+34705', '-7.338586454172221e+34705', 19, 3);
    T('-6.3831358510683543032484541767e+107', '-1.83266825900614779696954318930512573607267799140638638439753105492542835336826968066028256562680281e-757865829', '-6.383135851068354303248454177e+107', 28, 0);
    T('-1.9029704537791694560892394810236837693903913460607378946307541317287709480362113e-25970871', '9.210512814357476974624562498000442299236409968397811e-917168', '9.2105128143574769746245624980004422992364099683978109999999999999999999999999999999999999e-917168', 89, 1);
    T('-8.945222177853056549e-902', '-4.239678845320801266690091122830306570871430346225354263217051070050633e-845291024', '-8.945222177853056549e-902', 27, 6);
    T('5.921705160501903251025965648123072381441517557264501e+647', '-4.386974261607611000089987987746286599238921131853748606800061106485079396066444860580e+92', '5.921705160501903251025965648123072381441517557264500999999999e+647', 61, 1);
    T('8.008546275588255254062365425440104624626602165874781105697962e-836251964', '4.260920917920662111604072126206409035071465977925935446847945879802e+410460', '4.2609209179206621116040721262e+410460', 29, 6);
    T('-1.7320255141e-61134171', '-7.2673456561286539800475934181e+9962245', '-7.2673456561286539800475934181e+9962245', 49, 1);
    T('2.21919498456475794569373117039209415942044255936098224699325778373371336613272382422047e-7862', '8.32508196726840669642341871044383111983473588716953457e+54168', '8.3250819672684066964234187104438311198347358872e+54168', 47, 6);
    T('8.0847739722535967224449701192373332440898183306354909731349e-1726757', '-1.85701662205650398976091008844792398409691736e+82254862', '-1.85701662205650398976091008844792398409691736e+82254862', 57, 3);
    T('6.38804076548740372856197717366675177114908883636186053870000663111055e-976008364', '-6.51e+31403', '-6.51e+31403', 74, 5);
    T('2.06285385893439516110526894353188195451189939484176888725357517e-809', '1.0430089260509088908509564073370105399080750312829611249080149246132345052402767427205501e+3018', '1.043008926050908890850956407337010539908e+3018', 40, 4);
    T('-80961391180599777145289149991927920319652946773240031724208400328121671.638217286', '-9.5420295994816269267447254379334556368206139606125947784345087033170181e-4829534', '-8.0961391180599777145289149991927920319652946773240031724208400328121671638217286e+70', 90, 5);
    T('-9.98532399294093977562447007238168700581307753576444000788248418180765822925e+6041', '724574423307202246776.920980665379576205550866932911674020776148304999807427727913101431945349', '-9.9853239929e+6041', 11, 5);
    T('7.7631393881e-3858', '-6.7112936890150338826123530741716778445979265440773081691892942610916322e+75', '-6.71129368901e+75', 12, 1);
    T('-6.627013927517595616261863041327086817672e+564148407', '-9.0880973106265373e-7219049', '-6.627013927517595616261863041327086817672e+564148407', 51, 4);
    T('2.3815259361412541123372932850085077172762160577649060224847331e+435', '-4.48783019563356966997059884385858339708084780005611282036460962112964957e+79', '2.381525936141254112337293285008507717276216e+435', 43, 5);
    T('-7.180432292452623649586e+254402', '6155946872171588655662339274575409205275507678620735890831855552649249661328729914488939562.6987', '-7.180432292452623649586e+254402', 45, 4);
    T('-953590199890137006334768122.96328228693728859200640998209247851386860182663889044956562973156890204', '-7.2541292524e-21507411', '-9.53590199890137006334768122964e+26', 30, 0);
    T('3.1313069236931159435274989238076391226685750989e-29', '-1.1801939523672285611e+75', '-1.1801939523672285611e+75', 40, 4);
    T('-3.26732055562603997237627778226412701758816322e+266125509', '3.5590558087e+62393', '-3.267320555626039972376277782264127017588163219999999999999999999999999999999999999999999999999999999e+266125509', 100, 2);
    T('-9.52126946613881671580417696004445493493982300900589e-2225', '1.96815725185143701675877408260357058977421559937447682374093253100020074837371232509796390192908046e-909678757', '-9.5212694661388167159e-2225', 20, 3);
    T('8.79469343108197580985662838430309848189612613662675211359238369915372881995e+83', '-7.47804603211356277665544160834153758480660115512516967804802105287984241477928732665003e+36995', '-7.47804603211356277665544160834153758480660115512516967804802105287984241478e+36995', 76, 0);
    T('5.7450025234994474054801286417143056657506413852770803869225485e+129', '-2.680574e+7404', '-2.680574e+7404', 7, 6);
    T('1.33556596212296912832129037349343105148e-634', '7.95241273985598104435979330466315524711143408e-7459824', '1.33556596212296912832129037349343105148e-634', 96, 3);
    T('3.870025781592503411516895005525075534836536443890787415620531246723479657921519450928548465552989e-251', '6.5848486717802481062536028641123052312686650559273426495400734e+86938', '6.5848486717802481062536028641123052312686650559273426495400734e+86938', 94, 1);
    T('-7.71345805226471576761089750157832666936039485818e-720796', '8.845224885055879508445811117442375904871282291331447881260210321695825084565e+84', '8.8452248850558795084458111174423759048712822913314478812602103217e+84', 65, 2);
    T('5.49e+99370', '9.83429522000831792265038636360662942975086664046688882819226007583480678023578975445535434850e-188', '5.49e+99370', 84, 6);
    T('8.44180409440850355e-459231', '-1.333618686311705172305663493348906887456151490926147807961791926e-760', '-1.333618686311705172305663493348906887456151490926147807961791926e-760', 69, 5);
    T('6.81253624075951700112960961407959092265e+281', '-2.85953809911182216898462502122469818356209042119119e-20153871', '6.81253624075951700112960961407959092265e+281', 95, 2);
    T('1.3251262663978778627992672218315376227998843785222911088763574254e-906107851', '-6.955513221532909290341844265678253343928e+103', '-6.955513221532909290341844265678253344e+103', 37, 0);
    T('-7.3249360791418500997089712117705983993360442284621342040741654983587e+9287266', '-75378997321711216954642244261824415562.7204620694673917595895450383818893683559969040530490304880', '-7.32493607914185009970897121177059839933604422846213420408e+9287266', 57, 0);
    T('3.15038668255970521328e+2743408', '-8.3768279812860320995352973054506009986168136948442308345780513073416997e+132', '3.150386682559705e+2743408', 16, 3);
    T('9.72508746782799038754743356568508216634721e+59', '-3.4016051616436256869116606856854613358279e+1646', '-3.401605161643625686911660685685461335827899999999999999999999999999999999999e+1646', 76, 1);
    T('1.972328476790226115268112231299106772476816880331349006e+5664479', '-3.34495e+9', '1.97232847679022611526811224e+5664479', 27, 2);
    T('-2.880483826263104760044530e-379795', '-8.688606923546873062404304861873764221780771134190537017096025387938241249247401965e+86958745', '-8.6886069235468730624043048618737642217807711341905e+86958745', 50, 1);
    T('-6.32e+247845242', '6.89262963012869464106112890e+68512', '-6.3199999999999999999999999999999999999999999999999999999999999999999999999e+247845242', 74, 1);
    T('9.54407628656703191367318254335881543869932e+617', '5.9347103395e-6644', '9.5440762865670319136731825433588154386993200001e+617', 47, 2);
    T('1.36480197846166351731022747509038584409650041703652670651643345052730433408684e+53795642', '1.22999251310735028850614028498723270414789916373753298111130413963708336070882686072410908844082654e+3286', '1.36480197846166351731022747509038584409650041703652670651643345052730433e+53795642', 72, 5);
    T('2019798388112616626241243.98254648110720866810989890', '-8.584979843372569210647103513057272005459293463001976741883851272544e-8', '2.01979838811261662624124398254639525741023438420679352896486942727994540706536998023258116148727456e+24', 99, 2);
    T('-2.940575062996339942522166606258620845137763674777979e-792658811', '2.72221935851477061697307141601407577933798377089e+8223599', '2.72e+8223599', 3, 5);
    T('-1.5863535589872026694407020855555734e+956647003', '-3.792e-866538', '-1.5863535589872026694407020855555734e+956647003', 91, 6);
    T('-8.17686030279163e+82', '-9.229670373250342692099677588434372911111909130502112416610366012349916523206143394340572e+79573383', '-9.2296703732503426920996775884e+79573383', 29, 5);
    T('2.952778593530788093199767844066223225398610223056611870874355841409794878394128763e-352809115', '-7.20455384e+16747046', '-7.204553839999999999999999999999999999999999999999999999999999999999999999999999999999e+16747046', 85, 1);
    T('2.5955691185201783706452495237272456921483807162770614011671475e+91', '5.1934779116131420184648507981733955913907e-7912', '2.5955691185201783706452495237272456921483807162770614011671475e+91', 100, 3);
    T('6.307181963646374481133844911764745786853500582095637342424269366783e-59495', '7.46950955072379823626e+68', '7.46950955072379823626e+68', 73, 6);
    T('4.1031e-387', '5.118234683383749967e-319', '5.118234683383749967e-319', 31, 5);
    T('-3.639271048009610482989758676335035749351233310571958120603683144272206374673258e-79586149', '-1.4108249120845591389851680947904097732889236605148063804069288185026325460469922567092515033e+22032367', '-1.4108249120845591389851680947904097732889236605148063804069288185e+22032367', 65, 2);
    T('-3.958067291325903999840199822830645084652291489963446217773811447533148494664398734819e-35425403', '-7.5727679777993072571786590060482819227368974893710646470e+58761', '-7.5727679777993072571786590060482819227368974893710646470001e+58761', 59, 3);
    T('5.107794905073694925900971441634745933303350949453183e-1103529', '7.841048807450188621113101816939507071576415662960948050508571294000325458370982951897e-101', '7.841048807450188621113101817e-101', 28, 6);
    T('8.07064234919766031808369367826326633646429936911546041096e+93747899', '9.9016196228e-958114896', '8.07064234919766031808369367826326633646429936911546041096e+93747899', 80, 3);
    T('5.018071132826604489353287917375569554938969927182579336524042e-4295', '3.4549944595832373208248445724428333121656263738703026638993300021203439349807373375219895453972423e+138', '3.4549944595832373208248445724428333121656263738703026639e+138', 56, 4);
    T('-8.925635939e+73275', '45978107.57792650446325283590670439883633613', '-8.92563594e+73275', 9, 4);
    T('-4.740e-601', '-7.1602009205043010552014692007570415641178736088260184809047740906853668379553270473220375913034847e+9206141', '-7.160200920504301055e+9206141', 19, 5);
    T('-6.89340985394195140719785907032968438002477143847027486545208412932000135e+305438573', '4.26232081477968666943379074218278195322545444058923409418000379217563e-23', '-6.89340985394195140719785907032968438002477143847027486545208412932000135e+305438573', 82, 5);
    T('3e-7743', '-2.69331179848318541440256957744114791e-13992329', '3e-7743', 99, 6);
    T('9.91010545517822747514977505477538915e+47249', '1.69602103313453404e-3486395', '9.91010545517822747514977505477538915e+47249', 82, 4);
    T('7.2364087546398078671110638357461662051073678008842321486036204909342777580419592459781e-585694', '-6.57655209745755003307528693825052336610144968469130150296429365785578315e+20637', '-6.5765521e+20637', 9, 6);
    T('2.7842429148497040034166596751435e+524484', '3398665504827179620515020565835724826580461466656901418023051833517454348.615351124', '2.784242914849704003417e+524484', 22, 0);
    T('-6.1251399537600148837012485952107223834932426e-7469523', '4.601513418059e+94499334', '4.601513418058999999999999999999999e+94499334', 34, 1);
    T('-4.18419145322e-4396', '4.73298691936577700968477985956049447793611618286646648556738447987273171808155170079e-54546898', '-4.18419145322e-4396', 48, 4);
    T('-1.4925278557545526954517141038608576732300665576941119687282216186828098755283885875090732e+115', '7652176721371702722086243.21102730263524068017393357118306137768295930372126712914435', '-1.492527855754552695451714e+115', 25, 5);
    T('6.04222258770261716483358452048185824973837629890453007221799403379567114e+151', '-1.503432091808111318678846234759699358e+1872', '-1.503432091808111318678846234759699358e+1872', 52, 5);
    T('9.567566596251954976e-3417659', '8.4086515332054727979268968793627471963380890438508928203768754043486274900e+726760615', '8.408651533205472797926896879362747196338089043e+726760615', 46, 1);
    T('9.90e-34093', '-3.911987992110977898412307457522487224084763649367886814148774925199749336990805845478249419859098896e+760889', '-3.9119879921109778984123074575224872240847636493678868141487749251997493369908058e+760889', 80, 2);
    T('1.1224379184311162095193804415926941630156909387998489605787672415448252079924994254796275114676e+1697580', '7.2048606488471149933151285890025382342300783773686561186986149259128287249907693947148191594996347e+2223091', '7.20486e+2223091', 6, 1);
    T('2.98031974803532626e+138682691', '-3.69918998651384431e-9187', '2.9804e+138682691', 5, 0);
    T('9.1849078090060026454412737624683518292814629235872880072879e-900198959', '-3.91198331428651877167349531370804353429007061090745208347490816227243776625447e-919815152', '9.185e-900198959', 5, 2);
    T('-5.04711387263827776217914299280577846752499079294323697336917623369260927701528405419007116e+7964385', '-9e+407', '-5.04711387263827776218e+7964385', 21, 5);
    T('-8.4105129237014641651e-419639', '-1.49568e+58', '-1.49568e+58', 20, 5);
    T('-8.761872151807879331e+26595420', '-9.0565098e-987114141', '-8.76187e+26595420', 6, 1);
    T('1.64123313881338781491320954283392021418062877377039998321463326320878701966263058059949053e+242', '7.293798867790570418607514726592451892340875426296287683474685772873853039520260231147258667786e-254', '1.641233138813387814913209542833920214180628773770399983214633263208787019663e+242', 76, 4);
    T('7.016834780e+9797', '-1.5127795804614449442417943208341227569475287208093121975210769392202919907944712211294627138061e-286', '7.01683478e+9797', 71, 5);
    T('5.114472356903360143740385134899734404685897414734798742619328147995740160282426163947198e-71846734', '-4.63795124913696114760708731626327616869612754637819136996222e+629519', '-4.63795124913696114760708731626328e+629519', 33, 4);
    T('8.7303e-473620750', '8.73e+772645', '8.730000000000000000000000000000000000000000000000000000000000000000001e+772645', 70, 0);
    T('4.2885272612124549641906298982107078995568233551525e-367306', '7.7689881500392664236856789825750085859632069273685027794272877624565340218947171863446085233e+1018', '7.768988150039266423685678982575008585963206927368502779427287762456534021895e+1018', 76, 2);
    T('-5.1660737006904966148021554455255075266343639193369556782350712031649e+72', '8.4945760275755280195718650317857049e+80359', '8.4945760275755280195718650317857049e+80359', 79, 2);
    T('7452506323996926968313393973899711123897681584625471702847970169641309462568947397005099204094.812175', '7.5811529895171980136436411946071310414434358821870764e+922', '7.5811529895171980136436411946071310414434358821870764e+922', 82, 5);
    T('8.102208322739697515977734218786423332332780e-274704434', '-5.03291606144358440608513263546729136034854396413614161827489885876130941338246495768232605e-7488', '-5.0329e-7488', 5, 2);
    T('2.210104827e-794309', '-7.91039743305296060932916274988193757748230556318856844934736516685621e+9773841', '-7.910397433053e+9773841', 14, 6);
    T('1.5364901891484249690280903036493662195157125610246839253295594470271148726916675e+548716', '4.96847791585033004474221031826632529676240461783727890481247624820192998074265636271303e-2219', '1.53649018914842496902809030364936622e+548716', 36, 2);
    T('7.4548809804435e-825980', '94694825811562012953945287055579862131705714437246671857.803080246840594587411454345823132103703', '9.4694825811562012953945287e+55', 26, 5);
    T('1.8426922508725069e-603485', '852558446.85', '8.5255844685e+8', 95, 1);
    T('-8.50918179998826745200689725491111604618719500615763570e+102', '-8.0e+91377175', '-8e+91377175', 95, 6);
    T('-4.10955080601652119112995610505456748049425799e+57864', '-4.618355785444011143e+8060972', '-4.618355785444011143e+8060972', 38, 4);
    T('-5850020139146438516151585173183293275635127050518536027570350819050034859060419.131499184', '-6.03593e+6', '-5.850020139146438516151585173183293275635127050519e+78', 49, 0);
    T('1.786374895921527963522304486182092151927578771215663824983208166118551e-26', '9.41876743534966413317e+581696', '9.4187674353496641331700000000000000000000000000000000000000000000000000000000000000000000000001e+581696', 95, 2);
    T('-8.55093691695141758129539416144071021777931194688219468761393073557775416332e-3216', '462601612535182024114443315.41665892904513541245683024473873498066334059935845748', '4.626016125351820241144433154166589290451354124568302447387349806633406e+26', 70, 2);
    T('4.36836440825969920501513976134656888090931435399090232529184053587206893314890611978922056868e-867', '-3.09351737185636776788356e+8960', '-3.09351737185636776788356e+8960', 75, 3);
    T('-7.80651692827761030967575121574658735032842417694519256077881835981e+8188213', '-6.560060264944002920504221218714854212110075197686636e+51491672', '-6.560060264944002920504221218714854212110075197686636e+51491672', 97, 6);
    T('-3.806981407104742057352896684159216516338959732011953789280165206410062511e+26611', '7426561760175221140659148703.3949279925270362517878790233693944993986290', '-3.8069814071047420573528966841592165163389597320119537892801e+26611', 59, 2);
    T('9.7491636458686911249528443046894800794903928195099058327e-49941', '-6.10919323582428696140470445511285489962716992269358920252e+72172053', '-6.109193235824286961404704455112855e+72172053', 34, 0);
    T('-3.5882108068911448384350915533179033276034812898663109439681888060188049714726469e+106', '-1.99867686401964878659570380471361399695547371622600232e+293046', '-1.998676864e+293046', 10, 4);
    T('6.945846669645213960387533300310738097917164309322475300e-57280596', '5.62882229206237682266600942915554402126491712165244e-930861731', '6.9458466696452139603875333003107380979171643e-57280596', 45, 3);
    T('-4.606072800261484683854438168444950194436586140958957e+984660', '-4.940865273696752529583474399659175167505663055362140002180549529332870496134487849183245471382141842e+3452', '-4.606072800261484683854438168444950194436586140958957000000000000000000000000000000000000000001e+984660', 94, 0);
    T('2.570713727418294669459781240109900547686699879497882711271e-33836', '6.81783778847e+20', '6.81783778847e+20', 65, 1);
    T('1.9392625735974028903820612546556042166519383248e-647840', '-4.3616576313825678013254814018347249978882068492382015667816557635e-17', '-4.36e-17', 3, 6);
    T('8.860950125001736529437248286799778824299796519851959707856019430215385600681610e+153980893', '5.268975694740875989288120034475812e+7934554', '8.86095012500173652943724828679977882429979651985195970785601943e+153980893', 64, 5);
    T('-7.68342303991826299714752491279915963289120884701312276946766940910051176610925422828144752950914e-709441', '4.332845229056059040395926199832466e+42', '4.332845229056059040395926e+42', 25, 3);
    T('4.373955948015332456417421842949416282723572845902003858e-685737372', '-8.6346155908253999478839936300945726178954185794146202618813966e+476', '-8.63461559082539994788399363009457261789541857942e+476', 48, 3);
    T('35721246676180297658304035.36', '5.8768910027179960132518628e+69', '5.876891002717996013251862800000000000001e+69', 40, 2);
    T('7.46714706926661958639787672902768678098622175003293900829398816904867719261783197047492609421472523e-83455', '8.793160730566652091e+3009', '8.8e+3009', 2, 0);
    T('-5.1449452321812439632125708049074188561786e-336945227', '-1.1341872218374e-763', '-1.134187222e-763', 10, 5);
    T('5.671297732831348642095829072121111018599654606271463083e+516', '-114323010651866014108857831335243234520102318537163498808631001578047472530265', '5.671297732831348642095829072121111018599654606271463083e+516', 91, 0);
    T('534873395525887823976915186406449191092754144130248491828516281104416040529', '6.66124109793052569771000594549288e+403910', '6.66124109793052569771000594549288e+403910', 79, 6);
    T('4.9922754445151686423690756948137160410970450164876621960962660581780142901766528370512744214969e-6702', '-3.801663031806788419037923915756550126256850851080613e+116', '-3.801663031806788419037923915756550126256850851080613e+116', 60, 6);
    T('-5.419198889938742575042336810913582819437821107706969231306e+408', '-6.26062113913853288705924154547e-64117327', '-5.41919888993874257504233681091358281944e+408', 39, 4);
    T('5.0735190207808657803067839469000351126336280894485116748e-6438', '-9.20667690311487525245572670248837977735457287189e+1023', '-9.20667690311487525245572670248e+1023', 30, 1);
    T('1.736039406229177068276560299907e+1837534', '-9.997159035108182477838560851952051544573085550743e-57777801', '1.736039406229177068276560299907e+1837534', 69, 2);
    T('-9.030661598722806906272081620266514477935237498878e-4184', '-82378167808510960861316607191.486921734323926107544104148602319031632209445446782036169562', '-8.23781678085109608613166071914869217343239261075441041486023190316322094454e+28', 75, 1);
    T('-8.1067978824576936806982048687626275e+38', '2.453038913441346160502946764153149011336e-7466', '-8.1067978824576936806982048687626275e+38', 91, 3);
    T('6.69338485694097618744042042991307265972500098476842414137450690525e-5563', '-7.15267608586530559919054022056256363583524989e-19', '-7.1526760858653055991905e-19', 23, 4);
    T('-1.89711356570182974492502530121158182865964238163e-49974', '1.6606528215500213111661653555097670561410290487101482375742e+986127', '1.66065282155002131116616535550976705614102904871014823757419999999999999999999999999999e+986127', 87, 1);
    T('28567155510786448522496416974987.95174236841071909180615047787222702459073413', '92394019214069480332849302004284043.10133', '9.242258636958026678137179842125903e+34', 34, 4);
    T('-5.4317128779934982761718344472233808392763200524702851770756183983556099207174765363432618923e+95', '-2.79975921106341604308159831011180437741976442975053774781985094545387e-50', '-5.43171287799349827617183444722338083928e+95', 39, 0);
    T('-7.4350403132379483208892134377666804934821379443478243e-238606007', '-8.07736265507753265028595299982856172566908518175519129973004057355264684670049939043e+84', '-8.07736265507753265028595299982856172566908518e+84', 45, 2);
    T('2.4802416639399854685400775275259806783525492547116292156804612296761739463189375297052e+846936', '8.69314718328637127004445484599752324608234325268221012043717117906709417139e-852', '2.48024166393998546854007752752598067836e+846936', 39, 0);
    T('-6.392058978522602273149368942754332284533443670822315929725e+94751802', '5.56337983784234051e+57', '-6.39205897852260228e+94751802', 18, 3);
    T('-3.9526394263059744380451890037750318285155650833e-1848928', '-8.091428522214695550564346428199415747477560122187494002905345580e-46964546', '-3.952639426305974438045189004e-1848928', 28, 6);
    T('5.53938867424467403029837006422962460229803838840610588511133697155e+119', '2.4e-731', '5.539388674244674030298370064229624602298038388406e+119', 49, 5);
    T('7.48379789077357415700543183325953001735e+37141051', '872403613.85084864424646711343107356165834141214270411278923407433115713941500791255751092168', '7.48379789077357415700543183325953001735e+37141051', 97, 1);
    T('-8.026900589277459155785536824798814058072078095606137908550998673307120974729791642314118193005e-249982069', '-1.531431896755337491489326711617350e-271459955', '-8.02690058e-249982069', 9, 1);
    T('2.14575756196663891502792754450475421582569897581148793407488005443050507e+72', '-3.19395447168186519859830983661501528535157783391387545357598303056316485941476466328e+20985', '-3.19395447168186519859830983661501528535157783391387545357598303056316486e+20985', 73, 3);
    T('1.22315558526121064571640680641076317503380272359564631261021130226469459486878942e+25933358', '-1e-4789646', '1.2231555852612106457164068064107631750338027235956463126102113022646945948687894e+25933358', 80, 4);
    T('2.442744226240460266391372157076768153801842844380278742792940248273496682117335793628344e-5626', '4.36789366376109735979935404020063636069306478237444545710630868114924010061695080078883322288e-37759', '2.442744226240460266391372157076768153801842844380278742792940248273497e-5626', 70, 0);
    T('7.10801348873188575517217394852160925661593807423019759457195e+151261', '1.33067411611552558258177512413856804691482412612460435e+86307997', '1.3306741161155255825817751e+86307997', 26, 5);
    T('-1.36187e+989829', '-4.78448744233023e-12445864', '-1.361870000000000000000000000000000000000000000000000000000000000000000000000000000001e+989829', 85, 3);
    T('1.159578321638025255778567009307278606664264584751583661123599110845556446476868923442e-38177388', '-3.4977352886218728076004974180331077402131274940833313479093681379385929626943379496578357989681965e-58747', '-3.497735288621872807600497418033107740213127494083331347909368e-58747', 61, 2);
    T('-6.251240012400614768736868903191656143559464346787005725515825107423536670e-6477', '-6.58887820090261717761757625129509054434503406531178579558278913512798468798e-18950', '-6.25124001240061476873686890319165614355946434678700572551582510742353667e-6477', 85, 1);
    T('1.6123834692040856420027665329757270137030940755248442770229134e-858897', '-4.5520242836342136667548523646787454e+686', '-4.5520242836342136667548523646787454e+686', 39, 5);
    T('2.970615016e+74', '4.56736728261349477e+26', '2.97061501600000000000000000000000000000000000000456736728261349477e+74', 97, 2);
    T('2.000734721315253e+657', '-9.8565409210981422515517902235328594260832190299730770372961091716086927190496402716711969073e-97740190', '2.000734721315253e+657', 22, 6);
    T('-7.83425436832289428353750836756773105558127364390131564384615734e-768704135', '4.54934520802972824e-8921', '4.5e-8921', 2, 5);
    T('-3.61288296994926354506229846243998e-496842430', '1.1731911890989437204481171752903852553098643999473019894539637472692264438468e+52734438', '1.17319118909894372044811717529e+52734438', 30, 3);
    T('7.9055029887630438326315126414588055512258646071057619260210100e-637', '9.334e-21', '9.3340000000000000000000000000001e-21', 32, 2);
    T('-7.65646233919937e+3222345', '-5.242592924528952683418818417910221130535e+48', '-7.65646233919937e+3222345', 33, 6);
    T('-6.79161952234e+50634', '-66758072284573936548916870349677193911112643549082966820108979944089559301.510969', '-6.79161952234e+50634', 66, 1);
    T('-6.2233779248455599019502372467377781978168710858937399394140266652371004257259472799e+86', '-5.264568348823018393224547939809e-42', '-6.2233779248455599019502372467377781978168710858937399394140266652371e+86', 69, 4);
    T('7.4690127700713455671238889993025443678177e+56', '9.8925271e+16', '7.469013e+56', 7, 5);
    T('-4.87186578433876529893601651733091744443190298165920863863713044128812737e+53781', '-3.68837907908797796802942691107241555645887847433455220102514160092077e+634', '-4.87186578433876529893601651733091744443190298165920863863713044128812737e+53781', 80, 1);
    T('9.24829992679341673107880e-913129604', '9.97821295840908510816600018383514490800732580460810518815408569026865600381379e+349', '9.97821295840908510816600018383514490800732580461e+349', 48, 4);
    T('7.8583361577700909461201163836225628578599420462043e+927941', '7.05112973483714108813155309771561303172958503714373697853422231e+442070', '7.8583361577700909461201163836225628578599420462043e+927941', 64, 5);
    T('-2.196571732815829137847594043033624783753005510924853636349276384978850238625651028e-649', '7283053647003059399891613470827259795392175775918308676222236394973663782242382782848553130143609.7', '7.28305364700305939989161347082725979539217577591830867622224e+96', 60, 2);
    T('5e+2549955', '7.627660655657011843589025168666491509957184303340985419545390668078497310293310702e+618501', '5.0000000000000000000000000000000000000000000000000000001e+2549955', 56, 0);
    T('-1684844564084669002573990057707896077778146279995250406407954.70561315', '-4.31195501737959747041119625759043108531273711333049554937378712362334107422227538482e+56199', '-4.3119550173795974704111962575904310853127371133305e+56199', 50, 5);
    T('5.629804302115755764378070225045253904200777730274112092466524867007634521084310145775574910982239e+415570', '-31315401.46', '5.629804302115755764378070225045253904200777730274112092466524867007634521084310145776e+415570', 85, 0);
    T('1304123643452510306.2266417967027117706498327334174594335692464', '9.6971547021626033221724067002310734153324e-2556', '1.3041236434525103e+18', 17, 4);
    T('6.4480385246423e-46612859', '-2.570057186009478408971106457769556248829407221897723852211e+618084', '-2.5700571860094784089711064577695562488e+618084', 38, 6);
    T('4.65785496e-3280189', '-2.6108308029736754744e+763', '-2.6108308029736754744e+763', 67, 3);
    T('-7.5945951381957078028965901645893326103178891857351928280194957121221747341014093960e-9446303', '-3.52999324698660129626082070635294144623885804e-56861689', '-7.59459513819570780289659016e-9446303', 27, 4);
    T('9.11103061735276e+36228158', '9.5008396763618761e-866', '9.11103061735276e+36228158', 97, 4);
    T('-4.733416492808904407393329110538650934274038244553826929559311853493248799796095401721315479200e+479466724', '-5.313800699754120562383317870086626331446349468140922637027128427254975476801564894552659e+6943', '-4.733416492808904407393329110538650934e+479466724', 37, 1);
    T('60934993644162217937235378.57697', '-9.5584399889903414516561730291805033537982664186528845092695248678631908e+3989', '-9.5584399889903414516561730291805034e+3989', 35, 3);
    T('2.229032972424794979434810356249204721520302786468792094624e-52646588', '-5.06611913846366584161097306071306302854593427410645468957740465486139811e-539867', '-5.06611913846366584161097306071306302854593427410645468957740465486139811e-539867', 89, 6);
    T('-2.4920382855113485872561767416792750367911165634000807125898168918725759941004541e+33111', '8.638824497931e-12187963', '-2.49203828e+33111', 9, 1);
    T('7.537607309609268385056835117173479599226848221730190687966647302198893422e-856', '1921502133926660616677974739220421517273206688.976568467345042374032734', '1.9215021339266606166779747392204215172732066889765684673450423740327340000000000000000000001e+45', 92, 2);
    T('-10.86811928475540075678106181188844424989358', '6.8601482863299172058374008498526701935813154537111245252093817068663713406365200313396590526751e+959709562', '6.860148286329917205837400849852670193581315453711e+959709562', 49, 1);
    T('-1.140131256681575419539046546176596127e+84', '7.136165889718901890630388777197903764768462084790401159634713231133344635990995e+8641', '7.136165889719e+8641', 14, 0);
    T('-2.480007896207211682006025883453070752127570835759026112730737326548672266676217920340686260900e-152466045', '-1.2277832095e+72116635', '-1.2277832095e+72116635', 31, 6);
    T('8.124218797593727062333365166638221938710663455257461e-30', '9.8193457872132554410042156440851602436892746e-32654', '8.124218797593727062333365167e-30', 28, 6);
    T('9.6106895922855355106608097752079036488021604982797303587581130429553082009690994096120715124401799e-636', '-2.21855466975596261765316593961e+8601', '-2.21855466975596261765316593961e+8601', 57, 3);
    T('5.8141121007211175768162188085132660154530123462037169578489e+282293584', '-8.56922263150401172909485427827001811323952118802328266286160039081483111387727085854425e-18117251', '5.8141121007211175768162188086e+282293584', 29, 2);
    T('-5.31795664383036952032448910256081479671e-696105487', '5.375704328501833969700790568071265260452239802297576943695e+328', '5.3757e+328', 5, 5);
    T('5.7646586635431300810007967235851956330920837058343076750676732425186679502861e+842', '-7.092151567868195923685751442117e+75971', '-7.09215156786819592368575144e+75971', 27, 6);
    T('9.9973539668504774071648651531448271540030404373447715800559353633e+5160116', '-6.41985519509433358941034009801288692220585793581357716807675188431e-61049', '9.997353966850477407164865153144827154003040437344771580055935364e+5160116', 64, 2);
    T('-4.28863122501434122385271e+90460454', '-9.4651327497618621617114026959215e-657626664', '-4.2886312250143412238527100001e+90460454', 29, 0);
    T('1.5318452687533099462442981701779e-545', '8.1304038018991034652861150906870163840158940367993749571117774440e-162786', '1.5318452687533099462442981701779e-545', 99, 5);
    T('-5.2536424189586351171591825072464350387624377714437e-654170', '-9.0041040654449e-71', '-9.0041040654449e-71', 78, 2);
    T('-9.2392162011588148678463422442071435101545896452427920790924360e-91748637', '-3.171652384639e-23837', '-3.1716523846390000000000000000000000000000000000000000000000000000000000000000000000000000001e-23837', 92, 3);
    T('6.923659110744318007676939361190588436790800645002749038769598692401818194464500080268534559415e+101', '-5.299140715e+95747', '-5.299140715e+95747', 83, 6);
    T('-9.558901031814703635743064869518639512037387831018882172862446301975940899641898004486066132358513247e-26814269', '-3.316e-9', '-3.3161e-9', 5, 3);
    T('-5.7380419647023235192600681644901920221e-272234', '-9.28e-36', '-9.28e-36', 88, 4);
    T('-9.7959301285e-64', '4.117428e+921426', '4.1174279999999999999999999999999999999999999999999999999999999999999999e+921426', 71, 3);
    T('1.5338904145e-49882', '9.47874176753918429178863474260961324132152543928938024284324704538139989469641763160600e-261', '9.47874176753918429178863474260961324132152544e-261', 45, 6);
    T('-6.0389504941151652147941936453796e+40123744', '7.33696285e+18780846', '-6.03895049412e+40123744', 12, 5);
    T('-2.391695899544534963653688883732124590207060230196e+7557', '-9.3437932055604478307027902640514687242290181e+6278460', '-9.3437932055604479e+6278460', 17, 3);
    T('8.839338222502283468633789253757068405e+48401940', '-7.57888196837076056488809e+271693435', '-7.57888196837076056488809e+271693435', 25, 6);
    T('3.396012281948082390608207907729622698956997386869561732159973403446849031299756404492926884847e+101', '-8.58890897871485792998149949930977885732875425475891668359705035790554230126813113e-3330', '3.396012281948082390608207907729622698957e+101', 42, 0);
    T('-1.926065805583327332340641839134827e+391428', '1.62519931167142173358647421481273261967032183220045e-62678', '-1.9260658055833273323406418391348269999999999999999999999999999999999999999999999999999999999e+391428', 92, 2);
    T('3.328779548435388833952162472265850593624e-24', '-6.334587841936513303795836459978573e+916104262', '-6.334e+916104262', 4, 2);
    T('1.058766877335895212488648657851116880845673461201545695e+11760757', '-2.2887273752434555768378030566367684762101208715921922450240264396773797037465446434167e+82212560', '-2.2887273752434555768378e+82212560', 23, 1);
    T('9.03223675147798807960434583007267652193445537e+928784176', '-7.1e-271', '9.033e+928784176', 4, 0);
    T('2.042089803904190349051899596870950659041678750403e+93233522', '7.2419252e+583069253', '7.2419252e+583069253', 43, 6);
    T('-5200798252069307.890', '-7.992604516780981420835740746243608952206019552364052299718642941631095e+9014', '-7.9926045167809814208357e+9014', 23, 4);
    T('-4.735385053744e+30', '-3.28909087547879971748416056532e+368', '-3.28909087547879971748416056532e+368', 94, 1);
    T('-7.90332958340622631860551396502569997150632231921637e+2678938', '43314527939364987196907369933688973575065862571477984823.31022291', '-7.9033295834062263186055139650256999715063223192163699999999999999999999999999999999e+2678938', 83, 1);
    T('8.43670322259576694914488969875808913971985370026444415408145e+603', '7.31e-981634005', '8.4367032225957669491448896987580891397198537002644441540814500000000000000000000000000000001e+603', 92, 0);
    T('-9.9643908118573095267657879913184433076e+46', '-9.8732219063823040130871093837376923712103476340764008822115798730253494918443579171334989e+497806', '-9.87322190638e+497806', 12, 5);
    T('-1.98347301748897505253410988973699252129657465324284374033717913e+9957908', '6.294351260924786518368e-9740', '-1.98347301748898e+9957908', 15, 5);
    T('-6.74903188569849745066614651545155178294728500749145725358370e+60', '8.7540321323819447030446679060271263936701105216791832049456888380196585445633631366248533334459e+809996180', '8.7540321323819447030446679060271263936701105216791832049456888380196585445633631366248533334459e+809996180', 95, 6);
    T('5.11046526822718569636045589598426672336513919120658638425398531004037843062411756953500687172782e+5119', '-8.03264588e+3957', '5.110465268227185696360455895984266723365139191206586384253985310040378430624117569535006872e+5119', 91, 2);
    T('-7.921739203675850525910409749019966166911623064947494749783700643964345962e-73239540', '-5.8620308880221186946640975840398912831128749720664900319473333374747325102223011e+86', '-5.862030888022118694664097584039891e+86', 34, 1);
    T('9.136808994150878943839953259367036129230e+45', '-14.42394112644668279496559801051555513905078414888082043', '1e+46', 1, 0);
    T('7.380049226221651184854044236841255898614573e+3236', '-5.1258536650914533589e-755795', '7.38004922622165e+3236', 15, 3);
    T('-5.805821365305494918924134048548922935476541273904741491824000337595106369774502511e+41467950', '-9.901868996883579980613703740994511466272229989848236651582615586443898660e-2246310', '-5.805821365305494918924134048548922935476541273904741491824e+41467950', 59, 1);
    T('1.7141888668e+314286123', '-7.6506694249534453e-4212', '1.71418886679999999999999999999999999999999999999999999999999999999999999999e+314286123', 75, 3);
    T('9.533805592413423623452e+58', '-1.4735828497705771383135852479265695253703526550190356572566686901046474702581574833767380306224e+41204', '-1.473582849770577138313585247926569525370353e+41204', 43, 5);
    T('-5.568468955560122970155373367e-8471759', '-3.55654320560339483663977936398423002998645725560432610693217211715921116726043619e-27007969', '-5.5684689555601229701553733670000000000000000000000001e-8471759', 53, 0);
    T('-456588857196899831925424763425268648538726233857225586698074444690570601697477778308679746.499161', '-3.271727946633541172541767521765732309840522140839475272615863136790356457369755184712209708e+98', '-3.2717279511994297445107658410199799440932086262267376111881190038e+98', 65, 4);
    T('-3.1300799102270995564815918240e+72950', '7.423466291118343048169935453100580662344871508282772474855234940912495921e+8103', '-3.130079910227099556481591823999999999999999999999999999999999999999999999999e+72950', 76, 1);
    T('-8.7869624872289775061827654615607913376605315439701011335e+3576445', '-5.753536554547474918695894327345290569731449e-55335051', '-8.786e+3576445', 4, 1);
    T('8.4707534360905593180436854928377120895569973209330406188378315672216e-66525617', '1.98802833e+25', '1.98802833e+25', 23, 6);
    T('-9.662921461269768418107553750059883158891e-7973337', '-7.279062802074879268734556270266368538444562922721300862781449238138705941482746749507578104138176411e+56031789', '-7.27906280207488e+56031789', 15, 6);
    T('-4.4599532948385468963257766360188717696945988862796560e-9571371', '-1.820786015907723252077179695662290704422517068038099109422061391471068509554471718130e+88306243', '-1.82078601590772325207717969e+88306243', 27, 2);
    T('1488115520973045914214656070832091044621515003770260397450714.799', '-9.9560475386133486115050187553287382336169712737425090820e-2050544', '1.488115520973045914214656070832e+60', 31, 3);
    T('1.24236e+605', '-3.6974063353931868946432144215805112387630028322373788e-895749', '1.24235999999999999999999999e+605', 27, 1);
    T('-7.3533122931604138509390385777e-4274', '4.3344021501175318286e+67', '4.33e+67', 3, 1);
    T('6.683335914801460897192122460412062211239603834578366610110198166275491002499747962447004250034247e+8827', '5.64464842263137235677766313020076124218012064715447044683279498168933991562647571100117119275488e-143774', '6.683335914801460897192122460412e+8827', 31, 4);
    T('-4.97398464476904362458392072097067877431987892014148870021169853000339618738971568411966462116863596e-76502502', '-4.058004306338748304260528260783032567596683575522267441826473248984155285612710844631745563e-2756', '-4.058004306338748304e-2756', 19, 2);
    T('2.1727936286580919055729479404199826529328399842795059399200723819e-3423', '-1.27811363517346932476191311289683978376203045102421120427167376175970731577910655569076066093665e+38462001', '-1.27811363517346932476191311289683978376203045102421120427167376175970731577910655569076066093665e+38462001', 100, 4);
    T('-1.29784878801577356753017575190275994325489218643044445366539429e-87920', '-4.2216135779398002024778204e+576679', '-4.2216135779398002024778204e+576679', 65, 1);
    T('-1.85134182392870860422985845997868409526631665442182e+359629230', '-3.2e-35', '-1.851342e+359629230', 7, 5);
    T('-1.48959e-37858302', '-8.74357967551308646166672069566081321601535962925849585349233765655829518063064808869861021904591e-19811639', '-8.7435796755130864e-19811639', 17, 2);
    T('5.001625324174851618046146934233897427803437381119334272785293280456103204491588635250659229586847e+422827', '2.29234812199264977e+898', '5.00162532417485161804614693423389742780343738111933427e+422827', 54, 5);
    T('-9.671005752771022690595323577125668244175525820343350167003933706036271159118729745215899059359e-4575268', '-9.60373352647901996e+594474', '-9.60373352647901996000000000000000000000000000000000000000000001e+594474', 63, 0);
    T('59314615916891330599224834888365899560917455.98024538384868962886103296579470921492966422902009', '9.088476916594727719070323105655182248684802701e+49', '9e+49', 1, 5);
    T('-1.85e+983', '6.87299586149453442843998269206e-220334530', '-1.85e+983', 72, 3);
    T('9811486278764973878920.612624336', '6.018769834e+44662916', '6.01876983400000000000000000001e+44662916', 30, 0);
    T('5.24421245703965285218175149412364304184349483755846703821225062879025317e+78582798', '-90575960216438648423841', '5.24421245703965285218175149412364304184349483755846703821225062879025e+78582798', 69, 3);
    T('-14700223644701854438164714155838333956538032324416557426433801760367378691.49156934', '-9.1097899843642124737702058805423708782406279083355412171381197852e-462', '-1.47002236448e+73', 12, 0);
    T('-5.6486115791688057705954528335e+62888', '-9.9810e+9', '-5.649e+62888', 4, 5);
    T('6.5053264743109099654603641868260833381549696413058455255263086489605068571071961220764605775e-76991', '-6.05798568961683840883e+270354112', '-6.05798568961683840883e+270354112', 48, 0);
    T('7.752257862564657305282635433697654818340444274250977455868294821262310980379647238310e+855', '4.690214708246683766348909655004644325933697024251615e-33', '7.75225786256466e+855', 15, 2);
    T('6.7489529118860558104380e+445055', '-4.216432095755816990624741877321096258915497136054e+977', '6.748952911886055810438e+445055', 32, 0);
    T('9.92475460363641507897688603310297752678608471512661353622e-731', '-3.1e-3635', '9.92475460363641507897688603310297752678608471512661353622e-731', 85, 4);
    T('-8.6677290473442883100415408248540435986570967277216688575957272056e-91700849', '7.68569316665311725490485e+523', '7.68569316665311725490485e+523', 84, 4);
    T('-2.85468915697347045540155900370373053372634026305165387506166083765030544842492195e+125', '4.020797e+841624', '4.0207969999999e+841624', 14, 1);
    T('5.40064917396700554487423941942493384122020360928201129799e-72810', '-9.9088350746197571983556622284897709239477265206581306797102333565025360968936976643651508004898e-4637268', '5.400649173967005544874239419424933841220203609e-72810', 46, 1);
    T('9.31786134026494106315034782254292711050561269009948418462747185455666744821684981e+757975481', '8.20128307671334053928688763259826304424930069267230e-268', '9.317861340264941063150347822542927110505612690099484184627471854556667448216849811e+757975481', 82, 2);
    T('42802109308214337832807087753917131111220052662740422208003016788476421546.17', '8.89176018860408169180392312901075379029232197757620742552053913086159905473555894084138448683e-1904235', '4.280210930821433783280708775391713111e+73', 37, 1);
    T('6.87409899252858935014e-10568', '-4.5509262262748765072555759256322255409089e+43335', '-4.55092622627487650725557e+43335', 24, 2);
    T('8.8970476789067782469871551253477612203301204322489281414e-538537', '-1.3424e+1263825', '-1.3424e+1263825', 32, 3);
    T('8.8344078905422844394e+23', '8.9532160783184821032576924241803560364048e-8041306', '8.8344078905422844394e+23', 61, 6);
    T('3.82370646104115603787765644875361762277686124660069627462389739625952582828e+83115', '-4.282618735481167337015116055829830733522244748686782776647395654524106490e-6826089', '3.823706461041156037877656448753617622777e+83115', 40, 4);
    T('5.967613128076946863655629195e-8413582', '-4.3693841184537813888009107983120808376572632695074903750688213540149478e+72', '-4.369384118453781388800910798312080837657263269507490375e+72', 55, 6);
    T('2.8028812599464229067860544494585782944694815111560e+124', '-6.913772372739787558834876088469377210920727009037675422286061938120577692286599871222503961649315564e-48359', '2.802881259946422906786054449458578294469481511156e+124', 55, 0);
    T('3.28143231794e-22742', '-9.372001672602067416791079443582041858739328158598083e-32', '-9.372001672602067416791079443582041858739328158598083e-32', 70, 5);
    T('6.949146130336182332454760673075211007975401982164481901889170175972490613112455137732e-8615', '-6.5245457476263229268715275182072122163524875411509140999490316135164990619431014283435334456475251e+663037', '-6.52454574763e+663037', 12, 0);
    T('-3.10186794244273549021440428464150473247819707237507153329073931049978439770692e+87', '7.247435162328390469061e+62512572', '7.2474351623283904690609999999999999999999999999999999999999999999999999999999999999999999e+62512572', 89, 3);
    T('5543833983209790950765513296870504.94515457842613', '6.20377406945028250874747501803669816327382445107166305729450479641531566134686e+112', '6.2037740694502825087474750180366981632738244510716630573e+112', 56, 6);
    T('-1.07923865173246434442873164906698375111401033125328812624495142727218662e-52040849', '-5.8374541502729719959317396583e+35', '-5.8374541502729719959317396583e+35', 87, 4);
    T('4.984251862805288832107136719526214040501822380058284656784862367357599814071537993966605594876672e+865771078', '7.5195e+62354767', '4.984251862805288832107136719526214040501822380058284656784862367357599814071537994e+865771078', 82, 6);
    T('-8.92719267362590968986965646432118138106650106821581379412460e-16', '-2.92169912620500967433925152959778041895793807028371493004216344960838655878482056162244529954e+4043590', '-2.92169912620500967433925152959778041895793807028371493004216344960838e+4043590', 69, 2);
    T('-8.22092452646642144843341678676380152532332091364102394648506740517512e+76', '6.06072788565755343139256041549761073686572076816577979082647986306536243769998832759200e+4387957', '6.0607278856575534313925604154976107368657e+4387957', 41, 4);
    T('8.6565846738664037549778957531002650317350348e-990076', '1.9645169994155202697180091988651920666412539504e+1325', '1.9645169994155202697180091988651920666412539504e+1325', 89, 5);
    T('-9.1e+148', '-1.52875916396658541e+86515', '-1.52875916396658541e+86515', 98, 6);
    T('-3.205338199480980114421038e-327745', '2.105383384865766558640783598448431518860943461930017527255060145e+96', '2.105383384865766558640783598448431518860943461930017527255060145e+96', 74, 6);
    T('9.7884231239539753676203018773788250831153291894200553208552973379558226595567069669827e+841254213', '-8.751280812584637914820693704828258936290886130147860027023182122104412830573e-12712', '9.7884231239539753676203018773788250831153291894200553208552973379558226595567069669827e+841254213', 100, 6);
    T('2.8322298890975166044187502868978853344239871754740449910209e-966139', '1.301602e-20387', '1.301602e-20387', 15, 6);
    T('4.578273726365881505041e+82681', '824292607695177', '4.578273726365881505e+82681', 19, 4);
    T('-7.5e+327', '-1.92593229699210e-67109', '-7.5e+327', 44, 5);
    T('-6.638430339e+661', '4.667165516136769776411927854273491374762984452e-366', '-6.638430339e+661', 91, 5);
    T('-6.8e-27002903', '-1.90828427337317931478928359707923226472831635e-41484251', '-6.8e-27002903', 3, 5);
    T('4.037280282105104834755295093517410426932099260e-950', '-4.771138212267190026977986004555250554931679100641629361692175199e+95910', '-4.771138212267190026977986004555250554931679100641629361692175199e+95910', 97, 5);
    T('90580544583585311848090876606117506721546107976229290518130511118531527104790182.8', '-8.8989990054667627574279874980949612e-2903', '9.05805445835853118481e+79', 21, 0);
    T('-5.041869598371828222584512990974162e-2851', '-3.61850540816468545886593668496808807286865311994642934006207813e+69339', '-3.61850540816468545886593668496808807286865311994642934006207813000000000000001e+69339', 78, 3);
    T('2.2548831966556329281396794469903388095074407206218746e+17496', '-9.6e-8', '2.2548831966556329281396794469903388095074407206218746e+17496', 86, 6);
    T('8.129059431287643838630491292202848885236090016988285546101849340205913273826107897427052e-3609268', '2.03314718197069514960668438543262865876476965576e+55651', '2.03314718197069514960668438543262865876476965576e+55651', 51, 6);
    T('-3.91e+6605', '-3.36218037463219591350842646251926232585986697475094872215988935733841930652731425808960e+571949544', '-3.362180374632195913508426462519262325859866974750949e+571949544', 52, 0);
    T('6.32217017299247870346100786533429309682402486552773538246709985459102e+4647267', '-7.16056452693550135549170648071292715778320724954192070778114233938991043006617117832127192637440651e-4880164', '6.3221701729924787034610078653342930968240248655e+4647267', 47, 3);
    T('6.600554613214890087805816086813184536244066616367939317071247615179510451754957582263977362847302e-25582', '9.415101039616123531425992735943536686263277275567813e+52', '9.415101039616123531425992735943536686263277275568e+52', 49, 4);
    T('1.5319024511829395440388530207967174e-531', '7.6807757276174224353230013494925365969602553642081425757177663392947841924458350228287e+302686', '7.680775727617422435323e+302686', 22, 6);
    T('-6.84570486257969901500e-436205538', '388232671.0786544', '3.882326710786544e+8', 79, 6);
    T('-4.0369384157366188333461781625697114217342465102830367010840425545033e-733451', '-8.7746e-21', '-8.7746e-21', 85, 1);
    T('8.22159286082769063370264148984754526000534e-58', '-4.34608999304315811766568228999008229969025423755154e+87', '-4.34608999304315811766568228999008229969025423755154e+87', 82, 0);
    T('4.842155969069926554289373621176738533253e-7945137', '-2.685338295929350709452476330726816687993125311283009515e+71776', '-2.7e+71776', 2, 3);
    T('-3.632469407481644672534042124e+97', '5.98883004802478922682061458054377258692637898484875338785358445070e+160', '5.988830048024789226820614580543772586926378984848753387853584447067530592518355327465957876e+160', 97, 2);
    T('8.027295453375117635204958565076626120909413676395600334747268464716875480862084191668480480180466e-85657', '-3.0319987703723983150515158880182966318951652466715437786881186e+771642', '-3.03199877037239831505151588801829663189516524668e+771642', 48, 3);
    T('-5.318971015224736237073608005026476e+105', '-6.4203868736622667458749547390304446435132471625844530833793548013823e+9560288', '-6.420386873662266745874954739030444643513247162584453083379354801382300000000001e+9560288', 79, 3);
    T('-4.030046511352361523780892273045623349149245444793350563538e-7620', '4.648939e+10', '4.648939e+10', 41, 2);
    T('-7.848205326827040646457746075148103566760453248146752755567580605122010208412974e+833045', '-2074648184452582295318784209192372600334.108042', '-7.8482e+833045', 6, 1);
    T('9.43909548818371224179805432e-2851503', '2.0578877652343812103e-15897', '2.0578877652343812103e-15897', 59, 1);
    T('6.4606326520334681089414592e-4097501', '-5.930101334993802198743254398448639629940167898960906522785219208477410610304758787676477743413589e-668193589', '6.4606326520334681089414592e-4097501', 85, 4);
    T('-5.1387232043430467363154983480767616130135572860991300858832509220832031e+53633', '7.43876554242172282908972768289754060665303389326986326496316e+12117128', '7.43876554242172282908972768e+12117128', 27, 3);
    T('5.3915870526642937514545517982490945873604010935233995e-865211584', '9.972179116874187763303183557303330669456701872172182101647850322714946942513e-31802240', '9.972179116874187763303183557303330669456701872172182101647850322714946942513e-31802240', 78, 3);
    T('-8.6359648146355623794318134129021718569800443595061999276254e-763', '-987730529580338059567753467666754520073917325437923225468984548043070657756724411.01041', '-9.877305295803380595677535e+80', 25, 5);
    T('-85701559262458186641003169406212306392617252768366105.7', '-9.33212702536693e-45', '-8.57015592624581866410031694e+52', 27, 1);
    T('-5.047167755969310098780002091986368709317041413903742200715e+499946', '2.206727235158306307452115e+367292065', '2.206727235158306307452114999999999999999999999999999999999999999999999999999999999999999999e+367292065', 91, 3);
    T('7.118657506831002343822717638310355e-957661582', '-620697720446452868449999201440986906303159175988152624404215595711337392933507561003760410.614573910', '-6.2069772044645286844999920144098690630315917598815262e+89', 53, 6);
    T('-4.0638402876726494353708144868315839521518489828477389960720537641422494288e+4363709', '6.14570187492705266555044055126797985232205488082253829464806736844062381013212e-7943713', '-4.0638402876726e+4363709', 14, 2);
    T('7.27594184868753299788119335387870385317e+9036804', '1.35581783170e-3394', '7.27594184868753299788119335387870385317e+9036804', 54, 1);
    T('8.481885971797951312178231776618868485327747037503705391832977702138812226126168285823086363e-217046', '-3.424827835994264395678634184470172010797644871762692375609334327414208714205009074e-22975', '-3.424827835994264395678634184470172010797644871762692375609334327414208714205009074e-22975', 82, 0);
    T('-8.493e+5', '8.4799805e-3999702', '-8.492999999999999999999999e+5', 25, 2);
    T('5.8602803552841096540743830362646555384121086325336024560593558898e+7321411', '-6.661226849215496716458107051949444185644463e-9821114', '5.8602803552841096540743830362646555384121086325336024560593558898e+7321411', 91, 0);
    T('6.547468302566945511033782860447112240106933393869213038117385352351039498034269635290745e-187569', '1.179820827700451193395017824245632383318372499632880857181712939590424692852482551768155344395e-285', '1.1798208277004511933950178242457e-285', 32, 2);
    T('-5.272372065604364747091541e-20', '7.973959731144238327498787270442522985775099995638977856789525e-15', '7.973907007423582283851316355032522985775099995638977856789525e-15', 91, 1);
    T('-6.508924407836617326775527448283058923847536886313550181371e-88037327', '3.684775085347288805597148627134599209816484256e+920942', '3.684775085347288805597148627135e+920942', 31, 6);
    T('-8.93979191145434097694538584870010363162865599656558772517369349893121179362221976e+951071', '6.5647686407823402e+897', '-8.939792e+951071', 8, 0);
    T('8.73599154219839615729129243302539078561084e-9', '7.931332965041195154556895131221932840291374167605315695e+14467439', '7.93133296504119515e+14467439', 18, 1);
    T('0.0000067153177779712429454963415126235708879964', '-2.243264189857874470528761161565506902805150239e+2849402', '-2.24326418985787447052876116156550690280515023899999999999999999999999999999999999999999999e+2849402', 90, 2);
    T('5.534133807675945000717571202566865147168272309285123e+2517', '9.94345978983667829692767958317065252751540347713928248851434780095444802552142e-38713', '5.534133807675945000717571202566865147168272309285123e+2517', 66, 5);
    T('-3.0819785e+3327', '-4.794001254107295096e+89', '-3.0819785e+3327', 76, 5);
    T('-7.48071694907358422904372230324563460322807718277246076e+443', '-1.2949899934081231327493531117284041839683362484160630235666705e-523934', '-7.4807169490735842290437223032456347e+443', 35, 0);
    T('2.2127573952019130164510721691152246901807255874590683250866220385374496e-50866658', '5.146555933401751450735749531308928089235203879720014192002764858102934738180e+1864916', '5.146555933401751450735749531308928089235203879720014192003e+1864916', 58, 2);
    T('9.86986004586656995107368360514376568711178945487994e+76078478', '-6.795724771321472108192118707030862109415e+40', '9.869860045866569951073683e+76078478', 25, 1);
    T('8.90016770013840340577452e+75', '-3.944981600828709319830941104624616156263375850965688051894097756305e-178', '8.900167700138403405774519999999999999999999999999999999999999999999999999999999999999999999999999e+75', 97, 3);
    T('-8.87557476500671026385199607830282e+892879169', '9.00755014761758277185633163328267598697374506463288420250618931775602009456855720653188368e-479641352', '-8.87557476500671026385199607830281999999999999999999999999999999999999999999999999e+892879169', 81, 2);
    T('1.764393262281398179576780378533172333497910239308286100256115015253960748657122546616472e+974', '-8.80021069867649381280785048372230293e-5459', '1.764393262281398179576780378533172333497910239308286100256115015253960748657e+974', 76, 1);
    T('-4.898042644473733693358731783813478304133558542480173564e+351', '-7.400138240207026709763e+7091090', '-7.400138240207026709763e+7091090', 80, 4);
    T('-4.017720667e-29', '5.514970847613351638547948891491573641855715677328108434300e+3916', '5.51497084761335163854794889149157364185571567733e+3916', 48, 6);
    T('8.29490408938265751526637887667652e-450917', '-5.208378435948150990483424962509320968811507580370976265381964877039511587360061017920728042448464926e+839783105', '-5.208378435948150990483424962509320968811507580371e+839783105', 50, 3);
    T('149752217594626191.303788034', '-8.5032315332072802768450855759494901878382244985617613e-393', '1.4975221759462619130378803399999999999999999999999999999999999999999999999999999999999e+17', 86, 3);
    T('5.91483770364664551125782937629582743975268944690853589e+772780583', '8.29474174054348538518142424147013888776746590848298579425965494123124694782194380e-91506', '5.91483770364664551125782937629582743975268944690853589e+772780583', 87, 3);
    T('-8.628857556935498497756140112438176587128789586e+597220', '8.2276646124359530182e-91247', '-8.628857556935498497756140112438176587128789586e+597220', 79, 6);
    T('-1.9845747171954548457526887407753446281693005738737628877e-35165518', '-7.69820726425928759353829031493e+3502583', '-7.69820726425928759353829031493e+3502583', 73, 1);
    T('-2.22476519675e+767461725', '-9.66401543820895105204328171526330797657429707626812863992663958838361878789e+372011005', '-2.22476519675e+767461725', 69, 4);
    T('-4145771659303032547544850018340899588271342464019055627267876817017008158007702.3031', '7.96163804094840321742781858311e+9285', '7.9616380409484032174278185831099999999999999999999999999999999999999999999999999999999999999e+9285', 92, 1);
    T('4.26046298351660855105697229681316951916039405594550821088810579452e+90149', '768393585126416568', '4.26046298351660855105697229681316951916039405594550821088810579452e+90149', 66, 3);
    T('-2.9906439226834621645510363502875391765178960012355678954196759874079907336e-26995', '0.00780', '7.8e-3', 43, 4);
    T('-3.8835207980883912522387e-273924', '-8.75055512830118384360568410584369720607822692820761311e-347', '-8.75055512830118384360568410584369720607822692820761311e-347', 61, 6);
    T('-8.720446088978200153717528417746936675746831646502769758755115146342787647824e-2378', '-6.651024931344031304391397382198552019353736804405339911108556294752424418924956e-956279', '-8.720446088978200153717528417746936675746831646502769758755115146342787647824e-2378', 98, 4);
    T('2.57228217809383028376954701711280374072494341336642499642085436111735e-23062', '-6.36547695849387981519376790155686008772486584595907447204099426730353139891277214525955669301e-77932999', '2.5722821780938302837695471e-23062', 26, 2);
    T('-1.39326207267582999688646e+37745', '-2.5212287443892139613817282874836034417904396981061599080149278412512e+61225', '-2.521228744389213961381728287483603441790439698106159908e+61225', 56, 4);
    T('8.582339892226444777394577400670445320323382775e-105646', '-4.8883399184783996914230990188793295771e-67715468', '8.582339892226444777394577400670445320323382775e-105646', 59, 4);
    T('-2.062887465517458730695950859346798521876223433390057876569109796474838546e+778523781', '-3.576212352744914134268731e+8723', '-2.06288746551745873069595085934679852187622343339005787656910979647483854600000000000000000000000001e+778523781', 99, 3);
    T('-4.193604899278861642298962746404945549295802234216542368841306436794235024295609439278046255790e-6579308', '7.0918788953012984355444269529489753750479160344675389606007513204818698784030e-5046988', '7.0918788e-5046988', 8, 1);
    T('3.25152174622876304621589496223772183161838564676059069875452783997967208219270e+906', '-4.0456393502659895486915105820181101696666714213124068759744276748986607516172056188e-907', '3.25152174622876304621589e+906', 24, 1);
    T('7.3064204226e-487361', '-655', '-6.55e+2', 47, 6);
    T('-8.6415e-209', '-5.456257312166439874067206892513432024793e-129209', '-8.6415e-209', 73, 2);
    T('2.5527273386736251497637003872100309395386274170e-798', '-3.84330448616102868280719642020308181312253509088152582795070762788499367100134621e+47573', '-3.843304486161028682807196420203081813122535090881525827950707627884993671e+47573', 75, 6);
    T('-7.00753085898478553756481747868485844355385003348635399052074340006562310988171207351632e-38115', '-5.49813183028986861928296714801806998480095579012752045078103768517203093271264438994e-962892433', '-7.008e-38115', 4, 4);
    T('8.7140191516283536789700e+86502', '-9.977355022635173588244570945106314288466348236644587317816684946161216054758091572101923417035367214e-77645205', '8.7140191516283537e+86502', 17, 4);
    T('-1.94910426123734905663216488841382961733484580421427806537601708461482280e+3943', '-5.3330775338885146302427517238806576509970342613054969346518255355318818719798759339361464549107691e-374915', '-1.9491042612373490566321648884138296173348458042142780653760170846148228e+3943', 93, 1);
    T('-9.7541009945494239448700585773435984046193779988724624300048299e-558098', '8.4217065895114792384981460461783941220729e-9856', '8.42170658951147923849814604617839412207289999e-9856', 45, 3);
    T('6.099090794203774098485418656419759050039475976347301007919350941814726029315877931515719520565e+8506', '-3.13990120477874330026995733018705540255264699754805158252947963119755e+74099472', '-3.1399012047787433002699573301870554025526469975480515825294796311975499999999999999999999999e+74099472', 92, 1);
    T('-1.5300739091734098762642172307074938071813172590020594997251450053546336515283e+21303', '3.346251762063598559733099469404419982443327833265559667289104373973781208479322132671678312464333e-41724', '-1.5300739091734098762642172307074938071813172590020594997251450053546336515283e+21303', 87, 4);
    T('2.13451099427870e-372', '2.2462740096793908832327744526188868556213764418725778660007204350e+6079', '2.24627400967939e+6079', 16, 1);
    T('-1.1680279307853470468971309127923995348887e+692', '-9.0156441263326240496132235999795e+5847', '-9.0156441263326240496132235999795e+5847', 96, 6);
    T('-8.8967530536733719090507862753016360052749799181605897789e+5489', '-4.6415266048721760275271212036505121225527862453e+2654', '-8.8967530536733719090507862753016360052749799181605897789e+5489', 59, 5);
    T('-3853850420141728717118285838889010306896538269455062196481834.733199464447408751525065569249499', '2.21883607564427087893584293729970380879368105626984484e-35', '-3.85385e+60', 6, 4);
    T('-4.4966315699161336724634529810129202954217920478183643584606885431293376075816466551072139374e+98', '7.6763131499520787616837607905618933022158916077949154007208244270e+128669356', '7.676313149952078761683760790561893302215891607794915400720824427e+128669356', 87, 2);
    T('-6.590138770658209000983364264007738106330845367377543771577743494640531955325533825689e+416968', '-9.250768810574512622151528248236643844305387e+6492', '-6.590138770658209000983364264007738106330845367377543771577743494640531955325533825689e+416968', 87, 5);
    T('-4.2463006854140140456428986251121e+7036788', '-9.524892802131972852775445171629863008194894033928734268433686e-7005721', '-4.2463006854140140456428986251121e+7036788', 35, 5);
    T('-3.710951408461485300081120147508281079595169227041322962480e+5715', '1.4328793159041584620195113866986827273932630610210240688271189805408454588518191154412459e-8642', '-3.710951408461485300081120147509e+5715', 31, 3);
    T('-7.3960e+42428336', '-8.46406029915203332359190522996514224779418e+352636288', '-8.46406029915203332359190522996514224779418e+352636288', 58, 6);
    T('1.682708972202795341773724583200053748009238228912e-3181977', '-1.92732653680214669618518421234465653849e-8213711', '1.68270897220279534177372458320005374800924e-3181977', 42, 4);
    T('1.109041987618866088161535322818602654098268798e+241875', '-4.9070897866234609610364766108072531148103724271076422177905692060230623e-2978605', '1.10904198761886608816153532281860265e+241875', 36, 3);
    T('119186768335046428801743870548492998842279958548865644629165265.86629', '6.161228223176169598436563157755715725869888811476707779574616949449699679462366938515394e+35181', '6.1612282231762e+35181', 14, 6);
    T('9.8621105784641554204253252885129563329758442187307521732793079374978926707846470894438214442520e+9663', '7.4549246375471173456604632576957402686236952583064569280648801e+140', '1e+9664', 1, 4);
    T('7.99230345587301015490025726504335501369088464729649512e+55', '3.406819665779881202326e+22', '7.992303455873010154900257265043358420510550427177697446e+55', 89, 6);
    T('1.063987166304497e-78394', '2.367618177027391972230496331454384770542574918312614e+836', '2.367618177027391972230496331454384770542574918312614e+836', 56, 3);
    T('-5.827668815005674595221909699273238316637470759302394033078775259563010638252389541259003703e-292', '-3.84378039699395985157330253639062837958987808384093298740001033409381641422980991423784036562451e-17616', '-5.827668815005674595221909699273238316637470759302394033e-292', 56, 2);
    T('-4.357726405857715637720830386787221176780152362631230612332e-27510', '9.414980752475842867145307448015356950570211259458e+55', '9.414980752475842867145307448015356950570211259458e+55', 83, 4);
    T('7.29412798282e-477', '6.98414135487156764388660785067724764798047616886857683055953e-9032441', '7.29412798282e-477', 21, 3);
    T('5.33483652752271323206355727403119e+5821', '4.074740599285526903293249e+41', '5.334836527522713232064e+5821', 22, 2);
    T('-4.50749562138600012866035334554359140969821266371427309398622649424193072466e-80558431', '-5.6277e+201056', '-5.6277e+201056', 100, 5);
    T('6.33171169638286136743626e-730208917', '9.0340465870731549426986859382437932431137342335e+256975856', '9.0340465870731549426986859382437932431137342335001e+256975856', 50, 2);
    T('-2.3814950762621073209389035156620754275342878013518051468713e+73', '-4.05507957781664456437433836058663059529616907550802790447188077019269219629320647798487728419695659e-73053518', '-2.38149507626210732093890351566207542753e+73', 39, 6);
    T('-6.884763347983807663654355930054601248362779331213489009989e+8874', '1.144721398498380963993394648626867563119053544426347381667182962284946600475158582734067e-902', '-6.88476334798380766365435593005460124836278e+8874', 43, 0);
    T('-3.1360e+11', '-8.744608837396783678314462842564737149757293410777e-92669757', '-3.136e+11', 26, 6);
    T('-4.8752510786345416848620e-420', '6.5473706981131771215583958069707275384616531800380e+60', '6.54737069811317712155839580697072753846165e+60', 42, 3);
    T('-8.7693780962194800915153413503750644277758291503266280090e-799524', '-7.2308444014e-502146', '-7.2308444014000000000000000000000000000000000000000000000000000000000000000000000001e-502146', 83, 0);
    T('9.4449883768435665111497510078347309151824619629001011983663541907805090398693817575534620771e+855293953', '5.9726226421e-69919', '9.4449883768435665111497510078347309151824619629001e+855293953', 51, 6);
    T('2.601895583348170752501172777198122003965945749874e-74308', '7.5958624458157738e-5041820', '2.6018955833481707525011727771981220039659457e-74308', 44, 5);
    T('-5.852984194113751468269198941118e-980167', '-7.86620385360972073814354e+102', '-7.86620385360972073814354e+102', 98, 4);
    T('5299306.6', '-8.882418542067808436706376140365890804704818768020684277456877703145491033199766514595171546271595579e+598', '-8.88241854206780843670637614036589080471e+598', 39, 3);
    T('4.8951331920448183970e-16222', '-9.93678571961344385056257890081815443e+40', '-9.93678571961344385056257890081815443e+40', 53, 4);
    T('-6.32924196302110247224734457759092382475345175612527184525274898564517e-74761', '-4.744262659780602806292448544576e-71767779', '-6.329241963021102472247344577590923824753451756125271845252748985645171e-74761', 70, 3);
    T('-4.5781611699368238e-394', '8.5984183522970634387341798679792093838916491792479787384364999615045718693542032e+64652604', '8.598418352297063438734179867979209383891649179247978738436499961504571869354e+64652604', 76, 1);
    T('-1.559909535569136838773864723089012234133697328699893557891281428864429385155238156e-983116477', '-9.1329464872780882302e-83489448', '-9.132946487278088230200000000000000000000000000000000000000000000000000000000000000000000001e-83489448', 91, 3);
    T('-5.32019780028415980362488108e+785235789', '6.075729e+295759103', '-5.32019780028415980362488108e+785235789', 78, 4);
    T('5.4880547914e-171267330', '-6.34099426292932517052842145079168749722040843707216521702003910488e+893026373', '-6.34099426292932517052842145079168749722040843707216521702003910487999999999999999999e+893026373', 84, 2);
    T('-3.788243011573114e+4976276', '9.674033380952095355809715713067684018023e-12395', '-3.78824301157311399999999999999999999e+4976276', 36, 2);
    T('8.915192482708376e+18', '2.7806035233905706977469706204632745700227054728865694310454466799400976394683745031e-193369', '8.915192482708376e+18', 41, 1);
    T('-9.5809460479117497641334451403209063098591357227342029578466353669436106e+78', '3.159544434770015272110500589801295541485497879e+95', '3.1595444347700151763010401106837979001510464757909369014086427726579704e+95', 71, 1);
    T('-7.102041271359958668277143773e-8615566', '-2.4141052183790730044039507093225151236949454904558337441910696410052546601477402822625249161631e-98895848', '-7.10204127135995866827714377300000000000000000000000000000000000000000001e-8615566', 72, 3);
    T('-9.26594451973963930061995117677680996800881962862200475625617213e+793', '-4.6408640621186299138254958217759673e-720631', '-9.265944519739639300619951176776809968008819628622004756256172130000000000000000001e+793', 82, 0);
    T('9.0694366982522091151816413e-3883936', '-9.9735173699461046286670772107586271812667510127728797384937025397120380518646e+4598', '-9.9735173699461046286670772107586271812667510127728797384937e+4598', 60, 5);
    T('-7.5107536239741459903789112650282051555494082690694186792e+247', '-9.0586019733405560706358483212824743681467071263e+75688', '-9.0586019733405560706358483212824743681467071263e+75688', 54, 2);
    T('1.6040140968028982011281124561160276993101426015417947782303220313041418751336436016215172e+91', '-9.208638198168632108264e+8751380', '-9.208638198168632108264e+8751380', 69, 3);
    T('7.7780991024000519106468793530854206073376284031627666104719167706730e+69', '-7.7353672832e+6037', '-7.7353672832e+6037', 88, 6);
    T('-6841436043362.0156751300841076463547097610747716331671228430430403267', '-4.82229023369092122360530435546200727499853200296e-10798', '-6.84143604336201567513008410764635470976107477163316712285e+12', 57, 3);
    T('6.09709e-87152', '5.4179261786897682822641776555932506022700613257492993336152623e+4431', '5.417926178689768282264177656e+4431', 28, 6);
    T('43247851324693410030082663736817919498461839196313211171076052854985336624766858879.6072', '6.566222396185736481e-985658796', '4.32478513246934100300826637368179194985e+82', 39, 6);
    T('-3.3482966504663507e+44676687', '-57981646192562341888639678722361511742423281169939098691038478782932233147940113795154.96177', '-3.3482966504663507e+44676687', 41, 6);
    T('1.454866787525097414823451472855501220039476283474210e-1638354', '-1.49688463631297577148e+542495421', '-1.49688463631297577148e+542495421', 50, 5);
    T('-2.68195979798085672827054787e+2091688', '9.886069724949e+809049926', '9.886069724949e+809049926', 63, 6);
    T('-341194630959293892718023957732575890697494684347463218.36784', '-4.1097672330684501628082449742002283459570037305663181341493192582271621488783582626e-751932223', '-3e+53', 1, 2);
    T('-2.802460343084290118984238813241708754593414166381344e-3906045', '-6.4849e-10', '-6.4849e-10', 64, 2);
    T('9.460446656479643662644685183254769533802624858e-385474', '7846810401', '7.846810401e+9', 89, 1);
    T('0.00814219612021405945697612667376857050670035580021410159599284919582986015105402233500943550', '-9.12959e+60', '-9.12958999999999999999999999999999999999999999999999999999999999e+60', 63, 2);
    T('4.445554517848191257358940464405945050747611724853131244129966996159788058751593041679591e+272', '-6e-4549', '4.44555451784819125735894046440594505074761172485313124412996699615978805875159304168e+272', 84, 0);
    T('6.0791430032271e+1678462', '-9.097002084251964633393786469209292390633578896487460633313783654995861031630576609662716e+48424309', '-9.09700208425196463339378646920929239e+48424309', 36, 1);
    T('1.934559222778733449041e-2266', '-1.2571406510564070833304926274234789914638696826446131461187636506602873830487377e-322', '-1.2571406510564070833304926274234789914638696826446131461187636506602873830487377e-322', 81, 3);
    T('67437334680799110732151378040808622713626755042146953.63327764261528907060157', '6.4e+5033', '6.4e+5033', 22, 4);
    T('-2853261787092524194545.06', '-2.253295306616760236620374001988677820772214995047610366997e+300430', '-2.253295306616760236620374001988e+300430', 31, 1);
    T('5.5256425071960014072795670174600677851131655617050000168106e+180047', '1.28184e+109309', '5.525642507196001407279567017460068e+180047', 34, 5);
    T('6.850472273181227830454315130550990636089393146892105551657745e-9783', '3.850080474040160258707655833739619814917711470044775765628465079478e-7516', '3.85008047404016025870765583373961981492e-7516', 39, 6);
    T('4.118117147767627066546921532526380208269143732217484476527017685995742941025376522979753e+9898984', '-4.39426162254834e+16', '4.11811714776762706654692153252638020826914373221748447652701768599574294102537652297975299999e+9898984', 93, 1);
    T('-6.266745537785030129303214907259794385958468811496442831826296904755661172980224762792761283565e+619809383', '1.6837515271280892267575393266635967081185742439965154328114473994968658e-127566', '-6.266745537785030129303214907259794385958468811496442831826296904755661172980224762792761e+619809383', 88, 2);
    T('7.6402321749743828796531545377099954803031623529e+9941', '1698345148842357870865.80893', '7.6402321749743828796531545377099954803031623529e+9941', 83, 4);
    T('9.6918676640320980321473895484225532573885450142835713449765158941431525641930646e-4237', '1.5887845116547311466067760434647531507386585412064507005794593824931061934098166717447327e+90', '1.5887845116547311466067760434647531507386585412064507005794593824931061934098166717447327e+90', 94, 5);
    T('-5.0909869530753257124721772113039444560453706141889383296137e+76337', '7.18159437099504287954958900978282858273497172e-2422933', '-5.09e+76337', 3, 4);
    T('3.28e-205496182', '478271814801519845353128266086350251536.44733', '4.7827181480151984e+38', 17, 3);
    T('8.7830788468989e-713802', '-6194181109692486962901260161933839118508591837', '-6.19418110969248696e+45', 18, 1);
    T('-8.27481816150250142314353685e+7080', '7.466202798210810369860078432364008352638384959094362366739371256756301038578e-35780', '-8.2748182e+7080', 8, 6);
    T('5.525293241616361315619728152220833741694793749286575e-33845', '1.875953881660034230146906e+96', '1.875953881660034230146906e+96', 79, 5);
    T('-3.554887239318735789753466762089781557253687049736e-41', '8.28306931071409532094134022815492547819e-4476425', '-3.554887239318735789753466762089781557253687049736e-41', 57, 3);
    T('-257639030395274562884203090783359279303356064796787946618675848917884.045', '-6.2444685582751431e-26893', '-2.57639030395274562884203090783359279303356064796787946618675848917884045e+68', 77, 4);
    T('-4.8467556520605859093215920337113058619827020746208722490328040715738768732499940188600e+984', '-9.0923778389014608895770638957356385939725921216473e-50613', '-4.846755652060585909321592033711305861982702074620872249032804071573876873249994e+984', 80, 5);
    T('-2.10087234817504259889030463406367195713034734215702429938094574795356706e-4930', '-9.926421470184568442917344478587660289832849077647456135306e-4098', '-9.92642147018456844291734447858766028983284907764745e-4098', 51, 1);
    T('3.354403737821113570523586872569505578667320769556965950035081366052847031202429e-707606', '-1.859911345421530471589930506943921889e+90035', '-1.8599113455e+90035', 11, 0);
    T('-1.238525574208330571820195813849844679458e+547285', '-5.0413617246642055581901454079739316684726399980403147999e-4964821', '-1.23852557420833057182019581384984467e+547285', 36, 2);
    T('-7.36841273816201480167306657517e-10', '-1.476747488905277010967671081922594468215527824801690224047497503637391185218374469980698796203e-485483801', '-7.36841273816201480167306657517e-10', 56, 1);
    T('-9.6644758576279652002262995172755852486832642670699933757933345938434850e-9833', '-2.79833641624698e-32717705', '-9.664475857627965200226299517275585248683264267069993375793e-9833', 58, 5);
    T('4.8483314330436297025745577314311e-38', '-1.3072147228695599e-59588240', '4.8483314330436297025745577314311e-38', 54, 4);
    T('-4.7868412579191688663262317361983381539706437402121836886003e-5345', '1.71446844717915132827170230517899705929567542859728662990617811e-695', '1.71e-695', 3, 5);
    T('5.00237554357290127969178990798722419530462426898091065438777196248453796661207706377240e+79110', '-8.13710177053471834912722922888696684163394875757547675004368719229612896854082846e-8625', '5.0023755435729012796917899079872241953046242689809106543877719624845379666120770637724e+79110', 90, 6);
    T('3.294062205540068276044288726830292412447150633137493687466091854031786213448446e-2776814', '-492007.6699901896321741782767336880679213145157804833561767156416109634142058', '-4.920076699901896321741782767336880679213145157804833561767156416109634142058e+5', 95, 4);
    T('1.75005899428683037988161846538346e-8089', '-4048968484629963181985789088464147905342653357120734749360843153772569553103.316', '-4.0489684846299631819857890884641479053426533571207e+75', 50, 1);
    T('561802651323945560677903099643817728906995039900950574670107353466.71837506166', '7.941522641632394711009557982738389615948019874081116709701556634247e-798776', '5.6180265132394556067790309964381772890699503990095057467010735346671837506166e+65', 92, 3);
    T('-3.1277072690561838503461990030059894270696851983584715221536392786233258587468e-354638395', '2.429269150425802929781557741879122722017546280359e+49', '2.429269150425802929781557741879122722017546280359e+49', 52, 5);
    T('3.32930649190827906496054504760864577052706782163006730851998340e+52062833', '2.09625404074325298406078532e+650881232', '2.09625404074325298406078532e+650881232', 39, 6);
    T('-6.761894658652207991078462579177504659583264097280873534820883721300427568113817065662161901e+3186896', '5.602379935097392526575934315324e-122', '-6.76189465865220799107846257917750465958326e+3186896', 42, 1);
    T('1.807308005404847270850284024818482169754168252880842033979440756513791135292219554949875e-9350', '-5.455791133851567120178056872646189953608798490304528600160615820710601852390207644725e+970416476', '-5.455791133851567120178056873e+970416476', 28, 5);
    T('6.428584849852294e-1462250', '9.3618152363715780356910311157120626837565693958018526874020770751332515255530780304892219129358842e-7062339', '6.428584849852294e-1462250', 35, 4);
    T('773735293258888387581534817791525314908478369304860481506398.6', '7.6171669640255315506592422170083458999403766e-653831524', '7.737352932588883875815348177915253149084783693048604815063986000000000000001e+59', 76, 2);
    T('-6.2833238924507408488456579315880364270572540688886906024800423403426377974264e+781', '-3.4122664973930106759698031650157608170747998149633934785227399907308112550989e+132', '-6.2833238924507408488456579315880364270572540688886906024800423403426377974264e+781', 83, 6);
    T('-5.3351816303817602169019502241566905817367619352175613288962169e-1894615', '-7.94208728658193499590265666033558495655e+8694985', '-7.94208728658193499590265666033558495655e+8694985', 47, 6);
    T('-2.9365421380541287041906231157084439900009257100202029595890725709019126238998963284482e-58386', '2.578924636020556598570664612169870078393181320942214e+86076740', '2.57892463602055659857066461216987007839318132095e+86076740', 48, 2);
    T('2.89566710e+14', '9.62374186357727498982e+686891230', '9.62374186357727498982e+686891230', 62, 6);
    T('-4.82250755376636475502378708135128008414050997425744373482925299e-6784', '6.64104e-1108', '6.64104e-1108', 49, 0);
    T('8.690659164743954311016905692588359535251e-46', '-4.312231556045416224212824123125441046172025115158750122716586357e-49063', '8.690659164743954311016905692588359535251e-46', 86, 6);
    T('-5.8281342520141119814534068311583842446205e+4703604', '-6.305511058928623581975961367572e+52792983', '-6.305511058928623581975961367572e+52792983', 39, 6);
    T('2.80790401792496141831766676931523104046289340678120478705827303482969007988337197022681668346e+848', '4.0638e+9361246', '4.0639e+9361246', 5, 2);
    T('-7.6061823769927367336905399645075061405561118e-400768868', '7.35731399995137535449610276969686782278061901899994532170764930906627126817494745505e-62', '7.35731399995137535449610277e-62', 28, 4);
    T('-40895100950407155801194478587112383702522480710103898773400979827639665989.840', '-2.465650881979e+9614244', '-2.5e+9614244', 2, 5);
    T('-9.8043542e-1175', '-7.6057714150018058723737649570217222357032234862716575671132636631876708590540216419219e+5828', '-7.6057714150018058723737649570217222357032234862716575671132636631876708590540216419e+5828', 83, 6);
    T('-4.372728368889211727705898650429830927134335005952066693645615719325455663714370e+350', '-6226383691558122186581560914095647793630641723787834798146957.011', '-4.3727283688892117e+350', 17, 2);
    T('3.207933511845193235718955788913019552301151450476895e-114258', '2.68565745867167113416067121965827901142280702418437925161e-343', '2.68565745867167113416067121965827901142280702418437925161e-343', 90, 5);
    T('-3.746112e-66002691', '9.51688130186640839588228559334317411444143675117401691665e+671948477', '9.5168813018664083958822855933431741144414368e+671948477', 44, 2);
    T('-4.932e-9733', '-5.821375800285460090518202619946777022988520621642335263865152199033920131370e+77488766', '-5.82137580028546009051820261994e+77488766', 30, 2);
    T('-5.2605e-722635756', '-4.64720233225976771990728982681091344949817699632840796162009416932232725664e+352928551', '-4.6472023322597677199072898268109134494981769963284079616200941693223272566e+352928551', 74, 6);
    T('-1063125942167754380.4', '-8.58903787932135237494684199321950708576227664938449612395e-381710', '-1.0631259421677543804e+18', 20, 2);
    T('-8.69976158229846e-26', '-3.113521296908071535751844254936749703932656099912362301613935590362954588112e-770177696', '-8.7e-26', 3, 6);
    T('5.736844693258e+5658887', '4.8345e+84668052', '4.8345e+84668052', 51, 5);
    T('3.346593473954298212051074e+729268', '-6.281299475757681929378139123193122736883881385621e+733858', '-6.2812994758e+733858', 11, 4);
    T('3.0601252658248978788455970604575e+4293798', '6.39049231059866392610747659023252958859058426861168731287295658023805103748e-9415', '3.0601252658248978788455970604575000000000000000000000000000000000000000000000000000000000000001e+4293798', 95, 0);
    T('-1.1339787391048407643472084165420901787099445281093e-43671', '6.98232455077965270547971785735095380154043481784173e+7209', '6.9823245507796527054797179e+7209', 26, 0);
    T('8.4540257326273476205e-39', '-3.667977425420531261550177213910089373412435720939248891174322963586679396496117e-6337', '8.454025732627347620499999999999999999999999999999e-39', 49, 1);
    T('8.772760233378052897261449907248126468193751113143467718898e+727340', '-6.1112848546871862311445489917411958081910768574175902465081869656422012636753645105e-718368', '8.772760233378052897261449907248126468193751113143467718898e+727340', 72, 0);
    T('-5e+336553477', '-82715685007367658079991170308249023903832528120664386869288685.1', '-5e+336553477', 79, 4);
    T('-4.3688938781e+4111591', '-5.86480205434913613372724742743750177112224900358308054418299478164138e-4859', '-4.3688938781e+4111591', 67, 4);
    T('1.36003107066572695046414614315342771706819784060795248095328082769242712616065331209599915927426842e-900', '2.9203092961949406754546839748153213794469513256363021392068419941885091316196738770027244e+682483', '2.9203092961949406754546839748153213794469513256363021e+682483', 53, 3);
    T('5.198256687139619644731508893e-47', '-3.27131559e-54334450', '5.19825668713961964473150889e-47', 27, 4);
    T('-2.042452039086034388591e-251', '-2816828909675.341098', '-2.816828909675341098e+12', 90, 6);
    T('4.888913012126511521628546401197958268356405901046678738572206808455e-1543115', '-9e-94', '-9e-94', 74, 5);
    T('-2.202592078698662709639984470868062781374604212017529470331885621573606972458046281e+2264', '-4.693818866860794769970662e-3747610', '-2.2025920786986627096399844708680627813746042120175294703318856215736e+2264', 69, 2);
    T('5793233470204162021649683.840107', '9363548672818204810273870605243170751942606116416679432889993419689273319318707759101', '9.363548672818204810273870605243170751942607e+84', 43, 2);
    T('-1.59291904454825688731036538500615e+45', '-8.308155130408677321791e+14114', '-8.308155130408677321791000000000000000000000000000000000000000000000000000000000001e+14114', 82, 0);
    T('6.00725932442301188982102578232301442618021e+5462', '4.390028997963418599207180125043178103406836954e-236408', '6.00725932442301188982102578232301442619e+5462', 39, 0);
    T('4.1368905006718211064567342430809575e+66324', '-7569.7508586802966640416999984749105352323304999941110601327743893973246572160312800649737', '4.1368905006718211064567342430809575e+66324', 67, 5);
    T('2.4e+2', '-9.2e+3514125', '-9.19999999999999999999999999999999999999e+3514125', 39, 1);
    T('7.936493321153687e+61445', '-8.035013226890870923258185654338e-36786', '7.936493321153687e+61445', 47, 0);
    T('-5.4446626109818245913580979413195307007582273857838502533625784588229e-1130', '1.471262737116439530973662717967504330384648680396188502934415197508611829e+8768344', '1.47126273711643953097366271796750433038464868e+8768344', 45, 5);
    T('-6.57755585939626051814154725473670840761137404405282e-1470803', '-4.69015863501272e-600155260', '-6.57755585939626051814154725473670840761137404405282e-1470803', 72, 2);
    T('7.69362378282557773784168937286317488012364580087562226329934088189779864824275720862888301e-63668756', '7.769726327708347340937788166827980305726e+150', '7.769726327708347340937788166827980305726e+150', 69, 3);
    T('-9.10827681506e-64', '-2.14523098553301937476234282027505476449368001958425453262693649445278878069556914065e+122', '-2.14523098553301937476234282027505476449368002e+122', 46, 6);

    T('2.129234936304075078083102409313390493654153e-356275067', '8.16912240145919460021696336e+34087238431226', '8.169122401459195e+34087238431226', 16, 5);
    T('1.79764847868660712953293123556e+286568201721542', '-4.921162597e-15', '1.797648478686607129532931235559999999999999999999999999999999e+286568201721542', 61, 1);
    T('-2284960305904808875357008032975611.972505542645545050', '3.866165567363110435888134726361897047666853540506240e+470433534', '3.866e+470433534', 4, 6);
    T('-3.10040060472e+419626188', '-3.89332149242369205777049839616707864957e-800840', '-3.10040060472000000000000000000000000000000000000000000000000000000001e+419626188', 69, 0);
    T('-2.895e-9065', '6.70649091359762936924825186133e+111', '6.7064909135976293693e+111', 20, 2);
    T('8.94768192575840277584309035422994206626938e-62604915', '-7.90994346763075373928885758289301711e+5507', '-7.90994346763075373928885758289301711e+5507', 58, 5);
    T('-1.3613459138756462394991526729196669e+554394087', '-7.1e-60817764488172', '-1.3613459138756462394991526729196669e+554394087', 40, 6);
    T('4.2e+96', '5.93454941527902524615187828172446017320869e+3131939185483', '5.93454941527902524615187828172446017320869e+3131939185483', 86, 3);
    T('-5.047107966343184142750590797520665839825e+464650', '-5.6485404802794953664786e-744744504', '-5.047107966343184142750590797520665839825e+464650', 99, 5);
    T('2.38363300451e-31065544222144', '7.69230039001240031478361780193752655721e-218655473', '7.69230039001240031478361780193753e-218655473', 33, 4);
    T('-1.754548152158599e-2201478682531', '9.56e+330265376164', '9.5599999999999999999999999999999999999999999999999999e+330265376164', 53, 3);
    T('-9.85353078175342676236221000903000811741032e+66709', '-9.9621070524632745532567129149739981496593514349987125261281e-6223746494359', '-9.853e+66709', 4, 2);
    T('-7.59657746e-650729', '3.11615661626536668939228456904003419897933995316032702876e+206743', '3.1161566162653666893922845690400341989793399531603270287599999999999999999999999999999999999e+206743', 92, 3);
    T('-9.3219729738090986524065434374782126507362834380e-572605', '-3.41903813641209591037445308454638979912745e-709078842192', '-9.321972973809098652406543437478212650736283438e-572605', 92, 5);
    T('3.96451591120441352037524e-1423913454586', '-1.9e+539174962', '-1.9e+539174962', 6, 6);
    T('-4.526096605922230377711668919598417503545915864983490e-7215139', '3.423315668296346e-7944', '3.423315668296346e-7944', 31, 2);
    T('-6.053435825886574142232424894e-5772', '-5.5388118350391185680008139854897623366007847145861704283108e+773937493', '-5.5388118350391185680008139854897623366007847145861704283108e+773937493', 77, 6);
    T('6.246600177501377529300e+3082866857308', '-7.08996682808e+17724851522962', '-7.08996682808e+17724851522962', 26, 3);
    T('-5.512905837687347639176286284740366022621048e-38820824772390', '5.07752527229521381152118122704740e-512', '5.07752527229521381152118122704739999999999999999999999999999999e-512', 63, 3);
    T('8.832312867732510085454342907086226982e-262007528', '6.22364509e+552280', '6.22364509e+552280', 42, 6);
    T('5.0394690389590907688626139447533344709365314179771723035712e+121', '7.80834989097710066732293063499e-2364984366', '5.039469038959090768862613944753334470936531417e+121', 46, 1);
    T('2.218455516576157515900819007e+80', '7.530954026913880827488413863727598878400042994e+5397465144349', '7.530954026913880827488413863727598878401e+5397465144349', 40, 0);
    T('-51401005230956054575.763', '7.37509267588016e-564486440142158', '-5.1401005230956054575763e+19', 44, 3);
    T('8.92687459579270837324855359917828865729998348286154788620873e-815956', '-5.40356201759676527468774560577179133277638262e+96260916670', '-5.40356201759676527468774560577179133277638262e+96260916670', 80, 6);
    T('8.39366149074786e-56586020852491', '5.176416513272721896605635167160166947e-582196643', '5.176416513272721896605635167160166947e-582196643', 46, 4);
    T('7.9790249e+3372414968886', '3.62464869845020687515e+92351379', '7.9791e+3372414968886', 5, 0);
    T('-5.680064239650201221e-449', '9.09323285243788702560163027831002095900266676610175154637161e+9136', '9.0932328524378870256016302783100209590026667661017e+9136', 50, 1);
    T('4.9041720013314453653437279423128671047276369638748933926565e-411020554568', '7.820403208995430758547541e+51445266662', '7.820403208995430758547541e+51445266662', 58, 6);
    T('-5.6244957896252e-16368576023432', '7.592346896541823611031081327554446941700575071001433960e+7178123582', '7.59234689654182361103108132755444694170057507100143395999e+7178123582', 57, 3);
    T('3.8159990104325091344141270e+233332618743992', '7.79865159320361652289531716516390876119962818583181000e-1969449', '3.815999010432509134414127e+233332618743992', 81, 4);
    T('2517332082063165219711664853854346745595385.488961', '-5.7751268212948278448862052535e+23343093', '-5.7751268212948278448862052535e+23343093', 60, 3);
    T('-2.4745330438897531145242111366679919610590e-6024384038697', '-8.28684944615584283e-72329135835438', '-2.474533043889753114524211136667991961059e-6024384038697', 80, 6);
    T('-4.35624e+276018395796169', '-7.60838424288589885184827498301598954181768596043142370060e+203262731', '-4.35624e+276018395796169', 24, 4);
    T('6.074881662528587600671595570469267772110313745604e+9920083', '-1.4692e-954625501', '6.074881662528587600671595e+9920083', 25, 3);
    T('2e-91114', '6.277944863415963943853e+77465782', '6.277944863415963943853000000000000000000000000000000000000000000000000000000000000000000001e+77465782', 91, 2);
    T('1.48597494742571021738468674611003952949794561292e+946530835563', '6.198573409752259093369372527147355356113979507212660431955e+58', '1.48597494742571e+946530835563', 16, 4);
    T('8.6901939306657493052442154186676858e+126963070842906', '7.05811873837926818893492813248715498e+385037219673', '8.690193930665749305244215418667685800000000000000000000000000000000000001e+126963070842906', 73, 2);
    T('7.22630401628627450646840793814983652614061236160529e-47', '-3.74323536767745e+8479059209644', '-3.743236e+8479059209644', 7, 0);
    T('5.856479610961198046063262481994545178e-26085', '9.452313181062258120314515211604533860681e-86890862380', '5.856479610961198046063262481994545178e-26085', 69, 1);
    T('-3.5e-58810942', '-6.6881463e-54909', '-6.68814630000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e-54909', 99, 0);
    T('-2.823564667201572e-7500595514', '-1.60776464010628132e-234796134483', '-2.823564667201572e-7500595514', 92, 5);
    T('-4.67446924468406e-63', '1.12076266285e+480767', '1.1207626628499999999999e+480767', 23, 3);
    T('516525780922750189145812522978586644239009.2999572', '8.62138109180657562907230604757430348575016628381700435882262e+5511467', '8.6213810918065756290723060475743034857501662838170043588e+5511467', 56, 4);
    T('7.44092053222e+35801', '4.85563897383332597159797320333493561056079738017450e+896114470557198', '4.8556389738333259715979732033349356105608e+896114470557198', 42, 2);
    T('-9.142392058450745716581e-82977254350399', '-4.25239355649815126e+398740811682529', '-4.252393556498151260000000000000000000000000000000000000000000000000000000001e+398740811682529', 76, 3);
    T('5.4254594157e+78026765', '-9.3781694770611269020423248e+8002742619229', '-9.378169477061126902042324799999999999999999999999999e+8002742619229', 52, 1);
    T('1.9551412277e-52008079', '-5.01872262018044994185760996432269347620073234564452797878187e-7772', '-5.018722620180449941857609964322693476200732345644527978781869999999999e-7772', 70, 2);
    T('-4608133134572825618701783851455349.64504', '8.308644187872585542476340652e+76771183', '8.308644187872585542476341e+76771183', 25, 4);
    T('-5.10338088508565e+104545691281327', '-5.796595468337393959088680e-677', '-5.10338088508565e+104545691281327', 64, 5);
    T('-1.9207182899699131493468523090e-31', '4.1263160131785e-42', '-1.92071828992865e-31', 16, 0);
    T('-1.56772954306736332772244154647619473444995248354184512e+36942162', '-3.3295940165650577488e-108469843586', '-1.56772954306736332772244154647619473444995248354184512e+36942162', 64, 5);
    T('-4.464705742132817190e-454572', '7.749020086594482367227707152080e+37', '7.74902008659448236722770715207999999999999999999999999999999999999999999999999999999999e+37', 87, 1);
    T('8.18744480245498874229312545941647459099121365171507687e+432426943', '7.6172700533214655850e+3055', '8.1874448024549887422931254594164745909912136517150768700000000001e+432426943', 65, 0);
    T('-1.8814458016222362191397071995801846471e-6756759678', '2.3895562995633495576178345671469650361140467105445690826697e-8060447871', '-1.88144580162224e-6756759678', 15, 6);
    T('9.31633494653954777820009099049831154608260935e-26129149303963', '-9.896144053837326e+260325923457815', '-9.8961440538373259999999999999999999999999999999999999999999999999999999999999999999999999999999e+260325923457815', 95, 2);
    T('9.146668146372992134526389354204401866408103816732516e-3026', '9.5502851000916483460176657075532877520988577507571583287768e+577537126644', '9.5502851000916483460176657075532877520988577507571583287768e+577537126644', 75, 5);
    T('-5.550981788262402103087824454179188156455885114257771909135e-694079', '45561077176402309.84791782920400402386377749761', '4.556107717640230984791782920400402386377749761e+16', 51, 4);
    T('7.9614520937367068618534448283736169455123e+24317533214', '3.3519655870112515036172524876012854544269e+233748', '7.9614520937367068618534448283736169455123e+24317533214', 92, 1);
    T('9.265854082e+2647', '-2.0476889139950463674265546252969359379813969482836139619e-88429', '9.265854082e+2647', 60, 0);
    T('-9.803146639e-4420560302', '-3.2095089405e+90006880', '-3.20950894050000000001e+90006880', 21, 3);
    T('3.2404458099752079242010066665e+331275', '-6.35466646590000083891877180772e+2505', '3.240446e+331275', 7, 4);
    T('-2.956266253579955382154520552907960997609e-4406118266', '7.76032995612045621336e+23', '7.7603299562e+23', 11, 0);
    T('-5.07042634558029211291346299587611e-107', '-8.797e+6433919', '-8.7970000000001e+6433919', 14, 3);
    T('4.015667315796617644995412008363245434863622283e+6132', '-2.9279343287890797435020505279e+43534332', '-2.9279343287890797435020505279e+43534332', 63, 0);
    T('-5.6e+62166896106', '7.51266166030869098793420937010145178e+8657', '-5.6e+62166896106', 23, 0);
    T('1.71988932613561e+135513913418869', '-1.00040534e+7472617788', '1.71988932613561e+135513913418869', 93, 6);
    T('5.0e+6977376490133', '-8.046969914324237936056441e+864369642', '5e+6977376490133', 57, 6);
    T('-5.2973781839673674765014971e+418134993217874', '3.125765213e+11', '-5.29737818396736747e+418134993217874', 18, 2);
    T('2.83478451949437955970641979e+9191392', '-7.7021485111225964879881629e+902761652', '-7.7021485111e+902761652', 11, 4);
    T('-9.61074183481339951747947408332923143344675146390599616968962e+96304507322', '7.6224826600336451499906063270730779411e+34808', '-9.610741834813399517479474083329231433446751463905996169689619e+96304507322', 61, 2);
    T('-1.3386816811069933396694296552140216278e-9712742262', '-5.178466405715177854361716095807096954e+40', '-5.178466405715177854361716095807096e+40', 34, 1);
    T('5.4303603363574055521371e-273312', '2.22666056155159820268306461037417007e+8511003', '2.22666056155159820268306461037417007e+8511003', 38, 5);
    T('-6.84418820133e-3025946805459', '2.081e-85103554197586', '-6.84418820133e-3025946805459', 20, 4);
    T('-5.06740656e-2463544022', '-3.12438249827213e-43166831271374', '-5.06e-2463544022', 3, 2);
    T('-6.79709229048167262e-2481115', '-9.556108302382109758501473511991959589765155050397e-44', '-9.556108302382109758501473511991959589765155050397e-44', 75, 2);
    T('3.363893804793821877081197817003891608723106e-6679', '-1.5160688392882758622613907e+6781984997', '-1.51606884e+6781984997', 9, 0);
    T('4.8433029993918340643501233884400980243689267588120920994941e+9532712596858', '-3.050081346385337175e+923553852979', '4.8433029993918340643501233884400980243689267588120920994941e+9532712596858', 93, 6);
    T('-9.7899143307669916981043685502e-428750288589', '-4.351791347816313226561496536154596412e-3504127', '-4.351791347816313226561496536154597e-3504127', 34, 3);
    T('6.98746982566e-785456919837', '-1.642299352363733373907395994799878311629402358481442973e-7', '-1.642299352363733373907395994799878311629402358481442973e-7', 78, 0);
    T('-9.641736463642103032883e-174', '-9.4544500799399870e+29', '-9.454450079939987e+29', 87, 6);
    T('1.0315645876495185486034397257933261064373828e+986859386', '-9.27071125373009299624878828999713847328018329803396602904654e+422921360378165', '-9.270711253730092996248788289997138473280183298033967e+422921360378165', 52, 0);
    T('9.5819001027862302756188337686279141e-19949732', '7.88238410680273430765018441e-50', '7.882384106802734307650184410000000000000000000000000000000000000000000001e-50', 73, 2);
    T('-5.8469933e+14', '3.844731507642355067355060052847978e+429', '3.8447316e+429', 8, 0);
    T('-8.7819461463749344946025124271325271506606791691221141228426e-23093578', '1.3679656041539718244628466595146933082149205643458098e+39745543', '1.3679656041539718244628466595146933082149205643458098e+39745543', 77, 2);
    T('4.151215115172030180069122654244e+91', '-7.11205989865037036625301155457e+60910014235369', '-7.11205989865037036625301155457e+60910014235369', 60, 4);
    T('-9.7092371826743022623917316252473603327023631e+1443295588826', '-9.917569e-87388768797466', '-9.7092e+1443295588826', 5, 4);
    T('1.991299524214606103151e+2936', '-5.44988106700e-5115446486', '1.9913e+2936', 5, 6);
    T('-6.6983769594517605935050567847106748914793e+71855', '642487955198028373267644626421267549297.8', '-6.69837695945176059350505678471067489147929999999999999999999999e+71855', 63, 1);
    T('-4.53226460957960532634156999293219965e-569385433743206', '-8.175819155493943500406279765341795733810953432e-45243015', '-8.17581915549394350040627976534179573381095343200000000000000000000000000000000000000000000001e-45243015', 93, 3);
    T('-8.712966225259223252762047425365461531674346884668217882e+32260374', '-5955547662786627359.805713197080640094422592956251453', '-8.712966225259223252762047425365461531e+32260374', 37, 1);
    T('4.455971098358872366931324025986774723308e+104550', '-1.377575508937046648e-1128480005', '4.455971e+104550', 7, 6);
    T('-5.5094829515788444e+190472785', '4.415227643469640640584386038e-4381594', '-5.5094829515788444e+190472785', 99, 0);
    T('-4.082724516638097035275110087e+916572304', '-8.835793186078976010e-63158904', '-4.082724516638097035275110087000000000000000000000000000000000000000000000001e+916572304', 76, 0);
    T('-7.77e+150931625679373', '-4.78153753681357823320015575350110660925083519e+57915775957778', '-7.77e+150931625679373', 83, 1);
    T('4.005480523265886494853e+7915839', '1.0962e-411077296930483', '4.005480523265886494853e+7915839', 51, 5);
    T('9.05870e+6686221292592', '2.097352855e+968', '9.0587e+6686221292592', 51, 4);
    T('-9.617652117813904475034988499183979e+255102866884749', '5.210328512542131156095024351018528031030198e-5093', '-9.61765211781390447503498849918397899999999999999999e+255102866884749', 51, 2);
    T('6.597124090704465401862461039597e-528337275561613', '5.7e+8146480766', '5.7e+8146480766', 2, 3);
    T('2.726783736528132902486025124534571909008311108140438e-97971898427', '-3.3189e+707615901941', '-3.3189e+707615901941', 16, 3);
    T('-2.8831444207551977082484280605812786387123381742592e-49', '6.54996321135417922726651002511592914282434963e-19667114', '-2.883144420756e-49', 13, 3);
    T('3.475373080813801346903622295988589673820747219e-57755367199', '7.997505576068e+32464363348936', '7.997505576068e+32464363348936', 100, 5);
    T('-7.29356641037717063635356338041340718002826732283177136308328e-78048372', '-8.303945268826463515977625707781e+175006983695', '-8.303945268826463515977625707781e+175006983695', 34, 4);
    T('-6.264716121092005839636125527239861188240885883007479e-62655', '-9.6629770134468249111319884633507167696578642426e-88115494069362', '-6.26e-62655', 3, 4);
    T('8.57e-35770754317895', '5482322229192118456992004027109343622342.5234', '5.482322229192118456992004027109343622342e+39', 40, 1);
    T('5e+2553727734480', '-9.6694419603e+902003311', '5e+2553727734480', 63, 5);
    T('-8.69471016619169782214466e+99717201', '-9.7392504156729306465334547339761876031231123456306588483195e-3555440013057', '-8.69471016619169782214466e+99717201', 65, 1);
    T('4.7158851410071e+7279650', '-7.13920e-7919986735', '4.7158851410070999999999999999999999999999999999999e+7279650', 50, 3);
    T('-7.14031755279950761129192996152e+7376', '-5.726405690137231272295521753559184167e-19398792', '-7.14031755279950761129192996152e+7376', 95, 4);
    T('-1.6094561042926473170843503775696422512081170271758604074899e-219199676107844', '3.865349806542417445413457536686e-274311170646035', '-1.60945610429264731708435037756964225121e-219199676107844', 39, 4);
    T('4.89850456983218816e+205', '7.98974e+510', '7.98974e+510', 36, 6);
    T('-9.329397723293916116e+31213018668762', '7.618890426472764764671990810257250644691410307e-13965084', '-9.329397723293916116e+31213018668762', 49, 0);
    T('-5.6316582225450036001e+319686', '-9e+707', '-5.6316582225450036001e+319686', 66, 1);
    T('8.5029586639314459272227682245801198736e-781', '-7.278701995246040397184480642854555e-51160747', '8.5029587e-781', 8, 4);
    T('4.9357447093446088342724454812852000922282548524163264e+996159252', '6.4506398e+609905792', '4.9357447093446088342724454812852000922282548524163264e+996159252', 86, 1);
    T('6.17633709632972911151424839139234904226e-705293184733', '6.3348019659865658264943159257958551941223864546e+37751781451', '6.3349e+37751781451', 5, 0);
    T('-2.20648653793656807053025606e+239613346', '9e+218761507', '-2.20648653793656807053025606e+239613346', 92, 3);
    T('-1.353318694526259984183083761224772362724168833e+3217660057', '-1.885530510858401196049031306e-417263243001', '-1.35331869453e+3217660057', 12, 6);
    T('8.85389757538410294880576066335194e+73', '5.8140707298310033005258136010347482021956960e+2158', '5.814070729831003300525813601034748202195696000000000000000000001e+2158', 64, 2);
    T('4.77516403771578249631883536696232369487e+848532', '5.8704028731606e+570556572682603', '5.8704028731606e+570556572682603', 94, 3);
    T('3.27507041876509576926658219386930731035521e+730802', '-1.866213408472804591963529321312e-195821526594944', '3.27507041876509576926658219386930731035521e+730802', 93, 4);
    T('-1.012e+920858600501463', '-8.1625110003767242737e+2956306457', '-1.012e+920858600501463', 32, 1);
    T('7.71008863763968856042453800056181194597256468357030e+855028326457933', '2.71544960760032165776168e-645624696448835', '7.7100886376396885604245380005618119459725646835703e+855028326457933', 67, 5);
    T('-4.652673891092403316189491266790207e+9025218657054', '1.962276112109800270273299416361919213e-906062971502', '-4.652673891092403316189491266790207e+9025218657054', 64, 3);
    T('-9.09941853214451194436e+36446', '6.4391763183565443e-737', '-9.09941853214451194436e+36446', 65, 6);
    T('-5.53172329643629818733504535163879e+66308482676', '-7.0743701574090901385990979462202599285868e+101', '-5.531723296436298187335045351e+66308482676', 28, 2);
    T('-7e-89996289705126', '4.6281008302813327797590715007298001e+68739335', '4.6281008302813327797590715007298001e+68739335', 53, 5);
    T('-7.20412384895376482182267731e+153928', '8.2052006526918e+2382185', '8.205200652691799999999999999e+2382185', 28, 1);
    T('5.9966215710605798244354050980531080032263e-28', '3.59496422755769553371785e-56418613639', '5.9966215710605798244354050980531e-28', 33, 3);
    T('-4.59789176517526047731943899067926e-10022298979', '2.252247949138874282095399588073768799097064e+3313', '2.252247949138874282095399588073768799097063999999999999999999999999999999999999999999999999e+3313', 91, 3);
    T('1.022839590581326781726163500971923877829334e+162', '-3.61509800827394335688891566309015782e-52', '1.022839590581326781726163500971923877829334e+162', 66, 6);
    T('6.68019303849821664646570029447065123e+238999405', '48114211196466144.868', '6.68019303849821664646570029447065123e+238999405', 65, 4);
    T('-9.25247196719051327e-8540', '2.1004717795e-200', '2.1004717795e-200', 21, 5);
    T('-9.58925320592801833495204278290826e-625705', '8.3022411897504782784823089921768063610564490e+44', '8.302241189750478278482308992176806361056449e+44', 43, 4);
    T('9.88738021757317785e+550499591459844', '-6.488268e-46583', '9.88738021757317785e+550499591459844', 87, 5);
    T('-2.2525941220615284446075537844479212509100798929e+254831468191814', '-2.2896593291e+638511312936', '-2.25259412206e+254831468191814', 12, 5);
    T('9.75327957e+89', '-2.9305401823353122720464725316150648054832958220792839114895e-61223', '9.7532e+89', 5, 1);
    T('-1.62428458723493979867742418670806872645131e+27316077543', '4.93730354914678e+977270', '-1.62428458723493979867742418670806872645131e+27316077543', 62, 0);
    T('1.884531254259804403686545e+114389299716', '-3.0942e+44', '1.884531254259804403686545e+114389299716', 41, 2);
    T('6.26876675662715154173065e+786710869380', '4.337060452809517629844965389033793701856445573784e-756', '6.26876675662715154173065e+786710869380', 79, 1);
    T('9.3266750150217723998576781881632637e-6157647773364', '9.632778910358858405744963755220479692107040809551e+55956532', '9.63277891036e+55956532', 12, 5);
    T('-4.70853645235496558284273357837401782556695935811767895730469e+60', '7784969098520953683259083', '-4.7085365e+60', 8, 4);
    T('-2.28787454707061280538648243959425e-106665102913', '-0.0000045615668968448248856314334866981726148762541621045136', '-4.56156689684482e-6', 15, 6);
    T('9.151657746724309353356082804661039959629340943e-721588', '1.7431283255025792089232563998771239099e-88097', '1.743128326e-88097', 10, 5);
    T('3.21836737996651256071969838095633430e+8603740', '-7.372115087350964445530900e-406173', '3.2183673799665125607196983809563343e+8603740', 97, 4);
    T('8.9022013955919045539787309e+676039440599', '9.49570852680469465314378004973e+9747', '8.9022013955919045539787309000001e+676039440599', 32, 2);
    T('9.369126165320072503428939545147892775347609201993787200e+952', '-9.7793262311406112288656769923e+307930', '-9.7793262311406112288656769923e+307930', 97, 3);
    T('-8.40027089837015702072268418716114173e-271', '-4.4801644937819731453118438762995206278965129174364908985e+80', '-4.480164493781973145311843876299520627896512e+80', 43, 1);
    T('1.11428002292951633931416633995612591657546941957234715388116e-262', '3.3967050492489271840398411018607479e+43', '3.39670504924892718403984110186e+43', 30, 5);
    T('2.89848192606757400887458860805678116352277645e+6820', '-1.69378739847775573183052931584507866784e-9135', '2.89848192606757400887458860805678116352277645e+6820', 82, 4);
    T('3.58957e-5010', '6.1081710112e+8040771509', '6.1081710112e+8040771509', 64, 6);
    T('-5.4526067820534222384913187729013934108999224862010878e+6398311', '1.55123685356974e-5321245766480', '-5.4526067820534222384913187729013934108999224862010878e+6398311', 100, 6);
    T('76833.00', '4.50487467895667259100162129697e-5104', '7.6833e+4', 30, 3);
    T('-6.3227510537747212e+7816', '-9.9088384790465677146517397235633872e+3159237601', '-9.9088384790465677146517397235633872e+3159237601', 68, 1);
    T('-7.4347361235065987914611501788073566e-531', '-4.507618595277959821587077561939933e-422055253909905', '-7.43e-531', 3, 6);
    T('5.4835550664376998409571595975992472336341694362700012250257e-3038811144498', '2.264e+5687755747210', '2.264000000000000000000000000000000000000000000001e+5687755747210', 49, 0);
    T('6.20274e-65239057592791', '9.4176e+747', '9.4176e+747', 72, 4);
    T('6.78177661784385013251701369553393602713031001e+107', '-1.1853e+84884169545', '-1.1853e+84884169545', 14, 0);
    T('3.41754293347763021828365420648096655e+37', '-5.7971775097e+833712240373', '-5.7971775097e+833712240373', 20, 3);
    T('9.5953660762704e-473829472749', '3223000474.4511148', '3.2230004744511148e+9', 59, 5);
    T('-9.23494616672626990398794064073844989749296604783275693104734e-184378512390908', '3.32141054522703047748e+6802190', '3.32141054522703047748e+6802190', 98, 4);
    T('-8.219e+7704191006', '1.931602e-207948992', '-8.219e+7704191006', 14, 6);
    T('6.3e-3639117440', '9.241666432376778e+357274004205', '9.241666432376778000000000000000000000000000000000000000000000000000000000000000000000000001e+357274004205', 91, 0);
    T('-6.8e-8138', '-3.34906855811965576525029535462883556e-729853551389072', '-6.8e-8138', 81, 2);
    T('3.7161107777173501303060341247942977671154e+55081355977174', '-8.38182608282749316813823682149e+89887', '3.7161107777173501303060341247942977671153999e+55081355977174', 44, 3);
    T('-7.12685587331540711e-5760182268233', '7.7839613188163772421841855568122394146851922e+969024', '7.7839613188163772421841855568122394146851922e+969024', 57, 6);
    T('4.25952818323829e-608966902965', '-7.4628545182412329140772878337e-277613667647', '-7.462854519e-277613667647', 10, 0);
    T('745582640344503476551905434.58803872', '-5.130252010222954215600905293529621e-4363', '7.4558264034450347655190543458803872e+26', 58, 5);
    T('-4.49664888e+15', '9.5591475783916701242239333558087482966e+34962188', '9.5591475783916701242239333558087482966e+34962188', 60, 5);
    T('6.9198578966690559593691158e+95', '-7.2819393547749e-91644507100', '6.919857896669055959369115799999999999999999999999999999e+95', 55, 1);
    T('6.201281347026919583071699956e-52013635', '-1.948655e+36703944256988', '-1.948655e+36703944256988', 12, 4);
    T('-7.365745803288596115229729e-76266', '-5.493360756686345189425448104549332825e+69393', '-5.493360756686345189425448104549332825e+69393', 65, 5);
    T('-7.734311980843096615207182891766050516692988050278142957e+148', '-6.39600547039564839830432938222855367587478641e+7803', '-6.39600547039564839830432938223e+7803', 30, 6);
    T('-1.4418465053433741813692375585e+56524', '-6.40596507897542663643e-82614639822608', '-1.4418465053433741813692375585e+56524', 82, 1);
    T('-716828305717692911080163291770459290195.19473470243124020', '7.033825033206913787245445108959636579742e+74201036258934', '7.033825033206913787245445108959636579742e+74201036258934', 56, 2);
    T('-4.86344234395418012875266955599680341e+8380898511', '-7.877007381e-39', '-4.86344234395418012875266955599680341e+8380898511', 90, 5);
    T('3.8461133079625256877832122467167356e+782236', '2.121568700782e-575710', '3.8461133079625256877832122467167356e+782236', 64, 4);
    T('9.8071117465809395430722571307834555754e-406925811', '-2.87695720877011555361759844e+87', '-2.87695720877011555361759844e+87', 61, 4);
    T('1.9303146619e+9251599', '6.866432874776067955649202969048e-6967431761905', '1.9303146619e+9251599', 67, 1);
    T('5.599362910957779783003946642286481394201e-223456', '-4.260225361598843748080674095496416440e-416605', '5.5993629109577797830039466422864813942009999999999999999999999999999e-223456', 68, 1);
    T('8.151e-31665', '-4.927533956633680479975809898260196066007732144769911e+797', '-4.92753395663368047997580989826e+797', 30, 4);
    T('-6.389006724105971072132588586769e-2841256710001', '-5.629092676832159960704040445715075558736600458e-3906947', '-5.629092676832159960704040445715075558736600458e-3906947', 64, 1);
    T('-4.67304564554667697211124781663755085e-9311833432', '-9.37511485e+6577691', '-9.37511485e+6577691', 25, 2);
    T('5.4616819502323453415819870868299119e-201', '-9.055382204756705415462599421429e+15249445', '-9.055382204756705415462599421429e+15249445', 84, 0);
    T('-9.58040295020528186930062312810491444e+84018169', '7.7708844601397017925e+385047290531', '7.77088446013970179249999999999e+385047290531', 30, 3);
    T('6.3667495801078945524536292927850213887495596514852e+978831936', '2.1096068967932632716051198542398050194176618495734e+84153249', '6.3667495801078945524536292927850213887495596514852e+978831936', 70, 3);
    T('1.5168057026140266886176137809477672360809111030e-1016623174', '5.3061282945191762140516e-35870', '5.306128294519176214051600000000000000000000000000000000000000000000000000000000000000000000000000001e-35870', 100, 0);
    T('2.95917945195546205e+93047', '-8.51819228584919441273176183227630681e+2709098297286', '-8.51819228584e+2709098297286', 12, 1);
    T('-6.54062766297352544393166985214223158859e-394825', '5.478403670037078085825591786051662956683873e-358304', '5.478403670037078085825591786051662956683873e-358304', 78, 2);
    T('-1.336418686175e+718', '1.623809743507304271821654e-55586625', '-1.336418686175e+718', 80, 4);
    T('-2.474777437080581857270166705363401422399788740216e-450541739832', '-7.657123338404545399717633650294953e+90173', '-7.65712333840454539e+90173', 18, 2);
    T('8.73769584165367135697085803976876143901129569e+119', '-2.29911239577807305155369653264e+90371', '-2.29911239577807305155e+90371', 21, 6);
    T('7.88629928699335389073443562380014479403e+9643234077', '-6.30166364310378377814912444238785168604397276092882e-28621973', '7.8863e+9643234077', 7, 0);
    T('-9.049453550e+10509454690735', '-6389248541118016501929839983393890038017500', '-9.04945355e+10509454690735', 67, 6);
    T('2.3963482979313629504720460788e+252', '6.37254312864072556270547729e-59235511175', '2.3963482979313629504720460788e+252', 42, 3);
    T('-2.256934105116305553235785667978636469845228042519152e-6780258590', '1.088692e+9557707585480', '1.088692e+9557707585480', 57, 6);
    T('-8.690e+71253532', '-1.445371739669392606e-33069', '-8.69e+71253532', 51, 5);
    T('3.14434588151392278135713786075896643e+37', '-3.28850683854304376648919549178148397325959e+31896986403', '-3.28850683854304376648919549178148397325959e+31896986403', 64, 5);
    T('6.2031316433618788e-605215454', '8.581587864913591076015990562929241203752766429285638103e-9861', '8.5815878649135910760159905629292412037527664292e-9861', 47, 1);
    T('5.143277411456728988562827355626198e-89318758', '8.84529521e-7644186053513', '5.1432774114567289885628274e-89318758', 26, 0);
    T('4.934320940028165688251626369040560842733824814e-1756649', '-5.699678043110245661087204951388459166502974e+65304889', '-5.699678043110245661087204951388459166502974e+65304889', 66, 5);
    T('-4.533e-165320704', '-5.8340392645315547311723144429392690995371977573410141e+388435410313239', '-5.8340392645315547311723144429392690995371977573410141e+388435410313239', 76, 2);
    T('-1.71476263087204782e+269', '-5.63e-4948', '-1.71476263087204782e+269', 40, 6);
    T('-8.2500333998540526767986332213055781992904009828e+9466847', '-8.9606494993289890768281092765416324286444381316168e+10250', '-8.2500333998540526767986332213055781992904009828e+9466847', 77, 5);
    T('5.20780548492483915568410666539524388489307e-558829', '-1.353632879326985799385976054056e+178751581', '-1.35364e+178751581', 6, 0);
    T('36191048481745340771155563096665547201604794431501805455.310', '-3.29307852549422120632023407384e+9262070491', '-3.29307852549422120632023407384e+9262070491', 83, 0);
    T('-4.71808683281578140149399532823493666564223e-683', '3.361524452276117764587507385068051793539533966990783e+14512893', '3.361524452276117764587507385068051794e+14512893', 37, 2);
    T('-264710343920444814762.511403', '-9.179643996238447992181602104924442e+36', '-9.179643996238448256891946e+36', 26, 2);
    T('3.286605563e-958468065760860', '-8.562e+13488303', '-8.562e+13488303', 43, 0);
    T('-1.360722679189180043789226305116379391890545512e-6454876016456', '-5.2989353769420965609391670614119554046295515135896416873419e-611158384108', '-5.29893537694209656093916706141195540462955151358964168734190001e-611158384108', 63, 0);
    T('5.4997836612947972722294083139346797788626919e+45', '-3.1724297246282480243349664849998813095316e+943483', '-3.17242e+943483', 6, 1);
    T('9.6271e+77', '-5.90243e-53135264994252', '9.6271e+77', 89, 0);
    T('-1.51145043365455e-7789920810', '-3.12016840744747045987368330367911261343778794884073232523034e+470573815790', '-3.12016840744747e+470573815790', 15, 6);
    T('-9.888851731571551243e-460922', '8.173968878955726645909476e+718', '8.173968878955726645909476e+718', 89, 2);
    T('-1.7873689002857552999073847155306418360e-185468688', '-3.2820265244059628575156745427946514911985775496052028872465e-9835965', '-3.282026524405962857515674542794651491198577549605202887246500000000000000000000000000000000001e-9835965', 94, 0);
    T('1e-46357411581247', '8.1214623200455236e+765339', '8.121462320045523600000001e+765339', 25, 0);
    T('9.152128754906084841022526292183742062472955061e-942', '7.9422789200007411241956e+4900974703666', '7.94227892000074112419560000000000000000000000000000000000000000001e+4900974703666', 66, 0);
    T('6.914482987656038020262515728538013e+4784617', '3.39388140700140247219970305e+57284955020457', '3.39388140700140247219970305e+57284955020457', 45, 5);
    T('9.54368848700196481079168e+135374604', '3.3975255855445300822054631772216876928e-462299816264', '9.54368848700196e+135374604', 15, 3);
    T('-4.29511338323184723872650906566e+9648', '-1.2390909462509526205370327356294715692433632774766979e+190', '-4.295113383231847238726509066e+9648', 28, 6);
    T('5.6933976e+3842155335', '-4172470448505281464287012294161078362', '5.6933976e+3842155335', 37, 6);
    T('7.505781685932597210395172826436e+10781364155943', '4.491925853247378351952190e+73439563699011', '4.49192585324737835195219e+73439563699011', 76, 3);
    T('-4.054935018670213393893938827250665552079816935548928e-315', '6.6630e+7622091632', '6.66299999999999999999999999999999e+7622091632', 33, 1);
    T('-9.3001933394747e-66400', '-6.774544934505302347276811371046445630034396908872681e+3168575896', '-6.77454493450530235e+3168575896', 18, 0);
    T('554998112186226945786.5564', '8.188398786674313352628185899866218440e+803845885937', '8.188398786674313352628185899866218440000000000000000000000000000000000001e+803845885937', 73, 0);
    T('-8.796498754342e-2761', '9.53522493643e+445279', '9.53522493643e+445279', 12, 0);
    T('-2.9364906e-92046', '-1.0082861325207721609525055331920e-564440787098127', '-2.936490600000000000000000000001e-92046', 31, 0);
    T('-0.000574003', '-5.121685e-3321654', '-5.7400300000000000000000000000000000000000000000000000000000000000000000001e-4', 74, 3);
    T('-9.899982698186857244814364456299110876254901e+8593911785959', '9.372464731162889843653467093e+905453411591', '-9.899982698186857244814364456299110876254900999999999e+8593911785959', 52, 1);
    T('7.075611357580320439467692019488886805337e+17086456119', '-7.87820711961261877e+317008996', '7.075611357580320439467692019488886805337e+17086456119', 75, 4);
    T('-7.6348032941279746e+21', '7.7458e-8927286474', '-7.6348032941279746e+21', 38, 6);
    T('-2.574354723329230864744249090150699390938611254313064913e-2260517792', '6.878167e-1356180252023', '-2.57435472332923086474424909015069939093861125432e-2260517792', 48, 3);
    T('-8.32423837990742780e-665423210785', '-7.03411994769528265179e-43178', '-7.03411994769528265179e-43178', 48, 5);
    T('4.17832652491905462870234936257e-981314066906', '-2.00191705095899780948063955280791e+816956', '-2.0019170509589978e+816956', 17, 5);
    T('-8.808621041e+22', '43738500631794477530957764299894.24', '4.373850054370826712095776429989424e+31', 87, 0);
    T('8.99027131243274234050892295735812385437e+48441772', '4.40063399398738e+9892355', '8.990271312432742340508923e+48441772', 26, 2);
    T('-8.6317032081558875989539006197195117419e+23629617', '-3.2918994283994e+633448383', '-3.2918994283994000000000000000000000000000000000000000000000001e+633448383', 62, 3);
    T('-3.652738609201730592689928366303307170830243551986519841802e+923726331', '6.952004471422350540889060284e-59747988666', '-3.65273860920173059268992836630330717e+923726331', 36, 6);
    T('-6.90e+31415203253', '-4.123264533245079332251003127103846986e-13324850997797', '-6.9e+31415203253', 13, 4);
    T('-4.7977104100211889788519008e+27', '1.35e-58161319', '-4.797710410021188978851900799999999999999999999999e+27', 49, 2);
    T('5.76847953787836998078809707217643665262242978891e-9687594584', '-5.57829291213553437608913388589561655208674196219086839e-18418', '-5.578292912135534376089133885895616552086741962191e-18418', 49, 5);
    T('-2.5735272191876804019631239170580e+6293', '-5.615094710189589376544e-774', '-2.573527219187680401963123917058e+6293', 61, 6);
    T('-1.6874412518437640973386867983303339549771670125767322805744e+979420096', '4.8222239563060854226188383363864548574850456e+90178388677203', '4.822223956306085422619e+90178388677203', 22, 2);
    T('-6.7721983598281562400633629798220161611e-26', '-1.135676393e+663445033094', '-1.135676393e+663445033094', 98, 4);
    T('-7.484206749965270147109469139411051320188638867654351462e+81947', '9.053347147253817235565780633232597844550927941710e-7465', '-7.484206749965270147109469139411051320188638867654351462e+81947', 73, 3);
    T('-1.16969836463135239470987075632899236705452955e+320219999239752', '9.644095718783279862895440924796323237119e+101', '-1.1696983646313523947098707563289923670545295499999999999999e+320219999239752', 59, 2);
    T('8.4890615500613009693098192591023e+208', '8.372356698440327615990528723563433451591754687e-495621261283', '8.4890615500613009693098192591023e+208', 37, 3);
    T('1.989837010754e+39', '-6.36589145169e-79277608614', '1.989837010754e+39', 87, 0);
    T('-8.168638343668800947934572279e-8967', '7.17119547667309397403412594e-320123', '-8.168638343668800947934572279e-8967', 88, 6);
    T('8.887715931694877250756151072361816823436202473236328032e-92964', '-4.592370711637255e-186851868769939', '8.887715931694877250756151072361816823436202473236328032e-92964', 97, 6);
    T('-9.8609411071367398233195965957264652701493634788825e+493', '7.4377148605598910725635832996479665935312679535500e+152234', '7.43771486055989107256358329964796659353126795355e+152234', 56, 0);
    T('4.4352770556605946443533164112070155877977060627296984743e-6608', '-1.501660145006317998521662788249823313756918e-613826427184389', '4.4352770556605946443533164112070155877977060627296984742999999999999999999999999999999999999999e-6608', 95, 1);
    T('-3.2091240285998e-890221128', '-9.65730659251509968428964261040090292719343868215009358630e+287539609052359', '-9.6573065925150996842896426104009029271934386821500935863e+287539609052359', 74, 5);
    T('1.836465367232539348e-239516519', '-1.683411570657346901401081102e-297836834817', '1.9e-239516519', 2, 0);
    T('1.8138639018418103753007916152754101e+42', '-6.70345837925227478044005370e+76897026592', '-6.7034583792522747804400537e+76897026592', 33, 6);
    T('-2.49234371287726757234950991525506245721821612968115e+85606154165', '8.258360905652407390283016360174e+4235740', '-2.49234e+85606154165', 6, 2);
    T('6.281e+6417943194', '1.18432421217150850173119509675077439e+916316454734674', '1.1843242121715085017311950967507743900000000000001e+916316454734674', 50, 0);
    T('9.5146445852916947455072895022963914024506332850e+3533166', '-5.5209800686364007e+98691', '9.52e+3533166', 3, 2);
    T('-9000', '-2.64554323809150457914945314416886033519436155155423e+227991408318688', '-2.64554323809150457914945314416886033519436155155423e+227991408318688', 80, 1);
    T('-1.92652039276118630382376452964234696403477e-85109', '-6.9e-8227', '-6.9e-8227', 79, 5);
    T('-1.84267672359145745780919657e+8238040205333', '-3.70991207379009742229841194783446574272614377773143e+154118739', '-1.84267672359146e+8238040205333', 15, 6);
    T('8.00328015e+6433', '2.15579461375e+3968', '8.003280150000000000000001e+6433', 25, 0);
    T('-5.0852106196074617898949103365722740e-1030647', '5.04e-87539', '5.04e-87539', 55, 4);
    T('-8.98892629347872650408004946202653979597330285104074334e-298370883', '8.861578143377846146979593552266399e+626410', '8.861578143377846146979593552266398e+626410', 34, 1);
    T('-2.53376589064407220412533716245979260120849447408e-97589849039756', '4.284317e+19769940682', '4.284316999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+19769940682', 91, 1);
    T('-7.69023174457934548207684e-38283', '-4.330767985e+92147650502', '-4.330767985000000000000001e+92147650502', 25, 3);
    T('5.3305741763995066966399137715614032138e-196777837', '-7.418926684e+28511402', '-7.418926684e+28511402', 33, 3);
    T('-6.467543013506839642033214781740151339298309533811e+4805017660', '8.34448740265336988363724078517922529731191391490360890e-51600012862', '-6.467543013506839642033215e+4805017660', 25, 0);
    T('-6.2033093444788521050916e+332471385870270', '4.16540416110e+1763420', '-6.2033093444788521050916e+332471385870270', 100, 6);
    T('9.21407384360e+87933', '6.797790720e-498', '9.2140738436e+87933', 99, 5);
    T('2.614754359464051977610470471063707431271470e-83950469681370', '7.238076016499052671183361216870028546846580122478e+75', '7.23807601649905267118336122e+75', 27, 6);
    T('7.499987523270906436e+44412483452160', '8.9371786849848219719619846854098e-822', '7.499987523270906436e+44412483452160', 57, 6);
    T('-5.2776153896265881471209529561073078593473090000e-538871664', '-4.39439706488433887003616602831e-1835392491', '-5.277615389626588147120952956107307859347309e-538871664', 54, 5);
    T('8.41496401765882762715157e-59', '-5.56611890287163281600333199973292983090431493952484904e-19227794480764', '8.41496401765882762715157e-59', 81, 0);
    T('-5.448834410881702420674402503897087447189898941755650e-337', '-5.87014480782282168e+26143360221537', '-5.87014480782282168e+26143360221537', 75, 5);
    T('7.3833416271224416895688632631041e-67991051', '8.8566950651941094682005830881e+37698189578', '8.8567e+37698189578', 5, 4);
    T('-9.14070749227863759e+17298027', '-9.0601757205592e+104', '-9.140707492278637590000000000000000001e+17298027', 37, 3);
    T('-4.986799e-44', '-6.54786657649886272930691e+42737', '-6.54786657649886272930691e+42737', 49, 4);
    T('2.98811123034633012943581100259e+38', '2.855918871134351389809092769e-2869', '2.98811123034633012943581100259e+38', 31, 4);
    T('7.894845884472835199785374245974087700782035196e+9408', '-5.3411916467088338163094841619095906304e-6060970351353', '7.894845884472835199785374245974087700782035196e+9408', 65, 4);
    T('7.6233078083339480079095348693061329100300399339848984047e-7214', '7.9019176511352481268619219158118381e-648312971648', '7.62330780833394800790953486930613291003003993398489840470000001e-7214', 63, 2);
    T('4.7942215762661010581365307183399142942309659520189e-31037648253', '-4.90665912064935169532771e-10455719816320', '4.7942215762661010581365307183399142942309659520189e-31037648253', 65, 6);
    T('-8.248761819501076954621875990e+2701915942', '6.0659611159272117287641043468437894702187e-60221', '-8.24876181950107695462187599e+2701915942', 70, 3);
    T('9.17013115687e-98342467294', '4.24178136897546233749215567303186853e+91203', '4.24178136897546e+91203', 15, 3);
    T('-5.14198279529538967230674605594232058198268046632426e-7591', '-9.5877382536158803972948428349176709292311120937645e-43701', '-5.14198279529538967230674605594232e-7591', 33, 4);
    T('6.13610395344031193836e+5329022', '-3.4779799088758e+99702134', '-3.4779799088758e+99702134', 58, 6);
    T('3.1551014535503553524865872811551945700462716112925401286e+74741', '-1.686215196601731312325903896094431255568527e+102855', '-1.68621519660174e+102855', 15, 3);
    T('-8.39991822160210661079655781e+538008495578', '5.6669219327940154513826973459206221173238954955591786e-85258', '-8.39991822160210661079655781e+538008495578', 89, 5);
    T('-7.312854086367030470764653440342703194702290217e-843894296087198', '8.011688e-368918936469185', '8.011687999999999999999999999999999999999999999999999999999999999999999999999e-368918936469185', 76, 3);
    T('-1.7430559489402167882192615352949943407159e-757902', '-8.157902439550860004590e+55299', '-8.15790243955086000459e+55299', 44, 1);
    T('6.94116e-224373741811127', '-4.3051e-2550974591635', '-4.3051e-2550974591635', 75, 4);
    T('8.826302032308988504230076652542648465e+3930819226699', '-8.7243972106236e+41', '8.826302032308988504230076652542648464999999999999999999999999999999999999999999999999999999e+3930819226699', 91, 1);
    T('7.19944453761423472489340438473345278e+5098193724', '-1.6854382174435553364898938384183564154048120382107249023131e+201', '7.1994445376142347248934043847335e+5098193724', 32, 2);
    T('-8.10372792118079e+62235960120', '-1.3273032902490365766702624916165959378402e-516085371496', '-8.10372792118079e+62235960120', 39, 1);
    T('-6.4764599909e+9265003', '2.93929928169167136417731579928e+216378173257068', '2.93929928169167136417731579928e+216378173257068', 32, 0);
    T('6.541354425859563399357455820495189601912852662110841e+2823239430', '9.77555652612813775662548972169822204038e+1353574222', '6.541354425859563399357455820495189601912852662110841e+2823239430', 68, 1);
    T('-6.483400327476e+5381210', '5.3806250432803241139215660301762015169770723534686653e+9418044567', '5.38062504328032411392156603017620151697707235e+9418044567', 45, 6);
    T('-5.7987127602687744432982924317419076588912578986770590710500e+79131833', '5.47292315773065318562227178240242759008226360031155e+5699759891', '5.47292315773065318562227178240242759009e+5699759891', 39, 0);
    T('-9.203876646157448129501750934224861039658734052e+516200044771', '-9.11438272755009252070530598976019402234e+664235', '-9.204e+516200044771', 4, 5);
    T('-6.805863185235628014788525797237832431e+47132087860123', '2.774101253297449889199237576021715077794e+70470', '-6.805863185235628014788525797237832431e+47132087860123', 54, 0);
    T('-5.218644e-5522981116', '1.46631872017517262286695480330e+7954524', '1.4663187201751726228669548033e+7954524', 73, 6);
    T('5.96995933492981013666130177107925e-53984', '-5.208913858351047843399674542148002686322e+17590540107', '-5.208913858351047843399674542148002686322e+17590540107', 53, 4);
    T('-1.861377889716068596620312810364449384040455695073e+49', '6.565493461161725871795981540317914054250016667019244441e+1021', '6.565493461161725871795981540317914e+1021', 34, 1);
    T('-5.467239844159228e-909239899413464', '-6.0204761282483808350959e+59703061', '-6.0204761282483808350959e+59703061', 66, 2);
    T('-7.70625673152442126775915512339788319836e-842251', '-6.8992999370371817112e+463', '-6.89929994e+463', 9, 6);
    T('9.98425622143251297587475421499700949456599157861e-821895003425', '7.415031257099442614657984255149e+28015423', '7.415031257099442614657984255149000000000000000000000000000000001e+28015423', 64, 2);
    T('-8.28257990661194334e-424354005820734', '-7.73710029344914623958e+881951392', '-7.73710029344914623958000000000000000000000001e+881951392', 45, 3);
    T('-9.79950817475263187242549688740552906663189e+604914407689', '-3.639089037763279249362840e-1421', '-9.79950817475263187242549688740552906663189e+604914407689', 75, 1);
    T('-6.05898e+71844281', '-2.43907911928354824421e-8796718', '-6.05898000000000000000000000000000000000000000000000000000000001e+71844281', 63, 3);
    T('-9.99743039284685450e-75490114769', '3.12146215e-391059372', '3.12146215e-391059372', 77, 4);
    T('2.78509687748229217413036e+8239790130', '2.1331184062864e-61985445843', '2.78509687748229217413036e+8239790130', 38, 1);
    T('1.23872962485409346271537614567442923679477467702767582095e+59', '-5.55582047983348407536299384010244e-1892067', '1.2387296248540934627153762e+59', 26, 0);
    T('-4.425645567849986628395211362247e+5054342536', '-2.35962987300574772001479e+1750705542170', '-2.35962987300574772001479e+1750705542170', 42, 1);
    T('-3.27144623115e-79082', '-4.13643642916445177e-399476165', '-3.27144623115e-79082', 40, 5);
    T('-4.78301130245679201523846586868768713286161916517693434e+354260', '-7.58262544076667066688121324552044716337639733995229464e-223195088', '-4.7830113025e+354260', 11, 3);
    T('-8.63515142016600714737221845398252229293533290137554943e+596679051403800', '8.8191e+7051163', '-8.63515142016600714737221845398252229293533290138e+596679051403800', 48, 3);
    T('5.813324412624247337180505647138e+473755', '-7.378323098626849559311543684401292266485928171149853966e-58692943349483', '5.813324412624247337180505647137999999999999999999999999999999999999999999999999999999999999e+473755', 91, 1);
    T('-3.212385060520925758715946021e-7037', '-2.283421667131215220033377960984046003633934854469646052999e+28429703', '-2.283421667131215220033377960984046003633934854469646052999e+28429703', 75, 2);
    T('8.5993771971092216283822975832355567664e+19754142815370', '-3.277574063322084e+59', '8.59937719710922e+19754142815370', 15, 3);
    T('6.853280394470584e-29940367689', '1.4242121352789389812965356776353542123891725780652919832e-3665', '1.42421213527893898129653567763535421e-3665', 36, 6);
    T('8.6899294709e-303', '9.1046716174602687321e+201', '9.1046716174602687321e+201', 53, 1);
    T('-3.710473909946056037464067e+502687729190', '5.277236353521230074675e-2469497870', '-3.710474e+502687729190', 7, 3);
    T('9.88442e+103', '-2.49217704793074872538960076421072073772797041892054574348219e-45724949844673', '9.88442e+103', 96, 5);
    T('-7.1930294504876798044e+28611', '9.08751164e-9799890814', '-7.1930294504876798044e+28611', 25, 3);
    T('-2.196352180265572793816816076048513525415005899e+22702842944', '-3.7e-43443796', '-2.19635218026557279381681607604851e+22702842944', 33, 2);
    T('-2.58336082390528151419476850356889034481731715405541833e-65644519519502', '742687860361707258293297121162196900.85602', '7.42687860361707258293297121162196900856019999999999999999999999999999999999999999999e+35', 84, 1);
    T('5.46675236323901e-6060583723', '-3.240061115069e-355', '-3.240061115069e-355', 29, 3);
    T('3.51479e+5402129147142', '7.9e-22296036529', '3.51479e+5402129147142', 99, 1);
    T('8.69959704043986258559377024496753846989304012345012708995698e-124292896390', '5.95846468481976568866905e-491', '5.958464684819765688669050000000000000000000000000000000000000000000000000000000000000001e-491', 88, 0);
    T('7.8527103469615060e+6515317369368', '-1.25352002061e-314761749', '7.85271034696150599999999999999999999999e+6515317369368', 39, 1);
    T('1.616877452345769e-160064739', '3.04e+165020929', '3.0400000000000000000000001e+165020929', 26, 0);
    T('9.65616072884110465e+7188746', '-1.791164461138850805960725656499363589251316e+64384947491545', '-1.791164461138850805960725656499363589251316e+64384947491545', 74, 3);
    T('4.9192058440673259103688260377523879458810443023641e-4132196', '-9.92563128648087137527055469155e-488553845', '4.919205844067325910368826037752387945881044302365e-4132196', 49, 2);
    T('-667106463.265403179500786330210669209703196113298986', '3.011635105e-39174', '-6.67106463265403179500786330210669209703196113298986e+8', 64, 3);
    T('8.2271819058962877520050233001987781359e-17', '-4.40317477929165422900026733143085979552111108902196510e-37123367388072', '8.22718190589628775200502330019877813589999999999999999999999999999999999999999999999e-17', 84, 1);
    T('-7.810671665168520470350891527105e-6620915035', '-8.767699489561134848997381290716463834250229546610206209694e-8190006173', '-7.810671665168520470351e-6620915035', 22, 0);
    T('6.150630735836941369031942786923685122836529884815665567e-64336416620928', '5.667730e-980668128882829', '6.150630735836941369031942786923685122836529884815665567001e-64336416620928', 58, 0);
    T('5.25e+727', '7.366565059516042e-476676603842332', '5.25e+727', 47, 5);
    T('-7892160175520455393345027414938804.362508', '-2.867095232477397732534461e+49439142091', '-2.867095232477397732534461e+49439142091', 48, 4);
    T('-3.2956354283922503200901526869155525421347e-131479347', '8523628610424264312.03765', '8.523628610424264312037649999999999999999999999999e+18', 49, 1);
    T('7.6181070428888e-642031290', '-3.337568235743131765095223629238179911444532986549154697413e-25779204', '-3.337568235743131765095223629238179911444532986549154697413e-25779204', 83, 0);
    T('8.30e+2926778497', '-6.20443931983e-55668532232', '8.3e+2926778497', 31, 5);
    T('9.1003e-79117', '4.2474129407270105644e+702113662044', '4.247412940727010564400000000000000000000000000000000000000000000000000000000000000000000001e+702113662044', 91, 0);
    T('-6.4849087990745257e-3613221890', '3.8252157857551186095648020530991625973e+505181680131018', '3.8252157857551186095648020531e+505181680131018', 30, 4);
    T('-2.951632321e-80', '-9.85507302995434413588044940e-85081', '-2.951632321e-80', 75, 5);
    T('-8.9440788e-12032037035', '8.82535841111892156996487212773183947606864385596212e-472475800805', '-8.9440788e-12032037035', 92, 0);
    T('8.24068214813818242161170293668257844276691013983e+31252869383', '-1.61402747250398513e+99364627', '8.24068214813818242161170293668257844276691013983e+31252869383', 73, 6);
    T('-6.56718894337e+95914767', '-6.21618115261074391895684396e-934882738362', '-6.56718894337e+95914767', 77, 1);
    T('-3.147170259601308369398699151408187026026566e-6972', '-3.36300596881295548856712450314261755892561799612e+5043277', '-3.3630059688129554885671245031426175589257e+5043277', 41, 3);
    T('2.56411628275929795093877194306647444955477098280681200488857e+804751429', '-7.24010231815330002571514289832617e-42279800784228', '2.5641163e+804751429', 8, 2);
    T('9.84134439e+22432', '4.9203800310720611881904127200086e-60025858', '9.84134439e+22432', 9, 5);
    T('6.68135512131883799701449762675786312548853309130219408e-579770010859', '-1.9405610069055039e-82295918657174', '6.68135512131883799701449762675786312548853309130219408e-579770010859', 58, 6);
    T('4.10023801379344734467077831632088407223748121452437465911220e-4941163364400', '6.0969714311693213394253076113989628391187687438514e+1131407', '6.09697143116932133943e+1131407', 21, 4);
    T('-7.1491554471765823780628247873069913456471054997e+4210585429', '-8.5624868965194816e+221640846471', '-8.562486896519481601e+221640846471', 19, 3);
    T('-9.674798546e+813', '-3.07163983281315985574920873948085689699125726955469e-1134038', '-9.674798546e+813', 73, 2);
    T('9.68596536924160794770e+49118370948571', '-9.429452268035783099723658353567e-5754510', '9.6859653692416079477e+49118370948571', 85, 0);
    T('3.889567506734632490713127683039068424e+37', '-1179254459833302854255332470944535.7', '3.88944958128864916042770214979e+37', 30, 6);
    T('-6.18445319392044e+54628772441', '-8.26798446650854173629371443784618112758300616e-86368365', '-6.18445319392044e+54628772441', 23, 2);
    T('5.8479318619521329265529536668195034146513180182799363998e+758147371794', '-2.526136085068538e-615438298327976', '5.84793186195213292655295e+758147371794', 24, 6);
    T('-3.08489934923662609195275274260e+1584658', '-3.337e-88392', '-3.084899349236626091952752742600000000000000000000000001e+1584658', 55, 0);
    T('4.0746838440751529560682346e-174352273880461', '4.337884e-407945541', '4.337884e-407945541', 14, 4);
    T('4.81624565273424008560643426e-78647802576', '-3.51921532195687767072708057588916488916117537779426057322e-1776074', '-3.519215321e-1776074', 10, 1);
    T('4.5182843786682831502936e+27', '5.1359611206926022457677555081002318421175903995e-8513325883', '4.51828437866828316e+27', 18, 2);
    T('-3.38018247776669001513766992955225238287314e+909422', '4.99012604555812688e+4696238561', '4.99012604555812688e+4696238561', 63, 5);
    T('3.42667701740771178903876e+534879291186448', '4.0177978732798985208288365e+533482918957859', '3.42667701740771178903876e+534879291186448', 42, 5);
    T('9.0403083016537601460657e+747201', '-4.282372870408574413776e-195883456', '9.0403083016537601460657e+747201', 56, 4);
    T('-4.50733841011952281885695033318647630165432379070645e-81918943', '-7.4307378434022502892220541130e+639619788', '-7.430737843402250289222054113e+639619788', 99, 2);
    T('3.2308460101344958037805711576842569877e-9179817292', '2.1185167336755106690238277061062534704472377581422885924161e-412109480851', '3.230846e-9179817292', 8, 1);
    T('-2.496091731102935058584091645728755731005394177221707e-4765', '1.62012566e+7192658104484', '1.62012566e+7192658104484', 82, 6);
    T('1.0367745e-343', '-2.95194306e+9', '-2.95194306e+9', 58, 4);
    T('-9.0513765871574158665e-912848128503692', '-3.481093249153343838017633810966008e+5193878271344', '-3.481093249e+5193878271344', 10, 5);
    T('3.47672866090397730089674503954687875558207114e+15326415361931', '-7.05159350215982008457613632e+63563482874', '3.47672866090397730089674503954687875558207114e+15326415361931', 63, 6);
    T('1.0644618352804265430955e-351561802', '-2812952051212129640217638928090839437573916.003616458', '-2.812952051212129640217638928090839437573916e+42', 44, 1);
    T('9.493789515541037051188710320148960019e-827452080096324', '-7.98546173594050857292069917839674330e-512', '-7.9854617359405085729206991783967433e-512', 93, 4);
    T('4.9619773982741663757484946333298442679344032541e-66760357', '-5.4661038696355843232867304908252e-5717192922409', '4.961977398274166375748495e-66760357', 25, 2);
    T('8.7430892969574477409017112921788e-5349395', '-3.8878198780796717760310783465512554854324264592e-10', '-3.8878198780796717760310783465512554854324265e-10', 44, 6);
    T('-7.225220489090104008055568198172e-935', '8.073770251503614535207e+235270', '8.07377025150361453520699999999999999999999e+235270', 42, 3);
    T('-7.2185539174e+9047173978', '9.351133653806217041742664430393985296799224e-96907058', '-7.21e+9047173978', 3, 1);
    T('4.641839298e-23516391105', '-4.6629084040964425099982463191119415743779176524616392955122e-369499', '-4.6629084040964425099982463191119415743779176524616392955122e-369499', 70, 5);
    T('-5.6338152126052688579708553989031983139949935813792e-58007', '-5.8861e+5650888401379', '-5.8861e+5650888401379', 11, 1);
    T('-2.94736227384e+48329535', '9.12654e-78633931219163', '-2.94736228e+48329535', 9, 3);
    T('1.32897030661e-578275', '-9419944728620125398332925283', '-9.419944728620125398332925283e+27', 48, 5);
    T('-6.1419517159257194349e+22', '7.1880043294695389803183573234615798937462797885010e-5142720880', '-6.14195171592571943489999999999999999999999999999999e+22', 51, 1);
    T('-6.75556310811100039028028829279e-5025', '8.845920929366074526074792940870880e+3596656826732', '8.84592092936607452607479294087087999999999999e+3596656826732', 45, 1);
    T('-2.53919100687916967627363579982884730663948394726216717648e+36428613435940', '-3.6964606151115153644513685472165258e-8796431', '-2.53919100687916967627363579982884730663948394726216717648e+36428613435940', 76, 1);
    T('1.9960410e-28835542', '-8.3647358237411364810140599076772903665489342085150406981630e-26934829600', '1.996041e-28835542', 89, 5);
    T('-3.0240e-42206', '-5.483787697594176819364494362760424254527426063754078816e-248078949700322', '-3.02400000000000000000000000000000000000000000000000001e-42206', 54, 0);
    T('1.46339400928374526362442670745e-4255077', '8.92782346244750971209016549114446323214280925814750815e-24967478741', '1.463394009283745263624426707450000000000000000000000000000000000000000000000000000000000000000001e-4255077', 97, 2);
    T('-6.3895265603e-5100037045', '1.40661003624295031182316550313107315441137e-53', '1.4066100362429503118231655031e-53', 29, 6);
    T('3.123297196107331366555551699e+61355', '5.09620905663702177007595136160620e-866607722310881', '3.123297196107331366555551699e+61355', 46, 4);
    T('-7.071738770375009020098149550224811e+9288985454', '4.1447066084427780810e+1636231421', '-7.071738770376e+9288985454', 13, 3);
    T('1.39809870645554579069200182179259002704577076333002229e-935787', '-4.782200925427325526272130e-215', '-4.78220092542732552627213e-215', 67, 5);
    T('-2.3473769115336888597656324059231946066206516223453686e+5196202332347', '-1.345084440556761191505488819859899307695520948e+70110', '-2.3473769115336888597656324059231946066206516223453686e+5196202332347', 94, 2);
    T('1027370589377834459532656340508097761.1474372379', '4.56895605265142323712911e+15255201', '4.568956052651423237129110000000000000000001e+15255201', 43, 0);
    T('-2.501056931717405323863796187427844072798019e-4120303467786', '9.26517316989451780928415902896387359373e+14840', '9.26517316989451780928415902896387359373e+14840', 44, 4);
    T('-6.66861116031775331284291397299794034157696557851580e-19605025514422', '-6.5e-60233237248', '-6.5e-60233237248', 43, 1);
    T('7.88425753623721151455412020137416390907e-8676', '-6.5201973658485412191371293040019058747827019e+918812404573', '-6.520197365848e+918812404573', 13, 2);
    T('-5.7543118861e+92862808', '-6.20910401009985044351048239165918e-1060649', '-5.754311886100000000000000000000000000000000000001e+92862808', 49, 3);
    T('5.21147022050157742767219604604385188e+556', '5.288187102224222042116972753918672390e-467187808110', '5.21147022050157742767219604604385188e+556', 83, 1);
    T('-8.539458640553559768997385640984750663082036996e+777112522859', '9.033967635529414886508951116762360305271262854155224e-43435752024', '-8.53945864055355976899738564e+777112522859', 28, 2);
    T('8.570596519542912425104465599750091626587876231390e-9857210230', '-5.293475e-27649333', '-5.293475e-27649333', 58, 6);
    T('3.753868104675062370e+935778396083', '-5.45412461042653442105233761701668129246395758597215736e+76487528277', '3.75386810467506237e+935778396083', 27, 0);
    T('8.9658853829927568820328164e+356297538730', '-2.2120797724805541e+3506665923064', '-2.2120797724805541e+3506665923064', 90, 3);
    T('8.7391380515631238286740e-1305882568252', '735208407365832845616243517536185179410342130815', '7.35208407365832845616244e+47', 24, 2);
    T('8.4502533496757788420182370760e+57015304', '-9.627431121648539886413352287502256513403591131404995e-125028217', '8.45025334967577884201823707599999999999999e+57015304', 42, 1);
    T('-6.51097080583058726980628723561030403e-143937334584534', '-2.5942130929550430764235739796017401e-13232', '-2.5942130929550430764235739796017401e-13232', 81, 6);
    T('-8.743e-4694240', '3.169703721731269386904303944386922047550274e+4254', '3.1697037217312693869043039443869220475502e+4254', 41, 3);
    T('-7.351229247550501157868144843500474197834281151130538024438e-1736386190713', '-4.8389402888246961796801716456804166008342894881017445016e-31', '-4.838940288824696179680171645680416600834289488101744501600000000000000000000000000000000000000001e-31', 97, 3);
    T('9.366278882592267894323e+91762', '1.0709655783743e+177135503', '1.07096558e+177135503', 9, 2);
    T('2e-414576723751343', '-0.00000175587156352483', '-1.75587156352483e-6', 93, 0);
    T('-6.4404203607368768337e+160208085', '-8.93013243859984397972583083741e+5490506', '-6.4404203607368768337e+160208085', 39, 4);
    T('-8.400804652907135e-53608', '-2.20468753815963572591008657928225e+445741212', '-2.2046875381596357e+445741212', 17, 2);
    T('-2.7e-8', '-3.40701330e-2374810', '-2.7000000000000000000000000000000000000000000000000000000000000001e-8', 65, 0);
    T('1.73889920577747472981584075314837e-452998616210', '6.115e-186155237597', '6.115e-186155237597', 60, 1);
    T('-7.5573736211049308457e-77957289237', '6.76974745733021672165031804e-14944118082628', '-7.5573736e-77957289237', 8, 4);
    T('6.167923724136265825065879537811210416495080100163e-15776', '5.269831979190812363919921e-7793', '5.269831979190812363919921e-7793', 95, 3);
    T('-6.639996783677499412532080250648581838986376309969213e+33791462', '-6.89676416929413545050e+589366466788', '-6.8967641692941354505e+589366466788', 56, 4);
    T('3e+165847990', '-2.120147337294695521883e-61481587027', '3e+165847990', 72, 5);
    T('-5.9926676148516559065917295907617258e+271735', '-1.1580248252402245e-81', '-5.9926676148516559065917295907617258e+271735', 86, 6);
    T('-1.426458671618079702223026165488338265717657171952776475e-1827256875', '-2.03e-7893170145134', '-1.4264586716180797022230261654883382657176571719527764751e-1827256875', 56, 0);
    T('8.589462528264454072311406377e-4114313587973', '-8.4160765575815e-557892692463135', '8.589462528264455e-4114313587973', 16, 0);
    T('-5.3275062358620538741672727381e+717269093', '-3.89375031373400049048992460419788702921480663255738e+670', '-5.3275062358620538741672727381e+717269093', 33, 1);
    T('2.633519994456215583031816371094e-5810', '8.79611678262848228203115100353136e+8568', '8.79611678262e+8568', 12, 1);
    T('-7.0897703842668520829e-832850923857242', '-7.115e+8317181000446', '-7.115e+8317181000446', 22, 2);
    T('4.8733026237e-183', '-4.764736948897421684321797015800030547e+555688', '-4.764736948897421684321797015800030547e+555688', 91, 0);
    T('-8.811892e+46614482088', '-8.537653840061227261145091927976148375354229502735642e+3551932475231', '-8.537653840061227261145091927976148375354229502735642e+3551932475231', 56, 5);
    T('-8.322806390056583863934677762450198838439764349663260636e+9761991252', '-2.9e-59347536785', '-8.32280639005658386393467776245e+9761991252', 30, 1);
    T('5.406e+3036170066995', '2.68006845694931458231e+28628800722', '5.4060000000000000000000000000000000000000000000001e+3036170066995', 50, 2);
    T('6.395122887589998366872062357963354722482471e+5276285', '-4.75824199218201e-98199420565383', '6.395122887589998366872062357963354722482471e+5276285', 49, 6);
    T('5.9259037952510e+184970430163159', '-9.6196808e+98', '5.925903795250999999999999999999e+184970430163159', 31, 3);
    T('2.592872655146271326693501007503532014e+9250589', '-5.2e+94442531', '-5.2e+94442531', 67, 4);
    T('-8.927560173e-95126460052', '-8.841492894e+2855', '-8.841e+2855', 4, 2);
    T('-4.12440751322253883e-206', '-1.569106978405252620551573347635227241980e+522334849737504', '-1.569106978405252620551573347635227241980001e+522334849737504', 43, 3);
    T('5245973874229049', '-7.60861886082845785936096597150581777730e-614193633', '5.25e+15', 3, 6);
    T('0.0000243', '-3.247384051088918748995094e+61454', '-3.247384051088918748995094e+61454', 64, 5);
    T('5.40689593698190897046886482e-927373651', '-7.7026605506001210749840341832764312670e-78110484914605', '5.406895936981908970468864819999999999999999999999999999999999999999999999999999999999e-927373651', 85, 1);
    T('2.99554638366623719e+402378982', '-1.810163451018051e-350173', '2.9955463836662371899999999999999999999999999999999999999999999999999999999999999999999999999e+402378982', 92, 1);
    T('-6.969841717221529604187190635577663380815291051961812360516e-5317', '-71653282037065906767275734368812089582098228091.810887374', '-7.1653282037065906767275734368812089582098228091810887374000000000000000001e+46', 74, 3);
    T('9.87012312759597038461166027808845e+233080409664167', '6.79782570064471271262973612238082854934e+69917130309', '9.87012312759597038461166027808845e+233080409664167', 63, 3);
    T('6375733020', '1.4491544318796676413450547272969222893543958178659e+64', '1.449154431879667641345054727297e+64', 32, 2);
    T('4.99693800465359687948300177986614037e+382046', '1.0813356180122244e+939031445', '1.0813356180122244e+939031445', 44, 6);
    T('-8.795721557194334199463845719512373005608951216600454750128e-18021420095', '-3.87061858531510084292974925024350419316229311e-89', '-3.870618585315100842929749250243e-89', 31, 2);
    T('-5.7890183e-71102', '-1.50175980597333711173092605215725426e+324403', '-1.5017598059733371117309260521572e+324403', 32, 1);
    T('-8.625211203276e-5602526201294', '7.8979914302503968530376381e-10999124627', '7.8979914302503968530376381e-10999124627', 39, 2);
    T('-1.74e-89898740400367', '2.307345e-45566', '2.30734499999999999999999999999999999999999999999999e-45566', 51, 3);
    T('1.233776243e+17', '7.6321226115887168302474314892005782637962676e-24321615497', '1.23377624300000000000000000000000000000000000000000000000001e+17', 60, 2);
    T('4.8725417136307544180338393044560201090067120723484743024e-946790825092', '6.83909198281591825404448808471711e-5836503', '6.83909198281591825404448808471711e-5836503', 64, 1);
    T('7.108972604551074082426723684856e-151411811891039', '3.57506418865524994330215149205e-532629', '3.57506418865524994330215149205e-532629', 38, 3);
    T('-9.941799453324120049216033161527937073463020852637e+97947062', '-8.34137428068337874170441925275469235e-35732931507', '-9.941799453324120049e+97947062', 19, 4);
    T('-1.154784910234293624e-648708', '-3.2982699806792368459e-23', '-3.2982699806792e-23', 14, 6);
    T('-4.782166707996345064060679875348455555685104e+64956748', '-1.607999919582040953132455116734109429386e-58490836002', '-4.78216670799634506406067987534845555568510400000000000000000000001e+64956748', 66, 0);
    T('8.5509718403684775e+44287297', '-2.51503320048604599725e-540882', '8.5509718403684775e+44287297', 24, 0);
    T('3.0792063193757335830680196898142084968686906907977e+198369051875', '7.412e-4506', '3.0792063193757335830680196898142084968686906907977000000000000000000000000001e+198369051875', 77, 0);
    T('9.6019082810802956198252659599361279602e+70211', '1.473888093712e-266648810491', '9.601908281080295619825265959936127960200000000001e+70211', 49, 2);
    T('-556091076827817727407.572', '-4.6384252334265593156236e-81199131561355', '-5.56091076827817727e+20', 18, 4);
    T('-6.66578611955581600e-267', '-5.86773687559357632274128063429855e-5405', '-6.665786119555816e-267', 31, 2);
    T('-1.4508474494493134849315147043273353023e-25', '1.42956853325127988968928457161984e-30443030098', '-1.4508474494493e-25', 14, 4);
    T('4.861963415803e-5589560', '5.3154902670317290931352135947431952074929216230348845697e-7675056243384', '4.86196341580300000000000000000000000000000000000000000000000000000001e-5589560', 69, 2);
    T('9.8187864262065e+5210', '-5.1232863224842653888526e+1120314853', '-5.1232863224842653888526e+1120314853', 71, 0);
    T('-7.1836771039905938073738090091395110141662452e+289740', '-9.193612827725988056464590137125176487612260623983467149e+10318435496', '-9.19361282772598805646459013712517648761226062398346714900000000000000000001e+10318435496', 75, 3);
    T('8e-135765', '79475579886.155', '7.9475579886155e+10', 64, 1);
    T('4.3307708826893910762042479696509251797958337153029532e-350891', '9.483210815557075e+345159003941', '9.483210815557075e+345159003941', 23, 3);
    T('-5.2347138454695894432e+26866247263572', '4.13523928479973289347038778e+76', '-5.24e+26866247263572', 3, 3);
    T('-7.342812404339244299886085494683120437789e+280', '-2.27359076e-8513690793', '-7.3428124043392442998860854946831204377890000000000000000000001e+280', 62, 0);
    T('9.70039129935875462110945334249538e-37897512199', '4.61064302049276314077026910170423943235727346460100287440937e+683555853384614', '4.610643020492763140770269101704239432357273464601002874409370000001e+683555853384614', 67, 0);
    T('-8.26585323197690578452836024153928633281968582225e+48215', '-4.9404571397229338349569633030483762839e+167715871020050', '-4.940457139722933e+167715871020050', 16, 2);
    T('-4.18701288395311476984607288870621269263802281702735809089e-68674082197', '-7.32403488470491393989125526554833082114905157730567984406573e+242', '-7.32403488470491393989125526554833082114905157730567984406573e+242', 71, 5);
    T('6.3236657e-666960315987', '6.72079321975e+48', '6.72079321975e+48', 53, 5);
    T('-3.9954423008909411021252469718701e+40', '566027351931653734121', '-3.9954423008909411020686443e+40', 26, 0);
    T('1.2959560047016282726444720943890028200615393024382072309e-6754594151', '7.443625205046338902039424e-268710440', '7.443625205046338902039424000000000000000000000000000000000000000000000000000000000000000000001e-268710440', 94, 0);
    T('9.41367953481e+91134744', '-7.43813605375739882474392400560048503962849173993e+967408', '9.41367953480999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+91134744', 99, 1);
    T('-5.4202466851635774e-38987655365700', '-1.7364544179352826849098939468233098033794006e-761', '-1.73645441793528268490989394682e-761', 30, 5);
    T('-7.633209092480887707401626e-9166196', '6.5124766144867144626239090639997020734919163620433482509e-198179', '6.51247661448671446262390906399970207e-198179', 36, 5);
    T('5.5468413654753496117401136077304556082e-296320', '-1.5203701461267282458346074597344349023e+7437274208609', '-1.520370146126728245834607459734434902e+7437274208609', 37, 2);
    T('7.2533858857742482532439224564462096336550e-735', '-9.62931270248311938088e-743', '7.253385789481121228412728647646209633655e-735', 91, 2);
    T('5.7157530476891913681297796750003657e-61936428679', '-3.33827128989213613726573151578477836244843182716355396637e+915', '-3.338271289892e+915', 13, 2);
    T('-2.3958105775203847933987652188797093715030221814364052e-188918', '8.635953095007602554239104912435290780187e+31156346690', '8.635953095007602554239104912435290780187e+31156346690', 66, 2);
    T('-2.535715311862830892785407135888547506859275949e+50', '-8.891039083318833548726832458032e+274', '-8.89103908331883354872684e+274', 24, 3);
    T('4.45311403659815036355113383433062251027940611756689336362e-548440294774007', '8.089201e+82451221895', '8.0892010000000000001e+82451221895', 20, 2);
    T('-1771490672784061566187465938862213.9928', '-319241652343574884380371321140.9388', '-1.7718e+33', 5, 2);
    T('2.547667775555282893749075359398997328757440454617021e+126', '6.4382022825678259544475186202082069221863803390893e+94370722235447', '6e+94370722235447', 1, 5);
    T('-7.5153116100802530976755078132548160495030950350349567008e-49833', '-3.25e-64960847', '-7.515311610080253097675507813254817e-49833', 34, 0);
    T('9.0879855817707077184400281288878365081199059e-577095529159', '-9.93721765417e-38727419', '-9.9372176541699999999e-38727419', 20, 2);
    T('5.740938996209065682719935797929540170575240511180371732e+880887455', '-5.676545046325100639635864015e+82906391800990', '-5.676545046325100639635864015e+82906391800990', 55, 5);
    T('-3.6158705398145622e-1217523146939', '-564605907055017342207877685479350426790061735885322243003.0', '-5.64605907055017342207877685479350426790061735885322243003e+56', 69, 1);
    T('-4.44482916950377191384154647712765811459e+463189360989', '2.9643936335096566e+35', '-4.44482916950377191384154647712765811459e+463189360989', 66, 3);
    T('-6.845581638206779961275102206490366099147705189188354059489e-545202054', '-4.1584411e-409797', '-4.1584411000000000000000000000000000000000001e-409797', 44, 0);
    T('-9.2613450759e+903006', '-8.35e+803', '-9.2613450759e+903006', 23, 2);
    T('-2.5201042321969654506411911346864e-525034', '1.48782379540609785581e+57791235924', '1.4878237954060978558099999999999999999999999999e+57791235924', 47, 3);
    T('3.222340108346324857214498929194267400776890004407e+7434774449238', '-1.825736e-507372', '3.222340108346324857214498929194267400776890004407e+7434774449238', 98, 6);
    T('9.89108487306316962718916144707959462324134763e-526', '6.33803482036445989901157168012906495443992892461919405373049e+61583033', '6.3e+61583033', 2, 5);
    T('1.0450180187576963e-304513838142096', '-3.424013276474198658145346531524686371507293039360e-23553055232953', '-3.42401327647419865814534653152468637150729303936e-23553055232953', 52, 3);
    T('5.582064929886028970027574930735472337864e-50', '4288237435632374450623266778229145.3', '4.2882374356323744506232667782291453e+33', 49, 4);
    T('4.35722316680665606799552086210793536267077487659064e+9230491047', '-9.6696928847068942236787658140373060239085159643062101e+920005473361099', '-9.669692884706894223678765814037306023908515964306210099999e+920005473361099', 58, 1);
    T('-2.5573092933884765e+6584293055', '-8.2770919221838561972982019022695529384178965147e-871108785261', '-2.5573092933884765e+6584293055', 66, 5);
    T('-1.4325605560766604481628645292518655790e+13961', '8.10e-539083', '-1.432560556076660448162864529251865579e+13961', 65, 5);
    T('-1.07140168679007317501296418942144990e-9114477470961', '9.82549200714698068787e-69', '9.82549200714698068787e-69', 68, 2);
    T('-7.26178104220946611908037917931695201322650425140e-90737023663', '-6.74944366992143483483353668301311490001e-2452514042', '-6.74944366992143483483353668301311490001e-2452514042', 46, 4);
    T('-6.0256455172953e+744485266', '8.66130892724522838906944529398854148433247098886700e-12902395003', '-6.02565e+744485266', 6, 3);
    T('-3.33071594285179e+2119326368', '-5.0472806905939419836740313040388085756588134e-3231192', '-3.3307159428517900000000000000000000000000000000000001e+2119326368', 53, 3);
    T('6.13896988200845907181813645903179278967e-358062194', '1.49995063463111590298143214654907340e+2307586495038', '1.4999506346311159029814321465490734e+2307586495038', 94, 1);
    T('1.19e-27', '7.80671793316728657396436e-5970382340834', '1.19e-27', 33, 6);
    T('3.491857519967012867623e-2169912639', '-1.62450118124369804879489868660302005617954199522048e+718833928174549', '-1.62450118124369804879489868660302005617954199522048e+718833928174549', 63, 3);
    T('1.4787410505928e-300', '4.69067653128688451854e-87122117929', '1.4787410505928e-300', 73, 4);
    T('3.8965045752319047132e+4161399120215', '-2.818405219791206425820642899722626517764737884167910135268e-555806', '3.89650457523e+4161399120215', 12, 1);
    T('1e-179100', '8.761242611075271250371e+30', '8.7612426111e+30', 11, 0);
    T('7.084450654661106731008764609448599487828e-187114', '-9.21005633626831988233702337977258799891e+8363', '-9.21005633626831988233702337977258799891e+8363', 50, 6);
    T('-5.5466798926e+50679', '7.8850854e-15277', '-5.5466798926e+50679', 63, 0);
    T('8.4717431176767752731089268029449994e+6177415981', '-9.11302839644144621048904161172954890335966e+134', '8.4717431176767752731089268029449994e+6177415981', 97, 6);
    T('9.0274194975372111959880955822978051e+51598896', '1.838963010654988657027874325348671177463916968800e+104', '9.027419497537211195988095582297805100000000000000000000000000000000000000000000000000000000000001e+51598896', 97, 2);
    T('-5.917891772943718454560783329403311656183742828523791554e+690601879340', '8.6144639213786654639405535692068381267521460580919e+429031745954179', '8.6e+429031745954179', 2, 1);
    T('688104838522919', '-7.42258980463876411892316352382810498e-615705289334', '6.881048385229189999999999e+14', 25, 3);
    T('7.1506351574307265289746128854048654685956944e+3437951110224', '2.86404026150196478143676964315037578870400e+57053881437367', '2.864040261501964781436769643150375788704e+57053881437367', 65, 6);
    T('2.59e-44', '-4.1330815248710319096717345e+273464422609445', '-4.133081524871031909671734499999999999999999999999999999999999999999999999999e+273464422609445', 76, 1);
    T('3.840995456410250541957429705985379832599436958219e+67916013389', '-3.8567881e+63327501101149', '-3.8567881e+63327501101149', 70, 0);
    T('3.77907260163222646536008304679623841617958252695051852563464e+524', '5.1470072210570832e-54493821442', '3.7790726016322264653600830467962384161795825269505185256e+524', 56, 6);
    T('6.99957911342303434066640092738499209e-8167165692631', '-8.0282701953394534106160500221898958793623103915860123540690e-7361989', '-8.0282701953394534106e-7361989', 20, 2);
    T('5.20691e-122446232860', '-3956303.1145017859191', '-3.956e+6', 4, 4);
    T('2.224137147378892443672610313175234228e+47610583252703', '-3.955968849134621184222913546700107806497713572430194830688e+889485864315833', '-3.95596884913462119e+889485864315833', 18, 3);
    T('-7.941201223e-577250791798', '5.44298e+443638', '5.44298e+443638', 45, 4);
    T('-121965244055400116579746424625050457016810698366483341875.21', '4.19264631097337260969773301616555e+119', '4.19264631097337261e+119', 18, 4);
    T('-9.48963170307793813312548716814371872589954285980461508269e+76721949012', '-4.4787203424104892e-25407323174', '-9.489631703077938133125487168143718725899542859804615082690000000000000000000000000001e+76721949012', 85, 3);
    T('-3.9776853113880598167325708956e+518556395666685', '-1.695955336726316059997e-637660098833828', '-3.97768531138805981673257089560000000000000000000000000000000000000000000000000000000000000001e+518556395666685', 93, 3);
    T('1.39850873276378719565131534798531586422e+132', '6.832480480367625487833472845385211772307544775714e+173782502046138', '6.832480480367625487833472845385211772307544775714000001e+173782502046138', 55, 0);
    T('-9.8826741974327290776278963556e+80854933941882', '9.12434979803460847e-629288607382', '-9.8826741975e+80854933941882', 11, 0);
    T('-9.2591237699434415688e-160595', '-1.2e+6822', '-1.2e+6822', 67, 5);
    T('1.5601745806542402205957043286344658075363297426e+1180', '-8.34305050132623e-2800091', '1.5601745806542402205957043286344658075363297426e+1180', 94, 5);
    T('-7.43997984446559696386051409597110e+21660', '-4.76363651153186532455058836777585607586044872220907e-20461', '-7.4399798444655969638605140959711e+21660', 45, 4);
    T('5.794576432178628104725643349118e+681682109680', '3.81635232098907206856038447985439499984416959121746810e-487', '5.794576432178628104725643349118e+681682109680', 75, 4);
    T('2.9787811893506115579388807057e+175204573692895', '4.38284275638469514580049717135653798461677899e-756017859', '2.9787811893506115579388807057e+175204573692895', 59, 4);
    T('-7.0e+32', '4.2208160982028e+67', '4.2208160982028e+67', 29, 4);
    T('-4.317130044e-63454041', '9.0330974448587563114973999375085889201318445230477952749247e+777592840305407', '9.0330974448587563114973999e+777592840305407', 26, 5);
    T('-3.36869724940041859645786530939574939192042e-6644711984', '4.7473963674125002e+88886729', '4.747396367412500199999999999999999999999999999999999999999999999999999999999999999999999e+88886729', 88, 1);
    T('6.36e+678535173', '6.2316656022960407230641449681889600372762776918850482341735e-8315070', '6.36e+678535173', 92, 3);
    T('-5.65850047224746973154593297878076247363458051e-499116647', '-7.34318165494352e-9432056792708', '-5.65850047224746973154593297878076247363458051000000000000000000000000000001e-499116647', 75, 0);
    T('-6.3598835846292e-372', '-3.3555630270429126e+468371380110876', '-3.35556302704291260000000000000000000000000000000000000000000000000000000000001e+468371380110876', 78, 3);
    T('7.684897294889409355004271e+7534055005', '6.1732284729514e+821656', '7.6848972948894094e+7534055005', 17, 2);
    T('8.21543932270331812063383004716896e+8538', '-6.7488278178782026615256499434725038207344018509400887830e+980313732', '-6.74882781787820266152564994347250382073440185094008878299999e+980313732', 60, 2);
    T('4843501820621312218351747623990256', '2.609660703792e-22334046582', '4.843501820621312218351747623990256e+33', 61, 3);
    T('55241970911471346418996231713211372402763041072.481140855', '2.4983120545e+3599', '2.4983120545e+3599', 33, 5);
    T('0.0494', '-7.0780345814589143334531519659289531947919148e-78720474822052', '4.94e-2', 95, 4);
    T('-2.165652952813147923919938868749242701270880e+107509143', '8.834766881207550865373999614606e+11260289', '-2.1656529528131479239199388687492427012708799999999999999999999999999999e+107509143', 71, 1);
    T('7.105132385662976802122227277060637e-97156', '940506138664441057215587714717276.73640366', '9.4050613866444105721558771471727673640366e+32', 78, 5);
    T('-5.363419158747013440018007318199613978439316e-450', '6.534409091574125625974155524131223956528529e+736568077147638', '6.5344090915741256259741555241312239565285289999999999999999999999999999999999e+736568077147638', 77, 1);
    T('9.93410353097546405e-66', '-1.9651833053395389e-86515656', '9.93410353097546405e-66', 71, 4);
    T('7.557694962130305064785205924687020591760682770693533070e-916198274389294', '-6.45991530724000832297070393616573274674771428e-91376520463385', '-6.45991530724000832297070393616573274674771428e-91376520463385', 70, 3);
    T('-1.64751522082977460610131538018489249923402121332004183560e+4510', '8.4593126953389870954221467493600475e+28914551538', '8.4593126953389870954221467493600475e+28914551538', 77, 0);
    T('-2.590501545393208081e-72398759961', '-87288', '-8.7288e+4', 50, 1);
    T('-7.174000185907147671234798230478951338231876794e-319', '-8.949352153878557315364890932433599964336005900630539564513e+909324858637785', '-8.949352153878557315364890932433599964336005900630539564513e+909324858637785', 73, 1);
    T('-2.27060544769101412646179599709e+1454265', '-8.9765874339492284056912986220044290160046482286408401e+146', '-2.2706054476910141264617959970900000000000000000000000000000000000000000000000000000000000000000001e+1454265', 98, 3);
    T('3.3530034477e-83582190', '3.4e+754271206383783', '3.4e+754271206383783', 27, 5);
    T('5.316923533700563675090365332432291588678814e+44', '-4.2849600840429103647130731719896838676350090606e-859111535956', '5.316923533700563675090365332432291588678814e+44', 80, 2);
    T('-4.96231332007008777537735511520130758699619723833e+68603614529', '-2.6340256697989827048144603731375474725e+658787', '-4.9623133201e+68603614529', 11, 0);
    T('1.9383968775699561939022500e-3879048', '9.755230061281549892253588639e+80', '9.755230061281549892253588639e+80', 62, 3);
    T('-9.693466204700123732126260539375956943e-47', '-7.8791572077996156612659283838931381576751507712238377873922e-81', '-9.69346620470012373212626053937595773091572077996156612659283838931381576751507712238377873922e-47', 99, 6);
    T('6.2563286054577206131549240e-644503490075', '2.976171553447900770239423451635958064621197e+208263475693428', '2.97617155344790077023942346e+208263475693428', 27, 2);
    T('-1.1871798889802752864438748124976e-704289244315530', '3.1287143432482006595e-270', '3.1287143432482006595e-270', 33, 6);
    T('7.61880952984483022935267668194695548e-18', '-2.97608812741255354199238981672366207803682441858681816e+141', '-2.97608812741255354199238981672366207803682441858681816e+141', 96, 4);
    T('-6.35818258828528189416106046250851048964e-776283753', '-2.19704523656877622301658e-32521031', '-2.1970452365687762230165800000000000000000000000000000000000000000000000000000000000001e-32521031', 86, 3);
    T('2.8211609e+941909', '3.38134056516493293120340863146101874341181565911254396e-10123157540121', '2.8211609e+941909', 43, 4);
    T('7.03208407e+533', '8.847694701242621740489972083875047385251208115279094763608e-6703539531', '7.03208407e+533', 68, 6);
    T('4.300440384316677702737498115789482258110302174806675815e-82960813', '2.93825256317373834169782390e-59822', '2.93825256317373834169782390000000000000000000000000000000000000000000000000000000000000000000001e-59822', 96, 2);
    T('-6.8e-7', '98901759096402320270974159246619725.125010805', '9.8901759096402320270974159246619725125010125e+34', 49, 6);
    T('-7.035968778466629242566322512871021125290096e-93258281733781', '-6.434778917e-227252802', '-6.434778917e-227252802', 28, 5);
    T('-2.0340216773288466223854914389e-495336059099', '2.516653576762929820577431150195933725356e+583', '2.516653576762929820577431150195933725356e+583', 71, 6);
    T('-8.6251436249361868600706364433689522411857020974476116e+54', '2.367136714025102237251476960889090118803529125094e+20371676', '2.367136714025e+20371676', 13, 3);
    T('-3.67320434407e+593233860562', '-1.98697796742180709635348254752174102998e-9222800874611', '-3.67320434407e+593233860562', 54, 2);
    T('-2.9e-283276642140688', '-1.9198e+11', '-1.9198e+11', 18, 1);
    T('6.198372899441892703239184472070163191901286031190e-522140914371780', '8.9883854530042482583862089021169e+227333699', '8.9883854530042482583862089021e+227333699', 29, 6);
    T('5.8924590673377419455e-4210870820', '4.3841924648036e-725333', '4.3841924648036e-725333', 94, 5);
    T('-1.837826972657299846348691237855392623224925425146263e-5804658109493', '7.657898627043e-81966476587744', '-1.83782697e-5804658109493', 9, 2);
    T('-8.215614914988e+445400', '3.37998467384597156427155579621228e-284086027', '-8.2156149149879999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+445400', 98, 1);
    T('-5.14308480683419907508191874579387647144e-699716591808', '-4.6476929012041386547570114611908195726932021426582673e+890', '-4.64769290120413865475701146119081957269320214265826730001e+890', 57, 0);
    T('-2.0470e-75', '-1.223307804786853150555774103547098619273e+6453', '-1.223307e+6453', 7, 2);
    T('1.1192036828936928545096068621032444e-594', '-5.84693608805390371096723885985712402439760898523714224925303e-8278575', '1.1192036828936928545096068621032443999999999999999999999999999999999999999999e-594', 77, 1);
    T('8.28725305696410156603e-5710', '5.1092e+574982', '5.1092e+574982', 69, 6);
    T('2.9663907653571393737723708e+79633', '2.2772435662226331973020414829771135051940578573410696625e+6852584744875', '2.27724356622263319730204148297711350519406e+6852584744875', 42, 0);
    T('-7.88858107802800505657963e+98423042529939', '5.43397399088130157624124555477830e-8777591447357', '-7.88858107802800505657963e+98423042529939', 60, 3);
    T('-1.4677650130891531286850876316462614398326414026933e-74', '2.59518859702725570508629854423397989674974856024154719255486e+35889', '2.5951885970272557050862985442339798967497485602415471925548599999999999999999999999999999e+35889', 89, 3);
    T('-8.117424838504776345e-766367034880483', '-9.292569667391941e-7072196555577', '-9.292569667391941e-7072196555577', 66, 5);
    T('4.80154658330598499e+712439028011', '1.6941537802084037296315858261e+251079', '4.80154658330598499e+712439028011', 25, 6);
    T('-8.69871243007236e-67925560', '-1.009971290526237972136718547839455744335547314679e+3516086950', '-1.009971290526237972136718547839455744335547314679e+3516086950', 68, 6);
    T('8.1432477252711925741594449396889264846713497005405210822158e-98301349', '1.314e-425602318489212', '8.1432477252711925741594449396889264846713497005405210822158e-98301349', 90, 5);
    T('-9.275830621415846743304e-61603010340864', '2.092136012445388219093931611140135467086142015e+78634362', '2.092136012445388219093931611140135467086142015e+78634362', 83, 0);
    T('7.5889690297816627152960224352e+136860046785010', '-4.921540323588157003797178429e+34612910396', '7.5889690297816627152960224351e+136860046785010', 29, 3);
    T('6.35021276469e-29', '-8.620672691408042e-258163', '6.35021276469e-29', 98, 0);
    T('-8.7171017150088219667e-1707885222', '-5.7084461577363058900029032162112130e-340110558903134', '-8.7171017150088219667e-1707885222', 40, 2);
    T('-1.8139897083835954859775819156133199420183163352274767980e+7194', '-4.9133771198592e+1516', '-1.813989708383595485977581915613319942018316335e+7194', 46, 1);
    T('-536233526658829900215.492364523', '-8.8467639193e-3951652859016', '-5.36233526659e+20', 12, 3);
    T('-7.84284363674170935350838766e+34', '2.93733981465046244803e+27', '-7e+34', 1, 2);
    T('3.8e-84', '-7.846514515454862041113599605695440031167e-906696260', '3.799999999999999999999999999999999999999999999999999999999999999999999999999999999e-84', 82, 1);
    T('-9.98921363191680603509687078919299235e-3391013', '-8.25449366051930206e-360', '-8.254493660519302060000000000000000001e-360', 37, 0);
    T('-3.3953774957639097987909802e+343439272716060', '7.8074752428279996918616017539e+9108509', '-3.4e+343439272716060', 3, 4);
    T('-3.080445635693873239658205795e+254655873', '-3.37170775730085350867396449485233163258e+46', '-3.080445635693873239658205795e+254655873', 59, 4);
    T('-4.710933539517029674427709983709916e-6436016784220', '4.29133801971e-8802', '4.29133801971e-8802', 54, 4);
    T('-6.312721008097451758649648574805214251800384384975956e-863401056961', '-7.456294392553455308916975e+379786', '-7.456294392553455308916975e+379786', 96, 1);
    T('8.99987293239905080e+2314304', '-4.10389131472157959602296618909356884718995495500897566e-89254208', '8.999872932e+2314304', 10, 3);
    T('-2.6122365058145291267193164957536586026111502559e+38061524', '-6.919500920521e-5416405', '-2.6122365058145291267193164957536586026111502559e+38061524', 64, 2);
    T('5.319009821483286015e-1063738568', '-8.600639182821974511697227432475083652e-50476', '-8.600639182821974511697227432475083652e-50476', 81, 0);
    T('-4.056360996944480581374964969362231881615504361803351317278e+127', '6e+804876881', '5.99999999e+804876881', 9, 3);
    T('7.8112783730487996249972635329958489076884591453844877096e+1549815387', '2.526920730129524998336044474909062833187913896024749e+2494604385', '2.5269207301295249983360444749090628331879138960247490000000001e+2494604385', 62, 0);
    T('-7.83252561524537047860e-818377077137', '-5.564669480736e-1744942205573', '-7.832525615245370478600000000000000000000000000000000000000000000000000000000001e-818377077137', 79, 3);
    T('5.8938248864120864656996707376847161639827925997717e-7319929908638', '2.37970776921e-3924815623', '2.379707769210000000000000000000000000000000000001e-3924815623', 49, 2);
    T('-7.3561671431593609877392318699716886893271204725e+575906988', '2.4252756732862273033612859021710270480071377350e-792', '-7.36e+575906988', 3, 3);
    T('-6.148853e+8592817572397', '1.7440769523197692809880948835874207984455563869e+35999455987177', '1e+35999455987177', 1, 1);
    T('-78217032945044074783602133909765225046256641066324075711.7', '-9.158e-549423763', '-7.9e+55', 2, 0);
    T('8.26323067767249607548682941418077778894408781378239567e-394941905454', '-2.17855452e+697812', '-2.17855452e+697812', 71, 6);
    T('-7.182503214647149064007085462484464197751793e+167133505890', '4.2677126646859018849532523e+191', '-7.182503214647149064007085462484464197751793e+167133505890', 82, 5);
    T('5.328800466773141746901385445296971315414518419e-69825680335', '-5.25799886484483969761768689209001001018566868541124301617186e+700080856354', '-5.2579988648448396976177e+700080856354', 23, 5);
    T('3.64813430705861460968158290865413239585697871850911916230e-757849136', '-1.231933e-602', '-1.23193299999999999999999999999999999999999999999999999999999999999999e-602', 69, 2);
    T('7.1966473629019524854893001344701252166866993529126e-26424276059293', '-6e-2356466016', '-5.9999999999999999999999999999999999999999999999999e-2356466016', 50, 2);
    T('8.917480597113054249988234161056593772624652068571206486346e-59805115303', '-2.45402699577491666749323657657511e+52421728932802', '-2.45402699577491666749323657657511e+52421728932802', 71, 4);
    T('-8.6728329309785267003508297405e+550', '2.4205654939570577570149610056180379943481e+44', '-8.6728329309785267003508297405e+550', 65, 5);
    T('5.1941614642789977501205e-13300', '4.48982008318157975891e-9969716096', '5.1941615e-13300', 8, 5);
    T('-8.58374723244556743593971109278598728009013942525300862e+375671261906131', '-1.561594454067135306262338876682906690374171309358326235714e+1565', '-8.583747232445567435939711092785987280090139425253008620000000000000001e+375671261906131', 70, 0);
    T('-4.980606947117473818540776382849057571324951072464e-1926', '-9.27821e+7979', '-9.27821e+7979', 31, 4);
    T('175733930543057974476800859103016024824435409699746.9306', '5.009639501927781448410498981938539212532563141859793174129e-5152138', '1.757339305430579744768008591030160248244354097e+50', 46, 0);
    T('2.1892e+722187221756', '-2.8582116911e-575958445444', '2.1891999999999999999999999999999999999999999999999e+722187221756', 50, 1);
    T('-54736494922835118267720162225757367039552323291837637201', '-4.528855495535150703090017761654e-4257103340535', '-5.4736494922835118267720162225757367039552323291837637201e+55', 74, 1);
    T('1.05e-62174752705940', '1.962166848359898648285972040374249606918901e-25', '1.962166848359898648285972040374249606918901e-25', 79, 3);
    T('-5.8885393108485512064641651014795735787e+99138009', '-7.84062e+3555', '-5.8885393108485512064641651014796e+99138009', 32, 4);
    T('-8.99183218795569091445375334e-6825', '-9e+889542', '-9.00000000000000001e+889542', 18, 3);
    T('-9.8057405e+53', '-5.177666350404464231458375238549e+367', '-5.17766635040446423145837523854900000000000000000000000000000000000000000000000000001e+367', 84, 3);
    T('5.608950062071e+4811627634834', '-6.1994355080853552884313026920409319e-44082655462651', '5.608950062071e+4811627634834', 85, 6);
    T('9.0566392532464702636142189049747717308402174769e-874', '25586711898254278856699692919821440988746488089030', '2.55867118982542788567e+49', 23, 6);
    T('-2.23743900299326e+19', '-6.7179509246265699654391852125e+65382457553', '-6.72e+65382457553', 3, 5);
    T('9.54117881073001521842821986838023733922773768492e-756', '-5.12e-23808', '9.54117881073001521843e-756', 21, 0);
    T('1.5096e+793528221872', '-8.598843775920167097090137034939541e-12658771387680', '1.509599999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+793528221872', 97, 3);
    T('-7.06020780968e+6050931334', '7.63091457986197636160827730282780164e-5313008255', '-7.0602079e+6050931334', 8, 0);
    T('-3.25064970593943135072081e+9269566', '9.54877583196174076247e-638', '-3.25064970593943135072081e+9269566', 38, 3);
    T('-856949181329919131892856015.99105613773492', '8.94952887239779322786007943294885039192196e+53', '8.9495288723977932278600794243793585786227686810714398400894386226508e+53', 73, 3);
    T('-4.71e+3812643', '9.66599724533358885303241182514e+4530', '-4.71e+3812643', 63, 4);
    T('4.4378681174721828586393209797034339684603262396346966142144e-14454', '8.6506827832127746284544e+492529491320', '8.6506827832127746284544e+492529491320', 66, 6);
    T('4.33848536619307e+514365', '8.06369903103310901058132e-51210810230', '4.338485366e+514365', 10, 4);
    T('2e+33912258798', '3.4064130979455006644954523e-617677031110841', '2e+33912258798', 18, 5);
    T('3.2052e-2947906492', '7.276369922e-823161965398', '3.20520000000000000000000000000000000000000000000000000000000000000000000000001e-2947906492', 78, 0);
    T('-9.7878696066670674915363310564504472250844503143725945575e+333258', '8.99050653272062795109034132025741564951336994729395e-30207', '-9.787869606667067491536331e+333258', 26, 2);
    T('-2.8745358440609375902357791219787186e-755646539', '4e+69', '4e+69', 55, 0);
    T('5.3e+4665125', '-4.9487004089746587848611257804613946012262538077708e-232281975077', '5.3e+4665125', 5, 2);
    T('-7.4426233047803440153570972442717701e-66832290657975', '6.09094710130751812245446288662e+11146', '6.09094710130751812245446288662e+11146', 55, 6);
    T('2.00548240821285e+3940535321', '8.0813454616602119672221794204798643500415e+5993265639441', '8.081345461660211967222179420479864350041500000000000001e+5993265639441', 55, 2);
    T('-1.926810321099269820306658163144076656785229504346758560e+18266456566037', '-2270196692015733796648743208850.45044', '-1.926810321099269820306658163144076656785229504346758560000000000000000000000000000000000001e+18266456566037', 91, 3);
    T('702454276977894761567723412177161391269.69324', '8.90017295250712551991195074e+84240982', '8.90017295250712551991195074e+84240982', 32, 6);
    T('-4.97015e-22376850008257', '-5.49872632554663145606648041039101e-293568032', '-5.49872632554663145606648041039101e-293568032', 48, 6);
    T('8.504382972463822045579026837848683897619849150619726555845e-281692588142', '-7.51997305726833271805400995892479802387605665455e+3560732', '-7.51997305726833271805400995892479802387605665455e+3560732', 90, 3);
    T('3.445669720334634658687407312472927771761427195223579021325e+27542721957666', '3066917.378', '3.44566e+27542721957666', 6, 1);
    T('-9.318688046595677188322164882593326e-75783648101', '5.2882219588647104084989839378813554240569594449523e+54895231977', '5.288221958864710408498983937881355424056959444e+54895231977', 46, 3);
    T('9.015206856e-3826542', '4.189780078616262275229291727298851e-1831290', '4.189780078616262275229291727298851e-1831290', 38, 5);
    T('8.5044178375160977e+9015890199', '-7121629808526003911.327162582853', '8.5044178375160977e+9015890199', 68, 6);
    T('-8.650488351649e-493881', '-7.429333198e+61924346', '-7.429333198e+61924346', 49, 5);
    T('-2.35875167660317026254359959085578485446718422276e-713305810979707', '-7.2708273185346927415258978137125514174538347e+237324570047', '-7.2708273185346927415258978137125514174538347e+237324570047', 97, 5);
    T('5.41420612250769321948965050601610036122002995233489931e+720189', '-7.5070422987622322638326659183393301190076184593214791772e+19147', '5.414206122507693e+720189', 16, 3);
    T('-5.10791230615851498193073703046527e-4918', '6.93285e-104599017258326', '-5.10791230615851498193073703046527e-4918', 62, 5);
    T('2.79446e-10851', '1.476027e-13423985617262', '2.79446e-10851', 82, 5);
    T('-9.473123762064636804651956105643040729163155905734e+1764154866', '6.7951914598771753011039178756972925512232462224559740344077e+15331', '-9.4731237620646368046519561056430407291631559057339999999999999999999999999999999999999999999e+1764154866', 92, 1);
    T('-7.098263050324352623309372025480378e-565892263639531', '-4.4542593033225749338255503863871414017181492507001969949197e-61', '-4.45425930332257e-61', 15, 6);
    T('7.06031086e+3632990', '-6.652e+198738', '7.06031086e+3632990', 53, 2);
    T('6.47525308668923257265072126e+582643', '3.17575857354332299550163596239e+51560102', '3.17575857354332299550163596239e+51560102', 57, 4);
    T('-4.2091334234709539222886538752660124906202721076537e+555927', '7.93422153013e+281417145458802', '7.93422153013e+281417145458802', 72, 5);
    T('6e+26178287', '6.20664621901e+298084432519', '6.20664621901000000000000000000000000000000000001e+298084432519', 48, 0);
    T('9.782408907379202666319601970086808013783556424e-9440728', '8.4312e+96290355611', '8.4312e+96290355611', 83, 6);
    T('-8.95523821703610329021337121479847736229493e+247055', '2.346828218793217716806222134935244373289800831969e-37809995', '-8.9552382170361032902133712147985e+247055', 32, 3);
    T('-4.0993371300423129875467174236892207058e-42818472903698', '9.9949054922782743513486008377649357e+66033007035', '9.9949054922782743513486008377649357e+66033007035', 60, 0);
    T('5.4981723149355944178168809478388424873453194055e+2320196850863', '-38215.5191203978520612801133267821', '5.4981723149355944178168809478388424873453194055e+2320196850863', 100, 0);
    T('2.52982556523822423754686500759570671327800e-778485374', '-4.839621392676056831188852566801787811878474e-451', '-4.839621392676056831188852566802e-451', 31, 6);
    T('7.003222594108344693542377538804436e+36', '-3.910379552619e-6924056694222', '7.003222594108344e+36', 16, 1);
    T('-401805323298727233063.76', '-2.888792680571248873379898185745265482822962805e+363337124710', '-2.888792680571248873379898185745265482822962805e+363337124710', 94, 6);
    T('-6.25918972e-40728776333', '-5.405174879541691305992487176989753314848282972e-9051790714', '-5.405175e-9051790714', 7, 6);
    T('3.50795317721344206836515801525567933043773898064162444557734e+791508', '-3.832564930356444241787414678856904870e+9611306', '-3.83256493035644424178741467885690486999999999999999999999999999999999999999999999999999999e+9611306', 90, 2);
    T('8.917865607075849453881852416465e+8087610167713', '-9.20809507988411019732362818015842734537795496141660020190e+61', '8.917865607075849453881852416465e+8087610167713', 99, 5);
    T('-9.2227031e+642269333401160', '6.3097672022789372830871841562668042480661787e-8682', '-9.2227031e+642269333401160', 26, 5);
    T('-5.6651417555249254790568693641695957e-123112037', '5.503821995844964233657385614351e+497032303', '5.503821995844964233657385614351e+497032303', 100, 6);
    T('-2.56173032886745669225e+1663568', '-635371908429245831330883670394597.7591399', '-2.56173032886745669225e+1663568', 37, 5);
    T('7.1181683452036718880977084327878420e+56810489314915', '9.049913454837650200334439229288438385708796951601005826e+3407732936', '7.118168345203671888097708432787842e+56810489314915', 67, 5);
    T('-1.62250641740814088805207986654e-30148571784804', '-3.426047703776260700166098135483588902124848128532256269e-17295', '-3.42604770377626070016609813549e-17295', 30, 3);
    T('2.848456754564770036684018403e-8945397353044', '-5.4285855287685429970775388770445875196975801370274e-672614304517138', '2.848456754564770036684018403e-8945397353044', 92, 5);
    T('-6.314441260013735164536176376920388803035330208056715800549e-62616659', '-5.8494e+20772136910695', '-5.8494e+20772136910695', 34, 6);
    T('9e-5288679', '-3.6344859899784514753955686030623179030460551616778485030e+105', '-3.6344859899784514753955686030623179030460551616e+105', 47, 1);
    T('-8.63096018382742522868715478276566827894236359e+53', '8.485621107007215249341373224582800446600e+2610803', '8.4856211070072152493413732245828004466e+2610803', 92, 5);
    T('6.3566273704839402622230157489475270515383689824515118883e+67516345338', '-2.850785538163245912951594997705694542614795847963e-75244172071', '6.35662737048394026222301574894752705153836898e+67516345338', 45, 5);
    T('515056708051444497812216100592.53', '6.94475014524012175023381540341647e+397', '6.94475014524012175023381540341647e+397', 63, 1);
    T('4.0810499289980535004428840169776852166917e-485565920995642', '9.0727900449152961002858049901612987771e+96054', '9.0727900449152961002858049901612987771e+96054', 41, 1);
    T('-83409764235799623375770226793418.26', '5.37023060617372301030851171912721708432765e+32783443936015', '5.37023060617e+32783443936015', 12, 4);
    T('-8.9991363335289292497e+5785484009', '-7.327985875045849316787722e+26', '-8.9991363335289292497e+5785484009', 36, 6);
    T('-9.519308242911093014963850046211625002093866740423159e+107', '8.2359135348443886405855185706910349233e-212720336916', '-9.5193082429110930149638500462116250020938667404231589999999999e+107', 62, 2);
    T('1.918152e+605', '9.7087732758095363136577133151e-34877791025', '1.918152000000000000000000000000000000000000000000000000000000000000000000000000000001e+605', 85, 0);
    T('-3.09200319994125018451730944717616139054e+465024209969007', '-6.3307642416672128378704700397822108001113059728239064e+2676314982526', '-3.09200319994125018451730944717616139054e+465024209969007', 97, 1);
    T('4.699029541247713634387e-9031221760', '-7.480006001463986214393816183045e+68615', '-7.480006001463986214393816183045e+68615', 55, 4);
    T('-8.81e-711', '-6.0935018067e+43546579', '-6.0935018067e+43546579', 98, 1);
    T('4.205280476880954464943e+656075548999435', '2.5159146880359469353648782856428582e+59330', '4.205280476880954464943e+656075548999435', 90, 6);
    T('2.1454481161385554383989768217097026424049416e+716869427', '-6.126890861500495232110986864602752600e-60', '2.1454481161385554383989768217097026424049415999999999999999999999999e+716869427', 68, 1);
    T('-2.840195165e+8217', '7.2e+4379', '-2.840195165e+8217', 97, 4);
    T('2.2903212621868046820587e-2858840', '-3.45e-41495', '-3e-41495', 1, 1);
    T('-6.83034633914366121329705388722453984178232738721733548686151e-7630112128', '-2.6783154567809052e+15636082272428', '-2.6783154567809052000000000000000000000000001e+15636082272428', 44, 0);
    T('-7.55957236252972153640808764611227420089308934548552818924e+62646', '2.472828806909432e-88944085058334', '-7.55957236252972153640808764611227420089308934548552818924e+62646', 58, 5);
    T('6.6065244068768912933240175059791e+571802', '-8.199140511531520155042184689073472933007770858160940e-92554', '6.6065244068768912933240175059791e+571802', 84, 6);
    T('-7.84714633968887446462714e+4547800751960', '-1.297297239825595665239695749263826134e-317', '-7.84714633968887446462714000000000000000000000000000000000000000000001e+4547800751960', 69, 3);
    T('-4.0e-669093', '-8.827036126061954495411397429e+6429628512790', '-9e+6429628512790', 1, 5);
    T('-7.957442821776637e-24420966303503', '-0.000009104437243229415111283914446', '-9.1044372432294151112839144e-6', 26, 1);
    T('-8e+938989', '-8.32863259e-8275251620526', '-8e+938989', 18, 2);
    T('-1.0332205052359124371220927e-182843', '6.27885392e-3430', '6.27885e-3430', 6, 5);
    T('-1.86743922e+8360934767486', '-8.0663e+9567', '-1.86743922e+8360934767486', 55, 5);
    T('7.7764045286250115423318392911716e-54417807', '-94450780054683071', '-9.4450780054683071e+16', 94, 0);
    T('2.88958517365884902718724111636303364930542917720969672e+67', '-9.29303878666312554697641548245985390e-179227544855', '2.8895851737e+67', 11, 4);
    T('-6.9e+210746598', '3.5794806023159423845723670005494678344411357e+96368512841', '3.5794806023159423845723670005494678344411357e+96368512841', 58, 5);
    T('8.09701043849116e-8729', '-6.53038425170568544081981193044557597855546893279870716e+89705218375421', '-6.530384251705685440819812e+89705218375421', 25, 6);
    T('5.52683011796517676527883856819957e+8689', '4.9421624022530694723337968429962e+35', '5.5269e+8689', 5, 0);
    T('2.53777022e+17', '-1.103170107458658741827425312638445405947124e-2209805771249', '2.53777021999999999999999999999999999999999999999999999999999e+17', 60, 1);
    T('4.61327994903977003187446197154947611633689280825494696e+100', '-8.23676409576104122601850325256586556908519820288266e-91145', '5e+100', 1, 0);
    T('-1.88199921163559391215309153088583756272678e-9861424368', '-1.74423659065346366857828427049047295814696e-294235', '-1.74423659065346366857828427049047295814696e-294235', 42, 4);
    T('-4.1320891251446960475070094622215816640112010669980e-96025807689984', '9.855108600824763169596188891182525381651855922669982e-91530295', '9.855108600824763169596188891182525381651855922669982e-91530295', 97, 4);
    T('7.574181692062065102620750992851856451164091721e-53688266964112', '-8583189.78752187', '-8.58318978752187e+6', 76, 0);
    T('3.11007374702817476602261326056622e+8898454', '-9.4814311e+95194463663528', '-9.4814310999999999999999999999999999999999999999999999999999e+95194463663528', 59, 2);
    T('-2.8235948e-7384614723314', '-5.897788251224555929025329180292192905117561996397e-10534084896', '-5.89778825122455593e-10534084896', 18, 4);
    T('6.58032076001e-64474515793950', '5.80438984841744931137694543e+93951291723972', '5.8043898484174493113769454300000000001e+93951291723972', 38, 0);
    T('-5.0910946286837253840641863620200966581225e+124', '1.15257371512141283958459468005809863e+9965263', '1.15257371512141283958459e+9965263', 24, 1);
    T('1.160094602225968313519490e-865079', '3.683e-3063763393', '1.16009460222596831351949e-865079', 79, 1);
    T('-433901316025794013.886', '5.56638483889303298542130221e+38', '5.566e+38', 4, 4);
    T('-82089407419585270314518103276591098081822.73', '-3.7957884468830859538884372970783841505963284593e-33902', '-8.208940741958527031451810327659109808182273e+40', 86, 1);
    T('7.65296638491534703989090511128530628635044e-42', '3.26977568066383853310460587380579075e+24539152772', '3.269775680663838533e+24539152772', 19, 5);
    T('-6.80470190748366277423317277249183515429608573570643774332e-303346894', '8.52103966446738376584611675849273836270709103e+9275383', '8.52103966446738376584611675849273836270709103e+9275383', 70, 6);
    T('6.377905623859575573287665330559035978640036733e-3249138', '9.292027534840983046249e+9270865', '9.292e+9270865', 5, 5);
    T('5.70594411081e-91972896', '7.355716160916525597960502585e-211091179837', '5.70594411081e-91972896', 48, 6);
    T('9.322292697793266947661267e+4446', '5.36617957949446743580122003e+96', '9.322292697793266947661267e+4446', 34, 1);
    T('5.6e+70619132', '-6.8824548507567517537100644719836e-519877673', '5.6e+70619132', 71, 4);
    T('-5.121195317714292225162696075900135861e-5781335', '8.912437342863352259767317980274730939740801e+168365007533', '8.912437342863352259767317980274730939740801e+168365007533', 66, 6);
    T('3.53329251e+7499', '-6.52954456522e-13681130116', '3.53329251e+7499', 95, 0);
    T('2.6877621018563887668493942173290e+3120760', '-8.006228516973528379812907658915730e+126', '2.687762101856388766849394217329e+3120760', 63, 5);
    T('-1.469587152757691949e-38588355323232', '7.16960910037801135976309323549559473346347274525255e+21808299457378', '7.1696091003780113597630932354955947334634727e+21808299457378', 44, 4);
    T('-6.4225085509134577664600897281522e+7335721305004', '6.9861e-808', '-6.42250855091345776646008972815219999999999999999999999999999999999e+7335721305004', 66, 2);
    T('-2.14476250630935755630e+17132', '-7e+4544735066883', '-7e+4544735066883', 26, 6);
    T('-8.215528850341567666823308966225309e-11378', '5.141878e-90292', '-8.2155288503415676668233089662253089e-11378', 35, 1);
    T('0.000049', '6.297778249884669064695434100758614905455819274112830e+73', '6.29777824988466906469543410075861490545581927411283001e+73', 54, 2);
    T('-3.9021709911347592504269554269592685660110353738244439542e-14', '-7.7062465442828e+827', '-7.706246544283e+827', 13, 6);
    T('-8.98e-236132133', '-4.98889943560e+2570379652', '-4.9888994356e+2570379652', 93, 5);
    T('3.32700737428966924432985169146e+42833538543', '-1.7300e+6551', '3.32701e+42833538543', 6, 0);
    T('-8.117e-73939914', '15048857893410118951268177960396948659029', '1.5048857893411e+40', 14, 0);
    T('-5.233448319539816928852672286663469142825104859e-43525733907', '8.80936676964916448654e+632', '8.80936676964916448654e+632', 66, 5);
    T('1.170096044e-63334185483', '2.6865442e-706596', '2.6865442e-706596', 55, 4);
    T('1.019213078950785021066388e-9783674586', '-1.9646016678722685662416155715274310294525752806e-363', '-1.9647e-363', 5, 3);
    T('6385554425073652252819252408', '3.2690350564203424338715e-8838587545421', '6.38555442507365e+27', 15, 3);
    T('6.28327e-35', '-8.74371487401636898011244499556620649957887858086586673428e-657928676', '6.28327e-35', 81, 0);
    T('5.86745276394199175499243e+373230915310', '588238017978921163202467020908528301845433501874.5', '5.86745276394199175499243e+373230915310', 54, 4);
    T('-4.46111504193382622186366975267964609487e+9957660', '-6.4196520407442356150157180798057503635210190172266816e-17519', '-4.461115041933826221863669752679646094870000000000000001e+9957660', 55, 3);
    T('5.642015127e-73962924255', '9.04070879906654259364853146923430216114755e+48', '9.04070879906e+48', 12, 3);
    T('76390500.06756338066799683992851208911020078148688871423', '-6.80491279459251633105347855537103814892603629266146386917436e+244599727', '-6.80491279459251633105348e+244599727', 24, 4);
    T('-2.3358778e-9144498', '-7.907703e+34565974998', '-7.9077030000000000000000000000000000000000000000000000000000000000000000000000000001e+34565974998', 83, 3);
    T('-7.17192052642316479580146403621712303391861159931843e+32457', '-8.852852717776902908e+9924', '-7.1719205e+32457', 8, 4);
    T('2.7802003170590562948063893250061908134195218105537938389583e-1102441', '-7.928267515099431113794586426475334518199497516692750154280e-53507365', '2.780200317059056294806389325006190813e-1102441', 37, 5);
    T('-4.97744625744770476101e-445', '-6.368985625234145017635396084241530e+373565814', '-6.3689856252341450177e+373565814', 20, 3);
    T('-6.30344983897321227842964449541254257769953400561850739e+771906129085981', '1.13713158165219320335067072410016819418238814370546258e-454715240095936', '-6.3034498389732122784296e+771906129085981', 23, 5);
    T('7.752996581361370566881103451113578e-5618827950', '7.1118175670744670619683452222238e+61619807341', '7.1118175670744670619683452222238e+61619807341', 70, 4);
    T('-1.56099986454492266532996606615336219956e+3810187469253', '4e+66506777', '-1.56099986454492266532996606615336219956e+3810187469253', 64, 4);
    T('5.691176853877826e-20584', '1382587560841393204703886322471.763125', '1.3e+30', 2, 1);
    T('-9.67047497870424703473928435397122720e-1248', '2.45991057627e+178428151', '2.45991057627e+178428151', 36, 5);
    T('-1.3288147134737172826564500e-88224276', '4.963153648998143590306306e+76965', '4.963153648998143590306306e+76965', 93, 2);
    T('8.8764201573625580092391274767679480261251155347235338174e-740693384268419', '4.86296192462107914835343026096784e-85633999346', '4.86296192462107914835343026096784e-85633999346', 71, 5);
    T('1.116602007305459112e-409280261', '8.566597492712432844069e-226', '8.566597492712432844069e-226', 39, 6);
    T('4.2e+667750297', '-9.35022512383965709452963121741600e-30448766158427', '4.2e+667750297', 66, 5);
    T('-8.00112925184347257625171223276892344206e-159786241968', '2675318517043006372981.129401573', '2.675318517043006372981129401572999999999e+21', 40, 3);
    T('-2.0360e+4582281330434', '5.473132623758130e+7400931753164', '5.474e+7400931753164', 4, 0);
    T('-4.67806912447004649761065860933643622200218781824491333293296e-764411916656313', '1.58217730160562360070750917474496977e+23266323445', '1.5821773016056236007075091747449697699999999999999999999999999e+23266323445', 62, 1);
    T('-7.82678314244650273824630315985121579138826486413e+3530472262', '-4.56017421578374740370741314234540605979750110797503e-22151829245', '-7.826783142446502738e+3530472262', 19, 5);
    T('8.92940690041758536e-4445385', '-3.06e-94748', '-3.059999999999999999999999999999999999999999999999999999999999999e-94748', 64, 2);
    T('9.033e+530922562788', '-4.6790513649190164633810223226848093343544523e-2731', '9.033e+530922562788', 27, 5);
    T('-8.16223417831645177859311169735882341e+37657412327050', '-9.7269918693577492100231331226185678904e-80629344001587', '-8.162234178316451778593111697358823410000000000000000000000000000000000000000000000000000001e+37657412327050', 91, 3);
    T('-9.533836896063296340333227673e+53039503178455', '7.719928542395335940862470727183414776712361004e-764333', '-9.533836896063296e+53039503178455', 16, 1);
    T('-5.28563343408089524650590150148426526588865209502e-465997908079693', '-8.65668992384032641400058039196329601807143260228327242742e-2111043748', '-8.65668992384032641400058039196329601807143260228327242742e-2111043748', 97, 4);
    T('3.984916560064708111191426632004779700859973633763e+9065', '5.880888e-6206067906', '3.9849165600647081e+9065', 17, 3);
    T('-9.205124231e-191263067', '5.215161530757437402427e+728', '5.21516153075743740242699999999999999999e+728', 39, 1);
    T('-5.05125924768932879624612383155632946443353e-946', '-9.926468072698223361164880663265565946419208761584548e+829062', '-9.9264680726982233612e+829062', 20, 0);
    T('-6.9167824350035132590e-865230496439', '-4.980372106740291975646890e+57', '-4.98037210674029197564689e+57', 98, 5);
    T('-1.658426854007639461044995475292706e+84', '1.928946048698701755793e+9662080', '1.928946048698701755793e+9662080', 23, 0);
    T('3.18894118071e-466848244982274', '-3.8539425584349293109754547572227904453682841553e+3037', '-3.8539425584349293109754547572227904453682841553e+3037', 52, 5);
    T('-8.876409e+950938021', '8.836855740437927792392990191386728430865150793535333772305e+55053158734', '8.9e+55053158734', 2, 2);
    T('1.93182248888517682988815944995725109e+484244', '-4.4150023627761710e+866108362558', '-4.415002362776171e+866108362558', 65, 4);
    T('4.9489497130e-2783609', '7.08018182753166805859048970296715011331845005159720302e-42382783899', '4.948949713e-2783609', 25, 3);
    T('-9467172725024158609712399532476631595868081351727011124.841', '-6.7440453944551507721590516428281808195976767294304603522e-8828', '-9.467172725024159e+54', 16, 3);
    T('5.90341185e+939', '-8.4510534344878299840836889336e-826', '5.90341185e+939', 90, 0);
    T('5.7568025188876774280366740e+66912510', '8.9123295157565537183088432127331105158150841658861288e+2855320283060', '8.9123295157565537183088432127331105158150841658861288e+2855320283060', 71, 5);
    T('2.84444200577355056751347434337327454771e+50554549844152', '8.6960e+82', '2.84444200577355056751347434337327454771000001e+50554549844152', 45, 0);
    T('-2.28544731649701617475002795951184544608009e+5909155854', '8.29839344774294678703371247959711e-80547781', '-2.28e+5909155854', 3, 1);
    T('-2.638436846044119669e-92052', '2.6488278636675498e-9697123554990', '-2.638436846044119668999999999999999999999999999999999999999999999999999e-92052', 70, 1);
    T('-5.7933374856030846348105444204360933026842649392955298e-83571630301', '3.370001639e+614', '3.370001639e+614', 70, 0);
    T('3.08500641274e-9256600282', '9.60730567811e+2768', '9.60730567811e+2768', 93, 4);
    T('6.1847e-380921477', '-4.5664069484730000578e+835894515', '-4.566406948473000057799999999999999999999999999999999999999999999999e+835894515', 67, 2);
    T('1.992525659599e-2504652068', '-1.0687729293322002362306688149740102060079404e-47', '-1.068772929332201e-47', 16, 0);
    T('-1.299062883929682790318e-41489206829', '-7.8707427687921583769077e+43', '-7.8707427687921583769077e+43', 70, 1);
    T('-4.595705681774990968689635574473479806882647e-1725', '-6.905434609983883391116072245924e+94264507', '-6.905434609983883391116072245924e+94264507', 95, 2);
    T('9.000162e+9844510403615', '4.941508956072105173674115821e-580', '9.00016200000000000000000000000000000000000000000000000000000000000000000000000000000001e+9844510403615', 87, 0);
    T('2.918380437852e+1739', '-6.009480789973730387e-4053910513', '2.91838e+1739', 7, 1);
    T('3.745755087414893880302194413613061332482759913324988882216e-3520581708933', '8.9741980e+576407', '8.9741980000000000000000000000000001e+576407', 35, 2);
    T('7.7951011120198501788849020337783919e+260820895662', '1.719752e-537266542179', '7.7951011120198501788849020337783919e+260820895662', 60, 6);
    T('-2.39852122269834e-304583808350', '6.017460531192164755591e-53083388', '6.017460531192164755591e-53083388', 52, 6);
    T('6.047783727e-7150128151599', '-8.3749e-71274492218', '-8.3749e-71274492218', 73, 0);
    T('58312392080867641504383789884879576.067', '7.9437504054851935685448642049389e+995213117', '7.9437504054851935685448642049389e+995213117', 73, 3);
    T('2.54021202925053065858714542987804987e+279785978559944', '-5.489261022648981352331610e-475466', '2.54021202925053065858714542987804987e+279785978559944', 62, 2);
    T('5.66231125136991335247895179041279882263546006843e-45692', '2.3501407714e-656210064341', '5.66231125136991335247895179041279882263546006843e-45692', 92, 3);
    T('-6.7915352163190915174239437271826902303510225533760640e+574893', '5.9574296365632733943715226567e-231401', '-6.791535216319091517423943727182690230351022553376064e+574893', 59, 0);
    T('-7.7915945999235775963173804012950086981021e+903999690503298', '5.50186058031600273369447714336392443436e+658', '-7.7915945999235775963173804012950086981021e+903999690503298', 70, 6);
    T('-1.7412683363843996e-5498579', '-1.120897588492268440180996059197792767846998191333818857645e-62963297776', '-1.7412683363843996e-5498579', 26, 1);
    T('2.6565938864e-12520073', '-2.5320224743405743966708943176364e+37123275', '-2.5320224743405743966708943176363999999999e+37123275', 41, 2);
    T('-4.402977831295184009e-71180849883', '9.650111866324053800792e+601360150', '9.650111866324053800792e+601360150', 56, 6);
    T('-5.35935868379670496439685e-8747955403', '-3.8743774970084764524291978717543640008e+53927592', '-3.8743774970084764524291978717543640008e+53927592', 64, 5);
    T('-2.659373748676309790e-6575053', '2.113e-975098105154530', '-2.6593737486763097899999999999999999999999999999999999999999999999999999999999999999999e-6575053', 86, 1);
    T('-2.8e+75926', '8.02733730574753860361509695449288641e+256532289425998', '8.02733730574753860361509695449288641e+256532289425998', 49, 0);
    T('-7.7049275682967244324973125875589949631025202572211e+2254827301365', '-8.5468056330497e-4680', '-7.7049275682967244324973125875589949631025202572211e+2254827301365', 90, 2);
    T('4.9084495818943080547499999433020702732261e-106218391043770', '-7.98342462263411957501296324400250147327267651035019e+80625075919', '-7.9834246226341195750129632e+80625075919', 26, 5);
    T('4.470190280853519267233425812751e+97168947', '-8.8931320884783304675969546372362986543318614795e-478953192789', '4.4701902808535192672334259e+97168947', 26, 0);
    T('-9.435664598225904163111e+225078055', '3.95e-1272367', '-9.4357e+225078055', 5, 3);
    T('-7.72235162e+853', '7.94053899686419477201490637871597464e+36', '-7.72235162e+853', 19, 5);
    T('-5.833793480887192814579451803769458012851658567202181354e-411758', '2.3335237814581906270428019828461431913774643e+207464237425', '2.3335237814581906270428019828461431913774643e+207464237425', 86, 0);
    T('-6.6002363730705650685105737890754788e-1090281013768', '-42446449304286344538956492209659713445719808412834730.4', '-4.24464493042863445389564922096597134457198084128347304e+52', 76, 6);
    T('8.1968999755174373e+42089982416', '3.30019222e-812', '8.1968999755174373000000000000000000000000000000000000000000000000001e+42089982416', 68, 2);
    T('-9.810032224e+8405233', '9.358e+533866589', '9.358e+533866589', 88, 5);
    T('5.678537198483732705773563459e+927570678371067', '-3.4141617190017027130e-64149467100652', '5.678537198483732705773563458999999999999999999e+927570678371067', 46, 3);
    T('8.2570341987638430e-456', '-8.6714399938750461948928624049502055124482732925035e+5348919', '-8.6714399938750461948928624049502055124482732925035e+5348919', 98, 3);
    T('5.45', '-9.92413868054761152163107725496672235e-68598500834129', '5.45e+0', 72, 5);
    T('-6.132389216322300950833407917926353e+5551983037856', '9.73161215825749791730213188408530776142909518165e+24738', '-6.1323892163223009508334079179264e+5551983037856', 32, 3);
    T('-2.2224186812061764774716478663789354809430e+77', '-6.12060809158159843178939013047533787049266573224183813e-2093', '-2.222418681206176477471647866378935480943e+77', 74, 6);
    T('-5.032361989e+891190', '5.9508272051804154923163454429390998568795892048e+52758', '-5.032361989e+891190', 98, 3);
    T('-3395901496906760835304822850365718073665500255929001.1949627', '-2.0272300126470143994594691366209129398454e+118', '-2.027230012647014399459469136620912939845e+118', 40, 5);
    T('-4.05717382796e+555', '5.8175095965881469382106504298639857300810761939965759e+448535138689388', '5.8175095965881469382106504298639857300810761939965759e+448535138689388', 73, 0);
    T('1.467389233337e+22969301', '-5.52131510673981004779941690593102735201855999e+59', '1.467389233337e+22969301', 62, 6);
    T('-9.393228612813302968654534322952911499545803626132e+7433223285916', '-6.91126126757581593610e+5394179569', '-9.3932286128133029686545343229529114e+7433223285916', 35, 1);
    T('-9.96191557882571267200472283238476e+22975989', '5.66705697256350827544865867490270231013799321850124e+827148081', '5.66705697256350827544865867490270231013799321850124e+827148081', 68, 4);
    T('191837332228.63', '6.41813000917e-470', '1.9183733222863e+11', 50, 5);
    T('8.1e-71', '7.0096058548757254800816616626190268403400e+70755478476209', '7.009605854875725480081661662619026840340000000001e+70755478476209', 49, 2);
    T('6.89258644356349791878519988705705386e+1114215351', '-7.926640832174568861056644494950027575419801763445447599e-806891959', '6.89258644356349791878519988705705e+1114215351', 33, 1);
    T('797325115885765549169601065799936000209566660377399556505.2', '6.996e+576464', '6.996e+576464', 57, 5);
    T('-8.419572676773129855168e-738727103470579', '4.908079716107146256111614894586880718e-16432', '4.908079716107146256111614894586880717999999999999999999999999e-16432', 61, 1);
    T('8.96518672038750409052384781656585749889179641865e+70531434240', '-2.006185041687483642985808811105e-31074270727', '8.96518672038750409052384781656585749889179641865e+70531434240', 90, 2);
    T('2.7086712116921641227746794820642e-4132531036216', '8.473558364e+75350860889388', '8.473558364e+75350860889388', 89, 6);
    T('2.15806231029642410784294147504441e+974', '-4.9226453148356109474742372231782315882459769e+3285', '-4.922645314835611e+3285', 16, 0);
    T('-8.4049997087844632003855948567884219876e-7504', '8.26973e-163558', '-8.40499971e-7504', 9, 0);
    T('-6.0011773297e+747320991255', '4.98949195456039697997336426326346198179315472e-82', '-6.0011773297e+747320991255', 73, 6);
    T('3.9150632532520615533e+24228623', '5.81261043377034669608022991404624255116721123837458e+975377307', '5.812610433770346696080229914e+975377307', 28, 6);
    T('8.301710732913726083976975745763387010263794e+307096674', '-1.8575091535164818989322089884939330104787852317503e+1957541863', '-1.857509153516481898932208988493933010478785231750299999999e+1957541863', 58, 2);
    T('-2.28056095550894017811617093142e+6726100829669', '-7.3111172040797054524339694896985273e-281572963', '-2.28056095550894017811617093142e+6726100829669', 47, 2);
    T('-7.46857560e+53195237634806', '3.053685347812879247080674e-966', '-7.47e+53195237634806', 3, 4);
    T('2.2375036260644738287378878234409740459388975113933600468857e+51150', '7.95312766939933478e-7939284365', '2.2375036260644738288e+51150', 20, 0);
    T('6.8883218022607550561580739391065059729307834500909330e-907', '3.3302318216431436252654701084793030e+859087806', '3.330231821643143625265470108479303e+859087806', 36, 3);
    T('-7.4667870e-6126943305', '5.16529266834777960339713687113570750e+6351677', '5.1652926683477796033971368712e+6351677', 29, 2);
    T('1.30848587381679769240055683795921093351844956641294e+2051249', '-4.2815519523204571812599113847415899203495512687017203935463e-672838890190', '1.3084858738167976924005568379592109335184495664129399999999e+2051249', 59, 1);
    T('-3.7918062861786923551756832277479329778727e-71225', '6.9200181973858473905042e-73284', '-3.7918062861786923551756832277479329778727e-71225', 58, 4);
    T('-8.683360803503106563893671854999288318019630770779497989e-7754839', '-4.792728479626278212471227146e-6432266335', '-8.683360803503106563893671854999288318019630770779497989e-7754839', 71, 6);
    T('-7.4578636346840345931781673744598406023457013870121783948e-2035', '-4e-375', '-4e-375', 79, 6);
    T('-4.56777880393432294437795328594000731677431728523157246058e+53957501308', '-3.05077849768296086e-7048560490', '-4.56778e+53957501308', 6, 0);
    T('-6.15774018378070e+763', '70554658848195633404467546521078479471061776.857', '-6.1577401837807e+763', 93, 6);
    T('7.236797557861315878247824960089407012750785e-75619269278085', '-1.239768326670637972321446e+25', '-1.239768326670637972321445999999999999999999999999999999999999999999999999999999999999e+25', 85, 1);
    T('3.2184987956036210179778e+153', '2.0e+900912802123690', '2e+900912802123690', 23, 3);
    T('4.3333424182192258599882384813608769030400720255544085e-554062221423995', '-5.820709314313673880486882e+158', '-5.820709314313673880486882e+158', 56, 3);
    T('8.8699819677234e+218100', '9.635440730117890271996007982e+5347', '8.8699819677234e+218100', 38, 6);
    T('-1.873133326065010062481049472829190717723991769030e+527903574', '-7.224747974175935040754151751653e+555435644075', '-7.22474797417593504075415175165300000000000000000001e+555435644075', 51, 0);
    T('9.755260954556473336329002450186979120011402714589051e+3782', '3.8666361186378651945966335062498319e-236021', '9.755260954556473336329002450186979120011402714589051e+3782', 81, 4);
    T('8.1944143073311988608245e+87', '3.995359119688030202808985313514568599897785e+82730698642', '3.9953591196880302028089853135145685998977850000001e+82730698642', 50, 2);
    T('-5.811587459708672637443e-9547381266487', '-8.112051244685762318395019e-9333886609380', '-8.112051244685762318395019e-9333886609380', 63, 6);
    T('8.7410693678042968606756307036901e+75', '6.3e-60760535', '8.7410693678042968606756307036901000000000000000000000000000000000000000000000001e+75', 80, 2);
    T('5.61290e+25922857842189', '4.4205329352697716359397500797341572153609757346007e-7839078511532', '5.6129e+25922857842189', 7, 1);
    T('-1.197627e-564182548', '-3.0960894287601620394826343732919777575604111795916954e-9409383974971', '-1.197627e-564182548', 8, 5);
    T('2.580767142051933143083690394298593662028030608873563482e+89983662061', '8.329e-8572', '2e+89983662061', 1, 1);
    T('-3.497797994113226877760512278956148947700949e+19001355224', '-6.17446662558531554193671e+601', '-3.4977979941132268e+19001355224', 17, 2);
    T('0.90', '7.7624852829732467876510354481e+7183918', '7.762485282e+7183918', 10, 3);
    T('-7.1542818003425047e-338733248387', '-1.9504057617010e+6030092', '-1.950405761701e+6030092', 35, 5);
    T('-2.876563392325190918054906761836519587178095e-65524637', '-8.58241135049689412381e+52822007', '-8.5824113504968941239e+52822007', 20, 0);
    T('6.76883610063760361961225232888632494e+734250', '9.46871245377696746415058836145971030366175615687401e-69860128951823', '6.76883610063760361961225232888632494e+734250', 62, 1);
    T('-9.18775741212567489376282354e+5623735412', '-3.922971964477369622678844048081947442425685e+26275555747', '-3.922972e+26275555747', 8, 3);
    T('0.00000669935104247917762183730885534112', '-5.9689767223483337577516445091477147740829e-868', '6.699351042479177621837308855341119999999999999e-6', 46, 1);
    T('7.14280951629135e-10190141', '-6.899878391792169930957e-742899602', '7.14280951629135e-10190141', 63, 6);
    T('3.8095943499618248351e-5501600', '6.35580269059581469417363520374658744715297199527e-591', '6.3558026905958146941736352037465875e-591', 35, 0);
    T('-4.25265782717e+37015', '-9.229472377051814539708372029756560941330117e-86123027707377', '-4.25265782717000000000000000000000000000000000000000000000000000000000000001e+37015', 75, 3);
    T('8.61005269371670340414105667915378e+15271556', '-5.68031636884026088697383468626299426347724031e-597152310575', '8.610052693717e+15271556', 13, 6);
    T('9.0174873558864595e-47621748573494', '-6.10535853036992143278518304814254303138048552e+971026669973634', '-6.10535853036992143278518304814254303138048552e+971026669973634', 71, 3);
    T('6.158723880058077671380580759179625703159946200683667e+9871', '9.082286939068116656469945243563232221e+60', '6.15872388005807767138058e+9871', 24, 5);
    T('2.10511026184922453544e-22', '-5.61202310813e-386282930336474', '2.10511026184922453544e-22', 74, 4);
    T('8.9123426921051285847856e-5719', '3e+586804993446', '3e+586804993446', 62, 5);
    T('-8.4929913934e+905', '-9.47141080e+44', '-8.4929913934e+905', 95, 2);
    T('18686237391', '-5.2794668104768458437448589906e+30', '-5.279466810476845843726172753209e+30', 55, 3);
    T('-2.4459359586685881034e+789292170', '-5.1104277858058993121072299434109906959e-9378985577043', '-2.445935958668588104e+789292170', 19, 0);
    T('-5.277764812739724e+4058734535367', '1.125143216160e+574384782962', '-5.277764812739724e+4058734535367', 100, 3);
    T('5.078969953155197873e-971776870', '7.448800468981382020e+696', '7e+696', 1, 4);
    T('-3.82905654e-60868979315541', '5.659e+173', '5.659e+173', 84, 0);
    T('9.030230896204505e-70700264', '7.3303635129631542292e-6509157', '7.3303635129631542292e-6509157', 89, 4);
    T('-1.8381979804999561024642080e+929655447604', '-6.95766884803663187281852e+95', '-1.838197980499956102465e+929655447604', 22, 3);
    T('-6.579189520082695979399719458e+46791592319495', '-6.382981974326619431961856976028853427523e-4543195', '-6.57918952e+46791592319495', 9, 5);
    T('-1.68531500964750433e-40198298385', '-8.3668768115959362710589144572769838275686791e-667437413', '-8.3668768115959362710589144572769838275686791e-667437413', 54, 4);
    T('-8e-67126612320', '8.2395486448135006137795056554984179069963793926518087319167e-9410352589975', '-7.9999999999999999999999999999999999999999999999999999999999999999999999e-67126612320', 71, 2);
    T('-2.9714517521873e-99182780827', '7.1663e-9481426243', '7.1662999999999999999999999999999999999e-9481426243', 38, 1);
    T('1.86127198112437293310e-5706115', '-5.76e+1082', '-5.75999999999999999999999999999999999999999999999e+1082', 48, 2);
    T('-7e+86514616', '-2.8059167963810648414152895115683626806e+353', '-7e+86514616', 85, 6);
    T('-5.73237217240065172932308521180623971e-747071', '-1.6329589656409659379617709297263e-70894', '-1.6329589656409659379617709297e-70894', 29, 5);
    T('-5.8462329933853950240188059008292500662668127e+60247243', '-2.03667262871083956053143582447517e+706776170140', '-2.037e+706776170140', 4, 5);
    T('4.8578643608884957665249720147512041570224e+42', '5.29728494394e+40024', '5.29728494394e+40024', 42, 4);
    T('-1.117845e-87471', '6.6964778135236881292e-98664832776', '-1.1178449999e-87471', 11, 1);
    T('-1.1027435825602049693395108767877447158060098e+1366795482', '3505706830792602767640494731609099726202.2141536', '-1.1027435825602049693395108767877447158060098e+1366795482', 62, 3);
    T('8.925710780e+50781166', '7.6014062295962928851118035297752018105242944590032e+76214004296671', '7.6014062295962928851118035297752018105242944590032e+76214004296671', 53, 5);
    T('-7.81028977685685723019183339084945516263716316495835705346546e+5542', '-8.4944162078536226094767107956792540095e+470', '-7.810289776856857230191833390849455162637163164958357053465e+5542', 58, 6);
    T('1.259e+6432025', '8.8917717871024932948616749594944802038973345e-821386985352', '1.259e+6432025', 63, 3);
    T('-7.0302720043158798876127647e-22', '-5.47203447283582772948156612417489828819e-999355481', '-7.03027200431587988761276470000001e-22', 33, 3);
    T('7.728933e-3687046', '5.61537493796577209685453276e-392', '5.61537493796577209685453276e-392', 87, 5);
    T('-8.9864581853354e-78558', '5.137348863e-4098167', '-8.9864581853354e-78558', 76, 0);
    T('3.4492551819070410453823958380e+824', '5.8568052917684284872378996357492038386903985e-82880812', '3.449255181907041045382395838000000000000000000000001e+824', 52, 2);
    T('4.590779740e+42818259210', '5.75694568677064388608219580757731364874991854316e-5651', '4.59077974e+42818259210', 40, 1);
    T('-6.72191694546837264823397083778e+335108', '-68520648487135100.10426', '-6.721916945468373e+335108', 16, 0);
    T('-7.8305684712e-21293304751323', '9.9266256883e+574155427304', '9.926625688299999999999999999999999999999999999e+574155427304', 46, 1);
    T('1.6312159089335599254850493819487e-85065534', '-9.8831894064424552312010456284631220467528337e+9988083093152', '-9.8831894064424552312010456284631220467528337e+9988083093152', 89, 0);
    T('2.59551326551064174859e+73', '-9.27297437740690199635836e-3206657481078', '2.595513265510641748589999999999999999999e+73', 40, 1);
    T('2.8649138267578580909949392177608826817085928071839085e-635090351', '-4.55575134813336819660e-273227044190684', '2.8649138267578580909949392177608826817085928071839085e-635090351', 73, 0);
    T('-6.28654207488292463985356243663704e+71597000', '5.6577652326626783611715777860e-53523506123154', '-6.28654208e+71597000', 9, 3);
    T('-4.373420779370278088219376866783974550465557609810e+2305184', '-8.7434865760206167575081865148580623901722236931e-3940', '-4.3734207793702780882193768667839745504655576098e+2305184', 47, 4);
    T('-1.21090649428050412251752573380297119338481e-487886', '-6.425569223073431731321051978881574052976543430249e+2548834836568', '-6.4255692230734317313211e+2548834836568', 23, 4);
    T('5.53498752418e+6268150662904', '6e-59046', '5.53498752418e+6268150662904', 64, 1);
    T('4.5495699162854662690989742305698715228197388732476884045e+7871907', '-3.7209194909324170370464016897420264357774924896e+2941', '4.5495699162854662690989742305698715228197388732476884045e+7871907', 90, 6);
    T('-1.969359315271422729850776051523266094923634616239133e+17645856', '1.161156421121294470764031775611045294195e-356323697519', '-1.96935931527e+17645856', 12, 1);
    T('-4.68384645498e-137195407', '6.52116339693129657533e-819142', '6.5211633969312965753299999999999999999999999999999999999999999999e-819142', 65, 1);
    T('-7.38132262392616440395740565910893178490352315962700e+59558580179', '-1.3105702318754228634600496040348030386517518e-660615', '-7.381322623926164403957405659108931784903523159627e+59558580179', 54, 2);
    T('-5.4674546370818775496077179181090739321148445200359399696273e-335866061', '-1.37869773218197351043545753926691893640e+51145829', '-1.37869773218197351e+51145829', 19, 1);
    T('2.1074952804299432202259859078931773781293583142e-2793221', '-2.875815690466616937028e+60', '-2.875815690466616937028e+60', 99, 3);
    T('-4.52282256665992125337e-910257814800224', '-5.84861092386025193680656153e-837588', '-5.848610923860251936806561530000000000000000000000000000000001e-837588', 61, 3);
    T('-6.27435836524155273831549049391618258e-23523', '8.14507603863318129532480007116724526261e-795073', '-6.27435836524155273831549049391618258e-23523', 58, 6);
    T('7.20841314852301179932111500729373996557487809587e-300425', '-1.5720578840992209471e+85767186498', '-1.572e+85767186498', 4, 2);
    T('-8347144700614231777327132074048892.112', '-9.96521943191928642e+431026148764876', '-9.965219431919286420000001e+431026148764876', 25, 3);
    T('7.441493752341163873441500970659032904044066453455932842853e-746562551', '1.41558e-29398695069', '7.441494e-746562551', 7, 6);
    T('-1.6760472630640520613253141968520407633534183e+32244996334', '3.93330456923949913147210680817618234776708294103459331701685e+198', '-1.6760472630640520613253141968e+32244996334', 29, 1);
    T('9.39608786938082586714198036897202871801065916122499439008828e-90429', '2.894014971323688e-8523519491275', '9.39608786938082586714198036897202871801065916122499439008828e-90429', 62, 1);
    T('-4.050068039718802414896844436466942066931079553848636494e+2377145', '6.71e+79829041635', '6.70999999999999999999e+79829041635', 21, 1);
    T('3.3292007698647799e+542814', '-1.0647177106e+6627659880331', '-1.0647177106e+6627659880331', 73, 0);
    T('-8.54287419335883876432e-2736513', '-9.871557351492861801551091265123133612014268816e-90255', '-9.8715573514928618015510912651231336120142688e-90255', 44, 2);
    T('-3.491724216733823623848989618668131828603e-471718386746', '9.370024311405711234855610505512e-86286858575497', '-3.491724216733823623848989618668131828603e-471718386746', 90, 4);
    T('186.867766', '5.337290580549133689628182484432e-6508', '1.8686776e+2', 8, 3);
    T('8.376e-12', '1.410133029167572774226376531449525156145827529683711190e-64529799516', '8.376e-12', 65, 1);
    T('7.935649853519984971908e+315', '2.29082968485316470618e-766409412', '7.935649853519984971908e+315', 47, 5);
    T('-7.7546593592e+627', '9.01725543968334754325070009005408705921775e-133', '-7.7546593592e+627', 64, 5);
    T('8.871191257571385857624298796422369e-69528616759', '41486560074578244188113366539692.5871621032418265', '4.148656007457824418811336653969258716210324182650001e+31', 52, 2);
    T('-6.682083060e+5125931930', '6.26515570407342159968627056866877415111520807e+140', '-6.68208306e+5125931930', 50, 3);
    T('-9.969870748e-517816', '6.65390644447601194043022889605592760495034e-529011', '-9.969870748e-517816', 50, 4);
    T('4.9838419396192954472815959315942981693115032563e+932389688800', '-83732980677.177231', '4.9838419396192954472815959315942982e+932389688800', 35, 6);
    T('7.1293538228735985367836159724929835175906398427791942384344e+707667', '-2.7760359375791866698165697426098175048760501e-421', '7.129353822873598536783615972492983e+707667', 34, 3);
    T('7.6841e+209209797492', '-5.910803161638748741215e-3579766589', '7.7e+209209797492', 2, 0);
    T('-1.1240689693750858988755802797892581291572e-20', '-7.09752188291375825521628e+4773064124', '-7.09752188291375825521628e+4773064124', 73, 5);
    T('-3.1e-232085797', '7.5581182037067710073e-111032171637054', '-3.1e-232085797', 32, 0);
    T('8.5297124266802129425661203462241625293237277611e-8971', '8.11019376778155578e-3253224', '8.52971242668021294256612e-8971', 24, 6);
    T('5.875304698656154624993908278095423623337758089e-7138586499043', '-1.919e+8271437', '-1.919e+8271437', 10, 3);
    T('-8.7326607045353714705e+12360719001159', '-4.72e-568348', '-8.7326607045353714705e+12360719001159', 84, 6);
    T('3e+85376365027751', '8.19089430319554702409195434208e+2667712261', '3e+85376365027751', 9, 6);
    T('3.24007206658963540982005487000555e+38', '5.4051e-77', '3.2400721e+38', 8, 6);
    T('6.612054136478269155471805194033392845547312649385385874117e-8701910', '-9.21740229847224e-11022106', '6.6120541e-8701910', 8, 6);
    T('-8.38356261504564e-5881543990746', '9.04767737665262855308183680932354875489505736898508e-414', '9.04767737665262855308183680932354875489505736899e-414', 48, 4);
    T('-9.3e+352244737', '8.674181e+1644', '-9e+352244737', 1, 2);
    T('-1.5992664537726042184606872373e-86122793', '38.7', '3.87e+1', 58, 0);
    T('-5.659977928031145081632274755e-49252970446', '-2.96162007560106437187340244012762592244118815702166843805396e-9184125', '-2.9616200756010643718734024401e-9184125', 29, 5);
    T('-1.84774642798329271255e-871825556351', '8.155142381715001403732144467110527398612211884416179303715e+6020', '8.155142381715001403732144467110527398612211884416179303715e+6020', 67, 4);
    T('6.9007139236106505581047869669140002174173262547e+247', '1.4668476267568982204296922938e+2259979', '1.4668476267568982204296922938e+2259979', 77, 4);
    T('9.075376358202365103439292671627945601488673619133551337e+21415268938558', '-6.7509342344282408640797165340774069e-883534', '9.07537635821e+21415268938558', 12, 0);
    T('4.69186641826132017320685714182769150922291084844e+97', '-9.503821510538503413648835284892144526522494593393741101752e+59166', '-9.503821510538503413648835284892144526522494593393741101752e+59166', 65, 0);
    T('9.1307714907569867151476466553e+883150055631660', '-7.2681771862670642371070762839492748443569290e-95542431948637', '9.13077149075698671514764665529999e+883150055631660', 33, 3);
    T('-9.63738095691e+47', '2.7064667525948266316285174980362573383649e-5390796980', '-9.63738095691e+47', 90, 3);
    T('-8.7252381173322854366731339149e+31', '-1.325347273722761114009997499926710236601945e+4111652', '-1.325347273722761114009997499926710236601945e+4111652', 97, 2);
    T('3.3604067710060231e+21294359', '-1.21653800493874928562958799112806681507368e-237735989735', '3.3604067710060231e+21294359', 29, 4);
    T('-4.4944384394944961135660926e-847748264153', '-5.2861432920568e+6493230816', '-5.2861432920568e+6493230816', 71, 1);
    T('7.1853571581178364e+64431', '-4.394332438242730067271677095871716794e+447497980559', '-4.39433243824273006727167709587171679399999999999999999999999999999999999999e+447497980559', 75, 2);
    T('-3.5642919629805831697336580082239100833455131436e+9632113803', '8.4024689686800157479612622514902142381977094083e+279591012', '-3.5642919629805831697336580082239100833455131436e+9632113803', 67, 3);
    T('-1.1331519792280289653760377508649853639440820e+71562273883', '-8.37857998927622090424152458e-2683445559', '-1.133151979228028965376037750864985363944082e+71562273883', 87, 6);
    T('-8.63602817845473370687870840283e+6407', '5e+758512067', '5e+758512067', 31, 4);
    T('-8.79121980461038195961759375359329723289845599426998224825e-801', '-9.20716040067757730103267841089178191308599607631372411e+511319784024318', '-9.207160400677577301032678410891781913085996076313724110000000000000000000000000001e+511319784024318', 82, 0);
    T('-1.8510850620135811226e+34760', '8.5493912456140658321055935521883865125150639399e+529163400001967', '8.5493912456140658321055935521883865125150639399e+529163400001967', 59, 4);
    T('9.3539265655381823738929130878e-2415478059', '5.53828973593186557762940e-3637036', '5.5382897359318655776294e-3637036', 88, 1);
    T('-4.482677883321155658631953493339002176206e+3015746590', '-8.3286803456106e+14', '-4.4826778833211556586319534933390021762e+3015746590', 38, 6);
    T('-5.50111180169e-2032192766436', '9.6073478445224952678924198877865627586844622803564406e+53581132160', '9.607347844522495267892419887786562758684462280356440599999999999e+53581132160', 64, 1);
    T('-3.560545264302381720207648500221442529319536489759474658e-58795', '-8.856097287891658611555325e+1018', '-8.856097287891658611555325e+1018', 98, 5);
    T('-8.602242328862761972431733252697254779311e-50714721529083', '-6.38200430335735747901596248590928207976950153565947745347892e+8133963977937', '-6.38200430335735747901596248590928207976950153565947745347892e+8133963977937', 86, 4);
    T('-5.7978431105463566904e+4057', '4.05545356021904179863919231983e-184365558', '-5.797843110546356690399999999999999999999999999999999999999999999999999999999999999e+4057', 82, 2);
    T('4.3523512478507156954080220685854931900492438450430615479e+93670', '-6.183746392969994483358018819195570894964787276094027093e+74668117682954', '-6.183746392969994483358e+74668117682954', 22, 5);
    T('6.8316008930426773303049003942905303584021954847e-707', '8.77090083338983e-7285', '6.83160089304267733e-707', 18, 1);
    T('7.9030164389927186815228226424200979188794104606747635491e+61', '-4.9732088503e+43529146062', '-4.9732088503e+43529146062', 56, 0);
    T('1.902609038732e-611445940463', '6.19905128356879e+2891156439', '6.19905128356879e+2891156439', 46, 4);
    T('1.12130225507539754320099147414037e-522754919346', '-4.3166722e+428280705886424', '-4.3166722e+428280705886424', 29, 4);
    T('1.05970180671419625238619498e-285818947041', '6.2536e-136005', '6.2536e-136005', 26, 1);
    T('2.01742916375698200598014352e+98', '-9.24288713761684507564696161251909e-8889', '2.01742916375698200598014352e+98', 37, 6);
    T('-1.333096244182e+9535334', '-9.5367181031648989010061668983099091012070461596357963498169e+65', '-1.3330962441820000000000000000000000000000000000000000000000000000000000000000000001e+9535334', 83, 3);
    T('5.8697e+13', '5.5769519678797966526961339095995598261060232186028e+8027831055', '5.5769519678797966526961339095995598261060232186028000000000000000000000001e+8027831055', 74, 2);
    T('-9.15184477292714e+274069512', '-5.82632743380698844324803682051043852933360413593e+135845279', '-9.15184477292714e+274069512', 82, 2);
    T('2.12097e-6770488', '6.4574e+8343116374522', '6.4574000000000000000000000000000000000000000000000000000000000000000001e+8343116374522', 71, 2);
    T('45702091724493230713654728472033522393773687325.12606', '-3.094703463538445457755092049431060405694436473084e-36774', '4.5702091724e+46', 11, 4);
    T('-8.9820013347438800407854335458694922487479577193027228894e+26977224727460', '7.573567273592723377261484591089286567090635842889352023720e-3853685621929', '-8.9820013347438800407854335458694922487479577193027228894e+26977224727460', 75, 6);
    T('-4.02851955448022691e+55574828319506', '4.88487674826193806402104001441827895097663738747465909558e+97665686649485', '4.8848767482619380640210400144182789509766373874746590955e+97665686649485', 56, 3);
    T('-8.37199e+4701793232352', '2.5908134871e+2237018589453', '-8.37199e+4701793232352', 58, 3);
    T('1.98130321934055784e-67', '-7.1214920702597083168834422711753450585e-6353712341628', '1.98130321934055784e-67', 37, 2);
    T('-1.715e-249043', '-5.51599535075202336686267100913302743830870964823809048084730e-533286016097', '-1.715e-249043', 9, 6);
    T('2.38000326963000133400225256592845600257010348353796489949e+18813939825', '5.51799866280327464e-168', '2.380003269630001334002252565928456002570103483537964899490000000000000000000000000001e+18813939825', 85, 0);
    T('4.7807159718447094736084673941812034790581138843450067e+74161', '-8.82998663048505912594934938e-14509505', '4.7807159718447094736084673941812034790581138843450067e+74161', 79, 6);
    T('2.3589337978041002060454472751867308805717e+63', '8.8230621926968989088976166729814733287671703357e+492411216064', '8.82306219269689890889761667298147332876718e+492411216064', 42, 2);
    T('6.0295e+72', '-4.0598326263099117684299163628870197e+91168189590622', '-4.059832626309911768429916e+91168189590622', 25, 2);
    T('629243652692.40593', '-5.390460282482735239346629609409491289333036333e-140410895662097', '6.2924365269240593e+11', 74, 4);
    T('-7.01734655654728492083e-52234639928', '-3.03307176862749445111522948e+25248293', '-3.03307176862749445111522948000000000000000000000000000000000000000000000000000000001e+25248293', 84, 0);
    T('1.8450490981058742589627871996598319899189e+24909067129', '4.1299262371293736727337824926801064724200e-9738', '1.845049098105874258962787199659831989918900000000000000000000000000000000000000001e+24909067129', 82, 2);
    T('4.3280089469438282424408090769995005447220364264552825534e-65918071237404', '-7.1932412743889612128648620079e-63957', '-7.193241274388961212864862e-63957', 25, 2);
    T('-6.988296411007828195981554177090479016319060487838950951e-4819625684', '-9e+51165', '-9e+51165', 67, 5);
    T('5.6018011064409481127003044163185e+171289346779', '-2.102e+303089187970209', '-2.102e+303089187970209', 29, 0);
    T('4.9226910443756467631157999621550818e-6836006773532', '454733211599138737894002570539948', '4.5473321159913873789400257e+32', 26, 5);
    T('8.973801839142e-295728594467445', '3.5077752331986356150e-9791521', '3.507775233198635615e-9791521', 56, 3);
    T('-9.08629652221430787137902660216065790727620971585156e-1520806', '-9.356374811e+769209', '-9.356374811e+769209', 28, 5);
    T('-7.0585264748723e-8417028546', '5.479278407004685248188594492411863736131167646139e+140', '5.479278407004685248188594492411863e+140', 34, 1);
    T('4.74e-14', '-1.8359647731849476737220657166e+481', '-1.8359647731849476737220657166e+481', 92, 4);
    T('5.905489033045891430207237037225003422912496736e+8162', '-4.51875118758663397106831100353923e+68965418122', '-4.51875118758663397106831100353923e+68965418122', 36, 4);
    T('-7.80698945208101153202613236521326e-9001757796106', '-6.948870884194914207852776148178312197e-348', '-6.94887088419491420785277614817831219700000000000000000000000000000000000001e-348', 75, 3);
    T('-8.6658483213996722911452644203511784613776683359322026140e+61', '-5.5369142297173943580e+173', '-5.536914229717394358000001e+173', 25, 3);
    T('-7.9878710e-839272', '-4.200544395e+5518415', '-4.201e+5518415', 4, 4);
    T('8.649215938263084794502573889497901482370678082928e+70386100465688', '9.044e+1792213', '8.649215938263084794502573889497901482370678082928e+70386100465688', 95, 4);
    T('-9.36860001286661728981001749640350444548469252e-362', '9.821529799393679790305503738709084378965889010e-7390601663721', '-9.368600012866617289810017496403504445484692519999999999999999999999999999999999999999999999999e-362', 94, 1);
    T('1.69936890327186479764e-179906866443486', '-6.01231735835770492902710737032591999e+8804', '-6.01231735835770492902710737032591999e+8804', 94, 3);
    T('4.435479034898156066373269849869676859418e-66103782990468', '-4.1883855088799e+458497837933211', '-4.188385508879899e+458497837933211', 16, 1);
    T('-2.0260e-95675', '-7.7738940687893692718711116979490329660610170699759114269303e-1617881465117', '-2.026e-95675', 43, 6);
    T('6.267243764143697985915735739751270036872324540297538786e-31125699', '-5.62528521627055655172958766049154320866979379996343873761e+240067', '-5.62528521627055655172958767e+240067', 27, 3);
    T('-5.11771201521046e+354165969531', '-9.32487460933896e-993229', '-5.117712015210460000000000000000000000000000000000000000000000001e+354165969531', 64, 0);
    T('36.7421597291981351587342141152476317382824597332097', '-8.048727466370258861084560910988528e-410153876205', '3.6742159729198135e+1', 17, 3);
    T('-2.93162590270490769522985043693001826132619495e-63753798087382', '-62565162563957969791427853347654240.6874872', '-6.25651626e+34', 9, 3);
    T('3.243661674925054600909771095845267314679214133969106444e+72607712', '-7.3678243944673129926145957983106203967790371251201044e-5121262', '3.2436616749251e+72607712', 14, 4);
    T('-5.9906194720515153151e-85461531', '-9.97207120928e-3502616', '-9.9720712092e-3502616', 11, 1);
    T('-4.73457137654637653678887541346800663429914885e-5396688668849', '1.24062831328e+43286479253', '1.2406283132799999999999999e+43286479253', 26, 3);
    T('7.129e+11', '-1.085882081169205043110e+883', '-1.08588208116920504311e+883', 50, 3);
    T('-4.0271067270272514687209711e+13357981', '-7.253e-473328', '-4.0271067270272514687209711e+13357981', 98, 2);
    T('-5.112928978802273588731138947845147328710389e+585891820570092', '9.758251505951474118808728135026112021208915955661e-42610051662', '-5.112928978802273588731138947845147328710389e+585891820570092', 84, 0);
    T('-7.896474839422433318463642992606519e+7569548672', '-7.099689904895e-1596', '-7.8964748394224333184636429926065e+7569548672', 32, 6);
    T('3.184252865809665664031737623414727197957879907e+526', '7130357279728159606268490588.6', '3.184252865809665664031737623414727197957879907e+526', 72, 4);
    T('-9.447871521573714236757144e+410136457473278', '9.7490298149150563693263972284366266657e+772264758897', '-9.44787152157371423675714399999999999999999999999999999999999e+410136457473278', 60, 1);
    T('-8.02262925894285850758e+123519316', '4e+336378280', '4e+336378280', 6, 2);
    T('-6.476095552251707941179513025e+988', '-2.3903818684826e-97150360856984', '-6.476095552251708e+988', 17, 0);
    T('-2.35698927447507682812364999092640966634458791e+8561623', '-8.6356414259889996201093102758801741078697951e+5616490255', '-8.63564142598899962010931027588e+5616490255', 31, 6);
    T('-4.239325310794568985249602516443258458975322499e-47135746648388', '-9.1837680967e+2691', '-9.1837680967e+2691', 53, 4);
    T('-292193.19898112720363738761875275996012691072777', '7.66084712784313387601367100495686563139807622e+92360', '7.66084712784313387601367100495686563e+92360', 36, 3);
    T('-6.185813723764004211695531e-54172025707', '5.90012278271187204543e-50385', '5.90012278271187204543e-50385', 77, 5);
    T('2.45241e-86792895569', '-4.05309127034530496842403658546552466601483101967458663e-62534321934245', '2.45241e-86792895569', 98, 2);
    T('-4.8364100399684328874957901478679579378116e-52', '6.24609545105736611e-792662511', '-4.8364100399684328874957901478679579378116e-52', 53, 5);
    T('7.205736561878967052e+602201412', '-71108242037259399635546556997507304865594', '7.2e+602201412', 2, 5);
    T('-7.723381467334220927942e-304018457', '5.29759287817322885452757191330608823959e+93', '5.29759287817322885452757191330608823959e+93', 61, 2);
    T('8.626583539918551386054027357253763137055438088e-6061060', '9.4533536e+41905255', '9.4533e+41905255', 5, 3);
    T('7.95237516248613947266654866e-173163', '2.7547536047358803004422650060147302346469852759257862e+7063', '2.7547536047358803004422650060147302346469852759257862e+7063', 85, 6);
    T('4.8775377790194157989172827159268e+63338977626517', '-8.89645561948105774258e+50045148', '4.87753777901942e+63338977626517', 15, 6);
    T('-5.13270064497590089403207072389906533287621839836e-7436294522023', '5.7e+377095009198', '5.7e+377095009198', 60, 5);
    T('-3.5705397689648063101906701944146e+475944028', '5.07672996525589795e+68024865', '-3.5705397689648063101906701944146e+475944028', 74, 4);
    T('6.5316687275983669252045076348820646541e-13843676787622', '-4.523724414594684532e-5065', '-4.52e-5065', 3, 4);
    T('8.02421e+14554', '-5.23458106e+2624409329055', '-5.23458106e+2624409329055', 21, 0);
    T('0.000009337327728322912417063', '-8.12322128525572893e-986', '9.33732772832291241706299999999999999999999999999999999999999999999999999999999e-6', 78, 3);
    T('943859525946848871892889837044457077509325276323850538602.34', '7.480621424894792938371503e-2670416991', '9.4385952594684887189288983704446e+56', 32, 6);
    T('-7.89793214654495888590419724423614861491198751738780503148906e-334', '-3.100638864e+40566143243384', '-3.100638864e+40566143243384', 64, 2);
    T('3.5e+66230966773', '4.105596740118723e+947575291741405', '4.105596740118723e+947575291741405', 78, 4);
    T('-7.68852081635e+670760144011', '-6.9609188e-33501043', '-7.68852081635e+670760144011', 29, 1);
    T('-2.4970280413254077615007229690684714e-7091375634558', '4.9491044160e-137', '4.949104416e-137', 71, 4);
    T('7.0928360636276206551222130696427709138627551808e+16285463', '-5.2288004877585e-90413190', '7.0928360636276206551222130696427709138627551808e+16285463', 98, 4);
    T('-7e-7', '8.734045052816520257662431716263463471e+29121', '8.7340451e+29121', 8, 0);
    T('6308975644047484722421615470514946267238719749', '-8.79946264736654170267201575665151541933e-207590056', '6.30897564404748472242161547051494627e+45', 36, 2);
    T('6.199358634992566934532e-5623', '-3.0227327927778201702967755100e+20185', '-3.02273279277782017029677551e+20185', 75, 0);
    T('-4.660084086725938326269396e-988671803362397', '-2.787738e-881', '-2.787738e-881', 66, 5);
    T('5.50143022014110515745e+667', '-1.3e+829979076357617', '-1.3e+829979076357617', 45, 5);
    T('8.15037125144807401747668605560212965541e+2018879073266', '-4.8974570801907562780240874280e+500', '8.15037125144807401747668605560212965541e+2018879073266', 48, 2);
    T('-4.57405e+7326', '5.37111808992872166642310190998e+37', '-4.57405e+7326', 97, 4);
    T('-2.56893287968328151211211093017827084457667392792041367e+23390864552', '4.47540052323870e+431588151161', '4.4754005232387e+431588151161', 57, 2);
    T('0.04755', '-2.5776641628558721929171655e-8065973814542', '4.7549999999999999999999999999999999e-2', 35, 3);
    T('1.055060797497792147078301009729861e+72166280635543', '-7.6189059561758892369892895351690473836e+94069604', '1.055060797497792147078301009729861e+72166280635543', 90, 0);
    T('-3.4058780860389456e-34013175760515', '9.050638875819907570e-46814167062604', '-3.4058780860389456e-34013175760515', 24, 0);
    T('-741098826053419841.28675', '-3.4995399002009568295823333e-848909', '-7.4109882605341984128675000000000000000000000000000000000000000000000001e+17', 71, 3);
    T('9.4970230831405536889872863721075e-7808394480136', '2.412984490095564644617e-901', '2.412984490095564644617e-901', 44, 4);
    T('-9.25689227965088135172079e-250459785', '3.8563864733637747561414665315540902474986e-3765', '3.8563864733637747561414665315540902474986e-3765', 81, 6);
    T('8.92710586589721797e+2747', '6.819489565738643382555484989500780e+40', '8.92710586589721797e+2747', 29, 3);
    T('-4.732e+9528252658', '-2.037717463958937265298e-7279540608628', '-4.73e+9528252658', 3, 1);
    T('4.6872372929282680306543002372448995939930136965539901693e-7490478', '-6.100018631717949176420912323330891073607177547563e+38600133301277', '-6.10001863171794917642091232333089107360717754756299999999999999999999999999999999999999999999999e+38600133301277', 96, 1);
    T('9.3406202366e-293', '-338371.15374581', '-3.38371153745809999999999999999999999999999999999e+5', 48, 2);
    T('-4.98531e+5208281', '5.336354481574462334162151522421891279271714955069744e-21416787878345', '-4.98531e+5208281', 71, 6);
    T('-8.625520046656019e+864017', '3.85326086913619130894705643058223217171e-52423450', '-8.625520046656019e+864017', 78, 6);
    T('-4.9062564185e-8', '7.776377638774559670211369135177559189846700814486e-457217121528706', '-4.90626e-8', 6, 3);
    T('1.1971291401652998328e+356184663', '9.98610406816827737057489187274076414750038339237193007225e+2167051', '1.197129140165299832800000000000000000000000000000000000000000000001e+356184663', 67, 2);
    T('-2.4242224583301469647444256010e-7635362', '-2.255730011498029724005036643253053620389243862167485100e-3969680', '-2.2557300114980297241e-3969680', 20, 3);
    T('5.655379981416e-584680218557', '2e-100519826081', '2e-100519826081', 7, 6);
    T('-2.99180326966977944e-316271382', '6.44131066611818851584563664010561138672e-1366', '6.44131066611819e-1366', 15, 5);
    T('-1.139852760442940896043133907609848829447925131833481076e-59020', '3.01e+9305516609', '3.009999999999999999999999e+9305516609', 25, 3);
    T('5.40227307595704085037460237610138399853505921287243517e+967540249904846', '2.43559279593159307e-453030128955', '5.40227307595704085037460237610138399854e+967540249904846', 39, 2);
    T('-5.85894105711440303879203671082396854533542449186116140e-85211465145', '5.933187899915832284903355328060802067430004130e-824765102', '5.933188e-824765102', 7, 6);
    T('-9.5284181812276368285389455109e-6926611', '4.513636540838134312590902046291820259895307131120008866512e+200668', '4.513636540838135e+200668', 16, 2);
    T('-3.2250769957365465589093447595271575914e-25985566108', '-2.32607471e+2593892', '-2.32607471e+2593892', 50, 1);
    T('-3.729900463060056071838e-368146', '9.7213316648753861136200481434254510003748305374e+5771198', '9.7213316648753861136200481434254510003748305374e+5771198', 97, 2);
    T('-2.90460682682481887882566709657058e+117503098093', '5.95885273836e-80619', '-2.90460682682481887882566709657058e+117503098093', 56, 6);
    T('8.23266740465918756427e-37420414', '-7.39391036907660251247e+916635253', '-7.39391036907660251247e+916635253', 22, 3);
    T('7.9688279961082358631530307313960925906744e-94646161', '-3.2545981395981e-62', '-3.254598139598099999999999999e-62', 28, 1);
    T('-3.04e-85058707', '-9.70351315271384936126777892905262e-23325180844', '-3.0400000000000000000000000000001e-85058707', 32, 0);
    T('-7.6299766605932106940730112875400476453067753086578787285941e-74759375929275', '-7.36340603454079e-8270531813', '-7.36340603454079e-8270531813', 44, 6);
    T('-5.8310302957783442789864346906789505380241899061562562e+5575865399901', '-927001389726120930193641933820322688025969', '-5.83103029577834427898643469e+5575865399901', 28, 2);
    T('1.9803765689262365267843350179889101757096625e+71308324639067', '-6.8905210022654464626683179230e-17304500319', '1.9803765689262365267843350179889101757096624999e+71308324639067', 47, 1);
    T('-4.197e-7765923250261', '-1.9283819409610477881e+521529510375', '-1.929e+521529510375', 4, 3);
    T('9.8597416662451144817994524398167153626191788532146771484468e+655', '-6.7634774628571316891264701990752043e-69019', '9.8597416662451144817994524398167153626191788532146771484468e+655', 94, 5);
    T('-7.7347546073304942592447326376833563424304479e+5983490', '1.8521454594765384069767875269194876821968729472291e-154261', '-7.7347546073304942592447326376833563424304478999999999999e+5983490', 56, 2);
    T('-3.8581902295154679498450414e-8671', '4e+11922424615490', '4e+11922424615490', 19, 5);
    T('8.006691110146341389879812017172200838502773386917017663329e+860177673859', '7.2709069806802760415563596e+463', '9e+860177673859', 1, 0);
    T('-7.51613001776674567261527036958336634670155554568985968657605e+97396489', '-4690229991778198160630143689577547737061661445968.4', '-7.516130017766745672615270369583366346701555546e+97396489', 46, 3);
    T('-86.051104278', '-1.4181742610616854465185717133692013490987497805286e-626', '-8.6051104278e+1', 41, 1);
    T('6.612031743071363617839e+8562680', '-8.6172970206718186980629e+97', '6.612031743071363617839e+8562680', 91, 4);
    T('9.9826327037766638981855978049033245346449945233e-11670497555', '-9.92529140786432080670588755e+650677', '-9.92529140786432080670588755e+650677', 71, 0);
    T('-7.524270e-43291964813', '8.0351949360564452838009063628688234147752355909594e-19932955636', '8.0351949360564452838009063628688234147752355909593999999999999999999999999999e-19932955636', 77, 1);
    T('8.0007159685244131179374960230366231708587468898704059e-41217196', '6.625005054972884684724865257197e+8936981575', '6.625005054972884684724865257197e+8936981575', 86, 1);
    T('9.18310508710923750488162456104346111993e-4102889972', '8.63779271e+5889776', '8.63779271e+5889776', 9, 6);
    T('2.0993428471249521070988118288875781e-9092250642', '-5.2e-794629341653', '2.0993428471249521070988118288875780999999999999999999999999999e-9092250642', 62, 1);
    T('-7.96535254812356128984406410086698655139426062335e+7386', '-4.684446130961869060614453084145453162133640966934016e+923588', '-4.6844461309618690606144530841454531621e+923588', 38, 5);
    T('-2.787755308161188986195433202094428033184373461565e-574591479', '3.9309841295091e-4883444721142', '-2.787755308161188986195433202094428033184373461565e-574591479', 85, 3);
    T('2695542684735231169154646023809.0089', '3081497761008705696273957662139775832070949783162', '3.0814977610087056989695003468750070012255958069710089e+48', 91, 2);
    T('-6.6185584514334330960505162057593290764609222e+280302600', '-883971437978245.646572', '-6.6185584514334330960505162057593290764e+280302600', 38, 1);
    T('416957254799225904183330744.6935514', '5.62794748778089605940873477182785526952252406174703425025803e+64224', '5.627947487780896e+64224', 17, 3);
    T('2.4979942640650761060141617725052e+189208284', '-1.2755709473400607456112501524057745557394625937943452e+6715', '2.498e+189208284', 5, 2);
    T('9.195372435080938253157429e+28087007144', '3.9977423978486799556984e-8999', '9.19537243508e+28087007144', 13, 1);
    T('7.57734173390142066027436736341789079457967342333554693e-63300903480090', '-4.437432944414779384102231740643920804966298112333865632055e-2765', '-4.437432944414779384102231740643920804966298112333865632055e-2765', 88, 3);
    T('6.6723795807254581369828597756153992647617543820997799531e-27767115888', '9.957115400496627402445791253761678547188296641238820e+117', '9.9571154e+117', 9, 5);
    T('7.258597e+8', '8.947763801529967234247368370551688890e+37', '8.94776380153e+37', 14, 0);
    T('-1.674193704345602756854804547557720e+119', '1.017334e-5642641', '-1.67419370434560275685480454755772e+119', 90, 5);
    T('3.894789e-9563674447804', '-7.205278136737982e+8102080392609', '-7.205278e+8102080392609', 7, 1);
    T('8.710704101674082676e+467658027992310', '4.00739915121706738981699019538970110781914462e+902524734554', '8.710704101674082676e+467658027992310', 24, 1);
    T('7.86173803220951404566850101191e-56128783916446', '9.554035854381467991017542619572871984029396684015e+55', '9.554035854381467991017542619572871984029396684015e+55', 61, 3);
    T('3.96e-366905550199616', '-2.007621427030247176e+64', '-2.0076214270302471759999999999999999999e+64', 38, 2);
    T('1.03118422864248928445421439524456306723874850e-670858462', '-2.31e+9188', '-2.3e+9188', 2, 6);
    T('-2.93880286246e+644094521804226', '-7.687876915326075905839074611935657031029696e-452827506', '-2.93880286246e+644094521804226', 45, 5);
    T('6.43747136734075362785139240961113898574150050e+249846', '1.53883997042908813345273071e+28', '6.437471367340753627851392409611138985742e+249846', 40, 0);
    T('-9.4024194359730230e+42897', '3.22137633196253114271184e+952', '-9.402419435973023e+42897', 67, 6);
    T('-1.5363766444643537151254313e+617022469986', '-2.7603718775119093849954376953314824e+73431350826699', '-2.7603718775119093849954376953314825e+73431350826699', 35, 0);
    T('3.15442331610462460878986872601015058826282053e+308836109625427', '-5.4085576355056739e+997571759773', '3.1544233161046246087899e+308836109625427', 23, 0);
    T('-4.956828475637032078544912337137913828421620137e-80833981215', '5.69e+93134', '5.69e+93134', 92, 2);
    T('-9.5950801374230205e-712907608', '2.0866055415278137787063620285583107319250592109872529e-5167461', '2.0866055415278137787063620285583107319250592109872529e-5167461', 72, 0);
    T('-3.97783089e-80450795672', '1.50438650e+5684684285506', '1.5043864999999999999999999999999999999999999999999999999999999999999e+5684684285506', 68, 1);
    T('451560531535214412601017424572289133526567292199093808425791', '4.972671214528881806724028532483334183293521129692698464e-807161689187', '4.515605315352144126010174246e+59', 28, 6);
    T('9.42330639759931670635004e-57951249', '4.29507107548596196e+9066776926960', '4.3e+9066776926960', 3, 6);
    T('-1.8738751565620743491583077100e-948', '-1.2184495905883350680660218809639e-89498038718', '-1.873875157e-948', 10, 4);
    T('5.11695934367e+3722774', '7.07406913485240138421117337791580344802119125e-36', '5.11695934367e+3722774', 84, 3);
    T('-6.817784986743005972736694606387380656486359361064234e+966692926245817', '1.789767026409498848082768934663735419931166403654823e+672133013', '-6.817784986743005972736694606387380656486359361064234e+966692926245817', 67, 6);
    T('8.1346724666215939921791e+22533656', '9.3579164248934294069997926812555155705925145838e+498708', '8.1346724666215939921791e+22533656', 55, 6);
    T('0.398599', '-4.74941890569080143956419891734759654436355436108608311221e+167288580996', '-4.7494189056908014395641989173475965e+167288580996', 35, 4);
    T('-5.3887325950896459e-4348108482978', '-4.27553225514505163586155862225281434760542278265304322971e+5464165965404', '-4.27553225514505163586155862225281434760542278265304322971e+5464165965404', 85, 6);
    T('6.083151782733350638254660199685928230617120563e+350', '2.741409862198e+95814550405', '2.741409862198000000000000000000000000000000000000000000000001e+95814550405', 61, 0);
    T('-13700677061468282460325624496787162627304873247', '8.3436544831301978627829641382646246256e-8173678918', '-1.3700677061468282460325624496787162627304873246999999999999999e+46', 62, 1);
    T('-2.85e-214934999', '-9.87587940344922594703733137366e+4113', '-9.875879403449225947037331373660000000000000000000000000000000000000000000000001e+4113', 79, 3);
    T('-587438750730015864053.6051', '-6.46017571840678823346417711920219476386051857141e+59801', '-6.460175718406788233464177119202194764e+59801', 37, 4);
    T('3.5170879044904780255019004302684100604630293931014e+187524529025', '1.567e-4404146399', '3.52e+187524529025', 3, 5);
    T('-1.1215043201497379152414e+757077', '-5.528939479e-7322', '-1.1215043201497379152414e+757077', 100, 5);
    T('4.1765793062252545287361089281497985991472759336559308079e-21542', '-8.6810092066026562e+7257913', '-8.6810092066026562e+7257913', 31, 4);
    T('1.032414970961852471051158039479e+23965091', '2.970794999546562217217875466567274128289e-86845420952', '1.032414970961852471051158039479e+23965091', 51, 6);
    T('-9.7537636657321312602946044145708400557181641e+532952872', '-7.33599096332139672e-57', '-9.75376366573213126e+532952872', 18, 2);
    T('4.90003520889621602304186151052019058216e+4080411', '-5.823877248883779318594214275e+108', '5e+4080411', 1, 4);
    T('-2.71326370737417193976611025461252390335981083381200280e-4993952802', '5.839532132e-5805719177', '-2.71326370737417e-4993952802', 15, 1);
    T('-6.2187069902229133984721812e-933358351532', '4.2261043962250866144942016789140119455350810e-313453', '4.226104396225086614494201678914011945535081e-313453', 60, 0);
    T('-2279216849549331174375144597835.24', '9.34184467660393521147705093e+7283', '9.34184467660393521147705093e+7283', 51, 6);
    T('-4.1343929911792807409047817121865373067054012e-40218499', '-4.0414e-8523858177713', '-4.13439299117928074090478171219e-40218499', 30, 0);
    T('8.9346770859533818121e+345', '-3.3034970189471523171473663536e+262388095170301', '-3.3034970189471523171473663536e+262388095170301', 31, 5);
    T('-6.8229491151445627522388e+1754', '6e-5377571999322', '-6.822949115144562753e+1754', 19, 3);
    T('6.1e+97914960', '-5.52111424916032606722864315268492693892181367425717918e-43744300', '6.1e+97914960', 56, 6);
    T('3.34504308952784629528537481838462753726328922030671802804e-817372557', '-6.7851427434314633800192254e-42998136622', '3.3450430895278463e-817372557', 17, 2);
    T('4.980671462004366668e+9793864191287', '9.0e+7', '4.9806714620043666680000000000000000000000000000000000000000000000000001e+9793864191287', 71, 2);
    T('-5.7892265143884255020110713496534353053564354210533e+18338644403', '-4.4983740886845000975e+297503851456', '-4.498374088684500097500000001e+297503851456', 28, 0);
    T('-1.4190979222531e+39557042800541', '-8.234e+5346', '-1.4190979222531e+39557042800541', 39, 5);
    T('5.2540367025892959836523314e-887246501686', '5.2935823878639e+31', '5.2935823878639e+31', 19, 6);
    T('5.4667168697602877498927151253696e+852490720', '9.16904597759976398269591997533528499579719e-190744972942', '5.4667168697602877498927151253696e+852490720', 49, 1);
    T('-2.01758297479482176799220461588073515163842e-5916', '-7.02008715930079026679849664379229600989455182191665231947e+7587527675', '-7.02008715930079026679849664379229600989455182191665231947e+7587527675', 61, 4);
    T('7.0062683783507763021913636946205945860e-1786555657540', '-9.6230963324e-31933', '-9.6230963324e-31933', 66, 4);
    T('-9e+2908626', '3e+550054036708977', '3e+550054036708977', 36, 2);
    T('-4.251622902377632191095091220963756128719361475356837114e+273626077456347', '1.884098637153026184146e+5252', '-4.251622902377632191095091220963756128719361475356837114e+273626077456347', 58, 4);
    T('3.569588372504145189932670450235666337188230738486839575833e+63', '-5.31280049123308559622340677239225279759019483593982e+126', '-5.31280049123308559622340677239225279759019483593981999999999999e+126', 63, 1);
    T('-5.2821699311794139430580917570662577730779606174317456375e-8137', '-7.93965645352e+720', '-7.93965645352e+720', 46, 1);
    T('-4.7345482848478158763774196092908151541366511555303537059476e+931106105571', '7.9e-816390791', '-4.7345482848478158763774196092908151541366511555303537059476e+931106105571', 80, 4);
    T('-4.1134364443186300455946346022877257268e-144949216734403', '8.7649717768153073570384354789272e-119946640', '8.7649717768153073570384354789272e-119946640', 62, 0);
    T('7.452230412646660778721900207482493596937612942797e+584932544', '5.5478979841309557860095e-9765', '7.45223041264666077872191e+584932544', 24, 0);
    T('-5.26102588363208e-41422', '-7.00e+494098758359', '-7e+494098758359', 88, 6);
    T('-2.1081476782210401647039688180487838150e-379', '-1.026559365180601484164035502207210136058352838068168432272e-55382689156', '-2.108147678221040164703968818048783815e-379', 82, 5);
    T('-4.34151160043e-883998', '-7.1407269134757791642929236503477598949588984112302940e+4010542388770', '-7.14072691347577916429292365034775989495889841123029400000000000000000000000000001e+4010542388770', 81, 0);
    T('-3.63452369727822701459424730029281422e-453083835', '-3.731e+60170213820', '-3.731e+60170213820', 47, 2);
    T('-9.68958985832e-616393479', '9.87937742228502e+492633560239300', '9.87937742228502e+492633560239300', 35, 5);
    T('5.6752358554230899e+225', '-6.829993885567919894124993839838e+85823900', '-6.82999388556791989412499383983799999999999999999999999999999999999999e+85823900', 69, 1);
    T('5.123481022710935677e-783979424838467', '3.324535666619180585675612164238541046734031e+62204682724088', '3.324535666619180585675612164238541046734031e+62204682724088', 55, 5);
    T('6.87758639744733095e+6247', '-3.52407695472344254804165234e+555608244666915', '-3.5240769547234425480416523399999999999999999999999999999999999999999999999999999e+555608244666915', 80, 2);
    T('2.0324859e-818010758', '-57956455201484351.364274378858390610139202173972110952', '-5.7956455201484351364274378858390610139e+16', 38, 2);
    T('-2.007647469282e-637330387752', '2.876789785817e-267', '2.9e-267', 2, 4);
    T('3.803454606956263663100042613889262095490886205385619252e-388', '8.58098801990687440134110241152901360095526572818916279944e+2838698221', '8.58098801990687440134110241e+2838698221', 27, 6);
    T('1.923954415879882576687e-295429104', '9.285812157446464130279550727708467396188616484090e+8381489489', '9.28581215744646413027955072770846739618861648409e+8381489489', 99, 4);
    T('8.0452758795989954754453793695635898738796894595e-6442530718844', '2.47678879617992e-258916479163', '2.4767887961799200000000000000000000000000000000000000000000000000000000000000000001e-258916479163', 83, 2);
    T('9.9591623872396647128779334e+4787', '3e+21', '9.9591623872396647128779334e+4787', 50, 5);
    T('-6.5991345337622866e-50288989682599', '7.425585162068548465760624319868275602e+8445', '7.4255851620685485e+8445', 17, 6);
    T('5.05753303464049382592266457757569718622914960211561382e+183105793970997', '-8.33091741709557482517e-70945331975605', '5.05753303464049382592266e+183105793970997', 24, 1);
    T('-2.091394414557481348852371e+65170503885', '3.86537076240488107257462442485647927476314e-160938', '-2.091394414557481348852371e+65170503885', 66, 5);
    T('4.6410494239815988477327149516690989485301071070876978868216e-806173', '1.8273128886151116e+195353572', '1.8273128886151116e+195353572', 21, 5);
    T('-2.3420296370084635373722e-719531248022', '8.69622087659142e-463668044842106', '-2.3420296370084635373722e-719531248022', 66, 6);
    T('2.500420758844191052616716972150866690552566064e-85955648092', '-1.2356e+6512472058082', '-1.2355999999999999999999999999999999999999999999999999999999999999999999999e+6512472058082', 74, 1);
    T('-4.5341e+823', '8.7354941334e-6769', '-4.5341e+823', 83, 6);
    T('7.837874743929941900548583155999748789e+506811547895', '3.2265154084853522856949337615730865e+672', '7.8378747439299419005485831559997487890000000000000000000000000000000000001e+506811547895', 74, 2);
    T('-5.7484139706825753827479378529383897111e-18660955702915', '-1.18411233e-63749337118720', '-5.7484139706825753827479378e-18660955702915', 26, 2);
    T('6.5686921739450883865606679374752258273074126264131808e-934928631058960', '-7.54205958874287e-7671725857779', '-7.54205958874287e-7671725857779', 56, 5);
    T('-8.62e+91924', '6.445498883919253289535769458054254579e-3297372', '-8.62e+91924', 70, 0);
    T('-8.71907866511585020045651287091e-269048169511082', '1.41270777646447300537e-9596', '1.41270777646447300537e-9596', 53, 0);
    T('6.66326e+74946', '-2649.96708629', '6.66326e+74946', 93, 5);
    T('-8.4965282039867283599766501748663756548335156096596413e-490720067812', '-2.15892317654e+13', '-2e+13', 1, 4);
    T('-3.115151e-10266396729466', '-2.090689127451e+583246713', '-2.0906891274510000000000000000000000000000000000000000000000000001e+583246713', 65, 0);
    T('2.225927439357281410433664780e+82332427', '5.5827819511508e+83783223', '5.5827819511508e+83783223', 94, 1);
    T('8e+70641498241', '-6.6087e+653', '8e+70641498241', 42, 5);
    T('6.661555366345231938241e+46684057', '-8.68003468995683807469421229806080844463e-6301', '6.661555366345231938241e+46684057', 99, 0);
    T('2.258945584724020854791909240893236e-218210650328026', '4.978945174976879405740805753296562915678066861e-4155182303155', '4.978945174976879405740805753e-4155182303155', 28, 3);
    T('1.77689993298062260051e+862539524177', '8.3794045190391748e-125938037', '1.776899933e+862539524177', 11, 6);
    T('-8.23967056448330249486e-7052531607049', '7.73013338906266556478811907283842819685597466058210698e+49351133232', '7.730133389062665564788119072838428196855974660582106979999999999999999999999999999999999999999999e+49351133232', 97, 3);
    T('-3.103083e-28448623166', '-9.811809474878e+51135839793437', '-9.8118094748780000000000000000000000000000000000000000000000000000000000000000000000000000001e+51135839793437', 92, 3);
    T('8.2563e+101', '8.375776424e-9037', '8.2563e+101', 22, 3);
    T('5.69513801479539973574415038166406e+171', '4.50e+35059', '4.5e+35059', 20, 6);
    T('-4.270278494e+208754092178750', '-6.784832070441240340689183732721671611260746267e-494021', '-4.2702785e+208754092178750', 9, 3);
    T('3.61067787e+921577401269104', '1.13904e+3525', '3.61067787e+921577401269104', 57, 5);
    T('-3.776350208299926467649059643391696e+876922', '-3.92606141865848019404025597266e-57819503', '-3.77635020829992646764905964339169600000000000000000000000000000001e+876922', 66, 0);
    T('6.0670850109086173907352060032213045266406486705411106825602e+3543208044', '-9.37975939044774767767044725249222166820554e-323880372714419', '6.0670850109086173907352060032213045e+3543208044', 35, 1);
    T('6.40e+57625', '-2.195387392494877062576918380e+9070073181502', '-2.19538739249487706257691838e+9070073181502', 94, 6);
    T('1.46357552311270336794372101e-9', '3.11874779798401002995609071e-39150802', '1.46357552311270336794372101e-9', 71, 5);
    T('-3.37560632e-672', '5.71136216548082401953474547922775562318328e-30674882', '-3.37560632e-672', 36, 3);
    T('-6.072952702549212e+975782246390285', '-9.2136248155907481e-95853898289701', '-6.072952702549212e+975782246390285', 75, 5);
    T('-3.1377483267052045460480829518513e-13912450', '5.83967156374416205813571e-778190', '5.83967156374416205813571e-778190', 86, 2);
    T('8.62315147636443225470277e+344899666221811', '-2.5175e-140', '8.62315147636443225470277e+344899666221811', 39, 0);
    T('-7.45482801332693454744998865277103136300781899875764662566960e+222445', '6.348e+7883354', '6.348e+7883354', 7, 5);
    T('5.88708231358134115232397797929060059059177488111066e-745901849034965', '-9.57860872174580700807954556047438080646928278190444215259219e+606376716591659', '-9.57860872174580700807954556047438080646928278190444215259219e+606376716591659', 99, 3);
    T('7.27559182127290069301639834484614055793617416245e+772849', '-5.637765909997350937771720148086729874288084370120577317211e-71038', '7.27559182127290069301639834484614055793617416245e+772849', 88, 0);
    T('-9.1191408464892898636186e-763945546559141', '-8.350638808274922760357583213696333424e-1881095733601', '-8.350638808274922760357583213696333424e-1881095733601', 66, 4);
    T('-6.3114e+9', '4.174623174166400e+992546966970', '4.1746231741664e+992546966970', 52, 2);
    T('7.3265871937801270202845205198608478604500211449629081853e-980091504503', '-2.785e-84885338624584', '7.3265871937801270202845205e-980091504503', 26, 3);
    T('4.320222361247189890724001444017025306e+82894495416', '1.3959757309720168893456062704828359194313978082289273660e+14490361585145', '1.3959757309720168893456062704828359194313978082289273661e+14490361585145', 56, 2);
    T('9.687896614430949157335603961027463030538482433125837571498e-7199155061', '-3.263460507524621701462929568866340e-677', '-3.26346050752462170146292956886634e-677', 41, 4);
    T('-6.5309411319367158405387948063652317614e+72114', '3.718310180271847334060626760862960187021186133e-6832', '-6.5309411319367158405387948064e+72114', 29, 6);
    T('7.23014707368306783857990822228043344554763503e+952670204', '-1.29675924293e-3092341617699', '7.230147073683068e+952670204', 16, 4);
    T('8.5383e-74535', '-2.5060452180551931876347992033172477724144e-479918624684', '8.5382999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e-74535', 95, 1);
    T('9.3275758e-3928', '-276135868552577007560772069865542567632532072268692.3', '-2.761358685525770075607720698655425676325320722686923e+50', 63, 3);
    T('-4.85081751161261541282238507311958317145761534540006375555414e+1672501108883', '-9.9037757112882085282232078196305053e+85619580', '-4.850817511613e+1672501108883', 13, 3);
    T('9.02773610948663261091705664890584820334e-5719780444596', '-4.35526887544468520969e-350', '-4.35526887544468520969e-350', 64, 3);
    T('6.4434e-5619993', '-4.8857770051649718202581046921160458020033732907504e-57404', '-4.88577700516e-57404', 12, 5);
    T('-9.115218669647360790487360053517432409000221043e+7716365', '-7.31626251089731271243507604520077069660923970714910687e-12743702750552', '-9.11521866964737e+7716365', 15, 0);
    T('-4.75011027974829737545919522424e+25698456', '7.472254411562310e+983', '-4.750110279748297375e+25698456', 19, 4);
    T('4.34476134153758249089181510759529377163836656427e+8594639355814', '57483717754323300087401141122023900090798708085441009.72584', '4.3447613415375824908918151075953e+8594639355814', 33, 2);
    T('-1.95639750654564172717869188621178458686841061772255218327e-12466592235', '-8.3911519707319137060164661119037784068771e+1309563', '-8.3911519707319137060164661119037784068771e+1309563', 59, 2);
    T('9.084754696862469937771946602e+36153', '-6.66438531150933462779737150628535748911830712e-3013967075143', '9.0847546968624699377719466019999999999999999999999999e+36153', 53, 1);
    T('1.402625987016921e-893', '9.39567897615511048589862912991848963470560986736123001657e+17681319', '9.39567897615511048589862912991848963470560986736123001657e+17681319', 67, 4);
    T('-3.9679119e+550466314', '5.524118186170591156205304733388783447769303969e-942986733', '-3.9679119e+550466314', 82, 5);
    T('-3554930822057463405617005527988384359228632707042673.3559896', '8.438676059033961695306880409496669e-73449291', '-3.55493082205746340561700552798838435922863e+51', 42, 4);
    T('-2.8720e+28024', '-4.603230856802681927623651545805e-3011321', '-2.8720000000000000000000000000000000000000000000000000000000000000000000000000000000001e+28024', 86, 3);
    T('2.8382474565239956737589e+9825792372', '4.101316227011465460161272702061868263306326097e+1542', '2.8382474565239956737589000000000000000000000000000000000000000000000000001e+9825792372', 74, 0);
    T('0.00000455079545633071708945445834509775872811280208796614218076039', '4.32683e-594298308', '4.550795456330717089454e-6', 22, 6);
    T('-8.314413072533578063303573955551994641501252839716517e+552112453213303', '-5.9066381332525007531702684863898133744332136628902997170955e+131', '-8.3144130725335780633035739555519946415012529e+552112453213303', 44, 0);
    T('-4.374083683836502979561415290225109736120385172609539e-273', '-1.42414992474845963727921682e+44903322', '-1.42414992474845963727921682e+44903322', 74, 5);
    T('8.5128410055e+5005', '-5.54494531664143831691559253e-6286610680920', '8.5128410055e+5005', 47, 0);
    T('-8.200423286908697564497814773442070164e+108', '-5.22850484096183643663e+117', '-5.23e+117', 3, 4);
    T('7.40892531146646435163670882738200726940531e+87040908534', '1.99099e+9476842', '7.408925e+87040908534', 7, 5);
    T('-8.0e-72764163690', '6.980585153786602e-39', '6.980585153786602e-39', 63, 2);
    T('-4.513925439105279024e-62323', '-7.428552437706112210811377839127854305e+21721', '-7.42e+21721', 3, 2);
    T('2.31650850987312246279754e+581796', '5.94437045387385096480120733857e-97476299', '2.31650850987312246279e+581796', 21, 1);
    T('-9.8701321913976945097472348e+28', '1.99837065630773272366783461204900901232614192693461391140e-443949587882', '-9.8701321913976945097472348e+28', 33, 3);
    T('-7.26479422208387916725345919756e+54184778460', '-2.451714791840288248701605207438673e-2127', '-7.26479422208387916725345919756e+54184778460', 98, 2);
    T('1.220349849145e-61777', '-7.508689417394e+962', '-7.50868941739399999999999999999999999999999999999999999999999999999e+962', 66, 1);
    T('7.4179211750635650474269433703614775928213495e-341', '1.118958596735448219221792176349328667094e-189', '1.11895859673544821922179217634932866709400000000001e-189', 51, 0);
    T('-4.5597372931620710612125880481117935198247362998696308983e+845', '-2.554595828774658829162e-59660996505', '-4.5597372931620710612125880481117935198247362998696308983e+845', 73, 1);
    T('4.694084187568648885528647109345212e-9559', '2e-4228', '2e-4228', 66, 3);
    T('-1.9166774583855690448682124619000e+35', '-2.86133524575513339092506396968543461284831485619353731e-39614375571', '-1.9166774583855690448682124619e+35', 77, 1);
    T('5.32811270288197825696285071446721987329e+173', '1.88814962255257e-94898006122', '5.32811270288197825696285071447e+173', 30, 6);
    T('8.491918242041669225018612255597e-928208007', '-5.69012179138396473875132187e+357307', '-5.69012179e+357307', 9, 4);
    T('4.13279659493115410131351992407494673663842e+5413', '-3.378723431267940399086090357187e-726052919', '4.13279659493115410131351992407494673663842e+5413', 47, 5);
    T('9e-42358', '-2.588447601520644468570276234555829183246713e-9242767873166', '9e-42358', 63, 4);
    T('-5.299649587229867945119646098278381011532276199412e-7701460744', '-2.945118186308871e-872', '-2.945118186308871e-872', 49, 1);
    T('9.478631844532318444308786e+910349495214008', '3.5299473456550463530557546676148880797830507898978e-97192572375', '9.478631844532318444308786e+910349495214008', 56, 1);
    T('-4e+52022491683443', '2.81534656865747892053904147096490e+355375499548', '-4e+52022491683443', 78, 3);
    T('8.64530699477175262726466861249883088871668397080950147e-78879702397', '-9.9420585261543082439477771306603543010722374381721e-2364759519', '-9.94205e-2364759519', 6, 1);
    T('-5.35632867929246296638042706408573871e+42', '8.6038112877454392876232581086804974707607e-48', '-5.35632867929246296638042706408573870999999999999999999999999999999e+42', 66, 2);
    T('4.226077594005400203404e-89797223', '-8.39349391438076129982045567242382407566665703116e-944692947', '4.226077594005400203404e-89797223', 36, 2);
    T('630127296134907662307442985899229.508540', '7.079663430117988613429996140e-710957978163901', '6.30127296134907662307442985899229508540000000000000000000000000000000000000000000000001e+32', 87, 2);
    T('-5.2439937665602504706858424358689178212144e+6344988156807', '8.3972341082718777390e-940', '-5.2439937665602504706858424358689178212144e+6344988156807', 50, 4);
    T('-8.868772702824172323114488449368575950766974197e-6683725996', '4.3879626850507738177084378356894726104329769563844179e-942164875', '4.38796268505077382e-942164875', 18, 6);
    T('-2.389196423298678560785826617e+756253819539791', '3.4074925419558575840414162800380007954392e+242783321', '-2.389196423298678560785826617e+756253819539791', 79, 6);
    T('6.83539644601564129539e+527023688', '-6.92012050384602652041916e+8682033', '6.8353964460156412953899999999999999999999999999999999999e+527023688', 56, 1);
    T('-8.06089397e+54904405', '3.49e+282794286', '3.4899999999999999999999999999999999999999999999999999999999999999e+282794286', 65, 1);
    T('-8.465e+523184796', '-4.735873219831701834057116170880811130e+51', '-8.4650000000000000000000001e+523184796', 26, 0);
    T('8447428136110873824313318740481365020832983388503567.2056', '5.8240783394479563186655105239105479e+72729052', '5.8240783394479563186655105239105479e+72729052', 79, 1);
    T('-4.126533808254984316359352099984659786348067e-45339795910318', '3.4363448741964332e+1346', '3.43634487419644e+1346', 15, 0);
    T('-2.785444933515914189491293129120010290502613194e-2155', '1.74199291921939310522107430394273660046623960696693e+55', '1.741992919219393105221074303942736600466e+55', 40, 1);
    T('-9.14022149754251073609435116e+614389', '-3.281660427555579886531601814741854399524891817556317e-615770016155', '-9.1402215e+614389', 9, 6);
    T('-1.980469369148125009129225806e+9260109', '8.66122369672609891494101967731426906690145494071619091644120e-908759172', '-1.980469369148125009e+9260109', 19, 5);
    T('3.67407176363701421589e+520209107', '-8.12705217629168670679854691477053012426071223e-50849', '3.67407176363701421589e+520209107', 28, 5);
    T('-1.0518688130696514352735973082426574830e+6824062596', '5.822567935279937679791552504195473019e-185558800375946', '-1.05186881306965143527359730824265748299999999999999999999999999999999999999999999999999e+6824062596', 87, 2);
    T('1.2110625671558991676298e-144870048256', '3.69658667141780269233443408639152710337435396857207168e+48993', '3.696586671417803e+48993', 16, 2);
    T('-2.70195847839565688380009040638333193415341e-9504866', '-84912293696.555420807843651855539278337945653197443028071539', '-8.49122936965554208078436518555392783379456531975e+10', 48, 3);
    T('-145750371292', '4.2487100348203603930585e-63597846495', '-1.45750371292e+11', 58, 5);
    T('-9.140906418393926644196833e-6199774538128', '6.642400461490274999191256349489345e+798176371909841', '6.642400461490274999191256349489345e+798176371909841', 67, 2);
    T('4.32972021533124433e+73', '-9.137599268476397691e-47790', '4.32972021533124433e+73', 78, 6);
    T('3.753111616865387159850250071516973e+418189', '-5.02565264128712817214795e+448445', '-5.02565264128712817214795e+448445', 78, 0);
    T('-2.5204902635328555737124175e-920243', '481659459931660140950858827807557554343208316456.535536321', '4.81659459931660140950858827807557554343208316456535536321e+47', 91, 6);
    T('6.807436495469972931823865774384750111232e-3061', '-8.3833384324695483135223996e-987889091', '6.8074364955e-3061', 11, 5);
    T('-1.11173503812865353265535367971356678966584183990917325784774e-3349166214', '8.840920e+2801', '8.840919999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+2801', 100, 1);
    T('9.4679955318202378563513622136206290741e-8956659', '1.226e+8', '1.226e+8', 18, 3);
    T('3.478378613660938994495484306102769e-13716', '-3.6389114257573060682170986169290e-67725573337', '3.47837861366093899449548430610276899999999999999999999999999999999999999999e-13716', 75, 1);
    T('7.553692427034712349022171944480924242501667134e+565767412', '-2.558382672214525347941884823874902052e+8080226025', '-2.558382672214525347941884823874902052e+8080226025', 84, 4);
    T('7.88759836333563e+479628978', '-1.666702410225408899e+46584757787052', '-1.666702410225408e+46584757787052', 16, 2);
    T('271893117795814217378952864254009339.0970', '-2.9596742720773920789308910310987372393389936182814571133e-28', '2.71893117795814217378952864254009339097e+35', 51, 4);
    T('-5.164500094439506e+12475599', '4.590142940341236801387894491967308749458548739506086564793e+842', '-5.1645000944395059999999999999999999999999999999999999999999999999999999999999999999999999999e+12475599', 92, 1);
    T('-2.743606863051515162773359256323015568331e-9897782449', '-7.224514438970947625937909818731e+48172360657582', '-7.224514438970947625937909818731e+48172360657582', 45, 2);
    T('-2.91080782959455105094345937123599709671e+47', '-5.5207637751988385636236903557400497328422151587e-6061876472', '-2.910807829594551050943459e+47', 25, 6);
    T('5.591228634681016169453030473998016842e+8501079579893', '6.573067994720280271060453070257137978e+423340599', '5.5912286346810161694530304739e+8501079579893', 29, 1);
    T('7.1946754441774457899675479938e-432289742955', '9.18933237299912834991206071130152e-5806445', '9.18933237299912834991206071130152e-5806445', 96, 4);
    T('3.708556004342e-50803466', '8.08972004899057140658045e+40346187003', '8.08972004899057140658045e+40346187003', 56, 4);
    T('-6.40868e+1182444759', '7.530554436543961890654871553327505366640038737065858171e+189885', '-6.40868e+1182444759', 30, 4);
    T('-0.605719', '9.4450837834577483828e+1620470815', '9.4450837834577483827999999999999999999999999999999999999999999999999999999999999999999999e+1620470815', 89, 3);
    T('9.498406290632885885912596368896142e-700088190', '7.0472384195021283082393919536947353434286938422746018e+746422612495', '7.0472384195021283082393919536947353434286938422746018000001e+746422612495', 59, 2);
    T('2.7958862742274e+97743113', '-9.216395401732515907217010134303232224490809778679224990800e+91052596382230', '-1e+91052596382231', 1, 0);
    T('6.49e-2771569890936', '-5.862385107253562e-484880', '-5.862385107253562e-484880', 93, 6);
    T('4.821118294377745000750561953157759272769e+6557', '-3.900368546675684126978e+73641148527', '-3.900368546675684126978e+73641148527', 84, 3);
    T('-3.73513403552374888559964143857703e+1401', '2.45954576434570840307560782872537267409e+389629463', '2.45954576434570840307560782872537267409e+389629463', 80, 0);
    T('-38760580.985540557', '-5.234100680748612477203949482443282947467e+2995573594', '-5.2341006807e+2995573594', 11, 1);
    T('-8.793773155124957140350250083e+3923736942', '1.38942834150478332698778679460494605532704945e-51055', '-8.793773155124957140350250082999999999999999999999999e+3923736942', 52, 2);
    T('1.03323643475120222471691177150e-71276422642554', '-8.98957737075107070696e+857617703624221', '-8.989577370751070706959999999999999999999999999999999999999999999999999e+857617703624221', 70, 1);
    T('-7.5985350264200768018688470200804134416101237868031655082697e+4499019286371', '-0.000080406566254586', '-7.5985350264200768018688470200805e+4499019286371', 32, 0);
    T('4.9451515255011879e+820313', '-9.13636063e-357336928603674', '4.94515152550118789999999999999999e+820313', 33, 3);
    T('-9.097378383815965596207995326959263668e+42', '-9.66588379601190150443061205742602688119331647143639399179585e+84189941019816', '-9.66588379601190150443061e+84189941019816', 24, 4);
    T('5.56341428049387678017429177427615893e+9950672502', '-4.51463271164e+279778161', '5.563414280493876780174291774e+9950672502', 28, 4);
    T('5.624237783397173565154173e+89305792668', '7.30747460577987582944032745648635068048062402e-436', '5.624237783397173565154173e+89305792668', 63, 3);
    T('-0.0000969242680269246019380507421023567166', '-2.816210230327758660229324533840e-451', '-9.69242680269246019380507421e-5', 28, 5);
    T('6.360640823280020793380868412228769499100443775657e+6154166950682', '-9.77541758371338596226431007361872605569971833804084760893e+193022387767979', '-9.77541758371338596226431007361872605569971833804084760893e+193022387767979', 83, 6);
    T('-6.12291551555025530690071079812007917840124738799082965e+128', '5.6504109e+9590', '5.6504109e+9590', 82, 2);
    T('-7.18183400108223536203079417514378633961557e-22927', '-4.01578079847492513686763e-64506', '-7.18183400108223536203079417514378633961557e-22927', 66, 5);
    T('-6.06393705216501559e+84083665698', '-8.36641537325699813e-289745', '-6.06393705216501559e+84083665698', 53, 2);
    T('-8.495395864287959413146962742459246760710420936380310e-470375505600', '-6.134721435685253016255350587007189243771377694e+239082119721', '-6.134721435685253016255350587007189243771377694e+239082119721', 56, 6);
    T('1.8284e+9', '-9.69588007496712706261e+9047763513859', '-9.69588007496712706261e+9047763513859', 44, 3);
    T('8.181122298414642731066953481992277e-57', '-2.6357038003352545146819464128927053369810011016200e+1381355903', '-2.63570380033525451468195e+1381355903', 24, 3);
    T('-4.86984968204754741234559831508543955624723835745130e-93239941921', '4.400926622388408645544802e-29627', '4.400926622388408645e-29627', 19, 1);
    T('-4.856e+80738', '2.289571413281e+56580015790', '2.289571413281e+56580015790', 71, 4);
    T('-3.2857319747960540971278e+67646', '-1.447662607555447658708834860388510995e+7320204829933', '-1.447662607555447658708834860388510995e+7320204829933', 75, 2);
    T('-5.317758821254695502224763e-195717530', '-9.40260729801299215198625095891007081561222872994435e+8107255089', '-9.40260729801299215198625095891007081561222872994435e+8107255089', 90, 6);
    T('-9.604756302604e-21779539626', '-7.635666779714753899068866426218596058551453478616613497e+40751998305', '-7.635666779714753899068866426218596058551453478616613497e+40751998305', 60, 2);
    T('0.0006', '7.29204333453891641565778860499389455559200380580011615551751e+91097645', '7.292044e+91097645', 7, 0);
    T('2.62354286021046619e-7697619413', '-3.458029105412352517503367764943658731108235484885755e-26909653', '-3.458029105412352517503367764943658731108235484885754999999999e-26909653', 61, 1);
    T('-7.6143e+69925', '-14350906531.477267', '-7.6143e+69925', 65, 4);
    T('-5.5961235129790415e+2676351400458', '5.181710139434078001135594880976129026079526185587e-165776', '-5.596e+2676351400458', 4, 6);
    T('-7.5238822300936620746639450e+8815483678550', '-5.6170202370822646438689807312070201936e+129', '-7.523882230093662074663945e+8815483678550', 40, 5);
    T('1.9074354526680524187204001837e-41871', '-8.60488526e-514564', '1.9074354526680524187204001837e-41871', 92, 6);
    T('-7.1481e-9542', '3.96852130e+282', '3.9685213e+282', 56, 2);
    T('-4.44872656366979227012186218e+906507', '-2.4873621205197817903832e+6405094', '-2.4873621205197817903832e+6405094', 96, 6);
    T('-3.69603343936247520281021066577532314e-4378601', '-6.3957021119037306251014057424650757633535456566672983888e+9815386', '-6.3957021119037306251014057424650757633535456566672983888e+9815386', 84, 5);
    T('1.8023877722275073394229359668746335786085238024150797e-73581138', '-3.69772601453883869242709e+56259513647829', '-3.69772601453883869242709e+56259513647829', 47, 0);
    T('6.233214118709591829550220e+9534925', '8.54930355606e+960244124508814', '8.54930355606e+960244124508814', 29, 1);
    T('-3.81624041e+1866818970322', '9.55791891165534e-266924198', '-3.8162404e+1866818970322', 8, 2);
    T('-176217818373073228911901440968181507602859467.95', '-5.942663769232823865510981e-521', '-1.76217818373073228911901440968181507602859467950000000000000001e+44', 63, 3);
    T('81500520524626334232781276289212967398.429', '8.0392092951514662e+961464', '8.0392092951514662e+961464', 84, 6);
    T('1.7517319224751525930088409905653588955720518866119692162698e+117', '-3.83390289685339243211703464177631509e+9830', '-3.833902896853393e+9830', 16, 3);
    T('-2.2472238891917303188785699351770990e+8015442', '9.765093909512452632788308e-9491641', '-2.247223889191730318878569935177099e+8015442', 62, 6);
    T('-3.0840144502e+14', '6.6539550076852273344019981398010654062107372389991724e-25331433497', '-3.084014450199999999e+14', 19, 2);
    T('-5.316674484974492984031203610437918e+9644992875', '-5.6052011951255351914860759041624e-6840', '-5.316674484974492984031203610437918e+9644992875', 69, 5);
    T('-2.23879760202541826508111707556327818235965936671818e-94968407458258', '1.83600720087373100476970550248e-812678485344', '1.8360072008737310047697055024799999999999999999999999999999999999999999999999999999999e-812678485344', 86, 3);
    T('2.48617851542213716844190e-862343468815816', '-3.8719327132913890476003941614743679708219508e+45794855411857', '-3.8719327132913890476003941614743679708219508e+45794855411857', 92, 4);
    T('3.48250121345179326266852550330389019966e+127', '239.584981', '3.48250121345179326266852550330389019966e+127', 40, 1);
    T('-4.243553626896972915585269e-48813509', '103452192937396273690995077270874096699874222803.1', '1.03e+47', 3, 3);
    T('7.80068370335599894019771707944919417023617599199661686971440e-492741', '-6.11463466018482314315550896045021746436121335920144382592359e-756494999463', '7.8006837033559989401977170794491941702361759919966168697143999999999999999999999999999999999e-492741', 92, 3);
    T('712146914963988672724668855080636697985156084.286', '1.75907150861965831844768724946e+4695982431', '1.75907150861965831844768724946e+4695982431', 41, 1);
    T('-8.12018629086951e+2764746064676', '9.38196157059126585701510982e+5999315374', '-8.120186290869509999999999999999999999999999999999999999999999999999999e+2764746064676', 70, 2);
    T('-5.19632639429550378167263520596378495924514248675871616780480e-307886222081', '-94461330', '-9.446133e+7', 37, 2);
    T('-68034777620249480798176045961251706609527.57', '7.867356272336127318841492510703e+864403351120455', '7.867356272336127318841492510703e+864403351120455', 83, 4);
    T('-6.4787974496847961390899911069443e-4368400123', '35697389294432205379185009062852187517325692759.0239166', '3.56973892944322053792e+46', 21, 4);
    T('1.5494783159937547329193561352359774743960e+474687', '-6.961072786861121365486454754007960696689e+111132192', '-6.96107278686112136548645475400796e+111132192', 33, 4);
    T('1.80179947423260326519940447831142491043084871409012879e+63198', '22442743034383429093.558615936635544560948064487012891747', '1.80179947423260326519940447831142e+63198', 33, 4);
    T('7.7501897920653238503399360512979238852172255747127130067461e-625541', '3397725236843.1', '3.3977252368431e+12', 33, 5);
    T('-5.32268760929208155122646936968313e+25889050344', '-4.59626096965806770739860006069547433475768738e-88742425594', '-5.32268760929208155122646936968313000000000000000000000000000000000000000000000000000001e+25889050344', 87, 3);
    T('-9.758998605280012144580797548941490339759569227960650e-702662626312', '9.72201781945416e+755', '9.7220178195e+755', 11, 5);
    T('-7.127626206903907e-7719910', '6.83247664312e+50432', '6.83247664311999999999999999999999999999999999999999999999999999999999999999999999999999999999e+50432', 93, 1);
    T('3.6705757346120660032148910850e-85952770', '-8.428738099831598779765347081540706684556839895903034466907e-37392', '-8.42873809983e-37392', 12, 6);
    T('6.358111e-1223101943', '7.027281375831362e-104', '7.027281375831362e-104', 51, 4);
    T('1.7656311189272083318277e-288225780719686', '-8.43632986184665793905e+45820', '-8.43632986184665793905e+45820', 94, 0);
    T('3.5415497413979862573054384864904677263857803584e-58', '-4.433721160645881909905349452425248237450173254e-369', '3.541549741397986257305438486490467726385780358399999999999999999999999999e-58', 73, 3);
    T('7.1222306e-85942', '7.1502802947619725396051071426317171845326e-8866', '7.1502802947619725396051071426317171845326000000000000000000000000000000000000000000000000001e-8866', 92, 0);
    T('4.94447422996349883073489e+7112', '-1.401414783212646846410044265021067186796177333699460901383e+7134997883892', '-1.4014147832126468464100442650210671867961773336995e+7134997883892', 50, 0);
    T('6.397e-43244542', '-5.663694755480e-51928620112', '6.39699999999999999999999999999999999999999999999999999999999999999999999999e-43244542', 75, 3);
    T('-1.160193209e+86524914', '3.2581635e-463051005473117', '-1.16019e+86524914', 6, 6);
    T('9.90083742396222176807609761e+8812', '3.804178905389432977448916584776598385772007055e+995653268939', '3.804178905389432977448916584776598385772007055e+995653268939', 65, 3);
    T('-7.114032459253451525580646403650e-31380', '2.26362985327654966819255777092848332994770464021304e-9584181', '-7.11403245925345152558064640365e-31380', 74, 4);
    T('-8.96657635779088041e-38472974619', '-7.6351491606630e-850804688', '-7.635149160663e-850804688', 77, 1);
    T('5.7677693e+51112', '-1.65335656428491e-9116', '5.7677693e+51112', 82, 2);
    T('8.443616645756e+302710625702', '-1.9913272078360337343996388143485208560e-821578587', '8.4436166458e+302710625702', 11, 4);
    T('1.8991894202746e-57512010', '8.9454898e-84087230497', '1.8991894202746e-57512010', 53, 3);
    T('-8.978619120398039819340172e-9506', '-8.625642620901499546e-4720470787', '-8.978619120398039819340172000000000000000001e-9506', 43, 3);
    T('-1.2112769270454689698726673396898610163919024e-12539573', '-3.8e-2189515', '-3.8e-2189515', 98, 4);
    T('5.160369455144887085234315327506561211e-10773', '3.354907887039836309740501890571707339437128813650e+6659', '3.35490788703983630974050189057170733943712881365000000000000000001e+6659', 66, 0);
    T('5243331.3776942054008408798', '-5.0e-4488407154028', '5.2433313776942054008408798e+6', 80, 5);
    T('6.8054e+3685203999192', '6.74377467125867535510582529319662885110140248908e+18880586913755', '6.7437746712586753551058252931966288511014024890800000000000000000000000000000000001e+18880586913755', 83, 2);
    T('-9.902602304442319179630662708851946606e+8746534', '6.077432582604460767e+49476', '-9.9026023e+8746534', 9, 5);
    T('4.2767247451881716855556162501506506487235910052393462088e-926162453220718', '7.46649012697824351346182848029760209233386e+21562941149', '7.4664901269782435134618284802976020923338600000000000000000000000000000001e+21562941149', 74, 0);
    T('-9.6120349466658869647179e-58781169', '7.409346503740753264380512089521707739367406e+31919509770270', '7.40934650374075326438051208952170773936740599999999999e+31919509770270', 54, 1);
    T('-1.0638281711159894860585279870e+31', '-6.22e-68303', '-1.0638281711159894860585279870000000000000000000000000000000000000000000000001e+31', 77, 3);
    T('5.97624780449816693708137165993565078766899264305308003e-432905239526360', '-4.66e-69', '-4.66e-69', 30, 0);
    T('-4.88064435527013176557e-19598331584669', '3.961725435389e+813532752', '3.961725435388999999999999e+813532752', 25, 1);
    T('-5.66510535259895964461813483718741508e-19671683235747', '3.524549896e+215', '3.6e+215', 2, 0);
    T('-4.487992545300165416680655167949821333574e+82188033078', '-6.0822959974594165497981145655602389417762079032117e+16092865', '-4.487992545300165416680655167949821333574000000000000000000000000000000000000000000000000000000000001e+82188033078', 100, 0);
    T('-8.1486141255762201002e+48250', '6.11596079790071632369872407931209543017657861393385172e-3766577254376', '-8.1486141255762201001999999999999999999999999999999999999999999999999999999999999999999999999999999e+48250', 98, 1);
    T('8.631005243805509275922157215507770385205043316498923693e-5388513079570', '6.5399582651419844374031873963271924095570096972e-442', '6.5399582651419844374031873963271924095570096972000000000000000000001e-442', 68, 2);
    T('-9.92073396965340658263e+1413', '9.83625129762834556027694532512287528777049628099003825841e-48299362', '-9.92073396965340658263e+1413', 90, 5);
    T('-6.446703104285868591546456185270e-806131', '-6.44221009192691725187468898430e-929', '-6.4422100919269172518746889843e-929', 81, 4);
    T('-4.084e-16887303639064', '-9.63456214423025182949588e-65268126520', '-9.63456214423025182949588e-65268126520', 37, 1);
    T('-87914889094.298998857', '1.7863123450304076073289894718671e+7961631', '1.7863123450304076073289894718671e+7961631', 70, 2);
    T('4.8255544417616e+296771080', '6.6264151957789103897757438215024379480453379466800794907e-7638879713', '4.8255544417616e+296771080', 74, 4);
    T('8.3848e+17265', '-7.5454669242e+51791878838185', '-7.5454669242e+51791878838185', 12, 0);
    T('4.81411763983218249084900947227402087263522371752906e+602488409', '-3.93631165425859856e+23', '4.8141176398321824908490094722740208726352237175290599999999999999999999999999999999999999999e+602488409', 92, 1);
    T('9.7270069978e+20084363', '-4.9238570943666430361092471844515340195423354e-2814495687851', '9.7270069978e+20084363', 44, 6);
    T('-1.82134281507745661568799e-74361', '3.8498603136932e+3551', '3.8498603136932e+3551', 36, 0);
    T('97804284180483174886552446361277746701956.8604', '-9.2203182424076985378203831580173804234938e-889811904333', '9.78042841804831e+40', 15, 3);
    T('3.5763595396483593824927067230775509527e+91296474', '-8.71397098131e+56106538', '3.5763595396483593824927067230775509526999999999999999e+91296474', 53, 1);
    T('-7.7701766e+66026154', '-8.9780482e+1425166558454', '-8.9780482000000000000000000000000000000001e+1425166558454', 41, 0);
    T('-3.671909e+56674546384', '-8.6908184638289163805591085137212725400e-8256166466', '-3.671909e+56674546384', 27, 6);
    T('-706065107169678658716913597437918.76', '-2.228095438232892062034022110672600227335834e-7553985714', '-7.06065107169678658716913597437918e+32', 33, 2);
    T('-1.52221124001000666567895163139632703240e+8119903', '-4.47595599249705724722285794118850428707655022703995425915e+64', '-1.5222112400100066656789516313963270324e+8119903', 78, 1);
    T('-7.08728800552792699273969e-493635370816678', '-9.587513607461051975187738384392084e+463287564719', '-9.587513607461051975187739e+463287564719', 25, 0);
    T('-9.3462129451383301633611223148873e-824', '8069333338383909437439107097274805166351951423.161966', '8.0693333383839094374391070972748051664e+45', 38, 0);
    T('7.053868771e+373', '-7.0947297529304993248179310491571720e+90278789', '-7.094729752930499325e+90278789', 19, 0);
    T('-2.9e-29', '3.30409225458668670075163183e-4915342498119', '-2.9e-29', 27, 5);
    T('-7.46820644218308250e-5689785597', '6.66e+645', '6.6e+645', 2, 1);
    T('1.53033e+9121027020', '-703493.161764109994694520', '1.53032999999999999999999999999999999999e+9121027020', 39, 3);
    T('3.75709248781503434788399319251695720127029115e-9695137', '-6.27745822916765690e+21319', '-6.2774582291676569e+21319', 25, 4);
    T('9.5915490396836919450582451373420004313749136e+49', '-6.1497767780167950258232761404742075e+321704934', '-6.149776778016795025823276e+321704934', 25, 5);
    T('3.5742e+1878683187', '-6.64380302347670889267556637001282130505199197742400610439e+3152460803', '-6.64380302347670889267556637001282130505199197742400610439e+3152460803', 76, 3);
    T('-5.836401001645556698935670349818107878087680628189e-614381645665', '-6.54466576638e+9850908', '-6.54466576638e+9850908', 30, 4);
    T('1.1548869620624399759739456746940306232617773278176e-9781163619357', '-9.224199e-51', '-9.224199e-51', 17, 6);
    T('9.60562994125324421444886672574253e-776675', '-5.791455406433624777928315704585384106313548106725650e-470', '-5.79145e-470', 6, 2);
    T('3.915980436112858145902404268624756e-5088', '-3.85813430788064825e-23805', '3.91598043611285814590240426862475599999999999999999999999999999999999999999999e-5088', 78, 1);
    T('2.04326018720581915113469349095140003807850578741814915659e+723786588863', '-6.536340473855684708079261239326340664104543857353035605e+977340665380289', '-6.53e+977340665380289', 3, 2);
    T('-8.4215956156256159022126868174057e-94011', '-8.7927943012752952404856e+876498609', '-8.7927943012752952404856000000000000000000000000000000000000000000000000000001e+876498609', 77, 0);
    T('-6.552433464628923188452173338451455062387313e-1464', '6.84e+2980410495758', '6.84e+2980410495758', 79, 4);
    T('3.4027709610e+31729716562', '-1.724566237519750095e+8327323773', '3.4e+31729716562', 2, 3);
    T('7.385375735470175976681366477004385449147e-41994', '3.57921784640359799091761e-988888107', '7.385375735470175976681366477004385449147000000000000000000000000000000000000000000000000000001e-41994', 94, 2);
    T('3.678578660207437e+1594461118439', '8.117047923154509705347368228e-340655690921173', '3.678578660207437e+1594461118439', 46, 6);
    T('4.890113575951026e-68', '-5.4291617186450227964777256105736241402e+314093664', '-5.4291617186450227964777256105736241402e+314093664', 80, 3);
    T('2.58915258383766906954022473272299909827895170938e-68612161', '3.949760040276868747e-107036930654', '2.58915258383766906954022473272299909827895170938e-68612161', 77, 6);
    T('-3.911531e+59312804', '6.3e-4574', '-3.9115309999e+59312804', 11, 1);
    T('-2.6414710073824836516109060367168140292664293432573775e-700', '0.671831', '6.71831e-1', 76, 6);
    T('3.120276046664139717661587368363760837872960e-473123694842', '7.83724428545137239797428056863715908621940e+729138', '7.8372442854513723979742805686371591e+729138', 35, 0);
    T('-9.19636515459488241169250564122198932836658975310907437293e+1049', '-9.23728973768524873985725723533828484500430054999745881e+22055', '-9.3e+22055', 2, 3);
    T('5.41845870676311661348432464517355889939631400371125353e-459138', '8.416756238424263599686474530632886e+92', '8.41675623842426359968647453063288600000000000000000000000000001e+92', 63, 2);
    T('-3.65070434151789302292586359e-69918504849', '3.7281642481427e-72995807550011', '-3.6507043415178930229258635899999999999999999999999999999999999999999999999999999999999999999e-69918504849', 92, 2);
    T('-4.48586306429772379390154909941196823e-43807694', '-2.5999958243483484380352115582977525e-171821895', '-4.48586306429772379390154909941196823e-43807694', 47, 6);
    T('2.01744638123190996268850410784e+402169800', '6.7253556862516684937095443075291456441452430e-25659605251', '2.017446381231909962688504107840000000000000000000000000000000000000000000000000000001e+402169800', 85, 2);
    T('4.081227796159849217379973e-590265', '-2.8829810587084608721427154542243217956396120215761361e+916287730', '-2.882981058708460872142715454224321795639612021576136099999999999e+916287730', 64, 1);
    T('9.9442120446e-7557403389', '-5.079629769354121348419781885905e-30', '-5.079629769354121348419781885904999999999999999999999999999999999999999999999999999999999999999999e-30', 97, 1);
    T('5.765311350714303173329506e-492', '9.40410159856361341724560993104171304766521260580340686522850e+1055857525', '9.4041015985636134172456099310417130476652126058034068652285e+1055857525', 59, 3);
    T('9.868914931483839647902912483957e+160812404250953', '-9.8844517405068311513e-2108079699', '9.8689149314838396479029124839569999e+160812404250953', 35, 1);
    T('-4.067904332150650964013050170378969e+258997', '-2.65713485864004499355454293213074e+380444170676', '-2.65713485864004499355454293213074e+380444170676', 52, 5);
    T('-5.03627923782608553190123471734356856618111855246795623632608e-381310', '3.596791725215027862e-9217713920', '-5.036279237826085531901234717343568566181118552467e-381310', 49, 1);
    T('4.5970873010350925638229713264016008e+7809261454211', '-8.47838515e-272820402', '4.59708730103509256382297132640160079999999999999999999999999999999999999999999999999999999e+7809261454211', 90, 1);
    T('-6.1291708617698930e-7325634', '2.486e-72', '2.486e-72', 14, 2);
    T('7.688056982063863e+995164', '-2.58456505019525933949209521190384e-57037097', '7.688056982063863e+995164', 64, 5);
    T('-9.2117876627611122512872317627e+73347073508987', '5.764e-526672', '-9.2117876627611122512872317627e+73347073508987', 84, 0);
    T('7.172074713661691764116650216703268e+9626846214344', '4.247338159989029019e-4779', '7.1720747136e+9626846214344', 11, 1);
    T('-3.67651538226191001533689888192379325274371408796615308484e-5509051005951', '-3.0053233169601650491651964383775223122e-895654276516517', '-3.67651538226191001534e-5509051005951', 21, 6);
    T('-4.9182095686864392539609459067851258e-54991691', '2.29252425150933e-1331396204352', '-4.91820956868644e-54991691', 16, 3);
    T('9.50751384783564616654140218741996022334994423237448e+192', '-7.40014276628219677104933795985853832508818022247e+397668', '-7.4e+397668', 2, 6);
    T('-7.0036028835528511730265334654809235559172618e-306', '1.97366105925828040664836326743789590547157004166054952e-6197390', '-7.0036028835528511730265334654809235559172618e-306', 62, 0);
    T('-1.7151007104642414104006364625295e-228618', '9.601048155787069733024842223872221265864062201518451288351e+324', '9.601048155787069733024842223872221265864e+324', 40, 5);
    T('-4133800287331366232690647', '-7.45948958784850078729e-2374816255514', '-4.133800287331366232690647e+24', 78, 6);
    T('9.501373262351613e-29867', '-7.99e-945287523986', '9.50137326235161e-29867', 15, 4);
    T('-1.75437710541321228837562418809298e+24901111', '8.2347e-59240320044291', '-1.754377105413212e+24901111', 16, 2);
    T('-4.47e+103361042261305', '-2.381402786883229222334663815337236682731420679e-82819549224263', '-4.47e+103361042261305', 30, 6);
    T('-3.2895512913248660111513939004e+672906110898', '2.11233546255708927847259e-13574', '-3.28955129132486601115139390039999999999999e+672906110898', 42, 1);
    T('6.44357182614451e+170', '7.30983353428748490583883142083358002471717968173000806278318e+72', '6.4435718261445100000000000000000000000000000000000000000000000001e+170', 65, 0);
    T('7.7851931553766768334808627996400943773619418716e-97176928', '-9.167797211041092894495201784465880496976693650231165012e-540738501', '7.785193155376676833481e-97176928', 22, 0);
    T('7.136391129106874567401988148789282857e-48', '-9.5356126156271457397412586114e-657', '7.136391129106e-48', 13, 3);
    T('4.31611303311256651e-662', '7.2434e+7709948', '7.243400000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+7709948', 100, 0);
    T('-4.55437073552975248242638326e+103836355', '2.683951751730274746e-98270286', '-4.55437073552975248242638326e+103836355', 68, 6);
    T('-9.8002962326783184125261420522665992218037372079217949520889e+4641708664385', '-9.978826060285340061655681564252893253493938e-56568005', '-9.80029623267831841252614205226659922180373720792179495208890000000000000001e+4641708664385', 75, 3);
    T('-4.775463622695890333840402490e-14232864311', '3.670720699481759326598830581345610957e-30308', '3.67072069948175932659883058134561095699999999999999999999999e-30308', 60, 1);
    T('-3.6814247971468470488814992984733818665547672005882172333e-59712567', '9.16067385712526876e-438242042202259', '-3.681424797146847048881499298473381866554767200588217233299999999999999999999999999999999999e-59712567', 91, 1);
    T('2.8002320361888922617197849787351320515383382458369e-16030', '5.145168876648917010e-5464858', '2.8002320361888922617197849787351320515383382458369000000000000000000000000000000000000000000001e-16030', 95, 2);
    T('-1.85338528911633331228356579718380491027384e+21138', '-8.908821889220110357757982587429761978306783700e-203234046208317', '-1.8533852891163333123e+21138', 20, 3);
    T('1.330785749001294770689810174393777309e+40', '-1.58693470483358510048e-6151475798', '1.33e+40', 4, 1);
    T('-2.3299762944440492e-6941322', '-8.837018838e-17', '-8.837018838e-17', 37, 6);
    T('-8.8509105597001965830267107622017808778436872670450182e+61', '-9.1279989583108922742517159670230226130759094546e+87131886', '-9.1279989583108922742517159670230226130759094546e+87131886', 90, 6);
    T('5.671802410058928365899829471985191184822368320845e-231550558', '-7.779066943230730265262605907e+74', '-7.779066943230730265262605907e+74', 96, 5);
    T('-8.2820697798799263534e+8473', '-1.615106880260e-2541163', '-8.2820697798799263534e+8473', 62, 5);
    T('-5.265369842709e-52858907', '8.66866526767599296439e+452607605', '8.668665267675992964389999999999999999999999999999999999999999999999999999999999999999999999999999e+452607605', 97, 3);
    T('4.798110019e+86555309', '3.00070276764398837015486534372298810e-584175421711', '4.798110019e+86555309', 79, 6);
    T('4.51983627925542402657388789139552009722978e+4412', '-5.06376504e-7576', '4.51983627925542402657388789139552009722978e+4412', 60, 4);
    T('8.9684506690582247867098e+98', '-4.547886640653051100e-38424137218346', '8.968450669058224786709799999999e+98', 31, 3);
    T('-2.207977021011439186058299142747427987927761556902e+7282327', '9.39022996926700949148216115e+8587609', '9.39022996926700949148216114999999999999999999999999999999999999999999999999999999999999999999999999e+8587609', 99, 3);
    T('6.36708123958178477032768894220338378308555307205840495e+3933755281', '5.40093078183121636350228531656166e+9168098004810', '5.400930781831216363502285316561660000000000000000000000000000000000000000000000001e+9168098004810', 82, 2);
    T('7.411150e+935', '-6.65406975025177042864275243202e+8229633', '-6.6540697502517704286e+8229633', 20, 5);
    T('2.20282721756845225161572015502e-40203991129208', '4.172473065772518e+7342', '4.172473065772518e+7342', 66, 1);
    T('3.44333009466700195743391141734338680650442e+72734547935', '4.8669872941069768114116994102653735270328e-7017736715', '3.4433300946670019574339114173433868065e+72734547935', 39, 1);
    T('-8.336909e+4846892957015', '4.072305949757346620511337929596654e+34459667390', '-8.336909e+4846892957015', 43, 5);
    T('3.0191374343799780788551e+80892', '-5.2295357949409140774436205227855119486921268101e+952671340920', '-5.2295357949409e+952671340920', 14, 4);
    T('-2.79183187531373388987292019751568767353146074851127819472596e+109', '9.69400852080031006508637155341909e+36', '-2.79183188e+109', 9, 5);
    T('3.032999037226505006965178145618475309739974864836120746009e+63', '5.7139725243614632853e+5466645687260', '5.71397252436146328530000001e+5466645687260', 27, 0);
    T('-6.7027550322444931195170e-1240316055250', '7.0910815475809529561695937367e+91', '7.0910815475809529561695937367e+91', 55, 0);
    T('7.9469565434060992404733958229232112625499175460340273993e-867485459', '-4.07441960467e+20', '-4.0744196e+20', 8, 1);
    T('-8.46965121227138604013603285985497225888614236193562426e-136328965', '4.31371850956242022e+124906184', '4.313718509562420219999999999999999999999999999999999999999999999999999999999e+124906184', 76, 3);
    T('3.486329190925789348814788776638809588e+7633224', '-2.9020995823850995042759343020620166348835765286995525735790e+7672', '3.486329e+7633224', 7, 3);
    T('7.7418626624887969868900e+89171', '-2.02385905382743006197522802580425040283987e+59591436', '-2.02385905382743006197522802580425040283987e+59591436', 94, 4);
    T('1.1912913767348353502775244090691646904e+28318604', '-5.38464653387349076565769780e-84569', '1.1912913767348353502775244090691646904e+28318604', 54, 4);
    T('-8.52534678e-2690845986', '-2.9233263822866e-835232294560792', '-8.52534678e-2690845986', 46, 5);
    T('-1.17447155582155016241916074749687049925108724457883908916e+4139328243', '3.883845e-179', '-1.174471555821550162419160747496870499251087244578839089159999999999999999999999999999999e+4139328243', 88, 1);
    T('7.8e+7', '-6.31295018e+5122270', '-6.313e+5122270', 5, 0);
    T('-5.0953164624744645907224142e-52761986149069', '-1.3314979860393506e+6716', '-1.3314979860393506000000000000001e+6716', 32, 0);
    T('2.19652005375739683413006328065170371403388668975601e-698', '-8.2609099426138681678075992836666617e-4726086117110', '2.19652005375739683413006328065170371403388668975600999999999999999999999999999999999999999999e-698', 93, 1);
    T('8.98408745253474267533137438574399522592707e-3557481', '1.3072066166459969788916675523467415002849089168449973318657e-8200', '1.3072066166459969788916675523467415002849089168449973318657e-8200', 95, 6);
    T('-4.273245615523288934630644320013e+4240661', '3.3534352554338220260584063e+818928', '-4.273245615524e+4240661', 13, 0);
    T('1.244034612e+538707', '-6.280e-123217', '1.24403461e+538707', 9, 4);
    T('-3.85621596693115641697950926642193602000442638676447956e+255652455299', '-6.445698e-8638180', '-3.8562159669311564169795e+255652455299', 23, 4);
    T('9.24637243115885398184290158188662e+20521816062012', '-9.44e+28569156', '9.24637243115885398184290158188662e+20521816062012', 97, 5);
    T('5.625205825931650348088355929566296375808321698e-90472', '-3.79e-178329556371', '5.625205825931650348088355929566296375808321698e-90472', 86, 0);
    T('9.514967590985198798040570862750401927747012080898922e-402364402326', '-2.9956243453e-79', '-2.9956243453e-79', 34, 6);
    T('3.853968946351532202e+127', '-3.775804229929379152e-11', '3.853968946351532202e+127', 26, 5);
    T('-3.93e-639010732', '867445414719280132.795940537', '8.67445414719280132795940537e+17', 83, 0);
    T('-4.938152333190277169309346775416e+7951026641', '-8.79890083047653831115644125265943772633117566015e+378489724551303', '-8.798900830476538311156441252659437726331175660150000000000000000000000000000000000000000001e+378489724551303', 91, 3);
    T('-6.705346537233442946558414335270946169736586681360217219281e+9869', '8.4419053713808278236e+7706', '-6.705346537233442947e+9869', 19, 6);
    T('-4.22818600183503896410677173695946733590e+366869051', '-3.7557071735753014985930085602e+39965', '-4.22818600183503896410677173695946733590000000000000000000000001e+366869051', 63, 0);
    T('6.104082789443319067160166476507621201776239822525772e+68327337331491', '5.64039461118852898982408344364034173143656117840200277347e+134', '6.104e+68327337331491', 5, 1);
    T('-3.1270786086115e+9764750296742', '9.717e+663920', '-3.1270786086115e+9764750296742', 78, 5);
    T('-2.1e-48580', '3.35276976587565985201366605549923979278224887460690185695497e-25650168', '-2.1e-48580', 13, 6);
    T('8.59e-31736463', '4.676884870308573543266169262176064335e+44', '4.6768848703085735432661692621760643e+44', 35, 4);
    T('5.349929588690113215807463449256935229663e-8538843', '-9.57422196048334270450475643454483036095445184648538434865841e+110814', '-9.57422196048334270450475643454483036095445184648538434865841e+110814', 87, 5);
    T('-9.3511481510181367050679350607264664016389219588946450e+59', '2.39836623451751432739106359565459e+80', '2.39836623451751432738171244750357186329493e+80', 42, 6);
    T('-2.121861267288115978316437594696610e+871821555606860', '1.34981247338724022155661097e-44938', '-2.1218612672881159783164375946966099999999999999999999999999999999999999999999e+871821555606860', 77, 1);
    T('-9.47176277768977385410990892398096030532119229319249949315858e-9192979', '7.57884838832407948074e+25', '7.578848388324079480739999999999999999999999999999999999999999999999999999999999999999999999999999999e+25', 100, 1);
    T('1.67353273654068222699364945785467850740192732196905071242e-40607', '-8.389104901298499960674543894124e+9898256110964', '-8.389104901298499960674543894124e+9898256110964', 47, 4);
    T('-5.1512498572637051743356566552079845e-826571831252563', '-7.515676e+904915891', '-7.515676e+904915891', 27, 4);
    T('8.297970277487269939e+83261099', '6.3605100439308956941031e+161812', '8.29797027748727e+83261099', 15, 5);
    T('2.6608766e+1788390259946', '-1.8259454130910e-342', '2.6608766e+1788390259946', 60, 4);
    T('7.46664573989245727165418e+84134355', '8.3388908e-738976489', '7.46664573989245727165418e+84134355', 70, 4);
    T('-8.010380582519311556142743086270145873269e+686008860039548', '4.2907805214652521580590198e-34153938424', '-8.01038058251931155614274308627015e+686008860039548', 33, 0);
    T('-9.9517021297599135269791039458110392196559001445578e-36674', '-9.19530743052064563537308347798896923074369629082234e+9174300715', '-9.19530743052064563537308347798896923074369629082234e+9174300715', 59, 5);
    T('5.87439314540760028968866820553642e-57448', '-5.279038978201425e+73039', '-5.279038978201424999999999999999999999999999999e+73039', 46, 1);
    T('-1.2870616618194749949082169256260577831599917736546219479e+137', '818145261150460183672.26673142', '-1.287061661819474994909e+137', 22, 3);
    T('-4.2593019614312e+954', '7.125568336161172143040317264760443210893160341063600593e+398103850907', '7.1255683361611721430403172647604e+398103850907', 32, 6);
    T('-5.065429341801671662535295561174e+115', '3.9987292337499537193662400916550096163964848e-99234791', '-5.06542934180167166253529556117399e+115', 33, 1);
    T('7.0717901e+892067485021611', '1.2725122482e+2739330508487', '7.0717901e+892067485021611', 88, 5);
    T('5.948423748352517224571697594709552981384197e+385328574617353', '-3.041e+63990846661099', '5.9484237e+385328574617353', 8, 5);
    T('9.191022748276628750695821063901366802561087435878138025513e+86028', '-4.1031833875135676625236273877009042861877559698978756553e+67586459', '-4.103183387513567662523627387700904286187756e+67586459', 44, 4);
    T('-1.4913e-5187573', '-9.50454163354087391242626394385733805e+39', '-9.5e+39', 2, 6);
    T('9.16568911153923080833161570807933691707882028072e+9475131543819', '3.70424722586105652e-3039667057342', '9.165689111539230808331615708079336917078820280720000000000000000000000000000000001e+9475131543819', 82, 0);
    T('-1.914695021185527290425842489382447e-8084674812461', '8.055807177030694285803990155639082953864e-367156808427110', '-1.9146950211855272904258425e-8084674812461', 26, 4);
    T('-3.7489522836847491899705289796867120831206563113829121949495e-22087', '-7.0e-25205236343', '-3.7489522836847491899705289796867120831206563113829121949495e-22087', 97, 4);
    T('-8.95772571867667912062051533337763682e-584211053', '-5.69014349374971173791044693736126202439631342488481e-2052767711045', '-8.9577257186766791207e-584211053', 20, 3);
    T('3.014668093921353185411e-819897264195', '-7.19206159137389012266390e+1254603070', '-7.1920615913739e+1254603070', 14, 6);
    T('4.0044005938134e-53576', '-6.20075922234724353080346165891407590917785939381033350130632e-5984', '-6.200759222347243530803461658914075909177859393810333501306319999999999999999999999999999e-5984', 88, 1);
    T('-6.95291774085320074485272260155666345685837449e+255064090', '2.192679642869833033435664491e+97', '-6.952917740853200744852722601556663456858374489999999999999999999999999999999999999999999e+255064090', 88, 1);
    T('-9.2625974009204612274447824138292e-41', '5.274657057761929610170066860414617190783127131144835856024e-12236750833', '-9.2625974009204612274447824138292e-41', 44, 3);
    T('-2.50040435476338925468363225204324344699486e+28141', '-3.27192477447032805e+110', '-2.500404354763389254683632252043244e+28141', 34, 3);
    T('-1.473719965587021231230731001275916932e+4884', '1.06193321672541601784e-6871249512', '-1.473719965587021231230731001275916932e+4884', 89, 4);
    T('4.62968861506551721151e+8402048926', '8.095069363979796006690328e+6563', '4.62968861506e+8402048926', 12, 1);
    T('-9.2571922117856641881609016198045135991322879345782e-1365567180', '5.4963007075586734763534600832324174114948999553749374429515e+148516', '5.49630070755867347635346008e+148516', 27, 4);
    T('-5.726415606843057375199548744002756351587648478149402036e+816784106683302', '6.16563004924695388661166765293240813416752735619545e-7076', '-5.726415606843057375199548744002756351587648478149402036e+816784106683302', 89, 6);
    T('-2.32482583328669113952e-13369184', '-8.2494674678282086169908723121816753724537e-2021044277', '-2.32482583328669113952e-13369184', 24, 6);
    T('-0.075', '7.00968766471953898297681711081169808312127003542379855871e-5519982', '-7.5e-2', 53, 5);
    T('3.5625211100075372940024634117044408087204075436e+1264364', '-747434228576480.613', '3.5625211100075372940024634117044408087204075436e+1264364', 50, 0);
    T('-1.03225118241316428333916067864e+7784048099145', '3.957017926387268e-603580843423', '-1.032251182413164283339160678639999999999999999999999999999999999999999999999999999999999999999999e+7784048099145', 97, 2);
    T('-8.61328363e-2525', '7.086520644212541517889518775098530925245016569918212786507e-15508136620951', '-8.613283629999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e-2525', 100, 1);
    T('-8.6022604652433254847427483829763e+614505936536', '-7.39203500329677420087146214769358972600557857205e+544187', '-8.6022604652433254847427483829763000000000001e+614505936536', 44, 3);
    T('-1.017947099884352406139873712306484883522012e-1322735950', '2.4697221888e-76', '2.4697221888e-76', 94, 2);
    T('3.068221673529494999951658353770373986408520e-357986826', '-9.7500622476631960993588818e+297242083653', '-9e+297242083653', 1, 2);
    T('-6.2043358400544025883012470406803120463634540276e+53425374', '-3.156863901536126459113315728e+793', '-6.20433584006e+53425374', 12, 0);
    T('9.6634735931654707e+288', '1.784896779536073621973e-6727788', '9.6634735931654707e+288', 18, 3);
    T('53406909228.6128708', '1.7747969165509316831277531478928145257937591194493e+2849749102748', '1.774796916550931683127753147892814525793759119449300000000000000000000000000000000000000000001e+2849749102748', 94, 0);
    T('4.9936205563261878895907474834420694295485811e+576618970834', '-8.243055029640980799162030204971134470787981854993100027e-11342457', '4.99362055632618788959074748344206942954858109999999999999999e+576618970834', 60, 1);
    T('-4033097912105705258036044972489651350686019253771.373', '6.814420777495330380359083e-26778', '-4.033097912105705258036045e+48', 26, 3);
    T('-6.665549464288556551829654734e+74627117024079', '-5.350850056724946847345949e+120', '-6.665549464288556551829654734e+74627117024079', 78, 5);
    T('-7.45092222407758599257127592392728947247237739930247347837916e+275258', '5.85e-14417927', '-7.45092222407758599257127592392728947247237739930247347837916e+275258', 60, 4);
    T('1.59383673787361887189252503854295603204850193780290e-587962', '7.093615335063614304441e-790743009117', '1.59383673787361887189252503854295603205e-587962', 39, 6);
    T('-7381.7529', '-9.4186305644314442131110752386518001e-8154712010003', '-7.3817529e+3', 90, 5);
    T('-1.98975426630946314871173338660596e+78644581832178', '9.03203101306458176252789250641738e-7801', '-1.98975426631e+78644581832178', 13, 3);
    T('2.033870785555923709370039077340998042847081709960028840190e-422191849', '3.9568381108482483899753235735451724902592385445359381977510e-538213212751', '2.03387078555592370937003907734099804284708170996002884019e-422191849', 60, 1);
    T('9.96563242388869285145811252714310e-361879368', '8.639472e+555708396308', '8.639472e+555708396308', 97, 3);
    T('2.06591362078654023580178363949700833147313937e+647206', '4.4020162196069983266844165408539330e+17214', '2.065913620786540235801783639497008e+647206', 34, 6);
    T('-9.886760329589e-90336199464267', '9.3464122091029466924573615854004103876049128775749e-4687', '9.3464122091029466924573615854005e-4687', 32, 0);
    T('5.6651692241822390681e-94580305155', '-9.21833871657194729213998432584881142577e+361671978674740', '-9.218339e+361671978674740', 7, 4);
    T('-1.864448825189652003491229977290328862669214463124062e-23', '2.5735867462433834873494445674394050926270732101123239936483e+979027392617310', '2.5735867462433834873494e+979027392617310', 23, 5);
    T('-6.3822272737048871e+18', '1.2743510475384612206722177473709614998516032396684535471e-690', '-6.3822272737048870999999999999999999999999999999e+18', 47, 1);
    T('7.93138778528938209838790224280716686954e+5407183', '8.4939253944907053748349766746501193100809293438963e-75717639', '7.9313877852893820983879022428071668695400000000000000000000000000000001e+5407183', 71, 2);
    T('5.39081963628913639381704097331792502325581e+22578110', '5.89754e+1211244188723', '5.89754e+1211244188723', 92, 4);
    T('4.0479916528190468e+21', '3.8517962e-3554225075', '4.0479917e+21', 8, 4);
    T('-4.4327325726605024601e-714251762690', '4.8530873039e-7588470339932', '-4.4327325726605024600999999999999999999999999999999999999999999999999999e-714251762690', 71, 1);
    T('5.0236586370067770784e+813', '-6.9933224585196491838922728875761879715604029620982510262842e-1849', '5.0236586370067770784e+813', 94, 2);
    T('-8.91444654364227042517712800459645851000722914249855304574e-20091228295', '7.8079042969920841915629805858332480218e-45960835321938', '-8.914446543642270425177128004596458510007229142e-20091228295', 46, 5);
    T('-8.200460682192925897654893431552145182e+14946432336774', '-3.160257467609767914561497015185847805792959e+870365', '-8.200460682192925897654893431552145182e+14946432336774', 78, 4);
    T('6.287380472338956377941326650e-323373', '5.30106193756477476891339e-7260721', '6.2873804723389563779413266500000000000000000000000000000001e-323373', 59, 2);
    T('-9.1672196299082044342267124846201926256e-5167426843886', '6.202e+2373337', '6e+2373337', 1, 1);
    T('8.22079078591297337251947e-56490', '-6.672947750598565845083033870e+75711836642', '-6.67294775059856584508303386999999999999999e+75711836642', 42, 2);
    T('-2.4015254e+15', '1.12963068201392154499e-432', '-2.401525399999999999999999999999999999999999999999999999999999999999999999999e+15', 76, 1);
    T('8.962941e+697163173', '-8.770667886065876914283e+670788934129', '-8.7706678860658769142829999999999999999999999999999999999999999999999999999999999999999999999999e+670788934129', 95, 2);
    T('6.708093098328080e+92199575388', '-6.9110857820188776501e-2234985382774', '6.70809309832807999999999999999999999999999999e+92199575388', 45, 1);
    T('-5.76490681590288e-357414668', '5.399303824024343362575201679814728582565490e+90694825', '5.39930382402434336257520167981472858256549e+90694825', 45, 5);
    T('7.5088525642709618400278338953150157719e+2620632', '-3.885827699010034906800112e-224419', '7.5088525642709618400278338953150157718999999999999999999999999999999999999e+2620632', 74, 1);
    T('-9.44919458333523987323169904960263163240265037128405846005e+86748716383', '-8.34148444675760658147701915097597306652168e-386007902', '-9.44919e+86748716383', 6, 5);
    T('5.207474226744798199937385303287903137086920940516e-59148745589523', '-8.46492804e+16460357401284', '-8.46492804e+16460357401284', 24, 6);
    T('2.2817464989647591891937e-5743', '-4.59768081475053157585925892469179e-8492970415', '2.2817464989647591891937e-5743', 94, 4);
    T('-3.91265522038e-56186964899942', '-8.0400e-6532874093991', '-8.04000000000000000001e-6532874093991', 21, 3);
    T('8.7816645374352646525761697102404307375e-146067894205833', '8.0099669e+80', '8.0099669e+80', 12, 6);
    T('9.16355515272439417e-7197654', '-6.83127867295135309944702e-90700671', '9.16355515272439416999999999999999999999999999999e-7197654', 48, 3);
    T('6.283804957721566e-966368', '6.6117152932597183899312105748778980681319227e+69236', '6.6117152932597183899312105748778980681319227e+69236', 62, 1);
    T('8.994e-392986184', '-3.663229e+836820368554146', '-3.663229e+836820368554146', 16, 6);
    T('2.95560530018990081116e+875474', '5.047806017236729601307e+74867738', '5.047806017236729601307e+74867738', 89, 6);
    T('3.23770407903468362647921478178422526433274375882399739e-34358003', '-5.6164553688106653948e+97517032374', '-5.6164553688106653948e+97517032374', 31, 6);
    T('7.85944e-477996', '8.5898566134024301200790795166512e-7456', '8.58985661340243012007907951665e-7456', 30, 5);
    T('-3.10690792885178e-73039', '-3.793933846657719854615834662378339455091529375e-31397330668308', '-3.10690792885178e-73039', 65, 1);
    T('-2.43577986991978897895889862746440607929e+5831293160193', '2.24946457720155251330090059346566664901561e+29667', '-2.43577986991978898e+5831293160193', 18, 0);
    T('3.8649928120e+113165821526', '5.16313924173265156203539225e-973167796542', '3.86e+113165821526', 3, 3);
    T('-4.91229873127624742671210608991650095378e-7031289685269', '-1.5445646558572049e-556655725035', '-1.6e-556655725035', 2, 3);
    T('-1.994754718653893034626314196904752370590323840e+160337', '-8.05686100383029701210323850276658178832e+8998', '-1.99475471865389303462631419690475237059032384e+160337', 54, 5);
    T('-6.13853486555708210334626241003300761784763629e+7299', '8.6e-9829238222894', '-6.138534865557082103346262410033007617847636289999999999999999999999999999999999999999999999999999e+7299', 97, 1);
    T('-9.3315889156695940112271626896550909074862701e-548410516212', '8.23486e+489611117', '8.23486e+489611117', 64, 0);
    T('1.98595155e-8010068325126', '-4.28149199891551069804835271e+35485', '-4.28149199891551069804835271e+35485', 29, 3);
    T('-9.220186451751296230924598674777207340842383638e-423715940', '-5.82377840654597790916158675655291725219158065135673281775e+2090172757', '-5.823778406545977909161586756552917252191580651356732e+2090172757', 52, 2);
    T('-3.99767361621028154123852875e-2850779887', '-68442676802056990271181370.4068366419144113', '-6.844267680205699027118137040683664191441130000000000000000000000000000000000000000001e+25', 85, 3);
    T('9.7217921594571404117e+997412562804', '7.127183999e+75027', '9.7217921594571404117e+997412562804', 90, 1);
    T('9.805141362098016704370293577716026473018076046e+703490329', '3.77739370566537806149052e+5683', '9.805141362098016704370293577716026473018076046e+703490329', 67, 4);
    T('-8.381492480826401749223281455453242217206079229950291e+844303', '1.628495834292479391851859e+7968199', '1.628495834292479391851859e+7968199', 41, 5);
    T('2.623694670308770309944849355831337008830979e+80', '-2.54674032198353321085230668481306043144305010e+305', '-2.5467403219835332108523066848130604314430501e+305', 56, 4);
    T('2.60697849888e+962084', '4.63959792148e+101', '2.60697849888e+962084', 49, 5);
    T('-8.469183303319115950e-4326369736097', '3.594335529984731936274945768782670040679517183014108860146e-30586', '3.594335529984731936274945768782670040679517183014108860145999999999999999999999999999999999999999e-30586', 97, 1);
    T('-9.88156e-45183823689', '-3.448951257841723344566e+22', '-3.4489512578417233445e+22', 20, 1);
    T('8.873376615001391585432431144458351e+229', '-1.237611147704514e+18', '8.873376615001391585432431144458350999999999999999999999999999e+229', 61, 1);
    T('8.92410605238978755115876598871695356475831e+8254', '3.8603538359408788701e-68', '8.92410605238978755115876598871695356475831e+8254', 88, 4);
    T('-3.218514223984667e-945680814', '6.6953373033057175290075510112820974523e-147775', '6.6953373033057175290075510112820974523e-147775', 65, 4);
    T('6.76845463406e-7626596076689', '-5.336823806934602846651724064758e-20855520', '-5.336823806934602846651724064758e-20855520', 72, 5);
    T('-8.3491e+261499', '5.03008379034656559978324569351180441484380e+51357048067', '5.03008379034656559978324569351180441484379999999e+51357048067', 48, 1);
    T('-2.4245511312727193770885804795038e+840015931', '1.92599342910146e-58', '-2.424551131272719377088580479504e+840015931', 31, 6);
    T('3.812676717106404791806e-9174015957643', '-4.9853527696557183591169338917863330620447800e-6496775', '-4.98535276965571835911693389178633306204478e-6496775', 65, 3);
    T('3.516260874183e-419092', '-8.6219275731279540108406820678608884535602088338934244e+76635245', '-8.6219275731279540108e+76635245', 20, 2);
    T('-51556803543501945370', '1.4736750468826175803773715766833345892305840431e-160935117', '-5.155680354350194536999999e+19', 25, 2);
    T('-9.767210926177372928402173448981481158268967672643289248e+29477598872', '-3.55436587572986270810768128e-545578043777', '-9.8e+29477598872', 2, 3);
    T('4.4586674438842404643978333179805719e-7436109291394', '-1.54213330411e+608579374', '-1.5421333041099999999999999999999999999999999999999999999999999999999999999999999999999999999e+608579374', 92, 2);
    T('-3.01093698578083396e-496656167693', '-4.730989957540612460698646350385e+3118', '-4.73098995754e+3118', 13, 2);
    T('6.95312e+99080599', '3.65357130163061397038185304094878712188124804554825467718082e+786180', '6.95312e+99080599', 47, 5);
    T('-711854454599188098861939196351875021478488086793151', '1.748309460753454132960611384244940494569865058990047993e+244529852982090', '1.748309460753454132960611384244940494569865058990047993e+244529852982090', 80, 6);
    T('-1.51064031598712531126948472419188759188870808756837866406e+58', '-8.56532348e-961614280654037', '-1.5106403159871253112694847241918e+58', 32, 2);
    T('-762833706319010236861113166199735170538869.3627395440', '-1.786023605064218115497e+556521443', '-1.786023605064218115497e+556521443', 82, 6);
    T('-1.9891112271263214498265048745661e-39334292499', '-7.355e+9169166', '-7.355000000000000000000000000000000000000000000000000001e+9169166', 55, 0);
    T('-1.183757756403825623907126217690e-98966319439', '2.6097777204641414291854858209396474819e+685919985', '2.6097777204641414291854858209396474819e+685919985', 83, 2);
    T('-4.76273257090400145912433141503406425237775092403052583329e+26610732', '-1.589478537972015642285890368593827917539260215e+97479382727', '-1.5894785379720156422858903685938279175392e+97479382727', 41, 1);
    T('9.3673617988925e-912446093451618', '-6.70684423774037976406963978319824058347246787916e+49', '-6.7068442377403797640696397832e+49', 30, 0);
    T('-3.65985243494467019022496233055842e+226820', '4.9971138942562904294674733e+16089', '-3.65985243494467019022496233055842e+226820', 59, 3);
    T('-7.2716875294111757539216714789256581e+630636', '8.97763615922999915341458928266002569085e+89460834882', '8.97763615922999915341458928266002569085e+89460834882', 98, 0);
    T('-2.705300902318861660066684751e+285943', '6.52532114947978508575853277733506080918757938235366956143147e+4309618904670', '6.53e+4309618904670', 3, 4);
    T('-4.61043960075781509299705195348299577e+129', '6.1175089268075471431002740806877288515172733713570014000e+722', '6.11750892680754714310027408068772885151727337135700139999999999999999999999999e+722', 78, 1);
    T('-2.19519368837430080e+19126360721', '5.00924973608959737065e-9373417107048', '-2.1951936883743008e+19126360721', 19, 5);
    T('6.60e+98842', '8.99789343202409140071580169002847360374629365654914982081e+80838827575', '8.99789343202409140071580169002847360374629365654914982081e+80838827575', 66, 5);
    T('4.4558915086365e-24880', '6.1232026377148379473079642668488073167622702604829964e-5302491512248', '4.4558915086365e-24880', 87, 5);
    T('-7.88254639684933609362025995299004500443587349399053531699608e-84159494233000', '7595474586641137.9098793', '7.5954745866411379098793e+15', 59, 6);
    T('5.22169628023861180096932313458983170195849e-442', '0.000017921242870987', '1.7921242870987e-5', 81, 5);
    T('-3.92440279226486997853503156205749924017368671190229267844807e+3362947799', '8.705687157104277939407013306e-4394697', '-3.9244027922649e+3362947799', 14, 4);
    T('3.952451058697052094591641951477792871e+108', '9.061531267393444862101567451e+8812469', '9.061531267393444862101567451e+8812469', 76, 6);
    T('3.542202893091301451849042420831911191626736730e+910', '-9.795785871235504552529670654851e-52230434517', '3.542202893091301e+910', 16, 6);
    T('-6.9738101737379174490747598675333975993575306232e-261', '-4.77638988134501196543e+80', '-4.77638988134501196543e+80', 60, 4);
    T('2.73198511808173147844127950898567547849885944331902e-4063866948148', '1.95953851394460439e+3638', '1.95953851394460439e+3638', 80, 3);
    T('7.97146343073693163048349377532779953546740042e-53', '5.87089936105981469357963564553353378e+764028574300', '5.87089936105981469357963564553353378e+764028574300', 96, 4);
    T('2.1105428071896949e-644', '6.1831587090153727669532808958927722803416e-46', '6.1831587090153727669532808958927722803416e-46', 62, 5);
    T('-2.41e-983', '7.72553532527918030757850510415659191934463973804776687e+77573', '7.72553532527918030757850510415659191934463973804776687e+77573', 81, 0);
    T('-1.411876001974250157695756850288555081336283501830578e+67', '8.9867829203634795954851721826377821643712e+2224548979', '8.98678292036348e+2224548979', 16, 6);
    T('-41224882759774769706609.90404', '8.275261999056050844550935086658021147531354170e+1366801', '8.275261999056050844550935086658021147531354169999999999e+1366801', 55, 1);
    T('1.9516197835380820808280622643025907106314570830285250055574e-74543', '-0.679993974', '-6.7999397399999999999999999999999999999999999999999999999e-1', 56, 2);
    T('6.28677519149391899669300704747234629319e+109232191986', '-2e+7437', '6.28677519149391899669300704747234629319e+109232191986', 45, 4);
    T('9.448674693749771721024e+3967558800133', '4.53446e+7971222', '9.448674693749771721024e+3967558800133', 64, 3);
    T('-8.9994098732394751125371385271230213e+7117', '-6.039974800723721763734498476352985023198870e-22538', '-8.9994098732394751125371385271230213000000000000000000000000000000000000000000000000001e+7117', 86, 3);
    T('-4.50616290555989807669467212060959293641483923e-5527490674496', '8.7219647503006472269862924128771972382379e+41572520142', '8.7219647503006e+41572520142', 14, 4);
    T('4.819826272459137116963201115634e-52', '5.345337551900596332023117682421324952e+7196055937', '5.345337551900596332023117682421324952e+7196055937', 91, 5);
    T('-5.28668044245553267320247389468347329984817119485847014e+4708900483977', '-2.4490595e+67429', '-5.2866804424555326732024738946834732998481711948584701e+4708900483977', 53, 4);
    T('-4.2217472707762006e-699', '5.57905931208144653792793986116588e-4541421', '-4.2217472707762006e-699', 59, 0);
    T('9.25025418819386635962650162363471077035414962820977648624e+85474912842', '-8.248559272551245851380684953342777062025634237e-5826', '9.2502541881938663596265016236347e+85474912842', 32, 5);
    T('6e-733361540147003', '-2.96138386093756057022624908944034248105670682544039777817e+8426788504', '-2.96138386093756057022624908944034248105670682544039777817e+8426788504', 91, 0);
    T('9.5903381583754611270234517060636965995108627449434520701867e-100358762039', '9776288732867570137161', '9.7762887328675701371610000000000000000000000000000000000000000000000001e+21', 71, 0);
    T('-4.74437552558705858990822354596956309e-7717803517', '3.71321732695095112564224876659607100215770815e-5156', '3.7132174e-5156', 8, 0);
    T('-4.61175117383705849405295615759487399151337740067990e+44624924411530', '-8.912447079463317476377700773406151228666517933650e+35603', '-4.6117511738370584940529561575948739915133774006799e+44624924411530', 59, 6);
    T('-7.9467309165394740150286555957410e-518900557202244', '7.34757602293882794200895083466082017371376329659857152790856e-106677919151', '7.34757602293882794200895083466082017371376329659857152790856e-106677919151', 82, 2);
    T('-2.5687e-54182934', '7.9912925658468e-33916077', '7.9912925658468e-33916077', 47, 4);
    T('-1.891978400176696446521940922190123610356117009444405e+473', '-6.13612264666016143077978508378e-5082023', '-1.892e+473', 5, 6);
    T('5.6899334224739113995620258500246277359901049519e-779822554296819', '3.06883507868e-36804181721122', '3.06883507868000000000000000000000000000000000000000000000000000000000000000000000000000000000001e-36804181721122', 96, 0);
    T('-9.3974825002e-7347', '-5260104.7', '-5.26010470000001e+6', 15, 0);
    T('-5.4761302349564880148964309745918e+34', '-3.58453044684907964127807327186870476381769e+346', '-3.58453044684907964127807327186870476381769e+346', 49, 2);
    T('-8.11039519734205611047945e-15943244046927', '-6.350706965715345327731800198527652908745904787096306e-160088384', '-6.350706965715345327731800198527652908745904787096306e-160088384', 66, 1);
    T('7.917383831910423678524757011962574153822397290158024452e+390', '-7.3893675520491571363417e-151681197', '7.917383831910423678524757011962574153822e+390', 40, 4);
    T('-1.340192e-688', '7.4619837e-9043296', '-1.340192e-688', 99, 4);
    T('2.7813908561209024509858452017624617455277677471702e-859844049274', '6.910726846153183375408823954661127686739765359962986e-35073360291', '6.91072684615318337540882395466112768673977e-35073360291', 42, 2);
    T('-9.0399160362909212454e+30170779306867', '3.83024084732214252e+714718656815', '-9.0399160362909212454e+30170779306867', 72, 4);
    T('-5.9261893206776530438053072427519686541424839932840604680962e-39516092749624', '-9.66553410907350054e+19', '-9.66553410907350054e+19', 52, 5);
    T('4.98949763995313017014289295435112536892222918460657711e+864350', '6.7237146079953e+22', '4.98949763995313017e+864350', 18, 4);
    T('8.761172867576302849251856051014129921200177e+18573876458089', '-8.89259e-12969', '8.761172867576302849251856051014129921200176999999999999999999999999999999999999999999999e+18573876458089', 88, 3);
    T('-4.899601277702003086442471e+8515982', '9.4551956000297555314494356788971930303978e-2695740012', '-5e+8515982', 1, 0);
    T('-1.7444217711962735577544657042996e+63980209868472', '170771546835404268944881703150271.92241', '-1.7e+63980209868472', 2, 5);
    T('4.4515636149655282e+90797', '9.55015665574824594926107577205242216745828499499182639212e-5575711341215', '4.451563614965528200000000000000000000000000000000000000001e+90797', 58, 0);
    T('35447028579332643019.1254170', '-3.9771593685733149463118698188688630469e+8372', '-3.97715936857331494631186981886886304689999999999999999999999999999999999999999999999999999999999e+8372', 96, 1);
    T('2.11062599353136735695374e-285755388', '9.3534786769223e-837258861327211', '2.11062599353136735695374e-285755388', 37, 6);
    T('-3.26970534720519869075174254969686917494280146513272155e+58', '-1.7555425166723e+403370190683855', '-1.7555425166723e+403370190683855', 53, 2);
    T('-3.431219e+80782581017843', '8.7687946610896317644578929469520920726324067878007871328e-1108112148253', '-3.431219e+80782581017843', 55, 3);
    T('1.467111318828015030106605593796808e+9446879129041', '7.100652923904837918920795641052890235835205470717161946e-69420843', '1.4671113e+9446879129041', 8, 5);
    T('-8.87302295514068529905810707756525739169251319049593028669e+452698622831', '-4.501194133314542973431541127944069097688875319e-66359', '-8.87302295514068529905810707756525739169251319049593028669e+452698622831', 73, 1);
    T('-6.62406751472869987659609754062526788876598041587971316e+6106024', '-6.7e-146', '-6.62406751472869987659609754062526788876598041587971316e+6106024', 90, 5);
    T('4.758767e-31515122549', '-3.284452520263670019608444178746125619030879215e+52586197858', '-3.2844525202636700196084441787461e+52586197858', 32, 2);
    T('-1.90737e+60', '2.200886667902695056209974986849231e+843621516734', '2.200886667902695056209974986849231e+843621516734', 48, 2);
    T('-7.562413432e-75974383067', '-8.03338828854782417451853482397448438803093209394961341682e+8798452004', '-8.03338828854782417e+8798452004', 18, 4);
    T('8.00838943803837142311409191849442123306821743441745520863914e+309147372', '8.1995337577342452575186605329912235130280120239427778164086e-5955', '8.008389439e+309147372', 10, 0);
    T('-2.6277e-6024333', '-2e-72382', '-2e-72382', 8, 4);
    T('-7.901814802406745289199429988242333742413436937658355753092e+956491251799038', '-3.465560488388782846515757644499773628758810e-3934647790', '-7.9018148024067452891994299882423337424134369376583557530920000000000000000000000000001e+956491251799038', 86, 0);
    T('-7.6e-98007338926382', '9.48e-69745', '9.479999999999999999999999999999999999999999999999999999999999999999999999999999999999999e-69745', 88, 3);
    T('1.37427159156377843e+493786721', '-4.63644871593105459143520205e+96643025833547', '-4.63644871593105459143520205e+96643025833547', 64, 6);
    T('-2.65432039012491907e-623199698265', '-3.1468346525236268943962621568e+34731860', '-3.1468346525236268943962621568e+34731860', 36, 1);
    T('-1.5517704393998744520888741487822432585e+923168393', '-6.5e+9', '-1.5517704393998744521e+923168393', 20, 4);
    T('-1.52106679076753745289141635324139809196478388591e-188', '-1.053541219813634e+7449', '-1.053541219813634000000000000000001e+7449', 34, 3);
    T('3.54393912593097395795636912718e+190643824918', '-7.67586755977660486632106352e-45832794', '3.54393912593097395795636912718e+190643824918', 39, 0);
    T('-3.33360e-4961506633', '-6.060006095e+15', '-6.060006095e+15', 94, 4);
    T('-9.07163850400906883249690496384420437798705e+16482', '-5.31556656162267243085384796568754122e+7569', '-9.071638504009068832496904963844204377987050000000000000000000000000000000000000000000000000001e+16482', 94, 3);
    T('-6.247606085614196824064623647209093394762201545305e-316469499479958', '-6.549e+28244600243015', '-6.549000000000000000000000000000000000000000000000000000001e+28244600243015', 58, 3);
    T('1.53271939714776970533791609586244434616e-153982', '4.263159440610779710677706641563847435894489228e+3145867823844', '4.263159440610779710677706641563847435894489228e+3145867823844', 69, 1);
    T('-2.0828627023701123645208842878571623790870675130937633864e+924835663', '-5.853970061786635875e+39730', '-2.0828627023701123645208842878571623790870675130937633864e+924835663', 64, 1);
    T('4.9278015139290996442768379991647470223e+59799439850', '8.46475818067078506e-485898', '4.9278015139291e+59799439850', 16, 4);
    T('6.26256234654710710131778686661219992897450241698740835756e+93353877788337', '9.2482842999640064496401080124350627113e-566903245360', '6.26256234654710710131778686661219992897450241698740835756e+93353877788337', 57, 1);
    T('4.40189292495829598065338336017266388233234246262657351e+62', '-5.2455264288926341098934788231063882728138957870e-448037', '4.40189292495829598065338336017266388233234246262657351e+62', 64, 5);
    T('5.096140051938701819453023616e+737', '2.08352950374331356796792957425376258237561610e+16498', '2.083529503743313568e+16498', 20, 2);
    T('-6.920470641743535367957036406016822312e+62766', '5.64731749320187847582815e+30028207815653', '5.64731749320187847582815e+30028207815653', 93, 5);
    T('-9.52783318989711838519e-29862775522664', '-2.084754031108290441518919723327869336871219410264119e+31879838388', '-2.0847540311082904415189197233278693369e+31879838388', 38, 4);
    T('-1.3168833810130395429020612309397759574084525e-1221327', '-2.56005778629e+275000143', '-2.560057786290000000000000000000000000000000000000000000000000000000000000000000000000000001e+275000143', 91, 3);
    T('-3.9104797755187301e+19960164203993', '7.967e+3018756781', '-3.910479775518730099999999999999999999999999999999999999999999e+19960164203993', 61, 2);
    T('-5.37909437e+439110', '-6.6132322148787e+29', '-5.37909437e+439110', 53, 6);
    T('3.67149764713886497312304171924285955357726224915410474076200e+42866989330', '-3.3784e-701260', '3.67149764713886497312304171924285956e+42866989330', 36, 0);
    T('-9.5718712901880257014603579391668e-929975', '-2.665690871940846345182141172e+31', '-2.665690871940846345182141172e+31', 74, 4);
    T('4.199512875607415993288e+216142', '9.38e-944878502446223', '4.19951287560741599328800000000000000000000000000000000000000000000000000000000000000001e+216142', 87, 0);
    T('-6.060012673543391237553716e+4598982', '-4.6904214431428349621528706608004539e-88294465870', '-6.060012673543391237553716e+4598982', 26, 1);
    T('2.88538164967978436e+725392', '-3.8659155433322438095313688540634954629483535e-2892397124018', '2.88538164967978436e+725392', 60, 5);
    T('-8.273e+75789', '-9.1680860324444261299e+61', '-8.273e+75789', 35, 1);
    T('-2.5958481385668145470426000782759248003464e+43965916426023', '-8.679401114855728965835210480186e+743431277190', '-2.5958481e+43965916426023', 8, 2);
    T('3.59951154156128916e-36441565530468', '6.42363106e-7208915885009', '6.423631060000000000000000000000000000000000000000000000000001e-7208915885009', 61, 2);
    T('-3.61841072437837818212e-10392', '3.2220897801067820399874053600149e+5922331788687', '3.2220897801067820399874053600149e+5922331788687', 69, 2);
    T('-6.093441939776111937105185525906079174392680002093043e+58682953185', '4.2130252271257207567123954394999800697639159468521129e-86117', '-6e+58682953185', 1, 5);
    T('-6.251980913820e+80057091', '1.2364323901908395173318470252939671574160321952054957e+7938', '-6.25198091382e+80057091', 85, 0);
    T('-4.4758711343614555e+66068203773903', '-7.70197254887865617807195403934922486304830949910121529100e-228444814987478', '-4.4758711343614555e+66068203773903', 31, 4);
    T('1.0080506016903879960877676324640373025705186e+447290726', '7.554638359261755531694031886436e+59308293', '1.0080506016903879960877676324640373025705186e+447290726', 73, 6);
    T('-4.9787393768426452694587028126e+20759', '1.14958903725828813275180524e+8399045761016', '1.14958903725828813275180524e+8399045761016', 43, 6);
    T('6.43773959361e+86849', '9.9625795149163229195948268e+29506773735', '9.9625795149163229195948268e+29506773735', 92, 4);
    T('-9.2828168196673805283807e-55831', '-6.78653680389499900150028803607797e-865141493297829', '-9.28281681966738052838070000000000001e-55831', 36, 3);
    T('-1.26665818710330906954643942917104271796325416574851416772e+26385061693', '-7.00253881563126353262070e+524', '-1.2666581871033090695464394291710427179632542e+26385061693', 44, 5);
    T('370806924178899851218491983315770594616718.40', '-5.02488669158034720535985520516274745e+700256427339', '-5.0248866915803472053598e+700256427339', 23, 2);
    T('4.53541e-83', '9.8285978635809060623649424449763520518089587221689681e+173492797', '9.82859786e+173492797', 9, 6);
    T('-2.0375937064063076698792342939e-2269098', '6.16432940276546777580519e-79456030266', '-2.04e-2269098', 3, 3);
    T('-3.67761381529517168400693215671528832513400579203561e+113', '2.439737570209440469315634685147444e-53203079134301', '-3.67761381529517168400693215671528832513400579203561e+113', 52, 5);
    T('-5.484595639e-5080', '-6.81003825634810752e+554914597', '-6.810038256348107520000000000000000001e+554914597', 37, 0);
    T('5.62558995215751655979073943e-1135616514', '6.26718905566513522761370941e+5260992', '6.267189056e+5260992', 10, 6);
    T('3.0803334722529703871576147410545619971764e-578232', '7.6e+692', '7.6e+692', 60, 5);
    T('4.4373956358024790033e-24742', '-2.79645195044246164832890141024713869777571471e-241483655', '4.4373956358024790033e-24742', 55, 2);
    T('-6.787417819803023089763294889430e-934657436', '-2.10317529497435889814309411132663e-24846', '-2.10317529497435889814309411132663e-24846', 91, 5);
    T('4.37267e-207761339798', '-5.65692657484266982052971963e+2169664303', '-5.7e+2169664303', 2, 0);
    T('3.2487779555959e+6672658', '6.83300378960117931960955074978817903255453024856616287e+252379721', '6.83300378960117931960955075e+252379721', 27, 6);
    T('540162613724659809285683621881807058147557734915883390057976', '-5.9235928e-13', '5.4016261372465980928568362188180705814755773491e+59', 47, 3);
    T('3.8252324068130e+907752135', '-5.186342365958664e-6977576710068', '3.825232406813e+907752135', 29, 0);
    T('-8.35404455917993003e+658105', '-5.5380952714034493621696435438967975e+3389', '-8.35404455917993003000000000000000000000000001e+658105', 45, 0);
    T('5.6353399477598376827560290730e-643', '-9.405789845154762173534631184344071689817549e+114', '-9.405789845154762173534631184344071689817549e+114', 62, 4);
    T('-4.6385781781629479117192014098302438489819670456465e-437448480016', '-9.4932021052e-580422785200980', '-4.6385781781629479117192014098302438489819670456465e-437448480016', 87, 2);
    T('-2.12361491529637751787605628242616998109682e+137', '6.6270659269889741322577929520212727033308475752162289490251e-20430619', '-2.12361491529637751787605628242616998109682e+137', 58, 0);
    T('7.32455775984672e+817', '9.9824078652014984717e+3579589798294', '9.9824078652015e+3579589798294', 14, 4);
    T('-2.5588820865972141268570469656229162356414196452615384590447e-9648314', '-6.660855997566035713826e-192852573011', '-2.558882086597214126857e-9648314', 22, 5);
    T('-9.85202e+726325883', '-7.514798298733820212e-694740216035', '-9.85e+726325883', 3, 1);
    T('-6.970844404792041216416879645775097549362198810e-44651237571', '4.02365281247150e+62805424539', '4.0236528124715e+62805424539', 89, 4);
    T('8.527084465220901521751744415381602405136912675882121739e+636462', '-3.75994582589228845194731389797e-59', '8.527084465220901521751744415381602405137e+636462', 41, 2);
    T('-82741109694293364.54079082', '-8.7512340993118280329279724624507491030e-571', '-8.2741109694293364540790820000000000000000000001e+16', 47, 3);
    T('7.614810248e-9904372', '-7.907213967e-884915922604', '7.614810248e-9904372', 10, 5);
    T('8.0416733327609846278e-58456564755', '6.51403164465101879599843461286003324943081657101e+55697083037961', '6.51403164465101879599843461286003324943081657101e+55697083037961', 55, 3);
    T('-9.001e-669', '2.76851003060212e+19', '2.768510030602119999999999999999999999999999999999999999999999999999999999e+19', 73, 3);
    T('8.79186343692560767034083319e+5307471336', '-1.10102278862731964323449653047891301209e-773', '8.7918634369256076703408e+5307471336', 23, 4);
    T('5.9830236797456435722249615035541685080424628166756839473e+4357855', '9.2074522869023641819982880232980680816540924928177234047798e-440860295144', '5.9830236797456435722249615035541685080424628166756839473e+4357855', 94, 5);
    T('2.509771410064031633258122783041979623e-29834379313', '-5.397653142277995025148762504340564913583463710964091286141e+2961212123', '-5.4e+2961212123', 2, 6);
    T('-6.227002e+90039', '-9.889210272290533302749e-4875', '-6.227002e+90039', 50, 5);
    T('-2.19836331940825400822465e-2526', '-2.15097519700560043e+66150845', '-2.15097519700560043e+66150845', 97, 5);
    T('5.8980254374414000752425570320649959e+36125', '-6.024146e+13', '5.8980254374e+36125', 11, 5);
    T('1.53393339168542905509043790835051815854563882539568832794197e-71525', '-7.6383178717e-54927', '-7.6383178717e-54927', 66, 6);
    T('4.70429055580665335294324657029711699e-909775678', '7.39442584230606644649e+996146984075119', '7.39442584230606644649e+996146984075119', 36, 6);
    T('4.6002161746647801766062937487143444068808292e-4448', '6.12162e+23', '6.122e+23', 4, 5);
    T('-4.62565203673e-94155', '-1.250836180022446477394900409000896654098813634554272e+55322863', '-1.250836180022e+55322863', 13, 2);
    T('-1.08503726395180179921445751793463568390461332657471857e+72747', '6.659209268616881378102256731122569070589941425070773e-626296', '-1.085037263951801799214457517934635683904614e+72747', 43, 3);
    T('4.08055017182496863350986e+128', '6.5406859892431305984528357072278289543e-56278', '4.0805501718249686e+128', 17, 5);
    T('1.08583107260017094e-2899', '7.284823e-885993370119', '1.08583107260017094e-2899', 93, 3);
    T('-4.01613022712298e+2718', '2.2321250657600619746759369846100678937930795188084409137890e+47739387333', '2.232125065760061974675936984610067893793079518808440913789e+47739387333', 66, 0);
    T('-37582350.96157351', '-6.4652839624654113293331e-531169075647743', '-3.758235096157351000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+7', 100, 3);
    T('4.83043283814815484115778496866750009972167443e+4746876179149', '2.84620693e-47', '4.83043283814815484115778496866750009972167443e+4746876179149', 45, 3);
    T('-7.4698125747820773488311729970504069582414020e-516640294664049', '1.31473152993161960185e+842887', '1.31473152993161960185e+842887', 72, 4);
    T('-4.879088686091751063970687193731339481554292e+832108', '6.713950378019641349081764e-944938575190654', '-4.879e+832108', 4, 5);
    T('-5.984739924787790215151323988404920730465634637095898108e-600', '2.251674133351118250794181193776725821e+37', '2.25167413335111825079418119377672582099999999999999999999999999999999999999999999999999999e+37', 90, 3);
    T('-2997120683726032.18151804795617720', '6.7881554107924389608586972940395466315e+134', '6.788155410792439e+134', 17, 2);
    T('-9.16728280881188770264364619e+706952430', '-4.759730961721174195709314954296788683055093274e+297568074419675', '-4.8e+297568074419675', 2, 3);
    T('-4.78005374271499407e+7963', '7.684107255381286094346154148054374925088694655510778203692e+133', '-4.78005374271499407e+7963', 43, 5);
    T('-9.410566595189358529794815720246690337996e-2992978301978', '-1.12698537490897525e+635134355', '-1.12698537490897525e+635134355', 28, 2);
    T('1.70971735955989620243943493651791764995526111912430031e+896380680', '-6.939180802212600299088423679005165884402393e-76991053', '1.70971735955989620243943493651791764e+896380680', 36, 3);
    T('-5.7235550871810066648189689359587899349384071642000694e-679362', '3.40726243962994996965191889421692e-83225403683', '-5.7235550871810066648189689359587899349384071642000694e-679362', 99, 6);
    T('1.922647239536730629054001490805945187179462008480e+93143972854341', '-2.475550987141855596530764801917e+50927', '1.92264723953673062905400149080594518717946200848e+93143972854341', 58, 0);
    T('5.50993095133201050391250575999563466e+94362873069973', '-1.8086606141201731966711674835074082195428e-355414812003', '5.50993095133201050391250575999563465999999999999999999999999999999999999999999999e+94362873069973', 81, 3);
    T('-7.072118174763684395874050065633235161852974125105902632706e-6592', '9.16504081356895552547900509663788160609256381958015e-27734719599645', '-7.07211817476368439587405006563323516185297412510590263270599999999999999999999999999999999999e-6592', 93, 2);
    T('-3.07138820668327600199960718030e+96', '-3.7721760609e+72145381149', '-3.7721760609e+72145381149', 53, 2);
    T('7.435544037851039705554676104669433e+103626112', '-3.169846e-599097', '8e+103626112', 1, 0);
    T('-3.24203486144415876452838946810404523e-4546100871707', '-2.59043741644440e+36134397', '-2.5904374164444e+36134397', 96, 5);
    T('-6.47778170e-58313267262550', '7.5e-8351037048191', '7.5e-8351037048191', 59, 5);
    T('-946065657583750032189792633.2834301', '6.6908542665027614299688674566134269480189837621502e+29553', '6.6908542665027614299e+29553', 20, 3);
    T('2.92e+6651160247863', '-3.1588991993033329661028942877e-292391863', '2.92e+6651160247863', 73, 4);
    T('3.93343087e+9584', '-5.6171905401990356733113628313639486192576156459e+3011220687', '-5.617190540199035673311362831363948619257616e+3011220687', 43, 3);
    T('8.238885627698e+79798965290880', '1.151742946063529239212342803805876963490892158246e-661088939', '8.238885627698e+79798965290880', 46, 5);
    T('-4.59614902321398581090804499467849932978042e-307951927961', '-5.06755330431437420546142055552504e+9190003536', '-5.06755330431437420546142055552504e+9190003536', 33, 2);
    T('-4.89406975793113202278034070e+396547884409648', '4.327248144759733571219048624527e+5387097038086', '-4.8940697579311320227803407e+396547884409648', 56, 3);
    T('-3.2002911454897707073770168038988748712892823338e-562447170', '4.573308155385e-3334095', '4.573308155385e-3334095', 54, 2);
    T('-5.684049102168415623997899930983637e-876045752687180', '8.8411660471519566996623899246520312770057171524877937e+2569353936486', '8.841166047151956699662389924652031277005717152487793e+2569353936486', 52, 3);
    T('4.432823708e-219022157674', '-7.5823228998097595540973560239168827297868e+15052812983242', '-7.5823228998097595540973560239168827297868e+15052812983242', 84, 3);
    T('6.685526093326705158081409375226362922709507e+72873', '2.172772130146075666382035545271e-65', '6.6855261e+72873', 8, 5);
    T('5.547349673659007742139305358139679188e-2639229037', '-5.439847823925638821016885240675e-8', '-5.439847823925638821016885240675e-8', 38, 3);
    T('-2.660769170e+917429', '7.3774113779067850779074869869141437049885811296328371101e+503403798679', '7.377411377906785077907487e+503403798679', 25, 6);
    T('3.627197784395356e-70300108', '-8.9e+29032755380266', '-8.9e+29032755380266', 39, 5);
    T('7.40498174760132791500415359468038880256834365e-722776283203', '-8.9285364853471785973610958742689707209312009372e+819', '-8.9285364853471785973610958742689707209312009372e+819', 93, 4);
    T('5.47135370079895673647665521745959321971173738329e+44298', '4.2254510777653935827973997501671616266166e+15038', '5.4714e+44298', 5, 4);
    T('-5.6173880490e-81531200', '8.26619003246228844153093009915213795153001285448858596029389e-266359643', '-5.617388049e-81531200', 12, 0);
    T('2.768662236674727867765e+98480087', '-6.9670016541e+242061657', '-6.9670016541e+242061657', 61, 4);
    T('3.79713049282330585003204725474207860988770618e-6936472745068', '-7.52090e-22637', '-7.5209e-22637', 34, 6);
    T('8.2476688390134612946600645829769040500898010e-475', '3.8303522839211624987066922843308444935353261e+5938360791895', '3.83035228392116249870669228433e+5938360791895', 30, 6);
    T('-4.4129856265747945864460159026662497e-63702350817', '-6.789870881673515464968983557858762668400213e-3300809283724', '-4.4129856265747945864460159026662497e-63702350817', 66, 5);
    T('1.228e+71323194019332', '9.1073839462636535554359556755468867678244909541638406503126e+6382596751', '1.228e+71323194019332', 8, 6);
    T('-6.5817936700809605626026248136e+6950972092271', '-8.53138812736418189998088428e-6278417993472', '-6.5817936700809605627e+6950972092271', 20, 0);
    T('-1.1401713404170088999328845924512954074403347946765e-9369188', '-1.0720384e+28561', '-1.0720384e+28561', 47, 1);
    T('-1.5832e-5114911', '1.5748762551e-114249777405', '-1.5832e-5114911', 58, 6);
    T('5.77351773474477e+19', '-3.9733530544286099108224e-46211221', '5.77351773474477e+19', 96, 6);
    T('9.34175651781102854042895264348978959e+642384185', '-3.51511266164442399458854909073139e+53730277', '9.34175651781102854042895264348978959e+642384185', 69, 5);
    T('-78801778105669405.8456045188', '-2.090182160241510339878996689e-6185931136781', '-7.88017781056694058456045188e+16', 82, 5);
    T('4.97759321238910088e-64855878947', '-6.7943863341092871758820551812621734095e+463339625339', '-6.794386334109287e+463339625339', 16, 2);
    T('6.0005540238806390981164043652442284001366e+3137030837', '-2.394020541672210610503120113171e-12', '6.0005540238e+3137030837', 11, 3);
    T('-6.6220942378591e-19265', '8.50899015154072945726070921282588879044110108091385158818279e-7368230117011', '-6.6220942378591e-19265', 43, 5);
    T('-2.9375980429818666914390731635722355770036887464909886e+34737', '-8.21500053344707226699639354050954737e+6978', '-2.9375980429818666914391e+34737', 23, 4);
    T('-2.2681526537639358580811891091556142620793421273689612e-632043', '-3.89214282362784813257212684503487578760e+548806393973244', '-3.8921428236278481325721268450348757876e+548806393973244', 99, 6);
    T('1.93322681492286495004944504406074557795e-9499', '2.140069375274766915466095049260086e-8998060959', '1.93322681492287e-9499', 15, 2);
    T('-5.4517609691436568102548935318995960215508e-4970807560706', '78650680569427482189620618683734235538794085638975649209', '7.86506805694274821896206187e+55', 27, 5);
    T('2.4103956493475918129724697028452321039324614564944e+3402501378531', '-8.41169268347882476104942735069752e-27', '2.4103956493475918129724697028452321039324614564944e+3402501378531', 57, 0);
    T('-4.3387245488783002577117106319937685389080524763020074110e+990752402889', '-6387637583538554010001854634535121677725720902.09', '-4.338724548878300257711710631993768538908052476302007411e+990752402889', 66, 4);
    T('9.8209206624319496983162277456419362e+35', '-3.49456135014272393421431907391732e+459716999173279', '-3.49456135014272393421431907391732e+459716999173279', 92, 0);
    T('-2.58328360249e+8141525', '-2.61580990795549879195328442e+66845', '-2.58328360249000000000000000000000000000000000000000000000000000001e+8141525', 66, 0);
    T('-2.8902375883759236750151734834512826955346530393153949177e-5919020', '7.7426083433831188459245946186852340509102915025e+4285080104', '7.7426083433831188459245946186852340509102915e+4285080104', 45, 1);
    T('5.0001414646236404927389548976072517345898416355436339e-3222570232', '9.3932041759559728228411621153203816e+35', '9.3932041759559728228411621153203816e+35', 99, 1);
    T('4.973712272734656107585032145274933139e-89217', '1.92416009296379960064790197531851e+4419424736152', '1.92416009296379960064790197531851e+4419424736152', 67, 3);
    T('2.994539e+10', '5.986183172702700128125305e+645', '5.986183172702700128125305e+645', 56, 4);
    T('8209104727456967560636030157909930038903683115881429199217.4', '-9.163471161314618114e+111', '-9.163471161314618113999999999999999999999999999999999991790895272543e+111', 67, 5);
    T('6.61019200846011663787913671264858970432900042314e+49', '-3.32874393e+29002584510', '-3.328743929999999999999999999999999999999e+29002584510', 40, 1);
    T('8.288108057904672099747716063361664e-588465002479', '5.6881646539887009991311825231007716901438009137818783e-63405759312743', '8.28810805790467209974771607e-588465002479', 27, 0);
    T('2.4548816699e-58559', '-7.5412e-4609416068168', '2.4548816699e-58559', 31, 5);
    T('6.5216166716609222580e-72', '-6.773402145118226504906583320618549360498962173415440824468e+65526', '-6.773402145118226504906583320618549360498963e+65526', 43, 3);
    T('4.0526142372e+735', '7.25429017544275904e+486824', '7.25429017544275904e+486824', 44, 4);
    T('4.36310043834695518797575748443879e+923668456500941', '-3.40e+30451807964', '4.36310043834695518797575748443878999999999999999999999999999999999999999999999999999999999999e+923668456500941', 93, 1);
    T('-5.097302e+704404139593', '-9.252928683619644422737897968988922454944e-76940034298', '-5.097302000000000000000000000000000001e+704404139593', 37, 0);
    T('-7.27849465906397681595257832844572827085756261538450516288021e-23523082090614', '6.58805256448052851882823145714238962214869582534822386481437e-7859', '6.5880525644e-7859', 11, 3);
    T('6.773557821290e-490741568610376', '-5.17065313633179848043884680905598963651388329e-353219558', '-5.1706531363317984804388468090559896365138e-353219558', 41, 2);
    T('5.6896198636808175051416362963e+4481794', '-2.803471533083002e+51368023', '-2.803471533083002e+51368023', 85, 3);
    T('9.8911184490960314158976162e+12026', '-8.70642137e+681281024', '-8.7064214e+681281024', 8, 4);
    T('8.235095293751657833802322e+32', '3.6472728931455254120168e-65522122979', '8.2350952937516578338e+32', 21, 6);
    T('9.37700786880620687026755269656678271664353880011852e-208', '9.16055787721322e-51991', '9.37700786880620687026755e-208', 24, 1);
    T('-7.769858727384786652194976759678422377392529005697577865589e+8166338732874', '9.15211333259808108330103283e-18482852054', '-7.769858727e+8166338732874', 10, 6);
    T('5.7234520675212097315754415105203756529668143e+925816095027059', '4.3544272626165499407538734983585233860253484211927521796e-4541698', '5.7234520675212097315754415105203756529668143000001e+925816095027059', 50, 2);
    T('-6.11173766718454501456e+9504', '-6.47499375226131915070773836057947860614896454033540e-31476590951', '-6.11173766718454501456e+9504', 68, 6);
    T('6.51500489529886655193493270535086789400604e-182777', '-2.500e+4343545', '-2.5e+4343545', 19, 5);
    T('-2.517253675269653187980041476084585923325680568056124278104e+30541', '4.35522475055e+96542908730', '4.355225e+96542908730', 7, 5);
    T('-8.45072256790e+21317434170639', '5.7756555703680565978677e+49', '-8.4507225679e+21317434170639', 20, 3);
    T('3.29277330e-3009', '-2.66699796310817269603921593663414134450214769e+78721273', '-2.6669979631081726960392159367e+78721273', 29, 3);
    T('-7.184325892482918181949976736105032212495392373967432e+88627122149495', '-3.63712818522594788133884035394333783587553130130e+542279', '-7.1843258924829181819499767361e+88627122149495', 29, 5);
    T('-2.9296494274455962e-316', '-4.61376749379869699293636459131811692592e+8074', '-4.6137674937986969929363645913181169259e+8074', 38, 1);
    T('4.0851043133606862994052059626795484477454136429817e-58162443659', '3.593188689430782319544354959007133351689564539886707012912e+102', '3.593188689430782319544354959007133351689564539886707012912e+102', 69, 1);
    T('-9e+642933', '-6.212390462581038300756689051681160237392046388856989753016e+9924700', '-6.212390462581038300756689051681160237392046388856989753016e+9924700', 80, 4);
    T('9.78678119014085511520461009976269119793e+41', '-1.6676678e-793464751882', '9.78678119014085511520461009976269119793e+41', 69, 4);
    T('4.893939392673485840349639e-233', '9.7191039916410561191992891372451740714298771880e-8817', '4.8939393926734858403496390000000000000000000000000000000000000000000000000000000000000000000000001e-233', 98, 0);
    T('-1.33e+7', '1.21e-97505', '-1.329999999999999999999999999999999999999999999999999999999e+7', 58, 2);
    T('3.3696714241070426289826934e+425917599243848', '9.636225e-6270', '3.3696714241070426289826934e+425917599243848', 70, 1);
    T('-7.4645057852069361696556384114219634849367072992e+51', '-4.23744068493195456830081955862578025827359026e+710441869777719', '-4.23744068493195456830081955862578025827359026e+710441869777719', 100, 5);
    T('5.1640980683197527495562864e-95981104500', '-2.987710109700968387065183954e-99227150', '-2.987710109700968387065183953999999999999999999999999999999e-99227150', 58, 2);
    T('1.82410505833310529581e-31581878', '-5.0e-11610679', '-5e-11610679', 68, 5);
    T('4e+48788421', '-3.96617205717938492140699e+5661', '3.99999999e+48788421', 9, 3);
    T('-3.308125764316457816885594547952849343771984956177819430348e-602379893059', '-3.56e-43303979408260', '-3.3e-602379893059', 2, 6);
    T('-6.37e-8956', '6.306868665695961829657948090501e-1754630714447', '-6.369999999999999999999999999999e-8956', 31, 2);
    T('1.27585749e+70188299157812', '-9.034212317090884195864277562876480215003531715914963e+598457749', '1.27585749e+70188299157812', 79, 6);
    T('-2.066933587843960416446e+357891726052010', '-1.7627314525086649821777889776158159516163e+46', '-2.066933587843960416446e+357891726052010', 91, 1);
    T('9.57555180972032832255420390870070472734004599443506770e+402', '-7.43519464893743252386493236605233e-3070930369979', '9.5755518097203283225542039087007047273400459944350677e+402', 56, 6);
    T('2.9912113494776510706683098624115976892114541440393579862756e-9063', '9.951710266949316628449839403093872890713379593123024400e-36512', '2.9912113494776510706683098624115976892114541440393579862756e-9063', 84, 1);
    T('-7.6399720191027158308350374775653345038680181830679e-7649', '3.44018605691069594587103011492285931e-7348392', '-7.6399720191027158308350374775653345038680181830679e-7649', 87, 3);
    T('-2.90013150e-2748821086', '9.994370798115258703302627383943801438663104e-448951', '9.994370798115258703302627383943801438663104e-448951', 45, 4);
    T('-6.383129849410194944563220406835570788195299775e-908404268218', '-2.89105124745560439806821747023e+11777', '-2.8910512474556043981e+11777', 20, 6);
    T('-9.50737918318520944560826559032106751446633342133583e+54897613505', '-2.6620165081960603e-4007260812', '-9.50737918318520944560826559032106751446633342133583e+54897613505', 62, 5);
    T('-3.8154357133479849925e-7022944', '-3122384953217471040540571084878147809', '-3.1223849532174710405405710848781478090000000000000000000001e+36', 59, 0);
    T('-3.53928177576576947497304887050073e-32747', '-6.3208e-742', '-6.3208e-742', 56, 5);
    T('-4.35696170540709e-60783250', '857171829046240865562.243', '8.57171829046240865562243e+20', 100, 4);
    T('-2.79524355820454831980780479545014895609631056e-8386231766', '-9.2391018503195132946422193975767950701517754030845763382e+907205390709', '-9.239101850319513294642219397576795070151775403084576338200000000000000000000000000000000001e+907205390709', 91, 0);
    T('-3.648408990905411722339408705232330417007856791309268053e-234389', '-3.58342344277471314409163085823509904826e+772347491474120', '-3.58342344277471314409163085823509904826e+772347491474120', 51, 1);
    T('1.0959796016960789e+838115683482', '-9.7894e+9731', '1.0959796016960789e+838115683482', 67, 2);
    T('-2.8757649678358731045940565078348227037454245e-393', '9.937383778237665199384110799328811110171790625062662e-56785361', '-2.87576496784e-393', 12, 6);
    T('3.4814705e+9169', '5.97105931266477712e-5553897251', '3.48147050000000000000000000000000000000000000000000000000000000000000000000000000000001e+9169', 87, 2);
    T('4.35654659279703121926831123355327047265047335469227605319e-2359', '-5.9199165113939344361427039974564081602529116669624463276684e+4188530', '-5.919916511393934436142703997456408160252911666962446327668399999999999999999999999999999999999999e+4188530', 97, 2);
    T('4.2833341297430258749766457098777359502951e-77844810216', '9.324776199329682692804984392e-775494673984051', '4.2833341297430258749766457098777359502951e-77844810216', 85, 6);
    T('-5.432e-5836', '2.022038987149866080973031169177674483257972893e-483', '2.0220389871498660809730311691776744833e-483', 38, 5);
    T('1.322717826692247623291488495356274066785282522026976514e+111', '4.50160363402610080771845799254180360173678463740e-33915011739900', '1.32e+111', 3, 5);
    T('-1.936536108925955729940182116591695020742e-962784', '-3.80318211810088808932510847331534640740851827185e-37', '-3.8031821181008880893251084733153464e-37', 36, 1);
    T('-9.8318958e+733420942081761', '-4.9302e+72605943', '-9.8318958e+733420942081761', 76, 6);
    T('4.3637948638349619735366785e-20721865711234', '4.84872902405520188529017900326230539880831124740e-28788', '4.84872902405520188529017900326230539880831124741e-28788', 48, 2);
    T('6.0869058170510148178202432086072114181876110131436767179e-459806', '-8.14022188620726150536569686171056819e+39', '-8.14022188620726150536569686171056819e+39', 96, 6);
    T('-5.0685494278501095991076615529495512150889871071198e+126', '-8.4860008537627342949204922277018629372228e+470886848', '-8.4860008537627342949204922277018629372228e+470886848', 100, 5);
    T('4.42455611317316132428273314096e-1628', '4.683422604843598580843648686052634442664712937917e+77708', '4.683422604843598580843648686052634442664712937917e+77708', 94, 1);
    T('-8.60364951874049867525797047236400819241905940060956e-4606968453887', '-9.87901124505882e-687429019874118', '-8.603649518e-4606968453887', 10, 1);
    T('-5.25671709001405195559314386490943727674920e+50', '9.1762322340090840319e-62263854729724', '-5.25671709001405195559314386490943727674919999999999999999999999e+50', 63, 1);
    T('-3.77523950463697908111567282934414691553543580245564458e-1510126960168', '7.56429688925024342782073092274722528545364085827099e+68', '7.564296889250243427820730922747225285453640858270989999999999999999e+68', 67, 1);
    T('4.656057025354210137e-8262571488429', '32104589569571103974285907763249848958657335.12164318', '3.21045e+43', 6, 1);
    T('6.491936955479714356411967655257597397161448015602e-8790786', '9.5524973529636694376138327563343099140523532710248183585157e-447097768728658', '6.491936955479714356411967655257597397161448015602e-8790786', 73, 6);
    T('6.85056350188301919908198e+5944316934', '-8.9334119762174879788334592827925604041422709565702e+773', '6.8505635018830191990819799999999999999999999999e+5944316934', 47, 3);
    T('-2.567781047267077285506671683531e-88831', '-4.340292626170241991178844e+72', '-4.340292626170241e+72', 16, 1);
    T('1.9185386837681494262297456831297415920269291202e+44577444208350', '2.3302e-811295390', '1.918538683768149426229745683129741592026929e+44577444208350', 43, 4);
    T('8.45154504539856075999260013445964818459427e+25722', '2.7269071059626433230e-8899702', '8.45154504539856075999e+25722', 21, 1);
    T('-1.0e+209', '-2.043081582424471731e-2562999', '-1e+209', 30, 4);
    T('-1.26544608110e-86622806875820', '-7.45689424555446273965652223830297836453902784598935e-59738709465', '-7.45689424555e-59738709465', 12, 4);
    T('-6.4781503318e+3302', '9.03948594e+69668951076', '9.0394859399999999999999999999999999999999999999999999999999999999999999999999e+69668951076', 77, 3);
    T('-4.4e+14737', '7.9185892958e-3342923', '-4.4e+14737', 100, 5);
    T('4.090667417499105327361900090347876300e-29109', '3.30422186543527885268343847591369078779420714241162850734e+1035', '3.3042218654352788526834e+1035', 23, 6);
    T('-7.53988513288527911899003011832e+45194795222487', '-4.42714069571941e+15', '-7.53988513288527911899003011832e+45194795222487', 80, 5);
    T('-6.861810e-4472648', '2.548520e+95373310', '2.54852e+95373310', 46, 6);
    T('2.832613870467857838858562411563044379174646945647994119e-27029655', '3488090.77', '3.4880907700000000000000000000000000000000000000000000000000001e+6', 62, 2);
    T('1.70555132558742079697438464805809390224754231570113942192252e+47278681606', '-8.29986e-55924432646', '1.70555132558742079697438464805809390224754231570113942192252e+47278681606', 76, 0);
    T('-8.23029686917577481728003476055963668379549972794933514e-26', '-5.20119791455825648e+88', '-5.20119791455825648e+88', 35, 5);
    T('-2.617e-37285101153', '4.145321245e+8845548', '4.145321245e+8845548', 96, 6);
    T('-5.5371891056167831818512415507964714e-839914199280', '1.7374965182038780335822237074855353132191e+4930', '1.7374965182038780335822237074855353132191e+4930', 100, 0);
    T('2.87365410021809911571890790986991181301621751824e-7987', '85180295.1113', '8.51802951113e+7', 36, 3);
    T('-3.570050e-23658', '-7.4355226690301317462720944479223e-5086826', '-3.57005e-23658', 60, 2);
    T('9.62039622698346122925062600001649613316886e+2054392', '-8.450008580647648681751624649724544846836039e-32', '9.6203962269e+2054392', 11, 3);
    T('-8.302253044914427815620619004e-57455376619237', '8.13290404666074577034812574910001420829210462637134315703e+32972562652208', '8.13290405e+32972562652208', 9, 2);
    T('-1.696837309682204917432946326299182120e-49', '9.14653605798916335499752780e-316', '-1.69683730968220491743294632629918212e-49', 64, 3);
    T('8.4188726941020243087836993637230532641024e-749', '-7.94345854978972751241680836441252350003086447091747e-382165864662', '8.4188726941e-749', 12, 5);
    T('-1.7099419564508083308197941870230936e+713324764', '-5.254185853304355738347546348562777410470008108787e-105', '-1.7099419564508083308197941870231e+713324764', 33, 0);
    T('-9.14980927575954812727502086064379246873515510047547e-591096177452', '8.5262761872157415932693174e+550924319', '8.526276187215742e+550924319', 16, 2);
    T('-1.70083296773730991371753404710726214186679825402385947e+8185', '3.9259918289174807375049624142312829e+9039616', '3.9259918289174807375049624142312829e+9039616', 76, 2);
    T('6.66668884254944833980290831e+26872781', '1.3865917927697437698e+50247130990974', '1.3865917927697437698e+50247130990974', 66, 3);
    T('-9.82375773412008357808025417426186141870259108428e+53784', '-7.291054175933956679238863e+91682186823455', '-7.291054175933956679238863e+91682186823455', 99, 2);
    T('1.54472566661802921361408540e+42', '1.42869050706539291974241770879239515807961961e+359789881190941', '1.42869050706539291974241770879239515807961961e+359789881190941', 66, 5);
    T('7.71154471507751262989050624202e-3061505385387', '7e-112509', '7e-112509', 97, 6);
    T('-1.269125957884991519904920598e+59113490143', '-4.5643e+571248724862', '-4.5643e+571248724862', 75, 6);
    T('4.53652863188692570829460931666949650976084002140916567295491e+602931', '9.5535121238072815114901265893940170358614072e-15342', '4.53652863188692570829460931666949650976084002140916567295491000000000000001e+602931', 75, 0);
    T('-84449512330715386418131223899045082582279.1026992', '-9.4937755342916080727319746992872011238240398e-455095668', '-8.44495123307153864181312238990450825822791026992e+40', 79, 5);
    T('5.69664e-5772207913245', '9.948393553514545037147e+126', '9.94839355351454503714700000000000000000000000000000000000000000000001e+126', 69, 2);
    T('-8.641470213583796605326959305106838610804237020e-598766211', '-1.483846068798360042666989352e-9144', '-1.4838460687983600427e-9144', 20, 4);
    T('1.944777035086681149706173138e+101', '4.2567303117541e+7076172761903', '4.2567303117541000000000000000001e+7076172761903', 32, 0);
    T('-1.501625721115735906214353484e-22', '-7.49787893304e+537632872', '-7.49787893305e+537632872', 12, 0);
    T('5.280088990309972960203069289040322068851710322418e+88056308', '9.57853371161170302604707360115575e-269498', '5.28008899030997296020306929e+88056308', 27, 5);
    T('-4.358513770176470044575904719588307031608328793946825379e+328', '5.045876641675779062485663267e+33', '-4.35851377017647004457590471958830703160832879394682537899999999999999999e+328', 72, 2);
    T('-2.101193463331079868309722364661747584766005716260744789791e-7224340634609', '-9.27235274446e+77265147137798', '-9.27235e+77265147137798', 6, 4);
    T('-3.55560099676442506424086307087264304281e+91091937417', '-4.7355036788199114e-22244863264', '-3.55560099676442506424086307087264304281e+91091937417', 67, 6);
    T('-8.2990961672e+283992701', '6.190884828879117162862437426880833e+366159', '-8.2990961672e+283992701', 80, 6);
    T('7.2413916956474572229746122506708e-340', '-0.03', '-3e-2', 83, 4);
    T('2.414243511015180502055322754742112596e-274', '-9.008196067816064567784850643749845477136373732915112162e+4635523746', '-9.008196067816e+4635523746', 13, 4);
    T('-5.0278277003303137282976221e-749510017100', '-1.5701994895706108997984829723619917109518458e-252', '-1.5701994895706108997984829723619917109518458000000000000000000000000000000000001e-252', 80, 3);
    T('-5.47253311679770697016355945471201278953942112239144914274e-833870', '4.25e+4851727048286', '4.2499999999999999999999999999999999999999999999999999999999999999999999999999999999e+4851727048286', 83, 3);
    T('-5.23128024360864280902418793743e+12141933', '-7.7749e-917', '-5.2312802436086428090241879374300001e+12141933', 35, 0);
    T('-7.1956226e-521149798106', '-9.613971141361540962829420271932411426017677098548901279e+40555037108', '-9.613971141361540962829420271932e+40555037108', 31, 1);
    T('-9.086704456e-661475', '72670', '7.267e+4', 18, 5);
    T('2.82218929095688780139258146393315554e+1491', '-5.4270462494947293601097e-9943493455881', '2.82218929095688780139258146393315554e+1491', 39, 2);
    T('6.2830910381441e-154', '-1.57698376585075214840231597022983898501260252894487061e+56', '-1.6e+56', 2, 5);
    T('-4.9152634411e-36202638212', '9.5667420859480679654013e+6385', '9.56674208594807e+6385', 15, 2);
    T('-2.8e-110345782', '-7.491646455807717533234034623e-7540', '-7.491646455807717533235e-7540', 22, 3);
    T('-9.41738068499678585e+5349433529055', '3.72002316916712855601095228164665217862e-77665', '-9.41738068499678584999999999999e+5349433529055', 30, 1);
    T('-3.85561629721204191775509249287036131505605701109940324e-59460322', '5.24733598717770487733647497199607382219e+7121073', '5.24733598717770487733647497199607382219e+7121073', 61, 6);
    T('-4.9964098488598695593253610828501e-560929', '-8.1498791631184056363318722461947236961840067009697503209e-21', '-8.1498791631184056363318722461947236961840067009697503209000000000000000000000000000000001e-21', 89, 3);
    T('1.9484342250339737144780212261530478973058080434e+18716222240852', '-4.2070831774996100e+843575297', '1.9484342250339737144780212e+18716222240852', 26, 5);
    T('-6.68852535897505300144380e+2682342068881', '-6.9883058628885675706942110402747e+152721', '-6.6885253589750530014438e+2682342068881', 92, 5);
    T('1.52154548689166211168555965008960720309774653728903216137e+118', '2.2156717257755144012999797131240380317402e-920', '1.52154548689166211168555965008960720309774653728903216137000000000000000000001e+118', 78, 0);
    T('-3.5624571758568483628648377268093e+4634086254', '8.909519090970064235838720214445687461893153995587167e-32', '-3.5624571758568483628648377268e+4634086254', 29, 6);
    T('3.56051156442265791949385179328789082266518396063937405567e+3530942', '-1.1724585164470803331843823032875475174649e-62951', '3.56051156442265791949385179328789082266518396e+3530942', 46, 3);
    T('-6.1085085859164484266187349860495066419052955822e-28', '-5.178493069308332840754790134717240363191276610e-144603403150617', '-6.1085085859164484266187349860495066419052955822e-28', 67, 6);
    T('-5.258100097660998635162838447644292e-2932589', '-2e-839164635173017', '-5.25811e-2932589', 6, 0);
    T('2.714139568763665223117063609235e+169932089820859', '1.1561153197816111983306826182258070e-9646819895', '2.714139568763665223117063609235e+169932089820859', 40, 4);
    T('2.121796911163451984322514980232663507008e-11325', '-9.191105396371975907657848e+105', '-9.191105396371975907657848e+105', 84, 4);
    T('-4.556455890360298830641469860119920030e-9667', '-1.510138685001e+390882917694566', '-1.510138685001001e+390882917694566', 16, 0);
    T('-9.8508158106463502522352659233171592e-37548', '-8.529193401791810585050370023251552751917346285032800727e-1096784716670', '-9.8508158106463502522352659233171592e-37548', 42, 4);
    T('5.5444767077463916021e-5367', '8.773744654597705676746411634313057364185e-5816', '5.5444767077463916021e-5367', 66, 1);
    T('4.28e-29776568859', '-5.943513804907854614260188815151e+752929539514', '-5.943513804907854614260188815151e+752929539514', 48, 6);
    T('-2.86377803044766635220345815080049078482111909882169e+90312658880482', '-4.5688172418163978495501397271481924e+6205055662', '-2.86377803044766635220345815080049078482111909882169e+90312658880482', 58, 6);
    T('7.67570566845573e-1529', '4.2110289804349042775031264450999335984767e+90340988', '4.211028980434904277503126445099933598476700000000001e+90340988', 52, 2);
    T('-4.125298e+221256620', '4.054743e+3376324', '-4.125298e+221256620', 80, 3);
    T('3.330775276484523316502626754326649647877075e-58521', '-2.923857908063155e+23', '-2.92385790806e+23', 12, 4);
    T('9147446474168104493186683975774.84', '7.581239357737453392876e+781', '7.581239357737453392876e+781', 62, 1);
    T('-5.15841422798775652968299614786502755258623427895593e-78929686626027', '-2.215961457997230795054145083308520e+174724638', '-2.21596145799723079505414e+174724638', 24, 2);
    T('8.18363984075015552436341919079573272552e+42', '-8.005994765384871717953875238679645290981257343e-4104', '8.1836398407502e+42', 14, 4);
    T('4.9191821549305682848942230207008973803737851e+2382808', '2.580e-1637', '4.9191821549305682848942230207008973803737851e+2382808', 51, 6);
    T('5.9656519167217561235012323870161109715e-1674410887', '-3.346649925226087309504938096689459491156690173099818660e-709976', '-3.34664992522608730950493809668945949115669017309981866e-709976', 82, 3);
    T('-5.37685758961e-79', '-5.8604557318785244570220213377721087734205771312898224e-660871042554038', '-5.37685758961e-79', 56, 2);
    T('161398.89877338', '-8.58223139274051768579122795619043614385230570994839e-17649', '2e+5', 1, 4);
    T('8.634186198285947774012751864145938143390020501615753920e-49899', '3.2579102731941671290001e+68665517', '3.2579102731941671290001e+68665517', 37, 3);
    T('3.4559150505034675860947680463863395041e+445833737168', '-9.72164695986105513690456241761800850527021866837905248656e+2624', '3.4559150505034675860947680463863395041e+445833737168', 88, 2);
    T('-91275305293656067965907880883298210796422507148787724997782', '-7.8414159848916506321354925860e+709', '-7.841415984891650632135492586e+709', 86, 5);
    T('7.033860537188558e+691', '4.4925e-48617', '7.033860537188558e+691', 59, 3);
    T('1.648694768543098e-94233153', '4.8954958060463e+850504843', '4.8954958060463e+850504843', 57, 1);
    T('-4.235495e-851786', '9.79873196958460512246179632338284114455637032e-888669521', '-4.235495e-851786', 100, 6);
    T('986616321243240693327851611881414166580745.6868', '7.82788380461032377672698970126507023560981013189265097e+518335', '7.827883804610323776726989701265071e+518335', 34, 0);
    T('8.1152747439050601225397662606564130708e+337970005', '4751568.7209530794703429761432990556758672', '8.1152747439050601225397662606564130708e+337970005', 85, 3);
    T('-7.961658278676696729009285323938386909769219854791487e+20217706866382', '2.922e-4514722221793', '-7.961658278676696729009285e+20217706866382', 25, 2);
    T('3.94592222719745510770667e+646', '8.4078397207814519e+37949', '8.4078397207814519e+37949', 70, 4);
    T('4.51700215813046163945959e-276758432', '6.481236597443839634424462513971e+3841468', '6.481237e+3841468', 7, 6);
    T('-1.55872113263570105804341e+65', '-7.13898922688905724051e-30', '-1.55872113263570105804341e+65', 78, 2);
    T('1.770065584708618241522021696014800e-404519413', '6.5827929203987041867151536266994080653500195e+364529804', '6.5827929203987041867152e+364529804', 23, 0);
    T('4.44905684314039708267646961240801332133506e-6572', '62808422302966331742679850370969273753540699335474340583', '6.2808422302966331742679850370969273753541e+55', 41, 6);
    T('-7.49689807247663834132323e-930952148878293', '1.129524416310e+110540', '1.1e+110540', 2, 5);
    T('-4.4672232602074694657791575537585947339913190693e-6547', '-1.5057027443734598532723335511230107301048303108957296529e+23821', '-1.5057027443734598532723335511230107301048303108957296529e+23821', 73, 2);
    T('4.993554125063438498744642237499776669e-3647', '-1.7971634020609547673339010005820722525106058297754155148941e+16103144', '-1.8e+16103144', 2, 6);
    T('-7.404763869405172692559097244440738350901199698307960886614e+6896798', '4.2316933544829016532466631464024342277642697176535098e-9385408', '-7.404763869e+6896798', 10, 5);
    T('76367.671716', '1.1e-706119114', '7.6367671716e+4', 52, 5);
    T('7.449402343400898479719288e-102', '9.127950087e+866145', '9.127950087e+866145', 43, 6);
    T('9.648e-61978', '9.6774232889748734938608899647389170e+69997', '9.67742329e+69997', 9, 2);
    T('5.843512926178167194866894232867757702650918955328932e-18', '4.56227379542727882166e+229030242263', '4.56227379542727882166e+229030242263', 28, 3);
    T('8.028628709810911e+984529322109637', '-34838516677281962134.7494', '8.02862870981091099999999999999999999999999999999999999999999e+984529322109637', 60, 3);
    T('-5.1334088269858669534709130550e+953795', '-5.4589488557726117e+763678', '-5.133408826985866953470913055e+953795', 39, 6);
    T('6.876020894125878593923996419173830e+90616', '7.218658356669103057e+289373134', '7.218658356669103057000000000000000000000000000000000000000000000000000000000000000000000000000001e+289373134', 97, 0);
    T('-7.3780464175024509608238709461310997e-48037394562', '3.466846326647921194015420131701e+5282853878804', '3.466846326647e+5282853878804', 13, 1);
    T('-7.6793360042932763364138252706429062810567e+46', '-8.96600718614e+1918', '-8.96600718614e+1918', 21, 5);
    T('-9.177329e-5645000150', '2.663808139907134712552218536547047423360820297275254159236e-44644534165814', '-9.177e-5645000150', 4, 5);
    T('-2.4477773315060879619312700284508209682349883259957e+338', '3.59611867722957780814401880729660373213853237e-3574', '-2.44777733150608796193127002845082096823498832599569999999999999999999999999999e+338', 78, 2);
    T('-690348707045075925024606701718541110840.08771', '-8.836469226005726e+84360147664', '-8.836469226005726e+84360147664', 37, 6);
    T('2.02265949239305549201095e-46789374544', '7.7941227528133075935058278932e+1345537793469', '7.7941227528133075935058278932e+1345537793469', 100, 1);
    T('-9.036984768398819897011635889715238e+899010393', '1.58562201785099769580979725e-70477258', '-9.036984768398819897011635889715238e+899010393', 55, 6);
    T('5.5052423190633672760864244882358356647e-20258', '-4.6303146425515425516072502961733442986326458871202e-987925492653', '5.5052423190633672760864244882358356647e-20258', 65, 2);
    T('4.02623636742861304976136081680e-35517282194620', '-3.6e+26787', '-3.6e+26787', 36, 6);
    T('312306138229954354807772583962754.113615', '7.82542057022113155501839760942586232960123e+9575760587206', '7.82542057022e+9575760587206', 12, 4);
    T('-4.61774385428142935537220769099253032618117e-29697886', '-4.53884612202455416274132760132566328658448576559843e+934730688924628', '-4.53885e+934730688924628', 6, 0);
    T('-1.320900932832421552726573053947603161673496733260e-227240617', '-1258068630393538691.04154', '-1.25806863039353869104154e+18', 34, 1);
    T('2.5624573458955517977564941841317344148298430725787573e-48782', '4.20020628170739966574633862135257264903801825444457949e-7272398', '2.562457345895551e-48782', 16, 3);
    T('-8.810022e+32970775', '8408505614377', '-8.810022e+32970775', 10, 5);
    T('3.9969288666198640214468079977233e+6367', '9.56300366144975029241371581532478170538715944686873e+47957864', '9.56300366144975e+47957864', 15, 4);
    T('-8.97387075147138618e-8664404179', '-3.181915083188474692027272215230e-43125695977', '-8.973870751471386180000000000000000000001e-8664404179', 40, 0);
    T('7.9051177658275573972599615620233310826482933921770377437879e+80803627', '-3e-985', '7.9051177658275573972599615620233310826482933921770377437879e+80803627', 61, 0);
    T('-2.3508672839305554698105279643642e-20491520441416', '3.2556100792744780271703162e+3057', '3.2556100792744780271703161999999999999999e+3057', 41, 3);
    T('7.669154320777310865586762e-7501333', '-7.3002883981527830e-97500062202975', '7.66915432077731086558676199999999999999e-7501333', 39, 1);
    T('3.22107937562964477715847452633750e-37', '8.677949809354545262e+4624779784557', '8.677949809354545262e+4624779784557', 60, 3);
    T('-3.986934659678386893239245043966953903373292074222625101425e+1124', '-3.0268346345e+56144765286', '-3.0268346345e+56144765286', 32, 6);
    T('4.6152252022511960637549144e-270230250', '-2.75408e+4014545652352', '-2.75408e+4014545652352', 88, 0);
    T('1.67982832923487029267648753680252848892462462588334515155e-379', '-1.69007108840722810890e+9836360', '-1.6900710884072281089e+9836360', 87, 0);
    T('1.9265094091726667434407564919840752838337720802107e-114538', '4.6896011066725318e-658956494048', '1.9265094091726667434407564919840752838337720802107e-114538', 67, 3);
    T('-1.678776407046271062070584099e-1818754893', '1.06955510952785297464426e+43154385', '1.06955510952785297464426e+43154385', 24, 0);
    T('6.5067955020888004354024696349650224896223348915913111e+87017', '-2.1262296763898856686602087393046293199315994264484732e+6809850', '-2.126229676389885668660208739304629319931599426448473199999999999999999e+6809850', 70, 2);
    T('-3.4e-76848', '-8.16047028330095117505833267099423800794640217485631529e+646', '-8.1604702833009511750583326709942380079464021748563152900000000000000000001e+646', 74, 0);
    T('6.19922109978199006957817617e+5239', '-6.76575427687116961846901888441179794385480890435022819734093e+1032', '6.19922109978199e+5239', 15, 1);
    T('2.2042047337484e+465628886', '-4.63070923651570517712188415349480204137003e-560864', '2.2042047337484e+465628886', 29, 4);
    T('-2.229793e+369063', '2.06241804619310004192754670948448461993103282236592701410e+80', '-2.22979299999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+369063', 96, 2);
    T('-4.016379287626699908781883707578674207148104527578458558092e+66615213256136', '1.4325286808938304168707609704957214574781485013e-22481655981', '-4.016379287626699908781883708e+66615213256136', 28, 4);
    T('4.9596e+9414', '-7.515063742824932618743265614471995752858948136599609482e+616361206043', '-7.515063742824932618743265614471995752859e+616361206043', 40, 0);
    T('857524568927011448350921683852031214251281461966', '2.473753478396521765844e+373', '2.4737534783965217658440000000000000000000001e+373', 44, 0);
    T('9.49551877701739768127382e-55316833', '7.03171219428634366940909854197580239790812e-872', '7.03171219428634366940909854197580239790812e-872', 90, 3);
    T('-2.6178e-2085793317', '2.62428129511565e-87143', '2.62428129511565e-87143', 34, 5);
    T('5.165995e-203', '0.00677288709348912550858454144127257', '6.77288709e-3', 9, 3);
    T('1.5137744892e-8229', '4.78140998950441628125860669940087251544397486551290250415035e-98301', '1.5137744892e-8229', 36, 5);
    T('5.773561082228098862538355751993031232831618715e+6969645778', '-52098516780602526313193053470633160.715525588240515507948', '5.77356108222809886253835575199303e+6969645778', 33, 5);
    T('7.3565881101254814386177130816040993164e-69002958', '-2.408e-7339402286', '7.3565881101254814386177130816040993164e-69002958', 70, 5);
    T('8.5247879254901142251717425284443224996666992e+95165', '-1.388980867886594609e-24230400', '8.52478792549011422517174252844432249966669919999999999999999999999999999999999999e+95165', 81, 1);
    T('-3.721409015049141629097050833552597572550e-4651728', '-4.176781299158467818187870382708778939122038513199828382510e+424099066', '-4.176781299158467818187870382708778939122038513199828382510000000000000001e+424099066', 73, 3);
    T('-6.560013282358504366406e-6088106', '-3.6633826569e+24128815541389', '-3.6633826569e+24128815541389', 78, 6);
    T('-6.857e-791468461335839', '-9.92481530e-817895459318', '-9.9248153e-817895459318', 17, 4);
    T('-2.718e+7469304449', '-7.64428271967798270919109061960e-133205941', '-2.718e+7469304449', 62, 6);
    T('-2.20529437747234917345586310269574230642410e+92000', '6.36470443986431938e-9010893362', '-2.205294377472349173455863102695742306424099999999999999999999999999999999999999e+92000', 79, 1);
    T('-6.6e-5245764770518', '-4.717556e+7892', '-4.717556e+7892', 51, 1);
    T('1.2633e+85814505', '1.9366060985797334699165e-43410357', '1.2633001e+85814505', 8, 2);
    T('-8.9770800289073979935521632873518184045097534256625397757694e-775717283388574', '9.94224484754e+24', '9.94224484754e+24', 50, 5);
    T('-9.59801013692094775971122358164903098511697984e+176', '4.5615927982236498784591923814661632566449857607402e+103', '-9.5980101369209477597112235817e+176', 29, 3);
    T('8e+203109', '3.234572590360625974468093222769843837437895123632858807578e+69796685', '3.234572590360626e+69796685', 16, 2);
    T('-2.4229296e-42399154994', '6.869158312042e+12714700', '6.869158312042e+12714700', 98, 2);
    T('-2.92564289877669896510693776790876729904446607074727e+145', '-3.56557613985086702980587226905e-893534429', '-2.9257e+145', 5, 3);
    T('-4.6900726029653003916548225153626856e-974884', '1.14776187968746147444992719555248970827485108208e-432075577655357', '-4.6900726029653003916548225153627e-974884', 32, 0);
    T('-4.405e+40', '7.3937067067785282048484542081749024e-2825', '-4.405e+40', 43, 0);
    T('1.26317229086035066025857090282937905874e+193042058874239', '1.3203908294416411507159630210726588546454336504e+91226', '1.26317229086035066025857090282937905874e+193042058874239', 60, 5);
    T('1.04040029930150721686862e+1837883627', '4.885204902174180378781978817348765344936566124151009634317e-2833', '1.04040029930150721686862e+1837883627', 41, 6);
    T('4.181313154026426258e+837354', '-4.894680974679152098e+9432', '4.18131315402642625799999e+837354', 24, 3);
    T('-4.427376418630834163330548658128430139584008889957e+4049', '-9.670100475909486362211855579021629928326688273e+8506', '-9.6701004759094863622118555790216299283266883e+8506', 44, 3);
    T('6.0666569671391168e-9025', '7.9232373775954892594099257e+2959818772297', '7.9232373775954892594099257e+2959818772297', 39, 6);
    T('1.54298981922292945e+36125722373051', '-6.6249081439200679293490434939524e-184434623594682', '1.54298981922292945e+36125722373051', 32, 2);
    T('-2.72895100593814088218872282891369586802271083966393907420900e-707505819539002', '-8.0096183561764732484e+963446', '-8.0096183561764732484000000000001e+963446', 32, 3);
    T('-6.23798e-58631441215607', '-5.499253289e+351099', '-5.499253289000000000000000000000000000000000000000000000000000000000000000000000000001e+351099', 85, 0);
    T('6.284943249554181057052064888459039623246977326589152e+14155702', '-9.5262037192e-522', '6.284943249554181057052064888459039623246977326589152e+14155702', 91, 0);
    T('7.24941942449846822424098e+3423', '3.0503703878237895545e-46226216309', '7.24941942449846822424098e+3423', 52, 4);
    T('-72025.51132', '7.546812219286620070739041084066535e-606433940334206', '-7.202551132e+4', 84, 4);
    T('1.9724957888430080814347386716204142455124756690483688e-858108607954077', '-8.13641556160795316333e+3859672340', '-8.1364155616e+3859672340', 11, 1);
    T('-390578774944888810609323995551', '-3.178136508214844766874223787940789669349406058837356640e+181674348797', '-3.17813650821484476687422378794078966934940605883735664e+181674348797', 55, 4);
    T('-8.3406e-5541', '-6.36470046489605525808986074075048e+789841172', '-6.3647e+789841172', 6, 4);
    T('-8.5e+5', '77295348980515096378421496371350145869579.575797425', '7.7295348980515096378421496371350145019579575797425e+40', 59, 6);
    T('-5.868320305322487226349073650322660e+31451948484', '-93412675830473873508312617612253018617024', '-5.86832030532248722634907365032266e+31451948484', 58, 5);
    T('-91026117.057605', '-4.5872971776329e-603442419', '-9.1026117057605e+7', 90, 2);
    T('-6.81668787330947968784870612688728684679466051887084824e-15528', '-3.9842304503178069628821911e-961982004', '-6.81668787330947968784870612688728684679466051887085e-15528', 51, 4);
    T('-4.9737323120025278035101124501848625450158802758537424033727e-359', '1.6194462217498614401820124124056961591204509722568493864685e+16924012769', '1.619446221749861440182012412405696159120450972256849e+16924012769', 52, 4);
    T('8.59889243045407679285175951503388430001e-84425154000', '-7.299523856e+6041305875', '-7.299523856e+6041305875', 89, 0);
    T('-1.3e+5391155956470', '-3.3396236436954929379693760218364649740776220407e-22100', '-1.30000000000000000000000000000000000000000001e+5391155956470', 45, 3);
    T('3.9265310263211538801556862503576867365705022863511073984e+99868', '-4.679328012226215369582777719721589933402459e-160778788', '3.93e+99868', 3, 0);
    T('7.4825963385397211022023214305517938748e+991618504', '-5.2626133388984119363969972e+28842', '7.483e+991618504', 4, 6);
    T('7.7339006e+501619', '2.46798755299837213907912e-81012455447', '7.7339006e+501619', 77, 4);
    T('-9.1922998050744840281e-676112920587', '-444052951001653097977121586202384158030490780966025037905', '-4.44052951001653097977121586202384158030490780966025038e+56', 54, 0);
    T('7.9894910001361252969900369437579947273314317349e-585565931', '-0.00530771974', '-5.30771974e-3', 94, 4);
    T('-9.292375630549222453005463682740740306955551678232831e+890508313', '9.7598325338613073535777124e+8322', '-9.292375630549222453005463682740740306955551678232831e+890508313', 92, 0);
    T('3.29404723663452230629669796022091208869916588968116840987e+9734070352', '9.9080630975267082796784440911486939453780591866791265499159e-3120690', '3.2940472366345223062966979602209120886991658896811684098700000000000000000000000001e+9734070352', 83, 2);
    T('3.9318149160828016506097950941387e+1182664259265', '3.34294988208e-42598813172', '3.9318149160828016506097950941387e+1182664259265', 34, 5);
    T('-4.4618266142350e-4378940771', '3.13582576771118674150151395038069997926640e+66912780', '3.1358257677111867415015139503806999792664e+66912780', 47, 4);
    T('-9.31169262456199e+183389014267752', '-6.20431019075037e-3045944', '-9.3116926245619900000000000000000000001e+183389014267752', 38, 0);
    T('-6.9273e+5839', '5.54085595075848696616882301544923017385342779951e+93', '-6.9273e+5839', 26, 4);
    T('8.21548349158256441704659493093852552633907e-2279', '-7.3230837503343224991249634786668137180e+283336920646', '-7.32308375033433e+283336920646', 15, 0);
    T('8363382490601709098375473055049653899349348295017.199', '-7.930392258e-59340', '8.363382490601709098375473055049653899349348295017199e+48', 89, 4);
    T('2.5861247596286453291744670320982177046001469970342072413e+735992411891481', '380137.7624221', '2.58612476e+735992411891481', 10, 2);
    T('-9.09434886914783e-911839123', '-8.3960909250113710515214638902974413027501696526527894e+258520091397573', '-8.39609092501137e+258520091397573', 15, 4);
    T('2.155691958883764481304113171235017584109e-271612343578', '8.98497804358700634395934102705046194516445e+706171', '8.98497804358700634395934102705046194516445e+706171', 62, 6);
    T('-9.0581272029866651850728060534023170702e+466', '-8.31226298729781725409075189444158628074515187470959e-69516221', '-9.0581272029866651850728060534023170702e+466', 76, 1);
    T('8.55797769641472026061183264135119922168495320236402250e+220796799910319', '-2.47725035909792626584290135539866600668824e-193056520871', '8.557977696414720260611832641351e+220796799910319', 31, 4);
    T('4.0e+9557', '-2.2354881688806e-870', '4e+9557', 46, 4);
    T('5.66473e+20525402480', '-8.7858254044104154791679654348961589621007881180864693379e-247663326846', '5.66473e+20525402480', 62, 0);
    T('6.78629843527045570492510588752230e+5676205088364', '2124959.228', '6.7862984352704557049251058875223e+5676205088364', 33, 4);
    T('9.54156611822e+7073263', '7.087005155342220797231417976104152752806853885752e+54', '9.54156611822e+7073263', 58, 6);
    T('2.31e+19489459', '-2.57254072878989871942167999574522857588494545972810074e-12021445098702', '2.31e+19489459', 45, 0);
    T('4.765041244115166251e+739313712', '4.949069732196721034136882677577850138e+94480201623522', '4.9490697321967210341368e+94480201623522', 23, 1);
    T('7.7395671304667553223440512747290733110763885e+78847480', '7.7887683114413053e+265', '7.7395671304667553223440512747290733110763885e+78847480', 52, 5);
    T('-9.982600088994496519978958962748837931646840e+1212', '5.947250767031348655216158952029472623713165812128102e+55', '-9.98260008899449651997895896274883793164684e+1212', 79, 6);
    T('2077269083225513715167700565841319540470817.777569799', '-2.211336884501409440410889896e+1791900954596', '-2.211336884501409440410889896e+1791900954596', 90, 5);
    T('-3.5241192964858274380008762201e-973989158440', '-3.773775e-71083436855140', '-3.52411929648582743800087622010000000000000000000000000000000000000001e-973989158440', 69, 0);
    T('-5.182793e-564515007', '-4.77368247827498283509219898942174845402e+2032929', '-4.774e+2032929', 4, 5);
    T('-6.76317796326754596266213860396162237184092517e+3890', '9.21916628786654021834731203815345580128374e+327', '-6.76317796326754596266213860396162237184092517e+3890', 59, 3);
    T('3.2858240820361e+2923938978660', '-6.280070717760099969430961023e+3900280', '3.2858240820360999999999999999e+2923938978660', 29, 3);
    T('1.028e-94395952185', '-7.54877819258486788266981439385893735584923e+67164435', '-7.548778193e+67164435', 10, 0);
    T('-15581631626596565944692696419349615453024.60834990', '5.6042231330202925780578e-4371774064', '-1.55816316265965659446926964193496154530246083499e+40', 98, 0);
    T('-5.756141439e-177519969', '-2.97463364798569374539760024743866e-75600', '-2.974633647985693745398e-75600', 22, 5);
    T('-5.988725127447e-1148160691586', '4.123935e+9', '4.123935e+9', 35, 0);
    T('4.26731651772781096867485851832956e+41', '-5.307434666725940e-7031', '4.26731651772782e+41', 15, 2);
    T('9.66198554266143131825231569928067e-7353615788', '-5.81167126596524978502720482e+599', '-5.811671266e+599', 10, 6);
    T('-5.9395569856019464780691987145348607493417331986e+41544915169', '9.33088e+3345', '-5.9395569856019464780691987145349e+41544915169', 32, 0);
    T('3.19872210e+44651', '7.07950150950926118883949e+67984', '7.07950150950926118883949e+67984', 64, 1);
    T('2.3739471618863165933400880738625658397813199521607731260e-773', '7.2054271464549239136529817152e-468373', '2.3739471618863165e-773', 17, 1);
    T('6.86609729968088116715399284881368007e+25097689825', '2.5682759341496078639489669137411669e+295063341088', '2.5682759341496078639489669137411669e+295063341088', 47, 5);
    T('-1.40757076058720879985678386921809503e-525659426224019', '-5.17795e+321738394746', '-5.17795e+321738394746', 7, 6);
    T('-195332391688131409.1804069', '2.68900437837882e-70', '-1.953323916881314091804069e+17', 76, 5);
    T('-8.37656364451620258277485799071132991848086950481788e-282030', '-7.02529576008431686080330323997273892440649985520194005e-8162266465177', '-8.37656364451620258e-282030', 18, 5);
    T('7.7827162175168298925965023751769331506180185997e+6056778798265', '-4.735144398687546701880248557e+96093874', '7.7827162175168298925965023751769331506180185996999999999999999999999999e+6056778798265', 71, 3);
    T('8.9991371927235288950306250978418580369284e+50679', '-9.332573913806e-9193', '8.9991371927235288950306250978418580369284e+50679', 96, 0);
    T('-6.4806444707280e-312464', '-2.1390290284189572e-416823', '-6.480644470729e-312464', 13, 3);
    T('29291151.5925', '-3.6619928466581106283921221318158159680230135222770618237275e-799797', '2.92911515925e+7', 44, 4);
    T('6.9340278244747654894684557937e-99513', '-8.475706858948247463675e+99782694223525', '-8.47570685894824746367499999999999999999999999999999999999999999999999999999e+99782694223525', 75, 2);
    T('-6.459056231647725533146174116745145709358952e+7296', '5.65403958e-507746', '-6.45905623e+7296', 9, 4);
    T('2.16624214626567071540260166352321389465389357598313141256090e-678605', '5.215152107148527161470618228980818e+35', '5.215152107148527161470618228980818e+35', 88, 3);
    T('5.0605203503756716e+92785560755596', '-7.266636e-681804026221', '5.0605203503756715999999999999999999e+92785560755596', 35, 3);
    T('-6.4716197574820127188798088661287691918316e-19', '-6.78e-2655077446', '-6.4716197574820127188798088661287691918316e-19', 84, 5);
    T('4.04009660469516032384304e+882662', '-5.734380593056115104655590164381160135e+15415806635941', '-5.734380593056115104655590164381160134999999999999999999999e+15415806635941', 58, 2);
    T('-4.5572316054419776901019471e-222642165548', '3.896043436495556781493257932838899826e+916820098042', '3.896043436495556781493257932838899826e+916820098042', 78, 2);
    T('-3.75509156e+6416050698722', '1.88552e-73', '-3.75509155999999999e+6416050698722', 18, 2);
    T('0.00123758237', '-2.28633600245530665846454654742256709e+56841122', '-2.2863360024553066584645465474225670899999999999999999999999999999999999999999999999e+56841122', 83, 1);
    T('4.28455599444261861008631422952222484130864240419498155077878e+77833737', '3.16465116481189970598124798e-25', '4.28455599444261861008631422952222484130864240419498155078e+77833737', 57, 5);
    T('2.548874842651560442090503895092552107012685379888e-634180', '1.4895748851540509870499171308348781281e-686284897401700', '2.5488748426515604420905038950925521070126853e-634180', 44, 3);
    T('90', '-5.47421643709571977704845661642608342880246138660602242159e-7890079997280', '9e+1', 42, 2);
    T('-8.828349e-405370913440', '8.79821734135216023090058738948953566017e+46607217404334', '8.79821734135216023090058738948953566017e+46607217404334', 70, 4);
    T('7.5145522300324695727333165662194838657542627778178492e+1215506534', '1.1708120377949663442633390415495262141404e-406473', '7.514552230032469572733316566e+1215506534', 28, 4);
    T('7.3167013901281750e+207520859', '-6.9872897e+244401', '7.316701390128175e+207520859', 68, 4);
    T('8.95460511294055591794527139946860244883614821028e+52', '-9.037588600194170397099460092714305e-5776132413', '8.95460511294055591794527139946860244883614821028e+52', 68, 2);
    T('6.4784114869722540323771641763091154584166817457259e-169', '6.080795747684892463266729e+55383', '6.080795747684892463266729e+55383', 33, 5);
    T('-7.925622699052193315905360974072794446184446e+261489', '9.77502360538534335099e+9897189851', '9.77502360538534335098999999999999999999999999999e+9897189851', 48, 3);
    T('7.446405779714546334354742099834304999128450014639726e-392', '2.63536843567164256279811205384667e-682215899291', '7.446405779714546334354742099834304999128450014639726e-392', 71, 3);
    T('1.52360585700970473e-3246', '9.5e+21804636201', '9.5e+21804636201', 73, 5);
    T('-9.68153518469812393261964639e+3765', '2.2438502749802234823348809893e+46', '-9.7e+3765', 2, 6);
    T('8.59178829873535373048133515224598857627026222132864599155220e+846557', '4.393270010069522807177e+93128237', '4.3932700100696e+93128237', 14, 0);
    T('-5.4496212966222363751793657695242120773e+78059', '5459663000607351853302347.524772873747622218969247434', '-5.4496212966222e+78059', 14, 4);
    T('-4.54252702168211e-70425268211693', '5.01475750598e-42274996187', '5.01475750598e-42274996187', 17, 6);
    T('6.070995627815996230717559077730e+663937507552807', '7.3492409930364522042927985237e-64902', '6.07099562781599623071755907773e+663937507552807', 41, 4);
    T('-7.3244167607e+13209539650', '5.5253126565533515299223143e-796099936922', '-7.32442e+13209539650', 6, 0);
    T('6.661181533578729080949276077768245793583441399991749680e+872271803379', '2.6416306076471843067600668147287105491761533958377e-66183469', '6.661181533578729080949276077768245793583441399991749680000000000000000001e+872271803379', 73, 2);
    T('4.1943258713072954922609910946972e+5279655', '5.6372698457618097782996776125356997470059578269682981e-284843', '4.1943258713072954922609910946972e+5279655', 63, 4);
    T('-6.5608210905343817756099087611682539908e+6517652701', '-9.08164150618897655443835154e+33', '-6.5608210905343817756099087611682539908e+6517652701', 85, 2);
    T('9.81850807368837483333e-146625011463', '2.116611482400609387573464861203640493781185393e+23572274', '2.11661148240060938757346486120364049378118539300000000000000000001e+23572274', 66, 2);
    T('-2.619949550840405947624596651678647e+843971', '-9.169849774518203629285914538582469106237463474959972914e+152', '-2.619949550840405947624596651678647e+843971', 95, 6);
    T('9.306924342410007261282370853317e+2952400060077', '9.2059484773755605254938971523405800263560639e-70831266874523', '9.306924342411e+2952400060077', 13, 0);
    T('9.76718e+59940169936', '7.576899093229127944091975730179156392528527693e+46862789766792', '7.576899093229127944091975730179156392528527693e+46862789766792', 93, 1);
    T('7.194421882553502708433e-5340036', '3.051138701480294729241158189723547e-144084173926225', '7.194421882553502708433e-5340036', 93, 1);
    T('5.425881535803826673997281080824986372725497614806e-1634875544007', '-5.639508469900e-386957657228', '-5.6395084698999999999999999999999999999999999999999999999999999999999999999999999999999999999e-386957657228', 92, 1);
    T('-8.028238142669061794185123620396018717865e-3274045942', '-6.73198033705983860679284781e+121', '-6.732e+121', 4, 5);
    T('-6e+1227050196', '3.756441252418462042525564158326583e-34', '-6e+1227050196', 10, 0);
    T('6e-151866', '-3.3975499541682579202486629411868311499364934114607023372798e+7262048256', '-3.397549954168258e+7262048256', 16, 3);
    T('-9.4926010780918790652779773512650550117554e+591', '8.54471693567795938e+6898650', '8.54471693567795938e+6898650', 37, 5);
    T('-5.3e+13877901', '-8.10593766007970680876909649053606393420446801e+724125', '-5.3e+13877901', 20, 6);
    T('-6.03647876423438593899768810779699459264924508066991196625816e-295532066541252', '9.737689e+333108601129', '9.737689e+333108601129', 14, 4);
    T('9.54578506972279282021120804269431738145220927551130e+40991161482279', '3.94087606682439203440447293e+1032230467', '9.5457850697227928202112080426943173814522092755113e+40991161482279', 76, 3);
    T('-8.44873933153128940611263e-5058377493', '7.22910679825463152709225753328504253418218189929908e+4294', '7.2291067982546e+4294', 14, 6);
    T('-1.8089792241377149285746700043091709277589562448082e+9394925', '8.721712934142281707262959847158960703615455430564515e+70', '-1.8089792241377149285746700043091709277589562448082e+9394925', 64, 5);
    T('1.135284831701572799689655e+184181857', '-9.36067e-32018695', '1.13528483e+184181857', 9, 6);
    T('5.822425627574294563543063667027e-83564598770945', '-4.01270157410747919471459065381326721569962937710104e-3240281757', '-4.01270157410747919471459065381326721569962937710103999999999999999999999999999999999999999e-3240281757', 90, 1);
    T('9e+1129184376', '2.244273590060053843173602172e+951979261', '9e+1129184376', 42, 1);
    T('-1.93440225149873140620746870790727879460018e-89307', '3.4225163884399456736777736282259117936101573102559219172e+87949421', '3.4225163884399456736777736282259117936101573102559219171999e+87949421', 59, 1);
    T('-5.031502064864e-5287686035140', '9.1335603083940037282992362753500514751058e+4041478194', '9.1335603083940037282992362753500514751058e+4041478194', 51, 4);
    T('5.539570735633842597295893712627875519e-6551618110883', '6.182940261988471944024695902318e+36509597102', '6e+36509597102', 1, 3);
    T('-8.582453234740828807894270e-51', '9.297100538992e+65410725816', '9.297100538992e+65410725816', 92, 6);
    T('2.51665339410475950154181344685021620011189307726913285e+451791', '-9.61119364e+47', '2.51665339410475950154181344685e+451791', 31, 4);
    T('4.876158893977885279e+91310290', '-5.70353791607592229998850259389276770740053302e+10206', '4.876158893977885279e+91310290', 89, 2);
    T('8.4273552641860391516558865968348743629164619469e-5240995', '-7.6159820187921075099437724477e+7794', '-7.6159820187921075099437724476999999999999999999999999999999999999999999999999999999999999999e+7794', 92, 1);
    T('2.8271672946687475e+321242', '-4.657985263824485354196286660583689470878078922200384691720e-97500096341', '2.8271672946687475e+321242', 58, 4);
    T('-6.377795007035864410e+86', '-8.75995e-66620723', '-6.377795007035864410000000000000000000000000000000000000000000000000001e+86', 70, 3);
    T('9.75e+146382', '-8.609379925003385447023e+261', '9.75e+146382', 59, 5);
    T('8.3551209484036361e+60152789354', '3403220848118678090002859935147.04399', '8.3551209484036361000000000000000000000000000000000000001e+60152789354', 56, 2);
    T('7.70985410353487326089620138465185353334707349085488993878e+8599939887', '-9455197163721297072887181183898108663295870', '7.709854103534873260896201384651853533347074e+8599939887', 43, 2);
    T('-5.435118933765142278193657593736895339e+5343624360', '0.00895', '-5.43511893376514227819365759373689533899999999999e+5343624360', 48, 2);
    T('4.51029619690806599605456446e-940286', '-4.30840834371426212306153631154839875844896296626219753e+6162384504944', '-4.308408343714262124e+6162384504944', 19, 0);
    T('-1.099670704779581e+91930701', '-8.645979159e-54', '-1.099670704779581e+91930701', 70, 4);
    T('3.986293708811648840087943113686870428039552e+209091023691376', '669847890110111397106085020067664934850844.900557843531', '3.986293708811648840087943113686870428039552e+209091023691376', 88, 4);
    T('-6.23137194071136462807922721942981624800850333261945114602e+960967587239', '4.97891881882325006363023300051298372770390720682643735e-31', '-6.231371940711364628079227219429816e+960967587239', 34, 6);
    T('-5.76946796149185e-41235054465034', '1.645768952555340493e+773788', '1.6457689525553404929999999999999999e+773788', 35, 1);
    T('1e-3366342', '1.09235e+7536804', '1.09235e+7536804', 53, 6);
    T('-3.3e-643962527773', '-2.1150528596214970458510336939579e-66185481', '-2.1150528596214970458510336939579e-66185481', 56, 4);
    T('8.697e+678159670', '-3.05968805250661810769006157685989119694e-49189', '8.696999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+678159670', 100, 1);
    T('3.6704515827010073689310765934594801030868301532e+7356', '4e-49855001803173', '3.6704515827010073689310765934594801030868301532e+7356', 96, 4);
    T('-4412028592579711299700612891995.13', '7.126529922428905465279471408336014637229735455148203e+65341585', '7.12652992242890546527947140833601463722973545514e+65341585', 48, 1);
    T('5.414785788723804555153322562731474028365e-21937378', '3.1429679511675498246995159705437456379287523607e+6897029', '3.1429679511675498246995159705437e+6897029', 32, 3);
    T('4.33705803899e-847866931', '8.041969868717862168744036088144864363515859297691911798e+10482589463', '8.041969868717862168744036088144864363515859297691911798e+10482589463', 63, 3);
    T('6.37441976235154846227339052929943967765224936e+15487', '3.87e+4568796052', '3.87e+4568796052', 35, 5);
    T('-4.603570766133680899363039657516100359e+496981', '4.19e+41531492143', '4.19e+41531492143', 75, 4);
    T('5.0137e+4391655726419', '4.776626620045844062857447048950939930809026e+5348', '5.0137e+4391655726419', 6, 3);
    T('4.742825620197019724592410094956339592346962157e+18103658798', '-1.6577746920293372593718571827e+56269710056', '-1.65777469e+56269710056', 9, 6);
    T('-7.505239735367385581025659755679286831e-248', '6.122872710755529948872856802258310e+7246214093150', '6.1228727107556e+7246214093150', 14, 0);
    T('6.492378566374244643805865958034577657946006992405e+890059475663', '3.8906459373001086538376483039530279e-70391', '6.492378566374244643805865958034577657946006992405e+890059475663', 65, 6);
    T('-7.52496453825642339390787054386484e+701', '-1.373738545481215e-2958633013', '-7.52e+701', 3, 4);
    T('-3.28319e+57', '6.58000941432258448806754e-9351648104', '-3.2831899999999999999999999999e+57', 29, 2);
    T('7.412154337671619246485420322800957e-4951067', '6.92388244043751035803e-3415865542', '7.412154337671619246485420322800957e-4951067', 50, 6);
    T('9.7793551693174665884956558175895565921084679555910e+72034580', '0.000482', '9.779355169317466588495655817589556592108467955591000000000000000000000000000000000000000000000000001e+72034580', 100, 0);
    T('-18615962730679375575640712.91', '-7.166014010e-388663', '-1.861596273067937557564071291e+25', 56, 1);
    T('-4.344403676905111773e+63419020558503', '-7.0758840e+87', '-4.344e+63419020558503', 4, 6);
    T('5e+37422600369835', '-6.08162396098233778884742554622392686406769154075e+359', '5e+37422600369835', 66, 2);
    T('-93901719418734606223499863648069655547.9911', '-9.281630578716900e-175', '-9.39017194187346062234998636480696555479911e+37', 68, 5);
    T('-1.54005147e-16617303427', '-2.714e+54308956493120', '-2.714e+54308956493120', 53, 6);
    T('8.14430320006343988896520654891682977373e+36236259522', '-1.0309779795412682e+25', '8.14430320006343988896520654891682977373e+36236259522', 58, 5);
    T('8.433968370778819439978500032222420202027715682963623e+266076932', '1.2806714186041476263579904865609480e-1548', '8.43396837077881943997850003222242020202771568e+266076932', 45, 6);
    T('-8.35557323193997631136e-487025', '-8.9226377083738e+8659142', '-8.9226377083738e+8659142', 65, 2);
    T('9.21636780022906393101e+6408575800532', '6.26634742689881973524e-747879947178', '9.21636780022906393101e+6408575800532', 55, 6);
    T('1.082876057330481e+306', '2e+17', '1.082876057330481e+306', 31, 6);
    T('8.33962702749750410699666690663825217391676900760045604051e-8249460020849', '-6.1308327497533747869287686808846e-66', '-6.1308327497533747869287686808846e-66', 34, 0);
    T('8.20207673115024833205e+216642034', '3.19724710107028948663220642157175612887612675539e-534364932', '8.2e+216642034', 2, 3);
    T('5.13825e-24', '-4.36760659196993230315935165409471019726228431226403996e+754361699085595', '-4.3676065919699323e+754361699085595', 17, 2);
    T('7.770294227414952979223e+126889133103435', '-8.55023e+544', '7.770294227414952979223e+126889133103435', 71, 0);
    T('4.51638942700830582788012607972449677265125734809370887215e+589', '-7.53006588683949923131604936496207622315888196253e+407300509', '-7.53006588683949923132e+407300509', 21, 0);
    T('-3230996853106619.9', '8.967825076684310537064336634629051038e+31326201', '8.967825076684310537064336634629051038e+31326201', 87, 5);
    T('5.40830112074451952678234237713922059096e+5735059046887', '7.738149e+68687894', '5.4083011207445195267823423771392206e+5735059046887', 35, 2);
    T('6.041946079677082641514695600936093e+734', '79111650.79903294', '6.041946079677082641514695600936093e+734', 100, 3);
    T('6.012229170437e+14', '-3.5e-82874063222171', '6.0122292e+14', 8, 4);
    T('-6.4564717068918e+27329184', '6.54975061889247168531231387529391615809745e-57990746174604', '-6.4564717068918e+27329184', 95, 5);
    T('-7.61725e-729', '-8.9245271239731e-8630937634454', '-7.61725e-729', 57, 1);
    T('3.340131846288881e-43', '-8.64629079496020084693156154532204386530e-978631', '3.340131846288881e-43', 38, 2);
    T('4.90e+146', '0.510', '4.900000000000000000000000000000000000000000000000000001e+146', 55, 2);
    T('4.9364686744415511589512501320e-80956', '-1.3994e-101', '-1.39939999999999999999999999999999999999999999e-101', 45, 1);
    T('-7.971653237305634082713800363674102074242003385078e+57', '-4.348786873637107699017019945873765759292246378568e+53', '-8e+57', 2, 3);
    T('3.2387848958e+6614', '-7.1740684726967998199183474630073232593173276e-4471539', '3.2387848958e+6614', 27, 6);
    T('-2.82396662587553721197127817702214e+1006714348743', '-7.32224814565e-9011930619149', '-2.82396662587553721197127817702214e+1006714348743', 42, 5);
    T('6.9144518267160142944443310887371361e-823182465818', '-6.418631432442079921516301049416256393490e-977198', '-6.41863143244207992151630104941625639349e-977198', 82, 3);
    T('257620410891903918464038647948567117592401892844877865', '-6.951568506959581269991173608310429480350893353367832063883e+73936', '-6.9515685e+73936', 9, 1);
    T('-1.80222390779464899835490920477176e+686819', '5.7769666465031855135931031630518767950954212122593203330e-8802548376639', '-1.80222390779464899835490920477176e+686819', 49, 5);
    T('3.867308419474474564157726267472426690420304e-389', '8.16103803907505821003345446856174386626791779536928303205212e+188711', '8.1610380390750582100334544685617438662679177953692830320521e+188711', 59, 1);
    T('-9.931543414360361073901607490e+795263', '4.904428259794376546897314306004135251795e-38', '-9.9315434143603610739016074899999999999999999999999999e+795263', 53, 1);
    T('1.54938088062332032064116956750813891001023115313609e-97385', '-6.795454867903e-656095149193621', '1.5493808806233203206411695675081389100102311531360899999999e-97385', 59, 1);
    T('-5.94154009027337109907978e+851358', '6.7359408288370120063956e-26024', '-5.94154009027337109907978e+851358', 89, 3);
    T('6.33657725408015155758207622509939369503200805704e-9447390', '-5.7e+291254348359', '-5.7e+291254348359', 28, 3);
    T('6.97e-86690250879', '-2.51716480285e-79512477395', '-2.5e-79512477395', 2, 6);
    T('-8.0157976691081453783082536816805e-7396610', '1.90942e-5612088776598', '-8.0157976691081453783e-7396610', 20, 2);
    T('-5.440946210132168409738680251221902084e-284213334949135', '9.1774878530862e+12227868131', '9.17748785308619999999999999999999e+12227868131', 33, 3);
    T('-1.6704307086701389026834196e+56', '6.932890835682213194944456282773633479954417e+731316', '6.9328908356822131e+731316', 17, 3);
    T('-8.16585520372986534464528952e+705435', '6.08232e+87079752', '6.08231999999999999999999999999999999999999999999999999999999999999999999999999999999e+87079752', 84, 3);
    T('-5.8123862332795802702518161054497106380474e+6549381556', '-2.59072384801008015217110176031414703114619834183880589644e-15303718180210', '-5.81238623328e+6549381556', 13, 3);
    T('-2.0e+17118871935639', '1.115879743590789033318538245492112956329417e+695837', '-2e+17118871935639', 79, 0);
    T('7.6678694817381e+20', '-8.2940452372277e+349097647', '-8.294045237227699999999999999999999999999999999999999999999999999999999999999999999999999999e+349097647', 91, 1);
    T('-2.987524469411999381793071515657239784785462e+64534562', '-3.8725576863739386135746843520115467464229762922953015e+5815315522138', '-3.87255768637393861e+5815315522138', 18, 1);
    T('-4.77981866518065302899622401e+508607', '3.141508329605633788814377171605e+20760451', '3.1415083296056337888143771716049999999999999999999999999999999999999999999999999999999999999999e+20760451', 95, 1);
    T('6.10229175e+9853', '-6.446233193993271374e+19', '6.10229175e+9853', 94, 6);
    T('9.09678754e+265007', '-2.4909793076228302901473423486161357485794987419572287e+123', '9.0967875399999999999999999999999999999999e+265007', 41, 1);
    T('4.140310283418358196351e-31079158334', '5.77500964371133230481362293226891e+415962018102', '5.77500964371133230481362293226891000000000001e+415962018102', 45, 2);
    T('-6504843961046938073507763718485324065234467794665.0433209', '-6.537794873350134578138381971300229771e+41', '-6.5048446148264254085212215323235211952574448946650433209e+48', 96, 0);
    T('-5.6207619104084490e+40', '-3.043009103445953019602478243469945328e+58', '-3.043009103445953025223240153878394328e+58', 73, 2);
    T('3203777421068488834339700119860520295265.510', '3.948018338412209560765431291220350207051491e-8979035273', '3.203777421068488834339700119860520295265510000000000000000000000000000000000000000000000000000001e+39', 97, 2);
    T('5.95524454912687e-944857064842508', '-4.930463733559732851957188204484e+4641811436', '-4.930463733559732851957188204483999999999999999999999999999999999999e+4641811436', 67, 2);
    T('-8.90607060681755547627369780529953785302059997939308605605598e+36836', '-0.009', '-8.906070606817555476273697805299537853020599979393086056055980000000000000000000000000000000001e+36836', 94, 0);
    T('-7.4753843151189459e-382142250327681', '1.010287249217347e+56132', '1.010287249217347e+56132', 56, 2);
    T('-6.47e+84296390', '-4.3704930190e-12', '-6.47e+84296390', 77, 1);
    T('-8.62504563282395096455578578561574859883979643478200401008e+72823925656', '-6.5440719326509539396235929616713950970334583918e+771743715', '-8.6250456328239509645557857856157e+72823925656', 32, 1);
    T('3.914945027681022698768e+78973609613', '-9.5303695935205451157822912390014567352788866e-16725776', '3.914945027681022698768e+78973609613', 26, 4);
    T('5.951822159590064763493098577e-3594997512271', '7.272511387052878591e-35150293110', '7.272511387052878591e-35150293110', 67, 6);
    T('-4.0105613412153527984335944207385119969941701e-902772489820462', '2.0934863752343024729267819e+8102766', '2.0934863752343024729267819e+8102766', 29, 4);
    T('1.1374006761908161175187e+58094098729', '1.144742809800e+3917', '1.1374006761908161175187e+58094098729', 31, 6);
    T('-1.67402179729696618822983542002145251745e-427425674976', '-8.807113083544112265843956283968783551e+15780', '-8.8071130835441122659e+15780', 20, 0);
    T('4.9794208197e+956874', '2.481778008249353860092484485981e-965774', '4.97942081970000000000000001e+956874', 27, 2);
    T('5.003787500933577222835110325440880e-4214466', '5.617913211327249528752e-46561', '5.617913211327249528752e-46561', 96, 3);
    T('1.448436445514e-5853909906760', '-4e+5', '-4e+5', 68, 0);
    T('2.4114409841244364508197538278668e+205244982', '8.39397801769072099799647155107431255896957161085709e-41110', '2.4114409841244364508197538278668e+205244982', 96, 3);
    T('-3.602218577285260736673460978549840099742176689876241e+8223397', '-2.034921524205031304768053831049434457042753160788451e+64', '-3.6022185772852607366734609785498400997421766898762410000000000000000000000001e+8223397', 77, 3);
    T('-9.8840025807987382578796221197744372842747921603729601503959e-65292629670', '-5.390834013356875e+548620784673524', '-5.39083401335687500000000001e+548620784673524', 27, 3);
    T('-8.80041100581537806e-508773798247021', '9.4564561127423611420352e+60923083', '9.4564561127423611420352e+60923083', 49, 6);
    T('1.187864994924263871614808605801985659024576460489e+882923575', '-7.674353077242403789151759388150583e+4111653', '1.187864994924263871614808605802e+882923575', 32, 5);
    T('9.77027257221030750177657138918898197512504e-64419054', '-9.080631255047238432998742339351273712953753195191e-275896', '-9.0806312550472384329987423393512737129537531951909999999999999999999999e-275896', 71, 1);
    T('-2.6829896981660974168617225747534650835649041678e-684814945', '-3.3887802216609025829943899665894501395e-42677177890050', '-2.68298969816609741686172257475346508356490416780000000000000000000000000001e-684814945', 75, 3);
    T('1.56237618368018556974165494288356936868030554343619397e-3568228', '-1.276046969771896374089440105569e+620', '-1.276046969771896374089440105569e+620', 35, 4);
    T('-8.2470330515200989754411787917888375364902185054325649551e-780', '1.39660e-89186326', '-8.2470330515200989754411787917888375364902185054325649551e-780', 99, 0);
    T('-119.020', '-7.669311420902148006374991170918391309753117e-5398834', '-1.1902e+2', 26, 4);
    T('1.82609976e+53565', '-4.7343438728486830990308914e+438001524756305', '-4.7343438728486830990308913999999999999999999999e+438001524756305', 47, 1);
    T('9.7407418288702238937625555369830908071e-911965853154', '-1.6492942e+33150', '-1.6492942e+33150', 16, 6);
    T('9.0791236751396870160718104362674319961198801771e+9232600576168', '5.78176299375174893092e-2268175009', '9.07912367513968701607181043626743199611988017710000001e+9232600576168', 54, 0);
    T('2.875410370683616089424557e+178080884', '5.49851692023643742323777146149927254e-391216', '2.875410370683616089424557e+178080884', 96, 6);
    T('2.329619270627493480330e+480162790589502', '-5.532402747977579851735960843259706305286e-53594283751', '2.32961927062749348033e+480162790589502', 65, 6);
    T('-4.37475826209258375121357964454160054255497e-55664', '1e-188538601', '-4.374758262092583751213579644541600542554969999999999999999999999999999999e-55664', 73, 2);
    T('8.117662207241786914287096104096150056976768961194732415554e-2900966919', '2.045710267674117403653916033366163691e-1416035458599', '8.1176622072417869142871e-2900966919', 24, 4);
    T('-2e-126334538451', '5.1135296825691051059988292325040e+642597', '5.113529682569105105998829232504e+642597', 96, 6);
    T('7.22412302097368424e+347173', '-3.665150541133259649897359709381e+48690', '7.22412302097368424e+347173', 60, 2);
    T('6.02767478306647551193864e+5976361674', '4.640292928020763637192365782286452953459178413711222901e-271947686287', '6.02767478306647551193864e+5976361674', 67, 3);
    T('1.0288250189483023700717573265965728781819832270e+126', '-8.8215033769180275396712803073211736047123986386499e-479572229958159', '1.028825018948302370071757326596572878181983227e+126', 87, 0);
    T('-9.56624935329459690788942972379106524120986001301969204959e-7859922', '-6.31835078911e-565116632', '-9.566249e-7859922', 7, 5);
    T('187415.4053731', '-5.5080061873476947674527036629489530390624949202049504673e+54995458410', '-5.50800618734769476745270366294895304e+54995458410', 36, 3);
    T('-3.42116465127932980573824166354809031403958349705171688756e+38665500929', '1.0455333426717598259581805316242175e-7167', '-3.4211646512793298057e+38665500929', 20, 1);
    T('4.9723653706477339351395991577120750169698167338923e+32755730670438', '-4.5480110861390458270231872296697e+44105639', '4.9723653706477339351396e+32755730670438', 24, 6);
    T('-2.17217148672233075431589856170209711236e-1711', '-3.37599609185432317013190237252150698608033e+960178855722321', '-3.37599609185432317013190237252150698608033e+960178855722321', 45, 5);
    T('9.38861254412192811847212618670128307736e+273031342559', '-7.97727281600346272484507367669097954759853933083985e+5836123', '9.38861254412192811847212618670128307736e+273031342559', 67, 2);
    T('-8.391461941474575036927909984461301e+561', '6.39408755116629875496293297e-18038580609894', '-8.391461941474575036927909984461301e+561', 60, 4);
    T('-6.3252276521669107564556e-165829629264', '3.962585028551400801078112559173798342e+654762578021', '3.9625850285514008010781125591737983e+654762578021', 35, 4);
    T('2.688078930944315591147146003812027046e-20417025', '8.9077147066773989433512445579891528214404e-95005191454', '2.688078930944315591147146003812027e-20417025', 34, 4);
    T('1.3138202188083826056640734618287653538745544273e+197084591894', '2.7435199620080067494309207215359547067894e-1609765', '1.31382021880838260566407346182876e+197084591894', 33, 3);
    T('-1.7956393950211786600552537625941689457730110866796205e-2428403', '-7.919586917231080563650306470442562131095243810201089e+184826599745292', '-7.91958691723108056365e+184826599745292', 21, 4);
    T('7.17769599530036e+480', '5.4463369719456228531373196518252846e+2381229', '5.44633697194562285314e+2381229', 21, 6);
    T('6.505e-44141457', '8.25813579223787683810659092658762242866760975703257109718e-353', '8.25813579223787683810659092658762242866760975703257109718e-353', 87, 6);
    T('-7.465636601154328466620499e-657949353121', '-8.3871458824665212602259260020021140e-1915924', '-8.387145882466521260225926002002114000000000000000000000000000001e-1915924', 64, 3);
    T('-9.4436123235598129535445260957302093378061033937866e+7111476504284', '7.940754814839404664497958433e-4416', '-9.443612323559812953544526095730209337806103393e+7111476504284', 46, 1);
    T('-1.125549262286253499513148913900184768e-893646266148456', '-3.5722438962932029460294302879e-609225810', '-3.5722438962932029460294302879e-609225810', 48, 6);
    T('9.07851661288045743505448774623129561388764781294407811477e-679', '9e-622', '9.0000000000000000000000000001e-622', 29, 2);
    T('5.751495353668869308789462e+8902279891', '3.364446975602e-486698412215', '5.7514953536688693e+8902279891', 17, 5);
    T('8.336940384000713690161347769835522322819002108115341259699e+8593', '5.84244852372046707849677018821557002e-16206736452716', '8.3369403840007e+8593', 14, 6);
    T('-1.9818946519564182018995330319972621552799285060935520800e-163143889659', '-9.004e+12', '-9.004e+12', 65, 1);
    T('4.26175812737140592e-206367', '4.373434071295654219346787596200752168664607484725976493771e+2387991177', '4.373434071295654219346787596200752168664607484725976493771e+2387991177', 77, 6);
    T('4.0098626211101556874962654383636496626897665310612619680e+883443304', '-9.2154328858264320182e-3330337038', '4.0098626211101556874962654383636496626897e+883443304', 41, 3);
    T('-3.07791849801308081981894281234114e-397307145876', '9.83715e+3260003619359', '9.83715e+3260003619359', 27, 6);
    T('-7.39e+8608359', '402056652503119369765852764423444064825447029202661224.91', '-7.389999999999e+8608359', 13, 2);
    T('8.801792094161084853649910631950056637819e-8200958486557', '3.605279757164613349683177e-9668375', '3.605279757164613349683177e-9668375', 43, 3);
    T('-4.2243216512651e+276417648', '5.670497804561033965806954e-22798', '-4.22432165126509999999999999999999999999999999999999999999999999999999999999999999999999999999999e+276417648', 96, 1);
    T('4.48e+4', '-404639488548364.36167099', '-4.0463948850356436167099e+14', 88, 0);
    T('-6.273310930400089059647012704989651148906499278078837949e-47315872', '-8.0836917569074245858533631331123936e+6033053', '-8.08369175690742458585336313311239360000000000000000000000000000000000000000000000000000001e+6033053', 90, 3);
    T('-8.60661448273631714285918738e+1250162566', '5.87861165683997035022455984444349898625073906602e+25275', '-8.60661e+1250162566', 6, 6);
    T('-67338950036275.703230337173620581023358261335', '7.0641027024558057034333380115695296e+298010052887', '7.0641027024558057034333380115695296e+298010052887', 72, 4);
    T('-4.8460553915336619583056593e-98419', '-3.7581133726095e-834601418', '-4.8460553915336619583056593e-98419', 84, 6);
    T('-1.06163852342144627e+928787818', '4.548773316877136592263655784160416439641865524508349860808e-77435570225', '-1.06163852342144627e+928787818', 32, 4);
    T('-7e+67', '3.9962408649287941214424417169827407524175447380697534649070e+2068933678', '3.99624086492879412144244171698274075241754473806975346490699999999999999999999999999999999999999e+2068933678', 96, 1);
    T('-3.592544219221953130373924438597380632e+194354343', '46951000596002987731239.7098731825598110222104302411209369192', '-3.59254421922195313037392443859738064e+194354343', 36, 3);
    T('6.2725629630279627592713652995e-86931528797952', '2.93328e-46224512', '2.93328e-46224512', 84, 1);
    T('2.946096683173463409561630961480695685081282921e+648269107601928', '2.3248437574797193424340e+90', '2.94609668317346340956163096148069568508128292100000000000000000000001e+648269107601928', 69, 2);
    T('6.722256355294485012736112995e+9052053', '6.688166223112104279021534047491934587425850290488012240126e-307969542015546', '6.722256355294485012736112995e+9052053', 51, 5);
    T('-0.0009959060757906905322374788621331220667055347', '2.6592374649915172448385826468845e-25866734441', '-9.959060757906905322374788621331220667055347e-4', 91, 6);
    T('3.0816729018058e-3801', '-8.7742e+3236854799', '-8.7742e+3236854799', 76, 6);
    T('7.59650073442607523695973064193e+2122644212', '9.97288364242850680114065701129077174531224732e-7821', '7.59650073442607523695973064193e+2122644212', 50, 5);
    T('5.0200762259677200511335915116e+28397913', '8.62207296709170183016e+13342304', '5.0200762259677200511335915116e+28397913', 66, 6);
    T('8.1648756894625680519658650460032711e+84', '9.209088926601712520818637901265342680002365190071599e+2360226265963', '9.2090889266017125208186379012653426800023651901e+2360226265963', 47, 6);
    T('-9.431692e+12', '-5.152673243730935492235226190653341436e-15095975440', '-9.43169200000000000000000000000000001e+12', 36, 0);
    T('-7.572462482802806414117359976728504422063353504048977994e+7599451299736', '8.301408565903355351088670291334711131553152834554025285e-338354', '-7.572462482802806414117359976728504422063353504048977994e+7599451299736', 79, 6);
    T('-6.91653168387349831094132096174147e-541350288538092', '3.362518396e+1974', '3.3625184e+1974', 9, 0);
    T('-5.40671104904e-889751735667658', '7.437376672175335604233494727964045693843477085233e-111109910411', '7.437376672175335604233494727964045693843477085233e-111109910411', 73, 5);
    T('3.83973029709092847754280101156288169e+36', '-5.5978759651317710161498078928308933178364349004543573218169e-97678071674211', '3.83973e+36', 6, 3);
    T('-7.4982348081406e-60410952133112', '8.14843627772316638123e+167833827656', '8.14843627772316638123e+167833827656', 70, 0);
    T('3.3239731504024906697255e+868904416881', '5.81894683892348989625338104089e+9459763230617', '5.818946838923489896253381040890000000000000000000000000000000000000000000000000000000000000001e+9459763230617', 94, 0);
    T('-9.029344295468094400666291479329975264322858832050040e+835', '8.47547396330809522223797298188776149492854525994264948e+5460', '8.47547396330809522223797298188776149492854525994264948e+5460', 83, 6);
    T('-9.05790681165470341007807824205038837167879947343631e-977834236504', '-6.1810890302829762302357704576948443458877102537e+36172798372', '-6.1810890302829762302357704576948443458877102537e+36172798372', 87, 5);
    T('-5.836e+18', '-2197826115.63', '-5.83600000219782611563e+18', 56, 2);
    T('-2.6384859517360e+76', '-2.81e+4', '-2.63848595173600000000000000000000000000000000000000000000000000000000000281e+76', 79, 6);
    T('8.168671766669970330e-4453', '7.08631897776846087e-9266750', '8.16867176666997033e-4453', 87, 5);
    T('4.269e+4468872', '-4.83894444771821822438805937076888213580934488762284313027275e+321075', '4.269e+4468872', 61, 4);
    T('-5.962379157540273633320649941026328e-45488719', '-5.136515388167755146555059970122104195394581477375206108e+395', '-5.137e+395', 4, 0);
    T('1.6006263709351e-34500242428', '-1.1106633976186005190454300014620370942078034e+23260054710', '-1.1106633976186005190454300014620370942078034e+23260054710', 77, 6);
    T('-6.78542446969912924312766810099138076693915e-6085', '-7.837635541849e+98984213389', '-7.837635541849000000000001e+98984213389', 25, 3);
    T('8.350256e+10', '6.975311868568611621204918448e-5109', '8.350256000000000000000000000000000001e+10', 37, 0);
    T('5.841838570359981554628213946755705040e-5275908927106', '-8.6474002101011427221244471023162859246e-222556130110105', '5.84183857035998155462821394675570504e-5275908927106', 43, 5);
    T('-2.42853817017954614323099336476560088714987521792e-628797605', '-9e-41449662', '-9.000000000000000000000000000000000000000000000000000000000000000000000000001e-41449662', 76, 0);
    T('9.039160142068553127643578395e-85085895457', '-4.127e+7934803660', '-4.1269999e+7934803660', 8, 1);
    T('-1.1547109880247584961544786590729281e-98274841551', '-4.899790887723052662303058008802957867682877644e+4638788284', '-4.899790887723052662303058008802957867682877644e+4638788284', 63, 1);
    T('3.61e-856069', '-4.4545110881e+374268804739911', '-4.4545110881e+374268804739911', 91, 4);
    T('-2.689084858082323765091107960857850545660842424901684e-619154', '4.99127225342151764924698892082307941e-9747522954250', '-2.689084858082323765091107960857850545660842424901684e-619154', 57, 4);
    T('-3.210764512547188114097569569957501109184566013610171e+819656', '-1.8201569638302455817101389673e+4859', '-3.210764512e+819656', 10, 1);
    T('-9.036514559952059840913693477985526881360738156474e+3744726382', '-8.784046774178594466050315861429519768475497211951064e-1060', '-9.036514559952059840913693477985526881360738156474e+3744726382', 69, 4);
    T('6.7724744663641553231363662302880449273528647937108775826e+64525', '8.3006496277520474e-900258159740', '6.7724744663641553231363662302880449273528647937108775826e+64525', 88, 5);
    T('3.786779526455266962332795383510543e-3578215401177', '74156613447311606646131877134907363600969524961610439885.3', '7.41566134473116066461318771349073636009695249616104398853e+55', 74, 1);
    T('-5.41581461723970775591172234137892081043e+42', '-3.21159958785635961e-568658725', '-5.41581461723970775591172234137892081043e+42', 70, 6);
    T('8.22114034826796574782868210328910e-163573013', '-1.0255103221166884202006231554907e+6410', '-1.0255103221166884202006231554907e+6410', 85, 0);
    T('6.104430086780407071244779815592455827765439395480e+357277979201', '-6.569480459255899972454802480951417869481585729457718670e-266151', '6.10443008678040707124477981e+357277979201', 27, 1);
    T('9.6831697330673692e+6587', '8.386968477132726613337992003470773195470000530062999045e+620', '9.6831697330673692e+6587', 98, 3);
    T('2.26345332952674012515e+340', '-6.7248322976241710901141715321830065e+5303', '-6.72483229762417109011417153e+5303', 27, 5);
    T('-5.324928773021509e-46905', '-7.0779167855626781e+62534', '-7.0779167855626781e+62534', 46, 6);
    T('-2.6357745643820651539934416977056e-77625885139', '855284669601670.7268517', '8.5528466960167072685169999999999e+14', 32, 3);
    T('7.168349939826379378735284851e-209841707578334', '-3.1284061505e-75585', '-3.1284061505e-75585', 60, 6);
    T('7.13967586227175986108055225087707036370248967217416551640e+382082', '7.003842065372809e+367253', '7.13967586227175986108055225087707036370248e+382082', 42, 3);
    T('-2.891969044913e+9009237116', '7e+38878', '-2.891969044913e+9009237116', 72, 5);
    T('1.80470289445012376474167925e+9883896580881', '7e-3083126806599', '1.80470289445012376474167925e+9883896580881', 89, 4);
    T('8.48921343167778525464928229200077649153925e+17806412', '-6.171093848431032416122853912440413e+90466694520', '-6.171093848431032416122853912440413e+90466694520', 44, 5);
    T('-4.747497457e-9523439', '1.8528326648603516426625008336566e+71333234020', '1.85283266486035164266251e+71333234020', 24, 2);
    T('-8.80902904587653332533361070470956980023428e-140828378', '4.03727652936208351793501724e-49', '4.03727652936208351793501724e-49', 67, 6);
    T('9.9528184769627333346e-628587579', '-4.127734994481281182960403355755184859520172509396983750e-49673', '-4.127734994481281182960403355755184859520172e-49673', 43, 2);
    T('5.7221963783938658225858725231597351433616873739745737375363e-8697', '-7.73711629801344918200896401091633198842642729971087219213110e-32839398810', '5.7221964e-8697', 8, 5);
    T('-8.655722428996387615142419486890762166908713364231079736e-305', '-6.6762247050409006796754618e+474462973', '-6.6762247050409006796754618e+474462973', 30, 4);
    T('-9.7037991129375073561451849692447027e-9281825083356', '-4.6773844907454646477153e+5060820408', '-4.6773844907454646477153e+5060820408', 45, 5);
    T('-74.989150', '-2.76487682125811718408773760379560575122890e-521815690960653', '-7.49891500000000000000000000000000000000000000000000000000000000000000001e+1', 72, 3);
    T('5.0072290504375353574247120264e+3661', '9.9996472127233754199737643556652315655e-664426363', '5.0072290504375353574247120264e+3661', 85, 3);
    T('8.38163506655772642149768271343576782238074282056823e+54', '9.0609354347251896053709e+81800840', '9.0609354347251896053709e+81800840', 76, 6);
    T('-6.76029139324789396338016889354089120216162336421e+6768636', '8.833845574732238305279439765021709209694685450871399418707e-2653486463', '-6.76029139324789396338016889354e+6768636', 30, 4);
    T('2.565854046048010676926086113377499030e-25353144927', '-3.9838111121656943788783671387952531131668176037e+434', '-3.98381111216569437888e+434', 21, 0);
    T('-5.975313382728947478752009536811197807353943123591e-9058', '9.87605501706963136462e-517156', '-5.97531338272894747875201e-9058', 25, 0);
    T('-9.901892285160000619232689228e+26957543091078', '3.82212621764752356314384982484114533e-24326016121', '-9.9018922851600006192326892279e+26957543091078', 29, 2);
    T('4.998614465345552682665847105958779990647430709615930703628e-772974817', '7.223346974460770060695958012297052e+82055647', '7.223346974460770060695958e+82055647', 26, 5);
    T('-5.44955663108506465e-30657809101', '-5.1342156056324996e+704966467391570', '-5.1342156056324996e+704966467391570', 63, 2);
    T('5.51856269229284e+6303459355', '9.5059406875984899276171531666367572e-6469', '5.51856269229284e+6303459355', 30, 1);
    T('1.8281791944541649614e-6223748789037', '-5.1398157577207132284521281119237556785268952093280798698e-4945714439', '-5.1398157577207132284521281119237556785268952093280798698e-4945714439', 69, 6);
    T('-5.33485563656003798507470923825e+34', '-4.62035633779460273184637974403402273898588996687289226e+17785377250', '-4.620356337794602731846379744034022738985889966e+17785377250', 46, 1);
    T('-5.67133491652760752509684980196856185178e+571813893259869', '-6.98757666810820532088866313635060793138108629569663e-878496109658367', '-5.67133491652760752509684980196856185178e+571813893259869', 69, 6);
    T('-4.3279379583379370820680875974116648735691910e+634299164', '-4.2529284971554812997403739437430987201585444e-41', '-4.327937958337937082e+634299164', 20, 1);
    T('-42109363873184079.249532', '-3.19797052745191064113554556219535525278918669719388576849e-203051311', '-4.2109363873184079249532e+16', 74, 6);
    T('-1.248672707783174487579641334680497963e-44', '9.4921e+7487000', '9.4921e+7487000', 57, 2);
    T('1.00105706403415289238047057411839745744281e-703007', '4.45580126720843353304e-150745526276', '1.00105e-703007', 6, 3);
    T('-9.413498384864174685548213e-5916812689480', '-5.80299877260468e+40441165827989', '-5.80299877260468e+40441165827989', 41, 1);
    T('1646107.9735', '6.3796289157747091051985971616846746250e+14333556293754', '6.3e+14333556293754', 2, 3);
    T('4.3913544478297188633880753167966634178833261126246181e-97280744147', '-3.098681458552122312217895188e+70', '-3.098681458552122312217895188e+70', 60, 0);
    T('9.04865805495041993267097e+5891931', '5.4771298046393796147106725099330e+9560302978844', '5.477129804639379614710672509933e+9560302978844', 75, 3);
    T('997106172208398327106482558445217126177440000785226', '9.39421775e-799359234630393', '9.971e+50', 4, 1);
    T('2.433497527741585177644585319120524e-66613287609', '-2.733768224968063360750929207825880520891e-50168691339396', '2.433497527741585e-66613287609', 16, 5);
    T('9.7212040329813632322291791e+33', '1.06357355472010817115215056920850320453566639289921e+1129', '1.063573554720108171152150569208503204535666392899e+1129', 49, 5);
    T('-2.754467057455180356993844664116619e-43841447394643', '3.692817414725316494459171880062444533801991489e-869729304831525', '-2.754467057455180356993844664116619e-43841447394643', 62, 6);
    T('-9.011272374071551567639040240238211919075364748789e+141', '-7.7410794798581783858041767664943933578529e-299817', '-9.011272374071551567639040240238211919075364748789e+141', 100, 5);
    T('-6.504080470829978327654363413900722132247e-5579654137', '6e-85199', '6e-85199', 72, 2);
    T('-6.1372602516552424e-93289789756', '6269862425211284450853863382135534394813733037818977.32613330', '6.2698624252112844508538633821355343948137330378189773261333e+51', 66, 2);
    T('-7.59639976e+629104200616541', '-5.3920386459835845264692197559312359397136297417328683e-257992', '-7.59639976e+629104200616541', 68, 5);
    T('5.215085215217084883819681563275354539480509260501817662993e-3604712883864', '-87600865553872498929.268894', '-8.7600865553872498929268894e+19', 59, 4);
    T('4.9685500761064632644602136392e+31', '1.118e+68711310706', '1.118e+68711310706', 40, 5);
    T('-2.56411918426718499190639481043685106766666548264158e-152670', '1.45855284335227277339971246104692028315638745272289757535225e+287454939314191', '1.45855284335227277339971246104692028315638745272289757535224999999999999999999999999999999e+287454939314191', 90, 1);
    T('4.222e+51090', '1.2005733810487852959902854175558589456331816491764733641843e+466489342442', '1.2005733810487852959902854175558589456331816491764733641843e+466489342442', 86, 5);
    T('-2.2309112409934720365708172670e+950778830', '-8.40238955217915e-48072', '-2.23091124099347203657081726700000000000000000000000000001e+950778830', 57, 3);
    T('-5.2932588595495492733924737438304482528e-27083688765540', '2e+7859', '2e+7859', 80, 6);
    T('-4.9e-82749', '2.23162520456077278964626967235965289304611e-6361654', '-4.9e-82749', 16, 4);
    T('6.5138111333690837806873124357627769863948904e-8788', '3.748479032344649368498102374540553112817184183840636e-676470', '6.5138111333690837806873124357627769863948904e-8788', 92, 3);
    T('4.6332872260682845449e+3239740319251', '9.96089947595100759677604352495911398956673507611797604753451e-2294961', '4.6332872260682845449e+3239740319251', 60, 3);
    T('5.7116374401538386324172677e+39367', '5.45046325e-170162456', '5.7116374401538386324172677e+39367', 53, 6);
    T('-47163309467377934695366448.712', '-9.805077109981176627097610096471989412854675935045639964e-891', '-4.7163309467377934695366448712e+25', 77, 4);
    T('5.31953386e+76168362302', '-1.86840879955642403e-211782567789103', '5.319533859999999999999999999999999999999999999999999999e+76168362302', 55, 3);
    T('1.26223953481902377917658417e-7443308295733', '6.0853571173399114905477933271661103356064311587e+269618776112', '6.085357e+269618776112', 7, 4);
    T('4.605819e+1918413', '6.064165827706304048164583567570553180650382631717e+293312869642168', '6.06416582770630404816e+293312869642168', 21, 5);
    T('-2.7746123607459e+514392', '9.8697977745454172045688047710062479077900744967088822854e-914348953', '-2.7746123607459e+514392', 27, 6);
    T('1.283700925723256657455999672060512706370433217597137179113e+404176562391', '-4.2856155389e+35818444845698', '-4.2856155389e+35818444845698', 13, 3);
    T('-1.403694690937617159618778052e+6063057', '-1.7264587586e-7079656795399', '-1.4036946909376171596187780520000000000000001e+6063057', 44, 0);
    T('4.337196740929092682138233749956498910657e-52624423888', '6.404712009901028324826e-47636209484', '6.404712009901028324826e-47636209484', 33, 5);
    T('-8.893423210332515004657315925684843711722222269291251e-819155147993415', '9.22349758297e+59271482652628', '9.2235e+59271482652628', 6, 2);
    T('-4.7565959551346185331536389e-234391977302', '-6.5147667647079592952175e-1530', '-6.5147667647079592952175e-1530', 47, 6);
    T('-2.6224302226248468220166e-744623723912000', '3.3516536715529749488217433991421426418305409e+353296122750364', '3.3516536715529749488217433991421426418305409e+353296122750364', 93, 5);
    T('2.7244941901136584591311916732363763614317154734299026035e-9765', '-4.2532497286173451988266245936e+163986889743', '-4.2532497286173451988266245936e+163986889743', 29, 0);
    T('5.4239660453873299677e+4601938', '5.211813655807488905166968042811616694822010996137752676e-95551433', '5.4239660453873299677e+4601938', 50, 3);
    T('-7.35633209586716759133107729364959899533e+95604615135', '-3.78e+9012504', '-7.35633209586716759133107729364959899533e+95604615135', 44, 1);
    T('7.840415666780471047369002621304237353e-516451211556', '-5.76921088759695414628933766e+20435', '-5.76921088759695414628933766e+20435', 29, 0);
    T('-3.84406300053e+880365328330', '4.15870e+32878228187', '-3.84406300053e+880365328330', 15, 4);
    T('9.738228092908e+272124324919870', '-85390513723.69347832496950802031', '9.738228092908e+272124324919870', 88, 4);
    T('7.29489354745462474583148635377259058233950436503305913542487e-6902363787', '-9.406604971011345841058694817187315161252187668547899e-43820956', '-9.406604971011345841058694817187315161252187668547899e-43820956', 74, 5);
    T('-98517351739972591119746553274877097571040427259444042', '-2.8461992836e-5734', '-9.85173517399725911197465532748770975710404e+52', 42, 5);
    T('-4.473811586031739008743274649116671e-37776735140', '-6.428494507330607109347961250360242795891868397832879962298e+868451926', '-6.42849450733060710934e+868451926', 21, 2);
    T('8.528981636395027203633619632882531296341598977634465257e-41449621', '7.86112086e+36046', '7.86112086e+36046', 35, 5);
    T('9.9866440821889516293363149531600630589263e-6894872093', '-4.74337066153132841e-96326190813', '9.986644082188951629336314953160063058926299999999999999999999999999999999e-6894872093', 73, 3);
    T('-3.8820565750923580406073007915344616188408040932303e+577361015097812', '6.68836635668363138497247001443724234931074248e-1268847986736', '-3.8820565750923580406073007915344616188408040932303e+577361015097812', 97, 3);
    T('-4.5704876820534015009123053722758748e+496544', '5.708214241257588243357393253934942138923101700452e+7447', '-4.5704876820534015009123053722758748e+496544', 78, 6);
    T('-9.7912775079317203630194370327298339764242962449842281045e-1636482', '-5.91668705692268671743547552163901421865600560123355889e-307854460', '-9.7912775079317203630194370328e-1636482', 29, 0);
    T('-6.856662955049015156455718145903741782325e+86958', '-6.6890303376370341e-53240368', '-6.856662955049015156455718145903741782325e+86958', 86, 2);
    T('1.948244461846631388480e-6322189817376', '75.44557572425688384807880078729820056668058424745911225', '7.544557572425688384807880078729820056668058424745911225e+1', 82, 4);
    T('-7.41914823401393098460076022992394704082641580294e-1713', '8.6e-2578', '-7.419148234013930984600760229923947040826415802939999999999999999999999999999e-1713', 76, 1);
    T('3.1643870451066881845797698904193006289940570e-8075953238', '2.10320732190588504157585944579979381343999e+900342', '2.103207321905885041575859445799793813439990000000000000000000000000000000000000000000000000000000001e+900342', 100, 2);
    T('3.798135005e-831', '5.267058030074253869394810775170290494833132725011839e+5978', '5.2670580300742538693948107751702904948331327250118391e+5978', 53, 2);
    T('-2.11846860e-28326783185', '8.09640462438e+157058992', '8.09640462438e+157058992', 12, 2);
    T('-9.924722019383616867814015326e-593097', '-4.569479868920572359151634566521265816782210520084878078e+854494401652209', '-4.569479868920572359151634566521265816782210520084878079e+854494401652209', 55, 3);
    T('18021426257183148598205309112087247310618.9018', '2.8634851180471843e+311', '2.863485118e+311', 11, 6);
    T('3.7566538203728917994449847167720e+4745947477', '5.30733681385172488348959186072309288446179e-89287593393396', '3.756653820372891e+4745947477', 16, 1);
    T('-8.15455153262042568883832e+92599', '-2.884238901395035768392781411428214087118757836633e+57', '-8.15455153262042568883832000000000000000000000000001e+92599', 51, 3);
    T('-4.241317114790e+57182247887', '-7.5400049e-875', '-4.24131711479e+57182247887', 48, 1);
    T('-592932825656191766', '-7.232554549509330192162267105780151377055909430419891400e-61400517917816', '-5.92932825656191766e+17', 41, 4);
    T('5.557179e-169662455', '-8.51981257925912562e-75452372009006', '5.557179e-169662455', 20, 0);
    T('9.5111367876062627657549422515514311e-2096', '2.5764118784314049060116486002700481e+802', '2.57641187843140490601164860027004810000000000000000000000000000000000000000000000000001e+802', 87, 2);
    T('-591183794.1505', '-2.66323014561617610878970148571866392265201810996e+5303689389', '-2.66323014561617610878970148571866392265201810996e+5303689389', 58, 1);
    T('8.996236767e+3617486930', '-8.8787743e-7711031221871', '8.996236767e+3617486930', 86, 0);
    T('-7.5912925849992283820434e+9821504848869', '3.893023084233796789497563490e+7693', '-7.591292584999228383e+9821504848869', 19, 3);
    T('1.790107632965631070391185884617099e+1083532156658', '-7.65904045087446642474e-8775045011', '1.790107632965631070391185884617099e+1083532156658', 42, 0);
    T('-2.0691e-69438221971675', '5.93542903155730941231843509345408038110710e+2678566', '5.935429031557309412318435093454080381107e+2678566', 41, 1);
    T('9.97548608829261554794619423812851630941208919e-7193145931', '2.787506982067161938473740662e+207028874610797', '2.787506982067161938473740662e+207028874610797', 42, 3);
    T('7.708137006085412125028729464358025618398399466335e+2741225361', '-8.75265625227e-31', '7.708137006085412125e+2741225361', 20, 5);
    T('2.514741457889286438746538806266309378036255710691305680821e+295', '-6.862e+17294', '-6.862e+17294', 24, 0);
    T('-7.6553070615104043364365989000980509651e-4905163183824', '-2.7875798866949620210e+22', '-2.787579886694962021e+22', 51, 6);
    T('-3.60993454554277793554048777312e-8355072', '-8.304216564605334880624507769796e+19020176189', '-8.304216564605334880624507769796e+19020176189', 52, 2);
    T('-7.01893901971685641733566743289870111880989589539293e+119', '3.05936e-560636972789', '-7.01893901971685641733566743289870111880989589539e+119', 48, 4);
    T('4.93583402242342377806527422894154563173e+6006622', '-1.24478678591706711437642241960641475725022756e-105926', '4.93583402242342377806527422894154563173e+6006622', 88, 6);
    T('-8.47071248e-99676945336650', '-1.607796563523829852596551325807010482294361700e-8007', '-1.607796563523829852597e-8007', 22, 4);
    T('3.425127787641131379991e-11978275350', '3.7140676566147915964247417778536516e+72395', '3.7140676566147e+72395', 14, 3);
    T('-1.1764738808e-33838407842', '7.6269500e-9552', '7.62695e-9552', 53, 0);
    T('6.675994091319421979775857153251793089e+93838', '-5.08283116299330255552866060057203595433700299739570330e+759891635832273', '-5.082831162993302555528660600572035954337e+759891635832273', 41, 6);
    T('-1.7047884005444714819013592537656616183747649274816e+84376', '6.0e-612', '-1.7047884005444714819013592537656616183747649274e+84376', 47, 2);
    T('6.510903584e+823214', '-2.10461768840e-8289', '6.510903584e+823214', 84, 0);
    T('-7007450189546795495984793158.7654', '4.9296903696e+105071505856', '4.9296903696e+105071505856', 15, 5);
    T('5.220947969266649424969705581777153458856351904199063048e-7191385', '-8.1546547e+427536', '-8.15465469999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+427536', 99, 1);
    T('-4.01532363604836303306401369648773409947512662987178341e+480101443159111', '2.2604766612725326598e-20', '-4.01532363604836303306401369648773409947512662988e+480101443159111', 48, 0);
    T('7.564189042762792130214645828011280982259722808001485746309e+484334', '2.6893400e+1245963119', '2.6893400000000000000000000000000000000000000000000000000000000000000000000000001e+1245963119', 80, 2);
    T('-6.94322972860156412023903154355e+665725228223', '-3.97600287245556958751561248796357518289457920823926540719e-482311', '-6.94322972860156412023903154355e+665725228223', 61, 1);
    T('2.754427654115793e+90963190', '-1.6310524323451937169669767017948e+296529275472', '-1.6310524323451937169669767017948e+296529275472', 60, 6);
    T('-1.59087e+9996406762', '8.42479798983118103340082593786101237790131e+77443831', '-1.59087e+9996406762', 88, 0);
    T('3e-149368484396', '-7.4955137448295049966096483349431642269774852344869094103096e-156', '-7.495513744829504996609648334943164226977485234e-156', 46, 5);
    T('8.6207706978128434508500210907906770606e-922066611714300', '-1.28258561402797606960190694832431179207735e-1833', '-1.28258561402797606960190694832431179207735e-1833', 86, 6);
    T('-1.15784922629689e+8196091293940', '8.7020677382137777071196974e+51', '-1.1578492262968899999999999999999999999999999999999999999999999999999999999999999999999999999999e+8196091293940', 95, 1);
    T('-5.33033292559080434497702593424866963085266e+5459309', '8.75321e-75086436844010', '-5.33033292559080434497702593424866963085266e+5459309', 98, 5);
    T('7.91740756493040e+9514216', '9.90878837e+415425210685688', '9.90878837e+415425210685688', 14, 5);
    T('-5.366256204611151032870736799132843e+5757456977864', '-7.3546e+9128', '-5.366256204611151032870736799132843e+5757456977864', 96, 2);
    T('6.657131019782751411612591e-557763', '8.51721160314876028736502059050150820055766e+63', '8.51721160314876028736502059050150820055766e+63', 42, 1);
    T('9.61356949785770287553e-52', '-2.5548195158632020926361167e-292605355138114', '9.61356949785770287553e-52', 95, 0);
    T('-9.601203e+916366158029', '6.75118714824438302543e+7294', '-9.601203e+916366158029', 56, 4);
    T('7.62560690e-35583666934', '2.269885487875650552807117792130216663468641e-84896949259', '7.6256069e-35583666934', 36, 4);
    T('2.07935597483749252739e+92543878', '-4.710572152637730175737402026578597618e+40635724771', '-4.710572152637730175737402026578597618e+40635724771', 72, 5);
    T('-5.092655018881456336284816801617018297962378611304394e+85', '5.52913229673173259419343281527883664578484642529100252485747e-61215', '-5.0926550188814564e+85', 17, 3);
    T('-4.14718635511340622344074247682e-82418340', '-5494122849395869386257763462060.38075680', '-5.49413e+30', 6, 0);
    T('7.45772e+673103181631', '4.97289414356660349308984034895899e-25354', '7.45772e+673103181631', 73, 6);
    T('-8.3825415759740271731846048384e-43585323', '6409867399225693316291341321.1864527558945705097', '6.4098673992256933163e+27', 20, 6);
    T('4.89293569973865e+14781842', '3.51296065946876926705707328139e-554', '4.89293569973865e+14781842', 63, 4);
    T('-8.064736671917925615408914823226506877447435927519392843900e+986052', '-7.257208044822668038754018100200910170521502681381e-79681110871085', '-8.0647366719179256154089148232265068774474359e+986052', 44, 1);
    T('-2.0278414715609121309661916712962546756007560483848e+46429683673', '3.124413e+232', '-2.027841471560912130966191671296e+46429683673', 31, 5);
    T('-4.0690323921e+15', '2.5840278125449145147862061743097145092847666414625914e+452299832', '2.5840278125449145147862061743097145092847666414625914e+452299832', 62, 5);
    T('-4.53101878179382338014981721336489904e+8648', '9.952246849614491053314664367109780564558954768792411e+92531751023', '9.9522468496144910533146643671e+92531751023', 30, 3);
    T('6.91888e-89', '8.671687078e-451531834666450', '6.91888e-89', 38, 6);
    T('-7.657564357480135458566871494932938078394674808530152178161e+676132292336', '-6e+26872478939879', '-6.000000000000000001e+26872478939879', 19, 0);
    T('-7.9791433912461e+6058833', '8.206434e+75934147192', '8.206433999999999999999999999999999999999999999999999e+75934147192', 52, 1);
    T('-2.22986816258743965197235721545e+868847943534', '-4.213559212698749486429239345420933562e+479', '-2.229868162587e+868847943534', 13, 2);
    T('-3.8e-770910165922444', '5.84542004019706625828154e+8635623674', '5.84542004019706625828153999999999999999999999999999999999999999999999999999999999999e+8635623674', 84, 3);
    T('-1.15530207275367226247965e-1050', '-1.5077859510647388974e-31773444049821', '-1.15530207275367226247965e-1050', 61, 5);
    T('-7.491078808315436074182465e+723712660902151', '-9.9848073199271197449385021964885130e+161885105', '-7.491078808315436074182465e+723712660902151', 70, 1);
    T('8.86406573596993792871451359755744628149630e+90954178177', '8.441308195921904224129178482869399342169133e-8472', '8.8640657359699379287145135975574462814963000000000000000000000000000000000000000000000000000001e+90954178177', 95, 0);
    T('6.10853321401491363036158743652005784039176e+134', '-2.48907433991954155383600655654841384869411054e-1439791494304', '6.10853321401491363036158743652006e+134', 33, 0);
    T('-9.654571497779097364216416252413567002317293002306975e+8154824', '6.093888387989052099506418196174969e-4851858068', '-9.654571497779097364216416252413567002317293002306975e+8154824', 64, 6);
    T('3.6e+32', '8.97761338840837357143575409721462769617265526167776e+314004577797', '8.97761338840837357143575409721462769617265526167776e+314004577797', 64, 4);
    T('9.3823209e+13416', '-4.75796482547188354262914769122994938933313978e-11600423', '9.3823209e+13416', 95, 0);
    T('7.765221447281065469957143358754764483437796e+10039816220', '7.0038255811735135038272866328131682145633e-181336297', '7.765221447281065469957143358754764483437796000000000000000000000001e+10039816220', 67, 0);
    T('7.31544216096378201986981394940668309906827e+54318557096', '-8.647135115569176459190851604324208956064587504993359e-7079315015534', '7.3154421609637820198698139494066830991e+54318557096', 38, 0);
    T('1.795575860090876366e+70674969794855', '1.188e-9349080420', '1.795575860090876366000001e+70674969794855', 25, 0);
    T('-5.9736616029682260317264113282e-976530757170', '2.9181567050e-1032841741', '2.91815670499999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e-1032841741', 99, 3);
    T('4.71068458015e-8653458674202', '-2.2782493598593238209770187005e+70619760', '-2.2782493598593238209770187005e+70619760', 76, 5);
    T('5.878488298662543958845e-65525809881239', '-6.182051212509330286920e+8461010', '-6.18205121250933028692e+8461010', 77, 4);
    T('-9.63807861361438602380956191514231413953066126842615356e-655933938051468', '6.518362721045e-56170885', '6.518362721044999999999999999999999999999999999999999999999999999999999999999999999999e-56170885', 85, 3);
    T('9.97027717244035026315190245e-41168', '9e+97212241338336', '9.000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+97212241338336', 91, 0);
    T('5.25462284734763973022489697209e+375', '8.082250398633207966524810516884947794349798895e-54007857988065', '5.25462284734763973022489697209e+375', 33, 3);
    T('-8.829179996081386318538593160781691419972676e-272191792241747', '-1.41380533811303744002e+22', '-1.4138054e+22', 8, 0);
    T('6.1e+28951672401', '5.338253390195367249e+399349', '6.1e+28951672401', 71, 6);
    T('4.77999604146148409e-6490824368', '5.958624766956067111088868825927479e-5531710684', '5.958624766956067111088868825927479e-5531710684', 35, 4);
    T('-8.9379713558748e+35748936386', '2.855605251093e+9156312592901', '2.855605251093e+9156312592901', 47, 2);
    T('-8.481984900542128562164746e-737', '-2.537080970146200735411382347899693e+391912304357', '-2.53708097014620073541138234789969300000001e+391912304357', 42, 3);
    T('3.28987194356955573795092265886283790849035584990788e+543200326150', '-6.1172813e-9479274411', '3.2898719435695557379509226e+543200326150', 26, 1);
    T('-6.269885971592441948803893241341892963669699545224569117e+896206766', '4.81057020544970954426154048263549215274598547e-3712707', '-6.26988597159244194880389324134189296366969954522456911699e+896206766', 57, 1);
    T('-4.7413836535991501973970731902766e+80', '-3.232669e-608', '-4.741383653599150197397073191e+80', 28, 3);
    T('1.112486459420040099323485171786565513237647e-57910405168349', '3.99949e-1298728', '3.9994900000000000000000000000000000000000000000000000000000000000000000001e-1298728', 74, 0);
    T('-9.00477046431683265784998645259245e+532', '-9.469824578060750695808874786928644906519e+9060683577', '-9.469824578060750695808874786928644906519e+9060683577', 49, 4);
    T('12562503249216597.35', '5.86292866764e+92540', '5.86292866764e+92540', 68, 6);
    T('3.5668605772304639979e-865', '3.651189802995952317927723808772044189924e+26079', '3.651189802995952317927723808772044189924e+26079', 59, 5);
    T('-8.29162844407225162364815990844e+45163375933483', '5.5654435364269891215329601449699267238990003976e-399941', '-8.2916284e+45163375933483', 8, 5);
    T('-0.09905309433495', '7.28560779031383162877853622978311832565607477e+283355407915', '7e+283355407915', 1, 3);
    T('9.7276693203878623053317421958130532966576e-333', '-2.55818419754994833942719726253539719082974173e+63732', '-2.55818419754994833942719726253539719082974173e+63732', 55, 4);
    T('7.3782659756491e-7593', '1.695887255089383971027582482e+740503927', '1.6958872550894e+740503927', 14, 4);
    T('-3.67700312927433430562906e+6121002108', '7.31285339332961462524798544303183387464941003175116802380235e-932', '-3.6770031292743343056290599999999999999999999999999999999999999999999999999999999999999999999999999e+6121002108', 98, 2);
    T('1.119101534772409190084223703056935697e-298440269344', '-9.1908017104750532426730809145315e+75', '-9.1908017104750532426730809146e+75', 29, 3);
    T('7.164569236058200914141195e-4935', '8.5786e+989828761', '8.5786e+989828761', 54, 6);
    T('-1.672417858e+13', '-6.79341525979000787e-84719040', '-1.672417858e+13', 65, 6);
    T('8.044537809723939586078278819188314642e-2616', '-2.04e-54447030617234', '8.04453780972393958607828e-2616', 24, 5);
    T('-2.8e-468688733', '-2.9e+7355873293', '-2.9e+7355873293', 76, 5);
    T('-2.196571e+3106056', '-320372.83659', '-2.196571e+3106056', 82, 1);
    T('1.836249e+556404', '-9.9644287827403266767768714053873938943318150589939063e-37287', '1.836249e+556404', 32, 4);
    T('2.1041609898775956822606287404436248890626408954483489800e+109', '3.817167373371549282905097e-599243619606', '2.104160989877595682260628740443624889062640895448348980000000000000000000000000000000000000000000001e+109', 100, 2);
    T('-7.6870746291143516394913978731673690545390525605181e+212307008701181', '-4.04703185592144419747150667362041517903112e-97708', '-7.6870746291143516394913978731673690545390525605181e+212307008701181', 86, 4);
    T('2.57193465565984963059228380e+5492430', '-6.689612120231046795086275572487012586153086250720628133e+108654', '2.5719346556598e+5492430', 14, 3);
    T('-9.37287222847e+7828575', '-7.91512031391648410026e-348342525613', '-9.37287222847e+7828575', 35, 1);
    T('4.899765075831410884230243852787542844380384924557193479658e-748', '-3.157860030461685133718148080e-16697368635106', '4.899765075831410884230243852787542844380384924557193479658e-748', 92, 2);
    T('-2.5724405699390635587730180862306791423299347498696612923902e+888', '7.310559021148821126340422436120453450e-40972', '-2.6e+888', 2, 0);
    T('2.289295053707258271969949254152906e-5412040694273', '-65442926064.2507', '-6.54429260642506999999999999999999999999999999999999999999999e+10', 60, 2);
    T('8.2729074631637650285093598599269347936238350e-839297199', '3.0721226193059429792749168618499337770208872171943960107e-893006573713', '8.27290746316376502850935985992693e-839297199', 33, 4);
    T('4.35082970656798722108498643180819308728488374e+263', '-2.8293194894252334022114509526038593282108e+630846004', '-2.83e+630846004', 4, 0);
    T('-0.00000211437579', '2.395839409591822567244161639882527914388437e+3547069', '2.395839409591822567244162e+3547069', 25, 0);
    T('-6966818114427358.6351915149957023462555573046262350853', '1.68893353253275103253e-757857', '-6.9668181144273586351915149957023462555573046262350853e+15', 85, 4);
    T('-6.9731231118293301794e-52485076', '4.05908283647908909149831942151852596e+2403', '4.05908283647908909149831942151852596e+2403', 69, 6);
    T('-8.4124883356e+3893966797', '2.639317517799e-82139251053724', '-8.4124883355999999999999999999999999e+3893966797', 35, 2);
    T('2799507952.6', '4.000915908841285064220640601129722447845070551e-41547124292408', '2.7995079526e+9', 61, 6);
    T('5.1e+852382095', '2.385189394617225008006650085726983e+950', '5.1e+852382095', 77, 5);
    T('-1.2313591022422346354133467277550807313223810251e+7189206', '-8.1995396234874458708840397939779e+54900607844', '-8.1995396234874458708840397939779e+54900607844', 67, 2);
    T('4.582356147797012430315e+5756062443175', '6.5160e+94855033', '4.582356147797012430315e+5756062443175', 74, 4);
    T('8.4685e+6117903035588', '9.4587148662179437210696880879664351076641060836281020259e-13', '8.4685e+6117903035588', 20, 1);
    T('8.97018433847008566475853628459286143134218462708e-78226051', '-7.59752648e-11340422246', '8.97018433847008566475853628459286143e-78226051', 36, 3);
    T('6.92400903656092666810745061940101105173262369404168434797158e+657', '8.21994352001359575975139e-8800494748', '6.924e+657', 5, 3);
    T('5.62241702270852101848741e-57630283331', '-1857185061647058839384389530214321521223834681590', '-1.857185061647058839384389530214322e+48', 34, 4);
    T('3.0741625595870727401049141214962972354421693e+15872438704670', '-3.084946897781359923704211401416807095993485877598971e-208', '3.0741625595870727401049141214962972354421693e+15872438704670', 79, 5);
    T('-2.82108064560027011040919561281095828621e-52', '3e+3139050471886', '3e+3139050471886', 70, 2);
    T('3260876857564979941296194187231039.455628720', '-8e+74655827264851', '-7.99999999999999999999999999999999999999999999999999999999999999e+74655827264851', 63, 2);
    T('8.6035757317106521164825296628026651215466738495105603324016e-166', '3.849e+499', '3.849e+499', 14, 6);
    T('-5.14322507974784904628150355e-63498357444', '-3.5694012765714711792e+103112155', '-3.5694013e+103112155', 8, 4);
    T('-2.50047271852607217939690896172404740082297730453220e+149', '-8.9264722851506602342e+28', '-2.5004727185260721793969089617240474008229773045322e+149', 86, 5);
    T('-3.2334323036108590303775237086291321e-8668597220', '-9.5107312739163610334406700e+6420691835746', '-9.51073127391636103344067e+6420691835746', 86, 5);
    T('5.357016e-96663075', '9.8322921529265829424770064e-808740674', '6e-96663075', 1, 0);
    T('-4e+9492045026354', '-5.184783219337077e+2696381', '-4e+9492045026354', 49, 6);
    T('3e+9', '4.776869975518921988117278749069764531053896688e+587174288479018', '4.776869975518921988117278749069764531053896688e+587174288479018', 48, 6);
    T('-6.0342251084817155649111472093995623479142219793367507011885e-36353486549', '-7.56531728483617137419045658775407701826749868442977759071e+94734205', '-7.56531729e+94734205', 9, 0);
    T('8.076304447258081485330662692436779e+839399732', '-2.7567411756333495585630621989793033261e+999117894333', '-2.7567411756333495585630621989793033261e+999117894333', 40, 4);
    T('-1.1155846362e+17', '-7.9e-17', '-1.11558463620000000000000000000000079e+17', 76, 0);
    T('-4.8846520203236327777752157676419342177422e+5486346724', '5.7368297448556039404188402956855847e+401177696592', '5.73682974485560394041e+401177696592', 21, 3);
    T('-4.8507150307632972338453459317647075e-1140415987716', '2.997037590650856207938940008811898488011763265212853617e-907', '2.997037590650856207938940008811898488011764e-907', 43, 0);
    T('4.23030952699889119717035593068753007255540786385080104e-174', '-7.99038346928975252984067313e+5611558015', '-7.99038346928975252984067e+5611558015', 24, 1);
    T('-9.37024330780200416139105973818488409e+91858161683', '-5.6342617063004e-72309', '-9.37024330781e+91858161683', 12, 3);
    T('3.679081882514105553188388378e+703946431823963', '-5.957807648487409813794565152136250304990421631318429939e-4765273635', '3.679081882514105553188388377999999999999999999e+703946431823963', 46, 1);
    T('9.25560894985389630411790746793183855477635489638450987e+767901', '9.15625642376298963967497743396772e+55971', '9.25560894985389630411790746e+767901', 27, 1);
    T('9.1983656901656123811988011203e+5873265', '1.6396023630584988014e+75', '9.1983656901656123811988011203e+5873265', 80, 1);
    T('-7.035344977e+866583898072741', '9.4298235722447643527978774466452594905928404743830e+581684', '-7.0354e+866583898072741', 5, 0);
    T('-5.029759127526514211269707061039505431e-731080148832', '-3.159934640567239347421438813969438703604137164701692435028e+365414', '-3.159934640567239e+365414', 16, 1);
    T('8.58800020772136559195727134238208e+40', '-3.48817444402848941523114135648386625804712916916887040e-618690708', '8.58800020772136559195727134238208e+40', 100, 0);
    T('5.883280856665793387627948606536832e+5466512931', '3.884879538708722746949014e+114', '5.883280856665793387627948606536832e+5466512931', 55, 5);
    T('-8.5035565670680277631109546087558008451423e+95139276490', '-8.535936785221477904133618999460e-3457431855', '-8.503556567068027763e+95139276490', 19, 1);
    T('3e+22', '-9.3740445033780801033839089075e-14626720934673', '2.9999999999999999999999999999999999999999999999999999999999999999999999999999999999e+22', 83, 3);
    T('2.8531021746954643834e+434', '-8.53327e-6061', '2.9e+434', 2, 5);
    T('-4.24179679771063323405552559903589818888869331e+574', '-6.2961339670991444303766294375450860653099531999983571202e-7560', '-4.24179679771063323405552559903589818888869e+574', 42, 5);
    T('-1.3200599907834000134088265323084e+91139263466', '-4.8839045495879520363291151075094865460729720363e-6695384', '-1.3200599907834000134088265323084e+91139263466', 63, 5);
    T('9.4949327332162730389369118656618238264e+8845420136', '-3.9343840685049024274642237449011505777652085e+44', '9.49493273321627303893691186567e+8845420136', 30, 2);
    T('-3.991147446e+8722415313132', '8.27358065684480912771247091e-179', '-3.991147446e+8722415313132', 38, 3);
    T('5.2062169458503135e+326625', '-7.9e+4509459', '-7.9e+4509459', 78, 0);
    T('-6.83864e-666', '-106130557382725293.8777', '-1.06130557382725293877700000000000000000000001e+17', 45, 3);
    T('8.8757662765697250147049269411e+405944308597', '-5.87284e-2424', '8.87576627656972501470492694109999999999999999999999999e+405944308597', 54, 3);
    T('4.67871100732e+84203190', '-1.45356345224402703247477542294522879795054749835e-7405719660558', '4.67871100732e+84203190', 30, 4);
    T('-3.85176783081662543200392873362638008e+8091', '3.87e+920510048', '3.87e+920510048', 13, 5);
    T('8.009865296104687911e-3938834', '820857.9829309', '8.208579829309e+5', 44, 6);
    T('-866947.8967712', '4.3038220503562911564129611002545613425e-9775755438211', '-8.669478967712e+5', 68, 5);
    T('4.2780858411427342003828071890616465640393584149658179e-345406252', '-2.6327507778347789808897190446840210170073719278356079831e-8705251319', '4.27808584114273420038280718906164656403935841496581789999999999e-345406252', 63, 3);
    T('-4.681797410622657561e+854', '-2.4919689614511216215653729087755488359744520e-4142788', '-4.681797410622657561e+854', 21, 6);
    T('-5.322716930e-58530826925', '-1.84631862373292888136736193581275664660439654273392376573095e+61', '-1.84631862373292888136736193581275664660439654273392376573095e+61', 92, 6);
    T('9.1455932352302470010112370798637129838479896835336368966298e-57059906047966', '-7.5040467538028593415239046942951e+609942655', '-7.5040467538028593415239046942951e+609942655', 60, 4);
    T('1.923511241563835e-9940259629', '-9.316e+583896338113', '-9.315999999999999999999e+583896338113', 22, 1);
    T('7.2768673795e-1413534254', '7.835822366940231316797956741636e+1573717', '7.835822366940231316797956741636000000000000000000000000000000000001e+1573717', 67, 0);
    T('3.63359847530147175562715616286889734709e+20863', '-3.1432132587889588439489128577267451481545320683904442e-790847278100', '3.6335984753014717556271561628688973470899999999999999e+20863', 53, 1);
    T('2.59912920329098883596013693653e+666', '1.25698010897e-4737', '2.59912920329098883596013693653e+666', 40, 1);
    T('-3.31373543420140533941793893881888443875e-936857579576', '-7.504783280827e+109763461304151', '-7.504783280827e+109763461304151', 62, 4);
    T('1.3277283815280310163422951180739778405866075732e-5188789', '3.679312343023e-8397', '3.679312343023e-8397', 61, 3);
    T('-1.781199001799209586565530466399e+927192', '2.6076766651677374047969728162629065062375936988e-3748', '-1.781199001799209586565530466398e+927192', 31, 2);
    T('9.7865923e-57118870', '6.756530169539889e+65064803426', '6.756530169539889000000000000000000000000000000000000000000000000000001e+65064803426', 70, 0);
    T('6.829735427e+545717021792', '-8.403666600479489802642422270931e+876', '6.82973542699999999999999999999999999999999999999999999999999999999999999999999999999999999999e+545717021792', 93, 3);
    T('4.336824851418743831811e-32379643885', '2.948625694082e-56998692', '2.948625694082e-56998692', 36, 4);
    T('-9e-7', '-16217081694421.5168874683484853037850509631', '-1.62170816944215168883683484853037850509631e+13', 48, 5);
    T('-4.8731201320805e+82438913', '-3.15779370e+5136279', '-4.87312013208050000000000000000000000000000000000000000000000000000000000000000000001e+82438913', 84, 0);
    T('-8.3810518661739713556482880729732006121234553e-77642467', '-1.50498045039756e-35788013', '-1.50498045039756e-35788013', 93, 4);
    T('7.50158716522620307028990705529508660612239254774320928388274e+5856694133349', '6.518017065457294488543058271772017544598063063922666177401e-898934311', '7.50158716522620307e+5856694133349', 18, 3);
    T('2.32073963708871269128879197684764953e-7972222148', '4.67451868532787693072448248364242243068765538899456e-4885', '4.6745186853278769307244824836424224306876553889946e-4885', 50, 4);
    T('8.694998230849792924674722e-56145630480011', '1.57837426338280841917e+22', '1.57837426338280841917e+22', 97, 5);
    T('-6.3690417639974965861755598881656557081711730632352764e-41691147479256', '7.5844394371180648170037e-473405', '7.5844394371180648170037e-473405', 34, 5);
    T('8.337745326186e+18055377', '2.64079717554156498432342204192508717e+9576', '8.337745326186e+18055377', 69, 3);
    T('-9.085196863189121935813286087142204494035704727175e-85973966', '7.573740011001858847876456081e+1472087', '7.573740011001858847876456081e+1472087', 74, 5);
    T('9.32138274219022571926115066230889797379801742573e-83764931', '9.445235719530429053e+89111977836263', '9.4452357195304290530000000000000000000000000000000000000000000000000000000001e+89111977836263', 77, 0);
    T('-7.4123008352854614143186e-7862154', '5.0813023048645e-980755519455942', '-7.41230083528546141e-7862154', 18, 4);
    T('-8.1558029968264037869007470082958107568589507758069553028e-3684648006', '-635031737078380629258476785018180.571196921055120', '-6.3503173707838062925847678501818057119692105512e+32', 99, 4);
    T('4.5340792182143e+945290557', '22468120382538829192.6394624343320', '4.5340792182143e+945290557', 26, 5);
    T('-8.725189841918860200e+249252', '9.5051540972028083692252490720110264994406300233389589841e+52644960', '9.5051540972028083692252490720110264994406300233389589841e+52644960', 90, 6);
    T('4.2263349422241062003742770e+46109574059', '4.459607e+330259319798253', '4.459607e+330259319798253', 30, 1);
    T('4.8051755372841408687150622143871e+559688081871690', '7.656e-90640', '4.8051755372841408687150622143871000000000000000000000000000000000000000000000000000000000000000001e+559688081871690', 98, 2);
    T('-2.269551739642011012191948868277555652e-46070', '-6.01935391488028878931313942e+32', '-6.01935391488028878931313942e+32', 51, 1);
    T('-3.179895300021526115051164610193e+28154004076', '-7.8754255340e+1702', '-3.179895300021526115051164610193e+28154004076', 54, 1);
    T('3.80550563126038514e-2689', '7.88414715853960235508573651e+839547002', '7.8841471585396023550857365e+839547002', 26, 3);
    T('-1.4439e+7882918', '6.1846446838765781835462623246355e-510502325', '-1.4439e+7882918', 27, 3);
    T('2.0001792581608880826493784519272e+5817766429914', '7.82389505520e-10255828818228', '2.00017925816089e+5817766429914', 15, 0);
    T('-672.6072894091504291980128935', '3.5944640925968777891516098571067739294139345639060440e-82960', '-6.726072894091504291980128935e+2', 86, 4);
    T('7.94493709108763952116980116531854256108707e-843019', '-1.6e+1032506', '-1.6e+1032506', 75, 6);
    T('8.8627335750686e+129792094456', '2.55306740301321398511208226874279756490760830752591870990906e-3681380070', '8.86273357506860000000000000000000000000000000000000000000000000000000000000000001e+129792094456', 81, 2);
    T('9.22818342745774467477996950532236324173709945102e-76589003981827', '-8.0845907636224963572563482750883757501424052e+248986185280', '-8.0845907636224963572563482750883757501424052e+248986185280', 91, 4);
    T('4.0555257742674921759277749996933608734289e+475326297068911', '-1.88546112317e-63579276209', '4.0555257742674921759277749996933608734289e+475326297068911', 60, 4);
    T('-8.7965347945951311601076055601875496e+125686', '-1.16663651989303114e-790637', '-8.7965347945951311601076055601875496e+125686', 46, 5);
    T('3.82483652826304e-8770', '2.614000592009599611538990162897321990244550e+667542632635456', '2.614000592009599611538990162897321990244550001e+667542632635456', 46, 2);
    T('2.65974744080008671181210182947663682340913992803123350077740e-91891504', '2.924659309023509652245026103143436940101497215989039039e+3816064040575', '2.92465930902350965224502610314343694010149721598903903900000000000000000000000000000000001e+3816064040575', 90, 2);
    T('8.48615342619134854e+549085843917728', '-7.269172478809417126547235283163873824e+653', '8.48615342619134854e+549085843917728', 41, 2);
    T('-4.77569710e-7323323', '-2.09865843353105549e+38', '-2.09865843353105549e+38', 53, 5);
    T('-5.47163836509259168919646915687101021954523728617e-526', '-7.904037e-354343055', '-5.4716383650925917e-526', 17, 5);
    T('-6.12526819e+33748', '3.642e+3863', '-6.12526819e+33748', 41, 6);
    T('3.1e-63621903763215', '-3.079e+90535245646728', '-3.079e+90535245646728', 98, 6);
    T('6.216006988110507887601e-9097', '7.3e+1143689525347', '7.3e+1143689525347', 37, 5);
    T('-5.0e-55566163873', '-58544099788104565336188890733870909122244.2767719', '-5.85e+40', 3, 4);
    T('2.3924826080789240782e+2533', '-2.6587568681681178732059e+22761042632713', '-2.6587568681681178732059e+22761042632713', 82, 4);
    T('-8.75418147612963627328538450062177172320379701e+7885514033', '6.316561609824020540137597807726974e-94882', '-8.754181476129636274e+7885514033', 19, 0);
    T('5.4939116116713118024886750170429773063863895430937397076e+14077234420', '-6389561414790161343740.4333787', '5.494e+14077234420', 4, 5);
    T('7.2089119211566632260004252749136770302773728707e-950', '-1.615e-7937828', '7.20891192115666322600042527491367703027e-950', 39, 3);
    T('5.434180523772882362896862109437429292e+295', '-9.891784636362490631e-21092040274', '5.434180523772882362896862109437429292e+295', 100, 4);
    T('6.8262005423990703741958646486774597264015527710694e-793136', '-1.44634005635834962752150165244235861835246e+1664572', '-1.44634005635834962752150165244235861835246e+1664572', 77, 3);
    T('-2.167547530173567807316779221883099e-8588', '-7206432746074.4643860', '-7.206432746074464387e+12', 19, 3);
    T('-4.71089417253351658646401e-7619916987', '3.483560026784826990429246161577702312602e-790805194', '3.483560026784826990429246161577702312602e-790805194', 76, 0);
    T('-2.463432133427259356645321849779208559342153443e+5315945841099', '8e+1', '-2.463432133427259356645321849779208559342153443e+5315945841099', 77, 4);
    T('9.89002417444509840935954604984084e-29', '-9.5301347111094e-6280464', '9.89002417445e-29', 12, 4);
    T('9.09e+73646577', '-9.595570329325220597939487000594183981593e-286', '9.09e+73646577', 100, 0);
    T('-8.0306288223307503279227865225e+108', '-8.9137988397606203880602460721e+59', '-8.03062882233075032792278652250000000000000000000089137988397606203880602460721e+108', 80, 2);
    T('-7.4959213396266065190620740125817708442675004435784770064417e-660', '4.87282780319351269256379e-9601', '-7.4959213396266065190620740125817708442675004435784770064417e-660', 85, 0);
    T('-1.80983346632e-83482125141927', '6.92269507056516899233e+466327', '6.92269507056516899233e+466327', 93, 5);
    T('-8.3930052458840988424940249998606640534084802243550664393850e-73699', '-7.346166713437581194288519110380e+6273751258134', '-7.346166713437581194289e+6273751258134', 22, 5);
    T('-4.21e-543129786450', '-6.444128927775023374723460830e+9496277320', '-6.44412892777502337472346083e+9496277320', 85, 6);
    T('-7.28582754115455412e-559025', '-5.478343e+525', '-5.478343e+525', 67, 6);
    T('-7.93032630454923747952614277936331e-27', '2.98656973367325902966851737169213739596144657955375470925e+62', '2.98656973367325902966851737169213739596144657955375470925e+62', 78, 4);
    T('-2.6147510489129205441680357169883258502e+5313767612', '6.66248694214673813609480058961573645364427e-9815710796', '-2.6147510489129205441680357169883258502e+5313767612', 93, 0);
    T('4.15468935372295249057902129788149425121791805412845276399831e+1117', '7.9774574690280452367644238947356564563494549586721365305e+9268075', '7.9774574690280452367644238947356564563494549586721365305e+9268075', 100, 4);
    T('2.43069627269377512322884151002787934714238e+88474621', '3.080070e-19815', '2.430696272693775123228841510027879347e+88474621', 37, 5);
    T('9.169065742352365e-133595441', '-3.66561260957699062606738663313661849551926741260144741e+7820115538', '-3.66e+7820115538', 3, 1);
    T('2.86973701803963934077433098e+64459', '3.8874305143948458e+3169119748233', '3.8874305143948458e+3169119748233', 21, 5);
    T('8.11609799631503082956755849415609e+3434002636', '-7.670921945322162720340473693237614401276e+28743604837', '-7.670921945322162720340473693237614401276e+28743604837', 80, 0);
    T('7.66823960524432042537660145744133067880e+1988041283', '-2.896856625206014407254023777735377136807547e-4587004222664', '7.6682396052443204253766014574413306788e+1988041283', 78, 2);
    T('2.6908538156165493898912405353897811026968044043521e-225', '7.03698256146021003251623002334809627046171e-17390148197', '2.6908538156165493898912405353897811026968044e-225', 44, 5);
    T('9.602242360530617762923378854598493173990181498273e+96', '-3.14475604415397162189949950398055063261203509e+463698852', '-3.14475604415397162189949950398055063261203509e+463698852', 70, 5);
    T('3.00702564061e-557897', '8.53345755761787202225863970189543987442933571e+955679652', '8.53345755761787202225863970189543987442933571e+955679652', 78, 5);
    T('1.872602908472073912702381733216349103210619524054293e+4293155', '1.4394745067002981e+99425492', '1.43947450670029810000000000000000000001e+99425492', 39, 0);
    T('-3.7652097e+16', '-7.16678715088165584764e-51304592814', '-3.7652097000000000000000000000000000000000000000000000000000000000000000000000000001e+16', 83, 0);
    T('-4.093246499333627181435792086278e-26833294776', '-2.31553551e+2081924', '-2e+2081924', 1, 6);
    T('9.815257851073793518785e+8196400007', '-3.235259971358885112181839942558211028818448432453375329188e+797248417', '9.81525785107379351878499999999999999999999999999999999999999e+8196400007', 60, 3);
    T('3.4914663298067576709001876e+74', '-5.1421982353819164338392013767152940777168867196423e-5300809', '3.4914663298067576709001875999999999999999999999999999999999999999999999999999e+74', 77, 3);
    T('8.628686710075619788336744288346123549850e+6938', '-129686403852545263044523631161140547162618361.44643216', '8.62868671007561978833674428834612354985e+6938', 68, 0);
    T('0.00007', '2.265318564661055996760104172585853386642782041411715464908e-793379720133', '7e-5', 38, 4);
    T('5.30619923e-16377103628155', '3.4103859583330416709617096677971401612775807e+5389882285', '3.410385958333041670961709667797140161277580700000000000000000000000000001e+5389882285', 73, 0);
    T('5.71664384539929965898681466537981943040505182139687341334e+26495627612', '-80087468055012602677.96', '5.7166438453992996589868146653798194304050518213969e+26495627612', 50, 0);
    T('0.0000027254564274419547902922010236194120925186369725680351485', '8.42143726e-71236820', '2.7254564274419547902922010236194120925186369725680351485e-6', 81, 3);
    T('1.7087948297776245010441279566304891666e-89064', '2.47492483690793362316092699438909060057427322606421916659167e-3217935697721', '1.7087948297776245010441279566304891666e-89064', 72, 4);
    T('8.7466901958872398466450558633145513362547879e+7621', '-2.562960976035841519727372117594154612854141893734403e+496874352499691', '-2.56296097603584151973e+496874352499691', 21, 5);
    T('-2.3149527016388912719974329877743580901e-107', '4.3e-5580', '-2.314952701638891271997432987774358090099999999999999999999999999999999999999999999999e-107', 85, 2);
    T('6.290366242289123961703624825223e-93599082', '-7.82119026695238048e+4985178', '-7.82119026695238047999999999999999999999999999999999999999999e+4985178', 60, 2);
    T('-6.1391156657497606374182e-25', '-6.16743897754746509224054785849446761444278213098198530e+955778663286', '-6.1674389775475e+955778663286', 14, 4);
    T('7.68631596382678625636285580635237970e-15455965789', '1.5564089338222913800416158e+37020503566', '1.5564089338222913800416158000000000000000000000000000000000000000000000000000000000000000000000001e+37020503566', 98, 0);
    T('-6.62015424184926041806910722735434e-698', '3.1877675448123631130260587970824867643954817458538580427e+206', '3.187767544812363113026058797082487e+206', 34, 4);
    T('1.296307230622014432e+29930863888145', '5.2119149e+56577792298', '1.296307230622014432e+29930863888145', 95, 4);
    T('-5.632573439320840330193891058995882578561635908211377218e-356256084', '9.2190916445812068954e-62882251727', '-5.63257344e-356256084', 10, 3);
    T('-4.5172e-30417695813', '-8.3436673702901457173830976583851485660461091129161871712e+949541', '-8.3436673702901457173830976583851485660461091129161871712001e+949541', 59, 3);
    T('6.033717048702e+3602', '-4.16383373435410759755350961148381810543429101565358394526459e-24604316', '6.033717048702e+3602', 55, 6);
    T('-1.857638664606207352592021410889755015371929726534692290e-742768720252359', '7.694511761279e+4786', '7.69451176127899999999999999999999999999999999999999999999999999999999999999e+4786', 75, 3);
    T('-6.085985344759877540601432484458028987577807509180e+73800491569', '-7.834337670813721635913155081717595415905264215524165e-405452164996', '-6.0859853447598775406014324844580289875778075091800000000000001e+73800491569', 62, 3);
    T('1.82219484016155123633754760671923744352574e-141', '-5.78287826898603186482749822e-2702846564', '1.822195e-141', 7, 2);
    T('8.06908742589357870728349e+24737987820', '-3.06106787484338656405639783925841556573164507690211043999e-56088867770', '8.069087425893578707283489999999999e+24737987820', 34, 3);
    T('4.528750122705e+328997911564335', '-7.12828158776221317357469411535495852941661e-294279821782', '4.528750122705e+328997911564335', 39, 5);
    T('-8.9204427428936910807823617956030e-513535', '8.0564431958e-75939277', '-8.920442742893691080782361795603e-513535', 90, 6);
    T('2.7002853496960788e-60452518271', '7.2538364899029283e-49124', '7.2538364899029283e-49124', 76, 5);
    T('-7.5470902351036553895531063930794158322245e-7660094', '-6.10407336508142699389595976436740679625838812836382e-782996701', '-7.54709023510365539e-7660094', 19, 3);
    T('7.471707437024552235993514e+1898268087796', '1.7753141e-2298133764', '7.4717074370245522359935140000000000000000000000000000000001e+1898268087796', 59, 0);
    T('6.6971906976542113505476651e+888319225', '-6.566781340630321e-4828093412', '6.697190697654211350547665099999999999999999999999999999999999999999999999999999e+888319225', 79, 1);
    T('-5.04030978e-960555137995108', '-7.88629179523759274691656904264291725548069560599358122973e-613160764457', '-7.8862917952375927469165690426429172554e-613160764457', 38, 2);
    T('2.06565634734623592336583e+11931558912714', '-6.4269159785608447e+256756796536994', '-6.4269159785608447e+256756796536994', 63, 6);
    T('6.697e-50936', '8.093822411462052780401791029311168e+52', '8.093822411462052780401791029311168e+52', 62, 3);
    T('-2e-481468849', '-8.2072226846939679104513e+13579', '-8.2072226846939679104513e+13579', 70, 6);
    T('-2.9844462332e-790500628051', '-5.0295860638760898986771693152706338728588711531335e-81512906407042', '-2.984446233200000000000000000000000000000000000001e-790500628051', 49, 3);
    T('4.10758321445201071391180364143785084192037480831330233e+11389167904', '8.812672848823925553844420722e-1232', '4.1075832144520107139118036415e+11389167904', 29, 0);
    T('-9.0088e+515822', '2.620903031771352998206390442844368438e-96039377046782', '-9.0088e+515822', 60, 3);
    T('6.514926333472638282782e+47822963991243', '7.0358003286999868897256e-179762149514', '6.51492633347263828278200000001e+47822963991243', 30, 0);
    T('4.3495252680950840579296586204700259390891003043e-405731586320178', '6.51068560691867659110317980729296924041347114e+4669403334', '6.51068560691867659110317980729296924041347114000000000000000000001e+4669403334', 66, 0);
    T('-6.91752733179952376460987528217e-328', '1.1798712181245779533491832899273e+700', '1.1798712181245779533491832899273e+700', 45, 4);
    T('4.4681161632768797391613317481481e+78', '-5.60e+44646', '-5.59999999999999999999999999999999999999999999999999999999999999999e+44646', 66, 1);
    T('3.916718986862142964856e+4008', '-3.32519e-42917858532864', '3.916718986862142964856e+4008', 80, 4);
    T('-3.04598207269829138268e-63909069996', '-2.00865705283545025354680834797002951746623771565171445305279e-12961', '-2.00865705283545025354680834797002951746623771565171445305279e-12961', 76, 6);
    T('-2.35667875835766544951275577080149e-67913', '8.432e-605033441', '-2.35667875835766544951275577080149e-67913', 57, 3);
    T('3.5228e+1378382416', '8.278504468123996e+46101', '3.522800000000000000000000000000000000001e+1378382416', 40, 0);
    T('6.6930946048035e+2018345952', '2.97188e+516894695219534', '2.97188e+516894695219534', 18, 1);
    T('-8.9799192729744586e-47834242895073', '7.423505970572325530553814116672346867e-5496598509201', '7.423505970572325530553814116672346866999999999999999999999999999999999999999999999999999999999999e-5496598509201', 97, 3);
    T('-4.79784490358584329333479677090266612254150689719533e-36258', '9.105778064e-959449', '-4.7978449035858432933347967709e-36258', 30, 1);
    T('-3.218971468541106360519527511770169008985112e-8057', '-6.2421613269600942720869835589222994011043629965e-7203429949', '-3.218971468541106360519527511770169008985112e-8057', 73, 2);
    T('-2.33390319085e-97934076271', '-8.14289266429898833929139236666646303951e-579', '-8.2e-579', 2, 0);
    T('5.9780934581231821551070643095335905e+384437', '1.8352456192259826314903429630417457598053100126168306791502e-58105059010133', '5.9780934581231821551070643095335905e+384437', 98, 5);
    T('5.043339151176247967842e+724672101068', '4.2243757199258462e+360535457', '5.043339151176247967842e+724672101068', 50, 6);
    T('2.620840483637303531243871032e-235092937', '334261339216713589', '3.34261339216713589e+17', 95, 6);
    T('-3.7924617211509975784022400383789543039843571891084528e-16157297539353', '9.13753262799080701792827935e-2423', '9.13753262799080701792827935e-2423', 64, 6);
    T('9.3661108079060669430597404850e+7738748', '-9517230.540010289671967306817566130308908435217450000236', '9.3661108079060669430597404849999999999999999e+7738748', 44, 1);
    T('-5.3249268132617586745239156481753e-454079839754', '-7.022275344552956320e-225257611', '-7.02227534455296e-225257611', 15, 5);
    T('-6.7671104827426590658184796381992223484786130036657194e-5247649', '-6.085181162570303441176173509725567351067733139e+61863867', '-6.1e+61863867', 2, 3);
    T('3.849326414128139203593896889376443348e+4309021', '-3.1816026855703833416368105e+95866202', '-3.2e+95866202', 2, 6);
    T('1.90321016169641989362980878654969207169e-385233493', '8.931e+24530870575', '8.931e+24530870575', 31, 6);
    T('8.364843318034e-56875459759901', '6.50009135151407298443090218481944549e+58227764626768', '6.50009135151407298443090218481944549e+58227764626768', 92, 5);
    T('8.235582692103471e-19922004066', '-4.15357055161512927053093333183274808671e-91473261904590', '8.235582692103471e-19922004066', 55, 2);
    T('-9.692616756410347985e-304379963', '2.5917955380672474662710384012785477273207592204076305829e+4551709156648', '2.59179553806724746e+4551709156648', 18, 3);
    T('-7.09650038500745500002474896779394536223828622207381637e+58', '3.4151531122353978082855497327541885487738e-37677', '-7.09650038500745500002474896779394536223828622207381637e+58', 93, 5);
    T('9.7447960590613999254275367e-17427', '-5.2876301482702521276403e+82384570', '-5.287630148270252127640299999999999999999999999999999999999e+82384570', 58, 1);
    T('2.142829801950871237433570107876827640780892e+5907795898877', '-3.40612115514005900046562e+43669579490', '2.142829801950871237433570107876827640780892e+5907795898877', 73, 4);
    T('-8.44100368828347346307900621515076213e-157103788221518', '-8.7885e+261750770', '-8.788500000000000000000001e+261750770', 25, 0);
    T('-9450836311059338151510072410252217.4005904162369327696378652', '1.33367509813356190043411637e+369', '1.33367509813356190043411636999e+369', 30, 1);
    T('9.29091101895483593101230865319396979832333371007e+3549190619', '-58350818682109062535313537785619575424625972303.9535', '9.29091101895483593101230865319396979832333371007e+3549190619', 62, 6);
    T('-5.06946649439041529e+24', '-2.90483744e+45565', '-2.90483744e+45565', 71, 1);
    T('4.8600748813432387676303061579e-241261', '2.02709e-90568956', '4.8600748813432387676303061579e-241261', 75, 5);
    T('-1.1331906498699347680393244897722983385744e+42', '3.121494848159494908657e-9142390', '-1.1331906498699347680393244897722983385744e+42', 49, 6);
    T('-1.7175883378e-2314', '-1.79087124582655858772603856076729542717086235966576300e+630', '-1.790871245826558587726038560767295427170862359665763e+630', 78, 6);
    T('-8.6704066280650709713e+93', '-7.381801809270e+171', '-7.38180180927e+171', 62, 6);
    T('-5.177880422522298146066564056460209830246416544133211e+359054569838', '-9.51186e+21973824374', '-5.177880422522298146066564056460209830246416545e+359054569838', 46, 3);
    T('6.7771951383703218900583230887960014452430565106e-54657073785', '-9.21960384259138751379928274e+41093', '-9.21960384259138751379928274e+41093', 78, 4);
    T('-8.539638318607e+57554158', '2.795137790017808693189246053039426128908932721e+365885180691', '2.795138e+365885180691', 7, 0);
    T('9.0165835390388376547844425e-86634247', '5.1422629018781697704698006887506286316e-899', '5.1422629018781697704698006887506286316e-899', 93, 3);
    T('2.317447731053915507320317990141980199461545e+10354', '9.4696572981677487456731070e+353293', '9.4696572981677487456731070000000000000000000001e+353293', 47, 0);
    T('154865.4', '-8.93889794857104812627481960451e-4135319436', '1.548654e+5', 33, 0);
    T('1.371619499566540e+7161534046588', '-5560389622743211238362.84738', '1.37161949956653999999999999999999999999999999999999999999999e+7161534046588', 60, 1);
    T('9.30912105643772927746051638444966178999597683457796125839620e+6999', '-5.34909649223115997239418867412012766582846480522268834e+41381057030722', '-5.349096492231159972394188674120127665828464805e+41381057030722', 46, 2);
    T('-6.905858492639557096899168254959824854759069459e+43094141523032', '4.9227045473583921414e+607735597520', '-6.905858492639557096899168254959824854759069459e+43094141523032', 63, 5);
    T('-3.10521838396433523829341789808e+111616452019876', '-7.8900908280504727856512375513382517910733628131e-416759080572', '-3.10521838396433523829341789808e+111616452019876', 49, 2);
    T('2.1817702716920e-6678', '4.4573805788100616e+880', '4.457380578810062e+880', 16, 2);
    T('6218172.6846028069210881346788', '2.6771953674743070366741977473e+99315833446559', '2.6771953674743070366741977473e+99315833446559', 44, 5);
    T('3.847734679823759354582222053972615080786e+86496', '-7.72936860142716e+84875121', '-7.7293686014271599999999999999999999999999999999999999999999999e+84875121', 62, 2);
    T('-8.554441289366296468177816446107562e+301021', '-2.927199e+640524508864041', '-2.92719901e+640524508864041', 9, 3);
    T('8.162975809649352133881693355706e-746360424584', '-1.62141645014492277637504039124137096727205305596e+435314817', '-1.621416450144922776375040391241370967272053055e+435314817', 46, 1);
    T('8.354095490672861238604817586254856776427641e-530177469301', '9.01595821149220140805244409282775702064934172262317043421e+962511201855655', '9.01595821149220140805244409282775702064934172262317043421e+962511201855655', 86, 1);
    T('-7.32588558947460435854769788198e-1717987', '-685105643658027635480679618517251149272.56327', '-6.85105643658e+38', 13, 6);
    T('-9.74440284805409174693201573037961112293483106e+39687146489328', '4.19978240149576503497660147e-1462762320', '-9.744402848054091746e+39687146489328', 19, 1);
    T('8.508836401339838766487612014256234846839762399633148528486e+96943253772383', '7.9764007100393723080536173829939356503703588371145e-10351027', '8.508836401339838766488e+96943253772383', 22, 0);
    T('9.451188190094551618700227058e+464554255433', '1.61477554118398696552486827673e-744285924465497', '9.451188190094551618700227058e+464554255433', 73, 6);
    T('9.421948354019795800595900042280880e+79', '-8.82133988754653e+5422382342', '-8.82133988754653e+5422382342', 20, 5);
    T('-4.732148322013095850942435002896853011556470328571e-501259330', '5.03869003814179e+99791443593668', '5.0386900381417899999999e+99791443593668', 23, 1);
    T('-1.73029878723251276385003294629362e-69232586', '-2.606391770e-9395', '-2.606391770000000000000000000000000000001e-9395', 40, 0);
    T('7.69120891487896e+45767939614340', '-3.4588252698638950200312236217486556760144738527e-848322604447471', '7.6912089148789599999999999999999999999999999999999999999999999999999999e+45767939614340', 71, 1);
    T('-8.962525087846440e+5275971923', '8.45659190953301802318600803417632439512216031291828131626e-954831021688321', '-8.96252508784644e+5275971923', 15, 3);
    T('8.414659853215656627999431825181224805675908701e-2503958347381', '9.523671660586502230880176665e+69157442499625', '9.523671660586502230880176665e+69157442499625', 62, 5);
    T('-8.63010409919396183278150e-2170', '3.3597102967882968554653913218269891539797661e-14473668260923', '-8.6301040991939618327815e-2170', 54, 0);
    T('3.7603707610144549546292432182783e-172451298', '-6.075649139589370706751890606383636683914044106376810358e+97348549', '-6.07564913958937070675189060638363668392e+97348549', 39, 0);
    T('4.5515971196078493890540694e+4696680', '-2.7908e-8', '4.5515971196078493890540694e+4696680', 47, 6);
    T('9.4022558062773884472525e-33', '-3.3340302183537761533304079282373305850627792345e-923106', '9.4022558062773884472525e-33', 32, 4);
    T('4.0046866229913e+96', '-836.996903', '4.0046866229913e+96', 90, 5);
    T('8.6324712983936447142966e-993307', '-8.06253839073217710701927057162738573063752e+45093577', '-8.06e+45093577', 3, 2);
    T('-1.40935312111464537291736645365448e-6569271493', '6.7125417850688717567403992148614789317392e+45491146766', '6.712541785e+45491146766', 10, 5);
    T('8.71963909922644366019650464e+30', '-8.23106245083579901360644830704844056179427749008958724e-26', '8.71963909923e+30', 12, 6);
    T('-8.2868082664404617570258478801084316199236693929e+2861841751', '2.864525817023848042e-680675', '-8.29e+2861841751', 3, 3);
    T('-8.89e+73232', '7.445389510708e-84', '-8.889999999999999999999999999999999999999e+73232', 40, 1);
    T('8.84711058809161704447701719335869473606635412485e+745', '-5.111e-22344321', '8.8471105880916170444770172e+745', 26, 6);
    T('6.0859621422243517345427227074579239843261282410990878115338e-3628776', '-1.772394376459778439e-480624612785', '6.0859621422243517345427227074579239e-3628776', 35, 3);
    T('7.6211510094372473245e-55634749658', '-1.106363013851802e-3406142379', '-1.106363013851802e-3406142379', 19, 4);
    T('1.7192e+2775186480349', '-8.0476902e+1624546', '1.7192e+2775186480349', 85, 2);
    T('-6.1899241179924976776526994e+823170922339', '6.982875427757452670906651850e-741', '-6.1899241179924976776526994e+823170922339', 79, 6);
    T('-2.9631875841162741254263580989692706469683931082885824676e-54226570', '-44113124654088709708.93122548283784842185856495897860357414', '-4.411312465408870970893122548283784842185856495897860357414e+19', 59, 4);
    T('1.18929725174066551570821261509088629042752811872383e-875950', '-1.2592110206875882538637956174361401952071e+212', '-1.2592110206875882538637956174361401952070999999999999999999999999e+212', 65, 1);
    T('-1.648115654721307e+16', '-8708.074', '-1.6481156547221778074e+16', 82, 0);
    T('-1.35426731010493806916371867138138e+552345387028', '-6.56963268191394640218935676305200854e+44', '-1.35426731010493806916371867138138e+552345387028', 63, 5);
    T('4.4445245492292571578504592666143043513949649169454006914e-3317302117', '-2.6311790837113650277044811660659e+93231510', '-2.631179083711365027704481166065899999999999999999999999999999999999999999999e+93231510', 76, 1);
    T('4.806662871465158609750269e+3918067883', '-3.503202872312045341645766062651939682061681690763825583e+531600', '4.80666287146515860975026899999999999999999999999999999999999999999999999999999999999e+3918067883', 84, 3);
    T('8.67054096862340635442877065522686249e+165', '1.987479665375599098325170674578998538506157221957697992679e+66', '8.6705409686234064e+165', 17, 2);
    T('6.831190842985635882733977228063142721015377198e+84842050693034', '-1.656659689011665999698731954969036216230195296e-359639714835557', '6.8311908429856358827339772280631427210153772e+84842050693034', 45, 2);
    T('6.9060425586541932934624337452643e-84250954001148', '1.186089669777001574476628920849552071450172365013e+573509013649', '1.1860896697770015744766289208495520714502e+573509013649', 41, 2);
    T('-8.37849465505909192059292478928660085481959269566773947326e-749136102853', '848145.33687', '8.481453368699999999999999999999999999999999999999999999999999999e+5', 64, 1);
    T('4.5293684171548038868407216228789370619878e-7568', '8.3762151862987592e+60323800920583', '8.3762151862987592e+60323800920583', 28, 1);
    T('-6.6325733798369e+659195090720', '8.59085e+855', '-6.63257338e+659195090720', 9, 5);
    T('-5.0045e+13', '8.76578227516800247500760652e-9116257983832', '-5.0044999999999999999e+13', 20, 2);
    T('-5.4678127205501921100982846096197075981171582e-739744698', '-4.629110872310351894e+94389617', '-4.62e+94389617', 3, 2);
    T('-8.224201901094197669168287e+7688259057630', '-2.29697598842782738773249087145045e-1628656094', '-8.224201901094197669168287000000000000001e+7688259057630', 40, 0);
    T('-9.513904807199122991632490039560294e-377653', '7.134443111417089957700046594065e+299822', '7.134443111417089957700046594065e+299822', 57, 5);
    T('-7.7152243407913514130e+5217', '-6.0187259377679e-399864', '-7.71522435e+5217', 9, 3);
    T('-6.66363824502645942944053595057367e-35536182', '-7.6709826634186545100118996825056771007074014975275705161e+76186', '-7.6709826634186545100118996825056771007074014975275705161e+76186', 73, 5);
    T('-7.9698049601042882486028444749945411634e-506505048531', '3.316635042966914449297112397443420e+197226357539', '3.3166350429669144492971123974434e+197226357539', 32, 6);
    T('2.30592578564e+93465188997', '8.6399265640697213842230685188798877713592540115126951029862e+89', '2.30592578564e+93465188997', 89, 6);
    T('-6.284113422050021889746876685658607290720238914180547139964e+99198', '-8.282967103067954092857857498601351946692e+5620607427988', '-8.282967103067954092857857498601351946692e+5620607427988', 44, 4);
    T('-3.673145090363957886533680031965e+48786321', '6.5413851827638637808e-989042134532955', '-3.673145090363957886533680031965e+48786321', 88, 0);
    T('5.6194691205727274620741972e+78837', '-6.19049810393270170931512091591656622510060090582559954e+798506511', '-6.1904981039327017093151e+798506511', 23, 1);
    T('-1.096695166864386152042759218558697604765136893550133e-40', '6.71370760608934830059970049911861651595e+62004', '6.7137076060893483005997004991186165159499999999999999999999999999999999999999999999999999e+62004', 89, 1);
    T('-7.143383558730508221758004793245333743e-39', '4.97000477509788736921094516929243124652280e+1206', '4.9700047750978873692109451692924312465227999999999999999999999999999999999999999999999999999e+1206', 92, 1);
    T('-62060773.9382077217001103009261312407665689741578858', '6.9944355271038181963462749579239551172681225015106828e+19002', '6.99443552e+19002', 9, 1);
    T('-3.371306849241612078476456e+96718924', '8.885939832963018344752837761907486332875164897515744824495e-568617', '-3.371306849241612078476456e+96718924', 80, 3);
    T('-3.9997905272000903566042717372473831360132313122347140e-321', '9.726945880034098818377540090334818477907056930340349832803e+62106557', '9.726945880034098818377540090334818477907056930340349832803e+62106557', 99, 2);
    T('-3.0348612719528515650676021259386910359438213041882136e+522108456230', '-8.38946777089606975258221809416265120094016608171292848e+99304181443051', '-8.38946777089606975258221809416265120094016608171292849e+99304181443051', 54, 3);
    T('6.735899199429509906692549997514056533503243839893810243095e-772', '-1.806080e-895130916', '6.7358991994295099066925499975140565335032438398938102430949999999999999999999999999999e-772', 86, 3);
    T('-1.6493890431e-556972484816063', '-2371683174734557415284571737987066238526.4224', '-2.3716831747345574152845717379870662385264224e+39', 52, 1);
    T('3.29e+739581072652', '-6.0099660473844754609239114331135533583668191643097564444e+109', '3.28999999999999999999999999999999e+739581072652', 33, 3);
    T('-4.0e+212621', '-7.684989683274241939066035e+69218', '-4e+212621', 29, 6);
    T('8.95163353030777885662078139319807599576e+86438374349', '-2.8707502301388736720510e-765', '8.95163e+86438374349', 6, 1);
    T('6.7402091171527129942007617e-684666333', '1.23389606487031493e-349', '1.23389606487031493e-349', 97, 6);
    T('1.11398861870394875063198189209539863573065251318e+43032', '6.1834816556481370562333555716361287174216216905599530e-476190650', '1.11398861870394875063198189209539863573065251318000000000000000000000001e+43032', 72, 0);
    T('9.6007420712829188787125107379326077177330981e-428585169', '-1.17110323829e+68456', '-1.1712e+68456', 5, 0);
    T('4.612256144455355230514763451030819283864756392809302e+372975384', '9.4876892109054237171960869125486774194193479919595904102e-212258418', '4.612256144455355230514763451030819283864756392809302e+372975384', 78, 4);
    T('8.445955e-2447', '-8.07623271699156225e-58554556565', '8.44e-2447', 3, 3);
    T('-1.0371972800586788859e-20672155', '6.0382867254908041426615578920402e-391201238142', '-1.0371972800586788859e-20672155', 47, 5);
    T('-8.1070020326223154500e-94309745725', '2.05722271797899923167339476603089080225479326433826027681e+34184622340211', '2.05722271797899923167339476604e+34184622340211', 30, 0);
    T('5.05912144144801767437e+12229636428', '-5.736413335115225477110409871642242728118496527e+954', '5.0591214414480176743699999999999999999999999999999999999999999999999999999999999999999999999999999e+12229636428', 98, 1);
    T('-2.28076e+604210233', '6.04103857976116401793910e-16', '-2.28076e+604210233', 40, 0);
    T('-5.5109295333514701253445061131690668811625739583058371e+784500', '6.29829737839917064285793177e+71914', '-5.51092953335147012534450611316906688116257395e+784500', 45, 2);
    T('-3.27021775210887656085662456045730e-77346900157068', '-2.92436545e-5421767165507', '-2.92436545e-5421767165507', 53, 5);
    T('-2.5942049333730335209636991941242319289e+629', '-3.77826927902125058967845589e+3135369', '-3.7782692790212505896784558900000000000000001e+3135369', 44, 0);
    T('5.4625220789902664223904627888e-487', '8.82564962275523591525494313581e+99744919056', '8.82564962275523591525494313581e+99744919056', 63, 6);
    T('2.859008218735726378202026265246218357744470433e+467695962889758', '9.8066e+314800', '2.859008218735726378202026265246218357744470433e+467695962889758', 47, 3);
    T('-1.61294211827218380721834786148263234372079385249497679870e-4177211594660', '-1.5176201939969e+86679192', '-1.5176201939969e+86679192', 84, 5);
    T('-1.28875017692824294553819170910294647069789e+50', '7.9263796598670249074545800759407e-25117234', '-1.28875017692824294553819170910294647069789e+50', 88, 6);
    T('6.64877920271633028707833213206156644259883989176e+366295309409', '3.5301790e-7416930041016', '6.64877920271633028707833213206156644259883989176e+366295309409', 56, 5);
    T('1.25059410263984704657974e+76683467163', '-9.9334128376e-76524603', '1.25059410264e+76683467163', 12, 0);
    T('5.96563753182136558258753441826815865e+69351521897', '6.71741573655842513821e+846', '5.965637531821365582587534418268e+69351521897', 31, 5);
    T('7.37528515974038100296001939763828431956e-96553', '3.00600024064100378328e-6777831548', '7.37528515974038100296001939763828432e-96553', 37, 6);
    T('-7.98697556140333e-4742185347', '6.830781698036623228273445776198569931e+101', '6.830781698036623228273445776198569931e+101', 78, 4);
    T('1.8679280521e-497', '-6.442665436847611354768390054695793859e+525196200141', '-6.442665437e+525196200141', 10, 0);
    T('-6.593102719626660901806656964051882e-3934527', '2.7232033411124270423294389078435631982612945598980921184678e+660', '2.7232033411124270423294389078435631982612945598980921184678e+660', 97, 6);
    T('-4.6926e+64337499', '-1.23163402848738262452e+3910', '-4.6926e+64337499', 13, 5);
    T('6.24814988715635370221856842420741579913162092056211646582e-8856324558', '-3.74245844805227095778704838334258580895350978417346896e+508607141', '-3.74245844805227095778704838334258580895350978417346896e+508607141', 99, 5);
    T('5.80685088456084824024491395271844157e+485842', '1.094735278e+73128467694', '1.094735278e+73128467694', 18, 5);
    T('6.00305257266331690005338627536155133e+3656085162509', '2.0217392860928932838496835598e+84', '6.00305257266331690005338627536155133e+3656085162509', 77, 1);
    T('7.36778005012894128467998e+1812', '-4967100673411551664974490128642957650121201191446924124.18725', '7.36778e+1812', 6, 1);
    T('-2.1520531545619668814944892553207961930788371804181919497e-950937866', '-1.35677830e+1336', '-1.35677830000000000001e+1336', 21, 0);
    T('2.5144716233987124836655974109214505811e-11027', '7.44329194120020942263494740338258e-9669692', '2.5144716233987124836655974109214505811e-11027', 40, 1);
    T('-534843522830311663835362721469704140141234871133527.14', '-9.702696881784184378127429606072e+32', '-5.348435e+50', 7, 5);
    T('3.3921791106301031019144586e+838259194768', '1.195043e+2339571129', '3.3921791106301031019144586000000000000000000000000000000000000000000000000000000000000000001e+838259194768', 92, 2);
    T('-2.9860372541356067235966382873415077178704055419e-619848305118', '-1.709691914097880782959855776001006339231699932859592e+90', '-1.709691914097880782959855776001006339231699932859592000000000000000000001e+90', 73, 0);
    T('-3.02927500478967943643040845408696033202160056155e-8859', '-4.56055989389842545350430020417275042e+47187161807', '-4.56055989389842545350430020417275042e+47187161807', 78, 1);
    T('7.85304023445172732038205781e-763637672997', '7.248610153225926e+7401105634', '7.248610153225926e+7401105634', 33, 6);
    T('7.62219130823787029798091369008586991196241203000774776e+8749681', '2.77e-942', '7.622191308237870297980913690085869911962412030007747760000000000000000000000000000000000000000001e+8749681', 97, 2);
    T('5.2934585152256346371069570685540999694485919e+104', '8.34764882526408586753886188361316836850619366248991826388028e-49154', '5.2934585152256346371069570685540999694485919001e+104', 47, 0);
    T('-1.33890002844244e+3785896889736', '-7.6008359450770596591465402487393067187954641951101645728e+8004300', '-1.33890003e+3785896889736', 9, 5);
    T('-9.390432304730471598160703007250556515093845072296418148685e+21418132', '-1.5608618121142000936084733929013893316e-18862897277', '-9.4e+21418132', 2, 6);
    T('9.5905705029e-435', '-6.100317883715923385014380e+18314', '-6.100317883715923385014379999999999999999999999999999999999e+18314', 58, 2);
    T('-1.774400509333111e+6553', '5.4078958764760875498526481431e-752728454006997', '-1.774400509333111e+6553', 77, 3);
    T('1.2229720080407098041800725764222437593305140950091069964701e+57470', '7.18871537988376454798558240460862231292071014581e-596', '1.2229720080407098041800725764222437593305140950091069964701e+57470', 95, 6);
    T('7.4934995e+37', '7.0431954967938932464824208e+14429', '7e+14429', 1, 6);
    T('7.84246139421267947583734522807064909852120670215e+95932452462', '2.72298025082141612960254846578665e-75964646', '7.8424613942126794758373452280706490985212067021500000000000000000000000000000001e+95932452462', 80, 2);
    T('-2.309692129605963143664040561286584523242333538841976e-8961049', '-9.6358772859805884542124137814284340958074033207e+65144672894262', '-9.63587728598058845421241378142843e+65144672894262', 33, 4);
    T('2.66029840938588352940215970528760885276114336e-78598', '9.93025026187107537686051817e+283442', '9.93025026187107537686051817e+283442', 84, 3);
    T('7.05941594118772049471984141469110031599e+39', '-4.4981928314839985717093e-364064', '7.05941594118772049471984141469110031599e+39', 86, 0);
    T('2.9100780317847165468081223671489278826248450241531703069e-2506954347244', '8.5235787955781377982459774312128020648716404104554801e-33932214', '8.523579e-33932214', 7, 0);
    T('-3.175975210863452054373072583865838e+200575162', '2.465866744490724974388160573132002455958919547069e-2166353', '-3.175975210863452054373072583865838e+200575162', 81, 0);
    T('-7.34024071710693718e+18533', '5.283430979359791319280867903354073748e-675049178', '-7.340240717106937179999999999999999999999999999999999e+18533', 52, 2);
    T('-7.27048450095905590425867762963519815013768837674207973281437e+5385837329', '3.87049534277049044324e+9238018', '-7.27048450095905590425867762963519815013768837674207973281437e+5385837329', 99, 4);
    T('-7.04705598755623578501470207101373517788788e+975751061552607', '7.988194109982759e-728', '-7.0470559875562357850147020710137351778878e+975751061552607', 41, 2);
    T('-4e+5', '9.54645881e+705766853037', '9.54645881e+705766853037', 67, 5);
    T('2.4734495537973120781712171331425511412204415e+387834790', '2.410953054025786292436821342949e+5674', '2.4734495537973120781712171331425511412204415e+387834790', 94, 1);
    T('3.624692853497427307289413768575581e-149147166204', '8.93282350116426926e+38310295205626', '8.93282350116426926000000000000000000000000000000000000000000000000000000000001e+38310295205626', 78, 0);
    T('9.22e+29058480621212', '-4.5831519461841e-56303420172752', '9.22e+29058480621212', 4, 2);
    T('8.2e-7049698', '2e+1720', '2.000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+1720', 85, 2);
    T('591676190870456298870.54382', '2.8948614324139455234507055933834733e+565238', '2.8948614324139455234507055933834733e+565238', 85, 1);
    T('5.46457704061206055279874764117e+54750173090166', '3.4899224949780e+940656308458107', '3.489922494978e+940656308458107', 29, 4);
    T('9.037219576034046703550401758386443937e-38908374943595', '-5.2690e+148', '-5.269e+148', 84, 3);
    T('5.0942726e+207787', '4.93e+75', '5.0942726e+207787', 98, 4);
    T('-6.424474582957819907223948242477287233869061591309432562848e+77163042289', '5.739425383577745e+6170', '-6.5e+77163042289', 2, 3);
    T('3.62803649843157267358706540627879024859712596859256172958763e-30661851044205', '-4.86180141375535877748557e+28394522', '-4.86180141375535877748557e+28394522', 83, 6);
    T('3.840536631e+881361', '5.415467033e+2203589337', '5.415467033e+2203589337', 74, 4);
    T('4.13e+385393984741', '6.83916978287293582712222119231818445030731771604687539563e+3731771638', '4.13e+385393984741', 23, 3);
    T('4.836169274731612203e+291', '-4.1802481893142931736381766097359279499521364426657764e-3380809', '4.836169274731612203e+291', 23, 4);
    T('-1.8223524609363e+50139997', '-6.45573849527294278582622859097711474299322448112558240302e+4249224', '-1.8223524609363e+50139997', 55, 5);
    T('-4.154935341854800e+426055', '-3.40796554495728847952170224505237110299382559531460143400e+17674728963867', '-3.407966e+17674728963867', 7, 0);
    T('-8.24499559938747922535332653490255452296646932e-7893254724148', '-7e+41897231651136', '-7.00000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+41897231651136', 90, 3);
    T('1.052789696389317978420362964565246585e-19', '8.4943043243714884277032919e+8491398744', '8.4943043243714884277032919e+8491398744', 61, 3);
    T('4.5997219132743617502671465365959178303062010792e+51', '-4.0065087000742638867675120456009167500753767859e+5487029759', '-4.0065087000742638867675120456009167500753767859e+5487029759', 61, 4);
    T('-7.2161596090e+296022230311', '-1.18912042179334225839821928752317941e-1522294160', '-7.2161596090000000000000000000000000000000000001e+296022230311', 47, 0);
    T('-3.0529004003938696895199353760811480309555637542950e+61243341244', '-6.854522539964467104862124166826006600721110048e-3505743711646', '-3.052900400393869689519935376081148030955563754296e+61243341244', 49, 0);
    T('-74364820509880.9011909525454381811395474', '-59190247125345570085607794063835288752173449003.473476178', '-5.91902471253455700856077940638353631169939588843746671305454381811395474e+46', 87, 6);
    T('-4.4985180825067022704948848667579238081599631342249871642202e+74', '8.61643e-74', '-4.49851e+74', 6, 2);
    T('-9.0876783200675162801919506232e-32', '-2.3825521673810044793476950706780172276966e-389', '-9.0876783200675162801919506232e-32', 88, 4);
    T('2.79522631647496502890e+26', '-5.884780325116377146721332390e-46082447063', '2.7952263164749650289e+26', 99, 4);
    T('5.850530188486392072189295687743e-4809042530119', '-6.363921079910283379694921938e+57563451189', '-6.363921079910283379694921938e+57563451189', 96, 0);
    T('1.2547830751706115643589087008554840259883257e+7971568255', '-9.41795656282137863891e+9426094762770', '-9.4179565628213786389099999999999999999999999999999999999999999999999999999999999999999999999999e+9426094762770', 95, 1);
    T('0.0000860866239940237061814593965256231480500490103965', '-4.7296827233677717451993e-80122718', '8.608662399402370618145939652562e-5', 31, 5);
    T('-4.0225336990848650400303535106859963587e-609404893', '-8.394707157322401616216490586057908802773476554968314e+38797310710', '-8.3e+38797310710', 2, 1);
    T('8.368829875144417231063477956902089360424006068553e+5392771830', '3.094928761908473787288681145728880628810436927701266404818e-7647', '8.368829875144417231063477956902089360424006068553e+5392771830', 75, 6);
    T('-4.805458597765890547338242922292806e-43461419983769', '2706682739079.9894927755495866', '2.7066827390799894927755495866e+12', 30, 5);
    T('3.479786893077990853678137783715425630728955708e-9627', '-9.83560060046024e+77107966162923', '-9.83560060046024e+77107966162923', 79, 4);
    T('9.4482645948312142600670774823e+101', '3.8202288953742255958531546734545844358251078062917e-657', '9.4482645948312142600670774823e+101', 32, 5);
    T('2.045908113875546614590538135235502216437052674049098e-9579002169', '-3.986702378849324095372229539149673226364498951244797863e+700155466', '-3.986702378849324095372229539149673226364498951244797863e+700155466', 75, 6);
    T('8.879405785742412061079312147407930061e+2135207134426', '1.9984015291472668884325e+9665943', '8.879405785742412061079312147407930061e+2135207134426', 87, 3);
    T('-9.4114725484708630268145731860057100745991e+76587', '1.772326272081572327381635850891044073737e-12', '-9.4114725484708630268145731860057100745991e+76587', 83, 0);
    T('-7.4569071644969e-49381641612', '9.44950339742302482592493957000277796479615e-429', '9.4495033974230248259249395700027779647e-429', 38, 3);
    T('-8.961970647348000717216174311604549787870843380180e+8193956', '-1.3321482148297860552355699390001119222631676979923630866e+1330275001', '-1.332148214829786e+1330275001', 17, 2);
    T('-9.6006201316438609506310570546794881779575079083e-89072134197639', '-4.21714610244296985891014719053005088078519437496e-548142', '-4.22e-548142', 3, 3);
    T('7.974026e+25183704349312', '5.85166802145645254003324343821685374816629355182497990239265e+23212436', '7.974026e+25183704349312', 51, 5);
    T('-8.26476688175000616432222285852874954994019254011139e-64002', '7.752031830489875257634989897108302763360078759394196e-8193850921', '-8.26476688175000616432222285852874954994019254011139e-64002', 76, 4);
    T('-7.46047206e-31116446150226', '5.5944696773779044126426468751008230304e+5219005347409', '5.59446967737790441264264687510082303039999999999999999999999999999999999999999e+5219005347409', 78, 1);
    T('-4.3022763602674068239e+9000504', '-4.90043113442534056526462545892743e-421985900479', '-4.3022763602674068239e+9000504', 38, 4);
    T('2.2053949405e+11', '-7.314286659180547379473621780074024e+10359', '-7.314286659180547379473621780074024e+10359', 55, 3);
    T('-6.51333070157617119595663876664e-155567', '-3.183135489442609647062456234809333194534365328e-107180240952', '-6.51333070157617119595663876664e-155567', 90, 6);
    T('-5.5031169903e-979', '4.13534838613161049977e+11760444710373', '4.13534838613161049977e+11760444710373', 57, 2);
    T('8.9722385082850411147e-165360007147', '4.65577023830273752373940547156102051791319673669042194e+4263367460286', '4.65577023830273752373940547156102051791319673669043e+4263367460286', 51, 2);
    T('2.392098730591571369127672013618219e-49186589953006', '2.0252544011361392300078e+1709038', '2.0252544011361392300078e+1709038', 86, 3);
    T('-2.61192046638499985010496203119946321765050045847453e+630015544507', '-3.9e-7607277341', '-2.61192046638499985010496203119946321765050045847453e+630015544507', 88, 5);
    T('7e-68394', '-1.344183849344145400511e-5095814356', '6.999999999e-68394', 10, 1);
    T('4.2654380105943138331116549089111095948382024039757445974902e+114', '6.0122560392708156624200864980273850151520836212088134e+715', '6.012256039270815662420086498027385015152083621208813400000000000000000000000000000000000000001e+715', 94, 2);
    T('5.3171200401683970295954987481635e+9991', '3.23785207306921093348022025018151603e+1490885416', '3.237852073069210933480220250181516030000000000000000000000001e+1490885416', 61, 2);
    T('-6.092887189303547845918440085671e-4751', '-2.148560902072821553015683133e-537076', '-6.092887189303547845918440085671000000000000000000000000000000000000000000001e-4751', 76, 0);
    T('-9.64313010400230394429980753e+31', '-7.30732656702711125049148271342680999207148910e-523444182556392', '-9.64313010400230394429980753e+31', 77, 1);
    T('8.28486e+4598955405', '0.004117548204435', '8.28486e+4598955405', 78, 3);
    T('3.8848284442261731313991721029825e-3393389', '-2.61259475925418937965842878428051805698298473478114208252397e+267777', '-2.61259475925418937965842878428051805698298473478114208252397e+267777', 85, 3);
    T('5.152722602e-25', '-4.96541510318768269142183455497242839953496442806310568318e+83789596738', '-4.96541510318768269142183455497242839953496442806310568317999999999999999999999999e+83789596738', 81, 2);
    T('7.51871649684377788714e+79128592', '-8.0950072400371e-172988', '7.51871649684377788714e+79128592', 73, 4);
    T('61061467935779532069738875610901150506.760', '9.937129922257378099056776358191084635625344473e+33436558481547', '9.937129922257378099056776358191084635625344473e+33436558481547', 85, 6);
    T('-3.498736729464660192556512760924737603393377433037e+60905131', '9.756283956269738836003481300513983356313311484e+52', '-3.498736729464660192556512760924737603393377433037e+60905131', 67, 5);
    T('4.415511941447283321123e-95510', '-2.07330927305949610465003334404e-527192357', '4.41551194144728332112299999999999999999999999999999999999999999999e-95510', 66, 1);
    T('8.634227088785e-4478533329893', '-1.39685031384848880224293116226e-485315949177', '-1.39685031384848880224293116226e-485315949177', 79, 0);
    T('-3.9923806439362892412573182297918690e+42', '-9.29471075558437061236846886843190327897931556335512968e+78', '-9.294710755584370612368468868431e+78', 31, 2);
    T('-5.331623610852867606394900397945137205212656214269e+910325731', '-4.48860305462351e+70900291', '-5.331623610852867606394900397945137205212656214269e+910325731', 67, 6);
    T('6.41264035054981522501077414279560836e-7701751873990', '-8.47498893909364869635126064606588744878159951742180110532e+9633715', '-8.47498893909364869635126064607e+9633715', 30, 5);
    T('9e+919766', '7.5841096216433048585e+761907151', '7.5841096216433048585e+761907151', 26, 5);
    T('-7.0260619426721482e-562623103751', '-9.733613586033631140902892160107937e+662361', '-9.733613586034e+662361', 13, 0);
    T('-6.198367950627124921194378248622361845714495722e-5650780', '2.83841902295763663186231534474008843406583e-99241', '2.83841902295763663186231534474008843406583e-99241', 99, 5);
    T('1.0629e+230', '1.86160998460634320636422693196697717315283032415e+462378404600', '1.86160998460634320636422693196697717315283032415e+462378404600', 77, 1);
    T('-3.581061545350413418830623196999551017428279832586705e-7904', '3.7956919356731571146981e+319059036', '3.7956919356731571146981e+319059036', 71, 5);
    T('-5.31887626548228853850611724439180614e-43', '1.3094517331408157e+19117095966', '1.309451733140816e+19117095966', 16, 6);
    T('-8.871235e-6361', '-694796760620902198289193044473164209910', '-6.94796760621e+38', 12, 4);
    T('649982687810884625846090.0', '9.500992846512084133987912366103050e-501797', '6e+23', 1, 5);
    T('-5.4157324119060600212905486854e+907646464499978', '-2.25226259760968184e+15894', '-5.41573241190606002129054868540001e+907646464499978', 33, 3);
    T('8.2452177539051186758511991412741338755964104981772267980e+79447673639230', '8.910771276e-5719015', '8.2452177e+79447673639230', 8, 3);
    T('-5.161081457803734451281321676300571e-17024112440829', '2.7660716353737775660498476161296809116449096659787e-443158922143', '2.766071635373777566049847616129680912e-443158922143', 37, 6);
    T('-2.6412020536642133469789479370927535615217312693361e-8742135', '16564487311.64778704', '1.656448731164778704e+10', 76, 2);
    T('910967424468117512039326613915193767122432617993379815736', '2.7781523333674005e+759886161763200', '3e+759886161763200', 1, 2);
    T('6.88217137312177764432805148506110633347571937649540180e-57832050152604', '-7.2841536121257712e-76113982', '-7.2841536121257712e-76113982', 93, 4);
    T('-2.00e+94556828', '9.08905e-7410', '-1.9999999999999999999999999999999999999e+94556828', 38, 1);
    T('4.8963814905737692846113806021114780188741362326480828276861e+8211766329647', '2.3944092312888001057729082294544716660946611378398258558e-810837750', '4.896381490573769284611380602111478018874136232648082828e+8211766329647', 55, 4);
    T('7.756946049312747948e-89544838259370', '39859330246621808342312.535368', '3.98593302466218083423125353680000000000000000000000001e+22', 54, 0);
    T('-5.40891591602e+17060493234415', '1.367729730319143804384444769560539853259311689855905e+196907', '-5.40891591601999999999999999999999999999999999999999999999999999999e+17060493234415', 66, 1);
    T('-2.0809e+2502630', '9.90816222308470255e+670155607224', '9.908162223084702549e+670155607224', 19, 1);
    T('-6.9493412765200454224249375092864609e+7827991060', '-7.94672440264836153418612151003345448277e+141306746', '-6.94934127652004542242493750928646090000000000000000000000000001e+7827991060', 63, 0);
    T('-4.5376297138548659e+43267380503', '-2.089638642148839246136e+14186573784', '-4.53762971385486590000000000000000000000000000000001e+43267380503', 51, 3);
    T('-1.57295395698644132110988289251931714305e+248159938594888', '9.24565485717763934111800119707823558e+9295775870117', '-1.57295395698644132110988289251931714304999999999999999999999999999999999999999999e+248159938594888', 81, 2);
    T('-3.406283292180405287075664604900078458232814550473158972266e+5639641', '5.105923230804e+870269631105238', '5.105923230803999999999999999999999999999999999999999999999999999999999999999999999999e+870269631105238', 85, 1);
    T('-7.99398138169228047376981342796945623985811015602690096237e+793783502541', '8.101114563e-479035702743', '-7.9939813816922804737698134279694562398581101e+793783502541', 44, 2);
    T('7.87508076103e-12713219092', '6.2396979034964072020909124e+131384', '6.2396979e+131384', 8, 4);
    T('4.83471191109272816e-2345103176', '4.2967901343000074094535909876642009293357159875861777834650e-45568', '4.296790134300007409453590987664200929335715987586177783465e-45568', 95, 1);
    T('7.24692060321113609366436056969051311391781506e+53', '8.737449256501e+661923099169', '8.737449256501e+661923099169', 20, 6);
    T('4.898293226666197777661760299e-6274728', '-8.408e+22956175726442', '-8.408e+22956175726442', 27, 5);
    T('3.59959316707325612745710779312953362560956637e-398', '5.24877770041330327331950e+9404909', '5.2487777004133032733195e+9404909', 84, 5);
    T('7.6222089951949973707576e+47960616', '-4.92476066870159451797217463206048177626113446e+447', '7.6222089951949973707576e+47960616', 77, 4);
    T('-1.8804885999184070e-33828883997', '-3.62477772784147049954421759240188669819875174005232459857e-842', '-3.6247777278414704995442175924018866981987517400523e-842', 50, 2);
    T('-3.18686765180908433266362565335910e+39', '-8.0046941815274154979043520180284670783464805265546938096537e-16975386207237', '-3.186867e+39', 7, 2);
    T('-5.1519939e+637635195', '-8.7421679003207809538300199955971155933735755355340937462799e+33630008', '-5.1519939e+637635195', 84, 5);
    T('5.73738383350410570527305e+323', '6.9183432916433407368800159777053440796522055800e+140', '5.73738383350410570527305e+323', 53, 5);
    T('-6.10038343987437030720929102773e+5944845203', '2.24400400846160968651e+44416703044', '2.244004e+44416703044', 9, 1);
    T('-7.8075554936037039556727214963121965348454814451058507e-332021156037', '-3.21909e-6573', '-3.21909e-6573', 88, 6);
    T('-94957656653919245434291380240433591564391.33630348002957198', '1.9603437672481531166855327196e+4098941672442', '1.9603437672481531166855327196e+4098941672442', 53, 4);
    T('-5.847615076424142e-333704129510', '7.974090534748218862007753869486334726e-292', '7.974090534748218862007753869486334726e-292', 43, 2);
    T('5.247164738596657955841976537916598e-253064911556393', '-9.59393298413351644769963051718069818469838e-217', '-9.59393298413352e-217', 15, 3);
    T('6.1440537936818686766572031943056287499300683884955495e-320075567532602', '-7.485681240319902919496922865883309238173e+565', '-7.48568124031990291949692286588330923817299999999999999999999999999999999e+565', 72, 2);
    T('-7.1306930909407273377484633770712326630021343367943169e-9705', '7.3753705445694449041909e+51', '7.3753705445694449041909e+51', 82, 6);
    T('-3.90464624711018239317e+272014771035367', '-7.8424264e-782952378622', '-3.9046462471102e+272014771035367', 14, 0);
    T('5.8359739061758724461822290319198479e+597505', '-8.3757424893200078869088550510e-642', '5.835973906175872446182229031919847899999999999999999999999999999999999999e+597505', 73, 3);
    T('1.823572037550188720338332281359183491643082460060', '4.1352400123853657191734857647294708e-98150150', '1.8235720375501e+0', 14, 3);
    T('-5.69308320316887179238388614657233534933127255673945151529e-4620326958', '-3.7654323772019006939707684598638e-30965753', '-3.7654323772019006939707684598638e-30965753', 50, 5);
    T('9.403004e-45724820', '8.043315e-188', '8.043315e-188', 49, 4);
    T('-7.350624778949470768537639829874172643672301207278688242e-20453', '-1.198269529515868e-36793328829862', '-7.35062477894947076853763982987417264367230120727868824200000000000000000000000000000000000001e-20453', 93, 3);
    T('7.4819861041645142702639723918881671828473541e+251403471376', '6.4947867442069419e-2359389', '7.4819861041645142702639723918881671828473541e+251403471376', 99, 6);
    T('6.6275009047804289767812297488264883481196788398175086271e+5245522925140', '-3.510e+9066', '6.6275009047804289767812297e+5245522925140', 26, 5);
    T('8.1279426e-55', '2.5853785554509900563718281689631500995024004e-9195271426758', '8.1279426e-55', 69, 5);
    T('9.21385936837233e-658884705417011', '5.6151235248778004419992234903482e-36911825675', '5.6151235248778004419992234903482000000000000000000000000000000000000000000001e-36911825675', 77, 0);
    T('4.7825923655013933e+78417', '-1.896972105035394678104746391501543833992402120372e+695646020', '-1.89697210503539467810474639150154383e+695646020', 36, 6);
    T('4.64310992674443083307074057366101364474e-4400170208', '3', '3e+0', 80, 6);
    T('-5.11080344586612391038e-128452', '9.550641e-8902009875', '-5.1109e-128452', 5, 3);
    T('3.0661207185633377319370743654394860618220533377632374e+82', '-5.5407129826015174880e+213323234', '-5.540712982601517488e+213323234', 49, 0);
    T('-9.76620068612422797264e+94545643129', '3.60581903950299279829259582900843503068917e+3841267832', '-9.76620068612422797264e+94545643129', 92, 4);
    T('-8.05760723078607109167947313e-21650242809', '-5.079964594817954864816447442703986148e-649821', '-5.079964594817954864816447442703986148e-649821', 83, 2);
    T('-2.6552253850239117306171705269815868214662574438601987442e-7676680768', '6.3519222941115819275902581530303427e+590357850160', '6.3519222941115819275902581530303427e+590357850160', 66, 2);
    T('2.46277747892117819857257520616254336587938504076205848e-465', '2.84955797666317527e-4685', '2.46277747892117819857257520616254336587938504076205848e-465', 86, 1);
    T('-2.26710512985481e-24445707', '3.4734013258998549e-314539055395176', '-2.2671051298548099999999999999999999999999999999e-24445707', 47, 2);
    T('2.8106026168557763480514530841693335242214234522310279380669e+344314820263', '-8.60040230514e+61396925853', '2.810602616855776348052e+344314820263', 22, 0);
    T('8.08331734256867229964006560e+11090', '8.5790084048615034585083871426168507172898282123e+49', '8.0833173425686722996400656000000000000000000000000000000001e+11090', 59, 0);
    T('-5.4e-40626', '5e+16', '4.9999999999999999999999999999999e+16', 32, 1);
    T('42302415379834269708037600923.6141', '5.85567796838906283295624e-38735833290170', '4.23025e+28', 6, 0);
    T('-8e+122972', '-7.1943384392131439902006825564788592426851042547003563870648e+16712194', '-7.1943384392131439902006825564788592426851e+16712194', 42, 6);
    T('5.844080527493192848828199564564e+5392872', '3.75741782944573373733883264377909694209997704411434875354e-9716', '5.8440805274931928488e+5392872', 20, 4);
    T('34191819974977362590019.502', '-3.716392e-17', '3.5e+22', 2, 2);
    T('-1.764981877997871844137044708293551689630e-835', '-4.383e+921546849', '-4.383e+921546849', 52, 5);
    T('5.44901663259087795199346929717509301658491022355e+785176033', '2.2417269955374582166e-592', '5.449016632590877951993469297175093016584910223551e+785176033', 49, 0);
    T('8.23663399698488257980858574096802354713922456246955e+1961725372', '-4.8883172290435098411497818666745157405174516e+7329', '8.23663399698488257980858574096802354713922456246955e+1961725372', 79, 5);
    T('-1.77284e+18240772075', '-5.6640408647898925770077926990e+428334690478342', '-5.66404086478989257700779269900000000000000000001e+428334690478342', 48, 0);
    T('5.80259757981999963658397091726061659888202e+60', '1.9983790e-951343902', '5.8025975798199996365839709172606166e+60', 35, 4);
    T('-8.207287356423268492030444066701770819239672097120298e-853694115', '3.9787526561263913987179014707032353000885e-777031939', '3.9787526561263913987179014707032353000885e-777031939', 96, 0);
    T('4.16934410978967202766123331677665149033284976e-613459672689591', '3.1438200414459551197455061e+49380681297822', '3.1438200414459551197455061e+49380681297822', 71, 6);
    T('8.7538659865469497182537345e+736133', '9.48330095640145541406132357567e+9813505355', '9.48330095640145541406132357567e+9813505355', 48, 1);
    T('361609823709358451.189202', '-4.440587233155203754922234889789366348759151922385983676e+64016956497541', '-4.440587233155203754922234889789366348759151922385983676e+64016956497541', 99, 3);
    T('2.7658e+1969', '-5.9368989477484999031510837313017440926815303889193557982633e-70206', '2.7658e+1969', 75, 6);
    T('-8.240252692767e+49217874216729', '8.9334883840202059772718701920161660024606358e+725624', '-8.240252692766999999999999999999999999999999999999999999999e+49217874216729', 58, 1);
    T('-9.5090766403213440745709e-7', '-1.351762489248376443456473623620904180169260e+174683', '-1.35176248924837644345647362362090418016926e+174683', 70, 6);
    T('5.643249123361369e-7111', '-1.30964898701926978417152892501187490e-97620557458', '5.643249123361369e-7111', 87, 4);
    T('-5.238626119797213844586483604356372936045838948519467319595e+1038', '3.501611968835541755952381112984703798951973107e+717895', '3e+717895', 1, 1);
    T('-2.771238244117416071482e+785', '-8.060661711028332e-426953', '-3e+785', 1, 5);
    T('-7.415034013226522797205455288729109824e-1334943164', '6.75664e+54483211860', '6.75664e+54483211860', 82, 5);
    T('-2.810107412953369535528379790774541827e+78909', '7.871932962446359770339416631145e+959408', '7.87193296244635977e+959408', 18, 5);
    T('2.3561374353476573613095947877470983548986e+96441', '7.2079066734204e-5222', '2.35613743534765736130959478774709835489860000000000000000000000001e+96441', 66, 2);
    T('8.717687081679747129397183947305996670e-4631020', '-4.146325723859164e+20', '-4.146325723859164e+20', 77, 4);
    T('-4.56549808681006139982343007461821121123e+8171451317759', '-6.18452048719279395029240756974225546076888641053565e+711671', '-4.56549808681006139982343007461821121123e+8171451317759', 70, 5);
    T('-7.513930968666358628225501820969619209341033402321e+569198', '-5.716072675328842774987e+556', '-7.513930968666358628225501820969619209341e+569198', 41, 6);
    T('8.891661490962663215e+41301831', '-7.878933015636151e+7684331', '8.891661490962663215e+41301831', 71, 6);
    T('-6.2667022343e+9095900075728', '-7.4745964701617255422792747532983183339765e-5436223983796', '-6.2667022343000000000000000000000000000000000000000000000000000000000000000000000000001e+9095900075728', 86, 3);
    T('-9.46569873723343730e-816860', '-1.5415183526153746819e+530167886', '-1.5415e+530167886', 5, 4);
    T('-9.077458087680928064453650540e-64259', '7.3950207160218792e-372367361795', '-9.077458087680928064453650539999999999999999999999999999999999999999999999999999999999999999e-64259', 91, 2);
    T('4e-79501220', '2.3269097806907913481405e+7758627', '2.32690979e+7758627', 9, 0);
    T('9.899180136798358163797e-480679050', '-8.8581360877156861772250654616880322834690562209990e+5019', '-8.858136087715686177225065461688032283469056220999e+5019', 96, 3);
    T('-4.918635624391682458930452366314999735596e-22', '-4e+8729612', '-4e+8729612', 10, 2);
    T('8.64905e+81235171', '-7.4279348385536396240626399000801017473324942601032348433668e-17560872727722', '8.64905e+81235171', 24, 6);
    T('3.55420821548084946823837714477651753659633000864e-40096', '-9.970795804941734e-758933291', '3.55420821548084946823837714477651753659633000864e-40096', 59, 2);
    T('1.1e+37738953763850', '-8.334731347511147219999634659671e-955005127871', '1.1e+37738953763850', 58, 6);
    T('-4.14764227845422875291853918620379620314918563805595573e+61', '7583652303491036231682118644520742.6919902540956081115193520', '-4.1476422784542287529185391854454309728000820148877438655479257308009745904391888480648e+61', 91, 1);
    T('-5.766e-7459', '4.6668846416871096423845043e-47', '4.6668846416871096423845043e-47', 67, 0);
    T('-5.9167760584758420584530300177074728032307530860e-290', '-9.59682393816628695861729e+9207725151', '-9.596823938166286958617290000001e+9207725151', 31, 0);
    T('-3.748119321240973217025111e-48465958153', '-6.78166990612057297027e-176032', '-6.8e-176032', 2, 6);
    T('-3.044434764006e+858455875', '5.73151273939722e+72696061014373', '5.73151273939722e+72696061014373', 47, 5);
    T('-7.3348701705979109725e-875748804', '3.4864008616745478867143058472112951022e+17501415929877', '3.48640086167454788671430585e+17501415929877', 27, 0);
    T('-8.4127820511273489400558085395e+33611466571887', '-4.9436775202479202679610e-597280', '-8.4127820511273489400558085395000000000000000000000000000000000000000000001e+33611466571887', 74, 0);
    T('7.27e+79248', '8.93214096127868e-9731998204924', '7e+79248', 1, 4);
    T('-7.32954749805846e+190893713902474', '-5.723301864068070e+719555', '-7.32954749805846e+190893713902474', 16, 6);
    T('8.08338459415097037981593452333456e-919088337772', '4.3658280306944919181139275517321787993912904214618e+955171', '4.36582803069449191811393e+955171', 24, 5);
    T('-9.746012388616302454630703167641e+36', '1.60e+9', '-9.746012388616302454630703166041e+36', 80, 0);
    T('-8.962889083519156061865848634970499332586126e+449969', '3.948151952576596814783838177226330588530e-54', '-8.962889083519156061865848634970499332586125999999999999999999999999999999999e+449969', 76, 2);
    T('6.510743e-707828209', '-3.7768711326913262287502714104535313027786e-491320965744', '6.510743e-707828209', 59, 5);
    T('7.86836117e+71740', '9.8614827839364142774261607452e+53843', '8e+71740', 1, 5);
    T('5.252e+342', '3.4959075880598098108026914188069746894e-7164', '5.252e+342', 84, 6);
    T('2.147314192008029765755156002006e+79220', '5.30854649e+3077006', '5.3085464e+3077006', 8, 1);
    T('9.9686003565894295045263328e+354119011', '2.5270711141970541224877802598700654086456563331250e+2760', '9.9686003565894295045263328e+354119011', 58, 5);
    T('9.84275023106153838137245591e-558834760', '-9.8599054842241330e+81730006944973', '-9.859905484224133e+81730006944973', 63, 5);
    T('7.390010708767836e-16', '-2.9e+74476863894980', '-2.9e+74476863894980', 58, 6);
    T('4.992498967212782495080e+788383576', '6.536501023829732618369986236167915947581863082478991e+32819', '4.99249896721278249508e+788383576', 50, 6);
    T('9.78496052e-1317483', '-7.1049941429595337984074588591426357583735226388e+560147979', '-7.104994142959533798407458859142635758373522638799999999999999999999999999999e+560147979', 76, 2);
    T('-3.02383277081332481114945570238e-972982613259286', '-1.32967122939972519988761e+3186888106', '-1.32967123e+3186888106', 9, 3);
    T('452296279787813163565.39', '-4.487764e-21945974633', '4.5229628e+20', 8, 5);
    T('-9.269024895226e+3779692838253', '0.6325', '-9.269024895225999999999999e+3779692838253', 25, 2);
    T('-5.95689675153679493274517085626019853060381253288803396921e+653067396197', '-3.023907403271999191164927850081187e+44940788865', '-5.95689675153679493274517085626019853060381253288803396921e+653067396197', 99, 4);
    T('1.45151415994701528979e-475', '9.9545892000234685857290700931576481128848845e+752851612', '9.95458920002346858572907009315765e+752851612', 33, 0);
    T('7.7126359782655685940724920663716472993458513895906214723e+55053222', '8.05467485088774166397725061469949e+15934132', '7.7126359782655685940724920663716473e+55053222', 35, 2);
    T('-4.394351428481e+7407', '-8.58895475421883187785e-56320', '-4.394351428481e+7407', 43, 5);
    T('1.22233285034132612225492747935388367150511094599309299417980e+640573374376', '4.829336766008524e+1547', '1.22233285034e+640573374376', 12, 6);
    T('-4.9402826072063537578936411894873634354910076e+94179372767', '-7.58857645478902e+46408412993689', '-7.58857645478902e+46408412993689', 93, 6);
    T('-3.58760656043744999111016940728e+60100341336229', '-5.656701662508810827528173643e+7737714', '-3.587606560437449991110169407e+60100341336229', 28, 6);
    T('3.29602669021386935762893965674203796698277183e-137776362627', '-7.33309541799e+50', '-7.33309541799e+50', 36, 5);
    T('-4.560090038256355397688e-950', '-4.33109762225796407259766491354979829535103784e-82109', '-4.5600900382564e-950', 14, 3);
    T('6.820251032374082048518020717487122498773143176481028379e-146070', '8.3387512527937652557187570499451524995076e+74413603404', '8.3387512527937652557187570499451524995076e+74413603404', 94, 6);
    T('8.004753732862380984141856347409893022464885e-2756202372773', '-1.61956306789954628849160532e-3251831378408', '8.004753732862380984141856347409893022464885e-2756202372773', 58, 6);
    T('-6.928292270622467491220603300600164053041167546661166e-18652438084', '1.6550713209600825613776902715649261616640780e-63789122883', '-6.9282922706224674912206033006001640530411675466611659999999999999999999999999999999999e-18652438084', 86, 1);
    T('-6.137785240326961261e-58640114', '5.029965405194e-97061085234725', '-6.1377852e-58640114', 8, 4);
    T('-6.792136830722668801691739313772206862225753751049669486133e+323536743', '4.32420371165e+83595688705546', '4.32420371165e+83595688705546', 83, 2);
    T('-9.648539302044500136773907460164518593787103456e-73889152457', '4.6881e-2742677069295', '-9.64853930204450013677390746e-73889152457', 28, 2);
    T('1.26449699840866590929326903976e-42371968', '1.5007364114843178947048703610778141832707508691595441483319e-994181080', '1.26449699840866590929326903976e-42371968', 65, 1);
    T('-781068525552879835548832432754054707672148.522', '-2.88389829205667350290186e+8845711204329', '-2.88389829205667350290186e+8845711204329', 84, 1);
    T('3.0832518e+498750', '8.785339933561658558507415883742008114511267485025524114619e+43838', '3.0832518e+498750', 23, 5);
    T('7.912690321986892006798823645090702e+70226589', '-9.450426e-71564107', '7.912690321986892006798823645090702e+70226589', 68, 2);
    T('9.4885016687881675273243143084853699198735070009e+211970048324', '1.58355348527544484861379606366533330743229638333e-37293', '9.5e+211970048324', 2, 6);
    T('6.896454078160536657204337485992776119518626615079016587962e+512726042787', '3.65451677283540191e-2341695633', '6.896454078160536657204337485992776119518626615079016588e+512726042787', 56, 4);
    T('-5.4505e-5722', '-4.696e+6934', '-4.696e+6934', 33, 6);
    T('431019738862455393774702513089826267779039141.43', '-5.78252848865044421446038e-7702386', '4.310197388624553937e+44', 19, 1);
    T('7.573997405779135174592268174571169125989793802393e-923164', '-6.6818755267269658610572580790815e+3310', '-6.6818755267269658610572580790815e+3310', 53, 3);
    T('4.2865554725775e+37978', '-4.601424117327102666815218772790153265e-37163288475833', '4.2865554725775e+37978', 65, 2);
    T('3.9953544548054218315146e+25', '-7.6962977961375043397699049949e+6809026', '-7.7e+6809026', 3, 0);
    T('-6.593188660283988198982548682070161e-295', '7.946963843184141028861108558120040066627776130913951891e-9318', '-6.593188660283988198982548682070161e-295', 51, 0);
    T('6.402764999574104601235758165496385526040406764754e-713366752618', '507754314498.061', '5.07754314498061e+11', 34, 4);
    T('-4.50067678189096550667849032127784833334432897312913381448558e+6584', '1.386007423155489437465511423094883958890812508123603e-1606925702', '-4.50067678189096550667849032127784833334432897312913381448558e+6584', 82, 6);
    T('6e+926376', '7.283542077790762602676729399e+6678', '6.0000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+926376', 86, 2);
    T('-2.36407841556435886534392726686262e+74031775612342', '8.567298860605778418718134360719608696e+57367819', '-2.36407841556435886534392726686262e+74031775612342', 35, 4);
    T('-6.87632295272588784251752626e-2983249966322', '-6e-17442805', '-6e-17442805', 97, 1);
    T('8.09047514465928289e-6132002045149', '7.92713326713062011533484711307529544062743688075900834414e-57257731', '7.92713326713062011533484711307529544062743688075900834414001e-57257731', 60, 2);
    T('-6.710444233274990085709809e+191935', '6.8352e+401254', '6.8352e+401254', 15, 2);
    T('-9.103276e-108916493916', '-7.7798151115024093900936767334149008774e+9741028018', '-7.7798151115024093900936767334149008774e+9741028018', 65, 1);
    T('9.9759513143651924706637760293e-978000', '9.47584902192536820388284001e-82960388786', '9.9759513143651924706637760293e-978000', 86, 3);
    T('-9.451415424825244660e+778761175', '-9.86232569775946234831728113428924760987029e+7803526346933', '-9.86232569775946234831728113428924760987029e+7803526346933', 75, 5);
    T('-5.979776108068405173665771624832e-83705895420', '3.44364290308e-557634789186', '-5.979776108068405173665771624832e-83705895420', 31, 5);
    T('-5.517463e-3239', '7.38641184552086304448604075252627587840448697454e-375659', '-5.517463e-3239', 31, 4);
    T('-4.531310095438799324242295929610049e-816621366128', '6.95373688089329058437175277372258618683e+7706682906', '6.9537368808932905843717527737225861868299999999999999999999999999999999999999999e+7706682906', 80, 1);
    T('-4.62654605051868978995380264326717263752562e+717176', '7.80592921290025989265686511631996947512021e-26972599', '-4.62654605051868978995380264326717e+717176', 33, 1);
    T('8.3802872802014182004283142877e+53012822579', '6.8e-438215988051056', '8.3802872802014182004283142877e+53012822579', 32, 6);
    T('9.98167e-264604425', '-4.827868245038601159112813848346386390777608350267987591284e+7321225309751', '-4.827868245038601159112813848346386390777608350267987591284e+7321225309751', 89, 6);
    T('1.2374651838891092835638715345e+44543', '-6.40e+870', '1.2374651838891092835638715345e+44543', 40, 4);
    T('-7.0e-120807', '2.6110449512888e-3810', '2.6110449512888e-3810', 25, 5);
    T('7.0276e-3243329228', '9.8056532200710e+717058', '9.805653220071e+717058', 21, 4);
    T('-2.2045119273818901916719705449727607032825872e+86605951', '-1.2799047443068878145e-7', '-2.2045119273818901916719705449727607032825872e+86605951', 56, 2);
    T('3.9920027140118127543936219392e-350361009776075', '-4.40645638486904111758018961189825e-88316', '-4.40645638486904111758018961189825e-88316', 63, 6);
    T('-5.7395694842348400368932e+442899528360', '5.45e-637426', '-5.73956948423484003689319999999999999999999999999999999999999999999999999999999999999999999999999e+442899528360', 96, 1);
    T('-2.64953481465013093865018103131612443711966900553245319325384e+6564223412961', '6e+1366045402', '-2.6495348146501309386501810313161244371196690055324e+6564223412961', 50, 2);
    T('6.970376264465089101314e-92327328713', '-2.90711650856523507043673880373657764542e-935861688333714', '6.970376264465e-92327328713', 13, 3);
    T('-7.94344106396880048543756581e+110', '-1.65362432502159042428792313436304187343633e-3028005', '-7.94344106396880048543756581e+110', 39, 5);
    T('-2.1411026043303e+77', '1.95451847492562734e-69650809227818', '-2.1411026043303e+77', 22, 4);
    T('-6.3492384510339464359481162274002508889691926194520440194e-4845860062', '-7.907716420294569461761742482708857919337648023344751944e-172112448', '-7.907716e-172112448', 7, 5);
    T('3.6568830960199502871886736963436312313843665e+64941075590', '-3.2217331917794414934035019423551786895211624926663194e+511', '3.65688309601995028718867369e+64941075590', 27, 1);
    T('-1.2e-1454625876544', '4.4951811536481795267991505652e+37', '4.4951811536481795267991505652e+37', 54, 0);
    T('-6.90846701e+966405038855', '6.369552339565424087933e-48743', '-6.9084670099999999e+966405038855', 17, 2);
    T('-1.398440076246077571022151683154620480223985501549570e-7323479775560', '5.4418077419870e-660892', '5.4418077419869999999999999999999999999999999999e-660892', 47, 1);
    T('3.809653476196015431767414417588536e+132050', '-4.732532013328832252271789403911e+31', '3.8096534e+132050', 8, 1);
    T('9.9273348628381210484767278054537146587e-9353705020561', '5.221133843e+282104043', '5.221133843e+282104043', 72, 4);
    T('6.49715293319607200291744596949e+3125', '2.20527887867512938771e-387212142120', '6.49715293319607200291744596949e+3125', 59, 3);
    T('-6.00495963305620771832005631885185e-6967772', '9.628968103841582856256958108122e+88275632609', '9.628968103841582856256958108121999999999999999999999999999999999999999999999999999999999999999999e+88275632609', 97, 1);
    T('2.27005e+72128650936589', '4.1e-7406879065', '2.270050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+72128650936589', 97, 0);
    T('1.2437150699546834e+16891509423', '7.161893529732439744080774947018516772760882422729e+95582032624431', '7.161893529732439744080774947018516772760882422729e+95582032624431', 65, 4);
    T('-0.1837357', '-4.511726659914700687449639381629238817e-280666655492', '-1.837357e-1', 55, 6);
    T('-4.2618054929048594315207822608279e-9679044178', '1.86202e-409648', '1.8620199999999999999999999999999999999999999999999999999e-409648', 56, 3);
    T('9.01927162404510993003599917173863838286168738e+58', '-6.102448460166587108e+56000406772', '-6.10244846016658711e+56000406772', 18, 0);
    T('-9.1373738236485379783844733014247796559893690115624436173695e-153', '-6.78486570116192809504485257337797837230252465193193209e-231', '-9.1373738236485379783844733014247796559893690115624436173695000000000000000000067848657e-153', 87, 6);
    T('1.68358074926406744186135867556545541e-60373503735', '-3.75116825e+3327182', '-3.751168249999999999999999999e+3327182', 28, 1);
    T('3.99793062421495153544155565e+286106542', '2.4842409797591742331552278759284744615177817328071195e-30757411', '3.99793062421495153544155565000000000000000000000000001e+286106542', 54, 0);
    T('-1.0982246186853425957821601851024720870704272263666196e-97294153076542', '3.917149206653800075867304609816269e+8524267189', '3.9171492066538e+8524267189', 15, 1);
    T('7.8808979974054116064246404951650971880e+705378555827', '-2.34898174063203951294211201971126093191473694404429e-30606843880317', '7.88089799740541e+705378555827', 15, 4);
    T('-1.55829465962e-6054', '-2.184386060859906e+6336000712', '-2.184386060859906000000000000000000000000000000000000000000000000000001e+6336000712', 70, 3);
    T('-9e+9352925726167', '-3.2086313852351860e-64287', '-9.000000000000000000000000000000000000000000001e+9352925726167', 46, 0);
    T('7.90487943667393979367290287340e-881194866', '-8.71214802088e+948', '-9e+948', 1, 0);
    T('5.233399525170173441577643295300955189105e-1237241', '7.76791052e+70206398', '7.76791052e+70206398', 41, 5);
    T('-5.04891899972404132615504928400839640880073911280172e+570169246546543', '-9.16080224560411922891720359566744792931e+977363680608', '-5.04891899972404132615504928400839640880073911280172e+570169246546543', 66, 6);
    T('5.628731e+19556', '-1.2222108811577181200e+807736349', '-1.22221088115771812e+807736349', 93, 3);
    T('1.53778640787811e+270056701466', '6.617539698229966560153326535785175344222015503395312737e-12996457707', '1.53778640787811e+270056701466', 18, 4);
    T('-6.1103243262163261693559302446850390909333034747784450e+1928666595464', '-5.6355845546e-589347172503035', '-6.110324326216326169355930244685039090933303474778445e+1928666595464', 92, 6);
    T('-3.867959231025018042694890224484359171087892103759962159672e+97945', '6.26679531979432684612e+711445879403', '6.266795319794326846119999999999999999999e+711445879403', 40, 1);
    T('2.72868551118731740454327276237965e+607993', '-2.80547955864770206726329580127533867724151939071e+1437223509819', '-2.80547955864770206726329580127533867724151939071e+1437223509819', 57, 3);
    T('8.0e-720768536594', '4.8656240581786802776333044e+32331', '4.8656240581786802776333044e+32331', 67, 1);
    T('4.3132088977595736e+812', '-8.5356438110519737420566372198602137344256890831217433e+513802', '-8.53564381105197374205663721986021373442568908312e+513802', 48, 6);
    T('-6.94052455784572519530241508075507475439869566249663326932e+589348151', '-8.056904591994813720446437454695e-89812346672760', '-6.94052455784572519530241508075507475439869566249663326e+589348151', 54, 1);
    T('-4.26401846797012347568097751653394282561608e+39213660', '-7.17257e+88806', '-4.26401846797012347568097751653394282561608e+39213660', 59, 1);
    T('59676536961835.691778', '-6.8800671186043810826262443772260247793183461745e-226615', '5.9676536961835691778e+13', 69, 4);
    T('-2.411762073839499824225e+91780340896', '-5.22672950243609619443356e-946', '-2.4117620738395e+91780340896', 16, 6);
    T('1.7769598234330067939e-93140724', '-6.67441036531972436336420168949196e-21', '-6.6744103654e-21', 11, 0);
    T('9.69975820998e-2338569', '-1.440413335534472240154605932838321690225581382e+254372329457', '-1.440413335534472240154605932838321690225581382e+254372329457', 88, 0);
    T('-9.00627584e+9881227516', '7.989055347243139945121262905682286712958776249304926280e+5633140760373', '7.98905534724313994512126290568228671e+5633140760373', 36, 1);
    T('-9.9477298791959498071782624e+3783777415', '9.022517204654757025541736732730981219e-70056688576', '-9.94772987919594980717826e+3783777415', 24, 5);
    T('3.45429596314456210334905295076389718701359828898274e+2880', '-1.402110940350099843569766511590704893404332e+7147917489', '-1.402110940350099843569766512e+7147917489', 28, 5);
    T('-7.044934347523191015681779960554335841623e-111', '3.78019735205865190046610071552172893789e-270786878860164', '-7.04493434752319101568177996055433584162299999999999999999999999999999999999999999999999999999e-111', 93, 2);
    T('-1.358903486512818781025595688583938243469752515126834e+149559', '2.5417891871001804175993864319601644167319504291167845066e+9802', '-1.3589034865128187810256e+149559', 23, 0);
    T('-8.9231117299834517722e-762902', '2.7416e-824007924', '-8.92311172998345177219999999999999999999999999999999999999999999999e-762902', 66, 1);
    T('-9.999175553447696865533930492408668776755546665154632523521e+7421', '-6.21642311588932e-8204119988291', '-9.999175553e+7421', 10, 4);
    T('2.302598176908784914392117674025661e+679881374639704', '-2.294839686823927001570347952708868567558873063952658230e-499264205987', '2.302598176908784914392117674025661e+679881374639704', 91, 4);
    T('9.55338179993657262563897432592932655021882790e-943', '-2.153091867e-37584207', '9.55338179993657262563897432e-943', 27, 3);
    T('-5.992674414e-51903661', '8.0075e-931', '8.0075e-931', 52, 0);
    T('6.859854892973860607105938564e+310', '1.43793901863319267596887e+305153252', '1.4379390186331926759688700000000000000000000000000000000000000000000000001e+305153252', 74, 0);
    T('8.988360852974120019914615187599512e-743608492987', '-8.833440043207130659e-913582501', '-8.833440043207130659e-913582501', 91, 3);
    T('-3.5287103559378303532767255e+13563096', '5.3239919101624001772614838859735470186885415e-624', '-3.52871035593783035327672549999999999999999999999999999999999999999999999999999999e+13563096', 81, 2);
    T('-9.7081454540854e-65', '9.113278152353526449650371064099094746e-9918174409627', '-9.7081454540854e-65', 31, 4);
    T('-1.90711666103423228e-7590733953', '-3.243474325165904337186276492029e-528809177267', '-1.90711666103423228e-7590733953', 39, 4);
    T('-9.804799247411661755557633936590099432e+2407', '-2.775e+5919037200', '-2.77500000000000000000000000000000000001e+5919037200', 39, 3);
    T('-641210625493883248937392474139237762252137426597117895.475627', '-9.40675027242560264522322184611250770967527029408361672062195e+94', '-9.4067502724256026452232e+94', 23, 4);
    T('-3.038484853870938615079155516e+620247634664', '-9.46342222112408270942848747732203700510697958822e+76362', '-3.03848485387093861508e+620247634664', 21, 5);
    T('4.876458986235753250624562491473309289983868850e+65246148667', '-1.4224645987916356842308064470e-55232437296394', '4.87645898623576e+65246148667', 15, 0);
    T('-5.819167130133e+25658770357426', '-3.821454503655971729e-452807669', '-5.819167e+25658770357426', 7, 5);
    T('-5.3062685139451058601696016745118024657930832206759129847844e+25357834562', '-6.02961448e+154586', '-5.3062685139451058601696016745118024657930832206759129847844e+25357834562', 80, 4);
    T('-6.799e+98169', '-9.14977e+73719053', '-9.1e+73719053', 2, 4);
    T('4.10336196717498976867103272357331379e-2666656937', '-1.085576047886448501990045759066991804449912895061919e-539102', '-1.085576047886448501990045759066991804449912895061918999999e-539102', 58, 1);
    T('-9800208210813356268.5764746', '-5.812397380882403605466034801647314755e+9694945998432', '-5.812397380882403605466034801647314755000000000000000000000000000000000000000000001e+9694945998432', 82, 0);
    T('-9.9381016e-967885548247174', '-2.5536048151897938746671846947170e+110', '-2.5536048151897938746671847e+110', 26, 0);
    T('2.151264e-41600970820', '1.982e-96933', '1.982e-96933', 52, 6);
    T('8.60516721e+419991218', '-5.7855779714475306640518450526657716226354347643686768679638e-658402774761', '8.60516721e+419991218', 23, 4);
    T('-1.7173649048467520107532e-18414873722', '-4.865101615502592160617903867e+827780011450', '-4.865101615502592160617903867e+827780011450', 53, 2);
    T('7.4823650991476128035586501540474625201675669027e+4769', '-9.8608810855449508314e-762', '7.48236509914761280355865015404746252016756690269999999999999999999999999999999999999999999e+4769', 90, 1);
    T('-7.80023393179234045376305e+88', '3.255e+12852688861605', '3.255e+12852688861605', 60, 0);
    T('6.9070164584078220852828181e+8975', '11.228', '6.9070164584078220852828181e+8975', 55, 5);
    T('1.124191712667948411229413661314139639217462001988e-2948602540', '-5.8565305407476125833644393449705508833720998029025535883e-2468', '-5.8565305407476125833644393449705508833720998029025535883e-2468', 76, 6);
    T('1840073.39505620639191295644262668248260354476426168297352394', '-5.1245638950e-3778510673', '1.84007339505620639191295644262668248260354476426168297352394e+6', 82, 2);
    T('-2.768437834700569134290137149e-569878', '-5e+58', '-5e+58', 21, 2);
    T('5.1493799206299384726302223511135131925066768589926728611e+628140191', '5.7526012428165376654021785421925933178138668947e+53', '5.1493799206299384726302223511135131926e+628140191', 38, 2);
    T('-6.64573528478435289379659210742521680892266799926e+4809048293', '8.278941259242272637144205754e+117', '-6.6457e+4809048293', 5, 5);
    T('-6.505473648e+94906747842', '8.13e+287661235', '-6.505473648e+94906747842', 47, 4);
    T('8.6116494292611790768472136516049993e-6502296753794', '8.60573468023539065945665274430610214e-9712', '8.605734680235390659456652744306102140000000000000000000000000000000000000000000001e-9712', 82, 2);
    T('-8.33924306775014e+89', '-8.29702333629203461979303384373001907841065712772471183714e-46018588688534', '-8.33924306775014000000000000000000000000000000000000000000001e+89', 60, 3);
    T('5.1061829e+543109879', '-5.755e-168546336', '5.10618289999999e+543109879', 15, 3);
    T('991163461677330781984122209.57', '7.89845443224380888132086741329833898671678559e-32261490', '9.9116346167733078198412220957e+26', 93, 4);
    T('-4.89592e+584464257', '6.628672890191837908497643e+53779', '-4.895919999999999999999999999999999999999999999999999999999999999999999999999999e+584464257', 79, 1);
    T('7.4565602327729988983113681711e+6614131', '-6.6871027803487575772e-6370326', '7.4565602327729988983113681711e+6614131', 59, 6);
    T('8.12e+97487395', '-3.903711128610385114355912745531236126e-3422271682253', '8.12e+97487395', 64, 2);
    T('2.78147833115215907979614e+454167256891300', '7.545272672e+194188917668', '2.7814783311521590797961400000000000000000000000000000000000000000000000000000000000001e+454167256891300', 86, 2);
    T('-1.44394899128564365490613287542247267e-75156', '2.408795898659945257752034107340392000223731251260768917060e-41985', '2.40879589865994e-41985', 15, 3);
    T('-6.2348046573853243592684861897133153e+874771', '6.714604976747285974856e+722', '-6.2348046573853243592684861897133153e+874771', 57, 6);
    T('-5.0453253797588749e-3223', '8.1767913490e-946468', '-5.04532537975887489999999999999999999999999999999999999999999999999999999999999999999e-3223', 84, 1);
    T('-8.88660151583281746985757182625e-1996073', '-6.47928720467601e-293445038500880', '-8.88660151583281746985757182625e-1996073', 91, 5);
    T('6.3857664595e-5160785', '4.4337201942495648513445233843099193510323022269206e-32891', '4.43e-32891', 3, 4);
    T('-6.6000635290561047257473521754577370290461e+243', '-8.33689534026163540e-81683258220', '-6.6000635290561047257473521754577370290461e+243', 64, 1);
    T('-9.1485479651166181e-367305265650', '-1.2994123652047837e+3687161', '-1.2e+3687161', 2, 1);
    T('2.7302992128369e+755469208274006', '-4.183912742675516e+802170413', '2.7302992128369e+755469208274006', 31, 5);
    T('-4.523036212454e-836890', '8.732554510e-3990', '8.73255451e-3990', 13, 2);
    T('-7.0712253996668721755780634248100e+7673', '-4.424301483591217752646898354849685e-83394831', '-7.071225399666872175578e+7673', 22, 5);
    T('-6.532371013934e-40154688', '-8.0856016712820669880898861226195722224588381e+99369', '-8.0856016712820669880898861226195722224588381e+99369', 83, 5);
    T('9.6621342197930393558671865850345112189544552372701694920e-964249113628', '9.3201431133922602890e-2327664', '9.32014311339226029e-2327664', 18, 5);
    T('-8.93645523875741523091e-79099679', '9.1955246135349492301083e-92781642695538', '-8.936455238757415230909999999999999999999999999999999e-79099679', 52, 1);
    T('4.7134631805871280740769694203303517781834152118778485e-23757746999414', '4.778091e+7758652', '4.778091e+7758652', 31, 5);
    T('-1.761853989738234453787461366778e-443', '-6.47756345836355e-9817225563', '-1.76185398973824e-443', 15, 3);
    T('1.299842696157627725454417608164959825313280e-957395042530', '1.98578780167380952936578384076621811665e-6264563', '1.98578780167380952936578384076621811665000000000000000000001e-6264563', 60, 2);
    T('-6.82585e+795335061789', '3.2604136122e-36274802940191', '-6.82585e+795335061789', 96, 5);
    T('-2.89226865425665313452489491479753993944340262e-563717727763942', '-4.93460116858479256686446785713415229954497329447810866431271e-382205312599491', '-4.9346011685847925668644678571341523e-382205312599491', 37, 4);
    T('1.148352832964577469e-4917041', '-7.506281870767966472875e+55625978770261', '-7.5062818707679664e+55625978770261', 17, 1);
    T('-2.668856921139079260471379437e-6903123', '-4.514403421582703887955649014912334080521086294525e+982222219526', '-4.51440342158270388795564901491233408052108629452500000001e+982222219526', 57, 0);
    T('9.69002201537219622683974e+85396290077', '1.6421475233235914681e-20', '9.69002201537219622683974e+85396290077', 92, 5);
    T('-6.4659987479617615168123060452464e+77552728', '-8.84318902749312462450484792091774279964733411091e+876209614', '-8.84318902749312462450484792091774279964e+876209614', 39, 1);
    T('-6.39688800627086363188619688081918946226161002e-629903', '6.33075776872909612646622840824922024914679683323736874e+479378183630669', '6.3307577687290961264662284082492e+479378183630669', 32, 1);
    T('-9.419e+694564138', '-4.16118004156572814443758613427612602398623861689750343709e-389063', '-9.419e+694564138', 88, 5);
    T('5.5858541234734224371e+6829798629043', '-1.4572293176412384282586e-96891', '5.5858541234734224371e+6829798629043', 54, 6);
    T('-5.561973e+14', '8.12690522588215150763309296e+31', '8.12690522588215145201336e+31', 24, 5);
    T('1.0657134345487664529082494031923e-519', '-9.26156717094884566e-17614', '1.0657134345487664529082494031923e-519', 42, 5);
    T('9.19933406422541998560769301186206580785799694245140258401e+421568', '-8.01964258398116160181382505338402220676261203e+28887097', '-8.019642583981e+28887097', 13, 6);
    T('4.4679885688261875326594182848037274613849839525e-343959487', '-3.3e-61073348298007', '4.467988568826187532659418284803727461384983952499999999999999999999999999999999999999999999e-343959487', 91, 3);
    T('8.877538649235421699192671721184766541289633314166952492637e-58631831301', '2.15219046362821e+50642048', '2.152190463628210000000000000000000000000001e+50642048', 43, 2);
    T('-8.99212521721644462056538417338100e-83242', '-4.84e+221877017', '-4.84e+221877017', 85, 2);
    T('-8.167969635516895420003794576278770900107446178031260091e-316', '-8.9994199574095709769837913260519173161044653721122867e-573873566625081', '-8.167969635516895420003794576278770900107446178031260091e-316', 98, 2);
    T('-6.911366497737852034375266343836942752e-31234174', '-4e+3286095', '-4e+3286095', 33, 1);
    T('-6.0209405984064481258724964454247839162552976139634e-94690119270027', '3.768191003229841212299366e-877158879702', '3.768191003229841212299366e-877158879702', 36, 6);
    T('-1.0e+737639556728879', '-3.68829283302678160072926104303345820394906e+1308971486068', '-1e+737639556728879', 65, 5);
    T('-3.179289e+30189088871', '2.2555e+76314928909010', '2.2555e+76314928909010', 32, 2);
    T('2.209284283297100394152381e-1843', '-6.7e+44', '-6.699999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+44', 88, 1);
    T('9.73841782875317738002793142e+35', '8.7208678753622396484e-3303233289', '9.73841783e+35', 9, 2);
    T('-4.86850014118491873492268421e-564410824916', '-9.95528825473215190635175995554943869955624e-138', '-9.9552882547321519063517599555e-138', 29, 6);
    T('-4.970449907550185049285920135440602438861849324045e-35', '5.9742154533680715048488084121100443677887674622484208997921e-86001619874808', '-5e-35', 1, 3);
    T('-8.3415697783856562060486584942358e+799152730880', '-4.779523904101872683331530212216e-951771135082', '-8.3415698e+799152730880', 8, 6);
    T('-7.81659656426655807835398962363e-284354679317467', '-5.1199e-30', '-5.11990000000000000000000000000000000000000000000000000000000000000000000000000000001e-30', 84, 0);
    T('-9.823697794560366574e+23010281540205', '6.43276915653272582579725934279738934510848595e+252048110974', '-1e+23010281540206', 1, 3);
    T('4.65842820632e-6745739', '0.00045', '4.5e-4', 42, 4);
    T('5.7007758383726012049541088359e+80276464406', '1.3825707237108e+9380', '5.7007758383726012049541088359e+80276464406', 91, 3);
    T('7.921e+6348', '-1.15197755e+8005280', '-1.15197755e+8005280', 47, 4);
    T('-4.625129167781351990360677512158139713078406809e-25896958951795', '8.5284756e-3883697389', '8.5284756e-3883697389', 8, 6);
    T('-4710731871923846.05980451', '-1.28017711370589194096026569213e-50696', '-4.71073187192384605980451e+15', 72, 6);
    T('9.2151e+2038', '6.6656967914284254866265618590848018e-34', '9.2151e+2038', 46, 5);
    T('-5.2732322924246e-71', '1.04305995528672984589991977378649179e-834905473', '-5.2732322924246e-71', 86, 4);
    T('2.653264086925997363362812e-312', '-7.434835e+7', '-7.434835e+7', 89, 5);
    T('2.069246829116915134983313503843727225100100791e-62691638300', '3.40682257768976863621182026327763616e+813486733', '3.40682257768976863621182026327763616e+813486733', 99, 6);
    T('-9.2145267341947121493e+88748751570', '9.0278609239504996899097023469336070109495164e+95258', '-9.2145267341947121493e+88748751570', 54, 6);
    T('2.342896923818008172908195459e+5767896984', '7.9069922249e-820328628616', '2.3428e+5767896984', 5, 3);
    T('4.594350384462352998579616943e-434', '2.6416893362825118827444599178355738777e+958692309260', '2.64168933628251188274445991783557387770001e+958692309260', 42, 0);
    T('1.281012169954312903879029049e-74947938560721', '-1.392521873389370029504039e-777', '-1.392521873389370029504039e-777', 77, 5);
    T('-4.2776865637913206268281095330550992852058932923400e-445', '3.6940630413837453325822893365096876225212e-7883', '-4.28e-445', 3, 5);
    T('-4.443041961560028898081400584e+270400703113', '-8.4321426131460061791518158186084515648181628994e+157837695', '-4.4430419615600288980814005840000000000000000000000001e+270400703113', 53, 0);
    T('8.779339584023109105465294053991074960475503e+442063571859819', '-8.2471651945026097249663230064599e+13797', '8.779339584e+442063571859819', 10, 1);
    T('8.96651579277124161080393693e+339561', '-999955630507103225070.23', '8.9665157927712e+339561', 14, 3);
    T('-8.4027112498946311131636587575451e+552436', '9.7239703413578177419928498249864615872e+348440525', '9.7239703413578177419928498249864615872e+348440525', 61, 5);
    T('7.386274881435995409796e-739364', '3.5e+73706353', '3.5e+73706353', 41, 1);
    T('-9.22355563875875037130640228475526e-455439770', '-2.55110192e+86089113098759', '-2.55110192e+86089113098759', 93, 6);
    T('8.853535121913324305965858809421124223577e-7789016656714', '9.9606733790808761e-373', '9.9606733791e-373', 11, 2);
    T('7.811579071e+360689491', '-8439176843272771339029260380349378610997895921.415', '7.811579071e+360689491', 81, 4);
    T('7.94482694817903538661e+51', '-9.4418406942779e+210206236050662', '-9.44184069427789999999999999999999999e+210206236050662', 36, 2);
    T('-6704330262981868567288074398837661776210090292860270477.3', '-4.409019247867588165870883231582152082432e+5842682883160', '-4.409019247e+5842682883160', 10, 1);
    T('-6.0139348842647919365342791e+480', '-6.96880759209745197092164746629398e+22387281505', '-6.96880759209745197092164746629398e+22387281505', 83, 1);
    T('3.2154667583173079283869912451e-2966468084220', '-2e+800899791552', '-2e+800899791552', 21, 6);
    T('7.519095339417574261662722522560877899577463e-47955882', '-6.06035485048521211985208800e-1774482', '-6.060354850485212119852087999999999999999999999999999999999999999999999999999999e-1774482', 79, 2);
    T('6.78772268869251948282968642268749e+751260627570', '5.418348415259509138836625311172098969e-969868682525', '6.787722688692519482829686423e+751260627570', 28, 0);
    T('-8.914562561905116908141528961177461e-255601026', '3.479292196645715051890725572232730940360233893953e-6407', '3.4792921966e-6407', 11, 4);
    T('-7.3766322489148055722e-701693572', '8.9041742678731058337863e+25614643', '8.9041742678731058337863e+25614643', 91, 5);
    T('9.406344550839876163093700018045122e-5599', '7.89006453224947e+21', '7.89006453224947e+21', 92, 6);
    T('-9.000054482634937606472774572240263491448646530563935e-1578412526', '-3.40e+597807932949', '-3.40000000000000000000000000000000000000000000001e+597807932949', 48, 3);
    T('3.7886527024388671390063028036e-69906579475076', '1.72253137163611715101919082230e-6769667365317', '1.7225313716361171510191908223e-6769667365317', 39, 3);
    T('-7.317948e-428846586', '-4.82727003547236388255236541499343912758173862456698757334e-14415', '-4.827e-14415', 4, 6);
    T('-9.8569981051803245482098823937956e+354426910445', '2.499432313463e-21623991', '-9.8569981051803245482098823937956e+354426910445', 99, 6);
    T('3.656382728e-4262548549252', '-4.284733412013619563056461678276e+736533', '-4.284733412013619563056461678275999999999999999999999999999999999e+736533', 64, 1);
    T('8.47498309627176087916144803235228430218006716837675120e-28640', '5.1576046007343525996768811701e-1306867128', '8.4749830962717608791614480323522843021800671683767512e-28640', 92, 1);
    T('-7.492459299301499840498530953576009e+164811254817256', '6.71208344867654028623535714957242834502e-991592', '-7.49245929930149984049853095e+164811254817256', 27, 1);
    T('-1.2371968446e+17455165451853', '-3.085439559e-56544947451655', '-1.2371968446e+17455165451853', 19, 5);
    T('-5.304819594e-255634208345398', '-6.930814298210e+17845374', '-6.9308143e+17845374', 9, 5);
    T('3.76611368e-4422895216838', '-1.68159724549e-42547', '-1.6815972454899999999999999999999999999999999999999999999999999999999999999999999e-42547', 80, 2);
    T('-2.0636817258439186185349243663714764284656e+311895', '4.559815657269952664285029759869050498954834893211e-553797271798064', '-2.0636817258439186185349243663714764284656e+311895', 97, 0);
    T('4.707951125284868002251914997855904957079078566791202e+92108566279', '7.08831449666128903214206610041472683894156763327480038345178e+92', '4.707951125284868002251914997855905e+92108566279', 34, 2);
    T('2.00087657186740253407877e+6733356571', '-7.18631471286810587128372571158415799583111172293621239e-45313773', '2.00087657186740253407877e+6733356571', 94, 0);
    T('7.47325633534260582e+549471439870119', '-8.78066934081650672137347835669401021e-16145211', '7.4732563353426058199999999999999999999e+549471439870119', 38, 3);
    T('-2.233987834463510694026574092534585508885416890441051e+17819514', '3.77376985140897575660752591457883235073631041953664e+9415126731', '3.7738e+9415126731', 5, 2);
    T('-4.042775464956707150012637478702799730655940e+129', '-5.835385564871162788801445e+939858076241', '-5.835385564871162788801445e+939858076241', 51, 1);
    T('27274279849802.30', '6.160947561833304913434885771091e+10786458598', '6.160947561833304913434885771091e+10786458598', 55, 3);
    T('-7.14198251862309e-234091390096999', '3.71668670889315338566913341608810620432454549988818805087e-5706881', '3.7166867088931533856691334160881062043245454998881880508699999999999999999999999999999e-5706881', 86, 1);
    T('-8.4902287759228227e+212973', '4.085569369897565904999061515455246258233316215598981526345e-85406', '-8.4902287759228227e+212973', 86, 4);
    T('-8e+897869411595685', '-8.506549e-71872102638', '-8.000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+897869411595685', 91, 0);
    T('-4.0571118319173412703067143672717232871977e-4914', '4.996174061114222884e+6469510424', '4.996174061114222883999999999999999999999999999999e+6469510424', 49, 3);
    T('9.7264361612335192227615574189071105002950215060998e+3192295', '-8.289199710432484592843341180e+5578', '9.7264361612335192227615574189071105002950215060997999999999999999999999999999999999999e+3192295', 86, 1);
    T('-502.34', '-3.6525652879710282082596337803533146692933699483966e-832138357', '-5.02340000000000000000000000000000000000000000000000000000000000000000001e+2', 72, 0);
    T('-9.814682252101869244e-7204', '9.18154453895992107035442211404199642592100842e+2136', '9.181544538959921e+2136', 16, 4);
    T('9.603782605811305379164364074329633034958885e+5242786162', '-7.62421716e-416549', '9.603782605811305379164364074329633034958885e+5242786162', 84, 0);
    T('7.12772049306755686267977493842623088496044210e+845321397841', '-4.369945117072785138354479250898455412612040564654008e-734819004541', '7.1277204930675568626797749384262308849604421e+845321397841', 45, 6);
    T('2.605868296525547083551317833514723962059786027386e-13', '7.403351833239889434800311314014208189338705790e-319600278060610', '2.605868296525547083551317833514723962059786027386e-13', 87, 4);
    T('-7.8259498906950287321870386525799880899613236301154053380786e-2081', '-2.51041396488282658410226895518014520142e+14438530', '-2.51041396488282658410226895518014520142e+14438530', 97, 4);
    T('4.7639567317163325960e-37730841693', '7.94985835269653615083386734038962641103678360263274983576e+2162565350', '7.94985835269653615083386734038962641103678360263274983576e+2162565350', 80, 3);
    T('5.9344117479382200744938905820714e+3818658425', '1.9795862775899521167371803814150443336635817579509466e-179035337099', '5.9344117479382200744938905820714e+3818658425', 55, 5);
    T('-2.5536407066251618214684974351058053884325550951847906810e+466263756009', '6.2997389314597108507145151788707458367884023526293138e+70', '-2.553640706625161821468497435105805388432555095184790681e+466263756009', 87, 5);
    T('-4.5473955488512232684007286398980755557842989897e+93196216', '7.36913271404704929778e-929', '-4.5473955488512232684007286398980755557842989897e+93196216', 74, 3);
    T('-8.60128656607418512659717802175270e-269853931325730', '-1.3680164000476494e-43640', '-1.3680164000476494000000000000000000000000000000000000000000000000001e-43640', 68, 0);
    T('5.868448598089640445999388875e+5400567', '4.1423778137092892934230702022442770685841532250178814186e-346930005009', '5.86844859808964044599938887500000000000000000000000000000000000000000000000000001e+5400567', 81, 0);
    T('9.926281728505220730492339794846107042698576010e+65', '5.41485791594541688388356651e-43767', '9.92628172850522073049233979484610704269857601e+65', 94, 1);
    T('5.4684605205613273e+941', '5.2e+309', '5.4684605205613273e+941', 96, 6);
    T('-4.012403106496639412190428646643944411528137e-1718258263323', '-1318629534639756199.96925', '-1.31862953463975619996925e+18', 98, 6);
    T('-3.6101272833861565277438235007e-74880566117779', '-6.38490468e-479924143792434', '-4e-74880566117779', 1, 0);
    T('-2.2533448425850969635343435040280e+831122340930', '1.914594900687658211129221773883e-8364803169', '-2.253344842585096963534343504028e+831122340930', 65, 3);
    T('5.23137334659e+571738', '4.40871505518977742522298224268693611323546800629e-79526676684680', '5.23137334659e+571738', 91, 1);
    T('6.608231039611294e+93445', '-7.654909155421762131944295710186433853119947702e-7744', '6.608231039611294e+93445', 48, 4);
    T('6.03693623758382674353565107690e-8348081', '5.02455135689318665782030301950129815191355903592007077e-11937549', '6.0369362375838267435356510769e-8348081', 36, 3);
    T('6e-61', '7.5893683409135632952540495673248906560524772724350e-666891', '6e-61', 90, 3);
    T('-5.01702155398148010e-66284060629', '-6.5113632273997457295402026369793e+93', '-6.5113632273997457295402026369793000000000000000000000000000001e+93', 62, 3);
    T('6.77665627503213e-4457481782413', '7.04785688164995247290258378940653135182222937724445196987e-57940071', '7.047856881649952472902583789406532e-57940071', 34, 2);
    T('9.30737311980486453596e-22796584', '8.71274048976941704807194932669990973902257147414492176764e-45537265901', '9.30737311980486453596e-22796584', 97, 4);
    T('6.19136016571020540461999350480800711221e-37380', '-978169042899670960253012.3029521284876751109186475870259', '-9.7816904289967096025301230295212848767512e+23', 41, 0);
    T('-6.677971954868946741352763386037284756844318400014e-67746', '5.342166748948e-47951977599', '-6.677971954868946741352763386037284756844318400014e-67746', 70, 3);
    T('-1.08804053140182823279028437078790578304908249454846362485e-2471518805', '-1.84716051869815587471102373145467974831e-9526892', '-1.84716051869815587471102373145467974831000000000000000000000000000000000000000000000000000000001e-9526892', 96, 0);
    T('-2.013879092395362586943210213074e-9819920076', '9.54648322618378980099301965378283352863e+50862', '9.54648322618378980099301965378283352863e+50862', 94, 2);
    T('25412805956555925926907255545488929502557922696.9765598', '-1.6701491396604e+1118094421849', '-1.6701491396604e+1118094421849', 57, 3);
    T('2.641667413699740883162097914430002190171332914e-360912925356', '-2.760802038737506074640341001766042250154148e-521', '-2.760802038737506074640341001766042250154148e-521', 65, 4);
    T('-9.42971587895933466465931639998054823511e+83696758259081', '4.0425934554946394366301797168133241177e+1474517842560', '-9.42971587895933466465931639998054823510999999999999999999999999999999999999999999999e+83696758259081', 84, 2);
    T('-6.41949252314380017109697587057056403877590597037453624694228e-3224067060', '4.2201703125693058e-19044336291023', '-6.41949252e-3224067060', 9, 1);
    T('-6.0762730298127554788738804617963017178e-211243561', '-8.2666235289867466701051220961e+735145', '-8.2666235289867466701051220961e+735145', 43, 4);
    T('-3.6281920418644862974018658420772076195360476304958278e+7500', '-8.884e-5569704218', '-3.6281920418644862974018658420772076195e+7500', 38, 4);
    T('7.1137737917197697638993694001983635670e-2022901712', '-6.64424685416369759823349650433e-600127520210411', '7.113773791719769763899369400198363567e-2022901712', 47, 5);
    T('-7.4310556902669130947049700570e+54037', '-9.57716110307699732684185889553715840386e-96458114683', '-7.43105569026691309470497005700000000000000000000000000000000000000000000000000000000000000000001e+54037', 96, 0);
    T('-6.69864509998470854133321197397926365184868165834607482e-2805538613', '-6e+25295420', '-6.0000000000000000000000000000000000000000000000000000001e+25295420', 56, 3);
    T('-9.5735013395314e-633', '-3.5228060652847871869876e+10280475854921', '-3.522806065284787187e+10280475854921', 20, 4);
    T('-3.83634931158255663403516049357520996594694011e+29687690', '8.2161864182869096009438959486617290366272142199879411e-351', '-3.83634931158255663403516049357520996594694011e+29687690', 77, 6);
    T('-8.3277070449818168154185524821023800234486925097462689510167e+5744717854', '-3.8720355973e+247', '-8.3277070449818168154185524821023800234487e+5744717854', 41, 5);
    T('-3.76720810130789694403678e-1437', '7.3959802776338170405665837035e-9839067592', '-3.76720810130789694403678e-1437', 99, 6);
    T('1.226842449520459200831283687e-55491', '-7228058.99391212764827875650584867994583120049413646379127', '-7.2280589939e+6', 11, 1);
    T('5.184614621305208061216417738899466e+85221349382584', '5.667644480633633517e+11936104778342', '5.1846146213052e+85221349382584', 15, 3);
    T('1.96472852109766354789e-26204962', '-8.82048077642917007786564665164457346e-2546498', '-8.82048077642917007786564665164457346e-2546498', 91, 0);
    T('-8.0457540907582872154918505715888116439832399212909775384119e+438009', '-2.85791479603574151763273765158813329837126398385272e+602411300409371', '-2.85791479603574151763273765158813e+602411300409371', 33, 6);
    T('8.80321039217630962308910182893754192347011863419e-18902764', '-9.455629846870e-913929', '-9.45562984687e-913929', 66, 0);
    T('-3.7454e-2104123173320', '-3.3703071125129860351387069116230752066617401854213537184e-8270814069611', '-3.7454e-2104123173320', 71, 6);
    T('-8.06194033338389643983286176647417917992914576336578679526114e+63538170914171', '4.6121774848064e-575664172521', '-8.06194033338389643983286176647417917992914576336578679526114e+63538170914171', 72, 5);
    T('6.164694873531349867710090236418397200e-5972165965', '7.9429393497450975e+15095', '7.942939349745097500000000000000000000001e+15095', 40, 2);
    T('-3.688339185676578369034610615039564e-677472', '5.15346611951437366271980805715957643e-99087900', '-3.688339185676578369034610615039564e-677472', 72, 0);
    T('3.0e-9142384659713', '5.36279028772450169638294780328394e+5049069046', '5.3628e+5049069046', 6, 0);
    T('-7.0275104595100477e+34715933', '-1.1336474415e-9902', '-7.0275104595100477e+34715933', 81, 2);
    T('-20607202.9880', '-8.4438090668547911436927720379919599972649553e-411759951', '-2.06073e+7', 6, 3);
    T('-2.753510556428389271899386315017278370493910275945e+53', '-4.83239472294797311e-9427767242163', '-2.753510556428389271899386315017278370493910275e+53', 46, 2);
    T('8168771763248680263545656944285862683.2261596', '-9.975782582430031823034034062187512162263234620e-43547395623', '8.16877176324868026354565694428586268322615959999999999999999999999e+36', 66, 3);
    T('2.48523311271e+690723785792', '1.02963820802582184663041437490463971850686811480295781e+299301814680109', '1.0296382080258218466304143749046397185068681148029578100000000000000000000000000000000000000000001e+299301814680109', 98, 0);
    T('-5.4502645044664855254e+79902', '6.65069357289725e-498', '-5.4502645044664855254e+79902', 39, 5);
    T('6.41336385503361942e+7654262', '-5.7005016985371811495195260674481886235775973153157494428e-34530', '6.41336385503361942e+7654262', 35, 0);
    T('-3.486318388784435823669477405972529e-314090339722', '-3e-9782', '-3e-9782', 22, 1);
    T('1.8445593716398695645916542345446853943424241209704945732e-72499966', '-2.2947625265326e+1602577781164', '-2.29476252653259999999999999999999999999999999999999999e+1602577781164', 54, 2);
    T('-8.819978006938707572691508293468084438804e+725835423', '6.204577082527899399380368462325235350372002728429177608e-677128202826', '-8.819978006938707572691508293468084438804e+725835423', 83, 3);
    T('-6.5880277442610719027444e+637197969', '9.1811942797748381953624446090573499686633736632841119e-74499321510', '-6.5880277442610719027444e+637197969', 25, 0);
    T('3.353099974527128092562550626901377459413e-40', '3.187090442031172274248784760506e+289729128036', '3.187090442031172274248784760506000000000000000000000000000000000000000000001e+289729128036', 76, 0);
    T('-2.408949247044115151562832804147e+75969025201', '-4.9920153615438379e+80335630060', '-4.9920153615438379e+80335630060', 44, 5);
    T('-5.68333303e+915576553685173', '-3.3215827433968454159140747922271534946802839666949453e+68436', '-5.68333303e+915576553685173', 61, 1);
    T('-7.2214401476354179058123e+5645343', '7.5107050020623635302904302026981479275138664414e+83755864', '7.510705002e+83755864', 10, 1);
    T('9.053865769681380691214310024333740936985403306688e+822689983', '-8.6166537783939000711859e-4338', '9.05386576968138069121431002434e+822689983', 30, 0);
    T('9.6629429484978188424826757335376682942630154e+8716', '3.4586420172644856354e-542732514463', '1e+8717', 1, 0);
    T('4.847381030e-147009007213615', '6.74517162409611950486e-589644360795', '6.74517162409611950486000000000000000000000000000000000000000000000000000001e-589644360795', 75, 2);
    T('-5.42826886608437260552428626092064430719739588273063774268240e-587697124170152', '5.832674638922330366476037020560524857454293381009277772e+92740970164', '5.83267463892233037e+92740970164', 18, 6);
    T('1.56334875812519068134622601405456465295941145380976032725056e-29450573', '-2.39782258200882751831562565306270106663665279546992e+493057', '-2.39782258200882751831562565306270106663e+493057', 39, 2);
    T('-3131849864365.26754284986010857273314091902640452989613', '-6.599355198420218e+9536', '-6.599355198420218e+9536', 67, 6);
    T('-4.900564185e-3922476', '67.50008002898019283774312', '6.750008002898019283774312e+1', 48, 5);
    T('4.19579880876719499084037777884424339008e+224852892962', '-7.0398205854884885115310326612532024652773491529197245362e+207467532053709', '-7.0398205854884885115310326612532024652773491529197245362e+207467532053709', 76, 3);
    T('-1.312010517438883441959078171415178805e+56213', '-6.88636976928479375564940480e+797965971773', '-6.886369769284794e+797965971773', 16, 4);
    T('6.80178610827820e+59', '-8.4290482040276447380762941992074639630317264120267088478121e-28386', '6.8017861082782e+59', 88, 0);
    T('2.453e-9611543', '3.5320582168e+946720489', '3.5320582168e+946720489', 79, 1);
    T('1.8838586082e-25', '6.05031822746498849718e-619032167860505', '1.8838586082e-25', 80, 1);
    T('1.306671e+1885', '7.9326341121488604762666515013673937012092577602e-6883', '1.3067e+1885', 5, 5);
    T('-5.92464131224222376302985e+8357916914', '6.77032686847255564086524297839e+37', '-5.92464131224222376302985e+8357916914', 97, 6);
    T('-1.9080896207306627895592556946024408082633e-955232', '4.8130581917930524931470e-3107', '4.813058191793052493147e-3107', 88, 6);
    T('2.4937969373651888713651534675153266508e+508748', '-5.76e-840493', '2.494e+508748', 4, 2);
    T('41769265267.43124256217128145917172359', '-2.1e-68', '4.176926526743124256217128145917172359e+10', 66, 5);
    T('9.00333291441555574597039398563988065536604e-689120064201300', '4.64103792e+12', '4.64103792e+12', 76, 5);
    T('4.967075651358336432350887274230487326083918996400128705465e-6994866956394', '8.83847368050461420749508161586881613965912043e-17381726714933', '4.967075651358336432350887274230487326083918996400128705465e-6994866956394', 61, 4);
    T('8.026552107518785700709246815312146682026e+42', '-3.0149398636e+218865995302', '-3.0149398636e+218865995302', 35, 6);
    T('1.1613e-682826197326', '-2177257851578005395325663826549301', '-2.177e+33', 4, 1);
    T('-4.0184e+647122645', '-6.4806053292825e-6256473071409', '-4.0184e+647122645', 36, 5);
    T('-3.336824087763139388938627335455160e-56989567', '8.6498803068832516217008005577907750e+128422843033', '8.649880306883251621700800557790774999999e+128422843033', 40, 1);
    T('-1.653894320874934569e-9979', '-2.68949294641644132395306e+887751862109072', '-2.6894929464164413239e+887751862109072', 20, 1);
    T('-6.4995781324508406718170619509911254122e-783732076359', '7.264020901605216333556e-5363', '7.264020901605216333556e-5363', 98, 4);
    T('42735787804306802.43', '3.891543380e+210', '3.89154338e+210', 27, 1);
    T('-4.9113356872361284262984090254590e+658829951542736', '1.591744915267466575176695641644e+2551241', '-4.911335687236128426298409025459e+658829951542736', 50, 6);
    T('1.88544091166824226074320455816025928335992036886824122568e-3696395503392', '9.50619510e-184813152179298', '1.885440911668242260743204558160259283359920369e-3696395503392', 46, 6);
    T('2.377113707811633602497426495e+617616469865660', '-8.6392993749915629383337417036550563186559122227262e+225250', '2.377113707811633602497426495e+617616469865660', 82, 6);
    T('-9.6910090037076483736e+23', '9.913117228871075909315949803581240603900758e+8668', '9.91311722887107590931594980358124060390076e+8668', 42, 4);
    T('-8.2527117283642775413628541e+41165', '-2.9034178220489330970863809566550071253569060747821e-767409546', '-8.2527117283642775413628541e+41165', 35, 4);
    T('-7.002941224736202450e-68049817045', '2.96306901e+5096431', '2.96306901e+5096431', 66, 6);
    T('1.3611079051154263790e+795067282', '2.4e-8758871904354', '1.361107905115426379e+795067282', 76, 5);
    T('-7.6042337734250612953294908418350503483867978400996e+880466774', '52.3095', '-7.6042337734250612953294908418350503483867978400996e+880466774', 57, 3);
    T('-5.20688623956e-8792221', '-9815', '-9.815e+3', 88, 1);
    T('2.70e-708273', '5.828419302256979642964676217266e+37', '5.828419302257e+37', 14, 6);
    T('-1.45733284818328843521431772221007095913129186042515228363e+3519924864962', '3.97820672721584859596622773081800164319125948e-80375873980', '-1.45733284818328843521431772221007095913129186042515228363e+3519924864962', 75, 6);
    T('-5.4597317974650234281751201090205978123e+379', '-5.63387525628086444604096429119568688149215546424584e-15', '-5.4597317974650234281751201090205978123e+379', 56, 4);
    T('-8.951403656210680695574888e+3507553098363', '-5.525924785786613195355851093754754030968185777737376e+8850', '-8.951403656210680695574888e+3507553098363', 77, 6);
    T('5.38325488062e-48882080', '-1.97013211533e+556091358729498', '-1.97e+556091358729498', 4, 1);
    T('4.752325307459731770853748489340398770505115e-407542759225115', '8.86786019943852539538809122121432389024495820560393658e+29412580855657', '8.86786019943852539538809122121432389024495820560393658e+29412580855657', 89, 1);
    T('-6.0770e+851', '-2.5487711725655201075257038593862021344026714025643823e+6941', '-2.5487711725655201075257038593862021344026714025643823e+6941', 85, 4);
    T('7.6687409639034999508410684548437897e-902220622', '1.68108606703982503893890927397e-16395330545886', '7.6687409639034999508410684548437897e-902220622', 88, 6);
    T('4.80264411358724281499667213397551639282548272224e+8826169184', '-8.96105365879137129175713736700551545286616647489e+1957399', '4.80264411358724281499667213397551639282548272224e+8826169184', 91, 0);
    T('-3.3100412884778811045597944678121841482670553242201747e+2948', '-5.56747723691704343912720424654088431265030457400e+91', '-3.3100412885e+2948', 11, 0);
    T('5.6276307705380881243002451036337475e-10', '-6.5400208260950386396923855168590541564153875191423840469893e-28327547764', '5.628e-10', 4, 0);
    T('-2.3498073011553082213732e+1760', '4.012198314551332361521267822e-4663793358844', '-2.34980730115530822137319999999999999999999999999999999999999999999999999999999e+1760', 78, 2);
    T('-8.378490096622289456901237053584623759e+17988102', '1.11429163804371062683e+36448183163', '1.114291638043710626829999999999999999999999999999999999999999999999999999999999999e+36448183163', 82, 1);
    T('-4e-652536', '7.75550048362777925826952618598895509839761e+774', '7.7555004836277792582695e+774', 23, 4);
    T('1.35105246109395133168846074762e+3461814', '5.2109202512434306777032809405937151431498697889024361e-813983794', '1.351052e+3461814', 7, 6);
    T('-2.10647270791969959378249724761412899290726530741222142861166e-5531745807', '9.787621831774493546730726e-28037367386', '-2.10647270791969959378249724761412899e-5531745807', 36, 1);
    T('7.86405742e+284914798', '9.868247043984200100973616e+26', '7.86405742e+284914798', 11, 1);
    T('-5.63075185782355603755815735770e+108', '1.63459768028e-529239335913976', '-5.63075185782e+108', 12, 1);
    T('-1.75830829604043518378876e+666097612478571', '6.91118359381396975033629894837013405733508618557142e-57910662000', '-1.75830829604043518378876e+666097612478571', 52, 3);
    T('-3.54791400314970370116427250781813555568002937375e-64132541435934', '-8.6555002006242788865485027598256e-48229', '-8.656e-48229', 4, 4);
    T('9.4352942257965709627e-49', '3.80091577951533386896e+3177923', '3.80091577951533386896e+3177923', 81, 4);
    T('6.0346766211228872e-9384743597477', '1.0491275037e+331', '1.0491275037e+331', 91, 4);
    T('-3.258e+10', '8.925738145725673006e-55807', '-3.258e+10', 40, 4);
    T('-3.0054815518388541912802241129e-2724949', '3.920262191330217522171078559265354184053996941343e+98512', '3.9202621913302175221710785592653541840539969413429999999999999999999999999999999999999e+98512', 86, 1);
    T('-5.53530740429140991344782475039508840253436079816573671208e-10', '7.1598937e+175698634370139', '7.1598936999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+175698634370139', 98, 1);
    T('-5.7502892645696211350365470940364343436964165789358194e-6858640', '5.13661832776936923325688e-787492309', '-5.750289264569621135036547094e-6858640', 29, 2);
    T('9.5791994370238591888143887595119421391e-40246113036', '3.3149917230799360811384053520467358e+309678037180', '3.3149917230799360811384053520467358e+309678037180', 97, 5);
    T('5.4370922e+62888', '8.850e+17849', '5.43709220000000000000000000000000000000000000001e+62888', 48, 0);
    T('8.24526024471283808602678668e+32085', '-5.91096134857929001517308305284694088795674077991664774e-442', '8.24526024471283808602678668e+32085', 77, 6);
    T('-7.71053070890e+544999', '-2.7887127e-417752583223675', '-7.710530708900000000000000000001e+544999', 31, 3);
    T('8.93940192150927277613774249825922909505612036618662e+56', '-1.89581507022350565550418467428211592774691817499307421617e-369114391133', '8.93940192150927277613774249825922909505612036618662e+56', 97, 6);
    T('3.22587285312e-756577528200', '8.621811941544214595472643480979123819959524045599e-29477703', '8.6218119415442146e-29477703', 18, 6);
    T('8.6231833554074934447617903735e-635695036', '9.4861e+9316645926801', '9.4e+9316645926801', 2, 3);
    T('1.12248470879334138867183022886198405e+196659920180', '3.841116694914531266640166735907e+4771793267', '1.12248470879334138867183022886198405e+196659920180', 77, 6);
    T('1.947840e+10', '-5.53803730649314467314971024962634129574784242346544519e-12601', '1.94784e+10', 24, 5);
    T('-7.8598530112600888378186515e-4057412', '-5.94791004e-65540', '-5.9479100400001e-65540', 14, 0);
    T('-2.97999633760729682849670997890e-79439761284023', '-6.79119026e+442857207494', '-6.79119026e+442857207494', 73, 6);
    T('5.10190387785155452693565148691163117335434467294e+6357', '3.33733504905784976523858867780617308713610568979625395e-681623', '5.10190387785155452693565e+6357', 24, 5);
    T('-1.474679275039065272e+3687393095567', '-2.2012499438466587787262968737786412348e-539202', '-1.4746792750391e+3687393095567', 14, 6);
    T('-9.432346e-10280279373', '2.865230291752468311904e+800021552310762', '2.865230291752468311904e+800021552310762', 78, 2);
    T('-1.1172665273e-692851056624', '5.6801805784285469e+3750232', '5.6801805784285469e+3750232', 27, 0);
    T('-6e+9624841233885', '-9.027565680072152e+3789836601', '-6.00000000000000000000000000000000000000000000000000000000001e+9624841233885', 60, 3);
    T('5.1e+267625693853260', '-2.032072401529572709392300417853423634212100270300745567017e-8957783071274', '5.1e+267625693853260', 92, 6);
    T('1.05091301565554150887335226689685840947617508002380425172416e+799117477', '-5.429742005725904989910044980446e-40163005551', '1.0509e+799117477', 5, 6);
    T('-6.2503313365662395772374760638630443231e-680', '4.47703955604468365251100e+7147023', '4.477039556044683652511e+7147023', 57, 5);
    T('-3.7493981299864470992375266055544e-4760', '-4.92159997831911301977824753115054673293e+45', '-4.92159997831911301977824753115054673293e+45', 68, 1);
    T('3.99847633382393494928229366931810e-573182144', '-3.89384249049526283940510e-2496', '-3.8938424904952628394051e-2496', 99, 3);
    T('-7.95958078917174381e-8263582', '-1.89107177950956011694877450175141755494298933341159e-5196248', '-1.89107177950956011694877450175141755494298933341159e-5196248', 80, 4);
    T('8.445566783208925133e+97086', '6.836849268371108355243251883578419e+827', '8.445566783208925133e+97086', 55, 5);
    T('4.24e-868758602', '-8.136744053339929612937809576376984531091881281942e-6466293114', '4.24e-868758602', 5, 4);
    T('-9.738686261681882878916e+82897441316756', '-7.8e+587567727755', '-9.738686261681882878916e+82897441316756', 96, 1);
    T('-1.41035919087281205786773817187e-32293443', '5.3648676477061955600058426654176828261354424244641925003e-64401', '5.36486764770619556000584266541768282613544243e-64401', 45, 2);
    T('4.74680214788372351358934032182305680928e+779574', '3.46337450005835085073e-99491780', '4.7468021478837235135893404e+779574', 26, 2);
    T('2.13325847317995629064961795051e+946', '7.9137405208535893898014e-9901', '2.13325847317995629064961795051e+946', 32, 6);
    T('-2.78104581593398649004486364625018605937467986e-148022', '8.7740516818422743773957618326730624568437015866876748226e-91014004', '-2.78104581593398649004486364625018605937467985999999999999999999999999999999e-148022', 75, 1);
    T('8.4025769928693388949865737548345625e+46330741', '-7.90805195576750972421058332360118502367972e+778989243', '-7.90805195576750972421058332360118502367972e+778989243', 97, 5);
    T('-7.26851298983848077411251511195101672e-402', '-6.914389e-3293', '-7.26851298983848077411251511195101672e-402', 100, 1);
    T('8.69153672172242218554785306e+348252940675876', '-8.24951521855325896555003148831902130610224792222315819028845e-783110', '8.6915367217224221855478530599999999999999999999999999999999999999999999999999999999999999999999e+348252940675876', 95, 1);
    T('-36693855373483727401011471538092282899533700476378799990156', '-3.7924561189876695996914e-411', '-3.669385537348372740101147153809228289953e+58', 40, 1);
    T('2.860781771832508e-81974893409476', '9.93476275297833937142936031795286910030102509665693501926086e+987984930614', '9.93476275297833937142936031795286910030102509665693501926086e+987984930614', 87, 6);
    T('5.49660e+11', '-7.87556888844897133130046627e-3668', '5.4966e+11', 42, 0);
    T('7.5104474447919299658773960328993237e+4344466813', '1.776e+355500240112875', '1.776e+355500240112875', 64, 5);
    T('5.65225274248993188705435353443662334018e+135', '-7.668024442309578709133223670570896373e+5728002', '-7.668024442309578709133223670570896372999999999999999999999999999999999999999999999999e+5728002', 85, 2);
    T('2.55654147261110417654169150324214944890808887763242309513e-61240918421952', '-8.28188648724855665402957e-376092755042', '-8.2818864872485566540295699999999999999999999999999999999999999999999999999999999e-376092755042', 80, 2);
    T('4.8554666418862586366691387375654569119e-9282080508994', '-5e+7914', '-4.999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+7914', 97, 1);
    T('2.15712705766607102805745185829889686825523e+310215454485', '2.075e+5245', '2.15712705766607102805745185829889686825523e+310215454485', 64, 3);
    T('-8.0012683e+24', '-7.988036254332312210621861623109655e+989297752039451', '-7.988036254332312210621861623109655e+989297752039451', 64, 6);
    T('2.00528e-880832031817', '3.836346503150885050010635225e+801727034', '3.836346503150885050010635225e+801727034', 28, 4);
    T('-7.477916805796e+76634257191', '5.2382742291612270701712626112624836925826959593135707527e+883115565460011', '5.23827422916122707017126261126248369258269595931357075269999999999999999999999999999999999999e+883115565460011', 93, 3);
    T('5.08790312675889578191809631751577908857514577967370045098e+50492493584', '3.28246367321258724197322299996794547178595688740843270e-2978194', '5.08790312675889578191809631751577909e+50492493584', 36, 6);
    T('3.55427893482423975100043863335468e-990389161755128', '-8.18e+141309315026622', '-8.18e+141309315026622', 35, 6);
    T('5.79053095610857414627171794383316773347691348927582775257e+60286797647', '3.01679559367362003282377993245518320033913255014771084598404e+84', '5.79053095610857414627171794383316773347691348927582775257e+60286797647', 87, 4);
    T('-5.5186e-545', '7.32030137210807e-29290520244272', '-5.5186e-545', 71, 4);
    T('50014566838365650692279072612908058561591722895673426.79375', '-4.70e-818881105700632', '5.001456684e+52', 10, 0);
    T('3.65946437579852178218023902453e+85519', '-2.53834e+996738467', '-2.53834e+996738467', 82, 6);
    T('-2.53813979254943143476813471937982845441706985050539088481730e+1463', '-9.914739135984483988914366128205122e+3255957550', '-9.914739135984483988914366128205122e+3255957550', 63, 6);
    T('3.14703465220043572728709492172e+12025719', '3.07535133299358056e+827791687', '3.07535133299358056e+827791687', 87, 1);
    T('4.94113518451019811800998472e-962954649954', '-3.5963539612729977e-35460143067', '-3.596353961272998e-35460143067', 16, 4);
    T('-7.9735235438400089514035100903812743381436816e+428888700', '-5.640012975e-251689', '-7.9735235438400089514035100903812743381436816e+428888700', 55, 4);
    T('1.4603783e+41277718107461', '-4.574110298e-21', '1.4603783e+41277718107461', 82, 2);
    T('-9.3239576445984929704636167761753460113092720e-52359400929753', '9.947841474e+81763', '9.94784147399e+81763', 12, 1);
    T('-2.0244647064358821769e+442386422', '-1.462383205229268322839363648443004022428e+8972475369', '-1.46238320522926832283936364844300402242800000001e+8972475369', 48, 3);
    T('4.275756373564689122075e-82197749048585', '5.9068463552152364435901115979462458350383819939918275e+356176883806695', '5.9068463552152364435901115979462458350383819939918275e+356176883806695', 82, 5);
    T('4.304075568753985570963474044e+19363733', '-4.86472713517267155316723374927e+181216667', '-4.86472713517267155316723374927e+181216667', 73, 5);
    T('-9.782975516495951e-690494157207735', '7.1150895090260316577549536389814487633221531115685540233e-83817545', '7.11508950902603165775495363898144876332215311156855e-83817545', 51, 3);
    T('-6.0685759893447626e-232300689', '3.267178739185567787144144553949e+5663131898', '3.2671787391855677871e+5663131898', 20, 5);
    T('-4.8110303923530715738505e-2612489620779', '-9.038361653766893044372572425840517351998571', '-9.038361653766893044372572425840517351998571e+0', 76, 4);
    T('-6.958703548742406347292369e+3671', '-6.49187936209828150e+82', '-6.958703548742406347292369e+3671', 37, 1);
    T('6.54098740008740e-679075', '1.4714659155e-2428', '1.4714659155e-2428', 83, 6);
    T('6.8483821465106819005213966408801196973398356602e-821656179369577', '-8.455389128301585286023791835283777900417800209719955e+8080', '-8.455389128301585286023791835283777900417800209719955e+8080', 53, 4);
    T('8.7308164499954408678496653054413602483e-616920929420292', '6.219737041729e+20988368', '6.219737041729e+20988368', 38, 3);
    T('8.82176848356969488986375144e-49851701', '-8.397042858274785060797637222953263828931507502883e-28534624', '-8.3970428582e-28534624', 11, 1);
    T('2.129418877514833638409795947946878365458123957e+8710', '9.4408784474772608697e+24384', '9.440878447477260869700000000000000000000000000000000000000000000000000000000000000000000000000000001e+24384', 100, 2);
    T('51851147150431465699979543.5824', '7.67439038005847268989999415049825178540608937614453977e-5307524100376', '5.18511471504314656999795435824e+25', 82, 6);
    T('-2.1490439979226e-10897821991', '1.29532710544721427939592931825963646984440982e-868', '1.29532710544721427939592931825963646984e-868', 39, 6);
    T('4.36547362731144212252599763369879425975928652e+10193876008138', '9.12474349371119184957137122114e-5851995214', '4.36547362731144212252599763369879425975928652e+10193876008138', 46, 5);
    T('-3.49682420959469969587868918277303426373e-646015', '-4.527961388700511208677461614751773096087121150656646e+9329909024552', '-4.527961388700511208677461614751773096087121150656646e+9329909024552', 56, 1);
    T('-5.35100748882067673096559457798839e-20121112852602', '2.628389711910037858989114733229497548010487867832580341e-26527047384194', '-5.351e-20121112852602', 4, 6);
    T('-4.32952031e-98703743', '4.505726137692001586828650140381572516161961373081220e-61891680', '4.50572613769200158682865014038157e-61891680', 33, 6);
    T('-3.5163619864362574207583642549814260364459782153e-82549200313', '9.477280647414332648002512570689383819087810347413646e+465', '9.477e+465', 4, 1);
    T('-9.57004248613128372255544445247250202096074685103182540096660e+73', '-2.60325396130274928851e-90616251930887', '-1e+74', 1, 0);
    T('1.59995698593221260132550475907993639214353e-312583705224010', '1.9171484031724e+8936811', '1.91714840317240000000000000000001e+8936811', 33, 0);
    T('6.027090816683687952808019149346757e+76632276452909', '5.75791439884384069788247943896064319938878417e+45', '6.027090816683687952808019149346757e+76632276452909', 60, 4);
    T('-4.27073125884525273901508034952986724602512928946209644260612e+3099', '-3.94491776723571493435048060611757134603981486149268945e+271602169167', '-3.94491776723571493435048060611757134603981486149268945e+271602169167', 90, 2);
    T('-8.1774007764909470031558171883920481610867022581287310803744e-13878', '99564309366027104900528079', '9.9564309366027104900528079e+25', 46, 0);
    T('1.1545151206e+452', '-8.770950489804266149755e+2920785', '-8.770950489804266149755e+2920785', 82, 5);
    T('-6.109968212875814029152220579074981869758838680e+10685104750', '-8.8393195708645342867154532e-480215', '-6.10996821287581402915222057907498186975883868e+10685104750', 72, 4);
    T('-4.91407510554467406659292563966025430597367270015049e-2710911', '-1.888952698396477581027493089768420479e-13993030', '-4.9140751055446740665929256396602543059736727001505e-2710911', 50, 4);
    T('-4.057653305648421592e-3170910', '-9.66712074588517764178666120958285593437647595628167190565e+44574468619', '-9.6671207458851776417866612095828559343764e+44574468619', 41, 2);
    T('3.866074970006592e+2962154375628', '-2.83622598634249948648506615272084e+3855980817', '3.866074970006592e+2962154375628', 61, 5);
    T('-8.7459062898733588069938863304856805397784033320e+5879728546966', '6.087418448814024091560897885737710340667865219e+52721105176', '-8.74590628987335880699388633048568053977840333199999999999999999999999999999999999999999e+5879728546966', 87, 1);
    T('-2007460656127.1', '-1e+45068835353', '-1e+45068835353', 77, 5);
    T('-1.2721050894223067156422897893830e-621553', '-4.2424011131903e-354303361', '-1.272105089422306715642289789383e-621553', 36, 4);
    T('-3.14e-30765427510', '3e+735', '3e+735', 42, 2);
    T('-1.8427704052151056775456e-3755219977', '9146723754910811249414743233856701521537825076018494413.79', '9.14672375491081124941474323385670152153782507e+54', 45, 3);
    T('2.8722999523433639362760255494e-96541786134736', '6.4776678369165388661158603462317819213e+132', '6.4776678369165388661158603462317819213e+132', 66, 6);
    T('-6.139226e-1220365247', '-5.98636074872e-94078793220', '-6.139226e-1220365247', 70, 4);
    T('6.3296213582845847555950742154e-663554629938845', '7.9623381324252661576579385999490044473859e-56671', '7.9623381324252661576579385999490044473859e-56671', 50, 4);
    T('-7.804931441627685867318268450048011e+5787356', '-5.6781934804761547e-11731106576', '-7.804931441627685867318268450048011e+5787356', 81, 1);
    T('1.68512014737521198265321988914421e+577283967455', '-2.3066492905538354843240221317729489666164037814681e-142991004818395', '1.68512014737521198265321988914421e+577283967455', 92, 2);
    T('-2.1693931106987503225936478866612731035e+131822568022', '7.478e+9434901170', '-2.16939311069875032259364788666127310349999999999999e+131822568022', 51, 1);
    T('-4.225945369605468095963287756e+30155419868345', '-6e+7192804', '-4.225945369605468095963287757e+30155419868345', 28, 0);
    T('-4.909453002420e+29', '4.3233857108293303380550e+25', '-4.9090206638489170669661945e+29', 57, 0);
    T('-2.58515160270e-4481', '-2.217673293390622296012324282e-15595952277786', '-2.5851516027e-4481', 22, 1);
    T('-4.2464343015329291459354176231174536737634667e-672257215658670', '-2.43631419350257e-312063231772645', '-2.436314193502570000000000000000000000000000000000000000000000000000000000000000000000000001e-312063231772645', 91, 0);
    T('1.3883299201339e-428139440', '5.0772932920881e-9146829254', '1.3883299201339e-428139440', 94, 1);
    T('-8.24842559476e+4588', '-3.39108140735262697655292066730514971566843403e-1793661345', '-8.24842559476e+4588', 48, 1);
    T('1.9402620278589472567e-22734', '8.2164102121196866351788539e-44', '8.2164102121196866351788539e-44', 29, 4);
    T('-22389711468462581208639940181630990.628172', '8.223719075835884572232747388439363358453481916864604e+28300', '8.22372e+28300', 6, 4);
    T('-8e-910481277585', '8.4049264980258544023030625085271949909343013116e+54', '8.4049264980258544023030625085271949909343013116e+54', 54, 2);
    T('2.85210047384558547430971225758929620777346e-38809', '6.02292548138790447068208489816175065182279155481675e-52264625', '2.8521004738455854743097122575892e-38809', 32, 1);
    T('-3.81865417488048e-83040821825', '6.14841505e+8555', '6.148415049999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+8555', 100, 3);
    T('25138809490753871351321860417763843125947066.819', '2.2029801278171417107324557230645747502121369e+378878106513944', '2.20298012781714171073245572306457475021214e+378878106513944', 42, 2);
    T('6.94001385739083839e-434', '-1.394e-375656604790', '6e-434', 1, 1);
    T('-6.169895185700450429854847358489264576039232244385975436483e+9856142223387', '-1.675594177759248e+29603699154126', '-1.675594177759248e+29603699154126', 84, 1);
    T('8.77571994258979274883961809347702713185326448210e-98240455135359', '-8.208476683792426544497020033170044109450057372072697e+296', '-8.208476683792426544497020033170044109450057372072696999999999999999999999999999e+296', 79, 2);
    T('1.6312e+7422', '4.0244002044240948973032204065640269304959963830558693257664e-438', '1.63e+7422', 3, 1);
    T('1.8818176e+92211738616316', '5.979523363622014803929331054282851823009445573484e+72290102578251', '1.8818176e+92211738616316', 49, 5);
    T('-8.5027157829686995106620896076160435992288938895e+600016', '7.90637456e+93', '-8.5027157829686995106620896076160435992288938895e+600016', 93, 4);
    T('-5.89682714e+8232241015', '9.480346237004010859018165112693295232545466458881706652865e-88336358623530', '-5.89682714e+8232241015', 53, 5);
    T('1.2090978088782319213744926e+179', '6e+3341293749158', '6e+3341293749158', 46, 4);
    T('-4.898308e-5030826309', '-1.225161410403841509975197075455868e-25078577', '-1.225161410403841509975197075455868e-25078577', 69, 6);
    T('-4.675612753809698282200e+71202105740415', '-8.1464196663561640e-6941299', '-4.6756127538096982822e+71202105740415', 25, 6);
    T('-1.2711459e+7838057075738', '-5.47742875418063834822546106284e+87252', '-1.2711459e+7838057075738', 92, 2);
    T('-4.830309e-75394320011321', '3.36059551960525487024341652e+90540327463', '3.36059551960525487024341652e+90540327463', 76, 4);
    T('3.49331350360860806309521811240703304316e+468', '8.48179274088457054369e-708103821', '3.4933136e+468', 8, 0);
    T('-8.53922157686665021384497220293920385489907889359e-4259200771559', '-6.996250343672887782547373e-278829902157', '-6.996250343672887782547373e-278829902157', 65, 2);
    T('5.119513342077176666238814128911369099077810728695e-5375536', '-8.2324251350118377562743223084130769089770861422397983812685e+404835911', '-8.23242513501183775628e+404835911', 21, 0);
    T('1.592365898920620700281826e+5993177', '6.468788931782494861500123311789710814e+67513', '1.592365898920620700281826e+5993177', 41, 4);
    T('3.3917826284862246895439734664199139066801e-211', '-9.2628166888490068172479318813751914650484473282994722714326e+5110', '-9.2628166888490068172479318813751914650484473282994722714e+5110', 56, 5);
    T('7.28676124484323940982701774385568602716293384e-7930954', '7.5687190277239138080784677946990e-3645505399', '7.28676124484323940982701774385568602716293384e-7930954', 86, 6);
    T('-3.37921934138116032764126e-636', '8.008274323337531996378e-945742082915', '-3.37921934138116032764126e-636', 72, 5);
    T('-8.90597469816298756017485354e-40134424', '820320941295346172996774626124106679154752179.6', '8.203209412953461729967746261241066791547521795999e+44', 49, 3);
    T('6.8231217020883606500398228171e-42275302065', '2.4477603470490669231947600327196112e+83012422435', '2.447760347049066923194760032719611200000000000000000000001e+83012422435', 58, 2);
    T('-4.887884505040888463063596617754128307520100271655767e-976103609676', '6.99124660e+492', '7e+492', 2, 0);
    T('-5.1948947e+197759730', '9.47424745917838511932716755518e+1803986', '-5.194894699999999999999999999999999999e+197759730', 37, 1);
    T('5.6677796351472730177838151850238009027322814335324779548181e+91', '-7.13236554561408923052340293017e+81462', '-7.132365545614089230523402930169999999999999999999999999e+81462', 55, 2);
    T('-9.2789845385267041354100910233836588811655531e+47274', '-4.66437695032631e-1056', '-9.2789845385267041354100910233836588811655531e+47274', 92, 5);
    T('-9.2695097e+69', '-4.3675554300e-91535', '-9.2695097000000000000000000000001e+69', 32, 0);
    T('6.5454508621123717667270205632939e-395688853', '3.25e+14890797', '3.250000000000000000000000000001e+14890797', 31, 2);
    T('3.140096776441601747296087834005795425239260e+77370535', '-9.98642459e+1963', '3.1400967764416017473e+77370535', 20, 4);
    T('-2.2585719843248211077256523335596521956e-2320637', '8.848094860741561024944921532203398753135638e-297005263979486', '-2.258e-2320637', 4, 1);
    T('-6.1633e-738428827511', '-4.1526704292126830752348830e-5264', '-4.152670429212683075234883e-5264', 44, 4);
    T('-3.8411868961214360342498792914908773344319602433639265206968e-762', '356925147209475694.000688', '3.56925147209475694e+17', 21, 1);
    T('-9.6326866419362522608727657055339954e+741598480452', '-1e+64010', '-9.632686641936252260872765705533995400000000000000000001e+741598480452', 55, 3);
    T('8.4207278321369410265701e-68821829902950', '-2.312533174222239019e-41666468', '-2.312533174222239019e-41666468', 25, 0);
    T('1.30646525708193772804447791731082844155564852391110562618382e-54840044', '-391009165492082047830301864847383153741078459046', '-3.910091654920820478303018648473831e+47', 34, 2);
    T('6.50453047898755547977352544169149329e-43594', '-6.2849379434991091135267745063117974255916043e+52', '-6.2849379434991091135267745063117975e+52', 35, 3);
    T('-2.702195676127019844579351848547826202573234921442e-450', '6.43771962204611865797001980173152188658e-90320448', '-2.7021956761270198445793518e-450', 26, 2);
    T('6.38747230492405247234145088477711811884286e-804816644', '4.3444958883269501702945353313639064791282949e-299350044', '4.3444958883269501702945353313639064791282949e-299350044', 76, 3);
    T('-5.2948545513114115333e-8908973079', '-6.96694225249169794222e+105792294', '-6.966942252491697942220000000000000000000000000000000000000001e+105792294', 61, 0);
    T('-7.909093834082844420590092e+643', '-2.8087213147132507252721865389045402763128863728e-42111690401209', '-7.9090938340828444205901e+643', 23, 4);
    T('9.5546540939847193735450484425744057050993838221708e-2139', '-3.53509649e+52', '-3.53509649e+52', 48, 4);
    T('-5.930774947914662505991e+98', '8.939432952530770583514540973990927997095278e-930539172704617', '-5.930774947914662505991e+98', 77, 4);
    T('-9.6690e+90054659', '-6.7874002650e+59605034', '-9.669e+90054659', 20, 2);
    T('7.47323673223e-24', '8.360230796158372473534626520200034e+6077009', '8.3602307961583724735346265202000340000000000000000000001e+6077009', 56, 2);
    T('8.23412251823826093562539219976154539219949021124643270842867e+133', '8.1519304656202160330057413823107606131938538840416536e-852849', '8.23412251823826093562539219976154539219949e+133', 42, 3);
    T('-2.08469000798761505390959899671042205709e+77', '-412812556151018166906095194200932.381923', '-2.0846900079876150539095989967104220570900000041281255615101816690609519420093238193e+77', 83, 3);
    T('-9.035871997864269073989226009e-3837935', '-6.862842596794440562029932079717822e+730119566', '-6.862842596794440562029932079717822e+730119566', 96, 4);
    T('-4.1815155335378817198929862161879343346056078869211060824e-4585', '3.1540532518096442652873403619483980591e+39', '3.1540532518096442652873e+39', 23, 6);
    T('-9.70171045154959829967510659194369e-437', '6.354687822587243840548083447250359874575124e-37346', '-9.7017104515495982996751065919436899999999999999999999999999999999999999999999999999999999999999e-437', 95, 2);
    T('1.36844681329909558803103032680468535e-589635955083849', '7.70619051915177675019208514275018857e-232973697898', '7.7061905191e-232973697898', 11, 3);
    T('5.266693679565966408944550e+121636', '4.436650626570383159e-62', '5.26669367956596640894455e+121636', 60, 4);
    T('7.01580188576168871029531615918156308131767e-4091', '-6.776697147477398702585828684556e-721030', '7.01580188e-4091', 9, 1);
    T('2e-7760', '8.42804625534892202070363e+24348', '8.4280462553489220208e+24348', 20, 0);
    T('1.8440e-2434', '-4348258.00611', '-4.34825800611e+6', 63, 0);
    T('-4.8775069916377941234144392616e-712891713', '-3.8934297488833488304581158263848299693697494e+23393', '-3.8934297488833488e+23393', 17, 1);
    T('1.74725857701574606582410512469874460349798773331940182037919e+70', '-4.683345058552005110838853706269521485913003e+91696418598859', '-4.683345058552005110838853706269521485913003e+91696418598859', 56, 3);
    T('7.9228995542477161972786522025733493e-23352152130', '-5.8876987674274e+126952527534760', '-5.8876987674274e+126952527534760', 28, 6);
    T('1.6536261e-61390841074654', '3.89233528497224743898754e-15826', '3.89233528497224743898754e-15826', 78, 5);
    T('-2.9e-35896431199897', '-5.6578774159281137282480957865922726340996192e-8942488436549', '-5.6578774159281137282480957865922726340996192e-8942488436549', 73, 2);
    T('627117909138629219739194648472670216061986579007825646238.55', '2.6870284035903676113907251e+18158', '2.6870285e+18158', 8, 2);
    T('9.4814997360e-71137711259', '-7.3e-856', '-7.299999999e-856', 10, 1);
    T('6.7473966853149332910282175775e+65753', '-4.386388304680538545215143635228e+8973632095', '-4.386388304680538545215143635228e+8973632095', 86, 5);
    T('9.684784154995152912238871607809048987462470e+608285449089890', '7.20881489388031615936827262546016916600594073835096010255e-46398327113', '9.684784154995152912238872e+608285449089890', 25, 0);
    T('2.35948400514754e-7856216', '-1.54068225914300639854760130853015671726431392e+7010705235', '-1.540682259e+7010705235', 10, 4);
    T('1.515664590588914195216157110508138953407e+22419884808', '1.756956e+157047891128', '1.756956e+157047891128', 76, 5);
    T('-9.5080783206544123620637265108468e-55', '-4.243461394258287056880185e+595', '-4.24346139425828705689e+595', 21, 0);
    T('3.8e-56672', '2.13486104781998963622594887271293458750144477820761216e+69', '3e+69', 1, 2);
    T('-1.92709151609772987237952601443483e+147104785354439', '2.150745721802223109030220e-34996103077126', '-1.92709151609772987237952601443482999999e+147104785354439', 39, 1);
    T('-9e-8', '2.048999825719473369430094762e-4412283584', '-8.9999999999999999999999999999999999999999999999999999999999999999999999999999e-8', 77, 2);
    T('9.8232648960400e+892619196', '1.9516177954343973182e-290709833216', '9.82326489604e+892619196', 81, 1);
    T('-5.402985667235563514319606300434183935678016e-300', '-1.38526038906677845265760576905774619e-446', '-5.4029856672355635143196063e-300', 28, 4);
    T('-6.30580396859154507296755429594597607966428583251e+879', '1.6930147423087393772588273600938076031112111957042390409219e-47690056659', '-6.305804e+879', 8, 4);
    T('-9.9796956968123e+582634215652140', '-2.66230365068906152956217427362085854946572045e-977327', '-9.98e+582634215652140', 3, 4);
    T('-7.33432510592303870548443412203881754e-44802111069382', '-7.069584913083433688e-14', '-7.069584913083433688e-14', 71, 5);
    T('2.42826547672733e-288160539', '1.164e-879602371557757', '2.42826548e-288160539', 9, 5);
    T('-9.53396354e+5786521', '3.194492081212e-43226508', '-9.53396e+5786521', 6, 4);
    T('5.28009564028445989391242e-910572', '-4.3950092010434e+53065916', '-4.3950092010434e+53065916', 63, 5);
    T('9.4371258634519868049884961e+582395', '5.7838759505005943085754091213919956e-770305729', '9.4371258634519868049884961e+582395', 75, 6);
    T('-4.8233e+67481936155', '121334955733628491919606106497009214292591401975021.956806', '-4.8233e+67481936155', 91, 6);
    T('-9.104735467720197201633e-52375', '2.389264203722379511147553867687780994e-8026901', '-9.104735467720197201633e-52375', 56, 0);
    T('2.08079441045108425117e+65385777701663', '-8.576921167457769796108978461e-21500690363989', '2.08079441045108425117e+65385777701663', 24, 0);
    T('-6347965096502677268026816878122587530206624449275486.413', '-6e-73622751173775', '-6.34796509650267726802681687812258753020662444927548641300000000000000000000000000000000000001e+51', 93, 3);
    T('4.8861658263236612311958669340668205412923024e-6682365012000', '-1.68958537e+99', '-1.68958537e+99', 52, 4);
    T('5.5889817517121654296e-1507619050', '3.3774770530e+95987', '3.377477053e+95987', 10, 3);
    T('6.77123588e-6006446146581', '6.23071269244705082397929625644115430325241591447689106e-74357096596', '6.23071269244705082397929625644115430325241591447689106e-74357096596', 60, 4);
    T('-3e+5512629', '3.9256729684245440e+600234', '-3e+5512629', 31, 6);
    T('1.8640642626692962637098e-10894164344791', '6.571681290255158e-932193', '6.571681290255158e-932193', 83, 4);
    T('9.450264342e-7215127483416', '-9.52679101786547669711647021449283611905750990e-1123628859133', '-9.526791017865477e-1123628859133', 16, 3);
    T('-1.86948251004101e+76108495', '-3.5824077944e+230279098788827', '-3.5824077944e+230279098788827', 56, 4);
    T('-6.3477034321328696e+87407385683', '-4.83770468663622446202382825537663542632144843e+71543044', '-6.3477034321328696e+87407385683', 21, 6);
    T('7.50527212016798586401001e-715939927', '-5.479e+98363397', '-5.47899999999999999999999999999e+98363397', 30, 2);
    T('4.688957909498615529984338050112424351815488634379950449e+55', '6.2586116390513535791108865006e+4295', '6.2586116390513535791108865006e+4295', 77, 5);
    T('9.7480315425154002042761093042774035e+56473080510227', '-5.7779091785629878654683269e+21662045727', '9.7480315425154002042761093042774035e+56473080510227', 42, 6);
    T('6.813851e+5874', '3.63954256e+42322983', '3.63954256e+42322983', 46, 3);
    T('-6.552720434323364e+7174119535984', '-4.344773438703317155928012900672e-282828823', '-6.552720434323364000000000000000000000000000000000000000000000000000000000000000000000000001e+7174119535984', 91, 3);
    T('-6.977148880696485109492322488206564e-5700', '-8.89479236954701160e-46454', '-6.977148880696485109492322488206564e-5700', 80, 4);
    T('-4.5125177e-35214277', '-3.921247156515630037872628204692432008e-6103', '-3.92124715651563003787262820469243200800000000000000000000000000000000000000001e-6103', 78, 0);
    T('-1.908227834727032954324091599814905008e+45889594', '-4.262489705824881e+4729568', '-1.908227834727032954324091599814905008e+45889594', 53, 2);
    T('7.48312758875e+36088494283848', '-9.72943739233526e+873797762527', '7.48312758875e+36088494283848', 38, 5);
    T('4.343422664739568400579374435178060990456167992e+17522037818', '-2.44731653837348644722091093406069764663e-60596457', '4.3434226647395684005794e+17522037818', 23, 6);
    T('-1.4e-43648007779', '-3.785137846506465723e-12770513524496', '-1.4e-43648007779', 69, 2);
    T('-9.1080647687714183600328923171e-352345691', '8.779030179661e-51762', '8.779030179661e-51762', 76, 2);
    T('-6.9e-106694901', '3.643367291934197e+202', '3.643367291934197e+202', 38, 6);
    T('-5.87152303229e-85401009925170', '-4.195920080558848953595829120076336513033209403023177e-71496850803', '-4.19592008055884895359582912007634e-71496850803', 33, 6);
    T('-9.3390963e-86', '-9.45670529069016800688958344107551268311680210029709136851e+76582491643177', '-9.4567052906901680068895834410755127e+76582491643177', 35, 4);
    T('-6.45368751075301961058763900972732898256585e+660', '7.31683921326873055943447594803e+180568842229992', '7.31683921326873055943447594803e+180568842229992', 49, 5);
    T('2.5e-542402', '-3.3144920791702670968489657038213096326651856026857e-37', '-3.3144920791703e-37', 14, 6);
    T('4.0033639061064681617911715973038247329059e-987026784', '3.0e+15098247853311', '3e+15098247853311', 70, 4);
    T('3.2506859e-71010022263', '8.794911296623381e+6905123', '8.794911296623381000000000000000000000000000000000000000000000000000000000000000000001e+6905123', 85, 2);
    T('-4.3737393e-857829', '-6.4357092938666e+644082901', '-6.4357092938666e+644082901', 64, 5);
    T('4.91213080608543575491390060349865743e+156', '2.18666323301370524865835625060416422749891e-7317978433595', '4.9121308e+156', 8, 6);
    T('92.37867098', '-8.24024146979497035647006939752967914994046483879137883e+9721658', '-8.240241469794970356470069e+9721658', 25, 5);
    T('-2.0569863130102805e-47717376678', '5.9311343953768739038984666106987956191466358221524573e+2015103', '5.93113439537687390389846661069879561914663582215245729999999999999999999999e+2015103', 75, 3);
    T('-1.539130568231691595168851396338614543e+220922079085', '3.4176534e+762732', '-1.539130568231691595168851396338614543e+220922079085', 69, 6);
    T('-1.385858e+2125715481014', '4.6791593593431720928933131803658141217e+23006210703689', '4.67915935934317209289331318036581412e+23006210703689', 36, 1);
    T('-4860737787676538600650101716810499919732861410.592', '-7.07204705056929254053007804171582239673288481693946474074e+152', '-7.07204705056929254053007804171582239673288481693946474074000000000000000000000000000000000000000001e+152', 99, 3);
    T('-8.27021791837274e+923579478', '3.1919265444752938927005939620e+79', '-8.27021791837274e+923579478', 96, 4);
    T('0.00073509', '-7.7375582039e+3934036384377', '-7.7375582039e+3934036384377', 71, 3);
    T('-2.57479751274665759489355e-40423949742', '-1.149e+8248', '-1.149e+8248', 34, 1);
    T('-8.8491982e-6394144', '2.18487873571986755315553353e+877514501928722', '2.18487873571986755315553352999999999999999999e+877514501928722', 45, 1);
    T('5.7071599950050753764939700128356714621806001213973393e-25850507204', '1.68998243339e-376569893177652', '5.707159995005075376493970012835671462180600121397339300000000000000000000000001e-25850507204', 79, 2);
    T('3.80583343e+6271424', '-3.237219913948979985048004997453127325693029783600386e-909', '3.80583343e+6271424', 63, 6);
    T('-2.67829413577243390962939458031368446767344e-69915', '-6.03147000049841172060779633486746599165057153853315688976e+13051708093536', '-6.031470000498411720607797e+13051708093536', 25, 3);
    T('9.170196435114765681603553997474816839e+446338173639028', '7.8588946328771663760684e+29', '9.17e+446338173639028', 3, 5);
    T('189141154341936678348386828721118.624', '9.59097418864360465869290464663179473824329e+98707728', '9.59097418864360465869290464663179473824329000000000000000000000000000000000000000001e+98707728', 84, 0);
    T('2732281246614504172344850', '-5.1416623e-67024', '2.73228124661450417234485e+24', 79, 4);
    T('-2.511848630604449e+31400848220', '9.38753259e+42388799470148', '9.387532589999999999999999999999e+42388799470148', 31, 3);
    T('5.0120000419460065176405249709068526382329804874690877381e+980913465610', '-5.501195675646288365836604487274468808735158991e+1530', '5.01200004194600651764052497090685264e+980913465610', 36, 4);
    T('8.967976885900500884635181337831688e+5633907', '8e-305', '8.9679768859005008846351813378316880000000000000000000000000000000000000001e+5633907', 74, 0);
    T('-8.62483823142220862938044491502465e-291424', '-3.90246476293970164515073618297900033610548690783570e+59754389292129', '-3.902464762939701645150736182979000336105e+59754389292129', 40, 4);
    T('-4.39630538026938556996578409237637e-87345928082', '-6.9302104877711343145933104813117509176503768705832278778167e+389357833', '-6.9302104878e+389357833', 11, 4);
    T('-7.78683703', '3.6366950503736739619e-336420326', '-7.786837029999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+0', 97, 2);
    T('-7.738777172e+54', '-8.56120150885481672811330400673348984248708018e+95024053343', '-9e+95024053343', 1, 3);
    T('3.364040262968463639640e+65', '-7.6946255375996365290748404738900461647518840e-30051409', '3.36404026296846363964e+65', 30, 2);
    T('6.3894447870e-959083246509', '-4.243746326123731e+70368010', '-4.243746326123731e+70368010', 68, 4);
    T('6.49760193400815338453031555e-606397262108', '2.9239492043e-201510017', '2.9239492043e-201510017', 41, 4);
    T('-3.53419049520054258895047806083851404e-418097726', '4.6453905366381613421186993e-281', '4.6453905366381613421186993e-281', 27, 2);
    T('5.98380516291575844847790e+1126160499', '6.73956082805299e+9320145333', '6.73956082805299e+9320145333', 33, 4);
    T('1.513514501557288498e+81653104703955', '-2.2510885199682379654259993433356386861267295110020108645824e+3179908604180', '1.5e+81653104703955', 2, 5);
    T('3.316497435951469169283226449e-437631743', '-3.0468716e+36621885248', '-3.0468716e+36621885248', 40, 4);
    T('-6e+205927143273', '-8e+142319', '-6e+205927143273', 97, 4);
    T('-8.71e-967014785708', '6.6288200827255054611158319067188078e-9', '6.6289e-9', 5, 0);
    T('-88505330828414187340376435508717368.2801443384374592132781', '5.303937174281078103038395695096166325948339428762997492e-6138544968', '-8.8505330828414187340376436e+34', 26, 0);
    T('-7.81610615151050753965787e-81319429335895', '-2.21562038363233432609568502442926879490e+934093119183', '-2.2156204e+934093119183', 8, 4);
    T('-5.13658085080796468787231956757549429769420e-33582', '9.73610147909394104114511471186499307002625152304621516473e+63', '9.7361014790939410411451147118649930700262515230462151647299999999999e+63', 68, 1);
    T('3.20277592431608706701847307106730e+33', '-3.376534144895675124435164323777887796568385246814e-8484323490', '3.2027759243160870670184730710672999999999999999999999999999e+33', 59, 3);
    T('-8.3269807633261070956520430828161e+110', '3.915418914319677e+96', '-8.326980763326067941462e+110', 22, 2);
    T('3.90544782422937431838491211088751379185027582865963317656809e+4554', '7.1124218918984083820793619529415265287390e+42', '3.90544782422937431838491211088751379185027582865963317656809e+4554', 85, 6);
    T('1.404002495149463714849431677789221359256321158852e+787', '-5.62058542696539e+558687612475', '-5.62058542696539e+558687612475', 89, 3);
    T('7.68346067216915687502396413881934211e+21467894155897', '-4.450994424582e+45300582567064', '-4.450994424581999999999999999999999999999999999999999999999999999999999999999e+45300582567064', 76, 1);
    T('-3.944607920746413766885697357635e+7567418792622', '2.758861197644085473955085645385594719005896e+47', '-3.9446079207464137668856974e+7567418792622', 26, 3);
    T('-4.623861693175756705843835974e+934773', '-9.1083252665204831041264554991612e-1413', '-4.623861693175756705843835974e+934773', 49, 2);
    T('-6.7212561972795496875969121e+30', '-6.88e+360096', '-6.88e+360096', 27, 4);
    T('9.67526258275665e-646015350481', '9.383815211046118649937535276e-719214945', '9.383815211046118649937535276e-719214945', 49, 3);
    T('8.255370323823758014384e-8337117837', '8.4614357302660871880826988725662818717e+23581857795', '8.4614357302660871880826988725662818717e+23581857795', 84, 6);
    T('-8.01349655377694251657427e-215268811', '-7.99535264788341e-693642400', '-8.01349655377694251657427e-215268811', 53, 5);
    T('-6.396e-88088', '2.268155476222274365490987321e-3484161', '-6.396e-88088', 43, 3);
    T('-6.572015473473978180521e+8099', '8953464', '-6.572015473473978180521e+8099', 54, 6);
    T('-4.12366350579818708e+4918429', '-2.5245170286832681234811033514532851102e+22573980151893', '-2.52451702868326812348110335145328511020000000000000000000000000000000000000000000000001e+22573980151893', 87, 0);
    T('8.696197320689785e-10004951573', '-4.265639982028340489225359726299086201480169e-77511263024', '8.696197320689785e-10004951573', 46, 0);
    T('-8.63619693168075759276202e+24', '-1.273510613099276710040903384e-66', '-8.636196931681e+24', 13, 6);
    T('4.3952623381039071579939215715939e-31461220049023', '-1.3749204003434211121922349321363468521820789208108e+417161581', '-1.3749204003434211121922349321363468521820789e+417161581', 44, 6);
    T('11123508014456538907426660706461157632827167281667.52931856', '-1.53330740039664226427266590013e+31', '1.112350801445653889209e+49', 22, 5);
    T('8.6739027352101406144774416662303171114703109809185380e+57', '8.3866e+66061', '8.3866e+66061', 14, 3);
    T('5.287e+6548186608759', '7.3721055e-644635143', '5.287e+6548186608759', 45, 6);
    T('-7.119061437164694998695e+1327581789', '1.8719320510438893426548971e+875088365454', '1.8719320510438893426548971e+875088365454', 27, 6);
    T('-6.65916600874e-646', '-8.5717856860618155151890677995975665316953401472648308004e-535', '-8.5717856860618155151890677995975665316953401472648308004e-535', 72, 4);
    T('6.9692828501053160106759080644449086624494962755259e-2137117281344', '3.3328735261797684916200074351e+9212159594', '3.33287352617976849162000743510000001e+9212159594', 36, 2);
    T('-1.08587125403240229519372774209240033914795773698848626436166e-37331773395', '5.661498079479838225535773134931208064815321893375e+691', '5.7e+691', 2, 0);
    T('-6.555387737484607534879470502928184868998e+16013120522', '-4.2608388079012517648596415528285248476570e+73819', '-6.555387737484607534879470502928184868998e+16013120522', 58, 4);
    T('7.8575752103462150280877756691291457697311377820220525756051e+6255828', '9.69887639579732e+74723403554', '9.69887639579732e+74723403554', 46, 5);
    T('-5.4944e+9175497', '-7.3772927247577379226803620974742871779830e+168809445141', '-7.377292724757737922680362097474287177983e+168809445141', 60, 1);
    T('9.00904420413913947168887453344866636189901338e+117', '-7.43621002918845123539979184525990e-177300', '9.0090442e+117', 9, 4);
    T('4.3422463694366937209761239399221205532314e+48', '-4.219789633241148152577503e-748818212', '4.34224636943669372097612393992212055323139999999999999999999999999999999999999e+48', 78, 1);
    T('-1.85311176140898794505035987542981494092869517458892564e+94', '-9.8882725410529334944449e-10089656', '-1.85311176140898794505035987542981494092869517458892564e+94', 77, 1);
    T('7.36354137458261090e-4148298620', '5.85525470203360127e+4366633', '5.855254702033601270000000000000000000000000000000000000000000000000000000000000000000000000000001e+4366633', 97, 0);
    T('9.260e-86', '7.283769092332666329523195374724234515500708188721583e-26396', '9.26e-86', 26, 3);
    T('9.51931594789836783189337900868e-223928', '-7.1307238e+8499', '-7.130723799999999999999999999999999999999999999999999999999999999999999999999999999e+8499', 82, 1);
    T('2.4473080425589588016333256803530e-12181670278', '-3.785327995473582157024073e+2971423', '-3.785327995473582157024072999999999999999999e+2971423', 43, 2);
    T('-6.755698842754e+13', '8.0613612463824754799129919312018146e-16889196938017', '-6.755698842754e+13', 80, 3);
    T('-8.7425e+223219512558458', '7.013859883379248536e-467716825190', '-8.7425e+223219512558458', 99, 4);
    T('-2.4646978851080934868168430759402692736066e-95455790910', '-2.85952470544498052720639612420213521622922272e-2483', '-2.85952470544498052720639612420213e-2483', 33, 1);
    T('4.3758352897686e+18', '1.42857771237751723389192329208547e+935685858325', '1.428577712377517233891923292085470000000000000000000000000000000000000001e+935685858325', 73, 0);
    T('3.4294428657714200995046731859329e-717885', '8.6195864609466648730582330295251816615049347303927439e+352123817252', '8.619586460946664873058233029e+352123817252', 28, 3);
    T('2.281131868947837022e+5042838', '4.6e+871848945', '4.60000000001e+871848945', 12, 2);
    T('-1.870640208834694713365996813201243196897915474e-217101', '-7.92462292268027461075185790049758804661e-3760965', '-1.8706402088346947133659968132012431968979e-217101', 41, 2);
    T('8.9556542779448e-57277972739267', '-2284613297.32861515', '-2.28461329732861515e+9', 55, 5);
    T('2.207632270597472125042957476318850322077784448664242281905e+69963935', '-5.07631918306006e+7590966', '2.207632e+69963935', 7, 5);
    T('3.207138340323673139496173e+667', '7.103466263793731443514171110270274917395406093759265186844e-238120884931', '3.207138340323673139496173e+667', 95, 1);
    T('4.7660310024023e+192032669116', '9.920552583e+387', '4.7660310024023e+192032669116', 59, 1);
    T('4.89908287385370909048328491226053e+2820087', '-4.76324792808410492595e-442917239455048', '4.89908287385370909048328491226053e+2820087', 78, 2);
    T('5.7563e-545164118434', '-9.77595064414001643774598135376528382199380e+56657679770', '-9.7759506441400164377459813537652838219938e+56657679770', 60, 6);
    T('-3.5578338102174679e+22', '8.246740213882001e-18181', '-3.557833810217467899999999999999999999999999999999999999999999999999999999999999999e+22', 82, 1);
    T('4.019648800231095003289668092490915e-910724479781240', '8.14273464018088781492149119624025901665766936959740681e+58939397668', '8.1427346401808878149214911962402590166576693695974069e+58939397668', 53, 0);
    T('-1.8e+416660433207415', '-5.233827035923527052980686058234111621480779613961e+597969488', '-1.8e+416660433207415', 73, 2);
    T('-3.128153758669233896541179569213909107321e-989068524', '7.12902957032958e+92585710816207', '7.12902957032958e+92585710816207', 34, 4);
    T('-1.955714865753683956779008179892457134137647429e+9678', '5.940242068664925825453871839749296818034881067137800697e+975463894599', '5.9402420686649258254e+975463894599', 20, 3);
    T('7.050773053482720522727220173498984095e+2651502261', '-7.758544274629708307048771360719676447998e+98279', '7.050773053482720522727220173498984e+2651502261', 34, 3);
    T('-9.7837137266495e-33228548101403', '1.25379198513644885419783107030825485537298102513751550e+7072', '1.2537919851364488541978310703082548553729810251375155e+7072', 69, 6);
    T('3.86725216776002372548381858604540663699226316386924e+39084', '-6.9882785452805866316261075333310172413373724790390207267864e-28173882037', '3.86725216776002372548381858604540663699226316386924e+39084', 51, 0);
    T('-1.2922960846126268892840137e-492075952933', '-4881', '-4.88100000000000000000000000000000000000000000000000000000000000001e+3', 66, 3);
    T('-8.2646893375698719446255185e+4476513', '2.8622747272288952099626573441e+70192204601986', '2.8622747272288952099626573441e+70192204601986', 65, 0);
    T('2.6671396956e+69', '3.9060244249481348756268138478e-29057890', '2.6671396956e+69', 34, 3);
    T('-37206405.2402', '-1.295869747725335441239184e+372878258148', '-1.2958697477253354412391840001e+372878258148', 29, 0);
    T('595955823406630947702', '2.117e-5184', '5.9595582340663094770200000000000000000000000000000000000000000000000000000000000000000000001e+20', 92, 0);
    T('7.959363715130228297385210133057877823226695965484810e-208464697', '4150818187099523919989593220403192475716114', '4.150818187099523919989593220403192475716114e+42', 83, 6);
    T('-3.98153604471174157538906339357279280e+665433', '-2.58e+62751729421935', '-2.58e+62751729421935', 5, 4);
    T('8.11708979257317237402407992995740442445411772937211043294e+1174701', '11935100258518316.36337041', '8.11708979257317237402407992995740442445411772937211043294e+1174701', 83, 6);
    T('-6.3745759475165507844863363375212195380785657975599207222e+184565591249938', '-4.5344267334480843227066778636010080396751363449e+813346226764', '-6.3746e+184565591249938', 5, 0);
    T('6.57245343520e+272849790813126', '-1.57318703967158252e+2826', '6.5724534352e+272849790813126', 76, 4);
    T('4.9664e-463970367415', '-9.43816730165397039270985034689328424094760654728062469982806e-36626685993480', '4.9664e-463970367415', 21, 5);
    T('6.6236164901547645703315062044155806877e+5183146474', '-2.7358275211238539678e-4684', '6.6236164901547645703315062044155806877e+5183146474', 53, 2);
    T('9.18070530867840024444051837759334543984181115563733e+3320', '3.64e+82805', '3.64e+82805', 14, 5);
    T('-8.937548533989841787e-299841085860', '1.3e-5276693', '1.3e-5276693', 52, 5);
    T('6.86583458367679015304379755022576870864182633658143e-273885708596', '4.43e-5870419', '4.43e-5870419', 80, 1);
    T('8.5315360449939745989409199e-3862969805', '-2.553129944249935e-8432635966744', '8.53153604499397459894091989999999999999e-3862969805', 39, 1);
    T('-2.39879277833199772978892426436e-64931096208135', '7e+264910', '7e+264910', 84, 6);
    T('9.3e-42', '6.37943123324509721101236090959413027077755797881651369248e-4034228707', '9.3e-42', 70, 5);
    T('-2.9766961163328396890499e+958398', '6.573476987757468538819175232438605915135e+3428065428274', '6.573476987757468538819175e+3428065428274', 25, 3);
    T('-9.45464438962891689119825139300178494070423768515074480531329e+1569751402297', '4.111867935110570459011019942269019696190e-189565', '-9.4546443896289168911982513930017849407042377e+1569751402297', 44, 4);
    T('-3.7600801136772619431009702058262782668122174969e-15328609', '1.7661484982515921381554601786821985438537939558611086516283e-1646711999488', '-3.760080113677261943100970205826278266812217496899999999999999999999999999999999999999999999999e-15328609', 94, 1);
    T('-7.45014267508394791539e+100', '5.22081767204e-5555', '-7.45014267508394791539e+100', 30, 3);
    T('6.443066203492431e+1045969991879', '5.16256872750865281487380928598600999910305887894e-785968073', '6.4430662034924310000001e+1045969991879', 23, 0);
    T('-7.060337006432875141192125556656088346610623548e-44968594', '-7e+1186555', '-7e+1186555', 8, 5);
    T('2.3866543074232437096550e-93542153213215', '4.158548051e+11', '4.158548051e+11', 47, 1);
    T('-4.71945718337154022720180642278404969115020e-2176639410', '9.23e-63', '9.23e-63', 26, 2);
    T('3.34564355336574486184351e-1971842444', '8.68017325532563663379209995440158269958818598233e+3314', '8.7e+3314', 2, 2);
    T('1.9e-2698', '-5e-3567454', '1.9e-2698', 28, 0);
    T('8.5025530162199218015726957314054951709089912e+526941531334', '-8.39813772769068701872597e-83280', '8.5025530162199218015726957314054951709089912e+526941531334', 44, 2);
    T('8.777653306498586609025443730365388e+121', '46621340702252762401111390', '8.77765330649858661e+121', 18, 5);
    T('1.7043993590321252565015650569e+8393', '-7.5791392418187738879509e-60', '1.7043993590321252565015650569e+8393', 48, 5);
    T('-6.321169731063696810319816541449e+68693', '8.117697364859756e+300616442300930', '8.117697364859756e+300616442300930', 79, 6);
    T('-9.9750225368953871121261431342510289778680505748836390840228e-62465694216', '-8.713695919402367973571e-752002', '-8.713695919402367973571e-752002', 54, 4);
    T('9.48823495284368397708033534361e+5078646649955', '-4.5894241414224616942988e-543', '9e+5078646649955', 1, 1);
    T('-5.632184617717047360161812310786389276639309468527295151e-466', '-2.0664103531e+689198136', '-2e+689198136', 1, 1);
    T('6.08e+2651', '5.0033e-84994886345', '6.08e+2651', 48, 4);
    T('-4.712326746087169943962541452e+4160187', '1323551487240739687081111382493549233316383051341711472', '-4.71232674608716994e+4160187', 18, 1);
    T('-862136387841.428737', '1.65660723856021800334193635300460680953179442191416066135796e+91524702', '1.65660723856021800334193635300460680953179442191416066135796e+91524702', 62, 0);
    T('4.811339968941342878180904708046226969126853e+89653', '9.7938576909623194532659e-3744950246297', '4.81133996894134287818e+89653', 21, 3);
    T('-4.1161820419769295887396788118415783175868097894245e+455', '4.3787825796255501380732501e+92', '-4.1161820419769295887396788118415783175868097894245e+455', 58, 4);
    T('2.70332877649756061827338169412547e+512', '72932855946237852771805666384348431410138417376230244.376826', '2.703328776497561e+512', 16, 6);
    T('791844716511268137390839143912919300691849290584468079453', '684006306500713274355244132736.6392', '7.918447165112681373908391445969256071925625649397122121896392e+56', 100, 2);
    T('9.58326026e+15', '-4816349698115.928701842', '9.578443910301884071298158e+15', 66, 2);
    T('3.562563e-921625171214', '1.774527383662378694590420619781e+32', '1.774527383662378694590420619781e+32', 70, 4);
    T('5.284194582126443988e+36628565', '-3.5703295776106733507179e-5891969475206', '5.284194582126443988e+36628565', 79, 5);
    T('-6.7543e-469582544879', '2.05631384748e-36088999180', '2.05631384748e-36088999180', 86, 2);
    T('-8.86174393124572669e+731442918179911', '-9.66300007806570e+5081472748', '-8.86174393124572669e+731442918179911', 91, 5);
    T('-5.59358446590004424112175681885520296056166e+795222403', '-2.1475837363700511282425303468330596e+92', '-5.59358446590004424112175681885520296056166000000000000000000000000000000000000000000000000001e+795222403', 93, 0);
    T('-4.2e-7785228338027', '5.406485914e-2375322', '5.406485914e-2375322', 13, 5);
    T('1.8154009780259212277e+84298', '-7.68592273665562062809218129846714676815771811514493821e-605188297738238', '1.8154009780259212277e+84298', 43, 0);
    T('4.86969522483305052975020990676612e+35', '-8.71277012387660523095416928784933891400945706511649002591e-1121', '4.86969522483305052975020990676612e+35', 75, 2);
    T('332888076966911.8518', '-4.49775771710104144051074e-18803184930340', '3.32888077e+14', 10, 5);
    T('-7.6208526310525940e+22', '-6.3604278825085872810233647756992819004013049003258e-62', '-7.620852631052594e+22', 41, 4);
    T('-1.48441683064205107298862544583e+35', '2.97423983430773506885284484682303811e-9448', '-1.484416830642051072988625445829999999999999999999999999e+35', 55, 2);
    T('8.7251620962716282182158491325168272222902988749687571892580e+48324', '-8.73327951518621050056589095320952507e+45433099', '-8.73327951518621050056589095320952507e+45433099', 44, 3);
    T('2.9824821800229095121e-22900438', '3.851312144231145630529450087759854166473931e+505497788627', '3.8513121442311456305294500877598542e+505497788627', 35, 0);
    T('-5.2474185172e-15', '6.747776e+20', '6.74777599999e+20', 12, 1);
    T('-2.1267294590988356507817350e+398', '-5.2080342464471888960218734621712459309611616028959e+4888346', '-5.2080342464471888960218734621712459309611616028959e+4888346', 75, 4);
    T('-4.3947030021912820052327061193761e-8813481290', '1.4e+7306299', '1.399999999999999999999e+7306299', 22, 1);
    T('7.89e-846803', '4.0698095961194066079336645791814e+794165678', '4.07e+794165678', 4, 6);
    T('-3.061664268317781598990923384227703351289e-85456400098', '8.642617006335434723690e-2404035', '9e-2404035', 1, 4);
    T('-8.64194017605537500e+21', '65484757.7534', '-8.6419401760553095152422466e+21', 27, 3);
    T('3.9278164590411149031528170557719389e+41', '-4.6314807522804121225395947474226940827357655214716363584110e+2348', '-4.6314807522804121225396e+2348', 23, 3);
    T('-5.249e-28193', '7.3422797060043758754647368572224105131022180000369825843586e+814', '7.3422797060043758754647368572224105131022180000369825843586e+814', 64, 5);
    T('-3.935907628210018259502623540382679612235077e-730839661', '-6.38913321125270419457694421188962631712619962769431297966e+5889', '-6.3891332e+5889', 8, 1);
    T('-7.20725491317669915078892426003171059882780e-77562122', '-1.99400011093056759379981538191428353159082796841050e+56', '-1.9940001109305675937998153819142835315908279684105e+56', 98, 6);
    T('-4.09518134399859e-818475894', '-6.5316950197325474538860704873911831605192067935073623385559e+873', '-6e+873', 1, 1);
    T('-8.019783645634269148188139758590874e+29560406', '-0.6', '-8.019783645634269148188139758590874e+29560406', 100, 1);
    T('-1.8712911157082e-977503', '-1.0940123826465480115953303039054075669282482843626338e-183423989767851', '-1.8712911157082e-977503', 21, 2);
    T('-70.73255269134362793150750005304762810467468773109562149', '-2.608622999096511359257781e-899999794709', '-7.073255269134362793150750005304762810467468773109562149e+1', 70, 4);
    T('9.5863442237738313e-64934131', '-3.5733794456495654733684985899238300612615348911125244e-880595553480551', '9.5863442237738313e-64934131', 87, 2);
    T('-8e-385194962702877', '-5.372221310145400597192571080860247991448616461766e+1445', '-5.372221310145400597192571080860247991448e+1445', 40, 2);
    T('4.55505107125367481784088416658e+8958292934115', '2.29823397e+320441049', '4.55505107125367481784088416658e+8958292934115', 49, 4);
    T('-1.75752900601112757621035525060023174489331924701258819113791e+617409152491', '-2.73665214701150033739077301686231604185374868197997e+59', '-1.757529006011127576210355251e+617409152491', 28, 5);
    T('5.62866105274011e+2975712', '8.36551527090760077449410135358377e+55320437309585', '8.4e+55320437309585', 2, 0);
    T('0.00531', '6.8696638046805013669144927139600785e-49954684', '5.31e-3', 28, 4);
    T('-8.467979239863421377987722004910266219827009511e-6803053620', '-7.5494069908640201020469947589253115063126295e-77626341', '-7.5494069908640201020469947589253115063126295e-77626341', 49, 6);
    T('-6.001953317149163760627276430720018177488442e-9591667963355', '3.8465416341584270e-1356', '3.846541634158427e-1356', 35, 6);
    T('-6.243961786180208575e+28220424906969', '-3.75850076961780e-4338646638', '-6.243961786180208575e+28220424906969', 26, 6);
    T('7.7399754004342278932176945740866924e-622', '8.648164630770544e+63762', '8.7e+63762', 2, 0);
    T('-4.97945613391257210858320e+2777469', '2.73286195427753643137340e-4947701', '-4.979456133912572108583199999999999999999999999e+2777469', 46, 1);
    T('3.10038911872621655565396139942249533952391535e+3684', '6.80939533251386174259942770746085153992105039053e+53070', '6.80939533251386174259942770746085153992105039053e+53070', 67, 4);
    T('-7.7058274594301e+892394013790', '-1.9796435080312450758888531946960329299502329117382126e+86502445897916', '-1.9796435080312450758888531946961e+86502445897916', 32, 0);
    T('-6e+73908816864422', '-8.22911180083456e-3126430346', '-6e+73908816864422', 91, 1);
    T('-7.81396298246426271171e+1876141114', '-5.056e+6', '-7.8139629824642627117100000000000000000000000000000000000000000001e+1876141114', 65, 0);
    T('-2.58206387093545452880109845983803576473441389738508055105e-540981816367', '6.2699399303462125845113581329411437539385184353166e+91531385', '6.26993993034621258451135813294114375394e+91531385', 39, 4);
    T('-7.283770318779457735696203949344001234571258092902730e+9480725', '-7.972651832306453458226572328098547000940731134344925e-863170697', '-7.28e+9480725', 3, 4);
    T('3.550086475911e-302', '-6e-2396', '3.550086475911e-302', 88, 0);
    T('-99455271412987407118549876344656229504133.271384', '4.386202156874364e-19', '-9.94552714129874071185498763446562295041332713839999999999995613797843125636e+40', 95, 4);
    T('-4.8303761297073900927890194e+109', '-3.52961440889573752577002827e-7221080', '-4.830376129e+109', 10, 2);
    T('-7.093360985439287280216935e+5864815411334', '-9.3575785130596e-400363777211899', '-7.0933609e+5864815411334', 8, 2);
    T('2.48430364293934154964941480164793890567e-5098527218', '7.2752895249701921716093188e+490386', '7.2752895249701921716093188e+490386', 83, 5);
    T('-4.8313174244828700962586139844068210930839085893e+76732053786432', '-5382589.6485967', '-4.831317424482870096258613984406821093084e+76732053786432', 40, 3);
    T('7.3257721213096491992e+87', '-5.7978444703349169201083517667866681833240892667402658269000e-283478', '8e+87', 1, 2);
    T('4.1300759093395537806555619059746777363e-28175', '9.05407230956750820460083843782287074031947693389101e-8648727521953', '4.1300759093395537806555e-28175', 23, 3);
    T('7.39556691785764255997304112484690514759761061859277752e-2732810576019', '7.456601817846560e-535854', '7.45660181784656e-535854', 77, 3);
    T('-2.882490931101282234573027129578809946230088678046e+20895268', '-9.161054401679634582060140040111522831276329614734742923e+34289522463', '-9.16105440168e+34289522463', 12, 4);
    T('-7.79616453260012037621079951064236854736666715e+324679108', '1.214713243279135721712e+368', '-7.7961645326001203762107995106423685473666671499e+324679108', 47, 1);
    T('-9.7334323633741160851196736010143803853511e+31878076786', '-8.7777716482773655091609870672181015646107638807e+1004', '-9.733e+31878076786', 4, 2);
    T('4.3788954596415645676e+56243', '7895203920092447555457343438142721470.7472865', '4.3788954596415645676e+56243', 97, 1);
    T('4.5794857687801586226059935137470622775520756e+68532598899516', '-1.6067679186778065940858637337565178309575556871008572410475e+7315212988852', '4.57948576878015862260599351374706227755207559999999999999999999999e+68532598899516', 66, 3);
    T('-8.3533485051102835331173938267738630200566120448e+708877538426272', '-6.768339687029374704111570103e-4651', '-8.3533485051102835331173938267738630200566120448e+708877538426272', 49, 6);
    T('1.5695886446988853480141612285500519630950963316e+26741694', '7.22754785304772e-1464489202792', '1.5695886446988853480141612285500519630950963316e+26741694', 60, 4);
    T('3.2792133851952701447e-762902186849', '5.3434313804516084406702397077506574970711057311759284373e-8946138', '5.34343138045160844067023970775065749707110573117592843730000000000000000000000000000001e-8946138', 87, 2);
    T('-2.1e+348061511', '-2.914204562050328727522779504790150139123910331045e-971840582', '-2.1e+348061511', 8, 4);
    T('5.5467011675387e-737862339971370', '5.0909318386211354e-28489858', '5.0909318386211354000000000000000000000000000000001e-28489858', 50, 2);
    T('-9.28959798536112749819302e+26', '2.1133647890801e+4564059767640', '2.1133647890801e+4564059767640', 33, 5);
    T('8.74234e+60618', '3.81351e+348372494', '3.81351e+348372494', 95, 5);
    T('4.349704210336672e-883119421756', '5.565474466883710329698801848013358505265198976188289058e-817337363355546', '4.3e-883119421756', 2, 4);
    T('4.654111e+8948013795736', '-6.69626e+74663133', '4.6541109999999999999999999999999999999999999999999999999e+8948013795736', 56, 3);
    T('-9.33085129253206515015199335095888e+742640448012', '7.30691436e-1216036', '-9.33085129253206515015199335095888e+742640448012', 65, 0);
    T('6362958795998875380.6086', '3.0496168738354316909259446749745484161174e-3841', '6.36295879599887e+18', 15, 3);
    T('3.75615380294085357929885208596505950010643903267359e-77066626128835', '8.983087772573070376e-77078269', '8.983087772573070376e-77078269', 94, 1);
    T('-3.3273864634351895057042026261308464514e-707626037006', '-3.68429685582165341257484431e-615909523730', '-3.6842968559e-615909523730', 11, 0);
    T('1.24328352598071854285950452661e+604047', '3.3817879922067003197597074465103367926276056077e+9012744435484', '3.3817879922067003197597074465103367926276056077e+9012744435484', 50, 3);
    T('9.83154902901691942924e-83727954', '6.560887815292783814766102949369897271562e+72456990454', '6.560887815292783814766102949369897271562e+72456990454', 79, 1);
    T('3.566413839399571554991392525481041591044094e+77270498984', '7.7924821317547120052157577467920493661843612221160202748e+950221', '3.566413839399571554991392525481041591044094e+77270498984', 90, 1);
    T('946436402365777736339682328.8710553', '1.213141297470962621445943179734861527e-2703', '9.464364023657777363396823288710553e+26', 82, 1);
    T('8.91714436247656263847925e+8885', '9.819973656155863860260195629882485e-17166522', '8.9e+8885', 2, 1);
    T('-7.149308448832881430550242184762323e+81', '-6.3465142075669914e-28146382', '-7.149308448832881430550242184762323e+81', 42, 5);
    T('-6.0553260558623e-4457116291', '4.58730751634855662773e+8469', '4.58730751634855662773e+8469', 65, 4);
    T('1.95064988010440336947336e-371', '-8.4106337892330019447345262711774309905530e+492984', '-8.410633789233001944734526271177430990553e+492984', 99, 0);
    T('-2255781183.817297851597556606735604799', '-8.938680924738434092879857660156678065264186201072864246e-82873284', '-2.255781183817297851597556606735604799e+9', 92, 2);
    T('-6.292159587129829310509754066e+2006802441', '-6.969920457e+8184', '-6.292159587129829310509754066e+2006802441', 42, 2);
    T('4.98148539767876783802372594573993838640146e-74877844', '4.24778737936564852240097682681338112079359734470930292888e+611786449994', '4.2477873793656485224009768268133811207935973447093029288800000000000001e+611786449994', 71, 2);
    T('2.55427651830598635567251602410628966793572559e+407763353588', '6.734241152668961804370519606e+91', '2.554276518305986355672516024106289667935726e+407763353588', 43, 4);
    T('-9.3691346764127784004e+698331209995518', '9.16034150087527390395e+48791360', '-9.3691346764127784004e+698331209995518', 98, 5);
    T('1.5871087514227153226931503824225974487526463062165694e-48674', '-1.11987e+379318760068', '-1.1198699999999999999999999999999999999999e+379318760068', 41, 1);
    T('-6.694e+890533779010325', '-4.3385192e+700450', '-6.694e+890533779010325', 16, 2);
    T('-9.5495207373e-39045', '2.7679e-663995025028901', '-9.5495207373e-39045', 36, 0);
    T('1.894192159001419992543656935532679195743206440600930e-3893729796', '7.000269874930771105747e+41', '7.000269874930771105747e+41', 80, 3);
    T('4.732564866412857e-979611044', '-7.177052836276126946053183365557876503457963303564168e+6802472', '-7.17705283627612694605318336555787650345796330356417e+6802472', 51, 6);
    T('-3.32e+995032713', '-4.29433678525749602682682e-137555508', '-3.32e+995032713', 38, 6);
    T('-3.3428372976999261503286e+46251189107', '-5.684e-189018040', '-3.3428372976999262e+46251189107', 17, 4);
    T('-4.9692472100691819718133087282543e+377', '3.128461402285723706e+398299186878', '3.128461402285723706e+398299186878', 88, 0);
    T('3.5850e-747341008370303', '8e+858454429', '8e+858454429', 2, 5);
    T('-8.3172612862431e-2790002161', '9.793644162231137856693920801838e+253217965016762', '9.79364416223113785669392081e+253217965016762', 27, 2);
    T('-1.79258303143141435041048584351042961848561e+359', '-3.5033623071921471001694472e-2549655914477', '-1.792583031431414350410485843510429618485610000000000000000000000000001e+359', 70, 0);
    T('-275589.402154204', '8.893e-614876024', '-2.75589402154203999999999999999999999999999999999999999999999999999999999999999999e+5', 81, 1);
    T('5.2663930182252768607591076539288e+9365933951', '5.693e-113986782628293', '5.2663930182252768607591076539288000000000000000000000000000000000000000000000000000000000000001e+9365933951', 95, 0);
    T('-9.3677773252250601943982561759033278085617897130609e-819', '1.2761326009066920401165230335757369210776912536128431e+9834357', '1.2761326009066920401165230335757369210776912536128431e+9834357', 57, 5);
    T('-2.48335498168e+9442883819', '2.62e-39848900748021', '-2.48335498168e+9442883819', 60, 5);
    T('-3.48834591033944169009373511026226e+20794', '-9.78946821537792455539558045102703816935580514e-62962261661742', '-3.48834591033944169009373511026226e+20794', 98, 6);
    T('-5.4357857430975640861540217727e+948913', '2.2529851802622e+8561051511576', '2.25298518e+8561051511576', 10, 3);
    T('2.4990903613202547510e-4276970135388', '7.2153323534883520177433266e+431720', '7.215332353488352017743327e+431720', 25, 4);
    T('4.36221494852315592210437537696063931e-48', '9.4962915851723649710645087817937e+33', '9.4962915851723649710645e+33', 24, 3);
    T('-9.113651057791908476640724e+199549457601', '-8.8386504931554450696721810838284412057817063840529185615e-1918874887211', '-9.113651057791908476640724e+199549457601', 38, 5);
    T('3.546e+3427278873', '5.634667747452322177072386064052e+15981', '3.546e+3427278873', 26, 3);
    T('-6.2069009713490e+76884', '4.4733005720773e+4803786220', '4.4733005720773e+4803786220', 82, 0);
    T('-2.6459478095707161805373182480516e+3013859', '2.8928126716051936727e+35841', '-2.6459478096e+3013859', 11, 3);
    T('4.118149178293887633255329517969e-472522', '2.78767408159028364969450558456399941806901702070e-1162132981', '4.118149178293887633255329517969e-472522', 93, 4);
    T('-9.106277837244112666e+336', '-2.086572872644752336786766486320447297477472704888091e+3409', '-2.086572872644752336786766486320447297477472704888091000000000000000000000000000000000000000000000001e+3409', 100, 3);
    T('4.85870e-58', '3.148270058107784684886505264786331728978119270091407176167e-99940', '4.8587000000000000000000000000000000000000000000000000000000000000000000000000001e-58', 80, 2);
    T('3.92267937e+7801170', '9.985e+36335', '3.92267937000000000000000000000000000000001e+7801170', 42, 2);
    T('-8.91463283740509509498025007860410667125955701174201e+220815', '8.4931559318934560789315573347480152977087e-20', '-8.914632837405095094980250078604e+220815', 31, 2);
    T('-3.80921250602015373802882918780093356065724717e+5589178539388', '6.354059884394713367283702203716501564559023534596074368e-969329832', '-3.8092125060201537380288291878009335606572471699e+5589178539388', 47, 2);
    T('-3576918101729484681172.05', '3.3644555e+18', '-3.57355364622948468117205e+21', 100, 4);
    T('1.336376108280151012521969495976118e+131', '-7.0210184266346312348882e+2182151', '-7.0210184266346312348882e+2182151', 62, 6);
    T('8.5434754323e+109', '7.8945937800e-74246182348', '8.54348e+109', 6, 2);
    T('-7.05034019228461253e+7253', '5.46747769365843106597e-52191', '-7.05034019228461253e+7253', 20, 5);
    T('-1.73e-7494', '65584.5088191269', '6.55845088191269e+4', 49, 6);
    T('-2.6686560439836325657368639779282843487099e-88567756656074', '-1.227865166473569641538210391726181e+214', '-1.227865166473569641538210391726181e+214', 96, 6);
    T('9.29201e-19416259', '4180012905785160211964555202', '4.18001290578517e+27', 15, 2);
    T('-2.489152723802e+914705', '6.25965e+701200340', '6.25965e+701200340', 53, 6);
    T('-4.27849919686220887577613462766949671e-93607110298212', '8.916882903720e-76693994761', '8.91688290371999999999999999999999999999999999999999999999999999999999e-76693994761', 69, 1);
    T('-4.3780e+3365000', '4.282806508388615774921401831765e+8076979654', '4.2828065083886157749214018317649999999999999999999e+8076979654', 50, 1);
    T('-4.7912087562968608433874538e+21966088', '-3.2367533107228876314670e+659405180', '-3.236753310722887631467e+659405180', 35, 4);
    T('2.39910792683023320085800886859400009e+6022136', '6.17365583547983265e-9132142469679', '2.3e+6022136', 2, 1);
    T('-9.99816e+87633', '7.2421487080502388540256e-8631877824632', '-9.99815999999999999999999999999999999999999999999999999999999999999999999999999e+87633', 78, 2);
    T('2.89188922267307454848948e-90511321670', '-3.437420003411762953e+883535932120', '-3.437420003411762953e+883535932120', 23, 6);
    T('-9.020301413014404493132046517407363397e-3240', '9.7496090675963641019520724101283949064650399451591559e-841800339', '-9.020301413014404493132046517407363397e-3240', 97, 3);
    T('4.7080246138033699176201298562915998329274812177326e-3408902825', '-1.13143426831650369306676891063408332192917e+374484644', '-1.1314e+374484644', 5, 5);
    T('-3.8552855276767392414333134258010372578307345974983362e+27964', '-2.15523583930193412426616292734982179678734810156266752213e+23599407891808', '-2.15523583930193412427e+23599407891808', 21, 4);
    T('-3.59072143068682686574193465464528171033e+997611338898', '9.4130206e+33', '-3.590721430686826865741934654645281710329999999999e+997611338898', 49, 2);
    T('-8.47e+98841', '3.2e-43', '-8.4699999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999e+98841', 98, 2);
    T('1.95876177756e-42062216', '-1.81187748394e+642749909911596', '-1.81187748394e+642749909911596', 52, 6);
    T('4.823469285728e-613678', '7.20674918506200958063199e+876315401', '7.206749e+876315401', 7, 5);
    T('6.763620797634812512416629185797681869916630842e-69339', '2.38635929728858456039326394068e+779701993843192', '2.38635929728858456039326394068e+779701993843192', 51, 1);
    T('-7.8582261113739385886222256786742324674988369446374263e+95865569535', '8.702862761006290178723504291289819071287907e-13326152993831', '-7.8582261113739385886222256786742324674988e+95865569535', 41, 5);
    T('3.667180604156228488656661182181041493004e-7775284', '-7.4922e+612149406971746', '-7.49e+612149406971746', 3, 6);
    T('-8.351034909361794921497960e-19870', '3.30105313686110463277407023918387732246e-276', '3.30105313686110463277407023918387732246e-276', 68, 0);
    T('8.81833777594229890824910937e-850855778248291', '5.8511964785266489041566063698976449859529398183508044259e-275', '5.85119647852664890415660636989764498595e-275', 39, 3);
    T('-7.663392197170929733908970371134877386914214448e+568899', '7.38826185110998e+22307', '-7.663392197170929733908970371134877387e+568899', 37, 0);
    T('-2.737495836743021919254082057224378291161241931478265024e-60524757387178', '-2.5246326023365438085883695e+98700036885570', '-2.5246326023365438085883695000000000000000000000000000000000000000000000000000000000000000000001e+98700036885570', 95, 3);
    T('-1.57667312857161936689845373323334682226972e+296281485814698', '1.817315249252669194012778310899022432924636703716066e+53', '-1.57667312857161936689845373323334682226972e+296281485814698', 86, 5);
    T('7.58520650124700499741345374284961518694827181852229959918e+56818048614', '5.14016677020087473862287537914404446642996493203960218914246e+44684250420', '7.58520650124700499741345374284961518694827181852229959918000000000000000000001e+56818048614', 78, 2);
    T('-7.36787642918383567884800426030650378824865269049489e+51', '7.66903274816404012755937210748197459508709e+775696', '7.66903274816404012755937210748197459508709e+775696', 60, 6);
    T('6.5432824764986802484561981616033267920595947e+165436956', '-6.8256e+715124179370184', '-6.8256e+715124179370184', 20, 3);
    T('6.949383957754880757154e-147', '9.325111319479357814263809129663989602686653603e-276011940', '6.949383957754880757154e-147', 52, 3);
    T('-2.5958878170568446569056329182906233582530054316263816757e+3999156709719', '1.5750287471810608977939399825174045859182218783528325261e-193068', '-2.595887817056844656905632918290623e+3999156709719', 34, 6);
    T('-5.081554772482e-7836537', '-2.42e-2246510579', '-5.081554772482e-7836537', 47, 2);
    T('7.69325173806305375055336030e+460227010', '69674818164778313692836.050761279864243992264965417', '7.6932517380630537505533603e+460227010', 97, 6);
    T('-1.847121e+54386', '-2.52e-16489343', '-1.847121000000000000001e+54386', 22, 0);
    T('5.8692319209149755230812349135836739609369888e-23037154', '-9.365459009866e+299198466588', '-9.365459009866e+299198466588', 94, 5);
    T('-5.7808998359005152126818024688068584e-8780', '-1.42393867637735668465e-2674248270181', '-5.78089983590051521268180246880685840001e-8780', 39, 3);
    T('-5e-473753291916990', '-9.88495015350168040530035940e-6409195', '-9.88495015e-6409195', 9, 6);
    T('-7.67260864261884848570701643053832552127392392417793782e+31937743', '1.758e-86', '-7.67260864261884848570701643053832552127392392417793782e+31937743', 69, 4);
    T('-5767878480800306281145278639835.4677815', '-7.967755669502525110038191842100277260761835303486e-80031154420339', '-5.7678784808003062811452786398354677815e+30', 51, 6);
    T('8.4151904051227080899101889503123712882100233956482e+36548', '-3.88e+7471', '8.4151904051227080899101889503123712882100233956481999999999999999999999999999e+36548', 77, 1);
    T('8.3961096604384225769556597469943635225690662e-560180288130', '4.03622703507809005432666616754533626597409194217243e-307511', '4.03622703507809005432666616754533626597409194217243e-307511', 54, 4);
    T('-5.606460111686e+16', '-7.4559e+11', '-5.6065347e+16', 8, 0);
    T('4.6008092845606353e-9263', '-7.73689926867056631e+4131', '-7.73689926867056630999999999e+4131', 27, 1);
    T('5.156256690237293256848662485409622889698704066666939e+959282289', '1.92657380752847915845010433191495794046488659958857969734412e+67931047876', '1.92657380752847915845e+67931047876', 22, 1);
    T('-3e+2911142047412', '7.38e+6420261', '-3e+2911142047412', 41, 3);
    T('-4.65473551082215692674698434264667920444886374e+846989480770628', '6.62856995e-356197312255', '-4.6547355108222e+846989480770628', 14, 0);
    T('49571132979287676340672441253380978258950', '1.36561305199525982647334732811071647815326973441e+209752468484', '1.36561305199526e+209752468484', 16, 4);
    T('7.3842257e+3242304009852', '-5.6564595073527868758799471729696531084327e+48', '7.3842256999e+3242304009852', 11, 3);
    T('7.83e+474', '8.674916866939025927166673451453208652076552926833011e-824126827986', '7.83e+474', 20, 3);
    T('2.0749426299986180958553930492566307753532976023565742e-97566', '-5.4674359038002849234e+739', '-5.467435903801e+739', 13, 3);
    T('-7.239587910e-11235887340797', '1.978467320175373e+75674812832991', '1.978467320175373e+75674812832991', 31, 4);
    T('1.41657611705573251591826e+41', '1.2946951160892136028736033608935214366011388019e-26391441', '1.41657611705573251591826e+41', 34, 3);
    T('-9.899102861244054e+5576155055352', '-74714664769057995359412964650065480805232872190040.29090165', '-9.8991028612440540000000000000000000000000001e+5576155055352', 44, 0);
    T('-2.70406076628e-73947512', '2.462381863565813544061934232047840480098419201e-720254', '2.5e-720254', 2, 5);
    T('-2.94785616954123157995339643965293812e+269300988', '8.0864978371107572429596348e-962', '-2.9479e+269300988', 5, 4);
    T('9.605e+740', '-6.71727694921686379913640445371000692e-259718023', '9.6049999999999999999999999999999999999999999999e+740', 47, 3);
    T('8.65964941014099260902105188992e+7172401', '-7.709791014654511787368e-73', '8.65964941014e+7172401', 12, 5);
    T('-3.293497367975993137649599617893075220987398570688186430e+50089286393694', '3.529272934662058e+85', '-3.29349736797599313764959961789307522098739857068818643e+50089286393694', 77, 5);
    T('8.18826602538626438869358488382609e-258777621267', '-5.12834128890448062288334773641152517863199301109e+52038029145646', '-5.128341288904480622883347737e+52038029145646', 28, 3);
    T('9.1827687193363793742e+23', '-8.498566208397138251750438125032992312979056859617e-1153900134', '9.1827687193363793741999999999999999999999999999999999999999999999999e+23', 68, 3);
    T('2.080216875796771089034309931135004953384808303329980223e-2483288692', '808326585665530174382267644579245873147789.568', '8.083265856655301743822676445792458731477896e+41', 43, 6);
    T('7.121847693299690381507942e+94214529968467', '-4.2351987925195032403859791605451816057253038284218300138043e-62182', '7.12184769329969038150794199999999999999999999999999999999999999999e+94214529968467', 66, 1);
    T('3.9e+74', '8.887728110808572612315765858240e-88396052', '3.9000000000000000000000001e+74', 26, 2);
    T('5.01377139349241029058417538e-910', '-5.2586174362487662e+9749301478805', '-5.2586174362487662e+9749301478805', 69, 5);
    T('-7.65165274604910665316321123402909764704121332376e-99558', '-9.913880333340e+54', '-9.91388033334e+54', 69, 4);
    T('9.023875729995454140611438610348334880e+99931542555526', '-3.040012177834007547084307251399729175939449917227e+122', '9.023875729995454140611438610348334879999999999999999999999e+99931542555526', 58, 3);
    T('-3.2597280818251281e-124726897091210', '-1753192397035467757239.6936961', '-1.7531923970354677572396936961000000000000000000000000000000000000001e+21', 68, 0);
    T('-4.6559053956229065851741150e+27', '-7.2761396907709e-409226512801001', '-4.655905395622906585174115e+27', 89, 2);
    T('3.11012542e+2945934920725', '7.02e+3395820025093', '7.02e+3395820025093', 89, 5);
    T('6.65565133545867695791899819509811949144314521e+585504', '-5.876724342117149103913499994257158e+99498259', '-5.876724342117149103913499994257158e+99498259', 65, 0);
    T('-6.21086632795928893405367734822282622532680701242e+9632', '-7.94094e-2788908787266', '-6.21086632795928893405367734822282622532680701242e+9632', 80, 1);
    T('2.0776431143124782367107890137388768310422005e+19804811', '-4.1241e-43673533', '2.077643114312478236710789013738876831042200499999e+19804811', 49, 1);
    T('4.527562e+674486409888661', '-8.5691605334523e+385871775856231', '4.5275619999999999999999999999999999999999999e+674486409888661', 44, 1);
    T('-4.27353428494e-860041538', '7.634340683136209784471875e-934', '7.6343406831362097844718749999999999999999999999999999999999999999999999999e-934', 74, 3);
    T('-1.448849250151843155558313395197491042427163109e-399921', '-1.87990888532511067e-2747412628', '-1.448849250151843155558313395197491042427163109e-399921', 47, 2);
    T('3.8660129829e+45990022', '-3.79614211101133e-240399', '3.8660129829e+45990022', 22, 5);
    T('37309320857.46259', '-9.0639830343285e-258607', '4e+10', 1, 0);
    T('-223382332288094282634345610145.38149958', '4.6564033542e+6532818657016', '4.6564033542e+6532818657016', 75, 2);
    T('-2.89294884442509112363775541938138530086144622e-199920424471', '-1.09924501840724076556981355815892e-40013880286', '-1.09924501840724076556981355815892e-40013880286', 82, 2);
    T('3.92700310448369046137490528648e-3673', '1.79e-4898278', '3.9270031044836904613749052865e-3673', 29, 2);
    T('-1.592698865194404918267795868600657725548233e-50919053475829', '8.68526653303249076182e-768445101945652', '-1.592698865194404918267795868600657725548233e-50919053475829', 57, 5);
    T('3.7018419499594021536698861e-1494', '2.9943143763348567799298779832952193211967744336786584657e+82480416', '2.9943143763348567799298779832952193211967744336786584657e+82480416', 91, 6);
    T('-3.291997189438012e+635902877894974', '5.899201910931493436008112960e+84992648204', '-3.29199718943802e+635902877894974', 15, 3);
    T('-9.3255948e+9142943', '3.5974016807885793589531742043242904528810527e-94093350', '-9.3255948e+9142943', 32, 3);
    T('7.535674914767985e+5408393847523', '-6.96403575674825152297775329690416092642385227e-32048', '7.535674914767985e+5408393847523', 17, 4);
    T('-5.738061403907402e+45952030', '-5.65692125519968746e+243091504777330', '-5.656921e+243091504777330', 7, 5);
    T('5.0125291302363130864097951405235675640708223912011e-8508491', '4.097113130512235555332142594689675441911833678637091379589e+7139011364', '4.09711313051223555534e+7139011364', 21, 2);
    T('3.522962919595648983e-576552676341', '7.104e+171768793086184', '7.1040000000000000000000000000000000000000000000000000000000000000000000000001e+171768793086184', 77, 0);
    T('6.743541657002332e+34932594', '-3.684402182527866905002574102433345555070666557e+27093', '6.74354165700233199999999999999999999999999999999999999999999999999999e+34932594', 69, 3);
    T('-2.72332185069435900925e+53122664002508', '8.451070184779847287382636841e-808278627824730', '-2.723321850694359009249999999999999999999999999999999999999999999999e+53122664002508', 67, 2);
    T('9.4184576936378747037468943335732e-586393253', '9.1557972574801062819953400373756067621929e+206384961474', '9.1557972574801062819953400373756067621929e+206384961474', 95, 5);
    T('-7.20003267923459695916026143e-381', '5.2040475113632828585306872654664442e+848484426611887', '5.2040475113632828585306872654664442e+848484426611887', 52, 4);
    T('-7.4414774103029427655049656559279352150323028e+537288244810', '7.0e-19079908057516', '-7.4414774103029427655049656559279352150323028e+537288244810', 61, 3);
    T('-6.60874561832940537862105841834227e-3189611675', '5.1910153e-5375', '5.1910153e-5375', 24, 4);
    T('-6.6098892843268e+771951337', '-7.4954995655576e-1883281', '-7e+771951337', 1, 5);
    T('3.84015793764166135133047689207675509568049215164378e+911093', '-7.48477534935959841206341710463806660937e-422751115010464', '3.84015793764166135134e+911093', 21, 2);
    T('-6.90102207788583166123307985294523e-422', '-5.4897054725568601e-9965602', '-6.90102207788583166123307985294523e-422', 45, 1);
    T('-3.574604308881167e-45663974743768', '-8.51201942774414217277176609494451992208969993630478765000131e+59128735', '-8.51201942774414217277176609494451992208969993630478765000131e+59128735', 66, 6);
    T('-8.136552855497852280092103218862889556e-2712824569820', '-8.2336208901434741661021822239985903193604e+9265067796', '-8.233620890143474166102e+9265067796', 22, 4);
    T('9.8802183855621938439124e+198513519', '7.4009530e-30899', '9e+198513519', 1, 3);
    T('-9.71635836812e-17', '4.82271e-8293', '-9.7164e-17', 5, 6);
    T('-8.6359601938932e+9863778676091', '-3.55459287479232273110899405740677914001413936012641788984e-431886904478197', '-8.6359601938932e+9863778676091', 79, 6);
    T('3.39656642163455745703611986686392198e-421', '8.182115e-605912016392534', '3.39656642163455745703611986686392198e-421', 96, 6);
    T('9.364206965215611609025743503538e+34', '-8.0433285002999423572726034234802624e-4103419308930', '9.364206965215612e+34', 16, 6);
    T('9.8431540320307053274369531643182744977334254696e-80594', '-3.50349569297230973788252064407e-6885453', '9.8431540320307053274369531643182744977334254696e-80594', 53, 4);
    T('-1.456888409601623109151829768209878209978400678372e-4837000', '-5.12654032483127093464746662496641937568579591428893e-509645180223190', '-1.456888409601623109151829768209878209978400678372e-4837000', 99, 2);
    T('-1.3830819186266184729864300e+52177570090', '-9.83116839186703100e+167396', '-1e+52177570090', 1, 6);
    T('3.585219204378221778591878515891906e-305152', '2.679512725515e-95200953', '3.5852192043782217785918785158919060000000000000000000000000000000000000000000000000000001e-305152', 89, 0);
    T('8.83662423028909774986302031666086354266940392545051e+56', '4.37862779314481991867392768753111655534498511549e+7698', '4.378627793144819918673927687531116555344985115490000000000000000000000000001e+7698', 76, 0);
    T('-1.61118745511e+446186086422833', '-6.3633501666934529160187e+6416552', '-1.61118745511e+446186086422833', 20, 6);
    T('6.80326187094201493075674366058502594641618923e-367634214', '-2.976722793811895905e+48891', '-2.976723e+48891', 7, 6);
    T('5.86467720102335994658159467997237450481862722093494e+9632383375335', '2.5218693995e+89996723013098', '2.5218693995e+89996723013098', 50, 1);
    T('-4.853e+8', '7.20351418e+71828922550', '7.20351418e+71828922550', 86, 6);
    T('8.42064979460227159062911491435243920350655574922e-9448819400809', '3.66511010336478195230847e+50173652571764', '3.66511010336e+50173652571764', 12, 3);
    T('-2.713723166609735652865220375370504502e+388233', '2.781956602778e+4572226570131', '2.781956602778e+4572226570131', 36, 6);
    T('3.212214899373e-518', '-2.39996569074868538135402752574e+54722307834', '-2.399965690748e+54722307834', 13, 1);
    T('1.38291474655387575917238795013394e+37583', '3.865463663614636307503853800122331862007898946039296e+95', '1.3829147465538757592e+37583', 20, 0);
    T('7.820180181726092e-828199', '-4.04315589659772388179509647706509805805018761190804881e-693960945', '7.820180181726092e-828199', 53, 0);
    T('-9.02070e+4697433357', '5.4399816829646095517614646835656723295928729369e+9844105411', '5.439981683e+9844105411', 10, 5);
    T('2.012025184589072714171e-9728805662494', '-9.89932855849567252625952367552989364881473654979e-4614', '-9.8993285584956725262595236755298936488147365497899999999999999999999999999999999999e-4614', 83, 2);
    T('7.74326213825474601734086757815710551719386716936521926012e+3717', '8.8777123232241908105041795134e-89932862', '7.7432621382547460173408675781571055e+3717', 35, 3);
    T('3.3172505940504273492405085831784e+254', '-7.7416761842470746170637738379582923284076237722865e+259225067768', '-7.741676184247074617063773837958292328e+259225067768', 37, 4);
    T('-9.4046096571540487e+1767', '3.36709585745e-64619623', '-9.40460965715404869999999999999999999999e+1767', 39, 2);
    T('7.51703360059340382177524338e-2690227294', '2.959994209443120e+665094067', '2.95999420944312e+665094067', 71, 5);
    T('-7.165242468871688718058182e-2677114', '9.0152008548480951040150453338e+15380', '9.0152008548480951040150453338e+15380', 48, 5);
    T('6.659080469236902567114411032175889571165e-9143438922', '-5.056301530e-6002527582', '-5.056301529999999999999999999999999999999999999999999999999999999999999999999999999e-6002527582', 82, 1);
    T('-9.37168252e+633780941214136', '-6.44384651244120356318416843487528e-10', '-9.37e+633780941214136', 3, 1);
    T('-4.4574264089091335145204962e+72902464478761', '-1.768701515196203792973291474240426769392752529424886e-936540023', '-4.4574264089091335145204962e+72902464478761', 65, 6);
    T('8.22001855750266694661647426593e-62030803192368', '4.4349e-4140773709243', '4.4349e-4140773709243', 37, 1);
    T('-7.2863159443859270918029404316625549e+118', '5.47556204289449378e-42', '-7.2863159443859270918029404316625549e+118', 64, 0);
    T('3.2985426486665746099979703e+6333408594', '-5245794287.946440', '3.2985426486665746099979703e+6333408594', 81, 6);
    T('8.160529027186781847146141600813760e+6508761544', '-4.1699399e-7578', '8.160529027e+6508761544', 10, 1);
    T('-9e+6155655573', '-7.93504e+670780007209860', '-7.93504e+670780007209860', 67, 2);
    T('-3.06e+979385', '2.7001109735355735579729471956014e-2043719', '-3.06e+979385', 59, 4);
    T('-7.46e+15407302601478', '-2.6436440604e+329876821', '-7.46e+15407302601478', 21, 5);
    T('5.9235707001879196852962939562751158912581519448094e-435936348854', '6.299543358468035686245531286264850928316354431021412e-1896924', '6.299543358468035686245531286264850928317e-1896924', 40, 0);
    T('-7.051516892294491641082665565889225468885294348e-94748671545735', '5.59053344074465463290404907347622771207513345034268863097819e+46857852299668', '5.59053344074465463290404907348e+46857852299668', 30, 2);
    T('-3.49177094281807596358030928924295000190e-7614433', '-5.7306908598e-429540983396758', '-3.4917709428180759635803092892429500019e-7614433', 72, 2);
    T('-3.586874829706423663683976540394498e+907615042166921', '6.42816535103759353891798654894370804527523002416969193949469e-9122656178', '-3.58687482970643e+907615042166921', 15, 3);
    T('-7.50872259864192726741844827209874628111242314943474e+85617967434', '-4.3595389326495313634922788420166275093580754e+80724765364', '-7.5087225986419272674184482720987462811124231494347400000000000000000000001e+85617967434', 74, 0);
    T('-1.129282636129293901021231845228656752742306957878641e+556', '7.921241017965496409e-537016067143', '-1.129282636129293901021231845228656752742306e+556', 43, 2);
    T('-6.02186342282217e-85571707962237', '-6.72654824062715306023027841917754546225909022248251382970409e-3402270640', '-6.7265482406271530602302784191775454622590902224825138297041e-3402270640', 59, 0);
    T('6.311670081126923910574433396e+55830559', '7.8332385143563054395894135887233006347930324566e+89992393', '7.833238514356305439589e+89992393', 22, 1);
    T('8.4133446636510592491221070650646255230500223229e-6502311904165', '-4.244873683591442174561893161677443656496889434918221e-97837716', '-4.2448736835914421745618931616774436564968894349e-97837716', 47, 6);
    T('9.647734674006392003058866273765388779800759403889282058e-8962726375', '-2.0113795540174715258991258744168602115435e+41964494722', '-2.0113795540174715258991e+41964494722', 23, 1);
    T('-3.3758769828303662134040087003836304253262390725685491e+7447', '-8.899434154e+8324279', '-8.899434154e+8324279', 96, 1);
    T('-4.7545987e-7826566', '-1.951024291588558696269554493e+6135001373', '-1.951024291588558696269554493e+6135001373', 56, 5);
    T('7.1782994618095001717003466015974290e+299', '-5.0734252340007588071746896195485453043340927295336502793078e-583030637', '7.17829946180950017e+299', 18, 5);
    T('-1.680467116186227975204314922e+2705', '8.8731767622549349437499482687982e-4534123', '-1.680467116186227975204314922e+2705', 95, 4);
    T('-4.46315125077257086e-8827136', '1.4556169955717842249954191038065e-427809144690', '-4.46315125077257086e-8827136', 60, 3);
    T('8.094896912712732946256517057569440271290411e-705304325877', '6.1272081e+799158355344', '6.1272081e+799158355344', 56, 6);
    T('4.60907258162373426626555568631e+99358', '5.8098167631e-2816327713', '4.60907258162373426626555568631e+99358', 71, 5);
    T('9.22263626275119778e-44303301878', '-147945678224287266761989296073.9710571', '-1.47945678224287266761989e+29', 24, 6);
    T('8.8258924867622100746635440785842593682003503836940e+702578', '7.635954683493694467649205984388665396321712692741e+86157070049', '7.635954683493694467649205984388665396321712692741e+86157070049', 55, 5);
    T('-744010737581835262408654280478935218869948', '8.658059910774643683449253383e-770946264950342', '-7.44010737581835262408654280478935219e+41', 36, 3);
    T('-645267318746477.996225308283450575067026877417132570', '-1.729476256233656271647152241449241518369e+64209', '-1.72947625623365627165e+64209', 21, 0);
    T('-1.893714296622487723e-544553245230', '-4.357505884561977364740305e-65371638548', '-4.357505884561977364740305e-65371638548', 80, 4);
    T('9.99452666793714839331977449803475352e-353853', '-1.047399036058049575924523485428436166057075679110526286', '-1.047e+0', 4, 5);
    T('4.3751660099275363168070565465023977395177763286093e+944174553432201', '4.20902004581268842e+534981', '4.375166e+944174553432201', 8, 6);
    T('-9.1962319048674510781353779651708954182e+25661673163', '-7.6196274165751308e-472903106', '-9.196231904867451078135377965170895418200000000000000000000000000001e+25661673163', 67, 3);
    T('-8.766613453516444966378614441466e+97457934871416', '-4.953089835961803e+111', '-8.766613453516444966378614441466e+97457934871416', 44, 4);
    T('-748134207.11281185199157040024', '6.62469534e+744', '6.62469534e+744', 45, 0);
    T('-4.903351310807643912530095631341749886828597488919546871166e+652395359', '3.9337827565556557604440579485451168617587873e-8536972273', '-4.903351310807643912530095631341749886828597488919546871166e+652395359', 87, 4);
    T('-5.17648503505746577611010941138034e+45093595657', '-3.36849697022714531041595350313577639277230069e-39860243', '-5.17648503505746577611010941138034e+45093595657', 60, 2);
    T('1.82085719961677601e-677179620989454', '-3.03306596796916013863174e-45', '-3.03306596796916013863174e-45', 49, 6);
    T('1.962282478811934808073757537894689002691583e-227558729409941', '5.6060609614958836490527701687235820767340465862e-21623758', '5.6060609614958836490527701687235820767340465862e-21623758', 76, 1);
    T('8.673715050960567382632218321e-7023426', '5.064143535906442925651824946226737559061e-93751022541438', '8.673715050960567382632218321e-7023426', 61, 4);
    T('-3.0571e+30', '3.9396638397121204948448052164e+27639541', '3.9396638397121204948448052164e+27639541', 36, 4);
    T('6.706918012052499468590979e-9676', '-2.77163125850942822105575298599733538394646554173662708e-2312', '-2.77163125850942822105575298599733538394646554173662708e-2312', 78, 3);
    T('8.6727888001135134866763450792583263113392634e+5581', '987798942195276872809853282731599.7759832661', '8.6727888001135134866763450792583263113392634e+5581', 94, 5);
    T('9.2083643471281609e+154876686945', '3e+58', '9.2083643471281609000000000000000000000001e+154876686945', 41, 2);
    T('-2.460526565233263996546505293240923578387970e-1142', '5.2719581283588039314933302556556724372128e+85539945', '5.2719581283588039314933302556556724372128e+85539945', 80, 2);
    T('-6.53833303809441e-73650', '-3.61283837482730568787583979846e+31', '-3.61283837482730568787583979846e+31', 72, 2);
    T('3.313032748368436091824064378151e+6761721766309', '4.5190975992911622355006e-202577544946461', '3.313032748368e+6761721766309', 13, 3);
    T('5.556700356816863e-234360967957527', '-2.3511029393950995908737269308310874413e+864475581229290', '-2.3511029393950996e+864475581229290', 17, 5);
    T('9.81134760074991448572907e+56546790', '-9.1970845968033707594869517430880190575e-92520', '9.81134760074991448572907e+56546790', 87, 0);
    T('-4.0475744227137734e+2023977', '-5.5331585954889255397349926811070621562947810e-40377', '-4.0475744227137734e+2023977', 33, 1);
    T('-8.6678115738039585134317653349902421875189281e+3549193415', '4.26977705670822639905922552110274081672858610770130335e+6167501', '-8.66781157380395851343176533499024218751892809999999999999999999999999999999999999999e+3549193415', 84, 1);
    T('4.480779690393889150032713e+4165722851', '-5.049746239110864e-2327', '4.48077969039388916e+4165722851', 18, 0);
    T('-3.2922939619854574176370e-46160826226350', '7.7772502661601866008363676337528562003765495843e-750733524239', '7.78e-750733524239', 3, 6);
    T('5.752385479047310203773260586049281053035816e+36502674792175', '1.60907895389242957837815016358709593263496739627549573e+932', '5.752385479047310203773260586049281053035816e+36502674792175', 73, 5);
    T('2.5e+340568919423', '1.9022548981092624131243676e+775760764', '2.500000000000000000000000000000000000001e+340568919423', 40, 2);
    T('9.21505668964888198953739e-62689114472', '4.1003779818728675e+717995622985', '4.1003779818728675e+717995622985', 26, 3);
    T('7.3991005821696e+62038539', '-7.945111296731600351628828e+33', '7.399100582169599999999999999999999999999e+62038539', 40, 1);
    T('4.893904041451617326436668027960374684616175e-106191166611', '-7.1825796384772071243595229204495980e+1244121617356', '-7.1825796384772071243595229204495979999999999999999999999999e+1244121617356', 59, 2);
    T('4.004559769337310550610468611058883854383635659735928926e-684', '0.000075180626927333047472167054849915945949', '7.51806269273330474721670548499159459490000000000000000000000000000000000000000000001e-5', 84, 2);
    T('-1.981911170799048699895427970557282687e-14025545', '-5.21458051062e-5597169', '-5.215e-5597169', 4, 5);
    T('8.324730383691218933860911410366628749883e+5443524220324', '-5.97e-377630', '8.325e+5443524220324', 4, 4);
    T('-6.36937580510372274772589255753914064392109e+266669712148', '-7.9388671e-7292', '-6.36937580510372274772589255753914064392109e+266669712148', 49, 5);
    T('-9.85887772851305350389537110285847553890577167e-143', '-8.272e-69', '-8.27200000000000000000000000000000000000000000000000000000000000000000000009858877728513e-69', 87, 6);
    T('-9.59153541095841910134066180e+94617', '9.19904820810187484837244769195213188e+39270732602938', '9.19904820810187484837244769195213188e+39270732602938', 56, 2);
    T('-2.274320824756e-9', '2.4188763504160423131865707780e-591', '-2.274320824756e-9', 21, 6);
    T('-7.476843697856125515298708058495064874153e+37087871203778', '6.564177240682310821026623097453978406656676488176215358e-5112361533235', '-7.476843697856125515298708058495064874153e+37087871203778', 89, 3);
    T('-7e+84000389999', '-3.3347e-4276388096', '-7e+84000389999', 97, 2);
    T('2.09175153393e+76860', '-6.19185789150354012893520656177812392963203065558441808683524e+369', '2.09175153393e+76860', 50, 2);
    T('7.65039442534843413769484106396722e+8061525', '-2.4851017901136e+637304', '7.6503944253484341376948410639672199999999999e+8061525', 44, 3);
    T('-6.58695780989785039102326359739982590821e+5272602054334', '-3.1636632410062672925409305764780306002504454468264688434679e-83453213', '-6.586957809897850391023263597399825908210000000000000000000001e+5272602054334', 61, 3);
    T('2.20805034442765932476e-381908', '5.93091084598565847951492445509241197101459711796692911e-586954515312', '2.20805034442765932476e-381908', 75, 6);
    T('9.10223847610545445356420786420323614896683906502157e-97542349239', '4.3540229903e+102175567295299', '4.3540229903e+102175567295299', 36, 5);
    T('3.13206814224862723798238724109328378650183086e+20014', '1.93362472877564445000389646329456319779438704924907013511379e+128', '3.13206814224862723798238724109328378650183086000000000000000000001e+20014', 66, 2);
    T('-6.9952016287073265165792261175173238602763531630255e+1987882075625', '4.669809603740759896627746824762931807e+550030461115044', '4.669809603740759896627746824762931807e+550030461115044', 40, 2);
    T('-7.61515047e-649402', '-7.812359823361126444872047064282980888747194406123046661e-291278014936509', '-7.61515047e-649402', 32, 5);
    T('6.707608475848923746e-576', '1.11964435067e+100178157248', '1.11964435067e+100178157248', 46, 1);
    T('-8.65910607267121559477133553494894653074351769239321921e+136', '-5.959e-352192107761532', '-8.659106072671215594771335e+136', 25, 1);
    T('4.4640961936287652561982150900669621383676061e-54856911', '3.0e+307210', '3e+307210', 68, 6);
    T('-9.638998767469606656307135377656564084104509085161e+4073858788', '259673465952532.390779', '-9.638998767469606656307135377656564084104509085161e+4073858788', 89, 5);
    T('9.34945985e+547592629909790', '-284548657879283340858559213840360689065.018715', '9.3494598499999999999999999999999999999999999e+547592629909790', 44, 1);
    T('-9.354084201860305906328e-35003', '-8.76212264323544e+614645667585404', '-9e+614645667585404', 1, 3);
    T('-3.369128499645518173379018232192767759821725263470e-4109070481', '-7.637431240496864582962919199812484669847043766515192e-2746188987003', '-3.36912849964551817337901823219276775982172526347000000000000000001e-4109070481', 66, 3);
    T('3.23473147488619839507027425378025306597e+66577422112677', '-3.4e+9', '3.23473147488619839507027425378025306597e+66577422112677', 45, 2);
    T('4.755e-385177469', '-7.54479041427795054443864333876399575608328481577946801667711e+42342', '-7.54479041427795054443864333876399575608328481577946801667711e+42342', 81, 3);
    T('7.73561e-1607', '-2.8169022135517908616217961141147994900245729e+59685729544293', '-2.8169022135517908616217961141147994900245729e+59685729544293', 61, 5);
    T('8.9676e-99925164', '-7.86965527683214e+17', '-7.86965527683214e+17', 91, 5);
    T('-3.26811557258e+2131', '-6.9530589e+23177', '-6.9530589e+23177', 66, 5);
    T('-1.327e-29706702', '-1.331925229467e+66', '-1.331925229467e+66', 96, 2);
    T('-7.85693395944591694267132499970298301736678e+3070350', '7.976897437906834141978983e+5358803865639', '7.976897437906834141978982999e+5358803865639', 28, 3);
    T('-6.46273801629256636295194991869e-204579', '8.07680524234201197077875885660683042639384733082576911112e-18', '8.07680524234201197077875885660683042639384733082576911112e-18', 65, 2);
    T('-8.8885399937003683626376e+636072992432', '-9.458042007989680468993030684756541834203292e-86621391180', '-8.88854e+636072992432', 8, 0);
    T('-1.059146519751844192e-9490395', '1.09e+4533345903381', '1.0899999999999999999999999999e+4533345903381', 29, 3);
    T('2.16054807110786358463844433431571633387787189195288108e+828305865814', '-8.08449858224e-6860', '2.160548071107863584638444334315716333e+828305865814', 37, 1);
    T('8.28797752818263e+2286922672632', '9.39279211193028385244000783e-43303', '8.28797752818263e+2286922672632', 66, 3);
    T('-7.541094737952744932522e+94184', '6.4844794027615730021786e+93505841', '6.4844794027615730021786e+93505841', 41, 4);
    T('-8769311282882172300670763', '3.85080445900648249172482461e+3304188890', '3.85080445900648249172482460999999999999999999999999999999e+3304188890', 57, 1);
    T('-1.929524740654e-1715', '0.00523', '5.2299999999999999999999999999999999999999999999999999999e-3', 56, 3);
    T('-3.654884494315284248931e-65105829909', '-1733162040387073592917.3', '-1.733162040387073592917300000000000000000000000000000000000000000000000000000000000000000000001e+21', 94, 3);
    T('9.2224597788334591776446537444999277e-73268271', '4.06592694790856771650626181761434100407e+28763825', '4.06592694790856771650626181761434100407e+28763825', 85, 6);
    T('-2.6521693942363340354452e+63', '7.74762115593711814e-4336239', '-2.6521693942363340354451999999999999999999999999999999999999999e+63', 62, 2);
    T('-84510403144217185127602524', '-6.323543702049311155065452174767519214320846428125730e+8485964665', '-6.324e+8485964665', 4, 6);
    T('4.79472154599956352918420296535616411083e-912503134301953', '9.922467217023823e+4883276', '9.922467217023823e+4883276', 76, 5);
    T('4.61020745258e-9449401', '5.15063e+61', '5.15063e+61', 51, 1);
    T('6.9870659726321054994181692177199916382845933173747437e-31', '-7.16354e+681248583861425', '-7.16353999999999999999999999999999999999999999999999999999999999999999999e+681248583861425', 72, 2);
    T('-9760703549278409.77422288', '3.871879377329036595528937e-73', '-9.76070354927840977422287999999999e+15', 33, 1);
    T('-1.449729233597681773266362656993862065570035562317397429e-62877931128187', '-9.585983573088177191988087402024670e-407192609517', '-9.5859835730881771919880874020246700000000000000000000000000000000000000000000001e-407192609517', 80, 0);
    T('-9.74046149654054030013e+8009482840994', '9.5924688742426993153539573048038019280e+9074675683785', '9.59246887424269931535e+9074675683785', 21, 4);
    T('-6.96869251012654938199288833963e-607756', '7.3626945218482e+710031002', '7.3626945218482e+710031002', 21, 2);
    T('-9.75632229786816020388e-96667', '-2.93078690385341167407082530449649e+473684', '-2.93078690385341167407082530449649e+473684', 61, 6);
    T('-6e-16', '6.06885423261573924748460930600360263983569842885118783143634e+536021904692', '6.06885423261573924748460930600360263983569842885118783143634e+536021904692', 95, 6);
    T('5.3749898153861356445668219011462642185e+39292921861', '7.001e+420926', '5.3749898153861356445668219011462642185000000000000000000000000000000000000000001e+39292921861', 80, 2);
    T('6.094e-23582', '2.466907270057347589168376673899745320137651290e-1809924939', '6.094e-23582', 15, 6);
    T('-7e+7334', '-9.31313952525085729358612044462475409280712733378e-6968', '-7e+7334', 20, 2);
    T('3.38282e-432', '6.19395527979936883907606850424162474355556394831149238977e-1248360463067', '3.38e-432', 3, 4);
    T('-2.25471e+835568945268894', '7.330821505366187e+93115022454', '-2.254709999999999999999999999999999999999999999999e+835568945268894', 49, 1);
    T('2.152336420994085e+44086309', '8.825793e-740412122865', '2.152336420994085000000000000000000000000000000000000000000000000000000000001e+44086309', 76, 2);
    T('-2.07131620493785489668713790e-68497666', '-9.3417721e-32841591502523', '-2.0713162049378548966871379e-68497666', 40, 2);
    T('1.4e+466858', '7.0186632249760769325291178932409971734145343280189572456223e+24646329', '7.0186632249760769325291178932409971734145343280189572456223e+24646329', 92, 5);
    T('9.9e-59557260017', '-7.1955e+812342325647878', '-7.1955e+812342325647878', 6, 3);
    T('-2.06648e-26308764', '9.27e-9025210204', '-2.066479999999999999999999999999999999999999e-26308764', 43, 2);
    T('-5.0388420102595003995813396865597631078396723058e+54', '-7.5020362e-869428554901409', '-5.0388420102595003995813396865597631078396723058e+54', 82, 6);
    T('1.761548424348350618e+262130', '7.717230933896781084127504215910080794676101938699368140202e+26152759', '7.71723093e+26152759', 9, 4);
    T('8701971173126.4', '-4.21402824710544667800880498256718751170251310e-250064977', '8.7019711731264e+12', 87, 4);
    T('-3.2912254080800207236780339478587174e-59535355049', '-6394497427233248028664672692.710807', '-6.394497427233248028664672692710807e+27', 96, 5);
    T('2.628124043709979306212678697012539536812751355634e-6317108', '4.4650378203170984662481687975e+4040382053946', '4.465e+4040382053946', 4, 6);
    T('9.8295184891999362e-64573658181204', '7.4731094711088830459690871491696495796e+44', '7.47310947110888305e+44', 18, 4);
    T('-2.7631702718848925669870840821e+3231', '-7.405925449e-777270468401', '-2.76317027188489256698708408210000000000000000000000000000000000001e+3231', 66, 0);
    T('-2.77948416392993136225180745396430e+44208', '-6.7e-467', '-2.7794841639299313622518074539643e+44208', 58, 5);
    T('2.75224750977834149081478488578805099475755521880628796020e+814679', '4.70589551224576060304e+171', '2.75224750977834149081478e+814679', 24, 3);
    T('-4.2889217222931476495786858418236963493e+354996433522', '-9.833464640841717604175313262819265723252123881703103317e+271365011565148', '-9.833464640841717604175313262819265723252123881703103317000000000000000000000000001e+271365011565148', 82, 0);
    T('-7.145425663597821548982e-33097316153', '5.61289751967250286e+1019122803339', '5.6128975196725028599999999999999e+1019122803339', 32, 3);
    T('-4.4521658094329213366001852989373626375573462543319330e+3166', '-4.13259692343130635168369046184e-65', '-4.45216580943292133660018529893736263755734625433193300000000000000000000000000001e+3166', 81, 0);
    T('-1.95250002642681819328267928726262691994e+571744913', '7.42148182165033134718814029541e-4075848927051', '-1.95250002642681819328267928726262691993999999999e+571744913', 48, 2);
    T('-1.002719003595307e-18056402051777', '-5.82940142041038932420741962167438770836397e+22634138388372', '-5.82940142041038932420741962167e+22634138388372', 30, 1);
    T('-8.99623358247622776273443641860189471461584106094597348e+550976042729', '-5.9281995227298002208863736077283245733664277e-951', '-8.99623358247622776273443641860189471461584106094597348e+550976042729', 62, 6);
    T('-5.035229e-43841964', '5.13604148908092222814753088096355921502460562088712205963442e-36528698', '5.13604148908092222814753088096355921502460562088712205963442e-36528698', 82, 4);
    T('-6.1462182205538263411814016656716726e+38402540001', '5.99879706036657882228656648227576962413470002406925958474617e-8397916543975', '-6.1462182205538263411814016656716726e+38402540001', 39, 5);
    T('6.67394346803348934e+39', '4.40286222891492486895007737337019552269428114469e-30111269', '6.67394346803348934e+39', 79, 4);
    T('1.962e+653845940', '-8.7459953564247492017631459077497976569358527305352e-802629', '1.962e+653845940', 4, 2);
    T('5.034634177846951503632889780533655177e+1826698', '5.82611369556081889183888007967032828047943886498354e+233371632143234', '5.826113695560818891838880079670328280479438865e+233371632143234', 47, 4);
    T('-5.59664700591391224135880700555146763154775739431714125080e-67385190', '-5.73282e+522104610', '-5.73282e+522104610', 57, 6);
    T('3.4318226344131884398e-918598796', '6.175940e-630', '6.17594e-630', 15, 1);
    T('12631.708881', '-3e+43885447510', '-3e+43885447510', 65, 4);
    T('-6.6287645715e+15', '8.992095018661952825839254262636781958321573584e+176', '8.99209501866195282583925426263678195832157358399999999999999999999999999999e+176', 75, 1);
    T('9.111874487770827298e-228831', '-7.4180900994874661589809263e-75793', '-7.4180900994874661589809263e-75793', 49, 5);
    T('-866121361146269602347371850394631774130.7', '8.9511488241391083511439771117839134421976419e-7954555', '-8.661213611462696023473718503946317741307e+38', 45, 3);
    T('-6.45630067514080656494207114719120e+9823772895', '-6.12957154027477436947948018533434e-5620639', '-6.4563006751408065649420711471912e+9823772895', 99, 2);
    T('-6.6037586399429087501303e-3617828', '9.81010344056335316330719267487749895e+3895', '9.81010344056335316330719267487749895e+3895', 36, 5);
    T('-9.2221834979499892158520284229e+76120770339', '9.54601442563022064082072795150102691814749145767792e-162442259857', '-9.2221834979499892158520284228999999999999999999999999999999999999999999999999999999999999999999e+76120770339', 95, 1);
    T('-4.35551834330738787807e+435088004400', '6.42270210657718988940689641293589598303987e-8932279', '-4.35551834330738787807e+435088004400', 81, 3);
    T('-3.4039524061017986452870847210652303125937637006763236784e-36340193452', '3.573193437676733603738636e+21584211817', '3.573193437676733603738636e+21584211817', 83, 2);
    T('6.29916004380535009106866646665552575057477e+49', '-2.2437557039873196579971346456472656e-17185009831968', '6.29916004380535009106866646665552575057477e+49', 80, 6);
    T('2.375e-218277854849', '3.72923602115565384877641012993921645729833766646606e-68724440', '3.72923602e-68724440', 9, 3);
    T('-7.0747394776165094296214530653310283320352985216e+10989771763016', '930', '-7.0747394776165094296214530653310283320352985215999999999999999999999999999999999999999999999999e+10989771763016', 95, 1);
    T('-9.63491440916670592912489017193912314267283761168175e-122922476', '-6.635673546146772291048934837250282e+68121303', '-6.635673546146772291048934837250282e+68121303', 75, 6);
    T('-5.146051044657571080234185407185349947412663307e-68881770458', '-6.977773576620921586673665850332246446025273860571783687e+484171', '-6.9777735766209215866736658503322464460252738606e+484171', 47, 5);
    T('1.0614193256629763442983101040095035556919473733e+750950', '-6e-14020742048163', '1.06141932566297634429831010401e+750950', 30, 6);
    T('46138364343342933334350084403580546212431447', '-1.70203413881e+89074005', '-1.70203413881e+89074005', 62, 5);
    T('4.97e+794923', '-4.492156719e+42312100', '-4.49215671899999999e+42312100', 18, 1);
    T('9.007755395111e+9119651731465', '-9.4468e-23275649', '9.0077553951e+9119651731465', 11, 5);
    T('-2.93635833521576180065138466375878889298796195651892490e+182174998941', '-3.5582669964609532346658381874346839182438638148968607594101e-35821005', '-2.9363583352157618006513846637587888929879619565189249e+182174998941', 84, 4);
    T('2.34370655791678576231283269659810349679719926030551e+624142550', '-3.34683015337785036900413432064078493000151732740890906e+595655072', '2.343706557916785762312832696598e+624142550', 31, 3);
    T('7.582e-929', '8.340923631241e+42919', '8.340923631241e+42919', 42, 5);
    T('-4.91943727597237554196469012363381161303736043645767202e+54154254410', '-155695368.90431', '-4.91943727597237554196469012363382e+54154254410', 33, 0);
    T('-6.71087022e-73849758', '8.99010037760653270285746493e+907308', '8.9901003776065327028574649299999999999999999999999999999999e+907308', 59, 3);
    T('1.32866200250970223186e-601', '1.954131033566499277349051e-37431278020032', '1.32866200250970223186e-601', 94, 3);
    T('7.1468631083261491780336032417057474211914862871914e+57', '-5.34394132763607149884108195775248906881047263301803886875463e-312768854', '7.1468631083261491780336032417057474211914862871914e+57', 99, 6);
    T('5.5523470250931758683098140957486566795433345384491120948e+60267212693', '-43848973211155095904470664255983202350893.80', '5.552347e+60267212693', 7, 4);
    T('-7.88349903568814022196542843806472819449153459000806e-7285', '7.20436397934843801076272688506044404e-328131', '-7.883499035688140222e-7285', 20, 5);
    T('-7.82e+77148', '-9.42919687348138391077429081382386412256918610431870816162e-173880', '-7.82e+77148', 89, 6);
    T('-1e+606525', '191976614657081457091239864848547081076544252068345', '-9.99999999999999999999999999999999999999999999999999999999e+606524', 57, 1);
    T('-4.06724713814139350176559136256e+107', '-4.548724953007948383e-549374', '-4.06724713814139350176559136256e+107', 47, 2);
    T('-7.235e+6950', '8.091437122620980840343196797262609488110522302905908e-576336174164', '-7.235e+6950', 13, 6);
    T('-4.5784507030400199483777162e+99751544', '6.6728108024054e+15', '-4.5784507030400199483777162e+99751544', 52, 4);
    T('4.8968318442662067586e+20', '-9.0e-66651374', '4.896831844266206758e+20', 19, 3);
    T('2.365776739523631708701558542698679125919281637453e+8569102481', '7.4706648943431999e+948451120240', '7.4706648943431999e+948451120240', 84, 5);
    T('7.04056083170e+826556964', '3.26920043098391146771634244852775211486744984098e-4205853', '7.040560831700000000000000000000000000001e+826556964', 40, 0);
    T('-1.673467056284798160215080884698341069421055612942673115e+37040', '7.2916205596350363402362662768e+867723805300', '7.2916205596350363402362662768e+867723805300', 46, 6);
    T('-8.8272802e-38251', '1.285008829028191839910e-33111323308519', '-8.8272802e-38251', 19, 3);
    T('8.77393470005129464549908976771509334020508157516879584e-4387293', '-35351440757036542862378727347034342211430981809378.836767', '-3.535144075703654286237872734703434221143098180937883677e+49', 55, 4);
    T('-1.6339145291551243115686357064e-14149751066', '9.607231e-687085662460045', '-1.633914529155124311568635707e-14149751066', 28, 3);
    T('-7.001762062047e-984197583', '-1.093828346391412899745146926063166730386928074466e+88347018766180', '-1.0938283463914128997451469260631667303869e+88347018766180', 41, 1);
    T('-2.24474056404927591860311057436835791338299620676701263e-4228189', '-7.553597107084512760297187918257509476316929975990784000e+7448780', '-7.5535971e+7448780', 8, 6);
    T('9.9559796339e+574768547', '2.926590789974508380401020807267389648173981910511e-7230', '9.9559796339e+574768547', 64, 3);
    T('-4.922044383613383186774424e+332263544823654', '8.29145621554e+239424891407', '-4.922044383613383186774424e+332263544823654', 60, 6);
    T('4.252463261811513114530805157396902585256490e-3532020375', '2.20619451691434422077788791155727226576205211050453143014711e+734347535083', '2.2e+734347535083', 2, 4);
    T('-7.100544686983927193e-326056782237', '-7.71487956702e-2901109088754', '-7.1005446869839271930000000000000000000000000001e-326056782237', 47, 0);
    T('-1.796917985191934450200397363164017741766011313e+15782164662551', '-6.73e-14720780682', '-1.79691798519193445020039736316401774176601131300000000000000000000000000000000000000000000000000001e+15782164662551', 99, 0);
    T('2.623122459599364539759229245534467686971345395159401e-9377097632', '-8.7687e+30014507571285', '-8.7687e+30014507571285', 48, 5);
    T('74541010934800.02', '4.147716039093202928728775276643454815026433489862333933647e+7533319', '4.147716039093202928728775276643454815026433489862333933647e+7533319', 99, 5);
    T('-7.9984388034867199279355534218812124777915887266080143e-5589599', '-8.50885011866e-9744', '-8.50885011866e-9744', 37, 6);
    T('-5.9840678076808972733700e-369476904821148', '-9.60642772668e+6203', '-9.60642772668e+6203', 47, 1);
    T('-2.518484862909439969514475761533019117349823e-78989244', '1.819388155950672360715811791499879e-45501012355', '-2.518484862909439969514475761533019117349823e-78989244', 43, 4);
    T('0.0775805', '3.2132717561043770529571792253673808066856139782546610e-341362295801355', '7.75805e-2', 32, 5);
    T('-5.21780968995653259363116550270401376761761534417750476831e+48280630661', '-2.825451035444242836124621055619635275e-8564850315', '-5.21780968995653259363116550270401376761761534417750476831e+48280630661', 85, 4);
    T('6.80180877559838246295451351251295582e-7041', '-856798907897876.47', '-8.5679890789787647e+14', 92, 3);
    T('7.15814181403052108194446740245923e+80643', '-5111122923688718589658695761994221.725', '7.15814181403052108194446740245923e+80643', 60, 6);
    T('-8.9185060351459254058428344121053648930276134e-58847', '8.0288884801081627202566737280608275431418083391923991203565e+853238807667081', '8.0288884801081627202566737280608275431418083391923991203565e+853238807667081', 83, 2);
    T('5.944043e+1939094604', '6.0222725905487415479321205587864827175e+394681', '5.944043e+1939094604', 55, 3);
    T('5.9951382212015248119165780373430953328096008184706319e-894602169155737', '-5.56784586e-776', '-5.56784586e-776', 32, 5);
    T('-7.017366892518877644226782e-760096173001098', '-1.48358683888177357003653338769686229738089727286411e-225', '-1.48358683888177357003653338769686229738089727e-225', 45, 4);
    T('9.8783795672504553e-996621927', '1.58654298878133245232100862975153293939330e+45963595878337', '1.5865429887813324523210086297515329393933e+45963595878337', 96, 4);
    T('-1.92843803449e+18', '-5.7092173754386327639513337478494277799717582878779964e+23402682675920', '-5.70921737543863276395133374785e+23402682675920', 30, 3);
    T('-3.1299e-67', '137101531686456020880985992374.0095', '1.37101531686456020880985992374009499999999999999999999999999999999999999999999999999999999999999688e+29', 99, 0);
    T('2.45580671885038549290746101066009049917400558084561367e+211005', '-5.950679019090999632192594236367e-967', '2.455806718850385492907461010660090499174005581e+211005', 46, 5);
    T('2.6655861384773681322875368817959775598746935209635075335e+5070244654911', '-3e+5542', '2.6655861384773681322875368817959775598746935209635075335e+5070244654911', 85, 5);
    T('-9.895335709204734966943282632709472712670147e-2639345', '-8.536581170070033687072e+98', '-8.5e+98', 2, 5);
    T('-6.02422080429e-2072532342645', '-9.82892774470448128800e-227', '-9.828927744704481288e-227', 77, 1);
    T('-9.232391282502832861208981215422393508220e+25931947453', '3.206e+25821975', '-9.232391282502832861208e+25931947453', 22, 2);
    T('-7.501e-394583459872666', '9.998504837393576357781542330626248181991597528341e-15391169617057', '9.9985048373935763577815423306262481819915975283409999999999999999999999999999999999999e-15391169617057', 86, 3);
    T('-7.23801412912601471127240363768947657684995374825e-57010048934', '-2.340288073634540783275470004803162487535429703e+1666764767', '-2.34028807363454078327547000480316248753542e+1666764767', 42, 1);
    T('1.669e+2617964260189', '-2.77209848045514987381306337398770750674987767285802e+661783878050', '1.669e+2617964260189', 43, 2);
    T('7.119759891238460288314541869655187029236528089682137339e+2699529701', '-1.36886e+7416915708893', '-1.36886e+7416915708893', 6, 0);
    T('-7.7404965e+63414535617514', '7.024607778918608233093188127025590732433e-392', '-7.74049649999999999999999999e+63414535617514', 27, 2);
    T('5.693450833677643490e+8292', '1.39818018115745e+55411311005699', '1.39818018115745e+55411311005699', 53, 4);
    T('2.84074702e-4636293937021', '-5.7746972426817323454566e-951', '-5.774697242681732345456599999999999999999999999999999999999999999999999999e-951', 73, 1);
    T('-9.68887197555161988304618321702542819476858315099618e-422401472', '1.7805572019909046190910300636592574660e+107', '1.780557201990904619091030063e+107', 28, 3);
    T('416203137407821.39875591', '-4.452385104804718358366262389170029385048236409639763974e-3302870195', '4.162e+14', 5, 6);
    T('-4.738e+8', '2.67342315676006106778669963891955669534808096149e-2141540955898', '-4.738e+8', 21, 0);
    T('-2.979e+83684306767324', '2.06892802234765063397938810460e-305448719', '-2.979e+83684306767324', 23, 5);
    T('7.75397826023751506562314896410604624045038910364814924711e+949', '-3.8736402223060469135663180378002264e-83301714', '7.753978260237515065623148964106046240450389104e+949', 46, 4);
    T('-5.112632622735584e+504225725', '-7.8685e-44', '-5.12e+504225725', 3, 0);
    T('8.96360857830457395423000636700554e-914', '6.258245241576e+264859188', '6.258245241576e+264859188', 73, 3);
    T('2.2089770695061547198992852756945859835634429e-6706', '1.848861865427197725710e+524104524365', '1.84886e+524104524365', 6, 4);
    T('1.65955864032146e-792041393', '9.67383939413957861142717238071034811993203e+986', '9.673839394139578611427e+986', 22, 3);
    T('-9.11986620557027132665176e-3467', '-7.54044579608222811e+1037', '-7.54044579608222811e+1037', 80, 5);
    T('-3.698891969622107720892400732065303121639593064578e+3691999360', '4.67327380e+10', '-3.698891969622108e+3691999360', 16, 3);
    T('169168057483778341817695163713.46611', '-5.107322507974395e+553950', '-5.107322507974395e+553950', 28, 5);
    T('-4.648946204926180e+26155095', '-4.22432435774521143358813043701565778196425540134e+9333315', '-4.648946204926180000000000000000000000000000000000000000001e+26155095', 58, 0);
    T('1.9202404207460864412509538818541140837864338209112231875e+69645330273425', '5.7239e-822755257251509', '1.92024042074608644125095388186e+69645330273425', 30, 0);
    T('7.3991487021295427814194095642894844051e-452', '-2.98667119673641764380296682692441532422637e-687641083', '7.3991487021295427814194095642894844051e-452', 47, 6);
    T('-4.43384214592926325112671898e+11208912513762', '-1.1933915241101355785544758383197638720397678418e-818406801143', '-4.433e+11208912513762', 4, 1);
    T('8.860599189951190256590883562103247219e+365', '5.857236190526046388857131755786103812823655109722828e+67661918685049', '5.8572361905260463888571317557861038128236551097228280000000000001e+67661918685049', 65, 0);
    T('9.7463848085436490301443352769908479825757069460106097e+8254', '-6.0050242602635e+80011', '-6.0050242602635e+80011', 33, 6);
    T('9.69543263585559529971178165608850e+5855512084', '5.32062362173195789350365949596414866242865418443257979e-63318507360962', '9.6954326358555952997117816560885e+5855512084', 49, 3);
    T('8.61731e+8957903', '-2.2271e+9', '8.61731e+8957903', 72, 0);
    T('7.005113102758169247694691091711282488782777706772574236e-696003883', '3.44077664367e-791011583', '7.005113102758169247694691091711282488782777706772574236e-696003883', 72, 1);
    T('9.4805200836718827592565947785503000e-86170917', '2.28280109528168e+21582', '2.28280109528168e+21582', 29, 4);
    T('-7.08154896519594328628609482734060369482828790e-856353760221591', '-8.99604043625395846790172774749093281957541260561534114e-52992184782103', '-8.996040436253958467901727747490932819575412605615341140000000000000000000000000001e-52992184782103', 82, 3);
    T('-1.7166384513944039212414e-554920655986301', '5.957124773745035613920686338551324891684884207972766938635e-578951672045', '5.95712477374503561392068633855132489e-578951672045', 36, 6);
    T('6.82764952668463671632566841287340718297062086997149687558e-152083', '9.0206890083786884830404e+2733720', '9.0206890083786884830404e+2733720', 84, 6);
    T('-6.5954529835967486495e+925431412', '8.0836904742243738551969475097423219927732029567613e-856', '-6.59545298359674864e+925431412', 18, 1);
    T('-9.711104173801330939921673450719665541493400e-1857', '9.894105823121750486e+6913917', '9.8941058231218e+6913917', 14, 5);
    T('1.3973781778913219365975e+773926190150379', '-3.2434376009406e+86455476832', '1.3973781778913219365975e+773926190150379', 51, 2);
    T('538664749601193966031460131136779263885036095839.6332', '5.40209579617483984909225766598953232684266198130467561086e+357145068932635', '5.40209579617483985e+357145068932635', 18, 4);
    T('-2.284278657016504932048457273e-29662135', '1.35813580e-8348422043474', '-2.284278657016504932048457273e-29662135', 44, 6);
    T('-9.47922192071090108835774106277635245441213850338818509659928e-29', '1038511304193212265424984876.16556868906284604', '1.03851130419321226542498487616556869e+27', 36, 5);
    T('-1.731182200e+8318289', '-4.7705128345632231898e+28', '-1.7311822e+8318289', 55, 1);
    T('-8.74289792429638843934648264909679632461733199156576469e-13694270036', '-1.0528202e-46449', '-1.0528202e-46449', 64, 6);
    T('-2.383040641168037680875749106004173484784125632e+5944599714342', '-0.027', '-2.383040641168037680875749106004173484784125632e+5944599714342', 84, 1);
    T('-1.4594893670e-6064852', '-3.15959879807285960756606885521181e+318', '-3.15959879807285960756606885521181000000000000000000000000000000001e+318', 66, 3);
    T('-7.12979295973840834941193756134470243429828273949e-1500078670714', '8.6867215292974925767500846049914e-30', '8.6867215293e-30', 11, 4);
    T('9.29586995136941e-27912832753', '-2.779087762552046739341423444452953981609972476301e-290976948', '-2.77908776255204673934142344445295398160997e-290976948', 42, 6);
    T('-9.9844e-465208205231851', '-5.072234623937962673169972927471659101595e+110', '-5.072234623937962673169972927471659101595e+110', 68, 6);
    T('-9.32e+7', '-6.058142735289440099758195e+768', '-6.058142735289440099758195e+768', 29, 1);
    T('3.8432960891564356800440731233334533547207565059450343417954e-1561940', '-6.968518598976579322336568e+41480901614814', '-6.968518598976579322336568e+41480901614814', 85, 0);
    T('2.97201047765365851435947e-1138870708', '-2.41027257889791551216082090422579479817062446806013114e+293672008707', '-2.41027257889791551216082090422579479817062446806013114e+293672008707', 62, 5);
    T('6.47139256228184413099769720185154131535046164217276539e+67222277617', '6.8e-27780', '6.471392562281844130997697201851541315350462e+67222277617', 43, 4);
    T('6.68271794337340357345978537402e+10161667', '-3.00259162559681811589507e-85353163477', '6.68271794337340357345978537402e+10161667', 38, 0);
    T('2.224885215086852978385368018e-88684603754432', '-2.043285635093696991120e-32661748', '-2.0432856350936969911199999999999999999999999999999999999999999999999999999e-32661748', 74, 1);
    T('-2.25744260723842087683e+61868484090021', '-6.47057849774491453e-988508899', '-2.257442607238420876830000000001e+61868484090021', 31, 3);
    T('3.3179788405487244719712566364603856392135325274261427644e+1390681720814', '9.34075970654751e+2215364824168', '9.34075970654751e+2215364824168', 15, 4);
    T('3.10e+1149357121774', '-7.0243534697373690513200572955293627606869246996571320875e+59', '3.1e+1149357121774', 32, 6);
    T('-4.29650691040803952522209332268714008898143877603523203623869e-7567492409', '9.31744801406201471119652850072e+6213652816239', '9.31744801406201471119652850072e+6213652816239', 80, 5);
    T('2.76148515957401596e-1831975', '2.10607038181187226882125948460274e-690300351286', '2.76148515957401596e-1831975', 50, 6);
    T('-9.9203226727193951988699937062339253651155059480083356e-591', '-5.008259408488263699424203241250683834999e-79677057657252', '-9.9203226727193951988699937062339253651155059480083356000000000000000000000000000000000001e-591', 89, 3);
    T('-1.62e+4296888252', '-9.0647769659883628162755500565660888580990258591225202783e+57', '-1.62e+4296888252', 85, 4);
    T('-9.77603958311805513397954187156536e-82793173702724', '-9e-22247177', '-9e-22247177', 66, 1);
    T('-5.51758550673e+5064', '3.7116826657112403660322624334899e-73867753', '-5.51758550673e+5064', 26, 4);
    T('-1.6895952486061e-44347751507', '-7.31990791553096613439015902275398658e+67742164', '-7.3199079155309661343901590227539865800000000000000000000000000000000000000000000000000000000000001e+67742164', 98, 0);
    T('-5.77609399e-1549575', '9.59631186669e-541515415418', '-5.77609399e-1549575', 9, 5);
    T('1.7374802824141853038170588667439089e+308463', '9.570537348064411861145508722e+5569', '1.737480282414185303817e+308463', 23, 3);
    T('1.489e+5544465', '8.0632981582909696904774e+882994848290', '8.0632981582909696904774e+882994848290', 71, 3);
    T('-9.8e+2138482859293', '9.3632055533379070984683225435050269735840669e+54', '-9.8e+2138482859293', 92, 6);
    T('6.977073744288154312847536292524403455166572e+9340', '-2.942683754888966771991e+319311562', '-2.942683754888966771991e+319311562', 52, 4);
    T('6.2776287957938963598413171281e+31510808887', '6.893657094793013345558241121880192697958636848e+969525859', '6.2776287957938963598413171281e+31510808887', 67, 1);
    T('-9.57419057011316468167338591118913e+796', '-7.74129169031346963391303661813019908e+267', '-9.574191e+796', 7, 5);
    T('9.46626491404596711377854874e+797006389', '8.827907816061362283206981377863103598220133620652887e+309657401', '9.46626491404596711377854874e+797006389', 45, 1);
    T('5.271985940478129190389025703693790767024637e+724939177014001', '-2.7825128712697662229460025573498385e+42101', '5.271985940478129190389025703693790767024636999999999999999999999999999999999999999999999999999999e+724939177014001', 97, 3);
    T('8.115145596847e+1320', '70356721878768719025460453827910515459262810502.2871', '8.115145596847e+1320', 72, 1);
    T('7.15450296788361467558464638910053101638566636531126446980e-20073', '3.4119010853837012469973136863854624e+9928793529', '3.4119010853837012469973136863854624e+9928793529', 82, 6);
    T('3.019498671618610928438206100096e+823', '44225137189864523482.47802707', '3.0194986716e+823', 11, 6);
    T('2.143374611676949442111087e-5437897830853', '4.171598196481102543953644338939287863e-439656520807', '4.171598196481102543953644338939287863e-439656520807', 92, 4);
    T('-2.933648800145548620145727528010895055813137444395e-9918', '-6.11018650181970193283082021467185e-32091', '-2.933648800145548620145727528010895055813137e-9918', 43, 5);
    T('448174264017828960312856330692292528338919304898.37036705', '3.816e+581773933671901', '3.816e+581773933671901', 41, 1);
    T('8.484242291040408327614697647531719176789e+459536278431564', '6.5653029907764019729175137210769900066589283570e+4763', '8.484242291041e+459536278431564', 13, 0);
    T('3.228252821277825773e+11816706', '5.04e-53419584', '3.228252821277825773e+11816706', 51, 1);
    T('1.0884037402119396927453972783526360129889e+8103', '-5.755300590072e-631885479415', '1.088403740212e+8103', 13, 6);
    T('-6.6611e-5723', '8.0915489323884013211534815110369317434e+908', '8.0915489323884013211534815110369317434e+908', 84, 4);
    T('8.90838086', '-3.071078623985817320030475872756810124863e+47', '-3.07107862398581732004e+47', 21, 0);
    T('-5.75280801532780300980758452779855322e+9120', '-4.099040415924137632946717865769008479740093e-9738230150', '-5.75280801532780300980758452779855322000000001e+9120', 45, 0);
    T('-4.799141593815816082471e-952554378276', '-6.35206432441146382094657e+5259974', '-6.3520643244e+5259974', 11, 5);
    T('-9.82596317009e+106477649186', '-4.60269447485782817769510121952360813378639362797e-28', '-9.82596317009e+106477649186', 67, 4);
    T('1.72391978736607713170293714e-10738', '1.51872304851442600118232405899795964333214531931e+258505', '1.518723048514427e+258505', 16, 2);
    T('7.067272691564123267940338867773304023363852421652246363e-74642813657899', '-7.61050810572592411e+36266167852', '-7.6105081057259241099999999999999999999999999999999999999999999999999999999e+36266167852', 74, 2);
    T('-3.141032310518601594e-39522768949', '5.66222242383982350702648494476543382216623969374739921106548e+991520', '5.663e+991520', 4, 2);
    T('9.0259738505795e+77068682694814', '6.062615e+86353', '9.0259738505795e+77068682694814', 49, 6);
    T('3.47412709e+7078', '4.109705400602721000403634e+131223316342183', '4.109705400602721000403634e+131223316342183', 63, 4);
    T('-9.8895258670422384889527981544548866000890e+42', '-1.5181022115786805e+8721916122809', '-1.5181022115786805e+8721916122809', 25, 4);
    T('-4.75811e-2889581', '-1.23885007903823e+88636580952', '-1.23885007903823e+88636580952', 44, 6);
    T('8.329869551006912071858086956e+6214665777971', '-9.089628962971198161310869132458787042732098666898e-7803231840391', '8.329869551006912071858086956e+6214665777971', 59, 2);
    T('-7523010404518502382150820202684493232.6610', '-7.42352032876105714165769928764467205522935859428e-533526053255', '-7.5230104045185023821508202026844932e+36', 35, 1);
    T('4.6741395065781943452165312780717107109e-4546929', '9.42990969177526848387672499117563649871551655350735944e+87854401966393', '9.42990969177526848387672499117563649871551655350735944000001e+87854401966393', 60, 0);
    T('-6.0590040858572729655864384954196005786e-1390227349', '-4.841277939952198194963050103135001184641877214e-73515435966622', '-6.05900408585727296558643849541960057860000000001e-1390227349', 48, 3);
    T('-4.63993797670618953334972037141e+9364706964', '-6e+204424097570', '-6e+204424097570', 81, 4);
    T('1.861800350601785871487776094763784262e-17969873946677', '-9.736838422290034726514282e+5419', '-9.74e+5419', 3, 3);
    T('6.48978889120010529718600659337001646431e-6968665', '-8.0965640638349322689e+86151423232558', '-8.09656e+86151423232558', 6, 6);
    T('3.7915230903392839555526360462372596886535858e-69071532', '3.5757943721943e-3088381754118', '3.7915230903392839555526360462372596886535858e-69071532', 94, 3);
    T('-5.69928855e+15', '9.8891894013072843504025922924500871559e+69', '9.8891894013072843504025922924500871559e+69', 39, 0);
    T('-1.2123443831828443482424878901227742189528657003623e-2789765038', '4.8e-327452369338', '-1.212344383182844348242487890123e-2789765038', 31, 3);
    T('-4.589983502859965416884278014233689954089817e+107', '8.1096251819e+99323', '8.1096251819e+99323', 85, 0);
    T('-6.7923514525007448055236496760116638318759335e-65449034394239', '404925863224557783969820.17939428437', '4.0492586322455778396982017939428437e+23', 35, 4);
    T('-2.15166387385961879804312421824190708549061e-497363', '-3.00661882683726742367756408343764393111908806e-686837619083815', '-2.15166387385961879804312421824190708549061e-497363', 69, 6);
    T('-1.40304e-867401', '7.7997621503277781596998907301854609787074683459100034e+32796953439281', '7.7997621503277781596998907301854609788e+32796953439281', 38, 0);
    T('-8.61452345031997455167676496535862e-25180087504', '2.1196139512354726e+31', '2.12e+31', 3, 4);
    T('4.9287059292605254405694619638851107570926177811e-29108684', '9.96923547267003423053112e-789', '9.96923547267003423053112e-789', 45, 1);
    T('1.910154655286047845367679e-3303460318', '-2.187328310240e+836702482842309', '-2.18732831024e+836702482842309', 75, 4);
    T('7.928985008474685562779522004e+7354', '-7.0154154462273414021305e-30622848311535', '7.92898500847468556277952e+7354', 24, 3);
    T('4.9041123218420097361770124950003686101290308248340189e+7746874694630', '-6.975872158040111541212049678799266442737434636404752863e+748027300821', '4.90412e+7746874694630', 6, 2);
    T('-7.762880813509e+47213728', '3.678189649276845979002463246e+56988', '-7.762880813509e+47213728', 83, 5);
    T('-9.51888595347650650494067825749757153e-474280248', '-570.007532411', '-5.70007532411e+2', 22, 1);
    T('3.688147907022781013587e-67054643335731', '3.60402510560111126009799e+980991', '3.60402510560111126009799e+980991', 89, 3);
    T('-2.6659526753838845084480775666851794539e-767343', '7.2509562516977585893120155570535867325151839512e-457964084', '-2.6659526753838845084480776e-767343', 26, 4);
    T('91230748425733007506052830273553447072526512979532866780', '2.0071353740316921074061112120e-86003', '9.123074842573300750605283027355344707252651297953286678e+55', 66, 4);
    T('7.883e-3400', '5.52e-2727442', '7.883e-3400', 60, 5);
    T('-5.7769289366302388093096422099955656e+38', '-3e-670345151123392', '-5.7769289366302388093096422099955656000000000000000000000000001e+38', 62, 0);
    T('-4.1641282915777e-926', '-4.80495154467671002390929829455738280097061e-867', '-4.80495154467671002390929829455738280097061e-867', 56, 4);
    T('8.33089483340737419647023055e-34908833302743', '8.695963779951070311698612325e-31090439556146', '8.69596377996e-31090439556146', 12, 2);
    T('-3.609722914680430042e+5862', '6.51e-630188401268761', '-3.609722914680430042e+5862', 93, 5);
    T('-2.22535068638146161648880065816306e-980274', '5.330323040384479652361080237765232887896788318446885226e-8063276', '-2.22535068638146161648880065816306e-980274', 96, 6);
    T('9.652309018153210529113262056679226739627789109015955641e-25114920886', '6.75649557179776823701544703090e-2866670', '6.7564955717977682370154470309e-2866670', 99, 1);
    T('3.895711823e+9140', '3.0846877741640450453038546878985e-180119', '3.89571182300000000000000000000000000000000000000000001e+9140', 54, 2);
    T('-8.427306592072015136e-8708107', '-4.66148981841274574e-545', '-4.66148981841274574e-545', 36, 1);
    T('-822344', '9.48e-683729804', '-8.22344e+5', 94, 6);
    T('2.616073865355175910470729015252929720e-4852', '-5.224138826112858e+79939', '-5.22413882611285799999999999999999999999999999999999999999999999999999999999999999999999999999999e+79939', 96, 2);
    T('-6.9479063911909537420530079766266846090e+45051210960399', '7.2928694015384682553e-51', '-6.94791e+45051210960399', 6, 4);
    T('3.1738942240e-661178455', '5.9446445286711137925522476400314815754506101756532361866e+315666266517', '5.9446445286711137e+315666266517', 17, 3);
    T('5.945513151749e+836697865594384', '-9.2850072575072884269588641804887899701e+75467405', '5.945513151749e+836697865594384', 56, 6);
    T('7.40828162794960016981383873746326112708496669648280149e+136143892325505', '-7.523952656932083397991164953282291477338732975808752493e+56', '7.408281627949600169813839e+136143892325505', 25, 0);
    T('7.3539772498150764e+3255', '-8.65625238888501015615940305026210461995002158740531256456963e+69434011', '-8.65625238888501015615940305026210461995002158740531257e+69434011', 54, 0);
    T('-1.8937757998526e+21', '-7.1035276164297418264954543323570517958751224663070e+44725842', '-7.103527616429741826495454332357051795875122466307e+44725842', 67, 6);
    T('6.3484e-317297442', '3.623796939858173076155128186428980346e-55481000335096', '6.3484e-317297442', 11, 1);
    T('-3.147538307501986e-82593808559388', '-3.20141909190e+17', '-3.2014190919e+17', 20, 1);
    T('-5475827703.5091605141996619680021680579182292089437796', '-9.0768583241021474036329724592019647e-943846461389', '-5.47582770351e+9', 13, 0);
    T('-4.250305928131343039522020403790126164625772139e+5417734221893', '-7.69630398e+5403', '-4.2503059281313430395220204037e+5417734221893', 29, 2);
    T('-1.8773977797017418408619598969228895366533928680331717896e+920', '-8.535000201680207446208475283253200468845e+98705739482', '-8.535000201680207446208475283253200468845e+98705739482', 94, 1);
    T('6.99812863202992671714e+22', '-7.990455326839513645210749489387403401493431e-86', '6.99812863202992671e+22', 18, 1);
    T('-6.52014621644608150152068858768754112520e+56801175986289', '7.13298826405232892e+46776', '-6.5201462164460815015206885876875411252e+56801175986289', 89, 6);
    T('6.50e+519946', '3.408576898371774687877291023402908452600511515e-87795', '6.5e+519946', 22, 1);
    T('5.23396502188260251117104984317242185e+157839', '7.866460188063e+670', '5.23396502188260251117104984317242185e+157839', 81, 1);
    T('-7.19224476564022759143111379454512252505533671571568323e+419136396948523', '3.7221563426120552423024447720490123e-648', '-7.192e+419136396948523', 4, 5);
    T('-1.7269046983577001099744e+665', '-1.66984711725248439492987449801933984e-740237', '-1.7269046983577e+665', 16, 5);
    T('7.6696e+13', '-9.68911031645307371521296158849327746508997608865974788756e+321620633', '-9.68911031645e+321620633', 12, 5);
    T('8.4938557849556604e+937094982451', '-1.163891888512018609e+630', '8.4938557849556603999999999999999999999999999999999999999999999999999999999999999e+937094982451', 80, 1);
    T('2.85880038011731102117903853945e-80008387534529', '-3.79532288994809e+43887298011', '-3.79532288994809e+43887298011', 83, 6);
    T('9.1454292507101111915789473133398997779050825865647269496207e+62', '-8.98634008839e-1885726', '9.1454292507101111915789473133398997779050825865647269496207e+62', 65, 0);
    T('4.355149245699e-82557945135', '1.4628600709842272911462231106734321258045316158231053821e+232339', '1.4628600709842272911462231106734321258045316158231053821e+232339', 96, 5);
    T('-6.4741385023008192e+65725109085206', '-2.501676658998895888074787502829886148442207485747e+71410718', '-6.474e+65725109085206', 4, 4);
    T('-3e-625539', '-9.4254526787410783341664770989e+19836', '-9.4254526787410783341664770989000000000000000000000000000000000000000000000000000000000000000000001e+19836', 98, 3);
    T('-4.8376425900756033939e+82316', '9.85632190286078405453414717425038568752851995174236056e+19647', '-4.8376425900756033939e+82316', 92, 3);
    T('-8.9901831011958457443242997057004495e-576275345254860', '4.506276249382788095902e-85392091053497', '4.506276249382788095901999999999999999999999999999999999999999999999999999999999e-85392091053497', 79, 3);
    T('-4.1423795404503076342967041040056928622957436870176967784864e+574', '-52059722919783001496372571695643078719157949719413.1722143', '-4.1423795404503076342967041e+574', 27, 6);
    T('-2.42165648709167208100526992479182565768732717138304e-956154', '-2830839859941016876035618983138491897858463', '-2.83083985994101687603561898313849189786e+42', 39, 5);
    T('-6.6e-686418904716', '5.720648902291790957786095981015506428856035408787373017524e+82399913745764', '5.720648902e+82399913745764', 10, 6);
    T('-3.3386761e+40933882', '-1.121634778828949e+163867127', '-1.121634778828949e+163867127', 47, 2);
    T('-5417775388629486800278592422084716956260.2063', '72688013348261904525884467488828115300518838306375.3', '7.26880133428441291372549806885495228784341213501150937e+49', 63, 5);
    T('9.09150623580682636532057038315082389053104242e+62200367', '-8.21149013539565015108308387785053135699786462e+6725534266057', '-8.2115e+6725534266057', 5, 5);
    T('-4.48956721338198704935757276e+129213972', '8.846e-63828400', '-4.48956721338198704935757276e+129213972', 97, 5);
    T('5.85468182643043961286809459572083641e+1314562968908', '4.38369281742389439870687478159496912787425e+263464409008', '5.85468182643044e+1314562968908', 15, 6);
    T('4.3326015941569896671510034700216754122040078e-6181', '1.900035004379249730174422799750976209991e+72', '1.900035004379249730174422799750976209991e+72', 55, 4);
    T('1.51823741184100094624699641460073e-7163898845401', '5.48901287429400419e+5913862', '5.48901287429400419e+5913862', 90, 4);
    T('3.79773262507083859320538429351345593005e+474369', '1.535753e+11', '3.79773262507083859320538429351345593005e+474369', 93, 6);
    T('1.11678889963893018558018748960554956069219627775e+6480929498110', '9.74062e-416766756952716', '1.11678889963894e+6480929498110', 15, 2);
    T('3.717130306936696284244323799073e+59', '2.4021676129897588610503942278033708230571793981228694689303e-59535', '3.71713e+59', 6, 1);
    T('9.615523102465765682533911e+31712560', '-8.0983e+844', '9.615523102465765682533910999999999999999999999999999999999999999e+31712560', 64, 3);
    T('-5.83441755789432e+9333601807', '4.1749500401283981263869333156965727353045e-6349204645', '-5.83441755789432e+9333601807', 41, 3);
    T('-1.13e+47235307956747', '4.14454159830027364757771208294112e-79566', '-1.13e+47235307956747', 92, 4);
    T('7e+21599580667699', '-1153713156926038118790537778652879018.3087877', '7e+21599580667699', 94, 0);
    T('9.71453992609907587551611014583029452998665e-283272544371213', '-6.0275992756066139575987023336510729720505127537504626976e+75007279', '-6.0275992756066139576e+75007279', 21, 6);
    T('1.373903411378650587699e-3168566752', '3.65643756982149199659485404683e+32', '3.6564375698214919965948540468300000000000001e+32', 44, 2);
    T('7.187957544716883190993209373290610555173e-322981807741274', '-4.2198015617636582908154975126098889507603198829477573e+943', '-4.2198015617636582908154975126098889507603199e+943', 44, 0);
    T('2.73991759885e-186486206830', '-7.5260917182767e+70839721274459', '-7.5260917182767e+70839721274459', 24, 6);
    T('-9.0084574930679147821568297096915443e-23904', '-8.7110086225321765949824077059667479321848703e-5919', '-8.7110086225321765e-5919', 17, 1);
    T('-7.04064154e+71863', '3.75903123567944945186836461467501225865439903649874755297e-221078', '-7.04064154e+71863', 16, 5);
    T('-0.0007578737452661638129819382013733413811269', '-2.5070667473780109013030020160795852030e+643219798494214', '-2.507066747378e+643219798494214', 14, 1);
    T('-9.5685e-22956814309', '9.8293081576694678584e+4865', '9.8293081e+4865', 8, 1);
    T('-4019494481006520381.860404044638167', '-4.91538773e-50332043298268', '-4.01949448100652038186040404464e+18', 30, 3);
    T('1.871669936701596815255336744727383003137147973724e+26794', '2.223879802524159814157425686521341875137148e+59736975374', '2.2238798025241598141574256865213418751e+59736975374', 38, 1);
    T('-5.732822825870425585185752865935499961854617082440273e-4111824651557', '-5.78581739040427440282018689219607721559439e-24', '-5.78581739040427440282018689219607721559439000000000000000000000000000000000000000001e-24', 84, 3);
    T('-2.64324198157162e+8983', '6.9117716874037414143425394891935685513567074073256659847463e+678592466050', '6.911771687403741414342539489193568551356707407325666e+678592466050', 52, 4);
    T('0.0007223', '8.325e-3820444981747', '7.223e-4', 14, 3);
    T('-9.44891580955851e+183800634808', '2.53302332122914107784208861069282366073950103810987680e-7', '-9.44891580955851e+183800634808', 80, 6);
    T('-6.1679464883609848800016916356894662e-34331291071', '6.0e-619141783359205', '-6.1679464883609848800016916356894662e-34331291071', 46, 4);
    T('5.0434171165194023e+852', '8.192046648639233178964901987419152258522e+69233331281', '8.192046648639233178964901987419152258522000000000000000000000000000000000000000000000000000001e+69233331281', 94, 2);
    T('-6.431966094416188007646616e+30001991', '-5.5e+57230', '-6.4319660944161880076466e+30001991', 23, 2);
    T('-9.2e+9372378335', '-7.06555492283788137368822742898286439099541177e+29893050076', '-7.06555492283788137368822742898286439099541177e+29893050076', 89, 2);
    T('4.4831735148195827175864887385755241727844236514194764346153e+4034737', '3.43675428289284928487267e+754', '4.483e+4034737', 4, 1);
    T('8.687995538072421064414315393902563007759183e+113178872', '3.4970291924176254880725197708133581463131025201011759287e+139', '8.68799553807242106441e+113178872', 21, 6);
    T('-6.467600126081458126992225556797e-433', '1.098053e+6296443439', '1.098053e+6296443439', 74, 2);
    T('-5.59766719276326810235195566556050023524e+38650282440', '-1.1619019e+93', '-5.59766719276326810235195566556050023e+38650282440', 36, 1);
    T('-1.46177e-2783717053', '-8.66284117272807122551600004786890599e+9437435089584', '-8.66284117272807122551600004786890599e+9437435089584', 64, 5);
    T('2.8778107787417910963702413364457686953023655718382e+51', '-2.095653235738725e-783860681884', '2.877810778741791096370241336445768695302365571838199999999999999999999999999999e+51', 79, 1);
    T('8.248728495677635e+6455252624778', '-9e+959397393933233', '-9e+959397393933233', 58, 5);
    T('68961831233939847319516894383357.32321073', '-1.738250248358116215996619309396756669915e-54377938771573', '6.896183123393984731951689438335732321073e+31', 75, 4);
    T('-8.69555058049561390701668824702473360483975262228043380e+9209', '-9.4924451748071763754510783235731e-23411', '-8.6955505804956139070166882470247336048397526222804338e+9209', 94, 6);
    T('9.63272738605532012250931016297538878093508e-603471923640', '1.67325468438947846496540206762528951740420555354978275082298e+61511158931947', '1.67325468438947846496540206762529e+61511158931947', 33, 6);
    T('1.044543e+680144953631', '-3.2820662861254941109197431322381155051641762049717452955343e-39867967', '1.044543e+680144953631', 48, 5);
    T('-9.580603138295e+549', '7.55622824205951493009587514204e+352680135605', '7.5562282420595149300958751420399999999999999999999999999999999999999999999999999999999999e+352680135605', 89, 3);
    T('-9.353335702186169028648293122944683806455e+110368', '-7.862474965497158520e+41', '-9.35333570218616903e+110368', 18, 4);
    T('-8.369844344380216832348e-62014201177098', '-8.950856273665900071262766606811079192e-994848309', '-8.950856273665900071262766606811079192e-994848309', 85, 2);
    T('3.092070416997839383965276546876115544486769637e+86592322', '-1.203788697901e+5602', '3.092070416997839383965276546876115544486769637e+86592322', 95, 2);
    T('-1.01962611787701e-69371414029', '-9.40651315710611336094664827400147081401814933933e+580176660', '-9.4065131572e+580176660', 11, 3);
    T('-5.6e-3795554111', '1.85094991246505843511e+34752972985', '1.85094991246505843511e+34752972985', 80, 2);
    T('3.8186176010216023137418991068025118125467470558343003392e+7068', '9.604419358886085071e+52', '3.8186176010216023137418991068025118125467470558343e+7068', 50, 1);
    T('-4.60350638865141652313619608142393708092e-97466926', '4.0750547563716537129246701624852519074062459871280773450017e+19293', '4.075054756371653712924670162485251907406245e+19293', 43, 3);
    T('-57907188741178633828006008299899532989991124745561669.1725284', '6.02718131154501009587316374461364290817773517e+23774249729207', '6.027181311545010095873163744613642908177735169999999999999e+23774249729207', 58, 3);
    T('-9.298471821985697838260971265242272155062254110207e-491', '-8.071153810194203611933e-36904958633', '-9.29847182198569783826097126524228e-491', 33, 3);
    T('-7.1543502088692076254903925598010735032206525441510284321e+56', '4.4742136287703343376126122648303173e+40', '-7.154350208869207178069029682767639741959e+56', 40, 1);
    T('6.86652468632671e+9102557388', '-6992704777141519024431384118.9304', '6.86652468632671e+9102557388', 38, 5);
    T('3.7643070656624339478681604946610193030134234e-3910014372', '7.17323916215038e-4032', '7.17323916215038e-4032', 87, 4);
    T('-2.10341e-4563564236', '-4.5238895818119553e-6268', '-4.523889581811955300000000000000000000000000000000000000000000000001e-6268', 67, 3);
    T('8.7458906622443503394934337737283557974282511194896328036866e-292', '7.43935152981730889472229907262482239100e-782', '8.74589066224435033949343377372835579742825111948963280368661e-292', 60, 0);
    T('-4.8242278674576904658e-39224540189530', '4.805879369784716559659061006970796172163885e+1600406230', '4.805879369784716559659061006970796172163885e+1600406230', 56, 5);
    T('3.38207332177182057207385743884e+62032065042', '-8.8180363395e-7577419', '3.38207332177182057207385743883999999999999999999999999999999999999999e+62032065042', 69, 1);
    T('-2.3477027450115302102806918131e+447602581500', '3.700595040e-5411393', '-2.3477027450115302102806919e+447602581500', 26, 3);
    T('2.71349e+29168836566035', '-2.88739719617517622593135799652753e+70762960636643', '-2.88739719617517622593135799652753e+70762960636643', 84, 6);
    T('3.43842348726217727690831986846725797271292445999e+580698858748', '1.001026827490067310e+19694', '3.43842348726217727690831986846725797271292445999e+580698858748', 82, 1);
    T('9.366491430723581383490222256086494768361753992407e+28432270', '-1.3312395377368e-2395', '9.366491430723581383490222256086494768361753992407e+28432270', 58, 2);
    T('-1.5809415163677351895392370183777e-2100365', '7.2734151600546130412655124476493e-96700', '7.2734151600546130412655124476493e-96700', 36, 4);
    T('-1.35209984310803123e+942', '-8.259111723064087965283122559865669439861377205850745845337e-2663', '-1.352099843108e+942', 14, 2);
    T('-7.729e+4158029153', '-9.1003584063439087031607516949204188793989593e-2356051', '-7.729e+4158029153', 26, 6);
    T('-8.95942e-12360', '0.0575598259860115631051018587974351536499', '5.755982598601156310510185879743515364989999999999999999999999999999999999999e-2', 76, 1);
    T('-2.2334578322376843202679914672247566416417192347651495409085e+194296583544339', '-8.46477111276541506575368169604170618354433485e-7715132354538', '-2.233457832e+194296583544339', 10, 4);
    T('4.649159909266006784534165232862322e+79', '-5.09964579587184545922183477888280356379778397568e+687615138409488', '-5.09964579587184545922183477888280356379778397568e+687615138409488', 86, 0);
    T('3.853121e+43444145839958', '-8.7147959037456513507367531206793340830747319225e+3092560', '3.853120999999999999999999999999999999999999999999999999999999999e+43444145839958', 64, 3);
    T('6.6562934927327606825e+254', '4.513284467320617883675376512745064089161443e-142', '6.6562934927327606825e+254', 61, 4);
    T('-5.4450718616438843003209252515004e-50672445804818', '9.28645901029693899597137255484675e-379', '9.28645901029693899597137255484675e-379', 99, 6);
    T('-5.23735586601905355710330554677018257903e-5776', '-2.20467160477425252533369085116873243904723e+936981098', '-2.204671604774252525333690851168732439047230000000000000000000000000000001e+936981098', 73, 3);
    T('-5.31342628549751329337476599e+10196802841', '9.7053387e-21387093', '-5.3134263e+10196802841', 8, 5);
    T('2.713129154131939329068522944255084707279897e-4005354198', '-2.1107026e-1126', '-2.1107026e-1126', 67, 6);
    T('6.34378066494364036343126e-860000493950', '-2.0455206528399302070931744620931199103824634611052201e-27924', '-2.0455206528399302070931744620931199103824634611052201e-27924', 95, 3);
    T('3.591627440275e+30105340985', '3.762785065540109825607624585991830769939817901e-86665234106', '3.5916274402750000000000000000000000000000000000001e+30105340985', 50, 0);
    T('-9.38807312232364354027708286e+4286127599', '7.95263482709456469689319163514685782346306136269950546278e-6201235310482', '-9.39e+4286127599', 3, 0);
    T('-3.009155425935019550084236112197720652792011587583595582450e-200258', '2.66671103349114758256713763057081685155942568278285704328393e-6132', '2.6667110334911475825671376305708168515594256827828570432839299999999999e-6132', 71, 3);
    T('-89765067885050359418371532532298707952069350686', '2.3654280852366247428077887e+742512618731', '2.36542808523662474281e+742512618731', 21, 4);
    T('-7.76580064359083112496017962569757651378e-29052', '4.352329e-95062119944', '-7.76580064359083112496017962569757651378e-29052', 75, 0);
    T('-9.428834799303496915825023519526673407506977807244558e-3571', '7.3204390508318584210855180334e+3206269369', '7.3204390508318584e+3206269369', 17, 5);
    T('-4.2421976058969131080725e+387497781', '-4.631834794984657979261e-1991157178', '-4.2421976058969131080725e+387497781', 86, 1);
    T('-7.28255486190457e-72009', '5.858598113279135757229027117e+9586373', '5.85859811327913575722902711699999e+9586373', 33, 3);
    T('-56693037823147.67397', '9.1922074251722665990191070267400578336795e-37', '-5.669303782314767397e+13', 47, 0);
    T('-2.0002403066372256670591239451622911994310480298852828133373e-725', '2.942436302776470883694137583435318133720576832e-43983634992383', '-2.000240306638e-725', 13, 3);
    T('3.875237750796793904488457062739104114274910633111e+41499787795', '-459610800545613400860917.7', '3.875237750796793904488457062739104114274910633111e+41499787795', 83, 5);
    T('-2.833570876738364472503264398201499e+141', '6.3043667444831498295712589199121877657970423e-443994', '-2.833570876738364472503264399e+141', 28, 0);
    T('-1.1860420398617414684226553e-194578237', '-178659408', '-1.78659408e+8', 36, 1);
    T('6.602663914622675139045861494e-62517', '9.191519452240096460741597280714912708e-5455723457', '6.602663914622675139045861494000001e-62517', 34, 0);
    T('3.8986005172462815281286731898069731415256884492e+333012', '4.23177119064208009431157377e+48510752059912', '4.2317711906420800943115737700000000000000000000000000000000000000000000000000000000000001e+48510752059912', 89, 0);
    T('-5.071194430480490916705339392522204e+5783424130464', '6.481971924177138165143375877437552280865802917045319e+984885833637800', '6.481971924177138165143375877437552280865802918e+984885833637800', 46, 0);
    T('-9.1424112446098707127610529665e-5945', '-9.259469896086348897658652856585074728879926137458171126e+78', '-9.259469896086348897658652856585074728879926137458171e+78', 52, 6);
    T('-8.2126001083929910388785999890e+742', '4.4874792293613273e-60541444857', '-8.212600108392992e+742', 16, 0);
    T('-5.46e-1811', '-9.997384723893414220230820e-51887193974', '-5.46e-1811', 64, 2);
    T('-3.471882042619500112551978659682162694998950803121e+7181', '1.939286086084901253885207795671269761367657248004642e+56', '-3.4718820426195e+7181', 14, 2);
    T('8.0e-8', '6.36803863601810856e-483', '8.000000000000000000000000000000000000000000000000000000000000000001e-8', 67, 0);
    T('-3.980512881292024088599567e+317', '-6.2289053946529314735139213749502528825e+737251610662', '-6.22890539e+737251610662', 9, 2);
    T('-2.94832408123062570364807194087700386254e+97468193', '3.366185596946760582142683572174944884487847159650901e-59426629', '-2.94832408123062570364807194087700386254e+97468193', 77, 0);
    T('2.574812231706104688395549939957466834718973730298836082e-61677', '-2.2426805e+171515809', '-2.2426805e+171515809', 36, 0);
    T('-1.136579037674905060290609654099312842062e-62397526459755', '-4.473995e+6411080159540', '-4.473995e+6411080159540', 20, 4);
    T('3.990292966601345039437759840902944200444e+26624', '-4.112879735767984861781619495515401757456406359023301574e-589736078075', '3.990292966601345039437759841e+26624', 28, 0);
    T('4.495344992726736853276651549085e+904989', '-2.1795820804585438e+17', '4.49534499272673685328e+904989', 21, 4);
    T('3.9981296324e+848907995905', '4.17646106538484711498742747094150402406567646822113350e+359', '3.9981296324000000000000000000000000000000000000000000000000000000000000000000000000001e+848907995905', 86, 2);
    T('-7.689659507255883865811394798184906611428532e+98', '2.8569090588634893519523e-76445406', '-7.6896596e+98', 8, 3);
    T('-5.72434859256799715059662881992090429233023329570e-498710035417', '2.905922796063799558299041293038546958063e+594', '2.905922796063799558299041293038546958063e+594', 97, 5);
    T('-8.5445073017660476371870279139224e-3557749376005', '1.46743389395081088871578006911436159e+57209210545', '1.46743389395081088871578006911436159e+57209210545', 43, 5);
    T('3.345542561858301005727983754024039864532246782540854e+52', '-2.22439436400e-1446470249222', '3.3455425618583010057279837540240398645323e+52', 41, 2);
    T('-1.3673580053439833641395668193988961e-793539804587355', '7.5264904213605581310893e+152232904071', '7.5264904213605581310893e+152232904071', 74, 2);
    T('2.00220725e-29943529569914', '5.9840866855624214052264196291695081637403903570363139e-71118380311', '5.984086685e-71118380311', 10, 3);
    T('5.8754003620424733972030610511863438929746385146309941e+58', '-2.505582689501833658001001472405088158376174773e+27686945', '-2.505582689501833658001001472405088158376174773e+27686945', 62, 3);
    T('-9.87250860279669336844e+8460510', '-7.68635628433474601483827113e-18028280', '-9.87250860279669336844e+8460510', 82, 4);
    T('2.8817929126489172e+512', '7.8470099752821865177084774237562593968e+41', '2.8817929126489172e+512', 86, 4);
    T('-5.4322829e+422907978', '1.4369057714850159321772690318037663673044325e+5510881166102', '1.4369057714850159321772690318037663674e+5510881166102', 38, 0);
    T('5.40713724508111218386660785364957026911e+198517', '-1.1103777841944428895672013778e+32272', '5.40713724508111218386660785364957026911e+198517', 67, 0);
    T('2.54692346110071008465753741955438883850197952e-2601075386374', '1.1747153e-9209', '1.17471530000000000000000000000000000000000000000000000000000000000000000001e-9209', 75, 2);
    T('-9.99973084749e-709195', '2.08010018881857842155468668971571402e-4258792942', '-9.999730847489999999999999999e-709195', 28, 1);
    T('-1.85661582401515706984017123203326300301473160e-70294451', '7.8651572296300419375e+71', '7.8651572296300419374999999999999999999999999999999999999999999999999999999999999999999999999999e+71', 95, 1);
    T('-3.1236069533323957324673554060744055665694143325004449989e-1295', '-5.19911195e-32995076963569', '-3.13e-1295', 3, 0);
    T('-1122060132963392242946762059589265340568071079398195.9', '63083286314574948873541798663.0912157', '-1.1220601329633922429466989763029507656191975375995328087843e+51', 93, 6);
    T('-5.3392309323569657150359389215581702538705846405928600928395e-35150770', '4.402018375727016231552823132440674176573e-72183394030421', '-5.3392309323569657150359389215581702538705846405928600928395e-35150770', 64, 0);
    T('2.12152163194595809780504610092758e+674816', '1.44770558242655094501056968865213010875928263149066267683e-89567743917382', '2.1215216319459580978050461009275800000000000000000000000000000000000000000000000000001e+674816', 86, 2);
    T('-9.17e+296992611', '9.664806504460398793266002681426865092743919493606994e-5025651', '-9.17e+296992611', 42, 6);
    T('-6.93811661690788692187449893326268326352426886492e-622289112', '-3.00e+499655', '-3.00000000000000000000000000000000000001e+499655', 39, 3);
    T('8.90078463378145978093679135900e-242414761', '8.2204136256882914256899330464127e-879881099', '8.9007846337814597809367913590000000000000000000000000000000000000000000000000000000000000000001e-242414761', 95, 2);
    T('2.474224464562056148325880e-18900408337831', '8.98276530435087534450854590245451e-8775138183328', '8.98276530435087534450854590245451e-8775138183328', 59, 1);
    T('-7.325390870513949781126492172348571815793655719e+514908529', '2.3043045245312081897302656126376674421296180383168962e+178741', '-7.325390870513949781126492172348571815793655719e+514908529', 61, 5);
    T('5.1390329089860758995625139307882333694810216648908043363269e+7460321', '4.7926468200611514198268039984423713216828813e+61472162281', '4.7926468200611514198268039984423713216828813e+61472162281', 69, 1);
    T('-4.2803850930892581932134323437290884035439178966959574e-976826', '-4.946074447307321768309434848078730668835365558684302685672e-1120', '-4.946074447307321768309434848078730668835365558684302685672e-1120', 92, 5);
    T('1.44521965937281951032477820511025638127643847140137599910130e+1521', '4.987973144672765372104624845362355403244747491105023841713e-54201992', '1.44521965937281951033e+1521', 21, 2);
    T('-9.817e+8', '2.9533658104165457046036379139223422685073165e-18010023418519', '-9.817e+8', 48, 5);
    T('-3.27388847117276519103036462245464167193440277201442e-16027494850', '-5.3411892264839523e-279235', '-5.34118922648396e-279235', 15, 0);
    T('-7.1e+482919144627261', '9.814871313969126498068925043402848587681019834305619529e+58954489095225', '-7.1e+482919144627261', 65, 4);
    T('-1.26497e-241983303198970', '-8.32500157501076161830341125335636698903765016e+87', '-8.32500157502e+87', 12, 3);
    T('-4.870750168548974288506764166e-93926865325', '-8.8057824489610726349492710875644e+31791545525803', '-8.8057824489610726349492710875644e+31791545525803', 50, 6);
    T('-1.083499059739447574086080919076060935443915718283164e-71525', '7e-456831853404', '-1.08349905973944757408608091907606093544391571828316e-71525', 51, 2);
    T('-1.77181430878236151328387e+27', '-5e-43056', '-1.771814308782361513283870000000000000000000000000000001e+27', 55, 3);
    T('7.98665626779409101647731348074355239090452538675554e-42466494686', '1.93692650793647210889889211230283289116353351354284e-794355814', '1.93692650793647210889889211230283e-794355814', 33, 1);
    T('-5.5916015089155428268020423e+28', '-4.0219815329056e-52476480555539', '-5.5916015089155428268020423e+28', 90, 5);
    T('5.624507606948383433538824125494132902812980199685509e+773785255767214', '4.157e+93593416924035', '5.62451e+773785255767214', 6, 6);
    T('8.9006609318833220848889700677991236372073071712530538437e-8118816', '-8.26e+348', '-8.259999999999999999999999999999999999999e+348', 40, 1);
    T('9.247615530190080717365793988741345925618557e+909616741228239', '4.8712410022692645499658461669297779357059024e-68566470415622', '9.24761553019008071736579398874134593e+909616741228239', 36, 2);
    T('4.42927496333862779291029664522345374779271e-89509', '2.22151607995845e+71169955889', '2.2215160799584500000000000000000001e+71169955889', 35, 0);
    T('6.757697253504022479203223744926104047887461e+999600580303', '-4.81736722593124e+4886', '6.757697253504022479203223744926104047887461e+999600580303', 99, 2);
    T('6.41580149091216996163495438e+32', '-1.846096852901244851330433124737e-4485', '6.415801490912169961635e+32', 22, 2);
    T('-6.495948999349645898286441800109891628306756735597e-663625862', '8.966e-13385', '8.966e-13385', 4, 4);
    T('-8.87548960789793411614305135708232353256e-55', '-2e-613', '-8.875489607e-55', 10, 1);
    T('-8.12641008009387770489916869921328442510e+452438611', '9.4533781406997512277277776824537305422728e+75373692581', '9.4533781407e+75373692581', 13, 6);
    T('-1.8001789e-5646653', '-6.7557641754292462757612974454089406804621991891498e-436663', '-6.7557641754292462757612974454089406804621991891498e-436663', 99, 5);
    T('5.9687346763935271529427539683057e+49440390300712', '6.6845579265488e-51419174574730', '5.9687346763935271529427539683057e+49440390300712', 35, 5);
    T('-6.89876658831627342331334260e-453472403862', '8.531474892847100644511963829e+23237795378802', '8e+23237795378802', 1, 1);
    T('-7.27205262621457856336236552875156e+5249657367', '-6.4365184e-3567801563182', '-7.27205262621457856336236552875156e+5249657367', 57, 1);
    T('3.9936207278913939786679944825457294328945744710568575846e-716231551643044', '-4.17566e+872784053244', '-4.175659999999999999999999999999999999999999999999999999999999999999999999999999999e+872784053244', 82, 2);
    T('-4.1714657407868159759647791065221e+6177', '-3.72191125773296935377e+7773935802', '-3.72191125773296935e+7773935802', 18, 5);
    T('-8.1938316556048e-638855643107', '-4.59712564865310867261221267301717818e-9514178', '-4.59712564865310867261221267301717818e-9514178', 37, 4);
    T('2.669255941138159920279465761345911e+71098', '8.4079538514702784062332698562992237851e-186', '2.66925594113816e+71098', 16, 6);
    T('2.3699723221640360e-91387', '3.2933050164897809261204700992146284579739083041424866e+999697472779', '3.2933050164897809261204700992146284579739083041424866e+999697472779', 68, 1);
    T('-3.73040211287375825346873150864431444e-296192', '-1.5024586485620461418425860425419333e-145121', '-1.5024586485620461418425860425419333e-145121', 72, 6);
    T('3.8367795261020964102712515983585417447332232987107e+36354', '8.3012345244e+15915', '3.836779526102096410271251598358541744733223298710700000000000000000001e+36354', 70, 2);
    T('7.28014191743518974670268219022500859252055361891e-545190758112', '-4.4819843379994555197327788335646875250472584295550822e-2923344', '-4.48198433799945551973277883356468752504725842955508e-2923344', 51, 5);
    T('3.66019713680419722990961094927769110135869110006e+30085139202348', '5.0935827668474959579363008939384831788081945340056e-437547866529224', '3.66019713680419722990961094927769110135869110006e+30085139202348', 81, 1);
    T('6.3080841275451924e-41', '7.13191725040102696102960592055776921389064e+1439', '7.131917250401026961029605920557769213890640000000000000001e+1439', 58, 2);
    T('5.71783339927206129532603450875e+2952664514909', '-2.460110870329919114519603604452e-49126071201', '5.717833399272061295e+2952664514909', 19, 1);
    T('5.7821617860917995906290147998361514630286606214642779778e-5469', '1.48533734255074997110102e+83060030785182', '1.48533734255074997110102e+83060030785182', 62, 3);
    T('8.20483492862073114660016642025980351323623467019839499e-1931466027058', '3.3003436213530130515954159e+88950964690', '3.3003436213530130515954159e+88950964690', 50, 6);
    T('2.978817013533776e+23', '-9.55604960978726e+2963870988754', '-9.55604960978726e+2963870988754', 86, 5);
    T('7860756.83328', '-4.250589180756e+46544396183246', '-4.250589180756e+46544396183246', 16, 4);
    T('-6.6892012340490846452972930064682303906438312114181114033e+117', '-8.809447709475810861064928035999803958716566836444174646e+40482', '-8.809447709475810861064928035999803958716566836444174646e+40482', 93, 5);
    T('8.41066322346505299237699124362967315754943234761518e-64798530603', '2.7629894678491002322e-81637321783334', '8.41e-64798530603', 3, 6);
    T('1.5888223915425013438946231390927973811e-255326', '-82150522615.98352717412308325181859315541073240183786', '-8.215052261598352717412308e+10', 25, 5);
    T('-9.97913855399398603238e+9626844683', '-8.318354314e-83443528', '-9.97913855399398603238e+9626844683', 33, 6);
    T('-6.871982302007124e-34026844926', '-2.984478440603071140947849512662740353354739269896510e-86824527529', '-6.8719823e-34026844926', 8, 1);
    T('9.89613022172693278296067410811e+42211842523', '7.5319092386025551340822215994570878992289e-763024843645208', '9.8961302218e+42211842523', 11, 2);
    T('9.4995819404006502580944599338504112164428935911685982056593e-196066543071444', '-6e+9010', '-5.9999999999999999999999999999999999999999999999999999999999e+9010', 59, 2);
    T('-7.399765979562011269720e+34558', '-6.5068289041421790964579e-958370289310', '-7.39976597956201126972000000000001e+34558', 33, 0);
    T('1.19645102479951077125803604682710720482505359762702e-728680079032', '3.90754834183483513206726530923391540193645207e+2485', '3.9075483418348351321e+2485', 20, 2);
    T('-8.5547404859202264999599510355179922e-264451601770', '-5.030282088567115174e+60793575', '-5.030282088567115174e+60793575', 88, 5);
    T('-6.81115594908610970881330e-2782016985260', '-9.40688e+9334207170598', '-9.40688e+9334207170598', 72, 4);
    T('2.35084788005728277201386511938790355786081211e-9535370', '-3.50050336929378431765341123188e-68455295525', '2.35084788005728277201386512e-9535370', 27, 0);
    T('-8.8385274581211702090431207048019114401960848840196452118858e-90675116564427', '-5.054181156535665674771021389760293777908061987107e-762', '-5.054181156535665674771021e-762', 25, 4);
    T('2.3360842418318637e+7736', '6e-927410900', '2.3360842418318637e+7736', 28, 3);
    T('-6.266344135873322165267724752614587039936659192362849554e-605544148846', '3.6397412481150587608151984741341734e+53675', '3e+53675', 1, 1);
    T('-7.9169333797840307124401e-2989953121', '5.351482715299923928224801533467084285450953154819415e+433456304492', '5.3514827152999239282248015334670842854509531548e+433456304492', 47, 1);
    T('5.05418734727812066441994136437261452399134e-778583158227702', '5.0411338825843501703454613185e-469', '5.0411338825843501703454613185e-469', 87, 1);
    T('6.781e+113702359', '-2.4762202670032008358690648481217306666778659269e-202594901', '6.781e+113702359', 11, 4);
    T('-2.97272635429762e+732212702454', '-9.08273645736938305491786078801847e-66678511087', '-2.97272635429762e+732212702454', 51, 5);
    T('3e+9819', '3.508389063316e-8988002655966', '3e+9819', 55, 1);
    T('-8e-15573', '-1.880478812763428371078082426531e+323130897584', '-1.880478812763428371078082426531e+323130897584', 61, 6);
    T('2e-913182067', '2.05048158945e-8483190079807', '2e-913182067', 70, 5);
    T('3.94971878264117685142079229129541788191771565951795e-798628059', '-2436205651512005795493.59860', '-2.43620565151200579549359859999999999999999999999e+21', 48, 1);
    T('5.8900701513e-497951', '-5.912752337646650140418860637736118482035310333481901e-350013', '-5.91275233764665014041886063773611848203531e-350013', 43, 5);
    T('8.760238520049542532154e+47668', '7.69708817e+183', '8.7602385200495425321540000000000000000000000000001e+47668', 50, 2);
    T('-3.2417425645243171978001688e+29429371004', '4.33613966973394946763755183995809418802980090774952813e-58887', '-3.2417425645243171978001688e+29429371004', 94, 5);
    T('6.371614254431e-4012294027856', '4.441106631428890598397269312640238786225019981e-5996756', '4.4411066314288905983972693126402387862250199810001e-5996756', 50, 2);
    T('-2.1532307114428925565071557220e+6972444', '2.4706543327e+246216498950', '2.4706543326999e+246216498950', 14, 3);
    T('8.25173683539325095589562260368937e+36108165798', '2.43432490943525770031249853474519118319066258108988364870e-8251002776', '8.25173683539325095589562260369e+36108165798', 30, 5);
    T('2.740199232972339116608092132663295014e+88817706', '-9.22047975504321063810548236285376267564209107361221e+120', '2.740199232972339116608092132663295013999999999999999999999999999999999999999e+88817706', 76, 1);
    T('6.9099980200400504356786e-383400495082', '-9.0298532320660470512547599515272885576285354069983863e-3421785012', '-9.029853232066047051254759951527288557628535407e-3421785012', 47, 5);
    T('9.01e-95', '4.056418214758289400188199987e+42358305', '4.0564182147582894001882e+42358305', 26, 2);
    T('-1.082458729309315827453330699e-777351', '-4.212305132411912552631676770317313226938243545e+51', '-4.212305132411912552631676770317313226938243545e+51', 96, 2);
    T('5.74085690410009179907435495513407534447261617e-6168', '-1.002278401989136506880335773909174491919405873162e-722807', '5.74085690410009179907435495513407534447261617e-6168', 58, 5);
    T('-4.205200438119420162980623202593235103676955439e-439784390', '6.252161968340410917e-240', '6.252161968340410916999999999999999999999999999999e-240', 49, 3);
    T('3.00363943230605508883383141577712686950487e-18718280619152', '3.456835175869812938418e-61965', '3.4568351758698129384180000000000000000000000000000000001e-61965', 56, 2);
    T('1.062208214383855880263726662e+139822785346', '5.798943969559e-54554850135967', '1.062208214383855880263726662e+139822785346', 61, 5);
    T('-6.755618469150237716146566112257e-1742832496', '9.83533704846401830518448086272506434e-9505799297', '-6.755618469150237716146566112257e-1742832496', 98, 5);
    T('9.39335891841538574875368819138743615789114000266931630359582e+67407110386053', '-3.531062802481948216e-9454886207485', '9.39335891841538574875368819138743615789114000266932e+67407110386053', 51, 4);
    T('-1.41238767452235908086774471370563236202805386e+349225763', '-3.1759750086691588e+3604', '-1.41238767452235908086774471370563236202805386e+349225763', 88, 6);
    T('4.67379869121753e+5487', '1.0436742552545977745585e+4797597205', '1.0436742552545977745585e+4797597205', 73, 1);
    T('822597949655170571719751268426314.7535', '-2.784681517685416852734e-86027524655559', '8.225979496551705717197512684263147535e+32', 58, 5);
    T('-6.781978442670986528446969967941e+97814183547', '8.588063415428992702537332349496143318146266e+685', '-6.781978442670986528446969967941e+97814183547', 79, 6);
    T('-4.61997282558185452917612317666333369e+79193957954421', '8.616912904428157170267602321049051295520548340458115934e+94', '-4.6199728255818545291761231766633336899999999999999999999e+79193957954421', 56, 2);
    T('1.20316151069416478184e-31813986129203', '-4e+3926982788', '-4e+3926982788', 23, 3);
    T('1.541227435570687514343053124e+1709', '5.1235833712476412817212595574675583640073437858217833892e-71581729', '1.54122743557068751434305312400000000000000000000000000000000000000000000000000000000000000000000001e+1709', 99, 0);
    T('1.74208588029385572330540828956473109259967692e+2407126426', '-7.83065e+702', '1.742085880293855723e+2407126426', 19, 4);
    T('6.15504465334632e+13497915237998', '-992136213.606385121', '6.15504465334632e+13497915237998', 78, 4);
    T('-4.074805866388909180232401e+980490354791', '-6.9186477664923973776291514086704786801341179e-27556', '-4.074805866388909180232401e+980490354791', 26, 2);
    T('5.8669664876935525174021e+17831511387699', '1.561050970491278e+16', '5.866966487694e+17831511387699', 13, 2);
    T('4.06301330e+66292842', '-6.472439413e+31764142284', '-6.472439413e+31764142284', 33, 3);
    T('-9.416e+69692201371', '-9.58800256874884609803017828333447088255568e-1872557315235', '-9.416e+69692201371', 15, 1);
    T('-2.552113561787879e+613', '3.433087719260960815875975346030874208442e+2385', '3.433087719260960815875975346030874208441999999999999e+2385', 52, 3);
    T('8.72690176570637484751e-8671937305945', '8.351309060e+6297', '8.35130906000000000000000000000000000000000000000000000000000000000000000001e+6297', 75, 2);
    T('3.01008188479077149382102873868426549e-827173166631873', '4.99733534e+145789', '4.997335340000000000000000000000000001e+145789', 37, 0);
    T('-8.53551114368588340176526513098982135557095e-49435572673662', '-4.39188772237220926230113828833586011695843e-159472017565', '-4.39188772237220926230113828833586011695843e-159472017565', 56, 4);
    T('5.70151289494858928364227111098251333e-487', '-3.07341884643e+23922354841403', '-3.0735e+23922354841403', 5, 3);
    T('8.2638862739325673511e+894213', '6.0300938102038476938655193547655982869673188019415114392396e+110462', '8.263886273932567351100000000000000000000000000000000000000000000000000000000000001e+894213', 82, 2);
    T('-9.423298091792275448935431041457537794978597938997867120e+130', '-4.19499846051723349396083851929102338329804617473e-162156515', '-9.42329809179227544893543104145753779497859794e+130', 45, 4);
    T('94735416248266917066201564454703191757620070660256', '7.9e+90', '7.90000000000000000000000000000000000000009473541624826692e+90', 57, 5);
    T('-7.9655017811253571335313382832987292606008890761110e-7190', '9.569525253081689715274495119431834718e+37', '9.569525253081689715274495119431834717999999999999999999999999999999999999999999999999999999e+37', 91, 1);
    T('3.91958798725029882894968943781461167080858829573915151308818e+20330824301', '-4.6310759400352172364111e+483334298', '3.91958798725029882894968943781461167081e+20330824301', 39, 6);
    T('-7.44737539710839451652e+51584380380', '-2.6728164517463465286744342967520456002104e+4723361233', '-7.44737539710839451652e+51584380380', 28, 6);
    T('-7.215946228124e-1727847', '-7.557012545885823585750592238505286424002316227e+84511837220', '-7.5570125458858235857505922385052864240023e+84511837220', 41, 1);
    T('-3.56e-1816', '-8.681136356511289e-440904', '-3.56e-1816', 39, 5);
    T('8.596499376528874337478724185814947999126878e+138', '2.306376923839418716310327021920795717305893961142651e-930067', '8.596499376528874337478724185814947999e+138', 37, 1);
    T('-8.68282850846717820972963377790008828371750947065111636420e+24100479823394', '-1.2218709322033082435646735639203472901598545283027e-25832355', '-8.6828285084671782097296337779000882837175094706511163642e+24100479823394', 62, 2);
    T('-4.4287373562037669768738683502097462737712166e+10372721813414', '-7.53348125e-715448713', '-4.4287373562037669768738683502097462737712166e+10372721813414', 83, 4);
    T('-6.370059145506641570606511661367e+84974543', '2.088652e+846953243383963', '2.088652e+846953243383963', 27, 5);
    T('1.2460484064888326483102892643614416733455249120e-16684806', '4.882264622767224519339004256193047158e+50699418104822', '4.88226462276722e+50699418104822', 15, 4);
    T('8.73889721917437907088215495680208063381723407676428487853e-40872025161513', '-9.295979e-2025', '-9.295979e-2025', 86, 3);
    T('8.156383e+571830047', '-8.7827e+4183015379070', '-8.7827e+4183015379070', 82, 4);
    T('-5.64833923372418181496271111363526352890638652268364732058882e-6106819106207', '6.123709206951733423915247064006125683708705000923e+872394989054056', '6.123709206951733423915247064006125683708705000923e+872394989054056', 86, 4);
    T('-1.242174595821932262245803581616626520963469084141607e+6545167', '-7.523160e+2823863147386', '-7.523160000000000000000000000000000000000000000000000000000000000000000000000000000000000000001e+2823863147386', 94, 3);
    T('8.056612153849966689483215945705720263028964617561221162945e+7983973060147', '-7.67673684096927427463276252380014099272550683974803112012421e+170298025903', '8.05661215384996668948321594570572026302896461756122116294499999999999999999999999999e+7983973060147', 84, 3);
    T('4.977414763324429348327364052269318873916851205e-43603873472', '5.7747909339607019307078844001337915922167227227559388826e+1011', '5.77479093396070193070788440013379159221672272e+1011', 45, 4);
    T('-6.2263485938175529733014802124457954555e+53804784', '5.5080839304765756284117741730983298862744188801697361e+7431829', '-6.2263485938175529733015e+53804784', 23, 0);
    T('2.6304568661639904805036e+1003', '-3.3635686870125704e-518587180747293', '2.6304568662e+1003', 11, 0);
    T('7.457152263e+3846392514', '1.6202943501118774055665301765904738129037766567240745e+53', '7.4571522630000000000000000000000000000000000000000000000000000001e+3846392514', 65, 0);
    T('2.896627937134320365221575065874972211236519e-9045456491', '-2.24e-229017809', '-2.24e-229017809', 42, 6);
    T('4.048861e-180', '-2e+409734604', '-2e+409734604', 7, 0);
    T('-3.2e+5743658300697', '-9.9716448640533946863880872625278958754583446147956092914e-36469', '-3.2e+5743658300697', 7, 6);
    T('-2.0853038425995178946682605450222180611084886495471371775627e+79908', '-5.087450779459545900393984185877083926880288341721600060e-471325478665438', '-2.0853038425995178946682605450222181e+79908', 35, 0);

    T('-9.450381389967387297359265506650E+9326310698', '-4.89684633056248366418040660831389714084633087293253564E+86891672032934', '-4.8968463305624836641804066083138971408463308729325356400000000000000000000000001e+86891672032934', 80, 0);
    T('-4.8E+398571460422645', '7.1145806380920668978254132839900324023953323364108722776996E+783353', '-4.8e+398571460422645', 36, 3);
    T('4.67023090266403752623070887972203264571153488814E+81222909188150', '-6.8217470001458455710845856E-458446639', '4.670230902664037526230708879722032645711534888139999999999999999999e+81222909188150', 67, 1);
    T('-2.9598008909601892E+710828448829', '3.6669618496469263939458086034344605741943E-75898', '-2.959800890960189199999999999999999999999999999999999e+710828448829', 52, 1);
    T('-9.67821035E+575958114', '-9.949942E+732', '-9.67821035e+575958114', 45, 4);
    T('-2E+779264', '5.6096427910265253616573943296252097090951644246195625992E-3345', '-2e+779264', 77, 3);
    T('-9.92621534994E+27', '-1.473522712788392697311636163602178112009E+8066318343134', '-1.4735227127883926973116361636022e+8066318343134', 32, 5);
    T('-7.0570163873069050392235298215501946271E-410374236334', '2.1969096116064691684684556590414243E+144548', '2.196909611606469168468455659041424299999999999999999999999999999999999999e+144548', 73, 1);
    T('-1.50252437536674518434400029132995499168172256351055968126078E+6425653', '1.1996528283970081866896689855E-341089244585020', '-1.50252437536674518434400029132995499168172256351055968126078e+6425653', 64, 4);
    T('1.617090526253726044022186437053E-3324570214', '2.2909039892834858416823970695062690E+65', '2.290903989283485841682397069506269e+65', 73, 5);
    T('-2.78298E+1618829753669821', '4.61687E+9938', '-2.78298e+1618829753669821', 6, 6);
    T('-608772.619415685080343285124343360818892877614225506989254134', '-7.8558670488401743635771E+873', '-7.8558670488401743635771e+873', 41, 5);
    T('3.09901993100998580081236595058793544325241061E+447199514226777', '-8.9690858136895E-9333043', '3.09901993100998580081236595058793544325241061e+447199514226777', 71, 0);
    T('-4.453577628052143998654913261539810014632695E+4938', '-1.21696929674E-455', '-4.453577628052143998654913261539810014632695e+4938', 67, 5);
    T('-8.002017E-9790242784514', '-9.72663991842622864520084170489239425590E+991926872', '-9.7266399184262286452008417048924e+991926872', 32, 0);
    T('7.91195200860948130464974360289588091279097976635E-1032479193677', '-7.65258965633135609677E+6428906159523', '-7.65258965633135609677e+6428906159523', 26, 3);
    T('-8.92175043391021159499657292E+6254272547258771', '-9.9095147E+2062', '-8.9217504339102115949965729200000000000000000000000000000000001e+6254272547258771', 62, 3);
    T('6.093398652915572307458E-24195', '-8.91536297497E+953512', '-8.91536297497e+953512', 49, 5);
    T('-5.003801265716133262974401402506384150578E-83325', '-1.67044063572654395327317679502166E+5759', '-1.670440635726543953273176795e+5759', 28, 2);
    T('-1.2524293247247309935800846383810172740496025264009432526591E-23505659597110', '-4.55091749275843702162726178822725799751815240482942591E+419044', '-4.55091749275843702162726178822725799751816e+419044', 42, 0);
    T('-8949039.50570578248440', '-4.39572E+198', '-4.39572e+198', 54, 5);
    T('-6.4E+43', '-7.5540894295204554806E+35062839', '-7.5540894295204554806e+35062839', 69, 2);
    T('5.8055804622941146710435767441985700668361578819653E+7471351450473883', '-3.193096413532422604869696600051718010420382E+577', '5.805580462294114672e+7471351450473883', 19, 0);
    T('8.6216958761267742629392235893983816098391881401059951E+6911193071119', '3.467290368963244152314868E-9456253139379', '8.621e+6911193071119', 4, 1);
    T('4.7952900241322408848329746372934036675739886253996E+81418', '4.160072763554931292788193579548769799408974476580839197699E-1519667571', '4.7952900241322408848329746372934036675739886253996e+81418', 51, 5);
    T('-1.50987848676773964E+13603118595', '1.7185280395388758296253151871606809218901175785E+596197086', '-1.50987848676773964e+13603118595', 32, 5);
    T('-5.19990489E-7132831', '-1.16524734E+70105250359521', '-1.16524734000000000001e+70105250359521', 21, 0);
    T('1689.65062577771502485100618', '7.69285E-92', '1.68965062577771502485100618000000000000000000001e+3', 48, 2);
    T('-1.62973818485353456704839264E+33971307346721', '-8.7197630434946122779012244895637770740125730607E+2132469009', '-1.6297381848535345670483926400000001e+33971307346721', 35, 0);
    T('9.90901211084203352553336198148032185815461E+9484047884527', '-3788.8033412', '9.90901211084203352553336198148032185e+9484047884527', 36, 1);
    T('5.6394432096592699978134E+80', '2.22313302954721016866269394182274537711064022642161394298544E+481', '2.2231330295472101686626939418227453772e+481', 38, 2);
    T('-2.45617E+9950924610', '-7.45297793246182722780179266490903167709137235051437440565768E+915362443', '-2.45617e+9950924610', 13, 4);
    T('8.3686271994776901010788314571895819504983729715E-57994198448', '-3.885949641293908156E+48844', '-3.8859496412939081559999999999999999999999999999999999999999999999999999999999e+48844', 77, 1);
    T('2.11954573448351757121E+9528389170', '4.6522283492245681062849703959028967135061332E-4483563787283', '2.11954573448351757121e+9528389170', 47, 5);
    T('5.582824095846279607377403378153865789289E-97', '1.380746120055628640216307116880176397E+302781', '1.3e+302781', 2, 1);
    T('9.87507972859630651594919379307781102505041552065E+6829295571', '-5.62677130484558571558570517221157844137430568695228422938806E+66', '9.87507972859630651594919379307781102505041552065e+6829295571', 67, 6);
    T('-4.1956058293874094957111E+60', '5.6432699137423E+764084729', '5.6432699137423e+764084729', 39, 0);
    T('2.621030295407445645335E+2282064860947798', '-1.512377079776903003857279790286373557958028046E+157', '2.621030295407445645335e+2282064860947798', 31, 5);
    T('2.67E+65', '9.972714140933982756273555655952334E+550025458125480', '9.972714140933982756273555655952334e+550025458125480', 43, 5);
    T('5.871461E+81', '2.8721909228870130641792558090441067814980182454312116E+5428', '2.8721909228870130641792558090442e+5428', 32, 2);
    T('3.065244193345543058856677303227213822116350940189401E+20014025719508', '-3.91640933767071436513652707605598E-733308115598346', '3.06524419334554305885667730322721382211635094019e+20014025719508', 48, 6);
    T('6.513053E+189743828257349', '-7.583108357711323588759013258039974889824822E-52435', '6.513052999999999999999999999999999999999999999999999999e+189743828257349', 55, 3);
    T('7.623054433075552055519E+81825369331696', '-7.30781698107650047394180873645E+83120', '7.623054433075552055519e+81825369331696', 75, 2);
    T('3.8337535348605506582246682264E-117134561837', '6.867580E+2578719', '6.86758e+2578719', 18, 5);
    T('-8.89038992886308407401941E+432186627455457', '-9.7076E+5544657700380048', '-9.7076e+5544657700380048', 36, 4);
    T('-5.33893249396535562921624099908206360173E+71', '0.005970976151396180', '-5.33893249396535562921624099908206360173e+71', 56, 0);
    T('6.99836170834437946776745885130472575491379845970E+960', '-8.92471665169612405402776462438275603829E+9823948', '-8.924716651696124054027e+9823948', 22, 1);
    T('5E+8327042456572', '-4.023683576094220603991538603827587960021644151996479E+25484981726858', '-4.02368357609422060399153860382758796002164415199647899999e+25484981726858', 57, 1);
    T('-1.837468690096581299684057907453E+206113038712', '9.9893302738114010148151235739763818502691372515852057E+31493765', '-1.8374686900965812996840579074529999999e+206113038712', 38, 1);
    T('1.92864662861537421135029716E-50496917', '1.8889913102672047555267437068346370339459E-1629607722', '1.92864662861537421135029716e-50496917', 67, 3);
    T('3.060678891010581012760E-748894476', '-2.11698991771738851185726673266610835558493686907378496E-1468971', '-2.11698991771738851185726673266610836e-1468971', 36, 0);
    T('2.0887637479699751553741837992124662E+34798655184', '-0.00329836947653850632402298033791272732400612544673861473672676', '2.08876375e+34798655184', 9, 6);
    T('-9.16E+2585564593', '6.33489634060243990E+534116', '-9.16e+2585564593', 54, 0);
    T('9.2467579633715985248535688204436795924E+211011875093042', '3.029258845177665254837741855041243418170658271E+116', '9.2467579633715985248535688204436795924000000000000000000001e+211011875093042', 59, 2);
    T('7.8858983012265951231368074151859786467687225536E+5456807257272888', '-1.57646740337885292224E-1972995789', '7.8858983012265951231368074e+5456807257272888', 26, 3);
    T('-1.5976776272386493713321899652319584655902188437781124E+4606059686625', '-3.082E-69', '-1.59767762723864937133218996523195846559021884377811240000000001e+4606059686625', 63, 3);
    T('8.6923203324171992283021118680365383783831535263826514986E+58', '-2.374111099E+514305', '-2.374111099e+514305', 29, 4);
    T('-9833687182.606029', '-9.304960648E+938638', '-9.304960648e+938638', 42, 1);
    T('-7.2945403870177151548314E-4757140374', '-113888.3961559384063424763533996980', '-1.139e+5', 4, 4);
    T('-2.813177988056600361507393112E+7135', '3.273129484E+35342', '3.273129484e+35342', 78, 5);
    T('-3.2392810519153622414456494E+95574520679', '-8.10341495750484333041E+49352096193934', '-8.1034149575048433304100000000000000001e+49352096193934', 38, 3);
    T('8.95217929859993214631E+7882', '-8.1455037315313831520346954403187540660276952E+662894786', '-8.145503731531383152035e+662894786', 22, 4);
    T('-5.7882382E+7947972928', '-2.017176290679218269257823029703744266332603E+4157808796', '-5.788238200000000000001e+7947972928', 22, 0);
    T('6.5967461699326354097E+239051716799', '-7.3653229927063966871104E-6374760', '6.5967461699326354097e+239051716799', 56, 5);
    T('-6.793473012683230713344580E+54', '-8.55519945889277499107850753597022772080E+301331097574', '-8.5551994588927749910785075359702277208e+301331097574', 79, 6);
    T('-7.1860091409431363389396973238E+50105732532744', '-4.930416592294248633E-925', '-7.1860091409431363389396973238e+50105732532744', 47, 4);
    T('823921058.43708455280292639703999735444700215', '-8.91640151032475157125768259477051156396043200812783581184E+4135', '-8.916401510324751571257682594770511563961e+4135', 40, 0);
    T('-1.496603862057001512791639686251751504380356184504479686959E-191546663440317', '9.85875413151987440829637234691918035272E-50510', '9.8587541315198745e-50510', 17, 2);
    T('3.8627467812944761703019742708076114681984136798991095E+85', '6036732923288207503.79925622220326056184278879745', '3.8627467812944761703019742708076114681e+85', 38, 1);
    T('-2.66409165748020667185349264E-7905896', '8.86797757389926338125072773E-1139644357', '-2.66409165748020667185349264e-7905896', 39, 4);
    T('2.466814020090881302982936703E+1908696797', '9.8723144771332255925760594868114548741600187E+42895469603', '9.8723144771332255925760594868114548741600187000000000000001e+42895469603', 59, 2);
    T('-3.466545755896130818812E+259968858234675', '-4.49529015472519581565037696582704E+992546212', '-3.4665457558961308188120000000000000000000000000000000000000000001e+259968858234675', 65, 3);
    T('5.3737806806155E-905', '1.71628550E+123', '1.7162855e+123', 74, 5);
    T('-3.00442E+269679778221', '-9.930E+49516944549123', '-9.93e+49516944549123', 31, 1);
    T('2.89224714687606849582253689560262340091E-271642895', '8.01728006314550747230351831711704883592922708262179663E+2207', '8.0172800631455074723035183171170488359292270826217966301e+2207', 56, 0);
    T('2.80091924547257E+689308777', '2.4E+8472288444130', '2.40000000000000000000000001e+8472288444130', 27, 0);
    T('-2.83137189631420272373436412202664997972E+896905298', '-8.1862499488059517967550954528990981123626E+5405975352063524', '-8.1862499488059517967550954528990981123626e+5405975352063524', 44, 5);
    T('9E+3761', '-4.4570778727265939378E+898', '8.99999999999999999999999999999999999e+3761', 36, 1);
    T('-3.5023113297782599517652328672867E-25342507889957', '7.71831759820951974962226933169867634278E-5921', '7.718317598209519749622269e-5921', 25, 3);
    T('2.450716217226230848523312656219920E-213927844', '3.13761673285884131343391231245328782E+84697', '3.1e+84697', 2, 3);
    T('-4.61701440003740067743146E-9326', '-7.173764052613585878029006227029912313247674439E+523', '-7.173764052613585878029006227029912313e+523', 37, 6);
    T('-1.7E+8969905762845', '-2.49935485890299443548E-9', '-1.7000000000000001e+8969905762845', 17, 0);
    T('6.3830204360583303126461086313005111E+34193', '3.3865121842940965395003401704582009368782872848556946449621E-731', '6.38e+34193', 3, 1);
    T('-4.08003211E+4382959', '-5.137621E+1256648387', '-5.137621e+1256648387', 34, 5);
    T('-7.33269830247704584467972046024E-745', '2.5976222126781103882E+8986947944', '2.5976222126781103881999999999999999999999999999999999999999999999e+8986947944', 65, 1);
    T('6.262914E+9', '-4.0768818943331395032783637067548E-401700', '6.2629139999999999999999999999999999999999999999999999e+9', 53, 1);
    T('-3.4106784978826889169356139324702370E+44454', '6.8364632237438667706838572160408019267985597336834886965188E+91', '-3.410678497882688916935613932470237e+44454', 49, 5);
    T('520269748.20028058', '-6.997956131481322766390841734250303E+7721550882227', '-6.997956131481322766390841734250303e+7721550882227', 70, 4);
    T('2.781451251655155324E+85448', '6.3323522281612E+35682856', '6.332352228161200000000000000000000000000000000001e+35682856', 49, 2);
    T('3.6841642E+9', '-9.32577E+6156565155', '-9.32577e+6156565155', 40, 0);
    T('4.7967864E+423', '2.32383399078713497566698E-2600533597254048', '4.7967864e+423', 33, 4);
    T('-6.7244345307575E-288', '7.59098942405472906505E+9959', '7.59098942405472906505e+9959', 24, 4);
    T('-2.050908427859103355538723067972135759909890873858257E+541535', '3.6871331576539097379987060588754814577444045300E+552783003', '3.687133157653909737998706058875481457744405e+552783003', 43, 0);
    T('-4.6873307263743228871225313878056338605216106E+59935712633000', '-4.731776401657028115282494110829514980542315029429149334187E+67662567', '-4.6873e+59935712633000', 5, 5);
    T('-2.7513E+6274730116194', '-7.980E+91364581953070', '-7.98e+91364581953070', 36, 1);
    T('-45.765965640434421476742954719042', '1.2435294429141050306E-5465972', '-4.5765965640434421476742954719042e+1', 42, 6);
    T('45.42178111069397909404722134', '-4.29663178559727017905411232189865954443996E-89861775', '4.542178111069397909404722134e+1', 71, 0);
    T('7.94791254478573062013E-36641499454', '7.4571624501635667289362038132398942229020837706189678555297E+5359561357', '7.45716245016356672893620381323989e+5359561357', 33, 4);
    T('4.4292811132645226E+5121135801', '-9.51712278726417445787638227521891997343039315364707668257772E+1966', '4.429281113264522599999999999999999999999999999999999999999999999999999999e+5121135801', 73, 1);
    T('-9.3821712181334145798597046388825377192689255321E+1692248045777618', '-4.7552668779678714551896721652293814527318654E+9517189850', '-9.38217121813341457985970463888253771926892553210000000000000000000001e+1692248045777618', 69, 0);
    T('-6.590081501818550810124E+560575879141087', '2.139698664650713466676557760471945997301520491891907E-5838764751159425', '-6.6e+560575879141087', 2, 5);
    T('-59.70373469', '-4.30895478093667341864400715935123152241947430874E+43596396', '-4.30895478093667341864400715935123152241947430874e+43596396', 77, 2);
    T('7.321546220940E+9166', '-9.46233912702855423820746055618063549850000740E+8207998171206', '-9.4623391270285542382074605561806354985000074e+8207998171206', 48, 5);
    T('-9.3986807952302428116057536324831511042590359339776909E+1686040575669281', '9.40551E+259590695', '-9.3986807952302428116057536e+1686040575669281', 26, 5);
    T('-7.343898312385180990646946195406703557997370E-697598866', '3.6230026625566213593889398018645270E+126617', '3.623002662556621359388939801e+126617', 28, 1);
    T('7.85391520689214613980604197517177E+7463569280150', '-1E-3625618', '7.85391520689214613980604197517176999999999999999999999999999999999999999999999e+7463569280150', 78, 1);
    T('-1.288404362705256761956432065053520E-56597160', '-2.6831987412218013212747536412194175238751E+9283', '-2.683198741e+9283', 10, 6);
    T('6.99454277654457788450169774390125674621506350633613E+18456932', '-1.95725917117738089332993116731461948077375605552194255003377E-226', '6.9945427765445778845016977439012567462150635063361299999e+18456932', 56, 1);
    T('3E+9929207', '-7.54189657E+7443848', '2.999999999999999999999999e+9929207', 25, 1);
    T('-4.273482483945293331026424066371402063843E+12801635242539', '4.2199944E+3319755792', '-4.27348248394529333102642e+12801635242539', 24, 2);
    T('-6.77619644475074132E+73227248', '-2.978810059317337100127878567873417059633370E+44982', '-6.7761965e+73227248', 8, 3);
    T('-2763124579.456276163889915053206415423505716785', '-2.4480197186092548130915392507651238324808872875E+32861363576557', '-2.4480197186092548130915392507651238324808872875e+32861363576557', 54, 2);
    T('-1.231110310862260325270530746585320043246E+67594590', '-9.074479766807023406E-22008313', '-1.231110310862260325270530746585320043246e+67594590', 55, 6);
    T('1.8883252905642760646212179490825806725827E+9523171828', '-2.60107769E+486034', '1.8e+9523171828', 2, 3);
    T('5.723807518589148493E+185433229460', '2.19445E+3858', '5.72380751858914849300000000000000000000000000000000000001e+185433229460', 57, 2);
    T('-8.298540178358577873354386553547091115722709369415E+5423294738617', '6.005270183419763774517787536886175004638585084664789478E+7625554494', '-8.298540178358577873354386553547092e+5423294738617', 34, 3);
    T('-9.752178885471858130138E+22846659', '1.08898704906329510822189445094222407054282877972660338532909E+32246886', '1.088987e+32246886', 8, 3);
    T('-9.2119E-2415800825', '-5.8249294109140758986596319222018E-246708', '-5.8249294109140758986596319222018e-246708', 49, 1);
    T('5.275015870563269582116E+491765656551743', '-6.42137280468908810343890335937462994460437E+7284276527916546', '-6.4213728046890881034389033593746299446e+7284276527916546', 39, 6);
    T('3.325175055371706981871236190270444E+74447571940', '-2.2292E-191508377', '3.325175055371706981871236190270444e+74447571940', 78, 0);
    T('-2.528272185718067701411159291977406514184E+801', '-5.0564360909968216657935270964988059358983396472941798286996E+1942749704466', '-5.05643609099682166579352709649881e+1942749704466', 33, 6);
    T('-1.239473990496135085834637306414007787E+53', '9.015752994181808923965956316370968212969436992737681E-58779', '-1.239473990496135085834637306414007786999999999999999999999999e+53', 61, 2);
    T('-4.63209215431804480604779192545615E+77885450981', '-3.16293252444753624457242912E+27', '-4.632092154318044806047791925456e+77885450981', 31, 6);
    T('-2.416222669746101066907860591803544152019595347122E+49479810', '1.5060458314885358933743859634610833951970399089271112894E-303', '-2.416222669746101066908e+49479810', 22, 3);
    T('-6.51337402287252464E-546485430744047', '3.00980083842522838218E+85520', '3.0098008384252283821799e+85520', 23, 1);
    T('-5.74703681434608829353199238811785989276698E+56', '9.08267227305076182465', '-5.74e+56', 3, 1);
    T('-3.79840510465927E-9411386', '1.67798547856052352171890962018905561767131170257E+6658534314', '1.677985478560523521718909620189055617671311702e+6658534314', 46, 1);
    T('-1.91089378250194345125807856354199245384371582883119E-9399519840', '-7.20106747711295140655660475880500440229196938648E+792', '-7.20106747711295140655660475880500440229196938648e+792', 49, 6);
    T('-0.000325023687918488082141', '5.50412574961479337682798713250808887663799699739286280407371E+65946404503824', '5.50412574961479337682798713250808887663799699739e+65946404503824', 48, 5);
    T('7.89363E+8', '-2.0769976925924592811768462213636195391688519868044409E-37165910581', '7.89362999999999999999e+8', 21, 3);
    T('-5.66147095521541041910076054428263E+317856', '7.489553435226768475437668011849331107009777305410505E+31882676234', '7.489553435226768475437668011849331108e+31882676234', 37, 0);
    T('4.842322978568854226493987001153833672236476613988E-3345596733', '-1.1852874980411537499223865583861969112905E+8097984531720853', '-1.185287498041153749922386558386196911290499999999e+8097984531720853', 49, 1);
    T('5.6014108387E-61756', '-260309120469280439.14280854', '-2.60309120469280439e+17', 18, 6);
    T('9.114836003307736095230358453675278623350619606152665098E+799509678676', '-3.909185256E-315', '9.1148360033077360952303584536752786233506196061526650979999e+799509678676', 59, 1);
    T('-9.666407641288417322373683437306405765480475E+1554331797', '-2.365503987912960542151768622046235238300613518682953870061E+450930670', '-9.66640764128841732237368343730640576548047500000000001e+1554331797', 54, 3);
    T('-3.755456138970083650887216434316E-73199458557292', '-3.33976037857546319795164E+17784631', '-3.33976037857546319796e+17784631', 21, 3);
    T('7.434590899478733357008103415055458041572526394578990481330E+92131049', '-5.633832565684711365919316617512382379592810507752946477E+8104530543189', '-5.6338325656847113659e+8104530543189', 20, 6);
    T('7.5089650179003399947772318437383508734192788186943881099E-838480769', '8.52996208495E+81207131231335', '8.5299e+81207131231335', 5, 3);
    T('2.276931106185971E-31', '6.84873638380148E+1008188031285', '6.84873638380148000000000000000000000000000000000000000000000000000000000000001e+1008188031285', 78, 2);
    T('-5E+9711884931374', '5.0905327344459492053666604510777337614113105E-4373', '-5e+9711884931374', 5, 4);
    T('7.220020485293771783335697325209469363607E+51908', '-7.42977382967892871043619E+7110548354513457', '-7.42977382967892871043619e+7110548354513457', 69, 3);
    T('2.2872375276916540057849437853009E+8992772', '-7.0011703238597304168466949397040180085164121E-237431925294689', '2.2872375276916540057849437853008999999999999999999e+8992772', 50, 3);
    T('-2.31143263591878494816319117367E+5823577768494', '2.7384249237761004330169853E+797255279792695', '2.7384249237761004330169853e+797255279792695', 46, 6);
    T('6.4465789251736193980E+43673635995854', '-9.809506599120824054E+81150316', '6.446578925173619398e+43673635995854', 41, 0);
    T('-3.733598305005124861439010520224196059022864647260068331E-85772', '7.331287511553878995008287E-1555971709557196', '-3.7335983050051e-85772', 14, 4);
    T('3.4714932053532143787389918183E+2216704656608515', '3.4878359943718826865E+226902', '3.4714932053532143787389918183e+2216704656608515', 71, 1);
    T('-6.714782630237070916637346493870E+913512915116316', '-9.71838063049822514994204241050284967425E-476', '-7e+913512915116316', 1, 3);
    T('7.712218539929349642E-70125404371352', '7.016730492194474636582074269037215790315E-31631796640901', '7.016730492194474636582074269037215790315e-31631796640901', 50, 6);
    T('1.264262022301755841211508E+30926533266090', '6.9694330347302722375E+5614643049821454', '6.969433034730272237500000000000000000000000000000001e+5614643049821454', 52, 0);
    T('159207073.79691287060', '-7.8643E-2945530817', '1.592070737969128706e+8', 28, 2);
    T('7.2585140860065496058778854192768605270062E-2541905342050', '-1.571153352889267057641228651167699953419232724286228116102E+980494511771558', '-1.5711533528892670576412286511676999534193e+980494511771558', 41, 3);
    T('-9.558970257195E-96404587745', '3.331008511824233E-770422', '3.331008511824233e-770422', 55, 5);
    T('-2.76989310291867569844105085843992021663E-39209', '2E+12487', '1.9999999999999999999e+12487', 20, 1);
    T('2.2463365659730727006889578236916247831401513E+7821592', '-8.311470001249180736226373587617959932200136679269E+8529', '2.24633656597307270068895782369e+7821592', 30, 4);
    T('3.88221945584876E+6636072', '-6.1248353455449008578923287718269271011696100090932930711E+1454803899065', '-6.12483534555e+1454803899065', 12, 0);
    T('5.0795367621052456408490709621E+5318', '7.1490275559904295421531734020401352969135104E-3921836835', '5.0795367621052456408490709621e+5318', 66, 4);
    T('-1.2194763212061098981910634518244920037926525721043998E+847300712674643', '6.27881712398756E+2316336', '-1.219476321206109898191063e+847300712674643', 25, 1);
    T('-5.412054117147294194861E-34321083', '6.054567586311745721832937E+7944101635', '6.054567586311745721832937e+7944101635', 29, 6);
    T('-8.4395781806483361671623189450231169813854235334630109573343E-87405901', '2.49091403941496676700068275035152378290723817378661E+528011051537675', '2.49091403941496676700068275035152378290723817378661e+528011051537675', 71, 4);
    T('7.89212438543248970718309870868060918E+75854925', '9.431512986E+13008381', '7.89212438543248970718309870868060918000000000001e+75854925', 48, 0);
    T('9.9339281465821733335926165206689075560E-81926101870714', '4.896305013602025269983638549930566799464636862653645E+62493425940', '4.8963051e+62493425940', 8, 2);
    T('-5.94283422868434858320593298E+7449279336646404', '7.9898470E+115791436', '-5.94283422868434858320593298e+7449279336646404', 38, 5);
    T('-9.401394092401645977474E-703266', '1E+81316035470809', '9.999999999999999999999999999999999999999999999999999999999999999999999999999999e+81316035470808', 79, 1);
    T('7.30036065480739961814442824420468910713048344219074908E+5671973388241301', '-2.577337415330E-848804964649434', '7.30036065480739961814442824420468910713048344219074908e+5671973388241301', 59, 5);
    T('4.9748E+1445239348101988', '-6.6022756726746773908785025129916154E-407106655', '4.97479999999999999999999999999999999999999999e+1445239348101988', 45, 1);
    T('-8.89680945132E+1669587112324', '-2.18773661438587942987934356041157137315685359243870718E+4399834', '-8.89680945132e+1669587112324', 69, 6);
    T('-5.859683552597612326065607102E-96294', '-0.00000507561476927', '-5.07561476927000000000000000000000000000000000001e-6', 48, 3);
    T('-2.22681272360935317317618694094398013120222350850E+6237769188701110', '-2.0798264105873726910840398110786364997684E+420337786670579', '-2.2268e+6237769188701110', 5, 1);
    T('6.62429883452509E+51797071', '2.5065313603204E+325167761406873', '2.50653136032040000000000000000000000000000000000000000000000000000001e+325167761406873', 69, 2);
    T('-2.15673209980793061346428819684097584078373959545025346764002E-1883614525555', '-1.00274659815846159612118269321504584639923833836E+9120978327300', '-1.00274659815846159612118269321504584639e+9120978327300', 39, 2);
    T('-5.88150988727330977080107079475971206146381429E-7500322522', '-4.941674284453532215441202803367392116E+188246701049', '-4.941674284453532215441202803367392116e+188246701049', 48, 6);
    T('2.97121288842643093744224180E+55780', '-7.938E-59329194', '2.9712128884264309374422418e+55780', 38, 4);
    T('-6.574E+17786228316367', '1.54823392160187388E+421606290182289', '1.54823392160187388e+421606290182289', 76, 6);
    T('-1.4386362698762445937385430870E+270263290325309', '-6.567341309606E-249', '-1.438636269876244593738543087e+270263290325309', 73, 5);
    T('4.74380121442249691622E-174358136319', '-2.8405255893340854792847448961247476742E+409911735419', '-2.8405255893340854792847448961247476741999999999999999999999999999e+409911735419', 65, 1);
    T('-9.241972235716087687E+890565090315', '-9.9689727163346823821556494154215973694368825647338813087E+297', '-9.241972235716087687e+890565090315', 37, 2);
    T('-5.8623641076808713258420774734327563776643636940291590E+532', '-3.40565222809881E+789277491768', '-3.40565222809881e+789277491768', 20, 6);
    T('-6.1385400453458466096709814051558169719069966182416E+1934835825414', '-7.218612302246288798775300975637067914236735583286300E-88', '-6.14e+1934835825414', 3, 3);
    T('-8.2631315863973253420441851261410007E+263754870', '3.520983486E+9180', '-8.2631315863973253420441851261410007e+263754870', 71, 4);
    T('6.102E+6', '9.251E+9894416', '9.251e+9894416', 56, 1);
    T('-7.035216398953444739E+474881948', '5.08312758800475916402219276799E+5088906613', '5.08312758800475916402219276799e+5088906613', 36, 6);
    T('960244781217936275914.111125549570674020035910200758961089', '3.25203885128676800603760937310340465380381183818052193282E+63752847933696', '3.252038851286768006038e+63752847933696', 22, 0);
    T('-4.773451592309088268240278346E-25826289', '4.170722552158050767624621277713115585778E+5956514', '4.170722552158050767624621277713115585778e+5956514', 72, 0);
    T('-4.17011722949508741230437012724E-985', '-8.335847755892E+538801', '-8.335847755892e+538801', 45, 5);
    T('-5.8020476349845601170E+198', '4.58618990133116293411107746771686509956581213060984E+121195423', '4.58618990133116293411107746771686509956581213060984e+121195423', 73, 6);
    T('-8.2395E-855731', '-9.30280761396162E+2219', '-9.30280761396162000000000000000000000000000001e+2219', 45, 0);
    T('-0.000008809205987819152341844825808172264882416270329578', '9.2182173092811E+93116497455948', '9.218217309281099999999999999999999999999999999999999e+93116497455948', 52, 1);
    T('3.1995281354952202945255E-4386891217545', '6.29731900914663133721692145398179440501301428172E-85843001046', '6.2973190091466313372169e-85843001046', 23, 4);
    T('4.41636285308620900673952010291468784E+76583', '-2.091068081221531446590142523793810E+21675502356', '-2.0910680812215314465901425238e+21675502356', 29, 5);
    T('2.136589357326120549528153753394E+848769', '-3.72247014057885951278415738E+750341', '2.13658935732612e+848769', 15, 3);
    T('-6.40442439863509500143208808616854019180896350942645214416E-331955877941280', '-8.21073010928830553084E+8850098024787', '-8.210730109288305530840000000000000001e+8850098024787', 37, 0);
    T('1.668794367573413019305766059520930818425838E+95735604811864', '2.494493730168309249545E-71595112', '1.668794367573413019305766059520930818425838e+95735604811864', 68, 6);
    T('-3.4862712980635689E+50', '-85498355.6196581326631245025112741534218325281505309132155382', '-3.4862712980635689e+50', 27, 5);
    T('-6.537576738987090545992005506435285180180768391753880492847E+728912596060014', '2.1471420410430477236058119113438309667126521053038975771E+5268632225', '-6.537576738987090545992005506436e+728912596060014', 31, 3);
    T('-1.4145138145952253190766000848095787377061115744202866E+21986152', '-9.1164640885275853906840605524512881737568580527218825140125E+590228311281110', '-9.1164640885276e+590228311281110', 14, 3);
    T('1.2191641881730548832751674793333953987711048269207E+400295546', '-1.05614977612874418572811E-748507667555838', '1.219164188173054883276e+400295546', 22, 0);
    T('3.54033374443102164250269226469221759161796449320317126733E+36769', '1.594446418339203617497179858979771382634954E-28679268', '3.54033374443102164250269226469221759161796449320317126733e+36769', 61, 4);
    T('1.37745244320755252332373945290997295166312467692660758E+82181065161', '-9.1479372330482097512999614E+9189', '1.37745244320755252332373945290997295166312467692660758e+82181065161', 76, 2);
    T('-8.6875757957883497987974851198117E+56', '5.570812011096509764063574258083477342E+31343572962', '5.570812011096509764063574258083477e+31343572962', 34, 6);
    T('3.368342876E+392181393', '-2.2751345795742647408184774E-25487684', '3.3683428759999999999999999999999999999999999999e+392181393', 47, 3);
    T('-5.5455644687421800835E+8392632671494', '5.71596280803272621960563378142534739986192212249298006218E+561862', '-5.5455644687421800835e+8392632671494', 44, 6);
    T('3.03896923110015944219596858563095299869370519583541E+3901724739430758', '-5.260494499774723786598818585568699487710712795599948717E+133', '3.03896923110015944219596858563095299869370519583541e+3901724739430758', 60, 0);
    T('55.33184219182190801600356153324159', '-8.394E+9848934346', '-8.394e+9848934346', 15, 6);
    T('-6.1775058287018371773243793814721886075366028706578307579E-783323172253', '-4.49517740401001373910039424697835851835187070332039690114E+421781838276', '-4.49517740401001373910039424697835851835187070332039690114e+421781838276', 77, 1);
    T('2.04009887345523348052982427935831931E+1051493', '4.014730485328E+95', '2.04009887345523348052982427935831931e+1051493', 37, 1);
    T('3.94975957798228039278268817222592779404201108073116796610E+527447549322332', '8.782130291946E-9389', '3.94975957798228039278268817222592779404201109e+527447549322332', 45, 2);
    T('-3.2024496E+5889258152274363', '-6.9569703686360293547847E+37637', '-3.2024496e+5889258152274363', 18, 6);
    T('2.847671794732837259544930460598124168251234004934095217569E+4236901', '-3.87002298025401663316706650012320670636233915401E+914087421482', '-3.870022980254016633167066500123206706362339e+914087421482', 43, 2);
    T('-1.3551580906581216E+5635815361691936', '1.29064141069731030041396526740401985E+6289', '-1.3551580906581216e+5635815361691936', 64, 3);
    T('-2.720004526673863999629575095890E+873187538', '5.382E+4885963', '-2.72000452667386399962957509589e+873187538', 33, 0);
    T('4.293911664492178343077234E+590819486', '559060.02661860913849777359886791502539641849463599849', '4.293911664492178343077234e+590819486', 28, 1);
    T('-2.70486638646378521908434327506961843598925E+361787995168326', '-1.518558749249074333616329724419102134E+9026558586', '-2.704866386463785219e+361787995168326', 20, 1);
    T('7.6544911999002E-293266', '-6.3983816092E+1330689', '-6.398382e+1330689', 7, 0);
    T('9.233E+832363216', '3.08814225422429759849663E+3788043056192', '3.0881422542242976e+3788043056192', 17, 4);
    T('4.5329025538E+97319360205330', '-4.633E+29546', '4.5329025537999999999999999999999999999999999999999999999999999999e+97319360205330', 65, 1);
    T('-3.96528434616557964915044240328118912645587E+25289', '-3.73730695690839056038556228142710614E-81345', '-3.965285e+25289', 7, 0);
    T('7.727E-509826911170', '-6.1E+7671513616802993', '-6.0999999999999999999999999999999999999999999999999999999999999999999999999999e+7671513616802993', 77, 1);
    T('-8.5270675045501967737409551592103634369034E+3244428792', '-3.85035353794569350989563121413890012245202662261433038927E+36365003032', '-3.85035353794569350989563121413890012245202662261433038927e+36365003032', 76, 6);
    T('8.3147813011516405415226929028849611051413823953984120485669E+435302', '7.0134778131215306393718435768E-66854', '8.314781301151640541522692903e+435302', 28, 4);
    T('7.2819349696349501807975073776326364E+8633853', '2.177903360712119898112334722E-6979840822207', '7.2819349696349501807975073776326364e+8633853', 75, 6);
    T('-7.2140675448539522936846E-75921', '8.43729E-475', '8.43729e-475', 46, 4);
    T('-7.823053E-373425', '9.85206916410E-33716141266101', '-7.823052999999999999999999999999999999e-373425', 37, 2);
    T('6.7794431040141078973852661698539244530788729400959605E+868036', '-2.8441602546460245958267797765920867796257347120709405E+2440807504', '-2.8441603e+2440807504', 8, 4);
    T('7.304009993182999E-8794734998050', '-2.5678842185826247831E+9147670566', '-2.567884218582624783099999999999999999999999999999999999999999999e+9147670566', 64, 2);
    T('2.7E+5718787', '1.3794738975470296870879753918806610206E+372299405264490', '1.37948e+372299405264490', 6, 0);
    T('-1.782936374411184822219521821899406779361013372831360E+1477149788792', '-6.662592352E+27299802377', '-1.8e+1477149788792', 2, 6);
    T('-3.1902448983460653982E+475611971418', '2.4234799770604E-767742545502140', '-3.190244898346e+475611971418', 13, 2);
    T('-7.718603224833803340671053204421639905685305336759916435E+638', '5.246533769046125E+48694756312640', '5.246533769046125e+48694756312640', 58, 2);
    T('-9.8E-525106103415', '-4.028505114942823754172703468440E-7', '-4.02850511494282375417270346844e-7', 40, 6);
    T('-7.8482636747134074892906063343948710501E+3202', '-3.10760475709734550556543672108421893986427554609386E+91955437890660', '-3.1076048e+91955437890660', 8, 5);
    T('8.01839771326452958E+130114316478385', '-6.5401172824960039912E+635040461', '8.0183977132645295799999e+130114316478385', 23, 3);
    T('-1.696045768832720108709091393632504910673893187965E-2570084476049', '3.3E-9100954', '3.3e-9100954', 42, 5);
    T('-4.72661373699328293211474604E+405613837583100', '-9.2828357862076785271796564776877147E+6814122835770227', '-9.282835786207679e+6814122835770227', 16, 5);
    T('9.73198796888878086271E-633549279', '6.9E+22659980', '6.9e+22659980', 75, 6);
    T('-3.0818341489424903153971258159701483310406904871751461E+84', '6.03787257984252004489566376903739846579851E-31351940279', '-3.09e+84', 3, 3);
    T('4.177148392567308E+91921856726633', '3.3376064243157761621020809131E-23755', '4.177148392567308e+91921856726633', 55, 3);
    T('2.5922942878557606082795E+794936553', '-2.9966569240834604947703613646164846608479E+9453640', '2.5922942878557606082795e+794936553', 43, 2);
    T('-9.60809733836157376867799160793632582240375340185390688951E+67258818810', '9.554221579639989348105137699211172988E+34192225009214', '9.554221579639989348105137699211172988e+34192225009214', 52, 2);
    T('9.5402711051748514984077750195E+9885', '3.69741720601961866793702703697886470416500392195217E+420', '9.540271105174851498407775019500000000000000000000001e+9885', 52, 2);
    T('2.049051042757591499578669E-90', '-7.31866816688317546783902729394026E-1402', '2.049051042757591499578669e-90', 73, 4);
    T('2.9046123737E+763247551007224', '-864.6881068376781342692998194662028', '2.904612373699999999999999999999999999999999999999999999999999999999999999e+763247551007224', 73, 3);
    T('3624296.01261318665639781378330122905486652391021096968', '2.381715547241788549882604536576434E+8162051373847', '2.3817155472417885e+8162051373847', 17, 6);
    T('1.9410435E+89074866168105', '-3.43723232222245417075277357891527990636356874280620278537172E+3580089479', '1.9410434999999999999999e+89074866168105', 23, 3);
    T('1.6141488329982097172871134130321606303487641263628E+570335243487', '-6.4341260237091E+47', '1.61414883299820971728e+570335243487', 21, 3);
    T('-5E+90', '-1.714E-479040117', '-5e+90', 33, 5);
    T('-9.07153500988564331410197E+24', '2.51875395371259592073E+8082981', '2.51875395371259592073e+8082981', 48, 5);
    T('9E-29871271', '4.494017098481454432E+46', '4.494017098481454432e+46', 48, 4);
    T('-9.946753114133345987135085702197191987612917926932822406921E+178013698', '6.586822450988016339130E+1195582664491745', '6.58682245098801633913e+1195582664491745', 68, 0);
    T('956893990.5442400456645', '7.5687962084033579492569558411301916465153156400E+60793484', '7.568796208403357949256955841130191646515315640000000000000000000000000000001e+60793484', 76, 2);
    T('3.88671576905498535233775318705127187403529621264244E+134', '4.8673755187332E+358975572903880', '4.8673755187332e+358975572903880', 50, 5);
    T('2.430100540664082810519264876488635714600253519820E+907', '-2.91423285287419008801218183202051684013739331812568513048E+4842334865429', '-2.91423285287419008801218183202051684013739331812568514e+4842334865429', 54, 0);
    T('4.741978613875080E+438113', '-2.8347141427330870756707489771549342713470638690797666965E+46779975480', '-2.8347141427330870756707489771549342713470638690797666965e+46779975480', 79, 0);
    T('1.0482235455960695051880954E-401156', '3455.392719798605384437872029111295053107512725647946146', '3.455392719798605384437872029111295053107512725647946146e+3', 70, 4);
    T('-9.0538832876607217904039749874205971023105498297703041712E+118986537', '-2.5758105613827257378955893E+8702168308298320', '-2.5758105613827257378955893e+8702168308298320', 32, 5);
    T('-4688.953611247399', '5.0039236555257E-7691773594', '-4.688953611247399e+3', 49, 0);
    T('-2.2997809736516746162880882216178187054186780E+1303751422645', '2.2504009282903545894342108746592832264711335132395466791E+60745', '-2.299780973651674616288088221617818705418677999999999999999999999999999999e+1303751422645', 73, 1);
    T('8.72085131441052992741726487', '-1.2683556051373457794E+68473885432586', '-1.2683556051373457794e+68473885432586', 42, 3);
    T('6E+5455', '-9.672501494050001258472915098087880205120867215425588263E-99318736', '6e+5455', 13, 5);
    T('-8.426E-20396527504327', '9.314997938701390827648700E+3599785', '9.3149979387013908276487e+3599785', 33, 0);
    T('-5.486E+92571139', '6.945109342090833615486673474813777235868085876530110833E+312', '-5.4859999999999999999999999999999999999999999999999999999999999999e+92571139', 65, 1);
    T('2.53347807087371424470235501143882712866E-2973730', '-7.8611117E+490229038783', '-7.8611117e+490229038783', 35, 4);
    T('-2E+4592', '6.74671001860515160568345894974794112E-91169846', '-2e+4592', 64, 5);
    T('9900934391.793390471099729432390390394189188924136776487', '-7.89657767952566559193261215257E+9080159637', '-7.896577679525665591932612152e+9080159637', 28, 1);
    T('-9.58554480042848873276453038256412845660224843815870146E+49305', '-9.7822624908441195834581211386080055591668618918755E+446494297490756', '-9.78226e+446494297490756', 6, 5);
    T('-8.15965961501456355773E+776420', '3.2246624416974924E-782178', '-8.159659615014563557729999999999999999999999999999999999999999999999999999e+776420', 73, 2);
    T('2.036726837752332810644332718403978204625688E+9916845548888', '-8.8528468873949498269124988928743466352439139E+500338856322', '2.036726837752332810644332718403978204625688e+9916845548888', 45, 2);
    T('4.14E+608022943903', '-8.378747209497792953228497618815210287649482989627278E+44557', '4.14e+608022943903', 45, 0);
    T('9.466817031750787210245688723175951933E-35106695', '5.95663137602784994143E+60717', '5.95663137602784994143e+60717', 77, 4);
    T('-8.67375486926451891314219082023078032783250043499876E+2632', '-807263.684528074737744', '-8.67375486926451891314219082023078e+2632', 33, 6);
    T('8.5229607295871130906736186937209E+673', '-5.4472578025E-4483555123750782', '8.5229607295871130906736186937209e+673', 54, 4);
    T('-3.8188650875607219E+6718705532319552', '9.6489866414277301421380723690651769850446776349487984753', '-3.8188650875607218999e+6718705532319552', 20, 1);
    T('-9652420034.3335158597943380458607327028999071', '6.1474902492879232292231276882478495889252796628E+508677868709078', '6.14749024928792323e+508677868709078', 18, 2);
    T('7.949244762034484926E+616577574918', '-6.5032202791400E+5176364050161273', '-6.50322027913999999999999999999999999999999e+5176364050161273', 42, 1);
    T('-8.57518444992140430669947906108858371670559658907012583501432E+78980169', '-3.5915108716979403541592337305748387921815E-3981', '-8.57518444992140430669947906108858371670559658907012583501432e+78980169', 78, 4);
    T('7.807746305102933893409983405876953498883949902914138702792E+13545251914515', '-9.03094189608902666555087995505E-14817', '7.80774630510293e+13545251914515', 15, 3);
    T('3.1462577017104222240421442727810556491038865079727E-5702362175', '-5.385660152458105925057E-442782607660675', '3.1462577017104222240421442727e-5702362175', 29, 3);
    T('7.700139050063344729328861357137940769293474997397E-4663414266', '4.5758410263441382450310772937E-7710', '4.5758410263441382450310772937e-7710', 75, 3);
    T('-1.623818007905463403207140429303E-8194446', '8.8691681734714549714542346957E+47327456239010', '8.86916817347145497145e+47327456239010', 21, 3);
    T('3.0028159840509651064008871730977E-4368803014351', '-1.382096854116764505432102493768E-144675720453', '-1.382096854116764505432102493768e-144675720453', 66, 3);
    T('-5.584204425958898769852767712634771385486812949515846E+543520', '8.879107090303452922567195537089590521E+68084290', '8.87910709030345292256719553708959052e+68084290', 37, 3);
    T('6.118789987236774192719613546094454129691515137315249319E+87414587993', '4.03802983996110375799022569693939693037254142E+794', '6e+87414587993', 1, 5);
    T('-3.370175658396231320300189995256432551354146579E-84306645691556', '2.349517323005530759022996858745245866923384883151264E-931', '2.34951732300553075902299685874524586692338488315126399999e-931', 57, 3);
    T('8.75170219923029502380083203038882288023554536627012250E+17370', '4.5334261615823445576700337649206047491769530E+5194', '8.7517022e+17370', 8, 0);
    T('-8.18685987272185412901378E-248691100', '-7.037530388E-486', '-7.0375303880000000000000000000000000000000000000001e-486', 50, 0);
    T('3.08838211295678522337252946064388495798917E+202307913845557', '-5.7619272165359382002205793281046595013688522170223E+19773521919', '3.0883821129567852233725294606438849579891699999e+202307913845557', 47, 3);
    T('-9.96072069410667226E+52103866', '1.2397491153280177963407981158E+894730', '-9.960720694106672259999999999999999e+52103866', 34, 1);
    T('-2.2132543E+324368731960', '7.948411885362027636616064783002507400595966709985146363E+16075345330', '-2.2132543e+324368731960', 73, 6);
    T('8.109038127666669340478527510041264828028018034103284417E+605140448541', '2.341919683618818370083680E+951898', '8.10903812766666934047852751005e+605140448541', 30, 2);
    T('-8.27238791031984221819134429174283137245913520579219228831E+5447813762', '-8.9075126788215120240188265616883874500648176867744970022E+650', '-8.27238791e+5447813762', 9, 1);
    T('-6E+683', '-9.7716439607694688663269418763583646909482286797406713E+128', '-6.00001e+683', 6, 0);
    T('5.85563242173293539879415736596015E+4841', '5.8344335404395556219432214058438653385021071E-1987492839', '5.85563242173293539879415736596015e+4841', 73, 4);
    T('-4.52892744220504684370E+28468790878599', '1.957723429974739605733765945459E+917', '-4.52892744220504684369999e+28468790878599', 24, 1);
    T('-4.47336E+1684038158068118', '-8.92809568048988354546718541457531128743796395308793895342E+9756591785', '-4.47336e+1684038158068118', 59, 6);
    T('-4.39701897364701811E+2818889614', '-7.15322268835860150210225847275465218823543092406362890E+668953681', '-4.39701897364701811e+2818889614', 59, 4);
    T('-1.17167594804917094976742674716E+459934', '5.347573195114E-9169123630', '-1.171676e+459934', 7, 6);
    T('9.644031285866083525106620E+453332430372381', '4.24526340100182111980223923850513479485224E+9878698', '9.6440312858660835251e+453332430372381', 20, 4);
    T('5.05E-6534473152', '-8.4413010243436476977812919412722E+360531924315351', '-8.44130102434364769778129194127219e+360531924315351', 33, 1);
    T('2.14164838235429859E+4045', '5E-62650408168', '2.141648382355e+4045', 13, 0);
    T('9.95179905552952464455250701800871087946029969459437967766E-42175481', '-10510.7350776972162599305004264531899197395194152544088093781', '-1.05107350776972162599305004265e+4', 30, 6);
    T('-6.62999190E+6305856', '-8E-6830682691371852', '-6.6299919e+6305856', 28, 5);
    T('-1.09086611874374336169297919689785712E-47409318916986', '-2.6880806379162435725302E+613433710', '-2.688081e+613433710', 7, 4);
    T('942.40771377364727560776462846699514450069266864199623668', '-3.8813012091747128114013237033184197048438893E+323199072', '-3.8813012091747128114013237033184197048438893e+323199072', 45, 5);
    T('7.7032183604E-649', '6.028571113097890765780171603182E+2463864734834910', '6.028571113097890765780171603182e+2463864734834910', 70, 4);
    T('1336009.41', '6.461036770052259539241901811242212090223E+27173565437639', '6.461036770052259539241901811242212090223e+27173565437639', 55, 6);
    T('7.9531186164648E+4163', '-3.0446998E+964501458154', '-3.0446998e+964501458154', 34, 4);
    T('-1.4477611257E+587570', '5.7820690419711525861E-537110350', '-1e+587570', 1, 1);
    T('-6.3163E+82324968495', '-3.58934778250769990528953E+15513949302695', '-3.58934778250769990528953e+15513949302695', 73, 6);
    T('-1.86223795979390281789677E-5865670', '-1.4373148570760002618184024065885724061804818E-2333411986073', '-1.8622379597939e-5865670', 14, 1);
    T('7.0844214E+902620481076267', '-6.50767845800794805338524822E-886743344406722', '7.0844214e+902620481076267', 34, 6);
    T('-4.37498293050285576921579366417714142797861513804232511597348E-27', '-1.2972839424E+98806809975152', '-1.2972839424000000000000000001e+98806809975152', 29, 0);
    T('-5.2005754081774601458252370414073681334188116312025E+36419588103840', '6.097572539636551163400439360E+9544133447459', '-5.20057540817746014583e+36419588103840', 21, 0);
    T('7.1029097367673537270468760869680228104561228479E-77946627843', '1.1183507465879374022427985190171292499682748599279011050816E+8216156568', '1.11835e+8216156568', 6, 6);
    T('-5.2651813913883595726032475564173537392E+381105099228150', '6.544260E+97423651397', '-5.26518139138835958e+381105099228150', 18, 0);
    T('3.429231092173119984100930827328229995E+7922307603397', '-1.6480993492797E+164229', '3.429231092173119984100930827328229995e+7922307603397', 50, 2);
    T('-4.2532211234808728242E+7765109', '9.93918276804609387977475439430269927660808563908639456614373E+706314059901', '9.9391827680460938797747543943e+706314059901', 29, 6);
    T('2.94687361376347506900913070695393260833E+9467', '-5.496657016116754143787632936541946035732077435738439675743E+73906388', '-5.496657016116754143787632936541946035732077435738439675743e+73906388', 63, 4);
    T('1.2457665068163135206662789122357600243201144E-2599058564733', '9.1333088075630761713544292253304E+776209', '9.133308807563077e+776209', 16, 0);
    T('-4.46107668626883965681302418773349280428461551981E+5139291', '2.33233491274465916E+7957966931', '2.332334912744659159999999999999999999999999999999999999999999999999999999999999e+7957966931', 79, 1);
    T('4.462747066874806518094408187955363205196772518471554490E-5947225962208704', '-35197309.83482', '-3.519730983482e+7', 25, 6);
    T('-3.073931128403020979415906763820447275020204100817610E-48610', '-8.57772203770E+725404687175', '-8.5777220377e+725404687175', 57, 5);
    T('-4.7', '9.605182623787534657786039603227914725852E+81524071786', '9.60518262378753465778603960322791472585e+81524071786', 39, 5);
    T('-1.982562920729575391082638466247851352067813374512451E+11017517', '-6.0671521557E+5568', '-1.9825629207295753911e+11017517', 20, 5);
    T('-1.645568711004E+473705', '-1.157106390588083091817184810424396E+91646320', '-1.157106390588083091817184810424396e+91646320', 70, 1);
    T('-5.390300071548615232340261800108918771483792980964E+418093', '3.5743083546292099629742501822586669811569973E+4760', '-5.3903000715486152323402618001089187714837929809639999999e+418093', 56, 2);
    T('-5.0528656613363978E+887890821681018', '9.17909389E+5441322838', '-5.0528656613363978e+887890821681018', 74, 4);
    T('-7.811E+4896183', '4.35259789725038037495169835854707E+545688677', '4.35259789725038037495169835854707e+545688677', 71, 6);
    T('-1.319354568329261507234E+7021862', '4.0503204049308889003254673839717033528456399397267356E-1025528263', '-1.319354568329261507234e+7021862', 52, 0);
    T('-2.51900388863E+40886204', '-5.7736406825879510904906E+68687542030', '-5.8e+68687542030', 2, 5);
    T('9.40107332823052507396139821036890413470348683439623758576482E+66413314577194', '-1.641568556889344943014294E-8915', '9.401073e+66413314577194', 7, 3);
    T('-2.249E+31229', '4.1210704080373137823E+202978031672', '4.121070408037313782299e+202978031672', 22, 3);
    T('-2.7451475909294836630241551440687144590451011565508954339E+1172', '-2.751423621303901564505134377227701148279507E-49', '-2.7451475909294836630241551e+1172', 26, 1);
    T('-4.733918056121537753212206446323531919421E-25', '-9.1592921745979602138633062502E+64', '-9.1592921745979602138633062502e+64', 32, 5);
    T('-6.51835123E+715036798', '-4.918663079332732995252758277808805762137654979528', '-6.51835123e+715036798', 51, 6);
    T('6.643458679411719771200884028135E+8712732', '-9.3636990297688500E+943', '6.643458679411719771200884028135e+8712732', 38, 2);
    T('-1.351519380E+395701958693882', '-9.8509747687371553126414643901608289268028005360337E-143', '-1.35151938e+395701958693882', 31, 4);
    T('6.28541791639628047455469977142E+64123471', '8.3838E-12221674500', '6.2854179163962804745546997714200000000000000000001e+64123471', 50, 0);
    T('-5.457710341736589224788971297E-426189750', '8.861721443466267535597654908865E+290509364708', '8.8617214434662675355976549088649999999999999999999999999999999999999999e+290509364708', 71, 3);
    T('-6.411619118391711533660646682016E+435757983', '-5.474281728772323425797881247367E+157772', '-6.411619118391711533660647e+435757983', 25, 6);
    T('5.695204263200099203380447758362944472754E+15545328863', '-499309486106.151653487905428802535612318', '5.6952e+15545328863', 6, 5);
    T('-2.048928E-969169', '6.7166865E+15949350292', '6.716686499999999999999999999999999999999999999999999999999999999999999e+15949350292', 70, 3);
    T('1.77133363965649904224171271793956163236864225833125932329997E+743584781', '1.550303199297299005605318121E-22', '1.7713336396564990422418e+743584781', 23, 2);
    T('-8.60378472021315622394008380097E+46689765', '-4.2901178465416377E+574270764437', '-4.290117846541637700000000000000000000000000000000001e+574270764437', 52, 3);
    T('3.774311559291E+13081', '-3.501E+65439', '-3.500999999999999999999999999999999999999999999999999999999999999999999999e+65439', 73, 1);
    T('-3.8666175217101271754149013E-17698797711664', '4.818636181E+6390775114145', '4.818636181e+6390775114145', 53, 4);
    T('2.029346225823886908684E+73802', '6.4371046329690861747785516351656E+94', '2.0293e+73802', 5, 4);
    T('-3.08043919404897157031397026170558667131860291436189825E-749994732572082', '-6.1151993224792E+440993587', '-6.1151993224792e+440993587', 14, 2);
    T('-4.62210537469983385858854620809811E-17', '-6.6035285831626258476911391763895963811281E+406639011', '-6.6035285831626258476911391763895963811281e+406639011', 67, 5);
    T('-2.6189803401602791228185536640E+31397415535250', '-6.69610815806029579524552838E+5439326179048572', '-6.69610815806029579524552839e+5439326179048572', 27, 3);
    T('-9.4140494589785833476354749582247537499441020259004E+3204742989443', '6.552295016538E+79309', '-9.4140494589785833476354749582247537499441020259004e+3204742989443', 50, 5);
    T('-6.2900643343833732177055889236044306126412867E+525880', '6.37248467382110274309E+5830684', '6.37248467382110274309e+5830684', 64, 2);
    T('-1.01393164E+540609973', '4987433.31159457495876', '-1e+540609973', 1, 4);
    T('-7.59413235428857557517E+986372051996', '9.31991209459618773284190E+800194', '-7.59413235428857557516999999999999999999999999999999999999999999e+986372051996', 63, 1);
    T('-2.562081145750647574728230760424969371195882503E+9230902728', '8.657646310E-3747', '-2.562081145750647574728230760424969371195882503e+9230902728', 51, 0);
    T('4.441446066968349921123780465938217499682E-557876', '-6.796134488968077420E+436967860494', '-6.79613448896807742e+436967860494', 32, 4);
    T('6.69695066582981317876844940316055473053561E+38893', '6.3909275503668527075765E+87066262', '6.3909275503668527075765e+87066262', 69, 4);
    T('-6.910E+5239236106907', '-8.4285867499870E-7007260', '-6.91e+5239236106907', 24, 6);
    T('-6.876073403259581039648261367361587280096038073E-9482625', '2.8398035750934174E+89151', '2.8398035751e+89151', 11, 5);
    T('6.527036464900724E-165482030818511', '2.2E+4701306813', '2.2e+4701306813', 21, 1);
    T('-5.31933792555892198082590321947470670933407778153454803161E+1308428856555098', '8.224090883192195555103474923066659577068920109348E-97621552688824', '-5.31933792555892198082590321947470670933407778153454803161e+1308428856555098', 63, 5);
    T('7.94E+5428830935906347', '-8.1058624717428876341232969387007521825667387051917047E+36839560', '7.94e+5428830935906347', 12, 5);
    T('7.518012937790498467087479155914376318E+819912', '5.786946827084683729356902510E+58176842267', '5.78694682708468372935690251e+58176842267', 69, 5);
    T('-6.32948580378788208762566885E+4390736260801', '-5.49215615160545691587470697492087E-9500186272', '-6.3294858037878820876256688500000000000000001e+4390736260801', 44, 0);
    T('1.86256222212974761711366890E+76516', '5.159146695272227435824379E+8517', '1.8625622221297476171136689e+76516', 43, 3);
    T('8.61380199464212568014016394542566347420333002718E+736219151', '3.592554075E+963539', '8.61380199464212568014016394542566347420333e+736219151', 43, 4);
    T('2.2835679175698431045E+63753237', '-8.143308452870114628547582686063016688849903236538922664E+908', '2.2835679175698431045e+63753237', 73, 5);
    T('1.2395817448411281769344174E-7586', '-2.0735345777367544365061E+198693539', '-2.07353457773675443650609999999999999999999999999999999999e+198693539', 57, 1);
    T('3.325170888880941575756109152973949820887253456216E+1590', '1.509247739E-6582147', '3.32517088888094157575610915297394982088725345621600000000000000000000000001e+1590', 75, 2);
    T('-2.575821056249973214170262458080635462E+8594059966', '-8.349052292539794912765242174583477030023102436597739218E+315538773059', '-8.349052292539794912765242174583477030023102436597739218e+315538773059', 58, 6);
    T('-2.72935416400759427649243543115076E+6578286924', '-5.31437916200571754085662802276598285233206990736727E+70957924', '-2.7294e+6578286924', 5, 6);
    T('1.68562639259559142428326E+22851', '7.89072179560515241888186656028152749309000143E+980586721142', '7.89072179560515241888186656028152749309000143e+980586721142', 65, 6);
    T('2.0937722588086364053182539203105777049763799E-540', '-4.789326077E+160976216187', '-4.789326077e+160976216187', 56, 6);
    T('-8.64787108573642950E+870967228539', '-3.354E+4668488', '-8.6478710857364295e+870967228539', 28, 1);
    T('8.76723032713439165826527219923916342838364753E+726111', '-9.11782654588978476653640E+899', '8.76723032713439165826527219923916342838364753e+726111', 46, 2);
    T('-7.8375422800944624097E-19721', '8.110466296885322233089516760068E+356', '8.110466296885322233089516760068e+356', 44, 6);
    T('8.94814943904E+84244', '6.204371508210638305139870466239928568185918050062069E+84359326288', '6.20437150821063830513987047e+84359326288', 27, 6);
    T('-3.04877721250356816794821423E+30', '-3.2178230711345611368993015146259562637632906024671289E+19201050482', '-3.217823e+19201050482', 7, 5);
    T('-7.2397987320981610485875710766045622086024812656306E-8409124977452', '2.38243903319022542467148525242410637E-951074765484', '2.38243903319022542467148525242410636e-951074765484', 36, 1);
    T('-6867678710', '7.58760700583668347442255061170201480388725E+442871880585539', '7.58760700583668347442255061170201480388725e+442871880585539', 46, 2);
    T('5.6341E+10', '-8.93124645273973727694517761018981007077154E+911005036463', '-8.93124646e+911005036463', 9, 3);
    T('7.15636259901623197732011210734564423995501622E-8970135647461798', '8.0181297385695035103691462807441E+7196796114658390', '8.01812973856950351e+7196796114658390', 19, 6);
    T('7.77179167845057864025278E-2180992925613408', '4.103632373636782E+677309832842868', '4.103632373636782e+677309832842868', 21, 4);
    T('-9.1080284E+8030446067999095', '6.2380750366773978895718504181893000E-764', '-9.10803e+8030446067999095', 6, 0);
    T('-9.498E+69059606214', '5.19676660395634284228530959716127097805356513496492434731E+3475598625519', '5.196766603956342842285309597161270978053565135e+3475598625519', 46, 5);
    T('-2.4642465721167981645083916067498381E+20086264991237', '7.661059822434588838341987751556361612475E+75', '-2.4642e+20086264991237', 5, 1);
    T('-1.980035820925201829748478270396181143649032580E+7624', '136065505.9660435417065996734149', '-1.98003582092520182974847827039618114364903258e+7624', 48, 3);
    T('-9.6265939224033964286905104493576962890623502128E+288962342203714', '-2.3792131283170681E+12243', '-9.626593922403396428690510449357696289062350213e+288962342203714', 46, 4);
    T('-3.751907050930455324763423884E+52476331113106', '6.9975350030178185618449765967945222557632E+594605', '-3.751907050930455324763423884e+52476331113106', 35, 4);
    T('2.95062953140830877855901356665063691E+571613179', '4.32E+8108459', '2.95062953140830877855901356665063691e+571613179', 69, 5);
    T('929909916100877530437707911822737042.007130864534001791', '4.575448E+523079233538379', '4.575448000000000000000000000000000000000000000000001e+523079233538379', 52, 2);
    T('-280077212.5404283351720179035452027619825989662895717605', '-1.672993239304411E+697758', '-1.672993239304411e+697758', 38, 1);
    T('-3.628774E-617245663', '-6.4390869815634294700980698495480960650791667791767494057285E+1460404048274', '-6.43908698156343e+1460404048274', 16, 0);
    T('-3.5851453115509049525463E+25732764268', '2.13703045284553464147349765932136625E+431', '-3.5851453115509049525463e+25732764268', 24, 4);
    T('8.854666284858011881645364502642332743162342E+317235239', '-2.33813057378110064377170E+7665', '8.854666284858011881645364502642332743162342e+317235239', 55, 2);
    T('-7.5422344964368550E+931776', '59093249.6519987422271', '-7.542234496436855e+931776', 49, 6);
    T('-4.0757399121467364163346408037091241958785', '7.2406455529395709088191635091779211220578563222108823314E-8642', '-4.0757399121467364163346408037091241958785e+0', 66, 0);
    T('8.8906E+340', '-8.06840917573583395871168548E+961893', '-8.0684091757358339587e+961893', 20, 4);
    T('5.4778828852389670538610E-53690301', '9.90757549208643566400987681099903985874318619326678363E+81314754743', '9.907575492086435664e+81314754743', 20, 1);
    T('-243170151.455021152747405318746311502', '-1.16651067421138529005908353401206708027526810260755E+1370070335213117', '-1.16651067421138529005908353401206708027526810260755e+1370070335213117', 57, 6);
    T('-9.66170307402879703996904947392863419552797699585155138794830E+23395', '-8.2814661399846323655398837893074E+3060694457', '-8.281466139984632365e+3060694457', 19, 2);
    T('-8.4180883379714236689531198052806723259379402058998224794533E+1044562647944854', '8.31404725330917438393448948685327593298602004621974E+8690', '-8.41808833797142366895311980528e+1044562647944854', 31, 1);
    T('-6.1252423005560656186794E+147238503880', '6.297602991911227928914272097422E+5375085', '-6.125242300556065619e+147238503880', 19, 3);
    T('-6.17176188073559435870141829670582857E+45314176', '8.524341136955953780649001231022555717235972077054379E+547009652351639', '8.5243411369559537806490012310225557172e+547009652351639', 38, 3);
    T('-1.74737689283666838056495037151510935142726E+6731756416033524', '-4.6538E+1656786', '-1.74737689283666838056495e+6731756416033524', 25, 1);
    T('9.336478365931696493535714E+264553', '4.21402533937394877E+1664219722061619', '4.2140253393739e+1664219722061619', 14, 3);
    T('9.641405921864619125259202002341884819952811E-48', '-7.9450618757293403691406015605364199E+7381726693', '-7.95e+7381726693', 3, 0);
    T('7.1110248442277465911544339548591395717350927737586183833E-5436607542', '9.69277667846263059586946E-433144856193', '7.1110248442277465911544339548591395717350927737586183833e-5436607542', 62, 6);
    T('6.84526081458243510669936512972020392825818991939596E+32066', '-3.4691765892068E+22', '6.84526081458243510669936512972020392825818991939596e+32066', 60, 2);
    T('1.203326100019750031646588373892158097357866463E+70', '-3.639626743726414497424551678897342468013E+9517431', '-3.639626743727e+9517431', 13, 3);
    T('7.9129810050505067756916508864506E+3619', '3.23975993078918206523613063641730167468E-99552499734', '7.912981005050506775691651e+3619', 25, 4);
    T('4.748798320221654E+21169684', '4.413088806E+112', '4.748798320221654e+21169684', 67, 1);
    T('-1.715813814412372751333163227560698905E+149941587984355', '-5.48862136105305E+349166727', '-1.7158138144123727513e+149941587984355', 20, 6);
    T('-7.621207299802415932E+500008', '-2.142E+53115570', '-2.142e+53115570', 24, 5);
    T('4.3384162213176407309511E-796', '-6.7066033723633528971932072427519356979780381E-30277', '4.3384162213176407309511e-796', 37, 6);
    T('-8.9263E-7', '-29499694469191631185246492721.4350304285537775517351774424357', '-2.94996944691916311852464927214350313211837775517351774424357e+28', 72, 3);
    T('6.95440177533759972621962620807E+181954687011550', '-9.156773762196598187439606906035785E+4934110343', '6.95440177533759972621962620807e+181954687011550', 75, 4);
    T('-8.914E-404348919', '-4.5E-93331974547375', '-8.914e-404348919', 27, 2);
    T('-7.5204518598064037E+8132087151734224', '-6.13E+606495102339', '-7.5204518598064037e+8132087151734224', 17, 4);
    T('2.2391159533E-887328', '9.39868E+430748213771', '9.39868e+430748213771', 70, 4);
    T('3.8615470502846029142996329E+3033', '6.162112408786207059105017E-879387774301206', '3.8615470502846029142996329e+3033', 78, 5);
    T('-9.87039E+88', '6.0429069E+317427', '6.0429069e+317427', 45, 4);
    T('9.263171924582516986125614286185943E+4536874151642888', '7.65146643211E+900', '9.263171924582516986125614286185943e+4536874151642888', 68, 1);
    T('807245869122289771.52961416573638895239531', '7.7823093787070E+7600431853877', '7.7824e+7600431853877', 5, 2);
    T('-5.814408519622245960227582043E-651', '7.44636840556451196707017761623740E+4977148', '7.4463684055645119670701776162374e+4977148', 35, 6);
    T('-2.50662692115639400826792178025E-484222384760212', '-1.6142726038432889601841836984067175102E+109716811304', '-1.6142726038432889601841836984067175102e+109716811304', 77, 1);
    T('7.73880065430210088179472021574034628553890555E-124365397', '4.0853883323264E+19891762', '4.085388332326400000000000000000000000000000000000000000000000000000000000001e+19891762', 76, 2);
    T('5.38690E+1857423640890490', '-8.7996253419520474263143109072288379887525193E+6072299675', '5.3869e+1857423640890490', 79, 5);
    T('-4.80298836884699725464079348618652549373E-7375815848293595', '4.544349711662788184670685692577640408593915995814E+3517958', '4.544349711662788184670685692577640408593915995814e+3517958', 55, 2);
    T('4.8444667224444816556159271690846904720089817E+78', '3.16726046634E+495', '3.167260466340000000000000000000000000000000001e+495', 46, 0);
    T('2.1568836187809233207229E+662785591474705', '5.238E+6636', '2.156883618781e+662785591474705', 14, 0);
    T('-4.578760579359447152194991697734332436E+65640396', '4.347215405505646823920723114623787155103624835680326166E+305455199', '4.347215405505646823920723114623787155103624835680326166e+305455199', 70, 0);
    T('4.982437294014815E+410095407746', '1.7293485851179555966506374642644779452E+2975042167', '4.982437294014815e+410095407746', 18, 6);
    T('-4.1900889990402104750642735327077284E+420009790445', '-4.349158119242422716523345388319070447656800175E+970745156815332', '-4.3491581192424227165233453883e+970745156815332', 29, 1);
    T('5.592807E-7', '-3.9675196670384369E+46', '-3.9675196670384369e+46', 52, 3);
    T('-3.49261506429150775E+4913', '-9.745077331774845654493082659566643501E+4435125426728549', '-9.7450773317748456544930826595666435010000000000000000000000000000000000000000001e+4435125426728549', 80, 3);
    T('-5.6655614487E+15', '5.6614861029894839233334916987885982659836103625845776785E+9580548', '5.6614861e+9580548', 8, 6);
    T('4.05627387998669537539613576358532E+737675570689487', '6.415980E+676306987063', '4.05627387998669537539613576358532e+737675570689487', 44, 4);
    T('7.3025012622470229682164483657057E+579445546225761', '7.4557811388364176042193771085071913E+726260828', '7.30250126224702296821644836571e+579445546225761', 30, 6);
    T('-7.9862486763160E+4429888389', '2.389677496621608453626007770937013787518658849588708115421E-6354258774011340', '-7.986e+4429888389', 4, 2);
    T('1.5205744866451013E-31921342306950', '5.30199290739198441602085639562650136913E+487037121285382', '5.30199290739198441602085639562650136913e+487037121285382', 55, 1);
    T('4.67073E+416', '-9.815830890024840101142954018345635529553788742145E+3679244478270166', '-9.81583089002484010114295401834563552955378874214499999999999e+3679244478270166', 60, 1);
    T('-6.278865493276176357703224559436601E+628390848808481', '-6.2734742369791E+20101', '-6.278865493276176357703224559436601000000000001e+628390848808481', 46, 0);
    T('5.987786531880445E+300249', '-3.6613405401642412634572859545867E+830377070629910', '-3.6613405401642412634572859545867e+830377070629910', 58, 3);
    T('5.5E+168911869349818', '-7.20E+54633', '5.5e+168911869349818', 77, 6);
    T('8.1558194887E+514664348383', '-9.77413121376299622719140470E+64356', '8.155819488699999999999999999999999999999999999999999999999999999999e+514664348383', 67, 3);
    T('-8.8325E+9', '4.3553378059617098597356193912578E+618', '4.355337806e+618', 11, 4);
    T('5.80088211989285011260661E+102194115581886', '-6.196482331141584752980071246257874485813340871E-92668540', '5.8008821198928501e+102194115581886', 17, 1);
    T('-2.95783879208037397369836992E+17867', '-2.146271853125657201274275812095760522243467428E+99621465', '-2.146271853125657e+99621465', 16, 6);
    T('3.82E+4219024', '-9E+57456664048', '-8.99999999999999999999999999999999999999999999999999999999999999e+57456664048', 63, 2);
    T('-7.729402054990438301570592821582228987664513579178369E+68937294', '9.8054712365710630E-17', '-7.729402054990438301570592821582228988e+68937294', 37, 5);
    T('9.3656234876E+278596027', '8.61343570669455915874252E-69', '9.37e+278596027', 3, 5);
    T('-1.87161234E-8269', '5.976053146793129783719560378719200176625109839541373E-352', '5.976e-352', 4, 1);
    T('-8.1413574157524111907705094483546548E+4913560018147104', '2.55038E+68486', '-8.1413574157524111907705094483546547999e+4913560018147104', 38, 2);
    T('4.21538350967918781817E+2264959657', '-2.933274836378493730703840893565140549E-931021508', '4.21538350967918781817e+2264959657', 31, 5);
    T('5.7248623716468816904094626E+569070642', '9.48577886230666109330625546363690140762169130736907560181E+771758', '5.7248623716468816904094626e+569070642', 43, 3);
    T('1.1909846953353977746095476280325290826981684286475014673E+3781351', '-9.5E-332820', '1.1909846953353977746095476280325290826981684286475014673e+3781351', 79, 4);
    T('-7.9510840437164392994925394047125214205E+8232', '-5.341162520654409E-1161', '-7.9510840437164392994925394047e+8232', 29, 1);
    T('3.6109838626222966179803815769057772905804443248336840E+7391760216', '6.5843008528940041807867E+8758837116282', '6.5843008528940041807867e+8758837116282', 25, 5);
    T('-2.40158234577676121438260969880704298661731767703397805E-66', '-1.461593098435040E-921331719522', '-2.401582345776761214382609698807042986617317677033978e-66', 52, 1);
    T('1.3845676E+1623', '1E-1565', '1.3845676e+1623', 16, 5);
    T('-1.449452545867622486848348421649E+94349277480622', '-6.361890959583378153452083820219403847959624071764E-114044905', '-1.4494525458676224868483484216491e+94349277480622', 32, 3);
    T('0.0305634723362024', '6.0520280441928351685933620416200E-5177373004', '3.05634723362e-2', 12, 5);
    T('1.0743384038400382973450911416254E+52589053948298', '-9.546140852643E+5157066', '1.074338403840038297345e+52589053948298', 22, 1);
    T('8.796085158221962628669020907402325380181865482841462250E+275', '7.76353605114993997823249459963437425252521046678445364E-5084921435930096', '8.7960851582219626286690209074023254e+275', 35, 2);
    T('-7.214938695261875E-246976', '4.607443205588897536543792742739210262573340917078524362E+395391556', '4.60744320558e+395391556', 12, 1);
    T('5.149095001536082219368128183626568196E+3309365363660087', '-2.88568829037689577187014E+554', '5.149095001536082219368128183626568196e+3309365363660087', 44, 4);
    T('6.88287879100E-22425317653', '-8.158974878621047387E-22', '-8.158974878621047387e-22', 61, 6);
    T('-6.004645110259486E-5780', '-5.268836865926015589258776994730955610927383145499E+6361844738', '-5.268836865926015589258776994730956e+6361844738', 34, 0);
    T('6.224117873766308497806675955558018699099760E+2078821', '-8.0821488513847157486137840720861631444067411734348144371369E+407224521', '-8.082148851384715748613784072086163144406741173434814437136899999999e+407224521', 67, 2);
    T('-2.66443750253520232459649723101848E-6520705', '-8.5162259771815700640617394E+4622', '-8.5162259771815700640617394e+4622', 45, 5);
    T('2.689887652733703264941108915931634851671223851E+1885696', '-9.7470119956554490208554007282714066401361704411264E+935220', '2.689887652733703264941108915931634851671223851e+1885696', 76, 5);
    T('-7.447051258643794869470773447263915681827902458506E+9866563', '7.77787184927867100415564697109289720537712261499275011E+13189941146', '7.777871849278671004155646971092897205377122614992750109999999e+13189941146', 61, 1);
    T('6.2462553867244882386745696501630E+78535849351630', '1.4195247906504656031753095398305547471438399E+298884112', '6.2462553867244882386745696502e+78535849351630', 29, 2);
    T('4.4869822220935710306965711750160461771880793528272015415E-293624027', '4.614875869632E+4789031481847118', '4.614875869632e+4789031481847118', 65, 6);
    T('-6965.93', '2.60281842160961820197E+59159144245', '2.60281842161e+59159144245', 13, 6);
    T('-4.98645806766910418041792030428E+40', '9.6891523178725332697746139449664356873843268E+3350953113162066', '9.6891523178725332697746139449664356873843268e+3350953113162066', 56, 2);
    T('-0.0094149', '339811.437756816202371', '3.39811428341916202371e+5', 47, 2);
    T('-5.295E-790788962829116', '-1.885592285098E-358522', '-1.885592285098000000000000001e-358522', 28, 3);
    T('1.3195903997764116040866226943494E-38032242474458', '7.3976171369682E-54', '7.3976171369682e-54', 14, 3);
    T('9.16597471619840448E+1528896892731', '7.16510548545063529586469431019E+38', '9.16597471619840448e+1528896892731', 52, 4);
    T('-4.76124057396794910719E+6319', '-5.7929248583644471078881664640709090602388852802481987693E+97', '-4.7612405739679491071900000000000000000000000000000000000000000000000001e+6319', 71, 0);
    T('7.0432E-93', '-3.7153910249096955337171946382356E+788221456', '-3.715391e+788221456', 7, 2);
    T('7.5925898369200025706528835197727065E-438', '1.7322745135596767898585345728397157774219095917425825E+464769', '1.7322745135596767898585345728397157774219095917425825e+464769', 79, 4);
    T('-4.9731924269760277610610176057565083E+20975257905', '8.3175034334578507233E+527', '-4.973192426976027761061017605756508299999999999999999999999e+20975257905', 58, 2);
    T('-1.151249538578590410658548455764232621742083408104802E+17236465', '-5.36482730203574737489998199515248334882761E+548226412', '-5.4e+548226412', 2, 6);
    T('-7.0358843584596876888957479E+6102527103', '8.820605220845602949352276796748581E+43537840', '-7.0358843584596876888957479e+6102527103', 56, 3);
    T('-7.8190656603E+801105279374', '5.393281764334351838E+7676', '-7.81906566029999999999999999999999999999999999999999999999999999999999999e+801105279374', 72, 1);
    T('-3.08E-4985376571', '2.500277079157803186414711062771636523793969314181E+3585643373005761', '2.5002770791578e+3585643373005761', 15, 5);
    T('2.316794332345992860300209450496882125660203411303111727E+6206548265', '-7.75050154E+71781', '2.316794332345992860300209450496882e+6206548265', 34, 5);
    T('-8.13954918779990689202105541053799985469470824986696273E-40158828755', '1.816365875467966783759956060714418E+68811409686', '1.816365875467966783759956060714418e+68811409686', 45, 4);
    T('2.20234506195395700086088867626136253397162963E+9335760704211', '-6.92530564046830741944863248962462665239649E+4938890621', '2.20234506195395700086088867626136253397162963e+9335760704211', 73, 0);
    T('-7.2649674836846790718E-8375218599204', '3.37E+5088', '3.37e+5088', 27, 0);
    T('2.3024178062472397025728222049379478878414403E+2906435015323485', '2604976483.70271354512803303787593876200', '2.3024178062472397025728222049379478878414403e+2906435015323485', 50, 5);
    T('-8.819092510411906581062E-324127952', '-4.7838251877048910694961699823527245070850914173337070E-724277860', '-8.819092510411907e-324127952', 16, 3);
    T('2.451186E+6033741952153', '1.869307028903450003352E-914652', '2.45118600000000000000000000000000000000001e+6033741952153', 42, 2);
    T('-4.39510933595782636280325270219E+133241227020490', '4.8710182928930E+14361', '-4.39510933595782636280325270219e+133241227020490', 36, 3);
    T('-3.175E+7882809636334144', '6.6993423E+99112927192', '-3.175e+7882809636334144', 60, 3);
    T('-2.67106009844561331239082615207343669804004842611011E+29561003', '-91488.2129616362955821673757272209952496', '-2.67106009844561331239082615207343669804e+29561003', 40, 4);
    T('7.235445395653E-604', '-1.431987433882126478107E+84592', '-1.431987433882126478106e+84592', 22, 2);
    T('7.595634838673198918310484907419606407842805E-15161452409', '-3.93528902508300976903066083903539591395160E-704328114', '-3.9352e-704328114', 5, 1);
    T('-5.01302587822409921723634913991207097978436951278556408164047E-18', '-7.6E+844527238176211', '-7.600000000000000000000000000001e+844527238176211', 31, 0);
    T('1.97679384508E+903017268645841', '5E-55715', '1.97679384508e+903017268645841', 41, 4);
    T('-2.260E+29934674645849', '-7.2949959448410872138541758148904927398663652E+47589692616', '-2.26e+29934674645849', 31, 1);
    T('-8.7594021254495266E+57442472', '7.407313477513541230348074592355002678260002814390867683078E+818645120681', '7.40731347751354123034807459235500267826000281439086768e+818645120681', 54, 1);
    T('2.465823484623120048431201E+5643', '2.2744292230258547841468604821743581090978E+103771168847440', '2.2744292230258547841468604821743581090978e+103771168847440', 60, 4);
    T('1.940342154511308187680402299164021360683516E+205591900', '8.6997E+5925753', '1.940342154511308187680402299164021360683516e+205591900', 53, 1);
    T('8.172125328270422607029380274904145920030041978654E+62307616732', '8.32013434901072871259702356652752489728027248901945063789E-317276335226062', '8.1721253282704e+62307616732', 14, 4);
    T('-1.24840867E+680415651', '-4.86749E+7416674', '-1.24840867e+680415651', 35, 6);
    T('-9.0230142266657006480636047507696939010340958626625792385E+22552', '5.422056234869936245418988482961852943509795886E+76663671', '5.422056234869936245418988482961852943509796e+76663671', 43, 4);
    T('-2.02418521576832136003902790477414E+43894573', '-7.1902391322026830801002295586392182679583493749632510E+464236790', '-7.190239132202683080100229558639218267958349374963251000000000000001e+464236790', 67, 3);
    T('-7.1402712012555853241419E+81211715754216', '-8.181491357830223560963E-801930', '-7.140271201e+81211715754216', 10, 5);
    T('6.84001439532398110184867E+149101709', '3.734E-1034964', '6.84001439532398110184867e+149101709', 47, 1);
    T('2.001169761424E+51', '69084372.028569177264209915665324783908', '2.001169761424e+51', 30, 3);
    T('-3.3858358931793409560191391E+67293741', '5.0957064094424887702997251655163566475679391915E+781', '-3.385835893179340956019139099999999999999999e+67293741', 43, 2);
    T('5.65843732925762094E+476', '-1461659.95924578736184004385066092433952858', '5.65843732925762094e+476', 52, 6);
    T('6.05609906653330430994294123970227749416257029882304928028E+387994081', '-1.4812392563E+9048620450908', '-1.4812392563e+9048620450908', 46, 3);
    T('-615556338755040292022.224783020364184041339989717', '1.59036698545461558772226236419097E+782617423528', '1.59036699e+782617423528', 9, 4);
    T('-7.84034107941225968965027406655167397523856580717579500E+612691', '-6.487173623857924951E+644', '-7.8403410794122596896502740665516739752385658e+612691', 44, 5);
    T('-4.16502804229193317666E+343047', '-9.8152642947707893517831022768E+2636', '-4.1650280422919331766600000000000000000000000000000000000000000000001e+343047', 68, 3);
    T('-9.282E+58779535819', '210428913.52354560192181092766174732348337926797', '-9.282e+58779535819', 16, 0);
    T('8.5480E+152913', '-6.737070451546974700692551537E+247391', '-6.737070451546974700692551537e+247391', 59, 6);
    T('-4.16E+724969767', '-7.01074832E+6198496', '-4.16e+724969767', 71, 4);
    T('-6.5066E+20298946710358', '9.37000744084944234462884528180243853551E+869675', '-6.5066e+20298946710358', 32, 0);
    T('-8.633964467432720005609325546067940753281773814091494272260E+3081430499267333', '3.88329505494427591957011668938585003E+1909180', '-8e+3081430499267333', 1, 2);
    T('2.0318438069724825953106E+33', '3.55831069288202688854962084533145670707129574850080535E-991368247841320', '2.0318438069724825953106e+33', 34, 1);
    T('8.207342351358091E+99043878', '92656929344498585494328902012496533439.011528013312714125', '8.20734235136e+99043878', 12, 6);
    T('-5.36777212288939903172579745454201413254651443770304476E+60576157452', '-1.3400220468E-62948', '-5.3677721228893990317258e+60576157452', 24, 6);
    T('7.411206821701579315922727936204246207807839022484E+140', '-173923.04004489046791882461778823983560776122110321113405', '7.411206821701579315922727936204246207807839022484e+140', 70, 4);
    T('-4.9275568498407217594E+4680497318395081', '-1.905547909548076872545747624112300E-35614454755', '-4.9275568498407217594e+4680497318395081', 54, 4);
    T('4.9082439837385119029660307727235142542367337004E+61223022582', '-2.99476224E+88', '4.908243983738511902966030772723514254236733700399999999999999999999999999e+61223022582', 73, 3);
    T('6.17613716315540001722000957359062914E-4325', '-6.25046734162713513964975598417515024E+449', '-6.25046734162713513964975598417515024e+449', 64, 0);
    T('-2.2496E-51369062', '4.126949203016365633503996582E-6756', '4.126949203016365633503996582e-6756', 44, 0);
    T('-1.0E+8142740854', '1.673945965953315E+828743159152', '1.673945965953315e+828743159152', 70, 5);
    T('8.279052035568480636020154E+230472242411', '125.81847222273759295528937313707983878944', '8.279052035568e+230472242411', 13, 1);
    T('9.8942166412802344318946852516216442275E-305430594650', '4.815290196133605268421849113553423020069775394435627249E+7427175066648', '4.815290196133605268421849113553e+7427175066648', 31, 4);
    T('7.5993921086519229039099566829465263444064110816844583E-63', '-8.690301512508733E+199071894986297', '-8.690301512508733e+199071894986297', 50, 3);
    T('-7.3652511692652630210442439595710E+5834', '-4.6303212122758027695E-81700616703', '-7.365251169265263021044243959571e+5834', 40, 1);
    T('9631306.03378729575594991449', '4.834750735012E+83539616305', '4.834750735012e+83539616305', 23, 6);
    T('2.50903653005687333499679848113397528581773448000E+78', '-8.89070578389527965715656568069897471579512245457E+240178493', '-8.89070578389527965715656568069897471579512245457e+240178493', 65, 0);
    T('9.379015589277663383532365E+3289720924', '7.447160953660082169938068837778942983316104297026E+55390', '9.379015589277663383532365e+3289720924', 32, 1);
    T('8.794912015020313639059126700275013523808E+75', '1.344823606898844996177099305675E+9338114975', '1.34482360689884499617709931e+9338114975', 27, 5);
    T('-6.871385665672725779077604247121471209077118692E+404980', '5.18722069569508156511028700540432527209498002E-1199004790', '-6.87138566567272577907760424712147120908e+404980', 39, 0);
    T('-3.18079697618024673955327075858415097269722046878731846E-4316759131469', '7.18998774518881164896916943397829199933405665112855E-763590363187', '7.2e-763590363187', 2, 4);
    T('-9.853555903882820375188068880769092262953E+5195229342000', '-5.1356491954072448E+370316473', '-9.853555903882820375188068880769092262953e+5195229342000', 73, 5);
    T('-8.8695338404961241175834747796619383584E+4492530881948', '-5.45591039353921437553213795119968742E+817187', '-8.8695338404961241175834747796619383584e+4492530881948', 54, 6);
    T('9.2652301692419497252730422249170993207527916E+4046', '-1.76789694618114715471945096425921124507947813E-467377', '9.2652301692419497252730422249170993207527916e+4046', 70, 2);
    T('-4.04840411769627966147408760063763020932371620723726630E-439729', '-3.6677904E+890213071', '-3.66779040000000000001e+890213071', 21, 3);
    T('-8.360894922421058446367672303307832922622877810608796015565E-291740', '-5.5943359332308645423126945106370705034E+9782208565489', '-5.5943359332308645423126945106370705034000000000000000000000000000000001e+9782208565489', 71, 3);
    T('-1.08802913661153929847338906972E+3127439295', '1.656E-505112029582', '-1.09e+3127439295', 3, 4);
    T('-1.870585902346398150490E+155879274149287', '-7.1E+254617735901', '-1.8705859023463981504900000000001e+155879274149287', 32, 0);
    T('-5.13834011586976511654526433430E-3488822430808838', '5.0276441479963957041E+4102802888321', '5.027644147996395704099999999999999999999999999999999999999e+4102802888321', 58, 3);
    T('-3.33102434183623480286594866950957593467480163204101737E-32489852965026', '7.0285801442239950584365351844847826032376970497140704138E-492191008012668', '-3.33102434183623480286594866950957593467480163204101737e-32489852965026', 79, 3);
    T('3.3142450408992747921522026813709649932E+88483668546512', '-7.591450E+999', '3.3142450408992747921522026813709649932e+88483668546512', 39, 4);
    T('-2.0185845189671506149406E-5309', '8.834143E-27459', '-2.01858451896715061e-5309', 18, 2);
    T('-2.9427926969700180112567287205759276E+2455137497', '-4.7787373843483155972162E-3177524951267438', '-2.942792696970018011256728720576e+2455137497', 31, 6);
    T('9.390919799840641005834584449806167208069123679478224E-830', '7.1E-58525', '9.3909198e-830', 10, 6);
    T('2.52591607922550510E+84988', '-8.602483348364247944249220873195292801831786735003783985285E+3475217581', '-8.602483e+3475217581', 7, 6);
    T('6.9242208002516873082', '-9.93213128806119974948679E+117503681', '-9.93213128806119974948679e+117503681', 32, 4);
    T('1.5500866814E+21333591098', '-8.871079699214197786866395465949828624441082675380351380060E+89954', '1.5500866813999999999999999999999999999999999999e+21333591098', 47, 3);
    T('7.3077374989E+207911', '-2.1676619472022237415128E-9095062936699', '7.3077374988999999999999999999999999999999e+207911', 41, 1);
    T('-7.608656501026553941776592078933319967450563E+85', '9.590126447724683081324029508548970585395396882E+917873', '9.59012e+917873', 6, 3);
    T('1.26776341239801628073709E+83629004', '94.728059', '1.2678e+83629004', 5, 6);
    T('4.384E+262548717', '4.31379885661730595613686452E+84991483', '4.3841e+262548717', 5, 2);
    T('-2.4064754565648985248024743917713238244313360751990901870E+952575840', '-7.45465186137772534603606812E-8136619', '-2.4064754565648985248025e+952575840', 23, 6);
    T('4E+35822959364698', '7.8229140625611963665180521824696580279683429778E+37267386288', '4e+35822959364698', 77, 6);
    T('9.48073435089173692353999841202915233149E-48911694', '-6.965682964086916E+1201664', '-6.965682964086916e+1201664', 17, 5);
    T('7.70548135398351124209107543254970724906085992E+1024872318', '-8.081496936417778472E-8874', '7.7054813539835112420911e+1024872318', 23, 0);
    T('9.483486700280228785840897744160834953796260661059E+41308', '1.002987229062E+91', '9.48348670028e+41308', 12, 3);
    T('-6.6867690000516849804E+8113557331305', '1.82305203007341248797847254321938070425255171437527E+73621349884797', '1.823052030073412488e+73621349884797', 20, 5);
    T('8.1694992949036329253329400429620654394340808458133032E+19307152858467', '8.97055594674248898125372945763942228029105735664E+518', '8.16949929490363292533294004296206543943e+19307152858467', 39, 4);
    T('-8.503021626895727309826034218507E+379213378281501', '3.0915257995942537216253912261173324408301E+7046723021732824', '3.0915257995942537216254e+7046723021732824', 23, 2);
    T('7.4212104752512789367426597322540814101791463081822031E+8953201', '-8.1221265457787905987463E+278', '7.421210475251278936742659733e+8953201', 28, 2);
    T('-2.77464463362147517E+182384687375', '-4.140002649628146197614673848732325635254E+53475187643087', '-4.140002649628146197614673848732325635254e+53475187643087', 80, 2);
    T('1.646E+7', '-5.680E+2630527717', '-5.68e+2630527717', 56, 3);
    T('3749563.005644507985314531271307271050962763664291', '5.196444766607854559666393294009425984660627669457E-969', '3.7495630056445079853145e+6', 23, 6);
    T('9.854969441935191380968092543797552667E+2785441291441', '5.1071210288256357916178212849670E+33357051', '9.854969441935191380968092543797552667e+2785441291441', 74, 5);
    T('2.968478691956431459209331193155464E+231866', '5.70995108950E+73603134', '5.7099510895e+73603134', 38, 6);
    T('-8.51097011174923253515607726026458993002262203E-2631233', '8.93096167E-395116', '9e-395116', 1, 4);
    T('7.836568615759944810325691347465082354445E+212033945', '-8.60744287977428690728206259191989591680038929442013767E-10104109566', '7.836568615759944810325691347465082354445e+212033945', 64, 0);
    T('-982600.0312761609', '7293436879.6894487047500', '7.292454279659e+9', 13, 0);
    T('7.0682320101E+820103', '7.26647951457423844723064637212918306427927638249987083E-810790159', '7.0682320101e+820103', 49, 1);
    T('-5.68462462320571644136770112626680106847167E+184664094370', '755', '-5.68462462320571644136770112626680106847167e+184664094370', 68, 5);
    T('5.295181013555863234556611183E+1949064', '-8.20885706080239986119724494206044074798648083838628054775054E-2523054', '5.295181013555863234556611183e+1949064', 36, 6);
    T('-5960763729.300403130505988', '9.8770561838186953199775822E+121693', '9.8770561838186953199775822e+121693', 32, 5);
    T('8498623.57561737572864385408605740718602697359485921', '5.49374948772979446004E+953666189619677', '5.49374948772979446004000000000000000000001e+953666189619677', 42, 0);
    T('3.553951553619241322614333988128262977687065099174900E+6323822', '7.7156206909838053087317752836956396278E-5784', '3.5539515536192413226143339881282629776870650991749e+6323822', 69, 1);
    T('1.0976E+91455', '-2.16977754622824E+7736', '1.0976e+91455', 15, 6);
    T('8.554282004803743688430820375545921E+1099775', '-9.5442765592570198143258766640018874133994722743552E+76467415059637', '-9.54427655925701981432587666400188741339947e+76467415059637', 42, 1);
    T('6.4E+99183601245986', '-2.69628894868628881695145568506077625158E-5270858452', '6.4e+99183601245986', 53, 0);
    T('7.60119930238633071201702979462431656950441669E+3851342', '-2.05803807517894027622130096613834810E+275117968394085', '-2.0580380751789402762213009661383481e+275117968394085', 69, 0);
    T('-2.6122374618704419434780423287872332786432845284E-3270544688678830', '8.23580507564445610614845664404551713242448514314395447E+1208990', '8.23580507564445610614845664404551713242448514314395447e+1208990', 54, 0);
    T('-7.0336309981828926733365477242E+3532651', '8.62275585695405319641852011535055585924051527493589559E+42750099713', '8.62275585695405319641852011535055585924051527493589559e+42750099713', 66, 2);
    T('4.010459153549544331E+303695323420', '-2.18983756609600713828285082273392341173613435E+33247154849', '5e+303695323420', 1, 0);
    T('-7.50568636130776563131608844832611072800219826253180E+64', '3.271420356467194490961E-82', '-7.50568636130776563131608845e+64', 27, 6);
    T('-4.82E+25791476', '7.18710957212090552423E+7594851', '-4.82e+25791476', 59, 3);
    T('-7.04067867075277779304944975662292E+45317102663171', '-3.9148956560762436561292741829453685E+15324444854', '-7.04067867075277779304944975662292e+45317102663171', 62, 4);
    T('-1.159513666268E+8653', '1.21860967723766830862323980E+653027716931', '1.2186096772376683086232398e+653027716931', 29, 6);
    T('4.02650358482778287204796642515037032039E+898381944802', '7.84071168034380661546664243157953247045287660401E+2460', '4.02650358482778287204796642515e+898381944802', 30, 5);
    T('5.355683248532362813E+60', '1.191783445277815615100566E+6013841', '1.191783445277815615100566000000000000000000000000000000000000001e+6013841', 64, 0);
    T('-4.51904062693047000245975E+6246404006', '6.0195912631392998275491606930207993972031385E-78592902403', '-4.519040626931e+6246404006', 13, 3);
    T('-4.4942779044799122422645627629374E+604087832538', '-6.359549825438621451540822471705628406E-23785', '-4.49427790447991224226456276293740000000000000000000000000000000000001e+604087832538', 69, 3);
    T('-8.090753924811616855652271813E+283534', '-7.848446618895061E-5003', '-8.090753924811616855652271813e+283534', 49, 5);
    T('9.4232E+6513077', '-5.262E+1150006172905', '-5.26199999999999e+1150006172905', 15, 2);
    T('9.8760765874E+431', '4.84818419853499879087419850017866E+3852951290661531', '4.84818419853499879087419850017866000000001e+3852951290661531', 42, 2);
    T('9.891990361111177209E+5687104210', '-5.530591852E+1843', '9.891990361111177209e+5687104210', 77, 2);
    T('2.41124006787866085086933983131627966448468741627373656394E+63', '-4.3982400723535382646401725075451047797414996888449617E+944299', '-4.398240072353538264640172507545104779741499688844961699999999999999999e+944299', 70, 1);
    T('-4.96961599033365064E+26004027251882', '-5.09167790E+93627744', '-4.96961599033365064000000001e+26004027251882', 27, 3);
    T('8.74749457352103955354883798196254674763053E-526589', '5.33941966293231661499086993488270E+602087273437529', '5.339419663e+602087273437529', 10, 2);
    T('8.74306905489182758974845701E+9013612137771', '1.65791415872114385452735831648865314597731143938E+3431348357642644', '1.65791415872114385452735831648865314597731143938e+3431348357642644', 61, 3);
    T('7.6754475E+2008670606418', '-3.4658908682E+18533', '7.6754475e+2008670606418', 14, 2);
    T('-4.6618186019496996896209E-1379', '-1.8037463E-4542443007', '-4.6618186019496996896209e-1379', 60, 4);
    T('7.74384258353277518745273764133862137768415083E+615364748', '-1.78702123417923777123614155600879136844876945570408156E+45875674', '8e+615364748', 1, 2);
    T('-9.582612387250917E+738781668457174', '-1.099856135994191763584265282E+2881698568', '-9.582612387250917000000000000001e+738781668457174', 31, 3);
    T('1.7970673683711261354377E+78', '-1.2032210456947E-5466732774967', '1.7970673683711261355e+78', 20, 0);
    T('8.83159917114761637839086918459375487539331878549938826713000E+1492', '7.285239789284823893088053655226538276910931939538856656E+62', '8.83159917114761637839086918459375487539331878549938826713e+1492', 75, 5);
    T('70674.388975736779359', '-7.381164095851236359E+3542', '-7.381164095851236359e+3542', 59, 4);
    T('1.0992613993583E-4749', '1.77686341590505E-107', '1.77686341590505e-107', 37, 6);
    T('-5.3950159686088438357039499587E-8152107', '-95.0', '-9.5e+1', 17, 6);
    T('6.59139207008211418640306767392017726889893269061811742320E+75486128638', '-0.42630074', '6.5913920700821141864030676739201772688989326906181174232e+75486128638', 69, 4);
    T('-0.0807919551900393378527437634067553957093885512014706045246', '-2.60503377368527660944545795966222625214241271068389E+5854622738276678', '-2.60503377368527660944545795966222625e+5854622738276678', 36, 1);
    T('8.7186695575192721247259254772020982673754216499647E+61375976', '5.444672892210171419820705034848099937068506158E+7712', '8.71866955751927212472592547720209827e+61375976', 36, 6);
    T('-8.24658E+991859410133774', '-3.92790932778673120799896086864402931E-210957', '-8.2465e+991859410133774', 5, 2);
    T('4.53E+13', '8.964306541012320484614725263987882092170E+47', '8.96430654101232048461472526398788254517e+47', 58, 1);
    T('7.25095697770577793498049170688085023200235834760907775E+37787510', '6.1718015533144594740813802581687086720E+349', '7.250956977705777934980491706880850232002358347609e+37787510', 49, 3);
    T('-1.19779761264306593099356673290644938404307E+766551', '7.566481679941536784806107462001849862589911310407560267E+9938771', '7.56648167994153678480610746200184986259e+9938771', 40, 6);
    T('-7.06124568546466937876823372002787856598325681880477749E+298660084553', '-4.19383171306938E+17872', '-7.0612456854646693787682337200278785659832568188047774900000000001e+298660084553', 65, 3);
    T('-8.91536008265035558442396753707346218801762185E+51', '6E+67242', '6e+67242', 63, 5);
    T('2.46412471536935045964706689219744109044227908E+83778', '-5.708598312800068175400060864245973079748E+92', '2.464124715369350459647066892197441090442279079999999999e+83778', 55, 3);
    T('7.87568737345179678833E+1292293750', '-8.50229287574719699771066527404542608154739886714310642980535E+3360', '7.8756873734517967883299999999999999999999999999999999999999999999999999999999e+1292293750', 77, 3);
    T('-3.5729238759633249610E+1867291', '-6.47283746352788268E+80751432685121', '-6.47283746352788268e+80751432685121', 26, 2);
    T('6.64960096354426532063532E+53849350495740', '2.01261981114309783086495324139206868164E+18941', '6.649600963e+53849350495740', 10, 3);
    T('4.5674878256E+4476823292', '730.303213746466166070736662096765060039417188548950691911', '4.5674878256e+4476823292', 38, 1);
    T('0.0000048596667222371648567261121160780367433825034263279742605281', '8.5051329891097453126936564334075932991841574161281932E+904860380027767', '8.50513298910974531269365643340759329918415741612819320000001e+904860380027767', 60, 2);
    T('-4.33899119776436748207857096280836676223783E+61', '-4.624307337933984993314613779977338455E+994846043990076', '-4.624307337933984993314613779977338455e+994846043990076', 41, 1);
    T('-1.40166388386391925790008853500236664456255861725885545E+943447707', '-4.386150170964E+281', '-1.4017e+943447707', 5, 3);
    T('7.5986658074024081413019481274091431906037980E+53464209', '5.882595113886659578288836415748576684123564453747015506937E-200', '7.59866580740240814130194812740914319e+53464209', 37, 3);
    T('-5.214725E+5531991090431', '6.3258535324058837745269484908515195188969875468239735211249E+8585', '-5.214725e+5531991090431', 36, 5);
    T('0.9449672179799878336365789599069482077311242826', '4.15841E+390', '4.15841e+390', 21, 1);
    T('-7.53380048867601925505733177207438952187053363195E-67457', '-7.163495800077073E+28100542068', '-7.163495800077073e+28100542068', 51, 1);
    T('5770195.605670587807824364082427142223', '2.2526655773682657598159652506619683597749881038852560106379E+386599406', '2.2526655773682657598159652506619683597749881038852560106379001e+386599406', 62, 0);
    T('4.7E+56773386497891', '-2.290738773875128604696496647218E-7568299094', '4.7e+56773386497891', 11, 0);
    T('-8.4455434353E+4747', '4.72384293398819587708806651997213962525425901206928525772E+13283018', '4.7238429339881958770880665199721396252543e+13283018', 41, 0);
    T('-5.40654815824257774295718815865720208252601171117295246528E+842', '-6.15E-43028701138921', '-5.40654815824257774e+842', 18, 2);
    T('5.317780298266783152066994E+1296512263048', '-8.069488495037E+7258716556922692', '-8.0694884950369999999999999999999999999999999999999999999999e+7258716556922692', 59, 1);
    T('4.4657683611504347602794350143305985812352710034061663E+121888960401097', '8.334282102984251037071895405133205609E+6787284149', '4.465768361150434760279435014330598581235271003406166300000001e+121888960401097', 61, 0);
    T('7.273142147665848038303799105792252170009957E-77683287546147', '3.4450078355036201626119260119555353405E+5365766124', '3.4450078355036201626119260119555e+5365766124', 32, 5);
    T('-5.69890141139E+41949', '-4.1723614E+8203571221797472', '-4.1723614e+8203571221797472', 9, 2);
    T('9.756900619997125227377399E+46210', '-8.4761059897052E+6795374567772362', '-8.4761059897052e+6795374567772362', 31, 4);
    T('762801032.6422629297094699018562', '9.1708322959846069425288204362188291457995066279246482491E-228', '7.62801033e+8', 9, 2);
    T('6.71968333139273910148437896252253823797365435353881768520013E-45', '7.8522967376048939405653037144858958363595445245489856278111E+447949', '7.852296737604893940565303714485895836e+447949', 37, 3);
    T('7.9396539164942990373381104587199851592694819581151E-2517575439479509', '-4.99193933892159704327812204947719210337542493506146066E+407', '-4.99193933892159704327812204947719210337542493506146066e+407', 74, 4);
    T('9.652033700737116746382369669057916734E+7120358', '4.22E-1854383826701', '9.652033700737116746382369669057916734e+7120358', 54, 4);
    T('5.2248908457360155859673766008E+144761371236', '9.749987481242002399546226019799387888821327E+1155413802305', '9.749987481e+1155413802305', 10, 6);
    T('-7.9944078131703313981060740377856651705905890E+38371953615156', '-4.4756038830314985121569398069876262735705573558964778E+63549699', '-7.99440781317033139810607403778566517059059e+38371953615156', 42, 5);
    T('2.882273934824032198220685203399122002520903252077132E+828807355', '9.1943010921456E+257378878', '2.882273934824032198220685203399122002520903252077132000000000000001e+828807355', 67, 2);
    T('-40553142060007672605180097629863.509956859144388953', '7.42383085732656574986543390037373912E-118602', '-4.0553142060007672605180097629863509956859144388953e+31', 51, 6);
    T('-8.59328024706226531640147941089402912266E+371112', '-2.9069827070169349950759238974509111323E+24653780402862', '-2.90698e+24653780402862', 6, 1);
    T('-1.44128003053018440232885320831116651620023901934037201631672E+458604', '-7.531156915731369095384496790028740685E+66879', '-1.44128003053018440232885320831116651620023901934037201631672e+458604', 72, 5);
    T('6.195509843884199473397493406084029131760443655849328E+114082582839937', '5.1E+86531184480', '6.1955098438841994733974934060840291317604436558493280000000000000000001e+114082582839937', 71, 0);
    T('-3.8040122783319934622948649452334480120E+9463832207634', '-4.5651387035787711552775225901945137E+68366358063549', '-4.5651387035787711552775225901945137e+68366358063549', 38, 4);
    T('-6.5492868E+223503962', '-1.94877857419559739073836378539457790914551730184965E+8311304144697715', '-1.94877857419559739073836378539457790914551730184965000000000000000000001e+8311304144697715', 72, 3);
    T('-7.6829275225601E+94817708', '-1.1347135144203502025149325460214453324160309939425165E+5987674712010', '-1.13471351e+5987674712010', 9, 6);
    T('5.716224810460293986210256E-948', '-7.730491232798247882598471728966212432498260457E-558395107495', '5.7e-948', 2, 3);
    T('-6.96160823991950845141E-92', '8.0248557953E+171082', '8.0248557953e+171082', 77, 0);
    T('77.059203774753930339403348607534766', '-6.4143045909803421925180940032581667725063080838E+867871643', '-6.4143045909803421925180940032581667725063080838e+867871643', 51, 0);
    T('-9.62281919905549045329677617405E+4056848563188', '6203722.92310001735366432836', '-9.62281919905549045329677617405e+4056848563188', 53, 6);
    T('6.6141411659164958077640890165047020E+95353664464976', '-1.5523950836532437851432532216557052753355367634193E+9628668', '6.614141165916495807764089016504702e+95353664464976', 52, 6);
    T('4.326761778805226071530883566400130502737504760607588E+49110578886', '-4.97386113822071442E-9140', '4.3267617788e+49110578886', 11, 5);
    T('-7.9684459189494002283819987252713126834103226268629332426E-157', '-6.958595724417726398E+8637143053372', '-6.958595724417726398e+8637143053372', 49, 2);
    T('-5.584504650839118421336312731660411E+298809985858912', '-7.8896824574420532318671403E+2433173141', '-5.584504650839118421336312731660411000000000000001e+298809985858912', 49, 0);
    T('7.29976945005E+2970062', '4.4196918E-5890300904698', '7.2997694500500000000000000000000000000000000000000000000000000000000000001e+2970062', 74, 0);
    T('-96981050659015949635698.7050105561465577', '5.66804780462357752E+2428165293', '5.66804780462357752e+2428165293', 26, 4);
    T('-8.675894393476E+926', '7.1431971270597128800575307885748E+64718962212753', '7.1431971270597128800575307885748e+64718962212753', 65, 0);
    T('7.9679574456E+225870578', '-8.7949020259225933584985576267513021933784E+581', '7.9679574456e+225870578', 36, 2);
    T('4.741303556723061495817838706984164E+825117666', '4.230375929327276293120381820519164551367736758366E+673', '4.741303556723061495817838706984164e+825117666', 37, 4);
    T('-9.803465641386819575250545051877170041144088230157930779381E+4313120408', '8.589E+83768723733', '8.589e+83768723733', 22, 2);
    T('3.183011238968946987000063330438239564197631484895216052613E+26473859', '6.9939241646994966285043373522376343E-946577', '3.183011238968946987000063330438239564197631484895216052613e+26473859', 66, 1);
    T('1.66908098583845027364757190781799507950890492E+56', '7.89324031041750994942946364383520523219625631567478E+131742431', '7.8932403104175099494294636438352052321962563156747800000000000000000000000001e+131742431', 77, 2);
    T('-3.013869826348483284213235718E+778745880286853', '-7.271513E+691888197', '-3.013869826348483284213235718e+778745880286853', 34, 6);
    T('4.1272177175624786241857870363852084E+4421396013462613', '5.597222802163407266370688931810947E+3635100571437', '4.1272177175624786241857870363852084e+4421396013462613', 76, 5);
    T('-2.47080268502270718320638045155468E+972567806', '6.3682574315779605759868626889498621687789463761756341261677E+6584001029973', '6.36825743157796057598686268894986216877894637618e+6584001029973', 48, 2);
    T('-4.102018587378186E-3756591119950', '-9.4872698401579142455644424422765205661252209024E-73', '-9.4872698401579142455644424422765205661252209024e-73', 71, 6);
    T('7.074074271606364039890688632370152026E+46035', '-1.226E+3578', '7.07407427160636403989068863237e+46035', 30, 6);
    T('9.387431215676223776E-248112665311863', '-4.15600066308289464520464E+807609445767', '-4.15600066308289464520464e+807609445767', 42, 6);
    T('-9.06398926880681819748E+88079057901591', '-5.644409932982973253967886917447085080129357916098380E-9095', '-9.06398926880682e+88079057901591', 15, 6);
    T('-8.3396008396408670850464383047782958025E-656', '-1.470106116121407260E-96304236267', '-8.34e-656', 4, 6);
    T('7.1798150921535901100277E+955536117', '9.784694E+822289382093502', '9.784694e+822289382093502', 24, 5);
    T('-6.986184477652846910945934288479259698893045069756368435E+35715', '-4.779660121035465173039572112804521573936881489297E+21070589', '-4.779660121035465173039572112804521573936881489297000000000000000000000001e+21070589', 73, 3);
    T('1.299097019613688E+2797165313', '1.5940269714177863893309E+6726222657184996', '1.5940269714177863893309e+6726222657184996', 74, 3);
    T('-5.37257835300892370813336E-81', '5.4248781875955350353008719775907380515083197914E+5357543377743', '5.4248781875955350353008719775907380515083197914e+5357543377743', 56, 0);
    T('-3.291138741310440896703212720078006893959195E+393276', '-3.5E+86383007520', '-3.5e+86383007520', 5, 1);
    T('-6.00341051257189669463603499334168089E+75', '3.83495552700638748E+8200540401', '3.834955527006387479999999999999999999999999e+8200540401', 43, 1);
    T('-8.356963590161082817220620753073E+83', '-7.661639160665840377873961792993361580444538283555991591E+71', '-8.35696359016874445638128659345087396179299336158044453828355599e+83', 63, 6);
    T('-6.1479819420445571938091928E+49', '-7.196556465250664330E+161881046', '-7.19655646525066433e+161881046', 51, 5);
    T('-3.05E+2249769', '2.2755E+5754824151', '2.2755e+5754824151', 11, 4);
    T('1.231201070169774959319635E+714168815885784', '6.7167086935008E+605034849', '1.231201070169774959319635e+714168815885784', 34, 6);
    T('6.44177142165035437254285621E+49632916571413', '6.661585935401706302E+137081972', '6e+49632916571413', 1, 6);
    T('-3.5633076246717212095752E+760830251034', '-5E+7545683', '-3.5633076246717212095752e+760830251034', 64, 6);
    T('-4E+8703760622726512', '-6.34558E+9', '-4e+8703760622726512', 39, 5);
    T('5.6799937592963783277185E-2924565644068', '-2.087210897379327023612247849E+14963639806982', '-2.087210897379327023612247849e+14963639806982', 51, 3);
    T('-4.8348942480413E+16405', '-0.04117047280678378493750499511886', '-4.8348942480413e+16405', 23, 6);
    T('-9.175900509896555790612218573843469948365970077408696984E+668', '-1.21307489364045201217762627645107745605173110595214552855893E+45486881293', '-1.213074893640452012178e+45486881293', 22, 3);
    T('-5.73313278847353949434841125890296226E-1318826', '-8.14689722665242698823274539037856915990E+3425920367', '-8.146897226652426988232745390378569159e+3425920367', 37, 2);
    T('-3.515820105716911178241473456122017386797198301279970E+613916553222', '-7.017286853931036346530189805610E+7963119509', '-4e+613916553222', 1, 4);
    T('5.289671073247062424331235E-257900528248', '1.7578E+7390823', '1.7578e+7390823', 76, 4);
    T('6.851145550206497363588835022772121907642315877044604829E-72', '7E+593890555634878', '7e+593890555634878', 32, 4);
    T('2.54685232524965416376905587359630393E+87076449', '8.14726742594742820418246E+1020560751062669', '8.14726742594742820418246e+1020560751062669', 76, 1);
    T('-8.1549008015886864E+970876', '-71990858.03759384135535681651462963505293866', '-8.154900801588686e+970876', 16, 4);
    T('-5.921656632901993119764937499020978657E-682653636827610', '1.121011225945E+999701302', '1.121011225945e+999701302', 42, 2);
    T('3.64698180782183694267335319064040666686290955608939411995E+500540631971', '8.51021151126090590827127178010672976648111938E+29023738013', '3.6469818078e+500540631971', 11, 4);
    T('2.953975502999032E-87548873400', '-6.48632587733119012224740166692935096613935095167481770E+3302', '-6.48632587733119012e+3302', 18, 6);
    T('-4.134026033727086894937694676474554659268E+643', '-3.540247534432402209654E+5559023', '-3.54024753443240220965400000000000000000000000000000000000001e+5559023', 60, 3);
    T('9.2839817379821118717E+73713149820512', '-7.055594082274142251217128308842916882118258809363294633E-506652250787657', '9.2839817379821e+73713149820512', 14, 5);
    T('7.8100243437169466055891830874139812229E+39', '23258.443944765733524611', '7.810024343716946605e+39', 19, 3);
    T('-4.377587974712386635902590799877520980386010989024740565E+682', '1.1409303017106273613450074E+6501691732931', '1.1409303017106273613450074e+6501691732931', 54, 6);
    T('1.16463776512710173280613463895535364038972312377946E+8905424605', '-8.040352883198609184196528312715621318605E+15927', '1.164637765127101e+8905424605', 16, 3);
    T('-9.51413378736567032194879837625827043165114955870689830990E+38898186', '-9.5362551731350148465644E+639339785', '-9.5362551731351e+639339785', 14, 3);
    T('-1587428.130442729511', '5.534879231123194563915182733E+4862', '5.534879231123194563915182733e+4862', 51, 6);
    T('-6.47258389E-53357530440178', '-0.00920567180478342145', '-9.20567180478342145e-3', 65, 4);
    T('9.652337574869348204233202189811896791E-524247', '-4.297E-6908207658', '9.6523375748693482042332021898118968e-524247', 35, 6);
    T('5.465123719849394812082005766014E+608559598434', '2.38919186206662472519471270573916991422096763432176301E+1427082667236363', '2.389191862066624725194713e+1427082667236363', 25, 6);
    T('-6E-2561064', '6.08194235517364481859642406514419275619409634E-1431079189', '-5.99999999999999999e-2561064', 18, 1);
    T('2.00562034345608097262924347793650E-80691051', '-3.053991E+1558', '-3.053990999e+1558', 10, 1);
    T('-4.9941119511673655091966E+80', '7.6275541744694E+772', '7.627554174469e+772', 13, 6);
    T('-8.893951622407663415E+2579228707455988', '-5.01881532477940308473234860845940973268975E+865', '-8.893951622407663415e+2579228707455988', 64, 2);
    T('-3.7494048133880179384123758486934196618506842452594E+91', '3.828095559358001006587E+4913', '3.828095559358001006587e+4913', 69, 0);
    T('-2.888780222032200457612550495271396892711485143755733E+16676646', '-8.89755497E+64', '-2.88878022203220045761256e+16676646', 24, 0);
    T('6.35801647891631E-9665879', '2.147709660245709431867095310509603198794526792E+78456905079', '2.14770966024570943186709531050960319879453e+78456905079', 42, 2);
    T('-8.97207419396047E+9115792429', '1.4163648025924400850596668E+362', '-8.972e+9115792429', 4, 1);
    T('8.06050189919400075697118296273902898172111087056E+81656542537', '1.369895151969453322218568E+56088143033542', '1.369895151e+56088143033542', 10, 3);
    T('-2.845608228024709399220E-60429', '3.1689483251658241786949486282007E+862951256', '3.1689483251658241786949486282007e+862951256', 66, 6);
    T('-7.108457651077632232428168693E+516462456', '-3.099916824128674826892354942169252E+3276360776930474', '-3.099916824128674826892e+3276360776930474', 22, 4);
    T('-9.6300E+9233860', '-2.0884950849052518461133744318965534168694481229596E+87362', '-9.63e+9233860', 79, 4);
    T('-7.558639507444093441473241290183700923937E+3316285748', '5.07375392E-87973557405', '-7.5586395074440934e+3316285748', 17, 2);
    T('-2.21573466047615710268E-3017633316339', '1.658710982535942323359817099383643315551285682261781390E+12474303812807', '1.6587109825359423e+12474303812807', 17, 6);
    T('-948312306', '8.2634110313772244612311605508E+55134849', '8.2634110313772244612311605508e+55134849', 78, 2);
    T('4.856106652050317722554E+769032691', '-3.5761427E+6969914260759838', '-3.5761427e+6969914260759838', 65, 3);
    T('1.96639170089392E+5055242', '-4E+10740160210633', '-4e+10740160210633', 77, 0);
    T('5.367285366375241228177515594725924848430702182545367756279E+261587811435233', '6.98374372867930274178249230773652182298E+298', '5.367285366375241228177515594725924848430702182545367756279e+261587811435233', 65, 5);
    T('-6.508055531474E+5863118', '-1.694228125479E+73175444387672', '-1.694228125479e+73175444387672', 26, 2);
    T('7.2758597079526581706338454548862312734E+5232144', '4.643831079E+3235602456409527', '4.643831079000000000000000000000000000000000000000000000001e+3235602456409527', 58, 0);
    T('-3.3527847280189429681599143487704856132492459352848118906800E-3290', '4.88882476635432390773283336302631918524664175782459E+71417011518460', '4.88882477e+71417011518460', 9, 4);
    T('-6.28E+3927211247920', '-6.96003301689473162354619058045517278492626328620930E+81', '-6.28000000000000000000000001e+3927211247920', 27, 3);
    T('9.3027356542449168217186461952113047656430907539632823621E+1615627353219005', '-8.404956E+82406143533', '9.3027356542449168217186461e+1615627353219005', 26, 1);
    T('-1.5532921733626548780258965726159921302258384940988066418317E+5398592', '-1.86983724923595066E+8956', '-1.553292173362654878025e+5398592', 22, 2);
    T('2.3246024E+8255781381', '-2.536590235978333822638383347588255789961752051102460268E-106178501841', '2.3246024e+8255781381', 66, 2);
    T('-4.72415839E+4037', '1.803791971463E+6037426', '1.803791971463e+6037426', 55, 6);
    T('7.689788216037499502542152984301E+8637564215699', '5.27044727588839125663830862E+4144571882', '7.6897882160374995025421529843010000001e+8637564215699', 38, 0);
    T('3.484683929276122596076249034766632128958968465260607367E+53077491464095', '9.524004237897074949823E-841184150719', '3.484683929276122596076249034766632128958968465260607367e+53077491464095', 57, 6);
    T('6.81770782321967875503811926041709964085271117303000559521E+938675330671802', '-1.0479024150364019719818883586950548669E+682203003917', '6.81770782321967875503811926041709964085271117303000559521e+938675330671802', 61, 2);
    T('2.61000071905238027977225191976154708376971501928539127070736E+8879319', '3.13433992263778344147E+51', '2.61000071905238e+8879319', 16, 1);
    T('4.125083977349541953518135350351803142595397798471959E+58360655988', '-5.6674229106866873636080440E-7436743403367', '4.12508397734954195e+58360655988', 18, 6);
    T('-8.83874570761477345555034396391E-46013336', '4.352904301927776747632757623709468433824572595224E-295', '4.352904301927776747632757623709468433824572595224e-295', 71, 4);
    T('-3.35117877458769856489332400680893E+6126866', '-9.567486808425693064061E+45', '-3.35118e+6126866', 6, 3);
    T('-3.42130795301006397320937151090811435560658653958872E+18886317556304', '7.45713589721769167132599792561890444696E+749649970385', '-3.42130795301006397320937151090811435560659e+18886317556304', 42, 3);
    T('-2.05430E+55431', '-2.708034784195140940896688400326163899123821E+53891766156091', '-2.70803478419515e+53891766156091', 15, 3);
    T('-7.766475720794974414551588033278432E+770', '1.757269865088311696E-50308', '-7.7664757207949744145515880332784e+770', 32, 4);
    T('-2.6625838334842946946747836587533961310666E+269680356353', '-1.649511168240935271266251007879902873E-28663335637', '-2.7e+269680356353', 2, 6);
    T('1.0168356024727842526779181379317467308E+797627', '-1E+492188218761', '-1e+492188218761', 50, 5);
    T('-2.34006674194789604019080831079119201828E+7640127686361', '1.26120161152982442770872903042877096657901167707789E-3047583534224', '-2.34006e+7640127686361', 6, 1);
    T('-4.98845525773E+97162928331841', '-4.246758361786776760973373598901835106795228228341E+2630660', '-4.98845525773e+97162928331841', 17, 5);
    T('-4.664376699211183852032924244541091108869895076072940E+90508703377909', '9.1467778851986E+72', '-4.6643766992111838520329242445410911e+90508703377909', 35, 1);
    T('-1.38882520067962E+89', '-2.93496289931461040684973426636568477590974368637152188E-145695294', '-2e+89', 1, 3);
    T('8.544520649105303899943252659076266259646E-46615393673', '4.4978049069549378266E+785293956036398', '4.497804907e+785293956036398', 11, 0);
    T('6.33677390474507548829230461862531168136103551E+2904319023977020', '-7.530837000958145864130849331359777599361146264116811600E+585660233', '6.33677390474507548829230461862531168136103551e+2904319023977020', 67, 2);
    T('7.5819541603019444163484763951E+8894', '-1.6494749962904349816901170773E+1444117697865', '-1.6494749962904349816901170773e+1444117697865', 50, 6);
    T('1E+239545', '8.1943506331425982990785957812229E+3548654125', '8.1943506331425982990785957812229e+3548654125', 60, 3);
    T('6.342099780107985E-8911741274', '3.507872261811528431435895806024302742658773487926279E+9099', '3.5078722618115284314358958060243027426587734879262790001e+9099', 56, 0);
    T('-1.80616011466276684201920623666300838868816970349087982017290E-243', '2.29412424884929855593602223136158552266827806380468207440E+7246894199160544', '2.294124248849298555936023e+7246894199160544', 25, 0);
    T('9.93214687E-934898780', '2.22165190925603677175317810557281382E+83546902', '2.22165190925603677175317810557281382e+83546902', 53, 6);
    T('6.907772496506026766472368875171444096611E+93844', '-1.48894626644425275554695860690441874502232894653773725381074E+666', '6.907772496506026766472368875171444096611e+93844', 51, 5);
    T('-1.506189525521172912777864474642E-84093924865', '9.4483460039583249960296204582544117730131801710872896E+3267', '9.4483460039583249960296204582544117730131801710872896e+3267', 55, 2);
    T('9.9046007826209558743020647548001E+70890914', '-2.73685895515544840695371E+34', '9.904600782620955874302064755e+70890914', 28, 2);
    T('-2.905680566934999283585E+578614470706441', '-8.616048294876501426221204960423283E-906', '-3e+578614470706441', 1, 6);
    T('3.4E+10573512062656', '-7.76343985764270056719929903E+34246323', '3.4e+10573512062656', 69, 6);
    T('5.22823573973065914382725025075322896820038702862984238E+2730219638160455', '0.0000788751418925114170789299846657500888942293958469385821847', '5.22823573973065914382725e+2730219638160455', 24, 1);
    T('7.9081861294766118851242718757005103457E+8925131228', '-3E+541722203', '7.9081861294767e+8925131228', 14, 0);
    T('-1.543106697528018368085048099009161278467895E+11751960301', '4.76308500183467E+57232896121998', '4.76308500183467e+57232896121998', 69, 0);
    T('2.177390860148379061382395462651090E+504397503', '-888874.449011794112966834', '2.1773908601483791e+504397503', 17, 6);
    T('-5.028459072288834231', '-8.7166421209647966583625408E-9378880', '-5.02845907228883423100000001e+0', 27, 0);
    T('8E+887327226', '-2.847490888261633556871865994339977681E+477597', '7.99999999999999999999e+887327226', 21, 1);
    T('4.336965475415548641667672832760836727671364926759639452E-8876014668741034', '-2.0804902073374220E-561275758', '-2.0804902073374219999999999999999999999999999999999999999999e-561275758', 59, 2);
    T('4.4650882445E+8153080066048622', '8.51743839256799139125704205066378259586085E+9721555607', '4.4650882445e+8153080066048622', 30, 6);
    T('1.87690470425959537425093261525368096815535015788E+2298964', '6.00990754E+566183', '1.87690470425959537425093e+2298964', 24, 5);
    T('8.3030501535309932169542E-9408384045', '9.93796959001975603433886016590305444368676519073635E-703505725560331', '8.3030501535309932169542e-9408384045', 43, 6);
    T('8.65267E+63300820750', '9.819383278926945575756732552125540207E+982', '8.65267e+63300820750', 34, 6);
    T('-4.814244331863942549155E+8582866', '2.9847748290735029529780650842569918183E+7675', '-4.814244331863942549155e+8582866', 68, 0);
    T('-4.065660358106897590590762946625701996878205357E+90502666445', '9.9251949401770165330321E+738', '-4.0656603581068975905907629466257019968782053569999999999999e+90502666445', 59, 1);
    T('-3.912120042246167362464E+662723', '-2.5767787097881489109354974261557797388164501813301404975E+225', '-3.912120042246167362464e+662723', 32, 6);
    T('-4.60317384707850087569645481981039971834824E+23849776762', '7.909498960900236406503E+4182995797', '-4.60317384707850087569645481981039971834824e+23849776762', 48, 0);
    T('3.8028919045528249703468468344454692547715736947528E+357', '-9.2123019978037034220764722801115772E+8998307381645', '-9.2123019978037034220764722801115771999999999999999999999999e+8998307381645', 59, 2);
    T('-3.10138990118625281712584636409536E+32282', '-727372.3316484255222417702569047889923184021468445692008', '-3.10138990118625281712584636409536e+32282', 62, 2);
    T('-9.8433189281766648259074514026432988596702607663075434E-24441', '1.2556218917412830604855847172601602808E+36345201729', '1.2556218917412830604855847172601602808e+36345201729', 45, 2);
    T('6.5155358865805316863425880193185130610154655903549718327433E+18234', '7.575993471575753921243521506858346E+266039917453', '7.5759934715757539212435215e+266039917453', 26, 1);
    T('0.000004228404233299', '-2108.28573479497433985140829445', '-2.10828573056657010655240829445e+3', 79, 3);
    T('4.41581649148122558580541260218566886856813903440806E+388539944184524', '4.7957217891304448223916586754691006391E+20165169', '4.41581649148122558580541260218566886856813903440806e+388539944184524', 71, 6);
    T('-3.858606264877973441E+89885', '7.68048379362728825784734774564206699966047198869E+80230311815', '7.68048379362728825784734774564206699966047198869e+80230311815', 68, 4);
    T('-7.24383916921480784596763691E+608217756195', '-7.1589160917793281E+6914', '-7.24383916921480784596763691e+608217756195', 53, 2);
    T('-0.0548012495316949518084963936610055470295045894335165925', '3.2781327920561334E-86', '-5.480124953169495180849639366100554702950458943351659249999999999999999999999e-2', 76, 1);
    T('-4.911541E+66', '-3.53084386042638830010116324828E+889921', '-3.5308438604263883001012e+889921', 23, 4);
    T('4.872406342488330785E+960100', '1.9113765948328005380418153364543799010492528117E-65395653163748', '4.872406342488330785e+960100', 24, 1);
    T('3.042039750530691731106662372E+8140257', '56719.66', '3.042039750530691731106662372e+8140257', 55, 5);
    T('2.112709963020153464474239474147761020211291041655097180769E-600747047843152', '-5.2521E+5067577', '-5.2521e+5067577', 62, 3);
    T('-2.78513568435409467331869280149458278E+314456', '1.033044145546324406835803842812321064479642166654629E+3345406705', '1.0330441455463244e+3345406705', 17, 1);
    T('-5.4704002413787058505565522E+126127705083271', '-5.3580920483011807758482861551856980861682337624E-3874406935000', '-5.4704002413787058505565522e+126127705083271', 61, 4);
    T('8.62929339', '6.9415386312352945363915624567311361280764686390268675', '1.5570832021235295e+1', 17, 5);
    T('-4.939594974149401902679603452653847E+50783579', '-3.824791624627135540E+649778823651962', '-3.82479162463e+649778823651962', 12, 0);
    T('-3.438931339641912096923649508115404479969E-46229439654809', '-4.765953929989599E+46', '-4.76595392e+46', 9, 1);
    T('-8.5397897714407749965238144303733440128E+83702024454328', '1142435166.487925811442686', '-8.53979e+83702024454328', 6, 6);
    T('9.72084257125212624708E+915352012865018', '-1.27226873738681345911030999841931E+9451553455', '9.72084257125212624708e+915352012865018', 56, 0);
    T('-6.227935020E-583783', '-6.16456779160E+2880762166657916', '-6.1645677916e+2880762166657916', 30, 1);
    T('602', '5.92699711427424060178181712831684242906971526E-41', '6.0200000000000000000000000000000000000000005926997114274240601781817128e+2', 71, 6);
    T('-8.040649963418298049925306826979E-9144', '4.151025E+913619950263200', '4.151025e+913619950263200', 71, 4);
    T('6.6126437649825122249903464613345461268652787825538756E-5716473999613934', '-9.04993384654970207992321577856215393068956507634113E+521148042', '-9.04993384654970207992321577856215393068956507634113e+521148042', 75, 0);
    T('9.816432E+9770', '-7.742673748297088784364965710481824210E-8863', '9.816432e+9770', 53, 6);
    T('-8.45499210005281305879577584674312073921938295557108844035E+440', '7.2372358702599104392059568338329E+9776820383580', '7.2372358702599104392059568338329e+9776820383580', 66, 0);
    T('-5.11675653E-84231', '3.80203685637426430561780370208250480405722528804460688E+58810129503660', '3.802036856374264305617803702082504804057225288044607e+58810129503660', 52, 5);
    T('4.8E+36198799', '-900.9626405601', '4.8e+36198799', 76, 5);
    T('-8.2261871112314158290397E-91174733', '-8.991729475441950E+60862105057', '-8.99172947544195e+60862105057', 27, 2);
    T('3.064570570291689397459969843035919082464916455861782888E-5751162', '-7.85760688662276091767E+42517', '-7.85760688662276091767e+42517', 73, 0);
    T('-1.98486894503063386281628130663479655247E-33', '3.8760403240789545270678131461624119638336931429422E+922777748', '3.8760403240789545270678131461624119638337e+922777748', 41, 6);
    T('7.740869470898300959863418743615430566450103704468E+4743', '4.62832834690753633853217699792321283587695289E+171992', '4.62832834690753633853217699792321283587695289e+171992', 64, 3);
    T('4.350724033232478323950227556476541E+882106498', '-7.284231019288222226852493660395443633491583E-210610', '4.35072403e+882106498', 9, 6);
    T('8.74173702137690771001732565404114941506546812E+626399830310', '1.0113263E-3884', '8.74173702137690771001732565404115e+626399830310', 33, 2);
    T('-2.9898784473373932370386139822412174533598498698148E+85646', '1.6708491450730035067520989314211053077571E-11383082', '-2.98987844733739323703861398224121745335984986981479999999999999999999999e+85646', 72, 1);
    T('-8.31130216804346947780653037478438913189353909762282960975512E+9161221', '-349465861746089349720.109312259153800819485', '-8.3113021680434694778065303747843891318935390977e+9161221', 47, 0);
    T('6.09433899677229625802286195417902575750340093913035248E+108488419539', '5.1278002942045614058579286919599946202902052910388E-7935405475792', '6.0943389967722962580228619541790257575e+108488419539', 38, 4);
    T('6.6212205457343758783922985424580131E+64004836846054', '-7.6980075679134E-515060182257501', '6.6212206e+64004836846054', 8, 2);
    T('7.018069102332389288143805E+398189219125444', '21.93787539258094242091', '7.018069102332389288143805000000000000000000000000000001e+398189219125444', 55, 0);
    T('665.518237897', '3.46377416926324827881511E+60784919480433', '3.46377416926324827881511e+60784919480433', 55, 6);
    T('4.708E+167086936863', '-3.9681277820746322284383818842002967006645038764E-1841878845343', '4.708e+167086936863', 57, 5);
    T('-8.5742217322781822E+653', '1.598690312845032388461737234697559812509010565E+895', '1.59869031284503238846173723469755981251e+895', 39, 5);
    T('-9.601202059924569405455266E+784587', '-4.053566613274753837009479689688120759E+6442749099001', '-4.053566613274753837009479689688120759e+6442749099001', 38, 4);
    T('-4.511934959309837043310839969756972878348268706302E+9110067', '6.4446666598002566511900144687780659430499511431306214750E-3638963276774262', '-4.511934959309837043310839969756972878348268706301999999999e+9110067', 58, 1);
    T('6.0E+40992539512484', '2.5149278070648E+6812256444187', '6e+40992539512484', 19, 4);
    T('2.9E+51811316872252', '-2.75171704349837150E+3979461796517', '2.899999999999999999999999999999999999999999999999e+51811316872252', 49, 1);
    T('-5.113077361E-908570910', '3.669621325150280055646688282826013E+3828183224861364', '3.66962132515028005564e+3828183224861364', 21, 1);
    T('7.012264288996770018096221449757E+81132489098475', '7.1257540236143148585237869061158937348E+55985263763', '7.012264289e+81132489098475', 12, 2);
    T('-6.3456963531024882503340679025E+897335280675', '-8.3163453521830806262386149959807193E-503006127373', '-6.3456963531024882503340679025e+897335280675', 73, 4);
    T('-4.16855234771265419328975918379557237547441390E+4699', '-650745517603983453798960.596', '-4.168552347712654193289759183796e+4699', 31, 6);
    T('-1.5914598877643596028433713586430096391172E+658', '9.09E+1275284925', '1e+1275284926', 1, 2);
    T('-6.5025471321897310326190E+2770906950', '5.4715304222505161016557077962341342581500881925989E+908', '-6.502547132189731032619e+2770906950', 53, 6);
    T('-5.165E+253429660', '-6.88797107090581515986390182398E-488028809', '-5.1650000000001e+253429660', 14, 0);
    T('5.855288211033681054724821381538743245E-93', '-9.0801606648E+92497', '-9.0801606648e+92497', 58, 4);
    T('-5.7524639257547231301E-10', '5.8E+221292', '5.7999999999999999999999999999999999999999999999999999999999999999999999999e+221292', 74, 3);
    T('1.29422135684910251706903840651619298018931156694E+24221', '-8.2969965824728889893710436982324E+48278886739', '-8.2969965824728889893710436982324e+48278886739', 76, 5);
    T('-5.91131400248495557874208526174359056764565297834E-911400315587453', '-2.85292E-6770797763', '-2.85292e-6770797763', 75, 6);
    T('7.7779082359080E+2827300037', '-3.50073404484198547850E+3438427130543', '-3.5007340448419854784999999999e+3438427130543', 29, 1);
    T('7.27874749701354657559385128919867082912405771406602667325E+3536015891195762', '-6.748243197947246554440242669637855970969288838155239E+70343084553', '7.27874749701354657559385128919867082912405771406602667325e+3536015891195762', 73, 6);
    T('7.0099E+604693', '-5.9678164528274E+12773716', '-5.9678164528274e+12773716', 32, 6);
    T('-7.4501E+56135', '-9187.87537445137878643', '-7.4501e+56135', 47, 4);
    T('5.8522768760066664153728101310E-9451245216869', '6.95212228851237125050182942610214835829114888676168155E+164250', '6.952122288512371250501829426102148358291148886762e+164250', 49, 5);
    T('7.16215638379914E+9912793', '-5.13836294046163912222187049868E+425334', '7.16215638379914e+9912793', 38, 6);
    T('2.77395063945443525931346061885569714328E+294599103795653', '-4.618264708897608970544327981443907093194E-3061222457', '2.7739506e+294599103795653', 8, 4);
    T('-5.963902637862790588483822E+615720', '-6.821863156037195712279682052888570E+2097236707098600', '-6.8218631560371957e+2097236707098600', 17, 4);
    T('-1.923505390896148930725E+1832765', '8.100485905655358647706140482109115612E+22158', '-1.9235053909e+1832765', 11, 5);
    T('-9.46860919178793535320641921343E+83183850065839', '-2.79503246544652456022082656898172269163760474798E-360028017', '-9.46860919178793535320641921343e+83183850065839', 73, 2);
    T('-3.90450646550841488E-1404089523', '1.6676281652302E+238333213172403', '1.6676e+238333213172403', 5, 5);
    T('-8.665938338125151948799863064301177839127303213927689073E+260404', '-1.033513613087766838437791547295993100620585223694E+19302163321860', '-1.0335136130877668384377915472959931006205852e+19302163321860', 44, 5);
    T('5.06117812247108070E+976918263901', '-4.48069756639935382432466443586279938494093385709801E+2519744', '5.061179e+976918263901', 7, 0);
    T('-8.336149650E+9064789210962', '7.030E+245', '-8.3361496499999999999999999999999999999999999e+9064789210962', 44, 2);
    T('5.274914E-873605166542', '-4.70E+3183723', '-4.69999999999999e+3183723', 15, 2);
    T('1.7477601136714888338314104125E+26773242883775', '-7.33590129195664929510397631363279063627104610E-7599984480', '1.7477601136714888338314104125e+26773242883775', 38, 2);
    T('8.454126309518440672368729990076067085E+973645089', '-4.711E+753764', '8.454126309518440672368729990076067085e+973645089', 67, 2);
    T('3.87E+430753', '7.01406869494717842780713913461047802229121038582359398166E+27857149408418', '7.014068694947178427807139134610478022291e+27857149408418', 40, 6);
    T('6.1666752160508625334545444924600188363875366914405091E-495068060708215', '-64409.209227182803290215787145167200371815509353915547', '-6.4409209227182803290215787145167200371815509353915547e+4', 70, 5);
    T('-1.45637069068721799353444844152492517E+34209423307636', '2.78439074045194508166236395764191192948E+506194', '-1.45637069068721799353444844152492517e+34209423307636', 79, 0);
    T('-1.5489060583286749259380468285016794866469716992382197924462E+9690375792', '-9.22773E+878214067187535', '-9.22773e+878214067187535', 54, 1);
    T('-1.4196102212183158E+710', '-9.5385879600257090880891485472804291522280170051E+4568', '-9.53858796002570908808914855e+4568', 27, 5);
    T('-4.37267631830785269158E+8781', '4.14515183675243041430030217276349414144360811047507877928911E+833', '-4.3726763183078526915799999999999999999999999999999999999999999999999999999999999e+8781', 80, 1);
    T('3.180E+3407', '-5.59497294019151483011061486293095326636570524E+345112', '-5.59497294019151483011061486293095326636570523999999999999999999e+345112', 63, 1);
    T('-3.56697488284582294778397295137118524016119429197266295286E+449204905383510', '5.1130E+974802', '-3.56697488284582294778397295137118524016119429197266295285999e+449204905383510', 60, 1);
    T('61597301632205.756702518575081', '4.0E+46610', '4.000000000000000000000000000000000000000000001e+46610', 46, 0);
    T('4.99581752211833406051262450828623569100556E+7988857', '-2.014590492184098584111808173078528589626212276718E+6451149', '4.99581752211833e+7988857', 15, 6);
    T('-5.131095765079032130520097622312E+5985534784498', '4.092892391172283182719832559944269852116789638385278E+72', '-5.1310957650790321305200976223119e+5985534784498', 32, 2);
    T('-6.63148959490879259178968153282663540614295109E+654655760736121', '4.122621756008390598083207571542437931E+21611512510', '-6.631489594908e+654655760736121', 13, 2);
    T('5.160938294447564699718460105489879422226968926334390E+223712299230755', '-1.0579950177227992E-87151533138545', '5.16093829444756469971846010549e+223712299230755', 30, 0);
    T('-9.020888071805095483862168031425453983547056211756644085899E+482814', '2.27850021003601740734737978631096137461821467938046144E-2841905901383', '-9.0208880718050954838621680314e+482814', 29, 2);
    T('-89462.9', '-2.6720E+357', '-2.672e+357', 14, 1);
    T('-1.17605E-54445', '-3.52074925897928304066572819E-37615', '-3.52074925897928304066572819e-37615', 44, 5);
    T('2.82010964222158193411188029292680030764421E+6583', '-5.562424737E-501345689', '2.82010964222158e+6583', 15, 5);
    T('5110706555066014533.90087466584728', '-8.848785321507149662893439338517064154239825E+612840702977007', '-8.848785321507149662893439338517e+612840702977007', 31, 1);
    T('1.537280119250649684693907165206060066912E+220065480913967', '2.122341993166298692722327707947909E+336779802506066', '2.12234199317e+336779802506066', 12, 5);
    T('1.06055E+91027205166831', '-1.508523661182666341233162128131634437875435212E+2115525343', '1.06055e+91027205166831', 40, 5);
    T('9.88809032803376439554215649453866426336323570020516884684769E+527', '6.65032748382083301602039368540757257472804094824425043480E-32567048266808', '9.888090328033764395542156494538664263363236e+527', 43, 5);
    T('8.4163185245182036544400E-1973', '-9.84369186410867077606779511941786227646820683302939523E+7708749', '-9.8436918641086707760677951194178622764682068330294e+7708749', 50, 4);
    T('3.1416471299109821981514416855521951422391664262E-7176196554', '5.25848388749069E+32009', '5.2584838874906900000000000000000001e+32009', 35, 0);
    T('-4.568999030286021117E+3673', '6.7763191405127392311776865308E+80351', '6.776319140512739231e+80351', 19, 5);
    T('-0.000086433898997616357796', '7.7184156339420206831240601231761209432564160867E+6525967', '7.718415633942020683124060123176120943256416087e+6525967', 46, 6);
    T('1.6329E-5274', '2.87436670477604344158949437262258970730014728066364535651E+676261', '2.87436e+676261', 6, 1);
    T('7E+687312561376', '9.2699930471209834936381164363E+8639144657152', '9.26999304712098349363811643630000000000000000000001e+8639144657152', 51, 0);
    T('2.837439E-238', '-9.13853262707874659524776768658908026397050865E+661128802774', '-9.138532627078746595247767686589080263971e+661128802774', 40, 5);
    T('-6.46597940709829456863884063974266287504869100543E-394611', '1.10014729825968387806854561913467E+555936811115', '1.1001472982596838780685e+555936811115', 23, 1);
    T('-3.240796667519E+95334387', '-1.846644152692507581679008437077100741255856021594E-902371810402', '-3.240796667519e+95334387', 19, 6);
    T('-4.82639442341171630577178229548162764684393262390044202058752E+44097289', '4.81763773209590760256079579E+2344120046753', '4.81763773209590760256079579e+2344120046753', 61, 4);
    T('2.8168515858613183283812230435849347154828811279038E+1336', '6.89276721645644618236156265740411356188E+355615612462', '6.89276721645644618236156265740411356188000000001e+355615612462', 48, 0);
    T('-9.318794378653974131988E-798079394717', '-3.55073374672489282456130828046503409552851170066301894111845E+17091993', '-3.5507337467248928245613082804650340955e+17091993', 38, 1);
    T('7.81850440272977638322160283942462913593268600710571034E+200228897', '1.4097700562455044379062784912794785724087170341874672E+981327490663047', '1.40977005624550443791e+981327490663047', 21, 4);
    T('-5.0467801904508057331916699118E+7584388485', '2.6594442189085428862972585425', '-5.04678019045080573319166991179e+7584388485', 30, 2);
    T('8.76016279616787094819960989032620E-465645481877149', '-3.76306748619347719589E+7203125396839', '-3.76306748619347719589e+7203125396839', 45, 3);
    T('-3.6323423E+8255171', '9305.5378702', '-3.6323423e+8255171', 23, 4);
    T('-7.651940066791006518193948E+271568044883', '-1.4618321E-8859969017495', '-7.651940066791006518193948e+271568044883', 60, 6);
    T('-1.74006296386127349294940717797789857357634033E+759226', '-7.2768482723119E+146', '-1.74006296386127349294940717797789857357634033e+759226', 67, 5);
    T('2.162572E+82474', '8.79862109193882183221185669952934905215545935551098835E+816', '2.1625720000000000000000000000000001e+82474', 35, 0);
    T('-6.974207307706719696915709782996808580923975E+165584009011706', '1.59298565905849014807582612461032285693780765090568986', '-6.9742e+165584009011706', 5, 2);
    T('-5.54714992599494503263325462029997992692221231290846277E-7256', '5.61E+4758', '6e+4758', 1, 6);
    T('8.51768197534774976404388601592514901647841402287650108E-6635973587', '2.262178314927525177763042322457541184564057844E+4433569534738', '2.2621783149e+4433569534738', 11, 1);
    T('9.6502777307132820559422E+83776662', '6.4071323679350767948223228319493429785E+437309', '9.6502777307132820559422e+83776662', 63, 3);
    T('1273718371886896.713395968469625442122939920202976719696', '-4.4666989781102117931501266103925595E+9947717861695', '-4.4666989781102117931501266103925595e+9947717861695', 36, 4);
    T('9.549339621641774193642633486388152563907226E-1510', '-4.86184541549122997440082796654065336068038985793E+3960965672', '-4.8618454154912299744008279665406e+3960965672', 32, 2);
    T('-9.58265846E+3985', '-6.6575434055610505863443126332094055572565223927E+362532481', '-6.65754340556105058634431263320940555725652239270000000000000000001e+362532481', 66, 0);
    T('8.6565151E+2883', '3.11510064789240805E+557101414', '3.115100647892408050000000000000000000000000000000000000000001e+557101414', 61, 0);
    T('8.867359637871043567934477177711542128533364E+6607934', '2.5080888313068306448846480231431285429E+53074149963', '2.5080888313068306448846480231431285429e+53074149963', 44, 3);
    T('7.588349406663432878455E+34117662601067', '-4.7130798160633435603159426360434860219209E+9144645853907', '7.588349406663432878455e+34117662601067', 64, 4);
    T('5.3E+456701336', '8.17426760973217E+7249308728699', '8.17426760973217e+7249308728699', 61, 4);
    T('6.926238242500450843080184E+94153628609', '-2.88E+26886511950', '6.926238242500450843080184e+94153628609', 36, 4);
    T('-3.1402178796778E+6920391297452228', '5.39E+1140596599994665', '-3.1402178796778e+6920391297452228', 72, 0);
    T('-4.9116742953196877839892E+4331', '4.8444785E+24', '-4.9116742953196877839892e+4331', 39, 6);
    T('1.7249368039458559034881E+98', '-4.399855254870675029444393215861208338887E-68787', '1.72493680394585590348809999999999999999999999e+98', 45, 1);
    T('-1.19496E+67170675351593', '2.571315214202343041838935776281E-7041298', '-1.19496e+67170675351593', 42, 0);
    T('1.81383756881337204434259E+479', '3.291521658258054459815986966554354473521358219E+161772', '3.291521658258054459815986966554354473e+161772', 37, 3);
    T('-7.14143707942485348018E+1512309127635', '-5.406104052219232170705444635718497626162369914188E+958469930526', '-7.14143707942485348e+1512309127635', 18, 6);
    T('3.4E+451214534124632', '3.10568834080673454188597637977168462733E-171541329157', '3.4e+451214534124632', 16, 5);
    T('-6.1404892615888E+2766', '-4.204854494995908483E+347', '-6.1404892615888000000000000000000000000000000000000000001e+2766', 56, 3);
    T('-6.04718E+41475660', '1.9723743606004184703792E+543786075', '1.9723743606004184703e+543786075', 20, 3);
    T('-9.918674484362347949280337121907246627703953769457358E+1867', '1.335E+561014', '1.335e+561014', 78, 5);
    T('-4.831169923E+36363022908755', '2.87040406491900530193E+7173189', '-4.8311699229999999999999999999999999999999999999999999999999999999999999999999999e+36363022908755', 80, 1);
    T('6.16508818533929379091234142971322566580382838788427E-839', '-3.23306360E-8010955335', '6.165088185339293790912341429713225665803828387884e-839', 49, 4);
    T('3.691079063698847303398637441E+84076308', '-9.311073115936942979839091641999699482365648280636045E-7534', '3.691079063698847303398637441e+84076308', 39, 5);
    T('4956.9052180846824886759848369213167236224', '-9.063108633513402E+53061316250765', '-9.063108633513402e+53061316250765', 19, 5);
    T('1.03402733984031764367940152099095604586E-682', '2.21241431941972706260070287387498461619721E-1778501', '1.034027339840317643679401520990956045860000000000000000000000000000000000001e-682', 76, 2);
    T('7.3621081565934573727703223925E-108594', '-4.22040366E+99', '-4.22040366e+99', 46, 5);
    T('4.543650938985371E+1802244270', '-1.8693137635201E+5280923905', '-1.869313e+5280923905', 7, 2);
    T('-9.01131814492E+335', '-51735250101233186730816945.935846204838', '-9.011e+335', 4, 1);
    T('12639549022.214676120297094911044824422399424944', '-3.958307810881537856000520371801173970274331E-9', '1.2639549022214676116339e+10', 23, 4);
    T('1.433192E+965587', '7.41896751430162181058531E+599876026', '7.41896751430162181058531000000000001e+599876026', 36, 0);
    T('-6.439778245268109293228478588245098710205625563565843107634E+682079514', '-4.224834833811817326941522896078475252605556227711E-187205', '-6.439778245268109293228478588245098710205625563565843107634e+682079514', 65, 2);
    T('9.5487301106770488205230682701E+7851893196385635', '4.28020816E-67908645216', '9.5487301106770488205230682701e+7851893196385635', 30, 4);
    T('-3.52079494420491036820556990188529748E+764', '5.836185752136128132270039895280682E+812636478329497', '5.836185752136128132270039895280682e+812636478329497', 76, 0);
    T('-5.0190968620359246954309212182441023391E+4369181', '-1E-6524628495615515', '-5.0190968620359246954309212182441023391e+4369181', 52, 2);
    T('2.770476601971972728765351960E+15010', '-9.3621918557532624278412867443696445017485665460383669494327E+24485466', '-9.3621918557532624278412867443696445017485665460383669494326e+24485466', 59, 1);
    T('5.38367552379554685494311342855691980829057282523E+638467', '3.02908080735657080798780075E-9013747742', '5.38367552379554685494311342855691980829057282523e+638467', 52, 6);
    T('9.94717036116255713E+86911', '9.9116026877E+21505557', '9.9116026877e+21505557', 49, 5);
    T('5.6087489065579264E+646', '-5.967409559343308102685E+82584674693', '-5.9674095594e+82584674693', 11, 3);
    T('-5.9338871911850985955858919706E-75337195906954', '-7.309E+63', '-7.31e+63', 3, 0);
    T('-3.2646925963E-72281474137', '7.3359120381224885089641589782739657E+20179446', '7.33591203812248850896415897828e+20179446', 30, 2);
    T('5.07578313703739670161901225824833600081197664906E+911448457026', '6.09142396075224959014758581984663130E+69', '5.08e+911448457026', 3, 4);
    T('-2.995203293338128187886136260868782969733668244E+4484353653571756', '7.697557238400846124789204553687732409833518600399767E+90713133462556', '-2.99520329333812818788613e+4484353653571756', 24, 2);
    T('-9.6224994337E-66586597419', '6.5960052096355773871289824110554307896124903216919724719591E-1354266385', '6.5960052e-1354266385', 8, 3);
    T('-4.00478100026873E+309511884', '-3.1481530774702548047847053333512917365394518338E+95204816', '-4.00478100026873e+309511884', 58, 1);
    T('7.6807926764E+43245634191', '-1.19013255470935231752690616912567E+5226', '7.6808e+43245634191', 5, 4);
    T('7.506428299090929654523765955806262701868335270999E+2295', '-3.2829826243862368568349868908369639E+84', '7.5064e+2295', 5, 6);
    T('-1497051525.1624428917717049106038383713332137343', '8.10677087048267255385424507364928901160107716511608077515E+14545623890264', '8.1067708704826725538542450736492890116010772e+14545623890264', 44, 0);
    T('-9.27938487564202240029702713234978442765764165361E-9463140843', '-2.46696E-28233475857281', '-9.27938487564202240029702713234978442765764165361e-9463140843', 52, 5);
    T('-9.5320879785639833139373565E+325', '-2.13261E-640298034657', '-9.5320879785639833139373565e+325', 64, 4);
    T('1.24326874E+644199394796018', '6.241529364806310657576169492383288019779774442841710560E+8452326714', '1.24326874e+644199394796018', 15, 5);
    T('-2.15641234074265452878397083821E-10527678', '8.46607669350037077590923806974961557064E+14821337806', '8.46607669350037077590923806974961557064e+14821337806', 66, 6);
    T('3.029500766522490302412612095221700817625413641518404602783E+8796', '4.44258E+396272697', '4.44258e+396272697', 30, 1);
    T('4.143148423255432717935E-626123837', '5.084274091274941154975503864040112305E+106', '5.084274091274941154975503864040112305e+106', 50, 6);
    T('-2.7662056379489540526372703844614696077039E+8849', '7.6799908E+35823064479', '7.6799908e+35823064479', 20, 6);
    T('2.69626780188118342983185586530773229979618475E+6102562689935', '-8.8851E+7896838697', '2.696267801881184e+6102562689935', 16, 2);
    T('5.011036913315071825000179170651884464616452055627962880E+56653763', '9.2E+922302442', '9.2e+922302442', 57, 5);
    T('9.867E+587506449254546', '-9.12299426427551309288E+12789584', '9.86699e+587506449254546', 6, 3);
    T('7.14551516E+56192', '-1.39548747689728944E-52843751', '7.14551516e+56192', 11, 6);
    T('6.80573378407846014294145285716082542453748086272852E+667588842', '-4.226586561776829886353634250933E-7253953965896927', '6.8057337840784601429414528571608254245374808627285199999e+667588842', 56, 3);
    T('-8.50564197446358947441E+1434685720795', '-1.79953145E+4317533391834179', '-1.7995314500000000000000000000000000000000000000001e+4317533391834179', 50, 0);
    T('-4945991676.58', '2.34075559753525186311277312241E+761220492192601', '2.3407556e+761220492192601', 9, 4);
    T('-4.52684746286455074938497641274815754E+139', '-8.9728791583450925041443608306185E-89932566042427', '-4.52684746286455074938497641e+139', 27, 5);
    T('-6.07498067884296937574611414337262651661778761434243329871134E-838442911', '-7.7584816894323407053210956576355398923069946093760608813177E-5675081182920', '-6.07498067884296937574611415e-838442911', 27, 0);
    T('3.6492003647150357881E+22061609', '8.3903177170894E+295546782200', '8.390317717e+295546782200', 10, 3);
    T('2.408471625128965022480634317644835372287822658637581E+604885470239808', '9.0292671E+37851738', '2.408471625128965022480634317644835372287822658637581e+604885470239808', 61, 5);
    T('-2.56550448242606875358820167E-11807494625042', '6.85068702174550090593479537832590555922508353321E-6794859266011', '6.8506870217455e-6794859266011', 15, 4);
    T('-5.05206876267376024093761339755378280E+707432', '7.793354341610630878298314836882161607754766E+24976', '-5.05206876267376024093761339755378279999999999999999999999999999999e+707432', 66, 1);
    T('9.724625134927495645446474575011E+708027903363', '-4.69634157907782972107089519109676443721436537722E+14787833', '9.72462513492749564545e+708027903363', 21, 6);
    T('-3.569695385995450891868068586552478E+35828032638', '6.054342E+11497503217258', '6.05434199999999e+11497503217258', 15, 3);
    T('9.2318E+90461', '6.7798516919E+9241702779008', '6.7798516919000000000000000000000000000000001e+9241702779008', 44, 2);
    T('8.3152820833E-70287382720490', '2.2697585834154083462287549E+440477', '2.2697585834154083462287549e+440477', 29, 4);
    T('5.79198473290E+22', '7.441022406842131243330247726274786706614714417039106564503E-1948633572', '5.7919847329e+22', 48, 1);
    T('2.34343225904012701236166410E+664525637490189', '8.413149302649553491550580594900694576969590487859500410E-89666558', '2.34343225904013e+664525637490189', 15, 6);
    T('-1.0581815176391974032969210128E+99', '-9.964678728462631584124460392223E+156', '-9.965e+156', 4, 4);
    T('-9.6631585739846859314276127958332498304496E+63681750647', '-7.1256207910523734230E-3362', '-9.66315857398468593142761279583324983044960000000000000000000000000000000000001e+63681750647', 78, 3);
    T('4E+285001684988', '-5.854513850E+8050215292443', '-5.85451385e+8050215292443', 53, 5);
    T('5.13705373909627162704754207408112345466E+685578', '2.16406430065965866183088609080E+98527004337', '2.1640643006596586618308860908e+98527004337', 35, 5);
    T('2.904529795948841104786877644724794931E+4781308', '-8.0795328677017552516434475728934946961969E+74862846454', '-8.0795328677017552516434475728934946961969e+74862846454', 74, 3);
    T('2.77193687256090194763000771', '-7.6201080886073055490247230546781602641956595296813E+1288231', '-7.620108088607305549024723e+1288231', 25, 2);
    T('-6.8916248874129238145599380959E-548647', '-5.98462381589488863E+7606165609301', '-5.98462381589488863e+7606165609301', 58, 2);
    T('-7.0564144898262344553065884205643554944570630594776701E+703899254639', '-1.301548928019510465708823235676208902895694359745562799E+987211', '-7.056414489826234455e+703899254639', 19, 2);
    T('-1.18555838734986974151E+81628695609645', '-7.505466065326291949603829333258066421238072056E-7116880241', '-1.18555838734986974151e+81628695609645', 60, 4);
    T('-8.32623723808178506684', '-4.5396949289487874619676039753E+4795642468661', '-4.5396949289487874619676039753e+4795642468661', 62, 1);
    T('5.506901072218677159058961003364511687711218919906E+87161037', '-6.370790559020578003893242406354E-2039205427', '5.506901072218677159058961e+87161037', 26, 6);
    T('2.13501868147409229930649171606234921536962257265791596080278E+941628259444227', '3E+77096583676115', '2.135018681474092299306491e+941628259444227', 25, 3);
    T('9.39637E+43336', '6.0063452574773777524659307464246131254492806815017418057E+629737102587', '6.0063452574773777524659307464246e+629737102587', 32, 1);
    T('-6.0212179717E-265603298732', '6.7367533267676820979921137E+124142321600', '6.74e+124142321600', 3, 2);
    T('8.6508508801918E+25892501376562', '-1.9879324143E+6602295582135', '8.65085e+25892501376562', 6, 4);
    T('4.096731436305E+67176559619821', '474614278.0361118614779184980586196975750542045', '4.09673143630500000001e+67176559619821', 21, 0);
    T('8.4815535641231E+530454398', '-7E+45379390', '8.4815535641231e+530454398', 43, 0);
    T('3.709093187E-5601', '-2.694758726236043124E+22', '-2.694758726236043124e+22', 27, 4);
    T('-4.79847280494309549550328654412968421711485228998989682377E+102572', '7.536911582E+65575', '-4.79847280494309549550328654412968421711485228998989682377e+102572', 73, 3);
    T('6.91E-20916', '9.10021723093215257405300284238389503921367207622350E+48350795', '9.100217231e+48350795', 11, 2);
    T('9.976676403537996338266663E+28', '-7.47014651846E+51597482926887', '-7.47014651846e+51597482926887', 80, 5);
    T('-3.0707576641404951901E-366', '-7.754803454385403784145439230128192414654281838247101E+2077077', '-7.75480345438540378415e+2077077', 21, 6);
    T('6.772E+962189210061', '3.48302004448163550637675277412665E+8262555071055', '3.483020044481635506376752774126650000000000000000000000000000000000000000000001e+8262555071055', 79, 2);
    T('-8.1862065729254635321053869497880428105169018E-16430617954357', '-2E+8438886469381115', '-2.00000000000000000000000000000000001e+8438886469381115', 36, 3);
    T('1.617518095735423E+709340091767', '-1.53250180931883880830659539E+976431367186728', '-1.53250180931883880830659539e+976431367186728', 45, 4);
    T('-9.697144928128964911422E+9981', '3.94143487337125745133392E+805785876', '3.94143487337126e+805785876', 15, 0);
    T('1.3968148207474717922287328589203658332542979861190E+5782168093', '-8.14829567877349309358098564538378343611348E+59990855621', '-8.148295678773493093580985645383783436e+59990855621', 37, 5);
    T('-2.299765356651596733706327720051179153795962513126590E-8615956847108', '5.28361262019016908206514E+4423', '5.2836126201901690820651399999999999999999e+4423', 41, 3);
    T('6.77265685812954184127289046034186867E+7153504535038', '-2.355568706840E+6971711780421164', '-2.35556870684e+6971711780421164', 57, 6);
    T('3.71234355986777772904966341841981113609205E+138008086619', '-3.0606930422983683314596631118245725445E-17009092442067', '3.71234355986777772904967e+138008086619', 24, 0);
    T('-6.38318417936358262779378492636982890334820198990199707750E+210295578978420', '6.7680388E+183694802842', '-6.3831841793635826277937849263698289e+210295578978420', 35, 6);
    T('-1.970544053935754804843188358798419519E+5097268', '4.61E+1703948070075', '4.61e+1703948070075', 17, 4);
    T('7.3818176183054685644524997040666293551138649028E+149849', '-8.4778315885708582024239178051736514947199E-7398374543289023', '7.3818176183054685644524997040666293551138649028e+149849', 61, 2);
    T('-7.46078134691760302470250555558300989654E+1496', '6.61514153813655171681480531125304E+989367', '6.61514153813655171681480531125304e+989367', 58, 4);
    T('7.353309574168587363284424792903824079596E-773425750284', '2.91829989204591725911794435920346621477E+18808748', '2.9182998920459172591179443592034662147700000000000000000000000000000000001e+18808748', 74, 2);
    T('-3.85696219946940272108592118145496787850726464417E-6964287434192432', '-6.0082007391874112167622663650566757169917E+21588', '-6.008200739187411216762267e+21588', 25, 3);
    T('-1.8524489125744731E+391969', '-2.8236320026274539287801522376620097066852519480402228121E+277', '-1.8524489125744731e+391969', 19, 5);
    T('1.907233282992970749128849745402253850E+514114', '-7.7789457310283669152570E-27617227192', '1.90723328299297074912884974540225385e+514114', 57, 2);
    T('-7.7311971864758625039269548890E-978155225347986', '-5.5021674248995650718190132645337E-135778164', '-5.502167424899e-135778164', 13, 1);
    T('-7492.416754159733', '3.43452287981412343253E+89', '3.43452287981412343253e+89', 34, 4);
    T('-7.6123352918646975371914368634425441E+52776', '9.99506751814167339441070148480143062813362E+64559661138776', '9.9950675181416733944107e+64559661138776', 24, 6);
    T('4.87313498474820086303013E+209132907236029', '4.18602394315909567253588958373451298123410748124892000933E+579', '4.87313498474820086303013000000000000000000000000000001e+209132907236029', 54, 2);
    T('-1.459912591733E+41643', '4.1146144957858116E-69460656078068', '-1.459912591732999999999999999999999999999999999999999999999999999999999999999999e+41643', 79, 1);
    T('-6.0823455232428449E+263997', '4.565713067780E-718018532', '-6.082345523242845e+263997', 16, 6);
    T('-9.8138023759092264706123630035104175498289328823E+12399593', '-2.11349142710562024796447720581731550125158261E-28992360974760', '-9.8138023759092264706123630035104175e+12399593', 35, 6);
    T('-9.0452809512749146983413454277434804073298467874149046E+2747749187', '1.447539067323885687888143198269537898645623687448E+24260989461013', '1.4475390674e+24260989461013', 11, 0);
    T('-7.76900108020877925494327E+280522', '-1.40935440702755968197898459687664325132271E+167822177045', '-1.41e+167822177045', 3, 3);
    T('6.249395131740360437749041676158633553E+7504902420', '-8.07E+54890432317', '-8.07e+54890432317', 52, 6);
    T('-6.27965755E+61443568183868', '-1.398460024203922522', '-6.27965755e+61443568183868', 19, 1);
    T('-5.28147597520780226049417E+52995796248', '6.833513E-79054142819326', '-5.281476e+52995796248', 8, 4);
    T('2.40727870315896221973896355652326841327E+944009735', '-6E+43394', '2.40727870315896221973896355652326841327e+944009735', 67, 0);
    T('-6.5995313155958207E-98882761528633', '-6.730556060549902394524860473184463E+13212908085207', '-6.7305560605499023945e+13212908085207', 20, 6);
    T('7.39E+9526', '-4.17947294919498028169106498121343043190167079E+691', '7.39e+9526', 4, 0);
    T('4.67999680599965E+4640076', '8.7119750980967091650433360618468185383426439E+9298978255210', '8.71197509809670917e+9298978255210', 18, 4);
    T('5.6888350629700553413050425003082573924215134885541252168E+13642393146', '8.07623571234849819491022921461171548631633355303E+5455', '5.6888350629700553413050425003082573924215134885541252169e+13642393146', 56, 2);
    T('5.520964862E+4790135626094587', '1.8975340448486730801179976141E+7504140154997780', '1.897534044848673080117997614100000000001e+7504140154997780', 40, 0);
    T('1.96123107233870968802094431275863084942895E+6695182672821587', '-98.70910782244939892685787478402', '1.96123107233870968802094431275863084942895e+6695182672821587', 58, 5);
    T('8.54126250930835968176043031092624032821900439500986313670950E+94', '9.1464897505218878343514618145302900577744247551006847238584E-941', '8.5412625093083596817604303109262403282190043950098631367e+94', 56, 5);
    T('3.813102725869442951610862059569538549465762208350E+410957404', '-1.2984594727734763900845265188E-50508910441075', '3.813102725869443e+410957404', 16, 0);
    T('2.09021416119131713318796468475568221E-3615402520', '-4.28664E+92977124', '-4.2866399999999999999999999999999999999999999999999999999999999999999e+92977124', 68, 2);
    T('-7E+327969711201', '-1.78995316069757347800083E+470190933', '-7e+327969711201', 73, 6);
    T('9.60998252487391442486478405989646549777678030691188E-8377', '-5.8967026694516E-9', '-5.8967026694516e-9', 52, 3);
    T('1.1346967495850033572E+379710925', '7863447286169234933437589227199377509.7191047175', '1.1346967495851e+379710925', 14, 0);
    T('4.99376987328613388805214909571769090054134E+95450', '-8.41923119643137280696129177676E-7595', '4.9937698732861338880521490957177e+95450', 32, 4);
    T('0.000006530909717228146080', '-2.094E+6527', '-2.094e+6527', 56, 4);
    T('-9.85524308442E+173504423', '-3.306290023521912419663959972273578470562960762990E+6188369633', '-3.30629002352191241966395997227357847056296076299e+6188369633', 69, 6);
    T('1.2724971303521560903604830261323071971E+370158', '5.609E+97', '1.2724971303521560903604830261323071971e+370158', 65, 6);
    T('6.308716616208578732E+2707511849571314', '-3.8249426927638190593023205705166251765110867942228007836815E+913', '6.308716616208578732e+2707511849571314', 43, 0);
    T('-6.704711695140582655394427994553280218910834741127E-24', '-7.474341143522138803507157893633956457249380076E+587647833542', '-7.474341143522138803507157893633956457e+587647833542', 37, 2);
    T('-5.60E+598234470552746', '4.489892864591563E+3874193088651', '-5.5999999e+598234470552746', 8, 1);
    T('-9.2865E-4903400389319', '-9.2676167928517842E+9379571', '-9.2676167928517842e+9379571', 23, 5);
    T('-1.420192781790315424923404356896011512937713147619E-602260524306', '8.7735029635932704830013E+6822086025585', '8.773502963593270483001299999999999999e+6822086025585', 37, 1);
    T('3.707293075797653244409978875E+1825189689', '3.754115E+47351', '3.707293075797654e+1825189689', 16, 2);
    T('-4E+727977449713421', '3.9064376408E+721988347104', '-4e+727977449713421', 79, 4);
    T('7.1535381792618E+62862784', '2.887486279396276927824935526297446805456478213500867219918E+7862546164531658', '2.887e+7862546164531658', 4, 6);
    T('5.3794759423455345179525571166599682062932631493128647685817E+7988483346', '6.819862350027144886614322192725705472597287459503945119E-5685633750', '5e+7988483346', 1, 1);
    T('-72807455137.02', '-3.4903833841760846441549895567736E-967944597231118', '-7.280745513702e+10', 36, 5);
    T('-1.44112E+2520616551276922', '-8.5991475034755569499252872906666784451705462E+527388', '-1.44112e+2520616551276922', 59, 4);
    T('-3.86E+91429', '-7.772218720216744458503044856979736906923203056988E+78579634318', '-7.772218720216744458503044856979736906923203056988e+78579634318', 78, 4);
    T('-6.15225697653439988743697864709355800432550225264772E+695', '-9.4464089925417543392022338280082849327687539314267E+89954888', '-9.446e+89954888', 4, 5);
    T('-8.733662209615856406349387E+6667762917', '1.5107135885178207772375796598877779124E+83688778160658', '1.510713588517820777238e+83688778160658', 22, 5);
    T('2.46536298384855159650935609242E+24107', '-6.4860547885873969573E+42121293151468', '-6.486054788587396957e+42121293151468', 19, 5);
    T('-8.17E-12254053138512', '-4.103366802359331843917776128061', '-4.103366802359331843917776128061e+0', 77, 4);
    T('-39203437.292854897567', '3.5329676846078057737E+16849488946033', '3.5329676846e+16849488946033', 11, 1);
    T('-2.75579435034021157707405842810748869035960786152130497E+9272', '-6.2610438094704582633931669368919795234359810482306417135953E+77192', '-6.261043809470458263393166936891979523435981e+77192', 44, 1);
    T('8.2655798858756823506926111002457013164099813969108686983E+39036391195789', '-7.1823587972054554828214163508640040409E+545453126291543', '-7.1823587972054554828214163508640040409e+545453126291543', 71, 3);
    T('3.13978500197E+37179422', '1.495096854162264994793722E+94170860449', '1.495096854162264994793722e+94170860449', 58, 3);
    T('4.068746746771935650848E+33282', '-5.98100E-6338', '4.0687467467719357e+33282', 17, 5);
    T('4.3467977528660327E+819', '-6.4079009761004505719172957624617666076850101E+6685', '-6.407900976100450571917e+6685', 22, 1);
    T('3.2064186566546513492275633140446686973053E+124637966923', '2.9535E+3821', '3.2064186566546513492275633140446686973053e+124637966923', 54, 6);
    T('329120729.44995473115707622299470607033715442531268874', '-7.086311878801119740330657594458185377162680978548779717E+300383', '-7.086311878801119740330657594458185377162680978548779717e+300383', 69, 0);
    T('6.1003873376733957997675413229025638242801E+4184475', '3.4273930981876E+80', '6.100387338e+4184475', 10, 0);
    T('-3.04024025983746410151907244310456E+82319755531', '6.2938498572761183752252007216044133931431469713573389765655E+79230', '-3.0402402598374641015190724431045599999999999999999999999999e+82319755531', 59, 2);
    T('4.07738578335399170323E+77', '9.0211645035900795E+844426760428', '9.0211645035900795e+844426760428', 47, 6);
    T('5.60272480075309090806081979E+85858', '-4.170993372462272734021054726447589E-990353095946413', '5.60272480075309090806081979e+85858', 27, 5);
    T('-2.8188311181759916760190362609170023143888037E+78', '1.094E+1551041491', '1.094e+1551041491', 18, 2);
    T('-3.238360706131327368177703158826075544181737E-8613614867', '-1.688915954366194472240E+9086714192', '-1.68891595436619447224e+9086714192', 42, 6);
    T('-1.6554844159873772458E+92333', '-6.801787162346218220654193352588615372475721393922769092071E-62731', '-1.6554844159873772458e+92333', 36, 5);
    T('-5.1E+117638417636', '5.769211356353121964925484640547307538467847692241291E-40646680781', '-5.0999999999999999999999999999e+117638417636', 29, 2);
    T('-3.726111368358825338939269759002043967961E+3128241410', '-1.6914592E+409180', '-3.72611136835882533893926975900204e+3128241410', 33, 1);
    T('4.80010E+1501974131', '-1.87736635493673116267814E+9937689365731', '-1.87736635493673116267814e+9937689365731', 42, 4);
    T('8.03643924005104798E+44', '-2.8895740859834067372312864169228978588641989795E+62', '-2.8895740859834067291948471768718498788641989795e+62', 64, 0);
    T('-5.53706165483912323993719050197946E+47154483', '-2.48630868356850033079698450090434658252083432014426030114E-384403401434', '-5.53706165483912323994e+47154483', 21, 5);
    T('-2.055772428857842923490238E-129216', '7.624358943387298923E+22117', '7.6243589434e+22117', 11, 0);
    T('3.0439654961430884E+7571', '4.9000203593250029901020E+55', '3.0439654961430884e+7571', 52, 1);
    T('5.65540136571523047778189021453E+72296201', '-8.828936851696988865374196436946116506396129241953544914E-3847710217346', '5.65540136571523047778189021453e+72296201', 79, 0);
    T('3.2060589287192754E-1335248395218', '3.4958881721E+262', '3.49588817210000000000000001e+262', 27, 0);
    T('-9.7471E+493743897', '-1.382811250307883467222001714875765347E+65174', '-9.7471e+493743897', 20, 5);
    T('2.5878518120E+7447', '4.51576151786438170586853274174508630495000633449E+68268185', '4.5157615178643817058685327417450863049500063344900000000001e+68268185', 59, 2);
    T('8.0342092062390E+6710703329399445', '60849145.450268730033875664858136922', '8.03420920623900000000000000000000000000000000000001e+6710703329399445', 51, 0);
    T('3.59640073E-6240', '-7.336950949212079591464644462387091494931147E+96962', '-7.3369509492120796e+96962', 17, 4);
    T('-7.1146692426464928950535296724196080785965611E+36235316270', '1.0543767824401859342963919E-586354473', '-7.11466924264649289505352967242e+36235316270', 30, 5);
    T('-1.130457603391925183785510097125637872836E+24083065527692', '4.3534942726901E+143399', '-1.13045760339192518378551e+24083065527692', 24, 6);
    T('-3.530228040536702200E+4430733', '2.62583130846461896127398562E+894322890', '2.625831308464618961273985619999999999999999999999999999999999999999e+894322890', 67, 3);
    T('5.22345603068229417244595E+8073477', '9.925529583339962825078615125243346877710931025440099021673E-738', '5.22345603068229417244595e+8073477', 56, 3);
    T('-2.13544878730749244097094347206755935E+88566883329', '-9.192797357968280961038625419728304991824E+860', '-2.135449e+88566883329', 7, 3);
    T('3.59727766E+86307926622606', '3.972875024848559975206017548069946877803691132E-70214649789', '3.59727766e+86307926622606', 23, 5);
    T('-3.646264893020729078083667767570108425946828579942866503E+265253471', '8.0325833781083503034565554277775906677807816909226E+308461021', '8.032583378108350304e+308461021', 19, 0);
    T('5.800265697487097539627521395428663056273843031329334034635E+8786082644567528', '-51.6689751325045458784786758020311998802952459055', '5.8002656974870975396275213954286630562738430313e+8786082644567528', 47, 4);
    T('6.91796539232372189581201800702170066740310030815345261202344E+80526337', '-9.1847189968476171349246744836563208192170444242736130E-779894062122564', '6.917965392323721895812018007021700667403100308153452612023439999999999999e+80526337', 73, 3);
    T('-2.60711944434178E+610842335544', '4.8452738500178591887162172281628164222E+817291750725', '5e+817291750725', 1, 0);
    T('5.99722952726108378E-1853', '-2.66094166857022661510639782162490027080682960122731097961E+5391583', '-2.66094166857022661510639782162490027080682960122731098e+5391583', 55, 3);
    T('-9.0096440352681301992E+715', '5.0492291324677922617661610763164936E+120968544285065', '5.04e+120968544285065', 3, 3);
    T('2.2643412084925718514508667719407152E-275956486', '3.1156356580260103E+9436672568', '3.1156356580260103e+9436672568', 79, 4);
    T('3.80544267322140728828978869E+67065188', '-5.17859674794320492722517378239200875564951E+540393748984', '-5.17859674794320492722517378239200875564951e+540393748984', 69, 6);
    T('-7.37631315964639749057016447213504787290110296816808669391568E-4099', '-7.863906682259427614E-1510487991747', '-7.376313159646397490570164472135048e-4099', 34, 3);
    T('7.4829755522918671424650871681482595050279864E+65', '7.37004866789149743869644038712962750143948640E-931668416808', '7.482975552292e+65', 13, 5);
    T('8.2593281240724584768559983782749304E+66281', '-6.077611E-7545625002249', '8.25932812407245847685599837827493e+66281', 33, 3);
    T('-6.994927046706467936637937665E-30756', '-6.84355452116261299948070354931590058320347140546177877787231E+96', '-6.84355452116261299948070355e+96', 27, 4);
    T('6.4894725353823874252654034526111175587650548009111E+7915055391999', '-8E+4048493', '6.4e+7915055391999', 2, 1);
    T('-4.782E-8332', '-7E-3520136', '-4.782e-8332', 53, 1);

        T('-8.3316E-888', '-2.355253791E+5421756', '-2.355253791e+5421756', 71, 6);
    T('-6.6200199468628E+412773', '-4.822492668459934166240968807251782E+35496633042638', '-4.8224926684599341662409688072517820000000000000000000000001e+35496633042638', 59, 0);
    T('-8.7528555937975734418205700807571539087784304869781771E+365102210140471', '-4.3883E+6837790879266', '-8.752855593797574e+365102210140471', 16, 0);
    T('5.4895303E+8472451', '-8.98008068503792035989139695656E+447713424843', '-8.980080685037920359891396956559999999999999e+447713424843', 43, 2);
    T('6.52541018620809765E+35', '5.8512672127508119818454757224188128794415484279963646659054E+32347', '5.8512672127508119818454757224188128794415484279963646659054e+32347', 74, 4);
    T('-6.25791E+9826638280072', '-7.3536830842530668219E-66582560191319', '-6.25791e+9826638280072', 46, 5);
    T('-3.204517962302766218606406148171E+566036560884', '139208818204038247511755016.46591051299432073753501', '-3.204517962302766218606406148171e+566036560884', 54, 0);
    T('9.9047663972328630689772628373545625621865757E+20664919269', '5.595427094938775575895752926759535811867E+72413105767677', '5.595427094938775575895752926759535811867e+72413105767677', 57, 1);
    T('-2.724783665275496386651674558881787005244E+4185063', '7.56338E+2914335226281778', '7.56338e+2914335226281778', 11, 2);
    T('-9.0754430071250490755495498597999390502098131006E+807425330', '-7.46059554130104050525319910133333807206063591110362330209E+5146649109', '-7.460595541e+5146649109', 10, 5);
    T('0.000009093271490317713452988677', '9.2759E+222188321', '9.275900000000000000000000000000000000000000000000000000000000000000000000001e+222188321', 76, 0);
    T('-8.126101170529429935405719391028735828E+8448', '9.9495045425828053285212285720336425E+396468820096731', '9.9495045425828053285212285720336425e+396468820096731', 39, 6);
    T('-9.5822580666117554073598842369904313159669E+89681963', '-7.242901617686155926647788484601243753E+871638175128', '-7.242901617686155926647788484601243753e+871638175128', 51, 1);
    T('-8.6725893276789063089684942103231457185027347132056024056E+1105', '-6.41876849653253562083298972053E+71078', '-6.418768496532535620832989720530000000000000000000000000000000000000000000000001e+71078', 79, 0);
    T('5.682563E-929191485618', '-2263132', '-2.2631319999999999999999999999999999999999999999e+6', 47, 1);
    T('3.3088452520604187280723694741100E+429170622404437', '3.544182080945551220323350040610078760093E+17861', '3.30884525206041872807236947411e+429170622404437', 77, 5);
    T('-9.231509597176163830008062521073767375E+735055', '4.3158059340417185360844341557273310049181944705618746E+3962059977', '4.3158059340417185360844341557273310049181944705618746e+3962059977', 53, 5);
    T('6.212307021772167022448162811199463E+756', '-5.6472380011E+205690683102626', '-5.6472380011e+205690683102626', 36, 3);
    T('8857.35120926683203788', '-5.607900077135742165216005495803270487818743539127159E+83229122607', '-5.607900077135742165216005495803270487818743539127159e+83229122607', 75, 3);
    T('-8.3425640854852260972743332618478731389833510488713054E+6230290', '-4.1312751307073084300927530246419269E+307005106', '-4.1312751307073084300927530246419269e+307005106', 58, 1);
    T('9.7449703414270249E+8980', '7.67241057E+1794', '9.744970341427024900000000001e+8980', 28, 2);
    T('-5.1829088536580101658731284312645909E+745758070697', '2.84260416364E+44560968372', '-5.1829088536580101658731284312645908999999999999999999999e+745758070697', 56, 2);
    T('2.6055840235884646728936251616427856821E+5751035010', '1.5618397141081719965766506618268076172649219488E+815899', '2.6055840235884646728936251616427856821e+5751035010', 47, 4);
    T('9.3867057408097362162793689017782806214070146839E-2393017528035936', '2.93782780505896816545610471601395681074574895689299341E+581864405717', '2.93782780505896816545610471601395681074574895689299341e+581864405717', 62, 6);
    T('6E+84032', '-6.450250172449965836979870968355437613202742500027188323379E+7295782338', '-6.45025017244996583697987096835543761320274250002718832338e+7295782338', 57, 0);
    T('-7.92E-432323360875865', '2.7198999080159385107403872923085863787855E+775424120679', '2.719899908015938510740387292308586e+775424120679', 34, 6);
    T('-7.94599731593268864791879206351556E+982655', '6.45217468869558285898154691875214969672515739E+121034', '-7.945997315932688647918792063515559e+982655', 34, 1);
    T('1.8907132144380062447220232115560573908057640249036415756', '3.82049961967302205273077021825040E+5293225224482067', '3.8204996196730220527307702182504e+5293225224482067', 56, 1);
    T('-4.5174156847851568003071522E+68604018', '1.925177594775977968094576496831897982690124776E+5006422842136', '1.925177594775977968094576496832e+5006422842136', 31, 5);
    T('-9.061946713438943060116206438988877488571832926690266499E+6689', '-9.64859494249501E-278749', '-9.0619467134389430601162064389888774886e+6689', 38, 6);
    T('-7.8284251032738737635756158975999348963909E-875030', '8.57270032949331520E-28639957658', '-7.8e-875030', 2, 6);
    T('4.71166719375843654613914124817540422723166210592620725406E+3965418361', '2.54083086216788332305239125508871641614099543589238377E-84250031679035', '4.71166719375843654613914124817540422723166210592620725406e+3965418361', 72, 1);
    T('-7.7E+27', '7.710335117098902439304706E+4651246229840', '7.710335117098902439304706e+4651246229840', 28, 6);
    T('5.2582247377902816E+5757689', '-2.5E+764', '5.2582247377902815999999999999999999999999999999999999999999999999e+5757689', 65, 1);
    T('-5.76569841226269978586966536708833803927516803514784327668E+8969295375993', '9.22250244599220E+20659008347', '-5.765698412262699786e+8969295375993', 19, 0);
    T('-7.0549961E+565', '-5.258517050228535217E-83', '-7.06e+565', 3, 3);
    T('1.3058772432449575610199528305736961702478E+2540', '-8.997183917213456284517E+50532252677612', '-9e+50532252677612', 2, 3);
    T('-8.28984851273170138227329198453322E+955625795', '-8.64564332544126586742288E-5780', '-8.28984851273170138227329198453322e+955625795', 64, 5);
    T('5.5E+5', '7.838423902807E+946969670823', '7.838423902807e+946969670823', 14, 1);
    T('-855451998.4197664816734231250060249505963', '-8.20139150022548094866572339040969E+7878914842111', '-8.20139150022548094866572339040969e+7878914842111', 72, 1);
    T('7.763979846883772231E+290710674848178', '-5.55323336874214946543587E-468', '7.763979846883772231e+290710674848178', 41, 6);
    T('7.0593609115243459661237720489102955E+774618419', '9.69696516792701834247465400765594E+977999009218120', '9.696965167927018e+977999009218120', 16, 3);
    T('-9.9180403811352056697529017E+895280970', '2.10982531091041E-461293543', '-9.9180403811352056697529017e+895280970', 54, 4);
    T('-6.92338177945590132693393459759099190E+8578682800650', '-1.23887688564910364614170662714042389109596632971658462664E+49130704799', '-6.9233817794559013269339345975909919e+8578682800650', 38, 4);
    T('8.0270409221981330106884808997E+98866', '9.31565080391005963053878367106189248822283577303163673E+4473423557256795', '9.3156508e+4473423557256795', 8, 1);
    T('-1.21321168730E+33091977028', '2.1303363008244708038034888717937566992428842351E+775078312', '-1.2132116873e+33091977028', 14, 4);
    T('7.64584757604648009076053063338919175385640295301E+84', '2.23715595645752764110997070450230E+915711639', '2.237155956457527641109970704502300000000000000000000000000000000000001e+915711639', 70, 0);
    T('-9.774955334917238019659E+681169213837102', '-3.15828067591070167455915E+6871501242', '-9.77495533491723801966e+681169213837102', 22, 3);
    T('8.30645244601063836E+695267555', '8.4929940346E-345072866', '8.30645244601063836e+695267555', 26, 1);
    T('7.1448912845764E+569', '-1.7071172962048357489959149640345710483114663398940586578E+608', '-1.7071e+608', 5, 1);
    T('1.2525012143256359951968920909467370194495560E+770870217324448', '-2.4412203468303398795440737129237355989018385455929629382E-433737', '1.25250121432563599519689209094673701944956e+770870217324448', 42, 5);
    T('4.48836131313637306532082872770773860810148394673470E+66422', '-5.1905515024579637822E-7357', '4.488361e+66422', 7, 5);
    T('2.1963E+533256836', '-3.796754983308108628468591080964473711021915572455E+13465903', '2.196299999e+533256836', 10, 3);
    T('-1142974.86891693313253988', '62.41249875669093708146934190064816615237667', '-1.14291245641817644160279853065809935183384762333e+6', 53, 1);
    T('5.2154173576928086207702118918185768818662E+615386794784877', '2.2151141645661600854220802E+6224467311', '5.215417357e+615386794784877', 10, 3);
    T('5.07322427847932752132797838029791133009386319823518E+1412242367317355', '-8963626475.890545500022377343712969827690848255482401', '5.073224278479327e+1412242367317355', 16, 1);
    T('-5.194126773E+1419025', '-3.139564152404981896E+7609034754733', '-3.139564152404981896e+7609034754733', 77, 5);
    T('-1.0114125718494099572649541215070769358E+2717184097', '-6.82824953084594183556948911834815707820478205E+4017', '-1.0114125718494099573e+2717184097', 20, 5);
    T('-7.0795684557800956011910733511278113851686274651746806521214E+5159850', '8.8914910989339885857468059365406760325933085950500390296343E+87509980470151', '8.8e+87509980470151', 2, 3);
    T('-6.14543889086965514063454297295155E-6327565446345', '-8.5102416996735884332740223741383844896547932461431E+25901526683', '-8.510241699673588433274022374139e+25901526683', 31, 3);
    T('6.80909517492465567238603214967E+932599', '-9.778124435827131807E+3865813587115', '-9.778124435827131807e+3865813587115', 79, 3);
    T('-3.92527149601468919791730700365953783324602110839E+64617996', '27936.85857267952918271679971735', '-3.925271496014689197917307003659537833246021108389999999999e+64617996', 58, 2);
    T('-4.156896328995712583041201444375E-548320', '-8.689717309307804696523857031591187E+5459472146255461', '-8.689717309307804696523857031591187e+5459472146255461', 79, 2);
    T('7.965393660525041726222453070247881269378E+15891691474093', '6.445287293235309447326334615048017798749760185265937857E+426793130294767', '6.44528729323530944732633461504801779874976e+426793130294767', 43, 5);
    T('1.85235E+9544', '3.8668154943784112286099960167540405072040921016835312E+3191256634', '3.866815494378411228609996016754040507204e+3191256634', 40, 3);
    T('4.8540856681508E+17778209', '5.888424118682278104718446644107E-365677785555757', '4.85408566815080001e+17778209', 18, 2);
    T('-5.9921631925413495025552E-131', '6.800342554E-40283', '-5.9921631925413495025552e-131', 72, 3);
    T('7.059072828431692E+61', '8.35315078703975048020333079106399244253E-7284152344749648', '7.05907282843169200000000000000000000000000001e+61', 45, 0);
    T('9.6416077072903658510098665235968981716906860828577233E-767', '2.67630E+36316', '2.67630000001e+36316', 12, 0);
    T('-8.4704317418248776761657033838841180865410008185555991216572E+367369210613520', '-1931394049.838611813356111456996732015705775434518656', '-8.4704317418248776761657033838841180865410008185555991216572000001e+367369210613520', 65, 0);
    T('3.5094677397476668773324057418794380237712461126127861E+4873', '3.4923517439384471437492587156087E-57', '3.509467739747667e+4873', 16, 6);
    T('3.9814214354098422344084486493605427841115612348957071676226E+5748', '9.50475E+541041', '9.504750000000000000000000000000000000000000000000001e+541041', 52, 2);
    T('-2.8769026991838373273302814831E+617384143', '-3.91204E+8704', '-2.8769026991838373273302814831e+617384143', 51, 6);
    T('3.305002112204E+853671881615773', '4.8581666441758369348995602083094299200352E+804', '3.305002112204e+853671881615773', 41, 1);
    T('-1.1636592706120970546823107144712E-71217121825', '-1.5235939352629521E+309275869', '-1.5235939352629521e+309275869', 37, 2);
    T('-8.198926919735994928667986054273489224980E+76626068534', '9.959516396400338374890823425945732313439442692171E+947884340872', '9.95951639640033837489082342594573231343944269217e+947884340872', 49, 3);
    T('-7.4158093737750730567342723696965694190332157648543E+539158316655', '5.9173476015156632052901768542386E+573649936941000', '5.91734e+573649936941000', 6, 3);
    T('-9.31488230286E+5808602385', '6.052175017610757362970457296871043070081588263819315626152E+3228800', '-9.31488231e+5808602385', 9, 0);
    T('8.070272805758956793996205156288457017984E+59', '5.301450E+2893', '5.30145e+2893', 51, 6);
    T('71069733.03903927650627943774748102557719018248', '3.33849509E+88173097535991', '3.3384950900000000000000000000000000000000001e+88173097535991', 44, 2);
    T('9.1967719508211199422442149791319996754110353950295002853E+5185531179568', '-1.978241624464470607147349389969840054E+7207', '9.196771950821119942244214979131999675e+5185531179568', 37, 4);
    T('-6.096120080E+43269', '-2.477138897102269285712892243098876767E+27378', '-6.09612008e+43269', 23, 4);
    T('-2.2525263E+51973', '-2.7710E+58987682803', '-2.771e+58987682803', 18, 6);
    T('-1.4332619338479123655591877021170638937165050332265E-66268714096', '8.52625890800242772094357024193E-70649846115770', '-1.4332619338479123655591877021170638937165050332265e-66268714096', 70, 5);
    T('7.89626034454189801898106872365E-2683', '8.315240585302995720812612824377673848524720776063245652737E+5671741', '8.31524e+5671741', 6, 1);
    T('-1.281029171384330061E-665205910167', '3.940562201857259112327975896556865985E+871625959013', '3.94056220185725911232797589655686598499e+871625959013', 39, 1);
    T('9.53380115743476014231701E-142', '-4.64419674373E+3714', '-4.64419674373e+3714', 73, 5);
    T('-5.1307199965519738474123436718252575171015095575138011477E+630809', '9.407270154631388645407110732849918719165425922366027667302E+238967568477726', '1e+238967568477727', 1, 0);
    T('9.79514462834589195824509561607787194710872798E+275399917677', '-6.29697425923664844264878664173592857572635349462103574E+661563583', '9.79514462834589195824509561607787194710872798e+275399917677', 67, 5);
    T('-9.535214137372416177472133179431954910350470189064672E+858363197104', '8.83680214574E+6202504782', '-9.535214137372416177472133179432e+858363197104', 31, 3);
    T('-8.9090816286E+51', '-8.52328957E+32', '-8.9090816e+51', 8, 2);
    T('7.167276062652273574664E+2296001962979', '9.921282158775161620250926591771568801305498080733195056657E-4775861', '7.167276062652273574664e+2296001962979', 31, 4);
    T('2E+631', '-3.825143653024918490626342374729224645718726907', '2e+631', 10, 0);
    T('-6.2504483337844628947864963103966960600057176638403797E-7674722', '-8.8609476886645639605262663E+1961', '-8.86094768866456397e+1961', 18, 3);
    T('5.306E+27718926886', '281977075251036.361426426606975', '5.4e+27718926886', 2, 2);
    T('1.65588188544332E-94049733897067', '-6.61096531913796052897E+1857267', '-6.61096531913796052897e+1857267', 63, 6);
    T('-8.868118152282924992934143364602318320E+48', '-6.736775759660157567844761194570E+43839175223', '-6.7367757596601575678447e+43839175223', 23, 2);
    T('1.6714254627704839350373893276049246351086E+2055', '2.9687814939229701830893049E+430723', '2.9687814939229701830893049e+430723', 68, 4);
    T('-2.07823775796259714034421760254021063637E-6692531', '2.78608572206043748712772E+20452307377', '2.78608572206043748712772e+20452307377', 52, 5);
    T('-3.037935976661791183803831657E+8901047608', '9.1629584716285045454310E+382669183', '-3.037935976661791183803831657e+8901047608', 59, 4);
    T('2.99311165972294817099458262472351074880685097255686166E-698', '7.94990164465732256716249E+4241240432920', '7.949901644657322567162490000000001e+4241240432920', 34, 2);
    T('7.112864E+99522809253', '9.26088414639575257571E-9029653436', '7.112864000000000000000000000000000000000000000000000000001e+99522809253', 58, 0);
    T('-2.632E-27371366088019', '1.8035225697503910426881706469229E+6738157594', '1.8035225697503910426881706469229e+6738157594', 52, 5);
    T('-7.6831307720064192173404922705553932580040526613903E+827', '-4.0282140346396759287123847813070127115487645960468E+47161458244999', '-4.028214034639675928712384781307012711549e+47161458244999', 40, 0);
    T('-3.11688342600430804800457E+5646', '3.8137007529451663213017245125188409E-387165168', '-3.11688342600430804800457e+5646', 73, 6);
    T('-9.44242982649318300702033401743191796015034993E+3604', '31590.9548687307368434748681000862401115', '-9.44242982649318300702033401743191796015034993e+3604', 68, 4);
    T('-5.825752083700523449520311537030E+67495', '5.21623636014044851398661253731447039E+173795469', '5.21623636014044851398661253731447039e+173795469', 53, 2);
    T('-5.276901965625938248176034794605189E+308541', '3.703601829651612578794559072222972313095657033E-81796', '-5.2769019656259382481760347946051889999999999999e+308541', 47, 2);
    T('6.9331067424593100415846134792668059E+912479412349652', '1.680237252402452355491662955E+240714302399624', '6.9331067424593100415846134792668059e+912479412349652', 69, 1);
    T('-5E+383729434259', '5.1698938385330575823513813308344E+94', '-5e+383729434259', 77, 3);
    T('8.7243E+2850945587541238', '5.008535584790243998706736E-2151148950', '8.7243000001e+2850945587541238', 11, 2);
    T('-8.2814124651507115617859674945020193077995688E+7389', '-6.685940927798568645328142E+9408744376476', '-6.685940927798568645328142000000000000001e+9408744376476', 40, 0);
    T('1.38894085712478387E+8111350280260', '-4079096153.336470783463281917', '1.38894085712478387e+8111350280260', 23, 5);
    T('-6.9379412604937590585903298996647446557280233E-86402', '-9.298949377170890800082719839013629452679792120255668252E-3314171151892669', '-6.93794126049375905859032989966e-86402', 30, 4);
    T('-1.171706171663165227238632357263196175E-46', '-2.463659143449236951790664409993872511E-616693020719', '-1.1717061716631652272386323572631961750000000000000000000000000000001e-46', 68, 3);
    T('2.527599473786248877520515210997634651E+143398591', '-3.6424221366324101067807829891E-8147030154', '2.5275994737862488775205152109976346509999999e+143398591', 44, 3);
    T('-1.5215420318543285199E-536', '1.7699874170718756677493911E+81532', '1.7699874170718756677493911e+81532', 63, 5);
    T('5.1669460704447401201196968339132366417265E+27110077389671', '-6.942575107142604831609380273850689043695579646782995E+47082366280719', '-6.942575107142604831609380273850689043695579646782994999999999999e+47082366280719', 64, 1);
    T('2.970654952097906007795696246168515702398439309941455645E+8597011', '5.2296492508479136E+41494778473', '5.2296492508479136e+41494778473', 36, 6);
    T('6.0584218266998349514166054436274234517424E+721837', '8.204055622430237035914767107403E-7828', '6.05842e+721837', 6, 4);
    T('6.728450555762058503897888640682E+8985', '-5.8932979585170935219027817844384217639872785482160469E+259944', '-5.8932979585170935219027817844384217639872785482160469e+259944', 55, 5);
    T('-5.9745669924785853331155894231865836004E+687670611414', '-1294651603.9794385', '-5.97456699247859e+687670611414', 15, 0);
    T('1.92075258547490124336019895329562888494939003625868465830E+175383533827414', '-67875.5183590771982', '2e+175383533827414', 1, 0);
    T('-8.2223671477819135622852621767445907634349105E-217865324615610', '986305586', '9.863055859999999999999999999999999999999999e+8', 43, 1);
    T('2.61911748649590772062006898736E+35938742', '-8.93523762571242192091428686767725263534304436E-613359953', '2.61911748649590772062006898736e+35938742', 66, 6);
    T('6.9134628586665848050715908580704808601061933454766964051E+78742', '-1.50890264421262241458375871577594414934338875708396125320798E-8166161', '6.91346286e+78742', 9, 6);
    T('-3.298897849818437168375307753E-998243277', '2.9244319317E-7', '2.924432e-7', 7, 0);
    T('-9.86724323115190412020277893591840387145583E+50617122439', '-8.068893815448987075999677E+64584852492', '-8.06889381544898708e+64584852492', 18, 3);
    T('1.91749184574672336334420691070298270E+5068347272751', '3.70322391169854097800357120507607278108543265295E+68032', '1.9174918457467233633442069107029827e+5068347272751', 68, 3);
    T('-7.25040796502593072338783301468852361151141949834737300E+60033027548', '-2.47244032744242741063428749E+812415464', '-7.250407965025930723387833014688523611511419498347373e+60033027548', 59, 1);
    T('0.5599527224325105760261433012929893449593446986695616664', '3.9575315972652657361191539095360047E+3375775212651091', '3.957531597266e+3375775212651091', 13, 2);
    T('-2.422189168710442574005377068765E+6642160290553846', '-6.79E-397265075929', '-2.4221891687104e+6642160290553846', 14, 2);
    T('-5.21417072028682195991226214948980237226078143124522433E+83755730306765', '-2.0037E+3985562', '-5.21417072028682195991226214948980237226078143124522433e+83755730306765', 72, 4);
    T('-2.07541E+7445683543129', '4.86270668939537286613289622484E-97146334042420', '-2.07541e+7445683543129', 31, 5);
    T('-3.4012689453783498352453068584540276379244795718661109E+2208650698', '402873.5490261605151246206', '-3.4012689453783498352453068584540276379244795718661e+2208650698', 50, 5);
    T('5.9149391775838E+91', '6.105981674340173334816250638158002769007E+745154525283', '6.105981674340173334816250638158002769007e+745154525283', 69, 5);
    T('-8.2053974337042208396557591935771540982739276058019E+17334571557', '-3.4278519602330811302599594E+115', '-8.205397433704220839655759193577e+17334571557', 31, 6);
    T('-5.22203E+6', '2.00095378445660734591613419746144966969E+186532', '2.00095378445660734591613419746144966969e+186532', 41, 0);
    T('9E-27830', '-5.03755534926553004028195180E+3998553094710508', '-5.0375553492655300402819518e+3998553094710508', 60, 4);
    T('3.07297475642325E-827642194182', '-4.48530936595494784548817112332213273E+99005', '-4.4853093659549478454881711e+99005', 26, 4);
    T('-6.6722E+42', '3.901582775993730962246348547778691585950716654331187E+510420', '3.90158e+510420', 6, 3);
    T('9.65673274613439E+8246651207', '-9.057239713408679556118870551716782102E-8279839520979789', '9.65673274613439e+8246651207', 22, 4);
    T('2.8653215869856794975951496E+76710522817324', '-3.05812952716484241195552632732093253086433735693629483335762E-70743141389247', '2.8653215869856794975951496e+76710522817324', 36, 5);
    T('-8.68237147977797360297669371996171571180505964139119E+108515438005267', '-1.45965436179581520E+89657983', '-8.6823714797779736029766937199617157118050596413911e+108515438005267', 50, 2);
    T('8.42067527427532752287890490665E+431340267', '-1.538094494081099777725238692267544E+407', '8.42067527427532752287890490665e+431340267', 50, 5);
    T('-6.48102910790904024462529729423879072376251682063363E+3470294', '-657.367147723279175564438483300267550931778581483590230809', '-6.4810291079090402446253e+3470294', 23, 0);
    T('-2.3364625778204142261021652742592222228853402407668226E+8756', '7.800248940868897177963572208647054999339E-588', '-2.336462577820414226102165e+8756', 25, 1);
    T('1.4734940E+19620757936333', '-9.821648317563379634022754597004942695265992153042316E-49', '1.4734939999e+19620757936333', 11, 3);
    T('-3.560933E+34039154377148', '2.67071793403914040887661294618892494847226089246E+25219922907496', '-3.560933e+34039154377148', 66, 3);
    T('4.79241526319328665160339797263967794739464919886294E+43393', '-5.01968984817028262380373971941503084820376296912714662280334E+45133818620606', '-5.01968984817028262380373971941503084820376296912714662280334e+45133818620606', 63, 6);
    T('4.57235375193867934732E-7194', '-6.52582487578571373652831446165809E-92149523015', '4.57235375193867934732e-7194', 46, 2);
    T('-868041.528817017834364212508063941762280656376136', '5.477069678996081322043660906E+9265123675104', '5.477069678996081322e+9265123675104', 20, 1);
    T('9.9603812129707640006144250065565043364852162648308E-85626', '-3.987496200764605784620E+1348431563624469', '-3.98749620076461e+1348431563624469', 15, 3);
    T('1.910308315322401279886246914553882725559046E+265065808932414', '5.509517201313387989033674E+113072', '1.9103083153224012798862469145538827255590461e+265065808932414', 44, 2);
    T('-1.384E-5816178', '-5.823606548263134E+2827711687', '-5.823606548263134e+2827711687', 41, 5);
    T('-2.1515149961146676542770007E+3979277765135489', '7.71216690200695369115412567302746877841068E-126245782813705', '-2.151515e+3979277765135489', 9, 4);
    T('-2.98084455071531426920827469558E+254', '-1.3933545329E-54471', '-2.9808445507153142692082746955801e+254', 32, 3);
    T('9.23498225895649583067E+868280755', '5.092174825733118391E+73887295307', '5.092174825733118391e+73887295307', 23, 1);
    T('5.5287051603E+8028017425', '4.3928664173713859329E+60', '5.5287051603e+8028017425', 62, 3);
    T('4.0253565710082920921337473151322666218E+12239049', '-5E+2688753291615', '-5e+2688753291615', 70, 0);
    T('5.28255675137715868911962014909351428037157201833421883E-81887537684290', '-3.5E+1221385675', '-3.5e+1221385675', 71, 0);
    T('5.5502315119310712918406573979327693125451931E+54672975338', '-9.6282187686977625211101953116E+97274107', '5.5502315119310712918406573979e+54672975338', 29, 3);
    T('-1.821523660169117427187710721', '-2.8622460669310712397E+84343078', '-2.8622460669310712397e+84343078', 80, 6);
    T('-3.530736252048530E+5400908605', '-1.3874377572356297030600649052584114112308884E+254', '-3.53073625204853e+5400908605', 74, 1);
    T('1.59122442353159873136301574367336847330069470901736E+5324', '-6.674888106160258E+66066520093471', '-6.674888106160257999999999999999999e+66066520093471', 34, 2);
    T('8.658309100708641982617600193373858517853460291913739495E-86483271', '4.436421924445537862717241003766681367195696E-4443052391', '8.658309100708641982617600194e-86483271', 28, 2);
    T('9.03E+91191487035', '-3.98776056293609049616054503943395030664042036E+8424974412770598', '-3.98776056293609049616054503943395030664042036e+8424974412770598', 45, 5);
    T('6.608890993494648E-119647086877028', '-3.6007449361825583610250507241E+70711137484', '-3.600744936e+70711137484', 10, 4);
    T('-7.172688E+2869', '-2.4374420045478193191958937651935E-83', '-7.172688e+2869', 42, 5);
    T('-7.12E+93', '4.1512454664962909411020386111146665989053E-9902932579', '-8e+93', 1, 3);
    T('-9.5E+63515521500', '91673.7171974231780910998195', '-9.5e+63515521500', 47, 3);
    T('2.218464417592559819876666792E+903619722707174', '-8E+1630', '2.21846441759255982e+903619722707174', 18, 2);
    T('514328372895982746.54612934039584396920652200523664659457', '-6.74062435338405157968251227875E-254950814591826', '5.1432837289598274654612934039584396920652200523664659457e+17', 79, 2);
    T('3.378332224E+5372968624154218', '-8.51478589910629606395083784111373172E+4679045', '3.378332224e+5372968624154218', 40, 4);
    T('-1.8415688110347051212759E-98903377476309', '6.568426332014906433896692983888879357531126960855E+827966', '6.5684263320149064339e+827966', 21, 6);
    T('-4.777588362861875275153905106598E+176283303403008', '8.432718028774457026860825253793548594362354271960483282E+60588138362', '-4.777588362861875275153905106598e+176283303403008', 59, 3);
    T('9.963837564679846844887692E+46568980863244', '-5623.146490807834671285827871608136692574909', '9.96383756467984684488e+46568980863244', 21, 3);
    T('-7.7641253079545E+17883607', '-4.23859693738127112803235137766559458982E+53464', '-7.7641253079545e+17883607', 67, 4);
    T('1.7626194890786793428140104970146873317269621813562916507E+947437', '5.53320356296365818420244253875316100804029923610158324E+90917307448460', '5.53320356296365818420244253875316100804029923610158324e+90917307448460', 71, 4);
    T('-5.52608050823525259809341308234402705425047E+167', '-2132237118.5502218738', '-5.526080508235252598093413083e+167', 28, 3);
    T('6.8144930528302487243333678948845788333514794779E+9965', '-1.545116234930549084599575801457094763796541280E+2597', '6.81449305e+9965', 9, 4);
    T('7.077560247149585686286143979942084E+455982483446563', '2.77E+20506', '7.077560247149585686286143979942084000000001e+455982483446563', 43, 0);
    T('13646242.01100736094930143119957245600885224457026564676', '-8.125570314E+103', '-8.125570314e+103', 32, 5);
    T('-9.76756E+229300', '-7.85498079235511369E-38', '-9.76756e+229300', 53, 6);
    T('3.844748755683375318502866428665049366413754469510947E+9915097', '-4.23840818374329394185648446097972864007180952E-1359781858', '3.84474875568338e+9915097', 15, 5);
    T('2.431598107351890764271706631985269241978503107679137556E-89', '2.532593890347E-1674189734666170', '2.431598107351890764271706631985269241978503107679137556e-89', 56, 3);
    T('3.64790306996373E+83', '-1.3604456640E+2645569921522835', '-1.360445664e+2645569921522835', 14, 4);
    T('-2.4224552332794626962624E+7226386231042435', '5.20018202701272460693177380158523921365849705421192E+72357818984', '-2.422455233279462696262399999999999999999999999999999999999999999e+7226386231042435', 64, 1);
    T('-7.573002687287660267628985418162866603170185389976E+5778', '4.6423895E+3111', '-7.57300268728766026762898541816286660317018538998e+5778', 48, 5);
    T('-3.79187169892527540129264866667966', '-7.8929E+38574853907', '-7.8929e+38574853907', 38, 1);
    T('-1.948731547085668464615518761184038391376530637965E+951735', '4.2607651413488533411355206902204648316701536133E+945681516755031', '4.260765141348853341135520690220464831670153613e+945681516755031', 46, 5);
    T('-6.57030269568873341244429373067501189485382392913E+2011157', '-35178651325308625881372771883843442320620224731.69202874537', '-6.570302695688733412444293730675011894854e+2011157', 40, 3);
    T('0.413724318583923906337786842', '9.712891E-65', '4.137243185839e-1', 13, 3);
    T('7.9435520718831927910664870504266130960E+481277811', '-8.447927112213830328629E+9398576795', '-8.447927112213830328629e+9398576795', 55, 0);
    T('-9.9777674201488821590E-6908051700', '3.1169023133005179097111104810253590635277272360193887601755E+3934651252196', '3.11690231330051790971111048102535906352772723601938876e+3934651252196', 55, 1);
    T('4.126179772225415121273687E-1365893016', '-7.7815087235679942E+5595', '-7.7815087235679942e+5595', 43, 3);
    T('-7.077480388127002265369977608191360E+4734887627', '-7.05411914567312560632949578E+6294020688915092', '-7.054e+6294020688915092', 4, 6);
    T('9.0925388506166362821E+8605880332379248', '9.77235601825800627740270204731939915E+92341245381', '9.0925388506166362821000000000000000000000000000001e+8605880332379248', 50, 2);
    T('4.76423E-9421027642', '3410716.25527017718380954179868985386', '3.4107162552701771838095417986898538600000000000000000001e+6', 56, 0);
    T('7.396301497989734E+73900710546', '-4.576520257862386451938830E+34546767196963', '-4.5765202578623864519388299999999e+34546767196963', 32, 1);
    T('-5.682736349148971666931594772485633383805724335920181275600E+17639002647', '2.68814206269718E+778378', '-5.682736349148971666931594772485633383805724335920181275599999999999999999999999e+17639002647', 79, 1);
    T('9.295569381290926956875895811524429599898E+7444', '-8.0239568059158017158412820293264310245E+5994', '9.295e+7444', 4, 1);
    T('9.94462414149910194777884329752868818579E+887755647541', '2.73006485849911348846173077518215706E-652371380058413', '9.94462414149910194777884329752868818579e+887755647541', 55, 3);
    T('-9.7596755423935940312718', '3.946893723238273E-29487529524', '-9.7596755423935940312718e+0', 29, 6);
    T('1.031191755040724474477521077998920346272E+461350931504', '-5.155737708874251114944176965076293E+1744364761829', '-5.155737708874251114944176965076293e+1744364761829', 50, 4);
    T('-1.133107095E+16750275795', '-6.71120905236440931234655591E-9125', '-1.13310709500000000000000000000000000000000000000000000000001e+16750275795', 60, 0);
    T('-5.82278334752894543634949304245742850096501E+348739', '-6.1996084365361365561915107794089626739810066345299317335E+96749786', '-6.1996084365361365561915107794089626739810066345299317335e+96749786', 70, 4);
    T('2.28344593287542113481E+287406173752', '9.0244149657755548521345143059108803587703E-69', '2.28344593287542113481e+287406173752', 52, 6);
    T('-5.80460992790101E+547830100', '-1.67217853655502706369540659081202844166E+683', '-5.80460992790101e+547830100', 52, 2);
    T('-8.65181784970725907829738566085006909572289E+97862599', '3.25420254E+4526', '-8.65181784970725907829738566085006909572289e+97862599', 69, 0);
    T('7.054999374900680169E+70767998339523', '-9.859919774E+405985', '7.054999374900680169e+70767998339523', 60, 6);
    T('-5.3472062112818714039076238844272376169672740417269753767346E+30765', '0.0663053622520', '-5.34720621128187140390762388442723762e+30765', 36, 5);
    T('3.5816219151524636728183085824410321723553529197884533620E-826074008280008', '-1.32233737609745371002894235776108247006001703790755528834749E-688902318', '-1.322337376097453710028942357761082470060017037907556e-688902318', 52, 3);
    T('-2.00471461855930301128840402E-9766524', '7.1775725783822305052566165837880284905306302817465707E+5737043', '7.17757257838223050525661658378802e+5737043', 33, 1);
    T('8.799266326624455173634271331100E+51697490769', '-4.1029541443948269608E+6136484878', '8.79927e+51697490769', 6, 4);
    T('6.1405643934E+13177859', '-4.24807242E-76149879', '6.1405643934e+13177859', 55, 2);
    T('7.970556685442848804101848270E+2811115351556503', '-38226188890839.368671326909421898804', '7.97055668544284880410184827e+2811115351556503', 29, 4);
    T('-9.58337794291354250428216872043227801753887081249261281405266E+74567533', '-4.53340745183879776560429359190663016E-754491395874', '-9.5833779429135425042821687204322780175388708124927e+74567533', 50, 0);
    T('7.9025053357560625899319356039655334876900E+403', '7.0376984159797704388671233618E+6034709454', '7.03769841597977043886712336180000000000000000000000000000001e+6034709454', 60, 0);
    T('5.83697893456038353822811416567098979E+774892343', '8.8937847358363013760614123101664979829553861241851861038E+230517200', '5.83697893456038353822811416567098979e+774892343', 62, 4);
    T('1.85856361231398887640249527289939584980866673721970806046E+67639727729', '-2.37548572993092345540559838319849725804768E+265568699282', '-2.3754857299309234554055983831e+265568699282', 29, 2);
    T('-5.44902642021E+3173085', '4.0085080264129E-90231661899', '-5.44902642021e+3173085', 14, 6);
    T('-8.3726647528812355184943E+7270975716988', '-1.9024482742E-336', '-8.3726647528812355184943000000000000000000000000000000000000000000000000001e+7270975716988', 74, 0);
    T('5.47015572826217610396E+93', '4.881594072724571814238215933747720069235192656968823E+3929326', '4.881594072724571814238215933747720069235192656968823e+3929326', 54, 4);
    T('-5.51394739944291669875851904979075E+6696081', '-3.8300981992325149662E+3661589985109', '-3.83009819924e+3661589985109', 12, 3);
    T('-2.93303421801254403465831469474571144209290027385E+94', '-4.39490921010127134386465509176951639540E-1487620158', '-2.93303421801254403465831469474571144209290027385000001e+94', 54, 0);
    T('7.408283248783699241E+95442836', '-1.87196501183280406E+11080550', '7.40828324878369924099999999999999999999e+95442836', 39, 3);
    T('1.2195048719716380579876E+74880513678013', '-4.15E+23959097', '1.2195048719716380579876e+74880513678013', 71, 5);
    T('-7E+8811761819346600', '4.655865162009364E+404732094846', '-7e+8811761819346600', 75, 6);
    T('3.8256226078129022296507076895648006984385E+61', '-2.657096837203251910835265664036922096175E+99527802517190', '-2e+99527802517190', 1, 2);
    T('-3.9664595898208913297440363E+5851358', '2.97046857532793063683126E+919', '-3.9664595898208913297440362999999999999999e+5851358', 41, 1);
    T('-1.790830954382023741737627615795156553635818949609E+508', '3.4190933585927513515207416470561012874561355E-3284973', '-1.790830954382023741737627615795156553635818949609e+508', 80, 4);
    T('6.311744093890068398123333898863E+92525', '-1.8710625977933129625199598929073614326499043539848E+707078612949920', '-1.87106259779331296251995989290736143264990435398479999999999999e+707078612949920', 63, 2);
    T('-46858905613733945038659997070286674910213227267.396676251005', '3.0577155334932813830311964677077E-5768967628', '-4.6858905613733945038659997070286674910213227267396676251e+46', 58, 1);
    T('-8.8823901589641342930978526392256295E+931068713583', '2496.756984093850482418967491573044', '-8.8823901589641342930978526e+931068713583', 26, 4);
    T('3.62251496381423590582535332237298186E+370935639796', '9099925549.01639624293659981', '3.62251496381423e+370935639796', 15, 3);
    T('-8E-69428', '-9.6661204579379578E-29628011', '-8e-69428', 6, 6);
    T('-5.226883535230409006955095787840117906031044593870130372E+59523845440', '9.11625200573486648115401680794867376589257957550671E+3452695195', '-5.22688353523040900695509579e+59523845440', 27, 4);
    T('-1.0232E+9189042212087', '-7.67951799E-74', '-1.0232e+9189042212087', 62, 4);
    T('-3.75499286602178922207323831E+43621150', '7.60794034263692662342687427493446571361467995479334405E-15894118651600', '-3.75499286602178922207323831e+43621150', 28, 3);
    T('-1.2876617152821635047764720894951045532E+816438144', '7.4337164182637233601E+235194', '-1.2876617152821635047764720894951045531999999999e+816438144', 47, 2);
    T('5069975.5697478884946527573160769660451', '5.787592510E+382428677', '5.78759251e+382428677', 44, 3);
    T('8.541880556698246362580856E+580177', '-5.5941966103415343395282575152775191853761387519E+93556955290897', '-5.5941966103415343395282575152776e+93556955290897', 32, 3);
    T('6.8407791126770551023549811E+601', '-6.71057849589996948401012690E-23336601505759', '6.840779112677055102354981099e+601', 28, 3);
    T('-5.81043327366808E+165917791135019', '7.4738E+8767107', '-5.8105e+165917791135019', 5, 3);
    T('4.3558955757747698644739046505840348867867311095209473866E+55363190220531', '-1.7E+84', '4.355896e+55363190220531', 7, 6);
    T('-3.015868498236145219494457887825822766241504239289964894660E+3980960380', '-5.77122029990329226954407963597668938915547487226680152E+6770', '-3.015868498236145219494457887825822766241504239289964894660001e+3980960380', 61, 0);
    T('1.3062402293651743288486750E+1381823114472', '-4.053922692766690830338845928275087778256825442118470E+492987641001226', '-4.05392269276669083033884592827508777825682544211847e+492987641001226', 60, 0);
    T('-9.937260974755749111E-193423534766682', '2676667462331398139587538383270.8381768016', '2.67667e+30', 6, 2);
    T('-1.02003660726983E+18255948536319', '7.730556464186863700396580226101605143240245E+838', '-1.020036607269829999999999999999e+18255948536319', 31, 1);
    T('1.3124873023161408956886753600371006E+750744325', '2.18930614013545744901836425116651E+479', '1.3124873023161408956886753600371006e+750744325', 41, 6);
    T('6.62001637402620464636198657428963583978424924E+70576', '-5.449563628367447694524725886179209456732038E+72466040923033', '-5.449563628367447694524725886179209456732038e+72466040923033', 51, 3);
    T('2.86343027955523E+947', '-6.19602556842483759500861325589226531519414380892614024E+849410653596', '-6.196025568424837595008613255892265315194143808926141e+849410653596', 52, 3);
    T('2.41993969978686505865108307487955244472204342872E+6469236', '-2.3709423266467594655622100067106438659767679990042161E-49324650241115', '2.4199396997868650586510830748795524447e+6469236', 38, 6);
    T('-1.976051466387598585805E+82492', '-2.6017999075608178E+3113', '-1.976051466387598585805e+82492', 73, 4);
    T('6E+176961344278', '4.6700032026641587610726258E+95581609065809', '4.6700032026641587610726258e+95581609065809', 66, 6);
    T('3.68000389187392999747494922051178256319720533031656422E-390462', '-6.9220376312506726904582478817443953967967425263884081954E+1100428291', '-6.92203763125067269045824788174439539679674252639e+1100428291', 48, 3);
    T('2.069169747044234745349870139098001764624424366505644E+4478', '-1.7776580078692088673E-7742', '2.069169747044234745349870139098001764624424366505644e+4478', 56, 4);
    T('-4.08910519743890256720520557457207150224937682152E+4689052090407472', '8.395594018892194459002888E+48760098762', '-4.089105197438902567205205574572071502249376821519999999999999999999999e+4689052090407472', 70, 2);
    T('8.5776172505652708E+2026736632775958', '9.44671930246556170624002502029', '8.5776172505652708e+2026736632775958', 21, 6);
    T('-5.59982387205253728141416841192432213276E-795421058789722', '4.764597152275478194941706540151995110982E+6691367502', '4.764597152275e+6691367502', 13, 4);
    T('1.53172053819825226510571321621435114922136633080010248607E+2314', '-1.418356680070824501002603836963462296E-90097', '1.53172053819825226510571321621435114922136633080010248e+2314', 54, 1);
    T('-3.68554653901295412191868475370030053785849E+6819311429', '-5E+9704600', '-3.68554653901295412191868475370030053785849e+6819311429', 63, 4);
    T('-3.38280194728902536E-876626223007001', '-7.814943922E-226', '-7.814943922e-226', 68, 5);
    T('9.8400023785353532946442468201713663663894686315E+2205013585460688', '-3.14065806728373087815714811981918E-4721743356', '9.8400023785353532946442468201713663663894686315e+2205013585460688', 63, 5);
    T('-1.305122E+71473835107', '2.74094300486279913E+786144', '-1.305122e+71473835107', 54, 3);
    T('-8.2502754183215085446E-44', '2.1039780065020062E+2020', '2.1039780065020062e+2020', 55, 4);
    T('5.131392621498549E+16726', '3.248674352471374513625906143E+77092431088', '3.248674e+77092431088', 7, 6);
    T('-9.4809983589940E+29', '-8.2385352492077E-1105984572680', '-9.5e+29', 2, 6);
    T('3.98603786453160093954089718950111814644E+292964115', '-8875286358467817363827024726.2589022801266994967607714763580', '3.986037e+292964115', 7, 3);
    T('7.064788503E+893445922', '5.75083527238513935115868636493676059526416359843472835E+296417417', '7.0647885030000000000000000000000000000000000001e+893445922', 47, 2);
    T('-9393348.531779096558036582753146', '3.8146933745000276669600762917501162394E+597388702145', '3.8146933745000276669600762917501162394e+597388702145', 76, 6);
    T('2.7167492818697118149461873962E+802', '8.220242792E-63113809158356', '2.7167492818697118149461873962e+802', 54, 3);
    T('8.94380351472392053264531397417381337100965351E+87716319', '-1.3228823E+36310401192391', '-1.3228823e+36310401192391', 53, 0);
    T('-8.94558180683073505529466878884981965545747E+5485694', '-6.7758E+161426650', '-6.7758000000000000000000000000000001e+161426650', 35, 0);
    T('-7.38313527335452616734855466249632909108E+522715', '-9.521829369219861843858436963358940473552311448E+280059254', '-9.52182936921986184385843696336e+280059254', 30, 3);
    T('0.7375310836', '-4.4967184894395629E+5911733499902', '-4.4967184894395628999999999999999999999999999999999e+5911733499902', 50, 2);
    T('-1.77E+6727097589', '-3.1185181259862164E+69085033701', '-3.1185181259862164e+69085033701', 35, 1);
    T('-3.79647971534823021260774502130434889892329E+8593708211620662', '-5.38', '-3.796479715348230212607745021304348898923290000000001e+8593708211620662', 52, 0);
    T('9.8359233944873856783181320E+13240935929', '7.922291358235554468049219440468E+2961319', '9.8359233944873856783e+13240935929', 20, 1);
    T('3.069231770396623388870657557324640E-51', '6.307235301039626090521E+4946460537274', '6e+4946460537274', 1, 5);
    T('1.3824523190371641303E+5925', '1.54053597326483683619778332058936573959947969582655221161716E+360', '1.3824523190371641303e+5925', 24, 1);
    T('9.234140546693518E+9105', '-6.08710676802553672892889635337047245542010E+7591', '9.234140546693518e+9105', 23, 5);
    T('-1.6291370894466509482009027755604563189586E-70053', '-6.574120340E+244', '-6.5741203e+244', 8, 2);
    T('-4.557531053693330057460614478820158108183620E+249286', '-4.941768906175021364550360247118259401255167397402E-4583710825', '-4.55753105369333005746061447882015810818362e+249286', 51, 4);
    T('3.73456690E+479119750', '1E+6451', '3.7345669e+479119750', 46, 5);
    T('1.14627367343623581749018220205993260163426E+375', '7.2005409220402005911005776082123E+67', '1.146273673436235817490182202059932601634e+375', 40, 6);
    T('-5.099056475944E+6208273', '5.03497986026700501656013840133977464853838278724408E-41', '-5.099056475943999999999999999999999999999999999999e+6208273', 49, 1);
    T('4.18E+4', '-4.9E+867039112020373', '-4.8999999999999999999999999999999999999999999999999999999999999999e+867039112020373', 65, 1);
    T('-2.4240548447E+5151', '-6.9E+6885929', '-6.9e+6885929', 29, 1);
    T('5.84841E+563224', '7.67113498402080350396968177979755835003006E-519808', '5.84841e+563224', 76, 6);
    T('-5.6848828029104588988235278327944223209706533729777865E+882185505570', '8.8431247036618948886320578402528846302525754E+676029610380395', '8.84312470366189488863205784025288463025258e+676029610380395', 42, 6);
    T('2.32921826875E+92159', '4663806877.380931747280143700613411339077942198788', '2.32921826875e+92159', 40, 6);
    T('-4.336299940615515001319956793283226527E+32605917', '7.76753685082816481302239646414381312074E+563689', '-4.3362999406155150013e+32605917', 20, 4);
    T('9.891803691505024531E-21194', '-1.703705704655726049100E+85851760385623', '-1.7037057046557260491e+85851760385623', 72, 0);
    T('-9.8036343E+18710', '7.1525619690641587340484803838007958288549622457372E+49580513617', '7.1525619690641587340484803838007958e+49580513617', 35, 3);
    T('3.1027139322852897073302896774722004140846840E-965', '1.78895106553340980730646100348589920307176074580E+185343277623735', '1.78895106553340980730646100348589920307176075e+185343277623735', 45, 2);
    T('-7.655724879E+628256263652878', '8.13202017065326628681665194616521E-67531674440', '-7.655724879e+628256263652878', 46, 4);
    T('7.92650700E+33', '-3.8352553897467361093663241845917696639200382296693E+1004', '-3.9e+1004', 2, 3);
    T('8.073413837163864750702901595132247092964896E-2821166188194', '-7.9579014025029548017943018869452247893597821549248826928928E-154330', '-7.957901402502954801794301886945224789359782154925e-154330', 49, 0);
    T('7.376082930405746235625953608914105289395009109720649E+6943776464249348', '2.90781896909425829787783895068296694081071E+556536', '7.3760829304057462e+6943776464249348', 17, 3);
    T('4.6011800481717419E-806104423910', '-8.6437098622551564549067098493197906561E-280114', '-8.64370986225515645490670984931979065609999999999999e-280114', 51, 1);
    T('3.4681913622863819250677595389410956725648165265014983E-53212647902', '-8.18410385421852186881398297236729212185E+188', '-8.18410385421852186881398297236729213e+188', 36, 3);
    T('6.66144513902114901386407229147E+6153706610103500', '9.0511259314724948248692894888182029430402308963553381379066E+953113360042358', '6.66144513902114901386407229147e+6153706610103500', 74, 4);
    T('3.372650568051595258210907487596E+409215479667897', '-4.183206656034048517819669530602649203970090968725509970E+26570793338', '3.372650568051595258210907487596e+409215479667897', 74, 4);
    T('-5.7656240966847308176727533610432E+62642145507', '-1.122213E+7374576131771581', '-1.12221300001e+7374576131771581', 12, 3);
    T('8.774639235678526352460104724518622822913117021566676E-1991326', '8E+45629', '8e+45629', 34, 4);
    T('4.9590402774499089404125161028829404E+1528628', '-8.110292813306920410771458213952017595580929120805599E+676055705', '-8.110292813306920410771458213952e+676055705', 32, 5);
    T('-9.94827679900718E+47261881238', '6.191921784108606448822612520770987741963619338418111021E+2397', '-9.94827679900718e+47261881238', 79, 3);
    T('3.000279398050E+68452755140307', '-2.314E+6336521643193', '3.00027939805e+68452755140307', 25, 2);
    T('-6.39233174154524919846265317272190742174E+27843778404', '-4.9206748933094636318481451339835260640508733245821012047E+581960334', '-6.3923317415452491984626531727219e+27843778404', 32, 2);
    T('9.766364616787074202022021617942580306437E+853339', '4.75E-94400', '9.76636e+853339', 6, 6);
    T('-2.477879366633922436942620379465344075853263140147E-179', '5.001E+607036568', '5.001e+607036568', 10, 5);
    T('-9.91773038789242415841400646241871432E+509270', '8.90240027196285459406834697800823E+384', '-9.9177304e+509270', 8, 3);
    T('-6.4216117911855785555196586953491283327671659034E-4267768410', '-2.71549651651035474742851384005686907E+932623751145375', '-2.71549651651035474742851384006e+932623751145375', 30, 6);
    T('7.505003437177338901537585712517292440418147498390807673406E-41013596', '-1.6377957063913576274E-2362545', '-1.637795706391357627399999999999999999999999999999999999999999999999999999e-2362545', 73, 2);
    T('6.52497142975743790E+59803', '7.70070919859302927342891E+79853350467950', '7.70070919859303e+79853350467950', 15, 5);
    T('-1.986586534614591954662873350021191E-8494702330585', '934883196.36', '9.3488319636e+8', 31, 4);
    T('-9.34393830187381530019933058983221630473183E+1322081', '-5.8113420569E+76250834', '-5.8113420569000000000000000000000000000000000001e+76250834', 47, 3);
    T('2.8472949996959453188308291822038006419E+58488338835694', '-8.40525201109E+450', '2.8472949996959453188308291823e+58488338835694', 29, 2);
    T('9.648367041406758034023991036E-59498', '-4661212.7762884155284840804331492224870144396865168', '-4.6612127763e+6', 11, 0);
    T('6.590E+563652649062720', '-2.470866644580659989139348934871662E+5393', '6.59e+563652649062720', 52, 4);
    T('7.50298455E+409885709899897', '7.8195903135920240406104586171019944177061863195512E+38137135', '7.50298455e+409885709899897', 21, 3);
    T('-2.085709164207E+556', '-8.65687769072345279667007546405481189141239951957731228E-445', '-2.085709164207e+556', 24, 6);
    T('-7.59901046473456174157176600977643736E+928', '3.1554648E+9567428', '3.1554647999999999999999999999999999999999999999999999999999999999999e+9567428', 68, 1);
    T('-1.764893485808065014931071720288894996414317946E+4373', '8.207697685678414861699991146707E-54431221200', '-1.764893485808065014931071720288894996414317946e+4373', 69, 0);
    T('1E+3438', '6.10225432462566325701089103E+413299377797', '6.1022543246256632570108910300000000000000000000000000000000000000000000000000001e+413299377797', 80, 0);
    T('-2.52752725708781022951571784E-699814829', '8.17875673731E+550628791', '8.17875673731e+550628791', 41, 5);
    T('3.4142120654239282339061777091E-137153793', '-1.920473198839259830870283143608138835313005551161E-5700', '-1.920473198839e-5700', 13, 2);
    T('4.916E+67267431864055', '-3.11232626041972E-22119634', '4.916e+67267431864055', 43, 0);
    T('2.7805554685841989593647311623180945E+631402138248535', '-3.5045033749573135402009952508161E+422962095', '2.7805554685841989594e+631402138248535', 20, 4);
    T('-6.7851124456156495E-283663', '-1.9748544E+991285', '-1.974854400000000000000000000000000000000000000000000000000001e+991285', 61, 3);
    T('3.3999495398879477127431E+26', '6.94787088935990247137662674286E-30255690293', '3.3999495398879477127431e+26', 39, 6);
    T('7.38629782790796531025130356311386902379257377504731E-65498', '-9.996570772195629416377330436034192E+586560433', '-9.99657077219562941637733043603419199e+586560433', 36, 1);
    T('-65427.2626599177561639832012637131774577343', '-2.2602461224542757053386510475184E+45', '-2.2602461224542757053387e+45', 23, 6);
    T('-7603337863100331037227641358.6358488010354381615972252', '0.079095503', '-7.60333786310033103722764135855675329804e+27', 39, 6);
    T('-8.566805532967771E+1133407170134', '-2.62014507E+59410816402904', '-2.6e+59410816402904', 2, 2);
    T('-6.378824969667553954007858399705411E+6125807', '1.5498189328683385253997376566304691018861347216E+978678273514', '1.5498189328683385254e+978678273514', 21, 4);
    T('-2.7745277583174075108879767960293E+57', '-1.04898713859989870135460729303852467980207715713082811008E-52522188', '-2.7745277583174075108879767960293e+57', 43, 4);
    T('5.6416413863730610446514344935740516542544173047838529477345E+3643189785224903', '4.325844E+10237387', '5.6416413863730610446514344935740516543e+3643189785224903', 38, 0);
    T('9.5700358273E+3281436620', '-3.49529922278383002447248797548714279747957316073562965362E+231765765674926', '-3.4952992227838300244724879754872e+231765765674926', 32, 0);
    T('5.746256598050270E+270179101110', '-8.69E+9', '5.74625659805027e+270179101110', 52, 0);
    T('-3.43703754975841976988985601863218657090565779695942448E-584', '-7.04870954464451748226487792925E+6452', '-7.04870954464451748226487792925e+6452', 65, 1);
    T('3.669191803775848342191491816028514958908276103E+6104885', '6349200469.49622348667724526141571168', '3.669191803775848342191491816028514958908276103e+6104885', 57, 4);
    T('-6.6413415085265200380682995061672743168E+6742608268982', '-5.7854761668979413E+40390639739697', '-5.7854761668979413e+40390639739697', 74, 5);
    T('8.131061529004687961E+4703014322894957', '6.6631960163605E+336774893', '8.1310615290046879610000000000000000000000000000000001e+4703014322894957', 53, 0);
    T('2.61586047680708591504734E-73618452078', '-2.82E+649040159313928', '-2.81999999999999999999999999999999999999999999999999999999999999e+649040159313928', 63, 2);
    T('3.806193950290037447482606404806098753E-310306977712459', '6.73574380489237E-6148', '6.73574380489237e-6148', 61, 4);
    T('-2.999244594782511304449990054060750394689E+665698507', '-7.86539E+624', '-2.99924459478251e+665698507', 15, 5);
    T('-1.73138291075097215041056825045207084790023E+9919178', '6.901091644374740436286975650710270008065430429870778E+112688678', '6.901091644374740436286975650710270008065430429871e+112688678', 49, 4);
    T('-4.4158131733719201035971650235401451699E+8478781', '3.43659130529850845160044047E-84414725879', '-4.415813173371920103597165023540145169899999999999e+8478781', 49, 2);
    T('8.2490530603E+563080552827', '-7.0892036675186933319756716833232912692817124518421644588185E+503', '8.249053e+563080552827', 7, 4);
    T('4.4812397022812291341308065939147761041239042598E+173199103', '-2.55358479553032556192092676679E-406', '4.4812397e+173199103', 9, 3);
    T('3.63707497063891323415475164E+19564220', '6.86E+394624254369', '6.860000000000000000001e+394624254369', 22, 2);
    T('-1.76521690385342100091110955772116032382348E+22342', '2.066371211836190804699093814076260613148846118E+930', '-1.76521690385342100091110955772116032382348e+22342', 46, 3);
    T('-9.16540861788939936221696041264140509892899684806066624E+545382', '-6.795036513456510225425505285065601005815059E+53651315619590', '-6.795036513456510225425505285065601005815059e+53651315619590', 52, 6);
    T('3.8824853845483747950121911961308E-232889426110', '-9.39085569262226048802461108556961372903363E+20482563', '-9.3908556926222604880246110856e+20482563', 29, 3);
    T('5.669348078931699485812E+91135', '-1.09634534582515905312199364521354019E+2398', '5.669348078931699485812e+91135', 43, 6);
    T('-7.108420703596696174864577599199E-17865', '-3.5821234495870040888988341424E-38476', '-7.108420703596696174864577599199e-17865', 68, 2);
    T('3.0587937455435385568686E+858411', '7.5633990990140136053230800443528E+6979747049', '7.563399099015e+6979747049', 13, 2);
    T('-9.4201452050384019635554408654254853829585833173937E-67881175400', '8.7594589610901982328939776149140582313134280', '8.759458961090198232893977614914058231313428e+0', 51, 5);
    T('-4.872E+9', '7.049434162615329092286249077949314195166656228776654449E+7179602312410905', '7.049434162615329092286249077949314195166656228776654449e+7179602312410905', 57, 2);
    T('2.498050094918515195272495E+25602819644', '1.690189280483229506407151914477E-6281659114721093', '2.498050094918515195272495e+25602819644', 37, 4);
    T('-2.872594834762158998189624857E-3872766009', '7.84730498600204522802E+1918690570', '7.8e+1918690570', 2, 3);
    T('-8.5989284208919716747311350415768937298660825849E+8362030996', '7.246737029123850373402361691951291E-8667053', '-8.598928420891971674731e+8362030996', 22, 2);
    T('8.9119677110510714267573580010087886701724745920E+21469', '2.11712768007778858425E-2709430696', '8.911967711051071426757358001008788670172474592e+21469', 61, 3);
    T('-4.6389773729060896356268643578053980E+271990', '5.4794710777462795179129364629207044179495471E+68796013', '5.479471077746279517912936462920704417949547099999999999e+68796013', 55, 1);
    T('-8.692108349146276787672E+502575719452551', '2.101910086621574891507701870425441356167851002049453E-36058', '-8.692108349146276787672e+502575719452551', 71, 4);
    T('-4.329643128068417315145546886117425E+47', '-5.180272153613851594615192162296169326241993204242351102905E+61', '-5.18027215361389489104647284646932078171085437849235111e+61', 54, 3);
    T('-4.366295227912472100879989655235852323995260684E-3443005916835', '6.6495301021110825661911619989342504206315E+243165734316192', '6.6495301021110825661911619989342504206314999999999999999999999e+243165734316192', 62, 3);
    T('-50.3340777108096062382934887949536591632954', '-2.327861974724190621584E-95290518158703', '-5.0334077711e+1', 11, 5);
    T('2.891402E+308984', '-4.4030600568577068902242588950310064E+3309250', '-4.4030600568577068902242588950310064e+3309250', 38, 5);
    T('2.9063245181838078406E-30082705149', '-5.21164603510E+700', '-5.2116460351e+700', 47, 3);
    T('9.172623893884291050640178462622576979394001E+74551900', '-0.0006992318506880861375813973557823740630779243140040', '9.17262389388429105064017846262257697939400099999999999999999999999999999e+74551900', 72, 1);
    T('-8.45743812419841308359736070937E-7', '-3.7150162603440907497300108323452688532070E-47707102611', '-8.45743812419841308359736070937000000000000000000000000000000001e-7', 63, 3);
    T('-5.64048998369545130320593838176021852293273725718E-66', '6.4546418096890813418E+8932686', '6.4546418096890813418e+8932686', 47, 5);
    T('0.000206', '-2.46E-16691451673', '2.06e-4', 72, 4);
    T('9523206424.817811834542598837948480678179531962407', '-9.246E+122', '-9.246e+122', 61, 5);
    T('-7.10637843522872405148177939345694170244998E+393578612', '-5.1686557840917978536969896875975293786383104244203680E+2623', '-7.10637843522872405148177939345694170245e+393578612', 39, 0);
    T('-9.701743299218118469518328922466346421531319820575184691E+987052143', '-2.162146263864726687779E+279733', '-9.701743299218118469518328922466346421531319820575184691e+987052143', 76, 6);
    T('2.425681641011836402167519E-745580', '-6.975349406316865333556880973581197441792824E+77261', '-6.9753494063e+77261', 11, 4);
    T('-9.9615667808682717784639047503416E+431098', '-5.411587171905E+752176928289', '-5.4115e+752176928289', 5, 2);
    T('-7.93800341064442267449278738313266140E+6661', '-4.77520007398717199039734239605332385947778750068587220E+772955635746', '-4.7752000739871719903973423960533238594777875006858722e+772955635746', 67, 5);
    T('-4.3950852214287588188735202190E+667593915681', '6.903412426009159788320109541849891301209580355174E+192215971290', '-4.395085221428758818873520219e+667593915681', 44, 3);
    T('-8.52768540649733978324430E+689', '6.5091092511E+6976032213402', '6.5091092511e+6976032213402', 47, 2);
    T('-551.540567153246944', '8.86155528029344977E+1979683', '8.86155528029344977e+1979683', 40, 6);
    T('-2.717789E+3942366982137037', '-2.4634911223721540585053403E+461', '-2.717789e+3942366982137037', 53, 5);
    T('6.166E-6244253', '-9.3214367939176849033412886654725530336291E+4194', '-9.3214367939e+4194', 11, 1);
    T('5.20725596945602124349E-6628650332', '8.3926653673662259021000369E+619', '8.39266536736622590210003690000000000000000000001e+619', 48, 2);
    T('-5.47534147329743050378910264292307917745886652072040802E+4736849', '-6.36656218431E+903', '-5.47534147329743e+4736849', 16, 2);
    T('-9.2797481508655335153340193408747852559E-762', '2.521199983786748708733455394509E-4192408404897193', '-9.2797481508655335153340193409e-762', 29, 0);
    T('3.861229821495293537193961672144521609397610358852924709084E+69897997373384', '-4.187275007792772117094721014960E-4794184599539', '3.861229821495293537193961672144521609397610358852924709084e+69897997373384', 58, 4);
    T('2.411723E+4353339572', '-7.68531916850E+98133448828193', '-7.6853191685e+98133448828193', 63, 0);
    T('0.188933017020775851110962818646204457535104092933546166', '-3.19557555791310599673359150099731090768E+6558', '-3.1955755579131059967335915009974e+6558', 32, 3);
    T('-3.77704574461102506821116718590213463452971444226E+24181884', '7.89843889931508016397563E+54501333520', '7.89e+54501333520', 3, 1);
    T('-1.694348343970E-19062624043964', '4.74085362358688899617185491288841032345E+1818992', '4.74085362358689e+1818992', 15, 6);
    T('7.827548798871167738828135403355260658546930138086182E+88209119442', '-6.873098406387768E+8107575038668', '-6.873098406387768e+8107575038668', 45, 6);
    T('-7.793847268035948651213204898194065634671277463640679113E-71021', '-9.25192017503693470678244673283726041560455402088E+468797185420', '-9.251920175037e+468797185420', 13, 0);
    T('1.5324356680755304707388397601754351260260E+51800671', '7.750256E+2067810458386606', '7.750256e+2067810458386606', 68, 3);
    T('-4284207863.48162898771759870845159620427053872002953436', '2.678461563685438181787E+2175', '2.678461563685438181787e+2175', 41, 2);
    T('3.509925697559788603605165685667402961597749778E+579972621', '9.49354694944266950726785182650136718343516254583305492E+615166919', '9.49354694944266950726785182651e+615166919', 30, 2);
    T('4.997254278769027827125462337E+116287155053', '-8.3420550960652692014E+73857', '4.99725427877e+116287155053', 12, 0);
    T('-1.5471238E+12818502839', '-2.51624E+6670', '-1.54712380000000000000000000000000000000000000001e+12818502839', 48, 3);
    T('4.03845806575613111013609373029E+6641', '2.50582882960752577743961966535833E+34733022', '2.5058289e+34733022', 8, 0);
    T('-6.27404897554E+512651292', '3.850202960342140538175124315512092E-2468457050130455', '-6.27404897554e+512651292', 37, 4);
    T('3.73543241298342843814894899458386501E+2691695896154', '-9.724815059386122233255407618425E+4203582', '3.7354324129835e+2691695896154', 14, 0);
    T('-8.8231459456021072857818669866309764613E+930262643401808', '-5.249691790665394148103E-74240164', '-8.8231459456021072857818669866309764613e+930262643401808', 73, 4);
    T('-2.15335223663222506124798820406557843215348E+4202729672452141', '8.45980008962682894307E+774202820651', '-2.1534e+4202729672452141', 5, 0);
    T('-1.7799886819301366984596688945034565698432742036E-947941197049654', '2.213372627032946638192686843830E+240141770080', '2.21337262703294663819268684383e+240141770080', 40, 2);
    T('-9.4144E+2477', '5.13E-1594851437729515', '-9.4144e+2477', 28, 6);
    T('-6.84E+94', '2.4673099077000465752320309838269659987987170400370E-6504', '-6.84e+94', 52, 3);
    T('-5.071806225212E+5417270823657', '-9.624503E+2818', '-5.07180622521200000000000000000000000000001e+5417270823657', 42, 0);
    T('-1334623402060722808866044690398799614351953.37498706276', '-9.2877551960513631E-16062', '-1.334623402061e+42', 13, 0);
    T('-2.6442090E-332', '5.661168834684021053299709525328255302698212731427678781E+171776', '5.6611688346840210532997e+171776', 23, 1);
    T('-1.78E-383610148117221', '-8.4613992413390360803256099173E+37044824', '-8.4613992413390360803256099173e+37044824', 61, 1);
    T('-0.0000062017052170970729498384391456766778884529974040624', '-5.518773320738164882E+85135165', '-5.518773320738164882e+85135165', 65, 6);
    T('1.93714405569220407313824065271476230676E+467591435511', '-5.7783767686955108423425328373372848769672002449421056718363E-7519814813', '1.937144055692204073138240652714762306759999999e+467591435511', 46, 3);
    T('4.4360034512789350E+155348', '2.25210627639241929951317896718E+9412244', '2.25210627639241929951317896718e+9412244', 76, 4);
    T('-6.481935214532330996490E-46', '-3.025123871773769737508727467472456142864922733668871E+2854517544177134', '-3.025123871773769737508727467472456142864922733668871000000000001e+2854517544177134', 64, 3);
    T('1.494273480155723127276787860353929741454585651339923384424E+7030', '3.599955E+6368385830772', '3.599955e+6368385830772', 63, 1);
    T('-3.28608292746150680004217754E+688438', '7.3551633873882835599850217061608240606832358782E-74572376626450', '-3.28608292746150680004217754e+688438', 50, 6);
    T('-1.03266796446E-8278', '-4.0460202659250511215483363070320924549117537263881023827559E+8774208', '-4.04602026593e+8774208', 12, 6);
    T('2.48389743898982686504586E+5990118820', '7.105424023427069964143E+50201321230', '7.10542402342706996e+50201321230', 18, 1);
    T('-3.6762713182092660572370854E-293263731', '2E+81311851770632', '2e+81311851770632', 45, 5);
    T('-7.922246796162E+560934167437503', '-795.111719224266738776273297740051263152270177611848', '-7.922246796162e+560934167437503', 20, 4);
    T('-7.84986371289748940384675258290319452577143E+784559037253', '4.8553700563277E+499797945', '-7.84986371289748940384675258290319452577143e+784559037253', 62, 5);
    T('-5.01451226729986E+634128641536684', '5.1772935756137731440883468553701654868056893184113342E+12620', '-5.01451226729986e+634128641536684', 73, 5);
    T('4.8148097678679009821027564058818895196777521129850224405E+68964', '9.681157078623298477505420042500552450626592789128905E+207', '4.81480976786790098210275640588188951967775212e+68964', 45, 0);
    T('2.7266500623596292085843253101131091745735239E+7581005', '-5.1142051E+379807216817167', '-5.1142051e+379807216817167', 14, 6);
    T('9.345E+4050716137', '8.25864200040756587945263712166329463926916E+989168197213499', '8.2587e+989168197213499', 5, 0);
    T('-4.4958852106290149081075323202084592539E-86', '-5.01557417551668E+7208357370', '-5.01557e+7208357370', 6, 1);
    T('-1.942702E+87956369', '-6.0893584767832E-464800', '-1.942702e+87956369', 41, 2);
    T('-1.20679614544803855898645801212207601717304402844E+3013791221389', '-8.2076647453271044803940573308E+2853907916091', '-1.20679614544803855898646e+3013791221389', 24, 5);
    T('-2.851894798868666358828959513926E+98768599339296', '-3.990652968467072965998006858161830578E+683573721', '-2.8518947988686663588289595139261e+98768599339296', 32, 3);
    T('6.25935603116583092362404788361107180421574164239073421E-9473', '-9.577742473E+686', '-9.57774247299999999999999999999999999999999999999999999999999999999999999e+686', 72, 2);
    T('-0.0058890488', '-2.8E-962', '-5.89e-3', 4, 3);
    T('5.983877037E+772055650933', '5.97790820447058150727307175945652943256022347739E+7594931381', '5.983877037e+772055650933', 52, 1);
    T('-3.49906948286962159619846775300283489060141E-4413058701961', '-5.78971523E+368830178139', '-5.78971523e+368830178139', 34, 5);
    T('7.99749151865001217378490121516E+9002510549', '2.916E+632970249941742', '2.916e+632970249941742', 32, 6);
    T('6.330E+94716', '-2.547152E+802', '6.33e+94716', 11, 5);
    T('7.6823442662814070799812306776069858647829286E-3799184', '-7.769E+52', '-7.769e+52', 17, 5);
    T('-9.12907988042607711581374160076730389488382781685292E+741508832', '2.268473599477838877813920073444162787769215604E-560719592104759', '-9.12907988042607711581374160076730389488382781685292e+741508832', 67, 3);
    T('7.7870558162217243698180022032041422347135479211630E+866', '-7.7020027224428659E-145491', '7.78706e+866', 6, 5);
    T('6.80478260690757549309246441979E+1869908145', '2.177206989317966240E+7522960', '6.80478260690757549309246442e+1869908145', 27, 2);
    T('5.07211949960267751398063785947343590299568602130E+720174', '-2.08432572E+83019', '5.07211949960267751398063786e+720174', 27, 0);
    T('4.89816458567686737756451385267113E+12101101', '-9.90080E+857552754863', '-9.900799999999999999999999e+857552754863', 25, 1);
    T('-2.48566946979385137E-538', '-7.217064208353626291334100467692468361768858565308282E+415524765', '-7.2170642083536262913341004677e+415524765', 30, 0);
    T('5.048E-551635094', '-1.439E+8813632164337913', '-1.439e+8813632164337913', 59, 5);
    T('-4.3E+752212513104', '53992722691780303486849512407.39246980952144', '-4.3e+752212513104', 30, 0);
    T('-7.142585472595838E+92892003796883', '4.74357300426553962404753275927780859812305367191190071314120E+4633239443606751', '4.7435730042655396240475327592778085981230536719119007131412e+4633239443606751', 67, 0);
    T('9.56250E+282', '-1.976191E+524639328111177', '-1.976191e+524639328111177', 80, 4);
    T('58097.0062256871554607945977818233891850896884623286405535', '9.82736E+3437548955564', '9.82736e+3437548955564', 20, 3);
    T('7.9045762440256318425790659793143E-4965904173099', '-3.6938684611042279394235224593675E+38389', '-3.69386846110422793942352245e+38389', 27, 1);
    T('-7E-462234653159', '-6.6847478792421727085344807185389002E+637004169', '-6.68474787924217270853e+637004169', 21, 6);
    T('-561.2720091692360113067783104954803622754208325102871818', '-7.4130142024929865261318486934E+32164358643', '-7.41301420249299e+32164358643', 15, 5);
    T('2.9903723964619271448953261629215250845808357901962E-51544547', '-7.2845375732905385235824832397797E+2060566536801', '-7.2845375732905385235824832397797e+2060566536801', 71, 4);
    T('-8.9582345656083E+809069666', '2.046451423561257913699884388407E+1935885444277270', '2.046451423561257913699884388407e+1935885444277270', 67, 5);
    T('-2.856771363027963E+1198409198036', '4.605132333E+709', '-2.9e+1198409198036', 2, 5);
    T('9091734240572059777337372169615492503869182301.1247975', '-8.3901616123211619672445427440292E+133319243', '-8.39016161232116196724e+133319243', 21, 1);
    T('5.233579714171457111588E-49936', '-1.2617273074210E+6667', '-1.261727307421e+6667', 14, 4);
    T('-8.654883657758180169090304049968356295218972434990113E+14841560166', '8.33657387806916277E+597026', '-8.6548836577581801690903040499683e+14841560166', 32, 2);
    T('-8059331253.58719027386485290667153', '7.9565819816680714877389072141477744303559651093340424969392E+715299', '7.95659e+715299', 6, 0);
    T('-8.2287694685E+511', '-6.606E+375989867821', '-6.607e+375989867821', 4, 0);
    T('-9.32984690617E+31581741', '-9.42018175E+74364348717760', '-9.42018175e+74364348717760', 26, 4);
    T('-6.20085923820376E+8160673', '-8.795906485970620E+579650017409', '-8.7959064859706200000000000000000000000000000000000000000000000000000000000001e+579650017409', 77, 3);
    T('6111077.73449423349820841419947108867573056064817', '4.8591E-2967852', '6.11107773449423349821e+6', 21, 6);
    T('-5.6173317869768505246856457356986780E-645911516', '-6.48234137251910937E+13672413115716', '-6.48234137251910937e+13672413115716', 75, 2);
    T('1.9262000404646981284372692010779E+487924', '-1.535749951E+3638', '1.9262000405e+487924', 11, 0);
    T('-4.10762806215627722794306907745530381615754854740321381196382E-791154205', '3.028775154170974E+25608327', '3.028775154170974e+25608327', 39, 6);
    T('-5E+1218', '2.866422112642480598913127853764805280E+99948556185675', '2.866422113e+99948556185675', 10, 6);
    T('1.3687497629329880657607582012353633048331464879213827E+17963259014145', '-5.490245106915E+23852306753', '1.3687497629329880657607582012353633048331464879213827e+17963259014145', 69, 5);
    T('3.85212724377624295090535229379E+9478758', '4.0451450081831497E+87972', '3.85212724377624295090535229379e+9478758', 48, 4);
    T('-9.08908544982E+6617337143', '-6.5142538665739447016963430365560E+7631260', '-9.08908544982e+6617337143', 44, 5);
    T('5.959457309231332E+41', '-5.070718567389E+561214349', '-5.0707185673889999999999999999999999999999999999e+561214349', 47, 2);
    T('-3.728622750545506512080484234354229671633274E+158233', '7.179319446281008245342486526801997438459E+109038903341', '7.179319446281008245342486526801997438459e+109038903341', 45, 5);
    T('-5.7614024596800067218526669656238695848489E+224985466854', '3.2427446481735120819953731236322538554448401584592050E+8378', '-5.7614024596800067218526669656238695848488999999999e+224985466854', 50, 1);
    T('-3.4835323431888473771840263159327068602635020872E+7998239783115', '6.7428100586109632952356510043358059719760985E+2038539186746204', '6.742810058610963295235651004335805971976098499999999999999999999999999999999999e+2038539186746204', 79, 1);
    T('-6229133496.4893924446161519601394345107', '-2.276916342205932216117805016795613418000924136833E+16242080', '-2.276916342205932216117805016795613418000924136833e+16242080', 70, 2);
    T('5.35984699E-72107915904', '-7.76276928415635E+61389759002525', '-7.7627692841563499999999999999999999999999999999999999999999999999999999999e+61389759002525', 74, 1);
    T('7.1119339064979752108567832E+929808812', '-1.67553193934111426722745645998029754581442648081654760882657E-2916957357', '7.1119339064979752108567832e+929808812', 56, 6);
    T('-243072788544178675499517239569256568712198846202767712.62', '-4.33480855071507017886773199908E-35324630', '-2.43072788544178675499517239569256568712198846203e+53', 48, 5);
    T('-7.86878210709758E+9527', '-1.7336937634386E-9343103', '-7.86878210709758e+9527', 65, 2);
    T('4.079210074712408981696154435299450106E+93', '2.23415225492269115022951067888975527E+47', '4.07921007471240898169615443529945010600000000023e+93', 48, 0);
    T('-3.01237791565426512047386800693447E+678081', '-2.4751113813529797739111066953E+2882403566', '-2.475111381352979773911106e+2882403566', 25, 2);
    T('-4.64E-187769790111', '-3.186089369230379696332522E+2132959976908', '-3.186089369230379696332522e+2132959976908', 78, 5);
    T('1.3727020422764194180E+4584765', '-6.3616E+78692', '1.38e+4584765', 3, 2);
    T('-4.8117877099593937804832068934927E-3538', '9.13916952072141473559928404471708806E+93168534173363', '9.13916952072141473559928404471708806e+93168534173363', 39, 2);
    T('3.9617E+17367957171734', '-2.9450450488701070872960195710709108035833204E+2040162', '3.9617e+17367957171734', 39, 6);
    T('-5.371658726088474609E+287219966201', '-6.20518769279687306882E+49565185', '-5e+287219966201', 1, 4);
    T('-7419029829396747927663790849547151266796', '-5.9454693938125138515717817737024996E+918166986842', '-5.9454693938125138515717817737024996e+918166986842', 73, 4);
    T('3.07E+95277', '7.57199363734149184622458305448877770753853237840990735194E+5053804512331', '7.57199363734149184622458305448877770753853237841e+5053804512331', 48, 5);
    T('372', '4.0265633445977342886914858196025527970624174904E+64814780518', '4.0265633445977342886914858196025527970624174904e+64814780518', 49, 1);
    T('8.2434115226873751346578963854690734024845578463E+7994295397592428', '3.278854063807E+9359', '8.243411522687375134657896385469073402484557846e+7994295397592428', 46, 6);
    T('-7.45390E-812', '7.709625209977879913214128557108751E-7981196011445', '-7.45e-812', 3, 5);
    T('-3.3067553560851282138678987501752281E+7837691181', '2.95450700E-324345', '-3.30675535608512821e+7837691181', 18, 2);
    T('-4.132725853511692382203983950527438919940659554630E-64400138', '4.360967917876345295518228613189623E-7924', '4.3609679179e-7924', 11, 5);
    T('-7.623450291473352063748519929E+11275984419428', '-4.025668803346594949576561567E-93708553284332', '-7.6234503e+11275984419428', 8, 3);
    T('3.3495505285238580755935342866555E+716714246983274', '-4.76240159579095E+7974238', '3.34955052852385807559353428665549999999999999999999999999999999999999999999999e+716714246983274', 78, 1);
    T('-1.136660052521958775081132655E+780346585615836', '-1.4135244992E-646125418487', '-1.136660052521958775081132655e+780346585615836', 57, 1);
    T('7.7725969446340451788873889127408996E-54272576921', '-9.9100709E+53123563', '-9.9100709e+53123563', 22, 4);
    T('-2.07E+838401', '-2.172095490971831419956111597314147500878965497E+926', '-2.07e+838401', 56, 2);
    T('6.80671234274E+24353107821', '1E+4750938956', '6.80671234274e+24353107821', 35, 1);
    T('4.2085E+131473', '-5.1978121837876172298218903357871360922005E+2099146', '-5.1978121837876172298e+2099146', 20, 1);
    T('7.069049473296496848033268695052361630426E+9568897081', '4.386868537351368673177869E+390306297083752', '4.38686853735136867e+390306297083752', 18, 6);
    T('-6.16478547257945184155358047295366261455E-38', '-2.05173968079268247011459019923423420235943277500358858E+5308523078501', '-2.05173968079268247011459019923423420235943277500358858e+5308523078501', 60, 5);
    T('1.383306490919538688E+81879426409289', '-2.265643768191163573874470768963025734277031037574241340460E-2379', '1.383306490919538688e+81879426409289', 55, 2);
    T('6.65075972731238922E-930707069', '9.45703E+4509158241767', '9.4570300000001e+4509158241767', 14, 2);
    T('-9.67250937130732794417157057421320331204604270225E-574', '7.737619718560572622199221865663491231727577E+782142770968324', '7.737619718560572622199221865663491231727577e+782142770968324', 61, 2);
    T('-89022.41482369848442890908256566643503482318530572528', '-5.695027009E+39', '-5.69502700900000000000000000000000008902241482369848442890908256566644e+39', 69, 4);
    T('-1.179762010268198245134282222435204048846859133388E+68346761', '8.362818497E+8896956578994', '8.3e+8896956578994', 2, 3);
    T('-3.236195523012E-83254192', '3.378251334898933005797757298070303713450750E-2142943475475', '-3.236195523012e-83254192', 47, 5);
    T('2.175532119419909257429360635876470707716347E+1182196405810284', '9.464118566590516974257535899157E+154099855', '2.175532119419909257429360635876470707716347e+1182196405810284', 58, 4);
    T('-1.85132291032742043751664841732E+6749817', '-5.51291831643359159273855611591861000641174141E-3605', '-1.85132291032742043751664841732e+6749817', 37, 1);
    T('-2.2E+623', '-9.68323984843808465603512E+183103449888', '-9.68323984843808465603512e+183103449888', 44, 1);
    T('-4.1828527968981705591657649860294016662956868724355853E+19307151738', '5E+544965526789', '5e+544965526789', 8, 5);
    T('-0.00002106140160087411819018460948258476', '5E+95325', '5e+95325', 18, 4);
    T('-9.03580108114826811062868525189870794017579791E+85492885978', '4.475647546864518189E+4051302309', '-9.0358010811482681106286852518987079401758e+85492885978', 42, 3);
    T('6.2061097171903904854635354E+72', '6.9015321E+72', '1.31076418171903904854635354e+73', 58, 0);
    T('1.1271190477663817879311605112662756265493718175791E-99055996513', '4.0284287354371351719008499649E+611668504239704', '4.0284287354371351719008499649e+611668504239704', 53, 5);
    T('-9.070491212596496609867481793574894990535496090177868969854E+658840970078', '-8.762526950581E+1026278', '-9.070491212596496609867481793574894990535496e+658840970078', 43, 6);
    T('8.369641142361236162304528128393153546691876416984E+868902939086', '88804.9739647026947806803702930945515597341930611269861526294', '8.369641142361236162304528128393153546691876416984e+868902939086', 63, 4);
    T('6.25303187109832956485812114E-332', '-4.16057982689671800552E+47', '-4.16057982689671800552e+47', 71, 3);
    T('-6.9097762286078E+103218989497771', '7.0148859392951759145156525969964358929113614070644E+965971647', '-6.90977622860779999999999999999999999999999999999999999999999999999999e+103218989497771', 69, 2);
    T('-8462160.18230790532620920473885801869', '-6.823491622045784142358976141661068180768388482486E-88', '-8.46216018230790532620920473885801869e+6', 63, 1);
    T('-1.38690581025619355662279638E+350', '-8.955517678E+1853909510957206', '-8.955517678e+1853909510957206', 43, 6);
    T('-5.662020023720939855003554858E+986417261', '-9.2325541872875317742243079569723516648762943536933E+8451619714730', '-9.232554187287531774224307956972351665e+8451619714730', 37, 6);
    T('6.3E+53457735417', '1.0697531572559526955214421502791847069093610417E+337274762167831', '1.069753157255952695521442150279184706909e+337274762167831', 40, 6);
    T('3.9639194078060371658112341512E+8634764363825', '5.6698008620633286745528866293971251897089840345693E+92903695103312', '5.6698008620633286745528866293971251897089840345693e+92903695103312', 58, 4);
    T('9.584900900395488487116295413021E+117531807633', '6.9324408293249E-1363303', '9.584900900395488487116295413021e+117531807633', 36, 1);
    T('-4.604250E-48595989500099', '9.4091100983915005728854438780350386722124E-37700994', '9.4091100984e-37700994', 11, 6);
    T('-1.2205998125948126093868793881263340265049E-998794', '3.649848500316357500547879920E+99', '3.649848500316357500547879919e+99', 28, 1);
    T('5.918177310439220307623448011380E-257', '-2E+6749795432', '-2e+6749795432', 66, 0);
    T('-6.5872656998444641442648407054E-767', '3.51329355E+43', '3.51329355e+43', 30, 0);
    T('-5.562719334179628563480E+318', '-39.08012730982717', '-5.56271933417962856348e+318', 63, 6);
    T('2.697889121257369578794302804713840199125414782E+722936866339985', '4.751392590949911333231539875346676790E+62614526', '2.697889121257369578794302804713840199125414782e+722936866339985', 80, 4);
    T('-8.4159737886565185959840386901023E+27998158934', '-0.0006910115130926718132200726', '-8.4e+27998158934', 2, 4);
    T('-3E+716433394', '3.7578484576584432474906E+2055714482908573', '3.75784845765844324e+2055714482908573', 18, 3);
    T('4.1991889000120954623433E+120229080204960', '-3.3546959E+9', '4.19918890001209546234329999999999999999999999e+120229080204960', 45, 3);
    T('-6.08543E+593', '-4.2796104838229607523910876902809840004902562E+34653525253701', '-4e+34653525253701', 1, 4);
    T('2.097885E+2218000078041', '4.0901314719937469579558216315076E+7927180549480', '4.0901314719937469579558216315076e+7927180549480', 59, 6);
    T('-6.84838207210244388737666E+751865040904331', '5.6342237604877455194367788E+3566137', '-6.848382072102444e+751865040904331', 16, 0);
    T('-6.8451100837629162348231001524006244741E-755406185', '2.81427E+40', '2.81427e+40', 22, 2);
    T('-8.10074412915767163954342093963E-879027289', '6.625725742052367170881318E+42326835776941', '6.6257257421e+42326835776941', 11, 5);
    T('-1.5468294728676E+95900', '-5.367279217862540884E+306281312532061', '-5.367279217862540884e+306281312532061', 51, 4);
    T('1.264071719E+321503958769375', '-6.1691460640507553006E+342', '1.264071719e+321503958769375', 19, 4);
    T('-2.0190695742646710403319E+66234', '3.427522354645614160599771916552730553886396746E+1672076199400898', '3.427522354645614160599771916552730553886396746e+1672076199400898', 68, 0);
    T('-1.27950222977734E+3618101661471', '-6.038794398E+18', '-1.27950222977734000000000000000000000001e+3618101661471', 39, 0);
    T('-2.89312941853532013161583001147737377090976778027249679E+941233067914692', '4.253036163217764443378E-507244721095', '-2.89312941853532013161583001147737377090976778027249679e+941233067914692', 80, 5);
    T('-1.3536812468636840678594E-236803', '4.36418509785519617E+73502037', '4.36418509785519617e+73502037', 32, 5);
    T('7.9674746447477270E+32024230', '9.378127937107139691153209079100915E+74421', '7.967474644747727e+32024230', 53, 1);
    T('8.6526468560473890123167324816104194119468735E-45492', '-9.09989833699618989418290584169989921293754881009537E+328', '-9.0998983369961898941829058416998992129375488100953699e+328', 53, 1);
    T('-2.017312910445149289870836785763240127750931575919499156E+650307495816', '-5.190143261E-8152', '-2.017312910445149289870836786e+650307495816', 28, 4);
    T('-5.6E-269576', '6.2177568119546215E+88218225511', '6.2177568119546214999999999e+88218225511', 26, 3);
    T('-68.5923815861262', '-3.811396801043640697131493382343876010462109E+70269300763209', '-3.811396801043640697131493382343876010462109e+70269300763209', 69, 5);
    T('1.34183756713717E-74940093', '-2.175904333624324764545701179292E-7612659894530876', '1.342e-74940093', 4, 5);
    T('7.3823394401894199541813669526973321281009006E+342964619755', '-2.7835169984777076297255968060492117442353849489382980019E-3932414405016', '7.382339440189419954181366952697332128100900599999999999999999999999999e+342964619755', 70, 1);
    T('3.50120867311240338577225E+98746437806395', '4.634063E+694234208', '3.5012086731124033857722500000000000000000000000001e+98746437806395', 50, 2);
    T('-8.8828437E+56', '-7.25397665E+8727060744642966', '-7.25397665000001e+8727060744642966', 15, 0);
    T('9.74635382016114470156547448407257344E+49850', '-1.1076849722027014065336890857136074823222610810E-9548340671471', '9.746353820161e+49850', 13, 6);
    T('8.897670839087223245089442771565E-378988393824', '2.3526883180071781814641756178335866180190260975904818586E+635805332', '2.3526883180071781814641756178335866181e+635805332', 38, 2);
    T('608.55', '77150540721455106827873201970.2593264916906504211808899164', '7.71506e+28', 6, 0);
    T('6.4067091417754590779085450081350996049285898291514993E+694340', '-8.5034905E+494', '6.4067091417754590779085450081350996049285898291514993e+694340', 67, 6);
    T('4.36091111444062354E+999893190041790', '-8.558549934089025241421814657431478803E+540863327424681', '4.36091111444062354e+999893190041790', 23, 4);
    T('2.02780522531675E+136520669', '-9.0788306407510641081565940668E+8724564', '2.02780522531675e+136520669', 67, 5);
    T('5.93767365838216619745732348758017158E+955', '8.9721E+431904020646180', '8.9e+431904020646180', 2, 1);
    T('2.8711643148116200158949330849885047181E-4600476362', '9.75359230961676936389552548160010E+48898513453', '9.7535923096167693638955254816001e+48898513453', 55, 4);
    T('-6.030923423855781598475618709786333713169903670511765E+2251933072', '-6.13420556973715923255349754248653E-3225321584897', '-6e+2251933072', 2, 5);
    T('-5.0643343191104039786855760123159419069764106624E-98164034180568', '3.26E+831244', '3.26e+831244', 9, 5);
    T('7.9721847193575692498E-6862754', '9.7863576467540E-2650985287153', '7.9721847193575692498000000000000000000000000000000001e-6862754', 53, 2);
    T('1.573624345267447620812635567020744272980281482501E-221456', '2.8539739189E+650', '2.8539739189e+650', 12, 5);
    T('9.637986740398752374655875060E-3225576540261', '9.922432863495047234277553E+488324', '9.922432863495047234277553e+488324', 64, 5);
    T('-1.97125828493768337239967389442233708197656129981818E-7773', '-3.96013273553579497952556645107287914498537E+749953246360250', '-3.9601327355357949795255664510728791449e+749953246360250', 38, 2);
    T('8.82633392266893780638594971747302E+61229617', '9.79196617176522263400962907E+69216771', '9.79196617176522263400962907e+69216771', 27, 6);
    T('-9.302E+58430888245', '2.6636053535239380639211536504878285554494542504889E+79', '-9.302e+58430888245', 39, 3);
    T('-2.535839780522381110942953914534457002360765692E+6526170967', '2.62901893438991069611578793857879593638283E+788506702', '-2.535839780522381110942954e+6526170967', 25, 3);
    T('-3.9345030935445477843107831E+64762840931', '-3.1095283664E+7659727206401166', '-3.1095283664e+7659727206401166', 31, 6);
    T('-5.5484066781366337113E+2491043234', '-8.550643596361753733664982399991E+5499636640', '-8.55064359636e+5499636640', 12, 5);
    T('-3.976379752653358042097409682967108553805676770208E-40967289670', '9.1059206981657962655E-61061817374', '-3.976379752653358042097409682967108553805676770208e-40967289670', 53, 6);
    T('5.9462859572100716863693502905346778635821306287894865E-832187', '7.8737352420E+68285205095552', '7.8737352420000000000000000000000000000000000000000000000000000000000000001e+68285205095552', 74, 2);
    T('-9.11734E+18', '3.2853037519476420901581776332264959408326665988606202841463E+6304', '3.28530375194764209015817763322649594083266659886063e+6304', 51, 0);
    T('5.408E-422768', '-4.8335290132E+201375094', '-4.83352901319999999999999999999e+201375094', 30, 1);
    T('6.6092880346019434575388497651410286421289864004619E+4656', '3.9295689345852656614938567253540505E+508', '6.6092880346019434575388497651410286421289864004619e+4656', 80, 4);
    T('-4.91401093941520751075270E+4185533155', '-6.7381334573014505967256547E+42772145075452', '-6.73813345730145059672565470000000000000000000000000000000000000000000000001e+42772145075452', 75, 3);
    T('-5.8801E+99602', '5.6576405606427550227433851380E+128', '-5.8801e+99602', 65, 4);
    T('5.0710745757516767273108615078180531823602028727499202502506E-610041', '-9.0852256033047655667019322791175377236422690704992E+9179', '-9.0852256033047655667019322791175377236422690704992e+9179', 69, 3);
    T('-5255068.0825602021074390953683541500', '-4.592648796456153745297889532992250988472527208E+7065130', '-4.592648796456153745297889532992250988472527208e+7065130', 57, 1);
    T('-8.61396448876019522705889E-41106369062753', '-4.41483E-492481', '-4.41483e-492481', 26, 6);
    T('1.14016605836177006720482326997467845E+14993503', '-6.177069655310505213411310366510839650686664264E+2157211', '1.14016605836177006720482326997467845e+14993503', 44, 2);
    T('9.300645020005693E+3504806', '-5146273.1057618938602833714714181477791932845153454810', '9.300645020005693e+3504806', 53, 4);
    T('-5.2599E-3314798970191587', '5.4855558375519149825404452047974310885988201117831335654E+26514', '5.4855558375519149825404452047974310885988201117831335654e+26514', 67, 2);
    T('6.59649576007342568868449989E+538872427755185', '891005.33873644936728042736', '6.59649576e+538872427755185', 10, 1);
    T('2.4554170563505956971541656717107582335E+34170418740515', '5.83837E-7653573454209795', '2.455417056350595697154165671710758233501e+34170418740515', 40, 0);
    T('3.41080223464725380344854090346498493563054230776E+7037668', '3.14842150141E+41335484610639', '3.14842150141e+41335484610639', 77, 1);
    T('8.681311157248225923748470344192829943E+231223026', '-1.07602358980463083281256556022721406134270139605003141390391E+14272011657878', '-1.0760235898e+14272011657878', 11, 2);
    T('8.225279625048415710E-9666971511366', '36496651859919549044121555436.37268263369', '4e+28', 1, 0);
    T('-8.614413777937082274727372801222121623313883147740278E-61723045895580', '5.28378902145634261235395340604401595937012799E+7419410', '5.28378902145634261235395340604401595937012799e+7419410', 73, 6);
    T('-7.077924E-1266241', '-3.913E+834901020942', '-3.91300000000000000000000000001e+834901020942', 30, 3);
    T('-3.732216167925363337458992550E+4177548766584500', '-8.7934860531224834587594863442634283E+1462688214811066', '-3.73221616792536333745899255e+4177548766584500', 52, 4);
    T('6.97755E+6', '2.2877809304854828E+4407', '2.2877e+4407', 5, 1);
    T('-45.4067286479389782840054', '8.91218613981377815287835E-188925528640', '-4.54067286479389782840054e+1', 62, 0);
    T('-4.800831476904E-85082047472366', '-4.379291E-93', '-4.379291e-93', 42, 2);
    T('-1.194105902974498122551096123E-94894098890', '1.5568136322038996483E-354014309250', '-1.194105902974498122551096122999999999999999999999999999999999999999999999999999e-94894098890', 79, 1);
    T('6.75696439265896E-86458437937', '-6.18815899276E-2388', '-6.189e-2388', 4, 3);
    T('-3.409024632670161023496280178531339457629E+27112316', '4.46068565317202874282395165E-495827551', '-3.409024632670161023496280178531339457629e+27112316', 53, 5);
    T('-8.813022012056952980968848339036938602362901545401799834976E+391789', '4.731941223371247463346388871E-23952433', '-8.813022012056952980968848339036938602e+391789', 37, 4);
    T('2.923027E+7530', '-4.419058716377620251E+4887595742770', '-4.41905871637762025099999999999999999999999999999999999999999e+4887595742770', 60, 2);
    T('-6.664622043984176E+4712094542244', '-9.32102112282264088803042213727564362E+66', '-6.66462204398417600001e+4712094542244', 21, 3);
    T('9.02650078E-467498', '4.9512729715812779516286425980E-549954546', '9.026500780000000000000000000000000000000000000000000000001e-467498', 58, 2);
    T('6.689E+85003676', '-6.638166578074876738622496356890159228155342785776E+60234', '6.689e+85003676', 15, 4);
    T('-2.05652510811513048748544693966238135E+818235279987367', '4.1709344738E+74574013379', '-2.05652510811513048748544693966238134999999999999999e+818235279987367', 51, 2);
    T('6.398711908004381554802283360719752E+4730095275933669', '-4.31188257820938081843719397433E-920870554', '6.39872e+4730095275933669', 6, 2);
    T('8.73156173934528358105E-953382092198', '1.340875238602177021401678575690530646009809571023927E-271', '1.3408752386021770214016786e-271', 26, 4);
    T('-5.95550E+205', '2.6250371757678874307877154206659878727065719650450089880E+41229666', '2.6250371757678875e+41229666', 17, 2);
    T('7E+4830388', '-9.8714354879284623160331871741477171205989649E+4186330687', '-9.871435487928462316033187174147717120598965e+4186330687', 43, 6);
    T('-4.3582926017193518533450323833708928536970E+625928309325', '-7.78629536748566236211756044018672869112292055163649E+3878224580', '-4.35829260171935185e+625928309325', 18, 1);
    T('-6.609279764226070813913560543984805556E+5187974', '3.905435051270559004992988312452E+230155836810', '3.905435051270559004992988312452e+230155836810', 68, 0);
    T('9.45887186206705E+73502', '5.96429970044883173203099491097390061980598800252174E-515181547125762', '9.4588718620671e+73502', 14, 4);
    T('8.89591337480739908E-3743', '-3.8735632122461E+397767219633', '-3.8735632122461e+397767219633', 72, 6);
    T('-3.153382278776414362284133112135439578116973502546065164648E+74851409249', '-3.48681230697632258727902322960960915324023093E+766', '-3.1533822787764143622841331121354395781169735025460651646480000000000000001e+74851409249', 74, 3);
    T('3.9336988381067397011905489E+527055189766', '8.64378899166797231243984509974017E+117148', '3.9336988381067397011905489e+527055189766', 47, 3);
    T('-6.328346210870118641610455940563281499E-7093', '4.452E+232368123', '4.452e+232368123', 6, 6);
    T('-5.41149301394611E+3527860638', '-5.5223448959516291E+420253319108', '-5.5223448959516291e+420253319108', 48, 6);
    T('2.574581610771706413056866071726985676312804627268933154E+94644971150599', '4.19564452200065037E+5871645', '2.574581610771706413056866071726985676312804627268933155e+94644971150599', 55, 0);
    T('4.338817746880378436738495919632651616263614596719991E-16853', '2E+5897995878425272', '2e+5897995878425272', 69, 1);
    T('9.71853832510229703370187576961518504499101205577E+9111269390', '3.2496E+50', '9.71853832510229703370187576961518504499101205577e+9111269390', 48, 1);
    T('-6.0218320829566386022648890733157200033176949278160808E+82557730169', '-5.361170016575593681E+143113', '-6e+82557730169', 1, 6);
    T('4.6490451654393844446651609606687868475756153576312E-376341', '2.0912257288342702429330869412758769521111E+78', '2.09122572883427e+78', 15, 6);
    T('3.66994928396252359649930732470531579201351871187E+99703133411809', '5.529480595445504035611119719527454873014652197E+812902246788', '3.669949283962523596499307324705e+99703133411809', 31, 5);
    T('-8.74517397956014322789450002358854144583299779882180502E-87701728', '27389405822313039778073140591625089949761116819676335.6', '2.7389405822313039778073140591626e+52', 32, 2);
    T('-5.021020055608114534378724810450139799833E+767625484', '-8.884576767698250355498196340E+7492833393', '-8.8845767676982503554981963400000000000000000000000000000000000001e+7492833393', 65, 3);
    T('7.62247750609277312122360492978483581579463629778131427161448E+142155730', '-1.366156567618910E+846', '7.622477506092773121223604929784835815794636298e+142155730', 46, 6);
    T('3.80236397982391857012210459271537523992857680345840985E+66118241', '-1.884520251072700423391655E+6217874997702719', '-1.884520251072700423391655e+6217874997702719', 56, 4);
    T('-9.2321759503981412397425795826926029942098E+8733527', '-8.7062475124E+556924354520332', '-8.7062475124e+556924354520332', 72, 1);
    T('-1E+2739', '1.07504967942503E+72399100', '1.07504968e+72399100', 9, 2);
    T('-4.501139811060428663086503396605E+7512952841277', '9.74754E+612863833268', '-4.501139811060428663086503396605e+7512952841277', 61, 4);
    T('-2.791920E-6979589', '4.583273832236155230494571397462096592220280771098596264E+469', '4.583273832236155230494571397462096592220280771099e+469', 49, 6);
    T('7.0187499136E+7492068713516986', '3.7142E+2095489320006', '7.01875e+7492068713516986', 7, 4);
    T('1.9019789382830376195918378941236968514484E+71410', '-7.99725347286638007091478935373392E+7365295302', '-7.997253472866380070914789353733919999999999999999e+7365295302', 49, 1);
    T('3338.5172841353250437158190589780380385578461660', '6.77505835256019499058101854770781E+7539', '6.77505835256019499058101854770781e+7539', 59, 3);
    T('-4.9444824029891309040139610391966744050392458E+8876674546259', '4.89404922143193474090165342957850665932E-6025919172081577', '-4.944482402989e+8876674546259', 13, 2);
    T('4.6936762453E+233633', '2.0298247403278282426503226083480322850955621952E-8131464822976878', '4.6936762453e+233633', 45, 6);
    T('-9.32605858974123003945425542726428573E+68805', '2.264033411227266600823672868731041126239E+6324382', '2.2640334112272666008236729e+6324382', 26, 6);
    T('9.1539648386573763709509451330207457217916752551140E+23040194', '1.627941876070761636770835706311844E+828318811692', '1.6279418760707e+828318811692', 14, 3);
    T('-6.4390826883656826596492556633348105286077347172155E+63033524473414', '-7.956411056870579819719891052088596785E+153011', '-6.4390826883657e+63033524473414', 14, 0);
    T('2.7064812231096664184153270330230630627E+8960', '4.44890E+14', '2.7064812e+8960', 8, 1);
    T('-7.327642780E+587240674278', '2.07656E-985537910254605', '-7.32764278e+587240674278', 69, 6);
    T('-1.42E+96055746', '-8.5604850422145679122796946005700E-4653494', '-1.42e+96055746', 7, 5);
    T('4992508.301645767523155459894678718123351604802445323895619', '3.72990E+13842745125710', '3.7299e+13842745125710', 80, 5);
    T('-1.256953594E+7236118344136', '-4.09465177284425514185539130054401074055459986086339943540E-83546135946', '-1.2569535940000000000000000000000000000000000000000000000000000000000000000001e+7236118344136', 77, 3);
    T('9.41029318105473329613253011839087290833275595995122371849056E-29625', '1.9307258474166719484463724285178525232866288768E+70920510546', '1.9307258474166719484463724285178525232866288768e+70920510546', 76, 5);
    T('9.813278809076871149704216239300830845772E+29235417875422', '7.726439068E-988962', '9.813278809076871149704216239300830845772000000000001e+29235417875422', 52, 0);
    T('-8.463639551664145703187886E-156', '279081667.59356802248980941113608257729512599', '2.7908166759356802e+8', 17, 3);
    T('4.47214184653505223492E+883515139', '3.309873456105815E+64313779158', '3.309873456105815e+64313779158', 32, 5);
    T('3.491602376219586080929052381565390173E+7489419310036271', '8.10306339952872193677742862466525541949283E+7626', '3.4916023762195860809290524e+7489419310036271', 26, 6);
    T('6.46583186092292730436357802632474725780631273259822452268E+2939004825', '-6.65941847768578026064519388121523E-28546746064', '6.465831860922927304363578026324e+2939004825', 31, 1);
    T('1.2710796647887570986885389E+640281', '5.9877E+5313', '1.2710796647887570986885389e+640281', 70, 6);
    T('2.771760951631352296263177504506321685499956876587860325415E+343435941', '-6.792699861338523228756017166452E+7780345', '2.771760951631352296263177504506321685499956876587860325415e+343435941', 74, 6);
    T('-6.20173382474746097657595687E+1448469961988', '-1.657862720655925862E-2011442610356', '-6.201733824747460976576e+1448469961988', 22, 0);
    T('7.19E-428778666962', '9.28150017607E-3474', '9.28150017607e-3474', 79, 5);
    T('-747.551680232156958777701799199491737550380489867119185020', '-9.73709891915652547616933631936419903292094E+651', '-9.73709891915652547616933631936419903292094e+651', 53, 6);
    T('-8.078E+3315302', '2.15169943796281232304472863921988890E+2567', '-8.07799999e+3315302', 9, 1);
    T('5.8E+39461267787915', '2.5148686588857181692588619317395E-31', '5.8e+39461267787915', 15, 1);
    T('-2.22064998995261039631383890501518945859422759945176340341E-635060', '-1.39843508580856562818161E+64778849', '-1.39843508580856562818161e+64778849', 73, 2);
    T('5.0034947083288E+363060935274072', '-4.66561393145727043E+619806221', '5.0034947083288e+363060935274072', 73, 6);
    T('24.963545620543750269937636534', '1.66087688E-6943765783047602', '2.4963545620543750269937636534e+1', 63, 5);
    T('2.92620971559898998370677002811838966620180373404214E+765499459035', '-9.423154983933572852327650800018652947E+6448557538461', '-9.4231549839335728523277e+6448557538461', 23, 6);
    T('2.5036763992E+495898874868823', '-7.56061998765E+82514018', '2.503676399199999999999999e+495898874868823', 25, 3);
    T('9.96718458512404477043E-1710123367716', '8.104905016458678631054282511661E+33', '8.104905e+33', 7, 6);
    T('-1.1900511840900E+5036591923240275', '8.8806031666267441124095697286490841782E+2241911599', '-1.19005118409e+5036591923240275', 38, 6);
    T('-9.7E-34', '5.7452E-54', '-1e-33', 1, 3);
    T('-2.310978793269215902937528986527468629E+99530314910', '-7.923E-1175647952319965', '-2.3109787932692159029375289e+99530314910', 26, 1);
    T('6.9518585698785793563297313E-945937', '6.932072394145325883412710200096049709866325251010349E+68', '6.9320723941453258834127102000960497098663e+68', 41, 1);
    T('4.271660437098185580170228994794504852515473072113049511E+6867343324461', '-9.619475429747231385107E-874917830245325', '4.2716604370981855801702289947946e+6867343324461', 32, 2);
    T('2.8244742854519608761599348370226032496180257E+797560705822049', '-7.27641103678856E+159287', '2.8244742854519608761599348370226032496180257e+797560705822049', 63, 2);
    T('-5.079803926450090090557901718539991325403110917548785208E+6562216', '1.271E+9', '-5.0798039264500900905579018e+6562216', 26, 3);
    T('-49607.52920242', '-3.0423605762260014205798310416784E+687914', '-3.042360576226001420579831041e+687914', 28, 2);
    T('8.319E+65', '3.450099689207871250018932572724713674993594305684395175E+1105524162723988', '3.45009968920787125001893257272471367499359430568439517e+1105524162723988', 54, 1);
    T('-4.155002715998248749260902E+351917', '-8.85351844209282695711327416305205905957026104336372532549E+918163', '-8.853518442092826957e+918163', 19, 6);
    T('6.677529483504817634992289924634252287435032024E-1398276710', '-8.02352672161419209922046679721242087229537998864E+5636072179776', '-8.02352672161419209922046679721242087229537998864e+5636072179776', 60, 3);
    T('1.00324334090313821425119773E+18162716', '-9.8311587249496686218477917338573149891E+4389576', '1.00324334090313821425119773e+18162716', 66, 4);
    T('-1.80675212281025983502183816255800130855681677499017073164E+38154248585736', '3.839745515139E+163721165580865', '3.839745515139e+163721165580865', 73, 4);
    T('-6.467118897564271943317E+31710530634', '-4.331781383544615507962118E+933', '-6.467118897564271943317e+31710530634', 54, 4);
    T('-2.00021771512644929873355287776353191315467611434075665555E-1268699717413292', '4.8989E+30374843', '4.8989e+30374843', 26, 6);
    T('4.7593360865E+590704292', '4.39E-602', '4.7593360865e+590704292', 14, 1);
    T('-2.5938885083523496740704149254713151E+49220864964', '1.811700261417346237315E+73185', '-2.5938885083523496740704149254713151e+49220864964', 35, 4);
    T('-7.4285E-5714', '-6.532075721513261186334596133875081264260077128E-2890085', '-7.4285e-5714', 5, 1);
    T('5.57351254343543582636699210879757142834691520880008862E-253659', '1.090332810401880412785395916897131E+7634050', '1.0903328104018804e+7634050', 17, 4);
    T('1.34759310826671526469601E-8181', '-72.38464556646799', '-7.238464556646799e+1', 48, 3);
    T('1.5682248146079441047586601742341487542547034070484E+945939911', '4.79066844416290593239787E+824630834216', '4.79066844416290593239787000001e+824630834216', 30, 2);
    T('4.2383985061254134405021602500526708871317E+4881', '4.526045039605043193476', '4.23839850612541344050216025e+4881', 27, 6);
    T('1.599E-969990', '1.5975593E+4022964489627609', '1.59755930000000000000000000000000000000000000000000001e+4022964489627609', 54, 2);
    T('-7142119.720911165874810462516269453305725667289912712701', '-1.848408768057294511758173408760522534264253031503300E+8459', '-1.8484e+8459', 5, 5);
    T('7.037419557287745E+313', '7.05860865497331E-10810', '7.037419557287745e+313', 26, 6);
    T('4.56495E-5808318263076', '-2.91194632540987269142712896E+6924', '-2.91194632540987269142712896e+6924', 32, 3);
    T('-9.8244430339083147076468477082627178751690687E+4254083643929', '-38621.038110350129118309575926520846538317922338', '-9.8244430339083147076468477082627178751690687e+4254083643929', 49, 2);
    T('-4.584909897360563298559054720592232664161817685E+628120', '-5.1917450941716E-190914', '-4.5849098973605632985590547205922326641618176850000000000000001e+628120', 62, 3);
    T('8.03243446143872757E+211483377', '5.61727436301333E-260', '8.03243446143872757000000000000000000000000000000000000000001e+211483377', 60, 2);
    T('-4.827225749886953538495076813742475360224238189712524E-3303158225', '-6.71729542E-978132542253', '-4.82722574988695353849507681374247536022423818971e-3303158225', 48, 2);
    T('-5.649122235004848E+536', '2.68181547624628851020596865E+3343', '2.68181547624628851020596865e+3343', 67, 6);
    T('-2.6380E-4507628', '8.99487E-73271464863', '-2.638e-4507628', 61, 6);
    T('-4.929E+70234249851318', '5.627635992026919817804603195821345E-5786080113', '-4.929e+70234249851318', 20, 6);
    T('8.64202393453221450512127541149945167949410714E+565440368128', '6.33233383642357448522006484378656939213809E-1610829306794', '8.642023934532e+565440368128', 13, 3);
    T('-5.92855144169776069339856502548175905921889095165203E-8082006', '-7.316449962573146326313114898373E+7824299', '-7.316449962573146326313114898373e+7824299', 43, 4);
    T('-5.85651314448476344633017015538879112075882122776671257E+6013591480', '4.26163133590791573434721691328064198300379701792739380E+6595493', '-5.85651314448476344633017015538879112075882122776671257e+6013591480', 62, 0);
    T('-3.164150183623036654534E+51875', '1.0331715804445681688E+698957', '1.0331715804445681688e+698957', 42, 5);
    T('9E+48483', '-7.214794132691567671310611996E+247698569870', '-7.21479413269156767131061199e+247698569870', 27, 2);
    T('-2.4216039528322470122855038816218344802073120714E+7506689889966981', '-9.0251413964597101E+352784708996694', '-2.4216039528322470122855038816218344802073120714e+7506689889966981', 56, 5);
    T('-0.000273294', '-2.455139510312E+6923306', '-2.455139510312e+6923306', 74, 4);
    T('-8.302800118630199459723591606687249E+98774131', '-95.478225292506152790138888725617720152620824243', '-8.302800118630199459723591606687249e+98774131', 74, 5);
    T('8.5388207560433728E-487531241199571', '4.6189690903892583097304455452838826E+122511151', '4.6189690903892583097304455452838826e+122511151', 47, 1);
    T('6.56365772838268429E+80232692861310', '9.64896627700934397894E+7824211', '6.563657728382684290000000000000000000000000000000000000001e+80232692861310', 58, 0);
    T('-2E+291838639227', '-8.0532964996850802E+8656261582854', '-8.0532964996850803e+8656261582854', 17, 0);
    T('-8.90E+4686', '-1E-7', '-8.900000000000000000000000000000000000000000000000001e+4686', 52, 0);
    T('4.402131638E-6217', '6146491945.8', '6.1464919458e+9', 36, 1);
    T('5.12097706894147087504468951401E+256864997288296', '-1.777884422157E+77650', '5.12097706894147087504468951401e+256864997288296', 44, 6);
    T('2.5811533962478340838808059474995493626E+982498068', '-5.6831181893984052911450512752979067947489E+8631', '2.5812e+982498068', 5, 2);
    T('-2.8734291715863002559123E+147', '6.25942844695424081412114007619833760502265344E+399756305', '6.259428446954240814121140076198337605022653439999999e+399756305', 52, 3);
    T('6.9289577407962086513411E+107322', '4.8537743672675427493798440456781101916796E-99502', '6.9289577407962086513411e+107322', 35, 6);
    T('-5.061791767042498992573E+2309319519375', '7.642114490594556355251068470707459434284820307467411805810E+884217491', '-5.061791767042498992573e+2309319519375', 68, 0);
    T('4.8182518E-2515', '-4.83640399353409456275E+697526474723855', '-4.83640399353409456275e+697526474723855', 33, 0);
    T('-1.177038E-78878', '7.137322321377794375953981899619219081E+93515792', '7.13732232137779438e+93515792', 18, 5);
    T('8.0072790197256363087348481654E-886', '9.659444066130731326587575E+6881660627', '9.659444066130731326587575e+6881660627', 62, 4);
    T('-7.17670797285406422768223041889654735567494905100658013956648E+638026293', '-1.7E+60214', '-7.1767079728540642276822304188965473556749490510065801395664800000000000000000001e+638026293', 80, 0);
    T('-4.03388180749058619699003510605847924E+78545958', '4.3625456613499683721958241204896001040021717005325752328E+39364297178981', '4.3625456613499683721958241204896001040021717005325752328e+39364297178981', 58, 4);
    T('-2.972372382721668939819639679916327411826264121880723457E+1781010824756504', '9.71153144441156540627678120156141000907940917403794829825976E-573', '-2.972372382721668939819639679916327411826264121880723e+1781010824756504', 52, 5);
    T('1.862475926914447653233418227051858E+740720742', '6.8493448181126371550788106130738344303E-645359978319030', '1.862475926914447653233418227051858e+740720742', 40, 6);
    T('-5.578567892243038503277319633407036215243259262150063121E+594731667695', '-6.3517297363775545121884948155101764722841E+368781436541', '-5.57856789224303850327731963340703621524325927e+594731667695', 45, 3);
    T('291.124807017175824801547138875810139596516418378', '-89.92102721555465498', '2.01203779801621169821547138875810139596516418378e+2', 77, 5);
    T('-8.509683427852276646670570409266557284828335318865907116E-39839', '1.557861459370794009408683571127872493667E-1255779494263298', '-9e-39839', 1, 3);
    T('6.422043355924505799317111081083971466157E+3349989', '0.0033184335419253', '6.422043355924505799317111081083971466157000000000000000000000000000000000000001e+3349989', 79, 2);
    T('-84.367077976340', '-4.16576872870125019268679945E+84305', '-4.16576872870125019268679945001e+84305', 30, 3);
    T('6.419442715248065526915355491681740863421E+6495050058826785', '-5.206680E+9281303', '6.419442715248065526915355491681740863421e+6495050058826785', 61, 5);
    T('-8.4088228473734913640627E+327', '-2.09574304941817920598E+6913681643', '-2.0957430494182e+6913681643', 14, 4);
    T('5.99446087297375E-7932', '-5.772420691411103390267667347783985689260171E+2767252', '-5.772420691411103390267667347783985689260171e+2767252', 75, 4);
    T('-6.41203656665513688375399802402278705152875299525431086826233E+979666858266955', '-4.27219E-380100827', '-6.4120365666551368837539980240227870515287529952543108682623e+979666858266955', 59, 2);
    T('-6.25902119991670457733749303467418929679599586332398460842E-174878', '-8.2780689664804985927E-64480593', '-6.25902119991670457733749303467418929679599586332398460842e-174878', 59, 5);
    T('9.8431309607901348197595528724708873E+49050', '1.15500901869589638010747292352285E+3911847', '1.15500901869589638010747292352285e+3911847', 56, 3);
    T('-2.12106081580260210255945848745137201793581158315964E+25131294640592', '-7.85165105852117376421049993408929860044848E+444126704454', '-2.1210608158026021025594584874513720179e+25131294640592', 38, 6);
    T('2.226189547517E+773466371451', '7.0003089470039364E+4621403733677585', '7.00030894700393640000000000000000000000000000000000000000000000000000000000001e+4621403733677585', 78, 0);
    T('6.61443100729216275764908834386016575464653E+312916490935810', '9.43649732874441825128E-676693579', '6.614431007292162757649088343e+312916490935810', 28, 1);
    T('2.00316367455750763441863132393624952003E+1664776295953', '8.9041429682881927604452761496890904865503241514246712651051E-41', '2.00316367e+1664776295953', 9, 4);
    T('6.16310007323656E+923827938105', '-5.237117564730560853952E+2717', '6.163100073236559999999999999e+923827938105', 28, 1);
    T('7.973804113785698272010640978103310705492180197703029625E+426', '-1.616596006582394668283412445265827926465E+181546784669', '-1.616596006582394668283412445265827926465e+181546784669', 48, 6);
    T('4.4E+57', '3.99E+6199817', '3.9900000000000000000000000000000000000001e+6199817', 41, 2);
    T('6.407773391879524E-624218926406576', '5.864631545659357543848307123243933897691521705569062751724E-51503787491', '5.8646e-51503787491', 5, 1);
    T('-2.364965462745980617994777296333455262151884322083226414E+1213629023', '-3.79453064026732668425808079905E+2673033', '-2.365e+1213629023', 4, 4);
    T('-5.62345042766371782919812370405114004564152137E-141', '-1.8069303919949469931013818326394724624E-63', '-1.8069303919949469931013818326394724624e-63', 69, 6);
    T('7.19796265933963494574706299811E+83107792', '5.37310896774246898951619110285502982706E+68306410255', '5.3731089677424689895161911028550298270600000000000000000000001e+68306410255', 62, 2);
    T('6.60297728E+9963', '9.4213714886539761623977E+394199349', '9.421371488e+394199349', 10, 3);
    T('-8.47595362473254148933622419130878331368251361E+2400722106', '-9.52465269051174472096029260815165200339158227E+8041815834545', '-9.5246526905118e+8041815834545', 14, 3);
    T('4.4656488642942594963839E+373796430679938', '-1.056640657382110218E+3128091', '4.4656488642942594963839e+373796430679938', 63, 6);
    T('6.7297112E-73453', '5.0386353730752848497832812296405428989277853279570314153E-394223256223534', '6.72971120000000000000000000000000000000000000000000001e-73453', 54, 2);
    T('2.84839799575E+623615817625', '5.50889327941821228297829749394910904479858259360828E+7798477', '2.84839799575e+623615817625', 30, 5);
    T('2.07068453940E+4906058175651', '-4.508495233170590884E+8233', '2.07e+4906058175651', 3, 4);
    T('6.76396620108847357118847693333451331E+491605', '2.791082362780758603053178517336857867402793002307167849E+3596', '6.76396620108847357118847693334e+491605', 30, 2);
    T('-5.0645434564003444438468E+6344', '1.984E+5910981958', '1.98399999999999999999999e+5910981958', 24, 3);
    T('-1.651695591500904E+118915709719', '2.93926765163691371499680590478E+492564945217', '2.93926765163691371499680590478e+492564945217', 40, 4);
    T('7E-836803474', '4.791582739946E+366882796783', '4.791582739946e+366882796783', 60, 5);
    T('-4.72160666280E+23484590', '7.33136321653969173774205876750E+90080', '-4.7216066628e+23484590', 27, 4);
    T('5.57904842681446727324E-69190592808', '-17.918415343989609375257262175893344195530635918723369165', '-1.791841534398960937525726217589334419553064e+1', 43, 5);
    T('4.563266233542410668424258176720105070E+7046704441902', '-5.700E+82397532290', '4.56326623354241066842425817672010507e+7046704441902', 60, 6);
    T('7.698906499359180629654244E+34137', '1.20483887475046673151612268808663688597266E+8611324064', '1.204838874750466731516122688086636886e+8611324064', 37, 6);
    T('8.44580825136855356627150417912859140889386281884686612892931E+4770', '-6.744664458952416199794371843229337722429516E+4541615', '-6.7e+4541615', 2, 5);
    T('3.1569652506796065351705526396160648867426528048027430959087E+26547', '-4.989994E+61342', '-4.989994e+61342', 36, 5);
    T('9.62508871777897399988555008669904E-5128244', '-9.3321340E+2418196670', '-9.33213399999999999999999999999999999999999999999999999999999999999999999999e+2418196670', 75, 2);
    T('1.05049542597264733425411022852426765E+85258', '-5.0188301016972641281185242503012E+7140229282', '-5.0188301016972641281185242503012e+7140229282', 76, 5);
    T('-0.23155200325363154603083821560839407666', '-1.191983215485E+86367116961620', '-1.1919832154850000000000000000000000000000000000000000000000000000000000000001e+86367116961620', 77, 0);
    T('6.577296990559E-3569926217661', '1.89097325994409302057883856628223469536E+6101204', '1.890973259944093020578838566282234695360000000000000000000000000001e+6101204', 67, 2);
    T('7E+8031229', '4.05349021928512412448845923836308834593E+297765528415', '4.053490219285124124488e+297765528415', 22, 1);
    T('-6.20526072758893684201403135481280705447918196549080710989E+721750762', '6.468211363736011041528394819884E+7049631408', '6.468211363736011041528394819884e+7049631408', 70, 0);
    T('5.77864067417853822250311418757717518211106638198899844E+75405982757', '4.458E+9331399499959', '4.458e+9331399499959', 73, 4);
    T('-3.5890615418212225883486026221245704777866326281771080336057E+736', '-3.10899334878902412E-451', '-3.589061541821222588348602622124570477e+736', 37, 2);
    T('3.80504878191229324354011932160391143603E+2032224', '6.57240456626171385593681865143840308158898900925449E+4774791267621', '6.57240456626171385593681865143840308158898900925449e+4774791267621', 65, 4);
    T('9.6610172921069549979395883660358763791806249868652E+4869', '-4.044922249E+27423985', '-4.044922248e+27423985', 10, 2);
    T('-3.23441338692447615E+731667319946640', '1.6564004817431066315512571583E+8737112727', '-3.234413e+731667319946640', 7, 1);
    T('5.32E+687286785614', '3.84454572069275180741366546137713388314265322533246334048E+20663709', '5.32e+687286785614', 49, 4);
    T('8.8207743377180485184933796865273E+2818', '156708.35111610655960356251113', '8.8207743377180485185e+2818', 21, 2);
    T('8.07600E-3167114890609796', '4.49379785612193203546441944699154417651288200E-67617878473229', '4.493797856121932035464419446991544176512882e-67617878473229', 55, 1);
    T('-9.66660328901624012717348158441891393E-1614', '7.31E-629961403', '-9.6666032890162401271734815844189139e-1614', 35, 6);
    T('2.12254666498470963829759595300436242711E+966104378854970', '9.071274948263947819068114449605E+6819759', '2.122546664984709638297596e+966104378854970', 25, 2);
    T('8.496919048E-31325', '-2.566E+709415962476', '-2.566e+709415962476', 69, 4);
    T('8.306712361416869109630518877958672E+323737', '-7.9951255548188953895539E+783959319', '-7.9951255548188953895539e+783959319', 54, 3);
    T('-6.681886785165379127381E+9275', '1.0726545292422254866258191427482759408534602693264821059E+1989743', '1.072654529242225486625819142748275940853460269326482105899999999999e+1989743', 67, 3);
    T('-6.728802075E+876107948', '7.3512384720247429540931554335845784511696731313599649E+150584118891', '7.35123847202474295409315543358457845116967313136e+150584118891', 50, 6);
    T('-28.81037133826276750', '5.1936556447492191735350890130856026298360983970431E+2249', '5.19365564474921917353508901308560262983609839e+2249', 45, 3);
    T('7.10099910618603162602913895464216183877374670763177E-664701', '2.9497134176736888244719710189373062001792229363001946E-7040302304129787', '7.10099910618603162602913895464216183877374670763177e-664701', 69, 1);
    T('1.64175944015415812140933751787E+444', '1.2954473668084313244420E+223426141', '1.295447366808431324442e+223426141', 62, 5);
    T('6.581535322866298020044368782141255E+70', '-5.62744989463671461270153048386765E+284644000508', '-5.62744989e+284644000508', 9, 6);
    T('6.0512562041716870055332392106040106E+2357992', '63003817715628643691298785558004.2909447940061335543016897566', '6.0512562041716870055332392106040106e+2357992', 38, 3);
    T('1.06688046725342509E+2896110354804', '1.716332293516042191413285237950474221E+61590174884072', '1.716332294e+61590174884072', 10, 2);
    T('-1.512451018475257876277127821968523125838297056202426270213E+3144717232', '-18738205.4490511337042595567871167004228561726499011786', '-1.512451018475257876277127821968523125838297056202426270213e+3144717232', 60, 2);
    T('4886.4347871', '9.151247525058993769771469427723658911334614944817E+134177', '9.151247525059e+134177', 14, 4);
    T('-3.094340E+554283948', '5.0768121656851995197401968847961083039099467E+8108', '-3.09434e+554283948', 15, 3);
    T('5.6313414235294575732096707798E-31287', '8.8E+79835', '8.8e+79835', 31, 3);
    T('-5.3E+574998754356951', '-1.69638098336E-58755671610', '-5.3000000000000000000000000000000000000000000001e+574998754356951', 47, 0);
    T('-8.792120704243169221963458368538849883234893489905922752388E+587', '7.49572234668198069505142815588526899353E+567570708', '7.5e+567570708', 3, 4);
    T('-2.33920259118167E+50114', '6.78548455924243699957687591127497874706583278840772902107E+634851546479226', '6.7854845592424369995e+634851546479226', 20, 3);
    T('-3.646645421734242504698443629037473098514801281E+650392548649896', '9.96492526364441139607916306374819748246849068534981018489926E-62440', '-3.64664542173424250469844362903747309851480128099999999999e+650392548649896', 57, 2);
    T('9.5102722654268157800697653108392846930298E-47263', '-9.0610824786E+91453766140531', '-9.0610824786e+91453766140531', 78, 4);
    T('-5.831689E+777404', '7.622159514755800E-3478851665170356', '-5.831689e+777404', 70, 0);
    T('-39.80065889251010300744667', '1.514895557005345609827084129E+713371460278', '1.514896e+713371460278', 7, 4);
    T('-7.149679568403758452881575701400163658449105777641515E+127502583928', '3.822698618550147519252184348622514773822087862894E+4545853236', '-7.149679568403758452881575701400163658449105777641515e+127502583928', 72, 6);
    T('-3.745E+24544', '3.375497742775844691015092354361010E+120', '-3.745e+24544', 49, 5);
    T('-3.60348795228017780049569753533186294E+499', '8.80786821E+82223318', '8.80786821e+82223318', 28, 0);
    T('-9.8967145096522848735804634E+98677519', '-8.39861E+5784323495519916', '-8.39e+5784323495519916', 3, 1);
    T('-5.54480709948277E-718373854', '-7.119627881088288640389987E-896127305535', '-5.54480709948277e-718373854', 56, 6);

    log('\n ' + passed + ' of ' + total + ' tests passed in ' + (+new Date() - start) + ' ms \n');
    return [passed, total];
})(this.Decimal);
if (typeof module !== 'undefined' && module.exports) module.exports = count;
