var count = (function sqrt(Decimal) {
    var start = +new Date(),
        log,
        error,
        passed = 0,
        total = 0;

    if (typeof window === 'undefined') {
        log = console.log;
        error = console.error;
    } else {
        log = function (str) { document.body.innerHTML += str.replace('\n', '<br>') };
        error = function (str) { document.body.innerHTML += '<div style="color: red">' +
          str.replace('\n', '<br>') + '</div>' };
    }

    if (!Decimal && typeof require === 'function') Decimal = require('../decimal');

    function assert(expected, actual) {
        total++;
        if (expected !== actual) {
           error('\n Test number: ' + total + ' failed');
           error(' Expected: ' + expected);
           error(' Actual:   ' + actual);
           //process.exit();
        } else {
            passed++;
            //log('\n Expected and actual: ' + actual);
        }
    }

    function T(root, value, pr, rm) {

        if ( pr != null ) {
            Decimal.precision = pr;
            Decimal.rounding = rm;
        }
        assert(root, new Decimal(value).sqrt().toString())
    }

    function isMinusZero(n) {
        return n.toString() === '0' && n.s == -1;
    }

    log('\n Testing sqrt...');

    Decimal.config({
        precision: 20,
        rounding: 4,
        toExpNeg: -9e15,
        toExpPos: 9e15,
        minE: -9e15,
        maxE: 9e15,
        errors: true
    });

    // Test the special cases specified by ES 15.8.2.17
    T('NaN', NaN);
    T('2', 4);
    T('0.1', 0.01);

    assert(false, isMinusZero(new Decimal(0).sqrt()));
    assert(false, isMinusZero(new Decimal('0').sqrt()));
    assert(true, isMinusZero(new Decimal(-0).sqrt()));
    assert(true, isMinusZero(new Decimal('-0').sqrt()));

    T('Infinity', Infinity);
    T('NaN', -Infinity);

    Decimal.errors = false;

    T('NaN', undefined);
    T('NaN', null);
    T('NaN', []);

    Decimal.errors = true;

    T('NaN', -1);
    T('NaN', -35.999);
    T('NaN', '-0.00000000000001');

    // Test against Math.sqrt of squared integers.
    for (var i = 0; i < 3000; i++) {
        var j = Math.floor(Math.random() * Math.pow(2, Math.floor(Math.random() * 26) + 1));
        j *= j;
        T(Math.sqrt(j).toString(), j.toString());
    }

    // Initial rounding tests.
    T('11', '101', 2, 0);             // 10.04...
    T('11', '111', 2, 0);             // 10.53...
    T('1000', 999000.25, 3, 0);       // 999.5

    T('10', '101', 2, 1);
    T('10', '111', 2, 1);
    T('999', 999000.25, 3, 1);

    T('11', '101', 2, 2);
    T('11', '111', 2, 2);
    T('1000', 999000.25, 3, 2);

    T('10', '101', 2, 3);
    T('10', '111', 2, 3);
    T('999', 999000.25, 3, 3);

    T('10', '101', 2, 4);
    T('11', '111', 2, 4);
    T('1000', 999000.25, 3, 4);

    T('10', '101', 2, 5);
    T('11', '111', 2, 5);
    T('999', 999000.25, 3, 5);

    T('10', '101', 2, 6);
    T('11', '111', 2, 6);
    T('1000', 999000.25, 3, 6);

    T('10', '101', 2, 7);
    T('11', '111', 2, 7);
    T('1000', 999000.25, 2, 7);

    T('10', '101', 2, 8);
    T('11', '111', 2, 8);
    T('999', 999000.25, 3, 8);
    T('3.7', '13.69', 2, 6);
    T('14.8', '219.04', 3, 6);
    T('3.578708384438441', '12.80715370085', 16, 6);
    T('49.56954417823', '2457.139710037', 13, 6);
    T('4.9808503290100979', '24.80887', 17, 6);
    T('54108953.7650348268963935650889893674872', '2927778877546676.5690784769302', 39, 6);
    T('1.0000000000001', '1.000000000000000001402', 14, 2);
    T('18.000001', '324.0000000000000000000008467', 8, 2);
    T('11.00000000000000001', '121.000000000000000000000000001944', 19, 0);
    T('2.00000000000001', '4.0000000000000000000000000000000005485', 15, 0);
    T('7.1', '49.00000000000000000000007338', 2, 0);
    T('19.00000000000001', '361.0000000000000000000000005645', 16, 2);
    T('1.0000001', '1.00000000000000000000000000000006986', 8, 0);
    T('30771277.0274560007853901874282412', '946871489900441.411712749999999999999999999999999999999999999999', 33, 6);
    T('668.093253221434649746', '446348.594999999999999999999999999999', 21, 5);
    T('0.9999999999999999999999999999999999999999', '0.9999999999999999999999999999999999999998', 40, 5);
    T('0.9999999999999999999997187097975391240474677563824', '0.9999999999999999999994374195950782480949355918890297218761543222489261', 49, 3);
    T('0.0000000000000298840312673111495324166498096924391126062701', '8.9305532478563043E-28', 45, 5);
    T('0.0000000000000000000000000000000000000000025385115498645', '6.4440408887956349899441566682289191111111111111111111E-84', 14, 5);
    T('0.0000000000000000000000000000000000000000000000000000140471526414353053458592377645343274655348630930173397', '1.97322497331782876158645511100784258994087355048761111111111111E-106', 54, 5);
    T('0.00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000002598902022257132166703490111187879281161770014282895', '6.7542917212922111E-232', 52, 5);
    T('0.00999999999999999999999999999949', '0.00009999999999999999999999999999', 30, 1);

    Decimal.toExpNeg = Decimal.toExpPos = 0;

    T = function (value, root, sd, rm) {
        Decimal.precision = sd;
        Decimal.rounding = rm;
        assert(root, new Decimal(value).sqrt().toString())
    };

    T('5.673474190707827242511954659916889896808177096292E+5527890', '2.381905579721376621997401692376763128098896247600842643458999047531881813e+2763945', 73, 6);
    T('2.4243067414399499211289451653514095936017185263165775841595802781681024396317601E+93564572', '1.55701854242008e+46782286', 16, 4);
    T('9.831517146017554599160038642659577233621137326296138739740339E+71067122669', '9.9154007211093362846141108573489066595648018e+35533561334', 44, 0);
    T('498935009536979677258304315852850344980736298107223.17', '2.233685316997e+25', 13, 4);
    T('3.2251633136582109206181686755297199980106981532535904612E+90465', '5.6790521336e+45232', 11, 1);
    T('9.262423536278148172467483107114956962585715251496115897712401E+3974801387007393', '9.6241485526139652228166208472049625290167818011e+1987400693503696', 47, 1);
    T('2.365349102526598437724926901530502896022219085536E+4424243805', '4.86348548114065869236934236337303e+2212121902', 33, 3);
    T('7.9972751000198231728604329670070818587227114744864529357097524020E+6671706171817', '8.94274851486936491099499873074488855754871805417147437494613942878964986078571e+3335853085908', 78, 6);
    T('4.36221283967684692666426416376714916311505871923000872592271965E-8', '2.0885911135683898927279398736031752344015545504649127170644896370353379e-4', 71, 6);
    T('6.442826670208612947748038374549405718510075898758412768822080597099708051296751984704859E+2047408259', '8.026722039667633182260408511913520451120702453123749778e+1023704129', 55, 6);
    T('6.05547475474079E+7716547504038688', '2.460787425752332228460163766079909e+3858273752019344', 34, 4);
    T('1.986589642800241768950253744134264443767020159900398126805300834E+858381', '4.4571175021534285e+429190', 17, 5);
    T('8.935356909E+235670062680425', '9.4527e+117835031340212', 6, 1);
    T('9.24E+9808723612', '3e+4904361806', 1, 5);
    T('3.83064972286605667223095591399211128840797098184470613547390920507003215236913837823828877362E+876', '1.957204568476697755184197782894827661216187868664840919284816726085640525e+438', 73, 0);
    T('6.89437868256825998885730788226521728356623423193748251E+69057745', '8.3032395380167e+34528872', 14, 0);
    T('7.041883770880118425878239862595091826896816E+3423985', '8.39e+1711992', 3, 6);
    T('9.585780485802596005974448193E+2405', '9.790699916656927140739512410042075323596621289368295024430694756362358e+1202', 70, 2);
    T('9.696453136853983765292198202484493417225363497463637965E+538865', '9.847056990214885399626178374025660624889212963684855438227064306989910207e+269432', 73, 2);
    T('3.013383328415989492093650804382794630962008467500528389245351791E+5494', '1.7359099425e+2747', 11, 5);
    T('4.670450646729720489233321886824609831695025907588563949E-408349586002047', '6.834069539249451103168749378990917542555757999511237679195e-204174793001024', 58, 4);
    T('9.848828875917495153223800564466445801240606621764417514110475E-69696042354140', '3.138284384168760310983231869196862541278664719980474337283125219910900471e-34848021177070', 73, 0);
    T('1.6074418819849188E+15095629429', '4.00929156084328544059400729461e+7547814714', 30, 5);
    T('8.80913641710599005691978950691155189458746110141164551755555797229303947815057725607373E+327', '9.385699983e+163', 10, 5);
    T('4.2611631259291605782486841695067759569661374268622981237845E+8790', '2.064258493001581567732632230025244966622767522065920193424751662709921e+4395', 70, 0);
    T('4.93520377319001414151248980920968837885642082455528005489000815E+436516', '2.2215318528416409230049e+218258', 25, 3);
    T('2.6348230972226040854267304152051578667034961411058694242668334095640189241199450266226838E+92609835059379', '5.1330527926591638410946628719813202245977800293e+46304917529689', 48, 0);
    T('5.407289389E-392', '2.3253579055706672e-196', 18, 6);
    T('6.2667203408185268363604194770009720671739738664656179598783769617970284596514793072E+353510805403548', '2.503e+176755402701774', 4, 5);
    T('9.287023667623263145602226199798895429249583886386354524632320596574924425720535087689428938434E+721848493356927', '9.6369204975569156895116858673802898447044593766985526107265769189051484968e+360924246678463', 74, 3);
    T('2.16620350490895E+185587049650215', '4.65424913912969424744230257268e+92793524825107', 30, 3);
    T('3.61572856993018165706134046141609828036662065978484170009821375392882703132369124228069E-3579', '6.01309285636782899752364197007684920017398969437949e-1790', 51, 1);
    T('9.04357854390761345545547419189409265295580849E+824874022101', '9.509773153923080543680696577222501710491282865e+412437011050', 46, 0);
    T('1.582109397061640889607372828919871087547023353618955613E-88622660', '1.257819302229712520962174337277372176639810260661e-44311330', 49, 1);
    T('3.144985535630261990939138487693918541845307202896600571E+9406698917', '5.60801706098533725826615935476591733e+4703349458', 36, 6);
    T('2.043593980785019831258664920277034009002215803890866800625E+66559', '4.520612769066843919448985334359993005870337091660370281988247839555503e+33279', 70, 5);
    T('1.5234893142617446461800063194E+49928995749465', '3.9031901238112199422244215131891752614639270293050848390039e+24964497874732', 59, 1);
    T('2.802265850566844477905639463307E+87038925514', '1.67399696850587052605991771251848e+43519462757', 33, 2);
    T('9.1027340475372175567247E-161499', '9.5408249368370749687627590642478e-80750', 32, 4);
    T('5.896932135288822595493869E+343698', '2.4283599683e+171849', 11, 0);
    T('4.2961949068353E+997208191559988', '2.1e+498604095779994', 2, 0);
    T('4.859974646744411255356204868440980E-715603808578', '2.20453501e-357801904289', 9, 1);
    T('3.3811160746774874681847190789838042E+993004253599', '5.8147365156793540374533779611537425212e+496502126799', 38, 4);
    T('9.3343471095787978641040077990615E+67653', '9.6614424956e+33826', 12, 1);
    T('4.6791271910133013331845019336634770531377970763582548051313437429709245928538732275037320149E+730795511370333', '6.840414600748481921789126213883996657219910998072878638534975744655859056193334e+365397755685166', 79, 1);
    T('5.8757722224252300019927334806719E+4314193400', '2.4239992207971581775833393874986290518e+2157096700', 39, 2);
    T('67601922.776862534581068550153256371638256888653847213492979247915305924432354685660', '8.2220388454970543626652913918343189618675218734756075784049153881385219e+3', 71, 6);
    T('2.6976251482762043514129832714354246217401306620466914174989739297066373978821E+18681017572774', '1.64244486917406282584413656933972205192398555455095912262410121870819135e+9340508786387', 72, 5);
    T('9.586185845626716371804688230667770283863635486094832181975090257E+6231354877', '9.79090692715782911695847092370781210596001172136e+3115677438', 48, 5);
    T('3.78021968581403435601384837583880755915455510843E+4232001854477068', '1.944278705796582640753477900250165813717781441628885212874406345152815184e+2116000927238534', 73, 5);
    T('2.3826705341252710709297819787306105797004429874259653221553023657650287E+89500217435', '4.8812606303344129781885860648384540427484823894462977105e+44750108717', 56, 3);
    T('372824401.9171471', '1.93086613186193490703915304629355648384e+4', 39, 5);
    T('9.982629046836824729046499174522512414789196378588531892708638648812537073974580E+323789492', '3.1595298775034276812747891834935798726890112101053025250708e+161894746', 59, 1);
    T('1.93305608E+90502884', '1.390343871134044306144904825487640471083e+45251442', 40, 0);
    T('8.8153594369939084379355385E-963178569784340', '2.96906709876922593571912431731393589808133769511469663747755797342953511989756e-481589284892170', 78, 4);
    T('2.563810328886424765192984306061855294460082173029294221865432386446793287430868E+74895328808423', '5.06340826804083097846774064356525633462189965902266516580957088231376938e+37447664404211', 72, 6);
    T('3.567572052133644795106468790474408870787279617581832818761015389739299185393099191398142701264032348E+665507', '5.972915579625786738e+332753', 19, 0);
    T('9.7977982184E+591625720348', '3.13014348207873691395112227e+295812860174', 27, 5);
    T('2.59549542566482274489688514562643282E+58764161568196', '1.612e+29382080784098', 4, 0);
    T('4.88485916656072684684336288223350318510382769049147063964695662933003E+1914', '2.21e+957', 3, 1);
    T('7.44462024840675394866139306388888378538969579877716771591568819584308250214770680342900129457E-17481835', '8.62822128158912829157811086596256560701532576e-8740918', 45, 0);
    T('2.588795374034416984035765330740978157904525653087970199E+994893608395544', '1.608973391338221287e+497446804197772', 19, 4);
    T('6.892258379923950714603548436010577093809697037965095264743890886686628674142064667547853365035865E+6150615437', '8e+3075307718', 1, 3);
    T('3.116613269355929545695883758738082553843696E+52270', '1.7653932336326458078973623890876147360894729071996321e+26135', 53, 6);
    T('8.5690693762507461E+690431144251215', '9.256926799025012329e+345215572125607', 20, 3);
    T('2.53556566595150796637413433477866041357009510845074403852348490716635E+92391640330945', '5.03544006612282891778e+46195820165472', 21, 1);
    T('4.896457515808248945640496172912496043632630141762245831365224969393680585716718487583992616E-96591232194676', '2e-48295616097338', 1, 5);
    T('6.19407396991944743243101974561954532539988692605208840104874752842254662E-727147183563578', '2.48878965963768168184605548022681e-363573591781789', 33, 6);
    T('2.9E+1480722117', '5.3851648071345040312507104915e+740361058', 29, 3);
    T('6.95780153928886055351192302500612019875561321E+81', '8.3413437402428516353e+40', 20, 5);
    T('7.81933048856490367778167731312448221121779492491922341813551588784321465352788E+1827982585608', '2.79630657985938724890492968896027809909551e+913991292804', 42, 6);
    T('9.4186465107248546707630495330709378212295744168927053819144383155698062121265279905897752271E-268', '3.068981347405821994329370885223153147810167995e-134', 46, 5);
    T('3.6326235162953181809577645353279210E-21562694801', '6.0271249500033e-10781347401', 14, 1);
    T('1.023440095025536568635708921517360961793E+51591', '3.199125028856e+25795', 13, 6);
    T('3484238.1773551099774', '1.8666114157357738546399e+3', 23, 0);
    T('2.598313772753815009002620426366701559965597345321262947368693172122554312019668885696226081E+3384126875841', '5.0973657635624059747024576803245965274249298527568449922667e+1692063437920', 59, 1);
    T('4.05349168598253649731512E-6476', '2.013328509206219057784323487222458501070404001508e-3238', 49, 2);
    T('7.28327820781098773E+8040122579382280', '2.69875493659779856084765987819363578742030895189293632989705922592411229915e+4020061289691140', 75, 3);
    T('2.56E+20', '1.6e+10', 13, 5);
    T('5.51476778824035712778945842083575307125895176285682237794763477393558870924644509568829084003207E+5467092689345068', '2.3483542723023e+2733546344672534', 14, 6);
    T('287553939655483170302812649.82525337373758443', '1.69575e+13', 6, 0);
    T('1.75120965737370433697922956385416755835455646442365E+16571888578160', '1.32333278406215884733018366812891959496232303384e+8285944289080', 48, 3);
    T('7.4301940172078196756317577124190795116444648589485897991299472748929474290863699960E+847349358786893', '8.619857317385143165541404858165098391611310078189659021366304034052302190198e+423674679393446', 76, 6);
    T('2.3473718E+7480', '1.5321135075443986e+3740', 17, 3);
    T('2.9701307839704062069609474255783085619215767002151887992880369960243164553E+2599738335304512', '1.72340673782204014190446948179592390656068185428e+1299869167652256', 48, 6);
    T('7.01672069590995012448612570344661497015681231635520477773E+74951314', '2.6489093408250026638898665191805392071832816e+37475657', 44, 0);
    T('57034702.6067040290231940776601570114357688435285519762590782006101866084636618667', '7.552132321848183098292052759964393e+3', 34, 1);
    T('6.2595818799207554600051E+692734998306', '2.501915642047260445562789772566841521129383e+346367499153', 43, 2);
    T('1.20E+47010259969', '3.46410161513775458705489268301e+23505129984', 30, 5);
    T('3.0422745572463449598911698968741967861068325178395628338533947208549426458197501E+5071597585', '5.5156817867298553573934377553326519175269141671462400489745247227004977385606e+2535798792', 77, 0);
    T('4.9446671019E+826967119083757', '7e+413483559541878', 2, 1);
    T('5.54138693771155115387408E+4320600', '2.3540150674351154530407163580243074454938968196e+2160300', 47, 1);
    T('7.49365480783773944781718916811056453166482639687697E+5757092', '2.737454074105672e+2878546', 16, 5);
    T('5.556317327015934668013876961309779446079811E+4683213943251', '7.454070919313777677e+2341606971625', 19, 5);
    T('1.33943032639686490800430635561822376604163268393006414770600700814939E+716778442', '1.1573376026021382616589933753247694e+358389221', 35, 1);
    T('3.49591541130498529E+65808471966', '1.869736722457198060474037674032156648711972906521e+32904235983', 49, 3);
    T('5.5859818323020679E+72237', '7.47395e+36118', 6, 0);
    T('8018207.9519871366038039483866918309675753055797380955099583126482331855373', '2.8316440369487011e+3', 17, 0);
    T('9.50092114211839024068782916975E+2619072017', '9.7472668692913041841429e+1309536008', 23, 4);
    T('6.58531E-5193812114577', '8.114992298209531e-2596906057289', 16, 4);
    T('4.49381922866E+803', '6.703595e+401', 7, 6);
    T('9.527345E+2499', '9.7608119539308818711822517856627039556134970352903e+1249', 50, 0);
    T('9.04E+278099851346', '3.0066592756745816541757861126198711210412117e+139049925673', 44, 4);
    T('3.594873509E+695170', '1.896015165815e+347585', 13, 5);
    T('7.1236555794326143979309006340E+2010', '2e+1005', 1, 3);
    T('2E-5867', '4.4721359549995793928183473374625524708812367e-2934', 44, 6);
    T('9.63107223078929964397336E+9220828195635', '9.81380264259950340885274731932875646303790396039793934033661431416500117441683e+4610414097817', 78, 0);
    T('7.98812467625871776375673355260343E+72781515', '8.9376309368079848924360696e+36390757', 26, 2);
    T('2.526723146387E+5141324359', '5.026652112874930197997606836236916458650894248379748587283939631792571414e+2570662179', 73, 3);
    T('7.212992952916182126166161894531743826E+5837806549469', '8.49293409e+2918903274734', 9, 6);
    T('1.4225997558149815433987209975829468787464264438709401098671706038917792294221101238217259236638882E+9663757', '3.7717367e+4831878', 8, 6);
    T('3.19515162002174718875005691558428708625371358895103E-391', '5.65256722208745359096703556418756741838091330344004243880813554127204475354963e-196', 78, 3);
    T('4.39882266444868312494846698153144297198308093870920987520682921E+46635', '6.63236207127e+23317', 12, 1);
    T('1.06071279858520053783126987482987178921249572000906994047388785433323881546006E+510', '1.0299091215176223197e+255', 20, 2);
    T('2540.90347226867370536208924268972271855242897', '5.040737517733564457496704009949621179767208845752392579636770019086715865e+1', 73, 3);
    T('0.000053288059927007419736192864488572091336455240976257828090775274140470718273556545610819', '7.2998671170787362309999885642096413366997803441454046e-3', 53, 1);
    T('26124059285086859749.111136068760', '5.11117005049595857693e+9', 21, 5);
    T('3.607550940261579490044139098611195000453932161335896883380180003006836843018711910962E+99370', '1.89935540125106114644973371135677739395218e+49685', 42, 0);
    T('3.82471809615120769875272018837130277169788007341654336023385958906492891E-5230806369', '6.18443052847326250790598455586e-2615403185', 31, 3);
    T('1.1739137610476275565237282E+37', '3.426242491487763270227e+18', 22, 4);
    T('8.7779172807427879568E-7356', '2.96275501530970122781213e-3678', 24, 5);
    T('5.9676230838955962140555831660429425584144796031304001629260382152102420202408466740096475145421788E+9718', '2.4428719e+4859', 9, 0);
    T('1.05745837668529806545913238826109255689541898043645621468495695830204838516639758771694008345342674E+885568', '1.02832795192e+442784', 12, 2);
    T('4.2442727888456152E-4089', '6.514808353931537716266282122142219803851312224850046724512e-2045', 58, 0);
    T('6.5051088543E-2894345501082', '2.550511488760636812199575483724868e-1447172750541', 34, 5);
    T('6116030396839332618454241.9526022655923587418704940810349550246004', '2.4730609367420230671388e+12', 23, 5);
    T('9.21925827069403988819584961758405864669074310023419843237E-2699970', '3.036323149912413372105696029126241615660685857583888271128260449310702453141608e-1349985', 79, 2);
    T('7.222649305417681173051748752199025762E-182', '2.6874987079843743011487811643134819286318198642391692062301e-91', 59, 5);
    T('7.888036125057898099450298304157513376364713379776394983E+326880501', '8.8814616618312878350673062471290132392165769044e+163440250', 48, 6);
    T('6.4062501571203598084182532613957872182544E+60293795', '8.003905394943370898574902868328421705723747639106911357526019275287081172297e+30146897', 76, 0);
    T('4.47141713850001280335127436138703209048665710727894216146638123448970973413333877E+7765', '6.6869e+3882', 5, 0);
    T('8.962707657665966551992881052251375867047729017714166968495362566E-654147055', '9.46715778767100012677112937995161815357113020444e-327073528', 48, 6);
    T('3.97332464199955790588897588438551310212027677784886403329179755857444550904597263134605E+550112649312565', '6.303431321113571e+275056324656282', 18, 4);
    T('4.32460691516810613850278067385309182411399820938966096E+43204703953977', '6.576174355328564536662125352346e+21602351976988', 31, 2);
    T('4.311036068901801732099430365860481210539754465419778810616524065681770638162008279484055169E-6181237058088934', '2.07630346262337137491871083e-3090618529044467', 27, 2);
    T('1.7765290096628665705861617573159354925958E-17424578889', '4.2148890965989444057133600997628746172165814692968051092e-8712289445', 56, 6);
    T('7.0092097860506405451155195833366486325362949236190817489245792816903602999857931117317190E+325146', '2.6474912249241999225341670266712257663525163332933359876990081330726543e+162573', 71, 0);
    T('0.00008333392362252263723524459', '9.12874162316595732639551387628400033604695187619e-3', 48, 4);
    T('7.4175502864775620583462352475547105037730945778584608279E+2067520692', '2.7235179981923310319738235894879139608807e+1033760346', 41, 5);
    T('5.0396617140180980112290131689304401510152E-9', '7.099057482524069554788222715721382261331093541531504796459e-5', 58, 5);
    T('8.295747080405131128480935E+54761', '9.10809918720977210707253937500924e+27380', 33, 6);
    T('1.652976883733583122E+7669564', '1.285681486112942019135229306830687883569187630472e+3834782', 49, 4);
    T('9.15482993151972E+4037', '9.568088e+2018', 7, 0);
    T('3.9495809205262E+30', '1.987355257754938992377160858509897774952049248e+15', 46, 2);
    T('3.908095326929485719118550903578424879636511190787119806290E+93849864136', '1.9768903173746098651102397889231284119168749816365290144722209101587590107964e+46924932068', 77, 4);
    T('4695312276972318913824783267.23915150099092256', '6.85223487409204247121078477321652642e+13', 36, 2);
    T('360065571.110993325', '1.89753938328297503876364212659e+4', 30, 2);
    T('3.020428028143E-52318', '1.73793786659448789397819131927279578513780780522093623215e-26159', 57, 6);
    T('7.473332056206242784465850133264347507502598072E+399544789951', '8.6448435822785381294005301901316419084244682800468991364630579854249684e+199772394975', 71, 3);
    T('2.6545058030759325273618511956767499403152E-43793685004131', '5.152189634588319893862857733009123865557155838508261990197856939965755e-21896842502066', 70, 4);
    T('4.04919516596767938142398731881702747229058975396741269844545638111172628708619095858703698307014719E+91581991', '6.363328661925045799106506614064522175879e+45790995', 41, 5);
    T('7.8938711836403788689485387722739902228108362611929174E+6904', '2.809603385469e+3452', 13, 6);
    T('7.03466750541446573370027044489E-26457987325', '8.387292474579902077507212611207840585e-13228993663', 38, 4);
    T('3.341598620037889427132828227624777069946213265099768868E+5141896', '1.8280039989118977381747692279217097e+2570948', 35, 5);
    T('1.7995217874523433061846222162668474214071163108613415E+779709', '4.242077070790137478716517628874194271194060827692514184105e+389854', 58, 2);
    T('8.64829156977511683564833506322917189811629558525289718715023664251559870029893040656186008376212E-54031389', '9.29961911573539491036147371e-27015695', 27, 5);
    T('8.52078995826261119108447319667254438902793796629435580E+62', '2.9190392183495259081103419481759400578499514075079378347e+31', 57, 3);
    T('7.987403679E+3581877', '8.937227578505540103845413636584692573241532357433996111863e+1790938', 58, 1);
    T('9.426638848735104E+5142', '3.070283187058663486241563846238e+2571', 31, 6);
    T('8.1446368524E+6946523287309106', '2.8538810158098743e+3473261643654553', 18, 2);
    T('4.032219417259540272204478866331908221389931400048743477485727321671916860580259161402773798107E+73458050', '2.008039e+36729025', 7, 6);
    T('5.6504128266382441683929602449473591619319549900002408802881702717481E-4881751023366453', '7.51692279236539994967372050974313334001215192497833104202e-2440875511683227', 57, 2);
    T('92629672232077598.90063863301233712326343840597041607655456994461', '3.0435123169140879772323396228229690669146412039176588151e+8', 56, 6);
    T('5E+1405777118244', '2.236067977499789696409173668731276235440618359611525724e+702888559122', 55, 6);
    T('3.575922461E+741', '5.97990172243658198399814740059437396277732066046e+370', 49, 4);
    T('7.984108816476710E-8561250542906', '2.82561653740855466419452851014456248628635025976235076e-4280625271453', 54, 0);
    T('9E-869473', '9.48683298050513799599668063329815560115867e-434737', 42, 5);
    T('7.88869662270039764413484643614524697066192137552471897225988700736686713038E-8195890013553929', '8.882e-4097945006776965', 4, 5);
    T('1.35342176496813904540880394246590582429142627731764911712414478149020053276032E+29783', '3.67889e+14891', 6, 0);
    T('7.179707176136375341401913486978796960524639825994932608331410951477613527254629E+913', '8.473e+456', 4, 6);
    T('3.67371987435196618057562397883355060133829216965154000472540684500E+313', '6.061122e+156', 8, 0);
    T('1.4792866032949044394019E-95483047434688', '1.2162592664785351757282744e-47741523717344', 27, 5);
    T('1.0457313716925544603251531895365106855371052455015295127376768840262251474879407584E-6141472', '1.0226100780319713142530417271134325435661232545102289895560582934036408017e-3070736', 74, 5);
    T('4.100849735154850424998568954883333023944579871020634818881424885849663612599045128003798737145657E-578845078136619', '6.40378773473547483807376041857055923862407e-289422539068310', 42, 0);
    T('2.675572405517448082E+5649924549', '5.1725935520949720972771088800926908173244154e+2824962274', 44, 2);
    T('4.712458167789150819E+57', '6.864734640020072049718705304013039739459327696999280834990366263684919488693e+28', 76, 3);
    T('8.9120799726952952463715474436836438839285693903355297048145204533165099122605783431631909389375E+4551057196565', '9.4403813337678765859e+2275528598282', 20, 2);
    T('1.9630579142679499665419988066E+9836986', '1.401091686602968166559525283171170204216e+4918493', 40, 1);
    T('3.327678484307667442355379899894139310335808838737692146122103503686495579534E+4922799812583', '5.76860337023413964786931e+2461399906291', 24, 4);
    T('2.43134939662854741919117399694122150170471210277624540E+6874064195453221', '4.930871521981228238e+3437032097726610', 19, 5);
    T('1.0496805442444426583754480311188446871883996972187399715782318606844535332564E-76', '1.0245391862903256753964503966393139032236946080277416684518e-38', 59, 1);
    T('2.9338987207395937897002373949538359655732613253567763605075447290049969107E-8004', '1.7128627267646388333852869565663674027391841238236919212997718917108311794e-4002', 74, 1);
    T('7.988712293440338047E-33535719398', '2.8264310169258223109845135255342978316223511737459003e-16767859699', 53, 1);
    T('4.76582490151389356813825520590783598202999337271191949072678494086332063317360130E+89536205', '6.903495420085316148538509004607763967209023254e+44768102', 47, 6);
    T('8.73584212085361806007883039348577463893244695975888298999502229344131381589611093355072253081E-406596476707782', '2.95564580436384800253055711281369395501536452408051186314e-203298238353891', 57, 5);
    T('4.42129752204E+33837', '6.649283812592150138142528590709054870664998836855393073088808237856384012e+16918', 73, 1);
    T('9.8711358404803585142715995165965562371006E-20', '3.1418363802846828888954674287282088166631670326029874978921587241662126193072e-10', 77, 0);
    T('8.1304290319388533208865555457549157885068286554112709492694227672649E+3292359080127', '9.0168891708498077340870422538771763991860996900736805340459998564777522117427e+1646179540063', 77, 0);
    T('8.0601949156183038822620916355105579058051056436917994500749067325934920E+81', '8.977858829152028877e+40', 19, 2);
    T('1.7654539859E+1293617057', '4.2017305790590619e+646808528', 17, 3);
    T('3.08300502616906098687255826013116108848672731202638827475459671548604743541183982705838E+4994746588441', '5.5524814508191389246383339131600308902340023710576826212458e+2497373294220', 59, 4);
    T('2.79799125754792741188580781151558042045245397938161747962330378605256347960E+8169770428', '1.67271971876579712062058760927597244127329496726295947385574838534557761275087e+4084885214', 78, 3);
    T('9.520479935171901322697389268713897072458655100066709408989906359E+84493818', '3.085527497069488331267423581241150396840258e+42246909', 43, 3);
    T('9.6977619149278219422781E+316214000386', '3e+158107000193', 1, 3);
    T('8.4520063395676629612501328633225132880746874537228627270266E+760', '2.90723345116412399903677294614733205e+380', 36, 1);
    T('9.5568058168056182185442353049791687180E-1419', '9.775891681481346189754834e-710', 26, 4);
    T('2.17593863693410473953572E-466141712640621', '4.6646957424189037236e-233070856320311', 20, 5);
    T('7.28335312439196245774226929758991441342086268907461367722377958885353595551884935516785591E+4210249487', '8.53425633807185478645377489395742278102449403e+2105124743', 46, 4);
    T('7.9309336869113277937283E+61899575942512', '2.8161913e+30949787971256', 8, 5);
    T('4.4147967707989123421155030572424236370263103192892052949087277363710E+523215', '6.6443937050711499376864490195641899272283176042527e+261607', 50, 6);
    T('1.5109782641945858394E+8925857055025', '3.887130387e+4462928527512', 10, 4);
    T('4.8019144653804829012013755178407300496433787057977746765392593580534900280159767925073E-675171631447', '6.9295847389150837205027607357674e-337585815724', 32, 0);
    T('2.81441790886617380509520155082877145938117037134E+5449802988028248', '1.67762269562204e+2724901494014124', 15, 3);
    T('1.9011083194510316937617450165824777298618984876923310846235512081728267868E+7230', '1.378806846317145211527751556938e+3615', 31, 4);
    T('5.2247425552952276174806603769135304185149515310446734318136860424E+259', '7.2282380669809344745123929711647933739752741869853493e+129', 53, 3);
    T('2.6137806849406565416963702733639313080984002961475E+1841673625605981', '5.112514728527103756119083043541744424478486522645230981812126003154941892e+920836812802990', 74, 5);
    T('9.93763323288983179604546E+25524', '3.15240118526970354837192462164291327592722e+12762', 42, 4);
    T('1.22191971639784089879E+265', '3.49559682514708909727859773952360298460003998789958980573672291458393004e+132', 72, 1);
    T('5.597E+568261661681569', '7.4813100457072356252346588698556564868296818103801416175589911024114791569995e+284130830840784', 77, 0);
    T('7.2713214641933444306829997615377506867292707248931E-7537149256', '2e-3768574628', 1, 3);
    T('8.422051691577715414082824473132488156131471218330734604226191089990683764449E+3177521524', '2.902077e+1588760762', 7, 1);
    T('4.79327125880129020E-385767184560', '2.18935e-192883592280', 6, 1);
    T('2.011190796104263304745319026570265454712539877236299939E+946993', '4.48463019222796485951602654336860567938893190030116242172e+473496', 57, 4);
    T('6.4847910760149283558212070695669048254074329962785677108576986411961086444961000E+58043130451638', '2.546525295e+29021565225819', 10, 6);
    T('2.285710298190E+36', '1.51185657328663290102911156905e+18', 30, 5);
    T('7.074611290376269E-2044443778', '2.6598141458335522206769057345931e-1022221889', 32, 6);
    T('4.5797056831596035941207627491245833008597417378183068825736455247163834770170E+947244', '2.1400246921846e+473622', 14, 0);
    T('5.65229017403016496862404612990739165210984813062645469906426138589591E+845432', '2.3774545577213805010498159234873880660721178e+422716', 44, 1);
    T('4.85360341581551E+670', '2.203089e+335', 7, 3);
    T('2.411424651853099211331932586857671419333629451402446313055708226355472325441681706664245120160E+316957360963', '4.910625878493595047710096578187419155296332340361665271615e+158478680481', 58, 0);
    T('4.607818769678757580622617058038482756337638794621523984131268073532516530956591238950305713306', '2.14658304513912472592069256949618e+0', 33, 4);
    T('7.59338313040692946359911961843531565497832081882557075265876961832563840683131278114866271E-64408', '2.755609393656315990095837385538e-32204', 31, 3);
    T('3696175227022267849032414092.016773404702311234937146358672775619', '6.0796177733655821078861178138872182814062298888138e+13', 51, 0);
    T('8.68998990809537336703600486E-49863061147', '9.322011536195057384357328326782263568235613559128e-24931530574', 49, 0);
    T('7.39767059718065993935568543488443748452082512031990E+860602814', '2.71986591529447641250156426872327014101855e+430301407', 43, 4);
    T('9.5039550809869757481802467684142816071629587849671095519782938514286705E-7404377293360565', '9.748823047418070634441154844604945179095866321330356660832934187282699223e-3702188646680283', 73, 0);
    T('3.52832233371119558076316851E+240486', '1.878382903912616955109258360633381976169159358491282086149580414136009e+120243', 70, 0);
    T('2.135367337758877761572808626061137813597705703296648884770002410767354094917580791680252657855994460E+44515836418845', '4.621003503308429392531555236351223553159256034625873954713601722906751e+22257918209422', 71, 1);
    T('6.55574507268594587077340408367234333953545565465682867336894127238643198993462187195128E+5394741655273791', '8.09675556793333478408703745283365e+2697370827636895', 33, 2);
    T('4.08879109740221741578687735602952948096051E-56024460344100', '2.02207593759537562422226807353364843462858518254559656e-28012230172050', 54, 1);
    T('2.88702046383025278824610014957155384890357084971884613239765424667693727485508799293067464689E+909026279171533', '5.3731000956898735111861968797250897647473152496311640907459600751218763861e+454513139585766', 74, 1);
    T('5.85707760360226795537203615034919E-6421677', '7.6531546e-3210839', 8, 3);
    T('2.6569033546401399505222840937237E-1698917059747', '5.1545158401e-849458529874', 11, 3);
    T('1.8572130578086568903670432137573555816173399866229650898073903609534634685E+1640733394', '1.362796e+820366697', 7, 3);
    T('3.2509100368912099980074744E+784', '1.803028018886897386311411e+392', 27, 5);
    T('1.020203979681842516145028680059578232774062730558684033505119017106807172711E+119708327', '3.194063e+59854163', 7, 4);
    T('1.816272572881346033232615148233362658915452774043972856579675E+1533241883139905', '4.261774950512222945050604e+766620941569952', 25, 0);
    T('4.222768721702916271457625051478794776571420E-13381259538681', '6.4982834054101674517e-6690629769341', 21, 3);
    T('2.1519453588511791178106435314081634243235207843258897805541E+7368828404115', '4.63890650784339381668800704976722120910871991068098e+3684414202057', 51, 2);
    T('8.498297359084546458280003048168914699436364769023695844300652045190667728E+61518', '2.9151839322904732383444730694225e+30759', 32, 3);
    T('7.65370328642798200007936195028468256694001674934111932023715848409184276391931286002453021414093E+911558956', '2.766532719204307910633754595392000551442511483883250429903676087804405565590643e+455779478', 79, 2);
    T('5.1238280508743809055025574209743282729559196766140070189951812797362009162E+66585163287', '7.15809196006477e+33292581643', 15, 0);
    T('1.490875882460634284411863820052569490029137331918395153873776527023749083262931386830E+813442463', '3.86118619398318356259490388719113598185201634813542416e+406721231', 54, 4);
    T('9.389089060631627013431474273256976055318787324073601735849070341827498807062287216165E+80136724383', '9.689731193707917826178936968367758565636403e+40068362191', 44, 3);
    T('5.666918381029663696953608649066197933231308460645707103242820405560172691798118435135524461103430183E+397363194304852', '2.380529012851904667362202911642711024107714e+198681597152426', 45, 5);
    T('6.489572912185759966860293134205566227925787911969794771866E+65064', '2.54746401587652656029608220458044130513888776543718542834122918527624080395488e+32532', 78, 2);
    T('4.79831452540234321937856594164763711E+214800463274533', '6.92698673696026591387308445e+107400231637266', 27, 4);
    T('2.87850545774362267882888183908800107382947760934094690201972229957797333658228601684365936E-7375351718705896', '2e-3687675859352948', 1, 6);
    T('2.6180115082520583359185144823153085187283124373602323397858607127478347167082796349429269911729E+101608973332', '1.618027041879108501297810734567701e+50804486666', 34, 2);
    T('2.15754923662172353660E-72', '1.468859842402168586360692550046800918056478722e-36', 46, 6);
    T('7.21167370160847769931713219E+7858200300659', '8.49215738290834886674655744364e+3929100150329', 30, 3);
    T('9.49103445710705218035789748393813719855362668161846E-72768611120', '3.080752255068078689981071323748588537107e-36384305560', 40, 0);
    T('1.3780183157276814234744165885721310974270604540920153529405097095176115665588872141E+65129420416642', '1.173890248587e+32564710208321', 13, 4);
    T('1.1272670886833805240621800360101550017149869005946946591265232560766193937035542597E-437866766961351', '3.3574798416124266047747707108073142426282137e-218933383480676', 44, 0);
    T('5.01513385E+6920702534', '2.23944945243244996980913928295535257356722976e+3460351267', 45, 0);
    T('8.1260108434782378341695197333453540906E-91372053', '9.0144388862969379134e-45686027', 20, 0);
    T('9.092830750792350171931235260936892651832323052358883721226795146110156733755114783658E+200173887092', '3.0154321001794005197296683806041624213563546270536367710423e+100086943546', 59, 6);
    T('5.9716669548188311704969626601027296830809246015644883490860409903708929680E+7946962', '2.44369944036062486685492557249769572445990273975135928852505135432044491079041e+3973481', 78, 3);
    T('9.9533859183595836960393341304494815647224E+9804010363055', '9.9766657347831312895713804962380972879872133625410810249905126198373397e+4902005181527', 71, 2);
    T('2.401659339220326238148181906366427704224780832955898700E+52492', '1.55e+26246', 4, 6);
    T('5.4899100421598422967909002186123690839377971716E-14', '2.34305570615805937922628235012415265e-7', 37, 0);
    T('9E+1993181371', '9.486832980505137995996680633298155601159e+996590685', 40, 0);
    T('5.05028103859772651487706403994190016339078351595454246848636742496802495048742202447079630739E+94727864856644', '2.2472831e+47363932428322', 8, 0);
    T('9.59981293146338228219992128891802378333E+94959207228', '3.09835648876358033e+47479603614', 18, 5);
    T('7.3125650335738419022159017437377735662828E+32057', '8.55135371363729866747197e+16028', 24, 3);
    T('56873071800579577894480029.9913916338494896187006254131382792065617290802105695797397', '7.541423725038898641083750859464767137139684e+12', 43, 5);
    T('462437348616560861444716.469443110926995223677083942242611169677743650813236751337419207369559128520', '6.8002746166354257e+11', 17, 4);
    T('6.5803421671668747304683336293E-489168512768714', '2.5652177e-244584256384357', 8, 3);
    T('5.00835833723732739992314400670207943302403206440405252937804763314822035527687053E+82174742', '2.23793617809742898841363911292888853966977964817766258759532718005259729e+41087371', 72, 3);
    T('2.739550602341585775997338219789670446820E-7085173', '5.23407164867045527149076121386206081046e-3542587', 39, 0);
    T('9.450786377428306361560309171177304320240134953027243301587893245054872736802088043071493639E+193412653006', '3.0742131314253906754227821858e+96706326503', 29, 1);
    T('93172790036170.613444694347666418780549753491881062334620953897827553473780799108203', '9.6526054e+6', 8, 0);
    T('7.6427381608928405257681569294347818369102356176996211973E-863824318', '2.764550263766756063892234262466594517907118311533593420860995828119465404401809e-431912159', 79, 3);
    T('2.2336825110261464289548790065115800838432265E-4773', '4.726185048245726940640138085401458332003103e-2387', 44, 0);
    T('4.2724906128719667659691356003183719988200264E+55328', '2.06700039e+27664', 10, 4);
    T('9.23827959495961249186367387478693953484941E+212291748', '3.0394538316874649464598276081160724e+106145874', 37, 4);
    T('9.041300268911201088838782013152679784074371858101273E+41572', '3.006875499403192963673188892347469682563777463469587792654e+20786', 58, 1);
    T('8.174408057E-61767957283488', '2.85909217357538160219868784409618103905259254542127e-30883978641744', 51, 1);
    T('1.10624157550602430015244848049439711918773950918589248425841376113701453944821290918195E+7630343938669', '3.3260210094135369128622169846679339403575787017793503761274203309290260436e+3815171969334', 74, 3);
    T('4.9371324802073E+78010286', '2.2219659043755149394630379e+39005143', 26, 5);
    T('9.10859E+727317301', '9.5438933355313647910084093295703513e+363658650', 35, 3);
    T('9.04E+945245740', '3.00666e+472622870', 7, 2);
    T('8.053E+7598672850665742', '2.837780823108084927845098354231270362354547981102e+3799336425332871', 49, 6);
    T('6.922073064989180646258163061425595248339645318548644890836321470068254933488292945284735322631E+36948', '2.63098328861837901379246055794913493258563428302007e+18474', 51, 6);
    T('7.64297999231708049848271603772247952906531812E+3313526944', '2.764594001353016120387309e+1656763472', 25, 0);
    T('2.49359370192553324844573464720642409E+42327505', '4.99358959259322115994443893492663456542067e+21163752', 42, 3);
    T('4.11235850876675305992625132144861230486080916955691E-66', '2.027895093136415935e-33', 19, 3);
    T('8.729084711960792819353415468437350816065576927664405E+6050276', '2.9545024474453888695924718967207697287659510106764672439630841479565241968e+3025138', 74, 5);
    T('2.782271147832686219E+295', '5.274723829579e+147', 13, 4);
    T('8.08740020331005190220391258674305873512363906032235E+10092380254774', '2.8438354740227241040111232893777368e+5046190127387', 35, 4);
    T('2.21111576728942057265374867434598092629448251165002373458069847941389903130518761599813923261736272E+1571', '4.7022502775686244544842e+785', 23, 0);
    T('3.6786635336023303507152224116804583704219412283495446611E+7652222', '1.9179842370578362891185422e+3826111', 26, 3);
    T('1.6747098452733561507694771361540079238390082400869150000987128223006851969007181E+66024', '1.294105809149064963260379265e+33012', 28, 6);
    T('6.8958664511E+678680', '2.6259981818539021981659e+339340', 23, 2);
    T('2.72281587055538841173686928082096620997022877836901707243689996E+920696963716', '1.650095715573914339995571593605530228843869509809576660267100705710199e+460348481858', 71, 1);
    T('6.7400E+55012369528', '2.5961509971494339104070358328276508155305955410565531e+27506184764', 53, 0);
    T('4876.49277516542655231337713128136167574490306845440883621473529483335813066', '6.98318893856197088096157423785895630275546e+1', 42, 6);
    T('8.1417510446134179304795008671281142077622655031292884E+945', '9.0231652121710693691311381001357983609378069176681239052663552490593273e+472', 71, 0);
    T('1.30650191820664E+1811', '3.614556567833e+905', 13, 3);
    T('835362.39168127574079882209155115811445457403061495355495204551760538374', '9.1398161452037739597682402466114e+2', 32, 6);
    T('2.1962027843207439856796146949959914521607703076789660614150E+530', '1.48195910345e+265', 12, 3);
    T('2.419900615649606588649129133146016064773740775801480E+29108', '1.5556e+14554', 6, 6);
    T('9.1729727224358697241613267368499559222661984280657781520944350143634252209378072E+296991188051', '9.577563741597269956621661494375676941761541216043791043763111011243284670677e+148495594025', 76, 0);
    T('3.424807968122417190265786224803414598744313823843255233059744E+1739781203248', '1.8506236700427283766082924116700167702e+869890601624', 38, 3);
    T('9.655628829651672320852209236659488021285268281974300650419875326E+644188963599', '9.826305933387008239476331241821733277265221856e+322094481799', 46, 6);
    T('6.88815366941467416777000064485475542793749913477244591699232199964730E+414157302019374', '2.624529228150198019388839742009995469e+207078651009687', 37, 0);
    T('2.5829424561385859658292788297683281827868597224516601440089262848450E+889761496', '1.6072e+444880748', 5, 5);
    T('1.71704201338064129520393922040426764484532952194442522349724910206873998394075971283951735644E+71945098331669', '4.14372056656893949780344576110128553784727806963084424e+35972549165834', 54, 2);
    T('2.553408490E+87080195', '5.0531262501544526519574040306536300193230181695512694054594791474960429e+43540097', 71, 1);
    T('6124701461.30253396696741160592218892208', '7.82604e+4', 6, 3);
    T('5.0169654666967765488407729714423591421E+94', '2.2398583586237716716486413363400318012837209715720882626185e+47', 59, 6);
    T('9.7618965965442852147114138973091944182243234357713569889829E+1288707', '9.880231068423594927351688244474459194937005e+644353', 43, 0);
    T('2.463032205471434562E+199', '4.962894523835293602541700098665678245210425304527057712473799410572054423102e+99', 76, 1);
    T('9.48405028900394137180787430531224638858239194922517032274376483841159408163465435135E+66946604', '3.079618529786431417491547e+33473302', 25, 2);
    T('6.507928171301145077839025428E-72', '2.55106412528206453900085028710092456169084330858943744594958242411029891729e-36', 75, 3);
    T('2.558836913254730207106858879274826091287738890342230033749159837590085448680594119987E+8288633532798', '1.599636494099434519120495776736793381468862256137471e+4144316766399', 52, 0);
    T('4.2961855225494E-25769386486', '2.07272418e-12884693243', 9, 6);
    T('7.5555769229321270082798708075E-3722043711898897', '8.69228216461714213735790110145447192421003957520348071e-1861021855949449', 54, 2);
    T('6.805632039816747163674E+5731', '8.24962547017544740347522820382134272105255583449e+2865', 48, 2);
    T('5.292319408658357715078502155762867585E-296443981167', '7.27483292499446870635760964657613202175753147622053e-148221990584', 51, 5);
    T('2.4088051450080E+8173', '4.90795797150709105376962677572069305380908614111297365553e+4086', 57, 1);
    T('5.399112747466886993138311223824932339056153376414279E+5591552693056637', '7.34786550466656744803006128046e+2795776346528318', 30, 3);
    T('9.4213631185705702766011583868449956283599883063266590478542230153211906005516607505881153E+18527703', '9.70637064951188141912617586468613169489040932e+9263851', 45, 4);
    T('9.8662795800192214301268467968737763973E-1820', '3.14106344730876479475193343625504e-910', 33, 2);
    T('4.742201122283735467286752580639151338402147978758E+2869', '6.8863641511930920159e+1434', 20, 5);
    T('8.3789538901926503620943368526446290753065226079800268864837819950868226858046E+187347605', '9.1536625949357836061871605822069283444244503528403697e+93673802', 53, 5);
    T('4.2034018272497E+83761770', '2.050219946066689694568885168e+41880885', 28, 5);
    T('6.69760776776444294842577371961987758394584550633726992061849437263890087941E+6923870', '2.58797367988247572680336566731031833e+3461935', 36, 0);
    T('3.670573218788407507689514263121673716122995116133303311009468985401267408779E-680021113', '6.0585255787e-340010557', 11, 6);
    T('4.135448217987841390586067157756058403489565598382242753255526546540540095751E+6960', '2.033581e+3480', 7, 2);
    T('6.038769448594689087910861123468200183610626181299843431865263126830407743069E+77235886', '2.4573907806034206508145897417338517767341675713702823338890812489722423677833e+38617943', 77, 5);
    T('3.587825847880514585709358773677324854320097535772497696497628452E+609237590588', '1.8941557084570725928674400770347757019608538011645519465e+304618795294', 56, 3);
    T('5.4201544097990303858141E+627674', '2.32812250747228299412e+313837', 21, 5);
    T('7.40711009024513815760164703475393233970877193508242149988958928091491914E+309', '8.60645693084276008079123831702972740385e+154', 39, 4);
    T('515584639404838941246901894159595954745567752911318738407100564309750626516.1791617', '2.2706488927283296329062209956647601583635908763632678803367026671603381e+37', 71, 1);
    T('1.1552775585057706466940388889077953341599272409994467228416821444233007E-3794949', '3.39893741999727121585845223399480273146968e-1897475', 43, 4);
    T('3.04E+333760813', '5.5136e+166880406', 5, 5);
    T('9.181258489302838079121284106371E-9689946237756', '3.030059156073167836053285715808361515527841237119577e-4844973118878', 52, 3);
    T('9.097530205431698341141736543467287960383290070775938518339807290620844422120352E+825114268', '3.01621123e+412557134', 9, 1);
    T('7.66901510445223617671270401903922314736658973717857677577742198978502434481269112716631157122727993E+28348', '2.7692987e+14174', 8, 2);
    T('6.58755948903762446750976327895628343674985502155875440613151906714494399548343042750830709062E+409943024309', '8.1163781879836183909409375363404068136137574908934e+204971512154', 50, 2);
    T('5.0951226947283250806636033744756427856016086366224E+93962', '2.2572378462909762847780411032884150727038656700793e+46981', 50, 2);
    T('8.150670201928153993411272129594404183375704672E+453404805472187', '9.0281062255204740105304768996726311145672013275416789795275497995048942097e+226702402736093', 74, 3);
    T('7.0104619676474369342365609690167930592871571199219650021680E+590007778221512', '2.64772769892363307093059365640630051052232478304226906666646516317583776e+295003889110756', 72, 2);
    T('98234.08585858166222466900865352520714306', '3.13423173774023388222090399070014419184070085892e+2', 48, 4);
    T('5.7570159774745196978712550411544E-289', '7.5875e-145', 6, 4);
    T('8.13570542097616366291558237185955227709177843801016027464366840371504045382881466535960323E+7175181725292135', '9.019814533002418483218041677971520150196e+3587590862646067', 40, 5);
    T('1.8328561374200688659993455967130023996868394E-635028', '1.35383017303503354367849439544524371635138052709984972e-317514', 54, 6);
    T('8.791350103263475495836258678650727954866E+4395', '9.37621997569568303472309542853412757889934365090285260995e+2197', 57, 3);
    T('9.30527459538166136476938267777527342524470683073510498487415309409144843033563741336552377142559E-24040358572', '3.0504548177905637298e-12020179286', 20, 5);
    T('9.5255828385096767511E-24', '3.086354295687660799750652512900547113767349372521616998883899383188989461278e-12', 76, 4);
    T('3.61440703564099768E+5758825', '6.01199e+2879412', 6, 1);
    T('4.1558989809468218098954125905590E+6184683', '6.4466262346647814710667613602892424745155045175684165271079e+3092341', 59, 3);
    T('5.46587726793634459876766095307E+2013632', '2.33792157009946435002902338510510938904819610116259987000091846057423155518e+1006816', 75, 6);
    T('5.66463800E+83', '7.52637894342292600459229452291474918313e+41', 39, 5);
    T('6.56731294321225271290059931898686901695490329384659866588358271112311008476E-1052', '2.5626769096419963476340903043996166294265167882558946536e-526', 56, 5);
    T('3.90E+78140', '1.97484176581314990174384610437230990660852618680820080988126186147286133697613e+39070', 78, 5);
    T('2.115293184811715756111035E+954073670396', '1.45440475274653711374621434742103752e+477036835198', 36, 5);
    T('3.843011439827436064752567943485332632909203518838336622636143406313606434503297E-7', '6.1992027227922108538280901695224715931051043157545802926356061644672662141e-4', 74, 0);
    T('3.46496820172893471274450231665851340050925461712723830282019910966406769659794792E+50088842263264', '1.861442505620019066170426433408338273749989194997433001e+25044421131632', 55, 2);
    T('8.815334024295000171588787248553700566011567E+4275716516866980', '2.96906281918975236955954438611698479173299363074681248825253049431984948e+2137858258433490', 72, 3);
    T('2.678099075228103912053736336553650966855863923E-39013929', '5.175035338263985141e-19506965', 19, 1);
    T('3.23269E+888287264343913', '5.6856749819172745252775650565521e+444143632171956', 32, 4);
    T('6.234249730E+23571077139', '7.8957265213531807075664078327696484000996048294363154908871e+11785538569', 59, 0);
    T('3.3429040814931426E-786', '1.8283610369653862758941525973088006356e-393', 38, 2);
    T('4E+5519', '6.324555320336758664e+2759', 21, 5);
    T('6.8777973506862387855796638472705729011046488677364623807540946604353320E-84606258686', '2.622555500020207157785460692451466974946442382680558e-42303129343', 52, 3);
    T('3.6864658410597401179937293588455580384542595E+93778172720', '1.9200171460327483539026314778967671885846184667896514124254004538864392e+46889086360', 71, 5);
    T('1.7686814349E+4275992', '1.32991783013086940115047303330105611393593893453e+2137996', 48, 5);
    T('7.23329170140551722658511E+3908310149', '8.504876072821706451854804787270754918424940970464942356230569216208409282293e+1954155074', 76, 0);
    T('8.080984534788025628E-4290277537595', '8.989429645304548256737997439398050197279956353931606298464860617251463e-2145138768798', 70, 5);
    T('4.168731982E+175976', '2e+87988', 2, 5);
    T('2.71070985594230989175938821603202E-57033', '5.2064477870639497e-28517', 17, 1);
    T('5.5985767004843265514E+42', '2.3661311672188265546717501437168828657231435e+21', 44, 0);
    T('5.2331753650276045585717442902492648981393074557533991154034107110886396028171E+2978161930704868', '2.28761346495154302294e+1489080965352434', 21, 2);
    T('9.0403617088038070744647273955803000953854077509153313322758256E-29952', '3.00671942635221736687922511896674815104e-14976', 39, 1);
    T('5.222488573277200227050900075314803902522307294599971E+68585294440873', '7.2266787484135478395239424677393393e+34292647220436', 35, 5);
    T('7.06991331155019503904475228560073103E+82', '2.65893085873818746784799462956040824680806779629363141352213215464903828502e+41', 75, 2);
    T('7.888073352767506531631057273649734668539553289948510498240822582019489548040318306E+4190799574972373', '8.881482619e+2095399787486186', 10, 1);
    T('9610.523194266382981096232960460004238837580104560636452314738102557965139756002214949', '9.80332759539657e+1', 15, 4);
    T('8.2500604348286709226838364974088837798915132923152211474564761463852274150514234984081825E+8699226944877', '9.082984330509808229878255253074802885120895971e+4349613472438', 46, 0);
    T('5.3138740657125134679225950980E-7610664', '2.3051841717555917375661e-3805332', 23, 5);
    T('25169773910890495218058815869093616.163248025716892103133483661936354157144385347', '1.58649846866898977e+17', 18, 6);
    T('3.35801087705824464519194153474023E+978378', '1.832487619892217619778605787594363341371e+489189', 41, 6);
    T('5.104836387172125488331409435652961484465E+917', '7.144813774460553067950542305354350723953387943845856625532408544114307e+458', 70, 2);
    T('6.30858003121673652127772516117213759E-995923301483', '7.942657005824144563894461998710252939098532921518093894e-497961650742', 55, 1);
    T('8.73776426890211047125E+94338383', '9.3476e+47169191', 7, 1);
    T('5.7042720564145905883469300763192190047506851519303947207607129371848518E+62216925425', '7.55266314382853340582138241132398207433924764948629378926e+31108462712', 57, 2);
    T('7.2407209816E+862455334216374', '2.69085879e+431227667108187', 9, 2);
    T('4.2612404528488231554025594085624274104481461847806425081562024705235828953860312763137966173365E-58811596270', '2.0642772228673219450585770335507374241863342224249654123935e-29405798135', 59, 3);
    T('9.9213447170784751525938843109811158287247500224604830350E+7638705', '1e+3819353', 2, 6);
    T('308.34252022227193055388513127395377509307860', '1.755968451374545426385501454636681471243911191069083644e+1', 55, 3);
    T('4.3910095420614774291812791033804135167250621896899134722702633445047E-17164', '2.09547358419558165261581911703556902768854e-8582', 42, 1);
    T('6.772770935968462116710055712328007209215555377820308084587401019978E-437245631', '8.2296846453e-218622816', 11, 0);
    T('4.4E+110203958850', '2.097617696340304e+55101979425', 16, 2);
    T('8.0930478686414771558886481808781238227281596995619937281696054372920869490E+85308527846873', '8.99613687e+42654263923436', 9, 3);
    T('4E+8629', '6.3245553203367586639977870888654370674391102786504337e+4314', 53, 6);
    T('5.3262804078365E-526992518484', '2.3078735684253806e-263496259242', 17, 1);
    T('5.8694140324209796543898268793071E+16040276005542', '2.43e+8020138002771', 3, 2);
    T('2.95012650429596039320302E+9286311622327', '5.4315067009955537688132717109049769508e+4643155811163', 38, 2);
    T('3.51577471997435435624502027044516745365522505248752E-323', '5.9293968664395827875975547558583370423079177721328276156016e-162', 59, 1);
    T('4.16757409649143743705105141489565020041625369954878516449149E+97221266982', '2.041463714223556175586541e+48610633491', 25, 3);
    T('5.795113927665900834432796538330915366217369700887575968254663E-1980', '2.4073042864718828133657306895147027424738187e-990', 44, 0);
    T('1.07053468884523103174E-15263054417', '3.27190264042992427140423180820220561757361865e-7631527209', 45, 6);
    T('2.2852764382672123769096451383928680514464883377349812210272456999575612855634701709512263E-28', '1.51171308066948087016004402862037097687304939e-14', 46, 0);
    T('4.586646164E-6630638', '2.1416456672381638397663281582956081889959478688e-3315319', 47, 4);
    T('4.9195174175016861076236E+782384', '2.2179985161180081609237817168033188e+391192', 35, 6);
    T('8.793727684045753308196554055319862469393984303857541746475471122617836439858759854820176776E+217707', '9.377487768078267831122561198728e+108853', 31, 5);
    T('1.035627068030540419382180862764468125195271479305436062633551664347056626642676297133337526E+87474077562', '1.017657637e+43737038781', 10, 3);
    T('9.50415693769131761271186862152220309506E-4008879303263245', '9.7489265756242e-2004439651631623', 14, 1);
    T('4.633906649087268856260649992486827743013421366685944E+5457545757', '6.80728040342637043061032755438629176784277169791907590543e+2728772878', 57, 6);
    T('4.97327E+3591932867499', '7.0521e+1795966433749', 5, 3);
    T('6.11816338755730742378475407503253592809490421159E+47839', '7.82186895029398164090700570642614663276846e+23919', 42, 5);
    T('1.2074988755366E-846138', '1.098862537143113274130810863546419811229353857593222484e-423069', 55, 5);
    T('2.163465405878903470789198075458915192820924562974E-4735085', '4.65130671e-2367543', 9, 2);
    T('6.541972006540813662175697460062E+321331138319805', '8.0882457965499624535306664102970177335381690778145e+160665569159902', 50, 6);
    T('9.943814302115857323371508737125867376602944882212090476632441317979571299247684858717E-82319558462868', '3.153381408e-41159779231434', 10, 4);
    T('9.62882754539493968866765046938225096891006766246100148907306915419613596325E-534549724520564', '3.103e-267274862260282', 4, 6);
    T('5.066371819763791810199026206024937471915544989903273088648440237E+4711682369432194', '2.2508602399446731997442820241914267758450924009e+2355841184716097', 48, 4);
    T('5.25381187370000305921239686275718492817355575820282938524907888152296875913490983688873780211E+549522342', '2.2921195155794130315837124665788e+274761171', 32, 5);
    T('3.623143915169365868634062328002521105179770898602247025108879837850E-3475', '6.019255697e-1738', 10, 3);
    T('8.3524532909E-531953462316', '2.89006112234672296385428888973097944744e-265976731158', 39, 5);
    T('7.3693376673921660314955506885279996270E+2183264', '2.71465240268292287150603067156550769840786116887e+1091632', 48, 2);
    T('9.51485737826579210080E+49408', '3.08461624489429674999796e+24704', 25, 3);
    T('3.96089520149380929537194634173844781385969321585287886772366509500669720327224020738219137410238E+5532', '1.99019978934121317411503351122825177801316399693713070167624844746779943e+2766', 73, 6);
    T('7.17926915007464975790128888271731E+155', '8.473056797e+77', 10, 1);
    T('6.8421755297337603154670960812765768689729959217803509407833020048548475715203641E+9865155565', '8.27174439265005354255441257864839419257426574718e+4932577782', 48, 4);
    T('2.314746467213045470276543735706547E+975325571906', '1.521429087145715598920750323454843356485134e+487662785953', 43, 4);
    T('9.997335845060625506993973840944024805811620759996431756604024517304408904010053362496689907879670E+3549', '9.99866783379697312914548204698122342618195571862e+1774', 48, 6);
    T('1.5029156055502376090418415580750290578523141417244455238684158790691E+33835', '3.876745549491528866288357858169500951e+16917', 37, 1);
    T('5.91916499847047329631E-18', '2.4329334143108958698804207713699673978466363806596792671130443610397375e-9', 71, 2);
    T('2.83892017577234529071040578190059892314786470535459773247507172347712639970583E+3720751327', '5.328151814440299232707842134758932256500678774620529531357e+1860375663', 59, 1);
    T('1.48834474437364106128257094815778199983293508E-1454', '1.219977354041312137864841159757633647490923980014780536410343577322506538656587e-727', 79, 3);
    T('7.237667730573588828023177406258054E+2636548739', '8.50744834281913e+1318274369', 15, 5);
    T('7.99567173385280773117025778933416929868E+7487060', '2.82766188464123265502118966771570438226831e+3743530', 43, 4);
    T('7.66900090011941293253203609576189368015014985608984151571747705526153764571545020829380831454700262E+12124984110104', '2.769296101921824634216367504197369671841182e+6062492055052', 43, 6);
    T('8.6171196867292334035946871047529053549590113E-410603543253362', '2.935493090901294116618471757346537304735961121594506034833136951835006923277e-205301771626681', 76, 4);
    T('9.8386973163112723042229304944322499331446875E-31583433516', '3.1366697812028719042869775211576894655614e-15791716758', 41, 1);
    T('2.724578353470529035270517261809657617693972954267954894317587587618508532907071685941088E+9264971', '5.2197493747023228464505119012875259611457463795479460285722274529211704e+4632485', 71, 0);
    T('5.833515502077279375571390725019015E-31', '7.6377454147655899565394822199e-16', 29, 2);
    T('7.1430893893512730596865104461938863863432551864825169374E+353798288', '2.67265586811158182443450664682613626e+176899144', 36, 6);
    T('5.625539951711E+57378961654007', '7.500359959169293096121611077973435098182677120140956179022e+28689480827003', 58, 6);
    T('9.20917538019008169201264948805727724073981E+2515', '9.59644485223047675906517965910275954e+1257', 36, 1);
    T('9.351813670274579183741822947499572031832926312574254128859890022587883348839578811665707885704137E+8064', '3.058073522e+4032', 10, 1);
    T('4.402536035506858552E-10728661303211', '6.6351e-5364330651606', 5, 1);
    T('2.145605223848606978761891468884155712062788407E+62254', '1.464788457030095174951595672401174772665529469725e+31127', 49, 4);
    T('4.451899510930971422621343604847897725534426357969446008759092115304957659577032526276282349112872959E+66662', '2.109952490207059478049780632121638468297623909800491525353e+33331', 58, 0);
    T('9.1368885167215E-2713222897', '9.5587072958227463447149674674e-1356611449', 29, 6);
    T('1.704720289634195085234835724594248719978396316215851069249736655530357208861864390006E+14999448715', '4.12882585e+7499724357', 9, 4);
    T('4.449552983107541596369779659708466936149651657061334537706521898E+6743693009519', '6.670496970322032607384131050498479385794969634351670680648474472886600966385e+3371846504759', 77, 5);
    T('9.5739589485506682559657017802E+4232402900', '3.094181466648436289797301784735386566585834503e+2116201450', 46, 3);
    T('9.919204871550576967820307660920464956916653355794926701381957474614224E-5466380595430942', '3.14947692030765276535598e-2733190297715471', 24, 0);
    T('6.9865644906E+6809', '8.35856715627744418255907021904198536515784808893363e+3404', 51, 5);
    T('5.33988525182733926516797587129765762853814449799322E-212', '2.310819173329522827630483438387168931706062722309e-106', 49, 1);
    T('2.953180916875923483269349159962149943151658586330868051271137636349E+824815783', '5.434317727991181307746073e+412407891', 25, 4);
    T('6.851102859281171216324208550220368249526783E-9630518737937', '8.27713891346591525617089812373618241781329504769102162819900824889681499463e-4815259368969', 75, 4);
    T('2.3381035649979007867346779545079780000911288962003083882405E+47368', '1.5290858592629456354372e+23684', 23, 2);
    T('6.868779663101088184893864867735987601500009314139798917867081393745254E+8353998064808548', '2.6208356802938043382e+4176999032404274', 20, 6);
    T('9.34640156881218916970641626792719158166', '3.057188507241937995e+0', 19, 3);
    T('899763.4360347182724262967464999314796276759035533311041773940250', '9.485586096993259975447863e+2', 25, 4);
    T('3.168472066458713428196659046168517478204163543128368582937362736793455E+3187003331271396', '1.7800202432721694832e+1593501665635698', 20, 0);
    T('1.446082583440684918669772285920262472185622450383688383403538452292003554160977682452812826302581E+57658926722', '1.2025317390575123271929143726912964391112686771e+28829463361', 47, 6);
    T('3.724628424818522E+10709800035', '6.102973394025671155989914265694580339908766e+5354900017', 43, 5);
    T('6.31791421158214073310149944E-946956', '2.5135461427199105097198626039774e-473478', 32, 2);
    T('5.6468517986317469594856794473373074368301256491623389902427491029686842933271983324928260585421314E-23653274413765', '7.51455374498828570290947178850849706781759670400092591e-11826637206883', 54, 5);
    T('2.68895059907043198510892001811497074538398196666584667999490583E+3986371750355', '5.185509e+1993185875177', 7, 1);
    T('5.969633504110821161498358978542210846168847763303505962232954115554408385250590268497657670066618E-765590081', '7.72634033945620412669317359074532914391615689250076467671440953534942092e-382795041', 72, 1);
    T('4.686993221126E+3134465959696', '2.16494647073e+1567232979848', 13, 4);
    T('1.6051585906149280415997935575562505882666885422439211082457049832189982082519661E-167444994616535', '4.006443049e-83722497308268', 10, 1);
    T('6.18754439658451362135046864820384126468364018749079536272805555743086222356857E-5263', '7.8661e-2632', 5, 0);
    T('3.920858334386412432213050501E-8307654957', '6.3e-4153827479', 2, 5);
    T('6.7428513965961170206578929575305476825938475016824771216329132311623210986161560889E+101', '8.2114867086272002445154265132243617839e+50', 38, 4);
    T('6.26519162948443473825170380126443463289281715413516772909412156558902330770940167583573E+6365055452422', '2.50303648185247886095931487801634316747054026445160426353969317281120798392075e+3182527726211', 78, 2);
    T('8.063881999735791466273131659237922654675617570758258899993E+6805233789910', '2.839697519056526461554783718917286e+3402616894955', 34, 4);
    T('417972085862624033129784.0440977487523758541927239688604665974641811969877968444281244896740166126', '6.4650760696423675810554309117893125934033690069687851402364271294144e+11', 70, 3);
    T('919197811500647683634294931755783253513276324777619993148601396.0', '3.03182752065589589277043535369134489937339052563425042382863354282860369883793e+31', 78, 5);
    T('1.306647389528986E+770', '1.1430867812764637162414914732540958689405918149841837366e+385', 56, 6);
    T('4.368386254265937348923635874E+42', '2.090068480759885855e+21', 19, 1);
    T('3.1730623196493625425801400908445813302874255E-874520', '1.781309158919181704935479000146836765600329665923032145584e-437260', 58, 4);
    T('8.88441749295247920049683950754026536650733067268538691784350229263601822032700272907201020305574794E+8776', '2.9806739997779829663437779416e+4388', 29, 2);
    T('6821413.2154197911911541987923166909957', '2.6117835315009916018769792539834782e+3', 35, 3);
    T('9.728471009849857326385706597090933512816823956643772361312077303723822418259050E+41467', '9.86330117650771998689292007815187434119441291040736040987e+20733', 57, 3);
    T('81381.491747143787281391731325860095', '2.85274414813427997471456771190556e+2', 33, 6);
    T('3.293548952240464475E+7609685', '5.738944983392e+3804842', 13, 4);
    T('7.4210971551438625984190962199477803914449375722660396872872016610E+25227869', '8.614579011851863326328790872630017826050859908890424032200833330195828667888e+12613934', 77, 2);
    T('2.069659570800420074875414774904932752119458365265347166367263459383263134454E+148046232', '1.438631144804e+74023116', 13, 3);
    T('7.7620633657154028805254639377941583185673698479140377075693404587456687216800890460485029E+485145680', '2.78604798338352436936086556193036476564711031338282555e+242572840', 55, 2);
    T('7.394391198645525964109690768806123364052998473881322933384219709335077586538672325348569101742E-57029883755506', '2.719262988135852586408835209647651e-28514941877753', 34, 6);
    T('8.071625E-11914528', '2.841060541417588306300302103706780015139339e-5957264', 44, 4);
    T('7.3958386328387027849293504E-480546314830822', '2.719529e-240273157415411', 7, 3);
    T('5.172365135131935286824664032147153165149274760100535092056743006559077883110048257225367198E+3759520163874694', '2.2742834333327794978740281397e+1879760081937347', 29, 0);
    T('9.827743357683932920733E+96811', '9.913497545106839295248879055680984830931312591915235602686757260974871e+48405', 71, 6);
    T('8.33927815576087140245999696847696897768056588410E+502556', '2.887780835825473685363416812628e+251278', 31, 3);
    T('6.1837545768142607E+511612', '2.4867156204146586532134418734e+255806', 29, 1);
    T('0.006899204224187481261510747376772643053927475970612024404910279863073297075867272135808082879033812', '8.3061e-2', 5, 3);
    T('6.87813736E+9840179', '8.29345365936290741869257113826105571710080400763673e+4920089', 51, 4);
    T('6.72117782580544531509587361209028412677E-483421889935', '8.198278981473e-241710944968', 13, 2);
    T('7.76015062431610523314557640140186212839055149390228682024E+31936571', '8.809171711526632078166470347524406767111631e+15968285', 43, 4);
    T('3.37931626005900509233951612577166337E-6965972', '1.838291668930424406262887590371762614576038271841e-3482986', 50, 6);
    T('1.82325556876558749672490000383531617727287571328E+7018324', '1.35027981128564145348215865603574367959917777906302941018e+3509162', 57, 1);
    T('6.344127943E+599516445009', '7.965003416822870390550682741876363738558163287242280225e+299758222504', 56, 6);
    T('3.564164185953657078527486527154774015402318E-486656074172980', '1.887899410973385805968339859774133686927994601660626373933911522256861782e-243328037086490', 73, 5);
    T('8.56159212791702212074058837061314975026661895226851144912309899515559E-614789669898', '2.926019844074373689848267e-307394834949', 25, 2);
    T('6.2061832218096E-49629', '7.878e-24815', 4, 6);
    T('6.0250129470687293316007559956860842799788346239366856141035619558469709348027270712874040E-2290501211', '7.762095688e-1145250606', 10, 3);
    T('6.86613884906820195275083680722883496957486530208410026087562318182432014123033901080545548236991286E+457897152560000', '2.6203318e+228948576280000', 8, 1);
    T('4.5415881789403E-92092161', '6.73913064047603991839022918808552908053232497506530471515987511768581371804319e-46046081', 78, 4);
    T('1.88923E+297', '4.3e+148', 2, 6);
    T('8.6858956296551247852583731481E-521640427112', '2.94718435623819175857515271e-260820213556', 28, 4);
    T('7.192824766954661969213959062263978766562096729813E+4210921', '8.48105227371e+2105460', 12, 3);
    T('1.316366814176955565198E-534289836643063', '3.62817697222305e-267144918321532', 15, 0);
    T('6.21878155341781810363451654960387929640909910643341410E+7848', '2.4937484944191581613961313504735e+3924', 32, 5);
    T('8.85804511569359615682704642158760507979516526E+1523103528535493', '9.4117188206e+761551764267746', 11, 2);
    T('9.89350990618E-42', '3.14539503181714840070469172354106204645011521983e-21', 49, 0);
    T('9.5157916787424470792465348447490197303E+4999046653912', '3.084767686349e+2499523326956', 13, 6);
    T('3.95073363240734504746373130E+1251', '6.285486e+625', 7, 3);
    T('6.4161831060852443312711685237513202332435399728E+7607562', '2.533018575945554397558473373020794511346443762e+3803781', 46, 1);
    T('7.9081115091724135895000365079544857917547947477E+8108486', '2.812136467025100400026107e+4054243', 26, 0);
    T('1.19438364045904207382589495158286275863787536558889144898002408972372459179397190913905857397E+3190856161', '3.45598559091187e+1595428080', 15, 2);
    T('9.48484894482917852670086151675313701746515134945118024654840174100288945066890460154318510982E+761016852', '3.079748195036272527419980486506576843892948e+380508426', 43, 0);
    T('6.4645148671226613832346930019113171198394386562360E+524', '2.54254102565183033925869695863279318104311599621046382e+262', 54, 2);
    T('9.1517499300374913142396058118146546798237370868969153052599918181274956198E-2826750', '3.025185933134935644172420225211067824230003684744552e-1413375', 52, 6);
    T('4.736914E-439220', '2.17644526694332472018851e-219610', 24, 3);
    T('3.45961362402205105157077430569088300091759087207102937955936E+86', '1.8600036623679134711927825181361821e+43', 35, 3);
    T('2.814697390325323155320273691E-36', '1.6777059904301835553246179325283e-18', 33, 0);
    T('6.64437284406074324611318431195970660211158835689957231929501418475767194597058994634790877082179127E+86068', '2.5776681019985376011207633304043626267918e+43034', 41, 4);
    T('2.68236605036122557322552885418163539833170319346072067416722765706163670135263341201E-59', '5.17915635e-30', 9, 1);
    T('6.148872207401570376020119420235046224698559701521720419E+9152', '2.47969e+4576', 6, 3);
    T('3.799506420648930342305109948301099700905295218232558544524023287787752E-8469288134631', '6.1640136442491189072868508443751915243428620122054121e-4234644067316', 53, 3);
    T('673008.822011357375117245694', '8.203711489389161686093829584093788433184742452342587e+2', 52, 1);
    T('8.7256328621678299923448733E+97', '9.341109603343614548333153844601870403748560996214965094961575087007068e+48', 70, 1);
    T('8.503149024679409958990750471E+14368', '2.916015950690155254211561530739256566821467638306228513194325176614498395e+7184', 73, 5);
    T('8.050480045568151005663856786E+6221334381430', '2.8373367874766208313424107747859100419930385723890060907e+3110667190715', 56, 0);
    T('6.525650741287709103184567373786220431680237E+7589130002867', '8.078149999404386587986786479657606151961208085188055e+3794565001433', 52, 5);
    T('2.57099313540653005272855347873564353933221737154020352355717233182254361527199E-7748874', '1.6034316746922926034980161025176250204810767557407663041913889661612641982354e-3874437', 77, 2);
    T('39.33673236455028961686454426970066080157607046977987667194998651', '6.27190021959455997346049636728598238059467046110914892902305117328889069e+0', 72, 0);
    T('4.41263226873940121226797448096198093682689114151676851733800502309E-732', '2.100626637158398315893317041e-366', 28, 1);
    T('4.52682453095202930256924673640732117738366411815984062618735E+4051', '6.7281680500356330669899958e+2025', 26, 2);
    T('3.679885612470268620458617840835868967940491584362478387788832203993413633775323180904E+909716840', '1.91830279478247870806321e+454858420', 24, 3);
    T('7.41885405103E+4763920', '2.7237573407023614665149811712587e+2381960', 32, 2);
    T('9.6555018579004403729E-7869883980580412', '3.10733034257711974811446243013197781182681044168848810063e-3934941990290206', 58, 4);
    T('6.7599544809747843346348726104396321138740295189572164E+3637733', '8.2218942348918502337884324373375197e+1818866', 35, 1);
    T('4.130259108236161382894920727047144697056899E-835', '6.426709195e-418', 10, 1);
    T('7.803843045018745138829614444341568097999088537803039821881897220889985940489649559632E+859', '8.8339362942115136916e+429', 20, 2);
    T('6.175132073426224779505591693766161598301413289160697006654719477735246679367408E+20568162', '2.4849813024299045577528401388731045944800682437193213190235e+10284081', 59, 4);
    T('4.660002266703089671796788753785682E+49183', '6.826420926593297668677623338678259897e+24591', 37, 6);
    T('5.7524E+80785726980', '2.3984161440417298858871962747728739390055327089310860177e+40392863490', 56, 1);
    T('2.3101041303458E-8438263', '4.80635426320802948e-4219132', 18, 1);
    T('2.277723687458622681855270265596994038851E+896478', '1.5092129364203789125481468293887694184994495791e+448239', 47, 0);
    T('5.96321219679099557759902292763064586121776194E-71', '7.7221838e-36', 8, 0);
    T('1.796886498610126636904763058808820608954083534309134734298881564E-90701222613', '4.23896980245215551701201e-45350611307', 24, 2);
    T('2.953287', '1.71851302002632496784903817489487135e+0', 36, 1);
    T('5.2029318476349918545948001625697580609E+26591', '7.21313513505e+13295', 12, 2);
    T('2.5498181358137286987669E+69441', '5.049572393593074034013285923612583e+34720', 34, 0);
    T('9.8942E+625759864', '3.145504728974350151263012468556325235004597957151795293544021829937805931e+312879932', 73, 5);
    T('0.00062763794461861761198645049589690206777813320801191732', '2.5052703339532394901e-2', 20, 1);
    T('7.8395263606570778926720746015789885661979155479108274613E+845988924', '2.7999154202684548043597700211235570410653476720606701506678674433122078943e+422994462', 74, 4);
    T('4163042420074733.521273420856556', '6.4521643036075371794019476585e+7', 29, 6);
    T('1.718974466269228229727101189684919236182016680838354E-1034', '1.31109666549390179956073727106749392157687266983376e-517', 51, 5);
    T('9.69431551752765058168530264938223198146497795875958733276412894925973411574202025566878579317E+185752229581602', '3.1135695780771707e+92876114790801', 17, 4);
    T('9.8653228767019088866375293402960653E-6505873305644350', '3.1409111539013498314680118909e-3252936652822175', 29, 5);
    T('8685978197.231291681066417302200827733503315235415854925646762158', '9.319859546812543e+4', 16, 3);
    T('7.2174808454859637295207909174581543467461105967563778390018842727E+5698858862', '2.686537e+2849429431', 8, 5);
    T('1.3902199879372293030660128312953946E+4797785194', '1.17907590423060944811439048530321371177917921536679966547e+2398892597', 57, 1);
    T('5.42333519650383569259575719870042929800581595820E+219015415895', '7.3643297023584133475444111681426116e+109507707947', 35, 4);
    T('8.756551894984173040413E+43297134423840', '2.9591471566963636e+21648567211920', 18, 5);
    T('2.1318695240481893645196235312464250259085314168726447761243434532534745854163718448510938828E-77875', '4.617217261563710913914309015e-38938', 29, 6);
    T('8.85507693355117131060658577261810004340189608111324758877799760492112898731E+749925291', '9.410141833974220874749101744669957061456080317049801144419503828233059920642e+374962645', 76, 6);
    T('2.50556909774165724821438531792064873434290571315323662677899054132450491201360429620412E-67391', '5.00556599970638410142405902582344634816938403206970774e-33696', 54, 0);
    T('667744067195573.634342', '2.584074432355952153798e+7', 22, 1);
    T('8.4911649427418713596137179123488937651671261135991828702920297345936546613256429484419299E+9439788392772', '2.91396035366678785560146429829810635058472565349543295311955957602496149493e+4719894196386', 76, 1);
    T('7.27065846861729346606867283021807279627300299399443388357157562987241211476980958577092796120079E-22611598069726', '2.696415856023935612487382568757770769449288273e-11305799034863', 46, 1);
    T('66631.2258364122736081908037301588676088254066126889718050316938244640550613360264057969626374361', '2.5813024975080366122547045236179945806517666667991564e+2', 53, 5);
    T('8.21987779861900487592441233958051249770027604290865840680754569449311553202494663144E-46745293880924', '2.867032925973994613259585575376994113971240136644657028821735913836127e-23372646940462', 71, 4);
    T('4.01107402482288017010026917772529940357750614189932237831098002827388360432E+894044036436364', '2.0027665926969323626541086370414804193802e+447022018218182', 41, 5);
    T('8.71240317011309636919360898935017060678681626956895164794248386315181383987602113412438726493E+53433593328028', '2.951678026159543094618e+26716796664014', 22, 1);
    T('2.8275495307931679537768132726448097053848336202471172106061731402142860934610054634E-322068', '1.68153190002246699982e-161034', 21, 5);
    T('5.8364771328859338575657983880487421862146043344044E+46616950', '2.415880198371999955733e+23308475', 22, 1);
    T('5.739185986341335750155449561541100125942765571005168437189309530227E+592', '2.395659822750579033084456152939376507548e+296', 40, 4);
    T('4.638572220753132205517515259902492552253454679676405180008643964697250575009736139981E+51007040464469', '6.8107064396824006743619303071429377955701e+25503520232234', 41, 3);
    T('6.58716054837884598571274074E+5818238923', '8.116132421528647031044291762213477543124367688103562663566418172495215e+2909119461', 70, 0);
    T('2.599459552094195270E+469036949708445', '5.0984895e+234518474854222', 8, 3);
    T('2.9960370806134519780432128433211708677549246945628817813560E-502', '1.73090643323475228744155467786112519395e-251', 39, 2);
    T('3.157955637261556582111166846154000066123593266909698060964491E+8630065625', '5.6195690557742562383649003614649090965639409713655176329357318379874588850975e+4315032812', 77, 1);
    T('2225977893.6654421436944016083630020081330148332028321646125766793609071291265153', '4.7180270173722e+4', 14, 6);
    T('5.491E+2553865267218556', '2.34328828785533770055250513399880982262369e+1276932633609278', 42, 5);
    T('4159244151253092883270753236020188706.6091040', '2.0394e+18', 5, 1);
    T('9.262504220502E-98090', '3.0434362520844756474947e-49045', 23, 6);
    T('1.137E+52804170780', '1.0663020210053059821594260330832308591539475686196386837481208780197485604832e+26402085390', 78, 6);
    T('8.2196911663285967302E+244724331338107', '9.06625124642406803965490756806e+122362165669053', 30, 0);
    T('9.94126699E+96802', '3.153e+48401', 5, 6);
    T('4.31533391416314376391402691257001583038394014047689058837250160662886128315864561E+2491287598236', '2.077338180018637158999715580463272270605500896740675662e+1245643799118', 55, 0);
    T('1.994038605043382700297077308550E-82390348280075', '4.4654659387833009203839562188175005939294704e-41195174140038', 44, 2);
    T('6.09584178908199706489252977708482285244178953806200604570079451728352827269984140329091417771542708E-3136892714173', '7.807587200334e-1568446357087', 13, 6);
    T('9.51055120313677662228755057507192496928792E+84364948053497', '9.75220549575160286587616655235001192338926658783127022593581300742178316192e+42182474026748', 75, 2);
    T('4.355541852435656193866690888843091463209919530184200319783504776674194756073162220688E-6622', '2.08699349602141e-3311', 15, 4);
    T('2.5956162E-58990', '1.6110916175065898467595858414674189613505725845217714995591e-29495', 59, 1);
    T('379.040870689383684551786954118859394876001383809202868991053911196696947', '1.94689719987826703061236377178842898579992816088186467063854073472750441014e+1', 75, 1);
    T('1.98029E+252522631153', '4.450044943593266676885338891519755478187007066077734472048278750858938326826e+126261315576', 76, 3);
    T('2.02725051474952244439692828130137398E+52140927786255', '4.502499877567485780593974984e+26070463893127', 28, 5);
    T('3.561840125792179726150793420313408241754858E+542419863', '5.968115385774792245065452197195e+271209931', 31, 4);
    T('6.601905294313148194996227647818494970266644850656532180492004817371775427', '2.56941730637768301e+0', 18, 3);
    T('6.50521592077291532971911971708691076121828602814324672629679923342155756044727693648437655403552E-8367265938345294', '2.5505324779e-4183632969172647', 11, 0);
    T('5.5966131641E+418', '2.36571e+209', 6, 1);
    T('3.395629660258253405047798824585207451758057266741790152589138267146486687786106056896111186449E+679', '5.827203154394270305401003286535758395857263887604931238361e+339', 58, 6);
    T('4.25982341229989115260424162670812028350287433248143E+96039613305578', '2.0639339651015705770397370839788314442039914211817343363e+48019806652789', 56, 1);
    T('6.435E+855749444', '2.5367301787931644347258776884164893061994619807415789257e+427874722', 56, 3);
    T('1.6242401876638568413134362634680033648840994909541499859757762470511613030387816026893E+35688', '1.274456820635307691675980203223103721664100275000587674754926657473205e+17844', 71, 2);
    T('8.222691587375906994563434301553023803916375812375860900564E+94765102305848', '2.86752359839913209293214818627355317544011675206264030054846398158167668603912e+47382551152924', 78, 1);
    T('6.06034968237236498921556248704410756746899539345801059808154520E+656', '2.461777748370547968142219289072431002877108880545684e+328', 52, 3);
    T('4.50214138957349018688364488180461186364548396E-178313286639807', '6.709799840213931587919818044358848102289063575199181257396304231918036502232e-89156643319904', 76, 1);
    T('3.70134397612645332938899805037962761054234391172864551786310202471050502817935E+6979177070615', '6.083867171566497415116647455686408255755168083478e+3489588535307', 49, 1);
    T('8.3187569851741090832102629450483170036711172658024948069365822774814E+6973246793753', '9.120722002766068893693475819448532737e+3486623396876', 38, 0);
    T('1.1696487E+984933', '3.4200127192e+492466', 11, 3);
    T('5.222235783579513790529929154385765176194335496958065967288500995030575405E+899138789', '7.22650384596833504690134284297196177016e+449569394', 41, 6);
    T('9.2240613E+8565435', '9.60419767601646150124491299667501573651807560357e+4282717', 48, 4);
    T('9.8348002796661524729808583683513983666449182875677237355029523478E-689', '9.91705615576827944029215e-345', 24, 0);
    T('9.83802456385696737055988965368545007353591233323511034632741174717060924593362754391012E+61532923488', '3.1365e+30766461744', 5, 1);
    T('9.807E+956846870138614', '3.1316130029108002795248393931047860726988519427769546871498933220401054117e+478423435069307', 74, 1);
    T('8.928881096534023615519931E+937372795132', '2.988123e+468686397566', 7, 3);
    T('8.360563553518585392E-12417347449', '9.144e-6208673725', 4, 0);
    T('4.79590258382715682694628910104410905058616624822719823644209078881E+1751598', '2.1899549273505965987e+875799', 20, 3);
    T('1.7348523194834630451435358158369773113211096056783445124765552584488293363872236974474445503566880E-12021847286809', '4.16516e-6010923643405', 6, 5);
    T('9.8538E+5064', '3.139076297256885726618898062e+2532', 28, 3);
    T('1.4794038163271443297939148506607667268531929094939970113227687467429820698E-57400113', '3.846301881453332219445e-28700057', 22, 0);
    T('2019.964313', '4.49440130940707163e+1', 18, 6);
    T('4.921445469629401773179332138045569E+39857580572', '2.218433111371492708945896477210848943079051152133961e+19928790286', 52, 1);
    T('5.5075849982917614378159327209371544037936251735940439685426357739917E+68', '2.34682444982400876592728523142553703241106283406043603905325186131135072969e+34', 75, 0);
    T('8.9408104097329E+146', '2.9901187952542788563247271022264165246588297866172485e+73', 53, 0);
    T('8.725902021891902074942776784E+69618', '2.953963781411664850025679586227e+34809', 31, 4);
    T('1.8626169844E+7193570', '1.36477726549060011809301041e+3596785', 28, 2);
    T('5.5331260271810440962168419107048539063203287289214E+152741', '7.4384985226731369541915804488984357149497658e+76370', 44, 3);
    T('9.99388743338758033073593789534299675828289351134641152614E+817', '9.99694324950761152951012963625449277431793836e+408', 45, 3);
    T('8E+60199355871', '8.94427190999915878563669467492510494176247343845e+30099677935', 48, 2);
    T('2.805243637550309126860725127148107943092883571027206338834901200597069051078678622368441767026E+405018393', '5.296455076322567211792886513039239693817333278984488519885675267369340117130382e+202509196', 79, 3);
    T('2975.1559948986970101157123702507792118050120746563098050822446405379272789', '5.4544990557325217e+1', 17, 0);
    T('8.8218268186677490797660446100142246250847982751982995432725582E+45087419', '9.392458048173e+22543709', 13, 2);
    T('993678578.5802842644615529170837840343489475760709986863139147136506697852351097833829', '3.15226677e+4', 9, 5);
    T('9.43773184360E-77374818766925', '9.71479893955608326773065145570636501123634556540092e-38687409383463', 51, 3);
    T('2.927384127614387346157484363658695970216640727155369880169433497018461253938990E+71419530488', '1.71096000175760606415939159503694668671859524633665466740023961887473001e+35709765244', 72, 6);
    T('7.1041105804856837734384832129125221434217973833819120095874E+4092', '2.6653537e+2046', 8, 5);
    T('2.5959427258804419322439119948163972971E-87127866', '1.611192951163963456670134097332324524142864026410624408e-43563933', 55, 0);
    T('2.633312036506013976011894E+3353', '5.1315806887410568865608154587084990885739261894900084e+1676', 53, 1);
    T('2.67529629755298864953379359069172073253724880863165681152277449640510536231495354411351494319843145E-44706186514505', '5.172326650118e-22353093257253', 13, 3);
    T('6.40430E+52842442', '2.5306718475535305958406e+26421221', 23, 0);
    T('3.165188323757543254052091E-85', '5.62600064322565044447951655494560140034385107782e-43', 48, 6);
    T('4.9950995653184343223970985966192852837254830602727811E+331458661', '7.0676018318227536681030427e+165729330', 26, 1);
    T('8481317543290666788311223342601617320496666257023400682411543300078512620710871044.82100', '9.2094069e+40', 10, 0);
    T('2E+75', '4.472135954999579392818347337462552471e+37', 37, 0);
    T('1.158196823499344240548640381042029166428615159274504E+55', '3.403229089408093668054543266821800475718e+27', 40, 1);
    T('3.318712899792976566996385733858256431495E-3447228264', '1.821733487586e-1723614132', 13, 4);
    T('4.813103898978207E+1120920101911', '6.93765370927247059867093748112442008037278601455038812688460525728168766765807e+560460050955', 78, 0);
    T('8.0307778084917252782089929755595348077864280144418283267061995487787460864E+2290491224727932', '2.83386270106576004495086309265192507173281611548e+1145245612363966', 48, 0);
    T('8.536120527292488361E+935574448591', '9.23911279685040341e+467787224295', 18, 5);
    T('2.05735659957960719E+88738988', '1e+44369494', 1, 5);
    T('1.845721034489887941E+91808', '1.3585731612577542527846105246191622276628011625135751449254372184408151825e+45904', 74, 5);
    T('2.8225521712671764817627858760915345778599307422779E+9945966', '1.6800452884571821808989741738319116764010207769744911131287423726321215670329e+4972983', 77, 6);
    T('6.2501145392733957403469539797E+81993723', '7.9057665910861520579176252599231687931e+40996861', 38, 1);
    T('9.4872605473708369914385913815964025362786765647351671826040620736039630E+584', '3.08013969608e+292', 13, 6);
    T('6.6444961479659666698078657697068699143742E+63', '8.151377888410012040035025046804394707627499120061481868784e+31', 58, 1);
    T('8.2410932856985393480939371104673260570069486528909302275180442332552540643E+78145848317', '9.0780467534038066046175335957063295521789418873872413e+39072924158', 54, 3);
    T('4.268960235049606152955000533E-1446726317', '6.54e-723363159', 3, 2);
    T('3.25852461571806765707E-1569', '5.708348811800193116392558423886227845914544e-785', 43, 5);
    T('4.3E+77970283544671', '6.5574385243020006523441099976360016279269663198837898e+38985141772335', 53, 6);
    T('9.29338427E-95040395', '9.64022005454232357e-47520198', 18, 5);
    T('9.159288681037700288065355618141265241021678588267853019743744482232688822E-292612555', '9.57041727462167674948689359521859770449e-146306278', 40, 4);
    T('2.828450365E+46560455092', '1.68179973986203244388091205004542524105077e+23280227546', 42, 5);
    T('5.0637720736278627857092822849298876563680780027217193848032741486512655989E-7334496739', '7.11601860145676037545564257362967065920961207092965866e-3667248370', 54, 5);
    T('6.9342E-5867107347', '8.327184398102398212557842928054789e-2933553674', 34, 5);
    T('9.3223E+1581', '9.655205849695800245414108002125603727e+790', 38, 1);
    T('6.98795417332787565403460577663220445884644148258380206032687417E+4171880616040613', '8.359398407378293902459944776363598336625054e+2085940308020306', 43, 6);
    T('1.192827557174224814656E+307075180532', '1.09216645122171044219652764257131542991e+153537590266', 39, 6);
    T('3.62703425114152599498767115685458097426649921790943842159438345947894933416E+554829', '6.0224864060797397494258746809e+277414', 29, 3);
    T('8.088226676949031611241514632058E+4288742320470', '2.84398e+2144371160235', 6, 6);
    T('7.61894E-7', '8.7286539626680126515681982936044833e-4', 35, 1);
    T('6.2070226732357810028617E+178986266', '2.491389707218800605692200183548e+89493133', 31, 2);
    T('8.0155587E+2747643666615187', '8e+1373821833307593', 1, 3);
    T('1.662730663130770E+345124367961791', '4.07765945504375095308174599074071511214460240756938865814094665414772939773e+172562183980895', 75, 3);
    T('9.104898410302425371961971219773410798811514E+2119', '9.54195913337634525207927635173678614448981063465739e+1059', 51, 0);
    T('1.83824035720400166536401770840689101281247107711086810758949720340518648227691580684127493786478E+18156968403', '4.2874705330812498e+9078484201', 17, 6);
    T('6.812891194593866464108963775783865206702256E+4512', '2.61015156544478590248285625450573715880666763211032e+2256', 51, 1);
    T('5.12684665347394686935E+2828079994536768', '2.26425410532341684641561057111445125627343817767496848086e+1414039997268384', 57, 6);
    T('1.6232257750334452690874362295838801835245E+40542652', '1.27405878005429767890477077974747546333019860672108762e+20271326', 54, 2);
    T('1.67986819577277975548116049859019287304903890271953044338805152023333977921E+8104288735517894', '1.2960972940997831415390655939656593694796337226503254065953320193563250115e+4052144367758947', 74, 4);
    T('4.4973486052754365277724054232258965396780993739E-31923744612575', '6.70622740836861013747389617292340116548477436959e-15961872306288', 48, 6);
    T('8.089207927482433085889746576712467057681703557256236942549980699642021103464082904424955938513136E+58394888', '2.8441532883237e+29197444', 14, 6);
    T('5.25408811442485888629689977218491874742632607032332805258265352858130914E+6382', '2.2921797735834026171202369209900049557938e+3191', 41, 6);
    T('5.3539412195E-6032596424', '2.3e-3016298212', 2, 3);
    T('9.9406282477201179431064782086639799682142961953467263566337520134269E+20028581', '9.9702699300069694467672618725906229e+10014290', 35, 5);
    T('6.524952131775E+899710322597', '8.07772e+449855161298', 6, 2);
    T('9.697849668694925543026319675628916241933118838540650773599808046384533841582522821E+56330088', '3.11413706646e+28165044', 12, 2);
    T('1.9335721458246461689708097037077620633402053174836125511653522192E+6084181746041', '4.397240209295651122640396952073706191539558214996391408138918703448680754842e+3042090873020', 76, 2);
    T('1.7832671759618764454458716429953541E-8686887156452053', '4.222875e-4343443578226027', 7, 4);
    T('3.239065418456051673170705623288173433832264007515207715671954686E+8407831882', '1.799740375291961847020702611945468323226673463808727939340114108654129019299277e+4203915941', 79, 5);
    T('2.83451554E+3151098057', '5.32401684820775149282142536e+1575549028', 27, 5);
    T('97003.30805577677493464622273333', '3.11453540766157890431504341968431e+2', 33, 5);
    T('5.7996120334791158041549072178244833170357251011031076E+885994616722378', '2.408238367246713351007342318506299924759498e+442997308361189', 43, 6);
    T('5.494963003520177654113996671497347640808074109703843626157235782409734298889973471274392', '2.34413374267e+0', 12, 6);
    T('3E+2929478163430', '1.732050807568877293527446341505872366943e+1464739081715', 40, 6);
    T('8550.9751530117804', '9.24714829177719005e+1', 18, 5);
    T('8653760.380377792315020011932393915977517769944661365953857673659183046569352493', '2.941727448350338304165232612166275871e+3', 37, 2);
    T('6.45652948656427539987702255361012045245350E+407068616911', '8.0352532546051561273170053837e+203534308455', 29, 0);
    T('2.96174247990531624454267287971044081873442366724E+5205832141948', '1.7209713768407992e+2602916070974', 17, 5);
    T('8E+2175629', '8.945e+1087814', 4, 2);
    T('5.2371027562773411408662073777651089174633247328065041743846767018434E+50434050', '2.288471707554485179737193395683288045997e+25217025', 40, 3);
    T('4.45502736752314561211500702927278E-3854444126', '2.110693574994519570922315664454018265541747642300182941e-1927222063', 55, 2);
    T('8.4078E+3092571182121', '9.169405651404021071858061205405311135573e+1546285591060', 40, 4);
    T('7.5912268810899888165407257872702312482075540E-206286958', '2.7552181186051293852907481886571770282506280449970731e-103143479', 53, 5);
    T('4.68815E+33', '6.84700664524286846408785798024821435225627919e+16', 45, 0);
    T('8E+96026884', '2.828427124746190097603377448419396158e+48013442', 37, 2);
    T('6.69011367833161194708244948357970406668095606180907093686E+3488121068518', '2.5865e+1744060534259', 5, 1);
    T('9.004411300356165421449126894032107285118476756021640829007986759981159925733066E+875645424923351', '9.489157655111524924889596595676407417057844494502527578293307860175325254e+437822712461675', 73, 4);
    T('5.2181491E+7264914830', '2.28432683738557867515245785410077800093317250787e+3632457415', 48, 3);
    T('1.350403176724795325705369635591019468672021343E+9065854851', '3.674783227245921951900922121e+4532927425', 28, 0);
    T('552004471068295846046466536061848747218768887482171761739937848479.098058580683020998397', '7.42970033761992211e+32', 18, 4);
    T('2.3418265231996666867680931416247228353834039772390655925833066481147943135668313998353457006053849E+282', '2e+141', 1, 4);
    T('2.744925407654092974145888132053340E+15467886', '1.6567816415128739e+7733943', 17, 4);
    T('77771.4', '2.7887524092324868845941980947817594546e+2', 39, 3);
    T('1.67174568E+92282166996552', '1.292960045786411762005204321909216e+46141083498276', 34, 4);
    T('1.1511449324747515721605698610966451091639618596276819597459E+903215', '3.3928526824410628574133832915379130822021911051613634185458e+451607', 59, 5);
    T('6983828669366648773101887594213629609099017845703.4816512466928296', '3e+24', 1, 2);
    T('2.60873798614602913089505057379949474727362265357942E+705189596888', '1.6151588114319e+352594798444', 14, 5);
    T('2910300218.0142541004778827946669120959', '5.39471984260003449947631094668690804252204749873261782902778670866553626653828e+4', 78, 2);
    T('1.4938985660024718532294308003104848307836568084679442713114249706291789135759667683252E+23842094260', '1.2222514332175977666381193812290614719954139921233525308e+11921047130', 56, 4);
    T('3.112688589085790457880444138326632411833879893230324105355539E+5684659', '5.57914741612532627465428623728515e+2842329', 33, 6);
    T('3.994003414583341301039992983073195935995E+998513200', '1.99850029136433735498196414194015801237529848095536209282452293443360723788841e+499256600', 78, 1);
    T('2.379997526716252906020270046704853599739989121030041274463991781599434E-939564546', '1.542724060458075861e-469782273', 19, 3);
    T('4.7976758489613424352102762675733617038305801443494348E-455', '6.9265257156538027504222804e-228', 26, 4);
    T('4.4261195338290791706279070728954652762977039988360052625647654010375438250382789132506371736588038E+6277118702', '2.1038344834680030433289747e+3138559351', 26, 4);
    T('3.2029054323200030115137321612984773408908332150172935304553648459552901824858883449E-3113728', '1.78966629077043383068922974288910493455398850266483323788e-1556864', 57, 0);
    T('7.438852426697785582626698936058872304211E+267644', '2.7274259708922963877605275739983488024144967441e+133822', 47, 6);
    T('9.05955021273492758735628096659693549221483665476619274288306762270380716197259391359059314567636E+52246313', '9.518166952063263524144517831550753562129680273422569106923895126665741529436e+26123156', 76, 2);
    T('3.16690040179365995973595601825507105718070514758190429487824516251868723685656720E+47397355', '5.627522013989514337007749485226831448498711771325580883374236699096192616e+23698677', 73, 0);
    T('5.422938173476023891911924755835665113961958799898363774E+13141', '7.365e+6570', 4, 0);
    T('3.3503991872083455236690473051E+951474703414', '1.83040956815908977690716970399480204562938681e+475737351707', 45, 5);
    T('7.596227635992359641297663665190830062595754810350059193003731151415E+165048790141230', '2.756125475371605725998300721340726678990451815510044175932715265984263111055e+82524395070615', 76, 3);
    T('5.19355920251144093737305006109306568900086619747724639071900804994582353461244816746E-168792991430', '2.278938174350379549448e-84396495715', 22, 5);
    T('5.1858884544469614176654676857120723330671934318351E+4249980079783783', '7.20131130728769523708440259960002259001591870155848764068e+2124990039891891', 58, 5);
    T('5.9078789757933467513379440668733594076694685173059011567154023993003211234417647941382171430529E+910466314110', '2.4306128806935395551053253187e+455233157055', 29, 6);
    T('3566184.78956327843023454750593853361118178017931746194443319196', '1.88843448114126491402869902638761389047944855106527139009965085371257889637662e+3', 78, 4);
    T('9.6883402254559630848651806482549909066314702524883670229408022009383E+241751', '9.8429366682184657268e+120875', 20, 2);
    T('1.256067414046735928934483285378629240E+7711310345002', '1.12074413406751138299062e+3855655172501', 24, 0);
    T('368518.2302682188883715522957738475878758029959868064189', '6.07057023901559752286023967827327721067375055933306e+2', 52, 3);
    T('2.013733413686777036168744143906E+226420059793599', '4.487464109813890239854509342943646176308341754682539041e+113210029896799', 55, 5);
    T('8.7625211880545011344822682781895494E+796178360', '2.96016e+398089180', 6, 5);
    T('2.49845046977536739882532394091077270905544615254504507965645331018324506E+670691380933', '4.998450229596537053856e+335345690466', 22, 0);
    T('3.4499489427534524565115864543182555807205235218592612659987984193542772025226903859649019094E-82588931', '5.87362659925999420567254445214004133031799984105543264078418181898837479e-41294466', 72, 6);
    T('9.61783815599211327109878101473388245548770502262175361E+75805480', '3.1012639610313910316158663183633426514263122060731683e+37902740', 53, 3);
    T('6.01400074097133346589880390198165819716E+75919', '7.75499886587440673230991194260527361e+37959', 37, 5);
    T('4.22951938865550684387342074547471437094933389215263813514298185633784758997165033705620743E+7159970384', '2.05657953618514517297014036e+3579985192', 27, 3);
    T('7.7863663136854456886468575985390903765154563997256498574870066413601071470E+27512736277', '8.8240389355926153963143966441169173266769409763336e+13756368138', 50, 4);
    T('8.557329526914357011081299554907282770757769551450529238430619593856977055996E+7032049211', '9.25058350965729999721366507922931413788824901565089022147181318609786932092898e+3516024605', 78, 4);
    T('2.6267185137176927787829219295468647911660798562968443802881640918794199677E+719295', '5.12515220624489627340535270639148883926872882758962532315346647818379833526e+359647', 75, 5);
    T('5.8788747312693102290411629645318756811400942479443604523685924566567585193431070954552938292760E+45892300630363', '7.66738203774229e+22946150315181', 16, 2);
    T('94.585636633938584', '9.7255147233418e+0', 14, 5);
    T('8.90345744601690717969950556280823139957252439382903125760505230541E-136', '2.9838661910375450312373099195448508002389123449711619717481119835765443425e-68', 75, 1);
    T('2.31685800688630202776195005084086992901179308823532423E+57278', '1.522122861955072938047734155229187798544646400406538836e+28639', 55, 2);
    T('3.2172054631958880073736289E+7136454830987389', '5.672041487150713716448665881882171002110385374432989733046e+3568227415493694', 58, 3);
    T('5.1461923636343148270393817233103671972849458012490934893581313428979411E+4880127', '7.173696650705488834e+2440063', 20, 4);
    T('1.054076927848730948917522348907824617645107847042135292845447777429837381082346529343187918962E-734', '1.0266824863845350259387083588e-367', 29, 0);
    T('8.37261517162678602003021958714462697648402100483157624815582506383231873175801865950256E-3859', '9.150199e-1930', 7, 3);
    T('2.95948366743778928314281401930110485035215597411150507414714790964E+846543335094646', '1.720314991e+423271667547323', 10, 6);
    T('1.1223474652E+169048108012662', '1.0594090169523761420834366789975806809175870220014276256515134085172644e+84524054006331', 71, 4);
    T('1.7676247632E+3556860715732', '1.329520501233433406434365634787773860982231749829071846912115465140716790219397e+1778430357866', 79, 5);
    T('8.773E+310', '2.9619250496931889785506e+155', 23, 4);
    T('8.47320607468410E+22936', '2.91087720020685517066079100457602523191140909237407189412e+11468', 57, 1);
    T('1.600000478202096801378986453840073661059E-829729121678', '1.264911253093313579039793327622036e-414864560839', 35, 2);
    T('1.0333641368338985172725E+766640135386', '1.016545196650841746169102461354582802814000984e+383320067693', 46, 4);
    T('272.491845088230699540597213630789102105750116292960', '1.65073270122158390450304428468854715316871165745367084866e+1', 57, 1);
    T('6.739753617187798702821833951385214162698431E-939900311', '8.2096002443406455445982309955942305590271267982965077e-469950156', 53, 2);
    T('9.699895407173301521909259976043042334862167361228840687436777039439126851032832745E+704877512', '3.11446550906785633677485e+352438756', 24, 0);
    T('8.8E+4069420479', '1e+2034710240', 1, 0);
    T('6.191849499E+2754380510028', '2.488342721371e+1377190255014', 14, 5);
    T('2.96790329961911851938815991117121338584573072936051888442E+7119', '5.4479e+3559', 5, 2);
    T('5.8271572401014884E+548', '2.413950546324735377552044294e+274', 28, 1);
    T('5.6674575219607943930322447154996063377661141573683832998919014649E+826962', '2.38064224988988935507217020141795428e+413481', 36, 5);
    T('6.761043349783859E+57510064', '2.600201e+28755032', 7, 5);
    T('7.703221712568111691E+990', '2.77546783670214987613118669040287865673978698100023005977064389089531109229636e+495', 79, 5);
    T('5.39350027113157464357E-506414', '2.32239106765668874581034544840635048e-253207', 36, 4);
    T('1.6176906924144073097714E+1570', '1.2718847009121570960557074993178307760092356135708418501757516146277573812e+785', 74, 1);
    T('6.31775138264658168121623624076337190935910142897251403520798752493720057813813369552421E+439396685', '7.948428387201196694956263075002049176675006667954514912776941551924354376735885e+219698342', 79, 2);
    T('2.783768391903361E+1867956103', '5.276142901688089228700605773579881043e+933978051', 37, 3);
    T('3.848676632110127822918831E+16136563098', '1.961804432686940337804e+8068281549', 24, 1);
    T('6.09685256336E+7003', '7.808234476089969469302595869854959595058371303959994385713e+3501', 58, 0);
    T('1.263605822486010448585044020858096339E+2845095563', '3.554723368e+1422547781', 10, 6);
    T('44446.1701141', '2.1082260342311495134365108363e+2', 29, 1);
    T('2.59367104206117949356047397621848601727558837776023023171401178694912870E+53617727', '5.09280967842033096876981524213930077050330084052619838166883506086104778e+26808863', 72, 2);
    T('6.9249292908559000095274990881638310570815442E+4723156829829', '8.321616003431004271496257491971075448604703362688846825575554218360573e+2361578414914', 70, 4);
    T('5.4E-8492287979', '7.4e-4246143990', 2, 2);
    T('7.5174285944045873372427707805509582591743421533424167786419402281245231607E+243399648845162', '2.741792952504726848715748976207964901287e+121699824422581', 40, 0);
    T('9.7377725616247122614735997071413671341579104E+52', '3.120540427814501534386015976716617741103937828e+26', 46, 0);
    T('4.0810753238546162525452182123047739358557037922198056642283454753616517160192194684354907640E+1849022894723', '6.3883294560116545e+924511447361', 17, 3);
    T('6.059773995207047444174449944650952196065420406669348831265411535743363326407653E+9353457543048', '2.4616608205045323514390372128931937992e+4676728771524', 38, 4);
    T('2.413346836528027055296186070137681312856215205917564995746379252631665740466020317947E+32548599554', '1.553495039106345860781624494145647752593599e+16274299777', 43, 2);
    T('4.27591862339039789033853409694006820119868E+742', '2.06782944736513458191535534e+371', 27, 2);
    T('7.04751350761964532023202586574701971485070795125042549980577350E+910962991', '8.395e+455481495', 4, 6);
    T('2.3062536966953710599853454167970097847855914217435175200759577635580117631504420038569765618E-4060490690320598', '1.519e-2030245345160299', 4, 4);
    T('6.78379589825605359085604842983793927777158416066401E+210801361020170', '2.60457211423605117097562695369e+105400680510085', 30, 4);
    T('7.9449135087755601207E-991790188038110', '2.818672295385818723838406321784864238260934131990461130607e-495895094019055', 58, 6);
    T('6.10303115178926594760816061984867788732449448545987082837143135193243525989303481E+80227', '7.8121899309919916639237135608908986804794740458133156337e+40113', 56, 6);
    T('7.16705035726432810010670322173664451578472033223447097363376165331844526597872775500327745E+759821031466', '2.677134729008670586990586240805119224244471007498e+379910515733', 49, 3);
    T('9.3242016775484261195091198367540267372248025520716414004E+5054591506', '3.05355557958e+2527295753', 12, 0);
    T('5.11E-193', '7.14842640026e-97', 12, 5);
    T('0.0708107013718167258275220557915252541004632388371115789843796452742784148745282189715773338548394', '2.661028022622398575095656319411622879793577745838924e-1', 54, 4);
    T('4.01341787496598925337808374138351823885896520595021698539433054171766365299730994640E-9890163', '6.335154200937802945104427217413953796067096891475352163814760926918781418416e-4945082', 76, 6);
    T('4.61651852904417999961719151499650542125063388452E+14935416', '2.14860850995340237659559027262941591627131555484746509e+7467708', 54, 1);
    T('7.11E-20585', '8.4320815935331176087159445976720752655046387823289991259e-10293', 56, 1);
    T('9429583173932593246342588090005597604941492591056323.6592', '9.71e+25', 3, 4);
    T('7.4261199526346611724906739485709161201535223556786828227901914994083E+8155739065', '8.61749380773474137370893816990876914147870402876478939295849726411134207324033e+4077869532', 78, 2);
    T('2.64891007296443697150503957799734595208054122572939833457609755591640423578622360112E-50', '1.627547256753067975722705590656672292409383e-25', 43, 2);
    T('4.14845105855108132946327427696473994786501E+51214', '2.036775e+25607', 7, 6);
    T('6.405974285092599249556585717606E-2102432882096038', '2.531002624473668784574958939300823e-1051216441048019', 34, 1);
    T('5.79884094068807732549309730082709694939953980332589E-4224069388171', '7.61501210812437075371985113880334526371e-2112034694086', 39, 3);
    T('5.3317343330548083249822729936240576E-259820', '2.3090548570908419032796544647515465032975e-129910', 41, 2);
    T('1.45718298035263241801458276995204213E+964085893', '3.8173066163888805671e+482042946', 20, 3);
    T('4.808906598759817994006089616349538750962474009957986349825519005171731070755156320E+1970612135976809', '6.93462803527328204886091003475615910883e+985306067988404', 39, 0);
    T('5.4301692237337756577249466637873898869131986025717843351234281003512849109969727965055709E+42319846097148', '2.33027234969086299239429695295e+21159923048574', 30, 3);
    T('6.6508421007666411179006738677777271659154570500908660256948718221083022435480E+3618151842867', '8.15526952391313742649268654453736961653967732176110391007158488834858273e+1809075921433', 72, 5);
    T('900.134690198610245202791998002621028898243184136804706246', '3.00022447526615955577075263191733776992402833234e+1', 48, 0);
    T('5.3328211719962051996661909483404486856599952540E+56736405342', '2.309290187914070967260093678117e+28368202671', 31, 4);
    T('6.751023152156461575154321169495599520472122697831186349338E-8903374106', '2.5982731096165509858598521e-4451687053', 26, 5);
    T('5.43156821590828956146151423309709767039298101972217853177497462063048183147405835004450E+244', '2.330572508185121787557627054285274025798377180799205414744623176319622e+122', 70, 6);
    T('9.22284449156833783732045381E-6494980192748470', '3.036913645721316494957523944363289976614529175968238440326241033118798915e-3247490096374235', 73, 1);
    T('5.999557621638926488257196468256558132913297582472382938617122633753431E+93431208308945', '7.7456811331469916689346342945969935e+46715604154472', 35, 4);
    T('3.47319254284275035723202843505016612575256630578449412640036003412635686911E-8573553644286459', '5.89337979672339661204120550558430078712452156244546055553955349667977033e-4286776822143230', 72, 6);
    T('4.4595066463159654583653959167815922719236806405865536070894335867440255E+221556', '2.1117544000939042576060402448279617e+110778', 36, 0);
    T('4.4112462660028069958823174394135233126657103223E-39373', '6.641721362721e-19687', 13, 3);
    T('8.994460015429982742939681619993214180586987928187803742299554469E+6857392430677574', '2.99907652710463237737993434096805846e+3428696215338787', 36, 6);
    T('9.92435731301054005064201E+280092612', '3.15029479779441277142973968296042615946281813122798710005579556453948664042714e+140046306', 78, 3);
    T('9.100934540073081632560972925990086986870378913483549501423632250127845260683168875832958241637632764E+806', '3.01677552033177994504060663321395206574733966875e+403', 48, 5);
    T('8.53097020193605604246569686669770762600962776098126E-7746407582086', '2.9207824639873569404012857391185862353222890998461599e-3873203791043', 53, 6);
    T('9.460266659223881326885128306269769862190675565785743601565545632288426074636692451532688869E-1287251', '9.7263902138583157540193261e-643626', 26, 0);
    T('17117164.34767832257244399042649355671594091984593814368922787903152', '4.13728949285378416489534871376454175290755812791450855037e+3', 57, 6);
    T('7.454917201442246136018558121680955810191862493363107584935860378290678512708354260997430623234E+8777012', '2.730369425818097263587192965470616256450048e+4388506', 43, 0);
    T('6.157521913402929053711066293892105807537754542389957977E+5703056', '2.4814354542084968472582312347697043104e+2851528', 38, 5);
    T('6.325014567733513853490113056938544767974323E-364708733283', '7.9529960189437501399494895511420565737106453932e-182354366642', 47, 6);
    T('3.355572772196178406814487648449233473959533024170327704690632614531718253336E-7775062337066767', '5.79273059290364237012736142722058594535953e-3887531168533384', 42, 5);
    T('4.9824489433385472442962473772021319315630977255288685538938314080775E+7599429037', '7.05864643068240615520509e+3799714518', 24, 0);
    T('7.353003963096636984650E+252632161', '8.5749658676269e+126316080', 15, 6);
    T('3.822986824776556324042E+951587350', '1.95524597551728931415e+475793675', 21, 0);
    T('1.46935015812097954189315563446110451955E+187100738', '1.2121675453999663745878e+93550369', 23, 4);
    T('1.7801406008912E+501303345105', '4.2191712466919377222549006474387482078138954638963530863e+250651672552', 56, 4);
    T('1.126793493956124882008157497788361628704252361420003374051873E+491648226153', '3.3567744844659e+245824113076', 14, 2);
    T('2.746139814091270117838120747566975847071144828338573229968051701230024536E+162', '1.6571480966079254191e+81', 20, 4);
    T('0.09485513677581853324222715080627545870051164786723478500606042255031314914559684975175881753776642', '3.079856113129613937455706604928133219982e-1', 41, 3);
    T('9.899758318641743213345978248640412098037771190865345648044568996363358296664081871028110201E-3565611910', '3.14638813858712339466003675355886541035734056626e-1782805955', 48, 2);
    T('8.353097177009631388333259568000919197283E+46481218712', '2.89017251682484023103e+23240609356', 21, 3);
    T('2.9807755404273419221508374006928443033618881101523442163219E+444278', '1.726492264803796e+222139', 16, 5);
    T('1.415920E+34617', '3.8e+17308', 2, 0);
    T('1.497570100264369505826825458575068045375900964377295663161E+1441', '3.869845087680344738668525624927261594417052567609188537916762692763638e+720', 71, 4);
    T('8.002015360508075651005958616397025550223959559683311616577155E+9287', '8.94539845982730701e+4643', 18, 1);
    T('6.5249798394490696903090193985189977812609901130496464419567155381092E+59799619', '8.077734731624373966069296955208770535887399038e+29899809', 46, 5);
    T('9.32631084414581845408974827E+535012859052162', '3.0539009224507953008644679054061521812751231180438985e+267506429526081', 53, 6);
    T('5.23857247432922544022630886367274856524768572E+83985342442', '2.288792798470238423916341047562e+41992671221', 31, 6);
    T('4.68662883645659373483958986347121196172440025544452952728E+261774725402', '2.1648623135101672638289674051867e+130887362701', 32, 1);
    T('5.7304238178517039130433540422194125734005768050958666765782524072559083281740126695479448389423332E+516266584243725', '7.5699562864336963838476152508677531e+258133292121862', 35, 0);
    T('4.0243500292198990815181163E+799574661703805', '6.3437765008076215180759371396155522455793054081319599831071e+399787330851902', 59, 5);
    T('9.3378990515289426654470849037194111055327657318642317200392400084891879006408E+4313', '9.66328052554e+2156', 13, 5);
    T('2.89343365519228289816827011385342487882716324163994369137938734647096E-49174324215', '5.37906465400099359892195675319869756994461092441579612968094932407151517967318e-24587162108', 78, 1);
    T('1.94665768824518747467620285266775492941465890166289381223731775380250636960910845918973798080253527E+336456', '1.3952267515515847055931187364481328875823127093057320022638e+168228', 59, 4);
    T('8.8819965672268358847538524480618105863620664503397560921777E+6546503065943066', '2.98026786836801565459e+3273251532971533', 21, 3);
    T('5.9804663958614851268135229106672708856620394586E+71', '7.733347526046844564382e+35', 22, 6);
    T('9.1119855863538094303135440151176116002581733087643441836863090765389100843137972451647914717825E+842445300259', '9.54568e+421222650129', 6, 0);
    T('9.35534636229432226656955057802038432363355209E+188851097', '9.6723039459553390663108877855153095268e+94425548', 38, 2);
    T('2.80430886E+321885', '5.2955725469490076659124843618033e+160942', 32, 2);
    T('2.2913329962624670495334640675819715265863539146661052739928547166214837466193123553969415672287839E+31578267997818', '1.51371496532949262162605979120909256591052124379e+15789133998909', 49, 2);
    T('9.10462141482729147936699774489112821207014194931906339034729496247520543039661604582438700983796458E+800295', '9.54181398625402437129450078695443747698334541858e+400147', 48, 5);
    T('2.776268940673139048E+873642031742', '1.6662139540506612405904246312361152845565084280384456993e+436821015871', 56, 6);
    T('5.6826699850819740766951118E+963', '7.53834861563325026667993332779488291960353581135078514540205626113129751e+481', 72, 6);
    T('4.01313940317093245E+345811132', '2.0032821576530182958579526511688426672663421663345140118253e+172905566', 59, 1);
    T('7.1639133702526594332799545526522345503587074484333715915098E+8583708350999', '8.4639904124784188186867868678091719024554245761754007943396337055784225556829e+4291854175499', 77, 1);
    T('7.89455221960027134375811716983443871654032813269E+324586977938165', '8.885129272892022e+162293488969082', 16, 6);
    T('5.476971951643984529344598118082692208843860856152324743770653830251588172437185785505662652E+316810228', '2.34029313370013213e+158405114', 18, 1);
    T('2.505611412197651642979285793847401630032219991522701539671764935487074E+17490889', '5.00560826693185375055121288291176621372e+8745444', 39, 0);
    T('9.19305000223096977929237428663624652814830274798015780884980856512E-1891091', '9.588039425e-945546', 10, 5);
    T('1.3187963413174256169925855978E-57379041', '3.631523566380129684360329633761781316e-28689521', 37, 3);
    T('1.080890274226258579458045355166939592010305992493813131456813287053289030233229951650872719156077E+931780452', '1.039658729692709587194837462368527599e+465890226', 37, 3);
    T('4.7075784320042E+541747986', '2.16969546987686730525e+270873993', 21, 5);
    T('1.5927479536808621799E-4863193', '3.99092e-2431597', 6, 5);
    T('3.3054669317248E+890', '1.8180943132095210423102239066376853475037e+445', 41, 2);
    T('3.725651743175964225852588269624089432E+96468', '1.93019474229310971508352470803377216957e+48234', 40, 6);
    T('5.59360846366395728467597516129326362470677231117358929097966246858749178522859339690135564864898908E-5900', '2.36508107e-2950', 10, 2);
    T('9.780847739987174283513044697661978957631111E+422733', '9.88981685370723853806184638285434242113392846584e+211366', 48, 6);
    T('4.11581048880424537951824741447873382174915315815132719878575130230131717330496153208E+157975', '6.4154e+78987', 5, 1);
    T('3.44709385820332477337302146316638842996844E+72649417114271', '5.871195668859388962625837770203639099007909884016872728990531299637528119e+36324708557135', 73, 1);
    T('9.320139559E-5399789', '9.654086988938933317964340069421408629111595347094875732491895600550204127911238e-2699895', 79, 1);
    T('8.44542805341E-684287467064858', '2.90610186562859492618e-342143733532429', 21, 4);
    T('8.0296315698773189913252429257453724740209295447E+5662257246427425', '8.9608211509198859756511399097198851677425308e+2831128623213712', 44, 2);
    T('4.23410796712153376066263084621773043327615989071140056314984606635173081622518784E-62', '2.05769481875e-31', 12, 4);
    T('3.256759537168732178070815362992143115319143955813405313221600616493919693429268466968009112102629649E-210', '1.80464942223378449125066690505465109794571950671e-105', 48, 0);
    T('3.36231401959E-213616', '1.83366136993448165949956925445238278000127268e-106808', 45, 2);
    T('6.058074944943386291677647111640215151942869601127532778660315085289E+660717811358904', '2.46132e+330358905679452', 6, 0);
    T('7.93198E-44739425473375', '8.906166403116438515582181968809606238324240887133947101610136610093551e-22369712736688', 70, 0);
    T('4.544638435863153191367264836772066625720297975239793554256732587889844913823686317730421149511768E+3931832965', '6.741393354391325822457e+1965916482', 22, 0);
    T('2.3212895757856323951411098362124506101201243844E+5254', '1.5235778863535767383197e+2627', 23, 1);
    T('2.9349735369665900796392192280685139516586346182393470158888352777793620709949605458599178866953E+7917789', '6e+3958894', 1, 0);
    T('9.96E+5873130049105252', '3.155946767611900009057659e+2936565024552626', 25, 5);
    T('3.613479159729815803212738438223E+569360149602', '1.90091534786e+284680074801', 12, 6);
    T('2.41521940583482199413576042895488346836154887899073051667724757023853004829219389067675523319E+82966350141', '4.914488178676211e+41483175070', 16, 4);
    T('2.2176749755261E+887416964', '1.4891860110564092993607962353285979374662469845e+443708482', 47, 0);
    T('1E+315119721', '3.162277660168379331998893544432718533e+157559860', 37, 1);
    T('3.286849593219018301624634E+40510', '1.81296706898360906e+20255', 18, 6);
    T('4.64229696425639278425726424867742665031336583857139134E+88811001', '6.813440367579650861222298517980007303951207e+44405500', 43, 0);
    T('6.7433744064868311308876341660619869652238234130669E+826373357766', '2e+413186678883', 1, 3);
    T('8.113353316763445667232494533E+86122513970', '2.84839486672115838989727e+43061256985', 25, 3);
    T('8.9412832652665631E+2794870', '2.99019786389907032519868946049526401847298409600086e+1397435', 51, 0);
    T('7.63008946868196152964088326E+118245', '8.735038333448778591937487172067949205060383803194633397470755584490211862e+59122', 73, 0);
    T('362953.56322099206315620726461306316612066342197843560187062401022579320663770', '6.02456274945320876733516849909042699544583813868763015760204089193349292427e+2', 75, 6);
    T('9.995E+108147517', '9.997499687421850577389372519935156314371146e+54073758', 44, 2);
    T('4.625047039240003297519722325769650414717426641622180163804013145E+480711', '6.8007698382168494745013747929683267242089254069230325e+240355', 53, 6);
    T('8.1153529548351980947207344989121E+47300854', '3e+23650427', 1, 4);
    T('8.40678671462177804062208891479902367829319561972808415311610980981275890862E+7349726', '2.8994459323501409345300201032970094808441e+3674863', 41, 6);
    T('7.342203759185343365361092455116000674420316709613E+1649', '8.56866603e+824', 9, 5);
    T('5.94048500018311207E+73113814726476', '2.4373110183526254411482858397875130763635182935086964e+36556907363238', 53, 1);
    T('8.12522478423323737276944069145608857E+71', '9.01400287565587443593222925607572905288720969009927929707e+35', 57, 0);
    T('1.9606406575495140521364709154279143328827260596574053331275193677055276952E+253612439262', '1.400228787573485810932547603767752204e+126806219631', 37, 4);
    T('5.1624590071431842196464491629388E-42810054643249', '7.18502540506516526516444830285217e-21405027321625', 33, 3);
    T('8.5667067085975816991005523777605075004478111183450311643861023370592741290775855024052374059735386E+6304', '2.926893696155974452424428664311730291760046766e+3152', 47, 5);
    T('1.929485205297242143019308621052366637837064524719876E+9163959822', '1.38905910792062486048399e+4581979911', 24, 2);
    T('3.2529158E+2743693399231', '5.70343387793704504169199002268370268336212251195180012355321001165536480033e+1371846699615', 75, 2);
    T('4.03825994625624323685649573E+7617', '6.354730479144055120322924825519476421756308532225058386840686866096985499e+3808', 73, 5);
    T('2.6800548489887642105251045507160139515707758273635517720265611376456168090267694194924089931908333E-541901488310', '1.6370873064649802636454191359835302708542596897545289568e-270950744155', 56, 4);
    T('1.17495375856910530863597608514314937863E+53712932587', '3.427759849477651648508414767027147061282560801e+26856466293', 47, 3);
    T('4.7540029350825964004967221065135077E+7702706209208925', '6.894927798811671675775567719431451446983287338643404628e+3851353104604462', 55, 0);
    T('2.3905276162638372483066655984554848703098337536098008670758605652619506549611156892E+2488262', '1.54613311725214567577336491287518227602755708054255240339471730062488955e+1244131', 72, 6);
    T('1.327757235526843822438245E-75315643822', '1.2e-37657821911', 2, 2);
    T('3.434648678515876976009638832956406318034416618570025582297432172238E+4643076457398195', '5.86058758019695238135640983706979279432350900092707e+2321538228699097', 51, 5);
    T('3.71021941386063161375376328396809563752274233942617341522801200E+433247754', '1.926192984584e+216623877', 14, 4);
    T('1.9547559292098657E+9177515318863', '4.42126218314393305105e+4588757659431', 22, 2);
    T('5.50118614770129510899945860763791592760808302912474288E+631812080313', '7.41699814460088368505553214234519048940057960906e+315906040156', 48, 2);
    T('8.75547738718150634152302874892E+508829097', '9.3570707954901711011362514640892419762e+254414548', 38, 1);
    T('6.8808088074879101816034333663537711198529721940655016826324212328467724437318945747269862376316E-416202832', '2.623129582671795912e-208101416', 19, 4);
    T('2.6594202738701578503458253753334384195468745203341E+526', '1.630772906897265647097015154e+263', 28, 6);
    T('9.26081683832854315498E+8900439981', '9.623313794285491943385676443e+4450219990', 28, 2);
    T('6.211761126732992777087366739903175657478654688600771792E+535818', '2.4923404917332207835863e+267909', 24, 6);
    T('9.267378529209212985344214370017226296809626757327008128454576431358287450347336549955E+806871078557', '9.626722458453454855862248e+403435539278', 25, 3);

    /*
    T('4.329E+252511963', '6.5795136598383926449644215088382691e+126255981', 36, 4);
    T('6.45153607150822504902060809187672194648109903776661357818613E-919478818652111', '8.032145461524103305263298234246165471068994713e-459739409326056', 46, 2);
    T('2.5778101891090921821872465824391E-800614276543830', '1.605556037361851840197e-400307138271915', 22, 5);
    T('8.14010535961145076846751246733031555669E+3968', '2.8530869877400252109663201016283225197519685774e+1984', 47, 1);
    T('1.743417291811941726057943692807E+5032281187', '4.17542488e+2516140593', 9, 6);
    T('8.99181290356555553333E+6571', '9.4e+3285', 2, 1);
    T('6.03E+646782', '2.4556058315617349909861111174201788376642e+323391', 41, 0);
    T('3.153642537782823186544391123180098297958642497193165193869416019885820203646870E-19929070297966', '1.7758498072142315e-9964535148983', 19, 6);
    T('1.202466900939E+16844516035', '3.46766045185943775096873906012e+8422258017', 30, 5);
    T('8.9278687245359188977676918298302675587086E-737926769737', '9.448739981889605844254925e-368963384869', 25, 0);
    T('9.043379555546065478773926172573542621871338E-43002074', '3.00722123488546573524059464064830685087135568132042382937899498434600518964739e-21501037', 78, 0);
    T('2.6662017228251556E+562598', '1.63285079625333667532958735e+281299', 27, 5);
    T('340.9262761393207161868994206563688722732818410946206964347308989265846618739407781920827054287', '1.8464189019269725732756989511987e+1', 33, 6);
    T('3.002868459588941172332092107E+179247908023', '5.4798434827912203539370078e+89623954011', 26, 3);
    T('1.0146844676971568079200117824947708465328572619235593474399807126477116649603297002246805E+346085', '3.18541122572448533763097e+173042', 24, 5);
    T('2.531091275133754743989358296070226465511383724568518166528800E+4594752763', '5.03099520486131525391449359390767529e+2297376381', 36, 1);
    T('3.7880377158737394876819344237453264339095163254E+264686652', '1.946288189316715626583300823705320647793589404932506135526e+132343326', 58, 4);
    T('93234002785152743742122242350258281644240.40557563764178455394929057934220086', '3.05342435283982012951256609109065703843573621148616e+20', 51, 2);
    T('4.9604174109620382135580673238147117760131606269922236558039327410015343850252220290243461068661069E-163909144514984', '2.227199e-81954572257492', 7, 3);
    T('2.06E+8798105472408', '1.4352700094407323747004299641472268e+4399052736204', 35, 6);
    T('8232021442.3780671063062724923195460705062590247915278736807867564750720', '9.073048794302e+4', 13, 5);
    T('4.3458277709897408549341030343583787997650816453214055486936450829593063025162863307168384655131E+1644', '2.0846649061636e+822', 14, 0);
    T('1.5814404783767253905274565798318693188665632549615677764843559525996132998E+2231804161297', '3.976732928393262607490838057087171863447663e+1115902080648', 43, 4);
    T('4.98224557848E-916750386', '2.2321e-458375193', 5, 0);
    T('4.174786376983291383095425964E+2391', '6.4612586830920888e+1195', 17, 1);
    T('1179.1084395478378970927044595914682414243927508', '3e+1', 1, 3);
    T('7.141137731959013478499749675960800049546847521436366661566264793177785245725E+1454187724', '2.67229072743947780935188062175e+727093862', 30, 4);
    T('2.442013507322327756394521622200871013959314549205316365054744404468955E+6854', '1.5626943102e+3427', 11, 1);
    T('1.170095654E+28289677359209', '3.420666095952658165557144568300416958781825379047614002874656935718931946552597e+14144838679604', 79, 6);
    T('4.74770076114E+563157863160953', '6.8903561309557868164273296829973656230761984207842141954671615594828090297584e+281578931580476', 77, 3);
    T('5.9751085053936287295E-8720', '2.444403507073582303303123522445635064529890663720280908694904006002233e-4360', 71, 2);
    T('7.45838037703971003E+261771', '8.63619151e+130885', 9, 6);
    T('5.22841619758858369128E-1638130108061772', '2.2865730247662294354726220307704e-819065054030886', 32, 5);
    T('6.96436472E-63', '8.34527693968270893595409707162316285731478001862762654042453993735087040342991e-32', 79, 1);
    T('3.87634240472063607064536688936901859839402475E+132603518', '1.96884291e+66301759', 9, 5);
    T('9.396460240911592999313425739117576116E-567032157062740', '3.065364617938882813077098423757595420995762964255102e-283516078531370', 52, 1);
    T('9.3976710825070960244640675782977303973696829590835054732E+822287014841882', '3.065562115258325021640824877151771194214395980040623804217905926100228573192807e+411143507420941', 79, 6);
    T('6.2808909117268379058750540763390524397367398042387864638472052741921561219732305316705290391364499', '2.506170567165538696322829134949056042355949435763559773609e+0', 58, 4);
    T('4.6610699365E+479291', '6.8272028946707010840119145e+239645', 26, 5);
    T('6.3131798E-8102633', '7.945552089062156541668362e-4051317', 25, 0);
    T('7.0E+624514067000582', '2.64575131106459059050161575363926042571025918308245018036833445920106882323e+312257033500291', 76, 1);
    T('9360062942763161192535396659170259467984917624012348293139737948.404310849630572325983422204226', '9e+31', 1, 3);
    T('9.3852031693896808124653684053478179813444278635924466553474659E+6380299962215113', '9.68772582673027592349e+3190149981107556', 21, 2);
    T('5.448490724061E+946000586569958', '2.33420023221252379345038845776e+473000293284979', 30, 1);
    T('1.2254237861710412202281966811956713392958356088359101735959E+880031292', '1.1069886115814567042958063378961e+440015646', 32, 6);
    T('7.509E+5275', '8.6654486323559728317522431620511098993289075624419947032e+2637', 56, 3);
    T('6.76239694977983588457853192630496783387251236921056587714037147356526288875508E+92', '2.600461e+46', 8, 2);
    T('4.779132117962486045079582668306741806304408091532107640820985E-9333297027336', '2.186122621895324069582000240126024339e-4666648513668', 37, 3);
    T('5.35844777965205417655691435805415647332110596479561974627032963E+2699899368', '2.31483212774750777964257675032431e+1349949684', 33, 1);
    T('2.999795724280613003397437752415089178175327189318031274923681905E+2214', '1.7319918372442212533181223e+1107', 26, 0);
    T('2931.6402374811011881611', '5.414462334785515144466e+1', 22, 6);
    T('3.88407343751000722579897160651467822602642559634E+42129', '6.2322334981208841304211916653831937816156e+21064', 41, 4);
    T('100241108331630338374513308152546114622765232688566077726256597209590160.300975895635', '3.16608762247083645640645567808818261823315444e+35', 45, 2);
    T('40490332.1508222078465639231043499663110600', '6.363e+3', 4, 4);
    T('7.31667967174398915135378910807658294746593682737756E+392191453', '8.55375921554025809330932939251432958474658228075017043589586033010898873e+196095726', 72, 4);
    T('9.45247333726931713195437491158890835782726856136E-619413162387135', '9.7223831118e-309706581193568', 12, 5);
    T('7.61238883971410334065783486022304901316713560634555295712885977013765940144E-9249', '8.724900480643950210774420015405956250401488371922407647e-4625', 55, 6);
    T('8.9053115664198604597112311943088615628923583911995972469169692671454248858078512751694730E+6261582763', '9.4367958367339178e+3130791381', 17, 1);
    T('9.1620455025105807362817788230705853443687695236869745080670212608401093674617069023975607210815E+9324', '3.0268870977475491e+4662', 17, 5);
    T('2.1091366589636484408928886952268232963121977708454676924297593611658765142723530326788212491789838E+833589077998', '1.452286699988555441874100243883017794686894022711839403050068749920487701e+416794538999', 73, 0);
    T('4.07779257E+8661857', '6.38575960242788970587816612890429135e+4330928', 36, 3);
    T('4.41965159674432945514444699E+5263', '6.648046026272929960260324703172587329668738576731998172413015143666478221979853e+2631', 79, 3);
    T('6.1424E+485921838', '2e+242960919', 1, 3);
    T('5.62579291007466392321876238286747877403186528133969715426492899628147E+361271677094', '2.3718754e+180635838547', 8, 2);
    T('4.51793E+7417478186159976', '2.12554228374784395866114356039230808412548944145755e+3708739093079988', 51, 1);
    T('7.9531759269111597165882579983987569677732325781657240905692812506189952535564320025751218208E+23903432', '2.820137572337768013764502047362592303217713347100899458896563140739933e+11951716', 70, 2);
    T('2.616095872363088297437955821305813555165096531957181252276598489679882317749357725668232727730E+89159866013433', '5.11477846281057249999448074192180262e+44579933006716', 36, 1);
    T('3.79443574405490988107613967000018E+5707', '6.1599e+2853', 5, 0);
    T('4.74044794672227000875927596770167E+81308922084451', '6.885091100865891962333148415899939906744838149e+40654461042225', 46, 2);
    T('8.65856792063131276287672262575369617400465645440590932011417830723916739206945132145536825324320E+57172185611', '9.305143e+28586092805', 7, 4);
    T('2.893169041603743259E+3406', '1.700931815683316e+1703', 16, 1);
    T('3.09974474642579339386269199426140147563459730949424470341069234319531323431649885357E+6731389225', '5.5675351336348057156682105526e+3365694612', 29, 1);
    T('2.966727264938024877519462073136059072134735021242E+205848586', '1.7224190154947851357396474581e+102924293', 29, 6);
    T('8.2786400710114444568907006838267329181594043E+38927541455', '9.0987032433261e+19463770727', 14, 1);
    T('1.95653065406791445232E+445834686964516', '1.399e+222917343482258', 4, 4);
    T('5.7580030356827888340388164172219595180735631397E+955753', '7.58815065459482613324735039211739934289897115913e+477876', 48, 3);
    T('5.84157075624365080059795483564778158395732902876690659E+5211248133440460', '2.416934164648191e+2605624066720230', 16, 6);
    T('8.0656776392422614850355102544613240616E+5351874', '2.84001366884778593050010090136428916402e+2675937', 39, 6);
    T('5.746209322222618452027890586016519981227344657419069550579433093299323762194376205947838E+4885', '7.5803755330607588916449539583286e+2442', 33, 3);
    T('7.2504336515777175886351027E+55526848005', '8.5149478281300805563981e+27763424002', 24, 3);
    T('3.263479158365135488429E+693', '5.71268689704340044222872769860594994810140053e+346', 45, 0);
    T('3.12404698740588257742889542178246565E-1948687', '5.589317478374155279836204899541702836094e-974344', 40, 0);
    T('3.55584254403727535836151639370324958108937703228673915679403E+832440519621324', '1.88569418094166991528678689678806379159638460060981924453996950321231727279147e+416220259810662', 78, 6);
    T('8.88433591328317068479969017061978548261302504738650384689771278929971879212385364329127E+838345361371', '9.42567552660453652049635797e+419172680685', 27, 2);
    T('4.52422314250396592373831397105246545110713288407621931707530634368091757E+6143', '6.726234565121830423584722030758811e+3071', 34, 6);
    T('2.327258214726750108068305502804158972878636364898870287910677174862562908280E+831972', '1.5255353862584604458337696339558e+415986', 32, 5);
    T('6.6964749061392842474707287915294590801006894234870582781214776E+307839036065696', '2.587754800234999901732479413759600426425269321476373690955737554389522952084937e+153919518032848', 79, 3);
    T('9.75097863675449201766558515623633052447091873051624253885E+6155098292498808', '3.12265570256384365575306474951610754576653117898e+3077549146249404', 48, 0);
    T('1.553267810009862194721356128820631740034160504683756382606701237312990536045203E-1566172972732', '1.2463016529e-783086486366', 11, 4);
    T('9.446243012187357798745030904105901331373E+648966641488', '3.0734740949270026015527068e+324483320744', 26, 4);
    T('4.35968476110136828719140056891153352802136634294663E-605489', '6.60279089559965414417038119730540269987884426e-302745', 45, 6);
    T('1.137223709108943669027387057499111747990E+323035112', '1.0664069153512385532046266486568274001623e+161517556', 41, 3);
    T('7.3812018664557937240791018245189021641582209126812971775973671824249900213437418254105111101067E+615', '8.59139212610843046955922758206905449e+307', 37, 2);
    T('7252.44794519460469866478275801', '8.51614e+1', 6, 0);
    T('7.151369552318228361725218892594953757661009957694E+1836739', '8.45657705713028334947951832840479344241593216023369e+918369', 51, 1);
    T('8.841307692118632272259E+891434', '2.973434e+445717', 7, 5);
    T('1.51936E-67659883493', '3.897896868825546543849511734174764497579635069e-33829941747', 46, 1);
    T('5.70041481399171134499057742716586421411E+378', '2.387554148913006610511644897849642773092244487e+189', 46, 0);
    T('7.6051E+1004418', '2.757734577511041036045804537823226254e+502209', 37, 6);
    T('3.556022662039441883252679393192384675400265157E-8188715120994185', '5.9632396078301615426634344985667212161772e-4094357560497093', 42, 1);
    T('8.3835684471280621472390097032E-585767878', '2.8e-292883939', 2, 1);
    T('4.5570062286506601257690432688280137E+131459069', '6.75056015e+65729534', 9, 4);
    T('1.404E-37702118', '1.184905059487889941046307662623385943965115712084920485928757116883716802185676e-18851059', 79, 6);
    T('5.6244769598865349745313694904410382024690669833292600925608955865E+86246482027045', '7.499651298484840204705e+43123241013522', 22, 6);
    T('4.07781450823862379297767381E+229447192', '2.019359925381957210483857279518754029626376602822e+114723596', 49, 4);
    T('1.58780187421901999079777111978410160001120E+4029308831', '3.98472317008223294066e+2014654415', 21, 4);
    T('4.5268914770072504040222430821369242114047412998803641567662741866514445E+706003664403722', '2.1276492844e+353001832201861', 11, 2);
    T('2.16636336372626674085424078590687049188659683589898780010938527676131544470931E+948187', '4.6544208702332310043642455519575682668295206e+474093', 44, 0);
    T('6.81601488597175840254209291834768967578885455168085208079221725843539336887548090904E+767995805', '8.255915991561298828271099882662213676e+383997902', 37, 1);
    T('3.0748021026867931726545132676163863949571217987514051790024576133239898127525534039317176695121E+997', '5.54509e+498', 7, 4);
    T('4.657777248042377115066735080622743169936598758499E+884', '2.158188e+442', 7, 4);
    T('1.402694219363617540308349623873561897774889004686283241739862179108387906576692683888090166E+3832272475009518', '1.184353924873649219886861566418316737161166140259848e+1916136237504759', 53, 6);
    T('2.7770121863909664530820027513322606216953498654911361962813977059637247333864896297470874933249539E+97570466975', '5.269736413133930604035516311755353062170487707732258884174991681213342433646736e+48785233487', 79, 1);
    T('6.629005206494727141248723635649974079353668962E+73336420108072', '2.57468545777823647947902936e+36668210054036', 27, 0);
    T('2.7172978451868970494216E+39', '5.21277070777805941413e+19', 21, 4);
    T('2.94309708607815666201997752006289377928517552E+7164270500998527', '5.4250318764760789763120945769851699469997591e+3582135250499263', 44, 2);
    T('5.511793723209526798054909347226747E+68617565392927', '7.424145556e+34308782696463', 10, 0);
    T('4.367170789837537E-83063478', '2.0897776890945929036069472992127458e-41531739', 35, 2);
    T('6.8431717788841E-835163835', '8.2723465708854945193148441468321711e-417581918', 35, 2);
    T('5.5648925124135044581885174674221119110272059306715624693856265409E+819', '7.4598207166214e+409', 14, 3);
    T('7.233883096444513528422729961329092704481183225868466695538220766390703001326708081028320865315E+416754', '2.68958790457655678542386697e+208377', 28, 1);
    T('90.01490827369062231356897382745933344273303617', '9.487618682983134220180836992272829e+0', 34, 2);
    T('3.476629564479516622341230722091946209373237171071399446882E+3530', '1.8645722202369949023173e+1765', 23, 5);
    T('6.721825990754588628407973129002459845410663502936185577483607E-372022', '2.5926484510543631797238633363973901102626713786121684e-186011', 53, 6);
    T('7.434352929934747825189925773870298892091313302930842005884800591853E-268174960052', '2.72660098473075957795809190779137676121e-134087480026', 39, 6);
    T('9.720903177375188875279300E+54', '3.1178362973984361e+27', 17, 2);
    T('2.42437E+3032193', '4.923789191263167799e+1516096', 19, 2);
    T('5.2897827645392369E-8295679', '7.273089277974825533552275353097836150747023841376902e-4147840', 52, 2);
    T('4.1336911728791794596354585E+8572435182027', '6.4293787980483e+4286217591013', 14, 0);
    T('1.372480381742864795140076850866498070764840113414674306291498E-598', '1.17152907848796685134518069676746358761334997260055404090737136804818393272e-299', 75, 2);
    T('3.54781716955232661569955367568934971997144485307722270641999600812955340367208407501221287E+4885906767', '5.956355571616193075456663570182012201047965450511409486e+2442953383', 55, 0);
    T('5.4303139746842452027304493772184846349995940317890399E+3132683', '7.3690664094471595437147689651e+1566341', 29, 0);
    T('4.35302710844940627819731461068100639965327529950280865128949606020918873004826755764814667554833178E+130123', '6.597747425030308234857310221883e+65061', 32, 5);
    T('1.2447E+52', '1.1156612389072231275175602170097338605212851972405949895e+26', 56, 0);
    T('3.36523978290495747509579740762097870835116708066739187444155160871771402926390037575E+2488371749626', '1.8344589891586449622370460045148997756e+1244185874813', 39, 0);
    T('7.740928614878256430852684606960387768629145549574845835797184721694039946052732439828189E-57793956205', '8.798254721749226800373145577129881345292427883584e-28896978103', 50, 6);
    T('8.87041E+9692', '2.978323354e+4846', 10, 4);
    T('7.46612079072686847567966144E+22702319', '8.640671727780698691805950375078546119188823804163661e+11351159', 53, 5);
    T('9.70764196E-566050078896', '3.1157089016787174817875167375779198131243150167556648328e-283025039448', 56, 3);
    T('8.7034235175372082546395117512211460222008468360503838941387246975795970167203351980407818272976765E+9881', '9.32921407061559942e+4940', 18, 3);
    T('7.26446733053E+759539332229', '8.523184458012157793663923786691630548225456415869e+379769666114', 49, 4);
    T('4.837322712565845913695859580306661138427187122558531874764958699524910927385532986136883E+2130300777912578', '2.199391441414157616184961103807135253551710350553263389862383678549166426641718e+1065150388956289', 79, 6);
    T('6.47524445894880621175692294966948072998131029656365707397951595854088E-7693', '8.0468903676816712374443956321354858e-3847', 35, 1);
    T('97157116243465631298013193551942608220453565214744832585509604225829.5047937236443188923932245028168', '9.85683094e+33', 9, 5);
    T('4.171603465860E+3303', '6.458796e+1651', 7, 0);
    T('5.42997347054988173948443067249148136075995210010184611481736794025839439585910768770465187782E-8651525161911', '7.36883536968351718615910300581584045794e-4325762580956', 39, 3);
    T('1.8068271790417491229720740389787251E+4735264', '1.3441827178779487354475783134565935824981947104578546997e+2367632', 56, 0);
    T('3.5513661359361609109402841474519741735734299111633E+8376655982654534', '1.884506868105330336989037088391696447487215359e+4188327991327267', 46, 1);
    T('5.459020891143902530E+291924249805531', '7.388518722412431249425065777379353792e+145962124902765', 37, 6);
    T('3.25941363994716081924034856160435755546E+74587207152352', '1.80538462382594829145117793037e+37293603576176', 31, 0);
    T('3.728465425E-840127', '6.1061161346636701518359576887767425541092e-420064', 41, 2);
    T('3.320622674E-2115523632', '1.8222575761949790835e-1057761816', 20, 6);
    T('3.98930304362053108480315050353707622397444757453427032500805534820925181459021589134088775148465E+364791841568', '1.9973239706218245566e+182395920784', 20, 3);
    T('8.9422062818035742884199134288609096E+727', '9.45632395902529e+363', 16, 3);
    T('6.24000728894378841355531446944868E+724369672', '2.49800065832e+362184836', 12, 5);
    T('2.978127438416551292211457109936213582468880716563934493104750806377916435232E+7433', '5.457222222355024541e+3716', 19, 3);
    T('1.75534882594714989264403361346414415306118237351068721207454945064799497280841252564395030725241608E+3996511908', '1.32489577e+1998255954', 9, 1);
    T('1.783695916981230752463805983730856974492939977413024156602664799099683125473E+5556703', '4.223382432341677902367298104102e+2778351', 31, 4);
    T('8.87990496660794118894084613551030558335639137862589191228188821712E-7552', '2.97991693954847358744451616e-3776', 28, 1);
    T('7.673918726266331138509563588429165043831184974158409818885902E+416', '2.7701838795044510796634e+208', 23, 2);
    T('7.52756494782091669552656E-73363', '8.67615407183443e-36682', 15, 5);
    T('4.77290352105E+5976378', '2.184697581142525090061809932196485772484998111843375101995e+2988189', 59, 4);
    T('4.96940508732287508E-29760771783118', '2.229216249564603160373164692309004085073077472137e-14880385891559', 49, 2);
    T('1.48768876388434692002292561916566577742072767021983E+855047', '3.857056e+427523', 7, 1);
    T('7.56174849177978888504888103247130109176199821734007664079416043195277724414E+9561492103150', '2.74986335874708307024373025567376258998490113912328013323e+4780746051575', 57, 4);
    T('4.32865926249672042413712573631195467957933806115425E+710153329334', '2.0805430210636646251625557114146465798400018554133602756635e+355076664667', 59, 4);
    T('3.0989773154475429111421459887304038048323375767689428267885214048766937600567586259947E-8316353465771', '5.566845889233456147538e-4158176732886', 23, 6);
    T('3.2321277417391405440016173241039471228316201254502708785045011327122178444195E+528601017556846', '1.797811932e+264300508778423', 10, 0);
    T('9.9015819607859272571617905598813236624677237368306306045069981679072E+857316691489', '9.950669304517122408600872242057581554962799667184754557108012048324142260635791e+428658345744', 79, 2);
    T('1.297927950326404657452058414726604408236532715493397538E+32995800928712', '1.139266408846677379052e+16497900464356', 22, 4);
    T('3.58584155470178406345E+846', '1.8936318424397557810122140027539570921408929589e+423', 47, 0);
    T('8.559356715700169996493033357771519833746298438225177649644974058175660416334722123973972655813524619E+554', '2.92563783057646594e+277', 18, 0);
    T('4.428786398014', '2.104468198385045684556298488860455443e+0', 38, 6);
    T('4.026574638529964582446784533669426543535627081242383E+833110', '2.0066326615825739076583763867543956233e+416555', 38, 1);
    T('2.087921523882633635943837536313E+860852', '1.44496419467149207141285603868142349482725644196310739726983029277865999164e+430426', 76, 6);
    T('3.99084760515217114444685991688972674530037161183381636508167813171111587088390890268180E+992', '1.99771059093958130430795956522528126295266995415898e+496', 51, 5);
    T('9.222490893179333326709289638994192646552201802447348438632257231645239377133073701E+6672147859702599', '9.6033800784824367622441273931831009720917612331213e+3336073929851299', 50, 3);
    T('4.751521802397E+870639411718738', '2.1797985692254e+435319705859369', 14, 4);
    T('2.10566224415798753394812095933914024704812743036621573456011903500386988547252730E+1037668889378285', '4.58874955e+518834444689142', 9, 1);
    T('1.80846660568316378083526649858E+48732907117413', '4.2526069718270036491303971422339333523715995e+24366453558706', 44, 6);
    T('1.7535151519208041882605504594157400874311079113312332961856444331316397791053136748573540584527284E-801074', '1.32420359156770307721445518853640475347557514413583382287e-400537', 57, 1);
    T('2.576153525865310642876327089331874288722798810813E-822025306470260', '1.605040038710969553511801362088857782189058840540255992479e-411012653235130', 58, 0);
    T('4.633359345706483312715957308354742344158394873554540E-642', '2.1525239477660831155271402814407278185751907e-321', 44, 3);
    T('2.29780328688691101923813525929034820331342822047872782341E+451189949', '5e+225594974', 1, 2);
    T('3.3222989683604470777654999901936411400063616E+357341468099087', '5.7639387300356056766805475860535848990450024143480841898059e+178670734049543', 59, 6);
    T('24929.097924417083847889948918564434946874328899535943954179033510818417435301811006302', '1.5788951176191876440534971751731e+2', 32, 2);
    T('2.186128865792833884047954068943055418405233341391790889823753382246698910636885748433820634972E+65492', '1.4785563452e+32746', 11, 5);
    T('3.89686938563034305702826391458464442888921570289255721237539344E+602259637255950', '1.9740489825813e+301129818627975', 14, 3);
    T('3.00705910953623150247936334583872388434402667459395529157476809856906510846714894173375331529E+799284', '1.7340873996244340113244331093084e+399642', 32, 2);
    T('2.61885019490383758099264154284725562978447846E-60603822551596', '1.6182861906671012500414458412452978990010046159238e-30301911275798', 50, 1);
    T('5.06254732830811988341781306314505E+40997', '7.115157994245890168641957823011043705411897740939803886763467275243998072842822e+20498', 79, 1);
    T('9.7132347535170698505500573281889966762760676684E+175921942', '3.116606287858168e+87960971', 17, 5);
    T('5.37778142843368211343450575950790738444360325183445455249437188183545129', '2.3190044045740150584692491321716e+0', 32, 1);
    T('4.64740439271728553583617820E+467683', '6.8171873912320215285180759094013978592343e+233841', 42, 6);
    T('7.58094200206364727568702458715880081694064281998007797204504326133163948323795579795438E+8539222864697785', '8.7069e+4269611432348892', 5, 5);
    T('2.799519541714171651988999408320779185711E+4293', '5.291048612245186745208255662042476597856e+2146', 40, 6);
    T('1.666809542218113989806568231180486770816765930514098971646315563765720399182E+57159674183484', '1.29104978301307730393166303460134104e+28579837091742', 37, 5);
    T('5.0826788009973165382524771048790810101963036246481E+140497681333', '7.2e+70248840666', 2, 0);
    T('6.5309134321611140939334433427628456941477231871081253447617E-975', '8.0814067044e-488', 11, 4);
    T('4.432639950601485517253580620918207807917005377517692290624386050130244020800E+452029', '6.6578074098020329543068455368553e+226014', 32, 5);
    T('6E-1997127635572', '2.4494897427831780981972840747058913919659474806566701285e-998563817786', 56, 2);
    T('8.1469702420491963386597528965177369861695145902318771646722502E+95635280324009', '9.02605685892194916803545016493935048162928907256211634883684102484703770198e+47817640162004', 75, 5);
    T('1.0934954151082519195279025796097360088039797510565234511447756554205437486339700240E+28487165', '3.30680422025291953861348609006628e+14243582', 33, 0);
    T('9.941799123057238701812231286659564273731957105669569445627828445768193484866689770271696536864884222E+835690172638', '3.153061864768472744e+417845086319', 19, 4);
    T('7.5023147683505541541250119872195566023698695677352213800374623905413369841511966E-1674378725', '8.6615903668729071074433302896e-837189363', 29, 2);
    T('4.4065341597813187054468466340777901849998082490419E+87', '6.638173061755258806960625764505532682529795606e+43', 46, 6);
    T('1.12203631260178705186955446414632382408810631436241175375338586430995127271542102671879E+702517725277', '3.34968104840115643489986067416793529976e+351258862638', 39, 2);
    T('8.981254811971818909806227487571025642E+960394222', '2.9968741735301165142586725426862391450727747876232901310290372271512617e+480197111', 71, 2);
    T('4.496254075403679753365620408686201279E-255197545930102', '2.1204372368e-127598772965051', 11, 6);
    T('221.371', '1.487854159519675e+1', 16, 1);
    T('1.7104923635145373910840534514778734287247091190333685450285955105988149444300425223247314E+1447431', '4.13580991283997165174052880153876642742757302179968073782e+723715', 57, 3);
    T('2.672807822848153313979781643621040535182092068998131944795464352227228300326304885420740971855695105E-78925', '5.16992052438734624111291070648827098e-39463', 36, 4);
    T('3.9669925596649681549223089076714473753749938563979647576337583348805254579930205126335238E+507793266074360', '1.9917310460162456771088672846963097950517800953e+253896633037180', 47, 2);
    T('5.68167532092587667828527212815212445530496983060587127276393925800275449142E+92480980768906', '2.383626506172029602832348673316328029753100432781200130821528397347375762e+46240490384453', 73, 3);
    T('8.973864391948024471963925175016486621607880458031889006325037037864247422321206624702135076284793E+155192301733749', '9.4730482907816027199323159440137970573246766619e+77596150866874', 48, 1);
    T('6.7611139400251993641007649760E-3248338', '2.6e-1624169', 4, 5);
    T('4.33056247497997215184362137260600762461768628995470710E+72806128951', '6.580700931496562207104e+36403064475', 22, 1);
    T('8.72885015027221856E+2335756', '2.954462751545908144824328553095787519439101e+1167878', 43, 3);
    T('47208456951539087203132203.07485981723055934780467996337681443374375788833', '6.8708410658041e+12', 14, 2);
    T('8.187894682236643355221928174908643963235174310308734572394E+9949', '9.048698625900104180804733907953768242725085915678874371740146002934365423962e+4974', 76, 4);
    T('463295662928700015372972769.054445530138372959998817560582194105722410759943784470066247935862354530', '2.152430400568e+13', 13, 2);
    T('7.22835822753284883536075091783210532444625878628990861122891660315134849210104186166255E+819640691374', '2.68856062374141915783965726671665954638681299753299081307e+409820345687', 57, 4);
    T('6.7127000213378642882984604094608044513172438342703332396829226841947922081519948687884E+169326356701', '8.19e+84663178350', 3, 3);
    T('7019038165461664316006.20649853712404300416474806909160351375675931208714368143075227980023914793117', '8.37797001991631879580628186835144163484474709e+10', 45, 3);
    T('9.99068956932914904382476E+1040305821395', '9.995343700608373298752972443026867927e+520152910697', 37, 6);
    T('5.0574700473169166089663119611022429708602131505687933786282688441248E-69181794060429', '7.1e-34590897030215', 2, 3);
    T('2.475279984132373278674905714514819955189600656348628109708137351198346779202536271872348063E+6307993519810', '1.57330225453737060316515e+3153996759905', 25, 5);
    T('5.100737394147617169849739894994730195860194E-985971', '7.1419446891e-492986', 11, 5);
    T('4.9288270963332588011665E-858', '2.220096190783917092473361253804077646897091461159052316e-429', 55, 5);
    T('1.49413024907896430659116078764183368721970630815716815241323E+694', '1.222346206718442067171189707836595282521742930027652647e+347', 55, 0);
    T('5.31299373646522643991323345649249592E+12013', '7.2890285611082814853111267487091e+6006', 32, 4);
    T('3.2544968705222E+969973634494731', '5.70481977850501426337476848806669173338312417931626235745756129205884052254309e+484986817247365', 78, 1);
    T('2.750513946178587999633E+21155302', '1.658467348541e+10577651', 13, 3);
    T('7.8858164894229027351700946514663450470397825436554797730064875413276336340641545045731628067243E+451626978100487', '8.88021198475e+225813489050243', 12, 3);
    T('4.814829680430762915471867456778713234116866202536062800210200E-650361687', '6.93889737669520726820411772451203494106195447290242589088479902680591111880135e-325180844', 78, 1);
    T('3.6856284004352644044199142235483488261071268729175E+49048785730', '1.9197990520977096275e+24524392865', 20, 5);
    T('1.70021369312960827099766838969893192760432237224473535E+814', '1.303922426039834729783258434425464322463544818722635466e+407', 55, 2);
    T('6.865760929572562543645498393954900957952301052324948710458710202168128392137726439603832E+750142664818', '2e+375071332409', 1, 1);
    T('6.23718101178853222976344643697428299338755978712902616163530468523E+6147638', '2.4974348863961463109006e+3073819', 23, 2);
    T('7253612112467960333775.095134174843110508686632363479501946986912334', '8.516814024309771397308075880979e+10', 31, 4);
    T('3.837790680569270744048747112830789351045672010566129867148186627749004924079856E+92147019932', '1.959027993819708219038405937793986914281916e+46073509966', 43, 4);
    T('78880762.7489810749439620', '8.88148426497401812201270194684155380625820644479831812515070317825984873818703e+3', 78, 0);
    T('8E+1576976992', '2.828427124746190097603377448419396157139343751e+788488496', 46, 2);
    T('9.3405134040821894938672715302859736399163773864351E-9779', '9.66463315603970014124099151692856978852627858284193937476e-4890', 57, 2);
    T('4.36820226280870557134804007698132598503272964610244727726E+48301479851', '6.609e+24150739925', 4, 5);
    T('5.045822751804061773442273079876009481865E-6632795636', '2.246290887619869630459404236792e-3316397818', 31, 5);
    T('8.7197611047689335324040729543554570938314269733120737366222489994292341E+786620508425626', '2.95292416170292689289793899613737938011e+393310254212813', 39, 1);
    T('6.8008629078249797213362418760901675912747E-5629005693', '8.2467344493593218412017589736833e-2814502847', 32, 0);
    T('3.79482214352232373425342861726801201198861577696390726965813721E+522554889648217', '6.1602127751582767339972e+261277444824108', 23, 0);
    T('1.05081728855004804589870405917161510696294761985834335416124604969631706525907321883451560187525052E-605', '3.24163120750965137591718897632482398512871e-303', 42, 6);
    T('9.4255720670556111819124129285982223421275783550244183472009391595968472416232E+898354', '3.07010945522396826188437166750244021046622287516361266211e+449177', 57, 4);
    T('9.98336109133321921018889917282943946795316643556177369E+39106459', '9.991677082118506500882678743157110886592634092767427768787445395917202671495e+19553229', 76, 6);
    T('0.000081335645264', '9.01862768185936369e-3', 18, 0);
    T('3.0135270602533805287696958394866783352157620362630376010874590867560801583065403009586E+4429183984068', '1.735951341556951742191574653231149552566517428835442656471e+2214591992034', 58, 4);
    T('9.35639132035129188259665078299868742281685004317502592829316527353310E-706917397819', '9.67284411140347691e-353458698910', 20, 2);
    T('500176869.3892547050840640228960967613849442166473092148357125890035945519472639065525956475390', '2.23646343450827450608394280937439e+4', 33, 5);
    T('6.3619534371975659328526174472449026760641935091779240708339646739132953822555982438999687187673E-898889272309930', '2.522291306966e-449444636154965', 13, 5);
    T('6.191081536497963606661856103568522284089859E+83', '7.868342605973613555600292628592052692856188e+41', 43, 2);
    T('2.73E-8', '1.6522711641858306061817426313e-4', 29, 5);
    T('4.705912875765171E+46', '2.169311613338473555870240863357364725634050227045254e+23', 52, 1);
    T('2.82764973274334763324303473637274665018217377050961083097662582487530081376857548879702E+979', '5.3175649810259466477998526347670237247119908476113179e+489', 53, 1);
    T('5.493727969335560299508163059368420240932991557713546725864958862060165E+40757878944417', '7.41196867865451809985734e+20378939472208', 24, 0);
    T('1.9765050813321940053371186282464759859505081444509518827022E+70941917524', '1.4058823141e+35470958762', 11, 3);
    T('4.262168677142976128407312612135343941031094271118E-9757360759333', '6.52852868351e-4878680379667', 12, 3);
    T('2.224801E+8092445', '4.716779622e+4046222', 10, 5);
    T('1.33936944364546836370819298853733604490E+1310253457918775', '3.659739667852713e+655126728959387', 16, 0);
    T('7.1545460724327251648098003778943448556218979508158871076656859637603343391493189227911E+1689897618', '2.6747983236933443922547789655585177472869730772776895e+844948809', 53, 3);
    T('8.17543941355E-3728666702771', '9.04181365299573e-1864333351386', 15, 6);
    T('2.73478681123828660438137122015319043312826513840909227562138562671542614916712117559915071185E+5698041290903', '5.229518917872e+2849020645451', 13, 1);
    T('7.068251289777989822822963643974642E-8262', '2.658618304642091684e-4131', 19, 2);
    T('4.60610024199462532755182449958226237035E+8888', '2.146182714028473701413e+4444', 22, 2);
    T('4.514E+28665792', '2.12461761265409828099743612564861838550264988450631207201e+14332896', 57, 5);
    T('3.899735938138114710530E+447222907', '6.2447865762e+223611453', 11, 2);
    T('2.569504304226979436533498545820153350758467607941525153430924280323226746187650359879411361322955E+54466071562321', '5.0690278202304033483566381132297260870855e+27233035781160', 41, 6);
    T('2.801395089888687353677596319265480764896483254235136137938350581071379057414197E+2761876681321', '5.292820694004934221636787120500142027935693757597946e+1380938340660', 52, 5);
    T('3.8990217212036756205069075949570236979E+5304078601762285', '6.2442146993866855831120703e+2652039300881142', 26, 2);
    T('8.07031906991904623141096276072567676939074651505659E-727851', '8.9835e-363926', 5, 2);
    T('4.148210923695161524586428391826974226451695856553829E-12404914', '2.03671571990181327921e-6202457', 21, 1);
    T('3.727624179328034429136944666466971711804735025143711698444148931784308965593887027261271549596773E-9531133534', '1.9307056169514902069440116686739397666743088822356477e-4765566767', 54, 2);
    T('2.07920431305818215736772652632881893957021564241500898368923770681890859768258075131442507200E+66532896', '1.44194e+33266448', 6, 4);
    T('5.0156842584380418054509416133289827629724487618942424045959410E+3736483', '7.08214957370856564250421196e+1868241', 27, 3);
    T('3.3568874308208175469172664604275888294929129246E+5468253059', '5.79386523041468194187e+2734126529', 21, 5);
    T('8.0750561809547436881052466E+8338880053', '8.98613163767076343851708381164173782319091078314801838382640295176985978e+4169440026', 72, 0);
    T('3.28674867266196278275213892147196250181879388631610795298077761118533891192159952573802E+476522', '1.812939235788657860650844621470613009451513e+238261', 43, 5);
    T('6.28106175972573536684326700593228902208113068552662559891129567970868304E+75', '7.925314984103619440724927348252051724923130325759136218e+37', 55, 4);
    T('9191498140372.086196019854755672794484103397782143354518996290950862028', '3.031748363629819e+6', 16, 4);
    T('1.788586951556699002046E+18540995815609', '4.229169e+9270497907804', 7, 5);
    T('8.4364811584957230849590643832818888891955182123145402539687503183600507799194885E-910', '2.904562128530860516108273703918366687e-455', 37, 2);
    T('2.85740660288925763427548226132413405622133132E-521496161725006', '1.7e-260748080862503', 2, 6);
    T('1.5812287164408569E+971609718', '1.2574691711691610692264e+485804859', 23, 3);
    T('2.83524801635875456319554650516424773375466874774506685856574693901210971009E-21301318707564', '1.6838194726153853822450769640711603100483516395224474019e-10650659353782', 56, 5);
    T('3.04225234284577288905567793643664821307309710618177447201100863E+208', '1.7442053614313232264712e+104', 23, 6);
    T('8.2539957512607254823555498758340753490155782342621E+9011301467', '9.0851503846995980710155992337711427177981207099440032785e+4505650733', 58, 1);
    T('5.898204679879034954437833913578922482159988009231282578305818738833792477826E-1007429', '7.6e-503715', 2, 3);
    T('363235536.7304619330946439554594623', '1.9058739117015635698720281480423284094740658e+4', 44, 6);
    T('3.5334195141124906837688973068162966954890030101361355265320602107491338167645198E+3549118952471545', '5.9442573246053965591515893192454922707988e+1774559476235772', 42, 6);
    T('6.7699111529335071173980023852066799873443362877570246983696077732195050171694852E+84', '2.60190529284474670661540846e+42', 27, 4);
    T('0.0000401421847212714101248539995025719310', '6.335786038154335442287719823551417165e-3', 37, 6);
    T('8.64763373236653179391267464119E+7320235686362', '2.940685928889131096818395176450226748412831922565204863699144719971361698207e+3660117843181', 76, 2);
    T('2.86730444014167033254232631E+958175712', '1.6933116783810564425804197098225e+479087856', 32, 1);
    T('9.021277533822237321163807773769824289458479102380601374506064399E+99236', '3.0035441621228473825164812566486035281290170920313664e+49618', 54, 0);
    T('0.08859211', '2.9764e-1', 5, 5);
    T('9.44770E+60214', '3.0737111119947495807471922938959771939310896e+30107', 44, 5);
    T('5.2424634253076384810050331320247398798370076452438385800190769525363262141314549921306E-72359365', '7.2404857746615582180973063565275720984181372548627058338e-36179683', 56, 2);
    T('8.12381377871916927096006225480301033505954853008894585481988455204905979305333063476202634952205E-8291', '9.013220167464661057046644794896553982192e-4146', 40, 5);
    T('4.404836375602923939500633445733846509116284126117164252612395906736E-7483882943787659', '6.636894134761322181308935957301007985044947779644098376387e-3741941471893830', 58, 1);
    T('7.92977118676007580038918588260245063406066718758255517791401817694423324352308261E+98591911616459', '8.9049262e+49295955808229', 8, 1);
    T('2.4065002085383506183786201347371965886643646221271889986551283027E+6102778754312396', '1.6e+3051389377156198', 2, 0);
    T('8.1026916491875830175144475286988131E+19767355900', '2.84652272943456100861606790607316433887172609704588187e+9883677950', 54, 5);
    T('5.28263456818133557702009012699207329575886864186136243079E+2193570941564', '2.2983982614380249258786365509047625082940819136e+1096785470782', 48, 5);
    T('4.2E-6428087538690', '2.049390153191919676644207736104210398147006532690966585908395699978069597141071e-3214043769345', 79, 2);
    T('6.9383270998276500462796722008093E-23825333107639', '8.32966211789388852535839811858274853741622269139e-11912666553820', 48, 6);
    T('3638879626214.20897890573863724680610997633966033136906438193309413333913706808317040545224426569473', '1.9075847625241214897666065811264954481834495349144677e+6', 53, 5);
    T('3.1677311519605E+7935328577', '5.6282600792434068184911922867017172690869810732387544957234402324502887029e+3967664288', 74, 5);
    T('4.33146976868921145294190069E+9915', '6.58139e+4957', 7, 3);
    T('9.1627114503452389380960608E+55477498', '3.0269971011458268224e+27738749', 20, 5);
    T('1.340564402743792544923897356993101351E-807', '3.66137187778541903319782484255717786897061392837527210757808814848636889809e-404', 75, 6);
    T('9.10981314589570703485071516968122152431823450674509397046063E+7464428', '3.018246700635272298060848418553345902928748700697e+3732214', 49, 3);
    T('6.3841442196049094191E+1176283648603517', '7.99008399180190684627290956157738819146225e+588141824301758', 42, 5);
    T('8.93014359540076391142846800844994135360694943415628190410741858680809676822E-82', '2.9883345855845466072680033036497163e-41', 35, 4);
    T('8.0437158619107625611685198900698872282275377439384954E+7091466733', '8.968676525502947673566153203320780179470877028990393506959749137498028528e+3545733366', 73, 2);
    T('1.603E-44201700608306', '1.27e-22100850304153', 3, 4);
    T('787.60923373553095931646199111632429951847913833840286825', '2.80643765963e+1', 12, 0);
    T('7.841993479136004752740284647810503358227177506407665431431883736836563E-7254956131724802', '2.8003559557913356013189531e-3627478065862401', 26, 6);
    T('7.76043163702607930307039E-76330', '2.785755128690617577962767257447338904973317239943409384619592683895246326e-38165', 73, 1);
    T('1.04521756836856149632578946018008025266451214177041625309588893973102846304466481937E-60', '1.022358825642230906997515731464062312825753529363359659400022560615028e-30', 70, 0);
    T('2.705468463049769475659774049930174797658234079911001967154430056716E+87', '5.2014117920520092923066756807951946464946e+43', 41, 4);
    T('2.90344960994671641258402352058762596060175378279926617113769925186E-20328', '1.7039511759280887366748272049065458723915911829602237885553732554862064e-10164', 71, 2);
    T('5.7410483874865620278310807221051086440566767629489449058E+8078568', '2.39604849439375121087929881982517892118423e+4039284', 42, 4);
    T('4.3526429752770101859880263126367254649769031862222048738422141546237545770667096E+40910413984', '2.086298870075188960338739226964136965785943240919251053423740869456378797321e+20455206992', 76, 6);
    T('4.0237489E+2949140569493', '6.3433026886630595345445032035691520482321478392575032e+1474570284746', 53, 6);
    T('5.5108398760300881399724338795416828419078225305632691907042074139E+2577820', '2.3475178117e+1288910', 11, 5);
    T('3.93813080739008091786451786429100337348504E+9558537603162', '1.984472425454705308555184622e+4779268801581', 29, 3);
    T('5.6999943263392575883965503960727873381905191073645578590623206167509492315785215645306094534117E+35608', '2.38746608904488056780825e+17804', 24, 1);
    T('7.703784499363921452706663380317426471905102030644009048325098677E+11690', '2.77556922078408e+5845', 15, 2);
    T('9.70035733226236547791618454388097E-40', '3.11453967e-20', 9, 4);
    T('7.1172094011697609247060203355332134620576640593528391571185836776511545911124949562400E+23938836254', '2.667809851014453564536637478291813743942709805031964992328496958506823412716e+11969418127', 76, 3);
    T('1.1222341377007369695E+7621', '3.349976324842814421975e+3810', 22, 1);
    T('4.997242785608127004016679148980463405639555035812363121E+40600596753234', '2.2354513605999409004490149284547368914474613551870719511777e+20300298376617', 59, 1);
    T('19235257536528238363406397315462414575041630355324636753887837497364802642.7542', '4.385801812271986186517223869235593320942679921852913373479194286126949122e+36', 73, 5);
    T('1.58980953668090935212832648433492991090440979997E+27107623807', '3.9872415736708371055391333848812146e+13553811903', 35, 2);
    T('3.1545862169286323414449193774E+5542208221193875', '5.61657032087076458197811389787535683886317561735314663939e+2771104110596937', 57, 1);
    T('5.63680115133787778567145160224542353475E+38023', '7.507863312113425647346258629180695819272248756364652741096790465014497e+19011', 70, 4);
    T('4.77139889922903367524246341E-18907027422192', '2.1843531992855536537616e-9453513711096', 23, 3);
    T('3.494429773863E+6797', '5.9113702082199182798966763637828388156255508041511961437802810873633202514e+3398', 74, 5);
    T('5.0335361205870301090310E+378629646589729', '7.0947417e+189314823294864', 8, 3);
    T('3.0408413729942384373254625634583668099820655862030835726240399337699E+7096929727', '5.51e+3548464863', 3, 1);
    T('7.72968181319575157091652016268885786614373904501612252005359942724236082876390982202556622361E+8594854441', '8.79186090267342182938870298488437978646315847167175186398091003384935265682e+4297427220', 75, 4);
    T('1.02730736771815843662081046805414390E+1151', '3.21e+575', 3, 5);
    T('1.7540165611219987670332071044150127630490185073856864010589228211284246933586182216168E+656805030', '1.324392902850962414139044545459841947610198470457326682431046297483609296e+328402515', 73, 2);
    T('5.55456883673708191E+544', '2.356813279989970251270796344999470861278326116542800202831891779615152730901e+272', 76, 1);
    T('7.727558454207525373361815938416139504931007992360204830332343029064752422038182741125887138997E-88837', '8.7906532488817493e-44419', 17, 0);
    T('8.7893902269751379455988427431475343355768821713541034678615334', '2.964690578622858231583533970825094162768193402169397593580952743228815501e+0', 73, 5);
    T('3.66398272E+69573741', '6.053084106470023269471739116634297838000849703637300553302160887488652e+34786870', 70, 2);
    T('2.0041098636530993528309805427581623336130861932844434862527574237304E+251', '4.47672856408907078248109212628262325310989530507321760524035503344910224553254e+125', 78, 6);
    T('7.38960572907615744230022222027453764879067165556213572E+95776790107', '8.5962815967580758645977e+47888395053', 23, 1);
    T('9.5401148370128330319768313603E+650226888582443', '9.76735114e+325113444291221', 9, 6);
    T('8.597480674329885668710344632653028917402395E+4982399190870', '2.9321460867988630400170591084164234808496572109644469509e+2491199595435', 56, 1);
    T('9.4006899373824719292459063048044084152659676292640975365763894888063671408607888140553E-9311541367', '9.695715516341469024406473827146011692780584089697e-4655770684', 49, 4);
    T('7.4480746903718742889057107431465368791549E+1663916140352288', '2.7291161005666054823729684806175e+831958070176144', 32, 6);
    T('1.949502075849713309120933870560801900774221699324793056865691844494915063481863110312752318240E+913467376768', '1.3962457075492527039222815450328415684257e+456733688384', 41, 2);
    T('4.851E+2067043080173059', '6.9649e+1033521540086529', 5, 3);
    T('4.09419166691E+5686135345990', '2e+2843067672995', 2, 1);
    T('7.486422679711662941085867384689597196592218488608457353968128559536748E+94', '2.7361327964e+47', 11, 5);
    T('7.342274185019083039960987405342388206800797856934422665898E+9227070059', '8.568707128277336005568573027873e+4613535029', 31, 2);
    T('3.117416205029998131772801163983223422048219432254880384874822512005747E+23402707516', '1.7656206288526417564730114829350082794197906773630101662872276369059501932e+11701353758', 74, 3);
    T('8.26405901834927625095914139E+97397984772', '2.8747276424644607105954522349e+48698992386', 30, 4);
    T('130385094874449498.578595529558962721089960247654375797928893898211616853730965', '3.6108876315173461595033370767254734077171083010576328e+8', 53, 5);
    T('2.95314757468178314175487582713726034977544464402808248641031775245993180663444673573048E+499461784', '1.7184724538617961628776e+249730892', 23, 2);
    T('9.09577592120205E+4570273955', '9.537177738305001307290671266849728784949823072628235935408610478767297940993826e+2285136977', 79, 2);
    T('1.07026041422490668351742591802261196110701426330783070177037742780008665733759550325645014929588907E-91124727487', '3.27148347730033122892e-45562363744', 21, 5);
    T('2.377924234473342034627735274E+54571375933995', '4.87639645073423253412243126026352439157e+27285687966997', 39, 6);
    T('8.485816604795279141650361085703271118975085023581729368435363779997336439407500048968421E+89782', '2.913042499654833244199358745434256e+44891', 34, 0);
    T('4.6468821855093480548468531331171039098233841540E-4398272995420001', '6.816804372658311848e-2199136497710001', 19, 0);
    T('4.7994306806419534485E+6151840826578', '2.19076029739493714618473950063635229867786375e+3075920413289', 46, 1);
    T('8.649511300391069725015792557217634397236411793706594530317145014892555740246012201813398E+4230885129335', '9.3002748886208034087611652484656570479633468264494393263073130764625421791e+2115442564667', 74, 2);
    T('3.00E+34170003380892', '1.732050807568877293527446342e+17085001690446', 28, 6);
    T('5.941663661525897326622791298E+71743', '7.708218770588894830101008917e+35871', 29, 2);
    T('9.77855391016454941620651220671547093115853952765682831360886494370858779E+3855640370', '3.127067941405263e+1927820185', 16, 1);
    T('7.8307742429867576E-3098825044191', '8.849166199697436804831875319108011288891469966e-1549412522096', 46, 6);
    T('2.11385546770629744705E+19829', '4.59766839572657463207197e+9914', 24, 2);
    T('6.16374045235224501536548805492290202176388341467569E-86711604', '2.482688150443435176336756853515833508214569464812194664914096635938360792600967e-43355802', 79, 6);
    T('3.3046775166936497651502232197423651768542815245661331609964154E+159802467', '5.748632460588909283926930514942707061567139593e+79901233', 46, 0);
    T('7.92887295E+66', '2.815825447359974406977227e+33', 25, 4);
    T('6.6892000042401144201598596266186680066920907852847696615658174423807984290148924781657444857360546E-2272680684853125', '8.17875296377150299618380706145960216683971289717235477916e-1136340342426563', 57, 5);
    T('2.898887719320515406144738364059891078321247026823445908006510598601866511E+668', '1.70261202842e+334', 13, 4);
    T('3.814E+87', '6.1757590626578e+43', 15, 4);
    T('3.4967747742818E+19168222', '1.8699665168878826745825175931054465916500019e+9584111', 44, 5);
    T('6.5197655079351113826189602016103106617997591322664535355739130668153937367571867E-572822', '2.553383149457815189687649468720914668601987211186e-286411', 49, 3);
    T('8.0440387647072333914939504416463269656484615932E-940450', '2.8362014675807558702170068831811e-470225', 32, 3);
    T('3.694320834974773534604675E+1417600', '1.92206161e+708800', 9, 5);
    T('2.3155388279715034232424116900215977238571382025712135269924578721E+28765', '4.81200460096569673182e+14382', 21, 6);
    T('1.111023283355108953122749140378807969902982506214512949999008949118286775067642020768028753750494928E-6373216317', '3.333e-3186608159', 4, 3);
    T('4.2115612172786238731062348191670E+1228455', '6.48965424138961649281420840338704540018713006051832175269e+614227', 57, 4);
    T('4.4722317389669601784280990988678439839397782342354283E+30560234', '2.1147651734807248e+15280117', 17, 6);
    T('9.400403235763906105204929493793510519571E+15325', '9.695567665569616406608041143883762887295670798255908e+7662', 52, 4);
    T('1.95995288389211343444354034753219868E+344', '1.3999831727174842848499455565332898391228143227118379110342e+172', 59, 4);
    T('8.456565220331331586333372280630466358437502752713570879878160716571277430E+167591', '9.19595847116076e+83795', 15, 0);
    T('1.3102080608519253120596064717781581172007903863863696506130313948748834777042E+81437', '3.61967962788411056369083644718813e+40718', 33, 5);
    T('5.14537372E+4284846545268', '2.2683416233010406054056e+2142423272634', 23, 4);
    T('7.2931574597255953490340314321232941591435966428075820238550971794023236E+404961', '8.5399985127197740917957e+202480', 23, 5);
    T('6.3827663642349191624235029E+5516063', '7.989221716935210430913043548453164458407123016349942582039396254862092727496672e+2758031', 79, 5);
    T('5.189591003754707754241949939173826592691546384378460E+78117241355', '7.20388159519207e+39058620677', 15, 3);
    T('4.519265155994255660984831586639966159403451643899E-179456', '2.12585e-89728', 6, 1);
    T('375004.3705725316464418509224362', '6.123760042429256e+2', 16, 5);
    T('1.43212255951664719708413408062784530798489999993982226964724042837709628228E+132', '1.1967132319468382969635736065622913245957072707441287379e+66', 56, 1);
    T('7.625375363551E+533118788716', '2.761408221098611852251e+266559394358', 22, 3);
    T('3.179616E+400', '1.7831477785e+200', 11, 3);
    T('1.3159951E+899568217', '3.62766467579350558694766198084210284020061330431e+449784108', 48, 2);
    T('1.40131776195920421345446149720574104371001856154125431941116245730119694480E+2636130727177', '3.7434179060842301493411620516e+1318065363588', 29, 1);
    T('4.730723715169418874982305914349736745988840912677507761249710995611677693512646E+7298', '2.1750226930240104681017526771550037026e+3649', 38, 0);
    T('7.17871128454675616560261345581300845528497953141294613798533E-7976453304469', '8.472727591836501482083728651096308446e-3988226652235', 37, 3);
    T('3.1675441035336587537126737094384194342646586602738604959513643074687530591867E+2129921416', '1.77975956340559068049e+1064960708', 21, 4);
    T('1.078923456769907907622816436887873E+6640978', '1.038712403300310989226e+3320489', 22, 3);
    T('8.728589228747665140470255749757433296056155301961876097820065228942195630883693748248863194067E+2406575997131', '9.3426919e+1203287998565', 8, 1);
    T('3.46357240109696868505616084108008111E+396919380', '1.861067543400015529123691889977787312198521194799151827763944507910984834101e+198459690', 76, 6);
    T('8.65E+3829552169471', '9.300537618869137179982e+1914776084735', 22, 0);
    T('5.08667055009654381266002844039288138008E-446532', '2.2553648374701029316080961126748193354301952963336369e-223266', 53, 4);
    T('6.26210445084868288349076048047851365905313732594562929742184073292039222506878473609784779912523E-1649', '7.91334597426947009348810449681348968232240143446979960601647198570636887630321e-825', 79, 6);
    T('6.0709843661253848474880243242393E-97313463', '7.79165217789230171144766443917588705966812805043210022464317486453826e-48656732', 70, 6);
    T('3.717118986316690779554286111870957213664E+2075264', '1.927983139531228052993901952267868176805576752962382635725782508421129e+1037632', 70, 2);
    T('5.9331565870553030940549550010399897239812269057396991808738400254259493264131016783135439060E+626213', '7.702698609614232526774193328265304785985778859788866311e+313106', 55, 5);
    T('1.36E-88792533842209', '3.687817782917154924001e-44396266921105', 22, 4);
    T('2.66686726495086474901245525136018689232385381658184653509249468063872574738681834951052130902E+795902153621474', '1.6330545811303628160838124453180432257523599052180282997779142094043991476e+397951076810737', 74, 6);
    T('4.308077931748893742966163224102440059110945873E-2520', '2.0755909837318367342874938232591696e-1260', 36, 1);
    T('2.280290553150565253893754926857086968936538151126860915224444422791405907679430453719866676519E+3122515', '4.7752387931e+1561257', 11, 4);
    T('9.38376866201984690222362763E+31419753380', '3.06329376032071839269518300174172952753118251501806952209192237706111000572e+15709876690', 75, 0);
    T('4.1107872689787240120760078270461541772564915392035E+99335', '6.41155e+49667', 6, 2);
    T('7.0667848214362876128787402523243417586569143305766651104115900191331822E+8407617244618986', '2.6583424e+4203808622309493', 8, 3);
    T('5.5005874847E+8947873', '7.4e+4473936', 2, 6);
    T('7.51732348988729336394067104088898437252668765476669040043424276439916051550E+1188', '2.7417737853235254397247806369882320002342472414430713482298035842433170506e+594', 74, 0);
    T('6.9724734E+304972997734', '2.6405441484663724516116e+152486498867', 23, 2);
    T('2.99166018868269264948993509121382277679758537695044211420599891093078295938875155488036428849E+2398550434417950', '1.7296416359126802222406463704097e+1199275217208975', 32, 3);
    T('2.31249188937318E-540672', '1.520687965814545400290754517434868530212862e-270336', 43, 3);
    T('9.9405512230682020316782E+12356753070', '3.1528639715452682421453068e+6178376535', 26, 2);
    T('8.605304659E+49', '9.3e+24', 2, 6);
    T('3.57719846275365795914194725514363552377638124158310630343346587E+41510816258232', '1.891348318727583811088510734e+20755408129116', 28, 4);
    T('4.1920666065494688E-234565968', '2.047453688499319586e-117282984', 19, 1);
    T('2.9E+3415', '5.38516480713450403125071e+1707', 25, 3);
    T('7.451339979686344617249241177606518252460743433235797066145803127886303106943703406887960366055E+59913', '8.63211444530616995297486882056511874361e+29956', 39, 6);
    T('7.04773749711012167656943703989972976558691E+34981184', '2.6547575213397779686437e+17490592', 23, 4);
    T('5.7E-21997588495', '7.54983443527074969723668480694611705822219470462338014e-10998794248', 54, 5);
    T('6.1490215811790328464351E+51203094568073', '7.84156972881006322e+25601547284036', 18, 5);
    T('9.446181471893293387773542202237656475676196159276673160249724876232128087303132597E-40055418', '3.07346408339e-20027709', 12, 5);
    T('4.133848562062779496276426375036243113760342115E+360918984', '2e+180459492', 1, 3);
    T('7.16203711338E+137951795', '8.46288196383477866594835930945604975e+68975897', 37, 0);
    T('4.661446228719970353170383319438925741090187385001608608366288E-852244', '2.15903826476511767107e-426122', 21, 5);
    T('1.05811312157440264046383518534621E+26296559395', '3.25286507801e+13148279697', 12, 3);
    T('1.542085477015226E+5403300846154411', '3.926939618857446e+2701650423077205', 16, 4);
    T('5.292987816621844920407591519790997148291859608855681470609202096937600310235150613372875022280048179E+66338', '2.300649433664730442433670197221423917e+33169', 37, 5);
    T('73.753384728665013020438300', '8.587979082919625568762e+0', 22, 4);
    T('4.2794648579182945572703279910205170577550420354118714938358645443205E+68', '2.068686747170362320265134050006653074465e+34', 40, 4);
    T('8.0646865989393833752952494990584973957357958941E+5658766876943', '8.98036001446455562715353980795679909502705e+2829383438471', 42, 1);
    T('5.3910904354900550762252455535660846996765124370E+2752412924', '2.32187218328013376375136379072e+1376206462', 32, 6);
    T('5.986124292724735016744822079612459376608163544515767105897796258431451078611220024066E-4872581014', '2.4466557364543003921288976213619596117887051389961964937871245111735696e-2436290507', 71, 4);
    T('6.69682461295553468596792007096642E+542123463033', '8.1834e+271061731516', 5, 4);
    T('1739531136377917819934369284232931785942417249487422979.1005489031565735119392084', '1.31891286155603085768423596012529319094905e+27', 42, 3);
    T('2398.54813628', '4.897e+1', 4, 1);
    T('4.77167986588177668987E+4613547943411267', '6.907734698062583157322647031e+2306773971705633', 28, 0);
    T('4.64554206101424563744893461328350338123359731937492571453E+2182462', '2.155351957573111918e+1091231', 20, 6);
    T('8.454134495852968507547764200311682871814590790258852817796E-1290989935892', '2.907599438687001897658881132464433205496327147e-645494967946', 46, 6);
    T('5.527037507482882972397643996952896E-7247103847119', '7.434404823173730678947872625197859178204386138e-3623551923560', 46, 2);
    T('3.74702605603293977437118895740361098853913210603581629071143943886813781391753277552118258944288093E+763290562975368', '1.9357237e+381645281487684', 8, 0);
    T('5.67378563735832336944024685575440846398644558E+829', '7.53245354274311032039089181e+414', 27, 0);
    T('4.81934281E-293268481035', '6.94214866593909813829362113542792120779209597565809846494091783649702919e-146634240518', 72, 2);
    T('3.037146687423928543260059681340318620465034594623024E+4262365621', '5.511031380262616997885e+2131182810', 23, 1);
    T('5.7080481337195166503539634071994197717285929E+11344913584', '2.389152178853309597e+5672456792', 19, 5);
    T('3.445579959979806042741365295521301116888199039E-94', '1.856227345984269795026798903805396851531124141438197349831e-47', 58, 0);
    T('3.242726280732920859265528606629430427502604972528045300E-5390', '1.8007571409640226272644930289576679865988847562452e-2695', 50, 2);
    T('0.7978620507120453294507540688138928682047007398199121371992577068603161924763826157863143497', '8.932312414554505098435752662490676463e-1', 37, 3);
    T('9.433048448725014517771886842898641708174615785340950307E-23395833', '9.7123881968983377933811665386798895624761839820680039186218e-11697917', 59, 4);
    T('5.50330567266219839129742267300055899713362657677692793519465585771460282592356317040679205667E-934820', '2.345912545825653405936264159895697558508589187753826697212e-467410', 58, 4);
    T('6.300744181377997510098118204217347630381094644337E-77800', '2.510128319703595852424441549170951188208e-38900', 40, 4);
    T('1.27513049475581877412462685050201399930465009390427670E-466728', '1.129216761634283736389877670374620914990236913849724284e-233364', 55, 5);
    T('5.34073979179018727321803178801304978518698268483882948751820498756440340142137311076491348714334746E+395990304609', '7.30803652959547491698270679945139449761993781528e+197995152304', 48, 4);
    T('5.8243983658215702618361816261897514419498830662782E-90345114034031', '7.631774607403949178710241e-45172557017016', 25, 0);
    T('8.44546465404453934980985847869279478E-226905036899266', '2.90610816282610846490706137779700666902535e-113452518449633', 42, 6);
    T('6.2432930931045069603154192874980470764360580123822675101147105584748796379093479492085019107E+579315836848805', '7.9014511914613e+289657918424402', 14, 4);
    T('8.66086301459540581909346532234690485108E+470904871', '9.3063757793221555572342889067e+235452435', 29, 3);
    T('3.190426546296719009017869506244592199658557067771700E-2598214', '1.8e-1299107', 2, 6);
    T('7.21108E+82359', '8.491807816949227213593064181e+41179', 28, 0);
    T('3.1693955336956672850491067470521035716E+72366687', '5.62973847855801138816058613026785534753268006e+36183343', 45, 6);
    T('9.081110640072082893614830415501913672234958447172787308388406950940350731372567494820E+15332206', '3.0134881184554358374621951327463e+7666103', 32, 4);
    T('6.42994492575975155272578587053975414267474094483355488701952E+2767', '8.018693737610728547512106418952e+1383', 31, 2);
    T('8.314309929034070842903648725957261211632843241373960929722670E+8687', '9.1182837908424800478998206e+4343', 26, 6);
    T('1.87984812268212109644354997914165587230412563190616739873676712055E-57381033166', '1.37107553500240135e-28690516583', 18, 1);
    T('8.7161397888209586216499087025526216917688868508729688932390297672E-5030315770', '2.952310923466727042697744e-2515157885', 26, 1);
    T('9E-10', '3e-5', 24, 2);
    T('4.247468586750111642471885678654670720318855464208124261385790498393480344990E+327208264983273', '6.5172606106784709752659817237436155572e+163604132491636', 38, 0);
    T('8.5769573613073E+51812782878264', '2.928644287261138172e+25906391439132', 20, 2);
    T('7.8213307123593476748089155801E+14155220745', '8.843828759287092662e+7077610372', 19, 0);
    T('3.80188541514483872785657665089889333E+397248033190490', '1.949842407772e+198624016595245', 13, 2);
    T('4.758924800169502008147070385281987960854071204426990874384344600E+60332142920', '2.181496000493583762680162020581399520928591e+30166071460', 43, 5);
    T('3.284136941275397751926398972075570885359063205896628106915778371571E-1506910056517', '5.7307389935988166166954070857e-753455028259', 29, 6);
    T('5.88901974148362950346647859978436156480377638481319E+74', '2.426730257e+37', 10, 6);
    T('4.1859828134078374948537838443E+4349886', '2.0459674516980560937004128161830958870434209e+2174943', 44, 1);
    T('1.634542581144E+27444253014039', '4.042947663702808804886829397446684998908733e+13722126507019', 43, 1);
    T('328951620.23996743929010188211817619446898484657467032140187054934970459640270077032751', '1.8137e+4', 6, 6);
    T('9.9151271877023565283289796714361578060E+2529402156757124', '3.1488294948603292909179054e+1264701078378562', 26, 0);
    T('2.9721154293031423153342916735186829814237760837471E+9549123153', '5.45171113440829907232353817177931276279035365973799e+4774561576', 51, 3);
    T('5.1463006337587775532767672733578419502101057096695624440943078772142E-3947', '7.1737721135806771271931597534657509458353981380566759e-1974', 53, 3);
    T('1.88980615773156754205954823149543437784960306386680372847681098515608435802959652192394E+17241859361', '4.347190078351264757426612734501227243e+8620929680', 37, 2);
    T('6.580323269517564881213074043352095351728E-9', '8.11191917459583967298141841e-5', 27, 4);
    T('9.381938275963857628727756659322343911962478E+234225', '9.68604061315244415320181392746437893776939038367229e+117112', 51, 2);
    T('3.54815215327753E+296647511498', '1.8836539367085266174651494032031672097653870692357737e+148323755749', 54, 2);
    T('1.92394277535725E+49', '4.3862772089292874e+24', 17, 1);
    T('8.7690137974469108229183041847757817521042200425679974781411541561548585E+675593892678', '2.961252066e+337796946339', 10, 0);
    T('6.167105498852589107900610851857147224701453554144283140882592917100008474695866E+77284025', '7.853092066e+38642012', 10, 2);
    T('7.051871775401496257652483263593729285069668362090816819149E+323', '8.397542363930947e+161', 16, 1);
    T('6E+5798685104885', '7.7459666924148337703585307995647992216658434105831816531751e+2899342552442', 59, 4);
    T('8.69594599901657921465445285776951464236696127753419356969146546306770337648413718821623540949E+412300', '2.9488889431473304e+206150', 18, 2);
    T('5.28762923934033642144886070029E+62', '2.299484559491612768e+31', 19, 4);
    T('9.3471916727204535187988E+5907', '9.668087542384198704840419583945826881578586514e+2953', 46, 0);
    T('7.954482704058192483124183933339771508060477171036402272590400874435921798569E+701035854127036', '2.820369249594491037469605516130076696619e+350517927063518', 40, 6);
    T('8618.4639327727645273833789336368164756985631730495080092827488985109431842902319423536932', '9.2835682432848871299428e+1', 23, 5);
    T('6324495.478702158886511186170734166264532893', '2.514855e+3', 7, 5);
    T('8.466435240885788053829433869674858944888592268E-22951', '9.201323405296538132234381768960306083e-11476', 37, 6);
    T('4.098885263840582204133801481752636621689E+256557886969944', '2.02457038994463766e+128278943484972', 18, 6);
    T('7.1823179730638717408697557381E+9406067836831', '8.474855735093000236993940306697095118e+4703033918415', 37, 5);
    T('9.88829924828854913899750908E+9689773', '9.943992783730561989409e+4844886', 22, 5);
    T('4670.483886665925407', '6.83409385849062319343985287727803406359112349523913654407e+1', 57, 0);
    T('6.98025708136240168E-56396', '2.64e-28198', 3, 1);
    T('9.1710595940555184509398848106652147561098850378767695026613099E+933434894', '3.0283757352837706375e+466717447', 21, 3);
    T('9.245757224453322950588235389E-95593500', '3.040683677144553448327558422575208880743324112e-47796750', 46, 3);
    T('5.5565010240165151864531727821882657622053368103080663152E-723', '7.45419413754197522540577686e-362', 27, 3);
    T('8.7494055768566981640849728506537713042195044586100711366820E-177574188', '2.9579394139935824865554416096e-88787094', 29, 4);
    T('2.028231720749346532272072715495169057942417198180801611412469932806259476558658293E+201705870', '1.4241600053186954078135714647223611753280350673891e+100852935', 50, 6);
    T('7.682621491133245E+78808732123508', '2.77175422632188747240975406293214032911772e+39404366061754', 42, 3);
    T('4.15348198907405624317499040230559486379859E+392', '2.038009320163687632976132994601876e+196', 34, 4);
    T('4.426698204932075619945688391351818745244E+3357892018289', '6.65334367437311599750360861683071e+1678946009144', 33, 1);
    T('2.6E+97002478093194', '1.612451549659709930473322647e+48501239046597', 28, 2);
    T('9.69426069891531605560472633665147368E+819', '9.8459436820019e+409', 14, 5);
    T('6.1057502266536484189665E+4250182281518787', '7.81393001418213907652947564e+2125091140759393', 27, 4);
    T('5.47680550982695184088048819995376641651863213480295700205E-3845', '7.40054424338301746287243834147e-1923', 30, 0);
    T('9.48015E+88749', '9.7366061849085794e+44374', 17, 0);
    T('9.26381819515225707684655425469617575946318104756305021820755477914791E+85600519', '9.624873087554067627840428874568340456e+42800259', 37, 1);
    T('4.6599791571994135731571620480003567356305217825560902454961990964513707041500862E+270725412810', '2.158698486866429795700162791776593413e+135362706405', 38, 1);
    T('8.737073595706914415883100450851855691779201336616846220019225757990376784361550467216300606E-13966', '2.955854122873271027263e-6983', 22, 3);
    T('53.94556799464149143834544838735450393', '7.344764665708595148217954598227e+0', 31, 1);
    T('6.42994806637259033691918575438359001696488038021502906173190600504930054E-314606792', '2.535734e-157303396', 7, 6);
    T('8.56239412114284844861880004710097694484055479381008395652509395066961289045921658822E+681502070052', '2.9261568859414986002591553516125004595101e+340751035026', 41, 4);
    T('8.59823041876442089521934983534293227248642327877E+813746', '2.93227393310454896442453545989706983683765e+406873', 42, 2);
    T('0.027779838550', '1.66672848868674468337532302e-1', 27, 4);
    T('7.19408567731364146594092219200044802561214702754058445228456048355862349012698584128512716141259629E+8857178805574424', '2.682179277623634438838445e+4428589402787212', 26, 6);
    T('2.2366E-657635532', '1.49552666308561680554694e-328817766', 24, 4);
    T('4.33881729191547404176180871999281554378421527255574652E+67029548960023', '6.587e+33514774480011', 5, 2);
    T('0.00007938967592668156343432000802251215854227361037783310535713445508209660765923', '8.910088435401837437495e-3', 22, 1);
    T('9.569791076447111060859887136028848285925989327E-663511', '9.78253089770081485780497861970454609938040581984395392e-331756', 54, 1);
    T('8.19855169107053881527328070418205797992015792797727898522E-57212016326', '2.86331131578e-28606008163', 12, 5);
    T('125666896.0829082923', '1.1210124713084520020049167510373e+4', 32, 4);
    T('8.11648724926933692790180697188323751686884025113436838E+7078852847', '9.1e+3539426423', 2, 0);
    T('2.254903893513876880658507804566308E+982281234017127', '4.74858283439793948874249775896006193e+491140617008563', 36, 4);
    T('5.469143746830100735253348935559', '2.339e+0', 4, 4);
    T('4.7627332515414197549E-9546592730', '2.1823687249274399e-4773296365', 17, 4);
    T('1.2945961E+896304134263', '3.59804961055291732062781124862669169443e+448152067131', 39, 3);
    T('32680527.721145832868453840156261544897015934877292074837704795006969524543881059732', '5.716688528e+3', 10, 6);
    T('7.737466929706423975628211097425524554009865590373586105079382E+98559858', '2.7816302647380050347129e+49279929', 24, 4);
    T('67317170637665263350087.1310072604631816804078322910320925708364368254', '2.5945552728294e+11', 14, 2);
    T('7.6032251731515811671178255235614504192822589872975842019099252011720755E+508756150', '2.7573946350044966e+254378075', 17, 1);
    T('9.9848097300959648734586571016676939146677480278058842266861123899943004138800380063E+6939025783803902', '3.15987495481956767786497761567e+3469512891901951', 30, 6);
    T('7.4293085411192703298634905416770160787470764147710751E+1704688451175236', '2.7256757953064172760079852e+852344225587618', 27, 3);
    T('6.170668109195084977785439851803605841743E-635173437', '7.855360023063923820544813765259744916913861325802792e-317586719', 52, 4);
    T('9.5439860702461969186063023163282709609129879673557619248183413E+40635199034', '3.0893343e+20317599517', 8, 0);
    T('4.468E+76817200', '2.113764414498455881e+38408600', 19, 2);
    T('1.00777870841521477571356255915423488904723383092745460155240E+88901', '3.1745530526598776243573369e+44450', 26, 1);
    T('6.517116165E+909872543470', '2.55286431e+454936271735', 9, 6);
    T('1.17985940191588068300405071225510353677413328242375807963514688189265009414508457156773249442128E+4179893434743', '3.434908152943657215604933907704059425628745060543446e+2089946717371', 52, 3);
    T('1.05344216013464632208075452745691796774826936E+160643496425516', '1.026373305e+80321748212758', 10, 0);
    T('4.683579042536430644272216483022866393700274198058236790133865858919014496E+7005418922938082', '2.1641578136855987250505956681893475477094134636095065e+3502709461469041', 53, 0);
    T('0.000395234480068477549436048466926547932607392183388725393635867924418847389208459354056746', '1e-2', 1, 1);
    T('8.1383435668542E+598143343764', '2.8527782190093e+299071671882', 14, 3);
    T('9.81235557401192E+882525610999', '9.9057334781488644693766431341166965767688e+441262805499', 41, 5);
    T('9.17662217225670365580132623427206966441266172084065049E-830921660137', '9.57946875993481365860255767897174669327048875038496e-415460830069', 51, 1);
    T('5.2141857918068E+309358816', '2.28345917235382161321052300952309364289118312e+154679408', 45, 0);
    T('5.8292063047359516289362926110569329408948377528422124849577781626029965E+71511240331904', '2.4143749304397508237759e+35755620165952', 23, 3);
    T('7.3476552476159935042221685634374819663329992258622464423E+576', '2.71065587037824e+288', 15, 4);
    T('1.392135632874387320032481504844773664734895510354552863116E+9997075175', '3.73113338393897e+4998537587', 16, 1);
    T('7.02482467267E+48', '2.65043858119179966029992168e+24', 27, 0);
    T('9.98233380661853360704853E+6810663359981', '9.991162998679650010151069295152e+3405331679990', 31, 2);
    T('6.57789580284877375006369109606729108414765271644459837793021469121186643599E+526', '2.564740884153558e+263', 16, 6);
    T('6.504238456702103379007797325438707029444E+3647', '8.065e+1823', 4, 0);
    T('6.81850767220638054274412750469365723860361232E+5465839462503', '8.2574255e+2732919731251', 8, 1);
    T('1.19604959849908082216823463785318338E+2312874', '1.0936405252637087708041416634840557237e+1156437', 38, 2);
    T('5.5143888667698425319055499060431992271862077917983664532773445925406093E+4945138', '2.34827359282725881695755848618536522e+2472569', 36, 1);
    T('4.785603116656929212926699304743047E-32942678563', '6.9178053721226714e-16471339282', 18, 6);
    T('5.712206249724500E+956593455516', '2.3900222278724731559240830843101340341730811808797e+478296727758', 50, 3);
    T('7.611751040072142E-55428', '2.75894020233714779220980219095884123933594985311457e-27714', 51, 6);
    T('4.105706651130321043893270073562756036693192753694544832915634994651525538267312600662988546993716431E+5665', '6.40757884e+2832', 9, 0);
    T('2.757138880476894959215646620093166318321386241819551260442696423366868358392579E-39844', '2e-19922', 1, 6);
    T('2.64365632326608604466838016343240976E+8833708995484', '1.6259324473255603384740099935545162929949642103619795285e+4416854497742', 56, 3);
    T('1044.1741675043528745896963981035516625565492922589193763033241', '3.23136839048777116459616095830288e+1', 33, 0);
    T('3.84015942498301980226529611857008321366703157E-9982585507', '6.1969019880768001399e-4991292754', 20, 4);
    T('3.43542949838980446403594147873343440722864421107567077078660457637444645283004022644264863406617E+5628141128962', '1.8534911649074037234559376516649352187e+2814070564481', 38, 5);
    T('3.323E-7866778488855', '5.764546817e-3933389244428', 10, 0);
    T('6.93865018150729678629280950820164762873972202270834604763129020387954968208262172921301666E+763519870', '2.63413e+381759935', 6, 6);
    T('7.71468861450707836815770830310E+52560', '2.777532828700153881973756611285870083147290416763765924489e+26280', 58, 3);
    T('2.606532362335176491325466113776838354347806783180E+90385324855036', '1.6144758785e+45192662427518', 11, 1);
    T('7.868393853197808762950601E+6246', '2.805065748462557653164969846100469275224413814628482583e+3123', 55, 5);
    T('6.511532832579055456470900614798386565310414631708375097478822E-586213047003190', '2.5517705289816040699070559343e-293106523501595', 29, 5);
    T('0.0000613751689825397355142036642320752732384422957741160566957431037479', '7.8342305928878386260177754291857677002872852903559e-3', 50, 3);
    T('9.3316454865294942574240974545915540073823785055715168E+55008168302443', '9.660044247584735704933e+27504084151221', 22, 5);
    T('8.8812261520614461E-9416', '2.9801386128939449215124062285231345252244590858853638e-4708', 53, 1);
    T('4.63523618893604287336733983904936009830559143837356007832892997084272076160104174858625E+322813706844', '2.15295986700543093829238349289644e+161406853422', 33, 6);
    T('0.000502306845609341390231895187631938039180931475534782030177701824702995519009795875703959', '2.241220305122504886306558e-2', 25, 0);
    T('1.11662921983563637396344E+29080479005', '3.3416002451454847443329108011192385429301756190037e+14540239502', 50, 5);
    T('1.00590258895481238344713498709973978803276198391935772868999219211052285825098161628821662878873803E+702496668841977', '3.171596741319445206758916995443487034e+351248334420988', 37, 4);
    T('7.6012293E+9892107479994', '2.7570326983915152688273840331281811319992611551591538e+4946053739997', 53, 4);
    T('5.322387338003432573256770322174959454295450836303338407051181012693607179628696297283022E+43340698353', '7.29546938e+21670349176', 9, 0);
    T('2.41541422010929716290327180309219489645704739520114860452833255489313291980782526213E+246000466', '1.55416029421334051880550669444910942401e+123000233', 40, 6);
    T('8.179070664140E+797634898913', '9.04382e+398817449456', 6, 6);
    T('7E-941344', '2.64575131106e-470672', 12, 1);
    T('9.0529595776E+241311', '9.51470418751944174177457169730200668278e+120655', 39, 3);
    T('5.77830279127392E+73751', '7.6015148433e+36875', 11, 4);
    T('6.38092940393221700240410362126190167886061191293458850790633624729E+4347448561', '7.98807198511143676486464574968746469321880158e+2173724280', 45, 0);
    T('5.6341676058931750971182465094501591449326721039640E+70780368', '2.373640159310836882841437243671857205416743e+35390184', 43, 2);
    T('3.803941E+505', '6.2e+252', 2, 2);
    T('9.640598662581895670201306057239214613195033E-2782567548778756', '3.1049313458725453226470569057245866375e-1391283774389378', 38, 1);
    T('5.9251864447342554700259999667718810841386332991356806627693042686574050056277955938008500339', '2.4341705866134887e+0', 17, 4);
    T('3.559533318194514333941034468806379817719137880702744707830542547995088321419381776759938502951309E+7426597066602', '1.887e+3713298533301', 4, 0);
    T('2.477422764378241007929E+7686678', '1.573983088974e+3843339', 13, 1);
    T('302198.6632515718651630206351384507948717', '5.497259892451619074743401610051893223928e+2', 40, 4);
    T('9.5792985591006463660327578237600594728601372003657E-50226282', '3.09504419340025682387284611846951e-25113141', 33, 2);
    T('9.18022412014389590847807200126390552366428056775659854727498049452345197296E-9046263176', '3.02988846661785488048660193617879404173e-4523131588', 40, 6);
    T('3.86599595907911663261304019074208545383912152223917814923952344754506E+50161441532', '1.9662136097278740565067565730103957e+25080720766', 35, 1);
    T('2.477805996129736565717289578938014954127178720132853734915459657E+71531551565', '4.977756518e+35765775782', 10, 1);
    T('9.2664383545386705331E-425569568064', '3.04408251441032237553394315580897e-212784784032', 33, 3);
    T('6.337155960261111906974831481672325743099250100892317961791640533811372652329905E+33963511862465', '7.960625578596893101696387399283767919538993721413163586453e+16981755931232', 58, 5);
    T('4.46765405566301034100732184E-345455080', '2.113682581577236336621e-172727540', 22, 5);
    T('4.730768337448507208461301912877217E-8867873719329', '6.878058110723191039170357769021223641203768250386814387e-4433936859665', 55, 1);
    T('9.874204E+4196232', '3.142324617222097249819348121732052433e+2098116', 37, 0);
    T('9.6608138599388729190320251E+609554712878', '3.108184978397983461792799750179382518954252616661e+304777356439', 49, 2);
    T('5.176813292548635899665031108150305521279598945213223604392623871176973653423877108367173831E-350047', '7.196e-175024', 4, 2);
    T('9.5128426177082384157761572615928661984650E+60694846801221', '9.7533802436428358577196e+30347423400610', 23, 5);
    T('1.2482313887E+545033', '3.533031826491236124483001907439217143069207887e+272516', 46, 6);
    T('4.2180749054401670548E+37157483577620', '2.053795244283170272722623608492053497e+18578741788810', 37, 1);
    T('5.670E+5450074', '2.38117617995813153145145417827533438314e+2725037', 39, 6);
    T('2.5395286810018570475638420E+36', '1.5935898722701073465064303638355625671e+18', 39, 1);
    T('7.32839592123157409578386817809651753E+251474145885200', '2.70710101792149125108446628245481957358735005924599288e+125737072942600', 54, 0);
    T('48996.22431787259993046706897917483044798369999565421828683299605206712657496702751264', '2.213509076508894732173154425657276e+2', 34, 1);
    T('9.8224682485255003345130898275573453595151355193741836775E-742', '3.13408172333229057017200709487244404384140752373185533556e-371', 57, 4);
    T('9.9642197609616234179085125421280569341869E+4071807002', '3.1566152e+2035903501', 8, 4);
    T('5.3780206003796510684588181149511860587497148318841069489746566777525338454414833315664177E+6296402482', '2.319055971808281100949223225280986043199162544987995920561e+3148201241', 59, 6);
    T('3.82429853291182665213075284540147650774209390207111E+320266', '1.955581379772221765600690553040398081077734611784e+160133', 49, 0);
    T('4.222782483E-572527657293', '6.498293993810991004846515689957184025844608e-286263828647', 43, 1);
    T('4.3903038382935297604844299346392961E+96583417150734', '2.095305189773921635834185689172899330176657654757e+48291708575367', 50, 4);
    T('9909444992.836643228565753504477745138285120891205577775751233959613103612170687837300', '9.9546195270520727363734442944171819553663573e+4', 44, 0);
    T('7.566515974156E+32', '2.7507300802070711e+16', 17, 6);
    T('7.503303012992161616292964416340393996708767011060169387467045789128211677183166E-806053510', '2.73921576605278789350582648746327555974e-403026755', 39, 0);
    T('4.34899507639009736713660316209629311E+4534402698', '2.0854244355502544478035290702750220752853910559759e+2267201349', 50, 4);
    T('5.9689E-9280', '2.44313323418924494806413540638533237944222252379774818e-4640', 54, 5);
    T('1.47294094715225953523392443553651419969795279845412072232374443602682284E+266261', '3.837891279273371411371931209259466408780049494149e+133130', 49, 6);
    T('4.7815651648332497761234837829766779E+6934855', '6.9148862353861251186647863355992641049576105008415e+3467427', 51, 4);
    T('4.892003226E+2272668419161', '6.9942856861869747334742369035855134838369686e+1136334209580', 44, 6);
    T('6.1021870719899224262812355658121599084620434926108158533330095529E+5962428070217931', '7.811649679798705890173241635912086371647e+2981214035108965', 40, 5);
    T('4.3021957E-221', '6.55911251618692694613664547150513e-111', 33, 4);
    T('3.1523392747332903571840536384322E-62264434526890', '1.7754828286224821603e-31132217263445', 20, 6);
    T('7.3371048451882940102915895826509445744563310E+745694', '2.7087090735603729671890037857e+372847', 29, 6);
    T('3015968.965786283969219452851', '1.736654532653597161370338e+3', 25, 0);
    T('8.17182949614000432300033942444452414965030982463245419E+759909818421609', '9.0398171973442053243593326690988797925795074495884771e+379954909210804', 53, 4);
    T('5.733631667599139701686180684910068093261278100075167065265126182099E+476', '2.39450029601149552701622396e+238', 29, 0);
    T('2.7685250069024184479109065982150069923375061220478352007780669E-46702307042', '1.66388851997435e-23351153521', 15, 0);
    T('4.47784612366328777489743951674998871512114562893371561463886E+80125227271', '6.6916710346992460138086e+40062613635', 23, 5);
    T('8.99861535505033517097438440717E+6302273649313818', '2.999769216964920981781838497619807046212282319040217024591e+3151136824656909', 58, 3);
    T('6.99173704023041688178553002697165260248676027081686441850E+9343413', '8.36166074427228653620327e+4671706', 24, 6);
    T('7.16557734885468280967535456543145283222801494222103939935157759409341790462334010789221606248E-5228524', '2.677e-2614262', 4, 0);
    T('8.316533594164212588346852168188104491264055679159096473623712869269E+87671806098136', '2.88384007777203425638342276133757328710423e+43835903049068', 42, 2);
    T('4.724120033581854495840085106267387645832700638867214980395556069995822107574798299123957594380E+566854526613', '6.873223431245236695066250497e+283427263306', 28, 2);
    T('3.075897568537E+2962260', '1.7538236994e+1481130', 11, 5);
    T('42938591.284880105944665666761545221392', '6.5527544807416755483348724628659814866949957427833722603e+3', 56, 1);
    T('9.163123018763583179296376614593153128784016577291902678727611785642E+43165330024231', '9.57242028891522459e+21582665012115', 19, 2);
    T('7.5173522982551338226195834315050177E-76441896', '2.741779038918915840397e-38220948', 22, 4);
    T('3.7229749958881355144440302922091424477012972758744E+575937112729', '6.10161863433641312531817677112504710894e+287968556364', 39, 3);
    T('3.213921579440428355413004918194883461702127899276816684275919996E+75', '5.669145949294680449498091816954246945181847e+37', 43, 0);
    T('7.270015112535664689094678492889155641E+2219884128530', '2.69629655500571e+1109942064265', 16, 3);
    T('3.086688298813789440211491596186356590E-723', '5.5557972414531016974265435412829846003750734113660705507338e-362', 59, 6);
    T('5.045244057E+515488287176127', '7.102988143732185619703906087338291014373e+257744143588063', 40, 5);
    T('5.539052067452906192653430016232501495108799553E-930596128144867', '7.442480814e-465298064072434', 10, 1);
    T('9.832896342256918265301578E+816661', '9.91609617856589683920144913944287729640955027846703803293e+408330', 57, 4);
    T('5.8946838571717610117484523557E-666628183', '7.6776844537736512864398900586069e-333314092', 32, 2);
    T('2.6970598385937639036282024802673747E+84373345', '5.193322480449065954795124196766e+42186672', 31, 5);
    T('6.64574830960272837776645E+19910654020', '2.57793489242896287265552105609250647161310519194e+9955327010', 49, 2);
    T('4.36743610876825562762215589482423799748072264558839592179598572096646278251961608936E+511859970', '2.0898411683e+255929985', 11, 3);
    T('1.8275793445848912555519981E+54740521242268', '1.351879929795871795918417760776442e+27370260621134', 34, 3);
    T('9.781092607E+12877', '9.88994065047915239510413103349947679952957845345509e+6438', 52, 1);
    T('5.8290281159642131075655125233418253412799435348919296976915162324056370850269389145719289793958E+842463', '7.634807211687936988051725405505368539e+421231', 37, 0);
    T('2.7476067172041493705394497743604567802629660713484017430592167E+239465416337', '5.24176e+119732708168', 6, 1);
    T('0.0000280111104325773056392179265525079409784531611998484005617778335795346608205484819345378573183066', '6e-3', 1, 0);
    T('3.383889790442193140883830538460380056904434820072370081124E+62268962408', '1e+31134481204', 1, 1);
    T('6.16335E-9893522299', '7.8507006056784511670050945735937554118e-4946761150', 38, 3);
    T('9.514800039778985236553648E+22817', '9.75438365032818731019610721839027953852376e+11408', 42, 4);
    T('2.625098706891241033631727388097826659808984167190457839217086535E+4288342655047', '5.1235717101366318571833364284639727386e+2144171327523', 38, 3);
    T('1.0406157056476653052660046E+9938432645', '3.2259e+4969216322', 5, 6);
    T('7.2145380274423731176330094428743E+93000136', '2.6859892083629772315714488478009973444777059774532740525066e+46500068', 59, 6);
    T('5.95871063245908733898527E+9708056438', '2.4410470361013298874785e+4854028219', 23, 6);
    T('5.58980606998098034839000627636E-285220478', '2.3642770713224e-142610239', 14, 4);
    T('2.7201729026904024759127837439621357946184115530064927160414493006310800394351782878E-311623229648', '1.6492946682416706569545458076e-155811614824', 29, 6);
    T('3.34271252730099149001743E+210766', '1.82831e+105383', 6, 0);
    T('5.2321972939609930807364617475218E+88811300310917', '7.233392906486549400993363318042e+44405650155458', 32, 4);
    T('9.005102743563E+9086710498', '3.000850336748402386076193482732132e+4543355249', 34, 3);
    T('7.9095430079357519796626451723797502818429503240556511E+28459', '8.8935611584650117e+14229', 17, 2);
    T('3.83479180686399380566739976492323531213212529775830842583427283704020457672027218E+2287', '6.192569585288480071362070685367945504172015746194067e+1143', 52, 6);
    T('3.50E+2608156735467816', '1.87082869e+1304078367733908', 9, 3);
    T('3.0340047089347743222093013781998873293726E+7210', '1.741839461297961960433654467635930042494722e+3605', 43, 6);
    T('2.641777246989002E+701234741', '5.1398222216230417009798575909577287420914e+350617370', 41, 5);
    T('9.72301698207E+73865074549564', '3.1181752648095327997805348259730612297001508561912754808684e+36932537274782', 59, 5);
    T('5.8726238465824943471872725821986339702747220162718890849109428746079810562364484406E+195302', '2.423349716112491748e+97651', 20, 1);
    T('51479272.0256847652072541702153249116402', '7.174e+3', 4, 3);
    T('7.860228047056475789407483601393825532997165494731024558778606490540633654277E-4190735944', '2.80360982433e-2095367972', 12, 6);
    T('9.63014865642023965317689575284348444E+909419245551', '9.8133320826415732037874259187e+454709622775', 29, 3);
    T('3.1553010679817541026859447E-7', '5.617206661661785523847713e-4', 25, 2);
    T('9.932925138950288828083324636215E+7033951780492', '3.15166e+3516975890246', 6, 2);
    T('8.0234243255640456681913976522538289235947963901206748702119487215572185722502877206166E+44366108123021', '8.957356934701243481642317799702e+22183054061510', 31, 5);
    T('3.145475031537113749699887435261949E+22301626054715', '5.608453469127753893644949958242940074812e+11150813027357', 40, 4);
    T('8.816879098522E+10611591901714', '2.969323003401e+5305795950857', 13, 2);
    T('6E-53468665389517', '7.8e-26734332694759', 2, 0);
    T('5.765046640889604093065256460197568681779448243480580446343270027735006054659306952795224420909945833E+4010407', '7.592790423085312640641371114749816603186e+2005203', 40, 0);
    T('4.5524592263310881517611278464697764850822E-514692029620', '2.1336492744429877718424142657e-257346014810', 29, 2);
    T('3.549955586568567247266013356917190226303547559189845658899480400602474827616887E+454055', '5.958150372866202885e+227027', 19, 5);
    T('3.70466274099383726631076449676122650678581763E+45750022229', '6.086594073037758665075529446337038407319521616e+22875011114', 46, 1);
    T('5.1022999468920931841350145718624990016193677271613989555539375402223788600E+61665728484', '2.25882711753070939e+30832864242', 18, 2);
    T('9.94129362939264644087E+103695', '9.970603607301137934888585123157982458687696e+51847', 43, 1);
    T('75518618186866833059695341938332839200088893268521521410134008012564840344255273523.89216652584131', '2.74806510452112165856788e+41', 24, 4);
    T('2.9391E+276053561768', '1.7143803546e+138026780884', 11, 0);
    T('2.718183103082704607697346582403761295795496463340E+44782026', '1e+22391013', 1, 1);
    T('1.33273801375245047920150E+900803', '3.65066845078055599535555363e+450401', 27, 6);
    T('7.00450315680011003540182621671539478005685478113107694125912758538414876371961635078932E+14034', '2.646602190885534280960006168127602062733571012572e+7017', 49, 4);
    T('9.307105982051855780151E+890', '3.050754985581742154362582491336579958885632568822212423e+445', 55, 5);
    T('7.764764595672928697466727357579276470555534428191794694885526727529968002964E+48805', '8.811790167538562477402284699326923855025115614e+24402', 46, 4);
    T('7.217060546229357077411349657040640907351060806806387216379694402226024925E+3176371', '8.49e+1588185', 3, 3);
    T('0.00031056619463769736265314160787852936547', '1.76228883739e-2', 12, 6);
    T('3.216461579356304995840691654217056571542854309295161091070266621609569807963959353555765E-68992493497412', '1.79344963111772531202657243880112703601354194994357846141e-34496246748706', 57, 1);
    T('4.0789905007624563595520131877626876102296729264841E+68535418974', '2.019651083915847173022122738473e+34267709487', 31, 6);
    T('9.2867384568292884961393617922261589394E+3664625567', '9.636772518239334519348687240129e+1832312783', 31, 3);
    T('2.958992309649489406047717638379653702623878573946110361E+5180016917703383', '5.4396620388122361788368008142774558e+2590008458851691', 35, 5);
    T('7.120712878883511758034245753869337345190995086995465694069688179610372600E+33721890794', '2.6684663908101806479e+16860945397', 20, 2);
    T('5.577535802676355552924726209805466135729927975860272645941007667879505921094021742995013961478295158E+741823905', '7.468290167552648e+370911952', 16, 6);
    T('3.0500888913000703082178891475608508807343002E+9735289740146', '1.746450368e+4867644870073', 10, 3);
    T('3462.65803822110845431', '5.8844354344500275554e+1', 20, 3);
    T('2.580357460510E+2303598', '1.6063491091633848196610495958834333e+1151799', 37, 1);
    T('7.988183560629505331251193062559945835328575639760039901392783119995952159490854005691643909211184E-877137907378', '2.8263374817295802425299e-438568953689', 23, 5);
    T('5.41E-8768671', '7.35527e-4384336', 7, 3);
    T('6.824901906335617727291560238733196039115801800363383374787546625730902158836486114416863919E+6262', '2.6124513213332067227790717403e+3131', 29, 1);
    T('5.4448291780086763173E+21795', '7.378e+10897', 4, 3);
    T('1.0853124446979864336117185855520779046654176802855268610346074081789748839398282549587213521E+12006', '1.0417833002587373178014918e+6003', 26, 0);
    T('5.260283694E+97', '7.2527813244299596765158953713251610652258595575258394e+48', 53, 4);
    T('6.6078130501961293563670845406178434071777508700259543592257E-7', '8.12884e-4', 6, 1);
    T('337138283.43471681687732585569487908831', '1.8361325753733492783422e+4', 23, 3);
    T('8.18799384640477465455235100803785649295264816123850945894454456459486211564191672926484E-4104', '2.8614670793851140608219519145820447e-2052', 35, 3);
    T('8.88122968353786614390159590948250361561247387280617852132398120437947634101173771548775955078E+5345', '9.4240276334154847085985476519742827933066841856e+2672', 47, 5);
    T('7.86811122478265831566431366108418262860082915437693564383142770156954927E+9676', '2.805015369794372145565468121938460821562493477e+4838', 46, 4);
    T('8.0748784996001232737481534189124329E+1089471505834', '2.84163306913474021700477305587583932460054183488072035e+544735752917', 54, 2);
    T('7.3860129533128689793E+201348', '2.717722015459430497362498573661745017e+100674', 37, 2);
    T('1.800012329405740701497829450363698176517875649052887131494820E+21254', '1.34164538139023185827430972504927411407222482198701889772e+10627', 57, 1);
    T('6.7809505212885573878659196390876275475191555204E+5981216586975', '8.2346527074847287623094312007e+2990608293487', 29, 3);
    T('7.134300561814246068932009443191580229824420085344752745412403531202726398069643129688280E+1186004559', '8.44647888875254696740051613173e+593002279', 30, 2);
    T('1.352117899566E+86438841', '3.67711558094928530898923462750349736612216706e+43219420', 45, 0);
    T('1.63937659967656756664160655E+6671630779188802', '1.28038142741784862e+3335815389594401', 18, 4);
    T('1.03496309225536861918398865364491009406444077898740399568899E+99042', '1.017331358140192501156e+49521', 22, 2);
    T('4.487315740769031097710E+7099621012', '2.118328525221956948972477041256579694809030477e+3549810506', 46, 3);
    T('6.45703168668063E+92952257', '8.03556574653000396941305144243955820849064500453e+46476128', 48, 6);
    T('6.897120215664694201268301893019950245649112383811268746645521808015717447181898892E-311897986000', '2.6262368925260139603947828493321131187693944199114555444e-155948993000', 56, 2);
    T('5.41265808409E-4212372', '2.3265119995585666439298693907765490321174824519156341648081e-2106186', 59, 5);
    T('3.1978700568016126945775457330718348551669619113311806855130E+31462760890235', '5.6549713145e+15731380445117', 11, 3);
    T('3.482666558459729826202520441814779E+82', '1.86619038644499769221415801885362e+41', 33, 6);
    T('1.47846929769182971823138082996133780885397026025594281954317578201203688647028046169533E+9578685402199', '3.84508686207714912037114030648384e+4789342701099', 34, 1);
    T('7.0226111787353761583514571999000198929847724280595732452446898417449130378578E+826750', '2.650020977036856619446695570346608119e+413375', 37, 3);
    T('7E+806', '2.64575131106459059050161575363926042571025919e+403', 45, 2);
    T('6.76614630936923831680853402388423397012901834426079546074367894E+4146128952847174', '2.601181714023308737e+2073064476423587', 20, 2);
    T('9.34928274304783254148448263901164390216217007247255256672095791804985E-193406613284', '3.05765968398182477e-96703306642', 18, 4);
    T('5.155229789617314671663323E-3049725116745367', '7.17999288971e-1524862558372684', 12, 0);
    T('1.538752842422110960E+139', '3.9226940263320448787e+69', 20, 6);
    T('1.110783925032759511879142099217644E+5465353753289825', '3.33284251808086412602362299575e+2732676876644912', 30, 4);
    T('9.262515547183858941460911930488997549123332850579897328939146579713951172E+52018575', '9.624196354597021074568814e+26009287', 26, 1);
    T('4.126069441863281412245197E-7829210015883383', '6.4234487947389146e-3914605007941692', 17, 4);
    T('5.2593866578649889767832273892689376276E+6715065', '7.25216288969365950547054172649042796262957733381e+3357532', 48, 5);
    T('6.2722133988741029626082215608974806768450759865776391582392152693501989184667999707E+99', '7.91e+49', 3, 3);
    T('8.6556561629534833074817761736997437566915939820275459E+26458468956132', '2.94205e+13229234478066', 7, 2);
    T('4.3E+5105574008', '2.073644135332772087455898679984992632401184434584868958e+2552787004', 55, 4);
    T('3.57667124876848432862603854777226881290523975742194281536024038574905173742330276833046659967E+110061', '5.98052777668e+55030', 12, 5);
    T('8.865106758897210471442052911846498145240295492039691461223432321780333619E+19028033895', '9.4154695894029685254426105866087137311838676971703607e+9514016947', 53, 5);
    T('9.04275693914026897710092723371431406665617908401025601365746550755784129E+13535093023822', '3.00711771288392848924824490867e+6767546511911', 30, 3);
    T('4.37196055715114312026203578956748534623746553520685451E+74855', '6.6120802756402943721981945396606798895619613474e+37427', 47, 4);
    T('3.0268104866166772532631803933169550349895905486593673046621924915952111167039713305838703165490276E+2162', '1.7397731135457511872599378e+1081', 26, 3);
    T('6.310975988472048889335E+2177397292', '2.5121655973426689869237903518269984486361728036664713526e+1088698646', 56, 3);
    T('2.791406140703455436219646714245913851961683540451200817460825045879490828978042E+10822853', '5.28337594791763369194465687242406991e+5411426', 38, 0);
    T('2.67369035E+798', '1.63514230267582521497974806806679303545577276128662e+399', 51, 2);
    T('1.458146674498853841356723E-5440466334010349', '3.81856867752676911489390299364175793600316390424010601e-2720233167005175', 54, 2);
    T('3.298983226610722158253153064429E+50', '1.8163e+25', 5, 4);
    T('5.61368894488386646915301509846162483684043628851057119017552875303532866109286798459665791829209E+3527560167', '7.49245550195919846889258750646601699490636475902e+1763780083', 48, 6);
    T('6.735225759643149114802911290348919E+329', '8.20684212084230916990758609633024006906230886244118975e+164', 54, 1);
    T('5.858443978302665901573611660964788303860269223476563579294763353583974105740651863E-8303', '7.6540472812118e-4152', 14, 1);
    T('8.9917751220520838989342947990428301894815820026774093180011681337705941032416487130477E+4513', '9.48249709836606483046924442783108275e+2256', 36, 3);
    T('7.56E-4609094456214', '2.749545416973504003e-2304547228107', 19, 1);
    T('4.354909320004010185139112965328197480573669897469981523860737121540977519745834151589235956307904E-9702625', '6.599173675547575846766246104692812420269180384879513e-4851313', 52, 6);
    T('4.09825603483731496476091029403983847488989010834589462600179421052878E+7112841393222077', '6.40176228e+3556420696611038', 9, 6);
    T('6.63909151902088700675715383950027273920841181858145967596821907738240680610501989365565148117771022E-47065408', '2.576643459817614542355792094018e-23532704', 31, 6);
    T('7.499846E+52439576', '2.7385846709568795072318106579921458507749e+26219788', 41, 1);
    T('5.94690286575782639250829926461595855167007182297881E-4526661749', '7.71161647500563035651244319339e-2263330875', 30, 6);
    T('5.88402391E+21932', '2.4257e+10966', 5, 4);
    T('7.870877590307529323404668373180962832632497978554332921505946503308372830E-7824254424002', '2.805508437040875e-3912127212001', 16, 2);
    T('3.527286122679253321997963149034979772725709489917010087071821096E-8476', '1.87810705e-4238', 9, 3);
    T('4.41797946876870716074391930886843541E+7014937922513279', '6.646788298696376534644540954407934006978453397e+3507468961256639', 46, 0);
    T('5.116350879777690950383394791E+82618009', '7.152867173223399757e+41309004', 19, 4);
    T('9.27300813961912202768592208469853906854861746260E-41', '9.6296459642185818147702072070298364248334916816528842935488e-21', 59, 3);
    T('6.9140633986922393765186829250003305599960506825610622589649756659424040806479850969422882842892E-265203', '8.3150847251800383423125261928886e-132602', 32, 0);
    T('8.28675186093021808026343035779598963856882883020044961125768E-442073665904396', '2.8786718918505141231707076970383270218738316960173271799607e-221036832952198', 59, 3);
    T('6.06691604111281515796380712015951780729595898780046443180043875E+156348', '2.46311104928560116228894e+78174', 24, 0);
    T('1.061635057289160371714659500661334470783996017983364908411760207568439E+884007688729679', '3.258274170921103397553890245158225e+442003844364839', 34, 6);
    T('3.5156179119274953518902324654505523538863897451976717951E+132366730', '2e+66183365', 1, 6);
    T('7.40276094217641601355437436413343615217760423403365545298510603764115552608396482371378413312E+2458055166', '2.7208015256862114626467449132780185358791e+1229027583', 41, 6);
    T('1.3756324859035942408350274967319105951115504131206472117379490247E+6263663', '3.70895198931395477e+3131831', 18, 0);
    T('6.997767249641450869001818901E-8285229670097617', '8.36526583537035651867487992741566134075959e-4142614835048809', 42, 1);
    T('8.4247607032037143100200E+1173656267', '9.178649521146187827143782992212312593027e+586828133', 40, 3);
    T('8.849E+855015151987', '9.4069e+427507575993', 5, 5);
    T('7.5170665098E+439', '8.670101792828040378289617959535139122536036243050607945e+219', 55, 2);
    T('8.548614466343114298487688667042103705475526828931215262797395518734228978E+892', '2.923801372587254651945423028690364087096e+446', 40, 6);
    T('8.5476955009909992540376901265844502563874717664944502416065080522040353926665030819186731655255327E+72702494034620', '2.923644215870152565448277e+36351247017310', 25, 0);
    T('8.5E+8763496802', '2.9154759474226503e+4381748401', 17, 0);
    T('9.172749614565118701498052251836070841554232626774712561162843507029390864566442140E+597429892154283', '9.577447266659899862e+298714946077141', 20, 4);
    T('2.472762209738950380919431032619E-63477', '4.972687613091084149124693267150565188613585832e-31739', 46, 0);
    T('6.9826559944818254667841973085521569E+9195033270340', '2.64247156928543423577374440345405574174986328519625178166e+4597516635170', 58, 5);
    T('176873792335473497088177405982046036708907141583138068015100851095364222333257942.0', '1.32993906753457504816497114664422409428701e+40', 42, 0);
    T('86026027.912407211257209139644084375458879792935232289271524429494229356932', '9.27502172031996975512779e+3', 24, 3);
    T('9.807637607498538844891235013121408365586430433186059940050782770E-308', '3.13172e-154', 6, 2);
    T('9.068893626107618387547685536799310810225887724412029132500939076034573530391951700871174633143448E+2034542443152', '3.01146038096263494368486752297836021054794588686452097e+1017271221576', 54, 2);
    T('2.517287299593400672536638456009437330447586398867122655339566588463021144805261751616745674074E+78611870107', '5.01725751740271e+39305935053', 16, 3);
    T('4.708943383435421486885286885954549285239500033108678101633082726862779284430009287227E-458352710', '2.17000999616025e-229176355', 15, 6);
    T('5.653048380890135077234791547593744907422843669073736419836476176447929123002934531909E+7875847124', '2.377614010071890343805137513902293171989971e+3937923562', 43, 3);
    T('7.959986138882661854211621303298620754806621159063097243622828626126639362920952254172047E+3256248904707', '8.9218754413e+1628124452353', 11, 2);
    T('9.841794342006369510164473842891336167046479026241164760771E+81', '9.920581808546497496175179916201323103051572378302294e+40', 52, 3);
    T('6.07227720254687537967282978617336295434686488205270812031896120582186309E+29990347271', '7.792481762921794210795266362708617757155751216859414e+14995173635', 52, 0);
    T('5.53805232259412345989459064037443E+93992515', '7.4418091366240530987031527741766644699741928756569372065862e+46996257', 59, 4);
    T('2.95022144293857729524069062479E-2454591', '5e-1227296', 1, 4);
    T('2.29377135E+1061511437712347', '4.8e+530755718856173', 2, 4);
    T('6.473599220485444271934675864319103330041834228257926736230815798290852055774573555100E+5717330084', '2.54432686981949005080115591094e+2858665042', 30, 3);
    T('9.209741638720686763936E-1276400513685', '9.59673988327321851e-638200256843', 18, 6);
    T('4.8536505131045142195892515435263713896444446493917606442E+489', '6.9668145612643618143249e+244', 23, 1);
    T('1.78266541332477633622402E-326', '1.3351649386217331075e-163', 20, 6);
    T('9.0008752672017155633317074524390E+6471516376362717', '9.48729427560973021773087688828835539957324e+3235758188181358', 42, 6);
    T('2.752105084390660143722079680933385301574892653893872E+351993', '5.246050976106370379006538474761082685166806048583033918e+175996', 55, 1);
    T('2.422495083794980345116096890428444142323454774977228510500281601544E+4152692574419', '4.921884886702e+2076346287209', 13, 6);
    T('7.70318284811421110789713417832012949193926384337E+93357154016321', '8.77677779604463445931868214909075634912291e+46678577008160', 42, 2);
    T('5.552E+3632919736', '2.35626823600370252922256562034250417311635e+1816459868', 42, 3);
    T('4.153785710453062855830599732347469624736520483873168356235994388488862010735050E+892421551', '6.444986974737080766810771623032945260441071075e+446210775', 46, 0);
    T('5.605E+5675632812', '2.36748812036e+2837816406', 12, 2);
    T('9.42966806244765077846048695980361182251360849643886949915047546718447826210831580612914211E-70', '3.0707764592e-35', 11, 5);
    T('7.114353872030280365988875E+67571893501', '8.4346629286714e+33785946750', 14, 0);
    T('2.1755E+29685947206', '1.47e+14842973603', 3, 3);
    T('3.1957416164026041488064710595803317913510936087326E+670', '1.78766373135514615127590298211801815259228392431608378911e+335', 57, 5);
    T('6.889011005574444132777283904963940668985824922494660705303433367179790882388068464811133291E-22245567', '8.3000066298614750761803698640611912593886475990802e-11122784', 50, 2);
    T('7.49773480466228924183406217955548387605474066242543E+832760452', '2.738199190099633e+416380226', 16, 1);
    T('5.494919442288998416712912089649171267855041680293157882234998769530130E+2596425', '7.41277238e+1298212', 9, 5);
    T('9.888E+4074331960288', '3.144519041e+2037165980144', 10, 5);
    T('7.445087402697457306999759256881915086282566E+7741679357', '8.6284919903175765230036e+3870839678', 23, 4);
    T('7.4215236554929332558721151202413802608518312907457946553651956762905846430086668143238396E+284233947489426', '2.7242e+142116973744713', 5, 6);
    T('3.587741969703932853498962153401963517215180078944373194302586019144083507357057836071900067100536E+63118681', '5.98977626435573372452e+31559340', 21, 6);
    T('7.65629872801256586254250327882669708758111314709326E+52040', '2.76700175787e+26020', 12, 2);
    T('5.44739510393955953793608434598635194710985947044023366946203E+77912369', '7.38064e+38956184', 6, 3);
    T('2.47074913173163578130952088004108E+948852246172', '1.5718616770351123301507553721e+474426123086', 29, 1);
    T('5.4303E+99165', '7.3690569274500790763668250886492837e+49582', 35, 6);
    T('1.517655185431552826354337815151066019863831881135082412474956409828745604786E+136686467', '3.8957094160519118810401390713117724292106e+68343233', 42, 3);
    T('7.85061870146143442116093900894835834105448562653604302124057378583801172400142245524E+4628023385337935', '8.86037171988932e+2314011692668967', 15, 6);
    T('2.4192E+7570', '1.555377767617886455431832184661119197849249033694003e+3785', 52, 4);
    T('1.58085756518029370781570135783897499298287E+592797910', '1.257321583836169459964056e+296398955', 25, 0);
    T('3.65783426742790972632703576990173544628580515359174630022857770729964806516047116317430769E+640', '1.9125465399377631161653097184741665721253446136954721e+320', 53, 5);
    T('3.564963257788426097621257565520611983869641359491973703397854980297356036E+1459009106', '1.888111028988609583145e+729504553', 22, 5);
    T('5.1572774887284456827384151538945E+690272608346', '2.270963999875041102e+345136304173', 19, 4);
    T('6.538008086079810913559430509913323776531429834294387511394939967993589628E+9402', '2.5569529e+4701', 9, 0);
    T('6.51385643609209961519163129501727601406409175357051743734834101369339806985293192264E+773152455997', '8.07084657027507840802880266001e+386576227998', 30, 3);
    T('4.1002729581171802047466910110703462446557929607129454415E-3854816', '2.0249130742126142449662227312745844e-1927408', 35, 3);
    T('1.783389211358800356483586836499013576639093667448366523590812594838897E+1681305792', '1.33543596303184839626884172661527927e+840652896', 38, 5);
    T('1.8701051558099E+715557041215616', '1.3675178813492348627206e+357778520607808', 23, 5);
    T('8.604580570582888425819467758E+896161806956614', '2.93335653656061530351678638212815e+448080903478307', 33, 4);
    T('9.90091432735959351225557752494222497164893000937827341618534898331017489407022E-766262739485', '9.950333827243985241e-383131369743', 20, 5);
    T('4.7488991107742067670580909E+1348', '2e+674', 1, 5);
    T('7.35838879252594474825531673336778736320201817667556597615E-1036439997873', '8.5781051477e-518219998937', 11, 1);
    T('9.33827208790589095937620691505742001048457364171086110482100633468785745949866903244021671972E+92066312', '3.055858649857007204727182e+46033156', 26, 2);
    T('4.48745134238725571781296250535963115788143112035E+782914143996814', '2.11836053172e+391457071998407', 12, 3);
    T('1.994289744164783259741485688177325751307412501442299447296420632871E+24690235411', '4.4657471314045349333787892399868034606248150785003727605e+12345117705', 56, 1);
    T('9.84698131988307574207013355317243890E-47727917273', '9.92319571503206609110502197212410989067944167033261565718e-23863958637', 57, 3);
    T('6.647831304195853521766574852501787543210772975500716604888526554135165043', '2.578338865276605475729441068503281691641e+0', 40, 3);
    T('2.14082278E+99243', '4.62691e+49621', 6, 0);
    T('3.070463886E+581634896351', '5.5411766674597192452940661868e+290817448175', 29, 0);
    T('4.327641722273459645398037809890921E+4340295526448610', '2.08029846951668443643196158191277004952813682123e+2170147763224305', 48, 5);
    T('49763.803238135686683868943219008987900255166', '2.230780205177903372456015834148410622250607643958e+2', 49, 5);
    T('6.20656218495872591E+8292310800702', '2.491297289557937505283558662e+4146155400351', 28, 4);
    T('6.72921867254639574404868458922942427394861080883099422779297042688279273296942444278339371141E+9412661', '8.203181500214654975050885387211057e+4706330', 34, 0);
    T('4.572791768706831715253851567E+21841661', '6.762242061e+10920830', 10, 0);
    T('6.307503915000762424826163834453670653766914785978689064E+471702185365390', '2.5114744503977663250103830013364071874588852432768574426e+235851092682695', 56, 1);
    T('4.704537833318613287253723975961068963510297788209157E+88520772050', '2.1689946595873e+44260386025', 14, 1);
    T('5.7568820615424131877204842546492373692646211790113238250107140283261857931747075827992397752896253E+10924', '2.3993503415596509e+5462', 17, 0);
    T('1.995298163814444884362E-59280', '1.4125502340853032064113917250520718464208375748549e-29640', 50, 6);
    T('8.0828638526931288346009737687608647274020750531754411029487806603579224074056E-48883470', '2.84303778601219594036743221600632971058691869e-24441735', 47, 1);
    T('8.2193623042261414129453289243663612025051427153405589E+66618842931', '9.06607e+33309421465', 7, 0);
    T('10920.539104498625142055406567733447849484729158951211635149275126377822214420015897', '1.04501383265957896802771517846705356495474346985988947e+2', 54, 5);
    T('9.859E+51869080441', '9.92924972e+25934540220', 9, 4);
    T('1E-662404', '1e-331202', 30, 1);
    T('8.275865535264119827502852854543505645595605921816445902604299669E+721', '9.09717842809742e+360', 15, 3);
    T('9.96466475855133264812459570359644858240864647658634035118201559072908089345E+622603164978751', '9.9823167443992342233518776966825682634e+311301582489375', 38, 6);
    T('5.20245035948145821805092195325552849151551123542621107187489054552244321405835428472872693306E-10421022540', '2.2808e-5210511270', 5, 3);
    T('8.440337186226020264861286665996931836533929569514370E+361', '9.19e+180', 3, 5);
    T('1.163281E+8538591', '3.41069054591588534517e+4269295', 21, 1);
    T('5.7528E+928955029332651', '7.584721484669031668793732902885789924639088038617e+464477514666325', 49, 0);
    T('1.984749324606319241623746E+492444886', '1.408811316183369972870845911436924188664247236e+246222443', 46, 1);
    T('1.3684198207725413637693034307515768513504448316845750424298132022E+7784340733', '3.69921589093221398019e+3892170366', 21, 4);
    T('5.9005339416302925205864521551022253E-46839', '7.68149330640227305574e-23420', 23, 4);
    T('7.4212994370574839514704803506143448068128840273E+41248218482', '2.724206203109e+20624109241', 14, 5);
    T('9.09203645782785110534601658325600950389458829210574357680480121537625607749180269672550030285502E-8', '3.015300392e-4', 10, 1);
    T('9E+77473529613', '9.486832980505137995996680633298155601158665417976e+38736764806', 49, 5);
    T('0.00001088873842754897288693803271407816415673593995183591942551875258619057', '3e-3', 1, 5);
    T('2.45117185288680027624714620996E-862895', '4.950931076966028898272047997634385e-431448', 34, 2);
    T('5.90732819263386142392768556953367415759246157476789637134221E+79000011428', '2.43049957676068345863168466672832948477923734031907489e+39500005714', 54, 5);
    T('608973973.559137584875308863615', '2.46773981e+4', 9, 2);
    T('8.27640921691822301344113811021584619201568042833519575511123791775471516612050185022752666E+646405171545946', '2.877e+323202585772973', 4, 0);
    T('4.277066009E+67789826626', '2.06810686595253099362751854367520128e+33894913313', 36, 3);
    T('9.8782424499026651185728054056251434709646592770875681587031479072616747747001454570170E+573722', '3.142967141079057657187029956999674773494e+286861', 40, 2);
    T('7.13037700754447285014285548347022747856525830962293501458031351388234783059268574102345E-5848580', '2.67027657884805499226898027594e-2924290', 30, 2);
    T('2.922278605199787630304867885909773373974449977870310638688E+454', '1.709467345461675565114e+227', 22, 2);
    T('5.68220200536945892056688445935889035696154238616131657475623691995256733062108152113201E-91747822295', '7.53803820988555278931491025365572578127e-45873911148', 39, 4);
    T('5.9449233250687226680091681667433932971227844252778711969878349127405741403268108312433141894809E+727469968836166', '2.4382213445601534779476108646284142950017e+363734984418083', 41, 6);
    T('5.2345725304517230004230671337149130955907634128E+56144161', '7.2350345752122864e+28072080', 17, 5);
    T('7.20285573962155232994963487105877460287502941329082945121164728271E+457', '8.4869639681228483e+228', 18, 1);
    T('4224495.95894160125965710774738870660317543250200355', '2.05535786639251736405475139439883223751044014e+3', 46, 5);
    T('9.1833781260402963681098E+914284125636826', '3.030408904098636053918e+457142062818413', 22, 4);
    T('4.84838E+3739367', '6.9e+1869683', 2, 1);
    T('2.81299972619449521865420575887930000794092957521107236941640422E-75269509748', '1.6771999660727683244685e-37634754874', 24, 1);
    T('7.5029308044429169367550991202625755580814420898', '2.7391478244963189530491e+0', 23, 2);
    T('3.8369934770723776049086698402629318248312546748406637E-640501239722', '2e-320250619861', 2, 4);
    T('0.2321074234545198863890375874530156046299', '4.8177528315026694236455612593706137891795980252597e-1', 50, 5);
    T('6.71832E-61', '8.196535853639633778137691300468060229355334465e-31', 46, 1);
    T('2.0957655313595604554973316994384295245384818279401199065324891241480E+33821003639', '4.57795317948923594656689314110731879601016221408884915e+16910501819', 54, 4);
    T('3.390600672960915311893098205284006882773230821901200542616504416E+7640384', '1.8413583771121023446704452333787589415987331662739511921288e+3820192', 59, 1);
    T('2.639618859305855876948558351729E-38098', '1.62469038875285e-19049', 15, 0);
    T('2.89503002982E+6187430695646593', '5.3805483e+3093715347823296', 8, 6);
    T('4.07270645584188751324E+34', '2.0180947588856890723997299206680032e+17', 35, 0);
    T('5.4225412023822653900723542047638269834874E+9531415', '7.363790601573530207271751096129865e+4765707', 34, 6);
    T('1.58912806234973797336257718436193055577544368619566342153156557139988030947558481733747035166312E+68317578386', '1.26060622811000659174e+34158789193', 21, 3);
    T('1.7722160155210612814272517E+984', '1.33124603868e+492', 12, 3);
    T('7.841907356414355351370177997277144662382279060714610117E+428975915779193', '8.8554544527168990184529851935662461820059526523e+214487957889596', 47, 6);
    T('1.2927807379404742215955246331869587E-978', '1.1370051617914820273929613383262762066e-489', 38, 5);
    T('2686.171008145923880849716255872853407227250546116927496527883', '5.19e+1', 3, 2);
    T('7.938294963156640337787724273035381689056977355660017186316135854374E-5103125', '8.90971097351459562921854e-2551563', 24, 2);
    T('2.44811895437570769603445913326178325581347465500139E-5138', '1.56464659088744628115889679321099199736613e-2569', 42, 0);
    T('2.316361209293616391100059325879380667458121324E-5971060', '1.521959660862802366e-2985530', 19, 4);
    T('1.6346341280319728642058748453423528427775241741249588871222594E+9950687555669', '4.04306088011542668658076515699765003743e+4975343777834', 39, 1);
    T('9.04489710473186480412E-640', '3.007473541817427993201389144977809e-320', 34, 6);
    T('4.0640394879273833787382149801380613578353069666928179940840894943310871769954605362041E+551884581702503', '6.375e+275942290851251', 4, 2);
    T('2980437.5997280049767582421668', '1.7263943928685603145e+3', 20, 5);
    T('4.745135E+9795871', '6.888494029902326916874054398275545295098608257e+4897935', 46, 3);
    T('5.9182163506875549546680576005992720902216355454971484890245944725555E-92280748829', '7.69e-46140374415', 3, 5);
    T('3.303589774800085601596674E+979761455691700', '1.8175779968958926637429567383064725023e+489880727845850', 38, 2);
    T('79553272734900230263.41763128677730026531493235388640071381802652591', '8.919264136401625271884219060858252e+9', 34, 1);
    T('7.205710371469E+89130', '2.684345427e+44565', 12, 4);
    T('8.722622591246883914133438987850898330671972588389730184309208739334634887878346367329202E+450220156', '2.9534086394e+225110078', 11, 4);
    T('4.970012461272812033092318803761409304947044073671573877964943E-58', '2.2293525e-29', 8, 0);
    T('4.5082744268662442849713715921E+258260', '2.12326974896414051580077536570530341003518e+129130', 42, 3);
    T('6.6738268050802343551355260020766554923882450343012342734378848E+37826792', '2.5833750802158470838638220584e+18913396', 30, 1);
    T('109.8855089805002144728403440947030093340246871753530', '1.0482628915520200483e+1', 20, 5);
    T('2.24331941382337940650496041556953252175484515884385876892851088684239094331708000189663420216840E-989669311041694', '1.49777148251106029879789692329117828615e-494834655520847', 39, 5);
    T('4.8366003779897772450979302026969538397871395603455798715295598491356099722574E-77254982845569', '6.95456711664340560996070853420178762114841e-38627491422785', 42, 3);
    T('9.6192494360944667705321067387464882437575358E-8', '3.10149148573625247e-4', 18, 5);
    T('3.83184936239138541387679462617387283711241414931326900113204911095477961E+515842994', '1.9575110120741046328968e+257921497', 23, 0);
    T('6.0327727774912176230101E+2882947903', '7.7671e+1441473951', 5, 6);
    T('2.40280850307738845039513019493014927568794808938729593338641386181207198737391954656000091310896E+9520', '1.55009951392721507762173021041426457646284757348955667596e+4760', 57, 3);
    T('4.8683370006224153769363821987754694567429898400322317885097538299174433336292355572118157532513893E-57', '6.97734691743388618569498e-29', 24, 4);
    T('2.2480E+4679221067152566', '1.4993331851193049612203995625656077760057082447306644612348e+2339610533576283', 59, 3);
    T('3.65654283E-439', '6.04693544698469e-220', 15, 1);
    T('2.4300466966592452130220374934414E+461964862951', '4.92955e+230982431475', 7, 6);
    T('9.46438764886203575285E+8363931971652', '3.0764244910060828198383307889390376085e+4181965985826', 38, 0);
    T('5.5522725470052584411450816E+587', '7.4513572904573959e+293', 17, 0);
    T('1.550886563744476367740974406041099100674095756961953547612335899E+84', '1.245345961467927340281476373719978090064081519e+42', 46, 1);
    T('2.40221400019421603693694579636622069903401E-8', '1.5499077392523e-4', 14, 5);
    T('9.43631065159385809410362801786965384978640704874013E+25838', '3.07185786e+12919', 9, 0);
    T('5.5756717804303509418805557E+563953670', '2.361286043755e+281976835', 13, 5);
    T('3E+811230914259', '5.47722557505166113456969782800802e+405615457129', 33, 6);
    T('2.9510767649015787272590443004662758146251322168953340785510127474489250982992775086288323854164153E+289123223169490', '1.7178698335153e+144561611584745', 14, 2);
    T('4.84515086803287598716648233379771635161708841282201798840424143306973602425697887937893645861532E+24757918696', '2.2011e+12378959348', 5, 3);
    T('9.435314E+56219023076234', '3.071695622941830478526650605376658041302645824732e+28109511538117', 49, 2);
    T('5.325547125476089750844425629104815411440491176993481E+568', '2.307714697590689989764e+284', 22, 0);
    T('6.6887132461730491917584965749675495749719281864675587283858319408183402627833958E-67547666', '2.5862546754279728496914821072086e-33773833', 32, 3);
    T('3.295554729322001530304774161762307365140729343455866587659018918667945699638475469150683153123653323E-55', '5.74069223118780469804395537303763292412320069e-28', 45, 5);
    T('6.422957969154553102396E+51886342692', '2.53435553329728248339409747785119424231260889160560714e+25943171346', 54, 2);
    T('1.979619230295024840325033863086398472578753367030579325792446632992E+228904', '1.4069894208184455429860946489038258613329117231491e+114452', 50, 0);
    T('125.058148097218257166460091', '1.118294004710828525178841841394721727540545598338176097e+1', 55, 4);
    T('3.467355417124563636358862403139990401147625419807197821070468E+4130408716972827', '5.888425440747775688e+2065204358486413', 19, 0);
    T('9.3381340355468272974654269623217477382915449047483811417E+404', '3.0558360616281147776649165585589148e+202', 35, 4);
    T('7.2112757E+2774907791', '8.49192304e+1387453895', 9, 4);
    T('2.88495794741340509204059372399499124467343954532861012543791870383E+7873674', '1.6985163959801521e+3936837', 17, 3);
    T('2.4911E-248903967', '4.991092064869170494e-124451984', 19, 0);
    T('3.57887079780417347338180368431509195268242653234667944437089019357950841991042084010E+4455801915', '5.9823664195735900053440901869404771842353600892916293363151e+2227900957', 59, 0);
    T('3.6456162796951313121087727108063524737724086412612828706606881122179782708607529986314383405E+845109329430', '1.90934970073455410278410575623977888122382805484e+422554664715', 48, 1);
    T('6.75641355423662185068716628560209294936028E+796934', '2.599310207388995318083e+398467', 22, 6);
    T('1.416250413871436327348908304665878915237348394E+1557200879514359', '3.7633102634136297417364966e+778600439757179', 26, 0);
    T('1.45131851760962261131924534974E+62722943220', '1.2047068181e+31361471610', 11, 4);
    T('1.90053822003230473596535984483816E-42305849800043', '4.35951628054340993989341222113828625466454197009e-21152924900022', 48, 6);
    T('2.486438927889398461388938E+670714680520852', '1.576844611205999099662237540201844126209316481620105534e+335357340260426', 55, 1);
    T('6.47961009652494321346331325444614339994562352820030434758628977937286889619097368135E-828068', '3e-414034', 1, 2);
    T('7.535112732490029474794930517417328483099871453263111571224994801873955541077813E+58986805697912', '2.7450159803706115172884307982554068123557452994357916574e+29493402848956', 56, 1);
    T('7.1414468471857852311230634694477427663911004991250862311231174531532437446075636178902521E-6712441917', '8.450708163926728117884231904577733365414280601e-3356220959', 46, 2);
    T('4.91586962013551338731303957E+4466056', '2.21717604626595076e+2233028', 18, 2);
    */

    log('\n ' + passed + ' of ' + total + ' tests passed in ' + (+new Date() - start) + ' ms \n');
    return [passed, total];
})(this.Decimal);
if (typeof module !== 'undefined' && module.exports) module.exports = count;
