var count = (function times(Decimal) {
    var start = +new Date(),
        log,
        error,
        undefined,
        passed = 0,
        total = 0,
        n = 'null',
        N = 'NaN',
        I = 'Infinity';

    if (typeof window === 'undefined') {
        log = console.log;
        error = console.error;
    } else {
        log = function (str) { document.body.innerHTML += str.replace('\n', '<br>') };
        error = function (str) { document.body.innerHTML += '<div style="color: red">' +
          str.replace('\n', '<br>') + '</div>' };
    }

    if (!Decimal && typeof require === 'function') Decimal = require('../decimal');

    function assert(expected, actual) {
        total++;
        if (expected !== actual) {
           error('\n Test number: ' + total + ' failed');
           error(' Expected: ' + expected);
           error(' Actual:   ' + actual);
           //process.exit();
        } else {
            passed++;
            //log('\n Expected and actual: ' + actual);
        }
    }

    function T(multiplicand, multiplier, expected, sd, rm) {

        if ( sd != null ) {
            Decimal.precision = sd;
            Decimal.rounding = rm;
        }
        assert(String(expected), String(new Decimal(multiplicand).times(multiplier)));
    }

    function isMinusZero(n) {
        return n.toString() === '0' && n.s == -1;
    }

    log('\n Testing times...');

    Decimal.config({
        precision: 300,
        rounding: 4,
        toExpNeg: -7,
        toExpPos: 21,
        minE: -9e15,
        maxE: 9e15,
        errors: false
    });

    assert(false, isMinusZero(new Decimal(1).times(0)));
    assert(true, isMinusZero(new Decimal(1).times(-0)));
    assert(true, isMinusZero(new Decimal(-1).times(0)));
    assert(false, isMinusZero(new Decimal(-1).times(-0)));
    T(1, N, N);
    T(-1, N, N);
    T(1, I, I);
    T(1, -I, -I);
    T(-1, I, -I);
    T(-1, -I, I);
    assert(false, isMinusZero(new Decimal(0).times(1)));
    assert(true, isMinusZero(new Decimal(0).times(-1)));
    assert(true, isMinusZero(new Decimal(-0).times(1)));
    assert(false, isMinusZero(new Decimal(-0).times(-1)));
    assert(false, isMinusZero(new Decimal(0).times(0)));
    assert(true, isMinusZero(new Decimal(0).times(-0)));
    assert(true, isMinusZero(new Decimal(-0).times(0)));
    assert(false, isMinusZero(new Decimal(-0).times(-0)));
    T(0, N, N);
    T(-0, N, N);
    T(0, I, N);
    T(0, -I, N);
    T(-0, I, N);
    T(-0, -I, N);
    T(N, 1, N);
    T(N, -1, N);
    T(N, 0, N);
    T(N, -0, N);
    T(N, N, N);
    T(N, I, N);
    T(N, -I, N);
    T(I, 1, I);
    T(I, -1, -I);
    T(-I, 1, -I);
    T(-I, -1, I);
    T(I, 0, N);
    T(I, -0, N);
    T(-I, 0, N);
    T(-I, -0, N);
    T(I, N, N);
    T(-I, N, N);
    T(I, I, I);
    T(I, -I, -I);
    T(-I, I, -I);
    T(-I, -I, I);

    T(1, '1','1');
    T(1, '-45', '-45');
    T(1, '22', '22');
    T(1, 0144, '100');
    T(1, '0144', '144');
    T(1, '6.1915', '6.1915');
    T(1, '-1.02', '-1.02');
    T(1, '0.09', '0.09');
    T(1, '-0.0001', '-0.0001');
    T(1, '8e5', '800000');
    T(1, '9E12', '9000000000000');
    T(1, '1e-14', '1e-14');
    T(1, '3.345E-9', '3.345e-9');
    T(1, '-345.43e+4', '-3454300');
    T(1, '-94.12E+0', '-94.12');
    T(1, '', N);
    T(1, '    ', N);
    T(1, '\t\t', N);
    T(1, 'ertgrt546', N);
    T(1, 'qweqwdewee', N);
    T(1, true, N);
    T(1, false, N);
    T(1, 'e 4.3', N);
    T(1, '4 .3', N);
    T(1, '4.0 01e', N);
    T(1, ' 4.001', '4.001');
    T(1, '4.001 ', '4.001');
    T(1, ' 4.001 ', '4.001');
    T(1, '    4.001', '4.001');
    T(1, ' 4.0 01', N);
    T(1, '4. 001', N);
    T(1, '4. 001 ', N);
    T(1, '  4.001e ', N);
    T(1, ' 4 .001 e ', N);
    T(1, undefined, N);
    T(1, null, N);
    T(1, Number.POSITIVE_INFINITY, I);
    T(1, Number.NEGATIVE_INFINITY, -I);
    T(1, new Date(2012, 11, 4), N);
    T(1, new Object(), N);
    T(1, function () {}, N);
    T('0', 0, '0');
    T(0, '+0', '0');
    T('0', '0', '0');
    T(3, -0, '0');
    T(9.654, 0, '0');
    T(0, '0.001', '0');
    T(0, '111.1111111110000', '0');
    T(N, '0', N);
    T(-1, 1, '-1');
    T(-0.01, 0.01, '-0.0001');
    T(54, -54, '-2916');
    T(9.99, '-9.99', '-99.8001');
    T('0.0000023432495704937', '-0.0000023432495704937', '-5.49081854961890952566173969e-12');
    T(NaN, NaN, N);
    T(NaN, N, N);
    T(N, NaN, N);
    T(N, 4, N);
    T(N, '4534534.45435435', N);
    T(N, 99999.999, N);
    T(Infinity, '354.345341', I);
    T(3, -I, -I);
    T(-Infinity, -I, I);
    T(-I, -Infinity, I);
    T(I, '-999e999', -I);
    T(1.21123e43, -I, -I);
    T('-999.0', I, -I);
    T('657.342e-45', -I, -I);
    T(I, 123, I);
    T(-0, I, N);
    T(100, 100, '10000');
    T(-999.99, '0.01', '-9.9999');
    T('10 ', 4, '40');
    T('03.333', -4, '-13.332');
    T(-1, -0.1, '0.1');
    T(43534.5435, '0.054645', '2378.9451295575');
    T('99999', '1', '99999');
    T('3e', 8, N);
    T('-3..0', 13, N);
    T('0 0', -0.4, N);
    T(' +3e0', 4, '12');
    T(9.9806, '+ 1', N);
    T(' +2 0', '1e1', N);
    T('e3', 4, N);
    T(' ', 0, N);
    T(323, null, N);
    T(undefined, undefined, N);
    T('undefined', undefined, N);
    T(null, null, N);

    T('-19.851', '69.11', '-1371.90261');
    T('-0.0251', '-1.09821', '0.027565071');
    T('-2.11', '-21', '44.31');
    T('-1', '7161', '-7161');
    T('-4.61', '1', '-4.61');
    T('11', '-0.0000271', '-0.0002981');
    T('-1', '0.0000224801', '-0.0000224801');
    T('-3.6', '-3336', '12009.6');
    T('0.000000219', '-110.42', '-0.00002418198');
    T('312', '22.87694', '7137.60528');
    T('0.29976', '-5', '-1.4988');
    T('0.00000000000000000010018', '-74.6', '-7.473428e-18');
    T('138.583905', '9.73380', '1348.948014489');
    T('5360260.6', '-12.12', '-64966358.472');
    T('-7.01', '-0.0000000988077', '6.92641977e-7');
    T('0.0000202', '-56583117802', '-1142978.9796004');
    T('116727.96', '0.0000000000000019308', '2.25378345168e-10');
    T('-136636.46', '6.5636554051', '-896834.639212729946');
    T('12.1712598149', '53.270389', '648.3677449597909961');
    T('76', '76019.61784', '5777490.95584');
    T('0.0000714939', '1425786', '101.9350017054');
    T('-3221.71685', '-230', '740994.8755');
    T('0.00000000237457904', '-892315', '-0.0021188724960776');
    T('6511.4', '0.0000000000000186', '1.2111204e-10');
    T('0.000000011188562', '0', '0');
    T('-25379.8', '3', '-76139.4');
    T('41.9279', '23.4198661', '981.94580385419');
    T('0.0000010', '58236.222', '0.058236222');
    T('0', '-16.39257632', '0');
    T('0.000000004846959', '1', '4.846959e-9');
    T('0', '-30878.5', '0');
    T('-8065.5', '-0.00000000058314173', '0.000004703329623315');
    T('-443850607', '-1', '443850607');
    T('-0.070884', '0.0000000000000000002106611401', '-1.49325042548484e-20');
    T('15510739.6998', '4.13', '64059354.960174');
    T('-1182.8', '0.00000000000006116702878', '-7.2348361640984e-11');
    T('0.000000000000000024429766', '1.492', '3.6449210872e-17');
    T('-521641.171', '0.000000044916564267', '-0.023430329181534636657');
    T('-1.4', '-50.3861389', '70.54059446');
    T('-906235823', '169450901242', '-153562476945135592166');
    T('818.5', '-0.0000000000000002572', '-2.105182e-13');
    T('0', '2', '0');
    T('0.00000000000000000723208622905', '7281.27407', '5.26588019315858457335e-14');
    T('-8.30486506', '-1110043.7', '9218763.139203122');
    T('1.8', '0', '0');
    T('-0.000000000000339156493611', '-0.000000000000000000167265', '5.6729010903843915e-32');
    T('0.00000000000000283', '4.074', '1.152942e-14');
    T('39.85', '33086328', '1318490170.8');
    T('4.1183805089', '-5.126069', '-21.1111026568765141');
    T('-464.2025675', '0.00000000000000000106831881858', '-4.9591633849340270415e-16');
    T('0', '-0.0000000000073', '0');
    T('-0.0000002358363833', '-37968.0', '0.0089542358011344');
    T('3.4', '-32545254.52', '-110653865.368');
    T('0.000000247', '12.88', '0.00000318136');
    T('-446908', '0.000000490399599', '-0.219163503989892');
    T('-0.0000000015', '91', '-1.365e-7');
    T('794881.3', '45.2898548', '36000058.66023524');
    T('5', '3046563.8', '15232819');
    T('-95094833817', '3.101', '-294889079666.517');
    T('-12786.5', '-0.0000000684878615', '0.00087572004106975');
    T('66754192.630', '7', '467279348.41');
    T('414951476.81', '-12.875', '-5342500263.92875');
    T('-2.486307', '0.00000000000131610556936', '-3.27224248983875352e-12');
    T('-0.000000000000000477405019421', '-1.1', '5.251455213631e-16');
    T('-367046.223', '-0.0000129', '4.7348962767');
    T('0', '0.000000000000093338247', '0');
    T('0.3341376', '-8', '-2.6731008');
    T('422580779', '-10829.0', '-4576127255791');
    T('-11.6639', '-56.51836497', '659.224557173583');
    T('-1', '-36577.4', '36577.4');
    T('61.5', '-68269.0', '-4198543.5');
    T('-18.10728', '0', '0');
    T('531751766', '41.50116', '22068315121.04856');
    T('-7.63315110', '3', '-22.8994533');
    T('1249.58475875', '38582.13', '48211641.6081111375');
    T('-2', '0.000000000000001022', '-2.044e-15');
    T('10.750', '0.000000000014145865', '1.5206804875e-10');
    T('14', '0.044276206', '0.619866884');
    T('12175855.915', '52.1893', '635449397.1047095');
    T('-1', '-0.00000000000029721', '2.9721e-13');
    T('311354.171224', '194857.1428', '60669584203.5706187872');
    T('0.0000008188', '6375.414052', '0.0052201890257776');
    T('0.00000055577528074', '3', '0.00000166732584222');
    T('526170.543', '6065.11546', '3191285094.94589478');
    T('0', '-27078031462', '0');
    T('0.0003450371', '4044.753', '1.3955898453363');
    T('0', '-5.681398758', '0');
    T('-2830933', '-1', '2830933');
    T('2.8', '14.36338836', '40.217487408');
    T('725.6556', '-0.000058977271', '-0.0427971869738676');
    T('0', '0', '0');
    T('591.1', '-182368.115408', '-107797793.0176688');
    T('1', '-8.44', '-8.44');
    T('-48988695', '3.99', '-195464893.05');
    T('-54145231.29', '-164.1', '8885232454.689');
    T('1.9615', '370.5', '726.73575');
    T('-2317170.34468', '-0.0000117290196265', '27.17813645069548986202');
    T('-3865301765', '9.90928729', '-38302385651.92906685');
    T('8.9648', '-3', '-26.8944');
    T('-10645.1', '-4208', '44794580.8');
    T('-390304.9', '-9.540', '3723508.746');
    T('0.00000000043946', '-0.0000000000135', '-5.93271e-21');
    T('11.167962', '907.089236', '10130.338118257032');
    T('328719912.6', '1.13', '371453501.238');
    T('8.0902', '3', '24.2706');
    T('-0.0000000000000000000103885880', '54949757', '-5.70850386173116e-13');
    T('-60096046', '147.9066', '-8888601837.3036');
    T('-1.1', '-4', '4.4');
    T('-2.3781033', '1.488', '-3.5386177104');
    T('18.64', '-13492494', '-251500088.16');
    T('0.0000000000000000078080114', '1362291', '1.06367836581174e-11');
    T('0.0000006699', '-0.0000000000000059', '-3.95241e-21');
    T('-279.40', '-4199.537182', '1173350.6886508');
    T('-346081282.623', '5715.5169', '-1978033419605.4328287');
    T('26098.51437', '-0.00004277160560', '-1.116275363379572472');
    T('-0.000000000038243914682', '6258350.6', '-0.0002393438263964435092');
    T('-0.23215784', '1.7', '-0.394668328');
    T('270.08569', '73.4', '19824.289646');
    T('0.000000000000000253826', '-1', '-2.53826e-16');
    T('1773.911', '167', '296243.137');
    T('-609057.1664', '-2', '1218114.3328');
    T('178326076.8', '-299396.226', '-53390154391306.1568');
    T('-99153591.8357', '-67230.718614', '6666167232273.3544197198');
    T('0.0000000000001596', '-0.00349909609', '-5.58455735964e-16');
    T('1.84', '0.00000000003464', '6.37376e-11');
    T('73.5', '0', '0');
    T('0.000000000000000049', '-198604', '-9.731596e-12');
    T('1052854077', '2.8', '2947991415.6');
    T('28.3', '5964', '168781.2');
    T('2085.7', '-338.192', '-705367.0544');
    T('59.12705', '-0.0000000000016294', '-9.634161527e-11');
    T('-0.0000000000000629', '-0.0000133901915', '8.4224304535e-19');
    T('0.000000000000000000010713336246', '12.8722', '1.379042068257612e-19');
    T('0.000620507896', '3.9', '0.0024199807944');
    T('3', '9.6', '28.8');
    T('-1', '7', '-7');
    T('4.07', '4', '16.28');
    T('-2473.25538', '12.3', '-30421.041174');
    T('-1310633', '-9', '11795697');
    T('-3.90', '3031.9', '-11824.41');
    T('150.0276', '0.00897', '1.345747572');
    T('2', '-8.8', '-17.6');
    T('-3030148.8', '229674', '-695946395491.2');
    T('2', '775.7549', '1551.5098');
    T('-613183.18515', '-3', '1839549.55545');
    T('-0.000000000001224', '367.376621', '-4.49668984104e-10');
    T('-0.00000000000000000737581', '1.3490', '-9.94996769e-18');
    T('0', '-1.15490644', '0');
    T('8.437', '-12103106905', '-102113912957.485');
    T('155.8091088', '3.465', '539.878561992');
    T('179142.3', '0.0060', '1074.8538');
    T('-301.3766', '9.6052', '-2894.78251832');
    T('48.15', '518165.306', '24949659.4839');
    T('4387518.42726', '-8069', '-35402886189.56094');
    T('-28.3', '-77.67', '2198.061');
    T('9', '0.00000000000001022', '9.198e-14');
    T('-1071.62294', '0.0000010', '-0.00107162294');
    T('-4.15', '-3', '12.45');
    T('-661.167', '453', '-299508.651');
    T('-104436', '-1098478.95443', '114720748084.85148');
    T('0', '3.630588', '0');
    T('0', '-13181687.0', '0');
    T('-1.111', '-3.78770', '4.2081347');
    T('0.000000042325027395', '2.89323850', '1.224563987727687075e-7');
    T('-21.7', '-0.00001926174637', '0.000417979896229');
    T('28151356.2', '0.00000000003236052', '0.000910992525337224');
    T('0.0120', '-862436.56', '-10349.23872');
    T('3649.71055', '-692509.6', '-2527459593.09628');
    T('-3', '-1', '3');
    T('-1135.91888551', '12.37808265', '-14060.4978485386674015');
    T('1', '2172752.822', '2172752.822');
    T('-0.00000000000001091997', '-117.1', '1.278728487e-12');
    T('-605.91910660', '2036476909', '-1233940269312.8094994');
    T('-85793626', '-403556966', '34622615410698716');
    T('387.034155496', '1.51', '584.42157479896');
    T('-212937', '7812.350', '-1663538371.95');
    T('838667.59', '-297101', '-249168979656.59');
    T('0.00382632083', '0.000316896', '0.00000121254576574368');
    T('0.000000000000002582', '-295757466.2', '-7.636457777284e-7');
    T('-0.000000000000798713', '21.52', '-1.718830376e-11');
    T('0.00000974', '-2.9', '-0.000028246');
    T('0.0000000000281', '38.82', '1.090842e-9');
    T('-3.237', '16967081.13', '-54922441.61781');
    T('9.8', '-0.0000000001333', '-1.30634e-9');
    T('5614.9', '-76.1668', '-427668.96532');
    T('-1.8', '3545.0927', '-6381.16686');
    T('0.00000249806256', '-1131.1175', '-0.0028256022777108');
    T('-0.20706', '11338.58322', '-2347.7670415332');
    T('1908.3417145', '-138.575', '-264448.4530868375');
    T('0', '0.0000669436', '0');
    T('325.22245849', '-14504.56', '-4717208.6625157144');
    T('-13.8825', '0', '0');
    T('0.0000000000869261985', '1.1243', '9.773112497355e-11');
    T('2588680', '-2.639437', '-6832657.77316');
    T('847973030.3', '1300.4', '1102704128602.12');
    T('-669.7', '-1150.41', '770429.577');
    T('-1', '-6.761867', '6.761867');
    T('-4', '-128.68', '514.72');
    T('-0.0000000000000000148', '-389336', '5.7621728e-12');
    T('5.1', '8.1', '41.31');
    T('959895377.84', '976.5', '937337836460.76');
    T('152', '-1325188.6822', '-201428679.6944');
    T('-1013851164640', '-116895', '118514131890592800');
    T('-0.555030', '0.0025719815451', '-0.001427526916976853');
    T('-8.61456', '-0.000000000050', '4.30728e-10');
    T('-0.10822', '-42439', '4592.74858');
    T('5', '248', '1240');
    T('1.00560719', '-0.00000000000000538', '-5.4101666822e-15');
    T('-0.0000000000000000880', '20.26149480', '-1.7830115424e-15');
    T('-215.544104755', '-4060438.71', '875203626.65949706605');
    T('0', '13031.6', '0');
    T('-0.0000000000000047877659854', '-208.19', '9.96765000500426e-13');
    T('-4', '-3160383', '12641532');
    T('-0.00104402960', '-6', '0.0062641776');
    T('-6', '-221362529.2', '1328175175.2');
    T('-5658.7', '-0.000009428149284', '0.0533510683533708');
    T('-3', '-0.0000000000000000171257563600', '5.137726908e-17');
    T('1577941089.0', '-0.0000000000000054178560', '-0.000008549057596685184');
    T('-3', '-19154.88287', '57464.64861');
    T('-5.3', '-0.000000000000000094150109', '4.989955777e-16');
    T('-275.9', '-618029.6', '170514366.64');
    T('-0.00277412', '-2.0', '0.00554824');
    T('20896.13', '-2', '-41792.26');
    T('0', '144.50', '0');
    T('0', '37399', '0');
    T('0.0000000103050173', '-6.752', '-6.95794768096e-8');
    T('-1679844', '0.00008795', '-147.7422798');
    T('1731362.632', '-0.0000000010867236', '-0.0018815126323525152');
    T('-116536', '8818.8637874', '-1027715110.3284464');
    T('112359.12', '-6.3736', '-716132.087232');
    T('0.02767509', '4.002781363', '0.11077733447134767');
    T('-942581.9', '7.504907', '-7073989.4993833');
    T('6603793', '1.604', '10592483.972');
    T('91689.093', '5', '458445.465');
    T('4351694708.6', '0.0003242690', '1411119.6914630134');
    T('-2.38', '-39', '92.82');
    T('-759.5', '0.000000324', '-0.000246078');
    T('-6.1623', '-10.1485', '62.53810155');
    T('1819.975133', '-1918.4931', '-3491609.7348320823');
    T('-544', '-2373759.39656', '1291325111.72864');
    T('-2143.2', '-0.000000000000000000162130', '3.47477016e-16');
    T('0.621559060', '53.8', '33.439877428');
    T('-4624.6', '0.0000000000000190537313036', '-8.811588578662856e-11');
    T('-6', '-50.943', '305.658');
    T('-3021.479', '3', '-9064.437');
    T('-965', '-0.000415509858', '0.40096701297');
    T('0', '6465897.6979', '0');
    T('74528068.573', '-119403349', '-8898900982117850.977');
    T('64.56', '0.000000017086', '0.00000110307216');
    T('-9626.07', '-1051902.812582', '10125690107.11121274');
    T('1.4514', '-1', '-1.4514');
    T('0.00120840179209', '-492240', '-594.8236981383816');
    T('-1.163', '-3.5', '4.0705');
    T('-390918.9935', '-3.9645296', '1549809.9209329576');
    T('1130.013737', '3', '3390.041211');
    T('-5', '-6733821263', '33669106315');
    T('27774.358123', '-0.0000000000000128441670', '-3.56738494049618541e-10');
    T('0.363631323', '-3819136980.1', '-1388757832.7919876723');
    T('-0.0000000065598472', '53857.08', '-0.000353294215438176');
    T('-240695', '2', '-481390');
    T('0.000059', '0.00000000000017987993', '1.061291587e-17');
    T('37.17', '-61.7668', '-2295.871956');
    T('1.475572354', '266461086.2', '393182612.2135309148');
    T('-0.000000000003907', '-31.5', '1.230705e-10');
    T('-0.0000000000000000000894018', '0.00000000000000001599913222', '-1.430351218905996e-36');
    T('0', '-0.00005850856211', '0');
    T('32.75', '-2.57293', '-84.2634575');
    T('0.00000131', '43', '0.00005633');
    T('-129602.62', '-0.000000000000000123490', '1.60046275438e-11');
    T('-2941.6442', '2.013048311', '-5921.6718883729462');
    T('1.01983639584', '10710047', '10922495.73175700448');
    T('0.4077750076', '-3', '-1.2233250228');
    T('0.359', '207160.173586', '74370.502317374');
    T('-26222.646879', '9.8', '-256981.9394142');
    T('-2', '-1.1360', '2.272');
    T('6717280984', '564735824499', '3.793489214890694027016e+21');
    T('-7', '0.000000000000000000179598', '-1.257186e-18');
    T('-0.000212', '754.3', '-0.1599116');
    T('-8.6', '-44061.183', '378926.1738');
    T('0.000000000000000000013', '956', '1.2428e-17');
    T('-3', '3.8968771877', '-11.6906315631');
    T('-4485.1947117', '444609963508', '-1994162257095211.5806436');
    T('0', '-3353199.6579', '0');
    T('0.0000000000000000000293', '-4', '-1.172e-19');
    T('-225', '5.09648', '-1146.708');
    T('0.00000036757', '0', '0');
    T('-78694.836', '-43.8', '3446833.8168');
    T('0.000000000000018028989162', '495', '8.92434963519e-12');
    T('-4036233', '-0.0000000000000000000134620360255', '5.43359140533119415e-14');
    T('-2.19', '-481624.5154', '1054757.688726');
    T('-6', '-33692.483', '202154.898');
    T('-0.00000000387466078726', '137145711.9', '-0.531393112039787150394');
    T('-52.6685', '-107972.8', '5686765.4168');
    T('-7.137137210', '3.06', '-21.8396398626');
    T('-174.1619', '-2084784202', '363089977710.3038');
    T('11', '-0.15', '-1.65');
    T('177.3211', '-85578219.525', '-15174824022.2144775');
    T('2', '-1', '-2');
    T('-681.685707574', '222753546489', '-151847908952971.868407686');
    T('2225288.04', '-2945.03251212', '-6553545626.6317910448');
    T('-829419822664', '0.000750870', '-622786462.24371768');
    T('-3.887', '-0.0000000000215299', '8.36867213e-11');
    T('14780708.1', '-2000820.86', '-29573549092050.966');
    T('-65280872367', '2756.0', '-179914084243452');
    T('-1.1', '-16652196529', '18317416181.9');
    T('6153.32891899', '3.07', '18890.7197812993');
    T('1.6664', '10.5', '17.4972');
    T('-335.16960', '25.16', '-8432.867136');
    T('20.7', '834', '17263.8');
    T('8029.51', '-1', '-8029.51');
    T('3544067.44', '0.00000000000000000108040', '3.829010462176e-12');
    T('-0.00000000000000861840983', '3', '-2.585522949e-14');
    T('-1', '-5.4071', '5.4071');
    T('-0.000000000000014799305202', '-77.4875919308', '1.1467625223519416640216e-12');
    T('0', '-31045268779', '0');
    T('-437', '640.03', '-279693.11');
    T('60', '0.000000000000000371281463', '2.227688778e-14');
    T('-1', '-0.019533338878', '0.019533338878');
    T('-1117.048', '8.967', '-10016.569416');
    T('-0.0000000000001992331', '17762516', '-0.0000035388811264796');
    T('-1.209', '-3960.47030', '4788.2085927');
    T('-994748.5', '-10809803.2', '10753035518495.2');
    T('-3.0521', '0', '0');
    T('4553015.460', '-6', '-27318092.76');
    T('0.00000000403775332', '0', '0');
    T('10628.68130', '-1.196558', '-12717.8336389654');
    T('-0.00000000003389', '-113275.58', '0.0000038389094062');
    T('0.0000000000000000000810', '-0.000000002954', '-2.39274e-28');
    T('0', '0.00000002678', '0');
    T('12309', '25980042.9', '319788348056.1');
    T('-7.6', '0', '0');
    T('-61.22', '-349.3', '21384.146');
    T('-1.845209', '0.000000052784913420', '-9.739919730680478e-8');
    T('395.326251', '-371872028', '-147010774681.007028');
    T('332.72', '3715', '1236054.8');
    T('-1681921.54', '76348.91498', '-128412884660.4906692');
    T('517583042', '-13.9', '-7194404283.8');
    T('-21.723616577', '-0.000013157028', '0.000285818231564853156');
    T('-116.13334149', '0.0000000006536', '-7.5904751997864e-8');
    T('-4.1', '-294.0876', '1205.75916');
    T('-22.05', '-6.7', '147.735');
    T('-22.9', '-0.00000217178', '0.000049733762');
    T('-50.5', '0.012432393', '-0.6278358465');
    T('-2', '-0.00000000000119764950', '2.395299e-12');
    T('19381.0141', '65.916071785', '1277520.3166816971685');
    T('-4.90', '-22.5701', '110.59349');
    T('0.00000000000192902409442', '289522860.5', '0.00055849657378990048841');
    T('-27006298.37', '-8164.7', '220498324301.539');
    T('-8776869', '-143972778', '1263630212072082');
    T('-79325483.3', '248065.2', '-19677891879911.16');
    T('-26.2', '28441.8', '-745175.16');
    T('18.71', '97792.4647', '1829697.014537');
    T('-150.68501', '-1.3460', '202.82202346');
    T('-43.0091494663', '-5.6724', '243.96509943264012');
    T('12.359', '-1.261', '-15.584699');
    T('0.0000000048694525', '-13', '-6.33028825e-8');
    T('-26799', '-3.0401', '81471.6399');
    T('0.000000000003271187802', '0', '0');
    T('1', '0.0000000000250081008', '2.50081008e-11');
    T('23.590033588', '0.000000000000000880594309', '2.0773249326711650692e-14');
    T('2.34', '-35556883', '-83203106.22');
    T('127410324', '48.0', '6115695552');
    T('2.4', '2.0', '4.8');
    T('17', '-0.00000000000000000467960391085', '-7.955326648445e-17');
    T('-5827035', '63.01', '-367161475.35');
    T('-452352446648', '13167459.5', '-5956332520963450756');
    T('0', '27.51619908', '0');
    T('-6.8253', '-203490405', '1388883061.2465');
    T('-663513.1', '235798', '-156455061953.8');
    T('45846991454.7', '0.000000000000011371170', '0.000521333933819940999');
    T('-6940453.30', '-0.0000000000000000000429869089', '2.9834863373180437e-13');
    T('-299.7692790', '-2.0', '599.538558');
    T('-0.000000000000781997707', '-0.00000000002498', '1.953430272086e-23');
    T('-211968.876', '-11448', '2426619692.448');
    T('-2', '224979.5', '-449959');
    T('0.0000000000007372', '3', '2.2116e-12');
    T('2.14', '-96.5220369', '-206.557158966');
    T('246.3484', '-119174406049', '-29358424251121.4716');
    T('2301100.91737', '-431951.5', '-993963992909.347555');
    T('-13819448.15', '0', '0');
    T('0.0000000155919108142', '8.51', '1.32687161028842e-7');
    T('124616457', '513907736', '64041361285211352');
    T('-5298.3584942', '-4.1932', '22217.07683787944');
    T('158463114', '0.0000000000000000001405398', '2.22703743489372e-11');
    T('-65667302.5', '7', '-459671117.5');
    T('-1693481.437', '-27743783', '46983581502656.171');
    T('-0.0000000000000003099934391', '125', '-3.87491798875e-14');
    T('-0.000000000000004272052', '0.0000005661', '-2.4184086372e-21');
    T('12.95933', '209420271.72', '2713946409.9091476');
    T('3.1', '-1.31573054798', '-4.078764698738');
    T('0.000086866', '-103057641552', '-8952205.091056032');
    T('11.1', '660.353292988', '7329.9215521668');
    T('-92.940180', '-1', '92.94018');
    T('97808795959', '-97', '-9487453208023');
    T('0.000000000000000388699', '-399', '-1.55090901e-13');
    T('227578760.552', '-40082.64751', '-9121959239968.50902552');
    T('-4.31547', '-29.771127352', '128.47640695373544');
    T('27.8', '60.0346682', '1668.96377596');
    T('12.7', '60805396.70', '772228538.09');
    T('6.808549', '434235.18', '2956511.50055382');
    T('-0.000000000000000000764', '798867', '-6.10334388e-13');
    T('0.000000000166682635066', '21.7', '3.6170131809322e-9');
    T('-17.0856584365', '-73.70395', '1259.280515120874175');
    T('-0.000032761108', '137.87702888', '-0.00451700423385679904');
    T('6.7467', '0.000000000000000000017375144', '1.172248840248e-19');
    T('156.9', '-90.01049', '-14122.645881');
    T('-294748892', '-46912.02', '13827265916481.84');
    T('-1145.2', '77', '-88180.4');
    T('0.000000000000000000049300', '336.35', '1.6582055e-17');
    T('1.0', '4', '4');
    T('2791.3', '372.0576', '1038524.37888');
    T('0', '-1.1', '0');
    T('-67196.498357', '36318.97052', '-2440507642.87511143564');
    T('0.00000000484838500152', '-0.00004296', '-2.082866196652992e-13');
    T('-1488.65084', '-2472469.6620', '3680644039.21081608');
    T('-0.000000000020', '-9.9', '1.98e-10');
    T('-2841.5725628', '-0.00000000000000000049499064214', '1.406551827547777476392e-15');
    T('75.099315636', '0.0000000000000014', '1.051390418904e-13');
    T('3', '-2.7', '-8.1');
    T('0.0000096', '-0.000000000027265607', '-2.617498272e-16');
    T('3256481', '-8.7', '-28331384.7');
    T('13650.44', '-53', '-723473.32');
    T('6', '-8.1', '-48.6');
    T('12.819672478', '-1.0', '-12.819672478');
    T('0', '-0.00000000000000000001480', '0');
    T('-0.00000000932497', '1', '-9.32497e-9');
    T('0.21304', '-0.000000000000159938', '-3.407319152e-14');
    T('-40', '-10124.4', '404976');
    T('-138.46343', '-72.2', '9997.059646');
    T('5', '-65625447009', '-328127235045');
    T('127.3760', '-572993.9', '-72985671.0064');
    T('3793.636', '-14016.5', '-53173498.994');
    T('-0.0000342', '-3.21', '0.000109782');
    T('1.054585905', '1', '1.054585905');
    T('-18462488840.1', '6264.198', '-115652685667176.7398');
    T('-55413', '-9808.30460312', '543507582.97268856');
    T('-1.11918007', '0.166606078', '-0.18646220203846546');
    T('-1.6', '68.418', '-109.4688');
    T('73090459.50', '2990.3', '218562401042.85');
    T('672.231', '19.8', '13310.1738');
    T('-0.000028', '4.8', '-0.0001344');
    T('-0.13900815895', '103606359', '-14402129.22010276305');
    T('0.000000000000054', '-0.0000000000000000492', '-2.6568e-30');
    T('-0.0000000000000000000484559', '5.193', '-2.516314887e-19');
    T('418.8538369', '0.0000000000000079', '3.30894531151e-12');
    T('0', '675966', '0');
    T('106621', '13.56840585', '1446677.00013285');
    T('12.198402747', '1', '12.198402747');
    T('65.314804858', '-0.0000000036124', '-2.359432010690392e-7');
    T('-16207.72', '-0.0000000000052877', '8.5701561044e-8');
    T('-26109.4977159', '-60149.26838', '1570467185.380666093242');
    T('-0.000000000000030215', '2614373', '-7.8993280195e-8');
    T('0.00001270329688', '2444.29', '0.0310505415308152');
    T('-871.457466', '2348528202.8', '-2046642436441.6221048');
    T('-0.00000000000000000063864554', '397.146', '-2.5363552162884e-16');
    T('-24310822171', '20.60', '-500802936722.6');
    T('-19.860', '47497302.8', '-943296433.608');
    T('-3.55', '0.402200053', '-1.42781018815');
    T('1311.50024013', '4.4', '5770.601056572');
    T('-361', '0.000000007669817618', '-0.000002768804160098');
    T('-0.00000001735304418', '164354775', '-2.8520556717689595');
    T('0', '194.3', '0');
    T('-3.7', '-1475870.0', '5460719');
    T('-5888926.57', '-1', '5888926.57');
    T('-11.644', '-83797427.047', '975737240.535268');
    T('-0.005286102', '25.9', '-0.1369100418');
    T('0.0000000000000545367483', '-176.8675', '-9.64577832995025e-12');
    T('-112.5258392', '0.05538900297', '-6.232694041650542424');
    T('-2032574', '4.7', '-9553097.8');
    T('44405.894', '8', '355247.152');
    T('1430.7023529', '15', '21460.5352935');
    T('-0.0000000000000000010538367753', '-5.05921', '5.331581551965513e-18');
    T('1107.1', '-660486.6', '-731224714.86');
    T('-0.000000001271832580', '0', '0');
    T('0.00000000003329420451', '-4.58', '-1.524874566558e-10');
    T('-1.1331472', '22318736.7503', '-25290414.05613954416');
    T('-1320771192.0', '0.000000000000000144797721037', '-1.91244658612921966104e-7');
    T('6.0088825', '-2.06', '-12.37829795');
    T('-0.00000000000000130', '-14.851119896', '1.93064558648e-14');
    T('1', '-54959924930', '-54959924930');
    T('1.22', '-0.000000000000000000447152061675', '-5.455255152435e-19');
    T('0', '2460.4318049', '0');
    T('-48926117262', '0.00000000000019101657111', '-0.00934569915711302150082');
    T('-1.9', '-0.0000000000152246057022', '2.892675083418e-11');
    T('-0.0000000000000000262046', '-18965.321', '4.969786506766e-13');
    T('-1.5349403', '-3984579.1', '6116091.03912773');
    T('-0.00000012', '663568053', '-79.62816636');
    T('-54228857.6', '0.162776', '-8827156.5246976');
    T('0.0000000000000000001875935465', '14.25202', '2.67358697658893e-18');
    T('-13.1992', '-8.93496384457', '117.934374777248344');
    T('1.4', '0', '0');
    T('383.512', '-4258970898.7', '-1633366447302.2344');
    T('0.00000000485581523775', '0.026476965410', '1.285672520872576762275e-10');
    T('-113.2617', '-1.3158604', '149.03658586668');
    T('229813.79204', '-4.40918501496', '-1013291.5280939017289184');
    T('-0.0000735399', '6', '-0.0004412394');
    T('20950', '18.05', '378147.5');
    T('-224.09235215', '-0.0000000000000000000330291504', '7.40158000265211336e-18');
    T('-1212.5', '-89187.79', '108140195.375');
    T('0.000000005360379781', '-5.853730207', '-3.1378217045031744667e-8');
    T('1.721', '-59400291.7', '-102227902.0157');
    T('3', '510831.1558', '1532493.4674');
    T('-79731.7076', '-0.00000000199301902', '0.000158906809743878552');
    T('1', '7', '7');
    T('1557292', '116069515', '180754127153380');
    T('13174514.5181', '0', '0');
    T('11.5557387', '-0.00000000000003525009362', '-4.07340871023257094e-13');

    T('59429490631462462063898399891833', '85.00891132032463441584595955', '5.05203629890205609747397099201262363916734883085091009335515e+33');
    T('-0.000000000000000042109349317791897577655162141761006017167', '-13008827246961207056988713615056579178448396076639.2633976291', '5.477932507570985534957063193452340625430871365299670940723162464475095465880668702087454656216987597e+32');
    T('8591855.554373470161526044552421978378391766637549711729320', '-4.18144454943', '-35926367.5773248178055078206847122253956635626957707604812328535202876');
    T('-2283433141014849078.33114178969776355920', '-453.97', '1.036610143026511036089988438269093722970024e+21');
    T('-834090482593334176100123.7933568971710257756', '-2311068.681', '1.9276403914416302737519348190500409072951706328939836e+30');
    T('189532.2', '2968613550973672055370920.6', '5.6264785726585220673297239734332e+29');
    T('5.5765', '206791541', '1153173028.3865');
    T('223004397.925983046564773982957421494273449', '-0.00000765764399', '-1707.68828752147254136863163630226113151989615142151');
    T('-9', '-861605.04321559065136010082968313802', '7754445.38894031586224090746714824218');
    T('0.000000000000000000028090298007095127212699390661641605', '-6917964065283873879.11378929124954598', '-0.1943276721961993069547944346297165198267413742521737520171142577284979');
    T('-12902.95204428144349964281942884855605291385221512', '936511255610118037649.1', '-1.2083759820067154006199526430520916600744894425420426553143497592274392e+25');
    T('0.000000054421393267623019090', '0.0693540337668146867938144', '3.774343146319828330922834963772562065116896e-9');
    T('-0.00000000007483922', '11963887185334360.01202905021298539073645325022668', '-895367.9851184189424994447352806605141113868134295543896');
    T('-136818356017962729836947190424992612288254', '0.000001250702751', '-1.71119094258963391622539632506259119343595682786754e+35');
    T('89886141515.402264741', '165920325588054483132342147189817', '1.4913937866089484761163201125053970911515450613342397e+43');
    T('-4910559.69867254476135299557110880352698481', '-0.000000000000001251373615473295746877725841601861845423548', '6.14494484412532005943449278569391549262463086794696425713068030612078654982181230588e-9');
    T('188388334026044880874021952902236669508901633565820006', '-1428610610603915751990.6', '-2.691335729036024155624090667679961618627273975496253699574106150646158039436e+74');
    T('2569238997875.3944611244282198384307876980880850', '-45354701.13646736794503189060', '-116527066896795434703.141029468202286044275680813636330729593599972883501');
    T('-20273871998054251005', '-0.00001018031347', '206394372190847.50529696253735');
    T('0.000000000127882266384995374972957752031634640610096', '56084430050054721499878916115459816143192971582.9', '7.1722040237117373866688717282573432808847848059761445842371981626319357370527308468409584e+36');
    T('954627502168195877866647607616261900931', '9.96209474259629834949998607', '9.51008962048762055845159388440432218351076192328571371885622003117e+39');
    T('27452502565473996463690067929.18801932606917287121704780', '-15999511778834005759916561695256821.291408', '-4.392266381547719678372690910805800180664096055687896418712826017984317410615883824886612653024e+62');
    T('24739272583405.47186561318831947000688841802596024659835438', '0', '0');
    T('-14628477765152447836551575115758391.03361640644696947989', '1896549844807094292937664785889813670098706728.8', '-2.7743637235263904499250092914009953677085505785952534916815852412079339245487773243303684333379283832e+79');
    T('-0.0000000065376926', '400694118953983940534743481756583594', '-2.6196149763489805485528325035782515637752044e+27');
    T('6159556.6345747363', '12428.3903194610108', '76553374049.32049544464848654145204');
    T('1', '-214152.677228898078143751453160882566438516677', '-214152.677228898078143751453160882566438516677');
    T('1234245458508975352.84372029387871714', '711.09788699', '877669337572736089357.1591886677375895858960086');
    T('-12435092008252.828827', '-0.000000000000000130972832307119388', '0.001628659220320498210781727802616997876');
    T('1222155005674.47827107178853812873657', '-1.9217613579329307675966802225522', '-2348690263309.514070047950271986297573047714307144600821921806873954');
    T('-9610326782.75790', '-1925856563138573766.221666413429', '1.85081109084807161320054127390660041302158391e+28');
    T('1897933.1084909834795612591951194123752666123', '-19444.86721593176768385769', '-36905057279.327795521442821058258453663270782572671411150172721603587');
    T('0.000584193054195683632690208', '-15427224.032385517626932779704097', '-9012.477125240345688831570386428613444841606254309382176');
    T('-9193123344.97483292546402603464388933977', '3', '-27579370034.92449877639207810393166801931');
    T('-208355', '18.5077', '-3856171.8335');
    T('0', '-7.9', '0');
    T('-146.9974821', '27.85613660690160', '-4094.78194224817268246136');
    T('0.0000002221395758167297538432', '-11018022952341415341382543191308.248305367013916352', '-2.4475389449721144325377299337884145675514764946137036517351977553240064e+24');
    T('-62634425863308865890231971109828114770592016347.2857222333045', '-0.0000000000044191162667', '2.7678881018796339994185288016784587422371771822944265151385376844431015e+35');
    T('0.014785725196930589141008331862', '1405730020396633.72', '20784737782660.25819955773732132563645741958664');
    T('3060880958477929473167827.136432332645114184906595927797020', '-134990224.755007177950183445', '-4.131890085332574931410011369432785413898866996911658019131150911202211096837243339e+32');
    T('-251157.56', '-35553.75153', '8929593483.1210668');
    T('10660825.6584256928', '-54.80641419272238648411', '-584281626.672080870616452029619482941408');
    T('0.00000000000000000018196806510154644625549186906147312734531', '290466772346954491087868523893813560578350699536244189', '5.285567654026668587345538632825206167793448119823731989523008725260266169443990119026948390359e+34');
    T('801946936195133455122.3263199289', '3353455956475898285353850471499087737905097359235396953.528', '2.6892937299611674351748616442698675657013699635861635172559630870783247800626545335641592e+75');
    T('88876291341359997.22', '-242985336224765018278774996045975', '-2.15956355339905308693512776069018053139845369921895e+49');
    T('-677.8706245402', '-40041.8855', '27143217.9316521785471');
    T('3.1852520702012', '0.00000998870874036575237809441602188548308619567919990770190850', '0.0000318165551938868335179942506676244968068581347935045219680140052189902');
    T('-2897031868709978067512290.19826378134251899155435029', '-0.0000000000000313426136', '90800550447.862773034512424935249089493462002949664538517944');
    T('-286245211287.816038991926811942762853058522756387871', '-905725.51698541', '259259591978255100.08416583456412844730858064475954566698860796211');
    T('-301283745846510635986864779646783.59226306843', '164455867753899', '-4.954787986333307027587976990843711993792791415951383630857e+46');
    T('16627939773', '-24446.50', '-406494929660644.5');
    T('2835613541302657637165162939561661439198237', '-34702274210562114517526690656890036690211485', '-9.8402238665467925464317665495284869217720281504532926830801758242091305082784369151945e+85');
    T('0.000000000000167329000479220734904842317239446253923254', '-335462.843', '-5.6132662217107750155727738207852552086794690651122e-8');
    T('1606900543967463817538016297544283302866330961.38117', '-12172415089417043266077235.5931339752547820885496331722', '-1.9559860428582011548778770036179464667831089769951903599695351323242935536918541220406695361968160447474e+70');
    T('25175391597018424433550189069315474558138086224811394.9498', '2697842514071148.65', '6.791924175882585620347727294751149080055684613881481427569426305249508777e+67');
    T('33605644614556574.500804536703659893409117374', '-32.543409918351171133101366277860678601667282494', '-1093642268261945046.079483918411013199812861395190797870188216612955697572700923777661450756');
    T('-16194637333856709349021519708.8552324881761540256020238975', '-468.317375400951782165492196622874246631857734334382', '7.584230051762041449005379580598839525796458346850505580761259136785876093027076066542485326077054219893845e+30');
    T('45938105933592236851533187176.70670204189811056', '-23728.6', '-1.090046940455836751355290385241202650071383506234016e+33');
    T('0.0000019359500318287540988972385182980431073131086249', '-1.433', '-0.0000027742163956106046237197427967210957727796846594817');
    T('3.383292222691', '-0.00000000000000000008416407858503881', '-2.8475167250671594974339763771e-19');
    T('-1781216226076936151337005862078252102700230837', '-139003643961.53840777847576996', '2.4759554610811353724544585499901643072996118138112041996180785490331025652e+56');
    T('-4942215729757062263164723577.197780606517', '-15241705.579', '7.5327797060859772211128133542568249856768162658343e+34');
    T('20212448898283508798.6625218057881201405', '0.000000000000000000473337907308896740021174588136160473489005777205', '9.5673182631015315198993602152061224371243489547543953762429365302559862133611162973025');
    T('0.0000000000000000000336083585075308914988259705527435231286920', '-1042029834722073772731648833.0978', '-35020912.2608826168412442748054117197175486895048242174316983271784220776');
    T('2376653285350901814422.788590453876', '-7116898082702.215', '-1.691439920976175340680854757410757210391140053534e+34');
    T('-0.0000067077602282', '10325935005362413727223741709915.3224340753', '-6.926389614794815252662513824455946141306951378398346e+25');
    T('118620396179863704912953340.8484888231431511', '0.00002664482432944840555300828814', '3.160619618102041148915830017998160333002812044061355761312703976357954e+21');
    T('-0.0000000000152038475299141990408716170165065', '0.000000000000005856774815228245375399878344009978104812414293866282', '-8.9045511307771647940461821641351376995341987855772716940834996589432243659697783833e-26');
    T('-552929555032574543198346', '0.00000000031628803556881', '-174885002769189.22345730619750676118826');
    T('0.000000000000000045838', '3539579713.731326460', '1.6224725491801654227348e-7');
    T('1392529024903490704995506782982906.381429', '-696.803141131', '-9.70318598668840847367530799623705904459432004456199e+35');
    T('0.00000017046264046382452', '77820777096586290650583567570533323989282', '1.326553514683081867495692236827890756347852656781320879464e+34');
    T('173.2', '400230281235477723305446756183676238679630391', '6.93198847099847416765033781710127245393119837212e+46');
    T('441.339801980', '3859926956832539877913158087979570115173.37608', '1.7035393987857371577386465661853388929161435925152686384e+42');
    T('146444036588433930865962234014095372651938826216413', '-52287041949860150983876138.09252718579', '-7.65712548440629977355786783708824277254343945936126868688503521615280343003456839837127e+75');
    T('-152996991626635397079841283901340.65158799', '139164451.4055465586500905725256511816907743', '-2.1291742406419715449781823729219758981469966625530964344454535418866326578427680657e+40');
    T('-2392095943969007513831013778238', '283213.66941826693939894680290224', '-6.7747426989201568951320097088428601075588412800688116615345312e+35');
    T('-15192560288102281367737239879605.47240096086', '-81553445736510122', '1.23900564105440798918085016560369292282227123306201591582492e+48');
    T('1.7067661013360140299015089144175628', '-8518082392971818901480808577069938345874733', '-1.45383742767114563412540521449653172561766842289564535605710831784007839607324e+43');
    T('111116813960956208515375359292296052638950332201721672', '-6723639335232846900768096240894018745358463215413', '-7.47109381153635523540473499319528741906993971058857734750058361188185043050515216086975306279606530536e+101');
    T('-19017520.49675261317806237143645102654220495050', '4439261178196804.5636234243239638305', '-8.442374044679588557398641259522603028251217372386419857709859483039768004289025e+22');
    T('17312453768.05379', '1726.44336801935101862147113581125657128654142147602', '29888970991998.0896280136870221704774967797321308901016337045551158');
    T('-2', '-8.63876', '17.27752');
    T('734608.2', '0', '0');
    T('4.1351657034625927353146715451605855050687982255845069770', '29034498780299324669067501513285520', '1.2006246357352024765485891656111571028229380504313388083084112136514462841267544463307304e+35');
    T('8113870024531627718699652079910.7068190690295904975254176585', '16923534495569343509614671152502455712381.505727413', '1.373153592527270771522358333649474524182371877317191394431842566433920202044363434189424789373922626777224605e+71');
    T('-10.01', '-4864429445729841.069760518121786', '48692938751755709.10830278639907786');
    T('-2257142436339514797101935880.78', '361.115', '-8.150879908987438859554655755878697e+29');
    T('2334599.31164827194227', '-29960204503173960913.6', '-6.9945072809951386426110474550748535657872e+25');
    T('0.0000000000000042471515826788460724024393725389035009593598059706180678', '828398091382.612675070213', '0.0035183322649037987809384707757042363075619742175811901626183872501809043713944414');
    T('-0.0000000000000000000284493127383335026590175755701910596367998889566726', '-0.00000000000000000013237', '3.765835527173205746974156478226190564123201301194752062e-39');
    T('-0.025286262', '-0.10797436581647216600797099911020807463243214320798554', '0.00273026810331915910538504877190248824967123287037864285665148');
    T('-2.16564935772054852403', '123868749404533277771925133166764554.44147780538237283827', '-2.682562775895750704487504519843216927715513029094342900139628307846103986281e+35');
    T('488112856156098659519479388744402.61', '-3114334168535743226695852106494538175980162249914143.70', '-1.520146546028510353469309736526975923164123845906295275264771051373672808879616446195057e+84');
    T('13062458273.90637045335784972', '-422551949801626.741267985176602715381458097174737967000518', '-5.51956721284152853764296726669744927261362132472755924039708895768532345915860615496e+24');
    T('-1774897196938188058987.040101468413944266333770483137507', '-546064776871.3594927107191784', '9.692088418156530694143698199431520418181775404052225935078568528036161040292642488e+32');
    T('17667764535653225248288857836.95612603671740964890', '-0.0000000000000000319890032591629766', '-565174177313.13507635499469894977748029671843469133582435048491574');
    T('-281835819.9184911063', '-7175814054420814885484554.4', '2.02240143761032232334599327109026978241853272e+33');
    T('0.000000000000000001503903188910', '2473558551492927739005724.481043938460207033289961875', '3719992.59354581446800290523978683389613115250358419250482280625');
    T('224371010310.537303586', '-8046248744083810452149728.077812213', '-1.805344959919976464767416627850984855572049419079495818e+36');
    T('21241841371778633003043.69805019497112440', '3019928870075002318121482.417', '6.41488500121878843990786691028203447626143414100289387726373196748e+46');
    T('-0.000000000000000017364919937061080720023417741392471545683503908', '-0.0000000000000001101', '1.9118776850704249872745782933273111171797537802708e-33');
    T('439522909103802363083392888521183473696.816065', '-2.2', '-9.66950400028365198783464354746603642132995343e+38');
    T('0.0000000000000081177461857815960900264491259569062173810350', '0.0610415335877925918303617472579311', '4.955196764565624988836180976890473307042478930774835424434663605140744766885e-16');
    T('-186798563956757871041672400714377.094855823', '-1262567588714917541983982710.193577', '2.35845812470293091731202495176066945197325419055745256762489749711235648871e+59');
    T('202739871381.909125537', '20323942031180.7301604586345358572504650', '4.120473393374958139010105109821762644139100410749336624705e+24');
    T('-0.0000000000871057582376852096451379468926610553156501430073631916', '-0.00000000000000000204556363069958967723715901466977445824097114499093', '1.78180371075520049530019591335531323553330829840490421849868550913658281328132369671225257713629597852188e-28');
    T('43650.3286859870996376890566772833', '0.00000000000000000001310330289622533562857521522228834136757813518300419196026', '5.71963478292282611553143218521780871633322080859969244917523136988043505512171959238361658e-16');
    T('-1092205601185294', '2', '-2184411202370588');
    T('5860768904954696696.24176142662833543847', '0.0000000000000048972255', '28701.506930951217005600908223400663331828964985');
    T('-85896656824567.02163635501735158916726224558523587655', '32321.39', '-2776299344922992287.4470686942774805948582718361870079644045');
    T('0.00026', '95786603846831218908589843126114668233645', '2.49045170001761169162333592127898137407477e+37');
    T('4045878451917596479346482', '-2.2547', '-9.1222421455386047819825129654e+24');
    T('-0.00007402891873793970469482766866928008041088343139', '580', '-0.0429367728680050287230000478281824466383123902062');
    T('-294517422548703614947348777318332.649869292704089396942', '-0.000000000000018464905130542546160683866082', '5438236266653724351.349152153311431266266033202633265447500622081254643984468321244');
    T('-83.506', '-8171012757524040761112.5', '6.82328591329802547797460425e+23');
    T('0', '-0.000000000000000052054568154502662326801153388437977', '0');
    T('-848457792560875.315304639536969249869368391843456', '2786837672040308423168750746536175069.979124227', '-2.364514139444808677729215519365182902026897625419714103499744958027008402375235640420961008512e+51');
    T('3114192', '0.00000000000000007442771895146280027117559688227552768887141498150', '2.31782206936893840902092874406007390124461849564067448e-10');
    T('0.00000000000000000004917302646400565894337036110048', '-422918872', '-2.079620088498342188194690499484768025856e-11');
    T('-497313848.92', '72015813415347226684601.47954307807272704460158688818499730', '-3.5814461352690899900766892687894799291942231391683303086680262807916e+31');
    T('4026.7272674898467547880850473160814529617052037657449361', '-21157682685', '-85196217784587.2940970833078499068622793775121577888984306494014285');
    T('161789945200634672.93574932528', '-8005803048483811151653134568908048072385822541265', '-1.2952584365012698158369216496463941318557463003471359935387477829065387076792e+66');
    T('-0.00000000000021548823070578223953868560097628121116323968225230682924', '-49653567182460198793826479862157.077878', '10699759340379041130.65114538401855139934323670216223964167385329900350710562551938222192755272');
    T('-5565648526946998.206396582068274786458395468246488801913', '0.0022945055453979742566004395333553200630524525428518539025', '-12770411408815.9541406930749916678604063671114068656466120089976699763004963995450912686342679576306781385154825');
    T('3.271826', '-23890.6474306628013', '-78166.0414204757505261738');
    T('-0.0000000008805699092114903173390188646922', '9284821552.25745813417849639083469216233186', '-8.175934471316238510307959904561765618357930068136953080998844682973153492');
    T('361333176.6552043788873570', '425816532338214481.0652246362221874080822', '1.538616403020706012031174663425818903103785031033415757451967454e+26');
    T('2805.024', '3138666568668923617772609.35105', '8.8040350531139788020189957723196752e+27');
    T('15516261613526.6610074517562', '-0.00000000000000000236624580694534592762621412652683109621677385955', '-0.00003671528898247448920639683215816512848818717755547603887938060370189964171');
    T('-0.000000003258489294057571', '-0.0000000000000000001263694100620034334234551033555508871', '4.117733697834092773082498224393477748428351275212341e-28');
    T('-21061048707054855665564434392186020553808698597863.879494', '181831629693595281071297779845', '-3.82956480945997219565546562178683388176250894166635033461551679226752688144082199843e+78');
    T('-0.0000000000000004150087739116129831368627055614', '-1518028660414097725821098740744462450073.304250189994494', '6.299952131211430047024557585815135401837971947761937657558761468690680184406091789316e+23');
    T('-329353403844205335310.931518048486742714840512646142', '0.000000000004737902605006197777999824176859382087', '-1560444350.041118731605073747643365481135986445785795088564232590700292190358983404458354');
    T('-36414599958286461846528123709871289767333710768520', '-11982666807671338.82516899790506494507', '4.363440182347893057208595495864483712233866372669435753275016489496855915098212851964e+65');
    T('119742066861.890807047', '22054829383.984629241', '2.640890854724681514607753859168565061327e+21');
    T('-484485733573777836334476960525005005848019789', '-14406177805616860655.8627771810687838', '6.9795876221485617831711438674257823866674330443283469578093046623068838325626182e+63');
    T('5988848948477.078723617', '-6.5', '-38927518165101.0117035105');
    T('-1.0', '-63379703.864490098873312332', '63379703.864490098873312332');
    T('-0.506613427677516177791327725446313117452', '-0.000000000000025064675288473359447414244747114622651598944039', '1.2698101061517425226884571964189454600296022874042153446180321519393920461477382268628e-14');
    T('-23042537.21970212804132180', '10390764065108.50823342162627716811120', '-239429567711406187063.05734442567122738046800108051885738416');
    T('9812550833139441428690193893295885866528739.860970333644852', '-6934084675959416258239634601815942.0149731256600469716450344', '-6.80410583641450037528576307977037672169914839765125030421279633360096772675136555394720496053502446210349236989229088e+76');
    T('0.0000347689', '0.000000000085057549281766287602723911010', '2.957357425222803877030347389515589e-15');
    T('17309197932605.95272164347202723030822567422328', '10372291249693770815173154703247665392420536', '1.7953604225558623153970783373103876250250653717733283278048673842023984171292886092127808e+56');
    T('-10.68586744857482180099209534824648355512284245440', '674588.26676653525901087', '-7208560.801031027382439380269948592059745431252505648164243884987079328');
    T('-158425376406388668464703992996848728.8980383980010724', '516624932849644105480296.4740763529887132394681136194056', '-8.184649944763013743333361097416811851476935800617569830052596874617614630479435342869900779234127425056544e+58');
    T('43701371988184830897740842', '7444696727806908941269732116534493096172849746185.16', '3.2534346104111212105848066810541857916878449567515264310528659180255182630472e+74');
    T('18985671206766137596695783', '-4918195942924959263752039745340080571253768938', '-9.3375251102824433332331539143355814981375199246602425748207454160988454e+70');
    T('-8164454926564686613685695436631269148628112649072225137', '3017566861500503598.7', '-2.46367886286161259704778334978048504960179943566519245177920238542851005219e+73');
    T('-167.140200050688164319586013710074703990054053511303270', '-124707.791051624249704808849761672566349962496678', '20843685.14424789642526902036712772659312890967899817539551993752122248668554302671208072486282553706');
    T('0.00013717932814418', '-236889624553106847157555062629.1419261258', '-3.2496359540522243673339058837639891572432570371217844e+25');
    T('0', '-7377966474', '0');
    T('-1165159893843574357027338982426', '-7258741844369034621359206819796872729971572863114', '8.457594876822935516127237999861301735400382433503942145536493041682252149634564e+78');
    T('127327741', '-0.000000000000001314349415125924728376790639', '-1.67353141912655226200255348893816499e-7');
    T('-19', '674693830910632347225468600360472778098653805584.98', '-1.281918278730201459728390340684898278387442230611462e+49');
    T('1115499473064514172110108362485045185533368299248344166641563', '26727699066738.5277545215239', '2.98147342251737349166633853134205297966348378257402631237265170730493167637686598378557e+73');
    T('9531022923010697243825538443.3659', '-10497868918011538716', '-1.000554293003294813441624977396834705527412812041844e+47');
    T('36948.619152016593259861704969912416', '-109424973578674628984651491823525807602358729239', '-4.043101674477927092300739573396847025094408695070482727807518216276085630888331424e+51');
    T('-0.000224994118800115645797961334226721592496197539765940949883662', '231860.641001', '-52.167280606449958627644387092820869098935873225250256147940687697225662');
    T('-81737025015671727638688.506795636777200442300', '5419138.800783060', '-4.42944283723002262118621171335050352165263579845818347438e+29');
    T('-1466856541.9549101', '-24128674849693158122765920.4', '3.539330455197531614891444994295155372575604e+34');
    T('3719432135548', '54707696881', '2.03481565841010488825788e+23');
    T('0.0000000000000000347350240379700980370747179429442891350492370249744117', '0.0005534400198558475169798243101266211771697229801', '1.922375239326751185420171365530525629522813111237423841789162415238003577693927327244648617996830717e-20');
    T('2248441073845808666637746763786280194954645599162114623.5', '-6334923145497633015473724024497.97', '-1.4243701399993365995217919270799399913979461349413953499228708640778427689308423183064295e+85');
    T('-2833.50779749', '0.000000000000000000027787099520949762309607915049358999802', '-7.873496316224179511483613883698024416527436609698e-17');
    T('0.000000000004441882942767312311877169439858885409', '-0.9432493370398481379786084999178021934162', '-4.1898031409738770478382967231410155261701150266906290976528356507851700442258e-12');
    T('-2107306488.021444213082463895093082025238625832167567', '-0.000000016561760020640192077800', '34.9007043445492445900173806825761285679208256730753889935834487955007126');
    T('-129070822817.42205', '336811489052744821.06', '-4.3472536026398913255504341077748373e+28');
    T('0.0055725231631263368659868273181674133608110946135789308', '-152412560828753089910310350.5653132', '-8.4932252556962839520515736693985586025569561774936573592091065011026694067884882312656e+23');
    T('68902.0767970349411722981874624462811895157551795177137623704', '0.00000000000298788286086686468161985148648212755894442142', '2.05871334339993176415426311877635990320333360195205545566288301397039058040718828895421507390094595733968e-7');
    T('88153860095637039945277243438872776.064214155', '106.735', '9.409102257307819458559166578448085753213897833925e+36');
    T('3.7075516820964288985011546648796', '23633186593.70572408884166096265287527701858061993092', '87621260708.792430109916115951342109977008394099657174865349158237730095429102117232');
    T('-35505760506401161908713691885731701346896104.7570686644891', '-5457648514664450630675291184021238402035211', '1.937779610897920137400487822865364585421914753043232864073023463191678738063333894770237751635257001e+86');
    T('-120.55403235772170', '6237.155894', '-751914.2934654306175666998');
    T('-50246944566472610231911381270921126440112308695', '425722751177060307857193841626947340', '-2.13912674790799614679030549405709134417922674778586880217918152980768287325891213e+82');
    T('8119749715097219059818517125264159.064450132566482', '560418609424712067.0940695322762258473926083185', '4.550458844211485491057203230529751310726950775388516102150563306194801442473073934282587480517e+51');
    T('0.00014717424858', '-7244528491321.3877', '-1066208037.026626286030354466');
    T('-42181198581127024501.50753334291048422600', '-0.008682426489650895544', '366235155926002047.202675730675546783334970920687585688944');
    T('-57.3', '-96900776925224476370583151961084756', '5.5524145178153624960344146073701565188e+36');
    T('1083976078583942678025084625.271006', '-0.0000000000000000534905554182086699326584150063088902383604755', '-57982482503.506901999157564033733434733806444040999020955818569250676306523353');
    T('1058587.736740719', '2.2', '2328893.0208295818');
    T('-337676088587247469427246609', '2694990217595978128', '-9.10033755458704844211026116003411819226167952e+44');
    T('-180266997707357275243150935948776784257767.114595713620465125', '0.00000000000000012929550822108208448723385741716775647229370346', '-2.33077130840613978574569769854573934386258812596377482210267408630672963476159438278046789605387340218325e+25');
    T('667863393175854.7645678', '16647944315358092755156.00', '1.11185525798577381655823461635163932295615768e+37');
    T('-2607482796153.37663635999672298', '67905.1', '-177061380021074655.829689213473629198');
    T('0.02649777513989417670980608', '-3.7', '-0.098041768017608453826282496');
    T('41727029296529241379347405093482273986363238008823.7495', '-0.002796170808176706621979157103112461500671267733114714664', '-1.16675901230889282855531874612261889002345278810932959186816789276573413038428661947418401753559128959112668e+47');
    T('-11232695034932926234846674550549966790.9183', '-535612384919640506677833697180021817295233383415483', '6.0163705767354292540256743939915265501958563028272108803751138896375435530577152378000080389e+87');
    T('3.481826810306751', '0.01981896398982153971994255430994006244', '0.06900620017226469113521762438686245737379444049353244');
    T('-500336401148192185730936', '-0.0000000015351207243732', '768076778560892.7643589222652324493152');
    T('0.000000000000000010152819846152021274527085743667507483', '2034271405274844529575.93770552252121', '20653.59109593400315620700194269765880413700716562682807038424236270121443');
    T('-8', '14804.920611063196753', '-118439.364888505574024');
    T('-7644880', '32.933', '-251768833.04');
    T('0.0000000000000400490024778789822215877572796299412810612705023875', '0.00000000000023561893047293924977558189853184387736389392033215150171', '9.436303130345939646688036612188083884147595770701765354726302304762424603125782629215278913888534265332625e-27');
    T('-669622098209324787.5786201166', '-442008.19657257', '2.95978456014644004507422179531566173761662e+23');
    T('3750095561598843110.3554570', '-0.000000000921072821137', '-3454111098.455188785930556597852894609');
    T('509.01003417', '-0.0000000000695541947', '-3.5403783020913832899e-8');
    T('0.0000000000000000635110973210625767420869099697944141102671291', '-302.6', '-1.921845804935353572215549895685978970976683326566e-14');
    T('-10277403.8756777588787330367830', '3.73924958', '-38429778.12561843210264377872295730114');
    T('-566623967.574094904804042946014964033941069', '-0.00000000000001662953909014460365351667', '0.00000942269541818623958668998623752446360099484415277625603154138912023');
    T('0.0000000000000000060428627571787342505274', '18.65854757486578220960', '1.1275104224320402610478848029208041233734304e-16');
    T('6026260424.00', '18673139236042341642133200165212521277908.5537452', '1.125291999700035578262744750920904785263182229257757399648e+50');
    T('1351407130504734.056748656589882', '-16213.40426049972577134801362523633', '-21910910127395164076.55243444398575049081525274237838315413681306');
    T('-1749812757762436212668429.46101826944561361814636104616270309', '-259646', '4.5433188330198551287450703583554958847579349723006019196120650614e+29');
    T('70245313181709005413989461536786.3825442617', '-509184726219286329641748', '-3.57678405606165250356068075267063781878365698344246156333141574516e+55');
    T('-16.20', '-83495.2179048376073493099472538', '1352622.53005836923905882114551156');
    T('17522790821382422258618581410703033443568', '5481690529703238.18676804112985115150792', '9.605451649954285042649291504450317447918361312248424196122189225108309342505856e+55');
    T('3', '-0.0000000000000000000298256458973606523936524717292889013', '-8.94769376920819571809574151878667039e-20');
    T('-57135112812568.466051', '2713703246.301287433494800761485396936819', '-1.55047741117257327421574162146466373193666197915868993431769e+23');
    T('0.000000000000000018433', '224.613695971469169', '4.140304257842091192177e-15');
    T('351655210258904818533038059328', '-317039471793459879795242832647363526701869253095475', '-1.114885821139012575684668438594527497497409491712701452561241865985548506983408e+80');
    T('-1317999447639091934290806166798244489675877673.24510838', '51973652129741187036804.3069426597647651181729', '-6.8501244798785198639385417890806274968448700953804306642353363730416792643087698202422857568078902e+67');
    T('416970579045160473905951389152873165527755895', '900715.23677692450', '3.755717538336730295514426258945023234083011232063852048449275e+50');
    T('337048915259856.8484', '-0.0000000007165482473721426233042063689577689', '-241511.80950813224169860503564044515006631691904953476');
    T('-124735.945324536963828', '43656784749.045127925982651536524807752332', '-5445570315501972.252440976832268388353995138128083224266646896');
    T('-229.309150279271759668506974147516151893493701324327298', '-56277750392608320423429992502109.4808173149501499937169775', '1.2905003122157966621553106534643794501210439187980937861478914275127685562045563793515134308625165671789301795e+34');
    T('-773.26721', '506120106.96085084420866486187778058202', '-391366083034.5187115273789355692667516507815642');
    T('-16179029077993579729752892854', '-0.03205622515938674669127103', '5.1863859898442754973726890489016131925911697266421962e+26');
    T('-379694977175773597575402239067729598591749627164211967089', '427.4134', '-1.622867211576197909699344273675511380147349200949881951741975926e+59');
    T('-0.000000057593691704585518467986423410653803725013114132854316233', '24689.19870977557359290982947617857947691050', '-0.0014219420989240649383441724944840134755882327254494635758934109877771747341652064425031912888381465');
    T('-104', '15074917718070702275412', '-1.567791442679353036642848e+24');
    T('-2', '-971091107444027282033715.48309206', '1.94218221488805456406743096618412e+24');
    T('1.0', '0.00000000000563083386598228603', '5.63083386598228603e-12');
    T('219100199509205287520379236835.595422414343', '249215953689501338499522.46564075955864255', '5.460326517424660882913977937259236613421658567164008732143598687724455221273009465e+52');
    T('-3956981388.58456771827964370777', '-3850656424026801387755000322904.790283281327', '1.523697580370765854419130687951064298276236466076285768791284492428581079e+40');
    T('734.960286303554479116990306863761301115192463389214', '-130000607393220114986132790422441608', '-9.5545283629357036833302332771199553820796316440028291953441171730648555190680692016112e+37');
    T('-0.0000002666120264878701311448', '322570889825538918018562439468.0160867345870', '-8.60012786223824197974573415201224841889283841235410512800651976e+22');
    T('0.0000000004031032595796909999049', '-2712762676362394173603186', '-1093523477307807.4647872649597513148096163370114');
    T('0.0000054604495949732794611395765459328765263001334150', '-298907.681810114352964823611311466161', '-1.632170330074440811222971888249135550087671561067062039655656486575779653557869815');
    T('670489501929686.291032', '627223395167741302962881364608526379428084315920', '4.2054670182466566942266803112300612071880467870075440896463575082944e+62');
    T('53314.110410802505016043147844064319142110836740', '-290216592240562577134439451.5', '-1.547263944176020278887164669780998856796671698801721678198701042353164811e+31');
    T('0.00000000000587199496778346240428737868027766', '-551005.316277', '-0.00000323550044440047912772450593052565891047747182');
    T('-89694219111352199451730456185908593505562.881', '6.32', '-5.6686746478374590053493648309494231095515740792e+41');
    T('12678032520049858276397.8', '-0.0000000000000074933793548296330814995251238051875041756343', '-95001307.14560031424594545397066655517994845175009207951656553412454');
    T('-523228.1027985605', '2361103.2', '-1235395547847.6101519436');
    T('-422058160.089943171946211', '9455722182962.552320564582039151365712747392896268147', '-3.990864706862835827499020129041439986893906907628633530927174587244916641017e+21');
    T('-265.20593471797540408456443', '-1', '265.20593471797540408456443');
    T('-117096397.7', '0.0000000000020', '-0.0002341927954');
    T('-0.00000084780432839108612959331480507149065703369783040', '-0.000010825079810960431632673973072527105', '9.177549518911214340736347681158868989278870466087868848313439029850170283692992e-12');
    T('621.284586089902789096175355124878176656898069', '0.00000000000000000001841815997332304201462126563776625573631078', '1.144291889556362115393585911981542504972877752024640554149220155131163404208000456588382e-17');
    T('62838970696961.93575529768157435967750130012543934643251895', '7799025017418', '4.900827045344027526036434980262455014867740735222443957786401853650711e+26');
    T('-247353051265858.992388279974222435772900701328741022620334222', '13.03830659799478334978416300680018326059', '-3225064920353791.19824546858361898880804876150878659765687851458354866130547449520822131539552091098');
    T('-2139441231754725391266252450690990152166777909', '26220446827901878.81347', '-5.609710505864567774726728960484243595395230887890837315312192763423e+61');
    T('14286913921.940840650', '-1242844090.352406582', '-17756406537257697634.0436449596731583');
    T('2437674767843424972272.34179653188260607311276734', '403927988104479.6110322848268399591134551070', '9.8464506462804905964641906770409128324751565746334602350400894172348609324599790162580538e+35');
    T('-282975256.593232221485634', '0.0412485784400225687488894518785', '-11672327.068171452858593608252189019369153357222063469');
    T('-256178508.2486827293794161556160000481333795995', '0.00000060622407729698677', '-155.301579786376201792071982290374904153826835212494097939398615');
    T('-92215568824012627934.74189852734', '802821222.0162469626002', '-7.4032615652217143799840729492067368601093636437189468e+28');
    T('0.00000000000631216424157187020311390', '-28896.442960029403', '-1.823990939609188046299993634327261580017e-7');
    T('-5868684173183824149727150676541321228.27946209763645', '1767678018.67', '-1.03739440114535694186833871290131127571890503338033620259725215e+46');
    T('-1464504298805396118871695503432730.5230', '2208833322892572033925', '-3.234845896720779321449912432206390896261116508330538992775e+54');
    T('195.36', '-13214599838.47575096481496546334113027', '-2581604224444.6227084862516529183232095472');
    T('-178.8581172026568', '1542628302874005920400457301.7905298303732', '-2.7591159379557450261955452194656628921038300499237551776e+29');
    T('351875781344.0280054729542235343559317', '0.0000178856834451374927', '6293538.83712970189834558679045935052951438284631044859');
    T('-307.61455274208', '0.0000002438400533036931515021270613358401352036899', '-0.000075008748937620515500399556886599434410402744867379000992');
    T('8320.987246030626877296', '0.00000000723034', '0.00006016356692446508273598836064');
    T('0.3178832760474045258804752', '57.66675963865299188481', '18.331298472973254687278485883883193525626461712');
    T('0.000000000000000263835146673878760137331888121481766389116183835336431047', '-2.082954760', '-5.4955667461965393109095371006242790355341756731284907525076043372e-16');
    T('0.000000000233346878395806113798662159173733434871136164480744154169890', '10289471873381.87490244310848109593247868', '2401.0161419951076855155738296198869665643993521903141787021173990086767696510557356775276055229452');
    T('-0.0000000000301270068082410889159', '-18488503331.384508718698847', '0.5570032657388091474587525501237955790818099673');
    T('-538244397964164173070105649994388625', '-2280.52363653', '1.22747907178713620859177763155650252117906647125e+39');
    T('11816719921.6984115', '24346834400764849090360336454820099097991.85306443422359', '2.87699723093810199016709496254407379606350048843381923576632239138827285e+50');
    T('-0.00000000000000000614056701921479118164625780', '-0.0000000000000000004526129846', '2.77930036570313218519067348427902988e-36');
    T('0.0000000000000000000211718849571685401', '-21818432978682095763187832118707.42059881629', '-461937352970.349445849940275487640330004420702796016087398229');
    T('354307691910025309530301150.530797676735046789927', '-0.0000000267343273366', '-9472177813397840300.1738065546659359293415312312254736184282');
    T('-0.00000000000000000001564140879', '0.0000000000000000012127764653067414424836974110', '-1.896953246475399564472178411611564269e-38');
    T('-2192995405.921', '-1492963957482', '3.274063099963661175050922e+21');
    T('-69077326289723825817911969975.2', '-160.497210320954020046818', '1.10867181659309712810734480548942940388027429142989136e+31');
    T('-34636483655.3', '-57826095450336990.7625812016394354487909856618', '2.00289260991541487342513087342333948596839502091235357754e+27');
    T('0.00000000000000000918981946289', '1.09999674198752514', '1.01087714686325481931551720546e-17');
    T('213150740', '-13337847744665673.1379595', '-2.84297211678281928195418951503e+24');
    T('-7062099495908.198413696', '-1.058736021405', '7476899123064.10206563783530116288');
    T('0.000095694936784775916676475094824420202134897751844986130', '-336195659475425245294044969885823799340891152365601.95879727', '-3.21722223808168692686768515627014434144106705991738803812388959163318091496999877734952100401367322242946318651e+46');
    T('26868896167028748101420733486377', '480507267329881259405862882', '1.2910699873389304567069920812073966334143151295977876958514e+58');
    T('-17.084476787578052926829789715003022556', '-42587983371473135737559540519744259126585114576504024.779', '7.27593413339692893938094555667816327719803206600514992485982114723845688369293247684119915124e+53');
    T('-76372412484569384.3511', '9012097234036431278745.3269231299', '-6.8827560730887716103930962801811543430595527769250789e+38');
    T('60654377820540327528', '-0.000000000000015117053045659', '-916915.446964550856786185798600952');
    T('-1331839674', '0.00000000618492062771114573665144', '-8.23732267252668770406834370123056');
    T('57436192370710.8629215108579', '105737032624.1773726358981981702798798174015047365867', '6.07313254651038202363973449861725599835500772846363988048609447329832343472993e+24');
    T('-965563477.0996002946011879837917175', '-4271362438679787505570708994903.70810816', '4.1242715682442838707409662264403439596770359705507151544781327654021648e+39');
    T('-0.0000000316616', '-2.4', '7.598784e-8');
    T('-2.385', '27.5499221426824606177', '-65.7065643102976685732145');
    T('-2266803.57818655', '-765116.77815066069', '1734369450442.4824101199545717195');
    T('-32731559035569126148642.446165008', '53704467616503577993.2664889330721452187634866311652474', '-1.7578309522633972201948025668355694620574985124468833198585070981942000057664555429792e+42');
    T('0.000000000000000022575861041290712552639790861752973874588224619267844', '31.3944076810173444579596934007549380765', '7.08755785280277370389292313902506994649964414578494551292873810891605957242649610987662066e-16');
    T('-11.858586370905899290715', '-991047772227373783020319210519807971818.824767583950836915', '1.1752425604652188758145553608664648912870842306418133290829487635928729138744225e+40');
    T('-2011971696758986575760307857385008882209510.6', '-5525155372891673776761633609.52070459365483366973020593', '1.1116456230453892069924547730202138929609342602851169377795116783945568725725156231784447542517858e+70');
    T('101703650999596029193296586451725488', '25620023337044890752720058833900.03506', '2.60564991207231919895254185864449131548456396842795048651050944069560928e+66');
    T('0.89310', '-4483944764830427741251990095', '-4.0046110694700550157121523538445e+27');
    T('83.0660898', '380025540261.5474990692628313817425000', '31567235653659.2200446528027713580805854765');
    T('12249047873.6982', '-142475289409910528840622723.707048987614004620751135579865', '-1.745186640801000235481534461055817862532388247975966963088442739006743e+36');
    T('-0.000498899320558508419779744140203069146893', '-0.000059386343271843835168818993', '2.9627806308777237249170821580126645410922765507304403542845338749e-8');
    T('-2297433.28538549', '-285796548778125338857408803711103260149820899037579434872', '6.5659850401116294498139439137406267368280581599271391787641901086880728e+62');
    T('34167188438641414252422741077347811417020.59175671639136', '-1876839.0519778124207491962458916326915672158166', '-6.4126313557927024892196883907714297942957465376354294400472573526515361182444034644427066219867584576e+46');
    T('12880636073104135025.684594372389347', '0.00000000000002717486381885835955193357797034462478107970045332718', '350029.53118687937886399193088901600225479341922014167752066254086091217423503253755146');
    T('1602471.60609897', '4948980398.148665647375032297236242244605323466522685367809', '7930600567173612.25671087112877473741757520842649668372026733910210605673');
    T('370646280676040583641049478094.841', '1176.901804728', '4.36214276643352994796521564113760876046208248e+32');
    T('5547404728207', '-411636889958330609055.5065082479', '-2.2835164296592677798830108543636613032785153e+33');
    T('-0.0000000000000000717041779275895653254585995816746449', '0.00321217261726873009981380480107282271306898444464862589', '-2.30326196882768081444918769519152717431393087666881910579487955526028795479141279838696461e-19');
    T('0', '-399327.602', '0');
    T('157.8', '584.2802770', '92199.4277106');
    T('-10542363099197242246550684296314627705', '-4850153229755188101.33556814417023579013954', '5.11320764348234189624759367689059634200461502178300975633252267426243050999557e+55');
    T('-2562650836927093177981.565429254128045484667799300595712', '-0.000000000000000000317395222174952988716141595565429848789500451120', '813.37313174330395999646102020198262859585476827525658527707087140151929076326008633676971110873759744');
    T('-0.0021383531474526630012273967098296654028460337299367', '-1407391260514289948513', '3009499531618102701.7588489539730050863932112182078160238774067897491271');
    T('85399454218074127051769295076.287333823347313330158', '-0.4092038950326248225015994759', '-3.49457892996962542239070012424224836131258038173055372705839875319563364641922e+28');
    T('33.127135132152', '204.48590', '6774.0320419197206568');
    T('-20073920159741041234.22437514622641538676580', '71857.713649597096', '-1.4424660066635461364336974246302806972842556483481805121168e+24');
    T('-0.000007522084816187962395438426656921829236', '256085.25', '-1.926294970674698397026448350044490870358369');
    T('5220.8153048375064912183804758', '-0.0000848053870645192181', '-0.44275326271911063141541592413054755924692197198');
    T('218437.2', '-0.00000000000001820193740822812', '-3.975980242028607494064e-9');
    T('0.157577570127431677', '101.0319296822', '15.9203659846066164839488230494');
    T('-14.33490', '290.67243363958433117', '-4166.760268980077428888833');
    T('0.00000778430585574695128213792486514057213166437662675439', '-0.0000000000000658092384657839808153501892941795', '-5.12279240351449754074675940343526036050666862079292855468757563270534826950003073005e-19');
    T('45215297824632433459231894077144.81', '77667936.103345159', '3.51177886233727320835487431074370227708584865547479e+39');
    T('14342668379445830286426805277231689892104589', '-0.00000000153230966', '-2.197740930800139119461236060924109647979621945502974e+34');
    T('1087327540724260701412332282353.246979912226498912687', '-31181.1155292590461395304941342272532923707', '-3.39040856654682932656823122420611037707430356255272533055260947181604908300590629044625370709e+34');
    T('29098578027968222529887', '-0.0038633633262596785860098746949622820633214200572090', '-112418379199558110812.573170785518748243461139468530838004471322252305383');
    T('45167443.85086931970589523202133', '-0.13322343827726157227081356406399769062787658346986566850596', '-6017362.1680079665574862292748307786959075342072628967876653391403467312455019622919521268');
    T('-1277104346263109522212350488222850419104.598041084894694', '-0.000000000000872545304590848146374055', '1.11433140080444089775987766493651597528395567239169697675262925481314760876417e+27');
    T('347.048067026229351381168979388696389522228278049448558', '-580372453576496014579764354612', '-2.01417138168992971509920131695177453316685553118682774156732799236522441451364049496e+32');
    T('85.9498795419121819013954981', '0.086819059343222766234152013147244', '7.4620876924921221071832372576722493425329853011417540222364');
    T('0.000000000955538852', '2796816770377393398221585073517643123820923752.2574913', '2.6724670860207620944890322427693843122815393358116556452019876e+36');
    T('13.3499409389365943266698006822510800901', '-142605023.688711727210398330', '-1903768643.839955511708245141067882976861272713746574494353289533');
    T('0.000000000000000000780', '-722296330274145516124092200004225017552775396318.66', '-5.633911376138335025767919160032955136911648091285548e+29');
    T('0.00000000032533771679445565923566197211480872204653266', '-33902055423889399559039392315', '-11029617306247268880.3792855595203258365194772546962226666267013992005079');
    T('0.0000000000000000477109133480553184558657389005693925237508746393136844750692', '9544079599812049753877281093753.806913927594670', '455356754773575.186639624987152382218896113143126632346387917730524427522867133635682709085290515537801164');
    T('-0.000000000005212381313104274295521085956757682135609555632962511189216', '-0.20258211424784772185707', '1.05593522667463662312577979622174586506157166623770153687938400565136484167735712e-12');
    T('5.939336816620258503935', '-181.5', '-1077.9896322165769184642025');
    T('0.00000000000000028', '0.0000000807854522591970390113967313771890503193275103759', '2.2619926632575170923191084785612934089411702905252e-23');
    T('-1420735136825498697880142035376838692614.60263288147781831', '4794758649393115914238372958725703', '-6.81208208579077187760278480672518379981721734276502252436200522274670249079192835816102193e+72');
    T('-14302450490637670545279313.292191928068', '-0.00000000000000758231718292500972333975206250950391475327599176587054817182', '108445716113.09624528335189894213613395592525495466249866985446295108569636686802306848015834464376');
    T('-0.000000000008450771622', '-69748785787966613339823993495254469.001291622669751588', '5.89431059605905165095631246704409058304793926203868597659835736e+23');
    T('-1179004888.823228', '49931113655424934777966552716734550837488773547523258643790', '-5.886902710413423665263191856093556409577570847449923030469713801232995412e+67');
    T('-739484608100691619531927074065663059218837785449156', '27.88287', '-2.061895319467253133749818345565325454400115552276670835772e+52');
    T('366.123540796397283114420', '0.000183575228961067940921266500079', '0.06721121282973553030230705091816031104204619829603918');
    T('612.24659918919924258642657076118376830990940740', '412296990330.330273291603849583', '252427430185686.8746606867196534961374184185494504035865409741651638262671142');
    T('-1253.044774927718837481270804718997', '0', '0');
    T('89.955', '3.072147874', '276.35506200567');
    T('137440172231079014714878951462.97980416140395', '238.92817663651290090629461162158684942088308705117640', '3.283832974778000222556804923252233207004641698923980593985160071015425606241073037967932310678e+31');
    T('0.00000000000013007478162945817837283572658276879385007', '-9605773255988262156131.54662615327683681096194', '-1249468858.6547626741062732720588036874502171676524751175204550553355667396106328363358');
    T('-959.0561817087475466288631232777115', '6.5232691288079290675200146125939736359903603475043077091979', '-6256.18158293308052505705738568867991673577610410424533837553637598105441295850976942461260585');
    T('1232651308.483980691765841289013994011162', '0', '0');
    T('0.244059666606121846715993202008963667291853198185038924811401', '-91.8912', '-22.4269356360364638409486745244460821438491406050610488476294115712');
    T('0.000000000000009298467342175115268976111043399158', '298.30', '2.7737328081708368847355739242459688314e-12');
    T('222487581688568714558163241.2498688', '622765319764779384998.16290716', '1.38557549953973970068746587697772147617465686740636673376580608e+47');
    T('-0.0000000000000001094619824042797469992', '819334825', '-8.968601419736362575863380714e-8');
    T('-0.000000000003213357903869884', '502881.861925654510', '-0.00000161593940573160560373542857777684');
    T('-0.0000000000000225850887358266958', '-9657.24', '2.18109622343174999747592e-10');
    T('0.0040522596025230943461061874749446981', '-0.0000000000000035111456723673138306010083', '-1.422807376670785399057088730766337512497448065110252859909423e-17');
    T('50.4', '-0.000000000000000012807615227380978390330730165878274412863', '-6.455038074600013108726688003602650304082952e-16');
    T('34509655900.4271549037478532640245880287700031396795737736', '64.71924', '2233438702537.161140732834214879190678535092737997675858151324064');
    T('-0.00000110516283770184738962025591', '-12341997806576.504800', '13639917.318826066245556761603362081169343368');
    T('1443495892579134888970475033389', '-263878898.64', '-3.8090810632514586354552763528830654669096e+38');
    T('-0.00000000000000000069532890943744308434473313878', '-57497447967704543918096254112146', '39979637790820.12873785593753087723013293250902050457270162188');
    T('-0.0000000000000000007659553263900774084490506', '-46179282173901700884047955896435818818778979617487', '3.53712671499703607070284553536887181481970539221145787819001524983163078422e+31');
    T('-1', '-80.5351111304083830696255443', '80.5351111304083830696255443');
    T('0.00000444632', '7', '0.00003112424');
    T('1748652238406104265.08687786280471813580897620802429129', '0.000000000000000000175013', '0.30603687420016752574564975540304213510233635309495529153677');
    T('605.35473313', '-8105969267.633939755450293', '-4906986862768.52514719110958199530709');
    T('3531.27893', '303826595361824791936490465665622559869413', '1.07289645457484761411696267955090137099952167836809e+45');
    T('0.0000000000000000000139500802384964877256638688566332365116099', '-669823.809722799400662636451451747822858695293904', '-9.3440958912884554771068695881482229864014701239462146074951636707559395049094067886960496e-15');
    T('-14328.20453041484587995049563830419841249274151406721890138', '0.0000000001737045483', '-0.000002488874295905724415190916871212750963235628741729704355101435146654');
    T('435835890288235470264209317728368193', '2502287905711035099019387753673794537858483', '1.090586877143053196441416655157728683500541851046587666747641275079364352431219e+78');
    T('1481352883141719129158247138.8038050', '-606388568.443511765388700681359582935123762319163277946633', '-8.98275454167975836154301999007742562427000217610359536587169076287724303365227616247338565e+35');
    T('-2254195545465844946364682.85626170910', '-68792607248889229037631606.018565330920642673825', '1.550719888214274947292592996004771619292172071432516539437011341424210097883343075e+50');
    T('-26784210511779136785823800676221849936631.56183', '-0.00000000000005019147414519109521634075017431467167600186829521963593473', '1.3443390094013180964383583232553726446281441447994273009124279348757959259512742782394910442001518393559e+27');
    T('-8770530121852166665068897500281', '11987190432239629.9682', '-1.051340147623357682201798135592086804701933955210642e+47');
    T('-392092775092295510505846799423331218731967579086912', '-0.000000001347740911042761337967635461776381177269782335301', '5.28439473916174872080749764195178381030762509320641795527675022371094325058097973324110528604680512e+41');
    T('4015679.55', '4.713361052', '18927347.5882828866');
    T('-908765.598832994503098008148905108990', '-393227867096205311879272472100531195041.383622646628607', '3.5735195811950419553683668422021541455075148867700124530307768773407398811212098138687693e+44');
    T('-2369253560877872972694', '-11.60552475086080768', '2.749643084183325817302731177345942548992e+22');
    T('174098.34305585608863845094930842', '1.38', '240255.7134170814023210623100456196');
    T('116951064421932980', '4525883193151806788', '5.2930685688844070019747986244506824e+35');
    T('600225026818135855', '-127962986020756918725', '-7.6806586716037565057838021298243384875e+37');
    T('-355276.30', '457734732040.627839182124360081830', '-162622301980885708.381620168789740259629');
    T('-0.000000000023259793972698088534519974595546178', '-2551236.5', '0.000059341235365627366949498869167230146749097');
    T('-138631', '2.3', '-318851.3');
    T('795255541856527997999067123124137485004290563483625370482792', '-64991504618917159513144692611330263343972554177.85101068', '-5.168485422178800786099741762775612989901343114104453511664262171013611241941946296162984705040765106162199674821856e+106');
    T('96045012606945538626605174.6240481976236369294325652245732', '-0.000000000000000000038130457744062248428043049303932284765937539238468630', '-3662240.294737062792567245617774026857299030801472930205579945585215779769210281070679229934569639844413338716');
    T('421989371626234493675627719459467707.14836401276576921755856', '3', '1.26596811487870348102688315837840312144509203829730765267568e+36');
    T('0.00003674095297032643311401261648', '-25144265134553986531', '-923824262782066.66182891576601261318521398863088');
    T('-383466.04881547', '53724954971092', '-20601696205553692514.69239324');
    T('-0.000000000000000014783370871610989705993827634590792', '-5.75544555955339866871', '8.508488623824452763835260497721954711915248063842451832e-17');
    T('0.000000221838708829447458147931597976839', '29539052824125603227.9202610', '6552905338548.867330870922184373002548907167329945216834979');
    T('1417021083070041108879006664525950600252.582747034547', '20386.077908379675293435494662791207239424304698951', '2.8887502197282405761889040724952549395790613139212743588512735905545957366555623236526443052831660197e+43');
    T('442.735', '0.00000000000000878277865798641273232492264611347738525943901036', '3.8884435091436144410458746277270504101628377302517346e-12');
    T('0.00000000000892', '16797743856811194', '149835.87520275585048');
    T('-28.32380770960', '3481300309.0', '-98603680531.4870622664');
    T('-0.0000000000002514438408283789730670345516227699', '0.001175682622527705754776427124', '-2.956181542035476048733420921784296951833346097731013663707676e-16');
    T('-0.0000000000000125727922878', '-13.313', '1.673815837274814e-13');
    T('-0.00000000000000311161', '995915309552512177', '-3098.90003635669241507497');
    T('0.0000132726799331352407876197984124', '-0.00000000000000000145179351917753148242751859389117949850796347504360670810576', '-1.9269190709043414470286356436843451767202221286209940752290325354825996720087060259295424e-23');
    T('-1305365.7803110685680577657011785720', '-1.13', '1475063.33175150748190527524233178636');
    T('8499682132744325149884.2', '12753066384420249097484638996.8248053537637250207', '1.0839701048535906248094606671098895649828222408103379536302544124760294e+50');
    T('-27597428686882882379490857905609.4308', '0.001777076303502550583927', '-4.90427365570610811144061901744727200974205360973380987516e+28');
    T('3851.5801', '2.4681503', '9506.27857928903');
    T('-254136480693014397618955753399722664599889.09', '-23582875375875972784153503.3854654728889123761140614', '5.993268952647068812676163151455910299034928531722912670375198720068520525866107064371629450126e+66');
    T('-36756.5', '278541826958821478.87226349825573265453570107573513272911903', '-1.0238222662611921688168353273636837316441496590258406157863626195e+22');
    T('434677757893257627333239169971.36582235872933', '-0.000000000000068725189442692668129', '-29873311257739028.68226613696221684789246137143439950127182852357');
    T('13480614688014170819.34130485033', '-74790845792983147611.766920195511401471625734274890', '-1.0082265743258914745957730201044196850059871310317319787175533254133776825272137e+39');
    T('43288920440971609207698898895824622861', '-8750.436', '-3.78796927827813844188979922058384029569317396e+41');
    T('-0.00000000000016299662731892671518950597', '0.00000000189502990635948720606833476337904040553896306692718740072', '-3.088834834050979272599008422371925718929124189963816260765249621332298025452222984e-22');
    T('0.00000000000000014', '2825.871550945769438271943129439745088808390801', '3.9562201713240772135807203812156431243317471214e-13');
    T('-0.00000000000508', '0.000000000000002145872435', '-1.09010319698e-26');
    T('6668343.896256', '-30256115.746220409', '-201758184760723.915056805888704');
    T('1910671461073245653125406909', '-41009469551.2913732728', '-7.83556231054046679894479520055900512189070617752e+37');
    T('8289.8635', '-66806493292579558025822119188753113864024114483099524877158', '-5.53816710309150098924394843355494049132717469773268118146494087933e+62');
    T('-0.0000000000000005326', '2424634.43200073035390412054', '-1.291360298483588986489334599604e-9');
    T('424284.1840077187212673385', '970.3396784215570868735428784067', '411699778.66940253812593312501264900866190088462962956795');
    T('-35760922777.0406357', '-7761344319.06347958972953830', '277552834840052130228.57982929972823169949731');
    T('-0.0000561650355483525921131526685343539', '-111333300972076302345', '6253038806812103.7204293271893852389242141697156298955');
    T('13512458606466223813008490671514544370375', '764.975563481753223231576796031842139048653', '1.0336700636505365489288540708376614928455337024474216176633151377504345359376854875e+43');
    T('0', '-1183974879.82128974148434577518', '0');
    T('11143820497441188022611097072500689220.103841042188', '14333893036294350850719062076858.50886743820472767247', '1.5973433102598649386055415009433149402160776376532075449909743742363111723311690413638261800831616436e+68');
    T('-23955066895875385312189828216196343.2', '-0.000006008550114741720774296140761236324588', '1.439352199458576433136766578653842412823876234519218069316485509338466016e+29');
    T('-2668354.95894041993791850076178801129928639334916993835', '2120902450621765046861513300898', '-5.6593205715454758964489377686065817529188126202588690397558408061055411762596596383e+36');
    T('-357473707639070742000106223385951624969553570180519146672', '-2880014494.9', '1.0295294595461685945268041406912380369425230533019395520223390959728e+66');
    T('0.1576', '-17946110296096494652', '-2828306982664807557.1552');
    T('-0.0000018223152394757689929568141403865013237549366458944677798', '-5052597332068.33289300934043046685446478', '9207425.117162735564750314224408158434255360168962984445546645230422805472786347453701483895444');
    T('-0.0000000000018113502438791982724283161334170828780578698749690297210', '467569918734591161565.975984', '-846932886.330478617799083574720382758183324318393500300146312171222449646868220464');
    T('-43456291089.3917040495022986696', '0.00000000000000000160606215298685200471957781912132661', '-6.9793504427851792555934505781676792541584728405006755687190078056e-8');
    T('-850368', '-16365313992958404661804115212963911454627853490846168779', '1.3916539329564052655449041845417695455848978517303874852260672e+61');
    T('-573001586626626136559291430.708491137026124496587', '0.00000000000000659091710510594985802988231762972361602034', '-3776605958550.27888864268809093797447339962539247097589384819969254713146626305667085257958');
    T('944831849965628473424387135285.11628879031122854553239231', '-9167656497046734408050.66645416827001933707275561667810469', '-8.6618938479540792582928343057162520481532362870957700270661322678213334143541246056098522215623568551067323309339e+51');
    T('-0.00000026160509100', '24622004042950607868289417294478649240.52229821030', '-6.4412416082584616798891690256590608321239167108346686373e+30');
    T('-0.000004846613578004694945875951019296774584875442', '1', '-0.000004846613578004694945875951019296774584875442');
    T('0.01331120519980531178226823546536111232129742873981595748', '-566030091005684186432778.7962173603', '-7.534542690641137182401329425400191730488017551596028318788787538488664119896612542640044e+21');
    T('29160798123715526401', '81.21430510259265355460162080077758', '2.36827395585454415364839027902703656652482945981888958e+21');
    T('-0.000000000000000016948492474809672735628387832748072035504038167841614', '1562956864069228066615011300882107599329713429.314843972', '-2.6489762649129436461814187882951473845942013738702479876336587925692515948461245980490938479637262418650808e+28');
    T('12.90597368208644830150566402904732366667482473', '604431199603296.47474454963005587100955', '7800773154712085.1940792711683581731610862958146209226570359806010190800492344061715');
    T('-0.0000000207428179', '-197640319459879.188', '4099617.1562541003526838652');
    T('-1', '60210209731827764276966194255', '-6.0210209731827764276966194255e+28');
    T('4583913614531584761660.71', '-0.00004564661231066039596029736975879852374375394905', '-209240127628081229.8988189542842449222791835905813694075712860987268255');
    T('-79843798348570415955954652124494715927', '56861144183144013650375039269786414203977063811583167481', '-4.540009730027938310459807409573958130149645615816392783432439536138537446203669899681959169887e+93');
    T('-0.00000000000000000011314', '-3.526', '3.9893164e-19');
    T('132407155319180608772091166836747467896048163329803.57610', '-723665204004.7159777491682569458925', '-9.581845106573894955821546492296266642173629363834000697071816757006018496183651070616925e+61');
    T('-322521422170779513210597092315316261621', '41262058870589483714064.04798992976852435145735153', '-1.330789790863694859369204982948722773695665386328207265159910537452461498636765724463013e+61');
    T('45.38814', '269533555865235093537499029287212271314833820844669369384370', '1.22336267683091115583931011911520907801556615372327715913294993718e+61');
    T('1001606998247.1329297768570186489508291933755', '5.20052838227', '5208885622264.472940958083743279523476777800178015652385');
    T('-208725364670066089848651768887053413406981733447545233.37431', '46428363895460548499939303066133160314818660457470.1898', '-9.690777185114533186791052743948505833183422796131294760780293146823565795246823794373752025269078880430890144038e+102');
    T('-0.0000000000030479119204492183257505452809182072468706814247166', '0.1831953518167469517279635239571573090896007407998', '-5.5836329657315139902372580139052711078742199613978215448274609146206088053118483963543265377233668e-13');
    T('28.71', '0.0000000000061362523', '1.76171803533e-10');
    T('-76698670010634912042.111088324461002234771829964608794', '-204.3415360766065', '1.5672724045005890926079579860206010656480530112197034797251387577561e+22');
    T('-1524.1774171342348399240363', '-287256761224.40001067164619002', '437830268377351.631000336578138459542103286567115177726');
    T('-46386057531584301786849973746261104490857090', '-4.80529036', '2.228984750949274409036809536091615714528682827146524e+44');
    T('-1333480998557541026668203845446224930066438742734237449871', '0.000000363031301806973352170691602189509034079482081910252260722435', '-4.84095342841206873734133792646560873902386616947445713637359576061752353209536356871478780155826152779923290557555885e+50');
    T('0.0000000000000011328263767573913302830489935960640008976331311323634607', '86336988667731117573.758284472483734537177637', '97804.8180526097967631959070730320562045880918766169966194605514255032840380206192312923918439683659');
    T('0.00000000023383059155254587815392387016693739292933749', '-739769226366265.5531389494840049176644417', '-172980.675813593093567219554957028220425661619804809222561967458157553863975311729333');
    T('-6750356340240.984376193207629621761115', '6785896266076286.1666119258989601680226719364211', '-4.58072178839256802270353210343421354282744424281215396734873712247233250402922455265e+28');
    T('0.038352390', '4364267397803.554078375378060505', '167380085304.84704939994306577393135695');
    T('-3677.765452', '-153097604617851523106898348175.2705689839321002774', '5.630570810474899941481304477948773393614882215140213363848e+32');
    T('3564276717.02076515', '-3724708.5652442', '-13275892016787721.61135498059963');
    T('163961034566390719', '13906850976432268007907879396507766911641292643', '2.280181673656455617109020414398350949017013169011794072158180317e+63');
    T('-0.000000000000000000097053790888310241217758521607840846169737665', '-174167724848769605456951958316132823954.290613019997555', '16903637946965240719.160592454311893126986143375067836000624105979381554916207439134564513291409075');
    T('18174121469196263476021234179885', '598847655451717090028982590433609440', '1.08835300317228983812760587141239606682288560521096139829987941144e+67');
    T('2862482160141173353.377689153266444043', '-2.5521116576031457307341661427802694156525', '-7305374090577323175.0048432033414644577500405016939717138581879110681995830575');
    T('-0.00000000000000828251723257466281232072431185606927126573858797644180340', '59246603229200010807575028.948615845949794986267082749', '-490711012217.3627547204685981714157886628710287556950766147835640216213833020293343764393555822441764905895466');
    T('-0.0219725', '-42388.442458578279694893', '931.3800519211112505960364425');
    T('295211627171624947864455852710547075994077285904641205668', '-32114221914552039416309', '-9.480491706745564194049269607588763971795849392568309012222499652763032742439412e+78');
    T('18076895076744235516275796550556474017044744513472', '1110616783295453.907507694897679', '2.0076503062103110248911265925095057715921072859968017387492670202835863677031488e+64');
    T('-30509248597412319207476984175234018573.80700', '56489216853846845634764972404741.4102289', '-1.7234435600671472183595024375663260504322309432173684665884657236439010994144223e+69');
    T('0', '-37.8349097', '0');
    T('-2851475309', '21590640', '-61565176865507760');
    T('-82809184867575581.300036713702885747495702565', '1983849666549698977150.0321478', '-1.64280973786792195412167897641267648031706071732558941631564732096919107e+38');
    T('-0.000000000000015392923450383', '-100527447118222995497036068167.83978088878', '1547411298153231.68196185030647190431319404613997140274');
    T('-90815267.41287893625890527712', '-1377.468328268', '125095154584.42972566162438210224966962816');
    T('-61531326072626014632609689592827828109737644', '6514.9664391', '-4.008745243164772945370376441467418441994843425659034804e+47');
    T('-0.000000000000000127087490497875', '0.000000000001476615944219678865852759023586345893215179308071704', '-1.87659414780029158895646469218881896708577129747276999389159629e-28');
    T('-500.0464508771550', '3031251990944702206548861174233303005657140.843654', '-1.51576679978620832480902504570356096303779609394637710501532437e+45');
    T('-155.3658', '-1.487280412', '231.0725110347096');
    T('-7246163852901276.04365667671658743999435118996620258393', '5112736623675367552557768126133416.065405138439108', '-3.704772731188096277860254002053708999588540456247342592474290019026821786006110036086254157751656433444e+49');
    T('0.000000000000006689933708640329486606383032', '729332114805.1424421852307098335222592037429719271268', '0.0048791834996288611343551405747438154907187127049172382481758395023666868320324576');
    T('-1.8', '216561.426678096257755924379012116313', '-389810.5680205732639606638822218093634');
    T('-8058045.0444', '1329279770290237732458883752.79771', '-1.0711396265608420509738201250987261701168324e+34');
    T('-0.122934172485398496827140448263042340165490502305', '912702090990315108628442662833971663.8', '-1.12202276281587270916014575371637467822687977597732727219667936221931017460398585059e+35');
    T('0.00000000000000175417191262247437274343236218710277615977753', '159777383.0958810142116283', '2.80276997699115404345306815091460487951017642480659328034789667052099e-7');
    T('1.832596787409', '0.04233218045723559484149346857663058272000354', '0.07757781790994800381556406308518949782645811636510742786');
    T('7764032078986972842.005528691471157135', '3814413634570635467562869035571042157068430937090750616', '2.961522982133170619249946192718193418202399599536127455799771152758136779037896695123404516e+73');
    T('-17135339502484788763', '-115136630978196546781', '1.972905260983705133684367626946832621903e+39');
    T('2732768992944344069188.78233921094751884724609431', '547802187776316166043551795761072525805362767.4739838', '1.497016833022191997667345517507111060165004561416898656598340429974721721725596504177233879086212178e+66');
    T('0.00000000000137207147854597410975017122036947235', '-31922289389067969062352250596.5480447', '-43799662800630948.884223648169873969145520154519727429336909041963214045');
    T('794.15', '17485441159196003764239999', '1.388606309657550638937119520585e+28');
    T('1.7375578177138444239721', '5', '8.6877890885692221198605');
    T('67346906173389358', '-466873.6162827458603696333231123965906193638807697765', '-3.1442493630625071474606471872766565213027787746877671241569093138487e+22');
    T('0.00000000000000008330954900095391223717198750270140816415917', '-1255116.84123905597608550718171171454784329986782439', '-1.045632179871276258673769748937031166781342825771332297107555550761723897353461461042815681563e-10');
    T('-149004811516278801454193569023908186952042538093822927716', '-397664736597845838069915', '5.925395912343267570007772381296883214350181601356111567003076787081507599926414e+79');
    T('1000073564757252670366408149537245435047377806404.1015', '143493737303.17', '1.43504293385121986151561356719338126856345712035100585929836951755e+59');
    T('-120791171186.75073588795', '-3356641846898638185.78959337380788876290302345', '4.054526999413445597874488228850145076904199735090551400294754224275e+29');
    T('-0.000000000000000021645765186475073728477012180542023658755317897440133896819', '-2892.41876465002535699255875339650679491474379', '6.260861740056875851324591106893702672467555630128663303516124593419559504118801178122464786716968100401e-14');
    T('303590.733684425609632655982112479958446', '-1079042632393906270486', '-3.27587344445239960885966209642701728797406978059884116224756e+26');
    T('5560006241690.3561204958103356609', '-26134.5', '-145307983123456612.03109775521732979105');
    T('-21241252966066639793327', '-9918174602168819276153472753.0892247103917800902408030', '2.10674455686285247587982556750283161206049081499338374032384978521582521581e+50');
    T('0.00000000000000000011710898531900697150827118078795784040', '1416082645912.362', '1.658360017906513490504921316067138338281826491830248e-7');
    T('171974954737627841.3571', '12776560683821406610094.4', '2.19724844530274182316145798323728537865511024e+39');
    T('-0.00000000000000068754763933835169744516274824757850275393656608356918004', '-6.09469365736868', '4.1903922366142608309254800108912871683678914634852117466941692915771472e-15');
    T('0.000000000000000001400696', '8637225390591.816150963722258184', '0.000012098127055700394515390281912149296064');
    T('-360824784680774106563254315862091.5', '19099425092005944.36343728914640', '-6.8915459463496190545354254880571561783649870472046644721536956e+48');
    T('-61.150780875663855271841725817415551', '-0.0000023333939193095575001578508180794664583584954721238120', '0.000142688860256305217839982589618557627868898235253332040049384167607214599532790926200412');
    T('-29783331896.867208404770028328881856', '0.00000000160', '-47.6533310349875334476320453262109696');

    T('-838472500998941781878534497598346150189438395681961909493802795126225423951505690327678802846577883568622319410118', '-2884829948773734005744162960155700874127.0681001892763437635520740604769472972181496111013', '2.4188505821049618555759179210300843443018778137981932674088381260022683344838428233063800820435344194302980110031387348601685636929132993133036482870813353080292469079495502993056630049417824321203429534e+153');
    T('796691.919974655750', '-344177691021', '-274203585471964324.87434102075');
    T('-0.000000000000000000037241136189493487515236242635317406', '-236633306915237035867656873537286323242534.531', '8.812493209800553514615822924219854143174591731693607256359134469397049100346586e+21');
    T('-36918999197803067902650019640975752121441727061986591604616945161242737830954924822209930.0610216157567', '-630816248640.220238311747288806', '2.32891045775094316018876017621573827476363455570577127733350185252461212627985116895575049148402627634872854943933909453127151295002e+100');
    T('-356285686518395271578252895087932787716516776378851836556183.490949109645090654856023328687155015449097603555460351788748359722414863145', '536192552176752561042209607723742909268910764247931780563.2', '-1.91037731558344763167084704445859558938320679866577929319697485737868113003820206731226346686676150919266490086231386325706954806002421869189844369048659180671649946432503759091893660299023264e+116');
    T('-10868240043.323', '-0.0000006987483137538548533133640114148563911523216092560534998', '7594.1644037440686677326890935142678911031312404730399664663577718354');
    T('-0.00000000000011591017649385438295796980510866337031921451760232710693439920068628049494341909991139678975519329366780780539554769715433124600', '26133984739171981586185319', '-3029194783605.121387520277919164440375428744596566999709645269433197968011145816158998774967743687046263521357854521385482216635698201872145379668177474');
    T('-1711273540905209744011506141193.37033113722759550410761924564724', '0.0000000001293732763469758906747311439702612890496046217356893762894231050676460791112722303394750148856', '-221393064712797651093.576041646381279748254325855686348423517448454794870494445248164160406818767744894597164595179089851596941716698075867505673842006555744');
    T('-63062954745.8598353024049107700099585768452939988302739544094597635', '-0.0000000000000000007066107653369429290153271704760473112899492658449746935736601687393998875372553088763481649809909245944916440347928986666226', '4.45609627173810154902373805273164378070520471666107189671611539018934456392187448488034184668558120645401564953529878995106460890572197110823797727327567501341707822274253599872751551397551e-8');
    T('-0.05396306543212365254900360568584', '0.000000135925517648212', '-7.33495760274574245053942000380666682810971808e-9');
    T('-51311181621759783527978578699650628367670018837603370537970304306.5', '-0.0000000000000171073617238152543', '8.7779894447982604608252727389859997980275463423154981803549625152925376395768264295e+50');
    T('780360810680597937530828803485426316757997822572046380524663475663587783713055764905934635773988266249773631591980851269802869685048538490187809566', '0.0000000000021114006241457087', '1.6476543027298657092509209956166007749872751971298560901265194443520620935869459922698982198083609063920515096925628739206594316734184930365483909751507976717094242e+135');
    T('-0.0000000000000001558591643933212408712480642149933587830724439341932663613727360989832586450936995132509861377299507330923270242750447693180038078839868789871', '-899234589140276920.9563731186', '140.15395165697510406375504229464266777072446771644043491265354600675571562268623460258428521212930915552512830838365872392420849552201444571287189837930690619037363617006');
    T('-192128620208427347208760897481755790502927310252099946847941391701541372730741701901004842683652084499599551669587901.095113423628696460069833', '-150027.013347965686304316841127', '2.8824483068535959518771144524961420793873335181555629802553828843970441540984506755823649963472492988719283593979140183046911245632653583841810467259116227616668386421791e+121');
    T('-0.00000000000088119558575179620805745334627526', '-2342970366196339369130467230746566743437618438527.293007742982263818470063380858096529305084867946540343494845177603781376', '2.06461514423948373213684382167619940227586580537675167441830145669955691286212463457073538034124876268856699464885544965251476720909351479389090095755776e+36');
    T('-6635456782135376051461227670659894416124826680462504047183309508524677837974389566804395212269550349799505694436932101464692080338', '-36033.774951979165187136955049120300', '2.391005563910499861359276356470099138701070047532575485529233046218042255556441673237652214828582412128576389116490428262479007258867518065861096006448051838266614e+134');
    T('1.0795693', '-0.000000000000000006209261744497244037804821960982585015091967690457481640260325487223225820327780029749670765451353356', '-6.7033283550236685978221251810425966169333249952098001341386914040137368425931872572708312434887817265895708e-18');
    T('0.0000000003205206100151839365357713821428922286825107015046701389076172910755564034571464044704259976070021361285911146439621197151315883353504731', '181280954003418224438635307210393590651533286700798186230089972712042801144671899167602413575288941132381001', '5.81042819613101098829202896183160808144566548341644020145672547798396356491584355574818346795299101366223635188614907521125965341055497357077467758341149862217405975928504053356664898994511933823140023871980627595465353142713441016064148015731e+97');
    T('60.273280', '-1948051519028114.87884169', '-117415454660806895.9645912570432');
    T('-83193336723730904831044', '-8597513002342', '7.15255794190492657393641372686305048e+35');
    T('-20357731434704594485933.435192883517106779904094737872189392050585959795325610691798979690389882855327158774541534046886050', '57101254487124.26556892385946827693467498483828207557125700861375680904187175758285473565759248405589101', '-1.1624520034335964384881547289755586531335208559562766632031511288008463494925147406785220580646246607391536758053170339436847553410796246157934498139151632371860559137134623242512914876401966834601783673486087092780486894105e+36');
    T('0.000000000000196393242447331955075189676455271786947264076645295021258862881929479958110333580', '194877.520544633956108976283205922142563827316236038257841914758325718909245819798140', '3.82726281398572106089641076236903647054748740721743415697957940132522454065934785001420361718935697399839562856732355534966588993484025198806123215220764836635412e-8');
    T('-8071576425880681257819461150', '-1591541556785189268475083517665125202173160434133152.98399538151538621882421945398094', '1.2846249310556773308688003659431424152963795381736069812397375172279784904016650744805487687134414853173170481e+79');
    T('6215707454580399344821610342', '145319540897984402903779835609411326583090144758472486922947576644983828825850438698150473186142966520100265693', '9.03263753655802873031730925036374079283112281131923966584171630907427735951790460941752213245331152538824074995769485027917034562316597006e+137');
    T('-129348079468144673220614626836759706', '-88248178940421789818564716716211678654401184759743634646878788943183256531349288163631708018145656009375447214258101552585320093130', '1.141473246250472885515118898707262713605663944616163164881785239559268827685334536245384449464192711247404249908389352393586709492073897298343118518559610042335141978e+166');
    T('-5780259200186462371596484655187034250137478170873403737601291296553629304246338.1', '20317804349924669254706871559', '-1.174421755212405947511968636545739798982946944457429152629522410545955272538056812944623926565594717327880979e+107');
    T('-774.0', '31209403299230592237921875076234864775333333322171087161573695825372946329.47956111641', '-2.415607815360447839215153130900578533610799999136042146305804056883866045901718030410134e+76');
    T('-35300012822981309972161307966314653.5746532880996436', '0.000000000000000000330318102723213448', '-11660233261792292.2796305229097940998500752778977839907195278155271328');
    T('-372377963884127526301703864441812802765174863559283045963633022003526893443086066813180.3717468998054326701855782319548515', '0.93426382293185456943254764459757361885785459231935724261799663689196168735639052920121741580138452492', '-3.4789926011396505505944781064660804224349193623867972766690000964018527357693140898258100416659583488178405935796093105315794205956790824509036660608118406568210299647359494344226952122917410385314738577518950473221664938e+86');
    T('24266165641723085835767787003697599948414575.4598063181549354935463146230534386950113305', '-114046436224578769506685348541641847469702436631296179876858707928490406027412029206934467.22255852787626', '-2.76746971227383645895497368320257172653365323571255355106207704032170926692669369847993108061766552826817226048614230592101594822309797349385303976310250855456022415694582167365528080196393e+132');
    T('3296856412996342216590213367517526056709903701721644610.791416', '-17873043539632614693589701036988.4471469786937536273599351623416309394', '-5.89248582134006296943775735259576640781981176325731275908604121314963017378308625998703430431944837580972601701118670108255361904e+85');
    T('-579102350347312720630005129054780607559620407558223441199264163920137899902249163147321939579047776302298780448678174736222915', '-219492399337234160290652558813948662413370806048542871624923.1888570368753037439767245955297245131205931820059590', '1.27108564339563247096015910701131062607103942871909341223369812880561545119021551254208157534581993545107836369462180155951136922383213991699434467561508645543330227906067804094605876984754845169252387320636979899325902200224418682350485e+185');
    T('-0.00000000000000006163518675166495062601637166684339458092498600493044751669369089730774084108109728725760785705955545530', '-0.0000000006634126736829635005496725883566808801526308721290044433770358933738076656970776547028344255382880585757916925487', '4.088956403587081499616844261176845350066362599955649570127728925758519680545486693543687506509190342330458809847232254048349532057461695086500962967500458536634636810198879285867211023138454828380781894359544592311e-26');
    T('749330908047468986174543866383750336261268464649133307531895834594094676053587911029106522535833685617906328029183872567352948262609142814490', '1.72', '1.2888491618416466562202154501800505783693817591965092889548608355018428428121712069700632187616339392627988842101962608158470710116877256409228e+141');
    T('-135986987299837104129797979843691621285208421388144053924300522288644.5900287109133895483', '-17962494534979782634275.37015590780316964667993394', '2.442665516201689090665690743015967257236753951326674341795336278365795371226492213646697861453424431171591440002808134392672856130439302e+90');
    T('44006442863122570049153802624466247185058307274072774466983112669894864433785226136933.2', '46790892915789.3411524668861472382045424537291525565291001608139420856967742081828733644952251814', '2.05910075561317027401508721128826736996297356066394448240229549087424917958568533386066515904891204498637800861760830895225068854234088512703986966936540011209421699582958756520968248e+99');
    T('-0.000000000001357086727370149233250058639705046544732320894727648668175942', '-879468576980759082.033452675408474697409162575095118715838878', '1193515.132959700504049250941775088529954670578739886689678357661665327443107825618722158521731867085586347456416627873076');
    T('9809841.727201839210709', '-0.0000000000368880827799595678609037922161591383619613563735115949331671506211385891506096678952335776345631106186087', '-0.0003618662536913229896876574465145932640958294417169072448132935373449094813864973395333355003851497225742989014305327367357205683');
    T('-126.76', '-302985778493402807197323438', '3.840647728182373984033271900088e+28');
    T('124867100493.457541072094721668121610280880363817443143804305', '-2790.963779772728898849783002651075720477860575346862492661675565992697657575735773', '-348499554762481.440673495837344907334116857684623288727299421812123101984255650489865544241755462317555675735472100548275529499295106699902765');
    T('15521419716240482198113978224037109244552806478255554247304501803343108406125114544385881082700492397181091385635182384771169625695908585249481', '-2836586261373793936479454372136.74149007596046739828188227468752645', '-4.402784592410408295066853625044872038760273286404138338015606642746110634158840973424236636715346429517243853338608937452735485971423525672323975187535426255868008698725535902235665305652922137074089203627245e+172');
    T('39568702947499980.31043424352839120626194373', '-0.0000000000000000110532060314654733243303464490568893422715164279635383512851140023129429730151730074105250113850458826687408847800122155909183172465757401122037579', '-0.437361026076572434440085826634462269408809302675686544261100147571961623746441124718245098250484230095790472367852085991196762852617202769222972427292049884929603478563067747014298264342967');
    T('-99609250253802741981.4964874360244285', '883485457775161795418874408210783527590114513274007831727657306342804460473168388.873823683132057229380427284612779592765741821135482015', '-8.80033240591215667544412145489909369089691076369957723858897861506204032139051216834675560840680969094658338799237500038744626460350984691160948888063757133290716624034275e+100');
    T('23.384657', '6358331922737965853097929789343578523627007160605428122392114297855155.13891249819092314840480065973772043562526917541987676891872705', '1.4868741110537783235240747553388183892758395838730184898029361236013863860525612320785833880636058134030234898750019986664922343269294087185e+71');
    T('-14723458729544396106728.8730102546343094999696645194153', '-133727558.268817344562', '1.9689321851736756223818822532522079871040093348444209468925006088388745986e+30');
    T('14302541264229922914', '-0.000000000235627544641501596076737344190020766746331735586147665546911192908467378811964334722559869070841801090200096029602404948591646484821330808', '-3370072680.224254836492035508073157091195119213264445104738613033613662085628770149276109216999634545944893478819818145213075837742045845104662438460733334512');
    T('11859540992913956563027495896462810455142079490183890981402926486193538086518056969511874125054152356620542433.487767032922718042853047303427', '955192395684300504821406379834312349326154.0693', '1.13281433727376500867653857943948863816132096768226862985682668025650778093862402486711549896946612114014654961667306187363301784449098292029152064116984771872264804563492799411078854911e+151');
    T('-89992.89188302466393156671880484242772885323991845613962309150678501746987154469202208826173765453391736170812306681487216203314', '46028644449145217432303817391775774045082825030.2022387362122235067512142165', '-4.14225082343410889148864685916871924839200546263861436771489245494114989348152499951206927264038619116817258198774738485287373362091029303295538404627178066368057815540301560861820242440073822631213481e+51');
    T('322550604700426997899532770743249875139977021368583.850723996627574052487902691354294316078353979975', '5051247930821098558310030929577132446128131203947693097941889018946508950010.817010609850054994957659209938580530674194319', '1.629283074578125979757678549954520448550932971582730260571009370631223791910921098102919850864213539220310377679206053786662498134329234965312607315294965577812311728138336208121595820208987331544983981561498920184762025e+126');
    T('-18162869148539225267245', '-0.0000000004082605569565299430913769521458041940383792216884928840151413981544379047647242539071550738140637320695619458956310444200164926', '7415183074511.198888443043591378591772024538336265966741297786740921169768779262078997101641290939777807031074347273941892988987901012164049542564887');
    T('56.437', '-0.0000000000000000363636129586261117394059679281224058126111740191432401889546', '-2.0522532245459818682368546119594442168463368281183870465440307602e-15');
    T('0.0000000000000098328737846670488651996142598', '-0.0000021680510741625274383109785118138365114445962436956704809668310586255130207174432385040177431879924758023695894509584', '-2.131817257095195181206361651183585087468603124720413551197211179755387449478899738202568963063585962944228052493622001429362279056649894941659232e-20');
    T('15353773892289910580447352912374084465426576602840471898589013750906506890933921327042063374884073.37459230997064', '-0.0000000000010024383418176318716576007775600816833200531168899324061052647625728403071186802669732334255658386493460274446', '-1.5391211641229945537862597203687187815529728355277111549164845611017692299846743962405790219289790424490140365517893874940113367036665712735367218576203409806863531895103340714121294126379877846187972625242851311660226544e+85');
    T('-131317855179200803975486750.21706572131621841705391042457406222193797630804760305734762283213938032', '0.0292170474534535827560241963251081294772380763588549279699135998743535144719442580701100219852823099329042846816782892289228', '-3.836720006256455223034915613925802473646371948151381349060669000949432685656690149745351896865724762974757845038322594373778118540209910046753684013004635293983638022635663512882017379734293615901239976258135149413119296e+24');
    T('-1.2652', '-11354', '14365.0808');
    T('9907551115374943050635613568695786302517118070997561724624950969636860315016878905709.592450615294116872101107020677899332545772518264112371602200', '-1.4', '-1.387057156152492027088985899617410082352396529939658641447493135749160444102363046799342943086141176362094154982894905906556408152556975732024308e+85');
    T('-490.0344807942463855', '-9323932874.185090992679891416847', '4569048604961.6964723167091758356936009232035565185');
    T('3604969916487266.77582902266201875313967241643046258256599229303814897912723083869145048', '-32491432188732331176981674049239559807795.05', '-1.17130635583966083474238734587571010296057134092517034186160396265057194098968738205898514560270240397013328401070407840495064124e+56');
    T('34703234002309873855644196344429559904012545590136.8731266774035', '-30509195306542390332337786607824589011847240852960.400475884124826629604413425289601053748170330700373748814', '-1.058767743945114695217780865659236851886438030887854601194770945013467876376008466686358099101594157300090732314055284541644607734865797363613111496055241101601318724449e+99');
    T('-0.00000000000000000001846850219978014536016505660530611315774081617188936233750581313204111802184283556512413812', '172294.1773686139898820857102416866146212540421600646329719718', '-3.182015393741558008777043196173134950988956232467393142872076553361750999454632625636035727486190076054675229759462180914335313087408443832697591945016e-15');
    T('-18106034022632087299566240966', '0.1062593952762739525593394732713320570280318748229897368327', '-1.9239362260965274886455362609833980864707868629715502819707822658572629454695338283882e+27');
    T('-15954726647545197343337193.777996734931237024726479085', '0.0000004481748063179037', '-7150506525118665829.3250763667151579960827175193664627204716990941145');
    T('-0.0072925304890', '-1691234031.569314001265549', '12333375.739253610871043600667823461');
    T('-1865227748847755571775855702643769331381101998226093529507611331692677750602855545329939048493285354105512957820154901.7086', '2210049489063501217982642732440647263267341254731067280328753568541239370364.8214774408688801128406378268765465307868425869089870377072', '-4.12224563332804677406698803644377193820506176786228016964994619508823923498100669710299638948960004422089468802904369403398678960200548365227202005570666224092394691081191293375038692545892613850007520900934668836420552876480141825643798919514009790652192e+192');
    T('-29657447.9015389403059670276084073033511112446764971506273354986137667779619286113465', '-1827855316102176003105358531801341705072592351.135', '5.42095238088512761081531495719462889138251775255086295922416345110850703827501143639893274817453208270263341869965571991680731532775e+52');
    T('0.000000002303893648170578350944498774746492391128813731693885869943488294353601573494998681870582486295872268039945356590815', '-32442262.91320156091101969855721550263008139081880143903731313592785661221100831', '-0.07474352345800499923522516975799224318500890706048459253531640566042618663498550717397089326803150428347426693483873444675443808499665437424291063143582928567184402307096634148635840535523467265');
    T('0.0000000000016576004870465706885062', '-55848857027244311015.8188471734270775276780246873885866419886637427519716023962487120351937093495264631807155', '-92575092.6093544619318438795137350300013067044584954352726853911684860351652174681495037636300286854678776361173457113848293421861');
    T('4060428408888.0232603235503282900294503488016711959024243468046984749052603493273497544632749564430853860097672053826937821420346478', '-56342167050275452604675195079699398.2675991078629882729119248786198064047635210295605913955475957472202624', '-2.2877333570925316685876047368022072172371889058683599394540501407358757119526517889443532452252466779232915679501821225464672490791576862382231853032125642473217365895411111636095821515831491602289971848581424815174345498160459900758272e+47');
    T('26299685148174659.21498806517835627517278416660123809400463889', '-0.00000000000000002810938946123709113663641801589824230970112320150509166', '-0.7392680925379544113700127042857131426674666337496775201943962811858815880839349410511049278268042443619259946506574');
    T('-0.0000000000000000000237245065931924834498369898311931358407431388348456381998654995741405648243043023156623847512508812546011581686473004618459451222129595962653', '78470.0567305612051127506377842349815403261', '-1.86166337827238752251588053847936462363465535270736342476553827797069992977676755247798085804488792284146627152984361709483184324719876052483334166262924374381677162633704456690411433e-15');
    T('-183824463031586708', '143807363604069401113874611858643135635193494053567940110697592615174793997761093.2327486120912246439210504605813271646434876', '-2.64353113945062034769011701589673384301679059470040712450349324003153646920125387527703799373682836286840975022033298195495859192205669228208e+97');
    T('4709101207483984726787440699170240831795242716.2138423114488136479566547686742376976336667025624304570806054934', '-1', '-4.7091012074839847267874406991702408317952427162138423114488136479566547686742376976336667025624304570806054934e+45');
    T('0.000000000000000000038532944598313834956548814879372', '-40661.8380161978165598078232608918885279594361853172147', '-1.5668203515437617982746501980121696087743512254235020270033803551407710522231355251684e-15');
    T('2334886001264116563500830341789350499377618860287461405266703471620193083331061.9777697176990366889825738659887530082768575891812891822435971810032332', '26035538.8683211892476845585304290220478694825782227287031175818318794980679029048408253481320254733533048872460665035942196191856378829543', '6.079001523901094420088368729082607832663817841432796296671692743644677805701345873443864752415327061436957752682222783317280683048921810139315200905217552914921575411464462577484189557662106960763012918216551898062340176680217599395447139612485659290222974247963252522982643623146784276e+85');
    T('-7778469807.563', '38672644915681385296338808681298386088589', '-3.00814000855232415447033790891667731035489355100198607e+50');
    T('64622392353741.4865906677596431143150394253242646025467', '0.00003378584792447508795371328648463423391931923728963301558475258018764078318119181', '2183322320.579271598509382338738236518894882788661215479990430711558769726756727680434235068590712469349522851338102909922320467182527');
    T('-0.000000000000000001648136935803060530512720919823086792624816339685214001103485134801966927382343845709538610434741849', '-4391532746732420685914713997682658712351988556515487377264609238454668698925221533684202870806667745235943356', '7.237847324678369711768998633093781574995948295740118328002527697546317476553671589822452609098233955280840336196399777925163783629478472302267347352300512328538053758750290707036775767388047947510115846705244e+90');
    T('10333000414188250420538344710287739407808614536951039.810025625233444425363102180125538936079684897578202747788108257631148570', '9335', '9.645855886644731767572544787053604737189341670243795662658921155420371076455885147190596830385851889252265060199058498677190095e+55');
    T('0', '0.0000000000000601453886227057059980502655211529623918792629968455990334450008543269999122651600055853233978797789838530557724224959611814658608426039511317644', '0');
    T('-2297750624251138979504978190262936680523712405752289172826124262469129120913490453904199762531745620.96952540734690968359393491107', '-40416204690650952371389732084289462889292036515283404683019111928741464.2687913586781776783223', '9.2866359557805037179029429569778710628234868409496948754232191472442648400693401233340833685175058579432758055493241234301333517014914341123314199162326187649292261109846406979086483744843709435080273472996954914317297861e+169');
    T('4343355589964158622184590891340741623549153108779.436816', '-0.0000001064974529896251240308460910120111005761688204576301219728', '-4.625563077594334790148387774508543628581450404563732480957583510623242646260781687362461885461530966385708706048e+41');
    T('-11563484501688268.66686509003783', '1322408781913756518185807148018074515.664891437493116279057', '-1.529165345455618514610777438327506360176848752673257181206491978239020817017600896672631e+52');
    T('-7105422094.2179667', '30559895492351516.966189644412389996', '-2.171409566283465162387924857949138932897137105411332e+26');
    T('0', '111016869919462691384357858914534606927438531477.1293238954660357640541427103317553925', '0');
    T('0.0000000060932574133709382983', '-22.267165855523514466854715520800113116277499916129314878467293624269528017916046898922489352991777889888803407846', '-1.356795734239288861313869942530919648656125995428613119992994317378914675143711278820250703114800246598130872149242193747276361084618e-7');
    T('-44333.76', '-0.000000000000000001470079352177070580940', '6.51741451803737246384545344e-14');
    T('-21357371607823146.894996126592622902999367699791992019397626730636288058416175548036595334380731548047707750374539844136475793453543167632520829674', '-2082610833875897815259749572708491274194096.1328444788624505568347915241817370559885019276541831998071798466253171806140327187679310', '4.4479093493565988402652146155895503067864965364415050875276153305362992271006919883790488712645218749008589188930440013166012155736226914204651235463946650075417869390037652928234175055618130254249241986895815834958749240488411354040161221049340462201305247819550627184384494e+58');
    T('0.00000038779', '904315692695033441415603523306856029010929442362969497588323373295.4261037009085085062076151428035395486146494122744116706', '3.50684582470207018246556890303165699490148328453935941469775920930233288754175310513622251076227784601557274895585894101741974e+59');
    T('-108092399126602892192263936618253.11634898323699426667272696837431678187861415020697', '-10577768091951217923840244460858974390254947969430786022673684947578297621240080038309605361343066868902863031903186141346699486064112972330793516', '1.14337633046383576973976076259460800215089889515619540807214976204561990493150424651385392676061643973011767877937603525233712134586553308518682088934561533581793269433365489470332685181218952765840412804140387512176915573400652e+177');
    T('15467641140816407251201603.17330929847758676216749067445955101305983585263772950', '-577622788567995038.889222735426051145123753952194531865720030753211409854898055689387177412129339425045834175708407053831', '-8.9344620083274171838850431285315863222279965262274085151307500736726521153163765171506606924039712216379344449734092210186563577666902614357166638103925323472931546721201173791263892223252481167145e+42');
    T('-147577569629276477202061028275376955899710718777', '-1836553374816.4194414', '2.710340835498528403441827678365194491127557859039387630435108311678e+59');
    T('19607260477306481515065963386476904146461.13', '0', '0');
    T('-7587887623467943790564171894728147122394166719543547263769028178587424899740120.3214878', '-0.00000016920971191816124290279600506057889818397766687630546391612272642869578133220841440723450090978526958058420544559571', '1.283944278834391917935678304507329917170307310293334436436941584511090251795678415124232910095377726352226968886370226267649132152891311373385442150371798069034937442982381537637461944916510337784497338e+72');
    T('2183744843588421584150515470170983080333529023791.649929295654521467545253432380589261845763800323659', '7388076664655153708968991031349105352802.87524801859750438051677178784807521032697033391834', '1.613367432047663606017519920365670781068592286431464441643858801885153492675086468494328491512664083724900475004088922960826586460789211331922609342243762356185315284210154800101939767600606e+88');
    T('47663056245673578412988857668839696666285127435258253279539956729954000870254229114622792532345', '346546822020292249515761823895055766746157088669680742020667195407870883898961673353876509569868596891010435068343.891429081941502924502622209358', '1.651748066971262047841009039410535405983991643212862547257807586948128858978665397570993212728686939913279771853555026022126342707912524337215453128506654700200787070955669974964782729940854858026583385489299889179192801828417090757668451e+208');
    T('-108127035861913115550264284316138800046955088009916843670136654902573526429629906417162564166515174290526454135619208.14401004096640476727920000', '-41876811.2693259587009484684', '4.52801547390097523332116417273670654479155983631591114406304348746150113439265113473586255636518700022503707400603551613864908028442586791089724406027095241367517728e+123');
    T('-22930937565270017519558999696163059408815.2103786274010528531899481200409337071', '442710909275310619619476456914560821720723777371027183.19798799', '-1.0151776220056066916079907416732180895063328275280083903607063874114903814021738290370272558746844782372813173058269500111226243281691977729e+94');
    T('-0.000000000000935', '-1394991975035345206923314783009363790456653341293996952.89914882995729104423', '1.30431749665804776847329932211375514407697087410988715096070415601006712635505e+42');
    T('-0.00046024', '0.01162786573878994938984477754797863400638858513306671988962769958859110982', '-0.0000053516089276206863071821604186816865151002824216426271620022524586531723835568');
    T('-1627.2108799123697489132363803597735986233989579218110364523089725598273552977977776687575769964049313924', '56699994277601502191648', '-9.22628475794822699354291906781040053919455063547635119292864057974253393511179708715340019643722079742642013165442036762906752e+25');
    T('-27116910291.9182040738629485', '12691061832825245519.40893289319955481488011346305686826840916092434155363686258532264816410801529729986396433852053976574899754971', '-3.44142385229909206506408450914121490994466313457211336682505249135405026892531659087017902656062527076358954643091265672241689615713870521464190780855919935e+29');
    T('74334146952185.622333564428648946767', '476584680993387873045.31749635019901122634977692103894159588456540699130081843563616833608386', '3.542651571212300025591240199136282046613545603127423889427355578233920699125928663269397001117473297183365853355090147038788062e+34');
    T('-0.0000000000157591044652124620704467757472117236708299027185378061', '825889432875054704437419479225672139169234.4648865373257367774758779311867911088245084945960951054201828797', '-1.301527784939306255868774017049829614469141796766685094069536216576279534218241902166490354194411368867404472498297448815923977414878533155578679521444041822617e+31');
    T('0.000000000000001100192990777212594294335234679657995433198981402645', '0.000000000000000000432645284322379079219999', '4.75993309304295726917904095312991723862503538234489037361811165973555497355e-34');
    T('-300700711.27857809815492793761168238841641169113962704371', '2796485968452997110905186933271347618098487118212692028161371341773219361074458463009437202075408782.868217914378353128764067341119788514123', '-8.4090531979437954393871330563049695911136868591602684229475806517303872799072423007746634357788114292441025309971392965563899108585661101482006280251358427683195796604902336825200320040807331633e+107');
    T('1001261567690509940798222741590486780493666562063067481645961.90629710999960001', '-1650177790476279206501558139505.7050853', '-1.652259601460341163039584257033432854399411843740615967044468360193262523967905931854751303094262379472596022930853e+90');
    T('110.8604067291480106069111398751254426097826209058897035561124403844051579732', '3550086474131605383663436798312168506354131157360313848273263892613620358824680555084478737815742184487956353593187968793241400.17721303537436259886645', '3.9356403044587676036027794309018622288005376717971688259462205139902681207884104868717633863448326626441423669459931832858572563749403841913604929877516515276589416960678197046131472363670530588356080270487825651541549947914e+128');
    T('-9565239171980107624419624526395838241183446774838567911610629737292737676405985674082492.7402072097442088866372528167684894637964203298148137737241', '-36054341543178006514284988556899580296546999631717365906312843826084434013452', '3.448684000487559907760299189700848766200218042465011401311363613150401470742277874500718876731897516621371622912032460093110240951501324564382188117403446347500943089873810691365498189390424553234241009759752460955435365932e+164');
    T('0.00000000000000021536749330373908821656161952581400559880285070951784338817932051856533596270829157789717921906316591168026012467390896523070770210560122939454467353', '663477678146178303688345264055398845777282913816.2865250058407981284027739848602730238855', '1.42891524405327413808513101984282436944310120570351565859962360818367363905163907249459291799819814848604018845827474322713798614022002843947155226737103321232324111623370722541558737029258436237486465744476494522921928738155070489600815e+32');
    T('0.00000945', '-18898734150623755402431379573.925485263576945358998697169620788', '-1.785930377233944885529765369735958357408021336425376882529164466e+23');
    T('-118155484782065759745414566117547423380595772557270766076629821949550123865.7556876205552', '-3222725953437374057075335595917494.78154846586409115254934373165610464807636269976395928338097797032693881785669329213286427799731102411887254283256', '3.80782747348138016629045371008871196005073810159724255259507985221047370481336261384421677347963935331112831607680385035083546977933094742953609409763116007720964740775926559725243391158892800768754854303883925126424611683360687837312e+107');
    T('-190628036687751797516.3923598265032797533714657001', '865709203990726346661478248439122009697620464322316832519149040247619305112126151915240.927219690074057652788651697648519684669722701', '-1.650284458992685868486399903806413990004373003855903554255634262462272823920733859164088044294272768929194834986945666561794180291325817436074210916748703508868419322426333998279701e+107');
    T('-788627400197335866651447870.157709244984248131980873718306382781375', '-86747290916497451031316747659315856743776649872846183098534827784400266496743256523741518237922.2996321602121937', '6.84112905096393537461269413785832954919219508550269374591141540623116216431585547212308389464605406974151833014851983606637628384743188851383796216195778432408403881159962523375e+121');
    T('-1.04017', '148558325637199565292759778627500454502992119749146789207503706844555418', '-1.5452591357804587183056993893496714776037731319947001572996913074850120914106e+71');
    T('-0.00000000000000000071132398470582962692145075368461', '387290994676565.8926187930695693123250251643', '-0.000275489373574019100458714153409095201115930261231849509402017099019195631423');
    T('215107310441729713046298412931278726261089816.7425287833514081638904884659853465814509169056957391447566', '-0.000000000000000079530922946391176631073888012201003836910614023124515004051787554271', '-1.71076829319466519842618108119001097767954970538486717238178897562962688286230746264794767938468573458599488912245410770935262628286074407769931453705541079989415475854386e+28');
    T('15.20', '0.00000000168020242551215734349782244623375617292165135038473086956135734916317120833176', '2.5539076867784791621166901182753093828409100525847909217332631707280202366642752e-8');
    T('53.7', '-203077092210298580698048848822677', '-1.09052398516930337834852231817777549e+34');
    T('3762.1500774483361388784232631064469116842540525321933222132260527508280723', '-0.00000000000451157562475195223090446528742025080950824631951180', '-1.697322458607458258725685631392223266464385065379983966983024481605281774480724387957456810916611803963822277644621467110314e-8');
    T('10153060082382209456279162718229817773.21414954173571898129540', '13877.5946499736990', '1.409000522801288735724123829540759783413794325474382601016313019646829496846e+41');
    T('18897399203675243.753780', '-1910633732732845629536812142230308713253202119068241546428657394632038831097695534323128365467560033377437820071074546249694941907476629', '-3.610600837946073550532020697653346279388929417890343273657708768347378585716753592761967644855725372607255782094846473883590443720333438235921023941358040762e+151');
    T('-227851381988110698228234897200521545909.8859817013555168022664133019970486786396950937267160151127235063398214239139286950658576124761', '-1268874936070643773798575676971891302001120798002567287940.21946823635809002112374440535791936322353876611612756137359816293380527820953454', '2.89114907753771796464127821062921785105163932083151495322011931184509082675890958425989774091841783969857985176854191074417481688593300580341868089326459718767784372087364119694552216708565314138516855049665928747274756209929360259354799135676979673768285573391477874494e+95');
    T('696500910829095709265254100912961.759143821807206113527355359619517140', '1114499552366341774527606900484796988675131209334791717032223696865951521317145662953110337604968236933370458980418848545855167865.33', '7.762499533417764961744733643290572565963244168792774190520762259732634644303990430873725013181100121300237274692989559835517257654859131921380793007417874521366460660750927680178271377182536191467562e+161');
    T('-70.10', '-2517567781055870080456863350280219592820297586247709916931434675762.61506437297379043545134128087973548639372915', '1.76481501452016492640026120854643393456702860795964465176893570770959316012545462709525139023789669457596200413415e+68');
    T('0.000000000000002306594150940581693662924121203297813803898269978979627841840215966618235769199313660292073294', '116992676693387498070382.7747684654799920318525566565823', '269854623.7638501167227065187659703931498533193140103582402361671530932334102659535168542374567463444185883703688336666394633637064104492276191430962');
    T('-0.00000000001181358510850947772334650728239469358105', '-481890547669680463659904229596086456299864735360996516824114722268856.54851170927985881751230609513583653813147810', '5.692854997882013727740130126310283585722443259230619411261749891601386990764956284506368357210347041750883604578383235465621769499444496072167018650005e+57');
    T('-1834.126675', '0.00000000000008674517502282013089240795076167689517217195581862368207764378830001739241505540256750749795374881231669246429195318071966775335388150', '-1.591016394368981357967569774740781611664593018538591570853258932691148023923957854519289902594786129196141965293563166090383235636747739390125e-10');
    T('0.00000000000035250317144887670812951006342432995417313795500599613972504272719157358113604799695738826228638715709865929871185179926814011340468405980601224', '-0.000000000000000000021272512743884326405045478763658819652652411080930898072232071938774490149150675831747076538', '-7.49862820690587140909540970755250892258210729930767016105970954978799195506491402524535800885748254897893970247659807375624928755168639617915215048372226158776475176073323855448724956084743637169140454360251047844711344183201584482512e-33');
    T('15721631316833446647076872477517236.12954053920703267418936679341513919357706705809389209070714586294463311338245994298023554035379477', '-357511350671198870300372232100538.1303938329586437211936443603854018463080112537158549352615274628829848363889696735642169838', '-5.620661646835744415086989091837302307955447632522363902002938200959151381457803806059537394592953324800087575638876011854291111449657686052527475421964247595605170282782797258746087685871582969776908398036007564256205557694516425413130055587991480013614726e+66');
    T('3253729487345424036939309028740036451', '41382.193', '1.34646461615119395163461615513962735242317043e+41');
    T('-53618931066856441239525329925922774166747059804.731067937979588616198646926180', '428.71708', '-2.29873515597039782674008800318782280462672325780696736316522303311379246101428651544e+49');
    T('0.00460104302362018630202736794501376789373696947488687556056401041393190634456941939438735', '0.0097403733746400832576516808174515324023665911967721621686382928687157274600680878342008', '0.00004481587696284356635234124520882413754979811951974130365547261171837147431228113809768987174613255553265451013766186937944499945070524997906900028884951249482196583289287988');
    T('1677453254494.8933968213578859249745170393218098919671494230225238019960047762641999542148114948195560618412253838', '21698.9831813409037769663882983019728670821859931084790618654907569761557036686614568057014715225588686176', '36399029956770254.61815647295605403222195084034899389723728455774968833742619776793560171529933737657028158467430352145406981302848175254658241102272245868058796169820204651822347385243123767173210904612588453433543488');
    T('-0.0000000002919695530321722484984277810687310403212039464661356722375751251668297055979816704921817077930395835966342868337436171143003475385582696559', '0.0000467435515211462536099607610605749574554406010634606838859023639329665349097075031538491899684093202279174', '-1.364769384476538686936225893376326381400072912004108048321079964657333163471060859282060687267616249897947560336772938595308007366926166649543531172638654882898076593187120930620811092498003694929915967620884049583354836924979303135355647162266e-14');
    T('603854611243120547.787810466452796370957795270702329858515593134870178861102533', '-1.443648977792806959560328531759843896504517453521508829432901295106075946137', '-871754092256603815.457694771840809554026399109994406269191599674109909866767981863687374560559981613911867626356905487893301607765864732838103294942265021');
    T('-40.440526242216028465959594055135360244752337785292869659524201380035562721496158514354001447065595303342069691098', '-6241679606047503379025590422716839198006778643413679187399633462', '2.52416807903868662768373787395500221586271219331254483212435251241310894921233219736835628583965316050937610054902736153912767627312787284137615949322472984890293258224764321276e+65');
    T('190202434.508038647841031698547078216727', '-94800.49', '-18031283990554.97275426724712779530301404579623');
    T('240457787673551.0553964370137451859068301029702133344054038041468987152326', '-113811730.49021', '-2.7366916924974332953445795236260775735980418505302834775806957412715746250777592172846e+22');
    T('-0.000000000000000037347671655579143058079422192039757670106144334496120979342554992077029390338777020220866582016017618507350840075639338', '-66002749861804986801728090866144973725446555526.44841912823309868929360696717462761165075411', '2.46504903020401430710916622478946584523872946340038310101035741960964095064955939570946178742591696202144017351188867915011670426023840138265647815988159839485263479191111855834656201283339042728373652808117918e+30');
    T('2217096250855392361576457107363.550101329506097526554933312928808962970058721968159857675935', '-119.02573598179826220767289082192322334412324905093960546113476851847137178273867550322447', '-2.6389151300054870078699458746934883192689259758705923255722692155151793736620554306044775120676679617433953726881548219320386556081319536307857183199189515857363954254037282212945e+32');
    T('-7810543038413013183543974906750692474736112.830520775223157129955459725598357936920887935346548627357711660362938363582877899220351', '-0.000000024840601734880447844125586432330812489626056994428818563969', '1.94018588950360699674928168207639163235428509329502583916963340815490425553873121778659312254666940895194771657722665285619316279071834113799397487954421488584967639435397144832420520133119e+35');
    T('-215969900665491429229701.3940073328175247468', '115249.193647', '-2.48903569037205758977126520019369452248876591678435796e+28');
    T('-4035575127405764517339706203.4788226338445902', '-88902309983026598393661019935338468232233526534344738519100289640088125353379283048690019987369761984422527235042814970844399233265479337257', '3.587719509364193355575242986206913087493287494035221267169492881995478618778834577094297983987290045033929823209982268817160327957706935779582674240650855654673552267470295842307570814e+167');
    T('-28499916611563776888621.53827878180215801449832746', '5995.369746918629670729501318845959128845800374147641916915834504403849426193388405031670359811808264465541348192', '-1.7086753784267317072023465954287299476765112854412427476967519098923744309002169668338318340958052443834272196779007400662171578997777665622369069868825749495232e+26');
    T('33940277567855896556536207237998924729047510555567616051277715731698465468759889055793654824231539562678564552090900045034922322.1690641489242517', '-58557275066496364264951120471607827120696885728717812654290030305387606748831.9183665907629793676083704092979597697586525', '-1.98745016937417395543833870238306804764880834969233299088069121150704606919273152334891301543693138976149441332170609730695515633779077917894486225657656974898750258313446228915338253648124299689091087492416283574756990938405638221490525215652696287427161316283425e+204');
    T('-0.000000000000000001047064413365314260621529874256953696906592212846819', '-434397528124418421.6091661017223', '0.4548421929529367773911801643725801508623453120069985877346513296252893958219763637');
    T('-33370590738262171579.39614973283917480277678311111671774535779751556185640913284598686491', '-99094653.33173767906444057476506', '3.3068471206833860362071327479835859150736781389077457604266441207582457927611004156098420714391130319622364115692080446e+27');
    T('-0.00000000135893504172734', '-145791096031983569080162023719458335233663720930454033985903749.924827775901017923575087659941838981388109045', '1.981206291696982245471792411246101316206613787316920816048059429054568140323128384390929922747612723701453667023730777903e+53');
    T('-13865781910.2079575101787283939774081870084', '0.000000000000005111340576657977529690857668009570333996738347099523635843713836084214963667601914442680495175289459369931831218047341605612305371', '-0.0000708727337047360947483712809665199876351338098500645907437129231083200095113355222531742276112479425377592938317792723399224219930646266745465349945511268888900466157421164');
    T('2221303.063062607', '-2.525965215750429650596003299885690890329317816527744727340100271151083414522167788474814497266651607680421514303744', '-5610934.270936028330697409054729056703155654194124398987862017089179613782122545869272030899868261055124767892056053009313886500608');
    T('2955433876854127435547184981659750312874750', '-0.000039202878947452162560625708217061180169183736624538113434241595982106528764', '-1.15861516511511599582586163812175464411359319256995079161567403334139282606046617827274676780880858338633660404309e+38');
    T('-1925003256423680784876.1957689269203', '-0.0000000000000614144439205338076312802493579740530216692609642343477458243733481030491978062242259372884320561187326591221623660', '118223004.5384771047512790006924893059325401413863133808621379188835488070846162209706362064697933581769897034762423998928884355927161113795649414298');
    T('0.0001074280907302126774691277050759016234153511565364175703838489020817316583420637215', '0.0001532977223327450369874510044074807629409774927247949577150151195776859623266960380925487660870709945455405380437750382992', '1.64684816234970840550538622604713249522114056250203424510426725871022595010818109545611749855873201375214405305332022403202928107807661566637499538331414567985551667985554396394756099731355830293824728e-8');
    T('-51719564234003853480795874799029206659495378941873129625597250275327964731715068962774998751', '466532423420562663.592331220126361', '-2.4128853640345274449820011797672270168670456082913030458542405572274060582037181131606130323033781460111131337883517837175111e+109');
    T('-1724059986073957512357853.05703381906907621063941826220185749299041791046931825802106508069', '-2665994918609117220274024374289170787044923934794140833582674324712895082643859871981.5384174394916234884976873', '4.596335162250476126397000620689369599669734677111641881617412368616916429689082558324657810003289232013968094235442049486840526116938235377972333081125575786496773286246190410954882029026469852888237e+108');
    T('0.00000000000002795129561961146401319754718204318652084938036308535981529129883306076220009274119013027424', '-367284334695386275069239735137863507066443274836293057467619925457742005840610489941285870890827991341527453.1239055840633353121379761935', '-1.026607301552306124313300154008632390701208483751483889410351517683637370661669822560424911336313305943490756696083944005783385779384493150385978269507423733524787840537586575485518520814488626005350057699017603079788874630544e+94');
    T('0.065024658867154009934139288250506881782640024913856247796477081504212332678', '-665460681339474763575385930611037367230025491535512485820105677925713917024055708892630604.560260598070556046', '-4.3271353793603226678150325769871841452491924531112895833715271367485564019397270216591883369859017756012808701471442567394097379898539481503530181916728520372454137708175982196271188e+88');
    T('-276510.208587680613648136603273411746807332340011833', '2605351321798093.990970489610', '-720406237434580367010.08789881220282603686585579572198020594015398214120355513');
    T('-1141731373426523828', '0.00000000001228914544864000403177026028329087085', '-14030902.9113140661455213068073751399665217637223956138');
    T('-6945202132900812232267754762195507765101114.626933181713698994880961083764777532865251281976086752243853355949851812697573518483229325256100662271783', '198.500596182566233231418150714777497924550531544984107394883271', '-1.378626763989241829423752237699325897620912267936419616678649561070893719008729731590007841901671107837003264905764835754363036476641340160319240720502391830298256035482481144857802524759297321191775928962042193e+45');
    T('-0.00000000000003508292319046289357668381194881192144875489890254082933630984748085694196310674317752270255439632992013292438026236847255227135350509221', '-12646.289', '4.4366878563139579594698714752632876528625314168731416208665252478883885572167521207173040056393420870934788012794380752838459114284655905930869e-10');
    T('-0.287610207934561349588295090691549344568062900733', '43561994519313302798849015634739297829426206398579890440129222701736044543165037398693801937437.9243029452694915', '-1.25288743017439209045579990822356125048347677549255399828298801437378973121007271697418144364441489766184499365629256770107139603488677894036054186818978872695e+94');
    T('0.000000000000000192', '-12108079130041324.8599910305913709996877522568944631978562414383996429587415771634862023459492435054277466841987842715088475336753781193', '-2.3247511929679343731182778735432319400484333237369339883983561727314480783828153893508504222547530421273633661665801296987264656725989056');
    T('-20568177346839726327867757072303820722593133821514591458610274374266.37198646', '-534342814455338031486092101151521752991657451149454616317240974476834470502228759280758223653930620503606363601006870', '1.09904577717268667573895775359758445299657810692590923812884672106543889507154870122646024207383034114557837930915217170509202835912718118550506629814473881108189227609033460252466188180069802e+184');
    T('-1182.3299256870', '-147162546450055552210266162228061556539.797326023627550726867910750435749282172653416371899801978172967970344774632377', '1.73994682608203866701785740185594706489760121335556873257063559751827647851659356907682594962027863260816409968610867061154167999e+41');
    T('9106311100510091578555175.291251271962559609876', '1014206950166689350187558262800656656127736355378743002804820184380658978941538.13338393728923507197611', '9.23568400851740850405690546305915901453077329283645359090099261781771022159231929782452163389786781433252697559132084134612891947899159122199206236e+102');
    T('0.00000000002380790398795067678584772975148957851144823495943191934104092082796285119597281', '-311335492270699981.8187024249236948798391482088578', '-7412245.508022185205534542741414892061741870570633829375932307947952750675656606815024557059080304215250354778137764084129956418');
    T('-459107433083516704803135533549.9370726940122774506638331684', '5216864295258985882129407260355552822654299758311025336128418645120106444368253186174452684656258241404364726655870378516906394208438578086566108931', '-2.3951011753414023938728351576018231982234001717277301016858655888837322273016599246665882605588507148724528061606913078187945915687420371837702713713354429660018711277008065538782057324960908955177252669804e+177');
    T('0.0000000000321891162991981855413480133794894989319146938941669481853712504240526430027314092932643025', '-134763534415664538713532770059667389.177391730107496493', '-4.3379190821968230280422342891216605083499498614921463348518339967882865388159072601421584951618987475295234426996826071260203876918460408411325e+24');
    T('33279.339415206582540482746683123451895750217730735', '2358123523728734728589743.77319797', '7.847679312915151656953999591683173680703507048673633215999463330697682756730860795e+28');
    T('0.000000000000353739089246492910027064870427354171506304617311189028621762245079583293104983994786058488849', '89.2', '3.15535267607871675744141864421199920983623718641580613530611922610988297449645723349164172053308e-11');
    T('171.11526841158167203333557925506243707903346', '-892230436.9596630521366514219721041194232334089033401680786089416696171067640821518918159', '-152674250705.335543438952814693411686830371790621030553860131456031688753597684538585227522590915650930288422940391639997604056260014');
    T('-733600573426', '0.00000033040751206672045761849003644598960142858564314273065863456912502993453964294661705657514355828281447882514699', '-242387.14031640414208016942107704561068604361659765821593779056830839506769730896756901035759801454275354730316665273788774');
    T('5.407060250', '894983509287282057739447083.8543243233619681646503346546908188657720214623', '4.839229757472768644941169184087133839458644424846279660576202729065982810949203575e+27');
    T('-808729895781.624674969350592579908381307343041664310702922570660', '-0.000018', '14557138.12406924414944831066643835086353217474995759265260627188');
    T('-7874173051342200985556358445162694781995734030142926506325840936292568550580411005561682118906679842482219.0492', '-0.0000012325012891970851064894932661125', '9.704928437140208129135757974157260618986599406272057601438444485095519271950063154371970461800830427187707721546146166916789646799572330235e+99');
    T('-0.000000430107308276532278514', '224144306746606979302588712185931212406889.197263709151077', '-9.6406104440292501862428318854477364480895829071122484854759676386315467059578e+34');
    T('-78447932062514438523504874.05548646556284585439748424869932707256764818646061107800377213587367912600020326136', '-3618396947489172957173653777903437148775132', '2.8385575791184026440804730054109713727456120179794155436480050335238908442702365065025115219164988251654681117750347278731430170767508687816655566449952e+68');
    T('398344525499512577582336511121666224709052905420447547608128587685795296475632034766287454674444514141282803194155553367745120537420220770', '-27125618478287015045404726742380927685738591197795175985782379683976.193104266048757690370843265062495822', '-1.080534162161405142653380276457947626239920154503690438713413845124333388848808189678412350793281352453783298170743129726689403893393610196781463690430258145190692262396630679800143839228210109253361757862038429643553279024184277250644262294e+205');
    T('560555227022785413491923481073654909048982696.2480925260763', '-0.000000000055945909195749319312168314920740513457467773655137010097489061064437802811189813038277307920819234756363832360445689012158203384', '-3.13607718302193977963347461210034229231284614823237752102010861278943979945733599665261238405005674005758593157383769276422304789339955395466489473545510344991715182034227247859789021992e+34');
    T('53683812799175377405416129296695521380051667500787926235235161394163291.6772285236819124409364862784464258457438293794967799666118416541092194902', '1002085239338631.91098069654292203203638801404514242082065101889787248609067618438261168776', '5.3795756397471969187136355919525769970238781938603258307213243094992246282155354543484962264142098786182227068998815395594712635092562656959880937968190303422424114781273414341203880090074688223986213775703111763870360005321708779952e+85');
    T('85251406536953414226544773130024365036572408507583595068846068418600.1205319812043138751554685608133508306427759344342315262715891140344502', '-14640.836750817778351989018346781730874633202', '-1.2481519258851345347885051287664718872340382940818875870448237208943268320517859270539035447679237658640425284982425765828931101626280359992735997817678878451142046283911403167355404e+72');
    T('-1168964957570.815587458251340', '56538572696115.72608565692028718229319011374125733775862205437667843677102186943732802', '-6.60916102328293923983280676454563885183081101391883074333220411142712357555724202247544863269382868083511845468e+25');
    T('-0.0000035740568614143300428297245610418', '-220936232135526', '789638656.3990059006672364554989705373573509868');
    T('1346034386225456518679533104316275599194652955318802468313820086088149624400847.185', '54939560960135761994417267108815425036339300614381.4483866588714259041240857418173085472293222902548715740451745805798856530861', '7.39505382164723930246517776761647354791771483640955760963662039805511757685269752798232203634586599328591387494964164173089232999789108334073422192252509759081688578894765226433014479831461369862878497476285e+127');
    T('-155646778363951461204593301246056264317969228058189198896139899890727803045.11936198483334033209653315521827243605631548584036361770', '70010086124901511162734982850', '-1.0896844358323698924028061858087281789212298425539029658881075600265807741939071442787428242144667285959576990020034537638802277308860865062577335864173456445e+103');
    T('-96230341173782142876521207175875305814979162902343913723999663923462086933.11747987173088004223522808909664360812790761033270386408', '-32312766854681308785639606074893295460039.2098266500874834272005936578144638474562541816848168427108675212737366956591462833878750808347288369144371132', '3.109468578694861657044341174482395073888223668003904483303702568979989305498812009034748469873885971391784305796766553860166319366182690854126591215571252169482729464411083827831940112574078691557408084659925085229544889092188654423122525035195011188050614487145148823899800373856e+114');
    T('-0.000000548494172595574330396761429371414006142446808314253568896786945188386738792461354439029277', '-23599662.27946574215991340', '12.9442772355105479080497098082314463897599465046293055191920810370276950781896553331072508851470742134787983846118');
    T('-714491311271945967722626072689371246725304', '-354207932078705268748581131.065984233837379302661', '2.53078489853838501514954773016484025244072911508100754790233958626529737581211243343233944e+68');
    T('-2652957183438', '-195185962067717725512037169887386249542535008172118583806939423414913218407717522397396925805258634149', '5.17820000173808723699241726590004932229973891258816030817795368301911879552134013341717829627238806141061924024262e+113');
    T('-0.00000000000000000880590826878036444443656266764836587490740155586607932367514263056628321181262993864472136637', '221.48126943989476644014754266743958579083215022106146593457414', '-1.9503437419407411606247922977674915717900791357974569097861654104985704464836690328779827178384445759527338871093471950304259661832134813136971685648676718e-15');
    T('-347873602.23510519131378528375967688328126', '-0.00000000000000345204441953134921260412723999268616841434091539516160053217948913648444927706020879947861434828121114161104100848431013272935429403082369582072600641', '0.0000012008751272979631661860642678932206188355136599447128288384619284997247859154163277648673794145259219010025658692572618722220704615506152641552019366413559230553625413995603960631242165928766');
    T('0.0000000000000001442618965364152298486684187600173330090418464315351795690935142411476466184129035934014488253341654078976849545022104268819111681009555', '0.0000000000249240364882037699910299485535757120039682410050255081964884056237893758868', '3.5955887731310902548303070850838164719748874637094192870760300960851465410299817503704107844345957619545899785709783227712047477876114102968736355229590341265414695145023045767563414004483980850712921197398374e-27');
    T('11203361460741696775550516477439054249886501188874281520876798975981497083993389664224944576772981555476668884535000964169301021', '-0.009425091022859976408210896617129512523351916940774040126129926398280814501901637533674397', '-1.05592701529491998289184550400386588919196745317033009594950262014856245582393791969590569776888682089563381256271169054068993633728958894678614097027273021445496799602248296257190716802772690940886533141705293659337e+125');
    T('-50257275098882173690313530324319207319.7', '-0.00000000000002613102305812786358153112867873275089759049662992988804013925307324391675101078591150104728275428248350857948737528851613002722', '1.313274014447565385918882096099252569252443657688483069512563244738248437919967250811168909580079875716332790199500861141616087147571179420643034295663475485584242234e+24');
    T('-0.00000000019732481798', '11713877058294598799452.6300621325958315801323987275857206144411781296421875252993224990921811411', '-2311438658368.079557319116750642590563091456361363221635925091739233848401495958715457150683358908746196978');
    T('16669921204672133066496720461645831860294144062172262348452206781980345719813505226331586428501288033918045572872571033439052885163185959983499', '-251909114234', '-4.199305085019531263164571072959105073744794265399005234122464071331276442675313245235919630379775040752398208810774467987877807125340295724868202346024766e+153');
    T('127594991665.979', '2931972609362423110559000705020991756049379484274406531578719920328328631360570935031987.8511584086593935337222578544201427', '3.741050206564770789405448806731198284533698395535872424847129715200781225838343397951497453223347295962658413239336292329816540879152033e+98');
    T('146200133592544330684264498065226245560031705674109303503477692120859154893091630.871735716026997526317545935283624142687003866188151982212989698909', '46730789676874137872952436533.29531686073021669792243838852461391282932511897427150915122385179702857290277711857', '6.83204769364409047271650618653599900430147034336920824087779336527502529052104798054004653483290666541743622267860756708314673328113463875187686742000697177806992937541292873782614801211759243277925667772455483685539661134063357932775410662696035415889264013e+108');
    T('3305168235.08883', '-0.00000000015960253276367470672820299075290130045238822873945326349880534378217699817857400746554521112484146318053536850963687569374932812857474', '-0.5275132215302218955363945278671543840140049260367406641422620161137050522156464498199756178493590046194472374663980497123349110510329792890451941542');
    T('-14662664077162165311256', '0.000000141784657393925274142572490861043212868340562120628819802636951941686924487200580031', '-2078940802662653.108290285852001564512570305578867791848575532841089886824260278364785279496635830853128936');
    T('-0.0000000000000056998852880676332652472177552050285678774883196500512', '-36498373273900144419354144402538260265.64880867165981459073157696308446822865007237717278768', '2.08036540862304331717677642102316632522246004626115723942329232479603003027974589958566297786436869850538183264991752492295191922735133158729216e+23');
    T('-67968411827513303349511763330956716200597055731528479256856033995905188987752276494336231454282073.2592237192851649247647340481118', '-0.0000000000004483593999475811573860351927852363194231748110211408', '3.047427634237394269949752053446075793452954414318045278492820191953525157364829988068926830766709378481812043618023757341388343262182042155382714412297436468809738064216143412194144e+85');
    T('-39147071622153.33743465738395064998231007612242609707852571665922928037980935209746718640869047087190554662186743691289', '63424511281756665049556884134307448242418039636636009044809552022044426453480310050617427620937346481967898179810376485', '-2.48288388574700054052521039639443124893165696752230434717540647035194354741975215494021537917119812595584047998724074329330913195295973963059478012613800118844862490179027576674476090214756182225141979897773406174093090617028873704939165e+132');
    T('-0.0000000000000000000212', '166963558962441896333912989698468664503074325140998167446135880665447621153757238111137.174513086233214289455390442249465', '-3.539627450003768202278955381607535687465175692989161149858080670107489568459653447956108099677428144142936454277375688658e+66');
    T('-860642938911.89435988366786337586704323346250899248118740045', '140830725536103329346326306.425423317117473697116679279', '-1.2120496951448633875490135507286030453466902766109934494954321323997081079892561368828279080230440233059839027555e+38');
    T('5120155281492.6960891', '0', '0');
    T('483497234304151.93475846468237990040573513326581342398440051651712771865349390238040778985950490599787315241732009163910863036888245878997967132752', '-0.000000000000002446148373855415167', '-1.18270597345669190978054478684634031633264057012297038134354632718936884883092588556284387957205148298464894749852350309479922902362917548718181008887660019563249584');
    T('49107229571.216725255807725802556664025020334907', '0.0183027672070302023790702223019211', '898798191.0241693052645246478450553949374710665324134212540239364754063914898377');
    T('4777991472272023913003337327759575484434911659878.9391160', '-272.978878813339754436881', '-1.304290755080515609320060076869253031973150159686523391389068499267988263937196e+51');
    T('691961893816226173138683615558104657029075119080', '20888395139664451446632.865882039316262400286', '1.445397345962386803567566022135550445519666093682044976235972214321728596830297084207605688e+70');
    T('0.0000000000003033805695921308583790907838465039054504904757732225737126675767790336448122804692481183103833', '1205.341037096038791925207662937489878783324398262713191296436049856974303800264977495244455397149', '3.656770503869659792755630630441878632712190709939383192077151780666062487640831547779168381626737843643632860934266858208328198722651475946630361327414380262986385955514241668191884519172117e-10');
    T('-0.000000000000000013736762253040628738016656811899978900772420917465579992026756713474712981645066867464219941729626816955275801365161507762', '85532614698100819357606499054389025847104053990809', '-1.174941192988739408334389452599416522954290918785451326292660032528327766954585591614133632582363824486991674256064796714808554605539806946168016562596219749595467730159458e+33');
    T('-0.000000000000000000015811255247299305828768713319620074244982509614182062286978069545537489406407030102060780870055866980118153434550305', '-401137413593688737860912906172596426086830141009', '6.342486035571282948613963880018634581799042582084278022045824002489491031661976947293164151517578765536914386152201758345969760642775856850743822412630635653957745e+27');
    T('868871903240454856454229094081624011359909603881418403823490415408076494742619348242915473328227861764639800179679657635955448452.19436', '-0.00000000006357246685755280701854672125777125138', '-5.52363302722126457498705427469309802606911867827348426238286090544579757343893328680598652010805938682728728170222222699144643350355823304335981947914590621447842841782168e+118');
    T('-817490632.47780', '-0.05082', '41544873.942521796');
    T('-2566509873328000254011581163819375336312675622809818.499935237794106355143', '-14085586641244278298102529457882882173828.1386', '3.61507971863704252530700366641383269581722371260221251588353625777177940442779032699974632687837606902398909382268198e+91');
    T('503419281104.463633640902474726192736893485104591050840525886329718599066457645293563757497708980772570127618974706495417595', '-121337599145999931368859624997930779.73179', '-6.108368693302086597981507602493580554469919267653677670683934587044246440236052848418720220332789575988975507246301734145970281303551608797453924222097493814684505e+46');
    T('154.0752879034465903', '-0.008726217915728513196107992191445520422089690044688368256659372978169115950', '-1.344494537674084306953324167403048279838310386922350436364043480401117060195371791792845285');
    T('-0.000000000000000011925048592029545210459398688308718254092506044948381860935240918133166559166620982921389189346569737569028538401055', '2083276415584377368536532532727643929994867840790251749.88373757843573756621600', '-2.484317248647283703600678453474259458742779398299485727046453258915073511408926866511770738523415042481132395288773086582850710882881051220356645201811022177650375833106965396711340182675788e+37');
    T('0.00000000000000446904154566584455897206481462213986849554065179928600359644945423277723320322131526226161353244832569', '2848064161461423289727866756193778408645084968560840504840991868260331965438561614918967119347024529487381145007808689.574200435202790467855', '1.272811706229305662213683326319709782050267108388413502237287169373774908480672399099191144562736507324487505708069393969774714426776963142634656549866268950975004320862082201493957308813283909556739183697464382316012026987207034028651569495e+103');
    T('-0.00000004733747496524284528744980210835293091798469780', '-60369256501669614091439562465874779052299549893280439175064875222.062643118496795029', '2.8577281683181092272030103013062657800563801201278542890132513734588525148374106855868764763504825752739825308041149449651072362e+57');
    T('96456491774549605983464536192479065464697606538688448511393590515441846121829477068030573224599124', '552591880159850811255567071754556187177795094968199872657.3807356618203751867762303325381820718525675700180057477640927815410828', '5.33010741433215513846896037986563851532355661394443362975026204428856124749065340874569306244290877879518226101551796386327528977418142095601897892583643746390474594920802738822250754452900709400116274658794833132949668914672e+154');
    T('0.00000435296825302837417997979246513279920760355177075128726543859320043830213235385947701661297471019483638796996791390033912786070', '-2358972609625663685288774473895.6434231', '-1.026853287946401014950993771491057785216233603666290400912634027422541733751360917945594249383195357423341794633440301199534687372368265325725161906863912592796217e+25');
    T('-999234622854868865674773818245832433202378284835.73666608911561114433921', '-7.254', '7.24844795418921875160480927755526847045005207819843377581044464324103662934e+48');
    T('-78760433369517189268813718850989812439872780124626693423016073535122', '-11.0900738491866377', '8.734590224618892006011443929973965209196433850642586975929968912375943222522040392994e+68');
    T('-1510939307579695343987573960', '0.00000004320067801896720565303672103306587888166187016245346353772091834756228523943136134993165134358763411118907', '-65273602532951674469.8274051759179425105664207275230087993219234918464759004305190552070563017062681064499654251448984977190801686172');
    T('0.0894174964177934303779732054608056301726883008695547961408331314203105438641593697918217224788171493946008016972690054132568541347357906742329332833', '1542.6581508582549101995706396231', '137.94062967824784574736167582697025827541997000463104865008652399044560913829513820967821268072546354439954364140784723516217249346765923537667015058098902147542450230708975752423');
    T('-34577745227575239042650497034045377305765805548524711395.5081303310220196', '766658358890973242394467250763846956795577882707608858967751880636.0954859315385805368772376186546221646109', '-2.650931741032301486599844865029472285624680782442203783493297156082528515125839636070674383514156414623150702190980592575815090407165174168924973668598655192785690042142376617364e+121');
    T('83.3290153999204844106326', '1311627337486915320509440687345026266029743794924400721935513620655368692498627634744007885.7578920', '1.092966146044038692204564531458641541903270444270154411744236954983690234590929647426240374543484806952751680089941624792e+92');
    T('-1192151.908799751109585637800720757640651255179973011410490516408835753100899', '-6097036742802358386983942122025791952308256986659597643479361457987833051908958694867787164384275533358919693.249523575055559871322198016668187', '7.268593990954048718183298042908900396708340089605455889963655461676079135319303447970644868475734604822116794986972434401410410290069948464123089061651877757219473044801702473574398904984445482549630675240973614400113e+114');
    T('12873087032432609726511318658818382838704088', '-11137.85789449932987810195', '-1.433786140307564932441456398853868326855191909251446120695471457716e+47');
    T('-514407318470542052534960135899863052948602062295713.404163', '0.57617979470448909978527108240023648361885980490885247972540173726145900170394342', '-2.9639110315084366361019792930292554963023439694418106043485007867268143561606223483230236306497936769993015568968024431726476622934445746e+50');
    T('19864031434939435816482549499647863708767614292250', '266674221057214323300502059.20145204832171982196589122398', '5.297225109968493345109943564396209963433048692089789948298291839648394867940757685353484913375893928155e+75');
    T('-4754055625084332185755418342.562', '342203501523496667277706676371384769113403050858703638741982429.86030457236777579646956050309789441181343032128562816083443', '-1.62685448134133416999696896663497605074404274391890811668228808135026703965039960122943110479134355342110442043580444401677907314671327937812747350400966e+90');
    T('150750927242228755712.122222703133987729780', '11153110586322698752791057735178422360301.378975227', '1.68134176252326445776964103880327253856485045353577333614876702243490238494407508859016006e+60');
    T('0.0000000005454515643719086225314528110151760642884722399372', '-0.000000015543649516517081944335531513584387020723567068639189715742064587530870420218940682937593923251202031320330', '-8.478307944832903460053527019222484930464321162422574722274786852848535790350689038575946698474468453359077459886259429789018774414058766868470265072283276e-18');
    T('-2767036068909663273137653237379082714767201822111902341892857916392935025490148010094342382799970940711420469586533427545731180775808468923.9088539', '-106163436291247150687286.993186225478584736592', '2.937580574172739975981907042564559944340709871604734255385615604239771087687041986895276609517297709801222177972170328755157939564075490962060559952941922254377647597925548092640347481119088e+161');
    T('322929059228302.42232788641548949211838', '-18539056294225888575098493056536343700876218241905416897333383900668766550246400797654743627780376379843529081553892', '-5.98680000807490479064022688673406386728717075134452992916398938660729018368078136257909679075702485459797355840674648444941273018869686024213037721373496e+129');
    T('-0.0000119088486605', '-678482308015968521186583296185550742292089152413308996093003534425067540438719433029394014', '8.079943124988915136143008357350970716799926602463815180494764874359731216575673502367418360750618247e+84');
    T('-15743621782091209915924278', '3274082843013731438255081795999294296956951330200864645074905956456916040.35509269056', '-5.154592196364209761725129736597728957541850874359227108512452329645582903082160123855441179599042326424541568e+97');
    T('-22.04', '-5944551299976709752172805837341550253708966615717585971886595904737262.553978573497219171324864133558843421716871647969330963830798611962604850', '1.31017910651486682937888640655007767591745624210415594820380573740409266689687759878710536000005503636909014639851121244054442830801407655810894e+71');
    T('-0.00000000000000000555645728539942948834543543660522131579870362355003609281304516197635570002691794651513876516995679257618212853838', '-29306695827152569391012711584981295617371666610349814272.592165236226105635164951770704628516992499358459197490584668922484814646542200', '1.62841403539766953518495453372218761268090502299476097416218349383440252143468559124456611158096941013793431583395346583297449705537413859250236966991939668836100087684424197338992971973055292271335773446637724947073443337667809840027147206989636e+38');
    T('-0.000000000000020742270836226952980998683613591994841969458265756792', '0.0000085068798515187907992734965918590053220157266548600897228989451266931328328578374770020103860942048211855612454052631692704585242999552', '-1.764520058514448864430556069534618615275204892312035230025568038037623488871862099421613268028548477427078877470868347138110966013281940578093221932329336549462641760519469552917396957184e-19');
    T('0.0000000002219866544727448068271267401609565019963347224974926712349689958555511499', '112575212318901234841480884640021.488017899845441345152302010098074629364453', '2.49901947592318130801503047727550173178035497206246305960969166018454548964265488494296651336239176806475308333361242019778922151710635090703345047e+22');
    T('1315378829978163097323201091487091023884510464263405789489832239136944342027634418586990942689761050161994364084475', '-0.0000000000005431717253475371481154070141404996706352278219461841093282411', '-7.144765885648635691663776677388815688808217253830298250560692436430854724042690565481563931808762010822550658080798534063657410136765651632899419238667830655937868686635669225e+101');
    T('-11.974214873897', '-69620602709317858077606.7777617143271605405821834772501233928978990960832960289391723644435809527901763729636', '8.336520564915876725076403852155386259151906493644770055032514166066102032748457719926765893410251855328880867224141711492e+23');
    T('0.5799373425407396613321531402639174796275099785413933819926994175405948655805025226256707030596694071531988549812177396212002923068020219', '-0.000027700615530600411086471711309277', '-0.0000160646213575591435303047842679851674438314220751425839463942452348275488872543258932816330972165180823475893661313040152587644081646971818065194220662714757232798271663');
    T('3552978384198918.49229744871077180734861394888625630207038892347651291160127968', '576121651802272791027406554970', '2.0469477755224511189481484697284307523813499893489378816814079522112320677347783327847826609021022722640096e+45');
    T('-18851687060584485145327914172608439977632012360714293181151849.676443352833892494771644740339', '-710051449483.09545735', '1.338566772256972884671120004800472882868495911897525532968781498881932306536347328148798533337590085679119904165e+73');
    T('-0.00000000000000000288249562210477058762080967034190536380', '11030576382737761767940218.42753523411870710120067239740266744266743407186080051248604580', '-31795588.13253387463958232003043592787423655701203989880802112096311021607776492918973045954883598861925991488365393167246204');
    T('0.0002651045160796089842082260007521946896171492324757847312814436724243594046468985409527086868800904047336619267796004334725939839850264', '1.5', '0.0003976567741194134763123390011282920344257238487136770969221655086365391069703478114290630303201356071004928901694006502088909759775396');
    T('-45630450424356227610905727217979234106654796.6161863250137780341068134637326896292372723834060729494223789898', '-0.001886550312601736573060142331616041564292703429340077718703409523396153539560814169322896332023638792684056481', '8.60841405122272844640400481006709825973377910773092658383432035084194649814689826091063041039246763382818233815115923828311073497379982604148882785954606467953532261460988494258202205778413397486064962158760109228938e+40');
    T('1189249849326945544423032382429163536272388440707777088772904521287431570230109527744855960682093580486474033014', '6504.8683354042275751628267409987442515379734097114877784676929229119207894859954124', '7.7359136877710967170429074459214076095443123103950889587500385890668195031469937128361362071610022299701776151817353045025434440446556587282911741416992344550889955487947858660554683425301449736e+114');
    T('1128441920157431650427749624713171449845850142623381662768791487676773466730760868814283339962300837813340670485089015543152474935830631262056277625', '79061776144493796403585114968361776195990966081.4355019441515980323901471378982885369231', '8.92166224835496029459276238758633651524460082513027939264421048853489989185952160214129336946076678283640307013713880795010401659801489431413165623876916877756744842119523338662139879091814025301405422477576287618845313203749568756375e+193');
    T('0.0000000000000082641501664127551186707163886525969338633571732489264646653134052232915073527623626871791859115793743174790277', '77102203524923153670', '637184.188091283794704877392959386970281584723592270762046140361002187615856825504876006527697363766914328733379224518124369286659');
    T('-154596.64783108125962775029015421009207072050712414460575385753089518786884795842152456687953719640163452440709563114', '0.00000953332223165578725654281988611433', '-1.4738196597075074167285758911774621445701892232319827414730606050858237209327721967567187682158190683858892715205135805644946318114951019760935482362');
    T('626094027960191400373416029.419', '5878856.79754771959507653634625689098360680482358703172927092067871619778735053759313320563725801048848752364854140567987016428762009823015837615', '3.68071713217780322698039882190698318126112699362991022750878399644122991392056045799737440726628318401494664115665446003913196526833417240938632301385924540360002145766795685e+33');
    T('142.135419283057828760365017829388355940936300046104351305886714809787574819399072768256095762422384093950730185429', '-0.0000000000278515979862993195214307567411728513299170817121298333088852493894670155456948388404536227491002693927300401260266753267364837037', '-3.9586985574858228931018800299373061499410310433476663516734373778684106112589529389671174026955027434401703451041795013355130980833809943565348714743599848089313344119919746925003574800408491140122381937990968341446536009851615571381376933873e-9');
    T('-3414337375638988582664180574338695013525096328524003701356935598956539630495832919756207376.096050041684723108331618893085565788050841', '2373966573751563392591661339321041667260.9102329522532997791967528732655638468045668247650320116224114705207987248257276765954175435815311222155414632', '-8.1055228012775943922611598191168220209920422725184693447914123413517420845343602166955572458315096659666380761391540010074112348208219442252225691108563375371596959919631253585651006293756277244412869691262300395317383824508608721200136269451608738907028690741099725299413451305512e+129');
    T('-302467810214374958256067.032373830', '5575069632929333955431348.9768', '-1.686279103664794846091850650203678409366941699204314045597144e+48');
    T('82697078376296338907581778590825184109578442102991030323732250149625618801404609650700917759037258851030792.4579476961905660977126995470583812166076', '209841367832573904068305526440706142.946', '1.73532680422395935522579385025838046972848188875755297012803632653935308780712618867460319171676636851490602740492068944818297966533205669240452066743937774780200047624598512303547899896e+142');
    T('-8610723620694674804743328843770809170499375.7454758671', '-390208977887824714144', '3.3599816629058183220228074369266852148945886756309325082365153093080342624e+63');
    T('0.000000000001665736409252239537708350662977465625669000790963980', '-0.00000000005397325311245537834096132969282540541809141350200755301957235584804115716973637524555508536156078477308323210', '-8.9905212835203683504110846121534305137961327547168950515670333480752564275581116048493890275734599276350080922308328478982195507458726674696580816210133079758e-23');
    T('-55009.833387304033632191465037502050438168517958786455204058304623268020750747185676992712088', '-29561521305527851502676865044574530904951941035976693977553088501233865440936246487818963545187685243602080828543148264209993996454896778956464642639', '1.626174361692325529945956942756425331503823214619006101333940515950960317910361711714132411783105515670069490701197466001427788281114415821163283764678271096259297038980050657582474248326862791843561305280964954948600547092666115456335520232e+153');
    T('0.0000005510346818658465664306888534549271604193320861182474130165534720766271492178988616151168719125', '-0.01183566974341274034927329940717360016909321111795523305608990101374740733837917774832376107936390300470266447548265172072145593380615098624169574', '-6.52186451173066523843915450437892584263602058413986588969423688481780355800652939405885567360640536676093311565481528848197657456939614662844329592819951096602632005849951491911991775291110362116460575276787590328868726322084826837690275e-9');
    T('-0.000001849521254273283959945614449797', '-0.0000000015253958577166611145202508909209879321211559950068605128152947738844883802079744653882403', '2.8212520600273908617019782505256127608067239079286482382697098430394398747360047423507203058647464997472737712785222191e-15');
    T('-309.68', '-96055.1516378444925', '29746359.3592076824374');
    T('-168028626126473481129367789758521978889200869466491962302048311336890634520017.5719333163304589587068817821858799825706754229048601236351696679905637', '0.00000000000000040505144173091841345427692344370044084035179023758893212083311438716720662223829934353555898735535197317508971014099022876688542096545250', '-6.806023726459354860300059210154347949752163001995863383108662941548990570394989515646898112331550406123056248481183044010335252896104139538361899382958674774865348411475636358567387218161462977943906814852515597246369885881277132093738749070553676123096546436403361124299691370057425e+61');
    T('0.0000041405315442707777667317116430726678375268137201769082592971414397218413001195015562151372405002624925536507988661290804269379616336095616', '0.00000000020534154280912678549286088770865816', '8.50223135350417750807611605428001794994175049772882453947533102952716919656365787338607257071961225914246011148972124658358852928539815938659942645226103867281485281862656e-16');
    T('1043304625.7986054483292567142119628829071667318336', '-0.00000000000000000003922920237128071855393930461155089778347745292989380337114027290711582558804238092936477', '-4.0928008300346795587963967318969522795914555559316007448013653233486980241052075387119842284031574844035339709938086685807849050185342272e-11');
    T('0.0000000000577244850492733199753691968434531131534014494738041562671682385265947122048380667867064559302522087090151313923160444265178252188520771169749', '-2993829090737873263916664.0234418356069276036115961287838007488804067542802424350078567987894997758950299', '-172817242588377.90289435436410530982271580018756182908074690988652090894327112554557513833605455776187642341828319053607251432332962573366023560105756765820651824786361583489449624159271997847165480283033635286456942690484125635118492744583304951');
    T('-0.000000000019733986238945494869277217117479256444529951600715483268803452518656509029860', '0.00000000000000164627700103694856499141527646022342482582124935390975524916641794007931969687515151257818754024349160456008195507607', '-3.24876076839556011689439563662251683467519484990438085916787388341510942412905771246984808263084148148601145015559577022588882488930808591955187411190280597100157082627777854794533428182014502e-26');
    T('-11427925335330818004816066050106833418219402199072704112963107004970597981437954806793580004070019399256039487816876806901413994111491.92397663856039', '-10254398713648367399130671587602705.557162008583916929614232815559112128534704736523', '1.1718650285828592780495104592683632759974828335670562590852557778855769937220071548729686706912740577068594561471384906191535657977013468872849927726389729680735882934586903957393565745867332347237612282016833288027776053697412397e+167');
    T('194053275594004162536549158604917941716280933587153407126108810987941717.1', '29178', '5.6620864752818534544914313497742957033976450802059621131256028870061634215438e+75');
    T('2.93518376745944436515386335020372331712985006976884316865242', '-966659011212373171674223610192618777699658.9609647969094449683040295199948844301663048672586994151702046353417857765604445661', '-2.837321838378954758771215418938164310917942593004475873952183940640231881169627087395424110407746819193011896535794052106353148150659770269806093135838159918310696520280932274648614962e+42');
    T('-0.00000000000000000868510085783592320951067786028338014000842721104291268902134885144965679501434098211679237543640508019493070439633157498534035049527', '1.66186927561028596926036092509335854186047127558602856573976536260498222833648503213450552687521363067', '-1.44335022712140589682935718532287891186909515934902995812983197529942616459692310661388930461065190889143976230808707540505066234871399330617979826977854860688501351020336858399810037738138869688388995164376763031097320446355893619309e-17');
    T('-1181859428272891621990692513.97972526073548792078905838349464918869621234377765379776830944783886329774493550743931156263730', '2.9941', '-3.53860531419186480540233245610669540316812438363451970602132913587532937850467323589809531777434059977811140282404274969233993e+27');
    T('0.0407860495', '-0.00000000000000000067006193014429870550943276612546055857881417429492755101247850388305957467261270841516039308586250108088434662298612276060', '-2.73291790509309091456936475161149575524931645640945426945087233770604100240461282197497983428394457192787524651402698407269082497e-20');
    T('3534937774831542866352533130845566313027721824245809049775432305752915295270444679.5880933827075988', '-2.2480869970190', '-7.9468476468700692010170378639263153729049188935676493712057455085625903839492076731999624985988712896442479772e+81');
    T('-2313111414785056499431180531976979973.274', '7671974887242891859494.557233282222057816576248', '-1.7746132685625829899267375266227186594583558197192135674091228488559599633565223195952e+58');
    T('-0.049506264282575986354429257789413492175083859994025520010200438659280593549597224675611141989487638448856569022072780436586696496251665748576110537033', '-0.000000000000000002289376387781964560762840433567145550', '1.1333847249582310289779818259083630345427337255960896766203043455335894797786980608848419608105331975026702045607659909764116027289661369664795559011503382603816675539847618267537615315e-19');
    T('-0.00000000875646084168554507977167823489990902563799552509732315823818127203671896010029263145066030926978731172618906161505494', '-12149799422754916719269147530161643795542307713126182634497110622412638327814575847624536131684703485487672095750214266758123266193160', '1.063892428796870678069788900402015365706552939454485642953799687841866869693207537469912665183234951380917914005709858797891441500773219383040079223313752838870891947612334390210947680627998406712051291289215921823307231964371826198854050237980522104e+125');
    T('72132913784335037210708416954845481013.4936052396', '0.000000034758907396517145945375764660651269929701596542982929267988814344071126980935379146949580197412941449868688705948558', '2.5072612704706567177369364733761303227157324401681208246105574307760108529931727084685203750541786884736657231851778565267996322876935570476425834209668059168644968e+30');
    T('319.66733043418522086907604142851830292549270071301298537533804754923073658110499293992079580', '-151.725', '-48501.525710126752636360562385741939511370380015681895206073165264407033507768155053809482742755');
    T('0.00000000000000000062968826433918348928022932804766807886926653', '-0.000000000000000030428038130246925313880152590782914692618238491169604901884578172392510096079664807819', '-1.916017851748168043720386562747740471345349440563638610201143130913848507924678149083014143799108415035645952111410793293793899807e-35');
    T('-1180774887.3', '-1449.996894078842975987074068709452557439051769626362706935102826112305930', '1712119919191.295852285533989737156299954793129899433988390218967916669531638871689');
    T('3647164297747831538659156395004173038414109110759206277814463425856379161036312539485631.38948348284728053244339540610363604645614853959905', '-44392.7', '-1.61907470520630161146234332096501752442406021521200216529134130525014483182136711671623788583923408994470492599919244536883919513865273858746935e+92');
    T('17498312279239.00932701739727337804928343115988922324826729272279616797560451780334324648186406856', '-165176662774787477578875.101155425022270058043785237734912330043953397824558634005249868631913924077857553397476499801921399000', '-2.89031282647578469354264890469746211626219999574969073812160564852073608013566136037691524827633370435651099998507837807111037742964559522767880633090381980310349646177225049281653222840024159118865374106303342746711544e+36');
    T('-0.000000000000000000924379194701874135396133614362003093078303247619775269472734328426147105167965036903810455485324946', '-0.072771076651632335504153815223990944', '6.7268069232824253584377498803810917142357774490329000970581887327408582544296355494518772848722889546489345140622499161796902801289024e-20');
    T('-120631710673724347689914070602312.9151429', '-0.00002802807358811', '3.381074463822740563254319054484144416394255156390919e+27');
    T('512196210721455510182901457014276459874633282215822699880063790892256101411937345231821904839765586258', '0.0000000000000003806505766975721', '1.949677829934332018345035402585815281370286067595371424284055069028866856886403261381419605884609510991608281217242018e+86');
    T('-12004236887475073738586111006769029393080783935378309149099727.68396298775060968922', '-463794927584484417474190358889847435118899635706815444621012022509301590070944948658314884349', '5.56750417793349844330521891159840232636935946658560169085150684088850605724977314027337726117947637117197173970007342920606127105666878224551145677898907304818177310313201778e+153');
    T('6584641990290656838269978401214918847381615636.518803901787', '-10753941414172447057853743167450817.50583947377863062378420539493289850486515772906013498800287501534', '-7.081085419688558260857966029738760695277290042994837834892489736689162992602408154398193125842351999687246222800816463918193016012785992028984897208947841258e+79');
    T('-24326422199657762368640227996078927157504200019578797624814562110060901133357274750788.36441683868312318843748', '-0.0000000000006902888661514497196311777408365084768865919183378538264433157093409003947610911', '1.6792258397723212195313963158909185623783542066080048568383342089993434418422010559055047531507911817998642508826398204207544200195899037683811355076482082358044434397074053955290508934428e+73');
    T('21501938889227442926984999198582213780754593552865516434.28095561', '1275573970012590600214127925446975918013722828276558737918022879.290868107247330446595401054145784513619143055092036112746185085177017939280940668661320', '2.74273135518999619237055481685367562157141441273837681617196746425111391922129347808060414256695769199642463364622072974665066305632558241309767154727027823910405233929660720731520333952798686255808870704290440052e+118');
    T('-71541409973383879506417184487015218354.023574350361463350873760009451721040985761853384121351993525', '5992111795695111627085788865.512914195836892777247907010', '-4.2868412658217344634387154093342340829086139811418993420175349903462119580242132265078555221121329803402221988433584258370336736020674652911879732211025e+65');
    T('34142255447580822559151401211062905510', '-263926755678947832441215581540094557891886476037097583650654682043897799149499355822112508751738221779921998856296667571952006518196822431846845', '-9.01105471184188942905690908904627239542616349082031787011252961375696523930047374090057038095264861536129494798476922062541028503074147788683121044719759772706042769498406102661595e+180');
    T('291910578907226789405906209195597276379329314216442687970828.12263463390116587613', '905766156845279032080418309.2899374863460', '2.6440272319927938125314442437268631851150779004590992519957935351389114318457620877828712701683102714405409392300232098e+86');
    T('0.1447018135289410248822059919088140128370855413338568649453428485887', '-31531043177144606534540520126449707592.441165834043265865498607709043737814229488899678036083217728937497668745235164920031109748', '-4.5625991301921670224436171827297160639926118138546258979023381434676064727971907159899790572618884224647144712250542121029109924335769933935852618344978968520165094454255764789533981971966126476e+36');
    T('-431470847375352240973824243957438268864164144677055441855.1122835808', '-283926397264833307842262874592214804147456.35181960193341', '1.22505963220088520133492369378146202611227577382619697690196693448533411156116335413333343159557018405513846376263877954528e+98');
    T('43.645259992317026943608411', '0.0000000000000000000343476666438615307467230521728116897722871849771845212242115930032028373012984891992888161271678789193222138335354663458270965038', '1.4991128408007717160640135031205392680494419814377001223461964538550601533847575845289929527748139763281076549960076386814116184051973077956967114943734618e-18');
    T('-5333642642837544.62687709591677124286121489157762662337705954241856563514693012115', '0.000001092403520420007654847736885', '-5826489999.69800727677170880845575478173936457495066350494224786180067840981405695080871119365234031637361775');
    T('-72389949638529267084220162178469797619868037471866853897782590886837793391170450755475695445589382444913313865539107.825554816619359474078198937945329', '670516724101646254739075469170811171513447894797271200380414002599176331305558057030387883188608917', '-4.8538671889509795604939125299480164673783805114262471410372408819538068738391487961206173796503249288962752085213903270623213542356008521417558442327238499890085384270939054062390701273409174804755549787657232352912465878135282338593044925707898693e+214');
    T('0.00000000000093389590550371354609879546798604119093458584514838', '-436244725.94251597789915707849842497816062786670486162986', '-0.0004074071633553053150284952158066178287741661729136343492697908838085289189761762237068783682124527003386268');
    T('-210380536819319.64260798498975775170296381658684501981152412255070635544143685331980809905', '0.00030621239427093550265058736827171066633126148745977590497591435654761612693434536026517812303379', '-64421127887.4485697052582576831125251079439300578331516148925455276348682374579663836142871112701981303186673805005030452875519450671912320654524554714586680026745270896999533988168995');
    T('-2986425419210684489372385430630802680.1700076164378161986', '-0.000000000000000000174383713906052937878565432107027682509555072891773406016182697852642942595757484268776430171027151127581004748120018091764749919908464469126', '520783955905400215.3593326533209447186843411978634958052248208818590205254321998670705004791818240022569816115585565216755711284287526937950905747371255207376455031602089558157107952374519123844236');
    T('1005275502281605.7203063810011538411846251379520165504332670989327995886451', '-0.000000000000004001673036573006142766297120381368448210468914793665305672717', '-4.0227838718076871277132373250062298425694133513895898591969064598434150270288761898553450928567831052706301853626314400639691300657367');
    T('-592443055123445281554039884803217796883007011697299809752867837', '-128767025696067370042375219001771824840953470096164458063850.3083988758376157095926583752', '7.62871301025373359356475793746178927840450521702002697787427513491791642079585760593083059843100862890831979506992101025525289259825474551988167584424e+121');
    T('440119636500787798119778.1614067114799921393251', '-0.00000000000000000008016651264632167527854789402319094526310291781550499853504152970195676032634610434380960951927984204641071043406830501', '-35282.856405434903813597479779632705556601606542704671605957970152900017367125637334582859462287579739167125492423421901400646478981405910526064055108265969922348751');
    T('-71579.2', '0.00000000000000993906325920139009275587686844510659032911932338073755273459775736935248889756221559252800727756986462003224559457716', '-7.11430196843028141727391461541805973650486097872134489434700319794292355673296385342340680738522628853610212113863357451072e-10');
    T('-0.0000000000000000028252704404880797079738', '205661433462063.7105307433432819704421946280137966217786600779', '-0.00058104916870877463514486303435785934208231367367250497766551677412065596208835915902');
    T('31511547820580.4394382615198628132988414617215767834965945746200468417564092109397320304169501963915067332074901099286788653074716563043835', '39899961813398664937157171453593.90023813309746209966498824019924844847011387015958705088743565647183663555197208597527', '1.257309554722245458313010717601632793452987822633399487410911056875703458652204694414418020975982056391949850738256638817586034701650856414371301609334505840392336061143829107090319885694135741359164686790967904196966939338727755954779566674337378573596045e+45');
    T('-495789074385555943868529017377472010212654135435451505422731855063537551062677555928072920281877990746962752595360995056758586999317669647', '17819185082774026694204023148577882778297684550452324826832075039839807470403992920788019844375217029982704245209320846148492.201', '-8.834557278493440769527646032308030350371362475818934570787903683344802633346230326610094249834365327683855191264428319975098552784711089348596608431374093713258545096383387667958040878506159831317309788540734024264316478718947843168246822604530057437227626073923047e+261');
    T('-12262785235810622775007408988251878393679277845775461370985491116596241792393397087453508079734404478228220643516598880369382712979645406460335974', '-17339.4599485985180854570917062071088513028304627905849022429965380282572583049408147112553558507464317495273478217946882351121108297352639325', '2.12630073454603527661886448347291489452541354119893214674919284918523671819041714914484722504179832434598604666638655329939148126697219715586452788601752336764716691015791681243794848208962144155133293203564192719333981843088873686066209217435640467778541139472308985673249979514457755e+149');
    T('-114396868590854104192672352367580744901653263217987528344911658496559573111650716561664806136683756515973087947398968856550342371023724981841598828474', '-0.000000000905511055947334', '1.03587629274772706348234847002960858371371655011375374225792356944117914008860239779411591084427150020831085014159874092768298943748156678048631480006184964643588316e+140');
    T('-1112241862840106360548102452612787954701297842442745550953310708233414570014516673446141946344195030870599027775528597.64281848005244542515', '-6348759644967567183872592130062154839790.5635484413196596709331464307', '7.061356254242819213528914158861347824577247501609657702110667409707299733668207944491015232613785616142229000235550257974209644089293373052607176525740858805900274429133922740980503476595397297785266512105e+156');
    T('1108564966618799.684583511467562033896416379326883221554515730782746395008239020932004268892597106459172116648365972554761', '-980241488498709.989691822562361049071', '-1.086661372975934954665249154869827138993085763955003193792673025483663764315894189566505244862737823287988968852517424536981368134752599081608376589955677031e+30');
    T('0.00000000000000000002838698369819955658555277373271853993941828559300730453309757421421453', '43513833879.28', '1.2352264929772849320906997234371440706324351943721359734528135469244723396680419384e-9');
    T('-8511924985976613630478857403041973777452.09177107175699163666864323186579824676938621436941711058306956193771462975562483458844', '0.00000000000000002272278230386744849803445244424', '-1.934146184431965759189809035913423340545514889094246651943997790659345002151610321611910790026540789359818166350027713628491983499298272486356031374524485856e+23');
    T('-0.0000000000000000004316812723968992118505000444628085260048886312014882985221720513162602276841485038550245898508488835', '-76290177257460517.999473', '0.0329330407898855391252124225768438464877945578562524623859269076191844422317697448247001533505237464666728778267802556383955');
    T('-315381216106477443798809516461.945916162611156190552321997295114376958046547275354056014333919065425673748199256144380585548066158378662409475843035', '3589.161838511444431440733729960263707637958181400868325358533381567057455464602280029192715692411789833427', '-1.131954225432699752649141397708477473892667834397120107511962996304791377678417665209758473239521558381343101328776946782981992193865294194693770979873300748942276223781245879606796533776372414124841041914736508348886070269056497656762529089763048130945e+33');
    T('-429554289753358320963153203206734665772198736780.45185520786347258238318355302967532112035290', '0', '0');
    T('1463451471384605194456038106049436218092078691173664424376798273282402411629304', '5.34145410132810384872275111495210652002710418898122721083801583300401806143614616798518502693357282268142295737452661760061198730650979349477', '7.81695886392194762437336924065337257968714608565726584447407244448623703884415455226257285276362586454773057274509044560620808790280438685449822124417639104621469226432169534908781180559594976229125827018100297590274008e+78');
    T('473586353851131907618376944471474603970238923713969727224418527992189433742423567976837249481727621512554255639.8', '1286427255593.352272327514643909503984179320306560931658975694284708', '6.092343934711738377612937734307383870923174240356564095828149119059002227331699554845917346771273434048376444070758413332992760270463202297025312107690635975666422869160358961784e+122');
    T('-4048155812027991860998261874719005077642504207127030646327029367933424762620898655788666633040116830297381192885448334', '106705036497734766140329279268252590446.2335278949', '-4.319586136709639909348938311306692360922506424385494354567553905770000978932851239614330898183516223949738019390821277918151726982015490940436949045524065702817320966e+155');
    T('0.0000000000000000000915543732852883697633526279049753860806942872601198742215774747341', '-34152105784317312126489961356698852045721114680628002017655109882376030549039796836.83153535701271122760014588481469402075074299376935', '-3.126774641456043327979536556784366250157747125357071312778820430770070528036829464402201655884092881178626009467738410063014596140203388175163040146059581509034963217801746892436203877630743347979835e+63');
    T('0.000000606056178575650297523536090137837690540483833323054730991196951139940145168062878729776556286621722228058423687246723575421211', '-0.2110854059592627472281909321180610299910818145371006157746105830582652226002916315944873681237089216185278265414045060813822588197350141208020650990498', '-1.279296144887605810262234886377441922843377685091005337867319233708631851469414819167991307615079977909703057135546912192644428529187862612840717830395670958420462971297341460134768276466347220568187569616383072997479301070810241184256490139848082355340819140700544014708653078e-7');
    T('-0.0000000174023009167704116970725301110863248980544485359112331870888076484651233171728545', '812850723207869191281548186903956096.809765707322509394052629000163', '-1.41454728856777941909802731675169558153278554336163713719217823280456330004090313022045897019660317794534957404145735837838652198044782601796752835e+28');
    T('-2130698.0548286272900651895094179021589945303831135867320568', '-9165947089300259943733983387964594800521219964489263104.59571046797687349727612030295852399844295170184636', '1.9529865633834181981295933885978264475195798583990822467641671177002901506455322929500790695192903123945610282587760480932085484711958893867394353314006055160393248e+61');
    T('-326083155399690502439723947025744049762670117683.432074400669883911818553035189833194671636060400687941284537881087120570924444', '-0.000003445571861995590614007298331872337442294157873364826604749208387112491508', '1.123542944915909132217079826299777527201239718962104484441084589157856017183525173415387994466036390860102540946603848752279383640837621156142175767860780613378089921458710577851612660512939659621552e+42');
    T('-23.2079521420353162165629346282760720426054683804749740406', '0.0000000000000000100667680781319844905727827890837263875737788345658863306751556891847753254997696006873526462699281042889508598121471367829087954284592931196522647093', '-2.3362907178225593297823553952359877724440294726489222805197185879544455403490267478912169784494515330509275220029014923616029924211711606761207039492261868123451812767302600437565610895313205736991700539758e-16');
    T('2285581244840240639260487167234266.6210924312942573974763401176849472', '-0.000000000000000000918107', '-2098408139956538.8125895280916479508246913088182767764248101964273738189504');
    T('0.01199215867887013759356474044', '59219069282514349738400251796939074988014798278062054247507850372485277499304966991398291720701282976974931657695.89320830399601932636241718811893409', '7.101644756509164313135142859363774349015749437363131851507359602111507670252881439883943580721027909131341909904887723338723647408378876006443896747938535856090800949803175996e+110');
    T('10418927567085948892137885878723808945364687260625931932.039351', '599680853267121291448.38087680537808199', '6.24803137355843394317878805129936255762113268577294303839429005376139089696541797413362353758438849e+75');
    T('50831867721704613054022871310081961685717811744979380239875687986166840752637964249209739.95742211098733326424751917109066551862656', '-1632931078636285445774108746380428401452629936953024735101', '-8.300493658790009540471378875888368284941161669991237260884060249140601673097481745352940033672506184736998843139872004445042382050600280557809637308250946613651294900726384303212534288256e+145');
    T('7100289077488660412990.83', '0.842207839712975032430755755635', '5.97991912528935706872007298550620521929833614697582705e+21');
    T('71.01883', '1761970243586301256729933.7274571136169644922999518937631730701127745550670073949565808130784043676096', '1.25133065194314119280489519301543084253886394686592551344848526917216954629436791164270145276976454523688768e+26');
    T('39875156712019325311551200466142836628229123594325201379832637310133453471608904683559565236344258.967154811777952228563', '-0.00000000000000000070081075140934755506891264428157216975205610555238498806', '-2.794493853791575200615635819282954125975165218925920550666607458660524754995294291334846227959636949211292863866638640564674087684191891139999119222389967229521626102314595778e+79');
    T('5797150130947834056182621078494110956594842841052090075870166233975375056203178648058074850906399043556974734623', '5994943195099.1328888761373637634842940850229730116836324780692494703604483', '3.47535857284937649148307887121857419057110304246614305622459943600636593236940298049819575939494986921577262671435104495090825963819204860688181879997983293665235062953417708960478114909e+124');
    T('264016746346020173260674845542.56087744794671212', '44725565833116388541385131784578311136952.11515649206925514017678106974916075594822233327402018434035090', '1.1808298369744115980724840684122092341038122639077601880904123414433464879079693475998082113642766387031979298492866976697783365167381515796702082908e+70');
    T('-1204258217830.1047572987899835756536353', '3745937592881714229717990115898213651381593777161096717540301831660255825874475111361207039098321786912324541172077464946165364.14390772997939018900027', '-4.511076129706525686579204142051251397641532749272813862145610194389074922134315457597697830334597270566875350716009107169694204547970082327051675594936881425342543316554216670569498181531e+138');
    T('0.00001085056051858810623505714593197033408403035596225606886953365', '-392929408749548979689032253667965895311572351400702674561774493496364595721866845891578496858294676481040176.798243172529157', '-4.26350432917002414454392491493274497473851341095485596409214868708449409899775190195407623094650897889780302985371766392684564356442617678251382828651108441963579450892957153361763305e+102');
    T('5029414448475920188151028145401135310150737258101072206698406572639528330082067.9336479422747818663155268690086170021619688524908875089697158589347416', '-0.0005229224526200471446836303862066185994349069612658407253355478392257635933309390353751513396483385196163571877433495709051459895367873144796893', '-2.62999373863972991595232699135390284144005436661268919696790615365070651823630721931258425392749116101462135216825040904659228944045018325148257573243654477716244945479320745317393867583646412220760107366403103406898337360145950155726573614185125610071875889949474494080865196631936734378488e+75');
    T('-0.0000000000149459394556759606943604866160845855165524674884428420450005789266873424201139870021367301867234201955279838923502786908774569432473681039899', '0.000000000000072681121445777643724015525', '-1.0862876406992243098110834770746303342214020691767313742628853277726465868820586019765285955443439023247300317509304826500549194884609623643817234580349404842020431975e-24');
    T('-424139.3397723', '24.9352665519292948435181504860531561629245676859909123534754923791320492627466971516264644236625241080447', '-10576027.49238160664772089118930618199418545494982358874950081130097372753242050309994330560576035740865221100909622181');
    T('-6547933974356063515673579783328852017212683547729632232078003886270959230998276343908132763800852667899598581090407618744044143894167068562238.97', '-11922174512798027988.715520057164002099095961564815407735892440769625640988450493600253780499938690761112753154279778415248891368390538551750986', '7.806561154055215663901405645925227739464677757390641017715015265801022422067920560519544881469304761133200469523238148889436630472098386093418009954492665836535393992406767441416546896285667079520157050657599018677219464362114136929451396800416962537931259941624014093037081081406512442e+160');
    T('116971949153.5479374797237476837460980054925', '-7392852461118226343653879737274414841079276534234761286995956308826140702012532258763158080532', '-8.6475636218160290233694135116834390249179472139880806248404813419843222333387482382624808997980446329708149926437827055618140989332201e+104');
    T('0.094736370343830975556009350140982661429417188805833955451765939740836399749318571449220541572349173464405792553494108387767327206545387797', '-14063088472465823296636520114.7840889695829209677553679799383909290850887704576208141405158860265325506024127861905793354266036', '-1.3322859577055824770031704859177438035519692243531107997889353707418395463470343576295679587059797728931003195157525260292910231399354545892908442725681827168435407603526548235032165419174525789568586355685792534406527613767949403942954190037378443595809925962692e+27');
    T('0.0000000060238848102401094509085687708934429831403505207577935573', '-26397976144679538690.6353645112721463552724006844', '-159018367519.01583899352216272937345079686163173486566881281209888168576630569925947787199801314243061612');
    T('-3054832823620065247866462699675075655044.24965543051115834991093974', '-2755412779439299724.2984036924450889829105682176654094195240144615415006313170056714410711837165723063574799946207221645', '8.41732540125336804197906254683195597019539860395893943850239114957397624723405493043107443806607146233611284249167142426017200712427976573775245903141353510518411863114852845764186723e+57');
    T('-2450974019399956698469496563794.9748364333942615279874848551026505204635039431974104398444982010445818729382963397261226755940436258158321945719', '-997.24040302293', '2.444210318905143470429111243581921471467515949023475365592695716839122688195540502841004451264728108191685433698308378621706762322759533121835213000697233667e+33');
    T('-1593.003418430705083401260653839458373', '81482817738806321636670141568925456324894595.35515', '-1.2980240720128456541718518746184368534776478125249076285377131164060683232627757617095e+47');
    T('98261907174900314009700603815243346086200.8875197975215499156', '128758244401452895992559910962986981947835519218827223291426174867182696533638857.4611504455387701659979843027781766', '1.265203065937869250805348094516614652285677394459393456309889824224472248261724047240316582653646970392779473698285791164886759948690959417233989768983784053494623305125189496e+121');
    T('346496136331578345.6139565562', '-770.118198136243584727', '-266842980172845321727.9109640073233064983318371574');
    T('8094462815401859042799401705641425818247198451174542181431184727369479972319.283362207236223116403463114800998274436638818181016', '-44716739326169.50679740', '-3.619579837016970552211688799002785780552856029741836983454305219747190234820032824275611045529918418220339456506414179332920358828891256009156381584e+89');
    T('2729467653762704.3220008736708120084460677187730093661396089914034', '-0.000000000055233920094409680756264812311791211428686742640081962817736283081966745025774163718774682975', '-150759.198288205079331338550122412955542882826158428646159812354506434469254169683873619612365382943072278940175544745711284510129905967399982740638335337115');
    T('1', '98776520222261745669591551900242713389385936698028187815152403272299821654553681210659893142881686053028403096887300880878591448.26475', '9.877652022226174566959155190024271338938593669802818781515240327229982165455368121065989314288168605302840309688730088087859144826475e+127');
    T('0.0000000000000000576769268158760793228889322171711548712267346092999553202187549995149774466310618281592827181436884024', '7214897443070769737.590335358131669664171271', '416.1331118080442374873530552327247807419822048775263491897727868156732966268031408562709186237110762022701532092252726416209067252838924499674504');
    T('2563816381746605865113037452511580729789619725008213300126058517120912912629866105171095179961598', '-155142400902758369245790224073470757889134578664867029684200360213407983500899972118045933740420731880423071611781.6961697019774959718821', '-3.977566289379913215989211867005456529599318466838788807437369058021539398823672950585044348343474526479765303904605036245529631372734211590112787398817987454064611577927370338439226841015272470235504418419674575665836030379657835958e+209');
    T('21021355066083990352764111626531024748686292365672970531361459847295.43100373125999514720142962920161062', '-8692703317497385289852042779299.702257622199418950085745', '-1.827324029212387699226954866895393730056663738212949267989848701641289722199007782369745673635690581526275165825369689624008225949130316222067517051411026119e+98');
    T('-281371829459700411922161394173847749045', '4262.059148155501', '-1.199223379781965638825029107725389720131878012484246545e+42');
    T('-0.000000000002953739179434260253787432770866951567504749973', '309547007791848826075373946352866230269769314074366092085025869.7201225815415300560895', '-9.143211247914261165868175330266593984659858074863087669862336974871905611291966919042811817692587876933585012091675114363636105835e+50');
    T('-0.00000000000239144278978796819150765880370364103148388159945376101153092134918047349666341937807866834', '0.00000000000017790542655638507387717413464523193083130128517052867317685503104374366725848235452275428161059560313199945661107215', '-4.25450649602420004053833718175053649747723590806183497288310789806254330950196751041980603311711723907162265791776589017621131644531690317327724762991500539399348391156280420594867129685071798617771660731e-25');
    T('-418172866061503521911971552233519656327026674166049022680423475766548238660534576', '-90576902506171370108917411667870811129738963466006966688282481517335876800502911324072307434897211522733182430116410086632830502124065921343765439120', '3.787680291997906303359954806060523264425173800836418334594593478681399250024197130234921086970287168083653907490426018256804158591064373885502948936834304787910451280809462727218951755689222074507604311322831439358572513258301312e+229');
    T('-3430311267169742574955929826622072681758112689207488355884200258692448298952.3897753652042205852531267397500939984515487', '29178195104748190876234156211954739009200852852565396294', '-1.000902914234947463302590931129086303752468365453949881103219746142108956587996112362845161460507830640398391094393578265988318777026054838820693518977773210730880397735405178e+131');
    T('1217229941749445.9572811858261781662798606', '-7688.762839498170', '-9358992343247662165.263684546710966628782232475881555102');
    T('0.00000000000000074169408783659430746120211730010355927129635815598173767929633977753817875737683684742368617111968470780652946190285273916758224772647017665583547211', '20277751314053396744396463.85165787361341700070969293684', '15039888264.2541356850348902729846895882151710461905734702387731438442631412304238135592144651640509390778276828843359389845898882091777630101738827628755435200638106447703368100930825028533561255938115324');
    T('564381466888810169919601150245392062064224480660735222387818244250114897197669997801703266070021836470489338410440', '-0.0000000000000000002645655440014235375760091744435849282689052519875067111364162666736597818970677355465081835', '-1.4931588981175946834283111355443773198028167610654600849890869545798597157736617913162581558616543097793344871513878994482479287199763866058458885636014581139878719012494628251898131796309084420634183574e+95');
    T('0.0002901387540443867013405807910333533336446690397588380128405647147868703703433303126537291208249472115', '1532908562403392492424723937748.472475940195350', '4.44756180359692297666882276804364242887051626129128210500015334075754912171335276467442794092825498742382966447793841999956236194956994747766525e+26');
    T('-0.000000000225264190604268810784537320407445', '14950.700814220268672859713833679549117660443531292433032865528784491376833526537556454874844228048068717928075898239', '-0.000003367857517881911505578414004206612694308367570237761083545225919631139495505710310659653325677787421658102687518302736419664735602327222663837989355');
    T('-0.00000000000000000132422391255571832661986637', '-8685.9441213328949678022978068', '1.1502134908591787160769054777064094203031244635960077316e-14');
    T('0.00000000009742067142157691787222596751006921098920279989592268344573189495144592', '-118923723014528400203978798877563012809910061320326.53737153377573592386', '-1.15856289440289961067833399896083071996865135302176383556314783677919776746205807113913241561786412522310269565833727285343010237305940276512e+40');
    T('-0.00005563546373241014529561050614463373299027558680685062277259553861178444745012994288955540119513731389', '43565560449530398941183073235722526166955055524605404664121285806765679773941818978977694475240560247441941103738049224883.328637095', '-2.42379015837197033639269209261239035396339708689516443560833885654548577622441367344082188020100430582832598711242044837458341177912171781239207907145798643930933786640347757166079449567939368292304555736474640815549897983291274955e+117');
    T('-0.00000000000427087426712799927089415837629262439038369512842420608188380156879105597329387384339607914876727209877963249157620642500', '925233928141580102390081693311444.71', '-3.95155777477363086022568547662201291712827700290280460065453355510078335629837128123107501594590100763786464243066847452995303895192418602700657793426175e+21');
    T('-0.000000000005091071907525480076989317935464438083994221617598954017569512203716892574912245558148612628718382099960321042186876280738556548120017521', '-29849639725760264066839307790055758277806690063171510919436665089162295348507536342284533528662967910860808922458810578140340107788205629605', '1.51966662257574655588783076840748333702840299074683399397337485352697121303070516861952435606970784590094155132031721724526937976956111602321500551446919936325059565949482713352248442850374534543475972410515760725383665948678467615176539411200064096625744148848636243436309205e+128');
    T('-2532623712961933744380270221514261860541083273159263155049.929201610', '-0.000000000000000000012646589646423131239682610662957792721023875661746627573361000945171', '3.202905282663009949513269540690749564425465035038152243969308063023239476727318839202045865432868677151160472201684195600784341492531e+37');
    T('0.0000000113044864730', '0.0000000000000601669081479195180359444266582823155394447964452777', '6.801559992803896747300132989382930294307713993458802443785521e-22');
    T('33142360002065196584755139040522165.652289540188139925590361076693810562392719569608216426237486549208210658659065', '131866276226694967054888110190126225070936217278.208055190299743946923315513586077993', '4.370359598836895991628989736519636562344422619159613116474448702113850541696634666515624356906681122560009861221133939297280288166624498899890285407603655628297984634754638006796334523457886456545e+81');
    T('-28271980269206475098440832341.0', '0.00000000000000001997128856200158301984619314093468302232590147010149380231144706644261177500747535724053029495', '-564627876175.53771205568655371502005478701323612123758214517745046808228380387390782642610030197268449800392442510422897795');
    T('-613824085218213402416390718799790910276979950485725176851655692450022488003571999524663227579006467246542215327798466491629903216135', '144351', '-8.8606120525334322852208416649468617689392332832564915003713350860853196165803621703384661564257162553505615324783036436533268159152303385e+136');
    T('-2.6', '83102555490369.945045038594374940850', '-216066644274961.85711710034537484621');
    T('-19356482713244897360632880493785297953636172550378939890381044618408211766861486580267956414189437617939.34723659706434', '2096663623111671700117269467244749170.87006096167689688793276499897977784848766593786643930915441409', '-4.05840331762504879192862674091277246812216284989188033185403840621110675874956150938505763166302102413483441063564098934208044556598498878291756337336898624598153836135376440509322121089075137894784969112914467325506e+139');
    T('-0.0000000000961833377275620802044513471680247526397846980939658945385489196881637745633153811677968300170851695260551929219', '-0.00000000000015523082518616051240657860119777317366287857462125986656514276221641140341472994194237', '1.4930618884608626385191111567704817106628003163415171598035214188415333416493921720574942746386069040797685559452034643996408611374739944262662458626993363351126829421325696180467755211836761710903e-23');
    T('219117317452336788720880703911762541205752470989076742619261301626536476137450009416861100269148147444076', '1703053723960030665485934204490791810931377645235473549551623.9346390876850553262664923156', '3.731685634713343871742198830618170312603304239495468980986930004166517969952154798942905166375391398101794403344384735628903779970526327721072669544702681593941605616683824986240005980347423856e+164');
    T('-229026139172552757937028186554164451744686.4623797082584117720', '142692877312662716284497225068488555041426628476953112845882495240122781240223136406046110921847532189803193193212616817958104193853628', '-3.2680398778341887238160775782128956864982778025883686830736123355786002576636730049750417595903040868869020303794371246492434867047430164188551479071257052965280634450368893979035874971520108816e+175');
    T('-1073665452168840118738785397447323896819780779969711513798521784434976119794308932309275291527935867797575529606291327909398442666759773299', '-2201739131.40829642230910815', '2.36393124008131787118729847898108542201223436609958551819420766250456830353949023025701558031680369332682327153434667887063193820677205089342493716611601619687328685e+147');
    T('75900276157586733312083235.382581622118846776984080849994704930212640859922732548092821867297114657673701742587369980172993', '-148738026482687430520904431699391318319406299324616632922', '-1.1289257285170424910960307900697967942479647538118082242484827272252207217650225144658178111388034339556050832590401486962625381452243039292709188298700811929089250125886739075546e+82');
    T('3674513449.5', '0.123520835051992626211564329410225789797398349299387483082158104725685223328601760497980995245750734368030304555231401223', '453878969.6920179367892193608523230674423001146096982086973436692999598612243583269792089846259066311598292369117788993786442487385');
    T('-0.00000000000000000003630523450169119195624332326712988718220493077008794952403238435717494340472630938510', '-63664735097223219209774981628599525664223257757965282787982134793736977272874143846.500', '2.31136313719273856014857129786830896070271022062940478374598697364961332865316791963355096598256182029643618880489361616574337962520294943300606808278625781284036378715e+63');
    T('16255476060889702058292641519175308063089604.27385206453835321058', '-179922219665363.16403587084722191165281635104378752030512', '-2.9247213345924492933598059283240736222331848885894205158919467816021766897744680020212514605436690223674872782932121696e+57');
    T('-37337860121811988659501056235845132337764643336213883612586814001890615154275803513.70825146506875697717604166850546696', '-0.00000000000000499312215357184649280143043628595160134265705278153950603622340397592808084159267174620857129590490362315100615527713253386', '1.864324965411862434361080737439842005056233440510613636437519526417257892974279749554865180460594721444483260908377047401237999183242090755697516312142757552953410741754553786322115284832812113468103953081138139580969418569870010178873112656e+68');
    T('595287249.54365530407831622115559910904721', '-1768420334499109507714731763995306162286136171277649865409003891266116172008027525', '-1.05271807696104578648763801372031670967180464094032778762503630348461739223752684712175511968739692938359619321797420445525e+90');
    T('0.002815450816912', '-0.00000000000000018862881579393125180980026941613678890399387821675560344766145135638', '-5.3107515352016691075986894697522001159488606156481160553297195685733162144309856e-19');
    T('1746219149830810782027149888420786044489923228652389093.034680474110962166857959336903', '-1668828010190656.963346733783314', '-2.914139429168972634501569834110127443990322655116665224756513968188822900984956290803407975336799390526947925836542e+69');
    T('64662722228212130661621650365.32166567378954381', '-151753087137277233145197949.129178182548659416754566676577139611814504364107493968621776', '-9.81276772083142891387388835588024421410101593988612095879269873008015700198765486880253915926871899215898047800746249703290547200656e+54');
    T('84653669316972.30754613548716676668630807374200076888152269704972414345399368365406435043236681022822051377689088018808589089790', '-891907791289670302078281451791247530469932219962285948', '-7.55032672250669035947286471317344472517407746317731670298580361066330532006715429733763760659799434857873085352566481727017961595406096340089709578771542056983935886992682102727092e+67');
    T('4345027127164155965936089847807081938532105519246471030938300717258607661737728621832645960975070377608443948838275506481065283884757.660457292004587', '-0.0031549860729161313643870355404353', '-1.37085000726457005604716494913217288695044606549706377095554606468800726719208784612404038925962557552104905114304146931613984052107356347854256094879310971318422341980712315767211e+130');
    T('-61520610530668063822131867448046406754862343293090074441673351607446445867575168630626', '35605600761.929728479163049', '-2.190478297185136890729555264108152753112278766935841211441225374439764529201743516915979466007768687467908938674e+96');
    T('0.000000000000001947822987703', '-33311862590868220285105571017136406095755495416393335802876470598016841787968672731566765403', '-6.4885611717696735160488683809368479132926370588940189388177405221668999878153400559367803107491154839309e+76');
    T('-72035384592407021674980.84729507555', '40650025408409173438512516280344375884605.60024427411502645869761254054503221638189674172185', '-2.9282402139858721209738248775767586249381088104728312874024564502752576950437939324991537671241774101883194117425137328357675e+63');
    T('13803733920957278337452274188531173497859274744002844428727547285003127728359.612974872709554753729793023049535183014700983630415660322292927376558448', '1362366267088885135733900716922.19506685185416683405604525950523770517636717536975000710349005385816611', '1.880574145378278711827375831194045930202239941319475711762021264109852456142716124505511107065080979058954130330974983743544175658555804894085980685328101923082779320611251686876404105639204387659101043361189777091188374895482149778850837141250779728e+106');
    T('99584230876132495816697493861076736200925464688036538302231243310765908574488488858315092454091884601410984087967.53071632699032523', '-125146154822705515.5242914429658322722160073394188945992631305', '-1.2462583575124528246919426619953914719378688358583931729348000794287085154257080128549535423454367290540927132043549939350598411440675286201751061031732125669579309586594569286579560542932515e+130');
    T('-0.00000000012758277143478046864136383313563417', '-16907614855016026181418628647663025463883570088670313587786762910136346.2239859700447043819351603364622', '2.157120361554808579905490037140343865957737705652239636688633884876362102934163912327493095734496347577324386761512911704717665571233374e+60');
    T('11129202925396407157427240475525317417101748679895585149981795954110715244429638119040083910255519563482482.88385201997256314', '21121732286510044978255138780723583329997392678716700.9365222644226293', '2.35068044752467336469513267242489085155640464226702784189610421069783258880761029174834312728324802606762609982209717555410045636138013876460541334152485164353688420275011580410260339339064002e+158');
    T('0.0007623108288892070002324554289862', '-85942313073979832197394484133961158027209650751540287422505752621999709082525249310360508378232261949707995297970944910223297349', '-6.55147559160812975392489905452196370592406606471681012737363795317578187614200614611166153102989166031105288845674348069177007346051710313415670312952762175838e+124');
    T('0.000000000000000044051571525227145682264900344291236843501956911752', '0.000000000017712290570674458162141325247384093633836742093', '7.80254234949672231546338647163006869638114832232612780787802882189163497793578826950951184776936e-28');
    T('2497912210234.005032121063', '-36215243145530210288660899586798506035608544499.347970495067719072170763689884', '-9.0462498049813268339455589123940089929239003754340990669581623284783162621680610224389952660876426692e+58');
    T('1502959485835975870623286636747688.63', '-18.6', '-2.7955046436549151193593131443507008518e+34');
    T('0.0000000000000002380492844605809635389684807457429719545734217366913382', '-0.0000000691300263175410058423613287987265961212556285556225346008695026090127373853991949000359920026136176879063299466975538116975010485068', '-1.645635329963176721222121671096715786436871824519931699863275362362108471140125039004626432868692100285050495434763779725307894808337972261206505074892100990010512693296587878317760379976e-23');
    T('254809590039852410310539126422956405351723766563016602332405.68422228', '-0.00065877273880782339590525760713666216419301807744760105200422', '-1.678616115050522497913933697207225162184024783104908394273110760014145206072260328917980848745450315587200941660750818259780216e+56');
    T('0.0000000000000278510475015254355758436577787533929830', '457360.858470081742985', '1.2737978994588698477266685597352939795610307887931308474255e-8');
    T('-29020220592844713119041607050693801334769100420291179736770161092227817.69', '21673233203860610976626552967575106778601581986955436993267108076405912742379179', '-6.2896200853620150098094834197552528045796701728777707177496046234367216918962571321926709133055901673078103556052889626187364066099134572001385766387651e+149');
    T('-0.078265485392684407726026843083593796846197554496715815079396449815003056610109470601087662461730977229533667789541208863921705388520252404620853855', '0.0000000000000000206254796419314596280180899887058415', '-1.6142631756326962818018348319288616709731477037044772633530435371256854543702769134066969398322149439228993616976959581068599334680519507769306737453547711855684460684428941562939825e-18');
    T('4795305568846848112838662002421291203736834738342747362288755835416190464055590424568070003158113658996255292.35950008158232661087', '-8285467911375377.9458513915656103724404947804473238339', '-3.9731350415920213266579109721401773253848863967652965582593311148492458733327726591229165001156155949294034378674742185027297365357231327284258441411135637685909564095376656361814493e+124');
    T('19117591065319548455532984279934187350039995069275770.12929477022443443469354713128796911998383266548606048053491', '-65695070326024876590276239417471386610369211243031736907459699925951944200466086828836449.27041754314379361686911712287570702272', '-1.2559314895003525759288047359281823933287108050703977637835742243565477054442990115080659989437373048015263572203486399172864054362529301399064585328306575664987751933695137203217341064876484312700608044517901164971224777248000985491231552e+141');
    T('-4449958.97652133495121566', '-252540313515710072725302657773.59471991586456171844177757978469637150589979632648440614666063', '1.1237940350627462471642735367835141315103130147959353855777341370569264164618648847956641021208508179253842679614658e+36');
    T('-1258441607483954377789927095378751802020005502122407.10214476628115698182122581797494', '80891657196033813941356884134762328483.27517852967702319172611835658583660879767525096710103201452929464', '-1.017974271138177784694027841372230917155036713620577632485672383505466603972062360612431726994977661204677518975067557466197345901755339173598338486348474216803531725823378534129113963216e+89');
    T('39808017028002860936031338217973215314567637.102901794', '-57902551664450505458473980503658853607888324171.49605292290375427619687232427940212558542266526276651604284541035424385620765024160046792819', '-2.30498576262326111905050241194114191659986825956183943894195202199609220277997857123964697896577008957095100523340998232771311496428225722320228966585150676608054417194697150023556808021417286e+90');
    T('-25293916752365259143834809987793203971606533992589538543882915368792362597533769162577303959554268549391839092976944430801.45', '-18870536.1312144', '4.7730976997543781813462001933060950776383666075435301423297088056421996528868308501593986755779476128550409285754930853295224839815378088e+128');
    T('0.6419603235395239183208947139633444062611507467701373727615510595813003', '0.000000000000000000017952483446429057318650827831163686139172587165173817840377201106607889427351733260566157112519038473019359253459211', '1.15247820816075450454750045127898262549290759007121217263958499250330256525778703472457460506113402017388245419660594286457976772844458813380078355709054799484514557844989901980643920633e-20');
    T('-7754484348993.123525069214650584107205341890439441675333976581900424068864364093826806102195', '-1752105446017682.49037253813167091733073426140377278798600289379784348182745501211856094498254897275180116436441751208660573331994338298842267191', '1.358667425892973493966955704447747036342266355969033025086821842915237172969178185006052744518082018289583598988482542984184766184862727882606439182799978323965924231683530282046591309550608146208712260753677314753286354780307441584245e+28');
    T('116204805405208024790396987614152308611917319625528164258053148681924874821595877349738878308604676525193568190095523284.4546294386699942279642', '-0.00000708667732873965881702012704870812130887887783663833386408', '-8.23505959955691471303838517278590066464473883263415152528959807184051797771275805894703979322626180999206021924871332410324136063220356505352392864892088775379405012455775331410005572958884177905936e+113');
    T('0.00000000010139065992977811728', '0.2823639034545530635547682690320968784587487', '2.8629062511605290206092872865262220006122998552333654430647536e-11');
    T('-182348440817089344.0', '-365494613346284385774359945728371759418255497043130583704938461995522713031855368282526623182864551158365546976602079453.8724', '6.66473728707398913969618817118679541450450804984276803582504571866881111782170812402431629687112708340501385877734505804826605490564775757056e+136');
    T('35078436593053772121948478354132824046030742485793509661339795486768350160379351940260957647345172644134739', '-470315326468850136120.98841520254693896261066046950478030733711497927705290819414286702121213017731741', '-1.649792635827894394262863251819308110416062211172951136883946185746058880577253761809396392427026832922884334966951567100285510447677219200061048033820977456094076440088034531057882865490348221044489361050599e+127');
    T('-2329710971489131138898317317972150487.226887180882642662346810938', '0.00000000000000080780934683985383898326280513685583649350582', '-1.88196229820427637454825838040513247697835727714033766676609406173243323151115501339627651370238022434265916e+21');
    T('1689256363719086734358832495291997380220279912721648779060.022015003200579854886304', '-2259059589029194007856573438.47207932659178007', '-3.81613078678819075313019801142236253002420775187972657992120430084415285628119278545102507880850081033723214556951631082316128e+84');
    T('50563883339463774987198862916277845601202057166416042879.417778572998829235141574907848222440125486172315940364098463976083652560077213', '380058072907244.784369350321644073081361404543907907939563914660519351353607790299736723712817719111414068853481728614572841297911762329370311135686891', '1.9217212060703343272665422309643023974471329394919471100483903476011902558587720793118211708282887302802938658740501486070477164588947572483424138819524122982371487252360991540904867150086111696239666594880341944141794568786850282040411150653341851298511444181398173001624310751914783e+70');
    T('46294959374709179919178793118885893663797283026751959408680198256485686362073381865322911852807891547580600.01450863642145', '-172254', '-7.9744919321311550777982238299005707271637371904901320159827988704726854186125883198293328582935705506369486748991706581404483e+111');
    T('0.0000005527834082142680454399157347331241068314235320652520705781207217168327100784683628889780310703', '0.00000000000000266632277268185128455781020487589646959589371419742000347748975254376924718637309495578143940081972955041', '1.473898989682390821841521121371414207169953198718769475649011583919402328920280159796586990896854154136491120298716299421424991847787084749540756307376833028770272042850712186481647279973624030103823e-21');
    T('-3018957351229289626132713890715830302577902789980653248950686120322288204403609991088469882148969888336650704961738500782', '0.000000001082120521596790393125823482055072836760853689259276513674877920764922256435650861059862899919835', '-3.26687570359070362516085008576334548950877338799335353420537013795825989339548239608169241156257202285080364129276771767713053661332672872002678878052870887315156625253726777042206003909947050207392523720502088481097e+111');
    T('0.00010578119530223846946702960127201887002317294288', '-6085787778834088.70933788474669577642209247160329424394222', '-643761905600.8247941857377235732296308415119494854462530749858584648799519132617834281892261058500803936');
    T('0.000000000000000039289970150049295306833630957794488396625415545568122904585548214355360114104351072386053992611376825118885803929657418065649', '-4605169823674888008751577292618614971794.977555723667587656862637975931940238288328992', '-1.80936984908094126427921442372226709393792657401598962039210959706690306344862877971519465406694496205108752617960289681246511462795560122095156580067059723548593631880883004929720338763881489190940653165995808e+23');
    T('2514613511.2170783808206929866132543988564534527876194954598859691571100338559560', '-1964495794114274085237575951315085841.361649436638538985888913723131864823298797343859193187321313839924840812597640408726788696766685', '-4.93994766660887745881096220582576848896286904471239767301554888303067612828309748777331041576614562636212559985704314683156812973279114418194558517790467846327290437698974640024899151989425196795301938022962586e+45');
    T('5799359.56694741019279510257555786015925813988005191176124258432699765886085', '-0.0000000000007231844570746320510997217888', '-0.00000419400669980343608737887111010110559927717748555612552954077474280846281811020478125660977836613228848');
    T('-1791068145886144126734036774645051199503204286826392100458612918035133189286803934233671078314145954721325200457399093789769043791391200626757', '1.902', '-3.406611613475446129048137945374887381455094553543797775072281770102823326023501082912442390953505605879960531269973076388140721291226063592091814e+141');
    T('809656351460719225181711866.1488944461227535', '-33867.144809805871082384828278160351409245109', '-2.74207489010992553148307158515078420513099421238160223165035288480514204794793834876315e+31');
    T('-15343174181682054141420677206782114884223299540732121123190447946670136382584737580470391369', '987099689944419071747444299624152052403106779038923393493086986368730055942594131963617011747402639883887116360086428917.245', '-1.5145242477501571458201031358834948135288567189066332534201297139323596513233299183085395688552572218144560229423247824057601351422866714799537508985833062886631036720671807768726086293443750449585213823804063258405e+211');
    T('-6.1499', '-14122475646217852912636972117760232196408460299060710372963438774127.30802263764662251543254158300069146620610278670238171347884347702832', '8.6851812976675173627426114827013651984692389993193462722687852117005531608419262963807658587481295952448020911527940977299723539499376465168e+67');
    T('-252103580004042401308.937513693319696445897233569009398', '322191.07259437929639208064231154', '-8.122552284638533411190811393904476674853317117242659440655935323513978821796270385292e+25');
    T('-332385.3378738577828707399978866231142878436743418042927022688847501694303446138653358154658874513293895263254386919164060798805928631172377176121', '-1788688861556141964646237161499263851038929089394372356769751862189978338244963495542562526495618342381239342', '5.945339515995442741438198459409608195257376497495695646466626108763151210023427170436149956435379334122608797739249630758718894850267317176420365157416700495312818386418495814195596242152257323833373715286243473836090341661932706350920849774582188152382e+113');
    T('-2460917323354918446645703674853641464.424607984385990612569219446652731606596614873511538897698788038533', '-0.000000019704561911223284586046679188808477807835576405248282475083383206973764355330345760276507856652823084974', '4.8491297756448881664126623064115779205015987535516032622595439441897517259719875348865351513634911289023687531425512745832163192476366584227547545673644644097372482366494539257564003995127411984043445303142e+28');
    T('162243669579467948721172981736248370537958026297935040744.847805681773372416035821372548485508877545367724814214913560686058189175343393608601489512', '0.00000000000000043034872397155', '6.98213561759858159315183613323524194776931640410698827712570108401602008047713604207319502488289379085249184148133416471274154998412862192528868878238703113836e+40');
    T('2942112647049268674939521727079875936312678683322556734336248.11424534652681157516904544382063913587308466251549871327575873372911717', '-0.0000000160518477504399168052667437532822966025370257875590966186547766187236177521021479405317977917073345500604689071434454454499836553580673874665532292', '-4.7226344275078632315398240344022433291786804929459211275348315431326149856316741173727784448214255046618190169485768234658926420021181206553535921500696882020342752168452799288030099975139315001872319829531828024996830308930197541236951052531735501834035920248180157363728665364e+52');
    T('0.000000000000085659689652397679263363379344304760754897339', '6.8', '5.824858896363042189908709795412723731333019052e-13');
    T('-5.7783188901971298313183550831107549667729942738531150581362739386230606910679155155791688758', '-221439820124804823862034700.57305405733456151859741120374945380903497467757168028277653394776910998991571631868', '1.279549895669014265673487530332542622307080646887562219389924149894188918975837243317437652487919487957552517902434975981852190469467703844441311320668017835940689319709278323479039083091077241610139944e+27');
    T('-2847941374822', '0.00000000000000000333453862751515460136267946372896613495219431745263240560720335', '-0.0000094965705232425743530447087065739774971604318716917458420879678503055240537');
    T('4600164782241764006590425774876842089865111591027822696582831157578.739719314165811327385420927586685826049515030783212', '24936677097532922790671460124317897120172195409691302414459854884214881356875622080300.40462', '1.1471282377020572147332327802533044122369637791300506227907535665141877615112393905214612571695198704856557157162943485152720303637986357916099145417183717023786789993994223448696820674347492278452767910323944e+152');
    T('-0.00000260679215621874990889590646697464210606210167340857181603112129227967025884261071747351602786622983212594677321781361453743686420364802073156', '-0.000000001556976867693202856950382092896216853975131678027766508144349094921207161', '4.05871508611667956997596806284421681761326338922686367392307074925675080478202951240305304779651248204530654676779205376458787347615247100101002125251986813572273619622642745986755165968519176975902222802953070116e-15');
    T('112005422206895581683067200.66464306253548778644799733891467497177', '-569288990012093221416615337507679289992168170439593101496410.0385811397750145927182473472588677939073196039082322785618764898744620304415', '-6.3763453684041663102425299537134781104795400180652201585238496486086184313570463723811857606393356939337147660320432983174726251428956040334352123843908741307666227939307913535913017213265015893136455e+85');
    T('9234638972249568361641387675279591873348137130244080421631134092282316805807483776006033822789591399832066971', '-67062258760784733815191025025646336427603744103904997634972806618201463866617680780550570309742758409129550780654178901', '-6.19295748319427746239309128664117623036037907234855321924344705389173421377284837396403653127764945278959634318829418910758542716822570206539596799575970845747768528847286120187750160040083010843243686852268940286480535647178871e+227');
    T('-0.0000000000000000213737912857665573638079572432586728390000134190', '-181285730.138353', '3.874763359064957107044461619607858392925857065967379726558907e-9');
    T('-2506504.0733758501826133673932299604035650762719626337255035252843580215290504920568908', '-0.0000000000000000063307590401488217267263854328109316460163087745601605669460055525432479875260852024893294778103375548445194197525199702769907', '1.586807332169400912551173916660722325707840899719608136941744779695862574081783246376975996756197880571888252763332023979673897643998859766655287848000511959009182082181258704230204193519120752866923945862251556e-11');
    T('0.000000000000000029191265291623096374212238517809915598918566680688197689722067072109843781983891539215143111679796529799027', '-144459614607567236763647582499049882580370335188691777.776222539161298865028936239380239788850306312227587059786395648441342513', '-4.216958933935126326569517318236903884107951508793088777740048528137080091324499213666957905481836509907780548281139659624937009921418073052266920044400057847392420013165462539973879514483989622402887652609685114451199224677961134851e+36');
    T('-0.00000000000000000011367487320425818863869979220926084589271902620348942087918357405316552888186055679751253463369024199347442851320519227185478', '-21839', '2.48254555590779458168056476205804761345109081325800546258049007374708198525095269990087624386516119489548804429988819402503654042e-15');
    T('70555122248707171314736', '-81101086022274692488621411837767361950248378195397909.200440873455379623557692889032043422151272310592459622568950827', '-5.722097038804507341200180488072549073313712476829304641270901056694148836393515478119400477210997323863769882713087618465778390141724486672e+75');
    T('-2328737444159883.22585049478830', '-465474361749661451599.9951730137496', '1.08396757550285951917153850391172159248110224634381083904120968e+36');
    T('-20.3014066928633757647889348362325618964962565804945772008656587579802123642954157', '-329435370768184084561406820034073934012588977709700270652517248023565145783776412004629743988394401273.28545348', '6.688001440979140070188345542361244069517729332398623464379411528839275521534924214116901390055160850670677514105783601320874524580245478001576704187186030310275122663282192690671340259611636e+102');
    T('-32476062941804823808177212128.37047244026179396179206', '-23170264858.839398780672784729575796922981615252587039', '7.5247897993395677581189377054108136654093646195424085291052102413168985736328290334867475461403556911034e+38');
    T('0.00000000000000078020818918877022784107814476742023039526884490519321760350204369809967323559671586528756587388436', '0.000000000205123365786662618531603763262183979221647', '1.6003892978071778645361752741729365945858649779217485973276644181520266580188064434359630899417624168875864924106561648427968680391728674092e-25');
    T('16411.97479231952475505598642359391159894419529343140530675214816439312142161965104995811520160184950', '0.0000000000000027918217792698757486367905214734859040448642073369967425369035344103512043731', '4.581930866602584512178030964640390866062580122108157686730832013698006926287481502511491005515718454308878140351176787687577295267975260433555460720995298677352745266944804845e-11');
    T('-64342576782108353170532624771581267539756520906919.37', '10.7', '-6.88465571568559378924699085055919562675394773704037259e+50');
    T('-8221425425564202135543311357128878704299939907773337451688454087709054980064079701204709862893688091178322203851683767887893871786', '-0.0053653210895499081114778560023351704116389771011759394571181993140755819983', '4.41105872219414419707174038902324084811431456260158178418632957765849297276315251194170903369456489104239536734987785292755508500736273353438544501239198533098378964478014299793109607485495440119098699638e+127');
    T('-0.00000000001910608511655268019857813374990127054476192504583955285846583482452368825713345318314648457310037047203752064778987032587853577350', '642301865080793548011195664841322815727082571813145907.638646106551631362492680834939823385729510144767143760057', '-1.22718741047541772674654955433282115912253583024496254728712579960888844323021707984449130867906756160401346562961603346122134581742784760100299534939794183794586063570878664967245581007170245594883361157548393966363447660442291523748990895e+43');
    T('0.000000000000001899500268191217', '883952560680798125471977636433525656958767.4329146255465789678475476012476204472687459005798142889552058775500478237', '1.6790681260814890585836193220538636972839642793176878790660318627202422075166836170289452406463821318133532926386268184542463044429e+27');
    T('-506222074114478440.4212018359818757090468806843375176631454826045115958516526575279719878856236353605111026111752257846026439938084657014050761145504', '-5159467544042334960313178363211289944335613842064316815811191728903.574367721298576480151967142002627219994962105347353373165540336969132995273967', '2.6118363614714449452582198430103897223785994901299317101044259761348107189762413607706765732326606750493547847042725797461693311578242536847852702975212843374994786292094043375579903644474919183423720511760494827145234240751467691929508470719482500262148133264801792166487508484367183230294368e+84');
    T('8650434955204837420156251340980467125431552732.08397236030027468302684214339589220209558182529147', '-0.303570960693217884327420647078548564916276955021262691277956348864969879961811223297756', '-2.62602084976572571043981976309152638775264794392233958131099791715831899377187947876427492733764795120380985528043759980136856459126265934320288206950485593122394243946315413929694132e+45');
    T('166664781364.0082914723488023478368828305505987873686361307000655181790946128760191770642121717143556656346592000446628174724572', '-480041.0942847509055712529720906841565497160321331762055830088670429871444554935489589608266394637805469063', '-80005944024707299.88369343694993429517249943540140207247881593079529829548920836320477337300843753304432472775747784579377690170478442441481497272861023856984874550780497963725202716372180855504618999968964525770542110325165291916036');
    T('-43211558252159093527866775647205601084011492340.205459697805340231598755202803639527161744593', '0.00000000001119203318598549392536941753377009590965307584900110755022862027840043858728011387253539611180', '-4.836251939763099008266531873875699139740918027346229229666486821933921583162396652285702155362998553549998984080376784393703700976212048695921578240422117362224246009514826407968734974e+35');
    T('22228080669906830995384898726318434540920054474075066362723430068303595334', '0.0000000000000005446682564983826904165557014954366345827192263245436061673439169771964847765827397722973716801924439429612435686218283131447915481741738621944216616', '1.21069299437835559678074390552059815249561615752757133499522502582835547433276587506411508638591250461101776265492507290334698412116695650224909917771154983775120278858482062610470094757493360226297555949389464060902869744e+58');
    T('-0.0000000003773157928', '43297647023077445.54510130448338353765777486', '-16336886.012887026261648726857462053937811976384809008');
    T('-704246.0642491', '-4876468942183457666260007295840240729718873395834733463009561654592496364003930788774251638836263860.464149571397235826605163', '3.4342340599656720408721350433721409241108083813038963987160054952477748964430703827960612606796544474314358764054099696015239692357781033e+105');
    T('90714014677317257217388754359562724228350422511592300196367622823480490782750230695', '0.10551171877203891416100509846755006675817523511356135349573800436749642311', '9.57139160531570883071608150743717558732170393411960576472278636404390923564848224638754968731571966908325157310107689739849841961000661380686664893982936145e+81');
    T('16256367650251592857824070659043447853047682031758021447060795635763437999068158011730593356058028259231482670.842728511458309985368672626159', '167180302632355.05677826254070883700709474391390089985215716385024', '2.71774446347188795843522760152851417095331335558235538784521529465054664495076688729866682142620092642030334321689665117663066458919556787136329889313558843246657535262982668786110905407308427324058242816e+123');
    T('-0.000000000000000005140869693986476129573937292110832444827883905213371700103686737472419302083991260187191661701298052551015422048926889376', '-23431072984.29109719929265906334268220161090515289622128095638741310550195556470', '1.204560930025273608708687379702964400245754501914517994869604548744993478713638266232843096136298708651286988188211879699693589237471003927285113544165412103336583365360013429733201256002657445106272e-7');
    T('-600918601565864357615830916531004971541558836', '0.0002398203266606694050330194859330980464992506791531578245580976766372221543075302994826964325096371', '-1.441124953239982356850028499314436394823612382489365920407488055245261701578444032269000578787663442887261399958157232897473876372957266584156e+41');
    T('-0.000002378573545299614819606175997977561025598320050', '-2882653077216085525237966723240931593332010153721729409.189099950085184011450067199947461092968807651228141791418167', '6.85660234974270886050574307192845780707197555258670774440795390596137425374082134400993260259690471606609110758329634878468480296762286592478289418958225034835e+48');
    T('0.000000000000000022357617532781889076479320065100116692254213222725326015200603762919174701589837856906', '-896457719826992659089225870644004458751127732719124385827686320360654074025236', '-2.0042658834201645580346859820519392326977523701452584805357472701178299220185960756765568847872663515490447098727456428199599156161369223396351318781899083200879816e+61');
    T('-0.0000037591700832106546820837009314715135833879626195863911405199', '0.00000000128660973296737392590374219331743360889744123526004704657440431966332363276599678510386', '-4.836584816938601241595643894689727875085794631034207181521858384737358785559745009296624885016550841630384096559285949624409877337769212375896814e-15');
    T('-10897404485088933268299142109127940457874105398307905112497480738702104026493870', '6459527220783833615589828721781406862908330296731571946257487596628.142', '-7.039208090732380052543394587491979904440526984139707627324909518932550024566922074139234694919548719030400279757073454088117750506466392244643248954e+145');
    T('1252391487512578363516059332047552591823815440694963818513488609376352517930673851276.19244762777307975100', '22549776.9253795732526305732013511684558623600507908536351482864251791287592196740297262530719650195640957326260', '2.8241148666652939539350687862707201683973473917640485175382134027259511907319776870024731139293000940076904783362318907295063149688420418024813050345759700762391301983348609040284923961341431326021436020670656126e+91');
    T('-0.000000000004618950388937140493422755399930264176038354392749822437964314306322496775785511138074695877738144276032647315124913748488917', '-2713380462202072657447653028643460793172301', '1.2532969741222701540713742524625604310117116627579838128477044145948287310122585599028058960352298239073038474733050249887724953498915163176662179193294848309569888017e+31');
    T('-0.00000000000000000004741599927671153114706', '0.0000098605921767576416944956043265259701017397049855054246352774023236279136700079166226671683330780112375', '-4.6754983152108772107589021499824280537386842022454892219026854810089318448711395624891020610400431916945696952628194508675e-25');
    T('-600334522639128350301416069108895484950049158632569154577774908833218708159964256760025260070745036754849229258748618.286225296633751065085057954880221', '-20603943628058420.9233', '1.23692586624339624137684561274248493253093335427682675519597952187340115485564971154200167383285889714823571236526154847178654496322656526986532466296684549399002517280493e+133');
    T('-0.0028569903450426823012406248893590913347605950672345342794', '0.000000000000000000380602538484379176276476072502', '-1.0873777777486072321414819892742916452188931452780514481064796648209244384695987250588e-21');
    T('7633371138864371851511200109335895009200379214065654.14141197969364686617958181976', '46557774899433776362769332468929072168263071780847173536.32424609893425634', '3.553927752070818711219752919654829713936648328701795633611695455871498173222792379572585838047775680404274006272352702005591443767922861383251901395172784e+107');
    T('11816689310524760.33060749', '3240086146153074.763149055913088596613954276720424786571958312186345692516028911031344618033329875358392613', '3.828709132842640485458626662093761049902194180604315455762384568460513896245457880390402922939380711647608722106357184891221847137e+31');
    T('15417668979208863688007903355418023589336919725455814', '-2287252965683244062854057265399517873076559773521003653290361219949584429534587223153067', '-3.5264109096618027617770541672930242149319875097968700968868729901509384897856549235466082618911136178623675867468948846074586307080867081538e+139');
    T('-44231051261766981720932141949492661571946235314205992269845098822302266502035476097047272345277610.27', '-21829364252494551993461871095.1405414373418522117', '9.65535729263870199707274609187215999675619566645960063838135526473595293696284813052387836041619820102648568607484181212361915051957155111290134159e+125');
    T('-0.010471648280929367353726533714112845102028595890585874570958083', '31', '-0.324621096708810387965522545137498198162886472608162111699700573');
    T('9801144054.04251480', '-79669022889354091.1326070042372454589141728234307836693721872475118', '-7.8084756998336986277926157641231569523889190793366151638111003547152953122771467464e+26');
    T('29857288634175691192253935741525569423464902722491247366897306044982457983212231816738628626832523979506745945', '133673.6558945837023809692393557234541133', '3.9911329268300663632380133640663780097335900438380448470493780783062334184720774070186349410361053454199720794804033925701541209810757057340083455685e+114');
    T('-13620158075540606.0220850459251063150637623012169134311683793968716003579443803708152', '-14094712052845720944279961661927524928294844400168253261.8801483866631477108668134130846798999339316583941852334437626140968', '1.9197220618898615906375442169544326587585370014151488298656204877054492796167185482130313382428085300372006548553535032109971116684812746007217292632043932773898787721990877866744793392082655385848282771136e+71');
    T('118440515054.96300744924345961909763451770037124776047596926946', '3978918406101218410350317969873779924817925511337423100533163173007579791459856530353048879996496120205533862684343054321905', '4.712651453803007726438461712612354142589559988228956502605819373002120961113842720385180079632764208616661380363663300333950937121937309186788267547560787574638782242683284222185255213e+134');
    T('0.0000000000000000004979626118', '119953.13939299602151864098685411551208280009209744651134782647617437471653240473066119015267896990648067942927449283677891199638584032687735555', '5.973217858574576550243146820040482597564559171812500490156201032898210607636089901472178932446062156472087484005983212281811688264520970961370795522549e-14');
    T('4133032609317612147361.233208455670', '60149718767070729539657807610332371421265360276297353368024670202.145813386468110250239535755215076073418035935987159939685', '2.4860074910558688193760888076283428107792208632944279345307661267298591118711344604905557070531163947655318143762909457571230898163106422046776822919626395e+86');
    T('-0.000000000000000025988801412726981607051504723536759179533093115290316536803305521555847577878002593862833182498872', '-62190864847540861920221512167077698610328189518126509.945114225', '1.6162660362084827318707380558579363611834016595419194188990226173706520381985407013085330782761618687291735642283204415795057016756891900469697853795529736542e+36');
    T('-517391939544713325422033696525596189851051748767955039585693350056993951682945080450723027214906277369681756055494613881445451892', '-6355737319244501.33875834602457354455598517141883018726489236', '3.28840725884062937342287062927549804683222950446951497853928995858263827438383088820860677614531136832120859449547463148055636204133361775822367974808862660315286528874721657070976693834512e+144');
    T('248488108396888727192484307620472597201728811.43632801228625119529655174763542491519552462322578674423725814425253', '-244376619784336994670734053495423487949530922624294868474118463232994026818752162901576014805701186113050589437429269164232162718718', '-6.072468398663559342194397673594158367331416443540937376483832956458681282701204862291922555840434608500693418850945313497296767648108091241452234573071111256003091374254946595152378328332428736545902967819673107271115746769252798457479074985654e+175');

    // Exponentials
    T('1.153024e+0', '5.28973573008e+2', '609.919225043976192');
    T('-3.699875785122930862283231104748329088664056423999601e+15', '1.1e+0', '-4069863363635223.9485115542152231619975304620663995611');
    T('-1.2682525953252723e+14', '1.788378029508237622037088155833989378609386207655e+18', '-2.2681150773465187729527785601467557683746716925715084680772194565e+32');
    T('-3.87722166e+2', '-1.74278705869601992403691e+8', '67571717327.438998052674620914706');
    T('-3e+0', '-3.64329808384254180872888049513111969646643282968e+25', '1.092989425152762542618664148539335908939929848904e+26');
    T('6.953012278719261355e+11', '-4e+0', '-2781204911487.704542');
    T('-7.04663671550547513365402564941508037562051247993316079593e+15', '5.535517336770122093358205271769744142052184795629511766877e-18', '-0.03900677970460142822373864049115522460524845334243105937826127540936694775384656847162726619838459694385924193041061');
    T('5.08438408986e-9', '-2.606367e+4', '-0.0001325177090713613862');
    T('-1.2967e+0', '5.94990983e+8', '-771524807.6561');
    T('1.3117e+3', '-1.959557842524477358350764324331855847355274e+37', '-2.5703520220393569509486975642260953149759129058e+40');
    T('-5.3548e-11', '1.742694574037778940228462366629620551e+11', '-9.3317809050574986691353702808282921264948');
    T('4.34526424962996265825639250671e+21', '-3.754914933534859756171859142125715053796299171e+1', '-1.631609762109069348677868227008975151085048810070540943592037216417378493741e+23');
    T('4.7426914347369245430071315112274335473949505818891357635e+55', '2.099290936966289806252483785990857067e+13', '9.9562891457708756252380705318075217577762765683497811755225761339325645531279572292864156545e+68');
    T('1.2197662968047362383e+19', '-7.9257e+3', '-9.66750173858529800389431e+22');
    T('-2.18492048239986982664891448764e+26', '5.0217571e+0', '-1.0972139945426971341049955535599032244e+27');
    T('2.8477441033458111e-12', '-2.6015762266879431727233849707015975771758956198e+40', '-7.40862335895523532800693698581126602552353137494951139716821978e+28');
    T('1.17305251157767339030842411702548e-7', '3.1679877726710109082864387936446078459964397e+14', '37162160.13379088759968293791782997863641886182709969796604371507532234183556');
    T('5.516400791447843686549580838221025644959140288781e+40', '-4.534755696021348164010120097087738141133999137556364474656e+25', '-2.5015529910554782273339735168586403296046609840907360623635477256369630649686086353650780443350045195634336e+66');
    T('-1.086178703865e+6', '-5.412964e+6', '5879446221587.90586');
    T('2.193003362433320722921995078848109409779629e-11', '-1.87156484301625964803880921322257778262063404562091823011824e+56', '-4.10434799374664745958524117450745644772403340614160247100180995137281346990234693655901738534901333296e+45');
    T('5.690242268084896613323736558476149127814487192716110263e+4', '5.85123113232510962819522680703136616268672961423348513284699e+54', '3.329492270949058963099905818513145552769091859265632399465562585831994642969765945460416258196265143264172794765837e+59');
    T('-2.2964852129902920477142981e+11', '-1.01192386417445983894690800351420572044711646857422992584148e+47', '2.323868190748643763813766975749316051539457362388056941019107244885259571701270065188e+58');
    T('1.64014114524e-9', '2.33457695670849217356126573862065622e+13', '38290.357234267802543783512378452575905911293928');
    T('4.52030709268304468292307103018250587082757e-13', '2.4202633390406694150089337e+3', '1.0940333537626286438097741446790925743401036958119194290469762262109e-9');
    T('-2.531530179588420531938806173298571611356854e+42', '-3.7916295786409190517918e-6', '9.5986247081496130774858068268051041666243771242517966898979109972e+36');
    T('-1.1137e-20', '3.15532351096658148747044e+19', '-0.3514083794163481802595829028');
    T('1.26704207115899080748696654617058814620502905910804474e+17', '-1.1036640030158560360573681189153991777396167698e+36', '-1.398388724244832908797767208849960696897136903372124358147141607664782163824521550287804299792680852e+53');
    T('-1.92043925491801737087120450765880605794833102824745131e+40', '1.48390768547e+10', '-2.8497545698511264714107436784310097859137225929760606638486194657e+50');
    T('3.8135622104716384192078983238750322e+13', '-2.39302666425671400328234877933630532203640643176e-15', '-0.09125956055460405574747767900047040915488905349535492182534023415362543138557102672');
    T('-3.234931242339998871854389747974137006095446e+30', '-5.92883411067302421812958123642550690642906114730631103678724e+9', '1.917937069526724859889065751200796792104177866908520783368616010428214659504892064497475599582063490904e+40');
    T('3e+0', '1.99273095508512817553537488885e+0', '5.97819286525538452660612466655');
    T('-1.953336949e-2', '7.78542087974602115e+8', '-15207550.26792398884803047135');
    T('-6.76927248655350571291737558944587651874684243378e+47', '2.106647491079819039765594992079775666086664783295561e+10', '-1.426047090023359087715991793962565118985535044059537158764939014517624689564606363869607106631045058e+58');
    T('2.84765064643062294633e+0', '2.55136e+1', '72.653819532772341603485088');
    T('3.03393155013395755198972274893138442728102576e+11', '-1.794588238175697749312009789507e-7', '-54446.5787530056249211726327679537663382541795271931196078506483571264470032');
    T('-7.4371366140844e+11', '9.1277001754e+8', '-678839531768519399904.0376');
    T('2.56791237096035223695516010175181403674163385e-14', '-5.44464273285261087885992104112953082914e-20', '-1.3981365429151599690537623761404625947844618835506028383097657567011072106548790389e-33');
    T('-6.879908191615608804901708679355480110539e-15', '-4.0968480016502745852031526855040036466e+37', '2.81859381263577613382981765247793574699886457413439013342011267606204270915174e+23');
    T('1.407896156490752151388208148659e-8', '-1.377363868116052e+8', '-1.939185296009824854218448713898983585080174268');
    T('2.425103747079320530219213960814750892199747341e-18', '-2.365116997875961139721649192832330725431640562e+24', '-5735654.093829986735467358771314018265857755686256663053548830868280136074150452583527245642');
    T('-1.0967188654311383e-17', '5.50107804404633954170295415508097739e-8', '-6.033136071114646945658547999340034265651011304263037e-25');
    T('-3.2712219404248354159015350722161266784e+4', '1.40636e+2', '-4600515.688135871535507282864161871915434624');
    T('2.7739526916093949721614335309760564882447898334454312345e+55', '1.24528453636520646201893749622334742900013735e+0', '3.454360391469821959459359614081123116456332059470735649519645107811472952525654511905494830480058575e+55');
    T('-3.26272930891827103933126340157752087353407692e+13', '2.0026083903335067609700262e-19', '-0.000006533969089426773691398112669586901327358725244893395121554920405215304');
    T('-1.06137697482182359209772074638146895e+28', '3.0663106098e+10', '-3.25451147889358514503318856302707319949426571e+38');
    T('-7.1333686145776974736348748988848e+21', '1.62872571577602913313701931021625476208216466585507507747565e+15', '-1.161830090267232160311720624731057789079310830776668256181105708907630639297571581978415512e+37');
    T('3.271164612807454534123020114329520320585016126992181854263e+9', '1.2e+0', '3925397535.3689454409476241371954243847020193523906182251156');
    T('5.65403478840648195280973451110905199e-11', '-8.18034629055388719783063246807413028100652148732e+15', '-462519.625080035971400064460401798364338310053450183034154989804876490732716900057668');
    T('9.44482836533037505193155281233841e+7', '-2.067341641721856797e+18', '-1.952568697856325861901776699253387295308175672267277e+26');
    T('2.5111226381792956734986397e+9', '-3.65360655537789827590991569211646510131461e+40', '-9.174654132209716853544084660541570252769600723604642707862516736017e+49');
    T('3.28741239410951819553626698975108924106326e-4', '-2.62227844885453731582e+18', '-862051067357068.82790743503875251297002496305525999591232187732');
    T('2.2772892974034099e+15', '6.177601666756392528944e+9', '1.40681861593257990831209635468300461456e+25');
    T('-6.50848921524116628e+6', '-1.144580764938138569486790679009e+30', '7.44949156457235930691999052822025420513027461652e+36');
    T('-1.4551927019037e+5', '-1.3e+0', '189175.051247481');
    T('-2.2911981542413e-12', '-4.53314582071257190328650375668441355537191e+31', '103863353373233077959.40247540692426855128645966365381883');
    T('-1.4602128592500770116678944302583044846446637e+30', '-1.427582797083786e-16', '208457475794593.76574073813183484779514002948555208066927682');
    T('1.6700609441460181151986345895359954232196252236e+46', '1.68243152329e+7', '2.809763178246720865741573493401128674173862301164254857644e+53');
    T('1.05083997100929235995740938763615497143966041793319335e+49', '-2.524496056110348375423188927e+27', '-2.65284136241607138569403868866619712437928400385166771000044188224282590147003545e+76');
    T('3.100786363878827739e-10', '-2.86304063902707066e+17', '-88776773.7272606582182668524792103774');
    T('2.502713517421925552323011065985443554034244e+42', '-1.22434e+2', '-3.06417226792036033073115536852861796094628629896e+44');
    T('-7.9489222898867596260428e-12', '-1.0788548012690995443645e+20', '857573297.73592957352865844131522518881758006');
    T('-1.0755932899013878523165928263944458509156806951399812108e-5', '-1.11614e+3', '0.01200512694590535037484641917251896792041027851073538628622312');
    T('-6.4497917487303e-17', '-1.004446091246e+12', '0.000064784681113628528184449538');
    T('3.084474103157632963053e-3', '-1.340722625329e+11', '-413542421.7344814434949807418969437');
    T('2.007409677882519495029550241974657552014828965e-7', '5.222151840789271106861316521519869023311e-5', '1.0482998144572396943190891671428030237589403912003291601853143252123470079223263003115e-11');
    T('-2.488498956742602121296262808732706422946353052713e+14', '-2.23172e+0', '555363289174160.000613929563550495557821783503480065636');
    T('-2.9688672164600521862302335509380676629e+23', '-2.34340486619e+9', '6.957257882124446358953277414968471000555340527351e+32');
    T('9.6116339631e+3', '4.941558826e-16', '4.74964546426381633206e-12');
    T('-3.510917030490276336587658593367687803429e-5', '1.180564327372e+12', '-41448634.02559652698690017440995500011934045540158588');
    T('1.249503228e-13', '5.879424084035664549067080010774031325182324371002647085e+2', '7.34635937178350612118448086199642691938843199011087712925229038e-11');
    T('4.5e-14', '1.6224776733813843876610106340899993022215e+40', '7.30114953021622974447454785340499685999675e+26');
    T('-5.98567638319111415891202402487540407421463486631972864107e+17', '-4.54793136981650498391485340038861239718e-18', '2.72224453926846670043530235188742096666820419395601557608672857967843105674091877295333165001826');
    T('8.92108677749256373753068098541e-2', '-4.701266425389385608e+0', '-0.4194040574501097838530020557051108449340791197928');
    T('1.0445719060110887517378867373770257738782319e+19', '5.799e-7', '6057472482958.3036713280051900493724627198667881');
    T('-1.03890765460564576e-14', '-1.34206940160748687807435785969646342084e+11', '0.0013942861743420366640453267735350857670895370671868416384');
    T('6.39749371e+3', '2.5234527992374730183240434196649804550511817034984615e+13', '161437734106036264.31022782519073602768462872676198492440927165');
    T('7.914175439604437611967814601653913774828611014407671e-5', '8.32100403850523895222657658496599490340606e+15', '658538857943.8750019874631462394490423060270196984652115300400533954357304447023953861129188626');
    T('-2.77837203022891121310898693340776679571778100244092e-20', '-1.29e-1', '3.5840999189952954649105931440960191664759374931487868e-21');
    T('2e+0', '-3.785802336743340316719402396217939978601e+36', '-7.571604673486680633438804792435879957202e+36');
    T('4.743769864188027932328495491240985e+24', '-5.9891655632114551610420257227248006595584101099705918e-15', '-28411223110.395218469762107537426925959156755933965249012600778016167994609252836864923');
    T('-1.696501776305087785563120340668939029159482e+36', '-2.20704482451194839742722899349253308746537e-4', '3.74425546516947120760454369427493495577833827584040523256623062924543284078088213834e+32');
    T('4.0131499023574625019065378616938094123e+12', '1.177461924351806545225557804503154113578719e+28', '4.7253312067420823358733892576894302742518304481762080832752336264184374791768437e+40');
    T('-4.62155685185e+11', '1.65583875e+7', '-7652552920621239187.5');
    T('-2.776598951854774016146205e+24', '3.5341178108401263600119940986415244381516593e+29', '-9.8128278093099833546223999989350419583424311768423158174047121479565e+53');
    T('1.12e+1', '-1.313e-3', '-0.0147056');
    T('-1.400922738050340928068459005e-15', '1.669774637213352507258799185019866e+5', '-2.33922525669194448953065643831459603361384442257691893159333e-10');
    T('7.50109935679466448466358706299522e+28', '-5.03e+0', '-3.77305297646771623578578429268659566e+29');
    T('-2.38556354949153835408756963109992783478e+26', '-4.99378748e+0', '1.19129973861952045985823120474150382502278725544e+27');
    T('-2.94e+3', '-8.61615789244720791078571983086407412571238524628016e+2', '2533150.42037947912577100163027403779295944126240636704');
    T('-5.80381041944064404262216137492215065819338e+3', '5.677138821495566551897442869075850448375293389320539675033e+37', '-3.294903744480694771704130256831336545059429064900653708388680125250845128363784484521559308407188154e+41');
    T('4.58722638789532058289643e+4', '-2.292585677620131398099727011e+18', '-1.051660951688994125699713239960797492772817868647073e+23');
    T('-2.288825775843864143902825178240786758e-14', '-1.022792029007781346809e+13', '0.2340992759320655142089938144423568704277940784273012755222');
    T('-1.432063323237600412250117767285385708080998075274668364678e-14', '-9.23012609141738073365221260162e-7', '1.321812504437725779783150336060293956648246141009265660188911448444031899918334929357836e-20');
    T('-6.0806e+3', '-2.9505713511085379204927821042777582523174534e-19', '1.794124415755057567934841086327133682904150714404e-15');
    T('-2.1971689665281316657e+12', '5.06674545635318811308850459987394e+4', '-111324958779966411.74779876603862140385999575942221858');
    T('2e+0', '-7.9498279886418403969498688721389799731931074e-8', '-1.58996559772836807938997377442779599463862148e-7');
    T('-4.31446036399049665749135715315413935059097082785e-3', '-5.09957831192823753379633205333620130303488425232090581e-8', '2.20019284998799462125934843416682842768267375970275614241257898190943464155895136069876374903975748085e-10');
    T('5.6684216814938374789992297e+22', '-2.466e+0', '-1.39783278665638032232121004402e+23');
    T('1.4726466812656072908248567521825181138629034e+8', '-3.47948862867465172229571384341201136985691220463e+47', '-5.124057381519144835787886917183826378015360092082220951469308136228382069243924518066722742e+55');
    T('4.3284361218626063643720235128240704200079650833e+42', '1.4641063731834e-17', '6.33729091193628179321704177525739784198638290425046666717722e+25');
    T('-8.504503345e+0', '-1.6126451605490274984274251623885903733401871912282447644089e+21', '1.37147461621872663968730195332709350199064207907267622573942269977705e+22');
    T('7.7309990341924055295760169328629798135433331298237e+42', '-5.99563576510407238824175823080622306644936962384464739893e+32', '-4.6352254309389028017492862352717241512621152128519099776509088009611537721357769693774958840413184214468641e+75');
    T('1.31099220633236524445967265953445007243e-2', '1.7160044747e+6', '22496.68492363164434927557667458365943093619102521');
    T('-4.71244448219168433e+17', '-3.661845152473087014511441938e+27', '1.725624198341216588965585001399981214121943154e+45');
    T('-1.6192523166690470927016735287431037161e+8', '2.1063669951791069077512605999904333513315096716248026e+52', '-3.41073963669898840947516853657748402318394608095484774905332123128295765265305433698894186e+60');
    T('-7.9546255726227721651851197010394795e+32', '-1.6352825346150572721595214349862e+4', '1.30080602683123181981363225429281630656820439050469581096187376829e+37');
    T('-4.65033001e+1', '-1.78402957430342048751447188050189199e+35', '8.2963262681107211387373788951990821828756199e+36');
    T('-1.0747111621539139354559268707283340006236738352646385975e+55', '3.4569644402407395181922991072e+11', '-3.715238271095879729064413458963542564475178869110977001563130120111735883029910152e+66');
    T('6.3652937877835233734658553177768102e+23', '-2.739191316024006563208208745870453e+33', '-1.74357574674381829401359393298451323167803405210747026007699767690206e+57');
    T('-1.38596e+1', '-2.01668654e+0', '27.950468769784');
    T('3.1e+1', '3.99532408029111361202961444968519230785160272779405533506e+35', '1.2385504648902452197291804794024096154339968456161571538686e+37');
    T('8.752130293411895486461455570456266543110235191058256368242e-9', '4e+0', '3.5008521173647581945845822281825066172440940764233025472968e-8');
    T('1.7666567718645722011538689771389140608479155e+5', '5.79849364509293864972181107177185722325631e+34', '1.0243948064717127403933357900393416605736130527033851840351793404944700646104585721805e+40');
    T('-1.88074285551825927e-1', '-3.5892420997049074284601888671967281230296723689451e+31', '6.750441435745360244959256403222145003141120524314248902529158196077e+30');
    T('-1.47443449855473683902404463491959178173791276282416e+11', '-1e+0', '147443449855.473683902404463491959178173791276282416');
    T('3.6674516424397608e+15', '-1.621851622618484198437145865398587268422606620129963582e+14', '-5.9480623971657509801533671437416378775411584549470454175597505327911856e+29');
    T('1.852799859644936708675646349852062e+33', '-4.07939656803717490972435529358637164801722794978e+42', '-7.55830538869531417546824622551390200682044219768001754254044667676246763456544636e+75');
    T('-3.6242110037e+0', '-1.1278795340499000045446830565915076472512673379718862044e+0', '4.08767341815167642135552034202789183114074114504811682323070735628');
    T('1.4274012216017920060904e-5', '7.3181715e-4', '1.04459669389914186078985917036e-8');
    T('-2.3756329516520898291145904664793103180184540970355072e+42', '-2.270259567e+3', '5.3933034361686053900907941458116470570432078963452065594973824e+45');
    T('4.929295626989455124178205785e+23', '-2.09905e+2', '-1.034683798583221577840626285300425e+26');
    T('2e+0', '-7.18e+2', '-1436');
    T('-6.31146996107278795852155517509682305e+35', '0e+0', '0');
    T('5.62374544362317131178832531913e+25', '-5.028617311364517574605216817082040721294634e+10', '-2.827966369251080786848400856170544022762054465978646727405142173480654842e+36');
    T('1.117412077388384866590003439477599421523143e+1', '-1.61338414788659224331375974470976667613219454556537612e-14', '-1.80281493231544618608867814708290945943864723087524854355671733079894613280168308370993407954516e-13');
    T('-4.2815998483904942107791375e-20', '6.9376170311235847480494e+17', '-0.0297041000286500510137199365646358515056073393925');
    T('-5.86940489151059825558e+20', '1.186355039910488859404215239846695660365360637322362023701281e+45', '-6.96319807431887432746494916572974977870149668248845809245935171579248369361139798e+65');
    T('-1.18405412e+8', '-2.8902735e+7', '3422240245601820');
    T('8.642504975809385471415e+17', '7.5109001974465513712091271541148798280521311221e+5', '6.49129923292395160193292467556975546756516371487956307369563014247715e+23');
    T('1.9923555553060256693196663e+21', '-7.20338812908556886039013517835885875517875e+8', '-1.4351710356009111862906055289239352924294602722052150633824346851125e+30');
    T('2.47767224522360228025008764692368247199028078613e+13', '0e+0', '0');
    T('9.159951983506971778872e-16', '4.35572412991174861535348709e-17', '3.989822388339430055462173193590823971110558676248e-32');
    T('-1.2584903289619528522717972e+12', '-1.9048067354292e-12', '2.39718085507923740095024952130919735824');
    T('4.8222e+3', '-3.915307e+6', '-18880393415.4');
    T('-1.145743e+0', '-1.40499568633e+6', '1609763.97264279319');
    T('2.2428057847026326379001011260518421703171e+33', '-5.8239094523813465985489463975171171984367780708670099e-3', '-1.306189780938522558635234193371919783301088290650039938567007189410557075386960930089941183929e+31');
    T('-6.26193904e-2', '-5e+0', '0.313096952');
    T('3.0019845694844821786908363e-13', '2.59803721322e+2', '7.799267625032905531560694408803215886e-11');
    T('-4.6771111417557737587832171805566392575e+17', '-2.41963969908791889847485e-18', '1.1316923795638693208841937864685962710236300920790258386423875');
    T('1.368790147448792225307101642172296e+32', '2.382834531191820192940003254735240956141547e+34', '3.261600429296125258884096700972863323244136904532628866746657143899537981912e+66');
    T('2.5158133090673533e+6', '-4.50742632133e-15', '-1.1339843128842514618218435889e-8');
    T('5.45290152125555300060203251343e+25', '-2.3624677987030823546e+17', '-1.2882304253465295333570852578315749764228120522278e+43');
    T('9.10645280474914206117691354111985771303355e-16', '-7.46868647274590839548386658906247692173836809486984638e-1', '-6.8013240877528954266058199752847353931423569799021123026453827361109324522139420649351012286049e-16');
    T('-2.8051865003118537531e-11', '5.348454627859893296622387289464589021238848335043530157e+29', '-15003412719603032216.1087879676948803801286988008339781735642916313334822367');
    T('-1.7561380718435796202851095012247e-5', '6.128268306422028776297429697021742180071567401e+35', '-1.07620852873801007780502248253390491162313640920758504239716098138490580960047e+31');
    T('-4.378710689215449958543087541470086791901689639419136544e+54', '-1.9783905014509610108940939605033592e+20', '8.6627996361456371392438623941839474271341962222925165084899855051115783079519792754786048e+74');
    T('1.121716338034368180915e+5', '-8.734522008301422444882962812919703e+24', '-9.797656041632466817652476303536047619163255652072068245e+29');
    T('-4.8582993368534106923769355421057e+16', '6.98715048402856908663443073960052421774798629769213e+50', '-3.3945668563050984730942842784405126039162330280795884968022377310599078522350518141e+67');
    T('-1.147697695159307927e+18', '-3.14666e+5', '3.61141442944998788157382e+23');
    T('5.21292708738338107676256190382983420967375413562063935697e+56', '2.0716e+0', '1.0799099754223412238621323239973884548760149067351716491899052e+57');
    T('2.40767713664497193342119078614448816473118895591100004e+6', '-1.9484666614e-19', '-4.691278632167740060209890691091392232345592973376458043474066456e-13');
    T('8.84323002496195395098550834342755e-16', '2.074843038583988890307904822332637330993388021856189e-20', '1.834831425588922445909395217500126101937521988613705732420133880473831365442944060695e-35');
    T('1.4139590392221052468075225255352223739775768735149e+28', '-8.5071158697e+10', '-1.202871338167213628815991013832091374398315253263031540940853e+39');
    T('7.60905256044027294859360806368103238783888705635835726e+22', '7.626236267e+6', '5.802843259393881904794360045962793471613853022211697708475474842e+29');
    T('-6.49983097039610238828951299410887525e+23', '4.100424999934130974333860679470965920172048633176651e+51', '-2.665206940635830060248743820457885128628819326617038782969842356414737979985236017178775e+75');
    T('-6.44091388086048854999712e-2', '-6.4903080679715410063843348214810202569974e+22', '4.1803515326058717691453471440857864505008045873090454741429847488e+21');
    T('2.78029662683945916043494079823001566462283856388915e+47', '1.72292937157713197163667918785093807960421637821735095106e+24', '4.790254720078529163251026200113350912677847102429264772360101417505440468715765792439145764556851324914999e+71');
    T('-4.4878498622339775396888494885583054208940416614412529627e+46', '1.724e+2', '-7.7370531624913772784235765182745185456213278243247201076948e+48');
    T('1.352549686604181695307046110709197982598092383194e+48', '-2.3555586703119991494894059087716726417170242847945e-13', '-3.186010141308257402498477975479367405804581618494048262835305948601981051698397443353715881543633e+35');
    T('-2.019515564917190266675805443119046e-17', '-1.4788018868472268212026912165159431467984470085815808703e+27', '29864634279.168841526151723443673432805367034874393502449296552705312364840770199191857338');
    T('-2.1655275560855920418985633841581563513185032946266e+30', '-5.9625533125793712547572194147395797672923522437e+21', '1.291207350302005700228376617552289919885062421326450164415825007449363425520444307151212866370242e+52');
    T('1.012506744028479038160052533215313925528515964937e-1', '-2.212147e+0', '-0.2239813756282367818828645731194657054416130006287489739');
    T('1.433496071288945453514799813e+23', '1.3979267765863662e+5', '2.0039225421861753263084300184572426981095206e+28');
    T('-1.0851e+3', '8.013590881173773221229825407826356e-3', '-8.6955474651616613223564835500323788956');
    T('1.0544140083165e-20', '2.7639893671307065522219750157782938300627387865356111e-13', '2.91438910754047439029710260584090912925195329258292696061880971315e-33');
    T('6.9910144713252743945138327844029390845842716601381876e+52', '-8.482563824399426864364409762208780384877e-12', '-5.93017264503166569047842082409955073105581203136314428395603709690554785251717257461332289252e+41');
    T('-5.359798702e+9', '-4.932148432974659892667336636982770197907445436167e+48', '2.6435322769128916091609850224697296903108609164903710455234e+58');
    T('-1.71427280396517551988450013042709589140000549e+38', '2.403519034309521273463675792599073e+22', '-4.12028731432945393654068892074968286875285779764235024381198392132877356891077e+60');
    T('-3.1889196169851645e+16', '6.3303265668790672589901e+16', '-2.018690257104300648907301509647095237145e+33');
    T('-1.23339224241833842568898143251348600283e+19', '3.077450385078067e+0', '-37957034313826161513.8572222562474073059332490133292961');
    T('2.92677040682294450913892109671419035013083833e+15', '-1.8864e-13', '-552.1059695430802522039660756841648676486813425712');
    T('-8.6400999e+3', '-3e+0', '25920.2997');
    T('2.8174216976e+7', '-3.80064379799368614537758774048216223e+5', '-10708016301316.282693791323644782201752565201648');
    T('1.025429e+3', '-3.7942691388774640999508942702866141989404398978535327058969e-11', '-3.8907536088099791345485455606857325114052963440160501890751522701e-8');
    T('7.5475945189824540258952479977984445146259416e+1', '2.241307785984759464484472e-1', '16.9164823608512696233027083957170305568142612592815464214011615788352');
    T('-2.390294178360939572894052478069342172015510960221e-5', '1.0258531018946647560644677321558488928826911786e+6', '-24.520906973123289161325866068624287873503133376117055452224336029773591730793542320906922064706');
    T('4.13335e+0', '-4.9088577675430835705980477517385482283e+2', '-2029.0027253474204476531440674648528319443805');
    T('-9.49e+0', '3.11206e+1', '-295.334494');
    T('-3.08616078174107176e+14', '2.076558336804330489690054e+22', '-6.40859290004299236964222747218844417227504e+36');
    T('5.34715396168270921229718951112033768e+33', '1.185276582214392362269858422207e+30', '6.33785637227742951291731320592812398407062758490420416723385085976e+63');
    T('3.09857552788127670265426890018069173e+27', '-2.775660244927720047697549780961273164125e+26', '-8.60059290864598393204134197688911975222918946093511063812459672352832018625e+53');
    T('-2.152994e+4', '2.65842427162e+2', '-5723571.50625223028');
    T('5.330956358309028663608422004436504723721138561972562601791e+36', '-2.7e+0', '-1.43935821674343773917427394119785627540470741173259190248357e+37');
    T('-9.47262472923053e-20', '1.7815435405588685078649855417813049031471e-4', '-1.6875893398498851540266228993423166652387013603958400963e-23');
    T('-3.7245117604208154109238688461979388885871895e-5', '1.099513130741858342635426870578056147376018748384434822302e-13', '-4.095149586185160991329081422183979955901725647516234149636345755464052206534381389581417571266100229e-18');
    T('1.0834155200334824535073546679153022e+1', '-3.2878072e+3', '-35620.6134735782785171514593012553956333584');
    T('4.1364165902025092463453669062207235153719556580492365e+19', '6.95e+0', '287480953019074392621.00299998234028431835091823442193675');
    T('8.42371582961533136243e+0', '-4.7071941235449e-10', '-3.9652065651577439895485473058278107e-9');
    T('-7.45065795641536747833628483446224560146193991951403486e+17', '1.27523917338709350722670736519147669728e+36', '-9.5013708935291045870600560683168551870317994342017416888060151292391323390390803175995871808e+53');
    T('2.05416636316876816247008603104084729112e+37', '-2.67555245581664e+14', '-5.4960298576321336552408203766121162314254132314202368e+51');
    T('6.02518139223983254774103596e+1', '-5.541610436514578623372748271926826526e+28', '-3.338920808512969500784406497155751094267976654888337866644787496e+30');
    T('-1.886901000866449319960349324191903886e+27', '2.9119588739125107881913973682892608e-14', '-54945781136674.553048031354602674611785517327046962179124845815195874688');
    T('-3.167630633483217259922541256223568e+20', '-1.976822247538314258143070135525568198864845149591441e-1', '62618427082535077150.84042450833457702256004904708731141210786339513630567393755281488');
    T('-4.342121805414178979168499106872259725865474348828e+4', '2.87800764291519653e+5', '-12496659742.45073888635509104978249087676725149425279111442739516684');
    T('3.48934964368845845248178606547230673370875533672897e+45', '3.60994798002748312248366391691804980174190472460657e+19', '1.25963706978427685634963685436483508162346103715778631358243181904380097203373116193801232752539713329e+65');
    T('5.42527560030712498895e+17', '3.3118136084722e+4', '1.796750156280932065622648405138219e+22');
    T('1.1270004847878401285903765398526874938853992e+30', '2.0458381647e+0', '2.30566060341436511755234793837415859085314757145484824e+30');
    T('6.25593016708055943396626471164327645453174090870769922934835e+33', '5e+0', '3.127965083540279716983132355821638227265870454353849614674175e+34');
    T('2.3033584574555686865161618899734095161045e+20', '1.3010709745e+8', '2.99683283286453354141726776018746569820565525883525e+28');
    T('1.34211871248298281139772721139220298063196737768440691e-20', '-1.478359144e+9', '-1.98413347093272458362465744377928424672132387150944450161528504e-11');
    T('-6e+0', '1.05671309643762e-9', '-6.34027857862572e-9');
    T('6.4756599396864465846766547296e+24', '2.94968462813429622e+16', '1.9101154581118175282669146567918601241230402112e+41');
    T('-3.091e+1', '5.667830565788117917508080275569794124182460349e+18', '-175192642788510724830.17476131786233637847984938759');
    T('1e+0', '-4.6172103778523998784e+19', '-46172103778523998784');
    T('-2.5570672065082385679679091913879025e+1', '-1.2063049743853270960595806834829465677969295004031501739e+29', '3.08460289104848063763132960929740754910194979655799625586510645610283250550119127523124475e+30');
    T('-3.70293468184958402068082830264510960391603268883687825303e+45', '1.411163039553177355119999727715e+20', '-5.22544456090573674099746706657675074188431054961983786246940351702162084606420487372645e+65');
    T('1.74787882197705299292198556022976167231705195e+35', '7.7333671996220816e+0', '1.351698875079142520032926258655417527276870238724265543433912e+36');
    T('-4.096618331817365021e+11', '-1.013711236829518925258935118422711554106672e+5', '41527880359650616.57506268237434302892786107960903774595520112');
    T('2.12048782119154454967332210439e+28', '-9.2197418205e+4', '-1.9550350244900609425611967419011549641995e+33');
    T('-1.1505865830923532120744213733257299415660413583393e+16', '-8.46565236265629547979844640868246149886017535754876e-14', '974.0466025596414007536363196582134432561026804409558931080223917261617484174979797491453160432374268');
    T('9.07420636404752362993662380875781571271e-14', '2.51474955483770475273773727474896143461917754897697324764e-9', '2.28193564144939774816732185434917998866319599098823390828301213887829237747907197261589099255044e-22');
    T('1.246260472438897833535289124112549387151e+19', '-6.680412909470472488789414671059720951e+11', '-8.325534548643583067029478564395020366314784333266559701669016772756124900601e+30');
    T('4.0294278740792929940247525071574944742596836e+33', '6.00085175738377949776908065147e+14', '2.4179999339419911926322112001876929096606969209945947707543711809044074892e+48');
    T('1.55945832e+8', '1.1527307e+3', '179763548083.4424');
    T('-3.211101384979001257677085706325437228950660943386036671e+5', '9.228686729748967e+7', '-29634248739434.237821402397860319787760148942989060826624350048886368857');
    T('7.1768007105380238999489810862e+28', '-3.15802039762103328762026415391128e+24', '-2.2664483033540204459973246271168430597758318255590370472832336e+53');
    T('-8.146245193307731671e+1', '2.43845e+5', '-19864211.59162123829314995');
    T('-1.7260614660794362658661235883950941802e+20', '-6.765975e+4', '1.1678488727956813788943545545991497345878695e+25');
    T('-1.0423339556844604967203052423172210806005595277e-15', '-1.65790186e-3', '1.728087403870424630609117961005471539558877558014551522e-18');
    T('3.3874450138496810026558092372911790009595011319748e-7', '-1.844083119777027949884472374398549549372684360122809715262e+1', '-0.0000062467301692130573949905677401149904072205407708817196623282883165369601854286961699466138329393918717593976');
    T('-4.5677632943648704446515762998538554442851348e-17', '3.582872388623075466216438867772511708e+23', '-16365712.9851458715575421433474376658852503648131234292126180950207040799233582384');
    T('-2.86829101569688038860522495486e+2', '3.7e+0', '-1061.2676758078457437839332332982');
    T('1.06077027679826318938182294804181993899494704147284057e+37', '-3.261253544293047623648926725904240616776040078855e+31', '-3.45944082488905300826935851931788717711099607836502248838085413796777056450136214579772327128364314735e+68');
    T('-2.18507353576174076070959256350000774887e+0', '-1.59831147321681301466931141149354431e+1', '34.924281020304184328565238680515232418857862826204480512208558649806974297');
    T('-1.99647745343238059689196134517981116780466772090884720632224e-1', '8.6467613748664636508931985451006098243078232494163917e+32', '-1.7263064130130867409299452087506032624045135897393310593388261234352860098403285199668529693824805691038528261408e+32');
    T('-6e+0', '-3.4876006775596915566721213748733893e+7', '209256040.653581493400327282492403358');
    T('5.28124391735772273591469590185765758579874870468433745e+33', '-2.147e+0', '-1.133883069056703071400885210128839083670991346895727250515e+34');
    T('6.85038e+0', '-1.5901255569632960384981532502369e-9', '-1.0892964312910223916206979062357855022e-8');
    T('7.983812710598619216930700499e-18', '-1.335137072858e+8', '-1.0659484332675135134229403579270339956142e-9');
    T('-1.960334206940101e+2', '-2.959228e-9', '5.801075874534941202028e-7');
    T('1e+0', '-9.164e+1', '-91.64');
    T('2.027899432561029386047096809176e+12', '1.40213651157523255556127427614979949106e-5', '28433918.3619651531093388731497850317019747168257172892378111473796656');
    T('-8.3594749784821954269345060852858640828490521460887e-13', '-7.164172033751721e-15', '5.98887168576894141563842978474039341215235493797906672056370436527e-27');
    T('4.550077739716154666542802337e+11', '-2.7306574188036000270848865e+19', '-1.24247035360890335495046606524591731837703486895797505e+31');
    T('9.4633378246343026774101810918412358e+19', '1.578450860979302005254127126947731754671912e+27', '1.49374137370320099547200047736473092846440961340814649863546698920428216288496e+47');
    T('-5.2190603156319182229289905569617150480323e+40', '4.04355690292489982440257971296300398805279069834596e+48', '-2.1103567366054849391298527839468485796895719049012578983409941699614247722561603581562654508e+89');
    T('-5.13097206174328014241389093854003358779365258633983078e+53', '-1.59332306217979e+12', '8.1752961173757536414155211183332535775263410017806193494494865359362e+65');
    T('-3.00025764484794075488331e+16', '1.21837943722606727484247960651563238163989e+31', '-3.6554522208630400771401722295434531318286045235250273221033912359e+47');
    T('-8.034506919e+9', '9.59917303867585553e-16', '-0.000007712462219591941585402941207');
    T('-1.03623135167581303e+11', '-6.0727847191820070012464062e-17', '0.000006292809917994193771401113453514323750632786');
    T('-1.776830835655537305590797996218970905e+19', '1.4297825967432706650609604367607227448e+19', '-2.540481806177089727480969780595820579837949700040264880146150148682940044e+38');
    T('-7.06672750227869765640275129774849369371986497683548567624e+47', '0e+0', '0');
    T('1.201486e+6', '-6.4618819204288244e+10', '-77638606610483465.130584');
    T('-8.392439e+1', '-4.559646760722e+0', '382.66557300906980958');
    T('-4.892684791905e+0', '1.325742128654573684900693679335408e+0', '-6.48643835085599458713483949506930948917827224');
    T('-5.583623025078921446646733308770747651954925640892627777043e+48', '-2.49487928392226886507353963712136114e+9', '1.393046541450079222756134578078800193311267254493670540053212261265672987328881907800860430902e+58');
    T('-2.96526320854720999501457709614986308646e+12', '1.58934709e+3', '-4712832451588571.3331953326153464351003636194014');
    T('-2.278047134327102481e-9', '6.53948173493481078043032097446045568663006609238767572331335e+59', '-1.489724762625267407514295696211403251163397687148157704642933789435816632542135e+51');
    T('-4.34649668141838505956e+10', '2.7566141566303188003001360098750592488e+19', '-1.1981614283744620987582158070935888399955230644767716858528e+30');
    T('-3e-10', '3.285427839965717144526763107629871339325200898862235e+18', '-985628351.9897151433580289322889614017975602696586705');
    T('2.6431444802225494336841837930134668e+25', '1.5757672131509605631302369006921e-6', '41649804115556309200.4022799068651144359462695399494757742954037228');
    T('5.4197814884600979459047471764155193322e+12', '-3.45317672489168691123605172192567891610375269351773e-14', '-0.18715463289949233044577799707655967434015297575177174844144791767778460868554918438459906');
    T('2.1804041368605641467299918902653637965488411876504e-18', '1.5595945850567196835592942943641516156866395747026257882e+46', '3.40054648508300657595934821590957250693570708065275875065240265811212908037363934338779869619908640604528e+28');
    T('1.6644802913995172764608435e+25', '-1.64481887574951745057302561e+14', '-2.737768601606983206750201541817125086018588487702035e+39');
    T('1.345297580765936668864521475544600213e+34', '5.255894981691269736920878216903973555596e+1', '7.070742803629092178385260348541240791936901232154070386845812623023948941948e+35');
    T('-1.236303590387333100113303929204537712340659855471201673025413e+21', '-7.9e+2', '9.7667983640599314908951010407158479274912128582224932169007627e+23');
    T('1.8325580806128635358765e+2', '-3.171748e+0', '-581.2412427067688694189217122');
    T('-1.0243e+2', '-3.409589101402e-3', '0.34924421165660686');
    T('-1.273997601117917175698667546852761764039275631891486e-10', '7.856245279828564906705161224850856859963860198e+2', '-1.0008837640295551637408536690012436719563557239798756225068083188519724150459019497133976810474228e-7');
    T('2e+0', '2.0782554146127983844e+17', '415651082922559676.88');
    T('-1.0017426e+1', '1.05631201445e+2', '-1058.15274376638057');
    T('1.23321e+0', '2.76e+2', '340.36596');
    T('6.5482994824671797204789687e+6', '-1.810350618538593530488926394601031399259372856417988086e+12', '-11854718018460410708.6469761335753156476928972597617168054353701088346263301669082');
    T('-1.8464263e+0', '1.2578353631044943640519616250643946541369931149527985242e-12', '-2.32250029550618804198731651110963748297794789036777045368406646e-12');
    T('2.306905117986512123647e-15', '1.92224194054550245e-13', '4.43442977065274435217191058558414143515e-28');
    T('-7.7660157326825169689845327710896583341071831411984679e+23', '3.6431726508659866242336616738158288e+18', '-2.829293612350392270265658763343483917211646344882471424632373955590786112801497332869552e+42');
    T('-1.804410431608216174757910743628838673593296632e-7', '-2.708299883920527521842766517861172987221865596e+20', '48868845624695.20830931105195115261702836652954741973993651473991827943522861106002863472672');
    T('-1.800090865364229e+5', '-5.8264534155111055348482827550238e+10', '10488145570731753.6804095274061076304171665636502');
    T('-3.60855354781654573978597356957428456e-15', '1.866546962201332812e-19', '-6.73553466261781541696603311041890681571157507335298272e-34');
    T('-4.513629265554556014503659605626189283778885640833465154e+54', '-1.6679267957795545731497e-20', '7.5284031982332348463844383049275530663291324432825482209928639026696189755538e+34');
    T('1.0941972e-2', '3.45343163878253161341020408675072450114411418654261458e+9', '37787352.29547257500304927763151199847123286539395206554115176');
    T('1.24194883524432e-3', '0e+0', '0');
    T('4.5115e-19', '-4.72349600277e+11', '-2.1310052216496855e-7');
    T('5.7379616e+1', '-1.014091e-3', '-0.058188152169056');
    T('3.665008e-19', '1.20318611886502374360861219376e+18', '0.440968675112926294051551255902795008');
    T('3.33610310938677239000000947724228838577011e+13', '-6.096671817812092833664313029588e+1', '-2033912580831362.881020502005617192477576549025176845225368255784259601468');
    T('2.583458940580655231541895311105492814e+17', '-1.5156221815587830068151379463e+26', '-3.9155476754903950432396966951972758516587513180224498790533678882e+43');
    T('3.702736747441730847035655529e+20', '1.0018677975945079169067e+10', '3.7096527102316985801620293314995993182468964321443e+30');
    T('-2.83630154283079925e+17', '6.41074464694029283507262773575118086890104876e+10', '-1.818280493281103999466755581106175564517622369590252755402143e+28');
    T('4.4674110594344555786220486754e+28', '-6.727853004096336279764957767621729768e+4', '-3.00560849167492982673836524473329588821711484064022839144411493072e+33');
    T('-1.5688404306246196345593833419224858e+19', '2.63605019e+6', '-4.135542115227710406357993024757603658362302e+25');
    T('0e+0', '-2.64954402306017667049973249287275628634791783589648e+50', '0');
    T('-3.5009076896538e+3', '-6.6430229e+3', '23256609.95315628647202');
    T('-1.521924675204764636287003e-12', '3.7693609744915860629583e-20', '-5.7366834768326222380785330790588380575060209749e-32');
    T('1.11236129544558345600503719108591547020411211e+44', '-3.1253884214607362312987e+14', '-3.476561113266691720093444060759595557505158595083676675212825697257e+58');
    T('1.33815573609017992048200249790557568805710357380887861899e+52', '-2.5e+0', '-3.345389340225449801205006244763939220142758934522196547475e+52');
    T('1.611548475376932804e-7', '4.9802375112902234248e+15', '802589416.83347697323623227532963471392');
    T('4.166575179521503556226149051605877512974975774232e+25', '6.399872460241347446318418520381529620092936e+33', '2.6665549744944818866755013936324316057322003893851265592949522620090738985837783100394025152e+59');
    T('2.35506389265e+0', '4.006712248230173437693154988324e-9', '9.4360633440453853293582738230620847394186e-9');
    T('-7.242328039718679745240983432e-1', '-6.92464621048019340876524126396387834446077132758404786e+24', '5.015055941529240335287534191035479214939020902123532657328599638598099582175505552e+24');
    T('-4.193894403896958727143737666044e+30', '3.261588840189662e-1', '-1.3678759184684195494824524833564292922177237128e+30');
    T('1.5765195258987684156516463952714058260238e+39', '-6.4745770126610248e+16', '-1.020729708239542312739313896297935609020156882675181719024e+56');
    T('-2.040489518370981132782249800266e+7', '-1.174334689904e+12', '23962176258085484397.98077967950839046714464');
    T('-2.999e+0', '-2.745829033902881255806415521483615257e+36', '8.234741272674740886163440148929362155743e+36');
    T('-1.421995402238039424217983713e-10', '-6.62685949e+4', '0.00000942336372605751879737308119715948637');
    T('-2.91998097054133242068815e+15', '-2.008599636825475699484477749463054e+31', '5.8650727169666203572881875569551215932356120857036806101e+46');
    T('2.6e+0', '7.4773359790825494286810609654593232269035859893021495060685e-13', '1.94410735456146285145707585101942403899493235721855887157781e-12');
    T('7.50302677e+7', '-9.16389319e+6', '-687569359219906.963');
    T('-3.4496300003784699749429090893221619407742937e+17', '2e+0', '-689926000075693994.98858181786443238815485874');
    T('5.58909651552271e+14', '-2.9302356759838764984959768e+18', '-1.6377370006301816824127798758929684033128e+33');
    T('-3.21467608558886180085443357632011447658817714543954802587541e+59', '-6.05654450954627843543147e+21', '1.94698287961429431939792169536178895975298009949974623312844888292490502665078131527e+81');
    T('1.4473044599244751554764848267885956615840337279448e+31', '3.2978760548379995995551379e-19', '4773030722445.16982161490157947831158781808944842335126216028865139564758792');
    T('-1.52212585961814532229294847288693e+27', '5.4807677063154665054707604339457540829262049e-20', '-83424182.56342800099858567431320449627410280372645389871308978482983651711957');
    T('-9.90550793992759173966e+6', '3.335265462233194052458441615728696e+23', '-3.303749851791717304829547839188687665630386634122328336e+30');
    T('2.51287438279233642085429923935090523723e+30', '-9.451335136867210002148028438967398611467216454584204034e+38', '-2.375001794861871280485178924285492901455882867966860732042873907744155803603055976356149298582e+69');
    T('3.4082333651834518484189e+13', '4.35506385179170328272063457227625384554944253e+44', '1.4843073927180842671942017049002743738759608944705687993398956915817e+58');
    T('6.5818547070316952187958008e-6', '-3.73951282e+8', '-2461.2930056322368417019602053766256');
    T('1.97206174719719816981292592386e+16', '-9.0105789e+2', '-17769417968792207968.034967276795922554');
    T('3.6619571048211390692219928266376703435e+7', '-2.4799589894315235042761055970638006e+26', '-9.0815034410138196343173744241842493541845089867523197724164419200175061e+33');
    T('-7.84666331967208510490549760574105355103150267368e+19', '1.619717764380338390313718e+9', '-1.270937996998447419567301371334187807713815536655396765563064753098154224e+29');
    T('9.483006538328174932945681176608e+30', '-5.3373488401022043702090517246506626e+23', '-5.06141139480275047286128600549610642611475683804095134453348204608e+54');
    T('-1.4803912145480433328776377901902060963e+8', '-1.582058e+1', '2342064764.1054483391257298870727370763001854');
    T('8.9882523984348881655495577517148277303020468059964733526e-10', '-1.9880593317254807355650321572764e-8', '-1.786917905661241317492297819003818060408519912904334361445796515881479577099002479285864e-17');
    T('-1.28471301581497146597153639797443e+32', '1.42004323959128318e+2', '-1.8243480329229795026092654483949144472588365290874e+34');
    T('9.7783e+3', '-1.36173859052909811217858436668287617315269353636409624e+45', '-1.3315488459770680070315851512735168083938983206629042263592e+49');
    T('-4.67119137023742882426401e+14', '-1.212047464364320398948015745451394730100809445563379321597567e-2', '5661705655856.77098789712337972493426181265394901439758588512945491280590876618166367');
    T('4.21209472e-13', '-4.905686e-9', '-2.066321409857792e-21');
    T('-8.0534398142355474142562564558981481e+19', '-3.52823954947651480531133809737181713783475797e+44', '2.8414464861914654893810641177621530490552863751100871276411283589068922334715357e+64');
    T('4.1668071977029939763287789711116957000695558377e-7', '-1.3024216206228e+9', '-542.693978325508121622979460863479145783998910882101044971956');
    T('2.14936794322219777817392300235280096983947067201554505296068e-1', '5.56270924878341e+7', '11956308.9368006948399328359990270847329026003904275869850287225663085663188');
    T('4.4808187810392761874440073637915549517e+38', '-7.646754766255997951108787766099794e+20', '-3.42637223706414760467636032872081913139464887999833237904238953370499498e+59');
    T('2.6590306926035e-19', '6.065851175402e+9', '1.6129284452158934622539107e-9');
    T('-2.99e-3', '5.30673982e+7', '-158671.520618');
    T('2.7453398931708285228618465251335055626300456581132e+13', '-3.22074749108902263854425833e+12', '-8.842046573116551400389407780741144904662936292441998341089622224330041182956e+25');
    T('-1.927554010995289666334e+11', '6.691946123062138e+4', '-12899087590872802.405021000232368662092');
    T('3.1043442867282686296635094758709963529734603e-10', '-6.9061694467717526338079e-19', '-2.143912766526321799419146642311363281841178348205083711921322847637e-28');
    T('-9.9460755574581062016784211082305572914306783520097982603599e+8', '-1.69419433e+4', '16850584815197.112739421417524916526495882012852028744317145606339367');
    T('2.61564884146360410916221e+1', '-3.37673411255562774e+10', '-883235066943.67590562874468917360320357054');
    T('-6.63e+3', '-2.3664010008e+2', '1568923.8635304');
    T('2.6042340596970639861661053132244e-13', '-4.19696881552846756035609e+12', '-1.0929889136885679117123078859859216841502622841450076596');
    T('9.66932408746122723475707863151647060844849114776240053344e+35', '3.543132415517187808951809e+20', '3.425969561042502580998764203892614576219316524854247783921994540516844698885299296e+56');
    T('-8.75613931133e+11', '2.85579203853134364863246620418932562599339642887412e+50', '-2.50057129335675362001562521254298208477229393538495184240597796e+62');
    T('-2.50801681357e+11', '5.6482598133343504471083657127414336e-11', '-14.165930579254300605382428195360012246785733952');
    T('-2.30985517042381424317138251138e+7', '-1.356837662673868976569988292803227160100334665194252e-8', '0.313409849055299940654534970732121932438107278390194102752542104703069838330058776');
    T('5.862908424337665828006849479349960917695532524969888e+2', '-7.671778766584342308528805252661904591265896365900259438e+11', '-449789363602621.67756677207730170927431389968466555033513219445179870196178056661700171117319697232337802944');
    T('9.581282282850454426590480159624386252965047e+42', '-3e+0', '-2.8743846848551363279771440478873158758895141e+43');
    T('6.80255891e+1', '6.4939628006286648775556960482793713797e-10', '4.4175564510625077864220559174474627948177228127e-8');
    T('1.17e+1', '4.39880043917181857989519e+14', '5146596513831027.7384773723');
    T('5.18057e+5', '1.12095183227955985516792e+15', '580716943375251939888.72713144');
    T('7.9079109607883946443459466318246948653463050994769e+5', '3.36736267476382737799206797447753274243e-8', '0.026628804204714596632325780373652132822901517398195299752987744361561977288621298015434867');
    T('1.9192337832e+6', '4.3746589806622647519097525128325765043728672520610978379588e-6', '8.39599330566629402128776374017937235269096935960460880109100828973216');
    T('1.97054154322487963934e-16', '-5.29966794868061391983563001785802904827e+35', '-104432158581725291847.012347689094982662143786123082010509418');
    T('6.9435803780806824e+0', '1.312029461736620484829099495123927022691450102807689e+51', '9.1101820259781574883358006463573816901697004118371527017012130869736e+51');
    T('-2.02777011165205949332910260826373220997764e+19', '1.719346099690327466591195210611602452021664087e+46', '-3.48643863253758833917742037463014837753623268813161576588808665593545590271493786101468e+65');
    T('-1.35689510306015930639203912344e-5', '-1.7775524374291928790914680036598415373039480612e+32', '2.411952197780322048592902790431885908730809493500413918235779385964135474528e+27');
    T('-1.197093336042660715753664692387461089572e+10', '9.30322759e-19', '-1.113683175167722258780864100982789097855769169148e-8');
    T('-7.22954e+3', '-1.80621201849899035625232371577275379091004324e-18', '1.30580820362191907401404243961277544415357940053096e-14');
    T('-2.5487146811e+1', '1.23224022125341461664322693e+11', '-3140628742550.490076943920877369882023');
    T('-1.18509703682338023839102000456883783748380745862220202e+28', '-6.3419775864e-20', '751585884.5242932927360537668009595161218552675515150636252804528');
    T('7.1388416426585452586630469980942729444722426421412280025841e+11', '2.2273053664463959959973918980036896886955225978954e+4', '15900380300904382.68571628113627797346230258274544097276905843398058418502349718417824485312566047149642150314');
    T('-4.048812287518331136928265811389922096053793e+42', '1.883843038956722868136968070669521788203167379034479609892e+28', '-7.627326843883853713943674629620475969636818573439332541709592124635715493176124730990752735286920356e+70');
    T('-4.40496773173133229276988080861e+25', '2.6836361405552532182670933448535260065953975259e+0', '-1.1821330602853900620632833373905984586754469500681938106134905203144285417999e+26');
    T('-2.10698470492551613923273067e+22', '4.9829981981459258350705361168e+25', '-1.0499100988164872148563717105895284884047064402922062256e+48');
    T('4.153010660772414005280043833935812744967827327617095506753e-20', '2.527751586e-1', '1.0497779284442377486895243175380771288291439046358254767908369460258e-20');
    T('-1.775786834688476262481212524832086861722161695595273e+16', '-7.18e-17', '1.275014947306325956461510592829438366716512097437406014');
    T('-8.4572173659763154903842071410504969649015559593e-5', '1.55e+1', '-0.0013108686917263289010095521068628270295597411736915');
    T('2.1957687570313298348687272820162816911371646883313165861e+55', '-2.38569977592e-5', '-5.238445031621780511465933279321833876337532446936158618177349386712e+50');
    T('-1.27304464823058989292690796372853e+19', '7.469e+2', '-9.50837047763427591027107558108839057e+21');
    T('1.075014243303345850436753650150338343736e+25', '6.02230306841873236690335765889046842703055475561e+27', '6.47406157603958142830356266103466419129027532770065365087223464829450025112366565435896e+52');
    T('5.4644781816078472076480587668e-10', '5.533388289372726449619555652757e+18', '3023707957.76416324809553379608185627596205958651886704400676');
    T('-3.51050282698153075e-6', '-2.38e+2', '0.0008354996728216043185');
    T('6.8429855416436924582601339528425e+26', '-5.06163347121826253493439522309e-18', '-3463668466.0646345473598332283493482158871668931014551327133325');
    T('-6.6943286454175063790486144e+19', '3.4328180225062095992833728348740385020789e+37', '-2.298041202256879703336604670399729877443829865654374596865856447616e+57');
    T('3.4586536216937580446126883522083952059e+12', '-2.88703672447082268049428783480859803803275492e+13', '-9.985260023053895125434393341048712560458518029877932175692101389180863832497638028e+25');
    T('1.544220975398677252000528288503556128937132048384443331e+49', '0e+0', '0');
    T('7.7e+0', '5.629239278180308472085767973107775059451282267e-16', '4.33451424419883752350604133929298679577748734559e-15');
    T('1.072717719201024052696349931792614756e+25', '1.3771846282397089347926178e-3', '1.4773303533240107890345190437679728361254161112010424681482568e+22');
    T('-6.2982194473322433078922784904265197191653069665364692e+7', '-2.8678684613182319044997019428886e-19', '1.806246491546528554076417787115541085256514512890108233024447001160805124352749293112e-11');
    T('-1.732465708088e+1', '2.71524489294637185383888961456871107141931061365e+31', '-4.7040686660906618703715451371754514270759411014339610482012e+32');
    T('-8.96598237114090468243115825396177729e-12', '7.604366591101533e+15', '-68180.61679950920120868401395898211540605801952358557');
    T('-9.71839881931499436381251357007593505557676492579357e+50', '6.57868431363921445315728822689471152257263421455e-10', '-6.39342778663174158137496675720663924284622907149091698424887623612741936565068134530253299923904435e+41');
    T('-1.006931641466488693025e-9', '-3.4521295426580450451664697531545584755870088979e+4', '0.000034760584669436241976889349243420680611555496619415686043504166671475');
    T('6.00947607e+5', '6.812758675764029679963178704882229955116307e+38', '4.094111023268882532850848090812335308350862098327349e+44');
    T('-9.6836087377e-15', '-2.6459022649270745998e-12', '2.562188229174803984792301067246e-26');
    T('1.055921827168468904939466656863008144e+27', '6.32912299980432425349782865630544e+17', '6.68305912232736312925806512224733448892859379422335164811681167150336e+44');
    T('-9.98028905218791239e+3', '3.3600371506928e+6', '-33534141990.004018666701074203792');
    T('-1.45239016384017840495355131293445684596538291022094579841e+42', '-5.6775437718634921734497447e+2', '8.246008729026601881844137255200165069034156821222471237266712949036717046752165927e+44');
    T('-2.2979549681655763076566677319272798755304678418e+1', '-3.183497604578998e+15', '73155341365855197.813621009301952050894390937967417408666665164');
    T('4.0066264588364721690045277668336e+31', '6.737893728955e+12', '2.6996223291259444273370994565149283876986888e+44');
    T('5.846436541728802080043719344611e+21', '-1.2e-10', '-701572385007.45624960524632135332');
    T('-3.182300165687177420393599644356615e+0', '2.365832971505851366517267883747439591747414512839432822e-15', '-7.52879065721125810057489253493734512785942465676134438014645481107031765439734970381753e-15');
    T('3.825832626826099559783359922877e-15', '-6.5955112433918682619536836375146720871309857645017e+49', '-2.52333221055669854343885405756298648535866081882102237578852058574273365963353909e+35');
    T('-2.86423604077436742725253566687e-18', '-3.9616e+1', '1.1346957499131733999803645297872192e-16');
    T('-5.81865579e+4', '-3.0827768621936331602e-20', '1.7937617438481015688733727558e-15');
    T('-3.65098386509026325842574744310157864397940281992715898768e+16', '7.543511472508780470316703042149589376987868468e+20', '-2.754123867225285049343404492717016138666722719241254048568925571123034276751089068825616530843187247424e+37');
    T('4.71978532088277616359995233744199725174e+38', '-2.90845032007102932970578279149703516018735003308119692077e-12', '-1.37272611271880562032081502654149993248115310939203471275734361729098063870184524627400905246398e+27');
    T('6.66954264794400293153673670798603436474391677055481161e-10', '-6.0831130005762409912165773287576e-8', '-4.0571581589605851371281721138429286134755181825367445092430588795115847031409503355736e-17');
    T('-1.5359590725191838049406967873638064365696777077105943281364e+38', '-3.46671719279686986011120227659402959170189585e-3', '5.32473572413458873765078208479002392295654434125039963877442769432916923725014543243576464416487739394e+35');
    T('-3.8918273124023755760911999559263803895614e+13', '-5.9049071497197518967775767842381303529437097e+25', '2.298087892247939399387630879792186664312313395569266522629770751383705096713167192558e+39');
    T('1.6494930061e+4', '-1.4134663182e+6', '-23315028062.2881714102');
    T('-1.38082e+2', '1.086557106158916053738635237440757e+4', '-1500339.78332635446532338230856294608074');
    T('-3.236453714001566320662909602293056422741e-12', '-2.6292012409417075744241522002879275006552063e+30', '8509288121103316508.9776133643121846506311098158231648344441177311116642919353664683');
    T('-5.56185157584785e-9', '-5.3863586550733398733233e+22', '299581273738013.61300657427800794659905');
    T('-7.306228492885575e+1', '8.01710749999e-16', '-5.857481924695357768938364425e-14');
    T('9.078851355998902214e+6', '-3e+0', '-27236554.067996706642');
    T('-7.471572766864317095e+11', '-9.947035107615899737979837388819482785995e+39', '7.4319996621106228158954275728754194694522889053163705084525e+51');
    T('-1.265903731885e-16', '-1.207e+1', '1.527945804385195e-15');
    T('-4.726326821555207871753620716339193235463e+9', '-6.4126109518e+10', '303080951376910103067.328380769000717453065099436834');
    T('-1.91406e-20', '-6.456816473704e+1', '1.235873413965787824e-18');
    T('-3.1983053456730161e-8', '9e+0', '-2.87847481110571449e-7');
    T('-1.1250473182304781788249e-4', '-1.657377060472918137667968348296e+8', '18646.276171817696089408668926562813557672385751973704');
    T('1.1497865844851967254212601851139326807506796369434785706622e+39', '-7.76071990827718702947360345575041554086866713806786843198744e+45', '-8.923171636484296086155235679649166775015520367811462823386249811629379239580605875232547783752918665760034624822882768e+84');
    T('-3.3497708586119180439409399519615946383638197694153e+35', '-4.8561981458432843062333291774478231682e+38', '1.626715103259106287442319579436450524467374682092885927992940109533202075360145310755346e+74');
    T('1.06339694463513e+14', '3.012233219e+5', '32031996016130424203.8347');
    T('1.9633039492389670984077672122431784083773501182422065509489e+19', '1.6692272776747540826414430546718929e-7', '3277200506436.25462718284982490613239885158226875205709983451693837979539541036807585665417281');
    T('1.1467e-4', '6.77733085233686770034e-10', '7.771565288374686191979878e-14');
    T('-5.1701628249523227514579183256087778174687577e-13', '2.132619358400722076540485378725800253187225047660162663738e+9', '-0.00110259893265770873100902961300808852480005642266384867114383755051015481735901792658688929318256982826');
    T('2.18348342946585692342012771e+25', '-3.81634959382e-10', '-8332936099154723.6892526392312710267522');
    T('1.023307551757890323892037392256508719427002369e+26', '-5.20694036686411877720526e+13', '-5.32830139896505265680420021857733687673247020557939546162596921926094e+39');
    T('-2.8265491339447e+7', '-4.5434392049769450607387334481043271753316588900535258158e+55', '1.284225414995798036336141936613726795940619730566835387167617353958626e+63');
    T('-8.02970908483254769090703403995505109862035476919e-11', '2.3071404145426391103284923337981992716315980206711699715e+9', '-0.18525666346637359394264144605477583625489813085571289205234643919692091911432433571550028203557141378085');
    T('1.587431829883573713849409705849497e+25', '-1.49418e-12', '-23719088915754.3817175951099428620142746');
    T('1.6140449345603180298849693965438089354593840997322e+28', '-9.03841799979613578000119003816765e+23', '-1.458841278900975455540967582013269342009334677839779329905619862068104285214370333e+52');
    T('-3.723294e-8', '8.1668979393732624484790294912e+9', '-304.077620962808318348472796304080128');
    T('-2.145634432722793043073725435570720754433983080847542e+20', '1.465950677153e-4', '-31453942495727714.83031405298777297962982768524603399340055607926');
    T('2.278200008209245655472133211650862842e+14', '6.7270967926351916022156783304e-6', '1532567196.82058836270487268466918972063318028121075008094819589968');
    T('3.1343999e+5', '3.14336961975866116898902330896465119306450054556291e+20', '9.852577421834585592213077760716471803076251203562330547709e+25');
    T('-8.041439619908106697385565e+20', '7.78883706e+2', '-6.26334629272925752390308937810389e+23');
    T('1.77601311247551979976524852558057621908110590977554e+24', '-9.63987036475e+11', '-1.7120536170360171827608873268662828982548556336301315370428215e+36');
    T('9.807194626e+5', '-7.62599457122846035815911841262358376674896189395e-17', '-7.47896129768569306427921413491796612781212565774252219127e-11');
    T('-1.92719707950824442875770342240518e+16', '-1.70525973966481953e-11', '328637.15900850293995476091245818788722108232371654');
    T('1.16239377795254501916971749785697228868532854085e+35', '6.473102172409450243517628017455112739783779499e-12', '7.52429368925984729196132515935558757033934932864160658040770594113623888987378706054781403415e+23');
    T('-1.179784405848068e+16', '-4e+0', '47191376233922720');
    T('-7.538813675402315501713570566310033336449e+12', '3.41223043965380709597413486586383914131435e+12', '-2.572416950208617640223348424864689151961024519972222888201004983894402712962174315e+25');
    T('-2.519610125898322011290936135004574898e+10', '-1.363124864524128033329953790490685211016e+16', '3.434543211518771369514488568838182872952969792353237320906862717950506676368e+26');
    T('-4.4018359795112479954572037e+25', '8.064374150981933179268739012e+27', '-3.54980522900327467667853946121209296167777117106207444e+53');
    T('-5.5876311411746274e+7', '5.34197880056852641608623989794750613e+11', '-29849007101551382575.802997685240605484431232661965962');
    T('2.983561815076415871361124408351471429654035718756708688e-8', '1.118284411811800788510463e-16', '3.336470669476878476744869376687068610902363203935757551720993296308551931002544e-24');
    T('2.134734577507378691205480523075055640400370419357e+26', '-2.40845e-11', '-5141401493197646.20883383956580011775712227213650036665');
    T('3.47245322842069354627e-3', '-1.00754361885679357737028320286543060818544720573186927742e-9', '-3.4986480920739416255076813924095074265312312003153482496363088688338822362234e-12');
    T('-3.794245568368302794408011786555814e+20', '-8.37060859067434635093e-16', '317601.4454971178330815419624905170794066260320367580702');
    T('1.28940799891798555024002319648354556806250881774458599981e-9', '1.0028654150809960226369756911508906070980659188266e+9', '1.293102688043642049274122897660291144835301385402718280038596441757921696250413746340720089018990263022946');
    T('-4.27636406710816966834122598758048794168539e+7', '8.8982999317788742062040151406652152536669424005e+25', '-3.8052370086610255197545273472972163674292703635630344454164911934066209770042628022378695e+33');
    T('-2.948850630230385521369167273e+27', '6.672909985285753803977844488e+27', '-1.9677414815580527680894543930730406872388735650053041224e+55');
    T('-4.58435850212e-12', '-3.37588403664294220132546582786902937780785e+14', '1547.6262685555257703338148001260909269985470795177642');
    T('-2.5407577921477732e-3', '9.212688937755560725e+17', '-2340721120523603.242830965922612757');
    T('9.70814828823927437608576282702e-4', '-1.0450399929711279076844e+23', '-101454032189042387227.18166504580735273535431725952488');
    T('-8.986580647129736635012468025475589251e+25', '-2.32842674940416e+7', '2.092459476445462528863223485743121453574148915068416e+33');
    T('1.13e+1', '-4.694730618579590045444086579344431326591112917465804e+45', '-5.30504559899493675135181783465920739904795759673635852e+46');
    T('2.5304029888089e+11', '4.387914643257e-16', '0.000111031923279358510068465873');
    T('7.292170304772241302551375e+0', '-3.6216283825126026376967892815e+18', '-26409530945878724879.8331593964512856913252717030870625');
    T('9.49e+1', '-6.3604126376648187672456674994000370673e+26', '-6.0360315931439130101161384569306351768677e+28');
    T('-1.176852634451796072215996403677e+9', '2.44904391e+8', '-288216377737162735.922250619700705845707');
    T('1.373680713942569963597709e-10', '-4.5471428840104475371083297e+25', '-6246322483306348.1748793733532597161392782837366573');
    T('-5.3314956749576410979116539202451154407e+37', '2.70915269844828616659642472037869598192445e+24', '-1.4443835894576860174475218340998015353555909569592060809911322322844730795855115e+62');
    T('5.9e+0', '2.208687915206977e-18', '1.30312586997211643e-17');
    T('-4.933577839564e-2', '-5.81527983789613597327083109083065394990114741585e+20', '28690135739107706650.0330893037590591643386059592539244906894');
    T('-2.999683795083396322669132758034104118057863174352390023e+27', '-2.87555441825734267003450825531215971389862724174149604e+38', '8.62575399032701361118218619119383526371855243131272664024482478550553556850589314665513164176854330959000892e+65');
    T('4e+0', '4.73e-13', '1.892e-12');
    T('1.08754268552842900657636111617643472953987707185276968598108e+7', '1.81351345637832644608e-15', '1.9722732945916286331432129685647633013480203078627843626297182977144595945201664e-8');
    T('1.476869487377797869802075095495419497297018e+4', '9.16978415362210821336e+12', '135425744223249370.6870672636130364222488062879808291121723576048');
    T('-6.44960562766724813383113663723696894522318058842884424114e-4', '6.2987e+2', '-0.40624130966987695820562180336964496295277247572336761221668518');
    T('3.45e-18', '-4.5651228859493759221433089678212482448435184321088e+19', '-157.4967395652534693139441593898330644471013859077536');
    T('1.1116801887311179022882314500092e-16', '-2.3731060633715843891182383816509351972782343643683136984815e+27', '-263813499640.78831745125440250018216990705243508298365311908844491303220133333740292010298');
    T('-3.411622226349151970376832394365767151948e+39', '2.272882453750169129e+6', '-7.754216297093075438917262330382150382606746020237641813292e+45');
    T('4.813825375694300536878e-13', '-4.994458765e-10', '-2.404245234081581727695453283567e-22');
    T('-3.4004823042485333889969440789299923572353e+11', '-5.055670795945782666596715941784e+23', '1.71917190777197318977613475600295720724871164612058192437613842059897752e+35');
    T('-4.30182750049173941901359056714e-6', '-2.73059273e-2', '1.17465388985568150826129341738290608922e-7');
    T('-4.383659202779121282988e+9', '-3.26708101440047648993067e-10', '1.432176975500159562983467763399865447357044196');
    T('2.1032287098642227578833893517123706840232e+4', '-1.38738398481150077730631608976218467847e-19', '-2.917985828461377201803501253006512530189396739380234897663356402988476658020504e-15');
    T('-1.82989876256793e+1', '-1.42680066252682303444e+22', '2.6109007667889361627836979692295092e+23');
    T('-5.401957277401719742017451884201647908e-4', '5.57331798132165019229595829181742725699386949139427e-15', '-3.010682562847435019555936026646401099636466062637082424633510028806828395006935854868716e-18');
    T('-1.7025584579461e+1', '-1.89357506715785170663032403281086721920633943e-19', '3.223922246345454748113384654355437033006187500650177244723e-18');
    T('1.75701556278545512743452095698613449533241870186405314374e+21', '-8.0008069425919153153324522596175843489e+10', '-1.4057542312975910661097788666304204083938593139961296775572442278829856249685309185751666010886e+32');
    T('-6.0477171077679290199698548748670345744496299140266e+49', '-2.133215459449e-13', '1.29010836286647401512022142630193335625187354733582678886073434e+37');
    T('3.1064394721189e-1', '2.338713865381214106067869220656146181070789175e+10', '7265073065.4119709049662500151915998866163745003396324329075');
    T('6.6922115778644e+14', '-1.502476598814833512714989209432e-14', '-10.0548912900589540851343695771817446668970208');
    T('2.6e-17', '-2.671783935940821017261675012382562970940216397519591773e+33', '-69466382334461346.448803550321946637244445626335509386098');
    T('3.5201757425530436264024119194857657081987590263e+15', '9.331e-7', '3284675985.37624500779609056207216798232026204744053');
    T('-5.533255769459408437481792739718320122946890691e+22', '-3.531878e+2', '1.9542784320526756553356319177970861039193418399947698e+25');
    T('4.645259e+4', '-2.1e+0', '-97550.439');
    T('7.9497003230879251678395405634595e+18', '-6.1333445694821304513176871453320732006332281e+9', '-4.875825130562166374225780425406151552485642713795500618653851261127238861195e+28');
    T('-1.005369818933571484481949e+22', '3.135206365030571431738055544939894677892415929474640739e-9', '-31520418555301.66425121334176205932818570557123668182427124267161869460305520311');
    T('-4.60955142802279111987154280657676965775556e+21', '-6.999600895208299766764771673363791869163449812613584345e+21', '3.22650203020970252903982709059471354761827496584870916987798376422723879587819595266245394527082e+43');
    T('-4.42474356385592335769671439693513e-8', '1.265050947084194578462859951e-16', '-5.59752603606063023437875039568685608110784446235152882197863e-24');
    T('-7.077847732205e+4', '2.48584792112e+8', '-17594453071105.7057236696');
    T('-6.37752233724066550730707883629e+6', '-7.6039885222e-19', '4.8494606652452138136652467701959692830638e-12');
    T('-4.9433724780681818765776307e+5', '9.9959526923402244116455519e+25', '-4.941371743138620958161734043507096664065327269588333e+31');
    T('-7.687435480669860197725359437664161385731838770011635683114e-7', '-1.35484826677385443562865421591361401540341e+40', '1.041530863692139265267608718767277789101867106231335917307662291374878215844509357688744319363501874e+34');
    T('1.5012510295250038070878447757125272e+34', '2.88320380186540202497627762051674629931756151e+44', '4.328412675880839882083804634396863682183719498695681206471762208157714263548072e+78');
    T('-3.38921068317929605930700761538953746656360127100567655710865e+13', '5.69632435242926704290367713950973308658385274436e+8', '-1.9306043350107657372594071514185142475260849897600810020660411944811434813428061119686645756800301569194714e+22');
    T('-3.76914657349036275475693770766652109404782733181e+48', '3.827777748906093915789127662698600476674896035688672166e+4', '-1.442745538637205802306364313516282574241071243477898711033577246633708775227079459568580532954159340046e+53');
    T('6.662411223035412967621756635110157016806417500183356102e-11', '4.11713781541407721882458165080172938861902902e+10', '2.743006518819825052303708822086363491806643065891389452604894242568840422017902043029187832413208004');
    T('9.26006e+4', '1.782395040553430407053890679803958035e+8', '16505085019227.1987751434509284254396415821');
    T('1.669194958318640515063913971576976e-16', '-1.244248914671339151175490292104155e+12', '-0.000207689401526283965328871962139541102573486386336878117842659193528');
    T('3.16363014327124199446241593475618574106690619648e+25', '7.05008522459e+4', '2.23038621291441279938178011816995585372014349584271634674432e+30');
    T('-1.2499416265e-6', '-2.481016161522992373850612e-15', '3.101125376306835804917930031300418e-21');
    T('2.394272820993002468870139325593176e+14', '2.11414e-7', '50618279.4177414623953711635380955710864');
    T('1.1378282788937355438301626273e-8', '-2.48363251338640047864014818842048577888e-19', '-2.825947308110970660255163888048770257922851289911580643177307651424e-27');
    T('-1.117116020795727380964220670027372e+33', '1.2858e-9', '-1.4363877795391462664437949375211949176e+24');
    T('4.22729745775217473e+7', '2.7e+0', '114137031.35930871771');
    T('-3.209792098971668208636730400079e+8', '2.561617927875316631903780965290528364325200943605912675e+5', '-82222609854783.67957523073005932482022485150215647664198292591044805997494462687101325');
    T('2.791486045907779287369132431599698256688456376693553e+31', '3.01582028398776570554202910416394185546415e+41', '8.41862023971748410598194194885358670636908149176554482057465593277092965649017887260312762495e+72');
    T('-4.7987451889500086409222751881136004594238796502777e+3', '1.5763922061071358398417356638094106322793630666033e+48', '-7.56470451495490854118717493020497246765927486273345255064102022092233152887091090239341261644073641e+51');
    T('1.596e+0', '1.1278061910963898e-3', '0.0017999786809898381208');
    T('1.0019691597360265022444e+4', '-4.78747885958105888012393959e-8', '-0.0004796906170188423978033721353034376183189305015796');
    T('-7.840057154240753070926001e+13', '1.6771465194137118428e+11', '-1.31489245682394495937803684715730739761446428e+25');
    T('1.579380351053037416538973330388820879276958625730937001789542e+60', '-8.95594912974988579348053906659158935e+17', '-1.41448500805575195712136077006981122282508931148355054995764602589961835792946511370273959885777e+78');
    T('-8.6259000350458193370640181250303642061228049e+9', '2.50287016417625378022383686736285763674288500041793504864e+46', '-2.1589507836883083080681542896478167565436950513317074358030118564198671094847163184099349451694730336e+56');
    T('2.3565164149775678e-17', '3.29e-10', '7.752939005276198062e-27');

    T('-7.3082509513985065717321588249117283909294426665053038123171823802E45', '8.1139587436599483304895082011722489767478591E-41', '-592988.4670796104848902291110510024833882877779324139587779775769322761921828542989063538752670078960459222982');
    T('4.1580690407774531635854811718695276837491313289E8', '4.51151976050001428945887983E-3', '1875921.0642990819646935715021378546121515533349338737465486653798953306087');
    T('-5.94098742155317032555791061122172112467693488807700534435506921715E-24', '4.61016941217052394016E20', '-0.0027388958488934255950077406845338327546662445400484871083292926528860519832777944645744');
    T('3.7504E-12', '-7.88007879736459852E17', '-2955344.7521636190289408');
    T('-2.1186022052008407937878760699384240447915477894859451125820040738844629725E-19', '-7.216767917254229694821186934734207628E33', '1528946042391748.997410886033946108685159225196274137373381385049759221715038347354480872320019684962009305423');
    T('-5E-13', '-2.75399279758638331766833072E26', '137699639879319.165883416536');
    T('6E-5', '2.417496760754967572844203198215987E23', '14504980564529805437.065219189295922');
    T('-4.945015198370467209539041539486792258261028978257673527352148835976343131637595762491692863E23', '3.21738997057546245991016675193089492784008451753637062802134E-9', '-1591004230358037.215453223084728812766073228229224818741711464362077222232342773738670279322956619570773005741693064394880256558629265661028200068969642');
    T('8.944326872147001036632309667E-41', '1.34989853214019743744001484188796117541380712420727841534671603347856989127834147166127622e+48', '120739337.1569336009465330585026690676314127638673504378646869299090176172459431821906069831664699038568059542346321874');
    T('-1.06387569312217787644613114942E-25', '-3.49651047707433733724368394353972842316097364325909366154237667988339210449181953609691895611E35', '37198525073.064174722364904333744771229085129180022174384101305415837978911883846550330098510374779170577866567699408319562');

    Decimal.toExpNeg = Decimal.toExpPos = 0;

    T('8.7640115353288983769636573243142380477', '98132771361613972783640738.8', '8.6003674020697822292e+26', 21, 1);
    T('9883.35324545076423007', '11534864.85576058804', '1.140031440080173692239106128721010503628e+11', 96, 5);
    T('783715483584289803764074.0162744727280534', '587170409.865560908036543', '4.60174541714173715664e+32', 21, 3);
    T('31262.9696080945278832538654512053499', '1521666893653481239743.043288560492693823973095115099368', '4.75718258499323920319594724294097666996623614819732348e+25', 54, 2);
    T('166409773943488.683641092651970053074130172697160981650', '29383374581263422.64023860404731207688485848306803', '4.8896807217648976193745354023928027035617334126548609476076571e+30', 62, 5);
    T('99801119833708.031510783672287782392', '87030725026830312515179448.179673604958603757057054897072499933901', '8.6e+39', 2, 3);
    T('69731726468060380.008754386406977183117439', '6507714968375561.33257030119935', '4.537942001068668490255113904663360975e+32', 37, 0);
    T('826.39677733802082780060', '868237729898283157855076252483.31358937218256594842233795879804', '7.175088619512201756727486736e+32', 28, 0);
    T('97217122196114863.76307918894', '364113335834514329508664304298161.52242179479054037', '3.5398050663058988634098095121967083071e+49', 38, 1);
    T('694287.0306858418134468623', '447593578620913867261109598650200200326.4456503480625181364964224', '3.107584166547641764350683201983126563462027750463328256303247683293325772050963164554356e+44', 88, 0);
    T('93531032436395964.711423', '6448227682408.97162274053113046870693355787761', '6.031093925206599020774031e+29', 25, 2);
    T('59420641950920240756465.98423943966502979950309', '463601148613543697810692411885405.567307456576230992948033603539274944', '2.75474778598007436599867793211769930534477108061706131687471730901078e+55', 70, 0);
    T('2277315731915149177050340.03547342356244316415202385200697752', '6094500.77365339', '1.3879e+31', 5, 3);
    T('42499210185.324283508514337889872833', '6164109749819966303123685.0308732625627133229964813958076024', '2.61969795863005432962609968421706697692808308727433211318709180244255952739662715e+35', 81, 6);
    T('50420118559072004.14730128780580482018515', '291795344982121444321307800114867269.98968', '1.471235588898387943301857933580557e+52', 34, 4);
    T('539939290941744314049.4828453820231463558332989927861649996', '56.0', '3.0236600292737681586771e+22', 24, 5);
    T('974009303998280915261274173.014298757403849048', '477646393839119546.9057', '4.652320316205296032282599543500856192917503854138162003909582907736e+44', 88, 4);
    T('425036.71458946184', '828502.099774406', '3.5214381051858403971876e+11', 23, 1);
    T('3.0394775192110017953283324', '179399012406133.674', '5.45279265177098e+14', 15, 3);
    T('59591887481255758486.04419478394445992184038150', '4703540070912994338607753391.51553618', '2.8029283066942488985468816630836697194164252863314946121251268601762735534825267e+47', 80, 3);
    T('95721039331714561128531.5388333857551', '55776353283228975465181218023408074000.0635473992531779', '5.338970506403567357128883289406493806727217099380238503907444806e+60', 64, 2);
    T('526585509414392639783563978.124078638369185164439406399142823206590', '87507182704426188499191996302912.695351138484022', '4.608001438182859346468676488909777512497686963711987878e+58', 55, 4);
    T('1329389548866606584.0219028742633681448', '82512404176983746737881954764.075223596979', '1.096911277647395278024698413191e+47', 31, 2);
    T('9700740614644246975635351274850205744655.57062922663780504', '2272806307120089287441.2888975497068890265', '2.20479044526994561155551002188480195431e+61', 39, 2);
    T('29511272033245325473630.9914233118168', '3652962711970478409731.7096578440', '1.078035763202623786222162267776426022008745922576294592360500109792e+44', 85, 2);
    T('14296983312767601794907267485.024743090402168607', '9785251609055993162691824779826.478', '1.39899578965905859019321727491823760297927041482177805249500602271527916358976146e+59', 90, 1);
    T('86375361639533439579498381796.1294234762451026', '906739120390293252717325255078545106.1444764334577910073347022', '7.83199194364240290638163189680542533845422404219e+64', 48, 2);
    T('740414973164848.916954686179490899', '134168713446.24554435393694302876451603', '9.934052436586419877500774282012701064743967970168529777534242152461097e+25', 92, 0);
    T('23606.18678792636558', '92428538816127859383.480160037586533528917526119290144', '2.1818853518286167136498581177678567e+24', 36, 2);
    T('535842268080888.29369988', '5.264688269022545313985', '2.8210425028118864751215030126668946919568218e+15', 76, 2);
    T('14658164987181989.82049386326834040499563407226202', '865918141679263984332567963156509.5151684', '1.2692770986128681e+49', 17, 6);
    T('438605277378.084981547556366823181451761142976', '409837.00129', '1.7975667163060302240023548490605992984733562662678643904e+17', 94, 3);
    T('194.513960466935914010006205083791781614053', '85.244', '1.65811480460434850538689689461627466319083e+4', 42, 1);
    T('8110580235.6382837236573605190662841083192', '2057103081978872059674080.6009522119', '1.668429959936843982989884567921186647594125196439595677358416394e+34', 64, 6);
    T('4.5887147480', '51988100241496916591068.9610819637924597', '2.385585622986592627579640268018452912618365856556e+23', 93, 1);
    T('7676366921217607760860295721277.3344143440442432031081737854', '8348.78539390326791027347342077', '6.40883400301037613556945195698437202665104518299869750342e+34', 57, 4);
    T('5102042116133689582311304196162865.76919740094081', '342750911375306450611311104.4237583746438291417250861', '1.74872958518001921960396676220239838414831078549160736435254645814652e+60', 69, 0);
    T('416964390042404136126.61281911461', '42315466443571711892540.091194700689', '1.76440426550036990698630563709571259144e+43', 40, 3);
    T('8894.01891424357325612407678389525209718', '61507346556.2719', '5.47047503636416588602979767650939326821777169357303242e+14', 77, 6);
    T('7.052226648725256147903290392872', '203.25947931', '1.43343191659600183986919915349088805547832e+3', 44, 2);
    T('8.302651823063267846936861915172188838', '46587438.465153520209819497938540', '3.8679928090455468347091900316678684524926685096441747224388059801652e+8', 86, 1);
    T('47196903533640865695.85181', '7548684897618579888.31298887969470855504537218010', '3.56274552918755789616840430593530110693602959065825950186306650146230981e+38', 96, 2);
    T('948698303284247.09', '963610081.51741360635489845', '9.14175249363165314863989777185115741658e+23', 40, 4);
    T('151495660322825317441956135806.5873896', '513593007402.048381711031814348', '7.78071117935590305897798901995419145652222377444851576375051459808e+40', 100, 6);
    T('1698849066227268978208020649965467.60', '1618080928469442013.1753028', '2.7489e+51', 5, 5);
    T('644891646784255658.526636', '393435472578505282937727.8797019657103155806611814', '2.537232498144941317682009487047483158667772137267488634890500514431277704e+41', 73, 0);
    T('81450742561707021129045.0839100677194', '474345084691935624955665.0700971851066121014', '3.863575937865396257838440093691663423020116e+46', 43, 3);
    T('3375323436299.497', '7141080.48661308419037878058876827384', '2.410345632696615951452574271413730671541204595025848e+19', 91, 3);
    T('12.45602997074775839630609316912009481', '1394019487926011385111089334354.0', '1.736394852141284073221593601094362559979e+31', 40, 2);
    T('875385775703139513404694.394069137734181882699054248836', '36597703915300459735551784394.441973', '3.203710943084911902619573795221771189808682432568388769440832e+52', 61, 2);
    T('71669491904405159480532.48516359188914478869059712', '24730530.97989744112534450830319105611043888', '1.7724245898554006525683026655806323405700592994972638314078762447073576301927041849324640256e+30', 96, 6);
    T('83.31826029585625025147633468151013881', '38504779449.37226956529044178855323654', '3.20815123679733525708234133567e+12', 30, 1);
    T('78856605783750419703.502712110294345079445346682019', '4340866298808558921533.73509572', '3.42305982485114285062610212276078810120858860662207943955698685857334995368e+41', 75, 2);
    T('7149368115591762723384334078743739442.973', '747656609629.48982150', '5.3452723262965118050197863046502415470375055447565831993195e+48', 77, 4);
    T('873034981260588417303080801800498.68', '7903800158251062388.27375614', '6.9002940230461520192948772400456711565693799486006574087118952e+51', 63, 3);
    T('43032415777697358680204.964061543489494859386876592', '9293157905791503631735982520326186.3', '3.999070348898152448869873828230594291007629949294389177639042986320581615973841010896e+56', 87, 2);
    T('78579074963460700694795453552311046.575625409286354003774655392', '312956087740587960.556438223', '2.45917998788390457e+52', 18, 4);
    T('3313509565334014355664414865871515447625.4636727414666854313', '996815605697.865893971732748279', '3.3029580443540978620160184094560514883149439641254969265660241013695067451303779194477327e+51', 94, 2);
    T('161113.645060056887315867288293162247652478', '621680474557022295651202457432.569684', '1.0016120731854781660557886529945638003156366029999657105085624857e+35', 65, 4);
    T('9511194.7102883947524439156280', '809886382254238364588254006424568916.00434758055776293900348696719307858', '7.702987074831116791068762330133019263504692743125754911278123185343034215092050444426943e+42', 88, 3);
    T('409921725375005181563505351214122811282.9073042720351790317333906178070071', '6116692361680355031327056955.75543889102403295271', '2.50736508648812636271406410668101262e+66', 36, 1);
    T('5024038910398322594877.0871820', '3243432804171210.757839784003640256', '1.6295132611418505720064606407684561476212432771985574e+37', 53, 5);
    T('655607550027316067541164193584495.251174', '48465687987319445276184399.7739043360114082591323926969423460863534', '3.17744709617548245913689357578e+58', 30, 2);
    T('3663987140729979398451301031368079.3305027009267047097003391574183', '51445107269229.46', '1.88494211487931127606799175104109810514870872351726646976094461753360426683903118e+47', 97, 2);
    T('8847882664937980132687298883.9484324156876147202554', '8878619818802477377336624441644890.0', '7.8556986383358e+61', 14, 2);
    T('6459471466435588994169257675207.93481931648179601905028223779510736095', '7335157277081830335751556.269567379751520558052994585925888656092', '4.738123913313e+55', 13, 2);
    T('170914171991713121627.717302980953242358081326477218', '138439.4500', '2.36612639677381691159242881801665273477694818927764974501e+25', 86, 1);
    T('9849442413906208920618.344875671732787293453329961755806', '866549420.232206336139263924039044849202', '8.535028613380928210508481014637793691212099971032296378511726751213239726463524248e+30', 82, 6);
    T('3655788630513.1171654', '7346874.066500372907028623', '2.68586186821237343598287294920539747704252442e+19', 90, 0);
    T('85.98586063847', '1634199614332774094898149027548.62452233594046289578659', '1.405e+32', 4, 3);
    T('7231544768.58754951381269878980266', '6723057939003476226655.64935844', '4.861809446771158106e+31', 19, 4);
    T('990810359470140756.42493184620749787445868331870507', '402527307889296.18390636603756', '3.9882822662634157e+32', 17, 3);
    T('25441744590218115980366060185302255224.401796973098502632620416393130717964', '5381274349983.3698193870072734712565594', '1.369090075821689076228240253937031975705673568276777826905444495420351663768e+50', 76, 4);
    T('241626248618374568.70998108555276775', '57934437103901956712796629566893059.9544838886', '1.39984807032329985179523613318414083803393011073958509947145e+52', 60, 3);
    T('4170806099890307895414143875083.8', '4338153034558482.67093', '1.8093595138794169194282898471658262144105300901574e+46', 51, 0);
    T('98926604778383.8719225', '58118872517226942.762565932', '5.74950273167698596653230776901828674134984427e+30', 66, 2);
    T('6319.09953669183', '96387290515806195225307.314627', '6.0908088284141174811171506735960607255039741e+26', 65, 5);
    T('13858392331778807334459.36000962877229862391057152403558663979', '145017835.27807555568470601686277796774394786', '2.00971405638884448709e+30', 21, 2);
    T('47721578510683856268349444294254.304844005150995908775144624849', '258096235674556834448698249.1898', '1.231675977405532751320406722229555346307540575378121e+58', 52, 4);
    T('656165.64', '830114370694697222591833.06737', '5.446925273200832476681926034239991668e+29', 78, 6);
    T('898334600482.86178337567210659357719127632214878963', '235572173243879752613096437763592.12345859021278582528978996', '2.116226341359202196980075089414955070916821331442353569e+44', 55, 1);
    T('614404348690472274.8211134913712095509341158470671008184579', '8190243.1295321874734371581622836387816328852273', '5.032120995616838994617009651918462596441637954897305636217882538e+24', 64, 5);
    T('956648316250168162827858449.1', '81650.55678703069340438939295339866633', '7.811087e+31', 7, 5);
    T('7494.88393177403097', '14420197563057292634989545746959.3144', '1.08077707008365141309672667155709571641715199075566968e+35', 71, 5);
    T('4550463952037308753785.2394158622400993161770062', '4.71363910312929149', '2.14493e+22', 6, 2);
    T('64815476070639.9764', '230546653.32828641630916481600129645836585462', '1.4942991091965678487621814612096424272138973530466672922630968e+22', 90, 1);
    T('756057408952103776726013861489627.09596254971505646418', '6858918733357524469363707474315716592214.30275798115300', '5.185736325755335517e+72', 19, 3);
    T('380.136879804435680276714692374', '6247840756016425211504026822.530581', '2.37503469050707038146e+30', 21, 6);
    T('1552399843285.5077750871', '892.97623086', '1.386256160844747411858503166207906e+15', 53, 3);
    T('555524461367496860188646986.5161873716136640607', '421827431784724292534478.6044316064518379260585709299559784543', '2.343354568322434873661493979510935463022944838860317450419990289965436872412471190043868e+50', 88, 5);
    T('2165087139156394708711189205475559428.217288139643398740947077508762728680', '826428231574.1093643', '1.79e+48', 3, 0);
    T('610897.06856777564147071503984294489385394', '650436542440445546349158744.1313935503224058997266613145', '3.973497770662277739984771184505809851593308569587013290621750264747783295e+32', 74, 3);
    T('692059260599365897.621096335186008692369', '18410205556.502930607582460', '1.2740954e+28', 8, 2);
    T('68110032388514.512867249120274199100166007212', '92231450842286192779908672121131786200.7820956731096263179550124090', '6.281887104107796738428202372057675292548299357948261640752466876518250701264365779224100758443e+51', 94, 2);
    T('523196467273631584331892093564.7127', '818.7255738483877649812245421', '4.2835432790405324818937486649795717177668085079004837577555467e+32', 90, 6);
    T('85380156556509250410472914.995', '73398028.2571541', '6.2667351435349068610351379390244228899157295e+33', 91, 3);
    T('946214440758812715640079180989984.12572845744', '973876520259593335.458400031', '9.2149602698556964957479473777687812184953765883848170517859518807818064e+50', 90, 3);
    T('17647.4980', '99462032690945980.360056535128836046728919617461429219', '2e+21', 1, 6);
    T('9.91564926660321218770739', '690080451860595922.070793696670953045334739052', '6.84259572638873122806479301455003419140222896420407939737056778199428e+18', 77, 2);
    T('5257593834.939635903244854630032518144293615', '64181288.772242081781773424226277147196689', '3.37439148167420442778100224364095015905034288003910257906186395130719493799371840735e+17', 90, 6);
    T('710663625749615777290427664605564.72282956064537478', '839614617496831487.76902556787', '5.966835683026750552313713160505211158642552919512245046935336482299e+50', 67, 0);
    T('932702716016272484191268828210705033.0860603300604064266365026421', '899655883207724697517511547741.60435322554644507592768707153325656', '8.3911148574786325369194117353072499890951242658665e+65', 50, 2);
    T('955426894820250722176974017000.59', '24087837422542652.2280492642420512050', '2.301416771155495784229e+46', 22, 2);
    T('6110645108087286891234921667555068.83857537', '32587382481644.990252338', '1.991299293468333107884630232611987680597340369337966584847317151e+47', 64, 4);
    T('3.955840329441910672043070', '2549398928894849392520631273.8506279836583187', '1.0085015098758255221288069881892042403158940698640051482363330186409e+28', 93, 3);
    T('7391607200644127287.7215815599578010588767189', '88534375755490505191850931887609.8247998824180164827740457496263', '6.544113293388164650332301768981896874383688507602513015237915211467376359419906968418e+50', 85, 6);
    T('862541218833265084325397539320186368.1864263428963627571127', '79750.9783683300734', '6.8789e+40', 5, 0);
    T('456558018836739106.59654042977974', '9.515375963421921882', '4.3443e+18', 5, 1);
    T('300321600.09991190231474487681315951', '433743262361086815329070577655203.7221225966468560184927766620385221', '1.3026247058483748457723066659e+41', 29, 0);
    T('2508169589583181983214318994617575.14654', '7827105953735269888172532.3895', '1.9631709127604272063149714954e+58', 29, 5);
    T('83320152022831665810487.9405408532307649947990522025458', '787525341138.13146721713980665498387377436632', '7e+34', 1, 4);
    T('711522903665644515771689846873792834.731946011421348410380', '58768292011047682650973698765643.21509610893733542', '4.181498577518e+67', 13, 2);
    T('4025098506.9161250861687848', '423581393192323933297667.589746702902010552515162350505053351', '1.7049568332958751749222971474399465101815104039372656611744839249589249169885148378648e+33', 97, 0);
    T('359714.09690010300022685894117130363478692012', '708.4634875268070612', '2.54844303602402788460037568046164129542314083820633916616351344e+8', 66, 4);
    T('54975035374395.58', '82519411.0867945727847584140', '4.53650754357082245175102569479633156941012e+21', 61, 2);
    T('8938526396894039.18730', '6308172115099143377035217689.116527192833447838865', '5.63857629669645963037496337395e+43', 30, 3);
    T('18774998505692235609284673570021.787915288509028032521946065270874793112', '91336.91434021960722399238', '1.71485043025216285155062809311183761570546963073071280632e+36', 57, 3);
    T('75816685706791431479880.98', '0.9142483737519161398561657057675613497698', '6.9315281610694210934160913662393169498245246823217455321454398404e+22', 73, 4);
    T('77511038592126778999599473089334.84', '5040263984559.539951179759422905452697520367181952473', '3.9067609622170119290333984111089830928497101024374092227496998032873152078770406305932e+44', 89, 6);
    T('6430065041262210044.8140116903299066873417747', '59349148288507356541.75766486383010464', '3.816188836386180782028815567058141081347239678558181311128e+38', 58, 6);
    T('393287.948760347993170', '9584357708605095800390633598738815.0192863828', '3.76941238340272721924035411833045710575255495865275e+39', 52, 1);
    T('94493249276334870891851243210.849388579099598929136', '897121248427619367522940982847.5816312991188938409678663958449685935', '8.4771901758767779731150378559496973815523342678405789364640869341243773888310790333885914e+58', 91, 6);
    T('477780717048083666181.719684328857963', '44440742390176659928715966753514326132.98196001634', '2.12329297653277721593641373603331987268940209871700203317259152377200541232810192e+58', 81, 0);
    T('10846.262373365451', '445893623500751991711913941632.7410533162099747215259245', '4.8362792310997871355117133570684281333196219704648300395e+33', 56, 6);
    T('3805757766862386650968.5048686', '5039812532073368766209066724255735848492.27313846836567430870328', '1.918030568746861431482584708409846882987810195728926887383721269817936027014e+61', 76, 2);
    T('8.6593303296881263638320638892809735', '5943702189.456752828945814', '5.1468480639796581980878588498252473475188214708094452669929e+10', 75, 0);
    T('863872810772052353697671872022.954138767741023807031006281', '7472313.403036833797538870358668', '6.455128382451109296564122780266962325702754666027484968376261480548092404131839430793708e+36', 94, 0);
    T('24682593125096331266477272.24415', '419470697316277535194854646657018.30029658705892488372', '1.0353624549758e+58', 14, 4);
    T('32465361684778679226929453.4558182869290424828767461931465603', '4764517.5124445005143553599735658282014043268', '1.5468178429497271098855292194155441423721e+32', 41, 5);
    T('1508918754071859291311293.0534850925204460638683', '52147877986177965.28409543803292708990345176264185133', '7.8686911078395e+40', 15, 0);
    T('549758695726.0397', '603781890841195643580726276681124706.3', '3.3193434481185779214639963772708148137142705592464011e+47', 54, 0);
    T('4215291598709308475123063823897.94624524191870511338440', '295948257.00085918267402811', '1.247508201388384998395585889633538952275565396e+39', 46, 4);
    T('327789151493497783323.9432066504619465906233', '62311221527783571328945440.9989575709914272801713915', '2.0424942433115549458e+46', 20, 4);
    T('44756715324517985381.077632620977387473402', '65171672510898006476273900896626.511321036579284865600980108', '2.91e+51', 3, 3);
    T('35663628554135592871904818817.3782354036178143190222964693777482367', '20234853495181939765.98188866967386650370711370091', '7.2164829889952103036e+47', 20, 1);
    T('18771056130.46449771495704357177', '9286397231415322265.314028004022670869523', '1.743154837e+29', 10, 4);
    T('722483876393474596.9483402046927144', '284.25470472084662177868875977', '2.0536944094979977067487765149843160525843277931024e+20', 51, 4);
    T('8953209639582350364702682.6649', '981984.2902', '8.79191121293707214733027497072767095398e+30', 48, 1);
    T('7.87878416570780', '911530336023640.51166374339527458', '7.181750778025369100786332021e+15', 28, 3);
    T('5526054873.8263221523020157481293967', '863426917841536989.227758399791747610888813856089492', '4.7713445275310649102381173785091221425213863131141564732459342236883128612122711694764e+27', 87, 4);
    T('38236234.5309887452904434761103038290965037808', '85961270505463618751973.979615607118604589', '3.286835e+30', 7, 5);
    T('655910165366926740301887697658495530536.9875217655570304830155741615507133686', '83218394335.8880999', '5.4583790790422483193769978469999369657164e+49', 41, 4);
    T('710106376432095166.0', '624457075318053415545399628.0097553497222859764', '4.434309509914868419067945867047118968391185885946647392720300824e+44', 85, 1);
    T('846554868970420073163689870039465.7185636991718248441433503', '9530693633855753308313998804143.95348570032048', '8.06825510040597398642733395943011615298504164010077513537102499474758692753467359165e+63', 84, 0);
    T('57805199251483910122881.7797457491431958373845818', '545165.50037565404571880458181507926301587314', '3.15134003742496085664975e+28', 24, 0);
    T('255365970603768708156933199234.0347459154199420285452541753379052607661', '64637110.8394546057622697182764', '1.650611854654070458198360734829507356884e+37', 40, 1);
    T('68969505547953647080031295.25589250138448858', '29697760043303518955830801131412421.167', '2.048239826068418192977558560032344836718116945931008041e+60', 55, 4);
    T('6837897825364.6448', '6795583160935843716300181388.0', '4.64675033182478047748317459048e+40', 30, 3);
    T('7086086197280161999190376555502775137.8548', '58.021015889930083720239', '4.111419198498065213518194048182791877898168237422218684618032972e+38', 72, 4);
    T('8488081008122192891.2115308363', '735511176244025978250662433940108251.22133706270765560', '6.24307844633e+54', 12, 1);
    T('188152527305192501680287620404614735.4157643041043691131400905036582524355721', '2836506703460993024209527153475507253.1514193115692940890367', '5.336e+71', 4, 3);
    T('7728363654.017816548783734719', '3116602962126484008159249765514782905.59394538892199458107', '2.41e+46', 3, 0);
    T('86960998130.6365399705968896', '65728897526235.515599191477884320163412430553', '5.7158505349077673684595918624756216685982037200345902404448837178079488e+24', 97, 2);
    T('64518073384047979786192878922120539331.0141', '121.3921756370462896', '7.83198929600019383205026989303784270884378e+39', 42, 5);
    T('278750963584588120736546314287491544358.697', '17068789094.87352834553237698594434089714', '4.75794140741810572974551811e+48', 27, 0);
    T('7.9073357183376946443726763472663531', '73423363413376849442808160152558798.94', '5.80583185e+35', 9, 2);
    T('75351979968843.28575076357261252730084254560142984', '5303453041927493769.7', '4e+32', 3, 5);
    T('2338193274601452284.83806192821747982971324370902', '12959407230.3131094', '3.030159882873954640192489509576362378507199753e+28', 46, 0);
    T('46677299967336146337985.529279361713', '613158672821831316570307583973403919298.253669941770331753606984751481549839', '2.8620591298878341752299910618239096564355585381076313166815505438943495140715259603e+61', 84, 0);
    T('4654825020429654.750143214672609529574', '4905728058655077524.4433955386', '2.283530571085e+34', 13, 4);
    T('30096494.2038892955779269907059063666455465780378', '71657716284765423.245656', '2.156646042e+24', 10, 1);
    T('29637511025829896607585468184.51402964785', '4394475122666861201157098464251688.103164978507447520867000389537206411521', '1.302413049007742862478582107654100299860723120118730620180017010549272495888713e+62', 79, 1);
    T('7150205.3713960486610514686', '17894152524084242461992074736.684211749993685286686938638987759', '1.27946865494287312456016317e+35', 27, 3);
    T('71667561.7898347939247865853100232520157037977645', '6345903891135447387444.95373892106506693692021645130805', '4.5479545923030272702346926864001926333627708981803741341642e+29', 59, 6);
    T('360623356702274258521791167442948558.55455725857099', '642825371075164.679', '2.3181784309051092562444926200778997609263297269890557691489846206221e+50', 92, 1);
    T('86490759520166957879693.06570199259741754592451697614', '761363151.283924175225', '6.5850877225e+31', 11, 6);
    T('525493.41', '276903994.066271085805470566039646789905789886592', '1.455112240845045e+14', 16, 1);
    T('79919138.66881065022909829950004661102058180841', '76688077538203000710.22570313266000355274803', '6.128845103020148888919682872692629547944446642086711902672e+27', 58, 4);
    T('7171754806297951834953.684291860887493616', '59575438431737490554620.331', '4.2726043691012106201438439328494106124541216e+44', 44, 2);
    T('81842479697388880113056639248743.37163274', '84718929355411607509938870391.0', '6.933607255754797291537e+60', 22, 2);
    T('9324015969267118926.0', '827349906873443759430668526828300921734.7069244849', '7.71422374385965329350518200743330526374020447405067e+57', 51, 2);
    T('20358.08290131661656', '50526861343059564781431.9626908528874549', '1.0286300319653364618274083412277162571033e+27', 41, 5);
    T('4.9251414895517738', '276877341476988058571381423736251.443873873875738297824083', '1.363660082025108088868128031719855079e+33', 37, 0);
    T('62055527834.21536', '287308988566046858757408133769432.477232132487028375', '1.782911093698058345474619095032900784495075380626168081269618084e+43', 68, 5);
    T('3937410415598952440.6530376143578573305802329339582142', '87.0', '3.425547061571088623368142724491335877604802652543646354e+20', 64, 3);
    T('397969547531136529.4005041565346114417965', '52505361103375837971310653070.259057858979166342983945145272172', '2.09e+46', 3, 0);
    T('436128.5400042793323986733464743', '4929209436895457428907.171379399709459519841485894879', '2.1497689150885317067445915825373534478204214815069792e+27', 53, 0);
    T('63458619265901301119423476.808816093327489073346596313829244031987', '7898785854786508663433157950648920185.450667222093063786', '5.012460442217838156644342025585996740161221e+62', 43, 5);
    T('466513306908892908177424.3728929460355704202434726849704995004', '309327199091561942300804360180094.690855358055361934', '1.44305254565070055967236364844e+56', 30, 1);
    T('68802410823115995.296856', '5188.2520037114', '3.569642458132064763598164406567513584e+20', 79, 3);
    T('593951891186265356902469172295.229', '97126971458763611775056314409787742.50292433669638314', '5.7688748383127065739407105582243362410983871211991904798583968632928e+64', 68, 4);
    T('9044297802048594633936493.0999559823700286', '879052046318236909549129553.393278537202181338263315248285507', '7.9504084904023494859524251304738803439657099941891489068117605447312e+51', 68, 1);
    T('60083632871260960228.87010587756321421822354105546414', '1222979843353516410145421051645094.98793452908163694807537', '7.34810719170049185538763980655951420872346964176409797395236549344766e+52', 69, 5);
    T('4234724371799965941336778465360011.31717956', '8687172.696482727816601215368599694', '3.678778193983063574921775054672198345714280573038104e+40', 52, 5);
    T('846591970166154908632285311751353479142.22520532791825296', '1068975175715.915', '9e+50', 1, 1);
    T('270161600630702023.26', '810968996.800088669186', '2.1e+26', 2, 3);
    T('781529618039650759.632197285417345464816973533264477512334', '92334663583.296155976990108653518642530425005', '7.21622743620730955130265723793166176e+28', 36, 5);
    T('69583467343656.0', '9311125875778.3513357435796303418', '6.479004233098932834516044003060564404653416208e+26', 60, 1);
    T('297190318659320507968621356279551110.154298073535302169936128981553', '3954629937985896745945787.12633972573217330801386056497649', '1.17527773144972e+60', 15, 4);
    T('11841921804491031496710959.82', '56.5269170313945910690698397998599', '6.6938733133472705718286373770149678153610464815894763052921e+26', 59, 1);
    T('4.47352565400637589121781614850339618', '3904571559379.370253900764', '1.7467201038787292272704342955207083703e+13', 38, 4);
    T('511564.01528616101083604088167355739', '18477026035519827226803212.7872795842252709590168179287499261792106', '9.45218162927745e+30', 15, 3);
    T('47872105792860952657330.7982445324883365835018771', '513568813269047277820843227.81347741611743514899192554090232', '2.458562056072988307739523299610805399689804418825119134263015491790980154463108134371136304119945655e+49', 100, 0);
    T('139254935142702029286972.3927', '3207540861161.2557969419461', '4.4666589458857729043011807695109528026381251531143347e+35', 92, 3);
    T('336045825088133026183756075.077817984770615775579143999566', '426313162932909815239395179564235667645.0835213037048526906101583593735', '1.432607585837213676639661361943973542110200112167e+65', 49, 0);
    T('7683136296679135958003813383371701166.6325238', '88926261265911403451232.4357709', '6.83232585660095833012985736855886757216578592533043953842071e+59', 60, 2);
    T('684663531522941086489447728.643824635653132554708080916204432', '3322896744401640532544176030.4610496973302237052827439509839575256574', '2.275066219908110923014058558836638063988886211210513086992904940079703811062796147647603197479e+54', 94, 4);
    T('23133745632291446882724692.2433191468167', '41371644365.8762', '9.5708109714980358838638759545331716463933047933629254e+35', 81, 3);
    T('2169530852420.47173134406807974190138137653317', '40003674346584202426277075109825455.298268', '8.67892057e+46', 9, 4);
    T('601669855291.55204840873841281627992459730', '870.8115007850143553341374271', '5.23941029663538850179e+14', 21, 6);
    T('9885348079333.3560112294607144283305490744725', '3836810.6279773400732931784952466', '3.79282085720416062358437696348354415899827598188e+19', 48, 3);
    T('9026053514.468251025056591508723', '331014835.29500381737666', '2.98775761745559846841184059838781347288588955336660518e+18', 79, 3);
    T('122520772199.43', '42594615298.6359', '5.2187251579265e+21', 14, 4);
    T('805136896074932869490981107201.2338365412599556573116166271', '885633407379.0', '7.1305e+41', 5, 1);
    T('5725534702041599337960441777.50156145', '623118096116623250091965767.0', '3.56768428278581915773119714406479584830123e+54', 42, 3);
    T('57868476314464817406884236885120282.161405077829318672372862102788041731722', '93807968482115.607', '5.42852420221536911294416779768794294975e+48', 40, 4);
    T('29832.4376408230871004517648231213578', '443935523939292170472489562055.393208621828188818893872116815745682235', '1.324367883446505842455e+34', 22, 2);
    T('8519753380800625.889223704856402984216703504676', '897764012951209158819423826265367.268272012833316', '7.64872798e+48', 9, 4);
    T('1324321423037187894812890751.7311161861389256340910611446179', '1973851714683513177.9596733120117116369813776031847837479609', '2.6140141116540635564167013329428741073477e+45', 41, 3);
    T('27792503558888832773947194.376478363983', '6707841272454478219526617623.12266534154307980', '1.8642770243715248247462021729005048101852333608865643346794896083863580781921e+53', 77, 6);
    T('902.9435740551209266067542051144636', '749042484787626000388.114711745', '6.763e+23', 4, 4);
    T('9972783474415523744493438167209406.3039615223128358', '11.78', '1.17479389328614869710132701609726806261e+35', 39, 5);
    T('396931891813373.772705397600979374655568160745442397587', '4578448.0560147264271472528351304975950897827', '1.8173320484431888530316243998217734544001115295207860733500790666171e+21', 68, 4);
    T('6562025784.076506423542677854182311162842815', '4741551188120516619492681928371122.476167530296982', '3.111418115296542367995913145701641394187228175269238024410744682676670178149213056033e+43', 85, 3);
    T('8778646311206878116.83131975416908903603901692576', '5347906443561765124695882.205428045438596696590', '4.6947379173453e+43', 14, 6);
    T('3100797105596340119996437222950598213690.929908180427740632742721871836838', '56.944996037319196', '1.765748788907144206878191514959834835e+41', 37, 5);
    T('36.6459426667219401', '38126960584859160.18507153060879116722603951106', '1.397198411649115993564370540388551749047525940701716573178607506e+18', 88, 2);
    T('25315430849895110961861.5402313928193436078579528151', '809080757009388691954004969597014.04939867014507946607115212339073', '2.0482227956051968529554512517122385095297281285113e+55', 50, 2);
    T('82896375610864314519289307711194023005.5500589935262061286', '753494.70905625210380545052282668816', '6.24619804227e+43', 12, 6);
    T('4305570656749.06879370803000397297782591737735', '28652056350487082424619371940140641492.16287', '1.23363453078177994682208359160189835e+50', 36, 0);
    T('268733428116292104879794932020.3', '50400113502851330782614846510248358639.0750023812405135064084501495682', '1.354419527907e+67', 13, 5);
    T('91556080456626095986882725.0229610772464646647525', '863830002504.75881945850093886723751789', '7.9088889210173e+37', 14, 3);
    T('2.98565075797118286627', '64.0542895270609469330214543105306', '1.91243738077775116695491826530868740670369154752542862e+2', 96, 5);
    T('7841163353770347039903244892822.715', '94982699888781268548376777475788.142028453', '7.44775e+62', 6, 2);
    T('7354467910771367569053637840076109.27291135419064632318487932', '369.8048', '2.71971753484922343e+36', 18, 6);
    T('35812286371059.484', '566.0', '2.0269754086019667944e+16', 46, 0);
    T('6165760138525028806997346071605857149100.2325614878840386464755304815185', '791077424134695162733119676.57338893315000182691848270779007', '4.8775936482167610133972649799602181442714450528236122176596750716738840955860042765616329e+66', 89, 3);
    T('236838938652119513916719289597.34596249595879', '182326901854317367523115311881111867027.81545342076370093138189837549670739988', '4.318210992290568665030914751645212768603419423004690737266459905659746135e+67', 73, 3);
    T('40975989170.533069694428532401', '91939542551.0081884679', '3.767313699913875887095554279977302986e+21', 37, 6);
    T('22373322899139331.9279197873146566151160622486519111', '5902446.4109299', '1.3205734e+23', 8, 2);
    T('29116113335318033023775462407291067296.391674726509581923598424754419', '3278493423855960.390854814320641', '9.5456986098085e+52', 14, 5);
    T('2322854821937063158.427406706237545982705376788', '15609604767991109.2254470072658776694518133', '3.6258845703859920095164412593843253008190725509605563e+34', 53, 3);
    T('963.178886597257331704401205', '6073760291525062281.8252933163567750', '5.850117675049742594817508749354913776489822291787683519913875e+21', 91, 3);
    T('69195912877440351311937.27109310131', '22914625580759067727.2162705002241718739848963', '1.5855984353053704634172380633e+42', 29, 0);
    T('115012049101294932828926681095465550.305418623715702306755938757188292', '579489849729531835306.08427193110112866440911', '6.664831505079493759268103779544502247039e+55', 40, 0);
    T('303733439630612045613.13373107339173085664019787', '529065078139.644578047362933164005664', '1.6069475597179278111614315684703605845452286848024143810619877470807611717076073568e+32', 88, 6);
    T('48727292617817763890808994.2202805548876620379132571233', '987863236850279606016664.4', '4.813590100838819066479533164433010675065986206997196426010028304081910180552052e+49', 90, 2);
    T('6786547236683049.3638', '98155590.0', '6.66137548079494353302913642e+23', 90, 0);
    T('617133.8273527675221208802120', '949.50016860', '5.859686731202160539303799908744037432e+8', 43, 2);
    T('521.93350515729894528444598790', '9188675914664671.51239382', '4.7958778278953819320747022686499308961e+18', 39, 0);
    T('4872.50268534284129000036429', '9274620967366251170885155673878.82', '4.5190615569029079225239e+34', 23, 4);
    T('3175568244047164985.955462147482449652803254160158398591211', '605206.73189163777106517195956553980032517', '1.92187527887865152195532237001732186988957646e+24', 45, 3);
    T('58979770378437477709.4145033732247973', '370292523427614155.5805024', '2.18397680046128631254828278140379033033283810254508721e+37', 54, 6);
    T('8228219798097128062440676955.3467707396131002700244026583024357', '747118.039953837454544440345767', '6.147451447863686477191681628424413584051635509e+33', 46, 3);
    T('95762286812663861546.6378402155090148385880460', '219210495828033561808015509844026911857.45655553475473318013914926106361688769', '2.09920983738304047946207567146106611752e+58', 39, 0);
    T('466805161821560084563.597973307622495', '80852287764837130363865526012708811.09412362479233220935664273482', '3.77422652737e+55', 13, 3);
    T('1815167474673201494515110541236568.44114460217', '2446397.538', '4.44062124109819749075968693187898850998465265067745746e+39', 82, 5);
    T('644983126517961350375641201998.83', '9912561.1252245793249953544049448640089656775720', '6.3934346663477501711728822e+36', 26, 0);
    T('8555689850706733442340585185611784634.759421', '54311.894515348881128884863109540', '4.6467572467762512365412802839043422766409320305102745162628890243586997634e+41', 94, 6);
    T('1366526297.0566684991267017550722307370834098', '40241024509.268660705840564500468374', '5.499041821241754355476278790038395915061087832385e+19', 50, 4);

    T('32538525661800629274238615.58854', '4221257960257328197334.99687127737124932626496743934439', '1.3735351046e+47', 11, 4);
    T('9016577257202221469913.973970893892587089845961069', '717041984762.1704', '6.4652644522657274667472322240270365e+33', 35, 4);
    T('636458802810705.0625167255453931606640198', '1025500890675450.780096456096656042394836946', '6.5268906916060913799615494531479109e+29', 35, 2);
    T('79110380.399139564488282', '253618226821001.00507451769506062628335690320494776', '2.006383e+22', 7, 4);
    T('376873444696269355299365.4531356', '291637790197265810665.5250538930136484438850490707423377654', '1.0991053859525146161055022709845243436052743435939690299036572244868802096241063972718824e+44', 91, 1);
    T('736596301647821915390749553538601616524.5755259726118', '795142.3465487', '5.8569891175134317430817399167404057094937405736308106950045e+44', 59, 4);
    T('1834131571267152475321650.70965445617470881921005618405', '968157051313385666904.75179596420634009857645265066', '1.775727413758793219327376503051947507155974684643899649e+45', 55, 4);
    T('11467445190748986299.92690201609446082017', '523498237.024622524327463960286539476473502743517', '6.00318734053358048573196294397431136916711567368667712641480116428920771869736097033789e+27', 94, 2);
    T('1740.0', '674329600699095.9235720144834769329772751', '1.173333505216426e+18', 16, 3);
    T('2073608914815968480642598159487130353638.403507', '8202785661188.8837288359362245881395096', '1.70093694733658676908209396165038634317062656908141094441774e+52', 60, 0);
    T('9408583565372039515608744.3684350485521856091581100947114429124279', '86796159427820718186657687754456418494.479528815877464198372228379', '8.16628919130005414007709894042072728383299297277052582630118037721442350956148223604782621526e+62', 93, 1);
    T('9724576927702863.00514157261352484328', '4521791471398152.5408432324329596777062957089914679067692', '4.3972509014642054571320962124e+31', 29, 6);
    T('150097.98671404890780338257994', '6793507639662.344499876512736846104161284', '1.01969181943982833972447444409226797605134869153468537641830978304296e+18', 88, 0);
    T('65118175343527.476611386886747972', '1.60950590', '1.048080874126420004181e+14', 22, 6);
    T('76805406.79668422897749934', '59816821233324078222364364762.2859557060466229096389', '4.59425528810999466160015772436770505992265745907144432659598870867010439e+36', 72, 2);
    T('1469386976133686530916720194473533.439610543942924779731287', '9728007344019623222764920709573063.7390426', '1.429420729503528940651228166512782815270522737997255826171601930012314473272257e+67', 79, 2);
    T('1884577993841445785992941888727.692681772714424110428791067182984298928', '71.215134629008945058285774463766', '1.3421047555028615218949107869049097752568550421115097800823629315e+32', 65, 0);
    T('61128482.374', '60842716563.57307948576466459526793664258319', '3.7192229270426548398064662836238579088e+18', 38, 3);
    T('135566228123986394372179852820663.73785657', '50104554646753293.82010', '6.792485485292499562041458065250227993920664022431740601183057e+48', 72, 6);
    T('90.2842162098226193', '2945079612003223.425352321291', '2.658942044452395346683129106885849346127775163e+17', 64, 4);
    T('724.3046165965260324', '9336761295608.4424681600303811056430668100247918', '6.76265931046895657939393076121022270844330653566993116239809005432e+15', 82, 6);
    T('833948566289742156240.447936107644380531030033433244', '52191656593247958792015529.15', '4.35251571882257036361170975172568356720234605201463e+46', 51, 0);
    T('1416602871768308518575661680760149688.0812273123382771087180501509778144371482', '8988960023614.936525007233865813885234186518687317', '1.2733786583663441439087447509752725556340548704454166581566848096781678967647462456892205237e+49', 93, 0);
    T('9569593587222898608259132413.90668285543794', '657072004.304598929942955875345494864161946378604', '6.2879120387369867498747065845987464742501e+36', 41, 6);
    T('804334215574564.25044091930742448256033235093982', '55688438273971108617042699218.90358138209948168910', '4.4792116315667092377278387277247989195238061393700184267551514519131272798066909360107266e+43', 90, 1);
    T('9822620772475417727940129541187011947.820413104363741726', '6136578440346805045758.683005231', '6.02772828600753283057e+58', 21, 5);
    T('44262415003768171285.79666967441127542455960075662622', '914422146623526378080312597381.446639551542695668166751149', '4.047453254248707258604102371217281613039726579222140035456583290394e+49', 67, 0);
    T('58171306190.64729896', '27.17369925536002253', '1.580729579716112376545275777709246e+12', 34, 4);
    T('556.16559337993700126568018186397', '45903.04535419524297409932373281244', '2.55296944573621577473044868466168773429115544627362374236896037868e+7', 99, 5);
    T('528259021140625408426666870782747007917.5843643434346', '6122833.0', '3.234441767187518891353274e+45', 25, 5);
    T('197.735790949550075111726825', '90360323056302543712.1309929032068139511219', '1.7867469949994849505159736071e+22', 29, 4);
    T('71619953185522739506202152303738321.9612100652865166041156411', '47565302086610062.6192', '3.406624708698259762662133230895007110859e+51', 40, 2);
    T('6276369365764186.8194482249901490921901493', '1320031407492171444589488102596.8289899552098', '8.2850046878304469350124644564338609371e+45', 39, 2);
    T('2923422177042977476316730.315839828833557295866099888526668281', '6299597473653588678.99', '1.8416382960922815161024381198489253370595917838548629715782010162385292164724116e+43', 80, 4);
    T('159225072757897568925.9099', '353410532580.3047034605580001830887429', '5.62718177635063456658700070714230804830434136658109469e+31', 54, 0);
    T('522813442883999098430399988308.3123058878474729779796', '2426114525230.764536270', '1.26840528776677490449174629654702e+42', 33, 2);
    T('9018.71964162144783065145965026315', '75265664004309688166339482389999.0', '6.78799922295348176930371074412e+35', 30, 5);
    T('110230586900.988221591373603420', '30156757609583961158.0684071782380014858804080603941', '3.3241970903352826622762952803281079681659851396e+30', 47, 5);
    T('8639024432.97', '952834034967301867429221408260.44896959427109080', '8.231556508647912167758551588105810033626768581158933383676e+39', 69, 2);
    T('903953131738311.8490067884901571191843149457', '5678877957338330661272701596199421093.53467600250844738138027588478', '5.13343951429565131253711608892767776339522683080388522069098556324772446188261216081e+51', 84, 1);
    T('421111220474715661932459737113.027541052379330389975145', '73627047.093505397723783387', '3.1005175661495424509173445780034407638049910287284430899628261911805806293916115e+37', 87, 1);
    T('148533670697419179575568722569177514.0402831977279749', '48.6929635941324164446689819793472', '7.23254461977228500141790177666570576061833091005562749306286528779055087851547998528e+36', 95, 4);
    T('994778643976094469007520865632923450.463834013240303', '61483764.228964590080277851911573', '6.11627356062352984127648596911334101446017106693403971084550241840756e+43', 69, 5);
    T('92067521335422217972.75', '901384435623720923564329027673682.372', '8.2988230758204440871422217330465645908401767439633251363e+52', 67, 5);
    T('9496246313801087348749503706.43456990373203250', '9142350937300384166142611522501.051816384566896248833393291804815349545', '8.6818016387814688984914859046810508038773198407488448e+58', 53, 4);
    T('559341795912458633289195030003812841468.375135351949327721631229664', '9824274319541945494385751828.1', '5.49512724142923928928926892852733864127487176395083220859724670945009292012926918878e+66', 84, 2);
    T('49479753259735412227.1187312946468', '221210193574775520017.0911029391732772849726736642', '1.094542579661820057787e+40', 22, 3);
    T('21154297876243032.22708024609786590571513706213', '5694595279956.383921618511946326510287471690777', '1.204651648368449279389434952459355677177465845319854706307043063705671344889849e+29', 79, 0);
    T('2096556755046.95972529119764249684987', '40557265448768.229715405695058204504922764906281019783', '8.50306088428476964832618752807213050340371161538978017501067e+25', 60, 5);
    T('41873765186979983759.83715763523586234071509', '2489534507900128.89573884259586286785350861', '1.042461834086937627065980871718788417626465976209477908188315636826522461381128719249e+35', 89, 5);
    T('644980815821648662.64681263895112502275202769', '77.1020945421320527501955', '4.9729371839342e+19', 14, 6);
    T('9045993312660747762889452263086.258564199128231763565437', '13191122091278631239429604550624957.3', '1.193268022241979561322084621663694448906782869057e+65', 49, 6);
    T('918959006934378986.64312067167086438220', '803576967101373941.834467490795288', '7.384542916828187e+35', 16, 1);
    T('165.69', '688772794639210270731326558463682441.573623746773718807887020257295652', '1.14123e+38', 6, 2);
    T('79641954294859484411234829950.7032', '932144505557172198816983052387591.16422907777571729229819120611593386', '7.42378101077887009336142e+61', 24, 1);
    T('3546540835541937139544230855209755.72173344400889299111', '7295730581382173195580.192809276916448481550213254983', '2.587460643198399534132721861617475657721016368849944705858253966268908e+55', 71, 3);
    T('849378303926916777022143819016.95222148674675699320351424196453657', '7637.684404356029206080226355959956901648', '6.48728342530098770687684279745755278633844325212566802246841151e+33', 63, 0);
    T('87408861349028127277490029340.2842823140698765216976197282699339', '951601562849928220126.66', '8.317840906666785104887180068499880188002035055261e+49', 49, 6);
    T('3427367206485586196.547215684', '7507078651198654863261755103579005157.1915400993355950423135179747920137', '2.57295151856263160390721104703354606597733824868872989918714637357585e+55', 69, 5);
    T('2492175834477364323548208389899353.5366103107869118174185050', '856641500974.044541074', '2.134901247537931357140039876607582155086593820583188669e+45', 55, 4);
    T('6861153045260362.896897503128', '75987475149864.22649831740', '5.213616965261370881334457970402102694487307532368272e+29', 70, 1);
    T('577888634825.6702360626342046935865162364130639211', '869991183664632390584355561730335.285274912973421146177480951761', '5.028e+44', 4, 5);
    T('94175275883712561274683127200560.33295358', '46067217741438126625730039981561622.867423735498733969351714', '4.3383929399949934515477587251381015167695177229731207465697e+66', 59, 0);
    T('552356061759153047178068558987.138440303804280858609', '65968526975612265202697278316287144.96356682558285', '3.643811576030164214203390158245623704903138859e+64', 47, 4);
    T('6253407696894764886682399524964126.1965093801721170', '79296890426877.5887748660752470831423869780368104', '4.958757849353e+47', 13, 5);
    T('67348199603532133002034315.7650408776601378406752', '6225572631139.0925', '4.1928110820825e+38', 14, 2);
    T('861354738766017469954879276610680518.2533214639730121793797046', '38499357286445853.214', '3.316160383812e+52', 13, 1);
    T('91690827253567755616.945908163897363', '2760898381763777457488108012984012.52141', '2.531490565869572798713e+53', 22, 1);
    T('290584771704208229600333032235885912233.769842595966516420560', '92904.0', '2.6996487630407761362789340026842744790166153e+43', 44, 6);
    T('7528822270910725270880976811.7458', '4294356338675920124454666.4824277863191799103', '3.233144564185e+52', 14, 6);
    T('1000692292308436612763452547002.24728582168520761', '2846566058725540554749274977411105413.403845358', '2.8485367145134529698317203368033967439011505950486528470983446714992102985476e+66', 77, 3);
    T('27.664158769285', '8707.013950758561944991383636315764807558661', '2.40872216e+5', 9, 3);
    T('999891005546980658321363332234723010.4452', '29746376122129908635441.7650518211750122015770772508558563829719', '2.97431339321351694800075e+58', 24, 3);
    T('4618212789026387215672474072304905.96935258053740822767466871', '39864361485689403112801977680.8468799103', '1.841021040395817514414868581e+62', 28, 1);
    T('415938779518721111649642.71155340', '139903375155471049600520360107437123539.7304904', '5.819123911271639782487183e+61', 25, 2);
    T('9889.0', '96308741365573669422816399.920440497680', '9.5239714336415801692223137881323608155752e+29', 88, 3);
    T('95635309.9', '926.98684379', '8.865267407907954e+10', 17, 1);
    T('88575788.0502419685072388096705249', '553191028114598.42594197951861073359460565583566', '4.89993312575741160943911912110965e+22', 33, 0);
    T('1775159542169756455.321005557881810870409647432609982484', '55625099.423855260047403993205027', '9.8743426026398e+25', 14, 6);
    T('515259150086258840852151814051017716178.7097622949662457007718800', '541700636370.5177952650306141', '2.791162094974585532303276e+50', 25, 5);
    T('813485557706989196103431659174.5666336212414352580979118078876990706', '124810077011030766106.266139073980858247', '1.015311951047706339217286691425262793285723632059709476289204474598814e+50', 70, 6);
    T('19.96623727686275182933207101788132428', '553117577331.7274', '1.1043676791008751433029233470992013414681994771961272e+13', 92, 2);
    T('3819151518353677284680944411568019254.14562218551061521935', '58944439455303243048348096.1424370030306775688404841002050', '2.251177454442277831320135558126280638735912053866400669887115736149152269e+62', 73, 1);
    T('511121754.424904741917619489', '130965449181561912535251.43300109374763492317321149069201933551', '6.69392901547256295792289742424284941212639899396207100737763e+31', 60, 3);
    T('106389.7895048958790', '30942614361497460152820441125162.0296968314605896582', '3.2919782286508829863007583e+36', 26, 1);
    T('4903736.0', '219522363006209.940637847474606637058451852728478', '1.076479714278619909463675623737651982464454491335794e+21', 52, 0);
    T('735.44373567153541381975604149989144559178', '419043713810371110536069839858823163920.21338139641061', '3.0818307429e+41', 11, 6);
    T('1195897934243581064495878.2830333272056649403345178339790', '9625816576230399881577959161736410285690.7811', '1.1511494158921555374653197330372049079207561863322600091849345305834188595079558e+64', 80, 1);
    T('18628062601744214738.819414527362490', '4746793938010939417545178.80655661682792940778461', '8.84235746348e+43', 12, 4);
    T('178466494.09350323', '40994604075661.62767782703099589917040366328', '7.316163266134569315640678335201680098877039e+21', 43, 1);
    T('296086943052388313479063.031531462078749575087', '3007824626392908323903.0', '8.906e+44', 4, 5);
    T('1912183586.1332', '28219882246146599521915132.214', '5.3961595633693227670708e+34', 23, 1);
    T('5254567.25408512605022243', '9989121174690089599151750.6019128467518390654680155898775709245', '5.2488509022e+31', 11, 0);
    T('783839843342909870942960.464542579366857743816', '8250800378466363306.1993783338378699252607047035126164420', '6.4673060761106956876683116383214081652321854128893587677715969214095587877521941373e+42', 83, 4);
    T('666321717442203561467556195.9', '35589004454131024230817955.5916', '2.371372656993481632714868409811130641207734854942362999444e+52', 83, 1);
    T('8651004828702236370690096348261528380.6245852346549364653005838383', '5606540666015403911603687440588226812.227', '4.8502210374014711531082678118097075111859605176927412795364533874001790251398295924461682582565166e+73', 98, 2);
    T('734862887931089782312127645706848977.3772', '5710492520.828531312100695847005548519024', '4.1964290253649e+45', 14, 5);
    T('216009.88974234781503700', '682524.715573398412417406151347114377', '1.5e+11', 2, 4);
    T('746944822.812534747627', '523061101176159564.3123313732200800757848584607850', '3.906977815381558162465434344324907271763e+26', 40, 2);
    T('40919634392.96', '347753.70522950364732303694', '1.42299544767884712571095046053405559424e+16', 70, 6);
    T('789679083978006.93282430283877892652878090', '6433970.5703077825052475077841727843378854363', '5.08077198630210453985583950444608925331366e+21', 42, 4);
    T('444557.0161537959987', '418707786524579490667643359613.272101', '1.861394842177276512180214845666586994837570445481422687e+35', 91, 2);
    T('45643467143423939437.407', '424161873247226306448316.0387401941723', '1.9360218525052923580172890506496877494601183473342802e+43', 53, 1);
    T('24210.946857370', '7378216075.13930528994935319', '1.786335972973907791946643092618946845103e+14', 83, 1);
    T('7291983375761053455.6032671221296', '46201194060.548747044970', '3.3689833902983178522177e+29', 23, 3);
    T('3.93424288', '545449.70532949934664674558', '2.1459316195906808585096106732864704e+6', 87, 0);
    T('37018928968492118906722679764.167871792473914632210', '900913968724536689.38161798527403906709', '3.33508702149359540602679055471596554969709059e+46', 45, 4);
    T('144219061071193982467555896498635351454.869685816780511847', '3203184417609782988328207836763417.47290538317', '4.61960249145e+71', 12, 1);
    T('4439779478586804920360571565837791049.34088823958650445899268236768888', '924005874721412101534033302414.6224976', '4.11e+66', 3, 2);
    T('8203030188.418278321622541535341316163834591465706', '258410772695384763038964924199765.8115156355436746739', '2.1197513694327349637760655157992524918930565203927707092190815e+42', 62, 6);
    T('89684.3907319423210323467', '259064668594087344915155914407297394810.159716956053938', '2.323405696303327596366734516805e+43', 31, 5);
    T('8907594115715436115004526701.338994', '16157925033718408507.62558502447429208272770', '1.43928237952521235302349478075003e+47', 33, 4);
    T('5255022560508838913194742.896231893309636820259406384416822525867', '70.688349953948835008630230425881488', '3.7146887377314507222216663484929899758420814580855530507192567661073704823214620454954e+26', 86, 3);
    T('861748.4464', '578273.4728337', '4.99e+11', 3, 2);
    T('411337488998888674294918289.7473980152808547401602779635116553535', '46012805747930264936993675908173377586.1873', '1.89267919781472669124971154849953009991202910560428387847777411827616385932303202172068404286195e+64', 96, 1);
    T('24555036.09324841138650460970210256801868', '7615341548747128084.29172069075586258346', '1.86994986591899986592707583783846125623520817656538375070931491e+26', 63, 6);
    T('3277334494365215389953.868797424482353142953564482553228078633', '67630302127234815.1966210328760589224263014552808316869531', '2.21647122025927863845697528776552879150455335963796501506340515833018e+38', 69, 0);
    T('52812.702604160774350557006', '965495062383.168011', '5.099040359542790651e+16', 19, 2);
    T('615709783403996786167449825.4', '29583348751318606117269106.374345238634727558998897089012365', '1.8214757252039277755929790754884290333306994665361722985021204103479890606578777691071e+52', 93, 2);
    T('60823835009751311342179448210875750.3694261620986001131', '86.4789339811946774418601565054022773327', '5.259980412291361173392335615188974731727490321702872877027886861237e+36', 67, 4);
    T('22983.3429318432317', '44723164272990030074347040.19796413146829105514057306922877', '1.0278878214832891517820140979172945426693995039326518244690037694649e+30', 68, 6);
    T('9258940535726456107245728.03920546540354498729286831897', '94674.2953', '8.76583670444506705519870526047228208989151793799762814180371841e+29', 100, 4);
    T('29673957.496435849492589903429392', '7093459431980235.161575175460589386453979770924889647', '2.104910136872734819908020991206629239677984498043665603150932918561962984144563046e+23', 82, 4);
    T('30.62503282419001381053145998245895351', '6580257408503921283143487810522.577955596491031943016108159292105498', '2.01520599127052105813e+32', 21, 6);
    T('26966816875768436130.87289709564603436562', '7469326553048041942.0849425832950130660947000702', '2.0142396134136122052617443284724692971207858836758030125578944078763797675969582646653e+38', 86, 0);
    T('4872162921614934207351410138.73536127374359', '8515949520004761.043949301708569545489', '4.149109349371169316972378855940814980096963505796040970243180471929767582716551e+43', 91, 4);
    T('1272460679563584444418153111380209992.6566539629', '1045147325.75147001907386', '1.329908876369778500637627565747033841381891840987812166534508756799794e+45', 78, 6);
    T('61748217188.5389407017', '54763227251342397832785.4979797421', '3.38153165026120477805373447042481239057400093975903157e+33', 95, 5);
    T('369112388727117026680.8984633304463697711781681', '923381410.723973683615339200202860847', '3.4083151821854108106644356097939185241465450618620961241866387e+29', 62, 2);
    T('1337808447158.6752709094440571733513635', '61110915841653711515820502358902.660515', '8.1754699427e+43', 11, 2);
    T('8738151080334510608871.522142081367601465713155883313', '8270122368.5267051', '7.2265578709040229848e+31', 20, 0);
    T('9955952798.64159784154', '2698680313003683702.10515518674973454588870', '2.6867933814888008002110473329084776099823227454392762e+28', 54, 1);
    T('1445.70', '8323993669.1298813583745291478', '1.203399764746106947980205678897446e+13', 89, 4);
    T('9955041303827030361.0940879506406836090', '146978151427.0', '1.463173568215928783028613348203722497150398860043e+30', 57, 2);
    T('2316766535060762393668807817297664.538', '54.75194732111626456391652193898432', '1.26847479282971919963147660608589333801108763816626453229964071802e+35', 67, 6);
    T('7090545582417693409606907493123506803246.7770707713205923425174', '88811215715599.115050', '6.297e+53', 4, 1);
    T('641229245482.50171228680422017010527335498', '2634193544251115784562222846742093978690.277631938167544405863371440', '1.68912193883501996053842576073574628538900280240292252510910808234503847275e+51', 75, 2);
    T('5912957089633283564275173125319187.2406635560764944108306530713747', '1748822071315269183303328380215444105.942346983907434656527', '1.0340709865090784745962280654102495835940671300936701896098668894756854394715331018834641672292e+70', 95, 0);
    T('49262809507049536453650.40813024332130247406399135605839', '10789.005', '5.31496698085604984046116521569235844748999189e+26', 45, 4);
    T('464858362870.6178706663889377247127450928754', '14.432682667120079479280247076905702928', '6.7091532364685828556998112966673746250340858045338746964803e+12', 59, 2);
    T('38055.79703693544739696118831141742407', '80439111076959035977451.897810869162511386944583225041875874', '3.0611744849762590063149445146952e+27', 32, 4);
    T('159470788.8', '91371245732582208457064311598052.521621684215209942417854', '1.45710446306135186435e+40', 21, 4);
    T('935939308260143154199898073919310190104.52114916114849958795', '21909838482427157115964.996353858704712733', '2.050628e+61', 7, 5);
    T('26940504993231304156820467268272964177.8159892450077950051570801737653', '72830708130526013110388.0876482530613883428029550477511115761', '1.962096056051007797451562604900264719250335607224011941032744945648128552885870836496534385e+60', 91, 3);
    T('7156725285.722552000682008100604927202680417225', '33670457460854.7022858088052039088190103228', '2.4097021429194440196221418446384606519e+23', 39, 1);
    T('43255459300094991.709', '1426109019899618650948114.0489706885987411071046705745167815007780', '6.1687000667766313174627633441111663399658625003920108930684073651058259338487049602e+40', 91, 6);
    T('863005133178.05255832610321460', '1301430524805690845.34262666088611698133360854222232008', '1.1231412233819180615671323377355071717791630250868235406469e+30', 59, 0);
    T('556.9637807940028007590693438', '764726124.74790403478819965900227667920549827310', '4.25924753711538863156352436925e+11', 30, 6);
    T('8250242546013509082052.0', '76633773299795510257340977400811613.0', '6.3224721693952698375635524492428809140253613977141146988e+56', 56, 4);
    T('3852713901341545427223804924525.179552644092331041717617247157193705', '2644.631898956658655246634246486218720532288', '1.1e+34', 2, 0);
    T('7680.04515539566708022320690772', '6749053800.04329437056134019889508', '5.18330379405272201322008650107549178833e+13', 40, 0);
    T('8160.127641', '96983633048203586975162910644615756672.3848710', '7.91398824761247175471374447629142e+41', 33, 3);
    T('1088.32456004850', '7377062813429749.07521444746890350', '8.02863864087608129964451327738541602220181975e+18', 55, 0);
    T('6551281606364271.34', '107908464.652299248', '7.0693873964761721004626796914995232e+23', 95, 3);
    T('7496178599798225831635.30973511', '3593267676600732078763515879.636882738356223882', '2.69357762606810999560027466735105614e+49', 36, 1);
    T('567.27577707981768293763420819691868', '2894454075567.584200052757805203', '1.64195368493944666077904818543453791122088105380657818158427189204e+15', 70, 3);
    T('971852099456125469.04760077923344199', '81620367510642040588760198.07815607734407665100655', '7.93229255235980003984589506594035042317693061367888014980831706713681e+43', 69, 0);
    T('969.6344751774763970128018231', '9544308463531930272295.89629239', '9.254490527968729333400675873870297309656502614021122656209e+24', 65, 5);
    T('62715697273861191342.555739610703570317717', '576445796602535308641380532.23023323895786', '3.615220007451436645053319256579378449964531195305865514195032168136119554217440562e+46', 92, 2);
    T('81554932550361985428678562700911876728.85084', '5988761238.597521225965318140740', '4.884130188740431444850021134492413377118512820973516323055573659855872216e+47', 93, 2);
    T('131165.00662', '1753084701101.259149758706148515419948027369089', '2.2994336642536737766843626337265976065508992249986837e+17', 53, 2);
    T('2645974971285594476434155261541748885507.2364727867709243652397902054503425', '65304355708820.461190786336', '1.7279369072147046752102791868923556491e+53', 38, 5);
    T('8445547477320974946059933.3361770', '81341641880365402845948806216.12064503968241111372190915052214409514', '6.8697469838386619295866825962e+53', 29, 2);
    T('43.63869135131034494752305470187657022045', '159811110257518.305927081569', '6.97394771503806805021077712942858956505349009857231830156346188605e+15', 73, 4);
    T('768757604255378003733255365968254364063.2854800223802723262120449040829723521375', '675988335731724812876258.5483993033504387321498314388282', '5.196711734818e+62', 13, 0);
    T('4536913339965236892534874.5283803941560', '4049.4055963906950449', '1.837180226919483027916149873254947819067354139506e+28', 49, 0);
    T('75701.74438085', '4266483352930494807013472368857.741905', '3.2298023218869615263744223706942330178985332451925e+35', 53, 1);
    T('88529024367.74991266029', '7717111917624806.902260725', '6.8318838900405979e+26', 17, 0);
    T('4505398778224201127.94295622487844601114', '8890712576133323510706461057654076952160.2133522577462247', '4.005620557805361549800127676648525e+58', 34, 5);
    T('175.551294705705894034357924269687527', '4634758158008063368.30209109439361450741052017517500230', '8.13637795286148136269544962864111128423150630527522864951533354647551975549778362506312e+20', 87, 6);
    T('600562301480351240793911.86220913228529587817752966345203', '76267376912045103247093.50', '4.5803311406167210953654855432860398094960335623926419886681236401462805274174805e+46', 84, 2);
    T('6264736.36324265866852331123033733', '2.01671751088028977180566', '1.2634203524799973459385747916e+7', 29, 3);
    T('4299890.13', '568617871106285836305.94', '2.4449943717115306484907070663722e+27', 33, 2);
    T('8052.30910038684022442119', '66359950761202.520092819306887287653457546035388675', '5.34350835415653677731384450547380657159542066913e+17', 49, 2);
    T('68099722156.3398164141976', '48391033050037694735525778730006082592.232587868244909623690', '3.29541590556582432414643079420948926173335551262e+48', 48, 6);
    T('7870846084.51', '95415934900493874367390.35637034494165097', '7.5100413761141326752981423966426302e+32', 36, 0);
    T('635634055028855631759693883233.3801', '4.704264', '2.990190402246264509684384585914994e+30', 34, 2);
    T('8.83931307625221775139', '8065924078002574842497000020.962589256', '7.129722817474577300079e+28', 22, 2);
    T('3008792.5036514267998003962852677914363135259099', '82613523965369607777062595968159526.967939067669516272030945866424908', '2.5e+41', 2, 2);
    T('437548.12677611783967563640121', '52693528700934.350329358', '2.305595477631742709589338704015340003835881732972318e+19', 80, 3);
    T('68.992837608362388559583930', '53835658291.91853522889087032183864354', '3.71427484e+12', 9, 2);
    T('629757372223627.93974742319570806', '25024.4091024311586731720273', '1.5759306117796e+19', 15, 3);
    T('9014766937681169060874668371565.8175296141774744115045715', '8306757121.09152569596641923697794161975', '7.48834794545634971419718688561746908110132810856340425985488475316352228019894883326872e+40', 87, 1);
    T('30457387669903904669.76244254710554594', '35561293179.8622', '1.083104092422172788367280542307033015e+30', 38, 5);
    T('203242426660890825637697607265.953495437898579143919543816352856', '50905073635215861.0', '1.034607069497260676724438908424965275695e+46', 40, 3);
    T('5434131484416796037741.58075329442528608703993424379886042', '44432714.196701', '2.4145321115438605225757306093884243228484037270344036314425725952347442e+29', 71, 0);
    T('995805638776079339163015550.217853726550228441569', '775817908901874739641984321225160830.7253697472837998069981030835655', '7.7e+62', 2, 1);
    T('4523223.450361', '136950287944110657143293236174.9548', '6.194567539624926677332779233216727123092186828e+35', 60, 5);
    T('54086365.656089584385541385733', '28223243.885258227023623522386991917', '1.526492688779070936743407912318106747634439e+15', 43, 3);
    T('9204003641145824.5301611454025128', '2239822.525648541', '2.06153346815896083177848295389636004413110538248e+22', 97, 6);
    T('359978.87623216506481985710798', '1839409867874093382408113.9784103037803946893495382', '6.62148697167671356523584731961913788867526438276391168386505606507696000534836e+29', 86, 5);
    T('937.625670823368172475', '5972104440793785491940.02762652191290521744641', '5.6e+24', 3, 6);
    T('1407547084373366121264.68', '64443955.0707574445222887772735797878965', '9.0707901065e+28', 11, 3);
    T('610161607840563712353284.19993253057', '104214959004313936.00997597491016524', '6.3587966947111e+40', 14, 5);
    T('516816038665210214116028889255075.007423206276288271164', '840416209055259064478920710926672600.66327840408442209787315653418018', '4.34340575993972159139812516e+68', 27, 6);
    T('861312008486.10609733806716412050', '811026588453256313375251.504612', '6.98546939836308779162334283162551598533e+35', 39, 0);
    T('148767214141453978216504.07233862984462613761083323918195', '9452047.804809984961721646550677340207977', '1.406154819853427026420172749473450758987338128937526707241851592508779580061e+30', 76, 4);
    T('6003964653340.731820125999573', '2816552824157148.677818153', '1.691048360050653434914e+28', 22, 4);
    T('8191006350135813582368.782288', '68775131855405548097519520781040.5860276140593675', '5.633375417591e+53', 13, 5);
    T('55528924028986750726475829700763548144.0', '47.48506390808', '2.63679450626335501665429e+39', 24, 0);
    T('590754214149314455579929.88284879272694', '4201.994672396091942', '2.482346060550959338726531878543538383755641667249141e+27', 52, 0);
    T('2544.9507631565142099001664362279', '671704550032772023811909897835008238.67', '1.709455e+39', 7, 5);
    T('9224326940.126341529242706256', '7425893175.008682', '6.8498866468732918806217e+19', 23, 2);
    T('6350561928972456742.977867728228120389590431527300436207258', '1092836950312688541292892108905579.1225354288090948596265878451974', '6.94012873123012420696343948988e+51', 30, 4);
    T('68830.375416658409002674904651710', '103105523772783095849190733899346.74902235934315019', '7.09679190881185877551602132459237760646391464751e+36', 48, 0);
    T('36230089.1531689024445', '627237850467.520596541', '2.27248832426802959497710631163828731444745e+19', 44, 0);
    T('2186284147078259032810387695230027.0173923832464283', '897776869563853152795737977.9905459397945557285314499', '1.96279533754099810168489540853118759933349579681526592840768102596836901568539088708502164338e+60', 93, 0);
    T('3095792482.16', '2024142282207266169507145321243.027', '6.2663244600794397384863207e+39', 26, 5);
    T('265704517714976660.3', '70411948003248501572.284493154371786295', '1.870877e+37', 7, 6);
    T('1508114107761805205504922605314229.1647588497059775590680665308100748', '7536643220052483463746161674780523892964.8530876735088800000990421425', '1.13661179653285096291e+73', 21, 6);
    T('2710032784468839022388927498682.49960266', '840954714058125379914477175.36582402075904800524419797370878980528', '2.27901484535113784716305982917746560075171363469e+57', 48, 2);
    T('969572299297211.5876063282643818100015803437', '5.537943255521590643', '5.3694363756335539909177282471357370332313860102248659866439991e+15', 98, 5);
    T('1402295757087189163200343369432575.349491643187388945510', '5766278317.0650809336485845536692416512493436', '8.0860276182042206673621972989250133370991124210556538009940500486512528830892917845204713e+42', 89, 5);
    T('5354649110052.69397892581790381481326', '104990339427409186728.54155151392761183144106', '5.621864275791068700550111986640330145025196924811177084380407244376882405964556e+32', 83, 0);
    T('9162364895661062412.375725825542183', '25687311401692516502790431198806844859.41596700854831748166867698545531', '2.35356520250781672832025e+56', 24, 4);
    T('514964715480127970521617293.74984758223588', '13829.36922641262611351463511383047', '7.121637188949215459219224428122249077e+30', 38, 4);
    T('380346441287247806262194739.506357', '11854034427235293894584820.83209823475987395422047338612411224', '4.5086398092954629e+51', 17, 5);
    T('10059301409680.877153557416485396746871450175872', '5781159889214593115.64779685', '5.815442982316690012122632e+31', 25, 1);
    T('4832629569868076417309733698.017245106198834540785136969661131255', '963834347285846954913829.9195201124633696728407466018395010924621', '4.6578543671480807565971479922026268670546565498760420748577838665826709436277679817065624170993e+51', 95, 0);
    T('31729.01591752229925025953910619810', '558040501573691008600.8874064385', '1.770607595705376969592373638573677700417603453820254272919646685e+25', 90, 5);
    T('95993000172711628125252309437961.22278034044800031220386916202201183400', '854303896050910547457.95694737368332153417437029', '8.200719404116327298283072872530149558087800177716350767382820215570696332778677164312507160846695508e+52', 100, 0);
    T('31549.8946135108744192', '58334840330794.742586784781501', '1.8404580647325579648206423628622191349577222792192e+18', 85, 0);
    T('649427283805794915441273.3225521111848607136752116820014', '527103475637317676066202453688863.271154979714816238713878861259', '3.423153784677372123555216572640999046530525992536959152942509209e+56', 65, 0);
    T('39.8934375', '5256856529488254755.23352868483738', '2.0971407740560659806198657449301721669375e+20', 49, 5);
    T('6.011768816343683088559674812963924207', '24544.44661757433257825079047125796', '1.47556e+5', 6, 0);
    T('2466119731361.172574055847556207492931604138754', '29362987634631934751150.743395812857253473512954573971441360908', '7.2412643177479939038487461762750220410925822276950833e+34', 53, 6);
    T('5829517148837052945144357166539.3770620102384918282112', '56647738774302373.7490604431014845689652839606154443234', '3.30228964627637346084050901276045429132707901013291906315496095902743224329e+47', 75, 4);
    T('16631783077410031371360109.961503809151552383572649156596724961', '4531644106445400381839483787032987.902923669556199197806546', '7.5369321762423512942649032219521704036260233e+58', 44, 2);
    T('961876857471357.1187', '714072660279949272692.466680912400570645099592788349', '6.868499664762895782288666031717234391277363e+35', 43, 5);
    T('9474792754.700', '214530.9309150201409', '2.03263610989267907240389313723e+15', 95, 3);
    T('5571294602574731813411418.41971280027507742361338398', '22629423556.526', '1.260751853198507953098e+35', 22, 0);
    T('1229683499788.0', '8.08375', '9.940453991411245e+12', 100, 6);
    T('400962282951414773048843331522376751535.099374972', '26727658015113922763655520970963204.8225861', '1.0716782775684757645863775606988031980813159252371794934656945583548607797136938258655e+73', 86, 3);
    T('33802297582166.97875606726398666', '11224086257327.0606615299175092852834337860', '3.7939990376e+26', 11, 4);
    T('1808189499865914.615', '9970941759542407429709861494858283219913.67', '1.8e+55', 2, 5);
    T('5909409003886416366826767576255143649100.632950851615961', '3062157.305564648298001511014895', '1.8e+46', 2, 6);
    T('627880090233281426307703577042597839136.496778904471172497235542094', '7607136602688294035656154906019.450971083', '4.77636961651282397784691356128946055171713190985840608234583997729749e+69', 70, 1);
    T('462094592919709768967.58390567098856', '72908288113737.57', '3.36905257163904768e+34', 18, 6);
    T('61309527113908065666666424.90', '2534376123490672442525612.72458', '1.553814016599926e+50', 17, 0);
    T('939509146528128015234582789748470.703178342061342640', '8856811055192644.5351369329571238860357975452274367493187', '8.3210549954249303765746691221513700272631778128339783591326538883602743752105568108096145634662317e+48', 98, 0);
    T('65.8297181324226590256164', '60521052321918615602816340847880.8965845384168954', '3.984083815429506359580718268230904088202014499115e+33', 49, 5);
    T('8759441182531698525695872958.585554196397028142208609950', '781146906378481162036402793037706865.0267695512124', '6.84241038133890102782621214290090035652965477767009582849136233310266301471433717963832416e+63', 90, 3);
    T('40386487330.20455391655177414691931173953', '315289430733516408897365156.8611884025722', '1.2733432599666566742184457268523722719262387941200401785697250283452911108419066e+37', 89, 2);
    T('298955115667425907427008663684614.2496589246521190548', '396171750722654948607319248580958834515.2598151032998076', '1.184375715614579334892348701891875707776784426370715698264961026163870938235072e+71', 80, 0);
    T('50085756372613222649.8893747633274358164691996571', '7908184450869832120550491011288977628525.3643459087841530246210053', '3.96087399755954492899208375797886959545997005213094462135074847321743143236019158209919962203339e+59', 96, 0);
    T('5250901490155837333647.18955848791489776691477921449792442', '6791149052586.5131335300511045627524897', '3.5659654680096924727513234020192585873518840601625932142648267295990184547795e+34', 77, 6);
    T('1532141978.6', '394286940178676074.00886464487642306327670058258755758', '6.04103572661496597560421910940549184659437029885473178362627788e+26', 73, 4);
    T('6050236408697.47985807923357715594413992448538436', '97051981285356337960.91099340682828942873', '5.871874307088893553178749448851272098638733085822193845783869371784772e+32', 70, 4);
    T('3768180874493657331676689852.42', '4172.488090205953837', '1.572268982056664130657415229946830359636486273554e+31', 76, 5);
    T('64242.101265008897922865877574000226987', '737471385.325298636580653847637357883755', '4.73767114161142319410832142057460833505653905872983596661404670713239599e+13', 73, 0);
    T('7469286180.9205965312', '2623430588.948547931359103145663837151408946102', '1.95951538446377708947981440239875518182305454509022353341489613824e+19', 89, 3);
    T('4727777935451.245825', '99362.69621896574260742824546660571531', '4.6976476279097116824984020318240888994139069080477608075e+17', 86, 4);
    T('726599070609449017543771589477.44065527425110942322377981042484216', '87465091821377.260127040258854472807248', '6.355205442818283760737e+43', 22, 0);
    T('47517925714.1154613', '13.7659623912323562302223', '6.5413e+11', 6, 0);
    T('2167772.636771874105365396', '54253104.75632412022220826686943225282760211999', '1.1760839595067744246156011e+14', 26, 1);
    T('491401637773205472132528747826.075343649275777068730134417', '88632.922', '4.35543630314247743014956e+34', 24, 0);
    T('621802730910270432797980090262401.08945075376977', '570939108495006408629451458932305375347.36370150862232920465365370227200857986', '3.550114968456701656443e+71', 22, 6);
    T('75.17908193923291929060002044252655994275', '8301748833998612602.40574359069483642541078949', '6.241178558301130432977339296465287842110637499483924198528303965422436745390867016975e+20', 99, 3);
    T('607220220388030748114.595331839896813180645538869', '9931430633675994549322669225199725732050.3820707735082052898627149461574979253', '6.03056549814917727759201780164163e+60', 33, 2);
    T('2355848289603539838407953.555', '132841786.8824', '3.1295509642e+32', 11, 0);
    T('79418336678774919184865645266029863040.0', '438173710229666729467.6641670770876760413848196625403799520716', '3.4799027242807634238958825327714675282473633449369268552046702920539926797345902e+58', 80, 0);
    T('1.3489064913910891704215424355851', '55542584422377231096.9492584879588636', '7.492175267598223594209790212474856411589354396875082068308159709236e+19', 94, 2);
    T('59948255.9814081128497467900548830064406', '47.017174357127930758117222245329', '2.8185976038836026180663838142654238734811761438488085931301731e+9', 62, 3);
    T('477.06659373', '383073197200377905.50609099838027888030445085856884881', '1.827514253376448596485426443646945926343027564556146844740639613e+20', 84, 6);
    T('28229721177989607857372.414399138744898546748072120900', '1466089502682990.96658284215017460676', '4.138729788271828209850966165364802314127550345e+37', 46, 1);
    T('82003364011846127558683209977.300885', '2636148839773205489240359177720231.04185', '2.161730728973280027e+62', 19, 2);
    T('609464556192359073.038649', '347033002736529176480205331193671.27887286602013853029047771', '2.1150431499692048624477167911273363248805951592722961572275297011780707995950301379e+50', 98, 6);
    T('842346276133818077031301960252399.0455968196671284285', '6739442.912327891352696046181411', '5.67694464041585306266607284928329006e+39', 36, 5);
    T('4500710780941152540027270717865820.38', '4375887491.5850644005899155878849595972491861', '2e+43', 1, 2);
    T('78653439190.841976755240313245806872', '78899235.8196041517199', '6.20569624674113627973901916338213996344401715e+18', 45, 2);
    T('5591965161022757689244589367480260031770.211411504537775668717427', '96176456578340332070003441582674104629.1974', '5.378153944966971580480858705748071146792408769010124138675382e+77', 61, 6);
    T('3119385778003319710489444344009200800908.205390652', '55940453008856925566577775182.92526388106607946026', '1.74499853530891307757825074787e+68', 30, 1);
    T('58079340998077559225086838622704048779.533736868059266008279059', '206787491.765445069462757564061320558390431568', '1.201008124838243950018427216686620521422627334417986034477096881277311312264570193805e+46', 85, 5);
    T('849187950.125000518758291207726616228832', '677205599996720.913029743443', '5.75074835274386490163466088118004006179098877684465733001139548576e+23', 80, 6);
    T('94936257269792207.119155465003269053', '6124165170303014177752731.87280', '5.814053201705877599e+41', 19, 1);
    T('9347248980249819530793831575736615.201857737636661333740339870', '8932229286202334072721069.947070768', '8.34917710868123405640832551193227941954108437079783160025317325118791381152180711e+58', 81, 6);
    T('1900062424870486811384.5944144720652980129046970827', '35701877808392582351807011.51681389404445350288', '6.78357965210442e+46', 15, 6);
    T('85977584886253466799424282284326413607.6434731521727294087688358949204560', '3164407249337119936590502547648464.67983045583659579996964', '2.720680928945580687565373468103396256712374724772581419758662964015650499682620432724127359778e+71', 94, 4);
    T('40.430228557118250666581119592326182748633', '7501719440157015415287758.632970266269', '3.0329623153692530053135946381547722976134738548214905711591275e+26', 62, 1);
    T('26995807035452069366263621496.33070736093592975208562593158271972', '52933768158835402.18704681562120365924806931501788303878', '1.428989790875277482793642997754745847961482617e+45', 46, 6);
    T('33927976964.90', '7777318.3775194', '2.6386867876117164520286906e+17', 46, 2);
    T('66806.7166576691062602583102338219409369', '1372090556002308795396.83373817110963530929283419', '9.1664865003509908712557707842734360758369104401124401228170850634623586739680585e+25', 81, 2);
    T('43857822.23', '3128913517.3520306958166731466442921', '1.37227332817069382586630855535540083965993383e+17', 91, 1);
    T('6256727115959461728554547778394375.5906104', '769.822098622', '4.8165667989130863769533536733657260067249888459788688e+36', 85, 1);
    T('5450.48482588537751472306894488300264988', '3403615.3', '1.8551353545801306955387412723758644529072111164e+10', 86, 0);
    T('9992470587.4736214671341072200448294767375786', '934736390173658706476171.988862232259472910609147772', '9.34032588585155166713730253513864285546822313733133988970590367880952575373365e+33', 78, 0);
    T('71506276098321761493781095.951399', '4757184118219622004715746218366172.9444157285143752', '3.4016852100796364193e+59', 20, 3);
    T('997470230013805444784477.9193187240417048057277900920573', '60719078434967459454882057652.084606652159101049132077704388465486', '6.0565473132753285709675207886038416863575e+52', 41, 3);
    T('7.590325990279622', '89244441132713872550435922.611476421951768739975273', '6.77394401017617855171533087697816151365183053562864087854635787e+26', 63, 4);
    T('17471939292733469330.9384535787273033036660', '21169.3', '3.6986872446966263230744e+23', 23, 4);
    T('6232541923470374.25909799508148355281075663062243', '322823032971415822068789668264.0378176222819396301', '2.0120080868562080166629375522321626679634741665e+45', 47, 0);
    T('7100679922.6363535387635', '1730.031724289876222585731055935', '1.22844015301890756115046859258893393987749188092363725e+13', 83, 1);
    T('7474.026737276201166649285221361116416533', '10848133535577.355783295144790484013413335', '8.10792401e+16', 9, 0);
    T('681753551281695492474269.026350076768338482471137434', '2240558937754438980735416521200562553846.118538780123869007858075445449', '1.527509012670032094517552574543552017817536779886094216e+63', 55, 5);
    T('5158.334524924755602555304', '5594783450.50271778890', '2.88597646322058216672478976043082896168473256e+13', 87, 6);
    T('471611508607209113116149.352903401076', '386535113783912.2258501034', '1.822945e+38', 7, 2);
    T('647080852654329741730011590.4071048831737639657628802212178311562', '73364983633.7426664484130167807033112381235448180', '4.7473076164693151347284275126846979602437962640071e+37', 50, 2);
    T('180686279781172899819813834372.3687150027', '34409486794620.768865111464280523252', '6.217322158099422523e+42', 19, 1);
    T('82378923833991056.05143273910167671649017325104156', '3080899926700679523407379556166501611.1241641665297597104983722891172', '2.53801220401823906e+53', 19, 6);
    T('59771.39155846628898', '3622.065688263710211180521', '2.1649590650369591750958727351882866773295858e+8', 46, 0);
    T('6746.19904031207001', '5.343513176', '3.6048403459e+4', 11, 3);
    T('3787.4', '4807.716613531250437780', '1.8208745902088257908047972e+7', 70, 5);
    T('3918360623452732222450229.7507153510044612', '85.73856149273452863512741682417234717', '3.359546032646116870906405958442151621169884833751767334628812521564e+26', 67, 4);
    T('87041.7100640637279', '225531402933576.1086947181712137', '1.963063898448586334637601919187247251482008955223e+19', 74, 3);
    T('885249.8592', '909439673780957960792583.0', '8.050813431654869664327532211543136e+29', 92, 3);
    T('521926623803.677', '83568335342735.088975436740', '4.36165391223272216157e+25', 21, 1);
    T('246374159853635464976943555824643263.44521340574809883935456993394', '4.065464533607455231', '1.001625408882288726161048746515354412291310558773740129e+36', 55, 6);
    T('267539044538370.5005488435286961902112372208216546', '39133850552990.14', '1.046983298605436410764309879584246306982595644e+28', 46, 0);
    T('598718964.78283156039', '1577921027096058232424615939.905641388457263646379084144746325204', '9.447312438520142932147360001454592591354541661247134455359055e+35', 61, 0);
    T('906028.65920349973651445197648324626237882413', '7639998539.2213859076729443694694965391106243', '6.92205763280744886775276562424916926458e+15', 39, 4);
    T('613050792957616130956261.5149479053080262836636678272', '33945291682051815003980839225974.7', '2.082e+55', 4, 0);
    T('7287512220483224002870264974937748383026.74405097882687810566616889815', '88056335499912059.2962518808739', '6.41711621047e+56', 12, 6);
    T('6779421300901858426317782677156694398.398127533', '710510676786810713249874953682.99436740864533433497516', '4.816851216726700149323330315764202542418944561562744569788311135e+66', 64, 0);
    T('5112737679878078041658059851.95612152035730419585571738', '470908756198285458343402.34409771', '2.407632941599493496512321334934932061294013e+51', 44, 2);
    T('5237916996531562582622.48697952673797', '1260830314.701770434837809273931105617', '6.60412453511864225025886780593825805764390628804726541985390553555e+30', 66, 6);
    T('220808724743.311655899363', '4160727003482049007878272569236317.31409', '9.18724823643931677e+44', 18, 5);
    T('490519031042095144533.156112117273493045851332832', '5353991458644208997602512517.750498476749016245', '2.6262347025018110155463770562923547723484452412672318983683095313594002729562219422e+48', 83, 5);
    T('96.45', '727370297946.93951683385124846467156383242', '7.0154865236982316398624953e+13', 26, 2);
    T('22591042995443308.87', '4615530.26246975654846763303', '1.04269642606224010578721370331288411480103976e+23', 45, 4);
    T('905491019859975577201.93231652190111408543600904', '108220956998598038661202538093419.2795610480781452246156492974104882472157', '9.7993104722883099558948527015276449579606569169376420925627916915694957000254290736825598493427e+52', 95, 3);
    T('42603005525818273985.03105165137873937815129696155293095', '7523631399373584462867044567466930497163.755366108979470722075872452593170', '3.205293100817326922576490276896946619499e+59', 40, 6);
    T('3862958916869102803102.15390809', '95610478321839452.0148341605', '3.69339349779469763396706172441856509291561215648309308445e+38', 62, 2);
    T('41854806280886766.54613207691992709', '191722112032343681008204120088332474.8829121204276930295202017482556555', '8.02449185887621461808456704932673162428e+51', 39, 2);
    T('876570672579.661782616982206099', '10824056860280.765780', '9.488e+24', 5, 3);
    T('231321633621011263768085178893.1118184437150476079', '1516.830157', '3.508756298428549937201130534898118857885357913263534314403e+32', 83, 0);
    T('52597050248141240054521.71358006576891714153422406', '3789083095560052.76130246661309334293947', '1.992945939715546510907611914944134639196083857863213419877740291946868e+38', 70, 4);
    T('703244414658479432020229582884.2993033052', '810.53926043853455846', '5.70007207764814050426614620557131588837508530393431500621992e+32', 79, 5);
    T('361764.80599956261921415', '2581336363041.097001786224', '9.3383664859517899995136488293005666249e+17', 38, 1);
    T('6873921058238522612262.4096400029667312699489410210074147006675', '1894844880073538299609947.41468389854', '1.302501412323294285688553e+46', 25, 0);
    T('9100476005639924340443.6081511175862', '104010192383.5296696671979439259945677556059660', '9.465422601283041697723542441566907664347925421318704610047918049048531002392692e+32', 84, 3);
    T('484209867768.0', '496899305826758966026481678952.9412719112356641181117247', '2.406035471683859e+41', 16, 3);
    T('706376184729960.443146643', '807884629514918363546213742083.33240956536440', '5.706704622987256268853704802701699652016516938243665494163229317092e+44', 97, 2);
    T('133942.717364879319941448175', '654.885557261454324026758253753673155367', '8.77171511026124683045781526280854e+7', 33, 3);
    T('739.77266668230301564131557025995169319077', '90095.5473812', '6.6650223342392105953247791664656775374692183945511524e+7', 88, 5);
    T('23261266236663323512199755455220.1976080507422656968636815', '440427571407100910531061.518', '1.0244902996e+55', 11, 6);
    T('58210.177152', '1.0', '5.8210177152e+4', 39, 5);
    T('2903167131641043374609218458.684399424202', '296642194020467881079502683411902405068.759895791378173502971', '8.61201867538107606697593e+65', 24, 2);
    T('3208685640353511325179238771870.57456059577180829733834715878', '95335418345976094606362539087565.757893438993', '3.0591e+62', 5, 0);
    T('486835388661052397159519748095.2174039', '6088922.65', '2.964303024839835113901395457299263607e+36', 37, 5);
    T('52138081359010033172201058551300623.80843497864930027193348943122', '60340592666296570091.0871543114', '3.146042729686254716188682976484401387968146556675448591e+54', 55, 2);
    T('964170100852953.6902630831648', '428649912572438039192531652461722.39422840217184498503112', '4.132914294355773e+47', 16, 3);
    T('96783269228194.2302375676711993932082877049502828050063', '765712408576690539110.614069', '7.41081501906469012350093894298856733386771e+34', 42, 3);
    T('5545166733315940898.93405133605028626584685550065685487', '74777103642738264421326968588376713800.6957592140730767416009838', '4.15e+56', 3, 5);
    T('843679366504784452854791.475804782361305437017169779', '54966911497994.6551237935933', '4.64e+37', 3, 4);
    T('908438593174325691178.77056723571655700305543929814868', '305024118316614802.0136523691068769035646993649159907', '2.77095680927784619515994891776461610225913980069e+38', 48, 0);
    T('73612711202878388927050.0', '83.91448805430597', '6.2e+24', 2, 2);
    T('197308564659097479497541872689746214809.610196664089095833986991128097424469', '91131762172866372193452.99633236568904944844', '1.798107718918249841168539663742142145151921602971231108640755416450542380497377059463844874474e+61', 95, 2);
    T('14124520616942815186519175841453.48982606804006041869019957239', '5770707673032182373153713.5051536737506724833443', '8.150847950209315799217890756056012046704403134315024092848519480038905461594192e+55', 79, 2);
    T('2.951092714091496635175303892814409491580', '4055577420592163194729932264844.62902089378188003315625229445965918', '1.1968384977343518057143587453625169479506908292182226142895514679055882780650788904565076628e+31', 92, 1);
    T('994611327711.585', '87068711.2489985906282689', '8.65995264975031045400309564540470252065e+19', 42, 2);
    T('7135457193578167422229.1034570913033', '2267009014.47446', '1.617614578023833747985961882551536722835948596372e+31', 49, 0);
    T('26111594098157914.53751290072795876409397531408196', '8932772540129747938216678462.58199115069451576751307', '2.33248930739039009062148818592832431319622801673682862127323729520592319225284223086394264659351218e+44', 99, 0);
    T('21923259472732978509.36773513437974', '8051833481420584893630775241466745014.41832800865', '1.7652243464442239468795988736772380258581213760724254642e+56', 56, 3);
    T('3626497701428296599560.53', '4255383934172712205808051876.70997305267314016', '1.5432140055972242620128221851152659757623131e+49', 44, 6);
    T('6498593553844529.9611736251345', '82215249060029041133232537875134670578.95799924204793283276351038897002', '5.3428348756922727779774303e+53', 26, 5);
    T('7043953201016195686862013706738944816.437829664985814035980486296054896587816', '91068.797279372329424411118124059216432', '6.414843461087297328435441e+41', 26, 4);
    T('7.03171156292455788427445847416809324', '33823896321709931536745.61', '2.378398828685291472182813062280174986302492197789888886406764e+23', 82, 5);
    T('9564033690607480399199575.046016296847233679396229419239522211', '1.2623944471716732125273853123012862888685', '1.207358302e+25', 10, 5);
    T('53770692630.641879016577217846002460434', '6644718399130.5150020861499539366154346601700406054', '3.572911106568176867422446182199657359527316310708905999742591353715787441847820869067436e+23', 88, 3);
    T('27375850636672.26288318674224110869266', '280130041558378743.1317295168076384569666735398194', '7.6687981765469701756e+30', 20, 3);
    T('78.282515050195', '215424888315607474463246722892324828948.36', '1.6864e+40', 7, 4);
    T('5.90178281146087446087655255', '15468158655532610600904.381', '9.128971287817211057549542867678875453538902747172155e+22', 99, 0);
    T('981731509.6706411837404039859412607930738219149863', '236.7', '2.3237584833904076819135362347229642972057364727725721e+11', 61, 5);
    T('8674920105607039637648.31', '172569001758991142646137672426660.176215795087069497', '1.497022302963608852155011149380436680468065836282725399700697965551e+54', 67, 0);
    T('157140866915896057271955517.54765273716705605719881', '2934347356634697333991400.34900914375291777104856657', '4.6110588745394435931940391902531059808194774905194620368774205451163441964866493333e+50', 83, 2);
    T('72167637159316531327804286500.318', '1671.00', '1.20592121693217923848760962742031378e+32', 81, 6);
    T('829573991616204690481531229104.46247210580630567328', '3593821467265.4178099397583852', '2.9813408197553781537563740307948064496233084564282251730321e+42', 59, 2);
    T('5972559030022.1082563825717571662446767813266', '6848664745933127.96584652389569105', '4.090405447191697124464355814536420240214861478625429885936004704058884274693e+28', 81, 0);
    T('4892.124783903970995238859674', '409113323264696334108292781220510.5302695538989056676503230', '2.0014334e+36', 8, 4);
    T('533616649.3192859176991709307606', '84473.808146103515399498593238028997876136', '4.5076630458163957650072697561547172107768984028520211005582887711069042e+13', 71, 4);
    T('406750627954016.10717177691333679246256107644202941', '2791443193645095227.9926785236139', '1.1354212719131066684531763255e+33', 29, 4);
    T('704663366164.3347951406729751922852498947201878419', '1339520111672350582767256640243649943814.822795485099684', '9.439107509358642139233237783259808e+50', 34, 4);
    T('1260530863585588563.1129269341754667434279749', '385735902331303508135412139.6262132076349924370331454459', '4.862320100816442558121e+44', 22, 4);
    T('756738475007358888968370.155', '15746230525928100680988544599840.427211224511331', '1.19e+55', 3, 6);
    T('6877923944907605.0210348544180667647081153165977763922', '166602586073125.549051673', '1.14587991603588049237801e+30', 24, 4);
    T('409.9052365065217439567362345537388', '1103912612.86743791092602340954', '4.524995606599595154389306793267182065812743455298e+11', 49, 4);
    T('50273576978967797083200909.062925063945146531535938938', '6083217733127095924323505503.928192', '3.0582511498618703798766426600469381583573167552e+53', 47, 2);
    T('1541327480354269086943243116398042096362.2852', '634605170638557.0', '9.7813438868011804601794258358432578690692894786129575045e+53', 56, 3);
    T('20563436788175796075653382966671.647091580', '835114969386677523179147882.70939', '1.71728338838423083115724e+58', 24, 4);
    T('6765587458207751174274708562.76322103048231538880183639157661749538', '89853359944410175775406544483418563996.505244850850025811480', '6.079107651e+65', 10, 4);
    T('313577388240930348458655240390317834329.555624148', '4288460387108000.5428759', '1.3447642077640159395546768442818549736301587401894962303923900594072332e+54', 82, 1);
    T('5508663213581820540906591.6451680404187803175', '332553894365159027010748173.851100461547385', '1.831927404422726207689112185381301989067882366593813321468538e+51', 61, 4);
    T('82084118227721611136499754395958.33911728599747363474662027', '29060957451409981246959485385859.9228893745678', '2.38544306725232421905755824529846214469429699812953087372515e+63', 60, 5);
    T('471506562433038060150914209354808266.781754', '52289642.95803766130', '2.4654909802e+43', 11, 6);
    T('10462624.349945543980', '1812165999100154217661296.60581698', '1.8960012108328668258567963940513039e+31', 35, 6);
    T('1743096947510.89958868378673482126', '109393.8126364085658609281213727155', '1.9068402088310304607732460300154338496195768051236455741473333153e+17', 65, 3);
    T('6264568211165778639893751757.32279461751456', '4138917100446743916.0', '2.59285284961091098821763062063e+46', 30, 5);
    T('600470609534.578288307', '2892244132897107387038151362861656.452532843131171228879904410443', '1.73670759740353392496329655100613914686534e+45', 43, 0);
    T('550510608687.32087119932', '377938411979226.818205305', '2.080591052250035976413576e+26', 25, 1);
    T('59852987560.81493399300584533134', '1.8058367183076404651', '1.08084722637730066738149931516596766546872101206234e+11', 97, 5);
    T('864117131527765398866230270362230993.5853278976343827320008', '91.79431288419441405', '7.93210383400523e+37', 15, 4);
    T('81842493425692.939407457', '731892.4513292075', '5.98999031362249544507569997570874003275e+19', 46, 0);
    T('15168432853152485073155.34416902669431151154560', '460.380032463475300494186581', '6.9832436093543863534995196253612196912869756025031544099617196620896e+24', 68, 4);
    T('1711619925268239421228186263789028351.10179337059', '57306625188397671995540945.3536561393434485024553969456', '9.8087161522340230184258785366584525172452393050334134398e+61', 57, 0);
    T('455490909239900203688165500.69083051528226368342454805968', '4894252495.383097383496310840047658667762831654', '2.229287519171697500987973566742051345948995552654791288722199617642067542061116369e+36', 82, 1);
    T('9133693782816584077140.31418672041720226187260593928457912', '2227381778060525669.292908828335396548413379', '2.0344223098230371818948737e+40', 26, 2);
    T('111820244393056.020937963625761202091893460323253855225', '168262037360102247.7610310893453', '1.881510214e+31', 11, 0);
    T('4632190733402672985930824019144583.917', '374784825480195523798.6662', '2e+54', 1, 5);
    T('5145837867950654700619122274219.127046847232490727455362421020838', '17447.9377710600885099316087195', '8.978e+34', 4, 6);
    T('8.313823396108722754166295802238', '6544851374880879357635605590163.1204304032648308734409651695417843367957', '5.44127384845389957836504659337447251733374897706158384475106e+31', 60, 2);
    T('8376528633.233910527114471822', '6991903539586913327321184.8190143495', '5.8567880200159308320873522020080406359833818e+34', 44, 1);
    T('5165823215581752701593875.9729011681', '2468219571372679743094345378600857107.403843794052506199796329', '1.27503859629502318374099403094400476843440377201104600410815190251082128286577883992154661919049e+61', 100, 1);
    T('301280964.48098', '4976089675111187.9753618406405561549521758711986', '1.49920109666135e+24', 15, 6);
    T('105639848.3', '75174206529913816849093275770890664298.24158306645824155568249036052', '7.941391773892965024012198e+45', 25, 2);
    T('59818243952681703.25715251', '768572408068570554158.22209708842076162889171858748221877532', '4.59746518011457848664983e+37', 24, 6);
    T('8816141269401730493.84724633587394943420170008', '37767733.589986274627013519148670', '3.329656747544479711493778882550342189075798831530389225859489053261482708936e+26', 76, 2);
    T('92758047587723213537832452603963.457148548430178147636', '656009566583741198069365257757.80111035999700798357', '6.0850166595176346070511536456458573818914421722791922e+61', 53, 6);
    T('73369868580883299749698123428931073.089413817855281485855113970881', '9967069922443164272.49779410439932304340255', '7.31282610346129665560232203278991987233056316404362828506125923358184210887905e+53', 78, 4);
    T('466.0204308941', '41349252981869945693432958903281.0', '1.92695966917601813872201941287777485862535421e+34', 82, 5);
    T('1084313898053247905000614326697337.113749684122541765368011411385917756658', '9935.3', '1.07729838713284339105526035200360534262372366626892014608037755425086877242274e+37', 97, 5);
    T('265235619670861523.127260771963440452540961', '71365647646945550802949.77764818', '1.892871177684996370590963722138932570404444075764816236535655084369710098e+40', 97, 5);
    T('9806786065047629496167104994086675075610.7267996843819416', '8550940695215433895.353076957439', '8.38572460528874062949451289832518514261580440927155184009358752775305697334e+58', 75, 5);
    T('741810205630558618994494504666340319.9382121440321509617', '58450381492939701652.629636639955254232118034517', '4.33590895144621979694654477667548720797429e+55', 42, 2);
    T('9475889516653770000008847433095700703863.1521268654533811269471803567376382172030', '846537776935056670482446.11959020429031202843757743', '8.02169845e+63', 9, 6);
    T('821742610735279485889771689481283.765036235344298086822', '328033181051991499036405598730023416.5264384514110911970355856855', '2.6956e+68', 5, 4);
    T('312660028984031565708.3457445437679472117588250', '6401787144621818999490857040895296217.3280527925959146334194', '2.0015829541870586050455119127339073494668706934289966341279126277922302e+57', 71, 1);
    T('46270368251628032488583156962639.3', '36064516529910390512327519151.1', '1.6687184606558801131024388891318e+60', 33, 6);
    T('963933652928329559772010579.2638288469008230724252190042671005', '1478933361699667.011386605460614829859771330955123467', '1.425593637780734506068607753919332174875737720545028387197832846089512829446702e+42', 79, 0);
    T('3570327166.187599320756275', '79564174.972731545689784734846', '2.8407013536044693198282650045423572335603185506565865e+17', 65, 0);
    T('390807869940937676187092.13', '7482279848139881271.952138798991731768280674', '2.924133849753549626694873315017314430850718139044e+42', 49, 5);
    T('18283744.392995327414656435487649382293650360316', '581298594750118116482299.68669896884343677568377919946791', '1.06283149224185351810269037315332011511601123942e+31', 48, 1);
    T('42877731996626906021.07150', '1410727442.54', '6.048879320151700303765912454548161e+28', 46, 3);
    T('318433879801452210.457', '70956773.169283134476969', '2.259504057848641486419779241340739160746e+25', 40, 4);
    T('33425758085492025166993907.7718336785402729202768793812', '8685602195306312816315.08505520638095', '2.903228378071272695420713147007933208088e+47', 40, 0);
    T('422749681459709556857337202531054.02212430483804412146435', '9.245357224980566667376469788566832744832', '3.908471821841758862716232820975910272692494e+33', 43, 4);
    T('367613704549135507614613917458.78820899583186486255358812', '6188189741025460.35704977451500788', '2.274863355151324954813792533613721583508083340945541353118361266387132460446e+45', 76, 2);
    T('1446563613809788373911767562234019372503.3127080973', '2054507800487539813892002186.3052805169061106480128', '2.97197622847365528561080931593129416743830681e+66', 45, 2);
    T('4.78141289395387220153389044256476394240', '49628662359585061000765516273374.313123035240835792366159343249893054894', '2.372951261158032142233712464738888292386180788816373516551e+32', 58, 6);
    T('4032707739765161478716458301555608.9419007494612663416808083477645187758892', '15485931177869814957824449269480732.493596425810209647036655780', '6.245023451846622631427678762686696796583833814502419409298589731621941382567157585e+67', 82, 0);
    T('379504212338804202439530554.828514', '6574004190944204788951862951366593260413.4970964014670961913726691174508492', '2.4948622823962782211447856948e+66', 29, 4);
    T('5562069.0229263218802532882534740601109025951', '839524.310811035352170669573494419992891', '4.669492163155629166164165930334803597855693411569868391908e+12', 59, 3);
    T('418980191455910.05', '809645559451315615357904496460.389419641979670', '3.392254515103396192156646340503e+44', 31, 4);
    T('492771420127264967416.5596501510936968', '478386721.15740870923722983922543508', '2.3573530395476e+29', 14, 1);
    T('161463318259202384267427085538144456.3897675858979942536023057', '42742185558.5953059276125275', '6.901295109941357e+45', 16, 3);
    T('429010977249155011473084655203752.067775486991510463962482905663', '9913824435260145567160268564.3394645365775708', '4.253139509247507339839034503722657022542148e+60', 43, 3);
    T('80525297266.72090070049604316587502296240065560', '1368932066337962.220702', '1.102336615798109039282605819982637460230197430956306378156366922312e+26', 98, 4);
    T('1228211380004671643.997721', '957450350319.0641480106802966017931', '1.17595141605e+30', 12, 3);
    T('284952226789853503327869736991643.01578057539324581651763', '1476956150244824318506516782.630258176645434585', '4.2086194388323212406988297283777034715817454728478124556332540307542016574846903469298102992e+59', 92, 5);
    T('1314174761780339191171677479577372313.4597045', '50265450.884106288764062215427053475', '6.60575869414017220219e+43', 21, 5);
    T('3149697947858612755909.060857569221829342630865239251241738', '4214806483657099758790039247821.273602437', '1.327536733219594277279138231863828021146370665977e+52', 49, 5);
    T('7770907679256740631842658248533.8', '533071682462391433106376.917555646321', '4.1424508308413083768029824730929721606141238910336339209527096141498e+54', 95, 0);
    T('94576875805116550746505930564912092404.3353823348884650', '260755.469476786169680047338199693', '2.466143765221086514499451751116819754354266286336853612424411815419265565475e+43', 76, 1);
    T('4906050543490.76545479705004708846488871122', '4088747909156382.09054', '2.01e+28', 3, 6);
    T('7812250796662037779438138434267117586842.24301785844958504762', '8992370.8972', '7.025065670553122343186642862404840339886e+46', 40, 0);
    T('4349904486779278004.4903293693247333469818857307', '3818968228942.023026113147164277070886574935666916', '1.66121470339424189358667e+31', 24, 1);
    T('32.85582849', '16342319165044722189292804.947069374330268402690121927773446830716', '5.369404356155493954311017137319349e+26', 35, 2);
    T('62879566895707287782838.934479158621496', '7521117025835349678580949.5786961834308252545052532', '4.72924581156456907555722813272366885111349253958950644526521218950097310489286802e+47', 81, 4);
    T('37013202117.260969054966198389822747731953526830', '637.283', '2.3587884484894422142256023808461410142862539439e+13', 47, 0);
    T('895789144102110094015920035326791788242.067625712250417', '8369107073881505851286925480513743343.7237802019011', '7.4969552626112291940986482621875494375645442652974257951e+75', 56, 5);
    T('295410917835584.57407891961041550', '8679467238965699560050893.250925030718797475', '2.5640093834e+39', 11, 5);
    T('564529814.73022664756447931648428120', '5536923.96064965282813389527', '3.1257586576809012e+15', 17, 1);
    T('7601655481763707998856.9550577156', '4683546541349.5388302622', '3.560270724014517694253245071205689938727638691288103032e+34', 57, 0);
    T('14688435056.05', '471399747198031261926626437550.3750885854785308101118405541', '6.924124572156670149226879428227655539005567059e+39', 47, 0);
    T('13.500624501537', '39277.09899601562835076275711918286634627212', '5.3026536505490289565207829322591198306689093251316024844e+5', 84, 2);
    T('28387109251175528634341519699.61465778702298919219875460075', '8641198851062835681750675.431715684743717627448333291', '2.45298655846253172e+53', 18, 5);
    T('349724921791.5843494449544028', '813734008722909.23511724137783423966381643176', '2.84583062559771849057706624756732141662244715217683217249e+26', 57, 2);
    T('7934905869500163637041049427154.52583475732103089970089186', '729801541253104558471953109921562557379.4550', '5.7909065332595251686810706525209500926e+69', 38, 1);
    T('5763809701726988330395416882620707.10131601686308036519606', '39623255768086.5989555965', '2.2838090601010738982403542308685509799e+47', 38, 0);
    T('7297315344532849317580343172.01', '5456.84335502', '3.98203e+31', 6, 1);
    T('116920062941661185068231.351468966314867841271820392', '89470927605415728056385.1844396891494826', '1.04609464870740181781974e+46', 25, 5);
    T('4001415348784158855.33822022801080293753305949185240', '5319363904003701.987100610034548062857642', '2.128498437124883809009346835085403505162194214713614715457709465350939e+34', 71, 2);
    T('60173226461760151949722172154027979.1600', '98022.2776419355598313438299163001597099168691', '5.89831671084571733756599264283420353403760149321437270585483851e+39', 63, 5);
    T('24698050664776898267792282950992.0566647583', '7.37', '1.82024633399405740233629125348811457619268671e+32', 80, 0);
    T('4934266758289102298457458474737653048.5775888471', '4498704218348.638512704592', '2.21978066799726463974016845087685495976656203775e+49', 48, 1);
    T('5627779668716055084569.9682807684534890827536905970370450042955', '640195965.8128287331875650037040317516330803034', '3.60288e+30', 6, 1);
    T('36400365558681611.174675716669296791256593', '32121886753770769004464499324.94367', '1.16924842027182856366032154657819637673936978811907922e+45', 55, 6);
    T('2459260977246148960176378658922264.52', '302950668133.0', '7.4503475617e+44', 12, 3);
    T('651615507299651049374669163105.019', '12433197434671359277721.16553138564172172834731', '8.10166425375009781251314853775759624605454503114750923151625569798519083614889e+51', 78, 3);
    T('507352875790126.3016540618941615831417806757', '8262097643984608923.44792764608237995262071998', '4.1917989997344184484082581884e+33', 29, 5);
    T('80682.7492119346333868818942', '296833063488827.097892991', '2.39494e+19', 6, 0);
    T('146906.63133270053', '83599577525643807637461710.39378928020592216682713334393', '1.2281332315129271636589352e+31', 26, 1);
    T('55357626370208684814152523972795856143.2178110247902886619215793763195137461', '1387.5149', '7.680953141729746628904035788486144505697124674228079489371725401617e+40', 67, 3);
    T('192796907159855025813284.9614277541778545', '68474871174288988254025919.81306', '1.3202e+49', 5, 4);
    T('6020.8884390769237', '13656666638572223.0729133907781191051325910279719734', '8.222526628e+19', 11, 1);
    T('2385464941719288999076782404309.09851760400997501140378570373227280222', '32.6649', '7.7920973774766403225943e+31', 23, 6);
    T('806539.0070268512384415947803', '60643267808335257319.074254045', '4.89111610009981316050147299192816085145953199243881613135e+25', 83, 1);
    T('747963050097.1549279624836063339140', '6733757.99249962587413422535520769033045750101', '5.0366021666861150653544506821981562269063839967444692806778641808578280522532e+18', 77, 0);
    T('187489.061557486372251703174059246924', '652902580321306265632182802.617572224724127386201421', '1.22412092072903080973895801189616032026677926560403441153048494684574765226848238679004e+32', 91, 1);
    T('439298571226.156567839282', '92490325218188479314849717552191055426.1259', '4.0630867720593e+49', 14, 6);
    T('23.9099655678332025969319094599702191164', '23595.0258935', '5.641562566857178459636398473643949217903266905034e+5', 99, 4);
    T('21907570291194363652234756150757.77651608258469605', '2.46405', '5.39813485760174717572890508933e+31', 30, 0);
    T('5370322300521698463137.539355913189996281533374098946397316524', '542509975656005458953445997449044638810.8197', '2.91345342052092986567594763923017727827679734895822603267e+60', 57, 6);
    T('2.731137645879496065905', '42580948082240156984168965885674.0455', '1.16294430304646424991008804238e+32', 30, 3);
    T('1781129206.13453192694413166677961219180728', '6258680.8150389258375447072854', '1.1147519191539707226294344684387e+16', 32, 1);
    T('26096747519186878172730006186469325.19551116665629347063970472190657361576', '49346671209541192352968558243.195506732497394992446', '1.2877876194677246564115814568683752906439392190438789159173909429e+63', 65, 3);
    T('88697604.60839004041072412570', '256693522.914867661579452852365137843', '2.2768100601037640338527799630845179665792732780326e+16', 50, 5);
    T('78730243639905251290490767600321.12960999334435251', '3648676753607987298031.74660408897115037458003987312270538', '2.87261209774815381610244010287198106786100892812488468570038002424965191133e+53', 75, 2);
    T('39703364368472863.43712986184544866', '915792066315326056023552825474598149.7', '3.6360026094674054169201622472119784469807515648527604045931e+52', 59, 3);
    T('605547274620904040166853643249945391156.7858128126835967989259002431152515700', '430933791.5966945', '2.609507830434309941326015388232011791863761258939888474354344353699901676e+47', 73, 6);
    T('1612.3826303905659756793236817353228', '56006.503375912847468172561', '9.03039132322324702869216520598954289130681770379865994376908e+7', 77, 2);
    T('69571249896183530543670.4403972295', '80090085623273770098.209782927238627281460663775325614472', '5.5719673611035153501e+42', 20, 2);
    T('3053939177.12104436528557890593191149', '766894526984871834.19079714514460027', '2.3420492e+27', 8, 6);
    T('2786147190183.0280', '91615121227757548802.0214851800429592828977164448565', '2.55253212586994177004114035896139723474e+32', 39, 3);
    T('699623882627740900445877.167016070317934720754', '3024246205148583944777828660.869780', '2.1158348720682637222383371647797134650949836e+51', 44, 6);
    T('47470161850904.96921554563033522', '26721202870610537814.13472304551247537918395232342009147', '1.2e+33', 2, 1);
    T('43617689731718146.0668534731836470794724', '6435131794317119377720906.66251745551470977018721704598328754', '2.8068558198723878662689087357494411e+41', 35, 5);
    T('7343922.546894716332739143349208853406597401', '30201514845780164685740927045949029803.48865', '2.21797585826300453001e+44', 21, 3);
    T('58441002826.8688003079952356535501367080664', '741684310349849944237923394437529566.07476906600896614318631179520', '4.33447748777998171971516070889919341102e+46', 39, 6);
    T('352.32855040125071080401948228875', '18417.1221453', '6.488877948016321666766073175512778754055375e+6', 69, 5);
    T('910342197148886638096587723917.6', '32510797385924290043759808.1921268619488555910', '2.959595072336459839873112991932432343752998566807694401136397e+55', 62, 1);
    T('4.9254442252021761195076', '0.86003123260410811031013', '4.236035868123413780465929462368021246578891988e+0', 77, 0);
    T('97.0881496153446', '2159119659451024584753274.9', '2.0962493253421295628421602442433014203054e+26', 72, 3);
    T('39990630233581613.95617427502112960533537151515318', '92490575234360875271164082151037709898.60503038258611703948515690824', '3.69875639428858688842224746952432622182062592059067011315491713619835455439832383094e+54', 84, 4);
    T('53911583546756.23883820202963956319095750318', '612755125.9669245329044', '3.3034599167268995212916151963439311656518413455828844111977635992e+22', 89, 5);
    T('1713137239012136332247016.105515834', '388942631.93080608457481478650729236771445575971', '6.66312106600054711779039850172123513401043856662888482341408048885998702e+32', 72, 2);
    T('5577022702609405883574998711468370418019.14250526110795264690116119611', '166323.259681037554435789149887', '9.275885952131460930279535394259981479689916286e+44', 47, 2);
    T('5712963415848065416244038962991524133.07656771159184342382034210404815316235', '1426559117132727090336759856649346685070.0', '8.1498800467237850177412040253504508506304294237331878256768050298182597e+75', 71, 2);
    T('1462929947246125108842921930112131.34456630094187048', '32741555491400.20963768808626755590391779365095908', '4.78986020477901865500593660639101251915579739786029514616021280934719437140427238278335737811399584e+46', 99, 5);
    T('9227761222528504798788931158.58158809876062519', '613431449057386797714952724357181.89555626059', '5.660598938291223809e+60', 19, 1);
    T('13281154547200654493984948.0', '7228089848482014379812588912.9036646888500251517585836973785503', '9.59973783587417950391922483035349608179876075659127221965360265404830860350871062608844e+52', 95, 3);
    T('670026477286726749659949965781.874659914239780271755065863998942289801', '8347171972608869588.1964997745373103096208003491665536', '5.592826232113618877690515610951726176092398387257691045528488588683687216915403041e+48', 82, 1);
    T('57245995367463762407800.79796561849', '48889152617699.4148512541470723218', '2.798708204272049575906830862832645113345188570811250782687897310082e+36', 97, 1);
    T('23416562333440.74958586791197121601265987260', '572769419292571038013850021.78210289409091887298234', '1.341229080955315035983283666671211e+40', 35, 0);
    T('4390461115898217169905380635400526923687.6591298460932127974533873498831223265366', '434674294658036431.941863484499153028165526791721203123995', '1.9084205887765930915e+57', 20, 4);
    T('6834871578002333928588518374.32715232204697102018266837330366', '732276394.863', '5.00501511849113298462666792532692063572161509120840160811829281796309858e+36', 76, 4);
    T('455.551950', '22311351072.3780625626632047769256152270503101', '1.016397948815641753764322012937777902163246151416e+13', 49, 4);
    T('80646.2163421740236385295324', '98592986515120950921.0959811287424712453384231', '7.951151320319490372526325136517142897470009905056413e+24', 52, 0);
    T('33638287516238048565576353554.35628513954', '90917601438419685511132362.527714032909', '3.058312417472299354505828254185847e+54', 34, 4);
    T('89755.432343516271121535059036801015237', '9607972956783187394500337696585296459314.95901211722352775332279', '8.623677666808873580810424838824924117035118826131039280234315385688421662287607234036795343722e+44', 94, 4);
    T('9327.1606485645814446344281702281669532611346', '8658073393426143526.192954079748353417832198573244192213', '8.07552414e+22', 9, 1);
    T('95512285.913654068596358510279366848279643521', '5606721166818916741104603224.1349013767038951707728950', '5.355107551e+35', 10, 1);
    T('60474930356436731917.34398962367104920540993266833447009698', '959529901682549.7449309063919779829026011550455108716', '5.80275039791707803789128378921561880318083549897512706227841538572552901423030694691828013e+34', 90, 2);
    T('21.14341735001283257117402353671916', '8944572265238559067499670596.723', '1.8911882442128853353698722715524623718219917775989302957886731268e+29', 75, 2);
    T('40392.3646584806230', '10165425655536785707515.769023004342444265476586888', '4.10605579987116282406020530219595142e+26', 36, 5);
    T('28289884142051029295035781545447033350.9164', '2229314137559127284335106518510471601.89929279734351437925635', '6.3067038667784121877324198503153890933145159582169333942256053403933740403e+73', 74, 6);
    T('336584855246733.2145505626717678546770935', '545604345415005640919657849639610517.49187721358', '1.836421596234983024901249957226341629226316160611476571215e+50', 58, 4);
    T('515344901822.499588904470501387426', '88016480694794318987650740.9314710201950197397671242678476972860', '4.535884460242070872233988284436621102717676111909621073788268038330757805220101056e+37', 82, 2);
    T('192673937068052039355.38401830924394', '15066060748672794202053.50875476146', '2.90283724055323094269971041467549100894569275e+42', 45, 4);
    T('1606781427981150050348559011109712243.304732426894141198601815075688', '3985887113260833842815719446982271.527448317906196', '6.40444938761690656702887504956730221080995277632604057074679781476802916492863811e+69', 81, 0);
    T('9340286858587171509737.43136746204073804216726947879243914', '298578521.7660749727774123', '2.78880904310805381987613e+30', 24, 4);
    T('547680956214749790689.3723254', '1117.702750152334362990586', '6.121445109672860612024847759207180595300879691286844e+23', 58, 2);
    T('997026429064625507985170211.33038656918147429266625135239181', '815.57363458204923016897', '8.1314846852659831189174374024344829386e+29', 38, 6);
    T('4803551390807364269965379824.619991315895109845', '894615.1484603023613649072', '4.2973298406238220739913041844063543314956772257055200451154441e+33', 62, 0);
    T('980966379931994.396250183', '7119820044060763.40580749872198370334204458587025208', '6.98430409438953991754505210812919779927678682856982080125678229412095613064e+30', 96, 4);
    T('5978631964139477368699658921790841881819.07', '2945593120014.68519593211', '1.76106171806691286394367825388252115639327330166375416241310233377e+52', 91, 5);
    T('5826181419437155537695.1162', '82740995886425168.1833', '4.8206405285921643374965203815497332042403639946e+38', 77, 0);
    T('411829547854767341088811031635901.14322757517043', '12452152043029848154.1587012940669761232571853080', '5.1281641456997997654601934794364826597942584939815990969747363289592392074e+51', 75, 3);
    T('6806.45305160414288315802405439406088524275', '409638321832394170081723426957.10439810000449426494700', '2.7882e+33', 5, 2);
    T('492521266611875820788306.4716118932564872409646602822279403694056', '13.8437778026466206867', '6.8184e+24', 5, 0);
    T('43819733665495979.892857326226869842', '4214715453976537351.889', '1.9e+35', 2, 0);
    T('515788458908475242032417847.137404', '33756869076658015.5406118538690737867604982648796', '1.7411403478624601433705756724712476791e+43', 38, 5);
    T('1914100041112425016478272784380.8334739628391176050', '137.40742808079469403068343198', '2.6301156373860170752293404e+32', 26, 5);
    T('92.913927316856042418844756', '2485191714128.52490686921976752282', '2.3090892229499064300539788243295562273714196731691226733192e+14', 83, 6);
    T('6720.12646443752537269481516890193', '7620593958651225878665010.7', '5.1211355136264827985173125895245379341318357581986630700651e+28', 88, 3);
    T('45446206130.913742242502542438185255823', '189.65094171737555486519699398', '8.61891579e+12', 10, 3);
    T('288.97197474837726464664139331467782546', '306496124844473826393864262592757.145326039410065070689614', '8.8568790449032776140271086380714511143359878e+34', 44, 5);
    T('59901185046093293960.92', '29349543966367480481862989223605.56924', '1.7580724641478293839886862738203348628468270241581709141008e+51', 90, 4);
    T('169638828516492322600472.561314315418023976921753', '61267213636904.74165367362364864010', '1.03932983478342e+37', 15, 6);
    T('8296118725708809953565364388985141056820.82312387619', '851398.5960611349595837193015605848325745', '7.063e+45', 4, 1);
    T('6468885214.03117852709159360', '62341463029516548538077825.32437004784725543541488', '4.03279768412710961400280322755773753328737657032876353467801967730976352768e+35', 79, 1);
    T('3777474524931716116484737106479058.4877693144877', '163158.2028690338397389799292483', '6.1632595487141e+38', 14, 3);
    T('4063364803626300603098596469554.3', '4861065393.6028342049381131085524591', '1.97522820284915860571051888896685695922790382073e+40', 48, 2);
    T('1122445575982233283206307954.799775287937782252545847', '453651521421427299653176844475990.10097310373070507507034698696', '5.0919914325709040599721266444e+59', 29, 0);
    T('647.6182210506639', '556.00216624985530012875610922236', '3.600771338071e+5', 13, 2);
    T('93813287685310941883.0811058823022080201307040069', '9771940069115279444745710209181176600.4092', '9.1673782494752899942554279047928556864975948182818414949174945040452061247151400261962348e+56', 93, 1);
    T('83793905127.356389298343074946', '96467078274227215279886461170922574.5', '8.1e+45', 2, 5);
    T('4314296.2708378770810625532494750019000007844681', '2041918.4887721884644655411804276751970', '8.809441321464766275120813e+12', 25, 1);
    T('67363146876603991604037961440.849184782192869', '497895367267814852819626086541208.09441362681352536935251788060878004', '3.3539798755e+61', 11, 0);
    T('64.5336', '838919862898093212618598939304960090088.25025339861342593217687464114195506', '5.41385188643e+40', 12, 4);
    T('7655582303566332144527.435', '403718412982774644977437721187281.126039137403682590', '3.09069953805481373053612747351647841833127480313269556765537712668428685665e+54', 88, 2);
    T('56353965732035267599837.0314178943559830284841887357', '43134176887755059228392.64725800232838321925221168491558798760', '2.4307819262e+45', 11, 6);
    T('4944812058.21', '123.665218818790722289', '6.1150126519633457650512261244269e+11', 78, 3);
    T('5278323637165982847497.6114835811586053578509483524', '566175031368.5680121043671794109', '2.98845505084590434102119e+33', 24, 4);
    T('6358.914115171291373226192414214558621', '737008884337309149049685256.8955517', '4.69e+30', 3, 2);
    T('5.7348559241055935077929', '1713306185.0608216752864703970632025519861728920896', '9.82556412520280749482806279677837850288205001223523841344430990836504384e+9', 86, 2);
    T('868.92358', '11029450266602067.620801059590875', '9.58374941108782e+18', 15, 4);
    T('818749316454298500634449036545.055640160216223772776', '9700554942046966432.570045672148', '7.942322728028320972035716532960934048271511137448358634319301747144798471343842848e+48', 83, 1);
    T('234003730621412031.076', '5711933447348092465.131634586175091537903560402', '1.336613735740676410113e+36', 22, 1);
    T('1.5522342400492688645206596928430412', '289198487223437697025144320621183397.3963745373610482', '4.48903794038671004915766440445371225092569431536846884098249567963407114235457307786e+35', 84, 5);
    T('5804.66687851780703756399555204223873644', '776913931997845.735208652945532688268623089155724588', '4.509726568526931008089226164428644e+18', 34, 4);
    T('18334979652.01287425926229985410711438', '60754991896046563615472.496419242', '1.113941540172220818651360404259102430692452e+33', 43, 5);
    T('536605450860779240749884625284.9476642300907674718145171589737695', '1831852553857562008631630.318805', '9.82982065573206948e+53', 19, 6);
    T('901619743241267552820204387.5485152215', '5873.25061349791776960684977230606', '5.29543871013360975308646780247954391272323519464518868945890306669229e+30', 96, 6);
    T('2395620049990724070729447.9977745172368390377995', '54294638110421739096889285830.53091151077', '1.300693236643167989134943765029988810208042466647841e+53', 52, 4);
    T('9231037734506060513910513217529935262.53956890140', '205385378695439664055.9071', '1.8959201808534206629126814948407803940360421412e+57', 48, 4);
    T('5927068428549336611308825226342798.69668990613366047522867754', '453775495207468214888751.76074331', '2.6895584112935256591128947445045510841814649346656244302082989367e+57', 65, 5);
    T('71584306109534.316807786462767487938499023271', '1468645679124651575323339.21776570636949907849', '1.05131981860903e+38', 15, 3);
    T('148.56474376249939228933', '8350.5824931798108336007379104506', '1.240602148366871925594033332323740899151434445665872098e+6', 58, 5);
    T('77753.40996997718', '5520804276407064156549.7', '4.29261358267481673214713475576456535846e+26', 44, 6);
    T('991193522919398358131.730202265881702449235182', '659749290331715065994.53858155045595612946', '6.539392233274656188731421438712898058599502826615096945312164831314991e+41', 70, 6);
    T('65932205551135431202517489267893265.59422045185717122649631106942508', '141.9303428343163336', '9.3577805376952662404757e+36', 23, 4);
    T('73450865787.0217894024679330098594485811056720', '439454014196047177080275509853970454.5497320065980080669392032016193413128922', '3.2278277816281829350025005417576920335989826e+46', 45, 2);
    T('464104813779200488851481851964.177091213458139978272090704', '7618699.6374318494618', '3.535875176469970776612344287616373079e+36', 37, 1);
    T('435861380640979.0566029758568309770025259146686027', '350198047144500914312978936905.95111669', '1.52637805e+44', 9, 2);
    T('73060886734836106284.0249690950429857241082', '69.047266852764748074757719165007937', '5.0446545428798487675482638408143128529100660279985307242792923418513452467834e+21', 95, 5);
    T('53.946804222400', '551995910.42237', '2.9778415311121042083415088e+10', 49, 1);
    T('5855745724.52783857117944769', '3.833443362506181289', '2.244766958021519227379315850240026486203227241e+10', 84, 4);
    T('9491985349468.947840641813958080385797', '1949273921752152135105.961121434727715017003684926', '1.85024795073733082720699234974339989643883095687298491881169918030125717068727133e+34', 81, 1);
    T('5205758503998236956.48577209051633050', '54142160966951.904866121458972404492792610909634', '2.818510148785512867300716985e+32', 29, 3);
    T('5100601580996910177585408041058.667769523050051562816637', '582332524793728208700240087.04456733685862887755878559183', '2.97025e+57', 6, 5);
    T('8842052339605237212408488167259285.502126897996556159827262346306741', '8517284.53411616687834097795742788', '7.5310275641965356193067570262963435832504015e+40', 44, 3);
    T('193241.833759868921710', '6743812129806216121138569350.000829214221711584749899206644886734', '1.3031866224958003893336351834166e+33', 32, 6);
    T('72514.716827', '167700528091668164414.2782300569', '1.216075630630567564055226416817e+25', 31, 0);
    T('43431256589417720459520445195123.055024009114', '74497101454.335767300993377658510660601458986670239', '3.23550272843114073966208574243614846966184454769904459859743360311034e+42', 69, 1);
    T('8058.2346', '8989227556549232243789430423229942.219669739175638393204179102118628300', '7.243730452345847987033962335076416415054349279810477720632090528926387159918e+37', 76, 5);
    T('970749659.90652359233467168311201873', '910752668978011815023729747334553.1617486413', '8.841128436693636291425941e+41', 25, 5);
    T('277364074328233799332028594204023248.44554784401', '3715827291912124111775269176821427301.575', '1.03063699718479410332157941473584e+72', 34, 5);
    T('7.11053909700921', '83455915715926509497633.94583540667887749205', '5.93416551574801e+23', 15, 0);
    T('6347967612669599230868569227042472.649720', '36405468224300837510220.9', '2.31100733211933941381758969646390109521396e+56', 42, 2);
    T('557191261648542522.217921', '7026121841676086062021457121454.0', '3.914893693459879527053292962172722983878415424352377134e+48', 85, 2);
    T('84503487453356698.39649405', '21884070145675.344437534837063015568172230118', '1.849280246983454363618404153e+30', 28, 5);
    T('6539916101913.2610990085744606092562313463189110494', '600149670543916943239550332370744850.805400570065912049895', '3.9249284939481011923896097937287450285442552557512625158223207581882e+48', 68, 4);
    T('65324284447044677932538530130659.693941602998515588577830', '2760870712810308457708665324318621491.97569570894380312793439129161769019', '1.80351903765135586e+68', 18, 1);
    T('2693269521712854975.219743708812911582350998240943540', '141925952718.8770822775563674530010081', '3.82244842797811348552005466562936384937070618733587648592151153631127985707930315182674e+29', 93, 6);
    T('73086535964934305104036507.70740504379120493663395663831014', '379186275962887900996612796.522698930530504184254', '2.771341139557111097850213449733469285221722152e+52', 46, 3);
    T('3563139709158443679164540917151284089.10808534205', '601897570783167717561605862657428421.9365983279184681030472142328096', '2.1446451353035e+72', 14, 6);
    T('2524636773769622687916259093171794423133.26167763356403989', '394.964278934035117475902', '9.971413429222677688372e+41', 22, 5);
    T('25638795.0942024699', '7708881970481.50024249441643659648151546552756', '1.97646445246566957796700846762950652582855299084237944458520444e+20', 93, 2);
    T('33.9507444068301248275772764779', '2804801512.98', '9.5225099279074406762231006693082e+10', 32, 4);
    T('487.26051649985836598867115655649118093903', '8463604133064429552219123252536.110311064123237402008199321362315013', '4.12398e+33', 7, 3);
    T('76041121007964223430.67', '98007195540669002047897663539.49', '7.4525770157592232087726864338204801538655544228221583e+48', 86, 0);
    T('4249620136202272.7804229912953934236401', '39249059982445102448754013804.1', '1.667935957e+44', 10, 0);
    T('263536558619455214106078007091.68241796600', '287562762349774733080362201.0907909008838892464495935224067075634019', '7.5783300776763877751682326744702715944197e+55', 41, 3);
    T('6876020.04715', '414.8078', '2.85222674851418777e+9', 99, 4);
    T('27187119.0', '301269682412.4978896181701372160620', '8.190654706840787212298056082739406305378e+18', 50, 3);
    T('9076308346510478616118624431569.38391304223422135792262062038232902291', '317943777068955521309374244300965.880301214540341428366504139478042', '2.885755e+63', 7, 3);
    T('21666016088024267.1', '49448675044211274880241541.4889467185418735978596775392479289', '1.07135578903936556876846889065408538761450769180374343262449954889282661540919e+42', 83, 4);
    T('4137788854463958291511794314108484351.5277777807762221666031', '9069813776549.72651760', '3.752897435667112026537166094288402685758940321393034729898613234083624918436456e+49', 100, 2);
    T('397154518.34288385070052533240', '130514974104974.185617403665212569064145956285', '5.18346116771949809865153836021056638178461009215213432513002e+22', 61, 5);
    T('1703460845615360578873548380535059.3892875430918920323567701619626113', '9743545196.20228843900043959', '1.65977477392142346674490415249115483587136165274114682414454793599995556940124e+43', 78, 0);
    T('6348633860230597.1989307', '2597249807652573585570457.860054274845371584', '1.648898807234e+40', 13, 3);
    T('5519.14378768124123515251981867950', '7473997905851476100153347909405967103.187099928', '4.1250069111222780827849945971900262464322676017201378222152569427582007505076e+40', 95, 0);
    T('7788864.0', '17864453.110817206742710195984', '1.39143795714532152178852707932722176e+14', 56, 5);
    T('3007535540672997732261024.6', '1038680839181085.024273453997580', '3.123869539253167555576023785487030527404642109e+39', 46, 5);
    T('31815153215682111453562188824584544435.486286880320', '88233340366985944172.96613971726475423', '2.8071572425070871136647263296323576179441325140397843139955336975909e+57', 68, 4);
    T('724613635879.4820256767868887311709971352839000310137', '97.68111292329008430719490245783012193', '7.0781066392099487e+13', 17, 6);
    T('1.38430151654467176915315', '2635210021496465065.7693085108707497742519747949654662017792', '3.65e+18', 3, 6);
    T('579207941917657696557.2145744901822', '4496672469360790223689555296.177829783931240364335723963927460', '2.60450840645625499200202153235167747e+48', 36, 3);
    T('30879218243093.82637659669', '15475.8296997', '4.77881522789989492652650292110713993e+17', 65, 5);
    T('740688861390753029.105641430', '5845433842749856094559752203.821486513511103671774919751520089850', '4.329647737321364999084507104672600218275579163194e+45', 49, 4);
    T('6358137861689736013800398508.053472367136392391094064376489467', '441306942652541378712921051227134134455.52670742921966354192289473373280', '2.81e+66', 3, 4);
    T('30128.36200562800822875296251711175906962', '621083506649.547443475392987776934099651069797385607', '1.87122287240624355e+16', 18, 1);
    T('4706763988851382.26008156497712750428', '66.0', '3.1064642326419122916538328849041528248e+17', 85, 0);
    T('765156703543106398840973145.9755125443887980275', '962.03124714039', '7.361046577674043167410774576496195845060004253337086475807e+29', 58, 5);
    T('55794819209826864129567.039955432028247', '21124087013111742477.7835907553554', '1.1786146158692212338838961991039202913086221228258940231844085993239838e+42', 84, 2);
    T('574869227057.2003989564223', '685372843760319.3648934853', '3.9399975693849e+26', 15, 5);
    T('502219685791524.41505112964176257113343396907678892410', '473220261098545872930431492290693453.87321641440914305695331973573', '2.376605308390948526175406928189951444435384145784165569673783945497689115351870063e+50', 82, 6);
    T('580756414598870898.46673424096189611186032061703', '777249424619658487.0', '4.5139258909114823806951272480093811847681573736844111960371623361e+35', 70, 2);
    T('9026961464036131027419006244161948535142.615469460408127637607422045835', '925061172751.945126393925951770573674307577181', '8.3504915583078788975975374515815884512957211e+51', 44, 0);
    T('395762411588903839447311315252880428.4917121455660018867757284569273323285156', '2276997261430350302390.2405635483556109018030464695099', '9.0114992736500517391588723233e+56', 29, 4);
    T('5149547713290916450865.5151454771185', '177072374.16120940239372567650240', '9.118426394488494380480163717154338499325304387241886330323103344e+29', 95, 3);
    T('86307867633444.738', '7839038668876460554214033431333.05901023529833652972853', '6.7657071180684439272216939705649049259255032301157e+44', 50, 6);
    T('312277631548433774.04229750477722767883099998787294539', '650616076758235480000174988166557.586', '2.0317284749739576586371473138212572119893368643477292269550835578478977091405237886822854e+50', 95, 6);
    T('65555342119217759504909653348.4132463546954232017970', '91829542327251696687755457.0659507513393384028651967', '6.019917063914173189062290491982741962410184288809712859539036659778440621476097787928027167692198469e+54', 100, 1);
    T('12457664231731087171612115658287147643.48967241', '2.66398157701575069175346219856166185720', '3.3186988005979691872637841333787396217745955200915181739090144190671570074820199852e+37', 98, 4);
    T('5725327553056026193928.764824193657514142690613', '79.666465729', '4.5611661129283734e+23', 17, 0);
    T('33645613465336193993786033252109424.057196528719493743143147', '2379535760.0759503948162928280859602638289573', '8.00609404104603916570397461565383e+43', 34, 3);
    T('63286.0450210123906803003011170899091', '8043070157935.654633', '5.09014100122277078636105105403409e+17', 33, 6);
    T('38015480463712256850941480205845.6357958', '619198939051115151547363564.2436316907334771632137776', '2.3539145170649024488446936607087466895259524352541842702222e+58', 60, 0);
    T('117255803285408497547740.2625756897524', '25978344720061757442221682106.8354250615789054764749454643433497257', '3.046111678176091913809026736459184702602605576586740391e+51', 55, 2);
    T('61443249986904.33114178458686', '7273239138.2322875765050595123641570560831085636277', '4.46891450584943072354192421016593881584436557641903921853563599087181467352022e+23', 80, 3);
    T('225278386113392669.4777933080822404061022137437502403677', '8120.314068224859860499147972091537474', '1.829331248023574403643529882584832074e+21', 37, 5);
    T('54900471981079.08611038104996414938442431', '86483374929085733787.49936175588594459894628913293820511', '4.7479781021234288234832011215923940450237323924923877099603753089225058616061248988692e+33', 86, 6);
    T('839697434896666889449219.8547', '304661013650963609911549.11344714240689113', '2.558230716757325583169606057631792717838765988821e+47', 49, 5);
    T('937.010888156897451', '773092170022626249030296138185525400571.15509943232967182171842', '7.24395780860044192560595594142734550192186e+41', 42, 6);
    T('2947.33409867865108113189765294415800', '589013631824381982220910.0', '1.73601996166255370201556016157007374470579861787304994378e+27', 99, 4);
    T('151.9494710829485905105534849', '6088465572341480.1295144097658734349644', '9.2513912342402977463448214171055570350260063188695550940792743756e+17', 77, 1);
    T('62601066163.91410753083975258491808480567', '5659296483.93553840603647559776573', '3.54277993632055111918537376029832275481e+20', 39, 5);
    T('132816776404587304278528331957555.78778945521558', '90177043369.51281066606970278174130755835993738009', '1.1977024206035355088044915846171266702628915863897688e+43', 53, 5);
    T('461301962.73418868', '791.275931939108319190530178184319', '4e+11', 1, 4);
    T('60672.64111433882380985126', '274747212339974407973998103517.078500405247725754456698788006593128568', '1.6669639011468310308182050022257121649777277286624108493901606388790081611e+34', 74, 3);
    T('16374506164110017143.018603029256838571216751851292353857907', '72920643.0137', '1.19403952e+27', 9, 4);
    T('315043033851189364.99640532', '16926010894976106714155458905.4330227', '5.332421823351557587376150178424748515481738397025403521960764e+45', 95, 6);
    T('71891771829326844313230058.6', '1097544259107855351407080421.2363669565122968660512953233442631216584', '7.89044014483695181245e+52', 21, 6);
    T('2038941063152746810603106073435027.0', '6102338.0', '1.2442307529437406666722137109953355793126e+40', 64, 3);
    T('158027728676501173627053.12849574583011624076987532912', '4008493398487221656524813458657.7606362990766601704591675201383295124', '6.33454e+53', 6, 2);
    T('5727143007410642523917105138999362180646.0913935', '23521663783454169223932.36187502876152679466595372574039145', '1.347119322600737029586060277921699157576e+62', 40, 3);
    T('900666453258795418186893787214154538.8009855301028', '53801559110832801996075.959029507748308676946026556', '4.8457259424147211e+58', 17, 2);
    T('906692685.102407601811273284778268063146636456', '353287383477170410266001373350.1791564586806808158090328841521067540843', '3.203230863377195891787090254643320225451001358672023823310187912651351215e+38', 73, 3);
    T('4994078.440347801610731606946108051', '831099557554974.23177117048909322897', '4.150576382167893690439911463677823259859694008952571999e+21', 55, 2);
    T('916547055748485280080542514.9601186006813527464', '79299075226892596104605.966012141225754435155019912381791', '7.26813339227860562923097853619266077100067191774803841137267875064e+49', 66, 3);
    T('972052828087579001.0', '6185863176070691597.15460962979764544667615930112', '6.0129857944623294122129333193387294781055414692042963694689478e+36', 62, 4);
    T('147295555892014062015631168435439.3998', '44981673544.15551985549981050', '6.6256e+42', 7, 3);
    T('17650570751.821975033172', '954851257532278143360.846591301961054', '1.68536696786e+31', 12, 0);
    T('6242642190057765271942664226078717565517.070810391206', '124768344650685983011.870680993032778571647197627961924335555', '7.78884132300040407350110115184981228558240890826072700894638124500212e+59', 69, 0);
    T('8562082312545202319536159854.8811076554652333623', '61946530931170833095997529774121602157.78179648638433182', '5.3039129680931207185243745829824432857814409210566709674349794445292203812123e+65', 77, 6);
    T('1752.211931765837352700610264149080834862931', '58474111849617284815540327744369.47083103723634808694430', '1.0245903648230954325804518305074098153724823062251521022240576661655327965450200455985e+35', 86, 5);
    T('385111183684727468195616086.920038389514947015087684924486100000388', '2318904946270097961366399380.09547', '8.9303622872e+53', 11, 2);
    T('377310086.1280', '62095563115777901248372.7820096982346353936', '2.342928226738081995574e+31', 22, 5);
    T('48717102107497827230759112863184.8133168', '973579002234283402034148570406024194596.5379933266163240647624074114878755698', '4e+70', 1, 3);
    T('2.490449544382293623555992540862439242', '4525925219756556795533985982351687894.7778749591203171230853530029216820687712', '1.1271588401451049014651902564306e+37', 33, 6);
    T('14348695041635833589754762097915846763.003497', '505010117478647282433982307469044934821.9162460727435409434285361', '8e+75', 1, 0);
    T('6009447743528002.4204325513446367325', '1541550813392.17663118', '9.26386905707337259079283197422912215979053542e+27', 45, 0);
    T('53206482582893.7725', '905776431544219284512975091.522450642491159773899379152177028', '4.81931779289531767898904672906e+40', 30, 5);
    T('398536116589308.1175489300149', '4699426773160321348727457199044345450224.8173440403072653363623109517314678', '1.87289129637113786088387385640920461370753862418328895441172610862610266111039203084139653e+54', 90, 2);
    T('3028273643379837.5748582747134382024536517639652294432993', '998299.874482990475935062', '3.02312519808624011339911317e+21', 27, 1);
    T('61.744841314464344427775563672', '32897170958364682376096394780.536243', '2.0312e+30', 5, 4);
    T('447521318098078819656430787961188901.8', '675626538014478333682364163404.432556753080104024638529693', '3.023572788342811003489961043477073831895132711003897332651560051047804489587709268885813611474e+65', 98, 0);
    T('771886340859540679377392106311185.15223433', '3506827405387863839.6340899633952862', '2.70687217397e+51', 12, 3);
    T('914795728156614704509.50', '847.488798939059958794796810625760394177610', '7.75279132930032190462288083532820963587520195172954634410932295e+23', 98, 6);
    T('886474.973442699527804762882385509991', '833709987698904689357168193825064955749.9294', '7e+44', 1, 6);
    T('5933651918986625615737928722017561764106.93708086631652368693769', '5960624852864650544731959555869714242.26478273934468180552345023467719424', '3.536827309655970666478892325014154138173804272564685863377834676553462549874634790556899e+76', 88, 1);
    T('4.10556897539944349605758951628', '3955440.753810352715', '1.62393348428743722236616916541588284197130347002e+7', 96, 0);
    T('45267609784.11292970515704102548917732', '76.2521031499026253756', '3.5e+12', 2, 4);
    T('699792399247182881183639.498393780636733276', '63376.733019026924850138713017076122', '4.435055e+28', 7, 1);
    T('37031.28802102', '8987309896167.1552020193466225135547741', '3.32811661299129274444110208696573396748706962151582e+17', 79, 4);
    T('229221972654159571433581728995804999.6231', '1555771328153746268773629.2580', '3.566169728381835441896422237224660112605220221099747385619091326598e+59', 90, 1);
    T('4121263366268907.6277674322140557079', '8355024426597786306636096140722.96713894', '3.4433256093619342520731147011379612688336014203154669907663750794239e+46', 68, 4);
    T('79256108760611110855371309244466.90279812600490927495050640033359', '1115975646206537546.629322732908706711383002976048514', '8.8448e+49', 5, 5);
    T('70466229167622333.8809076776907936', '7947427304935162083.96', '5.60025233762580274762526599893353264394140943482230656e+35', 60, 1);
    T('929302014075869871398036164149.1705631', '21994951530776961664741.0878', '2.043995275705216760389077243032406984559747679313574068283254018e+52', 70, 0);
    T('4001868455945243060204485737.20470313414555831726563484674', '6136716637499842528659102412.0307082655968', '2.455833273468497869694642831826349382611709317978009403562592069974929451475520073089106e+55', 88, 0);
    T('217570627356959147292656140846510.1224182492461', '4892.24839169436', '1.064409551767016311806807164686737242324075671872023571621996e+36', 79, 6);
    T('3865781784785790.074', '9124.09453983147328514909752', '3.5271758474743994e+19', 17, 1);
    T('2564367354937787060933574643555390978.1988547907168963317', '39152182925513957.14197', '1.00400579768740615921074429323616206212153640018201523053543774748529309111449e+53', 93, 0);
    T('876812263858950663270931426360907514.738', '8490027674702487570135021.778396', '7.44416038568103088047372499493714109747835564199365145e+60', 54, 3);
    T('4783407298869470743137316300590026.1067714396946055226110236590479706956980', '5831200939916493.557357787716336894', '2.789300913717107340744743431e+49', 28, 4);
    T('474.0815393777963023', '256013348625593934.17114527653624660616975710052793', '1.213712024176860035511114175640526811679903e+20', 43, 2);
    T('47889767236680412708361824691645808787.795511546', '544013895873619456794.359526772761368', '2.60526988e+58', 9, 3);
    T('18450016544779.399954010661282', '737083142906063662853.48', '1.3599196181494873385246963944366161664959553875e+34', 48, 6);
    T('65971863219584846364132465651911.79866963070717', '87612779206.2', '5.779978286079113812128086560170153761759062044576777248454e+42', 91, 3);
    T('873239434403.26172587353092770014', '821268153192.528014999950224996729801991424630', '7.171637375872544697090862e+23', 25, 3);
    T('525589630.31014437', '59595394354.61902572305635615501882046974', '3.13227212870314785577565470235556725479210072089306163638e+19', 79, 2);
    T('98571356269681034493797451275.39270212099289478', '4358861480813686630098.66658399486', '4.29658887955475347848896840421527678851036721313610957484735196527e+50', 66, 4);
    T('7881501248183507830285.36727699041997912805441', '513.1430313808175834139345', '4.0443374423245827112520188302701446547187681888041639e+24', 53, 2);
    T('35845319286025124823121860.851303446533594244869454', '4414821187024227440207.4509206', '1.58250675039591874413157179804432e+47', 33, 6);
    T('55704962785747203499055194903610.12621604008510234275118107691213', '9884401353384.82618416498', '5.50610209549691035649918813398528068047734046238506497e+44', 54, 1);
    T('27.368548940713', '3904134161061604542291067186232.01907083935180783669855889119396', '1.0685048685812401392745629604987263903906355642816461787602868010178182369348e+32', 87, 6);
    T('3297272434730567263842564359724927341341.8183701307', '142491895167125659240006694998967.45576318', '4.698345981070811731981399782610641012943894584192051041560419388898602855528e+71', 76, 6);
    T('7820947526.911368061605819994875441', '5780302646407970151213452.70', '4.520744368722365015916652208326568e+34', 34, 4);
    T('20152.303755417633116850', '218492190236239031561905392227125768268.1263526134945316302881731552819278', '4.4031209858272e+42', 14, 5);
    T('82793160661521601628414.536528185470750080821573', '2844726782876363.03269469', '2.35523921572816202388e+38', 21, 5);
    T('16105380211242010571268835935013.7250925397', '434765068907.9637672574148749', '7.00205673732958878018673040080395774087900293140455218352899779878353e+42', 88, 1);
    T('2180749707517287779610621.5068640794983', '6475834512842006219357123450.2892242010304879792', '1.4122174219810562856429096896130486e+52', 36, 2);
    T('9.299127299077', '471824810135316312067349908142213716580.7', '4.3875589723111423119259160134195878890747867627e+39', 48, 3);
    T('5498065027763062584.3741', '41626584331578158949.834495018206105919939909284931532970060', '2.28865667538679736457852532462471544640339839995192588040519291966328064719139446e+38', 95, 0);
    T('457.95974862036478354', '5159208508987981379781800273610547821.395', '2.3627098318561829576296207614189045895979234819405277558383e+39', 67, 4);
    T('8132183.15114954747886057', '7211316165244.776046159568', '5.864374381661593370512606700974143990201304343376e+19', 49, 2);
    T('393.434771640599570513665419', '755613.330294192587602', '2.97284558052888598193950492164580359917275535e+8', 45, 3);
    T('7492598.0', '35510582182298819719690849004135524684.354077153104453891814', '2.66066517037927772034116215866687823978941989769196e+44', 51, 3);
    T('3.404834', '46.2944537417048269858410234176811245845', '1.57624930111183812885509035127316894143541473e+2', 94, 3);
    T('35778456425935005923889552589994.021567973725', '6082451131.6328518019600310159231213', '2.176207127760050551347551285487237993200672844e+41', 46, 3);
    T('232166522477325193330328641131475240.19', '88527841935771834234538909592543543.214', '2.0553201204650463406902497022684102419601949559876030597785749e+70', 62, 3);
    T('795405575583729409852668365159.60374512213962771618191', '59424223104177078980647693735761714.9692758', '5e+64', 1, 6);
    T('92.90277093178079185768', '7869275474467226151354814665547766263.1', '7.31077496803509316622594011092336509371034326261440280635608e+38', 94, 4);
    T('27290542.661881646320573876', '73195763.791816564320340570501', '1.997552114429581847e+15', 19, 4);
    T('74951905870785146601940294348929.5114144', '4116554217409268432477.364696136691479', '3.08543584215243101372107764413846535730799862742e+53', 48, 3);
    T('51405557937775652394.599273002653781407', '48983.56331556301188600570', '2.5180274020168764394620072055582707387000564885361019301560199e+24', 93, 2);
    T('91727530830.05295796830266135975030414', '8933.9160967047352962265', '8.1948606419358999033707338688398481834801338331217636932771e+14', 79, 3);
    T('730133270782051768188731909702300697609.337306', '8107344180248851690154567775.87178025', '5.91944172368092634975582106964125728513878850428294191e+66', 54, 5);
    T('986951471919692912214995696541201.0885992493297863', '475938992521367945512670386.52829327516591818295876', '4.6972868921294e+59', 14, 4);
    T('66907192485822556911003119162644226.546773395847145853342742093475', '56339156593500427890224953923228190.9391741947825545396198156646569176924', '3.769494794690232191397214167726899845221883995446388542864e+69', 58, 4);
    T('4980871441412008557769269077167.509368325430954630285347009101', '20939186.48', '1e+38', 2, 6);
    T('57624177670783.9301539649386987760633925', '61890639741421996.666574874', '3.566397220618181933102339789396762968934504098544259840071700045e+30', 94, 6);
    T('8406825728203582245448804065177076326.7161659704574378366678717', '73530.124933', '6.181549460847681041659912335679086970711164094384985912892698545282e+41', 67, 4);
    T('561776870252948141135197194431907992.86', '56.9971109100276649842207066875374', '3.2e+37', 2, 6);
    T('996518.72417487907609967862236272', '70314850461866535786402.198098625377283539657564256702564544697', '7.00700650728066469850241354e+28', 27, 5);
    T('243825205835814738.5196053276811', '2687193339626774242945080.4111232748', '6.5520546915512865178658134241514696267371809959690030778092206628e+41', 83, 5);
    T('6215283117381281.661870276924', '9835676697568480913876785567563029.73956234107190621248911954465', '6.113151532641785753222278388221187617329338299325344000585100848624051988323700443628266e+49', 88, 0);
    T('37048829952104388117374721.2657048996722884883', '936540.0126861624639040588767', '3.4697711673351319517397221435954501e+31', 35, 3);
    T('685143921326.951795853924047693943685016914', '7042161158470714.3955010580252245075', '4.8248939107309748620594899425466308519697851928284273186904751e+27', 62, 0);
    T('5110440901930691346720142276203253.4131910212058062679563971', '368.759094773060477182421730512172158607593', '1.884521560887184474069265483769437665793376313620727215e+36', 55, 6);
    T('1299535201565341647331425891.0793380162884377118471192738321512782580', '71228215734086232594489766507.422124751837018923731633745043181842', '9.2563573691135391645599509856e+55', 29, 2);
    T('1043330447112.7', '268279059628659065595533442619444168009.434238812862683643024757093997489897', '2.7990371123334356693854010458421015368403748233239201179e+50', 57, 2);
    T('29857.7590', '693401156974767202630279319077.9169536570', '2.0703404635273768217e+34', 20, 6);
    T('8632.80464654096364582955218609', '654772472163150424518065703998659502.6304970788887267688249621116377622', '5.65252284011715875850663674441797382947e+39', 39, 0);
    T('93809284333597277741501201679022334602.957', '68891498096549831351678754578.529737943719833819857132999998110332', '6.47e+66', 3, 0);
    T('7456684.1212618768064363399126', '15747285769748771756794203022348108822.55867633139142038928368535240', '1.174225357522587774255628366081450286706782887526150548e+44', 56, 1);
    T('5949.8580991360861747860470', '80354060641793595222139.4964912525920478259850614886955722610', '4.78095258508047837160042130483046891765858507078582617893779173996146744707903042267e+26', 97, 1);
    T('543408136731.9423322556989090091272316117', '6041963.73816643633834951729694', '3.283252257158984258303078864572876459798264297127506349736066e+18', 61, 3);
    T('960.699606375742557122', '37635562947513334885896526711014185916.4', '3.61564705094055421652515040892604953101908754766452549166008e+40', 100, 3);
    T('7153029708271773.5476996030987921988886053121', '5.3008351690233075882884435030349492193807', '3.7917031442675547303105358789665121230984189453912216890062243900691901e+16', 71, 2);
    T('94722379312916861997292965.93807697661489388677101836675991288', '4254889.523148195303752307654777371847109', '4.0303325934619930645673771234485073814845673823448674206786312962749937e+32', 71, 1);
    T('913691389148757630974994692337210.5278604', '662929777506266822946016726916.86850398266839145502108480241371', '6.05713229317777752866775672423884386486122753210835541129e+62', 57, 0);
    T('49781662630047180.0', '19162064262937174206647379.33717219360752904', '9.539194184328220854327831174158087837441810813021784201072e+41', 90, 2);
    T('5819448308302230759659.392422521', '38982.349', '2.2685576494169715695157755654266909e+26', 35, 2);
    T('6749137860524.592444478248', '94.0110040430320776704422466041941837', '6e+14', 1, 5);
    T('3217298154562184249.54967343908565023608495383530851433564', '6217446429126479610596.987', '2.0003378922517865138566091939988025146278225859804635e+40', 53, 2);
    T('363668528268340254964832.1637172816646094121829', '2506089796645.7956082', '9.1138598805448060121965591210403787e+35', 35, 2);
    T('773292844799.73844443466046133773762407', '7901778373936446177.303017848019225645', '6.11038867775836588473895317560738431004812919903e+30', 48, 1);
    T('52102085970712658131.4970135417123501909995468197781762668106', '5177020174.4026550557064305274447738849626037', '2.69733550198840972632883721748593651504985048784619541597297809785635210811669362710853876508054471e+29', 99, 0);
    T('70607794763469678203388.004271640098155395', '380889534426251535817643.886611', '2.68937700743222869126443581638003465533634495029362673404957e+46', 60, 6);
    T('9169136920214928902884338.0', '22417791791907.676250862013559', '2.055518023888718629381103e+38', 26, 6);
    T('8216824742320.3801317286603305748288383989546395654', '5747564832836952033869.6774', '4.722673292654516710203090554922534880401573674863182000429566680011337420196e+34', 88, 4);
    T('76535842398860620595643.7811409142', '9784896841242643869229.24868063454393362515', '7.488953225304561013999938955112870084e+44', 37, 2);
    T('4965607.524907130725715402812912345202548', '7393730.16777', '3.671436215821157411431306709318752401673937867147796e+13', 78, 3);
    T('98102558598645752136853712832351687.193882439640440120992783', '77592872956447393079365960027.3408204016791139808733343071917', '7.6120594e+63', 8, 5);
    T('93520600848395284490645427478.33966437594378697358701194658711481', '39434.235754421786007', '3.6879134217507977419900970542058944171990995477948338768981786213522171931733605e+33', 80, 4);
    T('44271.4889077250306', '953450183747629196.5193270494014624559311206392367739860213', '4.221065923385155822004737e+22', 25, 2);
    T('82472629908413568740750.4635812307043', '5.459185', '4.502333441065627282659738195257009424539955e+23', 76, 0);
    T('74369673411883531277826217.5764596', '907964489184.7539007041328967552123695035', '6.75250225302578042611249209922119853325962194305746925196145936985e+37', 66, 5);
    T('48792044349820786467137532418425234585.165381636005497558296424963170950', '4577838727542742069830706045671923974.1216988250', '2.2336211022059262694055304838875519842031965481749846185478214608504785398156031e+74', 80, 5);
    T('149135722146866976487916745.881360388110934961313044420831621253', '2264749342194935438953812942988209090.3583816334114', '3.37755028629883649822932317e+62', 27, 1);
    T('90320.261818807631130049', '2693563805735960845322010788777.0126', '2.432833881597358797075662341261847069205178553328123116174e+35', 76, 5);
    T('28634030146227738811666295079142497.508', '497550771440031368569036714552.61622682491442650369', '1.424688378869e+64', 13, 6);
    T('427051481886623018598680567152472.633746659634896333747', '402297281.408741049258202', '1.71801650184562661447869960768638694665223141012552981678781380052383769142894e+41', 97, 2);
    T('467599908065287978.445695284679', '97.290558515671210629', '4.5493056217548378535709819961000506066056725653091e+19', 88, 2);
    T('844501440949919775715243696450165133.73851068734249565906', '39095947157282658032511602229083931.93502946516089283863633524', '3.301658370962712455050457678980912820290485787108397701598465093678536318117372410547806551269e+70', 94, 4);
    T('48600405871945601811777013662057.4912245', '96657164735398653463.9', '4.69757743657188206942744228661447157063809021088509077754555e+51', 94, 1);
    T('65818757430122837634566910246481.6151841497218300348738', '6161886994.8807403737681960301930354015727323803607', '4.05567745427884014062776186694706014e+41', 36, 2);
    T('99138398.1621943624360409421914571715', '61313315248821269591434132.97527226623987250793529225512986', '6.078503859781786131243447289077990647e+33', 38, 2);
    T('86219779491844893228366421548500578.499721239727793439142', '2838005338314.215852008855804', '2.446e+47', 4, 1);
    T('3239522457427096412117370887.85960694559257413', '5236142880549847087295563707.9702260014560875401637687698059073', '1.69626024518382359848267730454262184508598131442637e+55', 52, 4);
    T('864514316654704322510241972.62384852117143597084085509854256', '12818801093233439223814.3936564570414802682093157505096696', '1.1082037067449283424042419414106483042495e+49', 41, 4);
    T('30555901276675384752469794231390078945.9219137080624497314766202942705729545713', '979513447.83255083254994793453711058', '2.99299162111473478662e+46', 21, 5);
    T('322438540091.60427354710089286776', '931208709791081394578.0', '3.0025757690562268727004e+32', 24, 2);
    T('75285719250010.13718648444537997198407', '536119039456408068532874860578.819917948', '4.036210748910024519968160748818222206523197230199484482463715170193616308836e+43', 94, 2);
    T('292995758967.8769999295153677', '504137570023556254601842947.58953697', '1.477101699532731014988635684094927390343154508494933153423e+38', 58, 2);
    T('43612734563648800668.921', '66380569771287811867384.5735074838273202', '2.895038169618944715720293405742027568365622e+42', 44, 1);
    T('31463929772092462575302889195.979269096375811160164066882521184', '24024261476677053.34426672751072', '7.558976759285531674337896052718441610580832669550250917507441e+44', 61, 0);
    T('2215170427610023597358902936896188995.4', '9273980828228785321035810647.36152219809451062472142701', '2.054344807691471918e+64', 20, 2);
    T('15862125605.60177540206037669856408', '970.8804072761565781286494968305', '1.540022696823220351136e+13', 22, 3);
    T('445561775552459493536598650.44743397487', '5470307011348330010822740441998481422.76850322453', '2.4373597047934301038871524456995934008991180697631335970000482855866092020918699875611e+63', 86, 2);
    T('8742847803176513.454943', '49965002918330129453.3918109432529254082806', '4.3683641600023065601164371e+35', 26, 6);
    T('8369875217913192170829145392565.1833022599', '967703.475', '8.0995573336909783120541576276655670456088805831525e+36', 58, 3);
    T('402991228897259692402.57758726741', '646.5733071803846828863841', '2.60563371632788607585163783943018676000964574183814672181e+23', 98, 3);
    T('879209433025440135.86534970846286265798954600', '6616212235405681787555.4783833857386858874620708143032541670', '5.8170362082670093476e+39', 20, 6);
    T('6103508079659858294529031696210426683668.474350800339247639467871993863360173165', '925219597118462834330082.9654825', '5.64708528647217685481432662715687805924457421372329019129832798686607399506869234896539898462e+63', 93, 0);
    T('9766439411093279035139910500144863013.007670744583252183430940797080', '1864313915.335', '1.82077088973773626709053575940465476484675556802225376209598248509868005e+46', 72, 4);
    T('8789238065414387.82227875254685475', '6409814733.0600350037656111', '5.6337387644065222753772138900268299153459872800839516687725e+25', 98, 5);
    T('32291611265.115322', '63842337213748104090453649.54349549733916864376', '2.06157193556275941693184950741902060512693069031933556273569072e+36', 63, 1);
    T('9532.865425312305849971854555166515', '6544754982322800.7828686759509445653394788843956894', '6.23902684881254790397835855208592678118072392656252364614028394310731e+19', 69, 2);
    T('61838605602.0', '8074.7699133448972463210184482493', '4.993325119982308174144613053042545100725786e+14', 82, 5);
    T('71621441683879272380776482.37651986183005148901221157872175892', '36047723584927173.957130', '2.5817899325744610551106741074991050384426480872908258729053226e+42', 62, 0);
    T('9013.940957117669841006156962000278', '76.24005581786557748464665946787', '6.872233617095958164e+5', 19, 1);
    T('75306785954003652164.45343843677418846032190735177', '5458187741225220025557605038200835670.6994057196426874559065', '4.1103857592521432037047e+56', 24, 6);
    T('41785601507809841484.456553747456512091170183783588', '36107662413117891848564635305583.23390369799820712041824556343354660', '1.508780393e+51', 11, 6);
    T('67901334689024.71491981431827228976239981846455012383', '30793863895278036352055.441299729539', '2.090944458721548260928569979971777601957648346430689515831770894852e+36', 67, 4);
    T('7378831811011547319001409694743512723.97817894521733055', '702014.2237319066077085792350949854331149363570', '5.2e+42', 2, 5);
    T('70869538769419.6113288023627337744517338', '29758927649276656082947374929.03664759689409025', '2.1090014767767651962517650177614e+42', 32, 2);
    T('1349616370.416261553626714995741128', '7080531209674182.519', '9.557e+24', 4, 0);
    T('58035.831787123876379514', '289300.4', '1.67897893503476522861439520056e+10', 66, 6);
    T('67424571409.3331994233909229847113831152', '93127047787169908287.1232713692536053667692832976876699143520', '6.27905128366642279394590782934447738125937365255007296738377086876256174135380189184666904295e+30', 94, 0);
    T('881.06669253505320404877312870612795752757', '3380179125422826419686869492444214.12176590892841885957278', '2.9781632422124e+36', 14, 0);
    T('6198204126.5', '746557750735045333362150481287171366.848767480937475387867879415142622737990', '4.6273173312765163934098492320281065975146759e+45', 45, 5);
    T('375459140628243463061065213.1', '427119376841633014545297.7615018957607132000115', '1.6036587417463040428425665197646765172432694415032647571833799118516995065e+50', 93, 3);
    T('44171675192195088800942851790443480.779373171', '76923552961012016969385078023489619.331532562007434474529392075082848', '3.39784219602343957689189766523786188073608755532153536638892921565566876318047562475870442124669569e+69', 100, 3);
    T('9236904741321935081197040552.8473894380988130518760311200844', '132555297485592478.9035038674761851117185798461', '1.22440065583200874804e+45', 21, 0);
    T('331953778674628620770582534293.29405343222656', '160141133576442689049725666041833630501.344397607589', '5.3159454411938594503848821084794649374338559896898197778773171530753755777606e+67', 77, 4);
    T('99428810848679946172160990.95166960', '751.107505240856260214', '7.46817260656169784514443856656816497197034326753532944e+28', 63, 2);
    T('111289197781138290591251433027.199025335005643394159850528739965644302', '29315019930125.83217300688268932736738800', '3.262445050961784528357660621325821801289799631815628173645238402688408498878509233195710436772262883e+42', 100, 6);
    T('84678709881317028094.907913389701', '21395211562282116210076205999769851348.265996926', '1.811718912731886963989512357400951342280713334126372524379928705381682706059126e+57', 94, 0);
    T('5420869672958077.8072836935331405139928672172191758172493', '90348065986.7', '4.8976509091771726e+26', 17, 3);
    T('248463908876467.4527827293780415361305843519117973910741', '222588243040202648080383415936.993689407866468962', '5.53051449357139014508302672229773796463404832549054954506003458240786834877e+43', 75, 1);
    T('715866.62821277886908018593', '15.3531951529459644753859050', '1.099084004643220735661353138207559560565e+7', 40, 0);
    T('78124.25716311', '91421645541915103068956736621301.6620', '7.1422481465912643886824372752614160269156e+36', 41, 1);
    T('7113727119470833862415.5738678071196500368477499752', '938409530272260.1516186216134370942546', '6.675589325e+36', 10, 4);
    T('483550043270914.574736903', '4.19041436558', '2.02627504e+15', 9, 1);
    T('568931691987704303454135339993.7218747465873431722963742614145654119', '832570079083359211569.27413527157014000433940', '4.736755037912323362363175823173978346215e+50', 40, 3);
    T('7218260053.0', '1326269717767922.2843137351873052603999540627', '9.5733597232677777496703432217460338617512138224673231e+24', 89, 0);
    T('46369675.815949552557600286282465274431463276', '28609992259391525334157560741095651.139843681107', '1.3266360661648111102260752537357e+42', 33, 1);
    T('1258176312138607689125381791252.71', '8770945188043789896.18479524583979785215416708118124014661', '1.10353954706628025100447855198538423124947922777032088624477341811652591325584e+49', 78, 2);
    T('50781193824968505009604715.05378935946251818095529036809', '59708357.18547682794092490597019', '3.0320616592061497605782084642424999912058828617465974102948836e+33', 64, 1);
    T('36490280959490609357381413049982319561.784981322793804577676527258', '784485837532509291074472826236.41985099011862628455065924335268667', '2.86261086203025674202e+67', 21, 5);
    T('3148952220073959002156160262.9920991877623671', '27933795347.0570632438084416983270785430888845', '8.79621868732069653985737266030144868915967e+37', 42, 3);
    T('8248321816098088346503230099.006707936536695851573172', '6254411163.55163248682004', '5.158839604717035913283619146328743192436741133331216660722578394738965596688e+37', 98, 5);
    T('822252.6458658779945509', '55556005281.0246866066599310036117', '4.568107233606123931061506809600709282714116154081948553e+16', 84, 0);
    T('643549372389815206206023775603195274.47', '1236815690356460740373859294.3898689989199', '7.959519612907e+62', 13, 3);
    T('45.0500499465262', '909597456707530928889.02899', '4.0977410855907471242510033065723341594538e+22', 81, 4);
    T('675594186579288.4299174582871646984170561356536927295', '4875256965329297463526381871.19100611939011912506127030535296', '3.29369526385665689471273175001110993069869884405e+42', 48, 6);
    T('2708.6566738050147309323503456534635', '690952.729741570796774871240136063295', '1.8715537226982984300060654644566987964414727281170678826637391719722325e+9', 74, 1);
    T('909596209558763872634960479824.60303', '70305126374111576796108738949357.854776673336904684349428842', '6.394927646244177067564175688736086e+61', 34, 4);
    T('7955731.2287389718134425617803659190', '3661233361609071666931621.77142921322455813041927526', '2.91277885906543e+31', 15, 5);
    T('985024550615978833325.34466407212736551841140929178040087', '17312561806336308004416.8173758183376998661', '1.705329841329778056405203416324729765945255001886425352163392034112009832533776003069886440332351e+43', 97, 0);
    T('639040425449017580095835325.040418699606306005508417', '768879350881292327670057610.3391551330576973821', '4.91344987506145519141887425837169187184221872798139307013635178804174226495446057423804840415e+53', 93, 5);
    T('593521694476463046773145165628.7543087534998096230189652448314740399102', '32815515653059730369255048734028153.5304420948003110605', '1.94767204555229080214321776792e+64', 30, 2);
    T('879971.8667685799823255013536302382970568102423', '95736147646027033193301.8978954095606094104027890647335404721741', '8.42451165613068025512488549857125501269297955406855491666652280578214864371083876602948842e+28', 90, 5);
    T('326894016056231488194132246.03906', '5951030570836.3272171809465649583073298437517372729', '1.945356282974094788001287e+39', 25, 4);
    T('61891.17057925949315440155716879640979839', '8241829.365796584339199761100446458666', '6e+11', 1, 0);
    T('3590209928799149.99642013651744539140706732869', '37505540619512525244900652464.57471106689904511', '1.3465276431715369130641167314419471151118736074655215264573467669243100540176218484e+44', 84, 2);
    T('146731720991015941417251247731226790.966278235', '6221195958677221185757192948368182445.3', '9.1284678963906195913629767e+71', 26, 6);
    T('661441928608508351174849031461525.60818491738019802033894984467766986', '868137510863310266269616851693658797357.773376120917493659122446699032', '5.7422254948281781231203925768450867832333088057841817795936083465738e+71', 68, 3);
    T('139070469980.372656', '1568619648930800474014406799792999217.0', '2.1814867179725358e+47', 17, 3);
    T('53055.413397512448937705265965561622519', '25721428705326638593636.423395700512911782', '1.36466103313574822531810150721199201106285055064552946524e+27', 57, 4);
    T('70975859475677554827358419639394491.0', '327147931857774530024293354202316539.49', '2.321960563929594138972897e+70', 25, 4);
    T('108463242609956369079029.196869568720974', '651589267020472379089053201378469838495.47309875009456192701189927813776', '7.06734847508851380502e+61', 21, 2);
    T('4048324259.321080210884950064842113851', '87647038665430.8755731445719168081487928630', '3.548236328869165e+23', 16, 4);
    T('7839786.17156503270871093721782732145606', '61058267923542813879.54334500834', '4.78683765e+26', 9, 2);
    T('92238114622446.0607734572310313', '8173910019546559493.201387152919489688906', '7.5394604929649587562514052668540470741660556214e+32', 47, 1);
    T('8491643168711606181468305.560823345613', '8.33', '7.07353875953676794916309853216584689563e+25', 39, 2);
    T('86473583868818816290927402768034487194.02179463273366921706603215648566432', '25721310818107521282708871108740303869.965621467792075600526592184', '2.224213928245577463003794184297248849119e+75', 40, 2);
    T('3572855911947497322727541588019.506649593', '58305659147707923081.70381063544163575183144681819', '2.08317718985883931026316647185006159147e+50', 40, 5);
    T('1.82166379940429', '8316059093471145654867110.3394233158430129675165165399', '1.5149064e+25', 8, 5);
    T('88941984916774954160367347815.4138207734814412304772066286530597651', '998392776817446141450116359.475', '8.87990352967143578345740404642518269494419673312e+55', 48, 3);
    T('316544.1124495', '71041507306936106.83537179257707638608', '2.24877708775487589134076e+22', 24, 1);
    T('38631324221264140375.452610445501728491721778849937187818504', '92752090739334246786610.7770371242495742207431823', '3.583136089551332457374290669761193373421185588707677639466107925692945583e+42', 73, 2);
    T('745261803713386991106644510.4', '9.62729845267239646345352341530503', '7.17485780972572983232181181897e+27', 30, 1);
    T('56051.412296949030743296947672780688516049', '514686853874637191.83520240484557011052662111926269', '2.8848925050346847984468389307101441956269308018765478350361709394868177e+22', 71, 2);
    T('3679899783535.0404229920110736', '4787333883.21701224332069', '1.7616908920560247841984567503827503996212082115992784e+22', 89, 1);
    T('973454570398175950734283061260472.493072163196239097279989921144544717609', '177.33255490219335480965000948', '1.72625186049925582907405569238592130153927445553943968131e+35', 57, 2);
    T('888865507571238819483371124048584869287.225', '53036808714852805541623621951000008.4444012446468760511071', '4.71425898982863414275655765695338673367594325493e+73', 49, 2);
    T('6681661474705.0325357272814456721002719666280', '212271274249860867308593150832561928053.90740761490673706', '1.41832479534184176170249351494517379959125247287467412919797737203665496502701054292573381269083368e+51', 100, 1);
    T('61112011865073379421280471277992.201424261544234259797244265', '2377.381653040931826365668448423031670450693', '1.452865757884451899933778037859072398e+35', 37, 4);
    T('205980985926642749875929092598706.637622273090216', '706575802.0871137864687605721629301770118486', '1.4554118034581209778570833212686e+41', 32, 5);
    T('602626984313949248593910479716.2376445859868936479022859690', '210069132521059974500737242073317542.9084542278086581531521072951589', '1.26593327828613735224257572393683518072290539097034085833792963587122804367820934985800711591076959e+65', 99, 0);
    T('68507658711406215206853.23797873340141210122', '26957653897788494536274538266522221195.811570', '1.8468057528899036703079524273376401778001685211929236613511935641735273731533768871154e+60', 94, 0);
    T('9865360430.820409134186291539748034403957', '272444.65874992691099302734368193757196157', '2.687765e+15', 7, 0);
    T('94663120.5068', '1070353644861239836.827167', '1.013230160683921572380268782850482356e+26', 67, 5);
    T('5048119026803945484763936018080580.44067202', '622.68847537273522260068191857571213289212', '3.1434055403006447070612974470547031267799826852197239e+36', 53, 5);
    T('25144730643519997975039651.723', '81518.782', '2.0497678157779e+30', 14, 0);
    T('89670969171069608128179535104.230031183205717262952119', '8545892482783076711147069219261281968.442499511', '7.66318461362916783625147005904959898653323970828628418814236913527276275961124045443351e+65', 87, 5);
    T('1242143908394033789588618060593155528857.34947864640322773301246', '51613.3157565', '6.4111165758954276802767626199873185993628126287e+43', 47, 6);
    T('315946325811001494755488076879740.8080275415533638944470865233018', '91319.81097544179', '2.88521587514460020037936673749e+37', 31, 0);
    T('296360.525012878819124', '1576.529631488', '4.67221149286144051167560326976512e+8', 80, 4);
    T('55757779462.6', '9.4868742823990817627264602755401379', '5.2896704402741963376684763293310663812259189254e+11', 89, 6);
    T('528507.5782248746669806509296415', '15920102715479.692797225102255882556191385', '8.4138949312494233448867363895407559359788552530650614935350618434384775e+18', 85, 2);
    T('7736945131145.826824003812736092448901883462294558', '899740530279621326.459240121611813732831493450', '6.9612431150414805942307628120182245109997365941e+30', 47, 2);
    T('8576422626051559.1352884446190828040', '8832576.5805090', '7.57519096314104982074941e+22', 24, 0);
    T('61921604918626332.094368703706027035', '238008114124382407391727.766609662930', '1.4737844410237335061856559508834647e+40', 35, 6);
    T('64077492935718939126.3448660491045154809633418520', '6.718354823', '4.3049533371043576365192244e+20', 26, 0);
    T('45058894469841968.60988313491325183719506250711142681', '252769546634899835147048920.1505606349292302185816437807', '1.1389516327011749768077351246537727812513997458947727212169e+43', 59, 2);
    T('56133216571923.77', '80488170476667319.48811459', '5e+30', 1, 2);
    T('80653199437066781805554471.97676425069661699409178523', '87190494362447756176011804.2', '7.03219233083094578099773979054862686500434284368e+51', 48, 5);
    T('145217944360287.1812', '676222278.056724151197528407129936', '9.81996091500280150377499713059243609833546235764032e+22', 71, 4);
    T('917255067291250890943323873656503365017.641840901547730562207494498467', '3489302755198787825269765.166838394310996009341709032', '3.20058063351941126120924441360803156342979953696813586342661016325e+63', 66, 3);
    T('53.42961826822761371664971954041975011', '96.9711702576115871', '5.1e+3', 2, 1);
    T('849545.1301', '64190362830641137469214169826.1003860189271951125', '5.453261014212322939769553664967794681610972503427689663625e+34', 100, 4);
    T('43993978736646272830844126.55550893866001370424296157', '56.32731589834542666620208', '2.4780627379241662522352167517584289674807052376113035841594116984446432940656e+27', 87, 1);
    T('20453215.799538867026', '8.552720147704545291806', '1.74930630854064998216681373491071801388956e+8', 62, 5);
    T('40835266140303111976764437.0', '37472841577206209438689724050425158447.30470227001055', '1.53021345883863138717492245178145600157985943288e+63', 48, 0);
    T('4407358280380939336772310410767572552661.9051492', '910827376972751098917962258933035136.8', '4.0143425818985058670932429619992712919080270924e+75', 47, 0);
    T('7258050607802085392400.0', '48683060642143947292278879606904126293.67525587148', '3.5334411788337865822448776e+59', 26, 0);
    T('6521568973831380632663976.29954970850663380130826', '24697382987493297976876.672185067943406008971', '1.610656866260672650247292799020089875186354e+47', 43, 4);
    T('90290626627387724624710915577870485353.94355984648457953735478308', '300318337525612961788179182.6496459202872989031', '3e+64', 1, 0);
    T('9886034955560312649216939009.746641981690142313074808574724114261', '4059684943.79977168471', '4.01341872629664462219088692678963272719735434078785304943423e+37', 60, 4);
    T('224328224706385.40586', '886083087.0023248822446365', '1.9877344584958518574276739227853026186066989e+23', 49, 4);
    T('8682435094981814767859255267186449846321.10', '171996784631924503688683245564449.0007819262', '1.493350919112250166761087142423401584646100423971752062951660495176683993724e+72', 76, 1);
    T('12312757.6550103904116991', '69517629188.087887882823', '8.559537209438028931296202402846816932291345593e+17', 50, 6);
    T('299534541590833411845612556.825171032', '2506038.63615113784767890827122472906', '7.506451340884484394463321290961586987890885e+32', 44, 0);
    T('9998076013811165618359201073623.0', '1136846.6043837484125897170808465', '1.136627876667182652e+37', 19, 3);
    T('8719009208034360397225719857605.5', '90085414530067536048340266.499703719684032559173069', '7.854555587972512102948474726435827288800759573114385954284831592569638875104262e+56', 79, 3);
    T('43929974916669522581138837377932883.5525012332503354730255807100', '40809491642699737593.896483959870697402379772121957033', '1.792759944225833983199238875994050747583143477033661671338495e+54', 61, 0);
    T('2768.17305494347646987199692507267595', '344750.7117625771021916159522698173284', '9.5432963097376e+8', 14, 0);
    T('3395.98277696683698732835737448599', '1915155457786521670.7', '6.503834949857065811719725582361090690214816408543493e+21', 71, 3);
    T('805493704.67695248440142389959243828', '2118073593338103.29423160984', '1.7060949454763337279605014822446682386926849651164750152406752e+24', 66, 2);
    T('469050992.76629251054537709333635645497663368', '484495591103797446799744354240671482.8565277220207308019555744418320549997', '2.272531379981e+44', 13, 5);
    T('56641021.4913325122971839293576101', '8.878040247147383785050727564', '5.0286127e+8', 8, 0);
    T('3460396480993448677956248204609.0189479913326420006711394081421863774', '885872937314230989.06153458966866', '3.06547159489e+48', 13, 2);
    T('156347805221152966225743.940757289381607', '9941582848.86722214879156187223995228', '1.55434465884464745541691210930092058863801332772394156566745580204038971396e+33', 87, 6);
    T('543399334276304362994968320668724946.2522460823254533720291195546644', '1868467820592527169058.8438470116707988548', '1.015324169826676559988896124369108996783754539766396231795913040140685582454684955438148336797e+57', 94, 1);
    T('52.737063102947386675240370883620754', '537445309883.3082011720044468119', '2.83432872216991373869759870660868301656322997343029293862839741726e+13', 98, 2);
    T('126039280185568461701691839873.43391977531884078581862', '5339644436.77010', '6.73005e+38', 6, 5);
    T('95762572065430803271.97980387519515954263809140685225395', '832052384059185081732234090082804862.5377', '7.967947639068121950406727542607551199339165483661987049459543141838516744649144399218316185e+55', 91, 2);
    T('268195939658199310703460288880626.82244196261164735491', '913361609453381018893061493.8834806', '2.449598750950747808520198533778825411646137971227358829005611653781829387412527342998e+59', 85, 2);
    T('9847513649250659895358783.3928195897313519127545463163155724', '0.89343229039926754786679', '8.7980866743880664816426139388459332313956790708198826793904572625138715681e+24', 74, 5);
    T('5383894.0547747174425463033809942523', '86573352201806745902.306665643267', '4.661017562212250333309618270169835102797067972693446139636091942641e+26', 80, 2);
    T('81395928893829881799481.14206947770378804801310654228', '6693505982324665550.6521168642199791', '5.44824136987723410790335777029621139275835564889781358529160351672284642196975294866348e+41', 95, 6);
    T('8115108274022998106272277067630911522.0219182678094708', '3794141055663064814934595576241730487557.90586943377085593763878446040305206616', '3.0789865473621689895084163995315791452024275796202e+76', 50, 0);
    T('7802386.0552744527453072489069024957', '95762360271141190509063728804.9221350606643926193898', '7.47174904399720286437533713267317882230604057774135821892e+35', 57, 5);
    T('117949.014999', '596041312094483517.3844', '7.03024856602558764971308728486156e+22', 93, 1);
    T('96349986116253070253695.966839871649442778391018474535376', '783559980148200351856650787315317058423.33695994885691322137685391943436605512', '7.54959932085306352467367299838639544982050004147706215492425451216068e+61', 69, 6);
    T('91059513454058176961497.07476223308964512', '2239902802274220789727328331.136196160608599419692', '2.039644594e+50', 10, 4);
    T('284969341394172043.66455893842510887938053585069528', '72562249489244369649314061.09044348174887', '2.06780164470295648e+43', 18, 6);
    T('526454639.06009898894637595', '85952878122689917996980744700039262.6000785968630', '4.52502914282573995602448e+43', 24, 4);
    T('9150000950735042931033.753561', '69909.58718127', '6.3967278917411487007558042661777781500247e+26', 54, 3);
    T('6896.6560626165', '265109.25426671538', '1.828367345694e+9', 13, 6);
    T('354233921131071508670790976.1731970432804966364543129070', '4298765113239298.6998628339688208802594891750945299849009', '1.522768422084211418556252331049946306244243014e+42', 46, 2);
    T('87894193493039822350069933.746662703634', '3548686612377069.5664', '3.119089477544301581860497783819828349932303811143240842976e+41', 93, 4);
    T('32255319644577662725318273.94122139893877506', '6936643064521364.07', '2.237436393064793540819583057963845879844011099269972860960942e+41', 61, 6);
    T('71031.2885243735508426571217622099003', '1395384333924462380597855758701728147131.8281260469612166338330', '9.911594722537929557159822151121556e+43', 35, 5);
    T('7658336096343114444690594.34826367066090637584', '8260611235668946578429527775.5630597024092614134065276391697691817', '6.4e+52', 2, 0);
    T('526945.443154188906757090728', '21146315307186438736239599038376.753344759862380399193910391300944', '1.11429544906235662826159458466319431563092190431116078931590331263637335647034440605600472e+37', 90, 5);
    T('913669.322344688779753975398944175', '5864785394416339043287256334194831868218.21940948595846017359364997158769531', '5.35847449701340480052270137606173993867106165977701760535810826104843978846054747767e+45', 84, 0);
    T('66248036579983.2549691566', '3582730048235659607350275596461.90550394680386688181945291258698', '2.37348831291721e+44', 15, 5);
    T('4876091687.1611280383001460774337957693066481294', '26516984531842475173066828303400154.371488889631', '1.2929924784429730968414614322797093126259241992803501014e+44', 56, 4);
    T('58608856.965510319146252', '2500261193372.59780349493962616712672531', '1.46537450658790721773117004442950609480550527e+20', 45, 4);
    T('53550002019975334553617437438341600289.0391455648310767284884032491539028329', '10414363592238810112758570996688548.37021604050418596075646352824', '5.57689191401145862933825026641155825192679793295672840536077310313753165749307e+71', 78, 4);
    T('3542135306074387057875145943750.364366907608865027823', '71891483386791591846269945258014451486.10142552193396643952528526177778270339', '2.5464936151041474747803785677103685453079260385478520317537e+68', 59, 0);
    T('48385506229.097', '1482064254554.191963327', '7.1710429220653857092312055e+22', 26, 5);
    T('67212401821344813309248264113113381683.162366119393707685698', '433033834214631583138892048.32594212450824615895', '2.91052440674714317e+64', 18, 3);
    T('3455283497515463096773110621988927.66974823956553', '90389251461350543484094678255006.737', '3.12320488927179e+65', 15, 3);
    T('4535958885758076351433238278610720.20683893756135453', '23377264670651334.3316148241677225402', '1.060383114075592702066103043092255654167532501080780247756455872410876876123e+50', 76, 2);
    T('22315706247054.184424401517933085656262014669184645', '62952.38861104238530950273827', '1.4048270117945e+18', 14, 2);
    T('8394229483645390097.5884829639204225300399828242586754', '223493321122044275.36689519713300431066', '1.8760542255604910738e+36', 20, 1);
    T('326733290.02087403', '461200860840590575157792.432738128329795852623985494', '1.5068967462290544476951010356919339161115262908992188844391192132082e+32', 85, 1);
    T('686349096074548594384.98407856144923840748231582', '109348505537301053911998959744566.80408198', '7.505124793262935001825219134383091507534938949097049816099542010850999752e+52', 73, 4);
    T('3184028787150.3395466935028128608586211657857719482', '86885664630430905.3539885698', '2.766464573739820702962747244259354905604438701729628e+29', 54, 2);
    T('945009.40577516761056332969348', '153735737935884269284851.7926169204077744256', '1.46e+29', 3, 0);
    T('469100021910032778594707212.75452016491936275865245851136389432863', '38188554261703.9168908902181953160088939488385209', '1.791425164087778305459201922991795907433106383266689457190343087726894098252512237169861e+40', 89, 6);
    T('86775287103.952574754', '549834254793930612609246996168019723.327587639234917985699779520892', '4.771202531933114124102918181911957507421803424562e+46', 49, 5);
    T('639287306939606591676734391836.4787557500768968030773550', '23586327511667.3136928544164449385843980188617989005', '1.50784397955293493423707276886216310665992688306453848563552315152285317757266485122e+43', 84, 1);
    T('3.465535784922943305218', '61917758.660957025212061253', '2.14578208361769075388822583371654122202490518154e+8', 52, 6);
    T('89.44623481537553721870', '252543059680.318013271097383656621418192204887396556', '2.25890258171591231671551975691176403017335256038897322037305893241987972e+13', 89, 5);
    T('1206550177052135265351.3852835987350', '91238539874129177.68811048980', '1.1008387643911e+38', 14, 0);
    T('5994440.9803776567759577', '1854886800039.0089579334505062837562785698857817', '1.111900944811541146469259891331571260642972530466047213042155563063409e+19', 75, 3);
    T('51377609363594876514694.342697691955206813764804954049', '4889311780755766771334612146682.0668836223303604215027287402680977474680', '2.5120115072849222283855542978738167464828844646246429169807924233e+53', 65, 1);
    T('4096613593849981985803578678622.539313404572172573041003539977711264', '869785616959744982665321485.6957898449', '3.5631755821724847357271402502373902817421820870212158738700257640096579e+57', 71, 5);
    T('96649160439905828315.64', '888858532775327701191058.509836888737831232469565205335815901676', '8.5907430942581940160801782135580649095037233152641465927741481e+43', 62, 1);
    T('141976896098970463533.640706443287422342', '23061368867876229912361504469.450113793930401658417815', '3.27418157165449560171341990978622831619688128312659412181683028372998455568614517740182273e+48', 96, 3);
    T('82770251713332282760238785647783342.801034062132992732', '85591813134461772510728871.940787426401936050458861491569069', '7.084455917739901109411334029064138516885954731983182319645771051146580895689031896622531e+60', 88, 2);
    T('69928791738.81613975774', '5387680362418373488135609346350427946.0', '3.767539780188639018223e+47', 24, 1);
    T('74693590991485530473679887.16786114132075107222348268284158626655', '919972959651775609.58201164993303230062759890421792876944', '6.87160839714561480759008929881516824074013356292645793329476010393156187004428799314826e+43', 88, 1);
    T('3066135774405297055716482401983508.15852575305813035526108', '5153424.7958901033098', '1.58011001273859618273641579365084041216503811527351250301877e+40', 60, 2);
    T('42342.1737117809022484729153536769', '7626395628341015307604986.465973', '3.229181684899817343946359472622007573224893402281376591222861237e+29', 95, 0);
    T('7904297941421442873704.834794977755406003481396145990826248', '2909473428402742.4033310606', '2.29973448307441845400370035088664656135322811714653973329e+37', 57, 1);
    T('60845037881826679244374930346.5148226917', '265814.99940580', '1.617352370840363723396e+34', 22, 5);
    T('4.40874728684373489878427864649245403', '341.40218607396', '1.505155961576091084184404105459730287041092e+3', 43, 0);
    T('746461528210502356019063032.6108502', '94612552840114460724898081.56768889703721421417480548259267', '7.0624630780928745329425597083733093214268243159979173114e+52', 56, 1);
    T('85590542566699013819009250599682.0607526117648555148979', '2835017.903459001547', '2.426507205433614672360198251990117523741527509674077281887986733176470513e+38', 97, 2);
    T('27519568073591302212.40552822471803534', '220750785659981803202.58940268567412674366787123978', '6.0749662732686318931259542714040619047146914072695504143002685661887e+39', 68, 0);
    T('9023606752224787336149561498951663.975082858643998562418919000483', '2131.6971824821921864356072', '1.923559708954486406440968561809176531411155681418486714066611323141e+37', 67, 4);
    T('54188.65', '89377325957743019343224554.7', '4.843236634260051265133225266044155e+30', 88, 3);
    T('960836537447994015898379495153239.0787396224627651041622288873059054', '79913377946383296237119189.9090831840390586175', '7.6783693361775814e+58', 17, 0);
    T('15330495627867044898489802916712645.28887403', '9657001994871190369213742.146936669900689548843034320498130', '1.4804662686067611469787470493251946e+59', 35, 1);
    T('38519551.327530567028164728', '5136281874965877083238638129198.9483', '1.978472733154130404916007753e+38', 28, 6);
    T('41908699916648390.69207460282721653918013218', '20057470374909496148851.59826347622972583001972597578289208', '8.4058250702914716685863271128311746385460893498e+38', 47, 2);
    T('353242888655142791.151722166012438061464382500737484421354', '194414107876970603261586884.958705875671845520913', '6.8675401061773645861015705901956535234225616469181161101392832369159428388440041212912082479800511e+43', 98, 5);
    T('885288240552152047218307203728.74099613229711892238405', '388675328468452196858124079277049062504.73425', '3.4408969768586581919776607537795713476672918607492772165453986e+68', 62, 4);
    T('26965.46', '11500354441991979608.80399797621648434809974797320', '3.101123476913570464620198552677465600293e+23', 40, 5);
    T('8781216837033050071058354.8440714534226', '26334665.6653689023467424104426246', '2.312504095383735756749617264530681237679427861e+32', 46, 6);
    T('21425971388912712.5681568579066769703302078511938', '417715.31618026780659346168251482185', '8.94996e+21', 6, 5);
    T('87154.4439711507016493254579967994145422897', '7382493204241299174166768076149488.0010927769638142636745285215747603148463', '6.43417090336449122341179307932008947917139652303750730427985714080691334766701239415818295e+38', 90, 1);
    T('95878428219868.554076182936974990994544531912668034480', '80728446936726076623221246.289506785095970033589', '7.7401166049243585940358235367491092058791466575439594812864977437746421623967579889804836522101e+39', 95, 3);
    T('83.7495686601414553621404799781235276664', '929433307643578239469275.945209', '7.784e+25', 4, 4);
    T('8851929063.4606757784460507600041873845005', '3982573474243702543673365.3', '3.525345788402538762653509963395194094871329169981373827419189453265e+34', 92, 2);
    T('196946627004807686514630342898.1', '6176530954085426013574.0132730837480423137001635', '1.216446937997911347806611517258281195473767631211682468598500764229395033e+51', 73, 3);
    T('3244570.233999400577416709993398066822529906089', '36439673354216906.0863040309', '1.182310795017533e+23', 16, 6);
    T('6105361795997415221.1687', '398558519465.35480618316818836855015942865926054394', '2.43334395821306939363155131741111174380178283749181801675704e+30', 60, 1);
    T('648111795612971696500214469246298406.3586322', '498456.87154218438982934934560809248585708', '3.23055778050829497294681588483201715246653279587462142494e+41', 57, 1);
    T('2121107247584241353985777344159.76972442782301249850024305056819', '17216.8228397543', '3.6518727705776745316643408313410158507209393751769868336127692178851934995717e+34', 93, 3);
    T('49599700.843', '38232333998713072514956112256407816.18490798047510230423531997656239', '1.89631232886582634373518881119215339257836940304835929989184271187631552509477e+42', 93, 0);
    T('901440552561811462.472788491404771843', '24250181850.95547598757079315877306554296444', '2.186009732745e+28', 13, 0);
    T('7.64615', '2602023389.696779756735198', '1.98954611411300325369608341877e+10', 75, 5);
    T('472113700521635.0', '646.45029047', '3.0519803883707753626931845e+17', 47, 5);
    T('54889521302285968244285.2834301575536091786994516925772644', '419828323433684166356657645182112.6528676972', '2.3044175702416210402127436320434803657554437862768048545236762890259e+55', 68, 4);
    T('88.64298958891669123156137596', '49072867794.1785096546940097739560625090692605', '4.34996570897765082594889266463577436258e+12', 39, 6);
    T('547622038308232805014157258.68097', '3142202053273610218285355036.79786328321283608', '1.72073909319000875215682026815963774e+54', 36, 6);
    T('6373.36455463253200', '471065424390988945671899.6', '3.0022716787264599383719976442705163977872e+27', 66, 0);
    T('8345924045636004264971608825142782649.0', '61718436718031918810578475.813247253292466656365166978', '5.150973850640866652587433e+62', 25, 5);
    T('229238847647075152.9257630486529', '83357366224378281683264.62374377032', '1.9108746576171701082041213752164718219e+40', 38, 2);
    T('9.81385463979113626060793149610302635', '109784297186728509819805386013516569.0', '1.07740713432218457367843238853749653414967e+36', 42, 1);
    T('9950078564811252523654500387.7930274355', '209990856417638970341.062799328349826', '2.089425519247506962494820181925491217690132439535252891284195046959651223e+48', 98, 0);
    T('989564247674795525321420013470169.8282757', '744565611340233617007419706334074.63893016601912798199533131967659', '7.368e+65', 4, 5);
    T('87075240351538871498276836392714614488.18514692690340687834311118256433', '641221813072025166.09861541143', '5.58345434918961211714955456353900465e+55', 36, 3);
    T('64593035050088883587.653026847', '861085.84637980673695223', '5.562014825634630857148500292806631120795028314651881e+25', 89, 1);
    T('751252749704840865764095602295.477516155467', '14497521445934287499069080191.3105515291471294', '1.089130285016303392441784774964879641207317021509541704012964663e+58', 64, 4);
    T('921372889198.4844482525238481337812280318190544', '360454820617930.65426696606', '3.321132994982642083139271191624166182187426e+26', 43, 5);
    T('47754702812777478.5', '9105291301987654409885564880.859359813662686', '4.3482048e+44', 9, 5);
    T('2296486558.923768946600918064607568548497', '81258393360502445457.2586235936548533787447832248', '1.866088081521342945558369711541979393409497334561340300070641081528699e+29', 70, 2);
    T('682243154994978836.92671276555924787809257330', '268289723.70268434097471943695', '1.830388275516505203021e+26', 23, 0);
    T('2303928797186742347137772009257.5037436777766762731953551522352727160015', '506372003602881755545310405431.4312371889249152481110126', '1.16664504e+60', 9, 1);
    T('889172436555824509562564383204000899.5860829523021253732', '789073091229787374121422782029.7285139305604705684', '7.016220431494264391514020451492695171472405643637993547881772263479886842e+65', 73, 6);
    T('8483115470107071750170321990219.710349441165013922957484234702538', '84877433661639204496372709.363030174367324064346006487857579', '7.20025070558038256593790000603361439477e+56', 39, 1);
    T('212430389341276000752605477969.290850284979616411481079377073343307', '960744054.30', '2.0409123e+38', 8, 6);
    T('211712615885664.10575990886845392369332483', '6899391653667329665728920328.0', '1.4606882550176282428312000044640769909e+42', 38, 4);
    T('47132883438304331184476277880646042497.086982944017368728472546102', '640707215365854839.2400441142931399344093', '3.01983784999193858488840160120036434335891581330797430387509303795659434436308657632831587337e+55', 93, 1);
    T('5590435965790074475.69708866954294', '5830351604518411469683458332607202757.98058347425526740218661', '3.2594207303101595971856476240891660212134057870135411e+55', 53, 6);
    T('70310403062520078193553346206777.408246524455225959514291556885450661', '833800768267.32525564331252954284715856318244361', '5.862486809071453968627735680921941656772534e+43', 43, 5);
    T('96844326127837470317296877854493300.9858396', '276518479091363587932474877345745.933212698116497974311', '2.67792457694976219572885330246542051325175451334782135369929e+67', 60, 4);
    T('265234425624125185.85819857', '93922.8987198537227784', '2.4911586094913284865107791693640909501599306888e+22', 55, 5);
    T('144669587028445294.0229022', '4573779043018820049334601421673926.515013255458753', '6.616867253128904194390149987186226599415206872767001102644705375494529566e+50', 73, 4);
    T('591063991276419170765271933058.919450474126259823826308291145990', '1993.26080288042040612174573605576228', '1.1781446858053410792512858322624604502783958313414e+33', 51, 1);
    T('6114971403201478925280.6467082118673277231535', '40238504358302875498865836988009.82505804711793891029994528904339', '2.46057303458620159901245434086786640534221049299318248834808109095018317976318656930485065576911898e+53', 99, 1);
    T('4265749118510329691595683207235213408.34001039487310426155454476862681245', '19307117248.859259413392505379381625581658274672', '8.23593183852969675400243186940996568346796623464319915e+46', 54, 1);
    T('175659329471731484993232037213379053112.924548144311', '6139752580679260792223809.67748674672240260122775', '1.078504821444451917352899211003945367272238417558549955612496495069e+63', 67, 5);
    T('768523973628329750647954673.642425229703811436', '946250513376014437229486.80217777', '7.272162045875816073250145598800517518521144e+50', 44, 4);
    T('62341265183391757802602594398096898158.28810242482452', '6887524146273.8603159896565', '4.293769e+50', 7, 1);
    T('963331531170489351.87618072044061988439325083', '6485675172894570179.2615144853648348083398', '6.24785539498e+36', 12, 2);
    T('567.46081870714819', '62695330121146535297230.7189850498', '3.5577143359660741234253670090192304221413003129862e+25', 77, 1);
    T('165097132450648.2443422618', '53727496978636796302968975817440777.14357641900126858417879552', '9e+48', 1, 6);
    T('579591634573531159395792860212436416933.4578138466020948806876', '702007.02760', '4.068774006087900028434223577430347936054510268884473e+44', 52, 6);
    T('16563727003944690.78738756166658171633213576865317', '593125589427246854733367386509137724667.61725120791834119517399693532299948265', '9.824370342326700311789633197412968567035649297045650401984276754e+54', 65, 1);
    T('639002631232663.4898466954976798370', '42753070852842439477492262168.08379', '2.731932476824e+43', 13, 6);
    T('666305306301200750077623197.703', '13583330388926443184796.4610675495', '9.0506451153840420394342384616685441197967627233861e+48', 50, 1);
    T('858872969226880.8786445518558349739527993014853410250261', '342557947071132927695926899285075.1210802821539859057192524948737', '2.9421376113324863982204e+47', 23, 0);
    T('19.8847825461589801756971271581981676', '94413836427942044.5182770802401889715729379524490', '1.87739860671825088184014511376292250324577487543824987636027439511918785e+18', 72, 5);
    T('13.8178117111415992563058366', '183489756787.0082742222', '2.54e+12', 3, 5);
    T('411668039864962138281348068673104.8959579903244153239574798', '6939237579477266899784577.143658', '2.8566623325006908845008691862579700134029195677940443550054672649627676505844381282331084e+57', 99, 5);
    T('4013777916988880410426739863731161980.405629571272293325814973934458317885913', '276295242.7820972300878727757879605974586342', '1.10898774404786321537279053067889619901625897064009129736296408995656105011097548363662744004301368e+45', 99, 2);
    T('28097889934278801957616259999228613.6592418204235985225303920489885716317062', '881264429758430067951155.103958726811731176279857', '2.4761670950347340506121924356190193428155442288163465810096757836080820717e+58', 74, 6);
    T('68559068031317306913244012.1803577124749789216', '326713350935593173969453859.9557510349302884354702865711983345584789', '2.23991628535329783184119862695917697854280295622847911e+52', 54, 4);
    T('303275511083167493.347797838871152', '7279056880.145728583982506483192988498', '2.207559695529642505432363041674306004108108898625528738209805640009696e+27', 86, 1);
    T('6528185777040165608340960368692528.94213723287722', '858828931891787513414445826.485', '5.6065948180865643346886328157411209748803e+60', 41, 0);
    T('713186539006439477570890474906053634812.95445867137919151601214466814316891793', '295639988148713450043426450.85840796820801248647116977686837785153', '2.108464599396857298119626762653459413714564531413186519365871804027637951295097990548084785e+65', 91, 1);
    T('5649996797894.5330195122588008818939', '6.8925415063156540699', '3.894283744003860673165312875946122371307867691875498361e+13', 62, 1);
    T('16011853134016209878077903.278472520742200730644263', '29111963628794850451092112887832826683.19107016', '4.661364860670847402194116806880241852124644324569458704540204783740234366337346196259452344e+62', 91, 3);
    T('75.2351581822631575998053', '4146436959238231918912788032.14366946169706735457308248719', '3.1195784052107063103934212473e+29', 29, 5);
    T('3116478323931005003.09304947249618639', '32641845857563.98408811859564084858', '1.0172760507e+32', 11, 2);
    T('8974574621151.26615420371', '88893973797079.4260210361949866', '7.97785601212554670237808020258e+26', 30, 3);
    T('9416466823323439141357715899.30', '7443853.460977570893751731664044226514', '7.00947991529766550399548864984637125598468156748334504364284140402e+34', 77, 4);
    T('807484916233288.78941094048700', '83400351959775719656902368748023923.899630635553889411738277585253065178036', '6.73445262160662995111357936493830110991245158555985252014048e+49', 60, 5);
    T('180124464097543936114340.809305080927812257', '76.442457374037510907966585613376581009533', '1.3769156668797852346707970030542162815187931081044585957698e+25', 60, 1);
    T('9287268504934046682545802759171.6372977329214812', '734937132981370736900727.758220867275750079411623269247635036584', '6.825558488244409654506372303216132118900278353419341705e+54', 55, 1);
    T('383338644469116550570662595509164965.69333286829', '468602107703.681', '1.796332967625000327046569480638e+47', 31, 0);
    T('190934153295663947381379119.622141351742192', '5569370142302865348226261393516354543777.16072367531', '1.0633829725107490254e+66', 20, 6);
    T('44489612201164556759373851710559458446.6420283', '48537517625860080274572519747.51326550511250513593840340860', '2.15941533638170435711102374800648075583192997353256334879281716216869614155459007960986426e+66', 90, 4);
    T('16579375.5204396988354674', '13605600911195397816877513861057344451.24', '2.25572366687945039378287308889810133033108105846862145101909576e+44', 95, 5);
    T('9089762926065302132084210774444871936.7818474500727460578911594', '14950042250915912372819667728641867.7867528443729868480926225139676384848675', '1.35892339795485319463654748470050966734599e+71', 42, 0);
    T('97925240534620024688563284470887.7877', '5716.0021923473212225029765224384188079', '5.5974088958202682726330410858986499189390957215233280305105559e+35', 62, 4);
    T('755641571323509129503.1512097618449049743976898', '0.81869641836258784', '6.18641048008434888674925131631216927115024028856407250045572032e+20', 75, 3);
    T('3590605062572034938581333258129800.6369029312215476648144519500040', '5822433607914715677.880293286328841', '2.090606e+52', 7, 5);
    T('85335430090403563824375.41775681304036721664446075736379512078', '293194008039633094958937541796.35', '2.501983677599133041928717733080672597905107717916753885199385899e+52', 64, 3);
    T('7383315059374852076608026226564049892014.7061078189538709967156475093981999', '1151023337622622388948124181927088.0', '8.4983679423610126317233817259474497689870414420993345906545232461778754828211578093039753e+72', 89, 4);
    T('799.151206', '236910030995896857994298722633730194676.98572867271146540217349816505454561', '1.8932693698386836e+41', 17, 4);
    T('779353.77903189', '86300122913168.2342447538830414558', '6.725832692329426314110708466335e+19', 31, 4);
    T('777645008432635604505.227247949312717852401685954', '6171487404.0508778074509', '4.79e+30', 3, 3);
    T('435674984632959.30478', '264475604617189931973349895615410981.7608316272657310023799566402666', '1.152254049773868445672170015777128930484907163050045652389447587537545911640917e+50', 79, 2);
    T('71971598845162558178.820048108560944', '258670007111512691442178297918462335043.7033987508651428564129334655360813386132', '1.861689398510513753487413736677043e+58', 34, 5);
    T('8210565980784314043.493966086992040437063082562540599', '343.9798591412', '2.8242693295397169705795194648956518151465336236813811836e+21', 56, 6);
    T('282548.6064493717665595338430235', '2731.74209542706251513817857338767', '7.71849922242003259534248012713172850173644572818827720390420245e+8', 71, 6);
    T('42547840884714541458354093.503791892039945475718202356855426', '11225973225274032980424825332175254.53946671', '4.77640922565025265827915069125069265210387818223464632734060479816e+59', 66, 0);
    T('317621922697116337799342.8', '558685676112.94067706425331127833904855662529548', '1.7745081863033061943393242998043362460712e+35', 41, 6);
    T('164529451295140458787.69607709097098246', '3804662191482.8163', '6.25978982728034386499290884799216469909263543018702098e+32', 97, 0);
    T('108.31608179750', '906.453037', '9.81834413012842940075e+4', 96, 5);
    T('5.37674662068147521330805122629214611755', '230348.64333115159605488', '1.23853e+6', 6, 0);
    T('2662569651201183518072567736346081.254093448', '750761770058281863586369496821735885.1370053093360254539243798', '1.99895550423926268532350273205e+69', 31, 5);
    T('851757628.5282', '45328798102192847221823.5786731928619487535197221203597022811', '3.860914957555735230580510310771735353518e+31', 40, 6);
    T('82661370409.2144057081960301032772426722547743276', '7994813.72015462404832740297721308061955787', '6.60862258274370780910495664602219534312e+17', 39, 3);
    T('634392047178325565181.732519779', '9409186604668477804244632447482257987588.595372761628969288', '5e+60', 1, 1);
    T('8052719243389.10464744932259988', '163984792043480386379953166.4066616252', '1.320523490511695044790052890057258262540917551964371783186290124976e+39', 67, 3);
    T('2.8502176666682671841302642060269', '73871626377479042192493064056.191714383274962510521011223751', '2.10550214566608334348203973058399877336799242202783169429758e+29', 60, 4);
    T('5755669.0613410424', '17576186554906031655503151973603549941.3320538773697416193734303463244337', '1.01162713170431049007398987012719809233735758950073852747459173619029059296979e+44', 78, 5);
    T('28590497.3486006039304015903289113011962310889', '5195.534973391535446348032', '1.4854292888131240347001928339766454339560045746348043058945385e+11', 62, 4);
    T('3323675668349153029566.03295255702188805392523978385428741', '883259983686877464729.3052', '2.93566e+42', 6, 3);
    T('33319890720.484149578893670859075134946201459', '73193472095579096987635818319.30481', '2.43879849167750149543714692611554337752292343313724929501265069197967908771779e+39', 87, 2);
    T('6994945974720.5586292996669673', '45573625879167126.804225156281238', '3.18785050897e+29', 12, 5);
    T('871953233959867488053192084433379799.825255248584015046734096', '6583706831352535453291291417318.7462399168539324401898875580', '5.740684463041515189533460979e+66', 28, 1);
    T('201855420266793772557346598021.699967674139', '8.362062', '1.6879275393069860673384308e+30', 26, 5);
    T('162495069241881699326809440849412.9863895295732055', '6770138342824809.388136275705865836039', '1.1001141e+48', 9, 6);
    T('4370747487513090789447791792355475.39751883944614035300', '6988892252462025402834079809145941.711406746675941149293994089774', '3.05466832529481033355485133985695112154880480868760896526166389265570593876318790602558723e+67', 90, 3);
    T('2.91209109242', '4139660.230002', '1.205506768143415e+7', 16, 6);
    T('474253.8923046085', '1038801149597962533546379320192311.55724615371513067110', '4.9265548852733562678186192550928022965536893641639e+38', 50, 1);
    T('190763035880218208182580624388646.48721328199744066815057040596945', '5120920035227926778916841656.0', '9.768822524199132867585540852826249399764694824282784535e+59', 55, 6);
    T('42976310058.0', '665643007496691893528941.99782750122383203293', '2.860688027811744922559493509533265499477990611214620994e+34', 82, 2);
    T('3848445568319.1998751392658832293021753374690549628', '34766367548607975791167382746119978520.604293553724194947', '1.3379647311899680918353089885534019408739079351869790493321790553136147382877849622602e+50', 86, 5);
    T('1021771.948855834585191', '4683172747837416481126148715.0', '4.785134545386371031230801182696951186045402679565e+33', 83, 2);
    T('560741557785685648531272867252113.04515740327', '56481681255780.5465610627733082524995035386816148', '3.1671625933720945297e+46', 20, 6);
    T('6430725456726269372036.9262084498846366364', '5.45345139061309711394', '4e+22', 1, 5);
    T('626315987402683581953977126516.886', '9065857013803696.0', '5.678091187252006260289010959756782806694633210656e+45', 94, 2);
    T('851431131174.201144084', '383636476860714932906427392108519.39886893825312564940517025', '3.2664003945320375797460012565882009753680076999421293741843e+44', 59, 0);
    T('3987381.33335449377349947010939', '68508164571599592660495.1032245', '2.73168176595173875499371029835446396481994401377600356728055e+29', 87, 6);
    T('18507719516975666304690169475568043.5224', '22410485.645', '4.14766982556869503535569239204834817579480435948e+41', 59, 5);
    T('474807343808141656656683.39540154671004748760488830742051595', '289305064413661438315555037100829036.8571888887', '1.373641691844939e+59', 16, 5);
    T('56802667392123411948.74618728144198566552594368123542991', '5850767524191919146441317471.86657', '3.3233920166531095147128412698166173862e+47', 38, 0);
    T('878878620637044067113725943.38755139262009365569463256', '34054032460083552156041563459.59054326093761472282187769573063', '2.9929361075647356743594302148971e+55', 32, 3);
    T('84924215225704226963.2113850823589977932240450114877775073053', '224859356717460978781905359198.233092179590216792659691', '1.9096004405387058e+49', 17, 5);
    T('1952878297466751705097.7241336230686806440631680465827823909', '6831593719709111595.919255836506117534441004031729566398996', '1.3341271112330083206030880006457e+40', 32, 6);
    T('54585646794885.33412350217', '4114805097149301641100673832421.610686912536311513796062381', '2.246092976627856130289528736560861772756309865131639255e+44', 55, 0);
    T('857776312998166723265.8', '4367808546236930886554018900365.41345796092403434627519', '3.7466027106729971985168715173326258462287549358837899659566738419877423155e+51', 74, 4);
    T('58.3425880256802726088114', '321250757443.46728554837676938867570239196712', '1.8742600594461952159162687831416241935554646647920124775893491081168e+13', 88, 3);
    T('2245750983154002376195.5', '98268601193325648150531119.107442239665624076319009', '2.206868077430796454623474893978454545627107986786539944664092268857502595e+47', 99, 5);
    T('4917512095589672153209654366710029379.20644878526999193006305', '82496076.0271451700', '4.05675451702171560578105191412984567e+44', 36, 4);
    T('22614.59225973260998594616', '787085845877074448867802460608237260625.61678393212894451970', '1.77996254779167818474189218077761715739693625855244627793308468963501663713512594e+43', 81, 4);
    T('86.83340496523', '75790372888841363259.606423185879141404188908', '6.5811361415225508113285104845643184324291863982480916e+21', 53, 3);
    T('7131329212871176.2433649730', '65854738648.225748939147556477508728103548766', '4.696e+26', 4, 6);
    T('9341089.25993554551964034240403386162370171482', '380076620724078675068968109715625817961.53708961804720041632238', '3.5503296397982870939961426107739187746095335230256564818184244348322063626358692e+45', 80, 0);
    T('2630148332473659.9450260042', '751158413514896557109139028921764.3251830', '1.9756580487297650896844311924124064228639868174610292387999237686e+48', 97, 0);
    T('7110317702516740747981123228242486712.1487805960472134165245057957397', '4.6', '3.270746143157700744071316684991543887588439074181718171601272666040262e+37', 87, 4);
    T('665911248807854.90971770', '170943741767210.85826221286436', '1.13833360556090849176697230299181425930967315991172e+29', 93, 6);
    T('385.6326808997069889730394959', '7372.64271266644745127490131335', '2.84313197460125025250146104383326707146374248e+6', 45, 6);
    T('5528387419046731855887808801321.1175952476085231554178466797443', '44085426139293599706801587091006.562121412433184540541753467263739185982', '2.43721315231784671947e+62', 21, 4);
    T('2699455788184.15', '90371272985729981848942776584327216335.7321969036', '2.43953255946898710827585697e+50', 27, 0);
    T('788594855463361794355510.85', '731253184380.78479977985099895730514796478774174', '5.8e+35', 2, 4);
    T('248607318542049750162879975643235.518567546231592715703', '4220385550060106160923986548603.341904436190329', '1.049218734814056664463698022817513382209234046593132804640995184895097987806000888259855368495036287e+63', 100, 4);
    T('34103430825.3485679783110002424', '15541982.4621263137952640949699497559397', '5.30034923785905362335e+17', 21, 5);
    T('1405601996560404476846937864663.43098261674706266824896154522371451', '4382473623.367214137011446789117756', '6.16001367487826627014131492818183856757195939655550071359947376047088377249956364240351158e+39', 90, 2);
    T('755502949854493780632.946177899429635226480028370787153523', '8657.4', '6.5406912380702944564516682405465221240097281976172527029100202e+24', 90, 2);
    T('9570997297907098609.266749130195492100', '593859966182515530227.568996893767743', '5.68383213166805709789e+39', 21, 5);
    T('78146033312136368286190.1940891279536858296418313224733369095', '7941498963.7200155082856190733635739158083972', '6.20596642567160779955108300978086064247705302137968525838752300988727547096097e+32', 78, 5);
    T('1367.78011889045879653956641', '345607.204707019470588874600406232504129823737', '4.7271466354356622248916455967386782765258035891794425241987981970587417e+8', 91, 2);
    T('3.2935560042794252688561279', '462791993041383266516957907.11493639761216', '1.52423134741388985e+27', 18, 6);
    T('40036463723190371194622445606182.591379435915163260550063700443421492', '333294.801529830551221253', '1.33439452305769954958354738345343289538884601914709054352e+37', 57, 6);
    T('5908476081873809472446619424295320483.0550313803473967516911772444503386287', '16321615038915418.66318321705', '9.643587207498361718015052500754170288532240224618680641588564669207240747641741155373712734315e+52', 94, 3);
    T('73361410833295.9939247310334474127268726849561717319', '1673637898347534944054749053228661206379.0958652772552', '1.227804374468475894937346014049930749047439844466287571892686883456101247616544127667128352389782623e+53', 100, 5);
    T('571868137934554582734022344982.286287928', '411488085126487016868269386758929535335.57654439951', '2.35316925023539615386551e+68', 24, 0);
    T('246240521562308631379.30038089416156493525224804388', '1439073692614649301182931908397356.2280100332455907', '3.544e+53', 4, 5);
    T('9756602849.9320090787837772028944188848910225684', '131106310305187153498789730.56907211458193802215567', '1.2791522007676593127947697180448101884e+36', 38, 1);
    T('58192219310445839.090175293349922419192720827247', '41129162110568905.56244', '2.39339722159310520957123556756051804804221984143636830414885661180268e+33', 80, 3);
    T('44.761067285247582190', '31491.43635797283000254', '1.4095903017283139087253121981164265587626e+6', 57, 3);
    T('92372.19983764239636206180358302', '80621853011696294297685360165903.4', '7e+36', 1, 4);
    T('813131.0043931298226305328', '708618457619612566333001289714033364309.6844035404547074149009713799205', '5.761996381757460647100899491644984585268128489941191297884451744245049405709e+44', 76, 5);
    T('5166516208.582240448158696066', '2658679905668396832838373.4776395148395004', '1.37361128260676742900223e+34', 24, 2);
    T('23045196655931683.0', '975896614531026782.253', '2.248973e+34', 7, 0);
    T('52996231530013712463519.211574402000973350101', '618666926173813.07770611728247682789899080373924384409', '3.2787015659469298336710973971642496445e+37', 38, 1);
    T('7.9557', '9235801788573047722592654054466944.7194399029998329961405384572023518839', '7.347726828935e+34', 13, 6);
    T('1.75893279570608025358553593', '83454331707703265264903242.231907', '1.4679056098441308309800922857969773058e+26', 38, 6);
    T('805125226285620.7020525217131891985015071986', '59501713217762269174165699812641705.859643991636936', '4.790633031883295528570806e+49', 25, 3);
    T('866641767823975616898.235153', '3599378933644.525574', '3.11937212212206787063e+33', 21, 4);
    T('20477615955193.36671943994442918', '25799364668461421.05020374413880954693246225', '5.2830948156873761950920024150645246508893e+29', 41, 6);
    T('68070645858509783385422592178706.04690936084411718420', '5263330387802682.0676028772', '3.5827829886444933219005432509539530937719826189799071392491572336674588238024e+47', 97, 2);
    T('271.509588411177917420947333765934253806070', '60311801499713589341.1362568670', '1.6375232401523899696570898351627e+22', 32, 3);
    T('9344252944049353436908871757.5636427004118568109285557', '50070231074.715123261962317917453484634', '4.678689041291382124922728554e+38', 28, 0);
    T('615640183804094465046572.826168928753958906758404606498751140', '6611.0', '4.06999725512886850842289295380278799242233257981285356324378654e+27', 96, 2);
    T('230863036005672.8214140134715503868541653614475260519994', '44326934338749588769.799349800335477092437', '1.02334506382678412894818107554178929693e+34', 40, 1);
    T('2167021511326878232430.64578878546209607835783907031', '98823734770582128442093.2801311579520194', '2.14153159077513450070183892879719186929954933825577636888457679705688108876327585e+44', 81, 1);
    T('441967548545794598687768248.6812074225216', '143996938931190.2895323010445222427667198715744', '6.36419740975166644822172356862798452844750402026217957458263123082825897e+40', 72, 0);
    T('358361108169240571878835559805754313790.6746283564844684881498471851943605', '7106452998007364164158716130957378.8837321691041', '2.54667637151854098344024317263160118040030009417188659015497816567734404462791032707432e+72', 87, 6);
    T('3643847404.1892237400289665427971', '529562409741788.765889353647819151', '1.9296446120938070846523783376208324865044285509275504385224872621e+24', 97, 0);
    T('653137115903155693249.0', '14874250905791142088237241605.2', '9.7149253378283277244457125390836644683239115632948e+48', 90, 0);
    T('2583653324842.344703940200776', '2256339993377248964201585907781.290493520311710160625510534498422884', '5.8296003258638833160447777951112554990998976e+42', 44, 6);
    T('41031714516.838274409679447195578306', '77943714881555.7239652447255182411646516677743847561882', '3.198164257401833441268180351536e+24', 32, 1);
    T('5385452170603093.52661995521508466017581924608143977225', '864821989627252418095483083383341034924.5959', '4.65745746122e+54', 12, 4);
    T('24492226.7878986843960039682483875582', '4546071805876454848767189322.236434', '1.113434216635982552546265392632785694791861960235469341975328043355e+35', 67, 2);
    T('34328134894.1186089490655388874336331407985530106557', '247713679653131750.5', '8.503548610251190919931167e+27', 25, 1);
    T('674630933776130430599262281.7345131472667297722836642025907286983', '9651240489403896779997912311207756.12421744173', '6.5110253834645489353837685693463684936874998519924068199088475710306e+60', 68, 0);
    T('803660234.419502390785699240868172074769', '2945359216.52', '2.36706807839810509873283398330383806674616469998388e+18', 74, 4);
    T('1352418550426340688950.7760577382979294627018896924', '9776129934588336.56', '1.323e+37', 4, 2);
    T('80871447.6209520112061111821278820', '847120098746147443376509437487319573.841369674783926416447196519', '6.85078286944047584707486424427643234246700489866983634520388662644558518309237021054758432427e+43', 93, 3);
    T('42475.460040293233342402784', '64315.099193501372881872264069548571412101066', '2.731813425781063123032581998035732958038857e+9', 43, 5);
    T('2.12566883920893407734381483501', '807141032235499525823441563203270193196.0771827586', '1.7157145410699351184986239475283482614816433010690069740319885059816e+39', 68, 6);
    T('809947907000772694742062096.0850', '55610339236958646225598135841744.040576010103021', '4.5041477872577602372709347370484363738869884387995315189435056387900050772785e+58', 82, 4);
    T('65462976734672641111859077.5929227791526677517088', '5302766.482858878576', '3.4713487889679263676898e+32', 23, 2);
    T('7612485217462.307847950633466513404984', '477991650729912137831487857.57074763984830328', '3.6387043752518627003383842717102014414804e+39', 41, 2);
    T('748476687.0505', '85699013636040518023186847015.99506615', '6.4143713809799230916378726596678758894605e+37', 41, 3);
    T('5883305031510729763598640654350590594842.7514682091047517668650810531458', '94947594333551350697925580955854.7031010832367332681', '5.5860565947242231606618524599231078851351284712480554852842036467590601e+71', 71, 1);
    T('38681923484996.1844034760312560', '7288945669147290536088987.190974909', '2.819504386602498060363002382399088281244703e+38', 44, 5);
    T('790339.1622432', '83568904067.424898708468654832084374', '6.6047777630230943412383735e+16', 26, 0);
    T('13689594903403123.0040059151685144144942035127392', '4679594890551.5253412247701874911830519729197204715', '6.406e+28', 4, 5);
    T('954219984874088.70', '957.677237', '9.138347586043990591089219e+17', 26, 5);
    T('59386715669549950653392600848.8828372618322300555243952930574905952', '1.4380109685417250675253728556665467760', '8.539874851848156521963439170826414900828833360939042212339951210461510275479885206347292e+28', 88, 2);
    T('243657308532851712.9', '99567248143245919947837274.266764621207226', '2.426028770060587795834146368004e+43', 31, 0);
    T('7731961407170836547.48515737878932574', '80979216811229184122906287474143130.3792622600427571324444318610950747546', '6.261281791673438656884909686334551e+53', 34, 1);
    T('8869494294729796725188425544795668236383.0', '730326456841821252112361191.28649894428804393600643405889425', '6.4776263422487607127001501863187556902970249353463578196611373727371535864935841945031722525995e+66', 95, 5);
    T('9.1615193901361', '44876368980962509990453240426629685401.9859735132645925125657823048', '4.111357245779902499589150746666005e+38', 34, 0);
    T('9147131949977094578656801679568732211.48499673500342456', '47172425361621483617.10', '4.31492399183197672321688461865327888483008206316132719069245885e+56', 63, 2);
    T('604094647538122062077458951859459.1947', '76853377807.9039982192860613134523566', '4.6426714178979897757775563312836166604976318e+43', 44, 1);
    T('260335974668.9326461', '8.7690255685329844838913', '2.28289281828082574401805510156585013376893e+12', 55, 6);
    T('4812562.919476426939951707621089', '789936.5448305663395666141025', '3.8016193243909117543775011847620842325498481302367538076225e+12', 90, 0);
    T('8229881375172614059624813121140797228723.7534040231661318824067790196275567194033', '4247711521427899.15602397940690015906', '3.495816193730559539936872821680423e+55', 34, 5);
    T('682975889131463.87615822312270823553822', '530.7960126345353399307639872740490237354', '3.625208786765075071613959225440703502199549e+17', 43, 4);
    T('7830483796908389450493.5825119203546259368488116968605', '574922.853396832857083626636465860017', '4.5019240879962370999870736906880503233545855012239949900139827933932234e+27', 71, 6);
    T('1705642231.47637989258323705', '1245428.7', '2.124255787012726890326080560973335e+15', 59, 6);
    T('397622872798909802392775290.3238287560882751413794190831', '895774176888753130565782413241102.0', '3.56180301593584814921302090039275927456741497290588013790797764534e+59', 66, 1);
    T('439310504023279509177756035585521.09128110335182', '311733560512267743.274574851105851480395624', '1.3694782758961584476385599164062658336221324682933901516703332210738316086010292606043568e+50', 100, 4);
    T('368531461221827865443649202776337471216.6519854618962223', '53059889731555.2152924', '1.95542386950391033737040085131332888266414318605e+52', 48, 6);
    T('557604.3572284403457757282', '14957303095876512329000863793305.5359037072532007582900265439966004', '8.3402573786471834997094994691631394136214700183239e+36', 50, 2);
    T('474025204904558077935621572039.9608248179374279637690255054747856573', '0.2', '9.480504098091161558712431440799216496358748559275380510109495713146e+28', 87, 3);
    T('6532016558938468687154638439.2086159502549291939301750237', '3.50077', '2.2867087609035023025930343608828346460173948474234938817718249e+28', 94, 2);
    T('559188016652803761723233.179852153', '248521732114233307706.0330868644', '1.38971e+44', 6, 2);
    T('28174789022213227576.28725745284946', '526.0588581264046628636253', '1.4821597340977851838882416681053251096290452532076870747338e+22', 78, 2);
    T('1280.297', '66897591634702165483243.256723588858647553673788187454051', '8.564878587713427836169989185344064495988702588999503e+25', 52, 5);
    T('1547393517580156876247684355657659565.00499061435053778769532316337259', '43475082841263999900092876191.61', '6.727306136483222178668938827078500422346541708674733494e+64', 55, 2);
    T('5445792981401212469992032850140598.88713615697541281597897680269513846', '793116695117.45642927611', '4.31914933170276950136515550352516468680381391879731666373186570344129e+45', 69, 1);
    T('409595418063092495761442314.42854277657609897', '33943266620180658274614.2446332510579287719646632266', '1.390300648171990936686903489513399288587029204676806805480952664614e+49', 67, 6);
    T('6767552937347802388721.47822', '198478530773261435448953761543.297117096186888388754420505988264', '1.34321396393506161565427269339749991187662985147633805871662064694032746184296634405161008e+51', 98, 5);
    T('7738870245598386.7670962329', '971406831784541215887266553602024.41', '7.51759142686838326021283416130987332e+48', 36, 4);
    T('32281558257781.5159346703132694508283611', '470390491353999987.735199267365941', '1.5184938050550623079e+31', 20, 5);
    T('65081.88715093639697', '24355422.3518314876282726072469152699247483237937', '1.58509684901529081733318425324571873861427090189750973936799959e+12', 63, 2);
    T('254688530138519160947.233923794790014', '1792499556777138428511966187786321.38122241705573314', '4.5652907738951645856728066666463057115101093116572597584633337979687603214820112086396e+53', 87, 5);
    T('168772710150263494225116343613578276686.021396903842365369', '286730.3068101525266001', '4.83922509625659952291791548631599863360523076152016414289950846055060756e+43', 72, 0);
    T('272647668591542982794.55174666870064385605074', '77180305354576747525399506748603.80108215622077091996858284771554', '2.10430303161087313822655892902956573053034158434946387860765638819075780253429106e+52', 81, 5);
    T('90497753596557267645.99925879378133822979252963', '6378948348256869233008577453839836840854.34849851', '5.7728049582571612949318771777e+59', 29, 1);
    T('275298119819698454455865302.042182502199812623935834461837908004', '217121812587.125952484475658353557784680432709960', '5.9773226777080712541382709675799566211830533718937e+37', 50, 0);
    T('60398815542254978376875368.034848219474918093111186600519908675', '6587106849.4212962670604055037562470873426253', '4e+35', 1, 4);
    T('539521600402840957102.473961292027154', '2.47254919660762273027278699', '1.33399369962850327247751027812782875108374704400183901833792646e+21', 97, 3);
    T('785972251273414.832194975598645176232240871355674', '3494.8209376868711578378', '2.7468322801912167373018951095968614159313020285861856046693609166e+18', 65, 5);
    T('186442853255130767079051709576251335.4056', '9258722161655286507495745.34215', '1.7262225773155238e+60', 17, 2);
    T('847432110.38959344553460', '90024065300.8028486449880595779', '7.628928364370992850280448063212918936942992351584534e+19', 99, 6);
    T('54.756528509622608715', '974008687239570104141359419398.52744053367675403912588652900342620', '5.3333334451453611217481511654003023396665936599776888905222946390014036169216979333e+31', 98, 4);
    T('661555.16716015277232888970373182526', '38578168801426755.3923367429697513997365619018681507', '2.552158691016046768676585945425624e+22', 34, 2);
    T('8737215858985127252162141791306.92966433031706308874135055360', '289462498406021498275628429405890.9823887412584945042395682935819433297653', '2.529096331654548153081452675177708381566699142053970658454131e+63', 62, 6);
    T('7698852480056395054513388796900645.46876317858351710611', '616977835409844516317940407061821404981.2596283899784503678798217362', '4.750021338284907769530266774e+72', 28, 6);
    T('1496830956549.136581173218391164908666227463736494825', '960117688262797298613711328244993804839.81183854', '1.4371338777221486045747453815116064182437049331047522935164909735237066252691863294511970784455555e+51', 99, 4);
    T('4679.4448', '343510453099344188.17596437', '1.6074382035e+21', 11, 6);
    T('8741377844587903754740863.98766', '2.0079', '1.755181257414805194914418080083e+25', 31, 0);
    T('3897727405923696685576.6570816', '9113703415591686808609150510755233.028555470', '3.5522731572412119602403575912413974862286239065121810067791676793916352e+55', 99, 5);
    T('9348284.26674549658618635639281431682885', '607530805509019033929766.7498991812606298527477809', '5.679370670703180897490179588574e+30', 31, 1);
    T('17383833787346.88489796620458621343249', '46008906456536101128190.485', '7.9981118257801451659919874815652062123741804232724571440785765e+35', 83, 1);
    T('875455599675.866629338632684', '54567539562628057.93404355', '4.7771458070637123508350710835e+28', 29, 5);
    T('5784267.742', '7582090713711256955230647602040.913417', '4.385684273223778070841377309425490884216809442e+37', 46, 2);
    T('664246189744198888535434414.8959681875556314', '7331.0', '4.8695888170147220518532696956023427829703337934e+30', 96, 0);
    T('823460418748951621082260470226400.90034313038940611679966084', '60.3115', '4.966413304537739569490275235005957790104470848066701336274475166e+34', 76, 4);
    T('996420332.74399962504647', '916.9977304461398946441792698395240805626', '9e+11', 1, 6);
    T('7919162957.864818153785530211650805', '98383300537394586167917029.3994596188339545386071421791216041776812', '7.7911338928821706440738966940566224907866031190343325828319369417745390821835389705923e+35', 87, 1);
    T('268254.0', '48523131.03255389937591060992874779185', '1.30166e+13', 6, 2);
    T('41252572339925.3937662249098729840678', '460630.276286611032431962000979729104558', '1.9002183794473242298765075316727063e+19', 35, 5);
    T('4465218923298344010175401.20434554374', '20186360796949748817163434.1245759230792921362197619', '9.01365202230678589295045290624643029227e+49', 39, 5);
    T('4928799861383832600417897269811931445740.29146384627959927110116087498', '992749075027738482192760379.47680160896270319676514186217495267', '4.8930615033856454612063873953740801433106837e+66', 44, 1);
    T('8730.39242764197817547469670', '5.3723914501945895', '4.6903085635e+4', 11, 5);
    T('9.568915238732578651820', '5222974135701736626468505.32402933330412205368014529513990', '4.9978196798622466778900302669639427570762309e+25', 44, 5);
    T('4163384292930750.6389465241167453', '75457167702300266.0454415264476220585605913276063', '3.141571868007984669730232051174184146704389131098327790090073968644282199957754e+32', 79, 2);
    T('4496602622590161390242218896637293135.349871957166747512432207676046855', '532516673245396605416929278710.9557245', '2.394515869488238405557328270628850726733242984875626177966986375392115817326627353257e+66', 85, 6);
    T('80884791693892.07560699852372692657', '20213488.5485689365844205667232510400611226', '1.634963810657871308947906790971195100918587951993776824299265723e+21', 64, 2);
    T('69923989679692.9010527567699', '203707677168376488775418128993008676214.7340870010818624322653659525224113315', '1.424405351599578e+52', 16, 2);
    T('757627.607474741188571', '97557212299749497784054706523217.13301118560846440042387037392767855281', '7.391203734656460562644162696383474849118394550616389382697637239139054912375142584e+37', 82, 4);
    T('7274136466.87063133', '117258652537902518240.70742403550947270827957645271992606439', '8.529554405e+29', 10, 5);
    T('4470.78', '838935922956825516136871.743113776773830', '3.7506979456369163810344034516782109249036874e+27', 96, 1);
    T('85536885854791759303.2771383', '1437257333019021320405910536693314.1820306923483556046899524770940232548', '1.229385164384104537589738287843940426333324130435004717777274372667233558954903734688659493957e+53', 94, 4);
    T('2400687.41211749752289032739854577', '42025528539294837181555071.06119091507244866200', '1.0089015735186975847987272017449408235299093706976043e+32', 54, 2);
    T('470249941650891115.51077195', '6252.224816699667773361904523', '2.94010835524127217050881541e+21', 27, 3);
    T('2608052704290184993499.6', '9936228676377212395.5768095819324089357185095985', '2.59142080698712741661314763609399537023599842218359240060961227059106e+40', 76, 4);
    T('1372473356.6426501690751425949', '432851648.392900965802166', '5.940773547981089801342919822291061937751449132805534e+17', 70, 2);
    T('120143.53404892763256974799698590538', '2156424270809040934.23230113775754840824494821555986', '2.59080432803879951375961741690709594638037231410884962061047e+23', 60, 0);
    T('49489722278495896011574028464.731895439066520267', '76.5942296689691156715336174748135177556', '3.79062715445261218214588e+30', 24, 4);
    T('444511015254734909975995706761.73615', '7976761524426232026247835687306.67929152', '3.5457583636676113194376345770392468478681913625031346676042e+60', 59, 5);
    T('6945757015920181.9940440834', '3263636113987020646.8555241018409058756056729841', '2.266842343613582746404945434707306782337343515368079597579774341982727394e+34', 92, 6);
    T('29111435758450115.13628691717535130659507', '465587349561484660372.328514549325783280580087737251', '1.35539162167062180748e+37', 21, 3);
    T('439313.727191818749', '2301024994896198212728726664.3976968215201622283592', '1.010871866869384550903315876922490627e+33', 37, 0);
    T('50567589593540009872946582887415356420.556', '5286729805801597134447059538079898949360.79181857791534791', '2.6733718311171064056937758299778931e+77', 35, 4);
    T('1771465349990059.7019014530072080130376913694756', '1368380720.13157933907567228833305866187369015874', '2.424039031307538127678042183932666774036e+24', 40, 4);
    T('152996880690965828744340805375.2062116136620015466383', '801345811.273143292881589202', '1.3e+38', 2, 2);
    T('440830322294599749456174087247517.4389282', '21448905630534788643221561399.362', '9.455327981975106134657974447552e+60', 31, 5);
    T('903055378250846396.630218880481262209793095787', '2903954113508207381756238496320.74804109044411038789', '2.62243138039725554858730570767e+48', 30, 4);
    T('1409252.07937269871418958', '23950950884812072058334368438530.43800093888363496472990', '3.3752927337374790670892760655710228177465237575896375862813455928603962094442e+37', 80, 5);
    T('644457629343.61224730574111708535096', '4479.2529423574005990692580629', '2.886688732462050230453633353235197546804869261047438541456255384e+15', 90, 4);
    T('7012026176025035.547417087525630', '32.7371839720441332263071', '2.296e+17', 4, 2);
    T('1430005496503764097374113.6232938048878426803608451583770154379', '5455444665162390525743387812557.4712098087166', '7.801315857054355341831329249541834771908772238244883823813469650889156262889295469243618386898864e+54', 97, 4);
    T('11141081820516044801799016659661705.967680280071610832678322035', '25222345134138749.664589024', '2.81004210844734545341336896392909566530453e+50', 42, 4);
    T('352590757916967199089909648443364878.9047159', '5181150202759.4028223405259469722061669', '1.826825676872586119485389625597725300809675851444475804160923946367e+48', 67, 2);
    T('104919583105.34634024259842299133105695', '70249623768642951923904841671.64654593530', '7.3705612391134477591911334e+39', 26, 1);
    T('493028602154791561030997.62997794', '17587407663858887439.71466131711491692299167113031531742767', '8.671095016038815486455127173632855935754454602081453768414e+42', 58, 3);
    T('670491642.208207988054825667', '318232.7745457125422334670152298378614824', '2.1337241560962921222337939296481750076268832763875e+14', 50, 4);
    T('6136594.106024171401292835397825320251791', '9242790631198444334096796008542479700621.06660872056381778033668258', '5.67192545106278044183998501995848775339433933863799093180672212543658952815814297e+46', 81, 0);
    T('712739432680764541907882755538276070326.0', '5729363601745509076628099381.1168068806503971726477480946143376102', '4.0835433631299159355927347277322212790398382515268310787653334275211215e+66', 71, 3);
    T('7771662758778342663935463457455925216796.6303625562980372', '58198.203351603580485960333', '4.522968096154e+44', 13, 3);
    T('7165773635.788004334435516469237', '247.756349553859606862580627215040197143357', '1.77537e+12', 6, 0);
    T('3760.725388605411353084074296272935', '1354346179724856500236415.8292313961395425298859258016868754373135', '5.093324063052015248307389255e+27', 28, 0);
    T('23242009869708.996372281691056', '230954867964599641260848766905.9142938768', '5.367855320690562968e+42', 19, 3);
    T('896399367673879.6461794315909991006851910378036301375', '1831151987047254596685757389824077.54249152919282', '1.6414434833039272726891202494373998159108323753636e+48', 51, 3);
    T('1.75041791894842656985523518494313899', '3674354152238662381385556212347895511317.911072', '6.431655348641109549770882444953264795e+39', 37, 5);
    T('5207317878381693657698757572415714325.5350811853241351066765615881', '227410275756922721.4461583280311114133981949862205499635', '1.18419759467673472960293720482046699646085588498328363802022858082083043430287378e+54', 81, 2);
    T('5827087368054592387621104080429292586.96098', '28175903777455417010917498.036781759797', '1.6418345298513213350808344770696286130997442141580027612250813380427150057172106e+62', 86, 5);
    T('5968350196810911073961664012560987405.16783209641928877', '458762582077801105659778889.8190331756562804', '2.73805574703352597433969377593302426885930363276032118168437230715290084129647965790583415169e+63', 93, 1);
    T('953860976235510.882181040414', '380265057388.5061842637299444134221225', '3.627199988688530793e+26', 19, 2);
    T('1247568901437380387419831060.60132069536062865861740059', '64438372901666319746428673086281.668084025858984327975944996', '8.0392e+58', 5, 0);
    T('53888850040842.06708348', '608.68159185942684052418925111562', '3.28011510263336885021840608156941839987032954696719576e+16', 73, 2);
    T('450804880299073827286529043619.651115535925388601685622', '7720569378179.0', '3.480470354370678946283171181161793380458123163811854896955784842338e+42', 98, 3);
    T('299099068019469124507225408376024833.2580046555040200472500241564771723179', '4093022226893368417.37873656888602464738658612217279', '1.224219133446778588364843e+54', 25, 3);
    T('10016289956752268990386799825.205281349386935029441', '859741202550.0842633072343892433982407122064257548', '8.61141717250852724e+39', 20, 0);
    T('444634015871.7317142221105964493', '7939315086808136398807938757204839346750.07', '3.530089550318527972042336564898611087492085929346801895723023029236526451e+51', 89, 4);
    T('481415857300497809523491493449939.963', '484712538554021881824686929564263.57984486', '2.3334830229228504111943887628081045004360733842e+65', 47, 2);
    T('534942985856870827470026312747.7551619033430075557209924211561922733604', '53221714716.8099013751', '2.8470582983032853047918175993025468460963702679887655767437521109683461773e+40', 74, 1);
    T('1575262174485019.2652721431932823757323408114518007636', '74994542926382.8530463', '1.1813606676472397315561e+29', 23, 5);
    T('2100799974099184118953226.300461092182275', '38496443456884615921.2186214009948', '8.087e+43', 4, 1);
    T('79215576201373457781.29316884111831808436432830793', '6829027284453349.939344528322922877', '5.409653312e+35', 10, 5);
    T('53441571747827.3', '14016768877391358.230306851027289568101966186079976366', '7.49078159633822989954988399664468966520745204851766269397295e+29', 60, 1);
    T('4697890676911963333024805586.6598571914034357628709218817', '599165512008688461011716.986178184189668632649495', '2.81481407279280052934069246201192758032935e+51', 42, 0);
    T('32254800498105880480583582033552045.0', '7347842083766432807398635.262992484177870555112', '2.3700318050347288768284649750428916e+59', 35, 3);
    T('1284846198826.29', '950810814919688849605553808014506257507.58782062199527850234483524504377296', '1.22164566e+51', 9, 4);
    T('429354249318617.4276400771765092413264377866883863704425', '59395687781554345147457534503.3377592683912979509288647168552832', '2.55e+43', 3, 4);
    T('9931395162735477031.1783242', '434899.14294790', '4.31915524455057880857711912143176990918e+24', 50, 3);
    T('9528253978.415954041441057784821732176463659825823', '76892222900.29155812049922437111978', '7.32648628758949366892227865140866070669537836826756637348227959993761273906270079e+20', 81, 6);
    T('828904630062966888213756183.64980781079852', '7041967603581845021602547167.1939768045305032383705788834946', '5.83711955136240670924843192131631452149229138783681263641608741884390235391819867574e+54', 84, 2);
    T('875495854743365276710365318479819.8186', '8440537107745895882766738993934065329.22737990567076809505408186', '7.3896552496390853335615844997e+69', 29, 3);
    T('249443925869863.75774602', '1173174323764066174306513.47345627599821176016088038353152', '2.92641209049431266239307366877666127825175632189892318919e+38', 57, 3);
    T('194.4350013887983478903', '1327218041636914551.720', '2.58e+20', 3, 1);
    T('3456627049577272202588580607774478164810.243', '622409864273744940090.0', '2.15143877277234541370414274e+60', 27, 6);
    T('818220412941.130942243', '8701316473617.58184944265835577552134617299815029', '7.1195948e+24', 8, 4);
    T('1759107956523594741475501804049944574.2997364833', '593268891319271675894129012301324562778.385959217404619414662886953', '1.043624e+75', 7, 5);
    T('18396610049128000077.6615416218', '5471248097298004399559365768.4785403192073740887363890039', '1.006524177280249177e+47', 19, 0);
    T('886668916088301090305.0', '767820732183843046723011394889699.24805776474', '6.808027763555738347626466104931325353430927162158611201631848457e+53', 96, 6);
    T('6880383063788359023.312169843998272173', '28113157701.4735601687', '1.9342929411882995507478960434e+29', 29, 4);
    T('60951535410696269838697134381.639569790428948415883628453290760711002', '19143684484124794942492974820699336397.4232421876', '1.16683696272532919190417987365163017180812736212193008698424930640285821322522820048477046786e+66', 93, 0);
    T('29612100616726089062766957369699.0', '6.792971933170638298180609311', '2.011541683716453e+32', 16, 0);
    T('80469267.2059748042868656179335370501930602579499', '5635909169264303.8025583789396235699568568', '4.5351748089013274437495184436645469842555544878889362049406499827564239831668182222e+23', 83, 5);
    T('8863701857178400939619483478044956880597.8265198628995988', '323036921734819698111315510.532847931348172133712', '2.86330296311811511007872406237488511480175579975324018695477078207361e+66', 70, 5);
    T('554341349237109354104029786690.89767793554126237', '98313255590627737788.9434021544629739860945522345974778', '5.449910275200136445569930120455297696310645485138658329241e+49', 58, 4);
    T('5897018152037660701077248887069041188.72896108901136606155', '4482514767.190', '3e+46', 1, 4);
    T('443435176188467085764212391102.013813812996200338879846', '8603035148862438006.011288521234944328050753194338424333', '3.8148884069913903602178956022335904047163720119891617730456463448062959928254e+48', 77, 5);
    T('96703943.232952193926', '2996388185971085700.51', '2.8976255304003647300614581966397107710226e+26', 96, 0);
    T('6851276320.5685433061502598655418138109577810653', '717390075237697809917710893550334616182.19311877369422885', '4.915e+48', 5, 6);
    T('6742.346931548', '9764374150619209555.571981168473', '6.5834798092914036331195948017297171186686204e+22', 49, 1);
    T('479108302428634247767750970042083802.7', '463007398779656741552832357058691.042886258814102957415292144', '2.2183068884121904167505161976476247959804145299502852e+68', 53, 5);
    T('54156261980728624525007505260422000.33', '1400186689657512786460515025913725.2', '7.5828877187021429300428985101571708190452435342699476118100102585929316e+67', 78, 4);
    T('6907639142849723306246721109475955407.56970', '2438326345906017.133329263162646530575070183', '1.68430785100221381278595980214674559586242235149280631537139391833481484719469642551e+52', 88, 6);
    T('5708960163162.67272624197709897783278356289362', '78357291.73998408763805205050014536545', '4.47338657036884704847821084097499994235e+20', 39, 4);
    T('602581008759920115654276385331864.287657546', '4699951992380772057486641503005086834728.15829428945896257', '2.832101812692002007838077409937660781182902436514580159e+72', 55, 6);
    T('93267797430614764812814012722844611307.9', '1933428.3155530269007520282', '1.80326600481614435099181149e+44', 27, 4);
    T('46421880477.492322516712412839782', '91.1776', '4.2326356494246039854997976929405072832e+12', 61, 5);
    T('3589.99105696559276558582990592', '64533620622314606887827218871.229612898582159833601514680515759', '2.316751209077197899554232178836e+32', 31, 5);
    T('6535594.3425571244010311', '5710871306.38468', '3.73239382010795288394158204182052435e+16', 36, 6);
    T('336882559488929265825983627.28410430667570065273087373466647', '6767874.931842047129525523180925989998070229', '2.27997902933991154242077378934649564207818015284339563e+33', 54, 5);
    T('42900710322866057681798006857214.0', '1810500.5961', '7.76717616126624208933522755347778346e+37', 36, 2);
    T('57490904360386334567438778025151.0574269477420846385682687812', '1810240415.4778528700708361295908507033472082029', '1.04072358595543261499267776452740689425515674986232762471930333e+41', 63, 4);
    T('296718438155.689015772058111268', '81007102044749272153119906.704703770118892387018869185041908', '2.40363007982365261228043258515748612492915121e+37', 45, 0);
    T('139524218326334835347679614321141646.34824750852543624008650513', '50866337929.080001213273325796000412116625273297600', '7.097086038678084641546764652790929314990364100072098021358026365771489928878e+45', 76, 1);
    T('45.45341790948235830993', '615995839.4995346811', '2.7999116323274769983142250905696817513323e+10', 67, 2);
    T('986824025131771102770154486577456913333.31479537998339', '2.54697033885167398234', '2.5134115216768398875138968473439927750134302972977055888367985072843533326e+39', 78, 3);
    T('91.668', '174144431197788298382430966099760.89203862', '1.596347171903885773612068180043288145139621816e+34', 70, 5);
    T('85738948572312635976568817054.56505797677402297925009286416108043919', '986.0', '8.45386032923002590728968536158011471650991866575405915640628e+31', 60, 4);
    T('818819608742897736934723207046430.7897480846309229944951435703', '977524666237736316137536442095246128.212542070533203', '8.0041636e+68', 8, 3);
    T('97916279713.5299503327844', '6.906063625733218174406110891', '6.76216057696728606197491530682631742988518419195e+11', 48, 4);
    T('5439279.55006357584477300505958279', '815842214887273424578018475850026418.4201851', '4.437593875514919751578621207968627589451632469755428548631815166407520574429e+42', 87, 0);
    T('4442213676.389315419301965190372325038', '89141993421716761596.5985153747128305', '3.959877823185565863092302038329057083374922029463170434650038e+29', 61, 4);
    T('6400102921022283764957182030284456759.0924158704091178033175741578584673', '180086995621187641530610.28506696492016352309', '1.1525753067132902503314274112698260453e+60', 38, 1);
    T('2506407465240814534.06839', '3003511059473849.838', '7.52802254139860532260869222528136208242082e+33', 73, 2);
    T('11377338173739777603697186013036815414.96443602703598185762287', '82773260411076664419484032741616.9239', '9.41739375439846010366416019540717970902625065313280649075169137177474122688933542878320615689593e+68', 98, 6);
    T('956569555699290698168.0299482327130', '9590359540016067373672271859773782009.16212918573450034', '9.17384596418962347890396611073514392883659747770643833590598330878623432314e+57', 75, 0);
    T('75429395247666517871483735652704.689096027442290717795111205686097083648', '9938.16', '7.49629398674549481249664802314283632986576085875919962662379e+35', 60, 3);
    T('6662006804085853998387923129012.822985419566745', '5860579724024666541130249721.054830186447008789024019284479', '3.90432219973399249630195781895e+58', 30, 0);
    T('66066184066944115930929025000231597.654121549172877975671934716267575542', '540464683381737515160657766821.621268908179521415784717100670492', '3.57064392539805433371866914462154210949916755352413097e+64', 55, 4);
    T('62994463978206082723718540.28099864679511080916782713858320626', '3231870.0652686129617908150570152439829982202569', '2.035899224e+32', 11, 3);
    T('164013.249096036353840556570021', '700.521075156960296365500711', '1.1489473759674173301744962471301958322e+8', 39, 3);
    T('823845196063424227271882628739383955773.2943453237', '106527.760332235359268275409209', '8.7762383597107905556579109438273682842689990338057659892939139234021373065e+43', 74, 3);
    T('54728263172.9138754379598659076423563973778870760', '9479613.4680316688439943805966272888', '5.188027806559359672342651576615842101120591864080801515281164938295370752264395488e+17', 99, 5);
    T('7907.06245882044956511', '9673753010130.928001810139731', '7.649096926230758091795738521397016923308728238541e+16', 63, 0);
    T('787056.0169987878070493075255209272184238318468', '8628.6', '6.79119154827574047190565491470987259689187547329848e+9', 88, 1);
    T('196.07103007978643570239643716542558', '84174378811379966666780483.90794007257126513', '1.65041571598734194485446450122156620218497287705921994292412590046854354640254e+28', 79, 1);
    T('89624154897046542638.8156865924378567423090995632', '540020256184647476.85944434365562846364408171819961322625', '4.839885908783560166712781954443030067976203864112608214018714417753944125378450493226654e+37', 89, 6);
    T('2250082182193375868498340910814.51872896379233553343', '524122507347650872354715739479967.1425578841207852297306489450922413581435', '1.1793187150694659524999823219713959940278168608076447894513050444118013365904355649874055081060072e+63', 98, 5);
    T('98820643676226348748273270814990959845.3233409697402010542050526', '4889.221755678494386963946', '4.8315604097195829265475843783376560641641835088336717867408585293104591072733145e+41', 80, 6);
    T('91538293646596495905921124501311.59422827697747014788918750121612459', '22791667282418198164964777317.995022007160561091', '2.0863103323935229728218430322440858894331497002200504e+60', 54, 0);
    T('658904744176244491720143.309169990194500901638781858451502055', '1477865371113361282410387367.327029288944535602980007980816982', '1e+51', 1, 4);
    T('5668173551660468.4478212712976458245366271597458', '41422.065191876993713771', '2.34787454375752882916620554955286853434090518e+20', 45, 1);
    T('414006375876.67181', '88292839175487.605803', '3.655379836290545566594262113688251343e+25', 37, 1);
    T('78.7079690', '85781317499510197.05421969393026125907494', '6.75167327853060610492741498905249134117134619686e+18', 96, 3);
    T('43946398333832933046853977004610789353.215471233218553542993528', '46545976161733.60', '2.0455280092406368991732e+51', 23, 1);
    T('153638472734710986748137461272229.6252', '1094809677828.7454673225698794711846605543264912011', '1.6820488683678942997533362178183833047619866161e+44', 47, 3);
    T('325263.2001191069297401295628415', '28501339832647930863616754189.34753468905988551090092799596315484', '9.270437001649237547025502215345e+33', 31, 1);
    T('4495430774.304253953891529262546086365437739', '79753.77813774710569', '3.5852758860746215228445386648890115488161182610376219714763491e+14', 99, 0);
    T('70827871468634137115233642317767.6783682', '70787247233194231015020171190323.96314078761360200544344', '5.01371004865110843816673e+63', 24, 1);
    T('7655545680651324965212381.011146874996554', '963565891885337625199.92408542375239363399', '7.37662270164573803295279237966553071738214403826543704994168199460863153278727046e+45', 98, 5);
    T('6058117814767508.20', '12733060777738999814099.56486', '7.7138382334138057974352323e+37', 26, 2);
    T('462377878728697926293283974686911328.19563849117452748909664', '503033764157866788.654760', '2.325916848002265635422048106302214682319779708878646892220170395557851971532360064e+53', 83, 2);
    T('507704607.98736792733315790633789325844568', '6836.3980839133', '3.470870809238795205110947141258040138155068224569879544e+12', 96, 0);
    T('28708803430093225212.29179278262689', '257.4', '7.389646002905996169643907462248161486e+21', 74, 5);
    T('3175407282747526279459277865370209.358090040791714', '7344310332135895265.7584401131', '2.33211765154222254182545550704045137259852102077654243352192727722611797028534e+52', 79, 6);
    T('40096245542201926944799969144344.79039545901804816218427', '96267649398381853242068792621390578139.92195331093688954871810507', '3.859971308048126397397088908290289713144624998036392019427173088934313055580276126039358435217e+69', 94, 6);
    T('669413947911970596919400210601188.926087376176951', '94091945.181522719934657482692', '6.2986460490679842834135404884054083175330113791056208510104516157272411832092e+40', 79, 6);
    T('67616645749.4469947868853381820735989880009', '34492062077101020088523.4152', '2.3322375426352745721496100115802035516517708026e+33', 47, 5);
    T('26033035532998.6073102350666161690886', '804672901499191060987.6764510352723143267325171612', '2.09480782371695292017341139525882291e+34', 36, 2);
    T('58540443959609413266.625372280225493962479872292262493340', '942.5', '5.517436843193187200379441337411252805963727963545739997295e+22', 82, 0);
    T('72800266454359711701805774.24709990934506181241245296', '1185411805954161232.7338687', '8.6298295331606687951163175800751604691371804015402449079990172150417437566352e+43', 98, 6);
    T('792072411301471427151737378527854129.2795208711509257905474', '473959.35655234', '3.75410130403306e+41', 15, 4);
    T('887.8412', '71834004127770572691.27407143830829863092', '6.3777188425604778582908001114673365826434369904e+22', 47, 1);
    T('2510.9850837003', '798612036167.84292010692112511701105', '2.0053029104809780656039801071772682400156099884e+15', 47, 2);
    T('3676386336877568480514.639889032235849514360617860030452706', '10480379760620.05763710434587358235864', '3.85299249572317817e+34', 18, 3);
    T('70675367721.0', '6834580.978641451693551786193733169', '4.8303652388443664e+17', 17, 1);
    T('53665922138713253941969557829222888006.1457996936100', '95739173244273979.587548104743676328', '5.1379310169519866e+54', 17, 2);
    T('2311265959510302645167253290773504.819', '1554327856150832735201.196115759664406713424', '3.592465063840046086824301511046244352e+54', 37, 0);
    T('4286841688272227261426.99308399161166443', '3948101947494582098.4756', '1.6924888018069e+40', 14, 0);
    T('116356814928.266784562747057425', '98097677210905047313198721890.371195823261108097576229', '1.1414333272122132759e+40', 20, 3);
    T('25650179438985080214479314475036.704957799836', '7171453546867.675850918778185020767044', '1.8394907031550188541616284825847533639087020474545271578146135906685e+44', 68, 1);
    T('221004453198225853987769124.58768775', '92371207082000869612257432.256', '2.041444811241768974189196559304936673e+52', 37, 3);
    T('60516087.07500647660414329493796958355', '7186841.171237613118546738569644989', '4.34919506112856927322443e+14', 24, 3);
    T('66818932702906294897264617610.472227143602705429881414110654119137', '1078157231493167516531.5120034933649538272700084795158267', '7.2041315494293723657282e+49', 23, 1);
    T('729244560611960666350167274560076789.42102075993773443262528631983893688379', '6904860282673966232.7843285132576974794900466177582', '5.03533180292555502852162996462433664140269306e+54', 46, 4);
    T('617274611073840180610440483419213883222.66990011173625', '837050.0', '5.16689713199357923179969206646052980951535e+44', 42, 3);
    T('795953173366575595000674.5563365', '15114578280180036536529731.70386024125435', '1.2030496546206818618748039593578801725484831931448381455498540500688775e+49', 87, 3);
    T('789918845881825803446917536219670401617.945997589313198337667656', '806289715167897.6981', '6.369034412518118066502283749309807253862752536540336743787703632e+53', 64, 1);
    T('650734846.88018975214580683984', '61398239088753194.9893162966466777284201971429195819', '3.995397371213309151909937584379196645724e+25', 42, 6);
    T('7327165246448587167793013.5', '26724300211547611081655373393.0', '1.958133637457102819349780944608904007791953610791e+53', 49, 3);
    T('308498696036391572618.0918864338039524784', '577628850.5841435386795828594', '1.78197747198207942221814681511187326923954913794e+29', 48, 5);
    T('5826771953288803.66604200072382762240497716650', '15786739298845796254355244627989714244.482486065465626764', '9.1985729780396839071185583501200744500577287771674115991979759414086e+52', 69, 1);
    T('46763770670986604608.304434455117369418607911447808', '5878013126164013213692111.1865688985484534588379', '2.7487805783298296556989496204e+44', 30, 4);
    T('2.4606642069072875075318279', '41043410574317608649092934912.3189556806', '1.009940513296234161589170915382453e+29', 34, 5);
    T('402646483882852063610.89693752649655164039557672602169830', '262.713421655356064148140066458', '1.057806355e+23', 10, 2);
    T('5105541297164961730162746.42336755542092995954695199444954904237', '3697993427.66387803832722263714165543392320977', '1.9e+34', 2, 2);
    T('73279732213.50802431570046046077981150', '3975.0', '2.912869355486943966549093303315997507125e+14', 75, 2);
    T('98.007952451934938297', '33804671562449291624.7831205', '3.313126643145807332721617277012e+21', 31, 4);
    T('16160556615215874046896047094781520.5413832', '18367163762921.286578717683801906916504965184', '2.96823589851e+47', 12, 3);
    T('66465418880816701.9541622494849133', '28380475528247619035.922906328625797', '1.886320194021745661409251466717655058083550634025815598342272688401e+36', 67, 2);
    T('802841559606829412306214638302.1559423965915708285600042743128', '6042086838564401.458974', '4.8508384207529413945737107978083496184828646223032873410194788348148522e+45', 71, 1);
    T('663.67', '438353660199905743733767957015070640.7688825', '2.90922173664871444943789780032191932159084248775e+38', 56, 3);
    T('6218213.0', '96544544914653205456.31022470', '6.003345442673804526600991712624611e+26', 80, 2);
    T('447681213.627324380195968825210409426436698678', '582842834639815643.9136812088607', '2.609277875655426055557367685971849583889748635416891222416893309811095762e+26', 73, 4);
    T('440880933.6048202353119843874', '96145994182149075230699313653370.654812416770', '4.2388935677389499058349989227903674238015832571943501436129657901936698e+40', 92, 0);
    T('606013942079359218.77151518590', '88189564056614781465199617.27492152852821741788768638874', '5.3444105365e+43', 11, 2);
    T('44.717', '59612043032574593819062583.55294036271886499878011', '2.66567172828763811180702154873683419969948615045017887e+27', 88, 4);
    T('111538570562549660056824199508932137.22882159554', '3923913576.47332417930817507', '4.3766771133081647064388320068237043498125e+44', 41, 1);
    T('8384095275.60818368553946170719271739824', '8043.71164236915619751032', '6.74392447791417864733559906370388159494979275093246004359498368e+13', 89, 2);
    T('67027.48498271237566800', '373163543692470508431557.21861765683', '2.5012213820942800459085276710782914178747e+28', 41, 3);
    T('75779740018499329.042226', '95515014594834193545984948762692.2742041046703847776426528532179341', '7.2381029738597042134872739142884630920777066072575391364732091281566203399309961993e+48', 83, 1);
    T('18055632289.0790092454955292', '97955948753479920.730348924235020468469390', '1.768656591220700783454859671234867814497768880054063114234812051188e+27', 67, 3);
    T('6.406226557557017755008142405088410768', '665925016.42883136839092927', '4.26606652558797287031556944471958691515021759306462733099437e+9', 60, 3);
    T('9093761272894660558659138178600089.317578217865621530365094312238589169727', '816807888915399614084041585002.932730978817938029022956304', '7.42785594761370489717544581663246e+63', 33, 6);
    T('9475915.826108362', '555127607814603442733646098656025849.81810804643672008920777868200065', '5.26e+42', 4, 4);
    T('893417032953109.0', '83203189805669783125723797530818056818.400055', '7.4335146968415863444759959366927879795502807442865218020995e+52', 79, 4);
    T('343072139111271285502970905693.218', '2518631.3228985455838994794467047160879', '8.640722355794550584643677441046831024442463716784056130848724265218622e+35', 97, 1);
    T('957268249847976263.7927944749579539501', '8494.87', '8.131869337586078124005496001486074272085987e+21', 82, 2);
    T('2.03177453', '145348308896877437588149104.3104', '2.95e+26', 3, 6);
    T('8487279254029715481641872423210.76479192578929476890577', '5237145802327698.20720869403', '4.44491189184246822135602200279508622640696970787940446862537468690371e+46', 69, 3);
    T('11913345.655603905719312011305', '8925627245413912271.598346724334146509611556250015690', '1.063340825e+26', 10, 1);
    T('0.51922613443259073241983999', '924591606687354.35553023009', '4.800721258690933090361421364690853285137941e+14', 43, 5);
    T('256954346692414734037301.757803381553807412580700', '215552526284719281369284670356828.05583136', '5.538715856938959789633692456272838e+55', 34, 5);
    T('97054573071701204765855760358717.8703565528107', '2666024892042630547.3774169810535863223066', '2.58749907695725802267688625343822292433733502576940322176596475715521889475494337161e+50', 84, 0);
    T('9015696180364768861616521644915993875159.867737466850991956271864481874903079685', '8063514721166300981958721392441190.805125122277417548949', '7.26981988719341039932433889359863687434005621814055e+73', 51, 0);
    T('50748320762777635457842676093303303.619576337170', '35487.83658629572554373081326665678144276187', '1.8009481142583711733343983178040947971107552720788075275463268077433591e+39', 71, 6);
    T('867877443651552918832098148.02663208677433', '13864628198073089665866342.975530062615984904116626184344617', '1.203279807772290955729814111712934213735651232185775802967600729009083627043973284526084690463e+52', 94, 4);
    T('84925551974011908618074872619886357678.953727999442', '157690031421640722645699495877.41', '1.3391912959282120169760686306804313576573098688936e+67', 50, 1);
    T('3213182.50492054886353594562102827', '837058305861538374003998809712.564750174667', '2.68962110399272882204842052059732194814023488e+36', 45, 2);
    T('739551294392612774.228790360335573', '752737950754.20071355952540086410875385', '5.5668832581871194876157167752961941393919481073088605011666445285570605e+29', 100, 5);
    T('1049335555769273081336.99112121674303811951', '8755145.3967512765936382082', '9.187085360740793698256086870291e+27', 31, 5);
    T('828897439510.85284509753412365362197660686213550238', '95259325863.2172877880388679732482', '7.89602112975507717873290597e+22', 27, 6);
    T('302010.0430225195499718723223008', '4140.94064345414932954179126', '1.250605661883287427e+9', 19, 4);
    T('209423015249112930067.5520696375237913369012', '34247908220677472526882149971785.679822779', '7.17230020554915840469689724400236629156806749181413469658e+51', 57, 3);
    T('561712461919256.394778587303666868068698720464', '107640486896233.88438639383439862480707', '6.046300289667e+28', 14, 3);
    T('987838113988422642475320290199486.204838168762992103', '188.897264234070642287172215', '1.865999172385570666562571180893815e+35', 34, 5);
    T('63643768657465841332502376487671452.8640743704578111918746446382', '67113023208084.05132395576', '4.2713257229584373548326717440556122021020866441274865629619055e+48', 63, 4);
    T('4664636264775.5', '4536.6457076961749750131959', '2.1161802088557690429e+16', 20, 0);
    T('4009335273232631143705108484388436704.2866669', '12255231976255091882072231816221.016579437', '4.913533384404798695965579428331019358519679327653276243142232632999131e+67', 70, 5);
    T('559323615073052486740135086270410621068.534440397357627884306', '34734876551502977624888.62551367535745333286049738178', '1.9428036721902848237115720399629341748796153765200568696565988840476149311760906691200871094991833e+61', 98, 2);
    T('6268605620104422748753.0', '2705454433195564663965871784375903741.80296239383595815850835634870258', '1.69594268648661422001501458275574914903515757819863439957929731191226113407511619688628827e+58', 90, 1);
    T('1.78878661522393715214472', '5.99374', '1.07215218871323210662958941e+1', 27, 6);
    T('7947752215019983062813507300482929268228.5872852', '47864235336016209866666210269650.2850', '3.80413082412060575264941334968921582287262919961845491017051477731153098601556282e+71', 81, 1);
    T('414367701036831406282072.1294741193486894785219173316453848', '538726320334576037106938190180606.2584669502329500076695', '2.232307868450698712847e+56', 22, 1);
    T('613996403079057.053969833', '8529841.71494299122995778', '5.237292e+21', 7, 1);
    T('687397020316906317.58213349771081112335895109783692', '1300115213682322563927.220832771381585244', '8.936953239539064819229045658370676898934065811128527989632522114233185e+38', 71, 5);
    T('562643147301368.0', '137299184568.136583818926360869833708561', '7.7250445327327784100640191833403094674061926548611448e+25', 78, 0);
    T('7373199618594777.9578', '612369893160916936996899054735.578953480794183183590', '4.515125462692997686867720540123378715521809510737835e+45', 52, 1);
    T('143985185987103216991775120962611.7708256630379395604122359704612444', '890860979874120489829407802594500236084.61589769976497477429831430409', '1.28270783875828254566567267654643028882843610515855271815659e+71', 60, 5);
    T('8.667134153372926349981356', '2827823229334279850841601672283.77875975453158744870139', '2.450912329066445814419711562543784702e+31', 37, 1);
    T('9689887570364695.64955685532890957195752768632061', '21961711443232690306915441720629702056.123617438428306817862', '2.12806514737716547042809392468424148742121575e+53', 45, 3);
    T('55557494448715627561042501.019860814455441594376173296521171919566', '8.901671589343077022', '4.94554569909217619145684283778502121237730758185111466076498095711662032488726812452e+26', 93, 4);
    T('5523972441507.8879024', '30633.1385262764050944269380382', '1.6921661301604441656020555907705257673647616907168e+17', 60, 6);
    T('210890149203.08153114', '6515935262533824.585589088589943261013484657932', '1.37414655971337849431923311838894230640245157283664438772970600248e+27', 96, 6);
    T('82547315849796905130385067425418748.99268294969210961', '88974607661.327899502539704065688', '7.34461504123139366431118565799883e+45', 33, 2);
    T('759.08566064668496592642569930907', '1852.218697807104', '1.40599265388704807800831562938064132329693763328e+6', 49, 1);
    T('332856975.91056639134297356277356', '55623.086592914821400854402182', '1.851453239412919711842184200035371188458631856458604363590792e+13', 94, 5);
    T('4934783610024690.522882140360', '1396276722034232964069879401382.28446', '6.890323482954e+45', 13, 2);
    T('6518101287059813975018171.78740846', '33672345747284798.052867793176988', '2.194798e+41', 7, 4);
    T('93439061.26520239392063632504075594', '3365175499296752613018.137', '3.144388396469473228862825121405231856238142977176e+29', 49, 2);
    T('152689332083599238336556.1250133431437563139029114274625287', '90079332668298369479304637655366459.56', '1.3754153139658819213419214539842626464e+58', 38, 3);
    T('4202587385148913680540343.17709234393471', '7847993197133051482239728329163.96880968865', '3.29818772090058538776568915457593704048455967327925813836845284464874151110280415e+55', 90, 6);
    T('1530168149688680897836613417.56049813607', '726947013067383929.5045609', '1.112351165907032201e+45', 19, 6);
    T('3777550842274508.644123656439172076335412', '63105817414338952441.5184466', '2.38385433725957e+35', 15, 6);
    T('24351680048374022272266899066299.58012638198433', '142497281582263253182334625287068255.30353887513976289890764267604277036377', '3.47004820885433509025308484892455874996948600317516743238688281193481100072784993280317071413265e+66', 96, 3);
    T('143.1204845420237904611322322994280207227014', '227029232133326.9', '3.24925337081253434046057e+16', 24, 3);
    T('404257687039480683044151600646158216.4420265893201072263185', '37961.0587778729835668208284846', '1.534604981911270764614161025233996042280805414741824152004445203347292334065952359194e+40', 85, 1);
    T('25658006423.15887456574177255', '397308918707229800926710756634991385.55352914814302665982888746', '1.0194154788168409370639500292748271158292724014591627767465948925705253699820711315e+46', 83, 5);
    T('705409178336473544875245659.5470751486377028023', '78147108316537348473.2197542946432445995013091922467379334', '5.51256874669400093498086146996979538549471888813245619759743e+46', 60, 6);
    T('65657181143681616767167145994.12790126980', '749500060810619351366768.876', '4.92100612598432219791504224164558727272280331667115448487635187448e+52', 91, 1);
    T('51664994.4965674236465761509615266826501804344', '23044866796062481529526074662714.6580830055040015821723822044', '1.1906129161926974650473497428137349294806735287690063687250164511466176208182682754126677046e+39', 92, 4);
    T('2357252.49895966434953128132500465323808', '5960993275683352535784209529803273.44622736', '1.40515662953863381564609688e+40', 27, 2);
    T('3723360895810133898674057.71577724349', '8935990645988.40094401', '3.32719181365983496390574132515494570640986027863916269949e+37', 88, 2);
    T('3196357658859884298742086751911462307.175106861912879', '6911583638839202256651838.60', '2.2091893298854352614140513e+61', 26, 5);
    T('877121731782270541340980646.60209928011545665248767079531', '9146349701889.3289107392511289960125172778832715365', '8.02246209000742207720683872443876284555e+39', 39, 5);
    T('713207202216247759673106758611816825.92446967039682719627061828212713', '94396727362620020.7346189', '6.73244258206641451899591652363292294992729223432753925433492502976389551952212278787219e+52', 89, 3);
    T('5693011418.628830757061856848177308821', '7723103043328636783018.46308286456975358059274055959552523984', '4.39677138129170026657019761160119118347745365374816782344942857128258349564149017e+31', 82, 1);
    T('63871005.16872856534844764340650602', '80022965755561317966034928.40996694054494563173993926029265221070', '5.11114725939044592447902252849030542500032783763288571007472566465174514261518228046446552e+33', 90, 2);
    T('4331.3850330875119910742831', '365840608919639065992321236074353.81637395166771494060909075219347998', '1.584596537970146390303625243063322658172026508726650807656180551827425686e+36', 74, 1);
    T('27691993478686236496.678449068151124746983', '11319421661745003835514716745069321107.2199711070379768918', '3.1346e+56', 5, 0);
    T('381.85241774710861721300', '29789139794412100485302.23977344177508254483280134359478136', '1.137505505310286670412750873085123681154797389974216148202100077271647266754968e+25', 82, 1);
    T('32776219534205892794869574975157.529431467460691', '89649686910248067761805264310838.75188050065019324', '2.938377819343115e+63', 16, 4);
    T('9163670004168665259604517287947.9667760388125', '2521058057.396157735463593337253513372', '2.3102144099328895897049080288317591523250853970184e+40', 50, 0);
    T('9701799588454632606360131496994629966405.08223387624681041377713225284820355', '987574685037655263723266099954.76339838914288', '9.581251e+69', 7, 3);
    T('76907.155519384151331926386279228940', '83169646051054103727.2942772746904166024858448285829377', '6.39634090334055189954453709429319942381657034914854078903595207657925e+24', 69, 0);
    T('3.87651', '6077746160105847418360.40313469', '2.35604437671119185757482863556571319e+22', 71, 3);
    T('40871.5812057', '169.699323034947305689821188849842474', '6.9358796619751653835219483759888520597439909018e+6', 54, 5);
    T('70420000864.7', '4244381274.0', '2.988893329851964876278e+20', 95, 6);
    T('9425104497252088.07052857696213257234820626422973', '6501441833907459340597266676.6216367127398454138420895538987190394968', '6.127677e+43', 7, 4);
    T('356000791105569072488504295.971506102', '5.18685539252704054', '1.846524623089813434050143686546e+27', 31, 3);
    T('441150608291596.264887711718974322', '553430523755825.83507785566106515101730033', '2.4414621220201928428875257553465965812427904726611299686456373574503212626e+29', 85, 6);
    T('5121258010182317672075388625262242207304.1080171855782517215', '2242585.80', '1.148486049177112102248532306049462565026084892100653375e+46', 55, 4);
    T('69353842134493584.86', '83425278055333208702527725777412091618.639133419572325146490197075726102', '5.785863564275811327312185574395468554355805329499475794064420479535762e+54', 70, 2);
    T('24.37646675955309576207', '631044069324967810168759.3078994689346731420059', '1.5382624779713197193746830430726759624522051178279264402214588936213e+25', 81, 5);
    T('39248.331902556612375254839248', '2133844.14520', '8.37498232391368042284834040873119708096e+10', 67, 3);
    T('696474099979883.498871389207932', '462263342316747544598.21743234687291573132011056824780692', '3.219544452937495400038230989010836923699528156687223333508728075284e+35', 68, 1);
    T('8289686892077699146414567751566174780.75653004777601', '4447031474135.8309194925824299358', '3.686449851980076515046503885048978123237076114853771059344e+49', 58, 4);
    T('2977705416.36610315699', '2904097889.027', '8.64754801381306427561277937934873e+18', 79, 3);
    T('19112417704847235900407.08702103794778410', '80938250.3129474621218632569794901181', '1.546925648280534406861032954093362751604447574129e+30', 49, 1);
    T('5350314057969931195479403775.83296086218688142822470574796', '234.01933', '1.252076911135705e+30', 16, 2);
    T('86830783728023974074.0727133725207059649', '7466723.151', '6.48341423e+26', 9, 4);
    T('398298164349.1700480870374759310112601951687206694042', '395873994109995526.44839952491560120006', '1.57675885167585373792405699996191173596258503e+29', 45, 3);
    T('68470146027912586692.43097884939590947546', '28279957019.66197067912613874753344', '1.93633278679934675501702264091e+30', 30, 4);
    T('326729539818163373092334968864833676375.04090945', '60078879157771289299284360738320960.11735734185081123419300', '1.962954454000966005e+73', 19, 2);
    T('32212758604484146008269195813973.740322505901331114577375937503451', '12037613286.5562498919710583', '3.87764730973967518659339222331934453431233183237570590793363579107864610915327e+41', 78, 1);
    T('12968107720362548781587.45190940209699835058672', '51800716879874552219853458784870577.90294435355958251941185165', '6.7175727649021578006397984851649041813700188094e+56', 49, 1);
    T('7209.542923551', '6935108595423019292.0397536407713271570', '4.99989630981897437642957597254003102e+22', 36, 1);
    T('53870696978825120.966059723148474085673165', '274950761290776453686.75753603437196', '1.48117891455926981206718679545612712191726140261548273821711220552346004534e+37', 89, 6);
    T('450291876550753959132.4759212372389330779445133057085643', '8641722775642687027176189521217473.75518135229632192339125535581539', '3.891297565275535679558528197435879452634976087916814571145091831803e+54', 67, 4);
    T('723750710169823270772300372287701265.0869199933398662745947529256', '3743624000396400967531828.651007056154792847312977', '2.70945052889568995398484825151343205324469223696980085242113713725118089e+60', 72, 6);
    T('95648098928855567987144.37', '27764.378320521114', '2.6556100042993763176587688255941112e+27', 35, 0);
    T('6202.9643874343475785461913751486872903917996', '16290944.480002', '1.010521484471225719758933115031592087754047421054669915992e+11', 75, 5);
    T('46810975311173015950948601998668469.95620081055535240956022122977842318', '4875319262650183195309.20', '2.28218449638003957983651510771068511789845067320394963505985555142602670561477889481948547256e+56', 99, 6);
    T('331362312236388054100951901933913413477.5209406581110940710', '672055.13193432965287889476501398558231423', '2.22693742468090310834622711551486093991422943956813720257613238493769574848757e+44', 78, 2);
    T('955.77995898096383997780330255951307259570', '623930923273.2948177200938659', '5.9634067225310461777719641882110196748792421394511667975392496071663e+14', 76, 0);
    T('260466201756405672977185631.12288290675997503264774', '198663733647643460113195.691540005', '5.1745188129947939804085220354020708289975546067755485348311623808628e+49', 68, 1);
    T('94931934413138646225363.644794236841495856147', '200038717792.0', '1.8990062437518494789244444173570873443999002927161467424e+34', 74, 4);
    T('7.3792743553125080228415', '3838883841126008926877177581887975255018.9152028080722334', '2.8e+40', 2, 3);
    T('571906386340188386775567328413125211975.01512459147834404150543', '89137212069177387753441655213642334.584', '5.09781408429222661984765614093178155098e+73', 39, 0);
    T('2804563500277.7553524784560475217186', '530327479958.87', '1.487337093486929599369987272325514373172559713982e+24', 97, 2);
    T('23786055.110967', '7607963925.8433881883662', '1.809634492223596857938920504394321154e+17', 84, 4);
    T('366476126884.69653', '72895.4309', '2.6714435183823028170084777e+16', 47, 1);
    T('645023359133393292363333782508.058080349208251736', '315004314058074370154536176977.49147232445806', '2.03185140795249513869748402309856561900279605547604189993786633197996e+59', 69, 2);
    T('897783899337939377482660.8710009662983797911629303798076', '402623773070593431947.84780088516055593', '3.6146914095347100080912642574595070251620514332662637801796356203565613345483947838126243907e+44', 92, 0);
    T('805862424668099114.2390577835023480691714575857371', '417149802294941.0315983994982912352117', '3.36165351127219356079194991231566687148899143796710549564579771555708293555515219e+32', 81, 5);
    T('728893856575799163987132539100164229.7469417', '13.21325473034445195611269', '9.631060198319188821779081e+36', 25, 3);
    T('950425727.354888', '30007148134953505083269245607986858.99697516439996490796366349', '2.85195655920090559672817478057e+43', 31, 3);
    T('45.316427962818758601786470515788980411', '51428781424134519229474.853814173744940874', '2.3305e+24', 5, 1);
    T('842631232712509016154022040065575075744.169035733893712', '34459807563900579465013105939.06702139102111', '2.903691012660538758432962321128795780255056778751126609493002717456493e+67', 71, 5);
    T('4009804185144627.096677941011544733871903919063862', '7332044582404856418724465938788514.0929019849793688', '2.9400063052193983e+49', 18, 5);
    T('5489546096.336063157768508938878787319393', '89948369978731158711617042194967.8', '4.9377572328853556859e+41', 21, 1);
    T('391156593617132066849382520.19828', '3015796667919116597603421102626.27218134156596', '1.1796487516651388787587114846242004676372167030907095523926068e+57', 62, 3);
    T('3520692431557112704789362096396414.2088623976853869691', '22942326886953412257595358825260.64067134035764956581', '8.07728766332061329710909231e+64', 27, 6);
    T('272795298119326412558.6857102908922301963627881', '252896861971626194.3969099181493795634937', '6.898907485499191053153423394426908341817666872077742691672818892046394732378390878497e+37', 86, 4);
    T('731254784463741234877416347740807200318.0773225858499167988675005', '5473427202879971.1017849', '4.0024698295199713360371597225637955344705885358151055381850195488343400306635e+54', 77, 4);
    T('418536270825.1403118321334643696142462734', '6661377362670856069397775314503286.891711611137867738835871790652899536', '2.7880280399312683312583510164420242300339093472532653092178442431451963053345976253145179882413613e+45', 98, 0);
    T('816605506651011872392972.13193879536519864475215456328475125160', '9881008706923100.4531695748953965923473262856965983', '8.0688861213399981280746380545875750663164232383218952870635760512e+39', 65, 4);
    T('141851799056335718339259736220.9414518', '560421430849368513264654115337160160523.236360999630583186954857163049337198', '7.949678819570876550016150121720581605909542856858155755737768851242267190250137048e+67', 82, 6);
    T('812511.95759610397795079244290550976743067', '65466459584.658301', '5.31922812340169403025746534490351673363743370624506575e+16', 54, 0);
    T('686135025316804259274761297642.23572990568699517701494', '756364647.6527', '5.189682766659210477587294449290682591119895483089955094e+38', 55, 6);
    T('35880102881668442128840.791586934135381179305423805147', '36010209993572741158848393.94158626355676264737468568975483', '1.292050039359e+48', 13, 3);
    T('121299755788581401220.79264093078205813463686849561562', '4627.12079805908155953645929896061105', '5.61268622808832471248093264433132372729516674471947296e+23', 54, 0);
    T('7585049593312349311336381121695900819483.5825669164695232157739650634356515264', '9069.61925087982', '6.8794e+43', 5, 0);
    T('3438.29056430293', '39696386721804255715328575697250.85475332430682601905151653364', '2e+35', 1, 2);
    T('8549.7573904657152574408102479', '855394566951662167280968695806983182.357703844508295239262267682103670242800', '7.31342e+39', 6, 2);
    T('374467123707870806.970286573193585192', '770980535659848352910042396949053280708.41', '2.8870686362329693326643291155008908369184218606581331715295284474309744586472e+56', 87, 0);
    T('499127465840917416514608693110535656.2400046374603456497', '830858551670210193329048872462902.77217', '4.14704323367406956573070748e+68', 28, 4);
    T('841288859588301135329585590552.2356303146174160652681217875', '511796465505863589311256481536239.09807189712886617442704969231', '4.30568664806751e+62', 15, 3);
    T('238966570288731346746625026541388069249.4890089593166120', '300474.0', '7.180324e+43', 7, 5);
    T('961870370213328689301322157964.16612432019176134460319902', '970.088324122350777299', '9.3309921546319313812191400478971479885467825382e+32', 47, 1);
    T('6258082823892822830926847.0', '68374409828426489846619944926112743.303194221911096978786', '4.2789271974108442737e+59', 20, 4);
    T('94.6387340451990934910561948432', '6646408364550369747355.52641735117641296726469', '6.290076736e+23', 10, 4);
    T('98857882446648.94560694309527073726177338', '923294665.0', '9.12749554561881186067657468220584444120701930177e+22', 96, 1);
    T('7.54882', '31063031394776.46225934005225311675589237267495612247', '2.34489232653516453832551373e+14', 27, 3);
    T('2401936865884521962438510988.9859253145396188974520027260662963', '528.6166782128156399127550', '1.2697038873207772625455381839305469866831631965289735924504428365910349516784779793065e+30', 88, 1);
    T('710.763345742974414485307049709', '72808273804701806340735525.49717836516923116', '5.1749452287200417202017635428021646362287811938999e+28', 50, 6);
    T('654413.7865302221102203801259264609869173079', '43607505048661.2851889567517928040032', '2.8537352500030209220477980093154310201383e+19', 41, 0);
    T('3.752599540167176314900907340501658', '6879794993221450166081.054711240', '2.58171155280072557856091643471539342782335650410735862459312e+22', 60, 5);
    T('756455671906452255910731939775723877.843377703902656653033', '956079923061690597290491.25202064515231567335620234194425865', '7.23232080595900338257997810903768239733787072133347816262561572743e+59', 66, 2);
    T('37169428388652301517366.70099', '949659052779983343553.5284', '3.5298284155940967249155082425575800090287171782273116e+43', 64, 5);
    T('1808252043111822596529414473863865587857.1403893356501965560765973406868755', '48776.582', '8.82004e+43', 6, 0);
    T('2.414558232907487', '39975404670512772475237147608.18601862206979636990', '9.65229424609950224994564952647801952227338291847920153151314413e+28', 68, 0);
    T('513034266433149419298722233573148990975.11536743', '984.0977376571226015822625187179', '5.048758609e+41', 10, 5);
    T('51612502207893843363187973420517.673180373715658234180727', '8059777441224143786595562.83553756031625359505707100360', '4.15985e+56', 6, 4);
    T('943730081644.92332091514400194296937', '1.0644988463', '1.004599583129625681366135450266655852761237831e+12', 56, 5);
    T('8579159903616081294978121333308828484332.5752835118665', '18256230375391500558770470767451726259.16509573479368527', '1.5662311962773672157189582760537559097003550490648e+77', 50, 2);
    T('9.5516601955456116312089952', '63020052575796298.42089518254536901308436704', '6.0194612770942519773996272938313639144073e+17', 41, 3);
    T('875100659556849975862971630496183410366.404354', '806606836.9725861764045150902438389103730016266925', '7.05862175037774725604973911335629608207848169819412289346663195830584051278115231494214619145e+47', 94, 0);
    T('0.89314920279856643477786702524871170458', '660056899593634520391930990370535871935.78732344478981481743354673912521634', '5.89529293673748081226727156318986481618405757466312586678884365094255312124231336472e+38', 84, 5);
    T('36.628819555230', '26083222699765.60186663939985642607270965410979', '9.553976576885933127900518400238505864462082906852893887017e+14', 68, 6);
    T('1633063977982986.418', '5506729800617355559245.391568639824394272766987009690982545', '8.9928420738736363264071e+36', 23, 0);
    T('4606205432552.5219412997194554519458627567467', '1018457342872.17636302067831032192066420984', '4.6912237455608252732225412405266028520058303e+24', 44, 5);
    T('677989916.66', '39736830417195116.1221383513644416474781619714', '2.69411703428866698106076032225675899e+25', 36, 2);
    T('8.5125857165291757218790850', '66989158907663254198255471981.995204737512246204676695', '5.7025095727967741714e+29', 20, 1);
    T('24865938540733019419840.02', '7568616402420.4748518281018720264193157529838895', '1.8820075030097137770746710361462028156892081159194041388181647172e+35', 65, 1);
    T('9765313243847222955.51406789357116', '6067643932848268724345469227107482202685.80754525', '5.9252443656392448509268239881294858e+58', 35, 4);
    T('94954709770846102770039788861312478151.18436363455379820974975569029', '67245120111432906653483379337841185408.71', '6.385e+75', 4, 6);
    T('1035696664468925.7521771043505924319675201895903', '94145117273043976970.954859484759961', '9.75057839357274940051746189492211432117598598572211086860989435732446402064339783e+34', 97, 1);
    T('7511384.2128368366238197024918559091', '7812816171421255173340386491397340351.376317534940', '5.8685064047809952417668e+43', 23, 6);
    T('31979836704970596806534.5984250331065667750263', '115850633608057690059304853237081.5157', '3.704884344953063521723e+54', 22, 4);
    T('1779819437945943.52153887', '843731769.1052', '1.5016903e+24', 8, 2);
    T('362653330271079676954.5821787917', '9481191448247641414205026305324202.1579923225423672018116189706910454837676', '3.438385653644688138442879885e+54', 28, 4);
    T('833607768787808744.44093941966', '8595886794.30712552015461235888291', '7.1655980113549527941314736642217911453630529771631339812920106e+27', 81, 1);
    T('3168378707942.8771099', '568396844489407114617.5732448762377164442105749303096452906', '1.8e+33', 3, 4);
    T('8647379931418082199965538436000456799.26779', '6053766497353.699821448227936203464', '5.234921891870752045988099634894322024702012400008182141260095476518766162456e+49', 86, 5);
    T('76904541096592607032678596349183.8107880521289202738', '546.034789599216803597', '4.1992554916902266098042920676565e+34', 32, 3);
    T('31525.756686', '8937339920500979714999340983946422082461.2520', '2.81756403754e+44', 12, 0);
    T('380850724638499904341229341148913604.0', '644638066837692424383766509.2742196', '2.455108748846968943264543482811365422665533632909148923878070649234384e+62', 70, 3);
    T('62209672494646230833725055466770.9379', '2046228.7290266249125193424881394296242', '1.27295219081882543311560733050013425685795509336888522185e+38', 58, 2);
    T('758450794077637318.5330795708112562686551825571', '419679427.59', '3.18306195113683791861585332757990902758006274650680490389e+26', 93, 6);
    T('7855703183379314.87253825598896159419702', '3786203460.61331613459677583655572', '2.97432905784618059738035920999605989973591213373546624876850963358879544e+25', 88, 5);
    T('4546612464804720716578295998348.76751640', '64467.282452986968922', '2.931077499728372046617689252445636694159285390684899253208e+35', 91, 6);
    T('6821672944650536819292295433579587.163977749401113', '58442029.0578975598373762046438250787911402465', '3.98672408454740285162562878795406569334557996889972398069622007063029579765623093525766194e+41', 90, 1);
    T('549536614757063128301835779114275618.0', '52079609945.501006073988555008668737499986666272984', '2.861965254731889983757349482360520686980746008e+46', 46, 4);
    T('432239739.934431648385207076064357043895', '995068112832693943243227084033653.02863000174734123', '4.3010798230785e+41', 14, 5);
    T('3152775326086356491445633562920676386.83283829276160318238826410935', '261203772513209980245220435.59348974918230906237551973745270', '8.235168090603220761206826516036668048504539867703992e+62', 52, 6);
    T('58560.51645507847', '52200134.243374813459507786142010351915814576', '3.05686682031645588286213880818198575612496380489168204977872e+12', 73, 4);
    T('9574061184502574.736308099612057', '3867344056183818322643023008295814833466.4339853605061', '3.70261886154262395909738337365193301344658885837263883665469209894546619466e+55', 75, 3);
    T('77911.315827723430169966823381263419', '627302431040843427236638315553183420.171640165482833279', '4.8873957824321850100199300532756e+40', 32, 0);
    T('8836050644452062960046536305443789356066.49000379639915757629', '194906948.418703534224186908157411126586156144783', '1.72220766718327035726249945852923836024411366829946e+48', 51, 2);
    T('31366235507.79743668697106', '5695768471104966314363363.8751093585226084948124223520884922637', '1.7865481526256571267297523210683501847744064e+35', 44, 1);
    T('667060174.9788042193294073', '838037779590378763.46', '5.590216278924066213000430297955128121886973e+26', 43, 0);
    T('304188770913060258941416407057.08292669749327071283', '3515409035557.24973057675777699957711608059542', '1.069347953782826344417872695135e+42', 31, 2);
    T('40488003834048781831.04543138349671163122431410809872946', '6828451201722481259.18100849201671792283', '2.76470358435954833e+38', 19, 5);
    T('379314308801931268061538872136005268.323796375824023054', '782725747.84743081815010408574322573', '2.96899e+44', 6, 6);
    T('55046001271.1999253417200010593611443927146489474', '83609349923640698261726892.612182802351626794503776223093', '4.602360382180925257316817032464066824373733968852472184358541104e+36', 64, 5);
    T('22913043194685016471691.0', '98980200128326.4850', '2.267937600958912161198368364179208036135e+36', 52, 0);
    T('611100658563112925007586207290.867354793085774', '6045100459848926015542292621079.239855793392994129919710612826689117978', '3.69416487209385547060144641947321552677332755880545e+60', 52, 6);
    T('69.86521798679416300969290149162693', '61830387543654312470025.5160786971665039070104485368134785287878', '4.319793503945371038409437767224216380393640599226927746900491165124155708700533168227136662735454e+24', 99, 0);
    T('3807741079750957346399358978.069558023986', '5008913308623886512323249577739713453.8122404936601410686372584410169361660603', '1.9072644970158457880195904371599418700779325450494077909058986806212494e+64', 71, 2);
    T('6638267236999840816421503486932847938782.699704237682018672533941001601754323721', '265067157421489537515938873130879.05', '1.75958662671575320229167913553341003622622629741412223506610464705497287e+72', 72, 5);
    T('8192257159190280954288605577.30988319403344073983977500558923943576', '62844213698447.746672121647715770640566', '5.14835959584792476816164690349345841310290665860875240346381699423804983087074324e+41', 81, 5);
    T('743906692153040182.4336677223856050462306946425682', '82.6744999', '6.15021137460159e+19', 15, 5);
    T('31910.1281606112125070398', '6345395543869109295791385600287926357.0', '2.024823850346345650122778766443499108442400936573848064680086e+41', 84, 6);
    T('2599259300747429763521211.755975431515989480231993738103947', '6990558472747096911209989833.18', '1.817027412770663966063040618691172341917778128215687487553514805e+52', 64, 1);
    T('24180076289405201254497258.2917558402941169818', '67859880305454379811403940016.061285471415509', '1.6408570827757924352787326076823609627655845941608446155093473115191386240034796807907362e+54', 94, 3);
    T('11485900368049354707539916974.8245848721', '104560842649671815257848684952322919782.7763221311461109118099345757557717', '1.2009754e+66', 8, 4);
    T('44982598067528149383920281.7324814703', '3726070804085123894596.8849383951840966471478462062353244', '1.6760834535131255180417025513341860696290285591861518178166206343046933998639568384430947e+47', 89, 6);
    T('4745559792411137469542445294879315.08030099712711340999', '37249098600401151001835786815.8056', '1.7676782462162167740613839798163563019358979e+62', 44, 4);
    T('2206061161.0', '75427355479540668154429807508500.9343963146969', '1.663973594003551981354771484452100887039188343645771009e+41', 75, 0);
    T('30656873300885.524218903678482588564287', '51868413.0', '1.59012336565900363590759840275411696152e+21', 39, 5);
    T('5272384.8896', '629474453959711740352557.4250432318783789250173', '3.31883159944639506700654234451422038231406920430393859008e+30', 85, 3);
    T('2767150633397738959614789.30646100220477866442344611633624965010', '4806455821302206112723065952893.85593211919248916979658374003038045146', '1.330018727031464926691728842984272057486831e+55', 43, 0);
    T('882748901406927610159482680937405264.273269459457516829897', '15414205624686060898940615530819017039.501134437', '1.360687308125210458591925502355973977938257595776959581963374198474861763182993358543944463695145e+73', 97, 3);
    T('793225318949556891549113.1334890454782913750810', '93636691650990573.9052041364037908161015195067', '7.4274994600238e+40', 14, 5);
    T('2199155236450531143899992116994634.91902432618794498326636373704004356', '8832103254.701', '1.94231661214474833468192849589565582428700169079423549375761439287e+43', 66, 5);
    T('63773626073006493.302476120538429191572499', '369716.191', '2.3578142117970248622058482153924909831393630631309e+22', 69, 5);
    T('9172294917944132.85', '55088464507058276901769690132148.26945', '5.052876430354363728460534131911256e+47', 34, 3);
    T('9006021225378669349280504874335929983471.82873953', '571067013827274.685698118565030541441637307470', '5.14304164764204987782399492386009964017e+54', 39, 0);
    T('9636494424586597060723615233579908129986.89863356169301722', '336148243529298213167407618053476427.7425616920575656696', '3.2392906746046598834425792353736308944794801055197492779959312961511115773155100123710134853e+75', 92, 0);
    T('8349835929797996094979441314386418764.41424356312667491126964369350', '92907.98435914300466657001786151050685573', '7.757664259670825087249481295632706839108e+41', 40, 5);
    T('10654169638568.118686548476006591344684552697', '755954727172.2110286712537979712273', '8.05406990237021634568921e+24', 24, 4);
    T('89.312160728231437857545438143464', '62.5926881318134727122392900400485000327627', '5.5902882228405892426305942481107073178711922203764541091439219303e+3', 65, 2);
    T('282296192164145916874570750689773.02326202147498015990972401507052176453', '5705360069555631531.72982122', '1.610601422560921473279229592967793108077541150774588511832321731511e+51', 67, 0);
    T('389394384590.941131715225100285352436175567', '4.26196455638538451170293554555', '1.6595e+12', 5, 1);
    T('77690734450230651707718575.0', '92284430.325725', '7.169645170326713540176714321783882841875e+33', 81, 6);
    T('8941448635041322953791741266773508247660.7746', '32994094482822943805931453.581355429430487602651365', '2.95015001077861655281941538504665569497953668024798482227550888771689056949938566754069647329e+65', 98, 3);
    T('6272056810934097099278.69901540725181559075661639090326', '67403578330225100305184951880099214.6328712', '4.2275907254741825620133619687671517065903592884026800202446363527111339723487458459929e+56', 86, 0);
    T('86877.24114111433731469733', '728027290275191470.1', '6.3249002454549854283220430367954544579244833e+22', 51, 3);
    T('388007.7659', '33447758623986847936865462398.58322765886715716652058821672046468', '1.297799009805559501896619231414473348912813297706582771e+34', 55, 3);
    T('9.4235114662734658', '8527098718523037879.5078584985918540423305591343413', '8.035521254804762391543450837073580155270469360382543512546575107754e+19', 91, 6);
    T('2651024837.993325329500474064392683238', '3.431', '9.095666219155099205516126514931296189578e+9', 62, 4);
    T('982286380257322.9994763678350992834269071726484456', '574.7179', '5.6453756566009013388075922181580646261689375945209349624e+17', 92, 4);
    T('471.7512092827', '8910989238.69841', '4.203769949261101161657730507e+12', 80, 0);
    T('6926001560504996271220.3066084911674778195911920', '99392357152.233322494', '6.8839162073863791882710189093556177615e+32', 38, 4);
    T('233564368374274671.0380295984342840', '687356382199205938.107936999650643982121219840130170881', '1.60541959256384068869525367250091402342575579026921280373629960581361654326826968884204e+35', 89, 0);
    T('4111709127087605191782317720201099.79', '572902694608122996167976972.8029195706109241383056375581', '3e+60', 1, 0);
    T('1835186516626643020545846.72257032615865826861047150145950245', '52256.7684237328568302168', '9.5900916813715452449985155570195e+28', 32, 2);
    T('2241931599682798690559175605438.5729689737503820594681', '81447167740181025897906765171658.3685', '1.825989790613772834235036589725331027105791e+62', 43, 1);
    T('44638401551557357993.5179893632994885092519627486', '708798458606702166056313853409857793272.4371786113979', '3.163963021441087775498089502529662709e+58', 37, 6);
    T('731274613575906751111420559077230.1717', '69860463522.668017577503438715711375139213138139', '5.10871834667727838599436694107621e+43', 33, 0);
    T('7689135137161164301108793296524514950203.328570831487839638585812889244915108843', '60646862075330.5754084275204339', '4.66321918141991177404180206864755786213882239114e+53', 48, 3);
    T('827610276805483073587024470071680432531.767908546450', '5094612776144828.67', '4.216353e+54', 7, 3);
    T('8665267.0752543154194353046', '23340449.328683301410632892585927521513547474', '2.022512270894811009766908e+14', 25, 5);
    T('1763187327541744412.0668124159188923846519985958606023', '1857645838896.89747065186', '3.27537760220366253269327423625918e+30', 33, 2);
    T('7503041298.9925915782488', '20853475241228993759562.0', '1.564644859624606363546417306066495718162150256e+32', 69, 4);
    T('169548605453234738245207105.2721339690644', '7858420281132365853310.06692525198661351', '1.3323841997314095099182111496265449e+48', 35, 0);
    T('95170437820499834509795774107.837391272425124825', '401.4988213895', '3.82108186160533787174628017966444934705970413286249174443375e+31', 66, 1);
    T('712.249318657077', '77551303943100398303533132479386.8538', '5.52358633944411476382404110149e+34', 30, 5);
    T('13878782898475255490948371938.617454895698545599797417795844693719', '3141883803546832.6819241736', '4.360552320166217069751e+43', 22, 1);
    T('46425813814495354222.1359222374096238117138501181085905', '24800389299431261.936440713668991477276411465401099716', '1.15137825614239864030234949684709534230307293361449364063435208932377294e+36', 72, 6);
    T('148823113779314744065857165099809.221717074635693089191694', '11092966595418520282949944722.0380939', '1.65088982978010814960193863639871867140588986080588040621e+60', 57, 0);
    T('41923149787316451.1', '5365110765296979807.87945865', '2.24922342239089281227745098911304050907197015e+35', 92, 1);
    T('89071601.662186951761671290573973934', '58554585.4', '5.215550706243307817694462030641971735e+15', 37, 1);
    T('703933483121413326324300606700263.1340777782191867591', '9698057990085120525.451358661', '6.8267877404e+51', 11, 3);
    T('7179267585198.5', '68546806162577983072893068.0', '4.92115863551880894933171624903570053998e+38', 51, 2);
    T('2179125091820737344895165347844676465414.755190409660182076784763997715968467', '4885204766020556106757.1043681756625822', '1.0645472284317648022940599463327356931716913e+61', 44, 6);
    T('6054082848853159727128424059.6895464383532', '658046046375801179818254610193868223.56', '3.9838652831193688703597758116e+63', 29, 5);
    T('64177751627.985801', '7198074167877702389951325456802.0', '4.61956216145876e+41', 15, 0);
    T('82483923179358315327694378049261657207.659978185931591066150871174283276215266', '21756647.4243432399149645986367977688', '1.794573634789911755925291040629554394263359314530481920956087997123e+45', 67, 1);
    T('8156843784006400954704505423.212078656058466382100593', '64351842906788758395450516194421739779.629978064566965793858390811769344640', '5.249079298035962885506061603517911123229660447887497341020268029118874888669e+65', 76, 3);
    T('18186098.9081628430743778767203948', '5534413997883.73276348550445428868411768184', '1.00649400364234507711282897578596447173494651606325083625635040192425590432e+20', 79, 2);
    T('94324178.73859930077', '9.0763881644318662', '8.56122859522778568087463759626196974e+8', 94, 2);
    T('982056026387178001.579875460235067502346795', '45477225872281214465658646.71554882358090', '4.466118373124465446322489327557901646561580527486e+43', 49, 4);
    T('31177853382950208.0121863060165270668454624', '72749.9287147126319600232715347352066832650698', '2.2681866110873897106636353998e+21', 30, 3);
    T('49113306534722280855.86898712460447571613967', '43637956262.979689714593354', '2.14320432249252547756702876543244882280287077510691092257042871775318e+30', 81, 3);
    T('8817461991727580966705210.32285550673364413535528279963933529', '7113601224937570440691848.078', '6.272390842519378956468e+49', 23, 1);
    T('32619363881.80669809', '7837950085.40702271475427700418127954633', '2.556689459233295612888154111970386060885248283305104775097e+20', 75, 2);
    T('89280781332537926838982505118727.4140184367883839336384016201256147719364', '597411690320578825552371968646.741796886838675451736843', '5.3337382489013462885956040064362820773758e+61', 41, 2);
    T('13172821288867519658.73', '661829009803.2012148428735248835', '8.718155269925719330151752886038977570827486083007955e+30', 69, 5);
    T('58081975059771310.81156504185485', '809148084908254.71485509693070122730142153318', '4.69969188873029693174797820674704795991161216535557052e+31', 54, 5);
    T('4420507753912698252083033230.353029138', '6528548110642.490967', '2.8859497544887227580079357692648500796127480622252796446e+40', 98, 4);
    T('298274889595291045714910.437068919245938702997', '709832002.246356162672296277611164647834407188571', '2.117250621012362699746871300704131163939652196563264744203657e+32', 61, 3);
    T('467980411.505612086372585658253131774215806134615', '3970463940.30101528814152895338053', '1.858099348650263154984469874293926867014504256779906711e+18', 55, 2);
    T('9424617699366757469668831623027263964.003362217427077381331', '849485.478758851882538643653816173744090885807', '8.006075878465719149803182560492073473914e+42', 40, 5);
    T('32498487079975546442369046732680.51451154984262207610901', '637209758079462582912282.036178188966050986532784375417551537', '2.070835309017975831857602459842425815e+55', 37, 0);
    T('158615093493147.97599294906', '9.57148191807735', '1.518181499303814186596505019228151689791e+15', 93, 6);
    T('357308203270199266.3386737126688', '62168616829231150665048706.5472196882483760715658635', '2.22133567790460549722991887709327286547087335767882935946911179858e+43', 66, 3);
    T('40926523127429249676361072154416790121.3942897651', '190562822870457296047693.87767', '7.799073777435974081805425292783453732553e+60', 40, 3);
    T('633575702499118431.58113965', '74050419334605.221166571', '4.691654645027680504968933702537461828701466264015e+31', 84, 4);
    T('23039737.32922554838717610099581174778973322', '9769265960680746697.306884277266104407907815520869411', '2.2508132163342868811486081075516988720765545506167241451233e+26', 59, 0);
    T('3050477569424622.982981302053193716466590565397797', '743085363.238915953907', '2.266765232728061428817794275522911716679202086471201573332137171342879e+24', 100, 4);
    T('34655152579344637126.22554', '449084784423434197430993659860.404336849668801663125372920428097', '1.55631017252562059446728037e+49', 27, 0);
    T('97.0', '3140143903910753456105.3989257807301706', '3.045939586793430852422236958007308265482e+23', 77, 0);
    T('4366671753622517.669449059340637991405320', '73638060841975560072815922446090680.9052320065476483797', '3.21553240270191068904414573e+50', 27, 3);
    T('777288859111990.14886415303', '97408597.22120028267443261277082', '7.57146174017661416e+22', 18, 5);
    T('73654.6814979646895046344081953813088064967', '9868401309161661621865389395416504.544124154334397293835366643', '7.268539553203999580122524155989409869431042771701780238336397907e+38', 64, 0);
    T('16761639133224043420.39825560727711698417660302', '46612810045210294904698848855818.4407711369309', '7.81307100963335671685973812622844080523897519042076280636699050552237738053898316430471318e+50', 98, 2);
    T('718393077216367102467368976156142687846.85', '93383044238975082.10255384056333439253513887998401428189', '6.708573251066945125955045482151641185348434649207543609957569999607509103356151217514253304855e+55', 94, 2);
    T('8067491936524653319233695.7740897855905406780', '96.818135072', '7.81079524002714735319541092989580459160486648114458816e+26', 82, 5);
    T('1996766876854.7981322543320758506104581372871382', '974053.5', '1.9449577650844851125157950486445525938852280174686937e+18', 94, 2);
    T('9751668354701697806550832998376134148930.45046441341081569162281232514913286983', '91600291588.50', '8.93255664765023564016067168098821635041421547946923673327192280905742072242624358e+50', 81, 4);
    T('6754363701472696589218899906073222.591008425126', '24616953350173.2961323368188', '1.662718561492572033413671896253387223599829863711903456989837611728291688e+47', 76, 4);
    T('198762291662205.445909851226943334653005944444', '86750989869132630312236441519197530.4989988947096', '1.7242825550353569713126120240957361517273590991850158e+49', 54, 0);
    T('1183834991047621.97479174502103590933457860472', '7946704413749017751559097181210351.4133987655388', '9.407586748508666463444808666606437e+48', 34, 5);
    T('47731743470060387.1526454155638081552', '787413984631922825394035823856732.26', '3.7584642319189012363030822440870627892339274762645467302964264610926752e+49', 81, 5);
    T('648047365686.310', '295328558115815683927193150086002.3945772668515219000724300222368', '1.9138689409889066151515583479e+44', 29, 3);
    T('1881.13189873171363694833876856', '3362888.916124', '6.32603761201217460282234900038510728826144e+9', 74, 2);
    T('95535816589476794662050106989892899175.599561', '4049471531994768155948926330572124772.5932439828364655313147394209158367', '3.8686956956495978226489334966682057211811923047052802828172490661807333380529429738194567320414902e+74', 98, 4);
    T('168124609390.798407019048456319650', '56730307677047217.904497786060', '9.537760818823375654622037631349302456419188e+27', 43, 2);
    T('4204770562938823194048741.050214139920', '7198700673061365186337790411648.44411460210295616627285518', '3.02688846814963219139290785642455361032e+55', 39, 5);
    T('8249578623897109213903954.6809196', '36533365949913.48642015098234404983', '3.013848747e+38', 10, 3);
    T('7153668481021519356268610453581859.68530', '47374733417462467935790297.5378473', '3.3890313724529814563927700341784327025520087137466503102999209481745469e+59', 96, 1);
    T('264564146826810727357562565592.8601694344107070838639780', '346564094047.58959', '9.1688433862507133912342408115553343384635361e+40', 44, 1);
    T('68166239757307649322729324062121.64721087885117543870923600067301712', '6904596903626612064304990277.44127928917092885765400541974', '4.7066040796017565534216e+59', 23, 5);
    T('487105465903648524389599165075089.68177221583509061005', '3899345884529803.716652588150429850615470836561378348', '1.899392693803364500398800657983246424056169884481651575114452582e+48', 64, 5);
    T('635457240957.0', '117631596254615842.46355417881846332', '7.474984960532595812783230030006851149770619724e+28', 65, 5);
    T('6384482375366302457120323671380031.69081294087', '530832837686537074503405174236693892222.502319058763030243675857093987527', '3.389e+72', 4, 3);
    T('3130624942884797.5913959627195031115126221927516', '335043319038570759008775453122.8674594671457577957975', '1.0488949715290585998833801378e+45', 29, 1);
    T('155651133212146973587860.5539582592140618', '95946065431137415079453763415083330.128610321476350233407646706491988', '1e+58', 1, 3);
    T('64175980513910602885502712.57472098525059029825902', '12414356.23516192', '7.967034838404959719290619957806209658511075285541501630838005184e+32', 71, 4);
    T('8836.080409577948542888259757519', '7363879135.054304633', '6.5067828163753149534879128845368755172206e+13', 41, 0);
    T('76370732344329734.38897', '58778702125184786394115126565581149.7458448289815128', '4.488972527549572662675476673651685470913635175347964388490656470534233816e+51', 97, 0);
    T('16.0657341753175626394011', '89538813032745226406637.3369', '1.43850676855754455974474609568509690763359831493059e+24', 59, 5);
    T('2917866642758575888725356493.9436', '42.1', '1.2284218566013604491533750839502556e+29', 70, 2);
    T('7667858671057545887952.636381', '440338353927036645390285.68688380424144976406007', '3.376452265358634504266987876281e+45', 31, 0);
    T('3084775441447428462.74106386639276193757190922341823759', '70944326.2067500532147551087675298967782721', '2.18847315193e+26', 12, 5);
    T('504805696852985398791281752313026651.249003894', '497270924843915452650765.70', '2.510251957405612695758215431639035339000054757670409495734839813816358e+59', 91, 2);
    T('7963911263.0', '697957463105538847277297132194928125435.0', '5.55847131e+48', 9, 0);
    T('83528557300792870259394.3771302913754376511125768703680903', '188762147714126637.332637772', '1.57670298715601547389174926176701971565287870962908222616299488242693893e+40', 72, 0);
    T('1665811918133366104406502180.1973212858194441798029076982', '3436.878424454', '6e+30', 1, 2);
    T('73688609184254082797124703840143.0', '62165090.5648534696409', '4.5808590635372482106889174270033890622496176972146487e+39', 93, 3);
    T('1.82320227569360306', '64996.526461583', '1.1850181495693761534444994124398e+5', 70, 3);
    T('223988908400671492360895.821219402593104758196097865017528815', '818196316769220349652955319672911.0', '1.832668998505876934143e+56', 22, 0);
    T('627789382305323395.67246781012146', '46.1394926560560897', '2.8965883594426e+19', 14, 1);
    T('3708579205661994194994161677633814751865.06200418233235116012481343', '9908806500694257034.7314132059048847605018470542403', '3.67475937414031120847098925958961426938765884346435198463337242932510699755592043876300422e+58', 90, 6);
    T('421271152387012.90', '558743407336879691270.832840214790090206437210907', '2.353824790974535e+35', 16, 0);
    T('66540493371132216652.6902761194956933828866887408347319447', '7684780858.261145017404984472', '5.113491097577294666192436447273769911784846661522597751214329286960244233157858626984e+29', 92, 0);
    T('287734897563611978922367.429536727272870925167411', '698196768.57456130071', '2.00895575685046295093488570786e+32', 30, 1);
    T('4201423124973.80186129206414902266', '9906762930423421518262632306982690931.4476', '4.16225028695141904590533634e+49', 27, 2);
    T('77355.114352778', '959881925750980427139573019.83291', '7.425177613163185255572973798594330479235432398e+31', 92, 6);
    T('46191672073826358.137376098775451', '9828833323674876781457996243456.451362', '4.54010245755486711955721550649382562774033549543689562023e+47', 57, 2);
    T('95955729278939507073854789077.0568138758173385', '78308682399376961421945630290008082.743', '7.51416672850507074713292750054595687118422440115138640048869223972e+63', 66, 4);
    T('7796945940463908106.5580', '1659965820060184273321363985.79124631182858', '1.294266376202709592605997706411835873905016989893659738646982764e+46', 66, 0);
    T('30213713137994174.24661694657155755371419616278234', '723998743361178459004818163453892504503.2690994598', '2.187469034418291004786800613412429851835159772579e+55', 50, 2);
    T('84604.7752379156558721328984112', '1297037337.851', '1.097355524440683274504658902125394755223312e+14', 94, 5);
    T('28327657031930.8216992367874422', '42456728280.65840802224676701484152808316', '1.2026996374324e+24', 14, 0);
    T('48965358.23330212887', '5187408497600127830686705790806.670482', '2.54003315387465845883776043771e+38', 30, 6);
    T('440273321646571997251270.3416834473938761536059461361874179', '513570305038829.6079895062456687782930759673930', '2.26111304098488723320568256136916113802690943697293049114263014676e+38', 67, 3);
    T('1477001575.050216', '386541263820306800526239.623325687334937', '5.70922055484494217464467278774672580584424240986196392e+32', 67, 6);
    T('686366047533.93113845472067787277016838606670', '584057428614777083.6974552614', '4.00877188811155680506835193497711729246e+29', 39, 5);
    T('6.974442', '5.8610835405763370789677550323882336786717', '4.08777872109043095297100273435998572743424086914e+1', 98, 5);
    T('44245905702923952355192236023201.11814337837716941037146727', '270429319637894289798614631757120000.396764328260', '1.1965390176004151339066230000227641410448832026696883566705369139624974e+67', 72, 3);
    T('252219115651874686238256.490355310', '791.4384', '2e+26', 1, 6);

    T('-4e-365375491017', '5.3456178527729640191921e+1054', '-2.13824714110918560767684e-365375489962', 42, 5);
    T('-4.6924256191488929573964083575920e+8966162', '8.550436492344965503e-1468', '-4.0122287251585113288323835621429831626734128148776e+8964695', 98, 0);
    T('-9.68459254757663583433091e+86', '-6.9606680062770491502398096119369507497559857e-950004457908', '6.741123349974583002086518155222821771487921652854710238321333e-950004457821', 61, 0);
    T('-8.91339373924825376e-23482771', '5.0115601249493193917e-80824813854486', '-4.467000865e-80824837337256', 10, 0);
    T('-4.394706873577184995767118905155822247915108479085562e+54', '7.9237011378040523487148906759012338e-19', '-3.4822343854478830391832469978313697746644482923465680649904919538924e+36', 68, 1);
    T('7.1358833399519764856237457359179708233185279714364522e-2417344014240', '4.37671617085244659276681590226736647251024701043240e-22861988', '3.1231736007284381947e-2417366876227', 20, 1);
    T('31774302268008533.47936425988014966871158636616', '-2.613532089535643913322769987530747495e+3329468754', '-8.30431586000454919840859692536279464161443483409682e+3329468770', 51, 4);
    T('-2.39150909e-1168355508', '3.215796199965e+2629576911', '-7.69060584380375518185e+1461221403', 91, 0);
    T('-6.9082866789157382542290957298198e-72655818', '8.891254420247380023781092224423429048113893e+221707437690326', '-6.1423334470246e+221707365034509', 14, 0);
    T('-2.721070432026806790620747310283351566563642e-5068456656', '-9.4677100342e+6118', '2.57623058330651276949447477562723484719608825762965564e-5068450537', 73, 3);
    T('2e-5209869', '3.27910237568338294554075886091e-5277537339290', '6.55820475136676589108151772182e-5277542549159', 76, 6);
    T('6.26260e+56748678844446', '-1.872138e+764869946', '-1.17244514e+56749443714393', 9, 5);
    T('7e+874744837146', '1.60336923529549588149235244559067834591e+81419007008', '1.122358465e+956163844155', 10, 0);
    T('-7.57556905292192684177698455476299185123042e+49617991', '-4.69770432418249789513381360662739376391192494279170548060926e+851708178607', '3.558778349805444596562651365178597200445e+851757796599', 40, 4);
    T('4.3820075845492018915403752511206537158519288610661554290253e+8044509104', '-4.92525503982641608802991854080274777547434662230714112743e+483447972', '-2.1582504940358536725478148620978172839425648132373306514280129251250213654095628258192648433320627e+8527957077', 98, 0);
    T('1.2006420179197055050620134730009436007425034087e+485975255', '7.516637468643930347617315463824563473003e-58440872053691', '9.024790778323515646837985202871454513694314709348414373727755282545085548294237925e-58440386078436', 82, 6);
    T('-5.41566718198134e+22', '-2.49687626546486470137799e-977766466824', '1.35222508483461960262094362447964667066e-977766466801', 69, 0);
    T('7.60589787667783461255e-77115542594', '-4.670859173489634458219e+26', '-3.5526e-77115542567', 6, 2);
    T('-7.4966433482988907548997544029e+43384284255', '8.76900864982930949736820387866014157492538731517869211928e+57247021058', '-6.5738130365918329993948527542385199e+100631305314', 35, 0);
    T('-2.23300485963889753534047190162729477570698e-38215659', '5.37946118157799971255797118048366499258974e-608216019943315', '-1.2012362960702479136012479770008061867627585046391657114667299178776e-608216058158973', 69, 1);
    T('7.950307253071502928186958056737115e-5878', '-8.704973781449061503360427299536e+277124617416459', '-6.920721619245174163537379399514986887646e+277124617410582', 40, 1);
    T('4.185076993934040597e-5086853158', '-2.4667268379017523061168e-116', '-1.03234417396222869790237878021338626237296e-5086853273', 43, 1);
    T('-7.482132467868120637437704032689e+4135534685', '-1.079267135546765197e-748067339578', '8.075219676377475750975773722686595196675895524733e-743931804893', 72, 0);
    T('5.9101695821676643986226220332888689185249544422e-55092776249763', '-7.58844318e-1732029427447', '-4.4848986058443662522256637562228650314694666196723294196e-56824805677209', 57, 2);
    T('2.79556895158022664670992944544461e+49239275976755', '-9.54863625764e-51201054', '-2.66938710517915937320633071185786613763093204e+49239224775702', 65, 6);
    T('9.650036957504357204966723503725144e-8903', '5.2827447619784929146709110907128357e+176836103358182', '5.09786821901550154484410112384027758259249566084949692469e+176836103349280', 57, 4);
    T('-1.9e-167030692314', '8.34442521761450863784910826143041497148671762872e+765279179197', '-1.5854407913467566411913305696717788445824763494568e+598248486884', 84, 1);
    T('2.09304287474447074946800529e-7501027', '-1.97551468538166830548717111e-48640743', '-4.1348369361911657154427198676910716621461934966151719e-56141770', 83, 6);
    T('7.54010672323117869343357346909666190496199671915807625579428e+402680', '9.8795169544988039450430700458722239734734119e+831990420712141', '7.449261221089285054277404538248070965499024539847287625863482314079e+831990421114822', 67, 5);
    T('-7.44811068460452297761245492918614454e-13', '-8.208290293545018795893146682753990657320e+119', '6.11362546376882508180338076266859e+107', 33, 2);
    T('-3.4947862359051914652538570079980473695015572764748710149371e+99239488211', '8.9871386736604015510723260042643593420e+2901294', '-3.1408128536879e+99242389506', 14, 2);
    T('-5.198610193327258581719800393756e-415765', '-4.56170383752936118953906047326694262086701651563e-18022', '2.371452007e-433786', 10, 2);
    T('-5.62347393994580578946857452144e+711739768', '-9.8960690774730861960125569474902825924013159547513602999908e-839', '5.5650286565074e+711738930', 14, 0);
    T('-3.517924319374981778598350e+375735', '8.492485850974888138219723828823196858269193119643e-2696023558502', '-2.987592250709249628886768531923248349253445248263374217689147056839238905e-2696023182766', 75, 3);
    T('9.6061823586786749362302456379012707e-343654', '9.3623408858023487846487883335034549863206358327911147180e+225975', '8.993635385313060167369366223494901340216176627883091665301927493811301362565071628167e-117678', 85, 1);
    T('-5.611093152235595539671319037568882e+3104', '9.3369326422025118886218886116828e-7672157', '-5.23903988115475203401593214770827968462651178511683614826549346296e-7669052', 76, 2);
    T('-9.268173510493555076929792e+9561251517307', '4.276798840211695e-2670', '-3.963811372055959017179337832905193231744e+9561251514638', 84, 4);
    T('-6.615877417372958388378856e+11909579161607', '-9.568712877072994844431212e-49997922705', '6.3305431436753055385323034282007146139188087e+11859581238903', 44, 3);
    T('3.13696781667007592359275297900129447587414e-6165', '-7.91986395633964933818612228839787393059722904755153e+741', '-2.484435834344281929388881062173111091514115783550998586959821989693011942927146107e-5423', 82, 6);
    T('2.4226792374285808859796587236605797513550223547057610369227e+30722', '6.81148096005699658115021e-181593', '1.650203349807018249842740334e-150870', 28, 2);
    T('-1.3180403596179e-345', '9.2679763275836e+655199759772', '-1.221556685173847231932630644e+655199759428', 34, 1);
    T('-1.4408036342178688012338920567403679974229638452863e-699604810', '91439128143.02', '-1.31745828138176620969072009666937014038469492179938471499246626e-699604799', 93, 6);
    T('3.94877786556023884229e+857531', '-9.70860079707045854948404928391432929266542925094323649e+68785601', '-3.83371079330323188364091459e+69643133', 28, 1);
    T('-4.9439248698911068391217474401099507941163e+649307985974', '1.0659e-932564002775', '-5.26972951881693077981987059641319655144856417e-283256016801', 68, 4);
    T('-9.2870266999975131659903038220e-81264776', '-1.76712009189e+924999989835', '1.64112915e+924918725060', 9, 6);
    T('-9.9529475625803343e-8380259903', '3.89049351525830609686187735273842625e+83', '-3.8721877949924754297843126436589003358841610180895375e-8380259819', 92, 4);
    T('-6.291572300165739092278873938892254093838293256588204e+47332', '-1.25946726517074615022270199687517660731573780400393629381e+115', '7.9240293585138e+47447', 14, 0);
    T('-7.4375147004077e+724764167890', '-2.046123e-61522512580', '1.52180698913423043471e+663241655311', 54, 1);
    T('-7.64873918080465260e+9221770793540', '4.95712558580718754707407332583639550130331618765745394974e-934580649', '-3.791576e+9220836212892', 7, 1);
    T('-4.468665877987782626633447501324645186e-809575808268', '-9.5350245e-9105030983741', '4.260883862892751804562427444459427431e-9914606792008', 37, 2);
    T('6.7408832472702741622132529394631707555674128253597403099175e+170534000', '-7.9179867351364661680332847787803068724e+124048', '-5.33742241349896582821325145987650647964654046213642776117202178e+170658049', 63, 4);
    T('6.37174319854601681702224859180082e-315836040445902', '9.4141740964002222143080707667710387e+779363231674', '5.998469976866620955471570079e-315056677214227', 28, 5);
    T('9.147593105550112441758773009301990293593320826590438491429e-6468282465', '2.89194477470027798214332547157179768796e+2172715875', '2.6454334082679936112479491809090965479076421585200688773e-4295566589', 56, 3);
    T('5.7328233094e-87', '8.60243873475743729e+77732', '4.9316261296302880451120767526e+77646', 75, 0);
    T('4.4272959570600702344721523032687e+82140', '9.78448664421871514457574326443803332763694313562061516421e+50671234', '4.3318818161857771390222719506200195109610995148251659695150283991607404529739733970253227e+50753375', 98, 5);
    T('-7.17604577218598540169363566e+5772723', '3.127327063580210420815871267029e+151367543', '-2.244184215284758135331341729882187823433226124840969665414e+157140267', 93, 3);
    T('8.0272981418554178155719080970815482702413702712420870829e+192896648', '-1.29962661886091748755422692499702992e-428344717673181', '-1.0432490342688082248763547244329005128241414461867927831419022300575868037823e-428344524776532', 77, 6);
    T('9.030305318442539974766e-2044981', '-8.1857968864571e-79', '-7.39202451594639346564301264011415386e-2045059', 65, 0);
    T('-6.5608612741597121202382622463991446649330286159397010e-4976048910903', '-9544.79102361', '6.2622049796750087e-4976048910899', 17, 1);
    T('8.88672626926547643727099024606594e-472493725692', '9e+21567598', '7.998053642338928793543891221459346e-472472158093', 88, 6);
    T('-8.30352241902031e+30', '3.146350777897979014892656694e+38683902030', '-2.612579422237786082925181769089189264345514e+38683902061', 66, 3);
    T('-3.4034919504372051e+7088679', '2.636634217e+98612377706092', '-8.9737633338068030765069067e+98612384794771', 83, 1);
    T('3.3575405454829458735114745019996271945e-9783649449864', '1.15841627130e+671787176653', '3e-9111862273211', 1, 1);
    T('-6.22583640216942157211e-1999725', '2.6765159946646384113372728910413804e-93898', '-1.6663550710571803151064686476326659403289636202433e-2093622', 50, 0);
    T('8.5630740764600636037659235918188415118968517e-446678503', '5.94e+8', '5e-446678494', 1, 3);
    T('9.80920020235694074696019748853999204510528026232495766927e-675', '-7.9293935250033714503e-769982678688', '-7.77810085700308869287198497669386958079e-769982679362', 39, 1);
    T('1.23266983159515938962212128310198232615224387748e-279397581329', '-5.04770145075021634008585697504463400244353793884066e-1933832459200', '-6.22214929723891091357026669475821729574922e-2213230040529', 42, 2);
    T('-8.1e+380906243482', '2.01144277035e+7613456', '-1.6292686439835e+380913856939', 47, 6);
    T('-4.28292853681719296737e-978476394', '-2.64904141488e-5926027062', '1.13456550710001450301918104724656e-6904503455', 72, 4);
    T('227194063254325279843932892.9', '4.40419630e+34810218805', '1.00060725276666535648511382435847627e+34810218832', 48, 6);
    T('8.794692061474975395346771016813936e-614', '-3.63562434528073414027085233713407011305328e-577', '-3.1974196567945627470423787674227935931852011809244708813241877e-1190', 62, 1);
    T('4.363043985077918265492217825619861248036342396371600e+27570510363', '5.952642568019124331704e+430888786626546', '2.5971641e+430916357136910', 8, 6);
    T('1.297559404000536434488255488320713e+86406', '1.58997384896212222248024877398885226761080813064222568741755e+415045', '2.06308551983573024638463460606e+501451', 30, 6);
    T('-0.000563524', '-9.39802943e-678715585355046', '5.29601513651132e-678715585355049', 98, 0);
    T('4.26037214000249381e+18', '-4.1266e-3766940347237', '-1.7580851672934290956346e-3766940347218', 24, 6);
    T('-5.7647950311401056856045645472525553e+859988872297888', '9.803563755076332904392583483121434e+533215', '-5.6515535623e+859988872831104', 11, 5);
    T('6.9676300359835368e-9798', '6.4076920018070279712234140885098506840e-87818', '4.46464272531221232532e-97615', 21, 3);
    T('-7.9713619e-89', '2.2626958255834569008092476790692346361173136405991893783e+839118668', '-1.80367673e+839118580', 10, 3);
    T('-6.2424033426958828301262405335979514552278677160672967595e+518637507', '7.235471e+80987752340', '-4.51667283563791220367763397198725034137090352514411597517562e+81506389848', 60, 2);
    T('8.9695176920416599105100434160370519315e-80447388962432', '-1.18846598339290447105422712709762565059964470e-410434', '-1.065996666443234623e-80447389372865', 19, 4);
    T('-8.45252044976042158638778013545908074590e-47255', '398782825471191672291983984986018851720.86832', '-3.37071998730848873921550615870906970248149245888e-47216', 48, 5);
    T('-7.9e+485727190581007', '-5.3590830557629277e-9524', '4.233675614052712883e+485727190571484', 93, 3);
    T('-9.72394936547968102765272784822295877e-44181', '6.85343139107691831655609026314522119604337177630589319838926e-5681418807', '-6.66424198266209275430517032401157089610520237e-5681462987', 46, 5);
    T('2.7052646236898214130468507371693864504619e+48', '-5.51828238857101856626e+85172966', '-1.492841412933174541e+85173015', 19, 4);
    T('1.78811043256396723212624980530752860337996e-63727688133', '-8.6889478e+18734228838009', '-1.55367982091837314408555e+18670501149877', 24, 4);
    T('-7.36905968105321663386088558258475215718782374073e-99416', '-4.52369e-884', '3.333534158858362555443014950108281748594898637770289e-100299', 52, 3);
    T('-9.9863949497018e+27300506', '6.5749162854482327123826306896903271e-639001133212190', '-6.565971078771234960904204259240599914362248945878e-639001105911683', 93, 4);
    T('-3.822074408306010563320025144e+30880998810', '3.4014215086920297521481861703079e+78', '-1.30004861002334273812238632113448350527114419508392942218376e+30880998889', 90, 4);
    T('-2.518763e+499106988048402', '-6.28020976962245977218e+60265502002', '1.58183599e+499167253550405', 9, 1);
    T('2.422210036404216818582074049318217e-48498', '-8.6584518876646069e+767494', '-2.097258906202424731105258940341874e+718997', 34, 5);
    T('-963817823584353.37356013419990166217622345310292199321036', '4.5092788396938132834531623377962688645996088686e-6309910760164', '-4.346e-6309910760149', 4, 1);
    T('-3.623718671637413655423352177101334731901163353e+45471282', '-8.07806188915620381637273e+33049', '2.927262369837793516237444302429452048616390255742003296219947566456369e+45504332', 71, 3);
    T('7.1614907974e-4353240', '-1.219548083041378530056525331821560712199871524146397296554e+914788586374296', '-8.7337823736876433464097454656600884193247698296367361296357097321596e+914788582021056', 94, 2);
    T('6.21124033869247931198069926286183993868754460433693693783e+954709480236176', '6.0159626134220300236643e-53048819017', '3.736658966055274275227489153392914114794492450392663358337596462608e+954656431417160', 67, 2);
    T('-6.60120972116702161736001125427e-56', '-84.66491935615826', '5.588908886956938388909659353031853560312207702e-54', 81, 1);
    T('7.40339084620292227820885932418514e+8606972642692', '3.631790719689896874398e-80241453', '2.688756616947690572541943164334221712330386669427804572e+8606892401240', 56, 2);
    T('1.438228171503925e+817287', '3.841236929406216034636413362e+46600348', '5.52457516529325352e+47417635', 18, 3);
    T('4.958107297108909094285085235e-576040', '-3.7846604660852989429545500028725527721e-8313', '-1.87647526739771256570272595615592495635545846154647718594840299435e-584352', 82, 4);
    T('6.9043937e-975710450', '-18280379', '-1.262149336012123e-975710442', 94, 2);
    T('-8.6432499e+2719761317', '-4.57279892851942673322343366130576205025674735533876542428e+260749842758', '3.9523843881645642259990769670737661710305426553357048719531567572e+263469604076', 88, 2);
    T('8.5057177486936585487291867945397225715369318530698785165e-601530276476732', '9.586172665239343133314853e+28', '8.15372789807682741757e-601530276476703', 21, 5);
    T('-3.75520897067845905964e-43030', '-1590850721565924126937231953304844201093484291', '5.97397690063465781239534440946e-42985', 30, 0);
    T('-2.41155568665261530679e-296868647', '1.38039422975918151828635812155301708084891365e-359602753775661', '-3.32889755459821e-359603050644308', 15, 4);
    T('-5.96935296842010575177112072710e-58', '9.41927831734043821591435319553829e-48493888018098', '-5.6226996983991283729236690945777398525324085997175484660690659e-48493888018155', 85, 0);
    T('7.47311879381584528254136741742850731129591579692890e+55', '1.6432e+2738224139', '1.2e+2738224195', 2, 1);
    T('9.13872628253910879093302418083429e-468', '-2.446368316834e+548', '-2.235669043382183749665324e+81', 25, 0);
    T('-3.03029233243424665986517073439482290096426e-1200', '5.3e+771060837', '-1.606054937e+771059638', 10, 3);
    T('3.7954946734852448304578077411246145373880868291519277e-30181110836977', '-3.1235492002087947572210740238941e+2210387', '-1.1855414351761577090864852321140547298000179395783940431424703385e-30181108626589', 65, 1);
    T('-4.2224745550295211943071750028e+2463423', '1.04595577051371537896885394380664463772243427864e+81065', '-4.416521626680460329746214253334506558916113720358040815e+2544488', 55, 0);
    T('5.91719500600095116906e+2275419', '-9.211511647791595056614489979725086570571081397049173e+87001201061228', '-5.4506310720032018902648756e+87001203336648', 26, 1);
    T('-3.37376666395787996829527585e+471085508842694', '-8.99897781272018174914355878143847098371858e+457187351', '3.03604513542e+471085966030046', 12, 3);
    T('-9.393283329292e+701715915398', '4.0653301417236011453025233582184349e+2075232140', '-3.81867978483205863653987908568146e+703791147539', 33, 1);
    T('2.64826261348050169134751935007036344691601755e-51070155061093', '9.6853376032658004515071616835770829482178993047656689655562e-769983', '2.5649317473665667136885882256880062151383050136280003e-51070155831075', 53, 3);
    T('-36407158662917923171420301.42678', '-8.008769552398242025223482301447923487990085e+46', '2.9157654e+72', 8, 3);
    T('8.1979255622757247e+632377', '-7.4500825963252853316703022412159098437138487945222e-939', '-6.107522255748055567629062930001899185323777042290147680119e+631439', 58, 2);
    T('-4.99454144730430102046264314625703e+87510336658826', '8.198447639638000e-24', '-4.09474866e+87510336658803', 9, 3);
    T('-1.433388526616417662565148450311292e+198609051', '-6.14251e+40692', '8.80460335862661166e+198649743', 18, 6);
    T('-3.3966362767920475e-9497', '-6.1863084679462548322e+489', '2.10126397616520825376564741614069295e-9007', 41, 4);
    T('-4.312794553725442903465825116116866319841126935245553753706e+87320742008', '8.11887153287361792e-743586582261', '-3.50150249293738e-656265840252', 15, 1);
    T('5.1053155850654687895388923420153161460896573036868657623160e-35986', '32659272441167644486263285719765765725355028.2', '1.667358925907923340274034222432633940980645644042884831904881284185892961966003171698064e-35942', 88, 6);
    T('2.664934016e-80641801732', '2e+8814889384', '5.32987e-71826912348', 6, 6);
    T('-6.4715e+59423030186229', '9.989876145517840e-3544846614', '-6.464948347571870156e+59419485339616', 87, 3);
    T('-1.718270096220648085971118466892093113733403707e-3558776729755', '7.0700067729172488960867e+7318256', '-1.21481812179811549229818230039452077980454113418806152655287435733969e-3558769411498', 86, 4);
    T('-2.96584717079098558049316765e+45329', '-8.20905747885970242576797562493492492820793e-49440024', '2.434680989853662936172370663127988180261e-49394694', 40, 3);
    T('-2.36934394896876456354306823573157373246071729180466e+51', '3.6655863691988863552639695968027891631773801812436e+41979660', '-8.6850348832837651732708436291271951880769955178584171031451910907856240144878e+41979711', 77, 4);
    T('-4.35463461158087129027505e+672860752827932', '-30639017040572120691323895289628446960551643114536.1', '1.33421724069691473365224653560570269125455973765563254653017031597222154305e+672860752827982', 98, 4);
    T('2.9383775867167814328e-544606985216', '3.7509026593651970740576931215763027552e-23932228649825', '1.1021568304235065453357023836503621990176e-24476835635040', 41, 5);
    T('3.501361077999357563196337395394445027654755152e+5564386746110', '7.197286825730360191818501456997702002e+37', '2.520029995880982829661054985567814511611271066233982794277283477300559589e+5564386746148', 73, 6);
    T('-5.77020697880525107108794e+871152887577896', '1.28019320e-369563717345', '-7.386979736859026545499497390008e+870783323860551', 45, 1);
    T('3.478076711720725041e+55838058990437', '-9.806442859980580980193228e-71387233234089', '-3.4107560536118441551783280072866692438222348e-15549174243651', 67, 4);
    T('-1.07459798368262193780041756939830e+2643889845', '1.412079213984242126050955978913450e+23238372619', '-1.517417476147608231815602909979214670370001787926664676804277135e+25882262464', 88, 2);
    T('6.949702108488125882555363223141926130367132e+16696', '6.03680246607820998150e-211050200', '4.1953978827030053932864010405233214062197850357477892e-211033503', 54, 5);
    T('-4.8578663341223960428822105578165405392252674812907491121e+5653640', '-5.541846355430819e-6893', '2.69215488389262737503134788149765830163232472936596591731330706372258099e+5646748', 72, 2);
    T('5.0968313425e-34', '-3.43518848177906461e+429489', '-1.750857632132652666458053892e+429456', 28, 2);
    T('-7.49750263614058342599317542777426437881509831e+84572505149093', '29254146641437283309119953026726604473.41592883', '-2.19333041562219226598165650473150845644455075939618914168505191102e+84572505149131', 66, 5);
    T('9.877439439250017522600975669606948e-378439815', '-9.1566614646336885889144845961148443625022582257222217139e-93', '-9.04443690826336251708717245618151941904514587258441033248056067e-378439907', 63, 6);
    T('5.0870502772945926851079654e-33642025', '-4.58841915907929662945e-463502', '-2.33415189555381577042827914895087854655673e-34105526', 42, 3);
    T('9.4922774237407200933029682534848966933815606096368815e+2133215427571', '-2.50198764045236485874556297184187807687457896782479634711277e+55970', '-2.3749560793944296974107840584948039907584916307821661686739434e+2133215483542', 62, 6);
    T('-3.0431040561799403834e-8448', '-5.80477809123237374338052077098e-29067', '1.7664543754653688573e-37514', 20, 0);
    T('-9.77161674086994276241154963268690681566501528300742229e+74214', '-9.754210338713394302951542e-4165591442', '9.53144050397384785201455466644e-4165517227', 30, 0);
    T('6.444540974837256578064404770896829498539630502055178559233e-39856787665', '-1.940826697e-90', '-1.2507737173874552796946261364770735323422837390904203914861402243401e-39856787754', 100, 0);
    T('4.0322498715658991417881258994262322647073153050595433e+38045', '8.98726644749854035590297036118318203448393e-1395829137', '3.623890397865450399243868936587502e-1395791091', 34, 4);
    T('4.25187708868496114538625704427209951e+360308687', '-6.0292284984737804116870071099708629095862318553230810242457e-27986', '-2.5635538515107097159305256321e+360280702', 29, 4);
    T('-2.444369978057532743909326416543e+80200772188', '1.20960361806961078243306709522145837850572204e+404275472', '-2.95671876935912672591467259873143746160369546521109645648178727765401570772e+80605047660', 75, 1);
    T('4.98853926617194136279698968305534205888912069294e+383516344468658', '-2.888552237419069693899212908276148701739974816008e-15', '-1.440966e+383516344468644', 7, 5);
    T('-3.7155358768069616730988513750572762026213055229496988860934e+105097', '3.056017186898156957212361743097e-22096', '-1.13547414980587880749597089270990275562599803943415628838879194182158e+83002', 69, 3);
    T('-3.26868e+39', '3.95521551698008947501148464400737581090e+33195839987598', '-1.292833385604247886518054e+33195839987638', 25, 4);
    T('3.02416864595690438558336001312439085e-4270', '-3.58124167387953167547748e-92094273', '-1.083027878374e-92098542', 13, 5);
    T('7.315684e+2162435', '9.7540866613e-454487747', '7.13578157226858292e-452325311', 37, 2);
    T('-3.10851451560e+927468', '6.1457261227386e-11632891639520', '-1.910407886143504532442216e-11632890712051', 30, 3);
    T('-7.772895610040503205198794282471364e-898009283898714', '9.7318631621728e-96257396292031', '-7.56447564507678468320320719182576412835040196992e-994266680190744', 68, 6);
    T('4.3754232618837781567310477907170070275217601379e-61142521528', '-9e+1151', '-4e-61142520376', 1, 0);
    T('-3.054334980342259182582e+2776435843', '-6.0099795676987371034130e+495579439072', '1.8356490824764501530953294778587e+498355874916', 32, 1);
    T('-8.2662948211111096614470870460150520e+498560', '-9.30479704225396886006808155920993067187666054702888966483941e-27', '7.6916195601873953803874751679640295668171088937941642322618254484351298316006870728518e+498534', 86, 3);
    T('3.51328289749715266930732819237443171058383915667438e+69781085', '-1.18770971e+60513566', '-4.1727602113343029226887326682398605e+130294651', 35, 4);
    T('-5.20937364493663786e-66217843', '-8.762857023230e-501413281781', '4.56489964311622813998441174878e-501479499623', 70, 5);
    T('2.99237412007023218728194765022912729708614060358687030373e+4643774847', '-1.04518084901e+62830900315', '-3.127572124e+67474675162', 10, 3);
    T('-9.31478196998604953561662003460278875e+21051048255629', '5.980484458804620884202e+64000', '-5.57069088086550598294123361663862166263610112199e+21051048319630', 48, 0);
    T('-210635434705666696508402350385678035969688128743.2097281', '-4.68910872965986822455580236929411890365310024028e-90924465', '9.87692455654042882689911202465252085845362588956553075067600277313925143e-90924418', 72, 1);
    T('-1.20e-12', '8.118831381863912039445600935425734e-34970969', '-9.7425976582366944473347211225108808e-34970981', 100, 5);
    T('1.944113984e+948253198005133', '2.297253892087602264665253330992582454599861270e-438', '4.46612341640593451576578807968526015026063541946699968e+948253198004695', 100, 0);
    T('9.15451606820876194665327315338595147061325966787e-52078739357', '-9.76457995066120265686279e-970096226312', '-8.939000405763709966520625611551444858168851360649352e-1022174965668', 53, 3);
    T('-7.8645517418934281967842741216011256555077423e-772198368133355', '-4.05569647286594173422253890838219928391e-5080141', '3.1896234760268874911941387e-772198373213495', 26, 2);
    T('4.3296499533992052581699117553375280696e-79824916314', '-4.24876592757140e+2889901799034', '-1.839566920031364311316386532719313722352242805816944e+2810076882721', 74, 1);
    T('-5.748143806273579544397890e-106617', '9.84305173878286173155250995238988954517e-22631386', '-5.657927688e-22738002', 10, 2);
    T('9.540373379589203120911821476755e+55', '-9.3379809507909884131693e+66', '-8.90878248820374223693190311614481986884581252158296215e+122', 97, 2);
    T('-8.7895497473719072325000742554182679089427457e+683', '3.555428771566160e+688148823944', '-3.1250618060918152096545829266102754402888652812843559605512e+688148824628', 73, 4);
    T('-7.69210235997225476471805082785489942902564242604555e-135100', '5.56209275294694724763e+466', '-4.2784186791327e-134633', 14, 2);
    T('-1.97513087631910313220e-348939182005', '-3.6144403262507989719550346881327726816240972009451613837e-7650', '7.138992688990845598434e-348939189655', 23, 3);
    T('-2.64524083703998644557262774412933371732582669694e-198677488783335', '2.079435358821807840931200948572702786191e+769', '-5.500607329140343535795338719234517915e-198677488782566', 37, 1);
    T('3.78e+341', '9.175414122470770916068769974816136012158087246669288425e-152642', '3.46830653829395140627399506e-152300', 27, 2);
    T('-3.962114432041330201334195036748630e-278026967', '7.87606949927743490938503934504040347e+70667311', '-3.120588e-207359655', 7, 2);
    T('7.335207791718387761e-100646478598', '-6.495456464180555947291373617176e+908246593', '-4.76455228668247828412587153905e-99738232004', 30, 5);
    T('5.245524684555342825359006837761363361218693446e+81745844', '-7.97170178415296665237871211818771e-243090978991462', '-4.181575848668825399861232591405172005506245487760162531085987876466173957474866e-243090897245617', 90, 3);
    T('-4.78430317e+1595033', '7.7676290604349882704414564007449319e-383657495651483', '-3.7162692337223235961185877158e-383657494056449', 29, 4);
    T('-6.886976288651563515009676207074e+746992140', '-4.976323644275253853163928853421151423e-214201299', '3.427182294277981115741035775656453055374059358617023829951e+532790842', 58, 2);
    T('-2.20402984172890375751506994079258969922201473668422412335039e-95793806027336', '9.089042962428292973123e+78', '-2.003252192194804e-95793806027257', 16, 3);
    T('-5.2020339849442227419652033092442899095932365580019850587277e+84731951', '5.2396298577812947133710231e+182429021529042', '-2.72567326e+182429106260994', 9, 5);
    T('-3.68357261278774e-9145585193', '4.53529549558478875403910421803991587378005e+304', '-1.6706090278435728451876010167692274392159004422402096587e-9145584888', 78, 1);
    T('-5.35200943849571150875203261e-23201865439', '-2e+709269507305211', '1.070401887699142301750406522e+709246305439773', 70, 5);
    T('-3.06876821e-80808621', '-8.899958486e-87671938338', '2.731190967215653006e-87752746958', 43, 5);
    T('-7.30299370179246811166536e+5364757', '8.92973658524770471094381405e+4598074926', '-6.5213810040729768518111028325793157203812335666308e+4603439684', 63, 2);
    T('1.0595123445018582194950784630492974928405408222500e+102', '6.56056856974492065475942206963389571088e-8840825', '6.9510033865956436263057294014158607017339016672761432352796561965e-8840723', 65, 3);
    T('2.438635393041257250465311600695326346646697923113e-199369847112', '9.8533286e+618652452', '2.4028675863225661045967218103043038977747423e-198751194659', 44, 4);
    T('8.518563596416084218848511548108979362053410e+54091561637', '-1.29728820075682e+7048329301630', '-1.10510320410271676483580124774344417502752543235512617562e+7102420863268', 94, 4);
    T('3.58519094132252554191446519355754015791845795108338784687400e-923768790', '-7.46392517551814663608311094611e+54414050602484', '-2.675959692597680081283780306654074552798513386646180626348212546983783867932177394596e+54413126833695', 85, 6);
    T('5.2424392377047794076253633e-264793472726', '-8.07350547856446965149914430063292332487641121e-9735079645', '-4.232486190665087854375762211445918075756345255565e-274528552370', 49, 5);
    T('-4.76251657022166782269e+595144', '-9.94532823e-5552', '4.73647904916683303546814915387e+589593', 94, 6);
    T('4.821786142502828886568633258336931509000e+48', '-2.9174354475e+16031267072970', '-1.40672498124020393627022873094995888827362652775e+16031267073019', 64, 0);
    T('-3.190035763e-37728724', '1.379098e+131886719283976', '-4.399371940681774e+131886681555252', 89, 1);
    T('-1.83790574603522793676e-2884', '8.5185770546595247772494421843196136555e+478416908', '-1.56563417168025885555028526680404e+478414025', 33, 4);
    T('5.7565171710471332008511167801e+1619522853', '-6.21619231863616950391375307114765290964205430617611307732127e-21445267', '-3.578361782076040209203e+1598077587', 22, 5);
    T('-5.56496609060520656663296753842156595084933398395485647390e-106745328', '41683877598528853149753412815650359754447397148', '-2.3196936536075105811032194944295288414181466852598767576585056818954718253e-106745281', 74, 3);
    T('-2.34043281033466622029650569488516830636e-464624894', '-58594114369513020887192286545763876203204033221836.335', '1.371355877629102085942662307490385150095040244885718943423884595e-464624844', 64, 3);
    T('5.0442032727903751996176086675236589483234160280e+461342407', '-1.445665675325e-404137862295', '-7.292231530835072960115322e-403676519888', 25, 4);
    T('5.29093242881381369e+25', '-8.17311558331230511088458457903347930472881175803405035e+703', '-4.324340228419060411426127376e+729', 28, 5);
    T('5e-670406', '-3.7364086965325514702228823581286223236147030409582e-4096536', '-1.868204348266275735111441179064311161807351e-4766941', 43, 1);
    T('9.0907430689350688860035848345207687654073e+45', '-8.05615e-62549354', '-7.3236389774801255205977779964624491289436019895e-62549308', 50, 1);
    T('-2.249563275854942e+784382670', '3.515482198998994230944974140575e-1286388595', '-7.9082996517899125660472804982547654e-502005925', 35, 1);
    T('6.1486090982205e-57359101668524', '3.7950001574086969346075e+5368230161', '3e-57353733438362', 1, 0);
    T('-5.09309603724204416513580892379275983255338676609673e-9316729', '-58619834405892377017619734984852982963171478881926.10136', '2.98556446316435303716122539523013980283882593391456881641224767435200084369445110075e-9316679', 84, 6);
    T('2.732363337547928935461456330765469413986001804627159e-267935', '0.8', '2.1858906700383431483691650646123755311888014437017272e-267935', 87, 0);
    T('-8.55815463008347540358730643182352158794811830923881457182567e-185', '7.941588179511019159617227776e-24', '-6.7965339648698427035305751402949245590572727638980956099820613740802582749055880554e-208', 83, 6);
    T('9.623236228227508596001207e+69476', '6.9015094020312784417994245380245695633535066983036e+3138854381007', '6.64148553070801682046248775520973079195027771702361546788914153003304524452e+3138854450484', 81, 0);
    T('-4.883232360e+37013118895671', '2.1725456499041641101538923989411349190586e+45', '-1.0609046e+37013118895717', 8, 0);
    T('4.4096620132938358679e+868090', '5.441342687621275814124495695813523918610110700464084038e+56', '2.3994482150917726939298953687e+868147', 29, 1);
    T('1.4634973786e-762327751', '-6.70737222372627943801370932113750906e+7167985727537', '-9.816221666717862681464144782347130079443750116e+7167223399786', 88, 2);
    T('6.698661e-475254724958', '4.782e-56536673', '3.20329969e-475311261630', 9, 6);
    T('-9.09861e+94995984', '1.2342665582556228870788059304e-5834824', '-1.1230110049610192956604094426396744e+89161161', 41, 6);
    T('-8.49264349594473440650816452873620311704240467308253318082479e+116421179641486', '-4.456842261e+957', '3.785037244033327442334634111321265897271451847605812303e+116421179642444', 55, 0);
    T('-9.16658797118928255387894967732e-879900', '2.701348588617826e+89586859279', '-2.476214947841330948754192431688425358749990632e+89585979380', 71, 5);
    T('8.7e+147720217', '-4.584243177095664611925711e+110991164481135', '-4e+110991312201353', 2, 5);
    T('1.453319326595929461515560073791006960e-38427064322', '-8.12074645030653406229202313677698987385746801930294e+178121', '-1e-38426886200', 1, 4);
    T('2.3053381727285435958473e-991771370783516', '-2.53145598863370406124146814479022312858798299803247e+140958', '-5.835862123e-991771370642558', 10, 6);
    T('8.0086065041467439408833865753661133807150e+185330108', '4.927210643982405006183e-74787737439645', '3.9460091210698555501539881152410096646058184761014998607960845e-74787552109536', 77, 0);
    T('6.59224973753261984992577426411524390708681377e+2745', '1e-60', '6.59224973753261984992577426411524390708681377e+2685', 98, 5);
    T('9e-538755403469', '-5.08577451587151348155998326743168763322962e+34408580545170', '-4.577197064284362133403984940688519e+33869825141702', 34, 5);
    T('-5.314401638168825035e+47297', '-9.7065387108555911282e+20', '5.1584445225920068610753398043764054487e+47318', 81, 0);
    T('4.30194120e+319165499217250', '-8.41858873090195137891805516722553913283e+135', '-3.6216273707322817797364392947760436487733649596e+319165499217386', 73, 4);
    T('-7.75762274496969577982211755809339e+438', '-5.0275532681637517045687506115198902358028312407721657916538e-705220535', '3.9001861584653848526700593380322207280134284536752628046594496461153e-705220096', 68, 4);
    T('-9.12931924496422907567448911054277759734734086480921e-671', '-5.0626706428945644347456759309003e-8228368901482', '4.621873653109277319156427892809543079561545342880399e-8228368902152', 52, 2);
    T('-1.6661002935818923339192618768715799177e-883', '2.52338848186555869524e+61615943140989', '-4.20421829045737294185e+61615943140106', 21, 3);
    T('9.578022239486718791e+672799', '8.11503383143265618453941290345328e-198167', '7.772597451164909762171838497733677167771250016658448e+474633', 72, 6);
    T('5.40188908e-140340', '-4.9886862436827716033353968695046675277e-83165', '-2.6948329743296182908191571926843448526913227516e-223504', 92, 2);
    T('-2.778042552437197462844071e+22851702', '-2.1779964062886034069799168850765790e-52522332313915', '6.050566695725035160155264769874630754048979439545971113109e-52522309462213', 96, 2);
    T('9.8598248155929256057657705e+121633315', '9.922593932033842193448223642133863108542e+3021', '9.7835037886119060696673750364036479422000917286898115437469761611e+121636337', 69, 6);
    T('4.49612382258987943655185249677214820139e+940690709521', '1.9131270267239457210073994653006640370680591825282495154522e+2438568', '8.60165600049407726663237653015072185972059e+940693148089', 42, 1);
    T('-6.70520552175983086437122651131727467323041472820711526464228e-5923813074', '-8.637505702125878769841819013474510959923792613802485e-279559', '5.791625092812646718817775735656348782330117189253136067959020516e-5924092632', 64, 2);
    T('5.5653351912056375889785081738009e+69823', '-0.1', '-5.5653351912056375889785081738009e+69822', 36, 1);
    T('93393097557910456121646.7659143', '-7.43332767893e+32145754', '-6.942214970982e+32145777', 13, 4);
    T('2.87e-2226426741', '9.406198443984177501969247791541659708e+95343077', '2.699578953423458943065174116172456e-2131083663', 34, 1);
    T('-3.114562982075857719485596e+8684', '-9.94304617148828776071394146202384e-67', '3.09682436e+8618', 9, 0);
    T('3.614680848553021965197e+842460', '6.69087120806822373790583689850192244312736602e+188035847423486', '2.41853640159390301671531148380123899293e+188035848265947', 40, 1);
    T('-7.530595283977873991942346863e+462912746', '-4582', '3.4505187591186618631079833326266e+462912750', 49, 0);
    T('4.69153e-119318669736', '6.635320291064100263587070681296579916017460214974e+54445', '3.11298042051359583e-119318615290', 18, 1);
    T('6.62757567188592775461058206e+44751', '8.976441991109839189e+36146768', '5.94920485603748475953148530614e+36191520', 30, 3);
    T('3.35831202679e-6406974', '-198564038326954718219234006539.148624', '-6.6683999800140254043138909606178032894647963696e-6406945', 47, 5);
    T('4.0243121289284135843624526909258e+15932154393829', '2.426822836989820340869363524428067903981514684526e+94205258130839', '9.766292577658596e+110137412524668', 16, 5);
    T('-7.0747361866020e-98961612476', '2.00175934719854795389752e-71420', '-1.416191929049436406310857034790502704e-98961683895', 96, 1);
    T('6.84787675240e-380431289477113', '-7.30315821175640818398968683e+28', '-5.0011127337385863974868078324513450892e-380431289477084', 66, 4);
    T('-9.22744709462901992948583438953e+39054', '8.56463627637696166217201922781395206843008066247313e+56', '-7.90297281250089026447970641906592350362541609618089841509397502e+39111', 63, 5);
    T('1.3535059325607815145044542184097586422991415e+337491', '-3.762309953019246619548315509386e-68', '-5.092308841544025483218907319776818855926264641408901669303158037762992119e+337423', 73, 4);
    T('9.81928111597529910780717e-436149', '-3.2727023986011841e-67057505652', '-3.2135584860791673179242391596951516469997e-67057941800', 91, 3);
    T('6.8324195333854206930558895233036e+61415823498777', '8.97705500364204790473581251133800934e+49064', '6.133500595915925700554324755929782503923496388321717873222436446e+61415823547842', 64, 2);
    T('-5.299548891688262010894968933038768e-134826226956737', '-8.317142268787788479838397844779530999848035998596774952290e+111', '4.40771020925e-134826226956625', 12, 1);
    T('9.2097353851228983000025672887319e+769', '9.6004471384e-869982138', '8.841757772352435091657387183975961591979496e-869981368', 100, 1);
    T('-2.6310973982133567380935299875375269211064e-45560', '-2.1746231992305538022e-13588', '5.72164544158991622343777422549373984357151563421953995075408e-59148', 64, 6);
    T('9.9926725809400584934695904716321668716e-3278151', '5.413850711726653e+19284470', '5.40988375643737462322097675561797902732652346613487548e+16006320', 91, 1);
    T('-4.52347e+78765', '8.873345693012408351409089355304747417957e-2604087784806', '-4.013831304197083880534847342604036580270595079e-2604087706040', 58, 1);
    T('3e+2608787', '4.55516520418515587902391621463304080401020830002e-99139627858750', '1.366549561255546763707174864389912241203062490006e-99139625249962', 88, 3);
    T('-8.122571507710309705713633799629977851e+486', '5.54034283465708215920921618670915307523586389185309396e+54938442', '-4.5001830851732586950588728762984675924186971517889307805582670180622061245297e+54938929', 77, 5);
    T('9.214571296058185417220881514915122178791e+43', '2.53950464858431283791763392310587294061128617589330142e+2937791', '2.340044664105133824967237629613323691240036e+2937835', 43, 4);
    T('-153750316013992851.3489173', '-7.439822895146153068997430533176608433384487343941111684e-5848883257', '1.1438751212168602363767764078342046294960178856961818016551910585e-5848883239', 66, 4);
    T('-43742708257138553141479.207246', '-4.46e-322750615653', '1.951e-322750615630', 4, 6);
    T('-6.599589057864374478152201140205273828488e-567214403', '2.763106320758965422162023552631586e+732113', '-1.8235366240196758718826955152052940873339260506536695126097952625615421968e-566482289', 80, 6);
    T('1.2168092025697675407554806e+530447806', '3.936369922554252265288541420314640828992440820125973920016e+85402506922', '4.7898111464828573109100203260431243652152357548324525920379460239969359333180396896e+85932954728', 87, 1);
    T('9.212550882771836457732637960321e-36443', '4.99593e+437', '4.602525933176630091428021796510649353e-36005', 73, 2);
    T('8.705259494150235486899907524623551574123340631229339423953e-1742469296381', '7.6515914068872935229914413756395787622121760717768e-19105161981', '6.6609088740163969489558813288e-1761574458361', 30, 3);
    T('4.4074040277227460479616896984821027711949912368e+93534808', '-9.389971616790e+50774155', '-4.138539872404251168983741e+144308964', 26, 4);
    T('6.1848312907561574116142905318803695272769453194941201939648e+451351015615884', '9.209619926074325204328345265969616735619992511060e+4446700394896', '5.69599454947558957546173486977734720419518115190468494489237573288142183380469828235212e+455797716010781', 87, 4);
    T('2.17519169533e-493141034534055', '-7.23535182249629227e+81', '-1.57382771970847152154204740991e-493141034533973', 89, 6);
    T('2.30512034203105222088e-586060628', '-6.69789783831862135334498811217752450667493e+9205182589889', '-1.54394605559540657620982461992097e+9204596529262', 33, 4);
    T('7.238339441877644e-4235498626593', '6.6513415669327177280182220035922160800531003340744233e-62646', '4.81446680053293421422033052129844057649634564118232975913177884627052e-4235498689238', 79, 2);
    T('-7.9121640842967257489047437582010425e-19066670', '-7.54314457766641083142599213244665e-11067727', '5.968259761006976953750213627669285864801270331170938e-30134396', 52, 0);
    T('2.1274438321200534842808098001126485216054929062278634e+183245', '-5.244020313086503105384419183e+8592901196533', '-1.115635867058815282422e+8592901379779', 22, 4);
    T('0.002148', '2.1024752589480831268320714e-8241060012', '4.5161168562204825564352893672e-8241060015', 53, 3);
    T('-8.26862552003997399315553314633503817185e+8192622736', '7.99697442210105606830616609373171928193e-731985489038302', '-6.612398678969171522616e-731977296415565', 22, 2);
    T('-5.378664652651304612241743241034369306506886e+646117726143405', '-1.21613469888816815468171736644080e-35288760851078', '6.5411807177725278931117090445489764959304817086473229e+610828965292327', 53, 4);
    T('7.3807781586281689764053854837646417819565e+426361604325084', '-7.350829351084549781741813224262549265892556e+6381011', '-5.43e+426361610706096', 3, 4);
    T('-2.1840907524590232349e+79346', '-6.41678835566437157610517126e+5661002568', '1.4014848108093295723948680307881594774383708974e+5661081915', 52, 1);
    T('-8.38634659911075373977e+875383475', '-9.2267690802117534137886595623662e+6300', '7.73788834966140956231808196197047680935012282e+875389776', 46, 3);
    T('7.35722233e-1659813769445', '-3.783200512566877e+852', '-2.8e-1659813768592', 2, 6);
    T('3.63011034674282183983598962348e+741189734590884', '-8.0004150790613725218792570742652369533140071625911e-795995990', '-2.9e+741188938594895', 2, 2);
    T('546940057055797289064335133138697.8', '-6.834770410162734612312370677787348410125739360e-28125', '-3.7382097180976811083982154248178889806181e-28092', 41, 6);
    T('2.700257904695891110231087927921558834787243179459027185295e+720', '4.49393105653e+9869339', '1.21347728585534899853506276643759e+9870060', 33, 6);
    T('7.5758178829614e+514282658', '-7.4343763277733735045030554206610879059772140e-400578150976267', '-5.63214811326104256404005760667105395010996824185472e-400577636693608', 51, 4);
    T('-4.427970749858567798270768744425992805618878553366256e-99031060217', '4.1182216210614618253298241160860747516009546253176584571e-400216210313025', '-1.8235364879495287763726423862981195829156587680638638746885770576906700375948592851e-400315241373241', 83, 0);
    T('4.48504273974372e+796801', '-9.256655908228032456404760938307835756e+3959660', '-4.151649737550394745692185938986573450929067929245232e+4756462', 70, 2);
    T('5.6585688528651337040294e-6893549', '8.052515179285430863382794e-60708322046', '4.55657115807282369847455183034777542104724301436e-60715215594', 77, 1);
    T('-8.513591947978227544217614e+91140', '-5.6141398348536395129110132655078973574671571002055e-864001372', '4.7796495692833761504303361871036344513666796590652839571452546085496119677e-863910231', 90, 1);
    T('6133.176490', '6.42389060332572070651217658810230027714530390935e+52', '3.9398854822649226049486671348877441774708e+56', 42, 1);
    T('-5.6308758430887166609362542057e+17643629781', '9.654260465279248146796e+26', '-5.43619420368273523912845770754638657237316859799372e+17643629808', 99, 5);
    T('-6.97033434543834672e-81694503', '7.65315652137342691459125097191707350704e+6726581990', '-5.33450597519446602519134298692390029622471725726098809088e+6644887488', 91, 1);
    T('7.795337304689273e+5348664267241', '-3.76460631450587916148546907327753951990246e+7328', '-2.934637604093647764306232796813328708937512637686956831158e+5348664274570', 77, 0);
    T('-9.070073425915e-791', '-5.0697129905791694349273401589377507569948853232102607951662e-450163261053', '4.59826690728681874366866748144750856157209841736e-450163261843', 48, 6);
    T('-9.0819851852406369176e-783655773', '-6.8096842811692062754239908298489931543888045e-982751', '6.18454517577447673068594438324210174017110679849969553268290092e-784638523', 77, 3);
    T('6.40313860e+7641669', '5.613254271178718695e+7166651', '3.5942445095399321174496127e+14808321', 46, 6);
    T('-5.4974453746696302577951273250358797393520600350468895733e+43736', '3e+128397', '-1.64923361240088907733853819751076392180561801051406687199e+172134', 61, 2);
    T('-4.3960039788747541504972023358483417483002334364387264e-2343398835', '21744687932.1881', '-9.558973466928873786420976868779217155584894060096733691692392923584e-2343398825', 89, 4);
    T('9.481840e+798742097', '9.4555e+83038186', '8.965553812e+881780284', 52, 4);
    T('-4.4286541054997570328175304797170246495983951121851e-961482105171099', '1.929333578601e-5966562725451', '-8e-967448667896550', 1, 2);
    T('-4.4685378397118301e+925', '4.36e-623907221199466', '-1.9482824981143579236e-623907221198540', 60, 4);
    T('2.80905530004464e-40255109751851', '6888273842528128891686631853379004737358961.43741', '1.93495421455124984066059148206080074121381373997278828115659824e-40255109751808', 69, 4);
    T('-5.2860040804e+407737', '2.148544993403959801123992938372159820954222351887250019473e-29324349', '-1.13572176020563225942589991785760225473249527736848882436692574576292e-28916611', 97, 0);
    T('-9.4273717686267425812201692112228217355767e-27', '8.837177616428726713833463508438167330e-3082', '-8.3311358775460346716202651541524930436361341878975869409209214620007208649211e-3108', 100, 6);
    T('-8.8532329129170467076494176353685659548272e-224388', '-1.69611771041036445475448241953220787e+345813460100', '1.5016125137986542778865011730831827962162262e+345813235713', 44, 6);
    T('-9.21e-2044961014', '-6.472437028216343400e-468', '5.96e-2044961481', 3, 5);
    T('87510789094010656311547103038964434742140729489812313763290.6', '-9.506342e-314443731', '-8.319074898175354505420253105976352424954715866596413704451474889852e-314443672', 68, 0);
    T('2.82462457925134072346475517068801307442e+543151983', '7.8843278e+594082408570881', '2.2270266e+594082951722865', 8, 6);
    T('-5.575900587846101066215700249e-68489600328896', '1.989120194359793366988991e+24377966986571', '-1.1091136461027322641471623416001167320212479738958759e-44111633342324', 69, 6);
    T('3.46910029340035238151262042363812149667712598379671e+33375304730', '8.8476219657e+617992322927', '3.0694e+651367627658', 5, 0);
    T('6.3095796740264705784660101e-9860556207', '8.4105247883630868e+3576', '5.306687625255151566152839127913773131597668e-9860552630', 78, 1);
    T('6.4687537735915181842872818012975601181153e-250', '2.5748264248217960823823802052527961810363081589685662e+622654815', '1.665591815191e+622654566', 13, 2);
    T('4.42e-6683', '-1.642520412018749702076e+18806519', '-7.25994022112287368317592e+18799836', 85, 2);
    T('3.31522333e+57', '-5.4570436923918624217349351366205085242546714697783660434508e-7614', '-1.8091318561846845802685956e-7556', 27, 1);
    T('-7.0526704609764039011597585142064743509932918377435e-37873557390992', '8.5494498352008296667344e+31', '-6.02964523103204757299290921538144696360918001400837971787065268243098264e-37873557390960', 100, 4);
    T('-3.5324191288330e-313968282550', '-3.2923011048010344431268002859e-624006', '1.16297874004771935213871991504736533333547e-313968906555', 69, 4);
    T('8.300073087248853871281276801566421340567157286113652e-359331178934953', '7935205473023975894.71', '6.586278538843611343079360836220381738008122551e-359331178934934', 46, 2);
    T('7.02686910080663793132816378640046841120e-6410909', '8.12771053400973189158145399475e+409881427', '5.71123580117336037398260047392779242959315172285115797e+403470519', 54, 0);
    T('1.9373494804371406323075974643152614e-72187394', '8.4310434800e+344722381716', '1.6333877705520942077860046955979717290965672e+344650194323', 67, 2);
    T('-8847363446091726491.60371851601', '-5.2421154018368e+1499076593', '4.6378900186405346432005106749818877724207168e+1499076612', 53, 0);
    T('-3.205939794030921957140319284407180699400957273780247e-421162736754', '6.73329861727e-35537', '-2.158654998219927541363619404106518567493549712885888595093906569e-421162772290', 83, 2);
    T('-7.21306603003024409534834837633858823640395236261707507e+93468', '20349936841423283091851554063517237336913674880389522.1073', '-1.46785438144131245551072687584609118501038418591059406756191585735755e+93521', 70, 1);
    T('-8.5704462555512490155270127029364346558351261e+133', '-5.30687e-41', '4.5482244120197256863029837902832276982011755646307e+93', 59, 2);
    T('-9.900370573496240784349469817972827e-3011', '-6.40357840418573940298192e-758689', '6.339779919787651118146524680128111627000386114289333228784e-761699', 73, 6);
    T('4.51417172249727398642673001521576358932702063770141e+176536357535804', '-4.3e-6491', '-1.9410938406738278141634939065427783434106188742116063e+176536357529314', 93, 2);
    T('-9.866566728235143484868811231406004665804057579033416e-823745645867', '-6.24882580773840128184417622439335239003609142465052133976623e+465', '6.165445680516880569212607149903384039581281350359427153371766328413631955365460685414887453e-823745645401', 93, 4);
    T('6.10250153151559507738638360e+34', '2.5798569464731439877875955624700579514266630913925792e-810209513916', '1.574358096694350777781986719006106786123627019787590598997922933e-810209513881', 64, 6);
    T('-2.929682675052532789618282291784846170923672825871996574711e-6667623060', '1.61844678401028766827e+178', '-4.74153550360942832790956422e-6667622882', 27, 1);
    T('-7.68820606611180733378068755290968222445262110e+44222', '1.40682040680760927033e+754324', '-1.0815925185548142125902360839175709464e+798547', 38, 3);
    T('9.65180610785513672896943094407669441000681e-95246', '-5.996276009711617182e-9701102948959', '-5.7874893414919813879016394527643227559315701197e-9701103044204', 47, 2);
    T('9.8719746590e-995883', '8.3401392302019996991990778125855189028328891448032618e+840924534373', '8.233364313308590848161891876201339387913e+840923538491', 40, 3);
    T('-3.725411449832597757197408210016e-7299', '7.20373252169561738685412291e-33030450', '-2.683686761785630544712361239733411832459919268297276e-33037748', 52, 0);
    T('-5.532474385493617642420867768284506479679741278152e+51226028640', '3.60039704141986952659127652e+68600658151', '-1.991910440926243168527305710677610273978377576368250971171480253339806659104e+119826686792', 78, 2);
    T('5.643834784683613267996206748705196e-4842', '-3.07513539165531999099419095356488805818538879264711808739371e-81782428789', '-1.8e-81782433630', 2, 3);
    T('-1e+1627892053641', '-5.244582013483871314437444e+759', '5.244582013483871314437444e+1627892054400', 26, 5);
    T('-1.602511786167821668541825353285849122259491e+7549', '4.8844673008222861098380333524376295373182929694799992869e-65084074238', '-7.827416418719040434841521e-65084066689', 26, 4);
    T('9.87326435151994789e+439508288109', '-1.5245755218925746490323243564700945e+1765708221', '-1.5052537151501877159650877710837047815393402983375605e+441273996331', 61, 0);
    T('4.23698894135171382726371850394755702984347276785500571e-26200', '-6e+50', '-2.542193364811028296358231102368534217906083660713003426e-26149', 96, 6);
    T('3.4123546608084577e-7879054378', '-8.95985081519004760321125133950897231210429717353236625e+2482512724', '-3.058e-5396541653', 4, 3);
    T('-7.20578739702171168904e+1371919131', '-5e-773465', '3.60289369851085584452e+1371145667', 85, 0);
    T('-753415.7533347259078014904116897', '-1.248097747155244826626042405278155570991480e+18893693', '9.40336504408343040393348559778476821e+18893698', 36, 4);
    T('9.395054226816553373272965578485281e-5894491481521', '8.6863543901534106286496889e-89283961182252', '8.160877052883732529373277756084e-95178452663772', 31, 4);
    T('5.6753255195800240322813729207898185046376397541011629109866e-27057273601', '-9.1393573963e+755', '-5.186882826378383310916251556633896178809461873801937650895662124018958e-27057272845', 83, 3);
    T('-7.401893762167576335421387546730965e+689558487293', '7.457879748900236251389620054545228144169975e-388185244229', '-5.520243359238054922782117203613405095642994e+301373243065', 43, 3);
    T('-9.6374521356756802762715468678139756491821189853385e-6108', '8.91691462352333576870674647713393428526058966711343316e+48643278069809', '-8.593633788212e+48643278063702', 13, 0);
    T('8.495666281303451198149940712109e+810926', '5.728542630916258889651053135714329e-751208', '4.866778647048462190847950434039665182689509e+59719', 43, 3);
    T('-4.848333326321383812815694e+9542808093200', '8.2594464772790794480924926064e-828510', '-4.00445496127599150914036219097342237201590052608848416e+9542807264691', 71, 1);
    T('-4.1468951570423333440210675660179254093641e+59464436287', '6.70958014656427236383442155550691756116442227710894329709791e+76', '-2.7823925415574770219395334981206243293196743804608e+59464436364', 50, 6);
    T('8.38170499504168439361893727392211415939890139744e+83424576', '2.3373156632685065433e+676', '1.95906903698068089062871856597e+83425253', 30, 5);
    T('-2.72759345613291851241665952300391582513811951162346e-60209986079', '-1419203618885991864484884083770640', '3.87101050379358785347687447697325842268421e-60209986046', 42, 6);
    T('8.5219e-52343197', '-1.72938229219184128822852605e-69190051915599', '-1.4737622955829652274154676145495e-69190104258795', 95, 2);
    T('-5.593353312812214e-23', '-9.4859214239914844197561249669201069e-775258944243', '5.30581100219591238225051393425162716523911005056766e-775258944265', 67, 1);
    T('-8.575057444968639035e+4527224504', '-7.01343800331700e+1059265', '6.0140633765169427361313305679095e+4528283770', 36, 6);
    T('2.77554281982738244336078995101e-861175212030752', '-8.9972403036749027762025035185633570318273892516994748501207e+3164628', '-2.497222572312641437697073955592073412571933e-861175208866123', 43, 6);
    T('7.1344439924147647830598904048803341032450687266634691773148e-3829375838187', '7.7439521070829752781325279e+754943705853964', '5.524879258792579223427759408722349516437596663174395992532196157341611040029865808292e+751114330015778', 89, 6);
    T('-3.94375026403350250335794e-2078', '4.6531517941496796573846471e-2683454', '-1e-2685531', 1, 1);
    T('-1.1623485e+469816290478', '5.88002916181335625e-1741074346590', '-6.834643076190011917153125e-1271258056112', 88, 3);
    T('5.8434209815656e+40688389', '3.7550035743716796729413795559758529521e+105', '2.19420666723372969147414973034894e+40688495', 34, 1);
    T('-4.7620353119449881604989666291353721626259895e+16341522', '9.046072583834205633423491934436748746991516941083e-829', '-4.307771707863592648673541047937165446185427114549556528e+16340694', 56, 3);
    T('6.0104694e+46665070291192', '4.991901238225817104535e+5407916888', '3.00036696401783839968e+46670478208081', 21, 5);
    T('-4.5776641017750122750795902e+284', '-80966338619940186699.15575', '3.7063670175265998171866729532439695183425439597365e+304', 79, 6);
    T('-4.433359376279136664947342596182366330050143743e+464539', '3.22791105817928030549735583649419551509436642506125828488e+90', '-1.43104897555742221588356587e+464630', 27, 6);
    T('7.44669426699889917398479503224836407276287386743132569e-40335117', '7.5485114129950319316561013436701842e-18736676345', '5.621145666352586398720592914985e-18777011461', 31, 3);
    T('5.498130006066267456e+240320839515', '3.66947476785160284271275918865681154e-3991988071', '2.01752493276279485e+236328851445', 19, 6);
    T('-3.8392447146724839600504971801822129e-39369098', '-73453822214571648301827233410930.858160', '2.820071987097864917607576050633419972792864434509449186108e-39369066', 58, 5);
    T('-6.15421897205707206733190185298704790201958530259569e+64410433292', '-8.296316629664290701860010e+121', '5.10573492004725637696050466789065896613547221369491805133354615857640093569e+64410433414', 86, 4);
    T('-2341334030811877882227227660606516367659005594978831', '4.740002800674880655875092707860307183073786533658e-10858242976538', '-1.109792986336370848135212401147851565176797205758272249434852189694e-10858242976486', 67, 4);
    T('3.7413336423833807537635e+21966413', '-2.5940655025134e-80447673106', '-9.7052645350995337634682796957591809e-80425706693', 90, 0);
    T('4.3353429341650697835656e-206453', '-3.069833380e-1850815590397', '-1.3308780453047073651619054299728e-1850815796849', 76, 0);
    T('-3.5565782419416381783e-867150098', '7.18012e-626712020', '-2.5536e-1493862117', 5, 2);
    T('-7.15049e+35730', '6.8517620237107617946520743864481861113679417473e+87191', '-4.89934558329235651050417113795539e+122922', 34, 3);
    T('1.76246422926777198921166726812482720853e-12125', '-9.930903066e+32672257262', '-1.750286141815064378265106539599269059591079835298e+32672245138', 76, 4);
    T('8.11716453899389127527542809e-629630498481834', '-1.7263847749937176983e+949934912', '-1.4013349276237953239058331946735051760141965247e-629629548546921', 62, 5);
    T('2.343136799e+2083634028969', '-9.3084378605135e-73515638893628', '-2.18109432921740108862865e-71432004864658', 87, 2);
    T('9.19475685858028590382889617966838261263392154529e+27786901970', '-9.729424130e-4747731', '-8.94596892493540312150117218817303771894329191392714138477e+27782154240', 87, 1);
    T('-6.7362331964258296134432562924951874192093427468e+2903', '9.589e+96707322364053', '-6.45937401205272801633073845887363521627983875990652e+96707322366957', 64, 6);
    T('6.729448104558279260168330466746299114316496799147962487458e-6376439904629', '7.086866607633204459769279306297795139495323e+54132955', '4.769070105999463032585983220627248451905252105005105266839129005e-6376385771673', 64, 3);
    T('-7.394206301661885907880273773471454334e-54', '3.981254445567736224877566272029594603207194358563384e+96', '-2.94382167099363529295026305772760663462865763485314890749718028665312349e+43', 72, 1);
    T('-6.47028649317788489750817800e+28461978157158', '-4.4333365569e-401179', '2.86849576438218195707434152441123282e+28461977755980', 63, 6);
    T('-9.7441028506367959697691538136230319252611811265e-5533880', '-9.94153439776e-771199002072601', '9.687133366491697865381497972474165597476666964114545587664e-771199007606480', 78, 3);
    T('-1.4135726501250767125643381265992737e-256', '8.11124461816263e-524', '-1.1465833550708914831714461833874899751362786481831e-779', 53, 2);
    T('3.1094178796815066380868782872608322331356862444e+242556854', '5.86854891302823604643865937e+2913463221', '1.82477709179554682347949187e+3156020076', 27, 4);
    T('-6.86484910996567179e-73690207', '-3.532488039386e+98133594209406', '2.42499973731433631e+98133520519200', 18, 6);
    T('-6.48827098548116698239854503872255e+18969964532', '4.1640337425546e-346881', '-2.701737931438156650879172e+18969617652', 25, 6);
    T('-8.4313303788099799175510e+910136065167152', '7.590206034447144894981e+269533959150954', '-6.39955347196610416463975261e+1179670024318107', 27, 6);
    T('-1446566966321.373706311760609450270473851162066817624171', '7.22807677999908412685160303440590574195370112505037355012e+5112168786092', '-1.04559e+5112168786105', 7, 0);
    T('-2.083429065440397565785199251911411e+92250', '1e-142', '-2.083429065440397565785199251911411e+92108', 88, 1);
    T('-756093387314.73948845', '-6.329369050498274701282579312e+7732589871', '4.7855940849563169337142478331954653155554329464e+7732589883', 68, 4);
    T('4.509e-171', '-4.89191559959581319854e+8148', '-2.205764743857752171221686e+7978', 83, 4);
    T('8.360308338462898489951759799198098421e-5004', '-9.74369900202342509769916106519253905706027768035504576240242e+198', '-8.146033e-4805', 7, 3);
    T('-4.667472874650863810998028008e+98676000884', '7.866715327459478227430416714535163496587086629176662011102e+96228331', '-3.6717680403517302278791948585093935428227007064949193961403976113816728685649402944e+98772229216', 83, 1);
    T('-2.630943777793526579535e+839834529097', '4.741777328667356835137e+992', '-1.247534955853979251297612226164456652e+839834530090', 37, 3);
    T('-7.691338719034671996233000e-43', '-4.04659e+87', '3.112369434705851335323649547e+45', 69, 5);
    T('7.2940005449281231746e+34741', '-2.243271724771968033424923921503469218179e-242', '-1.6362425182908585586437781698876977076e+34500', 39, 1);
    T('3.8947690076790932166e+5947487', '-7.92279249032067067009821471092690419e-642517', '-3.08574466e+5304971', 9, 1);
    T('4.13442e-60822376194046', '8.34858243118337338590094e-72806071027', '3.45165461751331625941365643548e-60895182265072', 47, 5);
    T('-5.13028255726769760688e-655141958176', '2.5e-13714', '-1.283e-655141971889', 4, 0);
    T('6.16150207488372730657371487167948e+5056', '-7.25491107979995e+103549252', '-4.4701149671284334458079711211628166000558920026e+103554309', 70, 3);
    T('2.46010997763995152187127162835e-5617904572165', '4.86957178181e-72471145', '1.19796821272647379910007093687148096103135e-5617977043309', 88, 2);
    T('-5.488961194171213469493526e+267328756988', '3.610061810008325593510913680875075273252779e+3888086855619', '-1.98154891836951912072760271184e+4155415612608', 32, 4);
    T('-4.09345977055385228697052467422913872707138e-73291945757953', '5.7571897116837900302257047380612463141918910863e+60614538600882', '-2.3566824476224126138195249051943874680554459814123246181639798184444997086e-12677407157070', 74, 3);
    T('-8.2e-6222349', '-6.3175206622624921120279494545606689637703e-140437276547', '5.180366943055243531862918552739748550291646e-140443498895', 72, 5);
    T('8.955842509646043593666414e+3263711167', '6.75760082738e+28301024', '6.052000875306907981950370777411761532e+3292012192', 37, 6);
    T('-6.00908562265182195259551782141684381788280e+19620620', '-2.01668350e-24667222331960', '1.211842382529e-24667202711339', 13, 0);
    T('1.01672326153538919044e-17469272', '3.919497392791270886672570116624909439426038128599431e-26357650825081', '3.9850441727781953642e-26357668294353', 20, 3);
    T('-5.936e+4558169891', '-1.7369668855393666716256475294e+102269', '1e+4558272161', 2, 5);
    T('-18.319370', '3.77034006779363e+26175', '-6.90702547277365916131e+26176', 45, 1);
    T('7.078235903423888092944493192903644e+468534', '-4.4339294412008169949296437051311025440894929795710160417518e+831', '-3.13843985639558401815246297125080453864701413913846371179001867286027883655544340737783635592e+469366', 94, 0);
    T('-8.1121623218951782776877819206739418158600316047181421928e+2799775073594', '5.9412820981984294429885901e+7530', '-4.81966447807556279847116e+2799775081125', 24, 0);
    T('4.371658309416667671281539339543420819063023e-56249', '8.4572939453519540837995174339281184444639107078762062868e+85164514678', '3.6972399351377142973381903615316862144546706705041752713631368380545039436155723240728287e+85164458430', 90, 4);
    T('2.99976500390575296485961397022908156812547753472223835456e-45463243188053', '-4.01e-947', '-1.20290576656620693890870520206186170881831649142362e-45463243188999', 51, 0);
    T('5.08384320936620141053448432e+6620412713726', '-8.84911775782426349922276e+663769816', '-4.4987527221996748010851203717254374771974497407123e+6621076483543', 50, 5);
    T('-6.90577277533136392245715737238076829756684484880770498128e+77336724508601', '-7.641e+13570516024667', '5.276700977630695173149513948236145056170826148973967376196048e+90907240533269', 88, 5);
    T('-3.29886352658035919207813129218492971902084602531e-8026', '6.149004888742488163e+9015422179', '-2e+9015414154', 1, 4);
    T('-3.42926219511545962315861998e-929418369', '-7.734492603730338834e+635146', '2.65236030843725885394e-928783222', 21, 0);
    T('-3.86545068235135671458725038412157376285725e-5949078260', '2.7624402481559427956333333489977006638811035819e-700', '-1.067807654218924025156952e-5949078959', 25, 3);
    T('9.338350789011152195396979607e-34', '-8.869525642648114281936835589466194e+62', '-8.2826741783177664737132349965804228243838105132525733033905758e+29', 62, 3);
    T('-1.9e+6183', '-7.314967499365957e+3081856', '1e+3088040', 1, 1);
    T('3.218126703180e+146', '3.578989607941693395124213385943701e+99829497280', '1.151764202772088251131746591029782745221766918e+99829497427', 61, 2);
    T('9.724102994021221279969081279233225841227482494953e+72', '3.1e+271334844342', '3.014471928146578596790415196562300010780519573435e+271334844415', 49, 3);
    T('-1.181839990777896980681982594475414293086978791778356689944e+257976884101', '-2.69e+3825599101623', '3.17914957519254287803453317913886444840397294988377949594936e+4083575985724', 67, 5);
    T('6.3178751e+9458', '9.85064e-68167', '6.2235113175064e-58708', 61, 4);
    T('4.2e+12757', '-8.52202e+655885494486471', '-3.5792484e+655885494499229', 19, 3);
    T('-4.6803343113384612813437946271875135432e-407', '-2.580891753500058923119e-78800753', '1.2079436227756812048294829730909218114705175e-78801159', 44, 3);
    T('-8.939103843e-383800903349', '-5.632120816022732485021e+857', '5.0346112830749103932212161035703e-383800902491', 74, 2);
    T('-2.89710e-405250', '-2.0965552e-512', '6.07393006992e-405762', 85, 0);
    T('-5.3582705732402932e+9602', '-8.59e+2814118051488', '4.6027544224134118588e+2814118061091', 41, 0);
    T('4.62544218e-490', '5.8363208271698080742964889e-32489208706', '2.6995564530003720289355553583961802e-32489209195', 60, 5);
    T('-5.1939392935196436e-4990032972041', '-8.00253972255894205928090962e-51391530', '4.1564705512950676220782981663975843946211432e-4990084363570', 67, 5);
    T('6.42983116308054675889422244994722369441548e+37528765', '2.855573e+9840267792420', '1.836085226385140614993585148406314340673309547004e+9840305321186', 83, 4);
    T('-9.9666477775793650e-85859828917', '6.13888e-101851', '-6.11840547088264122112e-85859930767', 97, 2);
    T('-6.022507816e-58', '3.8808190730218814086585494893311125782260295245750337173245e-96691854543', '-2.33722632e-96691854600', 10, 4);
    T('-4.6238654795684e-94914769', '-7.252553554768852388282445390e+93116386209', '3.3534832020616783823156519038023433769676e+93021471441', 44, 6);
    T('-3.581052e+874201', '7.8690278711928e+806105', '-2.81793979961907188256e+1680307', 46, 5);
    T('2.593125728097656e-9', '-4.2316e+28258714', '-1.09730708310180411296e+28258706', 41, 6);
    T('-2.7852215904054033518731008035592659781557574436307328930673e+98052138973', '-8.93508047388243156760993817405530068517e+7117345011', '2.488617904786709109717257618009471137e+105169483985', 37, 6);
    T('-9.91385874529219336591978e+54101199', '5.4015130604484248583555671463986878214e-2407', '-5.355e+54098793', 5, 0);
    T('9.472623686948989794610847874591032866e-14', '-9.8960206662141150128544596424308678104034335151013e-13', '-9.37412797693165484377885878364286192871556915432094924150877020299820619263069756193258e-26', 89, 5);
    T('-5.549684462711659943193818315570e+360776430', '-8.944028227824814184324947752157675406736e-369', '4.963653449001387395732308126104109967983492361385370205643e+360776062', 58, 5);
    T('2.5753218781601599e+9241325497937', '-5.9469962787186682429562878138841724274e-60', '-1.531542962592124246240910143868575617085049432475514126e+9241325497878', 97, 0);
    T('-5.599012441868085e-6742233', '4.8521057875761414053153206544223310509065012680764e+522003136', '-2.716700067389895922e+515260904', 19, 4);
    T('-7.7762905931592402696999870945083890775411e+2311987900', '7.85261478564796197e+54276257', '-6.1064214489337410573249436021503903323563985273373513911967e+2366264158', 74, 1);
    T('-5.48465723478464e-24', '3.33814234253630445821e-595213212371', '-1.83085665497326881619956131212298944e-595213212394', 98, 1);
    T('-3.0089584230796672493085e+100396378501', '-0.00775765', '2.3343e+100396378499', 5, 0);
    T('-7.43429723095119203735763e-695119419586', '7.5135634776273e-75', '-5.5858064156300645114479191120313951299e-695119419660', 99, 6);
    T('504184083911610.494', '-7.845354343265768139029041242936681799986e-267675338191', '-3.955502792521425882936526135948936108208613329e-267675338176', 46, 5);
    T('-441511600346705545903242037218281975.62968', '9.985344515262545282306e-56', '-4.40864543694676510885941825208122030549930925944120574771244208e-20', 84, 3);
    T('8.405818328278e-36406878286105', '-6.396742273689449139415376847042568175837193984505633635290e-314463561305253', '-5.376985344544945810840726876665474693211e-350870439591357', 40, 1);
    T('1.5244464112114608e+99', '3.9992219880393463562620851407e+808523', '6.09659960730454516055805368008881056122053456e+808622', 68, 3);
    T('-9479545.8637', '3.09226e+95132', '-2.9313220492484962e+95139', 52, 6);
    T('-6.16412030e-52713812992275', '-1.44738147568772171621982559302960997371624648384e+2537', '8.921833536130641891701466200453357340066781390841765952e-52713812989738', 77, 4);
    T('3.728714510765818023829210308481856095116e+881', '-5e+38119390026601', '-1.864357255382909011914605154e+38119390027483', 28, 4);
    T('1.3943817561737882381101683253111557521609138252949e-1969989188', '1.43665860274140864564452267424e-63962065979930', '2.0032505455127461685636369287610011221128e-63964035969118', 41, 3);
    T('-4.86390235672393805055468e-798873697548', '7.26043354438e-23235652', '-3.531403982734741667e-798896933199', 19, 5);
    T('-4.38851570417622785286228588005447467115727e+88300786622', '6.91444495722783234510023e-72677344423960', '-3.034415028045646834089822783e-72589043637337', 28, 5);
    T('3228340905342485328130978900479117818597385063028529.22714', '-4.4217802340928119033918196728920704792139856711722759e+26353', '-1.4275014e+26405', 10, 1);
    T('-9.30458319887590718227e+69666813703', '-2.74167617443e-5572452872', '2.55101540693597490797817743233561e+64094360832', 34, 6);
    T('6.44247091e-414181137', '8.80376779267e-91242898', '5.6718e-505424034', 5, 5);
    T('-8.13099999448097257986606405153756546881433579297168120288590e+686', '-2.54844553467339790455544001e-273890', '2.1e-273203', 2, 0);
    T('-5.546582877029066685409622924e-55801', '-3.65214613912502891644918768958279289633e-188', '2.0256931239678700933e-55988', 20, 6);
    T('6.719031895873317083871895118096969697816009035974264e-2664', '-8.214114579484409231125092788718392563644871e-696', '-5.519e-3359', 4, 2);
    T('150126662809', '-4.40441964879749e+42824036336', '-6.6122082348435498355554941e+42824036347', 28, 6);
    T('7.6438334576856590835e+78340696', '5.768889930627358581899e-86183218004619', '4.40964339e-86183139663922', 9, 5);
    T('-4.5688530417981266381362707559656000975765015e+63', '5.47884719750422181343709793e+681336032', '-2.503204768386430533726699273588402913606902018e+681336096', 46, 1);
    T('4.6197942079523330075214510857333221557340765131009e-416787107931728', '-5.5387829957e+8570762', '-2.558803760263973177820365347664902762104997024180601319836613e-416787099360965', 71, 0);
    T('11377427448060548071188827952862423385082402.418', '7.731385658956552e-6152777506', '8.7963279407753962253506080823181855962516354016557063241742736e-6152777463', 81, 4);
    T('2.5e-8', '468555065243173557785310790733349', '1.1713876631079338944632769768333725e+25', 59, 5);
    T('8.25e+90573', '2.06199311580311868461089377349695701855558670717075e+902', '1.70114432053757291480398736313498954030835903341586875e+91476', 54, 0);
    T('-1.675059670248031680545630378340e+1970', '5.5444301939886851386152458101046098376343505870358354704653e+61104491', '-9.2872514124559172507058e+61106461', 23, 0);
    T('7.139533011422606955775178551706129939334362291421327e-2314', '-2.32765151192128248806873e+7457988', '-1.661834480844973807658145639171130322208445477906828679563956891511096380471e+7455675', 85, 3);
    T('-2.44685e+52199852232614', '2.0224989174032028211698694614800762e+35', '-4.94875147604802682297949509182252444997e+52199852232649', 71, 2);
    T('7.4671919250e-18976984', '3.46846714314199870042964864673516904574e-423705288498236', '2.5899709843397751824209e-423705307475219', 23, 4);
    T('1.6314629070435416227326270788389725002833423752986e+454406', '-3.8e+98210014407978', '-6.19955904676545816638398289958809550107670102613468e+98210014862384', 67, 3);
    T('1.7353753386377907784925027457023766214469275953e+9541', '-1.98771482578762766870394722e-1718568', '-3e-1709027', 1, 5);
    T('3.77952449818269023353324758901535826594148e+92519172517', '-4.1997467179485345754936e+564465', '-1.5873045606648835439960599517639501789455012075316679546739714528e+92519736983', 99, 6);
    T('-6.379787954641281813969643e+421769', '4.6927439951e+16', '-2.99387116141541864070728774819407493e+421786', 69, 6);
    T('9.31383802442322585916727708776985894781704537e-890554796290535', '3.120424954449962360979e+1471218837', '2.9063132593115171975369120857223789819140109e-890553325071697', 44, 1);
    T('1.49175879946475775356891386727097961954000179521161317e+70452985110', '9.4387077144769422983865312035821232457859981407568763746157e-530810757973080', '1.40802752886468709502072724068239789893741065944953657818616062562029336832922588947822037664835393e-530740304987969', 99, 0);
    T('3.28032e-513619656', '4.016622829174076160811737645962254000e-464485', '1.3175808198996305511834e-514084140', 24, 2);
    T('-5.62792583036251788182580849876932154e-65669873', '-7.678495413239488206406124459086805404629e+34478', '4.321400267449063154563016469361467219944286563073918742826029e-65635394', 61, 2);
    T('-1.02633912123594924675278181683305928524324361351491963e-775170', '-6.01338490017449498476e+61152340', '6.17177217e+60377170', 9, 3);
    T('-9.836205597536531e+898', '6.91254538120e-5145043024', '-6.79932175717847334623206172e-5145042125', 31, 5);
    T('1.9385964822646871444931281150542695e-1189421082', '-1.3520847327864635965357896860914546815512711925569327867e-611442', '-2.62114670670362783249935247944844929499569778307496974962509e-1190032524', 60, 5);
    T('7.868892231986065121766551807175578910970282e+72', '-4.8169910746350448631217447556213208756568589533648e+58', '-3.790438364874191257431e+131', 22, 6);
    T('3.71575878463817369710119856e-7096433804544', '-6.91331203240024e+1408395725', '-2.5688199915335978282639222166996857631654e-7095025408818', 41, 4);
    T('-6.48323858301451173256041e-67609', '-4.8e-456746744', '3.1119545198469656316289968e-456814352', 68, 0);
    T('-6.8884375563235181e-46607', '1.15178726023328235170478288290259196251617272483172551654209e+567386', '-7.9340146202859114986701040985992278835293174919022152476414146695653e+520779', 68, 6);
    T('-1.86764602e-3874699744', '-2.3697432701087659127834152824827337e-75479601154', '4.5e-79354300898', 2, 2);
    T('-8.60506077304775359554471439486266440140916850987008e-41417397456', '1.7903687900173606869e+78915599553', '-1.5406232244267360982909893710946451702086141005094838680092394018557952e+37498202098', 88, 6);
    T('-8.8657008480056197109996440e+70325', '-1.28373164220769212450783431871197126001590720009463868602524e+7105640140', '1.13811807089323828612023339253003e+7105710466', 34, 3);
    T('-3.254700940058661468391124736375632875e+183885', '-6.6187978707630757317682066067000728083051580846932759747378e+682741421946260', '2.154220765203085e+682741422130146', 17, 5);
    T('-6.09e+66505', '-1.6997e+864322', '1.0351173e+930828', 32, 4);
    T('-5.3081066e+5001886652', '3.8420049403789521246667962991892640636349525868e-90533948276', '-2.03937717812581222740278442365821072253235118166802e-85532061623', 51, 3);
    T('-9.991598461133053556296779673572787097880217e-10', '-8.70426400798972842592934049145823853140e-303', '8.69695108675259955255326453587258522231562233328867642360180562767040456911933138e-312', 88, 4);
    T('6.675504417763249232e+180270825', '7.90367086211421867192617356e-153253', '5.276098975659013541843650079531632645036870592e+180117573', 85, 0);
    T('7.11636618447450314521845e+68473463749', '8.767227661940977284653695425e+69760812272700', '6.23908024650262316528588916817982534028672114e+69829285736450', 45, 5);
    T('-4.48672370942e+870313388', '6.264407119332285811698259746622338858043841472405e-545173648717342', '-2.81066639477676099906304516001240495169986712160553961685551e-545172778403953', 92, 6);
    T('-9.3672218260291762977195042940699729970e+528228542', '-6.0763721227635934312873876883173993371131772886719e-31494599', '5.6918725571426369869853696129964400790711467948645935958896e+496733944', 59, 5);
    T('-5.077315020870737726651375806442830e+311573621686', '8.75292288921287392442205629121013e+1772023', '-4.4441346861923820931942461203215585485531839345180568e+311575393710', 53, 5);
    T('-2.2408185012410386958673e+18662', '-1.821397550669229706332216641069575442828524549044293e-22059753', '4.0814213296547221476372335024137547404280509417742110705070722233037503189e-22041091', 81, 3);
    T('9.4794800859834473725645449817776174006360633237184934713896e-730674', '5.2325538421972406096987820909e-6566225', '4.96018899459449163e-7296898', 18, 6);
    T('6.84976e+44', '-5.5656424538801636016864972998394240723621405887e-82870752', '-3.8123315054890189432288101744548093433903296118853712e-82870707', 53, 3);
    T('-8.60478988496490166577764419e-8188648416722', '8.255643548618786606660012e-61582881801', '-7.103807810103068137747148717225893559723895363713028e-8250231298522', 76, 0);
    T('7.74556441397319e-7446158133', '-4.6011e-79340', '-3.5638116425132044509e-7446237472', 28, 6);
    T('-7.20573058524642806952987e+97225', '-4.7257612e+12', '3.4052562017410862209575161887044e+97238', 94, 1);
    T('6.6198e-186186743028', '7.6829574942034751485080632220935126763867106870e-3879138', '5.08596420201281647880936769176146352151447474058026e-186190622165', 53, 1);
    T('2.54404820550354693209778497209381528553922273804452258078e-167677237421660', '9.7314294745386046456646422811988847876892617372e-853030', '2.475722569168426180883766827193e-167677238274689', 31, 1);
    T('-3.67667e-754', '8.13870540e+766751875471', '-2.992333398e+766751874718', 10, 6);
    T('-31451671743989786826836464411270741284771631077', '-5.98785685441753177962985930584160e-4208', '1.8832810823513945083267385094602164897697981264465908821129802e-4161', 62, 6);
    T('2.7123307155939435351541680089223812207745e+519589270835701', '-7.33950017553923038813332626086663276388229973424914616821855e-43812410', '-1.99071517632221949497197994889203510213708783586418921043139611152177351209385812e+519589227023292', 81, 3);
    T('-6.483366278074389e-25627617823', '-9.197774463012761956788698e+71367241', '5.9632540786830712398814194133929598455522e-25556250581', 77, 4);
    T('2.72705131344581301632995433619357819346762719661342252608e-1190', '-9.7379720315824943753301042022436248214e-8945', '-2.6555949419025633438395600640976676359822562912900802320481294471201947067917190335918976842112e-10134', 98, 4);
    T('9.73432370898283063712492813401667520012581650124587924e-4299768061158', '-5.9684395582257183351510519002335e+603087153', '-6e-4299164974004', 1, 5);
    T('-7.6621718944110476e-84959017', '4.340372550262368130117208326460585272534610557054081569009e+3827159838406', '-3.32566805658935191323566343549503e+3827074879390', 33, 3);
    T('3.8293671413104274748560275804748915981974262822787280931e+71411376418', '-4.2566378804901319624028380429853506932202330554761456815558e-9225846064', '-1.6300229232006173660196340379761961916764194191379905313191532716280848270784387431831026827659e+62185530355', 95, 5);
    T('-7.10005555630181936124713878827174068322324274454505217885484e-533', '4.83934876651757e-245151401359470', '-3.4359645098595428803681381962854265170674201503748405240440963903338e-245151401360002', 68, 0);
    T('5.648326043108257e-335043932932813', '4.45159012166752016602642804e+1242', '2.514403241745810853279611805521096874032628e-335043932931570', 96, 5);
    T('-2.08574714406406953765038e-15864926955', '-6.98262903e-2234708440', '1.45639985573813641335362213785314e-18099635394', 48, 4);
    T('-5.62357727001225772228314595620626523421506581549380214e-39070704689', '-3.463506753117863851764774841e-9679088045', '1.947729785136757549164455652228266046201666998277167981846906112570124900810395974e-48749792733', 88, 6);
    T('1.48875744324e+104', '-8.84036192264855076895809496982e-522057941', '-1.3161154614e-522057836', 11, 3);
    T('5.576173366108e-27300253459420', '8.1470793793295048320510991834495020849672804033354588780e+7086625943537', '4.5429527046584880355419228939636980243968564191709389642452072906824e-20213627515882', 85, 6);
    T('7.8661439e+9', '-8.35761062408357497112e-61788541954767', '-6.574216e-61788541954757', 7, 2);
    T('2.9994871054654675182186817560772794790501905708680178015181e-50586072853', '9.695361052398297723982807622329931602238776545403949e-692561588317228', '2.908111045950079899433118287462338664411e-692612174390080', 40, 5);
    T('3.70276749789764819e-1907', '8.5817236983885356591585674949447706010918238123e+780690101', '3.1777e+780688195', 5, 0);
    T('2.9667484282561162075630661682407802e-96799253', '1.1975882297989449e-5080768', '3.552942998454e-101880021', 13, 5);
    T('-4.1796062518372975147331434368125868e-101851726914', '-8.9626308e+18293457309353', '3.746026772458951929431072514759434428135344e+18191605582440', 45, 6);
    T('1.843970397426843500297006911242199841324112226444096e+608', '3.9012408256494720459518342813e+42054127', '7.19377259573068404093601201145134e+42054735', 33, 0);
    T('1.446298522375158834170e+5957476044825', '-6.950029105935484076881401435434567450e+1397386587222', '-1.00518168263788368645784288988751185425177057518602297665e+7354862632048', 72, 3);
    T('-4.7619427831e+5594', '5.3892923005618556717412586122079425506763821489855667e+277268', '-2.566350157667692469119208905891433358569280399797948833312868277e+282863', 73, 3);
    T('3.4677617372376872799322152163894547795063436645594370317e+33591672', '8.590486655778236019263713770378449233617937634061006646e+5452', '2.97897609291587062314638224489840059964285e+33597125', 42, 4);
    T('7.3151383940605842e+4951311568030', '1.3381520398766876500608904443795502e+54398959311', '9.789e+5005710527341', 4, 4);
    T('-3.2365695756160002658145560186355830908400156234e-4828636068555', '-8.61948548711957384627e+938619346466429', '2.7897564485074872449053574592314698850978751772405473069116829814718e+933790710397875', 79, 0);
    T('4.916247888861512590920980157031497463965371448471907864215e+134', '9.000506766207e-77898443716', '4.4248722388048923424572805181034378292037686886715458982629401706582505e-77898443581', 97, 6);
    T('-1.57432294194794816e+499135858562530', '5.4927961565165374097302966476781239422126731523e-69954067', '-8.647e+499135788608463', 4, 4);
    T('3.251708623088e-437354048856', '-9.7142157783827827698356580326e-322659206', '-3.158779921310480271590812e-437676708061', 25, 3);
    T('-1.230963103756423346355722495440191928286144e-21952697990606', '7.394248388378360695e+48965898652', '-9.1020469461041581288930041845253533967423172237e-21903732091954', 47, 6);
    T('7.077474155446354781253655259278451349841065e-722', '6.32407710684964328510658003076682853202551882440449966308e+62', '4.475849228077830587497023813487806660646168091675323348026452901471344412665234018062e-659', 85, 5);
    T('-9.61365972230959891946347091391609718590434010149164237129779e+23436', '3.81773597444293134619740462558718784777e-706160349', '-3.670241456791439740303357946308339688417307290590656322516509730552376454e-706136912', 73, 5);
    T('9.65983345680392387613450530375682452447549269559770462512e+465089', '-9.551018e-88230245', '-9.226124322293649941159043057727689865610687129452219763320437216e-87765155', 65, 0);
    T('-1.321314346406861576e+2802', '5238270235093634020920465438192182092938092181.7', '-6.921401611985262168479961553799424043279808286762419e+2847', 52, 5);
    T('8.2537008530142e+55440848', '6.5896440319016914416120254423230879902326942e-834506037505229', '5.438895056716692280902897810683066440785863163648945685764e-834505982064380', 92, 3);
    T('1.753092e-902454321302', '9.754179863868554734492008334116364192242e+3095', '1.7099974685909052356600063874472725134505912264e-902454318206', 77, 6);
    T('-6.1e+8131', '-8.8412987430018572441308044239542257e-283795916396', '5.393192233231132918919790698612077677e-283795908264', 39, 3);
    T('4.07950095131897965e+408', '7.094585e+763', '2.894236625671336324019525e+1172', 51, 5);
    T('-5.79490575544274023067303247865020e+685241371', '9.1660526243808997974074545451673446286869645913586e+462351', '-5.3116411107715909801354346724827530278779e+685703723', 41, 1);
    T('-4.18723223580429598336848194175797494794564456072302766166241e-6643318', '6.431062e-46209898', '-2.69283501e-52853215', 9, 6);
    T('-7.091769401144435731847953122791408e-811437', '3.8327258148405211307400e+935967966961', '-2.7180807656662382007914885628176642e+935967155525', 35, 2);
    T('9.6576546042729827351939591970868e+594979', '5.35453846001773063854602083392782225176346e-25', '5.1712283012147002e+594955', 17, 1);
    T('-8.564983705820388995150703651465619046498661585555077955503e+757484241599297', '8.69e+95878702089', '-7.442970840357918036785961473123622951407336917847362743332107e+757580120301387', 82, 2);
    T('8.93924270178014121735697e-30177', '3.3e+28315306059', '2.9499500915874466017278001e+28315275883', 92, 5);
    T('-3.46751467452547971727216504500010890128862227501120544539e-187759629', '3.910732743818176233897316985431770659242342399521534e+2143000219', '-1.35605231773368196155055763562632702136e+1955240591', 39, 6);
    T('4.42300488313180765366567309356e-49', '6329654556.31', '2.79960930110966253773114581700950269183636e-39', 89, 3);
    T('-6.45032885346418108167459591e-74687360', '-2.079243703347073738e+78542', '1.341180565308334799662469981806274003212e-74608817', 40, 3);
    T('4.452804464137458164961161443021309829002396997e+1226', '4.7025491104360825105692969651714e+355', '2.09395e+1582', 6, 4);
    T('6.61271049004703605423505213403726552763890147144652541849e-190308820744203', '9.7691962997720601029347918787587717886767585828625618e-85276074218782', '6.460086685083139089857632729e-275584894962984', 28, 4);
    T('-8.152402389515115693635910465e-93640511377', '-9.373740435219722e+8014', '7.641850392277972217896704634424722179419073e-93640503362', 63, 1);
    T('-5.784250259328993082778494407880544373780216547738e-700138262050', '6.38e-764009565801358', '-3.690351665451897586812679432227787310471778157456844e-764709704063407', 74, 5);
    T('-7.1139876866259956e+438', '2.49553676919621530333514499341825083524007e-66', '-1.7753217847584295e+373', 17, 0);
    T('-3.995443e-240131780', '-8.90069605e+93383841170380', '3.556222372810015e+93383601038601', 28, 2);
    T('5.0221995e-83912295433', '9.7073716027301523467e-213', '4.875235680954556975052056665e-83912295645', 39, 1);
    T('6.279139794312556649350068068e+80742229329725', '-7.578473711579417930696e-96528932159', '-4.7586295862529904070469224733014252061267306615328e+80645700397567', 66, 1);
    T('7.6621152923e-6843224', '7.3285886347310756314307414669962414083567865820525027e+42382', '5.615249104914895349361336272259963704159e-6800841', 40, 3);
    T('1.2032133014155915229422631041e+3202657', '-9.2812433738115728461995942094737394453131210617745108e-8023', '-1.116731548104540558479791874532259407e+3194635', 37, 0);
    T('-2.4566227623640676507089897944e-541', '1.719512645074e-83277620596110', '-4.2241939040616345034444921727992154327856e-83277620596651', 85, 4);
    T('-5.722107e-4652', '-8.478810907e+232233', '4.8516663242621049e+227582', 28, 4);
    T('4.11811679884905987978035483706923897404036892801003791587e-66770', '-9.9526924830904634438e-162092718365810', '-4.09863501083e-162092718432579', 12, 2);
    T('-2.741338e+6429', '-1.576375273961518001523130724199964071013490086541563243e+906', '4.321377440771119835259416133216881106504e+7335', 41, 4);
    T('6.86e+4428525', '-1.32706529604130893e+202270177', '-9.1036679308433792598e+206698702', 83, 0);
    T('-6.61499348372903143e-2582168076', '-8.20621554022550988643118836197285444506086976719e+332660', '5.428406232466766e-2581835415', 16, 1);
    T('3.42419846759131e+7238', '-2.0314376623977680345418079154531960416239235660e-48', '-6.95604573058971025225208592800821593446573661659501556581146e+7190', 78, 5);
    T('-6.05860801356359120897452536457452179e-221924723583928', '6.80564391694586086420847e+4598586007', '-4.12327287726685002020781832588553271233842211154290771175613e-221920124997920', 80, 5);
    T('3.72e-983607144537', '-1.4682549949971492571133135376155366682438775328754313417e-7852951996601', '-5.461908581389395236461526359929796405867224422296604591124e-8836559141138', 69, 2);
    T('7.0908247464950630255153676297613072504487964902878e+387900178', '-9.8186717297135932536896635771916070563510428965682235282e+10615098253', '-6.962248047876463186889554e+11002998432', 25, 6);
    T('-98272106796809656', '7.002244927012437650300358190844557253497275067518412e+59', '-6.88125361284784908e+76', 18, 5);
    T('9.6e-5665776', '-9.58926280265885358527433307260007083295379784785e+51242355881875', '-9.205692290552499441863359749696e+51242350216100', 31, 6);
    T('9.467173154632748640329981726e+27000015667455', '-3.142525662919027025785945101917850459869310964676837255e+46134654526474', '-2.97508345937315e+73134670193930', 16, 0);
    T('-2.6709698804833032206546783e+2066', '-8.275671883783727e+66752', '2.21040703423488541256568038429342679600241e+68819', 65, 0);
    T('-4.7853409979620303895654025925581492e+1045828069', '-2.4e-91', '1.148481839510887293495696622213955808e+1045827979', 77, 3);
    T('3.998e-4475572332816', '-8.9656593381985867890884626274386669907187211585e-171907247351', '-3.584470603411794998277567358449979e-4647479580166', 34, 2);
    T('1.572319525612747661558622943972389887703919831233994829e-8094823901506', '-4.37067885608769629965547484e+983886847', '-6.87210370560947325271000687231608460911911227085080019057590639223936763409960236e-8093840014659', 81, 4);
    T('-3.47457e+478230358671', '-5.741591510523608915251299746158051535823079665212706201230e+4478768736', '1.99495616147200158286647085590083811248247e+482709127408', 42, 3);
    T('2.21606053764413887621914667765085675018e-328669042731915', '-8.78171421369301470265488914597440503907553982164974825081e-69156594542797', '-1.9460810321833718440000214760133236e-397825637274711', 35, 3);
    T('9.641283717314948963430600341647e+220453095227866', '-4.28e-39551', '-4.13e+220453095188316', 3, 4);
    T('-7.049832350e-6265800800', '1.962825200734485968e-1192', '-1.38375885975332229378274648e-6265801991', 41, 6);
    T('1.402522359736803759415320054486068108426e-1559693', '-6.1584464795335755791983096278661641510339070444902982169e+13112407231', '-8.6373588887882421e+13110847538', 17, 2);
    T('-6.2824377230839550031984566672e-71838759814', '-4.42702e-34389957', '2.78124774488471304782e-71873149770', 21, 1);
    T('2.382139044169709771439240997852e-636417955897', '-1.076432406700018463388618785211343396e-34549', '-2.56421e-636417990446', 6, 4);
    T('44596195984675133398826303141195173042494356857260746.2', '5.92954432045216265717271e+55020712353585', '2.64435120614701978766e+55020712353638', 21, 6);
    T('-1.940519573148e-4971001146542', '-9.47827714e+3608601', '1.839278230989124623672e-4970997537940', 42, 5);
    T('2.1052905848e+755', '4.30611672881537502616881713176420607674789143356873564902014e-141363107', '9.06562700622e-141362352', 12, 3);
    T('6.267318452973173943868450489633864345052722e-570687591814647', '2.418302964870e+929558195561', '1.5156274796609488215031304776334377346396378329e-569758033619085', 47, 3);
    T('-7.586758632148582918327589613662294660078567e-50502959', '-3.6508640039052776150294965483350e-993410401', '2.7698223996428902684298800544908837847967140041830295697185724846413e-1043913359', 68, 5);
    T('-7.1e-48262632370', '-9.20962615447119637199e-958011', '6.5388345696745494241129e-48263590380', 52, 6);
    T('-6.07e+639136868902152', '4.99142694175123598009671337944973222337092436410672e-2520423869872', '-3.02979615364300023991870502132598745958615108901278e+636616445032281', 52, 0);
    T('624420970446856041042293590165.4320484944251127804172', '-4.847010904460461047e-9161673845104', '-3.0265752527296945172709839472177657807324990491152096871631127850088084e-9161673845074', 91, 5);
    T('6.2965600707915580021054968116414990980291605617891615923812e+609668797130678', '6.10639445482025214462130149506577039361695290859348e+4294385365', '3.844927950072418407587995546760395105676148013679713597882578021121562412e+609673091516044', 73, 5);
    T('-1.499142388313e-7506031009', '-2.16807452577011e+3703', '3.25025242260357757098872443e-7506027306', 61, 2);
    T('-9e+724', '-4540473783439011226.615602', '4.086426e+743', 7, 6);
    T('-2.744475516125e+14', '7e-65484', '-1.9211328e-65469', 8, 1);
    T('-2.2531795970e+81814642', '-5.926501819646258842728e+24', '1.3353472981610324181815561420616e+81814667', 51, 3);
    T('-3.7380227185712832387711805690331716455e+476231559543', '-3.03505770198954713688992127489543786542156499245352842060e-293878172993', '1.1345114642211678590076386646975582542232122725560149308991443901635227e+182353386551', 71, 3);
    T('-6.7652611633878408622553002397688465017536e+214', '-4.5131780283842054204044783881597618881149459e+97955528', '3.0532828038882971431316711520890798994284612e+97955743', 44, 1);
    T('5.81158761015391301135491280955372885662e+27436699871', '7.3973145e-56908110181', '4.299e-29471410309', 5, 5);
    T('1.538088706e+14', '4.40690645218248551e+210859', '6.77822e+210873', 6, 2);
    T('8.5918634433261240399874418823832614580808649996434343126344e+39439926653678', '-9.32145455522384391085803495100553121636440380e-1722891183', '-8.0088664631653519597748244416235557689383244947098642993237844614195649416e+39438203762496', 74, 0);
    T('9.49677622600068285503148068638914238207e-816', '6.08713026352301993963901330545564219e-7', '5.78081139711946873940154616319655268611945213933577568623011754685381915333e-822', 77, 6);
    T('1.0684534434453723016789252822842328919719725960740e+9958670', '-2.485559660373563385570725561549152637447e-4895240', '-2.65570477801504389239263998489970834812977263e+5063430', 45, 5);
    T('6.37279450034435807925181680836006567442271e-1367626267762', '8.6445065299818278130198414153914627540048811776957201742e-473956143', '5.508966367245908305150420393771e-1368100223904', 31, 6);
    T('2.536843608531e+6466328', '-9.8116782367283762576999e+73764648892', '-2.48906932239e+73771115221', 12, 3);
    T('-4.3613749402353891318985409426094605522257312e+1112120495977', '4.7153e+55868923647469', '-2.05651912556919303736411901066863893419099903274e+56981044143447', 48, 0);
    T('5.7092608554291332966084959835131580689084959219131e-46027814189349', '4.29383694766006322777251773073641827448591996842e+103', '2.45146352048709112e-46027814189245', 18, 5);
    T('-1.67362564773360800848990694566080341064448896855007397e+8676280598936', '3.6808761491719766313e+89242995', '-6.160408729385e+8676369841931', 13, 6);
    T('2.302133382733179058141420444900049650e-705', '2.6545935494542948e+20', '6.11122842778689234068620367631092672340148692573682e-685', 99, 4);
    T('5.3070600856661573991702e-6223771', '-6.8586058529032695344e-74599', '-3.63990333653e-6298369', 12, 5);
    T('-2.436580215821421636149679e-1703793862528', '-8.8899204245346933370622e+42655045753065', '2.16610042266480073465398194832848601217573330338e+40951251890538', 54, 1);
    T('2.293963e-559283768955428', '-9.4110464719505e+88997', '-2.15885923979349848315e-559283768866430', 91, 3);
    T('6.76e+343', '3.90e-5764858663', '2.6364e-5764858319', 14, 5);
    T('5.959788655597077341611600147828809122380e+600256996283', '3.814457e+893785', '2.27333575558628608452518e+600257890069', 24, 2);
    T('5.1750652807117569838410246986e+43261902851', '-6.1164236714299e-186558', '-3.165289178414041070814279229476593707852814e+43261716294', 78, 5);
    T('-2.096e-3609728413702', '2.807361e+4442902131', '-5.884228656e-3605285511571', 32, 4);
    T('-341494', '9.2593824729382854851701055098629191791261275477785070e+3747891', '-3.16202355821358686347268e+3747897', 26, 2);
    T('-6.6660717904211840954544763975934955084e-4640', '-5.90075126366755961657979368936098067917142499e-22739', '3.9334831541026473681715458652043862703998099317783e-27378', 50, 6);
    T('3.8351e+287', '1.740249416596810145649494561907767959e+18925973155178', '6.674030537e+18925973155465', 10, 3);
    T('2.6271201416e+400216', '2.03232677e-47405682', '5.339166591779870632e-47005466', 34, 1);
    T('8.087734505954103773457908077124743671427e-448058', '2.402374685550855917796846450648673781e-86963525500', '1.94e-86963973557', 3, 3);
    T('9.29319262412758148787047849862704797790953e-916854', '6.6309316721263438862765894938824584654925273e+55', '6.1622525306498509528730455159759962904440225626964419713395010993272521400201690455169e-916798', 95, 5);
    T('9.34935722732288018934451928055193486344516178369950e-867', '6.1101642079789471364925916878e+9498', '5.712610789799755145081631466473188430223755769999987247494589177540859523e+8632', 73, 4);
    T('-2.43694287027608838516148447e+48', '-0.000001', '2.43694287027608838516148447e+42', 77, 1);
    T('-4.668e-9783185865286', '-7.9045469e+85651288', '3.68984249292e-9783100213997', 81, 1);
    T('9.483e-82583302619', '-3278002949513838567514439759627114890034', '-3.10853019702397311e-82583302579', 18, 4);
    T('-5.541017747925495787911460835863078811914953069e+865', '-6.89851969591151983630e-88093', '3.82248200694593256760711980776160058073322791793479104659770626047e-87227', 73, 4);
    T('-9.74348368331387727322917782619739441279617811594591711194e-5462', '-9.43e-707', '9.18810511336498626865511469010414293126679596333699983655942e-6168', 96, 3);
    T('2.1338793030968385e-10972809', '-6.695663165778642424440513013e+3385215193', '-1.42877370499629009264306071375620525584094005e+3374242385', 63, 4);
    T('-6.0506664061820963594804183379607926643918694197608e+467485100', '-4.462e+540737855957', '2.69980735043845139560016266239810568685165213509726896e+541205341058', 83, 6);
    T('2.10017206289754835405850568899630700e+108', '-6.3754294593990033788414755673353523204640891308233725e-6380353713', '-1.33894988396038064245861531184624033083575005384412928088952345303406236642560217853575e-6380353604', 89, 6);
    T('-9837277289.64103442425894137408513309898933804149042', '-2.515406881392423731520437504e+2091', '2.474475498852846907274100487639313457286679019594825128058771943008254262471168e+2101', 82, 1);
    T('8.987147392429622334197154359737e+96466764497', '6.60423224873765411328621484289260567607661105673845e-54268004190', '5.93532086332422291307010709692430147918535845650417242657470755875943441e+42198760308', 72, 6);
    T('-2.3214675454113566458637895521396747180144544641e-23066906229', '-2.48144135171435e+9468', '5.760585563846551027119673128213957439541308557737910545529835e-23066896761', 64, 4);
    T('-8.2002325443054418708190797000868605726374697497e-244567145417678', '-6.8986860600598224796271628509656757588641e+49', '5.657082994264884246067961599598911890907197547345571810582106956947769752555965538331577e-244567145417628', 97, 3);
    T('1514531348636902642752402742.27563', '-5.766118336890802236539039201327929636091930565615910477e+2609761833', '-8.73296698117120084729572715512569021981570685066e+2609761860', 48, 1);
    T('2.797032171516597e-9439', '-8.64736151323403003850e+1027791560', '-2.418694835125002529e+1027782122', 19, 6);
    T('-1.40267782295052836629721980074727707292078831531300244365e+36181589506', '-8.899985580555572342037303520419929645969988390468931366627e-9685928', '1.2483812398424784516646625436153436941573867660077163308593e+36171903579', 59, 0);
    T('8.30383342703921632446767475869202e-409171012', '-5.74039648188167717238109e+8372', '-4.76672961907073880138475068458114646065323e-409162639', 42, 2);
    T('-6.077101153192737364036706497079e+38', '56044058283650581385600968701781169200271713574.9', '-3.4058541122517393325106634291052358058317433687e+85', 47, 4);
    T('-4.719816179689097145345021772691962969337692e+539397339865', '-2.3e+84', '1.0855577213284923434293550077191514829476692e+539397339950', 44, 0);
    T('-6.24640892852015693056852e-8914789772955', '-5.12298297807293333912138669619637453190765388844e+4184', '3.20002466148915541458658741393275109805334297384543671313136522778559088e-8914789768770', 95, 1);
    T('-5.153618265718180159313003265930944453939914808015444303e-414488', '5.640053014362328216173812678577074577786208260232917e+82510939', '-2.906668023443657619518675061042435761379460277615630247117604066384e+82096452', 67, 1);
    T('-8.7198723182278705086618443833277e+589', '-5.84234583147830992566301832243009568178878765528669221e+71714', '5.1e+72304', 2, 2);
    T('5.34082826433537022794069829692599772313453137482e-491250701', '-2.8125431766102220699053775447046011e-86779244315743', '-1.5021310092303460988500050038721904956513984451938441452311393514623561297e-86779735566443', 74, 0);
    T('-7.0746423844553170022915465868393668663129828244647e-45948330135', '9.2e-48642', '-6.508670993698891642108222859892217517007944198507524e-45948378776', 53, 4);
    T('-3.869568422530854151763e-315384836', '5.585735848598530875272022425125140463555e-101163447143696', '-2.1614387056e-101163762528531', 11, 6);
    T('9.62800371356736e+51150506', '-1.3695138291274603425152091228471191857334994e+1974855151', '-1.3185684232621043094090441093672965699534210392901030419584e+2026005658', 71, 2);
    T('2.676998874447053242861304770673037504e+73', '-9.320372787417e+43995323753', '-2.4950627461342253247749028202167418516282660287168e+43995323827', 67, 1);
    T('-2.739363211478019275100968392045398e+29008756342', '6.270763832557444614221529e+27', '-1.7177899751e+29008756370', 11, 6);
    T('6.7e-86807465', '-4.4371947543544057058e-5610', '-2.972920485417451822886e-86813074', 38, 5);
    T('-9.841703262668e+86041789145577', '9.3058866360241309544014188635961662903642723207375373227037e-998520606', '-9.1586e+86040790624972', 5, 5);
    T('-6.9152560690828938790319649167506e+8756', '-6.99316360581488500447195672814619690533025509948924623544e-87186665048', '4.8359517067200997675702949571061182622012510172169308191062934378256403748255784047361264e-87186656291', 94, 0);
    T('9.8e+99509656631', '-6.8058786759755574199465871073653501859e+742760', '-6.669761102456046271547655365218043182182e+99510399392', 50, 6);
    T('-9e-41', '-4.6908821119747713930328953257001789994e+8242750', '4.22179390077729425372960579313016109946e+8242710', 61, 0);
    T('-5.0855891205419334859816146378938312387004248988829e+58', '7.06517395330443394399053494078820765294971957e+34118', '-3.5930571791661271662897731202901298534354279194727062903420435625210727969147962925735161268353e+34177', 97, 1);
    T('5.58679408932241281708161159459747238264113305e-94094083', '1.00525615644826563080728989370342350847546823633044483142e+62801348', '5.616159153100137129785142070951254222602349987017128067017e-31292735', 58, 0);
    T('-2.1011568199835041362021969842721173379757462459978602322e+2854387071', '-2.0042682796905354735086132826349970083070953e+1053371400079', '4.2112819649483739630585200941186287220921799225024685079367608017899326865932604883878e+1056225787150', 86, 6);
    T('4.1031816040439277489487546982817373551847026487304592232e-707034', '-9.4128841344971738516966734662835473990334405296668579653878e-706', '-3.8623e-707739', 5, 5);
    T('-6.005455158997869871108362030024962250683465976e-37887160060', '6e-3646', '-3.6032730953987219226650172180149773504100795856e-37887163705', 77, 5);
    T('3.7172207274e+7479337127', '6.9378724218840366144454937999e+82', '2.57896031706841782623973119e+7479337210', 27, 0);
    T('-9.22156264702065852508492432911552219165721873022415e+483937033', '2.121024012480394739683708154054e+447458174396', '-1.95591558069230871790501247959111228893333202506952979058379714e+447942111430', 63, 4);
    T('2.0289477018525833e-6224698', '3.37770927625102736949897311900161215e-669532', '6.853195473575674401419731574736874235445854342167095e-6894230', 79, 3);
    T('8.81745963112364968454172048962638577935482677e+231833367', '2.763603496015845952889950609424758090449399661e+378151666', '2.43679622625519097264026225342490694251e+609985034', 39, 3);
    T('-3.1482219138417671680896118e+2867553890', '2.4155410221555417376823682770030975399815103419546692e+3447101786076', '-7.604659179733818118581886246401089647400966266400961737149388185321999025e+3449969339966', 73, 4);
    T('6.30533495921236040278890666857269701057e-39586342383577', '2.1656607952316671692916602368821791557609615563339148e+57', '1.365521672196987210516681732518911671602111223687805786622210423885670169709e-39586342383519', 76, 4);
    T('-8.017424166213570088876487e-1234746501343', '-8.538741689e-49938812', '6.845871396644387619541309472e-1234796440154', 28, 6);
    T('-6.309232520520e+15', '-3.36620122369833533095521033502735317998542351764863590910491e+637172710080296', '2.123814623117175657625071008129138114690367083711398784054e+637172710080312', 58, 6);
    T('-7.950226877961997740615349139686984572e-7075', '4.96082660731115249648097026740194168482428920458004182405764e+8093', '-4e+1019', 2, 0);
    T('-2.00921073120988288422769815942685869e+113', '7.3136988696823105638422e-60833167362', '-1.46947622538032891588e-60833167248', 21, 2);
    T('-9.664472962750830549e+3023', '8.8922035633778504717447364400146622196063261e+29410087133', '-8.59384609175418273571666980544778473540647592800014845895360289e+29410090157', 82, 1);
    T('-5.1199005342734e+18', '-2.5668997512983232051043e+9139435833654', '1.3142271408e+9139435833673', 11, 5);
    T('1.0443945376776328201723723668876883942560072858237e-710335', '-2.3508616998568832313919e+26', '-2.45522711816608357222184424197905119360265170488245644241138180768900803e-710309', 86, 1);
    T('-2.43305896198e-85', '5.290823078963923792072204980637729517940636799207364838398e-3334405', '-1.2872884508523791995407024403400220179344651557939660780483e-3334489', 59, 1);
    T('-5.851320135594331935429112308266e+48427', '6.602442378822776280656875543328906731e+26689', '-3.863300403530705080517291983119430232089e+75117', 40, 3);
    T('1.38824661067935073819394617664395e-28114631', '-2.2063407438421024330034184971494705679e-4465734', '-3.062945059642556290926260442053459528421724619794502168508154922599205e-32580365', 75, 1);
    T('-9.31265252470102473374403e-37266522577', '-2.9433312824578078283335904e-91392', '2.7410221498612209024643228443252117403606404465312e-37266613968', 75, 0);
    T('-2.38e+3326249', '-8.524002301e+978', '2.028712547638e+3327228', 71, 5);
    T('5.212e+176365', '-8.1261670900686396145383374e+6599324976992', '-4.23535828734377496709738145288e+6599325153358', 76, 2);
    T('-5.658689202386330463251e+251982932623', '6.396030377816566099465340012491969744327e+6929655', '-3.6e+251989862279', 2, 4);
    T('-1.675262818547116378318486325593571591272941419307e+28229796940393', '-3.17300871240685573821832750606696011254e+44', '5.31562351882126574163214809681402443258e+28229796940437', 39, 1);
    T('-1.6159994763118948152863855562605650611183768204155314e+8050', '-6.6015516400104e+85299', '1.06681039931027367639149793446832239563378306095877679140738175266e+93350', 66, 4);
    T('-1.2534726623960476136839162766017520270909176645e-280806', '6.652857868100641590e-45731326818054', '-8.339175464470604580564738307400085367520840856401162314314366555e-45731327098860', 89, 2);
    T('-7.8298620513145786620559190884035437470936045e-52474489429457', '-4.0676469558373099355091569465705365417091657018e+1359812224', '3.18491145376558209310515384978653276860170325866620350873801054961325362576666722455e-52473129617232', 84, 6);
    T('-3.36515457e+872752582', '5.2387542272484e+472793125452510', '-1.7629217728931771785188e+472793998205093', 52, 0);
    T('-8.8019198313076488484765241490440675177350703828214137608862e-754050605654346', '6.2963416510489579289178416526941e+117834', '-5.54198944430561670049866275583548770195828952350394512e-754050605536511', 54, 2);
    T('7.312781219883851926797246340e-77218', '4.805137546757509260005895499110974165912823131020457616e+47789463916', '3.51389196109e+47789386699', 12, 0);
    T('9.27508109144169157763704434643493871389533104088181107647834e+2284014', '-5.8126596341609054670367965190157629566526824405e+1864687398', '-5.3912889463792194752831210559913913089309260642175480680306e+1866971413', 59, 0);
    T('2.8994910103201115146355837359431937726014558e+68919', '8.7058704626559440351354004530568899e-6268486297775', '2.5242593143482299832960202125867207252888496281228258678582603803183396960316e-6268486228855', 77, 5);
    T('7.6022985925077129411831462452792575045735208143048227445977e-82789', '9.796748032248729627211228597300681e-59192335', '7.4477803776717223596219811032447657811965302403105377e-59275123', 53, 6);
    T('-2.767380e+530395979', '3.364984951183331182274185424651258868e+2271591374588', '-9.31219205420572704720193526047140076612584e+2272121770567', 72, 3);
    T('5.0167845524765394503345795921797439677018233021406315e+1739489931', '-4.86847833148103913314006299106750854199579892818e+360406548606662', '-2.442410688744083439209432483451227967421402319774319898527256791398039481764e+360408288096594', 76, 0);
    T('-9.30218529343929446150239324638e+35', '3.40215698589550281194584343440970181316918645576e-3413376', '-3.1647495e-3413340', 8, 4);
    T('-2.5364315590639054363e-61299928519', '6.72178335323067378e+58795998444', '-1.7049343430324684080328106242453870214e-2503930074', 62, 3);
    T('-7.809057244159339809674792174742724516e-19742', '5.560215749463043341741991364591838486437808726324524464e-15775683306565', '-4.3420043077433231437386488500902147e-15775683326306', 35, 3);
    T('-4e-123978839854', '-6.3872058316061149215809163382527017348048e+187058', '2.55488233264244596863236653530108069392192e-123978652795', 72, 0);
    T('9.1338826529897e+58301', '-1.066679665869182191748489626414729546117326641544637064e+132', '-9.7429268963793725879495965610849393418155415996143e+58433', 50, 5);
    T('-9.6740452328429996542421438107940e-48986', '7.5030e-779', '-7.2584361382021026405778805012387382e-49764', 63, 5);
    T('1.28871e-17', '-8.0965561789598700640594903797543966968657518344553531342e-32752', '-1.043411291338737415025410584729329e-32768', 34, 3);
    T('5.6646144553971031859823794999315508444099263783501e-56785582833182', '-5.12549427557964140791879973830730211512052440797705383973e-6710397335759', '-2.903394896450354032958050829145416e-63495980168940', 34, 0);
    T('-6.185267914630983470920673196730346636674062489e+66643', '-6.72236923773873911e+70977466783', '4.157965475648776480633866627276318021267756485623734e+70977533427', 52, 5);
    T('2.782956361255571057984600225081980117620479604094e+7674', '2.4488945357322173425553608355116e-6554555941077', '6.8151666262599826134887465981118589563440900717733130634919697923527506239444904e-6554555933403', 88, 3);
    T('5.56262966681936984331363608659047612018e-32233410', '-8.01202329831307377509774578112967569057673786433e-509992675727', '-4.45679184904442822115693177887421655531299052228e-510024909136', 48, 5);
    T('-7.045377158064809023740543517157777878953373176336e+58', '4.852470971385e-29336148823', '-3.418748814196843453021478872691046683859293968409755281514536e-29336148764', 72, 0);
    T('3.4589718278574087976123173824798108325181e-6328491', '-6.7339127922774995037043328115581381590871215713218e-168580973', '-2.32924146397364900197726229274652e-174909463', 34, 2);
    T('7.939369162776693210266785764252875311777733765843014424e-344978174107', '3.53148611e+598460025', '2.80377719205082211037884633208047636911e-344379714081', 40, 4);
    T('-2.784298984039661982818828e-66100330066498', '1.357656426611712248670458560486261314218051941392123001e-554559047395265', '-3.780121409289908322339658269461562291647913024534972716860429456473677274662828e-620659377461763', 89, 4);
    T('-396996632.908', '3.61316879227190836078508891208822395085e+9015516', '-1.4344158446602125108272139655124297275252908845718e+9015525', 68, 3);
    T('7.12201420440049183552119802136759995401988192508027e+52', '-3.858059191549990878675e+5038', '-2.748e+5091', 4, 5);
    T('3.91961970052150034053484113503539117e+39030635061', '-9.46841e+513982', '-3.711256636861477e+39031149044', 16, 2);
    T('3.91929687030874164646693006929155450533345997368374589014376e-924291452344', '-2.70356115165984654069e+2550388279877', '-1.05960587603887337728941352e+1626096827534', 27, 1);
    T('6.90102985657843378231718446230112922525186e-823158808', '-1.0836448e-3012215504021', '-7.478265118725965560352348893213414719072206779328e-3013038662829', 52, 2);
    T('-3.3377342405438745624e+55023773599169', '-5.5e-2278398837', '1.83575383229913100932e+55021495200333', 72, 1);
    T('7.5049832502650374585479987462111811712983949054856e+86098474317', '1.661051863864226143777e+24', '1.2466166416122538447773467628323140118114440979618086701030322471603112e+86098474342', 71, 2);
    T('-4.87e+267', '1.7e-81721022530838', '-8.279e-81721022530571', 68, 3);
    T('6.83351369482981996529055615987839324981410477470204653128e+620504808022', '7.4783785689909515197880642364787065033504401486935857944e-65840527389', '5.110360236632149881711155937231900813608555028659018727494364076590255955183696750331433427e+554664280634', 91, 4);
    T('4.2361960060627185603038002132515561109e-228', '4.4052646402512181925124940935045349074740230e-28210067', '1.86615644746815291997497613644205474302638003224823320951446452322043555686571507e-28210294', 81, 5);
    T('9.5968511664912319362036690112118811621272513673e-5977939540', '4.36966234186734921249996e-627367', '4.1934999142722478600061179518295706793122596e-5978566906', 44, 4);
    T('-2.11073218635180677390342331841055283e-2094698760', '32738886142288407408751.08', '-6.9103020725835279127241739223774344905978812e-2094698738', 44, 0);
    T('3.6281018960497701767873179528e-9381124461651', '-3.9650170726771702605684008e-601637457610570', '-1.438548595924975081895490565969561047270229668222588224e-611018582072220', 58, 2);
    T('4.377826e+190820', '6.54580790492212658122241e+856048094', '2.865640803717361372256657828066e+856238915', 70, 4);
    T('1.0416122578642e-4126905126008', '1.9338522847119027386884108903455144287237e-7773504456', '2.01432424465460675085e-4134678630464', 21, 3);
    T('2.499078011e-8513', '8.8702e-4398693', '2.21673217731722e-4407205', 44, 3);
    T('8.25465549229764123272327e+102', '2.419145891603428452193554919e+85', '1.996921592079351492424286610808171963316885267426513e+188', 68, 0);
    T('-1.281373479058127663904303376740794082e+420821923576', '7.75233986708717e+20', '-9.93364270633051002543040492883291661247676211412794e+420821923596', 83, 1);
    T('-2.7719272362192177178310040416e+4004', '-3.890513559729944423386623517463565103421472e+5223626216', '1.078422049909561525570627113120226158e+5223630221', 37, 5);
    T('-5.967674347233940389e+26', '2.24075208790368338380860667313188005406e-504', '-1.337207875349370275146152708640315649315530306696813742934e-477', 96, 0);
    T('5.501956277790719607530292812136271481805538e-73', '-7.42693406285975355370e-7700593', '-4.086266649188895602279059895074308017953648932758e-7700665', 49, 1);
    T('-8.21038867733474453997e-5347246626', '-2.64654478179e+44767201', '2.17291613104679682073202975831463e-5302479424', 88, 6);
    T('7.992278723728321112583e-833858703956', '1.77101982337361239331616524257201147048335088268649565e-6815', '1.415448405365001153891591240694424793320514925611722024258325857379738976395e-833858710770', 85, 3);
    T('-6.467669562322557106365054384e+3465501194194', '2.5942840655421e+28080363674364', '-1.67789720867250579608304288686300869415664e+31545864868559', 49, 0);
    T('5.85958092e+82572564299', '2.532406336898190411250101650348638524e-90759173', '1.483883985337572851628804897844339364320736208e+82481805127', 65, 2);
    T('7.52686337408725701420378299151114765586392737271453687557413e+419776249', '1.17994510820e-9388532922', '8.881285618344005553865891658767940802304538148274595941362206759970866e-8968756673', 96, 3);
    T('-5.7454626167794924064489142362894385109133685469121116465620e-4408643738', '-2.392073591535942300590050238975533e+99', '1.374356939675521370865e-4408643638', 22, 2);
    T('2.689291957024047291092956299157814767038170082677897e-317276896', '7.1220723e+691640', '1.91533317537337576457831807833423858808535041885289600459531e-316585255', 81, 0);
    T('5.097024126592355994956604250094090306e-670040978900', '336410603830798656.4', '1.7146929641670836114653290657269477138626143178050648584e-670040978882', 85, 2);
    T('4.98733940749e-25762017512', '8744700.8', '4.3613e-25762017505', 5, 4);
    T('-2.43246228635322170849456239161801551379562771e+6710', '-3.0220796770910523900e+86', '7.351094840878507273473960489475633039073416e+6796', 43, 4);
    T('6.744372752991048470662100654e+53694569192', '8264694607438760630366722823268220097746488131', '5.5e+53694569238', 2, 1);
    T('-8.7208937903750508192903562e-44920970', '-9.0722345427735e-15416627340', '7.9117993888899454524626471253844950921e-15461548309', 38, 4);
    T('-1.5085780581380293774626139461924463e-781003339', '-7.5509018411757573376530173e+48515', '1.13911248367517947216e-780954823', 21, 2);
    T('-7.1217758788981131114863663239e-30473131158399', '-5.43446720411764518529160457044318297714e+60394610', '3.870305744894791400473330198947503675097323520572e-30473070763788', 50, 3);
    T('-3.515618536744972235898e+5026', '7.62602148620499160538157161410915738002127e-870885', '-2.6810182498517711061188172078403022707e-865858', 38, 6);
    T('6.811493261689817013252503813737007459525728867621e-867189', '-3.322415467716065702946749114e-97844', '-2.2630610570882003311778826383604323e-965032', 35, 5);
    T('6.03003902903843086413700940e+69967439033', '67212978451420.69369064582', '4.05296883e+69967439047', 9, 6);
    T('-9.3e+67389711172022', '-9.85345212964322516035508e+770186334222735', '9.1637104805682e+837576045394758', 14, 2);
    T('-706628153313384271657095.11641681', '2.0699155602176391409619734245296109327276e-468422180607', '-1.462660609831229604428958773756054228800359329513831051933297651271790956e-468422180583', 87, 0);
    T('3.398158204316305070291551913922245682811316219297441837383e-8741863650', '8.865243634363049869867156854966247807036083984133750e+52', '3.01255003893736957406691491e-8741863597', 29, 3);
    T('6.288944199014062190715109809813430117292494360615788003926e+41622', '-50745552.89031848', '-3.191359504753296817011848703025442072216145574004105167262232768096303525e+41630', 73, 0);
    T('-2.460884140349244432624384768e-112', '-4.448839272e+35709897107', '1.094807800742767842738672098071701e+35709896996', 34, 0);
    T('-6.08045265433469762948e+5390261', '3.1284766284877851357688803e-8785', '-1.902255401971261884680608287282e+5381477', 31, 6);
    T('-8.666441928401e-73570691', '9.43673678492e-688', '-8.178293134011473757651292e-73571378', 30, 2);
    T('-6.31989042020e-596', '8.7922568859489737010685480667098040429329278e+120', '-5.55661000654464028795e-475', 21, 3);
    T('-9.84053551048473181987800629294025e-427', '-1.823392108895530648872903317142e-420306', '1.7943154797124112405823272967638924948e-420732', 38, 5);
    T('-4.07584391126307481459237484390609356104923426997e+2633', '-7.64327110914641648929463594514144602104231818098643272578819e-819365073', '3.116e-819362439', 4, 0);
    T('-6.2432712411399814579593527718896858314980e+466682924497', '2.6564235607843e-3347059138', '-1.65847728213312856373124616830765875427825578905238814e+463335865360', 97, 5);
    T('9.7502012956058606420501788998238746720178679729806e-4661', '2.786010822907106661991561142215944809773822672066399e-9959143419', '2.716416633508082134637849363449315483064558473850055412167e-9959148079', 58, 4);
    T('7.7487452426630588208996884796828402161736288454e+62855008381', '-3.60681718562260525057623118813960301286945066931836292019624e-7604937017', '-2.7948307508248525193346446651e+55250071365', 29, 0);
    T('-1.20872981896600994360235e+178', '-5.05330431e-22396408', '6.1080796038064577915086121811285e-22396230', 81, 4);
    T('-8.02399765193047380054035379969766585819071439637771767e+8933', '1.65757869165424483941288570609239870788866127278e-3950644885', '-1.33004075297236474405460995510090380068520286048639373870416970219950872232131e-3950635951', 78, 5);
    T('-6.947563731022099898463550420106914481620164391e-123529813', '2.505317152703439184912547922337075178e+4864', '-1.7405850584829969934756357600900976765582994087359780881554598130228340142485586598e-123524948', 83, 3);
    T('-6.48652417515417565841498677632394734645539206e+28024657374961', '-160.2570', '1.03951090473768272849061053581334682990090177e+28024657374964', 45, 0);
    T('5.630975179832195683736989e-20744', '-1.530569993940824770948500e-4520916461793', '-8.6186016468766984226142675683887600224420640665e-4520916482537', 96, 6);
    T('2.4256056303763893143858949870682186437977008501214e+765167710229', '-7.90899248481153785405470545839032427757083574006847964469660e+401', '-1.9184096701763415966594622592684970849917591306026788068125328090526858644938208456738629e+765167710631', 89, 3);
    T('-9.447124679247743826613943755165480003684517416485e-530559734636142', '97475375954583448086114175425095181219087204239949', '-9.20862029e-530559734636092', 9, 2);
    T('-2.34535941644690170366155400878456825947755514028338e+123359452345817', '-6.56559193442844307745314368922315858059664e+44', '1.53986728679595777709673770575376482632e+123359452345862', 39, 3);
    T('4.368086152841e+23123576', '-4.0742e-758024257450158', '-1.77964566039048022e-758024234326581', 30, 4);
    T('9.42784932615887106074747766371439e-7859922918', '-8e-5909704259523', '-7.542279460927096848597982130971512e-5917564182440', 69, 5);
    T('-7.787696918342063708716228284689875682223293735954484818099e-452071937389832', '-1.79835049999930534350629964973854747439267957613e-207780556', '1.40050086469434996673663869729630269971083931528140966770306e-452072145170387', 60, 2);
    T('-1.416074970121555e+95758620693', '6.338822661397832035338401200696744840789372e+910630', '-9e+95759531323', 2, 6);
    T('-3.979184883289602142636899501741886899779e+7446', '-1.605874713284604782888390698581536580438967623302234e-475536371509279', '6.390072e-475536371501833', 7, 4);
    T('-5.5526898898110618305115e+9523055773391', '-2.9510844272439040821483884230e+150', '1.638645666313609427138560417978375423092809e+9523055773542', 43, 4);
    T('-4.5042997674018e+83698049', '-8.6835700039694041518085050920097442318007727011483975752e+150', '3.911340234909663462372e+83698200', 22, 0);
    T('4.1563140711781677219e-52993', '6.57127407672759107912449916276637852941537022e-2016013492196', '2.7312278910671209368330310940475993029829933613559374270942501818e-2016013545188', 85, 2);
    T('-798986475267887366008.4308', '1.07082614921926388583275119095e-7721942920567', '-8.5557561058938445080361179354e-7721942920547', 29, 2);
    T('-3.192450170693616e+18', '-4.3469858956e+104', '1.38775358644109612209624896e+123', 86, 3);
    T('-7.7e-61866164', '5.5466908224405553806082425233668780415702119363227646565e+97034', '-4.27095193327922764306834674299249609201e-61769129', 39, 6);
    T('6.03003648557918149131312307586e-39', '-6.56347258034927330e-691689', '-3.957797913160465387951583463334222574881178e-691727', 43, 3);
    T('9.650299398435495570744738860275528481e+3147888', '4.500877532850971574705e+35772140451', '4.343481574770356844e+35775288340', 19, 4);
    T('7.2038214599647736316005271374750410e-67410369811335', '5.55182139024270745479484419661377643e-43', '3.999433e-67410369811377', 9, 1);
    T('-4.9851370257377465399362e-9143077', '-66375478001794016526602183450186103.3093', '3.3089085298778464744266133615205410729008e-9143042', 41, 1);
    T('-8.0887445900633922156435716e+958346277458409', '8.48964072e-780', '-6.8670535445481881935258686461595552e+958346277457630', 64, 4);
    T('-1.482e+33549', '8.80221206594074871491226487717398272272875640500792350631322e+4244400002183', '-1.30448782817242e+4244400035733', 15, 5);
    T('-1.642258321186031861802767979133183300936370968377796035e-92524488', '4.0354275199104039106487892645e+874631261', '-6.6272144241159720914329736660571756549881218429495850403879093143194798976786662575e+782106773', 84, 5);
    T('2.8511347157718441718e+46730406158', '-6.80917027473263801775e+65010', '-1.94138617558919299872820026484894424e+46730471169', 36, 4);
    T('-5.79722957755763064685832269926819803417725341e+7340292', '5.02560866119438093391114578968161752399850552119583e-900617', '-2.913460717590587070487318842519024602693125064e+6439676', 46, 3);
    T('-4.8494936082114434439062211810136743746956941e+46', '-4.7641180e+5853686897', '2.31035597897650855170956186404485043346265007843038e+5853686944', 62, 0);
    T('1.24530917747961917416248969280867370016029e-56220405452294', '1.6221177798562e-729', '2.0201e-56220405453023', 5, 0);
    T('-9.569529536279125051388210159e+76163653', '-1.201606016941044e+45916559680', '1.14989e+45992723334', 6, 2);
    T('-7.97751874823985257611562475814334054850952361855e-69358685', '-4034060669680630106215466776276.9531922', '3.21817946239142416886167390528668343108577856143207900993757598357662338e-69358654', 72, 1);
    T('3.95191631373345785053322220774682262e+389', '-7.563e+697506729389354', '-2.9888343e+697506729389744', 8, 6);
    T('-2.5047671479973107744e+29530259', '8.8005632696138162131e+839660410', '-2.204336176160048679744358206212603642464e+869190670', 61, 1);
    T('-7.67105333517215e+79246643973', '-8.62445e-277', '6.61586159365254490675e+79246643697', 64, 1);
    T('-1.212594525919313571478902622283586e-516744221419', '-947153548027536209853476263.332', '1.14851320754324606828368500640163561287719e-516744221392', 42, 4);
    T('9.655803056151405175967646263712e-51', '-9.6593970439586111823194127788617e-26440281686563', '-9.32692354976354069009374747563715701915759145989288913e-26440281686613', 55, 3);
    T('-1.986438720322924995495143220636356e-936582414', '-3.53800282724466426464433121028143568e-480085833990', '7.02802580865078155573961373054691138374903804855407680046794288358208e-481022416404', 72, 1);
    T('-3.743906869673606119126830555411266e-115308430518202', '5.9722942405287981135036012041637220545458064127446840982519e-98984429', '-2e-115308529502630', 1, 6);
    T('5.425607463398272760797052900901e-773950', '4.98972611507588119078376e+79', '2.707229525026896979549011956120912271123061813380016776e-773870', 64, 6);
    T('8.264185269207535108963939088283127559594794752e-26869', '-8.5923855055768409523996e+25448496', '-7.10090657225404680085367650612703739660255795072269056718023638868992e+25421628', 75, 6);
    T('8.516573227810497718878443855854e-428', '-1.50657588195580039204901973022800133e-9524', '-1.283086382192975833192324102755568860747746838326837330688504028582e-9951', 83, 1);
    T('-8.0374540879908600617e-946186424', '1.079378658376003818470586176465246963924454643828224557e-626983169', '-8.6754564102543018775301293186855030421064712308633e-1573169593', 50, 3);
    T('2.56414965664590866718015110692998533e+122', '3.061186626750247116830200319701e-63432707592', '7.84934063791069351660910477665312198583723394288957113758923998633e-63432707470', 68, 6);
    T('8.892989476977809355631397493139837474026178537461714521e-4850', '9.6994506898220928341130710831495844114901e-103403728473335', '8.625711291705302551505600944575053585341721364509366471183190244422576270705357835e-103403728478184', 82, 2);
    T('3.478727161596102490623825372227488e+55443', '9.46914000857383977349048018682194e-8239062687572', '3.29406e-8239062632128', 6, 0);
    T('-3.904019645281703030037083379297691274e+22582', '7e+3555965535872', '-2.7328137516971921210259583655083838918e+3555965558455', 81, 1);
    T('-9e-8135585', '7.54823955157498449105075e-9957106', '-6.793415596417486041945675e-18092690', 94, 6);
    T('9.7182889150404082471641975593289475379453078925970086e+4703210', '-6.9049503317411697e+7601669', '-6.710430227e+12304880', 10, 6);
    T('8.39733091963214666227201037082658716989265e-20679849031985', '-8.7031173269467934355715366170270346970517e+400392', '-7.3082956226756586954022215163851933e-20679848631592', 35, 4);
    T('6.8988015e+645107', '3.3481743973782516553655146e-40479998', '2.30983905548946785875e-39834890', 21, 0);
    T('9.830853292879701862996197732104447287305e+657', '5.658898945612330412150526627e+7161', '5.563180533354645133304410758013190333387379198613148508151732157e+7819', 64, 4);
    T('1.143236208922396297113345957900376478497471648540446745642e+758806258205', '8.80663944835697003160e+2950', '1.00680690962860458664728130482450768449605872494790291086136466542859419e+758806261156', 73, 1);
    T('7.08929164419935719197605017457e-717527782', '-5.0588766e-74665', '-3.5863851609415653833529347988e-717602446', 29, 2);
    T('-2.9663740752224884238050264232375274268987946404791442e-18656', '9.4443759469e-412215426075357', '-2.801555196553900093574788492694345e-412215426094012', 34, 3);
    T('-3.32850897112815600268219258244195994839121682110787e+922197327867762', '7.646409814243089e+67976477271', '-2.54511436636304986279147e+922265304345034', 29, 5);
    T('2.88434254092161747365815810967382837595991156758416295e+8291946', '3.859312964180742050622776059117540767698085e-97683221463', '1.113158056131682080948254e-97674929516', 26, 6);
    T('2.04576216554848070225850806930946666380410313557e-41508026', '-8.40664383331383e+955', '-1.71979938934348818985750757314309520851792378678371451268459331e-41507070', 82, 2);
    T('-28026696467378921773317163466484395700240.45771', '5.6577428e-4933367553211', '-1.58567840146098529534748413718925131085386407877456988e-4933367553170', 86, 2);
    T('9.77691620e+95', '-4.180532682e+23757', '-4.08727177032752484e+23853', 68, 6);
    T('-4.681613578734966714e-612', '-2.68648625661509385798130640088070702e-463014772682281', '1.25770905380540937015e-463014772682892', 21, 4);
    T('-6.689751518688267918633181618000277549589e-479295282474', '-4.21197936348019967224e-1198922', '2.8177095343525309790139072631309079168893203295348668e-479296481395', 53, 3);
    T('-7.635237669720698833862794844906970548265448979852e-46014', '2.908605438262954340445e+944236658369719', '-2.220789380857979145e+944236658323706', 19, 5);
    T('9.729987838012868897756541473531279e+61232321465', '-1.5567064755015956174354244522400783392408e-3516612588', '-1.51467350739864034037109567388318630002703338563234227973638851137e+57715708878', 66, 3);
    T('3.208375870671424901505816219284926031754625709453779295691e+642685209', '-3.474252299697460472229167394873576858096e-5656044895043', '-1.115e-5655402209833', 4, 4);
    T('-4.788580680378723071264450759211859965805415870138949650e+98321383', '-7.8364012634882534604586926016651705497993951907303064e-79703072305345', '3.7525239694035265881864533116982311973344791635467148320846902801182805277435871706e-79702973983961', 83, 3);
    T('-3.6980141815614147e+61709562477982', '2.29314892802442580655320923014862417738006066842099258547390e-7788030533', '-8.480097256266682464498285071009e+61701774447449', 31, 0);
    T('-5.618625333438403945833723443496950231545648e+62472589', '7.6988994768828316551361207911736426300335712179e+100', '-4.3257231640409553719564750524022409342635482706776840400008337320442237737983e+62472690', 78, 0);
    T('-2.5e+2659227854', '5.0682878353593514936470001461199375e-7191956936', '-1.267071958839837873411750036529984375e-4532729081', 64, 6);
    T('7.97916836322352105552379491472503906608995887e-36093069931512', '1.92139287138285080592757860377489972e-554976491417', '1.533111721266124297409377800070420313445774426120885372459839572026422923317452e-36648046422928', 79, 5);
    T('-7.63008827110968896557858325431871650115315481539998013060455e+4447226498953', '-9.02339537036424904168867513e-65692160481', '6.8849303181001e+4381534338473', 14, 1);
    T('8.9e+608716153188816', '-9.965792558923452699561260044667e-1748804', '-8.86955537744187290260952143975363e+608716151440013', 62, 5);
    T('6.907930287e-295046787537', '3.68356737235245969751343518e+6287340', '2.54458266e-295040500196', 9, 1);
    T('7.581312048879738e-7148', '4.8103998372920243552581427309961336806556994260e-9373743', '3.64691422463911554708876005589e-9380890', 30, 0);
    T('-6.998370987204766326890429768384160112356599932e+9504251', '7.3978465929483264188e+17527', '-5.17728749638811962732724404925269396362242739176505717860578835216e+9521779', 70, 2);
    T('-5.18050420686437525985497264137200567177162939808068212377e-7985976', '8.7163476685440798642264954480280211933389e+120999352', '-4.51550757653850949140093425799983353917653168549479879425829773973830474065003754253e+113013377', 84, 6);
    T('9.33784890692954682930077686409e-6233942', '4.85922982788941436965963285e+66851119600730', '4.538e+66851113366789', 4, 2);
    T('-5.06589831510003697391644166946591566882042498434217638821325e+76746125', '9.7050498616246299579562062739772065011365909e-3294130240', '-4.91647957419660598863739751e-3217384114', 27, 2);
    T('5.5e+16343195586', '-7.73152100772e+624', '-4.252336554246e+16343196211', 50, 0);
    T('4.32198554e+948801349711711', '-7.9706525251751529781e+71', '-3.4449044958171497138636136674e+948801349711783', 51, 5);
    T('7.3922056206553616448e-157677', '4e-5063603007933', '2.95688224826214465792e-5063603165609', 54, 4);
    T('-3.6627366334140867432839267e-963075369643', '2.7312379759604343622079451681100457574783100e+71070126022', '-1.0003805389122025735649871478233103733635622638587821809814146879877e-892005243620', 97, 6);
    T('-940.47350914', '5.1161721457535e+484684738285', '-4.8116243712811177e+484684738288', 18, 0);
    T('6.7213400045354379556e+421223', '2.1236087077252549449e-188028181351', '1.427349616121356060721395421378604664644e-188027760127', 49, 2);
    T('-7.95600886e-327245546668259', '8.857560580164432556156209770029417275719523448433481436880e+664912960630000', '-7.047083045377496567365125247437260630626159144e+337667413961742', 46, 0);
    T('-9.9900119589412659730335782272721968101990183510004759945e+128', '4.3557476458957196162822e+4244561', '-4.35139710726285056338771017680968743324291747937262102476956705047e+4244690', 66, 1);
    T('2.73950861843019763986135239224452e+569', '7.97704530276172060e-43704536202645', '2.1853184356523858846523310236274382762276971121112e-43704536202075', 76, 4);
    T('-138371.3717', '9.982e-480007814607', '-1.3812230323094e-480007814601', 76, 4);
    T('-7.587138e-541490437239990', '-8.629413e+55898', '6.5472547289994e-541490437184091', 62, 0);
    T('8.922681087946371698219386850509071218530e+69', '-5.8e+361232828172020', '-5.1751550311e+361232828172090', 11, 0);
    T('1.2678456e-628635114205', '-6e-5515051983032', '-7.6070736e-6143687097237', 47, 2);
    T('4.4027314688394878706890291430686901905669064422e-61719438', '-5.114516182689e-37', '-2.25178413454136714572621701849991501967734342936125462190758e-61719474', 85, 3);
    T('-2.92975709898768639e+72134', '-8.279257030263969539741126e+625667930037562', '2.425621205875957506081589780278922227347514e+625667930109697', 49, 2);
    T('-3.10384846175706625824259e-73990', '5.381039375983810988543298735973e+49654343', '-1.670193078980155544353711160940871014028292793939369007e+49580354', 81, 5);
    T('1.56193630279496877503078236574887e-906336952861688', '2.7125043422959647573054407813805200162255587772442823e+78', '4.2367590037210576368086887498677108997911357029961e-906336952861610', 50, 6);
    T('-7.95213650870876896476184324998382074977452e+218134', '2.34721164236466476321164183531357641240531961937688593544051e-27426477', '-1.8665347394914320878552221669664993077361929e-27208342', 44, 3);
    T('-5.43352613848998133268974001260745715523e+844117', '-2.909547789810633e-785593465461', '1.580910396712182857940125551432914110655169710258556059e-785592621343', 80, 0);
    T('-8.9081760861635e+692', '2.839861e-20371', '-2.529799e-19678', 7, 0);
    T('8e+6159', '5.06921991090742e+58955808648914', '4.055375928725936e+58955808655074', 66, 2);
    T('-2.933318297987516018349236865655528423719440525114996417349e-722', '2.905729425426885398480108412339288869149673280e+90014168793', '-8.5234292926054343276816116522870014966915226903345204427746574094157130556e+90014168071', 74, 4);
    T('9.963079070346057676101660504644670197599551081617824015e-921', '7.90412675047490436e+3638344', '7.874943979701891592e+3637424', 19, 5);
    T('-5.7632652112979571344634596288885e-542857986604980', '2.98718940e+872615728', '-1.7215964748578017793723621291e-542857113989251', 29, 6);
    T('5.749129906158387427469421792941899804577493335357674817e-27731084977844', '-5.3e+60261428261', '-3.04703885026394533655879355025920689642607146773956765301e-27670823549582', 93, 3);
    T('1.584492728135566228755e+973431277', '-8.1443640853734396890906660390799273007560377e+593105899630027', '-1.290468566856268707637248334609116992900582983e+593106873061305', 46, 5);
    T('3.286045138455027246284e-87028710', '4.152141493881889731100109046421825117005877213419412e+68350186206589', '1.4e+68350099177880', 2, 4);
    T('9.83026823116961024199581905651882011836734023483751638110e-3835', '-4.637315551587844066585725029251288502720058256877775e-50454215', '-4.55860557446827613e-50458049', 18, 1);
    T('-5.811407244096110132684783633741361044742030392554762008e+58', '7.6033458389858834941686109856079976697546970101370574e+93593064844696', '-4.418614e+93593064844755', 7, 3);
    T('2731191144976683233414704269274658.330591', '3.384475613802298812896258072098011502280362855e-2895167755449', '9.2436498268063632705256181509639651e-2895167755416', 35, 2);
    T('-4.35869364888e-8250796287', '8.142440977439148692084517567841466663641591134215123147376e-50284612838028', '-3.54904057747442767e-50292863634314', 18, 1);
    T('1.05154682463575695429504592863137079435865891501528277211760e+3036666309', '-5.6212439698058287403447524387074322795427731550195e+1321668', '-5.9110012469522160541905918905311458e+3037987977', 35, 4);
    T('7.35512835867091107484221771122228501e-4380545002', '2.36915443569996047079348986093134240374351824668099517746667e+64', '1.74254349760877587871348962855065436275206794418363715859758163510668e-4380544937', 69, 5);
    T('2.11673083285847476e+7840539783', '-7.77024445126152271480000981811147709983860348795610e+418762', '-1.6447516008832745166123590905721762466891208183308858700533475838036e+7840958546', 90, 5);
    T('-8.555879246099369847079484096e+635617501892', '-2.47181909771452640439486575349620452347919565160193e+38538', '2.114858571824778678182907593423388095703888367835123977610738373017942035790528e+635617540431', 80, 6);
    T('-4.619861598112653066909188876941e+16154995668', '3.495689280842004391341911925204244938729088460845928e+48257026', '-1.6149600667496013311356291258802903651120142741590025968e+16203252695', 56, 1);
    T('-6.0253138767920847278480e+432', '-7.763712920802279378883316381702961946733138563e+555803723444', '4.6778807197139981429977982219213824e+555803723877', 35, 2);
    T('7.106407731813863778877432982e+6036152870354', '-7.01289635843e+218667122583231', '-4.983e+224703275453586', 4, 2);
    T('-1.96202958257240967003478153960430e+8411245585826', '-9.89e+585359115052', '1.9404472571641131636643989426686527e+8996604700879', 68, 0);
    T('6.680082825923e+91302', '-937418770470960670377692364557.4', '-6.2620350293209190605852057264e+91332', 29, 6);
    T('1.14529224956657602402e+18492', '0.1725735', '1.9764709203057750748121547e+18491', 28, 4);
    T('-6.413068617023142662680837283811731750663776739e+432099070', '126867318.347579213622785031496615342', '-8.136088178207446003595619863156424128380202429372235495966651009433441e+432099078', 70, 2);
    T('5.94197022247928208219587788088463773275559e-84287911737580', '8.361034622324638e-47879', '4.96810187549713094960292139243603e-84287911785458', 33, 3);
    T('6.1282909989469860987738538021885e+2877', '7.248479e+6508', '4.442078861175625e+9386', 16, 6);
    T('-8.26587286274431083e+7340', '7679903995443667927773067.00', '-6.348111002441942209108816330337290555041561e+7365', 72, 1);
    T('-4.43185381334496478540e+5512387', '5.66891958387122209634310985607365805574625264653178737442e+123', '-2.5123822875325626576348058203618661679486270689786730382093555749e+5512511', 65, 1);
    T('-2.79102294732227225211818740997122427016167e+631', '-3.011e+61822781762', '8.40377009438736175112786229142335627e+61822782393', 36, 3);
    T('1.6760495971005501940933452254631503268e+4592930468412', '2.022700639e-98151099243399', '3.3901465910509754248441834131919132369714188252e-93558168774987', 52, 1);
    T('5.89124094976757837825429619986393853194953505e-28621851086788', '-7.2005240796795189605116122e-549205832107265', '-4.242002231799548749244e-577827683194052', 22, 2);
    T('6.08507188776440004460674310283930825190666023779464319655e-14912548', '9.3982712380702137490405191130825881722424852918054301973e-93762976', '5.71891561e-108675523', 9, 4);
    T('-4.748050574145426608786949388108294399e+3490', '-4.5306703683134326278276906486062778050310693521238e-409216581640852', '2.1511852043534265227662936419175613714933204589308194265529242e-409216581637361', 62, 5);
    T('8.61346015436179790187148312144846679954792721e-6674', '8.828169812294159748798003640347050763483521351011009857e+54946', '7.60410889141354176386e+48273', 21, 1);
    T('5.156917288911278323341796600233492833185186264686747864820e-33488433', '-5.4069126305638863369301837580292029079839816322861110250224e-4954', '-2.788300122418766491581809871116422e-33493386', 34, 2);
    T('-5.80068371798978132411010164745120919637377047093387e+701661689', '7e-904195280421369', '-4.060478602592846926877071153215846437461639329653709e-904194578759679', 99, 6);
    T('-6.177006943997929272576486037562053680300448e+779320903', '-6.647423429633720162264442583282300984613684165919255036e-69517350933', '4.10611806845420198168688168200866064706470532524807945649491360901700009197e-68738030029', 75, 4);
    T('5.50e-1111527929706', '7.29116523086103282510333605217653198050e+8984946585583', '4.010140877e+7873418655878', 10, 6);
    T('-8.24161652378540355e-991597350009', '2.93684592917824391811128639992707924388178738116120842338899e+2785', '-2.4204357937727312106209878302474715925075933318627185858e-991597347223', 56, 4);
    T('38038.7958', '7.142088e-49994', '2.716764270176304e-49989', 69, 3);
    T('7.40054041063970385914e+47344376760636', '-8.3612494e+22887444537646', '-6.1877764068136977508412009516e+70231821298283', 79, 3);
    T('-2.060397403486290655609315122229528792674e-2573', '8.628e-105885875042935', '-1.7777108797279715776597170874596374423191272e-105885875045507', 88, 3);
    T('-1.26870272120496944595286146797831316820e+446889535328', '1.4e+273163066824100', '-1.7761838096869572243e+273609956359428', 20, 4);
    T('-5.6e-71527211', '-1.6616442696681914233621e+17517488', '9.30520791014187197082776e-54009723', 100, 0);
    T('9.86724455295728206608854497169359883310637e+14087', '-8.6021857636230210453947705018102137e+4403', '-8.4879870619635932352631407449717202222059525896968778612527333048078164531269e+18491', 95, 3);
    T('4.28411e+8097877791118', '-9.8018305970110525e-62992', '-4.1992120478961020125775e+8097877728127', 62, 0);
    T('-4.39675e-494952', '-5.41e+8968861328077', '2.37864175e+8968860833126', 40, 5);
    T('9.6420346312198942656052758017346800391156935e-95902702597', '-2.716337973342303837359312e+34498479', '-2.619102480906415556092650703465379634399527868651e-95868204117', 49, 6);
    T('-5.436e-97170', '-9.5129868064025873035881268152183055345033e+162395566', '5.171259627960446458230505736752671e+162298397', 34, 5);
    T('-7.8582e-94867638040', '-2.801812011581702779297e+577857479', '2.20171991494113367802716854e-94289780560', 88, 5);
    T('-5.74181486972500907e-5440822', '-7.84324179566626455505554096201240614070449035956294292900e+172112191677374', '4.503444236920524e+172112186236553', 16, 2);
    T('-1.550116903759195470511e-4592784', '3.67049696999590209796040229024876288213875264714813313512362e-9058085212', '-5.689699398387556356926820157797459048e-9062677996', 37, 1);
    T('-7.15150240e-871575147', '-7.592940e-4192647339', '5.4300928633056e-5064222485', 65, 2);
    T('-9.13859577954939321176827640258354121289839e+12663', '6e-313743932074549', '-5.483157467729635927060965841550124728e-313743932061885', 37, 4);
    T('-1.59325994047465e+5032', '7.752572053448800277042911545529687440780827244224358186281e-226959259632412', '-1.2351862488403270647642845677954355045202880513e-226959259627379', 47, 0);
    T('-7.87493574846743766167290405773e+490321750558827', '6.4303887232061468334662759131086561195160941767e+4977', '-5.0638898032917968740623871301855380438538047571831973786621944314242253620891e+490321750563805', 80, 2);
    T('6.72310150937786551951075068077e-6078', '-3.892e-60207615482', '-2.616631107449865260193584164955684e-60207621559', 83, 3);
    T('-8613352657917188557152547637841955258660056519.8', '-9.83067437769475780994921675543486763380803202059e-917963123095218', '8.4675065280235545765611961232376407482971071865222284823508969692138217434143105e-917963123095172', 80, 1);
    T('-3.147781703208557403346548460664172516402230822899841e-888091', '-6.9944697945312875428836239857e+22538063172', '2.201706404287050484665421111316532212092284461785e+22537175082', 49, 2);
    T('-1.3972105103370144158611120e+3925811473380', '-4.68764e-85', '6.54961987667620225636718305568e+3925811473295', 33, 0);
    T('2.5891905559239398999146453049344612e-450394192001', '4.301945633439793207221e+3405433', '1.11385570062005439512883789318246703133536958283555843252e-450390786567', 86, 1);
    T('-6.0541619e+584901', '-4.99e-9772514220466', '3.0210267881e-9772513635564', 70, 2);
    T('-2.0890e+7', '4.387981503503094101e-410113152', '-9.16649336081796358e-410113145', 18, 5);
    T('5.386985062783482216985659383106663877698479e-51503096744', '-3.52051968243210748525116898121460114e-326155825958464', '-1.8964986942497011417959989720336487431605024e-326207329055207', 45, 5);
    T('-6.53569230920613966003212028e-660881500', '-4.1e-9247631058', '2.6796338467745172606131693148e-9908512557', 37, 5);
    T('5.5480437981987889662594e+136', '-6.04399657943e+19558332120', '-3e+19558332257', 1, 1);
    T('-5.656723518024451702359431910639850753604978288993433e-28071539644', '-76085326469556689196028051505948363985.73891', '4.30393655616909650566142197642350896081648986177938274e-28071539606', 54, 3);
    T('5.9847978946704809860939549045391241890622321e+448080', '6.01381491792265904808e-15690095025031', '3.5991466859721461282184549660347884207962495239693296717266019368e-15690094576950', 69, 1);
    T('-6.983058221323466930215016533281987e-7803565537', '-7.5377115596691051247442407993229986384290e+492626924156611', '5.263627868e+492619120591075', 10, 4);
    T('9.85546049220e-1099408293995', '-3.7955883885937664367861641884059702054e+85', '-3.740727140843892623294040989841751665393001909788e-1099408293909', 66, 2);
    T('-5.1515261926567057031719766145856650630798165952110649737e+4386274426392', '-4.2590e+1499', '2.19403500545249095898094484015203475036569388790039257229883e+4386274427892', 84, 5);
    T('3.631617677524289420066738052315079232002531893428487231985e-15', '7.6752166917e-763', '2.78734526164072140890779408e-777', 27, 3);
    T('0.003048559633689912614498', '8.862328935025763731051179506386844166768849e+835691', '2.7017338251801655651582995122886541368171097851759213942412172802e+835689', 74, 6);
    T('-80704579738631728073578247798398.912425', '7.877863092333026868611e-91161631', '-6.35779630105214690643945619931112472156032712724325770391675e-91161599', 66, 1);
    T('7.639616329068685224027920864892613946e+338069', '4.4955799797350153415655934293651589306060671279828895535e+825', '3.43445062218178922145335554581601670699868009759784944196238047644701100882e+338895', 76, 2);
    T('1.74080157776722e-7954389470', '-8.03113222429598623903854059394759052775524e-3952689323144', '-1.39806076473116158248129569920376313605550544667178552328e-3960643712613', 58, 0);
    T('3.91447103385992094398149405058739232837645475384872e+84505245123', '-5.07e+475890378121', '-1.98463681416697992e+560395623245', 18, 4);
    T('-9.52635567674883058457821302608013963268757254195204673942535e+58733', '-8.79247630023048761541546144074894475338977934712051250e-33241860948', '8.37602565153802609711524658981883554984502147002179962661848204748926306e-33241802214', 72, 6);
    T('2.21848557979492769182660995e+75943258544', '-3.49524453258955001768512919331793408101757305e-90276358273', '-7.8e-14333099729', 2, 0);
    T('-5.17414970152099388428410934068e+38', '8.9850129768389822845074849281918896823370883143418932923093e-50713209072310', '-4.6489802212273676923675169609329787437673205907236661950488150887291677427702109737632324e-50713209072271', 91, 1);
    T('-7.32540669945447354830374478987e+8823325383', '-3.7840344117956406024e+80', '2.771959103113405383385e+8823325464', 22, 5);
    T('-9.7793392531446e+813860354531612', '-8.7499549390999100802671958630874656e-9788116696', '8.55687778e+813850566414917', 11, 2);
    T('-9.9039148374054250405231397098377097463617993797e-9080848949', '8.68570385527e-5453', '-8.60224712855180554784848305553822e-9080854401', 33, 4);
    T('-4.0701213539509720248074878e+142', '-6.3209215744367721e+87442989', '2.572691787676450466089612218e+87443132', 28, 3);
    T('9.811468905690e+76516781', '6.0138126220012375864586029533338605126085823088427473536479e-6028', '5.9004335545411192160236685881032774540866829513043643988188904842566551e+76510754', 80, 5);
    T('-5.0006138832812404488583756717916459187372689113894e+67805188570', '3.4e-104099559', '-1.700208720315621752611847728409159612370671429872396e+67701089012', 69, 6);
    T('4.0796991003686e-86217502', '-3.20405223955063643281013100817084e+4118', '-1.3071569039228729514861066534050482883855771624e-86213383', 87, 1);
    T('4.719078459815051159816791950021322560e-4951535762553', '6.452570483663601366632734e+55931518452723', '3.045018637989528766814911287166652119960546789764740846867904e+50979982690171', 86, 2);
    T('-9.330538474114364611398774e+502', '2.456046778029135807936032623995101e+8500089187', '-2.291623895662547438426473999e+8500089690', 29, 4);
    T('-29540323468643406282570183143826216936688789120.00220616', '-4.36222358090299192010428501668636436585191635420355e+2329749585148', '1.28861495622418330906799778275203445662302720505168599997457452e+2329749585195', 63, 2);
    T('2.2e-194950549', '1.3201624918943132549315284e+2530723023370', '2.90435748216748916084936248e+2530528072821', 63, 6);
    T('-3.16672e+40220196301', '9e+237745', '-2.850048e+40220434047', 21, 6);
    T('1.9198963604e+429627249', '-3.2e+658057', '-6.14366835328e+430285306', 89, 0);
    T('5.62832626169130934688104092444339595084526591212e+9451', '-3.59005e+5176', '-2.020597269578488512077e+14628', 23, 2);
    T('3.47334022e+820060', '-6.091363658064929886143300221746321772398280469528619e+61258173', '-2.11573783882032e+62078234', 15, 2);
    T('9.52313296261815755337893674315778492540384567519885323800e+69146722977485', '-4.029e+930428403405058', '-3.8368702706388556782563736138182715464452094225376179695902e+999575126382544', 81, 6);
    T('8.5811e+490311267', '4.18093814176902360561689269227313437682e-74473', '4e+490236795', 1, 5);
    T('-9.95090757748790615717100070197121618435906308580e-760', '-8.5205220e+884', '8.47869269339524091461109692431611908655874529219467876e+125', 62, 5);
    T('9.25042513165225038579314515641508262093388834323830e-1274305263597', '3.840210506809214432279267344765749478853799e+67133', '3.552357978302298259131723185438047692561253851e-1274305196463', 46, 6);
    T('-6.48667e-831090894', '8.98822861665391695058692207135034482974380059661988407e-98792626620', '-5.83036729207904634658636697925661412967542190160763034e-99623717513', 56, 2);
    T('-2.651780105937190406829480148374517270e+91169', '-6.82022130796150742484915695603267222596e-128', '1.80857271825412494773003114003711512572893273739168230155596566749513623292e+91042', 85, 5);
    T('326578572719310.980236921', '7.3400133680007863338e-8', '2.3970910894623595064582688992963994229902298e+7', 57, 1);
    T('9.3352111687727165136e+35342', '-2.71257432516103215780537732355640e-807275878207', '-2.532245413636938177337120228887451095066620056097e-807275842864', 49, 3);
    T('4.474324429341735869634052666187e+6571', '6.174772588137992776355e-524716', '2.76279358367355179824705571522456953472134342e-518144', 45, 0);
    T('3.0681e-806535319', '-9.3757924712030925302893864981206652522494023118865524001998e-2028', '-2.876586888089820819218086671488401306042639123309913141905300638e-806537346', 66, 4);
    T('5.9819930083667789803310333795482e-72835465008146', '-9.52372351528966241811492e-965152383', '-5.6970847482081043279736451147888884225501285715525279144e-72836430160528', 69, 2);
    T('4.6607773919201243107375921258673076e-10435060001', '-8.766067406603331569882066670548999921530958e-4797', '-4.08566887847446836165630350728210450171134182377598641692096763688489535086808e-10435064797', 90, 5);
    T('-9.31794046014458e-1636366942', '4.7941147860896403e+1767672284874', '-4.4671276135882037653989378194574e+1766035917933', 47, 1);
    T('-48633555397738141373', '-1.41781516423096e+4880', '6.895339233337959409447678130350808e+4899', 98, 6);
    T('-8.78033996134333490841830101138533481692e+702', '-3.180e-7548234', '2.79214810770718050087701972162053647178056e-7547531', 48, 6);
    T('1e-877227181', '3.349588009309615917102478e+608136538946', '3.349588009309615917102478e+607259311765', 79, 2);
    T('9.709589945745754849354433802544911447826900771224141145e+58', '1.431334377953819699358128518990643953380984321e-331126817', '1.38976698851806619806373806471687723941067461744051171062456709e-331126758', 63, 6);
    T('-3.60549806207e+852621', '-1.90672274766636680365243102e-596225291235643', '6.8746851716158711254866202404543534114e-596225290383022', 87, 5);
    T('6.6181614175576244884996863059885219338743074e-703122464962', '-2.7602017958465614167459525478e+605349993276524', '-1.8267461029944979734993775218485777286542199797402383372693001e+604646870811563', 62, 1);
    T('-8.1968990289138574425415835262593343689e-7217', '7.163926854768643870834437e-159295', '-5.8721985079063e-166511', 15, 2);
    T('6.213902035941274524933781344411395048438539073325596943782e-886366', '7.875364761546539235871716603e-91080616903', '4.8936745125554210129595565716184513869320348740941e-91081503268', 50, 5);
    T('6.459139302779159961474170302253040342006772392619597e+366570', '-3.0812640129755378699705262172079699048e+65536817777', '-1.9902313488449332170157758940263672208348969026937361250802088e+65537184348', 63, 1);
    T('-2.96259127655202062535210322246527262610507950078360e-220549442703521', '8.896834242789248059896805383e+79183', '-2.6357683516e-220549442624337', 11, 2);
    T('5.428022228089987767807356425084282879795683857935712372e+11512', '9e-484642121', '4.8852200052809889910266207825758545918161154721421411348e-484630608', 83, 3);
    T('-5752728.2464289345410712622063721089490098', '6.12891455971478771424769362967e+90119282435', '-3.5257979907620816141582723317071564744896444468724457773830126314400766e+90119282442', 76, 3);
    T('6.226e+3091816996', '-8.2215471868654215614693129305e+56488874', '-5.1187352785424114641707942305293e+3148305871', 73, 2);
    T('-4.370189643006696532797323178405920544169413552522246e+24527213230', '8.388697192e+13', '-3.6660197586797757641873040851810260705049071440330109537733232e+24527213244', 62, 3);
    T('1.61471437147517e+88', '-2.3450254646031894092151461613018272e+47348383823', '-3.786546319170007501510193849096305449864480430624e+47348383911', 85, 6);
    T('-3.7828652301e+63321240432958', '-6.740692275885825006331580693911890169e-53132938', '2.54991304372521241e+63321187300021', 19, 2);
    T('8.5704224929938030921120133916845026494746375220045e+10774283150233', '4.670324770670361557708170132611e+558110107708713', '4.0026656464139391810213792423331187218768687526316901768678593284445583195539e+568884390858947', 77, 6);
    T('-9.313747511081283618828026070e-817486621', '7e-53165644041', '-6.519623257756898533179618249e-53983130661', 68, 2);
    T('-4.0856096070520300873378386185280653084414903368443256153e-1934808492', '-3.4074369282193433923427e+76324776462410', '1e+76322841653919', 1, 3);
    T('2.897073972684854605529885413147387597832766644e+143442933365463', '7.068056341103378375650360002e+3323974', '2.047668206328074219071366e+143442936689438', 25, 6);
    T('1.54288454482555007024034745509655757303929484539099e+35640717089995', '-23.02291629597244361842591583840919565765978911663', '-3.5521701e+35640717089996', 8, 2);
    T('-4.86140766205037189684368624231102118071279061322415e-556584281912686', '7.531697627442917943643024400211778971e+833517933960', '-3.661465255e-555750763978725', 10, 5);
    T('-8.04761e-7959', '-8.155439810237740661279090e+856836', '6.56317989712673441231162174749e+848878', 76, 0);
    T('5.30681962223438273871952477223e+4220126', '-6.5592675879384524035077341232745e-27071978538', '-3.4808849943157768844503130841175943481120705283673483354267135e-27067758411', 85, 2);
    T('2.54068538301684391942541930111364340011261844104175e-14314', '-6.31603523590073931492764866e-16565524686', '-1.6047058402472352004931731893948331929068101818604738371221259686283168391555e-16565538999', 96, 4);
    T('-26298681483947579303116797.733', '-5.6160218315906575036e-539', '1.476939693558985949791877230757813045711999193388e-513', 65, 5);
    T('7.56315660206412206037565648544966061679276947e-64279551', '-219759579889178891854517079794250.25186', '-1.6620761175056812013383177092469363097460645775144211e-64279518', 53, 5);
    T('-9.0918216597065e+241917804', '7.2227017362708161891055301e+83046017040', '-6.566751608742675129439283003745949291565e+83287934845', 40, 0);
    T('-6.345697929611190888591919497082213e+807', '76.365851', '-4.84594622583696691330768124118175213e+809', 36, 0);
    T('-3.6862282199946754e-1146092494513', '1.78801322215977459259838978278804822637e-524', '-6.5910247972489699968724756e-1146092495037', 26, 4);
    T('-3.16355673900698327996738770398524435814315798539511e+440326', '8.27263651621252691046543923548330495925334703837e+859278575827', '-2.6170955000219392400682936864042e+859279016154', 32, 1);
    T('-4.41771917061990296437243966873424906891396100278e-82390558834614', '6.910699034288084067063629833008226498e-487519', '-3.05295276061589191028795025251706088557610945214098540091056244987323265e-82390559322132', 73, 0);
    T('8.6753141728312183228386996769854291165592730e+6965', '-8.96692034869339901202327858008391e+2679', '-7.77908511876684946251254822789596539906413585266013114980582777184428286e+9645', 73, 6);
    T('-1.86287984e-94715796', '-4.3195163778390600616708347092615111147e+5397252714', '8.046739978826207753435754695855530343510557648e+5302536918', 46, 3);
    T('-3.13e-30849951883688', '-8.88324951794e-481540190563', '2.78045709911522e-31331492074250', 70, 0);
    T('-4.125957815104283994738034446e+1337135', '1.5834881298604604990354880391223000834794e-53025508792193', '-6.5334052245226343232963801055434746323764244779760183170800727314124e-53025507455058', 84, 5);
    T('5.2268440515967890381040751845776669407e+96', '6.7e-531011468261', '3.501985514569e-531011468164', 13, 3);
    T('8.4031688446390583360515938987334086523e+697324921954961', '6.9141003195479335183012191264711160e-615468146726', '5.8100352393934152551076204428942564764124572442817070363842332440436967e+696709453808236', 71, 5);
    T('-7.026367290774971028456733573464035994762313e+93', '-9.2439658353537610429904225e-105390', '6.49514991825709978813872009253932508010897704482731615792326229472425e-105296', 98, 0);
    T('-3.285770e+722065428531', '-5.33574547931046065351439262879179163374471e+2759143795712', '1.75320324235539323014979858679052051964093557767e+3481209224244', 73, 5);
    T('-5.123911e-980366309', '9.99766097382189721806121065848283852786886137e+294982430837865', '-5.1227125e+294981450471557', 8, 5);
    T('1.218173e+1421275', '-5.036546e-1816', '-6.135384350458e+1419459', 34, 1);
    T('3.89136753222722589775920263058185814799273209477425554390e-73098179144340', '-4.62245622899721585311833160612203916341863027246e-12983580468', '-1.79876760886612644557068672e-73111162724807', 27, 2);
    T('1.271246210835695350085496750919974975141559e+410852', '7.47082136e-42581', '9.5e+368271', 2, 2);
    T('5.713449703069263867767236372e-753178178381713', '8.59926782975732570874570092873451521532153114367e-294393', '4.91314842285400656825627505755306e-753178178676105', 34, 3);
    T('-5.3147196625642159519816e-7693', '2.016596279681939205538390445e-8147884048', '-1.07176438990794491916668412083873e-8147891740', 33, 3);
    T('2.070042676791682248503547759454878148783331582086356334e+25862164922890', '8.2231e-95594844', '1.70221679355256822976695235807734085052602139326543167701154e+25862069328047', 83, 2);
    T('1.13e-826455018', '-7.73776576334e-559027', '-8.7436753125742e-827014045', 34, 6);
    T('-6.43706724446010783128445447858349620493650054707e+67445862834189', '-6.9251336310303285635835739e+26', '4.457755085981441818352979885488688128046419098281547043496e+67445862834216', 58, 5);
    T('11770218788365062597', '7.52781738937134330290537e-19003', '8.860405767175982101856042374362875201744589e-18984', 86, 6);
    T('7.05e+70027316008523', '-2.8120730501701483199164117877e+330556654', '-1.9825115003699545655410703103285e+70027646565178', 53, 5);
    T('-7.77219078263359e-576095838880', '-6.2578678160075000146318882994190084318804602580931958740276e+517', '4.86373425585128861261310192982686913974704776962244186839630139074e-576095838362', 67, 1);
    T('5.8890065440558928788161080160043743447055953297934e+8705', '8.9659e+38238198', '5.280024377335072996217734286069361993719589716739464506e+38246904', 79, 0);
    T('-2.6770611897471958274925582632814707629e-306734414958', '-1.23345649702040026824913983081004322586519458658512174e-335295748342', '3.3020385174148412473421914e-642030163300', 26, 3);
    T('5.920596782392563237840616839400613784377373067559603851690e-270', '3.354150339e+52', '1.985857170474432521528204259984467728167743877848451516185171922291e-217', 75, 2);
    T('7.57286029295455897527957664605439e-7124762', '7.2544374401400416915409079e-62374300003471', '5.49368412381594370141845270715994095e-62374307128232', 37, 0);
    T('-2.971327518863494100099084731850e-6868926255', '-5.76052875775379731313256633841e+95508631', '1.71164176211183964212767708230402055325466292001436912053585e-6773417623', 97, 3);
    T('7.44395369e-44006604', '3.55601175370276242331470184540865838059821625831e+116', '2.6470786815659049504226e-44006487', 23, 1);
    T('-6.7997647236970511207356e-147652361737140', '-3.69646640127491048259484011506095222211736236e+42690', '2.5135101837720524572301465780787021874163030944668258939073374952016e-147652361694449', 95, 6);
    T('-3.4960605496587316613e-908209', '3.975706500435e+91843882', '-1.38993106531925785867e+90935674', 21, 0);
    T('-1.71426659411206718651495406679120998662189503e-8795848369749', '-5.97027367713947199065163e+50748538124686', '1.02346407224268100860940794660658489873109245393513105509876351583989e+41952689754938', 78, 3);
    T('-9.1221505937619841918097265015789884146381e-706141986241', '6.74138e-2744610', '-6.1495883569775164990982254043214560918672994578e-706144730850', 88, 1);
    T('-3.756900426822069815374623563319330154083750270543468e-989116938809', '-3.798408120355767450391741066e-617492632', '1.427024108860899866816e-989734431440', 22, 0);
    T('5.037261751433049863301540537660e+33', '-682.693367187', '-3.4e+36', 2, 4);
    T('5.440e-48', '9.441892197582445765149891029206397107549416e-62271228292', '5.14e-62271228339', 3, 4);
    T('-9.97419628304494875538957787614117984438307275359e-5264820739', '-8.854289983187946562675842126295789892303379039740e-9580321092', '8.8314426239315338410446715685381575887152711659332148940756305954369245426612108e-14845141830', 81, 4);
    T('4.93537934685414167891343335689846438221e-7453204', '-9.23430e+70352532', '-4.5574773503e+62899329', 11, 4);
    T('7.151241694508976412344603432236715204161621254103142856339e+44170640530', '1.1855519879762014271718996640e-475', '8.478168807423416323222132810324284e+44170640055', 34, 2);
    T('-10996012257529286863.57463719', '3.689027561265062836810264737197933e-92699956799500', '-4.0564592282034003206983239685674449605733873459e-92699956799481', 47, 2);
    T('-1.809741742449037548686245599152788909503537724801651957e-7274312886971', '-5500906727144443056928210717.6448226', '9.955220525432016735773684018055124967327685936940476680479838e-7274312886944', 61, 4);
    T('5.6832560978498456461611697591583952944520540003666813038e+552140779626', '5.6096508e+6509123396', '3.188108211590826490886452286839869949e+558649903023', 37, 6);
    T('-5.8481127668651646354381e-570', '-6.6987259988069274440423646272020e+48920697158723', '3.9174905035354393e+48920697158154', 17, 1);
    T('-4.2159108099533270607244e-132401', '8.2881e-7497', '-3.49418e-139897', 6, 2);
    T('4.220773161e-58', '-3.4411711568409165787817399332276861376e-2184', '-1.45244028612014622423619099870493497517138329536e-2241', 82, 5);
    T('-7.9499908729006525538e+88733091245', '-9.7932e+9193', '7.78559e+88733100439', 6, 6);
    T('3.6394615739467543e+4859887', '-5.84899e+17', '-2.1287174351398826433157e+4859905', 71, 6);
    T('-4.47645241863471162080646449e+865767471607', '7.6844743914895176903176e-77538601407286', '-3.439918397571975528254408e-76672833935678', 25, 4);
    T('36154685214248411042167294985879880.341674', '-6.463102438420553295665643e-574058036433719', '-2.3367143417e-574058036433684', 11, 3);
    T('8.73902825774927204102029464613055536668986102339560965e-532971779', '5.4246120724942838382871729348882e-6548908', '4.7405838188855389093e-539520686', 21, 1);
    T('2.82159712486597622839452154933884e+726244910', '-5.581325530282599736227238973838783749823289963764e-2809538409', '-1.5748252069186453555138114788227596823943148932329520019258452e-2083293498', 62, 3);
    T('8.93206472e-338935121', '-4.9924554641e-9230', '-4.459293531705883655e-338944350', 19, 2);
    T('-5.4e+9', '3.458424740269273059647843986495136796275e+1576742519', '-1.8675493597454074522098357527073738699885e+1576742529', 41, 6);
    T('5.23726822478100780643483230748726326924e+1773748', '-2.3882675617773936332633037021184069977357090114175472e+718605826951667', '-1.2507997813571956246633037461064280053414134131228212e+718605828725416', 53, 1);
    T('-3.23540537253264629953338320706903494213e+315', '8.3915971748527138431689401418252377974948641439413831105e-7099014', '-2.71502185836482468593276453506322e-7098698', 33, 2);
    T('9.669275854242582828174337e-228136496181388', '6.58240359258963127482314512162449636e-916734004549', '6.364707612070655309608598433810677819967580837215315590191332e-229053230185936', 67, 1);
    T('5.52493e+9189', '-4.51024280183358419810338312e-85527283726', '-2.49187757631344243436273245011816e-85527274536', 64, 2);
    T('-3.42924e+53991928', '-1.22679878655741321595191464800145241326922117567796764734333e+7058722868455', '4.2069874708141436966709437875125006736793440244e+7058776860383', 47, 3);
    T('7.417555373e+17', '117422898236188240753181141463129.02901189', '8.7099084972507030823817634270210581053941755038497e+49', 82, 2);
    T('3.6849e-471406145573', '8.29411959454413629152309303e+32176', '3.0563001293935687e-471406113396', 17, 3);
    T('-1.24745713045897224676263954229e-744434', '3.015241754534066109175495e+29', '-3.76138482675114287839335325624166485802597067608418355e-744405', 84, 6);
    T('-3.337408195614044541216177520892e-29950763135543', '6.92e+74008444', '-2.309486471364918822521594844457264e-29950689127098', 84, 1);
    T('-1.48926775777780373456040295e+27599799858077', '-6.8158926350673026202115302803359927948666368867943e-21659475346', '1.0151e+27578140382732', 5, 0);
    T('-8.590705763803941699983440607165596235914995232658658064059e+768634305336744', '1.820326912751626628402688440735652815e+392', '-1.56378929013828337770063859534893e+768634305337137', 33, 0);
    T('1.8683199722845946830e-8722895347', '-4.4786071036893043186232723945674221214753539532717549e-2142951091062', '-8.36747109983838991022572e-2151673986409', 24, 0);
    T('-9e+876754', '-6.944763926e+994300082', '6.2502875334e+995176837', 22, 0);
    T('-8.1699097698233219773010373726466e-7830009218', '-4.9410e-776240093122954', '4.03675241726970338898444256582468506e-776247923132171', 77, 0);
    T('6.428013582753993506214641e-4518587132', '-7.561052872496383593607356599687003617753e-776069250763680', '-4.860255056432785275159788e-776073769350811', 25, 2);
    T('-2.75856536270184627059866477779195231680879040870521e+1036', '-4.9081e-530780669', '1.353931465667693168072530659588068116612922e-530779632', 43, 6);
    T('8.540e+6440', '4.5118291780125931e+108', '3.8531021180227545074e+6549', 28, 2);
    T('4.34116008617983837157965519593e+324', '-7.9255495153466050546485e-125311912', '-3.4406079217064644237559775870350505112406104624780605e-125311587', 85, 3);
    T('2.87312817016522697537848e-421', '1.542e-9935', '4.43036363839477999603361616e-10356', 29, 4);
    T('-8.23640158987986e-328042', '2.754904314324287513874262356280345e+619288652613414', '-2.26904982744674472505682016294277261642650293517e+619288652285373', 85, 0);
    T('9.314401507483970171235092150322745865755e-9806282662', '-3.78082073448014405852430550309725247606730e-112002', '-3.52160823487685051385714295667398442328734127458990756995070186860452e-9806394663', 69, 3);
    T('2.2153968420741573425042840e-41615279267244', '3.77474344e-648827694', '8.36255469661614142214587920089696e-41615928094938', 62, 5);
    T('-9.6120321411354586417800156765073885004454761502605573072e+25491802141', '8.569523970372456983018972961449716568574727e+107', '-8.237053983745080433964619964211169368930353913368135589060375997693871e+25491802249', 70, 4);
    T('3.27906212292199088724847085348958470558003968046851e+719304912743', '-8.63463503153326e-294097514', '-2.8313504677156043262967798493257756259775087739e+719010815230', 47, 4);
    T('7.66148399611e-7643', '-5.262154e+103', '-4.032e-7539', 4, 6);
    T('7.06e+84256586903', '736979513707456652574584514232348169455419807776685.596958263', '5.203075366774643967176566670480378e+84256586954', 35, 1);
    T('42646597531803592075267891459934982983.32672', '-9.420e+692', '-4.017309487495898373490235375525875397029377024e+730', 53, 3);
    T('-1.898426218239448950946723929835804154532950190751666629346e-76155613', '-6.4504173578131437025097701971623885716e+4425291526', '1.2245641430659304819343036169283128386294e+4349135914', 41, 1);
    T('-8.18119930263164315358010e+95936409772', '6.1467524660972639611032173026528e-592428', '-5.0287806989084268694649329065064938468e+95935817345', 38, 0);
    T('4.193036049509473831931585054935083822130790272739e+800506', '-88539688915103051698138626538001185596573', '-3.71250107433381451e+800547', 18, 6);
    T('8.5481108192994190665775592074e-885708336267775', '-5.2466157617e-83294684', '-4.484865295729463262635932381305979927658e-885708419562458', 97, 3);
    T('9.88e+7306', '3.37442e+467378191005470', '3.4e+467378191012777', 2, 0);
    T('9e-592660241', '-4.84061976059238322811802797676583599378284266410e-62425044236800', '-4.35655778453314490530622517908925239440455839769e-62425636897040', 95, 6);
    T('-4.461969097189161096486561731872742066248880389e+5919', '7.79536219451112e+7205395', '-3.478266521330529972308225357332414975474476150413974121042568e+7211315', 90, 6);
    T('1.2048792392075004324e-485910172', '-3.40789436881035831825650487083109001188355272225053950148e+74', '-4.10610117439174942091989182020391860586891328543076673019784582141038044e-485910098', 72, 5);
    T('-3.1001060280610349216257782446795045274e+3287570', '-7.95959929526431448810272292803281983923315973e+10373', '2.467560175619926671701404281665159e+3297944', 34, 2);
    T('2.3884622156691045835613140e+37540518', '2.424767266727768468780368142527268272955917786735329e+520732815', '5.791464998371e+558273333', 13, 2);
    T('-7.56311719197128039306729510880084394993866344951053936185e+3506677232', '6.9e+825429', '-5.218550862460183471216434e+3507502662', 25, 6);
    T('-7.9249995128707939897393630793506750793826167198649087e+2192', '1.68598814024647379362968116007006e+986647876', '-1.336145519015924071353449e+986650069', 25, 3);
    T('9.56993430732922311826624392368e+501231000251193', '6.24448551857436696059514365326311905689116738670847e-6078325', '5.9759316195825349101400238716427530869014469463853874627334049798182375e+501230994172869', 71, 4);
    T('-7.2561444528858243770624120473151584305360653e-834556225120', '5.932263354815613816125135301500e+349488664', '-4.304535983510316716570066441545694538474809829257312196068358213793918795e-834206736455', 83, 5);
    T('-2.487425958497936326990397384416014317891654744e-83603036016794', '3.34165734404102934189840453752274533440271180632074717e-6055786495989', '-8.312125221972925586030025282334951930384789638901127715994002403060442722471809585427936289755074e-89658822512783', 97, 2);
    T('-7.3790314639965248619621159428778677071575654e+6158469118540', '-7.55682523882901640284434034589404e+333', '5.5762051205242365541927389517262847231166554309103428965283997384103212770216e+6158469118874', 88, 0);
    T('-7.805897953861520348091e+95744909558', '-4.4057607465865987286520558178237627584425313850643707051e-35438407', '3.4390918796983735285222435534376307814278479e+95709471152', 44, 5);
    T('-8.606670e-6707086', '7.10408182690e+3118914696', '-6.1142487937125423e+3112207611', 92, 6);
    T('-1.374585623810631952721993852052587748584650615318042897e-50883408', '5.28144814471578537396742839857636550e+8988396367869', '-7.2598027e+8988345484461', 8, 5);
    T('-8.3492786251919025300203996948846e+39', '9.47514272451048689307213358036686851e+1012', '-7.9110606620397975665e+1052', 20, 2);
    T('-1123635611560633471950119283225427506504877906878443429.1992', '3.787181007112317099113274707446034260901512889707414e+121639976458', '-4.25541144701746420620881971016300457148109100572048369827560944706e+121639976512', 66, 5);
    T('-3.2825e+37169534', '9.78158450347792913e+91878887626', '-3.2108051132666302369225e+91916057161', 59, 5);
    T('-6.61656783712e+467030591623169', '-7.779656036796914114590622985915500598931204e+51', '5.147462191692690915586727816415228002095230103395749248e+467030591623221', 71, 4);
    T('-7.7276888106327431174541379799989267524534362513860932175334e+266823638705588', '8.359135466717378251564e-466619745', '-6.45967976127151967815806923671950314534181583588031342192086e+266823172085844', 60, 5);
    T('-8.3089866476557590726555839318573529381e-5679259728616', '-7.5668284569819118819405e+661', '6.287267661416433615946859655650888463636629250864248800438305e-5679259727954', 91, 0);
    T('6.30530230075226531376360172035379409061634536228766721765919e-80983594642', '-4.6397761484816201102577504405646e+2248181649', '-2.9255191223996643449064848921745192712122088775269608206077125323896127253783748904e-78735412992', 83, 3);
    T('-5.7891058506725707268913129714610303640199636743e+356379372', '-6.26479751149492283986132082350576e-38099362185267', '3.62675759270742194736728144533757312153286047976529e-38099005805894', 51, 3);
    T('6.8999e-190', '-5.815169073e+3287882700', '-4.01240850867927e+3287882511', 23, 0);
    T('9.3161969583235760858503570250026453324265343615857695e-4870596', '-2.2718159647489087291622376e-9348374', '-2.11646849806647240540866883773352595e-14218969', 36, 2);
    T('-8.8327933690207528461e-653', '-4.8316879728960087780023018958715771411132339050e-22045344013', '4.2677301488173189337809257094064725884287710780003776e-22045344665', 53, 1);
    T('9.55576546426511118012369620168312309692373331224343166548e-639616642', '-9.5646206182221410440133665338621852410858079e+578401800754', '-9.14e+577762184113', 3, 5);
    T('2.138946231592890e-936919', '-6.4232408668669916751845419974380775028497916105161e+52', '-1.3738966846798599899501123029181550635467515295113445007777990529e-936866', 73, 0);
    T('-8.84677e-715536', '2.418020263521e+30', '-2.139166912670967717e-715505', 42, 3);
    T('-5.00382844097924004224e-13367003566', '0.0008', '-4.003062752783392033792e-13367003569', 86, 4);
    T('5.8733349159925286e+221986546582', '5.4404893355e+738965747', '3.195381597e+222725512330', 10, 6);
    T('4.26931045218617097739461102623151613992144098651313e+429870302', '-2.54049273911241842396183399265e+33', '-1.084615220479572319672209661164617785900209915196e+429870336', 49, 5);
    T('9.6269134485372099847e+14163884', '-1.59010712335348958317655521363840284769111231211873169247e-737775600767', '-1.5307823650426525149418383096159939987396456492754770378462528813655213805209e-737761436882', 80, 2);
    T('-1.21e-29676704540', '3.9849812798799143186862965122550846696745010486419247039876e+7788', '-4.8218273486546963256104187798286524503061462689e-29676696752', 47, 5);
    T('8.88662280427576605750633916892450486643340683543034e+918857', '-4.74987e+337287394392467', '-4.22103030593453329235676352282994379299260461254055090558e+337287395311325', 72, 2);
    T('3.8047747619238966263830887557700094046838615701916671795483e+4099247', '-8.782323497e+252036', '-3.3414762792436818267683230503235e+4351284', 32, 6);
    T('1e-7415049637385', '6.79e-336445809157342', '6.79e-343860858794727', 76, 2);
    T('4.232822539200069e+16949363567', '-9.343352866382996186984661274727e+671216', '-3.95487546e+16950034784', 9, 2);
    T('6.138482571773581698475199438899784639609692339e-79600139', '9.05093925924728394623791354768993674512131302367423583e-8659808386', '5.555903290107074404803391e-8739408524', 26, 1);
    T('-7.46876007e-77', '5.802861708497813329906537027148e+35', '-4.334018182016044788071968058033948838036e-41', 73, 6);
    T('2.613531513379734732903958819e-28275237', '-7.233354282819313754326121424269251369073068343778779097e-16870', '-1.890459936558855e-28292106', 16, 5);
    T('-6.97572483023546314194392439936307450990206089977565452864e+6599275', '-9.55072589564318086646956429607202361999e+697621700', '6.662323577701e+704220976', 14, 1);
    T('4.22042636494948239429041483412708377568294293866e+46950', '-0.00357239628200760061754402464024325469684', '-1.5077035454632e+46948', 14, 2);
    T('-4.38841802391848717727464322956677089746586022551e-315585079359', '-6.17375273521e-66458', '2.70930077784116234127846408e-315585145816', 27, 2);
    T('3.77586361033735019329792825606321803560693822040080797e+148', '-9.053105804085981973601e-105185', '-3.4183292766182115638190999e-105036', 26, 3);
    T('-4.1736034962108457773296394544753e-40525322050684', '9.7235602294388550760138344845577013098390289939e+56904688694810', '-4.058228496920273927847628812e+16379366644127', 28, 3);
    T('-2.92373788e+51015655', '6.3006509386e+573493161', '-1.8421451817842374168e+624508817', 60, 0);
    T('-465909596104346581577023833549689753605798477.9', '-1.116037533821483831700764624007083290603617e+126', '5.1997259662005856971415919719800041596102799895023143132320342675594730590279555084e+170', 83, 2);
    T('3.408011285536643657032148770603198730525217e-78940980808', '8.0007190152552773962019330421521751302864271772e+80993600821', '2.7266540696397607633294848077198328747109809923799248885872312573768e+2052620014', 69, 0);
    T('-3.527973643153575425110489593004282213661642411597383e+5971143', '4.0472850389036416704133822615728764233431734943997971e-29468673285', '-1.4278714943581840950327856713615068725427310558911306170235730748346590646371578165971e-29462702141', 87, 3);
    T('3.73820326758444805181386939478992791945775175487603234e-42411774336', '3.9982026e+14272', '1.49460940237846359203271473303095162613885736565e-42411760063', 49, 6);
    T('-4.983456033786160005157935533863402e+853843', '7.2571661493395651105910131808769485488681109217813716e+652335244064', '-3.61657684e+652336097908', 9, 2);
    T('9.700316115521734396730985957568315e-94798473298460', '2.5304696409846513534892441497099505663153251e+892599318', '2.45463554382819110434934781973969149891852514308409898626752500380851146842065e-94797580699141', 86, 6);
    T('4.79920229379049632279279973301985501256014e+126', '5.4655993780577975460523680892601032198700e+325188182488', '2.623051707211489207975760022335e+325188182615', 31, 5);
    T('-2.251487319642273137144388324430106e+3629220185', '2.2663100053330701007870509452e-19402867413', '-5.10256823938581974e-15773647228', 19, 2);
    T('7.07998589989518184471587e-1873853212', '6.537023e+928506', '4.628203066729050131e-1872924705', 19, 2);
    T('9.637458574508392123886353600623793e+203133986795348', '-54142069526053551920303465875399944726715.7', '-5.2179195219549432205543578922488472231947835458326920187756993459243216665e+203133986795389', 75, 6);
    T('8.773834033442109158083079051501e-517997', '-9.3941739467596001167131018e-133427415703', '-8.242292309e-133427933699', 10, 5);
    T('-7.3649342231856490129140602713104701643437393700312026e+115', '-5.6448632789460381549844009071188e+601753956196463', '4.16e+601753956196579', 3, 2);
    T('4.63643648432217076300300095450820644434587684088223877596125e-18415', '-5.6798355293520798007089829268379e+62935260', '-2.63341966732373126155094908815933235445e+62916846', 39, 1);
    T('5.968710677127768752e+260498147', '5.503959555106596655700275275082099437328842552140e+36684077622606', '3.2852e+36684338120754', 5, 0);
    T('5.8952953500386726213928309579364621501e+145195202', '-4.7957248231089558834819978582253330117881810509391031e+170656981240', '-2.82722142497392634136348576511549460542058876748532851742846683943035673922251768131915753e+170802176443', 90, 6);
    T('-9e-48324', '-6.37037738e+87807137179', '5.733339642e+87807088856', 92, 2);
    T('4.9378857060677628692466452552685062738892906203660492e+2892062083', '-3.57613152884093653e-55855', '-1.765852875928191617233789415772443096662775725329339710337135034e+2892006229', 64, 2);
    T('4.474153465856140333661943220105634e-356973442532', '1.6e+468961209572', '7.1586455453698245338591091521690144e+111987767040', 52, 5);
    T('1.147223736905e-8645144', '6.481117713054404449500021498744717471620e+3981', '7.4352920820914613741246740226677535444203396841361e-8641163', 67, 5);
    T('-6.0014927e+343458427716744', '6.3206025274e-5846561939', '-3.793304992779264998e+343452581154806', 52, 4);
    T('2.3143496002582686253640499711819631891800e+448959655', '8.25e-71', '1.9093384202130716159253412262251196310735e+448959585', 73, 2);
    T('-3.832431164207150041519983e+49767237', '7.678085221118655855090010089600e+459024603', '-2.94257330828534833131867201062371999706483224e+508791841', 46, 0);
    T('7.4786292427655499615433746179123899908900e-20474333', '-1.1e-801781', '-8.226492167042104957697712079703628989979e-21276114', 78, 6);
    T('-5.4e-203803357011689', '3.1274443507683759592790273e+91', '-1.68882e-203803357011597', 6, 3);
    T('-6.806190655073187728011236655092312654e-667235554515', '8.18574204359745620578375989234942845e-744674670686460', '-5.57137210019727048718341e-745341906240974', 24, 1);
    T('7.098039507032183493522e+7098866', '5.03144346757680793069032e-99702', '3.57e+6999165', 3, 5);
    T('-4.576839695221160450542477025038889154584699050566576491264e+35679', '-1.639145747043651128441192634767232551770024993988370694919e+147484388', '7.502107321322325594302152885614756391517303323485546160907512942949473412220852067141434058636493e+147520067', 97, 5);
    T('-4.52467026143290154084293877826933390153164693656372e-7533', '9.01860894740669216271918142082960612956579419e+487261006576206', '-4.080623170382374281062952432045438952226413438210267453288728003706781023277637424887893134078e+487261006568674', 94, 1);
    T('-8.425908444021186894925558404328e-4223456112287', '-82010451328284446197263117899367530.2940336730', '6.910125543449804780624729790311e-4223456112252', 32, 5);
    T('-9.132075131610969627515156076645716795951772556e+822694451223654', '-2.108654108003877223890373489774632566260029925994182e-898539096614054', '1.9256387740871518862792031375588852790780169359402648683420238e-75844645390399', 62, 6);
    T('1.849746419142520529712059712152780110162721e+62405521', '9.465001372531528987663679608474634267843333145870041e+231878', '1.750785239602e+62637400', 13, 0);
    T('3.873e-819711436', '3.558063387218203805139269806781443879181516768353994e-41261', '1.3780379498696103337304391961664532144070014443835018762e-819752696', 60, 6);
    T('-5.86888300173899796e-5026165', '-3e+590491131051100', '1.760664900521699388e+590491126024936', 60, 6);
    T('2.897819670424989140143333793333602860040771813833e+85', '5e+579210887', '1.448909836e+579210973', 10, 0);
    T('-4.13397093657397099877118438605127215e-663219184', '-9.9810752701161957116014419185480433030245516986e+52', '4.2e-663219131', 2, 0);
    T('-3.86836132007972661270703306924068459122132618933821430754e-334303762072996', '-7.60062e+35084', '2.9401944416624371687073329686732133e-334303762037911', 35, 2);
    T('-1.1307515705347339e-526676553193', '-5.7e+6442061507923', '6.44528395204798323e+5915384954730', 54, 5);
    T('7.1227943287985755082072584928134e+3848856943', '-3.923539164666077644792945906240754806812081968934457060e+79878263407', '-2.7946562510902638144733089254703476165046649527390437599944883341516380370625640892604e+83727120351', 90, 5);
    T('-8.307195854833766936847875076217005827575276865735791811e+58', '-3.611820273049066472106899420608576818671923619474713354e+8224700', '3.0004098400677769261162017431077074533465766381340771686144530667329045810467601523847e+8224759', 86, 6);
    T('-8.3498146449356443491191409375169629173820267704974654138e+1091693', '-7.32263563909144091004927747824634368720409537204511149e+22030', '6.1142650298813394822164479706372214924541589857374458068759122532881961259306e+1113724', 77, 1);
    T('1.025095937080049477085688187603272e+753', '-7.851055676938470082543630779149e+102', '-8.048085276218883182121078759950785963429908748571936440261775528e+855', 76, 4);
    T('-3.12294326591744815073964059349858541045867938158968662354e+8082927485448', '-1.356557062357061170e+22', '4.2364507427207399742768051305698097612619894742925884721754124052145419418e+8082927485470', 79, 4);
    T('9.246913665274496859983609130955638894639369e+581069', '-1.149737501180879001093141320567e-4832003508', '-1.0631523411148023005799270795949894948648172801287949765332268005887602223e-4831422438', 94, 2);
    T('-3.88554498504562592621260084838013765282818559786e+712304217315', '2.22863844159e+678616', '-8.6594749201999236192405510857546218545e+712304895931', 38, 2);
    T('8.221420225721394714577029151261721169406351e-2910637820', '-3.7902092120926193e+867811065400', '-3.116090267601381177899012414619254123532881886301100965e+864900427581', 55, 0);
    T('4.910e+345100189085', '-1.63472162e+96820101151', '-8.0264831542e+441920290236', 84, 4);
    T('-1.92759e-9943483085', '9.726866610874055694895131573289159842674556e+383', '-1.874941081045472101692290666935645162114104740004e-9943482701', 97, 3);
    T('2.74584525052617530e+8956916', '-5.00114701907416775271495519034338014328111320750018e+263', '-1.3732375789507942954600990252902333597138249265431616353435369461554e+8957180', 98, 6);
    T('8.34633550300137696643e+11051', '-9.96506727888900355786372338597973610516154227367218378303704e+9835857', '-8.3171794819588614354407031e+9846909', 26, 6);
    T('-5.69323e-8609232491122', '6.98552351580e+355419707', '-3.9770192045858034e-8608877071414', 76, 2);
    T('-9.07659617801384493e+35511', '-8.3517e+13', '7.5805008299918228701881e+35525', 81, 0);
    T('-4.7448443981973388267799412346349978568e+9405', '4.7874685487691164517547157034547e-5285894358457', '-2.27157933251730854181779653919694299130911842981477018521e-5285894349051', 57, 4);
    T('4.291529886e-2302017946991', '-4.738099219544913645178197e-4769172141876', '-2.033369440352e-7071190088866', 13, 3);
    T('7.80262137862553732242978e+99', '7.2780821197316189460e-35863', '5.678811914281019761221040199562355780261e-35763', 40, 4);
    T('-8.470936764996695759969846437640711578721649549e+78744', '4.13773987527487845673633967321654205e+920392261', '-3.5050532833458810314344075787630590032475985905016150798e+920471006', 56, 3);
    T('1.92905108293445433647414130388896527091631817e-13130752390670', '1.6e-183120178', '3.086481732695126938358626086222344433466109072e-13130935510848', 53, 2);
    T('1.9100198180385705144e-831386887', '-7.2508399173287168747286620e+83126189192496', '-1.3849247939522999476686189876373e+83125357805610', 32, 2);
    T('-9.68176573867360580655260890631091255849651337e-8409952436695', '6.7154704618617887e-507802797354875', '-6.5e-516212749791569', 2, 1);
    T('9.4723885845747030958251681924538817109076938458e+1907949961', '-1.9411463519424639017570060e-689662135', '-1.83872925451286241057462266692058407393135381991511892470737501532336748e+1218287827', 93, 2);
    T('1.992901483012567069604375291305491069785e-61112896', '-3.5056198277965890547682985000964e+50078', '-6.986354953694082317971708159475578139460600778737414742151060161627274e-61062818', 100, 2);
    T('6.4679763670943e-417200898279', '6.0954941578297570664495786582893283683437e-456812', '3.942551215860424181423046945028913980222640619847271091e-417201355090', 91, 2);
    T('-5.902077243684e+130233900', '7.087602864856901373089834417e-17722283634', '-4.1831579580941442405171084846407319072e-17592049733', 38, 6);
    T('-9.12849821692050252141177177564996387848001085202e+750721015524', '-7.9522788628949054417e-466077886', '7.259236342039074566431144e+750254937639', 26, 0);
    T('-9e-709703', '2.8358719891301906678522579436653057e+713143', '-2.55228479021717160106703214929877513e+3441', 78, 2);
    T('-8.1492534756955174250e-81088464476', '-1.6443750674238179142580261430394549863148e-37160', '1.340042923355059894816328681219873873792144474941507993539e-81088501635', 80, 6);
    T('3.64936e-113', '3.60052757964615144611153112e+609180647650456', '1.313e+609180647650344', 4, 1);
    T('7.40708403213462515249718905450254304274e+49357149852', '9.809809721029609614072206807605077020405826155515225e-9341', '7.26620849429e+49357140512', 12, 4);
    T('2.604412689621472923111133655710458033254047222443e+7185029805', '1.01214034028885434455116047320721130946410764244381397758e-43239', '2.63603114593e+7184986566', 12, 5);
    T('9.624922995348049e+94333381', '-4.1395016819079577683917156830456303917377106341210332064245e-3415885160', '-3.984238492747782762e-3321551778', 19, 4);
    T('-2.08828305005e-64047220', '8.198257732308360589381584779739086540782e-252637189378812', '-1.71202826623208996787089915071361970846e-252637253426031', 39, 3);
    T('21846637807429706407018764353547500569525827997340144611', '-8.72080941932744779e+70042234096', '-1.90520364771668125083707190827485623906728612884492e+70042234152', 51, 2);
    T('3.85535619838406e-47850', '7.6536364386237616981906098e+56142', '2.9507494683826221863812014795636625999788e+8293', 92, 1);
    T('8.4415747552414579748190262454145e+379839572', '2.4628030267338129581999e+952258881', '2.078993585760840900404546071388765583727057152755e+1332098454', 49, 6);
    T('7.24696845903479746759333991517276118307059498134671e-272407088385', '7.0364713733958926347553708546899570469928355299353178223053e-4487', '5.09930861059012970288097873444857153303758219529780324e-272407092871', 55, 0);
    T('9.39428383832538335381475873e+80908', '-1.032840387284155340334639e+5874436', '-9.8e+5955344', 2, 3);
    T('-6.933247461e-57', '3.2784942647317794496105181923799792200956988176e+64288551', '-2.27306120368546717150241026962128004749612640041459021136e+64288495', 73, 6);
    T('-4.07002726445751994702309588782549312e+5408', '9.66e-4032497459', '-3.93164633746596426882431062763942635392e-4032492050', 63, 5);
    T('3.366939831057607194589876e+739978286', '6.0550547604689468e+729847', '2.03870050523e+740708134', 12, 5);
    T('-979665090259736080690050.6591342798992075066421647290285', '9.0617746e+1292958', '-8.87750423142238e+1292982', 15, 6);
    T('-3.24939247561254840425501770431103458463601122720796213463e-419782893246', '4.380200855471892364604279193766941e-187906379262633', '-1.4232991701e-188326162155878', 11, 2);
    T('3.939e+46352251113', '-8.6567608581491723382e-4897813600', '-3.40989810202495898401698e+41454437514', 27, 4);
    T('-2.30800372739208053933178204888819e+611', '1.46835003288425497048490026577706831e+60482', '-3.3889573490131445043185234064732922749498337523148626923882331822589e+61093', 99, 0);
    T('-3.069e-463973192258422', '7.5129273430163288990681e+8325', '-2.3057174016e-463973192250096', 11, 6);
    T('-9.0744465737542132833738291e-42', '2.5009639020632241763378967846945229193784e+64', '-2.2695e+23', 5, 3);
    T('-7.71382054490886941366e+248348858', '-6.7024155854782045019080881484289433105090778376e-58773989', '5.1701231043779182474331118895472880659490504478969740544766812701616e+189574870', 88, 2);
    T('-9.79587499909324252277772904e+5624', '8.5946344437648298298688368e+498070549734531', '-8.4191964674021553363790558561669119561785663370380672e+498070549740156', 93, 6);
    T('-5.77006170619050524742e-232637', '6.359597e+121058556', '-3.6695267117e+120825920', 11, 4);
    T('9.85559386383958513e+99565417644627', '-1.6966535582241e+73896184308925', '-1.6721528397495038236944843567633e+173461601953553', 100, 4);
    T('5.59836674e+989444100727', '-6.810010485066054205953527e+71086', '-3.8e+989444171814', 2, 6);
    T('4.9556781048154e+412412', '-5.516942349608598207575678553e+464226', '-2.734019e+876639', 7, 6);
    T('-1.23403626107469997782627291489446e+4128286732386', '-6.9498221822529293294215341633009008e-54610', '8.576332580921417028887e+4128286677776', 22, 3);
    T('6.664725838956977645833e+4899890', '-4.947813773453979744172427862925e-76181987', '-3.297582230228596448185469182997036737324e-71282096', 42, 5);
    T('-3.3462160506634744970829955300019633342e-58743606377132', '9.1845694250585801238349978564027054565215694669436e+644791561443', '-3.0733553628564020580802256229278515693497559281e-58098814815688', 47, 0);
    T('-4.2199485500e-122259655385093', '-9.227760555555912692049930950e+426514540895389', '3.89406747761653682087427026400526225e+304254885510297', 42, 3);
    T('-3.79663848728610e+466', '1.886865e+99981', '-7.1637442793130870765e+100447', 96, 2);
    T('-69235365637540539167164803654140768320870445280072.9602537', '-2.876699214190364244022729223707789e+7950987', '1.99169321923696e+7951037', 15, 2);
    T('6.031263975685319665842696997830590146087165273652165356e-2553087785', '6.01499794233829029607727707090094873167333209965810611836131e-519054287041588', '3.62780405e-519056840129372', 9, 0);
    T('-3.0523e-829650958', '-1.7992439863802241183791057803076467903671e-29599', '5.49183241962835807652854457323303029823749933e-829680557', 56, 1);
    T('-3.8186662080369045033529347986e-4054779856261', '-2.28318222786209766054e-9450', '9e-4054779865711', 1, 0);
    T('-2.1731507205443105e+428485987776778', '-9.876180298446805006422985629656368e-3276', '2.1462428331795197817745445502085988e+428485987773503', 35, 2);
    T('5.443555018833923345388803620755728827467e-260559299', '8.0845373014082715e+966447', '4.40086236020310591828423433548704524525493273885501932905e-259592851', 58, 4);
    T('-9.138718078976e-450610', '9.4185290072701527745673316427702184e-7930516182', '-8.6073281316099622901856001419983348575431969e-7930966791', 44, 0);
    T('-5.196640028289e-540348', '2.56691925363903095858812211288882901093689e-5622786988279', '-1.333935534284631260683192568422199084290716393999368121e-5622787528626', 89, 3);
    T('9.9702e+1373808130', '-75172672473217330995470.746886', '-7.49486579092471433491e+1373808153', 22, 6);
    T('-6.8729503997540432864524428321453823e+4345321985', '1.06106363826549430e-1258092227401', '-7.29263775678130863005012499996959348114926317197089e-1253746905416', 63, 6);
    T('7.3617973650778699913858888126796545497919498761825e+3763', '-5.2556308320958897266187870088557991e+736', '-3.86908892115455343443754751471197186479086408355813001894035384893867955e+4500', 72, 6);
    T('-8.600513950665876152580353440697606930185669663889085e-590752', '-9.701880384e+795029', '8.344115759028360758289272203009101995171080598998938691320864e+204278', 94, 4);
    T('3.934292192397889526057114e-1842976395', '5.4682207831108921841603005e-3936077', '2.1513578333301056366063349991922938170168694402757e-1846912471', 98, 6);
    T('6.4e+665198', '5.9490347439190048017038756909533095031193e+5248724536', '3.807382236108163073090480442210118081996352e+5249389735', 52, 3);
    T('7.383611953031733661e-4694496893132', '-5.656106581963502914608048999770545133e+51451854556', '-4.17624961662072e-4643045038575', 15, 0);
    T('1.29050195997024629922134236265015486202212690e-8457085', '-2.305517608989583171e-60476915792339', '-2.9752749931469730206807129049207243844267962655613790098663999e-60476924249424', 75, 0);
    T('9.634375221416479882139061519213457944586973293435271057e-38860681', '3.4166917675902102739359034746470148e-42514', '3.29176905e-38903194', 10, 1);
    T('5.1430181918150989704170965979461259733049960588550821360023e-548524', '3.319874001271124713999472286513516373097000363e+744943496', '1.7074172383071377408e+744394973', 20, 0);
    T('6.828987248016825261199191518860666e-544', '-5.87463600351790252867734960159e-38863057', '-4.0117814354764281793e-38863600', 20, 1);
    T('-6.331380971614915e+85452639517370', '-1.6549647622538749209457278e+3690055999283', '1.0478212404427385402271251813679036010137e+89142695516654', 81, 6);
    T('6.088191963628949041465038327851662715236298069997224846e+13600506210383', '1.578810588148758337261103091659859056339042e-20', '9.61210193485956496341330623262633030925151654738e+13600506210363', 49, 3);
    T('-1.68754212104831313994961199944139270452504e+254857426018644', '-2.87381984474931153219712669100672707e+7890913', '4.849692036318e+254857433909557', 13, 3);
    T('9.557458507742129543428561963351446640847174001450e+50', '-4.6010886208016902113774433153516072717569356768801371e-84577686190574', '-4.397471358375661506813788402529939015443339e-84577686190523', 43, 4);
    T('6.41505532488342245895919338e+6839276', '-3.33114626548641967759461036114735712056973462e+1952', '-2.136948758837418342746042e+6841229', 25, 4);
    T('-8.0250367859564554267505276e+32', '-9.910e+24151099', '7.9528114548828473279097728516e+24151132', 50, 6);
    T('3.65375675971924720674195842540769239235365279982215298001e+6233147', '3.6941300248915395614970961483200501065072322823506e-522072505211', '1.349745254972929361648076e-522066272063', 25, 0);
    T('3.62384481939121554e+880584132073', '-5.30805792455440378960027577579602105511783920615470e-843713', '-1.92355782109249638038828e+880583288361', 24, 4);
    T('-9.631e-20', '-2.175241143051121627634382915e-600', '2.0949747448725352395746741854365e-619', 65, 6);
    T('-5e-445602229777', '-6.747964871655996512366931873166454858604393728557100914708e+55015386', '3.373982435828e-445547214390', 13, 4);
    T('0.04173825469484585507789797325633121102783007729760', '4.5652767538345103893888132216372325354539973354196295308e+63', '1.9054668390400389828334e+62', 24, 6);
    T('4.967121860120318526e+3470505', '4.73509649550804153233113080e-2532402281', '2.35198013126171e-2528931775', 16, 5);
    T('3.31832961397359598300003185676515741715e+352372694525080', '-5.6829636500830307560788088630349181168866229660329545e-37057754926', '-1.88579465752060014479421286133978098140533036619882005292366960436616897e+352335636770155', 72, 0);
    T('-9.13233284106826433845112130072902130695168091406e+79911', '-9.9834065565022848398828329023768722256833726e+8558252', '9.117179156168204857886663692804964505671e+8638164', 40, 2);
    T('-3.03030118820720539545253383280048105e-761', '5.64888855010564967766984146811e-2705881500112', '-1.71178336854352279296466692924206611870741587087596403492462343155e-2705881500872', 94, 0);
    T('4.631950e-842217597178249', '9.636778633834920288543272946655212550349192398e+67313523393', '4.46370767929916590305180131252596117725899417279161e-842150283654855', 58, 0);
    T('-3.65127536128461383962161483249163259e-32924788385', '3.079837145884002723199753997252064234863087999253e-35918976884', '-1.1245333487535e-68843765268', 14, 5);
    T('1.7960523853172713312474708781789030703352217113267e+606726933', '-3.942075657731146800e+5671', '-7.080174388169177490731728310258794625204731145e+606732604', 46, 1);
    T('6.15800281694634254065351350034829e+929055674', '2.54123213993920830572845357243721450587233087857e-95782', '1.564891e+928959893', 7, 3);
    T('8.1814844026267614855809938590696441075e+900608325', '3.272576345909491615403841147016991542352379667762065360e+247325', '2.67745323304637869674157626759808555419284889094564198875123190960090597797790598e+900855651', 81, 5);
    T('-4.23404357e+8363', '2.26220148285241517160272497280963838032271682753220811e-1778939', '-9.578259642515733716294964265603074218230613709e-1770576', 46, 3);
    T('1.960410175023934837985330310871442896462446e+9390', '-5.71746170146489818855785803918614e+2491878552', '-1.12e+2491887943', 3, 1);
    T('-9.715748923893e-680', '-1.7629292e+16330388', '1.71281774777995473756e+16329709', 93, 2);
    T('2e+9944769', '-9.1917715005554e+3951445', '-1.83835430011108e+13896215', 34, 5);
    T('-5.899786366191694762092953631749378029504377620e-44835', '4.4131966224236953895011719641049760173350211251070e+58', '-2.60369172642985546109210106993772744538296500845968404777962e-44776', 60, 5);
    T('3.587869358583027702197258220483202244292798239e+31269944496423', '8.07945988703e+92976176147595', '2.898804656257562755967002038484095203668653412231352294017e+124246120644019', 96, 4);
    T('-5.68454242636490356e+24072', '4.38e+4604875', '-2.48982958274782776e+4628948', 18, 5);
    T('-7.4056473e-7914', '4.78346630e+10', '-3.542466428923599e-7903', 80, 2);
    T('-1.79e-1809401393829', '2.680214787180054515900218205e+54100396768123', '-4.79758446905229758346139058695e+52290995374294', 90, 5);
    T('-5.75008153041169124010670090486393031e-103907915', '-5.905e+318009585471', '3.395423143708103677283006884322150848055e+317905677557', 95, 4);
    T('25253014102231.7', '-8.820383380716865547378920663405e-75591', '-2.227412659003331233665099303978333562550209385e-75577', 65, 4);
    T('3.760309318179192170456213091e+620746240', '8.4009e-586597989', '3.15899825510915755047856005561819e+34148252', 76, 0);
    T('-7.5297e-4969496', '6.574950796982190906952845622517147104441480381569316055382e-90917473', '-4.95074070160368028720828416838673625523130148291024791022098454e-95886968', 79, 4);
    T('-4014545859961646097844105688992862195359410283261.79935', '1.11620681512626e-57861', '-4.481063448526101573310505013947900053108273913343980005997205035931e-57813', 82, 4);
    T('-61142149868400119757407137942843635.96067462709634477064280', '-1.34205805721874419118003852885484474966361374109792552e-24913907407', '8.2056314866562e-24913907373', 14, 6);
    T('7.5638298436744769888571840014449736048e-956', '9.6479513302628422034659314665925062880365554467989e+139', '7.297546220216095e-816', 16, 3);
    T('1.18190922151803621e+38238623015540', '1.543415113940413729086837851356773484495496207572325244e-70785170593978', '1.82417655579648554704513966029851018353493379443680184773481559428908524e-32546547578438', 100, 1);
    T('-8.14388371163450859e+80177125', '1.1232355297187709898033665796458475567948644567e+22061', '-9.14749953480585806699874465821264057227505344998885e+80199186', 51, 3);
    T('5.6480492598037e+33944564', '8.433694381303570e-7398240382', '4.7633921307732252166808309209e-7364295817', 67, 0);
    T('1.836580140322441e+1812730988852', '1.63769381352092631217037730452832873044949601e-99108189', '3.00775593384145637026105625482598924218437648915010734296041e+1812631880663', 85, 5);
    T('8.5919061e+830', '-3.2254317188877615409184226654e+2034', '-2.771260646064524359876239530122851894e+2865', 43, 4);
    T('0.000963123203895724210018615685058834', '5.739979422612789798927326174823681842045050856737976667821e+5524', '5.528307371802359273751563277437708022537679075689385082993783e+5521', 61, 4);
    T('3.13275529526e+385228210100615', '-5.60413478857251742435456709119084721754130520759833e+18995844597978', '-1.75563829342513344976553865426930619199981610468754818946329158e+404224054698594', 75, 3);
    T('-9.9542139e-96383', '5.363709872700195596405042658475e+650517506', '-5.33915153703995175384538656610847978025e+650421124', 45, 3);
    T('6.76967595768589242058093e+1332049', '24950.082798847', '1.6890397566562687590617479364007418771e+1332054', 77, 1);
    T('-4.51651739661220630133260248432523973058118480763026408241008e-5866923583080', '9.8857260355199e-154019839561', '-4.464905361756784602637344962966e-6020943422640', 31, 5);
    T('1.888984369528432457399845305525753308305433234825911948403e+60744723988560', '2.86860742e+69', '5.41875457869328324e+60744723988629', 18, 1);
    T('3.48904178e-447405399', '2.91011418317540273254840080769720681094033879297643404432e-16332313811492', '1.0153509969669553202187536290241300152671403136049548936046e-16332761216890', 59, 3);
    T('-1.466230277884e-70011048', '1.26618005668321298069241225198806593233202183168772642e+32926796', '-1.9e-37084252', 2, 0);
    T('-6.5366376e+4219427272433', '-1.3816756632538527041705663951657068890547389276711e+56', '9.03151309143007193094300111193661788157423493279859269336e+4219427272489', 94, 0);
    T('6.355337273867627628430855867981308490216225626e+102', '-9.2210184113e+6148117793784', '-5.86026820123545447208134772876722877525488224153408679738e+6148117793887', 90, 5);
    T('8.2826197088902878620556811931446409063e-693234', '-6.629776834e+55604396692428', '-5.49119202708326543154481427923998198918365046542e+55604395999195', 97, 1);
    T('2.57630457145311134307572e-158501435356714', '-9.7569768029268351292434014671e+32', '-2.513694394094236838556597385094651228273217e-158501435356681', 43, 0);
    T('-7521206.860678916130747549153261628967497985139638', '-7.536805964116442184983137197337380377506123e+50', '5.6685876724918357943841547187305016e+57', 35, 5);
    T('-3.534212321414412460590267320e+69197075650688', '8.48199603042929488962643252049923298048754762302297235e+309349', '-2.9977174880931349763947156917631130997e+69197075960038', 38, 4);
    T('9.906285852565505348991666235176e+255172195', '-1.0e+194', '-9.906285852565505348991666235176e+255172389', 96, 0);
    T('5.494410369199285804383444027896e-86586442', '-1.37816441979411344700430747446465e-7078', '-7.5722008785782944e-86593520', 17, 6);
    T('316982.810200', '-9.77386447852133449035692e+2682128405', '-3.098147028915650147407521302616584e+2682128411', 75, 4);
    T('3.58608e+7197264', '6.67174778506774944e+78028', '2.3926e+7275293', 5, 2);
    T('-8.74154569925178726478819120601e+259082852633302', '2.9366315244029670727169504e+3236887', '-2.5670698672431976776567677469063978949906798103145351904e+259082855870190', 82, 2);
    T('7.5305941246e-8037', '2.1e+9693109259989', '1.581424766166e+9693109251953', 51, 0);
    T('-6892353.44307', '-7.2129104938263041e+30', '4.97139e+37', 6, 4);
    T('1.8635920622232768435240654178582700371e+493079023196', '-1.759183886141889601999516e-401', '-3.2784011262051222932427973442114551346725e+493079022795', 41, 2);
    T('1.470377775906515343780055340351384026531250636696912e+7334', '-7.551390228230596354328951163317639974535306821690193273e+83728284925', '-1.1103396368787897563163124441450245124536657664490043554239276950692e+83728292260', 69, 5);
    T('7e+8907973085905', '-3.55500444606656089544878250053982081161239979188591644541543e+71510039', '-2.4885031122465926268141477504e+8908044595945', 29, 3);
    T('-7.61037549563666605135391890239313953440643800e-1300504628', '1.156888819e-53949492525070', '-8.804358319293642241248228590011375469661673923816722e-53950793029698', 54, 6);
    T('-2.16815260558087192026155658864362468766706393210978292261e+383215', '7.116578925162661776515120249350984868e+78993802061808', '-1.542982913941334604543734556e+78993802445024', 28, 6);
    T('-3.01699378089492823613486e-22131091862', '8.15350713007416031531965140985925243934251969e-79774888196842', '-2.459908031e-79797019288703', 10, 3);
    T('9.924342547541528508472865e-9854', '535174965866290282207470517.57789', '5.3112596841259098618018253530995e-9827', 32, 2);
    T('5.9e-829', '-66335936672622431654187391318417087.14', '-3.913820263684723e-794', 16, 2);
    T('-9.9272951889e+17416891271', '-7.3419741387652534257e-74592562', '7.288594454479252131944086e+17342298710', 25, 0);
    T('-1.82082425059390160983685e-2155305184783', '5.17396069948815128541217e+63318378306', '-9.4208731132478120368057579173308653943787044645e-2091986806477', 50, 3);
    T('5.9103857666911e-5535664777', '-1.5770492069319185594595170768530702227723e-797362338357205', '-9.32096918602189869181140330070878656988501e-797367874021982', 42, 0);
    T('8.2047475510403474136121766492173977305697617381357418440e-50385356', '8.657079083e-8248439760', '7.1029148405406866483435123944040962213007154015508654132380249052e-8298825115', 75, 3);
    T('8.04699335553030642e-9794228644', '3.1962248760496049610853e+60666093881971', '2e+60656299653328', 1, 1);
    T('-8.8774389e+63288644214', '8.8156668255460975626043512179910091844357322037830646743149e-65863947179', '-7.826054360654244024545905281185576478416704361584650550097892410961e-2575302964', 80, 4);
    T('-2.493371170568778604022e-33055920442', '3.886409308202462117388322989983422487401483731e+15603286', '-9.6902609262e-33040317156', 11, 0);
    T('-6.7535551709523576561136843619778809526693932652892909764125e-42769891', '-6.5790206434128e-8601320', '4.443177888612282256366015781268249200163291350388534624056288289830058e-51371210', 88, 2);
    T('9.456535092369e+2383130', '-8.65698866844e-133065', '-8.186511713734364171513436e+2250066', 98, 2);
    T('-9.437096748542201184610885655e+3786', '-2.055387400001636e+5491', '1.939688974955004777339490331926577040893158e+9278', 60, 4);
    T('2.7590956316746457e+7626', '4.901068835177612048172653382304229e-746433404327287', '1.352251e-746433404319660', 7, 1);
    T('2.20890329374810147185158903e+104', '6.369686910507e+957386092890', '1.407002239676308075253508755331935293821e+957386092995', 40, 4);
    T('3.595670054505324041428987343366e-244185', '9.74459114073624487562057323714273371602470909219e+56', '3.503833456e-244128', 10, 2);
    T('-4.492e-6576457477465', '-5.869264806648088e+16535', '2.6364737511463211296e-6576457460929', 72, 5);
    T('8.99076', '-9.45015156120650608183275954772512154e-652762', '-8.49640446504330066202987012313051137369704e-652761', 57, 3);
    T('9.34671280435087268795235007759322256656196314272622165854327e-818202111190', '3.532599374451424664023e+984202877', '3.301819180582701402123773406170040601889026280778e-817217908312', 49, 4);
    T('-1.614830e+7', '6.54671714272247422036804046886487e-174984078', '-1.05718352435825330452769227903370580221e-174984070', 62, 2);
    T('-1.6642561853740223214068342903105221745081058518e+92661172', '4.202804823615132288364e+7528', '-6.9945439236212607870513630382850462434987703e+92668700', 44, 4);
    T('-8.49207029963405240036130057503923421762832245941153183e+9611793', '2.5777301357152734868456104657e-9487219521', '-2.1890265525979329078337706085615247561753514906252517465119e-9477607727', 59, 0);
    T('7.74957824284387504129647073082552039e-833443768147896', '-1.211096968870243459243907082335141151178030668474288273785e-818', '-9.38549071993100453761880332264507993098320437213025625309e-833443768148714', 57, 5);
    T('5.43998334e+68600', '-3.673261e-6937611500', '-1.998247864347174e-6937542899', 17, 3);
    T('-8.7475496818433643625258951874731631235162651e+19177238367605', '7.8077338072427353627168012415863849553272828e-75426904336755', '-6.8298539381463869656817924116792351299493659498513854188897218678718475e-56249665969149', 71, 5);
    T('-5.897e-7768813061', '-2.7789830825430331123997432706524191320710360956257643357723e+41077', '1.63876632377562662638212860670373156218228998559051322880492531e-7768771983', 67, 1);
    T('7.6123462378554601999026036203650727290511763673e+75739', '7.66736678239320264236740611e+27225823', '5.8e+27301563', 2, 1);
    T('-8.0192872641950e-3201', '-8.6872154066034397318028822037597431e+9647566461992', '6.9665275871493552744126099e+9647566458792', 26, 3);
    T('-9235121635.3646558', '-8.970421755925783498702606e-281278414', '8.28429360364960089649389524238270077530148e-281278404', 81, 0);
    T('-9.115513239983777655832332253359e-83', '6.8334703953273585439272357730692349e+429791189279', '-6.22906e+429791189197', 6, 3);
    T('6.9766301174e-150652957665', '-1.25480450941193993434372389000882332833101405750078451977312e+994066976609', '-8.754306931812671909102202694705441784169861350042748654776844806964288e+843414018944', 79, 3);
    T('4.12226055725950435866360191865460569E+31759265942', '-5.150135128829E+418464222', '-2.123019890612838281139065648960341e+32177730165', 34, 0);
    T('6.40430397221166940643681521750689691643917634943221826811E-16', '7.4386521483927337690380689806004525070E-79141180', '4.763938950185245337792926605943621622711770429207155813047122e-79141195', 61, 5);
    T('-6.36996603495781163376417234694190231163164E+476740442999', '-3.7996207837209996872438947608940991374810493679986E+1772862975', '2.4203455338022549130808348359382833626972e+478513305975', 42, 5);
    T('1.3944328368640893975028146190E+244', '4.853374289E+262882', '6.767704478173502869237661276987930891e+263126', 55, 4);
    T('6.7283144630009346775632292280709463327E+2197', '70.61374', '4.75111448128587621078433702271402505891231298e+2199', 47, 0);
    T('-8.47462438734192225E+915815171', '-151217.416741244971516987454267', '1.28e+915815177', 3, 3);
    T('-8.762770851693909E+59296798768329', '-9.23645073479309604753927028066485089E-7443', '8.093690127195172985434997037847e+59296798760887', 31, 4);
    T('7.730851842998474400376411696252E+1676', '2475103167163853077962286.221527', '1.9e+1701', 2, 4);
    T('3.823109297527614311061048224961483434290103880240791E+872552525668', '-7.176248593221408E+91', '-2.7435582718114227561745505690868397539909438048621120234873416e+872552525760', 62, 4);
    T('2.085959044861191852197302202218404649E-529568437177', '-6.01E-4050155570511987', '-1.253661385961576303170578623533261194049e-4050685138949163', 45, 4);
    T('-5.0192416731212117605146E-8', '8.657798383732529755496681922660019703516302491968297E+94810887473394', '-4.34555824451117856187741250686258162090745274898597594504142334059396056362e+94810887473387', 80, 5);
    T('7.054124552203270942205823029E+34845807828784', '4E-5383', '2.8216498208813083768823292116e+34845807823402', 67, 2);
    T('1.496666461086156367163579962788574070905253215717638673497E+81', '-7.3069962089557730936589002849437E+16755', '-1.0936136157e+16837', 11, 2);
    T('8.0379400123724478436E-3270924', '6.224650911943761153340936253466511317039759200538E+27417527695957', '5.0033370628163404277526606315315125660493363037695976376e+27417524425034', 56, 5);
    T('8.77641694189101180260513383137791E+2592', '1.5933377934538978397696178711823978182621607198091169077667E+5549697858', '1.398379680462403068294162e+5549700451', 25, 5);
    T('9.19630485913E+5068', '2.292E-5209180989873475', '2.107793073712596e-5209180989868406', 22, 5);
    T('4.68060780802169420949357373696656832721100056341082873E+469692856379', '-8.167028754454858479670734663499834001982020633227957542166E-90841261552906', '-3.8226658556439102616221e-90371568696526', 23, 6);
    T('-5E+8871808418733285', '-9378684.7647829172694969746056', '4.6893423823914586347484873028e+8871808418733292', 49, 1);
    T('3.739277400349455547803E+65145084275', '8.02196786236761970749915868754741191870630674116491010620E+71058', '2.9996363134080872037847952052812995942427637e+65145155334', 44, 3);
    T('1.046002762185637871517670802226849644815640E+96206416928798', '-5.741646752192428178680E-8194256516081', '-6.0057783622874765124692593438812028702564002101473585095785552e+88012160412717', 66, 3);
    T('-3.226884086E+725', '3.894E+39258900', '-1.2565486630884e+39259626', 52, 3);
    T('3.09172E+958147756223', '-8.967074985E-4438', '-2.77236850726242e+958147751786', 56, 4);
    T('-5.41597312194029366432943508989346647769978498861146921157517E+442541101224671', '5.682786E+89716584854760', '-3.077781623373859367154001311675533279094165033629141667497041e+532257686079432', 61, 2);
    T('-4.4709010E+5244', '3.21587499590676610880E+6349901081120', '-1.437786e+6349901086365', 7, 6);
    T('3.17063524866455769091616792141170E+2030', '8.4717166187432702866962971848435935016054915E-76355', '2.686e-74324', 5, 3);
    T('4.6053811439698265E+8734164038763296', '-5.662695262312599E+5978335352', '-2.607886998510171393274e+8734170017098649', 22, 5);
    T('5.3255845361595E+105596', '-4.67618804449189303535864093307457573925E+136633', '-2.4903434738e+242230', 11, 3);
    T('-9.220169438538721E+6700014530987', '7.77660034428455308841899251287401969662540E+676', '-7.170157283010213227540262523029802601656354957951093211e+6700014531664', 55, 5);
    T('-3.23121833143057213178668780122820073129139888012E+49', '3.2023268305422191618446788113865092014056531264094512915719E-23384983295534', '-1.034741715807998191527625259327720865916680914955636087e-23384983295484', 55, 0);
    T('-2.93482648394199867408E-29639010', '3.595380384E-82283', '-1.055181757080875302654124124672e-29721292', 49, 1);
    T('5.325732541199689398685450214253417230500410343E+5463263', '3968.8793715677879096853696731895561049890614919263597', '2.11371900212548e+5463267', 15, 0);
    T('-5.22324086318795338068476103242550935939751387628657E+2696', '-7.509933737998232753644292695E+3944468260532', '3.92261927801462225753951940312093824832696315597212168330072819796797808e+3944468263229', 72, 5);
    T('2.289263603812621507365432148512038985369087753E+974061453', '8.0595218526472087648676073537216005614184572562E+55347', '1.845037004139772502149681374101292888056463319648676788243051672009487107e+974116801', 73, 6);
    T('-9.3217154366674817703E+556715815452916', '1.714446075412851347319837915E+5273365839', '-1.5981578446509957978585187e+556721088818756', 26, 1);
    T('5.7199883516462616061430367991E+330712676', '-7.2903318084050939792399853405429983290E+7898777', '-4.170061302371336299412849804117246116101340984499705e+338611454', 53, 5);
    T('6.43322684381112104882683160018023967480414694399E+86527', '4E+1939185', '2.573290737524448419530732640072095869921658777596e+2025713', 78, 3);
    T('-8.649144242E-17411225822', '5.56897845619589593729852753497663971417E+7297644297539645', '-4.816689794822878257011675246422165698832198130914e+7297626886313824', 51, 0);
    T('-6.137881468383994683644969610446449025089445808470061740E-278061', '-4.694784781411E+6189163028', '2.881603250787378018826713474930957000837570789464124897836357431514e+6188884968', 78, 6);
    T('7.8480471455776E+107', '-2.73628329853456288636102096E+86386', '-2.1474480330555836177362133507575284906496e+86494', 80, 4);
    T('-2.911442E+65835014011', '6.57077075240357841175126511171553E+31815', '-1.913041794091937913826592679938328609426e+65835045827', 54, 1);
    T('3.04125329898687138874094857745256362464125674736558072653225E-742548382033', '-389.35122041297993301383535826247569889711346526733950', '-1.18411568354553972288308653554869352539241965253710108464756653906682320659e-742548382030', 75, 4);
    T('-1.83973730116352490487586600428678743763850E+8792394051307', '8.5408468500168597830832608747826127681721784344E+181062055', '-1.5712914533501e+8792575113363', 15, 2);
    T('-8.95281820896790777350E+85633164585115', '-5.47694681400675624113711024308281374147439776E+36175', '4.903410916599e+85633164621291', 13, 0);
    T('7.4100461212483E+91736584649874', '9.2007520713987951208895131984190507968519173350826E+1708228651421', '6.817799719923591e+93444813301296', 16, 2);
    T('2.58162822475883431038894969393789135359622E+8655496448', '-7.0854252980967557535544348803125200886607061E+1162', '-1.82919339339868619561041873739863991995731929486979497744781331118918e+8655497611', 69, 2);
    T('-8.188458492309920504058521118573046138306209815E+4321138919617', '6.8215121390374611021946229099250E+34750', '-5.58576690052965095489570504e+4321138954368', 27, 3);
    T('-8.4744711054911981183498253422690100225E+949', '9.7E+27010219220', '-8.220236972326462174799330582000939721825e+27010220170', 77, 1);
    T('-4.1721515326872181192277921212286164652296494924908386735E+74878095095943', '-2.3840688E+3131476750', '9.9466962979517768768456592891069621819202922e+74881226572693', 44, 0);
    T('589.92', '2.2072433057000109325E-93124730281403', '1.302e-93124730281400', 4, 4);
    T('3.365146397351792490192051766E+89126699', '9.3351725505285407351226177057E+6689', '3.1414222277068462898388530861063707982489527149922554e+89133389', 53, 0);
    T('8.446912278363466357869362753798948217555760756610919462480E+18568482946934', '2.655057760440802015255524313527607323458939E+1837275055599', '2.242703999643161740894923698e+20405758002534', 29, 3);
    T('7.46851041130861808034482256866E+153036405', '6.48694946362692287172145753152909332163165882792895530E-966485058581200', '4.8447849606730529178104140416113601524737360370711210137779993e-966484905544794', 64, 4);
    T('2.3704407299116041524230093756259751E+85317087537', '8.242565285627669E+54239150909100', '1.953851247200730167e+54324467996638', 19, 4);
    T('-7.9242183E+213381100925594', '4.907064631446026549009740116286396380662893698993E+539630486839797', '-3.88846513517873590419488295077e+753011587765392', 30, 5);
    T('7.18743257842849537E+329', '4.9380270120382705497855100060479010714970E+941', '3.54917362194837856438996787232e+1271', 31, 5);
    T('9.5727173581790086300836671E+56707172', '-4.134216351295932816078612565717290243308915238E+466556922481091', '-3.957568462851806226879253755126260638105326423411e+466556979188264', 49, 1);
    T('-0.0000383518897312525', '-1.68580613157309773482236534344882E+2414829777', '6.465385086636078794021544170258773513128824705e+2414829772', 64, 0);
    T('-877.010', '4.690E+32214668093', '-4.1131769e+32214668096', 55, 4);
    T('3.91252440505261485902296837068154458795083306401188511339E+97678', '7.367087E+98510', '2.882390768164585324391494298505918827381293890505212666434899493e+196189', 71, 1);
    T('-2.013493036569552587127377784983741535817256358E+78740600500290', '3.81953164421036076218291455568650860596E-925051213591', '-7.6906003685746152432e+77815549286699', 20, 1);
    T('-1.854609451482985316E-125796141194447', '-2.172106572E+735993304', '4.028409e-125795405201143', 7, 3);
    T('3.3929E+996', '-5.4402041679626485407985137145E+693977819', '-1.845806872148047023407527718192705e+693978816', 50, 2);
    T('-9.9463511E+4265659254268', '-8.1912152E+38149816', '8.147270231485672e+4265697404085', 77, 3);
    T('9.1445E+6472442773269', '6.011890257845080874866272135419084959366928657086796094E+501056672631071', '5.4975730462864342060214625542339822410930879104730206881583e+507529115404341', 70, 3);
    T('-1.65889825280958788519904698695755834672720105E-83', '8.4325599687002977574985185479980448566E-2094', '-1.4e-2176', 2, 5);
    T('-4.7513E+4378349014', '-2.19439E+76343356131008', '1.0426205207e+76347734480023', 12, 5);
    T('-2.713363731484510062468485776257102814829497775709189543E-106910', '4.4696943992339036557849219484170915084202001E+15454126', '-1.21279066737007202777429752145242883225084915521821e+15347217', 51, 3);
    T('4.06340755057227113E-826016406686876', '-7.17559793464121123839243882E+1000266930', '-2.916e-826015406419945', 4, 5);
    T('-3.88437933577976776843865444560458060118E+694247459642725', '9.78023677677982127523112408769E+428644', '-3.799014963475685901273238556e+694247460071370', 28, 0);
    T('-6.51523131141574052932480472280389581140E+1019604', '1.461513277265896892785788412807056768219E-8', '-9.522097066092606211881332745704312578718867828008784943592837e+1019596', 61, 1);
    T('8.94962403918510475217221009315665819104E+6997', '-6.1838433728070172749E+47597367130', '-5.534307330382917950632851804092736015722178e+47597374128', 43, 3);
    T('2.0617129175322231138709498309672540086342471806012946577158E+74', '8.7533782424803381343008638548259217429767226856498E+12012', '1.80469529945672214751188794775639229538737e+12087', 42, 4);
    T('3.4808964279363596149197695095868246636254E+69642144322', '4.587015356192112814100617500E+6361428854', '1.59669253682583537527388708033314339156e+76003573177', 40, 4);
    T('6199513525237299834114441395994268400436088553.1940534202', '2.62217738E+14660814486555', '1.625622413288130675729264055991179320927229353986237e+14660814486601', 52, 1);
    T('8.8366802959943218E+55', '5.0825330914092892465247673046412672606E+90768257114776', '4.491272002e+90768257114832', 10, 4);
    T('-9.419917372801833047066761324255161965341885800991E+71955180966317', '-850437465.23140942170275851225949163836441754286', '8.0110506532149084756285226380375431967157021081945311033e+71955180966326', 56, 0);
    T('-4.651781E+710771643071', '-4.1802821142291163052573768024916067002575927361241467E+993280', '1.944575692e+710772636352', 10, 0);
    T('-2.7870693912824055244329992764139278369126140028606641446E+116', '6.2995097841243929682E+54602950116545', '-1.75571708994171297423823156218547e+54602950116662', 33, 2);
    T('-2.667202552935743606759E+6103820', '9.34378515834374E+4873', '-2.49217676284175346460742652e+6108694', 27, 0);
    T('-5.3234369222403733611499003524745658644', '-5.63176658310466733473056218153031423377703642E-511644', '2.99803541659388941430245661368e-511643', 30, 1);
    T('7.915501330E+92953849278', '-8.4642727157514574801642444277E+262', '-6.6998961939013373633678525385904438841e+92953849541', 46, 0);
    T('8.1705179161993863820724337091032275304514257126139542E+9147274550634', '-6.949037269586019396181938E+284', '-5.677723351148983677969728601392042946733399721435e+9147274550919', 49, 1);
    T('-7.52153443515879865917823353402790164954154384827830477643E+811', '1.10741889207372528453653173843477763960922216178322E+912946697', '-8.329489330877929921581648974e+912947508', 28, 3);
    T('9.42637858098686874564221660900434072988703E-93171', '9.89278523467283557848E-323377', '9.32531388424231711602952036783457062057600924439395852550991144e-416547', 69, 1);
    T('-3.2761481753848592091773147958975E+924030022', '-6.254606930886206E+21160674460', '2.0491019084372337996709867261541e+22084704483', 32, 0);
    T('-6.1325087803472806662490511322852459319469430244306355612650E+791156', '-5.4064530881E+191821897729', '3.32e+191822688886', 3, 4);
    T('-4.7412680349972331920282110437546E+843102287408', '5.479239936104238459990147218E+76922917653385', '-2.59785e+77766019940794', 6, 5);
    T('5.2424705157836329130763918399142666E+237175874768884', '-5.0131230878009E+225266348240', '-2.62811499797904226893674357274155002177203e+237401141117125', 42, 1);
    T('7.4827117231E+553417', '3.3027392857291378701111902227598410334641430469597367274350E-248385831163', '2.4713445971668340471937168180339162936995e-248385277745', 41, 5);
    T('-6.53914074175562E-652', '-9.03407E+948', '5.90750552008721939734e+297', 39, 4);
    T('9.537050404194810E-18', '-5.899639653568583584367244633363626033210521518200900100457E+8514991558108', '-5.6265160742669988915909094283057984465960889649738329353188e+8514991558091', 59, 5);
    T('4.7367187565095817232375873217774434072957487085262E-745842', '8.19488678320182411442413667646E+3053043', '3.881687393346455e+2307202', 17, 3);
    T('3.82869391705334123257827293258663072E+908887877885893', '-1.690458751580092161668279010771450443543269335139185E-32179355', '-6.47224913920428e+908887845706538', 15, 5);
    T('7.0148049863366500621581509163591534437E+852601306473955', '8.983643704730619184191425261E+865283630663869', '6.31e+1717884937137825', 3, 0);
    T('-3.246673577E+221', '1.20809513256256260061241491561419646585128079E+79', '-3.9222905453932e+300', 14, 3);
    T('1.7454892E+54645540807', '-4.5611339140043089205E+62458187', '-7.9614099866482499741964086e+54707998994', 80, 1);
    T('1.589438540905310856524752897059611791028376183478E+3240451793', '-8.49E+417224309', '-1.3494333212286089171895152096036104105830914e+3657676103', 44, 5);
    T('1E+890697755', '-1.112651864790694372242578793675422665388E-9155483', '-1.112651864790694372242578793675422665388e+881542272', 55, 5);
    T('-2.60868588350706700974659908904380230824744767401734008779063E+311049362', '-0.936657982716326936696769351037792735259137805439', '2.4434464571862884358247464496579397007894154830408544529534419689538e+311049362', 68, 5);
    T('-7.6375376961E-54956265490822', '4.2824464E+631913141265', '-3.270734581152773904e-54324352349556', 72, 0);
    T('7704.45067051021428426676097516631', '-4.8243927588091175097094985984328842846347951388070073561272E+228475', '-3.7169296025411527898247616281873795937682677835753061424578960974e+228479', 65, 0);
    T('2.4E+6075', '7.5430879851066321439409587132999469831076E-4782931027707', '1.810341116425591714545830091191987275945824e-4782931021631', 70, 0);
    T('5.8808859434713069395849999536025567670497048E+490759963', '2.6448258929475502730527E+566645', '1.5553919416764196034421709759598339903350091201667481529432929842e+491326609', 65, 1);
    T('-7.864353979767681583810225804967167803267661421271948E+74081330', '-2.157734311563561026068569034834301852233775063E+67438344149', '1.6969186421e+67512425480', 11, 0);
    T('-11243.438318066395549191865757584', '-7.015994856211563220011E+6046751137541829', '7.888390540568582105415661374993e+6046751137541833', 31, 5);
    T('-4.469279963346104203907085E+867683051', '-0.00710580630790357533412', '3.1757837755331807212158456e+867683049', 27, 1);
    T('-1.791914352E+910', '2.4837052316463299660859344324412045007986E+6643987', '-4.450587051e+6644897', 10, 4);
    T('1.0189854139495056665408446044314778150919984015297514777E+1927785972308334', '62064701.2287746263475815643104292914902836482624002', '6.3243025273255305621597110599344900379155125689738e+1927785972308341', 50, 5);
    T('-5.7336919652E-435543576223221', '2.821722353015558912E+3211167990219', '-1.61788867835105481243216538624e-432332408233001', 46, 2);
    T('5.77323769271803314720E+32514861458', '-7.5006299116077706096840871341855326807E+837495', '-4.33029193248223105053322e+32515698954', 24, 4);
    T('-3311.564', '-5.3932242816731E-8163925535', '1.78600073751144977284e-8163925531', 49, 5);
    T('6.1568079184625077548454653625424488640440482887186E-7709', '7.852031728232131831154776412190666837010017E+64613733456117', '4.8343451120398438965673569033759022822654112767525161335173e+64613733448409', 60, 0);
    T('2.12407401873401358199395059045133E+6396089', '-4.1074564900028903E-9455', '-8.724541613495544882245546412944684553023281479099e+6386634', 75, 1);
    T('-9.57587392766466525849450190447E+706904091', '6.896307442463754E+33326432', '-6.603817063544845054004468637216749330194558038e+740230524', 73, 5);
    T('-1.83368603029E+897125759', '8.99353087413601538751355E+771450', '-1.64913119268850236894480775330854295e+897897210', 73, 4);
    T('6.9184476418406161040340126402053436621177216640258571179E-66703289', '6.88837483296E+60398334387266', '4.765686061920635986256877780263301272863955564728988676563523952598e+60398267683978', 67, 3);
    T('9.74105530E-820736741058', '-7.37344960287368243445254934679978329449381430031194419339120E+85523121156', '-7.1825180333355579508640908413155567e-735213619901', 35, 2);
    T('-6.104187808400E+95304459', '-2.172116311356824242E-4393429626', '1.32590059062111050001579712e-4298125166', 27, 1);
    T('-1.94011906408594829703548470174E-46', '9.151619533120833773615436942424210313E+8266164', '-1.78e+8266119', 3, 4);
    T('7.5909365582335991110924091058360025779E+776407660541432', '8.00721568E+73210', '6.07822662349733079051e+776407660614643', 21, 3);
    T('-9.00437916061981640996888137789109291361968892373279637E+72671', '-1.544801599E+882161', '1.3909979325327770221206367492807483580817264327e+954833', 47, 1);
    T('-1.2975735747290013915909309165718E+800539', '-8.9684025468482744345E+422382', '1.16371621523225958307070349688941546903863103236e+1222922', 48, 5);
    T('1.44772294545326762661141842574424351939483508833515E-936', '-5.801385588484887731479658419720293221713616753447699E+80400409', '-8.39879903187148003136233233756259103922388907094807991294e+80399473', 57, 6);
    T('9.2282638392234236112', '8.555052550506519940385797E-9523', '7.89482820945e-9522', 12, 5);
    T('1.272633308814257668701062468360811E+598005', '4.0322514622570650641643376E+9786365', '5.1315775203833375342525811943689322696087795783073956137936e+10384370', 69, 6);
    T('8.9385500987246094450440157550537E+3292490541990760', '6.6787767218116571348826502936717060472E+342543363058312', '5.969858032610921131451254867228217960400757723388486598e+3635033905049073', 55, 1);
    T('-6.8629402690376503972949659129296823451109911793E+733962481361265', '5.762552048272255106736295846161969843009355576E+740515868', '-3.954805050451305382151178792e+733963221877134', 28, 1);
    T('-6.86051635718237061191359313647E-77632440603', '5.804355782317232967494525327050274808166830727163323E+779312961625', '-3.9820877787493453e+701680521023', 17, 3);
    T('5.643691024265586756586223916655083863764683064E+65997314605', '-8.35747257033E-5914914499549', '-4.716699293071726375113227685299628343495913331716589989112e-5848917184943', 65, 1);
    T('-3.1056142428082064186664002340952458952593E+3140180667', '2.278368237E+409198', '-7.075732847189023187229050192e+3140589865', 28, 1);
    T('3.84498676508584859926624948708348583164701E+490024272823', '-7.6627594898098057196759632450170174252497825E+7824015912', '-2.946320882e+497848288736', 10, 4);
    T('6.7135915864243456E+340340337259080', '-2.25351888654421768219353E-7440105', '-1.5129205436551619272148843010519602803968e+340340329818976', 69, 3);
    T('5.5419008051568149828461171667599621175223937692962129057271E+4065457924000255', '7.792829044853947106536E-4642056787', '4.31870855581255029301e+4065453281943469', 21, 3);
    T('-9.187471697324798361105865159462860043654733064904057E-8817', '9.2960918E-32915289', '-8.540758030823313998132967204078838565637e-32924105', 40, 5);
    T('7.5974E+4639250', '2.498940398788816499834310E-248', '1.8985449785758154475841186794e+4639003', 54, 4);
    T('4.6858341455837631580807723478776766020007058058E+314683214217912', '-1.58525747970631367188630E-6642784129192523', '-7.43e-6328100914974611', 3, 4);
    T('3.12751222042E+7583084197658718', '8E+1873', '2.6e+7583084197660592', 2, 0);
    T('7.835646039168044847767063654731940154E+168753586567621', '5.8119771990391036787283924236130649690763616495435271663E+15419413013', '4.554059611938574016997662784714401209577428e+168769005980635', 43, 6);
    T('-6.2675570242089330331E-69081', '-3.67596273508866118215394577979E+56238211131', '2.3039e+56238142051', 5, 6);
    T('-4.6323332490915081024394E+29643480534847', '9.214802739172040558168775504291E+30229235781', '-4.26860371124861473217967561286811813347801222222674654e+29673709770629', 54, 6);
    T('5.18877E-167203', '9.91764704697E+44701615194634', '5.146038946790652e+44701615027432', 16, 3);
    T('5.7316940132964144606239869484862844079097470948029344E+6813', '3.557E+3321520', '2.04e+3328334', 3, 5);
    T('-5.78515123253E+4019', '-2E-36', '1.157030246506e+3984', 72, 6);
    T('4.66778940780697E-662072266402473', '-9.3116707944891121148558629780018439083970744407050044962E-3975616', '-4.3464918303501790487588443343885717e-662072270378088', 35, 6);
    T('-1.402607599866392700421466350482386366E+457647', '2.6E+5359129', '-3.6467797596526211e+5816776', 17, 3);
    T('-5.55822446547E+511', '-7.29175841103814789882274349E-222068369605', '4.05292299965288861527128470769841722903e-222068369093', 52, 6);
    T('-8.7728356E-8', '4.890191755561409976832804424259E+45', '-4.29008483240156354309540019009768588204e+38', 73, 6);
    T('1.6429709508250786823724E-25377723892003', '9.43310758379549648455201201581022201440E+6825', '1.549832173618374744086116156724238580800971209645626369896256e-25377723885177', 75, 0);
    T('-9.8843745348767178243679600268255146618337254924032876808839E+6101663664283028', '9.1550305764803304689604012456742E+743452856', '-9.049175109617989632833045373122191202136501242786454587e+6101664407735885', 55, 6);
    T('2.19613616212774107E-134092092228', '2.7116704648858474076500197199581937688606018235E-786115604177', '5.955197567709552380626079322723242168546397228973077364777841145e-920207696405', 66, 0);
    T('7.50308057713099315846607183414898084331627240650859608178513E+633593950807375', '2.934749416032730259153787797E-18433493648611', '2.20196613421817028990495e+615160457158765', 24, 5);
    T('2.34743415164822537987E+862917759912659', '-6.109899851894262899728956668727E+9515620053879', '-1.434258e+872433379966539', 7, 2);
    T('-3.4716902294331268134058406434451133001210E+57480141861693', '-9.093559220E+97834028044', '3.157002069484552570748e+57577975889738', 22, 6);
    T('4.748E+1626755492046', '3.81299340696963451697121162265418064112971203485E-2599549481', '1.81040926962918246866e+1624155942566', 21, 4);
    T('6.08737861456708889492583047235893193781E+239922231186', '-7.310470840422645818561537E+188886863819', '-4.450160385640510770749080995792620101101413170481905014214201397e+428809095006', 75, 4);
    T('-6.972838358550307061790E-905836105', '2.571254502058256553472779764526681382441550699148257283E+6547098163241578', '-1.79289420215469807568018655223294691108933586e+6547097257405474', 45, 0);
    T('-5.5668467861082450213451990796E+79488559', '-3.55E+532', '1.976230609068426982577546e+79489092', 25, 0);
    T('-5.346E+205813239326', '4.60735967096819973758315050967679215284E+2887', '-2.46309448009e+205813242214', 12, 2);
    T('7.360104079944E+5850241050950', '5.11645943564808589950437410003086019087763633695E+160159898749', '3.76576739671814527447363209811112172671863418136528021211308e+6010400949700', 76, 3);
    T('3.4E+84192789', '3.5995179011123799E+10778802665107', '1.223836086378209166e+10778886857897', 35, 5);
    T('7.57046510952429316824124289456579734626914650E-788769646', '-5.2892811946038161572895650722633950766432631595E+2999841', '-4.0042318738211163292248769172908691507495964e-785769804', 44, 0);
    T('9.322750483577142203945809359854084E-6525120675', '-2.61047313561790147857136672672336254535538407447373806E+47669854', '-2.433678968744692973162946665242504386641185308393e-6477450820', 49, 0);
    T('-4.40504751025045879301291316626280099591149E-6444129', '-2.7945682884361927442917457370196829633914701297572581E+200', '1.231020608120073684263631268960366530269259622e-6443928', 46, 1);
    T('-2.7107274491747991544311609041469025627845010043242E+137', '2.442004098429E+9921094925', '-6.61960754060884832914381862407508928020547494287936305e+9921095062', 54, 6);
    T('-2.4258689E+725009', '-5.49862102670848392328529122E+38327281178657', '1.3338933741578180515647773798041058e+38327281903667', 50, 2);
    T('-4.49E+7125229', '-7.0547877756264754586461362735674950675233E+99844611518', '3.1675997112563e+99851736748', 14, 6);
    T('2.10469E+80', '-8.6723645781E+4135656', '-1.8252639003881289e+4135737', 21, 2);
    T('-6.764402777707856059780863661975254223560492127153E+8985', '-6.8252E-33378286121875', '4.61684018384116591792163506657135051266450708662446556e-33378286112889', 77, 2);
    T('-7.6994281768E+403917520', '-9.4822315201660388902055923E-78315201674728', '7.30077605453074972456935389e-78314797757207', 28, 2);
    T('9.055255196469720616287371507780772537335168E-6381135568224613', '-7.7797041340353681309033871721333285666385564E+7965226', '-7.04472062867207351355082098557808306343351551650420891666582354e-6381135560259386', 64, 5);
    T('5.60700552991969650203455531692917703282811E+696959378341098', '-5.7066353E+6270134', '-3.19971356843349462237969151914e+696959384611233', 30, 0);
    T('6.64850825909995294847194201149166685437728E+349374', '-7.0764198724230066555710794584349526468901160246498038071E+10047542829', '-4.7047635966663395122050413387948898205e+10047892204', 38, 1);
    T('-9.2502719347706068380822983077522425557704207922641633E+816507', '6.19911979434656597421327E-50', '-5.7344e+816458', 5, 0);
    T('-6.64242959147334748543841216214E+80429928', '2.260830393346102757225638426499849590645810285605501472066E-1472013', '-1.5017406706064480840815383763712292583795935848292284686343577735216797673457407e+78957916', 80, 1);
    T('-3.38045052901102089702239129215836540784E+4773362856', '-8.06698253566433766191334928796577770E+10025574595289', '2.8e+10030347958146', 2, 2);
    T('-7.3352912492444181239801508275521171216965066799245E-784428967', '4.3163720391107833080512930807518427458E+8835395726883271', '-3.166184604697261409728375724974412378682052204681403038367911e+8835394942454305', 61, 6);
    T('-311.9650', '-2.7236597223082182051E-2130731', '8.496865052698832923540215e-2130729', 29, 0);
    T('-4.9354657596539486930E-754384', '-9.36686230752343642423310453999E+1610938549197', '4.62298281941750959253318006078839235e+1610937794814', 36, 2);
    T('4.786843068339273152505E+38255', '-5.27352421996108721204062954655612165605E-8', '-2.524353285804e+38248', 15, 1);
    T('-1.48148635644397850E-934776166178', '-6.49749340522565266975579852564898978839824438464E-972498265169', '9.625947830926530907250166801473069212415176163e-1907274431347', 47, 5);
    T('4.3194508774557101186688762960875355E+55509', '3.938385110E-52', '1.701166101914e+55458', 13, 1);
    T('-4.59562191919554135185820E+20309', '342247940170334213825.1471', '-1.57284213564631213150207035070635786401247334122e+20330', 63, 6);
    T('7.779734506622802903676509003933631145008E-5053475517098095', '-7.99246321545948226766266254803633040E+848', '-6.21792418702235761407461971568361325331996e-5053475517097246', 42, 4);
    T('-5.9890942464355699550635659762E+301818', '9.91379884864686E+6533182116201', '-5.93746756447504870309819049227559737049e+6533182418020', 39, 1);
    T('-7.2638365619382186603305566662E+746629', '5.4838897543814271285225875526816E-383994464', '-3.983407889951e-383247834', 13, 6);
    T('-9.8730172989137544752377026E+4730', '-3.14581371451132852697975582957149717514E+1971', '3.10586732225304815237308559799536421819109065560083355159234333e+6702', 63, 3);
    T('2.256508576946394997143820476566111045545393E-84546853', '-2.898626844557140031522642564652906080464E+20994742', '-6.5407763361102513474394091859548988114467487698424192e-63552111', 53, 6);
    T('-6.35754688435226591755900563597373848194799756773E+882', '8.532866535E+878679359', '-5.424809905418296499936e+878680242', 23, 6);
    T('3.53624E+21', '-1.676914187946552379630836131028662386E+2253127964380', '-5.92997102798411638694574795998879707586864e+2253127964401', 72, 2);
    T('-4.0099524304852032592121571503E+713829058605', '-9.176310361281956562823538083746349055E+487226', '3.679656803610913532908498679881258e+713829545832', 34, 2);
    T('-7.551295245329750358331108837587582203381263179892625394560E+4582582', '554647122.06809641642730299998060974254552397951565275153515', '-4.18830417570864612281830608948249890363373000344162679e+4582591', 55, 6);
    T('4.6732E+8979531317111243', '-7.601203248493074447555473020438699068E+8606655819', '-3.55219430208578355083162365191141284845776e+8979539923767063', 73, 3);
    T('7.8552409417273835171884016964073265201E+7430549223', '-8.621148058633586262391860939855971E+5726908010', '-6.772119519487209632308657374902213601750308973033827e+13157457234', 52, 5);
    T('-9.9946164793576088135185648145352016228782843581E+8647', '-7.37384566501997823430108040399106793303039959763E+4352766681210', '7.369875939984834052410370820058e+4352766689858', 32, 5);
    T('-7509.0510', '-3.55999080252858495497714831E+6112808369', '2.67321524957181e+6112808373', 15, 6);
    T('-4658678148.454501408091596662', '6.0811E+74994294958', '-2.83298876885666685127458084612882e+74994294968', 70, 5);
    T('-966683390735966083.69131159869538624175591119301', '-46289475.780506240672552233820503980888986083', '4.474726740289015285580167055110412397621e+25', 40, 4);
    T('-4.0E-70', '-1.09965825408E+5320', '4.39863301632e+5250', 23, 3);
    T('-958217.38729632411074280877000260471939187328843590566242060', '-7.655550442266327581444755126577489869835523443625E+52841938', '7.3356815431036589502e+52841944', 20, 1);
    T('-1.047E+20676800383473', '7.8452917330178009347552978477654E+3842441401979', '-8.21402044446963e+24519241785452', 15, 1);
    T('5927341826632567880802072567739046837.1966758509547257910', '-1.81355172280483498377758275598690E+5557', '-1.07495409813426510043924999572131702938399756903221662563086123764258910111426e+5594', 78, 5);
    T('2.02214972189274137E+5826', '-4.092360392554182586325E+3089', '-8.27536543e+8915', 9, 3);
    T('-4.3873425E+83302417', '-6.980227717096806292181751724145096974E+755933513', '3.0624649722896794859956e+839235931', 23, 6);
    T('-7.1551792892188710995165617383274455056251561047011907E+689887890', '-4.2103591346453587165811549685614E+2041348624049', '3.0125874480387958982016529989873e+2042038511940', 32, 2);
    T('-4.252625142890642053339136523640212E+25664645445', '8.6160314145823436347595791827800489818776255412655E+332743851979', '-3.6640751825588499881180375171960317742567714824794978997288589794418113092881e+358408497425', 77, 1);
    T('-7.500E+437', '5.063929950E+63026286', '-3.7979474625e+63026724', 18, 4);
    T('3.22896682874284446364E+7313462870509450', '-1.8436244109649452336261294764366E+91605', '-5.9530020676663738169323479103617656883175e+7313462870601055', 41, 5);
    T('5.7272870757787600861138424169514707459108E+6151752557', '-9.76881660072E+97454023461109', '-5.59488170629566561512e+97460175213667', 22, 1);
    T('-1290.432', '-8.05645575861354612097465668745E+8220', '1.03963083174991955479815681784994784e+8224', 77, 1);
    T('-9.0789618998764980526416238882516E+292991038167', '-4.619278886395E+612526528', '4.193825701448414341263878648254728077157698e+293603564696', 43, 6);
    T('8.6395170232538780673953766340E+48316', '6.81E-6320845', '5.883511092835890963896251487754e-6272528', 60, 2);
    T('9.1925199845035473E-62801304924', '5.9078449462401659317418189096775135E+3489', '5.43079e-62801301434', 6, 3);
    T('-5.135371573253687742160734353469029441E-9644367260331', '-2.705943074789862059133537960672176404085322196E+3381580601', '1.38960231451185351561245310950756904560759210648345132447e-9640985679729', 57, 3);
    T('2.126776760920129982126398864434E+908797308184', '-7.428812984756457972051334411190485624461928601988E+543573025006', '-1.579942681720174263375982702628133e+1452370333191', 35, 5);
    T('-4.21E+5687', '-0.0025040', '1.054184e+5685', 54, 6);
    T('2.55908055293855332808646526786656076025538218178042102784180E+78285', '-3.26838155801311253134683036874700076000559782075706262571E+50050', '-8.36405168469436642868679728434125e+128335', 33, 0);
    T('-9.141749474447472716485472847724806333803867E-5039677', '2.44309182589353520549781339131743296570912724280315989324892E+31935186642', '-2.23341334153891419808161515494417e+31930146966', 33, 2);
    T('6.7623988941232000494803268251328179608929486720328E-3717', '-5.5744785603580759275208006554E-5954707685907031', '-3.76968476518789409307e-5954707685910747', 21, 0);
    T('7.4828729381E+539361929', '-2.154782526955918040093909813905813121996006650044270237E+937', '-1.61239638584492e+539362867', 15, 5);
    T('7.8450290893889E+3246677', '4.18108566020059111561681394969E+749', '3.2800738629500431090260877497682411477444441e+3247427', 51, 0);
    T('3.69073400046808585683412269E+59157', '1.7967227E+50606584258', '6.6e+50606643415', 2, 1);
    T('8.30715016132427249E+463380686773230', '-8.127164251098545512768252452621E-78192484701729', '-6.751357381962e+385188202071502', 13, 5);
    T('-8.50710772133848231E+808139200', '8.9499819500301827646334290658007023266444061631572344268E+76476', '-7.6138460553e+808215677', 12, 3);
    T('-4.331189008E-2174070155', '-6.1616904348854623E-924795227128515', '2.66874458822746540527583984e-924797401198669', 72, 5);
    T('3.890660059446861807', '-5.49127939E+673099345318', '-2.136470139793672704095725773e+673099345319', 43, 5);
    T('-8257955.8825844778838146', '4.886891796671180976607270017901051736155681655E+46897', '-4.0355736859874607142e+46904', 20, 3);
    T('5.033419555079804070298639911091587282930770105834122E+1545565264994', '2.4777219594250580312206278620962632366783644145112483E+530', '1.24714e+1545565265525', 6, 6);
    T('-4.4565353687387809570069142750865E-55290', '1.970681666157E+658291044660769', '-9e+658291044605479', 1, 3);
    T('-0.04809445986948473416905511127909523796113864194', '-572579716252035267261.93706750952', '2.753791218536446621e+19', 19, 2);
    T('-3.5421007E+498102219816540', '-7.98464443637959247680610668842343186E+14633844', '2.82824146473512599778096442653393198877083e+498102234450385', 43, 1);
    T('7.178423452326158E+65701971752', '-5.7537342535762130762624152886068170361079906791346545916654E+984', '-4.130274090432382927266909691842612066477215332656052e+65701972737', 52, 4);
    T('-9.9224566652901244185077E-818897769', '7.973E+677379', '-7.91117469923581619887618921e-818220389', 44, 1);
    T('7.30649688557268071E-8', '-8.18287E+7502539683366426', '-5.97881141700461218014377e+7502539683366419', 31, 4);
    T('7.8065236917882874337316099055870738049813343660E+3668093', '1.03529274617325033901398829970454478982588489554839064901599E+25017', '8.082037350938037e+3693110', 16, 5);
    T('3.4733855424329965254006697047737E+56', '2.3925864715685171E+666353', '8.31037525936686298642958431891198381522850008027e+666409', 63, 3);
    T('8.970908544614229822411513031331095361905E-839005420', '8.99462931760202057984536247315962E+334188', '8.068999700091362557980667415302930870285689998345e-838671231', 49, 1);
    T('1.1586773714537368205253947205716319258679850E-80955', '-3.6E-647567297400', '-4.171238537e-647567378355', 10, 2);
    T('3.384E+2717234957899240', '5.8741103035501224987041117E+8462781060', '1.98779892672136145356147139928e+2717243420680301', 60, 2);
    T('3.43295534993167256166905528906181325924630903147106226099516E+92592919245199', '0.00022635563492035459', '7.7706878788701181295167413664581310173812126984875870841387951681411e+92592919245195', 68, 3);
    T('3.7150356433721823881228593426428795419520498847765054334101E+3239729937398800', '-6.69086491088105988823622578355999980958409587321E-16467598130601', '-2.485680162891137809955433050755893016921221614e+3223262339268200', 47, 6);
    T('5.95939316048217613165180E+5849576446632712', '-5.4426356976639824432248489387384E+861110', '-3.243480595165487397720386784781434857017829427140008912e+5849576447493823', 68, 0);
    T('8823274.6197357', '-4.2419287438353184526797439269185695511294E-135659811', '-3.7427702224209504996151180911436493207300249e-135659804', 44, 6);
    T('6191240051.0667608524348331092011', '-35007535.2995', '-2.16740054635397813172737131e+17', 27, 6);
    T('4.96462713157324858816352E+67351948681', '2E+7180366904590992', '9.92925426314649717632704e+7180434256539673', 37, 4);
    T('63845420.59993964597057395864377271540637', '-3.6316401287016864E-46', '-2.31863591484578116022107385800903997555794877907080302368e-38', 78, 3);
    T('4.603919313035515915163460888554448784399255509146E-6119263', '-1.915211802140803243720856E-7309', '-8.817480604429599298984386728275545060239207364330566856492282689778948976e-6126572', 73, 6);
    T('3686.61443', '1.16683689460605413639024969459808E+464329663632', '4.3016777331110683445774826354083747782944e+464329663635', 76, 3);
    T('-8.201786935523514579927952452551744795032E+15716205464714', '-5.6065175211610048060492501E-6552418997219101', '4.59834621588420089146762571430189764764807e-6536702791754386', 42, 5);
    T('-2.512588982676800595416E-9316113', '-7E-331943297156', '1.7588122878737604167912e-331952613268', 56, 3);
    T('7.127149868130687E-903635497', '-5.3909253293789073E+44717762', '-3.8421932750385259543e-858917734', 20, 1);
    T('2.9870896523253615458495207406139E+64', '-7.3679027662078368725E+9946310513', '-2.20085861122e+9946310578', 12, 1);
    T('1.27228366025895653973644792980430774163752925613404387756E+85247', '3.32717600629837015187174610970525338826427012391E+19653461', '4.23311166761e+19738708', 12, 3);
    T('-8.99652429250907005191984694E-16', '2.347148502009704173318E-208719997864571', '-2.111617851645657742477083631416111733334979194692e-208719997864586', 70, 1);
    T('-6.742420137185303624200905733085568559627657646746024E+780', '-9.0342702803070E-901615', '6.09128458627166343670104e-900834', 24, 3);
    T('-5.99951712644976558857786E+75520128089967', '6.7296560953490097E+5825777717670261', '-4.037468699916344037694135268419658e+5901297845760229', 34, 2);
    T('6.1793370250832364553839186381945521992185E+9301446', '1.1128388E+506672', '6.8766059997891987571256935566260596359156764778e+9808118', 55, 4);
    T('9.235621784032820096640266E+80', '1.7867816978540047061342188871335E-55114700301', '1.650203997201159426361033738083236e-55114700220', 34, 6);
    T('-9.66463E+72229412112', '-2.98669958038417215777201582E+6397037894825', '2.88653463655682817612e+6469267306938', 21, 2);
    T('-6.47849198072E+313028835079', '1.290304283208291498E+1396759', '-8.359226e+313030231838', 8, 4);
    T('-8.562689950708497900492935162240239984426360110570E+74407283274085', '-9.88601964241221167266031129584592765584266029798365460475E+444771147', '8.46509210445898628076421068975893822901370259836e+74407728045233', 48, 4);
    T('-4.89105754655314646646580E+68350051', '8.9780551791619908896E+9503', '-4.4e+68359555', 2, 3);
    T('-4.397133669684888415503438983247269166351873353222849591E+99567266696', '9.684E+973961187529', '-4.2581842457228459415735303113766554607e+1073528454226', 38, 0);
    T('9.4120231217685470478221777107270501118607470290628655E-878167', '1.101706973890E+366581624', '1.03e+365703458', 3, 1);
    T('-1.555798268781672915299020385415859278946855615020E+4067202235', '2.33089193069786570246970930301480E+2569016309591', '-3.626397e+2573083511826', 7, 1);
    T('4.54500539804336486734573099671889955980424997533727790743742E+206', '7.266684304949514727666434996373768005129953735E+487898853852277', '3.30271193918725413556518881574064617874747e+487898853852484', 42, 6);
    T('-7.033706584467570971803E-31668864', '-6.5412798686657897251355347641102737479891809354035114965683E-11841536778', '4.60094432830797330702728246735e-11873205641', 30, 0);
    T('3.044936153564365608762407620901065E+2322530617', '-5.02497545745106025350313124697118306109309928598030949738765E+2739', '-1.5300729441166e+2322533357', 14, 4);
    T('-6.90E+1196584972482561', '9.95892389246718791770837846238487499674438597637576035029E+834255381048', '-6.872e+1197419227863610', 4, 3);
    T('492581.2806603078282342414', '9.71134392541671725014408758633175364080526479256232900E-48696135469', '4.78362622771446753310176e-48696135463', 24, 1);
    T('2.0336237144071229199610E-23015354', '9.7644759996346373126623231E-211839610133', '1.9857269951616195755679690642376280499261621399e-211862625486', 47, 3);
    T('2.39936716235397242692177213117E-4571638', '-4.9059335429823564E-1654829075', '-1.177113584372275e-1659400712', 16, 4);
    T('-5.6641330E+85623', '-6.12009092340473212327317114E+8287', '3.466500896225721557559163666872e+93911', 31, 4);
    T('-796488.35367204290246636723980857107267009157239929', '-8.225146083709593190E+27', '6.56e+33', 3, 2);
    T('5.042313696946472703168583928365419972744841026330567E+190927585210', '-5.4555186564E+511135', '-2.750843644511273754430954901556675e+190928096346', 34, 0);
    T('1.579E+217476', '6.553E+1425266', '1.0347187e+1642743', 46, 4);
    T('-4139967518147330711597535628794050255089352515.3', '3.741184773393902002611934307571751591898046E-92168178811', '-1.548838344123813632497054785361066483159355043031300346380118591110608720027e-92168178765', 76, 3);
    T('3.012340099747992992254928E+30578771342', '5.9471022669303759035587475517766522644577180422608747E-3794', '1.7914694635976563795320689466926526205573653987652831662021366556467583e+30578767549', 71, 0);
    T('-4.26950726862E+94166', '4.146347432752497323609379875681978639392356417461E+91515053572524', '-1.770286050236066397660734371283495978042924358441944258537382e+91515053666691', 80, 3);
    T('-8.9803177105749250E+2400656240548518', '7.8884233643800023E+60', '-7.08405480476547696272969058223275e+2400656240548579', 35, 6);
    T('-7.842110659299E+320910', '-7.08248394879214055E+429377573', '5.554162286913691828326997247445e+429698484', 64, 1);
    T('9.4929171019714759534E+669', '5.278E-273189', '5.01e-272519', 4, 5);
    T('-6.0137878898848740673276203369156765065573606673432179E+1095856', '-2.624705E-277141', '1.57844191435202783888851417364042607051436373303790807382195e+818716', 60, 3);
    T('4.352587949397182683603753380474777232970393417694E+79935352065', '5.16775201973601E+254', '2.24930951665759089027409551548858037498212247427e+79935352320', 48, 1);
    T('9.7135152035052542776284741580893145142308591363773792086E+379077695662', '-3.92595824E+35747742', '-3.813485505256672991455075577957780697309623868874006e+379113443405', 52, 0);
    T('8.81556357398555777991914366590325708293089378453E+606', '9.5757484920405421357E+6787970819088', '8.4416e+6787970819695', 5, 4);
    T('-4.378455989592E+653893173', '9.39368448070552496E-193391237', '-4.112983407888252191907865621632e+460501937', 33, 0);
    T('-6.577E+25576949', '-533238393439287274.8288860430045618337', '3.5071089136501924065495835048e+25576967', 29, 5);
    T('9.8271463657824825340835766135218663433E+2301824969107465', '6.2904220988621826848865957557964165558491960894155460858786E+2657476182721003', '6.18168986680713146321222613194861815476052583838006136857784058e+4959301151828469', 63, 2);
    T('-2.42233633014448674068402227712155293936086910972045128E+5928', '-7E+870445786', '1.695635431101140718478815593985087058e+870451715', 37, 4);
    T('1.074459250064373726301072035832748249E-32', '-1.179211536E+983782', '-1.267014742637818240669530754e+983750', 28, 4);
    T('-1.718323959510972296020576182180E+244764', '-3.8670103612306705062835345224881E+98689', '6.644776555379841019595050821094736469679750930746634452482058e+343453', 69, 0);
    T('-3.1974534870203526324945034593829019978419468E+80989', '5.106336129258261511153400883538064119547871331312640E-3443426716', '-1.6327272262394838375027529757164040807146476770395978376e-3443345726', 56, 2);
    T('-2.071693913629422097011217928E+9030222981399', '-2.68973435807E+233560519296', '5.57230629889355966796954151915795547896e+9263783500695', 55, 0);
    T('-8.56820858962922219995920948E-3078267452035', '9.882404647264394404563488795E+72164', '-8.46745043848825278841032432936190574890180139190377766e-3078267379870', 64, 2);
    T('5.888928118703328241658935307E+29094497', '5.952065584151232522226610509265719868284462695E+2549853', '3.50512863828745e+31644351', 15, 1);
    T('-9.38817951107330368833884757E+6596295379', '-80785.64925794390105695680', '7.5844e+6596295384', 5, 0);
    T('1.243811678565119290770573E-989065160', '6.311147539595689321530322537617E+889520936', '7.849879014896636998104074732867285584814702560709144541e-99544224', 80, 1);
    T('-2.7924208966E+646', '-8.074011831526524141140518920951439073779353489E+48', '2.254603935775030468888515279183248202346321582032e+695', 49, 3);
    T('-3.364447384845731E+10145469', '-4.6999271934378E+1909707629', '1.58126577549271423019485440318e+1919853099', 46, 3);
    T('8.6019593985879918E+45747', '1E+11', '8.6019593985879918e+45758', 32, 0);
    T('2.8408772305398655382766E-14', '4.1179512082039404163926082585088726108068413076997029707329E-3048', '1.1698593823860703470820275152712759348889138433377764e-3061', 54, 4);
    T('-3.77266844E+685', '9.60401916955477914725836931474193831010010924639548E+36085944132879', '-3.623278001813432414003176243959e+36085944133565', 31, 5);
    T('-3.4016982480676417849040975247E+3894047023425', '-5.354297290254386919235166087439103374470629040191E+78813271061', '1.8213703711891669683096361451300851952314880153932336812095039828832619915217e+3972860294487', 77, 1);
    T('4.186989357816729837723759190859974163274E-89874128694087', '-8.377926003218566008119093699483782378749871296E-591991311', '-3.507828701605218576586698274458340791928415266105849296669010977069533616591279e-89874720685397', 80, 3);
    T('8.9412570092450558020468081E-827934101395', '4.46031810188E+95122', '3.988085049189715e-827934006272', 16, 3);
    T('4.9887673849224399406341263341420362855874933459376914E-8310184084', '-9.18783899312052170600880521947987474476036430E+64743285660153', '-4.583599150679828872337076941e+64734975476070', 29, 4);
    T('-2.0529269E-35695', '8.5365392685676407183386643362147299843269965E+92', '-1.752489109734883410021276732589e-35602', 31, 4);
    T('-584732751', '-3.189687152245240E+97029642458', '1.865114543361715e+97029642467', 17, 5);
    T('6.946891009461067397441391E+453240', '5.97493657345271911264745939978570169718414876866957918E-832538987832849', '4.1507233164218811e-832538987379608', 17, 6);
    T('2.13585269632809770648441472902007423842E-6429362063469750', '7.466628753829084702867397E-540642714642242', '1.594761915634675465513359516452240724234683017494614798502279274e-6970004778111991', 66, 5);
    T('-8.685E+6944645', '9.94690398264501053E+32828682200659', '-8.6e+32828689145305', 2, 4);
    T('2.50515718821385159779024557696826179935490E+112705665', '3.56747076231556616E+6454246048', '8.9370750239575894126047273121586268533409e+6566951713', 41, 1);
    T('-8E-62014', '6.567994823377523688113274418916112912217072660447920E+3931053359452', '-5.254395858702018950490619535132890329773658128358336e+3931053297439', 74, 5);
    T('-3.9605040167306445421314948065390985408066186E-444204639134782', '-5.98398867666352415379900901462389076627E+84108998', '2.36996111899965815586148469860665630105741456762235586148499844875989405422e-444204555025783', 75, 4);
    T('9.78830261928571208087257328075535551025207E+4921631285', '-2.8879903763201340108093584280927839449834300877985399811E-962817051565982', '-2.8268523765006297060394608882972957651798220195372499476831913e-962812129934696', 62, 5);
    T('438397.6951076994115880115367822298953419', '6.817E+4', '2.9885570875491868887954746462444611965457323e+10', 67, 4);
    T('-2.82842758714E+8521478804408', '-3662661.909071515869875859174185', '1.035957398598473370953519457537451252598e+8521478804415', 41, 1);
    T('3.754383143461616333955988822547250109329208807504E+999181992266', '5.545798228613818195154304647954105E-4446790190729492', '2.0821051386547010335906e-4445791008737225', 23, 5);
    T('3.85222631209005160153799190922012801566070011952905E+16640', '-6.5217E+9760167556824', '-2.5123064339557689529750321834360908879734387969532605385e+9760167573465', 56, 2);
    T('-1.74994424284836159238991896416E+523430473300', '8.88025E-89209806815541', '-1.553994236255416303082057788148184e-88686376342240', 54, 5);
    T('5.30158398811367203641492548403635079639919644442219848551109E+209985', '3.819934837497018178452866584768443004E-48101883849', '2.02517053701e-48101673863', 12, 6);
    T('5.794255341044431206280551022822841483050463985006E+16697257423936', '-1.372289151007419632917938589934821465273437E+16930', '-7.95139374268206920100799798360228977541380174920710310194e+16697257440866', 58, 5);
    T('537511524816566826387994455384540601664347.6173700', '6.16850598494058E+6621', '3.3156e+6663', 5, 6);
    T('8.7342373436968695197483951507934272343580689E+7909072548083', '4.07282659753022872879789739318387339366077235500111E+98443360900', '3.55730341625503840494373956692404295672168619982723253460373661079316757564e+8007515908984', 75, 5);
    T('-3.003727E+810563034792', '-3.31328935426842339038185574636668713E-168643633761487', '9.952216692228628585121520415467e-167833070726695', 31, 0);
    T('4.84643E+44595157', '9.49166148641373E-9311999521879', '4.60006729776e-9311954926721', 13, 5);
    T('-3.72183487913411E+6352542123236', '6.7749681384E+92832503', '-2.5215312722519410230640824e+6352634955740', 66, 0);
    T('-1.284628157492601952953506815439832239667E+964237562501862', '4.827163E-535345905886', '-6.201e+963702216595976', 4, 2);
    T('-7.700675294737629243570704975010148914390086600871123105E-386', '4.8562405477598448423198003401432782866217102639275268E+2364036', '-3.739633161143736926436689246364450233708236634892470138542236573071e+2363651', 67, 2);
    T('-1.599708010484193730953201403933545260604979164582402595E+7450', '1.4734005064144668508265501485133045993389941039479E+2253602054878595', '-2.357010592762690289535864454977150220014201524703237695199573692122499e+2253602054886045', 70, 5);
    T('-5.33561065881488990E+8030', '5.520194562735665113618513019678469422E+7217725716', '-2.9453608947664415212149928884778283893904785655e+7217733747', 47, 4);
    T('5.82160574163004230783407042447231540E+489944686949', '1.551210492861272491306593313851988289482468096820807E-413150741365', '9.03053591171795169081457817267810447050958362e+76793945584', 45, 2);
    T('8.73044123813405540150911223825839558333208970709113E-41461006491498', '3.385192779495507522969016503193320211320048E+756353', '2.9554226641141223091508951787376766885873e-41461005735144', 41, 5);
    T('-3.86909568829831023E+101', '2.723356551427960811463582999143628655205947442E+9102', '-1.053692709082887853749521176911540940362919566696590697479093166e+9204', 74, 1);
    T('-5.42907999742E-5089290159647', '-1.1671733128941102469231966377790098E+94130631459313', '6.336677286555848912098983965171419199514154716e+89041341299666', 56, 0);
    T('-4.7138E+98', '2.595976272006412838067225787488662688808477E+83260542244857', '-1.2236912950983828836081288917064058182505398882e+83260542244956', 47, 1);
    T('737192586.9937118039', '-5.5767E+43542579', '-4.11110189988783261680913e+43542588', 50, 6);
    T('1.80477667477902633316688844082225506850562E+851333', '-3.5512529785986152353416418897408857599668762124259E+1420628264509', '-6.4092185420143215715112854816954738512358046826888650699607458292221492023462928e+1420629115842', 80, 5);
    T('-6.398944516E+89303842108', '1.4979121225E+37982', '-9.58505656172129521e+89303880090', 68, 3);
    T('8.48363211543357844551112128701E+75', '1.2843060868145419842936009681529637E+893107301', '1.089558036414667386365053905936223e+893107377', 34, 6);
    T('-3.709384617641969420611580588815834110623898E+8549235736737', '4.2838586E+3649246541643', '-1.58904791949932624234239367649917748e+12198482278381', 36, 5);
    T('9.24176883308357222002E+6539338773', '8.06375692220225990298116962651058025117E+7454388118696', '7.45233774e+7460927457470', 9, 5);
    T('-8.503147E+97270532287', '6.38413709313707258138785796611294953967816210511013E+12837611474', '-5.428525617109721930921042031e+110108143762', 28, 3);
    T('227401.4557285396201984693247635', '-8E+6127', '-1.81921164582831696158e+6133', 21, 1);
    T('-1.32099323843021650312E+6652208871606', '2.4332501755743644181346172594857902067725E+171936969397889', '-3.214e+178589178269495', 4, 4);
    T('1.634019349625461531856795952022852485590089797E+52', '5.739137346E+95766', '9.377861473522117389965706372155976945499011201456258762e+95818', 73, 1);
    T('-2.028443950049203743760416974978752437E-6806675664680', '8.56573139572179976544E+65981173347350', '-1.737510603e+59174497682671', 10, 5);
    T('-6.08E-98294408066', '6660.1559597096658231373917545515012274579496307190418', '-4.04937483e-98294408062', 9, 3);
    T('4.144239729228469750613984475841137954608598689714437536E+1932694278584829', '-8.66445022729109E+480', '-3.590755886386238000835108644078245e+1932694278585310', 34, 4);
    T('-5.244565567738E-4774097871', '-6.41E-81259', '3.361766528920058e-4774179129', 25, 1);
    T('-82221941295324774910801660130.8925', '4.597507263312710144991915366211394574369E+3848758248', '-3.78015972308926915780228573e+3848758277', 27, 1);
    T('2.5426291E+291379273848380', '3.00066212887193610944937857147259734900000E+24137766736357', '7.629570848137734925226774932742655e+315517040584737', 34, 3);
    T('1.3337E+67339867898348', '-6.66001309809E+873409570958971', '-8.882459468922633e+940749438857319', 54, 2);
    T('-9.87320060703324411602454091771886300348E+1616', '-7.06198578E+1591600', '6.9725e+1593217', 5, 0);
    T('-5.96854355812114867936293707516656581159526910E+999669294', '4.6872981857718E+18343', '-2.79763e+999687638', 6, 1);
    T('-4.09731313176322936442E+6679492610036', '-5.4671041324458161853104206589617534740852104368429535877E+891428967', '2.2401e+6680384039004', 5, 2);
    T('5.626199822637613349431302178E+7484270030', '8.91788481023438644821820651162699167425508104944193730833162E+3320746312238', '5.0173801937643371216940565452890855e+3328230582269', 35, 2);
    T('2.4995594201244099E+3523105595662', '-3.19431525781814012892547846548607001744067E+745852196271', '-7.98438079352646524793e+4268957791933', 22, 0);
    T('8.2349521195370506789130328996875203919988139814191431349E-38483', '9.4396161026390769614250678739486770910583075574206963811832E+217223672281236', '7.7734786632043740552292157881849562408136297100657582805e+217223672242754', 57, 5);
    T('59.63877', '-4.111626371911500956653207486323270195E+719', '-2.4521233952036e+721', 14, 2);
    T('758770.14112', '9.5112390449465103643425372251775443821404084237E+9845', '7.21684419236011769200372958636681e+9851', 33, 1);
    T('838674.882763886593754262271236575466404324631784477749343', '-5.0633291646980919E+855858098857588', '-4.24648699359814005853549525087547198e+855858098857594', 36, 5);
    T('7.1872044E-65', '-7.1512983624903423430962511498449734644621049E+74674', '-5.139784305660338344580768588767085260166528397054156e+74610', 67, 1);
    T('5.89199547202451083656467749778755860000899438535753048182430E+455934783', '-7.968215062656424736084612369485564997440952290596E+70953', '-4.69486870692891584538324e+456005737', 24, 0);
    T('6.543625293053929721099382582E+97067396', '-80.8249027291187197860785081', '-5e+97067398', 1, 6);
    T('-2.98756402232656720856027E+4014729278756054', '3.44238E+80', '-1.02843306391765284274037022426e+4014729278756135', 33, 0);
    T('7.15312090872355253E+432773304733', '-4.1356633070091408392716827009243658137847E+62', '-2.9582899672807877933525571608e+432773304796', 29, 0);
    T('-5.360286777556375213356192906803468072044657712E+995795579878093', '7.56060263466942560189589717450264831210E+529976823172488', '-4.0526998332976415723993439638509552169984709910180259855525677990817594282759048e+1525772403050582', 80, 5);
    T('-6.63339439853E+362627855963067', '9.44534608076938671382264823617944446202E-45547', '-6.2654705784352938780174558934e+362627855917521', 29, 4);
    T('1.26088903017492397254622443496724786165693639015049305506576E+1558669227', '8.2241037690364033793E+94220', '1.03696822e+1558763448', 9, 5);
    T('-3.599691582996683146792660459202688384129231E-7367892', '1.05720E+48755258131', '-3.8055939415440934227892006374690821597014230132e+48747890239', 52, 5);
    T('-2.218467826137E+25', '1.9365929239994986971E-498835543834', '-4.2962690942174643302332008262e-498835543809', 29, 3);
    T('6.3667837E+66287524534402', '-9.60159761952337452426822811E+144748453604', '-6.1131295217940222690106209158629807e+66432272988007', 62, 3);
    T('-2.9622308813557790100992040223360737792688085733E+870724544', '4.69848980375810960637753907086790855556E-1734170620', '-1.391801159e-863446075', 10, 2);
    T('-4.6379106788026872605009562871377178618121483023935657E-82661871794325', '8.1588116240226998099028726160824542405977125531E-9705103352858', '-3.784e-92366975147182', 4, 6);
    T('-5.2601429385822978721492049749704277782831888673277746E+829041', '8.57015851124779737504E+4759', '-4.5080258775471079995452584229856304113385072944634050633963993965164785984e+833801', 76, 5);
    T('-7.943099029703887240440883360239373126959E+40843', '6.900056486846E+19409585879', '-5.4807831985568475591946247334802148e+19409626723', 35, 5);
    T('-4.7004545768492664646937377924295940E-42', '5.36925255309076152930402713434972706188994332128472929E+1425784', '-2.5237927737435079107553089167119751e+1425743', 35, 5);
    T('-5.91833E+93470', '3.51247143286860966181523071407752674216826747229818553E+278410352', '-2.07879650552892786198109343920464488439767224293265203677649e+278503823', 65, 1);
    T('3.183861741754768996537789000341889244682634103516632883E-578004', '3.79290593427092789951668196653464903650583630240596044E+55392862514', '1.20760880941998358738125494380032881404994522299440415964376691554195199e+55392284511', 72, 1);
    T('-1.625422132143113125776467305948061486927719487435217929E+252', '-9.7419024706509752064370154946381620355E+907797203368767', '1.5834703884975769662e+907797203369020', 20, 6);
    T('-8.258939618267655E+467761984265213', '1.51036545881E+21528', '-1.247401712582891300145779055e+467761984286742', 38, 1);
    T('9.27865284226502590372646597070161158876314747215164E+2477436408434', '-7.414502774472871E+204352335', '-6.879659724234462485453644394110473954118654593482944083695317815844e+2477640760770', 72, 4);
    T('-1.424638906103797E+5556976226383', '-6.43359490954316432021278987918828794844034905E-942303', '9.1655e+5556975284080', 5, 4);
    T('2.222257592E-490015', '-4.14650539085542E+570870899142', '-9.214603085097e+570870409127', 13, 6);
    T('5.276901157971272529324946413098470202043E-255703185803', '9.3890041E+90178903723684', '4.9544847e+89923200537882', 8, 6);
    T('4E-694351509371671', '-4.506459971963643660392426395667E+1325030', '-1.802583988785457464156970558e-694351508046640', 28, 2);
    T('-1.499431443206826599712E+82307745767', '-6.8E+2401951604635008', '1.01961338138064208780416e+2402033912380776', 56, 2);
    T('2.1324952246934858E+3718803909', '1.00547789542707446548291354185209614235945849249036409E+12979858028103', '2.144176810533092380627227058959509821872503701423004248177580293e+12983576832012', 64, 4);
    T('3.57252855289163299250023012283126E+38770298697', '-3.548668441155E-42672257', '-1.26777193307716792191381594359663815993045053e+38727626441', 53, 1);
    T('9.297007219E+78796036', '4.25181574521955738737672498700692790141259939599977824E+119194134897721', '3.952916167716408977042619167678108990244546e+119194213693758', 43, 2);
    T('1.01973944831419E+3051150269', '-5.32558750E+52389547474', '-5.430711659198946336625e+55440697743', 22, 6);
    T('-2.84297916726231641127461E-7', '-6.8E-91231218', '1.9332258337383751596667348e-91231224', 72, 5);
    T('-5.70094474854004563233553E-494021672146', '-1.6499803244281676609906E+6966564', '9.406446665743163197834798288705813418351376018e-494014705582', 67, 5);
    T('-1.28810079606386281021352899667979877985625E+385196678486791', '-8.19835702220960755216811079720178037E+82114844', '1.0560310206723955285367479940345685317807304969474333e+385196760601636', 53, 0);
    T('2.91E-86583425099350', '-7.060425837153613008E-497', '-2.054583918611701385328e-86583425099846', 49, 4);
    T('-5.06080E+931602438', '5.52270977436455612583489251847631677143133E+6513949157271', '-2.79e+6514880759710', 3, 2);
    T('-6.49815251792580E+790235854', '-8.29630609987748567661483605230528308414182E+566062', '5.39106623724e+790801917', 12, 6);
    T('8.90375895369353607299150782951989407717695670656431484835E+81750082496416', '9.415651106812480508383860894666E+44038243843655', '8.38e+125788326340072', 3, 5);
    T('-5.21991984916026E+76693033742089', '-1.6853920684163139302250682100064404434954894270034E+77', '8.79761151154358401268143305724938832017410034745927138605816e+76693033742166', 60, 4);
    T('3.8E+381704568471095', '-7.19605913548835924101904199171517235401717758778E+42150605', '-2.734502471485576511587e+381704610621701', 22, 2);
    T('-3.877749393906151377231762E+1102785966281734', '-5.1331485354426112918476792204357813024183700054670758E-9453', '1.9905063622142834538223808242958338019006955319494e+1102785966272282', 50, 2);
    T('2.9998269380232786889E-894979625920', '7.72063133990837085453E-247559869249863', '2.316055787200389151618649044876843103e-248454848875782', 37, 4);
    T('2.2657950552329558377013915323294591249E-51438', '-4.698135732822759921521329800913466873023505319E+1232579754210009', '-1.0645012712243068766909784170838265374e+1232579754158572', 38, 4);
    T('-1.42044E+78604', '6.14149088477E-26413376201410', '-8.7236193123626988e-26413376122806', 72, 1);
    T('5.3959873582250160291399399663067543964228754314646852E+678', '-9.364229076679246210278157960E+9556134810', '-5.0529261717284326814421e+9556135489', 23, 1);
    T('9.312646511762119765078636670874256576804979104396123764E+622238', '8.5093997E+940844359321', '7.92450314333946284003242214e+940844981560', 27, 6);
    T('-7.431019852137818443549304647401272493E+56707265', '-9.166522282135612291330526E+8500945155', '6.81166090536134e+8557652421', 15, 0);
    T('1E+10', '-3.513665967445E+95171372275', '-3.513665967445e+95171372285', 71, 3);
    T('-3.28334298569841849149E+15065586995', '6.544909704064768627E-4010', '-2.148918336887057002e+15065582986', 20, 6);
    T('-7.896834603346663093302278037E-4684', '-6.6119301E+6618810332013', '5.22133184e+6618810327330', 9, 4);
    T('-2.9865198872390776002775216628655256509E+177864', '1.02117567153954E-20487095622', '-3.049761451417556345845439898747472977307984963586586e-20486917758', 68, 0);
    T('5.188E+91247', '3.63652301564778873E+917472', '1.8866281405180728e+1008720', 17, 6);
    T('-6.782367075263797763E+9375543', '7.857916588E+11019234', '-5.3295274746620440917754992644e+20394778', 75, 4);
    T('9.22301660189026150419772043363478491967968959E+78279218', '-4.68820965551327378335181771548545055401243593353579453930729E+36760064', '-4.32e+115039283', 3, 2);
    T('4.558956247052611009E+1801', '1.01528442E+388', '4.62863724909418687775817978e+2189', 65, 5);
    T('-8.237331745137012764064444708034504729972645469E-144313', '-8.1451298133982E+908760163032', '6.70941363801669059352564021273523869915290490675063747227558e+908760018720', 70, 2);
    T('-1.6914E+47114017517271', '4.296937274E+2374', '-7.26784e+47114017519645', 7, 4);
    T('-4.1097767774091356490487335703322956150965840245773380270E+12467122', '-8.7285350105342797685356509033236270465766315126011929353E+416051069155444', '3.5872330487096388191928290443739694834115558905151744239377057052649172287e+416051081622567', 75, 2);
    T('-6.212019735191031204E+238', '-2.27776764474E+141869565161229', '1.4149537561304e+141869565161468', 14, 3);
    T('-1.231844073637926579066042087220566095910875E+658', '6.79789602074710179447186E+281666', '-8.3739479263641609308096838201164940489795674604439870946262554775e+282324', 70, 2);
    T('1.54335971200060167077938E+927440', '1.326081934516987843719E+5520642', '2.04662143254533908234071942312778039636771422e+6448082', 70, 6);
    T('6.43E+157', '-5.123E+3092314652', '-3.294089e+3092314810', 37, 2);
    T('8.994209820490557582E+125770', '-5.8093318E+8412362', '-5.22503491260480877608437076e+8538133', 64, 1);
    T('1.729784394041783813735795608456613E+3700890823', '-1.9015552441458333120153735211068940529835694893252246034562E+448', '-3.28928058573177655337924064661010266616963651e+3700891271', 45, 5);
    T('4.0029991047520820104292761750429192111E+95686410655', '3.2675056979897618563221505732065443186253E+84792', '1.30798223838253435664642008841869664506785442086273771409778877217400849125e+95686495448', 75, 5);
    T('-4.553168800609691617059727285398318588315367643E+5163999132461233', '-8.9713576250329734439683510547796503560891587352338E-2406281718944', '4.08482e+5161592850742290', 6, 0);
    T('-4.74889645513045608993956025634E+428566', '5.68232661900580986550430713823574137773906553853261740359E+607939408', '-2.69847807378901202071724102340909193381334091692803878266699605259e+608367975', 66, 0);
    T('3.979971965926967746854170421170600010711857302945948927E+4393058', '-8.7872E+386258287555626', '-3.49729e+386258291948685', 6, 3);
    T('8.885725150848036874768016E+93504938935', '7.79300771835198E+1937016749', '6.924652468371306314881124016672833427168e+95441955685', 42, 1);
    T('-2.242658055305774002141701076195813517E-8', '-9.2641E+49908', '2.07762084901582209332409329399856360028397e+49901', 64, 1);
    T('-5.4793761173689484186815253E+480970', '-3.529719207467553321267695996474220215596884346010559E-67587343803', '1.934065912641616404096320213269480716549e-67586862832', 40, 2);
    T('-6432606.821237086294263502572954884718170805', '-7.2156411658662793E+510844', '4.64153825831505502238215207715308628847102405244255272358365e+510851', 75, 3);
    T('-5.03249615300862082645907686368417001868903448783E+4079050891016423', '1.74560458641383101906578513029973340569662103637957614E-89043353699912', '-8.7847483658018092236462756021303452038107638943e+3990007537316511', 47, 2);
    T('-4179.0428090137413551506447084', '7.6435337234842151195155706959275796846076165824E+84693129', '-3.19426546425807361317209476261207541694e+84693133', 39, 6);
    T('-5.84356689787799389580224132E+56056768', '4.2318138528806670775830551E-9548563', '-2.4728887348675000956842661339461710592e+46508206', 39, 4);
    T('2.81166850857339903974425939249E+174839894863', '-23354.20109423943293462499590839839169784041300856044', '-6.566427175956343027549434658437946672844720803532351745102294050013905e+174839894867', 70, 6);
    T('-1.237296089845961066114E+433308128', '-5.40E-767', '6.6814e+433307361', 5, 2);
    T('-3.307480590899E+30274901110283', '-6.8995037409688566E+389775836449', '2.281997471008953486e+30664676946733', 19, 1);
    T('4.674003694658829755317828664180879261561887932443', '-8937249596.547744751329', '-4.17727376343522945799887710303684928283407452138242664967277569864667e+10', 69, 2);
    T('2.912423186140905304807E+85', '-4.56793551582801932984140664509253E-30937995', '-1.3303761309094039830882762252684348761697e-30937909', 41, 4);
    T('-9.0182186749634908411315267095098147569725261052549078811E+9127', '9.0484962777621301592502036757260065E+4131871203025097', '-8.16013181124520764218053039532490147901818549419083306e+4131871203034225', 54, 4);
    T('4.96683518346505502281095003569882318030486880052448E+43392343', '-4.199503930538594926109266471687008471027779547260451421284E+634874890', '-2.085824387529888181459609e+678267234', 25, 2);
    T('3.1222057E+3258872667679', '2.6303E+2205285218', '8.21233765271e+3261077952897', 75, 0);
    T('-8.987E+701125224881300', '-9.1256186624326215084496472402308682070109E+379618', '8.20119349192819694964369797479548125764069583e+701125225260919', 58, 4);
    T('9.56195290652867464273062282941247003E-28824638', '-7.135586849902522952102E+67181249920720', '-6.823014542e+67181221096083', 10, 0);
    T('8.09677135109085E-6967432102099464', '-2.7398696472798676353675538055100E+3204819', '-2.218409807e-6967432098894644', 10, 5);
    T('-1.89288626728E+632491539637', '-3.91326534E+91747536628', '7.4073662223088000752e+724239076265', 74, 2);
    T('-9E+3488009945101258', '-6.08086174738727026529446259314231558485065870E+873484', '5.47277557264854323876501633382808402636559283e+3488009945974743', 48, 3);
    T('7.07826196200251091551024599137826693010932827102304E+7403391', '-6.144041585884573253984860979798574E+40', '-4.348913585032835815559720035047729960399748532889315e+7403432', 52, 0);
    T('-2.224811E+6730592263', '9.4364847775842568290407180504956485382191825E-5668', '-2.09943951345020080200749089666412743199639576370075e+6730586596', 71, 5);
    T('8.5811147035800375177383695029949002299E+105630272490041', '-1.5466E+8887', '-1.327155200055688602493416227333191269556334e+105630272498929', 79, 6);
    T('-9.1143168990644115820260748595561878493463835925200E-290', '-4.99245827086470856001157524656758662830566992949274754412E-2157970255', '4.550284678601610470664901393997325976e-2157970544', 37, 5);
    T('-48895.04231758692963442448', '8.265824943481690E+52', '-4.041578604013028234769701863694105677712e+57', 70, 4);
    T('2E+469', '9699087.7377', '1.93981754754e+476', 42, 1);
    T('1.567028145282E-848874077358', '-1.81335570317072606E-248831', '-2.84157942427615978439910344892e-848874326189', 53, 0);
    T('-8.4750264993E+48600866', '-2.1392805872910605426551342433703944E+7807', '1.813e+48608674', 5, 5);
    T('6.95972986060452488065654128296641459217127E+362', '-6.3010437114133E+954590554857257', '-4.3853562071297504508791775135222746602469429819327355891e+954590554857620', 74, 3);
    T('1.4549018531169021471226323294414017699053958206289053297E+591890', '4377729326.291483037925353393505713993948763282802', '6.36916650926568624776085498778592066391524e+591899', 42, 6);
    T('-655400253104199103817255365633962018932573980.6126716139089', '-1.7582584674294873517176180922842037194400828696680194E+3093029105594526', '1.15236304457588722656128034672e+3093029105594571', 30, 0);
    T('2.091266E+123848849767', '-8.052934775583940469952386498019E+16393166', '-1.6840828696396324850835447502166202054e+123865242934', 44, 3);
    T('-2.55295531801191309E+407336171', '6.165086804415872500348798274E-9084', '-1.573919114333857281733424009504270139723000666e+407327088', 56, 6);
    T('4.30357599088997106992548688E+48', '2.82441794994421553391293865150621134795431162426586326E+760835', '1.2155097277618598075961357276357880466062732280861191022804974313399571019004e+760884', 77, 6);
    T('-1.67022384567284491232705583102629159078794262633451021E+9365592612', '2.23909919965964377857346177911911801411624217240390027627E+7149831', '-3.739796876098519427865869845321471571909145410361418335363875872e+9372742443', 64, 0);
    T('-7.33740275001375217528737422068930140879E+773097128910905', '-5.511910601090894355969827817071995630858059E+311038543120', '4.044310800227428200820703172559796171381187e+773408167454026', 44, 0);
    T('-8.133245151022250318035510412434138492464609861445990E-9381506215363', '8911.8484862054352207857', '-7.248225e-9381506215359', 7, 5);
    T('6.61707348187051914411469058806989658055051737564350393960E+9611388025', '-2.10305189007E+933000', '-1.3916048892779871165042518758093635796996195567963447142693271e+9612321026', 62, 6);
    T('5.2018526152042617474870803434041242606E-80', '-4.46007847772E+58843058792', '-2.320067e+58843058713', 7, 5);
    T('-4.202E-52538', '22348275.098711149315760568982245835392355', '-9.39074519647842494248259108633970003186758e-52531', 42, 0);
    T('2.01803731689E+5880318384903245', '8E+31836162970', '1.614429853512e+5880350221066216', 16, 0);
    T('8.945528438169937315993054E-73403015597', '5.5398811382636356976665607390122278663193698853528', '4.955716426641859560630525646896504398927870826355381365e-73403015596', 55, 4);
    T('-8.7253973265333810876023670648183639786465715E+9436798', '-8.183208361604903534116195300922E-640257052', '7.14017443608130349410550079650926609912277454861419948191042445577261e-630820253', 69, 0);
    T('4.2436991995E+509721648224', '-3.7709955470204922465760908075438093243E-4564749189867112', '-1.600297e-4564239468218887', 7, 2);
    T('-7.9417865820197838117599655418178521515877707331805649336E+877190864744868', '4.052331064662253447466794804339801244E+6067488141567', '-3.218274847523662934594737164187329349e+883258352886436', 37, 3);
    T('8.45241799056245683472907474376382232566E+7968386', '-3.10570984330390884E+11375392165', '-2.6250757753008867844587679e+11383360552', 26, 4);
    T('7.897298840870906461996869239825303E-849814196504047', '-5.6497123661744070796054761E+41627875', '-4.4617466920643171275581027512350169558134442166059326141759e-849814154876171', 59, 3);
    T('-8.28261549808530349477837839593916429803428679970523153769820E+80', '-7.70007494010270558712590072865340637394617573420326135E+395', '6.37767600353129343101063481382778576393874117725e+476', 48, 1);
    T('-7.4286879897280516413820878516500637109227773735E-413751232755', '-2.6640560759360092337195544249E+61957169659189', '1.9790441375267875e+61543418426435', 17, 2);
    T('1.185574749896771658549649623402051196148279000581E+66849354628', '3.32642276829099919452440148862750059228169E+477180213444385', '3.9437228415675281919651056e+477247062799013', 26, 3);
    T('1.20E+9655', '-2.7777959069941343E-89672897698708', '-3.333355088393e-89672897689053', 14, 6);

    T('6.4038944491845633402085746441793926268479E-18949003521', '-3.5219375698254711184018523145157104E+169', '-2.2554117e-18949003351', 8, 0);
    T('8.444360295E+33541934', '-4.49610807458946254714842776956036053245838E+824000', '-3.796675650709216e+34365935', 16, 3);
    T('3.541484183833276126623687915345113288E+46748958428949', '9.82895571082678228808654540939215251179182297794694388E+13150221', '3.48090911934908e+46748971579171', 16, 1);
    T('7.06E+10', '-1.82507639239917629473E-1984975006', '-1.28850393303381846407938e-1984974995', 54, 0);
    T('7.288286609747926697115569099035319092188341034012E+629516519576605', '2.7606850911678524654535426132451663780619951010618774E-19209', '2.012066419e+629516519557397', 10, 0);
    T('-9.6200148919960881965228985914E+87540056541994', '-1.7098149735E+4503340471980', '1.644844550762789690184146264719777833e+92043397013975', 37, 0);
    T('-5.5770057048775E+2619291', '-2.353813791E+941274', '1.31272329406263354656025e+3560566', 36, 4);
    T('-8.17E+47398966264', '-9.9378647407417333E+45756039868430', '8.11923549318599e+45803438834695', 15, 3);
    T('-4.9637E+50428592075101', '7.802445639420249E+51374585', '-3.87289994203902899613e+50428643449687', 40, 2);
    T('-7.2974554038059191471863449E+65', '-9.9030140155620927111162736459085466787361165455540268194E+7349685006574', '7.22668031418293481467185919182680940892e+7349685006640', 39, 0);
    T('-1.225090449595008613845455691289480616E-86250293', '0.0895401830435253255467', '-1.096948231016118078130742534054657682573776673205592527672e-86250294', 65, 4);
    T('9.01507564748127264378730988582E-4705980572707873', '-6.0424891453634771059324E+3310908', '-5.44734967445362101469784e-4705980569396964', 24, 1);
    T('1E+97565569663974', '-4.4506938836167287444E-120', '-4.4506938836167287444e+97565569663854', 31, 0);
    T('-5.75976946130648597026700770502207504362642972906975E-45540', '8.520655858400629274455E+51918864', '-4.907701340351814627626167669658118773927803261911013724251475616458823625e+51873325', 77, 5);
    T('4.78277793459505461790018713905273883208668849801318E+903252724', '2.322025714258494808111014132989894402162718178020E+5296374333844441', '1.1105733349717850264439420369188443272046918548183748014854850301e+5296375237097166', 65, 2);
    T('-4.5538118077487322588642182086E-59042344', '-1.201907189985157346691979287358607E+2376715', '5.473259153572509e-56665629', 16, 2);
    T('8.72821448085667948693464E+928165', '7.293056100802125823873022750176874060636451944034390E-2222896', '6.365535786872126578933115814201978127609599073632319447228e-1294730', 58, 6);
    T('-1E+7350943526', '2.0213602985058100875491848E-14296228608159', '-2.0213602985058100875491848e-14288877664633', 62, 3);
    T('7.904396534306639227121985E+471317562657', '-9.92874811711E-5169160', '-7.848076220688785374565947421303566335e+471312393498', 66, 1);
    T('-1.9801E-295009988', '-3.53812858192217653369915329640E+3018299258806', '7.00584840506410175437769344220164e+3018004248818', 69, 0);
    T('6.322877628797853384187347858080795782633817437651050101263E+165403218563', '-7.613382778982588083087035854071210879536730688538E+988', '-4.814e+165403219552', 4, 3);
    T('5.225344331E-499530', '2E-66894', '1.0450688662e-566423', 40, 3);
    T('725657998.483', '1.750692943663682E+967990506881569', '1.2704043375e+967990506881578', 11, 5);
    T('5.540881630516582984984717977829074041197778901255885060E+5718329784567', '1.937186035883296930011306393183E+6790770', '1.0733718521118998128222349806086241954e+5718336575338', 38, 1);
    T('-9.07811384884503508158983766E+63517', '2.69E+34', '-2.44201262533931443694766633054e+63552', 39, 2);
    T('7.645995238237539320276572689777723250090514383E+182216682', '-4.965974766732203747608806419309882272448365339E-81', '-3.79698194196422049460628670678190659731578227188815257895937477319e+182216602', 66, 2);
    T('-6.63042705349324720149137E+39', '5.5529684934195676281461443733E-9896038', '-3.68185525259647398505721906701969183614142950940084e-9895998', 51, 1);
    T('-3.92434065374579551256312186061335084987420108510509824576E+44409118257422', '6.746678164E+6554345785234214', '-2.64762e+6598754903491637', 6, 2);
    T('2.831427813052833944616788E-2375334051224', '-8.76153469170388234761614E-6952366', '-2.48076530111176592e-2375341003589', 18, 2);
    T('2.480769635886165512489E+832415333', '54516.48099784123511062164050714228547595410304', '1.352428307148097e+832415338', 16, 0);
    T('5.81690994928709216415124688561937E+9864667880', '6.3939418755885410037796926012648035043155562259532628270946E-7339955517', '3.7192984111274355005800537302350809763934851395015039720321e+2524712364', 59, 5);
    T('-1.109426878120756629090E+785232485629', '-5.32467269552985274961288091957020287495024093104980384854112E+27536111843', '5.9073350056165186175e+812768597472', 20, 5);
    T('8.3577703049424548624463786753341587641498279E+202673582367', '-7.03997188168757501142638236649736627916E+43', '-5.8838467940398271567654116382306856799748807748498e+202673582411', 50, 3);
    T('-1.2977760780725474845702453415925678127734288534E+4337770204988693', '-4.863958919763576498876231699E+74553', '6.3123e+4337770205063246', 5, 1);
    T('-8.283320913530570634093661288276308E+7294072807269522', '7.58925085115068881E-16281508447', '-6.2864200293366184370618963e+7294056525761076', 26, 5);
    T('5.3778780356910E+39963137907658', '7.5552834342498600677548719833563290464802216147257581313E-7232115296', '4.06313928344723900132691911103475040978e+39955905792363', 40, 0);
    T('3.9885591490957974390125481290810260459E+7724567185', '7.8107966E-795883', '3.115382424065634771092791828396243936382716394e+7723771303', 73, 4);
    T('-3.115128181557871900704052840688545625922879215580137699837E+3882906', '-5.72397190702874270955433855705677E-1092167630', '1.7830906198030791478881744721057665733684998128656037423262794036e-1088284723', 65, 3);
    T('-4.553543801E+1244296', '1.4873374999993330489374386379938791669739541870913156990059E+796', '-6.77265645311680050912350334685491155671729e+1245092', 42, 4);
    T('4.023E+271', '1.3425819975993901E+22804', '5.4012073763423463723e+23075', 44, 4);
    T('-7.5296245764047219184211168158292E+69913112278267', '8.6874947827960901168759441383321707002E-32', '-6.5413574223929241695506e+69913112278236', 23, 4);
    T('-2.6223879590700936466099655942486790E-721052220439', '-6.199162214E+3360605309301360', '1.625661e+3359884257080922', 7, 6);
    T('-1.92267774290E+8411135', '7.0584538901315490320710694808963493023E+595881389', '-1.357113219384185127706308358191886755e+604292525', 37, 6);
    T('-6.693694610792229409765352835977853563662E+8850609146510', '4.8054415016513E-952', '-3.21661578820806249940081402044961977337944963626750606e+8850609145559', 58, 6);
    T('-4.3E+8459240650446', '9.50632525575094810E-92445249726', '-4.087719859972907683e+8366795400721', 27, 2);
    T('8.7373324582E-49092', '-3.43735717638122841381146237316014558700E-7', '-3.0033332427622409437182991368220140543932592e-49098', 45, 6);
    T('-4.682238463535879233759645820526811073284043317265627318E+9438416708', '0.00000925948373644962534417512206973717616939894334212936857272', '-4.335511090328935589813489705584743809230273022826604363257998209878610760147582e+9438416703', 79, 4);
    T('2.75510414004988950160166395384427752198E+992303077607', '4.51540338868798060909682E+164719964', '1.24404065701695557689544782961601e+992467797572', 33, 1);
    T('-6.433067217382202433588692008762711479507554583477003E-98712892', '-7.33343023195747612987417649810349632719697939480004908E-870', '4.71764496161652004e-98713761', 18, 2);
    T('1.57311328851822282305834483637E+2844618039597', '4.5841172623518537991119366956643378E-5453', '7.211335781531477531742739081996255779600500192963943677385405786e+2844618034144', 79, 0);
    T('-3.415186704929281095581842086001242321893288081457853044E+99', '-8.53432234423583580648489680582E+8585509986684', '2.91463042e+8585509986784', 9, 5);
    T('8.67463E+987261607744315', '9.67372004057136712526503912847145687529490775E+14256', '8.391e+987261607758572', 4, 1);
    T('-2.3605300062349337904E-861244273', '9.43661437932231861285626099686961582577451911137397783E-1261689320953', '-2.22754113996583786158168371431129e-1262550565225', 33, 1);
    T('-6.80793832946430714478E+6989788720060132', '7.611298641E+77233228003984', '-5.181725175506349122907060424398e+7067021948064117', 75, 3);
    T('-2.5717260182137E-67932601', '-1.9020910254569360616573415436721210655330E+1088236326', '4.8916569791783796604667846849500509e+1020303725', 35, 5);
    T('4.89455413907411002E+65396877', '8.41479898726996781389519270784198704604318692E+3753876232', '4.11866892126188501951400542404577444636770879288586633835049384e+3819273110', 70, 0);
    T('-4.47638265410815691352049508291968734E+19356682607592', '-4.613901700995303E+634032511745', '2.065358954209549425194077614842247569900969156857e+19990715119338', 49, 0);
    T('3.551089868862128721043220292619713604942212358937418379E+809', '-7.246020523517170E+48148', '-2.5731270070628881e+48958', 17, 5);
    T('2.4461065283E+7727973962469', '9.19365720E+625', '2.248866489587229876e+7727973963095', 65, 0);
    T('-8.4321067363951094018001936682982192040247230413E+6089397', '-8.6982030678123497022025873543851941230818866948765757297611E+65', '7.334417668263312051878997775921519521338029769013486386563899473392061120196e+6089463', 76, 2);
    T('9.749761905985781588E+6069456842282', '3.032877744485639E+34729', '2.9e+6069456877012', 2, 3);
    T('2.61479158693130027417940607485E+40871599', '-3.2044222992211E+33698112', '-8.378896468978385999516e+74569711', 22, 4);
    T('-1.2430294693563809793902784538463037385249744606773493021E-8869240489', '5.2E+88', '-6.463753e-8869240401', 7, 4);
    T('-9.013464822497961209870603716025572319394903525554E+204600321766387', '-6.1077000709924564174E+508', '5.50515397362588062624812245e+204600321766896', 27, 3);
    T('9.23475605644351420572831206560104631621669931510381025E+293', '2.481868341931372069918948745382E-4940048343', '2.29195e-4940048049', 6, 2);
    T('-891.14', '-5.834080173724617992946135409E+940', '5.19898221e+943', 9, 6);
    T('3.222165933660165604E-3694359412441897', '3.16500417E+1482689645247', '1.019816861646636749955056868e-3692876722796649', 29, 4);
    T('-5.006871E+95219', '1.2980888398795872093637571185865314E+30358064279', '-6.4993633678167486905e+30358159498', 20, 4);
    T('2.786155925500512733439377544687261159890951994703465118674E+58878457766', '7.789722990192E+297475207056', '2.170338286714e+356353664823', 13, 0);
    T('2.49029856457604710869060E-7302602', '4.888254633E+681455', '1.2173213495842111959883081e-6621146', 26, 0);
    T('9.6284258E+138', '8404618996.838953186569670385612605940926370646751', '8.0923250388334295306559627838328363846848743035540015e+148', 53, 6);
    T('-8.845159979814128178738096340507213902256E-137531075', '-4.59169069017594228177907155054800284E+5022', '4.0614238732429357918327797083142535e-137526052', 35, 2);
    T('-6.90903949425733367710685420212498171E-1948130', '-6.096016589738627088601491E+37', '4.211761937615208005722776491276435343041440468340091785372961e-1948092', 75, 0);
    T('-6.8661437357463055952516400834967253E+917', '-7.974835737602482770006487239511937E-87', '5.4756368443345055523911e+831', 24, 5);
    T('4.697660539269848330E-2790140277983547', '-4.5787560846037829039351735911E-8319534', '-2.1509441777584906080846762321334848624718437863e-2790140286303080', 53, 3);
    T('4.595018848542932E-5838468325601', '-6.0546293309198715581638320250E+67059078783', '-2.783e-5771409246817', 4, 0);
    T('-7.7859E-650', '-5.4449873691532775E+7292340726983', '4.239412715749050328725e+7292340726334', 54, 5);
    T('9.51595365552420196778517014266048762242111042942872641505E-5691912334700', '7.59962510023385953168613215127407228314227156667571591655034E-8431235879', '7.23176802531838e-5700343570578', 15, 3);
    T('-8.10964E-7083', '-2.84062431159465227525E+873', '2.303644054228045587745841e-6209', 31, 1);
    T('5.288437831019007061064431E+92717253015993', '2371.01', '1.253893898172437593185437654531e+92717253015997', 63, 4);
    T('1.02339426969001337784E+807050017', '-9.326175552884E+6954919487', '-9.5443546189445779e+7761969504', 17, 5);
    T('6.8297064394750579612772942585013213620056554412527398579E+70070', '-721431.35511', '-4.9271643716339842620500723033848309079173308724667091116851e+70076', 60, 2);
    T('-8.458548276725799598158139E-441627501637', '4.7795237489E+9684469696141', '-4.04278323698281283127389020483272971e+9242842194505', 55, 2);
    T('-0.00000955411022586670249082474101700000396900810000187914984', '-35949.41061710310937374545504328457669291674655', '3.434646315907458208725194535301908769810573629527556e-1', 52, 3);
    T('-9.038631E+980171024663', '-5.73306807094322043544354425E+7935327', '5.18190867911375914676335178e+980178959991', 27, 1);
    T('-2.722559660979784031990701521267E+315664513', '-9.39261095164194925670242050028E+958913698539', '2.557194368821731203918476374830946173012968291422852319945476e+959229363053', 67, 6);
    T('9.9782785777542462627695944207931659832708009852412082E+5123669365403027', '7.4187631563045981199928E+18113', '7.4026485e+5123669365421141', 8, 3);
    T('5.92754076945560514165214132105102258757E+6878', '9.506951430701699585966078954641E+98092', '5.635284219871861852666922868869686186910913145372366782727370448041237e+104971', 72, 3);
    T('5.25018679401687705453E+80288075', '7.71E+44296915566248', '4.04789401818701220904263e+44296995854324', 67, 6);
    T('-1E+41', '-2.52646964502072401223060931281749217364283985381084347E+953387122', '2.5265e+953387163', 5, 4);
    T('-3.7833525203356018233058840439869353236412749E+812294351719', '4.000654405866868942128339736510E+25847456679657', '-1.5135885929428148309736628e+26659751031377', 26, 3);
    T('-8.50279318697296344639706768527685457607243E+616634729', '-7.850369636778538175930848977074507836373405E+15508392', '6.67500694628199720906366348322037842928625625176e+632143122', 48, 1);
    T('-2.4265733667993444495E-855235300', '-7.2920392139271E+8815825182897', '1.769466814e+8814969947598', 10, 3);
    T('3.95900060E+188828743965309', '8.991099862336006292653E+345768', '3.559576974964816632e+188828744311078', 19, 0);
    T('-4.5311349386323134290429890333100198488511833803E+1470393', '-6.2894741581289394980426719665356175784511801E+356416499543571', '2.84984561035230934402e+356416501013965', 21, 1);
    T('2.9505681487493526395974816318960710191376315942308166164E+25637', '-6.15695230564738470078102863480028242085814752298348402353430E-78612', '-1.81665073664120622797396820525806032702208783652e-52974', 48, 0);
    T('-2.5237678792637577332079641070319548238021907286181E+1980136', '-6.83680610038766839586458864274841042687958083E+82911498720', '1.725452e+82913478857', 7, 2);
    T('-5.6827090346562760711355897868522833150847E-1123', '-4.90063607539302025022624623598144985E+340', '2.7848888901198391466129897340665348501822869917472313092333937512851881553e-782', 74, 0);
    T('4.75581E+403', '3.280315716928E+41', '1.560055828972335168e+445', 72, 3);
    T('-9.429305418561982745E-28', '-4.87406445736414414117497E+631893904891', '4.595904239824409447237053869602e+631893904864', 31, 1);
    T('8.5240056373599295491996293177597707434962567E+8910855167708993', '5.5035605362664645748E+4011532640249', '4.691238103668698103246726692093671309878784843755680110695715116e+8914866700349243', 77, 5);
    T('-9.66289407774965929929094936E-388591901', '5.421401083077181E+6635350', '-5.238642441877208095587177044678073164255416e-381956550', 47, 0);
    T('1.1860633278521368651792736801878581272133993219476425786089E+281212139993', '2.580420982566458800792593568652899E+57', '3.0605426978422549708034971876329784619395864841770875289571036592382671078e+281212140050', 74, 4);
    T('-3.7552811963800451859000256953363E+4920649352', '-3.619849054273153582296569974161488E-523119', '1.35935510872460633019796205750687974241755675636724417617446684144e+4920126234', 77, 3);
    T('-5.0200360E+19009', '2.0024067E+832189999438526', '-1.00521537206412e+832189999457536', 58, 5);
    T('-9.85496893492270147968954392889333496217875791E-3501781527', '45.46098125951270363618228', '-4.48016558063600800989216205854714248674861135513041657449326705e-3501781525', 63, 4);
    T('4.2290197051439903584445567813778E+719614737691', '5.001780880365338961872E+6109257117', '2.11526e+725723994809', 6, 4);
    T('391523069.7576747225051394969743111683', '5.7504376773158253742293812517806875142E+17685727563', '2.25142901187288490422972774998947120864867075945091481e+17685727572', 54, 2);
    T('8.8584540078825662697E+92401212432', '-7E+89080625', '-6.20091780551779638879e+92490293058', 64, 2);
    T('8.7404146877997773148E+249', '-7.4702669975353222137650284403239868749108106750134485462548E+8328341631', '-6.5293231387043673158597066365058263964013631703810541247357684096459e+8328341881', 68, 4);
    T('6.39247477096658372196300414245380074928711661403285696847E+4316215254443', '-6954.966116090078924021287374922471990319383732095757953554', '-4.44594454300332776037317541250961728218988179396694131e+4316215254447', 54, 5);
    T('-5.405308186668626809988497778263457E+50868824357', '9.3453952548516197975720373951109889871556664992000E-117816585', '-5.0514741478703598524508879e+50751007773', 26, 1);
    T('6.16888602955203400403087314467465734E-3003734499', '-1.459055654467607015909849419697004928E+6121686404', '-9.00074804318412068854829003426257227e+3117951905', 36, 6);
    T('9.1122550897E+4380343', '6.372472343431905908856816142055E+387028163823', '5.806759354e+387032544167', 10, 3);
    T('-8.8523964316889197288831850E+316256777023627', '8.55446445076662091886351692788E-44509688949693', '-7.57275105789761495662403223893259e+271747088073935', 33, 0);
    T('3.2781524219080183763200495E+706841', '-9.73E+794', '-3.1896423065165018801594081635e+707636', 56, 2);
    T('-3.1715939147049349663312317913123568740E+9107063455', '-3.32450357992338107603E+690998', '1.054397532349976682622086747885861616739560054214948713022e+9107754454', 80, 3);
    T('5.2473703986278313344624889737378096801154285E-578421105021823', '-2.506E+11374498', '-1.314991021896134532416299736818695106e-578421093647324', 37, 4);
    T('3.01413416869805E-9789431', '-1.6340180419130643255109259397427E+713', '-4.925149612399249563261770484718883050810991735e-9788718', 80, 2);
    T('9.8720844691634544935416557165702253379E+446482022911056', '3.8723192608292847585012552E-217', '3.822786283447529010838910654069211045717655520788e+446482022910840', 49, 3);
    T('-9.8009887971127512564771252304255507226E+8029460411962', '-3.052566202179535816126747763867424564233938935584579E+452005588161', '2.99181671500066481912938557190788646935552373294217416e+8481466000124', 55, 0);
    T('1.799E+1443353946065340', '5.42754E+95791645100', '9.76414446e+1443449737710440', 33, 0);
    T('6.106207691720182038717270151108570E-3928', '4.90612351091045209501237905786304684847480599222238835323344E+29921734779', '2.99578091188506270271378723206461034272178442313234562606739770499221297e+29921730852', 72, 1);
    T('9.44043388222701312184760419969406831174394772E+8087078371', '7.674758098148965057207E+59770', '7.245304638766164200457700916375762791926537160451613964204581721804e+8087138142', 76, 2);
    T('-9.16369048187731745953447197426138329227476998425233877606E-326646692274', '4.9629801917395917236696690629461769258495E+616809468283985', '-4.547922e+616482821591712', 7, 0);
    T('-4.4053931978481227676277490025293282888469023711505E+6514182280753', '-4.030808847467032879070E+392', '1.77572978784572980827224587868506460543260584090320783533231277232e+6514182281146', 66, 3);
    T('-7.1554632803505284525253215482228348732470004159105006E+451330869770060', '2.646685311382459567839E-25920706', '-1.8938259560240293980277025809621263121339e+451330843849355', 41, 5);
    T('-2.850951714907948347717077988802623E+640', '7055563186.600845743946053269124918', '-2.011507e+650', 8, 4);
    T('9.649923977870738346E+2700', '6.7208739275048607829148006642367060420842596338344301945E+8520168225271506', '6.48559224652754385004066332630688146283068388664047630082378809e+8520168225274207', 63, 6);
    T('-9.6474298022090960424643494695033E+512291964757', '-3.92665620297510693543706156234439896E+42984078', '3.7882140075611255985232581821370007819924144e+512334948836', 44, 5);
    T('0.0007267155218064794', '-2.5118497701124359792061420476461968302018615E+4682176024499968', '-1.8254002163867442366983519856037946680291705470454406414031e+4682176024499965', 68, 3);
    T('-1.5729689530495368407E+362675724', '-7.305770613648115468310530602259252294E+154', '1.14917503533701484936905479421191001680326198229889875658e+362675879', 79, 3);
    T('2.84159054329327042891292323292630077894286065296897E+2014', '5.24055575922497E-818359955305', '1.489151368701475e-818359953290', 16, 3);
    T('1.301188109E-25', '1.5306620888201E+6140097923', '1.9916793088698159601909e+6140097898', 44, 5);
    T('-8.891245254677617795402503062876871937587450577E+354330', '-6.9539767043E+88159005', '6.1829512373246074755848129941155165593237534856485593382e+88513336', 56, 2);
    T('-6.19683002355421256065074502523130892466519264700657809491025E+762926', '9.72995094195453035357746021891348612978347852925440E+46608567', '-6.03e+47371494', 3, 0);
    T('2.09253196786216829719659280E+72730625672', '5.766714061868793035277606285984044E-7697110', '1.206703352398074322916641426555467011118363e+72722928563', 44, 5);
    T('-8.3963622721566030977E+782967393', '7.9923717535740952E-43194853', '-6.710684865675964426839597875479470104e+739772541', 61, 4);
    T('-3.544836515110347444687497081729337015638745281644E+784065742717166', '4.401112084305599893858601040494906864572743388015690E+3571696', '-1.560122282353990039501648039266112723933673629737804014e+784065746288863', 55, 2);
    T('-1.4440927022102522102582118349160734764603695230286991E+8862855474463', '-2.28654214716423403E+88525860279355', '3.301978828016030898432886741196921681917408748414604204787148758850373e+97388715753818', 74, 5);
    T('-9.576400226981964738E-51945662079', '5.014118429165580117712903614377521376736141153568558E+78930349200552', '-4.80172048632e+78878403538474', 12, 5);
    T('3.39719973439867994491717751730737E+75', '1.09E+597679', '3.7029477104945611399597234938650333e+597754', 38, 2);
    T('-6.2989968762864910449757816031846093409438156017822100772432E+5655', '5.378439894198286248481467560E+91165', '-3.387877609285e+96821', 13, 3);
    T('1.10897E-985471', '-4.17009597384776107012646693703E-4204157366140', '-4.6245113321179515939381480391581591e-4204158351611', 68, 6);
    T('1.4016352E+285322', '-6.9860977E-156298415821594', '-9.79196044695904e-156298415536272', 42, 0);
    T('4.204908006293398316936132E+2764028275680852', '-8.96199946595230715838419713E+744', '-3.768438e+2764028275681597', 7, 6);
    T('-7.49877113574944959166354074354932885789877277943775E+992110406024629', '3.28358495147497872382922756E+68', '-3e+992110406024698', 1, 0);
    T('-6.03907561633494206103445342896887309539883329737394049E+50000', '3.31748111536486948475359077319E+492581375216', '-2.003451931145163021062108055899749272745593216621658156852829e+492581425217', 62, 5);
    T('-4.27404267468279955779037562807585422951873570493809E+2008202159833451', '-5.856E-906910496293', '2.502879390294247421042043967801220236806171628811745504e+2007295249337159', 73, 0);
    T('2.95858469012169101E+353419795001', '9.372154531566137E+103661378827460', '2.772831291054620160146489009332837e+104014798622462', 38, 4);
    T('6.0746042619995594E+44324660', '-7.81964606073825532E+6188499', '-4.750125528788e+50513160', 13, 1);
    T('8.08492250164876879400832112202755440455164907804476445637E+5695', '9.6028448507795525404389415698469900747278303042E-4019439256', '7.763825641390961779779577782001541065885630757317890227752134498275e-4019433560', 67, 0);
    T('2.121797028410561670839571892728394487704413610454E-282', '7.560008562960641230315540411033265431702491297089097330013E+3170', '1.6040803e+2889', 8, 3);
    T('-5.51174559364E-794783', '7.381821619460202E+96338137', '-4.068672278409625724884431528e+95543355', 40, 5);
    T('-9.4274653956714454063E+1129674346', '-2.91304570860450738E+92', '2.74626376138781983e+1129674439', 18, 0);
    T('1.69002594579365194511421669E+29289819419884', '8.671916316685178954024528157223478600E-3788195463606', '1.4655763574949272081874320559274123259026300375485742315977834e+25501623956279', 80, 5);
    T('41185450764.524425674210968793477408828081618726068688', '6.0895397150545458304922415660E+5152462', '2.50800438113e+5152473', 14, 2);
    T('3.8267803805581273611542873461929698253499E+89019330507', '4.92923565809E+612830909566017', '1.8863102307526341564557388688936463273190126691015691e+612919928896525', 78, 1);
    T('-6.2423593423E+565103917', '3.866E+10384', '-2.41329612173318e+565114302', 59, 0);
    T('8.339918685816536992710060038266421E+588', '-1.6658903448652057353134031696871172053448010054828732776480E+149569053370026', '-1.389339e+149569053370615', 7, 5);
    T('-2.55006999019532406719570936735E+9007', '-8.5312123380365118677725018100401211419406492794E+240', '2.17551885632109955302610134749167e+9248', 33, 1);
    T('1.254800433016940355449106411928502E+26009260', '8.2289043E+6553834171', '1.0325632678895e+6579843432', 14, 0);
    T('-3367322577077292.0', '8.21501525409713335110303727150136580907012254038821E-769333365923', '-2.76626063361556238431686009e-769333365907', 27, 0);
    T('4.40991605662329971754E-81198087684016', '-1.59337860E+38604656', '-7.026665872419954031314280644e-81198049079360', 59, 2);
    T('3.49843861357231270848303714E+67557861201449', '-9.25058337989890743193114188326476643270563209029017469576711E+27760480858', '-3.23625980943086122255786877393627366063919e+67585621682308', 42, 6);
    T('5.6471936637185798633409904757E+960608679', '4.46264499825462018030753896952594878453420869700234935832E+69280066', '2.52014205575689039757e+1029888746', 21, 4);
    T('3.6732060477004434E+7986412308', '6738685625905174040370783.661423635438395868513169', '2.47525808e+7986412333', 9, 4);
    T('-6.1151276536136218E-8819086742364', '-5.611010777459093570009728012012917499914618208763E+905582564', '3.4312047169964170699364827668067e-8818181159799', 32, 0);
    T('1.0464571071075438868619479449745202537162581898836785E+53', '2934.6903372113253136081765887726020921155850355536', '3.0710275605346259409141868373635815774858073054000807588768103425267664389071182e+56', 80, 5);
    T('-9.6759787674052282207695889193365685357E+255243', '-9.931687735884321480772E+957771407', '9.60987996569155987666034213195324678615720371908394877455604e+958026651', 78, 6);
    T('-6.436051E+77305429028', '3.0122219E+919639', '-1.93868137717169e+77306348668', 26, 3);
    T('-4.94325318735046178053416721289378007275E-675230676109816', '-2.31509009245480060194856766920541003723609384102338866515E+4210120', '1.144407647853066832493049475167335588e-675230671899695', 37, 3);
    T('-5.91930961852694751925414914207662557451E+730421', '-6.9110857108014612046524957701262989463615990049014420513E+847060619655', '4.09088561224112352678130125473975197e+847061350077', 36, 5);
    T('1.441972193661774063E+8040', '-8.359950447753812668E+2910395187', '-1.20548160860513e+2910403228', 15, 3);
    T('4.5E+8084', '-2.3073677577042940701053549839788943912891207081E-45', '-1.0383154909669323315474097428e+8040', 30, 0);
    T('9.440428854981531440E+374913', '1.98737586966628715680115123326855260911E+147622886093912', '1.87616805056916325252900135954346685455e+147622886468826', 39, 1);
    T('6.1870005355158763E-3288072518174778', '5.15614454285023550201903972005456982E+702', '3e-3288072518174075', 1, 6);
    T('6.02992283897041631671076657504175990601586881506908694E+414', '-1.4520478893894485219775589733067066007192340622809E+70576', '-8.755736731508224483406779005741004874287539120993480589171053427220693091191743e+70990', 79, 3);
    T('1.98663144454181513061617082E-53203783', '6.2371999768986162556731828222858013400E-784', '1.23910176000022739737723897699709171202753580218040306860248988e-53204566', 72, 6);
    T('1.90400140136729677544986556551671030331189394E+72848894', '-1.95473581552135411012363669965539235661150026113483845644293E+53979550181', '-3.721819732055503932989869200725571747696560617975208606688726e+54052399075', 61, 1);
    T('9.282591597330304917916E+508919810', '-6.6646024839868343579503569706189550757450530237614135330E+7786501012041226', '-6.1864783017e+7786501520961037', 11, 5);
    T('1.50318181539086693276610305598963297636383326406E+6585727790158780', '7.511677479763956021567068E+23110735', '1.1291416990662275520522913e+6585727813269516', 26, 6);
    T('1.98176559586969426604283289269843411765210258E+975961171609', '2.393671640682030788241980174223278702E+7582537846913089', '4.743696105312613452e+7583513808084698', 19, 2);
    T('5.432169713707154729867701651524583138126879297790802324E+208732', '-1.38989841976499533093475738232654665993394E-1576235', '-7.6e-1367503', 2, 0);
    T('-5.5465083E+724', '7.36693356368408893428524656179461626691636542E-968346465', '-4.0860758156522377851951274622540302019766636207862986e-968345740', 70, 6);
    T('6.45568905715854833041678952644113079850743522509E+462369919452', '-5.1098452301822660905109553577E-322522', '-3.2987571936261458745907339899260689192e+462369596931', 38, 6);
    T('-1.3515364481', '-3.71338472464E+34983993355919', '5.018774801168742151184e+34983993355919', 62, 4);
    T('6.2566505694609589258329855857431135680401805226825E+28029989074', '-5.21391588959600403890769154578467637435E+833695', '-3.262164981976238091800981593579118713642524191697e+28030822770', 51, 3);
    T('-5.262E+7525', '3.450405089633940483493126065943908806526231389014745016530E-334131', '-1.8156031e-326605', 8, 1);
    T('-3.98541379984845491310809288445476E+42', '1.6216321038229098242367554859E+2114400023', '-6.463e+2114400065', 4, 0);
    T('5.73334472537530686341714749087574025043897628394949476E+8057090', '7.73172817439411096319952510899851173578E+82133082', '4.43286629466981268118505081255357607317829731760782e+90190173', 51, 6);
    T('-1.16553331548925349499982517459438067934716E-5573299659', '-9.9378975E+884', '1.158295e-5573298774', 7, 4);
    T('9.26865597207395822520117E+1249', '-5.8727755E+360560584983', '-5.4432735710724626052984913747335e+360560586233', 36, 1);
    T('3.64102105436312746459943E+91132303395748', '-8.90624122935810286615866938109812942771724317E+8867610', '-3.242781183132979623790917774295689552137e+91132312263359', 41, 4);
    T('-7.8834036E-81543703113', '-7.7231683965392791E+5980170453767', '6.088485354068398039834476e+5898626750655', 57, 6);
    T('-8.2131369714049773881062190829E+98664852', '-6.214606E+458183860595', '5.1041410301315200906e+458282525448', 20, 2);
    T('9.277043035467781844495991513468E+93690', '-9.45066441747688019729562761961132633751249E-9081', '-8.767422051469707294077079823822944930652699816112447289328922929746e+84610', 67, 3);
    T('142092654.784234166472', '-5.75350766831736297144766411118E+89393085136', '-8.175311789126631094474522084e+89393085144', 28, 3);
    T('-6.20681045711092852494779001E+34417824801', '-1.55014807689015716646188229388956003287770340100E-1455252', '9.62147529371222318078917921951771015001997546887218841023946895921082401e+34416369549', 75, 6);
    T('6.349688626039414315E+292689259', '9.463187173412897739221243375965463588384281259686E-110617853', '6.00882919611019514165910987385021998570927826428068e+182071407', 51, 2);
    T('4.968086384661628045835562503308476039421059912348043146570E+361968', '-5.0594328923169627213903451649E+73', '-2.5135699666e+362042', 11, 4);
    T('1.010738965920788525930128749542338175079560308214610042E+386063962', '-7.16861939886620622672385977447774891E+478467991610958', '-7.245602958289733944311569070524024151531184197667750739502946897261768561029e+478468377674920', 76, 2);
    T('-8.0810769744232303536488150613834657511E+51', '0.00001787277024591075421113263312046958', '-1.4443123210338601238641828e+47', 26, 6);
    T('-4.32496683592501384570E-660144366494460', '4.1176010651260162658960089963319845403424E+841744127382', '-1.780848805023953344198141292923469082469244575787907021876768e-659302622367077', 73, 1);
    T('-9.079352208496406049583431905916027E+183', '9.2039372470541985926173290479277233565308E-8442348182363', '-8.35657879709038696169746212525679835108921865856306547908090474782868391316e-8442348182179', 79, 1);
    T('-3.3617927600823379808653300061607E-9050309268947', '-8.4093096E+65939', '2.827035613057090157293543592797523365272e-9050309203007', 65, 2);
    T('4.172537320091911161905984321885664638959335544011960856E+42481', '7.51183800756E-433839164462421', '3.134342442902896389852437384055420876364664e-433839164419939', 43, 1);
    T('-1.7134874979165565676046382311705323925405509326131386173280E+5401286695356', '-4.82973291333249780766394557813312598011621374243467339747E+716950124863', '8.2756869652713430187032654116905039155032735418478451577269e+6118236820219', 61, 6);
    T('1.682416439825598776079878427058E+17346107080395', '4.408939307930E+832534820', '7.41767197385472995770747423059163550596994e+17346939615215', 50, 1);
    T('-182070.7016253976173532205271584845159686511209574277', '8.03518536521E+6488821373144312', '-1.462971837133911494603801115085616128993696034261384578486670317e+6488821373144318', 66, 0);
    T('-1.535335439764073310054173269015E-12363981', '-5.49025005199804893956891E+992544311', '8.4293754779991508260318e+980180330', 23, 5);
    T('1.12177236780389E-3207431', '-7.39619387308441680495652630724553E+2583530', '-8.296845913746530123e-623901', 19, 5);
    T('9.050150059367011784302016E-742489168', '-1.94988673963699902295E+227025262289', '-1.764676759248473575763490037142734434e+226282773122', 37, 5);
    T('-9.728136507307033632188E+546831272935', '-6.7234914813414472343385014968599946E+49', '6.54e+546831272985', 3, 6);
    T('-8.032813425883002018E-68536140378551', '-9.850890935345109256073892932097868690061988461E+9633710', '7.9130368962349357215289768936512099243575460018539641281155714298e-68536130744840', 74, 0);
    T('-2.34446910169436801519044984E+386867274163', '-6.42043457941249810174385869462934977587397179E+167737', '1.50525104908826769483265e+386867441901', 24, 0);
    T('-2.0958162E+44242116491', '-5.9697282166345660853383431372E+6710786732907853', '1.251145310601983308162268202810258264e+6710830975024345', 56, 5);
    T('-5.063225502068019462596519566947800828942407823182276200', '-4.0387977287992253838044972E+75454921156396', '2.0449343658150634652173635094123795275094368338991934e+75454921156397', 53, 3);
    T('9.3738581500087787012008198739980478788251167127108070047E-72353', '-5.6128383816640289982289436999094631E+8966998541918489', '-5.2613950808643442217925841356895e+8966998541846137', 32, 2);
    T('1.7534814542477994997887E+341733', '2.9895202385619260661129129131943451821546363627432799360E+40047669903', '5.242068295416794607172011e+40048011636', 25, 1);
    T('6.80664E+67415862', '-1.292E+967385173814293', '-8.79417888e+967385241230155', 49, 3);
    T('5.243869769101067742960579280223599098497865310842418762078E+696', '-7.2771522889455369859979396791981313522942354728E+92', '-3.8160438893146139645370110871e+789', 29, 2);
    T('2.779874355557755721977081066375463E+8745836290', '8.3529011E+60118730', '2.32200155624001688836336546141167779057093e+8805955021', 55, 4);
    T('-7.25210936606748209552573799161E+2579270384507', '7.834892753106424393972217767437362995190938502E+686810425705', '-5.68194991169373409235e+3266080810213', 21, 3);
    T('-7759366036.09616948743215168153644897296071314533314953066965', '8.5001954853480532610798552346154836774E-296933019159342', '-6.5956e-296933019159332', 5, 1);
    T('8.83607359453E+9508265165', '-5E+117248414480124', '-4.418036797265e+117257922745290', 25, 0);
    T('-5.277568049309685411776E+52', '-2.30657128699850180191E+105702755588', '1.217308692771841369943397247920957633329216e+105702755641', 73, 5);
    T('-8.4019520134451898754679E+6623921770804379', '9.145327789586811477267491622956081833561235142E+2843536', '-7.68386052353351584694383827569174378e+6623921773647916', 37, 0);
    T('2.04757255721885641507563303692634378901045701246E+10370954', '8.381803435133989200922675337615E-83989969', '1.71623506937830973574256802702026794710542133892774077442466871827e-73619014', 66, 6);
    T('1.57322928897427162447E+7605508541', '-8.07948411560012619875791812437903857115115504E+223183', '-1.27108810504645083465673622357800132807306904026890350187376278288e+7605731725', 75, 6);
    T('-6.60195808067805496E+254065473', '-7.44E+197609160', '4.91185681202447289e+451674634', 18, 3);
    T('-5.2038641562210739167527837406038269E-8082936', '8.871084570525913E+609640564140', '-4.6163919023365618141725777992598698050913113e+609632481205', 44, 2);
    T('-1.24402674708148720E+652998', '9.13084247549192458092644113018225847707083932618009282E-4463679490816', '-1.135901226289969294814816436217850762400699806848e-4463678837817', 49, 5);
    T('5.41146620E+242', '9.368773E-962137881249662', '5.06987984249726e-962137881249419', 51, 1);
    T('-3.1388E+28643912308', '4.7237813100417264415700931927339243634E+110', '-1.482700477595897095480020851335324e+28643912419', 34, 6);
    T('3.3681479026954274265923472926435400148693E-75480413', '5.770675566758509160159850681159530992816E+89721365060772', '1.9436488807313419627530388293921760707515865833574029537718901363e+89721289580360', 65, 4);
    T('427.2666504786054307483989385131812748311246554194', '-3.95069175678628848808009E-8002628609203', '-1.6879988339955147782464e-8002628609200', 24, 4);
    T('6.75775013558150976917036708475E-667', '-9.37807283670436825498489570207006533404763126864506445048265E+78', '-6.3374672983732218500748740188634589606011278152377833022564e-588', 59, 0);
    T('-2.65984258371955248950E-761426982722998', '-4.23338541995083822195120636767E+8255504741713', '1.12601388132827202872284526552e-753171477981284', 30, 2);
    T('3.730001635972131336140184675178719384086943126727117E+641621461171', '9.060538984918253062365977678877628400729E-94407866601739', '3.37958252365343581335276676254358772555509494484418336602e-93766245140567', 58, 0);
    T('-5.80755200545591568271649023108983514270E+633846262', '-7.483335288109E-730564604332288', '4.34598588599564455254196178552379354566582176281543e-730563970486025', 75, 2);
    T('-2.2819168038332972985729797383681199805920744021930E+88490', '-5.21947238303285989434457E+83', '1.191040173798650733e+88574', 19, 3);
    T('5.342384159249701759094575187286259432864055468883081838E+3207460811864', '9.3173782218457037705766E-363226443', '4.9777013818126641298279355384533281789053739804067620709254e+3207097585422', 59, 1);
    T('4.1674683632553408343677049850038E+419589', '-4.43219435E+283962388569', '-1.847102973342406925340882785700067708853e+283962808159', 62, 0);
    T('-9.3266687291510290944530572742974736662697788715797506121458E+68', '1.0419440826091238816032122132705861003852897177888E+3171', '-9.7178672927944723079029552174712220056548153580482e+3239', 50, 6);
    T('7.3314914277342158684373425777948882', '8.403250561693419745932734193610628496907508455174713700513E+356', '6.16083594581580413784686798502817652954967805215045265242887168e+357', 63, 5);
    T('-6.9090341473837868601473443E+7993526026', '4.926532477725774496475891480753488876954E+31975', '-3.403758111680263132921395681722059e+7993558002', 34, 0);
    T('4.609944813921928869610273581814E-315932415014', '2.63688067705595E+38931240258731', '1.21558744021250212351827148488236100149804933e+38615307843718', 45, 3);
    T('-4.09929980202433909172402E-248040449', '-9.9891990621439761E+745777837', '4.094872173782851e+497737389', 16, 5);
    T('-7.0746725551645317143754850685938579564547517023761775E+57', '8.50496361500704895606750125948499423125876517588159E+6909441', '-6.016983266976329e+6909499', 16, 1);
    T('6.8485894850369E+58332180063304', '-6.06820006112397725903754E+549333575', '-4.1558611131713944519879092425125385226e+58332729396880', 58, 5);
    T('-1.27583007259712759953601034E+2328108', '-5182400812695473802247598313991353.25766590', '6.611862805088679413e+2328141', 19, 2);
    T('-5.40E-9786709170524', '-1.762002509367228481E+52723283993735', '9.5148135505830337974e+42936574823211', 44, 2);
    T('4.2264058944624E-89968682089', '9.35661025299553425E+602516219460876', '3.95448327254476536908640045372e+602426250778788', 36, 6);
    T('-8.63652792675292496400072015493735652374215E+960', '298.9378363299900141270215615346216', '-2.5817849718270538679663e+963', 23, 1);
    T('-5.16369701293465083E+595053482', '1.5015E-6163461573580706', '-7.7532910649213782e-6163460978527224', 17, 5);
    T('9.0312025961109973093246314955E+375328411182363', '7.920678911000900701156633416752E-39978', '7.15332559439929614177694603e+375328411142386', 27, 2);
    T('8.1120E+33', '-8.59840452675843180803928537169E-75524347190471', '-6.975025752106439882681468293514928e-75524347190437', 50, 0);
    T('6.663789E+2245', '-6.05E-2505633660', '-4.031592345e-2505631414', 78, 3);
    T('-3.1349016186538071528096411749258407998231E+643489351985270', '2.194739397386905218409306891E+5625217941440519', '-6.8802920893914904577686596861529422990382829132940339202723464e+6268707293425789', 62, 6);
    T('-6.6541111620491797026893707017E+12821468397536', '8.05554224123672636E+51056953', '-5.36024735437719667288413194e+12821519454490', 27, 5);
    T('6.5363239335581951E+12924581', '5.425649414280102662515697847852221892E-77775696530120', '3.54638021216550379160674011648262868116577970202271292e-77775683605538', 72, 3);
    T('-2.755653327095668898909058498E-826742522408736', '-7194.141252488458187366278', '1.9824559278016022405338079132198311900764635254530444e-826742522408732', 72, 0);
    T('-4.3465234253006777195657410191012048738737652699875943E-371725254', '-5.521979307421157290001152446328E+51293642903473', '2.40014124137316726472441309320946862650788033834542928159325e+51293271178220', 60, 6);
    T('-8.5449669558198173725578449884E+6902761', '-7.0752171336226390001205097E-73', '6.045749661205565561668675556887366974855412173219858748e+6902689', 65, 0);
    T('-2.7030731615323046736025855872954868286625781377517721410938E-9111250792855', '-5.497970620159920192056740079E+1867903', '1.48614168262474012555e-9111248924951', 21, 4);
    T('-8.1785048350825384570903597E+31073034621', '-4.8875089223618350203018471588241248877276924649932991E+935462241303705', '3.99725154e+935493314338327', 9, 0);
    T('-8.591797550847131429011689358E+712', '7.7646978155689485725337012913E+151707277', '-6.671271167487336e+151707990', 16, 6);
    T('-3.786136520675325E+533374808', '-5.908907909368377982E+6491771', '2.237194e+539866580', 7, 0);
    T('1.11152026367297627358E+5271859153323769', '-6.70045816511141475025298065157956406882155E-65360461932548', '-7.447695026414386514738636634524725700389734723984039380999649e+5206498691391221', 61, 2);
    T('4.0476134149221747747811127E+4133539496', '7.1957126242395218E+19451222', '2.9e+4152990719', 2, 5);
    T('-4', '-4.492791054721567833254868173101589283107812078054079516511E+595697657453136', '1.7971164218886271333019472692406357132431248312216318e+595697657453137', 53, 4);
    T('-8.1833138966154343336763227070394948333718634E-2889926', '-7.26461597167343200358862980854126393087142229279604666940E+4096', '5.944863283456963257891916643834949225746651e-2885829', 43, 1);
    T('1.87629E+2707', '-3.302277832830568899059615676162E+692034', '-6.19603087496166811961656629e+694741', 27, 1);
    T('3.238819493539507568597545955876627786982314999484007083613E+5059', '-7.2000540534974545658423463999397647980687979E+42240', '-2.33196754230057e+47300', 16, 5);
    T('-5.1207661044869238126930714934519976330809E-6471', '-4.931021990911616122564E+1842406', '2.52506102e+1835936', 9, 3);
    T('-9.92950801382759547038526E+4523748115743006', '3.773906137675105990763513816010757E+18966', '-3.747303123747811375110966356e+4523748115761973', 28, 3);
    T('3.30011658137446506136135643058838164860806422E-2813199309', '2.82706967992797285532261595804664525956547865513775722516820E+732258339', '9.33e-2080940970', 4, 2);
    T('-1.929E+99180', '7.135509E+434900184146', '-1.3764396861e+434900283327', 73, 5);
    T('-2.505230995470204777648180322888457002571325E+9595453', '-4.0182416166693979614E-6279800496549331', '1.0066623445368480846791792492700947448319844000373999513096855e-6279800486953877', 66, 2);
    T('-4.0033180360814258627856136746707377444381714080228E+646', '7.9989290306946399698696745595268967296583E+4768', '-3.2022256857915169497590968e+5415', 26, 2);
    T('-2.7688650672678094961730823507487205E+2419', '3.08874903817707423349271540191156633458E+700201', '-8.55232931336554652910307602895260892722499613177626818712692156480990489e+702620', 79, 5);
    T('-9.14795609032190691582833529217E+9583179', '-6.739020923E+5275086', '6.16482674953646085110255e+14858266', 24, 1);
    T('2.9445877415108909601303634554545752706715467323E+678970511', '-3.720615E+53129100', '-1.095568e+732099612', 7, 5);
    T('-6.9940577885327045620612E+8321', '-9.819030158240438935081454657604246473E+44338', '6.86748643540790564707560627216704810668860968357848897102e+52660', 57, 2);
    T('3.4114717278633912E+861817984', '2.63116007105618930753113E+932413810418', '8.976128193891221302143446721017981368056e+933275628402', 53, 3);
    T('5.5392052E-546074280153217', '1.7756744831604959685056E-7794755347', '9.83582533062993170332525574912e-546082074908564', 46, 1);
    T('8421006.605158396930', '2.426E+70', '2.042936202411427095218e+77', 66, 2);
    T('2.2907855871234401787943463E+83311', '6.5661E+120', '1.504152724361122055798155724043e+83432', 77, 0);
    T('9.56048263817560268304E-89201793577761', '-6.7809162339924425324894379E+2674241090', '-6.482883e-89199119336670', 7, 1);
    T('2.1438360461656451417E-8', '9.7002274488E-187', '2.079569726074285499507870549496e-194', 78, 3);
    T('-5.361061165061E-335789625388508', '4.2864050066000468833594E+1014670305591168', '-2.29796794186065e+678880680202661', 15, 1);
    T('-9.4484930181768136097285318136818E+828', '-4.1637041141307343684283633813975182599995540742E+6563', '3.934072925211831837356902499370551324302849424765287803494486424633615849238956e+7392', 80, 0);
    T('-6.61096582633037919836792620672350E+7066013855', '5.43727675360051599247853907531347043904440034811135655351E+83', '-3.59456508063535968174011935621423315018824e+7066013939', 42, 4);
    T('-1.95938872E+749577010377', '-3.025581652558309472768098407585844788382438620763701754331E-92380', '5.9282905614617107232109591956736667e+749576917997', 35, 4);
    T('8.4248026300135323813288994190E+7833354068719403', '-1.83547687532495709E-94984060', '-1.5463530406566718969466420796e+7833353973735344', 30, 2);
    T('-5.55342503900848693138073653061797752983625E+100460', '-8.4890094038595485977472325901935029099E+224956533', '4.71430773797721260622678035707865226360184126949066e+225056994', 52, 4);
    T('3.5191210990946E+4876170325059721', '-2.772910122666115026477560025272232726090961848561E-79609474286618', '-9.7582066e+4796560850773103', 8, 3);
    T('-3.134755465018764942612258E+506908703542701', '-1.47911588126371E+54692', '4.636666592187e+506908703597393', 13, 1);
    T('-6.9906884694366467390718481918758707966241167920535371272758E+272804990', '4.264559819712043229968707515818327626507E+1602561818002', '-2.9812209158883805647025851369250481099842923132768736208550407163722207105759e+1602834622993', 77, 3);
    T('-3.63111202228365075221384822067373073158273680668E-372852601105581', '9.661521425540517689539976E-4053236', '-3.5082066601831249450477049518152e-372852605158816', 32, 5);
    T('5.4244593758697325973746519411822788441300482272967067691493E+317725011', '6.5439757812538155808305843169636629680454347363E-6284010413877212', '3.549753078208671823534423155565292239881e-6284010096152200', 40, 6);
    T('-2.2813766356443958181766654323438041E+3016395', '-1.4782502998615510006304253953843961140822438347685468267E-77525', '3.37244569573846449914830793927795858823376395257901017974865e+2938870', 60, 5);
    T('3.7744922777396077646381939324338508614039372049553233826E+61932470545', '-73.9260399215005097066357467150202184436066058286681', '-2.79033266807574e+61932470547', 15, 4);
    T('-8.646037717991600797574178910949160285198522317793E-30923582275', '-3.828915E+7827', '3.31049435089838e-30923574447', 15, 1);
    T('1.031040444100854726410903374494405766036786086074844E+53357765', '-8.4970E+6903192902', '-8.760750653524962610313446e+6956550667', 25, 6);
    T('-2.505157724039602633811318076567E+7921680528766', '5.1491923761957858400992728E-2599854019', '-1.289954e+7919080674748', 7, 4);
    T('6.936272648444578164303206679892252147116137E+356', '2.55087235271889459663599E+821676594652661', '1.769354612983753917138e+821676594653018', 22, 3);
    T('-5.934109209333060084238784700794935120827E+4330232', '-5.61300798112715172119730934E+2187731767848252', '3.3308202352866598139724333859e+2187731772178485', 29, 5);
    T('9.062175848725942E+170605', '-1.82356518546796938653242244682994031114317179339E+1516369131762', '-1.6525468382325275310106483798e+1516369302368', 29, 1);
    T('-6581.4820933940164135943', '6.35301898032607646504041235449424178E+746', '-4.181228065e+750', 10, 1);
    T('6.60742937033374121177800026766943E+533138440', '3.3534068537E-242819', '2.2157398935815843235938489260483101087972391e+532895622', 79, 3);
    T('9.38967294E+884', '2.348593106749325060347655155437087226525530E+3817644767411883', '2.20525211415146688824102446054591118033264192601582e+3817644767412768', 54, 2);
    T('7.2795953191290787026788409871736355085185E+85495327888813', '9.691742715954900772783062E-948', '7.0551964909268639857133626212708128733543938043956589594853513647e+85495327887866', 78, 4);
    T('9.06E+258', '-7.548120584921832634177836087036423697069836384242301E+4879567374', '-6.8385972499391804e+4879567633', 17, 0);
    T('-1.0293125501511886713397339455812212707825E+180', '-7032.33359556674240167160205165024177474034588827', '7.2384692267666814893892306849426855378048e+183', 41, 5);
    T('-8.244166335593420125782091805329149417310874454648668350E+333996069', '4.3091192467195965356324E+659765954', '-3.5525095829863375029e+993762024', 20, 1);
    T('6.470135604567510180147061044781782903541758508042562E+43457728922434', '1.894812673754148958496790538947116E+796821', '2e+43457729719256', 1, 0);
    T('5.9165626550446404508597713233418390572856847E+78960', '9.60983107581508553813150309851980E+18375100', '5.685716766445499597245005642807478190767211362757708831632142355107829950706e+18454061', 76, 0);
    T('5.135479931E-708423186461', '-1.03075090389938433738829E+5954477480835185', '-5.2934005808353979079132962494079e+5953769057648724', 32, 1);
    T('-1.9225423449001561282252E-133411797', '-4.62729961810138648109272761673141281563E+6069081290691', '8.896179458340236503378014873872751148970638873e+6068947878894', 46, 0);
    T('-7.2397795460374586E+85171602', '-6.46427871483525783958294795E+307782739', '4.679995281954960929731630372810243251407987e+392954342', 64, 6);
    T('2.96853975117071832638932331586656071326086E-998397437946', '-6.345782364685813327528457268458564751542842384653749E+9816076154328', '-1.8837707201847956833463852584335301373881029919450323339043805525723811e+8817678716383', 71, 1);
    T('-7.389521610411745849370382649966286485E+639287300', '-7.16477513839736354102403064032589995851992808E+190491', '5.2944260718928125078330419298723143518821e+639477792', 41, 1);
    T('-5E+5775', '9.506296830081545228590686292E-95541503228', '-4.753148415040772614295343146e-95541497452', 68, 6);
    T('-8.20095907952175078670292511060375661815306', '6.517502716796725210725760908471748278968236970646443556E+68246648', '-5.3449773081121781e+68246649', 17, 1);
    T('9.81085380E-5397128691856', '-3.09292211E-9235995251', '-3.0344206635997518e-5406364687106', 75, 3);
    T('-1.094149157861016643431251E+51001988', '4.179933328598E+668780118', '-4.573470531400697856e+719782106', 19, 4);
    T('0.008866196', '6.28553638997910907489389713346062489721414868E-70250', '5.572879759868721696338797118910005862118049617002128e-70252', 74, 1);
    T('3.337117363406516834644623707290238804445E+62', '6.1789056796E+44107008037', '2.0619733430244304073238679472580464474425524225822e+44107008100', 62, 0);
    T('-1.048430729868105120892703694888666E+58041981894', '4.161172709344518802066292515509905E+174312', '-4.362701340765314297705512637075041578182e+58042156206', 40, 0);
    T('4.198366411574924809288508285800116050924585987463700E-892726601378124', '3.8521082782888606941180943491553505840E+319769989916757', '1.61e-572956611461366', 3, 1);
    T('-8.317299535816446252057523318491441289178877359570E+7043193626211023', '-4.36482666728466922698E-67919108811630', '3.6303570813726025447574861976269710547841106e+6975274517399394', 44, 0);
    T('-9.7720275480468519681210252994647349062281E-7020975', '1.577566342984271086033189320565809269916438788911599E-6890804', '-1.5416021762513825671426505615379801472481921868034079e-13911778', 54, 6);
    T('9.1798020498434778397491195650550798234747882513242946915534E-14235874', '-352740904339.47598581363354345033768467159', '-3.238091676719e-14235862', 13, 6);
    T('-6.656873809197803692430741240159738460907919072239952632E+276474060827274', '8.75241415547839967049278050206900671E+254203215576', '-5.8263716558856272469374335856606512354807577107050601192192260622e+276728264042851', 65, 0);
    T('-4.3480980800165879E+566054', '4.0387035675170785608915530E-33729767', '-1.75606792276771532687293412349784529920087e-33163712', 56, 0);
    T('-2.641190485727180117183597704529E+4367986089512', '-5.34842688542190923894766462676173742270165657304853311868E+10471033640', '1.41262142033838015817311716789201723930287684736757487325641038401374926e+4378457123153', 72, 0);
    T('-1.082823311917087156744195068127602E+430582915', '6.22356636487189812410592388380011389520642728E+55690740463464', '-6.73903e+55691171046379', 6, 3);
    T('5.646985341039E-8174231593780', '5.189E+64006355', '2.93022069347e-8174167587424', 12, 4);
    T('8.333330578E+32829345236861', '1.361178147832567261355382249075759095840475083152469888E+92429023872521', '1.13431474814385371832945246e+125258369109383', 27, 3);
    T('3.78646552181033592690382395225837937278792807216315601788435E-98632857044056', '-3.203E-81508613662774', '-1.2128049066358505973872948119083589131039733615138589e-180141470706829', 53, 0);
    T('3413139.962023893463', '5416149.686678755', '1.848607693591e+13', 13, 2);
    T('-9.6111721699623E+5707855977521524', '5.917668090301503794973655987405567740621464339119049871824E+61865', '-5.6875726860579764096395821616235756340985606656967025239e+5707855977583390', 56, 2);
    T('-1.95769822369670800754861955593753E+170125', '2.13685442720341682245E+5080997', '-4.18331611643457556323701075126489592e+5251122', 36, 6);
    T('9.035686812641299295239544448E-3327', '8.465729256988829E+696550', '7.649367e+693224', 7, 1);
    T('-6.38929736881135586E-3002', '5.081798498356020627529967083266394647790772E+8192022097210468', '-3.246912177437562191341128472866792828459416379284577491612392e+8192022097207467', 72, 5);
    T('9.3531E+3559078544565', '3.312449661009288411327275448331071789074717246980E-592697', '3.098167292438597e+3559077951869', 16, 1);
    T('2.942275761177111701E+375', '4.2340563887188154828009588E+6285535', '1.2457761484e+6285911', 12, 6);
    T('1.080182294904231236032966447051342481633E-77500592', '-9.9978692645570E-74456', '-1.07995213663416588365443813e-77575047', 27, 5);
    T('6.65248E-585', '-6.9305E+28137817937630', '-4.610501264e+28137817937046', 44, 4);
    T('-7.50711508167209433413884399296691793636826168036802194E-784454596686', '-2.35709720E-3648', '1.7694999939087064873134533587059141960443407775662759484312568e-784454600333', 79, 1);
    T('923712.5724530624933', '-33098.11787323112944666397969928222365', '-3.0573147604037012338826310342060401371478116267237226545e+10', 78, 3);
    T('3.15697250588490734261866816445534E+48', '-8.96350E-43', '-2.82975230564993669655624320921e+6', 30, 0);
    T('-7.63616600475687826635E+792673094404', '2.253033484E-25965042894487', '-1.72045376980997500133984204634e-25172369800082', 64, 0);
    T('-3.12302625489233895449E+55841415643776', '-4.113223055791598936074972645879822350312738515724430769E+7032186201', '1.2845703595465659391189846e+55848447829978', 26, 4);
    T('-7E+979', '3.119273979902731E-542335114738734', '-2.2e-542335114737754', 2, 0);
    T('5.56501051265860032961E+21073367', '-5.21412535193078056109818E+730512232050', '-2.901666239782e+730533305418', 13, 3);
    T('7.14052432550141352911313583E+9529078', '-3.1416738774879451203429716409E-39290377671976', '-2.24332e-39290368142897', 6, 3);
    T('-6.2171157405131151767540959288950588E+291223357159042', '-7.598393689908364761855126E-676131199536', '4.72400930121448248398353932106365327620627693162352628514088e+290547225959507', 79, 5);
    T('3.8E+30582939506', '3.1156208228462327108316512578143493513379303383801240869E-1711544245', '1.183935912681568430116027477969452753508413528584447153022e+28871395262', 68, 5);
    T('-8.3942915349575539535482857927324561462054356310903239997568E-8155034742', '-8.744214870135499E+4606339', '7.340148886422838620813942349497272e-8150428402', 34, 5);
    T('-5.23765781520191594350508012677690023530140535E+68228842497601', '6.47838373921099792487773742411334E+8162537624576685', '-3.393155722155549e+8230766467074287', 16, 6);
    T('-6.537984483473307614890825812713182746951E+4135033162895495', '-8.759E-858566', '5.7266206090742701398828743293554767680543809e+4135033162036930', 55, 5);
    T('-6.8544', '7.9116056635818894536539965820160E+474793', '-5.42293098604557030711259541717704704e+474794', 56, 1);
    T('9.91340189159415957740597261631620543319864300086029902E+11209', '4.711725344861675886663325207583823400675E+174', '4.670922694642388160803333372715729355965e+11384', 40, 4);
    T('-3.8886286792E+61526966720267', '-5.995860773901259468951892003776E-1190082', '2.33156761618827444399669e+61526965530186', 24, 5);
    T('4.190019874686109E-90823891', '9.5364527851064E+484', '3.99579267036015132885301669976e-90823406', 40, 2);
    T('-7.669441575078055464738E+53553109223', '-1.211629873614805603783545094E+99942', '9.2925245263079599660759e+53553209165', 23, 5);
    T('7.6407797214477213571820508553618453566498923939E+30251', '-8.340634619402896313000197423E+574615', '-6.37289519e+604867', 9, 4);
    T('-6.488269557214234986292721508617813853822818736E-449577403467', '-5.709757200768E+4954785202045078', '3.70464438248277811754173004108982348187430678419e+4954335624641612', 48, 1);
    T('-32051512.386301010388008473920547856161394290235774448247165', '-1.23E+15078321469861', '3.9423360235150242777250422922273863078514976990002571344013e+15078321469868', 60, 6);
    T('9.46996806311903780817505611838521157057E-331754', '5.915688721938043275300891861176383412619045229104121E+394298', '5.602138326810674790111676632e+62545', 28, 1);
    T('-1.2938953377141702741791836147006E+95995295183706', '-5.538485E+95328153', '7.166219919499866350987e+95995390511859', 22, 1);
    T('-9.8818889114348766578701343475631405525E+1173', '8.1365515813211157756E+5877854', '-8.0405e+5879028', 5, 3);
    T('-2.46950323298957801648412722743580727939848782450220E+512447593263472', '-7.5377278490827884560281658592841216E+737494929404508', '1.8614e+1249942522667981', 5, 1);
    T('-3.4033212133838383008122037302649985169905566992E+33841', '-8.2958052540078051532E+49433', '2.82332900030658643380235829211926404e+83275', 36, 6);
    T('-4.796354729E-32854088', '-4.6781E-542', '2.2e-32854629', 2, 6);
    T('8.241669433405374181889E-748279', '-5.0549151769310633172133224439269718204340386125847107E+156896', '-4.166093990216966339403134180445198921372204e-591382', 43, 3);
    T('9.80111161657554337298506774941151442790727023032381881858E-65937', '5.4503539450550763941103443746392651903E-836007850', '5.34195273653277e-836073786', 15, 0);
    T('29.07159420306055396257887921966306', '8.55999934105816360655324562515677834584686432360688E+5391327607', '2.48852827e+5391327609', 9, 1);
    T('7.38E+917', '9.5E-4351680', '7.011e-4350762', 19, 1);
    T('-1.36238880529076E+554938', '6.9779174424919026495705363451523254444775513897050830E-16661271591', '-9.5066366078940987485036823889e-16660716653', 29, 5);
    T('-8.00680851622185957434198160229319773727517E+6525676158970056', '1.80E+25995142', '-1.4412255329199347233815566884127755927095306e+6525676184965199', 63, 3);
    T('8.8363296091251234641470E-71357391', '3.1358420878766824640560274089032005776197550373061258178951E+29693906510', '2.770933429065e+29622549120', 13, 0);
    T('-2.214255844133251505197200533426703116746980121120E+6773', '4.3040244939794753640243207679632488029E+39504821888265', '-9.530211389086713883073264076054203736963539031371842542529866401e+39504821895038', 64, 6);
    T('1.4390844306196485882563033510338799791118980E+98528984679247', '-1.137296031927399765851013258191291E+95821329', '-1.6366650125522277740351478471082949380166e+98529080500576', 41, 1);
    T('-8.11640441971922637681322145143002366784092969E+4941', '-4.60394091618707935275197357732912942007E+4612', '3.736744640026699523359115448906934144334803972574908993920141200797766e+9554', 71, 3);
    T('-2.7354923746351610552380152110149128022566436598299391906536E+1109554', '-5.49660441589283612753668032730596039145905E+36508', '1.5035919466060806689132708462243825962492955188975864e+1146063', 53, 4);
    T('-3.29632573542566205E+4672292', '-9.9672412232337E+35', '3.28552737553e+4672328', 12, 5);
    T('-9.869825483158818454456841E+837', '-5.37175946965471007199707481202426128684178364046E-504023753323180', '5.3018328502997757216257954617e-504023753322342', 29, 4);
    T('2.2283661771725487323891839016915592118485162040200825673E+2786316210', '-4.931995161404386260979462221503229992655594935460974E+83', '-1.09902912036521996766600438146565574664682756485600389615130222090685954239708e+2786316294', 78, 5);
    T('-7.4401102121739692965E-5939373170939', '9.4685378103206253560340822488568952E+4920395', '-7.04469648569218385674e-5939368250543', 21, 6);
    T('-8.379E+95768121', '-7.828351081617374613592970414377856605340994190037061846E+6310498250', '6.559375371287198188729549911e+6406266372', 28, 0);
    T('5.49531179318332976733190879535405E+6550697012625', '-21789479530979381155649150381379026357781575189.3589546993728', '-1.1973998383391776231718e+6550697012672', 23, 4);
    T('4.152285115438322196547718872E+114871', '1463525201.9013907566858608573379301813693093571825653766', '6.0769739119240101179856481135390762306310439999e+114880', 47, 3);
    T('8.0319751963760913E+41371452693', '-9.47422356178405E+39715253565071', '-7.60967286531714362e+39756625017765', 18, 5);
    T('-3.8604986581E+356967644438785', '1.1263673203586271074675111386405093747501546754E+9591252699282', '-4.34833952877217275913661124e+366558897138067', 28, 2);
    T('7.84612581738271909103510862094374118427E-185309816', '-5.64792998543858656678336649836173406666996248648865E+799197286905356', '-4.5e+799197101595541', 2, 3);
    T('2.6399E+98965', '5.00309106409463422562019412E+112138773260623', '1.3207660100103424892214750457388e+112138773359589', 63, 6);
    T('-4.3798978539806710222417383611972863400805E+56561746080', '1.50205301539772525283616420242904608456951495246E+64202', '-6.5788387787056926420523705455221742853636035387900438841200883793042616084139e+56561810282', 77, 1);
    T('-5.965407301072281020684619043802011336175494263370E-904050837797', '8.1262104538472909977332820986559656233330175938624896932647E-184186933032974', '-4.84761551714305240428517991e-185090983870770', 27, 3);
    T('6.47966083791927017737034E+763715377', '5.025E-4139310244', '3.25602957105443326412859585e-3375594866', 47, 5);
    T('-6.4610130728003249793905457762034131451930180777385E+78557716118116', '-1.544171954028804012E-36616576160207', '9.976915181631725173341606234676736211106300858933880881056396686862e+41941139957909', 78, 5);
    T('1.9302643656868114632210613707978527E+3782876992586512', '2.47710308573406329666717693533157454751776821455934429791E-659357', '4.781463816525305043073493148878997496608698744e+3782876991927155', 46, 1);
    T('9.424467058015729017178861E+35112', '-4.98934208917942841620472468520325E+2754196915309955', '-4.702189016064289880722245864067893859554317482962538849825e+2754196915345068', 64, 3);
    T('-8.82375E+354232', '7.6106011238052204009603480044863953967792E+2923787568', '-6.715404166618e+2924141801', 13, 3);
    T('6.778343642386536427160840256218916224609341408735078381E+561592827177', '-6.95588575056466086E+18048773860', '-4.714938395450707007602951580764147566460311080556112111942093162078286766e+579641601038', 73, 5);
    T('3.59167068718271777904423396990949E+55047783949431', '-1.621789345240968652142E+651106122', '-5.82493325208723980662435335457893032911002959104e+55048435055553', 48, 0);
    T('-2.4250560060393979774390209345037484382048089785273047E-76', '-3.1594234379070153E+293364135563', '7.661778783718050416e+293364135487', 19, 6);
    T('9.827802093765139121551330658571E-82706082', '6.6421625713E+364097723', '6.527785922535e+281391642', 13, 3);
    T('-5.379340284727E+587617656670997', '3.19983162912941233698E-7414914521878', '-1.721298318691947322794017967130446e+580202742149120', 57, 5);
    T('-6.5633535178964239397350452861222726314251271951327862202E-919175259868478', '-0.0087364159001874792462', '5.7340186032301745230321231194796e-919175259868480', 32, 6);
    T('9.276087023079537430817093730641839226307670944586779006E-834975386110766', '4.7517E+635525481585483', '4.40771827075670380100135842798908274516461600273929978028102e-199449904525282', 61, 3);
    T('1.50901695163925935777E+82961893574191', '-63811431134345311054751473412.25', '-9.62925312900882871543138286055823692502261654506825e+82961893574219', 80, 0);
    T('5.05982940538293363854277814627523E-119717', '-4.56270378503180667808041E-7011', '-2.30865027795559470520217630518486158003447879307279312443e-126727', 63, 5);
    T('1.917034991993924138378E-8018791', '-7.466921750056409E-94', '-1.4314350277338646043568083188221764602e-8018884', 74, 4);
    T('-3.87727E+7714', '2.21682435107E+85996574043284', '-9e+85996574050998', 1, 6);
    T('5.9793676559809628030E+1603206', '2.16588002131030974743910644734E-996873', '1.295059294615822455853487e+606334', 25, 4);
    T('-9.84120431150768517121426875441589953534445257449E+7478644096019', '1.5E-49646749540868', '-1.4761806467261527756821403131623849303016678861735e-42168105444848', 75, 0);
    T('9.684078009876697205783739793660537089E+6121', '-2.5133986E-352287547', '-2.43399481123148769296407635001507e-352281425', 33, 0);
    T('1.65771178820161242312177E+1692293980666732', '-7.555343087127825839721175360468955726469277E-6809', '-1.252458129943935898462894623425228827715311620667477690984033486029e+1692293980659924', 78, 0);
    T('9.153668511263887624742966291864510246426490628E+4095177191416', '-2.4318525659936043242932427475618E-337425973898', '-2.226037225737194112804162839608690239003085139257727865536750837376687755081e+3757751217519', 77, 2);
    T('4.058550946855127792962946777805608608626022698452872243E+748877534474406', '-7E+34', '-2.84098566279e+748877534474441', 12, 1);
    T('-2.511505302769512646921075883570167025290045170466844543238E-620672024937532', '-8.2443085481169922421059046059558143722E-4916987', '2.0705624636263847825238e-620672029854518', 23, 0);
    T('-7.0035695303506288648957038748850909124092792E+5108470275', '-3.6822216129686130632733159390095338E+830726497', '2.57886951e+5939196773', 9, 2);
    T('-5.8631591202379505433224333205E-27245', '-1.0595325919E-2737637', '6.21220817938783948391e-2764882', 21, 4);
    T('-6.0487840192474161526401E+730932501062', '-8.57834483698591690298093372718E+2087502', '5.1888555161553995366400166759146475298163989090127918e+730934588565', 62, 5);
    T('-1.8822265222256631E+939803583717181', '1.875193523160806019261137419E-78070869', '-3.5295389835990523438869449787335546974975389e+939803505646312', 68, 6);
    T('4.32163131237357E+961939828102298', '2.5740852024092288376296829485E+242103', '2e+961939828344402', 1, 2);
    T('-3.843878340948331836681593821459255928087516598E+59439420711588', '-6.15602481801566321556427274428038615421219537212990554704425E+581', '2.3663010464310804e+59439420712170', 17, 0);
    T('-8.30138549423910586098725424070195816687476449581E+27922582', '6.99291724798971920977347127763943201680616E+184509', '-5.8050901804876303208539839780988750900503191169e+28107092', 47, 0);
    T('-241586503958088650483630979520.5972758960344855962', '-1332636284.52', '3.2194694102488353298128688962709696456132475e+38', 44, 5);
    T('7.9E-1965398', '-2.01728638330E+240387863337156', '-1.593656242807e+240387861371759', 53, 3);
    T('3.25934696542542772643563126021330465941590281402E-106433793277', '5.520903667882180645211040169001465402089318274658E+224996797', '1.7994540616317898958847599129497129310611788854945523680847983512e-106208796479', 65, 0);
    T('4.8815920893667129127933740821531288468076593545050636E+2970720420644', '-9.3950500046214073837442736226573949360E+127628193485983', '-4.58628017817645618775663522e+130598913906628', 27, 6);
    T('-7.6541923262167223261199103807792777E+4245427535', '-8.92471674077802296808037086732247152E+412668340', '6.831149839092106004375523601395131436970255803023756748713643836442e+4658095876', 67, 3);
    T('-2.0669291298143931957E+609293508868242', '-2.545248416875381692080049815780558165680015398820E-595997', '5.2608480954536945744230714688782646e+609293508272245', 35, 3);
    T('-1.15566542521900164402E-325067893174', '-2.1645761945652857917635E+41891', '2.5015258683112194400011797046e-325067851283', 29, 6);
    T('-7.92595228266395328250963067E+417061051259', '1.285480254448527150896728735844460712488E+8842341', '-1.0188655157065743257732e+417069893601', 23, 0);
    T('-5.99113108381938569E-15', '0.0000084872987487', '-5.0848519350997946983539686103e-20', 48, 6);
    T('9.70126066193572553352578E+254', '6.7349640059248720E+3534641573', '6.53376413702320094427813673851919552e+3534641828', 36, 1);
    T('5.39315865501753679887131736643824195475986449673E+632968', '-3.6173553739121612616039154546735220993756986263468E+27029419951650', '-1.95089714430885705514133e+27029420584619', 24, 4);
    T('-8.9757914030049716456436041757452487740573058743085546E-296', '6.1764894853142745753705154245040E+9243', '-5.543888122303446780383457759775805620598277735030044783182263368422130197e+8948', 73, 2);
    T('-6.3417876491998168855853904902143526581268372555148E+850', '9.39930145294895008E+12546892', '-5.9608373865417545387868056e+12547743', 26, 1);
    T('-9.7673136738990314751347003E-472205', '5.2975330557838459869811703E+697381', '-5.174266705368967959888355229545402502163535223376109e+225177', 80, 3);
    T('1.402280084993926E+435', '89106943111756401892906.1893008390347', '1.2495289176030269617321784365e+458', 29, 2);
    T('1706389731865462595507.756596644678', '-2.5461157161564897155E+2894080817073120', '-4.344665714190712755704197596466587931411621914809109e+2894080817073141', 80, 3);
    T('-4.1954221445686219417786584177367625134743320050790E-46829960249', '8.6243933088797147514E-526906836', '-3.61829706715434063703850673868992008905781021595554607414611360223606e-47356867084', 78, 6);
    T('-9.09172646046386242830624641763951182921275E+44475560821411', '2E+82', '-1.81834529209277248566124928352790236584255e+44475560821494', 79, 0);
    T('-3.2505899778920964E+5281', '-4.9624196573177467190545727816303505712865569024002E+299608398075277', '1.61307916041718e+299608398080559', 15, 5);
    T('9.679277992613081907732508139749059908340876084830057E+77357559', '1.3E-1472059293868', '1.2583061390397006480052260581673777880843139e-1471981936308', 45, 0);
    T('-4.997474826113380129121240841860010246864607283E+93473268686', '9.6982605460168114788902960816E-63533511728759', '-4.84668129358076199715009068783045e-63440038460072', 33, 4);
    T('-7.6546401237715292696858272651496485712333152226E-295969328171097', '-4.8722466044E+903407591742776', '3.7295294350949829e+607438263571680', 17, 4);
    T('-1.1639634524323442955050790646182779236606375E+145', '8.11101050286867002925561716316379839817587239328E-20257600579', '-9.44092e-20257600434', 7, 4);
    T('1.389622642557E-343861555', '-6.2794507E-142', '-8.7260668755404034399e-343861697', 56, 4);
    T('-8.650612412681866737464925484163192195781E+470903088778130', '-1.141778745453389307658511736060058575E-63758002', '9.8770853879554190603679064702497683979556828236769426509724245e+470903025020128', 62, 1);
    T('2.007658358890765206290026359182679985958E+804', '5.040302746151155902E+3810665', '1.011920593965044679345084966482782925678938281672282882412e+3811470', 58, 2);
    T('-6.06480400763023883792561587395736074044E-398739642013', '9.55467250766829447077389111E+736', '-5.79472161161e-398739641276', 13, 2);
    T('5.7120496E+81296737247', '-5.41E+473', '-3.0902188336e+81296737721', 34, 2);
    T('-7.17350430125014331392289500631949704E-945652680447', '-3.1959533581868612335233826288323462814239108927095476092563E+8602', '2.2926e-945652671844', 5, 4);
    T('1.17759138770131605338059103667878475894813144541E-189818518724', '6E+802', '7.06554832620789632e-189818517922', 18, 6);
    T('-2.8022154095974067047380904589E-609428229', '-4.47E+120843855981', '1.2525902880900407970179264351e+120234427753', 29, 1);
    T('-8.6110003325087647722228003034214776497E-58858190289', '-7E+900', '6.027700232756135e-58858189388', 16, 6);
    T('2.0757217610437013504E+52795', '6.59434531E+1343941861', '1.3688026059803472705e+1343994657', 21, 1);
    T('-3.584268187E-57', '4.23498815635129297991513709815337135115E+246070323', '-1.5179333321132e+246070267', 14, 4);
    T('-8.10767011101118537790872252239144297943542586150973304732E+278333878118', '9.0951979979042895533601520586037525862605E+95346', '-7.374086496133738227722886041883344827401848041e+278333973465', 46, 6);
    T('-3.114050113761084933933594759348218872E+3997093548877', '7.5E+98249', '-2.335537585320813700450196069511164154e+3997093647127', 67, 5);
    T('2.191198914206849507076808867904675687E+9383163680812', '-9.7417588183140342238608688105334746650888772862606057039E+159921854933862', '-2.13461313451547131119122e+169305018614675', 24, 0);
    T('1.1229381370223E+706074885498', '-4.0154723613982850562418E+503460', '-4.509127052773125969710289e+706075388958', 25, 3);
    T('-2.92603E+8779480', '-0.00924730', '2.7057877219e+8779478', 29, 2);
    T('-6.7363129493677794333299084348675406030013853174948055715E+6134008373', '-7.9328602027732347880393810103258915105183017670778005E+659', '5.3438228e+6134009033', 8, 3);
    T('-3.63723174406895187183668684917876922209964129481649000E+7046399494276452', '2.77130729743022916089629025899064896E+63768', '-1.0079886874783165954798955e+7046399494340221', 26, 0);
    T('-76.81024590650809508081980974292285096947', '6.33203635823763975946213716484303738877394425078363E+80', '-4.863652697651830952345221732595153999812331688182604746458568637347072028e+82', 73, 4);
    T('3.43251140507165502660704888155415265102541965621548897E+673221791349630', '-7.34370128483908504E+26026', '-2.52073384156495261005242448957867714139281996853402952229291972910120088e+673221791375657', 76, 1);
    T('-2.918559027778786496073626086402396358155731732778E+41610', '1.2505587472000852009196438901804E+12551', '-3.6498295214085379031793132882096945390155481668081137621883953e+54161', 62, 5);
    T('8.5631621153366762398496962E+4924', '4.4526967542581773E-19026028266955', '3.8129164157146205984e-19026028262030', 20, 3);
    T('6.44313978461632627191860233671549E+82453161010413', '-0.02', '-1.288627956923265254383720467343098e+82453161010412', 79, 0);
    T('-3.65455116304722071674521033877569879967644612283578728010303E-736679', '4.851248237693882434371E+3168', '-1.772913488929495790740865288354069153399749e-733510', 43, 6);
    T('4.048491544200380055360756125269021535992144848350641606E+348820', '5.706751135875E-558052983256784', '2.31037337184458516495366e-558052982907963', 24, 3);
    T('-6.33552434365E-290791647', '-6.4846208184632346243018489469214592008297461858606236E+162705920885420', '4.108347305471341034478592588892602149143713123957071824376970014e+162705630093774', 67, 6);
    T('-1.023837857931165974195356212445546081306278E+47064941', '-2.585304488173983609520088688666E+8083026839665250', '2.6469326092718807942463266235006872e+8083026886730191', 36, 4);
    T('9.1347278311533216395172419198564834871091231822671966E+323788', '3.04840694507197488385798510496027721135857282880693260E+18140986390330', '2.78463677618300440204447112127969074040918e+18140986714119', 42, 0);
    T('1.924080606033055227426838346012629530E-42', '-67437028051313349468004655927', '-1.3e-13', 3, 5);
    T('3.807852E+403', '7.504958285102363106067004985588503052470615E+803813201607', '2.85777704158436035581634570683831525253e+803813202011', 39, 3);
    T('6.37941785606846582E+400251', '8.0517225690495727579540712647880211530587925149E+5828937670', '5.13653e+5829337922', 6, 3);
    T('6.139202136E+464', '-4.27869092287956291452139075563223925068E+952551', '-2.6267748453026023915576107e+953016', 26, 2);
    T('-1.483834E+4895747', '-0.617656347', '9.16499487994398e+4895746', 67, 1);
    T('3.745324669173093495005207247E+34', '2.729374919802624187002272399E+2744', '1.02223952185591020225638734350952852888748395269428756e+2779', 54, 6);
    T('-5.496489124718684100811866227656740E+66', '8.2119590629944588035386553166988559E-470', '-4.5136943682384078101008299818173180934852e-403', 42, 6);
    T('-8.64661198477040998343496E+8613', '2.935722588453243995409691878838063150703113227010491117927E-1922494991510', '-2.538405411728102954e-1922494982896', 19, 2);
    T('-2.1557627849640057923272768155866270E+41655410', '1.14221569739695595171341889140123360630E+5882903441', '-2.46234609285006586396889505626305925807361e+5924558851', 42, 0);
    T('803549788.2', '9.391361959346441492625216851468814071808177311940767551056E+71573', '7.54642691334237007182268486297683640630664e+71582', 43, 3);
    T('-5.62116E+87633', '-2.4802160889465418873659354783027992664688131478413798515233E+314486657', '1.3941691470542743395585901873216563124703833714120050766188713028e+314574291', 76, 2);
    T('-3.10085143473901985125269675077488606186289738E+64560376', '6.920870293032074471256237024082354097E-94', '-2.14605905777911688666854658071219870948122101246e+64560283', 48, 0);
    T('9.302376398291698017699645628796308766157514988667375720125E+524915505', '-8.9317982437429360561876902980E+56601', '-8.308694917689752739049656279012182925037481566167589840249487585691727e+524972107', 70, 6);
    T('7.019567997E+7359585', '-7.52151249766581332553150685980605501E+93046178099170', '-5.2797768417650480420877108568280549375018e+93046185458756', 41, 4);
    T('-7.38162447835518796E+85', '2.629808517886836375149221749252876097E+901', '-1.9412258929020448543441135089878055178221063e+987', 45, 0);
    T('-6.64828355793E-641420668', '-3.32298935726223798377668145813E+8130904339', '2.2092175507062915426818e+7489483672', 23, 2);
    T('-644891014.85493780574101012554741923', '8.68508920599152489951768706413406282666287344541773728455E-87326', '-5.60093599215754047629153551084291120587140226925303529e-87317', 54, 5);
    T('1.02197500373742768986614E+7425736990777964', '9.3383911093763390359059182057055541558998634014194756279455E+687485433174828', '9.54360228890644559720286186799749990200845077771e+8113222423952792', 48, 0);
    T('-62345727', '-2.4993955365204997607672E+7336', '1.558266317849256079884e+7344', 22, 4);
    T('-3.1399910204096908054885057345338E+4505133832383', '1.43339718269085705033E+63600512081', '-4.500854282329840220501460071298699850230911085686154e+4568734344464', 52, 6);
    T('-2.22034E+641551598', '-6.8638603896738E+49', '1.5240103777608325092e+641551648', 22, 0);
    T('4.14810E+3997454788', '6.795717828744751205626592564587E+106700', '2.81893171254161024760596686171633347e+3997561489', 55, 2);
    T('-8470.683412702318695134141835603309545441250773090943832077', '-8.0045585413202344156756314330277342395498902E+364', '6.7804081261965977354679960740212060512415381407922755721180920313588348566e+368', 74, 1);
    T('-8.5721151815194626967565020089303264302626970690470571833E+6393955', '1.07703012935919254979826213010527030840928E+92683', '-9.2324263228338052335127116637328227537560612746241574629e+6486638', 56, 0);
    T('-8.468796624684142773200', '8.83606976550648120715990932447904602670E+393341457417', '-7.48308778055948929708775624709343296107672070547969792444e+393341457418', 57, 6);
    T('3.3816041337190227147473221E+992255', '4.5E+64883074129', '1.521721860173560221636294945e+64884066385', 79, 6);
    T('-2.05377354E+86042897171', '-3.66514774814455628911853303111824576845593645E+2832239', '7.527383465329873801632233062926649770471768936931533e+86045729410', 75, 5);
    T('1.44346710127291776691722553884E+50040745898506', '5.623909703115454916024E+37', '8.11792e+50040745898543', 6, 1);
    T('5.992336313525216237518E-338878476804329', '6.84657414871033564994977675871494352062132973E-91097521502', '4.10269748945599383478861501996e-338969574325830', 30, 2);
    T('-5.29761861393483014256223544867151E+895', '-1.697456239132253097351E+3846600681042485', '8.99247576876683623517471878414835144442022637565017001e+3846600681043380', 57, 3);
    T('2.8544930003133877686784971041492E+37140', '6.93729482843414802533292660669346586747248652348573E+99203599', '1.98024595288755398e+99240740', 18, 5);
    T('4E+455487036697', '-5.8987937698180194853591E+1400208392', '-2.4e+456887245090', 2, 4);
    T('-8.1234124142179E-1241090045', '-3.42102613958315156273776816802775025222374652154731734997078E+762504236243', '2.779040621165371178580265448833122397921926037868635061619537724e+761263146199', 64, 2);
    T('-4.312523635684487802596555794395772717526841520632E+4967', '3309523191769.524744407625', '-1.42723969873520411901518786063383641145177189e+4980', 45, 3);
    T('7.882507907710E+30263', '-8.73108072623902631919499901153788981183196126161E+499', '-6.8822812867433494648665394269933049970052078567e+30763', 47, 3);
    T('2.9969806384800365832772694E+55078', '2.04779144117731716321E+583', '6.137191300853550269612650660334951734882938774e+55661', 75, 1);
    T('9.828516322871547714E+16151', '-4.509080498432133772768421027148E-10632188848185', '-4.431757127998200099559e-10632188832033', 22, 6);
    T('-7.696760826704770434750055631308131292378138774802272E-884648127944', '4.53008473370001068019663719458443846708453072E+7030411405', '-3.48669787199955540263434376539135538827e-877617716538', 39, 4);
    T('-7.70128586949225349511851494495E+97057984457', '-2.708250901742925681043184436461357402891225182812612594E+66', '2.085701440063e+97057984524', 13, 1);
    T('-5.98E+5849', '-0.7337361652814786467856819514291018502425790980749125', '4.387742268383242307778379e+5849', 25, 2);
    T('7.928968180E+1718863693', '-8.3675316414925335688086842244052184228547468481876622432E+388481960283', '-6.634589213e+390200823977', 10, 1);
    T('5.573356552574572078742564402241554508996E-4600861743142', '7.66881901215045204976715608710507242599500E+7102350237434239', '4.274106269188e+7097749375691098', 13, 5);
    T('8.47614190298470278635867515234E-19', '5.723910545339816210341131732560366848582151642505967E+963096710141', '4.8516678022290837672315233218337948673515771882888e+963096710123', 51, 2);
    T('-8.1889641075588228708690E-45', '-660472843.9513035234169175401529187575198911450893', '5.4086e-36', 5, 4);
    T('35860946439904100496016089909511460772259.11327607950040125', '-7.6459E-72573915596857', '-2.74189e-72573915596816', 6, 2);
    T('-9.85099149551336317878035137440601448E+71361881602', '-39581413770134107458916922072203241778916063867931154', '3.8991617042998661798086307877261230427787072500448432764980481583784012319629045e+71361881655', 80, 4);
    T('2.449551569436981390981960933707035030644555264714E+665', '-2.956156562295354305170219729288175500546309680E+76980', '-7.2413e+77645', 5, 5);
    T('768725284856600646745695.7440137920', '9970.773851285619741364793346', '7.66478596907e+27', 13, 4);
    T('-7.675934725691823895428301594562565973910208E-806744892142855', '-5.786367566403586579060637060867456919838186503302618166E+58865359204726', '4.4415779738574180936924812760645677050583105007e-747879532938128', 47, 4);
    T('7.6301897395800993296E+93', '6.101064678236110454264988015251996216704E+5257853803280474', '4.65522811083917301370943477259644280403613987806277267216384e+5257853803280568', 74, 1);
    T('5.5413820770381366179450233902104272950854446857064723239340E+649768756265', '-5E-206544528070580', '-2.770691038519068308972511695105213647542722342853236161967e-205894759314314', 68, 0);
    T('3.6237791273821592828588064120236003123688185752688381E+2755', '-1.26053044350274118586812919647394766576E+27349008', '-4.56788391059500968694285472077587313309628461064606709270934315126356e+27351763', 69, 5);
    T('-9.71096228045928098638317776335149361E+62173563', '8.97066522723065230474E-7203597', '-8.711379165226454936520097268468941921362183977786789e+54969967', 52, 0);
    T('6.9151591629503118536622891752229298796449968E+667348072180', '9.75676283509015727424016733446305790E+510312663159725', '6.746956791980676354654069892356753105069333481103e+510980011231906', 49, 0);
    T('3.039692292736564273888443609463E-904765163004294', '5.34553894E+999', '1.6248793516441183487883500530378619e-904765163003294', 36, 0);
    T('-1.82908603E+76', '-2.47700622730248252919823267306040210728664909E+868064535504', '4.53065748658197537847555448298433884062057e+868064535580', 42, 5);
    T('6.9396265903430279512E-8', '8.582740730E+388203', '5.9561015787928130668292692376e+388196', 61, 5);
    T('-4.07E+1420691911513', '-2.0298236788169339246222250E-93093622220486', '8.26138e-91672930308973', 6, 1);
    T('-2.065087937013720642126835355269610427783138428023E+8226191401', '-5.49E-11882378416', '1.13373327742053263253e-3656187014', 21, 0);
    T('9.1E-2294840115585', '1.3038447193030813016E-252901359837', '1.186498694565803984456e-2547741475421', 41, 3);
    T('-2.353865264211436422444417026838203171651324855819287E+31186153172428', '9.9894701092991245026618832164243783426E+4548464278878', '-2.3513866698157630374452728858205113911110009783642e+35734617451307', 50, 6);
    T('7.46496734699072343310788549228928156866437407835065870010E+243920874668401', '6.10797617323506720139861740106E+918074663397', '4.55958426893971309642448949385562938052257149e+244838949331799', 45, 4);
    T('4.40300728943117560706451552698022377838310025778772481E+145528451404', '-6.691289149878E+3250496', '-2.946179490260457e+145531701901', 16, 6);
    T('-5.40578813960174013E+155229', '-8.034539459005959453656147730160175131055252347888503E+275819', '4.34330181146565971634416992965494266040670424000515258e+431049', 54, 4);
    T('6.0261626927582562175715426670409328099E+2809082538633', '9.438698E+411328', '5.687912975581196744428008e+2809082949962', 25, 1);
    T('-2.53828099836638456530273200186208069E+214025156900', '9.836672876894240460222338E+321603003806829', '-2.4968239850566648931851205914241910932328e+321817028963730', 41, 1);
    T('3.136300295544665397145825422278074256924E-479003672778696', '4.919954925112494334765048547281205720657E-62', '1.5430456085696748093535407015576043712121381282801035357624628103496473192079e-479003672778757', 77, 1);
    T('2.2940412566063451590267661824712539534296092441167729849E-31113599', '-1.83200843360287359314077E+9577939435128', '-4.2027029291357581878292e+9577908321529', 24, 3);
    T('7.6200534404972150805852073604E-536', '-3.48969118554966520334E-1006479', '-2.65916333247205317076e-1007014', 23, 5);
    T('-5.28649107155537479316541036278215215E+331636533', '4.532634608458744517403131778794838083E+61784088539', '-2.396173238824004497128901627637200697272974442086637184755812306152032845e+62115725073', 75, 0);
    T('9.43345627918435276225391021376573783E+9819304583', '3.23911062703050305346414719478836893157521713424030970967896E+993102954429885', '3.05560084835336651455691004239323123075450500078585e+993112773734469', 51, 3);
    T('-2.91207587684440028407610808057423130508784017154914017190673E+2214892881', '-9.3948E+1149899814', '2.7358370447777771788838220195378788265039240843669862087029347004e+3364792696', 70, 6);
    T('-8.4630083785824662087204216258771049937338E+30227927', '-6.00285795074242167292280306088429E+338259079158329', '5.08022371325734878500106358728553137974987731e+338259109386257', 45, 0);
    T('7.31581965568195093255786289371933775017E+75372', '6.476983001430025222819333583701127595E-78275648505904', '4.738443955137965623023406665887616449819073498366594959198712484e-78275648430531', 64, 5);
    T('9.8E+7229543143324', '-6.506645648585007604978108369830644754611972918532604652E-29562620819', '-6.3765e+7199980522506', 5, 1);
    T('4.3157226254604523948436364327393E+956', '-5.36818403801630390E-96219494047472', '-2.316759331050261605e-96219494046515', 19, 2);
    T('4.050286114120514265133257754741096626046978756707941E+8241078748997447', '1.3670409162161662550474975705864576864148873283988E-347586514540519', '5.53690684038492353655914848107409401785748922658039189e+7893492234456928', 54, 1);
    T('9.366753333800474497753E+799', '6.391873482524182758547610290575759703579155E+361', '5.987110225166423782235122714476077610759414776349255525511e+1161', 58, 0);
    T('9629979357.40765', '8.0930977002683014024514441E+33978951387051', '7.7936363791067067144575928944548774887365e+33978951387061', 55, 4);
    T('5.4499772605505384993253257005018218201957275539287E-4846724366441048', '-7.335236553915E+853994', '-3.99768724195958440974886376438339668745518613334861697076158605e-4846724365587053', 68, 3);
    T('-7.53861313351020045772518189700598045792222826285422860E-350152910', '-7.86337051862553850E+878753', '5.927890826536740088316257120412886289859007e-349274156', 43, 2);
    T('5.0097141277631440143679771415901457E+757299', '9.4781673E+65185', '4.748290862811265374217329111056718897597561e+822485', 54, 0);
    T('705638907.973130586557706593340916914097', '9.0325532584132547593532169303562052507496594545E-9017730', '6.37372101747587149317596656272718784738427234563336384608e-9017721', 57, 3);
    T('8.80242358637318059891504197438E+597482673663475', '-5.180671680907393791951700E-907674055', '-4.56024665972748351567288394942102779584442385e+597481765989421', 45, 1);
    T('7.3963134064864602559595434E-232872260', '-4284812811.18', '-3.1691818439615571815945353504103215212e-232872250', 80, 1);
    T('-8.68267091244907394434454E+4572293688', '7.24103060686756204772320747548937813382666750758E+43262847819', '-6.287148582640244660386700363449850331681650149023e+47835141508', 49, 2);
    T('3935528.106', '-1.2745277969201603133026915142188905747417650595666945E+943', '-5.0159399666575511510285081396562e+949', 32, 3);
    T('1.52414792659163667776516093E-6176022967610', '7.97197220161E-36918010', '1.2150464901930046509456756e-6176059885619', 26, 6);
    T('0.0000056483456376841712420325054995091510882', '-2.5338482E-72963804', '-1.431205043e-72963809', 10, 6);
    T('8.8842938289189232E+376323072457816', '5.5E+891', '4.88636160590540776e+376323072458708', 44, 3);
    T('5.32992387359181876054749294328164960707229E-89342949259978', '1.57451786871233216983823E-43', '8.3920603e-89342949260021', 8, 1);
    T('5.25861959603784171773990068260231189151942785853E+846571383909', '9.187656064173831039305930454E+9354170974', '4.83143882207204181411e+855925554884', 21, 1);
    T('-6.830E+407509', '-2929.8263146005857105601892818742552958966235299070193638756', '2.0010713728722000403126092795201163670973938709264942255270348e+407513', 73, 1);
    T('-4.5173606287232575796328317929141911020975220E+7987745173692', '9.6337873E-777585679', '-4.35192914545141340852955135896129642291601108050706e+7986967588014', 80, 6);
    T('-6.80319395420203E+9744953', '-5.13477575761995820462447506609126468075577331217419310486E+9183126', '3.4932875390423247833745883766136124985933835351431e+18928080', 50, 6);
    T('-6.0881280506563863919192595830632940706971821753433428E+3248384', '8.5567993041557124500533727931369976347520164383E+3004', '-5.2094889867467441156536162056822e+3251389', 32, 1);
    T('-9.07781937622059630803338184101182765330065106684293E+53891717', '9.7580196677221E-324563976431102', '-8.858154001318934427582622e-324563922539384', 25, 2);
    T('-5.9644178525894903778902200162399587611640219859342E-544356', '8.27209E+504331218688', '-4.9338201274226997460041910094138400468637294629626436478e+504330674333', 65, 1);
    T('-4.63431729496988770219401734445E+4917918784612', '-6.2416731E-418985504', '2.8925893596878313380805209039786999295e+4917499799109', 72, 4);
    T('3.287780888949595879E+74385', '-2.75790249E-23745212', '-9.06738e-23670827', 6, 4);
    T('-8.244238594646777068277290E-54615004', '8.81208608536915103795952339100650581483779593271502888340834E+78446794', '-7e+23831791', 1, 2);
    T('-6.41965209073616186747006E+887500', '7.73070188550456362751388955634869149517401201E+66960', '-4.9628416522137360532794876119794242808e+954461', 38, 1);
    T('8.1344909137784254903007302847358908190758908490786E+72432', '8.5547777682847E+956500578', '6.958876202550556882562190481111152644277004945668315668929747742e+956573011', 67, 4);
    T('2.3599057720813998719643026877773E+587', '4.967772924342709838882228E+11540', '1.1723475e+12128', 8, 3);
    T('-2.9002E+4717393326', '-6.97685896381323995965919018E+385636515641', '2.0234286366851158531003583360036e+390353908968', 57, 3);
    T('8.4169886831058144E+455273', '-5.889202337120696949637E+5564926305632585', '-4.95e+5564926306087859', 3, 2);
    T('2.87598692697633E+921875', '4E+3727143721130786', '1e+3727143722052662', 1, 3);
    T('-9.092594207043842422995584896591061483229710140054876774883E+8123403', '-8.2190121474606662401103336E+52423', '7.47321422396238250223625114488e+8175827', 30, 3);
    T('8.6928106832867645280202982275500727E+9567165095313', '-69424.209708409197116968716103296926777010847', '-6.0349151183200018436405262536778540744403542837386505174143137996780092e+9567165095318', 71, 2);
    T('-2.08209934474504871197571492401327162772942132E+95337556434', '9.2113212322181868434308513998E+3256187447283', '-1.91788859018376415112355217834959801378996e+3351525003718', 42, 4);
    T('-4.7613791100742964890731857027917620048401309174E+62875529', '-3.3E+873973125', '1.5712551063245178413941513e+936848655', 26, 6);
    T('-3.47659293732867222648911178261350594868067181335110943846047E+41395657', '-0.766295249299161043144670857505', '2.6640966516219774483574992077674310557667397113296792407841865747809968295753852e+41395657', 80, 4);
    T('-1.033644232465720511921569636118131454175384845337E-66567732247', '-1.225511405958924064543130E+5655', '1.26674279659039808758991470052357e-66567726592', 33, 2);
    T('2E+8476988545051660', '-66010392.5291338345865650800252387917445398200671913994', '-1.32020785058268e+8476988545051668', 15, 6);
    T('4.970928038982732481532665388009844900299E+64522', '-1.6342355937876498588448798865025537E+2401718825380523', '-8.12e+2401718825445045', 3, 1);
    T('-8.8287E-8183071671', '-712783.296', '6.2929498853952e-8183071665', 23, 5);
    T('-7.01673831142030895270453548053391E+4245', '-8.830359631367496370969500154647162746074443079448371005E+425551717694', '6.196032272903562831499964e+425551721940', 25, 1);
    T('-8.9018E+9525095110743', '-7.52353E-903299276423', '6.6972959354e+8621795834321', 50, 4);
    T('8.17422148880720496042943025515346135021928845490068E+4174', '-514526.250008420449644337785', '-4.2058515293742187616993180472199334148549556142588997815841855633121e+4180', 69, 6);
    T('-5.640868322346171703536075E+436889475', '-4.524903203115597E-70658389', '2.552438314013749628e+366231087', 19, 2);
    T('-3.736346930038858297238375277153155433512745536001368551201E+999454751', '-5.5958737687821232229399146877570169521073552708394155E-26510194', '2.090812577687406206912901885109794634191266050306854469e+972944558', 55, 0);
    T('-7.436112510997006903E-9752931377', '-8.9320100947112138151904764983083217509164374198833085710780E+2988173948010', '6.64e+2978421016634', 3, 6);
    T('-8.311696102058072961312172679E-4320', '1140805303.813351909841', '-9.4820269969126127468527693244302948908658e-4311', 41, 5);
    T('4.23683484848490091073486883961297913821481526921663146E-928455928279', '-7.393549E-21726464125966', '-3.1325246057180690643662878774251702e-22654920054244', 35, 5);
    T('4.335640191318553622722278270693395401666959983158751582E+452', '-6.603398725050987482253669838293317468447953E+997180', '-2.8629960911632756438441575817126344524503467261556e+997633', 50, 5);
    T('-5.3274540853233918910185949743270525030867E+205', '-5.70731206746745934257022989268E+96221', '3.04054429900450103212787342202776153929818241812e+96427', 48, 2);
    T('-5.4272482306076784898E+9053605476610', '6.3E+613311072706', '-3.41916638528283745e+9666916549317', 18, 6);
    T('3.778289E+542179354256714', '0.000095800827657978871552195095125663060', '3.6196321333103733261807165376724635730434e+542179354256710', 56, 1);
    T('-7.10983121910150E-4451', '4.2560630019047625E+60016328298351', '-3.025988960140532728197382089375e+60016328293901', 73, 0);
    T('4.980132973023473238901047938946471137209103508993447284E-458117293038', '-2.6941321915017396292E-371352785298949', '-1.34171365605818039228018377817322253249858503288621022533527103495e-371810902591986', 66, 2);
    T('-8.55594702742174172456552234835429258700164805368239539E+407', '8.401544217501568598707747065398849E-528', '-7.188316727e-120', 10, 1);
    T('1.12401559879546746319207500556322834974179170E+857408262005', '0.007158', '8.0457e+857408262002', 6, 1);
    T('-3.93923515784090911246846957852335388749911847988330591E-9756632', '-9.80E+8074888400777', '3.9e+8074878644146', 2, 6);
    T('-7.67889529043288851494315550325756003165413353536449E-57338111', '-1.736291175982976214099046681609418661445466850607914748E-91202838', '1.3332798134075857679307376797405261406722822523099e-148540948', 50, 5);
    T('-7.71448860107309529940609424560680763667652E+48793021765515', '5.4119285530257807032958353813375276700617E+3953175', '-4.1750261132139395832649295018335631172230963042788548137077e+48793025718691', 59, 0);
    T('-2.27339022391623532339642020507E-82597', '8.17547240678142641315588656376176302395170902E+9953816', '-1.85860390454738303102084411570520928523647202469928580389213932714853687314e+9871220', 75, 0);
    T('-9.7927081887740940010002588390E+432', '-4.5190038795403E-43766862754', '4.42532862962761952796254425621854763317117e-43766862321', 66, 6);
    T('-9.067397E+93636563439217', '3.778903E-611', '-3.42648137254e+93636563438607', 12, 1);
    T('-3.52405346596244756014247325531006285E+2295000984679665', '-5.3638640174012005740380052E+516510803908300', '1.8902543581474e+2811511788587966', 14, 5);
    T('-4E+841077055', '-7.15943104E-6553776342136132', '2.863772416e-6553775501059076', 21, 4);
    T('46331.9051195012950', '-4.5159655068220496771611478365968E-1421045', '-2.09233285385019783788077242231247551063765992856e-1421040', 59, 1);
    T('8.9317653E+6970648359253', '4.5532075323250864549447053051133143E-11327727073406', '4.066818104091983551777513226293701323273379e-4357078714152', 52, 2);
    T('51247641.17856937640449691676426', '-7E+97', '-3.5873348824998563483147841734982e+105', 58, 1);
    T('1.6622950551E-81106526', '3.9E+7669469510879', '6.483e+7669388404353', 4, 2);
    T('1.8621378422040860366033727751529507E+963482441238769', '-2.248425878711E+586896', '-4.186878914138727607844583741759567805072e+963482441825665', 40, 4);
    T('-8.157691668639341E+7227056', '-5.101E+1586430012585', '4.1612385201729278441e+1586437239642', 43, 1);
    T('-3.9517988772006290127717991826E+27972', '-4.4624103395760905160444331103010026283404195886884030521E+863876', '1.76345481695452721383665614389563e+891849', 35, 2);
    T('9.61635596876318295208950111602608778108299607632268349E+76191', '-9.96332485637211578767867462837579E+515941439938457', '-9.58108784513005e+515941440014649', 15, 2);
    T('6.375469061365288129610678386060287805E+11925845423959', '4.19561705285317338145757472E+273390177', '2.67490267138020177749509211780443758406502597479365068004922896e+11926118814137', 66, 1);
    T('5.705E+6774204375538', '2.0423997424057191067870E-20690', '1.1651890530424627504219835e+6774204354849', 58, 6);
    T('4.4E+371430', '-6.752368490215E+71693781', '-2.9710421356946e+72065212', 42, 5);
    T('1.2433107846200093584669924947410722434658467724591197673E-615542738', '2.7638897543703508E+28513627', '3.436373939109405792983681865325485900710399241559101633556987217e-587029111', 64, 3);
    T('-2.6583408214645430345E+591', '3.3762E+13304', '-8.975090282e+13895', 10, 0);
    T('74297311.122954506403973719605568127', '2.948064495550E+8685061533545', '2.1903327e+8685061533553', 8, 6);
    T('-8.384E+81114025003', '-8.34429873113274369422295350E+38', '6.9958600561816923132365242144e+81114025042', 32, 1);
    T('-7.3234868431397528552985759536333209E+154761982621', '4.3947316758361589052534319E+146', '-3.218475960712e+154761982768', 13, 3);
    T('8.7132161361696011327901293973176471267404E+2962798', '-3.2971667135217158799832594939096468115692391592373E+770578848', '-2.8728926211898707401489e+773541647', 23, 6);
    T('2.9199967910964413953552344993972191817543357E+1967995230', '-1.63827146039037944970041611306055626E+3198668256815', '-4.78374740728478879e+3200636252045', 18, 5);
    T('-3.38324291685673843573116376370315455E+9502714362455', '7.922406503780967568E+94571474017', '-2.68034256883767158953374525818849182348830179328416344e+9597285836473', 56, 5);
    T('-7E+626828837064879', '-4.29372947770678037531466255884259852086101772310118880343163E-9266537', '3.005610634394746262720263791189818964602712406170832162402141e+626828827798343', 67, 0);
    T('-9.9266443067367085991853E+280088', '5E-90593638149', '-4.96332215336835429959e-90593358060', 21, 4);
    T('-7.788368229068327795654698244658133964400129E+5265235295', '-89250146171416488003', '6.951130028811644288390773513727971935962964952137955418201524e+5265235315', 61, 2);
    T('-5.103197999137052965930720382857223890405E-55913164839', '-7.01029753858395035751468', '3.5774936372257022827704e-55913164838', 24, 3);
    T('-7.093212902792542594936692157833053033229334824749876', '9.043358189217873635840065269037627E+654083865702', '-6.414646499233482512897111967368839482633170052568e+654083865703', 49, 4);
    T('-1.9423706939846439032722679165280E+1765408260230', '6.2970947667344617829044650881227E+88', '-1.223129233214908585063495329285275012037631322890958144783e+1765408260319', 58, 3);
    T('1.37272118401724394381079391629243644843573321442295883E-745', '5.06619483516278799757326940102649083813456405E+85671910', '6.95447297258670835e+85671165', 18, 1);
    T('5.866614680939876477566526258425735300914498909367E+459848782856469', '5.65502254819757832999926803395547E-755484622', '3.317583830230194324438281162353269564282e+459848027371848', 40, 4);
    T('-6.45704746208829535E-7', '6.737752835661824957179669215958506939E+138628', '-4.350598984768840217488349510513732103408662077365643365e+138622', 80, 6);
    T('8.492284953861474981293750E+66', '-9.3795815546765615613376677741099156245534E-98', '-7.965407931e-31', 11, 2);
    T('-9.845091765855874390664379145837204E+771746831545703', '1.605888518E+7446869', '-1.5810119825444293126818172861898613e+771746838992573', 35, 5);
    T('7.895873213470938631673702119380276E+31252', '7.88263350290994996784515264405960097945893607987105069E+470405', '6.224027472723526813812444534547e+501658', 32, 4);
    T('-1.312481567881669575204373995147486E-6919230117774421', '-9.4059081988017485818683059173994222593228989699057993323731E+947098', '1.23450811401e-6919230116827322', 12, 3);
    T('2.69517647068722849190634276427388E-8460', '-1.2683373126352838317056184172838830605370289E+469754', '-3.418392881909288213575312412047480531259929695781221738436009890193761075e+461294', 73, 4);
    T('-3.8E-8377684962838', '7.756800920924864096646590E-724', '-2.9475843499514483567257042e-8377684963561', 28, 4);
    T('0.001586650897673', '-9.316409278372032007047961E+3652488', '-1.4781889144618050728039735183414294753e+3652486', 66, 6);
    T('-1.986866832580715080997225485989762262E+46', '2.8209250291339506763552963216066200E+91', '-5.6e+137', 3, 6);
    T('8.4581786750421083861002046970499292751E+6866345', '2.475348848992391026442729084854699005780129E+6388632470', '2.09369428478374699621853211482308409191917e+6395498816', 43, 5);
    T('-6.209578882601E+23850', '4.21966538942567807838272578E+208011985', '-2.620234509382001560310054724219299615378e+208035836', 51, 5);
    T('-6.133977490558108909414408051108796536558856720E-621451', '8.46787730090618188899613792038201072292887253562646E-588214876', '-5.194e-588836326', 4, 4);
    T('-7.19517578184891130247513542E+166', '-7.393201844803096396830427494915873155729E-361497441399', '5.31953868640479325158160276348353120643e-361497441232', 40, 5);
    T('-9904363.3675604150267963158123474554540407', '3.851687E+7', '-3.81485076261086722733160212623131336555e+14', 39, 3);
    T('-9.9786032019104505028680673560025831097837025252089E+571678995091', '1.553034E-1457149344391470', '-1.549711004e-1456577665396378', 10, 1);
    T('-7.1759068531615835E-50224471056678', '8.6993642851161631107644040162335270627785981E+70950650070', '-6.2425827791714194496564152390559e-50153520406607', 32, 5);
    T('4.9731839941406458690992E-7402159245460', '8.57537843242932144125123905444592177130964204603843147E-9696505040', '4.264693476385640348039523063392025434276464813646063780882782087e-7411855750499', 64, 4);
    T('8.4112267634870778074731232681867306631E+43160330', '-1.83143526128862451084112121256402393539E+100263166076627', '-1e+100263209236958', 1, 1);
    T('8.505684544958040868894405453594930091185622E-6994618329206570', '5.149028928341654912599649186705486838984484754021936E-79393024359', '4.379601577733747788995137537443329638040163822212783843e-6994697722230928', 55, 0);
    T('-2.9485516978495407623071912327475881503974E-150340', '9.581900083736967067048367967E+1770254051857218', '-2.82527277605272910486678206586748426507051971e+1770254051706879', 45, 3);
    T('-8.104845E+4874065059', '4.292742543707726665346E+74101929019', '-3.4792012e+78975994079', 8, 1);
    T('-8.43182542696096226064125854283320763238496E+1328', '29869.162433805494251638548812', '-2.5185156329138834622531344176918577934968781742669408254667370893466752e+1333', 73, 1);
    T('3.60891E+244951467', '-7.914426370730240610854624579795486E+4053951820', '-2.85624524735920726429194e+4298903288', 24, 3);
    T('8.1201E+322176', '-7.37071020011432207421659391013846642E-5459088759567559', '-5.98e-5459088759245382', 3, 2);
    T('0.00000248719300307989880781074669709356644555530004036', '-1.83017961485333601350255991125799494183460E+709004', '-4.5520099324426813732978e+708998', 23, 5);
    T('7.120536E-348602', '-9801376.45326647491869907674688214401414196838158875', '-6.979105388503625225169384914293719420988239497196443157e-348595', 76, 6);
    T('-5.65371702340270376E+6402720', '1.44082800403E-17714059207760', '-8.1460338141797504574261761528e-17714052805040', 42, 5);
    T('-8.8298610463348007726017235716299479851808326039395E+374394520795', '-2.2074264138176003152240891E+80791310', '1.9491268504019e+374475312106', 14, 6);
    T('5.0172789545184170015596416641798630957915711455353126369933E+10063638', '-2.4520160831168E-6473572184082', '-1.23024486899626021894126285500680754619953100743e-6473562120443', 48, 6);
    T('2.5917269633675274189817467814190117556039234588505663430197E+239', '8.8014326213888433E+218', '2.28109102411160035026727e+458', 24, 2);
    T('4.5175609282049315030063512349E-517555', '-7.7260744121542171698974448483331925230547740391951586970E+6507703', '-3.49030118927517758387771099197268770445541464730163254706850801335890818e+5990149', 72, 2);
    T('3.244075820067E-827988108', '-8.58764115699329058727249647233773483818241E+8462620657', '-2.7859e+7634632550', 6, 5);
    T('-3.8441160793492445802900222420446488124219183383124E+298810245948', '2.8487260953601665948331665962409531130648E+7227114755', '-1.0950833788835805852852149295385541808591884420935437842857820091316626228683501e+306037360704', 80, 0);
    T('-6.6671357E+7200879804175', '-9.27679889259473252058198707049024699E+160552359', '6.184967707e+7201040356535', 10, 3);
    T('-4.002110559457392679491040359788E+8193169500', '2.4385933791527551E+985207', '-9.7595203129301264195308900756957164629e+8194154707', 38, 1);
    T('1.327581961397E+1766', '-7.465341576500393908150915593617379544892E+53370637653', '-9.910852812628965067726102789255953310555108572534124e+53370639419', 58, 0);
    T('-3.40864405635308047290230877200868584E+880474289392', '-1600613213845840684257177695756.775003149021882463', '5.45592072e+880474289422', 9, 0);
    T('9758043.94800432692907', '-6.325385383732110383594459E+153322', '-6.1723388562522146875797692e+153329', 26, 2);
    T('-1E+960103180', '7.56737907681682614E-437549586568', '-7.567379e-436589483388', 8, 2);
    T('5.950971903969409215884628307894254420800295451153961609980E-60643148', '-2.6269583940E-5281481022902', '-1.56329555955907e-5281541666049', 15, 3);
    T('2.60710186816947036532436498922558791062897E+9440169', '-5.8778122990487999480532616E+145552982', '-1.5324055425599615965355807083912228184017662140320289956354315948552e+154993152', 74, 0);
    T('-8.9650066085123060291802139521845765481646760592194250859205E+300213317088590', '-1.38870E+296941991507841', '1.3e+597155308596432', 2, 0);
    T('-6.28236995322411997691889861238202908975022366447635488E+483636618484', '-9.93740293319848660220361440515E+1567478', '6.24304416006e+483638185963', 13, 3);
    T('-1.9376722525885E-29686343994244', '5.079376995543863626467204979370451426288E+47190', '-9.842167864701685559755159620698306287791591624946488e-29686343947054', 78, 1);
    T('7.53878507799979614943444165E+770', '2.55788120426468183567E+36308979', '1.928331665400673195963652370348785225e+36309750', 37, 6);
    T('-3.18773434895500692715889735102873133E-5765023735423', '1.485406391711321495748046633383055852994562856428706068508E+27183683586', '-4.7350809770154954261971987607483912090011440748199466370103125392664096e-5737840051837', 71, 3);
    T('3.3603624221173E-253749256929', '-6.9051760219325763633423230E+5938', '-2.3203894022207654576393651739e-253749250990', 29, 0);
    T('-7.051500623225286611561720E+8700477', '-3.3863722130E-232310483593460', '2.387900577044229302035353314248636e-232310474892982', 38, 3);
    T('-9.960468642764337512155203E+29170437119833', '7.9607170824846568742469448795843527572761508570963845E+306304635', '-7.929247287400682693186418e+29170743424469', 26, 1);
    T('-4.618313419E+7163086373014767', '-7.1821E+2558111675735', '3.31691888065999e+7165644484690503', 32, 0);
    T('-6694697.4628086855074872001057', '-2.411782443953230330094482196752557260896877654904E-9', '1.61461538e-2', 9, 1);
    T('2.81066658236174E+594928072417782', '-5.95334695059245E+840476737038', '-1.6732873327235368042413212863e+595768549154821', 39, 5);
    T('3.0086919436701309765770304920E+28324', '-6.61786663575929162065649669844106007773E-315366659485', '-1.991112203129233e-315366631160', 16, 2);
    T('-4892581426.173361194', '2.485161961667139009059676946464390E+53688977', '-1.21589e+53688987', 6, 0);
    T('-7.727E+4572', '-4.342836414993924254403308843863489538177906E+9109', '3.3557096978658052713774367436533183661500679662e+13682', 73, 6);
    T('9.7798146677278449417430294370E+325', '-7.2303E+158347660', '-7.07109939920726372822846257383411e+158347986', 73, 1);
    T('-8.31349350847056138529948406065329015E+43194572539446', '3.57464088671990136817705540976184219530281897307165E+770598098181', '-2.971775380685935141e+43965170637628', 19, 3);
    T('-8.08850033402277E+1076091927866054', '-6.29384698067162181204017154178058772980E-65061323715', '5.0907783405450615466758505565769733513209945735807546e+1076026866542340', 56, 1);
    T('9.490837258803705341530977258180345043E-4366968', '7.273201525970125123710336653175E+7331582406627', '6.90287720334652290354853666804933e+7331578039660', 33, 6);
    T('-9.41E+96214571589549', '9.426163422E+5750276469000855', '-8.870019780102e+5846491040590405', 38, 3);
    T('-39069143.6', '1.778139672856E+2088996', '-6.94703942196680861216e+2089003', 49, 1);
    T('-3.1517318464511813108701E-492', '3.371060092E+1524554268153', '-1.06246774482570491433304399060492e+1524554267662', 57, 4);
    T('8.935696252980908262436352236352578105322581024710689E-2131775716749910', '-8.5330953466836338864788462556265E+329', '-7.62491481156897716788683859806514e-2131775716749580', 33, 2);
    T('8.825122847168525163804974075E+166260', '-6.4161833470834842160492986E+1680519', '-5.662360624796867572099054016015e+1846780', 31, 6);
    T('2.8748432878507412201740249769233016E+331491207', '1.6673429578054414253030379610653034724490996E+24124529917343', '4.793349710792174915976119703630921818489356e+24124861408550', 43, 2);
    T('2.1182174789819662091154433676188520746964224E-621', '9.374713254307917625730164930E+92991551', '1.9857681475718941544482945124709571465842291004165717973205868032946432e+92990931', 77, 5);
    T('6.8176E+32261796', '-9.43398788169692911803366059260772625829859E+1881454327369598', '-6.4317155782256983955106284456162434538576467184e+1881454359631395', 72, 5);
    T('4.46803697341513750E+587875050', '-5.466986096475335E+66648', '-2.44266960121982927031864843835625e+587941699', 46, 0);
    T('-3.187041133421327028646471642947E+281911634', '-8.876225230144863E+50', '2.8288894917983863531463558044956526774422231261e+281911685', 58, 4);
    T('-5.118104912677E-50', '-9.1826999483108562217174327337326151001136790E+6279313511449', '4.6998021717088627196304203212649187399234172671068208683e+6279313511400', 61, 5);
    T('6.93426300767248366189795467458892208585578681695151E-31490', '-4.5283144271E+3270116738604979', '-3.140052321894914575786154672092290165e+3270116738573490', 37, 0);
    T('-598244672727060298902.00475583692585094261946', '7.4991439226967509906E-432286402082', '-4.486322901766840973782224305035480424216311912920940172513837e-432286402061', 62, 1);
    T('9.78575531899272438661396548142235635820E-7749386208034814', '8.3519E+495619', '8.1729649848695334804561178304291378068051e-7749386207539194', 41, 0);
    T('-6.0110746E+8776432', '8.220373534920783218446281234803277299062606561896622E+90', '-4.94132785582745330165086925949826161691518381140101123300012e+8776523', 69, 6);
    T('-8.1053534907755447262683612072866950406742E+76635878212', '-7.94E+8669677948206', '6.4356506716757825126570787985856358623e+8746313826419', 39, 6);
    T('5.4159139464681534682207086140349921E+5845191178064176', '6.96730774555371397325E-327321278', '3.7734339188479948285061723557433472333951630690223361325e+5845190850742899', 80, 2);
    T('2098026475136982.50263017905129', '2.169143224371153531235504781898246639267418788829798E+34414907463', '4.550919913094680002259485098992741940677129142958188378126e+34414907478', 58, 0);
    T('-7.334830348981377371313159E+2597156224678573', '-7.82776032562855E+30362499268884', '5.7415294001e+2627518723947458', 11, 5);
    T('1.735661828E+9851364', '-5.752648283310905157345623039531190073067726522804805963138E+2716', '-9.984652035252467537733131712592e+9854080', 31, 6);
    T('-1.433938882782567428686167276041560511174E+166285135', '2.1042846903798347441738140784979362980655062289719002354E+1800089', '-3.01741563797972104777956988622422e+168085224', 33, 2);
    T('1E+132', '-3.83377541E+24836', '-3.83377541e+24968', 34, 3);
    T('-3.677497341693026063026538486789353613371975033E+56', '7.7794762222E+9165', '-2.86090031269046e+9222', 15, 6);
    T('-1.670697700002424273854450011819290E+45470827513', '3.71568565992898163E+6938771', '-6.2077874859753396121799716681261897e+45477766284', 35, 6);
    T('9.851760116780647702743896365E-48297342308591', '2.53050703887133718475613720863598E-851024580055696', '2.4929948320785335839040404773033847903321476222e-899321922364286', 47, 5);
    T('-1.936042652543619266032915590947043937870935246173588123E+9114002632', '1.172431747E+517399095873573', '-2.26987786938822953e+517408209876205', 19, 4);
    T('4.9540991299477717559948718912190818603865703668827378110736E+428', '3.42640390E+679953632678', '1.6974744579e+679953633107', 11, 3);
    T('-820607396799586172099128893708.79020391522412736911128', '-0.8694747513045521993967396610379511883134', '7.134974122509961714e+29', 19, 4);
    T('-1.35195743396971560052405453924335440024498640E+46', '-8.6728919641063550944772804514773599673582035499687949E+979212694612404', '1.17253807648897946117696175524116522267804564831215267497e+979212694612451', 57, 6);
    T('5.7092127818776136232579873243736566850805865299114677942E+98444628201303', '-5.27365755829522372849258352630055012015134882898E+89000', '-3.0108433139064e+98444628290304', 14, 2);
    T('-5.6029530463975E+2090475', '5.503721001351821769159676004E+95442', '-3.083709035104608899719966468022587939559e+2185918', 52, 5);
    T('-6319.5', '9.72994048289E-54130616', '-6.1488358881623355e-54130612', 47, 3);
    T('3.63821230789337779277838725951E+7033159915', '4.506852776197641724289979792582E+87', '1.6396887240225698971265101418885912321390433510245181206955e+7033160003', 59, 5);
    T('-9.0689E+72077642', '-3.7655936910277902941049313598187E+914639205', '3.4149793e+986716848', 8, 0);
    T('-2.9352919671853145920927673E+18436', '9.9028E+4364733', '-2.9067609292642733342576256e+4383170', 27, 6);
    T('-2.8071955459547033570139623691373341448729812556924901E+56251', '-1.2115320255046381476344856974475177792447185136E+52436', '3.401e+108687', 5, 6);
    T('-7.0660483802863651269E+744973060942', '4.577615167803674531289777E-901271601', '-3.23e+744071789342', 3, 2);
    T('-9.01665376146752914E+5019880478870889', '7.7461582824467340962743044897565341504E+6794', '-6.984442721434619999143484235306224241e+5019880478877684', 37, 3);
    T('7.88880392791194480567129127042735471352800497067428649598135E+91', '-6.05847863219201758790250E+342068319413984', '-4.77941500308069750843963297127349774922338649308653627681010695874518e+342068319414076', 69, 3);
    T('-5.11E+44571801', '-2.93E+378166961', '1.497e+422738763', 4, 5);
    T('8.7249081025932219228150986971404610239499E+98035016397975', '8.0616713633547785824635757136493974467854035377796225E+13709558518', '7.033734179857785375e+98048725956494', 19, 2);
    T('7.902386490253255361577019747116581560064741048113189709E-7566840721640579', '-1.294827632220116107E+37', '-1.02322283880628562e-7566840721640541', 18, 1);
    T('4.17512651999435329369153446019345348387365E+9332294891912', '-4.43646E-5061244761229073', '-2e-5051912466337160', 1, 3);
    T('3.077989481803143074743022772564419E+34546131724', '8.5707283830147081438208035444600206185050906196123216E+482943', '2.638061181431093e+34546614668', 16, 1);
    T('4.49763326838359195041935930042955949E-8261665', '8.989735797247891710389990345E+504678', '4.0432534795681010887732495e-7756986', 26, 2);
    T('-4.22296932012402417441370203709590313594690084220031E+766181315709', '-3.5935501709252256068191376742910215055130606254339880081E-5747016477917', '1.517545212214367084483882013515462562548797987439727172321329736861019644e-4980835162207', 73, 2);
    T('1.12366516880428752E+851', '9.2355138093200321597869793413298682E+67', '1.0377625e+919', 8, 5);
    T('4.30069447449641765063630692103207929814507535379243264906997E+17743351241425', '-2.146050E-412', '-9.229505376993037099148046467880893777784238963007e+17743351241013', 49, 3);
    T('-4.29747064178403867415486338151318366225627141E-617', '2.6E+5957958503263', '-1.117342366863e+5957958502647', 13, 2);
    T('-8.801812878344293080160388384810988E-291344', '-6.479062966593916076374471093080167472625779907045307E-42983281', '5.70274998589699108628057e-43274624', 24, 1);
    T('-5.841944982489797603386148798E+602840125345', '2.713196961892602146199163561177415506120254E+594', '-1.5850347378035049700360210559030738524424877394252218701386059725554692e+602840125940', 76, 5);
    T('9E+65396540143912', '-2.753444E+7233694720', '-2.4780996e+65403773838633', 42, 0);
    T('9.601683007348163870E+337648147612', '-7.903414864999244859554932308995883644E+751620', '-7.588608420928613194143435978e+337648899233', 28, 0);
    T('-2.7993303642116593488329937128207506997593778716016672636565E+158437320339531', '8.841382090575778361111225501504434283755305257896E-80', '-3e+158437320339452', 1, 3);
    T('1.441369881966816068902710808613117420348452808E+6317529453', '7.15606031256250507601859753992362806E-675035431999830', '1.0314529808065634e-675029114470376', 17, 3);
    T('-7.74860E+9664677368', '-2.2250114742419855E+699537', '1.72407239093114488453e+9665376906', 74, 2);
    T('-9.258523251562429039313671200E+82', '-9.4648865E+24330980848', '8.76308717336493385214079358063188e+24330980931', 64, 4);
    T('1.55146E+895380065', '-2.71475478327879211207001267842249865336828727934471422040E+83', '-4.211833456065714810192141870065369760754762982412150324382e+895380148', 58, 0);
    T('1.18512382956E+13', '-9.53383441081134574794236993585472137499014178740536201544813E+953195451', '-1.1298774347331648339498683948562358947735e+953195465', 41, 1);
    T('-8.439802272254E+7149963', '-1.857161789058199204316300652228157083E-1545867', '1.5674078287236693479238089049206622377142244475082e+5604097', 68, 6);
    T('4.906355620311715749766692830153999911860610995791E+8511846', '4288440184644863740713527547.53168291000226628464656373143', '2.1040612602302939265870227034907861144245397053008435599342980403975e+8511874', 68, 0);
    T('-268.66831200377163', '-2.198227E-980330180729200', '5.9059393749111489890001e-980330180729198', 55, 0);
    T('2.16214573332058103519E+8226349533', '-2.349884156038894195214767504364664404E+444010410', '-5.080792001777129561541606769153950128009985089891264377e+8670359943', 55, 3);
    T('-5.32394080249059739082491760602654387221963691982469010E+13611748933', '5.533364437527851E+4540920', '-2.94593047040049601058110534979867783238191283947051680416523561939751e+13616289854', 70, 1);
    T('3.71529959E+720316481', '-5777653588.975376061915507690477176399078', '-2.146571401028224320293050033707170037985216977802e+720316491', 79, 6);
    T('9.5616098934125163482470117411508243040715738437438415E-8321589115', '7.30914755673152218492724955773097459784E+76', '6.98e-8321589038', 3, 1);
    T('1.7E-98', '-1E+84270', '-1.7e+84172', 33, 5);
    T('-4.599870641611177E-89', '2.0490033975984682132028593962328988675244931E+491889', '-9.4251505731747475901e+491800', 20, 3);
    T('-640.32972983721', '-5.0787248907063E+5023319', '3.252058537183478963340921423e+5023322', 63, 1);
    T('-721528062.001388035459293', '-7.056336663823012428237E+1812910502320', '5.091344917877558113849852297087090791497256441e+1812910502329', 47, 1);
    T('2.715369931417110894500519214003324E+88871701', '-4.8818090201152619761103625496943755928E+16230815577857', '-1.325591742414181225128e+16230904449559', 22, 3);
    T('-9.12747128251185866367750E+2900', '-6.011498286552463273413927762710458985956037746E-26', '5.487e+2875', 4, 4);
    T('6.9922819325845485883051388484847699649200307058964156035E-265425915774407', '-6.88710210881896860374E+2599', '-4.8156559643359817842029942477494599487106233e-265425915771807', 45, 4);
    T('-9.54827673536924031621E+7173182', '1.78785949343295898450903766729584E-874', '-1.70709772e+7172309', 9, 6);
    T('-8.233207765848080417293542553110E+42203362', '-4.74794332215092683E+758079676615207', '3.90908038317395450326386817177286464e+758079718818570', 36, 5);
    T('1.0786E+3795935188717333', '3.365270152910268384984996236319509821548403027771925224E+47816313634', '3.629780386929015480044816940494223293522107505e+3795983005030967', 46, 1);
    T('9.8367291380453403516E-5340800003279370', '-7608519.9810344139356713360672268528', '-7.484295019484139991286721474342398795312671314291344448e-5340800003279363', 64, 5);
    T('2.784063627273779647222589027531649410901690554889333E+43749', '-2.84957417676160810975808348394447371168136334641177722821E+36181160', '-7.93339581874061720125409661320581989248295667782349292519e+36224909', 57, 3);
    T('4.722354556116522115348737218530E+56132034', '-7.57605746035467672231702349843173061866066E+88749803', '-3.5777e+144881838', 5, 5);
    T('-8.93819918270883288098537664E+147545', '984911022.2905165784441948249733484', '-8.8033308944780163652517803940553547076796003937530181e+147554', 53, 5);
    T('-5.44349045058368762176937031E+95953501553', '3.647899036737295E+7330162144', '-1.985730357117289800454071853326846904271145e+103283663698', 58, 0);
    T('-4.181268698248612575437174960045599522423416E+14259142094232', '-7.35769144663869220832198604695608068449811287E-91389', '3.0764485e+14259142002844', 8, 5);
    T('3.072608606373501136652602666416582108325687682417314E+4849125', '4.4807127920E+7362190858', '1.38e+7367039984', 3, 4);
    T('4.04283882983091796673616626399262283972E+61', '-8.59393910236041106151332872580795E-533314385721459', '-3.4743910704224933797e-533314385721397', 21, 1);
    T('3.4912979202935092738624576905E+45183344411168', '5.410089396623574816870913831514727042436043690712E+888124', '1.8888233859033853191439769674037993860570691873183469853454401707824567820636e+45183345299293', 79, 0);
    T('-6.66892497209E+395', '-8.4320386475514997E+16339816', '5.6232633102284186483660143373e+16340212', 66, 4);
    T('-1.664379400914660E+68415505789', '-3.841965050212415E+4995064109514', '6.3944874886076009030782875039e+5063479615303', 58, 0);
    T('2.4323439762344E+8406', '-2.7459279993294363E+629802261', '-6.67904142e+629810667', 9, 1);
    T('2.62964518724115180344781546074690695276488801987280850147E+1793978', '-8.59E+251', '-2.25886521584014939916167348078159307242503880907074250276273e+1794230', 67, 0);
    T('4E+1322', '1.7003550750270900838083029734281802866771317842473022E+4128', '6.8014203001083603352332118937127211467085271369892088e+5450', 67, 1);
    T('5.89E+773482270903268', '2.742336249333304167002439560447796355930787389074823256527E+8655', '1.615236050857316154364436901103752053643233772165070898094403e+773482270911924', 75, 4);
    T('-8.3621781978984767773985766052372640060082100784E+85', '1.609633095021863737E+82109', '-1.346004e+82195', 7, 5);
    T('-2.02991850E+1596', '1.2324615756751016756626E+365', '-2.5017965530020388808085114981e+1961', 75, 5);
    T('-7.79464E+528524', '1.83287450803478869392553881365729790623989174781162829995617E+5841609058046415', '-1.42865969553082853452197618584857205518937098131624304119703609288e+5841609058574940', 73, 2);
    T('-5.74653547585286592427339552172284702606123532375327888E-252576126268', '3.2211273E+38039171543', '-1.851032230168815721191676697871920558936966e-214536954724', 43, 5);
    T('-2.37924364282378558776257892025325E+73213546', '-1.216551056969524818130681292737893632593017E-249383060860478', '2.89447136846529893880745642574659599316804773338897664824951026585583e-249382987646932', 69, 0);
    T('2.68437081668602835098409807020656489E+81', '-1.0450E+29955293492', '-2.8051675034368996267783824833659e+29955293573', 32, 4);
    T('3.96644648765711869523124092296830625512327003E+797', '-9.1037108948682894489212563515908351562226E+31057816', '-3.61093821035961716373711662647172786233793087456262e+31058614', 51, 5);
    T('-2.6505235126420580222315907213566E+8787347', '1747573469.0115012122932260469', '-5e+8787356', 1, 3);
    T('9.6257024176800808189552212E+395239533549397', '-9.594245315672E-5374963', '-9.2351350330879760590764e+395239528174435', 23, 6);
    T('-3.642670693599354405819738781671268443965452316335658204E-6262338899803', '-2.001711268691306671549E+86403686', '7.29157497550940574439607280891044212818026949980842075269571879e-6262252496117', 63, 5);
    T('8.6661999528586518528576822411440167476428836188344E-5858130', '-2.11955E+3562483', '-1.836844411008155553472450039421680069746647397430045252e-2295646', 56, 3);
    T('-7.266556E+57047988', '-8.225761639030729424185616951911416940880E+315731498', '5.977295759266858108169253997561361824025320928e+372779487', 77, 4);
    T('-2.937052476179629E+2998015427644', '-4.0912389908238249251403E+982420', '1.2016183608641761444856525609860826949e+2998016410065', 38, 0);
    T('3.91778697460908E-7', '1.696672642798136145E+7854', '6.6472019801301020736350639931966e+7847', 76, 2);
    T('-4.24827802736269400893326341273254E+755', '-9.0834546964342926676015252292673212E+3364374243343693', '3.8589041e+3364374243344449', 8, 2);
    T('-6.8828164375842752898611007675015814680E-93', '-8.60769374938358841E+6827496', '5.924517602794878368684737655412257391397356e+6827404', 43, 2);
    T('-8.10163316961557063747002758230256578447122150E+819202036', '-4.33231645294006483692196317627116513473966E+35491892672321', '3.50988386764105036524757571961054653779740131967655e+35492711874358', 51, 0);
    T('-1.34685204585056006918467214694217206039185056951E+726725914', '2.79204782388057943652749901369E+955', '-3.7604753237061626401187887046179162807e+726726869', 38, 6);
    T('1.42678711735057590045868695981179588596380985033009E+323743231694', '-7.73720968E+212792521', '-1.10393510956641718106036691856e+323956024216', 30, 3);
    T('-4.598216829399964048E+998411607', '-7.718380295E+1240169331027', '3.549078616817805918179163416e+1241167742635', 75, 2);
    T('-2.5062025758022149316101523173E+31', '-9.16016182228517770800809102395153656501080815631160E+1342442500', '2.2957221153776223345944907905124468462641183062436616739776774104619e+1342442532', 68, 4);
    T('-9.882018505917701066921609167280026340738085E+855', '-3.6677248573217267820368568366599021272422383E-4744204900280', '3.624452491466766381338137627631e-4744204899424', 31, 1);
    T('9.914231044449879525764334789557104902715295935400E+144', '-6.2541103797881696038833248221749695652857E+9331526', '-6.20046952827120974424941473889596272239144418449027816142e+9331671', 57, 6);
    T('4.35066474284862709721875558583805701129469E+3467453902405', '8.19617062547798400650168037703803163089431897237E+7741468', '3.565879056663864440053577349178769130801264986149053770866e+3467461643874', 58, 0);
    T('-3.67405398194988232122955897587588306E+23266512', '-9.771375747367963E+955893920', '3.590056197374577180852680034494635402413236337840678e+979160433', 57, 4);
    T('6.161247104276524E+539', '6.7001709509588535946043342791289695590761863134746781E+10520145813', '4.128140886975292080495889063452566027e+10520146353', 37, 1);
    T('5.752465145615436733919046361638719E-35430', '-7.205367413839500947598196444656625934338607E-56364611', '-4.1448624909464967613113204668249779850323910905559222202277688165313947724433e-56400040', 80, 3);
    T('-9.7980125789012670366999980229251255034283892159697482E-1172866060805', '2.3517856715945258180589149945180365553444120685663E+9148151', '-2.3042825593162928184478501659647763369672300129692787336433374718240996222202e-1172856912653', 77, 5);
    T('3.06773854624650691604138860E-2545985969415022', '-53907935.3792753405', '-1.653755e-2545985969415014', 7, 0);
    T('-6.031840423438413E-6285803037', '-4.964427695158715137210384143655158759159792766600E+6528650318419347', '2.9944635651e+6528644032616311', 11, 6);
    T('5.7696993808690592984630851729054E-17309812836595', '-6.820423032319240256270759746515425591E+88819689', '-3.93517905468373925243652706498165427025084962740990132529565029820914e-17309724016905', 75, 3);
    T('1.5711508997101313505775799641E+7027', '3.5168829717740627647879235806254546850251320237779E+6355361495636', '5.52555384527806e+6355361502663', 15, 6);
    T('-4.3146203600181603930142786174258E+632541', '-5.421044342686674745779268923510480779012959E+58155582481109', '2.338974829351719225464807947e+58155583113651', 28, 4);
    T('5.6300394668602429903040107764089970606692E-55363761709', '7E-3854544934', '3.94102762680217009321280754348629794246844e-59218306642', 60, 0);
    T('-1.02961748724306015735862992E-182771', '-3.69195423E+44', '3.8013006373089869861046594e-182727', 26, 0);
    T('-9.57029155129477627086350946325E+3162570704310794', '-3.3704703E-291532137', '3.225638343598e+3162570412778658', 13, 4);
    T('3.36209911544064904783918702720486996568490638E+6988702614941007', '1.0561380709654215066508082259173044074224197250E+885599897', '3.550840874176037e+6988703500540904', 16, 3);
    T('-2.6052231849023190272049176341E-7392311185992858', '-4.77223089008417705475725019728564201E-9617180489', '1.24327265585543285089524385575764028658930602397830562725887685e-7392320803173346', 63, 3);
    T('9.82822152477074404221675860030282774E+835345', '-1.373798090787745611266069187E+130820326553', '-1.350199196656907422576513064229940069754377906163188047058284738e+130821161899', 77, 6);
    T('7.3622424303139E+408', '-2.348107475414455928045986169393E+8672', '-1.72873364e+9081', 9, 1);
    T('-7.2300197670008865471E-8478885907', '4.831858625165862014459E+3479269637603914', '-3.49344333713029097e+3479261158718008', 18, 4);
    T('1.87451118605E+60881', '-7.7420599111073704593483922999830234317841562565862709788E-9', '-1.4512577906440034568649888160401864647978660012132058936617082e+60873', 62, 2);
    T('6.5915100700497704202326075866528193263729929E+96369390', '-6.77155637811342367312807300E+584005607', '-4.46347820562443829501675808700411064950589829741917133523074900196817e+680374998', 76, 6);
    T('-7.1043741278455856079201918472438E+169037', '-7.39886272078576585952454562621648725903E+727886651301', '5.25642888890315919139597749641525065e+727886820339', 36, 3);
    T('9.55498916227867120262575324536493991172536193927562E+516953610', '-7.412928873418342704771884298947482301E+728844077', '-7.083045504625490424023177986934886932535920429030624974638241e+1245797688', 61, 2);
    T('3.026950130995903160822189855447068E+917727', '5.754512208693383E+76651965488', '1.74186214839219596996e+76652883216', 21, 5);
    T('-2.8919E+50397488563053', '4.17421876453897047796870715E+667374', '-1.2071423245170248725237704207085e+50397489230428', 72, 0);
    T('-5.0417571914791515E-213155743', '-9.92367832300150173783E+615879300', '5.0032776550918587444487053295463851245e+402723558', 55, 5);
    T('-4.18710323133818014721020655497E-56262411710', '2.4777106198104066691482412977380712109487918E+431793062909', '-1.0374430142529078913966324608831972586408664798373238223e+375530651200', 56, 4);
    T('8272447357.092144298518774138263435748902713715', '-2.93048620389721546081182158203379418558317984650620940E+8536153694', '-2.5e+8536153704', 2, 3);
    T('3.2165391452585444435E+36586775391429', '-9.55011818796071302693185188175E+5778697', '-3.071832899342123116550327177440208226094148155613e+36586781170127', 49, 3);
    T('5.7002100287E+5400', '-40700.14814829294121702158579703301595', '-2.31999392644475158310686026504625080223197057765e+5405', 54, 2);
    T('4.12516847221900338262091806213992258434702E+8297', '-6.4169364895463807012475011944486348838472596789901160E-528109', '-2.64709440949084180477696997855186807282602117e-519811', 45, 0);
    T('-4.376581616525667193900160698943785284585082125828074347367E+5163', '-7.709536524156201925017207981549984579E+362175950120', '3.3741415823555223688338112081282209598215755358385629e+362175955284', 53, 0);
    T('-3459471.752589996944', '-1.954839616713154152639260909882348575669493278137E+58025174', '6.762712434863013277747911638554e+58025180', 31, 4);
    T('7.88644798599838159540273176480444091632361342E-542461', '23.850962143167477568747833524672749652980961683', '1.880993723581067966259697554e-542459', 28, 0);
    T('4.355507265E-74', '-4.607364958793094717357E+257188', '-2.0067411550529749673281535098605e+257115', 68, 3);
    T('-7.01347375607590806120758420195383591588441E-718711', '-6.36E+121268', '4.46056930886427752692802355244263964250248476e-597442', 77, 4);
    T('3.028711374E+133989', '-2.242350317778894777249E+1530609094071963', '-6.79143191194945302900324273e+1530609094205952', 27, 1);
    T('-3.212493791159940275637463968659447717035648439359111081E-8408', '-8.25976554483709827082691372538990021751151653918447191315E+47082', '2.65344455292259794794303700305542366706252278834630404899533510596686e+38675', 69, 3);
    T('-98.4320600654618854097986611727747', '7.3961207515653647474403477607E-4489244646676194', '-7.280154020694910858579390161457448029640890338387902749506e-4489244646676192', 58, 5);
    T('-6.0735727423871578E+290622231584', '3.68730553288297E+907027497559', '-2.2395118377371360366042538322666e+1197649729144', 73, 5);
    T('4.211007313386992518833060856933101135883364E+7894298937068', '3.72581280402169490003475474056497273518996E-80801', '1.5689424966046254716429981209963359519609421737153065620655397087300002692e+7894298856268', 74, 6);
    T('-7.72167799160824176050413791950570514542517486423E+872116876858', '-9.1799154940646182887339027894326352376E+41226', '7.088435143534226213332173691e+872116918085', 28, 2);
    T('-3.1859139E+2329859', '-1.00232309730434307957521780233015457135002998E+717125773', '3.19331508799295914758749249197109184e+719455632', 36, 2);
    T('-3.7847496607762082614758856789091795642512528806635696336E+195364', '-8.1771187974188913098E+8981242', '3.0948347594657904927968807310558573303840517624187e+9176607', 50, 3);
    T('-3.724E+59326813533', '-9.9470980670763275662042467070275877453282E+3621912785', '3.70429932017922438565446147369707367636022168e+62948726319', 46, 2);
    T('-2.86194414219E+167', '4.49967667252425792991281050957136126E+54297754359405', '-1.28778232946797909031906236153072262858412975594e+54297754359573', 79, 2);
    T('-1.656976751675482142172101816E+893', '-4.2342868655638534494781460793480950688522361621120E+8903', '7.016114896164152834278707360917668703705905e+9796', 43, 1);
    T('-1.27508353662152638085106921893524419287554644559135065703E+168041', '8.338E+7', '-1.06316465283502869635362151474821e+168049', 33, 0);
    T('7.6064817632447992377537267481310318246310794532629981E+66472104', '-7.52549689203401540036338370458941991004780529841045023022483E+73451', '-5.8e+66545556', 2, 3);
    T('8798837.530118889501235067842967649319758650653401003653389', '-6.65679548880670902457884672E+2744', '-5.85720619772385893767421282420683492937422275347e+2751', 48, 2);
    T('5.87899798640801619962467409188262609562993037081E+49492898', '-4.887341598663428069985391239646481193404840041022625108048E+5933597275569', '-2.8732671417430428461e+5933646768468', 20, 4);
    T('-7.443537651346955064337283423827877843130E+8311530302', '8.6130615334951184986108975119649786933918333948854E-85895370262', '-6.41116478179390574876115866952440952811341599300245109e-77583839959', 54, 5);
    T('2.5436723787077916333E+431097', '-1.03645419603300983865795195884594905224715173969878721610764E+3501678', '-2.6363999102449579110473114021861146583808673722250496468268615964859065842084e+3932775', 77, 5);
    T('1.69217109371248621342267772699079913968927E+67645506892', '4.86830089730923910421E+4380537231', '8.2379980539212531660954606183820323009e+72026044123', 38, 4);
    T('-0.6362816309703879125699', '8.286209435663426443E+449', '-5.2723628542861425859275835681729437458657e+449', 70, 2);
    T('3994.9193', '-6.726043067786837464787190304393492934438114762582E+54458695', '-2.6869999264132845274041e+54458699', 23, 1);
    T('-9.3149E-5706749958243', '-2.66907922376618268926831874778594644957238696597785508261414E+63365445', '2.48622061e-5706686592797', 9, 6);
    T('-6.5939509035', '-2.9576655510121206151019379937879214152831259E+28714339', '1.95027014323472e+28714340', 15, 6);
    T('-5.22968484814507518847380818670386489E+8285', '-2.8768701E+9588171821', '1.5045123972051607271972163405463566456480789e+9588180107', 78, 5);
    T('1.055681134090286123993600180042900842461426776544586E-212190200', '-6.113999514124146216E+9191335', '-6.46e-202998865', 3, 0);
    T('-4.7468751560907956592164566771097236042785005898407E+41505', '4.54E+4740211447171', '-2.155e+4740211488677', 5, 2);
    T('-4.03005767457285593968670812166E+392', '-2.93671851136193070917068381036E+6347461267328', '1.18351449747743217e+6347461267721', 18, 4);
    T('-2.848548149521304373656704406711051E-6065114373092278', '-3.1156043172E+991', '8.87494891240064701994007e-6065114373091287', 24, 1);
    T('8.98301949057069E-668909713116', '-1.66532008478E+7197565', '-1.495960277961758e-668902515550', 16, 0);
    T('-8.681396159653599006211E+997', '-6.251708789821944367669916951121E+6267173891559053', '5.4273560679232876779549497109659626661172588662412531e+6267173891560051', 69, 4);
    T('-7.664794288386680474521829684990923173105E-927325', '-4.2803657982678318756E+79', '3.28081233227689719322444221e-927245', 27, 1);
    T('8.84431860150467E+60', '-8.65948984279916E+161419098862', '-7.65872870963e+161419098923', 12, 3);
    T('1.13952369580', '5994383.81', '6.830742393214884998e+6', 56, 4);
    T('1.5751858011557681166894740806400E+97', '5.75755103702054259450328598564721290861E+805', '9.06921264294442692195749511367e+902', 30, 2);
    T('2.1635929807697450383393E+1000602556271', '-2.9658546793905521495165598854470290202623389E+4063137280', '-6.41690236632e+1004665693551', 12, 3);
    T('-7.2081165017540148415020488955345014E+80818', '-5.3590624601189420821822100738856E-73', '3.8628746552713813476707050866611173436724645401887040175998e+80746', 59, 2);
    T('-2.2452305960672E+3772', '7.17228884088298515241786899210264079202283264E+578', '-1.61034423493818317e+4351', 18, 4);
    T('6.796002961307847348E+525120141125', '3.830536E+39125796', '2.6032333999396316349018e+525159266922', 23, 1);
    T('-1.4518872429689544514E+28884029', '3.0616075587157257167275497586746542764926715607356E-2541627', '-4.44510895747668634534552478494452415248325237109186626346814507844984e+26342402', 71, 5);
    T('7.822271147650081392793111932761558848241463961625429415009E-4810069126334', '1.19E+4', '9.30850266570359685742380319998625502940734211433426100387e-4810069126330', 57, 0);
    T('7.3982454831948873760606329501728247995626174155918E+5914965', '-8.2653152716502015744958835123636565623092645235E+8496407593', '-6.11488313756678273601594436211918226206951578780496327503205310183301037997628e+8502322559', 78, 6);
    T('-1.21074682863882563E+42', '9.6446308013115387798434856007155E-932333779773', '-1.1677206156080281165259739114832912145707007738265e-932333779730', 64, 3);
    T('7.9E+396', '4.11634392754801809E-96', '3.2519117027629342911e+301', 43, 2);
    T('7.755895466495055692405666797909455839222941446018038586476E-653943', '4.29428035372484492491374307E+2005192995', '3.3305989527313308898588915160018824938795936780486e+2004539053', 50, 6);
    T('-1.6662516305358680440267947106331743388896405772732213750634E+9537987336158', '8.4250498348020515838010230673778771447676E+4707504099131284', '-1.4038253024584864154656e+4717042086467443', 23, 6);
    T('-2.32160118289490683389113924668970755926348381E+63012490648', '-8.5686974059307161103497864155618338887976078860421402E+6570282', '1.9893098033477270198007181698984252290426026541571778261075715351559784e+63019060931', 71, 1);
    T('-5.362227582332E+86801105344', '-1E+748856841', '5.362227582332e+87549962185', 68, 1);
    T('-2.633224887874573370706922927365E-96246', '-4.5846299442E+866', '1.207236167076245656914288597504e-95379', 32, 0);
    T('-2.8563454988156575084380596584882300540438542E-9176', '-7.8134E-53200676978', '2.231776992044625837642993533563193670426625040628e-53200686153', 49, 3);
    T('4.00E+9', '5.1671431680912690812274E+851490812170', '2.06685726723650763249096e+851490812180', 78, 1);
    T('4.7993100950815065812E+6001981770249702', '7.554780405167964621418698041510687538591126196391854073131E+87', '3.62577338646465671007196023404333621254697658562943835898701363584752025897372e+6001981770249790', 80, 5);
    T('-2.20192249605445677872541549222961034442237161397093862998E+22609684698', '-4.7E+695271524211923', '1.0349035731455947e+695294133896622', 17, 6);
    T('-9.43778649782026837324282330261240802E-101900277', '-2.468531461706519829864499554657116130573032818450911305146E+1346', '2.329747289873832371355064702307485e-101898930', 34, 0);
    T('-7.18246028887550172632065059610884E+573720', '7.90367154751198039437694433216197107E+96402850', '-5.676780702631998247e+96976571', 19, 2);
    T('5.97618098796519080596401118700273595034464053325834685771E-96890062589', '-4.317625260920930340046334797569074827113286977046E-92', '-2.58029099974739102139751975653404280001129712376850366384174352179684222346e-96890062680', 75, 5);
    T('4.82931779070962906297546E+856750467', '-2.4712810356399028093845891180256491878667485359488E+550181152011629', '-1.19346014712590995169021437789e+550182008762097', 30, 5);
    T('8.52333603656577696762401753526599876363520937481412963594260E-90895883476', '4.60750830190287304265842097E+32399', '3.92713415483847472517766e-90895851076', 24, 2);
    T('6.386112536609001889515044108689828932950E+8050343071895', '-8.468598890200761287E+67932163838177', '-5.4081425540224161937331463012e+75982506910073', 29, 5);
    T('-7.863927430113351164566763262422988289314307006592713862479', '59771037.5545275490666598226180625916699825302408', '-4.70035101751384e+8', 15, 2);
    T('6.880999993085848881E-711', '8.1010401802661868381961659242454491321743367365293E-486', '5.5743257424399815606176400279555842911860269736177344050043e-1196', 59, 5);
    T('-7.62152270923769727517042073108983020413200682633E+464006975882', '8.360776996002E+714795551038638', '-6.37218517419013791196752e+715259558014521', 24, 4);
    T('-6.82790E+7373891', '9.3440460051565031434464258917946879E+3135', '-6.380021171860808781313785134658494951241e+7377027', 65, 6);
    T('-5.11203107430587282823389419555133704994830776856625573921E+860347424815', '-3.295552096206282752E-676171', '1.684696472280037478251298993116155368489475832229450540046837121238e+860346748645', 69, 5);
    T('0.0008567263669026396415840014696549756752384284431028034465177', '8.1986926801007164297241065801166978584603177384433E+93799175', '7.0240361931739523230575153093773454418536051146e+93799172', 47, 6);
    T('-7.47555967853364320039966137245116404080064329652E-51703', '3.59318258E+604566128', '-2.68610508126574866916115122813904145321272807458494386216e+604514426', 69, 1);
    T('-5.8698156041081894071506029773907942772E+9159577', '2.1039277E+249785478081708', '-1.234966764337545349055073167583496580481e+249785487241286', 40, 0);
    T('1.46E+9030693893', '-7.43789522E+88979', '-1.08593270212e+9030782873', 27, 5);
    T('2.127646251257425905431E+302528', '3.686828754142626152084518E-767396', '7.84426737777964448198e-464868', 21, 4);
    T('-6.6170158E+17346673551736', '9.80938533125E+81616841010297', '-6.4908857e+98963514562034', 8, 1);
    T('-4.9228572661054886025894214241766212306385321E+320', '-6.3470714330096549804162423175601463071807E+589', '3.12457267224821559644986112372e+910', 32, 2);
    T('-3.36580E+3532232833064', '-4.098131799805568515873942983081172016360E+682503', '1.3793492011785582510728517292454608772664488e+3532233515568', 53, 3);
    T('-6.0655979183931440435315774147E+23847291811', '5.713928523643246221150207607820288536793189822E+7167478316075473', '-3.46583929588576850180055890926688e+7167502163367285', 33, 4);
    T('-2.6042539681536E-59111065310', '-9.2765346095256996884430639481138498503303E-500956774', '2.415845206757150972762736050109809421090917722264113408e-59612022083', 69, 3);
    T('5.0570918502357635E-680990888540', '-0.000007266919025014959497872098235081640190530373027741247565', '-3.67494769777263720683231947306073151244102668931101945711197188702908775e-680990888545', 80, 5);
    T('8.32211042096242752826824293205E-106642750685', '-9.118027159919232522495597205835921785E+324926', '-7.588122884618229167606726788698092067996980965484811251082286970925e-106642425758', 80, 4);
    T('-4.85434607221831744617451900603348390542416500E-9145', '4.5197666233449994736160185287168102E+46584561884413', '-2.19405113553782456021034966217163454859790383938307e+46584561875269', 51, 1);
    T('-5.10663358885732160811292536183272880945353965496443092947914E+8126587969531', '1.77E+885431', '-9.0387414522774592463598778904439299927327651892870427451780778e+8126588854962', 75, 4);
    T('-6.9006363857E+33471812228', '-3.8642608093500203509840562440785319414661E+357750381041880', '2.666585874483528120103606331846359768352063226307477e+357783852854109', 77, 3);
    T('4.707218650156064603374099831E+2083248815176909', '8.8071302118317650211252980665412768287E+56913358', '4.145709e+2083248872090268', 7, 4);
    T('-6.7491470626415775191204247E+913385', '5.19203288415401563185896639657075700554024590654850599964E-178405935378579', '-3.50417934892265525342702134192119267e-178405934465193', 36, 3);
    T('-6.374223750532615E+740', '-5.53978275789609605906715E+53933', '3.531181482817236692558405957731255009725e+54674', 51, 5);
    T('-6.4518187329456273553817978705427E+87924658', '9.933666E+4033802882', '-6.40902123856250583e+4121727541', 18, 5);
    T('-2.7220213E+94065', '-4.48275923293983881254458819740094661742991160537698895E+984586085', '1.22021661148339028663130762730539813328071706469533584517646e+984680151', 60, 6);
    T('-8E+25698', '1.1672664127266516887989389187233379368481480E+11441479891', '-9.33813130181321351039e+11441505589', 21, 2);
    T('-1.063E-7', '-6.197505297717411301783020471854827742457E+790137168358', '6.587948131473608213795350761581681890231791e+790137168351', 52, 5);
    T('-1.2945975097672990989711010442203186E+609', '7.517447031125E+322029892', '-9.732068206302000801170191087046421399971616425e+322030501', 64, 3);
    T('1.4878105601763282190634593887749199E-1610374396', '5.9464836824911193495515559009054644161392397E-99985739', '8.84724121872650735190671691187124911066694605305874515359314016895548e-1710360135', 69, 4);
    T('1.879225758899345326752093425759349E-910808755', '4.30976697597364498885866083025E+5541601', '8.09902511610340958144321778268935288464802532488090134903951e-905267154', 60, 5);
    T('-5.055729051282674871580588127408790045394164844902E+13169276086317', '9.409230225896655207433176495276195255029E-34044776', '-4.7570518603272765201727866775682401620137765789851823304777941237204825e+13169242041542', 71, 3);
    T('-6.95188206081748E+9591724', '-9.77480264566093938239461172227789314359848139717365E+487781904', '6.795327516e+497373629', 11, 5);
    T('-5.276231E+1274', '-7.6721906190902282907212565E+135444', '4.04802499823530543045805059042515e+136719', 42, 6);
    T('2.499568655510272E+9251322508977', '1.68251051590E+9117780126152', '4.2055505481100571205e+18369102635129', 20, 2);
    T('-6.71E+9614813112573', '-8.49092659358347358963625021891664024482', '5.69741174429451077864592389689306560427422e+9614813112574', 62, 1);
    T('-3.149263575E+925384131', '7.15059771763901762651633799337753595000303793E+72', '-2.251911e+925384204', 7, 2);
    T('-9.726254483346999288256841834126307267367871501219716696537E-473054889316', '-7.147259158747738616444175510699284725153218381376462E+524900497', '6.95160614364130952253750390030027836199861648856724757423874381488759e-472529988818', 69, 2);
    T('-6.0912128868345142336252E+98978', '6.93131207470911605103750055528576125729615050897823138812E+469152754660', '-4.222009743213984097579203004013647236800188999710193235872029299308668e+469152853639', 70, 5);
    T('-1.924306805457266414642963974696E-470501', '-3.44587192021082591190214E+529683014475', '6.630914786795790835217288490888561879331468668189e+529682543974', 49, 0);
    T('-8.98762651915895E-377118114', '-695494069994612232732333755177148024413441159677319.12', '6.2508409474013678724525663011817935773921108652425033e-377118063', 53, 0);
    T('6.21142276334319371716026846265383326578433E-3199807357', '8.3638530471375451635483031051825469302980E-11304424235792', '5.1951427206247481837833859962215284e-11307624043148', 35, 0);
    T('8.04641848126967740152587E-8175196710', '5.14359E+5142904576', '4.13874776360738999857144496733e-3032292133', 43, 6);
    T('-5.828735508276405397148104E+800031787309', '6.210907229E+2854147971927', '-3.6201735504282915611281775117243816e+3654179759237', 70, 6);
    T('4.883451312961E+18513250075', '8.86207217956214419391430612590575E+1769141', '4.3277498020837904013863222e+18515019217', 26, 0);
    T('3.2865679667E+478', '6.9962150277806647434078778754960302671860224519797045E-647596379', '2.2993536198449083339316406418030903949147e-647595900', 41, 1);
    T('-4.1011153694362165102031801162665943147916003222164265434565E-636793909851705', '-814689700.11857106343010010236', '3.34113645047765400843605579046482419123497786073e-636793909851696', 48, 2);
    T('9.3460668061155E+2129578079624694', '-752941.94003647805', '-7.0370456727071e+2129578079624700', 14, 5);
    T('9.8699322063942601524024429264209033401221859557338E+45881860901', '-8.36239806292604869474040312718823E+1202', '-8.2536301963962782943281e+45881862104', 23, 1);
    T('8.92438790878264606326225013975988438716E+2270939769066', '5.125220106113508495639319967896953734344633E+5321', '4.6e+2270939774388', 2, 0);
    T('-9.172525260E+7982428378579', '-7.37586559548809189857627471194714E-241451336440771', '6.76553134889794649688922378320343654347564e-233468908062191', 50, 0);
    T('-9.89582708795E+30823123715', '5.69810208913739018243907026282839189293E-8091084', '-5.638744e+30815032632', 7, 3);
    T('7.60526578467317572572301589088946799E-3281178949', '778856504682364577058949.1445683162829619493843453799519622', '5.92341072623093039889e-3281178925', 21, 3);
    T('-9.0073076676301222259896032869180594167E+69801537', '4.9085639709E+900132878268', '-4.4212945892140530146055874292e+900202679806', 29, 4);
    T('-3.9286771955033556099347889828975531279432417684792158038699E+91699115556', '-9.055009302228106133947789859024580727947085942656397E+9565273555849', '3.55742085507343129867685136955297541765265555270143531e+9656972671406', 54, 6);
    T('7.1077586988292068335607449543209870957574928E-9', '50714774024.06110061', '3.6046837622867778632742103e+2', 26, 5);
    T('4.585415854675526493758377453189031408461346E+33075844339', '-8.02503504051169E+3985', '-3.679812291e+33075848325', 10, 4);
    T('-7.265835771051065453508732456883876E+7235758', '-5.367538014605450074054316655114673525143305944E-745907798045770', '3.89996497089966953625596416755815647499282169786336986438884215935601e-745907790810011', 69, 5);
    T('-6.43854214371045773626629475323993201402890737174141188E+8100011356', '-1.98E+5677', '1.2748313444546706317807263611415065387777236596048e+8100017034', 51, 6);
    T('-4.4214752190409301175155262083764340568674251150195864818E+81345535224632', '-2.4358794510134224798335072805092130580724754521056E+8058547726', '1.0770180629226872763145014534740602262885755454671343422707943604776682312799e+81353593772359', 77, 6);
    T('-8.1015094513204493362624537554339190167545446562E+509813111205733', '1.134417402774631130287992242706430615816028E-8745', '-9.19049331e+509813111196988', 10, 2);
    T('-7.0451555953266164256551152030175497158235757474143040E+3240257257178', '-5.9741E+97', '4.208846404204073918850622373434714375731e+3240257257276', 40, 0);
    T('-9.797374393743683E+19', '-3.6251855628381496987763824444429999012676676102427E+5972169', '3.5517300205919769137220556476795708806212078379e+5972189', 47, 5);
    T('-9.609644825714E+26036154', '8.34048676910137918775094797017312E-344449761454', '-8.014911552463114596507198529047255381280760768e-344423725299', 76, 2);
    T('1.148510984858909E-65438051979038', '-2.983765397847828827123327783703076863E+6288831', '-3.426887335670144322603059892291e-65438045690207', 31, 6);
    T('-2.30935497920304771936729142490661140063762217129552268E+747', '4.14078412297336176235E+457476753408', '-9.5625404321934580428132183450145468288775885198269023134005304e+457476754155', 62, 2);
    T('3.6300560242876591313932847497006575846785', '-6.5611882005352262917284861550339766236322E+877288', '-2.38174807538380039231660047286350579976e+877289', 40, 2);
    T('-4.5562433E+6547643071863579', '-1.5694881494326358030061015829272209403383771430981673052277E+538593263168', '7.15096986528184567878667019633154479703643059111416602672276309941e+6548181665126747', 69, 5);
    T('-0.00679830941490169314977718153599885419157769997', '-6.0298090682122233021819813681987605356365E+93705', '4.09925077584867633563652e+93703', 24, 1);
    T('-3.778785E+3821365783', '9.52218359045618493572914695902969320984991926424924462180E-8', '-3.598228451886197479235926459157701925598272716695608184e+3821365776', 55, 4);
    T('7.254803786051893807E+31166780', '-8.65034396281299780843219761200726477961859635030430859E+17086678366', '-6.2756548132066878990713442845253e+17117845147', 32, 6);
    T('9.0418193519004145776117443880382920356899523338298581E-469703158', '-9.596543261E+33435007137', '-8.677021e+32965303980', 7, 1);
    T('-2.659515050559495982732821613581598497218259922371733488232E+795909', '-917921325.248', '2.44122557972657427634104374e+795918', 28, 6);
    T('-4.4330594779837040998284648963081457263955E+31369076711', '-3.47635819564163620344327871588683311203894443613E+870395330142350', '1.54109026480554832769854889438906878856213920114745322735881378024e+870426699219062', 66, 5);
    T('676036.53210219931263521132081170499323931964544604050882080', '5.133', '3.4700955192805890717565397097264817302974e+6', 41, 6);
    T('-4.18293953761874944495389001574139653707531E+11493441933', '-169711333185135962736678269824136163.165', '7.09892245562094152462497828e+11493441968', 27, 0);
    T('4.7740793985377530671885406795719411334381617688277277154E+64536620715', '0.000265998963350856030955865425842230667431', '1.26990017096572e+64536620712', 15, 1);
    T('-9.57836821469328370839748061366824444711489623E+702034244', '-7.276901062495058468402534500227042611099', '6.970083784e+702034245', 10, 6);
    T('-1.7512533362765802342757306558625664415761596767507029E+9255', '-6.8600804218238175339803624695800227E+9952384', '1.201373872584461031086e+9961640', 22, 2);
    T('-4.761293110615872102308E-80975128599223', '7.9775476E-123711944725707', '-3.79834424274901850116741398608e-204687073324929', 45, 5);
    T('-8.1954022316972E+9012998900601', '9.10347080691718879262501413622445700726472202884754E-879986026647', '-7.4e+8133012873955', 2, 1);
    T('2.78E-8473004', '-9.4744048158072142384E+2563', '-2.6338845387944055582752e-8470440', 45, 5);
    T('-7.08224503050784040021341726678787816864276512839E+3619808', '-5.785288733111647811845E-901197576', '4.09728323801e-897577767', 12, 4);
    T('5.9306E+5165512', '-4077792.62149271705135738098963675982895651018560577761', '-2.4183756921024707744780083697139767841609479306753624693866e+5165519', 71, 1);
    T('-3.168035773792227322490918846306224901E-329269358', '-3.9562465517256E+32036684', '1.253e-297232673', 4, 5);
    T('-8.55122229862853465140823195390214328907578666E+806', '-1.517888528543992947646E+56', '1.297980223211784750503017455581630331857283721345083e+863', 52, 4);
    T('4.76617234532744721849105076726639013918766046367E+563531', '-5.0281043434219104147916032632E+859', '-2.3964811871038330867583494072577130672e+564391', 38, 3);
    T('5.9102E+4205097817001', '1.48981270093717317717030934539307321271064977648603083845E-427', '8.80509102507888091171196229314214130176248230898773946140719e+4205097816574', 64, 5);
    T('4.29206740294E+56803', '1.2536936728158374458160809046994E-8074369294694', '5.380937746364981502965029337522079399376236e-8074369237891', 50, 0);
    T('5.446253649455799821602492342658124696608622200553212950902E+698720', '-6.9524887624272089243846975984278526416395854E+46042826398105', '-3.78650172951696238389231789702e+46042827096826', 30, 2);
    T('9.342047481266060977973969524953206562831536277241680128E+2919337', '1.27067257180553656E-4227867966', '1.187068349894978082934068543229532681856286141687519406888983838692947968e-4224948628', 80, 5);
    T('-6.6685574182218928647297815175510018570631812E+717801', '-3.2675273138118750402739347426050437E-975', '2.1789693507762834152552321031456262371855461952e+716827', 48, 6);
    T('2.775951471134E+7638', '3.870708144144629024755538040554016563087352025208722504E+27975718300', '1.07448e+27975725939', 6, 1);
    T('2.0103992692909923849884E-5199899700556', '-4.130885921131E+43254611804', '-8.30473003736621039954391331075e-5156645088752', 31, 4);
    T('-8.712208295915498475557771706492827934E+289', '1.007688214750455E+965513265150', '-8.7791896242451924304657990332101975506178e+965513265439', 41, 6);
    T('-5E+1727269313', '-3.520913114696435277092834537555E+1329417', '1.7604565573482176385464172687775e+1728598731', 75, 2);
    T('3.19558307253676815006146001683588730539228327938544473245E+39424433586', '4.8971001312424008137491961420026270512362363720800379544E-6765280265920', '1.56490902839e-6725855832333', 12, 3);
    T('-6.6299711108024057720934984740151620344336770596670551197646E+2410972', '5.2E+4616143771995', '-3.447584977617251001488619206487884257905512071026868662277592e+4616146182968', 66, 6);
    T('-1.48466209339036506192069066777104015389009931275E+918025700221', '6.40848750419707028309153587926338253285179048E+545', '-9.5144384734472182723566439058136506114000319975501596044273357e+918025700766', 62, 6);
    T('-3.63010119041214757058445980878716522311882970358985343E+2339089387', '4.6206619370388648057815750071578E-79459551151409', '-1.677347039813688279955946663101105378530576e-79457212062021', 43, 4);
    T('9.708E+2564385885213', '71800466.9023720679764516678342441506305717030', '6.9703893268822803591539279133484221e+2564385885221', 35, 4);
    T('4523.9081925561719073068622', '4.276968680580391674931697048848191376490686370796E+849101800', '1.934861365338379504136221564305424e+849101804', 34, 2);
    T('9E+61', '-6.2397189657766481007836011504224870204907303255E+1210071104061', '-5.61575e+1210071104123', 6, 5);
    T('687.76202402783912560242494277020020842494192', '4.90333819187810380071481810711E-44528064309', '3.372329799339089679056577694234000951854195939088444827433340457766089052e-44528064306', 73, 2);
    T('-7.9437777567001730877585763852646415E+435218286', '4.738299352048141071123433E-55', '-3.7639996997386865769988349696914800489326e+435218232', 41, 6);
    T('6.96467857894939402147559223640572064352119075E+77884146159212', '-7.391659381549688650755980095268037699E+1201552522', '-5.14805317575694422141171460103675283342528436e+77885347711735', 45, 5);
    T('-7.06747178954077149489713473E+72759445365030', '7.58458674191006338827747076565790237235030052368820859836E+16518615863979', '-5.36038528e+89278061229010', 9, 6);
    T('1.0141211630378695256475380182931E+836', '-8.8267628978945292190471490627584015293439769E-810744814', '-8.95140705587231554690413043582853066348791360330051699626802763031450382939e-810743978', 77, 1);
    T('5.524565E+188596164447517', '-7.9529156634901096949787258432613E-873', '-4.39363995224692378670401445382768638345e+188596164446645', 63, 1);
    T('-4.272156022104105791595044562935238489026924936013142113E-182198025822593', '1.0296551844608257240938905E+63536621624489', '-4.398847596985030508328460682715027636855549749299569817265057e-118661404198104', 61, 3);
    T('604425276.65650839087580138', '1.369164128015098012085646192078E+5245497', '8.2755740686369268664997975965376178701862414763365746764e+5245505', 76, 1);
    T('-9.515907057192721625995968880713440056160222147876403547664E+735335919070359', '-7.292284458E+3291038624', '6.93928e+735339210108984', 6, 0);
    T('7.790161438050091826829436766625752848627458092946600001492E+7623599179684', '1.4687711795417407926808125513276799612259778154501801325966E+667736776', '1.14419646041854170661113e+7624266916461', 24, 4);
    T('7105023749.932743505223520576660245848861851672422259754488', '-3.96862638673296632008390E+1175833998845', '-2.819718473234749471229710733549472927e+1175833998855', 37, 4);
    T('8.9812703103659358006210391647810589945309474178E+61858954823', '9.306373230031E-6238024565497356', '8.3583053588061755818419600267719e-6237962706542532', 32, 3);
    T('-6E+72582854539', '7.2458E+95430459636312', '-4.34748e+95503042490852', 80, 0);
    T('-21.7421580055', '-6.4229961472932654258719792633746713349111280084068194E+54672', '1.396497971031679280353588207593116e+54674', 35, 3);
    T('4.66712950759E+425319', '9.29215152574500469178383628549147426E-23363855051610', '4.33676745748019509550673355618269989242169596334e-23363854626290', 78, 0);
    T('6.08900961412E-52', '-9.105120501428923954914485236944055792E+990510906', '-5.54411662709e+990510855', 12, 1);
    T('3.232254047789846062240562719532206670887177595530747424E-956835895492389', '4.377256492787722589E+616674866', '1.41484050170275014544538864978651180594733962372682328097705517e-956835278817522', 63, 3);
    T('4.361421028019330321479399304295582294289E+969999582220202', '6.7238919954438446291919621922590183615717942973794445306117E+39406690', '2.932572393905963915278737575283028181e+969999621626893', 37, 4);
    T('-6.2042315464499586160850407124749576505510088967759E+49999367672', '-7.5462636707575504085686726848116977042089750609194762253342E+2857507470702', '4.68187671239432e+2907506838375', 15, 3);
    T('-3.5145037098043344174416125865044488E+7419890551049', '-6.0350604929541712839880494246513882E+43500864351693', '2.121024249138101021042247665604159441365003661480602183809796839582416e+50920754902743', 75, 3);
    T('-5.143110386902505254839756307578368E+408488298474', '-9.34865369354203348609611383E+217894969396027', '4.8081157914810502633999492e+218303457694502', 26, 4);
    T('6.67770522452368952E+7584051290439', '-7.9250390018696486748546899123314807570730988025230789266698E+783081116728', '-5.292107434733895859399223568557931970110022206203735624599312237507e+8367132407168', 67, 6);
    T('-4.7212603386127553747915079575743862541E+459301430', '-9.4226626657201829067297106662E-2856434562097510', '4.448684352779183895691585639236379197016206133796839914413543348142e-2856434102796079', 77, 5);
    T('3898.349476214961599382925219116322978307645898673503234570', '1.7179528650324557092862111862744E+344842', '6.6971806515612663328e+344845', 20, 1);
    T('7.28444341785824631120666605E+363376080', '-8.134841655750482843E-6119', '-5.9257793754550682783560974e+363369962', 27, 3);
    T('9.47114513980596301652403382393522539155201973483029426389E-68582338239163', '2.8935647691527394257653501532422804864E+9617270658422', '2.7405371900074731350725158901548837392264e-58965067580740', 41, 5);
    T('-7.3961327358366481928120597511429052447852989308974022600123E+76928', '3.2697579221554582181E-9027537', '-2.41835636063152033423184591691680110153354858562513784977690669317010250421e-8950608', 75, 3);
    T('-7.95337276873711601860900828239955458566192E+708', '9.09E+78803175', '-7.22961584678203846091558852870119511836668528e+78803884', 63, 2);
    T('4.4075054758100398381278459404827174523928621434778989613262E-629206118', '-7.454227549700022701133E+10113', '-3.2854548743236905940595122272260679589049569585800236802122544646e-629196004', 65, 0);
    T('9.9598297233614219693169941E-470602566', '-1.1362426532461256361E-6925939119', '-1.13167833507518076023792429865680071554e-7396541684', 40, 5);
    T('-1.08628632E+79619831666265', '93006717741065012389916.0424890793957880427531855678196932158', '-1.010319251502202251897963e+79619831666288', 25, 1);
    T('-4E+9', '7.61160430613556487765671213636E-575026436256', '-3.04464172245422595e-575026436246', 18, 2);
    T('9.04864759739E+71945161116', '-2E-8800651', '-1.809729519478e+71936360466', 19, 1);
    T('8.629650838718232290396290858753286118813E+972317', '-9.2394E-66', '-7.97327959592532354238874897603651117661608322e+972252', 77, 6);
    T('-8.190072137014482412873010284388213986822410564896747181929E+424427692065257', '2.856112294761361075994445570E-6356519', '-2.33917657255095178101628631090976882e+424427685708739', 36, 2);
    T('-2.1655746310541325908762501565162361E+6665991314480', '-3.0597950407676712272319338625408781578207E+206', '6.626214516511714208e+6665991314686', 19, 5);
    T('-5.7093600485160635979223347311736896040836991477514226742943E-22', '4.71106993971491930464770E+20405746', '-2.68971944995733404906525e+20405725', 24, 2);
    T('-6.581449862546871768E+26328797', '4.7478688187189674761495315339271165341414279190E+3665722595459246', '-3.12479e+3665722621788044', 6, 5);
    T('-7.017280E+61', '-5.560579280657462219E+94721512', '3.902014177457199648014432e+94721574', 55, 3);
    T('6.694235153885244132072291642384E+610435575943', '-9.6326219756129082318411741683324282217952774613019E-628142', '-6.44830366532354610865361942092196433118079613843549865222604373e+610434947802', 63, 6);
    T('4.141502763991679056166980133E+506462487771', '7.9747572577610340233271814038756679045804429984463270992894E+58242126069', '3.3027479225180025351883765158533931512459199976147585919840562242182216e+564704613841', 71, 1);
    T('1.45552337731640384836514669326107466E-36044611584', '5.964615809990800E+4406270986521', '8.681637748152626951232285704189267025048372713128e+4370226374937', 75, 3);
    T('-1.186198865922E+1477050', '-6.426510759004084911157952E+633526', '7.623119774166176971780954786212111744e+2110576', 58, 4);
    T('1.38783754790583E+89421482978', '7.58373301540720884762695385997E-5457944338933919', '1.0525e-5457854917450940', 6, 4);
    T('-6.22930474589443724742235601895023375', '7.2524733163386227402932544E+160775643333', '-4.5177866448940950932943675926443202394560529636989114428216e+160775643334', 70, 5);
    T('-9.212442539013724869E+4985299148212466', '1.0021110778240E+5108891', '-9.231890722162710998402583205056e+4985299153321357', 51, 2);
    T('8.286911830E-22890093615074', '-7.38E+6690550347709601', '-6.11574093054e+6667660254094528', 65, 4);
    T('-6.0671489474283E+88479805676969', '-1.941309835462468029751393881586809053859074800527241E+196751456933359', '1.1778215924858319167255909117594e+285231262610329', 32, 5);
    T('8.493390068283808560E+67333', '-5.441082302283662920499701407684787508E+68457690489', '-4.621323438693086210068464861798800783208266302055147e+68457757823', 52, 0);
    T('-3.86879355E+146417', '4.173904971856297620490063933795479243430446593971219809E-4914063070052', '-1.61479766334305757611e-4914062923634', 21, 0);
    T('-7.173770454559859078641966202157543165940512744483190352306E+75971086252', '-3.4780283350143483498528202305667202670128038E+138446', '2.49505769099e+75971224699', 12, 0);
    T('-4.906346424468301920136273696131928562608610057137596591E+58752608', '-1.5973064101569663620459982E-1418961079800', '7.8369385942539308480351582892e-1418902327192', 29, 2);
    T('6.6792E+365035899877', '-9.60819615227457596668E+7869353', '-6.4175063740272347796649056e+365043769231', 56, 1);
    T('2.61501E+2768885', '5.207901876042108012556418286506379313362581425005711196564E+78372062556103', '1.36e+78372065324989', 3, 3);
    T('-1.171045744998192136394568140902166126897722508917561439E-27800747346', '-6.205731213491879363942154605763756601255E+188797617', '7.267195132162132805591802122063419962900446265170458203555273423464e-27611949729', 67, 6);
    T('3.289645577962766033634491641098612227568176E+86370', '-8.0E+3', '-2.631716462370212826907593312878889e+86374', 34, 1);
    T('-5.78629E+35610', '-3.065166243831912631606037762100835316268494702160193981309E+339860369356', '1.77359407850221577411357002424664423821712282101e+339860404967', 48, 3);
    T('8.8837004746131822186424752909464E+98384', '-7.76361359465155951101771364875223346742409774880675910470277E+881776', '-6.8969617775519412901518597064961e+980161', 32, 4);
    T('1.8432165904476133482220323E+2656025186855', '2.4713604558020225200162694482279028765124926154165262324E+373083046310418', '4.555252593110463592732666088370467e+375739071497273', 34, 5);
    T('3.290629836587642231197146519291021984505469862812E+3355', '-6.857591134689924233933785784256816017443690995497E+5888223850', '-2.256579399492956944778892e+5888227206', 25, 0);
    T('-3.0386857981466E-573748274297201', '-8.36135923804948565738969316060406040005048009352481265405E+66', '2.5407543569862848552612809e-573748274297134', 26, 4);
    T('-17985249.850975731341865386338392754', '2.200219262236690471576E+65909721', '-3.9571493158256370861548959607840989035973156858161360304e+65909728', 63, 4);
    T('-3.19426900584144587287490232E+5905586060386613', '3.29574E+65', '-1.0527e+5905586060386679', 5, 1);
    T('-4.70154830003824082758882715952354691306386E+68337315334', '-7.639640140780589994348933752207496111279129124736492738E+4401891790287678', '3.591813711679088972239251288031350800383436752602181340743285693e+4401960127603013', 64, 4);
    T('-1.732688414886763085408882989E-64480953', '-1.73982575530916953358860706230944833247624E+6309295', '3.01457593014581029366256275882519738584096015822e-58171658', 48, 1);
    T('5.1829218924753143238684694289508846475656821383717879E+59738301212238', '4.9E+21', '2.53963172731290401869555002018593347730718425e+59738301212260', 45, 0);
    T('-7.0986999833434689916623758849951999634832217467433570651343E+119', '-5.1988985431752877773094994523330911744543400414792E+2631', '4e+2751', 1, 6);
    T('-3.6013522095487196492020E+56054097', '3.053894270769904E-312622258', '-1.0998148879765369694074795325339216608e-256568160', 79, 4);
    T('3.485640724186184011330000382037941556E+536377503', '5.30973359644392915559512138907E-54163132', '1.850782365834452854739288775160057329645083039441651563625019719292e+482214372', 71, 6);
    T('-3.03453746412354621816925973049266156430342714645623E+439090312446888', '-9.9298897028714207505982955623893384E-3005992312', '3.02e+439087306454577', 3, 2);
    T('2.72965501311659527618264490967767560615325730728E+21805120517', '-6.5687E-235', '-1.8e+21805120283', 2, 4);
    T('-1.01282545151677099726987127479692027193495904050411386E+67491196303', '-4.723159279312874500821556327751E+771106', '4.78373592965568881743595330467881203668551159949e+67491967409', 48, 1);
    T('-4E+5788425757040', '6.93876278472297854903530574726979233525670661E+335749755892', '-2.775505113889191419614123e+6124175512933', 25, 3);
    T('1.21764836684378297035372716157713E+361', '6.7682780151128712713835274312414002418300636827E-378', '8.2413826714468687369402619137e-17', 29, 0);
    T('3.253391342961703144317894E-2300552775555067', '1.58348790834239547500322909621613E-7', '5.1517058526856843103577298256086186e-2300552775555074', 36, 3);
    T('-1.6841464E-5013910792432', '4E+17011', '-6.7365856e-5013910775421', 23, 1);
    T('9.73124372205838379E+2636267', '9.4930620154319944349184128489774963617453759939E+47', '9.2379300140783503901904700941186e+2636315', 32, 5);
    T('2.919111221733450838856349015098703692553231316E+1858842903697', '-6.6492447660870430785028497841492657820062794761645966E+1898', '-1.94098850127371018646157826188464143983797894811403e+1858842905596', 51, 3);
    T('-8.8348749064458109383692E+4222', '-7.86520126828601605995626513089936358541929753908E+5212447501', '6.94880693193258896766978090426889564785099304661123115214151511084683e+5212451724', 69, 1);
    T('-5.020554835E-683726556', '2.31461080508690775892309E+4512', '-1.162063046862231734426033389264015e-683722043', 39, 0);
    T('-2.7361412123803584223191548661807242729976539846380E+4946204178616162', '-8.08204847824569522328797527159561821941001381803503E+277186710959695', '2.211362592178400737e+5223390889575858', 20, 1);
    T('-2.04603391334210450112944316998824742E+4450819443', '-6.94820609474140130977953899742421110345301203477881821E+342083643251072', '1.4216265306731210624737618983993819141454943680779972476e+342088094070516', 56, 3);
    T('-8.1065366406457473622880E+452051044363243', '-8.030E+186', '6.509548922438535131917264e+452051044363430', 74, 3);
    T('-4.083069679005763435175958695E+5286847923196', '3.664051E+380405502', '-1.49605755e+5287228328699', 9, 6);
    T('-179.0402587126325110934274118587570919138283367780', '5.82120957629273598090314489790564590411305E+9442783', '-1.042230868559905331501381193263909006680787864992e+9442786', 49, 1);
    T('-8.55925205201996775880604936296675508724131772470430781515188E+56939415', '-8.30398295302238011030928193572560462438674E+112533', '7.107588313059563848982420418700306576090508660979950317194439069033653e+57051949', 71, 3);
    T('-8.6625930440827203290176904227208E-806370410', '-5.68906261930386217519143803938246907E+252731733', '4.928203427333265773281911968103465301803118e-553638676', 43, 3);
    T('8.07085261222669006553141744200650589001821E+636', '-1.00138030395314765425740367390606129246385365307890620E+26127299', '-8.081992e+26127935', 7, 1);
    T('-1.4916607565944351947348682749843401655E+158085776095430', '-1.91770800032370747750550987689045136406336398E+1952', '2.86e+158085776097382', 4, 1);
    T('-1089273757.31974115107926195771137821', '6.76657632871E+544498277002684', '-7.3706540217647615676774218320270248420160914091e+544498277002693', 54, 4);
    T('6.903737006579275190516911336600E-17', '5.6955915937487651454156796340457849661241692024157031076973E+5156714', '3.93208664601251831067845333475308670138571981909055213908759883569641e+5156698', 69, 0);
    T('2.89133953516423752581906094097E+109586', '2.3496985210724800754724519672500500298534E+1928072', '6.8e+2037658', 2, 2);
    T('8.34264424501776377016983993545418907529924086851369914113987E+152', '-4.1995923238657269126333518492852948825947726097733183789E+926', '-3.5035704732119183373270425373478164634e+1079', 38, 6);
    T('6.023366329677408810049272833476830489E+25020', '6.967233442258488551019015044E-49992', '4.1966199327102210968e-24971', 21, 3);
    T('9.295276E+777', '-7.263372241515374401E+65', '-6.7515049675624e+843', 15, 1);
    T('-9.026769040E+437514205232', '1.91601971586978974692434606708344221537812E+68310983079920', '-1.72954674512430147588461223005945792864042255094048e+68748497285153', 56, 2);
    T('-5.16986837963077703480042E-810', '-2.34002E+384132', '1.20975954057036108769736788084e+383323', 35, 3);
    T('-1.9575835717076615E+1895', '5.39542E+860028', '-1.056198555446295101033e+861924', 64, 2);
    T('0.000647', '-2.5087201802E+972111563436', '-1.6231419565894e+972111563433', 30, 4);
    T('4.4371648140938481816996201210491846076352070699883E+2756462', '7.60923922011475770912716063227704032850114308669964E+1443504112526082', '3.37634485295161172139994892957919e+1443504115282545', 33, 5);
    T('7.88533515675234679326368077992590348229057933E+159543562', '4.06058177855922332286422513367806E+2343', '3.201904825534101637572037825914151776994e+159545906', 40, 0);
    T('-7.834830998235619354566784582428497116E+608683712118472', '-2.3796676254928243286855506634879170838457394153313578E-156942910', '1.86442936777089308269561850543885363791293723773510317661564219590533988e+608683555175563', 72, 2);
    T('-8.15556737904999640792773579050524053479E+97', '-3.9008238489163091184351E-384405411775', '3.1813431733642102327794209285322059474109353243180103371907129e-384405411677', 71, 2);
    T('8.1458209873315691625000272293286847167864E-6528495927', '-9.661E+8263453102426395', '-7.86967765586102896789127630625444230488734104e+8263446573930469', 64, 0);
    T('0.79388', '-9.26318302214204604317675552473629382923846482805578734046346E+15753873453', '-7.3538557376181275127571626759776489451558324576969e+15753873453', 50, 5);
    T('1.6241447E+37752', '4.271550225299677643289783997446700233879346989E+7371', '6.9376156592042773560575932435978717173439e+45123', 42, 5);
    T('8.946078494609738725E-841284', '3.56E-75812037365', '3.18480394408e-75812878648', 12, 6);
    T('3.02163372361220470146136E-80', '-9.21530E+3612294776464103', '-2.7845261253203549985376870808e+3612294776464024', 59, 1);
    T('1.2299954301031554494478759856891061037E-3703446280016', '-8.692183E+79595093', '-1.0691345367620336044048187028915091359777377e-3703366684922', 44, 6);
    T('-5.3639400108417513673E+2310', '3.971356314881987716098107E+735', '-2.13021170347045469467945597207284557881917011e+3046', 76, 2);
    T('1.94259371511981191330261541E+522820533', '-8.671713387107689973392127759839732564795778E+34025', '-1.68456159251157351430013405915068061291508593661481308208894e+522854559', 60, 0);
    T('6.85730516271018948262696136635806986831514875247987E+149', '3.4716317699331572165941275505515E-533139872', '2.380603845899135174686861400754e-533139722', 31, 0);
    T('4.32790261187249575856428388440496940053113787911573828E+92100025038000', '1.977118687335621700100998825700341E+9706047', '8.556777130901757458e+92100034744047', 19, 3);
    T('4.38231071754095747947566E+7986794', '-6.1011155600015223691245314007617213899612368337465171E+636467895575', '-2.67369841075505721098578172693951516843054384800026361304081806277863e+636475882370', 69, 3);
    T('3.54484860759457392760209736127295326698945623E+72656628', '6.7666682274552113040024120850197806100E+21907162', '2.39868144441490494506619712902181545048214529821372661292756e+94563791', 60, 0);
    T('5.728625E-788261921492', '5.38841683888680475940178681692879820451551188370366E+358215148501039', '3.08682194136679219148280610041287366143426742647818792675e+357426886579548', 66, 4);
    T('-9461792916.693743373', '3.0564639728869043493425748594744151373654184813811130711895E+36484161753714', '-2.8919629168790929267782705633416004845484065255105437304e+36484161753724', 56, 2);
    T('-1.96527203421101163284300206239584894E+5112966', '-8.10732705949493378044690066916818775E+99762100930', '1.5933103142227587844210470411508058305284033897588e+99767213897', 50, 4);
    T('-1.9011E+2134292', '-2.94314580446994317049107926390816686100128473466230798253263E+43906898', '5.595214488877808961420590788615816019449542409066514e+46041190', 52, 5);
    T('1.282822699778863847972483198759385064562264E+3679088', '3.263504434370260E+54', '4.18649756923915092016062295749204808648497474709e+3679142', 48, 2);
    T('0.085109214828655384184509385651745892755662674154762283', '9.732138538467151129872535808433520577E+59', '8.2829466961263699701303023656344575680242617967067422e+58', 53, 6);
    T('1.4960004878451862451385786372077025063E+1652005793043', '2.4831857835673669814147167272536748614701805E+7061084', '3.71484714362701206989893827831773960856399743533039773576604571444e+1652012854127', 66, 3);
    T('-4.319860423991430803813835526651311049441E+2817', '-5.3065049E-7104237458272', '2.29233e-7104237455454', 6, 1);
    T('1.74847814987600812659188837973949087340479225841700838465931E-211166718', '3.218202965788808237E+114514466', '5.6e-96652252', 2, 1);
    T('1.7550255331258943059972397228505E+31226000', '-4.67214900585047579803001337677795E+6048', '-8.19974079983634e+31232048', 15, 1);
    T('5.1549156599448114197126713080990229618562812924325174636E-75688', '-7.41417172943427587935060E-314803532569958', '-3.82194299535808540594707e-314803532645645', 24, 2);
    T('4.3664869511288707898988854478547085426091713410771073145094E+3688614169', '-8.23281560773475209142159201609330116680671089013987782E-2307', '-3.59484819222238991268729907273219693589133136722387495473e+3688611863', 57, 0);
    T('-9.202859E+87', '-9.501268859755807540891349452988E-764662121349', '8.743883764e-764662121261', 10, 0);
    T('5.47137E+8518754019733872', '-8.15518216599913396141788291E-360', '-4.4620019047582681e+8518754019733513', 17, 1);
    T('5.62655279653753653155633495437628762530061E-76', '9.978867355535042885734434432397688084341E+3920584723064', '5.614662402556282737186403388579e+3920584722989', 31, 5);
    T('6.2047372786413581554E+729389462269896', '-6.62133766137238604138179824129638859733224584802022229E+33220', '-4.10836606219892332195468375224438795638143e+729389462303117', 42, 1);
    T('8.351383248471507186962150753672155E+789740389', '9.74046415564561540547077E+3205', '8.134634918179595597e+789743595', 19, 1);
    T('9.09683293763526719388805294859646848354863206614E+2088', '-3.3183003630645266693226526E-254017', '-3.018602403969245182193092280873111173694872804243499344258962525e-251928', 65, 5);
    T('4.6895948099562964809306216E+2362867802', '9.743118901776E-37975722044066', '4.56913e-37973359176263', 6, 0);
    T('2.9482545100972033491343333E+87660', '-3.7650750085319747933677E-221080645313', '-1.11003993748e-221080557652', 12, 0);
    T('3.915666198964068118315942842E+84701445149', '-5.2496586343700031591308395370066E-9', '-2.0555910870702e+84701445141', 14, 2);
    T('-8510789990856400984369199371581855719419997624.88082920304579', '9.0E+814441607184', '-7.6597109917707608859322794344236701474779978623928e+814441607230', 50, 0);
    T('-5.80521799699702312440992495167917425E-843', '-4.3240316367881521972966566022E-521167158766', '2.5101946277467076307891378747429434e-521167159608', 35, 4);
    T('-9.247099952E-320444776860', '-6.750860374893418341847E+89', '6.2425880648635630754009313291344e-320444776770', 69, 5);
    T('-3.099354833557549101125819001320203261531695797445E+2007239', '-1.52264750189959054859610804468664098', '4.719224894816823393528059323979286325717502977754688223021049836975e+2007239', 67, 4);
    T('6.0370855947296132476648839670148749893E+57', '-1.8406487789950617765596863097374542114033626E-893638', '-1.1112154e-893580', 8, 2);
    T('-6.2045E+81189', '1.3532318283463346772784333669383551652468546226522825495E+12601', '-8.39612687897483350517403982516902462277410950624608707837275e+93790', 71, 0);
    T('-6.92117358126E+46736713904', '3.5057369629756336332962225065129676323218895732776E+6740769991072', '-2.426381405099e+6787506704977', 13, 1);
    T('1.1435076207252358879518148160948349590895217683637110908983E+11860641444', '-1.4514430290235381420045497703E+478785376588289', '-1.659736164736935598708202145044470453e+478797237229733', 37, 1);
    T('3.5094889941984852677830404E+852', '4.9317755302290936695342254982212791667292698E+8599', '1.730801194519640331858880520438805792452006970906376756019539071589992e+9452', 74, 2);
    T('-6.22693953943988394348052855398878E-495376950041926', '6.4685514664984356708700E+30209538540', '-4.02792788896409548882784814977626414291410685737528386e-495346740503385', 75, 4);
    T('7.9075904923968953614306054055079884E+498058', '9.2992330481871375165177531753976199415530062477E+4293998131', '7.35345268384276089239999423167798942621e+4294496190', 39, 1);
    T('-5.194434388419417361624934724720E-5360', '-5.4606746012E-75', '2.8e-5434', 2, 5);
    T('8.2238907780985793721065261927952804163878577306E-6512426044386', '-5.67762322568318256010270970083817615558166536918249742E+5962420', '-4.6692153287214234338364383035628581733617732498e-6512420081965', 47, 0);
    T('3.8780243820075386352960332472203916259399392700968224873E-559302', '8.551887859093E+62', '3.3164429629756903969188276690846e-559239', 32, 3);
    T('-3.827418621151341045174989E-7246', '6.55630957625712E+251947', '-2.509374135819935931559306178593005717168e+244702', 53, 0);
    T('-4.38696340600163708522407913561894E+734842152071901', '-1.70656008090761771110070595166761067990285824935509373126E+94659693', '7.486616625084911949528734145884357315482591538803629e+734842246731594', 52, 5);
    T('6.7480950925874170049437476720024656E+63128', '-6.3575933641422622742453093618428235490192E+69650282466412', '-4.290164458123472729522622278054019923327641369085425519153300754905e+69650282529541', 67, 3);
    T('-9.4406150627522626520166176527121390828928424168586E+5424911', '-7.389232444736272389654216597503308586092287051699795897303E+823434470828185', '6.97588991199549793e+823434476253097', 18, 4);
    T('2.721009180341519930919471943229289E+26761', '902517152.07315918914406', '2.455757456206749780692e+26770', 24, 3);
    T('-4.125738769751981997108E+9269603697621', '-4.9893328923936E+715349446', '2.0585e+9270319047068', 5, 0);
    T('-6.212656939065515369248239E+9152815', '4.25382806162565318809E+4158068806227', '-2.642757442465022501557073608918363327696827351e+4158077959043', 68, 2);
    T('-34859138917807702453.96485168187510941', '196008.640780962765741', '-6.83269243807425e+24', 16, 0);
    T('-5.8606E-52211264', '-52184699093063772945530517707406299570.2165565986417', '3.0583364750480954772457615207602535926121115160199954702e-52211226', 67, 2);
    T('1.5123242871783076286708161888576467888048438921E-7', '9.9549188978441063749058553301670210718721587E+1733526877', '1.50550656260999519928665e+1733526871', 24, 6);
    T('-6.3267156641124332993021119649932338800900325284345926E+45380512535', '4.372240392808E-890686201297347', '-2.766192178044347195721736545008601364494e-890640820784811', 40, 1);
    T('9.239923616339E+5765923198019936', '-2.9076522E+249126004605', '-2.68664842e+5766172324024542', 9, 6);
    T('8.607800785900704452665238636803150404323243335865E+48898', '-6.39781683694E-19477724586', '-5.507113279706089111026849100800220919321441478038574835885e-19477675687', 58, 2);
    T('1.644506096744252789389168391534205224025E+464', '0.000497122376292184082', '8.175207e+460', 7, 3);
    T('-2.5485E-472', '-3.325660869922613497616001153392E+96539152162', '8.4754467269977804986744e+96539151690', 23, 6);
    T('883353.09945931590568832653148213575305905440001056914', '8.798547164698665665049790244E-742561638040082', '7.7722239086755423760816533e-742561638040076', 26, 5);
    T('2.0118656884028875545773134272412869654E+6371068106', '-1.20746260E+451692076473011', '-2.42925257496974045415756477187167518658799404e+451698447541117', 64, 0);
    T('2.226056018915530338474E+821032256', '-6.53517294E+291831128745', '-1.454766105774090201374432569356e+292652161002', 37, 1);
    T('-2.8662362197457513054476612711849997459264438841E+879217', '2.52624035499037269424803826710985E+66928445', '-7.2408016052567706551709739186939243099373787683035766907506484379749327154e+67807662', 74, 0);
    T('7.7856958E+5649843', '3.241403E+41', '2.52365777232074e+5649885', 77, 1);
    T('-511706041.77023835906923382829668190', '-5.30168372516190240253007564555144600148504378847E+626', '2.712903593720289335199819968569744632490268330438862265228308734421e+635', 67, 5);
    T('-5.171201440956910886585019E-8899083134367', '-5.526400380815E+51633218561206', '2.8578129612575349061629468276474010485e+42734135426840', 78, 3);
    T('-9.327E-864547083230735', '2.8281539308625558058025456009319899944161340E-225621541', '-2.6378191713155058000720342819892670677919281818e-864547308852275', 74, 2);
    T('-7.32575635710574495758445719638964975E+425833817199', '-5.5276958146524517839014578330E+1914056241465', '4.049455275433701836101230006382700030818751634087e+2339890058665', 49, 3);
    T('-6.38981581292113588477731709246788229393453973853706E+585103311290', '1.66673566371027200942318164907683948602052346331148E+695', '-1.0650133899935500702914929348e+585103311986', 29, 4);
    T('-7.0241425579925188E+146982282296', '9.216041891990694106186894E+637', '-6.47347920697737267586247724664876466086072e+146982282934', 66, 0);
    T('-8.97576647926843027526906303498685E+51412', '6.43997E+775', '-5.7803666853494312919824507873e+52188', 29, 4);
    T('5.13136577663865285978E+995493449', '2.96384454844490794959480432245844130136175874822391E+381', '1.52085704831672424708288151e+995493831', 27, 3);
    T('-7.9883862292037485E+32681529475555', '-308305.26581042223427641893726965131741052396179133824', '2.4e+32681529475561', 2, 1);
    T('-5.07606239053882882489708436663890963235147507515434E+3013633704815', '9.257867538041997081074202886622E-7', '-4.699351322644528151030658e+3013633704809', 25, 0);
    T('-7.39986740263211058478849149286705117892822315128781092747E+905427', '-9.02563849265706140020739378532105868231965001347335049814375E+23915', '6.678852806975460664619014327393712953760690998625583635e+929343', 55, 3);
    T('8.2358029349922370136417E+926233605911862', '-9.348638248186986960837879517930E-4275690616', '-7.699354232259e+926229330221247', 13, 1);
    T('-2.2728324846258650310014101164923509474915640654189367E+7364729605', '5.0184386192073422015826441205916394840027358097935E+863610', '-1.14060703158354189290499867733762165193053888012e+7365593216', 48, 5);
    T('1.443718527989E-6003580', '-2.41070073216372921097480757879917438283591809E+7799', '-3.4803733124614236834053496214264651642363892122121764e-5995781', 53, 1);
    T('8.35E+2785', '-6.50165054967734E+1442', '-5.4288782089806e+4228', 14, 3);
    T('8.8136841798247280875385188E+305', '19438997675316266386212837.3604414265068', '1.7132918628258464324595e+331', 23, 2);
    T('-1.024009293959594319810817E+694', '2.594413177352443472562975782218657205325E-4852036', '-2.656703205980143400391e-4851342', 22, 2);
    T('-9.9794480762203082312E+929918100247', '9.23E+8964052', '-9.2110305743513444973976e+929927064300', 40, 6);
    T('7.52756987985291406521423866473791127477024270883E+7593218190', '4.173682164589723213238949E+2212183540701628', '3.1417684150244913074836990499028e+2212191133919819', 32, 6);
    T('-9.2829400681E+273056145991', '2.985605441110874075185259280984467E-8651027232919659', '-2.77151963768255079271507753099381051280222027e-8650754176773667', 74, 3);
    T('-7.56242E-2602946340518458', '-1707279649.703364364254142474411522458744861276324872104', '1.2911165768509716735e-2602946340518448', 20, 3);
    T('-3.70695721548376852526990819804432327416E+98769961578517', '4.0613226187E-33666899288981', '-1.5055149185797398974393820687189969218480849242792e+65103062289537', 79, 6);
    T('1.208106273644194227812465057401945909506406E+663410194', '6.39992354937750306E+43266775306', '7.73178779084618050081274654883989304998658095654804535460236e+43930185500', 76, 3);
    T('-9.38693389593408478633713498243965E+48', '5.44617110448313577621252974454013168928889E+57740084716', '-5.112284814e+57740084765', 10, 2);
    T('5.4436261657711803629307182559639809E-82402272373373', '-2.0914916021029776271843411705711005298077855903863305440349E-30217713280', '-1.1385298410698455288526618538490087725974269957184178995906e-82432490086652', 59, 5);
    T('5.8742708072700482361351124355398E+17857348640', '-3.44562E+384904', '-2.0240504978945823604e+17857733545', 20, 3);
    T('2.8871E+358456794502', '9.39460591217371338017299985196401796E+2947', '2.7123166729036727899e+358456797450', 20, 1);
    T('-8.67377775627282041907681821E-8692046665392158', '4.280360489315649428522628817320615720259237267531505864987E-51', '-3.7126895601e-8692046665392208', 11, 5);
    T('6.529537041182116738679569916073044E-546727', '-8.8754830782469464913938994850684435583154136877E+815', '-5.79527955177985124931174983532504303219394e-545911', 42, 4);
    T('4.4337654118654487303E+789', '6.3921E-458259389197909', '2.834107188918513482895063e-458259389197119', 69, 4);
    T('2.0299319306225272338087811787809123059176118337031449E+3557', '-5.504E-160089562002', '-1.11727453461463898948835316080101413317705355327021095296e-160089558444', 61, 3);
    T('-4.32426E+9795559132', '-1.5806909076E-474687813361', '6e-464892254229', 1, 3);
    T('4.39797423843862310215550179209167725E+9994369862742', '3.99158329691829999730918690575756997611860139440841040791109E-48425', '1.755488051042859e+9994369814318', 16, 2);
    T('-4.6261298640641886240208712885384918040E+67403', '-8.6982606E+2615679', '4.0239283127072887779289e+2683083', 24, 0);
    T('-4.3245256074E-43', '-3.5426531620934881806511154633702532035272014393E-117449486770201', '1e-117449486770243', 1, 1);
    T('-3.01884988903488971149216039956628176896182827E+2234378111726', '2.5583719582444653541E-2779899108279587', '-7.723340902256277728601913523662828471492544777449280397408940407e-2777664730167861', 73, 2);
    T('5.585240256233476268E-33', '9.85E+4364037', '5.50146165238997412398e+4364005', 47, 0);
    T('-0.0793321864349108913217168193364074645650020090204', '-8.654032601876650428788309190217584173450409566586E-9', '6.8654332778587541328298405249303694060479444507219781913e-10', 56, 5);
    T('-4.341402370955E+9343873', '-2.551049330158366220524397618266270328865449182747E-819507', '1.1075131610372695695413802203364341022e+8524367', 38, 5);
    T('-3.339366805899303E+90909386', '-4.653819292420274042826795678435989370318581977384E-81843200', '1.5540809665762044898750379560622863054579245478643002066127363352e+9066187', 66, 1);
    T('-3E+97551534367203', '-5.029827092978322325788134128674804181783759E+16071047', '1.5089481278934967e+97551550438251', 17, 0);
    T('2.3886107781539429E+53250881376', '6.936736897378840785909666359625973480831E-2949', '1.65691645182972404448893207502078374824883682694802185499e+53250878428', 79, 2);
    T('1.4019520301405026E-711592', '-1.45230666895932326053834910745994516011E+996597911114', '-2.036064282934114e+996597199522', 16, 1);
    T('-9.2983163537E+7874977089489758', '8.080722E-289018655', '-7.51371095223033714e+7874976800471104', 33, 3);
    T('-7.362583102759739462966341012339630941809733E+8706245', '3.610439900482681547110857644896754601559228940296969E+62595141029', '-2.6582163804823346473470579584387523484030372907258069845080334126945764e+62603847275', 71, 0);
    T('-2.15006235931976764095383157472E-8371105151', '1.60131548319290910371320379627484E+1262473626100640', '-3.4429281458090198e+1262465254995489', 17, 2);
    T('-3.73E+52457766437', '3.67836546415770773874154751910638947895912514108027920953662E+180', '-1.3720303181308e+52457766618', 14, 6);
    T('-1.9724315960555227166086759006180814899055998870518691392E+41892', '-2.0060880277280176380402313174987E+8689996438', '3.9568714104e+8690038330', 11, 0);
    T('7.704815747034E+329', '-7.579661289558059E+38927', '-6e+39257', 1, 6);
    T('2.09790527627029305468405686112871166891280618E-98365', '4.6400289298391767165E+106280783', '9.73434117395641025831033268284067951787657939596346503493330797e+106182418', 68, 6);
    T('1.2894887299081753547055127078E+806838169674', '-9.810253159595804513799126945808542532354636882E+72096153', '-1.265021088684485825919809569872457098e+806910265828', 37, 1);
    T('1E+914509453892', '-3E-581371', '-3e+914508872521', 80, 1);
    T('7.7784061533506E+779487274310', '-2.396343278355086358352583076196499334453138626E+83', '-1.86397313018975534020473884205689338867e+779487274394', 39, 6);
    T('-1.8406542929200902328256862928824544976659168146637708972043E+5941028970', '-9.41687428447833610379752892338275256688419851341839E+377783', '1.73e+5941406754', 3, 3);
    T('7.010948E+499617428932535', '-2.44659E+52549214', '-1.715291526732e+499617481481750', 41, 5);
    T('-6.13369354503343950646381791653889905846304632131E+80667', '1.51284006160976999631851299E+1640247', '-9.2792973205638371603724594896445856527786453471122e+1720914', 50, 4);
    T('-5.91290679342581051881366879171186256296206287E-82', '5.997946061483E+406', '-3.546529601354441487827547760279379753288849910146153143621e+325', 75, 4);
    T('6.4080916539020379E+288130631277853', '9.598582440317853624442326629407958353325E+29380455', '6.15085960250914936251058620697081072396187902925457410175e+288130660658309', 61, 1);
    T('-2.2678E+109338', '-8.547652299630582E+43728', '1.93843658851022338596e+153067', 43, 1);
    T('1.8060391E+94', '-3.0989538333037595028125896800684363569036914926E+929215', '-5.59683179204147184e+929309', 18, 4);
    T('7.52053035455672168473102903545915906674250906129059548E+24249920584', '-2.57450874403571190E-7255', '-1.9361671157592272649283339729040437108026883117467471797572780736722212e+24249913330', 74, 3);
    T('-7.07319983493758830158810512746304357206763484608631366719E+8002', '9.3508153784249457383754E+63212', '-6.614018579120718848725726611055617251420163295818062878703068114e+71215', 64, 4);
    T('2.3190130202181143891456063271008212677484874392388E+3828966922338', '8.328447E+63', '1.93137770311964941299365575781238535849160869678660661436e+3828966922402', 61, 2);
    T('7.4502739193029211413882889876114081838E+3595154', '-1.0E+7490407862096', '-7.450273e+7490411457250', 7, 1);
    T('9.3650315595721088211282860946912879289376787760756550E+694762466379443', '3.873296056875156102078851339199432385482E+83302796727', '3.62735398122020426599995528245430638049249072131498914112e+694845769176171', 57, 4);
    T('8.96682634292134029753942913E-485609459', '5.32860556774167460017751918823248183766509865913E-253859281393', '4.77806807758633722e-254344890851', 18, 1);
    T('-8.5031783748904573528102962190999625E+639', '-6.9900416638170882817218623243E+593197392739', '5.943757111535277736478240217422761199609249079198046348429283875e+593197393379', 65, 4);
    T('4.3227109233527553272270241957722E+808', '1.74493395786644532927039240225899831978265369013322099559E-15', '7.542845080198439749316181230655813525893902438972074582279e+793', 58, 1);
    T('-6.2543955705763059515486305674453120425617636324668807457766E-1942', '-3900.447274283529622318066234342455583594417267538127018849', '2.4394940155545333571661118571045413568687724384220469e-1938', 53, 3);
    T('-1.3830E-76353847', '2.14816505980139478788242178957140539600776518E+67761', '-2.97091227770532899164138933497725366267873924394e-76286086', 66, 6);
    T('-8.652155094124480157315663008E+149', '4118.69040512820471', '-3.563554816985161533426616346973244719439836768e+153', 69, 3);
    T('2E-848391572', '-8.4471741E+897728', '-1.68943482e-847493843', 58, 4);
    T('-9.76295867E+2313', '-9.17714E+7862669570227', '8.95960385288038e+7862669572541', 19, 5);
    T('4.421013019181199793813893379538947380249791815834E+784761', '-3.06225341021602768711375273683251828803748646733703E+838904', '-1.35382621945970856937313954627161142846560415898e+1623666', 48, 1);
    T('8.792774254745537522167056663885675266334E-33759184589', '4.5509390283419404873232710174168150841418967928450898E-9517779', '4.00153796e-33768702367', 9, 0);
    T('-3.266197236234E+308', '-7.88059136374443981530E-9', '2.57395657321516183142174094275802e+300', 79, 2);
    T('4.1338901490507E-7942736724566', '-6.244445597597200055372991880898945778290824354796E+664869915660228', '-2.58138521421900767706868437e+656927178935663', 27, 4);
    T('-0.9768713949216324654334456574885403544664476056314', '-1.931061328207375858149060453168383304726607791746529E+1859250733290', '1.8863985733651595884509669350598192854e+1859250733290', 38, 4);
    T('5.1971021069163749153546E+6496578661', '2.520384845459251E-222066218994', '1.30986973905763753587013e-215569640332', 24, 6);
    T('5.774E-79', '-6.90576719593873790543282135656691326E+71790686', '-3.987389978935027266596911051281735716324e+71790608', 64, 2);
    T('2.9782E+10571', '1.455481541902135234517E+882582015', '4.3347151280929391554385294e+882592586', 48, 3);
    T('4.151417713411086294022816703692064659114267375001E+240', '-8.63634383808311387459593418E+93', '-3.5853070788526925487497998117658084522412557352456831e+334', 53, 5);
    T('-7.265598934729794848929487626329272043108932E+5028659108469', '1.48588051072842226438145299759129420343455005086831965E-15489335883352', '-1.07958118558841883104460699133103776893794809657932589770799916394596e-10460676774882', 69, 3);
    T('8.28420103245520E+964990952791', '2.53542476E+586450668932', '2.1003968414504477670752e+1551441621724', 59, 4);
    T('-1.25881611040613002709691415699647653736358E+34118289039668', '9.760747E+506649789', '-1.228698557319830244359612356716088737264195139426e+34118795689458', 59, 1);
    T('-9.781760217025537718694966687500717232101E+49291937782808', '-3.91676285811913925272497399525168340547887030124193E+2063140', '3.8312835105073036976653799766984e+49291939845949', 32, 4);
    T('7.545342527585904968859067030942596783242118705E-222861562916068', '-4.9082654296819692700914848364293178678249294639088601256065E-4119', '-3.7034543883259067922458279182010784108586489517861e-222861562920186', 50, 1);
    T('3.008441537457419208885178937641366664580017910271322383E+762', '-5.395128673850482232173018429312608801480531527832E+24632360', '-1.6230929e+24633123', 8, 2);
    T('2.748480338639813E+5024684261', '9.7669224688182398860052285652878258018537E+7274382600210515', '2.684419437456635438799473659307420183120647809e+7274387624894777', 46, 6);
    T('-1.2317E+406624739222848', '-5.076765958747922816221756515846900E+1146504339702', '6.25305263138981653274033750056862673e+407771243562550', 55, 4);
    T('1.41905459235173101183460254256396420098310E-9955642601079', '-1.4369726444183520738E+71002', '-2.0391426301456735215985243940980405e-9955642530077', 36, 0);
    T('-3.8400377823010853442835529460595283336042909E+548', '-2.21425413982608771269927549231682126911611582055939162412E+2758450235696382', '8.502819556548767196031999098347672875412584e+2758450235696930', 43, 1);
    T('9.261913719824764993045392043479761148039717685015E-8007822945', '-1.10745593295364197510367771619011423807650882274E+2303964588626544', '-1.02571612995246716850675977677172495630235917522177344e+2303956580803600', 54, 3);
    T('-3.932342799456777326582905773640763845389264174193556902E+73042', '9.3329198857384E+29', '-3.670024031e+73072', 11, 1);
    T('-2.89673361020484E+91204253290', '-4.003031111607718676977163E-88085591', '1.159571476368972071992620635834043206892e+91116167700', 51, 1);
    T('-1.29399417894E+8670970461', '5.46515543867857601555734384045406395E+6010976692', '-7.07187932465236e+14681947153', 15, 3);
    T('1.01E+570', '-6.4029244678483859350628E-14992', '-6.466953712526869794413428e-14422', 57, 2);
    T('-4.362755770118321440E+91940360522', '8.043234485133312794057365866097563071316779086765E-114737', '-3.50906676604300266980272268050814816442097103016883583e+91940245786', 54, 2);
    T('-0.391764040615772979795813129125686670001826822713856218534', '5.3845008498509098638006547413410E+44484916', '-2.109453809636655978944177198905690353513952292500810458474749e+44484916', 61, 1);
    T('7.335977501524499640346777113E+5541', '-6.8617526994550414491503352376853645776966447376E+90878721873', '-5.0337663424227185855001991097768726944042e+90878727415', 41, 6);
    T('-7.05923676E+699934475408752', '2.9120189910023128216689165345329800218046974323735402105E+565891823', '-2.05566315071016359157445401499470220022693216552889091e+699935041300576', 54, 5);
    T('3.129948895139884424742323286560486416586152236844610992897E-6321073564298701', '-5.4953971802740754754397445257264799E-2532', '-2e-6321073564301232', 1, 3);
    T('-9.900447365425646061575912719E+88693410472', '2.5910602668251E-8371536635', '-2.565265579234763280874e+80321873838', 22, 6);
    T('8.555218099096280262925285E+475005154698', '8.477860500232859149942887205212626E+10781594816', '7.25299455932056009521154513273e+485786749515', 30, 2);
    T('2.132355333374995E+6187', '8.3E-133289411296615', '1.76985492670124585e-133289411290427', 61, 5);
    T('9.691619082980479576E-81485675129253', '2.92041957E+6093203', '2.830359403492e-81485669036049', 13, 5);
    T('-1E+1535004', '-2.165E+865402075360', '2.165e+865403610364', 46, 2);
    T('1.645354456261E+568', '5.2546624043121057707006459086831046000283E+18172241', '8.64578220308206373209109e+18172809', 24, 0);
    T('-5.8925071728066896463798102E+6527303896738729', '-1.7E+2250577', '1.001726219377e+6527303898989307', 13, 5);
    T('-2.0177219016316812224513866338634E-1874788389450407', '-6.789621982909151765614071321E+244873', '1.3699568978715719924959191132514560553854736727439242e-1874788389205533', 53, 2);
    T('-4.166943922145582332448789300161629805564970200481955721E+46164694', '7.602560651131513208258748885253979723356328956108074291E+1040837', '-3.1679443897975619897991625876338112977182049019834772876158002e+47205532', 62, 5);
    T('-8.361409E-52315081', '-6.421275142897231362335639207449277846336088319799E+382937471751', '5.369090777129719e+382885156671', 16, 3);
    T('2.40743092863612658259001575392E+869495372', '0.874', '2.10409463162797463318367376892608e+869495372', 49, 1);
    T('-9.716717144E+663', '-75.6406356863295890104309897590396101645579879878418', '7.349786615564169239721287930205486090610372630636084816198192e+665', 63, 6);
    T('-7.2713344102974929632419025122E-4422582514', '-2.8237792285700806813771339880308254E+236284837028633', '2.053264307178493720522204739869029191285112725092232159e+236280414446120', 55, 1);
    T('-5.2364982319276533809617505581075958605242551568505053765626E+769169671710', '1.94208517828005879503755046294367953297174070514669E-465020', '-1.016972560231642938453732520969645619327631e+769169206691', 43, 4);
    T('5.0869053020627432035506502692155E+70265147941', '9.96E+41', '5.066557e+70265147983', 7, 3);
    T('-9.560647234017718855E+8880392953', '3.720495440E+99332038694', '-3.55703444376115358792295212e+108212431648', 38, 4);
    T('-9.72654060543417752203584328147465769533761474528594750516E-67', '-8.3347851732062794938E+702', '8.10686264247616119106283350255e+636', 30, 0);
    T('7.7174431701543268969579934357661E-996827483025', '1.258735908862454030271334531884603566E+2957133', '9.7142228428785451322608260893657529734364639013565e-996824525892', 50, 2);
    T('3.450960E+6748208292', '6.8581775546218181810067599639547209824985081991E+81938723758169', '2.3667296413897709669927088365209183921763051854766136e+81945471966462', 55, 4);
    T('-4.58101376395375770383859636570311703556785386057849232E+5784', '-7.54419796990682189E+2209290', '3.456e+2215075', 6, 1);
    T('-7.0187717E+89833715978', '-3.234937240873120194550530656448403405043022950132476E+71931135367', '2.27052859575163393122097587914624763294996067648403337998e+161764851346', 57, 0);
    T('7.6990970316032659093732344476444940878E+339689845', '8.73219843E+7762838493311875', '6.7230043011783698956701480127743168471631442154e+7762838833001721', 63, 3);
    T('2.103282105623798871260847211527804861E+5874411923', '6.5969802982162850626557873122039859860400E+618829716889', '1.387531061239106465562008707963712072377219005282058388154e+624704128813', 58, 3);
    T('-7.550572254224899E+826627347607719', '7.4403E-1159576543666', '-6e+825467771064054', 1, 4);
    T('-9424.48634421', '-6.69783169796005684025562666E+221139928919340', '6.31236233732414330050245548627860126386e+221139928919344', 50, 6);
    T('4.565342352745632957160623268265506E+83313337976871', '40258.63952671595502558940773199686893387084284205', '1.8379447209523575339401933918299481238041257567377241204921759679221079081e+83313337976876', 74, 1);
    T('1.5129544081570486484975578350E+34827338228', '9.50866189537360871852602557443716645417542964244137451639E+747769699', '1.43862e+35575107928', 6, 5);
    T('2.95502443130630628613541742397037779187308464E+1971538188383844', '8.406E+47609429731154', '2.483993536956081064125431886589499571848514948384e+2019147618114999', 77, 4);
    T('3.3544268580056726227233836E-3845241', '1.2317683477712112509278379214058884090223535412422014437542E+75663647382', '4.13187682860502281642362858912025674343755629e+75659802141', 45, 5);
    T('4E+3', '1.26215516465702259681407481083589500325246204657361873E+52720027096198', '5.048620658628090387256299243343580013009848186e+52720027096201', 46, 3);
    T('6.2187993015425519514608027268077912E+668868', '-6.7162225106132371098187142681704498E-58727163255', '-4.176683985800596364946606417471331348080710898095058908162868424849e-58726494386', 67, 0);
    T('-3.985641355449E+828016333', '9.7827E+8', '-3.89903336879509323e+828016342', 78, 3);
    T('-7.2747683E+4011489638', '6.53152215475747773717964031238923E+4975', '-4.7515310322177393230390178749971267665409e+4011494614', 54, 5);
    T('9.175957928769524452512410487E+6202249274792689', '7.717947586403811E+907', '7.0819562349289663945810480591876223373165957e+6202249274793597', 62, 5);
    T('-1.53064327522900040093465773454740609420E-6433', '-8.8585802083701072846021209685E+161', '1.355932622e-6271', 10, 3);
    T('8.20326534440557555336054432524774738932079893562106869E+1055105636', '5.65648126563808379355769665399211414236315E+1300923415237', '4.64016167376882813496446213797867769331345549455e+1301978520874', 48, 5);
    T('-2.90048691925719022187138620365470346437E+55166035650', '0.00000481187134076075635802657574937703846527358199869', '-1.39567698810251315827489872e+55166035645', 27, 6);
    T('6.93986544895782211734313287282348124648422358E+4868046377', '-1.847840224094345662406390600251723270025770223448232E+46312726937819', '-1.282376252638682879065663795578705e+46317594984197', 34, 5);
    T('4.7149E-837767008200003', '2.01653659453853994520949246945136135E-58937585313', '9.507768389589761987668236044216223629115e-837825945785316', 65, 3);
    T('-5212067467.9460651570647203790829338296973635', '3.8178799221269460397578037186516772098264166211338645807996E+4251609374', '-1.989904773864231206606e+4251609384', 22, 1);
    T('-1.55430248335568092889508507368499370204515094607125774E+935', '5.27443836156077621812234946780017968309E-3914', '-8.198072643680383367074725e-2979', 25, 3);
    T('2.119318509671314309554346616880033019E-309', '7.91E-67373708', '1.676380941150009618857488173952106118029e-67374016', 71, 1);
    T('1.974964525793077422847750064180730975524491456525840852458E-36999862749', '9.959079502E+17842717405', '1.9668828726002987655382014131001502281722426563660625767028674116e-19157145343', 65, 4);
    T('5.2721436100112964214250E-217', '7.701656709363263511407043940098964701288571170377834945E-390030729', '4.06042402067701580502e-390030945', 21, 1);
    T('-64457.33186055771415409333239718', '83644275.704005544', '-5.3914868372890700773568407671451094455496e+12', 41, 3);
    T('-6.0470138662925737567392412231771515995395554614E+97140829626', '9.623667031739511999176810859E+127584223', '-5.819444798551152357617916747723134403e+97268413850', 37, 6);
    T('-1.32E+5913', '3.8187663395737649951995492362883162037523957375672E+2308583002332965', '-5.040771568237369793663404991900577e+2308583002338878', 34, 1);
    T('-8.390092539E+58669699', '3.919738009631519155163E+951668466', '-3.2886964629444119002968669628857e+1010338166', 71, 6);
    T('-5.411201269962029039432494732744681980256776255125024221580E+9606324', '9.12599935586065450036604139066428738731340390074833005918E+343943693', '-4.93826193041058326137314752570083782801307553647318832454893022913705804613e+353550018', 75, 0);
    T('-5.06E+944411426255', '7.26688082676E+45', '-3.67704169834056e+944411426301', 45, 3);
    T('3.36072507664203417638456149689021506065947705040189371E+9907', '-9.064706631986E-2670983', '-3.046398689051870533757080390480319597530978090158061149969045820806e-2661075', 76, 5);
    T('7.957778870393602243691587421237376067966757787220656122024E+661960582000', '5.7251004660732619742630456752E+844043554', '4.555908351979836839957113747900449315695924092722492246e+662804625555', 55, 0);
    T('9.20012173752295305752628835800551537212608172300094887618E+815', '-3.36954E-59238941576', '-3.10001781994530912454571296738339042669937174089206172762435572e-59238940760', 79, 0);
    T('-2.6935682265887043965E+2269722254811041', '-9.69028818859286803941671538174814602486849E-2081964342', '2.6101452371281560261160422688911222107e+2269720172846700', 38, 3);
    T('-957.428270325088700821517302569767504046326235797201417', '6.8968E+263414292', '-6.60319129477e+263414295', 12, 2);
    T('8.7045864971519356496474043492663598E+467', '9.1740743022455255821697410422826310794E+804094317447', '8e+804094317915', 1, 5);
    T('-1.5834639523033063764701096437797820552E+95349340185033', '-4.1827023754456973187130666117615047082755672615778910E+6876', '6.62316e+95349340191909', 6, 6);
    T('-7.88144517201760967237935887261999365E+439070504931', '-7.62718010744512691E+307428', '6.01132018339321488826172504097134989004967419276441215e+439070812360', 72, 3);
    T('-1.18453240707641520210576618343151593989983349E+3017087149118947', '7.072161115838880597829952581533120551020322391659409871E+7548796', '-8.37720402977685567972448165827139711953904558531370874570359390444e+3017087156667743', 66, 6);
    T('8.868571567669419833016145920326628251146E+6998417889497342', '1.6331121254316704963325368480405128251635509796695E-83405078785098', '1.44e+6915012810712245', 3, 1);
    T('-6.573016292775281932741350866056520935E+9735006649', '-4.661855168723081774908770786545133472237022980472989114595E+427096', '3.064244e+9735433746', 7, 1);
    T('-5703721665135837.4473748478845174714784', '7.6748595749653578724575551614552028548838E+790697', '-4.3775262835e+790713', 11, 4);
    T('-6.3327426962111460E+384664', '8.340741874631E+732992959354312', '-5.2819772187551927229036837126e+732992959738977', 63, 3);
    T('5.069984052480718199606600639082191558464183513E+78', '-3.53945783086847266185712727742647810877107990E+39296649776787', '-1.79449947569311515012522e+39296649776866', 26, 6);
    T('-6.67619993632527064782387772126916638534944688401123025223173E-698054244731614', '6.1542076502452406404913059247E+6136', '-4.108672072269976905759058932994318207309308420004803753023e-698054244725477', 58, 3);
    T('-8.9063209656119998085580130006E+739291600456299', '2.90795657180145095308145421E+989477', '-2.5899194582524459306086090752863196628643503759933602526e+739291601445777', 76, 2);
    T('1.25237972195345420299485980391008725456174E+8131237', '-7.1902801167022722518366368646111980203028734952E+2812774', '-9.004961013323041961e+10944011', 19, 5);
    T('1.031778263468070025372394018724323314281583', '-9.40736518135939954516656073220327099760474557342484741280E-3873091347248434', '-9.7063149106329869005612142825414992494246583530099140584897106517670233022779495e-3873091347248434', 80, 5);
    T('-3.53163369199797660426819829747672058390386557037767E-65', '98937.00513296873452709672097145088523062861', '-3.49409260712969134124595174654e-60', 31, 4);
    T('-4.17068351809202529640E+737', '-7.063544094874726220800541635742901361710469934E+64', '2.94598069358102736628205642e+802', 27, 0);
    T('3.97471895392663343305E+32823437850671', '-4.895E+61', '-1.945624927947087065477975e+32823437850733', 42, 2);
    T('-9.512207351029101326366892150877726164102036459E+46724350', '6.883516904320726686275849286443575851344E+938996', '-6.5477440098212699518839660995e+47663347', 29, 6);
    T('1.189087840235599689628750152517187646897765617353173197E+4632633610', '7.59913890500676E+176209672', '9.0360436682048082013e+4808843282', 20, 6);
    T('6.698085949926577655039451126797610411215301242041639562758E-552247725695', '-3.817907296882848033133591333191563506817162549547E+76144459852364', '-2.55726712233731635002e+75592212126670', 21, 3);
    T('-1.8232143816294592744472752139762400892685074049392099837E-7080', '6.138672624E+119', '-1.1192116212191850159972391087429687e-6960', 35, 6);
    T('8.40507193694621211000011714970321322467202563E+441282241109746', '-5.249167306196902910369385343407034859288971999426466466295E+62851177803819', '-4.41196288176511332146026466601179858351844336131823825581615887116148599386781e+504133418913566', 78, 2);
    T('6.7902290359778438714549665725399881989093E+638396', '2.3283953070772386648442036E-99030', '1.5810337e+539367', 8, 3);
    T('5.8503871311985352146183952168465545703709E+413187241401928', '3.97545648103892950694323984371374789677805445183759E+6935839', '2.32579594373099669e+413187248337768', 18, 0);
    T('-9990802.0762837476316285772756074981146243', '-9.424213E+8262005', '9.41554468077402861187132491322847666293178181759e+8262012', 59, 3);
    T('-8.20497586578512042612464220530408474213515454375082E+12183', '2.06783406426336885053E+79', '-1.6966528591729299183880949384546e+12263', 32, 0);
    T('-1.430E+283', '2.39262014E+93', '-3.4214468002e+376', 58, 4);
    T('5.02378884170112E+19129', '8.9385E+1313713240574', '4.490513656154546112e+1313713259704', 57, 2);
    T('-9.75910195290566749515528765535881415282E-21383', '-1.5617582791477178030911044088173976429201074961528E+8100213868402508', '1.524135827199708741720529476902416060538470727159e+8100213868381126', 49, 3);
    T('1.91200378788832198503123442E+798471135', '6.72992016537967787211666470729E+78', '1.28676328483919464247389735051789165697116949651166729218e+798471214', 77, 2);
    T('1.707113860396727383E+854', '-9.135149224338557758969745E+6264050', '-1.5595e+6264905', 5, 3);
    T('3.795206366522804015645528623989120693472271600310E+5392054479', '-5.6853429567513056198527834502766434146013E-72', '-2.1577049785328137895388465283638369683182288372986645804795e+5392054408', 59, 1);
    T('1.4370934922238401025452752917356411436313806893622061645341E+784', '-3.9564668587302146197073980630230683263853895069316E+385702', '-5.68581277488049076111991115222e+386486', 30, 0);
    T('6.8195006623651649195494223183651347305996794854595329E-9883838178', '-1.12171167994103553277749E+86257015', '-7.649513544340633692104931350531647718678067442263923e-9797581163', 52, 1);
    T('-1.75E-7472256711721271', '-1.685391016728452985116E+64936', '2.949434279274792723953e-7472256711656335', 56, 3);
    T('-8.106532126297412274026807808292E+673620134997974', '8.2336940154618205287414745394790939511704E+5983875', '-6.674670505444399050346848819723270289001451453323139e+673620140981850', 52, 4);
    T('9.76488487219254126867557720178760520967409E+39947886590790', '7.30544087734E+903965', '7.13367891078343724552374653728581448193e+39947887494756', 41, 0);
    T('5.41645840935802399616746736867E-712', '-5.7048939343830315320190E-4564388527625846', '-3.090032072538455429261582361719e-4564388527626557', 31, 2);
    T('-8.443182294593E-85670137', '9.452688602E+946890589126788', '-7.9810773040707457328986e+946890503456652', 51, 2);
    T('7.756973538673304398811595313527972687989464634241796625737E-56927', '1.05926154929657884410132284E+983', '8.216663808427650068258e-55944', 23, 4);
    T('7.688178686082763881771752114084431865247799916E+219818', '6.8155571250242686360224186487018268207967515423102175E-1228404603365', '5.239922102239e-1228404383546', 13, 3);
    T('621.4902118608474245803', '281334.9', '1.74846e+8', 6, 3);
    T('9.32273438405966773208518907306026E+48428', '9.6332888599302563E+346234', '8.9808593286050756748736174220098898965548e+394663', 41, 3);
    T('3.269787054549877017E+8299333', '3.4529002590209179799843E+28', '1.12902485675985148205774449826876325908331e+8299362', 74, 2);
    T('-3.61999387238519753633840248404493376641909533416629642021403E+8560', '6.3873846980498935802898411503006346654482724155573726E+9010143', '-2.31e+9018704', 3, 1);
    T('1.718141755661505065945742455785775638742853E-90', '-2.677263677275882051E+846873', '-4.5999185148435610916686628e+846783', 26, 1);
    T('-4.4E+598914888045341', '-7.0078551717535671611449115776352598712007316235153323718E+82', '3.083456275571569550903761094159514343328321914346746243592e+598914888045424', 62, 1);
    T('7.8224028430240269118256172680847130332708461922155548E+498080', '2.0713899434899E+934945709', '1.6203246582966772205314402162e+935443790', 29, 1);
    T('-2.95616521120240896714342658098144526443028235025629486E+45787', '3.65681259289560161698715668387562871926779203318583784322034E-3445', '-1.08101421711e+42343', 12, 0);
    T('2.6127028639823990E-44224', '-2.0024113270927528383555771376137E+429779', '-5.2317058091660316927e+385555', 20, 3);
    T('-2.737252035364E-1897', '7229145689496813330575658.51285135486', '-1.97e-1872', 3, 2);
    T('9.5064351430063521243996981878876E+61182029677500', '-7.617192345654452415571770717049618338E+780531969945950', '-7.24123450057684751311687475035439943465794198e+841713999623451', 45, 0);
    T('-6.4146869969165528922819228455182817696060E+942032', '-3.41', '2.19e+942033', 3, 4);
    T('8.2858553550701267945099879055893389177347979191E+2856996862161', '6203400877.8', '5.140048239e+2856996862171', 10, 0);
    T('-7.908235780418E+2195402458', '-2.3433426934056215006976680121053615607126438252015450E+9676120', '1.853170653377142325079814292e+2205078579', 28, 5);
    T('-9.6384374570745263310560248279768275459275E+4495962524', '5.295131666218256715571428669554310030275189084E+103', '-5.1036795391819493800147275406578116486371731876199483581e+4495962628', 56, 3);
    T('-8.80816E+81868478914915', '8.9847162684802550951414277211571392654804862589148E+914286', '-7.9138818447377043719e+81868479829202', 20, 4);
    T('-6.000442390683103659859250519621243976353651366941777806651E+86497', '6.0494795908857508325123589073912780612378608E+2480636471', '-3.6299553778723138591162846513363e+2480722969', 32, 0);
    T('-0.660', '9.31515252018314466700935497735257E+14770', '-6.1480006633208754802261742850526962e+14770', 66, 3);
    T('-1.95118886540398169351864216453865914583E+66039', '-901.46776515234528945276141840346641589', '1.7589338658858676318341400690245913563849686959e+66042', 47, 5);
    T('1.00813293424799998813106902486438E-772610064', '2.961113852148408144418569325851970E+81986776675', '2.9851963964087731062497373177019557757310706106646386302572058286e+81214166611', 74, 0);
    T('8.221497739501583E-696446731495', '-3.217345284317046382406310096772915231370141412849759549440E-4691', '-2.645e-696446736185', 4, 6);
    T('-1.25460011100075876904862072237196857522549474E+7024', '-6.4466303427454E+8079994031936', '8.087943143589238388311374408541933e+8079994038960', 34, 5);
    T('7.7472E-47424', '-7.001920844016015140852624715E+743871770', '-5.4245281162760872499213454192048e+743824347', 77, 4);
    T('-9.81533746672839320478673842532167506634894E+4482969', '-3.484142986125550707532959E-2609', '3.419803919115706211513028749516626748374388226096023594738684471346e+4480361', 72, 4);
    T('8.964255268991859771829089833143E-2369206835634581', '9.965333754561734376691250551614413006178E+19964', '8.933179561659246008147156813083e-2369206835614616', 31, 6);
    T('2.441179376345635505885965366900539972978E+631284425210498', '1.328445557216132735836253E+1420252', '3.2429738968740091614122781245223062096e+631284426630750', 38, 6);
    T('-1.79276067906376407968E+270241', '-9.3956645756E+33346364', '1.6844178004808008737045111783808e+33616606', 71, 4);
    T('-4.3015928633128966404958383514552675330691032E+9570', '2.634569916067930383527559809E+84050797', '-1.1332847e+84060368', 8, 2);
    T('7.688115651718543888047E+919492974', '-5.3577750056007714548532643143471065528811014748496E+99596', '-4.119119387894570016353683680580469100407058293774217e+919592571', 52, 4);
    T('5.5733204976700131836365181702408405207074185602819435271574E+816', '-9.790454244517589593152E+92195896407', '-5.456533932247026537416482496088645e+92195897224', 34, 4);
    T('2.8087387202E+130409749', '-4.3094505001287236540E+251336274', '-1.210412e+381746024', 7, 4);
    T('-7.2637024E+35191', '7411853233529750777574267.6711657', '-5.383749612083781119446807426128870588768e+35216', 68, 2);
    T('2.342238076374768E-59535289185372', '-1.937927906E+306412800134625', '-4.539088530702422221476e+246877510949253', 22, 3);
    T('-2.899897041944674397774869146758210207438269336217670E+7389439739197', '1.80514810336693889904E+19829', '-5.234743645225825447996503985454597266e+7389439759026', 37, 4);
    T('9.65662035332045809299455105319760017832591504E-4867937', '-1.62508588325514E+23147039512', '-1.56928374161353387389647277903208318149885628140594542833056e+23142171576', 63, 2);
    T('-4.557900690555217636711831583957426123782035418602888E+183540024246012', '7.8157565049501533126223236376372E+3729', '-3.56234419712e+183540024249742', 12, 3);
    T('4.32904606263670330822024E+9518037009', '-7.40606308663341430966E-81702792', '-3.20611882448294119229953683674152015228395184e+9436334218', 53, 0);
    T('4.4990660534184307E-1928144', '-2.89626284919159542420467517860', '-1.303047786657485075732062232739773124405422302e-1928143', 65, 5);
    T('9.0471501913447097812857529470336490584430093085E+51185830697216', '-8.25672300319111590566950893630E+940412302248994', '-7.469981309820077105604809055191408764625526932920489566184308e+991598132946211', 61, 3);
    T('-13777274.116713165523744', '7.71E-216', '-1.0622278343985850618806624e-208', 72, 6);
    T('-1.0836950991936814129718E+55847146320', '3.1675E+8817135656976', '-3.43260422669e+8872982803296', 12, 1);
    T('2.8370864E+798600086106', '-4.734478526072325226E+265343100', '-1.34321246374118393150615264e+798865429207', 30, 0);
    T('-8.69121191266168255009795909385545414196740213762361E+7615', '4.1517816091716580784191521136913194287E+866752622103', '-3.60840137804024045865540758488306531299176807284118134817664112009312e+866752629719', 69, 5);
    T('4.497264663419915880436695036998923E+85638626', '7.5554066787569876723517318606721E-196588249087077', '3.39786634741406286707615921875582965863936244091351587391537561483e-196588163448450', 75, 1);
    T('4.2032889357928659886962000932324902553661673603112E+403059125361', '-3.69214749730188794604340528549E+3415257395', '-1.5519162724724346134074584618675308227322720299e+406474382757', 47, 5);
    T('9.5730977638E+426894', '-4.90710530381145107097687E+22391622', '-4.6976198810648521864401789278523306e+22818517', 40, 1);
    T('4E+350205', '5.324832236799171933228560081781982426187674E+56932149', '2.129932894719668773291424032712793e+57282355', 35, 4);
    T('-3.3620338900639477441057E+3887', '-8.0225415288561329879892E-5519', '2.697205650445975547273129659873398169319525844e-1631', 62, 4);
    T('-8.6062467032E-1274', '7.5518173E+9322769768', '-6.499280274129372536e+9322768495', 37, 6);
    T('3.028289865908659356449E-5517076', '1.6447885261830984269433521489410E+6220205', '4.98e+703129', 3, 4);
    T('2.5971875619135440157122968287909163923E-949013717543640', '-8.83841379441394422068184891858130E+2444560495372', '-2.295501837389698724517861024247133936520651254363531324289437823024399e-946569157048267', 78, 0);
    T('3.941232505434583543908206056426455073925725101E-97', '-3.31007754358200497887325158866869733E+314857219', '-1.30457852102744573853079345433887176343197184212086115075800494657988e+314857123', 70, 6);
    T('5.714578650805479908853531492675495709643E-8178', '-2.8757022140745574521304834375411323766648036122797E-65372', '-1.64334264786245158814259309113032247089534895779120652495e-73549', 57, 1);
    T('5.1440356615861508801169227675522975606389E+86072748434710', '5.5837115699354740941E+393503653731907', '2.872281143975927166002879239354455772993e+479576402166618', 40, 5);
    T('7.1118606487176790322306477489226800213022149E+286169837890', '-1.227689E+360', '-8.731153087963558653400211704227136113e+286169838250', 37, 4);
    T('2.69852097784992598064242999706110821952372572612E+8810504', '4.977141559440204500548587638345426E-4751', '1.34309209078780861426035808e+8805754', 27, 5);
    T('3.832988084989118460102348524358304801966637E+716', '-9.489940864164931097061722446857928395593699990827518E+666260', '-3.63748302e+666977', 9, 2);
    T('8.006795234592261706731636507814469610447557517356144172583E+100005877', '-1.635279506138567355206251168481199967380464111524964354E+452606691829', '-1.30933481569766682745422925041643571564169074996506063156e+452706697707', 57, 0);
    T('3.481640909335826755053438600E+17027588972588', '5.2627107051637360239070690E+49148804', '1.83228688850976599430119845495599972277182552974e+17027638121393', 48, 1);
    T('4.6414761154884272568690602769115652387326812925E-701', '-4.3242328047110819403E+258908846482', '-2.007082328087801946857854251808417209705788293337368689649e+258908845782', 58, 1);
    T('-8.4811007088952290337344892530507267631084793295397E+5183261933', '2.955701053869822E+5906848077', '-2.506759830325772290249786831624021746720210316209e+11090110011', 50, 3);
    T('-6.085094272853890356088915277002311E+17571709', '-4.61280298339401195158834422181994376594994756348130E-58242016', '2.80693410160542412277680983347964283990759210855219868198464685021e-40670306', 66, 6);
    T('7.089105931986919187650E-11278437835', '7.47934972297087307398806412198E+45', '5.30219024885175370018e-11278437789', 22, 6);
    T('-4.680254938049644056560098971E+641707', '-6.64882420434999074598432015409913123222194815217563E+67207214', '3.111819231463303987400632405120483079665416355209e+67848922', 49, 2);
    T('3.1477642755861593202697339380138196944208222E+3479774368', '-2.9646953513943936751731012547686807723229E+9172229817', '-9.33216211511562765797602037131316741871762611033085289863588e+12652004185', 60, 1);
    T('-8.3771E-520574', '9.4002025585E+899853779510471', '-7.874643685281035e+899853778989898', 69, 0);
    T('6.7717164459341879772381762224396030292311847572405879E+14560', '8.44238706841622935394509E+8042', '5.7169451354136296919507185334463363e+22603', 35, 0);
    T('-1.5765536592153653957815528497617E+914', '-1.260587829478013074990710875877389084963033473510906042719E+24826', '1.9873843553259165702393875625895887177109054670525e+25740', 50, 3);
    T('0.693282854965312414867805453154981886753337572784308', '4.10121035400433027804578644236E-46838', '2.843298823037421694226e-46838', 22, 1);
    T('1.488756107803184904507578227013506729341E+3066247871091029', '-6.750347081353542374448174462924E+31843191', '-1.00496204471564889119729548120398700454242906013568871224503e+3066247902934221', 60, 6);
    T('-8.913041965938288509192500635466711949352612999727944E-64574', '-6.7803743325099710802419889490397947049946417423E-1315288511851', '6.0433e-1315288576424', 5, 3);
    T('5.91382223744140186136310446864E-2931998147484449', '9.3579140158797408001181402296346463598E+127', '5.534104000317418292662003145478225595080386888319909143561e-2931998147484321', 58, 4);
    T('9.5777729489290505481899586526858087173525042396E+39418200', '-3.1285941069043304302E+4760737608', '-2.996496400528713808850610452069869336265603447115087216e+4800155809', 55, 1);
    T('1.22031E+6111346', '-3.9920167490391232396460620702496906673180245185085E+623587276826', '-4.871497959019932480572486004946400018e+623593388172', 37, 2);
    T('9.6962990117466321761887E+7202799291993834', '1.23451549231430459807798570421244466465071262769460579614145E+866259263377227', '1.1970231348113098764061618876016746897435277340993e+8069058555371062', 51, 1);
    T('6.8789731246770E+89785813126', '-9.375310292293278205291771805662816511348E+50139442', '-6.449250753619313016793961e+89835952569', 25, 0);
    T('-9.92264670623572059766949766542748857923971487147517715944E+867367844168810', '-7.7428655811905633679470920342269553161810170851423E-369726608', '7.6829719656026472063717300753885693324e+867367474442203', 38, 1);
    T('5.0200772283995467472353457562825572106251167086715E+679', '9.31564999E+844633018986', '4.6765282382539465371887481222160146516334296360884491888285e+844633019666', 64, 0);
    T('2.77319017535307705915695459457797910707580366E+6181', '-9.6524E-64522185737', '-2.6767940848578041005806588528704485533138487247784e-64522179555', 71, 2);
    T('2.9958660560881175349658462E+664930540', '-3.2904664512967126428641335441088687631E+7563', '-9.85779675013654638371879024325496381948582957640408547653883522e+664938103', 75, 5);
    T('9.0584771015625986653393580996050067837E+472337168', '1.8397153601005786764E+203884501987023', '1.66650194628640824032151912538879801073609786e+203884974324192', 45, 6);
    T('-1.5219602083507760946610509680022630021183E+8598094', '-6.48282158748638679497064273575108958304293184348E+992', '9.86659649399169028248480229590424155817369875575639019843e+8599086', 57, 5);
    T('7.109E+216871', '2.426821241837223205867705040814354323E-78652455', '1.7252272208220819770513515135149244882207e-78435583', 72, 1);
    T('2.34E-793711153411', '6.625707834406417614799880588678892682938730060672548882370E+418676084899439', '1.5504156332511017218631720577508608878076628341973764e+417882373746029', 53, 1);
    T('-2.893361421330768870354714664480406086953643675600037E+212294322', '-7.23753450680864219327016366192713E+84008', '2.094080312755033826437107887852632695391239e+212378331', 43, 5);
    T('-7.3686614573798336852705096558E+5046', '-9.5344936569914E+7986002', '7.025645592590502962339314207268902687756012e+7991049', 49, 6);
    T('-1.5689212119969590857203526434587407756287E-478438627574102', '5.197084240622E+51569115', '-8.1538157056469639861841623218798201194966296241290514e-478438576004987', 79, 2);
    T('5.23581364183696779895616009261E+1793853294', '7.98322913059612181757225087295187494835285069425E+5067314897', '4.179869998788545078814024023350642693174685960173253753e+6861168192', 55, 3);
    T('-1.8055821779794289380921745994319519884172234085135365373990E+68166950', '-7.346352074588074438343185824627663244E+4676', '1.32644423790384316338867833287194146240438751096945434e+68171627', 54, 1);
    T('-9.70504334165461274289336E-602779246598247', '-7.9567480997798203782805161087506026056328876656679493E-362517', '8e-602779246960763', 1, 0);
    T('-8.141763540542399009452882931389164780675093444185788947E-60100998', '1.4913832628813237607284458868432670367366287501507E+6185477024643', '-1.2142489874702321946535983078709170671640395578841776937e+6185416923646', 57, 5);
    T('5.9221396996709570171942578372056379100345594554378216E+645921593780119', '-2.694875430500319746513570187202196475E+8383905865', '-1.59594287726338045835802168751875351805183622177831e+645929977685985', 51, 2);
    T('8.707966024E+568356307', '-1.3020355666471902E-85417434486060', '-1.13380814764033198566657648e-85416866129752', 77, 1);
    T('7.585914994722053526814042659213840782657553238353426E-7285524352360', '-5.0394719571220340174061680750730773574923341751205874670E+792046395923', '-3.823e-6493477956436', 4, 4);
    T('-2.2849940678E-11', '829662.733439472', '-1.8957744241839262103642016e-5', 66, 6);
    T('44.011189664', '7.6456783496335194E+7006282153620', '3.364953999556593272052234816e+7006282153622', 55, 1);
    T('8.60E+62623', '1.850541089914899292953234554431328901700E-2220105', '1.5914653373268133919397817168109428e-2157481', 35, 1);
    T('9.456785438181609E-545249', '1.656927851797864484318709574E-19884135764567', '1.566921118099957998503971042148701993902457e-19884136309815', 43, 2);
    T('9.338510071979654668530955809E+16942', '-1.84511644592599E+59476264', '-1.723063851425516147587993098126715877457591e+59493207', 74, 1);
    T('-9.15688541E+31543788', '4.92127184858838882246055520747216928E+1111', '-4.50635223889827467037961382798014298610822048e+31544900', 59, 0);
    T('3.71290078506516736949235E+4737681498646436', '-9.26722958738692236809573940364705190E+76284981933697', '-3.4408304010388051133928411590042570967423e+4813966480580134', 42, 6);
    T('2.922E+59265231', '-68087.675893044983868788063537763', '-1.98952188959477442864598721657343486e+59265236', 66, 3);
    T('-1.372147152803513858835267697704571512458443944E+46', '-8.0595319077446032243882419187972183167152718E-24', '1.1058863760140829641312175967830422118173414064228260015598e+23', 59, 2);
    T('-7.417134189647540961526576121325046654598E+25643894', '-1.5043218149691703544973894053714924808169753558673267E+53575', '1.115775676604047541181837757310234756622e+25697470', 40, 1);
    T('-1.865383273845E-59', '2.529958272378594807289322595575068E+9360', '-4.71934184482082341692207345344615325629849646e+9301', 58, 6);
    T('-6.82115E+711', '2.96723592979431112624951923174140515992E+53597908', '-2.0239961362516465338816908107592885806588308e+53598620', 65, 0);
    T('9.9824999238760006617916314045458974464E+509', '3.3626893737814165252274241955388236818156499560436152E-7637981862', '3.356804641779162679874589704708833821301298e-7637981352', 43, 3);
    T('2.9922668750387596745083506368644470338071287590E+54', '-8.4907578524253895586268102846816101E+2050329', '-2.54066134657877305959391052721197322431689569023620804712319419298e+2050384', 66, 1);
    T('-4.227052755254702135638788489761221546547072E+81', '6.6103667182104819130845843655E+166414142216', '-2.794236884945560076106564218555174612768107725507889843013e+166414142298', 58, 6);
    T('-9.9300129744784312435601317706414E+23004622832', '2.08837828756436841603066178E+66986', '-2.0737623491133226652443700308979908864965129399144817065692e+23004689819', 78, 4);
    T('6.9473565132992159447919442243E-79', '-8.712362227494285967230782E+76385140135', '-6.05278864674044929799946283781476599349891649e+76385140057', 45, 0);
    T('-7.9295306748780307933052909732205464901822960785831234395105E+89110522790', '2.097263801198200623971864604E-320', '-1.663031764491243200052736624139999558862097255022499165e+89110522471', 55, 3);
    T('-8.285974995555543845215E+24831952665093', '-2.650186067824171119881173E+677153331312', '2.1959375491560750514590807661728074582304637195e+25509105996406', 53, 0);
    T('-9.675497753529466291122717E-1903557', '-1.4696241887734409192083957054692810837158531595788E+91420428987122', '1.421934553700999190845319266137749e+91420427083566', 35, 0);
    T('-3.686862591E+8321', '-8.40E-25677155235', '3.09696457644e-25677146913', 24, 3);
    T('-2.15476522153654166054717822969519458443512088858E-2589611', '-2.27176620974659945817295495579E-47', '4.8951228202238609500409372071096087950887671673760773293247e-2589658', 59, 3);
    T('9.2764883669670491277', '-7.1270546E+71685034', '-6.611403908763899553400027242e+71685035', 41, 3);
    T('-5.95875378043268865998158671546090780536243912840699E+8154573323', '-3.301927185622363051337280E+155370839044', '1.96753711000407239339920634648388104305988355564e+163525412368', 48, 6);
    T('3.8861505506216272896386560E-2961913031079', '-6.2680753E+212786', '-2.43586842784328216599900055987968e-2961912818292', 41, 4);
    T('-9.15156380078092484109701828113679407419805257450094684211165E+135480129', '-3.9E+2399198535239', '3.5691098823045606880278371296433496889372405040553692684235435e+2399334015369', 65, 4);
    T('7.7239314182904763140429738406328243132948437635E-92557865215', '-3.94921424905406186736518932739192567658341460970515923221854E+942422', '-3.0503460015829098436288949507e-92556922792', 30, 1);
    T('8.397419483150407712908E-4645085', '9.5676597637075E+7693451865', '8.034365250791158663616364077768641e+7688806781', 39, 2);
    T('-72561.0088963751505', '-0.000347947473556615976262995585405822214917759541016', '2.524741972421286928195344877004364443220476087088835783e+1', 56, 0);
    T('-6.0817591393232179049087471547678658898407650712336468E-79050290593753', '-169', '1.02781729454562382592957827e-79050290593750', 27, 4);
    T('1.53507557259058340425883221891228E+5556780254', '5.4989146558720811589856902968E+4796227739525164', '8.441249563989585881154100033601771285289173356701287846364e+4796233296305418', 58, 3);
    T('9.672997142889192E+864430', '5.8047990159046128389312210787300545510991945568474E-95530790', '5.6149804295891313382148462328600904995724736573287851736523053301e-94666359', 65, 2);
    T('113051.554594046135026672177359', '1.1891990060719E+32244053047', '1.344407963581228040672e+32244053052', 22, 2);
    T('-1.17736304111273964311716674507348245159266295102361509027891E-1531445207342', '7.4E+700816047060', '-8.71248650423427335906703391354377014178570584e-830629160282', 45, 0);
    T('8.55604550457616527780231578078123177212271747706107930E+7982829', '4.8467411836399315120777122E+2845847253', '4.15e+2853830083', 3, 4);
    T('-6.889979411158511637363999268378359543354927309011E+383742685', '-2.40211601E+513789235504', '1.655052985211423345188337684019994419662916000136154036e+514172978190', 55, 1);
    T('-5.934E+81640731957', '8.467075134932029649574368853609834025072E+4070680188461', '-5.0243623850686663940574304777320755104777248e+4152320920419', 75, 4);
    T('7.9918169664309743340367677552730174E+44050128990', '2.060218735696947959360165556548805043325255089139E+90', '1.64648910465018399336207133902964696686052717134442886628901e+44050129081', 60, 2);
    T('5.474043', '4.486275739953192667490915447908984933E-266050269990', '2.4558066310360594649129973271218043609594119e-266050269989', 59, 6);
    T('6.3935858592268E+87669768', '9.9275566940534339454976496024395524190343604671342E+8499443', '6.347268609577239452253566803473336117846062000641484832922383656e+96169212', 66, 5);
    T('9.46834686295356799314584830683198517040360993E-652352837382603', '4.0854526537892953E+1187', '3.8682482818251203451112587116302742122777529053964912711e-652352837381415', 56, 6);
    T('8.635E+72413523308', '20525427408182499217350898727.526924589190068929547501', '1.7723706566965588074182501051219499382766e+72413523337', 41, 6);
    T('2.0312795162093054908273549152566E+6686987276003', '1.472993408014536387343387893063069195713616535575236E-1180518', '2.9920613e+6686986095485', 8, 3);
    T('-8.28358250964E+6208269287053076', '7.453171296592963375204787093260579615437783395798E+82043962841066', '-6.173895939380835233714347521893335282132913986904311827049272e+6290313249894143', 69, 4);
    T('2.22E+3952873044557', '8.82819763031685372070932052937677577601E+56775760793628', '1.95985987393034152599746915752164422227422e+60728633838186', 72, 4);
    T('3.05800912220100386670345080865435004423558530948302019E-392070253', '5.906163E+8350379', '1.806110033120605e-383719873', 16, 0);
    T('-2.99275271134E+12100', '5.98142E+892707694709035', '-1.79009109226633028e+892707694721136', 19, 3);
    T('4.06984788565766210638548617938139360003009582E+9730', '-9.7507359584E+74485618038', '-3.9684012123e+74485627769', 11, 1);
    T('6.69934657478E+6782132970972', '1.7480665E+268987461', '1.171090331926266287e+6782401958434', 44, 5);
    T('1.031053620621956715493733708635E-7810717053652077', '-1.8670970157791E+851886685', '-1.9250771381714917027841209356384561569225285e-7810716201765392', 72, 6);
    T('9.434187802830789482446610162349966010449822E+328038346788268', '8.195989069158938717E-5903', '7.7322500108393735560091559e+328038346782366', 26, 0);
    T('-9.523818599871194E+4554594238552386', '-6E-23234', '5.7142911599227164e+4554594238529153', 45, 2);
    T('9.3792073011953351183675698005279720432887687381230E+9015', '9.6872492438782922833982526885016415835786980966457E+445165258', '9.0858718836682268517569969979690654517177053319372846401645604604194552e+445174274', 71, 3);
    T('1.305260836804750329516941075206245012682440490471157523E+9201621595', '-7.401378E+93858492', '-9.66072884178826938e+9295480087', 18, 4);
    T('8.540737907850866444730184100442491124746912809374540175539E-78659402557563', '7.8190369700', '6.678034545256638e-78659402557562', 16, 4);
    T('5.28557531385600131243264638647206145071940648731419E+6380', '7856684298.60463582225047231', '4.1527096577464715520417086648897764439166163e+6390', 44, 6);
    T('3.006748775364629073323202767281489267290227471800018105E+1469294', '-8.4563211757908866950793236182206817324030640062823138704E+4333220', '-2.542603333939922878064556227942508e+5802515', 34, 6);
    T('4.0878835230614640729080E+7130', '-3.611296396814201804482498715488015859535383762118552677E-68000090119', '-1.476255903743e-68000082988', 13, 5);
    T('3.0576825568250272830119660402286056531741017683188165881E+952', '3.311865307300430592357383539943822E+9965', '1.012663278068648530972358279772932683260323979221171771613617807690032699e+10918', 73, 4);
    T('4.083288770809078273576543926E+3932876302', '6.44819792435335820905955445410755084135521813E+907', '2.632985417646647393115586001212885614772413305489473115247823e+3932877210', 62, 4);
    T('-3.632165545999482E+1524143', '5.5005460555016714478121313250279296396210683229507E+1472586718439', '-1.99788938669765254953e+1472588242583', 22, 3);
    T('-1.5034890E+336872170449159', '-4.13944676112E+633021738', '6.22361267142954768e+336872803470897', 63, 4);
    T('-1.5048278026343422172783965719748E-730962', '-5.96218468E-160349685', '8.97206127e-161080647', 9, 3);
    T('6.0105E+33', '8.62513721104640680447127407162112989851662616905E+4025647177', '5.1841387206994428098274592807478801255034181589075e+4025647211', 50, 1);
    T('-7.667388896811266869196458175628516601132484813741179303761E+16733477070', '4.03700223829846177523694320887749125526367392E+934321905801356', '-3.09532661383318579161356731953868955563925430651141937311201977e+934338639278427', 63, 1);
    T('-6.981681220276287963348810E+3496229198401081', '-2.837847110373E+2572922943', '1.98129438765066e+3496231771324025', 15, 0);
    T('3.5428422468651505245094951529692218923843111648851353441827E+3903247312977', '-6.750E-522113910', '-2.39141851663397660404390922825422477735941e+3902725199068', 42, 1);
    T('-6.774762725289227901352910163226277E-6102453269706095', '3.8394213202110188024349310925088254236E+382680', '-2.6011168446846367087864e-6102453269323414', 23, 1);
    T('-2.534089212421656701378113116019E-33306385874', '-9.552285380428029226788714777983981362855843642572E+4022736511963094', '2.420634333651576995015964068422987354464500807175552507413977801936795020356087e+4022703205577221', 79, 0);
    T('-6.7400248009616673008114231384E+334982829772351', '-2.92433804339421550576E+5651682434491', '1.9710110938872729e+340634512206843', 18, 0);
    T('-3.002007514770183E+87', '2.5417328487122636068482587135274272E-9307', '-7.6303011123724400023269718373385223402486752631776e-9220', 57, 2);
    T('3.082560578853313192479538237023122234911479652261651E+55494646910115', '1.9102E+89555261', '5.8883072177255988602744139403615680931279084317502057402e+55494736465376', 59, 0);
    T('-9.013886556277415161975879E+89562322', '8.58702214918913285050191284899706627E+562', '-7.7402443509032321042844913978959232595421512156986e+89562885', 50, 1);
    T('3.8611741208419185159934126446250160435642031122E+8278', '8.84602E+333926891', '3.41560234964500280308480481226057844e+333935170', 36, 1);
    T('-8.58775363221156077837990563866212375E+54264606', '-3.2181664241569435219320138828039188210594265920835061087E+1662862812445', '2.76368203981150820621412444154161197812554814512859993920866675e+1662917077052', 63, 3);
    T('7.2165716860206874644017789283391816438114313508E-3134440063', '-5.984263261E+54047', '-4.31858648110264272791828109839041164578663366444160429588e-3134386015', 58, 5);
    T('-8.045587363E+2844', '1.32621205919893756899268321023524666586337E-3520804109447093', '-1.067015498414918000811347267573097283205821315659331e-3520804109444248', 57, 5);
    T('4.230E-27287501169', '-1.895254932464684250244848672122E-98', '-8.016928e-27287501267', 7, 2);
    T('-629818992.89745629227', '-9.2584793556170368459516572865552732392E+6966322', '5.831166143516612238894536758759446851851144e+6966331', 43, 0);
    T('-1.696665528712487E+355313888038', '-3.6107289081355164165612383299343862E+86677388868', '6.1261992719592068640282910604784305813588746204794e+441991276906', 70, 3);
    T('-6.371661419608830047757433299738875487987220283417450E-5171568680204662', '-9.0009132265511375189669926751402534174E+901556', '5.73507715468627157897470100720728072912e-5171568679303105', 39, 4);
    T('-7.78641858356047775324297270091959920677413045428E-12662', '0.46340698014109450311645552488841482351171197539', '-3.60828072192225950409171222774141147684686e-12662', 42, 4);
    T('344920.62879786267129238328634770167525250844098616678', '4.329459755385287E+990', '1.493319981182534e+996', 16, 6);
    T('-7702336081708684201444654024856256.4567232', '8.247929893337E+7394607929226', '-6.35283280168532342023280222925415310638581005325333184e+7394607929260', 78, 1);
    T('0.00006', '-2.26039934848233348806313781165191E+73238', '-1.356239609089400092837882686991146e+73234', 77, 6);
    T('9.28893910058967019542226748409117991427E-474453', '-8858491.272431327669491260741734', '-8.228598595271969993381e-474446', 23, 1);
    T('3.50892861819891915589648284021912791297108228400340291037459E+4253', '9.6488438193584305513103422306087149215130460266987540E-6078297', '3.385710421027855922898e-6074043', 22, 5);
    T('1.8875485043607654272574680481910433821891229860518606E+696928468', '1.0683757260251967764735732756395265370139E+6773753952559480', '2.016611003754207067009303372705676500213926713197344992e+6773754649487948', 55, 4);
    T('8.0126988932372994E+422752551690', '-4.812282423854508603493088272115443504772772E-16837783486', '-3.856e+405914768205', 4, 4);
    T('-7E+8526821663520', '8.593491037727175E-40407581778', '-6.0154437264090225e+8486414081743', 30, 4);
    T('6.0E+2736684045036169', '-5.137911057363823051863380536667829246599E+10153772307', '-3.0827466344182938e+2736694198808477', 17, 4);
    T('-5.597325565604673229500727318485281235411673107E+13525577926721', '7.29810198484754800943092756530426018060221076256390168E+940', '-4.084985282017739e+13525577927662', 17, 0);
    T('-5.7546066289381E+43167564088465', '-6.80246E+9532455', '3.9145481409086267726e+43167573620921', 56, 1);
    T('7.77389717483007020974E-547716453', '-5.947125167388442324311816088155250690293755489E-32', '-4.623233953712182018027377551071124049599151e-547716484', 44, 4);
    T('1.66393622260073832724914164556406805375638E+2626905135', '4.19270651707723824889483004679377380361E+545098854', '6.97639624449899779377551401984742053160764253e+3172003989', 45, 6);
    T('3.9025E+29950195614', '-9.52727557686733931157E+97817', '-3.71801929387247e+29950293432', 15, 2);
    T('9.40968610267124556761E+647434873139', '4.0781247275380773012191426712247496857911636871742185370E+546115528709110', '3.837387357367500580460410774572358280416180853832937864143935157282674878657e+546762963582250', 77, 0);
    T('-4.9239789058002985E+2484946450', '5.3424484749323052415587633864286E+4461292', '-2.630610359589164581327189171902240568869673e+2489407743', 43, 5);
    T('-9.220E+7355', '7.844997312094193458129948496742299931793638509E-9322158', '-7.2330875217508463683958125e-9314802', 26, 1);
    T('-4473.32110322015574651191770', '-6.91667906715845027395448372025E-833197', '3.0940526435321e-833193', 15, 0);
    T('-8.93636058E+2111514', '9.4563334791241910666678339368733763367E+85588134349', '-8.450520573417967397255858314746145e+85590245864', 34, 5);
    T('3.55988E+82060943497', '-9.1423923693229E-94882033', '-3.2545819747705205252e+81966061465', 43, 2);
    T('-1.8385420857241275E-16', '6.73333122715780413946307954672545534233831670655542631810E+16143', '-1.237951283825010815479e+16128', 22, 0);
    T('-2.8077449734538E+876', '9.085033469267711766210836988877209714384922E-63', '-2.55084570569959558911451e+814', 24, 5);
    T('2.7659948897213878647700973989E-7881183585698', '4082202967.4834856784139485067584629758', '1.129135254686480626066874627186961336930699147876867709942553364662e-7881183585688', 77, 2);
    T('-5.29186259885989690282914525812062278916791383E+4543196957', '-6.40966475470387169151710787072E+77326174', '3.3919065186647914e+4620523132', 17, 5);
    T('3.047575846234749966635153969845909587095E+8138', '-6E+735', '-1.828545507740849979981092381907545752257e+8874', 78, 0);
    T('-2.753960955913E+1561488030', '-6.056271076951158627811380164110602588648288249416809505487E+3196655', '1.66787340843486667962603259038138819204922923e+1564684686', 45, 0);
    T('9.6220695684433504217E-13392', '7.81E+966', '7.5148363329542566793477e-12425', 55, 1);
    T('6.494186308320410259373324352379751039174667E+94043', '-5.85572181069736226997417082713E+192903988', '-3.80281484e+192998032', 9, 1);
    T('4.173922442297650992377364307683760927734168261947E-9', '7.7204955118617195331683350913891198E-3571', '3.2224749482617921510951705231407781176007417667708999261670809e-3579', 62, 3);
    T('-1.818E+824612211', '-7.31238E+8048735695', '1.329390684e+8873347907', 26, 2);
    T('5.605032552340259996E+435914638125169', '6.54972E-4856878', '3.671139380871404770100112e+435914633268292', 34, 6);
    T('-5.7198033860058931365032909785783200765856317306846929363561E+309824638', '-2.2598824810957324391719710939930291533E+46', '1.29260834673467691916683723e+309824685', 27, 1);
    T('2.4049710519E+45354', '-2.524697571452148247E+25565', '-6.0718245741446483801023310193e+70919', 54, 5);
    T('-6.63643089E+6826849728', '-6.853375760042145542E+1779701', '4.548195459472092237680459238e+6828629430', 60, 4);
    T('-7.190569083630944830594459225179479889940803056008E+564624', '-81138.1396455529837623599474559285783189969649427883', '5.834293984386435532534394605162393635605089142e+564629', 46, 4);
    T('5.01027020209519756439693968504266663468799446579989611345E+838081817', '1.225979E-280120944095', '6.142486052094468214801795718128923398128152767186891e-279282862278', 52, 5);
    T('-8.78608164371658248521500369557E+6616514894042764', '8.7670463067987327449605333956033044337E+2408', '-7.702798462578e+6616514894045173', 13, 4);
    T('0.004371950044', '-4.34313950399160028333E+2767259257', '-1.898798894557421503433500596652e+2767259255', 51, 1);
    T('7.23024959E+360016508', '7.1562E-8028855', '5.1741112115958e+351987654', 39, 3);
    T('-7.039310804042269358151654038636857946514957324823614230799E+840227', '-1.719177428143310E+416', '1.2101824243994804269757669045806239610767994009535e+840644', 50, 6);
    T('-9.1850910223790414025E+467793561077808', '9.92260502249034155606128525E+767', '-9.114003e+467793561078576', 8, 5);
    T('-5.4763482282440996418664243998E+728', '6810802680635753', '-3.72983271930197703501617545837907363754460494e+744', 50, 5);
    T('-2.89110123405162E+439795435188', '-1.71867E+55480695', '4.9688489579274977454e+439850915883', 25, 2);
    T('-2.7814980076952856809416871052875910097879508614E+117', '-7.48161370620471710261381975014924512758365228298647433E-7755175104519167', '2.09e-7755175104519049', 3, 2);
    T('1.72429234478884486128579500695E+1675093', '-7.5426313524119361398655199669868576068938573351E+5272705430477', '-1.3005701500528233403110311427431521165591534552308522746e+5272707105571', 56, 5);
    T('3.011866560974106470489336704105602860322E+6696482', '-5.1533212493797496891127572014300808951891E-92', '-1.552111594896417240334011637890052439707515575e+6696391', 46, 6);
    T('-4.54476717925085920486557676639691158967E+1216', '2.05475269035E+940458', '-9.3383725885800836487466056709e+941674', 29, 6);
    T('1.413524146672583412724797646513445835294999699533592006438E-20007918', '47224027540678774205943445604564937487678.3481598', '6.67523032318805421757025998442712653408198232438325e-20007878', 52, 6);
    T('9.4E+519826478736', '-5.50462255008944653048039939159513463384212262058272640524022E+3933675758980', '-5.1743451970840797386515754280994265558115952633477628209258068e+4453502237717', 69, 2);
    T('-4.9068908532904467E+860', '7.854000324212925280608199849493265639827959122996098108151E-727', '-3.8539e+134', 5, 6);
    T('7.38920250634175937425700690855824100202231101E+70845393699798', '6.68482473733966550093969281912229117052E+303298218', '4.939552370360564961199628818e+70845696998017', 28, 6);
    T('-3.9384012126042765666570728738164258119588996E+533039123345070', '-2.819829340618535968596662672789345871633639824881E+90', '1.1105619294429159680894536809231892845442314688182984002295e+533039123345161', 59, 6);
    T('8.008773706995366610595168709E+570918025968539', '9.18696412720403965676932525388440137363163421351512516286409E+557', '7.3576316749061349445069924311352554976e+570918025969097', 38, 1);
    T('-3.52247294482441E+197467678084485', '6.53455324632075504E+9236718678', '-2.30177870166793782158509314225264e+197476914803164', 52, 4);
    T('2.267E+706146731692', '-55.22532781494', '-1.2519581815646898e+706146731694', 24, 6);
    T('9.88189240997806202617814196657012811E+49158305', '-4.68183449060269408797196543E-53405299752', '-4.6265384718e-53356141446', 11, 3);
    T('-1.9609E+2086087686818', '5.6484771433591211E+51823128133264', '-1.107609883041290056499e+53909215820083', 45, 6);
    T('3.41579100346563622465415060117328328057E-6613166009020669', '1E+702', '3.41579100346e-6613166009019967', 12, 1);
    T('-9.512626406576668559984686432E-151356540', '-7.98055116731369334834444837182050765E+69345', '7.59160017732244963803868518637767011698660622274301213723072048e-151287194', 74, 2);
    T('4.91837128492192302256428576160524226425483287859383635E+5260346', '-3.6753476435977236227260554763658657195716310909E+3739962787062', '-1.8076724312376497922344419712364781558e+3739968047409', 38, 5);
    T('8.989E-7978135741616762', '-6.444092380781960454836219469241789034993150E+844220764991', '-5.792594641084904252852277680901444163555342535e-7977291520851770', 52, 0);
    T('5.621187137023548890429736287966936E+748', '-3.88791563809016731701192425389920150639745566710187707821E+3565248491240', '-2.1854701374665151867926673e+3565248491989', 26, 1);
    T('-5.47213799E-4508759', '2.90568920050769423E+114863545', '-1.59003322612308808832867977e+110354787', 64, 1);
    T('-1.77683424543220628451462E-59571427222140', '-2756.20859821555425819314942542778569230212383892', '4.8973258248640933751951535292498895923756834449558352380999467432650104e-59571427222137', 74, 3);
    T('-1.7503447356603575881931289E+66221663522465', '4.2107983301210781827905E-46070', '-7.37034869005485373941712906938257838557032819545e+66221663476395', 50, 1);
    T('-3.0233778714E+69024264496', '7.87078307407065554232E+64599195', '-2.3796351376734887086742054217648e+69088863692', 38, 1);
    T('-1.23209653373014557086717034686532623617939031998143579361E+120463493', '-7.7602509843888711399920981953540071E+52196', '9.561378338741478140437154791952e+120515689', 31, 2);
    T('-1.542970963747815510253462471343E+769288308052', '7.565135825138872873987677782560403822E+369', '-1.1672784914997652194324319899662356110399719906052214e+769288308422', 53, 3);
    T('1.18808621061149138931310509339404402018593E-4779', '2.051E+17953953328741', '2.436764817964168839e+17953953323962', 19, 4);
    T('-3.64645293092718079374434887842510E-770300551781', '4.122751161969349614405108240361304128058304199035845789E+295241598724969', '-1.5033418058046575166407055370648657352753672602637937645139055e+294471298173189', 62, 4);
    T('7.24279402376393536E+7563', '26317480.20072574647159913744239401393649917822548', '1.906120883183421305181458673117766877879997e+7571', 45, 6);
    T('9.248009950762263933762512E+98', '-1.208581606744192913012469058782280328985662978739983599154E-450935', '-1.1176974725478541334088645e-450836', 26, 0);
    T('-215821138535740757138855153.691350356416495230931881', '3.043120182473130787593274859146023328784466506E-14710413257955', '-6.567696624824422516239030021914684608120742856780138e-14710413257929', 52, 0);
    T('-1.700735016293735216188073E+45364052740975', '-4.68155E-28', '7.96208e+45364052740947', 6, 6);
    T('5.81413228595770890599261222755265421654265335657728E+67275372122', '-7.7842167712669110770189520E+558707676', '-4.52584660507164217731393879994494896208610078931908748601165e+67834079799', 61, 4);
    T('8.7082298631069645700831148092858191026283486744790493590973E+792870', '-8.23535555132376481768358385282109292103223E+86058640451', '-7.18e+86059433322', 3, 0);
    T('-7.0283768736369923366718E+70556321229831', '3.336339385878E+299593428', '-2.3449e+70556620823260', 6, 2);
    T('8.8878959495063497059384098577441715417194121260666930E+551747036257', '-7.0606599E-60104328077483', '-6.2754410526051908164096e-59552581041225', 23, 5);
    T('5.26841430924167886849973145795388425445031246625381942355037E+6113', '1.2502146415361670858460514321E+1906689', '6.5866487070925988760344e+1912802', 23, 3);
    T('2.72237087077783342613523744080671558362912E+97133924430269', '-4592524883.9181512278910523121782740574532172897354421452', '-1.25025559673011257321347215336391328492683240308978285760017431342830083843e+97133924430279', 75, 4);
    T('7.74875274649768458063249781104032859355412801338E+51275', '9.544451179529489E+37624196488368', '7.39575922911921930593782768e+37624196539644', 27, 1);
    T('6.634099144848316341495493843108E+94963956161136', '5.45340E-81', '3.61783962765158083e+94963956161056', 18, 1);
    T('4.5335586043186115413885799938684653292797654474219524E+917', '-4.6701739575705677498062800770354254400225462405675591E+180', '-2.117250732900874968247354375e+1098', 28, 4);
    T('3.80E+806338', '-1.806329377149998613651507426859230633116303549499734368E+80840', '-6.8640516331699947318757282220650764058419534880989905984e+887178', 59, 1);
    T('-7.982406687790418362768767868612665926E-80', '9.0961879981240740737E+83419435', '-7.260947190962454e+83419356', 16, 2);
    T('4.076E+90362', '8.28284299845864657781771811043745896359391305E+9720534', '3.3760868061717443451e+9810897', 20, 4);
    T('3.443337446004361984E+94', '-2.0939990980189400817452428344675876E+1126668438698086', '-7.2103455061079747912652687326189552976853318656297984e+1126668438698180', 66, 3);
    T('5.811207093753E+3222339', '2.05E+570236390236339', '1.191297454219365e+570236393458679', 17, 2);
    T('1E+2083', '8.21045E+806456', '8.21045e+808539', 77, 0);
    T('-9.3652970251060367724483212453200719329903240E+4075958997613', '-4.6297454050143750571044258668565446832664318533E+535571', '4.3358940868579470064539454855348238439343475103777871507989479408565369e+4075959533185', 71, 3);
    T('-3.1170064407498884915491553119643E+164114675', '7.63465447220381937552909536254452492496847796200057405958E-4897639528', '-2.3798e-4733524852', 5, 3);
    T('-8.8153018038159025338866912846061479E-3503', '-7.291238537054462749151589E-3618', '6.4274468227748227779e-7120', 20, 6);
    T('4.33119511305988298250805383878E+6961155988909260', '8.8451758562135163556398E+3407284', '3.8e+6961155992316545', 2, 5);
    T('5.8354505703135366101495501069961968669105312647255E+8052212498954487', '-8.6041698488613880241452442025665384678319E+958043985', '-5.0209207e+8052213456998473', 8, 2);
    T('2.303144353342144054898411327074424E+1078536208979', '-5.685384354617E+75967763938', '-1.30942608729159134846e+1154503972918', 21, 3);
    T('2.493370806424667659054475697163991806729715877943596455280E+92089', '-5.4587533468082755384610420695361791310154295367609568025912E-2903918922073', '-1.36106962344047035119475473534782005635914535562087511e-2903918829983', 54, 5);
    T('-4.9750306149807515519460668283024423156854110693E+288754', '2.576784043321068E-93', '-1.28195795037162004806928e+288662', 24, 0);
    T('-4.4638798567465234639029116E+18898', '-6.25884966132312419898411375238953410353616408866088460713E+6869372729', '2.793875292958509454805e+6869391628', 23, 4);
    T('5.61064849984E+61', '-2.066886434034487779609E+896903452842645', '-1.159657327045524597948604839176256e+896903452842707', 71, 2);
    T('8.76279420903921415504709422E-70520603', '8.1185093097239873024399168413E+90935', '7.1140826365280303805032829176185312406200160582006e-70429667', 50, 0);
    T('-5033884.716694921946125636294678338113606995113546900', '5.12914305396383784123009776E+35', '-2.581951482909048059868796349e+42', 28, 0);
    T('9.444321301057912258743776451001559466557385695E+988370', '-2.26639726236444389092515984505810259742656535158643086E-28790530674566', '-2.1404583941607855248758e-28790529686195', 23, 6);
    T('4.2329905971188831505005364817330589858228660E+19681655804', '3.54343478848120293227611827406238192909393715789911007E-3131', '1.49993261411448706148214839644210606702123178643403078552722180640968419185586e+19681652674', 78, 5);
    T('-7.2696848612089953920110003924025569067716147976410575988E+180978617', '2.884494612043486729549075347681747933894902983219E+43674407', '-2.09693668134114498336549614142947608902755410765904827067218359073695546977054e+224653025', 78, 6);
    T('5.9995070434031830E+9268238', '-8.285915363045755984132791362480106639E+81264', '-4.971140758163565517188295506e+9349503', 28, 5);
    T('-7.189282983567870729841584341064624838710229469623800022432E+13953', '-9.13639060E+3598825', '6.5684097e+3612779', 8, 4);
    T('-2.520418417105822826687343273209645989E+282781319763489', '3.626083172760542833178837192E+3536321402624268', '-9.139246810583187258806230654158045928214821404322048752986822888e+3819102722387757', 71, 1);
    T('-9.73195611958196937870473992515616367E-3912363315999', '-4.5061096576307E+3021', '4.3853261458e-3912363312977', 11, 4);
    T('-9.7643363011687291007297508084404260220517E+3568408646', '3.8734157855152772180052494365919436097788289660217E+224612068705817', '-3.7821334364026809292089361944424723598170141253e+224615637114464', 47, 3);
    T('-7.603100126914810339816122643183353234815720E+245837631209985', '5.262299246219138029676E+73928564720997', '-4.000978806679243913868884303843782389548759352522735784575130672e+319766195930983', 65, 0);
    T('1.610188765E-502092', '-2.04087477940547711135956264E-955728046603154', '-3.2861936405705526241758216382417e-955728047105246', 32, 1);
    T('-6.99211591633329E+65004284', '8.6371495835920068509739580339525874771307632588447E+766376237900', '-6.03919510751851191378e+766441242185', 22, 3);
    T('4.81612091126221886113E-7018', '-7.2241220587327247460301461428099596830920472166782978495E+106', '-3.479224531257334686826795712e-6911', 28, 2);
    T('5.85131234461414E-8569', '-3.096845396332852639276829789984710230E+1899', '-1.81206096969238896126373515032099347872832890606522e-6669', 69, 6);
    T('1.662737873087302104515862285321E-10205262', '6.653498833940268850633715160620740871316603455875864706E+2172', '1e-10203089', 1, 6);
    T('3.65027579929131715472110417E+3498087008855', '-230990717355.90502', '-8.431798254252009228e+3498087008866', 19, 6);
    T('5.55897642E+39021', '-9.976E+2320264', '-5.54e+2359286', 3, 1);
    T('6.7121332234668276E+914232133', '3.32759124888415623951749606683060707345728763E-996', '2.2335235775752818210085724175264523897899433679435020776822588e+914231138', 77, 2);
    T('-8.28599349895035966504684217425547434E+6316711707974546', '-3.303200247612131055355578302707295935447900614E+88413537700', '2.737029577744533623150351089556572720047762195170884588395e+6316800121512247', 58, 5);
    T('-2.2501454924350865219613021629744893E+63949302', '1.4692858281328414295868171298860490477689E-9767', '-3.3061068832718664e+63939535', 17, 5);
    T('8.695087472771576648609608E-9308725', '5.1298749990499049946568261060726483519E-8241501788475164', '4.46047118411229325813839373331114140876923223055124614477050552e-8241501797783888', 70, 6);
    T('8.075832714990463297489587991643159E+5847', '-4.07758E-820433', '-3.2929853962e-814585', 11, 4);
    T('-2.83652463662153174839106964745200022201E-195428341004', '4.41647485558866347781738E-854451824', '-1.25274397348967655758616567240809576348797337082775e-196282792827', 52, 3);
    T('1.412392026781860772257955421423992933423248302269027E-76051', '4.595114945E+4091', '6.49010371046416868951177e-71960', 24, 6);
    T('-8.611E-29680769499', '-2.00485869657409791E+559849903539', '1.726383823619955710301e+530169134041', 51, 6);
    T('-5.44072226718228305188641052918254352316251241929993109E-32574681277', '8.335206051663192226867E+2953', '-4.5349541166836449130215847258102110361818959101980729210941090518756747e-32574678323', 71, 5);
    T('-5.745410054388526441024268074185911871050882961969758061E+625711547', '-7.4269795318062E+44', '4.267104287577713218458754767798932188e+625711592', 37, 6);
    T('8.2278167234456405373811031130857673879582972851E+92371892514784', '-8.25569594116952037637355310841424353854773640145493E+439622', '-6.7926353129e+92371892954407', 11, 0);
    T('1.21128967746205924E+21291', '4.0340E-34', '4.88634255888194697416e+21257', 28, 1);
    T('9.416190833964592363669865850118404753124162166208868E+6241747', '-1.8388155201220297770828414E+401387693309693', '-1.731463784592489123722782714490304751083206305868e+401387699551441', 49, 3);
    T('3.5E+1292036', '5.123821578616592168E+117742', '1.7933375525158072588e+1409779', 26, 5);
    T('-8.40178730840715819211296224871559323700798019735353299E+74510', '-6.88440594356420E+900', '5.78413144825605021209269316843824074983536841135e+75411', 48, 0);
    T('6E+2441561', '872177466476920726899341.9', '5.233064798862e+2441585', 13, 6);
    T('-5.5831328858147543268810E-593814', '7.392986981996849909506559931020545E+83512812710', '-4.12760287435869838283321879873865393995749e+83512218897', 42, 0);
    T('-8.00616731445756719707879398727012135247056535553E+3475', '6.78E+56', '-5.42818143920223055961942232336914227697504331104934e+3532', 77, 6);
    T('5.06085461285295055825563736958997504751E+4676898142706', '1.756670935013162109177000533671E-4397263514260355', '8.89025620472606719563e-4392586616117649', 21, 2);
    T('9.8973799727124274393619482E+619199950', '-9.0851E+6490', '-8.991868679008967452934723559182e+619206441', 43, 0);
    T('-9.6E-92165784', '-5.66847634304345367270E+9134940607476', '5.45e+9134848441693', 3, 2);
    T('-8.4477760122202E+624230', '7.9233737193443726442145854327687203822771513289088770E+159239911048712', '-6.693e+159239911672943', 4, 4);
    T('-5.7470699257234621298380959E+867386', '6.607531132303389178210871330227E+59428', '-3.7973943453742e+926815', 14, 1);
    T('-8.313490785276410393649940715352174E+7358578556', '-1.7404603638394705295E-68021939', '1.4469301196918266800578307749386943602839460286756133e+7290556618', 58, 3);
    T('3.95106589530315605392448090277097318154222975009107097E+589786', '5.632357348035598538019446237657378013718935006E+63552', '2.225381502798358185740352103851335125251749707757098349000841082415e+653339', 67, 5);
    T('-3.780179092754165240823100557398142E+69756943627', '-4.139365054460650750074027452512765213337E-56452998195', '1.564754123614935854529599945530948890253379732851647134e+13303945433', 55, 6);
    T('9.807048988E+49550', '5.135108569960306889546676762860480358534834529231E-69363389514', '5.03602613042993548812981641259739898853629261326423349682e-69363339963', 57, 1);
    T('9.4271622E+72', '3.38551E+1358178602', '3.19157519e+1358178675', 9, 5);
    T('4.20451662E+464434822', '-8.0950340595208730093841707538706500236300571873045054206011E+731828216646', '-3.40357052427215798048e+732292651469', 21, 2);
    T('7.5419402405170185398586029E+650667583321', '-9.31465176536788305965858910E+96866869', '-7.025054697563092330627529451017790153716706251116839e+650764450191', 73, 4);
    T('-4.151478505531376452351784966127814862994412485043201530698E+2436113854', '9.8778353152557900439422444815956203752199532933158707E+2684438', '-4.100762099246316003095503057459943160482467984080834593468494852e+2438798293', 64, 3);
    T('1.257337415822945302086881719942E-532629189', '2.05013654057333296553495009859741420E-9818', '2.5777133800086673235214743883541360311825956e-532639007', 44, 1);
    T('-0.055446439581542292789464412211352', '-2.3922079819304448957823094007611E+91201418', '1.326394153365896297979437530833955e+91201417', 34, 2);
    T('8.3699378693209859259509102735681187942432268771634042E+65604850092590', '9.5E+6093751785', '7.951440975854936629e+65610943844376', 19, 3);
    T('7.153818284484363629890', '-7.9184788116435077123443007384569914945747200390477851E+417001585454431', '-5.6647e+417001585454432', 5, 2);
    T('5.8368753882935400629300936092609750615445504867116E-1519445095564724', '6.74874E-653412', '3.9391554407992145564298839944563912836848169651690043384e-1519445096218135', 61, 6);
    T('4.4854027164192611123368085190533828454E+68', '5.9686321325850735174423807460351437E-515', '2.67717187808043762032325634786471965502e-446', 39, 3);
    T('9.76261950011159003E+42', '8E+67346616', '7.810095600089272024e+67346659', 70, 5);
    T('3.13356269701652303E+208873083', '-9.128778869566704003952438775386169699045E+458026296', '-2.8605600935e+666899380', 12, 0);
    T('-1.005092812325471857173199E-4932285919526847', '-6.2919880808467044211127E+5732958081785', '6.4e-4926552961445062', 2, 0);
    T('-1.3364810759034940562478371E+562418138330023', '-4.91183464947266951846889162718117457833860875175547E-6142313283', '6.5645740569872949522337719355474441690027413462726615274e+562411996016740', 56, 2);
    T('6.13005093651949204631019227835451840998994161E+252', '-7.1188894E+5674658', '-4.3639e+5674911', 5, 6);
    T('-1.85E+72090843', '3.9E+91', '-7.215e+72090934', 65, 4);
    T('6.5235467853072323649507207723570256115887218951954774961E+7293', '2.4967989189380079751951E+1432051312', '1.62879845611966149775601e+1432058606', 24, 2);
    T('6.565782727572149592171321119E+3945', '1.545146071E+977039931', '1e+977043877', 2, 4);
    T('-6.5334612003376179555073998865397477E-5779499862419074', '-6.4058E+808328554', '4.185204575712271309938930219319631581e-5779499054090519', 37, 1);
    T('-3.42768200666E+8952985702795', '-2.76038157E+954547947', '9.4617102390048812562e+8953940250742', 68, 3);
    T('-2.2367910867165778388928640254364693042613195730500357623786E+46662691', '9.58179218000773611140979675697E+25788358288525', '-2.143246735e+25788404951217', 10, 0);
    T('3.50967677357883633707902462230848851270447E-7565', '-9.4447651248616841120590666E+87241', '-3.314807279063447081520348910994002837e+79677', 37, 6);
    T('2.21788836714967459E+508124485', '-4.84613541452837397412410077672164690338452991513E+7911260380785117', '-1.0748187361514546759889416544970300456973e+7911260888909603', 41, 0);
    T('5.9370626673084684804694594320431877058768188656280543419E+584725553', '5.272974892794853141659001429400195860045E+44209161', '3.1305982381667196429825358929771776229578557124384664680733615572137693728460456e+628934715', 80, 0);
    T('-6.4961024522409109141930460015352473478593768445706E-50648574162830', '2.207725785376215620989543536729544965953279161143E+846', '-1.4341612888258e-50648574161983', 14, 0);
    T('1.33101455372519240665967734412E+7443393184', '8.79E+1163365909681315', '1.169961792724e+1163373353074500', 13, 3);
    T('3.0004928006666950721396082999803080688183758135E+17051', '8.953404260943047817268E+3660', '2.6864625026418126685412851270739637585335744866553243152850098847518e+20712', 80, 6);
    T('6.519816824715416991914895E+14766', '7.70395553068683372100E+14026822045094', '5.0228378885831407461076415243242884248174295e+14026822059861', 56, 4);
    T('9.509214078201912008401533117E+137120919948442', '1.8603887175452937996854486939050634680E+10035912117', '1.769e+137130955860560', 5, 3);
    T('-9.095458732107025535571012142828075721E-581010946480613', '7.51839779661E+3152446483', '-6.838327689063064504933810287282615911e-581007794034129', 37, 2);
    T('2.270055167415007273984785E+585', '-2.50433243850753031047563736572505587981185136564E+726882634', '-5.6849727929590451281610141942534e+726883219', 33, 2);
    T('-3.2517891785599579E-270845731', '-2.954959330263569278017579695863833E+9533336', '9.6089047732358552870706336481857836005778246126307e-261312395', 74, 1);
    T('-4.617790871490E+269', '-3.39861228838E+321525', '1.569408080101491e+321795', 16, 2);
    T('6.77706430E+64650084721', '4853061183307258611713817690535778515345.57', '3.2889507691107378268313275687238472449055464610151e+64650084761', 77, 0);
    T('3.720E-82', '-7.6101974003405778723194796082684993963348359500788895E+936', '-2.83099343292669496850284641e+855', 27, 5);
    T('-2.27700294684255524914046115442E-840551975', '5.623742410500887880155306605288426302988500E+10649178002', '-1.2805278040993976726510681231500010629180438819321489e+9808626028', 53, 0);
    T('-9.14449007596578949471759902E-9', '6.624149687094317507483079730739730558348716069142E-354076166416391', '-6.05744710753458762153103233878e-354076166416399', 31, 4);
    T('90338.4995477503', '-9.312560531801802552', '-8.412827453905744331046463363987656e+5', 50, 4);
    T('7.5720669493679214E+976', '-4.3470486700598805324545471481994317898358354E+378', '-3.291614356185419946324416763497654473952738579e+1355', 46, 1);
    T('3.338577855E-3799', '7.0636990635577605459236961521263760E+8593626553865612', '2.358270926797817687201356249323783007500348e+8593626553861814', 45, 3);
    T('-40259048.31982173785', '1.8223123802102022318732790444272E-75154502846800', '-7.336456216869189408709029706465e-75154502846793', 32, 6);
    T('6.22892854502888614590233167668280431E+6047437712007109', '5.82183567046419989818735503655999244836E+933', '3.62637983922218385e+6047437712008043', 18, 4);
    T('-7.129032173E+612', '-382520147.142043203', '2.727e+621', 5, 0);
    T('-1.52293830929575435409077966195855994705614444018662E+463', '8.90418E+93', '-1.3560516834865070004608038450418171e+557', 35, 0);
    T('-5.6663352011902187823903648885483861452638155E+1533769987727', '-5.496E+317210811606467', '3.1142178265741442428017445427461930254369929988e+318744581594195', 65, 4);
    T('5.4608222195678020831791167100642523E-552074290731', '8.3491854437E-739859256', '4.5593417386249018e-552814149986', 17, 1);
    T('-4.655202812993038138099594815950824162034602175011159624032E-638832861645', '-9.8178264E+38731990714', '4.57039730747573128484410478133451425597811949473219832514354440448e-600100870930', 67, 1);
    T('6.91039901061110628508577830946676794334921737572257723E+270651063485', '-5.02738658261418086988026593250409945478382447614127994105028E-569958033', '-3.4741247e+270081105453', 8, 1);
    T('-9.78160793414558365980799441468752560719913905695E+60820', '-5.4E+8', '5.282068284438615176296316983931263827887535090753e+60829', 50, 0);
    T('5E+329045592589', '7.251262094934437787E+692246', '3.6256310474672188935e+329046284836', 63, 0);
    T('-6.244985138189146795436610470730026E+3545314345', '4.36281802724365692276013642224256190180803160048996E+54200340425', '-2.7245733740760329636334078319362e+57745654771', 32, 3);
    T('-4.38443542692209683929900458337579235138051036788617E+377607438940', '2.678619142925230532478342881359795118002912934344626845569E+937', '-1.174423266547308426119205379500614811605e+377607439878', 40, 5);
    T('6.346565339141081E+921221467', '5.1782595565E-94663', '3.28641626183589661899905765e+921126805', 39, 5);
    T('-7.4100263453101604013587438E-743', '3.6427975748899737145077886847553E+1173658087', '-2.69932260005666673e+1173657345', 18, 6);
    T('4.48140816423431074958371E+4244268', '-4.8665E+782039525040', '-2.18e+782043769309', 3, 2);
    T('-1.68881216804556253181306547064896251985247428096851026933E+2488341546843294', '5.4523389929479520563E+9158757330770', '-9.21e+2497500304174064', 3, 3);
    T('2.662867374179840695492538996441805703837730088157504968E+3662194781', '3.7744124E-348457755103260', '1.00507596366598305510916632956535073269558560325947799042808032e-348454092908478', 64, 2);
    T('-7.7200272211E+299598438', '9.5640214572685720264053431759115998561374133383E-726756', '-7.38345059932978664973752373005e+298871683', 30, 6);
    T('-3.026971749E+6242', '-9.626334293235365600439712016108654987246503E+36172123763', '2.9138642e+36172130006', 9, 4);
    T('-1.567253374294789473004009025427441E+60437725', '9.510152E+36230', '-1.4904817812056340696268022441186828881032e+60473956', 72, 0);
    T('-5.77741158583082114651622606625826457E+1814', '7.7659E+990242399551516', '-4.4866e+990242399553331', 5, 1);
    T('4.41864602905560709185018083841870E-72', '2.23402970848035412776E-774239923123', '9.871386500168972286335982439342687363976480136173112e-774239923195', 65, 6);
    T('9.51053580878532E-1920528013', '-7.5302677233497288781E-832812336', '-7.1616880832657904050762726087e-2753340348', 29, 1);
    T('-42745023971044602161.2122862687848', '-4E+9360538108', '1.709800958841784086448491450751392e+9360538128', 45, 5);
    T('-7.894766854021572990650255387991512594376055463E-159', '2.5649152078686994E+72214319806', '-2.02494075664576608990077369071271983056421615295510485960748222e+72214319648', 66, 3);
    T('-9.7187474079430413938910509768E+2056158', '-5.27773575035929E+42598', '5.129298064361268e+2098757', 16, 0);
    T('-1.49566967627437005474405072852E+769245799', '-2.640345976452774867691904163175E-7911347', '3.949085411854e+761334452', 13, 2);
    T('-3.7901622481309554603612431985E+251545120', '-9.406561908683635851755600529699269730112488024660193E+30768680', '3.56523958309993806258716016884e+282313801', 30, 1);
    T('7.2E+9455758109', '9.161574596993657229E+7806572238254', '6.596333709835433205e+7816027996364', 19, 2);
    T('1.02528676415700700952201117862352217118194622346629E-3889', '5.05182510898408765872349745292384452010082587428420605E+6763627545114467', '5.17956941907741451609475088550120692192674517054858697572546163139369629e+6763627545110578', 72, 5);
    T('82476.68590097968401483574689605945951392705392962596', '-2.4333510358165955619597997336363921231029609348E-227', '-2.006947290678689172833659167411945306920809001046229778046503068126702236e-222', 73, 3);
    T('-848161908.5216336585501', '-9.56928439060295985767648268334068492E+4974', '8.1163025119200845297109e+4983', 23, 6);
    T('5.979092476E+77580', '8.27196657841813350085370277007483140579801371120624687463279E-2322535672597', '4.9458853130743325996917913809294782415375406556419e-2322535595016', 50, 2);
    T('5.84300098349848628098745899127374327E+67173016434550', '4.0E-130489', '2.33720039339939451239e+67173016304062', 21, 1);
    T('-6.5467071709890203059821693228423E+94566584', '7.943254589956168E+8903', '-5.200216178505e+94575488', 13, 2);
    T('-1.103210488299497011082130525292928317249134292981415291E+85', '6.55202151788785488013498088338012550427272544045091E+384102853', '-7.2282588580978719727401103050213e+384102938', 32, 1);
    T('-6.98438925302577745022063E+55', '3.06285438517962804992680E+233217685083705', '-2.13921673e+233217685083761', 9, 5);
    T('8.7894095539052703127203576798E+57', '1.32301104916248243613759054057E-7', '1.1628485955430958399590259703069638309704379e+51', 45, 5);
    T('1.107429149880E+77', '1.89498257159037134020522387415352247936199E-362907', '2.098558938293741172876987339688915e-362830', 34, 3);
    T('6.224810206755160426899658140267900393446E-104', '1.4873051704658941827504082722696060336E+70978587000343', '9.25819240567582189115986403707956951834563414193927065887951405178886e+70978587000239', 69, 5);
    T('-6.85153478143105834754661224118002244921220755E+85380170104646', '-9.909234032976320E+830645390551057', '6.78934616342776154770933446125e+916025560655704', 30, 0);
    T('-6.6341321177979347362213760E+8122344932285', '-7.581E+4954253889375856', '5.0293355585026143235294251456e+4962376234308142', 72, 0);
    T('-3.021264416E-94', '8.7869596715321582142E-3299', '-2.65477285804271578122445659072e-3392', 65, 3);
    T('1.227210133454083670549134121E+98223050507307', '5.3123895136326820904211E+59403', '6.5194182439852384e+98223050566710', 17, 4);
    T('-4.0028787666707892952994321224976003320663128E+213599434101236', '5.446879E+394357999', '-2.1803196293725022125991275539957607e+213599828459236', 35, 5);
    T('-8.788655114407476333073E+56881620905021', '-3E-251118773', '2.6365965343222428999219e+56881369786249', 63, 5);
    T('7.7850216536785612463468574693633788E+71898957169779', '-6.41559504291003451835334E+95233532082', '-4.9945546330287457024731931354592e+71994190701862', 32, 0);
    T('-2.7204669485E+738823232721275', '5.80953121843004181156555916929598728779507262E-6252132155090851', '-1.580463766601786280783716876099084943012265250386730007e-5513308922369575', 74, 6);
    T('4.53216914887677161702352693E+40', '2.0E+5846745497', '9.06433829775354323404705386e+5846745537', 55, 6);
    T('1.02317506994215913865264964E+157891147191', '2.67834958007595094653117940409634874237280977350151292019E+2424860583', '2.7404205189237636683326986049824811425233659506465319165840224421563171430373522e+160316007774', 80, 1);
    T('9.2884384231314328181434737205302898112150951422399768E+54163', '-30108440476053215426343692385530779037287565.3', '-2.796603953784e+54207', 13, 0);
    T('8.20375065366899596145511438850344E+722501266', '8.634413991151129432965784411749E+690141', '7.08345794239548023969922149694537027118519765450143e+723191408', 51, 1);
    T('-96085758.79689510515391', '1.2E-91812125', '-1.15302910556274126184692e-91812117', 26, 2);
    T('6.605E-34353', '-9.207E+57754165616', '-6.0812235e+57754131264', 32, 6);
    T('-715139340.6522313374', '-2.9413951839459063029614905551718143296', '2.10350741244472414384867787827e+9', 30, 1);
    T('-3.4761523023611165113834189344226237872289698172617771946E-17', '-2.03433995319513168249155962E+6103811180449', '7.071675512084463000151901781107604776143957822447531274581792328456208e+6103811180432', 70, 5);
    T('7.5870940685615151E+51022960573', '-4.037649598814419522993911406398113658093809E+68906617239', '-3.06340273220946634e+119929577813', 18, 4);
    T('-7538.495965465485112644878093934509', '5.085344E+448141763164102', '-3.8335845227004111924677954945721291736096e+448141763164106', 42, 3);
    T('6.4022594772155625933481094333814727693623536811E-695', '1.607747741103322411563054618E+717429496248', '1.02932182124506586181378578262864803647968648885277263084966992875927e+717429495554', 69, 5);
    T('259.4365772050088172214652543555439615422183877720', '9.27545019121307833817E+3511969778509', '2.40639104964386559458488016e+3511969778512', 27, 5);
    T('7.975568444318318201037E-794539019528529', '-7.2E-554438055503', '-5.7424092799091891047467e-795093457584031', 23, 0);
    T('3726472.0465189532740763146430998458', '7.6705812751963769032312812073750099175060590281764906E+6302893', '2.858420670257100495720304895130292964049157252737594052894247375699376668965e+6302900', 76, 5);
    T('7.8621365142522455352185078072901971046095271878425476727676E+574', '-5.994760983253597150E+53951525723', '-4.713162922065280036758916683145579476827430624442344469370904e+53951526298', 61, 5);
    T('5.10558194804253213E-9', '7.83303363942175609288775189051506438E+9545', '3.9992195147841e+9537', 14, 1);
    T('-8.710153160501563673E+9187503', '-4.20689393667734224193732412879940298975242833487866355617E-4206606', '3.664269052e+4980898', 10, 5);
    T('-9.5911830536045190952353665693441900777172E+78316343510', '5.131834788701E+4431634', '-4.92203668592871592075159070942527209516238278304333572e+78320775145', 76, 3);
    T('9.504771938392460759103990811901E+425654702169155', '1.062625065391042237521403901529234226391228696455917E+537', '1.0100008902561231895572466789253856913e+425654702169693', 38, 3);
    T('-1.51329694281623479283708451624064677E+1675360439078217', '-7.9464605952E+214521844', '1.2025354524925837496111127321558353998367157504e+1675360653600062', 58, 1);
    T('-1.1029620388101371E-1345151', '-5.22101267460689497876E+80', '5.7585787842379878e-1345071', 17, 6);
    T('-8E+704505', '-5.2836074984447744900449565651965462E+74426926', '4.22688599875581959203596525215723696e+75131432', 78, 4);
    T('44.7325919', '6.221065595380E+4212026927', '3e+4212026929', 1, 2);
    T('-7.11E+663034731574', '6.26943410364415E+2768132665078', '-4.45756764769099065e+3431167396653', 49, 2);
    T('-8.044308226772500017048911532405E+8255354', '-1.50758443866696768785746475683128809459453779089403630175319E+384366551517628', '1.2127473902522889650480910320216495856819505717637841160865241467e+384366559772983', 65, 3);
    T('4.5067753997262842916853256089812317828E+155580024', '-6.49299486E+16', '-2.9262469505597209312811559916541507802189036408e+155580041', 77, 6);
    T('-1.5542393002714E+845867315642052', '5.183296124E+6537970937', '-8.0560825408652198e+845873853612989', 17, 5);
    T('-906016474.09502244728855068807029', '3.93708388E-82403172377654', '-3.56706285517e-82403172377645', 12, 5);
    T('6.454405647327582486469752996724690447172253E+92078', '3.644438996699350217725599622985841632531070409280826E-435892', '2.352268764163715479484489478204792442663450641059e-343813', 49, 6);
    T('5.230525306795279156383634961800451995026175243920875812846E+643911101', '2.09097271293587296655866502943245007279E+6074739', '1.09368857e+649985841', 10, 2);
    T('-3.604998506699809027456497126172080888E+40555', '7.33403835E-13194145605', '-2.64391973e-13194105049', 10, 6);
    T('4.738082074287082401145700', '-3.7770894173378705345592878088287740185661689910584564460032E+79257', '-1.7896159661268005080399527886612914670659190156271485518747410896506542613865867e+79258', 80, 3);
    T('-8.2047638658E+77290594', '6.2531788192181815637598761974601303E+91187233', '-5.1305855622307246700721e+168477828', 23, 6);
    T('994268557552628562353578.8371132', '-8.63735776446020788149448340483089259E+488701727', '-8.5878532455358473781607133202415e+488701751', 32, 4);
    T('-2.792530034203692E+106528583044047', '-1.3176221242549011588388E-4282412', '3.679499356e+106528578761635', 10, 5);
    T('-2.4494E+9161', '4.72985479455446866E+58536105', '-1.1585306333781715535804e+58545267', 68, 6);
    T('-4E+36', '4.521340330625486456283E-8016425846', '-1.8085361322501945825132e-8016425809', 28, 5);
    T('5.969680929333146984260042226171246013120560170614E+2950', '-7.355097893394334614925087050326E-82500427663496', '-4.3907587628e-82500427660545', 11, 5);
    T('2.7523937466346342E+62159', '-3.7927519734438285530567222670336147211039664E-34886', '-1.043914681424296190619449692974857362070115578679017323309088e+27274', 78, 0);
    T('-6.012240143532893460956044330513930159872394E-342534794274249', '-4.964488E+8596957110', '2.984769404568732719219475060630444e-342526197317138', 34, 5);
    T('-5.86556336877503737795110502013857842404538E+1670613475', '0.000101058409384121440837529134071751669', '-5.927645041901742088689689558926640622905042896627537240182335013135657e+1670613471', 70, 5);
    T('2E+23', '3.2249640715959917905138E+324313', '6.4499281431919835810276e+324336', 51, 2);
    T('-1.12544772676944398710183435940721594822695518689990243688E-4260756', '-6.13E+31751186976846', '6.8989945650966916409342446231662337626312352956964019380744e+31751182716090', 64, 6);
    T('9.56204E-65135568779849', '-5.85868290503995298E-21556873021430', '-5.60209602853082319928792e-86692441801278', 75, 2);
    T('-2.3557439046741079664461675972986035E-2571076248929', '2.6593980815212E+619257724843', '-6.2648608206456e-1951818524086', 14, 0);
    T('2.8461177069094218743670E+630158', '-7.36E+77321', '-2.094742632285334499534112e+707480', 61, 0);
    T('6.9799038E+34269173', '-3.9006592375017886119980725414853388836460669E-780074437282', '-2.72262262343438e-780040168108', 15, 1);
    T('3.733735189412132399173579412851309534975290119465E+506480', '3.14339541782926809990816271778670979740256142542E+6830387885', '1.1736606085785991393843487173153738512224909220773037663639102924e+6830894366', 65, 4);
    T('5.814880537E+859386423632024', '4.78461594113336523504915337869E+868484811344', '3e+860254908443369', 1, 4);
    T('-3.4839674343330213793713579E+492117795', '-4.01032377569549E+306135908623', '1.3971837435654531416180524179543058205871e+306628026419', 42, 4);
    T('6.503E+6869568295356372', '8.7083522048121873248141624730835295368E-6168591486662', '5.66304143878936541732664985624621925778104e+6863399703869711', 77, 1);
    T('3.22343158E+832211483876834', '-5.731814588027E+5574025596', '-1.8476112154e+832217057902431', 11, 5);
    T('3.603399E+43376228327', '-6.916E-2375854664', '-2.4921107484e+41000373664', 17, 1);
    T('-8.0476315984017090795090652274070704451107441055039132524E+7453', '5.36903442937203434064394483075304617E-38799447158478', '-4.320801112672108e-38799447151024', 16, 3);
    T('-3.73508190518307517430805341266477264139883044486244432269353E+1193250622', '-4.9021291210948865657826365011752761520073263988919599568842E+650273', '1.83098537771e+1193900896', 12, 2);
    T('-7.83282149167674942730953524538452402304730141509224203E-1919566853207189', '8.88636394791974862018196411E+629102930043686', '-6.960530251412725344809767537728639990218010298571241144767265e-1290463923163502', 62, 1);
    T('2.02338E+388718', '6.779318496082771E+23213549181', '1.37171374586039572e+23213937900', 18, 6);
    T('-8.52673363892257E+11776421009085', '-3.795838802850632017662384478854655561153418E-555137780744502', '3.236610640819406131890757144367387733484755492032799284426e-543361359735416', 58, 3);
    T('-6.46742692281823161630348970274367205561634961491E+176932232314397', '-3.5E+9855678500', '2.263599e+176942087992898', 7, 5);
    T('-5.42E+4325811155', '-3.6333989363784558430477888561760048961470842E+3193', '1.9693022235171230669319015600473946537117196364e+4325814349', 47, 1);
    T('-5829971641083.7087', '-8.82707869093E+170939', '5.1461618441736207198145452091e+170952', 77, 0);
    T('9.5815783E+5564828465123521', '4.1297E+3299', '3.956904390551e+5564828465126821', 38, 1);
    T('9.1719136240166688764081914099240038194170136971182859537435E+595255', '70284205177124315402424991384147.498927224656', '6.446406590172494e+595287', 17, 3);
    T('-2.6477351861302583454E+91401440', '-6.9939E+336735886260809', '1.851799511827641384189306e+336735977662250', 48, 3);
    T('1.855930175319629515655504916182883340593892616598087055E+5561733344478007', '-3.582143288783141734938628575226209071703928874533E-252568053219154', '-6.6482078219713305015768256150631426696144484215e+5309165291258853', 47, 5);
    T('5.8666E+29521', '9.16198997560199042853245831E+909142094993027', '5.3749730390866637048028519921446e+909142095022549', 48, 4);
    T('3.244507721854753767397109258217012515186654444338865E+8742796557825650', '-5.7148345651111087433E+4950', '-1.85418248756254459150873977e+8742796557830601', 27, 2);
    T('-3.4681590238055208496949431558199381796154073873184051233E+321791218738290', '9.348367176913E-9', '-3.2421623982458162907606277e+321791218738282', 26, 3);
    T('-6.1548512148515744E+44926', '-5.7478970477116051660966442481906857968E-202', '3.537745112694955095810538403465821e+44725', 35, 0);
    T('-3.23E+93323534659', '1.6419931341748203157209162079902432728687E+1540531177225125', '-5.3036378233846696197785593518e+1540624500759784', 29, 5);
    T('1.2271522988778773891359088691914E+36803', '6.5E+1642523010317531', '7.97648994270620302e+1642523010354334', 18, 1);
    T('1.614006224E-9668', '-3.075875878443305150838479623666552118723E+8874106840511', '-4.964482812058961944584564931294992e+8874106830843', 34, 2);
    T('-6.360952185419692931545347542181878960092254622292505255E+19898469467854', '3.565575379458E+430947632', '-2.268045450224201599939e+19898900415487', 22, 3);
    T('-4.59610453132109683107921941218987039378E-898710', '9.91556822184587553992403461511872627632464035512841E+9700037876603', '-4.55729880350492992870914479827012290949456605933470050025304878786512e+9700036977894', 69, 4);
    T('-1.3622664356377274095942376545553202E+9611375', '4.311069650025466188481E+33236484125', '-5.872825485926176764286127189087852e+33246095500', 34, 0);
    T('1.636068563215920488749195E-2981', '-2.927257582293244632334896415281469572647185835E-589300036441549', '-4.78919410682541787809516159738672398998827372239233e-589300036444530', 51, 1);
    T('-9.5949E+11166', '-6.479239091792503574436693657688167858278707685432539788518E-1890689578', '6.22e-1890678411', 3, 5);
    T('9744881.622303854687192480706705092815660', '-6.0105176315023398887999413550747091325422835E-84', '-5.857178280776044418714910565379922147158967557500417271223e-77', 58, 4);
    T('9.97703411545517801246752321950553128717683429900605502800448E+46361795838475', '3.9349458854342997384065E+584040699', '3.925908e+46362379879175', 7, 3);
    T('-7.6016155855599703192659178109776E+662', '2.950597299210548245604006734372E-65331405254', '-2.24293064163900586518419108545854040244230286675999309488420672e-65331404591', 68, 1);
    T('-5.6363059547349608648694E+98683', '3.675627284891E+27', '-2.07169599532174397492550538673082354e+98711', 67, 5);
    T('7.7977341142280487863558524E+209', '1.3451604632430918495213139441970489155472860773E+6631603564633', '1.04892036333414626009441676111692170110613847932486121e+6631603564843', 54, 5);
    T('-5.8718021825043643411298134646446128E+706076978905', '-67295691612332170266191641.04714508345648260253777160830298', '3.9514698888243268263874862814404705586585262e+706076978931', 44, 4);
    T('-6.728323145969899233581926543119249E+323066', '5.29306207111965684342328183072354149867E+77114935', '-3.561e+77438002', 4, 5);
    T('-7.98959239986137997404250306691773327027361138549E+6440284418', '-9.70540663142705011479273680866706E+2257', '7.75422430600137970323906970252300806977245e+6440286676', 44, 5);
    T('3.3608551E+190604', '1.915421007361429389395116688110910735320679514594650E+550', '6.437452461237797506638e+191154', 22, 6);
    T('8.3230546275176770871832286366847373924193966299076113899E+4012446447861', '84560.743430466976108240143051889469370117574', '7.038037e+4012446447866', 7, 2);
    T('6.35E+2865919977', '-8.7617978367589197160911913219710E-5331599', '-5.563741626341914019717906489451585e+2860588379', 41, 6);
    T('6.86447888402570800819223603E+1309122343550', '-863.5816771', '-5.928038187084457321229551729986445913e+1309122343553', 43, 0);
    T('5.7834367759896257994566569333397090616545006555709219714E+4046306219', '3.66524015700413825548036876806E-393089533964131', '2.1197684717e-393085487657911', 11, 4);
    T('9.254751482545130081915222899012033535386615616E+2942621236', '4.7546535634105099952E+73', '4.4003137114961903038953490654437148e+2942621310', 35, 5);
    T('3.80428104551373388E+32066863564561', '-8.388156164898530118E+677863595', '-3.1910903504932652488908254994e+32067541428157', 29, 3);
    T('-4E+477781718319396', '-4.55919135149168196016413383161485538E+957330726687598', '1.82367654059e+1435112445006995', 12, 1);
    T('-9.739439E+1436', '-7.5777E-6640823', '7.38025469103e-6639386', 13, 3);
    T('9.1376695716464820991268387734504154096E-96828745', '8.7801150295455139539172755692909720584936919103276E+7746846154432', '8.022978994103399601114147511879515785124279276868263246184e+7746749325688', 58, 1);
    T('-6.880703554898982817759933928843278964774069314466570782E+240712698057254', '7.3950287677780207408034358E+113186896', '-5.08830007310304717934047707061293e+240712811244151', 33, 2);
    T('9.926287099525144810516487945957112131125307652203155694600E+5200946439', '4.199279639768149462464289832169347398087620E+74', '4.168325531552917927109485551842025315411650694665819985611400670452401172e+5200946514', 74, 5);
    T('8.094572029783636650581809340623900670498953318088431E+9161', '2.89779349883443033677889526127206467237158174394391624417894E+885921792188', '2.3456398204e+885921801350', 11, 0);
    T('8.5455705948126566830875856E+6436', '-6.6046951764184202094804453E+2632029067721701', '-5.6440888887302243656944183e+2632029067728138', 26, 5);
    T('-6.979087505455728743E+406790', '8752.3096524129', '-6.10831349390344425670553e+406794', 24, 2);
    T('-5.7993835287728125498392E-912', '-8.2358893365040084E+1651', '5e+740', 1, 6);
    T('-1.064E+2988406746192987', '4.637829257604896779140853419184829125410781357378E+588243', '-4.934650330091610173005868e+2988406746781230', 25, 6);
    T('-3.68597195231474354E-7514888164186', '-3.9658428784885237116524583773154315971726810E-9', '1.461798561739586598169716495301154792333197930416529560923074e-7514888164194', 62, 2);
    T('4.909506374464019334669870356659643952750836690E+953491030594', '5045029502082559309560609795200108886.10', '2.4768604499833362437467069425e+953491030631', 29, 4);
    T('-7.2072355165E+2768724436727', '-9.4258431958217613727036418672921936868579694740431755980366E+94828253173', '6.79342718538864629689374812948446780231198368843968000246155111869039e+2863552689901', 70, 0);
    T('-2.226018416E+3067126659765392', '-8.2749716803E+4001238127004', '1.84202393522262644048e+3071127897892397', 68, 0);
    T('9.4556530436952078268E+552650866510280', '-1.478384728176957293E+676323417', '-1.3979093054738958704086704e+552651542833698', 26, 4);
    T('-8.919439210233397057E-3963031437859981', '-7.619376403762647985320575549616942496564503156E+5196842330', '6.7960564653247694002509291743321458918999471599831534408e-3963026241017650', 56, 2);
    T('9.8833616922349478348248296712660944803419689709681112380751E+7374', '-8E+6391392070188', '-7.9066e+6391392077563', 5, 2);
    T('7.52956132318892586232633595231709208398282656737788E+503754023414', '6.1583828400E-29519511', '4.63699212454543751085827098088246381286996779872360399875792e+503724503904', 68, 0);
    T('-9.19058793298907508574231169627415899608472380574715482E-205846269', '2.621455888521435836739483138072144923900569E+4635756184256', '-2.40927208559082622321592429985447249736030906274110597654126772016e+4635550337988', 66, 4);
    T('2.19076723354991109531240399237475940492984037681320E-7273634', '6.8607620317959163209859302171497984557203386356E+19777', '1.5030332656441806782808532783824307759e-7253856', 38, 3);
    T('-3.05482247E+737', '3.005372282E-42960066706', '-9.18087877776877654e-42960065969', 22, 2);
    T('-5.938E+2973', '-2.52965522203503060456785480182861726079411E-7171499', '1.502109270844401172992392181325832929459542518e-7168525', 62, 6);
    T('6.34078905652504635308416503E+29', '4.76550151779379964758318648199944296E+3197553598424', '3.021703987288042326258681338385e+3197553598454', 31, 3);
    T('7.83E+90', '-93875.4685178161410912443181786442857642775875', '-7.3504491849450038474444301133879e+95', 32, 0);
    T('-6.6399085584722520552676278156142376272525942385352000E-87587', '-9.21688397727424161162054867E+950', '6.119926680314900679139e-86636', 22, 4);
    T('-2.643488947921428573003976196820E+2960474130635483', '-7.127301624634365651906125405769609107529140175085738297E+139563823718096', '2e+3100037954353580', 1, 0);
    T('4.543973929287075416645380662045856545500567263498740E+7555', '-4.29040131203268E-1709', '-1.94954717080555606602274e+5847', 24, 0);
    T('3.5413488501972E+98064121918720', '4326675.6598423248268413531935467907', '1.532e+98064121918727', 4, 4);
    T('-1.666640E-283814120287601', '-3.7530865386153600746742828E+956386383605', '6.255044148717903714855146685792e-282857733903996', 75, 4);
    T('6.001674219922679143977800364832207605654453201725126550E+4522708', '-4.660686296E-601839076587', '-2.797192078985012086594234508859727032710068214865362088e-601834553878', 55, 3);
    T('7.1E+98861496038', '-6.0680855E+156305823442', '-4.308340705e+255167319481', 61, 2);
    T('-8.82174854639121580358985202433544886480024E+205210247973706', '-3.541833658267239E+639624894', '3.1245165926378497828485097247166277e+205210887598601', 35, 2);
    T('5.41886063450636119403489490460E+48995927', '-1.144595761917480911072920990026797E+430248156228386', '-6.2024049166774525421513221797693e+430248205224313', 32, 5);
    T('-1.685414986072638309848150678027481E+58', '-3.57250049687355441465281864669E-69356647667810', '6.021145875e-69356647667752', 10, 5);
    T('2.728625368330880920472162670480145237694E+787799353000402', '-7.554020E-9572', '-2.061209060487884109086512625606042672844522988e+787799352990831', 78, 3);
    T('9.1E+3136010461561', '2.6795582998103860747645037425305800778102834216E-70556', '2.43839805282745133e+3136010391006', 18, 0);
    T('-8.799793E+7737995593', '-3.6326528783740426753012071E-6394', '3.19665933705457521158168351301303e+7737989200', 60, 0);
    T('-9.9458851773278859599586913556152331357E+34609', '7.14513428679E+12357346890', '-7.1064685193001917126e+12357381500', 20, 6);
    T('-0.13181', '-2.86650745605327568087955299503568537759807549925240439909358E-3129607671710278', '3.7783434e-3129607671710279', 8, 3);
    T('-5.10873049147763092457771852828920337261871937345E+1524355084312', '4.215325634906E+8475711563', '-2.15349626025515859992423111552016461286689597387487637696457e+1532830795876', 72, 2);
    T('-2E-1339071193886', '1.4106742748624304675E+1449', '-2.821348549724861e-1339071192437', 16, 6);
    T('8.58E-7709859400694490', '6.79001563173791344719887002387496409927593925989672138063E-38', '5.82583341203112973769663048048471919717875588499138694458054e-7709859400694527', 77, 0);
    T('-6.9173802223660E+320921178', '5.8205458940461E+635906', '-4.02629290508481194934550726e+321557085', 40, 5);
    T('-9.87813089884666044008E+855445129326', '-8.322365792325878144376881744564745029645153196E+6720', '8.22094186846787260671895463943677359140610189362034e+855445136047', 52, 3);
    T('6.2181878862238419317718942788E+74', '-2.45831230369361646213005775924384610272774236567533763E+9824', '-1.528624e+9899', 7, 2);
    T('-8.473656728E+65811', '-9.71636883146066195127166654E+96199119129510', '8.233317412043613621072676533244348112e+96199119195322', 42, 5);
    T('4.553E+261582', '-4.47630810422531503976799E+226952581601', '-2.038063079853785937606365847e+226952843184', 58, 2);
    T('-8.44512920293018405207690068414718280361400806274E+23414744593121', '-1.962990789843437077676128880E+6349287713', '1.65777108443897981998184223840369531173745103736388865627146574347513659312e+23421093880835', 75, 5);
    T('1.260427805E+905698617', '-414479732.3976636714804589', '-5.224217793229746085723559117197145e+905698625', 77, 3);
    T('0.000085749954355239106005581197648634968', '-6.7142677716670956392628679179E+16512', '-5.7574815494e+16508', 11, 1);
    T('5.702736430608663818904800915485168529937769631E+289669697066', '2158386904.31535134', '1.23087116305878103107220090696115448623820885525610181937472e+289669697076', 60, 4);
    T('-5100.0857254551742908821104129999899374123885573998', '1.2454312981671706434600312536957331136865E+4702553124691', '-6.35180638581749398e+4702553124694', 18, 3);
    T('9856084517768.456359769059500882813902094700187206355728', '4655376109.059512689478687346213', '4.588378039289061982845964e+22', 26, 4);
    T('-2.02542999125038664146511773603473037E+70604435649705', '7.30116517633303408937482062603080216791470706E+36990', '-1.4787998919217844875929238656416870892877635068795267894882447239538896064635e+70604435686696', 77, 4);
    T('-2.6774710449143183833139504022464689756331348840213757013810E-63502', '-4.979146827564568841649176266280410055329E+169408', '1.33315e+105907', 6, 4);
    T('-2.093431789E+1216928899', '7.11858459225865E+8544089637', '-1.49022712781198612202249e+9761018537', 24, 0);
    T('-9.8300766427350393219407220419E+7139598', '4.817872186293641217242580E+76322', '-4.7360052846167920587755075413459330854358621345224102e+7215921', 73, 6);
    T('2963223688.12052329866803', '7.347603176799488234782367250737320021E-6828592221984', '2.17725917844018529357559869190657934906515769831224e-6828592221974', 52, 6);
    T('4.15445903390875778716156068E+9584823', '-7.35E+82', '-3.0535273899229369735637470998e+9584906', 39, 3);
    T('5.634771E-47120111', '6.199062806109978426E+10980659980973', '3.4930299327047129245450446e+10980612860863', 49, 6);
    T('-9.6631462405E-23991595629491', '4.28340804087024610164235291005931156290669029286647951188749E+63004', '-4.139119830666278896539508339841387116112e-23991595566486', 42, 5);
    T('2.8556912897E+9781', '-7.856242443362009789753E-19068752625', '-2.24350031152803369404837704144441e-19068742843', 80, 2);
    T('3.898730302E+997634498575568', '-71.7741047170944341883096227588173777847302261570544720473', '-2.798278769594572079655075004079901484535093656138212812354204872846e+997634498575570', 72, 1);
    T('9.36801144E+7971527485', '9.1743286653328267472310769810164618250558743121E+344', '8.59452158911578523755987174817e+7971527830', 30, 5);
    T('9.7464166100246191748239044990405834769108859436133763E-100969264130772', '-1.820E+8876', '-1.773847823024481e-100969264121895', 16, 0);
    T('-1.4219816787584493E+273496504124728', '1895970217055.02017284150350', '-2e+273496504124740', 1, 2);
    T('-7.78577303380327693933029241803146573545115838477727E+84471', '-3.3700529059355696286255323926826717E+6776', '2.6238467037523529433707577183915823323932413918698627483e+91248', 56, 0);
    T('1.576867314029E+218184', '5.2E-1879333461735', '8.1997100329508e-1879333243551', 45, 4);
    T('-2.66922928830054058683E+731216465813711', '-2.23511768105146589259892935493131043914048218E+8282906885239', '5.9660415770609589753554e+739499372698950', 23, 6);
    T('9.050E+28298', '-6.52893642710642795196000951333626940842079104484410320522E+4572089625', '-5.9086874665313172965238086095693238146208158955839134007241e+4572117924', 61, 0);
    T('9.83E-40515', '-5.9279793619284928586757708059331898043799086120998255962201E+5856302', '-5.827203712775708e+5815788', 16, 5);
    T('-1.179628460837491122088955667020705368499E+6271553621424', '-5.050996783E+10345229921', '5.95829956082540914346237e+6281898851345', 24, 3);
    T('2.3601995124259709E-942183613', '-5.4034714062574444994435915571394461795258E-18373229852', '-1.2753270578456495832001598329815878e-19315413464', 35, 0);
    T('-2.40460583116969979045917179151929491E+1786945822', '3.8760689742160464984588531133625430061942773926E-3590871', '-9.3204180574158621566910507503259154074174427899361839940015395660837389e+1783354951', 71, 6);
    T('-5.15001407020997091976739192902652436509552282888414E+8109179391446522', '-4582074.651771373956073480', '2.3597748927375028726085271427279388775218233635200819282687e+8109179391446529', 59, 4);
    T('-5.903932196037788141847E+826578694278', '3.8573763559E+4573979', '-2.27736884598329276412940131553473e+826583268258', 58, 3);
    T('-2.7889493043426869670156118891859495869303886E+41214863200', '-4.8850672444304E+33406102782505', '1.362420489302141083410276654708275950925460456026585765344e+33447317645706', 75, 3);
    T('6.292924636819631036685513319173175475E+98089383587808', '-4.487251507757426473179769593112745933308174705208276E-692892398', '-2.823793556477274477020341750125668e+98088690695411', 34, 2);
    T('-1.3266E+57', '3.992693416478491358154253128774652496349E+45', '-5.296707086e+102', 10, 1);
    T('9.7289449572299474593E+9196', '8.3284725628170943422124828703100844778476728E-1445714702', '8.102725114144734682e-1445705505', 19, 6);
    T('-46621003042136905153721382696830052739132898476799.94', '-1.583457773E+8038533', '7.382238965212832899582388330760324946977992937310872515e+8038582', 55, 1);
    T('-5.7884877194069808260709733748130844736869292424535802225E+213583277', '-4.9315204416593390E+8523202567', '2.85460455145495740462901502455382901502227024974767421006821792528229275e+8736785845', 79, 4);
    T('1.685053997468131562807E-757372228', '8.62452394334824669136022765896304190115210635E+841', '1.4532788546998576522284582039435456989408511536277541e-757371386', 54, 0);
    T('9.58457559764999865185600594962275139812870E-4964833158702692', '-4.8653323255725562E+93405918283', '-4.663214548214044065988465915020369164644790259428475558294e-4964739752784408', 75, 2);
    T('2.528420381023712193901036E-77389829', '7.449592268181701755820110039774475407834507091E+124947186647', '1.8835700921187278707025834684284317410553553511115981438775233494246276e+124869796819', 79, 1);
    T('-2.69664343614444008802302E+602777590383625', '2.36652083581252832877710145E+175916071811', '-6.3816628783929087221938929852000441e+602953506455436', 35, 6);
    T('3.304149342167889E-84', '-7.9362071336662330264288234194360782492612488349519E-5687', '-2.622241358e-5770', 11, 6);
    T('-3758774810.90050324031969592999477481062467057759642400', '4.28370033596194912749918220994293610765953825810E+97910', '-1.61014649e+97920', 9, 4);
    T('5.86980101951801766720414389174338519572370254760E+251693451003283', '3.805378789413378155702384302279564445324985E-5155684341822', '2.2336816297750886953894243324684482e+246537766661462', 35, 6);
    T('-6.49478133490942E+248687851', '-8.214689182196533656629434E+1436704716174670', '5.335260997261237454865278493619689586828e+1436704964862522', 63, 1);
    T('4.359143490534215493447860052058723296830915948954581190091E-25062', '9.7344384807572069170356218566184E+8563', '4.24338141373985566594143495456213686125371031131293496e-16498', 54, 1);
    T('-1.70368905841954E+327184648869', '-3.94864713242350531044455331191168189162857313519271465E+846', '6.72726691506961843734623601114550139762376887139443424366005120427e+327184649715', 66, 2);
    T('-8.694736624291544851772371031504721961261707794110E+41221305067', '-4.46693211357423652420428321185039E+620', '3.883879824611795291077e+41221305688', 22, 2);
    T('6.4029687806040910684104573525197506123820235E+274', '-1.7707E-63208922018076', '-1.1337736819815664054834396834106722409345e-63208922017801', 41, 5);
    T('5.299703464032E+6692', '-1.181117165E+55643209', '-6.25957073077815530928e+55649901', 59, 5);
    T('-6.693762372973611656681611387757861357E+7635264379', '-4.538982855253858028852411848318E+382', '3.0382872648070603998715504190556614483760510678e+7635264762', 47, 5);
    T('-9.38163151914093308995851988952202297722E+5458', '4.0762115521425292451268285521544792595782675905137723E+169', '-3.82415e+5628', 6, 6);
    T('7.5297105827681225737278695165020137213712065890584E+539274127758854', '6.93748196406073419211772380E+916786286148', '5.2237231362e+540190914045003', 11, 1);
    T('9.2273163814698136038451086427832967449341575095E-1405', '-9.99603011899316797E+307070023356887', '-9.23e+307070023355483', 3, 3);
    T('3.21E-473301', '-3.60280160539965554141868926878631635E+868175', '-1.15649931533328942879539925528040754835e+394875', 42, 0);
    T('2.45238814496217098946971189866300232571960415990610409752E+9610893110', '4.6376521480395217912426771340714025018168307681641792825E+20341504938594', '1.137332314831047044036870591917781906161037974995e+20351115831705', 49, 0);
    T('-5.576471543E+5272789619', '3.3140821E+6019495277326250', '-1.84808845218156803e+6019500550115870', 72, 1);
    T('-1.8499313313396440396656261446304033471457490981E-4152292', '4.258563513953103087487061401E-89918009619160', '-7.878050070961696781266824945504437523663945147986976611082e-89918013771452', 58, 2);
    T('4.454720951296828878844711038035234585559679E+362695750650975', '6.128929164002160553320250575207164741232121432E-541759690', '2.73026691558945827984108862727317546452488714465901e+362695208891286', 51, 5);
    T('7.733670835746250003668815570735859557654582797411241168777E-9123', '1.7947191374404402367744029415329013754744091E+8063365674344', '1.38797670515787983716021165467032260744803024263971249687924128283811345e+8063365665222', 72, 6);
    T('7.4160794002704829312976592443089764746411066184223066528451E+7198', '5.16354646972766958271312662414463078199E+38709167551', '3.829327060648674518738903464757575378312887675182193902939551199e+38709174750', 65, 0);
    T('4.141909709347263152232009037479688890520133436E-6160841', '6.8676733459947068E+55101', '2.8445282912400882252158191941413e-6105739', 32, 4);
    T('-2.7E-204615039224130', '-2.4762603345030989138127131652987187575383011449109288877451E+377117', '6.68590290315836706729432554630654064535341309125950799691177e-204615038847013', 78, 1);
    T('-4.5E+6378924924531', '-1.146244243180458068741029429752342892076650E+309215196346', '5.158099094312061309334632433885543014344925e+6688140120877', 48, 3);
    T('-2.203420898611778401537496996475194606180670987284363E-9261830680', '-8.152714893744630615731E+2226', '1.7963863e-9261828453', 8, 2);
    T('-2.9470930783811289569851580294206811272E+952285886443565', '-5.715679800924518400405E+74963330', '1.684464037954747725930470527881753717366442060454451633e+952285961406896', 55, 3);
    T('-6.034137144490192086104381389335009085217236109964719E-5264', '-6.855889311795146517136E+742875830516', '4.13693763548164e+742875825253', 15, 4);
    T('1.57685E-14348152', '7.0159528742347253462E-86056295754', '1.11e-86070643905', 3, 6);
    T('-2.0123308397316856143732499E+911', '-5.44434216181912698281285501E-9442', '1.0955817634280104407100146553086817619507025991196999e-8530', 54, 6);
    T('-4.1734915757322697339584518846039733970130031092970E+92815889951', '6.728018701500862306487E+803236', '-2.81e+92816693188', 3, 4);
    T('3.43207300965869439E-9652', '4.484E+41', '1.538941537530958564476e-9610', 77, 0);
    T('24690722.81139653814942234058775619265', '-9.28495936087169915631013190043421793289E+7749196470', '-2.29252357894364683805457730108160923171899729585050542840911577e+7749196478', 63, 6);
    T('-8699506.8581669623299142693082391972131065', '-4.26806530133794318732221939512962971206962949746592796008E+66', '3.713006336009387946066165720491e+73', 32, 2);
    T('-6.588536378E-257', '-9.42137278995445138', '6.207305735731425588016230164e-256', 64, 2);
    T('7.5490945E-9', '-4.7957808E+3342404752969141', '-3.6203802e+3342404752969133', 8, 1);
    T('-8.30846568868829376058817E+65741', '0.0009023272787597071057938596275005790235199831096205575446033', '-7.4969552355425039214e+65738', 20, 3);
    T('-5.966329261772150789005E+97427780', '-7.483687230751844842368981852770454450502123997066985E-354438126191', '4.46501421107853259126496694813569054952e-354340698410', 39, 1);
    T('-5.571615636504091768719761828394955967102096E+431958993342433', '-3.9027703047361E+496580118937', '2.17447360555514939994809845127671948136751882923410368656e+432455573461371', 61, 6);
    T('8.6090475086161565520430266959635523824314551176870348791113E-306503624305', '-2.8376061207041992271E+7731932719358121', '-2.4429085903882444e+7731626215733817', 17, 3);
    T('3.67547787979E+839', '2.51320844572156960875608350113058816639731852145289096298E-28144', '9.2372420495510359622617096059996542618e-27305', 38, 0);
    T('3.94719092999285139423903674311992735074194E+99', '9.960112431899621989151849248278415820885240525003617273250E+20561281097', '3.9314465452903229668898218353212083038787009723774827604e+20561281197', 57, 6);
    T('-7.061950136774544162E+1055506', '6.3336E+71177065818', '-4.47275673862752529e+71178121325', 18, 2);
    T('-9.0776037491335604401119011421521194137792E+136829333813', '-8.0369E+53686109', '7.295579357141151190113533828936236851660205e+136883019923', 43, 1);
    T('7.0261622971684464728101207854E-6401613790528', '8.7965920730285361E-419', '6.18062835670839258583745470404149e-6401613790946', 33, 5);
    T('-1.58E+643428', '4.2352080277144650256079390444788052E-7', '-7e+643421', 1, 3);
    T('1.33349574372914387175802388808340E+4855947', '-6.0680275235883922899251425E+58246', '-8.091688875536418285312693294e+4914193', 28, 0);
    T('-8.3759407147777193710071915184090337E+494437993740', '-7.478004831158284065527728120112456246645667957560124537598E+424088604414', '6.263532513060315649650830195146026977686885215210810493789163387042678943620464e+918526598155', 79, 4);
    T('-9.2479001056E+7975699901', '4.0891991457463367263620E+66619342792', '-3.78165052117669772025362981038272e+74595042694', 42, 5);
    T('-1.702342833681075048677748176482347E-35199551', '-4.32822423162468665215699588821631021068403E+1928', '7.3681215032710627974123e-35197623', 23, 0);
    T('7.049111246884546885259649243127504920826004782E+515534970896768', '-3.8000913667759270343476748519339278340743508', '-2.6787266792729057002079352671137e+515534970896769', 32, 5);
    T('1.84725024435401044705500298128557940794351237887765800526409E+8946045256', '2.434458844600595742701754471806000964472381583095348E+58183', '4.49705469555823243216524009688195132626252433902505176116676e+8946103439', 60, 6);
    T('9.879468000509174824096314E-51082718', '-1.3861330582630764489598041809975823729154267330575615220E+5374045', '-1.3694257193557983415167206408403026352621047844630037136984751280201228109e-45708672', 74, 3);
    T('-2.80E-9514', '-5.61E-3456', '1.5708e-12969', 56, 4);
    T('2.984160505037514754284E-7454254102', '4.658765048E+230899921562126', '1.3902502658490801666042607465632e+230892467308025', 58, 6);
    T('-9.3224316208882792E+889846837664355', '-2.65E-545837682549', '2.470444379535393988e+889300999981807', 74, 1);
    T('-2.074106828894509452967E-4913561', '-7.3080586249705E+750335204', '1.51576943000127328714471281615124735e+745421644', 53, 3);
    T('6.69026074697630780727456489851471095749885721027321E+28307', '-3.750414669804706528008321310553208E-22', '-2.5091252050278538693031415997190803736726216242759932179179481785383326364686e+28286', 78, 0);
    T('3.124822987E+8256629308396', '2E+84481541037196', '6.249645974e+92738170345592', 47, 2);
    T('7.95801884524878982329348256232E-9679320155', '-5.59274819971491182904637552667407277464084177814260031418999E-942', '-4.4507195570062510799396490456e-9679321096', 29, 2);
    T('2.908159592950760851900006563109216E+6629', '2.332476278774296162261256E+30', '6.7832132654475625211728670617185258e+6659', 35, 0);
    T('-3.146878822196095E-55703830', '9.64560500909666687712051644650919051204E+87266619950', '-3.03535501303948732606582404384915e+87210916121', 33, 3);
    T('-4.5673521044663270223899205163E+33226830170972', '-3.862011587819289500533701886398787801447829482E+395349', '1.7639166753099773036230850732971101212459677415799923025e+33226830566322', 56, 3);
    T('7.79934242847112206417130620222235812702968792628068101641E+342491428295961', '-8.50944274505423892E+755301680', '-6.636805784414729900100239691162337058424240862856e+342492183597642', 49, 1);
    T('-2.72338839706760631464578109971568503326E-96286007760487', '-6.695952474945791125E+6421595431', '1.82356792775834894236849584656896191778659042129926378175e-96279586165055', 74, 6);
    T('6.4142232960492890074447536E+7754659463549', '-8.5372318993065921371839070462820051350382E+80937', '-5.475971173230746121933795494880783443418942788588142218409412558752e+7754659544487', 78, 0);
    T('-3.38948E+639806', '-6.4610444010722614851355218623331174426932E-5324574', '2.1899580776546408858637148641940854909659747536e-4684767', 47, 6);
    T('-9.03580636701970946578083109705581952491337E+910495805795', '1.9045980304397E-76134', '-1.720957901006023967927708957131908410056e+910495729662', 40, 1);
    T('-7.195647E+25', '-3.124E-3453969773425', '2.2479201228e-3453969773399', 54, 0);
    T('-47748297848106259352557378087258899175884068209250839711', '-4.94237873118718804837145059066631431908416237E+80879672011', '2.3599017173487135553929516054e+80879672067', 29, 2);
    T('-1.020213E+53', '5.399067299689983828594844891509165E+420771010857944', '-5.508198647018617471722232491301239752145e+420771010857997', 47, 6);
    T('2.8772945489426184233778952282860871228914E+544016332372', '3.65543309049380318771120061878438848460E+2163', '1.051775770530228911631111391325192500495736813683518e+544016334536', 52, 0);
    T('2807733102.0112644355', '4.61300549502388475423871929951E+16489430121136', '1.29520882281384204079690291080941e+16489430121146', 33, 2);
    T('-7.747934866775307536E+842066322249681', '-2.574994418222710803867628658E+26405715907', '1.9950889034699539e+842092727965589', 17, 1);
    T('9.6877547086400336951335E+898', '7.20245981548641955226866300779208714304766619306535098737027E+465342765526', '6.97756639912691892973103118483294279330211474677483259678398451e+465342766425', 63, 1);
    T('-2.557945E-41497898201', '5.3E+14', '-1.35571085e-41497898186', 75, 1);
    T('5.45481431107593501808447196422206093002889214257301932224E-685389', '6.1612877964E+418764530011889', '3.3608680846460231780643270417499321427e+418764529326501', 38, 1);
    T('-6.26223795209056409348394111957315317428439453E+13079462', '-4.01500947201668957885629098559033210081214290214648673243012E-7547', '2.514294469366601115127939071547551e+13071916', 34, 6);
    T('-3.95918054005631836819966957120139520553979162742426E+3748180212', '-6.5640694744E-83989', '2.6e+3748096224', 2, 0);
    T('-2.09174276924262843384703435957329678800147832604927899E-12120813787', '-6.827E+74', '1.4280327885619424317873703572806897171686092531938e-12120813712', 50, 6);
    T('-1.49424594989639955763814273E+77885', '6.9751939514451908841738620296E+453201086255', '-1.0422655311688839946724239478223327880910382997412e+453201164141', 50, 1);
    T('2.7E-30862356', '1.59484167554190883542256774732912E-58544647', '4.306072523963153856e-89407003', 19, 5);
    T('3.504634258233850041145E+5055058509', '-4.7412398634936142069611730926493167555723922E+52320971103', '-1.6616311652104e+57376029613', 14, 5);
    T('-5.0026313E-4699276989843583', '2.789517527673597236736647369500638E+348', '-1.3954927695838553720092262e-4699276989843234', 26, 0);
    T('-825640757274597892689578.42127384875888273', '3E-447682471059033', '-2.47692227e-447682471059009', 9, 2);
    T('3.2454543897889061507162976451991396E+712456077', '-2.917589838165352720450286E-3369456484718', '-9.4689047478772482629144849638224963422480962759858518e-3368744028641', 53, 5);
    T('4E+37314375', '-2.815824186263997142091532316490897E-4990533877727', '-1.1263296745055988568366129265963588e-4990496563351', 56, 6);
    T('3.95060171834756527295991335255828572938156E+38147736271', '-3.096E+511560837138', '-1.223106292000406208508389173952045261816530976e+549708573410', 65, 4);
    T('2.16019824170417946652142755183421643009203634E-76659299709', '-9.9065467112886617064700107204058054215242674250062261647396E-490496836', '-2.140010478708608863978542e-77149796544', 25, 1);
    T('-5.5489270936616855937114884781237818346E+8374306952176', '1.149125248302328361620648637054785972358139E+901515059711880', '-6.376412224315501722950906369714739463074416077971704004217e+909889366664056', 58, 0);
    T('-3.598776179491013284481456291001561717153683E+28138315', '6.068820892714973118370176E-1336247549348419', '-2.18403280663e-1336247521210103', 14, 1);
    T('8.34239404197567703362497169E+333885681372580', '9.2774453380292331037008756173357764060026390685120834884648E+5246884', '7.7396105e+333885686619465', 8, 4);
    T('-6.8744505719071690919643407376155299931633359876306718E-2377569', '7.617639679323232635266323854816E+782839645', '-5.236708745010634075388e+780462077', 22, 5);
    T('-3.1728304146614388008224235709391459387274481E-4646', '-4.667351935096535592115483377966207882139733256035E+64052549', '1.480871617560320981966e+64047904', 22, 4);
    T('-2.9637767483847929222797110809528231761676574936203638E+5578148444014', '3.7875098033194888419077288860233465668289341480533E-2982', '-1.12253334893577612102739347061139167219329268e+5578148441033', 45, 4);
    T('5.9953836258722780123672848569314732E+20056495312', '4.82457E+680582007', '2.8925147979874616330126831502205877656524e+20737077320', 46, 1);
    T('-5.656882279548744E-69379127830326', '2.486540158785369203713355794202020435785E-89200988', '-1.4066064961619275205850942026165862061121617456029404e-69379217031313', 53, 2);
    T('-1.0670539954349763923443936198310245652E-288940896', '-9.47260E+6', '1.010777567715735737412150300321136329631352e-288940889', 48, 0);
    T('-2.5E-7524199', '-2.3232373929242181E+35', '5.80809348231054525e-7524164', 43, 0);
    T('-5.19363573017668870478808030941059134171E+17663', '1.87822921298355513872104227351204E-18366510607212', '-9.754838350013033757823911e-18366510589549', 25, 1);
    T('4.54052473208107143471229034056871324959855E+979603114264470', '-2.88278232162502866393007492846957798226E+31134', '-1.308934442854453237756078981706092250978978454353217906965921201e+979603114295605', 64, 6);
    T('-9.4216152185163685587252888357016299231645858424E+51003', '-3.67425955008E-77704752488303', '3.4617459693812833225697437815783118e-77704752437299', 35, 4);
    T('-3.7304792571401370202257966180327090928166020E+77771072', '-2.208433133513516995E-2183', '8.238513995353169917662976e+77768889', 25, 4);
    T('-2.272956432061024313771969174374421976870E+44233242', '7.5561985204859751460740183638768897663438677E-41', '-1.717491002906859280349850498095312176813306e+44233202', 43, 5);
    T('-3.640800387852506669E+77', '-6.3234743888185895875562472025064681E+98194726', '2.30225080073861135308875018537188765812325122708857589e+98194804', 54, 3);
    T('8.779338497245902757307986535E+55', '1.3466335832009312851482053557507343882082821459697479448154E-2283430501971637', '1.1822552058680129429145966529394188636286e-2283430501971581', 41, 6);
    T('7.18259547478044236933E+369726184947432', '1.817993421036473367257259229465708204E-321791', '1.3057911319e+369726184625642', 11, 3);
    T('71822395.464365945009540311765519866133', '-7.823503650529E+2326344000629562', '-5.619028e+2326344000629570', 7, 0);
    T('3.2890011845846102E+2370898575755', '-6.6515609343254622155504454573092245580788336452794015379912E+350340376', '-2.1876991792333161836420873651742850054529798e+2371248916132', 44, 6);
    T('3.3983954026605193615074930073068470E-3914643', '2.91327300884239570559474138614148921E+1525980547136', '9.90045359994497613625530704357422022669e+1525976632493', 39, 5);
    T('7144428.63510998352689870585025117149395', '8.251650539891148958010949443404484120E-59272', '5.895332840411908002777667869e-59265', 28, 2);
    T('1.39789798724132726923884663442529824668071E+3313382742193729', '-1.625430E-66300354237', '-2.2721853254016705832388984849939125291022264553e+3313316441839492', 51, 1);
    T('5.39171622069923380289E+201816622324', '-4.0263709911551357686818452947533693169818E+245579461190', '-2e+447396083515', 1, 2);
    T('6779885.678925625374230335725675571851668781262389287', '-7.90030787249987E+3871460', '-5.35631842038652441008794196305740707370339370453938553041978e+3871467', 60, 4);
    T('-9.50531543572142548E+10411', '-2.050121870157453989737286884189482083546408006377166950942E-5020777', '1.94870551e-5010365', 9, 6);
    T('9.427259843601660661853323470871503465468455324985532715E-1466490200', '1.285868834621583331123196609854254444125272833022786677E-74349109999', '1.21222196287669173327e-75815600198', 21, 6);
    T('-9.3051836907521708878752E-8', '-9.7025145221E-655152574', '9.028367989033101351170997881244192e-655152581', 71, 0);
    T('-1.163153336740255826007303895580100205513743427879197E+64574', '-6.247381691138035916291429921344322950650300897E+445113652154961', '7.2666628599371888066784873419749482056469116711521e+445113652219535', 50, 6);
    T('-3.14025586651187671701899E+7715112', '7.79251069930901369581991259016300042287486859268020E+674199845178108', '-2.4470477438361697199070472068170827624056509973561157527e+674199852893221', 56, 2);
    T('2.7389805572677196986893452231472832E-9', '2.79379072E-293812', '7.652138463155e-293821', 14, 2);
    T('3.6546060877346511845413629976E+32429873263', '-4.03E+395958082', '-1.4728062533570644273701692880328e+32825831346', 52, 0);
    T('5.198687172903940504070343155065587147030243235412966E+7435262896', '-8.25611334183927970530E+6357', '-4.29209505282609495455471342565e+7435269254', 30, 3);
    T('-3.3787984661144492414070579796173278366635E-55533', '3.5E-952', '-1.18e-56484', 3, 5);
    T('-4.9629385136949E+320895315', '-2.8636429566107307028540103025219505081319719E+361795392', '1.421208391883452884481452e+682690708', 25, 6);
    T('-5.339260210789917740911501083727618016699E-6427404', '3.20620767298447803042693775918284841302305694E+89651093933975', '-1.71187770558953558174147378e+89651087506572', 27, 3);
    T('4.61E+46', '4.63356941466815176301E-56370925', '2.13607550016201796274761e-56370878', 45, 1);
    T('-8456.0396845190978386302042496452945900684590326177336845', '8.662168361033832557450540896542322038538383E-343200513144', '-7.324763941488784024e-343200513140', 19, 2);
    T('4.881064197E+671534', '7.6961998454190500356426794985796190958091077136473167242E+4473355366400', '3.7565645518431859590727056785662891122451348407600445947611943e+4473356037935', 62, 6);
    T('-9.4658798933E+1124885730895', '-9.8597104420682678491E+7724161389380', '9.333083482733407109875452850103e+8849047120276', 54, 3);
    T('-7.403646635097002855775921423210224E+2903251912851', '-5.0303635021038487567863976396227E+227038158007', '3.72430338156659348940255849420782116788319250874350502244941424848e+3130290070859', 73, 3);
    T('2.2430566796190651139631047280781843551392317187E+80155647423', '-6.4838087658700585721282090404919310997347689667515927361E+9369219', '-1.4543550561657481938891707764219006730666903091778009487e+80165016643', 56, 0);
    T('-6.91476E-521267504821641', '-1.052523620807956701582764573E+2821302027817', '7.3e-518446202793824', 2, 6);
    T('7.629668754420E+31395', '8.5578468367760513391679283254616756872003E+8192345686', '6.5293536615662272669395340666137019511807510438050326e+8192377082', 70, 6);
    T('1.7225457804774731020837712131030675685786361103446E+6418', '-2.729893704E+77011838', '-4.70236688097722e+77018256', 15, 0);
    T('-7.6710158334356916087758206E+438077648133', '-1.3241604260208112240117927012402809830654037E-498663258020', '1.01576555940145936731888548097939017077291216937e-60585609886', 48, 2);
    T('-5.55126103671208856439974022851538018244700669662709832896E+3838235', '3.335939707641161229785934755813826672091841605220036445044E+68472802206', '-1.85186721198490943220427858592442191298418532074568668146982233433333415985e+68476640442', 75, 0);
    T('4.08631398414281551112147696366456131496163715623997668E+9714289676', '7.84295829706283507566618E+121256496613', '3.20487901663367851942798668003760808663651428799575809751288e+130970786290', 60, 0);
    T('2.081776018062419788942958034558813417266804E+626820395032523', '3.474762906742443924411295965603442416097557169297E-605939725', '7.23367808770928423181492343095676484e+626819789092798', 36, 1);
    T('-5.88934E+9727849', '9.136630555627416177520771790844228617940127076719363905E-61093044406995', '-5.38087237964787671909202e-61093034679145', 24, 6);
    T('-7.24053892424173615182343533177931424467959844757E+48725', '0.0000281529598192030311781', '-2.038426014035531e+48721', 16, 6);
    T('6.01357211E+2747062', '2.8701683609353242098193615897605700669402146E+76361', '1.7259964406325079181977500994189425732252507555974806e+2823424', 76, 5);
    T('-8.8E+66158', '4.763815840201569776995E-67842870081', '-4e-67842803922', 1, 2);
    T('-4.893625317360E+796718673421', '2.6097830313412039364799670922968760587185806085E+240402788', '-1.277130031498784194010126024332365651418330002519955441356e+796959076210', 73, 0);
    T('9.4230968871682645366944869431930425656184991102E-549635001212589', '8.953268392683614753938752342161560775E+8342042', '8.43675155210789813217438304130825496030838177690118282751997964303344792e-549634992870546', 72, 4);
    T('-7.49473047645568398252648556553097004154349548459452E-1959909979647', '1.86340901825305527316303734986162E-738016398', '-1.3965749e-1960647996044', 8, 0);
    T('4.09067427467253700817835738411840558939039785569491817516E+828397855526', '4.199695E-631899187514176', '1.717958429797088031056160661429514736173490692257266e-631070789658649', 52, 1);
    T('5.940648889969802057E+89706146', '3.9E-19', '2.31685306708822280223e+89706128', 75, 2);
    T('3.817892902762236817375635808223669202168677578441E+3462375557', '-3.76E-141285188297215', '-1.44e-141281725921657', 3, 0);
    T('-1.0612166953651454099047322434060944445963245830E+8631634231', '-9.0755863371127908155812629035981385261444E+3758768990', '9.63116e+12390403221', 6, 4);
    T('7.7096001129493278E+48160', '7.2113E+407692330', '5.559623929451148756414e+407740491', 47, 6);
    T('-7.953826090926246664517434E+155', '-8.068910418321687684E+686', '6.4178710210594e+842', 14, 0);
    T('-63792.094', '5.91465923E+415562075889168', '-3.7730849757812762e+415562075889173', 34, 2);
    T('-7.217089548929230068452393414E+7657274520052', '7.67031709317381615759955931320828160365559262784501031023E+719600383', '-5.5357365330118e+7657994120436', 15, 5);
    T('7.2561079571427619656777392848E+7418', '-9.828372573491090254408208717E+57', '-7e+7476', 1, 6);
    T('-7.58134670408847052634937E+914', '8.68546754685E+405', '-6.584754075977862096786019093994299e+1320', 34, 0);
    T('2.1774552370848714688318762230307400900457130373276E+16567', '-6.2634930605099455827718161164079534931E+652013417', '-1.363847576705213075585525083903972383084116699148886434e+652029985', 55, 0);
    T('-6.7E+4745822', '3.90036408E-8', '-2.6132439336e+4745815', 34, 3);
    T('2.67851458E+76', '-6.29919236367329420907744326150219502294810087E-965690316', '-1.687248e-965690239', 7, 5);
    T('2342524.76707776', '7.41228090235784974318339177347130504189956662E-11323332308809', '1.736345159431075068323903104845e-11323332308802', 31, 6);
    T('2.794831383598650027526566069278E+3171619366756', '6.6E+36', '1.84458871317510901816753360572348e+3171619366793', 50, 3);
    T('1.12776280299772216757906525800475871509764207766678654248E+207477546042', '363.52697519047093', '4.09972200506088901376843595710464879527189e+207477546044', 43, 0);
    T('-6.1897095155041023492530932057331E-12430495891383', '-6.48087831767327189213259479694131658461091203220E+99833047551985', '4.011475419172e+87402551660603', 13, 3);
    T('8.8244448573440987101694016855933331526935534134744166E-2877232', '8.9289560313107698632071392205441487611073803695213492670E+535642', '7.8793080132e-2341589', 11, 2);
    T('-8.773710864882926739912256372897071E-89865742964', '-8.176327445483462350123077992784302780349791E-71567396', '7.17367329432787194895056416e-89937310359', 27, 6);
    T('-3.68088634784003690086551695286645810081E+2174763549837', '-5.13191051936099699968E+33912827061890', '1.88899793690525672286332328245811407449765687704908409777408e+36087590611728', 75, 1);
    T('-7.446610278555414886592786790815492195887324221E+375029', '-3.0372001E-49238829758', '2.26168454826895339491011007003434919788982007127536221e-49238454728', 61, 5);
    T('-4.294133677761297181397879717789758633025361340305E+5514116', '-1.135863343301070250772291044992028356207964528548263435E+91', '4.8775490358036676756924072374220609648790160106131009457285812e+5514207', 62, 5);
    T('1.9014892716305328377744751298483746812638628423E+78133509098', '-6.7371270568771450547753772E+88608422550', '-1.281057482026367792839e+166741931649', 22, 3);
    T('-5.99372996903663922285720708709973352149221E+6276135055697', '2.870026412718800E-78936484', '-1.8e+6276056119214', 2, 3);
    T('-2.5305577894037318198745643303418000374705667105206240E+3411061063260684', '2.2770630071638704596898492621607765396858037827331396193E+142437359031361', '-5.76224e+3553498422292045', 7, 4);
    T('5.63912900125206422975E+94', '-3.13655E+900372910', '-1.76874100688771620598223625e+900373005', 43, 3);
    T('-9.57387156125375924027084755936466099311680993183649523998264E+124601030214215', '-5.6721E+556028358652', '5.430395688258744778674027444147229361905785761436978e+125157058572868', 52, 4);
    T('6.581263611958216055115018351574124980E+167234945449659', '-9.04630578450829161639700630777626546E+86570346438008', '-5.953612e+253805291887668', 7, 5);
    T('-4.2455790137E+34546', '7.3563254378E-353', '-3.123186089667114469786e+34194', 43, 3);
    T('6.787153444915171205327508258574E+647', '-5.03979005188645632921465180E-58446796795352', '-3.42058284123103715081673354050899512074184100520637745332e-58446796794704', 80, 2);
    T('-7.86641646794447673595412745422E+59779313073905', '4.422694884463926575552372955715357647216116405180946E+42366721', '-3.4790759871840826910003612448027570484528363476020868181678141e+59779355440627', 63, 4);
    T('8.157086879349880506022168867701708507605270E+8656796741585', '9.54382758372712766338984695520323317261546029954126225331E+145922873859909', '7.7849830761998026203778e+154579670601495', 24, 5);
    T('-5.20360856839124752653E-103965964838', '3.703205768914920848775777E+52514424786', '-1.927003326964158028905105505215158692982886381e-51451540051', 80, 2);
    T('-9.8E-91195934077234', '5.6488585846061056213330088843804656025987464095545699330E-1586', '-5.535881412913983508906348706692856e-91195934078819', 34, 4);
    T('-7.75253864680099266785965E-397', '-9.48120260361426742058360E-492827915921744', '7.350338960266980122303995e-492827915922140', 25, 0);
    T('-7.71921961931506123524748954677642941932439716986264719054469E+68318101135931', '-2.0310296852437785799528312E+27', '1.5677964193745069133521079465392254898840023053100162e+68318101135959', 53, 2);
    T('-6.893490540958013208966315942985931511179E-91', '5.546161E-33265039361241', '-3.8232408392130235497053831796666796896e-33265039361331', 39, 5);
    T('4.7113969833617677E+1702810236', '-0.0022462280884347823994072117445', '-1.05828722397941037580570862301135e+1702810234', 33, 5);
    T('9.826546938363E-706', '-4.0017452840059998876783291841436319022285809302066E+7129', '-3.93233378686577321079880075326298777148875156963e+6424', 48, 4);
    T('-5.7747735051077607681440905247351428703262641580165E+5783', '-480416422107120.7396009026958155484175859187338', '2.77429602580286716e+5798', 18, 1);
    T('1.7925812968018488642380933788240079995266815880E+976727201789', '-2.995274396823732356265348681251E-4958', '-5.36927287e+976727196831', 9, 3);
    T('6.0591905867782847575700220784383466106851369411398423878657E+4530866151841', '1.0271307444761879457953E+1005175098273', '6.22358093832068970478692627251966718719452669681673798141408992667097727006e+5536041250114', 75, 0);
    T('4.8764505057863252885851356397459166179898E+767357', '-1.68040002730445545812740253479790134798096E+544868', '-8.194387563072166643944024191954556137e+1312225', 37, 2);
    T('-4.0435397829895213058183112420128936274078411776992133E-7801725', '1.0650124820374697756113036E-405711839797', '-4.306420340498e-405719641522', 13, 1);
    T('-7.399477520177331165742700605045719E+36', '8.83914172115065E+3780659', '-6.54050304633157985139645858093906e+3780696', 33, 1);
    T('4.850948918264990E+992', '-9.3079963E+74643', '-4.51526146e+75636', 9, 6);
    T('8.1925185696622222219E-382', '2.0668513878120373835791067724996831E+7068572', '1.693271837538225146506799388985693100466756185952787989e+7068191', 61, 4);
    T('9.23550646785621215295159783961745637636291E+145905', '5.488E+266842', '5.0684459495594892295398368943820601e+412748', 35, 2);
    T('7.868511622700028378991263250670232591686E+529106792', '-8.501492308014964956739E+819886', '-6.689409103591064139140995793984173477379467429477764884e+529926679', 55, 6);
    T('-9.21078553186097491605796545322000725927346279E+85245905443', '-5.002231098096019608210135242112199705432459E-414431850689', '4.607447782536785455577996733042565539403731873253363042454255996e-329185945245', 64, 5);
    T('8.566475E-25661533134286', '-2.5769092271050039391334714948743167928097710485109299E+11498337643', '-2.20750284712643386194884052240535e-25650034796642', 33, 3);
    T('0.0644005041953007012315236486848749695679949290', '-3.2569255397469058886410397703836289E+76343694601', '-2.09747646886252613440480203386532408e+76343694600', 36, 3);
    T('-6.6865447397573689117882053931030434450636305170331E+2326', '8.91404788312964095485197852607895310038331413916094892E+7547772', '-5.9604179982885810326498285594952992216595429902486350114243555e+7550099', 62, 6);
    T('-6.4941964861864825E-24325', '-4.4487584304655997884320127556254609939915648591418E+92944439', '2.889111136702219e+92920115', 17, 0);
    T('-4.63850156238658713826196365629706E+704753155891', '-8.51490423955927548975172867926573914742E+73779550', '3.9496396618767874013347590611525282258472112062622e+704826935442', 50, 2);
    T('8.7548530695430950523159062258068E-615958627223', '-3.015666185251509993183789540464611314180468160071084696566E+1712227674351', '-2.6401714358666498185355747115089366551638e+1096269047129', 41, 2);
    T('1.5517404857543211115871872204E-17894410', '-3.418941080592851409046916897739999221474234260E+853385', '-5.305309293164554e-17041025', 16, 2);
    T('-3.72831131908014913E+7316010026', '-5.003381565236288354199172043323747190E+4356', '2e+7316014383', 1, 2);
    T('8.58E+2985082808', '-9.604023303676290362250409516939535230062893352678E+1857577015', '-8.24025199455425713081085136553e+4842659824', 30, 1);
    T('4.2373026106537499021945291905572870367885E+89340373775', '8.462280791867476584846774161979857702559421895668996147E+615660', '3.58572444914651405471111284536791452982603827646306e+89340989436', 51, 0);
    T('-2.0076633102775003819607207660512E+48914', '1.31195582E+8105509264522387', '-2.6339655645190324e+8105509264571301', 17, 5);
    T('-8.5302273535E-743891166687118', '-8.2003384489178265836E+5893', '6.99507513449166067971941745026e-743891166681224', 40, 5);
    T('4.88604098149705804359333364690541415E+17403584090', '-0.0000084667792900464011556633238667507610342996262', '-4.1369030592458e+17403584085', 14, 0);
    T('1.5679758372325807703725878154561103468901E+44860416418334', '1.8235995710041265367513643416931983199E-83557', '2.85936006412217042999189216808053974522478116390165002159734833058e+44860416334777', 66, 6);
    T('5.3459953674189016243241678509723705561199', '4.06048803789091214622E+8337', '2.1e+8338', 2, 3);
    T('7.0E+841136730', '19461.53', '1e+841136735', 1, 4);
    T('-8.932243012572270093E-91333254900058', '-4.25316271674484271726024965959830748E+3669', '3.79902829579770145717426606063992564e-91333254896388', 36, 1);
    T('-8.5328071549030405088926688848262579233164099460024384452E+6952696186800', '5.68707976575698820E+19', '-4.85267549158e+6952696186820', 12, 6);
    T('-8.31554E-20161244', '8.16624135251619430006622624947193035587255085323794432778135E+562243642925643', '-6.79067066165025143499727070265338157514724315221e+562243622764400', 48, 2);
    T('-1.4234336719120195675144787847398999345167112943E+777815848786', '-2.4465817869657352248908172224499877048E+50267474567', '3.48254689665370690561763704130848457017682895877e+828083323353', 48, 5);
    T('6.87468746149531743316724184484E-12864101', '6.808166253989372E+732266', '4.680401518207628034916634403261669058303304048e-12131834', 70, 3);
    T('-4.32419769E-16284167554280', '3409308799980213926524776984.76432607300647416373661623', '-1.47425252373711131067842703652830639993013669338745576701825087e-16284167554252', 75, 4);
    T('5.8688099178757813508200770515E+4039419232', '6.5E+543352', '3.814726446619257878033050083475e+4039962585', 50, 0);
    T('-3.243493028334236157501348998893278514850E+37636199283262', '8.37359090543170451786609909585914619606315630454259E+40661', '-2.715968372389069778257452861656739382477444584903260647262692e+37636199323924', 61, 5);
    T('-5.835195394828E+8941', '-2.518705524355296049309213870365653024499701957E+39591', '1.4697138876645866500601706786946600709025590718144939278396e+48533', 71, 1);
    T('-7.6969003735455467597860E+348561', '6.130424144413183414597E-5350892901', '-4.718526388712647031802753649089570881e-5350544339', 37, 0);
    T('1.010610262152603188378083532650278873969372615E-7869040699', '-2.29689910668397461534679E+3795291403077661', '-2.3212698083439716642797e+3795283534036962', 23, 4);
    T('3.45888391971010218555507103436022603E+9292', '-4.2050402160797538679961029152515159203918104782E+36691020', '-1.4544745985132554123274970506424459858024e+36700313', 41, 5);
    T('6.194826670487695614110120309733E+660299527', '1.6715838217097491327789630830500583109021E+9755', '1.035517204068e+660309283', 13, 5);
    T('-9.12276998E+7831193505', '-5.4007982E+1494834097730181', '4.9270239686998036e+1494841928923687', 25, 2);
    T('-4.945631E-416515365415677', '2.2989696713456142068517928678029715793710026220156103920283E+8054', '-1.1369855674666681335e-416515365407622', 20, 4);
    T('-7.8387198667317490239905780614405E+74553992730251', '-1.047720075453354241E+66789886398069', '8.2127841702298949878903784751995345762331092441605e+141343879128320', 50, 4);
    T('-2.126972697479843E+15450349924', '5.491996963275186607774521187191463228E+948418', '-1.1681327595528529911165309864838617967460999205713204e+15451298343', 69, 4);
    T('-8.0742132446064493540299192118E-8877208035', '-7.288602992331434625200015E-4676095', '5.8849734815360668464790072000998119186e-8881884129', 38, 4);
    T('-3.73584471602299685905498690153407E-312971', '-3.48836297555264488959071639433326375084019E+98621', '1.30319823897886069822833001374792927184256165e-214349', 45, 6);
    T('8.07516665386448308452275116146916274848858010515574E-8039', '8.1649373615323974003E+8240430934767', '6.5933229912738670701279033e+8240430926729', 26, 6);
    T('619502174.55780329630663455174186', '4.16E-2684272911', '2.5771290462e-2684272902', 11, 6);
    T('-6.167163760366904915419488293E+437294841272800', '-1.1373E+60622643', '7.0139153446652809603065840356289e+437294901895443', 57, 2);
    T('3.80188593040124376966E+369210', '-9.38774462677764946062669788823006429719E-2127557056949196', '-3.5691134214745820765482405531838516061194e-2127557056579985', 41, 0);
    T('4.167921748104695767876380521321674463007E-2849320', '1.602133882820091926760912312248E+9140116518959', '6.67756865358128135000864619727513817811864296e+9140113669639', 45, 1);
    T('8.4375758518326049973057079914921814409320802586225157681E+9879905020', '-8.035295542402788029432093383460477421523E+4323579661793', '-6.7798415630915938015640949138013e+4333459566814', 32, 1);
    T('5.38174621654612960703418918599231967945974529373202031918E-5033497581223', '9.48524346164E+98423', '5.1047173112699983438569065528372444114852249655e-5033497482799', 47, 6);
    T('9.164374127247905671437379933365607875052597397322555911993E+85', '-7.89477113819003176E+89', '-7.2350636359372227210509721543332391292e+175', 38, 6);
    T('-8.1549147737007555060823704110715603482341036E+982', '4.345057812542936294245542274549580670327516639E+4116136071', '-3.5433576148090279069126530251869053662122197726862577798447417650610724207076e+4116137054', 77, 1);
    T('7.7499209822109513723828000815392829744744921751E+9835175885352', '-1.47132283147400447394386797827972857088311E+18989716939', '-1.14026356832464148307e+9854165602292', 21, 6);
    T('-6.7935997362263920664047373183213782908E+5079274905', '-7.651012096234E-103999', '5.19779137588402374327780561977181762507898080368472e+5079170907', 74, 3);
    T('2.232369704E+9984819083', '1.64664283134661017570345540878213508834129012463540', '3.6759155700069540793385107426801739076484596865104510059216e+9984819083', 59, 4);
    T('5.568004700309338E+6582647613493', '-6.836214721538113721E+661647101461', '-3.80640757018481094171936457222e+7244294714955', 30, 4);
    T('-8.1142270698642939642308334327783112407936240E+1938074', '1.021233147887040916665905287481047154636235E-2419', '-8.28651765322775320611228548435807503277027895183256664631404954e+1935655', 63, 6);
    T('-3.86800214659551292973E+69000', '-4.56881567008572444481802280910548938588215318883493402E+9865999831107', '1.76721888192907989616098536894800397e+9865999900108', 36, 3);
    T('-9.80908875349207566914706E-84129', '-8E+1868237782888245', '7.847271002793660535317648e+1868237782804117', 67, 4);
    T('6.002989967466896E-455149221', '-8.5046267802E+567344093', '-5.10531892385908904785682592e+112194873', 37, 1);
    T('-5.7344658517341075760991946921E+2905282', '-6.631251347868547674466570523E+54904187', '3.80266844086179601296823105408593318305154647427719209683e+57809470', 71, 4);
    T('7.3947426471920811801225486E-7866933239936834', '4.2908870097907193193403713081672403E-951663440778', '3.1730005165581937336146170737035205542796096681463617864629e-7867884903377611', 59, 6);
    T('8.530692202261953090534719732944790524679571275085640252310E+359', '1.693998755079701783332530180054739191734520969602791E+337958', '1.44509819706e+338318', 14, 0);
    T('5.9743432603E-1001463741726', '-1.99815438476E+6453705200744', '-1.193766018163e+5452241459019', 14, 4);
    T('7.480649770069589030887388114676174469E+699166213658614', '9.4721271822434551198116357122188799E+57686259772', '7.08576660279194067770786434817941401208865199442124e+699223899918387', 51, 4);
    T('6.578386565090555E+859885280', '9.9913808626073631928E+78945537', '6.5727165633279158391604434505924004e+938830818', 68, 2);
    T('4.5324771562839805335564971975892112620824325', '8.46922995417269061416194167508361261939378844827405183931E-19048', '3.83865912986037435289432397168103605834545878884e-19047', 48, 3);
    T('1.598134046152070822E+90291748646757', '-3.573704147609646E+702297', '-5.711258269169840917919524202349012e+90291749349054', 75, 1);
    T('-9150585855201537447.17093949015331920', '-5.14387820140056095804051320336619478529719E-9907', '4.706949912e-9888', 10, 2);
    T('3.1183116652173302596374291125497175E+14529642537456', '6.050710744E-146289175', '1.886800189587103129698450187584296091141482e+14529496248282', 48, 5);
    T('-5.512825309913327508766218797522678285334861E-875124664223', '8.6395905689811513446566995822737939126870547566044E-57678177', '-4.762855355596777720791050912700786107786218987844804239e-875182342399', 55, 4);
    T('34463685622620730.5254278', '-6.767381973514615590702636973556973553710779916268769801211E+48380625571593', '-2.332289248233983627083458561821e+48380625571610', 31, 6);
    T('1.2158323309295795188092589715331456364183943160E-34776936117', '-8.66426979950164472094982403509814E+96', '-1e-34776936020', 1, 6);
    T('4.2728830699635409160929731139157927E+98', '-3.301583401443983514307550032102E+782', '-1.41072798201026380053e+881', 21, 1);
    T('-1.85781633E+43506295869', '-4.833916370556141420531529E+6110', '8.98052877107353071285287185606857e+43506301979', 55, 0);
    T('3.10695030779624770745754057801787E+5984632563850', '3.6524856264972234978E+19601334720', '1.13480913414e+6004233898571', 12, 1);
    T('-9.942593848452680608378064966492838E+255696004', '8.8E-379016105', '-8.74948258663835893537269e-123320100', 24, 1);
    T('3.423611948E+1283948127', '-5.5822831942876341954778915994E-75', '-1.91115714410827497802914772495546696312e+1283948053', 54, 6);
    T('6.76E+1884617825936454', '6.7304308318444943E+14271424241', '4.5497e+1884632097360696', 5, 1);
    T('7.01577039503744159824747714E+689895406239149', '7.2442304567846E+65641656933', '5.0823857573537959138292066800736746404044e+689961047896083', 50, 5);
    T('-6.576142435005608628351638116041995740926166662E+3115844', '-7.2903355979215127679723987344869137021492467E-4087616800', '4.7942285290923646695785059439879515958290901404170125e-4084500955', 53, 3);
    T('-6.1866784577835728E-71100857', '4.2272384169462571003127544684067E+694177625829695', '-2.61525648500365415719223521576341638989838494578e+694177554728839', 48, 5);
    T('8.65662447737894666034331185E+66056652', '-7.6E+1070332840909748', '-6.57903460280799946e+1070332906966401', 18, 6);
    T('8.47815134360704216544296029943806138562155920192421123E+4330860339', '2.7883781206504338016934085503125322073E+584123', '2.364029171007695446179297873e+4331444463', 28, 0);
    T('5.837026205625218667369389017355969E+88311528', '-1.310111019026942753022522E-835', '-7.64715235033862431584928127626465101520443896529154813382e+88310693', 57, 0);
    T('-3.36029645051E+4828224536', '-1.6062357535160031026521E+3074519', '5.397428301222080478323998797397571e+4831299055', 68, 3);
    T('-8.673E-50011', '-6.72808557122750035396892744707393621853E+8458317397', '5.8352686159256111e+8458267387', 17, 6);
    T('6.761256772E+39619526185334', '8.95569572062783693277542389884287800E-30', '6.05517583390663825534395435912220518415e+39619526185305', 39, 4);
    T('6.1877811132497960598861839789197475E+259', '-9.00302332517855E+62253', '-5.5708737693687208796419162293948473195046558416125e+62513', 57, 3);
    T('9.392426046861982775800267865415191367E+536767157708', '-1.708573E+937273', '-1.60476455481651184971973911e+536768094982', 27, 4);
    T('5.528942736E+35505136442455', '-2.26936743274451348085381330096132841103460302247475043124E+7131193856550515', '-1.254720258258774635382076612825031e+7166698992992971', 34, 2);
    T('-4.78999232119340054663004E+68', '-1.03607044421807700302245968905359037483289580525633099717E-635047', '4.96276947202002428412328015922849024e-634979', 36, 3);
    T('-3.630E+550557189968547', '-6.9297442563191745195552577566104780602E+446066', '2.515497165043860350598558565649603535852e+550557190414614', 40, 1);
    T('-87018.60344680032819', '-5.53145998797999474033085484732095201132479E+218190860899186', '4.813399231758740722502538517439473116217771059915267686828301e+218190860899191', 76, 1);
    T('1.372455339896724359805530478268085759E+1868875592', '-6.47640312665092184390252093103997E+206606576', '-8.9e+2075482168', 2, 5);
    T('4.959544E-31', '-1.4988381554505541E+94370179', '-7.43e+94370148', 3, 1);
    T('-4175.565995680', '2.83716E+9821', '-1.18467488203034688e+9825', 76, 3);
    T('-1.957250402417380E+713055116693', '1.24969368245479E+8017177902550593', '-2.4459634628830952235085602502e+8017890957667286', 72, 1);
    T('8.8307298587615035715478035E+29102647957', '6.99260174619911144877700648383185567950606868898E+1270', '6.17497770305883226879267794965782211692193461988408883403978610466e+29102649228', 66, 6);
    T('6.98109055347424E+35', '-8.42341962707084401578195598854344610E+4088700322', '-5.88046551864937747442540729255268626e+4088700358', 36, 4);
    T('4.8303643960057943733518098268236E+8748048442724916', '-8.83758614106360388288246E+2954111', '-4.26887614424278740409528914243420631711473667889939541e+8748048445679028', 54, 3);
    T('6E-383709676002', '-8.30265739599268492587787646723168E+93804221408', '-4.981594437595610955526725880339008e-289905454593', 47, 3);
    T('-2.7989889940E-49122', '4.98835186676995061260659136887049860587961303451150E+4218918', '-1.3962341973288446094609406893323919809149380572576630666431e+4169797', 78, 0);
    T('-1.0371177222357617346210209986E+8934116023', '-3.5223667440457414840540460432735820E+280161210', '3.6531089744637157652906534577e+9214277233', 30, 6);
    T('-7.7754058972988493183630423613007913E+704107829575', '-3.6068067177466344E+646070588', '2.804438622358428739472888061480163403385956163180072e+704753900164', 55, 1);
    T('2.7340247299594272410620519387E+202715194', '4.58655549149564986E+61591', '1.2539756139080322194349062908884272696429383582e+202776786', 64, 6);
    T('-4.19551427210514232121754446695633E+988721768905362', '-4.04472443385334586166551824E+7821775574383', '1.696969909e+996543544479746', 10, 5);
    T('6.63120609787840380106199200610285803780180287E+701900178', '-4.73496782994142735117809481631242E+54966765', '-3.139854754716566594374154402602578031945719893015482529079138540477096117e+756866944', 73, 4);
    T('-8.3892199047023189425150642E+56575040725', '-5.9045108880835114821046465940017389510195029357895074255E-817', '4.95342402698417607828723354e+56575039909', 27, 5);
    T('-8.51287056826924857014183032618803882692453605688E+425436598622', '9.758352767172626186490338652312865673750592798406543E+62', '-8.30715940664526285677404860295101012001208803647256e+425436598685', 51, 3);
    T('9.220177841E+3642123', '4.478974375333576801802125320E+21164263687991', '4.129694028585746181124760474216903412e+21164267330115', 63, 2);
    T('2.3018399232183618935486359044051510200806185462536791884921E+361062158979', '-3.320544214595285296487550432121723909698812758746492E+73', '-7.6433612399671873053409297616415497e+361062159052', 35, 0);
    T('-6E-99162294', '9.396302879373058117440247818542664461E+4743195482', '-5.6377817276238348704641486911255986766e+4644033189', 47, 1);
    T('-5.51405176822700037936278504402442266968E+7862580605013299', '7.68149735773333799786701381814705948822279998620E+76969078102', '-4.2356174088040543673959886436e+7862657574091402', 29, 6);
    T('4.82687576749088963618E-9129388117', '-3.05626390E+193519443193', '-1.4752206157967199573941067902e+184390055077', 45, 5);
    T('8.96461531304E+567787842516630', '-8.2024145090054783105268862030321352649103106690655381463682E+600330054013', '-7.353149071133198384379891228633138e+568388172570644', 34, 2);
    T('-6.7822689946791020754099776720473902821257E+835580260280286', '9.387858482049477127955952549205379E+485210641221541', '-6.367098150923938847766279010845182089824273649496711368216721302216199414e+1320790901501828', 73, 2);
    T('-4.24722E+9363613640', '-2.20490753439231723625632116510692658690684917E+8721740442336015', '9.36472737822173761217257e+8721749805949655', 24, 5);
    T('9.19658130425591E+81633499', '-6.162085566888940337802948459537814466664859881840002560690E+68395780329569', '-5.6670120919676009472432588961e+68395861963069', 29, 6);
    T('6.775E+174', '3.968969534724914524640096438726928257053134495435484506383E-977347277', '2.6889768597761295904436653372374938941534986206575407530744825e-977347102', 77, 0);
    T('-1.3952006326869720129585156365708E+43', '-9.71648869017897402537045771547245950322215389416662880E-951', '1.35564511680335125476893474071608120457585969714982339839255924235893539369e-907', 75, 1);
    T('2.08E-9653228', '-2.8E+75561', '-5.824e-9577667', 37, 1);
    T('-5420.624541469249915193742268838', '-4.50E-76', '2.4392810436611624618371840209771e-72', 77, 1);
    T('9906.2393980538153', '-3.5407955052322050641083062722506672733E+9446755', '-3.50759679343831339e+9446759', 18, 4);
    T('-4.823427508372904582493117940174767838E+4866542393235697', '-2.778778126678014E-56085958', '1.34e+4866542337149740', 4, 5);
    T('9.569173987684999648684395572843719569711755551109909434566E+25630915897', '5.3880E-532728', '5.1558709445646778107111523346481961041606938909380192033441608e+25630383170', 67, 6);
    T('-7.736831189737175355801076908746966385741656E+78', '4.1405386892E+4540923178949', '-3.20346488729160405402191345996915522132952179040773152e+4540923179028', 54, 1);
    T('-5.949509123338546394392584410314456E+717653', '-0.93277759162505732430608770871486', '5.549568791419035436771900165387002302931851356e+717653', 46, 6);
    T('3.953330391749398141052477722263501054121306798787374E+125303549824', '8.774599907048988096916129E+7612272032', '3.468889248797820862815754164643197286824759169351309e+132915821857', 52, 3);
    T('-1.9731688126418416867595662358537727241071509870224866455300E+7900', '1.5410801899741576314862142092143601440394374E+5355339739235837', '-3.04081136863717243298515319524338476386005072134020348637058379e+5355339739243737', 63, 3);
    T('6.86076666387539041676685252770E-384103669521', '-3.84407121648773464791333159901015E-1145', '-2.6373275656e-384103670665', 11, 5);
    T('3.4205336549357E+34', '38618615.05562787', '1.4e+42', 2, 0);
    T('5.286770221490776290434036322163077647942975676826E+35701378023', '-7.3559317070724077080006655568143167701297092E+5021602', '-3.8889120700270217037312e+35706399626', 23, 0);
    T('1.11E+582239017968963', '5.8327692917736929669575654243640089255136520535199634094785E+362282', '6.474373913868799193322897621e+582239018331245', 29, 6);
    T('5.97688521138323162732246292709938590E+767', '4.97737562770482346100596943980956E-64853838579', '2.974921e-64853837811', 7, 2);
    T('5.121214491848379439852458555496046688757E+659239380700611', '-3.27850E+2640835897920015', '-1.67899017115249119935562853741937890690898245e+3300075278620627', 49, 3);
    T('1.82884898E-8368', '9.132473800703990340184213130E+1555582746988128', '1.67019153952942160155757511949031074e+1555582746979761', 79, 2);
    T('-9.75590792971055249751848066230843789352348E+705999605364004', '-8E+6', '7.804726343768441998014784529846750314818784e+705999605364011', 72, 6);
    T('-7.46135393535854611112467839745306566754E+508238089422141', '2.104119073991416359041743556947915019428411247042098524240E+7612861517120519', '-1.56995771331888343180823678950061e+8121099606542661', 34, 0);
    T('5.7267073298E+325', '4.09789321235054323442253852171263080448953808104249472591881E+5515', '2.346743509590552383e+5841', 19, 6);
    T('-8.490369739695429E+42', '7.96432490125186873809420220879836041639875635293E+13192393', '-6.7620063138691652052e+13192436', 20, 1);
    T('3.128018287070845558439404816991058049271E+13994094', '-2.6081938823105668594395307296270846816334666363E+876870071059', '-8.15847816009375790177354208695914652329662602501885038386050641203192994720074e+876884065153', 78, 1);
    T('-9.988723280500097265756E+10615147', '-3.128067953662E+4101141311844', '3.1245405191729918882085332307398472e+4101151926992', 71, 4);
    T('-8.013458802008980403E+762115670175', '-7.321027367384966238E+413', '5.8666751196919691198507072928958633914e+762115670589', 61, 0);
    T('7.1932188781359748600673E-321297896075', '7.28855102795928494632102132922443029643142E-9493953232795', '5.24281428485740939974e-9815251128869', 21, 4);
    T('2.52449E+1348299067', '-8.17605358E+2502118', '-2.06403655021742e+1350801186', 51, 2);
    T('-4.142896937379302043037426110134E+3338084', '-1.3944132759200256137043027137181506427573734E+8630', '5.7769104902501137763720768211109796051824403310782012625244965463089620356e+3346714', 80, 3);
    T('-2.2540012468105446510424830415186596170249927E+2879', '-5.06466256932229454911683570517337141094031648307832702E+949080294577618', '1.14157557459271484441876e+949080294580498', 24, 4);
    T('2.839E+594036', '878698681.5713', '2.494625556980921e+594045', 16, 2);
    T('7.653886020587376650904919E+33395', '-4.2898314720962643868121725564239395344E-813', '-3.2833881e+32583', 8, 1);
    T('-4E+4', '5.09084154404901134648933092650486380193778E+166839617', '-2.036336617619604538595732370601945520775e+166839622', 40, 6);
    T('-8.4803144705332435163310857698466824866097574E+962164658486', '5.50218085287434815640095545715181319700178435E+628994148634', '-4.666022390612128e+1591158807121', 16, 4);
    T('-8.30311811867672599660222950503197E+1417790649875593', '-1.76E+13773', '1.461348788887103775e+1417790649889367', 19, 5);
    T('-8.664E+80415', '-1.9338623259961921659433394881E-84941649', '1.67549831924310089257330933249e-84861233', 30, 5);
    T('-9.2247023698307509127558184801258510062714E-236894', '-5.104685105551E+56028576866', '4.70892007904160465610865248505106261397122210719525414e+56028339973', 60, 3);
    T('9.115773563126810927402467946681336826E+993399', '4.9648274333036669E+841735555942', '4.52582426619963068491886172747971148765560344998072594e+841736549342', 72, 6);
    T('4.6770821644500767087600185E+47757523452', '-3.835243175755738745E-2573640364', '-1.7937747453656036530721708252030765e+45183883089', 35, 0);
    T('-4.158819655270889652878003173010422E+90530813', '-6.312639972330505554038769160E-456', '2.6253131193577e+90530358', 14, 5);
    T('3.8186183869831612962349460921E+830', '3.1848627486982368165920404367054E-186161914339', '1.21617754521968184619610273511e-186161913508', 30, 2);
    T('4.08E-970', '-6.715869700916E+939657636081', '-2.74e+939657635112', 3, 4);
    T('-7.4858189661777515084698568E+2232590', '9.26441652E+4556968040330239', '-6.9351744895986482331523061259954336e+4556968042562830', 75, 3);
    T('-8.5762281375867561568292044840844985E+5144', '7.6278223157267621542803429998848395114934467750E+8417860253866', '-6.541794437264802689839769039759559582425e+8417860259011', 40, 4);
    T('-5.0277979070585021883003790691838446211630295401E+82577748', '-6.39771232448839598524016415750315788958096328E+5370778500', '3.2167e+5453356249', 5, 0);
    T('9E+836602174', '-1.080573774694854323E-701033825', '-9.725163972253688907e+135568349', 37, 2);
    T('-4E+36688127986690', '-6.7103382916221E+7914351713594472', '2.68413531664884e+7951039841581163', 35, 5);
    T('8.017608005839527498078304413247064758079656104229E+35593', '-6.646986969650483101033054723E-744617', '-5.3292935942580733703349603743548785383905409136484497555734e-709023', 59, 2);
    T('7.13273177584811133696201158909108487376E+22502756274695', '-9.00674645744587786770386053722456911958156184216209035804756E+3238015278437168', '-6.4242706655e+3260518034711864', 11, 0);
    T('-8.1995983377767503761560272085681987445550E+5149029189728696', '1.6644724411041401332687495837429375244407484351552452399E+536452', '-1.3648005461352717475294379153325398682737867446326828376193e+5149029190265149', 59, 1);
    T('5.5465684484392088577225750175338834348775154E+691662812', '9.59441048920E+2148', '5.321605450077091483308302417186096514399e+691664961', 40, 1);
    T('7.430207562370274312830727721418113518829006E+435', '4.728E-6284760', '3.5130021354886656951063680666864840717023540368e-6284324', 55, 5);
    T('5.625963611342114887679777787951435803886895828483242103484E+8005482269814445', '2.8191076955450377586945898E+322030415433', '1.586019731159090795465534887e+8005804300229879', 28, 6);
    T('9.9021532486177501854194169035277124930764960161116335838E+163154902', '-6.7696670386332035023043692863451061471019103504187887E+14326085170316', '-6.70342804586622806093315622113219643725381142807033039349677602651291e+14326248325219', 69, 0);
    T('-9.5048817E-416472079', '49.745705351511432735360813476027664613990529757411', '-4.72827e-416472077', 7, 5);
    T('-1.2654713212006452254626995238789809462458126777886316758012E+720712', '9.51475606842043183626300924175862737350219054407E+4574', '-1.2040650932805860635797012051670432e+725287', 35, 6);
    T('1.58485E+65682899025', '-9.13869579582751855414446E+725768940', '-1.448346203e+66408667966', 10, 1);
    T('-8.4827830283035078167202599471402711932243811157469367261728E-304062917469', '4.98287772725673447123576558364284273748243E-51540710351675', '-4.22686706168849825112850088786e-51844773269143', 31, 2);
    T('8.73778E-390470', '9.919469121022383239113879290302408622555499968067E+9434038154', '8.7e+9433647685', 2, 6);
    T('4.842851664913308842403563790204331316E+23504336421303', '4.36761801309615593E-4873325807396', '2.11517261664280767e+18631010613908', 18, 0);
    T('-5.320291388012747126710302163804672566862999E+460215841', '-3.441340E+568', '1.830893156522378719703323e+460216410', 25, 4);
    T('6.970142962408475582190209701148E+11694043508715', '4.09713009042203377944178E+3921', '2.855758246582713995649654991949715398299794192006516344e+11694043512637', 70, 0);
    T('-2.12578513812386056186E+899595779884713', '-5.815364962381162291825573E+3384485071609', '1.236221640979609826279031049667943475689644578e+902980264956323', 65, 4);
    T('-4.760207305136646119871886E+77239487698224', '-9.549887636411798762123908495196950708E-60728699583', '4.5459444890081583546380355252401856042439e+77178758998642', 41, 1);
    T('-7.95579655204027997096004486088284921880049800848609E+4624229773126369', '-8.18775399176067981858253161778763892599754968094928358E+2341942635', '6.51401049766036554028710021444608646770532938252619590888e+4624232115069005', 57, 3);
    T('-7.158686739479523159164134968E+7703', '-9.26709969726463816721006047182573748199584278349886842E+5188', '6.6340263716243068744361262592242869518396e+12892', 41, 1);
    T('8.0402998480739673669539736826923156849523995028430710666887E+74827771', '4.318222958359487526033425903852541E+4141', '3.47198073960473054670745003693481034388149133979318030767882e+74831913', 60, 0);
    T('-8.7264885353265009E+3036435625236', '4.085429376608684108696366985E+822', '-3.56514526168617754243247146000600786223327865e+3036435626059', 51, 2);
    T('7.8927823360272360317418110088848244722730890300894562223517E+5358988457323', '-4.57695491032441608019143684361839820614050158568806389370759E+38094552655', '-3.6124908869e+5397083009979', 13, 1);
    T('-8797232214.91518600798368026755452142283009693684465', '-1.26437445998309750975824878965709984213009358717466905E-2123', '1.1122995731079297122976898109130521e-2113', 35, 2);
    T('-9.34845952792750874967295954709250198207106000215774E-468423', '2.500520484621173665795768530361E+899097202', '-2.3376014549234723e+898628780', 17, 3);
    T('2.1186871352E+9106414303209', '-5.72759190E+6140976236', '-1.213497527420572488e+9112555279446', 66, 2);
    T('-6.2701042701372665E-328246724621', '-9.962968794537E-2469901', '6.24688531818707782290253131105e-328249194521', 79, 3);
    T('1.80651E+68', '3.1467735454907204347315818610449587975929E+701415758900573', '5.68467787766444e+701415758900641', 15, 6);
    T('-7.8375043820238380888065335445516353980E+8894010', '-4.37110588E+15536499765925', '3.425856148879016497015020095900689545181394024e+15536508659936', 77, 5);
    T('6.0788474434744253055478272101669458564261357898349390E+901365222158225', '1.0329954799832672422980738996237266725200773190758E+198690289942', '6.27942193261692095484989395679080062e+901563912448167', 36, 2);
    T('5.964278669897848743774291764510456291551E+68499782556', '9.2E+777', '5.48713637630602084427234842334961978822692e+68499783334', 49, 5);
    T('4.236E+25508795776', '4.97154299318728423339423591180615511809067E+86234358001452', '2.105945611914133601265798332241087308023207812e+86259866797229', 46, 5);
    T('4.9242697154632090774230518E-47931', '-2.85547557724973408E+8365294', '-1.40611319082956907e+8317364', 18, 2);
    T('-6.332977384375635E+1515', '5.42608690254282E+4749360252', '-3.43632856394605193052195521907e+4749361768', 39, 6);
    T('-9.543699E+497', '4.1220788E+69555656879', '-3.93398793214812e+69555657377', 36, 3);
    T('-3.13579777139954321507868770539876479720E+52251469', '-1.956229419226042097238260038E+552292338558321', '6.1343398531552455452609879585671478910879541484473102341339342936e+552292390809790', 79, 3);
    T('-9.93472286847710551628128098824E-687020470083', '3.4829E-14346', '-3.4601646278618910802656073553941096e-687020484428', 52, 3);
    T('8.99955753697093E-9427', '-7.46011100365132082760416619622824034429033339229E+1859226', '-6.7137698209550013447194007647408028251381189850017727e+1849800', 53, 6);
    T('-2.20E-6940', '-3.7402197997865033505979061567669339024108E+33655391', '8.228483559530307371315393544887e+33648451', 31, 6);
    T('-7.820416579804629912779494832631256242E-326143872', '5.89411850284770840500477209697E-8', '-4.60944620630034615795153245604674988783e-326143879', 39, 6);
    T('-2.817E-856319', '-5147.4752257525167789010480463709574656831412403', '1.45004377109448397661642523466269871808294088739251e-856315', 54, 4);
    T('-1.6119731502835662460E+14811', '-7.511736881861081776890193777711405911416944829887273520E+102250898', '1.21087181656e+102265710', 12, 0);
    T('-1.266446289108549230120104330157088991E+698831708016637', '-6.17623948412626603E-77103016022665', '7.821875575317410062159559176462406945671e+621728691993972', 40, 4);
    T('-1.354808909480029022456538577305E+277655480', '5.71770258173162381426114289634500122830980E+580', '-7.74639439948696777160341589663883829835503811386321e+277656060', 51, 4);
    T('8.2207095235294713038377131826498224340077102285E+299241745', '-4.8564045E+828768333', '-3.992309072326138032257833757e+1128010079', 29, 0);
    T('-7.86377569844505270579E+494021525', '-9.53756519177353981E+780862290', '7.500127337740419108e+1274883816', 19, 0);
    T('1.28605830204544412736672909356284546699547221477049376929744E-12579030057', '-2.730278965169449535204861897657226912113531815098659721925E+8978652193', '-3.51129793005621455656011568552e-3600377864', 30, 0);
    T('-3.6105933711586276086260765863610949008253602483978631604E+6897985', '-9.0355081946687355031132950173053E-57713483799307', '3.2623545992720395006134832843249286265e-57713476901321', 38, 0);
    T('4.55253E+555082487272217', '7.5928890459920349981244958164536297970784E+5008121902', '3.4566855168550119090011710939279643260093328352e+555087495394120', 49, 1);
    T('-1.9274616009305303E+27977692', '-2.84011809691718116304204699E+377', '5.474218573915761016766658421420448898218797e+27978069', 53, 1);
    T('-6.72561690E+38151573384870', '-5E-664710', '3.36280845e+38151572720161', 49, 6);
    T('-1.748348192783166618989E+20177341727', '-6.95095899752464642E+96446485', '1.215269660143210710066881879694908286938e+20273788213', 64, 1);
    T('7.83164667060853185723882204810904413347928338334225144E+4628271781072212', '-8.58267618853945214417148763089510219229858728E+882', '-6.72164873968861244296163182501048398758876469427479630039466511e+4628271781073095', 63, 1);
    T('-6.2400954612129322584268168846440567722058440033470E+99748602', '1.7053285615668220519399564699178031970688526076883335E+693725066', '-1.064141301690990479647940707381791653958660327765719015578613135e+793473669', 64, 2);
    T('-6.01092971701E+36609', '-6.92490902E+321', '4.16251414159085964302e+36931', 37, 6);
    T('8.305201791997993444847415262079631792401056E-536620877', '-6.4887205125132566545996285E-888291', '-5.38901332282992376159586939041873493e-537509167', 36, 5);
    T('-4.492772052484758929097488794998930E+68807964298', '-2.30339940537775402306121178065871132459555878431865347250351E-152926044603726', '1.0348648474191185207e-152857236639427', 20, 2);
    T('-1.4309852872795539824996831352513324349662553592186E+8459470141', '-4.371939943077920773103110612994646342754410E-875978590', '6.256181735414315343199087367418090307304825763261407282805e+7583491551', 58, 2);
    T('1.92465117969521042302745977E+25481854', '3.751694914992469562074459404443861926E-890988', '7.2207040439968e+24590866', 14, 6);
    T('7.78217030330533767366629662E+2792', '-7.58100005853793228452E+95540734840789', '-5.89966335249099231455545213507705371449245543224e+95540734843582', 53, 3);
    T('8E-68', '9.83448028889479539092771173905476737841417295263914580619349E+1581330049190312', '7.86758423111583631274216939124381390273133837e+1581330049190245', 45, 2);
    T('3.092433453668623433722093656416436762E-347767917237039', '-3996413080.3143092849546473637763213967299581933742927082419', '-1.235864150424284122371390382240494280966811539087e-347767917237029', 49, 5);
    T('-7.45983844805032584200247720185359707563E-6879', '7.38078647626412E+8243104116801008', '-5.5059474732484966073e+8243104116794130', 20, 4);
    T('8.19492217130130771174531008801974292571486506E+38679161208', '5.471E+911', '4.48344191991894545e+38679162120', 19, 0);
    T('6.974319423E-98', '7.537643426061E+74555383', '5.2569932950025496682803e+74555286', 35, 0);
    T('-5.044072867680879381940390300969976030983629123407E+789', '1.785442145297780261646228194618E+7643521', '-9.0059002819104757974785775509678103791726633555404182198356215483599915e+7644310', 71, 6);
    T('-7.773964104860106311459636456258584783576909907E+965460464687', '-4.1287388532444438947724063092227241494E-65', '3.20966676434635851214468078125021575900440351e+965460464623', 45, 3);
    T('-6.6202823E+3382973', '-6.07565093753491667E+67', '4.0222524362740814462375941e+3383041', 58, 1);
    T('4.3456177921136637756833564793439E+5858191319714502', '-32166819.8989469015837', '-1.3978470486857949978684052534679048030494459524693443e+5858191319714510', 61, 1);
    T('-4E-73299677969165', '6.682594175947481328E-9585991', '-2.6730376703789925312e-73299687555155', 56, 1);
    T('3.0408E-59953584', '1.5849978522356773E+876', '4.81966146907824753384e-59952708', 78, 6);
    T('-7.39470142431036193654690988E+484264', '-8.8262197438324138192419950319E+321171108', '6.5267259710993788338590458056437971238194846952257025172e+321655373', 60, 2);
    T('9.85659338550016872623051759068997E+478533821439642', '3.5140E+42648661082834', '3.463606915664759290397e+521182482522477', 22, 5);
    T('-6.5462222200943641789587079980508874637E+8888845371', '-3.967677180359357027716817487805679680437E-239042907675105', '2.59732965202297771592507988233816491903904724e-239034018829733', 45, 6);
    T('-9.68459322687735339086645638844915532182252679092249E-71917622577', '9.7414704484287804772800E+620643', '-9.44e-71917001933', 3, 0);
    T('-9.6712506002940949071E-47283616', '-7.2765898603250034072572551201297164E+1769656228', '7.03737240547621134164158567877720977216853e+1722372613', 42, 1);
    T('-2.44485795209686430001594708589805415193853410507095283278E+81239656316', '6.517273944134169095858E+51075692223324', '-1.593380902831011824753180848577617584022653720006139524719734999883265647e+51156931879641', 73, 3);
    T('2.807815856555247E+2823471555162', '-207.38793361675878012458668547556812509', '-5.82307128467362258138e+2823471555164', 21, 6);
    T('-4.7863282760643413861760011727467E+982', '-9.12646410247275584767425106E+77', '4.368225319415152218e+1060', 19, 6);
    T('2.85446533202850332252969360772468E+8292947136965784', '-4.8238419893E-7', '-1.3e+8292947136965778', 2, 2);
    T('1.9353280462829214E-80', '6.862455709745747671559246484E+77619284', '1.32811030014453165748470668463963724621983576e+77619205', 50, 3);
    T('1.748182472269418312787684460656442620458042237984248E+1043638', '-8.90406325343341857980360982538698570370467352821938381E+751959310832', '-1.556592731163e+751960354471', 13, 2);
    T('6.201490108E-490634577', '9.33634089825872E+2927772367989', '5.8e+2927281733413', 2, 5);
    T('7.897161876758033800140296800599519067E+66667267602', '1.45805635714697932948450486826224090233419839059612530870E+21790176502', '1.151450707782582129059191712002907340711179432103055e+88457444105', 52, 4);
    T('5.91849191450095892258493508215702806E-746738901047', '-3.058116450355342555E-50', '-1.81e-746738901096', 4, 3);
    T('9.2368603064686935572989315692444747826321', '1.354763986945033646560358037169471625116E+5170441428989369', '1.2513765695645852646089555695e+5170441428989370', 29, 0);
    T('9.2314237E+6689910426', '-6.67362145138368383352670485135610956108771E-823', '-6.1607027231131736734125277747713766942021683872727e+6689909604', 75, 5);
    T('6.1031984171549051E+751670016463472', '4895.162897972724', '2.98761504506225473242046626084924e+751670016463476', 78, 6);
    T('-4.45252395938849604800516681431681014E-6520432218', '-2.822400589E+16104631', '1.25668062455147033e-6504327586', 18, 3);
    T('1.885317723259102405486698013602E-5175062', '-371587908.242884710438627285622759227974200951', '-7.005612691590876541111689748086822299858141324320473271347458283548793e-5175054', 70, 6);
    T('-8.52938807633049885E+960', '-9.456E+12406', '8.06538936497811971256e+13367', 49, 3);
    T('-5.509680082866739129581956873831220561E-668387430352', '2.1800922567E+613392220068', '-1.20116108855519923143805220886727828077459300087e-54995210283', 54, 6);
    T('-427340097896011181846443.739519029350955453397631003200677945', '4.884061064532958337554E+9398012', '-2.1e+9398036', 2, 4);
    T('5.39527E-56815054304', '1.785318852362111545974982946048483133703009E-55134', '9.63e-56815109438', 3, 6);
    T('-8.97113450484596577525995439E+1407976', '-6.050464838696413804747151708113715626E-437214', '5.427953388478667844277999827479488852765017765819885915e+970763', 55, 2);
    T('-6.849814674023455029029433438382911369152337100100E-490797592203', '-3.18016179673058630E+430470', '2.1783518941013966049623977303145261122276764550269e-490797161732', 50, 2);
    T('-5.76644670337320144632510277E+56636792910131', '7.496061562544991914642828632049864603367431934068371971891E+26039853', '-4.322563948562013793223759738855485921377372913678e+56636818949985', 49, 4);
    T('-8.111740512485539879459251977214608115978E+59803802225', '149.4281207744549513136802675296129298218904445060209113', '-1.212e+59803802228', 4, 4);
    T('3.3272067830845653105293211970025525578642728385726126E+898174076394126', '-7.38054E+437214752', '-2.4556582750826957656974076267325219255419580255998710198804e+898174513608879', 78, 0);
    T('4.5236507999104419574487E+864264005548396', '2.9E+695643896564926', '1.311858731974028167660123e+1559907902113323', 78, 6);
    T('7.1806443610057034591919471E+840', '5.9800620632234513734019999881034734954378E+1321395090883470', '4.294069893274960862731968253429972418009407747700111241672634894038e+1321395090884311', 67, 6);
    T('-3.44893180082595530830505438953326429E+684262', '-6.52475952877026516717565349437198630518857803959721767E+120540035606179', '2.25034506315179421977062132848172653887e+120540036290442', 39, 2);
    T('5.0143029389448239788471998801867044401680801802621797E-98675474667952', '1.79533657920050900633182997182990772726140376151493E-92420277', '9.002361485480259051675301544443909e-98675567088229', 34, 3);
    T('1.043599697712746827730377803E+169546283149', '-9.539106873911401139178438461145189104E+591171528382948', '-9.95500905006352359745346536920749020262294291642e+591341074666097', 48, 3);
    T('6.8663534032121167868769749174708E+19765475', '-4.33915514126783782370418931389E+8703773693', '-2.9794172671309771621e+8723539169', 20, 1);
    T('-3.8931378310220080910242042783500205E+60', '-6.66961543067348529243054238453818035025201077772087190E+215', '2.5965732151523088904078141981069125266544155611052946158e+276', 56, 4);
    T('5.0641065479186E+241380589451147', '6.084476742222140840631868079667547E+252584586382', '3.08124385109455750948051853352891763641128176742e+241633174037530', 72, 6);
    T('7.607618981429355E+963151', '9.519082870974135222219546244891217735089549E+5912396447067', '7.241755553502187090268517279017777559e+5912397410219', 37, 3);
    T('-8.57049331364915E-127', '8.186027027561059804559E-5601', '-7.015828990506328919910725838609647485e-5727', 79, 3);
    T('-9.890378371117191539838790E-518292', '9.9916076157462830809029085936733711848167E-802130098', '-9.882077985546684908895965840564964e-802648389', 34, 5);
    T('-7.92140506702719888136095886E-7694', '-4.015388781E+43', '3.180752103589756741007254421784654966e-7650', 62, 2);
    T('-8.8954E+957493', '6.0504111749818682519956403391616650074020707804518E-34', '-5.382082756593371084e+957460', 19, 1);
    T('1.55484478599297E+515975667', '-7.270366648015966476378722563189994356223973482101E+966013085', '-1.13042916749248120430126966943875758325250462e+1481988753', 45, 4);
    T('7.740710507068218872364671549435797146144552431325E+332636184973', '-4.152440657388116E+476866458', '-3.21428410266214515173430447081950763439360341600751694919611337e+333113051432', 64, 4);
    T('7.5019190700010143739244972084795793043566699037E-1834', '7.160327483943E-550', '5.371619729924437374250057248344391320485932842983e-2383', 49, 3);
    T('6.420611359212949411810975732717430921234061364604763877E+714010035', '-9.73999076315199374868197658620E+553763729', '-6.25366953325228950103420439e+1267773765', 27, 6);
    T('-3.056938E+78', '4.901713376145377449552063959359E+22898', '-1.4984233884647097849878787295794982742e+22977', 39, 2);
    T('4.570041888683739772958142594160987690729821094492418419E-68661254', '1.1521326943467341444725155336503082192E-1373', '5.26529467448663478308952875497790311e-68662627', 36, 4);
    T('-8.32402986206890996191210952788597112E+90894087653518', '3.5231376620001791287442144441466496601761536053875927E-7140844012965', '-2.93267031066691329991e+83753243640554', 21, 6);
    T('8.5178E+7228080250', '7.2359521919416641939641293066713712631317261894140694869E-116982179', '6.163439358e+7111098072', 10, 4);
    T('3.02271709E-4011379', '-4.7783079711658436891594018376599858E+238220', '-1.44434731657262229434907716688722446868174e-3773158', 42, 0);
    T('-6.615710797253204005129665842860612073E+25054537400', '6.610103352054986200757385439042E+822598604', '-4.3730532117e+25877136005', 11, 6);
    T('-4.01786107788541836298513017036763E+4111', '4.314534585763864400067919721890E+82051517723', '-1.7335200581331117236108618334371333710493566580748817748658421e+82051521835', 62, 5);
    T('-8.84763946246027608851528022220476070366861E+124781659742', '1.46721417314548553092767E+733516050724', '-1.2981382018203022051215259739909910987754985070438621e+858297710467', 53, 2);
    T('4.0430674886206533183571329E+4879', '8.89558300E-308874491392761', '3.59654424196265771076712993539807e-308874491387881', 59, 3);
    T('3.916504514015007447593561602167499031E-3230052425', '-2.387155360336967923691395E-132', '-9.3493047444148565e-3230052557', 17, 5);
    T('9.85665743261128630584622822479115697E+7162', '-6.8565480059671905612066958E-9977', '-6.75826448650726030947032353619090658327810689e-2814', 45, 5);
    T('8.18549432137545922323029015175103051931927159154663484E+258033592889', '-7.529791000838697949571517E-6080298185', '-6.16350614785e+251953294705', 12, 1);
    T('8.05586931844774214905498663E-132751954742310', '-6.30137067748919117776878507411111E-38', '-5.0763018704951437734754926e-132751954742347', 26, 2);
    T('-3.89528372616700E+620037093690', '1.42907122771145390393268129786505463180002E-15669', '-5.566637896837921510815953e+620037078021', 25, 1);
    T('-4.7841505891580387362195290166077032718817063171E+14230297097672', '1.981222561477463734072854028813871886016E-55941031', '-9.4784670847456067433763004724817647e+14230241156641', 36, 0);
    T('-4.4269853692E+6031570', '-2.21230610097704878688471476309342545339717E+419', '9.79384674121729280453324110332984e+6031989', 33, 6);
    T('1.53791047431675585769695934061701528E+38', '-3E+12774', '-4.61373142295026757309087802185104584e+12812', 78, 1);
    T('-4.233787724278538425615547944209835927296160312726170279E+53009136340457', '-2.63991797E-73454812001396', '1.11768522944883188751179933293162e-20445675660938', 33, 0);
    T('-7.484E+40556', '4.571155686E+8261215', '-3.4210529154024e+8301772', 44, 0);
    T('-8.32E+257', '-1.33830312131494334776816094E+183', '1.113468196934032865344e+441', 22, 0);
    T('-8.5823685680681952417812342721893588391239466144145700E+5713438911144', '2.00362970125924533295377644E+698882', '-1.719588857013521517686062194591409495141017383903000698780992170593362125873e+5713439610027', 76, 1);
    T('8.9864359125771354535051E-3845025610', '-2.310114282442616820220026569160192299450355073E-7471081', '-2.0759693949899691726526622831800174111e-3852496690', 38, 0);
    T('-9.5874886195147325962650E-3109117', '90.921306153345044358366294891726322037593715097548022147', '-8.7170698801661044153800322460457742384164553045392023504573006616824103e-3109115', 71, 5);
    T('-2.75388549757E+379', '-4.2226334585469119346504771031532571741329785710372E+70', '1.16286490430461925423419004612551e+450', 33, 6);
    T('1.716146166255368329E+8256040', '-8.91824991634836947679500501147E-251308', '-1.5305020403648513576695447e+8004733', 26, 4);
    T('-5.81432636336989448711477061789391399250E+986345', '-4.84903310429518430869787066420160902263370357054E+147152948077', '2.8193861015156849273121770826835986389908159983147701838760485883e+147153934423', 65, 4);
    T('5.12913601760552819259710359565838403192005019629799E+8315368539423697', '-3.2891208911261404555480857E+241175881113939', '-1.6870348428933878129189460423905272341236850870962441798696680830058571257743e+8556544420537637', 80, 2);
    T('2.94455008410961833153773089351412471097779112734E+9808', '0.00000179707814311856313177043527', '5.29158659747132179e+9802', 18, 3);
    T('-3.63778808949062E+9872893634', '3.857817707447151467154714356290232E-550005', '-1.4033923307577256728e+9872343630', 20, 5);
    T('-8.13407576109845040228677720284523471E+185539722937', '3.952731568761076505867030444297692723188280947523E-2256956226775', '-3.21518180435882e-2071416503837', 15, 3);
    T('-5.167806958440027195173820592413088652272945E+6656913', '5.934938029818879525453321341146634967629700E+5623', '-3.0670614048408351226193054490893714821133e+6662537', 41, 2);
    T('-7.74495E+84429605', '5.8422E-5971236869', '-4.524754689e-5886807263', 40, 5);
    T('8.2019157625921746E+646932415897', '-8.169399991744208269824842512239026E+3684049', '-6.70047305632072028528382546148011634456759523259396e+646936099947', 61, 2);
    T('3.99608033058195001841379749720836673E+507', '2.7889781928153316987393882208578984612991424E+4091570176687', '1.11449808987313402344225170143169992250133683e+4091570177195', 45, 0);
    T('7.87852677969930922521623703366381009205E+9065756', '5.120519433398892016268711435715359904790E-31760', '4.034214948200390421677721093095863000888331727230437178176605280407e+9033997', 67, 5);
    T('9.3437342354744815801791278194715637051806742612971768630748E-1189', '7E+93788224', '6.54061396483213710612538947363009459362647198290802380415236e+93787036', 60, 2);
    T('7.377789574618772E+2424954017', '9.63923223456366234097867E+17928', '7.11162271e+2424971946', 9, 4);
    T('-8.82101213308978340293222734443E+899183169765854', '-8.65E+20688633843921', '7.63017549512266264353e+919871803609776', 21, 1);
    T('3.34263E+64097', '8.2041088462829606093533181924549930567E+979403425', '2.7423300352850812621642681989645834e+979467523', 35, 0);
    T('3.1189321902917096E-665928445', '7.55617918201E+12824926749589', '2.35672e+12824260821145', 6, 1);
    T('-7.971833210664265820E+5772446085464', '-7.3237043E-99965', '5.8383349163824689442277026e+5772445985500', 51, 6);
    T('-8.4552466316E+56', '8.131008174677882253988331519783384760113034677888544984349E+1520021307', '-6.874967948045722834305625574835257275459251049547111098e+1520021364', 55, 3);
    T('-4.8231604747266621231302713391018463E+908065', '6.4805E+99017', '-3.125649145646613388895e+1007083', 22, 3);
    T('7.5236548136940304140087872615671941523074464485246E+67113792861', '4.07041E+53880', '3.062435979020831833748550775736e+67113846742', 31, 6);
    T('6.64849843', '-6.7778914123710E+14250', '-4.506280041385907607753e+14251', 33, 5);
    T('-1.018101623269407910432765896420952796E+732616', '-9.0354618236E+416168', '9.199018349595924e+1148784', 16, 1);
    T('6.1559100803382221298891505076962191E+707427148', '-4.3642535210292868297043481527023558050028504981714E+443083626390332', '-2.6865952243255765891446990109882278617590654664408343395346015898243763e+443084333817481', 71, 5);
    T('3.01784010222694086152925978930873E-77726599', '-2.4482070237735233412384757311E+1510640232787992', '-7.3883e+1510640155061393', 5, 0);
    T('4.4E+76116748768920', '2.10645990752066940845640960265827012930911491271798E-82185440', '9.268423593090945397e+76116666583480', 19, 5);
    T('-6.3655009947016302991638E-3040812029', '19003.8', '-1.2096870780311084187924902244e-3040812024', 32, 5);
    T('7.7490534309051964630097180856005299850319809920E+35185259297119', '-4.6713777875324574814920317252190137378743092228E-29101013687030', '-3.6198756071532715533941532817133345781933696592845183882188904469132e+6084245610090', 68, 0);
    T('9.9479884897722089E+577089280', '2.2951298271789347878E+169', '2.2831925103308922291519092969859e+577089450', 32, 5);
    T('-8.8351135736398112772435086937E+100218645711861', '4.757822297291143847717536186196624142617970910246128637648E-72', '-4.20359003597631345025921405072533990587441833718953899047400561387625e+100218645711790', 69, 0);
    T('6.5088571500E+67256757169', '-4.36966330502977926', '-2.8441514246035709699372709e+67256757170', 50, 6);
    T('5.080779114749125382672868461365386364428597505240E+77886156362', '5.2428286958336410753940884200982278067882002277E+412498695634610', '2.6637654539998958e+412576581790973', 17, 5);
    T('-5.0913539452228712068313232260397354647642092023742972054109E+73639930054', '7.46372103386029269995878998563971951037E-88032', '-3.8000445531787528331058661648495459137e+73639842023', 39, 4);
    T('3.717843621170022249310215701665280410980056159219590227798E-3952092299', '9.924765975704E+8932901980', '3.689872787437638841925069167931271771570123518194173118238268109342e+4980809682', 67, 4);
    T('5.06965102103620967006754925043444340349538585019450690748E+834948845', '-8.9981142650543220893E+852', '-4.5617e+834949698', 5, 4);
    T('1.9002715220438514075244169985E-54', '6.25E-9247929873027', '1.1876697013e-9247929873080', 11, 4);
    T('1.7157187426959819844884828793E+5748849', '-2.092932391080601911002367095268625761341198E+963132246252', '-3.5908833305725055702922800942253856828e+963137995101', 38, 6);
    T('2.34525933396839913198183244601305408738686630689832510E-36', '-7.0707829159873224912446592E+537551201081', '-1.658281963218356302054644096837354464231957239905795328458329212261e+537551201046', 67, 6);
    T('-6.2204015E+91151805900', '-2.28246731293171594129885903712232918875262624731737419557810E+72188282549', '1.41978630970614152388294e+163340088450', 24, 2);
    T('-9.71860880853986716877435514422734E+29591218350', '-6.541379098195809428168269021111404462624963735359000010E+9359545613982', '6.3574e+9389136832333', 5, 0);
    T('-6.672705975863573834019898E+5975495554076754', '-8.25363730E+985278164290910', '5.50740949343204927077706390749954e+6960773718367665', 62, 1);
    T('3.64518388207970456776671390123250912196360170680051302231580E+3227', '-8.78805986993518E-83288351', '-3.20340941926391830342099475071099185590593725189114139241910871929e-83285123', 66, 3);
    T('-5.4294416947129586458074082E+587', '7.60356E+17686448911', '-4.12830856922517e+17686449499', 15, 4);
    T('4.04544523438E-88649', '-7.46847E+2137081275791', '-3.02132863696099986e+2137081187143', 33, 3);
    T('-7E+1276922645477', '-9.7373423050758186036255776894894454847004911945624221088396E+5522508745471614', '6.816139613553073022e+5523785668117092', 19, 3);
    T('6.22214581965172340975E+241238411377', '9.43404057276628354629450975893874E-70436510', '5.8699976112262481521547962141861758845928266174168715e+241167974868', 76, 4);
    T('8.060232590311539827E+419573584', '2.24746228220747263807193154817744705993873268865661892E-6682004', '1.8115068732544622109508598632641177063593e+412891581', 41, 2);
    T('7.95893565741891846073927214837E-2224856813661893', '-1.4203476961362636251428216104343002E+111394', '-1.13044559247117195674812596336295988e-2224856813550498', 36, 5);
    T('6.89255280241788598557965013372049511937999978338758489477686E+158091576', '-354471.8125474474127511', '-2.4432156849520562254518398884386691558433990396776805261e+158091582', 56, 4);
    T('2E+43651235', '1.621282160706463970543544744248618037415E+5406774601', '3.24256432141292794108708948849723607483e+5450425836', 74, 0);
    T('-9.022E+5604849093062', '1.0642065236E+84', '-9.6012712559192e+5604849093146', 52, 2);
    T('0.000005250780245284877273364870560336681658583993131623', '-9.0366733259603260E+4551896796437', '-4.74495857830452682719485038e+4551896796432', 27, 1);
    T('9.2805697854911318296665682965557886994059010394410594E+54698252', '1.1082429695448601279E+6770045', '1.028512621814099750238765358204414715878171646e+61468298', 46, 6);
    T('-9.501850548767388079972686511993165E+60', '-2.582617469785588E+1209801', '2.453964522253843264174490409570380878918307150602e+1209862', 67, 0);
    T('-8.04281843306206001047469950E+9050021472977', '-9.86E-9973114', '8e+9050011499864', 1, 6);
    T('2.817663998386E+31997878007168', '8.9477372702164168832228142043654E-509', '2.52117171733054221067198307128074082997542444e+31997878006660', 45, 5);
    T('3.215806202E+6417286566211724', '-7.624357977843325953704515589547736E+534140786448341', '-2.4518457671416746186230546108273295803858672e+6951427352660066', 77, 0);
    T('-4.0030001317805484E-31350', '6.4828880823437644901250388956598990E+83853', '-2.59510018479406359610561725284505859383918701086116e+52504', 52, 2);
    T('-5.687626399606E+2184', '1.326493649186222891603617358817312997511611E+9289', '-7.544600298021261336789700606215796942336151709110825266e+11473', 56, 5);
    T('-7.33686745213485838409743383054573343582487448366974279E+44783', '2.24502802416777771326750524734628702171615335802021583885188E+432902204381', '-1.64714730396471985432360689458538397423924088420793033669906975192e+432902249165', 66, 6);
    T('-2.20907513988048217835E+61191971773', '-3.4778555027E+618780306542', '7.682844131111107164303830306545e+679972278315', 51, 2);
    T('5.44074', '-0.00008207544575093901831136884619026208518896040', '-4.465511607149639544874e-4', 23, 5);
    T('6.5293704091884398736675316619706971461E-41734592340732', '991960.13187764099854901315', '6.476875132176531586585389114509001185843057039622223455676371215e-41734592340726', 73, 2);
    T('5.5653113393998733626810465968240043625953509587560837E+4158889016', '4.2110982634620123690492030629338259318446351435545E+10015930340559', '2.3436073e+10020089229576', 9, 2);
    T('-5.450588568893006E+6010', '1.797746E-8778079261', '-9.798773797373125964476e-8778073251', 61, 6);
    T('-1.739358884823025672481950797595160E+90741892760', '-5.2302502396843675978475820642220115882887072003499211E+8194041515646', '9.0972822242427643580030421260754e+8284783408406', 32, 0);
    T('1.94075718947009588031925357197771487338E+3717859234931', '1.93374755169255734412879799438427493187679993535573058655700E-27478647', '3.752934463567526541e+3717831756284', 19, 5);
    T('5.7485418077235965188E+50', '-1.3E+18972514', '-7.47e+18972564', 3, 2);
    T('31772.206525223748210216332942656479455860734633826735', '-6.88802618990478936575023989517525777861210038719E-59107286', '-2.18847791e-59107281', 9, 3);
    T('2.64388929907843746089716428521E-59294447643987', '9.430916411809196488300268315980166496741273590217124E+70', '2.49342989816855489624811668581079614313268856387454055e-59294447643916', 54, 1);
    T('-4.527096844680093375190537537768605388219431E+2776293', '2.36434857845762126452920E-46204099008', '-1.0703634989259361419247623562676248885703699039080450695026506885e-46201322714', 65, 4);
    T('-8.321727561074492330042E+789143824050434', '8.610999521550341E-48', '-7.1658392e+789143824050387', 9, 5);
    T('5.49052172607995437482982515734477922E+4937', '9.8422811826846860311457155E+939806487546', '5.4039258667718177100355169867e+939806492484', 29, 5);
    T('1089487815843762662235025755', '5.5449428244E-626710448536917', '6.041147646734100405456746926536442422e-626710448536890', 55, 0);
    T('2.7404717086281901480789E+94095', '-7.86134427003750180714761585661559106E-6728', '-2.15437915638241048225809e+87368', 24, 2);
    T('-4.86414458954399384101363E+915457907112', '-4.5178546033E+841327782162', '2.19754980249881216224902766335429e+1756785689275', 33, 3);
    T('-1.4E+874189316698815', '4.8347334943984386058377129E+6711559494370', '-6.76862689215781404817279806e+880900876193185', 56, 0);
    T('4.557722988E-432473895159', '7.6881346332759420247710E+784566', '3e-432473110592', 1, 1);
    T('5.1855244245958792277118458822946855292168633193E-5696647764283', '-7.61860172038577237307349702375418E+6297851', '-3.950644530232860785206467321073341877781988462945782711293e-5696641466431', 59, 2);
    T('-4.517007358474848131437281166627905797986E+814', '-9.736053115492E+138256626118', '4.397782e+138256626933', 7, 1);
    T('3.551575838138435869707739474537920805856101E+36024', '-8.8653042373128875950424808E+8128', '-3.1485800326986745e+44153', 17, 2);
    T('-1.7337729107172627E-83', '-9.543316617569283953047460287836751E+2712165598603', '1.65459438299395196099640106365816559804119751814877e+2712165598521', 68, 1);
    T('-6E+883378', '3.76E+62493814', '-2.256e+63377193', 49, 5);
    T('-6.56062E-7618925', '9.80579361380653364884E+7671688783866072', '-6.43320856986114207872526808e+7671688776247148', 50, 2);
    T('667299262502921373229958085619318761786958.255625', '-9.52457216913688542594611965157021693048936018579173676587025E+20116', '-6.3557399841208937364425643209747757358407884648e+20158', 47, 2);
    T('-8848.4096365208891360771307246', '5.2273655852973602490763155989E-4944', '-4.6253872019e-4940', 11, 4);
    T('4919187438.8304585858', '-807434205.9', '-3.9719202e+18', 8, 6);
    T('27.4789420151035952567992920081100866945756488729173', '9.7156081E+758701094', '2.6697463162137081241608078150825962398152140025247119051013e+758701096', 68, 2);
    T('-9.7544411254532007416155519168E+86370421550504', '-7.4176321E+952', '7.235485560972178886875132375727220928e+86370421551457', 61, 6);
    T('-9.674E+45190767', '3.0968265260582877924107305072094271049849E+5885656502789569', '-2.99586998130879e+5885656547980337', 15, 5);
    T('-9.1640605832106032627161505236997126448673848959381E+4071', '-1.56942631199471249870366163995667504158716744782690352E-2013', '1.438231780400433122e+2059', 19, 6);
    T('3.3793821357963088813595765446646325012522172027957E+91378', '-5.20100342809858689280937292250205E+9353150', '-1.75761780731317267860816633426150642527446699394861016e+9444529', 54, 5);
    T('-7.06929696502991969033036E+99553288372', '2.345579108043856922944E+993956', '-1.658159526973202383282e+99554282329', 22, 2);
    T('2.187647E+62767410577364', '-7.3810632857761809050916856E+41', '-1.614717e+62767410577406', 7, 3);
    T('-2.7279634200345963102385756953108405488869E+915584037664883', '-1E-79427322', '2.7279634200345963102385756953108405488869e+915583958237561', 54, 1);
    T('-7.2706721091883188E-77132484011711', '-9E+98502114', '6.54360489826948692e-77132385509596', 37, 4);
    T('-9.9053264105421086470449358E-44296', '2.26929950719508192111055059985795887313325E+70547290342', '-2.2478152342049636860875138800517200361e+70547246047', 38, 3);
    T('8.7508453411E-84', '8.4875617991702838060218919316371942259354696886289385549E+321756', '7.4274e+321673', 5, 2);
    T('-4.2152032505117E-7', '-1.81313544631982677919966493720508847E+860830', '7.642734426945315787001315405134403383649371270099e+860823', 73, 2);
    T('-3.6509388863192720828268862781E-82462024061', '1.693845171992753087751681526757442153014123E-75364279164', '-6.1841252058324978343175761e-157826303225', 27, 6);
    T('-8.64617721263996450060207916295648E+75738972905878', '6.860700245312E+4531646936457', '-5.931883012377002883e+80270619842336', 19, 2);
    T('7.8752193702524E+41434935', '-5.370E+9197912', '-4.2289928018255388e+50632848', 20, 0);
    T('6.06377888169461980903E+5243', '3.6446E+31014110303111', '2.2100048512224211355990738e+31014110308355', 54, 1);
    T('-4.55179793116296408767663415710613020608647042231583369675151E+474598189517148', '-3.765829650022450859842267925766E+4833128346731', '1.714129561008434092064073333577411489214456157114769e+479431317863880', 52, 0);
    T('-3.00646292366316233611343022998139473018E+62014901716', '-8.090660880021377329029943792E+3607829423103829', '2.43242719637162439570178154112013e+3607891438005546', 33, 4);
    T('-5.0177683E-9444024493785', '6.45926869103988334816E+26', '-3.2411113679082420700095111328e-9444024493758', 80, 6);
    T('-2.0406580892466286876819872768955270297976696450536112371E+628206599', '3.8670971924061506327680390775762933197197961E+43209563498', '-7.89142316758653776781239378934737074913173628e+43837770097', 45, 6);
    T('-6.2732259831460260829573311E+775095109325', '7.29077801936420405826E+55', '-4.573669810842544579426984116038996231e+775095109381', 37, 6);
    T('3.0846070371751601202940945297390571356392649343694E+9202', '4.59855490882444590E+608972', '1.4184734832596262651774e+618175', 23, 3);
    T('8.79645742027984454295721127996906840562157980701749E-63836', '362640496.6753349572070365', '3.18995168787371847935910390442323879858832533572623104247973711511350656838e-63827', 75, 3);
    T('-65538618.5534003820373251526467196481', '-5.28743912881003575982803977E-88004527414303', '3.465314561874045621122851116904082596244061166217010365672e-88004527414295', 59, 6);
    T('6.442908753619984765380819971369250600222E+9359996161001', '-3.32308051853E+24608005496669', '-2.141030456182097498871239562337430903868323749311366e+33968001657671', 66, 2);
    T('6.5063841284E+6201398753', '2.5021304346291160796719437535875034298E-358', '1.627982174705747460129876471711925e+6201398396', 34, 4);
    T('6.258119387875212232178817621272296867605289252334903611893E+570062489016322', '-9.49201065622116667176608919309297482563121253719922085131E+54994538884825', '-5.95e+625057027901148', 3, 3);
    T('-2.3781746534641200978682722491194256826669608014913982E+542878121930', '4.528762578024333247906935331920E-8461512', '-1.077018837461429387833226139401025407e+542869660419', 37, 5);
    T('-3.1225254872862709106217093697241455857581E-32812814718', '5.9050748973393453959004894925024334014E+962026302220', '-1.843874687127646565474611513400200522729e+929213487503', 40, 5);
    T('-3.75139211169750805631181163348601531533760045E-66086', '4.9805724491823662E-7394834419', '-1.86840801976007e-7394900504', 15, 0);
    T('6.932507428E+4970256211901', '-4.41185173880911932288678372013143220E+797', '-3.05851949505289355800509585428406268627783816e+4970256212699', 46, 5);
    T('-7.83195831365441E+2077528598', '1.8236027969442786781378279580594123E+5196', '-1.42823810863311782973785184456474055049131099032e+2077533795', 48, 6);
    T('8.338655965239364846824E+1793442615', '3.0928180465931777687848411321E+121517222747', '2.57899456536241616479603851994316035627e+123310665363', 39, 0);
    T('-8.77995357160549895770262648859956117557985470346728E+59733029420307', '-5.5002838065864798924434766458E+8078475743', '4.82922364524829e+59741107896051', 15, 4);
    T('-9.0490981041526726642786561897E+239380293', '7.074359385236191457151140639325096615700541319861112676970E+30870921980581', '-6.40165721010354870024961096505e+30871161360875', 30, 1);
    T('3048399343927.9028478156455416715304', '0.0006177780572699948125496507797651402342231720170234', '1.88323422447490657884860874431e+9', 31, 3);
    T('-5.5728203595300178587028E+293', '-8.680462497875687227026256999432842399472E+3135', '4.8374658138298424175562210371859045004766928213069e+3429', 50, 6);
    T('4.4761150115401983559531157E+6423468835905844', '-7.3318695758814706672038457632E+85659', '-3.281829147125791830118115755011751235815582003328440224e+6423468835991504', 76, 0);
    T('-9E+4650', '-2.17424418045412496615471917568189199E-7168644678', '1.956819762408712469539247258113702791e-7168640027', 62, 1);
    T('2.701130576551254026372086047896722622103610277716E+54279735126', '-8.091359348041307273397817705740836E+458963897575', '-2.1855818140858195206617224587339264748334357018e+513243632702', 47, 5);
    T('8.144E+15875344702072', '-6.12E+7370713097', '-4.984128e+15882715415170', 73, 2);
    T('5.134261E+815531117', '3.184485049684770384348144128465E+7327224', '1.6349977395679578878313686821156839365e+822858342', 59, 6);
    T('4.775257309248887981689E-801020910419', '-7.90472557536550189E+36692253', '-3.78e-800984218165', 3, 3);
    T('8506781.647411687500', '7.71954940347528794531488435468903292992699611881346007E+625226135050', '6.566852119177141950628823047038292517325264887771e+625226135057', 49, 2);
    T('-6.565485926100955166288469065254457268891549E-5587', '-5.3215284520142750974973451562E-26367463076733', '3.49384201570455252934930179605104392062595348345657e-26367463082319', 51, 1);
    T('2.306408374568337520336851538207691045898756408181071E-28720', '4.86075821146544996508165706408E+384390572206397', '1.1210893445675727880003604725616764190609738635040269997e+384390572177678', 56, 3);
    T('-4.4498857794853211653594563458589160833316956124E+419', '-815904143420346748394.3', '3.630680245229353e+440', 17, 0);
    T('-3.518205289E+18', '2.112786675384426691703164504947191096154090368067E+8085874579', '-7.433217255866216094982845779342274380183028291917276106363e+8085874597', 64, 2);
    T('-8.9353646437233178116449419245202647459708274100892776E-1251', '-2.581313146728281E-9415202857', '2.30649742256540629520305020933532890819063902556e-9415204107', 48, 5);
    T('9.72322292665883018687201E+59323543395440', '6.1648614763075328191595947672614257E+35', '5.99423224461092057722e+59323543395476', 21, 5);
    T('-9E-113', '5.49E+217', '-4.941e+105', 13, 6);
    T('6.9321670503510258E-5324901', '-4.735000951528160280285673952087E+867090111470', '-3.282381757956426733912604724261628e+867084786570', 34, 2);
    T('-1.2589863547E+190', '1.489078928E-78', '-1.8e+112', 2, 1);
    T('-3764.9062456823690721489288397708266601084', '354.776436207609844955693796389692502872800', '-1.33570002049896288939246369459822044723537079677770937030843420662432349616e+6', 76, 0);
    T('-6.221563579930871666289753487745075665610680E-53783', '4.2282955060E-760', '-2.630660932531498e-54542', 16, 4);
    T('4.0387798020030482923204766394442694789707312620735317492E+1224', '2.26861610832012808360092657515E+4236', '9.162440916782092e+5460', 16, 1);
    T('9.14688285901406151975899819602174801685401651555892430840E+5201267374', '-4.5815209E+51772495634108', '-4.190663498842467624666161319813593539376e+51777696901483', 40, 3);
    T('3.971708422047316554137829180581709269070527996102947048705E+99476366832', '9.39001086642898418E+5610324301', '3.729438524131182e+105086691134', 16, 2);
    T('-1E+5809648698', '72841255.065465242811748839574860144614977272883', '-7.284125506546524e+5809648705', 16, 5);
    T('3.517020610149807785032146599662839E-142415253529153', '-1.93E+6148447', '-6.7878497775891290251120429373e-142415247380706', 29, 1);
    T('-0.0000010831229737002849', '-7.19928240152E+999249541872', '7.797708163242470875580193048e+999249541866', 66, 0);
    T('-2.77599E+580567406274929', '-3.04293742417447227665667E+57535938694', '8.4471638601340932952761493533e+580624942213623', 32, 2);
    T('-9.91198742E-339971', '7.431993796952334531103738910414790520724614298966973E+8974585978199409', '-7.3665829e+8974585977859439', 9, 5);
    T('5.7511715350114080372027458246611E+8969492983100', '8.48174602056480036964E-132539553503', '4.8779976280668564582977890792419880487213555907729004e+8836953429598', 63, 2);
    T('-1.66E+631252532911', '-2E-36017428209831', '3.32e-35386175676920', 35, 0);
    T('3.7716746289874060882782557323528302966228204979405E+10087397731875', '3.719300768876E+76', '1.40279923475429e+10087397731952', 15, 1);
    T('-8.9447411159109467983586E+17659', '-4.2653722324756719263604872625620525155222567658485552E+87', '3.81526503824900080959492901656549449927246635387500966e+17747', 54, 1);
    T('-5592.8870240598477360252664643672254416968757204951206', '-4.13158667462132422121722615344308498188653664337E+542097475846745', '2.31074975012681804592109751383215e+542097475846749', 35, 6);
    T('4.1438563894610631377520327763526365674908624825924E-8497441407506885', '8.8187878986200653281270E+272446233', '3.654379058099866001326387319e-8497441135060651', 28, 4);
    T('5.4398095713220114E-164836386186', '-19918.264198', '-1.0835157e-164836386181', 8, 0);
    T('7.83821221618128157305E+98502605311519', '-5.53268650816044E+71836549428465', '-4.3366370976564518609326876224980142e+170339154739985', 59, 6);
    T('-9.86954734697423414098015866143935561687140244451573575450E+79658384935046', '2.6006555488221919457754258374767994148187601310170E-541309202', '-2.566729307227188536892250123109481403258587e+79657843625845', 43, 6);
    T('4.768247004251507839E+235550002', '-2.936046626458762536716910966E+1865465110588244', '-1.399979553095474033747173429799e+1865465346138247', 31, 6);
    T('6.2850761928625125289257059628886348675611658390458104787E+164008', '4.198E+7299952', '2.63847498576368275964301136322064891740217741923143123895826e+7463961', 60, 1);
    T('-6.9393231342299E-22706856995', '9.385652397443985438334473316936371E+48356232', '-6.51300748114233721e-22658500762', 18, 1);
    T('-8.0269183E-495', '1.5252277322E-5387386514531', '-1.224287839526367926e-5387386515025', 66, 4);
    T('-8.16479458216470749742734E+819919', '2.3925735226310690160771797766986234351343E-1549215', '-1.9534871335008881484658507e-729295', 26, 5);
    T('7.77544432826805946128689430321796E+8705329515', '-1.88098239893369243509272574217E-383813968', '-1.46254739253610272210865382296133790172808030656928303602733732e+8321515548', 74, 0);
    T('-9.3365297034656897817468136103820329212278227062014632E+78', '9.4821721718515445824181615768578754828197529362216258E+8380915861', '-8.8530582135867717189371152524009416298040911642693156502644693818931252193788099e+8380915940', 80, 3);
    T('-7.7889830512143348353157424063415016146296370996320652170381E+8807659699459', '-302750222708185664551054543721109227114.1584363230083', '2.35811635342543e+8807659699498', 15, 0);
    T('-4.1440806436562922996492743359901323350843447290E+12100209', '8.1986469979207598414038742E+804533764', '-3.397585432825419099889158e+816633974', 26, 0);
    T('-9.828642707788111898032592035336909304135611730543058', '-6.9272255820573691152837568987189776696873783724853591749E+297407211', '6.808522520229141991211589977744523587034331707607552048418039905504483e+297407212', 70, 3);
    T('3.05992377263468338397724362351227631133E+68', '4.146683101549304649E+2721713124130275', '1.2688534200013238188717780863640114478835625e+2721713124130344', 44, 4);
    T('-7.1108194161622722689E-31814323', '2.5E+3688991871', '-1.777704854040568067225e+3657177549', 30, 5);
    T('-849017.737546638', '3.64586743083615321E+1115', '-3.095406117523484496e+1121', 19, 1);
    T('-8.2948633537094675512378989512877486471E+7998229939052415', '8.840208120654E-2314721368502769', '-7.33283183791777078006557821394230259607546220672034e+5683508570549647', 56, 6);
    T('-7.6449806998364350735623045373993544171429314700564239570749E+9837682397', '5.01917348500159474268593883844232E-7258091812', '-3.83714844219679705352031511669118723708774360676650971859152090989554502316e+2579590586', 75, 3);
    T('-3.650737587048318169614172690303638636013766E+276019196095', '-7.24109206082453787919260122E+92961099272', '2.643532695773e+368980295368', 13, 4);
    T('-41391185.12572301546150240338568693351635054321436', '1.0711615507133891092036100807438152460E+214202545814', '-4.4336646045134430798138137265742754655273730013918591906e+214202545821', 56, 0);
    T('-5.75431351775654870191608991E+98', '-7.5562624225177100936853912202417224353875352118743627664E-8226900367514', '4.348110300160950489107428552752296946079367462876313045267149030229051e-8226900367415', 70, 2);
    T('2.433129568019154046181243001512564984853794E+733', '-6.4359537406E+8462688', '-1.5659509344657336615967796041142364072170358e+8463422', 44, 3);
    T('-8.7136438034454948E+1237527', '-4.634865683694580554097476985263168E+2089924775432', '4.03865686445274485506801670156050708574023219755264e+2089926012960', 72, 3);
    T('-74686313223809747743109241124240564455109375.004380399502881', '8.3886550096845106075732673857887E+6141971766634', '-6.265177155797781522e+6141971766678', 19, 0);
    T('-4.8977174856E-774014836', '9163614984.029823418', '-4.48807973385890309062189577808e-774014826', 80, 3);
    T('-517337.01135115926439388870781574184', '3.19946260332479864091296368728573052650937220594237202827869E+417302249691614', '-1.65520042113385092530606737935152949877372409719e+417302249691620', 48, 2);
    T('-470.158312824311', '2709390.820364132018288', '-1.273842616884076191588617237e+9', 28, 2);
    T('1.60922990087793120110415199901278488E+6370755332819324', '6.2375226212359287526022541767997538807679885217313771613E-7736118', '1.0037607909495347229733227712717766950732844378284346158148802717746610634e+6370755325083207', 74, 1);
    T('-8514333298.5030351250737705347', '6.53577257E-189', '-5.5647746024193759032203669e-179', 26, 6);
    T('-3.03979E+221625', '-7.690E+37070694552', '2.33759851e+37070916178', 40, 4);
    T('-5.057016225281360E-328809', '-8.92E-667', '4.51085847295097312e-329475', 70, 4);
    T('1.4634423223E+8213283371526', '9.961827450E+98210296736109', '1.4578559897779887135e+106423580107636', 40, 4);
    T('-41501096693.1718243825060832988622072439664287', '-3.8188387812193939055033053459441020707910005683688149220863E-2494523134931599', '1.584859975150205085346208860855801090429484894425656020874e-2494523134931588', 58, 4);
    T('-6.23058766761784E+35323', '1.255597759500308542189741293864169271571608495E+4895492960', '-7.8231119158312130053918219091772255976554759390472337575508e+4895528283', 70, 1);
    T('6.0478930012E+916', '4.26027222342074943090344496358610936008371123271190734056612E+6235373371', '2.57656706e+6235374288', 9, 0);
    T('-2.608648444276081128778038776591579408564313888642651E+558160053180024', '7.043143807759455683017196238038178117791247780566490098E+1796489546', '-1.837308613692441828638205086078206999063461577920068e+558161849669571', 52, 3);
    T('-9.581E+1411005', '7.32554957621888330490764640647382012E-3211605264651839', '-7.018609048975312094432016e-3211605263240833', 26, 4);
    T('4.907731315379358382200143699009785181310420158931E+5974339', '2.949168662E-2099989928', '1.447372739683284238245168240902e-2094015588', 31, 0);
    T('-3.3870602897211030262553835506420959085712421284053931056889E+844627469322', '-3.1098492662243646369411986801688087319949065763655278E+59035530', '1.053324695664685614311190975985476845250577449986400562402254674936047231e+844686504853', 74, 4);
    T('1.5530104253116579454904364350412056216E+11922', '6.016536374433619111956634869071985847936E+538624', '9.343743713762e+550546', 13, 1);
    T('-8.16936E+8411003', '-8.558884604E+8751627', '6.992061e+17162631', 8, 6);
    T('-8.2675552643167245454972964224920012460588407017417750460E-2058634288194962', '-8.94428389907456113027585787464120102756663249474E+1071070461738485', '7.394736143533720685383513522343e-987563826456476', 32, 0);
    T('4.2274983808704E+2328681973037526', '3.9347631598950440182791551409787116227541197799046970E+3368', '1.6e+2328681973040895', 2, 3);
    T('5191144', '9.024560905422713536926454395499055485011E+91525646493', '4.6847795196819686840934542176468548886681942584e+91525646500', 72, 4);
    T('-8.7965E+7486082471540', '7.2309831576984955781332E+118565560', '-6.360734334669481635304869e+7486201037101', 25, 1);
    T('4.4365677778427598480E-516487717183', '9.8937548101039650705107808836477826701E+4327878670092965', '4.3894313792384064751675e+4327362182375783', 23, 2);
    T('-2.49136376749987001317232E+614799', '-1.002405211205067345818E-51589784', '2.49735602354935949829490340222077636066535776e-50974985', 75, 4);
    T('-4.97780535778044117929271095891838E+791304368877065', '1.2986981E-813755860325858', '-6.464666360319279176709203066196478161078e-22451491448793', 45, 2);
    T('4.7977546557866027738E-14381976410', '-9.72855564029466210364030060158411112046761E+6477', '-4.6e-14381969932', 2, 1);
    T('5.910894266883498599421923272948289277E+1941373', '6.7090432431795855E+3493173559805638', '3.9656445242383685849017e+3493173561747012', 24, 5);
    T('7.7342456350139946674036546222896963372322021155245E-61008911572', '-6.526707326276682424397E+432210095179320', '-5.047915764926929094144595285807279680884195e+432149086267749', 43, 4);
    T('-6.99683036375690E+385215783262576', '-4.1939483E+19501', '2.934434480946663236827e+385215783282078', 68, 6);
    T('-5.3311934169456022E+6554260130144', '4.10578147495265801282882774128586112027671748799E+7291575053', '-2.1888715170684815305127168302119901913890561474292475739261817578e+6561551705198', 65, 0);
    T('-2.47283929839303143335631900052E+77379583779504', '8.85869106299111924E+8077734586646305', '-2.19061193928875771278241063266515228892329420048e+8155114170425810', 57, 4);
    T('-7.140107914008330705907569474763827156E+754', '3.960979129829655259027888853912153417553229151604E+5769192221422', '-2.828181843211855273911338045916e+5769192222177', 31, 3);
    T('8.464756830335807E+947851', '-6.27267960E+6479747', '-5.30967074886080777184372e+7427599', 70, 0);
    T('9.173151375088134499870992186369522124809422E+646329797', '5.81388965923654825879808486329114653939768033163095814612E+64495290978633', '5.33316899222364283678694543799539565741016250917659803e+64495937308431', 54, 0);
    T('132072.305944603964229071830976503413110964', '7.6499476098625E+854587793253', '1.01034622118995194648333050081552887452579249729328245e+854587793259', 63, 6);
    T('9.89264113610353489496769937200586739960529057094887527', '5.1341770825092181553780343095751E-22952', '5.0790571406470724108e-22951', 20, 3);
    T('-1.452165E+443046700533940', '5.82698E+3254874542953', '-8.4617364117e+446301575076893', 73, 5);
    T('5.424719264791195E+386411807792', '-3.011465398316442694858784145201365926210707548650837310831E+6331255580248', '-1.6336354361499e+6717667388041', 14, 4);
    T('9.09259492226248225994938756239307488113142759E+42910806597', '-5.84611362913726E-81821914587110', '-5.31563e-81779003780512', 6, 6);
    T('5.97845576201567471817598590E-8872', '9.36978365954678761092172964733862860895339565538178852E+14613912764', '5.601683710825780741933858245487557901448665593e+14613903893', 46, 0);
    T('-6.88548576601162906173364874368E-618655828', '2.7670691343293674740463627665085410447272E+8371', '-1.905261513799498e-618647456', 16, 5);
    T('8.10024E+84', '588202408339465828690766084231801253.986287', '4.76e+120', 3, 1);
    T('5.711255624756479998411466114039555E+468149230', '-7.426350E+86', '-4.241378320891028523620299137599764927425e+468149317', 53, 5);
    T('-7.2577902604E+812365111877398', '1.05718562381445513596488774750116132444120731247E+2581670687599922', '-7.67283152e+3394035799477320', 9, 6);
    T('-789460514415230321660807781538807.6505', '-27757026891.2405578507', '2.191307672819615210285786554647893088935784030254499678035e+43', 58, 3);
    T('-3.8100165079E-565336323', '-8.0382550482598625468804812674788060081752033897433782499979E+1298432496205881', '3.062588442858058747259957127739096630452923e+1298431930869559', 43, 6);
    T('-5.596977259396788064E+87071', '-4.2090380655234835990513375E+38374811', '2.35578903366703856996711781648372449433e+38461883', 39, 2);
    T('-1.844352855662426397019654150188966E+3494', '8.6431997402241597873712053868746041140144920936034E-23324042395', '-1.59411101229431711064521764861817468628478694529951e-23324038900', 51, 2);
    T('2.2920E-941929349370', '-6.39432E-3135189647', '-1.465578144e-945064539016', 20, 3);
    T('-2.264104861314370620192230432810365E+39888781927', '1.8512606E+72514311466', '-4e+112403093393', 1, 6);
    T('-4.064884581612013438229476E+142', '3.013108532503123183463753184964E+3360902944807629', '-1.2247938416495545675541251752803647670845795772104798864e+3360902944807772', 71, 1);
    T('-2.892611E+5429109271', '3.94723048326879771098187083026507E+446', '-1.1417802315438640215561e+5429109718', 24, 6);
    T('2.870403601148527311E+278', '-9.9763245197248827267484053929975595701783916067226837210761E+5787', '-2.8636077827644655562739425152262303601011893233e+6066', 47, 5);
    T('1.68235793435918174180880763829E-464503002839', '8.02021897E+549944432', '1.34928790194775241993326411336943563613e-463953058406', 46, 3);
    T('8.7999031413561864186332E+3655084', '-5.359984483174042429198584156903433097460865239239415860294E+23619127764610', '-4.716734429110367129936317645326942371571675443839913640206400823263090640963e+23619131419695', 76, 5);
    T('2.11384574718235069221872779782707919262589822724E+91082566', '-2.823621287005484667792732177578277E+94', '-5.96869984919009942693635999405565722383136e+91082660', 42, 5);
    T('1.477539390676409347030250301090708166968041355805941E+836785', '7.87122954740087043511790456355755124877203402957121832605E+4795990', '1.163005170934083142665578e+5632776', 25, 2);
    T('-9.139242044344E+75655', '-6.75109492E+8206878', '6.169989053822119313248e+8282534', 34, 6);
    T('-1.7255928545648340207E+25103022', '-3.21423563152004067E+46469421449', '5.546462038638668973170040404263621869e+46494524471', 48, 3);
    T('-1.905990158934987081866944398197249054887393444485462007038E-32807339', '3.26170864710002919041810265004687357896761329555812499E+6276729976070326', '-6.2167845826858063283018470039417724467361664255462423375e+6276729943262987', 56, 4);
    T('-7.51113504189110429445956438567148750912856917E-799387575976', '-6.3605703600622276814441383878332091465793729E+174207', '4.7775102917877316841678922133090070896750486077e-799387401768', 47, 0);
    T('-6.7092212887540591921480292671133662429305193105741403E+4453546756608248', '-9.3108249651399713557378115E+242058835', '6.246838507197986687107149542555251446061077446455535136515324e+4453546998667084', 61, 3);
    T('-4.18583943089535530986253864614283365080723964854E+22612216', '-1.72992959716402567101945610466758782160300118477323841269390E+5162487984', '7.24120752048209648179462052109162991445303090533198443734603e+5185100200', 60, 6);
    T('6.0455E+3910670343008', '6.7673768336584519011883581960637395634E+76464255962', '4.0912176647882170968634219e+3987134598971', 26, 5);
    T('-1.958442329277185092517505359658665E+6881339817923', '-168.691186489946573856493567041907916396232439817370936166283', '3.303719601979029853231853644962954765123195378855027278218005687e+6881339817925', 64, 6);
    T('2.0841384016822616245021375E+346', '-7.19653767020E-625324', '-1.49985805176168148318614798391700525e-624977', 60, 6);
    T('-4.533367985166610609931342107870362E-42', '9.43151829718568889605105470271966081034446419E+130174892373328', '-4.275654309997471e+130174892373287', 16, 3);
    T('6.65558935664282563115645838996935998494236807924449E+607880508232364', '8.10089915427998775891852686681E+77601582', '5.391625819046275e+607880585833947', 16, 1);
    T('9.757655042E+2886', '5.81640365152E+452282097878398', '5.675446041656133896384e+452282097881285', 45, 1);
    T('5.2584056484440934811351487924670E+7424933418094605', '-3.9802689557E+54328723', '-2.092986875897955328999097634e+7424933472423329', 28, 0);
    T('3.125603E+5966', '3.268979834042817331703604075788398789869E-4524', '1.021753317622373198e+1443', 19, 5);
    T('6.60842321909339261823110427255119620673663433611E-52863204143', '-7.22555796517603457619004629896075045E-23', '-4.77495450279745140921895265359846566973838e-52863204165', 42, 5);
    T('2.42182989816380892697856063966794854E+7868163', '6.4761929320552661544697600259393642298971997955249859126485E+3268105992074353', '1.56842e+3268105999942517', 6, 3);
    T('2.1767E+7332945', '4.23554475239023722E+14837411935130', '9.219510262527829356774e+14837419268075', 76, 2);
    T('6.251084102778393E+944', '-6.476792039139991010013385', '-4.0486971752869649140839888802962618790305e+945', 55, 0);
    T('-4.510345159287820E+379029', '-3.1455421702241592260029532594262534279373117754E-8355717839', '1.4187480900806e-8355338809', 14, 1);
    T('5.227585717E+15294226837470', '-2.039009261628881451E+19', '-1.0659095692921856827933835367e+15294226837490', 51, 2);
    T('4.1577329870838570026715324688192654E+69104779224', '9.4273719164565908558729436E+172484244', '3.919649519855952710700933079636015623960625280030589428763144e+69277263469', 63, 1);
    T('3.68368746219819214E-7125064142', '6.0453E+369588673', '2.2268995815226730943942e-6755475468', 60, 6);
    T('9.313749961084375197094180327395E+1914617373', '-2.2581620914305270699662617741833914309E+42878205566918', '-2.1031957091183e+42880120184292', 14, 6);
    T('-3.6799882656650734E+1838808393898487', '4.831144810677182247E+85966', '-1.7778556213020743277e+1838808393984454', 20, 1);
    T('9.72787570958568692556715026976781639E+37732672', '-7.1604404853819837283111644706117687674033E-576913', '-6.9655875067681345470461359700343457327470435157152505363372469412990051480087e+37155760', 80, 3);
    T('8.656756550E+8252509895855298', '7.214573006846E-62799', '6.24548021324673053413e+8252509895792500', 29, 1);
    T('2.2715196686870558659168492073166915358489E+516670', '-8.271975043770965175478352088041465850128807E+43121126428712', '-1.8789954010814217259982775e+43121126945383', 26, 1);
    T('-2.5506905749708176149603162928645128492916002471967418857E-197256894526', '7.6603535505228428335131764231053E+73746577827832', '-1.953919160226285415079688746301159085780401801905911400217916877272189547228762e+73549320933307', 79, 3);
    T('-8.2703488097174020091295477781622502E+86114351728', '4.497810655280118E+443941174199', '-3.7198462999230171863461706487943096e+530055525928', 35, 3);
    T('8.5821E+65', '8.89E+958523793143', '7.7e+958523793209', 2, 0);
    T('8.4172632138360778268177535541041300497819E+5744', '9.8382707729173849278508446784881936282084833364721996E-9843725', '8.28113146646361408888249618220901638510821046785193287956448001548927e-9837980', 69, 6);
    T('1.4E-842833', '5.44912469931909484094528E+89493302051772', '7.628774579046732777323392e+89493301208939', 40, 3);
    T('-8.74981039996348853E+22360188', '-1.9E+1020279354745', '1.6624639759931e+1020301714934', 14, 6);
    T('-3.11167306422460830124802194085827495997383E+4457745750022', '2.04533751966537379990E+7946324', '-6.364421667190713732411038172663109108400141234604577671656617e+4457753696346', 74, 1);
    T('1.746803805179174082961375652221727632265009487E-52291129563762', '9.111561565633625634302268658464774375716904612E+16791489842686', '1.59161104e-35499639721075', 9, 6);
    T('-3.41777195618386291994161993240E+44694415', '-2.72446498679056839922359E+47330', '9.311600027457643207916218959880348500093703028285316e+44741745', 75, 5);
    T('8.69778102733316963407442489E+422042969549238', '8.3327124957360E+4198', '7.247610865163460597392375089152077726904e+422042969553437', 47, 3);
    T('7.6712E+505', '-4.7689655369245065335168618159268E+795447529144230', '-3.65836884268552745e+795447529144736', 18, 5);
    T('4.202209086745261890788865585362E+2810717478', '-3.603851696767624120949781204250229389476283411346E+57948897921875', '-1.51441383474392402413e+57951708639354', 21, 2);
    T('-1.121084849318880095161795372468002174678053E+46239102', '-8.150850655484E+1502', '9.1377951789239752e+46240604', 18, 2);
    T('6.4944592473826797300193101727366735439E+5902', '-0.00930', '-6.039847100065892148917958460645106395827e+5900', 64, 1);
    T('5.184760778137319216535487805910979587203E+160182408507', '9.78241731488400520815581997E+3568513', '5.0719493609581979704297083e+160185977021', 28, 3);
    T('2.5347372297019950146680940847174260201934E+22740401675', '-2.875957683715578E+6766', '-7.28979701196139056021809706651609967571324815e+22740408441', 45, 1);
    T('-7.6350970E-8483', '-1.2397791578107142510351396916383221142431536E+24479704', '9.4658341284631109459356419542086782594915593218992e+24471221', 63, 2);
    T('1.6181489602915E+51', '8.5910686956361735767710625E+43633307', '1.390162887763652733640951954009876471875e+43633359', 80, 0);
    T('-5.41660E+4751906419', '-1.969177709582636059139265337926797578941031148306211E+4562166', '1.06662479817253064779337446294142917660919893179154225026e+4756468586', 73, 1);
    T('-2.63712680074191794286385809580606066E+69026515623595', '-2.007360420901999857706625E+84529558063665', '5.29366396470924071241626739396085e+153556073687260', 33, 6);
    T('4.733394801954885212E+633156920554', '-8.04275891873436648009470165852378834E+957128997202282', '-3.81e+957762154122837', 3, 5);
    T('1.709941419024734426016096879467065565058997E+141854642023991', '-8.1439839972436367728776250870898265445609562749E+5501949995340188', '-1.3925735552761513121791046621283338649e+5643804637364180', 40, 4);
    T('-4.35857791955183736882086776792033328925162577649820886993E+65390874', '-5.69465859920056677540630253E+458', '2.4820613229861586817286911827534397e+65391333', 35, 6);
    T('-6.59637649960904611976545482251150817749120328443149808764E-727908865107742', '-4.2365182629037004499978832552625949024E-31700718', '2.79456695095825081578387454695672784094418483053345334131190144933032078899e-727908896808459', 75, 6);
    T('-3.0444345E-73112459', '3.805081055314512151542E-3525688', '-1.158432e-76638146', 7, 2);
    T('-2.286806742708839824026876035139967E-7', '54117207087061540470162.64', '-1.23755594063262943264492485971906580122160111e+16', 45, 3);
    T('-2.77631594020E-37', '-1.57154712E+22244902', '4.363111320031402224e+22244865', 50, 6);
    T('-9.7616313905437608488063E-99138', '-6.80141419395981399359966058829E-136507857527', '6.639289829584801143367451109167e-136507956664', 32, 0);
    T('8.317155822060533468312555330328809289901439040E+404274311297', '-169767262.90274124562961647127990100105655650797710747589691', '-1.41198077904681557207732902697290903213e+404274311306', 39, 1);
    T('5.813770968190606653578193420590661144E+5708', '-7.3712052987955415365688E+680013352', '-4.2854499366710285527983065283792856668453e+680019061', 41, 4);
    T('6.9682404E+848118932910', '5.40E+52944783', '3.762849816e+848171877694', 15, 0);
    T('4.014303686245063258981E+176544710', '9E+5766862392416418', '3.6128733176205569330829e+5766862568961129', 71, 1);
    T('-1.100166283213100477918228E-4719', '-2.3338864058664138147361023559569410713212477124453738116844E+2765', '2.5676631325836341895950565837462493e-1954', 35, 2);
    T('2.254162612078732853859950297E-44471', '4.6068217528033473948531888185893103038656767712384084711272E+44149673800', '1.0384525355680320109747965486736682116473779479271095491809e+44149629330', 59, 5);
    T('4.92033792E+865329428654296', '-3.37354580256256744E-7554969', '-1.65989853372054337475893248e+865329421099328', 70, 6);
    T('5.976319574619895340470477542556372761981E+6465', '-8.4546510791087E+446970238', '-5.05276967406585450929369779463286462992847495467263347e+446976704', 61, 2);
    T('1.00627492771466526093508531622176824554559523667969463646392E-4729894', '5.2E-11386738068', '5.232629624116259356862443644353194876837095230734412109612384e-11391467962', 74, 2);
    T('-1.2157379070591467182296156079552306557565803648E+893914949', '-0.00000275742770592741149', '3e+893914943', 1, 6);
    T('-7.333E+78050261155', '-8.46173151E+21979364762095', '6.204987716e+22057415023251', 10, 1);
    T('-2.2024640708494627903757672404469389667016278112142074E+500700419', '-6.1438175347064668855897553982128201540E+1758859388', '1.35315373780459156987476e+2259559808', 24, 5);
    T('2.500315E+617836', '8.37779284880685974097374018173823464202728214878551269151E-5882', '2.0947121126764523513252757182502834148980443965840649e+611955', 53, 5);
    T('-7.4726541088804427665267094057768202648767084E+3183080', '-7.1349727152165122678323341272133614393957892006807E+82203413587', '5e+82206596668', 1, 4);
    T('-5.2932337184261527147640817E-790080005', '1.96938895955E+24', '-1.042443604538626e-790079980', 16, 4);
    T('-2.26382E-2346895163', '-1.5015538908402576699934641312469328606717560E-400573454', '3.39924772916199211848460396959943155e-2747468617', 36, 6);
    T('6.462072508935584083570232379550E+6291692838954', '-2E+20728405770', '-1.29241450178711681671404647591e+6312421244725', 30, 0);
    T('3.54493162470346543084886434901092317817702979279886896895E-91976940768521', '-9.441081294017657521585223071988035432116845863724093E+44108043875', '-3.34679876505595104828442050816050016426e-91932832724645', 39, 5);
    T('-9.06441397048106039485893633592313399197E+99746831608432', '6.9751129603569146149462173193419395704257816383503392539E-2183603329', '-6.322531136354272361671064476220834624432131427218234735726158893355939463777e+99744648005104', 76, 3);
    T('-3.690044949464137519126816536569234944E+38541106622', '3.04382597682505933221158586720975201E+118', '-1.123185467283105508e+38541106741', 19, 1);
    T('4.1531539186754526066395584177444E+300257723483739', '9.711600365514647135224845012892001622136430E+38738', '4.033377111464711461757509757842771743194936602023795e+300257723522478', 52, 6);
    T('-3.142162879E+61', '-3.081993916867354371784347615355965032582004E-82641', '9.6841268788844128739591421e-82580', 26, 0);
    T('9.342304948025181161080064495951839093017E+745153570505816', '-5.9732814449140265684992657283209397E-411982018401082', '-5.58043e+333171552104735', 6, 3);
    T('9.88107060412723479102152564926726797720E-185915374', '3.7290647272480923E+827035933', '3.684715185729886941188590276835289205726887005157989556e+641120560', 72, 1);
    T('800727227.68675758', '-3.989280393514733711811929065473742906869764489461780398E+12578', '-3.19432542996419005769e+12587', 21, 6);
    T('7.635672626025460628624296778006570862230750757622326E+624489', '-8.9332059869286972961820834377210205207440E+2429964059655812', '-6.821103641703821279835528495634566476526481473975250699348536921e+2429964060280302', 64, 1);
    T('-9.37E+7141583733215', '4.500213456255E+28', '-4.216700008510935e+7141583733244', 77, 4);
    T('-9.90107439386084795114E+2102399374', '3.164439776207152428129660973449768420273044770516224E-7573222', '-3.13313536391193890674562064e+2094826153', 27, 5);
    T('7.4621104980776909146618264005921417856867771365872709489538E-895689548872', '9.58172712712337587038205220001540003613135923369E+7157263979738', '7.14999065850231367681240482306545867550603913427986617541753049362226433826e+6261574430867', 75, 3);
    T('-8.007355309845703027486150652325160119646444791955621124871E+1108312939', '-4.366153259921424E+121766961', '3.4961340489431940419624090314412872662e+1230079901', 38, 1);
    T('5.668E+994321753942492', '-9.654993516047272343653424615601914510534303982228950576511E+942942199700', '-5.472e+995264696142193', 4, 5);
    T('2.5507621838375100614713327852335E+7549543696', '9.9032612672375807976539188897858E+70376', '2.52608643371323589277534067371326555965956758077e+7549614073', 48, 2);
    T('5.23326436463729689075681862E+148', '-4.45736517686E-42285467', '-3e-42285318', 1, 3);
    T('-3.9646853514208005152725275410934608E+533829007510479', '5.927175322265624320935557819814E+86838775277', '-2.349938517548938330580251290048316304947757385e+533915846285757', 46, 2);
    T('-1.0234751456004545696505358060172277450878203374E+331864072', '-7.851881328253E+313498', '8.0362053856712297955e+332177570', 20, 1);
    T('1.5604775326912896149524886942589379076618081302E+859822550272', '-2.87118267059575295962860931180E+93662818159057', '-4.48041604971724831093734414186555e+94522640709329', 33, 3);
    T('-48.683637000854124521', '7E+345082872652', '-3.40785459005978871647e+345082872654', 26, 1);
    T('5.40658677910811182895151352E+571', '-6.22041843577394662442017389711591381294E-575953', '-3.36312320753757812661770249390644622e-575381', 36, 6);
    T('1.4176174312082633746673E+374838316752165', '-2.50407398192256540473043881094003E+449', '-3.5498189258085e+374838316752614', 14, 1);
    T('-725695.56945606748407046376769168883936', '7.164E+812979195', '-5.19888305958326745588080243174325884517504e+812979201', 63, 2);
    T('-8.374617774339108589408687751379E-881703938', '-5.138156504244582243974593174198036241389713245681E-2928342801422137', '4.30300967877827779079098868246597321325229807404011778650826050105079796736e-2928343683126074', 75, 2);
    T('-37.46097973228360155', '4.7663741426729330186363043E+20187187006253', '-1.78553045155151371210870138608815882385751665e+20187187006255', 53, 6);
    T('-5.73348192975298306049653940368679631091503568E-59084489', '-602.9135114843346925983618868980972372473081', '3.456793723299e-59084486', 13, 5);
    T('8.158303610904E+89013403728449', '-9.982256497167877844376071747213342178E-480205994', '-8.1438279225814612467252085904225585e+89012923522456', 35, 1);
    T('4.77352204527812689438951486396359681584561245858E+33199493', '9.1300631668103931090156349136959E-6440757101', '4.358255780155123995516729534492776922694e-6407557607', 40, 4);
    T('3.047531240988479797970517436357346634176420432920E-57029', '9313.90550649867277', '2.83844180066693357184926327351006937079279468014594239035e-57025', 57, 6);
    T('-4.79644053848504333036E+558323625972', '7.00478144540830E+18643354', '-3.35981e+558342269327', 6, 0);
    T('-6.69445732450021102907940515631E+671646', '1.68743601137868187864127316715355778226908942265031E+57831576911563', '-1.12964683659994383436935276139612626741558e+57831577583210', 42, 3);
    T('9.537E-2882582', '-2.944295674575316619605347267354524508569736E-681302', '-2.8079747848424794601e-3563883', 20, 4);
    T('-3.8385221344620398183328805828730957316501170E+5745', '-3.40180033131622932058288617525421E+34926311', '1.3057885868777646808e+34932057', 20, 2);
    T('4.8640405751246298456525925214841679869905107614812E+9011', '-4E+750505', '-1.94561623004985193826e+759517', 21, 2);
    T('-2.17507028976324107199833230263429043516473289034E+701533507607', '-8.1293947E-7841201101433', '1.7682004885728756225525561029873996701888873185645677198e-7139667593825', 74, 1);
    T('7.38349639669704866431770609E-3229911089453', '-7.8423477336407252596154733038970196502798650963098E+195096', '-5.7903946232981560926046268518239232450188314797659432266707360765583259986682e-3229910894356', 80, 4);
    T('-7.208791103514442180780599761424606661043884775536E-441978389234934', '-6.1686544467577329728107516942917E+8382', '4.44685412964419e-441978389226551', 15, 5);
    T('-8.44612492008038061745100414599E+8575504868890', '-2.9786066339951102443815996091260757285411312133E-988897578781294', '2.51576837185028420336029025257335345902701399108758496669937725575428990297e-980322073912403', 75, 6);
    T('-1.2768936702901319842E+783', '5.6789864936484971511206220038355537324453502969E-93802', '-7.251461907402916836926194818963007597415330629944316022704e-93019', 58, 5);
    T('4.2307498495753733811154793907819780601632566878189552602E+288501', '-8.93523518390980092206604144210660338778684007000845E+822224389660', '-3.780274491024697395947914007031791e+822224678162', 34, 3);
    T('-9.87751709872764714984920544750153E+522', '9.107974385456277125693568368199864375823183814140393E-6615387259', '-8.99641727271178114429413452440248300332501781022143120167152976e-6615386736', 63, 2);
    T('4.009E-56811', '3.8185118609929942827391652902453339843847499888233024E-42624512', '1.530841405072091407951e-42681322', 22, 0);
    T('8.46957859288437759621314238216527258619181381060230948167E+723803070978985', '-3.718919161257521963943499209041558975528E+50760971', '-3.149767811685423261312957969896737698218903334486e+723803121739957', 49, 0);
    T('4.7059946068347763580801623360156E-45', '-6.8541130370613763981651E+265239', '-3.2255419e+265195', 8, 5);
    T('4.04295709636034E-5005119717393', '6.9124319090939095377437365377099600758938503E+62778', '2.7946665639978874210126018827519532702710147626894318e-5005119654614', 53, 3);
    T('9.52214735325617580603063769E-387779806042364', '-7.0194079732993399254096317351201200531E+9885755728', '-6.68398370543776068428043855974081140444152831258344276495e-387769920286635', 57, 2);
    T('-5.661724872564912857493460500418726547115739071191239979062E+18392107', '-1.62798855E+5158159934604', '9.2e+5158178326711', 2, 1);
    T('8.1E+6443355694125', '-2.7619746587628192982E+5167440', '-2.2371994735978836316e+6443360861566', 20, 3);
    T('-4.093811939163056264777597823874383E+53027', '-5.6E+79', '2.292534686e+53107', 10, 2);
    T('796559967196.570320354353682', '2.71306712502665589396785189603677725653063593582E+241', '2.161120660113326366928742688241439303339459842e+253', 46, 5);
    T('-9.505411053189016031848152916806E+8699457407650193', '-3.25383282050174225605711892746521594472904909687888347233498E+232890939065267', '3.092901845722645241451101699979591777390346072335326023855672153e+8932348346715461', 64, 2);
    T('1.15826263754001250326556985106598410380E+2075979398117638', '-0.0009744796945690618931964', '-1.12870342126074742627558167e+2075979398117635', 27, 0);
    T('4.3641486270353736701239158869E+51020586', '6.2320101481808850319855781266531212710202E+58112860448452', '2.71974185318541250311753586574e+58112911469039', 30, 2);
    T('-1.347725483E-7410', '4.6858735652721296102595646362E+22187110', '-6.3152713e+22179700', 8, 3);
    T('-2.411075895168026230229226730905136709992E+46703188695241', '6.16E+9992704655719', '-1.4852227514235041578212036662375642133551e+56695893350961', 41, 5);
    T('4.50061192304534020996038000083802044E+33772730', '6.733968370630276783017745557347797474057460531E-186683430129', '3.0306978338268826254230684336478758157587510781475941849e-186649657398', 56, 3);
    T('-5.2951531291588831332371496370694636945610313045E+26358280', '-14047054.6631447511190367305963', '7.44e+26358287', 3, 0);
    T('-4.7764161725063308432066020261908891636116771069786781E+75951132417', '3.7257E+7414284629', '-1.779549373390683682253483716897939575686802539747046e+83365417047', 53, 2);
    T('1.63815670E-9', '7.1633030825156209356E-579', '1.173461293875361729031e-587', 22, 5);
    T('-4.06838182659375484446503527253789E+101060631004', '-7.0925181816807526334580215360E+311669', '2.885507207513575717779154441425831031646544776358635e+101060942674', 52, 5);
    T('-6E+151', '-2.3551482549E+849369658456960', '1.41308895294e+849369658457112', 55, 4);
    T('-5759105490707051.527736421733747735524269183584931423', '-3.73673909839046451117068453588452536930E+13581205748016', '2.152028e+13581205748032', 7, 2);
    T('4.37221531266730398707332819423122E-6875346936', '8.926371729464646441511219240070427533E+81289870', '3.902801916212585218e-6794057065', 19, 6);
    T('-8670198254954235032749260685330103219821173659139383.75', '1.40673928668877724857093058735E+1129860250933', '-1.219670850862460185125774334745333e+1129860250985', 34, 5);
    T('-6.69980803164078115611346663747828170499888997055889001049E+869921133358938', '-9.30141991064146177723882461682474785933793818959466118E+8098893915961', '6.23177278229791425805618635929858781079921493160120954e+878020027274900', 54, 6);
    T('7.20268430986692476842376910371221E+736536970357793', '1.2947739803311767E+45139661', '9.325848232955312673656127432969338969625167457507e+736537015497454', 80, 3);
    T('-2.98175308534030701965462192912885972776693E+54153365', '8.053134994792516E+56', '-2.401246011738458188686604040929601056178266485495935629588e+54153422', 72, 5);
    T('5.5424657370077942E+7415531724818', '-8.899631999E-6385', '-4.93259054264356837747266058e+7415531718434', 54, 3);
    T('-9.122260561995316894926E+2105343', '6.1780316614451032778527782317E+476667070076', '-5.63576145759590691881522806918360418378381289823542e+476669175420', 55, 6);
    T('-9.520049E+920527169180662', '-7.6611138441E-8790', '7.29341791904103609e+920527169171873', 61, 3);
    T('-7.772367117020850190396E+10464', '-8.1897416979291676682697307603546082871970646203537554584655E+51662245622', '6.36536790698791674537351031130287959650632085854256760793432027371461299e+51662256087', 72, 1);
    T('-1.61376278800377315553270638906677284066418054212620653800E-8448513141564', '-5.93217E+423886054279507', '9.573115198112343000056454860030237842202831887e+415437541137943', 46, 6);
    T('6.37466095482911344092990737228336090056188342831619694952E+40034', '4.454971572585800040342779319260899850930588357244966E+656892005', '2.839893333863635e+656932040', 16, 6);
    T('-3.4536819683710409521966E+46206626647576', '-4.927329411954330375429626911826749588269E+19311', '1.7017428e+46206626666888', 8, 1);
    T('1970332538.531690184456012934132027182485811478', '-3.44051572E+201339', '-6.77896007244578579779061214840456407680974306701543416e+201348', 73, 4);
    T('2.73083787728867670191795376016109673887534915503082678183968E+40606604684966', '2.894328062371377359167073643303044E+6648894', '7.903940702023300812387628657244370047080084265152437307471551529289962837e+40606611333860', 73, 4);
    T('1.5433339737E+183149104', '6.03906987695266185315571144444014749494E+1258239405661', '9.32030171064932166462321002839838e+1258422554765', 34, 5);
    T('7.8378567433025637356546483E-18030', '-73021564812366.670893', '-5.7233256417111331869e-18016', 20, 0);
    T('-7.78974E-16810459335394', '7.3785961659903889128490281516229E+5702', '-5.7477345698061972129976588553822969046e-16810459329691', 51, 0);
    T('-0.00003738733237219404463949774865253025303886435206', '7.49565379983884847154669629578478E+330828377709254', '-2.802424999614742792828501785175868307428406604084715161412290245186175839096e+330828377709250', 76, 6);
    T('-6.09230918352847624441577E+614962896889', '4138.920005668502187063667747989520407413806043212564', '-2.52155803604239488286365543424194706404899332738977993494e+614962896893', 57, 4);
    T('-9.955021077852727255321676406448460761E+977461032', '2.92899772555142136638949E+519211025', '-2.91582340948470973410684087072414852108497899099756305078e+1496672058', 57, 5);
    T('8.359536329298115658E-86', '-3.46570153308959446624E-338', '-2.897165787236664044537925726021629638592e-423', 69, 5);
    T('-6.097415962240073540924284540289300324030734334911410813E+6170', '-5.04117370340241559285967039485E+424285', '3.073813300755079496665045436651406370765e+430456', 41, 6);
    T('-1.12858877236441378540452497457053349234967297006992297E+952063841', '-4.079733613457856648833425129654988821114757575676E-54256032084', '4.60434155038623627858120911525865908e-53303968243', 36, 4);
    T('867134.8364455834445724762995135338734235', '-4.19679568736E+73', '-3.639187741954443551501e+79', 22, 6);
    T('9.96430400550009126658E-6971683202317880', '1.5356170407851359128832854506324711923110636584E+6569', '1.5301355030409526792120322449607e-6971683202311310', 32, 2);
    T('2.3E+153393180', '-6.742023046407078513129968743E+41948275', '-1.55066530067362805801989281089e+195341456', 70, 4);
    T('-1.34662560911320385196045894184644565529E+4452832', '1.14E+941123183202', '-1.535153194389052391234923193704948047e+941127636034', 37, 2);
    T('1.5827234467963899953924610819820073284008768E+32197893', '6.40628262E+300285497', '1.0139373709478207906224603508527728700647169437e+332483391', 47, 0);
    T('-5.54947688091439368760821145983476673125E+676178776047781', '2.62979E+3676486', '-1.45939588066598633757351984149588712021739375e+676178779724268', 54, 1);
    T('1.5178173768277921068232359705337986202357499241579416E+31913002225067', '-6.62748025521E+21', '-1.00593046959408283723496e+31913002225089', 24, 2);
    T('-4.9538841051610480294384093040432E+569218912', '-1.07790352959954760486405421157124396E-282211', '5.339809162180190134293572081853565768885971110879499792857681579072e+568936701', 67, 0);
    T('9.248849E+1845757113126', '-4.6726146098270023976249785533922273E+6320330', '-4.32163069614838612982713852685631480713777e+1845763433457', 51, 6);
    T('-8.9865212397068799480878491001472E+417778138723', '-7.072852937926329025801039701196939136869013350E+24574960291530', '6.35603431519981623252575767187479447012749980549910067596042149e+24992738430254', 63, 2);
    T('-1950014655951598123282773150.497635990641104362071314', '4.177567063849370960623583958E-70', '-8.14631700072696e-43', 15, 6);
    T('7.6132091580645104474170269E+65033588', '7.062350020105002820E+99948', '5.3767147850520486950452448214453787540515858e+65133537', 78, 4);
    T('-4.88864256180932831434863228360863166008E+2074888038', '7.03752105236235080010890958779162600E+58', '-3.4403924946207762765926252971736710699696109625292549387878404e+2074888097', 62, 3);
    T('-9.40283340319611357446245940E+290695524', '1.19880118771132639141378279239952200E-284865773781941', '-1.13e-284865483086416', 3, 6);
    T('-2.896342747244196494369638719519810602952923E-340788625', '-7.66425040793175082041578294013E+25249250', '2.21982960820765008413105594914660969917696364179435e-315539374', 51, 3);
    T('-2.243916321913970608181681437094007227E-3883835588198', '9.40996061932870608988065905576267776032703122919984091997606E+40246', '-2.111516422227937908910883302260150393546013572575810976755354625241687718115661e-3883835547951', 79, 6);
    T('-9.1711E+56627807332', '209.23302840314118863148032638555357148743763092415847042606', '-1.9e+56627807335', 2, 6);
    T('3.174377825312129807795307E+9373', '5.0071274769557203655463386207359E+99820011203', '1.5894514431359310972279980805164297361426004594412906421e+99820020577', 56, 3);
    T('-7.714E+3562', '7.08187890871462534532882577421432E+32576498', '-5.4629613902e+32580061', 11, 4);
    T('3.6938437938586041239061868975827623992869370E+325925743', '-8.63400087448059154985975854E+63004720546', '-3.189265054636989395205637895781871578257811751598808887787324699619198e+63330646290', 74, 0);
    T('4.3000571269E+36890099', '-7.94353703912E+901376893', '-3.4157663057862080104328e+938266993', 45, 6);
    T('2.01E+799452646582', '6.19909115846280E+599905520', '1.2460173228510228e+800052552103', 60, 5);
    T('9.336749844049205813351444912E+88587977', '-5.20143E+86617229667', '-4.856445074133286059374e+86705817645', 22, 4);
    T('9.563172923923661722137003815535720978E+165943053579', '5.593278629706804248104109E+4208', '5.34894907475729511858376769543856047653971964752247229e+165943057788', 54, 4);
    T('9.405777456267820700998741738E+6936151509', '9.5749655839959E+6933847287', '9.00599954344888846111750085037802501508742e+13869998797', 60, 0);
    T('3.98849458629409472792690683643615225665908801671648969E+284614028446540', '5.7E+9', '2.2734419141876339949183368967686067862956801695283e+284614028446550', 50, 3);
    T('-9.5201869516751289521734927E-40349826067', '-8.377424823251229754657980549684668E-889577', '7.97546504909556809478316479257261216137682124595e-40350715643', 48, 2);
    T('2.1008395E-9247314626756', '5.855838498740579630033052476693561698295530301253082961066E+21996', '1.23021768237749099396688229486106638114663327303193762e-9247314604759', 54, 0);
    T('-7.0631427284833101936178570894182849574E-710239808', '-9.96381038537124945220997362736805E-441', '7.037581487142142927517625258883633114083055191000846625918521826837107e-710240248', 73, 2);
    T('-5.301667963E-71582266218', '-1.79488035213214569E+5776147606117498', '9.51585966031715554712152947e+5776076023851280', 42, 5);
    T('7.15310310553228799762042E-578', '-2.4E+15035932069', '-1.7167447453277491194289e+15035931492', 24, 5);
    T('7.478945723501325E-339093819', '-9.797302684204965618375879767761362492200917E+19094273', '-7.3273495e-319999545', 8, 6);
    T('-2.6089008503E+5041616957409', '7.86325748098269840367765540525186204452199521593513308833662E+21636470', '-2.051445912826359794494308783387197397361172977590570082380507e+5041638593880', 61, 4);
    T('-3.9684E+8668948823029', '-3.02058635205573E-748060485799', '1.198689487949795e+7920888337231', 16, 1);
    T('-6.9168677216866612942119464842972653415012618E+599423545', '9.73888534429202044191196851747369216E+8525', '-6.7362581683140763e+599432071', 17, 5);
    T('9.6010853601097080193770205749622212396321E+19491814', '1.4995929115879770388E+5562354879', '1.4397719549571618067440968454880405e+5581846694', 35, 5);
    T('6.8715776388062507416500698271765664477755221E+17884671422', '5.329964307628101817738414E-4257203566674', '3.66252635519327049471519953952782430252285843683234828761013700759494e-4239318895251', 76, 0);
    T('-2.12496512135650317794826076728E+57486', '4.4395515570463898983955054342439454678632E+610925098167811', '-9.43389221318753455180937e+610925098225297', 25, 2);
    T('9.41701E-5770972582', '-6.1358913E-336434', '-5.7781749731013e-5771309015', 49, 5);
    T('-1.316077E+756425361043590', '-5.9831251677981795925224807948168243142557493143E+733226533925', '7.874253421460324803588208958e+757158587577515', 28, 2);
    T('1.810689656114401038562653677991636652870970952E-904429313', '6.47269E+9946974208890', '1.172003283023512245829410283499968664667140497130088e+9946069779578', 59, 5);
    T('-2.3811931002933770103794557111014439636E+76445748065', '3.54511504678617182691583340515653010562493766841923394745306E+35197624', '-8.44160348e+76480945689', 9, 2);
    T('9.2103619091445994103781226347026173188667E+3853894', '-2.05298717528265141761612909822683E+188963279225', '-1.890875487918569966039616592828643054120356281546769025325601294e+188967133120', 65, 3);
    T('-5.583E+157708902', '2.6087477162674945E-7474963', '-1.45646384999214217935e+150233940', 39, 2);
    T('2.9740317147251460989362875513263963480621618117258609E+75', '-2.7068818856290010909947257879766741957781045746522817348E+8361', '-8.05035257e+8436', 9, 1);
    T('9.06210026137468811827986124968801390516E-4079457433028', '9.5842217565542155324145571460031002E+30754677', '8.685317848514292905208441773387720237584e-4079426678350', 40, 2);
    T('5.9254009328821615044827E+57038226', '-2.063155594752970066371534991139591257103784669975E+73298', '-1.22250240858302995844951750825736975585516857417e+57111525', 48, 4);
    T('1.615559580602718399164240529E+37222985', '-5.086265280462430931361320187460001081896826657822477854E+74740563239970', '-8.21716460333805278881407652119405742937380052499041902678213313207791e+74740600462955', 69, 3);
    T('1.48661682203947340078305938866712E+73', '1.588336E-423551667', '2.36124701665088902350616141715797871232e-423551594', 49, 4);
    T('1.7758345E+5489192', '-7.8283366794820417758976838550204602008920E+933044661243779', '-1.39e+933044666732972', 4, 2);
    T('-7.746418779899839750219735660876984964552266377587826E-5293807055515', '5.388263098189901274374966E+8112432', '-4.17397425e-5293798943082', 9, 4);
    T('-2.5326811265793073125135241196917205586834500E+832103865', '-4.4849095E+12662623', '1.13588456450663e+844766489', 15, 0);
    T('1.326E+63857316', '6.9535081E-536', '9.2203517406e+63856780', 29, 5);
    T('-5.28296096130718057476E+912764668136', '-1.80696304253406E+210', '9.54611521223229e+912764668346', 15, 4);
    T('-8.3553754E+75535885', '-1.88013056669431E+984070339063180', '1.5709196685745697093974e+984070414599066', 62, 0);
    T('-9E+370563238412616', '-7.647185E+5275390497', '6.8824665e+370568513803114', 30, 2);
    T('-1.7623484856556643E-54', '5.7912870095416665808E+8657270640', '-1.020626589126307678625824022753362544e+8657270587', 54, 4);
    T('8.425973927811117070425704152996741E+6396479', '-6.4185542958774417362235180081821232E+8258074109138762', '-5.40825e+8258074115535242', 6, 1);
    T('-8.04721945741192725E-3576267221', '2.222386203991095014634426086067697865426814567238930870282E+8138551611585', '-1.7884029502641e+8134975344365', 15, 5);
    T('6.882607598E-30070763', '5.43107051071740413810012E+73806285783', '3.737992716233734615172452719671176e+73776215021', 38, 4);
    T('-2.15148136713634822011926706750147082403444146218E+51153906161755', '-6.8646469960712024903534181007076085943603450342396E+6172412', '1.47691601040156967629407580579063981648555339e+51153912334168', 45, 5);
    T('-4.70643E+6271615807571', '-5.129796409308855682391105004E+600126', '2.4143027714663e+6271616407698', 14, 4);
    T('7.047582755629721821565170101996455123175348452889268E+527808241223', '-6.007312184580189096036489286028364E+556573239934', '-4.23370297597316531594502787811199355997522565118441961e+1084381481158', 54, 3);
    T('4.7107500835E+6009904590056', '-5.0088384779227905120956139390229710746790040339888527600783E+6469389', '-2.359539e+6009911059446', 7, 0);
    T('-2.8465905982591209439E-24437826832', '2.33226870879896153105991700336148398740E+3612712', '-6.639014179081063435731265863697169575227077669e-24434214120', 47, 3);
    T('8.5217472739641568495258331284784800E+636684', '8.702612120E+62282259005', '7.41614611099774318442645316369943372071776e+62282895690', 55, 4);
    T('-6.36836E+557210461919', '-0.00687904581030866296197670339902720', '4.38e+557210461917', 3, 4);
    T('-1E+296262122890', '7.5811905628101326816643342694926837175845945E+765', '-7.5811905628101326816643342694926837175845945e+296262123655', 51, 3);
    T('3.23924723397130560391364034012252513464735872632075E+48838068000095', '-9.690490547392164E+3437', '-3.14e+48838068003533', 3, 5);
    T('-5.4345E+365802', '0.0005012489594540656451831134', '-2.7240374701531197487476297723e+365799', 43, 4);
    T('4.166590808024451901801E+5804798779398', '-4.51514E+848364206187661', '-1.881274082094352375989776714e+854169004967060', 74, 4);
    T('-9.336893541809751086833E-9', '-7.30659146811124630146487497880E+107570809128262', '6.8220866691250123443955e+107570809128254', 23, 3);
    T('9.7358852848377780710E+31940', '39485.91659294319734288', '3.8443035431556752829335613826041033198448e+31945', 61, 4);
    T('6.8991762024736793E-3945819', '7.04995272228996541286753795E+6232087', '4.863886605018746099038622e+2286269', 25, 3);
    T('-6.361643518195036899297984E-16', '-8.3072037116166120803507493208433239885527837550941156938895E+2461128456862', '5.284746864633157279583424767122574786e+2461128456847', 37, 5);
    T('-8.0577627278778835317023792684254066582633E-606948', '-4.03E+831910094591326', '3.2472783793347870632760588451754388832801099e+831910093984379', 49, 2);
    T('5.743264502690656649948404685418015E+872498442824376', '-2.9960550843030531474756559626E+572594508', '-1.720713681378359e+872499015418885', 16, 5);
    T('-77650.703748712720159784771321817', '3.472436E-470689338452', '-2.69637099122365003140762392189644936212e-470689338447', 68, 0);
    T('-2.54646940E+741504961', '-6.33270669926379789E+5925318', '1.6126e+747430280', 5, 6);
    T('-1953877.035591533710175262149690871227043231535', '-9.83183403553913168862022055797376498617E+783723', '1.9210194739787144515105702571280371224201e+783730', 41, 3);
    T('-1.8234076609066151791342396236386336190067426250819E+64466891', '7.82674236162237456611E+564', '-1.42713419821245712394539611991996030132456715e+64467456', 45, 6);
    T('7.525693821528845823112601883223115198394327391205585584145E+763003689928', '-2E+141', '-1.505138764305769164622520376644623039678865e+763003690070', 43, 4);
    T('1.636127386273217785700443E+2170', '-5.91517542863967618152357323407114470299286135417555095566E+49780819', '-9.6779805134077940594052264850361774288485775190263516293921710235228438511353e+49782989', 77, 2);
    T('-4.45715644680269376441616964009418E+991432794', '-7.9763114249324276249215697E+3867711', '3.5551667889343550346682357063692385626271151524e+995300506', 49, 4);
    T('-5.6031110042037163781739E-972779785873819', '4.47839630028823115952120851035976346E+6923191', '-2.509295159133019905593107470023976285211711848895951075e-972779778950627', 55, 3);
    T('5.46400643405877E+6071086', '7.491139717566109548501089633589298054220839012647236045E+6989924656', '4.0931635615214419674560149072e+6995995743', 29, 5);
    T('6.24322594537837730559244399347245210080656474E-54339', '-8.87954446220776600411705589226590633723968539374732E+371', '-5.543700236959641480589106179572462771911e-53967', 40, 0);
    T('-2.11711954924814478725804594410370383702822413905E-591854913203', '1E+8255128267877136', '-2.117119549248144787258045944103703837028224139e+8254536412963933', 46, 4);
    T('4.6511889387163550415857600247379210040370297131E+61251', '-4E-90892058701753', '-1.86047557548654201663430400989516840161481188524e-90892058640501', 55, 4);
    T('8.017974805312080316483854394502778064954080469390E+7663229765064424', '7.9379796035690178188852949022679142979896807221878819311E+71427279', '6.37e+7663229836491704', 3, 0);
    T('3.0302918930420642872762194677E+1483', '-3.76292973785606793387412869864E-90891', '-1.1402775478712142818369439133372514120884280097619634513928e-89407', 60, 4);
    T('7.733262E+775', '-7.0006871453923664661500776447113639024167217267E+2305169', '-5.413814787535126268e+2305945', 19, 4);
    T('-5.286583276217963031901796E-69920111', '8.2630632072542625555572027537290302910067521774E+25836901936', '-4.36833718e+25766981826', 9, 5);
    T('6.05498788925E+2197462276875796', '4.291767655240849867492727052351E+7318328440609', '2.598660117595821523954592956444115574012675e+2204780605316406', 65, 6);
    T('-3.97360614617930598177160941003271486543446019001E+80749599234', '-5.717186173522772792106949632919804669743423398267555E-2398539227023', '2.27178461179614381174923270024e-2317789627788', 30, 6);
    T('-5.900038395925581070E+4699066', '82.758257710445256637222510550801894317700741408024761968', '-4.88276898071531283428211069223e+4699068', 30, 5);
    T('-2.091563695860097550463E-357743174', '4.955930243827592849892982598115E+1721836081912', '-1.0365643777204874507e+1721478338739', 20, 5);
    T('-8.69669402284490679659663796943298207587858372E-684132745730', '5.8082096047933122993301304353751210329868463008623234656403E-135938950586188', '-5.0512221753436377390311162083506511034394435389508625650117404e-136623083331917', 63, 3);
    T('2.461298126459406818871963129689415413845430573253861203E+2737831292', '5.183059232225466694E+28042633544', '1.27570539776046727373192767059524561895556252938708486462138490041752729e+30780464837', 72, 5);
    T('-6.7E-7007191095', '-9.370114120606247247118E+74512450790', '6.2779764608061857e+67505259696', 17, 0);
    T('-3.10251244977439335370787234346236100658069311010429890E+48852617', '4.886593265044E+40940', '-1.5160716441782711879479725078606e+48893558', 32, 5);
    T('8.243762E+262974', '-7.0160585059423152698997510597943856225290806E+47066425', '-5.78387165010640328140193115961926840083515785452172e+47329400', 76, 6);
    T('-7.248851572190407964E+7761594004351', '-1.3458821193338650E+788342', '9.75609971671624557221214627090086e+7761594792693', 45, 6);
    T('-21827711.1142470207585597865540902582792175', '3.95862306023061554260406E+65', '-8.64076805689103603535502112314922054204578e+72', 42, 5);
    T('-39274883.74402919255099494287655', '-7.121577704E-7850558', '2.7969913639867034079628846821e-7850550', 29, 4);
    T('-2.2348E+5348', '6.746334923235703361952E+97979705', '-1.507670929e+97985054', 10, 3);
    T('7.96031900166746880E+542', '-8.4415948588E+801898505', '-6.719778795888405324928940544e+801899048', 32, 4);
    T('0.00048866257', '2.636992950872683946134728934366920734462E-95779552', '1.288599752445329479915938208e-95779555', 28, 2);
    T('8.481652070511567705345506594448808756854668372865170E+28579368', '8.8269848384929053661560809312063735773605067071', '7.486741423177756685811922450099966896174847119722650753672532101552236723679e+28579369', 76, 2);
    T('-7.1367135340196554744028918881622477E+76404850', '-5.38834405016539111813010466537255921479740919E+180', '3.84550679087696321897145113874620692701184804991554847397589e+76405031', 62, 0);
    T('9.38968389279123313580606646590621859023391853682102226E+3582', '9.7126967028E+53', '9.119915178584767876688968638364498e+3636', 34, 4);
    T('1.74260895031061524543676665470125137954946860E+1143024850537339', '2.7380629269346271252925384497679755322295173460824522122960E-4921859769', '4.7713729629899613815186597592499410841485e+1143019928677570', 41, 1);
    T('276760.829814683920', '1.929542397745198083852178003649862079542097644E+30440516460', '5.340218e+30440516465', 7, 6);
    T('-7.825024279053212183E-615999', '-5.14945830577212435540091621176501364E+69283720', '4.0294636266639092840137e+68667722', 23, 1);
    T('-4.9463050424690108546612E-6258298440834', '9.12616061023689282484073360976806224E+8104730168752', '-4.514077424479680818150592714254828512838e+1846431727919', 40, 3);
    T('-4.3414948164976724041380E+857465811', '5.311076550821276409653541315E+260223456715', '-2.305801131541290831062212334426842589788155996147e+261080922527', 56, 3);
    T('-8.24E+67134883579990', '6.042266312329465277870111421808821047E+16116', '-4.978827441359479388964971811570468542728e+67134883596107', 50, 5);
    T('-4.3884575957582419876051166841460123547851679090521472701411E+35682907', '7.72570198560400461764906226916602230905230556841095022E+6143', '-3.3903915561288426355946275384421924112398e+35689051', 41, 2);
    T('1.063706532648193586487690070091777716703431281369887E+77577541756', '5.454178736231128E+25', '5.801645551959919591710647793310650421402659313191101291323291242536e+77577541781', 72, 1);
    T('2.04312935857636472E-345', '-2.64296384374271065178034480084982750379639811705703959955E-415798', '-5.399917022806567846920628606833341854887544038889068104155276441723547876e-416143', 73, 3);
    T('-3.8668855134934862314143848132716636516120231494287318E+701', '3.439E+84891707', '-1.32982192809040991498340693728412512978937476108854086602e+84892409', 63, 0);
    T('4.777895089017444000702576165638611E+871922426', '3.29090070241744218025E+73977170603', '1.572e+74849093030', 4, 6);
    T('5.8303316419898361391485E+453291056923', '-3.68838403843202319895678346032964389E+739770671117521', '-2.150450216708048070082227928726635787540616021e+740223962174445', 46, 3);
    T('6.04745971512298159952939379128177736E+34034', '5.553048405100261E+84741', '3.358183652597175170861e+118776', 22, 4);
    T('1.03365610862174852E+6595399373518', '1.1065579E-516', '1.143800332878653936619308e+6595399373002', 38, 2);
    T('7.780781968894571234317629129274482641208227794514587760274E+584', '1.35862001707248721141E+9890481238504', '1.057112613141684302909070510007787943344822098985492e+9890481239089', 52, 2);
    T('9.6112190158500641297672E+118474449', '-5.002234089222312138324909354E+1472', '-4.80775674000669127540802660797988157715348331223888e+118475922', 79, 1);
    T('-4.72982839311019400577536308246101863122315082E+198429', '-5.99255112028479782219701103772869464078647678968364297E+2768', '2.83437e+201198', 6, 4);
    T('2.99361092822838510135216317339164843280138665193E-30499', '5.15391323186441991365523458605262375E+694', '1.54288e-29804', 6, 5);
    T('1.1875026776866706437053962953555343276089285654238781505759E+791', '-5.646718423320673354791476393954324933746427119E+8247293538', '-6.7054932478359546128221762190569445202e+8247294329', 38, 3);
    T('-7.95074751420741546760850401131627009825705236825E+2117498506496', '-7.5330754927E+624', '5.9893581247921326523648532245756237745954022180204570586775e+2117498507121', 62, 1);
    T('1.0650298431259666429472503054048939275982370878118E+65', '-1.9140E+50428153025405', '-2.038467119743100154601037084544966977423e+50428153025470', 41, 1);
    T('5.38511209085285523491556853043626851960687671919050306083E+36434476218891', '8.852652354595617569151179593059372E+8516610557232', '4.76725252308498581361569345684185265577858358686114509650898e+44951086776124', 60, 6);
    T('8.733883438829355277262051156134929397830087957783E-5395800', '6.042169693E+42905977', '5.2771605816289349855002377514614081426e+37510178', 38, 6);
    T('6.4E+582717259947888', '-6.434918103066942790413967829476029E+6847389533450', '-4.11834758596284338586493941086465856e+589564649481339', 47, 5);
    T('-1.1533467992571053297359239789892E+303', '4.595103E-220918956837', '-5.2998e-220918956534', 5, 3);
    T('-8.9277303474533770386244210864094211287445893E+39044227275273', '-8.2E+6958386093377249', '7.320738884911769171672025290855725325570563226e+6997430320652523', 48, 5);
    T('7.0321883903520065289462346536093644901E+41380397599694', '1.0878451049353160114152268963907066262810249046136299192E-43995148', '7.649931717427389535e+41380353604546', 19, 1);
    T('-8.731017139494319724386736545627375487705942076044954E+5043501393591', '6.3757160570356057250E+7210', '-5.56664861705270173223034520589537609708260393e+5043501400802', 45, 3);
    T('5.93821774712982793428810451726058783286736195533336891363E+3304898367', '-3.76277052941799974229393946708153444539047246238734E+935526', '-2.2344150736167064375965794149862184046039695228136644504969344484566739e+3305833894', 71, 5);
    T('7.9503074697711054656E+646', '-1.963436624E+181702', '-1.56099248582093613681256121344e+182349', 34, 1);
    T('2.4194707929385542928845267840102140338685475826750E-38164322297', '-97083177.6889195757380523274154570746', '-2.3488991290400480872205622874375951657e-38164322289', 38, 6);
    T('-5.4004176623472786829375664500948137836549849E+15665362434', '4.938724759033542429824672052418690E+7691664', '-2.667117641815655041815987336438271588403859319805e+15673054099', 49, 1);
    T('4.72419128594114566378903768177122610286506628075947867541E+59871958444964', '-9.159794589625249014606588125163327552342E+20736016', '-4.327262178131845376998337231837162e+59871979180981', 34, 4);
    T('9.526824064446105174261461E+405762284084', '-7.676340636191242125497982934536260655449E+1943627370620', '-7.31311466997522500644431202689519905639431032097912154403604e+2349389654705', 60, 0);
    T('-996317.9112305137848937360196712911540160434194338591686759', '-8.7240771956917E+86381565', '8.6919543690253127981227243e+86381571', 26, 4);
    T('5.52478782361126731576110334680061096E+5482627470567995', '-7.6483486857059284938674553E-9052195567', '-4.225550369e+5482618418372429', 10, 0);
    T('4.124985889E-591218', '-4.5973561540525955013388480657041224514399038E+5006219', '-1.89640292621743e+4415002', 15, 0);
    T('3.5177306173047490E+662', '-5.09207392835E+676743797', '-1.7912544363337e+676744460', 14, 0);
    T('-2.9192849282783764460065913238394538761E+59598662056', '-3.64737762233564537871408918547747899735865E+256832', '1e+59598918889', 2, 1);
    T('-2.7224408635830852977517795056092963176707E+973935', '-2.44522945245401488667076178488E+992515195450387', '6.656992582197703099238365948400139675997154578042578216001810216079016e+992515196424322', 72, 4);
    T('-3.571384371348E+505992550', '7.282297042284116379139494476274368703E+4322987', '-2.600788185e+510315538', 10, 0);
    T('-9.5918738883331410203986797699744E+7885', '2.062811257645786381663134613129460127080338629341E-24671893', '-1.978622543877226579476262409949364e-24664007', 34, 6);
    T('1.530921650953452229662410E+57308001099160', '-9.64909102310495638898952505443963492948905836392655378378E-8724', '-1.47720023592919753084e+57308001090437', 22, 2);
    T('3.7577467E+8257997873', '-9.9926749E-466', '-3.754994112964783e+8257997408', 24, 4);
    T('8.171388526406902376134385385215686088370908130E+7286', '-7.3794524261535586529011965460336787942658772407217E+90901391710', '-6.03003728862367682171573921816207651536642786732962345782479573711519e+90901398997', 70, 0);
    T('4.83074E+307412803220', '9.510583628181211941928770737497223439767614611235952E+93556715', '4.59431567560001077763529899524573371594230066070819627645e+307506359936', 57, 6);
    T('-4.53220765334407720358E+738865879810', '-7.41038652970011361922655270231919997343449329068674528365E+9378134', '3.3585410544144711814614225808370654144734689693629771339993953148420045895467e+738875257945', 80, 6);
    T('-5.058449E-981528591', '-8.5665484130336807530399205063133339277224012987549288E+4246087313689837', '4.3333448253361809e+4246086332161247', 17, 1);
    T('9.45366000123766264003862672935E+88645374', '-9.146E-72750726', '-8.64631743714e+15894649', 12, 0);
    T('-5.3362552769652802232E-774158', '-6.676184861009728347172413395096480194090995E+46826054387024', '3.56258266945588787927e+46826053612867', 21, 2);
    T('7.75415987510403743118044823141106560552506882165133691147E+248398676912874', '-4.452836E-54250470', '-3.452800224161876161890782238096352372664382535152665244752242892e+248398622662405', 66, 6);
    T('5.95969815908273787E+846482088772209', '-1.7445129159331023978128355046617501953788109968E+7928653', '-1.03967704135825694102554353385893952254330784868756415562078e+846482096700863', 60, 2);
    T('-5.742989542342092150445059E+52374', '8.64740585178411669250094231898286700531804187359E+403522', '-4.96619613751839938703485571792415567146929846e+455897', 45, 3);
    T('3.047312416510495715204529494424706225677574E+769018563', '8.3530E+533906054', '2.5454200615112170709103434866929571103084775622e+1302924618', 59, 3);
    T('-5.3678830683470244954088860100141E-50', '-8154266567.3067771220942825220838906', '4.377114944143426151628885121207221386392928886478939529627188885746e-40', 75, 4);
    T('7.36709188177928315906006224368573152E+34800353215381', '2E+71870', '1.4734183763558566319e+34800353287252', 20, 0);
    T('-3.5399656502282619115058501691649523034193661040459E-22460', '3.01908470535292243E-140707108453534', '-1.06874561520788585751564640122326597222255334432772753e-140707108475993', 55, 5);
    T('2.96235936849609038980188784319536188850790267734695054967835E+8920290150573', '8.82853800278734357574412668392471173568311E+353749', '2.61533022626808502126237849094941303726355787381705440594e+8920290504323', 57, 0);
    T('-8.291012244E-44536431', '6.4430464471E-42', '-5.34193769815667982924e-44536472', 47, 5);
    T('6.6165438E+715617315', '2.12266858436240707050305548949433138738E+8162927696', '1.4044729661317861455413154680069683476314537244e+8878545012', 73, 0);
    T('-3E+5337324', '-3.379723597529675851651755247644E+28763', '1.013917079258902755e+5366088', 19, 3);
    T('4.649994129190971454723537E-8', '-1736098.08510465', '-8.07284590343631002745503195286346314705e-2', 52, 1);
    T('-1.203633747209974E+16532', '-1.1995596766191331815172750997393714987963322E+186190', '1.44383050857107190670009484418500676231e+202722', 39, 1);
    T('7.2057012367961E+37421866940344', '9.9950016592963105028448539667406503753082E+6687', '7.20209958182e+37421866947032', 12, 4);
    T('7125.9276507506817019301822731798240678150', '4.07065983678130021E+93259', '2.900722748772012402364363092695349865377433469028692e+93263', 52, 0);
    T('-1.489303794764304032078723850E-548', '-5.95343E+73247534', '8.8664658908636505536984369303055e+73246986', 32, 4);
    T('6E+703150432', '-1.519894796677068622194528873497727171679676656054330599945E+3162750200431', '-9.119368780062411734e+3163453350863', 19, 3);
    T('7.93765095832282884905719499609341E+97270611', '-8.2638272313164427491500858673216744875310799203186601669452E+4148589510754', '-6.55953761420732512228367263042224274583629955536335291457266512683e+4148686781366', 66, 3);
    T('-5.2519693630005678595145595532431739888E+270', '7.723932652440149372385679092194323749519707711407042E-60488888749', '-4.0565857652495377803559770157856661466948845994946507892691339307024617941e-60488888478', 74, 3);
    T('6.77411364984024436071333933634048279536E-320508', '-3.0832782691485781341511761191812532281200592213429E+743794', '-2.088647740929518592565851363185590653455410885140790173012699253e+423287', 64, 1);
    T('1.4E+535270', '3.03613285293879460291E+86', '4.250586e+535356', 7, 0);
    T('-9.7757614892028066094840211140545828912350986222084937E-39', '7.632929493569515E-2502370872701', '-7.46176981930371464324292546799022348470923912455206335496617643895555e-2502370872739', 77, 1);
    T('-6.25125318055172103E+4409', '-2.8643659966256842020313643645295E+9017117', '1.7905877046670508595579627955903188518313511205385e+9021527', 69, 1);
    T('160.288701588586483117025508187533188474034484453584351738', '528662331491557.8179325573579182947543420820384319', '8.4738598693576697551867262759199480807165593382609211801618760566203137e+16', 71, 4);
    T('-2.64645787890396471179449977601695890831929E+358505223232', '4.65093281E+82', '-1.23084977792774563171672329e+358505223315', 27, 2);
    T('-7.13870085178692990336542737051029477574001131626800920656913E+8993963812845', '-5.422854405642057337151262204153796222619783E+21445369879', '3.8712135364673460207e+9015409182725', 20, 3);
    T('6.43285693590691436639628413819321091237617068E-933362439721', '6.1995761468935907687701361939825111441806326608474728465E-48919634064790', '3.98809864162274987571205e-49852996504510', 24, 0);
    T('-5.800105704634862293600698209103847283114019060269858153E-5341308', '428.04951395110332540334249129887141117', '-2.482732427733974470966957e-5341305', 25, 3);
    T('3.958354758536193586606566831156995257886116E-480', '9.215963790E+887', '3.648005412264375349859634889215791210188015699973964e+408', 70, 4);
    T('-3644.514474905073468833343798385877452936060757256293071', '3.77184886580175174799994195495325497938271294130823E+26318247092087', '-1.374655778856876819702689392178905884466787130181686922838889942662983e+26318247092091', 70, 5);
    T('-4.109543567915966578436616402705026527847904616379717251851E-777918010', '-3.0915768147521E+87290', '1.2704969613782625e-777830719', 17, 0);
    T('7.4152556860E-30652818545825', '8.4310130255932125687138308036305096E+187428037672', '6.2518117276770133023161899e-30465390508152', 26, 3);
    T('8.2901261977844883812209469278589139538801856979915546E+442992968', '1.23E+8', '1.0196855223274920708901764721266464163272628408529612158e+442992977', 71, 4);
    T('9.2613795469096412565730804832387259315195081885707077138E+4766370', '8.0961989072052E+91458', '7.49819709669024e+4857829', 15, 3);
    T('4.8988257242422469609974597313806918761210679213868E+579514', '-9.76897948365719908051067624254214085E-17', '-4.78565279941346300453573376446018091358e+579498', 40, 0);
    T('7.93E+3778363872', '-8.9380443075852045090447023435060835089999376448792E+910', '-7.087869135915067175672448958e+3778364783', 28, 1);
    T('-7.065800854233604489099085738607230428041115014624687E+695602229', '3.88136006599110396494846076756484049366389283867216455E+4714042324', '-2.742491727e+5409644554', 10, 4);
    T('-1.2E-19', '-9.8163740203384550207442342512970175873066245299781304155E+437', '1.1779648824406e+419', 14, 6);
    T('5.717969685646540485715E+3419751', '-5.706979232126574083730873725324205716963610367865998E+79483507837179', '-3.263233424591412181838432060042111526401405779170170716849072217495321857e+79483511256931', 74, 4);
    T('-5.2046053643892856414779648309044951241598E+938283361', '9.345151452431349780424429676490395464644046553556126354E+9722819', '-4.86378253803545272e+948006181', 18, 6);
    T('-7.9197455182862061298536E+4397370977102', '2.4154338464938950529955412782849465169307004620E+373692416741', '-1.91296213804868373334462227692984884450862664372518284033395775123632e+4771063393844', 78, 5);
    T('1.151057645551675830830336E+7559946', '-4.03554601290197581109695848736679E+306564320', '-4.6451460921264010924899772515269923196861725103e+314124266', 47, 3);
    T('-78516.440685257824679212071134745431385348', '-3.45154412688115432339E-30093089276', '2.71002959710814160734081684886339104542491092e-30093089271', 46, 0);
    T('-6.712893265177213087453653120190471489E-80643', '7.2691575059592069853408854490E+69841124257', '-4.8797078465265947781453711609407203138291934538258530504049463561e+69841043615', 68, 5);
    T('-7.92287813177494407508979592125133189616179517467187049E+363707038305', '-8.8990721821310773E-212', '7.0506264384893044576979425911878558014491381216457042130114726349778877e+363707038094', 79, 6);
    T('3.994170740742860118578792498124738716505000E+881173005520709', '-2.11819261E+57830255', '-8.46042294611975221343732197225126040748177602805e+881173063350964', 76, 5);
    T('5.295344110581390E-49277', '-5.58383069464861851318935664584397882547979530270857939404E+576762699', '-2.95683049833911538910159773466424293127963828e+576713423', 45, 4);
    T('-9.273510620065027616063239175606925150255724423309613297E+38876', '8.517694129908391301447317455654108411706403599087136465137E+260532095352510', '-7.898892697217101170471110873509536208743469544680637126882636457730119803731e+260532095391387', 78, 4);
    T('-5.040998768384E+216599028', '-7.164607E+8564619169708260', '3.6116775062955385088e+8564619386307289', 65, 3);
    T('2.49E-164195272771', '-1.70205567690375920637419512643531293447325042282339E-995', '-4.2381186354903604238717458648239292068383935528302411e-164195273766', 74, 0);
    T('-1.4409336663212898682171018951E+7402', '95554290623305918234188520.0183219977743957006333720', '-1.376873943205702472426261244574522011809938434e+7428', 46, 4);
    T('-6.34920656715853829721E-4256910054982751', '6.54061E+241046914572611', '-4.15276839652228071721146981e-4015863140410139', 41, 3);
    T('7.6554352519316253331819140132616088599293795825E+1990', '4.48887401483655170262290500E-966976125', '3.43642843746595836572271946046e-966974134', 30, 3);
    T('9.4107268118482588636924018933465E+63719', '-6.56294501357411214253748565665792351447918174231704820494412E+598732', '-6.1762082603928e+662452', 14, 4);
    T('4.209E+8148505316', '20817576.65547104712403134083928147034339965', '8.762118014287763734504791359253570867536912685e+8148505323', 78, 0);
    T('9.7724577540410868079583981033487E-249', '-1.16391523679874816977E+463009', '-1.137431248090049425098e+462761', 22, 0);
    T('-5.294567429764571053549E+155', '1.928751780482971634034242743106657398E+39995818', '-1.0211906357045567283526984696619965571752154546064655005502e+39995974', 71, 5);
    T('-7.0306774088584166360E+6566081373116', '2.29296052077535243100702264514938E+512749', '-1.61210657328195004366438e+6566081885866', 24, 0);
    T('4.17156086E+41', '-7.13282925820510682759307233602484300854784745941752035E+5309375330838356', '-2.9755031354591257494106028564110003082102845898956566290313501e+5309375330838398', 78, 5);
    T('-8.7354053543811292146118217026E+32241', '9.79448378154589526038560160624E+10482040525975', '-8.555878606871514376549e+10482040558217', 22, 0);
    T('4.26933398958766791806071790696785688934008347206013700E+584431395961391', '-9.6194507397603829832E+88495', '-4.106864e+584431396049887', 7, 1);
    T('3.353266326502678398893635222E+76486301978', '-7.409334407828861817817657480E+443531544396122', '-2.4845471571570185461590030751042349866731922938e+443608030698101', 47, 5);
    T('6.571577659402675925325378990601069934492809902409343E-55', '-535741142438593683111135323.55784961814067999794778935096453', '-3.5206645229e-28', 11, 6);
    T('-8.618702046333996117668559830471577244317563403732E+9398506011491', '-9.52738832007486E+9624283162860', '8.21137212104478094655893e+19022789174352', 24, 5);
    T('3.44568E-970666', '-4.69661700207E+94474', '-1.61830392716925576e-876191', 30, 6);
    T('-6.32298510495906877473659727416112E+91218915914', '-5.53795269969540298249028997808756343658257199169135816893215E+2892897897860', '3.501639243214189590580937753097e+2984116813775', 33, 2);
    T('915.456453980378553248627513', '8.20133804189825198261419890057523495552474606406516954553E+219163082098', '7.507967841730555071123352125531245916911124010004535711358e+219163082101', 59, 2);
    T('3.9788468210327697114415493767640E+6349', '-9.01092494327169038978E+342323397', '-3.58530900651e+342329747', 13, 2);
    T('1.50557970055899614529957655723255686145681992E+909082567535', '-8.9866980472475653454774565E-59390352', '-1.353019015498910482595696e+909023177184', 25, 1);
    T('-9.7058801958406573329110216586949974623E+82', '0.00305968392', '-2.96969256e+80', 9, 6);
    T('-1.1E-99978912399440', '-5.9356579212886453462291503862017506363068515001028364389627E+3370606624305', '6.529223713417509880852065424822e-96608305775135', 31, 6);
    T('9.577575491592242453839E+72099698475', '6.44E+8460720287840', '6.167958616585404140272316e+8532819986316', 52, 6);
    T('4.63528262409872652831E+21659067', '-9.91789898510482732512985903350490E+99129357', '-4.597226483e+120788425', 10, 5);
    T('-1.108739845E+2389845', '-6.504382812221173276859122196527711368801139445E+1986170926408048', '7.211669e+1986170928797893', 7, 0);
    T('4E+8974539', '9.37216727458006202393675092129E-82846', '3.748866909832024809e+8891694', 19, 1);
    T('-9.1E-79406373112', '-6.920915E+44085841431', '6.29803265e-35320531680', 75, 3);
    T('-5.69E+1752994', '-14183402.302919141946', '8.070355911e+1753001', 10, 2);
    T('-5.60570570145811174153848562306439220593094855427557249E+78979578', '6.937E+230', '-3.88867804510149211510524747671976887325e+78979809', 39, 5);
    T('6.339236723715516768885719883113892070236066211705460975691E+369702', '2.49766869370290041109572264746352492058384240E+292032', '1.583331310679598897109511775285e+661735', 31, 0);
    T('7.7092969561007241916688815525268583028556951439189847528E+58308', '6.651991146298935169817611697099148192680587693685917489262E+3847566093257', '5.128217509617134800191577398610913132605295969386806881640707731e+3847566151566', 64, 0);
    T('-4.9956263362382330060456549799858688E+289615', '-5.37559118351461077312865245870966404E+81013802293', '2.68544448892156418659520232162191454863263430831714e+81014091909', 51, 6);
    T('6.7739245727126127641381444953E+160', '7.44785275127660608253518699696392005881559048907956451764614E+1980618047886', '5.045119276581784124723086990175722239498667e+1980618048047', 43, 4);
    T('-7.7374832746135594834669146772686567690843869377563696001E+88936142', '4.8031433903399448916115935016304375286000502888077430574E-46427120', '-3.71642416483259909505209495512739597e+42509023', 36, 4);
    T('-2.55345765985714341282997792E+767814', '35567.2096963543908645288547144807320508465533003628', '-9.081936403890138323528352076797702467819187005279040319063698777727785e+767818', 70, 6);
    T('-3.117714817658E-99728920', '2358.86103026473473032474883672335402554', '-7.35425598685237945922216412260959930992662137498532e-99728917', 77, 3);
    T('-3692.1137757092824003639103789689', '2.92872376987654785545E-9974584909069', '-1.081318e-9974584909065', 7, 2);
    T('-6.8952402247024205369083560323472E+612', '8.723E-3518818846433', '-6.01471804800792143434515896701646256e-3518818845820', 67, 1);
    T('-1.9575245606537637331056222786856E+7999050', '-8.8878197068843681683422148497749737481233989309283660942542E+1353458147', '1.7398125366888685960354542156300908826554643397734258275578522778423076632e+1361457198', 74, 2);
    T('5.09996584051333951000041059502109542400214586523993647503467E+3490997170100060', '6.8468144765040698010953E+30497009515', '3.491851994650297899499394155718648e+3491027667109576', 34, 2);
    T('-878.3060868771138167852579085672845277720770957', '-7.1640004234908716118534886414407521425541806E+185310981222', '6.292185178342253657034831244273751166649268180179e+185310981225', 50, 4);
    T('7.01597810628300934697537406516737606E+3361184193', '5.653493534625741709056172E+69668521469047', '3.966478686294674824811305516328052893244680801881866838804232e+69671882653241', 68, 5);
    T('4.3393497177496529986920E-21751251983', '9.69870987052546219619144561018782801364657402438792547E+440569143', '4.208609393920043796022065978750364503125765810653581160171635466e-21310682839', 64, 0);
    T('-1.1980E+7690', '-8.195598933E+5812', '9.818327521734e+13502', 30, 0);
    T('1.31723465892E+737951380', '-2.94751340171481322626599840687735411559986094291387381E+68', '-3.8825668103699e+737951448', 14, 1);
    T('3.54864941190020E+979235258901707', '-3.6312009922721E+1590182402', '-1.28859e+979236849084110', 6, 5);
    T('-5.049048115E+1590429115421', '6.84684704958685E-57734830537', '-3.457006018940979652128775e+1532694284885', 69, 4);
    T('3.1954390355244439756783335204727716448073808047736E+4782085226092454', '-8.59931523288127612053333578639255E+330073654325', '-2.747858757e+4782415299746780', 10, 4);
    T('-8.3884462355271376296677445094933782921549E+3858879572', '-7.9883357564664685E+293396043574810', '6.700972500445797696822492046446330353618181389247845797065e+293399902454383', 67, 6);
    T('1.116972580031231896938026794156752048E+700', '-4.58008191482090357204761644445097731299007520920762798403E-2755024', '-5.1e-2754324', 2, 6);
    T('5.00732229183602409833E+31704', '9.1162720425002980503650935838842548205E+20333865651', '4.564811e+20333897356', 7, 1);
    T('8.0336314036694525285783446986406647131262069541777391891905E+98281723', '1.5457851314474479851161987E+248423345922561', '1.241826797532153074232487346910251628908158540328e+248423444204285', 50, 2);
    T('8.440601919227330303440197204522183659063290663289312024E+5362709244314', '-6.6130988687251080938770820556879078048569591294E+885815164133066', '-5.5818535003401235e+891177873377381', 17, 3);
    T('7.27044617E+524844950103', '-6.2814221101781573387092457435093206934105E+65174262983', '-4.5668741323098102040876028459486142994708113962785e+590019213087', 61, 3);
    T('-2.504099490244576023158798944191573937921533740824E-7', '6.547251024213584429104793514091E+8639076840569', '-1.6394967952236515038117786327438075060568646741891512460681176634817371786e+8639076840563', 75, 6);
    T('7.722761601054331503907734201183055802762349105637209347E+55174', '-9.15932233313421697330637810756E+974324210', '-7.073526280600830057778781770778497357810593037294826825308953516512683335e+974379385', 73, 1);
    T('9.27526179491163113047150330835755638389116923492016145644402E+924', '5.73986986010271255236584041213777096800604466423E-60', '5.32387956211754587020478492741613894915835386194393207e+865', 54, 0);
    T('-9.24486042428046632259764315263721425E+51', '-9.9260E+275696924648194', '9.17644845714079087e+275696924648246', 18, 1);
    T('-0.04985304866878598416339', '-2.8196977933E-639937', '1.4e-639938', 2, 4);
    T('-7.0015834851245271562882876246163125E+178130495444947', '-6.62519275721845788105144087550584E+991161948925', '4.63868401947073856510560369826324597696368877e+179121657393873', 45, 0);
    T('-817153962.539927481965195944191', '5.267203351E-734381587', '-4.304116089773234504304072142614444184041e-734381578', 53, 5);
    T('-6.2479613309E-21984096', '-2.0E+387154663524', '1.24959226618e+387132679429', 31, 6);
    T('-5.0432896485562737032109676162831636621418E-2158368817', '-6.2E+554842356004334', '3.126839582104889695990799922095561470528e+554840197635518', 41, 0);
    T('-2.0581130458157984574677826011625359518451772403380E+672045056842', '7.21490874849077570758061260289487188442934734395331827287844E+7243927569035903', '-1.4849097819639400973730348869606039499415881456211353310378299e+7244599614092746', 62, 4);
    T('6.68862E-1957118659425008', '-5.79374698759751333107450988119636539176157906707E-9', '-3.87521719761844796165e-1957118659425016', 21, 3);
    T('-8.0424308E+42092', '-6.28514110371270456745582453666619243164233763533E+88', '5.055e+42181', 4, 2);
    T('-5.005338502434400743E-157025', '8.795890162094474204341860575781757964650561743912242518837E+567257429272', '-4.4026407691e+567257272248', 11, 1);
    T('-8.5026267081133784205849379351796446723539429E+768', '6.1402261442178756498764650577717182888706277290419E-99830', '-5.22080508076829384139282547801129169800214815177e-99061', 48, 1);
    T('9.1E-616145614', '-1.5020413E+98431340', '-1.366857583e-517714273', 11, 5);
    T('-824375556.5390337543598510375875808752970432632581', '-8.81210641056924632185689E-4355', '7.264485126494209515389380403177567914626661478355244330442e-4346', 58, 6);
    T('6.7877254536362456342112890725186509285760283930E+5484535271', '1.044870613428945809142085402713716873533485071503643E-1955903388070', '7.0922948585281734417812514646022962296992e-1950418852799', 43, 4);
    T('-1.4179661842406478972441372710E+6353997858', '-4.8445125593322400762597829738527435295809409884831E+436208758858', '6.86935498826223180968207e+442562756716', 24, 3);
    T('-5.88013794272099114970103308072E-585970', '-6.448829007E+47797096', '3.792000413018e+47211127', 13, 3);
    T('2783432920.13847960423916141046', '6.2663209497530537101701384608430097788922785924939980946232E-9582287', '1.744188401969607321257307112518164303114982522937689719648919e-9582277', 61, 5);
    T('-8.1033720814379537974095621228224041791883676842117389E-89968951', '-4.8572951057694327E-67243845392327', '3.9360469551397233801099052452195e-67243935361277', 32, 6);
    T('-2.17781866317181816382378667829238E+493409186493', '4.842364913167071E-96409178101', '-1.054579268178362790879116935820804353139192621898e+397000008393', 65, 6);
    T('-7.24470488743851575256540553091471474650679523073790E+5351820951510630', '9E+94307649', '-6.52023439869466417730886497782324327185611570766411e+5351821045818280', 51, 5);
    T('-8.11111025938377867243373090874770E+75265508861494', '-1.30397402942427148806334331437549423723361320310514090429E-2459872774458141', '1.05767e-2384607265596646', 6, 6);
    T('2.0069604056099388044389113987788267453302762043893E+1351160746636502', '-6.065437097014116348150483270768973760951627393852678E-146', '-1.2173092096425020688153426483011429e+1351160746636357', 35, 1);
    T('8.0726797064936602510811777576044286758626780493220E+9057440', '1.97143345025257327860E+550', '1.59147508065567272124736219764674348207224377455566338991935100471092e+9057991', 80, 6);
    T('-1.630760611368328351982042057193410834990E-558472', '5.78932299084E-7955922342353', '-9.4409998999509575995478439645216235515651685214916e-7955922900825', 58, 6);
    T('-3.3945942591772535E-42490013987277', '2.5681962146956816438451997505534649246228903458E+6898292562311273', '-8.7179841268467141080301587641943392196674599028558482561092603e+6855802548323996', 73, 3);
    T('9.506958875462695035475532381916308450902364557733032161E+831953400624193', '-5.205024762550659909140335356483631386596767242035769132153E+80', '-4.948395636e+831953400624274', 10, 6);
    T('2.02111092595480544689348058444717006E+8854003008137972', '2.3890602398E-6896534893807', '4.828555753423987544917684890136000253974980388e+8847106473244165', 55, 1);
    T('-2.3364992938513950700424688564609332E+8372309836697173', '7.3118975431987615167495257052789772162841350552888E+496027736947', '-1.70842434463976567632959399811550835588157e+8372805864434121', 42, 5);
    T('1.883974114427395766163130055234481259667360E+8234806614', '6.07660216136935775012465E+41', '1.1448161175693434830467109584074812916619699799037839927057536424e+8234806656', 79, 2);
    T('-9.0210723632315498843650878831295420346131248098485628879E+88684', '9.7782551232369602560745363407980E+305194664815', '-8.8210347052860255109281248967461837341615e+305194753500', 41, 5);
    T('-2.16682282208841924732345836438105562534532838630E-3106', '2.55600944E+5854948026224829', '-5.538419588065e+5854948026221723', 13, 2);
    T('3.632318781495449772306959728238E+863310627', '9.8227290744158048208252005017932204709932E+93', '3.56792833025419433372659794796666620621425565857208217409973e+863310721', 61, 2);
    T('-7.579722648885365E+3491', '5.659298873E+7289984066133083', '-4.28959158e+7289984066136575', 9, 4);
    T('7.3112439E+7993429321', '5.804458237159638985394986014E+8427', '4.2e+7993437749', 2, 1);
    T('9.8267791056316289446321396447347338932094686851607700E+57811', '1.44849764295399466739749651819658470168979827724937103112E+5153', '1.423406637233697831138432e+62965', 25, 4);
    T('-5.66200E+5010', '9.43960049759870222510E+1155596473507367', '-5.34470180174038519985162e+1155596473512378', 54, 3);
    T('9.5812934179383024238000E+5797617217350979', '-4.1183E+658205397128', '-3.945864068309531087194e+5798275422748108', 22, 4);
    T('-3E+34366858', '-3.601333586653047065109233855E+722929616', '1.0804000759959141195327701565e+757296475', 45, 6);
    T('7.4522610E+19', '4.530788213E+96875341', '3.3764616298999593e+96875361', 54, 2);
    T('-401660.920700', '6.5368795276369997472142499685945455782174219270793744139E+1632396', '-2.62560904957565841385174290254563126194492355601106978906709687773e+1632402', 71, 3);
    T('-9.54295335002630310350881782547669522891502264E-8952', '-1.537875626317973684043297831235815395672338154E+635414885779396', '1.4675875360094906035135560288238961591480215448403241e+635414885770445', 53, 5);
    T('3.89864026E-4421790162', '-82.3510702133138632643983776271461918790787', '-3.21057197787712215338718539695875412565461271528462e-4421790160', 56, 1);
    T('-8.4549877046873E+8952742528192846', '5.742761221189E+5167549454434', '-4.85549755161080190473791997e+8957910077647281', 58, 6);
    T('9.08819431355926707323E+437', '-6.2229572783377592701686E-281', '-5.6555444950511476996200586177595293770646578e+157', 75, 0);
    T('1.272023702E+23182', '8.662110870098101E+5743516548333447', '1.1018410336116627537189902e+5743516548356630', 64, 2);
    T('-3.82454880292042503591869980905034737741598695384741302201358E+7406973258879', '-1.000134996875163950481166971507880509E-50947941592001', '4e-43540968333122', 1, 2);
    T('-8.61862628710545E+59471680', '-723969.2949753167573048249300621966653770601636476130', '6.239620796731464582814110566224778514057532131557850200859572e+59471686', 61, 0);
    T('-3.83493336014318243998766994098519E+341999655126593', '-7.04819205862201292645E+85091699660629', '2.702934685e+427091354787223', 10, 4);
    T('-2.948547976676054164595914E+7818', '-3.633464510291591869657E+2390877763638628', '1.0713444430144523191189287072e+2390877763646447', 29, 4);
    T('-5.993549321199284908701141723180679787411384274538398E-2551', '-8.332E+9', '4.993825294423244185929791283754142398871165377546e-2541', 49, 0);
    T('-9.27130590465841541777502542825548595E+62840237', '-7.728E+14013438457', '7.164865203120024e+14076278695', 16, 2);
    T('6.3962932785381045964430668619231785675511266E+7143324511311575', '8.5183748931551964811687882367999006861498203689567382133738E-82282927426', '5.4486024073156328147501203582844093814336e+7143242228384150', 41, 3);
    T('-2.036564E+3612042331518', '9.5049664603966902E+784', '-1.9e+3612042332303', 2, 2);
    T('-5.432971313399487152415394587E-443674220971', '-9.555938034454533420197687698367E+65276631545440', '5.191713721381456014877601642e+64832957324470', 28, 5);
    T('3.918422E+7888971350', '6.29711232966557569550752336145301768944002E+5251', '2.467474349e+7888976602', 10, 4);
    T('-4.2104463946361019706E+93822268947', '-1.89840E+726', '7.99311143557718e+93822269673', 15, 6);
    T('4.60916951765770201856536061222192247E+651917022924674', '2.959795730315711959435178849131849769E-9191', '1.364e+651917022915484', 4, 6);
    T('5954850562645290725517806840663655.82656139646753384791', '-2.1719573792890847379346486597397E-43462', '-1.29336816221e-43428', 13, 2);
    T('-1.0940512E+79819', '5.599194819308955128369652177436005767196E+2416856132159050', '-6.1258058110987455289389720083064750328077044352e+2416856132238869', 56, 6);
    T('-6.46484747518719190161848394923103497E+82809277261786', '5.3419406391588231312956906190564413138303236728895749E+409327398111', '-3.453483145366577187090573866292294e+83218604659898', 34, 4);
    T('-3.953111709536571632798965021223493147768920803360324223493E+4181469934098', '-9.3436546082274549246793966E-6657508', '3.6936510441649299807466015089e+4181463276591', 29, 6);
    T('3.751159643135241696717931743259450570894836225095E+17892816', '-0.000190093132439', '-7.1306968684233947704e+17892812', 20, 1);
    T('3.053E-148869668815', '4.4856661445421780314784895909821719920674269272167356913E+309708924668', '1.3694738739287269530103828721268571092e+160839255854', 38, 6);
    T('-3.8E+28300', '5.06224591924E-958930', '-1.92365e-930629', 6, 4);
    T('-1.1681950132393934055E+94037904398', '4.709991842750371438E-4916669073268529', '-5.50218898309920510488247059893275211e-4916575035364131', 36, 4);
    T('5.9483859143550218904E+994777355', '4.382E+30778', '2.60658270767037e+994808134', 15, 6);
    T('9.078081E+109', '3.935935563832684704083681850475043534369336934E+6141', '3.57e+6251', 3, 1);
    T('4.98070566196471542118153424092402495818099919569E+15055186921715', '-9.29145823603375496264768446668721E+655', '-4.627801864412201057511210467872792058e+15055186922371', 37, 2);
    T('6.3179835E+9832', '2.64664138072308750870019522186480437641717722156E+883296609628', '1.6721437e+883296619461', 8, 6);
    T('6.743071104125158634177145399182456102234450066184697E-287', '6.2187140766928922E-84724568399', '4.19332311953642070361840590127785761205768798887825599e-84724568685', 54, 0);
    T('-2.5660067245902998579092974669801435256470399540571399E+1872333539413933', '6.6142074825E+9972379724', '-1.697210087793047806710236211241836198625858e+1872343511793658', 43, 6);
    T('-9.77766823523244664452453203960788724744917E+97714691', '8.66420438294594758341073131019822070655890042657938029978E+315775794574892', '-8.471571597869233264458065e+315775892289584', 25, 3);
    T('2.89141603368339120984639157953353037356883884666862E+34508296045', '-8.213895014723893366848782714962581357303426202097E-31', '-2.3749787744564740000752327e+34508296015', 26, 3);
    T('9.903349992901E+3767456889567', '2.66779268284586811328467046832731669184567017297067677572542E+82932102', '2.642009e+3767539821670', 7, 2);
    T('-2.333238082615E+6067495', '8.421632338977105159588642192043908662151E+5988303377623910', '-1.9649673291083418572942e+5988303383691406', 23, 5);
    T('-9.402790807467106395823573E-403153', '6.52361149086407949E-905003', '-6.134015415778355176634001432193274108781777e-1308155', 74, 5);
    T('-5.17794385193033750869055440546969571586553424892923095E-310095318', '-7.0847207541161244818548020008333327E+7570691', '3.6684286271418851156923294216381514e-302524626', 35, 6);
    T('-7.22102049948500610959294462498625E+744949191403', '6.54005184875761229619319816531882900879315239171718E-7380921410691292', '-4.72258484675735311768076182308958871235396763180247328151084568489e-7380176461499888', 66, 4);
    T('-8.17251337335970758831520871061071935721584806737745E+385', '7.75536389371755296764858329498235019E-7477210', '-6.338081513667771555565987092725841248283464682943875174175e-7476824', 58, 2);
    T('-1.6175403369293431781519E+7640288248003514', '7.1788025293751771216E+5802620709', '-1.1612002662114745029307e+7640294050624224', 23, 4);
    T('-0.00057842745880220481281', '-3.734033239409868132348395E+646453044', '2.15986735775481487975660087020723851667893995e+646453041', 67, 6);
    T('-6.92933460513421925338150386569033101735383118856403E+644817992866', '-9.589834333078808588052485642337157E+79', '6.64511709017072249456015e+644817992946', 24, 2);
    T('-7.93356724265043939259536005E+395111028090', '-9.565392438462642E+45033181781', '7.58876841128834254595832547976264044642521e+440144209872', 61, 1);
    T('3.1945472419011481095208512E+389', '-0.6255130201671723551', '-1.9982308933482976813605027179451374276e+389', 38, 2);
    T('1.0090132951628104430149492318326966347E+21988227', '-8.44165560189310660206807418813081959330828458420E+721546591', '-8.517742735495761418662e+743534818', 22, 2);
    T('6337674980784747031198.671079335', '-5.32599992194237141280369935618967550386664740191E+97880253', '-3.3754456453e+97880275', 12, 3);
    T('2.6271472729693688408006323426659645239169940873134286455707E+3809665', '-3.1761587853850615686214E+537386312', '-8.34423e+541195977', 6, 2);
    T('-8.73216100014460001454681708539449999743552031153E+980120551', '2E+13', '-1.746432200028920002909363417078899999487e+980120565', 40, 2);
    T('9.59554833430189592970749683357634490E+760356865226', '-7.691991966484832847006E-18542', '-7.38088807014671027273814356249432929610896228707223883694e+760356846685', 64, 4);
    T('6.671062807838965049248797688649028057E+6332484', '7.4315973627785969711333E+7797', '4.957665276966643487723099090004072799911e+6340282', 40, 0);
    T('-3.804283128484423563377890378797146949417548179703978600193E+52043172', '-1.8977805109E-905973000151', '7.2196943791834196927992913328378440651280110419039815186369383786037e-905920956979', 78, 5);
    T('9.165078E+92717079307', '-3.600045158265680153055402754945950112850375546952390778469E-985', '-3.2994694679027303325804704570494518568382494217111323771149105582e+92717078323', 68, 2);
    T('-7.87655054661869420054572929E+95037', '-9.93104564362784276912398773655418042454E+8257523', '7.82223829928120867290131733158766999335959159627e+8352561', 48, 6);
    T('-2.875717833198900825E+54833698551', '-8.177029000255384866495791212001253E+783865496836761', '2.3514828118618989629059992268435214602e+783920330535313', 38, 2);
    T('1.18869813334857490301537892462735008242883200983967E+9242472908', '9.901259491926206441165990986352945332358468596436437652E+981534831', '1.17696086759e+10224007740', 12, 2);
    T('7.8907779794069666391857108359738534876200414334564E+477492', '-9.7712E+460', '-7.71023697924e+477953', 12, 4);
    T('8.25245119E-7', '-3.45658330460598726934008767140435489697E+240212', '-2.85252850054298121219904570185851975406824038943e+240206', 76, 5);
    T('9.47668849740808750804993007217708285906017895E+4895937', '-5.164078738258739535647638346089E+5619725802', '-4.8938365578566266790860731860050650984000573820356736676910234034098e+5624621740', 68, 4);
    T('5.50313677310217716335070538614879429470641843873886076951893E+38064702713709', '-9.9059675932960664945647170452882540598151713881557301E+9747', '-5.5e+38064702723457', 2, 3);
    T('-8.55798548164074464800276931744E+8962554', '-4.905889124396691085768529792865268505887178E+56363', '4.198452790112610739660137570048329778025811305365132517096698010470778432e+9018918', 74, 1);
    T('-3.605E+7643554', '4.33282260643084637397176113646580740123160763038433915807414E+263210711223', '-1.561982549618320117816819889695923568143994550753554266485727e+263218354778', 61, 4);
    T('2.349545140615843825417311277857212604480973758841342440E-9360878840497', '8.305014400647E+34', '1.95130062277847635442203071269315522208496081482986663015014845587e-9360878840462', 66, 5);
    T('4.157032753743169278952367093819214653E+2773379903246', '3.6315821373574272E-57', '1.509660589290345001668929692628242654e+2773379903190', 37, 3);
    T('2.0925179794020636251851882985140E-9686224', '5.28524639168341812903213046E+430', '1.1059473100167e-9685793', 14, 3);
    T('-3.99393384188682015949324040927313E+86335699', '7.10596763351842245184994514311973511892935959821E+398342', '-2.83808e+86734042', 6, 6);
    T('7.895275050194E-512970', '-5.67169651698315428086375846515382525E+2252823', '-4.47796040028093083882990565896428600506248545985e+1739854', 48, 1);
    T('3.9423661568310293308E+6874030415004590', '-3.70604735272917E+1724296512', '-1.4610595659012708093422153698739436e+6874032139301103', 62, 2);
    T('1.04614980793238464609375451847779409689053492978600067911299E-8', '-6.921208976571324747289114E+94796596627', '-7.240621441499987888272201917435828653198766326855664977401981824024446996504e+94796596619', 76, 1);
    T('-5.797987970432616588342313199208139039105465414798678013029E-531', '-9.47E+5771475448', '5.49069460799968791e+5771474918', 18, 4);
    T('2.83464801807487164285943735232934414E+94', '3.12330880982276618549715012675470379222048410977614E+22727094327707', '8.8534811275998903607457014387649113068098258951927705077016178290895571039e+22727094327801', 76, 0);
    T('1.955796641049951840486959236739322749754233452335E+152012300271', '3.12154841E+6814582', '6.10511389515281789824864123117844651397215532390513003735e+152019114853', 64, 1);
    T('1.2126747414928155443693416502727254264E-333757712327519', '6.7996990939382927249374E+22362766626373', '8.24582334097055121072256470259779201494299000140657032830736e-311394945701146', 75, 3);
    T('-0.00876723538296642452149551564033957', '-8.6173166684976E+594700897', '7.5550043602278509640011324056649038323585730032e+594700895', 59, 3);
    T('-4.22294527537710155567160144736320E-334860767177', '-5.505791675794545482576342516264748713964E+4260', '2.32506569445071503225642370761834705572468136929828284873169682388197248e-334860762916', 77, 5);
    T('5.144845680143976202708679294519685890732774E+6047105976993', '-2.9618857514307830830729834792279645912E+52', '-1.5e+6047105977046', 2, 5);
    T('-966232.663171517219039533', '6.668E+444826034838446', '-6.4428393980276768166e+444826034838452', 20, 4);
    T('541795588288704323833398405439023583.260', '5.006862192632431445765432576451476431111073614661226E+988189981', '2.71269585e+988190017', 9, 2);
    T('1.810846355905166969235462149372636E+40', '2.86E+42314247694479', '5.17902057788878e+42314247694519', 15, 4);
    T('1.8039373664570096870470666031618570131426001009037765849E-34114', '6.17E+704', '1.113029355103974976908040094151e-33409', 31, 0);
    T('3.72584899710145984E+4447569', '3.99226215E-60', '1.4874565927743617828977056e+4447510', 72, 2);
    T('-3.69807439601183772893941442251491646315971651597508237E+875', '-3.42574473653223338326932409600267982860302594E+75228136', '1.27e+75229012', 3, 0);
    T('-3.8742328052528028E+194211214769237', '-1.6368908169924068329239026198085042180692117634777E+93957904576', '6.34169610180904456984990562438984462883953930469691355354e+194305172673813', 57, 1);
    T('-8.5368125926336465608271E+2050014', '-1451247432041193733665345005203241078805.85655', '1.238902735287650487248787449312e+2050054', 31, 4);
    T('-3.2353908889783990681025786347082511E+209', '2.8942861200054488250415063E-19984531332401', '-9.36414694276227048152515490880209956647438224588574858263193e-19984531332192', 74, 6);
    T('-7.4650057177759292036287372858521003471E+65984772131187', '-1.369792916186772879826263160660E-58604', '1.02255119515032237142766968211617017986506411046377126349e+65984772072584', 57, 1);
    T('-2.5254858104441564534499723461473705064223623015597', '-5.391092898905344478552863945070309887965E-8843216', '1.36151286189717e-8843215', 15, 5);
    T('-7.261109177605324926471411820694827160428677329635055896E+4934', '-6.55037000100830468103354223264E+527815', '4.7562951731032002611885605155071557225129017e+532750', 44, 5);
    T('5.15437798860246110746908782306068104785019157431805192E-6850', '3.69058274336687388850388925954418875095332292E+744', '1.9022658457526300346051524226692916099060594649818331427e-6105', 56, 5);
    T('6.51217105060669206771780330335514198108754736247383606019210E+33087067695', '-3.311620198078935954965241635528765E+127', '-2.156583718453404604619730133646222607722145609525830074723062311341060498134366e+33087067823', 79, 0);
    T('5.24853455244E-8843478765188', '-9.6397257989922933107566146541616602421308391873939E+467390417579', '-5.059443393205833757478117433164991482233952608633005622649e-8376088347608', 58, 5);
    T('-9.20561871419036E+5145718296650', '-8.801032237692972792079224731253973733892351E+3597241189807', '8.101894707149909101782457645734340251142538886704036193636e+8742959486458', 71, 0);
    T('5.101E-9853', '-5.232681098E+313225101628', '-2.6691906280898e+313225091776', 63, 2);
    T('-9.49290251E+84362437', '7.568644825620667737955277623334006021356606E+24455', '-7.1848408e+84386893', 8, 0);
    T('3.08442947605333613141E+52193', '8.38035089632504109471888986403510506111172849898E+883193169485', '2.5848601324294952e+883193221679', 17, 6);
    T('-9.6079842960766707774209816297724729813069E+3994482684', '-6.40469634969E-96654', '6.1536221948961097516516055516620304728620454e+3994386031', 45, 6);
    T('-8.8640455010671077698738548766709532406777499567864590153E+5673229068292585', '-1.98878054504236286555431653095257419550819103532882E+8696', '1.762864124289254703997250987762947484109412902821e+5673229068301282', 49, 2);
    T('3.46519054525402563877156333767703E+61091', '9.721125994035215330430E+73525778114284', '3.368555388375396980025873097106660833e+73525778175376', 37, 3);
    T('7.539205847652275319094520438267923781558E+2809596564', '4.7302918464460471516317988760341510E+567', '3.566264395e+2809597132', 10, 4);
    T('3.32284369434801306501070E-531107146821976', '2.08800918400987636626379924822100505365311909648E+44074447', '6.9381281508279577932570674351965404129105088712278217748e-531107102747529', 56, 4);
    T('-2.6698298055665889742436489972124337787737721622E-30', '-654939688.2735916603155232609950206152166456226135834093208', '1.7485775006013256152802092347709134183e-21', 38, 2);
    T('1.78E+74602367830', '-5.5306850449767627732050314965300552734359683924815783E-984498818565483', '-9.844619380058637736304956063823498386716023738617209374e-984424216197653', 72, 3);
    T('1.6992021063162241004469849729400E-150620473968333', '6.9204825599261655381543795342112373903044E-711942586377', '1.17593e-151332416554709', 6, 0);
    T('-8.0980353417912780E+338450406938', '-9.084967129512539864264833148658E+6314641120', '7.35703848938e+344765048059', 12, 3);
    T('-80.4557723705436213622242927449673419071669725567257584956253', '3.3868250283935189879690273E+250352774', '-2.7248962354328890091485944280567077e+250352776', 35, 0);
    T('-8.27152376E+20264012470651', '5.622099736550741472938321E+70801', '-4.650333155196919853902671916600696e+20264012541453', 55, 5);
    T('6.89925222337897901050156742830505096343362963620399586E+1463747762242', '-7.80028931738087221568029664303848400548118823668803024E+48080', '-5.381616341593928109883817186997180909259459217063028e+1463747810323', 52, 2);
    T('63715.53482197039654714663400091234135719712136672', '4.2103837281859639670622928229396E+7565669021481331', '2.68266851047090328138802846855050269914335141e+7565669021481336', 45, 0);
    T('-8.123657456182404137085566122E+20308056929821', '-6.48799940260658843574994E+2913985653', '5.2706284722691995913248e+20310970915475', 23, 4);
    T('1.3034014665155078050031430130360E+162289625', '-7.2885894970174E+7660809626', '-9.49995823924200656068e+7823099251', 21, 2);
    T('-3.2312283119208111250064506928734888271608613277526E-6001053424860', '8.9765138494E+7603533410787', '-2.91e+1602479985928', 3, 3);
    T('3.114204561235589294661692941738885841229264962663272E+851865770', '4.02046662098E+6685139', '1.252055548935135318522755817371925436355698767392791103939064656e+858550910', 74, 0);
    T('0.00399380453468965886541998222359838601202446544990617', '-3.95582949972162707421829815465E+84886754', '-1.5798809794447358831443106889232845309061861002072209236282026550013778926e+84886752', 74, 1);
    T('-5.68587546266381057E+77678370783', '-3.9635550181E+9467', '2.2536280222332806e+77678380251', 17, 4);
    T('3.710528899144E+40912631788178', '4.89625085463804075106857596063927275009556882567858766921E+50058431402', '1.81676802935929585146474941008825829919048684155376547430064741242e+40962690219581', 66, 2);
    T('-4.78828453803882807629608576724219787595941690585586E+357252426190', '7.031954747443798881687437174239431166E-7', '-3.367100018937387448461001711557326796587149831e+357252426184', 46, 2);
    T('4.30444345559373150867994508079443930229131874423193381805E+5384528197', '-2.8264756666079225256708990883352E+41', '-1.21664046855254018285534463761589106804235731e+5384528239', 45, 0);
    T('4.5036203216016650289987252925020344165160E+712835037', '-6.6607662115022844432E-46717', '-2.999756206755942225211494184165218e+712788321', 34, 6);
    T('-5.1235536343614E+111549665226', '-1.9133659646803895191906048132875022145864115812242704308098E+9058266606', '9.80323314220161582922002880201255803784936016435282608022874768650786172e+120607931832', 80, 5);
    T('2.374809205355897237949064E-23473841402880', '2.9840652677347083658155004820116117489E+6234', '7.0865856671991955117756747520150076319882915973101160821580296e-23473841396646', 78, 2);
    T('-4.2110750862151491933737422991043E+3428785706', '1.63671200820877475098186503876170429035682842289258716786948E+348429', '-6.8923171610771361088e+3429134135', 21, 5);
    T('-5.91595702E-388475688871741', '-2.46030774345E+52072014190', '1.4555074866223386519e-388423616857550', 30, 3);
    T('-9.040243143075775005508264673976385203279781705010759892E-90355', '1.54148768358129824125225680511881026E+330551451349158', '-1.39e+330551451258804', 3, 6);
    T('1.4E+857716', '5.563625828102754628934061E+911', '7.78907615934e+858627', 12, 6);
    T('7.2066365508724067E+16645', '3.5738614E+9728', '2.575552019299203063023138e+26374', 49, 1);
    T('1.03255445986260960989337259454191199324483627E+210677012638106', '2.570582717687729465821828E+4022895', '2.654266649594212584823274451e+210677016661001', 28, 3);
    T('-88960583304481799.096431657251477449382529585856', '-285922982.6618077030653529679604131200597221141050670573', '2.543587531775164926246973221629872815753e+25', 41, 1);
    T('-3.148534244894670542467595700034365827359550679E+272650', '-7.9858325213745070706834348776695576E+23466893539714', '2.514366716755e+23466893812365', 13, 0);
    T('8.973175974640187530728923558020270200752992379311174379E+5824268333669091', '-7.3017886146E-58069', '-6.55202341684299796434853416748661314678738561021348976711374253334e+5824268333611023', 69, 0);
    T('3.79E+6647855344', '-9.047482495178549989429117E+390072679781', '-3.428995865672670445993635343e+396720535126', 61, 3);
    T('3.437035767097073781809013839762561338705442281145058955766E+309214', '-795.6', '-2.734505656302431900807e+309217', 22, 5);
    T('-2.4817593680693330398448853770000176364E-462505176065', '5.2756529713442936752216348215E-2801687752164604', '-1.309290118e-2802150257340668', 10, 4);
    T('1.872318824827648117334E-18442673', '5.76470550710938367373534E+9353355687525', '1.0793366641e+9353337244853', 11, 0);
    T('4.020247628430718E+8982589537841252', '6.696462698651E+4079152242', '2.6921438283126448570465561418e+8982593616993495', 49, 5);
    T('7.084805975667046929773953E-2893', '-7.9551386655908512E+1688839', '-5.6360613955238040309904358699106000858794e+1685947', 41, 6);
    T('-8.464859861830327E+449898840823', '-8.3171673556119033319934504803285583023E+514610670311841', '7.04036561126446822276866809437685090221173448097738521e+515060569152665', 69, 6);
    T('-9.41688391798485874863E+346366051', '-2.15E-49177094470398', '2.02463004236674463095545e-49176748104346', 36, 6);
    T('-2.2E+556', '-7.26379080603556501051076300777946913166406917475148E+4542857651', '1e+4542858208', 1, 1);
    T('7.21894648982455186482457937657414291E-85', '6.7221462873209257493553029E+415080', '4.8526814344942e+414996', 14, 1);
    T('-2.11547786917940753E+59717221716', '-9.87510694409050668785689942015325284E+410116', '2.089057019600335577648289e+59717631833', 25, 2);
    T('-6.44527791897682E-463616', '9.1812445140206812999246409502674546E+6097526430429', '-5.91756725349445618439055e+6097525966814', 25, 2);
    T('9.54100338811435904866438514650572318525030E-165733612883019', '2.1084463714E+302230075472', '2.011669397318482622439317887856204943927384503576142e-165431382807546', 66, 3);
    T('1.38172536E+50', '7.0867504527830021061391308114135E+13898750209', '9.79194282060175e+13898750259', 15, 3);
    T('-4.2560764570289729805491217451317618871E+2240161379464605', '-5.2609428309566811610956431945306493653991002285631E+405450893876', '2.2390974924610086670905940738034333311280907816238475358287581423149871205367e+2240566830358482', 77, 3);
    T('-5.729328851059997377E-847009', '-0.0000061201544805277914988863091', '3.50643776382319867575e-847014', 21, 4);
    T('7.46264693503902378749899069627826823073417539E-4233', '-2.32469871973302966450190114017609819638289624319779683E-91263347692', '-1.73484057757048363928972283246971515e-91263351924', 36, 0);
    T('5.638657E+106588', '-1.9312408936380514582963493817160209063424106484767612878E-399085876780140', '-1.08896049835984543216829185156587132956939781999080293727824846e-399085876673551', 67, 3);
    T('-2.1671441209890528141168609931303935229646243539410553E+86533462807', '8.1223929193263733272875823501E+159488472637', '-1.76023960634812598913787350659510790283925647592944511909e+246021935445', 57, 6);
    T('8.5586421268256769011890972680253332E-98122749', '-9.7846432102582863519844882328103973855777725E+18960', '-8.374325957527539879775230727222197772604011232466137983e-98103788', 55, 1);
    T('-5.301215347285826E+17758849733627', '-2.5102930742929545575967052100073095372E-83398203655', '1.33076041716271289029581071254e+17675451529973', 30, 0);
    T('-6.8691836984350469471751764038540091E+10112763224', '9.15768065575605615381257936415599997139702038588134661E-5631', '-6.2905790675993471809033790486987095372380689464389994216792876238218461e+10112757594', 71, 5);
    T('-7.54220605953991192681991648227752096928478E-17672366148', '4.4641540923377865414978224086194600243992505351E+239', '-3.366957004594994916554864079953e-17672365908', 31, 3);
    T('9.95783233650506961174488058714844891E+49724516388372', '-7.347795487220013745585152619477417E+160537921118', '-7.316811550466547583606e+49885054309491', 22, 1);
    T('6.7E-811280027656', '7.6784213248342563578322716003236E-754196929', '5.144542e-812034224584', 7, 4);
    T('3.340E+4563742', '8.79442787195315341726106242551267E-504547208950639', '2.937338909232353241366e-504547204386896', 22, 2);
    T('-6.74838869378069461283176482930662093765733472073287878502811E-6667646764', '4.0679293238520545225249489117338E+2882954128192', '-2.74519682562e+2876286481429', 12, 0);
    T('-1.24442872E-223283085259', '3.1293271384499531875875589659683242000775763446E+5841', '-3.894224565362538029289505891944485244847562231212856912e-223283079418', 67, 1);
    T('8.865033413016579499842724455837757E+31005341885', '-1.37247436E+8730910085', '-1.21670310599085456184357633482e+39736251971', 30, 0);
    T('-2.5374319740953458860375832036334397299700586927E+738455277', '-9.10595169912E+3061168', '2.3105732995914930695848032657304293466770969939517690940424e+741516446', 76, 2);
    T('-9.2E-766786', '-7.27947621842605447397118418851406055688634E+12514', '6.697118120951971e-754271', 16, 2);
    T('-1.258700467056374340182170295E-679983952151', '0.000098720', '-1.242589101078052748627838515224e-679983952155', 58, 3);
    T('6.67706962050653932807722209755284704329015659029E+2043', '-5.2874870454293922258212467158617358E+790738887996', '-3.5304919119858474821304047e+790738890040', 26, 3);
    T('3.056463210804232511913920027767011327650888744E+29497691090186', '-8.93268E+116263335357440', '-2.7302407793886751674523235e+145761026447627', 26, 6);
    T('6.974E-5635238191805311', '4E+47', '2.7896e-5635238191805263', 56, 2);
    T('1.0697509032153173373419295E+135242352250', '-2.83903795037728508290028914906565671358782382899473E+269354736681', '-3.0370634116786640002089401723675322715089545711933774094742065e+404597088931', 62, 3);
    T('9.062E+39', '1.589141E+67413443937943', '1.4400795742e+67413443937983', 60, 3);
    T('-8.249897084529E+6338837247', '-0.07836086742597355884189626433', '6.465e+6338837246', 4, 2);
    T('-9.38233723078362994416924527824619E+1426608341630803', '6.351919677259167754601883927457849386256701809678582073E+4417272997013', '-5.959585247489582844487489151492685682e+1431025614627817', 37, 2);
    T('-1.2532016237882809075021624115652846633868E+846717', '-9.2277442240016224807770E+46352459506', '1.156423e+46353306224', 7, 0);
    T('4.4806467033616429E+2633', '8.62547151E+319666', '3e+322300', 1, 3);
    T('5.06110225585790058415158577067932298E-5224801895', '-9.9998805760300711E+1776698368211', '-5.061041814165539517964262450207023295e+1771473566317', 37, 3);
    T('1.754889664343821815199963E+600', '-3.53333220537256214655E+97482737902', '-6.20060816790147127273393103541003063276057765e+97482738502', 73, 2);
    T('-3.531371869131111718796006821467E+89', '9.734195748888E+73', '-3.4375065036238738367405488487193920579778696e+163', 57, 6);
    T('51171.6119945061362662913846674279319406701777148533039030', '17869.71917137192712100989876360661005', '9.14422e+8', 6, 1);
    T('9.16352052527084173672114552869313E+37747', '-3.7011122288186908103856253529079951171928231E+63', '-3.39152178751109e+37811', 15, 1);
    T('5.382146789955769403726E-79', '3.57230792177479486743731904325697633288805404636817E-930499070', '1.92266856139137779892318092895407e-930499148', 33, 2);
    T('-6.5341472282221872959010431448979022298365973350E+397282', '8.595208246978783983632322111257144611477499502054696E+9375052', '-5.616235614298890681956e+9772335', 22, 2);
    T('-4.701114063060090038546492496E+99', '7E+5414817', '-3.290779844142063026e+5414917', 19, 1);
    T('9.08591272152405652E+333661748', '-9.72786746622208428709E+477348040517180', '-8.838655476464722580657481011052907e+477348374178929', 34, 5);
    T('-6.4749936916641639028061786334102492015653745549940071364E+811095633774', '-2851627200046.6155336', '1.84642681312797783364938141914186639363354422134487504773172582675e+811095633787', 66, 4);
    T('9.1E+5012221', '-1.90882407360554126288561549355831379644389414E+4711044121', '-1.73703e+4716056343', 8, 0);
    T('-1.694344415859343521954696874884189E+78438', '6.571623186139996987E+89466952843061', '-1.1134593048568e+89466952921500', 14, 5);
    T('2.996104620081239202353771869563376086295E+2814696365991724', '9.41862667195584320128806289046202339196006746512856389107E+9168102', '2.8219190886667287969472409e+2814696375159827', 26, 6);
    T('3.00634751203272037595117465940405829433661832E+439724', '2.19709215914539094171287590826545638366035818100620497237E+11384', '6.6e+451108', 3, 1);
    T('2.10019578011249196427038731727626998926089079090885631212E+73', '9.73159295800903214997089493E-94594061', '2.043825046418301253057950552383946686031123018580275e-94593987', 52, 2);
    T('-2.21494011963367253346654863245109935626010173234061E+685', '5.326443346123176721608750375804508653128332563497050449E+6429595871', '-1.17977530622840480859220294109762e+6429596557', 33, 2);
    T('-9.88380336304475056429462976210400896740187190138E-199429272515', '-2.268451817587426819020707576157930377912366012135089260734E+6260855882178', '2.24209317035755862392654777966894237890709003345042293555689766627505834563e+6061426609664', 76, 5);
    T('-1.9216795842780097112459366325953302178E+948895923', '6.23311966E+58', '-1.1978058796983889236837970719744149604761261948e+948895982', 69, 0);
    T('8.616621882197005369985745172804E-93176393787247', '-3.038107810647243821349876752628191171374364061446894591E+8969350', '-2.617822624169667724732578931e-93176384817896', 28, 5);
    T('-8.4297871236472E+2193119', '5.430839E+2387499124146279', '-4.57808166728010360008e+2387499126339399', 63, 6);
    T('-6.139128759641014745050E+73191717683', '4.46244167868239833975818829886188971442208952753286947E+56583787545', '-2.7395504047819839789664438482962952465939794323801e+129775505229', 50, 1);
    T('-6.9417394815562689E-6926', '-7.63602126940301053036004393525918755237017451641943E-90894', '5.3007270327818296650766482454695108444090095042139489357572498264727e-97819', 78, 2);
    T('-3.85467671353E+7116603', '-9.84301413454E+8651055', '3.79416373753579844583262e+15767659', 53, 1);
    T('-138399.74779930956320', '-6.5178574099046006716893390909E-447428615787743', '9.0206982172265778640856430933620223714396409488e-447428615787738', 59, 2);
    T('9.4331664033713283254543217717704333075663169E-372863413', '-6.47253E+85637980766', '-6.1056452540813023726352861297437282696222213124757e+85265117354', 77, 2);
    T('9.0642572590820271902633849934064479505860769298338190339E-8', '-5.1199209869652343846553314E+555576493', '-4.640828097202604288468e+555576486', 23, 6);
    T('6.062535825466550670227658361843294113817554364361834210E+764', '-9.7570302097440841424040929349743296979068E+4731198365223884', '-5.915234519673292317892682754409214478799899288678981448451837987742e+4731198365224649', 67, 3);
    T('506228473.32230220', '1.611570461638695909519495343770374713986953346E+4468097', '8.158228544466748132285676e+4468105', 25, 0);
    T('9.86058250335E+9231526989105', '9.3193307397E+629672', '9.1894029e+9231527618778', 8, 3);
    T('6.6660809414221047787624600421788543196328959159E+8589705355', '9.32373695827716276443867450741685085759274195839568573E-7417', '6.21527852404043008255924746228198340407294277661505126e+8589697939', 54, 0);
    T('-4.41482531520025600784063004487921021E+90', '3.1018075956929E+419448550', '-1.36939386963454554919667032106242977e+419448641', 36, 3);
    T('-3.52239577625611971965751582323E+253774410606195', '-2.10E+94', '7.397031130137851411280783228e+253774410606289', 28, 3);
    T('-5.78640571148995373383599995793873192858E+57733099', '9.7054944334562640185740168250568884157698381915578918976387E-8090', '-5.615993e+57725010', 7, 3);
    T('-7.324221014054326528752241635588210850134298193562007017520E-3619406277914101', '6.207601733889616080353632752E-51832', '-4.5465847066234399506197008811716970810597107715566266179e-3619406277965932', 56, 6);
    T('-4436639465129079980522215668.954224166', '-4E+5', '1.7746557860516319922e+33', 21, 1);
    T('9.1512919791467123644158963392976E+48', '6.719199590882080499001161999468E+64954', '6.1489357322325054464302284705e+65003', 29, 1);
    T('-174253476692855710343554514120706604227127644059081965851.8', '-1.701405786287892105132372248087901546651563239204641592199E+14641', '2.964758735260070506734046914904e+14697', 31, 1);
    T('-3.599650729722869E+73812588709281', '-2.79718070285504944863750895943E+4600168', '1.00688735581989063466111839224e+73812593309450', 30, 5);
    T('4.05348799392388130413213278295851501718082868469247155610E-45172', '5.7140009192002803291960098068672761E+2814814238', '2.3161634123248358097673e+2814769067', 24, 1);
    T('-2.2572737425545390740565849926713E-54084', '6.9904918546802434231272925668545926081739153231051848647035E+528013111151640', '-1.5779453711111094165487915e+528013111097557', 26, 3);
    T('-9.1624946344239427E-124717567', '3.30779586609736081055902966306E-10', '-3.0307661874886766858831153432936661564777746662e-124717576', 80, 2);
    T('0.00001482117599552752518975238218991128636347221183462111700', '5.6768041994E+8788', '8.41369141314571e+8783', 15, 1);
    T('-2.53525965663309511692317849597159511E-7', '2.803008639031889937687943361387572E+974718306909', '-7.1063547197315885385171705091155687e+974718306902', 35, 3);
    T('5.50257416E+532254903230', '9.875067413099108588330176215175E+56', '5.433829077557720043718e+532254903287', 23, 4);
    T('9.2406887880959094E+6760130051', '-5.2989932201073377148081134575935214E+1546143', '-4.896634723724211503810143829485e+6761676195', 31, 3);
    T('3.51013721003706E+55958020', '6.45535713828553398534961230488840177E+70588280521', '2.26591892951744039822401308999597005823260258695962e+70644238542', 68, 5);
    T('9.25728933E+61964229656', '-5.29325917408403537729E+67895', '-4.90012316731727532215292413157e+61964297552', 67, 5);
    T('3.63774290976086E+18908466912', '-4.760008678317905818331704229798368E+56689041', '-1.731568781995112414135096109277018017205689827648e+18965155954', 71, 2);
    T('-6.731537967657941621843950405659059069635841374344084556673E+60', '-9.337101510648126126101716540838283431664481227298199E-48202192', '6.285305332680418350649576207013592401043594608082292973084e-48202131', 58, 3);
    T('-1.63202892512704E+8753583', '-5.2299237155016028804257320082446640357815179E+27040', '8.535386779906496293535434581682915177630074312310941534016e+8780623', 70, 1);
    T('-2.1701432414715646974785E+4303700', '-7.518989922937501860613322886667764624893468506E+25381985', '1.631729e+29685686', 7, 0);
    T('-4.34787455948268931770693261468746779626060174235292461188E-710390188', '4.252078849064352E-595594562836771', '-1.8487505452761330053111719374553925089468714e-595595273226958', 44, 0);
    T('-5327951522.4966359605', '-7.3216852473231156070289129659437228038523533062883064E+93052446', '3.90096e+93052456', 6, 5);
    T('-5.25303609288842931244451973517012623683976393837048E-40', '-6.7264889291420201592169024641415556122084008E+91269084', '3.53344891231974724246412938983295273701432928757122009790423272e+91269045', 63, 0);
    T('-7.4886E+1992236019', '1.791399785124682826375327216473339264341488549882831035E+6168729702529', '-1.34150764308846998135942e+6170721938549', 24, 1);
    T('-3.585861764955966095293E+72261739164572', '-1.59048440190123350988800581560411922631554465085E+133298653827193', '5.7032572045364913108528606802722879198197197889850445e+205560392991765', 54, 0);
    T('6.64357233253369536735857586183302609104482E+74', '6.5880E+203738', '4.37678545267319850801582977777e+203813', 30, 1);
    T('-7.3050847189365562E+740171153164055', '-600608701999425886531813425816500417116899759.15751', '4.3875e+740171153164100', 6, 0);
    T('-7.93944049126327329730339111836433008749657406700849294338232E-663036022813496', '-2.28952653008407437129071700196024011738073155084077887269887E+85441910127', '1.82e-662950580903368', 3, 5);
    T('-6.37369444E-2043928634630', '3.909438407411168479556418082021818990E+14013440408012', '-2.49175658408390193320519955956979316552494156e+11969511773383', 62, 3);
    T('-7.55627597549197448133688775305350550753E-2610', '-7.544226676191073396453191042472870031E+305130', '5.70062587869682794211879136508442453008774064643928005478201e+302521', 60, 3);
    T('-9.8696220589049532E+92482230195', '9.7633535E+555952', '-9.63606090724868809925562e+92482786148', 26, 0);
    T('-8.682550613160293454398029705305190438682179949378892E+33014190147084', '9.828012E+5510128', '-8.5332211616746721993345288720095875293653728729e+33014195657213', 47, 0);
    T('-5.867056562067E+1967932', '-1.253030067552983652329810982212E-58771445739709', '7.35159828030399e-58771443771777', 15, 5);
    T('-2.84650317843517516335499634397767348103905839069245552640E+90240024636832', '8.7909216473305771723934597037995173986584239984E+340387396', '-2.5023386410501e+90240365024229', 14, 1);
    T('-0.06428965408714349750', '-6.70385347545504443505782740865692E+685545', '4.309884209878995378631761701831206866474057863777e+685544', 75, 0);
    T('2.59835088722767010286696622723158467417189E+13029324994', '-8.7806775276572274142038397394E-4954', '-2.282e+13029320041', 4, 0);
    T('-7.6522454433E+679735117685', '-6.1491126794588E-766452896', '4.705451948172685581009e+678968664790', 22, 0);
    T('-2.00921899764424841869157E+42706315603', '45.18647762961803127410955', '-9.07895292900553950488044410460558938e+42706315604', 36, 1);
    T('2.3840522127005126166854680293610915893336528838363645126934E+676283', '9.34487526491354802E+50', '2.2278670552727433170273116009797318798198626163399119869752e+676334', 60, 0);
    T('8.9522643553921860238921E+60', '-3.92105734277823650876E+968395096409296', '-3.5102341885202407255848913e+968395096409357', 26, 0);
    T('-1.7049430963520564093559002338809811276524389235626888493865E+429904024130845', '3.77107390752673852052044273384579613106499004086493653E-5120088', '-6.429466424e+429904019010757', 10, 1);
    T('-6.0972E+60476488433', '-1.962710223285974607E-67249701380', '1.19670367734192443738004e-6773212946', 61, 1);
    T('3.3354375729422151160677975387E-9117', '-6.10424245705726806950254984E+859667416071614', '-2.0360319645617917989876903553104552759849755e+859667416062498', 44, 3);
    T('4.69522164003509645318003848788614229526656482728623102747E+181168', '1.40571421038834773157402978878317907617552005E+1788', '6.600139780320218656100705345046636843862e+182956', 41, 5);
    T('6.756405378462768948203746931876183556197482000690191873E+1378074768', '-5.92877314435265294605908632289181481671418676067192732684387E+6750', '-4.0057194760189886805471853420561132820288331321e+1378081519', 47, 3);
    T('-205715836.081675247114341729717260697', '9.8435425801413509838666060183815702670452710007742090E-8166308', '-2.0249725918793487885291e-8166299', 23, 5);
    T('-5.125430029977908543669314764073580E+1260976193659', '8.94898800523E+4529449927755', '-4.58674118599179428791786354753078147531448234e+5790426121415', 60, 1);
    T('5.9678377845E+9896367569516', '7.48622551246601557942E+4758697', '4.467657947658256354702833659499e+9896372328214', 50, 6);
    T('9E-809815414196', '-2.397786487687353514151E+2181718952243', '-2.158007e+1371903538048', 7, 2);
    T('4.0865940159442256110450160386687E-2816380133831', '66.957841', '2.736295123511449233324800277596286662767e-2816380133829', 70, 5);
    T('1.2332522319112476818208E-726522265487', '4.70265906913E-571467587', '5.799564792922242904168273751471904e-727093733074', 37, 6);
    T('5.493089002072E-94115881', '-5.24241E+499', '-2.879702471535227352e-94115381', 40, 5);
    T('-0.51473', '8.33222398766359257819E+2829026545128195', '-4e+2829026545128195', 1, 1);
    T('8.2885692048424601181110613921277728529060021835334564486617E-674903669', '-7.52199646130E+549969705724', '-6.2346588228065139832417783831972159077009e+549294802056', 41, 6);
    T('8.073508850337709328726864812E+79954253', '3.080825873730865497833362012525279538039E+396', '2.487307495791554875265931139302246662899235096751543e+79954650', 52, 6);
    T('-7.08024130265413154856370317325034393E-71013372579844', '-4.1573033308401381811664058118550E+774405431', '2.943471075067593997534e-71012598174412', 22, 2);
    T('2.5778937815825784216023544393E-693374976305', '-2.35343103911253998374707020155E+35913255664469', '-6.066895241111642723331585768572745773453948336823e+35219880688164', 49, 3);
    T('3.23567553434664E-861829520131', '5.09021108987919702890385470608E-9', '1.64702714881820636141854183688530492420355712e-861829520139', 49, 1);
    T('6.302380786615407195394510586805001184515693E+329067847755', '1.1906530074637310216011116E+8566445105878', '7.50394863776526941e+8895512953633', 19, 4);
    T('4.05795973484116234232855911114605207829709605586737E+80907783475', '-2.3368946823518517030798E+3113606', '-9e+80910897081', 1, 2);
    T('-2.4244853420100845125401085646655261455246676276138897347281E+77487591', '-3.448832733E+94077186826639', '8.361644408203079482944675393192113767352795173060237602582958e+94077264314230', 61, 3);
    T('-6.82596722919661834666567575099330822661549685750582E+3721841961', '3.875860361294926638093302198952900403072089308676686E-669267449', '-2.64564958111413344917288610049695771810021547802007142e+3052574513', 54, 4);
    T('2.076565581367548206674491335681512966744761377E-4254', '3.84041345858988521299414092710659519305245672822204253E+7628230', '7.97487040632846150763778020692846034676126772917059817009e+7623976', 57, 4);
    T('-9.61668672578244398386551207E+1501553637309638', '-7.0618603198693E+836', '6.7911698397416861167222814474922839972451e+1501553637310475', 60, 2);
    T('6.15123062201803029224721061E-51798280307', '-2.9794305118356327806618713992882E-124721966454', '-1.8327164200578197794012966e-176520246760', 26, 5);
    T('-5.3903808699641465637135807141213E+451', '9.55329942501412658E+2491', '-5.14959e+2943', 6, 1);
    T('-6.4360900573521114115484772E+63342', '-3.7041E+3953617826678', '2.383992118143795587952e+3953617890021', 22, 6);
    T('5.6369814893E+5962946437465', '-4.411857447884886678287E+16800', '-2.48695587671574456427319832328291e+5962946454266', 66, 1);
    T('4.0888471336490820606168142785925764612700292778414838576226E+5175848', '3.38853001635178947238380923287047255E-7613907370', '1.3855181244643891549051668e-7608731521', 27, 2);
    T('-3.50954528337185100329974828869378313740043139098E+593', '-4.671370684506782747E+37798105791', '1.639438695269231444705851149431025970325870362897138308962587e+37798106385', 61, 1);
    T('1.01908296384102174288103070402E+321904313', '6.1049440045422950808824053503075227834474677774832830671590E-915', '6.22144443e+321903398', 10, 4);
    T('-2.571312429E+8390246', '3.012464026853291E+879', '-7.745986194163256907853839e+8391125', 63, 2);
    T('-2853690.349255416114229488667709892133055625861426523138', '-7.1622934545E+5311307544163', '2.0438967709641885783619690995620485929485352191696110703990200221e+5311307544170', 74, 4);
    T('1.82288364E-47767587925', '6.49281520276367142561510736687733932762922959E+689778666469', '1.183564661066117942808925615592417974706392260541491e+642011078545', 52, 4);
    T('2.954639717729694E-122194', '5.8544119072E-7066', '1.72976779449627675046123968e-129259', 31, 0);
    T('-9.339313037841414991637296940308E+515063', '8.7285666411974125983012398171107545643E-7584871860473', '-8.151881624e-7584871345409', 10, 3);
    T('7.4054656394998744660121839326873686E-97880922703', '-1.167142975936693768664649452777902671623E+6748203', '-8.643237204682814515e-97874174500', 20, 6);
    T('-9.371189479639259810036162406835132359524E-9780', '1.563911730124703841531389697003295497E-976859651212', '-1.465571315242905791326268664308891214077028311769314e-976859660991', 52, 2);
    T('1.69956563185180195E-412495179', '6.4437113620641427177331764E+7047107', '1.1e-405448071', 3, 2);
    T('5.004177010333915686309E+29355036', '5.57521332855716248545453140703635310354100E-53096817015970', '2.789935436647298016547027468748244851957656107069135515312e-53096787660933', 58, 6);
    T('7.38963014490818957062479779011266657633E+50228647773', '3.06E+207757026', '2.26122682434190600861118812377447597235698e+50436404800', 62, 2);
    T('-0.0000039168759277437740', '4.67009050422570440552732448299579069214461078596097816749E+961530277557554', '-1.82921650763864452553178e+961530277557549', 25, 0);
    T('9.62267036662462745146106975856749960166551970791766419293E+48799', '1.295048486020903219163186823672550321099525986289E+5444234874', '1.24618246897754e+5444283674', 15, 6);
    T('3.7061910597985729080719520576E+6246096469511650', '3.207935060622779742736616960769961901813142E+3123535695267', '1.18892202420945392846e+6249220005206918', 21, 0);
    T('8.6958E+658539980', '-4.766648230926340584562128092707809302E-7', '-4.14498196864893e+658539974', 15, 0);
    T('-7.160508948976E+661732619104', '6.5556253059386097E+395062137721456', '-4.69416136693069425941256786672e+395723870340561', 73, 5);
    T('3.599405564688342E+46103169', '-1.2901874855148293642752E-62423444', '-4.6439080148533364524369863814767197184e-16320275', 58, 5);
    T('7.2280700490198446364541145280467219892E+900', '-56.5', '-4.083859577e+902', 10, 2);
    T('296785557.6540512887943926253656008619', '6.2128677395131E+1002353', '1.843889416702260444434347862513857531324194046341e+1002362', 49, 0);
    T('8.90202894094570842412640472982742419407E+5485684321974877', '784.81240843557083374586094920920140544593251508946624766', '6.98642277310675539344203477425185e+5485684321974880', 33, 6);
    T('-4.2034335717436E+728658249622', '9.3953915164773214672315622635751183982121090824E+83', '-3.949290412003578584717266149252179285453898373658468956407264e+728658249706', 79, 3);
    T('-8.76196078537716316453045571965773E+7928495', '3.1202068911272146557812330168688071583825346043E-335', '-2.733913042232024636527971263074724759713944e+7928161', 43, 3);
    T('-3.0799739577733631137685568961812436100360032148115162428E+87708856154', '-5.5174009183784843784E+417191002', '1.6993451143200568908750013444717804926e+88126047157', 38, 1);
    T('-5.33073509047087500103830150746347064858896957676542446775E+1925602', '3.33494608584643953090987787289907341437428E+578524624827', '-1.777771412465011030062004736703859858283703e+578526550430', 45, 5);
    T('1.104514818443243307E-658031', '4.92699587491554326153572022612116367053E+860', '5.44193995425294997605917616699779446915638728155397564271e-657171', 59, 4);
    T('9.7139225390597232529039686871181074569814282130001E+839334', '3.4740149775157186108678974851252000401E+31295477', '3.3746312391120996715985786087110577e+32134812', 35, 6);
    T('-6.15417510551E-8593', '-4.23364158889880604778390326823453439286699E+6612654504566', '2.60545716720528337538335289974668995286479319746461149e+6612654495974', 65, 2);
    T('4.3498434172827348322829715411E+466649518779', '-2.427285E+585737122640901', '-1.05583096791191230173779725771389135e+586203772159681', 77, 2);
    T('-7.362823554121352447408359525121863573907833444305721131014E-656', '-7.33792180224933643708546523876E+7746814', '5.40278234839020192694374199143048528428e+7746159', 39, 0);
    T('1.6492112514964675760386411865911322E-295619792652734', '9.3010733106535164352113036228567549252', '1.533943475492327878876174528695270018004416827881600240906835311099431144e-295619792652733', 76, 2);
    T('-8.0632803050121477041E+51109490', '2.917064673122271191032126190729514855970468804471538149E+36264788502558', '-2.352111012723350779000788830953258356291043508231913286326285644731e+36264839612049', 67, 0);
    T('-9.656122494815270690651579012E+926346486099', '-8.183666689706408766630783917E-1879662839741', '7.902248801254e-953316353641', 13, 4);
    T('-5.928502650194E+6531301', '-2.0755999103270403326678408000997331821033345740209E-83382320', '1.230519956911628736149e-76851018', 22, 2);
    T('-80411.570905102568251866835929500262709408042730817579', '-3.43054975967237703405965272E+691642376', '2.7585589524337792093597315452897468540864796960662895008e+691642381', 56, 1);
    T('9.486563078874046E-694', '8.3008325874413169275603302239397774E+550948', '7.87463719479353131764129626151809603692925038773604e+550255', 68, 0);
    T('-9.821290219538401362733768714411376099143203387787651289923E+692687975221943', '-3.608060405060941959751730367500468938887387745358362E-2187358626838961', '3.543580836772879200714692581573e-1494670651617017', 31, 4);
    T('2.90E-69660156417997', '4.696897169659965533220E-755198229', '1.3621001792013900046338e-69660911616225', 80, 4);
    T('-8.42952467263381993726270936421627871E+333566489688715', '7.47972824857326540055461584294756648133852923820579E-3322144794034', '-6.3050553815944490382831918800365617673e+330244344894682', 39, 4);
    T('2.998642015220870004697594286216351585900787736162591E+6010', '-1.913875E-20397', '-5.73902598688084258e-14387', 18, 2);
    T('7.30832108049883259502917491355E+5181298020108196', '-9.428865288565872892569042902596245831219654E+997198273', '-6.8909174953609677169e+5181299017306470', 20, 1);
    T('5.3346761444413993308396448790127841724549154E+7781706769283847', '4.6914686010117448649819987709223997053E+93428', '2.5027465628213220696175758924099823641041347605891649437088746679e+7781706769377276', 65, 4);
    T('-1.30E+862433507670035', '-5.188748972807716387507471764257490019537E+1365420678', '6.74537366465003e+862434873090713', 15, 1);
    T('3.7044905724770621298120944123968E+611969891724', '-7.6536084186461944664153175623805356088092274146830E+69', '-2.835272023230590313746124400758e+611969891794', 31, 0);
    T('-2.5570E+655343123733640', '-3.6800848316735825416965836305398998438E-88496', '9.4099769145893505591181643432905239005966e+655343123645144', 54, 6);
    T('-6.72146094421846219289043221E+94631042774', '8.3687E+5008642', '-5.624989e+94636051417', 8, 4);
    T('-5E+69', '-4.7375466853876884273E+940009363', '2.36877334269384421365e+940009433', 65, 5);
    T('-3.7258201637058152130365047121068977543495E+301602', '8.064E+8', '-3.0045013800123693877926373998430023491074368e+301611', 78, 3);
    T('-7.82142250935948628993746657231266156129918514696844463E+598841400', '3.1621024816495096516800848847E-99339004163', '-2.47321395268749667279936037823487326602478e-98740162762', 42, 6);
    T('-7.8741946501061375194714E+323922656111', '-0.0001584274224378', '1.24749e+323922656108', 6, 6);
    T('-5.50535566617805E+485967525203803', '-9.67001677007030801397503253E+1138034', '5.3236881617143335929013670863459e+485967526341838', 32, 0);
    T('5.694149936448043734358192362566388113010641E+579140', '9.124E+8509338', '5.1953424020151951032284147116055725143109088484e+9088479', 61, 4);
    T('5.32751211830477983401834471113634751862737E-3405510422029', '4.791126344217230740651052488081336833750859925297400283E+357', '2.552478365914e-3405510421671', 13, 1);
    T('-6.057280877859995718541624759328816213804048989878744E+56703252', '5.370303515681083962054871712031119220326381402001157600682E+607', '-3.2529436793839337544741135456e+56703860', 29, 2);
    T('-8.12737072056811729873831602000809072981883148733551E+724338', '2E+14', '-1.6254e+724353', 5, 2);
    T('-6.402326281615894303813717396975217E+57482', '5.0399756104562348043393264726762339485659594460090748E+603590836', '-3.2267568309527063e+603648319', 17, 4);
    T('-6.94744484074162998281745137919E+22806192576794', '-9.413975589002450328223E+34635698948619', '6.5403076136682720834484981683086963199081e+57441891525414', 41, 3);
    T('2.514086910976801613913827935973835590458027770297E+2707507810', '3.2415799131244318858065834793348E-30671793162538', '8.1496136304715e-30669085654728', 14, 6);
    T('6.60249275928504711159035515905757726384066433960338519E+8504875', '4.73178205776037108248396761E+33983979305884', '3.1241556774877750637493048735300447779323e+33983987810760', 41, 6);
    T('-2.215199810101212312111063581020092151E+63550130738', '8.3667431556996136396209228924456885653358374678706154252257E+333329457206928', '-1.85340078497e+333393007337667', 12, 4);
    T('2.36610504326129948812E-60363819911', '-5.47797958554733715803405433945E+3176431', '-1.2961475124245997626480992887732032790776549722334e-60360643479', 67, 3);
    T('-9.930633506113E-7741620', '2.07586846956345120564E+3984364519', '-2.061468897813032287278535016007732e+3976622900', 51, 1);
    T('-1.0146E-54536551698394', '5.86977690888129181296670757891111734259517609477149E+812', '-5.955475651750958673436021509563219655797065665755153754e-54536551697582', 75, 1);
    T('7.754072896462507118107441719E-563515867', '1.53379375087955900498125953352771301E-853421364718', '1.189314855e-853984880584', 10, 3);
    T('1.896172430043142356241826758284437042950049E-877911678659879', '5.15307E+43638091903', '9.771109e-877868040567976', 7, 1);
    T('5.493103252376372E+96796577946619', '4.66945750828421093553647842371E+820921', '2.564981222558926909155733703213897036220858012e+96796578767541', 47, 1);
    T('8.8916189E+69433', '9.73164049962582735176927873848258776455633507093E-2986819200', '8.6530038594478449409128667270459934688237859031414028577e-2986749766', 59, 0);
    T('3.70269282958596271668506364306928189668E+63782010147773', '-1.41850261959847807396E-3562518', '-5.3e+63782006585255', 2, 0);
    T('3.794996243730475960594491432794411503989957434456770E+8549070235864', '-5.13567566412E+5923609139757637', '-1.94898698543535e+5932158209993502', 15, 0);
    T('-7.741691460500919E-6993694815472', '-5.51434695128671105257203E+865633522418237', '4.26903727030156081276760837e+858639827602766', 28, 0);
    T('1.0581616E-76079', '3374.44030050717576423548276697112104007324377714147233030527', '3.570703147489153918164641221470588593557567752410063787391552992e-76076', 64, 4);
    T('-4E+30', '276498178.865655', '-1.10599271546262e+39', 42, 3);
    T('4.28E-4510', '-4.68184451050533982234897590415001417057155769700836E-2722081019', '-2.003829450496285443965e-2722085528', 22, 5);
    T('-3.90811178268157842776E+464969491747', '-8.178807E+67346741', '3.196369200497857241601248232e+465036838489', 63, 6);
    T('7.052028534454999129622764337743786189435854836879379081E-7670056231113', '-4.2915749156377951746358846986850E+95398115', '-3.0264308762829037232e-7669960832997', 20, 4);
    T('-6.993366530E+262215', '-4.46437047150959207210617E+3706026650137', '3.1220979032975499772e+3706026912353', 20, 0);
    T('-7.95225191E+36', '-7E+8709639252', '5.566576337e+8709639289', 15, 1);
    T('-3.26624236527522639938921124E-350190', '-8.8240827991129040087649486E+909748911', '2.8821593073158972028318523617237884509850467913142264e+909398722', 59, 6);
    T('-2.43129018380640510525339593994882802918345631310268E+4054183539', '8.429525128929345745894603998540035724E+92236023203702', '-2.049462170011533971143677829148969399e+92240077387242', 38, 5);
    T('7.264706125169563561777455246002520371321280951811E-3445793', '2.3808141459441940634213471219457707352779E+66254', '1.7295915108931129847242146592740669873377060619474413292309372309471805804e-3379538', 74, 4);
    T('7.88731667688675665002559484578994926924462E+986813899', '-3.2794042186551713735124934684643973111027811116709414396597E+2363119864', '-2.58656995840517169669574e+3349933764', 24, 3);
    T('7.366269488328176470657612813572783174488002E+8227008576623470', '-3.500545956599824029001460100373406202E-4038281517581', '-2.578596e+8222970295105890', 7, 1);
    T('-4.2751257043512291269073668141965847205825091100255771E+6719060538693', '-9.45789746450757175427740146813644537509755872084037E+2186983319481', '4.043370055963463677817408370176278467338949436125e+8906043858175', 49, 3);
    T('7.94E+373883063', '-8.439545792780456E-754673280945', '-6.700999359467682064e-754299397881', 19, 4);
    T('3.809239E-4011448', '-928137184.4084041316251383830632460', '-3.535496360198684945e-4011439', 19, 2);
    T('-5.759971373E+96257', '7.955824384559493285E-83055382814', '-4.5825320703678024536985730305e-83055286556', 54, 0);
    T('-8.7727135817315260064422689548640531E-40792', '6.170656937372585009747472311709017600E+885', '-5.4133405923e-39906', 11, 3);
    T('-4.08934136482419130897475847910099E+751733144', '6.98871551039982292423606521815868143679688E+756386884887', '-2.86e+757138618032', 3, 4);
    T('5.75847214841721161129864881241771749158E+13186997', '2.511180821174335780017617307281854380907424085137748531204E+3660946353686688', '1.4460565e+3660946366873686', 8, 6);
    T('-9.7E+290133087946711', '4.64796671052212089604079337155681010663900872019522639E+900176537', '-4.50852770920645726915956957041010580343984e+290133988123249', 42, 5);
    T('1.2509894349202837472E+472146', '5.0308388738573735813059982046510398836602767951957E+931244481', '6.29352627998183242359743555259598844918432696881673424310570575e+931716627', 63, 1);
    T('8.809420366932780668200879619527844106519450982362604974E+246', '-3.296910317565045441731986073551095E+18767', '-2.90438688995083330531259217408e+19014', 30, 4);
    T('9.2800849026880E-20072698', '-6.578979541644189764302775439585418423453908651612438173E+432', '-6.10534887195054636123803274714333641533e-20072265', 39, 0);
    T('66.082809959740311809887867858597785635906167195', '-2.588616816840170227510160E+62876', '-1.71063073165836863622544814777e+62878', 30, 0);
    T('-1.8414553481538495242696666448168008422E+308', '-3.50671217461446325696021637315578597882290210903', '6.4574538883800192024939720256502145e+308', 35, 5);
    T('7.2322168731085993396732486271E-12100393', '78747.2110668242751370783570932', '5.69e-12100388', 3, 3);
    T('-2.3136792966E-2939', '5.80704886176268699568589377404922618636618653020E+709039', '-1.34e+706101', 3, 1);
    T('1.988626063240325E-74', '-1.302558469781700E+506003490658241', '-2.6e+506003490658167', 2, 5);
    T('945362757459017546895749.23585227', '8.37618480475065042859449150613147916573901963013E+6627764146019341', '7.91e+6627764146019365', 3, 1);
    T('4.30468957472E+152407586872', '8.64158226057560036066939940334692963568151E+54354', '3.71993290661850773389786355321113162780396038192e+152407641227', 48, 3);
    T('-8.79721E+7806310125899122', '-6.5E+492497671628638', '5.7181865e+8298807797527761', 36, 1);
    T('1.687319755365963111149981E-1827211', '-3.209921676648266702894853643722713349885084E-44408718501', '-5.4161642581860555176371652392215630111766520562587731e-44410545712', 54, 2);
    T('-6.619389541614949468878176276684529555662050590514E+21945', '-8.93241796165362443517E+576', '5.9127154036703526332223e+22522', 23, 3);
    T('-8.3470193839196739238663823E+97063289135', '5.5579057E+313', '-4.64e+97063289449', 3, 5);
    T('8.7789E+534', '4.58162148426783378620576148794050530995860892431502E-6880371291', '4.02215968482388860257217595264809020655e-6880370756', 39, 1);
    T('5.7435285891988859543E+58141005936623', '6.187596176218857061261643255808183956042478292833602724032E+7306923', '3.553863553653071342259109e+58141013243547', 25, 1);
    T('-9.078552718000363728253752281850E+37474', '-8.39070276E-65906', '7.6175437347731153615662649251675092906e-28431', 50, 0);
    T('7.4315424396682665457229348655183E+79263332627', '-3.343094724551282612716016099580885394E+609', '-2.4844350325333950331882326741069324740673160356748389e+79263333237', 53, 4);
    T('-8.068742907217996134689096475143636256785383E+739', '-7.7087881606295E-2730969', '6.22002297943253408021108e-2730229', 26, 3);
    T('2.31543298804E+39804441378', '-9.94267846847298211347337165805997481556903710573501E+87979', '-2.30216057153773679110091232811952566367401635e+39804529358', 46, 5);
    T('-4.041132518E+908656054', '2.92990297E+787', '-1.184012616665177846e+908656842', 65, 3);
    T('-3.79361451305E-17602989409', '3.197786736168496E+898111778970', '-1.21311701719675977757908728e+880508789562', 54, 0);
    T('9.3970E+49', '-7.99878172458136238905E+8389341', '-7.516455187e+8389391', 10, 0);
    T('-4.709547144712063518905777993296706E+848247879036625', '8.467766604280591053863356010E+744962251728188', '-3.98793460332778234578709866306141403434788257322099e+1593210130764814', 51, 0);
    T('-9.92525517659475751371246242153E+38559005475401', '-6.9565575E-7182717', '6.905e+38558998292685', 4, 0);
    T('8.9E+1859', '-3.02812403647E+500912', '-2.6950303924583e+502772', 19, 5);
    T('-9.666318922656984023565036682303262896154918092E+2548992', '-2.503949034483104582412992124478368108625771990673257746E+857784346035695', '2.4203969933392718825760113e+857784348584688', 26, 5);
    T('-1.8418612895894553984481073483326427936816178932077239E-751181970684111', '-2.6606428683290176091246059851144734024045302E-858373', '5e-751181971542484', 2, 2);
    T('-1.38769396445E+289217', '-867695449.4163', '1.20409573813e+289226', 12, 3);
    T('2.6953256301060622044518658625071067569898248142432E+28916434', '4.674337E+97407962819', '1.25988603198530804865709213201538818771475467527351167584e+97436879254', 62, 1);
    T('-3.1055324062052116264540629549399808983119983562423897287E+81', '4.56212337E+7873058652712', '-1.4167821966641128977e+7873058652794', 20, 0);
    T('-3.4037656970222898311613E+80800855858', '-2.415541123943153712206535057132155662157202E+923014', '8.22193601742437398742268394e+80801778872', 28, 6);
    T('8.279410453925175667513543770927919E-358303', '-7.90228299016136589474001E-63746111', '-6.542624439861710888597160084632868982321842238491895533919e-64104413', 65, 3);
    T('-7.877886E+9594362927', '7.59044048209056266972348977767773468893931029E+47375741037299', '-5.9796624807694494387937304e+47385335400227', 26, 5);
    T('-4.600407371112087856768187373682597611694E+95965562', '-5.03978922013378681311609994704189575394074065945E+278543324079', '2.31850834771547136334501163171159436800024137242806563354e+278639289642', 57, 3);
    T('6.04222586378E+1150254021870', '-5.425321119882694152492071950951028911283E+7894897809280', '-3.2781015589867088607814964483437e+9045151831151', 32, 4);
    T('-8.989363573220305369670322505734E+873434653', '-7.314132836063342900532575723319568838943265565455584112E+49333540256978', '6.5749399286202338128922477623e+49334413691632', 29, 2);
    T('5.615277625501886691861E-820631432280227', '-5.86E+4425075182', '-3.290552688544105601430546e-820627007205044', 58, 3);
    T('8.5073903102381676698220E+3143', '-6.1324826774763495320122200602690588974055121675738596E+4596493647045', '-5.217142370806571037234e+4596493650189', 22, 0);
    T('-2.208703674911062784420186197372262593835555136955054754618E+846943', '-9.65785145098024496083838E+257112462', '2.1331331991525206980870313405205083341093398596553112e+257959406', 53, 6);
    T('-9.589897793597085562012203349825893E-77065366', '6.1954357129302615134693157154569995569019266E+1775363206098562', '-5.9413595273802501665497671484e+1775363129033197', 29, 2);
    T('-2.10832628725801222644136788743690197843E+780', '-8.464248E-143444574513', '1.784539656027105547163189525850202269712e-143444573732', 40, 3);
    T('-2.7806915645120459004E-196781052695', '-7.064709331969219261934932202480064698879693160000224E-81150991552', '1.96447776451363389602812336744107515921809e-277932044246', 42, 2);
    T('-9.3179903716533E+8642', '79462.9626114561458442031862371261344497927277311', '-7.4043512051659453473912125560507898508433452358026406626482763e+8647', 66, 1);
    T('-2.3738799649162550409753288440057102750728831473416256E+65279574644', '6.0098207560561614469300626441220611191E+2125501639579773', '-1.426659308553958188071114499963536535897494118446933807033e+2125566919154418', 58, 6);
    T('-6.41567007442113636427375466518E+8209', '-4.53188847017376495460166796011976587961006682100101076434E-414530340533446', '2.90751012387080084328e-414530340525236', 21, 4);
    T('-9.9857532183837355771064274125274188117583974523008957966991E+951810', '2.179452317532502E+661554793', '-2.17634729941140730589e+662506604', 21, 5);
    T('-5.1921149E+4603177927', '-6.36689312026146250750665897143588914224412139019782563937E+302', '3.305764063641703138101668589481095441019392210745e+4603178230', 49, 3);
    T('6.030492041936201670948884E-8315960', '6.1453691018E-7744502358608563', '3.705959946316552359550280242e-7744502366924522', 28, 5);
    T('1.138928651642782080422E+23313372', '-6.27052548976977692597423543487413035921292185840315650552E-24', '-7.14168114115518775452416051489416258681574391796243213958346709494980912692944e+23313348', 80, 4);
    T('-141.871482995648521099', '54232126.2822426641880883051587030912452816743', '-7.69399218166905338096153226364526335622948192758666587848159e+9', 60, 2);
    T('-4.50018603420710545308400495459480567016422600630679654212513E+989', '-7.8342997319517109130841163820186127883080610463267571025698E+8159', '3.5255806241521559209048286065634801256206303435e+9149', 47, 3);
    T('5.021E+49504684535893', '-0.0003322455126521425577057165856444535840482052044537244104539', '-1.6682047190264077822404029765208014455060383315621502648890319e+49504684535890', 64, 5);
    T('-7.7065478877799263673209578400923761430214205836115544796125E+6322', '-3.00509236164643982764284356992919657659169E+33212370947', '2.315888819222996146387646621143911478854676040128e+33212377270', 49, 6);
    T('7.730488032433581389778961500E+62175825051958', '-4.29935341837194105E+461362154523094', '-3.3236100147926698841923148707e+523537979575053', 29, 5);
    T('-1.6187857359592039033E+1546500441048', '-5.6712067E-34', '9.18046851163626810306111211e+1546500441014', 64, 0);
    T('4.06663624203057917982E+7481361430714', '-0.598122', '-2.432344602355814080192298e+7481361430714', 26, 1);
    T('6.50E+6', '7.9222855045775774316058615099511413839625035000153563044682E+3610323373738183', '5.14948557797542533e+3610323373738190', 18, 6);
    T('4.23232905755005924359533374488466694660E+2827636', '-4.37313569294366156486586618654460226324105E+493758013610', '-1.850854926585477241635624193977376935735751883389657285570591715992410913e+493760841247', 73, 0);
    T('-3.4720471070317970E+48368143383', '-4.027452305661605379557031302218277686684422E+9860495795792', '1.39835041265809175800297495406291886488e+9908863939176', 39, 3);
    T('-3.8065878646E+4355979455543', '-8.20123142752753307996417524722958263938012777743E+498765101812', '3.121870802680244180456669089885e+4854744557356', 31, 6);
    T('-5.10026856729012E+86', '5.969046553523777309150472988476353912082887635E+68582806', '-3.0443740514e+68582893', 11, 0);
    T('-3.645063741874136907392852794038648890218671E-358591487424', '-4.1640875847025838134088593587554919660080604E+84678', '1.517836467298763717067503342860169917021226069840245307225994258538930904922e-358591402745', 76, 2);
    T('2.09090545030356835786940590902882097559883775164660892E-50406834080', '-1.09381002446975471795198023776151130255782764048237219E+8574629587177228', '-2.28705334176048961287838645197848484402157901932494141e+8574579180343148', 55, 6);
    T('5.056818623370318571596541170355090E+56195293916523', '665459518.4', '3.365108085744163181509210206330270548388656e+56195293916532', 68, 0);
    T('4.197239114209658587548190341752641077497358948109E+881', '8.491181151128210779072400E+488583286922', '3.5639517653355121e+488583287804', 17, 2);
    T('1.0312004806181526201381554964705111697238522044327711E+9285992793', '5.28423845773200229761093491219445706053411521594E-703134', '5.44910923731416632873934200041443416114e+9285289659', 39, 0);
    T('-6.1440092269930864404283844E-982972209426036', '-3.860544637755779125E+7644', '2.3719221875590189411241980629e-982972209418391', 29, 5);
    T('-4.3298594892186005618E+48', '-6.85629834266349E+10841961366906', '3e+10841961366955', 1, 6);
    T('1.098005503787150598696532106061220990598E-35907', '-2.83405024786954702038250098474660234580937513617864121751E-169656477', '-3.11180277017010100384380839706979323808240924760555569575999064849223083e-169692384', 73, 2);
    T('-8.145981607E+9695457549', '1.08060843418544521035415288697046219220911178E+372352', '-8.80261642924370671065117537332733497002432325768703046e+9695829901', 78, 1);
    T('6E-943', '3E+3009271', '1.8e+3008329', 45, 1);
    T('-5.24096767256404714327778885976562853051038165E+7866', '5.784E+129', '-3.03137570181104486767187307648843954204720474636e+7996', 61, 0);
    T('-5.2615704975466455470E+181719208', '-4.9458432316890514099E+8174', '2.60229028333458715550090029621759067153e+181727383', 80, 5);
    T('-7.697522054590552722566065111359800413348470839035E+409', '5.97872813359917836763312262910208188536660992339844046265473E+916024', '-4.60213916667806880579150050671047853818495689643766999760234e+916434', 60, 0);
    T('-8.65213646625717273207053454902395193929818411E+4774228418', '9.6708998200750163679697E+661318438', '-8.367394499479097970116664871484049428e+5435546857', 37, 3);
    T('3.46576E-550766423', '-5.25705527476637593085709481761305169009984268183312E+64', '-1.8219691889e-550766358', 11, 2);
    T('-8.91316399940012E+704003720112290', '-9.88780E+257', '8.8131582993268506536e+704003720112548', 47, 3);
    T('5.406028727835898912311923564628491188038994264398616713155E+5758082', '-2.23244005779836573113710831744025E-96', '-1.206863508562959973229478763047612110781949409972670859e+5757987', 55, 3);
    T('-8.28577516540471018739625156443E+314', '628480089.254052296642803796240803607933283215746', '-5.20744471549256e+323', 15, 6);
    T('-4.760276191629714719033756316283504473160391265671226479062E+217497020', '7.2776E+1657721757334', '-3.464338601220441183924006496e+1657939254355', 28, 2);
    T('-6.105075275621124729927777990158101417340466E+27829', '-5.268170725970745497186846617653920535E+47930', '3.216257884687498982757500474747536344754721992049462872504e+75760', 59, 6);
    T('4.026360801E+758440', '-4.09583825E+47684739776', '-1.649132257703643825e+47685498217', 75, 6);
    T('9.3118305959875201402090744613150748875166E+170227372', '-6.5668E-2892460438927', '-6.1148929157730847256724950172563833771e-2892290211554', 38, 4);
    T('-6.2448583490126E+83086639967251', '7.967074419346403631772009336827210205015475E+5271251017356889', '-4.97532512e+5354337657324141', 10, 4);
    T('9.544184552211041885872025E-39755658067', '9.009543086541777638233794263057315529783387919584171E-70', '8.5988741949051824201279065463883156826907e-39755658136', 42, 2);
    T('4.798392277607875127605859893E-914991278813312', '4.577124150539493775652329E+7546091', '2.196283717760121224365676395614707558991166905e-914991271267220', 46, 3);
    T('-2.553387E-286810744', '4.9886240097509911066670962393827890E+912257347679937', '-1.2737887694386053928879376865388901456343e+912257060869194', 77, 1);
    T('-2.0851556299092950E+583303603399941', '7.72121922296981548365462009671558300292227775E+9497515', '-1.6099943733e+583303612897457', 11, 3);
    T('5.134E-47878020', '7.637125830E+4249538', '3.920900401122e-43628481', 26, 1);
    T('354.21160072', '-6.07890061373562518500470302935589E-902778304305', '-2.2e-902778304302', 2, 3);
    T('5.947E+9080260037', '8.172506533589861977035278351649442521779932400E+251108', '4.86018963552589091774288003572592346770252579828e+9080511146', 59, 1);
    T('-8.62087144400582537909414632423738993932511623564939355E+12301971892', '6.5177388719170908786762856E-2746832253', '-5.6188588920396790591298829717104285356274894418943963e+9555139640', 53, 2);
    T('-6.192803646034E-40484', '3.92363654E+761272715', '-2.429831067062e+761232232', 13, 1);
    T('4.7633255E+23', '-1.40269366168787661490296666889932012821601958234E+47333400232', '-6.681486487406e+47333400255', 13, 4);
    T('-6.33017412846213406836E+50216907955', '2.61969199169835E+9310720955857', '-1.658310647038833486826247e+9360937863813', 26, 5);
    T('2.2019428195E+55', '9.032529951342855515953811421053795288858992204555132193979E+5942832318279431', '1.9889114468278085085980462752246495659525e+5942832318279487', 41, 3);
    T('6.1613355366626879052402E-56981923478', '-4.847E+650', '-2.98639933462040482766992494e-56981922827', 28, 5);
    T('6.5316511478327824405445610890329128902539655E-80465', '-8.70733637056430752359415077012524286539732251227318E+89564771030', '-5e+89564690566', 1, 2);
    T('-9.48878789867719279937128653531115605633060E-3322673911', '-4.986186337878433998015692904884024855E-419758', '4.73128645834104329998978231406077127492113e-3323093668', 42, 6);
    T('7.817861941059342637753654248334057733555299E-297884241', '-1.208328232168509533794555705025583716126395365E-49230', '-9.446543298577707967517199648974721769995366453939906649771607668415661309074e-297933471', 76, 2);
    T('-6E+2140521220897', '-5.92547822235365170626E+6360', '3.555286933412191023756e+2140521227258', 66, 1);
    T('8.3984402641877944538926195914E-6477408899', '2.77087352793E+45073661411', '2.32710158039393952545031305686e+38596252513', 30, 3);
    T('-4.5502024948829941305738603474226712558620E+7134', '53284184440085361714428960002254691971669706407152270.1248388', '-2.4245382897708202855889716128e+7187', 29, 0);
    T('-2.022463845186E+2909492377074386', '-4.234791550716651981441252969106911944707446274E-5131', '8.56471280322358370034584e+2909492377069255', 24, 3);
    T('-2.81853592194606962637846954E+3164692536590788', '2.0723643149225553475721786130018E+4773029867', '-5.8410332649683795133816e+3164697309620655', 23, 1);
    T('8.7748835927240006844489453107588662215604053682E+5168', '2.3085154760030716780583411611996609763587315605E+850324181298172', '2.025695457392879019408125095293024992217313610836153818670227588477e+850324181303341', 67, 5);
    T('-6.759223888116409813067442199873070963168916640E+63', '9.7539562243199873731943918049949721053842020763945631100148E+704667926582833', '-6.59291739150654e+704667926582897', 16, 6);
    T('-3.441190240238556895768951798213046310198872E+1275292908', '9.326985830902696962794213568161214511138539328662621708487E-4974948944', '-3.20959326121456679647462358203822969442045e-3699656035', 42, 3);
    T('1.270848249161999478630481670945460E-5259074635298', '4.38224244763708309E+33272', '5.5691651419829822242e-5259074602026', 20, 0);
    T('3.96033341042512E-8400120163', '6.215717133628513350556643026348339813773237820E+38066', '2.46163122340608616186886690702794805459e-8400082096', 39, 2);
    T('9.757406574609804850904939118311784193271131508E+1773', '8.3789069E-82966', '8.1756401274103458e-81192', 17, 1);
    T('-2.8E+88946984671103', '5.789951688751864391835897607696091799957109055E+886951407', '-1.62118647285053e+88947871622511', 15, 3);
    T('-2.41656309850897193768910829004781E+3184115662225', '-6.0371073242947937983846599731546E+4674172627545', '1.4589050781629035779579050364062399778055656650292982904950521426e+7858288289771', 72, 5);
    T('-3.72231638624868583882702928591922869431E+467814657', '-8.604098483086493592701391114102114877E+720264', '3.20271767e+468534922', 9, 1);
    T('3.78232108380267453791159333065183444063063846822704062233E-692828', '-5.10007690717447237754643283865879551461371104761350103E+681019059230', '-1.9290128415021142707774866425360755836683197101e+681018366403', 48, 4);
    T('-8.695301055278435070E-8275', '-2.5727433627248584488533192891805982355266077149105E+288869334861', '2.23707780768620513233322621084104712989210006e+288869326587', 45, 1);
    T('1.7601955585443429868270784453837E+249159153430', '6.903303336716147451135E-202242', '1.21511638725721058076423e+249158951189', 24, 4);
    T('4.12197E-827491881', '8.87E+1612398480900390', '3.65618739e+1612397653408510', 51, 0);
    T('-3.95323198E+9', '-1.7429726E-1908', '6.890375e-1899', 8, 1);
    T('-4.2419726966400594164897249574182E-5720138', '-1.2334205E+88', '5.2321360844761304055164648018412349531e-5720050', 75, 3);
    T('-5.58608969E+1071809946888', '-8.4391596742081158203963179987585954436119878642320769E+927372069017', '4.71419028483e+1999182015906', 12, 1);
    T('-6.40484225411573942425216410622100572411895121078E+74845', '-9.0627299465379833E+845773506695405', '5.8045355699226551601951510378154712748216560792829266982794419974e+845773506770251', 78, 1);
    T('7.354849690468445E+88444818960260', '9.9020708027531224049352220612E+9225328572078', '7.282824238e+97670147532339', 10, 6);
    T('8.14495879002488549600E+62190134355', '5.7392142149077457087506362825145405809855145886998484697763E+226', '4.674566326754861564196150475356386e+62190134582', 34, 4);
    T('1.632886527545386229616320801109663184062189472936198592269E+34111886210202', '-5E-285938595', '-8.1644326377e+34111600271607', 11, 5);
    T('-5.62774E+997', '6.0581433075124332E+215458018', '-3.4093655417420020816968e+215459016', 37, 6);
    T('-9.122203731770447748874854801E+938444334959', '-9.075526180616654609418810945234737250591E-37', '8.27887987926016452744895353920381727253486e+938444334923', 42, 0);
    T('3.223120415972041493728E+320625', '-1.972641526161058234779432159509E+2330254647', '-6.3580611763639527902009274514611884331757691190595e+2330575272', 50, 2);
    T('-9.781944233628249485724463549077071233410474669E+53100048299', '-8.6598889735772801823476E+523842', '8.4710551008945136053621173004062577742048279e+53100572142', 44, 1);
    T('-8.83870E+6071143469313000', '3.45916754778985638884434874490475047E-888623320872163', '-3.0574544204650203664078545251589617979189e+5182520148440838', 70, 5);
    T('1.59157E+6601', '-5.06E-5825447820962', '-8.0533442e-5825447814361', 17, 5);
    T('9.1806713163643371420728776815007621961671057324910241997E+302', '2.3796110598251E+334853625', '2.1846e+334853928', 5, 6);
    T('5.257003057179649E+1930', '4.7961426963067161387588709509311E+2255486', '2.52133368171542515903354260225251821038365211839e+2257417', 54, 3);
    T('-5.04474003341332705285218102645314435443638E-959', '1.47139276873517336158E+3409623053', '-7.42279400531320626904713e+3409622094', 25, 5);
    T('-1.320350173629041307915218355417657399636434752974540636E+851058313', '-124862960.72292854671993088667641169429095018871843', '1.64862831870354871851616840232977678278878508122565487013673481206611808925e+851058321', 76, 2);
    T('3.72373924737720966196632E-31', '5.6412895375154924266434157E-29776664102', '2.1006691256664866939123712102620178745793143159224e-29776664132', 56, 0);
    T('7.555946453908640E+739772415', '-20386788325283588135381955.674751', '-1.54041480953012589334537851304e+739772441', 30, 2);
    T('-8.81168852567640232689456444730E-75', '-6.5308841182056444130568453045E-15522490', '5.754811e-15522564', 7, 1);
    T('3.943319164831657897343402567029737320114820042678991E+7430935537971155', '8E-5858', '3.15465533186532631787472205362378985609185603414319e+7430935537965298', 51, 4);
    T('-3.19479967785155354196215092489684811800838414706601511931413E+2056012370', '-5.49953303905371E-62', '1.7569906381502767932195728839713411018128e+2056012309', 41, 1);
    T('-6.125E-851', '5.2367971253060975E+6662925', '-3.20753823924998471875e+6662075', 25, 5);
    T('-1.425326096457373E-64065518396', '5.216467325E+92', '-7.435167009639684509837225e-64065518304', 57, 0);
    T('-1.80480377054649911325737430340492853080224143634E+3006', '8.89451224983459078651343E-1731532', '-1.605284924e-1728525', 10, 1);
    T('9.19352141122373814692191E+7598408294683', '-3.6113575606858000354219254102999498903596866E+687126921', '-3.3201093057749632917816299715661431078956e+7599095421605', 41, 3);
    T('7.9086045560325172576E+34', '-7.24444468298152028119347152983E+69469108', '-5.7293448225753196433578550489293020454653748194208e+69469143', 79, 5);
    T('-3.693628196684190508674838308729149438E+1388', '-1.0673884158659866055400E+8015049075428', '3.94253594965667890678585303472569663350967144306501868652e+8015049076816', 68, 0);
    T('1881818.199145546726181016', '3.688640562211588207632566743302227370954E+56808586', '6.9413509400762279e+56808592', 17, 4);
    T('2.86511372208616334392261490553960456079047113287841E+8102231', '-4.6834599439E-646513069659', '-1.341864535210878e-646504967427', 16, 5);
    T('-8.151579633267251025461788249185234286843830644060E+66696155867', '7.2853321893339599493564290224998466910758750107787484491E+810948969552428', '-5.93869654961610202568408539376396619329604911615319327e+811015665708296', 54, 4);
    T('-4.83127213444463300381433034485477020908769807309090021627165E+4272352', '3.3732210067E+19745', '-1.6296948652992982686538620345757366129996e+4292098', 42, 0);
    T('-1.13831798E+16447992552', '-2.776742209332559E+458657', '3.16081558270817570911082e+16448451209', 68, 2);
    T('6.4457582106251377254892702E+651316407', '7947546358.90209560983424574911278', '5.1227962197218e+651316417', 14, 2);
    T('-2.9486198982609503226158952652420157390833416215561E+77679300835339', '9.5E+845953044', '-2.801188903e+77680146788384', 10, 4);
    T('5.297938189505304580340247904843643769745E+74535', '9.9642424861883468926793194234655466714778455631328806914003E+2346050754', '5.278994079706852541739712196e+2346125290', 29, 1);
    T('-6.70521E+42016658997', '7.7510411490359203961771369713754247313991305774758367383E+61', '-5.197235862292714379965090059183621166322476433939675525602e+42016659059', 58, 3);
    T('8.109183425934E+414', '6.784852192555406983021771199443265771048E+2194158124971', '5.501961094728226664869615354681003431873e+2194158125386', 40, 4);
    T('-3.90937564251E+7518', '2.60287729986427856020996124340019980E+108210058273404', '-1.0175625116531607932118434956419854590187373498e+108210058280923', 57, 2);
    T('-7.918795355823660838988686606126820314153473000390725E+68218', '2.55E-196299', '-2.01929281573503351394211508456233918010913561e-128080', 45, 2);
    T('5.1436945606468714655950727042632244051956772487179E+735719', '-3.8982230710039778907582935E-2904602062', '-2.0051268e-2903866342', 8, 1);
    T('1.2022036344880222E+3855615627214508', '-4.545020788981202532058410593723E-778', '-5.464040511336819886215e+3855615627213730', 23, 4);
    T('2.85372805', '-5.135224849E-1822367838182', '-1.465453519464831445e-1822367838181', 39, 4);
    T('-8.4659608096875E+8621914457', '3.72927786433045132822803030090177579409984E+886253584', '-3.1572e+9508168042', 5, 6);
    T('7.36672605361939087389041119033069445752904801099844647E+840492005678', '-5.62312832334648069580309505865914675089615311421E+519699802818', '-4.142404592244164185382972747008420813022921894787945546249530258e+1360191808497', 64, 0);
    T('7.335E+73', '-7.2024745092470972978442227181048E+8879869523', '-5.2830150525327458679687373637298708e+8879869597', 35, 6);
    T('-7.052437308873564546891712078187743713949168142766736E+4097558115921', '-7.771406321609267135523457E+737001032951768', '5.4807355884933067187151906572053593773457218711413233550187705924529607326352e+741098591067690', 80, 2);
    T('-8.27410645014604471903073033688666959922038E+1342679457584', '-2.8886329260075335905110961856753133265158294081437340521726E+17505059499331', '2.39008563251831760139012064445338993540800888944548768812393382603815e+18847738956916', 69, 6);
    T('2.708088909303688999390009502937586765877259E-3467868', '21671.50824683212822943886081379704525091920360301640216527', '5.868837113112951949776197436573123313042104149709968e-3467864', 54, 4);
    T('-1.6389165816863444051433422025092869722E+861', '-4.057126600659557214977013226593281053574E-68544972677', '6.64929205982169999906492e-68544971816', 24, 3);
    T('4E-74426263', '-4.0E+15563395390809', '-1.6e+15563320964547', 48, 2);
    T('6.4428E+6198977539', '1.83E+78647737873', '1.1790324e+84846715413', 19, 2);
    T('-3.05001295985100288342122974926012130770387526E-913', '-6.6178E+5710830695', '2.0184375765701966881905e+5710829783', 23, 4);
    T('7.3384644675007087716842884237387207520205038477E+681313602569', '-1.27982268587183761170238849041041825844337275720617542E+69889636', '-9.39193330497180167448869603835399851137671823318356731778e+681383492205', 57, 6);
    T('-5.83600788270790198799986372063549127291019718220082542E+661527205', '-4.12547354781016421095E+43923677021749', '2.40762961449230531007541767825245632356765602014e+43924338548955', 49, 3);
    T('8.376356112324118725102899805713E+735461309994084', '-6.70452382919252631382330065912330212962319922E-894651013', '-5.615947915687952352945452817185e+735460415343072', 31, 0);
    T('-6.2101481997794627174924497585746446198', '3488.981113903041405611834752884877886643153212032006113', '-2.1667089783569517146005463119508381627256296169703336390190615e+4', 63, 3);
    T('3.3619900070172009236787218751400407690850748160035388E+199587159835', '-6.52823523931744891034656418305020653318546267139E+33903046378', '-2.19478616380428084134264297644e+233490206214', 30, 5);
    T('-9.950294418337149363735342E+7661590695025', '6.387867989166138256891951942535653458034475357E+93', '-6.356116719767437560034e+7661590695119', 22, 5);
    T('-8.69E+324580619232525', '5.69568376472255277299471271921E+137697614580', '-4.94954919154e+324718316847106', 12, 5);
    T('3.030176109597812883306949451E+412854053', '-999256372.476', '-3.027922787240148709076720603247759711e+412854062', 37, 4);
    T('-3.66075940476275959141492174710758906056723628E-299784496585603', '-8.803550006845595146542103184823E-9580304', '3.222767848285926901490314001949658898737856345352812010316971031031515e-299784506165906', 71, 1);
    T('-6.4704355360967557235827917742230280451667811964012221E+2873656913138506', '-6.8016654990094E+851', '4.40097381494336946249631649846354457e+2873656913139358', 36, 0);
    T('-5.3993467150275142E-84661162577564', '-4.870102060687E-18198', '2.62953695632190810553231542554e-84661162595761', 47, 2);
    T('2.670691095545172879E-27', '5.076200548861449401393477911017617752924597E+3137681825477', '1.4e+3137681825451', 2, 5);
    T('1.51856008964019789815E+4875920571', '-7.6173282578885710466313732892491410928747488437351894196E+9189', '-1.156737068211808094087422571327474925879478798e+4875929761', 46, 2);
    T('5.788143659762663E+60', '3.5504553283372857693369415524E+712915240467', '2.0550545497986024552e+712915240528', 20, 2);
    T('6.98202123968638E+63', '3.081769E+123632266033', '2.15169e+123632266097', 6, 3);
    T('9.49624311920182881241539912647780642620E+3093305', '8.34428210457212950606600593607536582917905E+771583299016083', '7.9239e+771583302109389', 5, 4);
    T('-9.632828885546543744266824E+934184594324766', '8.827427400306140530E-550734911327', '-8.503309764673402356386008333011562996077672e+933633859413440', 69, 6);
    T('1.85089306036721664881332588105967051564E+398444465956', '-6.15131176E+41985802113223', '-1.13854202487392496903e+42384246579180', 21, 5);
    T('8.74692821874847957583590E+5834233736', '5E+13483', '4.37346410937423978e+5834247220', 18, 3);
    T('-2.246220283097742859264387023336908713419E-8', '6.37544804471207632E-4549411405789', '-1.43206607118681112460097024623235396286072201792633e-4549411405796', 51, 4);
    T('-3.02391593696118346124089969465E+27993', '-9.59407349423684E+21676854568', '2.90116717395996494e+21676882562', 18, 3);
    T('-1.0214337443770748686893796351487360564385978083923628569E+168894279', '5.71134828784510929351709725093919312441E-30062538', '-5.8337638670952255834449e+138831741', 23, 0);
    T('-63428096463596449362170130182.632953823691222496573', '-1.06086862139698382586003E+385257', '6.72888772531704703364793131000402627976927399806907e+385285', 51, 4);
    T('4.4761289996144038785676781703412811850371475479204346667E+6546', '-6.6520451083328620648010165746102521879960903279108906691222E+93242022', '-2.9775412016151862747889708817871497548476377105810081916749316560717108199e+93248569', 75, 6);
    T('-2.6596513716631940260829115775963892E+20970', '-4.76669622392982918367959699196212930219652807487304342784E+9412838784', '1.267775015027673765603148622606509202441515311e+9412859755', 46, 2);
    T('1.33E+835746679262626', '84725.95972493366475396103771693123', '1.126855264341617741227681801635185359e+835746679262631', 77, 6);
    T('-5.17739667539934663051734795309338300236532892789E-890389382168858', '5.88158329430564535961191E+6772358880', '-3.05e-890382609809977', 3, 3);
    T('-1.12650910501217255E+96275163', '-8.21583141501773026415623318E+894086', '9.255208894262514497696925892300058856195209e+97169249', 59, 5);
    T('5.743476635524122008768749848574542310074606117733798E-404392712436', '7.752092354372E+45564', '4.452396131376076431515432004253534218464858099419356e-404392666871', 52, 2);
    T('5.34751120693390734110895785884332700980512170420E+8373847', '-5.553982924048409061164181822838112014532359512128775909E+453530460', '-2.96999859294684197662100064524971732836609930250881e+461904308', 51, 0);
    T('-5.53148291469306995083648160343225172E+984', '-9.365181499904854451981827121615226324857E+59357873', '5.1803341459723321e+59358858', 17, 0);
    T('4.67838954987E+571890021781', '1.1861329568600705214759421141260472E+85238829314', '5.549192030130557455544288995340334128450373864e+657128851095', 46, 6);
    T('-2.9968787844921626958406558570812479619297631E+519118074134637', '-6.45322774004079921877E+49298987484', '1.934e+519167373122122', 5, 4);
    T('9.72079136107E+7838', '-7.85770460638864051066395127049634096783462678E+742930584220', '-7.638310705562264140704397072e+742930592059', 29, 5);
    T('7.6125245E+649194486763833', '-9.41164579493812223182292029600222444444E+7914234715', '-7.164638419928843147374666041486418563779838878e+649202400998549', 67, 5);
    T('1.763233229629295360849721778147025875E+5448', '2.48517156E+228', '4.381937075921674173623665996963618203134115e+5676', 43, 0);
    T('8.812793365679501528269686239083930300102101352E-67322707497410', '2.441733092799287814558496526222E+42668244', '2.15184892009816542970502522726128824558152278e-67322664829165', 45, 6);
    T('7.5133544535277672239625618773991E+546', '-7.428862333547845215564571852270E+947651647', '-5.5815675898406384189893406517062886039284753068564198537030957e+947652194', 67, 3);
    T('1.504840977720987412762433174508860051647309381616E+696235353203479', '71423.7127038651319632900285954983228091091927945128992516821', '1.0748132965774731469e+696235353203484', 20, 6);
    T('7.406195265911698772111480200530602965728238706980318E+85115', '1.107069066627748545560142206822901094885005950838E+15983', '8.199169680295714304425282796559277e+101098', 34, 6);
    T('5.4230892933975608737289427286438041488944E-902929', '3.5271466637694899636426795957642E+626', '1.9128031308531247551319898807743945566022057066425851329808e-902302', 59, 0);
    T('5.673646563868062582214780578817126381650874963253039E+2873986676', '-8.6229552018509E+8980652', '-4.8923600151369594782297e+2882967329', 23, 2);
    T('-4.81581429170955420132060265980261712516119290585316791535817E+2980481626452', '-0.0865762219874728084', '4.169350071694904957123013120183336939e+2980481626451', 37, 6);
    T('-9.6669884059069703480115960655049337679327932E+643626238278', '1.76E+9', '-1.7013899594396267812500409075288683431561716032e+643626238288', 50, 5);
    T('2.1778759801582241928237493288838641948533486665086120574308E+644622555', '-5.68E+58255664273576', '-1.2370335567298713415238896188060348626767020425768916486206944e+58256308896132', 78, 2);
    T('-3.7122784095043751053221171732644291502011E+54495983652', '9.85356168077756814339101212356428735955051413470E+684621', '-3.657916e+54496668274', 7, 5);
    T('-9.0282786410612203444525035385260857245190206286E+98900787453', '9.458223379966268229396248977848798110492053E+2194449706839', '-8.53914761237353224491653001692351863771813e+2293350494293', 42, 0);
    T('1.66611158529230417202649298801820606826968656338581463446E+6519', '-4.58357173151574481540918307263975E+36350727806951', '-7.636741963896689186761675036906092774816806781755e+36350727813470', 50, 1);
    T('8.4900938996736586131059293545736378384905087E+95188721', '-3.4675617172273241669979186183438453654575378E+4010', '-2.94399245821736209236108599859909349802522546636770006826e+95192732', 57, 3);
    T('-1.06589470962858422181054544244325979287E+948', '6.1038700E+39685756182931', '-6.5061e+39685756183879', 5, 6);
    T('7.739879597015247960341717415715715991837444E+15708673', '-5.78236723072215394462506903774434046710753059820326E-2261524', '-4.47548261515159601981119374212169446781657624108267612282415e+13447150', 60, 4);
    T('9.997805309977080568924593571888E+40369727', '7.079599476521665067672628E+97407615407783', '7.0780457238879262951375287828542963e+97407655777511', 35, 3);
    T('-5.66490291488031644E-8', '3.882500473465927589082649650E-520329007240535', '-2.19939882491613418742071364256451e-520329007240542', 33, 4);
    T('-6.60454912E+4239', '-4.48876131993165005440227687409847826960348946E+545', '2.96462446254446178269505098548e+4785', 30, 1);
    T('-567.67093389547665845734022553609', '-3.09865986338022342E-514867', '1.75901913846948154273545171221287156561257348e-514864', 45, 2);
    T('-5.5646861254356268040899461827187E-77622329503', '-8.391088511054367268865553236319433136076528217675350164E-3032022', '4.6693773814766529732616767648598e-77625361524', 32, 6);
    T('7.438879562238649165E+151284868930', '5.9E+29203405269', '4.38893894172080300735e+180488274200', 66, 5);
    T('-3.3139588696009568052826520084977062081384085276154050054476E+538564', '9.1777127125732667149571937558644383973267E+429744', '-3.0414562e+968309', 8, 5);
    T('-4.55043901664908190095475308778420054603593169884813183E-78797260872', '-936038115.090379839708436853855624606516954697273', '4.2593843599779281879780648229028149166035450807320799380802559252940119178725e-78797260863', 77, 4);
    T('7.7E+3', '2.17326840891474762089374390133782E+344786518385763', '1.6734166748643556680881828040301214e+344786518385767', 70, 6);
    T('-3.301935513426665119E+286847684761345', '7.25307249805447570203571822673E+4471784', '-2.394917766278432976964538455119285112194822443087e+286847689233130', 72, 6);
    T('-5.382950485703207968178E+28747', '-3.67474815284509386725084316E-8551050199475', '1.97809873542e-8551050170727', 12, 5);
    T('-2.693179134595004E+4266316448812290', '-9.299999326E-9264153', '2.50465641e+4266316439548138', 9, 3);
    T('-7.2257245948950269360054434605203998950148222640E-94257785', '-8934.11348', '6.455544350601919933392932977e-94257781', 28, 3);
    T('-7.6218618632670683247212420441185663365186E+338956079', '-5.79715041135500632827755607580718037234840E+866894959437165', '4.418507963593e+866895298393245', 13, 4);
    T('6.42192585545753521842319737876855734466E+3116806570910', '-4.22950952463372278297994074E-1316665363', '-2.71615965721492133090977194525377650777056281089409884171785554484e+3115489905548', 73, 6);
    T('2.9835216357966841753121309461083038410562458012050871779804E+378311816388780', '9.823214362676E+99469277706514', '2.9307773e+477781094095295', 8, 2);
    T('-2.20292489428657829279268094E+576', '5.232492990087092503007371E+6267', '-1.152678906704287021110453680407717841174116767120874e+6844', 52, 5);
    T('-9.6714320335053867052000753E+28672853525', '1.942372337371E-95397', '-1.8785522044644621561098698780666204e+28672758129', 36, 6);
    T('3.6447728102574597351134180825655396373924389513062E-9', '-5.4267362955763423644676E+815035551', '-1.97792208985539420184520754086141900244638922105832005939e+815035543', 57, 3);
    T('-8.135666306580E+5097782083361661', '6.93856857528723589372210101161907179559655E+33045675', '-5.6449878573859159106095290946116615501168832646290299e+5097782116407337', 60, 3);
    T('-5.242556269E+24577946096539', '-3.8739376E+592', '2.03093358505948144e+24577946097132', 52, 2);
    T('5.07087873157582073973881822767126E-5376587954617', '-3.171078656909033617743765006220788056784229E+97196975852829', '-1.608015531747403764e+91820387898213', 19, 0);
    T('-5.109541281124138816262399225E+875120143589414', '8.96426897187041526271342602541175519E+210617463451825', '-4.580330236687212830627245823683303558543337397793132990774572775e+1085737607041240', 76, 6);
    T('-9.03163597437675781838E+71', '7.69143379287432974145890595994197542096101602137634743E-79389774', '-6.9466230138260869170116e-79389702', 24, 0);
    T('-4.6996188004254408048E-2928810499449767', '9.645249197293311928467433709557858046613909634261480590005E+7602003249', '-4.53289944623880404342828272576964743915829506381919559262972680498513060361e-2928802897446517', 75, 0);
    T('-9.5712747080836510627499E+86507097272', '4.484526265168486319643158568545097536920744094227004001E-50', '-4.2922632819543969857855750928169645978916e+86507097223', 41, 2);
    T('-6.0856603593239E-41', '3.2249274899487742E+78763060411', '-1.962581338727518010357634576338e+78763060371', 50, 0);
    T('-5.009E-210507342239733', '5.21430174486936367399961357463273474744171575250E+19802520765', '-2.61184374400506426430640643953353683499e-210487539718967', 39, 6);
    T('-1.50969957736949727038986291707599920141671001601776763E-377789820', '4.5911851002694438402174162573031254375E+8238823824781', '-6.931310205501912314005657491089012409e+8238446034961', 37, 2);
    T('-1.3250322389637660056643803356312768077559053206499644343E+31725477648087', '7.01269695082869388261932193364842708156424449286964510543919E+584592', '-9.2920495419309191391389391402766e+31725478232679', 32, 6);
    T('2.635039280654085008585877372575442614990502925004537E+21159', '-6469905.201723181291104083195589275912638951711561', '-1.8e+21166', 2, 3);
    T('-8.415452297414E-560039150', '-8.16278501015018134246851337843899418349315425536E+242', '6.869352786696490488764536963654162841452084805401080242363904e-560038907', 71, 3);
    T('-4.71519703E+3074693979', '-1.7897887990527E-4735606736511665', '8.439206829620557853481e-4735603661817686', 38, 6);
    T('7.541E+306687235492650', '1.9047691511475125E+68538', '1.43638641688033917625e+306687235561189', 36, 5);
    T('8.8933274852871870500619E+4232114862', '-4.2980500771135941625956698324994212644086605798412393612924E-73919901320925', '-3.822396688393504075601220438557885161799316828900263835088833713201e-73915669206062', 67, 2);
    T('3.67167459872241301118195935595010628485427556E+86701425', '3.79156923265064420745558743618632240120752E-94181207', '1.3921e-7479781', 5, 4);
    T('-8.0577794815444864491E+60', '3.47979867962492820168492986718345600E+976046', '-2.80394504005873424662869031870927831198315905757060896e+976107', 63, 0);
    T('-9.62310685702271833948577002341370427728E+92258', '3.5459898E+24213', '-3.41234387593126176001e+116472', 21, 3);
    T('-7.238675591186423965837553661387030562530737174727E+896978814', '8.000293542256703834721135849731563444353019599042E+770921', '-5.79115295866599755545240492924344496299507542717199582894851981389178e+897749736', 69, 2);
    T('-8.014757553167953616492742128690642663678051995435E+66461385126', '3.2421326E+325248', '-2.5984906744222055695319016918821327894861448278894864681e+66461710375', 57, 6);
    T('-4.53080340E+7686120551785449', '7.26834323349724814378743319E+13203243343628', '-3.293143423469632578052e+7699323795129078', 22, 0);
    T('-4.1326853E-5120502320', '-6992363587.2756087062817368605464', '2.889723820938917514900255158204825724792e-5120502310', 41, 0);
    T('2.88054935445187518930E+409407179630347', '7.882079414E+163345361', '2.27047187677361147e+409407342975709', 18, 2);
    T('-6.428877012465589143596576508450340588216184556383534825E+3478077829', '-3.5009409766496259151197777329107811075726484522258926077E+808099603039678', '2.25071189667816089352134637788911637531371098040008492141527112571e+808103081117508', 66, 3);
    T('-1.840496857074826626274E+1320', '-9.7063495657623363287250303015709377780E-69245741820601', '1.786450587e-69245741819280', 10, 6);
    T('-5.16516493034540E-4645143', '-7E+2887', '3.61561545124178e-4642255', 48, 3);
    T('-5E+82145960965', '8.5728045E+6863', '-4.28640225e+82145967829', 79, 3);
    T('6.0930230149215854043925E-8', '-2.7649369787966364877545493747016436898344E+1728235033', '-1.7e+1728235026', 2, 5);
    T('2.632637331961E+58083998', '-8.04552277136E+101358726', '-2.1180943603024661023e+159442725', 20, 5);
    T('-4E+78055', '7.100404342084259560824E+66567', '-3e+144623', 1, 5);
    T('9.9513774203818076E+384402769', '92.05', '9.1602429154614538958e+384402771', 52, 0);
    T('3.164620216477E+810024226169', '7.66E-138680768855', '2.424e+671343457315', 4, 4);
    T('-4.138853500743221615667031684837272237440498346E+58', '7437624677518357174620725673128445.7120416246481722715322', '-3.07832389337610273359792805560683196141871604411416908748016864795334321932e+92', 75, 4);
    T('9.3E+2107158', '-7.4623308820E+2633536119', '-6.93996772026e+2635643278', 44, 1);
    T('-5.8264999349010106830956823384468959493653619094882190E+66274769', '-7.964489322525063962270109094136275869069970445629E-270316', '4.640509651921207985450914525533602056404388704702918238037113288713694963e+66004454', 73, 1);
    T('-4.38943949144924268346349449384992472E-478196125665478', '-41.78074085380', '1.83394033885676469554053966080106902594958525936e-478196125665476', 74, 3);
    T('8.6516392903576480775517260382128E+551', '-5.063399780876345770685E+7769743153', '-4.3806708487018098898789932154313039116093577976031768e+7769743705', 61, 6);
    T('-8.6189802033E+758', '-3.77E+62820745', '3.2493555366441e+62821504', 19, 2);
    T('1.769632973954822720408839872E-646', '-8.5412286863086599094387506753E+85219775125797', '-1.5114839921380637441081263e+85219775125152', 26, 2);
    T('-2.90109065800564403E+96394', '9.96945068657470616468581783332455869E+134645410095', '-2.892228025226984e+134645506490', 16, 3);
    T('5.1273472243597744546993181722E+5335298244384453', '5.863505263386285066551511539E+4700689113', '3.0064227437242596984416931e+5335302945073567', 26, 0);
    T('1.3356162855495360098658996561196209359656735994561766044955E+5817163', '5.4762411915844295862311481197949903726431768517750948E-310782410673888', '7.3141569190773608413e-310782404856725', 20, 2);
    T('3.7412752050E+264768662', '4.693224742563955736471E-758031816991', '1.7558645360847e-757767048328', 14, 3);
    T('7.3662883290952607315424E+69089', '97623886956917940102239350670494451383441158589', '7.19125699131659671e+69136', 18, 6);
    T('8.503862843914810E+31', '5.3E+3230237650801397', '4.5070473072748493e+3230237650801429', 30, 5);
    T('4.5095456E+498903339143117', '8.34277E+301332311526', '3.7622101745312e+499204671454644', 76, 4);
    T('9.43596271596646300277375148435309950029525330263934878013E-14286712096426', '-6.7404831521819590678961194285328281245213274276852E+729677851919173', '-6.360294771158906424772759427608072864313e+715391139822748', 40, 6);
    T('-1.98103232920752549413059352139400E-7407327447', '-3.4160252845591567095217981989409901172825770820347274450950E+193391612370122', '6.76725652610202628968068470176452183e+193384205042675', 37, 6);
    T('6.0694367552061954795462726431406820241900537190E+4897627084811', '-4.23316282855294318861837511507019201492995339061269176E+5485144', '-2.5692914062391855891858193636619833081899274890667716167e+4897632569956', 56, 6);
    T('6.9888724696304737246402795E+99', '6.4611000216211072939418375851E+6226', '4.51558040646366153118300491096834593611925967105e+6326', 48, 2);
    T('-4.5873676589495954726163232E+3652', '-7.758256762813858114989491733634862456639621807581887725E+2283991876275320', '3.5589976163559275288266650668058474e+2283991876278973', 35, 0);
    T('1.76827638174267934456343654983911279569032E+6058', '9624630215592.058318813730663290689795167924268', '1.701900629323e+6071', 13, 1);
    T('1.194E+7249696169933', '-3.10878847179845322897090051285471451816151031E+9516', '-3.71189343532735315539125521234852913468484331014e+7249696179449', 67, 4);
    T('-8.785E+7876', '7.0484665209858067639597674206202486990594422961421E+98432', '-6.1920779e+106309', 8, 3);
    T('-6.02888690034012274571786725962745329493521042287650987796884E+26924637729', '6.556873325966410286E-1256', '-3.95306477021084625722425933305064074e+26924636474', 36, 0);
    T('-5.26070436920748512390317600784612151412396024E+6093257', '-9.302257E+36407', '4.8936424043390912946224186341218638777610208e+6129665', 44, 3);
    T('-2.078546E+8941417688084024', '-9.051386808744107491853940449266552141749185055952E+635960767763', '1.8813723845767829650763040505061195e+8942053648851788', 35, 4);
    T('-8.44644615E+7821325', '-4880645.03870927082483583095099547706377943003073857232366', '4.12241054967225215277419287180865859127730712323261959e+7821332', 54, 6);
    T('-9.92E+318117283141', '-1.8581997681958521657724896653630910961384E+123851892', '1.8433341700502853484463097480401863673692928e+318241135034', 67, 5);
    T('4.52758989022615880383123417907113554050020820615901E-6270920786', '7.8099138367513421E+3125', '3.5360086930812767707236057254687810619678997456546054099115103507e-6270917660', 65, 1);
    T('8.89576699E+9', '9.514170262E+479', '8.47e+489', 3, 0);
    T('-6.3817230252008259E-117702087', '5342110474.929426382060788206660126823720706700189', '-3.40918694210236397364669085377524913466704560814e-117702077', 48, 0);
    T('-5.4953892968379585619252545E-976276683676467', '-3.75696934025439078580740418901481510197610978411057494607E+94', '2.064600910098234566688387916309681355351772548506e-976276683676372', 49, 6);
    T('-4.9E+776794', '1.7E-7248869360242', '-8.33e-7248868583448', 14, 5);
    T('5.493964625616956415973687710107E-4777457', '-7.6365099310348241454234437519133142543288409836985482024835E+78794140397', '-4.1954715424277907295650973831563e+78789362941', 32, 2);
    T('5.795406883021838082529874713123245266854937856954125E-99365415594680', '3.7792882277488049903558806350388E+91503567', '2.1902513008018828444354981596100129489059006037397096262711791654856094759719195e-99365324091112', 80, 6);
    T('6.548118289133764679831647317E+1767726728419576', '-1.97641520059622400516E+846017450118095', '-1.2941800521946112659051181282756188238e+2613744178537672', 38, 5);
    T('-4.658533165927579038598023168023537135980722E-760', '2.6E-39007775247', '-1.2112186231411705500354860237e-39007776006', 29, 4);
    T('2.7532799336818E+422', '2.524210876260555630540718138280514439678132E+64336', '6.949859153989541e+64758', 16, 0);
    T('-8.9297785023875616561293575628102192E+330', '8.68E+806248', '-7.7510477400724035175202823645192702656e+806579', 52, 4);
    T('9.21471562289920E-1098972', '-2.429519081E+5587457', '-2.23873274318224069396352e+4488486', 38, 4);
    T('-4.6402045988234041338431315436816493572613331396567456203218E-5512', '5.980276705516891091162357857614965950294547435202288872213E+6637', '-2.774970747118e+1126', 13, 6);
    T('5.477502942803013695618520588743714065E+2486395', '3.7663E-4111744710575', '2.06299193334789904818080340933854502830095e-4111742224179', 64, 6);
    T('5.6589230341281898893270213726849755122517750342931E+6361042514', '1.75835607358413009048306611035221077E-315635975', '9.9504016870044361763573724841e+6045406539', 29, 5);
    T('3.63729549960297393554243683777322282151923966752E+852431799533390', '-1.94576742595144079860482487E+4104', '-7.1e+852431799537494', 2, 6);
    T('-1.975714E-7', '2.535202723174964958380256277600459690252955499281E+7337', '-5.00883551301490271779e+7330', 21, 3);
    T('9.083531755166050209691508677962491661802012E+29331204287479', '8.7127093645929242079929196803191321619295520001939602E-4098932256', '7.91421721868124469085837371301827430881215451351e+29327105355224', 48, 2);
    T('4.1590018319426331173169339649478074831162855674030E+650065412415', '2.3941993077708776545377360579425734580127163559996256473903E+321', '9.9574793070548642506615375284e+650065412736', 29, 4);
    T('-2.47019829027241068620668E+307316222290613', '-6.368E+66', '1.57302e+307316222290680', 6, 1);
    T('2.305706977695775928780458670745705420821916240413478000401E+391323718', '3.66167991010606496746488907003125384344E+53382677', '8.44276091881999554585e+444706395', 21, 5);
    T('2.6980227569263211539104544E+636579678135825', '-3.15008653683059795713519624823775266940047621866172671958797E-84175', '-8.49900516265617720115257865944238113059585e+636579678051650', 43, 6);
    T('-9.03770716995E+525706575826', '3.7825135E-462699', '-3.4185249379382669325e+525706113128', 53, 3);
    T('-8.708564079786376127098938940869E+1234291604461', '9.3892831531375514903072094710524329064520490421369209E+5155720141520', '-8.17671740023570451767019024442372430374327602517039158129041e+6390011745982', 60, 3);
    T('-2.6157401671368512485606541880039239825934300619305971847E-917390272697204', '-8.3960508385177941E+156339', '2.196188742363403481183550235810089573203058849980216317e-917390272540864', 55, 1);
    T('-7.90612144073916286014604007633862158929839089294893790533479E+1445', '-7.00916388233E+468148048791989', '5.541530085174376377769536509224538247056747e+468148048793435', 43, 0);
    T('-8.686962471540206508E+9397468378', '2.00935901295146E+55629317915', '-1.74552263373604048e+65026786294', 18, 0);
    T('-9.48690720470423126288196378037191E+89273357', '-3.27038E-12177', '3.10257915841206238375039167080526870258e+89261181', 52, 4);
    T('2.565612916261882818464217E-60859435', '2.04303631241568221913295110519162336446633034562E+109', '5.24164035152572157007920111145580506369e-60859326', 39, 6);
    T('-3.30141246429204473785013245393811251E+43', '38.54157864303321', '-1.272416481256019e+45', 16, 1);
    T('-1.926122617184501142365571910E-568', '-9.45849196389013172543624677617206126061E+81', '1.821821529610663259186530014601072834098734022464e-486', 49, 1);
    T('2.2300853948534433E+90785', '1.89225E+6437668343', '4.21987908841143e+6437759128', 15, 0);
    T('1.706401374321098086752796702690668E-549139487', '-5.43208388658415960205692274272806604262439467845713169E+8545479', '-9.2693154094947e-540594008', 14, 2);
    T('-9.1912385152402891646903306395462169680501071461478008978E+1543559', '9.7895766791563753245110229538841174490054482210E+3003', '-8.9978334221360203793347764097594503009332407983703272847918e+1546563', 59, 2);
    T('7.0632564898393980384021237E-4324735088', '2.463033199770287739580886060142191558356E+883936832620564', '1e+883932507885477', 1, 3);
    T('4.6075848236275161237353325786385907319305193787E+147', '-5.28419869049761762425E+55', '-2.434739369136921716246000216e+203', 28, 4);
    T('-4.07624858355659404875133E-974', '-2.62037405231025966295534E+377', '1.06812960191181484305020590817409157263845556022e-596', 76, 3);
    T('-775418733671456035760297016814.2726909504365833996879064', '3.07983242293723296486292943180348831367698E-3345910', '-2.388159757314281393651311162582679341118905454111148250354481557321566283576e-3345880', 76, 4);
    T('7.331253022E+879389', '3.429731046E+997', '2.51442e+880387', 6, 6);
    T('6.5E-29531627805496', '-7.57040630422451705434881571160179759007104791141306E+4365358659', '-4.9207640977459360853267302125411684335461811424185e-29527262446836', 50, 6);
    T('5.3E+21786', '-9.76622618786797236903307526753201665260894875109978140359393E+9182165', '-5.17609988e+9203952', 10, 5);
    T('5.5297665250143884700019985961183449464816737E+936', '3.3796232965687616E+296384166834', '1.868852777252471286480617523122074247881936133470334630428992e+296384167771', 76, 4);
    T('-7.215109376109932339E+2652105', '7.24526971652521943869742892785010739038924616963926462555E+8683476633', '-5.227541346414646235922676100980921283032916687496302068942616930332667066145e+8686128739', 79, 4);
    T('-9.216143958548530311990439475093217294778682871E+582983187', '7.249053792737130562632580435185577983561593615E+942549691138547', '-6.68083233171276158556289735538540890585083e+942550274121735', 42, 3);
    T('1.60E+202', '-6.407128785144E-1033137', '-1.02514060562304e-1032934', 77, 3);
    T('2.205199208254798821474950998902191375631503824848921578E-31040239', '-3.3880545460122022299262396676563823061715925772E+85847945', '-7.471335202390180221185936884296420222859699405817191352101605427e+54807706', 64, 1);
    T('1.6311190345265678763766E-58492573080', '6.121758534229122431000830E-83814471595', '9.985316869956583505381162042624971114851992578e-142307044675', 67, 5);
    T('-2.84605178661532285450011432239310046E+54783459678', '4.6E+23843514', '-1.3091838218430485130700525883008262116e+54807303193', 70, 6);
    T('8.5973457945E+50399024742915', '0.92286611262186515446807927209452702698', '8e+50399024742915', 1, 2);
    T('-9.37175614356322301483264262858817450955115571074774280431816E+820', '-5.8990551313492234E+5043468', '5.52845061684402398929718264606605040162316531044852111077181139889284e+5044289', 69, 3);
    T('4.9309172981597834254716114236938809737852E+622197935770', '-90388850037180804810300832311714283089697161065337.1762604', '-4.456999442091054136760545232185631743357676415045555277342448095504338e+622197935820', 70, 5);
    T('-4.5318175283930901279E+257711762470', '-9.526792268847312998999055E-689170', '4.31736841933220293960040853567839111019291345e+257711073301', 46, 4);
    T('-5.8485208434582231935912884487991260791234986391206618078774E+704265708232', '-3.97E+84344039281604', '2.32186277485291e+85048304989837', 15, 5);
    T('8.948010032343111803060117863693331060830530240250993E+9216', '-4.295014E-23086885468', '-3.8431828361054117997708449066212948612901979009e-23086876251', 47, 4);
    T('3.185756757125125107816323513488886171902174516738662E+42054157', '8.06908950729962831216038361716544465020833388684669454928E-96447250096', '2.57061564217272374136338781469713519e-96405195938', 36, 0);
    T('5.2525916740929835502942298722584732448E+345', '4.71657145142231465197216614165779608834340065610E+79402', '2.4774224e+79748', 9, 2);
    T('-3.89225344424E+408', '6.89051755046380019455784229458968806687E+300417053593', '-2.6819640668388894316706943776e+300417054002', 29, 0);
    T('-2.62899171772083482619619847446355938432108410955548082905E+1194534854298247', '5.4889047121379790783170218428373272345901598058E+62770973956', '-1.44302850275696100321962196928350139526413e+1194597625272204', 42, 2);
    T('5.147388379257214801014394E+145685', '-871451.36315899031142129000758349639107634751903849', '-4.4856986198124456473522012534585768386213060847482824297e+145691', 56, 3);
    T('5.38114319975281081541720E+25890582551835', '-5.3746836395492276642812193294683520289555634332372353098E-8388', '-2.8921942317783013844648076943183991700501029553856777114332710316463890058e+25890582543448', 74, 5);
    T('280871964076135618401817180457296.105266734468', '-7.887061633E+4000357553507', '-2.2152e+4000357553540', 5, 1);
    T('-8.88835887E+968', '-4.342636646908781652776E+7024535543', '3.8598912959e+7024536512', 11, 3);
    T('-8.049E-677401925027', '6.1906233944265068994375629680216267831565782872875620E-914837747587860', '-4.9828327701738954e-915515149512886', 17, 4);
    T('8.8474201258056282592641810954E-3218', '6.734889818647060851595E+350385282', '5.95863997265814241e+350382065', 18, 2);
    T('69.4226', '9.3E+8893414103', '6.4563018e+8893414105', 21, 2);
    T('5.4496200747276662620115235573674567E+73497054', '6.1324122147419595938364630799329435142211E+9270', '3.34193167119629312060831661097219240448232664069e+73506325', 48, 6);
    T('-6.40155938120572162197123643404406239443989352196681E-16271723479128', '-6.5302607079733794046998E+622547', '4.1803851696846104251167518538e-16271722856580', 29, 6);
    T('-6.885314662857499063218876897969991960E+325601', '5.612724023214075E+86455290745927', '-3.86453710156084e+86455291071529', 16, 5);
    T('3.0116477E+34906110890763', '-3.2736027840035761593873554881121013808939467419E+967771967043', '-9.85893829515796693219376256485518746593607864916562863e+35873882857806', 57, 6);
    T('2.17214889901473709634572E+85', '-9.67073449015503646158535438556730E-6375241745195', '-2.1006275275454107335252775478289115070676115032739126956e-6375241745109', 69, 1);
    T('3.05540807398194033E-3214459', '-261138589.704559597740507442041713439449886297801317961162239', '-7.978849554115685928e-3214451', 20, 4);
    T('-8.73781828060529448236834604892444E+1948580683', '9.478856719915468792231243799601849139374345738197715542E+816680159268', '-8.2824527526515722939252636246780210601567507e+818628739952', 44, 3);
    T('7.7347070256738760192372923810083E+84912757', '-4.2406171585544227861811736221460633E+9292884376073', '-3.27999313294640829790116089024e+9292969288831', 30, 1);
    T('-3.0733080745642545872718080859554759E+853', '5.68364318456656220657503596215363350177814066E-2744', '-1.746758649207050956e-1890', 20, 0);
    T('2.03705113002970753490764690580603012081557995076025852E+2111662', '7.47470103253575010872293625E-75867326361147', '1.522634818496117146621996031155706488957762e-75867324249484', 43, 3);
    T('6.25088801982750097072E+47381290', '6.260997127481331024451319844070682924210806197112221662E+20202191389', '3.91367919363474489475816842734494993064509e+20249572680', 43, 3);
    T('1.8386551234237034678148677060345720766015066E+99289792', '-3.87113785421249924946369380501607446425E+41011861525864', '-7.1176874491272534089e+41011960815656', 20, 4);
    T('1.38458076206162843942007838321318483E+79178', '7.004377933941450094882084405E-469164', '9.69812693754430751693405317027143365258271565595283814892557615e-389986', 64, 5);
    T('-7.119393156946169853E-72', '1.265962432255999E-4035270019', '-9.01288427715428840897403e-4035270091', 24, 6);
    T('-3.437088094735134294557167180255176212E+310798354', '-6.435557845E+3482', '2.2119579252028797e+310801837', 17, 0);
    T('-3.63207504408171609E+49', '5.4E+791', '-1.961320523804126689e+841', 19, 5);
    T('-2.9337943909355522329992160E+5647', '-6.5698889458878729147799E+9345147230723', '1.927470333851532940029648358486997805615152e+9345147236371', 43, 0);
    T('-9.825189421836013013446677052993363359583686659522658576464E+214238', '3.1167194089935487908886238094518859573877028160573683E-1674069356', '-3.062235856807440582266669090472515057931733248188221626404165573423007123931154e-1673855117', 79, 6);
    T('6.701E+105916', '-2.9124960E+549610078769', '-1.9516635696e+549610184686', 42, 3);
    T('-7.40176744979E+1287276', '-6.6621168863945191094436595132729850692936897900365488E+46321370603', '4.9311439916411254856540217581243655352446700713738186362268884752e+46322657880', 65, 4);
    T('-8.918808885384742806671174747057876288E+1763712', '-8.4003738391593512440070605769E+519969', '7.4921328837247966213851255141437e+2283682', 32, 5);
    T('-3.7686243415791453304045387688224491933692808778079206E+44709492667151', '-1.2034E-5043203242', '4.5351625326563434906088219544009353593e+44704449463909', 38, 6);
    T('-3.73617288560281657918403493772051287946683185728289195426E+40547204', '9.09299795858826904406959462172327565978E+5650677840859', '-3.397301242172e+5650718388064', 14, 0);
    T('6.6139595744581450285013546686588487999358280258473489135157E-487794055832976', '-7E+4', '-4.6297717021207015199509482680611941599550796181e-487794055832971', 48, 3);
    T('7.122494472899604291456470028972764307537690E+86322342278', '-3E+1760346883093', '-2.1367e+1846669225372', 5, 4);
    T('9.20244792087162415396453576827457005169120E+5505945075536000', '-3.0467106491328930959906654469939666873099077467288E-2478440', '-2.8037196078610428466706859e+5505945073057561', 26, 4);
    T('-4.2210567526544976259E+4894', '3.438532765228373515E-336', '-1.45142419478909683e+4559', 18, 1);
    T('-6.080608056763958537E-31157141012336', '4.334297565996549', '-3e-31157141012335', 1, 6);
    T('7.883004787674E+8779715347958007', '-921.70034886657814134977171', '-7.26576826291603e+8779715347958010', 15, 4);
    T('5.91772672038197692527032654433656382886965570170756E-89763905502', '-9.0334397331315328187687100039496402783511598435173E-202357244', '-5.34574276858e-89966262745', 12, 0);
    T('8.900645799548909690567378674595525119476386279449E+255774', '-8.0444476449172836295159734562448874581293287939393E+66042757854', '-7.161e+66043013629', 4, 0);
    T('-8.6571251689720182468117989629379E+9159', '7.611106265318014710E+780290', '-6.5890299613205204838047936743611716e+789450', 35, 4);
    T('-1.7984545186344306693549101522157541837E-917606295901073', '-7232.0662832789693446638147979317054765022674381', '1.3006542286226774925213890694854050997935879322232583835511086817132288e-917606295901069', 71, 0);
    T('3.93376256147804658481293736906817781406191858E-65', '-6.656937808682576075290362554936275884361331520701419389E-8', '-2.6186812725883224882764369964078206661107579995874900802848345679822097e-72', 71, 6);
    T('-2.149465816E+138929948944', '-5.3564207E+5348703457899', '1.15134431907647912e+5487633406844', 28, 3);
    T('-3.907039034448406782015263428840879901203E+9290', '1E+663831925636', '-3.90703903444840678202e+663831934926', 21, 6);
    T('-7.82223549399761698791698178E-919496', '5.53809624E+91915361', '-4.3320293e+90995866', 8, 6);
    T('-4.19099109E+4089647656', '-1.43145673878838054505158894395674407848872E+7076667342352', '5.9992224379825602598405528544652237783564861855048e+7080756990008', 66, 1);
    T('-5.802857724249977226294028100855E+871547605739', '5.20449511615117743E+5690760994601587', '-3.020094468557914235476625626549626251681504e+5691632542207327', 43, 0);
    T('4E-8936', '-3.22895995617131E+34234648485434', '-1.291583982468524e+34234648476499', 40, 6);
    T('-7.953379574720339939529752388993716753357E-5897640545407698', '-2.1435069248172166445581105664823131091862E+11821042971984', '1.704812419411285819296573417363126788538469760437851287104393447367e-5885819502435713', 67, 4);
    T('5.67782008866580006E+2921497710982648', '9.688225096E+734', '5.500799907358494929821030576e+2921497710983383', 32, 2);
    T('-1.728733127756E-79731512', '-7.889132849919685715481982705286213652596121410E+4699', '1e-79726812', 1, 5);
    T('-4.9836647991068250676308837296998E+922121312998', '-7.60834026532646E+7355690366', '3.7917417559934560408683814949787143004844796708e+929477003365', 58, 2);
    T('2.4982203873996828522277329301572638926631985424012E-3815', '-2.5580928311960989136422623079192392842789047480248298863087E+685426605', '-6.390679663755069739965425483586e+685422790', 31, 2);
    T('-9.7843293258078674946E+9239504969564', '8.5890260078605815715619211042880714715553E-7488929054', '-8.4037859048836763704212458654338820758118494765563615e+9232016040511', 53, 4);
    T('-7.829133199E+5762753132198', '8.027889595535365452740321008248481E+359616502666', '-6.285141695031261184e+6122369634865', 19, 2);
    T('6.27289905593802746267813935E+570495764671', '-2.42029231362653838655130568984805190880E+4260', '-1.51822493692419769258094154e+570495768932', 27, 6);
    T('5.363251073667271291029867925788675949578386538E+986', '-3.8752025558244767725520E+6211388097', '-2.0783684268203778862480333396069739245562284880933e+6211389084', 50, 6);
    T('-5.0921572620E-179495053', '-2.17281705489243865721978013228374867125793627E-18123325', '1.1064326145067984137251232130652011441e-197618377', 38, 5);
    T('2.640239980359011041435877846577641E-38944729', '9.1525829297073555250734671538673311020732775E+9014', '2.41650153745647680867953360872951849287048542676198601043244628359929750883775e-38935714', 80, 0);
    T('-1.93663758006380022987687898502055761E-7', '1.2996882897952E+45032', '-2.517025184386235235273221471499519318855176701472e+45025', 79, 2);
    T('-7.79624787528536087470E+522844', '7.86052E-36308048613', '-6.1282562348638084862796844e-36307525768', 48, 0);
    T('-8.6916130130673447148591023E-35071865871144', '-9.9238734332603261512468E+827', '8.625446747255875821946163029823367918470147374764e-35071865870316', 78, 2);
    T('4.00947946956450859933153631898545750457438090160657404600478E-80256665343425', '-5.432347705196059882178310037458406326102535813834E-896670', '-2.178088659551947371143060643834448e-80256666240094', 34, 2);
    T('8.5237189533806E+1584254', '5.559104516659139407296405310312455019209323151352988851E+4168669121525932', '4.73842445324712059856820248906971310602994071998002270539e+4168669123110187', 57, 2);
    T('-9.5567131066523154803423960119663705401544923465878340E+923201', '7.231731765301542236921134645986541738098510067263E+891', '-6.9111585745251135317786803544757506203200056995334205874878457e+924093', 62, 0);
    T('-1.168262681298989991628574470330473680056939158E+2839315483445591', '-8.3276E+15', '9.728824305e+2839315483445606', 10, 6);
    T('-6.0781541404E+3894398400574661', '-3.5551844886168367843114091243483844694E-468799752342', '2.160895931937228316971341313211647151473466710376e+3893929600822320', 70, 5);
    T('-9.930650769E-29', '6.8663587981084446829E-201686', '-6.81874112786655419357e-201714', 21, 0);
    T('-7.335392379743827850E-8', '-2.7319376535378854572592620827E-2770314405323726', '2.0039834645697038682119651541820671550861263195e-2770314405323733', 79, 3);
    T('6.526581741082787243991555642172262538786965980061353690513E+139613898', '7.27127468783930012341873509649266453712072899934263E-36', '4.7456568612049419e+139613863', 17, 3);
    T('-4.22121443693737169738781063964E+56886231777', '6.46514882556814689849853E+284324335', '-2.72907795594369551182e+57170556113', 21, 4);
    T('-4.2756487463198090289095525688E+2144035393917', '-553.0796', '2.36477409835506144978568377093087648e+2144035393920', 65, 4);
    T('7.6896692111943855791178991668625756180248214081981494E-71512989309591', '9.182346838994448110268897241888100554017096E+9110', '7.060920977432369764143121866773047198394810733518419151011992e-71512989300480', 61, 6);
    T('2.180820361230329375484504788788352570472611566909E+490568502', '-8.7402170748877831954755703095889597E+1636791723110', '-1.906084335848826812330467513562092591013488107727e+1637282291613', 49, 6);
    T('7.029746564122600884257343821598015E+1986035361995620', '5.97888721757E-11321809790574', '4.20300618749892448e+1974713552205047', 18, 4);
    T('8.9082309066978726571172660992155288568183961846110E+92966609287793', '-6.372745936836552863893242035962375E+720', '-5.676989231506066923271e+92966609288514', 23, 5);
    T('7.89339268979572378738067183817058284E-34354', '9.40407207779209729787E-5641331923', '7.423003379315622392352526e-5641366276', 25, 3);
    T('-8.67452009658410368432993084472027446790739575E+52640', '-1.9127802223015687601128496484168401815516501946271396043E+68866409', '1.6592450478703567557084487969287833435458577809040885135e+68919050', 56, 1);
    T('3.78899780037211610061450014794833157E-25', '-1.781198215051115462248321974621842099810242322885821E-178', '-6.74895611886e-203', 12, 5);
    T('-2.155154073630209625200311251540E+1729442981108', '8.57370176906083278450588440893E+43217803528', '-1.847764829368198853875886103937513376099415878e+1772660784637', 46, 4);
    T('-7.573356931092417337447861170640744225137419531155902692733E+650708627', '-6.153212613277891964490791934374476575722515426852119022E+839147072', '4.6600475393253e+1489855700', 14, 5);
    T('9.9485070330656474601543085707824827201303972914296E+50365383', '-8.339564668921734296294E+51651946540', '-8.296621776147366141255e+51702311924', 22, 5);
    T('-9341189.1143291483367810197661330685131503844231817233', '-5.2902655709146386768479917571509005451E+361', '4.94173711629380999452122697321630573180513697514816e+368', 51, 3);
    T('-3.94205311338658631286509556625817971E+16157', '-9.211465098548013710604889811533128813247447560316297E+50484508', '3.6312084670583075556389877785380186469268808582939e+50500666', 50, 3);
    T('3.80682239719999E+9108569', '1.2570378527E+23171863693', '4.785319851787e+23180972262', 13, 0);
    T('-9.4785667878741097620937182099167353179206477903295E+319318083141787', '2.18914651047879948228632460987022499797894573E-77622588673349', '-2.0749971408014850575894868085e+241695494468439', 29, 2);
    T('8.9667725561135595682395580714154E-986', '1.1512563831473418E+8027419170', '1.032305414145614153439488817400778841194507358372e+8027418185', 60, 5);
    T('9.3824406374705879422603687090411415559E+28598052', '-3.50433070520586E-7746996567809884', '-3.287917481565942409009547240550780296503505331019e-7746996539211831', 49, 2);
    T('5.99E+13791044512809', '1.96046646889916754612071635552249438783E+248238115', '1.1743194e+13791292750925', 8, 6);
    T('3.064300584181522300878161202335034438E+404457', '6.0535215816314153102435315312283077826829E+77415331', '1.8549809718948598773839134903222460170305454724434488707237125796507991534e+77819789', 74, 6);
    T('5.81262639194966973950354535185905205219806501541037262161E+671621', '1.9031000617961150495272884354716297026994E-903289894955222', '1.106200964571714573808968647523e-903289894283600', 31, 4);
    T('9.5476838756708013471774773695351827963819089787235895E-8', '-8.408E+5139037', '-8.02769260266400977270682297230518169519790906931079e+5139030', 51, 5);
    T('-6.6E-7463867943633', '-1.4807684717021055514852E+387367', '9.77307191323389663980232e-7463867556266', 48, 1);
    T('-8.55084718716151381E+822668', '-9.9175121266567279254154973093072804774E+8063', '8.4803130671862884864960981662863480992266774404443492894e+830732', 62, 2);
    T('5.3373715876357243785148742308551E-56952990', '-6.5179989919672330632712640299738175964123E-41295149', '-3.478898262796420184523996e-98248138', 25, 4);
    T('6.971459479802497597082438607792354091586831018306743480E-761', '-6.033E+216403', '-4e+215643', 1, 5);
    T('1.777384386455398193216E+2597299297574', '-6.084886935E+523988312', '-1.08151830316154434261226e+2597823285887', 24, 1);
    T('-4.6521500157722287061856841778E+9887292058', '-2.83437170876054177504354221116689924238706435361497611914E-55276141643', '1.31859223896147132476824015424548234299417761151369391782106971716610693e-45388849584', 72, 6);
    T('7.5265257980887835089498587251138470751028760392372286407E-2808031', '-9.1E+5942', '-6.84913847626079299314437e-2802088', 24, 4);
    T('6.124447659875137256913380088076116744534502046E-3221', '-445.129416206308926513928576593405597136836057165908670', '-2.7261718114263147022892763710281790800347767629059444299404749637586549e-3218', 72, 3);
    T('-2.343998818E+1337502807076880', '9.28276989128108540103161939E+41', '-2.1758801652928852685775171e+1337502807076922', 26, 1);
    T('6.30E-742321395', '3010721501699960.4', '1.896754546070975052e-742321379', 32, 5);
    T('-8.19473472459505835840696858E+57148564414539', '-3.072155949788976057941148086E+91051363', '2.517550304110703465066659165e+57148655465903', 28, 1);
    T('8.30776117642162743211570060628744719866432894004311519342327E-33426', '-7.32744736817205E+483785', '-6.08746827675725878961361300787124655166036655e+450360', 46, 5);
    T('2.7724010038502924E+11725', '-4.7556689139834608E+928358674281', '-1.318462127110737658180757284393792e+928358686007', 40, 4);
    T('5.9091700317147668794795218E+666897567', '8.3938478805437150E+1881334366773516', '4.9600674346481433119807194038219992195487e+1881335033671084', 45, 3);
    T('-4.3691580894245294496446636805158600484074460956643847E-89474019484498', '-4.480405454599623134150719643985041E-85297159512', '2e-89559316644009', 2, 0);
    T('3.9841406473800271259191727447172370183389231944900E+7162819378682783', '-1.1756652068301468200152428806523472381832E+26219282019', '-4.684015538242e+7162845597964802', 13, 4);
    T('-108.838392673073428475338591374657485576815', '4.7679792205854258775825803754367415273006845283282054E-2392128538985014', '-5.189391946671311e-2392128538985012', 16, 1);
    T('-8.204662794316848196358517032121883095398020430371494E+4174', '-9.24067E+76425280674697', '7.581658134355986962264425758321772146315162545032095346098e+76425280678872', 77, 6);
    T('-1765008726556704110068716.971202764152', '-6.181972151135587E-438453547492', '1.0911234794084831168999690852448e-438453547467', 32, 4);
    T('4.2855951384E+6786330', '-8.467571964460057801596728600265E+803863', '-3.62e+7590194', 3, 2);
    T('5.931102910248257260028749055698492591626238811283040E+2994979930', '4.74554276498454773524121438E+4419395108969865', '2.81463025041074126217073937975121544226291777930659827735640858625099754981152e+4419398103949796', 79, 5);
    T('-3.252331914843376239065836709279067E+94440233387625', '-2.3623795146812686712069E+77052063', '7.6832422905700963877126387159924969901297229583598e+94440310439688', 51, 5);
    T('3.0492122248920255558204E+853564085', '-8.08984113814203797323E+830247744', '-2.46676e+1683811830', 6, 5);
    T('5.3E+3001536064', '-5.1284719857135251986348941516094288300538120647569E+8250887721405', '-2.7180901524281683552764939003e+8253889257470', 29, 1);
    T('-7.232364520576269255230605598E+13647', '-5.0920968774E+8733763326721', '3.68279007914449687231013903824867596852e+8733763340369', 43, 4);
    T('-4.570347590158040638E+2740239393431', '2.07889215232397404562727100828808310398753938E+677467169443', '-9.50125973857233712074162213187e+3417706562874', 30, 6);
    T('4.5448270760346241610118429522193090232405670818152E+841009', '-2.422423171496393324796293931481383527700912094875E-7804254834', '-1.10094944194304741888085692195231316338573093706e-7803413824', 48, 5);
    T('4.22383351811211262345546887870844027467352195456471978E+2536981668011', '9.68288662066799336541944E-58', '4.0898901060456795748e+2536981667954', 20, 3);
    T('-7.98E-5131801232', '-6.46961274175379599674705810329589944472E+51197530481496', '5.162750967919529205404152e+51192398680265', 25, 1);
    T('5.406750087593579807470489668413180550069465120969E+49446063043777', '-14357.71569753219', '-7.762858060527588408725771461418996736206026436467e+49446063043781', 49, 4);
    T('8.767284916129139715662943977492089255922416046307300E+93918407846859', '7.70727619934705517543277570E+843', '6.7571886366976561345510962397291254226922e+93918407847703', 43, 3);
    T('1.7215714164903441106850793864502663758372E+73233827435817', '3.40448860214041361777E+24287921', '5.861070265212103438724011835488590426212729427605887764547044e+73233851723738', 78, 3);
    T('-4.5628487502792802885938754300464085807968592197376E+6353199536', '99369621.8217038874', '-4.53408554744886282e+6353199544', 18, 1);
    T('-7.278632E+684731572310999', '-1.70098296173502667080983368271858703080836592E-67129927', '1.23808290167393406470099213577133545572e+684731505181073', 39, 3);
    T('3.62481321880718463782470515258531161626837808157570536E+135011330', '3.70907770474907003605508735070520261968241E-9929696201', '1.3444713893757440983671783972567382e-9794684870', 35, 6);
    T('-0.000814586801311722298757051', '-4.6011129412495143332669917195124045818036E-4163600013', '3.74800587328641232642672141632691e-4163600016', 34, 4);

    log('\n ' + passed + ' of ' + total + ' tests passed in ' + (+new Date() - start) + ' ms \n');
    return [passed, total];
})(this.Decimal);
if (typeof module !== 'undefined' && module.exports) module.exports = count;
